"""This module exists to support backward compatibility for the expansion of downloader"""

__copyright__ = "Copyright (C) 2019-present, DV Klopfenstein. All rights reserved."
__author__ = "DV Klopfenstein"


# pylint: disable=unused-import
from pmidcite.icite.dnldr.pmid_dnlder import NIHiCiteDownloader


# Copyright (C) 2019-present DV Klopfenstein. All rights reserved.
