"""MeSH descriptors from raw/mesh/d2020.bin."""

__copyright__ = "Copyright (C) 2014-2020, DV Klopfenstein. all rights reserved."
__author__ = "DV Klopfenstein"

from collections import namedtuple

# 29,640 MeSH unique identifiers
# pylint: disable=too-many-lines
NtO = namedtuple('NtMhDesc', 'MH MS')
UI2NT = {
  'D000001': NtO(MH='Calcimycin', MS='An ionophorous, polyether antibiotic from Streptomyces chartreusensis. It binds and transports CALCIUM and other divalent cations across membranes and uncouples oxidative phosphorylation while inhibiting ATPase of rat liver mitochondria. The substance is used mostly as a biochemical tool to study the role of divalent cations in various biological systems.'),
  'D000002': NtO(MH='Temefos', MS='An organothiophosphate insecticide.'),
  'D000003': NtO(MH='Abattoirs', MS='Places where animals are slaughtered and dressed for market.'),
  'D000004': NtO(MH='Abbreviations as Topic', MS='Works about shortened forms of written words or phrases used for brevity.'),
  'D000005': NtO(MH='Abdomen', MS='That portion of the body that lies between the THORAX and the PELVIS.'),
  'D000006': NtO(MH='Abdomen, Acute', MS='A clinical syndrome with acute abdominal pain that is severe, localized, and rapid in onset. Acute abdomen may be caused by a variety of disorders, injuries, or diseases.'),
  'D000007': NtO(MH='Abdominal Injuries', MS='General or unspecified injuries involving organs in the abdominal cavity.'),
  'D000008': NtO(MH='Abdominal Neoplasms', MS='New abnormal growth of tissue in the ABDOMEN.'),
  'D000009': NtO(MH='Abdominal Muscles', MS='Muscles forming the ABDOMINAL WALL including RECTUS ABDOMINIS; ABDOMINAL OBLIQUE MUSCLES, transversus abdominis, pyramidalis muscles and quadratus abdominis.'),
  'D000010': NtO(MH='Abducens Nerve', MS='The 6th cranial nerve which originates in the ABDUCENS NUCLEUS of the PONS and sends motor fibers to the lateral rectus muscles of the EYE. Damage to the nerve or its nucleus disrupts horizontal eye movement control.'),
  'D000011': NtO(MH='Abelson murine leukemia virus', MS='A replication-defective strain of Murine leukemia virus (LEUKEMIA VIRUS, MURINE) capable of transforming lymphoid cells and producing a rapidly progressing lymphoid leukemia after superinfection with FRIEND MURINE LEUKEMIA VIRUS; MOLONEY MURINE LEUKEMIA VIRUS; or RAUSCHER VIRUS.'),
  'D000012': NtO(MH='Abetalipoproteinemia', MS='An autosomal recessive disorder of lipid metabolism. It is caused by mutation of the microsomal triglyceride transfer protein that catalyzes the transport of lipids (TRIGLYCERIDES; CHOLESTEROL ESTERS; PHOSPHOLIPIDS) and is required in the secretion of BETA-LIPOPROTEINS (low density lipoproteins or LDL). Features include defective intestinal lipid absorption, very low serum cholesterol level, and near absent LDL.'),
  'D000013': NtO(MH='Congenital Abnormalities', MS='Malformations of organs or body parts during development in utero.'),
  'D000014': NtO(MH='Abnormalities, Drug-Induced', MS='Congenital abnormalities caused by medicinal substances or drugs of abuse given to or taken by the mother, or to which she is inadvertently exposed during the manufacture of such substances. The concept excludes abnormalities resulting from exposure to non-medicinal chemicals in the environment.'),
  'D000015': NtO(MH='Abnormalities, Multiple', MS='Congenital abnormalities that affect more than one organ or body structure.'),
  'D000016': NtO(MH='Abnormalities, Radiation-Induced', MS='Congenital changes in the morphology of organs produced by exposure to ionizing or non-ionizing radiation.'),
  'D000017': NtO(MH='ABO Blood-Group System', MS='The major human blood type system which depends on the presence or absence of two antigens A and B. Type O occurs when neither A nor B is present and AB when both are present. A and B are genetic factors that determine the presence of enzymes for the synthesis of certain glycoproteins mainly in the red cell membrane.'),
  'D000018': NtO(MH='Abomasum', MS='The fourth stomach of ruminating animals. It is also called the "true" stomach. It is an elongated pear-shaped sac lying on the floor of the abdomen, on the right-hand side, and roughly between the seventh and twelfth ribs. It leads to the beginning of the small intestine. (From Black\'s Veterinary Dictionary, 17th ed)'),
  'D000019': NtO(MH='Abortifacient Agents', MS='Chemical substances that interrupt pregnancy after implantation.'),
  'D000020': NtO(MH='Abortifacient Agents, Nonsteroidal', MS='Non-steroidal chemical compounds with abortifacient activity.'),
  'D000021': NtO(MH='Abortifacient Agents, Steroidal', MS='Steroidal compounds with abortifacient activity.'),
  'D000022': NtO(MH='Abortion, Spontaneous', MS='Expulsion of the product of FERTILIZATION before completing the term of GESTATION and without deliberate interference.'),
  'D000023': NtO(MH='Abortion Applicants', MS='Individuals requesting induced abortions.'),
  'D000024': NtO(MH='Abortion, Criminal', MS='Illegal termination of pregnancy.'),
  'D000025': NtO(MH='Abortion, Eugenic', MS='Abortion performed because of possible fetal defects.'),
  'D000026': NtO(MH='Abortion, Habitual', MS='Three or more consecutive spontaneous abortions.'),
  'D000027': NtO(MH='Abortion, Incomplete', MS='Premature loss of PREGNANCY in which not all the products of CONCEPTION have been expelled.'),
  'D000028': NtO(MH='Abortion, Induced', MS='Intentional removal of a fetus from the uterus by any of a number of techniques. (POPLINE, 1978)'),
  'D000029': NtO(MH='Abortion, Legal', MS='Termination of pregnancy under conditions allowed under local laws. (POPLINE Thesaurus, 1991)'),
  'D000030': NtO(MH='Abortion, Missed', MS='The retention in the UTERUS of a dead FETUS two months or more after its DEATH.'),
  'D000031': NtO(MH='Abortion, Septic', MS='Any type of abortion, induced or spontaneous, that is associated with infection of the UTERUS and its appendages. It is characterized by FEVER, uterine tenderness, and foul discharge.'),
  'D000032': NtO(MH='Abortion, Therapeutic', MS='Abortion induced to save the life or health of a pregnant woman. (From Dorland, 28th ed)'),
  'D000033': NtO(MH='Abortion, Threatened', MS='UTERINE BLEEDING from a GESTATION of less than 20 weeks without any CERVICAL DILATATION. It is characterized by vaginal bleeding, lower back discomfort, or midline pelvic cramping and a risk factor for MISCARRIAGE.'),
  'D000034': NtO(MH='Abortion, Veterinary', MS='Premature expulsion of the FETUS in animals.'),
  'D000035': NtO(MH='Abreaction', MS='A process in psychotherapy in which the patient is "desensitized" to emotionally painful, often forgotten (repressed) memories by recalling and reacting to them in the "safety" of the treatment setting.'),
  'D000036': NtO(MH='Abrin', MS='A toxic lectin from the seeds of jequirity, Abrus precatorius L. Very active poison. Five different proteins have so far been isolated: Abrus agglutinin, the component responsible for: hemagglutinating activity, & abrins a-d, the toxic principals each consisting of two peptide chains are held together by disulfide bonds.'),
  'D000037': NtO(MH='Abruptio Placentae', MS='Premature separation of the normally implanted PLACENTA from the UTERUS. Signs of varying degree of severity include UTERINE BLEEDING, uterine MUSCLE HYPERTONIA, and FETAL DISTRESS or FETAL DEATH.'),
  'D000038': NtO(MH='Abscess', MS='Accumulation of purulent material in tissues, organs, or circumscribed spaces, usually associated with signs of infection.'),
  'D000039': NtO(MH='Peritonsillar Abscess', MS='An accumulation of purulent material in the area between the PALATINE TONSIL and its capsule.'),
  'D000040': NtO(MH='Abscisic Acid', MS='Abscission-accelerating plant growth substance isolated from young cotton fruit, leaves of sycamore, birch, and other plants, and from potatoes, lemons, avocados, and other fruits.'),
  'D000041': NtO(MH='Absenteeism', MS='Chronic absence from work or other duty.'),
  'D000042': NtO(MH='Absorption', MS='The physical or physiological processes by which substances, tissue, cells, etc. take up or take in other substances or energy.'),
  'D000043': NtO(MH='Abstracting and Indexing', MS='Works about activities performed to identify concepts and aspects of published information and research reports.'),
  'D000044': NtO(MH='Dental Abutments', MS='Natural teeth or teeth roots used as anchorage for a fixed or removable denture or other prosthesis (such as an implant) serving the same purpose.'),
  'D000045': NtO(MH='Acacia', MS='A plant genus of the family FABACEAE. The gums and tanning agents obtained from Acacia are called GUM ARABIC. The common name catechu is more often used for Areca catechu (ARECA).'),
  'D000046': NtO(MH='Academic Medical Centers', MS='Medical complexes consisting of medical school, hospitals, clinics, libraries, administrative facilities, etc.'),
  'D000047': NtO(MH='Academies and Institutes', MS='Organizations representing specialized fields which are accepted as authoritative; may be non-governmental, university or an independent research organization, e.g., National Academy of Sciences, Brookings Institution, etc.'),
  'D000048': NtO(MH='Acanthamoeba', MS='A genus of free-living soil amoebae that produces no flagellate stage. Its organisms are pathogens for several infections in humans and have been found in the eye, bone, brain, and respiratory tract.'),
  'D000049': NtO(MH='Acanthocephala', MS='A phylum of parasitic worms, closely related to tapeworms and containing two genera: Moniliformis, which sometimes infects man, and Macracanthorhynchus, which infects swine.'),
  'D000050': NtO(MH='Acanthocytes', MS='Erythrocytes with protoplasmic projections giving the cell a thorny appearance.'),
  'D000051': NtO(MH='Acantholysis', MS='Separation of the prickle cells of the stratum spinosum of the epidermis, resulting in atrophy of the prickle cell layer. It is seen in diseases such as pemphigus vulgaris (see PEMPHIGUS) and DARIER DISEASE.'),
  'D000052': NtO(MH='Acanthosis Nigricans', MS='A circumscribed melanosis consisting of a brown-pigmented, velvety verrucosity or fine papillomatosis appearing in the axillae and other body folds. It occurs in association with endocrine disorders, underlying malignancy, administration of certain drugs, or as in inherited disorder.'),
  'D000053': NtO(MH='Acari', MS='A large, subclass of arachnids comprising the MITES and TICKS, including parasites of plants, animals, and humans, as well as several important disease vectors.'),
  'D000054': NtO(MH='Acceleration', MS='An increase in the rate of speed.'),
  'D000055': NtO(MH='Accessory Nerve', MS='The 11th cranial nerve which originates from NEURONS in the MEDULLA and in the CERVICAL SPINAL CORD. It has a cranial root, which joins the VAGUS NERVE (10th cranial) and sends motor fibers to the muscles of the LARYNX, and a spinal root, which sends motor fibers to the TRAPEZIUS and the sternocleidomastoid muscles.'),
  'D000056': NtO(MH='Accident Prevention', MS='Efforts and designs to reduce the incidence of unexpected undesirable events in various environments and situations.'),
  'D000057': NtO(MH='Accident Proneness', MS='Tendency toward involvement in accidents. Implies certain personality characteristics which predispose to accidents.'),
  'D000058': NtO(MH='Accidental Falls', MS='Falls due to slipping or tripping which may result in injury.'),
  'D000059': NtO(MH='Accidents', MS='Unexpected and unintentional events, typically resulting in ACCIDENTAL INJURIES.'),
  'D000060': NtO(MH='Accidents, Aviation', MS='Unanticipated events associated with AIRCRAFT.'),
  'D000061': NtO(MH='Accidents, Home', MS='ACCIDENTS which occur within the home.'),
  'D000062': NtO(MH='Accidents, Occupational', MS='Unforeseen occurrences, especially injuries in the course of work-related activities.'),
  'D000063': NtO(MH='Accidents, Traffic', MS='Accidents on streets, roads, and highways involving drivers, passengers, pedestrians, or vehicles. Traffic accidents refer to AUTOMOBILES (passenger cars, buses, and trucks), BICYCLING, and MOTORCYCLES but not OFF-ROAD MOTOR VEHICLES; RAILROADS nor snowmobiles.'),
  'D000064': NtO(MH='Acclimatization', MS='Adaptation to a new environment or to a change in the old.'),
  'D000065': NtO(MH='Accommodation, Ocular', MS='The dioptric adjustment of the EYE (to attain maximal sharpness of retinal imagery for an object of regard) referring to the ability, to the mechanism, or to the process. Ocular accommodation is the effecting of refractive changes by changes in the shape of the CRYSTALLINE LENS. Loosely, it refers to ocular adjustments for VISION, OCULAR at various distances. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D000066': NtO(MH='Accounting', MS='System of recording financial transactions.'),
  'D000066388': NtO(MH='Oil and Gas Industry', MS='INDUSTRY concerned with the extraction, processing, distribution, marketing and storage of PETROLEUM and NATURAL GAS.'),
  'D000066428': NtO(MH='Coal Industry', MS='Enterprise associated with the mining, processing, marketing and distribution of COAL.'),
  'D000066448': NtO(MH='Driving Under the Influence', MS='Operating a motor vehicle while impaired by alcohol or other drugs including those prescribed by physicians.'),
  'D000066449': NtO(MH='Human Embryonic Stem Cells', MS='A type of PLURIPOTENT STEM CELLS derived from early stage human embryos, up to and including the BLASTOCYST stage.'),
  'D000066450': NtO(MH='Mouse Embryonic Stem Cells', MS='PLURIPOTENT STEM CELLS derived from the BLASTOCYST INNER CELL MASS of day 3.5 mouse embryos.'),
  'D000066473': NtO(MH='Embryonic Germ Cells', MS='PLURIPOTENT STEM CELLS that are derived from early GERM CELLS.'),
  'D000066477': NtO(MH='Work Performance', MS='Measure of how well someone performs given tasks at their place of work.'),
  'D000066479': NtO(MH='Criminal Behavior', MS='Conduct leading to commission of unlawful acts.'),
  'D000066491': NtO(MH='Clinical Decision-Making', MS='Process of formulating a diagnosis based on medical history and physical or mental examinations, and/or choosing an appropriate intervention.'),
  'D000066492': NtO(MH='Cognitive Aging', MS='Phenomenon that occurs when learning and memory abilities decrease naturally due to age.'),
  'D000066494': NtO(MH='Cognitive Neuroscience', MS='Field of study concerning MENTAL PROCESSES; LEARNING; and the brain regions associated with them.'),
  'D000066498': NtO(MH='Emotional Adjustment', MS='Psychological process that occurs when an individual adapts to a significant emotional event or stimulus.'),
  'D000066499': NtO(MH='Facial Recognition', MS='Ability to identify an individual and interpret their feelings and emotions based on their facial expressions.'),
  'D000066502': NtO(MH='Grandparents', MS='Member of a family who has children and grandchildren.'),
  'D000066505': NtO(MH='Help-Seeking Behavior', MS='Form of social behavior in which assistance is sought from others.'),
  'D000066506': NtO(MH='Heuristics', MS='Experience-based techniques for problem-solving, learning, and discovery that find a solution which is not guaranteed to be optimal, but sufficient for a given set of goals.'),
  'D000066511': NtO(MH='Intimate Partner Violence', MS='A pattern of assaultive and coercive behavior by an individual against their partner or spouse that may include physical injury, psychological abuse, sexual assault, progressive isolation, stalking, deprivation, intimidation, and reproductive coercion.'),
  'D000066530': NtO(MH='Neurological Rehabilitation', MS='Physician-supervised programs designed to rehabilitate people with diseases, trauma, or disorders of the NERVOUS SYSTEM.'),
  'D000066550': NtO(MH='Physical Abuse', MS='Violence inflicted on an individual through physical contact.'),
  'D000066553': NtO(MH='Problem Behavior', MS='Troublesome or disruptive behavioral displays.'),
  'D000066608': NtO(MH='Underage Drinking', MS='Consumption of ALCOHOLIC BEVERAGES by persons under the legal drinking age.'),
  'D000066630': NtO(MH='Ectopic Gene Expression', MS='The expression of a gene in an abnormal place,  or at an abnormal  time in an organism. Ectopic Gene  Expression is often induced artificially by genetic techniques.'),
  'D000066670': NtO(MH='Cell Plasticity', MS='The ability of a cell to change its PHENOTYPE in response to changes in the environment.'),
  'D000066673': NtO(MH='Cell Self Renewal', MS='The ability of certain cell types, such as progenitor cells or tumor cells,  to go through numerous cycles of CELL DIVISION while still maintaining an undifferentiated or partially differentiated state.'),
  'D000066768': NtO(MH='Rehabilitation Research', MS='Studies and research concerning the psychological, educational, social, vocational, industrial, and economic aspects of REHABILITATION.'),
  'D000066792': NtO(MH='Orthodontists', MS='Dentists who specialize in the treatment of misaligned TEETH and MALOCCLUSION.'),
  'D000066794': NtO(MH='Oral and Maxillofacial Surgeons', MS='Dentists who specialize in the surgical treatment of injuries, diseases, and abnormalities of the oral and maxillofacial regions.'),
  'D000066829': NtO(MH='Neuroprotection', MS='The physiological processes and techniques used by the body to prevent neuronal injury and degeneration in the central nervous system following acute disorders or as a result of chronic neurodegenerative diseases.'),
  'D000066888': NtO(MH='Diet, Food, and Nutrition', MS='Concepts involved with nutritional physiology, including categories of substances eaten for sustenance, nutritional phenomena and processes, eating patterns and habits, and measurable nutritional parameters.'),
  'D000066889': NtO(MH='Triticale', MS='A CEREAL GRAIN plant resulting from a cross between TRITICUM and SECALE CEREALE.'),
  'D000066891': NtO(MH='Critical Care Outcomes', MS='A measure of the mortality and morbidity rates among patients who receive CRITICAL CARE and INTENSIVE CARE as determined from the date of hospital discharge or release.'),
  'D000066908': NtO(MH='Simulation Training', MS='A highly customized interactive medium or program that allows individuals to learn and practice real world activities in an accurate, realistic, safe and secure environment.'),
  'D000066928': NtO(MH='Millets', MS='An agronomic group especially of small-seeded grasses, mainly grown for human and animal consumption.'),
  'D000066949': NtO(MH='Biobehavioral Sciences', MS='An interdisciplinary scientific approach to the study of complex interactions among biological, social, behavioral and environmental factors.'),
  'D000066970': NtO(MH='Protein Corona', MS='A layer of protein coating adsorbed by NANOPARTICLES upon entry into PLASMA or other protein-containing biological fluids, which affects how nanoparticles are internalized by cells and cleared from the body.'),
  'D000066988': NtO(MH='Vegetarians', MS='Persons who do not eat MEAT.'),
  'D000066989': NtO(MH='Vegans', MS='Persons who avoid consuming animal products or animal-derived substances, such as MEAT; EGGS; or HONEY.  Some vegans also refrain from using animals or animal products in any form or for any purpose, such as in CLOTHING or household goods.'),
  'D000067': NtO(MH='Accounts Payable and Receivable', MS='Short-term debt obligations and assets occurring in the regular course of operational transactions.'),
  'D000067009': NtO(MH='Youth Sports', MS='SPORTS activities designed and/or organized for CHILDREN or YOUTH.'),
  'D000067010': NtO(MH='Literacy', MS='Ability to read and write.'),
  'D000067011': NtO(MH='Severe Acute Malnutrition', MS='Acute form of MALNUTRITION which usually affects children, characterized by a very low weight for height (below -3z scores of the median World Health Organization standards), visible severe wasting, or occurrence of nutritional EDEMA.  It can be a direct or indirect cause of fatality in children suffering from DIARRHEA and PNEUMONIA. Do not confuse with starvation, a condition in which the body is not getting enough food, usually for extended periods of time.'),
  'D000067012': NtO(MH='Banking, Personal', MS='A financial account set up between a financial institution (such as a commercial bank) and a customer (individual, group, or business), whereby the latter may deposit and withdraw money, and where applicable, be paid interest.'),
  'D000067013': NtO(MH='Evidence-Based Facility Design', MS='An approach, process, or methodology which emphasizes credible evidence and the best available scientific knowledge, judiciously integrated to achieve the best possible outcomes in structural design. For example, the design of a new OUTPATIENT CLINIC might incorporate a review of published research on outpatient clinic design, decisions on similar past projects, along with interviews with staff and consumers.'),
  'D000067029': NtO(MH='Physical Appearance, Body', MS='The superficial physical looks, characteristics or qualities of an individual as distinguished from the underlying unseen nature or attributes.'),
  'D000067030': NtO(MH='Fruit and Vegetable Juices', MS='The liquids extracted from plants.'),
  'D000067048': NtO(MH='Ghee', MS='The clarified portion of milk, cream, or butter obtained by the removal of the nonfat constituents. (from Dictionary of Food Ingredients, 4th ed., 2001)'),
  'D000067071': NtO(MH='Hydraulic Fracking', MS='A drilling technology in which NATURAL GAS is extracted from large deposits of impermeable rock formations known as shale, deep within the earth\xe2\x80\x99s crust. This drilling technique involves injecting large volumes of high-pressure fracturing fluid (a mixture of water, sand, and chemicals) several thousand feet underground, thereby creating cracks or fissures in the shale formation, and releasing trapped gas.'),
  'D000067073': NtO(MH='Psychological Trauma', MS='An emotionally painful, shocking, stressful, and sometimes life-threatening experience. It can result from witnessing distressing events such as natural disasters, physical or sexual abuse, and terrorism or other acts of violence. (https://www.nimh.nih.gov/health/)'),
  'D000067074': NtO(MH='Peer Influence', MS='The effect or sway that a PEER GROUP exerts on the beliefs, value systems and behavior of each member of a group. The social expectations for individuals to conform to peer group influence is known as peer pressure.'),
  'D000067075': NtO(MH='Vegetable Products', MS='Food that is manufactured from various parts of vegetables.'),
  'D000067088': NtO(MH='Direct-to-Consumer Advertising', MS='Advertising that directly targets consumers of certain products such as pharmaceuticals.'),
  'D000067108': NtO(MH='Direct-To-Consumer Screening and Testing', MS='Laboratory testing and diagnostic imaging services offered to consumers outside of the patient-physician relationship.'),
  'D000067128': NtO(MH='Extracellular Vesicles', MS='Membrane limited structures derived from cell membranes and cytoplasmic material, and released into EXTRACELLULAR SPACE. They circulate through the EXTRACELLULAR FLUID and through the peripheral blood in the MICROVASCULATURE where cells, much larger, cannot, thereby affecting a variety of intercellular communication processes.'),
  'D000067148': NtO(MH='Fungal Viruses', MS='Viruses infecting species of FUNGI.'),
  'D000067170': NtO(MH='Telocytes', MS='A type of interstitial cells that have very long and thin CELL SURFACE EXTENSIONS called TELOPODES.'),
  'D000067190': NtO(MH='Exoskeleton Device', MS='Device designed to wear externally to support muscular skeletal system in various movements such as RANGE OF MOTIONS; WEIGHT-BEARING; GAIT; and LOCOMOTION.'),
  'D000067208': NtO(MH='Shellfish Hypersensitivity', MS='Allergic reaction to SHELLFISH or shellfish products.'),
  'D000067248': NtO(MH='Failure to Rescue, Health Care', MS='Inability to prevent a clinical deterioration resulting from a complication of an underlying medical condition or a complication of medical care. It implies that common risk factors for such a complication were not recognized in a timely manner or treated appropriately.'),
  'D000067250': NtO(MH='Psychiatric Rehabilitation', MS='Specialty field that promotes recovery, community functioning, and increased well-being of individuals diagnosed with mental disorders that impair their ability to live meaningful lives.'),
  'D000067251': NtO(MH='Symptom Flare Up', MS='A transient exacerbation of symptoms of an existing disease or condition.'),
  'D000067268': NtO(MH='Isolated Heart Preparation', MS='Method of isolation and maintenance of a working heart for various cardiac measurements. The model allows for the separation of cardiac and systemic variables in the denervated heart.'),
  'D000067269': NtO(MH='Diet, Vegan', MS='Dietary practice of avoiding animal products in any form.'),
  'D000067289': NtO(MH='Behavior Observation Techniques', MS='A wide range of formal and informal techniques used to document behavior.'),
  'D000067290': NtO(MH='Behavior Rating Scale', MS='Instrument designed to assess human or animal behavior, using a standardized format.'),
  'D000067292': NtO(MH='Alcohol Drinking in College', MS='Alcohol consumption among college students.'),
  'D000067329': NtO(MH='Obesity, Metabolically Benign', MS='A sub-PHENOTYPE of obese individuals who have a risk for CARDIOVASCULAR DISEASES between that of healthy individuals with normal weight and unhealthy individuals with obesity.'),
  'D000067348': NtO(MH='Whole Grains', MS='Edible seeds which contain all components of the seed - germ, bran, and ENDOSPERM.'),
  'D000067368': NtO(MH='Transanal Endoscopic Surgery', MS='Surgical procedures performed through the anus using a PROCTOSCOPE.'),
  'D000067369': NtO(MH='Transanal Endoscopic Microsurgery', MS='A specialized form of transanal endoscopic surgery for microscopically controlled operations in the RECTUM.'),
  'D000067390': NtO(MH='Cold Injury', MS='A physical injury caused by exposure of the body to extremely low ambient temperatures that may lead to loss of body parts, or in extreme cases, death. Examples of cold injury are FROSTBITE and CHILBLAINS.'),
  'D000067393': NtO(MH='Personal Protective Equipment', MS='Specialized clothing or equipment worn for protection against health hazards. Personal Protective Equipment may include MASKS; RESPIRATORY PROTECTIVE DEVICES; HEAD PROTECTIVE DEVICES; EYE PROTECTIVE DEVICES; EAR PROTECTIVE DEVICES; PROTECTIVE CLOTHING; and protective footwear.'),
  'D000067397': NtO(MH='Nerve Agents', MS='A class of compounds that adversely affect the transmission of impulses through the NERVOUS SYSTEM.'),
  'D000067398': NtO(MH='War-Related Injuries', MS='WOUNDS and INJURIES and PSYCHOLOGICAL TRAUMA sustained during WAR.'),
  'D000067399': NtO(MH='Abnormal Involuntary Movement Scale', MS='A rating scale that measures INVOLUNTARY MOVEMENTS known as TARDIVE DYSKINESIA.'),
  'D000067401': NtO(MH='Blood Alcohol Content', MS='Mass of alcohol per unit of BLOOD volume.'),
  'D000067404': NtO(MH='Social Communication Disorder', MS='Persistent difficulties in the social uses of verbal and nonverbal communications. (DSM-V)'),
  'D000067408': NtO(MH='Legal Services', MS='Services provided by ATTORNEYS and paralegal staff.'),
  'D000067448': NtO(MH='Defamation', MS='Public false statements that are illegal and damaging to the reputation of a person, group, business, organization, or institution.'),
  'D000067449': NtO(MH='Anger Management Therapy', MS='System of psychological and/or therapeutic techniques used to control and or reduce the triggers, degrees, and effects of ANGER.'),
  'D000067450': NtO(MH='Psychology, Sports', MS='Field of research that studies mental factors influencing the attainment of athletic skills and athletic performance.'),
  'D000067453': NtO(MH='War Exposure', MS='The health and other ramifications of military and civilian exposure to WAR .'),
  'D000067454': NtO(MH='Childhood-Onset Fluency Disorder', MS="Disturbances in the normal fluency and time patterning of speech that are inappropriate for the individual's age and language skills. (DSM-V)"),
  'D000067455': NtO(MH='No-Show Patients', MS='PATIENTS who do not turn up for scheduled healthcare appointments. Do not confuse with PATIENT DROPOUTS.'),
  'D000067470': NtO(MH='Cellular Reprogramming Techniques', MS='Procedures used for the induction of CELLULAR REPROGRAMMING to change the terminal phenotype of a cell, such as the generation of INDUCED PLURIPOTENT STEM CELLS from differentiated adult cells by the forced expression of specific genes.'),
  'D000067489': NtO(MH='Metacognition', MS="Awareness and understanding of one's own thought processes."),
  'D000067490': NtO(MH='Prescription Drug Overuse', MS='The use of a medication in an amount more than as prescribed.'),
  'D000067491': NtO(MH='Near Miss, Healthcare', MS='Event that presented a risk but did not result in a patient harm.'),
  'D000067492': NtO(MH='RNAi Therapeutics', MS='A form of disease treatment which involves the silencing of genes that are associated with a disease by the process of RNA INTERFERENCE.'),
  'D000067493': NtO(MH='Dynamic Light Scattering', MS='An analytical technique for measuring particle size of molecules that are less than a micron in diameter dispersed or dissolved in a liquid.'),
  'D000067494': NtO(MH='Time Out, Healthcare', MS='A period of time away from normal workflow taken to ensure key procedural details have been reviewed for better PATIENT SAFETY and help prevent MEDICAL ERRORS.'),
  'D000067495': NtO(MH='Healthcare Failure Mode and Effect Analysis', MS='Prospective risk assessment tool aimed at identifying potential risks and their impact in healthcare settings.'),
  'D000067496': NtO(MH='Spirit Possession', MS='The idea that abnormal behavior or a medical condition is evidence that a person is under the control of a supernatural power or spirit. This belief was most commonly held in religion and medicine of the medieval period and is also held in many tribal cultures today.'),
  'D000067498': NtO(MH='Crew Resource Management, Healthcare', MS='A range of healthcare related training designed to reduce MEDICAL ERRORS and improve PATIENT SAFETY by emphasizing TEAMWORK and human factors (see HUMAN ENGINEERING).'),
  'D000067508': NtO(MH='Computer Heuristics', MS='Trial-and-error methods of problem-solving used when an algorithmic approach is impractical'),
  'D000067528': NtO(MH='Ataxins', MS='A family of predominantly nuclear proteins that regulate gene transcription and protein degradation. The expansion of CAG trinucleotide repeats in genes that encode Ataxins is associated with SPINOCEREBELLAR ATAXIAS (SCA). In SCA patients, the number of CAG repeats correlates with the severity of disease and inversely correlates with the age of disease onset.'),
  'D000067531': NtO(MH='Alarmins', MS='A structurally diverse group of endogenous molecules that are multifunctional, having physiological functions inside the cell, but when released from dying cells or from cells under stress or certain immune cells, they function to activate INNATE IMMUNITY. Uncontrolled and excessive release of alarmins may contribute to INFLAMMATION; CARCINOGENESIS, and NEOPLASM METASTASIS. Alarmins are also critical for heart and nerve tissue homeostasis.'),
  'D000067548': NtO(MH='Dermal Fillers', MS='Materials such as COLLAGEN or HYALURONIC ACID that are injected or deposited into the DERMIS for the purpose of skin augmentation.'),
  'D000067549': NtO(MH='Action Spectrum', MS='A technique used to measure the rate of physiological activity plotted against wavelength of light. An action spectrum depicts the most effective wavelength of light (also known as spectral sensitivity) that is required for each specific process.'),
  'D000067551': NtO(MH='Diplomacy', MS='The art and practice of conducting negotiations between people, nations or other parties such as organizations.'),
  'D000067552': NtO(MH='Mutation Accumulation', MS='Concept which describes the incremental effects of MUTATION in living organisms.'),
  'D000067556': NtO(MH='Internal Ribosome Entry Sites', MS="Sequences within MESSENGER RNA that enable PROTEIN TRANSLATION INITIATION independent of 5' CAPPED RNA."),
  'D000067558': NtO(MH='Biological Mimicry', MS='Resemblance in appearance, structure, function, sound, scent or behavior between related or unrelated species, occurring in the same geographic location.'),
  'D000067559': NtO(MH='Specific Learning Disorder', MS='Diagnosed when there are specific deficits in an individual\xe2\x80\x99s ability to perceive or process information efficiently and accurately. This  disorder first manifests during the years of formal schooling and is characterized by persistent and impairing difficulties with learning foundational academic skills in reading, writing, and/or math. The individual\xe2\x80\x99s performance of the affected academic skills is well below average for age, or acceptable performance levels are achieved only with extraordinary effort. Specific learning disorder may occur in individuals identified as intellectually gifted and manifest only when the learning demands or assessment procedures (e.g., timed tests) pose barriers that cannot be overcome by their innate intelligence and compensatory strategies. For all individuals, specific learning disorder can produce lifelong impairments in activities dependent on the skills, including occupational performance. (from DSM-V)'),
  'D000067561': NtO(MH='Potentially Inappropriate Medication List', MS="A list, criteria, or screening tool designed to improve PATIENT SAFETY by determining an individual's exposure to potentially inappropriate drugs. They are designed to prevent MEDICATION ERRORS by INAPPROPRIATE PRESCRIBING. Analysis for a list includes factors such as DOSE-RESPONSE RELATIONSHIP, DRUG; DRUG-RELATED SIDE EFFECTS AND ADVERSE REACTIONS; AGE FACTORS; GENDER; and existing medical conditions."),
  'D000067562': NtO(MH='Late Onset Disorders', MS='Pathological conditions (Disorder, SYNDROME, or DISEASE) whose SIGNS AND SYMPTOMS manifest late in the life of an individual.'),
  'D000067563': NtO(MH='Phakopsora pachyrhizi', MS='Fungal pathogen which causes Soybean Rust disease in Soybeans (Glycine max).'),
  'D000067565': NtO(MH='Farmers', MS='Persons who cultivate land or crops, or raise LIVESTOCK.'),
  'D000067568': NtO(MH='Culturally Appropriate Technology', MS='A SCIENCE or TECHNOLOGY that promotes self-sufficiency and is suitable to the cultural, environmental, economic and social conditions of the geographic area in which it is applied.'),
  'D000067570': NtO(MH='Social Learning', MS='The concept of new knowledge acquisition through observation and interaction with others.'),
  'D000067571': NtO(MH='Financial Statements', MS='Formal and structured record of financial activities of a business, person, or other entity.'),
  'D000067572': NtO(MH='Miners', MS='Persons who work in a mine to extract ore or MINERALS from the earth.'),
  'D000067573': NtO(MH='Chronopharmacokinetics', MS='Absorption, metabolism and elimination of drugs in relation to time of day at which they are administered, and the mechanisms responsible for time-dependent variations.'),
  'D000067575': NtO(MH='Administrative Claims, Healthcare', MS='Information relating to itemized coding of procedures and costs associated with healthcare delivery, used as a means for tracking healthcare utilization, patterns of care, and treatment outcomes.'),
  'D000067576': NtO(MH='Child Health', MS='The concept covering the physical and mental conditions of CHILDREN.'),
  'D000067596': NtO(MH='Interleukin-33', MS='A member of the INTERLEUKIN-1 protein  family involved in the maturation of TH2 CELLS and the activation of MAST CELLS; BASOPHILS; EOSINOPHILS and NK CELLS. It is also produced by ENDOTHELIAL CELLS; EPITHELIAL CELLS and FIBROBLASTS; where it can function as an alarmin to modulate immune and inflammatory responses to tissue damage.'),
  'D000067616': NtO(MH='S100A12 Protein', MS='A member of the S100 PROTEIN FAMILY that regulates INFLAMMATION and the immune response. It recruits LEUKOCYTES, promotes cytokine and chemokine production, and regulates leukocyte adhesion and migration. S100A12 can also function via binding to ADVANCED GLYCOSYLATION END PRODUCT-SPECIFIC RECEPTORS, to stimulate innate immune cells.'),
  'D000067617': NtO(MH='Telopodes', MS="The very long and thin extensions of telocytes' cell surface, that have alternating thick and thin sections called podoms and podomers."),
  'D000067636': NtO(MH='Drosophila simulans', MS='A species of fruit fly.'),
  'D000067656': NtO(MH='Optimism', MS='Mental attitude that interprets life events as generally being positive.'),
  'D000067657': NtO(MH='Pessimism', MS='Mental attitude that interprets life events as generally being negative.'),
  'D000067696': NtO(MH='Ataxin-1', MS='A chromatin-binding factor that represses Notch signaling and associates with RNA. Expansion of the polyglutamine tract by expanded CAG repeats in the ATXN1 gene coding region is associated with SPINOCEREBELLAR ATAXIA 1.'),
  'D000067697': NtO(MH='Adolescent Health', MS='The concept covering the physical and mental conditions of ADOLESCENTS.'),
  'D000067698': NtO(MH='Ataxin-2', MS='A Poly(A) RNA-binding protein that negatively regulates EGFR ENDOCYTOSIS. An increased risk for developing AMYOTROPHIC LATERAL SCLEROSIS 13 is observed in patients who have more than 23 CAG repeats in the ATXN2 gene coding sequence. Larger CAG expansions in the ATXN2 gene occur in SPINOCEREBELLAR ATAXIA 2 patients.'),
  'D000067699': NtO(MH='Ataxin-3', MS='A deubiquitinating enzyme of the ATAXINS family. It functions in protein homeostasis, GENETIC TRANSCRIPTION; CYTOSKELETON regulation, and MYOGENESIS. CAG TRINUCLEOTIDE REPEAT EXPANSION in the Ataxin-3 gene coding region is associated with spinocerebellar ataxia-3 (MACHADO-JOSEPH DISEASE).'),
  'D000067716': NtO(MH='Point-of-Care Testing', MS='Allows patient diagnoses in the physician\xe2\x80\x99s office, in other ambulatory setting or at bedside. The results of care are timely, and allow rapid treatment to the patient. (from NIH Fact Sheet Point-of-Care Diagnostic Testing, 2010.)'),
  'D000067719': NtO(MH='Ataxin-7', MS='A component of the STAGA transcription coactivator-HAT complex that functions in Cone Rod Homeobox Protein (CRX)-dependent gene activation. It also stabilizes MICROTUBULES. CAG repeat expansion in the ATXN-7 coding region is associated with the development of SPINOCEREBELLAR ATAXIA 7.'),
  'D000067736': NtO(MH='Ataxin-10', MS='An ataxin that is associated with the survival of cerebellar neurons. Expansion of the ATTCT pentanucleotide in the ATXN10 coding region is associated with SPINOCEREBELLAR ATAXIA 10.'),
  'D000067756': NtO(MH='Glucagon-Like Peptide Receptors', MS='G-PROTEIN COUPLED CELL SURFACE RECEPTORS that bind GLUCAGON-LIKE PEPTIDES and are expressed by cells in pancreatic, intestinal, and neural tissues. These receptors regulate cellular responses to BLOOD GLUCOSE, INSULIN, and INFLAMMATION signals.'),
  'D000067757': NtO(MH='Glucagon-Like Peptide-1 Receptor', MS='A receptor for GLUCAGON-LIKE PEPTIDE 1 (GLP-1) expressed primarily on the surface of beta and ductal exocrine cells of the pancreas, as well as cells of other tissues. GLP-1 acts through GLP-1R to potentiate signaling in pancreatic cells in response to glucose-stimulated insulin secretion (GSIS).'),
  'D000067758': NtO(MH='Glucagon-Like Peptide-2 Receptor', MS='A receptor for GLUCAGON-LIKE PEPTIDE 2 (GLP-2) that is expressed on the surface of intestinal cells as well as neural cells. GLP-2 and other peptides act through GLP-2R to regulate cellular responses to BLOOD GLUCOSE, INFLAMMATION, and FOOD INTAKE.'),
  'D000067759': NtO(MH='Receptor for Advanced Glycation End Products', MS='A single-pass transmembrane CELL SURFACE RECEPTOR that binds ADVANCED GLYCATION END PRODUCTS to mediate cellular responses to both acute and chronic vascular inflammation in conditions such as ATHEROSCLEROSIS and DIABETES MELLITUS, TYPE 2. It also binds AMYLOID BETA PEPTIDES and the ALARMINS - S100A12 and S100 CALCIUM BINDING PROTEIN BETA SUBUNIT.'),
  'D000067776': NtO(MH='RNA Recognition Motif Proteins', MS='A family of RNA-BINDING PROTEINS that contain an RNA RECOGNITION MOTIF and two ribonucleoprotein (RNP) domains which bind RNA, in addition to other domains that allow for high affinity binding, sequence specificity, and protein interactions. Examples of RNA recognition motif proteins include HETEROGENEOUS NUCLEAR RIBONUCLEARPROTEINS (hnRNP) and EMBRYONIC LETHAL ABNORMAL-VISION (ELAV) proteins.'),
  'D000067777': NtO(MH='ELAV-Like Protein 2', MS="A neural-specific RRM protein that binds to several 3'UTRs, including its own as well as that of PROTO-ONCOGENE PROTEINS C-FOS  and ID DNA BINDING PROTEIN INHIBITOR. It binds through the recognition of a GAAA motif in the RNAs."),
  'D000067778': NtO(MH='ELAV-Like Protein 3', MS='A neural-specific RRM protein which contains three RNP-type RNA recognition motifs (RRMs). Human ELAVL3 is recognized by the anti-Hu serum antibody from patients with paraneoplastic encephalomyelitis and sensory neuronopathy (PEM/PSN). It also functions in neurogenesis.'),
  'D000067779': NtO(MH='ELAV-Like Protein 4', MS="An RRM protein which binds AU-rich sequences in the 3' UNTRANSLATED REGIONS of mRNAs for PROTO-ONCOGENE PROTEINS C-FOS; CYCLIN-DEPENDENT KINASE INHIBITOR P21; and GAP-43 PROTEIN. It functions in neuron-specific RNA processing."),
  'D000067780': NtO(MH='ELAV-Like Protein 1', MS="An RRM protein that binds to the 3'-UTR region of mRNAs and increases their stability. In EMBRYONIC STEM CELLS, it binds to poly-U elements and AU-rich elements (AREs) in the 3'-UTR of target mRNAs and preferentially binds mRNAs that are not methylated by N6-methyladenosine (m6A), to stabilize them and promote differentiation."),
  'D000067796': NtO(MH='Whey', MS='The liquid components of milk that remain after the CASEIN, fat, and fat soluble components have been removed. It is also a byproduct of cheese production.'),
  'D000067816': NtO(MH='Whey Proteins', MS='The protein components of milk obtained from the whey.'),
  'D000067836': NtO(MH='Hoarding Disorder', MS='Disordered behavior associated with clinically significant distress or impairment in social, occupational or other important areas of functioning and persistent difficulty parting with possessions due to a perceived need to save the items and distress associated with discarding them. (from DSM-V) The quantity of collected items sets the behavior apart from normal collecting behaviors.'),
  'D000067856': NtO(MH='Poly(ADP-ribose) Polymerase Inhibitors', MS='Chemicals and drugs that inhibit the action of POLY(ADP-RIBOSE)POLYMERASES.'),
  'D000067877': NtO(MH='Autism Spectrum Disorder', MS='Wide continuum of associated cognitive and neurobehavioral disorders, including, but not limited to, three core-defining features: impairments in socialization, impairments in verbal and nonverbal communication, and restricted and repetitive patterns of behaviors. (from DSM-V)'),
  'D000067878': NtO(MH='CELF Proteins', MS='A family of RRM proteins which contain two N-terminal RNA RECOGNITION MOTIF (RRM) domains, one C-terminal RRM domain, and a divergent segment of 160-230 amino acids between the second and third RRM domains. They regulate pre-mRNA ALTERNATIVE SPLICING and also function in RNA EDITING and PROTEIN BIOSYNTHESIS.'),
  'D000067879': NtO(MH='CELF1 Protein', MS="A member of the CELF PROTEINS family which binds GU rich elements (GREs) and CUG-triplet repeats in the 3'UTR of mammalian mRNA transcripts that undergo rapid turnover. It also binds AU-rich elements (AREs or EDEN-like) in the 3'UTR of JUN and FOS mRNAs. Mutations in the human CELF1 gene are associated with MYOTONIC DYSTROPHY type 1."),
  'D000067896': NtO(MH='Industrial Development', MS='Activity concerned with the planning and building of industries through manufacturing, provision of specialized services, and COMMERCE.'),
  'D000067917': NtO(MH='Cloud Computing', MS='Storing and processing data on multiple servers that can be accessed through the Internet.'),
  'D000067936': NtO(MH='Natural Resources', MS='Materials or substances such as land, timber, fresh water, fossil fuels, or mineral deposits that occur in a natural state and have economic value or usefulness to humans.'),
  'D000067940': NtO(MH='Glycemic Load', MS='A quantitative value of a measured amount of a specific food that is equal to the GLYCEMIC INDEX of that food multiplied by the carbohydrate content of that food.'),
  'D000067956': NtO(MH='Adenylyl Cyclase Inhibitors', MS='Compounds that bind to and inhibit the action of ADENYLYL CYCLASES.'),
  'D000067996': NtO(MH='RAW 264.7 Cells', MS='A transformed macrophage cell line isolated from ASCITES of mice infected with ABELSON MURINE LEUKEMIA VIRUS.'),
  'D000068': NtO(MH='Accreditation', MS='Certification as complying with a standard set by non-governmental organizations, applied for by institutions, programs, and facilities on a voluntary basis.'),
  'D000068016': NtO(MH='Clostridiales', MS='An order of anaerobic bacteria that are predominantly GRAM POSITIVE within the phylum Firmicutes. It includes the Clostridium and Eubacterium genera.'),
  'D000068036': NtO(MH='Graphic Novel', MS='Book-length narratives told using a combination of words and sequential art, often presented in comic book style. from (Fletcher-Spear et al., ALA Review, Winter 2005)'),
  'D000068037': NtO(MH='South Sudan', MS='A nation located in East-Central Africa; south of Sudan, north of Uganda and Kenya, west of Ethiopia. Independence from SUDAN was attained on 9 July 2011. The capital is Juba. (from CIA. World Factbook, 2014.)'),
  'D000068079': NtO(MH='Motor Disorders', MS='Motor skills deficits that significantly and persistently interfere with ACTIVITIES OF DAILY LIVING appropriate to chronological age.  (from DSM-5)'),
  'D000068097': NtO(MH='Maternal Health', MS='The health of women during PREGNANCY; childbirth and the POSTPARTUM PERIOD.'),
  'D000068098': NtO(MH='Seed Bank', MS='A repository of seeds maintained under optimal conditions that ensure long term storage, intended for use as a source of replenishment of seed stock or plant genetic material.'),
  'D000068099': NtO(MH='Trauma and Stressor Related Disorders', MS='Disorders in which exposure to a traumatic or stressful event is explicitly a diagnostic criterion.'),
  'D000068101': NtO(MH='Maternal-Child Health Services', MS='Programs organized for the purpose of providing medical and social services for mothers and children.'),
  'D000068103': NtO(MH='Serine-Arginine Splicing Factors', MS='A family of regulatory factors essential for constitutive and alternative splicing in RNA metabolism.'),
  'D000068104': NtO(MH='Infant Health', MS='The concept covering the physical and mental conditions of INFANTS.'),
  'D000068105': NtO(MH='Bipolar and Related Disorders', MS='These disorders are related to both SCHIZOPHRENIA SPECTRUM AND OTHER PSYCHOTIC DISORDERS and DEPRESSIVE DISORDER in terms of symptomatology, family history, and genetics. (DSM-5).'),
  'D000068116': NtO(MH='Gender Dysphoria', MS='A marked difference between the individual\xe2\x80\x99s expressed/experienced gender and the gender others would assign him or her, and it must continue for at least six months. (from DSM-5)'),
  'D000068156': NtO(MH='Formative Feedback', MS='Information communicated to the learner that is intended to modify the learner\xe2\x80\x99s thinking or behavior for the purpose of improving learning.'),
  'D000068180': NtO(MH='Aripiprazole', MS='A piperazine and quinolone derivative that is used primarily as an antipsychotic agent. It is a partial agonist of SEROTONIN RECEPTOR, 5-HT1A and DOPAMINE D2 RECEPTORS, where it also functions as a post-synaptic antagonist, and an antagonist of SEROTONIN RECEPTOR, 5-HT2A. It is used for the treatment of SCHIZOPHRENIA and BIPOLAR DISORDER, and as an adjunct therapy for the treatment of depression.'),
  'D000068181': NtO(MH='Prunus dulcis', MS='A plant species in the family ROSACEAE, genus Prunus which produces edible seeds called almonds.'),
  'D000068196': NtO(MH='Albumin-Bound Paclitaxel', MS='An injectable formulation of albumin-bound paclitaxel NANOPARTICLES.'),
  'D000068216': NtO(MH='Child Protective Services', MS='Local governmental offices which investigate reports of abuse and neglect of children, and act to assure a safe environment for minors.'),
  'D000068217': NtO(MH='Social Workers', MS='Professionals who work with persons affected by social disadvantages such as POVERTY; mental and physical illness or disability, and social injustice.  Their focus is on both the individual and his or her environment, including violations of their civil liberties and human rights.'),
  'D000068218': NtO(MH='Prunus africana', MS='A plant species in the family ROSACEAE. The bark extract known as pygeum is an ingredient in FOLK REMEDIES.'),
  'D000068219': NtO(MH='Prunus armeniaca', MS='A plant species in the family ROSACEAE. It is the source of apricot fruit.'),
  'D000068220': NtO(MH='Prunus avium', MS='A plant species in the family ROSACEAE. It is the source of the sweet cherry fruit.'),
  'D000068236': NtO(MH='Refugium', MS='Area in an environment in which a population of organisms can survive through a period of unfavorable conditions.'),
  'D000068237': NtO(MH='Distance Counseling', MS='Counseling conducted via electronic or other non-face-to-face interactions.'),
  'D000068238': NtO(MH='Lubiprostone', MS='Member of a bicyclic fatty acid class of compounds derived from PROSTAGLANDIN E1 involved in chloride channel gating.'),
  'D000068239': NtO(MH='Prunus persica', MS='A plant species in the family ROSACEAE. It is a tree that produces peach fruit.'),
  'D000068240': NtO(MH='Data Anonymization', MS='Removal of personally identifiable information via encryption or other methods.'),
  'D000068241': NtO(MH='Personally Identifiable Information', MS='Information that can be used to distinguish or trace an individual\xe2\x80\x99s identity, either alone or when combined with other personal or identifying information that is linked or linkable to a specific individual.'),
  'D000068242': NtO(MH='Prunus domestica', MS='A plant species in the family ROSACEAE. It is a tree that produces fruits called plums.'),
  'D000068256': NtO(MH='Darbepoetin alfa', MS='A recombinant protein which stimulates ERYTHROPOIESIS used to treat ANEMIA.'),
  'D000068257': NtO(MH='Efavirenz, Emtricitabine, Tenofovir Disoproxil Fumarate Drug Combination', MS='Inhibitor of reverse transcriptases or of RNA-directed DNA polymerases.'),
  'D000068258': NtO(MH='Bevacizumab', MS='An anti-VEGF humanized murine monoclonal antibody. It inhibits VEGF RECEPTORS and helps to prevent PATHOLOGIC ANGIOGENESIS.'),
  'D000068276': NtO(MH='Giraffes', MS='Four-legged hoofed mammals with extremely long necks and legs belonging to the family Giraffidae.'),
  'D000068296': NtO(MH='Risedronic Acid', MS='A pyridine and diphosphonic acid derivative that acts as a CALCIUM CHANNEL BLOCKER and inhibits BONE RESORPTION.'),
  'D000068297': NtO(MH='Fluticasone-Salmeterol Drug Combination', MS='A drug combination of fluticasone and salmeterol that is used as an inhaler formulation to manage the symptoms of ASTHMA and CHRONIC OBSTRUCTIVE PULMONARY DISEASE.'),
  'D000068298': NtO(MH='Fluticasone', MS='A STEROID with GLUCOCORTICOID RECEPTOR activity that is used to manage the symptoms of ASTHMA; ALLERGIC RHINITIS, and ATOPIC DERMATITIS.'),
  'D000068299': NtO(MH='Salmeterol Xinafoate', MS='A selective ADRENERGIC BETA-2 RECEPTOR agonist that functions as a BRONCHODILATOR when administered by inhalation. It is used to manage the symptoms of ASTHMA and CHRONIC OBSTRUCTIVE PULMONARY DISEASE.'),
  'D000068316': NtO(MH='Parks, Recreational', MS='An area of land open to the public for enjoyment.'),
  'D000068336': NtO(MH='Warfare and Armed Conflicts', MS='Engaging in any type of conflict including war or military conflict against another nation or organized group.'),
  'D000068337': NtO(MH='Armed Conflicts', MS='Any differences arising between two nations or groups and leading to the intervention of armed forces.'),
  'D000068338': NtO(MH='Everolimus', MS='A derivative of sirolimus and an inhibitor of TOR SERINE-THREONINE KINASES. It is used to prevent GRAFT REJECTION in heart and kidney transplant patients by blocking cell proliferation signals. It is also an ANTINEOPLASTIC AGENT.'),
  'D000068342': NtO(MH='Aspirin, Dipyridamole Drug Combination', MS='A drug combination of aspirin and dipyridamole that functions as a PLATELET AGGREGATION INHIBITOR, used to prevent THROMBOSIS and STROKE in TRANSIENT ISCHEMIC ATTACK patients.'),
  'D000068356': NtO(MH='Self-Control', MS="An individual's ability to manage and monitor their emotions, behaviors, and desires in the face of external demands in order to function in society."),
  'D000068357': NtO(MH='Saline Waters', MS='Water from natural sources which contain significant concentrations of dissolved salts such as sodium chloride.'),
  'D000068376': NtO(MH='Compassion Fatigue', MS='Emotional distress caused by repeated or prolonged expression of compassion or empathy. It may occur in individuals working in care giving professions.'),
  'D000068377': NtO(MH='Plantaginaceae', MS='A plant family of order Lamiales. The Plantago genus is best known. Lesser known members include Hippuris, Littorella and Callitriche.'),
  'D000068378': NtO(MH='Lamiales', MS='An order of the Lamiidae clade. Lamiales is one of the largest orders of flowering plants and includes pot-herbs in the mint family (MINT; SAGE; OREGANO; BASIL); SESAME, foxgloves; SNAPDRAGONS; and the olive family.'),
  'D000068397': NtO(MH='Clinical Study', MS='A work that reports on the results of a research study to evaluate interventions or exposures on biomedical or health-related outcomes.  The two main types of clinical studies are interventional studies (clinical trials) and observational studies. While most clinical studies concern humans, this publication type may be used for clinical veterinary articles meeting the requisites for humans.'),
  'D000068436': NtO(MH='Surgical Clearance', MS='Process of evaluating the health of a patient and determining if they are healthy enough for surgery.'),
  'D000068437': NtO(MH='Pemetrexed', MS='A guanine-derived ANTINEOPLASTIC AGENT that functions as a NUCLEIC ACID SYNTHESIS INHIBITOR through its binding to, and inhibition of, THYMIDYLATE SYNTHASE.'),
  'D000068438': NtO(MH='Brimonidine Tartrate', MS='A quinoxaline derivative and ADRENERGIC ALHPA-2 RECEPTOR AGONIST that is used to manage INTRAOCULAR PRESSURE associated with OPEN-ANGLE GLAUCOMA and OCULAR HYPERTENSION.'),
  'D000068456': NtO(MH='Clinical Studies as Topic', MS='Works about research studies that evaluate interventions or exposures on biomedical or health-related outcomes.  The two main types of clinical studies are interventional studies (clinical trials) and observational studies.'),
  'D000068476': NtO(MH='Beijing', MS='Capital of CHINA.'),
  'D000068516': NtO(MH='African Union', MS='A geopolitical organization of African nations with a mission to integrate vital interests, promote international relations, political stability, and economic development among member states.'),
  'D000068536': NtO(MH='Firmicutes', MS='A phylum of predominantly GRAM-POSITIVE BACTERIA that includes the orders Bacillales and Clostridiales. Firmicutes and BACTEROIDETES are the two bacterial phyla that constitute the majority of the human gut MICROBIOTA.'),
  'D000068538': NtO(MH='Dutasteride', MS='A 5-ALPHA-REDUCTASE INHIBITOR that is reported to inhibit both type-1 and type2 isoforms of the enzyme and is used to treat BENIGN PROSTATIC HYPERPLASIA.'),
  'D000068543': NtO(MH='Calceolariaceae', MS='A family in the order Lamiales with several species that are used in TRADITIONAL MEDICINE.'),
  'D000068556': NtO(MH='Interferon beta-1a', MS='An interferon beta-1 subtype that has a methionine at position 1, a cysteine at position 17, and is glycosylated at position 80. It functions as an ANTI-VIRAL AGENT and IMMUNOMODULATOR and is used to manage the symptoms of RELAPSING-REMITTING MULTIPLE SCLEROSIS.'),
  'D000068557': NtO(MH='Olmesartan Medoxomil', MS='An ANGIOTENSIN II TYPE 1 RECEPTOR BLOCKER that is used to manage HYPERTENSION.'),
  'D000068558': NtO(MH='Amlodipine Besylate, Olmesartan Medoxomil Drug Combination', MS='A pharmaceutical preparation of amlodipine besylate and olmesartan medoxomil  that combines ANGIOTENSIN II TYPE I RECEPTOR ANTAGONIST and CALCIUM CHANNEL BLOCKER activities. It is used in the management of HYPERTENSION.'),
  'D000068576': NtO(MH='Interferon beta-1b', MS='A non-glycosylated form of interferon beta-1 that has a serine at position 17. It is used in the treatment of both RELAPSING-REMITTING MULTIPLE SCLEROSIS and CHRONIC PROGRESSIVE MULTIPLE SCLEROSIS.'),
  'D000068577': NtO(MH='Nebivolol', MS='A cardioselective ADRENERGIC BETA-1 RECEPTOR ANTAGONIST (beta-blocker) that functions as a VASODILATOR through the endothelial L-arginine/ NITRIC OXIDE system. It is used to manage HYPERTENSION and chronic HEART FAILURE in elderly patients.'),
  'D000068579': NtO(MH='Celecoxib', MS='A pyrazole derivative and selective CYCLOOXYGENASE 2 INHIBITOR that is used to treat symptoms associated with RHEUMATOID ARTHRITIS; OSTEOARTHRITIS and JUVENILE ARTHRITIS, as well as the management of ACUTE PAIN.'),
  'D000068580': NtO(MH='Varenicline', MS='A benzazepine derivative that functions as an ALPHA4-BETA2 NICOTINIC RECEPTOR partial agonist. It is used for SMOKING CESSATION.'),
  'D000068581': NtO(MH='Tadalafil', MS='A carboline derivative and PHOSPHODIESTERASE 5 INHIBITOR that is used primarily to treat ERECTILE DYSFUNCTION; BENIGN PROSTATIC HYPERPLASIA and PRIMARY PULMONARY HYPERTENSION.'),
  'D000068582': NtO(MH='Certolizumab Pegol', MS="A polyethylene-glycolated Fab' fragment of TUMOR NECROSIS FACTOR antibody that binds specifically to TNF-ALPHA and neutralises it in a dose-dependent manner. It also inhibits the production of lipopolysaccharide-induced TNF-ALPHA and IL-1 BETA and is used to treat RHEUMATOID ARTHRITIS and PSORIATIC ARTHRITIS."),
  'D000068597': NtO(MH='Censorship, Research', MS='The act or practice of scrutinizing the practice of scientific research for the purpose of preventing or suppressing the unethical applications as well knowledge derived from it.'),
  'D000068598': NtO(MH='Data Accuracy', MS='A measure of scientific precision, exactness, or correctness of quantitative or qualitative values, relative to the actual or true measurements.'),
  'D000068599': NtO(MH='Brimonidine Tartrate, Timolol Maleate Drug Combination', MS='A pharmaceutical preparation of brimonidine tartrate and timolol maleate. The combined ADRENERGIC ALPHA2 RECEPTOR AGONIST and ADRENERGIC BETA-ANTAGONIST activity of these drugs reduce INTRAOCULAR PRESSURE in GLAUCOMA patients.'),
  'D000068600': NtO(MH='Albuterol, Ipratropium Drug Combination', MS='A combined pharmaceutical preparation of Ipratropium Bromide and Albuterol Sulfate that is used to treat the symptoms of ASTHMA and CHRONIC OBSTRUCTIVE PULMONARY DISEASE.'),
  'D000068601': NtO(MH='Dual Use Research', MS='Concept relating to technologies and/or information emanating from research that holds the potential for both benevolent and malevolent applications to humanity and the environment.'),
  'D000068616': NtO(MH='Professionalism', MS='The level of skill, good judgment, and polite behavior expected from people trained to do their jobs well.'),
  'D000068617': NtO(MH='Genetic Background', MS='The entire genetic composition of a strain or individual that interacts with a gene of interest.'),
  'D000068618': NtO(MH='Selective Breeding', MS='A process in which offspring possessing desired genetic traits are produced.'),
  'D000068636': NtO(MH='Legendary Creatures', MS='An unverified animal or humanoid, sometimes supernatural, often believed to be real and described in MYTHOLOGY, FOLKLORE, or legends. Some legendary creatures, such as the unicorn, were documented in scholarly antiquarian accounts of natural history.'),
  'D000068656': NtO(MH='Mometasone Furoate', MS='A pregnadienediol derivative ANTI-ALLERGIC AGENT and ANTI-INFLAMMATORY AGENT that is used in the management of ASTHMA and ALLERGIC RHINITIS. It is also used as a topical treatment for skin disorders.'),
  'D000068676': NtO(MH='Fallopia multiflora', MS='A species of Fallopia in the buckwheat family (Polygonaceae). Its roots are used in TRADITIONAL CHINESE MEDICINE.'),
  'D000068677': NtO(MH='Sildenafil Citrate', MS='A PHOSPHODIESTERASE TYPE-5 INHIBITOR; VASODILATOR AGENT and UROLOGICAL AGENT  that is used in the treatment of ERECTILE DYSFUNCTION and PRIMARY PULMONARY HYPERTENSION.'),
  'D000068678': NtO(MH='Emtricitabine, Rilpivirine, Tenofovir Drug Combination', MS='A pharmaceutical preparation that contains emtricitabine, rilpivirine and tenofovir disoproxil fumarate. It is used to treat HIV INFECTIONS.'),
  'D000068679': NtO(MH='Emtricitabine', MS='A deoxycytidine analog and REVERSE TRANSCRIPTASE INHIBITOR with antiviral activity against HIV-1 and HEPATITIS B viruses. It is used to treat HIV INFECTIONS.'),
  'D000068696': NtO(MH='Rilpivirine', MS='A diarylpyrimidine derivative and REVERSE TRANSCRIPTASE INHIBITOR with antiviral activity against HIV-1 that is used in the treatment of HIV INFECTIONS. It is also used in combination with other ANTI-HIV AGENTS, since ANTIVIRAL DRUG RESISTANCE emerges rapidly when it is used alone.'),
  'D000068698': NtO(MH='Tenofovir', MS='An adenine analog REVERSE TRANSCRIPTASE INHIBITOR with antiviral activity against HIV-1 and HEPATITIS B. It is used to treat HIV INFECTIONS and CHRONIC HEPATITIS B, in combination with other ANTIVIRAL AGENTS, due to the emergence of ANTIVIRAL DRUG RESISTANCE when it is used alone.'),
  'D000068717': NtO(MH='Glatiramer Acetate', MS='A random polymer of L-ALANINE, L-GLUTAMIC ACID, L-LYSINE, and L-TYROSINE that structurally resembles MYELIN BASIC PROTEIN. It is used in the treatment of RELAPSING-REMITTING MULTIPLE SCLEROSIS.'),
  'D000068718': NtO(MH='Rosuvastatin Calcium', MS='A HYDROXYMETHYLGLUTARYL-COA-REDUCTASE INHIBITOR, or statin, that reduces the plasma concentrations of LDL-CHOLESTEROL; APOLIPOPROTEIN B, and TRIGLYCERIDES while increasing HDL-CHOLESTEROL levels in patients with HYPERCHOLESTEROLEMIA and those at risk for CARDIOVASCULAR DISEASES.'),
  'D000068736': NtO(MH='Duloxetine Hydrochloride', MS='A thiophene derivative and selective NEUROTRANSMITTER UPTAKE INHIBITOR for SEROTONIN and NORADRENALINE (SNRI). It is an ANTIDEPRESSIVE AGENT and ANXIOLYTIC, and is also used for the treatment of pain in patients with DIABETES MELLITUS and FIBROMYALGIA.'),
  'D000068737': NtO(MH='Tolterodine Tartrate', MS='An ANTIMUSCARINIC AGENT selective for the MUSCARINIC RECEPTORS of the BLADDER that is used in the treatment of URINARY INCONTINENCE and URINARY URGE INCONTINENCE.'),
  'D000068756': NtO(MH='Valsartan', MS='A tetrazole derivative and ANGIOTENSIN II TYPE 1 RECEPTOR BLOCKER that is used to treat HYPERTENSION.'),
  'D000068757': NtO(MH='Ononis', MS='A genus of the pea family (Fabaceae), native from Europe to Central Asia and used in TRADITIONAL MEDICINE.'),
  'D000068758': NtO(MH='Mometasone Furoate, Formoterol Fumarate Drug Combination', MS='A pharmaceutical preparation of mometasone furoate and formoterol fumarate that is used as an inhaled dosage form for the treatment of ASTHMA.'),
  'D000068759': NtO(MH='Formoterol Fumarate', MS='An ADRENERGIC BETA-2 RECEPTOR AGONIST with a prolonged duration of action. It is used to manage ASTHMA and in the treatment of CHRONIC OBSTRUCTIVE PULMONARY DISEASE.'),
  'D000068760': NtO(MH='Serotonin and Noradrenaline Reuptake Inhibitors', MS='Drugs that selectively block or suppress the plasma membrane transport of SEROTONIN and NORADRENALINE into axon terminals and are used as ANTIDEPRESSIVE AGENTS.'),
  'D000068776': NtO(MH='Sleep Aids, Pharmaceutical', MS='Drugs used to induce SLEEP, prevent SLEEPLESSNESS, or treat SLEEP INITIATION AND MAINTENANCE DISORDERS.'),
  'D000068796': NtO(MH='Orexin Receptor Antagonists', MS='Substances that bind to and inhibit the action of OREXIN RECEPTORS. Drugs in this class have been used as SLEEP AIDS, PHARMACEUTICAL.'),
  'D000068797': NtO(MH='Orexins', MS='Neuropeptide hormones that play a role in regulating a variety of behavioral and physiological processes in response to motivational stimuli.'),
  'D000068798': NtO(MH='Hoodia', MS='A genus in the dogbane family (Apocynaceae) used in TRADITIONAL MEDICINE for appetite-suppressant properties.'),
  'D000068799': NtO(MH='Prasugrel Hydrochloride', MS='A piperazine derivative and PLATELET AGGREGATION INHIBITOR that is used to prevent THROMBOSIS in patients with ACUTE CORONARY SYNDROME; UNSTABLE ANGINA and MYOCARDIAL INFARCTION, as well as in those undergoing PERCUTANEOUS CORONARY INTERVENTIONS.'),
  'D000068800': NtO(MH='Etanercept', MS='A recombinant version of soluble human TNF receptor fused to an IgG FC fragment that binds specifically to TUMOR NECROSIS FACTOR and inhibits its binding with endogenous TNF receptors. It prevents the inflammatory effect of TNF and is used to treat RHEUMATOID ARTHRITIS; PSORIATIC ARTHRITIS and ANKYLOSING SPONDYLITIS.'),
  'D000068801': NtO(MH='Adapalene, Benzoyl Peroxide Drug Combination', MS='A pharmaceutical preparation of adapalene and benzoyl peroxide that is used as a DERMATOLOGIC AGENT for the topical treatment of ACNE.'),
  'D000068816': NtO(MH='Adapalene', MS='A naphthalene derivative that has specificity for RETINOIC ACID RECEPTORS. It is used as a DERMATOLOGIC AGENT for the treatment of ACNE.'),
  'D000068817': NtO(MH='Epoetin Alfa', MS='A recombinant glycosylated form of erythropoietin which stimulates the differentiation and proliferation of erythroid precursors. It is used for the treatment of ANEMIA associated with CHRONIC RENAL FAILURE in dialysis and predialysis patients.'),
  'D000068818': NtO(MH='Cetuximab', MS='A chimeric monoclonal antibody that functions as an ANTINEOPLASTIC AGENT through its binding to the EPIDERMAL GROWTH FACTOR RECEPTOR, where it prevents the binding and signaling action of cell growth and survival factors.'),
  'D000068819': NtO(MH='Hyssopus Plant', MS='A genus in the mint family (Lamiaceae). Members contain thujone and PHENOL. Distinguished from AGASTACHE (giant hyssop) and BACOPA (water hyssop).  Also distinguished from the genus of WASPS with the same genus name Hyssopus.'),
  'D000068836': NtO(MH='Rivastigmine', MS='A carbamate-derived reversible CHOLINESTERASE INHIBITOR that is selective for the CENTRAL NERVOUS SYSTEM and is used for the treatment of DEMENTIA in ALZHEIMER DISEASE and PARKINSON DISEASE.'),
  'D000068837': NtO(MH='Parmeliaceae', MS='The largest family of lichen-forming fungi.'),
  'D000068838': NtO(MH='Amlodipine, Valsartan Drug Combination', MS='A pharmaceutical preparation of amlodipine and valsartan that is used for the treatment of HYPERTENSION.'),
  'D000068856': NtO(MH='Eugenia', MS='A genus in the family Myrtaceae sometimes known as "stoppers" in FOLK MEDICINE. Many species of the genus SYZYGIUM have synonymous names that begin with the Eugenia genus name.'),
  'D000068857': NtO(MH='Human Papillomavirus Recombinant Vaccine Quadrivalent, Types 6, 11, 16, 18', MS='A vaccine containing L1 capsid proteins from four types of HPV (ALPHAPAPILLOMAVIRUS), types 6, 11, 16 and 18 that is used to prevent infections from HUMAN PAPILLOMAVIRUSES of these subtypes.'),
  'D000068858': NtO(MH='Kombucha Tea', MS='A beverage made by FERMENTATION of black tea and/or green tea.'),
  'D000068876': NtO(MH='Fingolimod Hydrochloride', MS='A sphingosine-derivative and IMMUNOSUPPRESSIVE AGENT that blocks the migration and homing of LYMPHOCYTES to the CENTRAL NERVOUS SYSTEM through its action on SPHINGOSINE 1-PHOSPHATE RECEPTORS. It is used in the treatment of MULTIPLE SCLEROSIS.'),
  'D000068877': NtO(MH='Imatinib Mesylate', MS='A tyrosine kinase inhibitor and ANTINEOPLASTIC AGENT that inhibits the BCR-ABL kinase created by chromosome rearrangements in CHRONIC MYELOID LEUKEMIA and ACUTE LYMPHOBLASTIC LEUKEMIA, as well as PDG-derived tyrosine kinases that are overexpressed in gastrointestinal stromal tumors.'),
  'D000068878': NtO(MH='Trastuzumab', MS='A humanized monoclonal antibody against the ERBB-2 RECEPTOR (HER2). As an ANTINEOPLASTIC AGENT, it is used to treat BREAST CANCER where HER2 is overexpressed.'),
  'D000068879': NtO(MH='Adalimumab', MS="A humanized monoclonal antibody that binds specifically to TNF-ALPHA and blocks its interaction with endogenous TNF RECEPTORS to modulate INFLAMMATION. It is used in the treatment of RHEUMATOID ARTHRITIS; PSORIATIC ARTHRITIS; CROHN'S DISEASE and ULCERATIVE COLITIS."),
  'D000068880': NtO(MH='Isophane Insulin, Human', MS='Human isophane insulin that is used to manage BLOOD GLUCOSE levels in patients with DIABETES.'),
  'D000068881': NtO(MH='Animals, Exotic', MS='Animals native to a foreign country or of foreign origin or character, that are not native to the United States.'),
  'D000068882': NtO(MH='Paliperidone Palmitate', MS='A benzisoxazole derivative and active metabolite of RISPERIDONE that functions as a DOPAMINE D2 RECEPTOR ANTAGONIST and SEROTONIN 5-HT2 RECEPTOR ANTAGONIST. It is an ANTIPSYCHOTIC AGENT used in the treatment of SCHIZOPHRENIA.'),
  'D000068884': NtO(MH='Immunosenescence', MS='The decline in the function of the IMMUNE SYSTEM during aging.'),
  'D000068896': NtO(MH='Canagliflozin', MS='A glucoside-derived SODIUM-GLUCOSE TRANSPORTER 2 inhibitor that stimulates urinary excretion of glucose by suppressing renal glucose reabsorption. It is used to manage BLOOD GLUCOSE levels in patients with TYPE 2 DIABETES.'),
  'D000068897': NtO(MH='Iridescence', MS='The changing of the color of an object due to variations in the angle of illumination or observation.'),
  'D000068898': NtO(MH='Raltegravir Potassium', MS='A pyrrolidinone derivative and HIV INTEGRASE INHIBITOR that is used in combination with other ANTI-HIV AGENTS for the treatment of HIV INFECTION.'),
  'D000068899': NtO(MH='Sitagliptin Phosphate, Metformin Hydrochloride Drug Combination', MS='A pharmaceutical preparation of sitagliptin phosphate and metformin hydrochloride that is used in the treatment of TYPE 2 DIABETES.'),
  'D000068900': NtO(MH='Sitagliptin Phosphate', MS='A pyrazine-derived DIPEPTIDYL-PEPTIDASE IV INHIBITOR and HYPOGLYCEMIC AGENT that increases the levels of the INCRETIN hormones GLUCAGON-LIKE PEPTIDE-1 (GLP-1) and glucose-dependent insulinotropic polypeptide (GIP). It is used in the treatment of TYPE 2 DIABETES.'),
  'D000068916': NtO(MH='Spatial Regression', MS='Methods used to take into account and incorporate spatial autocorrelation and regional variation into regression analysis models of data that has spatial dependency, and also to provide information on the spatial relationships among the variables.'),
  'D000068936': NtO(MH='Theranostic Nanomedicine', MS='An integrated nano-scale approach to MEDICINE which involves concurrent diagnosis, drug delivery, therapy, and monitoring of therapeutic response.'),
  'D000068937': NtO(MH='Cosmeceuticals', MS='COSMETICS that are associated with therapeutic effects.'),
  'D000068956': NtO(MH='Prisoner Dilemma', MS='A scenario in decision analysis in which two individuals motivated entirely by self-preservation at the expense of the other, end up in a worse state than if they had cooperated with each other in the decision-making process.'),
  'D000068957': NtO(MH='Games, Recreational', MS='Activity typically involving several individuals for the purpose of leisure, amusement, entertainment or pleasure.'),
  'D000068958': NtO(MH='Return to Sport', MS='Resumption of sporting activity following a hiatus or period of absence due to injury or other reasons.'),
  'D000068959': NtO(MH='Body Weight Maintenance', MS='Sustaining BODY WEIGHT after BODY WEIGHT CHANGES have been achieved.'),
  'D000068977': NtO(MH='Concierge Medicine', MS='An arrangement in which a patient pays an annual fee to a physician in exchange for services over and beyond normal insurance-reimbursed services.'),
  'D000068997': NtO(MH='Smartphone', MS='A cellular phone with advanced computing and connectivity capability built on an operating system.'),
  'D000068998': NtO(MH='Humidifiers', MS='Machines that add moisture to the air.'),
  'D000069': NtO(MH='Acculturation', MS='Process of cultural change in which one group or members of a group assimilate various cultural patterns from another.'),
  'D000069036': NtO(MH='Insulin Glargine', MS='A recombinant LONG ACTING INSULIN and HYPOGLYCEMIC AGENT that is used to manage BLOOD GLUCOSE in patients with DIABETES MELLITUS.'),
  'D000069056': NtO(MH='Lurasidone Hydrochloride', MS='A thiazole derivative and atypical ANTIPSYCHOTIC AGENT that functions as a DOPAMINE D2 RECEPTOR ANTAGONIST; SEROTONIN 5-HT2 RECEPTOR ANTAGONIST, serotonin 5-HT7 receptor antagonist, and antagonist of the adrenergic \xce\xb12A and \xce\xb12C receptors, as well as a partial SEROTONIN 5-HT1A RECEPTOR AGONIST. It is used in the treatment of SCHIZOPHRENIA and BIPOLAR DISORDER.'),
  'D000069057': NtO(MH='Insulin Detemir', MS='A recombinant long-acting insulin and hypoglycemic agent in which a MYRISTIC ACID is conjugated to a LYSINE at position B29. It is used to manage BLOOD GLUCOSE levels in patients with DIABETES MELLITUS.'),
  'D000069058': NtO(MH='Vardenafil Dihydrochloride', MS='A piperazine derivative, PHOSPHODIESTERASE 5 INHIBITOR and VASODILATOR AGENT that is used as a UROLOGICAL AGENT in the treatment of ERECTILE DYSFUNCTION.'),
  'D000069059': NtO(MH='Atorvastatin', MS='A pyrrole and heptanoic acid derivative, HYDROXYMETHYLGLUTARYL-COA REDUCTASE INHIBITOR (statin), and ANTICHOLESTEREMIC AGENT that is used to reduce serum levels of LDL-CHOLESTEROL; APOLIPOPROTEIN B; and TRIGLYCERIDES. It is used to increase serum levels of HDL-CHOLESTEROL in the treatment of HYPERLIPIDEMIAS, and for the prevention of CARDIOVASCULAR DISEASES in patients with multiple risk factors.'),
  'D000069076': NtO(MH='Fractures, Multiple', MS='Injuries involving the breaking of either several bones or one bone in two or more places.'),
  'D000069077': NtO(MH='Memory Consolidation', MS='Neurological process involving the conversion of learned information into long-term memory.'),
  'D000069078': NtO(MH='Seroconversion', MS='The appearance of antibodies against causative agents in the blood of individuals during the course of an infection or following immunization.'),
  'D000069079': NtO(MH='Radiation Exposure', MS='Phenomenon in which organisms are subjected to radiation.'),
  'D000069098': NtO(MH='Health Smart Cards', MS="Hand-sized cards with built-in computer chips used for accessing, storing, and protecting patients' medical information."),
  'D000069177': NtO(MH='Type I Secretion Systems', MS='Gram-negative bacterial secretion systems which translocate effectors in a single step across the inner and outer membranes. The one-step secretion is carried out by a channel that passes from the CYTOPLASM, through the inner membrane, PERIPLASMIC SPACE, and outer membrane, to the EXTRACELLULAR SPACE. The specificity of type I secretions systems are determined by the specificity of the three subcomponents forming the channel - an ATP transporter (ATP-BINDING CASSETTE TRANSPORTERS); a membrane fusion protein (MEMBRANE FUSION PROTEINS); and an outer membrane protein (BACTERIAL OUTER MEMBRANE PROTEINS.)'),
  'D000069178': NtO(MH='Type II Secretion Systems', MS='Gram-negative bacterial secretion systems which carry out the secretion of folded proteins.T2SSs secrete folded proteins from the PERIPLASMIC SPACE that have been exported there by SEC TRANSLOCASE or TAT SECRETION SYSTEMS, or they secrete folded proteins directly from the CYTOPLASM. The T2SSs have four substructures, an ATPase, an inner membrane platform, a pseudopilin, and secretin, an outer membrane complex which is a channel for secretion. (This bacterial secretin is not the same as the mammalian hormone also named SECRETIN.)'),
  'D000069196': NtO(MH='Gastrointestinal Microbiome', MS='All of the microbial organisms that naturally exist within the GASTROINTESTINAL TRACT.'),
  'D000069216': NtO(MH='Contraceptive Prevalence Surveys', MS='Collections of data, obtained from voluntary subjects, that measure the usage rate of methods of CONTRACEPTION; CONTRACEPTIVE DEVICES; or CONTRACEPTIVE AGENTS.'),
  'D000069236': NtO(MH='Pericardial Fluid', MS='Watery fluid produced in the serous and visceral pericardium surrounding the surface of the HEART.'),
  'D000069237': NtO(MH='Arthrocentesis', MS='Puncture and aspiration of fluid (e.g., SYNOVIAL FLUID) from a joint cavity. It is used sometimes to irrigate or administer drugs into a joint cavity.'),
  'D000069258': NtO(MH='Thoracentesis', MS='Aspiration of fluid or air from the THORACIC CAVITY. It is coupled sometimes  with the administration of drugs into the PLEURAL CAVITY.'),
  'D000069259': NtO(MH='Superficial Musculoaponeurotic System', MS='Layer between the superficial fat compartment and superficial facial muscles in the head and the neck.'),
  'D000069261': NtO(MH='Podosomes', MS='Actin-rich adhesive structures found at the ventral surface of metazoan ENDOTHELIAL CELLS. When found  in cultured cancer cells they are referred to as invadopodia. These are matrix-degrading structures composed of an ACTIN CYTOSKELETON core and an INTEGRINS-containing ring complex.'),
  'D000069262': NtO(MH='Plantar Plate', MS='Fibrocartilaginous ligament at the metatarsophalangeal and the interphalangeal joint of the toe.'),
  'D000069263': NtO(MH='Fallopia', MS='Genus of flowering herbaceous perennial plants and vines in the family Polygonaceae.'),
  'D000069278': NtO(MH='Arterial Switch Operation', MS='Surgical procedure for anatomical correction of transposed great arteries (e.g., DOUBLE OUTLET RIGHT VENTRICLE).'),
  'D000069279': NtO(MH='Drug Resistant Epilepsy', MS='Epileptic condition in which adequate trials of two tolerated and appropriately chosen and used ANTIEPILEPTIC DRUGS schedules to achieve sustained seizure freedom failed.'),
  'D000069280': NtO(MH='Electrocorticography', MS='Recording of brain electrical activities in which the electrodes are placed directly on the CEREBRAL CORTEX.'),
  'D000069281': NtO(MH='Autoimmune Hypophysitis', MS='Immune-mediated inflammation of the PITUITARY GLAND often associated with other autoimmune diseases (e.g., HASHIMOTO DISEASE; GRAVES DISEASE; and ADDISON DISEASE).'),
  'D000069282': NtO(MH='Canaliculitis', MS='Inflammation and infection of the lacrimal canaliculus area in the LACRIMAL APPARATUS.'),
  'D000069283': NtO(MH='Rituximab', MS='A murine-derived monoclonal antibody and ANTINEOPLASTIC AGENT that binds specifically to the CD20 ANTIGEN and is used in the treatment of LEUKEMIA; LYMPHOMA and RHEUMATOID ARTHRITIS.'),
  'D000069284': NtO(MH='Protein Translocation Systems', MS='Multiprotein complexes that carry out PROTEIN TRANSPORT across CELL MEMBRANES.'),
  'D000069285': NtO(MH='Infliximab', MS="A chimeric monoclonal antibody to TNF-ALPHA that is used in the treatment of RHEUMATOID ARTHRITIS; ANKYLOSING SPONDYLITIS; PSORIATIC ARTHRITIS and CROHN'S DISEASE."),
  'D000069286': NtO(MH='Bortezomib', MS='A pyrazine and boronic acid derivative that functions as a reversible PROTEASOME INHIBITOR. It is used as an ANTINEOPLASTIC AGENT in the treatment of MULTIPLE MYELOMA and MANTLE CELL LYMPHOMA.'),
  'D000069287': NtO(MH='Capecitabine', MS='A deoxycytidine derivative and fluorouracil PRODRUG that is used as an ANTINEOPLASTIC ANTIMETABOLITE in the treatment of COLON CANCER; BREAST CANCER and GASTRIC CANCER.'),
  'D000069289': NtO(MH='Parasite Encystment', MS='Process by which certain parasites such as GIARDIA and ENTAMOEBA convert from the TROPHOZOITE to SCHIZONT when exposed to stress.'),
  'D000069290': NtO(MH='Incisional Hernia', MS='Protrusion of tissue at or near the site of an incision from a previous surgery.'),
  'D000069292': NtO(MH='Pyroptosis', MS='Type of programmed cell death associated with infection by intracellular pathogens. It is characterized by INFLAMMASOME formation; activation of CASPASE 1; and CYTOKINES mediated inflammation.'),
  'D000069293': NtO(MH='Plasmablastic Lymphoma', MS='Malignant lymphoma composed of large B lymphoid cells which have the immunophenotype of plasma cells and a predilection for the ORAL CAVITY.'),
  'D000069294': NtO(MH='Twin-Arginine-Translocation System', MS='Protein translocase in BACTERIA or CHLOROPLASTS that exports or secretes folded proteins. In GRAM-NEGATIVE BACTERIA, twin-arginine translocase (TAT) is involved in the export of folded proteins to the PERIPLASM. In chloroplasts, TAT is involved in transporting folded proteins across the membranes of THYLAKOIDS.'),
  'D000069295': NtO(MH='Mammary Analogue Secretory Carcinoma', MS='A malignant neoplasm of epithelial cells of the SALIVARY GLANDS, with microcystic architecture, low-grade nuclei, and granular vacuolated cytoplasm.'),
  'D000069296': NtO(MH='Type III Secretion Systems', MS='Multiprotein transmembrane complexes in GRAM-NEGATIVE BACTERIA involved in either the secretion of effector proteins from the bacterial cytoplasm into host cells, or the secretion and assembly of flagellar components. The major component of the T3SSs is called the injectosome or needle complex.'),
  'D000069316': NtO(MH='Acne Conglobata', MS='Severe and chronic form of acne characterized by large, burrowing abscesses associated with disfigurement.'),
  'D000069319': NtO(MH='Type IV Secretion Systems', MS='Multiprotein complexes that transport single proteins, multiprotein complexes, and nucleoprotein complexes from the CYTOPLASM into the EXTRACELLULAR SPACE or into recipient cells. T4SSs are found in GRAM-NEGATIVE BACTERIA, while homologous components to T4SSs are found in GRAM-POSTIVE BACTERIA.'),
  'D000069320': NtO(MH='Type V Secretion Systems', MS='Secretion systems of GRAM-NEGATIVE BACTERIA (single membrane proteins or multiprotein complexes) that carry out secretion of proteins from the PERIPLASMIC SPACE to the EXTRACELLULAR SPACE via a beta-barrel transmembrane pore. There are several T5SS subtypes: T5aSS monomeric autotransporters, T5bSS two-partner secretions systems (TPSS), T5cSS trimeric autotransporters, T5dSS fused two-partner secretion systems, and T5eSS autotransporters with inverted domains.'),
  'D000069321': NtO(MH='Mitochondrial Replacement Therapy', MS='In vitro fertilization technique that uses mitochondrial DNA from a healthy donor in order to prevent the transmission of MITOCHONDRIAL DISEASE.'),
  'D000069322': NtO(MH='Self Expandable Metallic Stents', MS='Cylindrical metallic tubes that are inserted to hold open a structure in order to treat vascular, bile duct, esophageal and other tubular organ constrictions.'),
  'D000069336': NtO(MH='Neurovascular Coupling', MS='Mechanism by which changes in neural activity are tightly associated with regional cortical HEMODYNAMICS and metabolism changes. The association is often inferred from NEUROIMAGING signals (e.g., fMRI; or NIR SPECTROSCOPY).'),
  'D000069337': NtO(MH='Fused Kidney', MS='Congenital fusion of two kidneys.'),
  'D000069338': NtO(MH='Neuropathology', MS='The branch of medicine and biomedical science concerned with diseases of the nervous system.'),
  'D000069339': NtO(MH='Trachelectomy', MS='Surgical removal or resection of the CERVIX UTERI.'),
  'D000069340': NtO(MH='Deprescriptions', MS='Directions written to discontinue use of PRESCRIPTION DRUGS in order to reduce unnecessary and/or excessive medications (see POLYPHARMACY), DRUG SIDE EFFECTS and ADVERSE DRUG REACTION.'),
  'D000069341': NtO(MH='Transitional Care', MS='Health care provided during a transition to a different mode of care (e.g., TRANSITION TO ADULT CARE).'),
  'D000069342': NtO(MH='Medical Overuse', MS='Excessive or unnecessary utilization of health services by patients or physicians.'),
  'D000069343': NtO(MH='Bone-Implant Interface', MS='Region of interaction between the bone and PROSTHESES AND IMPLANTS which becomes bonded through cellular and extracellular interactions similar to OSSEOINTEGRATION.'),
  'D000069347': NtO(MH='Erlotinib Hydrochloride', MS='A quinazoline derivative and ANTINEOPLASTIC AGENT that functions as a PROTEIN KINASE INHIBITOR for EGFR associated tyrosine kinase. It is used in the treatment of NON-SMALL CELL LUNG CANCER.'),
  'D000069348': NtO(MH='Quetiapine Fumarate', MS='A dibenzothiazepine and ANTIPSYCHOTIC AGENT that targets the SEROTONIN 5-HT2 RECEPTOR; HISTAMINE H1 RECEPTOR, adrenergic alpha1 and alpha2 receptors, as well as the DOPAMINE D1 RECEPTOR and DOPAMINE D2 RECEPTOR. It is used in the treatment of SCHIZOPHRENIA; BIPOLAR DISORDER and DEPRESSIVE DISORDER.'),
  'D000069349': NtO(MH='Linezolid', MS='An oxazolidinone and acetamide derived ANTI-BACTERIAL AGENT and PROTEIN SYNTHESIS INHIBITOR that is used in the treatment of GRAM-POSITIVE BACTERIAL INFECTIONS of the skin and respiratory tract.'),
  'D000069350': NtO(MH='Telerehabilitation', MS='Delivery of therapeutic rehabilitation at a distance or offsite using telecommunication technologies.'),
  'D000069356': NtO(MH='Tympanocentesis', MS='Needle aspiration of fluid from the MIDDLE EAR through a puncture in the TYMPANIC MEMBRANE.'),
  'D000069376': NtO(MH='Type VI Secretion Systems', MS='Multiprotein complexes in GRAM NEGATIVE BACTERIA that deliver effector proteins into eukaryotic or prokaryotic cells by a mechanism similar to the contractile tail of bacteriophages. They are composed of a cell envelope-spanning tube which is surrounded by a contractile sheath and tipped by a cell puncturing complex; a base plate structure that anchors the assembly to the cell envelope; and an ATPase involved in the sheath contraction and disassembly of the complex.'),
  'D000069377': NtO(MH='Type VII Secretion Systems', MS='Bacterial secretion systems found in bacteria that have a MYCOLIC ACID-containing outer membrane such as MYCOBACTERIACEAE; Corynebacteriaceae; and NOCARDIACEAE. These are also known as ESX secretion systems because the first to be discovered is involved in secreting major virulence factors EsxA and EsxB. There are several subtypes of T7SSs including ESX-1, ESX-2, ESX-3, ESX-4, and ESX-5 secretion systems. The subtypes share some core components including an inner membrane channel-forming ATPase complex, a membrane-anchored mycosin, and a second channel that spans the outer mycolic acid-containing membrane.'),
  'D000069396': NtO(MH='Mitochondrial Ribosomes', MS='Multicomponent ribonucleoprotein structures contained in MITOCHONDRIA that specialize in the PROTEIN BIOSYNTHESIS of a small number of proteins involved in energy production for the cell.'),
  'D000069416': NtO(MH='Intravital Microscopy', MS='Type of microscopy used to study biological systems at high resolution.'),
  'D000069436': NtO(MH='Toxicological Phenomena', MS='The nature and effects of chemical, physical or biological poisons on living organisms.'),
  'D000069437': NtO(MH='Pharmacological Phenomena', MS='Interactions that occur between a living organism and chemicals that affect normal or abnormal biochemical function.'),
  'D000069438': NtO(MH='Ezetimibe', MS='An azetidine derivative and ANTICHOLESTEREMIC AGENT that inhibits intestinal STEROL absorption. It is used to reduce total CHOLESTEROL; LDL CHOLESTEROL, and APOLIPOPROTEINS B in the treatment of HYPERLIPIDEMIAS.'),
  'D000069439': NtO(MH='Dasatinib', MS='A pyrimidine and thiazole derived ANTINEOPLASTIC AGENT and PROTEIN KINASE INHIBITOR of BCR-ABL KINASE. It is used in the treatment of patients with CHRONIC MYELOID LEUKEMIA who are resistant or intolerant to IMATINIB.'),
  'D000069442': NtO(MH='Natalizumab', MS="A humanized monoclonal immunoglobulin G4 antibody to human INTEGRIN ALPHA4 that binds to the alpha4 subunit of INTEGRIN ALPHA4BETA1 and integrin alpha4beta7. It is used as an IMMUNOLOGIC FACTOR in the treatment of RELAPSING-REMITTING MULTIPLE SCLEROSIS and CROHN'S DISEASE."),
  'D000069443': NtO(MH='Heptavalent Pneumococcal Conjugate Vaccine', MS='A pneumococcal vaccine which contains 7 pneumococcal serotypes (6B, 14, 19F, 23F, 18C, 4, 9V), each conjugated individually to the outer membrane protein complex of NEISSERIA MENINGITIDIS.'),
  'D000069444': NtO(MH='Omalizumab', MS='An anti-IgE, recombinant, humanized monoclonal antibody which specifically binds to the C epsilon3 domain of IMMUNOGLOBULIN E, the site of high-affinity IgE receptor binding. It inhibits the binding of IgE to MAST CELLS and BASOPHILS to reduce the severity of the allergic response and is used in the management of persistent allergic ASTHMA.'),
  'D000069445': NtO(MH='Atomoxetine Hydrochloride', MS='A propylamine derivative and selective ADRENERGIC UPTAKE INHIBITOR that is used in the treatment of ATTENTION DEFICIT HYPERACTIVITY DISORDER.'),
  'D000069446': NtO(MH='Atazanavir Sulfate', MS='An azapeptide and HIV-PROTEASE INHIBITOR that is used in the treatment of HIV INFECTIONS and AIDS in combination with other ANTI-HIV AGENTS.'),
  'D000069447': NtO(MH='Tiotropium Bromide', MS='A scopolamine derivative and CHOLINERGIC ANTAGONIST that functions as a BRONCHODILATOR AGENT. It is used in the treatment of CHRONIC OBSTRUCTIVE PULMONARY DISEASE.'),
  'D000069448': NtO(MH='Denosumab', MS='A humanized monoclonal antibody and an inhibitor of the RANK LIGAND, which regulates OSTEOCLAST differentiation and bone remodeling. It is used as a BONE DENSITY CONSERVATION AGENT in the treatment of OSTEOPOROSIS.'),
  'D000069449': NtO(MH='Cinacalcet', MS=None),
  'D000069450': NtO(MH='Liraglutide', MS='An analog of GLUCAGON-LIKE PEPTIDE 1 and agonist of the GLUCAGON-LIKE PEPTIDE 1 RECEPTOR that is used as a HYPOGLYCEMIC AGENT and supplemental therapy in the treatment of DIABETES MELLITUS by patients who do not respond to METFORMIN.'),
  'D000069451': NtO(MH='Long Term Adverse Effects', MS='Persistent detrimental effects from treatment for a condition. Included are effects from surgery such as POSTOPERATIVE COMPLICATIONS, and from DRUG THERAPY, such as CHEMICALLY INDUCED DISORDERS, or other THERAPEUTICS. Failure to attain a desired outcome from treatment for the condition is not considered an adverse effect.'),
  'D000069452': NtO(MH='Pathogen-Associated Molecular Pattern Molecules', MS="Pathogens' molecules with specific sequence patterns that are recognized by PATTERN RECOGNITION RECEPTORS. They include microbial DNA, double-stranded RNA, surface glycoproteins, lipopolysaccharides, peptidoglycans, and lipoteichoic acid."),
  'D000069453': NtO(MH='Ultrasonic Waves', MS='Oscillating sound wave with a frequency higher than the upper limit of the human hearing range.'),
  'D000069454': NtO(MH='Darunavir', MS='An HIV PROTEASE INHIBITOR that is used in the treatment of AIDS and HIV INFECTIONS. Due to the emergence of ANTIVIRAL DRUG RESISTANCE when used alone, it is administered in combination with other ANTI-HIV AGENTS.'),
  'D000069455': NtO(MH='Palivizumab', MS='A humanized monoclonal antibody and ANTIVIRAL AGENT  that is used to prevent RESPIRATORY SYNCYTIAL VIRUS INFECTIONS in high risk pediatric patients.'),
  'D000069456': NtO(MH='Silent Mutation', MS="A change to a DNA SEQUENCE that doesn't change the AMINO ACID SEQUENCE of the protein made."),
  'D000069457': NtO(MH='Population Forecast', MS='Estimates of projected population.'),
  'D000069458': NtO(MH='Ranolazine', MS='An acetanilide and piperazine derivative that functions as a SODIUM CHANNEL BLOCKER and prevents the release of enzymes during MYOCARDIAL ISCHEMIA. It is used in the treatment of ANGINA PECTORIS.'),
  'D000069459': NtO(MH='Teas, Herbal', MS='Beverages prepared from an infusion of plant tissues.'),
  'D000069460': NtO(MH='Teas, Medicinal', MS='Beverages used in TRADITIONAL MEDICINE that are prepared from the infusion of plant tissue (HERBAL TEA), and/or animal tissue or other organisms such as FUNGI.'),
  'D000069461': NtO(MH='Bendamustine Hydrochloride', MS="A nitrogen mustard compound that functions as an ALKYLATING ANTINEOPLASTIC AGENT and is used in the treatment of CHRONIC LYMPHOCYTIC LEUKEMIA and NON-HODGKIN'S LYMPHOMA."),
  'D000069462': NtO(MH='Dimethyl Fumarate', MS='A fumarate derivative that is used as a DERMATOLOGIC AGENT in the treatment of PSORIASIS and SKIN DISEASES. It also may be used as an IMMUNOSUPPRESSIVE AGENT in the treatment of MULTIPLE SCLEROSIS.'),
  'D000069463': NtO(MH='Olive Oil', MS='Oil extracted from fruit of the OLIVE TREE (genus Olea).'),
  'D000069464': NtO(MH='Solifenacin Succinate', MS='A quinuclidine and tetrahydroisoquinoline derivative and selective M3 MUSCARINIC ANTAGONIST. It is used as a UROLOGIC AGENT in the treatment of URINARY INCONTINENCE.'),
  'D000069465': NtO(MH='Febuxostat', MS='A thiazole derivative and inhibitor of XANTHINE OXIDASE that is used for the treatment of HYPERURICEMIA in patients with chronic GOUT.'),
  'D000069466': NtO(MH='Red Meat', MS='Meat such as beef, goat, pork, or lamb which contains more MYOGLOBIN than POULTRY or SEAFOOD.'),
  'D000069467': NtO(MH='Fecal Microbiota Transplantation', MS='Transfer of GASTROINTESTINAL MICROBIOTA from one individual to another by infusion of donor FECES to the upper or lower GASTROINTESTINAL TRACT of the recipient.'),
  'D000069468': NtO(MH='Desvenlafaxine Succinate', MS='A cyclohexanol and phenol derivative and metabolite of venlafaxine that functions as a SEROTONIN AND NORADRENALINE REUPTAKE INHIBITOR (SNRI) and is used as an ANTIDEPRESSIVE AGENT.'),
  'D000069469': NtO(MH='Medical-Surgical Nursing', MS='The area of nursing concerned with the needs and care of adult medical and surgical patients.'),
  'D000069470': NtO(MH='Venlafaxine Hydrochloride', MS='A cyclohexanol and phenylethylamine derivative that functions as a SEROTONIN AND NORADRENALINE REUPTAKE INHIBITOR (SNRI) and is used as an ANTIDEPRESSIVE AGENT.'),
  'D000069471': NtO(MH='Neurosurgeons', MS='Qualified medical professionals who specialize in surgical treatment of diseases of the BRAIN and other parts of the NERVOUS SYSTEM.'),
  'D000069472': NtO(MH='Colesevelam Hydrochloride', MS='An allylamine derivative that binds BILE ACIDS in the intestine and is used as an ANTICHOLESTEREMIC AGENT in the treatment of HYPERCHOLESTEROLEMIA and HYPERLIPIDEMIAS.'),
  'D000069473': NtO(MH='Radiation Dose Hypofractionation', MS='A treatment schedule in which the total dose of radiation is divided into large doses but fewer exposure times.'),
  'D000069474': NtO(MH='Sofosbuvir', MS='A uridine monophosphate analog inhibitor of HEPATITIS C VIRUS (HCV) polymerase NS5B that is used as an ANTIVIRAL AGENT in the treatment of CHRONIC HEPATITIS C.'),
  'D000069475': NtO(MH='Re-Irradiation', MS='Treatment in which a repeat course of RADIOTHERAPY is administered.'),
  'D000069476': NtO(MH='Linagliptin', MS='A purine and quinazoline derivative that functions as an INCRETIN and DIPEPTIDYL-PEPTIDASE IV INHIBTOR. It is used as a HYPOGLYCEMIC AGENT in the treatment of TYPE II DIABETES MELLITUS.'),
  'D000069478': NtO(MH='Lisdexamfetamine Dimesylate', MS='A dextroamphetamine drug precursor that also functions as a CENTRAL NERVOUS SYSTEM STIMULANT and DOPAMINE UPTAKE INHIBITOR and is used in the treatment of ATTENTION DEFICIT HYPERACTIVITY DISORDER.'),
  'D000069479': NtO(MH='Buprenorphine, Naloxone Drug Combination', MS='A pharmaceutical preparation that combines buprenorphine, an OPIOID ANALGESICS with naloxone, a NARCOTIC ANTAGONISTS to reduce the potential for NARCOTIC DEPENDENCE in the treatment of pain. It may also be used for OPIOID SUBSTITUTION THERAPY.'),
  'D000069480': NtO(MH='Emtricitabine, Tenofovir Disoproxil Fumarate Drug Combination', MS='A pharmaceutical preparation of emtricitabine and tenofovir that is used as an ANTI-HIV AGENT in the treatment and prevention of HIV INFECTIONS.'),
  'D000069497': NtO(MH='Origin of Life', MS='The process by which life began.'),
  'D000069499': NtO(MH='Ezetimibe, Simvastatin Drug Combination', MS='A pharmaceutical preparation of ezetimibe and simvastatin that is used in the treatment of HYPERCHOLESTEROLEMIA and HYPERLIPIDEMIAS.'),
  'D000069501': NtO(MH='Abiraterone Acetate', MS='An androstene derivative that inhibits STEROID 17-ALPHA-HYDROXYLASE and is used as an ANTINEOPLASTIC AGENT in the treatment of metastatic castration-resistant PROSTATE CANCER.'),
  'D000069502': NtO(MH='Budesonide, Formoterol Fumarate Drug Combination', MS='A pharmaceutical preparation of budesonide and formoterol fumarate that is used as an ANTI-ASTHMATIC AGENT and for the treatment of CHRONIC OBSTRUCTIVE PULMONARY DISEASE.'),
  'D000069503': NtO(MH='Vilazodone Hydrochloride', MS='A benzofuran, indole, and piperazine derivative that functions as a SEROTONIN UPTAKE INHIBITOR and partial SEROTONIN 5-HT1 RECEPTOR AGONIST. It is used as an ANTIDEPRESSIVE AGENT.'),
  'D000069516': NtO(MH='International Law', MS='The body of law that governs the legal relations between or among sovereign states or nations.'),
  'D000069520': NtO(MH='Blog', MS='A website that contains conversational personal reflections, comments and sometimes links to other sites; typically run by an individual or small group.'),
  'D000069537': NtO(MH='Adult Day Care Centers', MS='Non-residential facilities that are designed to provide care and companionship for seniors who need assistance or supervision during the day.'),
  'D000069544': NtO(MH='Infectious Encephalitis', MS='Brain inflammation caused by an infectious agent.'),
  'D000069545': NtO(MH='Elvitegravir, Cobicistat, Emtricitabine, Tenofovir Disoproxil Fumarate Drug Combination', MS='A pharmaceutical preparation of the ANTI-HIV AGENTS elvitegravir, cobicistat, emtricitabine, and tenofovir disoproxil fumarate that is used in the treatment of HIV INFECTIONS.'),
  'D000069547': NtO(MH='Cobicistat', MS='A carbamate and thiazole derivative that functions as a CYTOCHROME P450 CYP3A INHIBITOR to enhance the concentration of ANTI-HIV AGENTS, with which it is used in combination, for the treatment of HIV INFECTIONS.'),
  'D000069549': NtO(MH='Ustekinumab', MS='A humanized monoclonal antibody that binds to IL-12 and IL-23 and is used as a DERMATOLOGIC AGENT in the treatment of patients with plaque PSORIASIS who have not responded to other therapies.'),
  'D000069550': NtO(MH='Machine Learning', MS='A type of ARTIFICIAL INTELLIGENCE that enable COMPUTERS to independently initiate and execute LEARNING when exposed to new data.'),
  'D000069551': NtO(MH='Environmental Psychology', MS='The field that studies interactions between individuals and the built and natural environment. It includes investigating behaviors that inhibit or foster sustainable, climate-healthy, and nature-enhancing choices.'),
  'D000069552': NtO(MH='Rivaroxaban', MS='A morpholine and thiophene derivative that functions as a FACTOR XA INHIBITOR and is used in the treatment and prevention of DEEP-VEIN THROMBOSIS and PULMONARY EMBOLISM.  It is also used for the prevention of STROKE and systemic embolization in patients with non-valvular ATRIAL FIBRILLATION, and for the prevention of atherothrombotic events in patients after an ACUTE CORONARY SYNDROME.'),
  'D000069553': NtO(MH='Supervised Machine Learning', MS='A MACHINE LEARNING paradigm used to make predictions about future instances based on a given set of labeled paired input-output training (sample) data.'),
  'D000069555': NtO(MH='Injection, Intratympanic', MS='The administration of  liquid substances into the TYMPANIC CAVITY with a hypodermic syringe.'),
  'D000069557': NtO(MH='Travoprost', MS='A cloprostenol derivative that is used as an ANTIHYPERTENSIVE AGENT in the treatment of OPEN-ANGLE GLAUCOMA and OCULAR HYPERTENSION.'),
  'D000069558': NtO(MH='Unsupervised Machine Learning', MS='A MACHINE LEARNING paradigm used to make predictions about future instances based on a given set of unlabeled paired input-output training (sample) data.'),
  'D000069559': NtO(MH='Loteprednol Etabonate', MS='An androstadiene derivative corticosteroid that is used as an ANTI-ALLERGIC AGENT for the treatment of inflammatory and allergic eye conditions.'),
  'D000069576': NtO(MH='Health Equity', MS='Opportunity to attain full health potential and no one is disadvantaged from achieving this potential because of their social position or other socially determined circumstance.'),
  'D000069577': NtO(MH='Morcellation', MS='Use of a non-cautery\xe2\x80\x93dependent mechanical device that simultaneously cuts and aspirates polyp and/or myoma tissue.'),
  'D000069578': NtO(MH='Waterborne Diseases', MS='Illnesses due to micro-organisms and chemicals in drinking water, those caused by organisms having part of their lifecycle in water or those with water-related vectors, and others spread by aerosols containing pathogens.'),
  'D000069579': NtO(MH='Ranibizumab', MS='A recombinant humanized monoclonal antibody fragment that binds VEGF-A to prevent its binding to VEGFR-1 and VEGFR-2 receptors. This activity reduces vessel permeability and angiogenesis in the treatment of neovascular age-related MACULAR DEGENERATION.'),
  'D000069580': NtO(MH='Bimatoprost', MS='A cloprostenol-derived amide that is used as an ANTIHYPERTENSIVE AGENT in the treatment of OPEN-ANGLE GLAUCOMA and OCULAR HYPERTENSION.'),
  'D000069581': NtO(MH='Exposure to Violence', MS='Experience of and exposure to VIOLENCE.'),
  'D000069582': NtO(MH='Eszopiclone', MS='A pyridine, pyrazine, and piperazine derivative that is used as a HYPNOTIC AND SEDATIVE in the treatment of INSOMNIA.'),
  'D000069583': NtO(MH='Pregabalin', MS='A gamma-aminobutyric acid (GABA) derivative that functions as a CALCIUM CHANNEL BLOCKER and is used as an ANTICONVULSANT as well as an ANTI-ANXIETY AGENT. It is also used as an ANALGESIC in the treatment of NEUROPATHIC PAIN and FIBROMYALGIA.'),
  'D000069584': NtO(MH='Unilateral Breast Neoplasms', MS='Tumors or cancer found specifically in one human BREAST, but not in both.'),
  'D000069585': NtO(MH='Filgrastim', MS='A recombinant granulocyte colony-stimulating factor (G-CSF) that is used in the treatment and prevention of NEUTROPENIA, preparation and collection of blood progenitor cells, and for use in PERIPHERAL BLOOD STEM CELL TRANSPLANTATION.'),
  'D000069586': NtO(MH='Adult Survivors of Child Adverse Events', MS='Persons who experienced traumatic events during childhood.'),
  'D000069590': NtO(MH='Round Ligaments', MS='Cord-like fibrous tissue structure that connects two articular extremities of bones, ligaments and /or cartilages.'),
  'D000069592': NtO(MH='Round Ligament of Liver', MS='A cord-like remnant structure formed from the closed left fetal UMBILICAL VEIN. It is located along the lower edge of the falciform ligament.'),
  'D000069593': NtO(MH='Round Ligament of Femur', MS='Ligament that connects the fovea of the FEMUR HEAD and the ACETABULUM notch in the HIP JOINT.'),
  'D000069594': NtO(MH='Abatacept', MS='A fusion protein immunoconjugate of the extracellular domain of CTLA4 and the Fc domain of human IgG1. It functions as a T-cell co-stimulation blocker that inhibits TNF-ALPHA and prevents the activation of T-LYMPHOCYTES. It is used in the treatment of RHEUMATOID ARTHRITIS.'),
  'D000069597': NtO(MH='Presenteeism', MS='Reporting for work despite feeling ill.'),
  'D000069599': NtO(MH='Crop Production', MS='The science and practice of cultivating PLANTS that have economic value to man. It includes plant breeding, seed production, weeding, use of fertilizers, harvesting, storage, transportation, and marketing.'),
  'D000069600': NtO(MH='Plant Breeding', MS='The purposeful manipulation of plant species with the goal of creating desired genotypes and phenotypes, using techniques such as controlled pollination and or genetic engineering, followed by artificial selection of progeny.'),
  'D000069601': NtO(MH='Crop Protection', MS='The science and practice of preventing, minimizing, or eliminating damage to AGRICULTURAL CROPS by pest organisms, weeds, and disease-causing agents.'),
  'D000069602': NtO(MH='Working Poor', MS='People who are in the labor force either working or looking for work for 27 weeks or more in a year, but whose income fall below a given poverty line.'),
  'D000069603': NtO(MH='Sevelamer', MS='A polymeric amine that binds phosphate and is used to treat HYPERPHOSPHATEMIA in patients with kidney disease.'),
  'D000069604': NtO(MH='Dabigatran', MS='A THROMBIN inhibitor which acts by binding and blocking thrombogenic activity and the prevention of  thrombus formation. It is used to reduce the risk of stroke and systemic EMBOLISM in patients with nonvalvular atrial fibrillation.'),
  'D000069605': NtO(MH='Olopatadine Hydrochloride', MS='An antihistamine with mast-cell stabilizing properties used as eye drops in the treatment of ALLERGIC CONJUNCTIVITIS.'),
  'D000069616': NtO(MH='Simeprevir', MS='Oral HCV-PROTEASE INHIBITOR effective against hepatitis C virus (HCV) serine protease NS3/4A. It is used in the treatment of chronic hepatitis C (Antivirals) genotype 1 infection in adults with compensated liver disease, including CIRRHOSIS.'),
  'D000069636': NtO(MH='Pedestrians', MS='Persons traveling on foot.'),
  'D000069756': NtO(MH='Undocumented Immigrants', MS='Foreign nationals residing in a country without legal immigration status, or in violation of the immigration laws of that country.'),
  'D000069796': NtO(MH='Adipose Tissue, Beige', MS='Brown fat-like adipose tissue that develops in WHITE ADIPOSE TISSUE from non-MYOGENIC REGULATORY FACTOR 5 expressing cell lineage.'),
  'D000069797': NtO(MH='Adipocytes, Beige', MS='Brown fat-like cells that develop in the WHITE FAT from non-MYOGENIC REGULATORY FACTOR 5 expressing CELL LINEAGE.'),
  'D000069816': NtO(MH='SEC Translocation Channels', MS='Universally conserved multiprotein complexes that form the protein transport channel of the general secretory (SEC) pathway. The SEC translocase is present in all bacteria, archaea, and eukaryotes. It is in the ENDOPLASMIC RETICULUM membrane of eukaryotic cells, in the THYLAKOID MEMBRANE in CHLOROPLASTS and in some protozoa in the INNER MITOCHONDRIAL MEMBRANE.'),
  'D000069836': NtO(MH='Degloving Injuries', MS='Avulsions of the superficial tissues of SKIN and SUBCUTANEOUS TISSUE from the underlying FASCIA.'),
  'D000069856': NtO(MH='Staghorn Calculi', MS='Renal calculi with a characteristic shape like antlers of a deer, formed as it extends into multiple calices of the RENAL PELVIS. A large number of staghorn calculi as well as other URINARY CALCULI are composed of STRUVITE.'),
  'D000069877': NtO(MH='Struvite', MS='The mineral magnesium ammonium phosphate with the formula NH4MgPO4. It is associated with urea-splitting organisms in a high magnesium, high phosphate, alkaline environment. Accumulation of crystallized struvite is found in the urinary tract as struvite CALCULI and as scale on sewage system equipment and wastewater pipes.'),
  'D000069896': NtO(MH='Transcription Activator-Like Effector Nucleases', MS='Artificial nucleases that cleave DNA at a defined distance from specific DNA sequences recognized by TRANSCRIPTION ACTIVATOR-LIKE EFFECTORS. They are composed of an endodeoxyribonuclease fused to DNA-binding domains of the transcription activator-like effectors.'),
  'D000069916': NtO(MH='Endoscopic Mucosal Resection', MS='A method for removing lesions from gastrointestinal MUCOUS MEMBRANES. The mucosal  tissue with the lesion is elevated by injecting a solution into the submucosal layer underneath it. The elevated tissue with the lesion is then cut out.'),
  'D000069956': NtO(MH='Chocolate', MS='Food product prepared from fermenting, roasting, and grinding seeds of the COCOA plant.'),
  'D000069976': NtO(MH='Bacillus licheniformis', MS='A species of  Bacillus that occurs in soil and is frequently found on bird feathers. Some strains cause FOODBORNE DISEASE, whereas other strains produce ANTIFUNGAL AGENTS and ANTIBACTERIAL AGENTS and function as PROBIOTICS. This species can also produce enzymes for industrial applications.'),
  'D000069977': NtO(MH='Bacillus amyloliquefaciens', MS='A non-pathogenic species of Bacillus that occurs in soil as well as some fermented vegetables and FERMENTED DAIRY PRODUCTS. It produces BACTERIOCINS and ANTIFUNGAL AGENTS and is used in FOOD PRESERVATION as well as a PROBIOTIC.'),
  'D000069978': NtO(MH='Bifidobacterium longum', MS='A species of Bifidobacterium that occurs in the human GASTROINTESTINAL TRACT and VAGINA. It inhibits the growth of pathogenic bacteria, may modulate the immune response, and is used as a PROBIOTIC.'),
  'D000069979': NtO(MH='Bacillus pumilus', MS='A species of Bacillus that occurs in soil, especially around the roots of some plants where it has anti-fungal and anti-nematode activities. It can also produce enzymes for industrial applications.'),
  'D000069980': NtO(MH='Lactobacillus paracasei', MS='A species of Lactobacillus that occurs in the GUT MICROBIOTA of healthy humans as well as FERMENTED DAIRY PRODUCTS and fermented vegetables. It is used as a PROBIOTIC.'),
  'D000069981': NtO(MH='Escherichia coli O104', MS='An enterohemorrhagic Escherichia coli of the O subfamily that can cause severe FOODBORNE DISEASE. The H4 serotype strain produces SHIGA TOXINS and has been linked to human disease outbreaks, including some cases of HEMOLYTIC-UREMIC SYNDROME, resulting from contamination of foods by feces containing E. coli O104.'),
  'D000069982': NtO(MH='Bifidobacterium animalis', MS='A species of Bifidobacterium that occurs in the LARGE INTESTINE of humans and other mammals and in FERMENTED DAIRY PRODUCTS. It is used as a PROBIOTIC.'),
  'D000069983': NtO(MH='Saccharomyces boulardii', MS='A species of Saccharomyces that is used as a PROBIOTIC, such as in the treatment of DIARRHEA and PSEUDOMEMBRANOUS ENTEROCOLITIS associated with CLOSTRIDIUM INFECTIONS.'),
  'D000069984': NtO(MH='Bifidobacterium breve', MS='A species of Bifidobacterium present in the human GUT MICROBIOTA. It is used as a PROBIOTIC.'),
  'D000069985': NtO(MH='Bifidobacterium bifidum', MS='A species of Bifidobacterium that occurs in the human GASTROINTESTINAL TRACT, especially in infants, and in FERMENTED DAIRY PRODUCTS. It produces BACTERIOCINS and is used as a PROBIOTIC.'),
  'D000069996': NtO(MH='Lactobacillus salivarius', MS='A species of Lactobacillus that occurs in the human GASTROINTESTINAL TRACT and ORAL MUCOSA. It produces BACTERIOCINS and is used as a PROBIOTIC.'),
  'D000069997': NtO(MH='Mastic Resin', MS='Dried tree sap from the MASTIC TREE that is used in TRADITIONAL MEDICINE and chewed as gum.'),
  'D000069998': NtO(MH='Lactobacillus gasseri', MS='A species of Lactobacillus that occurs in the human ORAL MUCOSA; GASTROINTESTINAL TRACT; and VAGINA. It produces BACTERIOCINS, can modulate the immune response, and is used as a PROBIOTIC.'),
  'D000069999': NtO(MH='Lactobacillus johnsonii', MS='A species of Lactobacillus that occurs in the human GASTROINTESTINAL TRACT and VAGINA. It produces BACTERIOCINS and HYDROGEN PEROXIDE and is used as a PROBIOTIC.'),
  'D000070': NtO(MH='Acebutolol', MS='A cardioselective beta-1 adrenergic antagonist with little effect on the bronchial receptors. The drug has stabilizing and quinidine-like effects on cardiac rhythm, as well as weak inherent sympathomimetic action.'),
  'D000070000': NtO(MH='Bacillus coagulans', MS='A microaerophilic, LACTIC ACID producing species of Bacillus that occurs in FERMENTED FOODS AND BEVERAGES. It also produces ANTI-INFECTIVE AGENTS and is used as a PROBIOTIC.'),
  'D000070001': NtO(MH='Streptococcus salivarius', MS='A species of Streptococcus that occurs in the human oral cavity and upper respiratory tract. It is a constituent of DENTAL PLAQUE and some strains may be pathogenic if they enter the bloodstream. However, other strains such as Streptococcus salivarius K12 produce BACTERIOCINS and are used as PROBIOTICS.'),
  'D000070002': NtO(MH='Propionibacterium freudenreichii', MS='A species of Propionibacterium that occurs in raw milk and cheese. It produces ANTI-INFECTIVE AGENTS and ANTI-INFLAMMATORY AGENTS and is used as a PROBIOTIC.'),
  'D000070003': NtO(MH='Lactobacillus crispatus', MS='A species of Lactobacillus that occurs in the human GASTROINTESTINAL TRACT and the VAGINA of healthy women. It produces LACTIC ACID and HYDROGEN PEROXIDE, and is used as a PROBIOTIC. It is also used for the treatment and prevention of BACTERIAL VAGINOSIS.'),
  'D000070004': NtO(MH='Ocimum sanctum', MS='A plant species of the genus OCIMUM, family LAMIACEAE that has been used in TRADITIONAL MEDICINE.'),
  'D000070005': NtO(MH='Bifidobacterium adolescentis', MS='A species of Bifidobacterium that occurs in the GASTROINTESTINAL TRACT of mammals and healthy humans from birth until late adulthood. It produces ANTI-VIRAL AGENTS and ANTI-INFLAMMATORY AGENTS and is used as a PROBIOTIC.'),
  'D000070006': NtO(MH='Pediococcus pentosaceus', MS='A species of Pediococcus that occurs in FERMENTED FOODS AND BEVERAGES where its ability to produce LACTIC ACID and PEDIOCINS make it useful as a FOOD PRESERVATION agent. It is also used as a PROBIOTIC.'),
  'D000070007': NtO(MH='Leuconostoc mesenteroides', MS='A species of Leuconostoc that occurs on fruits and vegetables and in their fermented products, as well as FERMENTED DAIRY PRODUCTS. It produces LACTIC ACID and BACTERIOCINS and is used as a PROBIOTIC; however, it has also caused infections in immunocompromised patients.'),
  'D000070016': NtO(MH='Pediococcus acidilactici', MS='A species of Pediococcus that occurs in FERMENTED FOODS AND BEVERAGES where its ability to produce LACTIC ACID and PEDIOCINS makes it useful in FOOD PRESERVATION. It is also used as a PROBIOTIC.'),
  'D000070017': NtO(MH='Pediocins', MS='Bacteriocins produced by species of PEDIOCOCCUS. They range from 3.5-4.6 kDa in mass, contain a conserved YGNGVXCXK motif and beta sheet at their N-terminals, and a more diverse hydrophobic or amphiphilic C-terminal alpha helical domain. They function as antimicrobial peptides against several pathogenic species of GRAM POSITIVE BACTERIA, including LISTERIA MONOCYTOGENES and are useful as FOOD PRESERVATIVES.'),
  'D000070018': NtO(MH='Lactobacillus pentosus', MS='A species of Lactobacillus that occurs in FERMENTED FOODS AND BEVERAGES where its ability to produce LACTIC ACID; ANTI-INFECTIVE AGENTS; and BACTERIOCINS make it useful as a FOOD PRESERVATION agent. It is also used as a PROBIOTIC.'),
  'D000070019': NtO(MH='Bacillus clausii', MS='An alkalophilic species of Bacillus that occurs in soil. It produces ANTI-BACTERIAL AGENTS and is used as a PROBIOTIC.'),
  'D000070036': NtO(MH='Faecalibacterium', MS='A genus of Gram-negative, non-spore forming, anaerobic, rod shaped bacteria in the Ruminococcaceae family, order Clostridiales that occur in the human gut.'),
  'D000070037': NtO(MH='Faecalibacterium prausnitzii', MS='A species of FAECALIBACTERIUM, previously classified in the FUSOBACTERIUM genus, that is a major constituent of the GUT MICROBIOTA in healthy humans. It has anti-inflammatory activity and reduced numbers of this species occur in patients with INFLAMMATORY BOWEL DISEASES such as CROHN DISEASE.'),
  'D000070057': NtO(MH='Paenibacillus polymyxa', MS='A species of Paenibacillus formerly classified as Bacillus polymyxa that occurs in marine sediments and soil, especially around plant roots and RHIZOMES where it performs NITROGEN FIXATION. It has activity against several species of pathogenic bacteria and produces POLYMYXINS.'),
  'D000070058': NtO(MH='Streptococcus iniae', MS='A species of Streptococcus that is pathogenic in fish and may also cause disease in humans who routinely handle infected fish. Those who are of Asian descent, elderly, or have chronic diseases such as DIABETES MELLITUS; RHEUMATIC HEART DISEASE; or LIVER CIRRHOSIS are more susceptible to Streptococcus iniae infections.'),
  'D000070059': NtO(MH='Paenibacillus larvae', MS='A species of Paenibacillus that causes American Foulbrood, a fatal disease of honeybees (APIS) and honeybee colonies.'),
  'D000070060': NtO(MH='Bdellovibrio bacteriovorus', MS='A non-pathogenic, obligate aerobic, Gram-negative species of Bdellovibrio that occurs in water and soil, as well as the intestines of birds and mammals. It can infect and lyse pathogenic GRAM-NEGATIVE BACTERIA and can function as a PROBIOTIC and ANTI-BACTERIAL AGENT.'),
  'D000070076': NtO(MH='Enterococcus hirae', MS='A species of Enterococcus that occurs in the GASTROINTESTINAL TRACT of birds and mammals, and to a lesser extent humans. Some strains cause diseases including DIARRHEA; BACTEREMIA; and ENDOCARDITIS  in humans and animals.'),
  'D000070077': NtO(MH='Bacteroides thetaiotaomicron', MS='A species of Bacteroides that is a major constituent of the human GUT MICROBIOTA. It normally produces enzymes important for the digestion of vegetable matter. However, it can also cause OPPORTUNISTIC INFECTIONS that result in intra-abdominal SEPSIS; BACTEREMIA; and PERITONITIS.'),
  'D000070079': NtO(MH='Aeromonas veronii', MS='A species of Aeromonas that occurs mostly as single cells in soil and freshwater environments. It also inhabits the digestive tract of the leech HIRUDO MEDICINALIS, where it aids in the digestion of blood and prevents colonization by other bacteria. It is pathogenic in fish and potentially in humans who consume infected fish or immunocompromised patients receiving LEECH THERAPY.'),
  'D000070096': NtO(MH='Pseudomonas chlororaphis', MS='A non-pathogenic species of Pseudomonas that occurs in soil, especially in the RHIZOSPHERE surrounding plant roots, as well as on the leaves of plants. It has anti-fungal and anti-nematode activities and is used in agriculture as a biocontrol agent.'),
  'D000070097': NtO(MH='Butyrivibrio fibrisolvens', MS='A species of Butyrivibrio that occurs primarily in the RUMEN where it has important activities in LIPID METABOLISM and the production of BUTYRATES.'),
  'D000070098': NtO(MH='Bacillus firmus', MS='A species of Bacillus that occurs in soil and marine sediments. Many strains are alkalophilic and able to metabolize HEAVY METALS; it may therefore be a useful species for ENVIRONMENTAL BIODEGRADATION.'),
  'D000070099': NtO(MH='Streptococcus gallolyticus', MS='A species of Streptococcus that occurs in the GASTROINTESTINAL TRACT of birds, mammals and healthy humans.'),
  'D000070100': NtO(MH='Informal Sector', MS='A component of the private sector characterized by ease of entry; reliance on indigenous resources; family ownership; small scale operations;  skills acquired outside of the formal sector; and unregulated markets.'),
  'D000070101': NtO(MH='Megasphaera elsdenii', MS='A species of Megasphaera that occurs primarily in the RUMEN and is able to metabolize LACTIC ACID.'),
  'D000070116': NtO(MH='Lotus', MS='A genus of the PEA FAMILY. The genus Lotus, formerly known as Tetragonolobus, is unrelated to other plants with the common name of lotus (NELUMBO and NYMPHAEA).'),
  'D000070136': NtO(MH='Lipid Droplet Associated Proteins', MS='Proteins, such as PERILIPINS, that localize to LIPID DROPLETS either transiently or constitutively.'),
  'D000070137': NtO(MH='Perilipins', MS='A family of vertebrate and insect lipid droplet associated proteins. They consist of a conserved N-terminal PAT domain (an alpha-helical region of about 110 amino acids), an 11-mer repeat region, and lipid-binding hydrophobic regions or 4-helix bundles near their C-termini. Perilipins transiently or constitutively localize to LIPID DROPLETS in ADIPOCYTES and FOAM CELLS, especially in regions adjacent to the PLASMA MEMBRANE and ENDOPLASMIC RECTICULUM. They are critical for lipid droplet synthesis and homeostasis as well as the regulation of lipid metabolism. Genetic variations in perilipins are associated with ATHEROSCLEROSIS; OBESITY; and DIABETES MELLITUS.'),
  'D000070156': NtO(MH='Nogo Receptors', MS='GPI-linked proteins consisting of eight elongated leucine-rich repeats at their N-temini that are connected to the GPI  by a "stalk" region rich in prolines, serines and threonines. They bind to NOGO PROTEIN; however, some Nogo receptors also bind MYELIN ASSOCIATED GLYCOPROTEIN and other cell surface glycoproteins. Interactions between nogo receptors and their ligands modulate nerve growth and NEURONAL PLASTICITY.'),
  'D000070177': NtO(MH='Pogostemon', MS='Genus in the Lamiaceae family. VOLATILE OILS from several species of Pogostemon are used in PERFUME; INSECT REPELLENTS; and ALTERNATIVE THERAPIES.'),
  'D000070197': NtO(MH='Streptococcus gallolyticus subspecies gallolyticus', MS='A subspecies of Streptococcus gallolyticus. It and other subspecies have been reported to cause infections resulting in BACTEREMIA; ENDOCARDITIS; and neonatal MENINGITIS.'),
  'D000070216': NtO(MH='Psacalium', MS='A genus of the Asteraceae family. Several of its species are used in TRADITIONAL MEDICINE.'),
  'D000070236': NtO(MH='Bifidobacterium longum subspecies infantis', MS='A subspecies of Bifidobacterium longum that occurs in the GASTROINTESTINAL TRACT of human infants and is used as a PROBIOTIC. It may also be used in the treatment of IRRITABLE BOWEL SYNDROME.'),
  'D000070256': NtO(MH='Lactobacillus sakei', MS='A species of Lactobacillus that occurs in fermented meat and fish. It produces the BACTERIOCIN Sakacin P and is used for FOOD PRESERVATION and as a PROBIOTIC.'),
  'D000070257': NtO(MH='Bifidobacterium pseudocatenulatum', MS='A species of Bifidobacterium that occurs in the GASTROINTESTINAL TRACT of mammals and healthy humans. It has anti-inflammatory activity and is used as a PROBIOTIC.'),
  'D000070260': NtO(MH='Teacher Training', MS='A curriculum-centered process of equipping individuals with professional knowledge and skills required to become effective teachers.'),
  'D000070263': NtO(MH='Sleep Hygiene', MS="Habits and practices conducive to getting the right amount and quality of sleep, and include  responding to environmental factors that may influence one's sleep."),
  'D000070276': NtO(MH='Heart Rate Determination', MS='Methods, techniques, and processes by which HEART RATE is measured.'),
  'D000070297': NtO(MH='Orthodontic Friction', MS='Physical resistance to motion between dental surfaces.'),
  'D000070316': NtO(MH='Pediatric Emergency Medicine', MS='Branch of EMERGENCY MEDICINE dealing with the emergency care of children.'),
  'D000070318': NtO(MH='Strategic Planning', MS='A rigorous and systematic process which entails defining plans, decisions, and sequence of steps to be taken in the future.'),
  'D000070319': NtO(MH='Student Run Clinic', MS='Healthcare setting managed by medical personnel in training.'),
  'D000070337': NtO(MH='Expropriation', MS='Taking of goods, property, or right of use by GOVERNMENT or others.'),
  'D000070356': NtO(MH='Pilots', MS='Persons skilled and/or qualified to operate an aircraft or ship.'),
  'D000070357': NtO(MH='Asparagaceae', MS='A family of flowering subshrubs and shrubs in the class Magnoliopsida.'),
  'D000070376': NtO(MH='Value-Based Health Insurance', MS="Approach to improve the quality of care by selectively encouraging or discouraging the use of specific health care services, based on their potential benefit to patients' health, relative to their cost. One element is lowering beneficiary cost sharing or out-of-pocket spending to increase medication adherence."),
  'D000070377': NtO(MH='Anti-Vaccination Movement', MS='Group activities directed against VACCINATION.'),
  'D000070378': NtO(MH='Amaryllidaceae', MS='A family of herbaceous plants with bulbs or rhizomes in the order Asparagales.'),
  'D000070379': NtO(MH='Attentional Bias', MS='An increased focus or awareness of certain stimuli over others, which influence behavior.'),
  'D000070380': NtO(MH='Xanthorrhoeaceae', MS='A family of flowering plants in the order Asparagales.'),
  'D000070381': NtO(MH='Asparagales', MS='An order of monocotyledons plants in the superorder Lilianae of the class Magnoliopsida (Angiosperms).'),
  'D000070396': NtO(MH='Liliales', MS='An order of monocots that grow from bulbs or corms to become herbs or vines.'),
  'D000070416': NtO(MH='Antagomirs', MS='Chemically-engineered oligonucleotides used to selectively inhibit expression of target genes through sequence-specific binding of corresponding microRNA (miRNA) sites.'),
  'D000070436': NtO(MH='Tannerella forsythia', MS='A periodontal pathogen which is a gram-negative member of BACTEROIDETES. It is implicated in PERIODONTITIS.'),
  'D000070437': NtO(MH='Colchicaceae', MS='A family in the order Liliales.'),
  'D000070456': NtO(MH='Call Centers', MS='A facility set up for the purpose of handling large volumes of telephone calls. Call Centers typically utilize some form of computer automation for receiving, dispatching, screening, logging and forwarding telephone calls.'),
  'D000070476': NtO(MH='Hypoxidaceae', MS='A family of perennial herbaceous plants, in the order Asparagales, with the underground storage organs, rhizomes or corms.'),
  'D000070496': NtO(MH='Kinanthropometry', MS='A measure of human size, shape, proportion, composition, maturation, and gross function; in relation to body movement, physical exercise, fitness, and performance.'),
  'D000070497': NtO(MH='Healthy Lifestyle', MS='A pattern of behavior involving LIFE STYLE choices which ensure optimum health. Examples are eating right; maintaining physical, emotional, and spiritual wellness, and taking preemptive steps against communicable diseases.'),
  'D000070516': NtO(MH='Intersectoral Collaboration', MS='Cooperative actions and ventures among health and health-related groups and organizations intended to improve health outcomes.'),
  'D000070536': NtO(MH='Independent Medical Evaluation', MS='An assessment of HEALTH STATUS requested by third parties (e.g. insurers, adjudicating bodies, law enforcement, attorneys, etc) on a subject\xe2\x80\x99s functional ability. Such information may be used in optimizing treatment or making informed decisions on compensation.'),
  'D000070538': NtO(MH='C2 Domains', MS='Protein modules that function in the targeting of proteins to CELL MEMBRANES. They consist of an eight-stranded anti-parallel beta-sandwich composed of a pair of four-stranded beta-sheets. This structural unit forms a pocket on the membrane-interacting face of the protein and co-ordinates the binding of 2 to 3 calcium ions; however, not all C2 domains bind calcium. Examples of C2 domain-containing proteins include PROTEIN KINASE C and PTEN PHOSPHOHYDROLASE.'),
  'D000070539': NtO(MH='Pleckstrin Homology Domains', MS='Protein domains of approximately 120 amino acids that form two perpendicular anti-parallel beta sheets connected by a loop of variable length and a C-terminal amphipathic helix. PH domains occur in many INTRACELLULAR SIGNALING PROTEINS and CYTOSKELETAL PROTEINS where they bind PHOSPHATIDYLINOSITOLS within the CELL MEMBRANE and INTRACELLULAR MEMBRANES, as well as proteins that include the Betagamma-subunits of  HETEROTRIMERIC GTP-BINDING PROTEINS, and PROTEIN KINASE C. These interactions allow the targeting of proteins to different cellular compartments and SIGNAL TRANSDUCTION PATHWAYS.'),
  'D000070556': NtO(MH='Melanthiaceae', MS='A family in the order Liliales, of bulbous or rhizomatous woodland and alpine perennial herbs.'),
  'D000070557': NtO(MH='Immunoglobulin Domains', MS='Compact globular protein domains of about 80 to 110 amino acids.They are characterized by two antiparallel beta sheets stabilized by HYDROPHOBIC INTERACTIONS and disulfide bonds between opposing strands. Three major subtypes of Ig domains are recognized: variable (V), intermediate (I), and constant (C1 or C2). Members of the immunoglobulin superfamily include ANTIGEN RECEPTORS; CYTOKINE RECEPTORS; and CELL ADHESION MOLECULES which possess immunoglobulin domains.'),
  'D000070558': NtO(MH='Talipes', MS='Deformity in which the foot is misaligned with respect to the TALUS in the ANKLE JOINT. While mostly congenital, as in CLUBFOOT, acquired deformities are included. Acquired talipedes are often associated with other foot deformities such as SYNDACTYLY and POLYDACTYLY.'),
  'D000070559': NtO(MH='Gizzard, Non-avian', MS='A component of the gastrointestinal tract of some reptiles (ALLIGATORS AND CROCODILES), as well as certain fish, mollusks, and insects. It functions to mechanically grind food.'),
  'D000070576': NtO(MH='NLR Proteins', MS='Intracellular signaling proteins that are defined by the presence of a NUCLEOTIDE-binding region and LEUCINE-rich repeats. Their general structure consists of any of a variety of effector domains at their N-termini such as a caspase recruitment domain (CARD), a central nucleotide-binding domain, and a variable number of C-terminal leucine-rich repeats. They are important for pathogen recognition in the INNATE IMMUNE RESPONSE of animals and plants. Members of the NLR protein family include the NOD SIGNALING ADAPTOR PROTEINS.'),
  'D000070579': NtO(MH='Parkinson Disease Associated Proteins', MS='Proteins associated with sporadic or familial cases of PARKINSON DISEASE.'),
  'D000070580': NtO(MH='CYS2-HIS2 Zinc Fingers', MS='A zinc finger motif of approximately 30 amino acids with the general sequence X2-Cys-X2,4-Cys-X12-His-X3,4,5-His that forms a simple beta sheet-beta sheet-alpha helix fold stabilized by zinc ions. It recognizes and binds to a variety of eukaryotic DNA sequences and is very common among sequence-specific DNA BINDING PROTEINS and TRANSCRIPTION FACTORS.'),
  'D000070584': NtO(MH='Tripartite Motif Proteins', MS='A protein family defined by the presence of three ZINC FINGER domains, one of which is a RING FINGER DOMAIN, a coiled-coil region, and a highly variable C-terminal region. They function in many cellular processes including APOPTOSIS and CELL CYCLE regulation.'),
  'D000070585': NtO(MH='Cadherin Related Proteins', MS='Membrane proteins that contain at least two typical consecutive CADHERIN motifs but typically more than cadherin or protocadherin proteins. Their overall domain organization also differs from those of cadherins and protocadherins, and contains different and unique cytoplasmic domains.'),
  'D000070586': NtO(MH='Fiscal Policy', MS='Use of government spending and taxation to influence the economy.'),
  'D000070587': NtO(MH='Mental Navigation Tests', MS='Measures of spatial cognition and memory.'),
  'D000070589': NtO(MH='Talipes Cavus', MS='A foot deformity in which the arch of the foot is high and often the heel adducted.'),
  'D000070590': NtO(MH='Solute Carrier Proteins', MS='A large diverse group of membrane transport proteins whose families are generally classified according to function. Most SLCs localize to the CELL MEMBRANE; however, some families such as SLC25, localize to MITOCHONDRIAL MEMBRANES or other ORGANELLES.'),
  'D000070591': NtO(MH='Metatarsal Valgus', MS='A foot anomaly in which the forefoot is angled outward relative to the hindfoot.'),
  'D000070592': NtO(MH='Metatarsus Varus', MS='A foot anomaly in which the forefoot (METATARSUS) is angled inward relative to the heel.'),
  'D000070593': NtO(MH='Methyl CpG Binding Domain', MS='A protein domain of about 70 amino acids in length that folds into a sandwich structure consisting of a beta sheet, alpha helix, and C-terminal hairpin loop which forms a hydrophobic methylated-DNA binding patch. It binds to methylated CPG ISLANDS in eukaryotes and occurs with other DNA and CHROMATIN binding domains such as AT-HOOK MOTIFS in eukaryotic DNA BINDING PROTEINS.'),
  'D000070597': NtO(MH='Coracoid Process', MS='A beak-like bone projection located at the lateral end of the superior anterior border of the SCAPULA.'),
  'D000070598': NtO(MH='Anterior Cruciate Ligament Injuries', MS='Sprain or tear injuries to the ANTERIOR CRUCIATE LIGAMENT of the knee.'),
  'D000070599': NtO(MH='Shoulder Injuries', MS='Injuries involving the SHOULDERS.'),
  'D000070600': NtO(MH='Tibial Meniscus Injuries', MS='Injuries to the TIBIAL MENISCUS of the leg.'),
  'D000070601': NtO(MH='Diet, Carbohydrate Loading', MS='A diet rich in DIETARY CARBOHYDRATES.'),
  'D000070602': NtO(MH='Rib Cage', MS='The bony thoracic enclosure consisting of the vertebral column; the RIBS; the STERNUM; and the COSTAL CARTILAGE.'),
  'D000070603': NtO(MH='Coxa Magna', MS='Deformity of the hip characterized by enlargement and deformation of the FEMUR HEAD and FEMUR NECK, often with associated changes in the ACETABULUM. These changes may be secondary to other diseases (e.g.  LEGG-PERTHES DISEASE; ARTHRITIS; HIP DISLOCATION, CONGENITAL) or TRAUMA.'),
  'D000070604': NtO(MH='Tarsal Coalition', MS='Congenital, complete or partial fusion of the TARSAL BONES of the foot. PES PLANUS is usually a feature.'),
  'D000070606': NtO(MH='Aponeurosis', MS='The flattened and/or expanded \xe2\x80\x9ctendon-like\xe2\x80\x9d dense fibrous connective tissue that connects skeletal muscles to each other or to bone.'),
  'D000070607': NtO(MH='Morton Neuroma', MS='A nerve inflammation in the foot caused by chronic compression of the plantar nerve between the METATARSAL BONES.'),
  'D000070614': NtO(MH='Nucleus Pulposus', MS='Fibrocartilage inner core of the intervertebral disc. Prolapsed or bulged nucleus pulposus leads to INTERVERTEBRAL DISC DISPLACEMENT while proliferation of cells in the nucleus pulposus is associated with INTERVERTEBRAL DISC DEGENERATION.'),
  'D000070616': NtO(MH='Annulus Fibrosus', MS='Outer portion of the intervertebral disc that surrounds the NUCLEUS PULPOSUS.'),
  'D000070617': NtO(MH='Microtrauma, Physical', MS='Small injuries caused by external force applied to the body including bones, muscles, nerves and tendons.'),
  'D000070621': NtO(MH='Cystography', MS='Radiography of the URINARY BLADDER.'),
  'D000070624': NtO(MH='Brain Contusion', MS='A bruise of the brain from an impact of the skull.'),
  'D000070625': NtO(MH='Brain Injuries, Diffuse', MS='Brain injuries occurring over a wide area instead of specific focal area.'),
  'D000070627': NtO(MH='Chronic Traumatic Encephalopathy', MS='Degenerative brain disease linked to repetitive brain trauma. Progressive symptoms may include MEMORY LOSS; AGGRESSION; or DEPRESSION.'),
  'D000070628': NtO(MH='Closed Fracture Reduction', MS='A procedure in which normal alignment of a fractured bone is restored by ORTHOPEDIC MANIPULATION without incision.'),
  'D000070630': NtO(MH='Diastasis, Muscle', MS='Abnormal separation of muscles.'),
  'D000070631': NtO(MH='Diastasis, Bone', MS='Abnormal separation of bones, often from a LIGAMENT.'),
  'D000070632': NtO(MH='Hamstring Tendons', MS='A group of tendons that attach the HAMSTRING MUSCLES proximally to the PELVIS and to the TIBIA and FIBULA at the KNEE JOINT.'),
  'D000070633': NtO(MH='Hamstring Muscles', MS='A group of muscles in the posterior THIGH area that are involved in connecting the KNEE JOINT and the PELVIS.'),
  'D000070634': NtO(MH='Non-Neuronal Cholinergic System', MS='The system of ACETYLCHOLINE-synthesizing enzymes, transporters, receptors and degrading enzymes that characterize non-neuronal cholinergic cells such as airway and skin EPITHELIAL CELLS.'),
  'D000070635': NtO(MH='Punctal Plugs', MS='Small devices that are inserted into the tear ducts (NASOLACRIMAL DUCTS). They are used to block the drainage of TEARS for the treatment of DRY EYE SYNDROMES.'),
  'D000070636': NtO(MH='Rotator Cuff Injuries', MS='Injuries to the ROTATOR CUFF of the shoulder joint.'),
  'D000070637': NtO(MH='Collateral Ligament, Ulnar', MS='The internal lateral ligament that travels from the medial aspect of the ELBOW uniting the distal aspect of the HUMERUS to the proximal aspect of the ULNA.'),
  'D000070638': NtO(MH='Ulnar Collateral Ligament Reconstruction', MS='Surgical reconstruction of injured or insufficient ULNAR COLLATERAL LIGAMENT in the ELBOW JOINT.'),
  'D000070639': NtO(MH='Elbow Tendinopathy', MS='Inflammation (tendinitis) or degeneration (tendinosis) of the tendons of the elbow.'),
  'D000070642': NtO(MH='Brain Injuries, Traumatic', MS='A form of acquired brain injury which occurs when a sudden trauma causes damage to the brain.'),
  'D000070656': NtO(MH='Rotator Cuff Tear Arthropathy', MS='Rapidly destructive shoulder joint and bone disease found mainly in elderly, and predominantly in women. It is characterized by SHOULDER PAIN; JOINT INSTABILITY; and the presence of crystalline CALCIUM PHOSPHATES in the SYNOVIAL FLUID. It is associated with ROTATOR CUFF INJURIES.'),
  'D000070657': NtO(MH='Crystal Arthropathies', MS='Joint disorders that are characterized by accumulation of microcrystals in and around the joint including in the SYNOVIAL FLUID. They are classified according to the chemical nature of the crystals such as CALCIUM PYROPHOSPHATE; basic CALCIUM PHOSPHATES; and monosodium urate (see URIC ACID).'),
  'D000070658': NtO(MH='Vigna', MS='Plant genus in the pea family (Fabaceae).'),
  'D000070659': NtO(MH='Patient Comfort', MS='Patient care intended to prevent or relieve suffering in conditions that ensure optimal quality living.'),
  'D000070660': NtO(MH='Wolfiporia', MS='A genus of wood-decaying, edible fungi in the Polyporaceae family. Its type species is Wolfiporia extensa.'),
  'D000070676': NtO(MH='Enthesopathy', MS='A disorder occurring at the site of insertion of TENDONS or LIGAMENTS into bones or JOINT CAPSULES.'),
  'D000070677': NtO(MH='Gentianales', MS='An angiosperm plant order that includes the APOCYNACEAE; ASCLEPIADACEAE; GENTIANACEAE; LOGANIACEAE; and RUBIACEAE  families.'),
  'D000070696': NtO(MH='Buttermilk', MS="A beverage made from cow's milk fermented by LACTIC ACID-producing bacteria, especially LACTOCOCCUS LACTIS and LACTOBACILLUS BULGARICUS."),
  'D000070716': NtO(MH='Kefir', MS='A beverage made from milk fermented by a mixture of endogenous LACTIC ACID-producing yeast and bacteria (KEFIR GRAINS), many of which are PROBIOTICS. It should not be confused with KAFFIR LIME or KAFFIR CORN.'),
  'D000070736': NtO(MH='Koumiss', MS="A beverage made from horse's milk that is fermented by endogenous LACTIC ACID-producing bacteria and ETHANOL-producing yeast."),
  'D000070777': NtO(MH='School Teachers', MS='Members of staff responsible for TEACHING students in a school from pre-kindergarten through twelfth grade.'),
  'D000070778': NtO(MH='Perilipin-1', MS='A lipid droplet protein that is expressed primarily by ADIPOCYTES of WHITE ADIPOSE TISSUE and BROWN ADIPOSE TISSUE. It co-localizes with MACROPHAGES and FOAM CELLS of artherosclerotic lesions and stabilizes LIPID DROPLETS by inhibiting HORMONE SENSITIVE LIPASE. It may also protect TRIGLYCERIDES against hydrolysis within the PLASMA MEMBRANE and modulate CHOLESTEROL ESTER HYDROLASE activity.'),
  'D000070779': NtO(MH='Giant Cell Tumor of Tendon Sheath', MS='A tumor arising in the SYNOVIAL MEMBRANE; SYNOVIAL BURSA; or TENDON sheath. It is characterized by OSTEOCLAST-like GIANT CELLS; FOAM CELLS; pigmented HEMOSIDERIN-laden MACROPHAGES and inflammatory infiltrate. It is classified either as diffuse or localized tenosynovitis.'),
  'D000070780': NtO(MH='Perilipin-2', MS='A perilipin that is expressed by many different cell types. It binds FATTY ACIDS and CHOLESTEROL,  stabilizes TRIGLYCERIDES, and localizes to both the surface and hydrophobic core of LIPID DROPLETS, as well as the ENDOPLASMIC RECTICULUM and PLASMA MEMBRANE in MACROPHAGES. It also plays a central role in the biogenesis of lipid droplets and FOAM CELLS and is highly expressed by macrophages at atherosclerotic lesions in human arteries along with the INFLAMMATION markers TNF-ALPHA; MCP-1 RECEPTOR; and IL-6.'),
  'D000070796': NtO(MH='Perilipin-3', MS='A perilipin that localizes to LIPID DROPLETS; CYTOPLASM; ENDOSOMES; and PLASMA MEMBRANE, especially in MACROPHAGES. It functions as a transporter of free fatty acids to lipid droplets to promote their biogenesis and growth. It is also required for the transport of the MANNOSE-6-PHOSPHATE RECEPTOR from endosomes to the TRANS-GOLGI NETWORK. Its structure consists of four helix bundles that interact with the hydrophobic lipid droplet surface.'),
  'D000070797': NtO(MH='Perilipin-5', MS='A perilipin that functions in LIPOGENESIS; LIPOLYSIS; and fatty acid oxidation in BROWN ADIPOSE TISSUE; heart, liver, and skeletal muscle. It recruits MITOCHONDRIA to the surface of LIPID DROPLETS where it functions in both the storage of fatty acids as TRIGLYCERIDES, and their release for mitochondrial fatty acid oxidation in response to metabolic needs.'),
  'D000070798': NtO(MH='Nogo Proteins', MS='Myelin proteins that are expressed as three isoforms: Nogo-A, Nogo-B, and Nogo-C. These share a C-terminal reticulon homology domain (RHD), consisting of two hydrophobic membrane domains flanking a 66 amino acid (Nogo-66) hydrophilic region. A long transmembrane region allows conformations that either span the entire membrane or fold into a hairpin conformation. Nogo inhibits NEURITE outgrowth and modulates wiring and the restriction of SYNAPTIC PLASTICITY in the adult central nervous system. It also regulates neurite fasciculation, branching, and extension in the developing nervous system.'),
  'D000070799': NtO(MH='Open Fracture Reduction', MS='Restoration of normal alignment of a fractured bone via an OPERATIVE SURGICAL PROCEDURE.'),
  'D000070816': NtO(MH='Nogo Receptor 1', MS='A high affinity receptor for myelin-associated inhibitors (MAIs) that include NOGO-A PROTEIN; OLIGODENDROCYTE MYELIN GLYCOPROTEIN; and MYELIN-ASSOCIATED GLYCOPROTEIN. It is expressed primarily by neurons in the brain and OLFACTORY BULBS. During embryonic development, it is expressed in the PERIPHERAL NERVOUS SYSTEM.  It localizes to GROWTH CONES and may inhibit neurite outgrowth following SPINAL INJURY.'),
  'D000070817': NtO(MH='Nogo Receptor 2', MS='A Nogo receptor that binds to MYELIN-ASSOCIATED GLYCOPROTEIN. It localizes to the GROWTH CONES of neurons in the THALAMUS; CORTEX: AMYGDALA; OLFACTORY BULBS; and HYPOTHALAMUS.'),
  'D000070818': NtO(MH='Discoidin Domain Receptors', MS='Receptor tyrosine kinases that bind COLLAGENS. They are characterized by the presence of extracellular DISCOIDIN DOMAINS that activate SIGNAL TRANSDUCTION PATHWAYS in response to collagen binding. They may also be cleaved by MATRIX METALLOPROTEINASES to mediate cell interactions with the EXTRACELLULAR MATRIX.'),
  'D000070819': NtO(MH='Discoidin Domain Receptor 1', MS='A discoidin domain receptor for FIBRILLAR COLLAGEN that functions in a variety of cellular processes. For example, it regulates cell attachment to the EXTRACELLULAR MATRIX, remodeling of the extracellular matrix, CELL MIGRATION; CELL DIFFERENTIATION; CELL PROLIFERATION; and CELL SURVIVAL.'),
  'D000070820': NtO(MH='Discoidin Domain Receptor 2', MS='A discoidin domain receptor for FIBRILLAR COLLAGEN and non-fibrillar COLLAGEN TYPE X. It functions in many cellular and developmental processes that include remodeling of the EXTRACELLULAR MATRIX; CELL MIGRATION; CELL DIFFERENTIATION; and CELL PROLIFERATION; as well as OSTEOGENESIS and the maturation of CHONDROCYTES.'),
  'D000070821': NtO(MH='Discoidins', MS='Lectins that were identified in DICTYOSTELIUM DISCOIDEUM. They bind to GALACTOSE and are involved in cell-substratum adhesion, maintenance of morphology during aggregation, and spore formation.'),
  'D000070822': NtO(MH='Discoidin Domain', MS='A protein domain originally identified in DISCOIDINS. It occurs in a variety of unrelated proteins in multicellular organisms, where it recognizes different ligands with distinct biological functions. Structurally, it consists of two antiparallel beta sheets that form a "jellyroll" configuration stabilized by two intramolecular disulfide bonds. Sequence differences within this module determine ligand specificity; in DISCOIDIN DOMAIN RECEPTORS, it binds to FIBRILLAR COLLAGENS and NON-FIBRILLAR COLLAGENS.'),
  'D000070837': NtO(MH='Lysine Acetyltransferases', MS='Acetyltransferases that can transfer an acyl group  to specific LYSINE residues within HISTONES (HISTONE ACETYLTRANSFERASES) and/or non-histone proteins.'),
  'D000070838': NtO(MH='Pharmacy Research', MS='RESEARCH activity focused on PHARMACY.'),
  'D000070840': NtO(MH='Nuclear Pharmacy', MS='An application of PHARMACY involving the preparation and dispensing of radioactive materials for use in medications.'),
  'D000070856': NtO(MH='High Fidelity Simulation Training', MS='A controlled learning environment that closely represents reality.'),
  'D000070857': NtO(MH='Walk Test', MS='A measure of endurance tests that show how far and fast an individual can walk without stopping within a certain period of time.'),
  'D000070876': NtO(MH='Tensins', MS='A family of multidomain microfilament proteins that bind ACTIN FILAMENTS and INTEGRINS at FOCAL ADHESIONS. They generally consist of an N-terminal domain with homology to PHOSPHOTYROSINE PHOSPHATASE, a C2 DOMAIN; unique central regions rich in PROLINE; ALANINE; GLYCINE; and SERINE; an SH2 DOMAIN; and a C-terminal phosphotyrosine-binding region. They are involved in CELL MIGRATION; CELL ADHESION; SIGNAL TRANSDUCTION; and reorganization of the CYTOSKELETON.'),
  'D000070896': NtO(MH='Bankart Lesions', MS='An anterior capsulolabral injury associated with a tear of the anteroinferior GLENOID LABRUM.'),
  'D000070916': NtO(MH='Tenocytes', MS='Elongated FIBROBLASTS and fibrocytes that lie between the collagen fibers and form the TENDON proper.'),
  'D000070918': NtO(MH='Synoviocytes', MS='Cells on the luminal surface of the SYNOVIAL MEMBRANE. Type A synoviocytes are MACROPHAGES responsible for waste removal from the joint cavity. Fibroblast-like type B synoviocytes are involved in production of joint matrix constituents (e.g., HYALURONAN; COLLAGEN; and FIBRONECTIN).'),
  'D000070956': NtO(MH='MutL Proteins', MS='DNA repair proteins that include the bacterial MutL protein and its eukaryotic homologs. They consist of a conserved N-terminal region with weak ATPase activity, an endonuclease motif, and a C-terminal domain that forms MutL homodimers or heterodimers between MLH1 and the PMS1, MISMATCH REPAIR ENDONUCLEASE PMS2; or MLH3 proteins. These complexes function in DNA repair pathways, primarily DNA MISMATCH REPAIR, where MutL/MLH1 and the MUTS DNA MISMATCH-BINDING PROTEIN are targeted to damaged DNA.'),
  'D000070957': NtO(MH='MutL Protein Homolog 1', MS='Eukaryotic homolog of the bacterial MutL DNA MISMATCH REPAIR protein. It heterodimerizes with MISMATCH REPAIR ENDONUCLEASE PMS2 to form MutL alpha, which is recruited to DNA mismatch sites by the MUTS DNA MISMATCH-BINDING PROTEIN. Mutations in the human MLH1 gene are associated with COLORECTAL NEOPLASMS, HEREDITARY NONPOLYPOSIS.'),
  'D000070976': NtO(MH='Mismatch Repair Endonuclease PMS2', MS='A MutL protein and component of the DNA MISMATCH REPAIR system. Its ENDONUCLEASE activity introduces SINGLE-STRAND DNA BREAKS which create entry points for EXO1 exonuclease to remove the strand containing the mismatch. It may also function in DNA DAMAGE signaling.'),
  'D000070996': NtO(MH='ATP Binding Cassette Transporter, Subfamily G', MS='A subfamily of ATP binding cassette transporters that function primarily in the transport of lipids and STEROLS across the CELL MEMBRANE. They also export UREA and various drugs resulting in MULTIDRUG RESISTANCE. They are smaller than most other ATP binding cassette proteins, consisting of six transmembrane alpha helices and a distinct N-terminal cytoplasmic ATP-binding domain, and function as homo- or heterodimers with other ABCG transporters.'),
  'D000070997': NtO(MH='ATP Binding Cassette Transporter, Subfamily G, Member 2', MS='ATP-binding cassette transporter, sub-family G protein that functions as a high capacity UREA exporter, transporter of STEROLS, and in the absorption and efflux of many drugs. Its efflux activity for ANTINEOPLASTIC AGENTS contributes to DRUG RESISTANCE. It functions as a homodimer and is expressed by cells in a variety of organs, as well as by NEOPLASTIC STEM CELLS.'),
  'D000070998': NtO(MH='ATP Binding Cassette Transporter, Subfamily G, Member 1', MS='ATP binding cassette transporter that functions primarily as a lipid and CHOLESTEROL exporter in MACROPHAGES.  It may also function in intracellular lipid transport and homoeostasis.'),
  'D000071017': NtO(MH='Hyperekplexia', MS='A neurological disorder characterized by an excessive startle reaction with ABNORMAL REFLEX;  MYOCLONIC JERKS; and MUSCLE HYPERTONIA.'),
  'D000071018': NtO(MH='ATP Binding Cassette Transporter, Subfamily G, Member 5', MS='ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY G protein that functions as a heterodimer with ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY G, MEMBER 8 in the selective transport of DIETARY CHOLESTEROL and PHYTOSTEROLS into and export out of ENTEROCYTES. It also functions in selective sterol excretion by the liver into bile.'),
  'D000071019': NtO(MH='ATP Binding Cassette Transporter, Subfamily G, Member 8', MS='ATP-binding cassette transporter, sub-family G protein that functions as a heterodimer with ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY G, MEMBER 5 in the selective transport of DIETARY CHOLESTEROL and PHYTOSTEROLS into and export out of ENTEROCYTES. It also functions in selective sterol excretion by the liver into bile.'),
  'D000071020': NtO(MH='Hemochromatosis Protein', MS='A membrane protein and MHC class I antigen. It contains an IMMUNOGLOBULIN C1-SET DOMAIN and interacts with BETA 2-MICROGLOBULIN. It may also regulate the interaction of TRANSFERRIN with the TRANSFERRIN RECEPTOR. Mutations in the HFE gene are associated with cases of FAMILIAL HEMOCHROMATOSIS.'),
  'D000071036': NtO(MH='Sentinel Lymph Node', MS='First lymph node to receive drainage from the primary tumor. SENTINEL LYMPH NODE BIOPSY is performed to determine early METASTASIS status because cancer cells may appear first in the sentinel node.'),
  'D000071037': NtO(MH='Place Cells', MS='Pyramidal neurons in the HIPPOCAMPUS that play a role in cognitive representation of a specific location within a space.'),
  'D000071038': NtO(MH='Grid Cells', MS='Neurons in the ENTORHINAL CORTEX that project to the HIPPOCAMPUS. Grid cells and PLACE CELLS play a role in cognitive representation of spatial memory and navigation.'),
  'D000071039': NtO(MH='Perirhinal Cortex', MS='An area in the temporal lobe that is important for memory encoding and retrieval. It is bordered caudally by the parahippocampal cortex (see HIPPOCAMPUS) and ventrally and medially by the ENTORHINAL CORTEX.'),
  'D000071040': NtO(MH='Axon Initial Segment', MS='First segment of axon that connects distal axon segments to the neuronal CELL BODY at the axon hillock region. The axon initial segment is not protected by the MYELIN SHEATH and has properties critical for axonal growth. The axon initial segment and the axon hillock form an axonal trigger zone.'),
  'D000071041': NtO(MH='Esophageal Mucosa', MS='Circular innermost layer of the ESOPHAGUS wall that mediates esophageal PERISTALSIS which pushes ingested food bolus toward the stomach.'),
  'D000071056': NtO(MH='Anorectal Malformations', MS='Congenital defects in the anus and the rectum often involving the urinary and genital tracts.'),
  'D000071057': NtO(MH='Tardive Dyskinesia', MS='Drug-related movement disorder characterized by uncontrollable movements in certain muscles. It is associated with a long-term exposure to certain neuroleptic medications (e.g., METOCLOPRAMIDE).'),
  'D000071058': NtO(MH='Huntingtin Protein', MS='A protein that is highly expressed in the nervous system as well as other tissues; its size and structure vary due to polymorphisms. Expanded CAG TRINUCLEOTIDE REPEATS have been identified in the Huntingtin (HD) Gene of patients with HUNTINGTON DISEASE and are associated with abnormal PROTEIN AGGREGATES. Huntingtin interacts with proteins involved in a variety of gene expression and cellular processes; it is also essential for embryonic development.'),
  'D000071059': NtO(MH='Phage Therapy', MS='Therapeutic use of BACTERIOPHAGES to treat diseases.'),
  'D000071060': NtO(MH='Breast Density', MS='Measurement of relative composition of different BREAST tissue types often determined from MAMMOGRAPHY; ULTRASONOGRAPHY; or MRI.'),
  'D000071061': NtO(MH='Traditional Pulse Diagnosis', MS='Technique used in AYURVEDIC MEDICINE, and EAST ASIAN TRADITIONAL MEDICINE, in which reading PULSE is used in the diagnosis of diseases.'),
  'D000071062': NtO(MH='Sutureless Surgical Procedures', MS='Surgical techniques in which SUTURES are not applied to surgical wounds.'),
  'D000071063': NtO(MH='Endoglin', MS='A membrane glycoprotein and ANGIOGENESIS FACTOR that is expressed by cells of the VASCULAR ENDOTHELIUM; VASCULAR SMOOTH MUSCLE; and MONOCYTES. It functions as a co-receptor for TRANSFORMING GROWTH FACTOR BETA and modulates CELL ADHESION. Mutations in the endoglin gene are associated with cases of HEREDITARY HEMORRHAGIC TELANGIECTASIA.'),
  'D000071064': NtO(MH='Alert Fatigue, Health Personnel', MS='Mental fatigue experienced by health care providers who encounter numerous alerts and reminders from the use of CLINICAL DECISION SUPPORT SYSTEMS. As the numbers of alerts and reminders designed to provide meaningful assistance to the patient care process increases, many health personnel may ignore them.'),
  'D000071065': NtO(MH='Standing Orders', MS='Medical procedures and prescriptions regarding treatment of a patient that can be exercised by other health care personnel when predetermined conditions have been met.'),
  'D000071066': NtO(MH='Patient Reported Outcome Measures', MS='Assessment of the quality and effectiveness of health care as measured and directly reported by the patient.'),
  'D000071067': NtO(MH='Pneumonia, Necrotizing', MS='Severe complication of pneumonia characterized by liquefaction of lung tissue.'),
  'D000071068': NtO(MH='Lipocalin-2', MS='A lipocalin of approximately 200 amino acids that functions as an iron transporter and is expressed by cells of BONE MARROW and many other cells with secretory functions. It is involved in APOPTOSIS and may function to limit pathogenic bacterial growth as part of the INNATE IMMUNE RESPONSE.'),
  'D000071069': NtO(MH='Multiple Chronic Conditions', MS='Two or more concurrent chronic physical, mental, or behavioral health problems in an individual.'),
  'D000071070': NtO(MH='Familial Multiple Lipomatosis', MS="A rare autosomal disorder characterized by numerous encapsulated lipomas on the trunk and extremities. The lipomas are usually not painful but can cause pain when growing. In rare cases, one lipoma can become painful and progress to multiple painful lipomas; it is then referred to as Dercum's Disease Type III"),
  'D000071071': NtO(MH='Microaneurysm', MS='Aneurysm of the MICROVASCULATURE. Charcot\xe2\x80\x93Bouchard aneurysms are aneurysms of the brain vasculature which is a common cause of CEREBRAL HEMORRHAGE. Retinal microaneurysm is an early diagnostic sign of DIABETIC RETINOPATHY.'),
  'D000071072': NtO(MH='Acute Febrile Encephalopathy', MS='Acute onset of fever accompanied by seizures, cerebral inflammation and a change in mental status (e.g., confusion, disorientation, and coma).'),
  'D000071074': NtO(MH='Neonatal Sepsis', MS='Blood infection that occurs in an infant younger than 90 days old. Early-onset sepsis is seen in the first week of life and most often appears within 24 hours of birth. Late-onset occurs after 1 week and before 3 months of age.'),
  'D000071075': NtO(MH='Small Fiber Neuropathy', MS='Disorder of the peripheral nerves that primarily impair small nerve fibers. The affected small nerve fibers include myelinated A-delta fibers (see A FIBERS) and unmyelinated C FIBERS. Because these small fibers innervate skin and help control autonomic function, their neuropathy presents with neuropathic pain, reduced thermal and pain sensitivity, and autonomic dysfunction (e.g. abnormal sweating or facial flushing). Small fiber neuropathy can be idiopathic or associated with underlying diseases (e.g., AMYLOIDOSIS; DIABETES MELLITUS; SARCOIDOSIS; or VASCULITIS).'),
  'D000071076': NtO(MH='Network Meta-Analysis', MS='Meta-analysis of randomized trials in which estimates of comparative treatment effects are visualized and interpreted from a network of interventions that may or may not have been evaluated directly against each other. Common considerations in network meta-analysis include conceptual and statistical heterogeneity and incoherence.'),
  'D000071077': NtO(MH='Surgical Oncology', MS='A surgical specialty concerned with management of cancer.'),
  'D000071078': NtO(MH='Stenosis, Pulmonary Vein', MS='Narrowing of the PULMONARY VEIN.'),
  'D000071079': NtO(MH='Stenosis, Pulmonary Artery', MS='Narrowing of the PULMONARY ARTERIES.'),
  'D000071080': NtO(MH='Cortical Excitability', MS='Measurable changes in activities in the CEREBRAL CORTEX upon a stimulation. A change in cortical excitability as measured by various techniques (e.g., TRANSCRANIAL MAGNETIC STIMULATION) is associated with brain disorders.'),
  'D000071081': NtO(MH='Minimal Clinically Important Difference', MS='A statistically significant minimum set of clinical outcomes that demonstrates a clinical benefit of an intervention or treatment.'),
  'D000071085': NtO(MH='Single-Balloon Enteroscopy', MS='A balloon-assisted enteroscopy utilizing a flexible endoscope with one overtube balloon control unit.'),
  'D000071087': NtO(MH='Balloon Enteroscopy', MS='An endoscopy of the small intestines accomplished while advancing the endoscope, which is assisted by one or two balloons'),
  'D000071096': NtO(MH='ADAMTS Proteins', MS='A subfamily of ADAM proteases that are distinguished by the presence of one or more THROMBOSPONDIN type-1 repeats (TSRs). These are three-strand motifs that contain characteristic TRYPTOPHAN, ARGININE, and CYSTEINE residues respectively. In contrast to ADAM proteins, which reside on CELL MEMBRANES, ADAMTS proteases are secreted and function in the EXTRACELLULAR MATRIX.'),
  'D000071097': NtO(MH='ADAMTS1 Protein', MS='An ADAMTS protease that contains two disintegrin loops and three C-terminal thrombospondin (TS) motifs. It functions as an ANGIOGENESIS INHIBITOR as well as in normal tissue growth and fertility.'),
  'D000071116': NtO(MH='ADAMTS5 Protein', MS='An ADAMTS protease that contains two C-terminal thrombospondin (TS) motifs. It functions primarily as an aggrecanase, cleaving AGGRECAN in CARTILAGE, and may be involved in the destruction of aggrecan in ARTHRITIS.'),
  'D000071117': NtO(MH='Laboratory Critical Values', MS='Test results which deviate substantially from normal ranges of REFERENCE VALUES or other qualitative results. They trigger CLINICAL LABORATORY SERVICES to place a special alert to ensure PATIENT SAFETY.'),
  'D000071118': NtO(MH='ADAMTS7 Protein', MS='An ADAMTS protease that contains four central and four C-terminal thrombospondin (TS) motifs. It binds to and degrades CARTILAGE OLIGOMERIC MATRIX PROTEIN.'),
  'D000071119': NtO(MH='ADAMTS9 Protein', MS='An ADAMTS protease that contains 15 THROMBOSPONDIN (TS) motifs. It cleaves the PROTEOGLYCANS AGGRECAN and VERSICAN and also functions in protein transport from the ENDOPLASMIC RETICULUM to the GOLGI APPARATUS.'),
  'D000071120': NtO(MH='ADAMTS13 Protein', MS='An ADAMTS protease that contains eight thrombospondin (TS) motifs. It cleaves VON WILLEBRAND FACTOR to control vWF-mediated THROMBOSIS. Mutations in the ADAMTS13 gene have been identified in familial cases of PURPURA, THROMBOTIC THROMBOCYTOPENIC and defects in ADAMTS13 activity are associated with MYOCARDIAL INFARCTION; BRAIN ISCHEMIA; PRE-ECLAMPSIA; and MALARIA.'),
  'D000071121': NtO(MH='ADAMTS4 Protein', MS='An ADAMTS protease similar to ADAMTS5 PROTEIN. It contains a single C-terminal thrombospondin (TS) motif and cleaves AGGRECAN in CARTILAGE. It may also be involved in the destruction of aggrecan in ARTHRITIS.'),
  'D000071137': NtO(MH='Poly (ADP-Ribose) Polymerase-1', MS='A poly(ADP-ribose) polymerase that contains two ZINC FINGERS in its N-terminal DNA-binding region. It modifies NUCLEAR PROTEINS involved in chromatin architecture and BASE EXCISION REPAIR with POLY ADENOSINE DIPHOSPHATE RIBOSE.'),
  'D000071156': NtO(MH='Promyelocytic Leukemia Protein', MS='A tripartite motif protein that contains three ZINC FINGERS, including a RING FINGER DOMAIN, at its N-terminal. Several nuclear and one cytoplasmic isoforms result from alternative splicing of the PML gene; most nuclear isoforms localize to subnuclear structures (PML nuclear bodies) that are disrupted in ACUTE PROMYELOCYTIC LEUKEMIA cells.'),
  'D000071158': NtO(MH='Leucine-Rich Repeat Serine-Threonine Protein Kinase-2', MS='A serine/threonine protein kinase with GTPase activity that contains 12 LEUCINE-rich repeats in its central region and 7 WD repeats C-terminal to its kinase and GTPase domains. It localizes to TRANSPORT VESICLES; the OUTER MITOCHONDRIAL MEMBRANE; and the GOLGI APPARATUS. It functions in PROTEIN TRANSPORT; regulates neuron morphology in the central nervous system, and also functions in the trafficking of SYNAPTIC VESICLES. Mutations in the LRRK2 gene have been identified in autosomal dominant cases of PARKINSON DISEASE (PARK8).'),
  'D000071161': NtO(MH='Forkhead Box Protein O1', MS='A forkhead box transcription factor that is a major target of INSULIN signaling and regulator of metabolic homeostasis in response to OXIDATIVE STRESS. It binds to the insulin RESPONSE ELEMENT (IRE) and the related Daf-16 family binding element (DBE). Its activity is suppressed by insulin and it also regulates OSTEOBLAST proliferation, controls bone mass, and skeletal regulation of GLUCOSE metabolism. It promotes GLUCONEOGENESIS in HEPATOCYTES and regulates gene expression in ADIPOSE TISSUE. It is also an important CELL DEATH regulator. Chromosomal aberrations involving the FOXO1 gene occur in RHABDOMYOSARCOMA.'),
  'D000071162': NtO(MH='Trefoil Factors', MS='A family of small peptides expressed primarily by MUCOUS MEMBRANE EPITHELIAL CELLS in vertebrates. Their structure is highly conserved and is defined by a three-leaved (trefoil) domain of 42 or 43 amino acids; it includes six cysteines that create the trefoil structure through disulfide bridges. Trefoil peptides may form dimers with each other, or associate with MUCINS and other factors. They are important for maintaining epithelial integrity and for protection from noxious agents.'),
  'D000071163': NtO(MH='Trefoil Factor-1', MS='A 60 amino acid (6.5 kDa) trefoil factor that contains a single trefoil domain. It is expressed primarily by surface EPITHELIAL CELLS of the GASTRIC MUCOSA, where it associates with MUCIN 5AC.'),
  'D000071164': NtO(MH='Trefoil Factor-2', MS='A 106 amino acid (12 kDa) trefoil factor that contains two trefoil domains and associates with MUCIN-6. It is expressed in the GASTROINTESTINAL TRACT by cells of BRUNNER GLANDS; PYLORIC GLANDS and mucous neck cells. It inhibits GASTROINTESTINAL MOTILITY and GASTRIC ACID secretion and may help maintain the structural integrity of gastric mucus.'),
  'D000071165': NtO(MH='Trefoil Factor-3', MS='A 59 amino acid (6.6 kDa) trefoil factor that contains a single trefoil domain. It is expressed in all tissues with MUCOUS MEMBRANES, including GOBLET CELLS of the SMALL INTESTINE and LARGE INTESTINE. It associates with MUCIN-2 in the small intestine and may regulate epithelial CELL MIGRATION and WOUND HEALING.'),
  'D000071176': NtO(MH='Signaling Lymphocytic Activation Molecule Family', MS='Type-I membrane glycoproteins that are expressed primarily on the surface of CD4 or CD8-positive T-CELLS; NATURAL KILLER CELLS; and some populations of B CELLS. They are characterized by an N-terminal, extracellular IMMUNOGLOBULIN-LIKE DOMAIN and a membrane-proximal IMMUNOGLOBULIN C2-SET DOMAIN. SLAMF receptors typically signal through homophilic interactions and are important for mediating the immune response and immune cell differentiation.'),
  'D000071177': NtO(MH='Signaling Lymphocytic Activation Molecule Family Member 1', MS='A 70-kDa SLAM family receptor that consists of an extracellular IMMUNOGLOBULIN-LIKE DOMAIN and an IMMUNOGLOBULIN C2-SET DOMAIN. It is expressed primarily by THYMOCYTES and activated T-LYMPHOCYTES, as well as by MACROPHAGES and DENDRITIC CELLS. It binds SLAMF1 receptors on other cells to mediate T-cell maturation and proliferation. It is also a receptor for the MEASLES VIRUS.'),
  'D000071178': NtO(MH='CD48 Antigen', MS='A 40-45 KDa GPI-linked protein in the SLAM family that consists of two IMMUNOGLOBULIN C2-SET DOMAINS. It is expressed on the surface of a variety of cells with immune functions, including THYMOCYTES; mature B-LYMPHOCYTES and T-LYMPHOCYTES; NATURAL KILLER CELLS; DENDRITIC CELLS; MAST CELLS; and EOSINOPHILS. It  binds to CD2 and 2B4 (CD244) receptors to activate and modulate the immunologic response.'),
  'D000071179': NtO(MH='Signaling Lymphocytic Activation Molecule Associated Protein', MS='A signal transducing adaptor protein that contains a single SH2 DOMAIN and is essential for the regulation of SLAM FAMILY RECEPTORS in the immune response. Mutations in the SLAM-associated protein gene have been identified in cases of X-LINKED LYMPHOPROLIFERATIVE DISEASE.'),
  'D000071181': NtO(MH='ATP Binding Cassette Transporter, Subfamily B, Member 2', MS='An ATP-binding cassette, sub-family B protein (ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B) that functions in the transport of ANTIGENS from the CYTOPLASM to the ENDOPLASMIC RETICULUM for association with HISTOCOMPATIBILITY ANTIGENS CLASS I peptides. It also acts as a molecular scaffold for the final stage of MHC class I PROTEIN FOLDING.'),
  'D000071182': NtO(MH='Autophagosomes', MS='Large spherical double-layered structures which function in AUTOPHAGY to engulf intracellular components such as ORGANELLES or pathogens. Their outer membrane then fuses with the LYSOSOME and the inner membrane and contents are digested by lysosomal HYDROLASES.'),
  'D000071183': NtO(MH='Autophagy-Related Proteins', MS='Proteins and enzymes that function, often as components of MULTIPROTEIN COMPLEXES, to assemble AUTOPHAGOSOMES and carry out AUTOPHAGY.'),
  'D000071184': NtO(MH='Pharmacogenomic Variants', MS='Naturally occurring genetic variations associated with drug response (e.g., dosage, extent and rate of metabolic processes). While these variants are not markers for GENETIC PREDISPOSITION TO DISEASE they influence PHARMACOKINETICS and pharmacodynamics and often occur on genes encoding drug metabolism enzymes and transporters (e.g., ANGIOTENSIN CONVERTING ENZYME; CYTOCHROME P-450 CYP2D6).'),
  'D000071185': NtO(MH='Pharmacogenomic Testing', MS='The detection of genetic variability (e.g., PHARMACOGENOMIC VARIANTS) relevant to PHARMACOGENETICS and PRECISION MEDICINE. The purpose of such genetic testing is to help determine the most effective treatment options and their optimum dosages with least potential risks for DRUG-RELATED SIDE EFFECTS AND ADVERSE REACTIONS.'),
  'D000071186': NtO(MH='Beclin-1', MS='An autophagy related protein which functions as a core subunit of PHOSPHATIDYLINOSITOL 3-KINASE MULTIPROTEIN COMPLEXES. It mediates the formation of  phosphatidylinositol 3-phosphate and functions in AUTOPHAGY, where it is required for maturation of the AUTOPHAGOSOME. It also functions in ENDOCYTOSIS and CYTOKINESIS as part of a separate complex. Beclin-1 associates with INTRACELLULAR MEMBRANES and interacts with the PROTO-ONCOGENE PROTEINS C-BCL-2 and BCL-X PROTEIN.'),
  'D000071187': NtO(MH='Autophagy-Related Protein 5', MS='An autophagy-related protein that functions in AUTOPHAGOSOME biogenesis. It is conjugated to the ATG12 PROTEIN via a process that is similar to UBIQUITINATION and involves the ATG7 PROTEIN and ATG10 enzyme. The ATG12-ATG5 conjugate acts as an E3 UBIQUITIN LIGASE-like enzyme and is required for the localization of ATG8 PROTEINS to AUTOPHAGOSOME vesicle membranes and modification of membrane lipids.'),
  'D000071189': NtO(MH='Autophagy-Related Protein-1 Homolog', MS='A serine/threonine-protein kinase that functions in AUTOPHAGY in response to starvation. It acts on the PHOSPHATIDYLINOSITOL 3-KINASE complex PIK3C3 to regulate AUTOPHAGOSOME formation. It also functions as both a downstream effector and negative regulator of mammalian target of rapamycin complex 1 (mTORC1) and is activated by AMPK, which it also negatively regulates.'),
  'D000071190': NtO(MH='Autophagy-Related Protein 8 Family', MS='UBIQUITIN-like modifier proteins that function in CYTOPLASM to VACUOLE transport (Cvt), vesicle transport within the GOLGI APPARATUS, and AUTOPHAGOSOME biogenesis. They are also required for selective autophagic degradation of the nucleus (nucleophagy) and MITOPHAGY.'),
  'D000071192': NtO(MH='Autophagy-Related Protein 12', MS='A UBIQUITIN-like modifier protein that functions in AUTOPHAGOSOME formation, CYTOPLASM to VACUOLE transport, MITOPHAGY, and nucleophagy. Conjugation with ATG5 PROTEIN or ATG10 is essential for its function. The ATG12-ATG5 conjugate acts as an E3 UBIQUITIN LIGASE-like enzyme for lipid modification of ATG8 FAMILY PROTEINS and their localization to vesicle membranes.'),
  'D000071193': NtO(MH='Autophagy-Related Protein 7', MS='An autophagy related protein  that is similar to UBIQUITIN-ACTIVATING ENZYME E1. It functions in CYTOPLASM to VACUOLE transport (Cvt) and AUTOPHAGY by activating ATG12 PROTEIN for its conjugation with ATG5 PROTEIN, as well as the conjugation of ATG8 FAMILY PROTEINS with phosphatidylethanolamine for ATG8 association to Cvt vesicles and AUTOPHAGOSOME membranes. It is also required for the nitrogen starvation response in yeast, MITOPHAGY; and autophagic cell death induced by CASPASE 8 inhibition.'),
  'D000071194': NtO(MH='Camelidae', MS='The only extant family of the suborder Tylopoda (order Artiodactyla). Currently living members include domesticated species, DROMEDARIES (with one-hump); BACTRIAN CAMELS (with two humps); LLAMAS; ALPACAS, and wild feral camels; VICUNAS; and GUANACOS. Although they get nutrients from plants by rumination, they evolved separately from the RUMINANTS which have four-chambered stomachs. Camelidae have three-chambered stomachs.'),
  'D000071195': NtO(MH='Educational Personnel', MS='Professionals responsible for teaching in an institution or place of learning such as a school, college, vocational institute, or university.'),
  'D000071196': NtO(MH='Pyrin Domain', MS='A protein interaction domain of the death domain superfamily. It is characterized by a 6-helix bundle similar to those that occur in DEATH DOMAIN RECEPTORS. Pyrin domains are present in a variety of proteins, especially those that function in INFLAMMASOMES; AUTOPHAGY; and APOPTOSIS. They form homodimers or heterodimers with death domain proteins to facilitate the assembly of signaling complexes.'),
  'D000071197': NtO(MH='B30.2-SPRY Domain', MS='A protein interaction domain that is characterized by a bent "beta-sandwich" consisting of two antiparallel beta-sheets. It occurs in eukaryotic proteins, including many TRIPARTITE MOTIF PROTEINS, which function in a variety of cellular processes.'),
  'D000071198': NtO(MH='Pyrin', MS='A tripartite motif protein that consists of an N-terminal pyrin domain, a central coiled-coil region and B-box type ZINC FINGER, and C-terminal regions that mediate homotrimerization and interactions with other proteins (the B30.2/SPRY DOMAIN). It is expressed primarily by mature GRANULOCYTES and associates with the cytoskeleton in the perinuclear area as well as AUTOPHAGOSOMES, where it co-ordinates the assembly of AUTOPHAGY-RELATED PROTEINS and degradation of INFLAMMASOME components. It functions in INNATE IMMUNITY and INFLAMMATION; mutations in the Pyrin protein (MEFV) gene are associated with FAMILIAL MEDITERRANEAN FEVER.'),
  'D000071199': NtO(MH='NLR Family, Pyrin Domain-Containing 3 Protein', MS='An NLR protein that contains an N-terminal PYRIN DOMAIN and ATP-binding site and 9 C-terminal LEUCINE-rich repeats; it is expressed primarily by MACROPHAGES. It is a core component of the INFLAMMASOME and directs its assembly in response to pathogen infection and damage-associated stimuli. Mutations in the NLRP3 gene are associated with FAMILIAL COLD AUTOINFLAMMATORY SYNDROME.'),
  'D000071216': NtO(MH='Faculty, Pharmacy', MS='Teaching and administrative staff having academic rank in a pharmacy school or department.'),
  'D000071217': NtO(MH='Infradian Rhythm', MS='A biological rhythm with a period longer than 24 hours.'),
  'D000071218': NtO(MH='Ultradian Rhythm', MS='A biological rhythm with a period shorter than 24 hours.'),
  'D000071219': NtO(MH='Global Burden of Disease', MS='Measure of the burden of disease using the disability-adjusted-life-year (DALY). This time-based measure combines years of life lost due to premature mortality and years of life lost due to time lived in states of less than full health. The metric was developed to assess the burden of disease consistently across diseases, risk factors and regions.'),
  'D000071220': NtO(MH='Pharmaceutical Research', MS='RESEARCH activity focused on pharmaceutical products and services.'),
  'D000071221': NtO(MH='Enhancer of Zeste Homolog 2 Protein', MS='A histone-lysine N-methyltransferase and catalytic subunit of Polycomb Repressive Complex 2.  It methylates LYSINE 9 (H3K9me) and LYSINE 27 (H3K27me) of HISTONE H3, leading to transcriptional repression of the affected target gene. EZH2 also methylates non-histone proteins such as GATA4 TRANSCRIPTION FACTOR and the nuclear receptor RORA. It regulates CIRCADIAN CLOCKS via histone methylation of circadian protein gene PROMOTER REGIONS, and its repressive activity is also important for the identity and differentiation of EMBRYONIC STEM CELLS.'),
  'D000071222': NtO(MH='Suppressor of Cytokine Signaling 1 Protein', MS='A suppressor of cytokine signaling protein that consists of an N-terminal kinase-inhibitory region, a central SH2 DOMAIN, a characteristic C-terminal SOCS box (a 40-amino acid motif, which functions to recruit E3 UBIQUITIN-PROTEIN LIGASE COMPLEXES). SOCS1 functions as a negative regulator of CYTOKINES that signal through the JANUS KINASES-STAT 3 TRANSCRIPTION FACTOR (JAK/STAT3) pathway by inhibiting the activity of JANUS KINASES.'),
  'D000071223': NtO(MH='Suppressor of Cytokine Signaling 3 Protein', MS='A suppressor of cytokine signaling protein that consists of an N-terminal kinase-inhibitory region, a central SH2 DOMAIN, a characteristic C-terminal SOCS box (a 40-amino acid motif, which functions to recruit E3 UBIQUITIN-PROTEIN LIGASE COMPLEXES). SOCS3 inhibits cytokine signaling by binding to RECEPTOR PROTEIN-TYROSINE KINASES as well as CYTOKINE RECEPTOR GP130; ERYTHROPOIETIN RECEPTORS; INSULIN RECEPTOR; and the LEPTIN RECEPTOR. Its functions include suppression of ERYTHROPOIESIS in the fetal liver.'),
  'D000071225': NtO(MH='Influenza A Virus, H5N8 Subtype', MS='A subtype of INFLUENZA A VIRUS that is highly virulent in poultry and wild birds, but shows varying degrees of pathogenicity in mice. The H5N8 virus subtype has a polybasic amino acid motif at the HA cleavage site which explains its pathogenicity in birds, and expresses surface proteins HEMAGGLUTININ 5 and NEURAMINIDASE 8 which are typical of Highly Pathogenic Avian Influenza viruses.'),
  'D000071228': NtO(MH='Polyelectrolytes', MS='Naturally-occurring or artificially made water-soluble POLYMERS whose repeating units are ionizable. Polyelectrolytes demonstrate attributes that are typical of salts, such as electrical conductivity, and typical of  polymers, such as viscosity.'),
  'D000071229': NtO(MH='Perilipin-4', MS='A perilipin protein characterized by an extensive 11-mer repeat region, which forms five adjacent alpha-helices. It is expressed primarily in WHITE ADIPOSE TISSUE and differentiating ADIPOCYTES, as well as skeletal muscle and heart.  It is soluble in the cytoplasm but re-localizes to the surface of LIPID DROPLETS under high lipid conditions.'),
  'D000071230': NtO(MH='Transcriptional Regulator ERG', MS="A trans-activator and member of the erythroblast transformation-specific family of transcriptions factors that contain a characteristic ETS MOTIF. It is required for PLATELET CELL ADHESION to the subendothelium and associates with CHIMERIC ONCOGENE PROTEINS in PROSTATE CANCER; EWING'S SARCOMA; and ACUTE MYELOID LEUKEMIA."),
  'D000071231': NtO(MH='Carbonic Anhydrase IX', MS='A carbonic anhydrase and transmembrane protein that consists of an N-terminal PROTEOGLYCAN-like domain, a catalytic region, a single-pass transmembrane domain, and a short intracellular tail. It functions as a dimer and is expressed primarily by cells of the GASTROINTESTINAL TRACT; BILE DUCT EPITHELIUM; and GALL BLADDER. It is expressed at high levels in many solid tumors, especially CLEAR CELL RENAL CARCINOMA, in response to CELL HYPOXIA.'),
  'D000071232': NtO(MH='Small Leucine-Rich Proteoglycans', MS='A proteoglycan family (SLRPs) that is defined by a central domain which consists of a variable number of repeats of the motif LXXLxLXXNxL, where L may be LEUCINE; ISOLEUCINE; VALINE; or other hydrophobic amino acids. The N-terminal contains four conserved CYSTEINE residues and may be modified depending on function. SLRPs provide structural support to the EXTRACELLULAR MATRIX and are critical for regulating its assembly and dynamics at CELL-MATRIX JUNCTIONS.'),
  'D000071234': NtO(MH='Lumican', MS='A small leucine-rich proteoglycan that contains 10 tandem leucine repeats and four N-linked sites within the leucine repeat region that may be substituted with KERATAN SULFATE. These properties and its horseshoe shape allow it to mediate interactions among COLLAGEN molecules within fibrils. It is expressed in most mesenchymal tissues as well as the CORNEA, where it functions to maintain transparency.'),
  'D000071235': NtO(MH='Fibromodulin', MS='A small leucine-rich proteoglycan that contains 4 KERATAN SULFATE chains within the leucine repeat region. It interacts with COLLAGEN TYPE I and COLLAGEN TYPE II fibrils and may function to control the rate of EXTRACELLULAR MATRIX assembly. It also sequesters TRANSFORMING GROWTH FACTOR BETA in the extracellular matrix.'),
  'D000071238': NtO(MH='Fibronectin Type III Domain', MS='A highly conserved immunoglobulin-like protein domain of about 100 amino acids that forms a beta sandwich structure.'),
  'D000071241': NtO(MH='Double-Stranded RNA Binding Motif', MS='An RNA-binding motif characterized by an alpha-beta-beta-beta-alpha fold that binds DOUBLE-STRANDED RNA. It occurs in many eukaryotic proteins as well as in bacterial and viral proteins.'),
  'D000071243': NtO(MH='Zika Virus Infection', MS='A viral disease transmitted by the bite of AEDES mosquitoes infected with ZIKA VIRUS. Its mild DENGUE-like symptoms include fever, rash, headaches and ARTHRALGIA. The viral infection during pregnancy, in rare cases, is associated with congenital brain and ocular abnormalities, called Congenital Zika Syndrome, including MICROCEPHALY and may also lead to GUILLAIN-BARRE SYNDROME.'),
  'D000071244': NtO(MH='Zika Virus', MS='An arbovirus in the FLAVIVIRUS genus of the family FLAVIVIRIDAE. Originally isolated in the Zika Forest of UGANDA it has been introduced to Asia and the Americas.'),
  'D000071245': NtO(MH='Mitochondrial Uncoupling Proteins', MS='Mitochondrial anion carrier proteins that function as dimers and form proton channels in the INNER MITOCHONDRIAL MEMBRANE which creates proton leaks and uncouples OXIDATIVE PHOSPHORYLATION from ATP synthesis, resulting in the generation of heat instead of ATP.'),
  'D000071246': NtO(MH='Uncoupling Protein 2', MS='A mitochondrial uncoupling protein that is expressed in many tissues and exhibits the greatest expression in SKELETAL MUSCLE. It regulates mitochondrial ATP production and the generation of REACTIVE OXYGEN SPECIES.'),
  'D000071247': NtO(MH='Uncoupling Protein 3', MS='A mitochondrial uncoupling protein that is expressed in heart and skeletal muscle tissues. It functions to regulate tissue respiration, energy balance, and THERMOGENESIS. Mutations in the UCP3 gene are associated with OBESITY.'),
  'D000071248': NtO(MH='Peroxisome Proliferator-Activated Receptor Gamma Coactivator 1-alpha', MS='A transcriptional co-activator for NUCLEAR RECEPTORS. It is characterized by an N-terminal LxxLL sequence, a region that interacts with PPAR GAMMA, and a C-terminal RNA RECOGNITION MOTIF. It increases expression of MITOCHONDRIAL UNCOUPLING PROTEIN to regulate genes involved in metabolic reprogramming in response to dietary restriction and the integration of CIRCADIAN RHYTHMS with ENERGY METABOLISM.'),
  'D000071250': NtO(MH='Snail Family Transcription Factors', MS='A transcription factor family characterized by the presence of several  C-terminal CYS2-HIS2 ZINC FINGERS. They function in many developmental processes including the induction of the EPITHELIAL-MESENCHYMAL TRANSITION; maintenance of embryonic MESODERM; growth arrest, CELL SURVIVAL; and CELL MIGRATION.'),
  'D000071251': NtO(MH='Extreme Environments', MS='An ENVIRONMENT which falls outside the range of tolerance for the survival of organisms that are not specifically adapted to survive or thrive in it. Examples of such environments are among the hottest and coldest places on earth.'),
  'D000071252': NtO(MH='Mentoring', MS='The art and practice of training, guiding, COUNSELING, and providing support to a less experienced, trained, or knowledgeable person.'),
  'D000071253': NtO(MH='Metadata', MS='Structured data elements used to describe other data.'),
  'D000071256': NtO(MH='Uncoupling Protein 1', MS='A mitochondrial uncoupling protein that is expressed in BROWN ADIPOSE TISSUE. It is critical for NONSHIVERING THERMOGENESIS to prevent heat loss in NEONATES.'),
  'D000071257': NtO(MH='Emergence Delirium', MS='A form of DELIRIUM which occurs after GENERAL ANESTHESIA.'),
  'D000071277': NtO(MH='Harassment, Non-Sexual', MS='The act of systematic and/or continuous unwanted and irritating actions of a non-sexual nature, by a party or group against another. This behavior may include threats, BULLYING, taunts, blackmail, and demands.'),
  'D000071278': NtO(MH='Domestication', MS='A systematic process of adapting wild species of animals and plants into new environments often in close association or proximity to humans.'),
  'D000071296': NtO(MH='Government Employees', MS='Persons employed by the GOVERNMENT.'),
  'D000071297': NtO(MH='Acute Retroviral Syndrome', MS='Early stage of HIV infection. Symptoms resemble INFLUENZA or INFECTIOUS MONONUCLEOSIS.'),
  'D000071298': NtO(MH='Body Remains', MS='The physical entity of a deceased human or animal.'),
  'D000071316': NtO(MH='Forkhead Box Protein O3', MS="A forkhead box transcription factor and transcriptional activator which triggers type 1 programmed cell death (APOPTOSIS) in the absence of APOPTOSIS INHIBITING PROTEINS, including neuronal cell death induced by OXIDATIVE STRESS. It recognizes and binds to the DNA sequence 5'-(AG)TAAA(TC)A-3' and also functions in post-transcriptional regulation of the c-MYC PROTO-ONCOGENE."),
  'D000071317': NtO(MH='Nanog Homeobox Protein', MS='A homeodomain protein and transcription regulator that functions in BLASTOCYST INNER CELL MASS and EMBRYONIC STEM CELL proliferation and CELL SELF RENEWAL. It confers pluripotency on embryonic stem cells and prevents their differentiation towards extraembryonic ENDODERM and trophectoderm (TROPHOBLAST) CELL LINEAGES.'),
  'D000071336': NtO(MH='Transcription Activator-Like Effectors', MS='Virulence factors of pathogenic plant bacteria, such as XANTHOMONAS species, that bind to specific sequences in the PROMOTER REGIONS of host DNA to induce transcription of specific host genes.  Variations in the DNA binding domain of TALE family members account for the variations in DNA specificity. The TALE DNA-binding domains are used in various GENETIC TECHNIQUES by fusing them with various DNA modifying enzymes to detect and manipulate sequence-specific targeted DNA sites.'),
  'D000071376': NtO(MH='RNA-Binding Motifs', MS='Amino acid motifs that bind RNA.'),
  'D000071377': NtO(MH='RNA Recognition Motif', MS='An approximately 80 amino acid RNA binding motif that consists of four anti-parallel surface beta sheets and two alpha helices arranged in a beta-alpha-beta-beta-alpha-beta configuration. One of the surface beta sheets interacts with two or three specific RNA bases. Interactions between additional sequences and the RNA, as well as within the RNA recognition motif increase the affinity and specificity of the protein-RNA interaction.'),
  'D000071378': NtO(MH='Bunion', MS='Abnormal swelling of the inner aspect of the first metatarsal head affecting the first METATARSOPHALANGEAL JOINT.'),
  'D000071380': NtO(MH='Fibromatosis, Plantar', MS='A fibromatosis of the plantar fascia characterized by thickening of the fibrous bands on the plantar aponeurosis in the sole of the foot and toes.'),
  'D000071396': NtO(MH='Aldehyde Dehydrogenase, Mitochondrial', MS='An aldehyde dehydrogenase expressed in the mitochondrial matrix that is essential for the metabolism of ETHANOL.'),
  'D000071416': NtO(MH='Twist Transcription Factors', MS='A  highly-conserved family of basic helix-loop-helix (bHLH) transcription factors.  They function as dimers with other bHLH proteins and bind E-BOX ELEMENTS to control gene expression during EMBRYOGENESIS and the EPITHELIAL-MESENCHYMAL TRANSITION.'),
  'D000071417': NtO(MH='Twist-Related Protein 2', MS='A twist family transcription factor that is expressed in MESODERM as well as the DERMIS during mammalian EMBRYOGENESIS. It is structurally and functionally similar to TWIST-RELATED PROTEIN 1; it also regulates OSTEOGENESIS and the expression of CYTOKINES in response to INFLAMMATION.'),
  'D000071419': NtO(MH='ETS Motif', MS='A helix-turn-helix motif characterized by three alpha-helices and four-stranded beta-sheets arranged in the order alpha1-beta1-beta2-alpha2-alpha3-beta3-beta4. The third alpha-helix contacts the major groove of DNA. The ETS motif and the flanking amino acid sequences of Ets proteins influence the binding affinity, and the alteration of a single amino acid in the Ets domain can change its DNA binding specificity.'),
  'D000071421': NtO(MH='Life History Traits', MS='Morphological or behavioral traits influenced by various living conditions that a population encounters especially as it pertains to REPRODUCTION and survival of the population (see POPULATION DYNAMICS) such as age at first reproductive event, number and size of offspring, and lifespan.'),
  'D000071422': NtO(MH='Inbreeding Depression', MS='A reduced GENETIC VARIATION and GENETIC FITNESS due to INBREEDING.'),
  'D000071425': NtO(MH='Src Homology 2 Domain-Containing, Transforming Protein 1', MS='An SHC-signaling adaptor protein that transduces PHOSPHOTYROSINE-dependent signals downstream of RECEPTOR PROTEIN-TYROSINE KINASES and non-receptor tyrosine kinases. It is required for TGF-BETA-induced CELL MIGRATION; NEOLPASM INVASION; and METASTASIS of BREAST NEOPLASMS; its SH2 DOMAIN is essential for tumor survival. It also functions in signaling downstream of  ANGIOPOIETIN RECEPTOR TIE-2, regulating the migration of ENDOTHELIAL CELLS; and PHYSIOLOGIC NEOVASCULARIZATION.'),
  'D000071426': NtO(MH='Src Homology 2 Domain-Containing, Transforming Protein 2', MS='An SHC-signaling adaptor protein that links GROWTH FACTOR RECEPTORS to SIGNAL TRANSDUCTION PATHWAYS in neurons.'),
  'D000071427': NtO(MH='Src Homology 2 Domain-Containing, Transforming Protein 3', MS='An SHC-signaling adaptor protein that links GROWTH FACTOR RECEPTORS to SIGNAL TRANSDUCTION PATHWAYS in neurons, including NEUROTROPHINS signaling in the CENTRAL NERVOUS SYSTEM.'),
  'D000071436': NtO(MH='Thermotolerance', MS='The ability of an organism to reduce susceptibility to heat shock, and adapt to HOT TEMPERATURE.'),
  'D000071437': NtO(MH='Axon Guidance', MS='The mechanism by which a neuronal process outgrows toward a target led by the GROWTH CONE. Local guidance cues are provided by cell surface proteins that act on the growing axon.'),
  'D000071438': NtO(MH='Fatty Acid-Binding Protein 7', MS='A fatty acid-binding protein expressed by ASTROCYTES during CENTRAL NERVOUS SYSTEM development, and by MALIGNANT GLIOMA cells. It is also expressed by ASTROCYTES in response to injury or ISCHEMIA, and may function in repair of the MYELIN SHEATH.'),
  'D000071439': NtO(MH='Pseudoautosomal Regions', MS="Homologous chromosomal regions at either end of the X CHROMOSOME or Y CHROMOSOME. These two regions pair regularly at male MEIOSIS and undergo RECOMBINATION. Pseudoautosomal region 1 (PAR1) is located  at the tip of the short 'p' arms (Xp22 and Yp11) and Pseudoautosomal region 2 (PAR2) is located at the tip of the long 'q' arms (Xq28 and Yq12)."),
  'D000071441': NtO(MH='Emotion-Focused Therapy', MS='Intervention that relies on free expression of emotions with coaching to enhance awareness of emotional experiences.'),
  'D000071442': NtO(MH='Taxis Response', MS='A behavior in free-moving organism that results in motion or orientation toward or away from an external stimulus.'),
  'D000071443': NtO(MH='12E7 Antigen', MS='A cell adhesion molecule that, in humans, is encoded by a gene on the PSEUDOAUTOSOMAL REGION of SEX CHROMOSOMES. It functions in CELL ADHESION of T-LYMPHOCYTES and ROSETTE FORMATION by ERYTHROCYTES. It is also involved in the migration of LEUKOCYTES through the ENDOTHELIUM BASEMENT MEMBRANE. The 12E7 antigen co-localizes with, but functions independently of PECAM-1.'),
  'D000071444': NtO(MH='Phototaxis', MS='A behavioral change that results in an organism moving or orienting toward or away from the light source.'),
  'D000071446': NtO(MH='Neuronal Outgrowth', MS='Formation of neuronal processes (AXONS; NEURITES) toward a target cell.'),
  'D000071447': NtO(MH='N-Myc Proto-Oncogene Protein', MS='A basic helix-loop-helix leucine zipper (bHLHZ) transcription factor and proto-oncogene protein that functions in cell growth and proliferation. In mammals, it is highly expressed in the brain during embryogenesis and is essential for brain development; it is not expressed in adult tissues. Amplification or overexpression of N-Myc occurs in at least 20% of tumors and is associated with a poor prognosis in cases of NEUROBLASTOMA; ALVEOLAR RHABDOMYOSARCOMA; SMALL CELL LUNG CARCINOMA; and neuroendocrine prostate cancer.'),
  'D000071448': NtO(MH='Axon Fasciculation', MS='Process of organizing neighboring AXONS into a bundle or a fascicle during neurite outgrowth mediated by CELL ADHESION MOLECULES.'),
  'D000071449': NtO(MH='Proprotein Convertase 9', MS='A proprotein convertase that is essential for CHOLESTEROL homeostasis. It binds to and is required for the lysosomal degradation of the LDL RECEPTOR (LDLR); the VLDL receptor, and the APOLIPOPROTEIN E RECEPTOR. It also regulates neuronal APOPTOSIS.'),
  'D000071450': NtO(MH='ATP Binding Cassette Transporter, Subfamily B, Member 3', MS='ATP-binding cassette, subfamily B, protein that functions in the transport of ANTIGENS from the CYTOPLASM to the ENDOPLASMIC RETICULUM for association with HISTOCOMPATIBILITY ANTIGENS CLASS I peptides. It functions as a heterodimer with ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B, MEMBER 2.'),
  'D000071451': NtO(MH='Chitinase-3-Like Protein 1', MS='A lectin that binds CHITIN, but lacks chitinase activity. It may be involved in tissue remodeling and cellular responses to the environment, including the response of type 2 HELPER T-CELLS to INFLAMMATION and sensitization to ALLERGENS. Mutations in the CHI3L1 gene are associated with ASTHMA.'),
  'D000071456': NtO(MH='Sequestosome-1 Protein', MS='A multidomain protein that is highly conserved among multicellular organisms. It contains a ZZ-type ZINC FINGER domain, C-terminal UBIQUITIN - associated (UBA) domain, and interacts with many other signaling proteins and enzymes including, atypical PROTEIN KINASE C; TNF RECEPTOR-ASSOCIATED FACTOR 6; subunits of the mTORC1 complex, and CASPASE-8. It functions in AUTOPHAGY as a receptor for the degradation of ubiquitinated substrates, and to co-ordinate signaling in response to OXIDATIVE STRESS.'),
  'D000071457': NtO(MH='DEAD Box Protein 58', MS="A DEAD-box RNA helicase that contains an N-terminal DEATH-LIKE DOMAIN, AAA+ ATPase domain, and C-terminal RNA HELICASE activity. It functions as an innate immune receptor through its recognition of viral nucleic acids. It also induces the expression of INTERFERON TYPE I and proinflammatory CYTOKINES. Its ligands include: 5'-triphosphorylated SINGLE-STRANDED RNA, DOUBLE-STRANDED RNA (dsRNA), and short dsRNA (less than 1 kb in length)."),
  'D000071458': NtO(MH='Death Domain Superfamily', MS='A family of structurally-related protein interaction domains characterized by 6-helical closed bundle fold, with Greek key topology and an internal pseudo two-fold symmetry. Death-like domains occur in many proteins that are essential for APOPTOSIS, including CASPASE RECRUITMENT DOMAIN CONTAINING PROTEINS and DEATH DOMAIN RECEPTORS.'),
  'D000071459': NtO(MH='Death Domain', MS='A conserved protein interaction domain of the death domain superfamily that is structurally similar to the DEATH EFFECTOR DOMAIN and CASPASE RECRUITMENT DOMAIN. Death domains bind each other to form oligomers and occur on DEATH DOMAIN RECEPTORS, where they are required for APOPTOSIS signaling and non-apoptotic functions.'),
  'D000071461': NtO(MH='Death Effector Domain', MS='A homotypic protein interaction module of the death domain superfamily that is composed of a bundle of six alpha-helices. The death effector domain shares sequence and structural similarities with the DEATH DOMAIN and CASPASE RECRUITMENT DOMAIN. It occurs in many proteins with essential functions in APOPTOSIS.'),
  'D000071476': NtO(MH='Caspase Activation and Recruitment Domain', MS='A homotypic protein interaction module of the death domain superfamily. It is composed of a bundle of six alpha-helices that is related in sequence and structure to the DEATH DOMAIN and DEATH EFFECTOR DOMAIN. The Caspase Activation and Recruitment Domain (CARD domain) typically associates with other CARD-containing proteins, forming either dimers or trimers. CARD domains may occur in isolation, or in combination with other domains in CARD signaling adaptor proteins and initiator CASPASES that function in APOPTOSIS.'),
  'D000071477': NtO(MH='Gubernaculum', MS='An embryonic structure that helps guide proper descent of gonads into their final positions. It attaches the caudal end of the fetal GONADS to the developing SCROTUM in male and the labium majorum in female. It gives rise to the caudal ligaments of the gonad: the scrotal ligament in male and the uterine round and proper ovarian ligaments in female. It includes morphofunctional equivalent structures in non-mammals.'),
  'D000071478': NtO(MH='APOBEC Deaminases', MS='A family of conserved cytidine deaminases that catalyze the DEAMINATION of CYTIDINE to URIDINE. They are characterized by N and C-terminal zinc-dependent catalytic domains and have important functions in RNA EDITING; EPIGENTIC PROCESSES; and the INNATE IMMUNE RESPONSE against viruses.'),
  'D000071479': NtO(MH='APOBEC-1 Deaminase', MS='An APOBEC deaminase catalytic subunit of the apolipoprotein B (APOB) MESSENGER RNA (mRNA) editing enzyme complex that is involved in post-transcriptional editing of a CAA codon for GLYCINE to a UAA STOP CODON in the ApoB mRNA. It also functions in CGA (ARGININE) to UGA STOP CODON editing of NEUROFIBROMIN 1 mRNA and EPIGENETIC PROCESSES.'),
  'D000071480': NtO(MH='APOBEC-3G Deaminase', MS='An APOBEC deaminase that functions as an inhibitor of RETROVIRIDAE replication and inhibits the mobility of RETROTRANSPOSONS via deaminase-dependent and independent mechanisms. It is selective for SINGLE-STRANDED DNA and does not deaminate double-stranded DNA or single or DOUBLE-STRANDED RNA. It exhibits potent antiviral activity against VIF PROTEIN deficient HIV-1 through the creation of hypermutations in the VIRAL DNA. It also has anti-viral activity against SIMIAN IMMUNODEFICIENCY VIRUSES and HEPATITIS B VIRUS.'),
  'D000071496': NtO(MH='AlkB Enzymes', MS='A family of alpha- KETOGLUTARIC ACID and Fe(II)-dependent dioxygenases that are homologous to AlkB, an enzyme that repairs alkylated nucleic acids in E. coli. The mammalian homologs have diverse substrate specificities and functions that include DNA REPAIR, generating unique wobble modifications in URIDINE tRNA, demethylation of nucleotides in DNA and RNA, and demethylation of LYSINE residues on certain proteins, including ACTIN and histones (HISTONE CODE).'),
  'D000071497': NtO(MH='Immunogenicity, Vaccine', MS='The capacity of VACCINES to stimulate the ADAPTIVE IMMUNE RESPONSE to produce antibodies and antigen-specific T-CELL responses.'),
  'D000071498': NtO(MH='AlkB Homolog 1, Histone H2a Dioxygenase', MS='A dioxygenase and alkylation repair homolog that functions by oxidative demethylation in RNA and DNA REPAIR of substrates which contain 3-methylcytosine. It also functions as one of the HISTONE DEMETHYLASES involved in embryonic development and maintaining the pluripotency of EMBRYONIC STEM CELLS.'),
  'D000071499': NtO(MH='AlkB Homolog 2, Alpha-Ketoglutarate-Dependent Dioxygenase', MS='A dioxygenase and alkylation repair homolog that repairs alkylated DNA and RNA containing 1-methyladenine and 3-methylcytosine by oxidative demethylation. It can also repair alkylated DNA containing 1-ethenoadenine in vitro. It has highest affinity for double-stranded DNA.'),
  'D000071500': NtO(MH='AlkB Homolog 3, Alpha-Ketoglutarate-Dependent Dioxygenase', MS='A dioxygenase and alkylation repair homolog that repairs alkylated DNA containing 1-methyladenine (1meA) and 3-methylcytosine (3meC) by oxidative demethylation. It has a strong preference for SINGLE-STRANDED DNA and may also act on RNA.'),
  'D000071501': NtO(MH='AlkB Homolog 4, Lysine Demethylase', MS="A dioxygenase and alkylation repair homolog that mediates demethylation of ACTIN monomethylated at 'Lys-84' (K84me1).  Demethylation of actin LYSINE 84 (K84me1) is required for maintaining actin and MYOSIN dynamics to support normal cleavage furrow ingression during CYTOKINESIS and CELL MIGRATION. It may also be involved in transcription regulation."),
  'D000071502': NtO(MH='AlkB Homolog 5, RNA Demethylase', MS='A dioxygenase and alkylation repair homolog  that demethylates RNA by oxidative demethylation. It specifically demethylates N(6)-methyladenosine (m6A) RNA, the most common internal modification of MESSENGER RNA in higher eukaryotes.  It can also demethylate N(6)-methyladenosine in SINGLE-STRANDED DNA.'),
  'D000071503': NtO(MH='AlkB Homolog 8, tRNA Methyltransferase', MS='A dioxygenase and alkylation repair homolog that catalyzes the methylation of 5-carboxymethyl URIDINE to 5-methylcarboxymethyl uridine at the wobble position of the ANTICODON loop in TRANSFER RNA (tRNA) via its methyltransferase domain. It has a preference for tRNA (ARGININE) and tRNA (GLUTAMATE), and does not bind tRNA (LYSINE).'),
  'D000071516': NtO(MH='Alpha-Ketoglutarate-Dependent Dioxygenase FTO', MS='A dioxygenase that repairs alkylated DNA and RNA by oxidative demethylation. It has highest activity towards single-stranded RNA containing 3-methyluracil, followed by SINGLE-STRANDED DNA containing 3-methylthymine and specifically demethylates N(6)-methyladenosine (m6A) RNA, the most common internal modification of MESSENGER RNA (mRNA) in higher EUKARYOTES. It contributes to the regulation of the global metabolic rate, energy HOMEOSTASIS, as well as body fat accumulation by regulating the differentiation of ADIPOCYTES into BROWN FAT CELLS or WHITE FAT CELLS.'),
  'D000071518': NtO(MH='Liver X Receptors', MS='Nuclear receptors that bind OXYSTEROLS and function as heterodimers with RETINOID X RECEPTORS. They have important functions in regulating cholesterol homeostasis, ENERGY METABOLISM; INFLAMMATION; and the immune response.'),
  'D000071538': NtO(MH='Cortical Bone', MS='A type of osseous tissue which makes up the outer layer of bone. It is dense, rigid, strong, and forms concentric lamellar OSTEONS.'),
  'D000071556': NtO(MH='Cancellous Bone', MS='A type of osseous tissue which makes up the inner part of bone.  It has a spongy, honeycomb-like structure with struts or trabecula and contains the BONE MARROW. It has higher rate of BONE REMODELING turnover than CORTICAL BONE.'),
  'D000071557': NtO(MH='beta-Arrestins', MS='Non-visual system arrestins that negatively regulate G-PROTEIN-COUPLED RECEPTORS (GPCRs) and may also function independently of GPCR signaling. They bind and recruit many different signaling factors, including MITOGEN-ACTIVATED PROTEIN KINASES; SRC-FAMILY-KINASES; and FILAMIN to GPCRs and may recognize different phosphorylation states of the receptors to determine the specificity of the cellular response to signaling.'),
  'D000071559': NtO(MH='beta-Arrestin 1', MS='A beta-arrestin that functions in the down-regulation of signaling by G-PROTEIN-COUPLED RECEPTORS. It is also a major regulator of INSULIN signaling via the ERK 1-2 PATHWAY, and many other signaling processes, especially in NEURONS and LEUKOCYTES.'),
  'D000071560': NtO(MH='beta-Arrestin 2', MS='A beta-arrestin that functions similarly to BETA-ARRESTIN 1 in regulating signaling by G-PROTEIN-COUPLED RECEPTORS. It is expressed at high levels in the central nervous system where it may regulate signaling by SYNAPTIC RECEPTORS.'),
  'D000071562': NtO(MH='Fractures, Avulsion', MS='Tearing away of the CORTICAL BONE fragment at the location of a strong ligament or tendon attachment. The bone fragment detachment site often occurs near a soft site (e.g., GROWTH PLATE) at the base where LIGAMENTS; TENDONS; or JOINT CAPSULES attach. In younger patients it is most often caused by a sudden forceful pull on a tendon in the opposite direction of the bone movement. In the elderly it is associated with osteoporotic INSUFFICIENCY FRACTURES.'),
  'D000071576': NtO(MH='Crush Injuries', MS='Excessive compression of parts of the body that causes muscle swelling, fracture, and/or neurological disturbances in the affected areas. Crush injury with systemic manifestations is referred to as CRUSH SYNDROME.'),
  'D000071596': NtO(MH='Abdominal Oblique Muscles', MS='Muscles of the anterolateral abdominal wall consisting of the external oblique and the internal oblique muscles. The external abdominal oblique muscle fibers extend from lower thoracic ribs to the linea alba and the iliac crest. The internal abdominal oblique extend superomedially beneath the external oblique muscles.'),
  'D000071616': NtO(MH='CDX2 Transcription Factor', MS='An antennapedia-like homeodomain transcription factor that regulates the expression of multiple genes in the INTESTINAL MUCOSA. It plays a critical role in many processes from early differentiation to maintenance of the intestinal epithelial lining of both the small and large intestine.'),
  'D000071617': NtO(MH='Protein Deglycase DJ-1', MS='A protein deglycase that repairs methylglyoxal- and glyoxal-glycated amino acids and proteins, releasing repaired proteins and lactate or glycolate. It deglycates CYSTEINE, ARGININE and LYSINE residues to reactivate proteins by reversing glycation and prevent the formation of ADVANCED GLYCATION END PRODUCTS. It protects cells against OXIDATIVE STRESS and CELL DEATH by functioning as an oxidative stress sensor and redox-sensitive MOLECULAR CHAPERONE and PROTEASE. Mutations in the PARK7 gene are associated with autosomal-recessive, early-onset PARKINSON DISEASE.'),
  'D000071636': NtO(MH='Protein Phosphatase 2C', MS='One of four major classes of mammalian serine/threonine specific protein phosphatases. Protein phosphatase 2C is a monomeric enzyme about 42 kDa in size. It shows broad substrate specificity dependent on divalent cations (mainly manganese and magnesium). Three isozymes are known in mammals: PP2C -alpha, -beta and -gamma. In yeast, there are four PP2C homologues: phosphatase PTC1 that have weak tyrosine phosphatase activity, phosphatase PTC2, phosphatase PTC3, and PTC4. Isozymes of PP2C also occur in Arabidopsis thaliana where the kinase-associated protein phosphatase (KAPP) containing a C-terminal PP2C domain, dephosphorylates Ser/Thr receptor-like kinase RLK5.'),
  'D000071656': NtO(MH='Receptor, Notch3', MS='A notch receptor characterized by a large extracellular domain containing 34 EPIDERMAL GROWTH FACTOR-like repeats. It functions to regulate CELL DIFFERENTIATION; APOPTOSIS; and CELL PROLIFERATION. Mutations in the EGF repeats of Notch-3 are associated with CADASIL.'),
  'D000071657': NtO(MH='Werner Syndrome Helicase', MS="A DNA-dependent helicase and 3'-5' exonuclease. It has  3'->5' exonuclease activity towards double-stranded DNA with a 5'-overhang and binds preferentially to DNA substrates containing alternate secondary structures, such as replication forks and HOLLIDAY JUNCTIONS. Mutations in the WRN gene are associated with WERNER SYNDROME."),
  'D000071676': NtO(MH='Zinc Finger Protein GLI1', MS='A transcriptional activator and oncogene protein that contains two CYS2-HIS2 ZINC FINGERS. Two isoforms are expressed; both regulate the expression of specific genes during development of craniofacial features, digits, the CENTRAL NERVOUS SYSTEM; and the GASTROINTESTINAL TRACT. They also regulate SONIC HEDGEHOG PROTEIN signaling and cell proliferation.'),
  'D000071677': NtO(MH='Histidine Kinase', MS='A member of the transferase superfamily of proteins. In the activated state, protein-histidine kinase autophosphorylates at a histidine residue, subsequently transferring high-energy phosphoryl groups to an aspartate residue of the response-regulator domain, which results in a conformational shift in the effector domain. Histidine kinases mediate signal transduction in a wide range of processes involving cellular adaptation to environmental stress.'),
  'D000071678': NtO(MH='WD40 Repeats', MS='Protein interaction motifs of approximately 40 amino acids that usually terminate in TRYPTOPHAN and ASPARTIC ACID. They form characteristic beta-propeller structures and occur in many eukaryotic proteins that function in a variety of cellular processes. Proteins that contain WD40 repeats often function as assembly platforms for MULTIPROTEIN COMPLEXES.'),
  'D000071679': NtO(MH='Glycogen Synthase Kinase 3 beta', MS='A glycogen synthase kinase-3 type enzyme that functions in ENERGY METABOLISM; EMBRYONIC DEVELOPMENT; and NEUROGENESIS. It is also involved in PROTEIN BIOSYNTHESIS and regulates cell growth and proliferation as a component of the WNT SIGNALING PATHWAY and other signaling pathways. Certain polymorphisms in the GSK3B gene have been associated with PARKINSON DISEASE; ALZHEIMER DISEASE; and BIPOLAR DISORDER.'),
  'D000071681': NtO(MH='Tartrate-Resistant Acid Phosphatase', MS='One of several acid phosphatases in humans, other mammals, plants, and a few prokaryotes. The protein fold of tartrate-resistant acid phosphatase (TRAP) resembles that of the catalytic domain of plant purple acid phosphatase and other serine/threonine-protein phosphatases that also contain a metallophosphoesterase domain. One gene produces the various forms which include purple acid phosphatases from spleen and other tissues. Tartrate-resistant acid phosphatase is a biomarker for pathological states in which it is over-expressed. Such conditions include GAUCHER DISEASE; HODGKIN DISEASE; BONE RESORPTION; and NEOPLASM METASTASIS.'),
  'D000071682': NtO(MH='Sterile Alpha Motif', MS='A putative protein interaction module, approximately 70 amino acids long, that forms a small five-helix bundle with two large interfaces which may homo- and hetero-oligomerize, or bind non-sterile-alpha motif targets. The sterile alpha motif is present in a wide variety of eukaryotic proteins that function in diverse biological processes.'),
  'D000071696': NtO(MH='Proteogenomics', MS='The systematic study of annotated genomic information to global protein expression in order to determine the relationship between genomic sequences and both expressed proteins and predicted protein sequences.'),
  'D000071697': NtO(MH='Cellulite', MS='Skin lumpiness or skin surface dimpling often seen on the thighs, buttocks and abdomen. It is due to protrusion of SUBCUTANEOUS FAT into the DERMIS layer of skin.'),
  'D000071698': NtO(MH='Latent Autoimmune Diabetes in Adults', MS='Autoimmune diabetes in adults with slowly progressive PANCREATIC BETA CELL failure and the presence of circulating autoantibodies to PANCREATIC ISLETS cell antigens.'),
  'D000071699': NtO(MH='Bilateral Vestibulopathy', MS='Impairment of the vestibular function of both inner ears which can cause difficulties with balance, gait, VERTIGO, and visual blurring.'),
  'D000071700': NtO(MH='Cone-Rod Dystrophies', MS='Genetically heterogeneous and sometimes syndromic (e.g., BARDET BIEDL SYNDROME; and SPINOCEREBELLAR ATAXIA TYPE 7) retinopathies with initial RETINAL CONE involvement. They are characterized by decreased VISUAL ACUITY; COLOR VISION DEFECTS; progressive loss of peripheral vision and night blindness.'),
  'D000071701': NtO(MH='Regulatory Factor X Transcription Factors', MS='A family of eukaryotic transcription factors that recognize and bind to a highly-conserved cis-regulatory sequence (X-box) within the promoter region of MHC CLASS II GENES. They contain a conserved winged-helix DNA binding domain and function as homo or heterodimers.'),
  'D000071716': NtO(MH='Regulatory Factor X1', MS='A regulatory factor X transcription factor that is required for the expression of MHC CLASS II GENES. It also binds to inverted repeats of HEPATITIS B VIRUS DNA and is required for viral gene expression. RFX1 functions as a monomer or heterodimer with other RFX transcription factors.'),
  'D000071717': NtO(MH='X-Box Binding Protein 1', MS='A basic leucine zipper transcription factor that regulates expression of MHC CLASS II GENES through its binding to a conserved X-box DNA sequence in the promoter region. It regulates the expression of genes involved in the UNFOLDED PROTEIN RESPONSE.'),
  'D000071736': NtO(MH='Stromal Interaction Molecules', MS='Membrane-associated calcium-binding proteins that are characterized by N-terminal EF HAND MOTIFS and a STERILE ALPHA MOTIF. They function as sensors of ENDOPLASMIC RETICULUM calcium levels and translocate from the ER to the plasma membrane to activate CALCIUM CHANNELS in response to intracellular calcium depletion.'),
  'D000071737': NtO(MH='Stromal Interaction Molecule 1', MS='A stromal interaction molecule that functions in the regulation of calcium influx following depletion of intracellular calcium in the ENDOPLASMIC RETICULUM. It translocates to the plasma membrane upon calcium depletion where it activates the CALCIUM RELEASE ACTIVATED CALCIUM CHANNEL ORAI1.'),
  'D000071738': NtO(MH='Stromal Interaction Molecule 2', MS='A stromal interaction molecule that functions as a highly sensitive calcium sensor in the ENDOPLAMSIC RETICULUM and CYTOSOL. It can regulate CALCIUM RELEASE ACTIVATED CALCIUM CHANNELS in response to minor variations in intracellular calcium.'),
  'D000071739': NtO(MH='Calcium Release Activated Calcium Channels', MS='Specialized calcium channels that localize to the ENDOPLAMSIC RETICULUM and PLASMA MEMBRANE. They contain the pore subunit ORAI1 PROTEIN which is activated by STROMAL INTERACTION MOLECULES upon intracellular calcium depletion.'),
  'D000071740': NtO(MH='ORAI1 Protein', MS='The pore-forming subunit of calcium release activated calcium channels. It is activated by STROMAL INTERACTION MOLECULE 1 upon intracellular calcium depletion.'),
  'D000071741': NtO(MH='ORAI2 Protein', MS='A calcium release-activated calcium-like (CRAC-like) channel subunit which functions with STROMAL INTERACTION MOLECULE 1 to regulate cell calcium influx and increase (Ca2+)-selective current.'),
  'D000071756': NtO(MH='Soluble Guanylyl Cyclase', MS='A mammalian enzyme composed of a heterodimer of alpha and beta subunits. Each subunit consists of four domains; N-terminal HNOX domain, PAS-like domain, a coiled-coil domain, and a C-terminal catalytic domain. All four domains are homologous proteins with a similar conformation of functional domains. Soluble guanylate cyclase catalyzes the formation of cyclic GMP from GTP, and is a key enzyme of the nitric oxide signaling pathway involved in the regulation of a variety of biological and physiological processes in mammals.'),
  'D000071757': NtO(MH='BTB-POZ Domain', MS='A homodimerization protein interaction domain occurring at the N terminus of proteins that contain multiple copies of either CYS2-HIS2 ZINC FINGERS or KELCH REPEATS. It is characterized by a tightly intertwined dimer with an extensive hydrophobic interface. A surface-exposed groove lined with conserved amino acids is formed at the dimer interface, suggesting a peptide-binding site. Many BTB proteins are transcriptional regulators that are thought to regulate CHROMATIN structure.'),
  'D000071776': NtO(MH='Kelch Repeat', MS='An amino acid sequence of about 50 residues long that is composed of X n-long repeats which form a four-stranded anti-parallel BETA-SHEET as a structural component, or blade, of a beta-propeller.'),
  'D000071796': NtO(MH='SMARCB1 Protein', MS='A component of the SWI-SNF CHROMATIN REMODELING complex that functions  as a PROTEIN PHOSPHATASE 1 regulator and to stabilize CHROMATIN at PROMOTER REGIONS. It is important for regulating CELL PROLIFERATION and CELL DIFFERENTIATION. Mutations in the SMARCB1 gene are associated with malignant RHABDOID TUMORS.'),
  'D000071797': NtO(MH='Butyrophilins', MS='A family of mammalian membrane glycoproteins characterized by extracellular IMMUNOGLOBULIN DOMAINS. Some members also have an intracellular B30.2-SPRY DOMAIN or SPRY DOMAIN. The butyrophilin protein (BTN) is expressed by MAMMARY GLAND EPITHELIUM during LACTATION; whereas other members are widely expressed in other tissues, including skeletal muscle, intestine, ERYTHROID CELLS (ERMAP protein); and nerve tissue (MYELIN-OLIGODENDROCYTE GLYCOPROTEIN).'),
  'D000071799': NtO(MH='Zinc Finger E-box-Binding Homeobox 1', MS='A transcription factor characterized by N-terminal and C-terminal CYS2-HIS2 ZINC FINGERS separated by a homeobox. It represses the expression of E-CADHERIN to induce the EPITHELIAL-MESENCHYMAL TRANSITION. It also represses PROTO-ONCOGENE PROTEINS C-BCL-6; regulates the cell type-specific expression of SODIUM-POTASSIUM-EXCHANGING ATPASE; and promotes neuronal differentiation.'),
  'D000071816': NtO(MH='Glycodelin', MS='A lipocalin that is the most abundant protein synthesized and secreted by the ENDOMETRIUM from mid-LUTEAL PHASE of the MENSTRUAL CYCLE through early PREGNANCY.'),
  'D000071818': NtO(MH='Wnt-5a Protein', MS='A Wnt protein and ligand for FRIZZLED RECEPTORS that may function as an inhibitor or activator of the WNT SIGNALING PATHWAY. For example, it activates signaling in the presence of Frizzled-4 but is inhibitory when coupled with ROR2 TYROSINE KINASE. It is required for axis formation during EMBRYOGENESIS and inhibits the proliferation, migration, and invasiveness of cancer cells.'),
  'D000071819': NtO(MH='Virophages', MS='Circular, double-stranded DNA viruses that invade giant DNA viruses (GIANT VIRUSES) which infect eukaryotes.'),
  'D000071820': NtO(MH='Emergency Medical Dispatch', MS='The mobilization of EMERGENCY CARE to the locations and people that need them.'),
  'D000071821': NtO(MH='Dystonin', MS='A plakin characterized by repeat sequences homologous to SPECTRIN and PLECTIN and C-terminal EF HAND MOTIFS. It functions as an integrator of INTERMEDIATE FILAMENTS, ACTIN and MICROTUBULES in cytoskeleton networks. It is required for anchoring intermediate filaments to the actin cytoskeleton in neural and muscle cells as well as anchoring KERATIN-containing intermediate filaments to HEMIDESMOSOMES in EPITHELIAL CELLS.'),
  'D000071822': NtO(MH='Emergency Medical Dispatcher', MS='Professionals responsible for relaying calls for the deployment of EMERGENCY MEDICAL SERVICES.'),
  'D000071825': NtO(MH='Extremophiles', MS='Organisms specifically adapted to live in EXTREME ENVIRONMENTS.'),
  'D000071826': NtO(MH='Distracted Driving', MS='Behavior which detracts from the focus necessary to operate an automobile safely.'),
  'D000071836': NtO(MH='Professional Practice Gaps', MS='Difference between observed and ideal DELIVERY OF HEALTH CARE and healthcare outcomes that reflect the current state of knowledge.'),
  'D000071837': NtO(MH='Fibrillins', MS='A family of extracellular matrix glycoproteins that is structurally similar to  LATENT TGF-BETA BINDING PROTEINS, but contain additional TGF-beta binding domains, in addition to unique domains at their N and C-terminals. Fibrillins assemble into 10-12 nm MICROFIBRILS that function in a variety of cell interactions with the EXTRACELLULAR MATRIX and developmental processes such as ELASTIC TISSUE maintenance and assembly, and the targeting of growth factors to the extracellular matrix.'),
  'D000071838': NtO(MH='Fibrillin-1', MS='A fibrillin (FBN1) that functions as a structural support protein for MICROFIBRILS. It also regulates the maturation of OSTEOBLASTS by controlling the availability and concentration of TGF-BETA and BONE MORPHOGENETIC PROTEINS. Mutations in the FBN1 gene are associated with MARFAN SYNDROME.'),
  'D000071840': NtO(MH='Fibrillin-2', MS='A fibrillin (FBN2) that functions to regulate the assembly of ELASTIC FIBERS. Mutations in the FBN2 gene are associated with congenital contractural ARACHNODACTYLY.'),
  'D000071841': NtO(MH='PAX6 Transcription Factor', MS='A paired box transcription factor that has important functions in the development of the eye, nose, central nervous system and pancreas. Mutations in the PAX6 gene are associated with ocular disorders such as ANIRIDIA.'),
  'D000071856': NtO(MH='Tudor Domain', MS='A conserved, approximately 50 amino acid, protein sequence that adopts a characteristic strongly bent, five stranded anti-parallel beta-sheet barrel-like fold. The Tudor domain recognizes dimethylated ARGININE through its negatively-charged surface and occurs in many proteins that co-localize with RNA; RIBONUCLEOPROTEINS or SINGLE-STRANDED DNA complexes in the CELL NUCLEUS; MITOCHONDRIA; or KINETOCHORES.'),
  'D000071857': NtO(MH='Tumor Suppressor p53-Binding Protein 1', MS='A nuclear and cytoplasmic protein that associates with KINETOCHORES and contains a C-terminal TUDOR DOMAIN. It plays a critical role in the cellular response to DNA DAMAGE and localizes to DOUBLE-STRAND DNA BREAKS. It may also function in M PHASE CELL CYCLE CHECKPOINTS and as an enhancer of TUMOR SUPPRESSOR PROTEIN P53-mediated transcriptional activation.'),
  'D000071858': NtO(MH='Epithelial Cell Adhesion Molecule', MS='A cell adhesion molecule that is expressed on the membranes of nearly all EPITHELIAL CELLS, especially at the junctions between intestinal epithelial cells and intraepithelial LYMPHOCYTES. It also is expressed on the surface of ADENOCARCINOMA and epithelial tumor cells.  It may function in the MUCOSA through homophilic interactions to provide a barrier against infection. It also regulates the proliferation and differentiation of EMBRYONIC STEM CELLS.'),
  'D000071876': NtO(MH='Prostaglandin-E Synthases', MS='Oxidoreductases that catalyze the GLUTATHIONE-dependent oxidoreduction of PROSTAGLANDIN H2 to PROSTAGLANDIN E2.'),
  'D000071877': NtO(MH='Checkpoint Kinase 1', MS='A serine/threonine-specific protein kinase which is encoded by the CHEK1 gene in humans. Checkpoint kinase 1 (also known as Chk1) coordinates DNA damage response and cell cycle checkpoint response. Under these conditions, activation of Chk1 results in the initiation of cell cycle checkpoints, cell cycle arrest, DNA repair and cell death, to prevent damaged cells from progressing through the cell cycle.'),
  'D000071896': NtO(MH='Medically Unexplained Symptoms', MS='Persistent health symptoms which remain unexplained after a complete medical evaluation. A cluster of symptoms that consistently appear together but without a known cause are referred to as a MEDICALLY UNEXPLAINED SYNDROME (MUS).'),
  'D000071916': NtO(MH='AC133 Antigen', MS='A member of the prominin family, AC133 Antigen is a 5-transmembrane antigen occurring as several isoforms produced by alternative splicing which are processed into mature forms. In humans, it is expressed as a subset of CD34 (bright) human hematopoietic stem cells and CD34 positive leukemias. Functionally, it is associated with roles in cell differentiation, proliferation, and apoptosis. Specifically, it regulates the organization of apical plasma membrane in epithelial cells, disk morphogenesis during early retinal development, MAPK and Akt signaling pathways, and in cholesterol metabolism.'),
  'D000071936': NtO(MH='Nipple Discharge', MS='Fluid that seeps out of one or both nipples of the breast.'),
  'D000071937': NtO(MH='Cerebral Blood Volume', MS='Volume of circulating blood in a region of the brain. It is a functional measure of the brain perfusion status which relates changes in this to changes in CEREBROVASULAR CIRCULATION that are often seen in brain diseases.'),
  'D000071938': NtO(MH='Fasciotomy', MS="Surgical incision on the FASCIA. It is used to decompress compartment pressure (e.g. in COMPARTMENT SYNDROMES; circumferential burns and extremity injuries) or to release contractures (e.g. in DUPUYTREN'S CONTRACTURE)."),
  'D000071939': NtO(MH='Stroke Rehabilitation', MS='Restoration of functions to the maximum degree possible in a person or persons suffering from a stroke.'),
  'D000071940': NtO(MH='Nucleons', MS='Subatomic particles that are either protons or neutrons.'),
  'D000071941': NtO(MH='Barium Enema', MS='X-RAY examination of LOWER GASTROINTESTINAL TRACT. Imaging is done while a BARIUM COMPOUND (e.g., BARIUM SULFATE) fills the large intestine via the rectum as a CONTRAST MATERIAL.'),
  'D000071956': NtO(MH='Myocardial Contusions', MS='Bruise to the heart muscle due to blunt thoracic trauma.'),
  'D000071959': NtO(MH='Case Managers', MS='Professionals who are in charge of assessment, planning, facilitation, care coordination, evaluation, and advocacy for options and services to meet a patient\xe2\x80\x99s and family\xe2\x80\x99s comprehensive health needs.'),
  'D000071960': NtO(MH='Breast Carcinoma In Situ', MS='A condition in which abnormal cells have not spread outside the duct, lobule, or nipple to other tissues of the breast. There are 3 types of breast carcinoma in situ: DUCTAL CARCINOMA IN SITU; LOBULAR CARCINOMA IN SITU; and PAGET DISEASE OF THE NIPPLE.'),
  'D000071976': NtO(MH='Gracilis Muscle', MS='A slender and most superficial muscle in the inner THIGH which originates at the ramus of the PUBIC BONE near the PUBIC SYMPHYSIS and inserts at the proximal medial TIBIA. Its function is to abduct thigh, flex knee, and rotate leg medially. It is a common  source of a MYOCUTANEOUS FLAP.'),
  'D000071997': NtO(MH='Blood Culture', MS='Test to determine the presence of blood infection (e.g. SEPSIS; BACTEREMIA).'),
  'D000071999': NtO(MH='S100 Calcium-Binding Protein A4', MS='An S100 protein characterized by four helix bundles that form N- and C-terminal EF HAND MOTIFS. It functions as a homodimer and interacts with both intracellular and extracellular signaling proteins. Aberrant S100A4 activity is associated with NEOPLASM METASTASIS; FIBROSIS; and RHEUMATOID ARTHRITIS.'),
  'D000072': NtO(MH='Acedapsone', MS='Acetylated sulfone that is slowly metabolized to give long-term, low blood levels of DAPSONE. It has antimicrobial and antimalarial action, but is mainly used as a depot leprostatic agent.'),
  'D000072000': NtO(MH='NF-KappaB Inhibitor alpha', MS='An I-kappa B protein that inhibits the activity of dimeric NF-KAPPA B P50-REL complexes, sequesters transcription factor NF-kappaB as an inactive complex in the cytoplasm; and prevents NF-kappaB nuclear translocation and DNA binding.'),
  'D000072001': NtO(MH='Diet, Healthy', MS='Dietary patterns which have been found to be important in reducing disease risk.'),
  'D000072002': NtO(MH='Prion Proteins', MS='Membrane glycosylphosphatidylinositol-anchored glycoproteins that may aggregate into rod-like structures. The prion protein (PRNP) gene is characterized by five TANDEM REPEAT SEQUENCES that encode a highly unstable protein region of five octapeptide repeats. Mutations in the repeat region and elsewhere in this gene are associated with CREUTZFELDT-JAKOB DISEASE; FATAL FAMILIAL INSOMNIA; GERSTMANN-STRAUSSLER DISEASE; Huntington disease-like 1, and KURU.'),
  'D000072017': NtO(MH='Deubiquitinating Enzymes', MS='Enzymes that remove UBIQUITIN from a protein substrate, including POLYUBIQUITIN, or from other molecules.'),
  'D000072018': NtO(MH='Audiologists', MS='Professionals skilled at diagnostic testing of hearing, HEARING IMPAIRMENT, and CORRECTION OF HEARING IMPAIRMENT by non-medical or non-surgical means.'),
  'D000072019': NtO(MH='Kelch-Like ECH-Associated Protein 1', MS='An adaptor protein characterized by an N-terminal BTB-POZ DOMAIN and six KELCH REPEATS that functions as a substrate for the E3 UBIQUITIN LIGASE complex. It negatively-regulates NF-E2-RELATED FACTOR 2 by targeting it for ubiquitination and degradation by the PROTEASOME. It also represses genes regulated by ANTIOXIDANT RESPONSE ELEMENTS.'),
  'D000072020': NtO(MH='Synthetic Lethal Mutations', MS='Mutations in genes which lead to cell or organism death when occurring in combination with mutations in one or more other genes.'),
  'D000072036': NtO(MH='Prophylactic Mastectomy', MS='Surgical removal of one or both breasts to prevent or reduce the risk of developing breast cancer in persons who may be predisposed.'),
  'D000072038': NtO(MH='Cardiac Rehabilitation', MS='Restoration of functions to the maximum degree possible in a person or persons suffering from a CARDIOVASCULAR DISEASE. It also includes cardiac conditioning and SECONDARY PREVENTION in patients with elevated cardiovascular risk profile.'),
  'D000072039': NtO(MH='Fracture Dislocation', MS='Fracture of a bone near an articulation with concomitant dislocation of that joint.'),
  'D000072040': NtO(MH='Apolipoprotein A-V', MS='A minor apolipoprotein that associates with HIGH-DENSITY LIPOPROTEINS (HDL), VERY-LOW-DENSITY LIPOPROTEINS (VLDL), and CHYLOMICRONS. It regulates levels of plasma TRIGLYCERIDES by activating APOLIPOPROTEIN C-II LIPOPROTEIN LIPASE and inhibiting hepatic VLDL triglyceride hydrolysis.'),
  'D000072041': NtO(MH='PAX3 Transcription Factor', MS='A paired box transcription factor that functions in CELL PROLIFERATION; CELL MIGRATION; and APOPTOSIS. It also is involved in MYOGENESIS and neural development. Mutations in the PAX3 gene are associated with WAARDENBURG SYNDROME TYPE 1.'),
  'D000072042': NtO(MH='Salter-Harris Fractures', MS='Fractures involving a GROWTH PLATE.'),
  'D000072056': NtO(MH='Transcription Factor HES-1', MS='A basic-helix-loop-helix transcription factor that functions as a transcriptional repressor for genes transcribed by bHLH proteins. For example, it may negatively regulate MYOGENESIS by inhibiting MyoD1 and ASH1 proteins. It is also required for the stability of FANCONI ANEMIA COMPLEMENTATION GROUP PROTEINS and their localization to the cell nucleus in response to DNA DAMAGE.'),
  'D000072077': NtO(MH='Anesthetists', MS='Persons trained and certified to administer ANESTHETICS.'),
  'D000072078': NtO(MH='Positron Emission Tomography Computed Tomography', MS='An imaging technique that combines a POSITRON-EMISSION TOMOGRAPHY (PET) scanner and a CT X RAY scanner. This establishes a precise anatomic localization in the same session.'),
  'D000072079': NtO(MH='Patched Receptors', MS='A family of 12-pass transmembrane proteins originally identified in Drosophila that are receptors for HEDGEHOG PROTEINS. They have important roles in regulating CELL PROLIFERATION; CELL DIFFERENTIAION; and BODY PATTERNING and may also function as TUMOR SUPPRESSOR PROTEINS.'),
  'D000072080': NtO(MH='Anesthesiologists', MS='Physicians specializing in ANESTHESIOLOGY.'),
  'D000072081': NtO(MH='Patched-1 Receptor', MS='A patched receptor for several HEDGEHOG PROTEINS that associates with the SMOOTHENED RECEPTOR to modulate hedgehog signaling. It is also a TUMOR SUPPRESSOR PROTEIN; mutations in the patched-1 gene are associated with BASAL CELL NEVUS SYNDROME; SQUAMOUS CELL CARCNIOMA of the ESOPHAGUS; trichoepitheliomas, and CARCINOMA, TRANSITIONAL CELL of the URINARY BLADDER.'),
  'D000072082': NtO(MH='Patched-2 Receptor', MS='A patched receptor that may function redundantly with the PATCHED-1 RECEPTOR to modulate hedgehog signaling. It may also play a role in epidermal development and as a TUMOR SUPPRESSOR PROTEIN. Mutations in the patched-2 gene are associated with BASAL CELL NEVUS SYNDROME; CARCINOMA, BASAL CELL; and MEDULLOBLASTOMA.'),
  'D000072085': NtO(MH='Dermatologists', MS='Physicians who specialize in treating disorders of the skin.'),
  'D000072086': NtO(MH='Smoothened Receptor', MS='A frizzled-like, G-protein-coupled receptor that associates with PATCHED RECEPTORS to transduce signals from HEDGEHOG PROTEINS and initiate hedgehog signaling to ZINC FINGER PROTEIN GLI1. It may normally inhibit signaling in the absence of SONIC HEDGEHOG PROTEIN binding to PATCHED RECEPTOR-1.'),
  'D000072087': NtO(MH='Occupational Therapists', MS='Professionals trained to help individuals develop or regain skills needed to achieve independence in their lives.'),
  'D000072096': NtO(MH='Cardiologists', MS='Qualified medical professionals who specialize in preventing and treating disorders of the heart and CARDIOVASCULAR SYSTEM.'),
  'D000072097': NtO(MH='Endocrinologists', MS='Physicians who specialize in treating ENDOCRINE SYSTEM DISEASES.'),
  'D000072098': NtO(MH='Single Photon Emission Computed Tomography Computed Tomography', MS='An imaging technique using a device which combines TOMOGRAPHY, EMISSION-COMPUTED, SINGLE-PHOTON and TOMOGRAPHY, X-RAY COMPUTED in the same session.'),
  'D000072099': NtO(MH='Serrate-Jagged Proteins', MS='Cell surface proteins that consist of multiple extracellular EPIDERMAL GROWTH FACTOR - like repeat sequences (EGF repeats), including calcium-binding EGF repeats. They function as transmembrane ligands for NOTCH RECEPTORS to control CELL DIFFERENTIATION during development.'),
  'D000072100': NtO(MH='Jagged-1 Protein', MS='A serrate-jagged protein that functions as a ligand for NOTCH RECEPTORS. It may regulate CELL DIFFERENTIATION in HEMATOPOIESIS and PHYSIOLOGIC ANGIOGENESIS. Mutations in the Jagged-1 gene are associated with ALAGILLE SYNDROME 1.'),
  'D000072101': NtO(MH='Jagged-2 Protein', MS='A serrate-jagged protein that functions to modulate signal transduction from NOTCH RECEPTORS in CELL DIFFERENTIATION. It may also regulate the expression of CYTOKINES.'),
  'D000072102': NtO(MH='Gastroenterologists', MS='Physicians who specialize in treating diseases of the DIGESTIVE SYSTEM.'),
  'D000072103': NtO(MH='Oncologists', MS='Physicians specializing in MEDICAL ONCOLOGY or its sub-specialties of RADIATION ONCOLOGY or SURGICAL ONCOLOGY.'),
  'D000072104': NtO(MH='Nephrologists', MS='Qualified medical professionals who specialize in treating KIDNEY DISEASES.'),
  'D000072105': NtO(MH='Superoxide Dismutase-1', MS='A superoxide dismutase (SOD1) that requires copper and zinc ions for its activity to destroy SUPEROXIDE FREE RADICALS within the CYTOPLASM. Mutations in the SOD1 gene are associated with AMYOTROPHIC LATERAL SCLEROSIS-1.'),
  'D000072106': NtO(MH='Pulmonologists', MS='Physicians who specialize in treating disorders of the RESPIRATORY SYSTEM, especially diagnosing and treating diseases and defects of the lungs and bronchial tree.'),
  'D000072117': NtO(MH='Counselors', MS='Professionals trained in providing information, guidance, and recommendations to individual clients or families to support their decision making and development of coping skills.'),
  'D000072137': NtO(MH='Vaping', MS='SMOKING from ELECTRONIC NICOTINE DELIVERY SYSTEMS such as E-CIGARETTES.'),
  'D000072138': NtO(MH='Mosquito Vectors', MS='Mosquitoes (members of the family CULICIDAE) that transmit pathogens or their intermediate forms from one host to another.'),
  'D000072139': NtO(MH='Giant Viruses', MS='DNA viruses that have genomes up to several megabases in length and infect primarily single-celled EUKARYOTES.'),
  'D000072140': NtO(MH='Rheumatologists', MS='Physicians who specialize in treating RHEUMATIC DISEASES.'),
  'D000072141': NtO(MH='Neurologists', MS='Qualified medical professionals who specialize in treating NERVOUS SYSTEM DISEASES.'),
  'D000072142': NtO(MH='Pathologists', MS='Physicians who assess changes in cellular or tissue structure and function to diagnose disease processes.'),
  'D000072143': NtO(MH='Pediatricians', MS='Qualified medical professionals specializing in maintaining health and providing medical care to children from birth through adolescence.'),
  'D000072156': NtO(MH='Neonatologists', MS='Physicians who specialize in providing medical care to newborn infants.'),
  'D000072157': NtO(MH='Physiatrists', MS='Physicians specializing in treating physical disorders and injuries affecting the brain, spinal cord, nerves, bones, joints, ligaments, muscles, and tendons.'),
  'D000072158': NtO(MH='Epidemiologists', MS='Qualified medical professionals who study the causes, incidence, and characteristic behavior of disease outbreaks or other health-related conditions and determine the interrelationships of hosts, agents, and environment related to the distribution and control of diseases.'),
  'D000072159': NtO(MH='Dynactin Complex', MS='A multi-subunit protein of EUKARYOTIC CELLS. It functions in bidirectional intracellular transport of ORGANELLES and CYTOPLASMIC VESICLES by linking them to DYNEIN and KINESIN.'),
  'D000072160': NtO(MH='Tumor Protein p73', MS='A homolog of p53 TUMOR SUPPRESSOR PROTEIN that encodes full-length trans-activating and N-terminally-truncated (DeltaN) isoforms. Detection of splice variants and isoforms in the nervous system (human TELENCEPHALON, CHOROID PLEXUS; CEREBROSPINAL FLUID), embryonic tissue, human BREAST CANCER; OVARIAN CANCER, suggest roles in cellular differentiation.'),
  'D000072161': NtO(MH='Orthopedic Surgeons', MS='Physicians specializing in medical, surgical, and physical methods used to treat and correct deformities, diseases, and injuries to the skeletal system, its articulations, and associated structures.'),
  'D000072162': NtO(MH='Optometrists', MS='Qualified professionals trained in primary eye and vision care, including measurement of visual abilities, diagnosing disorders of the visual system and provision of treatment such as prescriptions for correction of visual defects with lenses or glasses and vision therapy.'),
  'D000072163': NtO(MH='Ophthalmologists', MS='Qualified medical professionals specializing in medical and surgical treatment of defects and diseases of the eye.'),
  'D000072176': NtO(MH='Superconductivity', MS='The ability of an element or metallic alloy to allow the passage of ELECTRONS without ELECTRICAL RESISTANCE and expel MAGNETIC FIELDS below a certain threshold temperature called superconducting transition temperature.'),
  'D000072177': NtO(MH='Radiologists', MS='Physicians specializing in the use of x-ray and other forms of radiant energy to diagnosis and treatment of disease.'),
  'D000072178': NtO(MH='Urologists', MS='Physicians specializing in the diagnosis, and treatment of diseases of the urinary tract in both sexes, and the genital tract in the male.'),
  'D000072179': NtO(MH='Interleukin-1 Receptor-Like 1 Protein', MS='A receptor for INTERLEUKIN-33 that is related structurally to the interleukin-1 receptor. It  contains three extracellular IMMUNOGLOBULIN-LIKE DOMAIN regions and associates with INTERLEUKIN-1 RECEPTOR ACCESSORY PROTEIN upon binding IL-33 to initiate signaling. It may function in the response of HELPER T CELLS to INFLAMMATION.'),
  'D000072180': NtO(MH='Phosphoinositide Phosphatases', MS='Phosphatases that catalyze the hydrolysis of phosphate groups on PHOSPHATIDYLINOSITOLS.'),
  'D000072181': NtO(MH='Inositol Polyphosphate 5-Phosphatases', MS="Phosphoinositide phosphatases that catalyze the removal of the 5' phosphate from INOSITOL 1,4,5-TRISPHOSPHATE  or myo-inositol 1,3,4,5-tetrakisphosphate, resulting in inositol 1,4-bisphosphate and phosphate. They have important functions in the metabolism of INOSITOL PHOSPHATES and inositol 1,4,5-trisphosphate signaling pathways such as CALCIUM SIGNALING."),
  'D000072182': NtO(MH='Non-Medical Public and Private Facilities', MS='Spaces and needed equipment provided for a specific, non-medical  function or service used by the public or specific group of persons.'),
  'D000072183': NtO(MH='Phosphatidylinositol-3,4,5-Trisphosphate 5-Phosphatases', MS='Phosphoinositide phosphatases that catalyze the dephosphorylation (hydrolysis) of  phosphatidylinositol-3,4,5-trisphosphate (PtdIns(3,4,5)P(3)) to produce PtdIns(3,4)P(2), which negatively regulates the PI3K ( 3-PHOSPHOINOSITIDE-DEPENDENT PROTEIN KINASES) pathways. They contain an SH2 DOMAIN and STERILE ALPHA MOTIF and have important functions in regulating the immune response and other cellular processes in vertebrates.'),
  'D000072184': NtO(MH='Nurse Specialists', MS='Nursing professionals whose practice is limited to A particular area or discipline of medicine.'),
  'D000072196': NtO(MH='Fertilins', MS='Sperm surface proteins involved in sperm-egg fusion. They consist of two subunits, fertilin alpha (ADAM1a) and beta (ADAM2), both of which belong to the metalloprotease-disintegrin protein family. The beta subunit does not have catalytic activity.'),
  'D000072197': NtO(MH='ADAM10 Protein', MS='A disintegrin and metalloproteinase domain-containing protein. It cleaves the membrane-bound precursor of TUMOR NECROSIS FACTOR-ALPHA between ALANINE 76 and VALINE 77 to its functional form, as well as several other CELL SURFACE PROTEINS to their soluble forms, including AMYLOID BETA-PROTEIN PRECURSOR and PRION PROTEIN.'),
  'D000072198': NtO(MH='ADAM17 Protein', MS='A disintegrin and metalloproteinase domain-containing protein that cleaves the membrane-bound precursor of TUMOR NECROSIS FACTOR-ALPHA to its mature form. It cleaves several other CELL SURFACE PROTEINS, including INTERLEUKIN-1 RECEPTOR TYPE II; TRANSFORMING GROWTH FACTOR ALPHA; L-SELECTIN; MUCIN-1; and AMYLOID BETA-PROTEIN PRECURSOR. It can also function as an activator of the Notch signaling pathway by mediating the cleavage of NOTCH RECEPTORS.'),
  'D000072199': NtO(MH='ADAM12 Protein', MS='A disintegrin and metalloproteinase domain-containing protein that is expressed as two alternatively-spliced forms: a long transmembrane form (ADAM12-L) and a short soluble form  (ADAM12-S). It modulates the cleavage of INSULIN-LIKE GROWTH FACTOR BINDING PROTEINS and may also regulate CELL FUSION during MYOGENESIS.'),
  'D000072200': NtO(MH='Ku Autoantigen', MS='An ATP-dependent DNA HELICASE that preferentially binds SINGLE-STRANDED DNA. It is a heterodimer consisting of an 80 kDa subunit (XRCC5) and 70 kDa subunit (XRCC6) that functions with DNA LIGASE IV in the repair of DOUBLE-STRANDED DNA BREAKS and V(D)J RECOMBINATION.'),
  'D000072216': NtO(MH='Zona Pellucida Glycoproteins', MS='Membrane glycoproteins that constitute the ZONA PELLUCIDA in mammals and function in the recognition and binding of SPERMATOZOA.'),
  'D000072218': NtO(MH='Endodontists', MS='Dentists specializing in the treatment of injuries and diseases of the DENTAL PULP.'),
  'D000072219': NtO(MH='Geriatricians', MS='Physicians specializing in physiological and pathological aspects of aging and the aged, including the clinical problems of senescence and senility.'),
  'D000072220': NtO(MH='Otolaryngologists', MS='Physicians specializing in the treatment of disorders of the ear, nose, and throat.'),
  'D000072221': NtO(MH='Nurses, Neonatal', MS='Qualified nursing professionals specializing in care of newborn infants.'),
  'D000072222': NtO(MH='Nurses, Pediatric', MS='Qualified nursing professionals specializing in the care of children from birth to adolescence.'),
  'D000072223': NtO(MH='Allergists', MS='Physicians specializing in treating individuals with hypersensitivity to ALLERGENS.'),
  'D000072224': NtO(MH='Bcl-2-Like Protein 11', MS='A BCL-2-like protein that has a C-terminal BCL-2 homology (BH3) domain and forms heterodimers with other BCL-2 FAMILY PROTEINS. It is a strong inducer of APOPTOSIS and ANOIKIS; several isoforms are expressed (BimEL, Bim L, Bim-alpha, Bim-s; and Bim-gamma) that have different potencies for inducing apoptosis.'),
  'D000072226': NtO(MH='Computed Tomography Angiography', MS='A non-invasive method that uses a CT scanner for capturing images of blood vessels and tissues. A CONTRAST MATERIAL is injected, which helps produce detailed images that aid in diagnosing VASCULAR DISEASES.'),
  'D000072227': NtO(MH='Shoulder Prosthesis', MS='Replacement for a SHOULDER JOINT.'),
  'D000072228': NtO(MH='Arthroplasty, Replacement, Shoulder', MS='Replacement of the SHOULDER JOINT.'),
  'D000072229': NtO(MH='Posterior Cruciate Ligament Reconstruction', MS='Rebuilding of the POSTERIOR CRUCIATE LIGAMENT to restore functional stability of the knee.'),
  'D000072230': NtO(MH='Sustained Virologic Response', MS='The continuous, long-term suppression of VIRAL LOAD, generally to undetectable levels, as the result of treatment with ANTIVIRAL AGENTS.'),
  'D000072231': NtO(MH='In Vivo Dosimetry', MS='A procedure to measure the amount of radiation absorbed by the body during RADIOTHERAPY by DOSIMETERS in the patient\xe2\x80\x99s skin or natural cavities.'),
  'D000072232': NtO(MH='Radiation Dosimeters', MS='Devices that measure exposure to IONIZING RADIATION.'),
  'D000072233': NtO(MH='Homer Scaffolding Proteins', MS='Homer proteins belong to a family of adaptor and scaffold proteins which include Homer1, Homer2 and Homer3. Homer1 and Homer2 play a role in the regulation of calcium homeostasis, whereas Homer3 functions in stimulating changes in actin dynamics in neurons and T-cells. Homer proteins are best known as scaffold proteins at the post-synaptic density where they facilitate synaptic signaling. They function as a molecular switch in metabotropic glutamate receptor (MGluR) signaling, and are associated with human Fragile X syndrome.'),
  'D000072234': NtO(MH='Donor Conception', MS='Assisted reproductive techniques where eggs, sperm, both eggs and sperm (double donation), or embryos from donors are used.'),
  'D000072235': NtO(MH='Mating Factor', MS='A protein also known as pheromone mating factor that occurs on the surfaces of organisms such as yeast and fungi.'),
  'D000072236': NtO(MH='Methyl-Accepting Chemotaxis Proteins', MS='Transmembrane sensor receptor proteins that are central components of the chemotactic systems of a number of motile bacterial species which include ESCHERICHIA COLI and SALMONELLA TYPHIMURIUM. Methyl-accepting chemotaxis proteins derive their name from a sensory adaptation process which involves methylation at several glutamyl residues in their cytoplasmic domain. Methyl-accepting chemotaxis proteins trigger chemotactic responses across spatial chemical gradients, causing organisms to move either toward favorable stimuli or away from toxic ones.'),
  'D000072237': NtO(MH='ERG1 Potassium Channel', MS='One of three members of the ether-a-go-go (EAG) POTASSIUM CHANNELS gene family comprising ether-a-go-go (eag), eag-like (elk) and eag-related (erg) subfamilies. Ether-a-go-go-related gene 1 (ERG1) also known as KCNH2, encodes the pore-forming subunit of a rapidly activating-delayed rectifier potassium channel that plays an essential role in the final repolarization of ventricular action potential. Loss-of-function mutations in human hERG1 is associated with life-threatening ARRHYTHMIA.'),
  'D000072257': NtO(MH='Oxygen Radical Absorbance Capacity', MS='A measure of the ability of a substance, such as a food, to quench oxygen free radicals in vitro.'),
  'D000072258': NtO(MH='Tumor Hypoxia', MS='Hypoxic conditions in tumor cells due to the tumor outgrowing its blood supply. It is associated with increased METASTASIS and resistance to RADIOTHERAPY and DRUG THERAPY.'),
  'D000072259': NtO(MH='Connexin 26', MS='A gap junction protein encoded by the Gap Junction Beta 2 or GJB2 gene. In the cochlea and epidermis, its hexamers form channels between cells that open to allow cell-to-cell diffusion of small molecules as well as recycling of potassium. Mutations in Connexin 26 are associated with congenital SENSORINEURAL HEARING LOSS.'),
  'D000072260': NtO(MH='RNA Splicing Factors', MS='RNA-binding proteins that facilitate or inhibit RNA SPLICING.'),
  'D000072261': NtO(MH='Dishevelled Proteins', MS='A family of proteins that are key components of the WNT SIGNALING PATHWAY, where they function downstream of FRIZZLED RECEPTORS. They contain an N-terminal dishevelled-AXIN PROTEIN (DIX) domain, which mediates oligomerization; a central PDZ DOMAIN which binds to the frizzled receptor; and a C-terminal DEP domain which facilitates binding to the CELL MEMBRANE. Dishevelled proteins have important functions in CELL DIFFERENTIATION and establishing CELL POLARITY.'),
  'D000072263': NtO(MH='Private Facilities', MS='Places designed, built, or installed to provide for a specific, non-medical function or service used by a specific, restricted group of persons.'),
  'D000072264': NtO(MH='Sports and Recreational Facilities', MS='Places designed, built, or installed for LEISURE ACTIVITIES outside of the home.'),
  'D000072277': NtO(MH='Maximal Respiratory Pressures', MS="A respiratory function test that includes the maximal inspiratory pressure and maximal expiratory pressure. It is determined by SPIROMETRY that measures the patient's RESIDUAL VOLUME and TOTAL LUNG CAPACITY and assesses the strength of RESPIRATORY MUSCLES."),
  'D000072278': NtO(MH='Forkhead Box Protein M1', MS='A forkhead box transcription factor that is expressed primarily in adult organs which contain proliferating cells such as the thymus, testis, ovary, and small intestine. It controls the expression of CELL CYCLE genes essential for DNA REPLICATION and MITOSIS, and also functions in DNA REPAIR.'),
  'D000072280': NtO(MH='Bacteria, Thermoduric', MS='Bacteria resistant to high temperatures, as in PASTEURIZATION.'),
  'D000072281': NtO(MH='Lymphadenopathy', MS='Disease of LYMPH NODES which are abnormal in size, number or consistency.'),
  'D000072282': NtO(MH='Extraintestinal Pathogenic Escherichia coli', MS='Strains of Escherichia coli that possess virulence traits which allow them to invade, colonize, and induce disease in tissues outside of the GASTROINTESTINAL TRACT. They are a cause of URINARY TRACT INFECTIONS (UROPATHOGENIC ESCHERICHIA COLI); neonatal MENINGITIS; SEPSIS; PNEUMONIA; and SURGICAL WOUND INFECTION.'),
  'D000072283': NtO(MH='A549 Cells', MS='An immortalized cell line derived from human ADENOCARCINOMA, ALVEOLAR basal epithelial cells isolated from the lungs of a male patient in 1972. The cell line is positive for KERATIN, can synthesize LECITHIN, and contains high levels of POLYUNSATURATED FATTY ACIDS in its PLASMA MEMBRANE. It is used as a model for PULMONARY ALVEOLI function and virus infections, as a TRANSFECTION host, and for PRECLINICAL DRUG EVALUATION.'),
  'D000072296': NtO(MH='Bathroom Equipment', MS='Receptacles used for collection of human excrement.'),
  'D000072316': NtO(MH='Dioxins and Dioxin-like Compounds', MS='A group of toxic chemical compounds that share chemical structures and biological characteristics that are related to the potent carcinogen 2,3,7,8-tetrachlorodibenzo-p-dioxin.'),
  'D000072317': NtO(MH='Polychlorinated Dibenzodioxins', MS='Dibenzodioxin derivatives that contain multiple chloride atoms bound to the benzene ring structures.'),
  'D000072318': NtO(MH='Dibenzofurans', MS='Compounds that include the structure of dibenzofuran.'),
  'D000072336': NtO(MH='Mucosal-Associated Invariant T Cells', MS='A subset of T-lymphocytes that are present in large numbers at MUCOUS MEMBRANES and respond to INFECTIONS. They express a conserved invariant T-CELL RECEPTOR ALPHA-CHAIN that enables them to respond to infections by sensing RIBOFLAVIN metabolites of pathogens.'),
  'D000072338': NtO(MH='Dibenzofurans, Polychlorinated', MS='Dibenzofurans that contain chloride atoms bound to the aromatic rings of the structure.'),
  'D000072339': NtO(MH='Sexual and Gender Minorities', MS='Individuals including lesbian, gay, bisexual, transgender, queer, intersex, gender non-conforming people, and other populations whose sexual orientation or GENDER IDENTITY and reproductive development is considered outside cultural, societal, or physiological norms.'),
  'D000072340': NtO(MH='NIMA-Interacting Peptidylprolyl Isomerase', MS='A highly-conserved peptidyl-prolyl cis/trans isomerase (PPIase) that binds to and isomerizes specific phosphorylated SERINE- or THREONINE-PROLINE (pSer/Thr-Pro) motifs and causes conformational changes in certain proteins associated with the CELL CYCLE. It displays a preference for an acidic residue N-terminal to the isomerized proline bond and regulates MITOSIS, possibly by attenuating the mitosis-promoting activity of NIMA-RELATED KINASE 1.'),
  'D000072356': NtO(MH='Parenchymal Tissue', MS='The tissue that constitutes the essential or functional part of an organ (e.g., parenchyma of lung, parenchyma of liver, or parenchyma of kidney).'),
  'D000072357': NtO(MH='Applied Behavior Analysis', MS='Behavioral science applied with the aim of improving socially important issues such as behavior problems and learning. For individuals diagnosed with intellectual and developmental disabilities including AUTISM, techniques can be categorized as comprehensive or focused.'),
  'D000072358': NtO(MH='Work-Life Balance', MS='The optimal arrangement of an individual\xe2\x80\x99s on-the-job and private time to facilitate health and personal satisfaction without negatively impacting productivity and professional success.'),
  'D000072359': NtO(MH='Trichodesmium', MS='A nitrogen-fixing genus of filamentous cyanobacteria that occurs in tropical and subtropical oceans.'),
  'D000072376': NtO(MH='Oxysterols', MS='Oxygenated derivatives of cholesterol or its sterol precursors. They are generated from sterol metabolism and the interaction of cholesterol with REACTIVE OXYGEN SPECIES.'),
  'D000072377': NtO(MH='Syk Kinase', MS='An SH2 domain-containing non-receptor tyrosine kinase that regulates signal transduction downstream of a variety of receptors including B-CELL ANTIGEN RECEPTORS. It functions in both INNATE IMMUNITY and ADAPTIVE IMMUNITY and also mediates signaling in CELL ADHESION; OSTEOGENESIS; PLATELET ACTIVATION; and vascular development.'),
  'D000072378': NtO(MH='Radiation Oncologists', MS='Physicians specializing in the treatment of cancer by RADIOTHERAPY.'),
  'D000072379': NtO(MH='NIMA-Related Kinases', MS='A highly-conserved family of protein serine-threonine kinases that regulate the CELL CYCLE; MITOSIS; and the response to DNA DAMAGE. They are also involved in the assembly and function of microtubule-based structures such as CILIA and CENTRIOLES.'),
  'D000072380': NtO(MH='NIMA-Related Kinase 1', MS='A NIMA-related kinase that functions in CELL CYCLE regulation, the control of CILIA assembly, and CENTROSOME duplication. It is activated at G2 PHASE CELL CYCLE CHECKPOINTS in response to DNA DAMAGE.'),
  'D000072396': NtO(MH='Endothelin-Converting Enzymes', MS='Metalloendopeptidases which convert BIG ENDOTHELIN to ENDOTHELIN-1.'),
  'D000072416': NtO(MH='Protein Structural Elements', MS='Distinguishing three-dimensional structures characteristically formed by homologous protein sequences.'),
  'D000072417': NtO(MH='Protein Domains', MS='Discrete protein structural units that may fold independently of the rest of the protein and have their own functions.'),
  'D000072436': NtO(MH='Transportation Facilities', MS='Places such as AIRPORTS, bus stations, and train stations that are used for managing arriving and departing transport vehicles, and included facilities for handling passengers.'),
  'D000072437': NtO(MH='Calendars as Topic', MS='Works about documents or charts that show days, weeks, and months of a year.'),
  'D000072438': NtO(MH='Calendar', MS='Document or chart that shows days, weeks, and months of a year.'),
  'D000072439': NtO(MH='Postcards as Topic', MS='Works about cards on which a message may be written or printed for mailing without an envelope.'),
  'D000072440': NtO(MH='Public Health Systems Research', MS='A field of study that examines the organization, financing, and delivery of public health services within communities, and the impact of these services on public health.'),
  'D000072441': NtO(MH='DNA, Ancient', MS='DNA isolated from fossils or other ancient specimens.'),
  'D000072458': NtO(MH='Orientation, Spatial', MS='Change in position or alignment in response to an external stimulus.'),
  'D000072461': NtO(MH='Cytochrome P450 Family 1', MS='A cytochrome P450 enzyme family whose members function primarily in the metabolism of XENOBIOTICS, including drugs and POLYCYCLIC AROMATIC HYDROCARBONS.'),
  'D000072466': NtO(MH='Cognitive Remediation', MS='Behavioral treatment that uses drill and practice, compensatory and adaptive strategies to facilitate improvement in targeted learning areas.'),
  'D000072467': NtO(MH='Cytochrome P450 Family 2', MS='A cytochrome P450 enzyme family that includes members which function in the metabolism of STEROIDS; COUMARINS; and NICOTINE.'),
  'D000072469': NtO(MH='Cytochrome P450 Family 3', MS='A cytochrome P450 enzyme family that includes members with critical functions in the metabolism of drugs and SEX HORMONES.'),
  'D000072470': NtO(MH='Cytochrome P450 Family 4', MS='A cytochrome P450 enzyme family that includes members which function in the metabolism of FATTY ACIDS, especially ARACHIDONIC ACIDS and their derivatives.'),
  'D000072471': NtO(MH='Heterocyclic Compounds, Fused-Ring', MS='Multiple ring heterocyclic compounds containing two or more rings that share two atoms and one bond in common.'),
  'D000072472': NtO(MH='Cytochrome P450 Family 6', MS='A cytochrome P450 enzyme family that occurs in insects. Its members function in the metabolism of XENOBIOTICS.'),
  'D000072473': NtO(MH='Fused-Ring Compounds', MS='Polycyclic compounds with adjacent rings that share the same two adjacent atoms.'),
  'D000072474': NtO(MH='Cytochrome P450 Family 7', MS='A cytochrome P450 enzyme family whose members function as steroid 7-alpha hydroxylases.'),
  'D000072475': NtO(MH='Cytochrome P450 Family 8', MS='A cytochrome P450 enzyme family whose members include prostacyclin synthase and STEROID 12-ALPHA-HYDROXYLASE, which function in the biosynthesis of BILE ACIDS.'),
  'D000072476': NtO(MH='Cytochrome P450 Family 11', MS='A cytochrome P450 enzyme family whose members localize to the INNER MITOCHONDRIAL MEMBRANE where they function in the biosynthesis of STEROIDS.'),
  'D000072477': NtO(MH='Cytochrome P450 Family 12', MS='A cytochrome P450 enzyme family that occurs in insects and is expressed in MITOCHONDRIA.'),
  'D000072478': NtO(MH='Cytochrome P450 Family 17', MS='A cytochrome P450 enzyme family that functions in the biosynthesis of STEROIDS and includes STEROID 17-ALPHA-HYDROXYLASE.'),
  'D000072480': NtO(MH='Farms', MS='Areas of land set aside to grow crops or raise LIVESTOCK.'),
  'D000072481': NtO(MH='DNA Ligase ATP', MS='ATP-dependent cellular enzyme which catalyzes DNA replication, repair and recombination through formation of internucleotide ester bonds between phosphate and deoxyribose moieties. Vertebrate cells encode three well-characterized DNA ligases, DNA ligase I, III and IV, all of which are related in structure and sequence. DNA ligases either require ATP or NAD. However, archaebacterial, viral, and some eubacterial DNA ligases are ATP-dependent.'),
  'D000072482': NtO(MH='Retinoic Acid Receptor alpha', MS='A nuclear receptor protein which belongs to a family of transcription regulators of ligand-responsive regulatory proteins which include steroid hormone receptors. In addition to a C-terminal ligand-binding domain, these nuclear receptors contain a highly-conserved N-terminal zinc-finger domain that mediates binding ligand-responsive elements.'),
  'D000072483': NtO(MH='Placenta Growth Factor', MS='An angiogenic protein belonging to the Vascular Endothelial Growth Factor family of growth factors originally isolated and cloned from human placental cDNA library. There are four isoforms of PLGF 1-4 which result from alternative splicing. Placenta Growth Factor is secreted as a glycosylated homodimer which acts as a mitogen for endothelial cells, and its expression is markedly upregulated during hypoxia and in tissue inflammation and cancer.'),
  'D000072496': NtO(MH='Cytochrome P450 Family 19', MS='A cytochrome P450 enzyme family that includes AROMATASE, which functions in the biosynthesis of ESTROGENS.'),
  'D000072497': NtO(MH='Cytochrome P450 Family 21', MS='A cytochrome P450 enzyme family that includes STEROID 21-HYDROXYLASE, which functions in the biosynthesis of STEROIDS.'),
  'D000072498': NtO(MH='Cytochrome P450 Family 24', MS='A cytochrome P450 enzyme family that includes VITAMIN D3 24-HYDROXYLASE.'),
  'D000072499': NtO(MH='Manufacturing and Industrial Facilities', MS='Places provided for the processing, fabrication, assembly, and production of a line of products, services, commodities, or merchandise.'),
  'D000072500': NtO(MH='Cytochrome P450 Family 26', MS='A cytochrome P450 enzyme family whose members function in the metabolism of RETINOIC ACID. It includes RETINOIC ACID 4-HYDROXYLASE.'),
  'D000072501': NtO(MH='Cytochrome P450 Family 27', MS='A cytochrome P450 enzyme family whose members function in VITAMIN D metabolism and the biosynthesis of BILE ACIDS.'),
  'D000072503': NtO(MH='Cytochrome P450 Family 46', MS='A cytochrome P450 enzyme family that includes CHOLESTEROL 24-HYDROXYLASE, which is expressed primarily in the brain and functions in CHOLESTEROL metabolism.'),
  'D000072505': NtO(MH='Cytochrome P450 Family 51', MS='A cytochrome P450 family that occurs in all EUKARYOTES. Its members function in the biosynthesis of STEROIDS and some are targets of ANTIFUNGAL AGENTS.'),
  'D000072506': NtO(MH='Gardens', MS='Places provided for the cultivation and/or display of PLANTS.'),
  'D000072516': NtO(MH='Retinoic Acid 4-Hydroxylase', MS='A cytochrome P450 enzyme that resides in the ENDOPLASMIC RETICULUM. It catalyzes the conversion of trans-RETINOIC ACID to 4-hydroxyretinoic acid.'),
  'D000072556': NtO(MH='Cholesterol 24-Hydroxylase', MS='A steroid hydroxylase that functions in CHOLESTEROL homeostasis in the brain. It converts cholesterol into 24S-hydroxycholesterol, which is transported over the BLOOD-BRAIN BARRIER more rapidly than unmetabolized cholesterol.'),
  'D000072576': NtO(MH='Homeobox Protein Nkx-2.5', MS='A homeobox-containing transcription factor that functions in heart development. Mutations in the NKX2-5 gene are associated with ATRIAL SEPTAL DEFECTS and TETRALOGY OF FALLOT.'),
  'D000072577': NtO(MH='Moral Status', MS='A philosophical description of intrinsic self-worth and dignity which bestows basic rights of life, liberty, and freedom from harm.'),
  'D000072596': NtO(MH='Hepatitis A Virus Cellular Receptor 1', MS='An Ig domain-containing membrane receptor for HEPATITIS A VIRUS; EBOLA VIRUS; MARBURG VIRUS; and DENGUE VIRUS. It may also function to modulate ASTHMA and HYPERSENSITIVITY.'),
  'D000072597': NtO(MH='Hepatitis A Virus Cellular Receptor 2', MS='An Ig domain-containing membrane receptor that is expressed by TH1 CELLS. It regulates the activation of MACROPHAGES and inhibits TH1-mediated auto- and alloimmune responses to promote IMMUNE TOLERANCE.'),
  'D000072598': NtO(MH='Tumor Necrosis Factor alpha-Induced Protein 3', MS='A UBIQUITIN editing enzyme that functions as both a ubiquitin ligase and deubiquitinase. It contains several ZINC FINGERS and functions in the immune response and INFLAMMATION by modulating signals from TNF-ALPHA; IL1-BETA; or pathogens via TOLL-LIKE RECEPTORS to terminate NF-KAPPA B activity.'),
  'D000072599': NtO(MH='Cardiorespiratory Fitness', MS='A measure of the functional capabilities of the heart, lungs and muscles, relative to the demands of specific exercise routines such as running or cycling.'),
  'D000072600': NtO(MH='Meniscus', MS='Crescent-shaped cartilaginous tissue interposed between two articulating bones.'),
  'D000072601': NtO(MH='Cannula', MS='Short, hollow cylinders or tubes for insertion into a canal, vessel, or body cavity.'),
  'D000072621': NtO(MH='Peptidyl-Prolyl Cis-Trans Isomerase NIMA-Interacting 4', MS='A NIMA-interacting peptidylprolyl isomerase. It binds to double-stranded DNA and functions in the biogenesis of RIBOSOMES.'),
  'D000072622': NtO(MH='Cyclophilin C', MS='A peptidyl-prolyl cis-trans isomerase and member of the cyclophilin family. It catalyzes the cis-trans isomerization of imide-like peptide bonds that occur between PROLINE and other amino acids to facilitate PROTEIN FOLDING.'),
  'D000072636': NtO(MH='PAX8 Transcription Factor', MS='A paired box transcription factor that functions in the development of THYROCYTES and the control of thyroid-specific gene expression. Mutations in the PAX8 gene are associated with THYROID DYSGENESIS and THYROID NEOPLASMS.'),
  'D000072637': NtO(MH='Thyroid Epithelial Cells', MS='Epithelial cells of the THYROID GLAND that produce and secrete THYROXINE and TRIIODOTHYRONINE.'),
  'D000072638': NtO(MH='Patient Portals', MS='A secure online website that provides patients convenient 24-hour access to personal health information via an Internet connection.'),
  'D000072639': NtO(MH='Perfectionism', MS="A personality trait characterized by a person's striving for flawlessness and setting excessively high performance standards, accompanied by overly critical self-evaluations and concerns regarding others' evaluations."),
  'D000072640': NtO(MH='Interferon-Induced Helicase, IFIH1', MS='A DEAD box RNA helicase that contains two N-terminal CASPASE ACTIVATION AND RECRUITMENT DOMAINS. It functions as a sensor of viral NUCLEIC ACIDS such as DOUBLE-STRANDED RNA and activates the INNATE IMMUNE RESPONSE by inducing the expression of INTERFERON-ALPHA and INTERFERON-BETA. It may also regulate cell growth and APOPTOSIS.'),
  'D000072641': NtO(MH='Medicare Access and CHIP Reauthorization Act of 2015', MS="An Act that amends Title XVIII of the Social Security Act to repeal the Medicare sustainable growth rate, that strengthens Medicare access by improving  physician payments, and that reauthorizes the Children's Health Insurance Program (CHIP)."),
  'D000072643': NtO(MH='Study Guide', MS='Tool used to help facilitate learning and comprehension of a topic or to help prepare for an examination.'),
  'D000072644': NtO(MH='Study Guides as Topic', MS='Works about use of aids to help facilitate learning and comprehension of a topic or to help prepare for an examination.'),
  'D000072645': NtO(MH='Cancer-Associated Fibroblasts', MS='Subpopulation of heterogeneous fibroblasts within the TUMOR MICROENVIRONMENT that support NEOPLASTIC CELL TRANSFORMATION and NEOPLASTIC PROCESSES.'),
  'D000072656': NtO(MH='Breast Cancer Lymphedema', MS='Abnormal accumulation of lymph in the arm, shoulder and breast area associated with surgical treatment (e.g., MASTECTOMY) or radiation treatment of breast cancer.'),
  'D000072657': NtO(MH='ST Elevation Myocardial Infarction', MS='A clinical syndrome defined by MYOCARDIAL ISCHEMIA symptoms; persistent elevation in the ST segments of the ELECTROCARDIOGRAM; and release of BIOMARKERS of myocardial NECROSIS (e.g., elevated TROPONIN levels). ST segment elevation in the ECG is often used in determining the treatment protocol (see also NON-ST ELEVATION MYOCARDIAL INFARCTION).'),
  'D000072658': NtO(MH='Non-ST Elevated Myocardial Infarction', MS='A myocardial infarction that does not produce elevations in the ST segments of the ELECTROCARDIOGRAM. ST segment elevation of the ECG is often used in determining the treatment protocol (see also ST Elevation Myocardial Infarction).'),
  'D000072659': NtO(MH='Hypophysitis', MS='Inflammation of the PITUITARY GLAND.'),
  'D000072660': NtO(MH='Teratozoospermia', MS='Conditions in which sperm show abnormal morphology.'),
  'D000072661': NtO(MH='Ciliopathies', MS='Genetic disorders caused by defects in genes related to the primary CILIUM; BASAL BODY; or CENTROSOME. Primary features may include obesity, SKELETAL DYSPLASIA; POLYDACTYLY and malformations that primarily involve the liver, eye or kidneys.'),
  'D000072662': NtO(MH='Margins of Excision', MS='The edges of tissue removed in a surgery for assessment of the effectiveness of a surgical procedure in achieving the local control of a neoplasm and the adequacy of tumor removal. When the margin is negative or not involved by tumor (e.g., CANCER) it suggests all of the tumor has been removed by the surgery.'),
  'D000072663': NtO(MH='Spontaneous Perforation', MS='A pathological hole in an organ, blood vessel or other soft part of the body, occurring in the absence of external force.'),
  'D000072664': NtO(MH='Provitamins', MS='Precursor forms of vitamins.'),
  'D000072666': NtO(MH='National Center for Advancing Translational Sciences (U.S.)', MS='A unit of the National Institutes of Health that seeks to catalyze the generation of innovative methods and technologies that will enhance the development, testing, and implementation of diagnostics and therapeutics across a wide range of human diseases and conditions. The Center was established in 2011.'),
  'D000072667': NtO(MH='National Center for Complementary and Integrative Health (U.S.)', MS='The mission of NCCIH is to define, through rigorous scientific investigation, the usefulness and safety of complementary and integrative health interventions and their roles in improving health and health care. The Center was established in 1999 and was known as NCCAM National Center of Complementary and Alternative Medicine until 2015.'),
  'D000072668': NtO(MH='Splicing Factor U2AF', MS="An RNA splicing factor that performs a critical function in both constitutive and enhancer-dependent RNA SPLICING. It recruits RIBONUCLEOPROTEIN, U2 SMALL NUCLEAR to the splice site and mediates interactions between it, the RNA molecule, and other splicing factors for accurate 3'-splice site selection."),
  'D000072669': NtO(MH='Gene Editing', MS="Genetic engineering or molecular biology techniques that involve DNA REPAIR mechanisms for incorporating site-specific modifications into a cell's genome."),
  'D000072670': NtO(MH='PTB-Associated Splicing Factor', MS='An RNA splicing factor that also binds DNA. It is essential for pre-mRNA spicing in the formation of early SPLICEOSOMES and catalytic step II. It binds specifically to polypyrimidine tracts in INTRONS and is involved in the regulation of ALTERNATIVE SPLICING by SIGNAL TRANSDUCTION PATHWAYS.'),
  'D000072676': NtO(MH='Ankyloglossia', MS='A severe congenital restriction of TONGUE movement, resulting from fusion or adherence of the tongue to the floor of the mouth. In partial ankyloglossia (tongue-tie) the LINGUAL FRENUM is abnormally short, or is attached too close to the tip of the tongue. OMIM: 106280'),
  'D000072696': NtO(MH='High-Intensity Interval Training', MS='A cardiovascular exercise strategy with alternating short periods of intense anaerobic exercise with less-intense recovery periods.'),
  'D000072697': NtO(MH='Desegregation', MS='Ending segregation (separation) of different groups of people, usually based on race, in schools, housing, organizations, etc.'),
  'D000072698': NtO(MH='Apartheid', MS='A former political system in SOUTH AFRICA which enforced segregation in all forms, including physical, political, legal and economic discrimination, against all non-white, non-European racial groups.'),
  'D000072699': NtO(MH='Social Segregation', MS='Barriers based on group membership.'),
  'D000072700': NtO(MH='Conservative Treatment', MS='Therapeutic approaches that are limited, gradual, or well-established as opposed to radical methods.'),
  'D000072716': NtO(MH='Cancer Pain', MS='Pain that may be caused by or related to cellular, tissue, and systemic changes that occur during NEOPLASM growth, tissue invasion, and METASTASIS.'),
  'D000072717': NtO(MH='Tertiary Lymphoid Structures', MS='Aggregates of immune cells formed near the sites of chronically inflamed, infected, or tumorous tissues.'),
  'D000072736': NtO(MH='Propofol Infusion Syndrome', MS='Rare and often fatal drug complication which affects patients undergoing long-term treatment with high doses of PROPOFOL. It is characterized by METABOLIC ACIDOSIS; HYPERLIPIDEMIA; RHABDOMYOLYSIS; cardiovascular CIRCULATORY COLLAPSE; CARDIAC FAILURE; and KIDNEY FAILURE.'),
  'D000072737': NtO(MH='Myeloid-Derived Suppressor Cells', MS='A heterogeneous, immature population of myeloid cells that can suppress the activity of T-CELLS and NATURAL KILLER CELLS in the INNATE IMMUNE RESPONSE and ADAPTIVE IMMUNE RESPONSE. They play important roles in ONCOGENESIS; INFLAMMATION; and INFECTION.'),
  'D000072738': NtO(MH='Nitrogen-Fixing Bacteria', MS='Bacteria that are capable of NITROGEN FIXATION.'),
  'D000072739': NtO(MH='Sentinel Species', MS='Organisms used to determine measurable environmental risks or hazards to human health and or well-being, thereby serving as advance or early warning signs of impending danger to humans. Examples of sentinel species are monkeys, guinea pigs, and the fabled canary in the coal mine.'),
  'D000072740': NtO(MH='Paternal Inheritance', MS='A form of inheritance where the traits of the offspring are paternal in origin due to the expression of extra-nuclear genetic material such as MITOCHONDRIAL DNA or Y chromosome genes. CENTRIOLES are also paternally inherited.'),
  'D000072741': NtO(MH='Maternal Inheritance', MS='Transmission of genetic characters, qualities, and traits, solely from maternal extra-nuclear elements such as MITOCHONDRIAL DNA or MATERNAL MESSENGER RNA.'),
  'D000072742': NtO(MH='Invasive Fungal Infections', MS='Mycoses which manifest as infections of deep tissue or blood.'),
  'D000072743': NtO(MH='Beta-Cryptoxanthin', MS='A mono-hydroxylated xanthophyll that is a provitamin A precursor.'),
  'D000072756': NtO(MH='Protein Conformation, alpha-Helical', MS='A secondary structure of proteins that is a right-handed helix or coil, where each amino (N-H) group of the peptide backbone contributes a hydrogen bond to the carbonyl(C=O) group of the amino acid four residues N-terminal to it (n-4). It is the most common type of secondary structure.'),
  'D000072757': NtO(MH='Protein Conformation, beta-Strand', MS='A secondary structure of proteins where the amino (N-H) groups of a polypeptide backbone, three to ten amino acids in length, establish hydrogen bonds with the carbonyl (C=O) groups in the backbone of adjacent strands.  These may form a beta-sheet, where the side chains of the adjacent strands point in the same direction.'),
  'D000072758': NtO(MH='Vaccination Refusal', MS='Refusal to receive VACCINATION.'),
  'D000072759': NtO(MH='Biofortification', MS="Improving the NUTRITIONAL VALUE of plant foods by augmenting the plant's environment, by using PLANT BREEDING, or by GENETIC ENGINEERING plants to increase their ability to synthesize VITAMINS or uptake essential NUTRIENTS from the soil."),
  'D000072760': NtO(MH='Single Molecule Imaging', MS='High resolution imaging techniques that allow visualization of individual molecules of proteins, lipids, or nucleic acids within cells or tissues.'),
  'D000072761': NtO(MH='Mycobiome', MS='The full spectrum of FUNGI that exist within a particular biological niche such as an organism, soil, a body of water, etc.'),
  'D000072776': NtO(MH='Slit Lamp Microscopy', MS='A procedure that uses a SLIT LAMP to examine structures in the front of the EYE, such as the CONJUNCTIVA; CORNEA; IRIS; and AQUEOUS HUMOR.'),
  'D000072777': NtO(MH='Host-Seeking Behavior', MS='Searching behavior by parasitic or commensal organisms, to find animals or plants that are a source of nourishment and habitation.'),
  'D000072797': NtO(MH='Walking Speed', MS='The rate at which steps are made while walking.'),
  'D000072799': NtO(MH='Stair Climbing', MS='Ascending or descending stairs.'),
  'D000072817': NtO(MH='Eryptosis', MS='Suicidal death of ERYTHROCYTES which results in features typical of apoptotic nucleated cells such as cell shrinkage, membrane blebbing, and scrambling of cell membrane to expose membrane PHOSPHATIDYLSERINES, which triggers engulfment and degradation by MACROPHAGES.'),
  'D000072836': NtO(MH='Surgical Wound', MS='An incision made during a surgical procedure.'),
  'D000072837': NtO(MH='Chromothripsis', MS='Massive number of chromosomal rearrangements and shattering that occurs in cancer cells. The breakpoints are located within one chromosome or chromosome arm.'),
  'D000072856': NtO(MH='Radiologic and Imaging Nursing', MS='A nursing specialty that deals specifically with the care of patients undergoing diagnostic, neurological, cardiovascular, interventional, ultrasonography, computerized tomography, nuclear medicine, magnetic resonance, radiation oncology or other related procedures .'),
  'D000072858': NtO(MH='Psychosocial Support Systems', MS='Organized efforts to address ongoing psychological and social problems of individuals, their partners, families and caregivers.'),
  'D000072860': NtO(MH='Ecological Momentary Assessment', MS="Repeated reports by a research participant of symptoms, affect, behavior, and cognition close in time to experience and in the participant's natural environment."),
  'D000072861': NtO(MH='Phobia, Social', MS='Anxiety disorder characterized by the persistent and irrational fear, anxiety, or avoidance of social or performance situations.'),
  'D000072878': NtO(MH='Simplified Acute Physiology Score', MS='A severity of disease classification system designed to measure the severity of disease for patients aged 15 and over admitted to intensive care units.'),
  'D000072896': NtO(MH='Morgue', MS='Location where dead bodies are kept until they are buried or cremated.'),
  'D000072916': NtO(MH='Peripheral Blood Stem Cells', MS='Hematopoietic stem cells found in peripheral blood circulation.'),
  'D000072917': NtO(MH='Standardized Nursing Terminology', MS='Standardized terms used in the nursing field (e.g., NURSING INFORMATICS).'),
  'D000072936': NtO(MH='Fitness Trackers', MS='Devices used for measuring physical activity as an indication of HEALTH STATUS.'),
  'D000072956': NtO(MH='Adult Germline Stem Cells', MS='Progenitor stem cells found in the testicles.'),
  'D000072957': NtO(MH='Alaska Natives', MS='Persons having origins in any indigenous people of ALASKA and their descendants and who maintain tribal affiliation, or community or cultural attachment.'),
  'D000072977': NtO(MH='Oogonial Stem Cells', MS='Primordial germ cells found in embryonic OOGONIA and postnatal OVARIES.'),
  'D000073': NtO(MH='Acenaphthenes', MS='Tricyclic ethylene-bridged naphthalene derivatives. They are found in petroleum residues and coal tar and used as dye intermediates, in the manufacture of plastics, and in insecticides and fungicides.'),
  'D000073017': NtO(MH='Refugee Camps', MS='Areas of shelter and protection for people who leave their own country or habitual residence to escape danger, persecution, or disaster.'),
  'D000073038': NtO(MH='Mental Health Recovery', MS='Recovery from mental disorders and/or substance abuse. The process of change in which individuals improve their MENTAL HEALTH and wellness, live a self-directed life, and work to achieve their full potential.'),
  'D000073039': NtO(MH='Nitrosative Stress', MS='A metabolic excess of REACTIVE NITROGEN SPECIES, including NITRIC OXIDE and PEROXYNITRITE, that leads to damaging effects of oxidation and nitration.'),
  'D000073056': NtO(MH='Transactinide Series Elements', MS='A series of radioactive elements with atomic numbers greater than 103. Current members include Rutherfordium, atomic number 104, to and including Ununoctium, atomic number 118. All members are synthetic with short half-lives.'),
  'D000073076': NtO(MH='Aruba', MS='Island in the Caribbean Sea, north of Venezuela. In 1986 it became a separate, autonomous member of the Kingdom of the Netherlands.'),
  'D000073096': NtO(MH='Curacao', MS='Constituent country within the Kingdom of the Netherlands. Full autonomy in internal affairs was granted in 2010. It is an island located in the Caribbean Sea - north of Venezuela. (from CIA World Factbook)'),
  'D000073097': NtO(MH='Sint Maarten', MS='Dutch part of the island of Saint Martin in the Caribbean Sea; Sint Maarten lies east of the US Virgin Islands.'),
  'D000073116': NtO(MH='Cancer Survivors', MS='Persons who have experienced prolonged survival with or following malignant neoplastic disease.'),
  'D000073136': NtO(MH='Sao Tome and Principe', MS='Islands in the Gulf of Guinea, just north of the Equator, and west of Gabon.'),
  'D000073137': NtO(MH='Caribbean Netherlands', MS='A group Caribbean islands including Bonaire, Sint Eustatius, and Saba.'),
  'D000073156': NtO(MH='Naja', MS='A genus of elapid snakes, also known as cobras, that are indigenous to Africa, Central and Southern Asia, and adjacent islands such as Taiwan and the Philippines.'),
  'D000073176': NtO(MH='Naja naja', MS='Species of cobra (Naja) indigenous to the Indian Subcontinent, Southern Asia, and adjacent islands.'),
  'D000073177': NtO(MH='Naja haje', MS='A species of cobra (Naja) that is indigenous primarily to Northern Africa.'),
  'D000073178': NtO(MH='Dendroaspis', MS='A genus of highly venomous elapid snake indigenous to sub-Saharan Africa.'),
  'D000073179': NtO(MH='Hemachatus', MS='A genus of elapid snake indigenous to Southern Africa. It is closely related to the cobras (Naja) and is capable of spitting its venom.'),
  'D000073180': NtO(MH='Hydrophiidae', MS='A subfamily of marine elapid snakes comprising about 50 species with flattened oar-like tails used as sculls. They are found mostly in the coastal waters of south Asia and Australia. The largest reach a length of almost 9 feet but most species are only about a third as long. They are all venomous. (Goin, Goin, and Zug, Introduction to Herpetology, 3d ed, pp331-3; Moore: Poisonous Snakes of the World, 1980, p159)'),
  'D000073181': NtO(MH='Coral Snakes', MS='Elapid snakes indigenous to the Southern United States, Central, and South America. They are generally less than 1 meter in length and have colored ring patterns.'),
  'D000073182': NtO(MH='Carbapenem-Resistant Enterobacteriaceae', MS='Strains of Enterobacteriaceae that are resistant to CARBAPENEMS, primarily due to the acquisition of carbapenemase (BETA-LACTAMASE) genes.'),
  'D000073183': NtO(MH='Platelet-Rich Fibrin', MS='A fibrin matrix derived from platelet-rich plasma that contains high concentration of BLOOD PLATELETS; LEUKOCYTES; CYTOKINES; and GROWTH FACTORS. It is used in a variety of clinical and TISSUE ENGINEERING applications.'),
  'D000073196': NtO(MH='Integrative Oncology', MS='These evidence-based therapies to reduce symptoms associated with treatment of cancer.'),
  'D000073197': NtO(MH='Ophiophagus hannah', MS='The longest of all venomous snakes and largest Elapid. It is not a member of the Naja genus, although its hood resembles the hood of Naja species. Its bite can deliver large quantities of neurotoxic and cardiotoxic venom, consisting primarily of ALPHA-NEUROTOXINS. King cobras are indigenous to forests of India and Southeast Asia.'),
  'D000073216': NtO(MH='Mental Status and Dementia Tests', MS='Tests designed to assess various aspects of neurocognitive function or dementia.'),
  'D000073218': NtO(MH='Wisconsin Card Sorting Test', MS='A neuropsychological test designed to assess EXECUTIVE FUNCTION typically assigned to the FRONTAL LOBE (e.g., abstract thinking, and strategic planning). The subjects are asked to sort numbered response cards according to different principles and to alter their approach during testing.'),
  'D000073219': NtO(MH='Memory and Learning Tests', MS='Tests designed to evaluate general and specific areas of behaviors and abilities associated with memory and/or learning.'),
  'D000073220': NtO(MH='Wechsler Memory Scale', MS='A neuropsychological test designed to assess different memory functions. It may incorporate an optional cognitive exam (Brief Cognitive Status Exam) that helps to assess memory related cognitive function.'),
  'D000073222': NtO(MH='Patient Health Questionnaire', MS='A self-administered version of the Primary Care Evaluation of Mental Disorders (PRIME-MD), a diagnostic tool containing modules on multiple mental health disorders including anxiety, alcohol, eating, and somatoform modules. The Patient Health Questionnaire (PHQ-9) is designed specifically for mood/depression scoring each of the 9 DSM-IV criteria based on the mood module from the original PRIME-MD. The Generalized Anxiety Disorder scale (GAD-7) scores 7 common anxiety symptoms.'),
  'D000073256': NtO(MH='Information Technology', MS='Interconnected system that is used in the automatic acquisition, storage, manipulation, movement, control, display, interchange, transmission, routing or reception of data or information, including computers, ancillary equipment, system software, support services, and related resources. (NAL Agricultural Thesaurus)'),
  'D000073276': NtO(MH='Models, Spatial Interaction', MS='Estimates of the flow of people, material or information between locations in geographic space.'),
  'D000073278': NtO(MH='Self-Management', MS="Individual's ability to manage the symptoms, treatment, physical and psychosocial consequences and lifestyle changes inherent in living with a chronic condition. Efficacious self-management encompasses ability to monitor one's condition and to affect the cognitive, behavioral, and emotional responses necessary to maintain a satisfactory quality of life."),
  'D000073296': NtO(MH='Noncommunicable Diseases', MS='Diseases which are typically non-infectious in origin and do not transmit from an affected individual to others. The four main types of noncommunicable diseases are CARDIOVASCULAR DISEASES (e.g., heart attacks and stroke), CANCER, chronic respiratory diseases (e.g., CHRONIC OBSTRUCTIVE PULMONARY DISEASE and ASTHMA) and DIABETES MELLITUS.'),
  'D000073297': NtO(MH='Manual Lymphatic Drainage', MS='The application of massage to control EDEMA and improve circulation by manually moving excess lymph out of a tissue.'),
  'D000073316': NtO(MH='Addiction Medicine', MS='A medical specialty focused on the diagnosis and treatment of ADDICTIVE BEHAVIOR disorders, including SUBSTANCE-RELATED DISORDERS and IMPULSE CONTROL DISORDERS; and the management of co-occurring medical and psychiatric conditions'),
  'D000073319': NtO(MH='Obesity Management', MS='An integrated professional approach to screening, evaluation, control, and reduction of abnormal WEIGHT GAIN.'),
  'D000073320': NtO(MH='Psycho-Oncology', MS='A specialty which deals with the interrelationship of physical, psychological, social, behavioral, and ethical aspects of cancer. Psycho-oncology examines the behavioral and psychosocial factors that may influence the course of the disease, cancer risk, prevention, and detection.'),
  'D000073336': NtO(MH='Whole Genome Sequencing', MS='Techniques to determine the entire sequence of the GENOME of an organism or individual.'),
  'D000073356': NtO(MH='Pictorial Works as Topic', MS='Works that discuss pictures but not technical drawings.'),
  'D000073358': NtO(MH='Mycobacterium abscessus', MS='A rapidly growing non-tuberculous environmental mycobacterium causing OPPORTUNISTIC INFECTION that infects the skin and subcutaneous tissues. It is associated with HEALTH CARE ASSOCIATED INFECTION and causes serious lung infections in persons with various chronic lung diseases.'),
  'D000073359': NtO(MH='Whole Exome Sequencing', MS='Techniques to determine the complete complement of sequences of all EXONS of an organism or individual.'),
  'D000073376': NtO(MH='Epileptic Syndromes', MS='EPILEPTIC SEIZURES that are of similar type and age of onset and have other similar features (e.g., clinical course, EEG findings, genetic association and neuropathology).'),
  'D000073396': NtO(MH='Metal-Organic Frameworks', MS='Supramolecular networks that consist of ordered arrangements of organic electron donor linkers (usually ditopic or polytopic organic carboxylates) and metal cations. They can have an extremely high surface area and adjustable pore size that allows for the insertion of other molecules capable of various functions such as catalysis, capture of carbon dioxide, and drug delivery.'),
  'D000073397': NtO(MH='Occupational Stress', MS='Adverse psychological and behavioral reactions caused by the pressures and demands of employers or clients or other factors, such as the physical environment of the workplace, WORKPLACE VIOLENCE; or WORKPLACE BULLYING.'),
  'D000073398': NtO(MH='Demethylation', MS='Removal of one or more methyl groups from a chemical compound.'),
  'D000073399': NtO(MH='DNA Demethylation', MS='Removal of methyl groups from DNA by enzymes (such as DIOXYGENASES and N-DEMETHYLASES) or by chemical reagents.'),
  'D000073417': NtO(MH='Dietary Sugars', MS='MONOSACCHARIDES and DISACCHARIDES present in food, such as those present in fruits and vegetables and milk products, or those added to food such as DIETARY SUCROSE (table sugar) or HIGH FRUCTOSE CORN SYRUP.'),
  'D000073436': NtO(MH='Microvascular Rarefaction', MS='The reduction in density of the MICROVASCULATURE.'),
  'D000073456': NtO(MH='Multitasking Behavior', MS='Simultaneous task performance, or switching between tasks in a concentrated period of time.'),
  'D000073458': NtO(MH='Data Warehousing', MS='A system for storing electronic data derived from various sources.'),
  'D000073476': NtO(MH='Laticauda', MS='A genus of semi-aquatic elapid snake that inhabits coastal waters of the tropical Indian and Western Pacific Oceans.'),
  'D000073496': NtO(MH='Frailty', MS='A state of increased vulnerability to stressors, following declines in function and reserves across multiple physiologic systems, characterized by MUSCLE WEAKNESS; FATIGUE; slowed motor performance; low physical activity; and unintentional weight loss.'),
  'D000073516': NtO(MH='Contraceptive Effectiveness', MS='The rate of success or failure of a method of CONTRACEPTION; CONTRACEPTIVE AGENTS; or CONTRACEPTIVE DEVICES.'),
  'D000073537': NtO(MH='Biological Variation, Population', MS='Differences in measurable biological values, characteristics, or traits, among individuals of a population or between population groups.'),
  'D000073556': NtO(MH='Burkholderiales', MS='An order of gram-negative bacteria of the class BETAPROTEOBACTERIA which include highly pathogenic species such as BURKHOLDERIA; BORDETELLA; and RALSTONIA.'),
  'D000073557': NtO(MH='Campylobacteraceae', MS='A large and diverse group in the order CAMPYLOBACTERALES, individual species of which grow in aerobic and anaerobic conditions as free-living, commensal, or pathogenic forms.'),
  'D000073559': NtO(MH='Caryophyllales', MS='Diverse and ecologically specialized dicotyledonous flowering plants that include trees, annuals, shrubs, lianas, mangroves, stem or leaf succulents, and insectivores; acting as important food sources in many cases.'),
  'D000073560': NtO(MH='Chromadorea', MS='A class of invertebrate freshwater roundworms of the phylum NEMATODA. Most members are parasites with well-adapted body surfaces and sophisticated esophageal glands and pharynx.'),
  'D000073561': NtO(MH='Clostridiaceae', MS='A non-sulfate spore-forming anaerobic family of Gram-positive bacteria, of the class clostridia.'),
  'D000073562': NtO(MH='Crotalinae', MS='A subfamily of snakes commonly known as pit vipers, crotaline snakes, or pit adders in the family VIPERIDAE. They are distinguished by a deep pit called a fossa or loreal, which functions as a heat-sensing organ located between the eye and nostril on either side of the head.'),
  'D000073563': NtO(MH='Culicomorpha', MS='Insects of the order DIPTERA, suborder NEMATOCERA. They include mosquitoes, gnats, black flies, and true flies.'),
  'D000073564': NtO(MH='Desulfovibrionaceae', MS='A family of obligately anaerobic Gram-negative bacteria of the order Desulfovibrionales, class DELTAPROTEOBACTERIA. Majority are sulfate-reducing.'),
  'D000073566': NtO(MH='Eutheria', MS='Mammals which nourish their young in utero by means of a complex placenta, and give birth to their young alive. They include PRIMATES; CARNIVORA, WHALES; RUMINANTS; BATS; and RODENTS.'),
  'D000073567': NtO(MH='Fagales', MS='Order of dicotyledonous flowering plants in the superorder ROSANAE, and class MAGNOLIOPSIDA. They include birch, beech, bayberry, and walnut species.'),
  'D000073568': NtO(MH='Feliformia', MS='A suborder of cat-like mammals in the order CARNIVORA. Examples include HYENAS; MONGOOSES; CIVETS; and related species. They are distinguishable from dog-like carnivores.'),
  'D000073569': NtO(MH='Helicobacteraceae', MS='A family of Gram-negative bacteria of the order CAMPYLOBACTERALES that have a helical shape, and occur in the mammalian digestive track.'),
  'D000073571': NtO(MH='Laurales', MS='Order of flowering plants in the superorder Magnolianae. Common examples are bay laurel, AVOCADO; sassafras and CINNAMON.'),
  'D000073576': NtO(MH='Faith-Based Organizations', MS='Organizations such as (1) congregations; (2) national networks, which include national denominations, their social service arms (for example, Catholic Charities, Lutheran Social Services), and networks of related organizations (such as YMCA and YWCA); and (3) freestanding religious organizations, which are incorporated separately from congregations and national networks. http://www.huduser.gov/portal/publications/faithbased.pdf'),
  'D000073577': NtO(MH='Shift Work Schedule', MS='Job schedule in which working hours deviate from the standard hours (e.g., evening shift, night shift or rotating shift).'),
  'D000073597': NtO(MH='Stakeholder Participation', MS='A process between an entity and those groups or individuals potentially or actually impacted by the actions of that entity over a range of activities and approaches.'),
  'D000073599': NtO(MH='Health Risk Behaviors', MS='Pattern of behavior which predisposes certain individuals to increased risk for contracting disease or sustaining personal injury. These behaviors may cluster into a risky lifestyle.'),
  'D000073600': NtO(MH='Diet, High-Protein', MS='A diet that includes foods with a high protein content.'),
  'D000073601': NtO(MH='Dietary Approaches To Stop Hypertension', MS='Dietary recommendations that promote reduction in or prevention of high blood pressure. Recommendations include increasing intake of fruits and vegetables, and high-fiber, low-fat foods and reducing the intake of DIETARY SODIUM and high fat foods.'),
  'D000073602': NtO(MH='Antimicrobial Stewardship', MS='Programs and guidelines for selecting optimal ANTI-INFECTIVE AGENTS regimens in an effort to maintain antibiotic efficacy, reduce CROSS INFECTION related to ANTIBIOTIC RESISTANCE while managing satisfactory clinical and economic outcomes.'),
  'D000073605': NtO(MH='Spotted Fever Group Rickettsiosis', MS='A group of arthropod-borne diseases caused by spotted fever bio-group members of RICKETTSIA. They are characterized by fever, headache, and petechial (spotted) rash.'),
  'D000073617': NtO(MH='Transfusion-Related Acute Lung Injury', MS='A rare but serious transfusion-related reaction in which fluid builds up in the lungs unrelated to excessively high infusion rate and/or volume (TRANSFUSION-ASSOCIATED CIRCULATORY OVERLOAD). Signs of Transfusion-Related Acute Lung Injury include pulmonary secretions; hypotension; fever; DYSPNEA; TACHYPNEA; TACHYCARDIA; and CYANOSIS.'),
  'D000073618': NtO(MH='Varicella Zoster Virus Infection', MS='Infection caused by HUMAN HERPES VIRUS 3 (e.g., CHICKENPOX and HERPES ZOSTER).'),
  'D000073623': NtO(MH='Pre-Analytical Phase', MS='Laboratory processes prior to specimen analysis. These processes include study design, compliance of the subjects investigated, compliance in adherence to protocols, choice of specimens utilized and sample collection.'),
  'D000073624': NtO(MH='Symbolic Interactionism', MS='Theoretical approach to social processes with a fundamental concern with power phenomena. There is a focus on the analysis of culture, the influence of Foucault, and the development of feminist perspectives.'),
  'D000073637': NtO(MH='Oligodendrocyte Precursor Cells', MS='Neuroglial cells that first appear during mid-embryogenesis in the central nervous system of mammals and give rise to OLIGODENDROCYTES. Mitotically-active populations remain through late adulthood and are capable of regenerating MYELIN lost to disease or injury.'),
  'D000073638': NtO(MH='Alphacoronavirus', MS='A genus of the family CORONAVIRIDAE which causes respiratory or gastrointestinal disease in a variety of mammals. Human alphacoronaviruses include Human coronavirus 229E and NL63. All members share the identical core transcription regulatory sequences of 5\xe2\x80\x99-CUAAAC-3\xe2\x80\x99 and most have 1 to 2 ORFs downstream to the N protein gene.'),
  'D000073639': NtO(MH='Alphacoronavirus 1', MS='The type species of ALPHACORONAVIRUS genus causing gastroenteritis, peritonitis and respiratory diseases in dogs, cats and swine. Previously separate species TRANSMISSIBLE GASTROENTERITIS VIRUS; PORCINE RESPIRATORY CORONAVIRUS; CANINE CORONAVIRUS AND FELINE CORONAVIRUS merged into this species on the basis of similar genome nucleotide sequence and genome organization.'),
  'D000073640': NtO(MH='Betacoronavirus', MS='A genus of the family CORONAVIRIDAE which causes respiratory or gastrointestinal disease in a variety of mostly mammals. Human betacoronaviruses include HUMAN ENTERIC CORONAVIRUS; HUMAN CORONAVIRUS OC43; MERS VIRUS; and SARS VIRUS. Members have either core transcription regulatory sequences of 5\xe2\x80\x99-CUAAAC-3\xe2\x80\x99 or 5\xe2\x80\x99-CUAAAC-3\xe2\x80\x99 and mostly have no ORF downstream to the N protein gene.'),
  'D000073641': NtO(MH='Betacoronavirus 1', MS='The type species of BETACORONAVIRUS genus causing gastroenteritis respiratory diseases in mammals. Previously separate species HUMAN CORONAVIRUS OC43; BOVINE CORONAVIRUS; Human enteric coronavirus; Equine coronavirus; and Porcine hemagglutinating encephalomyelitis virus merged into this species on the basis of similar genome nucleotide sequence and genome organization.'),
  'D000073642': NtO(MH='Gammacoronavirus', MS='A genus of the family CORONAVIRIDAE that causes respiratory or gastrointestinal disease in avian species (or birds). The type species, AVIAN CORONAVIRUS, includes the previously separate species TURKEY CORONAVIRUS; and INFECTIOUS BRONCHITIS VIRUS.'),
  'D000073656': NtO(MH='Healthy Aging', MS='The development and maintenance of optimal physical, mental, spiritual, and social well-being and function with advancing age.'),
  'D000073658': NtO(MH='Loss of Function Mutation', MS="A mutation that causes a decrease in or elimination of a gene product's activity."),
  'D000073659': NtO(MH='Gain of Function Mutation', MS="A mutation that results in an increase in a gene's activity or in acquiring a new molecular function or a new pattern of gene expression."),
  'D000073678': NtO(MH='Water Sports', MS='Activities or games performed in a body of water.'),
  'D000073718': NtO(MH='Pancreatitis-Associated Proteins', MS='C-type lectins that restrict growth of bacteria in the intestinal epithelia and have bactericidal activity against gram-positive and gram-negative bacteria. They also regulate proliferation and differentiation of KERATINOCYTES following injury. Human pancreatitis-associated protein-1 (Reg3a) is overexpressed by pancreatic ACINAR CELLS in patients with CHRONIC PANCREATITIS. It is also highly expressed by pancreatic, bladder, and gastrointestinal cancer cells and may serve as a diagnostic biomarker.'),
  'D000073738': NtO(MH='2-Hydroxypropyl-beta-cyclodextrin', MS='Derivative of beta-cyclodextrin that is used as an excipient for steroid drugs and as a lipid chelator.'),
  'D000073739': NtO(MH='Strobilurins', MS='Benzene or pyrimidine derivatives of methacrylic acid that block the Qo site of CYTOCHROME B. They are widely used as agricultural fungicides.'),
  'D000073758': NtO(MH='Nonlinear Optical Microscopy', MS='Microscopic imaging techniques that utilize nonlinear responses of light-matter interactions which occur with high-intensity illumination, such as from LASERS, and specialized light signal detection instrumentation to produce images without the need for dyes or fluorescent labels.'),
  'D000073779': NtO(MH='P-type ATPases', MS='A highly conserved family of ATPases that facilitate the transport of lipids and cations across the plasma membrane. Structurally, they are elongated ALPHA-HELICES constituting five functionally distinct domains: three cytoplasmic domains A, N, and P which contain the catalytic sites, and two transmembrane domains. The N domain phosphorylates the P-domain at an invariant ASPARTATE residue, which, in turn, is dephosphorylated by the A domain.  The phosphorylation and dephosphorylation cycles drive conformational changes in the protein between two states (E1 and E2), which allow the substrate to access the other side of the membrane.'),
  'D000073798': NtO(MH='Second Harmonic Generation Microscopy', MS='A microscopic imaging technique that takes advantage of the process of harmonic generation that occurs when photons interact to generate new photons of a different wavelength. In second harmonic generation, two photons of the same wavelength and frequency, such as from a LASER, interact inside a medium and are converted to a photon of twice the frequency and half of the wavelength of the two incident photons. The light signals captured are used to produce images that are dependent on the unique optical properties of the material.'),
  'D000073818': NtO(MH='Pain, Procedural', MS='Pain associated with examination, treatment or procedures.'),
  'D000073819': NtO(MH='Self-Directed Learning as Topic', MS='Process in which individuals take the initiative, in diagnosing their learning needs, formulating learning goals, identifying resources for learning, choosing and implementing learning strategies and evaluating learning outcomes (Knowles, 1975)'),
  'D000073820': NtO(MH='Scholarly Communication', MS='System through which research and other scholarly writings are created, evaluated for quality, disseminated to the scholarly community, and preserved for future use. It includes both formal means of communication, such as publication in peer-reviewed journals, and informal channels, such as electronic listservs. (from Association of College & Research Libraries, \xe2\x80\x9cPrinciples and Strategies for the Reform of Scholarly Communication 1,\xe2\x80\x9d 2003)'),
  'D000073839': NtO(MH='Trisomy 13 Syndrome', MS='A chromosome disorder associated with TRISOMY of all or part of CHROMOSOME 13. Clinical manifestations include CONGENITAL HEART DEFECTS (e.g., PATENT DUCTUS ARTERIOSUS), facial malformations (e.g., CLEFT LIP; CLEFT PALATE; COLOBOMA; MICROPHTHALMIA); HYPOTONIA, digit malformations (e.g., POLYDACTYLY or SYNDACTYLY), and SEIZURES and severe INTELLECTUAL DISABILITY associated with NERVOUS SYSTEM MALFORMATIONS.'),
  'D000073840': NtO(MH='Copper-Transporting ATPases', MS='P-type ATPases which transport copper ions across membranes in prokaryotic and eukaryotic cells. They possess a conserved CYSTEINE-HISTIDINE-SERINE (CPx) amino acid motif within their transmembrane helices that functions in cation translocation and catalytic activation, and an N-terminal copper-binding CxxC motif that regulates enzyme activity. They play essential roles in intracellular copper homeostasis through regulating the uptake, efflux and storage of copper ions, and in cuproprotein biosynthesis.'),
  'D000073842': NtO(MH='Trisomy 18 Syndrome', MS='A chromosome disorder associated with TRISOMY of all or part of CHROMOSOME 18. Clinical manifestations include INTRAUTERINE GROWTH RETARDATION; CLEFT PALATE; CONGENITAL HEART DEFECTS; MICROCEPHALY; MICROGNATHIA and clenched fists with overlapping fingers. Most affected fetuses do not survive to birth. Those who survive through their first year often have severe INTELLECTUAL DISABILITY.'),
  'D000073843': NtO(MH='Equivalence Trial', MS='Trial that aims to show a new treatment is no better and no worse than the standard treatment.'),
  'D000073846': NtO(MH='Tobacco, Waterpipe', MS='Flavored tobacco whose vapor or smoke is passed through a water basin before inhalation.'),
  'D000073847': NtO(MH='Smoking, Non-Tobacco Products', MS='SMOKING of non-TOBACCO (or NICOTINE-containing) substances.'),
  'D000073859': NtO(MH='Survivorship', MS='Functional, psychosocial, emotional and spiritual domains and needs of patients and families following life-threatening disease or events.'),
  'D000073860': NtO(MH='Immune Privilege', MS='Phenomenon which occurs in certain tissue sites and organs (e.g., the ANTERIOR CHAMBER and CORNEA of the eye, brain PARENCHYMAL TISSUE and fetus) to tolerate a known antigen, thereby suppressing the inflammatory immune response. Foreign tissue grafts survive for prolonged periods when placed within such immune privileged sites and organs.'),
  'D000073861': NtO(MH='Apelin', MS='A 77 amino acid secreted endogenous ligand for the angiotensin II receptor-like 1 protein (APELIN RECEPTOR) that is proteolytically cleaved into four smaller peptides: Apelin-36, Apelin-31, Apelin-28, and Apelin-13. It inhibits entry of HIV into cells that express both APJ and CD4 ANTIGEN and is highly expressed in breast milk, where it may modulate the neonatal immune response.'),
  'D000073863': NtO(MH='Cigar Smoking', MS='The SMOKING of CIGARS.'),
  'D000073864': NtO(MH='Histone Deacetylase 6', MS='A class II histone deacetylase that removes acetyl groups from N-terminal LYSINES of HISTONE H2A; HISTONE H2B; HISTONE H3; and HISTONE H4. It plays a critical role in EPIGENETIC REPRESSION and regulation of GENETIC TRANSCRIPTION, as well as CELL MOTILITY through deacetylation of TUBULIN. It also targets misfolded proteins for clearance by AUTOPHAGY when MOLECULAR CHAPERONE-mediated folding is overwhelmed.'),
  'D000073865': NtO(MH='Cigarette Smoking', MS='The SMOKING of CIGARETTES.'),
  'D000073867': NtO(MH='Water Pipe Smoking', MS='A process by which SMOKE produced from burning substance is mixed with flavor-altering substances and drawn through water or other liquid into the mouth.'),
  'D000073868': NtO(MH='Pipe Smoking', MS='SMOKING by use of a narrow conveying tube which feeds from an open cavity where smoked product is loaded and burned.'),
  'D000073869': NtO(MH='Tobacco Smoking', MS='The process of SMOKING specific to TOBACCO.'),
  'D000073871': NtO(MH='Triggering Receptor Expressed on Myeloid Cells-1', MS='An approximately 230 amino acid membrane glycoprotein characterized by an IMMUNOGLOBULIN V-SET DOMAIN in its N-terminal half. It is expressed by MONOCYTES and NEUTROPHILS in response to INFLAMMATION related to bacterial and fungal infections. It triggers the release of pro-inflammatory CHEMOKINES; CYTOKINES, and expression of cell activation markers and is a critical regulator of SEPTIC SHOCK.'),
  'D000073872': NtO(MH='Vascular Ring', MS='Congenital vascular malformation in which the AORTA arch and its branches encircle the TRACHEA and ESOPHAGUS. Signs and symptoms include DYSPNEA; RESPIRATORY SOUNDS, especially with eating, DYSPHAGIA, persistent cough, and GASTROESOPHAGEAL REFLUX or may be asymptomatic. Two most common types are double aortic arch and right aortic arch. It may be associated with other anomalies (e.g., DIGEORGE SYNDROME).'),
  'D000073874': NtO(MH='ATP Binding Cassette Transporter, Subfamily A', MS='A large, highly conserved, subfamily of ATP binding cassette transporters structurally characterized by a membrane-spanning domain composed of six ALPHA-HELICES, a large extracellular loop, nucleotide-binding domain, and a conserved cytoplasmic 80 amino acid sequence. In humans, it includes ABCA1(ATP BINDING CASSETTE TRANSPORTER 1) through ABCA10, as well as ABCA12 and ABCA13.'),
  'D000073878': NtO(MH='Palm Oil', MS='Nutritive oil extracted from the fleshy mesocarp of the fruit of the African palm tree, Elaeis guineensis.'),
  'D000073879': NtO(MH='Rice Bran Oil', MS='OIL extracted from the hard outer brown layer of rice bran after the husk has been removed.'),
  'D000073880': NtO(MH='Body Contouring', MS='Plastic surgery to shape many areas of the body simultaneously. This may include LIPECTOMY or RHYTIDOPLASTY or both.'),
  'D000073882': NtO(MH='ATP Binding Cassette Transporter, Subfamily A, Member 4', MS='An ATP binding cassette sub-family A transporter that translocates 11-cis and all-trans isomers of N-retinylidene-phosphatidylethanolamine (RETINOIDS) from the extracellular surface to the cytoplasmic membrane surface of RETINAL ROD CELLS and RETINAL CONE CELLS. Mutations in the ABCA4 gene are associated with Stargardt Disease 1, a hereditary juvenile form of MACULAR DEGENERATION.'),
  'D000073883': NtO(MH='CX3C Chemokine Receptor 1', MS='Receptor for CHEMOKINE CX3CL1 expressed by lymphocytes, neurons, and GLIAL CELLS. Its interaction with CX3CL1 mediates CELL ADHESION and CELL MIGRATION. It also functions as a co-receptor with the CD4 ANTIGEN for HIV-1 in vitro.'),
  'D000073885': NtO(MH='C9orf72 Protein', MS='A widely-expressed protein of approximately 400 to 500 amino acids. Its N-terminal region (DENN domain) interacts with RAB GTP-BINDING PROTEINS and may regulate AUTOPHAGY, as well as PROTEIN TRANSPORT to ENDOSOMES. Expansion of the GGGGCC hexanucleotide repeat in the first intron of the C9orf72 gene is associated with FRONTOTEMPORAL DEMENTIA with AMYOTROPHIC LATERAL SCLEROSIS (FTDALS1).'),
  'D000073886': NtO(MH='Scientific Experimental Error', MS='Deviation or aberration in the practical implementation of standard empirical procedures, distinct from MEDICAL ERRORS and SCIENTIFIC MISCONDUCT.'),
  'D000073887': NtO(MH='Vaccination Coverage', MS='Rate of VACCINATION as defined by GEOGRAPHY and or DEMOGRAPHY.'),
  'D000073888': NtO(MH='Cell-Free Nucleic Acids', MS='Nucleic acids (DNA or RNA) found circulating in SERUM; PLASMA; or other BODY FLUIDS.'),
  'D000073890': NtO(MH='Liquid Biopsy', MS='Obtaining material for pathological examination and analysis, from bodily fluids. Material retrieved includes CELL-FREE NUCLEIC ACIDS; CELL-DERIVED MICROPARTICLES; EXOSOMES; CIRCULATING NEOPLASM CELLS; and other circulating cells and CELLULAR STRUCTURES.'),
  'D000073892': NtO(MH='Health Information Interoperability', MS='Automatic and seamless exchange or cross-talk across HEALTH INFORMATION SYSTEMS.'),
  'D000073893': NtO(MH='Sugars', MS='Short chain carbohydrate molecules that have hydroxyl groups attached to each carbon atom unit with the exception of one carbon that has a doubly-bond aldehyde or ketone oxygen. Cyclical sugar molecules are formed when the aldehyde or ketone groups respectively form a hemiacetal or hemiketal bond with one of the hydroxyl carbons. The three dimensional structure of the sugar molecule occurs in a vast array of biological and synthetic classes of specialized compounds including AMINO SUGARS; CARBASUGARS; DEOXY SUGARS; SUGAR ACIDS; SUGAR ALCOHOLS; and SUGAR PHOSPHATES.'),
  'D000073919': NtO(MH='Interdisciplinary Placement', MS='Teaching strategy of shared learning based cross-discipline experiences and placements.'),
  'D000073920': NtO(MH='Cryobiology', MS='The study of biological materials or systems subjected to temperatures below their normal range.'),
  'D000073921': NtO(MH='Child, Foster', MS='Individual who is unable to live safely with his or her family, usually due to abuse or neglect in the family home.'),
  'D000073922': NtO(MH='Child, Adopted', MS='Individual who becomes a child of a family by means of legal action.'),
  'D000073923': NtO(MH='Food Intolerance', MS='Digestive system disorder where a particular food irritates the digestive tract or cannot be properly digested (i.e., due to a lack of a digestive enzyme). It differs from FOOD HYPERSENSITIVITY which is an immune system disorder, usually due to specific proteins in food.  http://my.clevelandclinic.org/health/articles/problem-foods-is-it-an-allergy-or-intolerance.'),
  'D000073924': NtO(MH='Graphic Novels as Topic', MS='Works about book-length narratives told using a combination of words and sequential art, often presented in comic book style.'),
  'D000073930': NtO(MH='Biological Coevolution', MS='The process of reciprocal evolutionary change occurring between pairs of species or among groups of species as they interact.'),
  'D000073931': NtO(MH='Adverse Outcome Pathways', MS='Models connecting initiating events at the cellular and molecular level to population-wide impacts. Computational models may be at levels relating toxicology to adverse effects.'),
  'D000073932': NtO(MH='Food Addiction', MS='A cluster of chemical dependencies to specific foods or food in general in which there develops a physical craving for these foods.'),
  'D000073935': NtO(MH='Narrative Medicine', MS="NARRATION as a tool to collect and interpret information on a patient's experience of illness"),
  'D000073938': NtO(MH='Risk Evaluation and Mitigation', MS='Strategies required by the US Food and Drug Administration (FDA) Amendments Act of 2007 when a question exists as to whether the benefits of a drug outweigh its risks. These constitute a safety plan with several potential components, including a medication guide, a communication plan, elements to ensure safe use and an implementation system to help guide the prescribers, pharmacists and patients.'),
  'D000073939': NtO(MH='Dysferlin', MS='A membrane protein that contains multiple C2 DOMAINS. It is highly expressed in skeletal muscle and functions as a calcium ion sensor in SYNAPTIC VESICLE-PLASMA MEMBRANE fusion, as well as in SARCOLEMMA repair following mechanical stress. Mutations in the dysferlin (DYSF) gene are associated with several hereditary MUSCULAR DYSTROPHIES.'),
  'D000073940': NtO(MH='Transcription Factor 4', MS="A basic helix-loop-helix leucine zipper transcription factor that functions in neuronal CELL DIFFERENTIATION. It dimerizes with other bHLH transcription factors and activates transcription through binding to E-BOX ELEMENTS containing 5'-ACANNTGT-3' or 5'-CCANNTGG-3' sequences. Mutations in the TCF-4 gene are associated with Pitt-Hopkins Syndrome, a severe developmental disorder."),
  'D000073942': NtO(MH='T-Cell Acute Lymphocytic Leukemia Protein 1', MS='A basic helix-loop-helix transcription factor that plays a critical role in HEMATOPOIESIS and as a positive regulator in the differentiation of ERYTHROID CELLS. Chromosome translocations involving the TAL-1 gene are associated with T-CELL ACUTE LYMPHOCYTIC LEUKEMIA.'),
  'D000073943': NtO(MH='Neonicotinoids', MS='A class of insecticides that are structurally similar to NICOTINE and have physiologically similar effects as agonists of NICOTINIC ACETYLCHOLINE RECEPTORS, but are less toxic to vertebrates. They are widely used in agriculture.'),
  'D000073958': NtO(MH='Prescription Drug Monitoring Programs', MS='Programs, usually run by state governments, that require pharmacists to collect and distribute data on the prescription and dispensation of CONTROLLED SUBSTANCES. They are intended to prevent the abuse of such substances by the patient, or their transfer to recreational users and drug dealers.'),
  'D000073978': NtO(MH='Academic Performance', MS='A quantitative or qualitative measure of intellectual, scholarly, or scholastic accomplishment.'),
  'D000073979': NtO(MH='F-Box-WD Repeat-Containing Protein 7', MS='A component of SCF(FBW7) UBIQUITIN LIGASE that contains an F-box motif and multiple WD REPEATS. It recognizes and binds phosphorylated signals in several proteins involved in CELL PROLIFERATION and targets them to the SCF complex for UBIQUITINATION. Targets include CYCLIN E; PROTO-ONCOGENE PROTEINS C-JUN; PROTO-ONCOGENE PROTEINS C-MYC; and JNK MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D000073999': NtO(MH='Triose Sugar Alcohols', MS='Sugar alcohol molecules that contain three carbons.'),
  'D000074': NtO(MH='Acenocoumarol', MS='A coumarin that is used as an anticoagulant. Its actions and uses are similar to those of WARFARIN. (From Martindale, The Extra Pharmacopoeia, 30th ed, p233)'),
  'D000074001': NtO(MH='Intercellular Adhesion Molecule-3', MS='A membrane glycoprotein and cell adhesion molecule expressed by LEUKOCYTES that contains multiple Ig-like domains. It is a ligand for LFA-1 (integrin alphaLbeta2) and integrin alpha-D/beta-2. Its interaction with LFA-1 may play a role in the PHAGOCYTOSIS of NEUTROPHILS by MACROPHAGES following APOPTOSIS.'),
  'D000074002': NtO(MH='DNA Polymerase gamma', MS='A DNA-directed DNA polymerase that functions in the replication of MITOCHONDRIAL DNA. Mutations in the gene that encodes this enzyme (POLG) are associated with some forms of OPHTHALMOPLEGIA, CHRONIC EXTERNAL PROGRESSIVE.'),
  'D000074008': NtO(MH='MDS1 and EVI1 Complex Locus Protein', MS='A DNA binding protein, transcriptional regulator, and proto-oncogene protein that contains 10 CYS2-HIS2 ZINC FINGERS. It functions as a positive or negative regulator of expression for target genes involved in organism development.'),
  'D000074009': NtO(MH='Tubular Sweat Gland Adenomas', MS='Various tubular forms of benign tumors of the SWEAT GLAND with glandular differentiation. Common types include syringocystadenoma papilliferum of the head and neck, and hidradenoma papilliferum of the vulva area. Hidradenoma papilliferum may be derived from mammary-like glands of the vulva.'),
  'D000074010': NtO(MH='Bone Marrow Stromal Antigen 2', MS='A GPI-linked membrane glycoprotein and antiviral factor that functions as a homodimer. It is induced by INTERFERON-GAMMA and blocks the release of enveloped mammalian viruses by directly tethering nascent VIRAL PARTICLES to the membranes of infected cells.'),
  'D000074011': NtO(MH='Peptide Transporter 1', MS='A proton-coupled symporter that transports OLIGOPEPTIDES and DIPEPTIDES. It localizes to the brush-border membrane of the INTESTINAL EPITHELIUM and plays a critical role in the assimilation of dietary proteins.'),
  'D000074018': NtO(MH='B-Cell CLL-Lymphoma 10 Protein', MS='A signal transducing adaptor protein that contains an N-terminal CARD DOMAIN and functions in the ADAPTIVE IMMUNE RESPONSE. It promotes PRO-CASPASE-9 maturation and APOPTOSIS, activation of NF-KAPPA B, and is a substrate for MALT1 PARACASPASE.'),
  'D000074019': NtO(MH='T-Cell Intracellular Antigen-1', MS="An RNA-binding protein characterized by three RNA RECOGNITION MOTIFS. It binds to AU RICH ELEMENTS in the 3'-untranslated regions of mRNA and regulates alternative pre-RNA splicing and mRNA translation; it may also function in APOPTOSIS. Mutations in the TIA-1 gene are associated with WELANDER DISTAL MYOPATHY."),
  'D000074020': NtO(MH='ATP Binding Cassette Transporter, Subfamily B, Member 11', MS='ATP-binding cassette, subfamily B, protein (P-glycoproteins) that functions in the ATP-dependent secretion of BILE SALTS into the BILE CANALICULI of HEPATOCYTES. Mutations in the ABCB11 gene are associated with progressive familial intrahepatic cholestasis 2 (see CHOLESTASIS, INTRAHEPATIC).'),
  'D000074021': NtO(MH='Interatrial Block', MS="Impaired or delayed impulse conduction between the right and left HEART ATRIA. Advanced interatrial blocks are often associated with arrhythmias (e.g., ATRIAL FLUTTER; and ATRIAL FIBRILLATION), direct conduction block via the Bachmann's bundle and concomitant left atrial enlargement. Syndrome of advanced interatrial block associated with SUPRAVENTRICULAR TACHYCARDIA is referred to as Bayes syndrome."),
  'D000074022': NtO(MH='Anesthesia, Cardiac Procedures', MS='A range of methods used to induce UNCONSCIOUSNESS; ANALGESIA; and MUSCLE RELAXATION during cardiac procedures.'),
  'D000074024': NtO(MH='Immunoturbidimetry', MS='Immunochemical analysis which measures specific target antigen bound to antibody complex. Antibodies in assay form insoluble light scattering immune-aggregates which cause changes in the measured turbidity proportional to the concentration of the antigen.'),
  'D000074025': NtO(MH='TRPA1 Cation Channel', MS='A highly conserved, non-selective TRP cation channel  that contains 14-17 ANKYRIN REPEATS. It functions in cold sensation and NOCICEPTION of endogenous inflammatory factors and volatile irritants. TRPA1 is also activated by CANNABINOIDS and may play a role in sound perception by hair cells of the inner ear.'),
  'D000074026': NtO(MH='Fatty Acid Binding Protein 3', MS='A small cytosolic fatty-acid binding protein that forms a lipid-binding beta-barrel structure and is expressed by CARDIOMYOCYTES and at lower levels in brain tissue. It is released into plasma immediately following cardiac injury and may therefore serve as a useful biomarker for the early detection of MYOCARDIAL INFARCTION.'),
  'D000074039': NtO(MH='Staphylococcus capitis', MS='A COAGULASE-negative species of STAPHYLOCOCCUS found on the skin and MUCOUS MEMBRANE of warm-blooded animals. Similar to STAPHYLOCOCCUS EPIDERMIDIS and STAPHYLOCOCCUS HAEMOLYTICUS, it is a nosocomial pathogen in NICU settings. Subspecies include generally antibiotic susceptible and BIOFILM negative capitis and antibiotic resistant and biofilm positive urealyticus isolates.'),
  'D000074040': NtO(MH='Receptors, Histamine H4', MS='G-Protein-coupled histamine receptors that are expressed primarily in BONE MARROW as well as in peripheral tissues and organs. They are expressed by immune cells that include EOSINOPHILS; T-CELLS; DENDRITIC CELLS; and MAST CELLS.'),
  'D000074041': NtO(MH='Microorganisms, Genetically-Modified', MS='Microorganisms whose GENOME has been altered by GENETIC ENGINEERING.'),
  'D000074042': NtO(MH='Cerebral Intraventricular Hemorrhage', MS='Bleeding within the CEREBRAL VENTRICLES. It is associated with intraventricular trauma, aneurysm, vascular malformations, hypertension and in VERY LOW BIRTH WEIGHT infants.'),
  'D000074043': NtO(MH='Night Eating Syndrome', MS='Little or no appetite for breakfast due to eating more food after dinner than during the meal and eating more than half of daily food intake after dinner hour.'),
  'D000074049': NtO(MH='Cytokine TWEAK', MS='A proteolytically-cleaved membrane glycoprotein and member of the TNF superfamily that is highly expressed in a variety of tissues including heart, pancreas, brain, and peripheral blood lymphocytes. The secreted extracellular form is a weak inducer of APOPTOSIS for some cell types and a ligand for the FN14 RECEPTOR. It mediates activation of NF-KAPPA-B and promotes ANGIOGENESIS and proliferation of ENDOTHELIAL CELLS, as well as expression of cytokines involved in INFLAMMATION.'),
  'D000074050': NtO(MH='TWEAK Receptor', MS='A receptor for TWEAK cytokine that is highly expressed by cells in the heart, placenta, and kidney. It plays a role in ANGIOGENESIS and the proliferation of endothelial cells; it may also modulate cellular adhesion to the extracellular matrix.'),
  'D000074058': NtO(MH='Solute Carrier Family 22 Member 5', MS='A high-affinity, ATP-binding, co-transporter for CARNITINE that is highly expressed in kidney, skeletal muscle, heart, and placental tissues. It transports one sodium ion with one carnitine molecule. It has a lower affinity for other organic cations and transports them independently of sodium. Mutations in the SLC22A5 gene are associated with systemic carnitine deficiency.'),
  'D000074059': NtO(MH='Extracorporeal Shockwave Therapy', MS='A nonsurgical treatment that uses either HIGH-ENERGY SHOCK WAVES or low energy ACOUSTIC WAVES to treat various musculoskeletal conditions (e.g., PLANTAR FASCIITIS; TENNIS ELBOW). A probe placed on the skin conducts the shock waves thereby delivering a mechanical force to the body\xe2\x80\x99s tissues.'),
  'D000074060': NtO(MH='Bronchial Thermoplasty', MS='Thermal destruction of the excess bronchial SMOOTH MUSCLE tissue with heat delivered through a catheter assembly attached to a BRONCHOSCOPE. It is often used to control BRONCHIAL HYPERREACTIVITY in severe ASTHMA for better AIRWAY MANAGEMENT.'),
  'D000074061': NtO(MH='ATP Binding Cassette Transporter, Subfamily D', MS='A sub-family of ATP-binding cassette transporters that localize to the membranes of PEROXISOMES; ENDOPLASMIC RETICULUM; and LYSOSOMES. Members contain a transmembrane domain in their N-terminal half and generally function as homodimers in the transport of LIPIDS; BILE ACIDS; and VITAMIN B12. Mutations in some ABCD transporter genes are associated with PEROXISOMAL DISORDERS.'),
  'D000074062': NtO(MH='Environmental Biomarkers', MS='A factor associated with the well-being of living organisms that is used as a measure of environmental change and or influence. For example, aldehyde dehydrogenase expression in earthworm tissue is used as an indication of heavy metal pollution in soils. Distinguish from BIOMARKERS.'),
  'D000074063': NtO(MH='ATP Binding Cassette Transporter, Subfamily D, Member 1', MS='ATP-binding cassette transporter that functions in the import of long chain (13-21 carbons) and very long chain fatty acids (> 22 carbons), or their acyl-CoA-derivatives, into PEROXISOMES. Mutations in the ABCD1 gene are associated with the X-linked form of ADRENOLEUKODYSTROPHY.'),
  'D000074067': NtO(MH='Conservation of Water Resources', MS='Preservation and or management of WATER RESOURCES especially under conditions of scarce supply.'),
  'D000074079': NtO(MH='Undifferentiated Connective Tissue Diseases', MS='Diseases that exhibit signs and symptoms suggestive of a connective tissue disease that do not fulfill clinical or diagnostic criteria for any one defined disease but overlap with criteria of multiple such diseases. Commonly overlapping diseases include systemic autoimmune connective tissue diseases such as RHEUMATOID ARTHRITIS; SYSTEMIC LUPUS ERYTHEMATOSUS; and SYSTEMIC SCLEROSIS.'),
  'D000074080': NtO(MH='MutS Proteins', MS='DNA repair proteins that include the bacterial MutS DNA mismatch-binding protein and its eukaryotic homologs that function in DNA MISMATCH REPAIR and recombination of DNA during MEIOSIS. MutS has a conserved mismatch recognition domain characterized by GxFxE, or similar AMINO ACID MOTIFS that also occur in eukaryotic homologs such as MSH1, MSH6, and MSH8. All MutS proteins also contain a highly-conserved ATP-binding domain and most have weak ATPase activity.'),
  'D000074081': NtO(MH='MutS Homolog 3 Protein', MS='A MutS homolog protein and component of post-replicative DNA MISMATCH REPAIR. It forms a heterodimer with MUTS HOMOLOG 2 PROTEIN (MSH2) and recognizes large insertion-deletion loops up to 13 nucleotides in length. This directs downstream events such as strand discrimination, excision, and resynthesis.'),
  'D000074082': NtO(MH='Sodium-Hydrogen Exchanger 1', MS='A sodium-hydrogen antiporter expressed by many cell types, especially on the basolateral surfaces of EPITHELIAL CELLS. It functions through an inward sodium ion chemical gradient to eliminate acids (protons) generated by metabolism and regulate intracellular pH. It is highly sensitive to AMILORIDE.'),
  'D000074083': NtO(MH='Sodium-Hydrogen Exchanger 3', MS='A sodium-hydrogen antiporter expressed primarily by EPITHELIAL CELLS in the kidneys, it localizes to the apical membrane of the PROXIMAL KIDNEY TUBULE, where it functions in sodium and water reabsorption and possibly calcium homeostasis.  It also is expressed in heart, brain, and lung tissues and is resistant to AMILORIDE inhibition.'),
  'D000074084': NtO(MH='THP-1 Cells', MS='A human leukemia monocytic cell line derived from a patient with LEUKEMIA, MONOCYTIC, ACUTE. It is used as a model to study the function of MONOCYTES and MACROPHAGES, their signaling pathways, nutrient and drug transport.'),
  'D000074085': NtO(MH='Mice, Knockout, ApoE', MS='Strains of mice that contain genetic disruptions (knockout) of APOLIPOPROTEINS E genes. They are used as models for ATHEROSCLEROSIS research.'),
  'D000074099': NtO(MH='Equivalence Trials as Topic', MS='Works about trials that aim to show a new treatment is no better and no worse than the standard treatment.'),
  'D000074100': NtO(MH='Funeral Homes', MS='Facilities for the preparation of the dead for burial or cremation, for the viewing of the body, and for funeral services.'),
  'D000074101': NtO(MH='Cocaine Smoking', MS='SMOKING of COCAINE.'),
  'D000074121': NtO(MH='Protein Kinase C-theta', MS='A calcium-independent, phospholipid- and diacylglycerol-dependent, protein kinase C subtype that contains an N-terminal C2 DOMAIN and two diacylglycerol-binding ZINC FINGERS. It is expressed primarily by T-LYMPHOCYTES and localizes to IMMUNOLOGICAL SYNAPSES where it regulates downstream signaling for the activation, proliferation, and survival of mature T-cells. It plays a critical role in allergic, autoimmune, and alloimmune responses of TH2 CELLS and TH17 CELLS.'),
  'D000074122': NtO(MH='Short Stature Homeobox Protein', MS='A homeodomain protein that is highly expressed in the nuclei of skeletal muscle, bone marrow, and osteogenic cells and has critical roles in growth and development. Its gene resides within PSEUDOAUTOSOMAL REGION 1 of X and Y chromosomes and mutations are associated with several growth disorders including LERI-WEIL SYNDROME; LANGER MESOMELIC DYSPLASIA; and SHORT STATURE, IDIOPATHIC, X-LINKED.'),
  'D000074141': NtO(MH='Circulating Tumor DNA', MS='DNA released from tumor cells that is found circulating in PLASMA; SERUM; or other BODY FLUIDS.'),
  'D000074161': NtO(MH='Kazal Motifs', MS='Highly conserved protein domains characteristic of SERINE PROTEASE INHIBITORS, KAZAL TYPE. They generally occur as tandem repeats, with each domain consisting of approximately 60 amino acids that form a large extended amino acid chain, two short ALPHA-HELICES, and a three-stranded anti-parallel BETA-SHEET. Kazal-like domains also occur in the extracellular portions of AGRIN.'),
  'D000074162': NtO(MH='Serine Peptidase Inhibitors, Kazal Type', MS='A family of serine peptidase inhibitors that occur in animals, some single-cell eukaryotes, and higher plants. They contain variable numbers of KAZAL MOTIFS and inhibit SERINE ENDOPEPTIDASES such as ACROSIN and TRYPSIN.'),
  'D000074163': NtO(MH='Serine Peptidase Inhibitor Kazal-Type 5', MS='A secreted serine peptidase inhibitor that consists of 15 KAZAL MOTIFS and inhibits KALLIKREINS KLK5, KLK7, and KLK14 as well as TRYPSIN. It plays important roles in protecting MUCOSA against infection and inflammation, and in maintaining the integrity of the skin. Mutations in the SPINK5 gene are associated with NETHERTON SYNDROME.'),
  'D000074164': NtO(MH='Nicotine Chewing Gum', MS='Chewing gum which contains NICOTINE.'),
  'D000074165': NtO(MH='Zinc Finger E-box Binding Homeobox 2', MS="A transcription factor that consists of 8 CYS2-HIS2 ZINC FINGERS flanking a central HOMEOBOX. It binds to the 5'-CACCT-3' DNA sequence located within E-BOX ELEMENTS of many genes essential for embryonic growth and development and regulates their activity; it represses transcription of the E-CADHERIN gene. Mutations in the ZEB2 gene are associated with MOWAT-WILSON SYNDROME."),
  'D000074166': NtO(MH='Circulating MicroRNA', MS='MicroRNAs found in cell-free BODY FLUIDS such as SERUM; PLASMA; SALIVA; OR URINE.'),
  'D000074181': NtO(MH='Spastin', MS='An AAA ATPase that binds and severs MICROTUBULES. It specifically recognizes and cuts polyglutamylated microtubules with short polyglutamate tails to promote reorganization of cellular microtubule arrays and the release of microtubules from the CENTROSOME following nucleation. It is critical for the biogenesis and maintenance of complex microtubule arrays in AXONS; SPINDLE APPARATUS; and CILIA. Mutations in the spastin gene (SPAST) are associated with type 4 of HEREDITARY SPASTIC PARAPLEGIA.'),
  'D000074182': NtO(MH='AAA Domain', MS='An approximately 250 amino acid domain common to AAA ATPases and AAA Proteins. It consists of a highly conserved N-terminal P-Loop ATPase subdomain with an alpha-beta-alpha conformation, and a less-conserved C-terminal subdomain with an all alpha conformation. The N-terminal subdomain includes Walker A and Walker B motifs which function in ATP binding and hydrolysis.'),
  'D000074183': NtO(MH='ATPases Associated with Diverse Cellular Activities', MS='A large highly-conserved family of ATPases with diverse functions in cells that are characterized by the presence of a P-LOOP and a ring shape. They couple the energy generated by ATP hydrolysis to remodeling or mechanical translocation of their target molecules.'),
  'D000074201': NtO(MH='Katanin', MS='An enzyme (AAA ATPase) consisting of the 60 kDa ATPase subunit (p60 subunit A1) which severs MICROTUBULES, and an 80 kDa accessory protein (p80 subunit B1), which targets the enzyme to the CENTROSOME. It releases microtubules from the mitotic SPINDLE POLES to allow depolymerization and poleward motion of chromosomes. It is also a regulator of microtubule dynamics in NEURONAL OUTGROWTH.'),
  'D000074221': NtO(MH='Rumination, Digestive', MS='Regurgitation and re-chewing of previously swallowed food in RUMINANTS.'),
  'D000074222': NtO(MH='Rumination, Cognitive', MS='Obsessive thinking about an idea, situation, or choice.'),
  'D000074241': NtO(MH='Peanut Oil', MS='Oil derived from PEANUTS.'),
  'D000074242': NtO(MH='Sunflower Oil', MS='Oil derived from the seeds of SUNFLOWER plant, Helianthus.'),
  'D000074243': NtO(MH='Immune Reconstitution', MS='Regeneration of normal immune function after immune depleting procedures or infections (e.g., HEMATOPOIETIC STEM CELL TRANSPLANTATION). Delayed and incomplete reconstitution of the ADAPTIVE IMMUNE system in particular involving T-CELLS is associated with increase or relapse of infection.'),
  'D000074261': NtO(MH='Receptors, Enterotoxin', MS='Guanylate cyclase-coupled receptors that bind bacterial ENTEROTOXINS, as well as the endogenous peptides guanylin and uroguanylin. Ligand binding stimulates production of CYCLIC GMP by EPITHELIAL CELLS of the intestinal lumen, altering barrier permeability and mucus secretion. Mutations in the gene (GUCY2C) encoding this receptor are associated with some cases of hereditary diarrhea (Diarrhea 6) and MECONIUM ILEUS.'),
  'D000074262': NtO(MH='Rapeseed Oil', MS='PLANT OILS derived from RAPESEED species known as BRASSICA RAPA.'),
  'D000074263': NtO(MH='Coconut Oil', MS='Oil derived from fruits of the coconut plant, COCOS NUCIFERA.'),
  'D000074264': NtO(MH='Smoking Reduction', MS='A decrease in the incidence and frequency of SMOKING. Smoking reduction differs from SMOKING CESSATION in that the smoker continues to smoke albeit at a lesser frequency without quitting.'),
  'D000074266': NtO(MH='Materials Science', MS='An interdisciplinary study of the microstructure and composition of various materials such as metals, semiconductors, ceramics, and polymers, in relation to their macromolecular physical and chemical properties. Materials science enables the custom creation of new materials with specific properties and uses.'),
  'D000074267': NtO(MH='Organism Hydration Status', MS='Quantitative measure of water or fluids contained in the body of a living organism.'),
  'D000074268': NtO(MH='Guanylyl Cyclase C Agonists', MS='Compunds that bind to and activate GUANYLYL CYCLASE-C RECEPTORS.'),
  'D000074269': NtO(MH='Resonance Frequency Analysis', MS='A non-invasive assessment of the stability of tissue-embedded prosthetic devices such as dental implants.'),
  'D000074270': NtO(MH='Meconium Ileus', MS='Intestinal obstruction caused by congealed MECONIUM in the distal ILEUM and CECUM. It presents shortly after birth as a failure to pass meconium and frequently occurs in infants with CYSTIC FIBROSIS.'),
  'D000074283': NtO(MH='Endothelial Protein C Receptor', MS='A glycosylated transmembrane receptor for PROTEIN C that is highly expressed by endothelial cells on the surface of large blood vessels in the heart and lungs.  It facilitates protein C activation by the THROMBIN and THROMBOMODULIN complex in blood coagulation.'),
  'D000074284': NtO(MH='Periphyton', MS='A complex mixture of organisms (algae, cyanobacteria, heterotrophic microbes, detritus, etc.) clinging on the surfaces of plants and other objects projecting from the bottom sediments of aquatic ecosystems.'),
  'D000074285': NtO(MH='Smokers', MS='Persons with a history or habit of SMOKING.'),
  'D000074286': NtO(MH='Preliminary Data', MS='First-released version of study results in a series of data collection efforts used for the purpose of generating further interest in and or funding of a research study.'),
  'D000074287': NtO(MH='TRPC6 Cation Channel', MS='A non-selective, calcium permeant TRPC cation channel that contains four ANKYRIN REPEATS and is activated by DIACYLGLYCEROL independently of PROTEIN KINASE C. It is expressed in placenta, lung, spleen, ovary and the small intestine, as well as by PODOCYTES in the kidney glomerulus. Mutations in the TRPC6 gene are associated with FOCAL SEGMENTAL GLOMERULOSCLEROSIS type 2.'),
  'D000074288': NtO(MH='Discs Large Homolog 1 Protein', MS='A signaling adaptor protein that contains three PDZ DOMAINS; an SH3 DOMAIN; and a GUANYLATE KINASE-like C-terminal region. It has an essential role in animal development and recruits ion channels, receptors, and signaling molecules to discrete plasma membrane regions of polarized cells in the HEART ATRIA MYOCARDIUM. It functions in the assembly of ADHERENS JUNCTIONS and regulation of CELL PROLIFERATION; synaptogenesis, LYMPHOCYTE ACTIVATION; and controls expression of KV4 POTASSIUM CHANNELS to regulate excitability of CARDIAC MYOCYTES.'),
  'D000074289': NtO(MH='High-Temperature Requirement A Serine Peptidase 1', MS='A secreted serine protease that contains a Kazal domain-like region and a C-terminal PDZ domain. It has a broad range of targets that include EXTRACELLULAR MATRIX PROTEINS; PROTEOGLYCANS; and INSULIN-LIKE GROWTH FACTOR BINDING PROTEINS. Mutations in the HTRA1 gene are associated with AGE-RELATED MACULAR DEGENERATION 7 and Cerebral Autosomal Recessive Arteriopathy with Subcortical Infarcts and Leukoencephalopathy (CARASIL).'),
  'D000074290': NtO(MH='Zinc Finger Protein Gli3', MS="A zinc finger transcription factor that contains five CYS2-HIS2 ZINC FINGERS and binds to the GLI consensus sequence 5'-GGGTGGTC-3'. The full-length protein functions as a transcriptional activator whereas the truncated C-terminal form functions as a transcriptional repressor of the Sonic Hedgehog (Shh) signaling pathway; a balance between these two forms is critical for limb and digit development. GLI3 also plays a critical role in the differentiation and proliferation of CHONDROCYTES."),
  'D000074301': NtO(MH='CD52 Antigen', MS='A small GPI-linked glycoprotein expressed on the surface of normal and malignant B-CELLS; T-CELLS; MONOCYTES; MACROPHAGES; NK CELLS; and GRANULOCYTES. It is expressed densely and without modulation in many malignant T-cell neoplasms and therefore a target for antibody therapies (e.g., ALEMTUZUMAB).'),
  'D000074322': NtO(MH='Antineoplastic Agents, Immunological', MS='Antineoplastic agents containing immunological agents (e.g. MAbs). These pharmacologic preparations inhibit or prevent the proliferation of NEOPLASMS.'),
  'D000074323': NtO(MH='Alemtuzumab', MS='An anti-CD52 ANTIGEN monoclonal antibody used for the treatment of certain types of CD52-positive lymphomas (e.g., CHRONIC LYMPHOCYTIC LEUKEMIA; CUTANEOUS T-CELL LYMPHOMA; and T-CELL LYMPHOMA). Its mode of actions include ANTIBODY-DEPENDENT CELL CYTOTOXICITY.'),
  'D000074324': NtO(MH='Ipilimumab', MS='An anti-CTLA-4 ANTIGEN monoclonal antibody initially indicated for the treatment of certain types of metastatic MELANOMA. Its mode of actions may include blocking of CTLA-4 mediated inhibition of CYTOTOXIC T LYMPHOCYTES, allowing for more efficient destruction of target tumor cells.'),
  'D000074341': NtO(MH='Dietary Exposure', MS='The exposure to potentially harmful factors such as trace heavy metals, chemicals, radiation, or toxins due to FOOD CONTAMINATION including DRINKING WATER contamination.'),
  'D000074361': NtO(MH='Sleep Latency', MS='The time it takes to reach REM SLEEP. It is typically measured by POLYSOMNOGRAPHY or EEG as a part of various sleep pattern tests (e.g., multiple sleep latency test).'),
  'D000074381': NtO(MH='Mass Drug Administration', MS='Administration of a medication to at-risk individuals in a population without individual diagnosis. It is often used in order to treat, control, and/or prevent spread of often endemic DISEASE OUTBREAKS such as NEGLECTED DISEASES in high disease burden areas.'),
  'D000074382': NtO(MH='Greenhouse Gases', MS='Gaseous elements, chemicals that are in the atmosphere that may contribute to GREENHOUSE EFFECT.'),
  'D000074384': NtO(MH='Sexual Health', MS='A state of physical, emotional, mental and social well-being in relation to SEXUALITY, according to the World Health Organization.'),
  'D000074385': NtO(MH='Food Ingredients', MS='Substances included in prepared foods and beverages.'),
  'D000074386': NtO(MH='Gender-Based Violence', MS='Violence based on gender that results in, or is likely to result in, physical, sexual or mental harm or suffering, including threats of such acts, coercion or arbitrary deprivation of liberty, whether occurring in public or in private life. (From www.who.int/topics/gender_based_violence/en/)'),
  'D000074388': NtO(MH='Early Goal-Directed Therapy', MS='Critical care treatment using intensive monitoring and aggressive management of perioperative hemodynamics in high risk patients,'),
  'D000074389': NtO(MH='Therapeutic Index, Drug', MS='The ratio of the dose that produces toxicity to the dose that produces a clinically desired or effective response.'),
  'D000074401': NtO(MH='Peroxins', MS='Proteins that are essential for the assembly of PEROXISOMES. They recognize and transport cytoplasmic proteins that contain PEROXISOMAL TARGETING SIGNALS (PTS) to the peroxisome. Mutations in peroxin (PEX) genes are associated with several PEROXISOMAL DISORDERS.'),
  'D000074402': NtO(MH='Synovectomy', MS='Surgical removal of the SYNOVIAL MEMBRANE.'),
  'D000074403': NtO(MH='Meniscectomy', MS='Surgical incision of a torn MENISCUS.'),
  'D000074404': NtO(MH='Peroxisomal Targeting Signals', MS='Protein sorting signals that target proteins to PEROXISOMES.'),
  'D000074405': NtO(MH='Valosin Containing Protein', MS='A highly-conserved AAA ATPase that functions in the biogenesis of the transitional ENDOPLASMIC RETICULUM and fragmentation and reassembly of the GOLGI APPARATUS during MITOSIS. It also functions in a complex with UFD1L and NPLOC4 proteins to export misfolded ubiquitinated proteins from the endoplasmic reticulum and outer mitochondrial membrane to the cytoplasm for degradation by the PROTEASOME and also plays a role in AUTOPHAGY of ubiquitinated proteins. It occurs in neuronal INCLUSION BODIES from patients with AMYOTROPHIC LATERAL SCLEROSIS and LEWY BODIES from PARKINSON DISEASE patients.'),
  'D000074406': NtO(MH='Mastoidectomy', MS='Surgical removal of the diseased cells from the MASTOID PROCESS. It often involves simultaneous resection and/or repair of the MIDDLE EAR and EAR DRUM in MIDDLE EAR CHOLESTEATOMAS and MASTOIDITIS.'),
  'D000074407': NtO(MH='SAP90-PSD95 Associated Proteins', MS='Proteins expressed at SYNAPSES throughout the brain where they interact with different scaffolding proteins, cytoskeletal proteins, and signaling factors to assemble functional multiprotein complexes.'),
  'D000074408': NtO(MH='Aldo-Keto Reductases', MS='A family of NADPH-dependent oxidoreductases that reduce carbonyl substrates including sugar-aldehydes, KETOSTEROIDS; keto-prostaglandins, and QUINONES. They are monomers of approximately 37 KDa and are characterized by a parallel beta-8 (BETA SHEET)-alpha 8 (ALPHA HELICES)-barrel structure that contains the NADP binding site. This conformation favors aromatic and apolar substrates.'),
  'D000074409': NtO(MH='Carbonyl Reductase (NADPH)', MS='NADPH-dependent reductase that catalyzes the reduction of many carbonyl compounds including QUINONES; PROSTAGLANDINS; and XENOBIOTICS.'),
  'D000074421': NtO(MH='Fermented Foods and Beverages', MS='Foods and beverages that are prepared by using microorganisms to convert their components into various FERMENTATION end products. Some pickled foods are considered fermented foods as their pickling results from the microbial production of LACTIC ACID.'),
  'D000074422': NtO(MH='Aldo-Keto Reductase Family 1 Member C2', MS='Aldo-keto reductase that functions with 5-alpha and 5-beta-steroid reductases to convert steroid hormones into the 3-alpha or 5-alpha and 3-alpha or 5-beta-tetrahydrosteroid, including 5-ALPHA-DIHYDROTESTOSTERONE  to ANDROSTANE-3,17-DIOL. It also binds to BILE ACIDS. Mutations in the AKR1C2 gene are associated with Type 8 DISORDER OF SEX DEVELOPMENT, 46,XY (SRXY8).'),
  'D000074423': NtO(MH='Celastrales', MS='A taxonomic order of plants within the class MAGNOLIOPSIDA, which includes the staff vine.'),
  'D000074424': NtO(MH='Rosanae', MS='A suborder of flowering plants of the class MAGNOLIOPSIDA which comprises two major groups, Eurosids I and II.'),
  'D000074425': NtO(MH='Aldo-Keto Reductase Family 1 Member C3', MS='Aldo-keto reductase that functions as a bi-directional  17 BETA, 20 ALPHA-HYDROXYSTEROID DEHYDROGENASE. It catalyzes the reduction of PROSTAGLANDIN D2 and PROSTAGLANDIN H2, as well as the oxidation of  9alpha,11beta-PGF2 to prostaglandin D2. It can also interconvert estrogens, ANDROGENS; and PROGESTINS between their active forms and inactive metabolites.'),
  'D000074426': NtO(MH='Aldo-Keto Reductase Family 1 member B10', MS='Aldo-keto reductase that functions as an all-trans-retinaldehyde reductase. It also reduces aromatic and aliphatic ALDEHYDES.'),
  'D000074427': NtO(MH='Caryophyllanae', MS='Superorder of dicot plants in the class MAGNOLIOPSIDA.'),
  'D000074428': NtO(MH='Peroxisomal Biogenesis Factor 2', MS='A multi-pass transmembrane protein that contains a C-terminal RING finger domain. It localizes to the PEROXISOME membrane and is essential for peroxisome biogenesis. Mutations in the PEX2 gene are associated with ZELLWEGER SYNDROME and INFANTILE REFSUM DISEASE.'),
  'D000074429': NtO(MH='Candida parapsilosis', MS='A species of MITOSPORIC FUNGI and opportunistic pathogen associated with its ability to form BIOFILMS in catheters and parenteral nutrition IV lines. C. parapsilosis complex includes closely related species C. orthopsilosis; and C. metapsilosis.'),
  'D000074431': NtO(MH='Keratectomy', MS='Surgical excision of a part of the CORNEA.'),
  'D000074432': NtO(MH='Sphincterotomy', MS='Surgical incision of a sphincter.'),
  'D000074433': NtO(MH='Myotomy', MS='Surgical incision of the muscle.'),
  'D000074434': NtO(MH='Heller Myotomy', MS='Surgical incision of the lower esophageal sphincter near the CARDIA often used to treat ESOPHAGEAL ACHALASIA.'),
  'D000074435': NtO(MH='Peroxisome-Targeting Signal 1 Receptor', MS='A cytoplasmic receptor and peroxin that contains a series of TETRATRICOPEPTIDE REPEATS and binds to PEROXISOME TARGETING SIGNAL 1 (SKL-type). It is essential for protein import into PEROXISOMES; mutations in the PEX5 gene are associated with PEROXISOMAL DISORDERS such as ZELLWEGER SYNDROME.'),
  'D000074436': NtO(MH='Tetratricopeptide Repeat', MS='A structural motif present in a variety of proteins that mediates protein-protein interactions and assembly of multiprotein complexes. It consists of 3 to 16 tandem repeats of 34 amino acids, mostly of small and large hydrophobic residues, that form ALPHA-HELIX pairs.'),
  'D000074437': NtO(MH='Peroxisomal Targeting Signal 2 Receptor', MS='A cytoplasmic receptor and peroxin that contains a series of WD40 REPEATS and binds to PEROXISOME TARGETING SIGNAL 2. It is essential for protein import into PEROXISOMES; mutations in the human PEX7 gene are associated with PEROXISOMAL DISORDERS such as Type 1 CHONDRODYSPLASIA PUNCTATA, RHIZOMELIC.'),
  'D000074439': NtO(MH='Political Activism', MS='Active involvement in the political process including promoting, impeding or raising awareness of a certain issue or set of issues.'),
  'D000074440': NtO(MH='Fascism', MS='Political movement which combines nationalism with demands for political and social renewal. Characteristics include militaristic nationalism, belief in a natural social hierarchy and the rule of elites, and the desire to create a \xe2\x80\x9cpeople\xe2\x80\x99s community\xe2\x80\x9d, in which individual interests would be subordinated to the good of the nation. (From www.britannica.com/topic/fascism)'),
  'D000074441': NtO(MH='Involuntary Treatment', MS='Procedures, surgery, or other treatment without consent of person or persons receiving treatment.'),
  'D000074443': NtO(MH='Lysine Acetyltransferase 5', MS='A catalytic subunit of the NuA4 histone acetyltransferase complex that functions in transcriptional activation of genes by acetylation of nucleosomal HISTONES H4 and H2A, altering nucleosome-DNA interactions and interaction of the modified histones with other activating transcription factors. It may control gene expression changes associated with oncogene and proto-oncogene mediated growth induction, tumor suppressor mediated growth arrest; CELL AGING; APOPTOSIS; and DNA REPAIR. It is polyubiquitinated and degraded during HIV-1 infection through its interaction with the viral TAT PROTEIN.'),
  'D000074462': NtO(MH='Positive Regulatory Domain I-Binding Factor 1', MS='A transcriptional repressor protein that contains an N-terminal PR-SET domain, four C-terminal CYS2-HIS2 ZINC FINGERS, and binds the PRDI element in the INTERFERON-BETA gene. It has methyltransferase activity and mediates gene transcription in tissue-specific innate and adaptive immune lymphocyte T-CELLS, repressing expression of proteins that promote exit of these tissue-specific T-cell populations from non-lymphoid organs.'),
  'D000074463': NtO(MH='PR-SET Domains', MS='Highly conserved protein domains of approximately 130 to 140 amino acids. The SET domain was first identified in the Drosophila proteins (S)u(var)3-9, (E)nhancer-of-zeste and (T)rithorax and occurs in other proteins with a variety of functions, including histone-lysine N-methyltransferases. Structurally, it consists of BETA-SHEETS interspersed among loops and turns that result in an "L" shape. The most conserved motifs are a stretch at the C-terminal that contains a strictly conserved tyrosine residue and an adjacent loop that the C-terminal segment passes through to form a "knot". These motifs and especially the tyrosine residue are essential for S-ADENOSYLMETHIONINE binding and catalysis. The PR domain has high homology to the catalytic region of the SET domain and occurs at the N-terminal of PRDM proteins such as PRDM1 PROTEIN.'),
  'D000074482': NtO(MH='Thyroid Nuclear Factor 1', MS='A homeobox protein and transcription factor that localizes to the cell nucleus where it activates expression of thyroid specific genes such as THYROGLOBULIN and the THYROTROPIN RECEPTOR. It is critical for maintaining thyroid tissue in a differentiated state and also plays a role in lung development. Mutations in the NKX2-1 gene are associated with CHOREA, BENIGN HEREDITARY.'),
  'D000074502': NtO(MH='Receptors, Kisspeptin-1', MS='G protein coupled receptors for the C-terminally amidated peptide of KISSPEPTIN-1. KISS1R plays an essential role in sexual development during puberty through its regulation of GONADOTROPIN-released hormones.'),
  'D000074522': NtO(MH='Consumer Health Informatics', MS='The field devoted in Informatics from multiple consumer or patient views.'),
  'D000074523': NtO(MH='Suprachiasmatic Nucleus Neurons', MS='Neurons of the SUPRACHIASMATIC NUCLEUS.'),
  'D000074542': NtO(MH='Solute Carrier Organic Anion Transporter Family Member 1B3', MS='A sodium-independent organic anion transporter that functions in the uptake of various drugs and endogenous compounds including ESTRADIOL; TAUROCHOLATE; LEUKOTRIENE C4; and METHOTREXATE. It also functions in clearing BILE ACIDS and organic anions from the liver. Mutations in the SLCO1B3 gene are associated with Rotor Type HYPERBILIRUBINEMIA.'),
  'D000074562': NtO(MH='Receptor, Notch4', MS='A notch receptor and proto-oncogene protein characterized by a large extracellular domain that consists of 29 EPIDERMAL GROWTH FACTOR  - like repeat sequences (EGF repeats) and five ANKYRIN REPEATS. It functions as a receptor for SERRATE-JAGGED PROTEINS and Delta1 (DLK1) protein to control cell fate determination.'),
  'D000074582': NtO(MH='AAA Proteins', MS='A large, highly conserved and functionally diverse superfamily of NTPases and nucleotide-binding proteins that are characterized by a conserved 200 to 250 amino acid nucleotide-binding and catalytic domain, the AAA+ module. They assemble into hexameric ring complexes that function in the energy-dependent remodeling of macromolecules. Members include ATPASES ASSOCIATED WITH DIVERSE CELLULAR ACTIVITIES.'),
  'D000074583': NtO(MH='Short Chain Dehydrogenase-Reductases', MS='A large family of oxidoreductases that are predominantly NAD- or NADP-dependent and 250 to 350 amino acids in length. They generally consist of two PROTEIN DOMAINS: A catalytic N-terminal domain that binds the substrate, and a C-terminal coenzyme-binding domain.'),
  'D000074584': NtO(MH='WW Domain-Containing Oxidoreductase', MS='A short chain oxidoreductase that contains two N-terminal WW DOMAINS and functions as a tumor suppressor and in APOPTOSIS. It is also required for bone development.'),
  'D000074585': NtO(MH='WW Domains', MS='An approximately 40 amino acid protein domain that occurs in a variety of unrelated proteins and may be repeated up to four times in some proteins. It is characterized by two TRYPTOPHAN residues (WW) about 20 amino acids apart and folds into a stable triple-stranded BETA-SHEET. It binds PROLINE-RICH PROTEIN DOMAINS and PHOSPHOSERINE or PHOSPHOTHREONINE-containing protein domains that occur in many signal-transducing and cytoskeletal proteins such as DYSTROPHIN.'),
  'D000074586': NtO(MH='Remyelination', MS='The reforming of the MYELIN SHEATH around AXONS following loss due to injury or DEMYELINATING DISEASES.'),
  'D000074602': NtO(MH='Smoking Devices', MS='Gadgets, utensils, apparatuses or instruments used for SMOKING.'),
  'D000074604': NtO(MH='Smoking Pipes', MS='Devices used for SMOKING which convey SMOKE directly into the mouth.'),
  'D000074606': NtO(MH='Smoking Prevention', MS='Efforts directed at preventing SMOKING of TOBACCO and non-tobacco products.'),
  'D000074607': NtO(MH='Opium Dependence', MS='Strong physiological and emotional dependence on OPIUM.'),
  'D000074608': NtO(MH='MYND Domains', MS='Zinc finger domains (named for myeloid, Nervy and DEAF-1) that occur in a variety of eukaryotic proteins, including RUNT-RELATED TRANSCRIPTION FACTOR 1 . They are characterized by a cluster of cysteine and histidine residues with conserved spacing that forms the zinc-binding motif and have beta-beta-alpha (see BETA-SHEET and ALPHA-HELIX) topology, similar to LIM domains (see LIM DOMAIN PROTEINS) and RING FINGER DOMAINS. MYND domains function as protein interaction motifs and have affinity for PROLINE-RICH PROTEIN DOMAINS.'),
  'D000074609': NtO(MH='Marijuana Use', MS='Medicinal or recreational utilization of MARIJUANA.'),
  'D000074623': NtO(MH='Dual Oxidases', MS='NADPH oxidases that contain two additional EF HAND MOTIFS and an N-terminal PEROXIDASE domain. They are expressed by THYROCYTES and EPITHELIAL CELLS of the kidney, liver, trachea, lung, and glandular tissues such as the testis, pancreas, and prostate. They are critical for the activity of THYROID PEROXIDASE and play a role in the production of thyroid hormones; they may also have antimicrobial activity through the generation of REACTIVE OXYGEN SPECIES.'),
  'D000074624': NtO(MH='NADPH Oxidase 1', MS='An NADPH oxidase that functions as a voltage-gated proton channel expressed by PHAGOCYTES, especially in the colon. It regulates intracellular pH, generates SUPEROXIDES upon activation by PHAGOCYTOSIS, and may play a role in INNATE IMMUNITY.'),
  'D000074642': NtO(MH='Nephrolithotomy, Percutaneous', MS="Surgical removal of large KIDNEY CALCULI by means of a percutaneous nephroscope which is passed into the KIDNEY PELVIS through a track created in the patient's back."),
  'D000074643': NtO(MH='Nephrotomy', MS='Surgical incision into any part of the kidney.'),
  'D000074662': NtO(MH='NADPH Oxidase 2', MS='An NADPH oxidase that is expressed by PHAGOCYTES where it transfers electrons across the plasma membrane from cytosolic NADPH to molecular oxygen on the exterior. It regulates proton (H+) flux into resting phagocytes to control intracellular pH. Mutations in the CYBB gene are associated with X-LINKED CHRONIC GRANULOMATOUS DISEASE.'),
  'D000074663': NtO(MH='NADPH Oxidase 4', MS='An NADPH oxidase that is strongly expressed in the kidney. It forms a complex with CYBA-P22PHOX and produces intracellular SUPEROXIDES that may regulate cellular signaling in APOPTOSIS; BONE RESORPTION; and NF-KAPPA B activation.'),
  'D000074664': NtO(MH='NADPH Oxidase 5', MS='An NADPH oxidase that contains four EF HANDS and is expressed primarily by SPERMATOCYTES and LYMPHOCYTES, as well as by endothelial cells. It functions as a calcium-dependent proton channel to generate SUPEROXIDES that regulate cell growth, APOPTOSIS; and PHYSIOLOGIC ANGIOGENESIS.'),
  'D000074682': NtO(MH='Nephroureterectomy', MS='Surgical removal of a kidney and adjoining ureter.'),
  'D000074702': NtO(MH='Proteostasis', MS='Regulation of the concentration, folding, interactions, and cellular localization of each of the proteins that comprise the PROTEOME.'),
  'D000074703': NtO(MH='Nematocera', MS='A suborder of insects which belong to the order DIPTERA. They include mosca, mosquito, gnats, black flies, true flies and long-horned flies.'),
  'D000074704': NtO(MH='Quasispecies', MS='Variations among a population of a given species due to errors in replication of the genome that result in a distribution of non-identical members.'),
  'D000074705': NtO(MH='Human Genetics', MS='The scientific study of inherited human variation.'),
  'D000074706': NtO(MH='Sublimation, Chemical', MS='The process of solids transforming into a gaseous state or vice versa, without passing through a liquid state.'),
  'D000074707': NtO(MH='Campylobacterales', MS='Gram negative microaerophilic bacteria of the phylum PROTEOBACTERIA, class  EPSILONPROTEOBACTERIA.'),
  'D000074722': NtO(MH='Lateral Internal Sphincterotomy', MS='Surgical incision of the INTERNAL ANAL SPHINCTER typically in the treatment of FISSURE IN ANO; chronic ANAL FISSURE and FECAL INCONTINENCE.'),
  'D000074723': NtO(MH='Oncogene Addiction', MS='The dependence of tumor cells on a single oncogenic pathway or protein for their continued proliferation and survival.'),
  'D000074742': NtO(MH='Median Arcuate Ligament Syndrome', MS='Compression of the CELIAC ARTERY by the median arcuate ligament, a fibrous band of the DIAPHRAGM, causing abdominal pain after eating and weight loss. OMIM: 116870'),
  'D000074743': NtO(MH='Hemodynamic Monitoring', MS='Continuous measurement of the movement and forces of blood in the CARDIOVASCULAR SYSTEM.'),
  'D000074744': NtO(MH='ADP-Ribosylation', MS='Post-translational modification of proteins with ADENOSINE DIPHOSPHATE RIBOSE.'),
  'D000074745': NtO(MH='Pterygota', MS='A subclass of winged insects belonging to the class Insecta.'),
  'D000074746': NtO(MH='Neoptera', MS='Modern insects belonging to the subclass PTERYGOTA, Many have the ability to fold their wings with exceptions being butterflies, moths, and a few others within the group.'),
  'D000074747': NtO(MH='Poly ADP Ribosylation', MS='Post-translational modification of proteins with POLY ADENOSINE DIPHOSPHATE RIBOSE.'),
  'D000074748': NtO(MH='Holometabola', MS='A superorder of insects within the NEOPTERA that go through metamorphosis from egg to larva, pupa, and adult stages. Orders of MOSQUITOES; BEES; BUTTERFLIES; and FLEAS belong to this group.'),
  'D000074749': NtO(MH='Deubiquitinating Enzyme CYLD', MS='A deubiquitinase and tumor-suppressor protein that specifically cleaves LYSINE-63-linked polyubiquitin chains and also has endodeubiquitinase activity. It functions to regulate NF-KAPPA B and WNT SIGNALING PATHWAY activity, contributing to cell survival, proliferation, and differentiation. Mutations in the CYLD gene are associated with cases of FAMILIAL CYLINDROMATOSIS.'),
  'D000074762': NtO(MH='Aeromonadales', MS='An order of Gram-negative bacteria in the class GAMMAPROTEOBACTERIA, phylum PROTEOBACTERIA, which include important pathogens.'),
  'D000074764': NtO(MH='Alethinophidia', MS='An infraorder of snakes in the order Serpentes that includes all snakes except blind snakes and thread snakes.'),
  'D000074765': NtO(MH='Dysbindin', MS='A dystrophin-associated protein and component of the Biogenesis of Lysosomal Organelles Complex-1 (BLOC-1 complex) which is essential for the formation of LYSOSOME - derived organelles such as platelet dense granules and MELANOSOMES. DTNBP1 is expressed primarily in the brain and neurons, where it functions with ADAPTOR PROTEIN COMPLEX 3 to transport membrane proteins to NEURITES and nerve terminals. It also regulates the release of neurotransmitters, transport of synaptic vesicles, and localization of DOPAMINE D2 RECEPTORS. Mutations in the DTNBP1 gene are associated with Type 7 HERMANSKY-PUDLAK SYNDROME and SCHIZOPHRENIA.'),
  'D000074766': NtO(MH='Dipsacales', MS='An order of dicotyledonous flowering plants which includes six families. It is best known for its ornamental plants such as LONICERA (honeysuckle), VIBURNUM (arrowwood and guelder rose), and SCABIOSA (scabious, or pincushion flower).'),
  'D000074767': NtO(MH='Diapause', MS='A period of arrested growth or development in animals that is triggered by external conditions, such as length of day, extreme temperatures, or reduced food availability. It can occur at the embryonic, larval, pupal, or adult stage, depending on the species.'),
  'D000074822': NtO(MH='Treatment Adherence and Compliance', MS='Extent to which the patient follows prescribed treatment such as keeping APPOINTMENTS AND SCHEDULES and MEDICATION ADHERENCE for desired therapeutic outcome. It implies active responsibility shared by patient and health care providers.'),
  'D000074823': NtO(MH='One Health', MS='An integrative effort of multiple disciplines working collaboratively and locally, nationally, and globally in all aspects of health care for humans, animals, and the environment.'),
  'D000074824': NtO(MH='Work Engagement', MS='Extent to which members of a workplace perceive their emotional commitment to and involvement in the organization and its goals.'),
  'D000074842': NtO(MH='Forkhead Box Protein L2', MS='A forkhead box transcription factor that is expressed in the developing eyelid and during very early development of the gonad, prior to sex determination. It is essential for development of the ovary and inhibits SOX9 TRANSCRIPTION FACTOR to prevent differentiation to testes. It also induces APOPTOSIS in ovarian cells. Mutations in the FOXL2 gene are associated with BLEPHAROPHIMOSIS; Ptosis, and Epicanthus inversus (BPES with ovarian failure).'),
  'D000074868': NtO(MH='Salpingo-oophorectomy', MS='Combined surgical resection of the fallopian tube and the ovary.'),
  'D000074882': NtO(MH='Pyloromyotomy', MS='Surgical incision of the PYLORUS used to treat pyloric stenoses (e.g. INFANTILE HYPERTROPHIC PYLORIC STENOSIS).'),
  'D000074883': NtO(MH='Vulvectomy', MS='Surgical resection of the VULVA.'),
  'D000074884': NtO(MH='Mucosa-Associated Lymphoid Tissue Lymphoma Translocation 1 Protein', MS='A caspase-like cysteine endopeptidase that also exhibits ubiquitin ligase activity. It contains an N-terminal DEATH DOMAIN, two IMMUNOGLOBULIN-LIKE DOMAINS, and localizes to the perinuclear region of MONOCYTES, where it functions in activation of NF-KAPPA B; it also binds to and activates TRAF6. Chromosomal translocations involving the MALT1 and BIRC2 genes are associated with MALT LYMPHOMA, and mutations in the MALT1 gene are associated with Type 12 IMMUNODEFICIENCY SYNDROMES.'),
  'D000074885': NtO(MH='Amniotomy', MS='A method of inducing labor by deliberate rupture of the AMNION to cause the release of amniotic fluid.'),
  'D000074923': NtO(MH='High-Temperature Requirement A Serine Peptidase 2', MS='A serine peptidase that contains a C-terminal PDZ domain. It localizes to the mitochondrial membrane and intermembrane space, translocating to the cytoplasm following APOPTOSIS stimuli, such as UV irradiation; it promotes cell death by binding to and inhibiting INHIBITOR OF APOPTOSIS PROTEINS, resulting in an increase in activity of CASPASES. Mutations in the HTRA2 gene are associated with Type 13 PARKINSON DISEASE.'),
  'D000074924': NtO(MH='Nut and Peanut Hypersensitivity', MS='Allergic reaction to tree nuts and peanuts, including other LEGUMES, that is triggered by the immune system. It includes co-sensitization to other food (e.g., sesame seed).'),
  'D000074962': NtO(MH='Kounis Syndrome', MS='A disorder of cardiac function secondary to hypersensitivity reactions. It is characterized by coexistence of acute coronary syndromes and cardiac MAST CELL and PLATELET ACTIVATION. It may be induced by exposure to drugs (e.g., antibiotics, anesthetics, contrast media), food, and environmental triggers (e.g., insect bites and stings, poison ivy).'),
  'D000074982': NtO(MH='Palaeoptera', MS='Infraclass of ancient winged insects belonging to the subclass PTERYGOTA.'),
  'D000075': NtO(MH='Acepromazine', MS='A phenothiazine that is used in the treatment of PSYCHOSES.'),
  'D000075002': NtO(MH='Bestrophins', MS='A protein family characterized by a highly conserved N-terminus and four to six transmembrane helices; they function as bicarbonate permeable, calcium-activated chloride channels. Bestrophin-1 (BEST-1) and bestrophin-2 are highly expressed in human RETINAL PIGMENT EPITHELIUM cells and mutations in the BEST-1 gene are associated with VITELLIFORM MACULAR DYSTROPHY, TYPE 2.'),
  'D000075022': NtO(MH='Academic Failure', MS='Student\xe2\x80\x99s unsuccessful attempt at academic achievement or a marked inadequacy in the areas of scholarship or study. This is not underachievement which is performance, usually in school work, poorer than that predicted from aptitude and/or intelligence testing.'),
  'D000075023': NtO(MH='S100 Calcium Binding Protein A7', MS='An S100 calcium binding protein that contains two EF HAND MOTIFS and also binds zinc. It is secreted via a non-classical secretory pathway and expressed by KERATINOCYTES and epithelial cells of the tongue. It has antimicrobial and immunomodulatory activities and is highly expressed in the skin of patients with PSORIASIS, as well as in bladder and skin epithelial carcinomas.'),
  'D000075024': NtO(MH='rhoC GTP-Binding Protein', MS='A rho GTP-binding protein that is prenylated (see PROTEIN PRENYLATION) at its C-terminus and cycles between the cytoplasm and plasma membrane, linking receptor signaling pathways with assembly of FOCAL ADHESIONS; STRESS FIBERS; and contractile ring formation during CYTOKINESIS. It is overexpressed in proliferating and metastatic tumor cells.'),
  'D000075042': NtO(MH='Sustainable Growth', MS='Achievement of an economic system that can continue to grow for the foreseeable future.'),
  'D000075062': NtO(MH='Pre-B-Cell Leukemia Transcription Factor 1', MS="A TALE-type homeodomain protein and transcription factor that binds the DNA sequence 5'-ATCAATCAA-3'. It forms a heterodimer with MEIS1 TRANSCRIPTION FACTOR and functions as a transcriptional activator of HOMEOBOX PROTEIN NKX-2.5 and ELONGIN A, and as a transcriptional repressor of CDKN2B PROTEIN, in the regulation of developmental and morphogenetic processes such as spleen and limb development. Chromosome translocations involving the PBX1 and TCF3 genes occur in cases of pre-B-cell ACUTE LYMPHOID LEUKEMIA."),
  'D000075063': NtO(MH='Lilianae', MS='A superorder of flowering plants (Angiosperms) which includes monocotyledonous plants.'),
  'D000075064': NtO(MH='Myeloid Ecotropic Viral Integration Site 1 Protein', MS='A TALE-type homeodomain protein and transcription factor that functions as a regulator of PAX6 PROTEIN expression and as an activator of PLATELET FACTOR 4 gene expression. It is essential for hematopoiesis, differentiation of MEGAKARYOCYTES, and vascular patterning. It may also have a role in the induction of myeloid leukemias.'),
  'D000075065': NtO(MH='Elongin', MS='A heterotrimeric protein complex composed of 110 kDa elongin A, 18 kDa elongin B, and 15 kDa elongin C subunits. It functions as a positive regulator of RNA POLYMERASE II, increasing its rate of transcriptional elongation by suppressing transient pausing along the DNA template. Elongin A is the transcriptionally active component; elongins B and C enhance its activity.'),
  'D000075066': NtO(MH='Body Packing', MS='Concealing of drugs within the gastrointestinal tract or other body orifice for purposes of DRUG SMUGGLING or to avoid detection by law enforcement. Contraband other than drugs is also sometimes smuggled this way.'),
  'D000075067': NtO(MH='Substance Abuse, Oral', MS='Abuse, overuse, or misuse of a substance by ingestion.'),
  'D000075082': NtO(MH='Proof of Concept Study', MS='An empirical investigation which pertains to the development of prototypes or models that demonstrate the feasibility of novel concepts, ideas, principles, schema or their practical application.'),
  'D000075102': NtO(MH='Organic Cation Transporter 2', MS='Organic cation transporter consisting of twelve transmembrane domains and expressed primarily in the kidney. It transports a wide range of metabolites, drugs, and neurotransmitters from the blood to the KIDNEY TUBULES, including DOPAMINE; SEROTONIN; CHOLINE; and CISPLATIN.'),
  'D000075122': NtO(MH='Smoldering Multiple Myeloma', MS='An asymptomatic and slow-growing PLASMA CELL dyscrasia characterized by presence of MYELOMA PROTEINS and clonal bone marrow plasma cells without end-organ damage (e.g., renal impairment). It is distinguished from MONOCLONAL GAMMOPATHY OF UNDETERMINED SIGNIFICANCE by a much higher risk of progression to symptomatic MULTIPLE MYELOMA.'),
  'D000075142': NtO(MH='RUNX1 Translocation Partner 1 Protein', MS='A transcriptional co-repressor that contains a MYND-type zinc finger (MYND DOMAIN) at its C-terminal and functions as a homo-oligomer. It associates with DNA-binding transcription factors, other repressor proteins, and HISTONE ACETYLTRANSFERASES to repress expression of genes involved in cell growth and differentiation such as MATRIX METALLOPROTEINASE 7 and TCF12. A CHROMOSOMAL TRANSLOCATION involving the RUNX1T1 and CORE BINDING FACTOR ALPHA 2 SUBUNIT (RUNX1) genes frequently occurs in cells of leukemia patients; the resulting fusion protein (AML1-ETO or RUNX1-RUNX1T1) plays a critical role in leukemogenesis.'),
  'D000075162': NtO(MH='Abuse-Deterrent Formulations', MS='Drug formulations or delivery systems intended to discourage the abuse of CONTROLLED SUBSTANCES. These may include physical barriers to prevent chewing or crushing the drug; chemical barriers that prevent extraction of psychoactive ingredients; agonist-antagonist combinations to reduce euphoria associated with abuse; aversion, where controlled substances are combined with others that will produce an unpleasant effect if the patient manipulates the dosage form or exceeds the recommended dose; delivery systems that are resistant to abuse such as implants; or combinations of these methods.'),
  'D000075163': NtO(MH='Metallocenes', MS='Organometallic compounds that generally consist of two cylcopentadiene ANIONS joined in their centers by a metallic cation such as NICKEL; IRON; or TITANIUM.'),
  'D000075182': NtO(MH='Agent Orange', MS='A herbicide that contains equal parts of 2,4-dichlorophenoxyacetic acid (2,4-D) and 2,4,5-trichlorophenoxyacetic acid (2,4,5-T), as well as traces of the contaminant 2,3,7,8-tetrachlorodibenzo-p-dioxin.'),
  'D000075202': NtO(MH='Contraindications', MS='A condition or factor associated with a recipient that makes the use of a drug, procedure, or physical agent improper or inadvisable.  Contraindications may be absolute (life threatening) or relative (higher risk of complications in which benefits may outweigh risks).'),
  'D000075203': NtO(MH='Contraindications, Drug', MS='A condition or factor associated with a recipient that makes the use of a specific drug improper or inadvisable.'),
  'D000075204': NtO(MH='Contraindications, Procedure', MS='A condition or factor associated with a recipient that makes the use of a procedure improper or inadvisable.'),
  'D000075222': NtO(MH='Essential Hypertension', MS='Hypertension that occurs without known cause, or preexisting renal disease. Associated polymorphisms for a number of genes have been identified, including AGT, GNB3, and ECE1. OMIM: 145500'),
  'D000075223': NtO(MH='Poly-ADP-Ribose Binding Proteins', MS='Proteins that contain POLY-ADP RIBOSE BINDING MOTIFS. They include HISTONES and other proteins that function in DNA REPAIR, replication, gene transcription, and APOPTOSIS.'),
  'D000075224': NtO(MH='Cardiac Conduction System Disease', MS='Diseases characterized by pathological irregularities in the HEART CONDUCTION SYSTEM. They may be associated with other heart diseases and syndromes (e.g., BRUGADA SYNDROME; NEUROMUSCULAR DISEASE, HEART BLOCKS), isolated or may result from injuries. You can have a conduction disorder without having an arrhythmia, but some arrhythmias arise from conduction disorders. OMIM: 601144.'),
  'D000075225': NtO(MH='Poly-ADP-Ribose Binding Motif', MS='A protein motif 22 to 26 amino acids in length that binds POLY(ADP RIBOSE) polymers through non-covalent interactions. It is characterized by basic and hydrophobic residues that frequently include ALANINE; VALINE; ISOLEUCINE; or LEUCINE and flank LYSINE and ARGININE amino acids.'),
  'D000075242': NtO(MH='Janus Kinase Inhibitors', MS='Agents that inhibit JANUS KINASES.'),
  'D000075243': NtO(MH='Apolipoproteins M', MS='Apolipoproteins and lipocalins that occur in HIGH-DENSITY LIPOPROTEINS. They bind or transport lipids in the blood including sphingosine-1-phosphate, MYRISTIC ACID; STEARIC ACIDS; and ALL-TRANS-RETINOIC ACID.'),
  'D000075244': NtO(MH='Apelin Receptors', MS='G-protein coupled receptors for APELIN that function in a broad range of physiologic processes including blood pressure regulation and heart contractility. They also have an essential role in early embryo development for GASTRULATION and heart morphogenesis, and can also function as a CD4 co-receptor for HIV-1.'),
  'D000075262': NtO(MH='Hypoadrenocorticism, Familial', MS='Hereditary forms of Addison disease that may exhibit autosomal recessive or X-linked  inheritance. They are characterized by severe neurological symptoms, APNEA;  and death in infancy. OMIM: 240200'),
  'D000075282': NtO(MH='Incivility', MS='Low-intensity deviant behavior with ambiguous intent to harm the target, in violation of norms for mutual respect.'),
  'D000075302': NtO(MH='Connexin 30', MS="A gap junction beta subunit that forms heteromeric hemichannels when paired with alpha subunits such as connexin-40 or CONNEXIN 43. Mutations in the connexin 30 gene (GJ6B) are associated with CLOUSTON'S SYNDROME and some hereditary forms of deafness."),
  'D000075322': NtO(MH='Heavy Metal Poisoning', MS='Poisoning that results from chronic or acute ingestion, injection, inhalation, or skin absorption of HEAVY METALS. Acute and chronic exposures can cause ANEMIA; KIDNEY and LIVER damage; PULMONARY EDEMA; MEMORY LOSS and behavioral changes; bone deformities in children; and MISCARRIAGE or PREMATURE LABOR in pregnant women.'),
  'D000075342': NtO(MH='Animal Scales', MS='Tough, horny, brittle, smooth, striated or plate-like extensions of the stratum corneum or outer layer of the skin. It serves as a protective layer in fishes, reptiles, birds, and mammals.'),
  'D000075362': NtO(MH='Leukocyte Immunoglobulin-like Receptor B1', MS='A receptor for HISTOCOMPATIBILITY ANTIGENS CLASS I that also functions as a receptor for the UL18 protein, an MHC class I homolog expressed by human CYTOMEGALOVIRUS. It consists of four Ig-like C2 domains and is expressed primarily by B-cells and MONOCYTES, as well as DENDRITIC CELLS; its interaction with MHC1 functions to down-regulate the immune response.'),
  'D000075363': NtO(MH='Immunoglobulin Light-chain Amyloidosis', MS='A nonproliferative disorder of the PLASMA CELL characterized by excessive production and misfolding of IMMUNOGLOBULIN LIGHT CHAINS that form insoluble amyloid fibrils (see AMYLOID DEPOSITS) in various tissues. Clinical features include LIVER FAILURE; MULTIPLE MYELOMA; NEPHROTIC SYNDROME; RESTRICTIVE CARDIOMYOPATHY, and neuropathies.'),
  'D000075364': NtO(MH='Deleted in Azoospermia 1 Protein', MS='An RNA recognition motif protein that is essential for SPERMATOGENESIS. It promotes entry of male GERM CELLS to MEIOSIS, possibly by regulating the translation of mRNAs. DAZ1 occurs within a cluster of similar genes on the Y CHROMOSOME that is prone to genetic deletions and duplications. Deletions in these genes, including DAZ1, are associated with AZOOSPERMIA and OLIGOSPERMIA.'),
  'D000075366': NtO(MH='Chemokine CCL18', MS='A CC-type chemokine highly expressed in the lungs, lymph nodes, placenta, and bone marrow; it is also expressed by DENDRITIC CELLS in the GERMINAL CENTER, and peripheral blood MACROPHAGES. It functions as a chemotactic factor that specifically attracts LYMPHOCYTES, especially B-Cells, into lymph node follicles, and naive T-cells towards dendritic cells and activated T-cells. It does not attract MONOCYTES or GRANULOCYTES.'),
  'D000075368': NtO(MH='Anoctamins', MS='A family of transmembrane proteins that function primarily as calcium-activated chloride channels. Structurally, they form a homodimer where each subunit consists of eight transmembrane helices with the N and C terminals exposed to the cytosol. The regions between helices 5 and 7 may be important for ion pore formation and calcium ion binding.'),
  'D000075369': NtO(MH='Anoctamin-1', MS='An anoctamin chloride channel expressed at high levels in the liver, skeletal muscle, and gastrointestinal muscles that functions in transepithelial anion transport and smooth muscle contraction. It is essential for the function of the INTERSTITIAL CELLS OF CAJAL and plays a major role in chloride conduction by airway epithelial cells and in tracheal cartilage development.'),
  'D000075370': NtO(MH='Receptors for Activated C Kinase', MS='Receptors for PROTEIN KINASE C that consist of seven WD40 REPEATS. They function in a wide variety of cellular and physiologic processes including the assembly of signaling complexes, protein translation, cell growth and proliferation, APOPTOSIS, and MUSCARINIC RECEPTOR transport. RACK1 depends on protein kinase C activity to translocate from the perinuclear region to the cell periphery and associates with the plasma membrane through interactions with KERATIN-1 and INTEGRIN BETA-1.'),
  'D000075383': NtO(MH='S100 Calcium Binding Protein A6', MS='An S100 calcium binding protein that contains two EF HAND MOTIFS and plays a role as a calcium sensor and modulator for many cellular processes such as CELL CYCLE progression; CELL MOTILITY and reorganization of the ACTIN CYTOSKELETON. Its expression is induced by growth factors and it is overexpressed in patients with ACUTE MYELOID LEUKEMIA.'),
  'D000075384': NtO(MH='Neuroticism', MS='Personality trait related to tendency to respond to threat, frustration or a loss with negative emotions (e.g., ANGER; ANXIETY; FRUSTRATION; embarrassment and sadness).'),
  'D000075385': NtO(MH='Netrins', MS='A family of extracellular proteins that are related structurally to LAMININ. They function as CHEMOTACTIC FACTORS for CELL MIGRATION and AXON GUIDANCE, acting as chemoattractants for some cell types, and as chemorepellents for others.'),
  'D000075386': NtO(MH='Netrin Receptors', MS='Cell surface receptors that bind NETRINS. They typically contain both IMMUNOGLOBULIN DOMAINS and FIBRONECTIN TYPE III DOMAINS and function to mediate CELL MIGRATION and AXON GUIDANCE.'),
  'D000075387': NtO(MH='Smoking Water Pipes', MS='Pipes for smoking tobacco, cannabis, and other substances, in which smoke is drawn through water. Do not confuse with SMOKING PIPES.'),
  'D000075388': NtO(MH='Netrin-1', MS='A netrin that binds the DCC RECEPTOR or UNC5 receptors, resulting respectively in axon attraction or repulsion. Its interaction with these receptors also prevents APOPTOSIS; it may function as a tumor suppressor protein.'),
  'D000075389': NtO(MH='DCC Receptor', MS='A receptor for NETRIN-1 that contains four membrane distal (N-terminal) Ig-like C2 domains and six membrane proximal (C-terminal) fibronectin type III domains. It is expressed primarily in the nerve axons and differentiated intestinal cells and is required for AXON GUIDANCE, mediating axon attraction of neuronal GROWTH CONES; however, it may also trigger axon repulsion through association with the UNC5 netrin-1 receptor. DCC also induces APOPTOSIS when it is unbound from netrin-1.  Deletions of the DCC gene are observed in tumor metastases and the protein is not expressed in undifferentiated COLORECTAL CARCINOMA cells.'),
  'D000075402': NtO(MH='Channelrhodopsins', MS='A subfamily of rhodopsin proteins that function as light-gated ion channels in GREEN ALGAE.'),
  'D000075403': NtO(MH='Semantic Web', MS='A framework for development and promotion of common data formats and exchange protocols linked in a way that can be read directly by computers. Semantic Web is a platform for sharing and reusing data across application, enterprise, and community boundaries, by linking concepts rather than just documents.'),
  'D000075422': NtO(MH='Anti-Citrullinated Protein Antibodies', MS='Autoantibodies to citrullinated-peptides and proteins.'),
  'D000075423': NtO(MH='Gene Drive Technology', MS='The techniques involved in creating and inserting synthetic selfish genetic elements called gene drives. Gene drives carry a "payload gene" and are designed to increase in frequency in the population over time, eventually to all members of the population.'),
  'D000075442': NtO(MH='Transplantation, Haploidentical', MS='Transplantation between individuals who share a partial haplotype match.'),
  'D000075462': NtO(MH='Serum Albumin, Human', MS='Serum albumin from humans. It is an essential carrier of both endogenous substances, such as fatty acids and BILIRUBIN, and of XENOBITOICS in the blood.'),
  'D000075482': NtO(MH='Bedridden Persons', MS='Persons unable or unwilling, or otherwise incapacitated and not able to leave the bed due to physical or mental conditions.'),
  'D000075485': NtO(MH='Population Health', MS='The health outcomes of a group of individuals, including the distribution of such outcomes within the group. These populations are often geographic regions, such as nations or communities, but they can also be other groups. (From Am J Public Health. 2003 March; 93(3): 380\xe2\x80\x93383)'),
  'D000075522': NtO(MH='Adaptive Clinical Trials as Topic', MS='Works about clinical studies in which a prospectively planned opportunity is included to modify trial designs and hypotheses based on analysis of data from subjects in the study.'),
  'D000075523': NtO(MH='Long-Acting Reversible Contraception', MS='Prevention of CONCEPTION by devices, chemical substances or agents with contraceptive activity in females which last for years and can be removed.'),
  'D000075527': NtO(MH='Prolotherapy', MS='Treatment of MUSCULOSKELETAL PAIN by injecting a substance into a joint space, ligament, or tendon to promote the growth of new tissue.'),
  'D000075528': NtO(MH='Aversive Agents', MS='Chemicals added to pharmacologic preparations, poisonous household goods, and other chemicals to discourage their abuse or consumption.'),
  'D000075529': NtO(MH='Solitary Kidney', MS='Either a single or a single functioning kidney due to NEPHRECTOMY, birth defects or other kidney diseases.'),
  'D000075530': NtO(MH='Organ Motion', MS='Movement of internal organs due to physiological processes.'),
  'D000075542': NtO(MH='Multifocal Intraocular Lenses', MS='Artificially implanted lenses that direct light toward distant and near focal points allowing clear vision for a range of distances.'),
  'D000075562': NtO(MH='Charles Bonnet Syndrome', MS='Repetitive visual hallucinations experienced mostly by elderly with diminished visual acuity or visual field loss, with awareness of the fictional nature of their hallucinations. It is not associated with delusions and other sensory hallucinations.'),
  'D000075563': NtO(MH='Vasopeptidase Inhibitors', MS='A class of cardiovascular drugs indicated for hypertension and congestive heart failure that simultaneously inhibit both NEUTRAL ENDOPEPTIDASE and ANGIOTENSIN CONVERTING ENZYME. They increase the availability of NATRIURETIC PEPTIDES and BRADYKININ and inhibit production of ANGIOTENSIN II.'),
  'D000075602': NtO(MH='Non-Point Source Pollution', MS='Water pollution from a variety of diffuse sources carried over or through the ground and into water sources such as LAKES; RIVERS; WETLANDS; coastal waters; and GROUNDWATER. Such diffuse sources include roadways and parking lots (GASOLINE; HEAVY METALS; and motor oil), lawns or agricultural land (excess FERTILIZERS, livestock excrement, and PESTICIDES), landfill seepage, and construction sites (chemicals and trash used in construction processes).'),
  'D000075662': NtO(MH='Injection Site Reaction', MS='Adverse reactions that occur initially at the site of injection or infusion. Milder type is confined to a local allergic flare reaction. A more severe reaction is caused by extravasation of VESICANTS from the blood vessel at the site of injection and can cause damage to the surrounding tissue. In tumor flare reaction symptoms involve well beyond the injection site such as an increase in the tumor size and tumor markers levels, bone pain, and HYPERCALCEMIA.'),
  'D000075663': NtO(MH='Ion Mobility Spectrometry', MS='Techniques for separation and identification of ionized molecules based on their drift through a gas phase after being accelerated by an electric field. Their mobilities will be effected by their shape, size, and charge.'),
  'D000075665': NtO(MH='Recidivism', MS='Repeated criminal behavior.'),
  'D000075682': NtO(MH='Procrastination', MS='The deferment of actions or tasks to a later time, or to infinity.'),
  'D000075684': NtO(MH='T-Lymphoma Invasion and Metastasis-inducing Protein 1', MS='A Rho guanine nucleotide exchange factor that consists of two pleckstrin homology domains flanking central Ras-binding, Dbl-homology (DH) and PDZ domains. It stimulates GTP-GDP exchange of rho-like GTPases such as RAC1 PROTEIN; CDC42 PROTEIN; and RHOA PROTEIN, and functions to link extracellular signals to remodeling of the CYTOSKELETON for CELL ADHESION and CELL MOVEMENT.'),
  'D000075686': NtO(MH='COP9 Signalosome Complex', MS='A multiprotein complex that functions as a peptide hydrolase, or isopeptidase to cleave NEDD8 PROTEIN from the CULLIN and UBIQUITIN-PROTEIN LIGASES, controlling the activity of the ligases. It is highly conserved in eukaryotes and typically consists of 8 subunits (CSN 1-8 proteins). The COP9 signalosome was originally identified in plants, where it controls gene transcription in response to light.'),
  'D000075687': NtO(MH='NEDD8 Protein', MS='A ubiquitin-like protein that functions in CELL CYCLE regulation and embryogenesis. It is attached covalently to its substrates following activation by the UBIQUITIN-ACTIVATING ENZYME E1-UBA3 enzyme complex. NEDD8 attaches to CULLINS, activating their E3 UBIQUITIN LIGASE activity, to promote polyubiquitination and degradation of CYCLINS and regulatory proteins'),
  'D000075702': NtO(MH='Nedd4 Ubiquitin Protein Ligases', MS='E3 ubiquitin ligases that consist of four WW DOMAINS. They accept UBIQUITIN from E2 UBIQUITIN-CONJUGATING ENZYME as a thioester via their C-terminal HECT domains and transfer it specifically to the 63rd LYSINE residue (Lys-63) of target proteins. NEDD4 targets include many proteins and receptors with important functions for cell growth and homeostasis such as VEGFR-2; FGFR1 TYROSINE KINASE; and ERBB-4 RECEPTOR. They play a critical role in the internalization of these receptors, their degradation by LYSOSOMES, and also function as part of the ESCRT complex in VIRUS RELEASE.'),
  'D000075703': NtO(MH='Zinc Transporter 8', MS='A zinc efflux transporter highly expressed by ISLET CELLS of the pancreas. It functions in the accumulation of zinc in intracellular vesicles and may be involved in INSULIN maturation and storage processes. Variations in the SLC30A8 gene are associated with susceptibility to DIABETES MELLITUS, TYPE 2.'),
  'D000075704': NtO(MH='Toxin-Antitoxin Systems', MS='Mechanisms that allow bacteria and archaea to rapidly adapt to changing environmental conditions via a toxin, produced during adverse conditions, that inhibits a specific vital process, and a partner antitoxin that blocks the effects of the toxin, under normal growth conditions.'),
  'D000075722': NtO(MH='Promyelocytic Leukemia Zinc Finger Protein', MS='A Kruppel-type transcription factor consisting of an N-terminal BTB DOMAIN and nine CYS2-HIS2 ZINC FINGERS. It localizes to the nucleus and regulates cell cycle progression and gene expression for tissue development and homeostasis; it may also function as an epigenetic regulator through its interactions with HISTONE DEACETYLASE. Genetic rearrangements involving the ZBTB16 gene are associated with ACUTE PROMYELOCYTIC LEUKEMIA.'),
  'D000075723': NtO(MH='Kruppel-Like Factor 6', MS='A Kruppel-like transcription factor that contains three C-terminal CYS2-HIS2 ZINC FINGERS and binds to GC RICH SEQUENCE (GC box) in upstream gene promoters. It functions as a transcriptional activator, tumor suppressor, and may regulate growth and development of B-cells.'),
  'D000075742': NtO(MH='Expression of Concern', MS='A notification about the integrity of a published article that is typically written by an editor and should be labelled prominently in the item title. It is the responsibility of the editor to initiate appropriate investigative procedures, discover the outcome of the investigation, and notify readers of that outcome in a subsequent published item. The outcome may require the publication of a retraction notice.'),
  'D000075743': NtO(MH='Chemokine CXCL16', MS='A CXCR6 receptor-binding chemokine that functions as a scavenger receptor for oxidized low density lipoprotein (OxLDL) when expressed by MACROPHAGES. Its secreted, or cytokine form induces a strong chemotactic response for MONOCYTES when it is expressed by DENDRITIC CELLS.'),
  'D000075744': NtO(MH='Receptors, CXCR6', MS='CXCR receptors that are specific for CHEMOKINE CXCL16. They are expressed by lymphoid tissues, activated T-cells, and also function as co-receptors for SIMIAN IMMUNODEFICIENCY VIRUSES; HIV-2; and HIV-1 infection of MACROPHAGES.'),
  'D000075745': NtO(MH='rab27 GTP-Binding Proteins', MS='GTP-binding proteins associated with membranes, MELANOSOMES; LYSOSOMES; and late ENDOSOMES. They play a role in the release of cytotoxic SECRETORY GRANULES from lymphocytes and are required for granule maturation, as well as granule docking and priming at IMMUNOLOGICAL SYNAPSES.'),
  'D000075746': NtO(MH='Zinc Finger Protein Gli2', MS='A transcriptional activator that contains five adjacent CYS2-HIS2 ZINC FINGERS. It functions in the hedgehog signaling pathway and is required for normal embryonic development. Mutations in the GLI2 gene are associated with type 9 HOLOPROSENCEPHALY and type 2 PALLISTER-HALL SYNDROME.'),
  'D000075747': NtO(MH='Baculoviral IAP Repeat-Containing 3 Protein', MS='A regulator of APOPTOSIS that functions as an E3 ubiquitin protein ligase. It contains three baculoviral IAP repeats in its N-terminal half, a CARD DOMAIN, and a RING finger domain at its C-terminus. It is highly expressed in fetal lung and kidney, and adult lymphoid tissues such as spleen, thymus, and peripheral blood lymphocytes. It functions in INFLAMMATION signaling, the INNATE IMMUNE RESPONSE, cell growth and proliferation, and metastasis of tumor cells.'),
  'D000075762': NtO(MH='Interdisciplinary Research', MS='Research combining mastery in distinct fields or disciplines that apply and exchange tools, concepts, ideas, data methods, or results around a common project.'),
  'D000075764': NtO(MH='Involuntary Fertility Control', MS="Behavior that interferes with a woman's autonomous reproductive decision-making."),
  'D000075782': NtO(MH='Cell Phone Use', MS='Utilization of wireless phones for communication.'),
  'D000075802': NtO(MH='Angiopoietin-like Proteins', MS='A family of proteins that is structurally similar to ANGIOPOIETINS but do not bind angiopoietin receptors. They are characterized by an amino-terminal coiled-coil domain, a linker region, and a carboxy-terminal FIBRINOGEN-like domain with the exception of ANGPTL8, which lacks the fibrinogen-like domain. They function in a variety of developmental and physiological processes, including INFLAMMATION, lipid metabolism, hematopoietic stem cell activity, and cancer metastasis.'),
  'D000075803': NtO(MH='Angiopoietin-like 4 Protein', MS='A secreted angiopoietin-like protein expressed under hypoxic conditions by ENDOTHELIAL CELLS. It inhibits cell proliferation, cell migration, and tubule formation; the inactive form accumulates in the endothelial EXTRACELLULAR MATRIX, reducing vascular leakage. ANGPTL4 has direct roles in regulating glucose and lipid metabolism, as well as INSULIN SENSITIVITY, and may also function as a regulator of angiogenesis and tumorigenesis.'),
  'D000075804': NtO(MH='Zinc Finger Nucleases', MS='Genetically engineered nucleases that cleave DNA at a defined distance from specific DNA sequences recognized by ZINC FINGER DNA-BINDING DOMAINS. They are composed of a DNA cleaving domain adapted from DNA endonucleases fused to a zinc finger DNA-binding domain.'),
  'D000075862': NtO(MH='Science in the Arts', MS='Depiction of scientific issues or themes via the arts including visual, oral, or written forms of expression.'),
  'D000075863': NtO(MH='Science in Literature', MS='Literary works whose subject matter is science or about the profession of science and related areas.'),
  'D000075884': NtO(MH='Indochina', MS='Area in Southeastern Asia that comprises CAMBODIA, LAOS, and VIETNAM.'),
  'D000075902': NtO(MH='Clinical Deterioration', MS='A critical disease progression, often measured by a set of clinical parameters, which activates HOSPITAL RAPID RESPONSE TEAM.'),
  'D000075906': NtO(MH='Chemokine CCL26', MS='A C-C chemokine expressed by all tissues that functions as a chemoattractant for EOSINOPHILS and BASOPHILS. It binds to the CCR3 RECEPTOR.'),
  'D000075923': NtO(MH='Wildfires', MS='Fires that occur outdoors in the natural environment caused by natural forces or human activity.'),
  'D000075924': NtO(MH='X-linked Nuclear Protein', MS='ATP-dependent DNA helicase that contains two N-terminal ZINC FINGERS and C-terminal ATP-binding and helicase domains. It functions in the regulation of gene transcription and CHROMATIN REMODELING. ATRX undergoes cell-cycle dependent phosphorylation, which causes it to translocate from the NUCLEAR MATRIX to CHROMATIN; thus, it may change its role from gene regulation during INTERPHASE to ensuring proper chromosome segregation at MITOSIS. Mutations in the ATRX gene are associated with cases of X-LINKED MENTAL RETARDATION co-morbid with ALPHA-THALASSEMIA (ATRX syndrome).'),
  'D000075925': NtO(MH='PHD Zinc Fingers', MS='Zinc finger domains of approximately 50 to 80 amino acids in length that are characterized by a conserved Cysteine(3)-Histidine-Cysteine(4) amino acid motif which coordinates binding of two zinc ions. They are similar structurally to RING FINGER DOMAINS, with a globular fold topology of two BETA-SHEETS and an ALPHA-HELIX. PHD fingers occur in many proteins that function in chromatin-mediated gene expression and EPIGENETICS such as POLYCOMB-GROUP PROTEINS.'),
  'D000075926': NtO(MH='FERM Domains', MS='Widely occuring protein domains that function to link signaling and CYTOSKELETAL PROTEINS to the PLASMA MEMBRANE. They occur mostly at the N-terminal region of proteins that contain them.'),
  'D000075927': NtO(MH='KRIT1 Protein', MS='A microtubule-associated protein consisting of four ANKYRIN REPEATS and a C-terminal FERM DOMAIN. It links the CYTOSKELETON to CELL JUNCTIONS via integrin cytoplasmic domain-associated protein-1 and plays an important role in regulating cell proliferation and integrity of endothelial cell junctions. It is also involved in REACTIVE OXYGEN SPECIES metabolism. Mutations in the KRIT1 gene are associated with type I CEREBRAL CAVERNOUS MALFORMATIONS.'),
  'D000075942': NtO(MH='Intraepithelial Lymphocytes', MS='T Lymphocytes with limited diversity of receptors (e.g., ALPHA E INTEGRINS) in the epidermis of the skin and the mucosal linings. They recognize common microbes via T-CELL RECEPTORS and PATHOGEN-ASSOCIATED MOLECULAR PATTERN MOLECULES and function as effector cells for INNATE IMMUNITY. Activation of intraepithelial lymphocytes is a marker for various gastrointestinal diseases (e.g., CELIAC DISEASE; HAIRY CELL LEUKEMIA; and ENTEROPATHY-ASSOCIATED T-CELL LYMPHOMA).'),
  'D000075943': NtO(MH='Apolipoproteins L', MS='A family of apolipoproteins occurring in humans that are structurally similar to B-CELL LEUKEMIA 2 FAMILY PROTEINS. In addition to their roles in cholesterol and lipid transport, they are expressed by MYELOID CELLS and ENDOTHELIAL CELLS during INFLAMMATION and may function to promote CELL DEATH.'),
  'D000075944': NtO(MH='Apolipoprotein L1', MS='An ApoL protein highly expressed by the liver. It has anti-trypanosomal activity through its ability to permeabilize TRYPANOSOMA membranes. Mutations in the APOL1 gene are associated with type 4 FOCAL SEGMENTAL GLOMERULOSCLEROSIS .'),
  'D000075982': NtO(MH='Cell Adhesion Molecule-1', MS='A cell adhesion molecule that contains extracellular immunoglobulin V and C2 domains. It mediates homophilic and heterophilic cell-cell adhesion independently of calcium, and acts as a tumor suppressor in NON-SMALL-CELL LUNG CANCER (NSCLC) cells. Its interaction with NATURAL KILLER CELLS is important for their cytotoxicity and its expression by MAST CELLS plays a role in their interaction with neurons; it may also function in synapse assembly, nerve growth and differentiation.'),
  'D000075983': NtO(MH='Nectins', MS='A family of calcium-independent cell adhesion molecules of the immunoglobulin superfamily. They are expressed by most cell types and mediate both homotypic and heterotypic cell-cell adhesion. Nectins function in a variety of morphogenetic and developmental processes that include organogenesis of the eye, ear, tooth, and cerebral cortex; they also play roles in viral infection and cell proliferation.'),
  'D000076': NtO(MH='Acetabularia', MS='A genus of green algae found in the Mediterranean and other warm seas.'),
  'D000076002': NtO(MH='Hyaluronan Synthases', MS='Membrane-associated glucuronosyltransferases that catalyze the reaction of UDP-N-acetyl-D-glucosamine and UDP-D-glucuronate to produce HYALURONAN. HYALURONAN SYNTHASE 2 (HAS2) is essential for embryogenesis and its expression by tumor cells is associated with metastasis.'),
  'D000076023': NtO(MH='Inhibitor of Growth Protein 1', MS='A nuclear protein and tumor suppressor that contains a C-terminal PHD ZINC FINGER. It is expressed in different isoforms in various tissues and interacts with TUMOR SUPPRESSOR PROTEIN P53 to negatively regulate cell growth. Reduced expression and chromosomal rearrangements of the ING1 gene are associated with different cancers including HEAD AND NECK NEOPLASMS.'),
  'D000076042': NtO(MH='Alcoholic Korsakoff Syndrome', MS='A neurological disorder characterized by inattentiveness and the inability to form short term memories. It is caused by THIAMINE DEFICIENCY due to chronic ALCOHOLISM.'),
  'D000076064': NtO(MH='Drug Misuse', MS='Use of a drug for a purpose not consistent with legal or medical guidelines.'),
  'D000076082': NtO(MH='Travel-Related Illness', MS='Health problems associated with TRAVEL.'),
  'D000076083': NtO(MH='Involuntary Treatment, Psychiatric', MS="Treatment of persons with MENTAL DISORDERS without the persons' consent."),
  'D000076084': NtO(MH='Ubiquitin-Specific Peptidase 7', MS='A ubiquitinyl hydrolase that deubiquitinates several proteins with critical roles in DNA REPAIR, cell growth, and survival, including TUMOR SUPPRESSOR PROTEIN P53; MDM-2 PROTEIN; and PTEN PHOSPHOHYDROLASE. It also stabilizes herpesvirus 1 trans-acting transcriptional protein VMW110 during HSV-1 infection, contributing to its function as a TRANS-ACTIVATOR.'),
  'D000076103': NtO(MH='Biological Variation, Individual', MS='Differences in measurable biological values, characteristics, or traits, within one individual under different conditions for the individual such as fasting, season of the year, age, or state of wellness.'),
  'D000076104': NtO(MH='Oligodendrocyte Transcription Factor 2', MS='A basic helix-loop-helix transcription factor that is required for differentiation of OLIGODENDROCYTES and motor neurons in the spinal cord, and development of somatic motor neurons in the hindbrain.'),
  'D000076105': NtO(MH='X-ray Repair Cross Complementing Protein 1', MS='A poly(ADP)-ribose-binding protein that functions in the rejoining of DNA single-strand breaks that arise following treatment with alkylating agents or ionizing radiation. It interacts with DNA LIGASE III and POLY ADP RIBOSE POLYMERASE in BASE EXCISION REPAIR, and may also function in DNA processing and chromosome recombination in GERM CELLS.'),
  'D000076106': NtO(MH='SAM Domain and HD Domain-Containing Protein 1', MS='A host restriction triphosphorylhydrolase and dNTPase that contains an N-terminal STERILE ALPHA MOTIF and central, conserved ASPARTATE and HISTIDINE (HD) domain. It acts on single-stranded RNA, yielding deoxynucleosides and triphosphate, and functions in anti-viral defense through its dNTPase activity, reducing cellular dNTP levels below what is required for retroviral reverse transcription in DENDRITIC CELLS and MYELOID CELLS. It also has RIBONUCLEASE activity which blocks early replication of retroviruses such as HIV-1. Mutations in the SAMHD1 gene are associated with type 5 Aicardi-Goutieres syndrome (AGS5) and type 2 chilblain LUPUS (CHBL2).'),
  'D000076107': NtO(MH='Diet, High-Protein Low-Carbohydrate', MS='A diet that consists mainly of foods with a high content of protein and limited amounts of CARBOHYDRATES.'),
  'D000076122': NtO(MH='Disks Large Homolog 4 Protein', MS='A neuronal protein consisting of three PDZ DOMAINS, an SH3 DOMAIN, and a C-terminal guanylate kinase-like region (see MAGUK PROTEINS). It localizes to the POST-SYNAPTIC DENSITY and associates with the cytoplasmic tail of NMDA RECEPTORS and SHAKER POTASSIUM CHANNELS, playing a critical role in NMDA receptor-mediated SYNAPTIC PLASTICITY.'),
  'D000076123': NtO(MH='DNA (Cytosine-5-)-Methyltransferase 1', MS='A DNA (cytosine-5-)-methyltransferase that contains a central CxxC type zinc finger motif. It binds poly(ADP)-ribose and its expression is regulated by POLY (ADP-RIBOSE) POLYMERASE-1. DNMT1 methylates CpG residues, with a preference for hemimethylated DNA, and associates with DNA replication sites in S PHASE to maintain the methylation pattern in the newly synthesized strand, which is essential for EPIGENETIC PROCESSES. It also associates with CHROMATIN during G2 PHASE and MITOSIS to maintain DNA methylation independently of replication. It is responsible for maintaining methylation patterns established in development; mutations in the DNMT1 gene are associated with HEREDITARY SENSORY NEUROPATHY TYPE 1 class E.'),
  'D000076142': NtO(MH='Virtual Reality', MS="Using computer technology to create and maintain an environment and project a user's physical presence in that environment allowing the user to interact with it."),
  'D000076144': NtO(MH='Egypt, Ancient', MS='A civilization of ancient Northeastern Africa, concentrated along the lower reaches of the Nile River in the area of Egypt. It was active from 3100 B.C. until its conquest by Alexander the Great in 332 B.C.'),
  'D000076145': NtO(MH='Greece, Ancient', MS='A civilization extant from about 1200 BC. to the death of Alexander the Great, in 323 B.C. It extended from the Greek city states to North Africa and eastward to the Indus River.'),
  'D000076162': NtO(MH='Therapeutic Index', MS='An indicator of the benefits and risks of treatment.'),
  'D000076164': NtO(MH='Fanconi Anemia Complementation Group N Protein', MS='A Fanconi anemia complementation group protein that contains an N-terminal DNA-binding region and seven, C-terminal, WD REPEATS. It is an essential factor in HOMOLOGOUS RECOMBINATION DNA REPAIR through its interactions with BRCA2 PROTEIN; RAD51 RECOMBINASE; and BRCA1 PROTEIN. It functions as a molecular scaffold to localize and stabilize these proteins at homologous recombination sites. Mutations in the PALB2 gene are associated with FANCONI ANEMIA complementation group N; type 3 PANCREATIC NEOPLASMS; and susceptibility to BREAST CANCER.'),
  'D000076182': NtO(MH='Sp7 Transcription Factor', MS='An Sp transcription factor that contains three CYS2-HIS2 ZINC FINGERS. It binds to GC RICH SEQUENCES and performs an essential function in regulating gene expression for differentiation of OSTEOBLASTS. Mutations in the SP7 gene are associated with type 12 OSTEOGENESIS IMPERFECTA.'),
  'D000076183': NtO(MH='Tripartite Motif-Containing Protein 28', MS='A tripartite motif protein consisting of an N-terminal RING finger, two B-box type ZINC FINGERS, and C-terminal PHD domain.  It functions as a transcriptional repressor by associating with Kruppel-association box domain (KRAB domain) transcription factors and has E3-SUMO-ligase activity towards itself and also sumoylates INTERFERON REGULATORY FACTOR-7 to reduce its activity as a transcriptional activator. It can also function as a ubiquitin protein ligase towards TUMOR SUPPRESSOR PROTEIN P53.'),
  'D000076202': NtO(MH='c-Mer Tyrosine Kinase', MS='A receptor tyrosine kinase that transduces signals from EXTRACELLULAR MATRIX to the CYTOPLASM by binding ligands such as GALECTIN 3. It regulates many physiologic processes that include cell survival, migration, differentiation, and PHAGOCYTOSIS of apoptotic cells and ROD PHOTORECEPTORS in the RETINAL PIGMENT EPITHELIUM. Mutations in the MERTK gene are associated with type 38 RETINITIS PIGMENTOSA; it also plays a critical role as an inhibitor of TOLL-LIKE RECEPTORS signaling.'),
  'D000076203': NtO(MH='Animal Fur', MS='Usually densely-packed hairs on the skin of MAMMALS.'),
  'D000076204': NtO(MH='WNK Lysine-Deficient Protein Kinase 1', MS='A serine-threonine kinase which plays an important role in the regulation of electrolyte homeostasis, cell signaling, survival, and proliferation. It functions as an activator and inhibitor of sodium-coupled chloride co-transporters and as an inhibitor of potassium-coupled chloride co-transporters. Mutations in the WNK1 gene are associated with type 2C PSEUDOHYPOALDOSTERONISM and type 2A HEREDITARY SENSORY AND AUTONOMIC NEUROPATHIES.'),
  'D000076205': NtO(MH='Ras Homolog Enriched in Brain Protein', MS='A GTP-binding protein with low intrinsic GTPase activity that activates MTORC1 protein kinase activity.'),
  'D000076206': NtO(MH='Patient Generated Health Data', MS='Health-related data created, recorded, or gathered by patients, family members, or caregivers, to help address a health concern. Distinct from data generated in clinical settings and through encounters with providers.'),
  'D000076207': NtO(MH='Popular Culture', MS='Choices and ways of doing things that predominate or are fashionable among ordinary people in a society during a point in time.'),
  'D000076222': NtO(MH='Mechanistic Target of Rapamycin Complex 1', MS='An evolutionarily conserved multiprotein complex that functions as a cellular energy sensor and regulator of protein synthesis for cell growth and proliferation. It consists of TOR SERINE-THREONINE KINASES; REGULATORY-ASSOCIATED PROTEIN OF MTOR (RAPTOR); MLST8 PROTEIN; and AKT1 substrate 1 protein. The activity of the complex is regulated by SIROLIMUS; INSULIN; GROWTH FACTORS; PHOSPHATIDIC ACIDS; some amino acids or amino acid derivatives, and OXIDATIVE STRESS.'),
  'D000076223': NtO(MH='Regulatory-Associated Protein of mTOR', MS='An adaptor protein component of the MECHANISTIC TARGET OF RAPAMYCIN COMPLEX 1 that forms stoichiometric complexes with TOR KINASES, which it negatively regulates. It functions as a positive regulator of RIBOSOMAL PROTEIN S6 KINASES.'),
  'D000076225': NtO(MH='Mechanistic Target of Rapamycin Complex 2', MS='A multiprotein complex consisting of mTOR KINASE; MLST8 PROTEIN;  rapamycin-insensitive companion of mTOR protein (RICTOR PROTEIN); and PRR5 (proline-rich protein 5). Like mTORC1, it also regulates cell growth and proliferation in response to growth factors but may not be as sensitive to nutrient availability and is insensitive to SIROLIMUS. In contrast to mTORC1, it can regulate the ACTIN CYTOSKELETON through RHO GTPASES to promote the formation of STRESS FIBERS. The mTORC2 complex also plays a critical role in AKT1 PROTEIN KINASE phosphorylation and activation.'),
  'D000076226': NtO(MH='Rapamycin-Insensitive Companion of mTOR Protein', MS='An adaptor protein subunit of MTORC2 COMPLEX. It functions as a structural component and is phosphorylated by RIBOSOMAL PROTEIN S6 KINASES, integrating signals for cell growth and proliferation, especially during embryonic development.'),
  'D000076227': NtO(MH='Stereolithography', MS='A 3D printing technology where a computer-controlled moving laser beam is used to build up the required structure, layer by layer, from liquid POLYMERS that harden on contact with laser light (photopolymerization).'),
  'D000076228': NtO(MH='MRE11 Homologue Protein', MS="A component of the MRN complex along with Rad50 and Nibrin. Together, these perform a critical function in the repair of DOUBLE-STRANDED DNA BREAKS; RECOMBINATIONAL DNA REPAIR; maintenance of TELOMERE integrity and MEIOSIS. MRE11, which contains a poly(ADP)-ribose binding motif and associates with PARP1, possesses single-strand endonuclease activity and double-strand-specific 3'-5' exonuclease activity. Mutations in the MRE11 gene are associated with ATAXIA-TELANGIECTASIA-like disorder 1."),
  'D000076242': NtO(MH='mTOR Associated Protein, LST8 Homolog', MS='An adaptor protein, consisting of seven WD REPEATS along its length, that functions as a component of the MECHANISTIC TARGET OF RAPAMYCIN COMPLEX 1 and mTORC2 COMPLEX. It interacts directly with mTOR to enhance its kinase activity and stabilizes the mTOR-RPTOR PROTEIN interaction in nutrient-poor conditions, favoring RPTOR inhibition of mTOR activity.'),
  'D000076244': NtO(MH='Lepisma', MS='A small nocturnal wingless insect with tapering physical outline.'),
  'D000076245': NtO(MH='Heterogeneous Nuclear Ribonucleoprotein A1', MS='A heterogeneous ribonucleoprotein that contains an RNA-BINDING MOTIF and has poly(ADP)ribose-binding capability. It functions in the packaging of pre-mRNA into hnRNP particles for export to the cytoplasm and may play a role in RNA SPLICING site selection.'),
  'D000076246': NtO(MH='CCCTC-Binding Factor', MS='A repressor protein with poly(ADP)-ribose binding activity that binds CHROMATIN and DNA; its structure consisting of 11 CYS2-HIS2 ZINC FINGERS allows it to recognize many different DNA target sites. It functions as a repressor by binding to INSULATOR ELEMENTS and preventing interaction between promoters and nearby enhancers and silencers. It plays a critical role in EPIGENETIC PROCESSES, including GENOMIC IMPRINTING.'),
  'D000076247': NtO(MH='Centromere Protein A', MS='A 17 kDa, centromeric, poly(ADP)-ribose binding protein that is structurally similar to HISTONE H3 and localizes to NUCLEOSOMES within the CENTROMERE; specifically within the region that binds KINETOCHORES, where it replaces histone H3. It plays a critical role in recruiting kinetochore proteins and progression through MITOSIS, chromosome segregation, and CYTOKINESIS. It also binds sera from patients with some scleroderma-like AUTOIMMUNE DISEASES.'),
  'D000076248': NtO(MH='Sida Plant', MS='A genus of flowering plants in the family of mallows (MALVACEAE). Their common name is fanpetals.'),
  'D000076249': NtO(MH='Heat Shock Transcription Factors', MS="Heat and cold stress-inducible, transcription factors that bind to inverted 5'-NGAAN-3' pentamer DNA sequences and are regulated by poly(ADP) ribosylation. They play essential roles as transcriptional activators of the HEAT-SHOCK RESPONSE by inducing expression of large classes of MOLECULAR CHAPERONES and heat-shock proteins. They also function in DNA REPAIR; transcriptional reactivation of latent HIV-1; and pre-mRNA processing and nuclear export of HSP70 HEAT-SHOCK PROTEINS during heat stress."),
  'D000076250': NtO(MH='Myristoylated Alanine-Rich C Kinase Substrate', MS='A membrane and ACTIN CYTOSKELETON associated, N-terminal myristoylated protein that binds CALMODULIN and is a prominent substrate for PROTEIN KINASE C. Both phosphorylation and poly(ADP)-ribosylation inhibit its F-ACTIN crosslinking activity; phosphorylation also causes MARCKS to relocate from the membrane to cytoplasm.'),
  'D000076251': NtO(MH='Wearable Electronic Devices', MS='Electronic implements worn on  the body as an implant or as an accessory. Examples include wearable diagnostic devices, wearable ACTIVITY TRACKERS, wearable INFUSION PUMPS, wearable computing devices, SENSORY AIDS, and electronic pest repellents.'),
  'D000076263': NtO(MH='Communicable Diseases, Imported', MS='Infectious diseases originating in one geographically delineated ecosystem that are carried (by travel or immigration) to another geographically delineated ecosystem by an infected individual, animal, or disease vector.'),
  'D000076282': NtO(MH='Composting', MS='A process of waste disposal involving the conversion of green waste (i.e. leaves, organic matter, food waste, manure) into soil-enhancing matter.'),
  'D000076302': NtO(MH='Citrullination', MS='Conversion of ARGININE residues in proteins into CITRULLINE residues by PROTEIN-ARGININE DEIMINASES.'),
  'D000076322': NtO(MH='Multimorbidity', MS='The complex interactions of several co-existing diseases.'),
  'D000076342': NtO(MH='Protein-Arginine Deiminases', MS='A family of ENZYMES that, in the presence of calcium ion, converts ARGININE to CITRULLINE in proteins. There are five PAD isotypes in mammals.  In humans: they include PAD1, 2, 3, 4 and 6. They are encoded by five paralogous genes named PADI and clustered on human chromosome 1.'),
  'D000076362': NtO(MH='Adaptive Clinical Trial', MS='Clinical study in which a prospectively planned opportunity is included to modify trial designs and hypotheses based on analysis of data from subjects in the study.'),
  'D000076363': NtO(MH='Academic Success', MS='Level of engagement in educationally purposeful activities, satisfaction, acquisition of desired knowledge, skills, and competencies, persistence and attainment of educational outcomes.'),
  'D000076385': NtO(MH='Diverticular Diseases', MS='Diseases of the DIVERTICULUM often due to infection and/or inflammation (DIVERTICULITIS).'),
  'D000076482': NtO(MH='Daphniphyllum', MS='Broad-leaved evergreen shrub or small tree comprising several species. It is the only genus of the family DAPHNIPHYLLACEAE, class MAGNOLIOPSIDA.'),
  'D000076502': NtO(MH='Sustainable Development', MS='A process of change in which the exploitation of resources, the direction of investments, the orientation of technological development; and institutional change are all in harmony and enhance both current and future potential to meet human needs and aspirations. (http://www.un-documents.net/ocf-02.htm#I)'),
  'D000076522': NtO(MH='Retention in Care', MS="Patient's regular engagement with medical care after initial entry into the system."),
  'D000076542': NtO(MH='Clinical Observation Units', MS='A hospital unit that serves as a dedicated observation area for emergency room patients when a decision to admit or not is unclear. Also known as clinical decision units or short-stay observation units.'),
  'D000076562': NtO(MH='Nurses Improving Care for Health System Elders', MS='Nonprofit programs designed to address the complex needs of older adults and improve outcomes. They provide the principles, education and tools to support healthcare systems in transforming and achieving patient-centered care.'),
  'D000076563': NtO(MH='Mentalization', MS='The mental process by which an individual implicitly and explicitly interprets the actions of self and others as meaningful on the basis of intentional mental states such as personal desires, needs, feelings, beliefs, and reasons.'),
  'D000076602': NtO(MH='Population Health Management', MS='Community engagement, facility management, and organization, access to care that is timely and affordable; the availability of effective services, and high-quality primary health care. (World Health Organization)'),
  'D000076603': NtO(MH='Syndemic', MS='Synergistically-interacting health conditions producing excess disease occurrence in a population.'),
  'D000076604': NtO(MH='Physical Functional Performance', MS=None),
  'D000076605': NtO(MH='Disenfranchised Grief', MS='Expression of grief that society limits or may not allow a person to express. Examples include the grief of parents for stillborn babies, of teachers for the death of students, of nurses for the death of patients, or for the loss of an animal companion. (from APA Dictionary of Psychology, 2nd ed.)'),
  'D000076607': NtO(MH='Saxifragales', MS='Order of dicotyledonous flowering plants consisting of 16 families, 112 genera, and nearly 2,500 species.'),
  'D000076608': NtO(MH='Daphniphyllaceae', MS='Vascular plants of the order SAXIFRAGALES, class MAGNOLIOPSIDA, of which the sole genus is DAPHNIPHYLLUM.'),
  'D000076609': NtO(MH='Brugmansia', MS='A genus of solanaceae that refers to small to mid-size trees with large, strong-smelling, trumpet-shaped flowers hanging downward. Most parts of the plant contain ATROPINE, SCOPOLAMINE HYDROBROMIDE and HYOSCYAMINE.'),
  'D000076610': NtO(MH='Genetic Profile', MS='A set of characteristics unique to the DNA of an individual organism or population obtained by analyzing small samples of biological material such as body tissues or fluids, used especially in the context of predicting susceptibility to a particular disease. For example, some populations have identifiable genetic profiles which put them at higher risk for cancer. Distinguish from GENETIC BACKGROUND and GENETIC VARIATION.'),
  'D000076622': NtO(MH='Malpighiales', MS='Order of flowering plants which includes many tropical species. Well-characterized members include SALICACEAE (willow family), VIOLACEAE (violet family), PASSIFLORACEAE (passion-flower family), EUPHORBIACEAE (spurge or croton family), RHIZOPHORACEAE (mangrove family), and ERYTHROXYLACEAE (coca family).'),
  'D000076623': NtO(MH='Flacourtia', MS='A genus of flowering plants in the family SALICACEAE, order MALPHIGIALES. An example is the willow, also known as salix or saules (French).'),
  'D000076624': NtO(MH='Built Environment', MS="The man-made physical elements of the environment (e.g., homes, buildings, streets, open spaces, and infrastructure). The built environment influences a person's level of physical activity and may have other impacts on health in a community."),
  'D000076625': NtO(MH='Dicentra', MS='A genus of herbaceous plants with asymmetrically-shaped flowers and finely divided leaves, native to east Asia and North America.'),
  'D000076642': NtO(MH='Desulfovibrionales', MS='An order of bacteria within the class DELTAPROTEOBACTERIA, of the phylum PROTEOBACTERIA, which comprises four families.'),
  'D000076643': NtO(MH='Maesa', MS='A genus of flowering plants in the family PRIMULACEAE, order ERICALES.'),
  'D000076644': NtO(MH='Myrsine', MS='A genus of flowering plants in the family PRIMULACEAE, found worldwide but primarily in tropical and subtropical regions.'),
  'D000076662': NtO(MH='Host Microbial Interactions', MS='Interactions between a host and microbe or microbiota.'),
  'D000076663': NtO(MH='Endurance Training', MS='Exercise programs of relatively long duration and moderate intensity, which enhance maximal oxygen uptake, increasing PHYSICAL STAMINA.'),
  'D000076664': NtO(MH='Ranunculales', MS='An order of flowering plants also known as the buttercup family.'),
  'D000076665': NtO(MH='Ventilation-Perfusion Scan', MS='A two-part radionuclide imaging that measures VENTILATION-PERFUSION RATIO of the lungs often used to investigate PULMONARY EMBOLISM. Image acquisitions may be performed using various combinations of gamma camera, PET and SPECT scanning techniques.'),
  'D000076666': NtO(MH='Solanales', MS='An order of flowering plants comprising tropical and temperate species, major crop plants such as potatoes, tomatoes, bell peppers, eggplants, tobacco, and the garden petunia. Its largest family is the SOLANACEAE.'),
  'D000076667': NtO(MH='Austrobaileyales', MS='Order of woody flowering plants which grow as trees and shrubs that include the genera, Illicium and Schisandra.'),
  'D000076682': NtO(MH='Preprints as Topic', MS='Works about scientific manuscripts made available prior to PEER REVIEW.'),
  'D000076702': NtO(MH='International Health Regulations', MS='Regulations to prevent, protect against, control and provide a public health response to the international spread of disease in ways that are commensurate with and restricted to public health risks. (www.who.int/features/qa/39/en)'),
  'D000076722': NtO(MH='Drug Development', MS='The entire process of bringing a new drug to the market. It includes both preclinical and clinical testing, and regulatory approval.'),
  'D000076723': NtO(MH='Negative Results', MS='Subject matter related to research studies in which the data do not demonstrate any clear evidence of effect, especially if an effect was expected.'),
  'D000076724': NtO(MH='Collaborative Cross Mice', MS='Recombinant inbred strains of mice derived from an eight-way cross of eight genetically diverse, founder inbred strains, as a reference population for analyzing MULTIFACTORIAL INHERITANCE.'),
  'D000076742': NtO(MH='Synthetic Drugs', MS='Legal or illegal drugs created using SYNTHETIC CHEMISTRY TECHNIQUES.'),
  'D000076764': NtO(MH='Traffic-Related Pollution', MS='Contamination of the ENVIRONMENT by VEHICLE EMISSIONS.'),
  'D000076782': NtO(MH='Race Factors', MS='A constituent element or influence that could be used in studies for racial disparities as variables such as those related to risk factors and disease and or differential access to services.'),
  'D000076783': NtO(MH='Adverse Childhood Experiences', MS='Deleterious experiences in first 18 years of life.'),
  'D000076822': NtO(MH='Pinealectomy', MS='Surgical removal of the PINEAL GLAND including resection of pineal tumor (PINEALOMA) and CYSTS.'),
  'D000076862': NtO(MH='Diagnostic Screening Programs', MS='Services for the presumptive identification of unrecognized disease in an apparently healthy, asymptomatic population by means of tests, examinations or other procedures that can be applied rapidly and easily to the target population. (from World Health Organization http://www.who.int/cancer/prevention/diagnosis-screening/screening/en/ Oct. 24,2017)'),
  'D000076889': NtO(MH='Gray Literature', MS='Works that include publications such as reports, theses, conference papers, translations and limited circulation government documents which are not normally available through commercial publication sources. (LCSH)'),
  'D000076892': NtO(MH='UNESCO', MS='An operating body of the UNITED NATIONS, responsible for coordinating international cooperation in education, science, culture and communication.'),
  'D000076942': NtO(MH='Preprint', MS='Scientific manuscript made available prior to PEER REVIEW.'),
  'D000076944': NtO(MH='Rickettsiales', MS='Order in class ALPHAPROTEOBACTERIA that are endosymbionts in eukaryotic cells. High morbidity and mortality are frequently associated with Rickettsiales transmitted by INSECT VECTORS.'),
  'D000076962': NtO(MH='Receptors, Chimeric Antigen', MS='Synthetic cellular receptors that reprogram T-LYMPHOCYTES to selectively bind antigens.'),
  'D000076983': NtO(MH='Histone Methyltransferases', MS='Enzymes that catalyze the transfer of methyl groups to LYSINE or ARGININE residues of HISTONES, especially histone H3 and histone H4 proteins. They play a critical role in EPIGENETIC PROCESSES.'),
  'D000076984': NtO(MH='Neonatal Brachial Plexus Palsy', MS='Perinatal nerve injury involving the BRACHIAL PLEXUS involving a weak or paralyzed upper extremity.'),
  'D000076985': NtO(MH='Saporins', MS='Type 1 ribosome-inactivating proteins derived from SAPONARIA OFFICINALIS that function through endohydrolysis of the N-glycosidic bond at single ADENOSINE residues of 28S RIBOSOMAL RNA. They are used as IMMUNOTOXINS.'),
  'D000076987': NtO(MH='CRISPR-Associated Protein 9', MS='An RNA-guided endodeoxyribonuclease that associates with CRISPR SEQUENCES in STREPTOCOCCUS PYOGENES and other bacteria where it participates in an adaptive immune function to cleave foreign DNA complimentary to small GUIDE RNA (sgRNAs). Structurally, Cas9 consists of an ALPHA-HELIX module and a nuclease module connected by a single helix. The nuclease module contains two enzymatic domains: RuvC, which cleaves non-target DNA strand, and an HNH nuclease domain, which cleaves the target strand. Specificity for the DNA target depends on the presence of a protospacer adjacent motif (PAM) sequence, a 2-6 nucleotide DNA sequence immediately following the sequence targeted by Cas9.'),
  'D000076988': NtO(MH='Sulfate Transporters', MS='Highly conserved families of transmembrane anion transporters that occur in both prokaryotes and eukaryotes. In humans, they are expressed by epithelial cells in a variety of tissues and participate in diverse physiologic functions such as cell motility and regulating the composition and acidity of body fluids. Some Slc13 family proteins function as sodium ion-coupled co-transporters of sulfate and other anions.'),
  'D000077': NtO(MH='Acetabulum', MS='The part of the pelvis that comprises the pelvic socket where the head of FEMUR joins to form HIP JOINT (acetabulofemoral joint).'),
  'D000077002': NtO(MH='GRADE Approach', MS='A common, sensible, structured  and transparent approach to grading quality (or certainty) of evidence and strength of recommendations developed by the Grading of Recommendations Assessment, Development and Evaluation (short GRADE) working group.'),
  'D000077003': NtO(MH='Sodium Sulfate Cotransporter', MS='A multi-pass protein that consists of 13 transmembrane helices. It is expressed primarily by renal epithelial cells and functions as a co-transporter of sodium and sulfate, to mediate sulfate reabsorption in the kidney.'),
  'D000077004': NtO(MH='Tuberous Sclerosis Complex 1 Protein', MS='An intracellular signaling and tumor suppressor protein that forms a complex with TUBEROUS SCLEROSIS COMPLEX 2 PROTEIN (TSC2) and other signaling factors to negatively regulate MTORC1 signaling and affect cell growth and proliferation. Structurally, it interacts with TSC2 through its N-terminal, which also contains GSK-3BETA phosphorylation sites and a RHO-KINASE activation domain. It also contains a C-terminal coiled-coil domain and ezrin-radixin-moesin (ERM) domain. Mutations in the TSC1 gene are associated with TUBEROUS SCLEROSIS.'),
  'D000077005': NtO(MH='Tuberous Sclerosis Complex 2 Protein', MS='An intracellular signaling and tumor suppressor protein that forms a complex with TUBEROUS SCLEROSIS COMPLEX 1 PROTEIN (TSC1) and other signaling factors to negatively regulate MTORC1 and affect cell growth and proliferation. It can also function as GTPASE-ACTIVATING PROTEIN (GAP) for RHEB GTPASE to activate mTORC1 independent of its role in the complex. Structurally, it interacts with TSC1 through its N-terminus, which also contains a leucine zipper and coiled-coil region. It also has multiple phosphorylation sites for different cell signaling kinases, a central coiled-coil region, a C-terminal GAP domain and CALMODULIN binding domain. Mutations in the TSC2 gene are associated with TUBEROUS SCLEROSIS.'),
  'D000077022': NtO(MH='Survivin', MS='An apoptosis inhibitory protein that contains a single baculoviral IAP repeat (BIR) domain. It associates with MICROTUBULES and functions to regulate cell proliferation as a component of the chromosome passage protein complex (CPC), performing essential roles for localization of the complex, chromosome alignment, segregation during MITOSIS and CYTOKINESIS, and assembly of the MITOTIC SPINDLE. It is expressed by fetal kidney and liver cells and highly expressed in ADENOCARCINOMA and high-grade LYMPHOMA.'),
  'D000077062': NtO(MH='Burnout, Psychological', MS="An excessive reaction to stress caused by one's environment that may be characterized by feelings of emotional and physical exhaustion, coupled with a sense of frustration and failure."),
  'D000077085': NtO(MH='Birth Setting', MS='Physical setting or location of birth.'),
  'D000077102': NtO(MH='Apium', MS='A genus comprising several species of flowering plants with global distribution in the family APIACEAE.'),
  'D000077105': NtO(MH='Chlorophyceae', MS='A class of green algae in the subdivision CHLOROPHYTINA distinguished by its ultrastructural morphology. Some species have distinct arrangement of flagella.'),
  'D000077107': NtO(MH='Gait Analysis', MS='Determination of the level of functional limitation that may be due to pathology, and evaluating rehabilitative intervention of the way in which the body moves from one point to another. Most often, this is done by walking.'),
  'D000077108': NtO(MH='Public Expenditures', MS='Expenses of public funds incurred by government bodies:  state, regional and local.'),
  'D000077109': NtO(MH='Disability Studies', MS='Interdisciplinary programs of academic study that de-stigmatize disease, illness, and impairment.'),
  'D000077122': NtO(MH='Sugammadex', MS='A gamma-cyclodextrin that functions as a reversal agent for the neuromuscular blocker ROCURONIUM BROMIDE.'),
  'D000077123': NtO(MH='Rocuronium', MS='An androstanol non-depolarizing neuromuscular blocking agent. It has a mono-quaternary structure and is a weaker nicotinic antagonist than PANCURONIUM.'),
  'D000077142': NtO(MH='Malvales', MS='An order of mostly woody plants in the superorder ROSANAE. Members of the MALVALES grow in various habitats around the world.'),
  'D000077143': NtO(MH='Docetaxel', MS='A semisynthetic analog of PACLITAXEL used in the treatment of locally advanced or metastatic BREAST NEOPLASMS and NON-SMALL CELL LUNG CANCER.'),
  'D000077144': NtO(MH='Clopidogrel', MS='A ticlopidine analog and platelet purinergic P2Y receptor antagonist that inhibits adenosine diphosphate-mediated PLATELET AGGREGATION. It is used to prevent THROMBOEMBOLISM in patients with ARTERIAL OCCLUSIVE DISEASES; MYOCARDIAL INFARCTION; STROKE; or ATRIAL FIBRILLATION.'),
  'D000077145': NtO(MH='Sulfanilamide', MS='A short-acting sulfonamide used as an anti-infective agent. It has lower anti-bacterial activity than SULFAMETHOXAZOLE.'),
  'D000077146': NtO(MH='Irinotecan', MS='A semisynthetic camptothecin derivative that inhibits DNA TOPOISOMERASE I to prevent nucleic acid synthesis during S PHASE. It is used as an antineoplastic agent for the treatment of COLORECTAL NEOPLASMS and PANCREATIC NEOPLASMS.'),
  'D000077148': NtO(MH='Myrtales', MS='Order of flowering plants distributed throughout the tropics and warmer regions of the world. The majority of these species belong to two families, Melastomataceae and Myrtaceae. Myrtales includes many trees (notably Eucalyptus), shrubs, such as the classic myrtle, several food, spices, and ornamental plants.'),
  'D000077149': NtO(MH='Sevoflurane', MS='A non-explosive inhalation anesthetic used in the induction and maintenance of general anesthesia. It does not cause respiratory irritation and may also prevent PLATELET AGGREGATION.'),
  'D000077150': NtO(MH='Oxaliplatin', MS='An organoplatinum complex in which the platinum atom is complexed with 1,2-diaminocyclohexane, and with an oxalate ligand which is displaced to yield active oxaliplatin derivatives. These derivatives form inter- and intra-strand DNA crosslinks that inhibit DNA replication and transcription. Oxaliplatin is an antineoplastic agent that is often administered with FLUOROURACIL and FOLINIC ACID in the treatment of metastatic COLORECTAL NEOPLASMS.'),
  'D000077152': NtO(MH='Olanzapine', MS='A benzodiazepine derivative that binds SEROTONIN RECEPTORS; MUSCARINIC RECEPTORS; HISTAMINE H1 RECEPTORS; ADRENERGIC ALPHA-1 RECEPTORS; and DOPAMINE RECEPTORS. It is an antipsychotic agent used in the treatment of SCHIZOPHRENIA; BIPOLAR DISORDER; and MAJOR DEPRESSIVE DISORDER; it may also reduce nausea and vomiting in patients undergoing chemotherapy.'),
  'D000077153': NtO(MH='Progranulins', MS='Cysteine-rich, precursor glycoproteins encoded by the granulin gene (GRN). They contain granulin-repeats separated by linker sequences that are cleaved by a variety of proteases to release the individual GRANULIN PEPTIDES. The mammalian progranulin is composed of a half-granulin repeat followed by 7 granulin repeats. The mammalian GRANULINS are named PARAGRANULIN and granulin 1-7 for their order in the progranulin propeptide. Progranulin has growth-factor like effects and is expressed over a broad range of tissues. Under-expression of progranulin is linked to neurodegenerative diseases including FRONTOTEMPORAL DEMENTIA and NEURONAL CEROID LIPOFUSCINOSIS, while over-expression is linked to metabolic diseases including OBESITY, and INSULIN RESISTANCE.'),
  'D000077154': NtO(MH='Rosiglitazone', MS='A thiazolidinedione that functions as a selective agonist for PPAR GAMMA. It improves INSULIN SENSITIVITY in adipose tissue, skeletal muscle, and the liver of patients with TYPE 2 DIABETES MELLITUS.'),
  'D000077155': NtO(MH='Granulins', MS='Small, cysteine-rich peptides generated by the proteolytic cleavage of progranulin. Granulin peptides are each composed of a granulin-motif, while paragranulin is composed of a half-granulin motif. The mammalian granulins are named granulin 1-7 for their sequence order in the progranulin propeptide, and are alternately named granulin G, F, B, A, C, D, and E; or epithelin 7, 6, 2, 1, 3, 4, and 5, i.e. granulin 1 aka granulin G aka epithelin 7, etc.'),
  'D000077156': NtO(MH='Gefitinib', MS='A selective tyrosine kinase inhibitor for the EPIDERMAL GROWTH FACTOR RECEPTOR (EGFR) that is used for the treatment of locally advanced or metastatic NON-SMALL CELL LUNG CANCER.'),
  'D000077157': NtO(MH='Sorafenib', MS='A niacinamide and phenylurea derivative that inhibits multiple intracellular and cell surface kinases thought to be involved in ANGIOGENESIS, including RAF KINASES and VEGF RECEPTORS. It is used in the treatment of advanced RENAL CELL CARCINOMA and HEPATOCELLULAR CARCINOMA, and for treatment of THYROID CARCINOMA refractory to radioactive iodine therapy.'),
  'D000077162': NtO(MH='Neurolymphomatosis', MS='Infiltration of the nervous system by malignant lymphoma cells.'),
  'D000077182': NtO(MH='Polylactic Acid-Polyglycolic Acid Copolymer', MS='A co-polymer that consists of varying ratios of polylactic acid and polyglycolic acid. It is used as a matrix for drug delivery and for BONE REGENERATION.'),
  'D000077183': NtO(MH='Occlusion Body Matrix Proteins', MS='Proteins that assemble into a crystalline polyhedral or ovicylindrical shape around insect viruses, including BACULOVIRIDAE and CYPOVIRUS.'),
  'D000077185': NtO(MH='Resveratrol', MS='A stilbene and non-flavonoid polyphenol produced by various plants including grapes and blueberries. It has anti-oxidant, anti-inflammatory, cardioprotective, anti-mutagenic, and anti-carcinogenic properties. It also inhibits platelet aggregation and the activity of several DNA HELICASES in vitro.'),
  'D000077190': NtO(MH='Interferon alpha-2', MS='Alpha interferon encoded by the human IFNA2 gene. Recombinant forms are used in the treatment of CHRONIC HEPATITIS B; CHRONIC HEPATITIS C; KAPOSI SARCOMA; MELANOMA; and HAIRY CELL LEUKEMIA.'),
  'D000077191': NtO(MH='Wortmannin', MS='An androstadiene metabolite produced by the fungi PENICILLIUM funiculosum that inhibits PHOSPHATIDYLINOSITOL-3-KINASES and alloantigen-specific activation of T-LYMPHOCYTES in human tumor cell lines. It is widely used in CELL BIOLOGY research and has broad therapeutic potential.'),
  'D000077192': NtO(MH='Adenocarcinoma of Lung', MS='A carcinoma originating in the lung and the most common lung cancer type in never-smokers. Malignant cells exhibit distinct features such as glandular epithelial, or tubular morphology. Mutations in KRAS, EGFR, BRAF, and ERBB2 genes are associated with this cancer.'),
  'D000077194': NtO(MH='Chlorophyll A', MS='A form of chlorophyll that absorbs light in the violet to red spectrum (approximately 400-700 nm wavelength range) and reflects green light (500-570 nm wavelength), which imparts the characteristic green color to land plants. It is essential for oxygenic PHOTOSYNTHESIS.'),
  'D000077195': NtO(MH='Squamous Cell Carcinoma of Head and Neck', MS='The most common type of head and neck carcinoma that originates from cells on the surface of the NASAL CAVITY; MOUTH; PARANASAL SINUSES, SALIVARY GLANDS, and LARYNX. Mutations in TNFRSF10B, PTEN, and ING1 genes are associated with this cancer.'),
  'D000077202': NtO(MH='Dispensatory', MS='Systematic description of the drugs and preparations used generally or in a specific setting.'),
  'D000077203': NtO(MH='Sodium-Glucose Transporter 2 Inhibitors', MS='Compounds that inhibit SODIUM-GLUCOSE TRANSPORTER 2. They lower blood sugar by preventing the reabsorption of glucose by the kidney, and are used in the treatment of TYPE 2 DIABETES MELLITUS.'),
  'D000077204': NtO(MH='Temozolomide', MS='A dacarbazine derivative that is used as an alkylating antineoplastic agent for the treatment of MALIGNANT GLIOMA and MALIGNANT MELANOMA.'),
  'D000077205': NtO(MH='Pioglitazone', MS='A thiazolidinedione and PPAR GAMMA agonist that is used in the treatment of TYPE 2 DIABETES MELLITUS.'),
  'D000077206': NtO(MH='Gabapentin', MS='A cyclohexane-gamma-aminobutyric acid derivative that is used for the treatment of PARTIAL SEIZURES; NEURALGIA; and RESTLESS LEGS SYNDROME.'),
  'D000077207': NtO(MH='Chondrosarcoma, Clear Cell', MS='A chondrosarcoma of adolescence to old age of the proximal EPIPHYSES of long bones. It has variably clear cytoplasm of the mostly neoplastic CHONDROCYTES with little intervening matrix.'),
  'D000077208': NtO(MH='Remifentanil', MS='A piperidine-propionate derivative and opioid analgesic structurally related to FENTANYL. It functions as a short-acting MU OPIOID RECEPTOR agonist, and is used as an analgesic during induction or maintenance of general anesthesia, following surgery, during childbirth, and in mechanically ventilated patients under intensive care.'),
  'D000077209': NtO(MH='Decitabine', MS='An azacitidine derivative and antineoplastic antimetabolite. It inhibits DNA methyltransferase to re-activate silent genes, limiting METASTASIS and NEOPLASM DRUG RESISTANCE. Decitabine is used in the treatment of MYELODISPLASTIC SYNDROMES, and ACUTE MYELOID LEUKEMIA.'),
  'D000077210': NtO(MH='Sunitinib', MS='An indole and pyrrole derivative that inhibits VEGFR-2 and PDGFR BETA RECEPTOR TYROSINE KINASES. It is used as an antineoplastic agent for the treatment of GASTROINTESTINAL STROMAL TUMORS, and for treatment of advanced or metastatic RENAL CELL CARCINOMA.'),
  'D000077211': NtO(MH='Zoledronic Acid', MS='An imidobisphosphonate inhibitor of BONE RESORPTION that is used for the treatment of malignancy-related HYPERCALCEMIA; OSTEITIS DEFORMANS; and OSTEOPOROSIS.'),
  'D000077212': NtO(MH='Ropivacaine', MS='An anilide used as a long-acting local anesthetic. It has a differential blocking effect on sensory and motor neurons.'),
  'D000077213': NtO(MH='Lamotrigine', MS='A phenyltriazine compound, sodium and calcium channel blocker that is used for the treatment of SEIZURES and BIPOLAR DISORDER.'),
  'D000077214': NtO(MH='Becaplermin', MS='A recombinant human platelet-derived growth factor B-chain dimer used to promote WOUND HEALING by stimulating PHYSIOLOGIC ANGIOGENESIS.'),
  'D000077215': NtO(MH='Clay', MS='A naturally-occurring rock or soil constituent characterized by particles with a diameter of less than 0.005 mm. It is composed primarily of hydrous aluminum silicates, trace amounts of metal OXIDES, and organic matter.'),
  'D000077216': NtO(MH='Carcinoma, Ovarian Epithelial', MS='A malignant neoplasm that originates in cells on the surface EPITHELIUM of the ovary and is the most common form of ovarian cancer. There are five histologic subtypes: papillary serous, endometrioid, mucinous, clear cell, and transitional cell. Mutations in BRCA1, OPCML, PRKN, PIK3CA, AKT1, CTNNB1, RRAS2, and CDH1 genes are associated with this cancer.'),
  'D000077221': NtO(MH='Calcitonin Gene-Related Peptide Receptor Antagonists', MS='Pharmacologic agents that block NOCICEPTIVE PAIN signaling from CALCITONIN GENE-RELATED PEPTIDE RECEPTORS. They may be useful for the treatment of pain associated with MIGRAINE DISORDERS and OSTEOARTHRITIS.'),
  'D000077222': NtO(MH='Limonene', MS='A naturally-occurring class of MONOTERPENES which occur as a clear colorless liquid at room temperature. Limonene is the major component in the oil of oranges which has many uses, including as flavor and fragrance. It is recognized as safe in food by the Food and Drug Administration (FDA).'),
  'D000077224': NtO(MH='Cyberbullying', MS='Bullying that takes place over digital devices like cell phones and computers. It includes sending, posting, or sharing negative, harmful, false, or mean content about someone else. It can include sharing personal or private information about someone else causing embarrassment or humiliation. (from https://www.stopbullying.gov)'),
  'D000077228': NtO(MH='Caspian Sea', MS='The largest inland body of water, located between ASIA and EUROPE.'),
  'D000077230': NtO(MH='Internet Access', MS='Ability to connect to the INTERNET and access services available there.'),
  'D000077235': NtO(MH='Vinorelbine', MS='A vinca alkaloid related to VINBLASTINE that is used as a first-line treatment for NON-SMALL CELL LUNG CANCER, or for advanced or metastatic BREAST CANCER refractory to treatment with ANTHRACYCLINES.'),
  'D000077236': NtO(MH='Topiramate', MS='A sulfamate-substituted fructose analog that was originally identified as a hypoglycemic agent. It is used for the treatment of EPILEPSY and MIGRAINE DISORDERS, and may also promote weight loss.'),
  'D000077237': NtO(MH='Arsenic Trioxide', MS='An inorganic compound with the chemical formula As2O3 that is used for the treatment of ACUTE PROMYELOCYTIC LEUKEMIA in patients who have relapsed from, or are resistant to, conventional drug therapy.'),
  'D000077239': NtO(MH='Meloxicam', MS='A benzothiazine and thiazole derivative that acts as a NSAID and cyclooxygenase-2 (COX-2) inhibitor. It is used in the treatment of RHEUMATOID ARTHRITIS; OSTEOARTHRITIS; and ANKYLOSING SPONDYLITIS.'),
  'D000077250': NtO(MH='Calcium Aluminosilicate', MS='Aluminosilicate salts of calcium, the most common of which are CaAl2Si2O8 and Ca2Al2SiO7; a naturally occurring form in CLAY may be used to treat DIARRHEA.'),
  'D000077252': NtO(MH='Dialectical Behavior Therapy', MS='A flexible, stage-based therapy that combines principles of behavior therapy, cognitive behavior therapy, and MINDFULNESS. It concurrently promotes acceptance and change, especially with difficult-to-treat patients.'),
  'D000077253': NtO(MH='Online Social Networking', MS='Connectivity facilitated by computer-based communications among persons with family, work, or other common interests.'),
  'D000077260': NtO(MH='Sleepiness', MS='Compelling urge to sleep.'),
  'D000077261': NtO(MH='Carvedilol', MS='A carbazole and propanol derivative that acts as a non-cardioselective beta blocker and vasodilator. It has blocking activity for ALPHA 1 ADRENERGIC RECEPTORS and, at higher doses, may function as a blocker of CALCIUM CHANNELS; it also has antioxidant properties. Carvedilol is used in the treatment of HYPERTENSION; ANGINA PECTORIS; and HEART FAILURE. It can also reduce the risk of death following MYOCARDIAL INFARCTION.'),
  'D000077262': NtO(MH='Ciona', MS='A genus of SEA SQUIRTS in the family Cionidae.'),
  'D000077264': NtO(MH='Calcium-Regulating Hormones and Agents', MS='Hormones and molecules with calcium-regulating hormone-like actions that modulate OSTEOLYSIS and other extra-skeletal activities to maintain calcium homeostasis.'),
  'D000077265': NtO(MH='Donepezil', MS='An indan and piperidine derivative that acts as a selective and reversible inhibitor of ACETYLCHOLINESTERASE. Donepezil is highly selective for the central nervous system and is used in the management of mild to moderate DEMENTIA in ALZHEIMER DISEASE.'),
  'D000077266': NtO(MH='Moxifloxacin', MS='A fluoroquinolone that acts as an inhibitor of DNA TOPOISOMERASE II and is used as a broad-spectrum antibacterial agent.'),
  'D000077267': NtO(MH='Fulvestrant', MS='An estradiol derivative and estrogen receptor antagonist that is used for the treatment of estrogen receptor-positive, locally advanced or metastatic breast cancer.'),
  'D000077268': NtO(MH='Pamidronate', MS='An aminobisphosphonate that inhibits BONE RESORPTION and is used for the treatment of osteolytic lesions, bone pain, and severe HYPERCALCEMIA associated with malignancies.'),
  'D000077269': NtO(MH='Lenalidomide', MS='A phthalimide and piperidone derivative that has immunomodulatory and antiangiogenic properties. It is used for the treatment of transfusion-dependent anemia in MYELODYSPLASTIC SYNDROMES, and for the treatment of MULTIPLE MYELOMA, and relapsed or refractory MANTLE CELL LYMPHOMA.'),
  'D000077270': NtO(MH='Exenatide', MS='A synthetic form of exendin-4, a 39-amino acid peptide isolated from the venom of the Gila monster lizard (Heloderma suspectum). Exenatide increases CYCLIC AMP levels in pancreatic acinar cells and acts as a GLUCAGON-LIKE PEPTIDE-1 RECEPTOR (GLP-1) agonist and incretin mimetic, enhancing insulin secretion in response to increased glucose levels; it also suppresses inappropriate glucagon secretion and slows gastric emptying. It is used an anti-diabetic and anti-obesity agent.'),
  'D000077271': NtO(MH='Imiquimod', MS='A topically-applied aminoquinoline immune modulator that induces interferon production. It is used in the treatment of external genital and perianal warts, superficial CARCINOMA, BASAL CELL; and ACTINIC KERATOSIS.'),
  'D000077272': NtO(MH='Latent Class Analysis', MS='A statistical algorithm used to analyze clusters of observed variables by constructing categorical unobserved or latent segment based on weighted analysis and the average probabilities. Such latent classes are used to infer variables whose relationships are not directly observed. In biomedical research, it is often used to categorize data that allows the determination of symptom clusters.'),
  'D000077273': NtO(MH='Thyroid Cancer, Papillary', MS='An ADENOCARCINOMA that originates from follicular cells of the THYROID GLAND and accounts for the majority of THYROID CANCER cases. Cells exhibit enlarged, oval, or elongated morphologies with clear, round, nuclei. Fusions of RET, NTRK1, TPM3, and PCM1 genes are associated with this cancer.'),
  'D000077274': NtO(MH='Nasopharyngeal Carcinoma', MS='A carcinoma that originates in the EPITHELIUM of the NASOPHARYNX and includes four subtypes: keratinizing squamous cell, non-keratinizing, basaloid squamous cell, and PAPILLARY ADENOCARCINOMA. It is most prevalent in Southeast Asian populations and is associated with EPSTEIN-BARR VIRUS INFECTIONS. Somatic mutations associated with this cancer have been identified in NPCR, BAP1, UBAP1, ERBB2, ERBB3, MLL2, PIK3CA, KRAS, NRAS, and ARID1A genes.'),
  'D000077275': NtO(MH='Craniofacial Fibrous Dysplasia', MS='Mostly benign fibro-osseous proliferation of the facial bones and skull. It can be either monostotic (localized to a single bone) or polyostotic (localized to more than one bone) type.'),
  'D000077276': NtO(MH='Lycopene', MS='A carotenoid and red pigment produced by tomatoes, other red fruits and vegetables, and photosynthetic algae. It is a key intermediate in the biosynthesis of other carotenoids, and has antioxidant, anti-carcinogenic, radioprotective, and anti-inflammatory properties.'),
  'D000077277': NtO(MH='Esophageal Squamous Cell Carcinoma', MS='A carcinoma that originates usually from cells on the surface of the middle and lower third of the ESOPHAGUS. Tumor cells exhibit typical squamous morphology and form large polypoid lesions. Mutations in RNF6, LZTS1, TGFBR2, DEC1, and WWOX1 genes are associated with this cancer.'),
  'D000077278': NtO(MH='RNA, Mitochondrial', MS='RNA molecules encoded by the MITOCHONDRIAL GENOME.'),
  'D000077279': NtO(MH='Protein Carbamylation', MS='Non-enzymatic addition of carbamoyl (-CONH2) on protein or amino acid functional groups from isocyanic acid or CARBAMOYL-PHOSPHATE. Carbamylation may induce a structural change that results in a functional loss and serves as a biomarker for protein aging and disease pathophysiology.'),
  'D000077280': NtO(MH='Bone-Anchored Prosthesis', MS='A prosthesis designed to attach directly to the bone via fixtures implanted into the bone rather than a socket.'),
  'D000077283': NtO(MH='Forensic Psychology', MS='A specialty in professional psychology characterized by activities primarily intended to provide professional psychological expertise within the judicial and legal systems.'),
  'D000077284': NtO(MH='Abciximab', MS='A Fab fragment of the chimeric monoclonal antibody 7E3 that binds to the glycoprotein IIb-IIIa receptor of human platelets, and blocks PLATELET GLYCOPROTEIN GPIIB-IIIA COMPLEX, potently inhibiting PLATELET AGGREGATION. It is used in treatment of refractory unstable angina, and for prevention of ischemic complications in patients undergoing percutaneous coronary procedures such as ANGIOPLASTY; ATHERECTOMY; or stenting.'),
  'D000077285': NtO(MH='Rimonabant', MS='A pyrazole and piperidine derivative that acts as a selective cannabinoid type-1 receptor (CB1 RECEPTOR) antagonist. It inhibits the proliferation and maturation of ADIPOCYTES, improves lipid and glucose metabolism, and regulates food intake and energy balance. It is used in the management of OBESITY.'),
  'D000077286': NtO(MH='Cetrimonium', MS='Cetyltrimethylammonium compound whose salts and derivatives are used primarily as topical antiseptics.'),
  'D000077287': NtO(MH='Levetiracetam', MS='A pyrrolidinone and acetamide derivative that is used primarily for the treatment of SEIZURES and some movement disorders, and as a nootropic agent.'),
  'D000077288': NtO(MH='Troglitazone', MS='A chroman and thiazolidinedione derivative that acts as a PEROXISOME PROLIFERATOR-ACTIVATED RECEPTORS (PPAR) agonist. It was formerly used in the treatment of TYPE 2 DIABETES MELLITUS, but has been withdrawn due to hepatotoxicity.'),
  'D000077289': NtO(MH='Letrozole', MS='A triazole and benzonitrile derivative that is a selective non-steroidal aromatase inhibitor, similar to ANASTROZOLE. It is used in the treatment of metastatic or locally advanced breast cancer in postmenopausal women.'),
  'D000077290': NtO(MH='Harmala Alkaloids', MS='Structurally-related ALKALOIDS derived from the seeds of PEGANUM HARMALA and other plant sources. They are associated with various pharmacologic and therapeutic effects.'),
  'D000077291': NtO(MH='Terbinafine', MS='A naphthalene derivative that inhibits fungal SQUALENE EPOXIDASE and is used to treat DERMATOMYCOSES of the skin and nails.'),
  'D000077293': NtO(MH='Receptor, Transforming Growth Factor-beta Type I', MS='A transmembrane serine-threonine kinase that forms a heteromeric complex with TYPE II TGF-BETA RECEPTORS to bind TGF-BETA and regulate a variety of physiological and pathological processes including CELL CYCLE ARREST; CELL PROLIFERATION; CELL DIFFERENTIATION; WOUND HEALING; EXTRACELLULAR MATRIX production, immunosuppression and ONCOGENESIS.'),
  'D000077294': NtO(MH='Receptor, Transforming Growth Factor-beta Type II', MS='A transmembrane serine-threonine kinase that forms a heteromeric complex with TYPE I TGF-BETA RECEPTORS when bound to TGF-BETA. This receptor complex regulates a variety of physiological and pathological processes including CELL CYCLE ARREST; CELL PROLIFERATION; CELL DIFFERENTIATION; WOUND HEALING; EXTRACELLULAR MATRIX production, immunosuppression and ONCOGENESIS.'),
  'D000077295': NtO(MH='Urinary Bladder, Underactive', MS='Symptom of underactive detrusor muscle of the URINARY BLADDER that contracts with abnormally reduced strength or duration resulting in an incomplete and/or prolonged bladder emptying.'),
  'D000077296': NtO(MH='Niemann-Pick C1 Protein', MS='A transmembrane intracellular lipid transporter that helps trafficking lipids, particularly cholesterol, from late endosome/lysosome to the ER and plasma membrane. It may also play a role in other cellular cholesterol homeostasis and is an endosomal entry receptor for filoviruses including EBOLAVIRUS. Mutations in the NPC1 are associated with type C NIEMANN-PICK DISEASE.'),
  'D000077297': NtO(MH='Pregnane X Receptor', MS='Steroid receptor that binds and is activated by variety of endogenous compounds and XENOBIOTICS. It binds the response element in promoters of  genes that encode CYTOCHROME P450 3A4 and ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B, MEMBER 1 proteins, and also activates the transcription of multiple genes involved in the metabolism and secretion of potentially harmful xenobiotics, drugs, and endogenous compounds. It is activated by the antibiotic RIFAMPICIN and various plant metabolites, such as hyperforin, guggulipid, colupulone, and ISOFLAVONES.'),
  'D000077298': NtO(MH='Flexural Strength', MS='The maximum MECHANICAL STRESS that a heterogeneous material, subject to a bending load or flexure, can withstand before fracturing.'),
  'D000077299': NtO(MH='Healthcare-Associated Pneumonia', MS='Infection of the lung often accompanied by inflammation that is acquired through an interaction within a healthcare institution often through a therapeutic experience (e.g., use of catheters or ventilators).'),
  'D000077300': NtO(MH='Bosentan', MS='A sulfonamide and pyrimidine derivative that acts as a dual endothelin receptor antagonist used to manage PULMONARY HYPERTENSION and SYSTEMIC SCLEROSIS.'),
  'D000077310': NtO(MH='Sleep, Slow-Wave', MS='A stage of sleep characterized by synchronized wave activity as observed in the ELECTROENCEPHALOGRAM; the waves exhibit a frequency of 0.5 to 2 Hz, and a peak-to-peak amplitude greater than 75 microvolts.'),
  'D000077314': NtO(MH='Aggressive Driving', MS='Combination of moving traffic offenses so as to endanger other persons or property. (https://nhtsa.gov/Driving-Safety/Aggressive-Driving)'),
  'D000077315': NtO(MH='Road Rage', MS='Extreme anger generated from a traffic incident or accident that leads to an assault with a motor vehicle or other dangerous weapon by the operator or passenger(s) of one motor vehicle on the operator or passenger(s) of another motor vehicle.'),
  'D000077317': NtO(MH='Extracellular Polymeric Substance Matrix', MS='Extracellular matrix formed by secreted polymeric organic compounds by microorganisms. This secreted compound is mostly polysaccharides but may include proteins, lipids, and nucleic acids. The matrix facilitates attachment of microbes to form various BIOFILM communities.'),
  'D000077318': NtO(MH='Density Functional Theory', MS='Density functional theory is an approach to describe the electronic ground state properties of matter.  It is a method of obtaining a solution to the Schrodinger equation for a many-body system, and is used to investigate the structural, magnetic, and electronic properties of molecules and materials.'),
  'D000077319': NtO(MH='Pyrolysis', MS='High temperature decomposition without addition of extra air or oxygen. It yields syngas, bio-oil and bio-char.'),
  'D000077320': NtO(MH='Biomineralization', MS='Process by which living organisms produce MINERALS that accumulate into hardened tissues (e.g., bones, teeth and shells), and cellular nanostructures that sequester away into macromolecules. Biosynthesized minerals include CALCIUM CARBONATE; FLUOROAPATITE; HYDROXYAPATITE; and MAGNETITE.'),
  'D000077321': NtO(MH='Deep Learning', MS='Supervised or unsupervised machine learning methods that use multiple layers of data representations generated by nonlinear transformations, instead of individual task-specific ALGORITHMS, to build and train neural network models.'),
  'D000077322': NtO(MH='Alginic Acid', MS='A polysaccharide and linear copolymer, consisting mainly of beta-1,4-linked D-mannuronic acid and alpha-1,4-linked L-glucuronic acid, that occurs in marine algae such as LAMINARIA, and in BIOFILMS produced by PSEUDOMONAS AERUGINOSA.'),
  'D000077323': NtO(MH='Salt Stress', MS='The condition that results from ion toxicity due to ion imbalances (e.g., excessive Na+ and deficiency in Mg++).'),
  'D000077324': NtO(MH='Crystalloid Solutions', MS="Isotonic solutions of mineral salts, such as RINGER'S LACTATE and SODIUM CHLORIDE (SALINE SOLUTION), used in FLUID THERAPY to rehydrate BLOOD VOLUME."),
  'D000077325': NtO(MH="Ringer's Lactate", MS='A crystalloid solution that contains SODIUM CHLORIDE; SODIUM LACTATE; POTASSIUM CHLORIDE; and CALCIUM CHLORIDE. It is used for FLUID THERAPY.'),
  'D000077329': NtO(MH='Agammaglobulinaemia Tyrosine Kinase', MS='A non-receptor tyrosine kinase that is essential for the development, maturation, and signaling of B-LYMPHOCYTES. It contains an N-terminal zinc finger motif and localizes primarily to the PLASMA MEMBRANE and nucleus of B-lymphocytes. Mutations in the gene that encode this kinase are associated with X-LINKED AGAMMAGLOBULINEMIA.'),
  'D000077330': NtO(MH='Saline Solution', MS='A crystalloid solution that contains 9.0g of SODIUM CHLORIDE per liter of water. It has a variety of uses, including: as a CONTACT LENS SOLUTION, in OPHTHALMIC SOLUTIONS and NASAL LAVAGE, in wound irrigation, and for FLUID THERAPY.'),
  'D000077331': NtO(MH="Ringer's Solution", MS='An isotonic solution; the base contains SODIUM CHLORIDE; POTASSIUM CHLORIDE; and CALCIUM CHLORIDE. Other chemicals, such as SODIUM BICARBONATE or acetate salts may be added, as needed for pH buffering, or as an energy source.'),
  'D000077332': NtO(MH='Artesunate', MS='A water-soluble, semi-synthetic derivative of the sesquiterpene lactone artemisinin with anti-malarial, anti-schistosomiasis, antiviral, and potential anti-neoplastic activities'),
  'D000077333': NtO(MH='Telmisartan', MS='A biphenyl compound and benzimidazole derivative that acts as an angiotensin II type 1 receptor antagonist. It is used in the management of HYPERTENSION.'),
  'D000077334': NtO(MH='Zolpidem', MS='An imidazopyridine derivative and short-acting GABA-A receptor agonist that is used for the treatment of INSOMNIA.'),
  'D000077335': NtO(MH='Desflurane', MS='A fluorinated ether that is used as a volatile anesthetic for maintenance of general anesthesia.'),
  'D000077336': NtO(MH='Caspofungin', MS='A cyclic lipopeptide echinocandin and beta-(1,3)-D-glucan synthase inhibitor that is used to treat internal or systemic MYCOSES.'),
  'D000077337': NtO(MH='Vorinostat', MS='A hydroxamic acid and anilide derivative that acts as a HISTONE DEACETYLASE inhibitor. It is used in the treatment of CUTANEOUS T-CELL LYMPHOMA and SEZARY SYNDROME.'),
  'D000077338': NtO(MH='Latanoprost', MS='A prostaglandin F analog used to treat OCULAR HYPERTENSION in patients with GLAUCOMA.'),
  'D000077339': NtO(MH='Leflunomide', MS='An isoxazole derivative that inhibits dihydroorotate dehydrogenase, the fourth enzyme in the pyrimidine biosynthetic pathway. It is used an immunosuppressive agent in the treatment of RHEUMATOID ARTHRITIS and PSORIATIC ARTHRITIS.'),
  'D000077340': NtO(MH='Fluvastatin', MS='An indole-heptanoic acid derivative that inhibits HMG COA REDUCTASE and is used to treat HYPERCHOLESTEROLEMIA. In contrast to other statins, it does not appear to interact with other drugs that inhibit CYP3A4.'),
  'D000077341': NtO(MH='Lapatinib', MS='A quinazoline derivative that inhibits EPIDERMAL GROWTH FACTOR RECEPTOR and HER2 (RECEPTOR, ERBB-2) tyrosine kinases. It is used for the treatment of advanced or metastatic breast cancer, where tumors overexpress HER2.'),
  'D000077342': NtO(MH='Post-Lyme Disease Syndrome', MS='A condition caused by long-lasting and ongoing infection with the spirochete Borrelia burgdorferi resulting in progressive inflammatory neurologic, neuromuscular, and dermatologic manifestations including ENCEPHALITIS; MYELITIS; acrodermatitis chronica atrophicans; and ARTHRITIS.'),
  'D000077362': NtO(MH='Verteporfin', MS='A benzoporphyrin derivative that is used in PHOTOCHEMOTHERAPY to treat wet type MACULAR DEGENERATION.'),
  'D000077384': NtO(MH='Anastrozole', MS='A nitrile and triazole derivative that acts as a selective nonsteroidal aromatase inhibitor. It is used in the treatment of ESTROGEN NUCLEAR RECEPTOR-positive breast cancer in postmenopausal women.'),
  'D000077385': NtO(MH='Silybin', MS='The major active component of silymarin flavonoids extracted from seeds of the MILK THISTLE, Silybum marianum; it is used in the treatment of HEPATITIS; LIVER CIRRHOSIS; and CHEMICAL AND DRUG INDUCED LIVER INJURY, and has antineoplastic activity; silybins A and B are diastereomers.'),
  'D000077402': NtO(MH='Pantoprazole', MS='2-pyridinylmethylsulfinylbenzimidazole proton pump inhibitor that is used in the treatment of GASTROESOPHAGEAL REFLUX and PEPTIC ULCER.'),
  'D000077403': NtO(MH='Orlistat', MS='A lactone derivative of LEUCINE that acts as a pancreatic lipase inhibitor to limit the absorption of dietary fat; it is used in the management of obesity.'),
  'D000077404': NtO(MH='Cidofovir', MS='An acyclic nucleoside phosphonate that acts as a competitive inhibitor of viral DNA polymerases. It is used in the treatment of RETINITIS caused by CYTOMEGALOVIRUS INFECTIONS and may also be useful for treating HERPESVIRUS INFECTIONS.'),
  'D000077405': NtO(MH='Irbesartan', MS='A spiro compound, biphenyl and tetrazole derivative that acts as an angiotensin II type 1 receptor antagonist. It is used in the management of HYPERTENSION, and in the treatment of kidney disease.'),
  'D000077406': NtO(MH='History of Pharmacy', MS='Study of the origins and development of the field of PHARMACY.'),
  'D000077407': NtO(MH='Cilostazol', MS='A quinoline and tetrazole derivative that acts as a phosphodiesterase type 3 inhibitor, with anti-platelet and vasodilating activity. It is used in the treatment of PERIPHERAL VASCULAR DISEASES; ISCHEMIC HEART DISEASE; and in the prevention of stroke.'),
  'D000077408': NtO(MH='Modafinil', MS='A benzhydryl acetamide compound, central nervous system stimulant, and CYP3A4 inducing agent that is used in the treatment of NARCOLEPSY and SLEEP WAKE DISORDERS.'),
  'D000077409': NtO(MH='Tamsulosin', MS='A sulfonamide derivative and adrenergic alpha-1 receptor antagonist that is used to relieve symptoms of urinary obstruction caused by BENIGN PROSTATIC HYPERPLASIA.'),
  'D000077410': NtO(MH='Aluminum Chloride', MS='A compound with the chemical formula AlCl3; the anhydrous salt is used as a catalyst in organic chemical synthesis, and hydrated salts are used topically as antiperspirants, and for the management of HYPERHYDROSIS.'),
  'D000077422': NtO(MH='Enrofloxacin', MS='A fluoroquinolone antibacterial and antimycoplasma agent that is used in veterinary practice.'),
  'D000077423': NtO(MH='Polidocanol', MS='An alkyl polyglycol ether of LAURYL ALCOHOL, chemically defined as an alcohol ethoxylate having an average alkyl chain of 12\xe2\x80\x9314 carbon atoms, and an ethylene oxide chain of 9 ethylene oxide units. It is used as a detergent, and medically as a local anesthetic, and as a sclerosing agent for the treatment of ESOPHAGEAL AND GASTRIC VARICES and VARICOSE VEINS.'),
  'D000077425': NtO(MH='Fondaparinux', MS='Synthetic pentasaccharide that mediates the interaction of HEPARIN with ANTITHROMBINS and inhibits FACTOR Xa; it is used for prevention of VENOUS THROMBOEMBOLISM after surgery.'),
  'D000077426': NtO(MH='Gastric Artery', MS='Any of several branches of the SPLENIC ARTERY distributed to the greater curvature of the stomach.'),
  'D000077427': NtO(MH='Lipoglycopeptides', MS='Glycopeptides that have an additional lipophilic side chains. Several semisynthetic lipoglycopeptide antibiotics are derived from naturally occurring lipoglycopeptide and glycopeptide antibiotics.'),
  'D000077428': NtO(MH='GATA2 Deficiency', MS='A rare disorder of the immune system with wide-ranging effects which include GATA2 Transcription Factor dysfunction, immunodeficiency, myelodysplastic syndrome (ineffective blood cell production), lung disease, and problems of the vascular and lymphatic system.'),
  'D000077430': NtO(MH='Nabumetone', MS='A butanone non-steroidal anti-inflammatory drug and cyclooxygenase-2 (COX2) inhibitor that is used in the management of pain associated with OSTEOARTHRITIS and RHEUMATOID ARTHRITIS.'),
  'D000077431': NtO(MH='Oxaprozin', MS='An oxazole-propionic acid derivative, cyclooxygenase inhibitor, and non-steroidal anti-inflammatory drug that is used in the treatment of pain and inflammation associated with of OSTEOARTHRITIS; RHEUMATOID ARTHRITIS; and ARTHRITIS, JUVENILE.'),
  'D000077432': NtO(MH='Tapentadol', MS='An opioid analgesic, MU OPIOID RECEPTOR agonist, and noradrenaline reuptake inhibitor that is used in the treatment of moderate to severe pain, and of pain associated with DIABETIC NEUROPATHIES.'),
  'D000077442': NtO(MH='Lidocaine, Prilocaine Drug Combination', MS='A topical local anesthetic preparation that is composed of a mixture of lidocaine and prilocaine. It is used to provide anesthesia during minor surgery and for the treatment of PREMATURE EJACULATION.'),
  'D000077443': NtO(MH='Acamprosate', MS='Structural analog of taurine that is used for the prevention of relapse in individuals with ALCOHOLISM.'),
  'D000077444': NtO(MH='Smoking Cessation Agents', MS='Substances that facilitate the cessation of tobacco smoking.'),
  'D000077463': NtO(MH='Carbamide Peroxide', MS='A urea peroxide compound that is commonly used in tooth whitening agents; topical anti-infective agents, and earwax remover.'),
  'D000077464': NtO(MH='Simendan', MS='A hydrazone and pyridazine derivative; the levo-form is a phosphodiesterase III inhibitor, calcium-sensitizing agent, and inotropic agent that is used in the treatment of HEART FAILURE.'),
  'D000077465': NtO(MH='Cabergoline', MS='An ergoline derivative and dopamine D2-agonist that inhibits PROLACTIN secretion. It is used in the management of HYPERPROLACTINEMIA, and to suppress lactation following childbirth for medical reasons. Cabergoline is also used in the management of PARKINSON DISEASE.'),
  'D000077466': NtO(MH='Tirofiban', MS='Tyrosine analog and PLATELET GLYCOPROTEIN GPIIB-IIIA COMPLEX antagonist that inhibits PLATELET AGGREGATION and is used in the treatment of ACUTE CORONARY SYNDROME.'),
  'D000077482': NtO(MH='Carbon Fiber', MS='Fiber with a diameter of approximately 5 to 10 micrometers that is composed mostly of carbon atoms. It has a variety of applications.'),
  'D000077483': NtO(MH='Valacyclovir', MS='A prodrug of acyclovir that is used in the treatment of HERPES ZOSTER and HERPES SIMPLEX VIRUS INFECTION of the skin and mucous membranes, including GENITAL HERPES.'),
  'D000077484': NtO(MH='Vemurafenib', MS='An indole sulfonamide compound and inhibitor of BRAF KINASES that is used for the treatment of unresectable or metastatic MELANOMA.'),
  'D000077485': NtO(MH='Meglumine Antimoniate', MS='ANTIMONY salt of meglumine that is used in the treatment of LEISHMANIASIS.'),
  'D000077486': NtO(MH='Ticagrelor', MS='An adenosine triphosphate analogue and reversible P2Y12 PURINORECEPTOR antagonist that inhibits ADP-mediated PLATELET AGGREGATION. It is used for the prevention of THROMBOEMBOLISM by patients with ACUTE CORONARY SYNDROME or a history of MYOCARDIAL INFARCTION.'),
  'D000077487': NtO(MH='Pramipexole', MS='A benzothiazole derivative and dopamine agonist with antioxidant properties that is used in the treatment of PARKINSON DISEASE and RESTLESS LEGS SYNDROME.'),
  'D000077488': NtO(MH='Data Science', MS='An interdisciplinary field involving processes, theories, concepts, tools, and technologies, that enable the review, analysis, and extraction of valuable knowledge and information from structured and unstructured (raw) data.'),
  'D000077489': NtO(MH='Piperazine', MS='An anti-nematodal agent effective against the intestinal nematodes ASCARIS LUMBRICOIDES (roundworm) and ENTEROBIUS VERMICULARIS (pinworm, threadworm).  It produces a neuromuscular block leading to flaccid muscle paralysis in susceptible worms, which are then dislodged from the gut and expelled in feces.'),
  'D000077502': NtO(MH='Glymphatic System', MS='A vascular waste clearance system in the brain analogous to the lymphatic system that facilitates transporting of solutes and waste products from CEREBROSPINAL FLUID (CSF) and brain INTERSTITIAL FLUID (ISF). CSF from the SUBARACHNOID SPACE enters the brain periarterial spaces, into the interstitium via channels (e.g., AQUAPORIN 4 PROTEIN) on astrocytic endfeet and exchanges with brain ISF before perivenous drainage of solutes and waste.'),
  'D000077522': NtO(MH='Clinical Trial, Veterinary', MS='A work that is a veterinary clinical study in which animal participants are assigned to receive one or more interventions so that researchers can evaluate the interventions on biomedical or health-related outcomes. The assignments are determined by the study protocol. Animal participants may receive diagnostic, therapeutic, or other types of interventions.'),
  'D000077525': NtO(MH='Cefdinir', MS='A third-generation oral cephalosporin antibacterial agent that is used to treat bacterial infections of the respiratory tract and skin.'),
  'D000077526': NtO(MH='Tropisetron', MS='An indole derivative and 5-HT3 RECEPTOR antagonist that is used for the prevention of nausea and vomiting.'),
  'D000077542': NtO(MH='Eptifibatide', MS='Cyclic peptide that acts as a platelet glycoprotein IIB-IIIA antagonist, reversibly inhibiting the binding of FIBRINOGEN; VON WILLEBRAND FACTOR; and other adhesive molecules to the GPIIB-IIIA RECEPTORS of platelets. It is used in the management of UNSTABLE ANGINA and in patients undergoing coronary ANGIOPLASTY and stenting procedures.'),
  'D000077543': NtO(MH='Deferiprone', MS='A pyridone derivative and iron chelator that is used in the treatment of IRON OVERLOAD in patients with THALASSEMIA.'),
  'D000077544': NtO(MH='Panitumumab', MS='Recombinant human monoclonal antibody that binds to and inhibits the function of the EPIDERMAL GROWTH FACTOR RECEPTOR. It is used in the treatment of EGFR-expressing metastatic colorectal cancer that expresses wild-type ras gene (see GENES, RAS).'),
  'D000077545': NtO(MH='Eplerenone', MS='A spironolactone derivative and selective ALDOSTERONE RECEPTOR antagonist that is used in the management of HYPERTENSION and CONGESTIVE HEART FAILURE, post-MYOCARDIAL INFARCTION.'),
  'D000077546': NtO(MH='Roscovitine', MS='A purine derivative and competitive inhibitor of CYCLIN-DEPENDENT KINASES that has therapeutic potential as an antineoplastic and antiviral agent.'),
  'D000077547': NtO(MH='Crizotinib', MS='A piperidine and aminopyridine derivative that acts as an inhibitor of RECEPTOR PROTEIN-TYROSINE KINASES, including ANAPLASTIC LYMPHOMA KINASE (ALK) and HEPATOCYTE GROWTH FACTOR RECEPTOR (HGFR; c-Met). It is used in the treatment of NON-SMALL CELL LUNG CANCER.'),
  'D000077548': NtO(MH='Anaplastic Lymphoma Kinase', MS='A receptor tyrosine kinase that is essential for development and differentiation of the nervous system in response to secreted growth factors. It phosphorylates the first tyrosine of the Y-x-x-x-Y-Y motif of targets that include PROTO-ONCOGENE PROTEINS C-CBL; INSULIN RECEPTOR SUBSTRATE-1; and MITOGEN-ACTIVATED PROTEIN KINASES, leading to activation of the MAPK signaling pathway and cell proliferation. A chromosomal aberration involving the ALK gene results in its constitutive expression in some cases of NON-HODGKIN LYMPHOMA.'),
  'D000077549': NtO(MH='Artemether', MS='An artemisinin derivative that is used in the treatment of MALARIA.'),
  'D000077550': NtO(MH='Ivabradine', MS='A benzazepine derivative and selective HYPERPOLARIZATION-ACTIVATED CYCLIC NUCLEOTIDE-GATED CHANNELS inhibitor that lowers the heart rate. It is used in the treatment of CHRONIC STABLE ANGINA in patients unable to take BETA-ADRENERGIC BLOCKERS, and in the treatment of HEART FAILURE.'),
  'D000077551': NtO(MH='Micafungin', MS='A cyclic lipo-hexapeptide echinocandin antifungal agent that is used for the treatment and prevention of CANDIDIASIS.'),
  'D000077552': NtO(MH='Basiliximab', MS='A chimeric murine-human monoclonal antibody that functions as an INTERLEUKIN 2 RECEPTOR antagonist by binding to the alpha chain (CD25 ANTIGEN) of the interleukin-2 receptor on the surface of activated T-LYMPHOCYTES. It is used in the prevention of acute graft rejection episodes in patients undergoing renal transplantation.'),
  'D000077553': NtO(MH='Edaravone', MS='An antipyrine derivative that functions as a free radical scavenger and neuroprotective agent. It is used in the treatment of AMYOTROPHIC LATERAL SCLEROSIS and STROKE.'),
  'D000077554': NtO(MH='Levobupivacaine', MS='S-enantiomer of bupivacaine that is used as a local anesthetic and for regional nerve blocks, including EPIDURAL ANESTHESIA.'),
  'D000077555': NtO(MH='Methylprednisolone Acetate', MS='Methylprednisolone derivative that is used as an anti-inflammatory agent for the treatment of ALLERGY and ALLERGIC RHINITIS; ASTHMA; and BURSITIS; and for the treatment of ADRENAL INSUFFICIENCY.'),
  'D000077556': NtO(MH='Alitretinoin', MS='A retinoid that is used for the treatment of chronic hand ECZEMA unresponsive to topical CORTICOSTEROIDS. It is also used to treat cutaneous lesions associated with AIDS-related KAPOSI SARCOMA.'),
  'D000077557': NtO(MH='Ibandronic Acid', MS='Aminobisphosphonate that is a potent inhibitor of BONE RESORPTION. It is used in the treatment of HYPERCALCEMIA associated with malignancy, for the prevention of fracture and bone complications in patients with breast cancer and bone metastases, and for the treatment and prevention of POSTMENOPAUSAL OSTEOPOROSIS.'),
  'D000077558': NtO(MH='Big Data', MS='Extremely large amounts of data which require rapid and often complex computational analyses to reveal patterns, trends, and associations, relating to various facets of human and non-human entities.'),
  'D000077559': NtO(MH='Sodium Citrate', MS='Sodium salts of citric acid that are used as buffers and food preservatives. They are used medically as anticoagulants in stored blood, and for urine alkalization in the prevention of KIDNEY STONES.'),
  'D000077560': NtO(MH='Enfuvirtide', MS='A synthetic 36-amino acid peptide that corresponds to the heptad repeat sequence of HIV-1 gp41. It blocks HIV cell fusion and viral entry and is used with other anti-retrovirals for combination therapy of HIV INFECTIONS and AIDS.'),
  'D000077561': NtO(MH='Daclizumab', MS='An anti-TAC (INTERLEUKIN-2 RECEPTOR ALPHA SUBUNIT) humanized monoclonal antibody (immunoglobulin G1 disulfide with human-mouse monoclonal clone 1H4 light chain, dimer) that is used in the treatment of ACUTE RELAPSING MULTIPLE SCLEROSIS.'),
  'D000077562': NtO(MH='Valganciclovir', MS='A ganciclovir prodrug and antiviral agent that is used to treat CYTOMEGALOVIRUS RETINITIS in patients with AIDS, and for the prevention of CYTOMEGALOVIRUS INFECTIONS in organ transplant recipients who have received an organ from a CMV-positive donor.'),
  'D000077563': NtO(MH='Norethindrone Acetate', MS='Acetate ester of norethindrone that is used as a long-term contraceptive (CONTRACEPTIVE AGENTS).'),
  'D000077564': NtO(MH='Focused Assessment with Sonography for Trauma', MS='Rapid bedside ultrasound examination performed by medical personnel (surgeons, emergency physicians, paramedics and others), as a screening test for blood around the heart (PERICARDIAL EFFUSION) or abdominal organs (HEMOPERITONEUM) after trauma.'),
  'D000077582': NtO(MH='Amisulpride', MS='A benzamide derivative that is used as an antipsychotic agent for the treatment of schizophrenia. It is also used as an antidepressive agent.'),
  'D000077583': NtO(MH='Quinapril', MS='A tetrahydroisoquinoline derivative and ANGIOTENSIN CONVERTING ENZYME inhibitor that is used in the treatment of HYPERTENSION and HEART FAILURE.'),
  'D000077584': NtO(MH='2-Methoxyestradiol', MS='A metabolite of estradiol that lacks estrogenic activity and inhibits TUBULIN polymerization. It has antineoplastic properties, including inhibition of angiogenesis and induction of APOPTOSIS.'),
  'D000077585': NtO(MH='Terlipressin', MS='An inactive peptide prodrug that is slowly converted in the body to lypressin. It is used to control bleeding of ESOPHAGEAL VARICES and for the treatment of HEPATORENAL SYNDROME.'),
  'D000077587': NtO(MH='Natural Disasters', MS='Disasters linked to natural hazards including widespread fires, floods, storms, earthquakes and drought.These events may result in significant damage and loss of lives.'),
  'D000077588': NtO(MH='Deferasirox', MS='A triazole and benzoate derivative that acts as a selective iron chelator. It is used in the management of chronic IRON OVERLOAD due to blood transfusion or non-transfusion dependent THALASSEMIA.'),
  'D000077589': NtO(MH='Sulfathiazole', MS='A sulfathiazole compound that is used as a short-acting anti-infective agent. It is no longer commonly used systemically due to its toxicity, but may still be applied topically in combination with other drugs for the treatment of vaginal and skin infections, and is still used in veterinary medicine.'),
  'D000077590': NtO(MH='Mivacurium', MS='An isoquinoline derivative that is used as a short-acting non-depolarizing agent.'),
  'D000077591': NtO(MH='Eucalyptol', MS='A monoterpene and cyclohexanol derivative that is the major component of EUCALYPTUS OIL. It is used in mouthwash, insect repellent, and as a cough suppressant, and also is widely used as a flavoring agent and solvent. It has antimicrobial properties.'),
  'D000077592': NtO(MH='Maraviroc', MS='A cyclohexane and triazole derivative that acts as an antagonist of the CCR5 RECEPTOR. It prevents infection by HIV-1 virus strains which use CCR5 as a co-receptor for membrane fusion and cellular entry.'),
  'D000077593': NtO(MH='Reboxetine', MS='A morpholine derivative that is a selective and potent noradrenaline reuptake inhibitor; it is used in the treatment of DEPRESSIVE DISORDER.'),
  'D000077594': NtO(MH='Nivolumab', MS="A genetically engineered, fully humanized immunoglobulin G4 monoclonal antibody that binds to the PD-1 RECEPTOR, activating an immune response to tumor cells. It is used as monotherapy or in combination with IPILIMUMAB for the treatment of advanced malignant MELANOMA. It is also used in the treatment of advanced or recurring NON-SMALL CELL LUNG CANCER; RENAL CELL CARCINOMA; and HODGKIN'S LYMPHOMA."),
  'D000077595': NtO(MH='Famciclovir', MS='An aminopurine derivative and prodrug of penciclovir which is a competitive inhibitor of herpes simplex 2 DNA polymerase. It is used to treat HERPES SIMPLEX VIRUS INFECTION.'),
  'D000077596': NtO(MH='Thymalfasin', MS='A thymus hormone polypeptide found in thymosin fraction 5 (a crude thymus gland extract) but now produced by synthesis. It is used alone or with interferon as an immunomodulator for the treatment of CHRONIC HEPATITIS B and HEPATITIS C. Thymalfasin is also used for the treatment of chemotherapy-induced immunosuppression, and to enhance the efficacy of influenza and hepatitis B vaccines in immunocompromised patients.'),
  'D000077597': NtO(MH='Vildagliptin', MS='A pyrrolidine-carbonitrile derivative and potent inhibitor of DIPEPTIDYL PEPTIDASE 4 that is used in the treatment of TYPE 2 DIABETES MELLITUS.'),
  'D000077602': NtO(MH='Tolvaptan', MS='A benzazepine derivative and selective VASOPRESSIN V2 RECEPTOR antagonist that is used to treat euvolemic and hypervolemic HYPONATREMIA. It is also used in the treatment of rapidly progressing AUTOSOMAL DOMINANT POLYCYSTIC KIDNEY DISEASE to slow the rate of cyst development and renal insufficiency.'),
  'D000077603': NtO(MH='Nandrolone Decanoate', MS='Decanoic acid ester of nandrolone that is used as an anabolic agent to prevent or treat WASTING SYNDROME associated with severe chronic illness or HIV infection (HIV WASTING SYNDROME). It may also be used in the treatment of POSTMENOPAUSAL OSTEOPOROSIS.'),
  'D000077604': NtO(MH='Fomepizole', MS='A pyrazole and competitive inhibitor of ALCOHOL DEHYDROGENASE that is used for the treatment of poisoning by ETHYLENE GLYCOL or METHANOL.'),
  'D000077605': NtO(MH='Ferric Oxide, Saccharated', MS='A glucaric acid-iron conjugate that is used in the treatment of IRON-DEFICIENCY ANEMIA, including in patients with chronic kidney disease, when oral iron therapy is ineffective or impractical.'),
  'D000077606': NtO(MH='Trabectedin', MS='A complex structure that includes isoquinolines joined by a cyclic ester; it is a DNA-binding agent and guanine N2 alkylator derived from the marine tunicate, Ecteinascidia turbinata. Trabectedin is used for the treatment of advanced soft-tissue SARCOMA, after failure of ANTHRACYCLINES or IFOSFAMIDE drug therapy.'),
  'D000077607': NtO(MH='Icodextrin', MS='A glucan that is structurally related to maltodextrin, with more than 85% of its molecules having molecular weights between 1640 and 45 000 Daltons (Da), and a weight-average molecular weight of about 20 000 Da; it is used in dialysis fluids as an alternative to glucose-based solutions, and to reduce adhesions after gynecological or abdominal surgery. It has also been used as a vehicle for drugs given via the peritoneal cavity.'),
  'D000077608': NtO(MH='Aprepitant', MS='A morpholine neurokinin-1 (NK1) receptor antagonist that is used in the management of nausea and vomiting caused by DRUG THERAPY, and for the prevention of POSTOPERATIVE NAUSEA AND VOMITING.'),
  'D000077609': NtO(MH='O-(Chloroacetylcarbamoyl)fumagillol', MS='Semisynthetic analog of fumagillin (a cyclohexane-sesquiterpene antibiotic isolated from ASPERGILLUS FUMIGATUS) that inhibits angiogenesis.'),
  'D000077610': NtO(MH='Bexarotene', MS='A tetrahydronaphthalene derivative and RETINOID X RECEPTOR antagonist that is used in the treatment of CUTANEOUS T-CELL LYMPHOMA.'),
  'D000077611': NtO(MH='Artemether, Lumefantrine Drug Combination', MS='Drug combination of artemether and lumefantrine that is used to treat PLASMODIUM FALCIPARUM MALARIA.'),
  'D000077612': NtO(MH='Anidulafungin', MS='Echinocandin antifungal agent that is used in the treatment of CANDIDEMIA and CANDIDIASIS.'),
  'D000077613': NtO(MH='Etoricoxib', MS='A sulfone and pyridine derivative that acts as a cyclooxygenase-2 inhibitor. It is used as a NSAID for the treatment of pain associated with RHEUMATOID ARTHRITIS and ANKYLOSING SPONDYLITIS. It is also used for the short-term treatment of moderate postoperative dental pain.'),
  'D000077614': NtO(MH='Golgi Matrix Proteins', MS='Proteins on the cytoplasmic side of the GOLGI APPARATUS cisternae that tether the cisternae to each other or to the transport vesicles of the TRANS-GOLGI NETWORK to maintain the Golgi structure and function.'),
  'D000077626': NtO(MH='Implementation Science', MS='The study of methods to promote the adoption and integration of evidence-based practices, interventions and policies into routine health care and public health settings.'),
  'D000077642': NtO(MH='Family Separation', MS='Geographic separation of members of a family.'),
  'D000077682': NtO(MH='Triclabendazole', MS='Benzimidazole antiplatyhelmintic agent that is used for the treatment of FASCIOLIASIS and PARAGONIMIASIS.'),
  'D000077684': NtO(MH='Cauda Equina Syndrome', MS='Compressive lesion affecting the nerve roots of the CAUDA EQUINA (e.g., compression, herniation, inflammation, rupture, or stenosis), which controls the function of the bladder and bowel. Symptoms may include neurological dysfunction of bladder or bowels, loss of sexual sensation and altered sensation or paralysis in the lower extremities.'),
  'D000077704': NtO(MH='Tirapazamine', MS='A triazine derivative that introduces breaks into DNA strands in hypoxic cells, sensitizing tumor cells to the cytotoxic activity of other drugs and radiation.'),
  'D000077705': NtO(MH='Screen Time', MS='Period of activities done in front of an electronic screen, such as watching TV, working on a computer, or playing video games.'),
  'D000077708': NtO(MH='Sitting Position', MS=None),
  'D000077711': NtO(MH='Self-Neglect', MS='Profound inattention by individuals to their own health and hygiene.'),
  'D000077712': NtO(MH='Telbivudine', MS='A thymidine derivative and antiviral agent that inhibits DNA synthesis by HEPATITIS B VIRUS and is used for the treatment of CHRONIC HEPATITIS B.'),
  'D000077713': NtO(MH='17 alpha-Hydroxyprogesterone Caproate', MS='Hydroxyprogesterone derivative that acts as a PROGESTIN and is used to reduce the risk of recurrent MISCARRIAGE and of PREMATURE BIRTH. It is also used in combination with ESTROGEN in the management of MENSTRUATION DISORDERS.'),
  'D000077715': NtO(MH='Nateglinide', MS='A phenylalanine and cyclohexane derivative that acts as a hypoglycemic agent by stimulating the release of insulin from the pancreas. It is used in the treatment of TYPE 2 DIABETES.'),
  'D000077716': NtO(MH='Afatinib', MS='A quinazoline and butenamide derivative that acts as a tyrosine kinase inhibitor of epidermal growth factor receptors (ERBB RECEPTORS) and is used in the treatment of metastatic NON-SMALL CELL LUNG CANCER.'),
  'D000077722': NtO(MH='Ceftibuten', MS='A cephalosporin antibacterial agent that is used in the treatment of infections, including urinary-tract and respiratory-tract infections.'),
  'D000077723': NtO(MH='Cefepime', MS='A fourth-generation cephalosporin antibacterial agent that is used in the treatment of infections, including those of the abdomen, urinary tract, respiratory tract, and skin. It is effective against PSEUDOMONAS AERUGINOSA and may also be used in the empiric treatment of FEBRILE NEUTROPENIA.'),
  'D000077725': NtO(MH='Piperacillin, Tazobactam Drug Combination', MS='An antibiotic combination product of piperacillin and tazobactam, a penicillanic acid derivative with enhanced beta-lactamase inhibitory activity, that is used for the intravenous treatment of intra-abdominal, pelvic, and skin infections and for community-acquired pneumonia of moderate severity. It is also used for the treatment of PSEUDOMONAS AERUGINOSA INFECTIONS.'),
  'D000077726': NtO(MH='Doripenem', MS='A carbapenem derivative antibacterial agent that is more stable to renal dehydropeptidase I than IMIPENEM, but does not need to be given with an enzyme inhibitor such as CILASTATIN. It is used in the treatment of infections such as HOSPITAL-ACQUIRED PNEUMONIA, and complicated intra-abdominal or urinary-tract infections, including PYELONEPHRITIS.'),
  'D000077727': NtO(MH='Ertapenem', MS='A carbapenem derivative antibacterial agent that is more stable to renal dehydropeptidase I than IMIPENEM, but does not need to be given with an enzyme inhibitor such as CILASTATIN. It is used in the treatment of Gram-positive and Gram-negative bacterial infections including intra-abdominal infections, acute gynecological infections, complicated urinary tract infections, skin infections, and respiratory tract infections. It is also used to prevent infection in colorectal surgery.'),
  'D000077728': NtO(MH='Cilastatin, Imipenem Drug Combination', MS='Combination of imipenem and cilastatin that is used in the treatment of bacterial infections; cilastatin inhibits renal dehydropeptidase I to prolong the half-life and increase the tissue penetration of imipenem, enhancing its efficacy as an anti-bacterial agent.'),
  'D000077731': NtO(MH='Meropenem', MS='A thienamycin derivative antibacterial agent that is more stable to renal dehydropeptidase I than IMIPENEM, but does not need to be given with an enzyme inhibitor such as CILASTATIN. It is used in the treatment of bacterial infections, including infections in immunocompromised patients.'),
  'D000077732': NtO(MH='Fidaxomicin', MS='A narrow-spectrum macrolide antibacterial agent that is used in the treatment of diarrhea associated with CLOSTRIDIUM DIFFICILE INFECTION.'),
  'D000077733': NtO(MH='Immunoglobulin G4-Related Disease', MS='A spectrum of systemic autoimmune diseases in which IMMUNOGLOBULIN G4 plays a pathophysiologic role. It can affect multiple organs in highly variable presentations, characterized by inflammatory lesions composed of IgG4-positive PLASMA CELLS, further infiltrated by T helper cells (T-LYMPHOCYTES, HELPER-INDUCER) when linked to progressive FIBROSIS and eventual organ damage.'),
  'D000077734': NtO(MH='Gatifloxacin', MS='A fluoroquinolone antibacterial agent and DNA TOPOISOMERASE II inhibitor that is used as an ophthalmic solution for the treatment of BACTERIAL CONJUNCTIVITIS.'),
  'D000077735': NtO(MH='Gemifloxacin', MS='A naphthyridine and fluoroquinolone derivative antibacterial agent and DNA TOPOISOMERASE II inhibitor that is used for the treatment of community-acquired pneumonia and acute bacterial infections associated with chronic bronchitis.'),
  'D000077740': NtO(MH='Procalcitonin', MS='A peptide prohormone precursor of CALCITONIN. It is normally present at low levels in serum, but is released into the bloodstream, primarily from neuroendocrine cells in the lungs and intestines, in response to INFLAMMATION and BACTERIAL INFECTIONS. It is a diagnostic marker for BACTEREMIA.'),
  'D000077743': NtO(MH='Diterpene Alkaloids', MS='Alkaloids derived from diterpenes (e.g., KAURANES; CLERODANES) often containing the nitrogen bridge structures. These alkaloids may be divided into C18-, C19-, C20- and bis-diterpene types depending on the number of carbon atoms in their central framework. They are found most often in RANUNCULACEAE (e.g., ACONITUM and DELPHINIUM) and may possess analgesic, antiarrhythmic and anti-inflammatory activities. Some are cardiotoxic by blocking L-TYPE CALCIUM CHANNELS.'),
  'D000077744': NtO(MH='Orthodontic Appliances, Fixed', MS='ORTHODONTIC APPLIANCES attached to the teeth by adhesive material which cannot be removed from the mouth by the patient. These devices apply force to the teeth and supporting structures, thereby producing changes which control their growth and development.'),
  'D000077764': NtO(MH='Dronedarone', MS='A non-iodinated derivative of amiodarone that is used for the treatment of ARRHYTHMIA.'),
  'D000077765': NtO(MH='Cone Dystrophy', MS='A general term which describes a group of rare eye disorders that affect the cone cells of the RETINA. Cone dystrophy can cause a variety of symptoms including decreased visual clarity or acuity when looking straight ahead (central vision), a reduced ability to see colors, and an increased sensitivity to light (PHOTOPHOBIA).'),
  'D000077767': NtO(MH='Panobinostat', MS='An indole and hydroxamic acid derivative that acts as a HISTONE DEACETYLASE inhibitor. It is used as an antineoplastic agent in combination with BORTEZOMIB and DEXAMETHASONE for the treatment of MULTIPLE MYELOMA.'),
  'D000077768': NtO(MH='Ciclopirox', MS='A cyclohexane and pyridinone derivative that is used for the treatment of fungal infections of the skin and nails, and for treatment of VAGINAL YEAST INFECTIONS.'),
  'D000077769': NtO(MH='Rilmenidine', MS='Oxazole derivative that acts as an agonist for ALPHA-2 ADRENERGIC RECEPTORS and IMIDAZOLINE RECEPTORS. It is used in the treatment of HYPERTENSION.'),
  'D000077770': NtO(MH='Amifampridine', MS='4-Aminopyridine derivative that acts as a POTASSIUM CHANNEL blocker to increase release of ACETYLCHOLINE from nerve terminals. It is used in the treatment of CONGENITAL MYASTHENIC SYNDROMES.'),
  'D000077771': NtO(MH='Methantheline', MS='A quaternary ammonium compound that acts as an antimuscarinic agent. It has been used in the treatment of PEPTIC ULCER, in gastrointestinal disorders associated with smooth muscle spasm, and in the management of urinary incontinence, and may also be used for the treatment of HYPERHIDROSIS.'),
  'D000077772': NtO(MH='Shared Governance, Nursing', MS='Model of nursing practice with shared decision processes designed to integrate core values and beliefs that professional practice embraces, as a means of achieving quality care.'),
  'D000077773': NtO(MH='Hospitals, Rehabilitation', MS='Free standing rehabilitation hospitals and rehabilitation units in acute care hospitals.  They provide an intensive inpatient rehabilitation program.'),
  'D000077776': NtO(MH='Insurance, Vision', MS='Insurance providing coverage for vision care.'),
  'D000077777': NtO(MH='Myopericytoma', MS='Rare myofibroma-like perivascular soft tissue tumor, usually benign, characterized by concentric perivascular proliferation.'),
  'D000077779': NtO(MH='Pancreatic Intraductal Neoplasms', MS='EPITHELIAL NEOPLASMS with pancreatic ductal differentiation that arise from the EXOCRINE PANCREAS and grow mostly within the PANCREATIC DUCTS. They are characterized by the presence of intraductal masses with mostly papillary growth pattern. Overt production of MUCIN within the pancreatic duct is associated with pancreatic intraductal papillary-mucinous neoplasm but not intraductal tubulopapillary neoplasm.'),
  'D000077780': NtO(MH='Plastination', MS='Preservation technique in which epoxy polymers are used to preserve tissue or an organ resulting in anatomically authentic dry specimens often used in teaching of anatomy.'),
  'D000077784': NtO(MH='Axitinib', MS='A benzamide and indazole derivative that acts as a TYROSINE KINASE inhibitor of the VASCULAR ENDOTHELIAL GROWTH FACTOR RECEPTOR. It is used in the treatment of advanced RENAL CELL CARCINOMA.'),
  'D000077785': NtO(MH='Tenecteplase', MS='A tissue plasminogen activator enzyme that acts as a fibrinolytic agent; it is used for the dissolution of blood clots, such as those that occur in acute MYOCARDIAL INFARCTION.'),
  'D000077786': NtO(MH='Torsemide', MS='A pyridine and sulfonamide derivative that acts as a sodium-potassium chloride symporter inhibitor (loop diuretic). It is used for the treatment of EDEMA associated with CONGESTIVE HEART FAILURE; CHRONIC RENAL INSUFFICIENCY; and LIVER DISEASES. It is also used for the management of HYPERTENSION.'),
  'D000077803': NtO(MH='Dictionary, Classical', MS='A reference work containing a list of words related to the Greco-Roman World giving information about form, pronunciation, etymology, grammar, and meaning.'),
  'D000077823': NtO(MH='Dictionary, Medical', MS='A reference book containing a list of words related to medicine-usually in alphabetic order-giving information about form, pronunciation, etymology, grammar, and meaning.'),
  'D000077824': NtO(MH='Dictionary, Dental', MS='A reference book containing a list of words related to dentistry-usually in alphabetic order-giving information about form, pronunciation, etymology, grammar, and meaning.'),
  'D000077825': NtO(MH='Dictionary, Pharmaceutic', MS='A reference book containing a list of words related to pharmacy-usually in alphabetic order-giving information about form, pronunciation, etymology, grammar, and meaning.'),
  'D000077844': NtO(MH='Dictionary, Chemical', MS='A reference book containing a list of words related to chemistry-usually in alphabetic order-giving information about form, pronunciation, etymology, grammar, and meaning.'),
  'D000077845': NtO(MH='Lipoabdominoplasty', MS='Aesthetic surgical treatment of the abdominal region using LIPOSUCTION associated with ABDOMINOPLASTY.'),
  'D000077862': NtO(MH='Manuscript, Medical', MS='Medical works prepared by hand including handwritten or typescript drafts of pre-publication papers or works not otherwise reproduced in multiple copies.'),
  'D000077863': NtO(MH='Homoharringtonine', MS='Semisynthetic derivative of harringtonine that acts as a protein synthesis inhibitor and induces APOPTOSIS in tumor cells. It is used in the treatment of MYELOID LEUKEMIA, CHRONIC.'),
  'D000077866': NtO(MH='Clofarabine', MS='An adenine arabinonucleoside derivative that acts as an antineoplastic antimetabolite. It is used in the treatment of ACUTE LYMPHOID LEUKEMIA in pediatric patients who have relapsed.'),
  'D000077867': NtO(MH='Tolcapone', MS='A benzophenone and nitrophenol compound that acts as an inhibitor of CATECHOL O-METHYLTRANSFERASE, an enzyme involved in the metabolism of DOPAMINE and LEVODOPA. It is used in the treatment of PARKINSON DISEASE in patients for whom levodopa is ineffective or contraindicated.'),
  'D000077868': NtO(MH='Atrasentan', MS='A pyrrolidine and benzodioxole derivative that acts a RECEPTOR, ENDOTHELIN A antagonist. It has therapeutic potential as an antineoplastic agent and for the treatment of DIABETIC NEPHROPATHIES.'),
  'D000077922': NtO(MH='Thiamethoxam', MS='A nitro-oxazine and thiazole derivative that is used as a broad spectrum neonicotinoid insecticide.'),
  'D000077923': NtO(MH='Societies, Veterinary', MS='Organizations having institutional membership limited to veterinarians and veterinary institutions.'),
  'D000077924': NtO(MH='Palonosetron', MS='Isoquinoline and quinuclidine derivative that acts as a 5-HT3 RECEPTOR antagonist. It is used in the prevention of nausea and vomiting induced by cytotoxic chemotherapy, and for the prevention of post-operative nausea and vomiting.'),
  'D000077944': NtO(MH='Alefacept', MS='A recombinant human fusion protein of LFA-3 antigen and immunoglobulin IGG1 that binds to CD2 ANTIGENS on memory T-LYMPHOCYTES, preventing their activation and proliferation. It is used in the management of moderate to severe chronic plaque PSORIASIS.'),
  'D000077962': NtO(MH='Body-Weight Trajectory', MS='A general pattern of body weight gain or loss over many years. Weight change trajectory is influenced by several determinants in children and adults.'),
  'D000077982': NtO(MH='Progression-Free Survival', MS='Length of time during and after the treatment of a disease, such as cancer, that a patient lives with the disease but the disease does not get worse.'),
  'D000078002': NtO(MH='Posttraumatic Growth, Psychological', MS='Positive change experienced as a result of the struggle with a major life crisis or a traumatic event.'),
  'D000078003': NtO(MH='Change Management', MS='Processes, tools, and techniques involved in the management of ORGANIZATIONAL INNOVATION, the purpose of which is to help individuals and work groups make successful personal transitions which ensure overall successful outcomes.'),
  'D000078022': NtO(MH='Economic Status', MS='Personal or family variables such as household income, assets such as inherited wealth, savings, employment benefits, or ownership of homes or other property.'),
  'D000078062': NtO(MH='Leucanthemum', MS='A genus of flowering plants in the family ASTERACEAE, mainly distributed in southern and central Europe, and on other continents as introduced species. Leucanthemum species include annuals and perennials grown for their daisy flowers, which are usually gleaming white with yellow centers.'),
  'D000078064': NtO(MH='Gestational Weight Gain', MS='Increase in body weight of the mother during the course of her PREGNANCY.'),
  'D000078065': NtO(MH='Aegilops', MS='A genus of grasses known generally as goatgrasses, presumed to be wild ancestors of domestic WHEAT. They belong to the grass family, POACEAE, and are distributed widely in Eurasia and North America.'),
  'D000078102': NtO(MH='Lumefantrine', MS='A fluorene derivative that is used in combination with ARTEMETHER for the treatment of MALARIA (see ARTEMETHER-LUMEFANTRINE DRUG COMBINATION).'),
  'D000078122': NtO(MH='Eucalyptus Oil', MS='An oily liquid that is extracted from the leaves of EUCALYPTUS trees.'),
  'D000078142': NtO(MH='Tazobactam', MS='A penicillanic acid and sulfone derivative and potent BETA-LACTAMASE inhibitor that enhances the activity of other anti-bacterial agents against beta-lactamase producing bacteria.'),
  'D000078162': NtO(MH='Tobramycin, Dexamethasone Drug Combination', MS='A topical preparation of tobramycin and dexamethasone that is used for treating or preventing superficial bacterial infections of the eye.'),
  'D000078182': NtO(MH='Systematic Review', MS='A review of primary literature in health and health policy that attempts to identify, appraise, and synthesize all the empirical evidence that meets specified eligibility criteria to answer a given research question.  Its conduct uses explicit methods aimed at minimizing bias in order to produce more reliable findings regarding the effects of interventions for prevention, treatment, and rehabilitation that can be used to inform decision making.'),
  'D000078183': NtO(MH='Oxindoles', MS='Benzopyrroles with the nitrogen at the number one carbon adjacent to the benzyl and a carbonyl at the pyrrole ring typically at the 2-position.'),
  'D000078202': NtO(MH='Systematic Reviews as Topic', MS='Works about a review of primary literature in health and health policy that attempt to identify, appraise, and synthesize all the empirical evidence that meets specified eligibility criteria to answer a given research question. Its conduct uses explicit methods aimed at minimizing bias in order to produce more reliable findings regarding the effects of interventions for prevention, treatment, and rehabilitation that can be used to inform decision making.'),
  'D000078222': NtO(MH='Tinzaparin', MS='A low-molecular-weight heparin with anticoagulant properties that is used in the prevention and treatment of VENOUS THROMBOEMBOLISM, and to prevent clotting during EXTRACORPOREAL CIRCULATION.'),
  'D000078223': NtO(MH='5-Methoxypsoralen', MS='A linear furanocoumarin that has phototoxic and anti-inflammatory properties, with effects similar to METHOXSALEN. It is used in PUVA THERAPY for the treatment of PSORIASIS.'),
  'D000078224': NtO(MH='Lenograstim', MS='A recombinant granulocyte colony-stimulating factor that is used to treat or prevent NEUTROPENIA in patients receiving myelosuppressive cancer chemotherapy, for the preparation and collection of blood progenitor cells in PERIPHERAL BLOOD STEM CELL TRANSPLANTATION; and in the treatment of severe congenital neutropenia. Component 1 and Component 2 differ by absence or presence of Neu5Ac in 2-6 linkage to GalNAc side chain.'),
  'D000078262': NtO(MH='Rifaximin', MS='A synthetic rifamycin derivative and anti-bacterial agent that is used for the treatment of GASTROENTERITIS caused by ESCHERICHIA COLI INFECTIONS. It may also be used in the treatment of HEPATIC ENCEPHALOPATHY.'),
  'D000078303': NtO(MH='Data Aggregation', MS='Process of searching, gathering, and presenting data in a summarized format.'),
  'D000078304': NtO(MH='Tigecycline', MS='A tetracycline derivative that acts as a protein synthesis inhibitor. It is used as an antibacterial agent for the systemic treatment of complicated skin and intra-abdominal infections. It is also used for the treatment of community-acquired pneumonia.'),
  'D000078305': NtO(MH='Zonisamide', MS='A benzisoxazole and sulfonamide derivative that acts as a CALCIUM CHANNEL blocker. It is used primarily as an adjunctive antiepileptic agent for the treatment of PARTIAL SEIZURES, with or without secondary generalization.'),
  'D000078306': NtO(MH='Clobazam', MS='A benzodiazepine derivative that is a long-acting GABA-A RECEPTOR agonist. It is used as an antiepileptic in the treatment of SEIZURES, including seizures associated with LENNOX-GASTAUT SYNDROME.  It is also used as an anxiolytic, for the short-term treatment of acute ANXIETY.'),
  'D000078308': NtO(MH='Tiagabine', MS='A nipecotic acid derivative that acts as a GABA uptake inhibitor and anticonvulsant agent. It is used in the treatment of EPILEPSY, for refractory PARTIAL SEIZURES.'),
  'D000078323': NtO(MH='Observational Study, Veterinary', MS='Reports on studies of the results of a clinical study in which animal subjects may receive diagnostic, therapeutic, or other types of interventions, where the investigator does not assign participants to specific interventions.'),
  'D000078325': NtO(MH='Clinical Trial Protocol', MS="The written description of a clinical study. It contains the study's objectives, design, and methods including subject target and/or enrollment criteria. It may also present relevant scientific background and statistical information."),
  'D000078326': NtO(MH='Data Visualization', MS='Facilitating understanding of the significance of data by placing it in a visual context. Patterns, trends and correlations that might go undetected in text-based data may be exposed and recognized easier.'),
  'D000078327': NtO(MH='Data Systems', MS='Organized collection of symbols and symbol-manipulating processes used to operate on such symbols.'),
  'D000078328': NtO(MH='Felbamate', MS='A PEGylated phenylcarbamate derivative that acts as an antagonist of NMDA RECEPTORS. It is used as an anticonvulsant, primarily for the treatment of SEIZURES in severe refractory EPILEPSY.'),
  'D000078329': NtO(MH='Workforce', MS='The number of people working or available for work or service.'),
  'D000078330': NtO(MH='Oxcarbazepine', MS='A carbamazepine derivative that acts as a voltage-gated sodium channel blocker. It is used for the treatment of PARTIAL SEIZURES with or without secondary generalization. It is also an inducer of CYTOCHROME P-450 CYP3A4.'),
  'D000078331': NtO(MH='Correlation of Data', MS='Determination of whether or not two variables are correlated. This means to study whether an increase or decrease in one variable corresponds to an increase or decrease in the other variable.'),
  'D000078332': NtO(MH='Data Analysis', MS='Process of systematically applying statistical and/or logical techniques to describe and illustrate, condense and recap, and evaluate data (https://ori.hhs.gov/education).'),
  'D000078334': NtO(MH='Lacosamide', MS='An acetamide derivative that acts as a blocker of VOLTAGE-GATED SODIUM CHANNELS. It is used as an anticonvulsant, for adjunctive or monotherapy, in the treatment of PARTIAL SEIZURES.'),
  'D000078336': NtO(MH='Facilities and Services Utilization', MS='Extent of use of facilities or services. It includes overuse and underuse.'),
  'D000078337': NtO(MH='Procedures and Techniques Utilization', MS='Extent of use of procedures or techniques. It includes overuse and underuse.'),
  'D000078338': NtO(MH='Equipment and Supplies Utilization', MS='Extent of use of equipment or supplies. It includes overuse and underuse.'),
  'D000078382': NtO(MH='Disgust', MS='A feeling of revulsion or strong disapproval aroused by something unpleasant or offensive.'),
  'D000078383': NtO(MH='Conscientious Refusal to Treat', MS='Refusal of health professionals to provide medical services on the basis of moral or religious beliefs.'),
  'D000078385': NtO(MH='Embarrassment', MS='A feeling of self-consciousness, shame, or awkwardness.'),
  'D000078402': NtO(MH='Gun Violence', MS='Violence using any type of firearm.'),
  'D000078404': NtO(MH='Epidermal Cells', MS='Cells from the outermost, non-vascular layer (EPIDERMIS) of the skin.'),
  'D000078405': NtO(MH='Non-Smokers', MS='Persons without a history of SMOKING.'),
  'D000078422': NtO(MH='Allogeneic Cells', MS='Cells from genetically different individuals of the same species to be used for CELL- AND TISSUE-BASED THERAPY; ADOPTIVE CELL TRANSFER; or ADOPTIVE IMMUNOTHERAPY.'),
  'D000078462': NtO(MH='Therapeutic Alliance', MS='A relational factor in counseling that includes three dimensions: goal consensus between counselor and client, collaboration on counseling-related tasks, and emotional bonding.  It is referred to as the working alliance between a therapist and a client in psychotherapy and closely correlated with positive client outcomes.'),
  'D000078463': NtO(MH='Child Labor', MS='Employment of MINORS in work that deprives children of their childhood, potential, dignity, and is harmful to their physical and mental development. (International Labor Organization, UN)'),
  'D000078482': NtO(MH='Prior Authorization', MS='A decision by a health insurer or plan that a health care service, treatment plan, prescription drug or durable medical equipment is medically necessary.'),
  'D000078483': NtO(MH='Regenerative Endodontics', MS='A dental specialty concerned with biologically-based procedures designed to physiologically replace damaged tooth structures, including dentin and root structures, as well as cells of the pulp-dentin complex.'),
  'D000078502': NtO(MH='Meat Proteins', MS='Dietary proteins from MEAT.'),
  'D000078503': NtO(MH='Poultry Proteins', MS='Dietary proteins from POULTRY.'),
  'D000078504': NtO(MH='Fish Proteins, Dietary', MS='Dietary protein from FISH.'),
  'D000078522': NtO(MH='Grain Proteins', MS='Dietary protein from EDIBLE GRAINS.'),
  'D000078542': NtO(MH='Proctectomy', MS='Surgical resection of the RECTUM.'),
  'D000078562': NtO(MH='Fruit Proteins', MS='Proteins found in FRUIT.'),
  'D000078563': NtO(MH='Nut Proteins', MS='Proteins from NUTS.'),
  'D000078582': NtO(MH='Shellfish Proteins', MS='Proteins found in SHELLFISH.'),
  'D000078602': NtO(MH='Sadness', MS='Feeling or showing sorrow; the sense of being unhappy. It is related to DEPRESSION but is not synonymous.'),
  'D000078604': NtO(MH='Secretagogues', MS='Substances that stimulate the secretion of another substance.'),
  'D000078622': NtO(MH='Nutrients', MS='Various components of food that are required for nourishment.'),
  'D000078623': NtO(MH='Wool Fiber', MS='Filament, thread, or yarn produced from WOOL.'),
  'D000078625': NtO(MH='Euthanasia, Involuntary', MS='The act or practice of killing or withholding life-sustaining treatment from a person with a medical condition, especially DISABLED PERSONS, without INFORMED CONSENT, despite the MENTAL COMPETENCY of the person to give consent.'),
  'D000078642': NtO(MH='Medical Countermeasures', MS='Regulated products (biologics, drugs, devices) that may be used in the event of a potential public health emergency stemming from a terrorist attack with a biological, chemical, or radiological/nuclear material, or a naturally occurring emerging disease.'),
  'D000078646': NtO(MH='Diaries as Topic', MS="Works on or about records, usually private, of writers' experiences, observations, feelings, attitudes, etc. They may also be works marked in calendar order in which to note appointments and the like."),
  'D000078662': NtO(MH='Collections as Topic', MS='Works about items gathered and assembled in some order to facilitate access or use.'),
  'D000078663': NtO(MH='Egocentrism', MS="Self-centered preoccupation or concern regarding one's own needs, abilities, wishes, desires, or preferences."),
  'D000078665': NtO(MH='Clinical Trials, Veterinary as Topic', MS='Works about clinical studies in which animal participants are assigned to receive one or more interventions so that researchers can evaluate the interventions on biomedical or health-related outcomes. The assignments are determined by the study protocol. Animal participants may receive diagnostic, therapeutic, or other types of interventions.'),
  'D000078682': NtO(MH='Respect', MS='A feeling of deep admiration for someone or something elicited by their abilities, qualities, or achievements and regard for the feelings, wishes, or rights of others.'),
  'D000078702': NtO(MH='Radiofrequency Therapy', MS='The use of RADIO WAVES to conduct therapeutic procedures. The specific frequency, dosage, and intensity used vary depending on the desired effects which include heating, electrical stimulation, or ablation of tissues.'),
  'D000078703': NtO(MH='Radiofrequency Ablation', MS='Removal of tissue using heat generated from electrodes delivering an alternating electrical current in the frequency of RADIO WAVES.'),
  'D000078704': NtO(MH='Quantitative Light-Induced Fluorescence', MS='A FLUORESCENCE technique used in DENTISTRY for the analysis of mineral changes in DENTAL ENAMEL to detect early DENTAL CARIES, or for the evaluation other normal or pathological processes or lesions of the dental enamel.'),
  'D000078722': NtO(MH='PC-3 Cells', MS='An epithelial cell line initiated from a human bone metastasis of a grade IV prostatic ADENOCARCINOMA.'),
  'D000078723': NtO(MH='Nuclear Receptor Interacting Protein 1', MS='A nuclear protein, characterized by multiple LXXLL motifs, that modulates activation of gene transcription by STEROID RECEPTORS, and transcriptional repression by NUCLEAR HORMONE RECEPTORS. It positively regulates gene expression of CIRCADIAN CLOCK PROTEINS AND PEPTIDES, including ARNTL TRANSCRIPTION FACTORS and CRYPTOCHROMES.'),
  'D000078724': NtO(MH='Pea Proteins', MS='Dietary proteins that are present in or isolated from the seeds of PISUM SATIVUM.'),
  'D000078742': NtO(MH='Substandard Drugs', MS='Drugs formulations that do not meet established standards.'),
  'D000078763': NtO(MH='Forms as Topic', MS='Works about documents used for acquiring particular information or for presenting particular information in a prescribed sequence and format, often with blank spaces or lines or other methods to prompt for insertion of the requested information.'),
  'D000078764': NtO(MH='Milnacipran', MS='A cyclopropanecarboxamide serotonin and norepinephrine reuptake inhibitor (SNRI) that is used in the treatment of FIBROMYALGIA.'),
  'D000078782': NtO(MH='Vaccinology', MS='Branch of medicine concerned with the development of vaccines to control disease by identifying genetic and other mechanisms and pathways that determine immune responses, and thereby provide new candidate vaccine approaches.'),
  'D000078783': NtO(MH='Standing Position', MS='Human position in which the body is held in an upright orthostatic position and supported only by the feet.'),
  'D000078784': NtO(MH='Vortioxetine', MS='A piperazine derivative that acts as a serotonin reuptake inhibitor, as a 5-HT3 receptor antagonist, and 5-HT1A receptor agonist. It is used for the treatment of anxiety and depression.'),
  'D000078785': NtO(MH='Mirtazapine', MS='A piperazinoazepine tetracyclic compound that enhances the release of NOREPINEPHRINE and SEROTONIN through blockage of presynaptic ALPHA-2 ADRENERGIC RECEPTORS. It also blocks both 5-HT2 and 5-HT3 serotonin receptors and is a potent HISTAMINE H1 RECEPTOR antagonist. It is used for the treatment of depression, and may also be useful for the treatment of anxiety disorders.'),
  'D000078787': NtO(MH='Neuroglobin', MS='A vertebrate globin that is expressed predominantly in the brain where it is involved in oxygen transport. It protects neurons from APOPTOSIS during OXIDATIVE STRESS.'),
  'D000078789': NtO(MH='Polyacetylene Polymer', MS='A polyene polymer that consists of alternating single and double bonds.'),
  'D000078790': NtO(MH='Insulin Secretion', MS='Production and release of insulin from PANCREATIC BETA CELLS that primarily occurs in response to elevated BLOOD GLUCOSE levels.'),
  'D000078791': NtO(MH='Tandem Affinity Purification', MS='A technique used to isolate specific interacting partners of MACROMOLECULAR COMPLEXES that have been assembled with a tagged component (bait) that interacts with other partners in the complex. The interacting bait component is labeled with a dual-affinity tag that allows for capture of the complex by IMMUNOPRECIPITATION, followed by purification of only the partners of the complex interacting with the bait by affinity chromatography.'),
  'D000078792': NtO(MH='Midkine', MS='A developmentally regulated, HEPARIN-binding secreted growth factor homologous to pleiotrophin (PTN). It regulates cell proliferation through its interaction with ANAPLASTIC LYMPHOMA KINASE (ALK) and is involved in the formation of NEOINTIMA following arterial injury.'),
  'D000078822': NtO(MH='Ex-Smokers', MS='Persons who do not smoke but that have a history of SMOKING.'),
  'D000078842': NtO(MH='Cytoglobin', MS='A globin protein that may protect cells from OXIDATIVE STRESS and function in intracellular oxygen storage and transfer.'),
  'D000078862': NtO(MH='Levomilnacipran', MS='The (1S,2R)-isomer of milnacipran that is used for the treatment of MAJOR DEPRESSIVE DISORDER.'),
  'D000078882': NtO(MH='Web Archives as Topic', MS='Works about collections of preserved web pages.'),
  'D000078903': NtO(MH='Catalog, Drug', MS='Lists of drugs for sale.'),
  'D000078922': NtO(MH='Catalog, Union', MS='Lists of books and other information sources held among a cooperating group of LIBRARIES.'),
  'D000078925': NtO(MH='Catalog, Commercial', MS='Lists of merchandise for sale.'),
  'D000078928': NtO(MH='Formulary, Homeopathic', MS='Work about Formulary concerned with HOMEOPATHIC REMEDIES.'),
  'D000078929': NtO(MH='Formulary, Hospital', MS='Formulary concerned with PHARMACEUTICAL PREPARATIONS prescribed in hospitals.'),
  'D000078930': NtO(MH='Pharmacopoeia, Homeopathic', MS='Authoritative resource describing the composition, properties, manufacture, and quality control of HOMEOPATHIC REMEDIES.'),
  'D000078942': NtO(MH='Web Archive', MS='Collection of preserved web pages.'),
  'D000078984': NtO(MH='Formulary, Dental', MS='Works on or about reference lists of descriptions and uses of drugs related to oral medicine and dentistry.'),
  'D000078985': NtO(MH='Clinical Trial Protocols as Topic', MS="Works about the written descriptions of a clinical study. It contains the study's objectives, design, and methods including subject target and/or enrollment criteria. It may also present relevant scientific background and statistical information."),
  'D000079': NtO(MH='Acetaldehyde', MS='A colorless, flammable liquid used in the manufacture of acetic acid, perfumes, and flavors. It is also an intermediate in the metabolism of alcohol. It has a general narcotic action and also causes irritation of mucous membranes. Large doses may cause death from respiratory paralysis.'),
  'D000079042': NtO(MH='Maze Procedure', MS='A surgical treatment in which heart tissue is scarred (cut-and-sew method), excised or otherwise destroyed in an open chest approach to direct electrical signals through a more controlled path via the ATRIOVENTRICULAR NODE. It is often used to restore normal rhythms in ATRIAL FIBRILLATION.'),
  'D000079062': NtO(MH='Interpersonal Psychotherapy', MS='Interpersonal modification of psychodynamic psychotherapy. Therapy is present-focused and seeks to resolve problems in interpersonal relations. Aspects of attachment theory are used to analyze how past relationships affect current relationships.'),
  'D000079102': NtO(MH='Empowerment', MS='Process of increasing the capacity of individuals or groups to make choices and to transform those choices into desired actions as deigned by the individuals or groups.'),
  'D000079103': NtO(MH='Scope of Practice', MS='The services that licensed professionals are deemed competent to perform, and permitted to undertake in keeping with the terms of their professional license.'),
  'D000079142': NtO(MH='Shared Medical Appointments', MS='Clinical encounters in which patients receive healthcare from one or more health professionals, in a group setting. These may include patient education and counseling, physical examination, and clinical support. Patients usually share a mutual attribute such as a medical condition.'),
  'D000079182': NtO(MH='Body Dissatisfaction', MS="Person\xe2\x80\x99s negative thoughts about his or her own body.  This includes judgements about size and shape, muscle tone and generally involves a discrepancy between one's own body type and an ideal body type. (Grogan, 2008)"),
  'D000079203': NtO(MH='Disruptive Technology', MS='A process by which a product or service takes root initially in simple applications at the bottom of a market and then relentlessly moves up market, eventually displacing established competitors. (http://www.claytonchristensen.com/key-concepts)'),
  'D000079204': NtO(MH='Public Reporting of Healthcare Data', MS='Required public availability of quality indicators data from health facilities.'),
  'D000079223': NtO(MH='Observational Studies, Veterinary as Topic', MS='Works about clinical studies in which animal subjects may receive diagnostic, therapeutic, or other types of interventions and where the investigator does not assign subjects  to specific interventions.'),
  'D000079224': NtO(MH='Demoralization', MS='Weakening of hope, courage, or confidence.'),
  'D000079225': NtO(MH='Psychological Distress', MS='Negative emotional state characterized by physical and/or emotional discomfort, pain, or anguish.'),
  'D000079243': NtO(MH='Involuntary Commitment', MS='Non-voluntary institutionalization or isolation following refusal or failure to take medication, to complete treatment, or to adhere to infection control precautions, or behavior that may or does endanger self or others.'),
  'D000079245': NtO(MH='Dry Needling', MS='Intervention that uses a thin filiform needle to penetrate the skin and stimulate underlying myofascial trigger points, muscular, and connective tissues for the management of neuromusculoskeletal pain and movement impairments.'),
  'D000079262': NtO(MH='Obesity, Maternal', MS='BODY MASS INDEX in PREGNANT WOMEN that is excessively above the recommended cut-off.'),
  'D000079263': NtO(MH='Vaccine-Preventable Diseases', MS='Diseases for which vaccines exist that can confer partial or complete protection. (World Health Organization vaccine-safety-training.org)'),
  'D000079264': NtO(MH='Environmental Indicators', MS='Data related to climate change, ozone layer depletion, eutrophication, acidification, toxic contamination, urban environmental quality, biodiversity, cultural landscapes, waste, water resources, forest resources, fish resources, and soil degradation including desertification and erosion. (from https://www.oecd.org/env/indicators-modelling-outlooks/37551205.pdfc).'),
  'D000079265': NtO(MH='Environmental Science', MS='Scientific discipline that studies external factors, both biotic (living) and abiotic (nonliving), to which an organism is exposed.'),
  'D000079282': NtO(MH='Vaccine Excipients', MS='Media added to a vaccine for a specific purpose. These include: preservatives to prevent contamination, adjuvants to stimulate a stronger immune response, and stabilizers. (From https://www.cdc.gov/vaccines pubs/pinkbook/downloads/appendices/b/excipient-table-2.pdf)'),
  'D000079302': NtO(MH='Necroptosis', MS='A form of regulated cell death initiated by perturbations of extracellular or intracellular homeostasis that trigger a specific cell death pathway involving the activation of RIPK3 and MLKL (mixed lineage kinase domain like pseudokinase). Morphological changes include cytoplasmic and organellar swelling, rupture of the PLASMA MEMBRANE, and CHROMATIN CONDENSATION.'),
  'D000079322': NtO(MH='Weight Prejudice', MS='Negative attitudes towards and beliefs about others because of their weight. These negative attitudes are manifested by stereotypes and/or prejudice towards people with weight over or under a cultural norm.'),
  'D000079323': NtO(MH='Parthanatos', MS='A type of regulated cell death initiated by hyperactivation of POLY (ADP-RIBOSE) POLYMERASE-1 (PARP-1) as a consequence of DNA DAMAGE; OXIDATIVE STRESS; HYPOXIA; HYPOGLYCEMIA; or INFLAMMATION. This leads to DNA FRAGMENTATION and CHROMATIN condensation.'),
  'D000079382': NtO(MH='Internet-Based Intervention', MS='Use of the INTERNET to facilitate the dissemination of health-related information and to connect patients to support.'),
  'D000079402': NtO(MH='National Program of Cancer Registries', MS='A unit of the United States Centers for Disease Control and Prevention that supports the collection of high-quality data on cancer occurrence (including the type, extent, and location of the cancer), the type of initial treatment, and outcomes.'),
  'D000079403': NtO(MH='Ferroptosis', MS='A form of REGULATED CELL DEATH initiated by oxidative perturbations of the intracellular microenvironment that is under constitutive control by glutathione peroxidase 4 and can be inhibited by iron chelators and lipophilic antioxidants.'),
  'D000079404': NtO(MH='Regulated Cell Death', MS='Genetically encoded mechanisms in multicellular and unicellular eukaryotes that function to eliminate cells that are superfluous either during development, or to maintain tissue and organism homeostasis; and cells that are irreversibly damaged, abnormal, and/or potentially harmful.'),
  'D000079424': NtO(MH='Tumor Necrosis Factor Inhibitors', MS='Compounds or agents that bind to and inhibit the synthesis or activity of TUMOR NECROSIS FACTOR-alpha. Such agents are used to treat inflammatory bowel diseases and other inflammatory diseases.'),
  'D000079425': NtO(MH='Genetic Introgression', MS='The transfer of genetic material from one species to another by inter-species hybridization and repeated backcrossing.'),
  'D000079426': NtO(MH='Vector Borne Diseases', MS='Infectious diseases transmitted by DISEASE VECTORS.'),
  'D000079482': NtO(MH='Claustrum', MS='A gray matter lamina located underneath the inner NEOCORTEX lateral to the PUTAMEN and medial to the INSULAR CORTEX. It is considered to be a part of the basal ganglia. Extensive claustral projections exist to most cortices and subcortical structures including the AMYGDALA, the CAUDATE NUCLEUS, the HIPPOCAMPUS and the THALAMUS.'),
  'D000079522': NtO(MH='E-Cigarette Vapor', MS='Complex mixture of vaporized chemicals emitted during SMOKING of E-CIGARETTES.'),
  'D000079524': NtO(MH='Narcotic-Related Disorders', MS='Disorders related to or resulting from abuse or misuse of NARCOTICS.'),
  'D000079527': NtO(MH='Immunogenic Cell Death', MS='A form of regulated cell death that results in release of specific DAMAGE ASSOCIATED MOLECULAR PATTERN MOLECULES which activate an ADAPTIVE IMMUNE RESPONSE. It can be initiated in any type of cell by a restricted set of stimuli, including viral infection, some chemotherapeutics or immunogenic radiation therapy.'),
  'D000079530': NtO(MH='Mitochondrial Transmembrane Permeability-Driven Necrosis', MS='A form of regulated cell death that is initiated by specific perturbations of the intracellular microenvironment such as severe OXIDATIVE STRESS and cytosolic CALCIUM overload, which result in osmotic breakdown of the MITOCHONDRIAL MEMBRANES.'),
  'D000079542': NtO(MH='Tithonia', MS='A genus of flowering plants in the ASTERACEAE family. It is found in MEXICO; CENTRAL AMERICA; and SOUTHWESTERN UNITED STATES.'),
  'D000079543': NtO(MH='Ligularia', MS='A genus of ornamental plants in the family ASTERACEAE, often found near water.'),
  'D000079562': NtO(MH='Rumination Syndrome', MS='A gastrointestinal disorder characterized by repetitive and effortless regurgitation of ingested food from the stomach to the oral cavity in the early POSTPRANDIAL PERIOD. It is associated with functional gastrointestinal pathology (e.g., increased intra-gastric pressure) and psychiatric disorders (e.g., anxiety, depression). It is not limited to infants and distinct from GERD and VOMITING.'),
  'D000079564': NtO(MH='Globus Sensation', MS='A feeling of a lump in the throat that occurs between meals in the absence of other gastrointestinal and motility disorders (e.g., DYSPHAGIA; GASTROESOPHAGEAL REFLUX).'),
  'D000079584': NtO(MH='Actinoplanes', MS='A genus of zoospore-forming bacteria in the Micromonosporaceae family that is a source of many clinically important natural products.'),
  'D000079585': NtO(MH='Cell Encapsulation', MS='The encapsulation of cells into semi-permeable microcapsules or MICROSPHERES that permits protection of encapsulated cells and allows passage of essential nutrients and release of a therapeutic product (e.g. INSULIN).'),
  'D000079602': NtO(MH='Sino-Nasal Outcome Test', MS='Patient-reported measure of outcome in sino-nasal disorders such as rhinosinusitis and nasal polyposis.'),
  'D000079603': NtO(MH='Rapid Sequence Induction and Intubation', MS='A method of maintaining airway patency and administering anesthesia during emergency situations when it cannot be determined if the patient is fasting or not; or in other instances where there is a risk of aspiration. The method includes a series of events that involve the administration of SEDATIVES to induce unconsciousness, followed by NEUROMUSCULAR BLOCK AGENTS to improve the intubating conditions, followed by endotracheal intubation.'),
  'D000079644': NtO(MH='Military Health', MS='The physical and mental conditions of MILITARY PERSONNEL, active duty and retired U.S. Military personnel and their dependents.'),
  'D000079645': NtO(MH='Perioperative Medicine', MS='A medical specialty concerned with the patient before and after surgical procedures.'),
  'D000079662': NtO(MH='Chronic Disease Indicators', MS='A set of surveillance indicators of disease developed by consensus among the CDC, the Council of State and Territorial Epidemiologists, and the National Association of Chronic Disease Directors. It enables public health professionals and policymakers to retrieve uniformly defined state and selected metropolitan-level data for CHRONIC DISEASES and RISK FACTORS that impact public health.(https://www.cdc.gov/socialdeterminants/data/index.htm)'),
  'D000079664': NtO(MH='Continuous Renal Replacement Therapy', MS='Procedures with continuous blood purification (e.g., dialysis, filtration or perfusion) via vascular access designed to mimic kidney function in kidney diseases (e.g., ACUTE RENAL FAILURE and poisoning).'),
  'D000079685': NtO(MH='Famine', MS='The conditions when at least 20 per cent of households in an area face extreme food shortages with a limited ability to cope; acute malnutrition rates exceed 30 per cent; and the death rate exceeds two persons per day per 10,000 persons. (From news.un.org/en/story/2011/07/382342-when-food-security-crisis-becomes-famine)'),
  'D000079686': NtO(MH='Wastewater-Based Epidemiological Monitoring', MS='Sampling of the liquids generated from industrial processes, sanitary fixtures and appliances, food handling, etc., to collect and analyze epidemiological information (such as pathogen levels, or concentrations of pesticides, pharmaceuticals, or other substances) from metabolic excretion products, household liquid wastes, and industrial wastes.'),
  'D000079687': NtO(MH='Biological Monitoring', MS='Monitoring of the level of toxins, chemical pollutants, microbial contaminants, or other harmful substances in the bodies of living organisms, by DIAGNOSTIC IMAGING or by analyzing BLOOD; URINE; BREAST MILK; SALIVA, etc.'),
  'D000079689': NtO(MH='Opioid-Induced Constipation', MS='CONSTIPATION associated with the use of OPIOIDS.'),
  'D000079690': NtO(MH='Postoperative Cognitive Complications', MS='COGNITIVE IMPAIRMENT or functional decline after a surgical procedure.'),
  'D000079691': NtO(MH='School Mental Health Services', MS='Services which focus on the mental well-being of pre-college and pre-university students that include COUNSELING and therapy.'),
  'D000079702': NtO(MH='Pomegranate', MS='A deciduous shrub or small tree in the family Lythraceae, order Myrtales, superorder Rosanae. It bears edible reddish-purple fruit berries with seeds and pulp produced from the ovary of a single flower.'),
  'D000079742': NtO(MH='Cupping Therapy', MS='A traditional practice, most notably in traditional medicine, which involves placing cups on the skin to create suction believed to help increase blood flow and promote healing associated with chronic neck and low back pain in the short term (From National Center for Complementary and Integrative Health, https://nccih.nih.gov/news/cupping (Accessed 11/5/2018)).'),
  'D000079763': NtO(MH='Intermittent Renal Replacement Therapy', MS='Procedures with non-continuous blood purification (e.g., dialysis, filtration and/or perfusion) via vascular access by increasing blood flow rate compared to CONTINUOUS RRT.'),
  'D000079782': NtO(MH='Hybrid Renal Replacement Therapy', MS='Procedures involving non-continuous blood purification (e.g., dialysis, filtration or perfusion) via vascular access with settings for dialysate and blood flow rates and treatment duration settings hybrid between CRRT TECHNIQUE and IRRT TECHNIQUE.'),
  'D000079803': NtO(MH='Data Management', MS='Processes that include acquiring, validating, storing, protecting, and processing data to ensure accessibility, reliability, and timeliness for users.'),
  'D000079822': NtO(MH='Extranodal Extension', MS='A penetration of LYMPH NODE metastatic cells through the nodal capsule into the perinodal fatty tissue used in a TUMOR STAGING system (the TNM staging system).'),
  'D000079883': NtO(MH='Securinega', MS='A plant genus of the family EUPHORBIACEAE, order MALPIGHIALES. Members contain securinine and virosecurinine compounds.'),
  'D000079922': NtO(MH='Copper Transport Proteins', MS='Membrane proteins that facilitate the transport of copper cations across biological membranes.'),
  'D000079923': NtO(MH='Copper Transporter 1', MS='A high-affinity, saturable copper transporter of the PLASMA MEMBRANE that functions in the absorption of dietary copper.'),
  'D000079962': NtO(MH='RNA, Circular', MS="RNA molecules in which the 3' and 5' ends are covalently joined to form a closed continuous loop. They are resistant to digestion by EXORIBONUCLEASES."),
  'D000079963': NtO(MH='Brentuximab Vedotin', MS='Immunoconjugate that consists of the chimeric monoclonal anti - CD30 ANTIGEN antibody cAC-10 conjugated to monomethyl auristatin E. It is used for the treatment of LYMPHOMA.'),
  'D000079982': NtO(MH='Gemtuzumab', MS='Immunotoxin that consists of an anti-CD33 (CD33 ANTIGEN) monoclonal antibody conjugated to CALICHEAMICIN. It is used for the treatment of ACUTE MYELOID LEUKEMIA.'),
  'D000080': NtO(MH='Acetals', MS="Diethers with the structure -C(OR'')(OR'''), where R'' and  R''' are not hydrogen. Mixed acetals have R'' and R''' groups which differ."),
  'D000080002': NtO(MH='Protein-Arginine Deiminase Type 4', MS='A histone modification enzyme, which converts both ARGININE and monomethyl-arginine to CITRULLINE. It is one of several protein-arginine deiminase isoenzymes. It is a gene regulator involved in APOPTOSIS and CELL DIFFERENTIATION and a potential therapeutic target for the treatment of a variety of diseases.'),
  'D000080027': NtO(MH='Cyclams', MS='Compounds made of fourteen-member tetraamine macrocycles which bind strongly to a wide range of metal ions.'),
  'D000080029': NtO(MH='Citizen Science', MS='Voluntary public participation in the scientific process, addressing real-world problems in ways that may include formulating research questions, conducting scientific experiments, collecting and analyzing data, interpreting results, making new discoveries, developing technologies and applications, for solving complex problems. (From https://www.citizenscience.gov/)'),
  'D000080030': NtO(MH='Legal Epidemiology', MS='Scientific study of law as a factor in the cause, distribution, and prevention of disease and injury.  (From https://read.dukeupress.edu/jhppl/article/41/6/1151/40084/Policy-Surveillance-A-Vital-Public-Health-Practice).'),
  'D000080031': NtO(MH='Limited English Proficiency', MS='Limited ability to communicate with or understand English.'),
  'D000080032': NtO(MH='Psychology, Positive', MS='The scientific study of the strengths that enable individuals and communities to thrive. It is founded on the belief that people want to lead meaningful and fulfilling lives, to cultivate what is best within themselves, and to enhance their experiences of love, work, and play. (https://ppc.sas.upenn.edu/)'),
  'D000080033': NtO(MH='Public Nondiscrimination Policies', MS='Policies that include the existence of a body responsible for the articulation and implementation of a platform based on an equality perspective that requires the integration of race-ethnicity, differences in sexual orientation, as well as the demands and needs arising from differences within the executive. Its objective is to translate a programmatic vision into action and action axes that reflects the struggle for egalitarian social relations. (From http://bit.ly/28WAaYO)'),
  'D000080037': NtO(MH='Historical Trauma', MS='Clinical distress among descendants of survivors of mass trauma, as if they had suffered through the same extreme historic circumstances. Symptoms may persist through generations.'),
  'D000080038': NtO(MH='Anomalous Left Coronary Artery', MS='Malformations of left CORONARY ARTERY where it is connected to the PULMONARY ARTERY instead of the AORTA.'),
  'D000080039': NtO(MH='Univentricular Heart', MS='Rare congenital heart malformation with a single ventricle (HEART VENTRICLES) instead of the usual two.'),
  'D000080041': NtO(MH='Congenitally Corrected Transposition of the Great Arteries', MS='A rare heart defect that occurs when the HEART VENTRICLES and attached valves, the MITRAL VALVE and the TRICUSPID VALVE are reversed so that the AORTA and the PULMONARY ARTERY are connected to the wrong ventricle. Although the heart valves and the two great arteries, the PULMONARY ARTERY and the AORTA are transposed, the blood flows to the correct place because the ventricles are also reversed and therefore corrects the transposition. It often occurs with other structural heart abnormalities.'),
  'D000080044': NtO(MH='Ado-Trastuzumab Emtansine', MS='Immunotoxin that consists of humanized monoclonal anti-HER2 antibody TRASTUZUMAB covalently linked to anti-microtubule agent MAYTANSINOID DM1 for treatment of metastatic breast cancer in patients who previously received trastuzumab and a TAXANES, separately or in combination.'),
  'D000080045': NtO(MH='Inotuzumab Ozogamicin', MS='Immunotoxin that consists of anti-CD22 (CD22 ANTIGEN) humanized monoclonal antibody covalently linked to the antineoplastic antibiotic CALICHEAMICIN. It is used for the treatment of adults with relapsed or refractory B-cell precursor acute lymphoblastic leukemia (LEUKEMIA, ACUTE LYMPHOBLASTIC).'),
  'D000080048': NtO(MH='Hot Melt Extrusion Technology', MS='Solvent-free method for preparing numerous types of drug delivery systems and dosage forms for active pharmaceutical ingredients that have solubility issues. It involves pumping polymeric materials at temperatures above their glass transition temperature to mix in active pharmaceutical ingredients with or without binders thus forming an amorphous product with an  increase in the dissolution profile for the active pharmaceutical ingredients.'),
  'D000080050': NtO(MH='Para-Athletes', MS='Physically disabled individuals who have developed skills, physical stamina and strength to participate in SPORTS or other physical activities.'),
  'D000080051': NtO(MH='Community-Based Health Insurance', MS='Low-cost HEALTH INSURANCE in which community, cooperative, mutual, and/or self-help resources are pooled to mitigate health risks and cover health care services in full or in part.'),
  'D000080052': NtO(MH='Opioid Epidemic', MS='Widespread use of OPIOIDS by disproportionately large numbers of individuals within a population, community, or region occurring within a relatively narrow window of time.'),
  'D000080054': NtO(MH='Pinales', MS='Order of Conifers of the class Pinopsida, subdivision Spermatophytina, and division Tracheophyta. The distinguishing attribute of the Pinales is the reproductive structure known as a cone.'),
  'D000080056': NtO(MH='Araucariaceae', MS='A family of dioecious or monoecious evergreen trees belonging to the order Pinales, subclass Pinidaes, with spirally arranged, narrow or broad leaves often with parallel veins.'),
  'D000080057': NtO(MH='Araucaria', MS='A genus of evergreen conifer of the family Araucariaceae, order Pinales, that includes the monkey puzzle (Araucaria Araucana).'),
  'D000080059': NtO(MH='Araucaria araucana', MS='An evergreen tree commonly known as the monkey-puzzle tree. It grows to 1-1.5 m in diameter and 30-40 m in height, and is native to central and southern Chile, and western Argentina. Araucaria araucana is the hardiest among the conifers, and due to its longevity, is often described as a living fossil.'),
  'D000080062': NtO(MH='Protein-Arginine Deiminase Type 1', MS='A peptidylarginine deiminase isoenzyme. It is a post-translational modification enzyme that catalyzes the conversion of protein-bound ARGININE residues to CITRULLINE residues in a CALCIUM ion-dependent manner. It functions within the epidermis, targeting INTERMEDIATE FILAMENTS; KERATINS; and filaggrin for CITRULLINATION to promote epidermal cornification.'),
  'D000080063': NtO(MH='Protein-Arginine Deiminase Type 3', MS='A post-translational enzyme that catalyzes the conversion of protein bound arginine residues to CITRULLINE residues in a CALCIUM ion-dependent manner. PADI3 is a peptidylarginine deiminase isoenzyme expressed in epidermal tissue.'),
  'D000080064': NtO(MH='Protein-Arginine Deiminase Type 2', MS='A widely expressed isoenzyme of peptidylarginine deiminase, a post-translational modification enzyme that catalyzes the conversion of protein-bound ARGININE residues to CITRULLINE residues in a CALCIUM ion-dependent manner.'),
  'D000080065': NtO(MH='Protein-Arginine Deiminase Type 6', MS='A peptidylarginine deiminase isoenzyme mainly expressed in egg and embryo tissues. It is a post-translational modification enzyme that catalyzes the conversion of protein-bound ARGININE residues to CITRULLINE residues in a CALCIUM ion-dependent manner.'),
  'D000080066': NtO(MH='Contraceptive Agents, Hormonal', MS='Contraceptive agents that act on the ENDOCRINE SYSTEM.'),
  'D000080083': NtO(MH='SLC31 Proteins', MS='Eukaryotic copper transport proteins that function primarily in the transport of dietary or environmental COPPER.'),
  'D000080084': NtO(MH='Calicheamicins', MS='Enediyne aminoglycosides derived from the bacterium MICROMONOSPORA ECHINOSPORA that are used as antineoplastic antibiotics. They bind to DNA in both healthy and tumor cells, resulting in strand scission and cell death.'),
  'D000080103': NtO(MH='Emotional Regulation', MS='The manifestation of an emotional response by which individuals influence the intensity, duration, valence, or manifestation of their response. Emotion regulation either precedes (avoidance, interpretation) or follows an emotion (suppression, reappraisal). Emotion regulation is distinct from coping (focuses primarily on decreasing a negative emotional experience), and can include increasing or decreasing both positive and negative emotions consciously or unconsciously.'),
  'D000080104': NtO(MH='Fertility Clinics', MS='Health clinics offering fertility treatment services.'),
  'D000080105': NtO(MH='Relational Autonomy', MS='The social context within which all individuals exist and acknowledge the emotional and embodied aspects of decision-makers. The approach acknowledges the central role of others in decision-making, including clinicians, who have a responsibility to engage patients\xe2\x80\x99 and surrogates\xe2\x80\x99 emotional experiences.'),
  'D000080122': NtO(MH='Edible Films', MS='Packaging materials that can be directly consumed. They provide barrier and protection, while enhancing quality and safety of food products. (Bourtoom, 2008).'),
  'D000080146': NtO(MH='Avoidant Restrictive Food Intake Disorder', MS='An eating or feeding disturbance (e.g., apparent lack of interest in eating or food; avoidance based on the sensory characteristics of food; concern about aversive consequences of eating) as manifested by persistent failure to meet appropriate nutritional and/or energy needs associated with one (or more) of the following: significant weight loss (or failure to achieve expected weight gain or faltering growth in children); significant nutritional deficiency; dependence on enteral feeding or oral nutritional supplements; or marked interference with psychosocial functioning. (from DSM-V)'),
  'D000080162': NtO(MH='Early Warning Score', MS='Tool for bedside evaluation based on five physiological parameters: SYSTOLIC PRESSURE, PULSE, RESPIRATORY RATE, BODY TEMPERATURE and AVPU (alert, voice, pain, unresponsive) score.'),
  'D000080165': NtO(MH='Extreme Weather', MS='Events that have extreme values of important meteorological variables. Extreme is generally defined as either taking maximum values or exceedance above pre-existing high thresholds. Such events are generally rare; for example, extreme wind speeds exceeding the 100-year return value, which have a probability of only 0.01 of occurring in any particular year. (Stephenson, 2008)'),
  'D000080183': NtO(MH='Intersex Persons', MS='Those individuals with INTERSEX CONDITIONS, with or without a desire to undergo SEX REASSIGNMENT PROCEDURES.'),
  'D000080203': NtO(MH='Hamman-Rich Syndrome', MS='Acute idiopathic interstitial pneumonitis characterized by diffuse PULMONARY ALVEOLI damage with uniform edematous connective tissue proliferation. It is often associated with extensive fibroblastic distortion of the lung parenchyma and leads to ADULT RESPIRATORY DISTRESS SYNDROME in later stages.'),
  'D000080222': NtO(MH='Pancreaticobiliary Maljunction', MS='A group of anatomical variations in which the COMMON BILE DUCT and MAIN PANCREATIC DUCT are joined outside the duodenal wall often with abnormally long common channel rather than at the SPHINCTER OF ODDI.'),
  'D000080223': NtO(MH='Chronic Urticaria', MS='Wheals and/or angioedema presented with daily symptoms lasting for more than 6 weeks. It may be classified into chronic spontaneous and chronic inducible urticaria depending on whether a specific trigger can be linked to the development of vascular reaction.'),
  'D000080224': NtO(MH='Animal Proteins, Dietary', MS='DIETARY PROTEINS derived from ANIMALS.'),
  'D000080242': NtO(MH="8-Hydroxy-2'-Deoxyguanosine", MS='Common oxidized form of deoxyguanosine in which C-8 position of guanine base has a carbonyl group.'),
  'D000080282': NtO(MH='Hormonal Contraception', MS='Contraceptive methods that utilize HORMONAL CONTRACEPTIVE AGENTS.'),
  'D000080304': NtO(MH='Cricket Sport', MS='A competitive sport played with a bat and ball between two teams of eleven players.'),
  'D000080305': NtO(MH='Pork Meat', MS='MEAT or flesh of a pig (Sus scrofa) used as food.'),
  'D000080307': NtO(MH='Hydrogen Deuterium Exchange-Mass Spectrometry', MS='A mass spectrometry method utilizing hydrogen-deuterium exchange kinetics to study conformational and dynamic changes especially of proteins, such as interactions during ligand binding and allosteric effects during catalysis.'),
  'D000080308': NtO(MH='Cardio Ankle Vascular Index', MS='An index of ARTERIAL STIFFNESS obtained from PULSE WAVE VELOCITY measurements between the heart and the ankle.'),
  'D000080309': NtO(MH='Environmental DNA', MS='DNA found in the ENVIRONMENT that is from the residual biological materials of the organisms in the environment.'),
  'D000080310': NtO(MH='Smart Glasses', MS='Computerized eyewear with multiple technological applications for advanced computing and interoperability. Smart glasses are optimized to facilitate information and data access, capture, display, transfer, and manipulation.'),
  'D000080311': NtO(MH='Synesthesia', MS='The experience of involuntary sensory cross activation where the presentation of a particular stimulus elicits a secondary sensory-perceptual experience. It most commonly occurs in the association of color with linguistic stimuli such as letters, numbers, words, or music, but can also occur between other senses. Although synesthesia can be acquired or transient due to trauma or drug use, there is also a strong genetic component, with a prevalence of about 1 in 2,000 individuals and a female to male ratio of 6:1.'),
  'D000080322': NtO(MH='Cyclodialysis Clefts', MS='Finding of a separation of the CILIARY BODY in the SCLERAL SPUR region, creating aqueous outflow from the ANTERIOR CHAMBER into suprachoroidal space between the CHOROID and the SCLERA. Persistent cyclodialysis clefts may be associated with OCULAR HYPOTENSION and OPTIC DISC EDEMA.'),
  'D000080323': NtO(MH='Uveal Effusion Syndrome', MS='Rare disease characterized by suprachoroidal fluid accumulation between the CHOROID and the SCLERA, annular detachment of the CHOROID, often with secondary RETINAL DETACHMENT; OPTIC DISK EDEMA, and minimal signs of UVEITIS and INTRAOCULAR PRESSURE changes.'),
  'D000080324': NtO(MH='Choroidal Effusions', MS='Finding of fluid accumulation between the CHOROID and the SCLERA.'),
  'D000080343': NtO(MH='Meibomian Gland Dysfunction', MS='A chronic dysfunction of MEIBOMIAN GLANDS characterized by altered tear film stability and function due to a decrease or alteration in lipid quality/content in meibum. It is often associated with evaporative-type DRY EYE SYNDROME.'),
  'D000080344': NtO(MH='Optic Nerve Hypoplasia', MS='A group of rare genetic disorders characterized by underdeveloped OPTIC NERVES, resulting in increased incidences of vision impairment, CONGENITAL NYSTAGMUS and STRABISMUS. It may be syndromic, and is often associated with CNS malformations.'),
  'D000080345': NtO(MH='Familial Exudative Vitreoretinopathies', MS='A group of inherited disorders characterized by incomplete development of the retinal vasculature. Its severity can vary from complete blindness in infancy, to mild or no visual problems, where small areas of vascular defects are observable only by FLUORESCEIN ANGIOGRAPHY. Exudative vitreoretinopathy 1 is associated with mutations in the FZD4 gene.'),
  'D000080346': NtO(MH='Retinal Arterial Macroaneurysm', MS='An acquired dilation of the retinal artery often associated with systemic HYPERTENSION.'),
  'D000080362': NtO(MH='Stargardt Disease', MS='A juvenile-onset macular dystrophy characterized by progressive loss of VISUAL ACUITY with normal acuity in peripheral VISUAL FIELDS. Other associated clinical features may include LIPOFUSCIN fundus autofluorescence, atrophy of the RETINAL PIGMENT EPITHELIUM, loss of color vision, PHOTOPHOBIA and PARACENTRAL SCOTOMA. Germline mutations in the ABCA4 gene have been identified in recessive and dominant diseases.'),
  'D000080363': NtO(MH='White Dot Syndromes', MS='A group of idiopathic multifocal posterior uveitis syndromes involving the CHOROID; RETINAL PIGMENT EPITHELIUM; and RETINA. They are characterized by multiple lesions of hypoautofluorescent dots in the FUNDUS OCULI and reduced VISUAL ACUITY. Several entities including BIRDSHOT CHORIORETINOPATHY are HLA-A ANTIGENS serotype A29 positive.'),
  'D000080364': NtO(MH='Multifocal Choroiditis', MS='A multifocal uveitis syndrome involving the RETINAL PIGMENT EPITHELIUM and capillary layer of the CHOROID. It is characterized by chronic UVEITIS and multiple CHOROID lesions referred to as white dots, blurry vision, floaters, sensitivity to light, blind spots, and eye discomfort.'),
  'D000080365': NtO(MH='Birdshot Chorioretinopathy', MS='A form of chorioretinitis characterized by multiple small, cream-colored LESIONS, symmetrically scattered mainly around the OPTIC DISK. These lesions are the most distinctive sign and often appear at the level of the RETINAL PIGMENT EPITHELIUM but, on occasion, suggest an even deeper infiltration and may ultimately lead to visual loss. An association with HLA-A29 antigen (see HLA-A ANTIGENS) has been observed in nearly all patients.'),
  'D000080383': NtO(MH='Mental Foramen', MS='An opening on the anterolateral aspect of the MANDIBLE through which the mental nerve bundle and vessels exit.'),
  'D000080384': NtO(MH='Allergoids', MS='Extracts of allergens, or purified allergens that have been chemically modified to reduce their ability to induce IGE-MEDIATED HYPERSENSITIVITY (allergenicity) while increasing their ability to induce IMMUNE TOLERANCE.'),
  'D000080385': NtO(MH='Nanogels', MS='Three-dimensional biocompatible materials formed by cross-linking a hydrophilic polymeric component (e.g., HYDROGELS). They are typically spherical particles in a size range of 20-250 nm when used for drug delivery applications.'),
  'D000080386': NtO(MH='Microgels', MS='Three-dimensional biocompatible polymers in micrometer size range. They are typically spherical particles in a size range of 1-350 micrometers when used for drug delivery and 3D cell culture applications.'),
  'D000080422': NtO(MH='Locked-In Syndrome', MS='Acquired neuromuscular disorder characterized by complete paralysis of voluntary muscles and lower CRANIAL NERVES except for limited voluntary eye movements. It is due to various cerebrospinal disconnections at or near the PONS and the POSTERIOR CRANIAL FOSSA, typically secondary to pontine hemorrhage or infarct. Because cognitive function is intact it is sometimes referred to as a pseudocoma state.'),
  'D000080423': NtO(MH='Universal Health Care', MS='Access for all people to needed health services (including prevention, promotion, treatment, rehabilitation and palliation) of sufficient quality to be effective while also ensuring that the use of these services does not expose the user to financial hardship. (From https://www.who.int/healthsystems/universal_health_coverage/en/)'),
  'D000080424': NtO(MH='Cytokine Release Syndrome', MS='A systemic inflammatory response triggered by a variety of factors used in IMMUNOTHERAPY. It is characterized by marked elevations of various inflammatory CYTOKINES and by the following associated conditions: DYSPNEA; FEVER; HEADACHE; HYPOTENSION; NAUSEA; RASH; TACHYCARDIA and/or HYPOXIA.'),
  'D000080442': NtO(MH='Social Integration', MS='A dynamic and principled process where all members participate in dialogue to achieve and maintain peaceful social relations. It does not include coerced assimilation or forced integration. (from https://www.un.org/esa/socdev/sib/peacedialogue/soc_integration.htm)'),
  'D000080443': NtO(MH='Diffuse Intrinsic Pontine Glioma', MS='A rare, aggressive brain tumor that forms in the GLIAL CELLS in the PONS.'),
  'D000080444': NtO(MH='Phenomics', MS='The systematic study of how genetic information or GENOMICS translates into biochemical, metabolic and morphological traits of an organism.'),
  'D000080445': NtO(MH='Asthma-Chronic Obstructive Pulmonary Disease Overlap Syndrome', MS='Syndrome with clinical features of both ASTHMA and COPD.'),
  'D000080462': NtO(MH='Acyclic Monoterpenes', MS='Linear compounds that contain a single monoterpene unit.'),
  'D000080463': NtO(MH='Sand', MS='A hard granular material that is a component of soil. It is composed of very small pieces (between 0.06 and 2.0 millimeters in diameter) of disintegrating rock.'),
  'D000080482': NtO(MH='Enhanced Recovery After Surgery', MS='A protocol of components related to preadmission, preoperative, intraoperative, and postoperative care. The protocol is implemented with the goal of improving patient recovery, facilitating earlier discharge from the hospital, and potentially reducing health care costs without increasing complications or hospital readmissions. The protocol components may contribute to minimizing, and/or improving the response to, physiological stress associated with surgery.'),
  'D000080483': NtO(MH='Atomic Bomb Survivors', MS='Persons who survived exposure to RADIATION due to ATOMIC BOMBS.'),
  'D000080485': NtO(MH='Sudden Unexpected Death in Epilepsy', MS='Sudden death in a patient with EPILEPSY associated with SEIZURES and seizure-related symptoms (e.g., APNEA; HYPOXEMIA) without other identifiable accidental causes (e.g., DROWNING; WOUNDS AND INJURIES).'),
  'D000080486': NtO(MH='Computational Chemistry', MS='A branch of chemistry that utilizes computers as well as principles of chemistry and mathematics to generate data and solve problems related to the properties, structures, and reactions of molecules and materials.'),
  'D000080487': NtO(MH='Internet of Things', MS='Networking capability which facilitates information flow to and from objects and devices using the INTERNET.'),
  'D000080503': NtO(MH='Fictional Works as Topic', MS='Works about creative writing, not presented as factual.'),
  'D000080505': NtO(MH='Trichothecenes, Type A', MS='A subclass of Trichothecene derivatives that include the 12,13-epoxytrichothec-9-ene structure of the molecule with the C-8 position containing either no oxygen substitution, a hydroxyl group at C-8, or an ester function at C-8.'),
  'D000080506': NtO(MH='Sialyl Lewis X Antigen', MS='A sialylated version of Lewis X antigen expressed on cell surfaces. It is a ligand for SELECTINS.'),
  'D000080522': NtO(MH='Sapajus apella', MS='A species of the genus SAPAJUS, subfamily CEBINAE, family CEBIDAE, consisting of tufted capuchin monkeys with characteristic black head cap and sideburn coloration with tufts of dark fur above the ears. S. apella was known as Cebus apella and C. a. apella.'),
  'D000080523': NtO(MH='Sapajus', MS='A genus of the family CEBIDAE, subfamily CEBINAE, consisting of capuchin monkeys with characteristic tufts of hair over the eyes and sides of the head. Many were formerly classified under genus CEBUS. Sapajus inhabits the forests of Central and South America.'),
  'D000080526': NtO(MH='Cebus capucinus', MS='A species of the genus CEBUS, family CEBIDAE, consisting of capuchin monkeys with white markings over most of frontal areas of the face, chest and upper arms.'),
  'D000080527': NtO(MH='Macaca fuscata', MS='A species of the genus MACACA characterized by red naked face. M. fuscata is primarily ground-dwelling in forest and mountains often near hot springs in Japan.'),
  'D000080530': NtO(MH='Twinning, Embryonic', MS='The development of two EMBRYOS in a single UTERUS at the same time.'),
  'D000080531': NtO(MH='Macaca arctoides', MS='A species of the genus MACACA inhabiting India, China, and other parts of southern Asia. The species is characterized by its thick brown fur covering most of its body except the face and short tail.'),
  'D000080532': NtO(MH='Alouatta caraya', MS='A species of the genus ALOUATTA, family ATELIDAE, inhabiting the forests of South America. Although sometimes known as black howler only adult males have black hair and adult females and juveniles are gold in color. Closely related A. pigra in Central America and A. nigerrima in the Amazon are also called black howlers.'),
  'D000080534': NtO(MH='Twinning, Dizygotic', MS='The development of two genetically distinct EMBRYOS in a single UTERUS at the same time, from two separate OVA fertilized by two separate SPERMATOZOA.'),
  'D000080536': NtO(MH='Decision Making, Shared', MS='Collaborative process of reaching a decision.'),
  'D000080542': NtO(MH='Presbytini', MS='The tribe of subfamily COLOBINAE, family CERCOPITHECIDAE. They are found in India, Indonesia, Malaysia, and Thailand.'),
  'D000080544': NtO(MH='Indriidae', MS='A family of primates of the superfamily LEMUROIDEA, suborder STREPSIRRHINI containing genera AVAHI (woolly lemurs); INDRI; and PROPITHECUS (sifakas). The family is distributed in Madagascar.'),
  'D000080545': NtO(MH='Microplastics', MS='Plastic particles, less than 5 millimeters in diameter, that are generated from the degradation of larger pieces or are manufactured to be microstructures or smaller.'),
  'D000080546': NtO(MH='Space Research', MS='Studies of interactions between living organisms and characteristics of the extraterrestrial environment. These studies specifically address the structure and function of living organisms in an extraterrestrial environment and interdependent relationships of organisms with each other and/or the extraterrestrial environment while also touching on the origin, evolution and potential for extraterrestrial life. (From www.nasa.gov/audience/foreducators/spacelife/)'),
  'D000080549': NtO(MH='Autophagic Cell Death', MS='A major activation of the autophagic pathway, characterized by massive accumulation of AUTOPHAGOSOMES and CELL DEATH.'),
  'D000080550': NtO(MH='Macroautophagy', MS='The segregation of large parts of the cytoplasm, including ORGANELLES and PROTEIN AGGREGATES, into AUTOPHAGOSOMES and lysosomal degradation after fusion with LYSOSOMES.'),
  'D000080551': NtO(MH='Microautophagy', MS='A type of autophagy in which the cytoplasmic entities, such as parts of CELL NUCLEI; damaged MITOCHONDRIA; and lipid droplets, are taken up by small vesicles such as VACUOLES or MULTIVESICULAR BODIES, and degraded by lysosomal digestion.'),
  'D000080562': NtO(MH='Universal Design', MS='The design of products and environments to be usable by all people, to the greatest extent possible, without the need for adaption or specialized design.'),
  'D000080564': NtO(MH='Right to Health', MS='Right to accessible health services physically within reach for all sections of the population, including children, adolescents, older persons, persons with disabilities and other vulnerable groups as well as financially and on the basis of non-discrimination. Accessibility also implies the right to seek, receive and impart health-related information in an accessible format for all, including persons with disabilities, but does not impair the right to have personal health data treated confidentially.'),
  'D000080583': NtO(MH='Ateles geoffroyi', MS="A species in the genus ATELES, subfamily ATELINAE, comprising several Mesoamerican spider monkeys including Geoffroy's spider monkey (Ateles geoffroyi geoffroyi). Sometimes referred to as Central American spider monkey, it inhibits rainforest from Mexico to Colombia."),
  'D000080602': NtO(MH='Callicebus', MS='A genus of the subfamily CALLICEBINAE, family PITHECIIDAE, generally comprising of five species: C. barbarabrownae; C. coimbrai; C. melanochir; C. nigrifrons; C. personatus. Titi monkeys inhibit lower levels of the forest in South America.'),
  'D000080623': NtO(MH='Galagidae', MS='A family of Primates of the suborder Strepsirhini containing six genera. The family is widely distributed in Africa near Subsaharan region. The six genera are: Euoticus (needle-clawed galagos), GALAGO (bush babies), Galagoides (dwarf galagos), Otolemur (greater galagos), Paragalago and Sciurocheirus (squirrel galagos). All are primarily arboreal and nocturnal.'),
  'D000080642': NtO(MH='Chaperone-Mediated Autophagy', MS='A selective type of autophagy in which a specific set of cytosolic proteins are moved across the membrane of the LYSOSOMES by protein translocation, rather than by invagination of the lysosomal membrane. The proteins that will be degraded by chaperone-mediated autophagy (CMA) contain a CMA-targeting motif that, when exposed, is recognized by the cytosolic chaperone HSPA8 (aka Hsc70) and cochaperones which deliver the protein to LAMP2A monomers on the lysosome cell surface for translocation via the LAMP2A translocation complex. CMA is distinguished from CHAPERONE-ASSISTED-SELECTIVE AUTOPHAGY - a type of selective MACROAUTOPHAGY.'),
  'D000080662': NtO(MH='Phospholipid Hydroperoxide Glutathione Peroxidase', MS='A selenoenzyme that converts GLUTATHIONE plus FATTY ACID HYDROPEROXIDES to GLUTATHIONE DISULFIDE plus hydroxy fatty acids and water.'),
  'D000080682': NtO(MH='Salads', MS='A prepared mixture of cooked and/or raw vegetables, FRUIT, or other foods, garnished with a dressing CONDIMENT, and eaten as a MEAL or side dish.'),
  'D000080702': NtO(MH='Edible Insects', MS='Organisms of the INSECTA class that are used as food or are suitable as a nutritional source, especially by humans.'),
  'D000080743': NtO(MH='Shape Memory Alloys', MS='ALLOYS designed to recover their original shape upon heating.'),
  'D000080745': NtO(MH='Smart Materials', MS='Materials designed to change properties in response to changes in their environment adopting various predetermined shapes and/or functionalities.'),
  'D000080762': NtO(MH='Stimuli Responsive Polymers', MS='Polymeric materials designed to change properties in response to a stimulus.'),
  'D000080763': NtO(MH='Sea Level Rise', MS='An increase in the volume of the oceans and seas causing an elevation of the air-sea interface.'),
  'D000080802': NtO(MH='Horticulture', MS='Branch of agriculture which involves the cultivation of vegetables, flowers, or ornamental plants for food, industrial, medicinal, or aesthetic purposes. It does not include CROP PRODUCTION on a large scale.'),
  'D000080803': NtO(MH='Biosolids', MS='Wastewater SLUDGE treated by processes such as COMPOSTING, anaerobic digestion, aerobic digestion, alkaline treatment, and PASTEURIZATION to remove HAZARDOUS CHEMICALS and pathogens.'),
  'D000080822': NtO(MH='Relative Energy Deficiency in Sport', MS='Health disorders associated with low caloric intake of individuals involved in high-activity SPORTS.'),
  'D000080842': NtO(MH='Undiagnosed Diseases', MS='Rare and common diseases lacking a diagnosis.'),
  'D000080843': NtO(MH='Brain Regeneration', MS=None),
  'D000080845': NtO(MH='Sugar-Sweetened Beverages', MS='Liquids or drinks sweetened with added sugars of various forms.'),
  'D000080864': NtO(MH='Lactate Dehydrogenase 5', MS='A member of the LACTATE DEHYDROGENASES isozyme family, Lactate Dehydrogenase 5 is localized to liver and skeletal muscle cells where its expression increases in liver disease and striated muscle trauma respectively.'),
  'D000080865': NtO(MH='Formins', MS='A family of multidomain microfilament proteins associated with ACTIN FILAMENTS. They generally have multiple Formin-Homology domains (e.g., FH2 domain) and additionally have a RhoGTPase-binding domain and a diaphanous inhibitory domain. Formins and homologs are involved in actin reorganization.'),
  'D000080866': NtO(MH='Nucleobindins', MS='A DNA and calcium binding protein which plays a role in calcium homeostasis.'),
  'D000080867': NtO(MH='Gingipain Cysteine Endopeptidases', MS='Cysteine endoproteinases, from periodontal pathogen PORPHYROMONAS GINGIVALIS, acting as virulence factors associated with PERIODONTITIS. They are produced as pre-proproteins which mature into ARGININE and LYSINE specific endopeptidases.'),
  'D000080868': NtO(MH='Water Decolorization', MS='The removal of DYES; STAINS; or other COLORING AGENTS, natural or synthetic, from water sources by various processes such as ADSORPTION; BIOREMEDIATION; enzymatic digestion; and ELECTROLYSIS.'),
  'D000080869': NtO(MH='Jugular Foramina', MS='A pair of openings from the posterior cranial fossa through which the GLOSSOPHARYNGEAL NERVE; VAGUS NERVE; ACCESSORY NERVE and the internal JUGULAR VEINS pass.'),
  'D000080870': NtO(MH='R-Loop Structures', MS='An RNA-DNA hybrid structure formed when newly transcribed RNA remains bound to its DNA template. Stability of R-loops may play a role in GENETIC INSTABILITY.'),
  'D000080871': NtO(MH='Movable Books', MS='Books having mechanisms or parts that move, or are moved by the reader.'),
  'D000080872': NtO(MH='Open Abdomen Techniques', MS='Abdominal wound closure techniques in which the opening from a laparoscopic abdominal incision is left open, but covered, to reduce or prevent INTRA-ABDOMINAL HYPERTENSION.'),
  'D000080873': NtO(MH='Multifunctional Nanoparticles', MS='Nanoparticles that are engineered to have two or more physical properties.'),
  'D000080874': NtO(MH='Synucleinopathies', MS='Neurodegenerative disorders involving deposition of abnormal ALPHA-SYNUCLEIN in dopaminergic neurons and glial cells in the brain. Pathological aggregations of alpha-synuclein proteins results in LEWY BODIES and Lewy neurites; melanin granules in the SUBSTANTIA NIGRA and LOCUS COERULEUS; and glial cytoplasmic inclusions. Synucleinopathies are associated with mutation in the ALPHA-SYNUCLEIN (SNCA) gene on chromosome 4. PARKINSON DISEASE; LEWY BODY DISEASE with dementia; and MULTIPLE SYSTEM ATROPHY are prominent examples of synucleinopathy.'),
  'D000080882': NtO(MH='Ribosomal Protein L10', MS='A RIBOSOMAL PROTEIN which plays a crucial role in protein synthesis and embryonic brain development. Diseases associated with RPL10 mutations include Mental Retardation, X-Linked, Syndromic, 35 and Autism X-Linked 5.'),
  'D000080883': NtO(MH='Shoulder Dystocia', MS='Obstetric complication during OBSTETRIC DELIVERY in which exit of the fetus is delayed due to physical obstruction involving fetal shoulder(s).'),
  'D000080884': NtO(MH='Infratemporal Fossa', MS='The compartment that lies posterior to the MAXILLA and inferior to the side wall of the SKULL deep to the ramus of the MANDIBLE. It contains several FACIAL NERVES and the CAROTID ARTERIES and the JUGULAR VEINS.'),
  'D000080886': NtO(MH='Parapharyngeal Space', MS='The deep neck space near the PHARYNX located anterior to the ptyergomandibular raphe,  posterior to the prevertebral fascia, inferior to the SKULL BASE, and superior to the HYOID BONE. Prestyloid parapharyngeal space contains cranial nerves IX, X, XI, XII and internal JUGULAR VEIN and internal CAROTID ARTERY.  Poststyloid parapharyngeal space contains AURICULARTEMPORAL NERVE and MAXILLARY ARTERY and ascending pharyngeal artery.'),
  'D000080887': NtO(MH='Diabulimia', MS='Eating disorder occurring typically in patients with Diabetes Mellitus, Type 1 who restrict insulin intake in order to lost weight.'),
  'D000080888': NtO(MH='Specific Language Disorder', MS='Language disorder that delays the mastery of oral language skills or  in children who have no hearing loss or other developmental delays.'),
  'D000080890': NtO(MH='Duocarmycins', MS='A group of pyrroloindole compounds often with additional spirocyclic unit(s) and their analogs originally isolated from STREPTOMYCES. They bind DNA minor grooves with adenine-N3 alkylation activity.'),
  'D000080891': NtO(MH='Learning Health System', MS='A system in which internal data and experience are systematically integrated with external evidence and that knowledge is put into practice. As a result, patients get higher quality, safer, more efficient care, and health care delivery organizations become better places to work. (https://healthit.ahrq.gov/health-care-theme/learning-health-system)'),
  'D000080902': NtO(MH='Mandibular Nerve Injuries', MS='Traumatic injuries to the MANDIBULAR NERVE and its branches. Note that marginal mandibular nerve injury is a type of FACIAL NERVE INJURIES. Injuries to LINGUAL NERVE branch of mandibular nerve is LINGUAL NERVE INJURIES.'),
  'D000080903': NtO(MH='Dual Anti-Platelet Therapy', MS='Treatment with a combination of two types of antiplatelet agents to prevent blood clotting - usually ASPIRIN and another of the PLATELET AGGREGATION INHIBITORS.'),
  'D000080905': NtO(MH='Homeobox A10 Proteins', MS="Sequence-specific transcription factor which is part of a developmental regulatory system that provides cells with specific positional identities on the anterior-posterior axis of tissues. They bind to the DNA sequence 5'-AA[AT]TTTTATTAC-3'."),
  'D000080906': NtO(MH='Cerebellar Golgi Cells', MS='Inhibitory interneurons embedded in the granular layer of the CEREBELLAR CORTEX.'),
  'D000080907': NtO(MH='Single-Case Studies as Topic', MS='Trials that consider an individual patient as the sole unit of observation in a study investigating the efficacy or side-effects of different interventions to determine the optimal intervention for an individual patient using objective data-driven criteria.'),
  'D000080908': NtO(MH='Broadly Neutralizing Antibodies', MS='Neutralizing antibodies that reduce or abolish some biological activity of a soluble antigen or infectious agent (e.g., HIV) by targeting several epitopes.'),
  'D000080909': NtO(MH='Carotid-Femoral Pulse Wave Velocity', MS='The transit time of an AORTIC PRESSURE wave between the COMMON CAROTID ARTERY and the COMMON FEMORAL ARTERY. It is a standard measure of ARTERIAL STIFFNESS.'),
  'D000080910': NtO(MH='Ambient Intelligence', MS='An emerging paradigm in which people are empowered through intelligent (digital) tools embedded in the surrounding environment. These tools proactively support human existence by being sensitive, adaptive, and responsive to the needs, habits, gestures, and emotions of individuals.'),
  'D000080911': NtO(MH='Cheminformatics', MS='The use of computational and data retrieval techniques in understanding problems of chemistry. Cheminformatics strategies are useful in high throughput drug discovery.'),
  'D000080924': NtO(MH='Aldehyde Dehydrogenase 1', MS='A member of the Aldehyde Dehydrogenase family of enzymes involved in the synthesis of acetate from ethanol. It is also a tumor marker.'),
  'D000080925': NtO(MH='Microwave Imaging', MS='Use of MICROWAVES as a diagnostic tool.'),
  'D000080942': NtO(MH='Post-Cardiac Arrest Syndrome', MS='A reperfusion syndrome characterized by various pathophysiological processes after CARDIAC ARREST.  It may include post-cardiac arrest brain injury (HYPOXIA-ISCHEMIA, BRAIN), cardiocirculatory dysfunction, (e.g., systemic ISCHEMIA), HYPERGLYCEMIA; MULTIPLE ORGAN FAILURE and delayed death.'),
  'D000080943': NtO(MH='Bacterial Outer Membrane', MS='The outermost cell envelope of GRAM-NEGATIVE BACTERIA composed of PHOSPHOLIPIDS; LIPOPOLYSACCHARIDES (LPS), and BACTERIAL OUTER MEMBRANE PROTEINS. The bacterial outer membrane LPS demonstrates cytotoxicity (see LIPID A and BACTERIAL O ANTIGEN).'),
  'D000080944': NtO(MH='Interosseous Membrane', MS='A sheet of fibrous connective tissue rich in COLLAGEN often linking two parallel bony structures forming a syndesmosis type joint. It provides longitudinal stability, TENSILE STRENGTH, and weight distribution/transfer and may allow limited movement in syndesmoses.'),
  'D000080945': NtO(MH='Lymph Node Ratio', MS='The ratio of metastatic to total examined LYMPH NODES.'),
  'D000080983': NtO(MH='Bone Marrow Failure Disorders', MS='Inherited or acquired diseases characterized by insufficient and/or dysplastic blood cells.'),
  'D000080984': NtO(MH='Congenital Bone Marrow Failure Syndromes', MS='Inherited syndromes characterized by deficiency or absence of various blood cells due to mutations that affect HEMATOPOIETIC STEM CELLS development and proliferation.'),
  'D000081': NtO(MH='Acetamides', MS='Derivatives of acetamide that are used as solvents, as mild irritants, and in organic synthesis.'),
  'D000081002': NtO(MH='Sino-Japanese War', MS='Military conflict involving China and Japan 1937 - 1945.'),
  'D000081003': NtO(MH='Shwachman-Diamond Syndrome', MS='An inherited syndrome characterized by EXOCRINE PANCREATIC INSUFFICIENCY; hematologic abnormalities (e.g., bone marrow hypoplasia), and skeletal abnormalities (e.g., metaphyseal chondroplasia). GERMLINE MUTATIONS in the SBDS gene are associated with Shwachman-Diamond Syndrome.'),
  'D000081004': NtO(MH='Cyclopentane Monoterpenes', MS='Monoterpenes that include a cyclopentane ring in their structure.'),
  'D000081005': NtO(MH='Cyclohexane Monoterpenes', MS='Monoterpenes that include a cyclohexane ring in their structure.'),
  'D000081006': NtO(MH='Missed Diagnosis', MS='Failure to identify or diagnose a medical condition at the time a health professional is acquainted with the symptoms.'),
  'D000081007': NtO(MH='Cymenes', MS='A subclass of monoterpenes that contain a single benzene ring structure with an isopropyl group and one or more methyl groups.'),
  'D000081008': NtO(MH='Bicyclic Monoterpenes', MS='Monoterpenes containing two ring structures that are joined either by bridgehead carbon atoms or fused ring structure.'),
  'D000081010': NtO(MH='Food Fussiness', MS='Patterns associated with eating including modes of feeding, rhythmic patterns of eating, and time intervals.'),
  'D000081011': NtO(MH='Hepatic Infarction', MS='Formation of infarct resulting from obstruction of HEPATIC ARTERY and/or PORTAL VEIN most often after LIVER TRANSPLANTATION or hepatobiliary surgery.'),
  'D000081012': NtO(MH='Autoimmune Pancreatitis', MS='Chronic pancreatitis associated with autoimmune diseases.'),
  'D000081013': NtO(MH='Suicide, Completed', MS='The successful attempt to kill oneself.'),
  'D000081014': NtO(MH='Dihydrostilbenoids', MS='Organic compounds characterized by the functional group 1,2-dihydrostilbene.'),
  'D000081015': NtO(MH='Ototoxicity', MS='Damage to the EAR or its function secondary to exposure to toxic substances such as drugs used in CHEMOTHERAPY; IMMUNOTHERAPY; or RADIATION.'),
  'D000081022': NtO(MH='Sphingosine Phosphorylcholine Receptors', MS='A subfamily of lysophospholipid receptors with specificity for lysophosphatidic acid.'),
  'D000081025': NtO(MH='Sphingosine-1-Phosphate Receptors', MS='A subfamily of lysophospholipid receptors with specificity for sphingosine-1-phosphate (e.g., FINGOLIMOD), sphinganine 1-phosphate, 4-hydroxysphinganine 1-phosphate.'),
  'D000081026': NtO(MH='Polyprenols', MS='Long chain isoprene compounds that include a hydroxyl group at the terminal carbon atom.'),
  'D000081027': NtO(MH='Extreme Hot Weather', MS=None),
  'D000081028': NtO(MH='Toxic Optic Neuropathy', MS='Damage to the eye or its function (e.g., VISUAL IMPAIRMENT) due to OPTIC NERVE damage secondary to toxic substances such as drugs used in CHEMOTHERAPY; IMMUNOTHERAPY; or RADIATION.'),
  'D000081029': NtO(MH='Pulmonary Arterial Hypertension', MS='A progressive rare pulmonary disease characterized by high blood pressure in the PULMONARY ARTERY.'),
  'D000081030': NtO(MH='Myotoxicity', MS='Damage to the muscle or its function secondary to toxic substances such as drugs used in CHEMOTHERAPY; IMMUNOTHERAPY; or RADIATION.'),
  'D000081031': NtO(MH='Fatty Acid Elongases', MS='Enzymes conserved from yeast to mammals which mediate elongation of FATTY ACIDS into very-long-chain fatty acids.'),
  'D000081032': NtO(MH='Pancreatitis, Acute Hemorrhagic', MS='An acute INFLAMMATION of the PANCREAS in which the initial edematous pancreatitis evolved into necrosis accompanied by HEMORRHAGE.'),
  'D000081033': NtO(MH='Potassium Channels, Sodium-Activated', MS='Potassium channels whose activation is dependent on intracellular sodium concentrations.'),
  'D000081034': NtO(MH='Indigenous Peoples', MS='Descendants who self-identify as members of a group who inhabited a country or region at the time when people of different cultures or ethnic origins arrived. They often maintain their distinct language, culture, and beliefs.'),
  'D000081035': NtO(MH='Heterogeneous Nuclear Ribonucleoprotein D0', MS="Member of a subfamily of ubiquitously expressed heterogeneous nuclear ribonucleoproteins (hnRNPs), heterogeneous nuclear ribonucleoprotein D0 (hnRNP D0) specifically binds RNA molecules that contain AU RICH ELEMENTS found within the 3'-UTR of many PROTO-ONCOGENES and CYTOKINES mRNAs. It functions as a transcription factor, binds to double- and single-stranded DNA sequences in a specific manner."),
  'D000081042': NtO(MH='Body Integrity Identity Disorder', MS='Condition in which individuals desire the amputation of one or more healthy limbs or who desire a paralysis.'),
  'D000081062': NtO(MH='Heme-Binding Proteins', MS='Metalloproteins that contain a HEME ligand as the prosthetic group.'),
  'D000081082': NtO(MH='Phosphoinositide-3 Kinase Inhibitors', MS='Agents that inhibit PHOSPHOINOSITIDE-3 KINASE activity.'),
  'D000081084': NtO(MH='Accidental Injuries', MS='Injuries resulting from events or circumstances that are unforeseen, unplanned, unanticipated, and without premeditation.'),
  'D000081102': NtO(MH='WAP Four-Disulfide Core Domain Protein 2', MS='A secreted 25-kDa glycoprotein member of the Whey-acidic-protein (WAP) four-disulfide core (WFDC) domain family with tissue specific expression in the EPIDIDYMIS. It is also expressed in a limited number of other organs which include female reproductive tract, breast tissue, kidney, regions of the respiratory tract and nasopharynx. The protein is a biomarker for epithelial ovarian cancer.'),
  'D000081122': NtO(MH='Epigenome', MS='A network of chemical compounds surrounding DNA that modify the genome without altering the DNA sequences and have a role in determining which genes are active in a cell.'),
  'D000081142': NtO(MH='Evidence-Based Pharmacy Practice', MS="The practice of pharmacy in which the pharmacist makes decisions, taking into account the best available current research evidence, the pharmacist's expertise, and the needs and preferences of the patient."),
  'D000081163': NtO(MH='Cell Line Authentication', MS='Techniques used to characterize and confirm the identity of a CELL LINE. Characterization involves authentication to verify that it matches the original description of the cells and ensures that they are not cross-contaminated with another cell line. This is done to avoid attributing data incorrectly to the wrong cell line.'),
  'D000081164': NtO(MH='Drug Collateral Sensitivity', MS='Increase in sensitivity of an organism, disease, or tissue to one drug that develops following the loss of sensitivity (i.e. resistance) to another drug.'),
  'D000081182': NtO(MH='Noninvasive Prenatal Testing', MS='The analysis of cell-free fetal DNA or the DNA from fetal cells that are biopsied from a pregnant woman\xe2\x80\x99s blood, to screen for genetically-determined traits and genetic abnormalities.'),
  'D000081183': NtO(MH='Codon Usage', MS="The frequency of occurrence, in a specific organism's DNA sequence, of one of several potential synonymous codons that code for a particular amino acid. Frequently, there is a nonrandom pattern (bias) in the usage of a particular codon or codons over other synonymous codons."),
  'D000081186': NtO(MH='Blockchain', MS='Decentralized transaction and data management technology providing security, anonymity and data integrity without any third party control of transactions. Related research is focused on improving aspects  regarding privacy and security. (From PLoS One. 2016 Oct 3;11(10):e0163477)'),
  'D000081202': NtO(MH='Right to Work', MS='The right of individuals to work, to free choice of employment, to just and favorable conditions of work and to protection against unemployment.'),
  'D000081203': NtO(MH='Monoterpene Aldehydes and Ketones', MS='Monoterpene compounds that include a double bonded oxygen attached to the one or more of the carbon atoms.'),
  'D000081204': NtO(MH='Chromatin Immunoprecipitation Sequencing', MS='A technique for identifying the BINDING SITES on nucleic acid sequences that are associated with binding proteins.'),
  'D000081205': NtO(MH='Hair Analysis', MS='Examination of hair by chemical, physical, or microscopic means.'),
  'D000081206': NtO(MH='Duration of Therapy', MS='Planned or actual duration of various treatments or preventive measures.'),
  'D000081207': NtO(MH='Primary Immunodeficiency Diseases', MS='Genetic immunologic deficiency diseases and syndromes due to mutations in genes involved in IMMUNITY generally characterized by an increased susceptibility to infectious diseases. They are often associated with AUTOIMMUNE DISEASE manifestations.'),
  'D000081208': NtO(MH='Hereditary Complement Deficiency Diseases', MS='Genetic disorders due to mutations in genes involved in COMPLEMENT SYSTEM PROTEINS. They are often classified into distinct pathway of complement activation where causative mutations are found (e.g., classical pathway, lectin pathway, alternative pathway, and terminal complement pathway).'),
  'D000081222': NtO(MH='Monocyclic Sesquiterpenes', MS='Sesquiterpene compounds that contain a single ring structure.'),
  'D000081223': NtO(MH='Polycyclic Sesquiterpenes', MS='Sesquiterpenes that contain two or more ring structures'),
  'D000081225': NtO(MH='Stilbestrols', MS='Stilbene derivatives the contain one or more hydroxyl groups attached to the benzene ring structures.'),
  'D000081226': NtO(MH='Sennosides', MS='Medications derived from SENNA EXTRACT that are used to treat CONSTIPATION.'),
  'D000081227': NtO(MH='Neurosteroids', MS='Endogenous compounds or drugs that affect neuronal excitability through modulation of specific ionotropic receptors (e.g., GABA-A RECEPTORS). Endogenous neurosteroids are steroid hormones de novo synthesized by neurons and glial cells from steroid metabolite precursors (e.g., PREGNENOLONE).'),
  'D000081229': NtO(MH='Artificially Sweetened Beverages', MS='Beverages sweetened with SWEETENING AGENTS that are synthetic or artificial as opposed to naturally-occurring. Examples of sweetening agents include ASPARTAME, sucralose, and saccharin.'),
  'D000081230': NtO(MH='Forensic Entomology', MS='The use of insects and related arthropods that inhabit decomposing corpses or remains in the conduct of medico-legal or medico-criminal investigations.'),
  'D000081242': NtO(MH='B-Cell Lymphoma 3 Protein', MS='A member of the I kappa B family, B-cell lymphoma 3-encoded (bcl-3) protein is involved in the regulation of TRANSCRIPTIONAL ACTIVATION of NF-KAPPA B target genes.'),
  'D000081243': NtO(MH='Sphingosine 1 Phosphate Receptor Modulators', MS='Agents that affect the function of G-protein coupled SPHINGOSINE 1-PHOSPHATE RECEPTORS. Their binding to the receptors blocks lymphocyte migration and are often used as IMMUNOSUPPRESSANTS.'),
  'D000081246': NtO(MH='RNA-Seq', MS='High-throughput nucleotide sequencing techniques developed for determining and analyzing the composition of the TRANSCRIPTOME of a sample.'),
  'D000081247': NtO(MH='CSK Tyrosine-Protein Kinase', MS='Protein tyrosine kinases that phosphorylate tyrosine residues located in the C-terminal tails of SRC-FAMILY KINASES.'),
  'D000081248': NtO(MH='Allylbenzene Derivatives', MS='Compounds that include a single 1-phenyl-2-propene moiety in their structure.'),
  'D000081262': NtO(MH='Randomized Controlled Trial, Veterinary', MS='A work that reports on a clinical trial with animal subjects that involves at least one test treatment and one control treatment, concurrent enrollment and follow-up of the test- and control-treated groups, and in which the treatments to be administered are selected by a random process, such as the use of a random-numbers table.'),
  'D000081282': NtO(MH='Perfusion Index', MS='The ratio of arterial blood flow (PULSATILE FLOW) to venous, capillary, and tissue blood flow  (non-pulsatile blood flow) - as measured regionally or peripherally.'),
  'D000081322': NtO(MH='Military Health Services', MS='Services organized for the diagnosis and treatment of disease and the maintenance of health primarily of MILITARY PERSONNEL.'),
  'D000081324': NtO(MH='Veterans Health Services', MS='Services organized for the diagnosis and treatment of diseases and maintenance of health primarily of VETERANS.'),
  'D000081342': NtO(MH='Nudiviridae', MS='A family of ARTHROPOD viruses, including INSECT VIRUSES and crustacean viruses. They are rod-shaped, enveloped, and circular dsDNA viruses that share structural and replication characteristics with BACULOVIRIDAE. Genera include Alphanudivirus and Betanudivirus.'),
  'D000081362': NtO(MH='Lipidomics', MS='A branch of metabolomics concerned with the study of global LIPIDS profiles in physiological systems.'),
  'D000081363': NtO(MH='Occlusion Bodies, Viral', MS='A polyhedral or ovocylindrical shaped matrix composed of a single crystalline protein (OCCLUSION BODY MATRIX PROTEINS) in which VIRIONS are embedded. It is distinguished from the area of the CYTOPLASM or CELL NUCLEUS in which virus components are being synthesized and assembled, known as VIRAL INCLUSION BODIES.'),
  'D000081364': NtO(MH='Multiparametric Magnetic Resonance Imaging', MS='Magnetic Resonance Imaging technique that combines functional imaging techniques such as diffusion-weighted imaging (DWI), dynamic contrast-enhanced imaging and magnetic spectroscopy. This technique is widely used for active surveillance in prostate cancer imaging.'),
  'D000081382': NtO(MH='Pyruvate Dehydrogenase (Acetyl-Transferring) Kinase', MS='A pyruvate dehydrogenase kinase isozyme located in the mitochondria which converts PYRUVATE to ACETYL CoA in the CITRIC ACID CYCLE, phosphorylates SERINE residues on pyruvate dehydrogenase using ATP, and plays a key role in the regulation of GLUCOSE and fatty acid metabolism.'),
  'D000081404': NtO(MH='Quaternary Prevention', MS='Actions taken to protect individuals (i.e. persons/patients) from unnecessary medical interventions that are likely to cause more harm than good.'),
  'D000081406': NtO(MH='Cyclophilin D', MS='A member of the cyclophilins family of isomerases which plays a role in the regulation of the permeability transition pore in mitochondria.'),
  'D000081410': NtO(MH='Germinal Center Kinases', MS='Serine-threonine kinases characterized by an N-terminal STE20 kinase domain.'),
  'D000081411': NtO(MH='Augmented Reality', MS='Technology in which images of virtual objects and/or other digital information are superimposed over the real time view of the physical objects providing a composite view with guides for carrying out a task with or on the physical objects.'),
  'D000081412': NtO(MH='Myosin VIIa', MS='Also referred to as unconventional myosin VIIa, these are actin-based motor molecules with ATPase activity that function in intracellular movements (e.g., HAIR CELLS, AUDITORY).'),
  'D000081413': NtO(MH='Exposome', MS='The measure of all the exposures of an individual from all sources, including environmental and occupational sources, in a lifetime and how those exposures relate to health. (from https://www.cdc.gov/niosh/topics/exposome/ on 06/06/2019)'),
  'D000081414': NtO(MH='Nanopore Sequencing', MS='A sequencing protocol that drives nucleic acids (DNA or RNA) in an electric field through NANOPORES allowing single molecule sequence analysis.'),
  'D000081415': NtO(MH='Clinical Decision Rules', MS='Scores or algorithms that correlate with clinically significant findings and that are used to predict an outcome, or suggest a diagnostic or therapeutic action.'),
  'D000081416': NtO(MH='SecA Proteins', MS='ATPases that provide energy for the translocation of proteins across bacterial PLASMA MEMBRANES and THYLAKOID membranes, by the SEC TRANSLOCATION CHANNELS. SecA is a component of the bacterial preprotein translocase which is comprised of SecA, the SECYEG TRANSLOCON, and the accessory domain proteins SecD, SecF, YajC, and YidC.'),
  'D000081442': NtO(MH='Trichothecenes, Type B', MS='A subclass of Trichothecene derivatives that include the 12,13-epoxytrichothec-9-ene structure with a keto (carbonyl) function at C-8.'),
  'D000081443': NtO(MH='Trichothecenes, Type C', MS='A subclass of Trichothecene derivatives that includes the 12,13-epoxytrichothec-9-ene structure with an epoxide oxygen bond to the C7 and C8 carbon atoms.'),
  'D000081482': NtO(MH='Bioaccumulation', MS='An increase in the concentration of an exogenous substance in the tissues of organisms higher than surrounding ENVIRONMENT. Accumulation of such XENOBIOTICS at successively higher levels up the FOOD CHAIN is called biomagnification. Bioaccumulation of toxic chemicals (e.g., Lead and DDT) may result in CHEMICALLY-INDUCED DISORDERS.'),
  'D000081483': NtO(MH='Squamous Intraepithelial Lesions', MS='A cytological test finding that shows abnormal lesions of SQUAMOUS EPITHELIAL cells.'),
  'D000081503': NtO(MH='Organoiron Compounds', MS='Organic compounds that contain iron in carbon-to-iron chemical bond.'),
  'D000082': NtO(MH='Acetaminophen', MS='Analgesic antipyretic derivative of acetanilide. It has weak anti-inflammatory properties and is used as a common analgesic, but may cause liver, blood cell, and kidney damage.'),
  'D000083': NtO(MH='Acetanilides', MS='Compounds based on N-phenylacetamide, that are similar in structure to 2-PHENYLACETAMIDES. They are precursors of many other compounds. They were formerly used as ANALGESICS and ANTIPYRETICS, but often caused lethal METHEMOGLOBINEMIA.'),
  'D000084': NtO(MH='Acetate Kinase', MS='An enzyme that catalyzes reversibly the phosphorylation of acetate in the presence of a divalent cation and ATP with the formation of acetylphosphate and ADP. It is important in the glycolysis process. EC 2.7.2.1.'),
  'D000085': NtO(MH='Acetates', MS='Derivatives of ACETIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the carboxymethane structure.'),
  'D000086': NtO(MH='Acetazolamide', MS='One of the CARBONIC ANHYDRASE INHIBITORS that is sometimes effective against absence seizures. It is sometimes useful also as an adjunct in the treatment of tonic-clonic, myoclonic, and atonic seizures, particularly in women whose seizures occur or are exacerbated at specific times in the menstrual cycle. However, its usefulness is transient often because of rapid development of tolerance. Its antiepileptic effect may be due to its inhibitory effect on brain carbonic anhydrase, which leads to an increased transneuronal chloride gradient, increased chloride current, and increased inhibition. (From Smith and Reynard, Textbook of Pharmacology, 1991, p337)'),
  'D000089': NtO(MH='Acetic Anhydrides', MS='Compounds used extensively as acetylation, oxidation and dehydrating agents and in the modification of proteins and enzymes.'),
  'D000090': NtO(MH='Acetoacetates', MS='Salts and derivatives of acetoacetic acid.'),
  'D000091': NtO(MH='Acetobacter', MS='A species of gram-negative bacteria of the family ACETOBACTERACEAE found in FLOWERS and FRUIT. Cells are ellipsoidal to rod-shaped and straight or slightly curved.'),
  'D000092': NtO(MH='Acetohexamide', MS='A sulfonylurea hypoglycemic agent that is metabolized in the liver to 1-hydrohexamide.'),
  'D000093': NtO(MH='Acetoin', MS='A product of fermentation. It is a component of the butanediol cycle in microorganisms. In mammals it is oxidized to carbon dioxide.'),
  'D000094': NtO(MH='Acetoin Dehydrogenase', MS='An enzyme that catalyzes the conversion of acetoin to diacetyl in the presence of NAD.'),
  'D000095': NtO(MH='Acetolactate Synthase', MS='A flavoprotein enzyme that catalyzes the formation of acetolactate from 2 moles of PYRUVATE in the biosynthesis of VALINE and the formation of acetohydroxybutyrate from pyruvate and alpha-ketobutyrate in the biosynthesis of ISOLEUCINE. This enzyme was formerly listed as EC 4.1.3.18.'),
  'D000096': NtO(MH='Acetone', MS='A colorless liquid used as a solvent and an antiseptic. It is one of the ketone bodies produced during ketoacidosis.'),
  'D000097': NtO(MH='Acetonitriles', MS='Compounds in which a methyl group is attached to the cyano moiety.'),
  'D000098': NtO(MH='Acetophenones', MS='Derivatives of the simplest aromatic ketone acetophenone (of general formula C6H5C(O)CH3).'),
  'D000099': NtO(MH='Acetoxyacetylaminofluorene', MS='An alkylating agent that forms DNA ADDUCTS at the C-8 position in GUANINE, resulting in single strand breaks. It has demonstrated carcinogenic action.'),
  'D000100': NtO(MH='Acetrizoic Acid', MS='An iodinated radiographic contrast medium used as acetrizoate sodium in HYSTEROSALPINGOGRAPHY.'),
  'D000101': NtO(MH='Acetyl-CoA C-Acetyltransferase', MS='An enzyme that catalyzes the formation of acetoacetyl-CoA from two molecules of ACETYL COA. Some enzymes called thiolase or thiolase-I have referred to this activity or to the activity of ACETYL-COA C-ACYLTRANSFERASE.'),
  'D000102': NtO(MH='Acetyl-CoA C-Acyltransferase', MS='Enzyme that catalyzes the final step of fatty acid oxidation in which ACETYL COA is released and the CoA ester of a fatty acid two carbons shorter is formed.'),
  'D000103': NtO(MH='Acetyl-CoA Carboxylase', MS='A carboxylating enzyme that catalyzes the conversion of ATP, acetyl-CoA, and HCO3- to ADP, orthophosphate, and malonyl-CoA. It is a biotinyl-protein that also catalyzes transcarboxylation. The plant enzyme also carboxylates propanoyl-CoA and butanoyl-CoA (From Enzyme Nomenclature, 1992) EC 6.4.1.2.'),
  'D000104': NtO(MH='Acetyl-CoA Hydrolase', MS='An enzyme that catalyzes reversibly the hydrolysis of acetyl-CoA to yield CoA and acetate. The enzyme is involved in the oxidation of fatty acids. EC 3.1.2.1.'),
  'D000105': NtO(MH='Acetyl Coenzyme A', MS='Acetyl CoA participates in the biosynthesis of fatty acids and sterols, in the oxidation of fatty acids and in the metabolism of many amino acids. It also acts as a biological acetylating agent.'),
  'D000106': NtO(MH='Acetate-CoA Ligase', MS='An enzyme that catalyzes the formation of CoA derivatives from ATP, acetate, and CoA to form AMP, pyrophosphate, and acetyl CoA. It acts also on propionates and acrylates. EC 6.2.1.1.'),
  'D000107': NtO(MH='Acetylation', MS='Formation of an acetyl derivative. (Stedman, 25th ed)'),
  'D000108': NtO(MH='Acetylcarnitine', MS='An acetic acid ester of CARNITINE that facilitates movement of ACETYL COA into the matrices of mammalian MITOCHONDRIA during the oxidation of FATTY ACIDS.'),
  'D000109': NtO(MH='Acetylcholine', MS='A neurotransmitter found at neuromuscular junctions, autonomic ganglia, parasympathetic effector junctions, a subset of sympathetic effector junctions, and at many sites in the central nervous system.'),
  'D000110': NtO(MH='Acetylcholinesterase', MS='An enzyme that catalyzes the hydrolysis of ACETYLCHOLINE to CHOLINE and acetate. In the CNS, this enzyme plays a role in the function of peripheral neuromuscular junctions. EC 3.1.1.7.'),
  'D000111': NtO(MH='Acetylcysteine', MS='The N-acetyl derivative of CYSTEINE. It is used as a mucolytic agent to reduce the viscosity of mucous secretions. It has also been shown to have antiviral effects in patients with HIV due to inhibition of viral stimulation by reactive oxygen intermediates.'),
  'D000112': NtO(MH='Acetyldigitoxins', MS='Cardioactive derivatives of lanatoside A or of DIGITOXIN. They are used for fast digitalization in congestive heart failure.'),
  'D000113': NtO(MH='Acetyldigoxins', MS='Alpha- or beta-acetyl derivatives of DIGOXIN or lanatoside C from Digitalis lanata. They are better absorbed and longer acting than digoxin and are used in congestive heart failure.'),
  'D000114': NtO(MH='Acetylene', MS='The simplest two carbon alkyne with the formula HCCH.'),
  'D000115': NtO(MH='Acetylesterase', MS='An enzyme that catalyzes the conversion of acetate esters and water to alcohols and acetate. EC 3.1.1.6.'),
  'D000116': NtO(MH='Acetylgalactosamine', MS='The N-acetyl derivative of galactosamine.'),
  'D000117': NtO(MH='Acetylglucosamine', MS='The N-acetyl derivative of glucosamine.'),
  'D000118': NtO(MH='Acetylglucosaminidase', MS='A beta-N-Acetylhexosaminidase that catalyzes the hydrolysis of terminal, non-reducing 2-acetamido-2-deoxy-beta-glucose residues in chitobiose and higher analogs as well as in glycoproteins. Has been used widely in structural studies on bacterial cell walls and in the study of diseases such as MUCOLIPIDOSIS and various inflammatory disorders of muscle and connective tissue.'),
  'D000119': NtO(MH='Acetylmuramyl-Alanyl-Isoglutamine', MS='Peptidoglycan immunoadjuvant originally isolated from bacterial cell wall fragments; also acts as pyrogen and may cause arthritis; stimulates both humoral and cellular immunity.'),
  'D000120': NtO(MH='Acecainide', MS='A major metabolite of PROCAINAMIDE. Its anti-arrhythmic action may cause cardiac toxicity in kidney failure.'),
  'D000121': NtO(MH='Acetylserotonin O-Methyltransferase', MS='An enzyme that catalyzes the transfer of a methyl group from S-adenosylmethionine to N-acetylserotonin to form N-acetyl-5-methoxytryptamine (MELATONIN).'),
  'D000122': NtO(MH='Acetylthiocholine', MS='An agent used as a substrate in assays for cholinesterases, especially to discriminate among enzyme types.'),
  'D000123': NtO(MH='Acetyltransferases', MS='Enzymes catalyzing the transfer of an acetyl group, usually from acetyl coenzyme A, to another compound. EC 2.3.1.'),
  'D000124': NtO(MH='Achievement', MS='Success in bringing an effort to the desired end; the degree or level of success attained in some specified area (esp. scholastic) or in general.'),
  'D000125': NtO(MH='Achilles Tendon', MS='Tendon that connects the muscles in the back of the calf to the HEEL BONE.'),
  'D000126': NtO(MH='Achlorhydria', MS='A lack of HYDROCHLORIC ACID in GASTRIC JUICE despite stimulation of gastric secretion.'),
  'D000127': NtO(MH='Acholeplasma', MS='A genus of gram-negative organisms including saprophytic and parasitic or pathogenic species.'),
  'D000128': NtO(MH='Acholeplasma laidlawii', MS='An organism originally isolated from sewage, manure, humus, and soil, but recently found as a parasite in mammals and birds.'),
  'D000129': NtO(MH='Acholeplasmataceae', MS='Gram-negative organisms including apparently free-living saphrophytes as well as mammalian and avian parasites, and possibly pathogens.'),
  'D000130': NtO(MH='Achondroplasia', MS='An autosomal dominant disorder that is the most frequent form of short-limb dwarfism. Affected individuals exhibit short stature caused by rhizomelic shortening of the limbs, characteristic facies with frontal bossing and mid-face hypoplasia, exaggerated lumbar lordosis, limitation of elbow extension, GENU VARUM, and trident hand. (Online Mendelian Inheritance in Man, http://www.ncbi.nlm.nih.gov/Omim, MIM#100800, April 20, 2001)'),
  'D000134': NtO(MH='Acid Etching, Dental', MS='Preparation of TOOTH surfaces and DENTAL MATERIALS with etching agents, usually phosphoric acid, to roughen the surface to increase adhesion or osteointegration.'),
  'D000135': NtO(MH='Acid Phosphatase', MS='An enzyme that catalyzes the conversion of an orthophosphoric monoester and water to an alcohol and orthophosphate. EC 3.1.3.2.'),
  'D000136': NtO(MH='Acid-Base Equilibrium', MS='The balance between acids and bases in the BODY FLUIDS. The pH (HYDROGEN-ION CONCENTRATION) of the arterial BLOOD provides an index for the total body acid-base balance.'),
  'D000137': NtO(MH='Acid-Base Imbalance', MS='Disturbances in the ACID-BASE EQUILIBRIUM of the body.'),
  'D000138': NtO(MH='Acidosis', MS='A pathologic condition of acid accumulation or depletion of base in the body. The two main types are RESPIRATORY ACIDOSIS and metabolic acidosis, due to metabolic acid build up.'),
  'D000140': NtO(MH='Acidosis, Lactic', MS='Acidosis caused by accumulation of lactic acid more rapidly than it can be metabolized. It may occur spontaneously or in association with diseases such as DIABETES MELLITUS; LEUKEMIA; or LIVER FAILURE.'),
  'D000141': NtO(MH='Acidosis, Renal Tubular', MS='A group of genetic disorders of the KIDNEY TUBULES characterized by the accumulation of metabolically produced acids with elevated plasma chloride, hyperchloremic metabolic ACIDOSIS. Defective renal acidification of URINE (proximal tubules) or low renal acid excretion (distal tubules) can lead to complications such as HYPOKALEMIA, hypercalcinuria with NEPHROLITHIASIS and NEPHROCALCINOSIS, and RICKETS.'),
  'D000142': NtO(MH='Acidosis, Respiratory', MS='Respiratory retention of carbon dioxide. It may be chronic or acute.'),
  'D000143': NtO(MH='Acids', MS="Chemical compounds which yield hydrogen ions or protons when dissolved in water, whose hydrogen can be replaced by metals or basic radicals, or which react with bases to form salts and water (neutralization). An extension of the term includes substances dissolved in media other than water. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000144': NtO(MH='Acids, Acyclic', MS='Carboxylic acids that have open-chain molecular structures as opposed to ring-shaped structures.'),
  'D000145': NtO(MH='Acids, Aldehydic', MS='Dicarboxylic acids in which one of the carboxyl groups (-COOH) has been replaced by an aldehyde group (-CHO).'),
  'D000146': NtO(MH='Acids, Carbocyclic', MS='Carboxylic acids that have a homocyclic ring structure in which all the ring atoms are carbon.'),
  'D000147': NtO(MH='Acids, Heterocyclic', MS='A class of acids containing a ring structure in which at least one atom other than CARBON is incorporated.'),
  'D000148': NtO(MH='Acids, Noncarboxylic', MS='Inorganic acids with a non metal, other than carbon, attached to hydrogen, or an acid radical containing no carbon.'),
  'D000149': NtO(MH='Acidulated Phosphate Fluoride', MS='A sodium fluoride solution, paste or powder, which has been acidulated to pH 3 to 4 and buffered with a phosphate. It is used in the prevention of dental caries.'),
  'D000150': NtO(MH='Acinetobacter', MS='A genus of gram-negative bacteria of the family MORAXELLACEAE, found in soil and water and of uncertain pathogenicity.'),
  'D000151': NtO(MH='Acinetobacter Infections', MS='Infections with bacteria of the genus ACINETOBACTER.'),
  'D000152': NtO(MH='Acne Vulgaris', MS='A chronic disorder of the pilosebaceous apparatus associated with an increase in sebum secretion. It is characterized by open comedones (blackheads), closed comedones (whiteheads), and pustular nodules. The cause is unknown, but heredity and age are predisposing factors.'),
  'D000153': NtO(MH='Acne Keloid', MS='A type of acneiform disorder in which secondary pyogenic infection in and around pilosebaceous structures ends in keloidal scarring. It manifests as persistent folliculitis of the back of the neck associated with occlusion of the follicular orifices. It is most often encountered in black or Asian men.'),
  'D000154': NtO(MH='Aconitate Hydratase', MS='An enzyme that catalyzes the reversible hydration of cis-aconitate to yield citrate or isocitrate. It is one of the citric acid cycle enzymes. EC 4.2.1.3.'),
  'D000156': NtO(MH='Aconitic Acid', MS='A tricarboxylic acid with the formula (COOH)-CH2-C(COOH)=CH-COOH.'),
  'D000157': NtO(MH='Aconitine', MS='A C19 norditerpenoid alkaloid (DITERPENES) from the root of ACONITUM; DELPHINIUM and larkspurs. It activates VOLTAGE-GATED SODIUM CHANNELS. It has been used to induce ARRHYTHMIAS in experimental animals and it has anti-inflammatory and anti-neuralgic properties.'),
  'D000158': NtO(MH='Acoustic Impedance Tests', MS='Objective tests of middle ear function based on the difficulty (impedance) or ease (admittance) of sound flow through the middle ear. These include static impedance and dynamic impedance (i.e., tympanometry and impedance tests in conjunction with intra-aural muscle reflex elicitation). This term is used also for various components of impedance and admittance (e.g., compliance, conductance, reactance, resistance, susceptance).'),
  'D000159': NtO(MH='Vestibulocochlear Nerve', MS="The 8th cranial nerve. The vestibulocochlear nerve has a cochlear part (COCHLEAR NERVE) which is concerned with hearing and a vestibular part (VESTIBULAR NERVE) which mediates the sense of balance and head position. The fibers of the cochlear nerve originate from neurons of the SPIRAL GANGLION and project to the cochlear nuclei (COCHLEAR NUCLEUS). The fibers of the vestibular nerve arise from neurons of Scarpa's ganglion and project to the VESTIBULAR NUCLEI."),
  'D000160': NtO(MH='Vestibulocochlear Nerve Diseases', MS='Pathological processes of the VESTIBULOCOCHLEAR NERVE, including the branches of COCHLEAR NERVE and VESTIBULAR NERVE. Common examples are VESTIBULAR NEURITIS, cochlear neuritis, and ACOUSTIC NEUROMA. Clinical signs are varying degree of HEARING LOSS; VERTIGO; and TINNITUS.'),
  'D000161': NtO(MH='Acoustic Stimulation', MS='Use of sound to elicit a response in the nervous system.'),
  'D000162': NtO(MH='Acoustics', MS='The branch of physics that deals with sound and sound waves. In medicine it is often applied in procedures in speech and hearing studies. With regard to the environment, it refers to the characteristics of a room, auditorium, theatre, building, etc. that determines the audibility or fidelity of sounds in it. (From Random House Unabridged Dictionary, 2d ed)'),
  'D000163': NtO(MH='Acquired Immunodeficiency Syndrome', MS='An acquired defect of cellular immunity associated with infection by the human immunodeficiency virus (HIV), a CD4-positive T-lymphocyte count under 200 cells/microliter or less than 14% of total lymphocytes, and increased susceptibility to opportunistic infections and malignant neoplasms. Clinical manifestations also include emaciation (wasting) and dementia. These elements reflect criteria for AIDS as defined by the CDC in 1993.'),
  'D000164': NtO(MH='Acremonium', MS='A mitosporic fungal genus with many reported ascomycetous teleomorphs. Cephalosporin antibiotics are derived from this genus.'),
  'D000165': NtO(MH='Acridine Orange', MS='A cationic cytochemical stain specific for cell nuclei, especially DNA. It is used as a supravital stain and in fluorescence cytochemistry. It may cause mutations in microorganisms.'),
  'D000166': NtO(MH='Acridines', MS='Compounds that include the structure of acridine.'),
  'D000167': NtO(MH='Acriflavine', MS='3,6-Diamino-10-methylacridinium chloride mixt. with 3,6-acridinediamine. Fluorescent dye used as a local antiseptic and also as a biological stain. It intercalates into nucleic acids thereby inhibiting bacterial and viral replication.'),
  'D000168': NtO(MH='Acrocephalosyndactylia', MS='Congenital craniostenosis with syndactyly.'),
  'D000169': NtO(MH='Acrodermatitis', MS='Inflammation involving the skin of the extremities, especially the hands and feet. Several forms are known, some idiopathic and some hereditary. The infantile form is called Gianotti-Crosti syndrome.'),
  'D000170': NtO(MH='Acrodynia', MS='A condition seen primarily in childhood, most often resulting from chronic exposure to MERCURY COMPOUNDS which may result in encephalopathy and POLYNEUROPATHY. Clinical features include pain, swelling and pinkish discoloration of the fingers and toes, weakness in the extremities, extreme irritability, HYPERESTHESIA, and alterations in level of consciousness. (From Menkes, Textbook of Child Neurology, 5th ed, p603)'),
  'D000171': NtO(MH='Acrolein', MS='Unsaturated three-carbon aldehyde.'),
  'D000172': NtO(MH='Acromegaly', MS='A condition caused by prolonged exposure to excessive HUMAN GROWTH HORMONE in adults. It is characterized by bony enlargement of the FACE; lower jaw (PROGNATHISM); hands; FEET; HEAD; and THORAX. The most common etiology is a GROWTH HORMONE-SECRETING PITUITARY ADENOMA. (From Joynt, Clinical Neurology, 1992, Ch36, pp79-80)'),
  'D000173': NtO(MH='Acromioclavicular Joint', MS='The gliding joint formed by the outer extremity of the CLAVICLE and the inner margin of the ACROMION PROCESS of the SCAPULA.'),
  'D000174': NtO(MH='Acromion', MS='The lateral extension of the spine of the SCAPULA and the highest point of the SHOULDER.'),
  'D000175': NtO(MH='Acronine', MS='A pyrano-acridone alkaloid found in RUTACEAE plants.'),
  'D000176': NtO(MH='Acrosin', MS='A trypsin-like enzyme of spermatozoa which is not inhibited by alpha 1 antitrypsin.'),
  'D000177': NtO(MH='Acrosome', MS='The cap-like structure covering the anterior portion of SPERM HEAD. Acrosome, derived from LYSOSOMES, is a membrane-bound organelle that contains the required hydrolytic and proteolytic enzymes necessary for sperm penetration of the egg in FERTILIZATION.'),
  'D000178': NtO(MH='Acrylamides', MS='Colorless, odorless crystals that are used extensively in research laboratories for the preparation of polyacrylamide gels for electrophoresis and in organic synthesis, and polymerization. Some of its polymers are used in sewage and wastewater treatment, permanent press fabrics, and as soil conditioning agents.'),
  'D000179': NtO(MH='Acrylates', MS='Derivatives of acrylic acid (the structural formula CH2=CHCO2H), including its salts and esters.'),
  'D000180': NtO(MH='Acrylic Resins', MS=None),
  'D000181': NtO(MH='Acrylonitrile', MS='A highly poisonous compound used widely in the manufacture of plastics, adhesives and synthetic rubber.'),
  'D000182': NtO(MH='ACTH Syndrome, Ectopic', MS='Symptom complex due to ACTH production by non-pituitary neoplasms.'),
  'D000183': NtO(MH='Actihaemyl', MS='An extract from calf blood containing inorganic salts, amino acids, polypeptides and purines, but no proteins nor antigenic substances or blood group characteristics. Its exact composition is unknown. It has been proposed as a radiation-protective agent.'),
  'D000184': NtO(MH='Acting Out', MS='Expressing unconscious emotional conflicts or feelings, often of hostility or love, through overt behavior.'),
  'D000185': NtO(MH='Actinin', MS='A protein factor that regulates the length of R-actin. It is chemically similar, but immunochemically distinguishable from actin.'),
  'D000186': NtO(MH='Actinium', MS='A trivalent radioactive element and the prototypical member of the actinide family. It has the atomic symbol Ac, and atomic number 89. Its principal isotope is 227 and it decays primarily by beta-emission.'),
  'D000187': NtO(MH='Actinobacillosis', MS='A disease characterized by suppurative and granulomatous lesions in the respiratory tract, upper alimentary tract, skin, kidneys, joints, and other tissues. Actinobacillus lignieresii infects cattle and sheep while A. equuli infects horses and pigs.'),
  'D000188': NtO(MH='Actinobacillus', MS='A genus of PASTEURELLACEAE described as gram-negative, nonsporeforming, nonmotile, facultative anaerobes. Most members are found both as pathogens and commensal organisms in the respiratory, alimentary, and genital tracts of animals.'),
  'D000189': NtO(MH='Actinobacillus Infections', MS='Infections with bacteria of the genus ACTINOBACILLUS.'),
  'D000190': NtO(MH='Actinomyces', MS='A genus of gram-positive, rod-shaped bacteria whose organisms are nonmotile. Filaments that may be present in certain species are either straight or wavy and may have swollen or clubbed heads.'),
  'D000191': NtO(MH='Actinomycetaceae', MS='A family of bacteria including numerous parasitic and pathogenic forms.'),
  'D000192': NtO(MH='Actinomycetales', MS='An order of gram-positive, primarily aerobic BACTERIA that tend to form branching filaments.'),
  'D000193': NtO(MH='Actinomycetales Infections', MS='Infections with bacteria of the order ACTINOMYCETALES.'),
  'D000196': NtO(MH='Actinomycosis', MS='Infections with bacteria of the genus ACTINOMYCES.'),
  'D000197': NtO(MH='Actinomycosis, Cervicofacial', MS='A form of ACTINOMYCOSIS characterized by slow-growing inflammatory lesions of the lymph nodes that drain the mouth (lumpy jaw), reddening of the overlying skin, and intraperitoneal abscesses.'),
  'D000198': NtO(MH='Spectinomycin', MS='An antibiotic produced by Streptomyces spectabilis. It is active against gram-negative bacteria and used for the treatment of GONORRHEA.'),
  'D000199': NtO(MH='Actins', MS='Filamentous proteins that are the main constituent of the thin filaments of muscle fibers. The filaments (known also as filamentous or F-actin) can be dissociated into their globular subunits; each subunit is composed of a single polypeptide 375 amino acids long. This is known as globular or G-actin. In conjunction with MYOSINS, actin is responsible for the contraction and relaxation of muscle.'),
  'D000200': NtO(MH='Action Potentials', MS='Abrupt changes in the membrane potential that sweep along the CELL MEMBRANE of excitable cells in response to excitation stimuli.'),
  'D000201': NtO(MH='Activation Analysis', MS='A method of chemical analysis based on the detection of characteristic radionuclides following a nuclear bombardment. It is also known as radioactivity analysis. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D000202': NtO(MH='Activator Appliances', MS='Loose-fitting removable orthodontic appliances which redirect the pressures of the facial and masticatory muscles onto the teeth and their supporting structures to produce improvements in tooth arrangements and occlusal relations.'),
  'D000203': NtO(MH='Activities of Daily Living', MS='The performance of the basic activities of self care, such as dressing, ambulation, or eating.'),
  'D000204': NtO(MH='Activity Cycles', MS='Bouts of physical irritability or movement alternating with periods of quiescence. It includes biochemical activity and hormonal activity which may be cellular. These cycles are shorter than 24 hours and include sleep-wakefulness cycles and the periodic activation of the digestive system.'),
  'D000205': NtO(MH='Actomyosin', MS='A protein complex of actin and MYOSINS occurring in muscle. It is the essential contractile substance of muscle.'),
  'D000206': NtO(MH='Actuarial Analysis', MS='The application of probability and statistical methods to calculate the risk of occurrence of any event, such as onset of illness, recurrent disease, hospitalization, disability, or death. It may include calculation of the anticipated money costs of such events and of the premiums necessary to provide for payment of such costs.'),
  'D000208': NtO(MH='Acute Disease', MS='Disease having a short and relatively severe course.'),
  'D000209': NtO(MH='Acute-Phase Proteins', MS='Proteins that are secreted into the blood in increased or decreased quantities by hepatocytes in response to trauma, inflammation, or disease. These proteins can serve as inhibitors or mediators of the inflammatory processes. Certain acute-phase proteins have been used to diagnose and follow the course of diseases or as tumor markers.'),
  'D000210': NtO(MH='Acute-Phase Reaction', MS='An early local inflammatory reaction to insult or injury that consists of fever, an increase in inflammatory humoral factors, and an increased synthesis by hepatocytes of a number of proteins or glycoproteins usually found in the plasma.'),
  'D000212': NtO(MH='Acyclovir', MS='A GUANOSINE analog that acts as an antimetabolite. Viruses are especially susceptible. Used especially against herpes.'),
  'D000213': NtO(MH='Acyl Carrier Protein', MS="Consists of a polypeptide chain and 4'-phosphopantetheine linked to a serine residue by a phosphodiester bond. Acyl groups are bound as thiol esters to the pantothenyl group. Acyl carrier protein is involved in every step of fatty acid synthesis by the cytoplasmic system."),
  'D000214': NtO(MH='Acyl Coenzyme A', MS='S-Acyl coenzyme A. Fatty acid coenzyme A derivatives that are involved in the biosynthesis and oxidation of fatty acids as well as in ceramide formation.'),
  'D000215': NtO(MH='Acylation', MS='The addition of an organic acid radical into a molecule.'),
  'D000216': NtO(MH='N-Acylneuraminate Cytidylyltransferase', MS='An enzyme that forms CMP-acylneuraminic acids, which donate the N-acylneuraminic acid residues to the terminal sugar residue of a ganglioside or glycoprotein. EC 2.7.7.43.'),
  'D000217': NtO(MH='Acyltransferases', MS='Enzymes from the transferase class that catalyze the transfer of acyl groups from donor to acceptor, forming either esters or amides. (From Enzyme Nomenclature 1992) EC 2.3.'),
  'D000218': NtO(MH='Adamantane', MS='A tricyclo bridged hydrocarbon.'),
  'D000219': NtO(MH='Adams-Stokes Syndrome', MS='A condition of fainting spells caused by heart block, often an atrioventricular block, that leads to BRADYCARDIA and drop in CARDIAC OUTPUT. When the cardiac output becomes too low, the patient faints (SYNCOPE). In some cases, the syncope attacks are transient and in others cases repetitive and persistent.'),
  'D000220': NtO(MH='Adaptation, Biological', MS='Changes in biological features that help an organism cope with its ENVIRONMENT. These changes include physiological (ADAPTATION, PHYSIOLOGICAL), phenotypic and genetic changes.'),
  'D000221': NtO(MH='Adaptation, Ocular', MS='The adjustment of the eye to variations in the intensity of light. Light adaptation is the adjustment of the eye when the light threshold is increased; DARK ADAPTATION when the light is greatly reduced. (From Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D000222': NtO(MH='Adaptation, Physiological', MS='The non-genetic biological changes of an organism in response to challenges in its ENVIRONMENT.'),
  'D000223': NtO(MH='Adaptation, Psychological', MS='A state of harmony between internal needs and external demands and the processes used in achieving this condition. (From APA Thesaurus of Psychological Index Terms, 8th ed)'),
  'D000224': NtO(MH='Addison Disease', MS='An adrenal disease characterized by the progressive destruction of the ADRENAL CORTEX, resulting in insufficient production of ALDOSTERONE and HYDROCORTISONE. Clinical symptoms include ANOREXIA; NAUSEA; WEIGHT LOSS; MUSCLE WEAKNESS; and HYPERPIGMENTATION of the SKIN due to increase in circulating levels of ACTH precursor hormone which stimulates MELANOCYTES.'),
  'D000225': NtO(MH='Adenine', MS='A purine base and a fundamental unit of ADENINE NUCLEOTIDES.'),
  'D000226': NtO(MH='Mitochondrial ADP, ATP Translocases', MS='A class of nucleotide translocases found abundantly in mitochondria that function as integral components of the inner mitochondrial membrane. They facilitate the exchange of ADP and ATP between the cytosol and the mitochondria, thereby linking the subcellular compartments of ATP production to those of ATP utilization.'),
  'D000227': NtO(MH='Adenine Nucleotides', MS=None),
  'D000228': NtO(MH='Adenine Phosphoribosyltransferase', MS='An enzyme catalyzing the formation of AMP from adenine and phosphoribosylpyrophosphate. It can act as a salvage enzyme for recycling of adenine into nucleic acids. EC 2.4.2.7.'),
  'D000229': NtO(MH='Dependovirus', MS='A genus of the family PARVOVIRIDAE, subfamily PARVOVIRINAE, which are dependent on a coinfection with helper adenoviruses or herpesviruses for their efficient replication. The type species is Adeno-associated virus 2.'),
  'D000230': NtO(MH='Adenocarcinoma', MS='A malignant epithelial tumor with a glandular organization.'),
  'D000231': NtO(MH='Adenocarcinoma, Papillary', MS='An adenocarcinoma containing finger-like processes of vascular connective tissue covered by neoplastic epithelium, projecting into cysts or the cavity of glands or follicles. It occurs most frequently in the ovary and thyroid gland. (Stedman, 25th ed)'),
  'D000232': NtO(MH='Adenofibroma', MS='A benign neoplasm composed of glandular and fibrous tissues, with a relatively large proportion of glands. (Stedman, 25th ed)'),
  'D000233': NtO(MH='Adenoidectomy', MS='Excision of the adenoids. (Dorland, 28th ed)'),
  'D000234': NtO(MH='Adenoids', MS='A collection of lymphoid nodules on the posterior wall and roof of the NASOPHARYNX.'),
  'D000235': NtO(MH='Adenolymphoma', MS='A benign tumor characterized histologically by tall columnar epithelium within a lymphoid tissue stroma. It is usually found in the salivary glands, especially the parotid.'),
  'D000236': NtO(MH='Adenoma', MS='A benign epithelial tumor with a glandular organization.'),
  'D000237': NtO(MH='Adenoma, Basophil', MS='A small tumor of the anterior lobe of the pituitary gland whose cells stain with basic dyes. It may give rise to excessive secretion of ACTH, resulting in CUSHING SYNDROME. (Dorland, 27th ed)'),
  'D000238': NtO(MH='Adenoma, Chromophobe', MS='A benign tumor of the anterior pituitary in which the cells do not stain with acidic or basic dyes.'),
  'D000239': NtO(MH='Adenoma, Acidophil', MS='A benign tumor, usually found in the anterior lobe of the pituitary gland, whose cells stain with acid dyes. Such pituitary tumors may give rise to excessive secretion of growth hormone, resulting in gigantism or acromegaly. A specific type of acidophil adenoma may give rise to nonpuerperal galactorrhea. (Dorland, 27th ed)'),
  'D000241': NtO(MH='Adenosine', MS='A nucleoside that is composed of ADENINE and D-RIBOSE. Adenosine or adenosine derivatives play many important biological roles in addition to being components of DNA and RNA. Adenosine itself is a neurotransmitter.'),
  'D000242': NtO(MH='Cyclic AMP', MS="An adenine nucleotide containing one phosphate group which is esterified to both the 3'- and 5'-positions of the sugar moiety. It is a second messenger and a key intracellular regulator, functioning as a mediator of activity for a number of hormones, including epinephrine, glucagon, and ACTH."),
  'D000243': NtO(MH='Adenosine Deaminase', MS='An enzyme that catalyzes the hydrolysis of ADENOSINE to INOSINE with the elimination of AMMONIA.'),
  'D000244': NtO(MH='Adenosine Diphosphate', MS="Adenosine 5'-(trihydrogen diphosphate). An adenine nucleotide containing two phosphate groups esterified to the sugar moiety at the 5'-position."),
  'D000245': NtO(MH='Adenosine Diphosphate Glucose', MS='Serves as the glycosyl donor for formation of bacterial glycogen, amylose in green algae, and amylopectin in higher plants.'),
  'D000246': NtO(MH='Adenosine Diphosphate Ribose', MS='An ester formed between the aldehydic carbon of RIBOSE and the terminal phosphate of ADENOSINE DIPHOSPHATE. It is produced by the hydrolysis of nicotinamide-adenine dinucleotide (NAD) by a variety of enzymes, some of which transfer an ADP-ribosyl group to target proteins.'),
  'D000247': NtO(MH='Adenosine Diphosphate Sugars', MS='Esters formed between the aldehydic carbon of sugars and the terminal phosphate of adenosine diphosphate.'),
  'D000248': NtO(MH='Adenosine Kinase', MS='An enzyme that catalyzes the formation of ADP plus AMP from adenosine plus ATP. It can serve as a salvage mechanism for returning adenosine to nucleic acids. EC 2.7.1.20.'),
  'D000249': NtO(MH='Adenosine Monophosphate', MS="Adenine nucleotide containing one phosphate group esterified to the sugar moiety in the 2'-, 3'-, or 5'-position."),
  'D000250': NtO(MH='Adenosine Phosphosulfate', MS="5'-Adenylic acid, monoanhydride with sulfuric acid. The initial compound formed by the action of ATP sulfurylase on sulfate ions after sulfate uptake. Synonyms: adenosine sulfatophosphate; APS."),
  'D000251': NtO(MH='Adenosine Triphosphatases', MS='A group of enzymes which catalyze the hydrolysis of ATP. The hydrolysis reaction is usually coupled with another function such as transporting Ca(2+) across a membrane. These enzymes may be dependent on Ca(2+), Mg(2+), anions, H+, or DNA.'),
  'D000252': NtO(MH='Calcium-Transporting ATPases', MS='Cation-transporting proteins that utilize the energy of ATP hydrolysis for the transport of CALCIUM. They differ from CALCIUM CHANNELS which allow calcium to pass through a membrane without the use of energy.'),
  'D000254': NtO(MH='Sodium-Potassium-Exchanging ATPase', MS='An enzyme that catalyzes the active transport system of sodium and potassium ions across the cell wall. Sodium and potassium ions are closely coupled with membrane ATPase which undergoes phosphorylation and dephosphorylation, thereby providing energy for transport of these ions against concentration gradients.'),
  'D000255': NtO(MH='Adenosine Triphosphate', MS='An adenine nucleotide containing three phosphate groups esterified to the sugar moiety. In addition to its crucial roles in metabolism adenosine triphosphate is a neurotransmitter.'),
  'D000256': NtO(MH='Adenoviridae', MS='A family of non-enveloped viruses infecting mammals (MASTADENOVIRUS) and birds (AVIADENOVIRUS) or both (ATADENOVIRUS). Infections may be asymptomatic or result in a variety of diseases.'),
  'D000257': NtO(MH='Adenoviridae Infections', MS='Virus diseases caused by the ADENOVIRIDAE.'),
  'D000258': NtO(MH='Adenovirus Infections, Human', MS='Respiratory and conjunctival infections caused by 33 identified serotypes of human adenoviruses.'),
  'D000259': NtO(MH='Aviadenovirus', MS='A genus of ADENOVIRIDAE that infects birds. The type species is FOWL ADENOVIRUS A.'),
  'D000260': NtO(MH='Adenoviruses, Human', MS='Species of the genus MASTADENOVIRUS, causing a wide range of diseases in humans. Infections are mostly asymptomatic, but can be associated with diseases of the respiratory, ocular, and gastrointestinal systems. Serotypes (named with Arabic numbers) have been grouped into species designated Human adenovirus A-G.'),
  'D000261': NtO(MH='Adenoviruses, Simian', MS='Species of the genus MASTADENOVIRUS associated with respiratory and enteric infections in primate hosts.'),
  'D000262': NtO(MH='Adenylyl Cyclases', MS='Enzymes of the lyase class that catalyze the formation of CYCLIC AMP and pyrophosphate from ATP.'),
  'D000263': NtO(MH='Adenylate Kinase', MS='An enzyme that catalyzes the phosphorylation of AMP to ADP in the presence of ATP or inorganic triphosphate. EC 2.7.4.3.'),
  'D000264': NtO(MH='Adenylosuccinate Lyase', MS="An enzyme that, in the course of purine ribonucleotide biosynthesis, catalyzes the conversion of 5'-phosphoribosyl-4-(N-succinocarboxamide)-5-aminoimidazole to 5'-phosphoribosyl-4-carboxamide-5-aminoimidazole and the conversion of adenylosuccinic acid to AMP. EC 4.3.2.2."),
  'D000265': NtO(MH='Adenylosuccinate Synthase', MS='A carbon-nitrogen ligase. During purine ribonucleotide biosynthesis, this enzyme catalyzes the synthesis of adenylosuccinate from GTP; IMP; and aspartate with the formation of orthophosphate and GDP. EC 6.3.4.4.'),
  'D000266': NtO(MH='Adenylyl Imidodiphosphate', MS="5'-Adenylic acid, monoanhydride with imidodiphosphoric acid. An analog of ATP, in which the oxygen atom bridging the beta to the gamma phosphate is replaced by a nitrogen atom. It is a potent competitive inhibitor of soluble and membrane-bound mitochondrial ATPase and also inhibits ATP-dependent reactions of oxidative phosphorylation."),
  'D000267': NtO(MH='Tissue Adhesions', MS='Pathological processes consisting of the union of the opposing surfaces of a wound.'),
  'D000268': NtO(MH='Adhesiveness', MS='A property of the surface of an object that makes it stick to another surface.'),
  'D000269': NtO(MH='Adhesives', MS='Substances that cause the adherence of two surfaces. They include glues (properly collagen-derived adhesives), mucilages, sticky pastes, gums, resins, or latex.'),
  'D000270': NtO(MH='Adie Syndrome', MS='A syndrome characterized by a TONIC PUPIL that occurs in combination with decreased lower extremity reflexes. The affected pupil will respond more briskly to accommodation than to light (light-near dissociation) and is supersensitive to dilute pilocarpine eye drops, which induce pupillary constriction. Pathologic features include degeneration of the ciliary ganglion and postganglionic parasympathetic fibers that innervate the pupillary constrictor muscle. (From Adams et al., Principles of Neurology, 6th ed, p279)'),
  'D000272': NtO(MH='Adipates', MS='Derivatives of adipic acid. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain a 1,6-carboxy terminated aliphatic structure.'),
  'D000273': NtO(MH='Adipose Tissue', MS='Specialized connective tissue composed of fat cells (ADIPOCYTES). It is the site of stored FATS, usually in the form of TRIGLYCERIDES. In mammals, there are two types of adipose tissue, the WHITE FAT and the BROWN FAT. Their relative distributions vary in different species with most adipose tissue being white.'),
  'D000274': NtO(MH='Adiposis Dolorosa', MS='A rare disease characterized by multiple tumor-like fatty deposits that press on nerves in various sites causing pain and weakness. Often these lipoma-like structures are located on the trunk and limbs but not on the face and hands.'),
  'D000275': NtO(MH='Adjustment Disorders', MS='Maladaptive reactions to identifiable psychosocial stressors occurring within a short time after onset of the stressor. They are manifested by either impairment in social or occupational functioning or by symptoms (depression, anxiety, etc.) that are in excess of a normal and expected reaction to the stressor.'),
  'D000276': NtO(MH='Adjuvants, Immunologic', MS="Substances that augment, stimulate, activate, potentiate, or modulate the immune response at either the cellular or humoral level. The classical agents (Freund's adjuvant, BCG, Corynebacterium parvum, et al.) contain bacterial antigens. Some are endogenous (e.g., histamine, interferon, transfer factor, tuftsin, interleukin-1). Their mode of action is either non-specific, resulting in increased immune responsiveness to a wide variety of antigens, or antigen-specific, i.e., affecting a restricted type of immune response to a narrow group of antigens. The therapeutic efficacy of many biological response modifiers is related to their antigen-specific immunoadjuvanticity."),
  'D000277': NtO(MH='Adjuvants, Pharmaceutic', MS='Agents that aid or increase the action of the principle drug (DRUG SYNERGISM) or that affect the absorption, mechanism of action, metabolism, or excretion of the primary drug (PHARMACOKINETICS) in such a way as to enhance its effects.'),
  'D000278': NtO(MH='Administration, Buccal', MS='Administration of a soluble dosage form between the cheek and gingiva. It may involve direct application of a drug onto the buccal mucosa, as by painting or spraying.'),
  'D000279': NtO(MH='Administration, Cutaneous', MS='The application of suitable drug dosage forms to the skin for either local or systemic effects.'),
  'D000280': NtO(MH='Administration, Inhalation', MS='The administration of drugs by the respiratory route. It includes insufflation into the respiratory tract.'),
  'D000281': NtO(MH='Administration, Intranasal', MS='Delivery of medications through the nasal mucosa.'),
  'D000282': NtO(MH='Administration, Intravaginal', MS='The insertion of drugs into the vagina to treat local infections, neoplasms, or to induce labor. The dosage forms may include medicated pessaries, irrigation fluids, and suppositories.'),
  'D000283': NtO(MH='Administration, Intravesical', MS='The instillation or other administration of drugs into the bladder, usually to treat local disease, including neoplasms.'),
  'D000284': NtO(MH='Administration, Oral', MS='The giving of drugs, chemicals, or other substances by mouth.'),
  'D000285': NtO(MH='Administration, Rectal', MS='The insertion of drugs into the rectum, usually for confused or incompetent patients, like children, infants, and the very old or comatose.'),
  'D000286': NtO(MH='Administration, Sublingual', MS='Administration of a soluble dosage form by placement under the tongue.'),
  'D000287': NtO(MH='Administration, Topical', MS='The application of drug preparations to the surfaces of the body, especially the skin (ADMINISTRATION, CUTANEOUS) or mucous membranes. This method of treatment is used to avoid systemic side effects when high doses are required at a localized area or as an alternative systemic administration route, to avoid hepatic processing for example.'),
  'D000288': NtO(MH='Administrative Personnel', MS='Individuals responsible for the development of policy and supervision of the execution of plans and functional operations.'),
  'D000289': NtO(MH='Admitting Department, Hospital', MS="Hospital department responsible for the flow of patients and the processing of admissions, discharges, transfers, and also most procedures to be carried out in the event of a patient's death."),
  'D000290': NtO(MH='Adnexa Uteri', MS='Appendages of the UTERUS which include the FALLOPIAN TUBES, the OVARY, and the supporting ligaments of the uterus (BROAD LIGAMENT; ROUND LIGAMENT).'),
  'D000291': NtO(MH='Adnexal Diseases', MS='Diseases of the uterine appendages (ADNEXA UTERI) including diseases involving the OVARY, the FALLOPIAN TUBES, and ligaments of the uterus (BROAD LIGAMENT; ROUND LIGAMENT).'),
  'D000292': NtO(MH='Pelvic Inflammatory Disease', MS='A spectrum of inflammation involving the female upper genital tract and the supporting tissues. It is usually caused by an ascending infection of organisms from the endocervix. Infection may be confined to the uterus (ENDOMETRITIS), the FALLOPIAN TUBES; (SALPINGITIS); the ovaries (OOPHORITIS), the supporting ligaments (PARAMETRITIS), or may involve several of the above uterine appendages. Such inflammation can lead to functional impairment and infertility.'),
  'D000293': NtO(MH='Adolescent', MS='A person 13 to 18 years of age.'),
  'D000294': NtO(MH='Adolescent Behavior', MS='Any observable response or action of an adolescent.'),
  'D000295': NtO(MH='Adolescent Medicine', MS='A branch of medicine pertaining to the diagnosis and treatment of diseases occurring during the period of ADOLESCENCE.'),
  'D000296': NtO(MH='Adolescent Psychiatry', MS='The medical science that deals with the origin, diagnosis, prevention, and treatment of mental disorders in individuals 13-18 years.'),
  'D000297': NtO(MH='Psychology, Adolescent', MS='Field of psychology involving the study of normal and abnormal behavior of adolescents. It includes mental processes as well as observable responses.'),
  'D000298': NtO(MH='Adolescent, Hospitalized', MS='Adolescent hospitalized for short term care.'),
  'D000299': NtO(MH='Adolescent, Institutionalized', MS='An adolescent who is receiving long-term in-patient services or who resides in an institutional setting.'),
  'D000300': NtO(MH='Adoption', MS="Voluntary acceptance of a child of other parents to be as one's own child, usually with legal confirmation."),
  'D000301': NtO(MH='Adosterol', MS='A sterol usually substituted with radioactive iodine. It is an adrenal cortex scanning agent with demonstrated high adrenal concentration and superior adrenal imaging.'),
  'D000302': NtO(MH='Adrenal Cortex', MS='The outer layer of the adrenal gland. It is derived from MESODERM and comprised of three zones (outer ZONA GLOMERULOSA, middle ZONA FASCICULATA, and inner ZONA RETICULARIS) with each producing various steroids preferentially, such as ALDOSTERONE; HYDROCORTISONE; DEHYDROEPIANDROSTERONE; and ANDROSTENEDIONE. Adrenal cortex function is regulated by pituitary ADRENOCORTICOTROPIN.'),
  'D000303': NtO(MH='Adrenal Cortex Diseases', MS='Pathological processes of the ADRENAL CORTEX.'),
  'D000304': NtO(MH='Adrenal Cortex Function Tests', MS='Examinations that evaluate and monitor hormone production in the adrenal cortex.'),
  'D000305': NtO(MH='Adrenal Cortex Hormones', MS=None),
  'D000306': NtO(MH='Adrenal Cortex Neoplasms', MS='Tumors or cancers of the ADRENAL CORTEX.'),
  'D000307': NtO(MH='Adrenal Gland Diseases', MS='Pathological processes of the ADRENAL GLANDS.'),
  'D000308': NtO(MH='Adrenocortical Hyperfunction', MS='Excess production of ADRENAL CORTEX HORMONES such as ALDOSTERONE; HYDROCORTISONE; DEHYDROEPIANDROSTERONE; and/or ANDROSTENEDIONE. Hyperadrenal syndromes include CUSHING SYNDROME; HYPERALDOSTERONISM; and VIRILISM.'),
  'D000309': NtO(MH='Adrenal Insufficiency', MS='Conditions in which the production of adrenal CORTICOSTEROIDS falls below the requirement of the body. Adrenal insufficiency can be caused by defects in the ADRENAL GLANDS, the PITUITARY GLAND, or the HYPOTHALAMUS.'),
  'D000310': NtO(MH='Adrenal Gland Neoplasms', MS='Tumors or cancer of the ADRENAL GLANDS.'),
  'D000311': NtO(MH='Adrenal Glands', MS='A pair of glands located at the cranial pole of each of the two KIDNEYS. Each adrenal gland is composed of two distinct endocrine tissues with separate embryonic origins, the ADRENAL CORTEX producing STEROIDS and the ADRENAL MEDULLA producing NEUROTRANSMITTERS.'),
  'D000312': NtO(MH='Adrenal Hyperplasia, Congenital', MS='A group of inherited disorders of the ADRENAL GLANDS, caused by enzyme defects in the synthesis of cortisol (HYDROCORTISONE) and/or ALDOSTERONE leading to accumulation of precursors for ANDROGENS. Depending on the hormone imbalance, congenital adrenal hyperplasia can be classified as salt-wasting, hypertensive, virilizing, or feminizing. Defects in STEROID 21-HYDROXYLASE; STEROID 11-BETA-HYDROXYLASE; STEROID 17-ALPHA-HYDROXYLASE; 3-beta-hydroxysteroid dehydrogenase (3-HYDROXYSTEROID DEHYDROGENASES); TESTOSTERONE 5-ALPHA-REDUCTASE; or steroidogenic acute regulatory protein; among others, underlie these disorders.'),
  'D000313': NtO(MH='Adrenal Medulla', MS='The inner portion of the adrenal gland. Derived from ECTODERM, adrenal medulla consists mainly of CHROMAFFIN CELLS that produces and stores a number of NEUROTRANSMITTERS, mainly adrenaline (EPINEPHRINE) and NOREPINEPHRINE. The activity of the adrenal medulla is regulated by the SYMPATHETIC NERVOUS SYSTEM.'),
  'D000314': NtO(MH='Adrenal Rest Tumor', MS='Neoplasm derived from displaced cells (rest cells) of the primordial ADRENAL GLANDS, generally in patients with CONGENITAL ADRENAL HYPERPLASIA. Adrenal rest tumors have been identified in TESTES; LIVER; and other tissues. They are dependent on ADRENOCORTICOTROPIN for growth and adrenal steroid secretion.'),
  'D000315': NtO(MH='Adrenalectomy', MS='Excision of one or both adrenal glands. (From Dorland, 28th ed)'),
  'D000316': NtO(MH='Adrenergic alpha-Agonists', MS='Drugs that selectively bind to and activate alpha adrenergic receptors.'),
  'D000317': NtO(MH='Adrenergic alpha-Antagonists', MS='Drugs that bind to but do not activate alpha-adrenergic receptors thereby blocking the actions of endogenous or exogenous adrenergic agonists. Adrenergic alpha-antagonists are used in the treatment of hypertension, vasospasm, peripheral vascular disease, shock, and pheochromocytoma.'),
  'D000318': NtO(MH='Adrenergic beta-Agonists', MS='Drugs that selectively bind to and activate beta-adrenergic receptors.'),
  'D000319': NtO(MH='Adrenergic beta-Antagonists', MS='Drugs that bind to but do not activate beta-adrenergic receptors thereby blocking the actions of beta-adrenergic agonists. Adrenergic beta-antagonists are used for treatment of hypertension, cardiac arrhythmias, angina pectoris, glaucoma, migraine headaches, and anxiety.'),
  'D000320': NtO(MH='Adrenergic Fibers', MS='Nerve fibers liberating catecholamines at a synapse after an impulse.'),
  'D000322': NtO(MH='Adrenergic Agonists', MS='Drugs that bind to and activate adrenergic receptors.'),
  'D000323': NtO(MH='Adrenochrome', MS='Pigment obtained by the oxidation of epinephrine.'),
  'D000324': NtO(MH='Adrenocorticotropic Hormone', MS='An anterior pituitary hormone that stimulates the ADRENAL CORTEX and its production of CORTICOSTEROIDS. ACTH is a 39-amino acid polypeptide of which the N-terminal 24-amino acid segment is identical in all species and contains the adrenocorticotrophic activity. Upon further tissue-specific processing, ACTH can yield ALPHA-MSH and corticotrophin-like intermediate lobe peptide (CLIP).'),
  'D000325': NtO(MH='Adrenodoxin', MS='An iron-sulfur protein which serves as an electron carrier in enzymatic steroid hydroxylation reactions in adrenal cortex mitochondria. The electron transport system which catalyzes this reaction consists of adrenodoxin reductase, NADP, adrenodoxin, and cytochrome P-450.'),
  'D000326': NtO(MH='Adrenoleukodystrophy', MS='An X-linked recessive disorder characterized by the accumulation of saturated very long chain fatty acids in the LYSOSOMES of ADRENAL CORTEX and the white matter of CENTRAL NERVOUS SYSTEM. This disease occurs almost exclusively in the males. Clinical features include the childhood onset of ATAXIA; NEUROBEHAVIORAL MANIFESTATIONS; HYPERPIGMENTATION; ADRENAL INSUFFICIENCY; SEIZURES; MUSCLE SPASTICITY; and DEMENTIA. The slowly progressive adult form is called adrenomyeloneuropathy. The defective gene ABCD1 is located at Xq28, and encodes the adrenoleukodystrophy protein (ATP-BINDING CASSETTE TRANSPORTERS).'),
  'D000327': NtO(MH='Adsorption', MS='The adhesion of gases, liquids, or dissolved solids onto a surface. It includes adsorptive phenomena of bacteria and viruses onto surfaces as well. ABSORPTION into the substance may follow but not necessarily.'),
  'D000328': NtO(MH='Adult', MS='A person having attained full growth or maturity. Adults are of 19 through 44 years of age. For a person between 19 and 24 years of age, YOUNG ADULT is available.'),
  'D000329': NtO(MH='Advertising', MS='Works about the act or practice of calling public attention to a product, service, need, etc., especially by paid announcements in newspapers, magazines, on radio, or on television. (Random House Unabridged Dictionary, 2d ed)'),
  'D000330': NtO(MH='Aedes', MS='A genus of mosquitoes (CULICIDAE) frequently found in tropical and subtropical regions. YELLOW FEVER and DENGUE are two of the diseases that can be transmitted by species of this genus.'),
  'D000331': NtO(MH='Aequorin', MS='A photoprotein isolated from the bioluminescent jellyfish Aequorea. It emits visible light by an intramolecular reaction when a trace amount of calcium ion is added. The light-emitting moiety in the bioluminescence reaction is believed to be 2-amino-3-benzyl-5-(p-hydroxyphenyl)pyrazine (AF-350).'),
  'D000332': NtO(MH='Aerobiosis', MS='Life or metabolic reactions occurring in an environment containing oxygen.'),
  'D000333': NtO(MH='Aeromonas', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs singly, in pairs, or in short chains. Its organisms are found in fresh water and sewage and are pathogenic to humans, frogs, and fish.'),
  'D000334': NtO(MH='Aerophagy', MS='Spasmodic swallowing of air.'),
  'D000335': NtO(MH='Aerosol Propellants', MS='Compressed gases or vapors in a container which, upon release of pressure and expansion through a valve, carry another substance from the container. They are used for cosmetics, household cleaners, and so on. Examples are BUTANES; CARBON DIOXIDE; FLUOROCARBONS; NITROGEN; and PROPANE. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D000336': NtO(MH='Aerosols', MS='Colloids with a gaseous dispersing phase and either liquid (fog) or solid (smoke) dispersed phase; used in fumigation or in inhalation therapy; may contain propellant agents.'),
  'D000337': NtO(MH='Aerospace Medicine', MS='That branch of medicine dealing with the studies and effects of flight through the atmosphere or in space upon the human body and with the prevention or cure of physiological or psychological malfunctions arising from these effects. (from NASA Thesaurus)'),
  'D000338': NtO(MH='beta-Aminoethyl Isothiourea', MS='A radiation-protective agent that can inhibit DNA damage by binding to the DNA. It also increases the susceptibility of blood cells to complement-mediated lysis.'),
  'D000339': NtO(MH='Affect', MS='The feeling-tone accompaniment of an idea or mental representation. It is the most direct psychic derivative of instinct and the psychic representative of the various bodily changes by means of which instincts manifest themselves.'),
  'D000341': NtO(MH='Affective Disorders, Psychotic', MS='Disorders in which the essential feature is a severe disturbance in mood (depression, anxiety, elation, and excitement) accompanied by psychotic symptoms such as delusions, hallucinations, gross impairment in reality testing, etc.'),
  'D000342': NtO(MH='Affective Symptoms', MS='Mood or emotional responses dissonant with or inappropriate to the behavior and/or stimulus.'),
  'D000343': NtO(MH='Afferent Loop Syndrome', MS='A complication of gastrojejunostomy (BILLROTH II PROCEDURE), a reconstructive GASTROENTEROSTOMY. It is caused by acute (complete) or chronic (intermittent) obstruction of the afferent jejunal loop due to HERNIA, intussusception, kinking, VOLVULUS, etc. It is characterized by PAIN and VOMITING of BILE-stained fluid.'),
  'D000344': NtO(MH='Afferent Pathways', MS='Nerve structures through which impulses are conducted from a peripheral part toward a nerve center.'),
  'D000345': NtO(MH='Affinity Labels', MS='Analogs of those substrates or compounds which bind naturally at the active sites of proteins, enzymes, antibodies, steroids, or physiological receptors. These analogs form a stable covalent bond at the binding site, thereby acting as inhibitors of the proteins or steroids.'),
  'D000346': NtO(MH='Afghanistan', MS='Country located north and west of Pakistan, and east of Iran. The capital is Kabul.'),
  'D000347': NtO(MH='Afibrinogenemia', MS='A deficiency or absence of FIBRINOGEN in the blood.'),
  'D000348': NtO(MH='Aflatoxins', MS='Furano-furano-benzopyrans that are produced by ASPERGILLUS from STERIGMATOCYSTIN. They are structurally related to COUMARINS and easily oxidized to an epoxide form to become ALKYLATING AGENTS. Members of the group include AFLATOXIN B1; aflatoxin B2, aflatoxin G1, aflatoxin G2; AFLATOXIN M1; and aflatoxin M2.'),
  'D000349': NtO(MH='Africa', MS='The continent south of EUROPE, east of the ATLANTIC OCEAN and west of the INDIAN OCEAN.'),
  'D000350': NtO(MH='Africa, Central', MS='The geographical area of Africa comprising CAMEROON; CENTRAL AFRICAN REPUBLIC; CHAD; CONGO; EQUATORIAL GUINEA; GABON; and DEMOCRATIC REPUBLIC OF THE CONGO.'),
  'D000351': NtO(MH='Africa, Eastern', MS='The geographical area of Africa comprising BURUNDI, DJIBOUTI, ERITREA, ETHIOPIA, KENYA, RWANDA, SOMALIA, SOUTH SUDAN, SUDAN, TANZANIA, and UGANDA.'),
  'D000352': NtO(MH='Africa, Northern', MS='The geographical area of Africa comprising ALGERIA; EGYPT; LIBYA; MOROCCO and TUNISIA. It includes also the vast deserts and oases of the Sahara. It is often referred to as North Africa or the Maghreb.'),
  'D000353': NtO(MH='Africa, Southern', MS='The geographical area of Africa comprising ANGOLA; BOTSWANA; LESOTHO; MALAWI; MOZAMBIQUE; NAMIBIA; SOUTH AFRICA; SWAZILAND; ZAMBIA; and ZIMBABWE.'),
  'D000354': NtO(MH='Africa, Western', MS="The geographical area of Africa comprising BENIN; BURKINA FASO; CAPO VERDE, COTE D'IVOIRE; GAMBIA; GHANA; GUINEA; GUINEA-BISSAU; LIBERIA; MALI; MAURITANIA; NIGER; NIGERIA; SENEGAL; SIERRA LEONE; and TOGO."),
  'D000355': NtO(MH='African Horse Sickness', MS='An insect-borne reovirus infection of horses, mules and donkeys in Africa and the Middle East; characterized by pulmonary edema, cardiac involvement, and edema of the head and neck.'),
  'D000356': NtO(MH='African Horse Sickness Virus', MS='A species of ORBIVIRUS that causes disease in horses, mules, and donkeys. Via its principal vector CULICOIDES, it can also infect dogs, elephants, camels, cattle, sheep, goats, and, in special circumstances, humans.'),
  'D000357': NtO(MH='African Swine Fever', MS='A sometimes fatal ASFIVIRUS infection of pigs, characterized by fever, cough, diarrhea, hemorrhagic lymph nodes, and edema of the gallbladder. It is transmitted between domestic swine by direct contact, ingestion of infected meat, or fomites, or mechanically by biting flies or soft ticks (genus Ornithodoros).'),
  'D000358': NtO(MH='African Swine Fever Virus', MS='The lone species of the genus Asfivirus. It infects domestic and wild pigs, warthogs, and bushpigs. Disease is endemic in domestic swine in many African countries and Sardinia. Soft ticks of the genus Ornithodoros are also infected and act as vectors.'),
  'D000359': NtO(MH='Aftercare', MS='The care and treatment of a convalescent patient, especially that of a patient after surgery.'),
  'D000360': NtO(MH='Afterimage', MS='Continuation of visual impression after cessation of stimuli causing the original image.'),
  'D000361': NtO(MH='Agammaglobulinemia', MS='An immunologic deficiency state characterized by an extremely low level of generally all classes of gamma-globulin in the blood.'),
  'D000362': NtO(MH='Agar', MS='A complex sulfated polymer of galactose units, extracted from Gelidium cartilagineum, Gracilaria confervoides, and related red algae. It is used as a gel in the preparation of solid culture media for microorganisms, as a bulk laxative, in making emulsions, and as a supporting medium for immunodiffusion and immunoelectrophoresis.'),
  'D000363': NtO(MH='Agaricales', MS='An extensive order of basidiomycetous fungi whose fruiting bodies are commonly called mushrooms.'),
  'D000364': NtO(MH='Agaricus', MS='A basidiomycetous fungal genus of the family Agaricaceae, order Agaricales, which includes the field mushroom (A. campestris) and the commercial mushroom (A. bisporus).'),
  'D000365': NtO(MH='Age Determination by Skeleton', MS='Establishment of the age of an individual by examination of their skeletal structure.'),
  'D000366': NtO(MH='Age Determination by Teeth', MS='A means of identifying the age of an animal or human through tooth examination.'),
  'D000367': NtO(MH='Age Factors', MS='Age as a constituent element or influence contributing to the production of a result. It may be applicable to the cause or the effect of a circumstance. It is used with human or animal concepts but should be differentiated from AGING, a physiological process, and TIME FACTORS which refers only to the passage of time.'),
  'D000368': NtO(MH='Aged', MS='A person 65 through 79 years of age. For a person older than 79 years, AGED, 80 AND OVER is available.'),
  'D000369': NtO(MH='Aged, 80 and over', MS='A person 80 years of age and older.'),
  'D000370': NtO(MH='Ageusia', MS='Complete or severe loss of the subjective sense of taste, frequently accompanied by OLFACTION DISORDERS.'),
  'D000371': NtO(MH='Agglutination', MS='The clumping together of suspended material resulting from the action of AGGLUTININS.'),
  'D000372': NtO(MH='Agglutination Tests', MS='Tests that are dependent on the clumping of cells, microorganisms, or particles when mixed with specific antiserum. (From Stedman, 26th ed)'),
  'D000373': NtO(MH='Agglutinins', MS='A substance that makes particles (such as bacteria or cells) stick together to form a clump or a mass.'),
  'D000374': NtO(MH='Aggression', MS='Behavior which may be manifested by destructive and attacking action which is verbal or physical, by covert attitudes of hostility or by obstructionism.'),
  'D000375': NtO(MH='Aging', MS='The gradual irreversible changes in structure and function of an organism that occur as a result of the passage of time.'),
  'D000376': NtO(MH='Agmatine', MS='Decarboxylated arginine, isolated from several plant and animal sources, e.g., pollen, ergot, herring sperm, octopus muscle.'),
  'D000377': NtO(MH='Agnosia', MS='Loss of the ability to comprehend the meaning or recognize the importance of various forms of stimulation that cannot be attributed to impairment of a primary sensory modality. Tactile agnosia is characterized by an inability to perceive the shape and nature of an object by touch alone, despite unimpaired sensation to light touch, position, and other primary sensory modalities.'),
  'D000378': NtO(MH='Agonistic Behavior', MS='Any behavior associated with conflict between two individuals.'),
  'D000379': NtO(MH='Agoraphobia', MS='Obsessive, persistent, intense fear of open places.'),
  'D000380': NtO(MH='Agranulocytosis', MS='A decrease in the number of GRANULOCYTES; (BASOPHILS; EOSINOPHILS; and NEUTROPHILS).'),
  'D000381': NtO(MH='Agraphia', MS='Loss or impairment of the ability to write (letters, syllables, words, or phrases) due to an injury to a specific cerebral area or occasionally due to emotional factors. This condition rarely occurs in isolation, and often accompanies APHASIA. (From Adams et al., Principles of Neurology, 6th ed, p485; APA, Thesaurus of Psychological Index Terms, 1994)'),
  'D000382': NtO(MH="Agricultural Workers' Diseases", MS='Diseases in persons engaged in cultivating and tilling soil, growing plants, harvesting crops, raising livestock, or otherwise engaged in husbandry and farming. The diseases are not restricted to farmers in the sense of those who perform conventional farm chores: the heading applies also to those engaged in the individual activities named above, as in those only gathering harvest or in those only dusting crops.'),
  'D000383': NtO(MH='Agriculture', MS='The science, art or practice of cultivating soil, producing crops, and raising livestock.'),
  'D000385': NtO(MH='Aid to Families with Dependent Children', MS='Financial assistance provided by the government to indigent families with dependent children who meet certain requirements as defined by the Social Security Act, Title IV, in the U.S.'),
  'D000386': NtO(MH='AIDS-Related Complex', MS="A prodromal phase of infection with the human immunodeficiency virus (HIV). Laboratory criteria separating AIDS-related complex (ARC) from AIDS include elevated or hyperactive B-cell humoral immune responses, compared to depressed or normal antibody reactivity in AIDS; follicular or mixed hyperplasia in ARC lymph nodes, leading to lymphocyte degeneration and depletion more typical of AIDS; evolving succession of histopathological lesions such as localization of Kaposi's sarcoma, signaling the transition to the full-blown AIDS."),
  'D000387': NtO(MH='Ainhum', MS='Spontaneous autoamputation of the fourth or fifth toe.'),
  'D000388': NtO(MH='Air', MS="The mixture of gases present in the earth's atmosphere consisting of oxygen, nitrogen, carbon dioxide, and small amounts of other gases."),
  'D000389': NtO(MH='Air Conditioning', MS='The maintenance of certain aspects of the environment within a defined space to facilitate the function of that space; aspects controlled include air temperature and motion, radiant heat level, moisture, and concentration of pollutants such as dust, microorganisms, and gases. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D000390': NtO(MH='Air Ionization', MS='The dissociation of molecules in the air into positive and negative ions under the influence of an electric field.'),
  'D000391': NtO(MH='Air Microbiology', MS='The presence of bacteria, viruses, and fungi in the air. This term is not restricted to pathogenic organisms.'),
  'D000392': NtO(MH='Air Movements', MS='The motion of air currents.'),
  'D000393': NtO(MH='Air Pollutants', MS='Any substance in the air which could, if present in high enough concentration, harm humans, animals, vegetation or materials. Substances include GASES; PARTICULATE MATTER; and volatile ORGANIC CHEMICALS.'),
  'D000395': NtO(MH='Air Pollutants, Occupational', MS='Toxic air-borne matter related to work performed They are usually produced by the specific nature of the occupation.'),
  'D000396': NtO(MH='Air Pollutants, Radioactive', MS='Pollutants present in air which exhibit radioactivity.'),
  'D000397': NtO(MH='Air Pollution', MS='The presence of contaminants or pollutant substances in the air (AIR POLLUTANTS) that interfere with human health or welfare, or produce other harmful environmental effects. The substances may include GASES; PARTICULATE MATTER; or volatile ORGANIC CHEMICALS.'),
  'D000398': NtO(MH='Air Pollution, Radioactive', MS='Contamination of air with radioactive substances.'),
  'D000399': NtO(MH='Air Pressure', MS='The force per unit area that the air exerts on any surface in contact with it. Primarily used for articles pertaining to air pressure within a closed environment.'),
  'D000400': NtO(MH='Air Sacs', MS='Thin-walled sacs or spaces which function as a part of the respiratory system in birds, fishes, insects, and mammals.'),
  'D000401': NtO(MH='Aircraft', MS='A weight-carrying structure for navigation of the air that is supported either by its own buoyancy or by the dynamic action of the air against its surfaces. (Webster, 1973)'),
  'D000402': NtO(MH='Airway Obstruction', MS='Any hindrance to the passage of air into and out of the lungs.'),
  'D000403': NtO(MH='Airway Resistance', MS='Physiologically, the opposition to flow of air caused by the forces of friction. As a part of pulmonary function testing, it is the ratio of driving pressure to the rate of air flow.'),
  'D000404': NtO(MH='Ajmaline', MS='An alkaloid found in the root of RAUWOLFIA SERPENTINA, among other plant sources. It is a class Ia antiarrhythmic agent that apparently acts by changing the shape and threshold of cardiac action potentials.'),
  'D000405': NtO(MH='Akinetic Mutism', MS='A syndrome characterized by a silent and inert state without voluntary motor activity despite preserved sensorimotor pathways and vigilance. Bilateral FRONTAL LOBE dysfunction involving the anterior cingulate gyrus and related brain injuries are associated with this condition. This may result in impaired abilities to communicate and initiate motor activities. (From Adams et al., Principles of Neurology, 6th ed, p348; Fortschr Neurol Psychiatr 1995 Feb;63(2):59-67)'),
  'D000406': NtO(MH='AKR murine leukemia virus', MS='A strain of Murine leukemia virus (LEUKEMIA VIRUS, MURINE) isolated from spontaneous leukemia in AKR strain mice.'),
  'D000407': NtO(MH='Alabama', MS='A state in the southeastern region of the United States. The capital is Montgomery.'),
  'D000408': NtO(MH='Alamethicin', MS='A cyclic nonadecapeptide antibiotic that can act as an ionophore and is produced by strains of Trichoderma viride. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D000409': NtO(MH='Alanine', MS='A non-essential amino acid that occurs in high levels in its free state in plasma. It is produced from pyruvate by transamination. It is involved in sugar and acid metabolism, increases IMMUNITY, and provides energy for muscle tissue, BRAIN, and the CENTRAL NERVOUS SYSTEM.'),
  'D000410': NtO(MH='Alanine Transaminase', MS='An enzyme that catalyzes the conversion of L-alanine and 2-oxoglutarate to pyruvate and L-glutamate. (From Enzyme Nomenclature, 1992) EC 2.6.1.2.'),
  'D000411': NtO(MH='Alanine Racemase', MS='A pyridoxal-phosphate protein that reversibly catalyzes the conversion of L-alanine to D-alanine. EC 5.1.1.1.'),
  'D000412': NtO(MH='Alanine-tRNA Ligase', MS='An enzyme that activates alanine with its specific transfer RNA. EC 6.1.1.7.'),
  'D000413': NtO(MH='Alaska', MS='State of the UNITED STATES OF AMERICA bounded on the east by Canada and on the north, west, and south by the Pacific Ocean and the Bering Sea.'),
  'D000415': NtO(MH='Albania', MS='A country in Europe, on the Adriatic Sea. The capital is Tirana'),
  'D000416': NtO(MH='Alberta', MS="A province of western Canada, lying between the provinces of British Columbia and Saskatchewan. Its capital is Edmonton. It was named in honor of Princess Louise Caroline Alberta, the fourth daughter of Queen Victoria. (From Webster's New Geographical Dictionary, 1988, p26 & Room, Brewer's Dictionary of Names, 1992, p12)"),
  'D000417': NtO(MH='Albinism', MS='General term for a number of inherited defects of amino acid metabolism in which there is a deficiency or absence of pigment in the eyes, skin, or hair.'),
  'D000418': NtO(MH='Albumins', MS='Water-soluble proteins found in egg whites, blood, lymph, and other tissues and fluids. They coagulate upon heating.'),
  'D000419': NtO(MH='Albuminuria', MS='The presence of albumin in the urine, an indicator of KIDNEY DISEASES.'),
  'D000420': NtO(MH='Albuterol', MS='A short-acting beta-2 adrenergic agonist that is primarily used as a bronchodilator agent to treat ASTHMA. Albuterol is prepared as a racemic mixture of R(-) and S(+) stereoisomers. The stereospecific preparation of R(-) isomer of albuterol is referred to as levalbuterol.'),
  'D000421': NtO(MH='Alcaligenes', MS='A genus of gram-negative, aerobic, motile bacteria that occur in water and soil. Some are common inhabitants of the intestinal tract of vertebrates. These bacteria occasionally cause opportunistic infections in humans.'),
  'D000422': NtO(MH='Alchemy', MS='Form of chemistry concerned with discovering methods for transmuting base metals into gold and with finding an elixir for eternal youth.'),
  'D000423': NtO(MH='Alcian Blue', MS='A copper-containing dye used as a gelling agent for lubricants, for staining of bacteria and for the dyeing of histiocytes and fibroblasts in vivo.'),
  'D000425': NtO(MH='Alcohol Amnestic Disorder', MS='A mental disorder associated with chronic ethanol abuse (ALCOHOLISM) and nutritional deficiencies characterized by short term memory loss, confabulations, and disturbances of attention. (Adams et al., Principles of Neurology, 6th ed, p1139)'),
  'D000426': NtO(MH='Alcohol Dehydrogenase', MS='A zinc-containing enzyme which oxidizes primary and secondary alcohols or hemiacetals in the presence of NAD. In alcoholic fermentation, it catalyzes the final step of reducing an aldehyde to an alcohol in the presence of NADH and hydrogen.'),
  'D000427': NtO(MH='Alcohol Deterrents', MS='Substances interfering with the metabolism of ethyl alcohol, causing unpleasant side effects thought to discourage the drinking of alcoholic beverages. Alcohol deterrents are used in the treatment of alcoholism.'),
  'D000428': NtO(MH='Alcohol Drinking', MS='Behaviors associated with the ingesting of ALCOHOLIC BEVERAGES, including social drinking.'),
  'D000429': NtO(MH='Alcohol Oxidoreductases', MS='A subclass of enzymes which includes all dehydrogenases acting on primary and secondary alcohols as well as hemiacetals. They are further classified according to the acceptor which can be NAD+ or NADP+ (subclass 1.1.1), cytochrome (1.1.2), oxygen (1.1.3), quinone (1.1.5), or another acceptor (1.1.99).'),
  'D000430': NtO(MH='Alcohol Withdrawal Delirium', MS='An acute organic mental disorder induced by cessation or reduction in chronic alcohol consumption. Clinical characteristics include CONFUSION; DELUSIONS; vivid HALLUCINATIONS; TREMOR; agitation; insomnia; and signs of autonomic hyperactivity (e.g., elevated blood pressure and heart rate, dilated pupils, and diaphoresis). This condition may occasionally be fatal. It was formerly called delirium tremens. (From Adams et al., Principles of Neurology, 6th ed, p1175)'),
  'D000431': NtO(MH='Ethanol', MS='A clear, colorless liquid rapidly absorbed from the gastrointestinal tract and distributed throughout the body. It has bactericidal activity and is used often as a topical disinfectant. It is widely used as a solvent and preservative in pharmaceutical preparations as well as serving as the primary ingredient in ALCOHOLIC BEVERAGES.'),
  'D000432': NtO(MH='Methanol', MS='A colorless, flammable liquid used in the manufacture of FORMALDEHYDE and ACETIC ACID, in chemical synthesis, antifreeze, and as a solvent. Ingestion of methanol is toxic and may cause blindness.'),
  'D000433': NtO(MH='1-Propanol', MS='A colorless liquid made by oxidation of aliphatic hydrocarbons that is used as a solvent and chemical intermediate.'),
  'D000434': NtO(MH='Alcoholic Beverages', MS='Drinkable liquids containing ETHANOL.'),
  'D000435': NtO(MH='Alcoholic Intoxication', MS='An acute brain syndrome which results from the excessive ingestion of ETHANOL or ALCOHOLIC BEVERAGES.'),
  'D000436': NtO(MH='Alcoholics Anonymous', MS='An organization of self-proclaimed alcoholics who meet frequently to reinforce their practice of abstinence.'),
  'D000437': NtO(MH='Alcoholism', MS='A primary, chronic disease with genetic, psychosocial, and environmental factors influencing its development and manifestations. The disease is often progressive and fatal. It is characterized by impaired control over drinking, preoccupation with the drug alcohol, use of alcohol despite adverse consequences, and distortions in thinking, most notably denial. Each of these symptoms may be continuous or periodic. (Morse & Flavin for the Joint Commission of the National Council on Alcoholism and Drug Dependence and the American Society of Addiction Medicine to Study the Definition and Criteria for the Diagnosis of Alcoholism: in JAMA 1992;268:1012-4)'),
  'D000438': NtO(MH='Alcohols', MS="Alkyl compounds containing a hydroxyl group. They are classified according to relation of the carbon atom: primary alcohols, R-CH2OH; secondary alcohols, R2-CHOH; tertiary alcohols, R3-COH. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000439': NtO(MH='Pentanols', MS='Isomeric forms and derivatives of pentanol (C5H11OH).'),
  'D000440': NtO(MH='Butanols', MS='Isomeric forms and derivatives of butanol (C4H9OH).'),
  'D000441': NtO(MH='Hexanols', MS='Isomeric forms and derivatives of hexanol (C6H11OH).'),
  'D000442': NtO(MH='Octanols', MS='Isomeric forms and derivatives of octanol (C8H17OH).'),
  'D000443': NtO(MH='Alcuronium', MS='A non-depolarizing skeletal muscle relaxant similar to TUBOCURARINE. It is used as an anesthesia adjuvant.'),
  'D000444': NtO(MH='Aldehyde Dehydrogenase', MS='An enzyme that oxidizes an aldehyde in the presence of NAD+ and water to an acid and NADH. This enzyme was formerly classified as EC 1.1.1.70.'),
  'D000445': NtO(MH='Aldehyde Oxidoreductases', MS='Oxidoreductases that are specific for ALDEHYDES.'),
  'D000446': NtO(MH='Aldehyde-Lyases', MS='Enzymes that catalyze a reverse aldol condensation. A molecule containing a hydroxyl group and a carbonyl group is cleaved at a C-C bond to produce two smaller molecules (ALDEHYDES or KETONES). EC 4.1.2.'),
  'D000447': NtO(MH='Aldehydes', MS='Organic compounds containing a carbonyl group in the form -CHO.'),
  'D000448': NtO(MH='Aldicarb', MS='Carbamate derivative used as an insecticide, acaricide, and nematocide.'),
  'D000449': NtO(MH='Aldehyde Reductase', MS='An enzyme that catalyzes reversibly the oxidation of an aldose to an alditol. It possesses broad specificity for many aldoses. EC 1.1.1.21.'),
  'D000450': NtO(MH='Aldosterone', MS='A hormone secreted by the ADRENAL CORTEX that regulates electrolyte and water balance by increasing the renal retention of sodium and the excretion of potassium.'),
  'D000451': NtO(MH='Mineralocorticoid Receptor Antagonists', MS='Drugs that bind to and block the activation of MINERALOCORTICOID RECEPTORS by MINERALOCORTICOIDS such as ALDOSTERONE.'),
  'D000452': NtO(MH='Aldrin', MS='A highly poisonous substance that was formerly used as an insecticide. The manufacture and use has been discontinued in the U.S. (From Merck Index, 11th ed)'),
  'D000453': NtO(MH='Aleutian Mink Disease', MS='A slow progressive disease of mink caused by the ALEUTIAN MINK DISEASE VIRUS. It is characterized by poor reproduction, weight loss, autoimmunity, hypergammaglobulinemia, increased susceptibility to bacterial infections, and death from renal failure. The disease occurs in all color types, but mink which are homozygous recessive for the Aleutian gene for light coat color are particularly susceptible.'),
  'D000454': NtO(MH='Aleutian Mink Disease Virus', MS='A species of PARVOVIRUS that causes a disease in mink, mainly those homozygous for the recessive Aleutian gene which determines a desirable coat color.'),
  'D000455': NtO(MH='Medicago sativa', MS='A plant species of the family FABACEAE widely cultivated for ANIMAL FEED.'),
  'D000458': NtO(MH='Cyanobacteria', MS='A phylum of oxygenic photosynthetic bacteria comprised of unicellular to multicellular bacteria possessing CHLOROPHYLL a and carrying out oxygenic PHOTOSYNTHESIS. Cyanobacteria are the only known organisms capable of fixing both CARBON DIOXIDE (in the presence of light) and NITROGEN. Cell morphology can include nitrogen-fixing heterocysts and/or resting cells called akinetes. Formerly called blue-green algae, cyanobacteria were traditionally treated as ALGAE.'),
  'D000459': NtO(MH='Phaeophyta', MS='A division of predominantly marine EUKARYOTA, commonly known as brown algae, having CHROMATOPHORES containing carotenoid PIGMENTS, BIOLOGICAL. ALGINATES and phlorotannins occur widely in all major orders. They are considered the most highly evolved algae because of their well-developed multicellular organization and structural complexity.'),
  'D000460': NtO(MH='Chlorophyta', MS='A phylum of photosynthetic EUKARYOTA bearing double membrane-bound plastids containing chlorophyll a and b. They comprise the classical green algae, and represent over 7000 species that live in a variety of primarily aquatic habitats. Only about ten percent are marine species, most live in freshwater.'),
  'D000461': NtO(MH='Rhodophyta', MS='Plants of the division Rhodophyta, commonly known as red algae, in which the red pigment (PHYCOERYTHRIN) predominates. However, if this pigment is destroyed, the algae can appear purple, brown, green, or yellow. Two important substances found in the cell walls of red algae are AGAR and CARRAGEENAN. Some rhodophyta are notable SEAWEED (macroalgae).'),
  'D000462': NtO(MH='Algeria', MS='A country in northern Africa bordering the Mediterranean Sea, between MOROCCO and TUNISIA. Its capital is Algiers.'),
  'D000463': NtO(MH='Algestone Acetophenide', MS='A progesterone that has been used in ESTRUS SYNCHRONIZATION and has been evaluated as an injectable contraceptive in combination with estradiol enanthate. It is also applied topically as an anti-inflammatory and in the treatment of ACNE.'),
  'D000464': NtO(MH='Alginates', MS='Salts and esters of ALGINIC ACID that are used as HYDROGELS; DENTAL IMPRESSION MATERIALS, and as absorbent materials for surgical dressings (BANDAGES, HYDROCOLLOID). They are also used to manufacture MICROSPHERES and NANOPARTICLES for DIAGNOSTIC REAGENT KITS and DRUG DELIVERY SYSTEMS.'),
  'D000465': NtO(MH='Algorithms', MS='A procedure consisting of a sequence of algebraic formulas and/or logical steps to calculate or determine a given task.'),
  'D000466': NtO(MH='Alkadienes', MS='Acyclic branched or unbranched hydrocarbons having two carbon-carbon double bonds.'),
  'D000468': NtO(MH='Alkalies', MS="Usually a hydroxide of lithium, sodium, potassium, rubidium or cesium, but also the carbonates of these metals, ammonia, and the amines. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000469': NtO(MH='Alkaline Phosphatase', MS='An enzyme that catalyzes the conversion of an orthophosphoric monoester and water to an alcohol and orthophosphate. EC 3.1.3.1.'),
  'D000470': NtO(MH='Alkaloids', MS="Organic nitrogenous bases. Many alkaloids of medical importance occur in the animal and vegetable kingdoms, and some have been synthesized. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000471': NtO(MH='Alkalosis', MS='A pathological condition that removes acid or adds base to the body fluids.'),
  'D000472': NtO(MH='Alkalosis, Respiratory', MS='A state due to excess loss of carbon dioxide from the body. (Dorland, 27th ed)'),
  'D000473': NtO(MH='Alkanes', MS="The generic name for the group of aliphatic hydrocarbons Cn-H2n+2. They are denoted by the suffix -ane. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000474': NtO(MH='Alkaptonuria', MS='An inborn error of amino acid metabolism resulting from a defect in the enzyme HOMOGENTISATE 1,2-DIOXYGENASE, an enzyme involved in the breakdown of PHENYLALANINE and TYROSINE. It is characterized by accumulation of HOMOGENTISIC ACID in the urine, OCHRONOSIS in various tissues, and ARTHRITIS.'),
  'D000475': NtO(MH='Alkenes', MS="Unsaturated hydrocarbons of the type Cn-H2n, indicated by the suffix -ene. (Grant & Hackh's Chemical Dictionary, 5th ed, p408)"),
  'D000476': NtO(MH='Alkanesulfonates', MS='Organic esters or salts of sulfonic acid derivatives containing an aliphatic hydrocarbon radical.'),
  'D000477': NtO(MH='Alkylating Agents', MS='Highly reactive chemicals that introduce alkyl radicals into biologically active molecules and thereby prevent their proper functioning. Many are used as antineoplastic agents, but most are very toxic, with carcinogenic, mutagenic, teratogenic, and immunosuppressant actions. They have also been used as components in poison gases.'),
  'D000478': NtO(MH='Alkylation', MS='The covalent bonding of an alkyl group to an organic compound. It can occur by a simple addition reaction or by substitution of another functional group.'),
  'D000479': NtO(MH='Alkylmercury Compounds', MS='Organic mercury compounds in which the mercury is attached to an alkyl group.'),
  'D000480': NtO(MH='Alkynes', MS='Hydrocarbons with at least one triple bond in the linear portion, of the general formula Cn-H2n-2.'),
  'D000481': NtO(MH='Allantoin', MS='A urea hydantoin that is found in URINE and PLANTS and is used in dermatological preparations.'),
  'D000482': NtO(MH='Allantois', MS='An extra-embryonic membranous sac derived from the YOLK SAC of REPTILES; BIRDS; and MAMMALS. It lies between two other extra-embryonic membranes, the AMNION and the CHORION. The allantois serves to store urinary wastes and mediate exchange of gas and nutrients for the developing embryo.'),
  'D000483': NtO(MH='Alleles', MS='Variant forms of the same gene, occupying the same locus on homologous CHROMOSOMES, and governing the variants in production of the same gene product.'),
  'D000485': NtO(MH='Allergens', MS='Antigen-type substances that produce immediate hypersensitivity (HYPERSENSITIVITY, IMMEDIATE).'),
  'D000486': NtO(MH='Allergy and Immunology', MS='A medical specialty concerned with the hypersensitivity of the individual to foreign substances and protection from the resultant infection or disorder.'),
  'D000487': NtO(MH='Allethrins', MS='Synthetic analogs of the naturally occurring insecticides cinerin, jasmolin, and pyrethrin. (From Merck Index, 11th ed)'),
  'D000488': NtO(MH='Allied Health Personnel', MS='Health care workers specially trained and licensed to assist and support the work of health professionals. Often used synonymously with paramedical personnel, the term generally refers to all health care workers who perform tasks which must otherwise be performed by a physician or other health professional.'),
  'D000489': NtO(MH='Alligators and Crocodiles', MS='Large, long-tailed reptiles, including caimans, of the order Loricata.'),
  'D000490': NtO(MH='Allium', MS='A genus of the plant family Amaryllidaceae. Many produce pungent, often bacteriostatic and physiologically active compounds and are used as VEGETABLES; CONDIMENTS; and medicament, the latter in traditional medicine.'),
  'D000492': NtO(MH='Allophanate Hydrolase', MS='An enzyme that catalyzes the hydrolysis of allophanic acid to two molecules of ammonia plus two molecules of "active carbon dioxide". EC 3.5.1.54.'),
  'D000493': NtO(MH='Allopurinol', MS='A XANTHINE OXIDASE inhibitor that decreases URIC ACID production. It also acts as an antimetabolite on some simpler organisms.'),
  'D000494': NtO(MH='Allosteric Regulation', MS='The modification of the reactivity of ENZYMES by the binding of effectors to sites (ALLOSTERIC SITES) on the enzymes other than the substrate BINDING SITES.'),
  'D000495': NtO(MH='Allosteric Site', MS='A site on an enzyme which upon binding of a modulator, causes the enzyme to undergo a conformational change that may alter its catalytic or binding properties.'),
  'D000496': NtO(MH='Alloxan', MS='Acidic compound formed by oxidation of URIC ACID. It is isolated as an efflorescent crystalline hydrate.'),
  'D000497': NtO(MH='Alloys', MS='A mixture of metallic elements or compounds with other metallic or metalloid elements in varying proportions.'),
  'D000498': NtO(MH='Allyl Compounds', MS='Alkenes with the general formula H2C=CH-CH2-R.'),
  'D000499': NtO(MH='Allylamine', MS='Possesses an unusual and selective cytotoxicity for VASCULAR SMOOTH MUSCLE cells in dogs and rats. Useful for experiments dealing with arterial injury, myocardial fibrosis or cardiac decompensation.'),
  'D000500': NtO(MH='Allylestrenol', MS='A synthetic steroid with progestational activity.'),
  'D000501': NtO(MH='Allylglycine', MS='An inhibitor of glutamate decarboxylase and an antagonist of GAMMA-AMINOBUTYRIC ACID. It is used to induce convulsions in experimental animals.'),
  'D000502': NtO(MH='Allylisopropylacetamide', MS='An allylic compound that acts as a suicide inactivator of CYTOCHROME P450 by covalently binding to its heme moiety or surrounding protein.'),
  'D000503': NtO(MH='Almanacs as Topic', MS='Works on publications, usually annual, containing a calendar for the coming year, the times of such events and phenomena as anniversaries, sunrises, sunsets, phases of the moon, tides, meteorological, and other statistical information and related topics or useful and interesting facts relating to countries of the world, sports, entertainment, population groups, etc.'),
  'D000504': NtO(MH='Aloe', MS='A plant genus of the family Xanthorrhoeaceae which is used medicinally. It contains anthraquinone glycosides such as aloin-emodin or aloe-emodin (EMODIN).'),
  'D000505': NtO(MH='Alopecia', MS='Absence of hair from areas where it is normally present.'),
  'D000506': NtO(MH='Alopecia Areata', MS='Loss of scalp and body hair involving microscopically inflammatory patchy areas.'),
  'D000507': NtO(MH='Mucinosis, Follicular', MS='A disease of the pilosebaceous unit, presenting clinically as grouped follicular papules or plaques with associated hair loss. It is caused by mucinous infiltration of tissues, and usually involving the scalp, face, and neck. It may be primary (idiopathic) or secondary to mycosis fungoides or reticulosis.'),
  'D000508': NtO(MH='Alouatta', MS='A genus of the subfamily ALOUATTINAE, family ATELIDAE, inhabiting the forests of Central and South America. Howlers travel in groups and define their territories by howling accompanied by vigorously shaking and breaking branches.'),
  'D000509': NtO(MH='alpha-Fetoproteins', MS='The first alpha-globulins to appear in mammalian sera during FETAL DEVELOPMENT and the dominant serum proteins in early embryonic life.'),
  'D000510': NtO(MH='Alpha-Globulins', MS='Serum proteins that have the most rapid migration during ELECTROPHORESIS. This subgroup of globulins is divided into faster and slower alpha(1)- and alpha(2)-globulins.'),
  'D000511': NtO(MH='alpha-Macroglobulins', MS='Glycoproteins with a molecular weight of approximately 620,000 to 680,000. Precipitation by electrophoresis is in the alpha region. They include alpha 1-macroglobulins and alpha 2-macroglobulins. These proteins exhibit trypsin-, chymotrypsin-, thrombin-, and plasmin-binding activity and function as hormonal transporters.'),
  'D000512': NtO(MH='Alpha Particles', MS='Positively charged particles composed of two protons and two NEUTRONS, i.e. equivalent to HELIUM nuclei, which are emitted during disintegration of heavy ISOTOPES. Alpha rays have very strong ionizing power, but weak penetrability.'),
  'D000513': NtO(MH='Alpha Rhythm', MS='Brain waves characterized by a relatively high voltage or amplitude and a frequency of 8-13 Hz. They constitute the majority of waves recorded by EEG registering the activity of the parietal and occipital lobes when the individual is awake, but relaxed with the eyes closed.'),
  'D000514': NtO(MH='alpha 1-Antichymotrypsin', MS='Glycoprotein found in alpha(1)-globulin region in human serum. It inhibits chymotrypsin-like proteinases in vivo and has cytotoxic killer-cell activity in vitro. The protein also has a role as an acute-phase protein and is active in the control of immunologic and inflammatory processes, and as a tumor marker. It is a member of the serpin superfamily.'),
  'D000515': NtO(MH='alpha 1-Antitrypsin', MS='Plasma glycoprotein member of the serpin superfamily which inhibits TRYPSIN; NEUTROPHIL ELASTASE; and other PROTEOLYTIC ENZYMES.'),
  'D000516': NtO(MH='alpha-Amylases', MS='Enzymes that catalyze the endohydrolysis of 1,4-alpha-glycosidic linkages in STARCH; GLYCOGEN; and related POLYSACCHARIDES and OLIGOSACCHARIDES containing 3 or more 1,4-alpha-linked D-glucose units.'),
  'D000517': NtO(MH='alpha-Chlorohydrin', MS='A chlorinated PROPANEDIOL with antifertility activity in males used as a chemosterilant in rodents.'),
  'D000518': NtO(MH='Eflornithine', MS='An inhibitor of ORNITHINE DECARBOXYLASE, the rate limiting enzyme of the polyamine biosynthetic pathway.'),
  'D000519': NtO(MH='alpha-Galactosidase', MS='An enzyme that catalyzes the hydrolysis of terminal, non-reducing alpha-D-galactose residues in alpha-galactosides including galactose oligosaccharides, galactomannans, and galactolipids.'),
  'D000520': NtO(MH='alpha-Glucosidases', MS='Enzymes that catalyze the exohydrolysis of 1,4-alpha-glucosidic linkages with release of alpha-glucose. Deficiency of alpha-1,4-glucosidase may cause GLYCOGEN STORAGE DISEASE TYPE II.'),
  'D000521': NtO(MH='alpha-MSH', MS='A 13-amino acid peptide derived from proteolytic cleavage of ADRENOCORTICOTROPIC HORMONE, the N-terminal segment of ACTH. ACTH (1-13) is amidated at the C-terminal to form ACTH (1-13)NH2 which in turn is acetylated to form alpha-MSH in the secretory granules. Alpha-MSH stimulates the synthesis and distribution of MELANIN in MELANOCYTES in mammals and MELANOPHORES in lower vertebrates.'),
  'D000522': NtO(MH='Alphaprodine', MS='An opioid analgesic chemically related to and with an action resembling that of MEPERIDINE, but more rapid in onset and of shorter duration. It has been used in obstetrics, as pre-operative medication, for minor surgical procedures, and for dental procedures. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1067)'),
  'D000523': NtO(MH='Algestone', MS='A synthetic progestational dihydroxy derivative of PROGESTERONE. Its acetonide possesses anti-inflammatory properties.'),
  'D000524': NtO(MH='Alphavirus', MS='A genus of TOGAVIRIDAE, also known as Group A arboviruses, serologically related to each other but not to other Togaviridae. The viruses are transmitted by mosquitoes. The type species is the SINDBIS VIRUS.'),
  'D000525': NtO(MH='Alprazolam', MS='A triazolobenzodiazepine compound with antianxiety and sedative-hypnotic actions, that is efficacious in the treatment of PANIC DISORDERS, with or without AGORAPHOBIA, and in generalized ANXIETY DISORDERS. (From AMA Drug Evaluations Annual, 1994, p238)'),
  'D000526': NtO(MH='Alprenolol', MS='One of the ADRENERGIC BETA-ANTAGONISTS used as an antihypertensive, anti-anginal, and anti-arrhythmic agent.'),
  'D000527': NtO(MH='Alprostadil', MS='A potent vasodilator agent that increases peripheral blood flow.'),
  'D000528': NtO(MH='Alternaria', MS='A mitosporic Loculoascomycetes fungal genus including several plant pathogens and at least one species which produces a highly phytotoxic antibiotic. Its teleomorph is Lewia.'),
  'D000529': NtO(MH='Complementary Therapies', MS='Therapeutic practices which are not currently considered an integral part of conventional allopathic medical practice. They may lack biomedical explanations but as they become better researched some (PHYSICAL THERAPY MODALITIES; DIET; ACUPUNCTURE) become widely accepted whereas others (humors, radium therapy) quietly fade away, yet are important historical footnotes. Therapies are termed as Complementary when used in addition to conventional treatments and as Alternative when used instead of conventional treatment.'),
  'D000530': NtO(MH='Alfaxalone Alfadolone Mixture', MS='A 3:1 mixture of alfaxalone with alfadolone acetate that previously had been used as a general anesthetic. It is no longer actively marketed. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1445)'),
  'D000531': NtO(MH='Altitude', MS='A vertical distance measured from a known level on the surface of a planet or other celestial body.'),
  'D000532': NtO(MH='Altitude Sickness', MS='Multiple symptoms associated with reduced oxygen at high ALTITUDE.'),
  'D000533': NtO(MH='Altruism', MS='Consideration and concern for others, as opposed to self-love or egoism, which can be a motivating influence.'),
  'D000534': NtO(MH='Alum Compounds', MS='Aluminum metal sulfate compounds used medically as astringents and for many industrial purposes. They are used in veterinary medicine for the treatment of ulcerative stomatitis, leukorrhea, conjunctivitis, pharyngitis, metritis, and minor wounds.'),
  'D000535': NtO(MH='Aluminum', MS='A metallic element that has the atomic number 13, atomic symbol Al, and atomic weight 26.98.'),
  'D000536': NtO(MH='Aluminum Hydroxide', MS='A compound with many biomedical applications: as a gastric antacid, an antiperspirant, in dentifrices, as an emulsifier, as an adjuvant in bacterins and vaccines, in water purification, etc.'),
  'D000537': NtO(MH='Aluminum Oxide', MS='An oxide of aluminum, occurring in nature as various minerals such as bauxite, corundum, etc. It is used as an adsorbent, desiccating agent, and catalyst, and in the manufacture of dental cements and refractories.'),
  'D000538': NtO(MH='Aluminum Silicates', MS="Any of the numerous types of clay which contain varying proportions of Al2O3 and SiO2. They are made synthetically by heating aluminum fluoride at 1000-2000 degrees C with silica and water vapor. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D000539': NtO(MH='Alveolar Process', MS='The thickest and spongiest part of the maxilla and mandible hollowed out into deep cavities for the teeth.'),
  'D000540': NtO(MH='Alveolar Ridge Augmentation', MS='Preprosthetic surgery involving rib, cartilage, or iliac crest bone grafts, usually autologous, or synthetic implants for rebuilding the alveolar ridge.'),
  'D000541': NtO(MH='Alveolectomy', MS='Subtotal or complete excision of the alveolar process of the maxilla or mandible. (Dorland, 28th ed)'),
  'D000542': NtO(MH='Alveolitis, Extrinsic Allergic', MS='A common interstitial lung disease caused by hypersensitivity reactions of PULMONARY ALVEOLI after inhalation of and sensitization to environmental antigens of microbial, animal, or chemical sources. The disease is characterized by lymphocytic alveolitis and granulomatous pneumonitis.'),
  'D000543': NtO(MH='Alveoloplasty', MS='Conservative contouring of the alveolar process, in preparation for immediate or future denture construction. (Dorland, 28th ed)'),
  'D000544': NtO(MH='Alzheimer Disease', MS='A degenerative disease of the BRAIN characterized by the insidious onset of DEMENTIA. Impairment of MEMORY, judgment, attention span, and problem solving skills are followed by severe APRAXIAS and a global loss of cognitive abilities. The condition primarily occurs after age 60, and is marked pathologically by severe cortical atrophy and the triad of SENILE PLAQUES; NEUROFIBRILLARY TANGLES; and NEUROPIL THREADS. (From Adams et al., Principles of Neurology, 6th ed, pp1049-57)'),
  'D000545': NtO(MH='Amanita', MS='A genus of fungi of the family Agaricaceae, order Agaricales; most species are poisonous.'),
  'D000546': NtO(MH='Amanitins', MS='Cyclic peptides extracted from carpophores of various mushroom species. They are potent inhibitors of RNA polymerases in most eukaryotic species, blocking the production of mRNA and protein synthesis. These peptides are important in the study of transcription. Alpha-amanitin is the main toxin from the species Amanitia phalloides, poisonous if ingested by humans or animals.'),
  'D000547': NtO(MH='Amantadine', MS='An antiviral that is used in the prophylactic or symptomatic treatment of influenza A. It is also used as an antiparkinsonian agent, to treat extrapyramidal reactions, and for postherpetic neuralgia. The mechanisms of its effects in movement disorders are not well understood but probably reflect an increase in synthesis and release of dopamine, with perhaps some inhibition of dopamine uptake.'),
  'D000548': NtO(MH='Amaranth Dye', MS='A sulfonic acid-based naphthylazo dye used as a coloring agent for foodstuffs and medicines and as a dye and chemical indicator. It was banned by the FDA in 1976 for use in foods, drugs, and cosmetics. (From Merck Index, 11th ed)'),
  'D000549': NtO(MH='Ambenonium Chloride', MS='A quaternary ammonium compound that is an inhibitor of cholinesterase activity with actions similar to those of NEOSTIGMINE, but of longer duration. Ambenonium is given by mouth in the treatment of myasthenia gravis. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1112)'),
  'D000550': NtO(MH='Amblyopia', MS='A nonspecific term referring to impaired vision. Major subcategories include stimulus deprivation-induced amblyopia and toxic amblyopia. Stimulus deprivation-induced amblyopia is a developmental disorder of the visual cortex. A discrepancy between visual information received by the visual cortex from each eye results in abnormal cortical development. STRABISMUS and REFRACTIVE ERRORS may cause this condition. Toxic amblyopia is a disorder of the OPTIC NERVE which is associated with ALCOHOLISM, tobacco SMOKING, and other toxins and as an adverse effect of the use of some medications.'),
  'D000551': NtO(MH='Ambroxol', MS='A metabolite of BROMHEXINE that stimulates mucociliary action and clears the air passages in the respiratory tract. It is usually administered as the hydrochloride.'),
  'D000552': NtO(MH='Ambulances', MS='A vehicle equipped for transporting patients in need of emergency care.'),
  'D000553': NtO(MH='Ambulatory Care', MS='Health care services provided to patients on an ambulatory basis, rather than by admission to a hospital or other health care facility. The services may be a part of a hospital, augmenting its inpatient services, or may be provided at a free-standing facility.'),
  'D000554': NtO(MH='Ambulatory Care Facilities', MS='Those facilities which administer health services to individuals who do not require hospitalization or institutionalization.'),
  'D000555': NtO(MH='Ambulatory Care Information Systems', MS='Information systems, usually computer-assisted, designed to store, manipulate, and retrieve information for planning, organizing, directing, and controlling administrative activities associated with the provision and utilization of ambulatory care services and facilities.'),
  'D000556': NtO(MH='Ambulatory Surgical Procedures', MS='Surgery performed on an outpatient basis. It may be hospital-based or performed in an office or surgicenter.'),
  'D000557': NtO(MH='Ambystoma', MS='A genus of the Ambystomatidae family. The best known species are the axolotl AMBYSTOMA MEXICANUM and the closely related tiger salamander Ambystoma tigrinum. They may retain gills and remain aquatic without developing all of the adult characteristics. However, under proper changes in the environment they metamorphose.'),
  'D000558': NtO(MH='Ambystoma mexicanum', MS='A salamander found in Mexican mountain lakes and accounting for about 30 percent of the urodeles used in research. The axolotl remains in larval form throughout its life, a phenomenon known as neoteny.'),
  'D000559': NtO(MH='Ambystomatidae', MS='A family of the class Urodela which includes 4 living genera, about 33 species, and occurs only in North America. Adults are usually terrestrial, but the larval forms are aquatic.'),
  'D000560': NtO(MH='Amdinocillin', MS='An amidinopenicillanic acid derivative with broad spectrum antibacterial action.'),
  'D000561': NtO(MH='Amdinocillin Pivoxil', MS='Pivaloyloxymethyl ester of amdinocillin that is well absorbed orally, but broken down to amdinocillin in the intestinal mucosa. It is active against gram-negative organisms and used as for amdinocillin.'),
  'D000562': NtO(MH='Amebiasis', MS='Infection with any of various amebae. It is an asymptomatic carrier state in most individuals, but diseases ranging from chronic, mild diarrhea to fulminant dysentery may occur.'),
  'D000563': NtO(MH='Amebicides', MS='Agents which are destructive to amebae, especially the parasitic species causing AMEBIASIS in man and animal.'),
  'D000564': NtO(MH='Ameloblastoma', MS='An immature epithelial tumor of the JAW originating from the epithelial rests of Malassez or from other epithelial remnants of the ENAMEL from the developmental period. It is a slowly growing tumor, usually benign, but displays a marked propensity for invasive growth.'),
  'D000565': NtO(MH='Ameloblasts', MS="Cylindrical epithelial cells in the innermost layer of the ENAMEL ORGAN. Their functions include contribution to the development of the dentinoenamel junction by the deposition of a layer of the matrix, thus producing the foundation for the prisms (the structural units of the DENTAL ENAMEL), and production of the matrix for the enamel prisms and interprismatic substance. (From Jablonski's Dictionary of Dentistry, 1992)"),
  'D000566': NtO(MH='Amelogenesis', MS='The elaboration of dental enamel by ameloblasts, beginning with its participation in the formation of the dentino-enamel junction to the production of the matrix for the enamel prisms and interprismatic substance. (Jablonski, Dictionary of Dentistry, 1992).'),
  'D000567': NtO(MH='Amelogenesis Imperfecta', MS='A clinically and genetically heterogeneous group of hereditary conditions characterized by malformed DENTAL ENAMEL, usually involving DENTAL ENAMEL HYPOPLASIA and/or TOOTH HYPOMINERALIZATION.'),
  'D000568': NtO(MH='Amenorrhea', MS='Absence of menstruation.'),
  'D000569': NtO(MH='Americas', MS='The general name for NORTH AMERICA; CENTRAL AMERICA; and SOUTH AMERICA unspecified or combined.'),
  'D000570': NtO(MH='American Cancer Society', MS='A voluntary organization concerned with the prevention and treatment of cancer through education and research.'),
  'D000571': NtO(MH='American Dental Association', MS='Professional society representing the field of dentistry.'),
  'D000572': NtO(MH='American Heart Association', MS='A voluntary organization concerned with the prevention and treatment of heart and vascular diseases.'),
  'D000573': NtO(MH='American Hospital Association', MS='A professional society in the United States whose membership is composed of hospitals.'),
  'D000574': NtO(MH='American Medical Association', MS='Professional society representing the field of medicine.'),
  'D000575': NtO(MH="American Nurses' Association", MS='Professional society representing the field of nursing.'),
  'D000576': NtO(MH='Americium', MS='A completely man-made radioactive actinide with atomic symbol Am, and atomic number 95. Its valence can range from +3 to +6. Because of its nonmagnetic ground state, it is an excellent superconductor. It is also used in bone mineral analysis and as a radiation source for radiotherapy.'),
  'D000577': NtO(MH='Amides', MS="Organic compounds containing the -CO-NH2 radical. Amides are derived from acids by replacement of -OH by -NH2 or from ammonia by the replacement of H by an acyl group. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000578': NtO(MH='Amidines', MS='Derivatives of oxoacids RnE(=O)OH in which the hydroxy group is replaced by an amino group and the oxo group is replaced by =NR. Amidines include carboxamidines, sulfinamidines and phosphinamidines.'),
  'D000579': NtO(MH='Amidinotransferases', MS='Enzymes of a subclass of TRANSFERASES that catalyze the transfer of an amidino group from donor to acceptor. EC 2.1.4.'),
  'D000580': NtO(MH='Amido Black', MS='A dye used to stain proteins in electrophoretic techniques. It is used interchangeably with its acid form.'),
  'D000581': NtO(MH='Amidohydrolases', MS=None),
  'D000582': NtO(MH='Amidophosphoribosyltransferase', MS='An enzyme, involved in the early steps of purine nucleotide biosynthesis, that catalyzes the formation of 5-phosphoribosylamine from glutamine and phosphoribosylpyrophosphate. EC 2.4.2.14.'),
  'D000583': NtO(MH='Amikacin', MS='A broad-spectrum antibiotic derived from KANAMYCIN. It is reno- and oto-toxic like the other aminoglycoside antibiotics.'),
  'D000584': NtO(MH='Amiloride', MS="A pyrazine compound inhibiting SODIUM reabsorption through SODIUM CHANNELS in renal EPITHELIAL CELLS. This inhibition creates a negative potential in the luminal membranes of principal cells, located in the distal convoluted tubule and collecting duct. Negative potential reduces secretion of potassium and hydrogen ions. Amiloride is used in conjunction with DIURETICS to spare POTASSIUM loss. (From Gilman et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 9th ed, p705)"),
  'D000585': NtO(MH='Aminacrine', MS='A highly fluorescent anti-infective dye used clinically as a topical antiseptic and experimentally as a mutagen, due to its interaction with DNA. It is also used as an intracellular pH indicator.'),
  'D000586': NtO(MH='Amination', MS='The creation of an amine. It can be produced by the addition of an amino group to an organic compound or reduction of a nitro group.'),
  'D000587': NtO(MH='Oxidoreductases Acting on CH-NH Group Donors', MS='Enzymes catalyzing the dehydrogenation of secondary amines, introducing a C=N double bond as the primary reaction. In some cases this is later hydrolyzed.'),
  'D000588': NtO(MH='Amines', MS="A group of compounds derived from ammonia by substituting organic radicals for the hydrogens. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D000590': NtO(MH='Amino Acid Chloromethyl Ketones', MS='Inhibitors of SERINE ENDOPEPTIDASES and sulfhydryl group-containing enzymes. They act as alkylating agents and are known to interfere in the translation process.'),
  'D000591': NtO(MH='Amino Acid Isomerases', MS='Enzymes that catalyze either the racemization or epimerization of chiral centers within amino acids or derivatives. EC 5.1.1.'),
  'D000592': NtO(MH='Amino Acid Metabolism, Inborn Errors', MS='Disorders affecting amino acid metabolism. The majority of these disorders are inherited and present in the neonatal period with metabolic disturbances (e.g., ACIDOSIS) and neurologic manifestations. They are present at birth, although they may not become symptomatic until later in life.'),
  'D000594': NtO(MH='Amino Acid Oxidoreductases', MS='A class of enzymes that catalyze oxidation-reduction reactions of amino acids.'),
  'D000595': NtO(MH='Amino Acid Sequence', MS='The order of amino acids as they occur in a polypeptide chain. This is referred to as the primary structure of proteins. It is of fundamental importance in determining PROTEIN CONFORMATION.'),
  'D000596': NtO(MH='Amino Acids', MS='Organic compounds that generally contain an amino (-NH2) and a carboxyl (-COOH) group. Twenty alpha-amino acids are the subunits which are polymerized to form proteins.'),
  'D000597': NtO(MH='Amino Acids, Branched-Chain', MS='Amino acids which have a branched carbon chain.'),
  'D000598': NtO(MH='Amino Acids, Cyclic', MS='A class of amino acids characterized by a closed ring structure.'),
  'D000599': NtO(MH='Amino Acids, Diamino', MS=None),
  'D000600': NtO(MH='Amino Acids, Dicarboxylic', MS=None),
  'D000601': NtO(MH='Amino Acids, Essential', MS='Amino acids that are not synthesized by the human body in amounts sufficient to carry out physiological functions. They are obtained from dietary foodstuffs.'),
  'D000602': NtO(MH='Amino Acids, Peptides, and Proteins', MS='Amino acids and chains of amino acids connected by peptide linkages.'),
  'D000603': NtO(MH='Amino Acids, Sulfur', MS=None),
  'D000604': NtO(MH='Amino Acyl-tRNA Synthetases', MS='A subclass of enzymes that aminoacylate AMINO ACID-SPECIFIC TRANSFER RNA with their corresponding AMINO ACIDS.'),
  'D000605': NtO(MH='Amino Alcohols', MS='Compounds possessing both a hydroxyl (-OH) and an amino group (-NH2).'),
  'D000606': NtO(MH='Amino Sugars', MS='SUGARS containing an amino group. GLYCOSYLATION of other compounds with these amino sugars results in AMINOGLYCOSIDES.'),
  'D000607': NtO(MH='Aminoacetonitrile', MS='Cyanomethylamine.'),
  'D000608': NtO(MH='Renal Aminoacidurias', MS='A group of inherited kidney disorders characterized by the abnormally elevated levels of AMINO ACIDS in URINE. Genetic mutations of transport proteins result in the defective reabsorption of free amino acids at the PROXIMAL RENAL TUBULES. Renal aminoaciduria are classified by the specific amino acid or acids involved.'),
  'D000609': NtO(MH='Aminoacridines', MS='Acridines which are substituted in any position by one or more amino groups or substituted amino groups.'),
  'D000611': NtO(MH='Aminobiphenyl Compounds', MS='Biphenyl compounds substituted in any position by one or more amino groups. Permitted are any substituents except fused rings.'),
  'D000612': NtO(MH='4-Aminobutyrate Transaminase', MS='An enzyme that converts brain gamma-aminobutyric acid (GAMMA-AMINOBUTYRIC ACID) into succinate semialdehyde, which can be converted to succinic acid and enter the citric acid cycle. It also acts on beta-alanine. EC 2.6.1.19.'),
  'D000613': NtO(MH='Aminobutyrates', MS='Derivatives of BUTYRIC ACID that contain one or more amino groups attached to the aliphatic structure. Included under this heading are a broad variety of acid forms, salts, esters, and amides that include the aminobutryrate structure.'),
  'D000614': NtO(MH='Aminocaproates', MS='Amino derivatives of caproic acid. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the amino caproic acid structure.'),
  'D000615': NtO(MH='Aminoethylphosphonic Acid', MS='An organophosphorus compound isolated from human and animal tissues.'),
  'D000616': NtO(MH='Aminoglutethimide', MS='An aromatase inhibitor that is used in the treatment of advanced BREAST CANCER.'),
  'D000617': NtO(MH='Aminoglycosides', MS='Glycosylated compounds in which there is an amino substituent on the glycoside. Some of them are clinically important ANTIBIOTICS.'),
  'D000618': NtO(MH='Aminohippuric Acids', MS='A group of glycine amides of aminobenzoic acids.'),
  'D000619': NtO(MH='Aminohydrolases', MS=None),
  'D000620': NtO(MH='Aminoimidazole Carboxamide', MS='An imidazole derivative which is a metabolite of the antineoplastic agents BIC and DIC. By itself, or as the ribonucleotide, it is used as a condensation agent in the preparation of nucleosides and nucleotides. Compounded with orotic acid, it is used to treat liver diseases.'),
  'D000621': NtO(MH='Aminoisobutyric Acids', MS='A group of compounds that are derivatives of the amino acid 2-amino-2-methylpropanoic acid.'),
  'D000622': NtO(MH='Aminolevulinic Acid', MS='A compound produced from succinyl-CoA and GLYCINE as an intermediate in heme synthesis. It is used as a PHOTOCHEMOTHERAPY for actinic KERATOSIS.'),
  'D000623': NtO(MH='Porphobilinogen Synthase', MS='An enzyme that catalyzes the formation of porphobilinogen from two molecules of 5-aminolevulinic acid. EC 4.2.1.24.'),
  'D000624': NtO(MH='5-Aminolevulinate Synthetase', MS='An enzyme of the transferase class that catalyzes condensation of the succinyl group from succinyl coenzyme A with glycine to form delta-aminolevulinate. It is a pyridoxyal phosphate protein and the reaction occurs in mitochondria as the first step of the heme biosynthetic pathway. The enzyme is a key regulatory enzyme in heme biosynthesis. In liver feedback is inhibited by heme. EC 2.3.1.37.'),
  'D000625': NtO(MH='Aminooxyacetic Acid', MS='A compound that inhibits aminobutyrate aminotransferase activity in vivo, thereby raising the level of gamma-aminobutyric acid in tissues.'),
  'D000626': NtO(MH='Aminopeptidases', MS='A subclass of EXOPEPTIDASES that act on the free N terminus end of a polypeptide liberating a single amino acid residue. EC 3.4.11.'),
  'D000627': NtO(MH='Aminophenols', MS='Phenols substituted in any position by an amino group.'),
  'D000628': NtO(MH='Aminophylline', MS="A drug combination that contains THEOPHYLLINE and ethylenediamine. It is more soluble in water than theophylline but has similar pharmacologic actions. It's most common use is in bronchial asthma, but it has been investigated for several other applications."),
  'D000629': NtO(MH='Aminopropionitrile', MS='Reagent used as an intermediate in the manufacture of beta-alanine and pantothenic acid.'),
  'D000630': NtO(MH='Aminopterin', MS='A folic acid derivative used as a rodenticide that has been shown to be teratogenic.'),
  'D000631': NtO(MH='Aminopyridines', MS='Pyridines substituted in any position with an amino group. May be hydrogenated, but must retain at least one double bond.'),
  'D000632': NtO(MH='Aminopyrine', MS='A pyrazolone with analgesic, anti-inflammatory, and antipyretic properties but has risk of AGRANULOCYTOSIS. A breath test with 13C-labeled aminopyrine has been used as a non-invasive measure of CYTOCHROME P-450 metabolic activity in LIVER FUNCTION TESTS.'),
  'D000633': NtO(MH='Aminopyrine N-Demethylase', MS=None),
  'D000634': NtO(MH='Aminoquinolines', MS='Quinolines substituted in any position by one or more amino groups.'),
  'D000635': NtO(MH='Aminorex', MS='An amphetamine-like anorectic agent. It may cause pulmonary hypertension.'),
  'D000636': NtO(MH='Aminosalicylic Acids', MS='A group of 2-hydroxybenzoic acids that can be substituted by amino groups at any of the 3-, 4-, 5-, or 6-positions.'),
  'D000637': NtO(MH='Transaminases', MS='A subclass of enzymes of the transferase class that catalyze the transfer of an amino group from a donor (generally an amino acid) to an acceptor (generally a 2-keto acid). Most of these enzymes are pyridoxyl phosphate proteins. (Dorland, 28th ed) EC 2.6.1.'),
  'D000638': NtO(MH='Amiodarone', MS='An antianginal and class III antiarrhythmic drug. It increases the duration of ventricular and atrial muscle action by inhibiting POTASSIUM CHANNELS and VOLTAGE-GATED SODIUM CHANNELS. There is a resulting decrease in heart rate and in vascular resistance.'),
  'D000639': NtO(MH='Amitriptyline', MS='Tricyclic antidepressant with anticholinergic and sedative properties. It appears to prevent the re-uptake of norepinephrine and serotonin at nerve terminals, thus potentiating the action of these neurotransmitters. Amitriptyline also appears to antagonize cholinergic and alpha-1 adrenergic responses to bioactive amines.'),
  'D000640': NtO(MH='Amitrole', MS='A non-selective post-emergence, translocated herbicide. According to the Seventh Annual Report on Carcinogens (PB95-109781, 1994) this substance may reasonably be anticipated to be a carcinogen. (From Merck Index, 12th ed) It is an irreversible inhibitor of CATALASE, and thus impairs activity of peroxisomes.'),
  'D000641': NtO(MH='Ammonia', MS='A colorless alkaline gas. It is formed in the body during decomposition of organic materials during a large number of metabolically important reactions. Note that the aqueous form of ammonia is referred to as AMMONIUM HYDROXIDE.'),
  'D000642': NtO(MH='Ammonia-Lyases', MS='Enzymes that catalyze the formation of a carbon-carbon double bond by the elimination of AMMONIA. EC 4.3.1.'),
  'D000643': NtO(MH='Ammonium Chloride', MS='An acidifying agent that has expectorant and diuretic effects. Also used in etching and batteries and as a flux in electroplating.'),
  'D000644': NtO(MH='Quaternary Ammonium Compounds', MS='Derivatives of ammonium compounds, NH4+ Y-, in which all four of the hydrogens bonded to nitrogen have been replaced with hydrocarbyl groups. These are distinguished from IMINES which are RN=CR2.'),
  'D000645': NtO(MH='Ammonium Sulfate', MS='Sulfuric acid diammonium salt. It is used in CHEMICAL FRACTIONATION of proteins.'),
  'D000646': NtO(MH='Ammotherapy', MS='Treatment by application of hot sand.'),
  'D000647': NtO(MH='Amnesia', MS='Pathologic partial or complete loss of the ability to recall past experiences (AMNESIA, RETROGRADE) or to form new memories (AMNESIA, ANTEROGRADE). This condition may be of organic or psychologic origin. Organic forms of amnesia are usually associated with dysfunction of the DIENCEPHALON or HIPPOCAMPUS. (From Adams et al., Principles of Neurology, 6th ed, pp426-7)'),
  'D000648': NtO(MH='Amnesia, Retrograde', MS='Loss of the ability to recall information that had been previously encoded in memory prior to a specified or approximate point in time. This process may be organic or psychogenic in origin. Organic forms may be associated with CRANIOCEREBRAL TRAUMA; CEREBROVASCULAR ACCIDENTS; SEIZURES; DEMENTIA; and a wide variety of other conditions that impair cerebral function. (From Adams et al., Principles of Neurology, 6th ed, pp426-9)'),
  'D000649': NtO(MH='Amniocentesis', MS='Percutaneous transabdominal puncture of the uterus during pregnancy to obtain amniotic fluid. It is commonly used for fetal karyotype determination in order to diagnose abnormal fetal conditions.'),
  'D000650': NtO(MH='Amnion', MS='The innermost membranous sac that surrounds and protects the developing embryo which is bathed in the AMNIOTIC FLUID. Amnion cells are secretory EPITHELIAL CELLS and contribute to the amniotic fluid.'),
  'D000652': NtO(MH='Amniotic Band Syndrome', MS='A disorder present in the newborn infant in which constriction rings or bands, causing soft tissue depressions, encircle digits, extremities, or limbs and sometimes the neck, thorax, or abdomen. They may be associated with intrauterine amputations.'),
  'D000653': NtO(MH='Amniotic Fluid', MS='A clear, yellowish liquid that envelopes the FETUS inside the sac of AMNION. In the first trimester, it is likely a transudate of maternal or fetal plasma. In the second trimester, amniotic fluid derives primarily from fetal lung and kidney. Cells or substances in this fluid can be removed for prenatal diagnostic tests (AMNIOCENTESIS).'),
  'D000654': NtO(MH='Amobarbital', MS='A barbiturate with hypnotic and sedative properties (but not antianxiety). Adverse effects are mainly a consequence of dose-related CNS depression and the risk of dependence with continued use is high. (From Martindale, The Extra Pharmacopoeia, 30th ed, p565)'),
  'D000655': NtO(MH='Amodiaquine', MS='A 4-aminoquinoline compound with anti-inflammatory properties.'),
  'D000656': NtO(MH='Amoeba', MS='A genus of ameboid protozoa. Characteristics include a vesicular nucleus and the formation of several PSEUDOPODIA, one of which is dominant at a given time. Reproduction occurs asexually by binary fission.'),
  'D000657': NtO(MH='Amoxapine', MS='The N-demethylated derivative of the antipsychotic agent LOXAPINE that works by blocking the reuptake of norepinephrine, serotonin, or both; it also blocks dopamine receptors. Amoxapine is used for the treatment of depression.'),
  'D000658': NtO(MH='Amoxicillin', MS='A broad-spectrum semisynthetic antibiotic similar to AMPICILLIN except that its resistance to gastric acid permits higher serum levels with oral administration.'),
  'D000659': NtO(MH='AMP Deaminase', MS='An enzyme that catalyzes the deamination of AMP to IMP. EC 3.5.4.6.'),
  'D000661': NtO(MH='Amphetamine', MS='A powerful central nervous system stimulant and sympathomimetic. Amphetamine has multiple mechanisms of action including blocking uptake of adrenergics and dopamine, stimulation of release of monamines, and inhibiting monoamine oxidase. Amphetamine is also a drug of abuse and a psychotomimetic. The l- and the d,l-forms are included here. The l-form has less central nervous system activity but stronger cardiovascular effects. The d-form is DEXTROAMPHETAMINE.'),
  'D000662': NtO(MH='Amphetamines', MS='Analogs or derivatives of AMPHETAMINE. Many are sympathomimetics and central nervous system stimulators causing excitation, vasopressin, bronchodilation, and to varying degrees, anorexia, analepsis, nasal decongestion, and some smooth muscle relaxation.'),
  'D000663': NtO(MH='Amphibians', MS='VERTEBRATES belonging to the class amphibia such as frogs, toads, newts and salamanders that live in a semiaquatic environment.'),
  'D000664': NtO(MH='Amphibian Venoms', MS='Venoms produced by frogs, toads, salamanders, etc. The venom glands are usually on the skin of the back and contain cardiotoxic glycosides, cholinolytics, and a number of other bioactive materials, many of which have been characterized. The venoms have been used as arrow poisons and include bufogenin, bufotoxin, bufagin, bufotalin, histrionicotoxins, and pumiliotoxin.'),
  'D000665': NtO(MH='Ampholyte Mixtures', MS='Such mixtures of amphoteric electrolytes or buffers that provide a continuous range of pH in an electric field; used for separating proteins by their isoelectric points, i.e., by isoelectric focusing.'),
  'D000666': NtO(MH='Amphotericin B', MS='Macrolide antifungal antibiotic produced by Streptomyces nodosus obtained from soil of the Orinoco river region of Venezuela.'),
  'D000667': NtO(MH='Ampicillin', MS='Semi-synthetic derivative of penicillin that functions as an orally active broad-spectrum antibiotic.'),
  'D000668': NtO(MH='Ampicillin Resistance', MS='Nonsusceptibility of a microbe to the action of ampicillin, a penicillin derivative that interferes with cell wall synthesis.'),
  'D000669': NtO(MH='Amplifiers, Electronic', MS="Electronic devices that increase the magnitude of a signal's power level or current."),
  'D000670': NtO(MH='Amprolium', MS='A veterinary coccidiostat that interferes with THIAMINE metabolism.'),
  'D000671': NtO(MH='Amputation', MS='The removal of a limb or other appendage or outgrowth of the body. (Dorland, 28th ed)'),
  'D000672': NtO(MH='Amputation Stumps', MS='The part of a limb or tail following amputation that is proximal to the amputated section.'),
  'D000673': NtO(MH='Amputation, Traumatic', MS='Loss of a limb or other bodily appendage by accidental injury.'),
  'D000674': NtO(MH='Amputees', MS=None),
  'D000675': NtO(MH='Ampyrone', MS='A metabolite of AMINOPYRINE with analgesic and anti-inflammatory properties. It is used as a reagent for biochemical reactions producing peroxides or phenols. Ampyrone stimulates LIVER MICROSOMES and is also used to measure extracellular water.'),
  'D000676': NtO(MH='Amrinone', MS='A positive inotropic cardiotonic (CARDIOTONIC AGENTS) with vasodilator properties, phosphodiesterase 3 inhibitory activity, and the ability to stimulate calcium ion influx into the cardiac cell.'),
  'D000677': NtO(MH='Amsacrine', MS='An aminoacridine derivative that intercalates into DNA and is used as an antineoplastic agent.'),
  'D000678': NtO(MH='Amygdalin', MS='A cyanogenic glycoside found in the seeds of Rosaceae.'),
  'D000679': NtO(MH='Amygdala', MS='Almond-shaped group of basal nuclei anterior to the INFERIOR HORN OF THE LATERAL VENTRICLE of the TEMPORAL LOBE. The amygdala is part of the limbic system.'),
  'D000680': NtO(MH='Amyl Nitrite', MS='A vasodilator that is administered by inhalation. It is also used recreationally due to its supposed ability to induce euphoria and act as an aphrodisiac.'),
  'D000681': NtO(MH='Amylases', MS='A group of amylolytic enzymes that cleave starch, glycogen, and related alpha-1,4-glucans. (Stedman, 25th ed) EC 3.2.1.-.'),
  'D000682': NtO(MH='Amyloid', MS='A fibrous protein complex that consists of proteins folded into a specific cross beta-pleated sheet structure. This fibrillar structure has been found as an alternative folding pattern for a variety of functional proteins. Deposits of amyloid in the form of AMYLOID PLAQUES are associated with a variety of degenerative diseases. The amyloid structure has also been found in a number of functional proteins that are unrelated to disease.'),
  'D000683': NtO(MH='Serum Amyloid P-Component', MS='Amyloid P component is a small, non-fibrillar glycoprotein found in normal serum and in all amyloid deposits. It has a pentagonal (pentaxin) structure. It is an acute phase protein, modulates immunologic responses, inhibits ELASTASE, and has been suggested as an indicator of LIVER DISEASE.'),
  'D000685': NtO(MH='Serum Amyloid A Protein', MS='An ACUTE PHASE REACTION protein present in low concentrations in normal sera, but found at higher concentrations in sera of older persons and in patients with AMYLOIDOSIS. It is the circulating precusor of amyloid A protein, which is found deposited in AA type AMYLOID FIBRILS.'),
  'D000686': NtO(MH='Amyloidosis', MS='A group of sporadic, familial and/or inherited, degenerative, and infectious disease processes, linked by the common theme of abnormal protein folding and deposition of AMYLOID. As the amyloid deposits enlarge they displace normal tissue structures, causing disruption of function. Various signs and symptoms depend on the location and size of the deposits.'),
  'D000687': NtO(MH='Amylopectin', MS='A highly branched glucan in starch.'),
  'D000688': NtO(MH='Amylose', MS='An unbranched glucan in starch.'),
  'D000690': NtO(MH='Amyotrophic Lateral Sclerosis', MS='A degenerative disorder affecting upper MOTOR NEURONS in the brain and lower motor neurons in the brain stem and SPINAL CORD. Disease onset is usually after the age of 50 and the process is usually fatal within 3 to 6 years. Clinical manifestations include progressive weakness, atrophy, FASCICULATION, hyperreflexia, DYSARTHRIA, dysphagia, and eventual paralysis of respiratory function. Pathologic features include the replacement of motor neurons with fibrous ASTROCYTES and atrophy of anterior SPINAL NERVE ROOTS and corticospinal tracts. (From Adams et al., Principles of Neurology, 6th ed, pp1089-94)'),
  'D000691': NtO(MH='Anabasine', MS='A piperidine botanical insecticide.'),
  'D000693': NtO(MH='Anaerobiosis', MS='The complete absence, or (loosely) the paucity, of gaseous or dissolved elemental oxygen in a given place or environment. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D000694': NtO(MH='Anal Gland Neoplasms', MS='Tumors or cancer of the anal gland.'),
  'D000695': NtO(MH='Anal Sacs', MS='A pair of anal glands or sacs, located on either side of the ANUS, that produce and store a dark, foul-smelling fluid in carnivorous animals such as MEPHITIDAE and DOGS. The expelled fluid is used as a defensive repellent (in skunks) or a material to mark territory (in dogs).'),
  'D000697': NtO(MH='Central Nervous System Stimulants', MS='A loosely defined group of drugs that tend to increase behavioral alertness, agitation, or excitation. They work by a variety of mechanisms, but usually not by direct excitation of neurons. The many drugs that have such actions as side effects to their main therapeutic use are not included here.'),
  'D000698': NtO(MH='Analgesia', MS='Methods of PAIN relief that may be used with or in place of ANALGESICS.'),
  'D000699': NtO(MH='Pain Insensitivity, Congenital', MS='A syndrome characterized by indifference to PAIN despite the ability to distinguish noxious from non-noxious stimuli. Absent corneal reflexes and INTELLECTUAL DISABILITY may be associated. Familial forms with autosomal recessive and autosomal dominant patterns of inheritance have been described. (Adams et al., Principles of Neurology, 6th ed, p1343)'),
  'D000700': NtO(MH='Analgesics', MS='Compounds capable of relieving pain without the loss of CONSCIOUSNESS.'),
  'D000701': NtO(MH='Analgesics, Opioid', MS='Compounds with activity like OPIATE ALKALOIDS, acting at OPIOID RECEPTORS. Properties include induction of ANALGESIA or NARCOSIS.'),
  'D000703': NtO(MH='Analog-Digital Conversion', MS='The process of converting analog data such as continually measured voltage to discrete, digital form.'),
  'D000704': NtO(MH='Analysis of Variance', MS='A statistical technique that isolates and assesses the contributions of categorical independent variables to variation in the mean of a continuous dependent variable.'),
  'D000705': NtO(MH='Anaphase', MS='The phase of cell nucleus division following METAPHASE, in which the CHROMATIDS separate and migrate to opposite poles of the spindle.'),
  'D000706': NtO(MH='Anaphylatoxins', MS='Serum peptides derived from certain cleaved COMPLEMENT PROTEINS during COMPLEMENT ACTIVATION. They induce smooth MUSCLE CONTRACTION; mast cell HISTAMINE RELEASE; PLATELET AGGREGATION; and act as mediators of the local inflammatory process. The order of anaphylatoxin activity from the strongest to the weakest is C5a, C3a, C4a, and C5a des-arginine.'),
  'D000707': NtO(MH='Anaphylaxis', MS='An acute hypersensitivity reaction due to exposure to a previously encountered ANTIGEN. The reaction may include rapidly progressing URTICARIA, respiratory distress, vascular collapse, systemic SHOCK, and death.'),
  'D000708': NtO(MH='Anaplasia', MS='Loss of structural differentiation and useful function of neoplastic cells.'),
  'D000709': NtO(MH='Anaplasma', MS='A genus of gram-negative bacteria whose organisms are obligate parasites of vertebrates. Species are transmitted by arthropod vectors with the host range limited to ruminants. Anaplasma marginale is the most pathogenic species and is the causative agent of severe bovine anaplasmosis.'),
  'D000710': NtO(MH='Anaplasmataceae', MS='A family of bacteria in the order RICKETTSIALES which inhabit RED BLOOD CELLS and other types of cells, causing disease in both animals and humans.'),
  'D000711': NtO(MH='Anaplasmataceae Infections', MS='Infections with bacteria of the family ANAPLASMATACEAE.'),
  'D000712': NtO(MH='Anaplasmosis', MS='A disease usually in cattle caused by parasitization of the red blood cells by bacteria of the genus ANAPLASMA.'),
  'D000713': NtO(MH='Anastomosis, Roux-en-Y', MS='A Y-shaped surgical anastomosis of any part of the digestive system which includes the small intestine as the eventual drainage site.'),
  'D000714': NtO(MH='Anastomosis, Surgical', MS='Surgical union or shunt between ducts, tubes or vessels. It may be end-to-end, end-to-side, side-to-end, or side-to-side.'),
  'D000715': NtO(MH='Anatomy', MS='A branch of biology dealing with the structure of organisms.'),
  'D000716': NtO(MH='Anatomy, Artistic', MS='The study of the structures of organisms for applications in art: drawing, painting, sculpture, illustration, etc.'),
  'D000717': NtO(MH='Anatomy, Comparative', MS='The comparative study of animal structure with regard to homologous organs or parts. (Stedman, 25th ed)'),
  'D000718': NtO(MH='Anatomy, Regional', MS='The anatomical study of specific regions or parts of organisms, emphasizing the relationship between the various structures (e.g. muscles, nerves, skeletal, cardiovascular, etc.).'),
  'D000719': NtO(MH='Anatomy, Veterinary', MS='The study of the anatomical structures of animals.'),
  'D000720': NtO(MH='Ancillary Services, Hospital', MS='Those support services other than room, board, and medical and nursing services that are provided to hospital patients in the course of care. They include such services as laboratory, radiology, pharmacy, and physical therapy services.'),
  'D000721': NtO(MH='Ancrod', MS='An enzyme fraction from the venom of the Malayan pit viper, Agkistrodon rhodostoma. It catalyzes the hydrolysis of a number of amino acid esters and a limited proteolysis of fibrinogen. It is used clinically to produce controlled defibrination in patients requiring anticoagulant therapy. EC 3.4.21.-.'),
  'D000722': NtO(MH='Ancylostoma', MS='A genus of nematode intestinal parasites that consists of several species. A. duodenale is the common hookworm in humans. A. braziliense, A. ceylonicum, and A. caninum occur primarily in cats and dogs, but all have been known to occur in humans.'),
  'D000723': NtO(MH='Ancylostomatoidea', MS='A superfamily of nematode parasitic hookworms consisting of four genera: ANCYLOSTOMA; NECATOR; Bunostomum; and Uncinaria. ANCYLOSTOMA and NECATOR occur in humans and other mammals. Bunostomum is common in ruminants and Uncinaria in wolves, foxes, and dogs.'),
  'D000724': NtO(MH='Ancylostomiasis', MS='Infection of humans or animals with hookworms of the genus ANCYLOSTOMA. Characteristics include anemia, dyspepsia, eosinophilia, and abdominal swelling.'),
  'D000725': NtO(MH='Andorra', MS="A principality in the Pyrenees between France and Spain. Its capital is also called Andorra. (From Webster's New Geographical Dictionary, 1988, p50)"),
  'D000726': NtO(MH='Androgen Antagonists', MS='Compounds which inhibit or antagonize the biosynthesis or actions of androgens.'),
  'D000727': NtO(MH='Androgen-Binding Protein', MS='Carrier proteins produced in the Sertoli cells of the testis, secreted into the seminiferous tubules, and transported via the efferent ducts to the epididymis. They participate in the transport of androgens. Androgen-binding protein has the same amino acid sequence as SEX HORMONE-BINDING GLOBULIN. They differ by their sites of synthesis and post-translational oligosaccharide modifications.'),
  'D000728': NtO(MH='Androgens', MS='Compounds that interact with ANDROGEN RECEPTORS in target tissues to bring about the effects similar to those of TESTOSTERONE. Depending on the target tissues, androgenic effects can be on SEX DIFFERENTIATION; male reproductive organs, SPERMATOGENESIS; secondary male SEX CHARACTERISTICS; LIBIDO; development of muscle mass, strength, and power.'),
  'D000730': NtO(MH='Androstadienes', MS='Derivatives of the steroid androstane having two double bonds at any site in any of the rings.'),
  'D000731': NtO(MH='Androstanes', MS='The family of steroids from which the androgens are derived.'),
  'D000732': NtO(MH='Androstanols', MS='Androstanes and androstane derivatives which are substituted in any position with one or more hydroxyl groups.'),
  'D000733': NtO(MH='Androstatrienes', MS='Derivatives of the steroid androstane having three double bonds at any site in any of the rings.'),
  'D000734': NtO(MH='Androstenediols', MS='Unsaturated androstane derivatives which are substituted with two hydroxy groups in any position in the ring system.'),
  'D000735': NtO(MH='Androstenedione', MS='A delta-4 C19 steroid that is produced not only in the TESTIS, but also in the OVARY and the ADRENAL CORTEX. Depending on the tissue type, androstenedione can serve as a precursor to TESTOSTERONE as well as ESTRONE and ESTRADIOL.'),
  'D000736': NtO(MH='Androstenes', MS='Unsaturated derivatives of the steroid androstane containing at least one double bond at any site in any of the rings.'),
  'D000737': NtO(MH='Androstenols', MS='Unsaturated androstanes which are substituted with one or more hydroxyl groups in any position in the ring system.'),
  'D000738': NtO(MH='Androsterone', MS='A metabolite of TESTOSTERONE or ANDROSTENEDIONE with a 3-alpha-hydroxyl group and without the double bond. The 3-beta hydroxyl isomer is epiandrosterone.'),
  'D000739': NtO(MH='Anecdotes as Topic', MS='Works about brief accounts or narratives of an incident or event.'),
  'D000740': NtO(MH='Anemia', MS='A reduction in the number of circulating ERYTHROCYTES or in the quantity of HEMOGLOBIN.'),
  'D000741': NtO(MH='Anemia, Aplastic', MS='A form of anemia in which the bone marrow fails to produce adequate numbers of peripheral blood elements.'),
  'D000742': NtO(MH='Anemia, Dyserythropoietic, Congenital', MS='A familial disorder characterized by ANEMIA with multinuclear ERYTHROBLASTS, karyorrhexis, asynchrony of nuclear and cytoplasmic maturation, and various nuclear abnormalities of bone marrow erythrocyte precursors (ERYTHROID PRECURSOR CELLS). Type II is the most common of the 3 types; it is often referred to as HEMPAS, based on the Hereditary Erythroblast Multinuclearity with Positive Acidified Serum test.'),
  'D000743': NtO(MH='Anemia, Hemolytic', MS='A condition of inadequate circulating red blood cells (ANEMIA) or insufficient HEMOGLOBIN due to premature destruction of red blood cells (ERYTHROCYTES).'),
  'D000744': NtO(MH='Anemia, Hemolytic, Autoimmune', MS="Acquired hemolytic anemia due to the presence of AUTOANTIBODIES which agglutinate or lyse the patient's own RED BLOOD CELLS."),
  'D000745': NtO(MH='Anemia, Hemolytic, Congenital', MS='Hemolytic anemia due to various intrinsic defects of the erythrocyte.'),
  'D000746': NtO(MH='Anemia, Hemolytic, Congenital Nonspherocytic', MS='Any one of a group of congenital hemolytic anemias in which there is no abnormal hemoglobin or spherocytosis and in which there is a defect of glycolysis in the erythrocyte. Common causes include deficiencies in GLUCOSE-6-PHOSPHATE ISOMERASE; PYRUVATE KINASE; and GLUCOSE-6-PHOSPHATE DEHYDROGENASE.'),
  'D000747': NtO(MH='Anemia, Hypochromic', MS='Anemia characterized by a decrease in the ratio of the weight of hemoglobin to the volume of the erythrocyte, i.e., the mean corpuscular hemoglobin concentration is less than normal. The individual cells contain less hemoglobin than they could have under optimal conditions. Hypochromic anemia may be caused by iron deficiency from a low iron intake, diminished iron absorption, or excessive iron loss. It can also be caused by infections or other diseases, therapeutic drugs, lead poisoning, and other conditions. (Stedman, 25th ed; from Miale, Laboratory Medicine: Hematology, 6th ed, p393)'),
  'D000748': NtO(MH='Anemia, Macrocytic', MS='Anemia characterized by larger than normal erythrocytes, increased mean corpuscular volume (MCV) and increased mean corpuscular hemoglobin (MCH).'),
  'D000749': NtO(MH='Anemia, Megaloblastic', MS='A disorder characterized by the presence of ANEMIA, abnormally large red blood cells (megalocytes or macrocytes), and MEGALOBLASTS.'),
  'D000750': NtO(MH='Anemia, Myelophthisic', MS='Anemia characterized by appearance of immature myeloid and nucleated erythrocytes in the peripheral blood, resulting from infiltration of the bone marrow by foreign or abnormal tissue.'),
  'D000751': NtO(MH='Anemia, Neonatal', MS='The mildest form of erythroblastosis fetalis in which anemia is the chief manifestation.'),
  'D000752': NtO(MH='Anemia, Pernicious', MS='A megaloblastic anemia occurring in children but more commonly in later life, characterized by histamine-fast achlorhydria, in which the laboratory and clinical manifestations are based on malabsorption of vitamin B 12 due to a failure of the gastric mucosa to secrete adequate and potent intrinsic factor. (Dorland, 27th ed)'),
  'D000753': NtO(MH='Anemia, Refractory', MS='A severe sometimes chronic anemia, usually macrocytic in type, that does not respond to ordinary antianemic therapy.'),
  'D000754': NtO(MH='Anemia, Refractory, with Excess of Blasts', MS='Chronic refractory anemia with granulocytopenia, and/or thrombocytopenia. Myeloblasts and progranulocytes constitute 5 to 40 percent of the nucleated marrow cells.'),
  'D000755': NtO(MH='Anemia, Sickle Cell', MS='A disease characterized by chronic hemolytic anemia, episodic painful crises, and pathologic involvement of many organs. It is the clinical expression of homozygosity for hemoglobin S.'),
  'D000756': NtO(MH='Anemia, Sideroblastic', MS='Anemia characterized by the presence of erythroblasts containing excessive deposits of iron in the marrow.'),
  'D000757': NtO(MH='Anencephaly', MS='A malformation of the nervous system caused by failure of the anterior neuropore to close. Infants are born with intact spinal cords, cerebellums, and brainstems, but lack formation of neural structures above this level. The skull is only partially formed but the eyes are usually normal. This condition may be associated with folate deficiency. Affected infants are only capable of primitive (brain stem) reflexes and usually do not survive for more than two weeks. (From Menkes, Textbook of Child Neurology, 5th ed, p247)'),
  'D000758': NtO(MH='Anesthesia', MS='A state characterized by loss of feeling or sensation. This depression of nerve function is usually the result of pharmacologic action and is induced to allow performance of surgery or other painful procedures.'),
  'D000759': NtO(MH='Adjuvants, Anesthesia', MS='Agents that are administered in association with anesthetics to increase effectiveness, improve delivery, or decrease required dosage.'),
  'D000760': NtO(MH='Anesthesia and Analgesia', MS='Medical methods of either relieving pain caused by a particular condition or removing the sensation of pain during a surgery or other medical procedure.'),
  'D000761': NtO(MH='Anesthesia Department, Hospital', MS='Hospital department responsible for the administration of functions and activities pertaining to the delivery of anesthetics.'),
  'D000762': NtO(MH='Anesthesia Recovery Period', MS='The period of emergence from general anesthesia, where different elements of consciousness return at different rates.'),
  'D000763': NtO(MH='Anesthesia, Caudal', MS='Epidural anesthesia administered via the sacral canal.'),
  'D000764': NtO(MH='Anesthesia, Closed-Circuit', MS='Inhalation anesthesia where the gases exhaled by the patient are rebreathed as some carbon dioxide is simultaneously removed and anesthetic gas and oxygen are added so that no anesthetic escapes into the room. Closed-circuit anesthesia is used especially with explosive anesthetics to prevent fires where electrical sparking from instruments is possible.'),
  'D000765': NtO(MH='Anesthesia, Conduction', MS='Injection of an anesthetic into the nerves to inhibit nerve transmission in a specific part of the body.'),
  'D000766': NtO(MH='Anesthesia, Dental', MS='A range of methods used to reduce pain and anxiety during dental procedures.'),
  'D000767': NtO(MH='Anesthesia, Epidural', MS='Procedure in which an anesthetic is injected into the epidural space.'),
  'D000768': NtO(MH='Anesthesia, General', MS='Procedure in which patients are induced into an unconscious state through use of various medications so that they do not feel pain during surgery.'),
  'D000769': NtO(MH='Anesthesia, Inhalation', MS='Anesthesia caused by the breathing of anesthetic gases or vapors or by insufflating anesthetic gases or vapors into the respiratory tract.'),
  'D000770': NtO(MH='Anesthesia, Endotracheal', MS='Procedure in which an anesthetic such as a gas or mixture of gases is inhaled through a tube into the lungs.'),
  'D000771': NtO(MH='Anesthesia, Intravenous', MS='Process of administering an anesthetic through injection directly into the bloodstream.'),
  'D000772': NtO(MH='Anesthesia, Local', MS='A blocking of nerve conduction to a specific area by an injection of an anesthetic agent.'),
  'D000773': NtO(MH='Anesthesia, Obstetrical', MS='A variety of anesthetic methods such as EPIDURAL ANESTHESIA used to control the pain of childbirth.'),
  'D000774': NtO(MH='Anesthesia, Rectal', MS='Procedure involving the instillation of an anesthetic into the rectum.'),
  'D000775': NtO(MH='Anesthesia, Spinal', MS='Procedure in which an anesthetic is injected directly into the spinal cord.'),
  'D000776': NtO(MH='Anesthesiology', MS='A specialty concerned with the study of anesthetics and anesthesia.'),
  'D000777': NtO(MH='Anesthetics', MS='Agents that are capable of inducing a total or partial loss of sensation, especially tactile sensation and pain. They may act to induce general ANESTHESIA, in which an unconscious state is achieved, or may act locally to induce numbness or lack of sensation at a targeted site.'),
  'D000778': NtO(MH='Anesthetics, Dissociative', MS="Intravenous anesthetics that induce a state of sedation, immobility, amnesia, and marked analgesia. Subjects may experience a strong feeling of dissociation from the environment. The condition produced is similar to NEUROLEPTANALGESIA, but is brought about by the administration of a single drug. (From Gilman et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed)"),
  'D000779': NtO(MH='Anesthetics, Local', MS="Drugs that block nerve conduction when applied locally to nerve tissue in appropriate concentrations. They act on any part of the nervous system and on every type of nerve fiber. In contact with a nerve trunk, these anesthetics can cause both sensory and motor paralysis in the innervated area. Their action is completely reversible. (From Gilman AG, et. al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed) Nearly all local anesthetics act by reducing the tendency of voltage-dependent sodium channels to activate."),
  'D000780': NtO(MH='Anestrus', MS='A state of sexual inactivity in female animals exhibiting no ESTROUS CYCLE. Causes of anestrus include pregnancy, presence of offspring, season, stress, and pathology.'),
  'D000781': NtO(MH='Anethole Trithione', MS='Choleretic used to allay dry mouth and constipation due to tranquilizers.'),
  'D000782': NtO(MH='Aneuploidy', MS='The chromosomal constitution of cells which deviate from the normal by the addition or subtraction of CHROMOSOMES, chromosome pairs, or chromosome fragments. In a normally diploid cell (DIPLOIDY) the loss of a chromosome pair is termed nullisomy (symbol: 2N-2), the loss of a single chromosome is MONOSOMY (symbol: 2N-1), the addition of a chromosome pair is tetrasomy (symbol: 2N+2), the addition of a single chromosome is TRISOMY (symbol: 2N+1).'),
  'D000783': NtO(MH='Aneurysm', MS='Pathological outpouching or sac-like dilatation in the wall of any blood vessel (ARTERIES or VEINS) or the heart (HEART ANEURYSM). It indicates a thin and weakened area in the wall which may later rupture. Aneurysms are classified by location, etiology, or other characteristics.'),
  'D000784': NtO(MH='Aneurysm, Dissecting', MS='An aneurysm caused by a tear in the TUNICA INTIMA of a blood vessel leading to interstitial HEMORRHAGE, and splitting (dissecting) of the vessel wall, often involving the AORTA. Dissection between the tunica intima and TUNICA MEDIA causes luminal occlusion. Dissection at the media, or between the media and the outer ADVENTITIA causes aneurismal dilation.'),
  'D000785': NtO(MH='Aneurysm, Infected', MS='Aneurysm due to growth of microorganisms in the arterial wall, or infection arising within preexisting arteriosclerotic aneurysms.'),
  'D000786': NtO(MH='Anger', MS='A strong emotional feeling of displeasure aroused by being interfered with, injured or threatened.'),
  'D000787': NtO(MH='Angina Pectoris', MS='The symptom of paroxysmal pain consequent to MYOCARDIAL ISCHEMIA usually of distinctive character, location and radiation. It is thought to be provoked by a transient stressful situation during which the oxygen requirements of the MYOCARDIUM exceed that supplied by the CORONARY CIRCULATION.'),
  'D000788': NtO(MH='Angina Pectoris, Variant', MS='A clinical syndrome characterized by the development of CHEST PAIN at rest with concomitant transient ST segment elevation in the ELECTROCARDIOGRAM, but with preserved exercise capacity.'),
  'D000789': NtO(MH='Angina, Unstable', MS='Precordial pain at rest, which may precede a MYOCARDIAL INFARCTION.'),
  'D000790': NtO(MH='Angiocardiography', MS='Radiography of the heart and great vessels after injection of a contrast medium.'),
  'D000792': NtO(MH='Angiography', MS='Radiography of blood vessels after injection of a contrast medium.'),
  'D000793': NtO(MH='Angioid Streaks', MS='Small breaks in the elastin-filled tissue of the retina.'),
  'D000794': NtO(MH='Angiokeratoma', MS='A vascular, horny neoplasm of the skin characterized by TELANGIECTASIS and secondary epithelial changes including acanthosis and hyperkeratosis.'),
  'D000795': NtO(MH='Fabry Disease', MS='An X-linked inherited metabolic disease caused by a deficiency of lysosomal ALPHA-GALACTOSIDASE A. It is characterized by intralysosomal accumulation of globotriaosylceramide and other GLYCOSPHINGOLIPIDS in blood vessels throughout the body leading to multi-system complications including renal, cardiac, cerebrovascular, and skin disorders.'),
  'D000796': NtO(MH='Angiolymphoid Hyperplasia with Eosinophilia', MS='Solitary or multiple benign cutaneous nodules comprised of immature and mature vascular structures intermingled with endothelial cells and a varied infiltrate of eosinophils, histiocytes, lymphocytes, and mast cells.'),
  'D000798': NtO(MH='Angiomatosis', MS='A condition with multiple tumor-like lesions caused either by congenital or developmental malformations of BLOOD VESSELS, or reactive vascular proliferations, such as in bacillary angiomatosis. Angiomatosis is considered non-neoplastic.'),
  'D000799': NtO(MH='Angioedema', MS='Swelling involving the deep DERMIS, subcutaneous, or submucosal tissues, representing localized EDEMA. Angioedema often occurs in the face, lips, tongue, and larynx.'),
  'D000800': NtO(MH='Angioplasty, Balloon', MS='Use of a balloon catheter for dilation of an occluded artery. It is used in treatment of arterial occlusive diseases, including renal artery stenosis and arterial occlusions in the leg. For the specific technique of BALLOON DILATION in coronary arteries, ANGIOPLASTY, BALLOON, CORONARY is available.'),
  'D000801': NtO(MH='Angiostrongylus', MS='A genus of parasitic nematodes of the superfamily METASTRONGYLOIDEA. Two species, ANGIOSTRONGYLUS CANTONENSIS and A. vasorum, infest the lungs of rats and dogs, respectively. A. cantonensis is transmissible to man where it causes frequently fatal infection of the central nervous system.'),
  'D000802': NtO(MH='Angiotensin Amide', MS='The octapeptide amide of bovine angiotensin II used to increase blood pressure by vasoconstriction.'),
  'D000803': NtO(MH='Angiotensin I', MS='A decapeptide that is cleaved from precursor angiotensinogen by RENIN. Angiotensin I has limited biological activity. It is converted to angiotensin II, a potent vasoconstrictor, after the removal of two amino acids at the C-terminal by ANGIOTENSIN CONVERTING ENZYME.'),
  'D000804': NtO(MH='Angiotensin II', MS='An octapeptide that is a potent but labile vasoconstrictor. It is produced from angiotensin I after the removal of two amino acids at the C-terminal by ANGIOTENSIN CONVERTING ENZYME. The amino acid in position 5 varies in different species. To block VASOCONSTRICTION and HYPERTENSION effect of angiotensin II, patients are often treated with ACE INHIBITORS or with ANGIOTENSIN II TYPE 1 RECEPTOR BLOCKERS.'),
  'D000805': NtO(MH='Angiotensin III', MS='A heptapeptide formed from ANGIOTENSIN II after the removal of an amino acid at the N-terminal by AMINOPEPTIDASE A. Angiotensin III has the same efficacy as ANGIOTENSIN II in promoting ALDOSTERONE secretion and modifying renal blood flow, but less vasopressor activity (about 40%).'),
  'D000806': NtO(MH='Angiotensin-Converting Enzyme Inhibitors', MS='A class of drugs whose main indications are the treatment of hypertension and heart failure. They exert their hemodynamic effect mainly by inhibiting the renin-angiotensin system. They also modulate sympathetic nervous system activity and increase prostaglandin synthesis. They cause mainly vasodilation and mild natriuresis without affecting heart rate and contractility.'),
  'D000808': NtO(MH='Angiotensinogen', MS='An alpha-globulin of about 453 amino acids, depending on the species. It is produced by the liver and secreted into blood circulation. Angiotensinogen is the inactive precursor of natural angiotensins. Upon successive enzyme cleavages, angiotensinogen yields angiotensin I, II, and III with amino acids numbered at 10, 8, and 7, respectively.'),
  'D000809': NtO(MH='Angiotensins', MS='Oligopeptides which are important in the regulation of blood pressure (VASOCONSTRICTION) and fluid homeostasis via the RENIN-ANGIOTENSIN SYSTEM. These include angiotensins derived naturally from precursor ANGIOTENSINOGEN, and those synthesized.'),
  'D000810': NtO(MH='Angola', MS='A republic in southern Africa, southwest of DEMOCRATIC REPUBLIC OF THE CONGO and west of ZAMBIA. Its capital is Luanda.'),
  'D000811': NtO(MH='Anguilla', MS='A genus in the Family Anguillidae, Order Anguilliformes, that includes fresh water eels.'),
  'D000812': NtO(MH='Anhydrides', MS='Chemical compounds derived from acids by the elimination of a molecule of water.'),
  'D000813': NtO(MH='Anilides', MS='Any aromatic amide obtained by acylation of aniline.'),
  'D000814': NtO(MH='Aniline Compounds', MS='Compounds that include the aminobenzene structure.'),
  'D000815': NtO(MH='Aniline Hydroxylase', MS='A drug-metabolizing, cytochrome P-450 enzyme which catalyzes the hydroxylation of aniline to hydroxyaniline in the presence of reduced flavoprotein and molecular oxygen. EC 1.14.14.-.'),
  'D000816': NtO(MH='Aniline Mustard', MS='Alkylating anti-neoplastic agent.'),
  'D000817': NtO(MH='Anilino Naphthalenesulfonates', MS='A class of organic compounds which contain an anilino (phenylamino) group linked to a salt or ester of naphthalenesulfonic acid. They are frequently used as fluorescent dyes and sulfhydryl reagents.'),
  'D000818': NtO(MH='Animals', MS='Unicellular or multicellular, heterotrophic organisms, that have sensation and the power of voluntary movement. Under the older five kingdom paradigm, Animalia was one of the kingdoms. Under the modern three domain model, Animalia represents one of the many groups in the domain EUKARYOTA.'),
  'D000819': NtO(MH='Animal Communication', MS='Communication between animals involving the giving off by one individual of some chemical or physical signal, that, on being received by another, influences its behavior.'),
  'D000820': NtO(MH='Animal Diseases', MS='Diseases that occur in VERTEBRATE animals.'),
  'D000821': NtO(MH='Animal Feed', MS='Foodstuff used especially for domestic and laboratory animals, or livestock.'),
  'D000822': NtO(MH='Animal Husbandry', MS='The science of breeding, feeding and care of domestic animals; includes housing and nutrition.'),
  'D000823': NtO(MH='Animal Identification Systems', MS='Procedures for recognizing individual animals and certain identifiable characteristics pertaining to them; includes computerized methods, ear tags, etc.'),
  'D000824': NtO(MH='Animal Nutritional Physiological Phenomena', MS='Nutritional physiology of animals.'),
  'D000825': NtO(MH='Animal Structures', MS='Organs and other anatomical structures of non-human vertebrate and invertebrate animals.'),
  'D000826': NtO(MH='Animal Testing Alternatives', MS='Procedures, such as TISSUE CULTURE TECHNIQUES; mathematical models; etc., when used or advocated for use in place of the use of animals in research or diagnostic laboratories.'),
  'D000827': NtO(MH='Animal Welfare', MS='The protection of animals in laboratories or other specific environments by promoting their health through better nutrition, housing, and care.'),
  'D000828': NtO(MH='Animal Population Groups', MS='Animals grouped according to ecological, morphological or genetic populations.'),
  'D000829': NtO(MH='Animals, Domestic', MS='Animals which have become adapted through breeding in captivity to a life intimately associated with humans. They include animals domesticated by humans to live and breed in a tame condition on farms or ranches for economic reasons, including LIVESTOCK (specifically CATTLE; SHEEP; HORSES; etc.), POULTRY; and those raised or kept for pleasure and companionship, e.g., PETS; or specifically DOGS; CATS; etc.'),
  'D000830': NtO(MH='Animals, Laboratory', MS='Animals used or intended for use in research, testing, or teaching.'),
  'D000831': NtO(MH='Animals, Newborn', MS='Refers to animals in the period of time just after birth.'),
  'D000832': NtO(MH='Animals, Poisonous', MS='Animal population groups that produce BIOLOGICAL TOXINS.'),
  'D000833': NtO(MH='Animals, Suckling', MS='Young, unweaned mammals. Refers to nursing animals whether nourished by their biological mother, foster mother, or bottle fed.'),
  'D000835': NtO(MH='Animals, Wild', MS='Animals considered to be wild or feral or not adapted for domestic use. It does not include wild animals in zoos for which ANIMALS, ZOO is available.'),
  'D000836': NtO(MH='Animals, Zoo', MS='Animal population groups or individual animals that reside in captivity at a zoological park.'),
  'D000837': NtO(MH='Anion Exchange Resins', MS='High-molecular-weight insoluble polymers that contain functional cationic groups capable of undergoing exchange reactions with anions.'),
  'D000838': NtO(MH='Anions', MS='Negatively charged atoms, radicals or groups of atoms which travel to the anode or positive pole during electrolysis.'),
  'D000839': NtO(MH='Aniseikonia', MS='A condition in which the ocular image of an object as seen by one eye differs in size and shape from that seen by the other.'),
  'D000840': NtO(MH='Anisoles', MS='A group of compounds that are derivatives of methoxybenzene and contain the general formula R-C7H7O.'),
  'D000841': NtO(MH='Anisomycin', MS='An antibiotic isolated from various Streptomyces species. It interferes with protein and DNA synthesis by inhibiting peptidyl transferase or the 80S ribosome system.'),
  'D000842': NtO(MH='Ankle', MS='The region of the lower limb between the FOOT and the LEG.'),
  'D000843': NtO(MH='Ankle Joint', MS='The joint that is formed by the inferior articular and malleolar articular surfaces of the TIBIA; the malleolar articular surface of the FIBULA; and the medial malleolar, lateral malleolar, and superior surfaces of the TALUS.'),
  'D000844': NtO(MH='Ankylosis', MS='Fixation and immobility of a joint.'),
  'D000845': NtO(MH='Annelida', MS='A phylum of metazoan invertebrates comprising the segmented worms, and including marine annelids (POLYCHAETA), freshwater annelids, earthworms (OLIGOCHAETA), and LEECHES. Only the leeches are of medical interest. (Dorland, 27th ed)'),
  'D000846': NtO(MH='Anniversaries and Special Events', MS='Occasions to commemorate an event or occasions designated for a specific purpose.'),
  'D000847': NtO(MH='Annual Reports as Topic', MS='Works on annual statements reviewing the status of the administrative and operational functions and accomplishments of an institution or organization.'),
  'D000848': NtO(MH='Anodontia', MS='Congenital absence of the teeth; it may involve all (total anodontia) or only some of the teeth (partial anodontia, hypodontia), and both the deciduous and the permanent dentition, or only teeth of the permanent dentition. (Dorland, 27th ed)'),
  'D000849': NtO(MH='Anomia', MS='A language dysfunction characterized by the inability to name people and objects that are correctly perceived. The individual is able to describe the object in question, but cannot provide the name. This condition is associated with lesions of the dominant hemisphere involving the language areas, in particular the TEMPORAL LOBE. (From Adams et al., Principles of Neurology, 6th ed, p484)'),
  'D000850': NtO(MH='Anomie', MS='A state of social disorganization and demoralization in society which is largely the result of disharmony between cultural goals and the means for attaining them. This may be reflected in the behavior of the individual in many ways - non-conformity, social withdrawal, deviant behavior, etc.'),
  'D000851': NtO(MH='Anonyms and Pseudonyms', MS='Designations for persons whose names are not known or who wish to remain anonymous (anonyms) and for persons who wish to conceal or obscure their identity by assuming a fictitious name (pseudonyms).'),
  'D000852': NtO(MH='Anopheles', MS='A genus of mosquitoes (CULICIDAE) that are known vectors of MALARIA.'),
  'D000853': NtO(MH='Anophthalmos', MS='Congenital absence of the eye or eyes.'),
  'D000854': NtO(MH='Anoplura', MS='An order of insects comprising the sucking lice, which are blood-sucking ectoparasites of mammals. Recognized families include: Echinphthiriidae, Haematopinidae, and Pediculidae. The latter contains the medically important genera affecting humans: PEDICULUS and PHTHIRUS.'),
  'D000855': NtO(MH='Anorexia', MS='The lack or loss of APPETITE accompanied by an aversion to food and the inability to eat. It is the defining characteristic of the disorder ANOREXIA NERVOSA.'),
  'D000856': NtO(MH='Anorexia Nervosa', MS='An eating disorder that is characterized by the lack or loss of APPETITE, known as ANOREXIA. Other features include excess fear of becoming OVERWEIGHT; BODY IMAGE disturbance; significant WEIGHT LOSS; refusal to maintain minimal normal weight; and AMENORRHEA. This disorder occurs most frequently in adolescent females. (APA, Thesaurus of Psychological Index Terms, 1994)'),
  'D000857': NtO(MH='Olfaction Disorders', MS='Loss of or impaired ability to smell. This may be caused by OLFACTORY NERVE DISEASES; PARANASAL SINUS DISEASES; viral RESPIRATORY TRACT INFECTIONS; CRANIOCEREBRAL TRAUMA; SMOKING; and other conditions.'),
  'D000858': NtO(MH='Anovulation', MS='Suspension or cessation of OVULATION in animals or humans with follicle-containing ovaries (OVARIAN FOLLICLE). Depending on the etiology, OVULATION may be induced with appropriate therapy.'),
  'D000860': NtO(MH='Hypoxia', MS='Sub-optimal OXYGEN levels in the ambient air of living organisms.'),
  'D000861': NtO(MH='Anserine', MS='A dipeptide containing BETA-ALANINE.'),
  'D000862': NtO(MH='Ant Venoms', MS='Venoms from the superfamily Formicoidea, Ants. They may contain protein factors and toxins, histamine, enzymes, and alkaloids and are often allergenic or immunogenic.'),
  'D000863': NtO(MH='Antacids', MS='Substances that counteract or neutralize acidity of the GASTROINTESTINAL TRACT.'),
  'D000864': NtO(MH='Antarctic Regions', MS="The continent lying around the South Pole and the southern waters of the Atlantic, Pacific, and Indian Oceans. It includes the Falkland Islands Dependencies. (From Webster's New Geographical Dictionary, 1988, p55)"),
  'D000865': NtO(MH='Antazoline', MS='An antagonist of histamine H1 receptors.'),
  'D000866': NtO(MH='Antelopes', MS='Any of various ruminant mammals of the order Bovidae. They include numerous species in Africa and the American pronghorn.'),
  'D000867': NtO(MH='Anterior Chamber', MS='The space in the eye, filled with aqueous humor, bounded anteriorly by the cornea and a small portion of the sclera and posteriorly by a small portion of the ciliary body, the iris, and that part of the crystalline lens which presents through the pupil. (Cline et al., Dictionary of Visual Science, 4th ed, p109)'),
  'D000868': NtO(MH='Anterior Compartment Syndrome', MS='Rapid swelling, increased tension, pain, and ischemic necrosis of the muscles of the anterior tibial compartment of the leg, often following excessive PHYSICAL EXERTION.'),
  'D000869': NtO(MH='Anterior Eye Segment', MS='The front third of the eyeball that includes the structures between the front surface of the cornea and the front of the VITREOUS BODY.'),
  'D000870': NtO(MH='Anterior Horn Cells', MS='MOTOR NEURONS in the anterior (ventral) horn of the SPINAL CORD which project to SKELETAL MUSCLES.'),
  'D000871': NtO(MH='Anthelmintics', MS='Agents that kill parasitic worms. They are used therapeutically in the treatment of HELMINTHIASIS in man and animal.'),
  'D000872': NtO(MH='Anthocyanins', MS='A group of FLAVONOIDS derived from FLAVONOLS, which lack the ketone oxygen at the 4-position. They are glycosylated versions of cyanidin, pelargonidin or delphinidin. The conjugated bonds result in blue, red, and purple colors in flowers of plants.'),
  'D000873': NtO(MH='Anthracenes', MS='A group of compounds with three aromatic rings joined in linear arrangement.'),
  'D000874': NtO(MH='Anthracosilicosis', MS='A form of pneumoconiosis caused by inhalation of dust that contains both CARBON and crystalline SILICON DIOXIDE. These foreign matters induce fibrous nodule formation in the lung.'),
  'D000875': NtO(MH='Anthralin', MS='An anthracene derivative that disrupts MITOCHONDRIA function and structure and is used for the treatment of DERMATOSES, especially PSORIASIS. It may cause FOLLICULITIS.'),
  'D000876': NtO(MH='Anthramycin', MS='A broad-spectrum spectrum antineoplastic antibiotic isolated from Streptomyces refuineus var. thermotolerans. It has low toxicity, some activity against Trichomonas and Endamoeba, and inhibits RNA and DNA synthesis. It binds irreversibly to DNA.'),
  'D000877': NtO(MH='Anthranilate Phosphoribosyltransferase', MS="An enzyme that catalyzes the formation of N-5'-phosphoribosylanthranilic acid from anthranilate and phosphoribosylpyrophosphate, the first step in tryptophan synthesis in E. coli. It exists in a complex with ANTHRANILATE SYNTHASE in bacteria. EC 2.4.2.18."),
  'D000878': NtO(MH='Anthranilate Synthase', MS='An enzyme that catalyzes the formation of anthranilate (o-aminobenzoate) and pyruvic acid from chorismate and glutamine. Anthranilate is the biosynthetic precursor of tryptophan and numerous secondary metabolites, including inducible plant defense compounds. EC 4.1.3.27.'),
  'D000880': NtO(MH='Anthraquinones', MS='Compounds based on ANTHRACENES which contain two KETONES in any position. Substitutions can be in any position except on the ketone groups.'),
  'D000881': NtO(MH='Anthrax', MS='An acute infection caused by the spore-forming bacteria BACILLUS ANTHRACIS. It commonly affects hoofed animals such as sheep and goats. Infection in humans often involves the skin (cutaneous anthrax), the lungs (inhalation anthrax), or the gastrointestinal tract. Anthrax is not contagious and can be treated with antibiotics.'),
  'D000882': NtO(MH='Haplorhini', MS='A suborder of PRIMATES consisting of six families: CEBIDAE (some New World monkeys), ATELIDAE (some New World monkeys), CERCOPITHECIDAE (Old World monkeys), HYLOBATIDAE (gibbons and siamangs), CALLITRICHINAE (marmosets and tamarins), and HOMINIDAE (humans and great apes).'),
  'D000883': NtO(MH='Anthropology', MS='The science devoted to the comparative study of man.'),
  'D000884': NtO(MH='Anthropology, Cultural', MS='Study of social phenomena which characterize the learned, shared, and transmitted social activities of particular ethnic groups with focus on the causes, consequences, and complexities of human social and cultural variability.'),
  'D000885': NtO(MH='Anthropology, Physical', MS='The comparative science dealing with the physical characteristics of humans as related to their origin, evolution, and development in the total environment.'),
  'D000886': NtO(MH='Anthropometry', MS='The technique that deals with the measurement of the size, weight, and proportions of the human or other primate body.'),
  'D000887': NtO(MH='Anthroposophy', MS='Knowledge of the nature of man. A spiritual and mystical doctrine that grew out of theosophy and derives mainly from the philosophy of Rudolph Steiner, Austrian social philosopher (1861-1925). (Webster, 3d ed)'),
  'D000888': NtO(MH='Antibodies, Anti-Idiotypic', MS='Antibodies which react with the individual structural determinants (idiotopes) on the variable region of other antibodies.'),
  'D000889': NtO(MH='Anti-Arrhythmia Agents', MS='Agents used for the treatment or prevention of cardiac arrhythmias. They may affect the polarization-repolarization phase of the action potential, its excitability or refractoriness, or impulse conduction or membrane responsiveness within cardiac fibers. Anti-arrhythmia agents are often classed into four main groups according to their mechanism of action: sodium channel blockade, beta-adrenergic blockade, repolarization prolongation, or calcium channel blockade.'),
  'D000890': NtO(MH='Anti-Infective Agents', MS='Substances that prevent infectious agents or organisms from spreading or kill infectious agents in order to prevent the spread of infection.'),
  'D000891': NtO(MH='Anti-Infective Agents, Local', MS='Substances used on humans and other animals that destroy harmful microorganisms or inhibit their activity. They are distinguished from DISINFECTANTS, which are used on inanimate objects.'),
  'D000892': NtO(MH='Anti-Infective Agents, Urinary', MS='Substances capable of killing agents causing urinary tract infections or of preventing them from spreading.'),
  'D000893': NtO(MH='Anti-Inflammatory Agents', MS='Substances that reduce or suppress INFLAMMATION.'),
  'D000894': NtO(MH='Anti-Inflammatory Agents, Non-Steroidal', MS='Anti-inflammatory agents that are non-steroidal in nature. In addition to anti-inflammatory actions, they have analgesic, antipyretic, and platelet-inhibitory actions.They act by blocking the synthesis of prostaglandins by inhibiting cyclooxygenase, which converts arachidonic acid to cyclic endoperoxides, precursors of prostaglandins. Inhibition of prostaglandin synthesis accounts for their analgesic, antipyretic, and platelet-inhibitory actions; other mechanisms may contribute to their anti-inflammatory effects.'),
  'D000897': NtO(MH='Anti-Ulcer Agents', MS='Various agents with different action mechanisms used to treat or ameliorate PEPTIC ULCER or irritation of the gastrointestinal tract. This has included ANTIBIOTICS to treat HELICOBACTER INFECTIONS; HISTAMINE H2 ANTAGONISTS to reduce GASTRIC ACID secretion; and ANTACIDS for symptomatic relief.'),
  'D000898': NtO(MH='Antibiosis', MS='A natural association between organisms that is detrimental to at least one of them. This often refers to the production of chemicals by one microorganism that is harmful to another.'),
  'D000899': NtO(MH='Aurodox', MS='Antibiotic obtained from a Streptomyces variant considered as possibly effective against Streptococcus pyogenes infections. It may promote growth in poultry.'),
  'D000900': NtO(MH='Anti-Bacterial Agents', MS='Substances that inhibit the growth or reproduction of BACTERIA.'),
  'D000903': NtO(MH='Antibiotics, Antineoplastic', MS='Chemical substances, produced by microorganisms, inhibiting or preventing the proliferation of neoplasms.'),
  'D000904': NtO(MH='Antibiotics, Antitubercular', MS='Substances obtained from various species of microorganisms that are, alone or in combination with other agents, of use in treating various forms of tuberculosis; most of these agents are merely bacteriostatic, induce resistance in the organisms, and may be toxic.'),
  'D000906': NtO(MH='Antibodies', MS='Immunoglobulin molecules having a specific amino acid sequence by virtue of which they interact only with the ANTIGEN (or a very similar shape) that induced their synthesis in cells of the lymphoid series (especially PLASMA CELLS).'),
  'D000907': NtO(MH='Antibodies, Bacterial', MS='Immunoglobulins produced in a response to BACTERIAL ANTIGENS.'),
  'D000908': NtO(MH='Antibodies, Fungal', MS='Immunoglobulins produced in a response to FUNGAL ANTIGENS.'),
  'D000909': NtO(MH='Antibodies, Helminth', MS='Immunoglobulins produced in a response to HELMINTH ANTIGENS.'),
  'D000910': NtO(MH='Antibodies, Heterophile', MS='Antibodies elicited in a different species from which the antigen originated. These antibodies are directed against a wide variety of interspecies-specific antigens, the best known of which are Forssman, Hanganutziu-Deicher (H-D), and Paul-Bunnell (P-B). Incidence of antibodies to these antigens--i.e., the phenomenon of heterophile antibody response--is useful in the serodiagnosis, pathogenesis, and prognosis of infection and latent infectious states as well as in cancer classification.'),
  'D000911': NtO(MH='Antibodies, Monoclonal', MS='Antibodies produced by a single clone of cells.'),
  'D000912': NtO(MH='Antibodies, Neoplasm', MS='Immunoglobulins induced by antigens specific for tumors other than the normally occurring HISTOCOMPATIBILITY ANTIGENS.'),
  'D000913': NtO(MH='Antibodies, Protozoan', MS='Immunoglobulins produced in a response to PROTOZOAN ANTIGENS.'),
  'D000914': NtO(MH='Antibodies, Viral', MS='Immunoglobulins produced in response to VIRAL ANTIGENS.'),
  'D000915': NtO(MH='Antibody Affinity', MS='A measure of the binding strength between antibody and a simple hapten or antigen determinant. It depends on the closeness of stereochemical fit between antibody combining sites and antigen determinants, on the size of the area of contact between them, and on the distribution of charged and hydrophobic groups. It includes the concept of "avidity," which refers to the strength of the antigen-antibody bond after formation of reversible complexes.'),
  'D000916': NtO(MH='Antibody Diversity', MS='The phenomenon of immense variability characteristic of ANTIBODIES. It enables the IMMUNE SYSTEM to react specifically against the essentially unlimited kinds of ANTIGENS it encounters. Antibody diversity is accounted for by three main theories: (1) the Germ Line Theory, which holds that each antibody-producing cell has genes coding for all possible antibody specificities, but expresses only the one stimulated by antigen; (2) the Somatic Mutation Theory, which holds that antibody-producing cells contain only a few genes, which produce antibody diversity by mutation; and (3) the Gene Rearrangement Theory, which holds that antibody diversity is generated by the rearrangement of IMMUNOGLOBULIN VARIABLE REGION gene segments during the differentiation of the ANTIBODY-PRODUCING CELLS.'),
  'D000917': NtO(MH='Antibody Formation', MS='The production of ANTIBODIES by proliferating and differentiated B-LYMPHOCYTES under stimulation by ANTIGENS.'),
  'D000918': NtO(MH='Antibody Specificity', MS='The property of antibodies which enables them to react with some ANTIGENIC DETERMINANTS and not with others. Specificity is dependent on chemical composition, physical forces, and molecular structure at the binding site.'),
  'D000919': NtO(MH='Antibody-Coated Bacteria Test, Urinary', MS='Fluorescent antibody technique for visualizing antibody-bacteria complexes in urine. The presence or absence of antibody-coated bacteria in urine correlates with localization of urinary tract infection in the kidney or bladder, respectively.'),
  'D000920': NtO(MH='Antibody-Dependent Cell Cytotoxicity', MS='The phenomenon of antibody-mediated target cell destruction by non-sensitized effector cells. The identity of the target cell varies, but it must possess surface IMMUNOGLOBULIN G whose Fc portion is intact. The effector cell is a "killer" cell possessing Fc receptors. It may be a lymphocyte lacking conventional B- or T-cell markers, or a monocyte, macrophage, or polynuclear leukocyte, depending on the identity of the target cell. The reaction is complement-independent.'),
  'D000921': NtO(MH='Antibody-Producing Cells', MS='Cells of the lymphoid series that can react with antigen to produce specific cell products called antibodies. Various cell subpopulations, often B-lymphocytes, can be defined, based on the different classes of immunoglobulins that they synthesize.'),
  'D000922': NtO(MH='Immunotoxins', MS='Semisynthetic conjugates of various toxic molecules, including RADIOACTIVE ISOTOPES and bacterial or plant toxins, with specific immune substances such as IMMUNOGLOBULINS; MONOCLONAL ANTIBODIES; and ANTIGENS. The antitumor or antiviral immune substance carries the toxin to the tumor or infected cell where the toxin exerts its poisonous effect.'),
  'D000923': NtO(MH='Anticestodal Agents', MS='Agents used to treat tapeworm infestations in man or animals.'),
  'D000924': NtO(MH='Anticholesteremic Agents', MS='Substances used to lower plasma CHOLESTEROL levels.'),
  'D000925': NtO(MH='Anticoagulants', MS='Agents that prevent BLOOD CLOTTING.'),
  'D000926': NtO(MH='Anticodon', MS='The sequential set of three nucleotides in TRANSFER RNA that interacts with its complement in MESSENGER RNA, the CODON, during translation in the ribosome.'),
  'D000927': NtO(MH='Anticonvulsants', MS='Drugs used to prevent SEIZURES or reduce their severity.'),
  'D000928': NtO(MH='Antidepressive Agents', MS='Mood-stimulating drugs used primarily in the treatment of affective disorders and related conditions. Several MONOAMINE OXIDASE INHIBITORS are useful as antidepressants apparently as a long-term consequence of their modulation of catecholamine levels. The tricyclic compounds useful as antidepressive agents (ANTIDEPRESSIVE AGENTS, TRICYCLIC) also appear to act through brain catecholamine systems. A third group (ANTIDEPRESSIVE AGENTS, SECOND-GENERATION) is a diverse group of drugs including some that act specifically on serotonergic systems.'),
  'D000929': NtO(MH='Antidepressive Agents, Tricyclic', MS='Substances that contain a fused three-ring moiety and are used in the treatment of depression. These drugs block the uptake of norepinephrine and serotonin into axon terminals and may block some subtypes of serotonin, adrenergic, and histamine receptors. However the mechanism of their antidepressant effects is not clear because the therapeutic effects usually take weeks to develop and may reflect compensatory changes in the central nervous system.'),
  'D000930': NtO(MH='Antidiarrheals', MS='Miscellaneous agents found useful in the symptomatic treatment of diarrhea. They have no effect on the agent(s) that cause diarrhea, but merely alleviate the condition.'),
  'D000931': NtO(MH='Antidotes', MS='Agents counteracting or neutralizing the action of POISONS.'),
  'D000932': NtO(MH='Antiemetics', MS='Drugs used to prevent NAUSEA or VOMITING.'),
  'D000933': NtO(MH='Antifibrinolytic Agents', MS='Agents that prevent fibrinolysis or lysis of a blood clot or thrombus. Several endogenous antiplasmins are known. The drugs are used to control massive hemorrhage and in other coagulation disorders.'),
  'D000934': NtO(MH='Antifoaming Agents', MS='Agents used to prevent the formation of foam or to treat flatulence or bloat.'),
  'D000935': NtO(MH='Antifungal Agents', MS='Substances that destroy fungi by suppressing their ability to grow or reproduce. They differ from FUNGICIDES, INDUSTRIAL because they defend against fungi present in human or animal tissues.'),
  'D000936': NtO(MH='Antigen-Antibody Complex', MS='The complex formed by the binding of antigen and antibody molecules. The deposition of large antigen-antibody complexes leading to tissue damage causes IMMUNE COMPLEX DISEASES.'),
  'D000937': NtO(MH='Antigen-Antibody Reactions', MS='The processes triggered by interactions of ANTIBODIES with their ANTIGENS.'),
  'D000938': NtO(MH='Antigen-Presenting Cells', MS='A heterogeneous group of immunocompetent cells that mediate the cellular immune response by processing and presenting antigens to the T-cells. Traditional antigen-presenting cells include MACROPHAGES; DENDRITIC CELLS; LANGERHANS CELLS; and B-LYMPHOCYTES. FOLLICULAR DENDRITIC CELLS are not traditional antigen-presenting cells, but because they hold antigen on their cell surface in the form of IMMUNE COMPLEXES for B-cell recognition they are considered so by some authors.'),
  'D000939': NtO(MH='Epitopes', MS='Sites on an antigen that interact with specific antibodies.'),
  'D000940': NtO(MH='Antigenic Variation', MS='Change in the surface ANTIGEN of a microorganism. There are two different types. One is a phenomenon, especially associated with INFLUENZA VIRUSES, where they undergo spontaneous variation both as slow antigenic drift and sudden emergence of new strains (antigenic shift). The second type is when certain PARASITES, especially trypanosomes, PLASMODIUM, and BORRELIA, survive the immune response of the host by changing the surface coat (antigen switching). (From Herbert et al., The Dictionary of Immunology, 4th ed)'),
  'D000941': NtO(MH='Antigens', MS='Substances that are recognized by the immune system and induce an immune reaction.'),
  'D000942': NtO(MH='Antigens, Bacterial', MS='Substances elaborated by bacteria that have antigenic activity.'),
  'D000943': NtO(MH='Antigens, Differentiation', MS='Antigens expressed primarily on the membranes of living cells during sequential stages of maturation and differentiation. As immunologic markers they have high organ and tissue specificity and are useful as probes in studies of normal cell development as well as neoplastic transformation.'),
  'D000944': NtO(MH='Antigens, Differentiation, B-Lymphocyte', MS='Membrane antigens associated with maturation stages of B-lymphocytes, often expressed in tumors of B-cell origin.'),
  'D000945': NtO(MH='Antigens, Differentiation, T-Lymphocyte', MS='Antigens expressed on the cell membrane of T-lymphocytes during differentiation, activation, and normal and neoplastic transformation. Their phenotypic characterization is important in differential diagnosis and studies of thymic ontogeny and T-cell function.'),
  'D000946': NtO(MH='Antigens, Fungal', MS='Substances of fungal origin that have antigenic activity.'),
  'D000947': NtO(MH='Antigens, Helminth', MS='Any part or derivative of a helminth that elicits an immune reaction. The most commonly seen helminth antigens are those of the schistosomes.'),
  'D000949': NtO(MH='Histocompatibility Antigens Class II', MS='Large, transmembrane, non-covalently linked glycoproteins (alpha and beta). Both chains can be polymorphic although there is more structural variation in the beta chains. The class II antigens in humans are called HLA-D ANTIGENS and are coded by a gene on chromosome 6. In mice, two genes named IA and IE on chromosome 17 code for the H-2 antigens. The antigens are found on B-lymphocytes, macrophages, epidermal cells, and sperm and are thought to mediate the competence of and cellular cooperation in the immune response. The term IA antigens used to refer only to the proteins encoded by the IA genes in the mouse, but is now used as a generic term for any class II histocompatibility antigen.'),
  'D000950': NtO(MH='Antigens, Ly', MS='A group of lymphocyte surface antigens located on mouse LYMPHOCYTES. Specific Ly antigens are useful markers for distinguishing subpopulations of lymphocytes.'),
  'D000951': NtO(MH='Antigens, Neoplasm', MS='Proteins, glycoprotein, or lipoprotein moieties on surfaces of tumor cells that are usually identified by monoclonal antibodies. Many of these are of either embryonic or viral origin.'),
  'D000952': NtO(MH='Antigens, Polyomavirus Transforming', MS='Polyomavirus antigens which cause infection and cellular transformation. The large T antigen is necessary for the initiation of viral DNA synthesis, repression of transcription of the early region and is responsible in conjunction with the middle T antigen for the transformation of primary cells. Small T antigen is necessary for the completion of the productive infection cycle.'),
  'D000953': NtO(MH='Antigens, Protozoan', MS='Any part or derivative of any protozoan that elicits immunity; malaria (Plasmodium) and trypanosome antigens are presently the most frequently encountered.'),
  'D000954': NtO(MH='Antigens, Surface', MS='Antigens on surfaces of cells, including infectious or foreign cells or viruses. They are usually protein-containing groups on cell membranes or walls and may be isolated.'),
  'D000955': NtO(MH='Antigens, T-Independent', MS='Antigens which may directly stimulate B lymphocytes without the cooperation of T lymphocytes.'),
  'D000956': NtO(MH='Antigens, Viral', MS='Substances elaborated by viruses that have antigenic activity.'),
  'D000957': NtO(MH='Antigens, Viral, Tumor', MS='Those proteins recognized by antibodies from serum of animals bearing tumors induced by viruses; these proteins are presumably coded for by the nucleic acids of the same viruses that caused the neoplastic transformation.'),
  'D000958': NtO(MH='Antigua and Barbuda', MS='Islands in the Lesser Antilles, within the Leeward Islands. ANTIGUA, BARBUDA, and Redonda, an uninhabited island, constitute the independent state of ANTIGUA AND BARBUDA. The capital is St. Johns.'),
  'D000959': NtO(MH='Antihypertensive Agents', MS='Drugs used in the treatment of acute or chronic vascular HYPERTENSION regardless of pharmacological mechanism. Among the antihypertensive agents are DIURETICS; (especially DIURETICS, THIAZIDE); ADRENERGIC BETA-ANTAGONISTS; ADRENERGIC ALPHA-ANTAGONISTS; ANGIOTENSIN-CONVERTING ENZYME INHIBITORS; CALCIUM CHANNEL BLOCKERS; GANGLIONIC BLOCKERS; and VASODILATOR AGENTS.'),
  'D000960': NtO(MH='Hypolipidemic Agents', MS='Substances that lower the levels of certain LIPIDS in the BLOOD. They are used to treat HYPERLIPIDEMIAS.'),
  'D000961': NtO(MH='Antilymphocyte Serum', MS='Serum containing GAMMA-GLOBULINS which are antibodies for lymphocyte ANTIGENS. It is used both as a test for HISTOCOMPATIBILITY and therapeutically in TRANSPLANTATION.'),
  'D000962': NtO(MH='Antimalarials', MS='Agents used in the treatment of malaria. They are usually classified on the basis of their action against plasmodia at different stages in their life cycle in the human. (From AMA, Drug Evaluations Annual, 1992, p1585)'),
  'D000963': NtO(MH='Antimetabolites', MS='Drugs that are chemically similar to naturally occurring metabolites, but differ enough to interfere with normal metabolic pathways. (From AMA Drug Evaluations Annual, 1994, p2033)'),
  'D000964': NtO(MH='Antimetabolites, Antineoplastic', MS='Antimetabolites that are useful in cancer chemotherapy.'),
  'D000965': NtO(MH='Antimony', MS='A metallic element that has the atomic symbol Sb, atomic number 51, and atomic weight 121.75. It is used as a metal alloy and as medicinal and poisonous salts. It is toxic and an irritant to the skin and the mucous membranes.'),
  'D000966': NtO(MH='Antimony Potassium Tartrate', MS='A schistosomicide possibly useful against other parasites. It has irritant emetic properties and may cause lethal cardiac toxicity among other adverse effects.'),
  'D000967': NtO(MH='Antimony Sodium Gluconate', MS='Antimony complex where the metal may exist in either the pentavalent or trivalent states. The pentavalent gluconate is used in leishmaniasis. The trivalent gluconate is most frequently used in schistosomiasis.'),
  'D000968': NtO(MH='Antimycin A', MS='An antibiotic substance produced by Streptomyces species. It inhibits mitochondrial respiration and may deplete cellular levels of ATP. Antimycin A1 has been used as a fungicide, insecticide, and miticide. (From Merck Index, 12th ed)'),
  'D000969': NtO(MH='Antinematodal Agents', MS='Substances used in the treatment or control of nematode infestations. They are used also in veterinary practice.'),
  'D000970': NtO(MH='Antineoplastic Agents', MS='Substances that inhibit or prevent the proliferation of NEOPLASMS.'),
  'D000971': NtO(MH='Antineoplastic Combined Chemotherapy Protocols', MS='The use of two or more chemicals simultaneously or sequentially in the drug therapy of neoplasms. The drugs need not be in the same dosage form.'),
  'D000972': NtO(MH='Antineoplastic Agents, Phytogenic', MS='Agents obtained from higher plants that have demonstrable cytostatic or antineoplastic activity.'),
  'D000974': NtO(MH='Antibodies, Antinuclear', MS="Autoantibodies directed against various nuclear antigens including DNA, RNA, histones, acidic nuclear proteins, or complexes of these molecular elements. Antinuclear antibodies are found in systemic autoimmune diseases including systemic lupus erythematosus, Sjogren's syndrome, scleroderma, polymyositis, and mixed connective tissue disease."),
  'D000975': NtO(MH='Antioxidants', MS='Naturally occurring or synthetic substances that inhibit or retard oxidation reactions. They counteract the damaging effects of oxidation in animal tissues.'),
  'D000976': NtO(MH='Antipain', MS='An oligopeptide produced by various bacteria which acts as a protease inhibitor.'),
  'D000977': NtO(MH='Antiparasitic Agents', MS='Drugs used to treat or prevent parasitic infections.'),
  'D000978': NtO(MH='Antiparkinson Agents', MS="Agents used in the treatment of Parkinson's disease. The most commonly used drugs act on the dopaminergic system in the striatum and basal ganglia or are centrally acting muscarinic antagonists."),
  'D000979': NtO(MH='alpha-2-Antiplasmin', MS='A member of the serpin superfamily found in plasma that inhibits the lysis of fibrin clots which are induced by plasminogen activator. It is a glycoprotein, molecular weight approximately 70,000 that migrates in the alpha 2 region in immunoelectrophoresis. It is the principal plasmin inactivator in blood, rapidly forming a very stable complex with plasmin.'),
  'D000980': NtO(MH='Antiplatyhelmintic Agents', MS='Agents used to treat cestode, trematode, or other flatworm infestations in man or animals.'),
  'D000981': NtO(MH='Antiprotozoal Agents', MS='Substances that are destructive to protozoans.'),
  'D000982': NtO(MH='Antipruritics', MS='Agents, usually topical, that relieve itching (pruritus).'),
  'D000983': NtO(MH='Antipyrine', MS='An analgesic and antipyretic that has been given by mouth and as ear drops. Antipyrine is often used in testing the effects of other drugs or diseases on drug-metabolizing enzymes in the liver. (From Martindale, The Extra Pharmacopoeia, 30th ed, p29)'),
  'D000985': NtO(MH='Antisepsis', MS='The destruction of germs causing disease.'),
  'D000986': NtO(MH='Antisickling Agents', MS='Agents used to prevent or reverse the pathological events leading to sickling of erythrocytes in sickle cell conditions.'),
  'D000987': NtO(MH='Antisocial Personality Disorder', MS='A personality disorder whose essential feature is a pervasive pattern of disregard for, and violation of, the rights of others that begins in childhood or early adolescence and continues into adulthood. The individual must be at least age 18 and must have a history of some symptoms of CONDUCT DISORDER before age 15. (From DSM-IV, 1994)'),
  'D000988': NtO(MH='Antispermatogenic Agents', MS='Agents, either mechanical or chemical, which destroy spermatozoa in the male genitalia and block spermatogenesis.'),
  'D000989': NtO(MH='Antistreptolysin', MS='Antibodies specific to STREPTOLYSINS which indicate STREPTOCOCCAL INFECTIONS.'),
  'D000990': NtO(MH='Antithrombin III', MS='A plasma alpha 2 glycoprotein that accounts for the major antithrombin activity of normal plasma and also inhibits several other enzymes. It is a member of the serpin superfamily.'),
  'D000991': NtO(MH='Antithrombins', MS='Endogenous factors and drugs that directly inhibit the action of THROMBIN, usually by blocking its enzymatic activity. They are distinguished from INDIRECT THROMBIN INHIBITORS, such as HEPARIN, which act by enhancing the inhibitory effects of antithrombins.'),
  'D000992': NtO(MH='Antitoxins', MS='Antisera from immunized animals that is purified and used as a passive immunizing agent against specific BACTERIAL TOXINS.'),
  'D000993': NtO(MH='Antitreponemal Agents', MS='Agents used to treat infections with bacteria of the genus TREPONEMA. This includes SYPHILIS & YAWS.'),
  'D000994': NtO(MH='Antitrichomonal Agents', MS='Agents used to treat trichomonas infections.'),
  'D000995': NtO(MH='Antitubercular Agents', MS='Drugs used in the treatment of tuberculosis. They are divided into two main classes: "first-line" agents, those with the greatest efficacy and acceptable degrees of toxicity used successfully in the great majority of cases; and "second-line" drugs used in drug-resistant cases or those in which some other patient-related condition has compromised the effectiveness of primary therapy.'),
  'D000996': NtO(MH='Antitussive Agents', MS='Agents that suppress cough. They act centrally on the medullary cough center. EXPECTORANTS, also used in the treatment of cough, act locally.'),
  'D000997': NtO(MH='Antivenins', MS='Antisera used to counteract poisoning by animal VENOMS, especially SNAKE VENOMS.'),
  'D000998': NtO(MH='Antiviral Agents', MS='Agents used in the prophylaxis or therapy of VIRUS DISEASES. Some of the ways they may act include preventing viral replication by inhibiting viral DNA polymerase; binding to specific cell-surface receptors and inhibiting viral penetration or uncoating; inhibiting viral protein synthesis; or blocking late stages of virus assembly.'),
  'D000999': NtO(MH='Antlers', MS='The horn of an animal of the deer family, typically present only in the male. It differs from the HORNS of other animals in being a solid, generally branched bony outgrowth that is shed and renewed annually. The word antler comes from the Latin anteocularis, ante (before) + oculus (eye). (From Webster, 3d ed)'),
  'D001000': NtO(MH='Ants', MS='Insects of the family Formicidae, very common and widespread, probably the most successful of all the insect groups. All ants are social insects, and most colonies contain three castes, queens, males, and workers. Their habits are often very elaborate and a great many studies have been made of ant behavior. Ants produce a number of secretions that function in offense, defense, and communication. (From Borror, et al., An Introduction to the Study of Insects, 4th ed, p676)'),
  'D001001': NtO(MH='Anura', MS='An order of the class Amphibia, which includes several families of frogs and toads. They are characterized by well developed hind limbs adapted for jumping, fused head and trunk and webbed toes. The term "toad" is ambiguous and is properly applied only to the family Bufonidae.'),
  'D001002': NtO(MH='Anuria', MS='Absence of urine formation. It is usually associated with complete bilateral ureteral (URETER) obstruction, complete lower urinary tract obstruction, or unilateral ureteral obstruction when a solitary kidney is present.'),
  'D001003': NtO(MH='Anal Canal', MS='The terminal segment of the LARGE INTESTINE, beginning from the ampulla of the RECTUM and ending at the anus.'),
  'D001004': NtO(MH='Anus Diseases', MS='Diseases involving the ANUS.'),
  'D001005': NtO(MH='Anus Neoplasms', MS='Tumors or cancer of the ANAL CANAL.'),
  'D001006': NtO(MH='Anus, Imperforate', MS='A congenital abnormality characterized by the persistence of the anal membrane, resulting in a thin membrane covering the normal ANAL CANAL. Imperforation is not always complete and is treated by surgery in infancy. This defect is often associated with NEURAL TUBE DEFECTS; MENTAL RETARDATION; and DOWN SYNDROME.'),
  'D001007': NtO(MH='Anxiety', MS='Feelings or emotions of dread, apprehension, and impending disaster but not disabling as with ANXIETY DISORDERS.'),
  'D001008': NtO(MH='Anxiety Disorders', MS='Persistent and disabling ANXIETY.'),
  'D001009': NtO(MH='Anxiety, Castration', MS='Anxiety due to fantasized injuries to or loss of the genitals.'),
  'D001010': NtO(MH='Anxiety, Separation', MS='Anxiety experienced by an individual upon separation from a person or object of particular significance to the individual.'),
  'D001011': NtO(MH='Aorta', MS='The main trunk of the systemic arteries.'),
  'D001012': NtO(MH='Aorta, Abdominal', MS='The aorta from the DIAPHRAGM to the bifurcation into the right and left common iliac arteries.'),
  'D001013': NtO(MH='Aorta, Thoracic', MS='The portion of the descending aorta proceeding from the arch of the aorta and extending to the DIAPHRAGM, eventually connecting to the ABDOMINAL AORTA.'),
  'D001014': NtO(MH='Aortic Aneurysm', MS='An abnormal balloon- or sac-like dilatation in the wall of AORTA.'),
  'D001015': NtO(MH='Aortic Arch Syndromes', MS='Conditions resulting from abnormalities in the arteries branching from the ASCENDING AORTA, the curved portion of the aorta. These syndromes are results of occlusion or abnormal blood flow to the head-neck or arm region leading to neurological defects and weakness in an arm. These syndromes are associated with vascular malformations; ATHEROSCLEROSIS; TRAUMA; and blood clots.'),
  'D001016': NtO(MH='Aortic Bodies', MS='Small clusters of chemoreceptive and supporting cells located near the ARCH OF THE AORTA; the PULMONARY ARTERIES; and the CORONARY ARTERIES. The aortic bodies sense PH; CARBON DIOXIDE; and OXYGEN concentrations in the BLOOD and participate in the control of RESPIRATION. The aortic bodies should not be confused with the PARA-AORTIC BODIES in the abdomen (which are sometimes also called aortic bodies).'),
  'D001017': NtO(MH='Aortic Coarctation', MS='A birth defect characterized by the narrowing of the AORTA that can be of varying degree and at any point from the transverse arch to the iliac bifurcation. Aortic coarctation causes arterial HYPERTENSION before the point of narrowing and arterial HYPOTENSION beyond the narrowed portion.'),
  'D001018': NtO(MH='Aortic Diseases', MS='Pathological processes involving any part of the AORTA.'),
  'D001019': NtO(MH='Aortic Rupture', MS='The tearing or bursting of the wall along any portion of the AORTA, such as thoracic or abdominal. It may result from the rupture of an aneurysm or it may be due to TRAUMA.'),
  'D001020': NtO(MH='Aortic Stenosis, Subvalvular', MS='A pathological constriction occurring in the region below the AORTIC VALVE. It is characterized by restricted outflow from the LEFT VENTRICLE into the AORTA.'),
  'D001021': NtO(MH='Aortic Valve', MS='The valve between the left ventricle and the ascending aorta which prevents backflow into the left ventricle.'),
  'D001022': NtO(MH='Aortic Valve Insufficiency', MS='Pathological condition characterized by the backflow of blood from the ASCENDING AORTA back into the LEFT VENTRICLE, leading to regurgitation. It is caused by diseases of the AORTIC VALVE or its surrounding tissue (aortic root).'),
  'D001023': NtO(MH='Aortic Valve Prolapse', MS='The downward displacement of the cuspal or pointed end of the trileaflet AORTIC VALVE causing misalignment of the cusps. Severe valve distortion can cause leakage and allow the backflow of blood from the ASCENDING AORTA back into the LEFT VENTRICLE, leading to aortic regurgitation.'),
  'D001024': NtO(MH='Aortic Valve Stenosis', MS='A pathological constriction that can occur above (supravalvular stenosis), below (subvalvular stenosis), or at the AORTIC VALVE. It is characterized by restricted outflow from the LEFT VENTRICLE into the AORTA.'),
  'D001025': NtO(MH='Aortitis', MS='Inflammation of the wall of the AORTA.'),
  'D001026': NtO(MH='Coronary Artery Bypass', MS='Surgical therapy of ischemic coronary artery disease achieved by grafting a section of saphenous vein, internal mammary artery, or other substitute between the aorta and the obstructed coronary artery distal to the obstructive lesion.'),
  'D001027': NtO(MH='Aortography', MS='Radiographic visualization of the aorta and its branches by injection of contrast media, using percutaneous puncture or catheterization procedures.'),
  'D001028': NtO(MH='Aortopulmonary Septal Defect', MS='A developmental abnormality in which the spiral (aortopulmonary) septum failed to completely divide the TRUNCUS ARTERIOSUS into ASCENDING AORTA and PULMONARY ARTERY. This abnormal communication between the two major vessels usually lies above their respective valves (AORTIC VALVE; PULMONARY VALVE).'),
  'D001029': NtO(MH='Aotus trivirgatus', MS='A species in the family AOTIDAE, inhabiting the forested regions of Central and South America (from Panama to the Amazon). Vocalizations occur primarily at night when they are active, thus they are also known as Northern night monkeys.'),
  'D001030': NtO(MH='Apamin', MS='A highly neurotoxic polypeptide from the venom of the honey bee (Apis mellifera). It consists of 18 amino acids with two disulfide bridges and causes hyperexcitability resulting in convulsions and respiratory paralysis.'),
  'D001031': NtO(MH='Apatites', MS='A group of phosphate minerals that includes ten mineral species and has the general formula X5(YO4)3Z, where X is usually calcium or lead, Y is phosphorus or arsenic, and Z is chlorine, fluorine, or OH-. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D001032': NtO(MH='Apazone', MS='An anti-inflammatory agent used in the treatment of rheumatoid arthritis. It also has uricosuric properties and has been used to treat gout.'),
  'D001034': NtO(MH='Apgar Score', MS="A method, developed by Dr. Virginia Apgar, to evaluate a newborn's adjustment to extrauterine life. Five items - heart rate, respiratory effort, muscle tone, reflex irritability, and color - are evaluated 60 seconds after birth and again five minutes later on a scale from 0-2, 0 being the lowest, 2 being normal. The five numbers are added for the Apgar score. A score of 0-3 represents severe distress, 4-7 indicates moderate distress, and a score of 7-10 predicts an absence of difficulty in adjusting to extrauterine life."),
  'D001035': NtO(MH='Aphakia', MS='Absence of crystalline lens totally or partially from field of vision, from any cause except after cataract extraction. Aphakia is mainly congenital or as result of LENS DISLOCATION AND SUBLUXATION.'),
  'D001036': NtO(MH='Aphakia, Postcataract', MS='Absence of the crystalline lens resulting from cataract extraction.'),
  'D001037': NtO(MH='Aphasia', MS='A cognitive disorder marked by an impaired ability to comprehend or express language in its written or spoken form. This condition is caused by diseases which affect the language areas of the dominant hemisphere. Clinical features are used to classify the various subtypes of this condition. General categories include receptive, expressive, and mixed forms of aphasia.'),
  'D001039': NtO(MH='Aphasia, Broca', MS='An aphasia characterized by impairment of expressive LANGUAGE (speech, writing, signs) and relative preservation of receptive language abilities (i.e., comprehension). This condition is caused by lesions of the motor association cortex in the FRONTAL LOBE (BROCA AREA and adjacent cortical and white matter regions).'),
  'D001041': NtO(MH='Aphasia, Wernicke', MS="Impairment in the comprehension of speech and meaning of words, both spoken and written, and of the meanings conveyed by their grammatical relationships in sentences. It is caused by lesions that primarily affect Wernicke's area, which lies in the posterior perisylvian region of the temporal lobe of the dominant hemisphere. (From Brain & Bannister, Clinical Neurology, 7th ed, p141; Kandel et al., Principles of Neural Science, 3d ed, p846)"),
  'D001042': NtO(MH='Aphids', MS='A family (Aphididae) of small insects, in the suborder Sternorrhyncha, that suck the juices of plants. Important genera include Schizaphis and Myzus. The latter is known to carry more than 100 virus diseases between plants.'),
  'D001044': NtO(MH='Aphonia', MS='Complete loss of phonation due to organic disease of the larynx or to nonorganic (i.e., psychogenic) causes.'),
  'D001045': NtO(MH='Aphorisms and Proverbs as Topic', MS='Works about short popular sayings effectively expressing or astutely professing general truths or useful thoughts.'),
  'D001046': NtO(MH='Aphrodisiacs', MS='Chemical agents or ODORANTS that stimulate sexual desires. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D001047': NtO(MH='Apicoectomy', MS='Excision of the apical portion of a tooth through an opening made in the overlying labial, buccal, or palatal alveolar bone. (Dorland, 28th ed)'),
  'D001048': NtO(MH='Aplysia', MS='An opisthobranch mollusk of the order Anaspidea. It is used frequently in studies of nervous system development because of its large identifiable neurons. Aplysiatoxin and its derivatives are not biosynthesized by Aplysia, but acquired by ingestion of Lyngbya (seaweed) species.'),
  'D001049': NtO(MH='Apnea', MS='A transient absence of spontaneous respiration.'),
  'D001050': NtO(MH='Apocrine Glands', MS='Large, branched, specialized sweat glands that empty into the upper portion of a HAIR FOLLICLE instead of directly onto the SKIN.'),
  'D001051': NtO(MH='Apoenzymes', MS='The protein components of enzyme complexes (HOLOENZYMES). An apoenzyme is the holoenzyme minus any cofactors (ENZYME COFACTORS) or prosthetic groups required for the enzymatic function.'),
  'D001052': NtO(MH='Apoferritins', MS='The protein components of ferritins. Apoferritins are shell-like structures containing nanocavities and ferroxidase activities. Apoferritin shells are composed of 24 subunits, heteropolymers in vertebrates and homopolymers in bacteria. In vertebrates, there are two types of subunits, light chain and heavy chain. The heavy chain contains the ferroxidase activity.'),
  'D001053': NtO(MH='Apolipoproteins', MS='Protein components on the surface of LIPOPROTEINS. They form a layer surrounding the hydrophobic lipid core. There are several classes of apolipoproteins with each playing a different role in lipid transport and LIPID METABOLISM. These proteins are synthesized mainly in the LIVER and the INTESTINES.'),
  'D001054': NtO(MH='Apolipoproteins A', MS='Structural proteins of the alpha-lipoproteins (HIGH DENSITY LIPOPROTEINS), including APOLIPOPROTEIN A-I and APOLIPOPROTEIN A-II. They can modulate the activity of LECITHIN CHOLESTEROL ACYLTRANSFERASE. These apolipoproteins are low in atherosclerotic patients. They are either absent or present in extremely low plasma concentration in TANGIER DISEASE.'),
  'D001055': NtO(MH='Apolipoproteins B', MS='Major structural proteins of triacylglycerol-rich LIPOPROTEINS. There are two forms, apolipoprotein B-100 and apolipoprotein B-48, both derived from a single gene. ApoB-100 expressed in the liver is found in low-density lipoproteins (LIPOPROTEINS, LDL; LIPOPROTEINS, VLDL). ApoB-48 expressed in the intestine is found in CHYLOMICRONS. They are important in the biosynthesis, transport, and metabolism of triacylglycerol-rich lipoproteins. Plasma Apo-B levels are high in atherosclerotic patients but non-detectable in ABETALIPOPROTEINEMIA.'),
  'D001056': NtO(MH='Apolipoproteins C', MS='A group of apolipoproteins that can readily exchange among the various classes of lipoproteins (HDL; VLDL; CHYLOMICRONS). After lipolysis of TRIGLYCERIDES on VLDL and chylomicrons, Apo-C proteins are normally transferred to HDL. The subtypes can modulate remnant binding to receptors, LECITHIN CHOLESTEROL ACYLTRANSFERASE, or LIPOPROTEIN LIPASE.'),
  'D001057': NtO(MH='Apolipoproteins E', MS='A class of protein components which can be found in several lipoproteins including HIGH-DENSITY LIPOPROTEINS; VERY-LOW-DENSITY LIPOPROTEINS; and CHYLOMICRONS. Synthesized in most organs, Apo E is important in the global transport of lipids and cholesterol throughout the body. Apo E is also a ligand for LDL receptors (RECEPTORS, LDL) that mediates the binding, internalization, and catabolism of lipoprotein particles in cells. There are several allelic isoforms (such as E2, E3, and E4). Deficiency or defects in Apo E are causes of HYPERLIPOPROTEINEMIA TYPE III.'),
  'D001058': NtO(MH='Apomorphine', MS='A derivative of morphine that is a dopamine D2 agonist. It is a powerful emetic and has been used for that effect in acute poisoning. It has also been used in the diagnosis and treatment of parkinsonism, but its adverse effects limit its use.'),
  'D001059': NtO(MH='Apoproteins', MS='The protein components of a number of complexes, such as enzymes (APOENZYMES), ferritin (APOFERRITINS), or lipoproteins (APOLIPOPROTEINS).'),
  'D001060': NtO(MH='Aporphines', MS='Dibenzoquinolines derived in plants from (S)-reticuline (BENZYLISOQUINOLINES).'),
  'D001061': NtO(MH='Appalachian Region', MS='A geographical area of the United States with no definite boundaries but comprising northeastern Alabama, northwestern Georgia, northwestern South Carolina, western North Carolina, eastern Kentucky, eastern Tennessee, western Virginia, West Virginia, western Maryland, southwestern Pennsylvania, southern Ohio, and southern New York.'),
  'D001062': NtO(MH='Appendectomy', MS='Surgical removal of the vermiform appendix. (Dorland, 28th ed)'),
  'D001063': NtO(MH='Appendiceal Neoplasms', MS='Tumors or cancer of the APPENDIX.'),
  'D001064': NtO(MH='Appendicitis', MS='Acute inflammation of the APPENDIX. Acute appendicitis is classified as simple, gangrenous, or perforated.'),
  'D001065': NtO(MH='Appendix', MS='A worm-like blind tube extension from the CECUM.'),
  'D001066': NtO(MH='Appetite', MS='Natural recurring desire for food. Alterations may be induced by APPETITE DEPRESSANTS or APPETITE STIMULANTS.'),
  'D001067': NtO(MH='Appetite Depressants', MS='Agents that are used to suppress appetite.'),
  'D001068': NtO(MH='Feeding and Eating Disorders', MS='A group of disorders characterized by physiological and psychological disturbances in appetite or food intake.'),
  'D001069': NtO(MH='Appetite Regulation', MS='Physiologic mechanisms which regulate or control the appetite and food intake.'),
  'D001070': NtO(MH='Appetitive Behavior', MS='Animal searching behavior. The variable introductory phase of an instinctive behavior pattern or sequence, e.g., looking for food, or sequential courtship patterns prior to mating.'),
  'D001071': NtO(MH='Appointments and Schedules', MS='The different methods of scheduling patient visits, appointment systems, individual or group appointments, waiting times, waiting lists for hospitals, walk-in clinics, etc.'),
  'D001072': NtO(MH='Apraxias', MS='A group of cognitive disorders characterized by the inability to perform previously learned skills that cannot be attributed to deficits of motor or sensory function. The two major subtypes of this condition are ideomotor (see APRAXIA, IDEOMOTOR) and ideational apraxia, which refers to loss of the ability to mentally formulate the processes involved with performing an action. For example, dressing apraxia may result from an inability to mentally formulate the act of placing clothes on the body. Apraxias are generally associated with lesions of the dominant PARIETAL LOBE and supramarginal gyrus. (From Adams et al., Principles of Neurology, 6th ed, pp56-7)'),
  'D001073': NtO(MH='Aprindine', MS='A class Ib anti-arrhythmia agent used to manage ventricular and supraventricular arrhythmias.'),
  'D001074': NtO(MH='Propoxur', MS='A carbamate insecticide.'),
  'D001076': NtO(MH='Aptitude', MS='The ability to acquire general or special types of knowledge or skill.'),
  'D001077': NtO(MH='Aptitude Tests', MS="Primarily non-verbal tests designed to predict an individual's future learning ability or performance."),
  'D001078': NtO(MH='APUD Cells', MS='Cells with the capacity to take up and decarboxylate the amine precursors DIHYDROXYPHENYLALANINE or 5-HYDROXYTRYPTOPHAN. This is a property of endocrine cells of neural and non-neural origin. APUDOMA is a general term collectively applied to tumors associated with APUD cells.'),
  'D001079': NtO(MH='Apudoma', MS='A general term collectively applied to tumors associated with the APUD CELLS series, irrespective of their specific identification.'),
  'D001080': NtO(MH='Apurinic Acid', MS='Hydrolysate of DNA in which purine bases have been removed.'),
  'D001081': NtO(MH='Apyrase', MS='A calcium-activated enzyme that catalyzes the hydrolysis of ATP to yield AMP and orthophosphate. It can also act on ADP and other nucleoside triphosphates and diphosphates. EC 3.6.1.5.'),
  'D001082': NtO(MH='Aqueous Humor', MS='The clear, watery fluid which fills the anterior and posterior chambers of the eye. It has a refractive index lower than the crystalline lens, which it surrounds, and is involved in the metabolism of the cornea and the crystalline lens. (Cline et al., Dictionary of Visual Science, 4th ed, p319)'),
  'D001083': NtO(MH='Arabia', MS="The great peninsula of southwest Asia comprising most of the present countries of the Middle East. It has been known since the first millennium B.C. In early times it was divided into Arabia Petraea, the northwest part, the only part ever conquered, becoming a Roman province; Arabia Deserta, the northern part between Syria and Mesopotamia; and Arabia Felix, the main part of the peninsula but by some geographers restricted to modern Yemen. (From Webster's New Geographical Dictionary, 1988, p63)"),
  'D001084': NtO(MH='Vidarabine Phosphate', MS='An adenosine monophosphate analog in which ribose is replaced by an arabinose moiety. It is the monophosphate ester of VIDARABINE with antiviral and possibly antineoplastic properties.'),
  'D001085': NtO(MH='Arabinofuranosylcytosine Triphosphate', MS='A triphosphate nucleotide analog which is the biologically active form of CYTARABINE. It inhibits nuclear DNA synthesis.'),
  'D001086': NtO(MH='Arabinofuranosyluracil', MS='A pyrimidine nucleoside formed in the body by the deamination of CYTARABINE.'),
  'D001087': NtO(MH='Arabinonucleosides', MS='Nucleosides containing arabinose as their sugar moiety.'),
  'D001088': NtO(MH='Arabinonucleotides', MS='Nucleotides containing arabinose as their sugar moiety.'),
  'D001089': NtO(MH='Arabinose', MS=None),
  'D001091': NtO(MH='Arachidonate Lipoxygenases', MS='Enzymes catalyzing the oxidation of arachidonic acid to hydroperoxyarachidonates. These products are then rapidly converted by a peroxidase to hydroxyeicosatetraenoic acids. The positional specificity of the enzyme reaction varies from tissue to tissue. The final lipoxygenase pathway leads to the leukotrienes. EC 1.13.11.- .'),
  'D001092': NtO(MH='Arachidonate 12-Lipoxygenase', MS='An enzyme that catalyzes the oxidation of arachidonic acid to yield 12-hydroperoxyarachidonate (12-HPETE) which is itself rapidly converted by a peroxidase to 12-hydroxy-5,8,10,14-eicosatetraenoate (12-HETE). The 12-hydroperoxides are preferentially formed in PLATELETS.'),
  'D001093': NtO(MH='Arachidonate 15-Lipoxygenase', MS='An enzyme that catalyzes the oxidation of arachidonic acid to yield 15-hydroperoxyarachidonate (15-HPETE) which is rapidly converted to 15-hydroxy-5,8,11,13-eicosatetraenoate (15-HETE). The 15-hydroperoxides are preferentially formed in NEUTROPHILS and LYMPHOCYTES.'),
  'D001094': NtO(MH='Arachidonate 5-Lipoxygenase', MS='An enzyme that catalyzes the oxidation of arachidonic acid to yield 5-hydroperoxyarachidonate (5-HPETE) which is rapidly converted by a peroxidase to 5-hydroxy-6,8,11,14-eicosatetraenoate (5-HETE). The 5-hydroperoxides are preferentially formed in leukocytes.'),
  'D001095': NtO(MH='Arachidonic Acids', MS=None),
  'D001096': NtO(MH='Arachnid Vectors', MS='Members of the class Arachnida, especially SPIDERS; SCORPIONS; MITES; and TICKS; which transmit infective organisms from one host to another or from an inanimate reservoir to an animate host.'),
  'D001097': NtO(MH='Arachnida', MS='A class of Arthropoda that includes SPIDERS; TICKS; MITES; and SCORPIONS.'),
  'D001098': NtO(MH='Spider Bites', MS='The effects, both local and systemic, caused by the bites of SPIDERS.'),
  'D001099': NtO(MH='Arachnoid', MS='A delicate membrane enveloping the brain and spinal cord. It lies between the PIA MATER and the DURA MATER. It is separated from the pia mater by the subarachnoid cavity which is filled with CEREBROSPINAL FLUID.'),
  'D001100': NtO(MH='Arachnoiditis', MS='Acute or chronic inflammation of the arachnoid membrane of the meninges most often involving the spinal cord or base of the brain. This term generally refers to a persistent inflammatory process characterized by thickening of the ARACHNOID membrane and dural adhesions. Associated conditions include prior surgery, infections, trauma, SUBARACHNOID HEMORRHAGE, and chemical irritation. Clinical features vary with the site of inflammation, but include cranial neuropathies, radiculopathies, and myelopathies. (From Joynt, Clinical Neurology, 1997, Ch48, p25)'),
  'D001101': NtO(MH='Arbaprostil', MS='A synthetic prostaglandin E analog that protects the gastric mucosa, prevents ulceration, and promotes healing of peptic ulcers. The protective effect is independent of acid inhibition. It is also a potent inhibitor of pancreatic function and can inhibit the growth of experimental tumors.'),
  'D001102': NtO(MH='Arbovirus Infections', MS='Infections caused by arthropod-borne viruses, general or unspecified.'),
  'D001103': NtO(MH='Arboviruses', MS='Arthropod-borne viruses. A non-taxonomic designation for viruses that can replicate in both vertebrate hosts and arthropod vectors. Included are some members of the following families: ARENAVIRIDAE; BUNYAVIRIDAE; REOVIRIDAE; TOGAVIRIDAE; and FLAVIVIRIDAE. (From Dictionary of Microbiology and Molecular Biology, 2nd ed)'),
  'D001104': NtO(MH='Arbutin', MS=None),
  'D001105': NtO(MH='Archaea', MS='One of the three domains of life (the others being BACTERIA and Eukarya), formerly called Archaebacteria under the taxon Bacteria, but now considered separate and distinct. They are characterized by: (1) the presence of characteristic tRNAs and ribosomal RNAs; (2) the absence of peptidoglycan cell walls; (3) the presence of ether-linked lipids built from branched-chain subunits; and (4) their occurrence in unusual habitats. While archaea resemble bacteria in morphology and genomic organization, they resemble eukarya in their method of genomic replication. The domain contains at least four kingdoms: CRENARCHAEOTA; EURYARCHAEOTA; NANOARCHAEOTA; and KORARCHAEOTA.'),
  'D001106': NtO(MH='Archaeology', MS='Study of the ancient and recent human or animal past through material remains. It analyzes the physical remains of the past in pursuit of a broad and comprehensive understanding of human culture.'),
  'D001107': NtO(MH='Architectural Accessibility', MS='Designs for approaching areas inside or outside facilities.'),
  'D001108': NtO(MH='Architecture', MS='Works on the art and science of designing buildings and structures, design of the total built environment, including town planning, urban design, and landscape architecture.'),
  'D001109': NtO(MH='Archives', MS='A collection of historical documents or records providing information about places, institutions, or groups of people; also the place where such records are kept.'),
  'D001110': NtO(MH='Arctic Regions', MS="The Arctic Ocean and the lands in it and adjacent to it. It includes Point Barrow, Alaska, most of the Franklin District in Canada, two thirds of Greenland, Svalbard, Franz Josef Land, Lapland, Novaya Zemlya, and Northern Siberia. (Webster's New Geographical Dictionary, 1988, p66)"),
  'D001111': NtO(MH='Arcuate Nucleus of Hypothalamus', MS='A nucleus located in the middle hypothalamus in the most ventral part of the THIRD VENTRICLE near the entrance of the infundibular recess. Its small cells are in close contact with the EPENDYMA.'),
  'D001112': NtO(MH='Arcus Senilis', MS='A corneal disease in which there is a deposition of phospholipid and cholesterol in the corneal stroma and anterior sclera.'),
  'D001113': NtO(MH='Area Health Education Centers', MS='Education centers authorized by the Comprehensive Health Manpower Training Act, 1971, for the training of health personnel in areas where health needs are the greatest. May be used for centers other than those established by the United States act.'),
  'D001114': NtO(MH='Areca', MS='A plant genus of the family ARECACEAE. Members contain ARECOLINE and CATECHIN. The leaves and nuts have been used as masticatories, stimulants, and astringents in traditional medicine. The common name of betel is also used for PIPER BETLE. The common name of catechu is sometimes used for ACACIA CATECHU.'),
  'D001115': NtO(MH='Arecoline', MS='An alkaloid obtained from the betel nut (Areca catechu), fruit of a palm tree. It is an agonist at both muscarinic and nicotinic acetylcholine receptors. It is used in the form of various salts as a ganglionic stimulant, a parasympathomimetic, and a vermifuge, especially in veterinary practice. It has been used as a euphoriant in the Pacific Islands.'),
  'D001116': NtO(MH='Arenaviridae', MS='A family of RNA viruses naturally infecting rodents and consisting of one genus (ARENAVIRUS) with two groups: Old World Arenaviruses (ARENAVIRUSES, OLD WORLD) and New World Arenaviruses (ARENAVIRUSES, NEW WORLD). Infection in rodents is persistent and silent. Vertical transmission is through milk-, saliva-, or urine-borne routes. Horizontal transmission to humans, monkeys, and other animals is important.'),
  'D001117': NtO(MH='Arenaviridae Infections', MS='Virus diseases caused by the ARENAVIRIDAE.'),
  'D001118': NtO(MH='Argentina', MS='Country located in southern South America, bordering the South Atlantic Ocean, between Chile and Uruguay.'),
  'D001119': NtO(MH='Arginase', MS='A ureahydrolase that catalyzes the hydrolysis of arginine or canavanine to yield L-ornithine (ORNITHINE) and urea. Deficiency of this enzyme causes HYPERARGININEMIA. EC 3.5.3.1.'),
  'D001120': NtO(MH='Arginine', MS='An essential amino acid that is physiologically active in the L-form.'),
  'D001121': NtO(MH='Lysine Carboxypeptidase', MS='A metallocarboxypeptidase that removes C-terminal basic amino acid from peptides and proteins, with preference shown for lysine over arginine. It is a plasma zinc enzyme that inactivates bradykinin and anaphylatoxins.'),
  'D001122': NtO(MH='Arginine Kinase', MS='An enzyme that catalyzes the phosphorylation of the guanidine nitrogen of arginine in the presence of ATP and a divalent cation with formation of phosphorylarginine and ADP. EC 2.7.3.3.'),
  'D001123': NtO(MH='Argininosuccinate Lyase', MS='An enzyme of the urea cycle which splits argininosuccinate to fumarate plus arginine. Its absence leads to the metabolic disease ARGININOSUCCINIC ACIDURIA in man. EC 4.3.2.1.'),
  'D001124': NtO(MH='Argininosuccinate Synthase', MS='An enzyme of the urea cycle that catalyzes the formation of argininosuccinic acid from citrulline and aspartic acid in the presence of ATP. Absence or deficiency of this enzyme causes the metabolic disease CITRULLINEMIA in humans. EC 6.3.4.5.'),
  'D001125': NtO(MH='Argininosuccinic Acid', MS='This amino acid is formed during the urea cycle from citrulline, aspartate and ATP. This reaction is catalyzed by argininosuccinic acid synthetase.'),
  'D001126': NtO(MH='Arginine-tRNA Ligase', MS='An enzyme that activates arginine with its specific transfer RNA. EC 6.1.1.19.'),
  'D001127': NtO(MH='Arginine Vasopressin', MS='The predominant form of mammalian antidiuretic hormone. It is a nonapeptide containing an ARGININE at residue 8 and two disulfide-linked cysteines at residues of 1 and 6. Arg-vasopressin is used to treat DIABETES INSIPIDUS or to improve vasomotor tone and BLOOD PRESSURE.'),
  'D001128': NtO(MH='Argon', MS='A noble gas with the atomic symbol Ar, atomic number 18, and atomic weight 39.948. It is used in fluorescent tubes and wherever an inert atmosphere is desired and nitrogen cannot be used.'),
  'D001129': NtO(MH='Argyria', MS='A permanent ashen-gray discoloration of the skin, conjunctiva, and internal organs resulting from long-continued use of silver salts. (Dorland, 27th ed)'),
  'D001130': NtO(MH='Arizona', MS='State of the UNITED STATES OF AMERICA bounded on the east by New Mexico, on the north by Utah, on the west by Nevada and California, and on the south by Mexico.'),
  'D001131': NtO(MH='Arkansas', MS='State of the UNITED STATES OF AMERICA bounded on the north by Missouri, on the east by Tennessee and Mississippi, on the south by Louisiana, and on the west by Oklahoma and Texas.'),
  'D001132': NtO(MH='Arm', MS='The superior part of the upper extremity between the SHOULDER and the ELBOW.'),
  'D001133': NtO(MH='Bones of Upper Extremity', MS='The bones of the upper and lower ARM. They include the CLAVICLE and SCAPULA.'),
  'D001134': NtO(MH='Arm Injuries', MS='General or unspecified injuries involving the UPPER ARM and the FOREARM.'),
  'D001135': NtO(MH='Armadillos', MS='Burrowing, chiefly nocturnal mammals of the family Dasypodidae having bodies and heads encased in small bony plates. They are widely distributed in the warmer parts of the Americas.'),
  'D001136': NtO(MH='Armenia', MS="An ancient country in western Asia, by the twentieth century divided among the former USSR, Turkey, and Iran. It was attacked at various times from before the 7th century B.C. to 69 B.C. by Assyrians, Medes, Persians, the Greeks under Alexander, and the Romans. It changed hands frequently in wars between Neo-Persian and Roman Empires from the 3d to 7th centuries and later under Arabs, Seljuks, Byzantines, and Mongols. In the 19th century Armenian nationalism arose but suffered during Russo-Turkish hostilities. It became part of the Soviet Republic in 1921, with part remaining under Turkey. (Webster's New Geographical Dictionary, 1988)"),
  'D001138': NtO(MH='Armin', MS='A reversible organophosphorus cholinesterase inhibitor. It also affects the presynaptic membrane and inhibits membrane postsynaptic cholinergic receptors. The compound had former use as a miotic.'),
  'D001139': NtO(MH='Arnold-Chiari Malformation', MS='A group of congenital malformations involving the brainstem, cerebellum, upper spinal cord, and surrounding bony structures. Type II is the most common, and features compression of the medulla and cerebellar tonsils into the upper cervical spinal canal and an associated MENINGOMYELOCELE. Type I features similar, but less severe malformations and is without an associated meningomyelocele. Type III has the features of type II with an additional herniation of the entire cerebellum through the bony defect involving the foramen magnum, forming an ENCEPHALOCELE. Type IV is a form a cerebellar hypoplasia. Clinical manifestations of types I-III include TORTICOLLIS; opisthotonus; HEADACHE; VERTIGO; VOCAL CORD PARALYSIS; APNEA; NYSTAGMUS, CONGENITAL; swallowing difficulties; and ATAXIA. (From Menkes, Textbook of Child Neurology, 5th ed, p261; Davis, Textbook of Neuropathology, 2nd ed, pp236-46)'),
  'D001140': NtO(MH='Aroclors', MS='Industrial chemicals which have become widespread environmental pollutants. Each aroclor is a mixture of chlorinated biphenyls (1200 series) or chlorinated terphenyls (5400 series) or a combination of both (4400 series).'),
  'D001141': NtO(MH='Aromatase', MS='An enzyme that catalyzes the desaturation (aromatization) of the ring A of C19 androgens and converts them to C18 estrogens. In this process, the 19-methyl is removed. This enzyme is membrane-bound, located in the endoplasmic reticulum of estrogen-producing cells of ovaries, placenta, testes, adipose, and brain tissues. Aromatase is encoded by the CYP19 gene, and functions in complex with NADPH-FERRIHEMOPROTEIN REDUCTASE in the cytochrome P-450 system.'),
  'D001142': NtO(MH='Aromatic-L-Amino-Acid Decarboxylases', MS='An enzyme group with broad specificity. The enzymes decarboxylate a range of aromatic amino acids including dihydroxyphenylalanine (DOPA DECARBOXYLASE); TRYPTOPHAN; and HYDROXYTRYPTOPHAN.'),
  'D001143': NtO(MH='Arousal', MS='Cortical vigilance or readiness of tone, presumed to be in response to sensory stimulation via the reticular activating system.'),
  'D001145': NtO(MH='Arrhythmias, Cardiac', MS='Any disturbances of the normal rhythmic beating of the heart or MYOCARDIAL CONTRACTION. Cardiac arrhythmias can be classified by the abnormalities in HEART RATE, disorders of electrical impulse generation, or impulse conduction.'),
  'D001146': NtO(MH='Arrhythmia, Sinus', MS='Irregular HEART RATE caused by abnormal function of the SINOATRIAL NODE. It is characterized by a greater than 10% change between the maximum and the minimum sinus cycle length or 120 milliseconds.'),
  'D001147': NtO(MH='Arsanilic Acid', MS='An arsenical which has been used as a feed additive for enteric conditions in pigs and poultry. It causes blindness and is ototoxic and nephrotoxic in animals.'),
  'D001148': NtO(MH='Arsenamide', MS='Proposed chemotherapeutic agent against filaria and trichomonas.'),
  'D001149': NtO(MH='Arsenates', MS='Inorganic or organic salts and esters of arsenic acid.'),
  'D001150': NtO(MH='Arsenazo III', MS='Metallochrome indicator that changes color when complexed to the calcium ion under physiological conditions. It is used to measure local calcium ion concentrations in vivo.'),
  'D001151': NtO(MH='Arsenic', MS='A shiny gray element with atomic symbol As, atomic number 33, and atomic weight 75. It occurs throughout the universe, mostly in the form of metallic arsenides. Most forms are toxic. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), arsenic and certain arsenic compounds have been listed as known carcinogens. (From Merck Index, 11th ed)'),
  'D001152': NtO(MH='Arsenicals', MS='Inorganic or organic compounds that contain arsenic.'),
  'D001153': NtO(MH='Arsphenamine', MS='An organoarsenic compound that was commonly used for treating SYPHILIS and other diseases.'),
  'D001154': NtO(MH='Art', MS='Processes and results of aesthetic expression.'),
  'D001155': NtO(MH='Art Therapy', MS='The use of art as an adjunctive therapy in the treatment of neurological, mental, or behavioral disorders.'),
  'D001156': NtO(MH='Artemia', MS='A genus of CRUSTACEA of the order ANOSTRACA, found in briny pools and lakes and often cultured for fish food. It has 168 chromosomes and differs from most crustaceans in that its blood contains hemoglobin.'),
  'D001157': NtO(MH='Arterial Occlusive Diseases', MS='Pathological processes which result in the partial or complete obstruction of ARTERIES. They are characterized by greatly reduced or absence of blood flow through these vessels. They are also known as arterial insufficiency.'),
  'D001158': NtO(MH='Arteries', MS='The vessels carrying blood away from the heart.'),
  'D001159': NtO(MH='Arterio-Arterial Fistula', MS='Abnormal communication between two ARTERIES that may result from injury or occur as a congenital abnormality.'),
  'D001160': NtO(MH='Arterioles', MS='The smallest divisions of the arteries located between the muscular arteries and the capillaries.'),
  'D001161': NtO(MH='Arteriosclerosis', MS='Thickening and loss of elasticity of the walls of ARTERIES of all sizes. There are many forms classified by the types of lesions and arteries involved, such as ATHEROSCLEROSIS with fatty lesions in the ARTERIAL INTIMA of medium and large muscular arteries.'),
  'D001162': NtO(MH='Arteriosclerosis Obliterans', MS='Common occlusive arterial disease which is caused by ATHEROSCLEROSIS. It is characterized by lesions in the innermost layer (ARTERIAL INTIMA) of arteries including the AORTA and its branches to the extremities. Risk factors include smoking, HYPERLIPIDEMIA, and HYPERTENSION.'),
  'D001163': NtO(MH='Arteriovenous Anastomosis', MS='A vessel that directly interconnects an artery and a vein, and that acts as a shunt to bypass the CAPILLARY BEDS. Not to be confused with SURGICAL ANASTOMOSIS or ARTERIOVENOUS FISTULA.'),
  'D001164': NtO(MH='Arteriovenous Fistula', MS='An abnormal direct communication between an artery and a vein without passing through the CAPILLARIES. An A-V fistula usually leads to the formation of a dilated sac-like connection, arteriovenous aneurysm. The locations and size of the shunts determine the degree of effects on the cardiovascular functions such as BLOOD PRESSURE and HEART RATE.'),
  'D001165': NtO(MH='Arteriovenous Malformations', MS='Abnormal formation of blood vessels that shunt arterial blood directly into veins without passing through the CAPILLARIES. They usually are crooked, dilated, and with thick vessel walls. A common type is the congenital arteriovenous fistula. The lack of blood flow and oxygen in the capillaries can lead to tissue damage in the affected areas.'),
  'D001166': NtO(MH='Arteriovenous Shunt, Surgical', MS='Surgical shunt allowing direct passage of blood from an artery to a vein. (From Dorland, 28th ed)'),
  'D001167': NtO(MH='Arteritis', MS='INFLAMMATION of any ARTERIES.'),
  'D001168': NtO(MH='Arthritis', MS='Acute or chronic inflammation of JOINTS.'),
  'D001169': NtO(MH='Arthritis, Experimental', MS='ARTHRITIS that is induced in experimental animals. Immunological methods and infectious agents can be used to develop experimental arthritis models. These methods include injections of stimulators of the immune response, such as an adjuvant (ADJUVANTS, IMMUNOLOGIC) or COLLAGEN.'),
  'D001170': NtO(MH='Arthritis, Infectious', MS='Arthritis caused by BACTERIA; RICKETTSIA; MYCOPLASMA; VIRUSES; FUNGI; or PARASITES.'),
  'D001171': NtO(MH='Arthritis, Juvenile', MS='Arthritis in children, with onset before 16 years of age. The terms juvenile rheumatoid arthritis (JRA) and juvenile idiopathic arthritis (JIA) refer to classification systems for chronic arthritis in children. Only one subtype of juvenile arthritis (polyarticular-onset, rheumatoid factor-positive) clinically resembles adult rheumatoid arthritis and is considered its childhood equivalent.'),
  'D001172': NtO(MH='Arthritis, Rheumatoid', MS='A chronic systemic disease, primarily of the joints, marked by inflammatory changes in the synovial membranes and articular structures, widespread fibrinoid degeneration of the collagen fibers in mesenchymal tissues, and by atrophy and rarefaction of bony structures. Etiology is unknown, but autoimmune mechanisms have been implicated.'),
  'D001173': NtO(MH='Arthrobacter', MS='A genus of asporogenous bacteria isolated from soil that displays a distinctive rod-coccus growth cycle.'),
  'D001174': NtO(MH='Arthrodesis', MS='The surgical fixation of a joint by a procedure designed to accomplish fusion of the joint surfaces by promoting the proliferation of bone cells. (Dorland, 28th ed)'),
  'D001175': NtO(MH='Arthrography', MS='Roentgenography of a joint, usually after injection of either positive or negative contrast medium.'),
  'D001176': NtO(MH='Arthrogryposis', MS='Persistent flexure or contracture of a joint.'),
  'D001177': NtO(MH='Arthropathy, Neurogenic', MS='Chronic progressive degeneration of the stress-bearing portion of a joint, with bizarre hypertrophic changes at the periphery. It is probably a complication of a variety of neurologic disorders, particularly TABES DORSALIS, involving loss of sensation, which leads to relaxation of supporting structures and chronic instability of the joint. (Dorland, 27th ed)'),
  'D001178': NtO(MH='Arthroplasty', MS='Surgical reconstruction of a joint to relieve pain or restore motion.'),
  'D001179': NtO(MH='Arthropod Vectors', MS='Arthropods, other than insects and arachnids, which transmit infective organisms from one host to another or from an inanimate reservoir to an animate host.'),
  'D001180': NtO(MH='Arthropod Venoms', MS='Venoms from animals of the phylum Arthropoda. Those most investigated are from scorpions and spiders of the class Arachnidae and from ant, bee, and wasp families of the Insecta order Hymenoptera. The venoms contain protein toxins, enzymes, and other bioactive substances and may be lethal to man.'),
  'D001181': NtO(MH='Arthropods', MS='Members of the phylum Arthropoda, composed of organisms having a hard, jointed exoskeleton and paired jointed legs. It includes the class INSECTS and the subclass ARACHNIDA, many species of which are important medically as parasites or as vectors of organisms capable of causing disease in man.'),
  'D001182': NtO(MH='Arthroscopy', MS='Endoscopic examination, therapy and surgery of the joint.'),
  'D001183': NtO(MH='Arthus Reaction', MS='A dermal inflammatory reaction produced under conditions of antibody excess, when a second injection of antigen produces intravascular antigen-antibody complexes which bind complement, causing cell clumping, endothelial damage, and vascular necrosis.'),
  'D001184': NtO(MH='Articulation Disorders', MS='Disorders of the quality of speech characterized by the substitution, omission, distortion, and addition of phonemes.'),
  'D001185': NtO(MH='Artificial Intelligence', MS='Theory and development of COMPUTER SYSTEMS which perform tasks that normally require human intelligence. Such tasks may include speech recognition, LEARNING; VISUAL PERCEPTION; MATHEMATICAL COMPUTING; reasoning, PROBLEM SOLVING, DECISION-MAKING, and translation of language.'),
  'D001186': NtO(MH='Artificial Limbs', MS='Prosthetic replacements for arms, legs, and parts thereof.'),
  'D001187': NtO(MH='Artificial Organs', MS='Devices intended to replace non-functioning organs. They may be temporary or permanent. Since they are intended always to function as the natural organs they are replacing, they should be differentiated from PROSTHESES AND IMPLANTS and specific types of prostheses which, though also replacements for body parts, are frequently cosmetic (EYE, ARTIFICIAL) as well as functional (ARTIFICIAL LIMBS).'),
  'D001188': NtO(MH='Artiodactyla', MS='An order of mammals which consists of the even-toed ungulates and includes both RUMINANTS and SWINE.'),
  'D001189': NtO(MH='Aryl Hydrocarbon Hydroxylases', MS='A large group of cytochrome P-450 (heme-thiolate) monooxygenases that complex with NAD(P)H-FLAVIN OXIDOREDUCTASE in numerous mixed-function oxidations of aromatic compounds. They catalyze hydroxylation of a broad spectrum of substrates and are important in the metabolism of steroids, drugs, and toxins such as PHENOBARBITAL, carcinogens, and insecticides.'),
  'D001190': NtO(MH='Arylsulfonates', MS='Organic sulfonic acid esters or salts which contain an aromatic hydrocarbon radical.'),
  'D001191': NtO(MH='Arylamine N-Acetyltransferase', MS='An enzyme that catalyzes the transfer of acetyl groups from ACETYL-COA to arylamines. It can also catalyze acetyl transfer between arylamines without COENZYME A and has a wide specificity for aromatic amines, including SEROTONIN. However, arylamine N-acetyltransferase should not be confused with the enzyme ARYLALKYLAMINE N-ACETYLTRANSFERASE which is also referred to as SEROTONIN ACETYLTRANSFERASE.'),
  'D001192': NtO(MH='Arylsulfatases', MS='Enzymes that catalyze the hydrolysis of a phenol sulfate to yield a phenol and sulfate. Arylsulfatase A, B, and C have been separated. A deficiency of arylsulfatases is one of the causes of metachromatic leukodystrophy (LEUKODYSTROPHY, METACHROMATIC). EC 3.1.6.1.'),
  'D001193': NtO(MH='Arytenoid Cartilage', MS='One of a pair of small pyramidal cartilages that articulate with the lamina of the CRICOID CARTILAGE. The corresponding VOCAL LIGAMENT and several muscles are attached to it.'),
  'D001194': NtO(MH='Asbestos', MS='Asbestos. Fibrous incombustible mineral composed of magnesium and calcium silicates with or without other elements. It is relatively inert chemically and used in thermal insulation and fireproofing. Inhalation of dust causes asbestosis and later lung and gastrointestinal neoplasms.'),
  'D001195': NtO(MH='Asbestosis', MS='A form of pneumoconiosis caused by inhalation of asbestos fibers which elicit potent inflammatory responses in the parenchyma of the lung. The disease is characterized by interstitial fibrosis of the lung, varying from scattered sites to extensive scarring of the alveolar interstitium.'),
  'D001196': NtO(MH='Ascariasis', MS='Infection by nematodes of the genus ASCARIS. Ingestion of infective eggs causes diarrhea and pneumonitis. Its distribution is more prevalent in areas of poor sanitation and where human feces are used for fertilizer.'),
  'D001197': NtO(MH='Ascaridia', MS='A genus of nematode worms in the superfamily Heterakoidea. A. galli and A. lineata are important intestinal parasites of domestic fowl.'),
  'D001198': NtO(MH='Ascaridiasis', MS='Infection with nematodes of the genus ASCARIDIA. This condition usually occurs in fowl, often manifesting diarrhea.'),
  'D001199': NtO(MH='Ascaridoidea', MS='A superfamily of polymyarian nematode worms. An important characteristic of this group is the presence of three prominent lips around the mouth of the organism.'),
  'D001200': NtO(MH='Ascaris', MS='A genus of nematodes of the superfamily ASCARIDOIDEA whose species usually inhabit the intestine.'),
  'D001201': NtO(MH='Ascites', MS='Accumulation or retention of free fluid within the peritoneal cavity.'),
  'D001202': NtO(MH='Ascitic Fluid', MS='The serous fluid of ASCITES, the accumulation of fluids in the PERITONEAL CAVITY.'),
  'D001203': NtO(MH='Ascomycota', MS='A phylum of fungi which have cross-walls or septa in the mycelium. The perfect state is characterized by the formation of a saclike cell (ascus) containing ascospores. Most pathogenic fungi with a known perfect state belong to this phylum.'),
  'D001204': NtO(MH='Ascorbate Oxidase', MS='An enzyme that converts ascorbic acid to dehydroascorbic acid. EC 1.10.3.3.'),
  'D001205': NtO(MH='Ascorbic Acid', MS='A six carbon compound related to glucose. It is found naturally in citrus fruits and many vegetables. Ascorbic acid is an essential nutrient in human diets, and necessary to maintain connective tissue and bone. Its biologically active form, vitamin C, functions as a reducing agent and coenzyme in several metabolic pathways. Vitamin C is considered an antioxidant.'),
  'D001206': NtO(MH='Ascorbic Acid Deficiency', MS="A condition due to a dietary deficiency of ascorbic acid (vitamin C), characterized by malaise, lethargy, and weakness. As the disease progresses, joints, muscles, and subcutaneous tissues may become the sites of hemorrhage. Ascorbic acid deficiency frequently develops into SCURVY in young children fed unsupplemented cow's milk exclusively during their first year. It develops also commonly in chronic alcoholism. (Cecil Textbook of Medicine, 19th ed, p1177)"),
  'D001207': NtO(MH='Asepsis', MS='The prevention of access by infecting organisms to the locus of potential infection.'),
  'D001208': NtO(MH='Asia', MS="The largest of the continents. It was known to the Romans more specifically as what we know today as Asia Minor. The name comes from at least two possible sources: from the Assyrian asu (to rise) or from the Sanskrit usa (dawn), both with reference to its being the land of the rising sun, i.e., eastern as opposed to Europe, to the west. (From Webster's New Geographical Dictionary, 1988, p82 & Room, Brewer's Dictionary of Names, 1992, p34)"),
  'D001209': NtO(MH='Asia, Central', MS="The geographical area of Asia comprising KAZAKHSTAN; KYRGYZSTAN; TAJIKISTAN; TURKMENISTAN; and UZBEKISTAN. The desert region of Kara Kum (Qara Qum) is largely in Turkmenistan and the desert region of Kyzyl Kum (Kizil Kum or Qizil Qum), is in Uzbekistan and Kazakhstan. (From Webster's New Geographical Dictionary, 1988, p233, 590, 636)"),
  'D001210': NtO(MH='Asia, Southeastern', MS='The geographical area of Asia comprising BORNEO; BRUNEI; CAMBODIA; INDONESIA; LAOS; MALAYSIA; the MEKONG VALLEY; MYANMAR (formerly Burma), the PHILIPPINES; SINGAPORE; THAILAND; and VIETNAM.'),
  'D001211': NtO(MH='Asia, Western', MS="The geographical designation for the countries of the MIDDLE EAST and the countries BANGLADESH; BHUTAN; INDIA; NEPAL; PAKISTAN; and SRI LANKA. (From Random House Unabridged Dictionary, 2d ed, 1993 & Webster's New Geographical Dictionary, 1988)"),
  'D001212': NtO(MH='Asialoglycoproteins', MS='Endogenous glycoproteins from which SIALIC ACID has been removed by the action of sialidases. They bind tightly to the ASIALOGLYCOPROTEIN RECEPTOR which is located on hepatocyte plasma membranes. After internalization by adsorptive ENDOCYTOSIS they are delivered to LYSOSOMES for degradation. Therefore receptor-mediated clearance of asialoglycoproteins is an important aspect of the turnover of plasma glycoproteins. They are elevated in serum of patients with HEPATIC CIRRHOSIS or HEPATITIS.'),
  'D001213': NtO(MH='Asian Americans', MS='Persons living in the United States having origins in any of the original peoples of the Far East, Southeast Asia, or the Indian subcontinent.'),
  'D001215': NtO(MH='Asparaginase', MS='A hydrolase enzyme that converts L-asparagine and water to L-aspartate and NH3. EC 3.5.1.1.'),
  'D001216': NtO(MH='Asparagine', MS='A non-essential amino acid that is involved in the metabolic control of cell functions in nerve and brain tissue. It is biosynthesized from ASPARTIC ACID and AMMONIA by asparagine synthetase. (From Concise Encyclopedia Biochemistry and Molecular Biology, 3rd ed)'),
  'D001217': NtO(MH='Aspartate-Ammonia Ligase', MS='An enzyme that catalyzes the formation of asparagine from ammonia and aspartic acid, in the presence of ATP. EC 6.3.1.1.'),
  'D001218': NtO(MH='Aspartame', MS='Flavoring agent sweeter than sugar, metabolized as PHENYLALANINE and ASPARTIC ACID.'),
  'D001219': NtO(MH='Aspartate Aminotransferases', MS='Enzymes of the transferase class that catalyze the conversion of L-aspartate and 2-ketoglutarate to oxaloacetate and L-glutamate. EC 2.6.1.1.'),
  'D001220': NtO(MH='Aspartate Ammonia-Lyase', MS='An enzyme that catalyzes the conversion of aspartic acid to ammonia and fumaric acid in plants and some microorganisms. EC 4.3.1.1.'),
  'D001221': NtO(MH='Aspartate Carbamoyltransferase', MS='An enzyme that catalyzes the conversion of carbamoyl phosphate and L-aspartate to yield orthophosphate and N-carbamoyl-L-aspartate. (From Enzyme Nomenclature, 1992) EC 2.1.3.2.'),
  'D001222': NtO(MH='Aspartate Kinase', MS='An enzyme that catalyzes the formation of beta-aspartyl phosphate from aspartic acid and ATP. Threonine serves as an allosteric regulator of this enzyme to control the biosynthetic pathway from aspartic acid to threonine. EC 2.7.2.4.'),
  'D001223': NtO(MH='Aspartate-Semialdehyde Dehydrogenase', MS='An enzyme that catalyzes the conversion of L-aspartate 4-semialdehyde, orthophosphate, and NADP+ to yield L-4-aspartyl phosphate and NADPH. EC 1.2.1.11.'),
  'D001224': NtO(MH='Aspartic Acid', MS='One of the non-essential amino acids commonly occurring in the L-form. It is found in animals and plants, especially in sugar cane and sugar beets. It may be a neurotransmitter.'),
  'D001225': NtO(MH='Aspartokinase Homoserine Dehydrogenase', MS='A bifunctional protein consisting of aspartokinase, and homoserine dehydrogenase activities. It is found primarily in BACTERIA and in PLANTS.'),
  'D001226': NtO(MH='Aspartate-tRNA Ligase', MS='An enzyme that activates aspartic acid with its specific transfer RNA. EC 6.1.1.12.'),
  'D001227': NtO(MH='Aspartylglucosylaminase', MS='An enzyme that catalyzes the conversion of N(4)-(beta-N-acetyl-D-glucosaminyl)-L-asparagine and water to N-acetyl-beta-D-glucosaminylamine and L-aspartate. It acts only on asparagine-oligosaccharides containing one amino acid, i.e. the ASPARAGINE has free alpha-amino and alpha-carboxyl groups. (From Enzyme Nomenclature, 1992)'),
  'D001228': NtO(MH='Aspergillosis', MS='Infections with fungi of the genus ASPERGILLUS.'),
  'D001229': NtO(MH='Aspergillosis, Allergic Bronchopulmonary', MS='Hypersensitivity reaction (ALLERGIC REACTION) to fungus ASPERGILLUS in an individual with long-standing BRONCHIAL ASTHMA. It is characterized by pulmonary infiltrates, EOSINOPHILIA, elevated serum IMMUNOGLOBULIN E, and skin reactivity to Aspergillus antigen.'),
  'D001230': NtO(MH='Aspergillus', MS='A genus of mitosporic fungi containing about 100 species and eleven different teleomorphs in the family Trichocomaceae.'),
  'D001231': NtO(MH='Aspergillus flavus', MS='A species of imperfect fungi which grows on peanuts and other plants and produces the carcinogenic substance aflatoxin. It is also used in the production of the antibiotic flavicin.'),
  'D001232': NtO(MH='Aspergillus fumigatus', MS='A species of imperfect fungi from which the antibiotic fumigatin is obtained. Its spores may cause respiratory infection in birds and mammals.'),
  'D001233': NtO(MH='Aspergillus nidulans', MS='A species of imperfect fungi from which the antibiotic nidulin is obtained. Its teleomorph is Emericella nidulans.'),
  'D001234': NtO(MH='Aspergillus niger', MS='An imperfect fungus causing smut or black mold of several fruits, vegetables, etc.'),
  'D001235': NtO(MH='Aspergillus ochraceus', MS='An imperfect fungus that produces ochratoxins and contaminates EDIBLE GRAIN and coffee beans.'),
  'D001236': NtO(MH='Aspergillus oryzae', MS='An imperfect fungus present on most agricultural seeds and often responsible for the spoilage of seeds in bulk storage. It is also used in the production of fermented food or drink, especially in Japan.'),
  'D001237': NtO(MH='Asphyxia', MS='A pathological condition caused by lack of oxygen, manifested in impending or actual cessation of life.'),
  'D001238': NtO(MH='Asphyxia Neonatorum', MS='Respiratory failure in the newborn. (Dorland, 27th ed)'),
  'D001239': NtO(MH='Inhalation', MS='The act of BREATHING in.'),
  'D001240': NtO(MH='Aspirations, Psychological', MS='Strong desires to accomplish something. This usually pertains to greater values or high ideals.'),
  'D001241': NtO(MH='Aspirin', MS='The prototypical analgesic used in the treatment of mild to moderate pain. It has anti-inflammatory and antipyretic properties and acts as an inhibitor of cyclooxygenase which results in the inhibition of the biosynthesis of prostaglandins. Aspirin also inhibits platelet aggregation and is used in the prevention of arterial and venous thrombosis. (From Martindale, The Extra Pharmacopoeia, 30th ed, p5)'),
  'D001242': NtO(MH='Assertiveness', MS='Strongly insistent, self-assured, and demanding behavior.'),
  'D001243': NtO(MH='Assisted Circulation', MS='Pumping that aids the natural activity of the heart. (Dorland, 27th ed)'),
  'D001244': NtO(MH='Association', MS='A functional relationship between psychological phenomena of such nature that the presence of one tends to evoke the other; also, the process by which such a relationship is established.'),
  'D001245': NtO(MH='Association Learning', MS='The principle that items experienced together enter into a connection, so that one tends to reinstate the other.'),
  'D001246': NtO(MH='Astatine', MS='Astatine. A radioactive halogen with the atomic symbol At, and atomic number 85. Its isotopes range in mass number from 200 to 219 and all have an extremely short half-life. Astatine may be of use in the treatment of hyperthyroidism because it emits ALPHA PARTICLES.'),
  'D001247': NtO(MH='Asthenia', MS='Clinical sign or symptom manifested as debility, or lack or loss of strength and energy.'),
  'D001248': NtO(MH='Asthenopia', MS='Term generally used to describe complaints related to refractive error, ocular muscle imbalance, including pain or aching around the eyes, burning and itchiness of the eyelids, ocular fatigue, and headaches.'),
  'D001249': NtO(MH='Asthma', MS='A form of bronchial disorder with three distinct components: airway hyper-responsiveness (RESPIRATORY HYPERSENSITIVITY), airway INFLAMMATION, and intermittent AIRWAY OBSTRUCTION. It is characterized by spasmodic contraction of airway smooth muscle, WHEEZING, and dyspnea (DYSPNEA, PAROXYSMAL).'),
  'D001250': NtO(MH='Asthma, Exercise-Induced', MS='Asthma attacks following a period of exercise. Usually the induced attack is short-lived and regresses spontaneously. The magnitude of postexertional airway obstruction is strongly influenced by the environment in which exercise is performed (i.e. inhalation of cold air during physical exertion markedly augments the severity of the airway obstruction; conversely, warm humid air blunts or abolishes it).'),
  'D001251': NtO(MH='Astigmatism', MS='Unequal curvature of the refractive surfaces of the eye. Thus a point source of light cannot be brought to a point focus on the retina but is spread over a more or less diffuse area. This results from the radius of curvature in one plane being longer or shorter than the radius at right angles to it. (Dorland, 27th ed)'),
  'D001252': NtO(MH='Astringents', MS='Agents, usually topical, that cause the contraction of tissues for the control of bleeding or secretions.'),
  'D001253': NtO(MH='Astrocytes', MS='A class of large neuroglial (macroglial) cells in the central nervous system - the largest and most numerous neuroglial cells in the brain and spinal cord. Astrocytes (from "star" cells) are irregularly shaped with many long processes, including those with "end feet" which form the glial (limiting) membrane and directly and indirectly contribute to the BLOOD-BRAIN BARRIER. They regulate the extracellular ionic and chemical environment, and "reactive astrocytes" (along with MICROGLIA) respond to injury.'),
  'D001254': NtO(MH='Astrocytoma', MS='Neoplasms of the brain and spinal cord derived from glial cells which vary from histologically benign forms to highly anaplastic and malignant tumors. Fibrillary astrocytomas are the most common type and may be classified in order of increasing malignancy (grades I through IV). In the first two decades of life, astrocytomas tend to originate in the cerebellar hemispheres; in adults, they most frequently arise in the cerebrum and frequently undergo malignant transformation. (From Devita et al., Cancer: Principles and Practice of Oncology, 5th ed, pp2013-7; Holland et al., Cancer Medicine, 3d ed, p1082)'),
  'D001255': NtO(MH='Astrology', MS='System of thought regarding forecasting of earthly and human events through the observation and interpretation of the fixed stars and planetary bodies.'),
  'D001256': NtO(MH='Astronomy', MS='The science concerned with celestial bodies and the observation and interpretation of the radiation received in the vicinity of the earth from the component parts of the universe (McGraw Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D001257': NtO(MH='Mamastrovirus', MS='A genus of small, circular RNA viruses in the family ASTROVIRIDAE. They cause GASTROENTERITIS and are found in the stools of several vertebrates including humans. Transmission is by the fecal-oral route and there are at least eight human serotypes. The type species is Human astrovirus.'),
  'D001259': NtO(MH='Ataxia', MS='Impairment of the ability to perform smoothly coordinated voluntary movements. This condition may affect the limbs, trunk, eyes, pharynx, larynx, and other structures. Ataxia may result from impaired sensory or motor function. Sensory ataxia may result from posterior column injury or PERIPHERAL NERVE DISEASES. Motor ataxia may be associated with CEREBELLAR DISEASES; CEREBRAL CORTEX diseases; THALAMIC DISEASES; BASAL GANGLIA DISEASES; injury to the RED NUCLEUS; and other conditions.'),
  'D001260': NtO(MH='Ataxia Telangiectasia', MS='An autosomal recessive inherited disorder characterized by choreoathetosis beginning in childhood, progressive CEREBELLAR ATAXIA; TELANGIECTASIS of CONJUNCTIVA and SKIN; DYSARTHRIA; B- and T-cell immunodeficiency, and RADIOSENSITIVITY to IONIZING RADIATION. Affected individuals are prone to recurrent sinobronchopulmonary infections, lymphoreticular neoplasms, and other malignancies. Serum ALPHA-FETOPROTEINS are usually elevated. (Menkes, Textbook of Child Neurology, 5th ed, p688) The gene for this disorder (ATM) encodes a cell cycle checkpoint protein kinase and has been mapped to chromosome 11 (11q22-q23).'),
  'D001261': NtO(MH='Pulmonary Atelectasis', MS='Absence of air in the entire or part of a lung, such as an incompletely inflated neonate lung or a collapsed adult lung. Pulmonary atelectasis can be caused by airway obstruction, lung compression, fibrotic contraction, or other factors.'),
  'D001262': NtO(MH='Atenolol', MS='A cardioselective beta-1 adrenergic blocker possessing properties and potency similar to PROPRANOLOL, but without a negative inotropic effect.'),
  'D001264': NtO(MH='Athetosis', MS='A dyskinesia characterized by an inability to maintain the fingers, toes, tongue, or other body parts in a stable position, resulting in continuous slow, sinusoidal, and flowing involuntary movements. This condition is frequently accompanied by CHOREA, where it is referred to as choreoathetosis. Athetosis may occur as a manifestation of BASAL GANGLIA DISEASES or DRUG TOXICITY. (From Adams et al., Principles of Neurology, 6th ed, p76)'),
  'D001265': NtO(MH='Athletic Injuries', MS='Injuries incurred during participation in competitive or non-competitive sports.'),
  'D001266': NtO(MH='Atlantic Islands', MS='Widely scattered islands in the Atlantic Ocean as far north as the AZORES and as far south as the South Sandwich Islands, with the greatest concentration found in the CARIBBEAN REGION. They include Annobon Island, Ascension, Canary Islands, Falkland Islands, Fernando Po (also called Isla de Bioko and Bioko), Gough Island, Madeira, Sao Tome and Principe, Saint Helena, and Tristan da Cunha.'),
  'D001267': NtO(MH='Atlantic Ocean', MS='Body of water separating North and South America from Africa and Europe.'),
  'D001268': NtO(MH='Atlanto-Axial Joint', MS='The joint involving the CERVICAL ATLAS and axis bones.'),
  'D001269': NtO(MH='Atlanto-Occipital Joint', MS='The point of articulation between the OCCIPITAL BONE and the CERVICAL ATLAS.'),
  'D001270': NtO(MH='Cervical Atlas', MS='The first cervical vertebra.'),
  'D001271': NtO(MH='Atlases as Topic', MS='Works about collections of illustrative plates and charts..'),
  'D001272': NtO(MH='Atmosphere', MS='The gaseous envelope surrounding a planet or similar body. (From Random House Unabridged Dictionary, 2d ed)'),
  'D001273': NtO(MH='Atmosphere Exposure Chambers', MS='Experimental devices used in inhalation studies in which a person or animal is either partially or completely immersed in a chemically controlled atmosphere.'),
  'D001274': NtO(MH='Atmospheric Pressure', MS='The pressure at any point in an atmosphere due solely to the weight of the atmospheric gases above the point concerned.'),
  'D001275': NtO(MH='ATP Citrate (pro-S)-Lyase', MS='An enzyme that, in the presence of ATP and COENZYME A, catalyzes the cleavage of citrate to yield acetyl CoA, oxaloacetate, ADP, and ORTHOPHOSPHATE. This reaction represents an important step in fatty acid biosynthesis. This enzyme was formerly listed as EC 4.1.3.8.'),
  'D001276': NtO(MH='ATP Phosphoribosyltransferase', MS="An enzyme that catalyzes the first step of the pathway for histidine biosynthesis in Salmonella typhimurium. ATP reacts reversibly with 5-phosphoribosyl-1-pyrophosphate to yield N-1-(5'-phosphoribosyl)-ATP and pyrophosphate. EC 2.4.2.17."),
  'D001277': NtO(MH='GTP Pyrophosphokinase', MS="An enzyme that catalyzes reversibly the transfer of a pyrophosphate group from ATP to the 3'-OH group of GDP or GTP with the formation of guanosine 3'-diphosphate 5'-diphosphate or guanosine 3'-diphosphate 5'-triphosphate and AMP. The enzyme, also called stringent factor, is located in the relA gene in stringent strains of bacteria. The above synthesis is induced by mRNA and uncharged tRNA which is bound to the aminoacyl-t-RNA binding site of the ribosome by a codon-specific association. EC 2.7.6.5."),
  'D001278': NtO(MH='Atractyloside', MS='A glycoside of a kaurene type diterpene that is found in some plants including Atractylis gummifera (ATRACTYLIS); COFFEE; XANTHIUM, and CALLILEPIS. Toxicity is due to inhibition of ADENINE NUCLEOTIDE TRANSLOCASE.'),
  'D001279': NtO(MH='Atracurium', MS='A non-depolarizing neuromuscular blocking agent with short duration of action. Its lack of significant cardiovascular effects and its lack of dependence on good kidney function for elimination provide clinical advantage over alternate non-depolarizing neuromuscular blocking agents.'),
  'D001280': NtO(MH='Atrazine', MS='A selective triazine herbicide. Inhalation hazard is low and there are no apparent skin manifestations or other toxicity in humans. Acutely poisoned sheep and cattle may show muscular spasms, fasciculations, stiff gait, increased respiratory rates, adrenal degeneration, and congestion of the lungs, liver, and kidneys. (From The Merck Index, 11th ed)'),
  'D001281': NtO(MH='Atrial Fibrillation', MS='Abnormal cardiac rhythm that is characterized by rapid, uncoordinated firing of electrical impulses in the upper chambers of the heart (HEART ATRIA). In such case, blood cannot be effectively pumped into the lower chambers of the heart (HEART VENTRICLES). It is caused by abnormal impulse generation.'),
  'D001282': NtO(MH='Atrial Flutter', MS='Rapid, irregular atrial contractions caused by a block of electrical impulse conduction in the right atrium and a reentrant wave front traveling up the inter-atrial septum and down the right atrial free wall or vice versa. Unlike ATRIAL FIBRILLATION which is caused by abnormal impulse generation, typical atrial flutter is caused by abnormal impulse conduction. As in atrial fibrillation, patients with atrial flutter cannot effectively pump blood into the lower chambers of the heart (HEART VENTRICLES).'),
  'D001283': NtO(MH='Atrioventricular Node', MS='A small nodular mass of specialized muscle fibers located in the interatrial septum near the opening of the coronary sinus. It gives rise to the atrioventricular bundle of the conduction system of the heart.'),
  'D001284': NtO(MH='Atrophy', MS='Decrease in the size of a cell, tissue, organ, or multiple organs, associated with a variety of pathological conditions such as abnormal cellular changes, ischemia, malnutrition, or hormonal changes.'),
  'D001285': NtO(MH='Atropine', MS='An alkaloid, originally from Atropa belladonna, but found in other plants, mainly SOLANACEAE. Hyoscyamine is the 3(S)-endo isomer of atropine.'),
  'D001286': NtO(MH='Atropine Derivatives', MS='Analogs and derivatives of atropine.'),
  'D001287': NtO(MH='Attachment Sites, Microbiological', MS='Specific loci on both the bacterial DNA (attB) and the phage DNA (attP) which delineate the sites where recombination takes place between them, as the phage DNA becomes integrated (inserted) into the BACTERIAL DNA during LYSOGENY.'),
  'D001288': NtO(MH='Attention', MS='Focusing on certain aspects of current experience to the exclusion of others. It is the act of heeding or taking notice or concentrating.'),
  'D001289': NtO(MH='Attention Deficit Disorder with Hyperactivity', MS='A behavior disorder originating in childhood in which the essential features are signs of developmentally inappropriate inattention, impulsivity, and hyperactivity. Although most individuals have symptoms of both inattention and hyperactivity-impulsivity, one or the other pattern may be predominant. The disorder is more frequent in males than females. Onset is in childhood. Symptoms often attenuate during late   adolescence although a minority experience the full complement of symptoms into mid-adulthood. (From DSM-V)'),
  'D001290': NtO(MH='Attitude', MS='An enduring, learned predisposition to behave in a consistent way toward a given class of objects, or a persistent mental and/or neural state of readiness to react to a certain class of objects, not as they are but as they are conceived to be.'),
  'D001291': NtO(MH='Attitude of Health Personnel', MS='Attitudes of personnel toward their patients, other professionals, toward the medical care system, etc.'),
  'D001292': NtO(MH='Attitude to Computers', MS='The attitude and behavior associated with an individual using the computer.'),
  'D001293': NtO(MH='Attitude to Death', MS='Conceptual response of the person to the various aspects of death, which are based on individual psychosocial and cultural experience.'),
  'D001294': NtO(MH='Attitude to Health', MS='Public attitudes toward health, disease, and the medical care system.'),
  'D001295': NtO(MH='Atypical Bacterial Forms', MS='Microorganisms that have undergone greater changes than normal in morphology, physiology, or cultural characteristics.'),
  'D001296': NtO(MH='Audiovisual Aids', MS='Auditory and visual instructional materials.'),
  'D001297': NtO(MH='Audioanalgesia', MS='Method of pain control in which auditory stimulation including music, white noise, and environmental sounds are used.'),
  'D001298': NtO(MH='Audiology', MS='Allied medical specialty that studies hearing and hearing impairment.'),
  'D001299': NtO(MH='Audiometry', MS='The testing of the acuity of the sense of hearing to determine the thresholds of the lowest intensity levels at which an individual can hear a set of tones. The frequencies between 125 and 8000 Hz are used to test air conduction thresholds and the frequencies between 250 and 4000 Hz are used to test bone conduction thresholds.'),
  'D001300': NtO(MH='Audiometry, Evoked Response', MS='A form of electrophysiologic audiometry in which an analog computer is included in the circuit to average out ongoing or spontaneous brain wave activity. A characteristic pattern of response to a sound stimulus may then become evident. Evoked response audiometry is known also as electric response audiometry.'),
  'D001301': NtO(MH='Audiometry, Pure-Tone', MS='Measurement of hearing based on the use of pure tones of various frequencies and intensities as auditory stimuli.'),
  'D001302': NtO(MH='Audiometry, Speech', MS='Measurement of the ability to hear speech under various conditions of intensity and noise interference using sound-field as well as earphones and bone oscillators.'),
  'D001303': NtO(MH='Auditory Cortex', MS='The region of the cerebral cortex that receives the auditory radiation from the MEDIAL GENICULATE BODY.'),
  'D001304': NtO(MH='Auditory Diseases, Central', MS='Disorders of hearing or auditory perception due to pathological processes of the AUDITORY PATHWAYS in the CENTRAL NERVOUS SYSTEM. These include CENTRAL HEARING LOSS and AUDITORY PERCEPTUAL DISORDERS.'),
  'D001305': NtO(MH='Auditory Fatigue', MS='Loss of sensitivity to sounds as a result of auditory stimulation, manifesting as a temporary shift in auditory threshold. The temporary threshold shift, TTS, is expressed in decibels.'),
  'D001306': NtO(MH='Auditory Pathways', MS='NEURAL PATHWAYS and connections within the CENTRAL NERVOUS SYSTEM, beginning at the hair cells of the ORGAN OF CORTI, continuing along the eighth cranial nerve, and terminating at the AUDITORY CORTEX.'),
  'D001307': NtO(MH='Auditory Perception', MS='The process whereby auditory stimuli are selected, organized, and interpreted by the organism.'),
  'D001308': NtO(MH='Auditory Perceptual Disorders', MS='Acquired or developmental cognitive disorders of AUDITORY PERCEPTION characterized by a reduced ability to perceive information contained in auditory stimuli despite intact auditory pathways. Affected individuals have difficulty with speech perception, sound localization, and comprehending the meaning of inflections of speech.'),
  'D001309': NtO(MH='Auditory Threshold', MS='The audibility limit of discriminating sound intensity and pitch.'),
  'D001310': NtO(MH='Auranofin', MS='An oral chrysotherapeutic agent for the treatment of rheumatoid arthritis. Its exact mechanism of action is unknown, but it is believed to act via immunological mechanisms and alteration of lysosomal enzyme activity. Its efficacy is slightly less than that of injected gold salts, but it is better tolerated, and side effects which occur are potentially less serious.'),
  'D001312': NtO(MH='Aurintricarboxylic Acid', MS='A dye which inhibits protein biosynthesis at the initial stages. The ammonium salt (aluminon) is a reagent for the colorimetric estimation of aluminum in water, foods, and tissues.'),
  'D001313': NtO(MH='Aurovertins', MS='Very toxic and complex pyrone derivatives from the fungus Calcarisporium arbuscula. They bind to and inhibit mitochondrial ATPase, thereby uncoupling oxidative phosphorylation. They are used as biochemical tools.'),
  'D001314': NtO(MH='Auscultation', MS='Act of listening for sounds within the body.'),
  'D001315': NtO(MH='Australia', MS='The smallest continent and an independent country, comprising six states and two territories. Its capital is Canberra.'),
  'D001317': NtO(MH='Austria', MS='A country located in Central Europe, north of Italy and Slovenia. The capital is Vienna.'),
  'D001318': NtO(MH='Authoritarianism', MS='The personality pattern or syndrome consisting of behavioral and attitudinal characteristics reflecting a preoccupation with the factors of power and authority in interpersonal relationships.'),
  'D001319': NtO(MH='Authorship', MS='The profession of writing. Also the identity of the writer as the creator of a literary production.'),
  'D001321': NtO(MH='Autistic Disorder', MS='A disorder beginning in childhood. It is marked by the presence of markedly abnormal or impaired development in social interaction and communication and a markedly restricted repertoire of activity and interest. Manifestations of the disorder vary greatly depending on the developmental level and chronological age of the individual. (DSM-V)'),
  'D001322': NtO(MH='Autoanalysis', MS='Method of analyzing chemicals using automation.'),
  'D001323': NtO(MH='Autoantibodies', MS='Antibodies that react with self-antigens (AUTOANTIGENS) of the organism that produced them.'),
  'D001324': NtO(MH='Autoantigens', MS='Endogenous tissue constituents that have the ability to interact with AUTOANTIBODIES and cause an immune response.'),
  'D001325': NtO(MH='Autobiographies as Topic', MS="Works about self-described narratives of a person's life."),
  'D001326': NtO(MH='Autogenic Training', MS='Technique based on muscle relaxation during self-hypnotic exercises. It is used in conjunction with psychotherapy.'),
  'D001327': NtO(MH='Autoimmune Diseases', MS='Disorders that are characterized by the production of antibodies that react with host tissues or immune effector cells that are autoreactive to endogenous peptides.'),
  'D001329': NtO(MH='Autolysis', MS='The spontaneous disintegration of tissues or cells by the action of their own autogenous enzymes.'),
  'D001330': NtO(MH='Electronic Data Processing', MS='Applications that store and process large quantities of data.'),
  'D001331': NtO(MH='Automation', MS="Controlled operation of an apparatus, process, or system by mechanical or electronic devices that take the place of human organs of observation, effort, and decision. (From Webster's Collegiate Dictionary, 1993)"),
  'D001332': NtO(MH='Automatism', MS='Automatic, mechanical, and apparently undirected behavior which is outside of conscious control.'),
  'D001333': NtO(MH='Automobile Driver Examination', MS="Government required written and driving test given to individuals prior to obtaining an operator's license."),
  'D001334': NtO(MH='Automobile Driving', MS='The effect of environmental or physiological factors on the driver and driving ability. Included are driving fatigue, and the effect of drugs, disease, and physical disabilities on driving.'),
  'D001335': NtO(MH='Vehicle Emissions', MS='Gases, fumes, vapors, and ODORANTS escaping from the cylinders of a gasoline or diesel internal-combustion engine. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed & Random House Unabridged Dictionary, 2d ed)'),
  'D001336': NtO(MH='Automobiles', MS='A usually four-wheeled automotive vehicle designed for passenger transportation.'),
  'D001337': NtO(MH='Autonomic Agents', MS='Agents affecting the function of, or mimicking the actions of, the autonomic nervous system and thereby having an effect on such processes as respiration, circulation, digestion, body temperature regulation, certain endocrine gland secretions, etc.'),
  'D001338': NtO(MH='Autonomic Fibers, Postganglionic', MS='Nerve fibers which project from cell bodies of AUTONOMIC GANGLIA to SYNAPSES on target organs.'),
  'D001339': NtO(MH='Autonomic Fibers, Preganglionic', MS='NERVE FIBERS which project from the central nervous system to AUTONOMIC GANGLIA. In the sympathetic division most preganglionic fibers originate with neurons in the intermediolateral column of the SPINAL CORD, exit via ventral roots from upper thoracic through lower lumbar segments, and project to the paravertebral ganglia; there they either terminate in SYNAPSES or continue through the SPLANCHNIC NERVES to the prevertebral ganglia. In the parasympathetic division the fibers originate in neurons of the BRAIN STEM and sacral spinal cord. In both divisions the principal transmitter is ACETYLCHOLINE but peptide cotransmitters may also be released.'),
  'D001340': NtO(MH='Autonomic Nerve Block', MS='Interruption of sympathetic pathways, by local injection of an anesthetic agent, at any of four levels: peripheral nerve block, sympathetic ganglion block, extradural block, and subarachnoid block.'),
  'D001341': NtO(MH='Autonomic Nervous System', MS='The ENTERIC NERVOUS SYSTEM; PARASYMPATHETIC NERVOUS SYSTEM; and SYMPATHETIC NERVOUS SYSTEM taken together. Generally speaking, the autonomic nervous system regulates the internal environment during both peaceful activity and physical or emotional stress. Autonomic activity is controlled and integrated by the CENTRAL NERVOUS SYSTEM, especially the HYPOTHALAMUS and the SOLITARY NUCLEUS, which receive information relayed from VISCERAL AFFERENTS.'),
  'D001342': NtO(MH='Autonomic Nervous System Diseases', MS='Diseases of the parasympathetic or sympathetic divisions of the AUTONOMIC NERVOUS SYSTEM; which has components located in the CENTRAL NERVOUS SYSTEM and PERIPHERAL NERVOUS SYSTEM. Autonomic dysfunction may be associated with HYPOTHALAMIC DISEASES; BRAIN STEM disorders; SPINAL CORD DISEASES; and PERIPHERAL NERVOUS SYSTEM DISEASES. Manifestations include impairments of vegetative functions including the maintenance of BLOOD PRESSURE; HEART RATE; pupil function; SWEATING; REPRODUCTIVE AND URINARY PHYSIOLOGY; and DIGESTION.'),
  'D001343': NtO(MH='Autophagy', MS='The segregation and degradation of various cytoplasmic constituents via engulfment by MULTIVESICULAR BODIES; VACUOLES; or AUTOPHAGOSOMES and their digestion by LYSOSOMES. It plays an important role in BIOLOGICAL METAMORPHOSIS and in the removal of bone by OSTEOCLASTS. Defective autophagy is associated with various diseases, including NEURODEGENERATIVE DISEASES and cancer.'),
  'D001344': NtO(MH='Autopsy', MS='Postmortem examination of the body.'),
  'D001345': NtO(MH='Autoradiography', MS='The making of a radiograph of an object or tissue by recording on a photographic plate the radiation emitted by radioactive material within the object. (Dorland, 27th ed)'),
  'D001346': NtO(MH='Autosuggestion', MS='Suggestion coming from the subject himself.'),
  'D001348': NtO(MH='Aversive Therapy', MS='A treatment that suppresses undesirable behavior by simultaneously exposing the subject to unpleasant consequences.'),
  'D001350': NtO(MH='Encephalomyelitis Virus, Avian', MS='A tentative species in the genus HEPATOVIRUS infecting primarily young chicks, but also found in turkeys, pheasants, and guinea fowl. It causes a fatal neuronal degeneration and is transmitted by mechanical contact.'),
  'D001351': NtO(MH='Infectious bronchitis virus', MS='A species of CORONAVIRUS causing infections in chickens and possibly pheasants. Chicks up to four weeks old are the most severely affected.'),
  'D001352': NtO(MH='Herpesvirus 1, Gallid', MS='The type species of the genus ILTOVIRUS found on every continent and affecting mainly chickens and occasionally pheasants.'),
  'D001353': NtO(MH='Avian Leukosis', MS='A group of transmissible viral diseases of chickens and turkeys. Liver tumors are found in most forms, but tumors can be found elsewhere.'),
  'D001354': NtO(MH='Avian Leukosis Virus', MS='The type species of ALPHARETROVIRUS producing latent or manifest lymphoid leukosis in fowl.'),
  'D001355': NtO(MH='Alpharetrovirus', MS='A genus of the family RETROVIRIDAE with type C morphology, that causes malignant and other diseases in wild birds and domestic fowl.'),
  'D001356': NtO(MH='Reticuloendotheliosis virus', MS='A species in the group RETICULOENDOTHELIOSIS VIRUSES, AVIAN of the genus GAMMARETROVIRUS that causes a chronic neoplastic and a more acute immunosuppressive disease in fowl.'),
  'D001357': NtO(MH='Sarcoma, Avian', MS='Connective tissue tumors, affecting primarily fowl, that are usually caused by avian sarcoma viruses.'),
  'D001358': NtO(MH='Avian Sarcoma Viruses', MS='Group of alpharetroviruses (ALPHARETROVIRUS) producing sarcomata and other tumors in chickens and other fowl and also in pigeons, ducks, and RATS.'),
  'D001359': NtO(MH='Aviation', MS='Design, development, manufacture, and operation of heavier-than-air AIRCRAFT.'),
  'D001360': NtO(MH='Avidin', MS='A specific protein in egg albumin that interacts with BIOTIN to render it unavailable to mammals, thereby producing biotin deficiency.'),
  'D001361': NtO(MH='Avitaminosis', MS='A condition due to a deficiency of one or more essential vitamins. (Dorland, 27th ed)'),
  'D001362': NtO(MH='Avoidance Learning', MS='A response to a cue that is instrumental in avoiding a noxious experience.'),
  'D001363': NtO(MH='Awards and Prizes', MS=None),
  'D001364': NtO(MH='Awareness', MS='The act of "taking account" of an object or state of affairs. It does not imply assessment of, nor attention to the qualities or nature of the object.'),
  'D001365': NtO(MH='Axilla', MS='Area of the human body underneath the SHOULDER JOINT, also known as the armpit or underarm.'),
  'D001366': NtO(MH='Axillary Artery', MS='The continuation of the subclavian artery; it distributes over the upper limb, axilla, chest and shoulder.'),
  'D001367': NtO(MH='Axillary Vein', MS='The venous trunk of the upper limb; a continuation of the basilar and brachial veins running from the lower border of the teres major muscle to the outer border of the first rib where it becomes the subclavian vein.'),
  'D001368': NtO(MH='Axis, Cervical Vertebra', MS='The second cervical vertebra.'),
  'D001369': NtO(MH='Axons', MS='Nerve fibers that are capable of rapidly conducting impulses away from the neuron cell body.'),
  'D001370': NtO(MH='Axonal Transport', MS='The directed transport of ORGANELLES and molecules along nerve cell AXONS. Transport can be anterograde (from the cell body) or retrograde (toward the cell body). (Alberts et al., Molecular Biology of the Cell, 3d ed, pG3)'),
  'D001371': NtO(MH='trans-1,4-Bis(2-chlorobenzaminomethyl)cyclohexane Dihydrochloride', MS='An anti-cholesteremic agent that inhibits delta 7-reductase, delta 14 reductase, and sterol biosynthesis.'),
  'D001372': NtO(MH='Aza Compounds', MS='Organic chemicals where carbon atoms have been replaced by nitrogen atoms.'),
  'D001373': NtO(MH='Azacosterol', MS='Diaza derivative of cholesterol which acts as a hypocholesteremic agent by blocking delta-24-reductase, which causes the accumulation of desmosterol.'),
  'D001374': NtO(MH='Azacitidine', MS='A pyrimidine analogue that inhibits DNA methyltransferase, impairing DNA methylation. It is also an antimetabolite of cytidine, incorporated primarily into RNA. Azacytidine has been used as an antineoplastic agent.'),
  'D001375': NtO(MH='Azaguanine', MS='One of the early purine analogs showing antineoplastic activity. It functions as an antimetabolite and is easily incorporated into ribonucleic acids.'),
  'D001376': NtO(MH='Azaperone', MS='A butyrophenone used in the treatment of PSYCHOSES.'),
  'D001377': NtO(MH='Azaserine', MS='Antibiotic substance produced by various Streptomyces species. It is an inhibitor of enzymatic activities that involve glutamine and is used as an antineoplastic and immunosuppressive agent.'),
  'D001378': NtO(MH='Azasteroids', MS='Steroidal compounds in which one or more carbon atoms in the steroid ring system have been substituted with nitrogen atoms.'),
  'D001379': NtO(MH='Azathioprine', MS='An immunosuppressive agent used in combination with cyclophosphamide and hydroxychloroquine in the treatment of rheumatoid arthritis. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), this substance has been listed as a known carcinogen. (Merck Index, 11th ed)'),
  'D001380': NtO(MH='Azauridine', MS='A triazine nucleoside used as an antineoplastic antimetabolite. It interferes with pyrimidine biosynthesis thereby preventing formation of cellular nucleic acids. As the triacetate, it is also effective as an antipsoriatic.'),
  'D001381': NtO(MH='Azepines', MS='Seven membered heterocyclic rings containing a NITROGEN atom.'),
  'D001382': NtO(MH='Azerbaijan', MS='A country located in Southwestern Asia, bordering the Caspian Sea, between Iran and Russia, with a small European portion north of the Caucasus range. The capital is Baku.'),
  'D001383': NtO(MH='Azetidinecarboxylic Acid', MS='A proline analog that acts as a stoichiometric replacement of proline. It causes the production of abnormal proteins with impaired biological activity.'),
  'D001384': NtO(MH='Azetidines', MS=None),
  'D001385': NtO(MH='Azetines', MS=None),
  'D001386': NtO(MH='Azides', MS='Organic or inorganic compounds that contain the -N3 group.'),
  'D001387': NtO(MH='Azinphosmethyl', MS='An organothiophosphorus cholinesterase inhibitor. It has been used as an acaricide and as an insecticide.'),
  'D001388': NtO(MH='Aziridines', MS='Saturated azacyclopropane compounds. They include compounds with substitutions on CARBON or NITROGEN atoms.'),
  'D001389': NtO(MH='Azirines', MS='Unsaturated azacyclopropane compounds that are three-membered heterocycles of a nitrogen and two carbon atoms.'),
  'D001390': NtO(MH='Azlocillin', MS='A semisynthetic ampicillin-derived acylureido penicillin.'),
  'D001391': NtO(MH='Azo Compounds', MS="Organic chemicals where aryl or alkyl groups are joined by two nitrogen atoms through a double bond (R-N=N-R'; R and R' may be either aryl or alkyl). They may be used as DYES."),
  'D001392': NtO(MH='Azocines', MS=None),
  'D001393': NtO(MH='Azoles', MS='Five membered rings containing a NITROGEN atom.'),
  'D001394': NtO(MH='Azores', MS="A group of nine islands and several islets belonging to Portugal in the north Atlantic Ocean off the coast of Portugal. The islands are named after the acores, the Portuguese for goshawks, living there in abundance. (Webster's New Geographical Dictionary, 1988, p102 & Room, Brewer's Dictionary of Names, 1992, p42)"),
  'D001395': NtO(MH='Azotobacter', MS='A genus of gram-negative, aerobic bacteria found in soil and water. Its organisms occur singly, in pairs or irregular clumps, and sometimes in chains of varying lengths.'),
  'D001397': NtO(MH='Azoxymethane', MS='A potent carcinogen and neurotoxic compound. It is particularly effective in inducing colon carcinomas.'),
  'D001398': NtO(MH='Aztreonam', MS='A monocyclic beta-lactam antibiotic originally isolated from Chromobacterium violaceum. It is resistant to beta-lactamases and is used in gram-negative infections, especially of the meninges, bladder, and kidneys. It may cause a superinfection with gram-positive organisms.'),
  'D001399': NtO(MH='Azure Stains', MS='PHENOTHIAZINES with an amino group at the 3-position that are green crystals or powder. They are used as biological stains.'),
  'D001400': NtO(MH='Azurin', MS='A bacterial protein from Pseudomonas, Bordetella, or Alcaligenes which operates as an electron transfer unit associated with the cytochrome chain. The protein has a molecular weight of approximately 16,000, contains a single copper atom, is intensively blue, and has a fluorescence emission band centered at 308nm.'),
  'D001401': NtO(MH='Azygos Vein', MS='A vein which arises from the right ascending lumbar vein or the vena cava, enters the thorax through the aortic orifice in the diaphragm, and terminates in the superior vena cava.'),
  'D001402': NtO(MH='B-Lymphocytes', MS='Lymphoid cells concerned with humoral immunity. They are short-lived cells resembling bursa-derived lymphocytes of birds in their production of immunoglobulin upon appropriate stimulation.'),
  'D001403': NtO(MH='Babesia', MS='A genus of tick-borne protozoan parasites that infests the red blood cells of mammals, including humans. There are many recognized species, and the distribution is world-wide.'),
  'D001404': NtO(MH='Babesiosis', MS="A group of tick-borne diseases of mammals including ZOONOSES in humans. They are caused by protozoa of the genus BABESIA, which parasitize erythrocytes, producing hemolysis. In the U.S., the organism's natural host is mice and transmission is by the deer tick IXODES SCAPULARIS."),
  'D001405': NtO(MH='Reflex, Babinski', MS='A reflex found in normal infants consisting of dorsiflexion of the HALLUX and abduction of the other TOES in response to cutaneous stimulation of the plantar surface of the FOOT. In adults, it is used as a diagnostic criterion, and if present is a NEUROLOGIC MANIFESTATION of dysfunction in the CENTRAL NERVOUS SYSTEM.'),
  'D001406': NtO(MH='Bacillaceae', MS='A family of bacteria which produce endospores. They are mostly saprophytes from soil, but a few are insect or animal parasites or pathogens.'),
  'D001407': NtO(MH='Bacillus', MS='A genus of BACILLACEAE that are spore-forming, rod-shaped cells. Most species are saprophytic soil forms with only a few species being pathogenic.'),
  'D001408': NtO(MH='Bacillus anthracis', MS='A species of bacteria that causes ANTHRAX in humans and animals.'),
  'D001409': NtO(MH='Bacillus cereus', MS='A species of rod-shaped bacteria that is a common soil saprophyte. Its spores are widespread and multiplication has been observed chiefly in foods. Contamination may lead to food poisoning.'),
  'D001410': NtO(MH='Bacillus megaterium', MS='A species of bacteria whose spores vary from round to elongate. It is a common soil saprophyte.'),
  'D001411': NtO(MH='Geobacillus stearothermophilus', MS='A species of GRAM-POSITIVE ENDOSPORE-FORMING BACTERIA in the family BACILLACEAE, found in soil, hot springs, Arctic waters, ocean sediments, and spoiled food products.'),
  'D001412': NtO(MH='Bacillus subtilis', MS='A species of gram-positive bacteria that is a common soil and water saprophyte.'),
  'D001413': NtO(MH='Bacillus thuringiensis', MS='A species of gram-positive bacteria which may be pathogenic for certain insects. It is used for the biological control of the Gypsy moth.'),
  'D001414': NtO(MH='Bacitracin', MS='A complex of cyclic peptide antibiotics produced by the Tracy-I strain of Bacillus subtilis. The commercial preparation is a mixture of at least nine bacitracins with bacitracin A as the major constituent. It is used topically to treat open infections such as infected eczema and infected dermal ulcers. (From Goodman and Gilman, The Pharmacological Basis of Therapeutics, 8th ed, p1140)'),
  'D001415': NtO(MH='Back', MS='The rear surface of an upright primate from the shoulders to the hip, or the dorsal surface of tetrapods.'),
  'D001416': NtO(MH='Back Pain', MS='Acute or chronic pain located in the posterior regions of the THORAX; LUMBOSACRAL REGION; or the adjacent regions.'),
  'D001417': NtO(MH='Background Radiation', MS='Radiation from sources other than the source of interest. It is due to cosmic rays and natural radioactivity in the environment.'),
  'D001418': NtO(MH='Baclofen', MS='A GAMMA-AMINOBUTYRIC ACID derivative that is a specific agonist of GABA-B RECEPTORS. It is used in the treatment of MUSCLE SPASTICITY, especially that due to SPINAL CORD INJURIES. Its therapeutic effects result from actions at spinal and supraspinal sites, generally the reduction of excitatory transmission.'),
  'D001419': NtO(MH='Bacteria', MS='One of the three domains of life (the others being Eukarya and ARCHAEA), also called Eubacteria. They are unicellular prokaryotic microorganisms which generally possess rigid cell walls, multiply by cell division, and exhibit three principal forms: round or coccal, rodlike or bacillary, and spiral or spirochetal. Bacteria can be classified by their response to OXYGEN: aerobic, anaerobic, or facultatively anaerobic; by the mode by which they obtain their energy: chemotrophy (via chemical reaction) or PHOTOTROPHY (via light reaction); for chemotrophs by their source of chemical energy: CHEMOLITHOTROPHY (from inorganic compounds) or chemoorganotrophy (from organic compounds); and by their source for CARBON; NITROGEN; etc.; HETEROTROPHY (from organic sources) or AUTOTROPHY (from CARBON DIOXIDE). They can also be classified by whether or not they stain (based on the structure of their CELL WALLS) with CRYSTAL VIOLET dye: gram-negative or gram-positive.'),
  'D001420': NtO(MH='Bacteria, Aerobic', MS='Bacteria which require oxygen in order to grow and survive.'),
  'D001421': NtO(MH='Bacteria, Anaerobic', MS='Bacteria that can survive and grow in the complete, or nearly complete absence of oxygen.'),
  'D001422': NtO(MH='Bacterial Adhesion', MS='Physicochemical property of fimbriated (FIMBRIAE, BACTERIAL) and non-fimbriated bacteria of attaching to cells, tissue, and nonbiological surfaces. It is a factor in bacterial colonization and pathogenicity.'),
  'D001423': NtO(MH='Bacterial Infections and Mycoses', MS='Infections caused by bacteria and fungi, general, specified, or unspecified.'),
  'D001424': NtO(MH='Bacterial Infections', MS='Infections by bacteria, general or unspecified.'),
  'D001425': NtO(MH='Bacterial Outer Membrane Proteins', MS='Proteins isolated from the outer membrane of Gram-negative bacteria.'),
  'D001426': NtO(MH='Bacterial Proteins', MS='Proteins found in any species of bacterium.'),
  'D001427': NtO(MH='Bacterial Toxins', MS='Toxic substances formed in or elaborated by bacteria; they are usually proteins with high molecular weight and antigenicity; some are used as antibiotics and some to skin test for the presence of or susceptibility to certain diseases.'),
  'D001428': NtO(MH='Bacterial Vaccines', MS='Suspensions of attenuated or killed bacteria administered for the prevention or treatment of infectious bacterial disease.'),
  'D001429': NtO(MH='Bacteriochlorophylls', MS='Pyrrole containing pigments found in photosynthetic bacteria.'),
  'D001430': NtO(MH='Bacteriocins', MS='Substances elaborated by specific strains of bacteria that are lethal against other strains of the same or related species. They are protein or lipopolysaccharide-protein complexes used in taxonomy studies of bacteria.'),
  'D001431': NtO(MH='Bacteriological Techniques', MS='Techniques used in studying bacteria.'),
  'D001432': NtO(MH='Bacteriology', MS='The study of the structure, growth, function, genetics, and reproduction of bacteria, and BACTERIAL INFECTIONS.'),
  'D001433': NtO(MH='Bacteriolysis', MS='Rupture of bacterial cells due to mechanical force, chemical action, or the lytic growth of BACTERIOPHAGES.'),
  'D001434': NtO(MH='Bacteriophage Typing', MS='A technique of bacterial typing which differentiates between bacteria or strains of bacteria by their susceptibility to one or more bacteriophages.'),
  'D001435': NtO(MH='Bacteriophages', MS='Viruses whose hosts are bacterial cells.'),
  'D001436': NtO(MH='Bacteriorhodopsins', MS='Rhodopsins found in the PURPLE MEMBRANE of halophilic archaea such as HALOBACTERIUM HALOBIUM. Bacteriorhodopsins function as an energy transducers, converting light energy into electrochemical energy via PROTON PUMPS.'),
  'D001437': NtO(MH='Bacteriuria', MS='The presence of bacteria in the urine which is normally bacteria-free. These bacteria are from the URINARY TRACT and are not contaminants of the surrounding tissues. Bacteriuria can be symptomatic or asymptomatic. Significant bacteriuria is an indicator of urinary tract infection.'),
  'D001438': NtO(MH='Bacteroidaceae', MS='A family of gram-negative bacteria found primarily in the intestinal tracts and mucous membranes of warm-blooded animals. Its organisms are sometimes pathogenic.'),
  'D001439': NtO(MH='Bacteroides', MS='A genus of gram-negative, anaerobic, rod-shaped bacteria. Its organisms are normal inhabitants of the oral, respiratory, intestinal, and urogenital cavities of humans, animals, and insects. Some species may be pathogenic.'),
  'D001441': NtO(MH='Bacteroides fragilis', MS='Gram-negative bacteria occurring in the lower intestinal tracts of man and other animals. It is the most common species of anaerobic bacteria isolated from human soft tissue infections.'),
  'D001442': NtO(MH='Bacteroides Infections', MS='Infections with bacteria of the genus BACTEROIDES.'),
  'D001443': NtO(MH='Prevotella melaninogenica', MS='A species of gram-negative, anaerobic, rod-shaped bacteria originally classified within the BACTEROIDES genus. This bacterium has been isolated from the mouth, urine, feces, and infections of the mouth, soft tissue, respiratory tract, urogenital tract, and intestinal tract. It is pathogenic, but usually in association with other kinds of organisms.'),
  'D001444': NtO(MH='Bahamas', MS="A chain of islands, cays, and reefs in the West Indies, lying southeast of Florida and north of Cuba. It is an independent state, called also the Commonwealth of the Bahamas or the Bahama Islands. The name likely represents the local name Guanahani, itself of uncertain origin. (From Webster's New Geographical Dictionary, 1988, p106 & Room, Brewer's Dictionary of Names, 1992, p45)"),
  'D001445': NtO(MH='Bahrain', MS='An independent state, an archipelago in the western Persian Gulf, northwest of Qatar. It comprises low-lying islands of Bahrain (the largest), Muharraq, Sitra, and several islets. It has extensive oil fields. The name comes from the Arabic al-bahrayn, "the two seas", with reference to its lying in the middle of a bay with its "two seas" east and west of it. (From Webster\'s New Geographical Dictionary, 1988, p107 & Room, Brewer\'s Dictionary of Names, 1992, p45)'),
  'D001446': NtO(MH='Balanitis', MS='Inflammation of the head of the PENIS, glans penis.'),
  'D001447': NtO(MH='Balantidiasis', MS='Infection by parasites of the genus BALANTIDIUM. The presence of Balantidium in the LARGE INTESTINE leads to DIARRHEA; DYSENTERY; and occasionally ULCER.'),
  'D001448': NtO(MH='Balantidium', MS='A genus of protozoa parasitic in the digestive tract of vertebrate or invertebrate hosts. Asexual multiplication is accomplished by transverse binary fission. Its organisms are ovoidal in shape and have a ciliated covering over the entire body.'),
  'D001449': NtO(MH='Balkan Nephropathy', MS='A form of chronic interstitial nephritis that is endemic to limited areas of BULGARIA, the former YUGOSLAVIA, and ROMANIA. It is characterized by a progressive shrinking of the KIDNEYS that is often associated with uroepithelial tumors.'),
  'D001450': NtO(MH='Ballistocardiography', MS='Technique of graphic representation of the movements of the body imparted by the ballistic forces (recoil and impact) associated with cardiac contraction and ejection of blood and with the deceleration of blood flow through the large blood vessels. These movements, quantitatively very minute, are translated by a pickup device (transducer) into an electrical potential which is suitably amplified and recorded on a conventional electrocardiograph or other recording machine.'),
  'D001452': NtO(MH='Balneology', MS='Therapy by various hot or warm baths in natural mineral waters, spas, or "cures". It includes not only bathing in, but also drinking the waters, but it does not include whirlpool baths (HYDROTHERAPY).'),
  'D001453': NtO(MH='Balsams', MS='Resinous substances which most commonly originate from trees. In addition to resins, they contain oils, cinnamic acid and BENZOIC ACID.'),
  'D001454': NtO(MH='Baltic States', MS="The collective name for the republics of ESTONIA; LATVIA; and LITHUANIA on the eastern shore of the Baltic Sea. (Webster's New Geographical Dictionary, 1988, p111)"),
  'D001455': NtO(MH='Bambermycins', MS='Antibiotic complex obtained from Streptomyces bambergiensis containing mainly Moenomycins A and C. They are used as feed additives and growth promoters for poultry, swine, and cattle.'),
  'D001457': NtO(MH='Anion Exchange Protein 1, Erythrocyte', MS='A major integral transmembrane protein of the ERYTHROCYTE MEMBRANE. It is the anion exchanger responsible for electroneutral transporting in CHLORIDE IONS in exchange of BICARBONATE IONS allowing CO2 uptake and transport from tissues to lungs by the red blood cells. Genetic mutations that result in a loss of the protein function have been associated with type 4 HEREDITARY SPHEROCYTOSIS.'),
  'D001458': NtO(MH='Bandages', MS='Material used for wrapping or binding any part of the body.'),
  'D001459': NtO(MH='Bangladesh', MS='A country in Southern Asia, bordering the Bay of Bengal, between Burma and India. The capital is Dhaka.'),
  'D001460': NtO(MH='Barbados', MS='An island in the Lesser Antilles in the West Indies. It is chiefly of coral formation with no good harbors and only small streams. It was probably discovered by the Portuguese in the sixteenth century. The name was given by 16th-century Spanish explorers from barbados, the plural for "bearded", with reference to the beard-like leaves or trails of moss on the trees that grew there in abundance. (From Webster\'s New Geographical Dictionary, 1988, p116 & Room, Brewer\'s Dictionary of Names, 1992, p49)'),
  'D001461': NtO(MH='Barbering', MS='The occupation concerned with the cutting and dressing of the hair of customers and, of men, the shaving and trimming of the beard and mustache. (From Random House Unabridged Dictionary, 2d ed)'),
  'D001462': NtO(MH='Barbital', MS='A long-acting barbiturate that depresses most metabolic processes at high doses. It is used as a hypnotic and sedative and may induce dependence. Barbital is also used in veterinary practice for central nervous system depression.'),
  'D001463': NtO(MH='Barbiturates', MS='A class of chemicals derived from barbituric acid or thiobarbituric acid. Many of these are GABA MODULATORS used as HYPNOTICS AND SEDATIVES, as ANESTHETICS, or as ANTICONVULSANTS.'),
  'D001464': NtO(MH='Barium', MS='An element of the alkaline earth group of metals. It has an atomic symbol Ba, atomic number 56, and atomic weight 138. All of its acid-soluble salts are poisonous.'),
  'D001465': NtO(MH='Barium Radioisotopes', MS='Unstable isotopes of barium that decay or disintegrate emitting radiation. Ba atoms with atomic weights 126-129, 131, 133, and 139-143 are radioactive barium isotopes.'),
  'D001466': NtO(MH='Barium Sulfate', MS='A compound used as an x-ray contrast medium that occurs in nature as the mineral barite. It is also used in various manufacturing applications and mixed into heavy concrete to serve as a radiation shield.'),
  'D001467': NtO(MH='Hordeum', MS='A plant genus of the family POACEAE. The EDIBLE GRAIN, barley, is widely used as food.'),
  'D001468': NtO(MH='Thoracica', MS='A superorder of marine CRUSTACEA, free swimming in the larval state, but permanently fixed as adults. There are some 800 described species, grouped in several genera, and comprising of two major orders of barnacles: stalked (Pedunculata) and sessile (Sessilia).'),
  'D001469': NtO(MH='Barotrauma', MS='Injury following pressure changes; includes injury to the eustachian tube, ear drum, lung and stomach.'),
  'D001471': NtO(MH='Barrett Esophagus', MS="A condition with damage to the lining of the lower ESOPHAGUS resulting from chronic acid reflux (ESOPHAGITIS, REFLUX). Through the process of metaplasia, the squamous cells are replaced by a columnar epithelium with cells resembling those of the INTESTINE or the salmon-pink mucosa of the STOMACH. Barrett's columnar epithelium is a marker for severe reflux and precursor to ADENOCARCINOMA of the esophagus."),
  'D001472': NtO(MH="Bartholin's Glands", MS='Mucus-secreting glands situated on the posterior and lateral aspect of the vestibule of the vagina.'),
  'D001473': NtO(MH='Bartonella', MS='A genus of gram-negative bacteria characteristically appearing in chains of several segmenting organisms. It occurs in man and arthropod vectors and is found only in the Andes region of South America. This genus is the etiologic agent of human bartonellosis. The genus Rochalimaea, once considered a separate genus, has recently been combined with the genus Bartonella as a result of high levels of relatedness in 16S rRNA sequence data and DNA hybridization data.'),
  'D001474': NtO(MH='Bartonella Infections', MS='Infections by the genus BARTONELLA. Bartonella bacilliformis can cause acute febrile anemia, designated Oroya fever, and a benign skin eruption, called verruga peruana. BARTONELLA QUINTANA causes TRENCH FEVER, while BARTONELLA HENSELAE is the etiologic agent of bacillary angiomatosis (ANGIOMATOSIS, BACILLARY) and is also one of the causes of CAT-SCRATCH DISEASE.'),
  'D001475': NtO(MH='Bartonellaceae', MS='A family of small gram-negative bacteria whose organisms are parasites of erythrocytes in man and other vertebrates and the etiologic agents of several diseases.'),
  'D001476': NtO(MH='Bartonellaceae Infections', MS='Infections with bacteria of the family BARTONELLACEAE.'),
  'D001477': NtO(MH='Bartter Syndrome', MS='A group of disorders caused by defective salt reabsorption in the ascending LOOP OF HENLE. It is characterized by severe salt-wasting, HYPOKALEMIA; HYPERCALCIURIA; metabolic ALKALOSIS, and hyper-reninemic HYPERALDOSTERONISM without HYPERTENSION. There are several subtypes including ones due to mutations in the renal specific SODIUM-POTASSIUM-CHLORIDE SYMPORTERS.'),
  'D001478': NtO(MH='Basal Cell Nevus Syndrome', MS='Hereditary disorder consisting of multiple basal cell carcinomas, odontogenic keratocysts, and multiple skeletal defects, e.g., frontal and temporoparietal bossing, bifurcated and splayed ribs, kyphoscoliosis, fusion of vertebrae, and cervicothoracic spina bifida. Genetic transmission is autosomal dominant.'),
  'D001479': NtO(MH='Basal Ganglia', MS='Large subcortical nuclear masses derived from the telencephalon and located in the basal regions of the cerebral hemispheres.'),
  'D001480': NtO(MH='Basal Ganglia Diseases', MS='Diseases of the BASAL GANGLIA including the PUTAMEN; GLOBUS PALLIDUS; claustrum; AMYGDALA; and CAUDATE NUCLEUS. DYSKINESIAS (most notably involuntary movements and alterations of the rate of movement) represent the primary clinical manifestations of these disorders. Common etiologies include CEREBROVASCULAR DISORDERS; NEURODEGENERATIVE DISEASES; and CRANIOCEREBRAL TRAUMA.'),
  'D001481': NtO(MH='Basal Metabolism', MS='Metabolism (e.g., heat production) of an organism in an inactive, awake, fasting state. It may be determined directly by means of a calorimeter or indirectly by calculating the end products of oxidation within the organism or from the amount of oxygen utilized.'),
  'D001482': NtO(MH='Base Composition', MS='The relative amounts of the PURINES and PYRIMIDINES in a nucleic acid.'),
  'D001483': NtO(MH='Base Sequence', MS='The sequence of PURINES and PYRIMIDINES in nucleic acids and polynucleotides. It is also called nucleotide sequence.'),
  'D001484': NtO(MH='Baseball', MS='A competitive nine-member team sport including softball.'),
  'D001485': NtO(MH='Basement Membrane', MS='A darkly stained mat-like EXTRACELLULAR MATRIX (ECM) that separates cell layers, such as EPITHELIUM from ENDOTHELIUM or a layer of CONNECTIVE TISSUE. The ECM layer that supports an overlying EPITHELIUM or ENDOTHELIUM is called basal lamina. Basement membrane (BM) can be formed by the fusion of either two adjacent basal laminae or a basal lamina with an adjacent reticular lamina of connective tissue. BM, composed mainly of TYPE IV COLLAGEN; glycoprotein LAMININ; and PROTEOGLYCAN, provides barriers as well as channels between interacting cell layers.'),
  'D001486': NtO(MH='Bashkiria', MS="A political subdivision of eastern RUSSIA located within Europe. It consists of a plateau and mountainous area of the Southern Urals. (From Webster's New Geographical Dictionary, 1997)"),
  'D001487': NtO(MH='Basidiomycota', MS="A phylum of fungi that produce their sexual spores (basidiospores) on the outside of the basidium. It includes forms commonly known as mushrooms, boletes, puffballs, earthstars, stinkhorns, bird's-nest fungi, jelly fungi, bracket or shelf fungi, and rust and smut fungi."),
  'D001488': NtO(MH='Basilar Artery', MS='The artery formed by the union of the right and left vertebral arteries; it runs from the lower to the upper border of the pons, where it bifurcates into the two posterior cerebral arteries.'),
  'D001489': NtO(MH='Basilar Membrane', MS='A basement membrane in the cochlea that supports the hair cells of the ORGAN OF CORTI, consisting keratin-like fibrils. It stretches from the SPIRAL LAMINA to the basilar crest. The movement of fluid in the cochlea, induced by sound, causes displacement of the basilar membrane and subsequent stimulation of the attached hair cells which transform the mechanical signal into neural activity.'),
  'D001490': NtO(MH='Basketball', MS='A competitive team sport played on a rectangular court having a raised basket at each end.'),
  'D001491': NtO(MH='Basophils', MS='Granular leukocytes characterized by a relatively pale-staining, lobate nucleus and cytoplasm containing coarse dark-staining granules of variable size and stainable by basic dyes.'),
  'D001492': NtO(MH='Bass', MS='Common name for FISHES belonging to the order Perciformes and occurring in three different families.'),
  'D001493': NtO(MH='Bathing Beaches', MS='Beaches, both natural and man-made, used for bathing and other activities.'),
  'D001494': NtO(MH='Baths', MS='The immersion or washing of the body or any of its parts in water or other medium for cleansing or medical treatment. It includes bathing for personal hygiene as well as for medical purposes with the addition of therapeutic agents, such as alkalines, antiseptics, oil, etc.'),
  'D001496': NtO(MH='Batrachotoxins', MS='Batrachotoxin is the 20-alpha-bromobenzoate of batrachotoxin A; they are toxins from the venom of a small Colombian frog, Phyllobates aurotaenia, cause release of acetylcholine, destruction of synaptic vesicles and depolarization of nerve and muscle fibers.'),
  'D001497': NtO(MH='Battered Child Syndrome', MS='A clinical condition resulting from repeated physical and psychological injuries inflicted on a child by the parents or caregivers.'),
  'D001498': NtO(MH='3-Pyridinecarboxylic acid, 1,4-dihydro-2,6-dimethyl-5-nitro-4-(2-(trifluoromethyl)phenyl)-, Methyl ester', MS='A dihydropyridine derivative, which, in contrast to NIFEDIPINE, functions as a calcium channel agonist. The compound facilitates Ca2+ influx through partially activated voltage-dependent Ca2+ channels, thereby causing vasoconstrictor and positive inotropic effects. It is used primarily as a research tool.'),
  'D001499': NtO(MH='Bayes Theorem', MS='A theorem in probability theory named for Thomas Bayes (1702-1761). In epidemiology, it is used to obtain the probability of disease in a group of people with some characteristic on the basis of the overall rate of that disease and of the likelihood of that characteristic in healthy and diseased individuals. The most familiar application is in clinical decision analysis where it is used for estimating the probability of a particular diagnosis given the appearance of some symptoms or test result.'),
  'D001500': NtO(MH='BCG Vaccine', MS='An active immunizing agent and a viable avirulent attenuated strain of Mycobacterium tuberculosis, var. bovis, which confers immunity to mycobacterial infections. It is used also in immunotherapy of neoplasms due to its stimulation of antibodies and non-specific immunity.'),
  'D001501': NtO(MH='Bdellovibrio', MS='A genus of bacteria capable of developing within other bacteria.'),
  'D001502': NtO(MH='Beak', MS='In some animals, the jaws together with their horny covering. The beak usually refers to the bill of birds in which the whole varies greatly in form according of the food and habits of the bird. While the beak refers most commonly to birds, the anatomical counterpart is found also in the turtle, squid, and octopus. (From Webster, 3d ed & Storer, et al., General Zoology, 6th ed, p491, 755)'),
  'D001503': NtO(MH='Ursidae', MS='The family of carnivorous or omnivorous bears, having massive bodies, coarse heavy fur, relatively short limbs, and almost rudimentary tails.'),
  'D001504': NtO(MH='Beauty', MS='Characteristics or attributes of persons or things which elicit pleasurable feelings.'),
  'D001505': NtO(MH='Beauty Culture', MS='An industry that creates products and procedures designed to enhance physical appearance and aesthetic appeal.'),
  'D001506': NtO(MH='Beckwith-Wiedemann Syndrome', MS='A syndrome of multiple defects characterized primarily by umbilical hernia (HERNIA, UMBILICAL); MACROGLOSSIA; and GIGANTISM; and secondarily by visceromegaly; HYPOGLYCEMIA; and ear abnormalities.'),
  'D001507': NtO(MH='Beclomethasone', MS='An anti-inflammatory, synthetic glucocorticoid. It is used topically as an anti-inflammatory agent and in aerosol form for the treatment of ASTHMA.'),
  'D001508': NtO(MH='Bed Conversion', MS='The reallocation of beds from one type of care service to another, as in converting acute care beds to long term care beds.'),
  'D001509': NtO(MH='Bed Occupancy', MS='A measure of inpatient health facility use based upon the average number or proportion of beds occupied for a given period of time.'),
  'D001510': NtO(MH='Bed Rest', MS='Confinement of an individual to bed for therapeutic or experimental reasons.'),
  'D001511': NtO(MH='Bedbugs', MS='Parasitic bugs of the family CIMICIDAE. Female bed bugs lay eggs in a sheltered locations such as in mattress seams. Adults and nymphal instars of Cimex feed on warm-blooded hosts. Cimex lectularius and Cimex hemipterus are implicated in human infestations.'),
  'D001512': NtO(MH='Bedding and Linens', MS="Articles of cloth, usually cotton or rayon and other synthetic or cotton-blend fabrics, used in households, hospitals, physicians' examining rooms, nursing homes, etc., for sheets, pillow cases, toweling, gowns, drapes, and the like."),
  'D001513': NtO(MH='Beds', MS='Equipment on which one may lie and sleep, especially as used to care for the hospital patient.'),
  'D001514': NtO(MH='Bee Venoms', MS='Toxins obtained from Apis mellifera (honey bee) and related species. They contain various enzymes, polypeptide toxins, and other substances, some of which are allergenic or immunogenic or both. These venoms were formerly used in rheumatism to stimulate the pituitary-adrenal system.'),
  'D001515': NtO(MH='Beer', MS='An alcoholic beverage usually made from malted cereal grain (as barley), flavored with hops, and brewed by slow fermentation.'),
  'D001516': NtO(MH='Bees', MS='Insect members of the superfamily Apoidea, found almost everywhere, particularly on flowers. About 3500 species occur in North America. They differ from most WASPS in that their young are fed honey and pollen rather than animal food.'),
  'D001517': NtO(MH='Coleoptera', MS='Order of winged insects also known as beetles comprising over 350,000 species in 150 families. They possess hard bodies with mouthparts adapted for chewing.'),
  'D001519': NtO(MH='Behavior', MS='The observable response of a man or animal to a situation.'),
  'D001520': NtO(MH='Behavior and Behavior Mechanisms', MS='The observable response made to a situation and the unconscious processes underlying it.'),
  'D001521': NtO(MH='Behavior Therapy', MS='The application of modern theories of learning and conditioning in the treatment of behavior disorders.'),
  'D001522': NtO(MH='Behavior, Animal', MS='The observable response an animal makes to any situation.'),
  'D001523': NtO(MH='Mental Disorders', MS='Psychiatric illness or diseases manifested by breakdowns in the adaptational process expressed primarily as abnormalities of thought, feeling, and behavior producing either distress or impairment of function.'),
  'D001524': NtO(MH='Behavioral Medicine', MS='The interdisciplinary field concerned with the development and integration of behavioral and biomedical science, knowledge, and techniques relevant to health and illness and the application of this knowledge and these techniques to prevention, diagnosis, treatment, and rehabilitation.'),
  'D001525': NtO(MH='Behavioral Sciences', MS='Disciplines concerned with the study of human and animal behavior.'),
  'D001526': NtO(MH='Behavioral Symptoms', MS='Observable manifestations of impaired psychological functioning.'),
  'D001527': NtO(MH='Behaviorism', MS='A psychologic theory, developed by John Broadus Watson, concerned with studying and measuring behaviors that are observable.'),
  'D001528': NtO(MH='Behcet Syndrome', MS='Rare chronic inflammatory disease involving the small blood vessels. It is of unknown etiology and characterized by mucocutaneous ulceration in the mouth and genital region and uveitis with hypopyon. The neuro-ocular form may cause blindness and death. SYNOVITIS; THROMBOPHLEBITIS; gastrointestinal ulcerations; RETINAL VASCULITIS; and OPTIC ATROPHY may occur as well.'),
  'D001530': NtO(MH='Belgium', MS='A country in Western Europe, bordering the North Sea, between France and the Netherlands. The capital is Brussels.'),
  'D001531': NtO(MH='Belize', MS='A country in CENTRAL AMERICA, bordering the Caribbean Sea, between GUATEMALA and MEXICO.'),
  'D001532': NtO(MH='Atropa belladonna', MS='A plant species of the genus ATROPA, family SOLANACEAE that contains ATROPINE; SCOPOLAMINE; BELLADONNA ALKALOIDS and other SOLANACEOUS ALKALOIDS. Some species in this genus are called deadly nightshade which is also a common name for SOLANUM.'),
  'D001533': NtO(MH='Belladonna Alkaloids', MS='Alkaloids obtained from various plants, especially the deadly nightshade (Atropa belladonna), variety acuminata; atropine, hyoscyamine and scopolamine are classical, specific antimuscarinic agents with many pharmacologic actions; used mainly as antispasmodics.'),
  'D001534': NtO(MH='Bemegride', MS='A CNS stimulant that is used to induce convulsions in experimental animals. It has also been used as a respiratory stimulant and in the treatment of barbiturate overdose.'),
  'D001535': NtO(MH='Benactyzine', MS='A centrally acting muscarinic antagonist. Benactyzine has been used in the treatment of depression and is used in research to investigate the role of cholinergic systems on behavior.'),
  'D001536': NtO(MH='Bence Jones Protein', MS='An abnormal protein with unusual thermosolubility characteristics that is found in the urine of patients with MULTIPLE MYELOMA.'),
  'D001537': NtO(MH='Bencyclane', MS='A vasodilator agent found to be effective in a variety of peripheral circulation disorders. It has various other potentially useful pharmacological effects. Its mechanism may involve block of calcium channels.'),
  'D001538': NtO(MH='Bender-Gestalt Test', MS='A psychological test consisting of nine geometric designs on cards. The subject is asked to redraw them from memory after each one is presented individually.'),
  'D001539': NtO(MH='Bendroflumethiazide', MS='A thiazide diuretic with actions and uses similar to those of HYDROCHLOROTHIAZIDE. It has been used in the treatment of familial hyperkalemia, hypertension, edema, and urinary tract disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p810)'),
  'D001541': NtO(MH='Benin', MS='A country in western Africa, south of NIGER and between TOGO and NIGERIA. Its capital is Porto-Novo.'),
  'D001542': NtO(MH='Benomyl', MS='A systemic agricultural fungicide used for control of certain fungal diseases of stone fruit.'),
  'D001544': NtO(MH='Benperidol', MS='A butyrophenone with general properties similar to those of HALOPERIDOL. It has been used in the treatment of aberrant sexual behavior. (From Martindale, The Extra Pharmacopoeia, 30th ed, p567)'),
  'D001545': NtO(MH='Benserazide', MS='An inhibitor of DOPA DECARBOXYLASE that does not enter the central nervous system. It is often given with LEVODOPA in the treatment of parkinsonism to prevent the conversion of levodopa to dopamine in the periphery, thereby increasing the amount that reaches the central nervous system and reducing the required dose. It has no antiparkinson actions when given alone.'),
  'D001546': NtO(MH='Bentonite', MS='A colloidal, hydrated aluminum silicate that swells 12 times its dry size when added to water.'),
  'D001547': NtO(MH='Benzaldehydes', MS='Compounds that consist of a benzene ring with a formyl group. They occur naturally in the seeds of many fruits and are used as FLAVORING AGENTS and fragrances, as well as precursors in pharmaceutical and plastics manufacturing.'),
  'D001548': NtO(MH='Benzalkonium Compounds', MS='A mixture of alkylbenzyldimethylammonium compounds. It is a bactericidal quaternary ammonium detergent used topically in medicaments, deodorants, mouthwashes, as a surgical antiseptic, and as a as preservative and emulsifier in drugs and cosmetics.'),
  'D001549': NtO(MH='Benzamides', MS='BENZOIC ACID amides.'),
  'D001550': NtO(MH='Benzamidines', MS='Amidines substituted with a benzene group. Benzamidine and its derivatives are known as peptidase inhibitors.'),
  'D001551': NtO(MH='Benz(a)Anthracenes', MS='Four fused benzyl rings with three linear and one angular, that can be viewed as a benzyl-phenanthrenes. Compare with NAPHTHACENES which are four linear rings.'),
  'D001552': NtO(MH='Benzazepines', MS='Compounds with BENZENE fused to AZEPINES.'),
  'D001553': NtO(MH='Benzbromarone', MS='Uricosuric that acts by increasing uric acid clearance. It is used in the treatment of gout.'),
  'D001554': NtO(MH='Benzene', MS='Toxic, volatile, flammable liquid hydrocarbon byproduct of coal distillation. It is used as an industrial solvent in paints, varnishes, lacquer thinners, gasoline, etc. Benzene causes central nervous system damage acutely and bone marrow damage chronically and is carcinogenic. It was formerly used as parasiticide.'),
  'D001555': NtO(MH='Benzene Derivatives', MS='Organic compounds derived from BENZENE.'),
  'D001556': NtO(MH='Hexachlorocyclohexane', MS='Any of several polyhalogenated organic compounds consisting of a six-carbon ring with one chlorine and one hydrogen attached to each carbon.'),
  'D001557': NtO(MH='Benzenesulfonates', MS='Organic salts and esters of benzenesulfonic acid.'),
  'D001558': NtO(MH='Benzethonium', MS='Bactericidal cationic quaternary ammonium surfactant used as a topical anti-infective agent. It is an ingredient in medicaments, deodorants, mouthwashes, etc., and is used to disinfect apparatus, etc., in the food processing and pharmaceutical industries, in surgery, and also as a preservative. The compound is toxic orally as a result of neuromuscular blockade.'),
  'D001559': NtO(MH='Benzhydryl Compounds', MS='Compounds which contain the methyl radical substituted with two benzene rings. Permitted are any substituents, but ring fusion to any of the benzene rings is not allowed.'),
  'D001560': NtO(MH='Benzidines', MS='Very toxic industrial chemicals. They are absorbed through the skin, causing lethal blood, bladder, liver, and kidney damage and are potent, broad-spectrum carcinogens in most species.'),
  'D001561': NtO(MH='Benzilates', MS='Derivatives of benzylic acid, including its salts and esters,'),
  'D001562': NtO(MH='Benzimidazoles', MS='Compounds with a BENZENE fused to IMIDAZOLES.'),
  'D001564': NtO(MH='Benzo(a)pyrene', MS='A potent mutagen and carcinogen. It is a public health concern because of its possible effects on industrial workers, as an environmental pollutant, an as a component of tobacco smoke.'),
  'D001565': NtO(MH='Benzoates', MS='Derivatives of BENZOIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the carboxybenzene structure.'),
  'D001566': NtO(MH='Benzocaine', MS='A surface anesthetic that acts by preventing transmission of impulses along NERVE FIBERS and at NERVE ENDINGS.'),
  'D001567': NtO(MH='Benzocycloheptenes', MS='Compounds consisting of the cycloheptene ring with one or two BENZENE rings fused to it.'),
  'D001569': NtO(MH='Benzodiazepines', MS='A group of two-ring heterocyclic compounds consisting of a benzene ring fused to a diazepine ring.'),
  'D001570': NtO(MH='Benzodiazepinones', MS=None),
  'D001571': NtO(MH='Benzoflavones', MS='Organic compounds containing a BENZENE ring attached to a flavone group. Some of these are potent arylhydrocarbon hydroxylase inhibitors. They may also inhibit the binding of NUCLEIC ACIDS to BENZOPYRENES and related compounds. The designation includes all isomers; the 7,8-isomer is most frequently encountered.'),
  'D001572': NtO(MH='Benzofurans', MS='Compounds that contain a BENZENE ring fused to a furan ring.'),
  'D001573': NtO(MH='Benzoin', MS='A white crystalline compound prepared by condensation of benzaldehyde in potassium cyanide and used in organic syntheses. This should not be confused with benzoin gum from STYRAX.'),
  'D001574': NtO(MH='Benzolamide', MS='Selective renal carbonic anhydrase inhibitor. It may also be of use in certain cases of respiratory failure.'),
  'D001575': NtO(MH='Benzomorphans', MS='Morphine derivatives of the methanobenzazocine family that act as potent analgesics.'),
  'D001576': NtO(MH='Benzophenoneidum', MS='An aniline dye used as a disinfectant and an antiseptic agent. It is weakly fluorescing and binds specifically to certain proteins.'),
  'D001577': NtO(MH='Benzophenones', MS='Derivatives of benzophenone (with the structural formula phenyl-(C=O)-phenyl).'),
  'D001578': NtO(MH='Benzopyrans', MS='Compounds with a core of fused benzo-pyran rings.'),
  'D001579': NtO(MH='Benzopyrene Hydroxylase', MS='A drug-metabolizing, cytochrome P-448 (P-450) enzyme which catalyzes the hydroxylation of benzopyrene to 3-hydroxybenzopyrene in the presence of reduced flavoprotein and molecular oxygen. Also acts on certain anthracene derivatives. An aspect of EC 1.14.14.1.'),
  'D001580': NtO(MH='Benzopyrenes', MS='A class of chemicals that contain an anthracene ring with a naphthalene ring attached to it.'),
  'D001581': NtO(MH='Benzothiadiazines', MS='Heterocyclic compounds of a ring with SULFUR and two NITROGEN atoms fused to a BENZENE ring. Members inhibit SODIUM-POTASSIUM-CHLORIDE SYMPORTERS and are used as DIURETICS.'),
  'D001582': NtO(MH='Benzothiepins', MS=None),
  'D001583': NtO(MH='Benzoxazoles', MS=None),
  'D001584': NtO(MH='Benzoxepins', MS=None),
  'D001585': NtO(MH='Benzoyl Peroxide', MS='A peroxide derivative that has been used topically for BURNS and as a dermatologic agent in the treatment of ACNE and POISON IVY DERMATITIS. It is used also as a bleach in the food industry.'),
  'D001586': NtO(MH='Benzoylarginine Nitroanilide', MS='A chromogenic substrate that permits direct measurement of peptide hydrolase activity, e.g., papain and trypsin, by colorimetry. The substrate liberates p-nitroaniline as a chromogenic product.'),
  'D001587': NtO(MH='Benzoylarginine-2-Naphthylamide', MS='An enzyme substrate which permits the measurement of peptide hydrolase activity, e.g. trypsin and thrombin. The enzymes liberate 2-naphthylamine, which is measured by colorimetric procedures.'),
  'D001588': NtO(MH='Benzoylcholine', MS='The benzoic acid ester of choline.'),
  'D001589': NtO(MH='Benzphetamine', MS='A sympathomimetic agent with properties similar to DEXTROAMPHETAMINE. It is used in the treatment of obesity. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1222)'),
  'D001590': NtO(MH='Benztropine', MS='A centrally active muscarinic antagonist that has been used in the symptomatic treatment of PARKINSON DISEASE. Benztropine also inhibits the uptake of dopamine.'),
  'D001591': NtO(MH='Benzydamine', MS='A benzyl-indazole having analgesic, antipyretic, and anti-inflammatory effects. It is used to reduce post-surgical and post-traumatic pain and edema and to promote healing. It is also used topically in treatment of RHEUMATIC DISEASES and INFLAMMATION of the mouth and throat.'),
  'D001592': NtO(MH='Benzyl Alcohols', MS='Alcohols derived from the aryl radical (C6H5CH2-) and defined by C6H5CHOH. The concept includes derivatives with any substituents on the benzene ring.'),
  'D001593': NtO(MH='Benzyl Compounds', MS='Benzene derivatives that contain a benzyl group with the general formula Phenyl-CH2-R.'),
  'D001594': NtO(MH='Benzyl Viologen', MS="1,1'-Bis(phenylmethyl)4,4'-bipyridinium dichloride. Oxidation-reduction indicator."),
  'D001595': NtO(MH='Benzylamine Oxidase', MS='An aspect of monoamine oxidase, EC 1.4.3.4. Catalyzes the oxidation of benzylamine to form benzaldehyde, ammonia and hydrogen peroxide.'),
  'D001596': NtO(MH='Benzylamines', MS='Toluenes in which one hydrogen of the methyl group is substituted by an amino group. Permitted are any substituents on the benzene ring or the amino group.'),
  'D001597': NtO(MH='Benzylidene Compounds', MS='Compounds which include a double-bonded carbon atom that is directly attached to a benzene ring. While this category is named after the highly-reactive compound benzylidene, the compounds listed under it occur through a variety of synthetic pathways.'),
  'D001598': NtO(MH='Bephenium Compounds', MS='Analogs or derivatives of bephenium (N,N-dimethyl-N-(2-phenoxyethyl)benzenemethanaminium).'),
  'D001599': NtO(MH='Berberine', MS='An alkaloid from Hydrastis canadensis L., Berberidaceae. It is also found in many other plants. It is relatively toxic parenterally, but has been used orally for various parasitic and fungal infections and as antidiarrheal.'),
  'D001600': NtO(MH='Berberine Alkaloids', MS='A group of related plant alkaloids that contain the BERBERINE heterocyclic ring structure.'),
  'D001601': NtO(MH='Bereavement', MS='Refers to the whole process of grieving and mourning and is associated with a deep sense of loss and sadness.'),
  'D001602': NtO(MH='Beriberi', MS='A disease caused by a deficiency of thiamine (vitamin B1) and characterized by polyneuritis, cardiac pathology, and edema. The epidemic form is found primarily in areas in which white (polished) rice is the staple food, as in Japan, China, the Philippines, India, and other countries of southeast Asia. (Dorland, 27th ed)'),
  'D001603': NtO(MH='Berkelium', MS='A man-made radioactive actinide with atomic symbol Bk, atomic number 97, and atomic weight 247. Its valence can be +3 or +4. Twelve isotopes have been produced with mass numbers 240-251.'),
  'D001604': NtO(MH='Berlin', MS='The capital of Germany.'),
  'D001605': NtO(MH='Bermuda', MS="A British colony in the western North Atlantic Ocean about 640 miles east southeast of Cape Hatteras, North Carolina. It comprises a group of about 300 islands of which only about 20 are inhabited. It is called also the Bermuda Islands or the Bermudas. It was named for the Spanish explorer Juan Bermudez who visited the islands in 1515. (From Webster's New Geographical Dictionary, 1988, p140 & Room, Brewer's Dictionary of Names, 1992, p61)"),
  'D001606': NtO(MH='Bernard-Soulier Syndrome', MS='A familial coagulation disorder characterized by a prolonged bleeding time, unusually large platelets, and impaired prothrombin consumption.'),
  'D001607': NtO(MH='Berylliosis', MS='A form of pneumoconiosis caused by inhaled rare metal BERYLLIUM or its soluble salts which are used in a wide variety of industry including alloys, ceramics, radiographic equipment, and vacuum tubes. Berylliosis is characterized by an acute inflammatory reaction in the upper airway leading to BRONCHIOLITIS; PULMONARY EDEMA; and pneumonia.'),
  'D001608': NtO(MH='Beryllium', MS='An element with the atomic symbol Be, atomic number 4, and atomic weight 9.01218. Short exposure to this element can lead to a type of poisoning known as BERYLLIOSIS.'),
  'D001609': NtO(MH='Beta-Globulins', MS='Serum proteins with an electrophoretic mobility that falls between ALPHA-GLOBULINS and GAMMA-GLOBULINS.'),
  'D001610': NtO(MH='Beta Particles', MS='High energy POSITRONS or ELECTRONS ejected from a disintegrating atomic nucleus.'),
  'D001611': NtO(MH='Beta Rhythm', MS='Brain waves with frequency between 15-30 Hz seen on EEG during wakefulness and mental activity.'),
  'D001613': NtO(MH='beta 2-Microglobulin', MS='An 11-kDa protein associated with the outer membrane of many cells including LYMPHOCYTES. It is the small subunit of MHC CLASS I MOLECULES. Association with beta 2-microglobulin is generally required for the transport of class I heavy chains from the endoplasmic reticulum to the cell surface. Beta 2-microglobulin is present in small amounts in serum, CEREBROSPINAL FLUID, and urine of healthy individuals, and to a much greater degree in the urine and plasma of patients with tubular PROTEINURIA, renal failure, or kidney transplants.'),
  'D001614': NtO(MH='beta-Amylase', MS='An enzyme that catalyzes the hydrolysis of 1,4-alpha-glycosidic linkages in starch, glycogen, and related polysaccharides and oligosaccharides so as to remove successive beta-maltose units from the non-reducing ends of the chains. EC 3.2.1.2.'),
  'D001615': NtO(MH='beta-Endorphin', MS='A 31-amino acid peptide that is the C-terminal fragment of BETA-LIPOTROPIN. It acts on OPIOID RECEPTORS and is an analgesic. Its first four amino acids at the N-terminal are identical to the tetrapeptide sequence of METHIONINE ENKEPHALIN and LEUCINE ENKEPHALIN.'),
  'D001616': NtO(MH='beta-Galactosidase', MS='A group of enzymes that catalyzes the hydrolysis of terminal, non-reducing beta-D-galactose residues in beta-galactosides. Deficiency of beta-Galactosidase A1 may cause GANGLIOSIDOSIS, GM1.'),
  'D001617': NtO(MH='beta-Glucosidase', MS='An exocellulase with specificity for a variety of beta-D-glycoside substrates. It catalyzes the hydrolysis of terminal non-reducing residues in beta-D-glucosides with release of GLUCOSE.'),
  'D001618': NtO(MH='beta-Lactamases', MS='Enzymes found in many bacteria which catalyze the hydrolysis of the amide bond in the beta-lactam ring. Well known antibiotics destroyed by these enzymes are penicillins and cephalosporins.'),
  'D001619': NtO(MH='beta-N-Acetylhexosaminidases', MS='A hexosaminidase specific for non-reducing N-acetyl-D-hexosamine residues in N-acetyl-beta-D-hexosaminides. It acts on GLUCOSIDES; GALACTOSIDES; and several OLIGOSACCHARIDES. Two specific mammalian isoenzymes of beta-N-acetylhexoaminidase are referred to as HEXOSAMINIDASE A and HEXOSAMINIDASE B. Deficiency of the type A isoenzyme causes TAY-SACHS DISEASE, while deficiency of both A and B isozymes causes SANDHOFF DISEASE. The enzyme has also been used as a tumor marker to distinguish between malignant and benign disease.'),
  'D001620': NtO(MH='beta-Thromboglobulin', MS='A platelet-specific protein which is released when platelets aggregate. Elevated plasma levels have been reported after deep venous thrombosis, pre-eclampsia, myocardial infarction with mural thrombosis, and myeloproliferative disorders. Measurement of beta-thromboglobulin in biological fluids by radioimmunoassay is used for the diagnosis and assessment of progress of thromboembolic disorders.'),
  'D001621': NtO(MH='Betahistine', MS='A histamine analog and H1 receptor agonist that serves as a vasodilator. It is used in MENIERE DISEASE and in vascular headaches but may exacerbate bronchial asthma and peptic ulcers.'),
  'D001622': NtO(MH='Betaine', MS='A naturally occurring compound that has been of interest for its role in osmoregulation. As a drug, betaine hydrochloride has been used as a source of hydrochloric acid in the treatment of hypochlorhydria. Betaine has also been used in the treatment of liver disorders, for hyperkalemia, for homocystinuria, and for gastrointestinal disturbances. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1341)'),
  'D001623': NtO(MH='Betamethasone', MS='A glucocorticoid given orally, parenterally, by local injection, by inhalation, or applied topically in the management of various disorders in which corticosteroids are indicated. Its lack of mineralocorticoid properties makes betamethasone particularly suitable for treating cerebral edema and congenital adrenal hyperplasia. (From Martindale, The Extra Pharmacopoeia, 30th ed, p724)'),
  'D001624': NtO(MH='Betamethasone Valerate', MS='The 17-valerate derivative of BETAMETHASONE. It has substantial topical anti-inflammatory activity and relatively low systemic anti-inflammatory activity.'),
  'D001625': NtO(MH='Betazole', MS='A histamine H2 agonist used clinically to test gastric secretory function.'),
  'D001626': NtO(MH='Bethanechol Compounds', MS='Quaternary ammonium compounds that include BETHANECHOL.'),
  'D001627': NtO(MH='Bethanidine', MS='A guanidinium antihypertensive agent that acts by blocking adrenergic transmission. The precise mode of action is not clear.'),
  'D001628': NtO(MH='Beverages', MS='Liquids that are suitable for drinking. (From Merriam Webster Collegiate Dictionary, 10th ed)'),
  'D001629': NtO(MH='Bezafibrate', MS='An antilipemic agent that lowers CHOLESTEROL and TRIGLYCERIDES. It decreases LOW DENSITY LIPOPROTEINS and increases HIGH DENSITY LIPOPROTEINS.'),
  'D001630': NtO(MH='Bezoars', MS='Concretions of swallowed hair, fruit or vegetable fibers, or similar substances found in the alimentary canal.'),
  'D001631': NtO(MH='Bhutan', MS="A kingdom in the eastern Himalayas on the northeast border of India, bounded on the north by Tibet, on the east by Assam, on the south by Assam and West Bengal, and on the west by Sikkim and Tibet. From 1720 to 1970 it was under Chinese or Indian domination. In 1971 it became a member of the United Nations. The name comes from the Sanskrit bhota, the name for Tibet, + anta, end, with reference to its location at the southern extremity of Tibet. (From Webster's New Geographical Dictionary, 1988, p144 & Room, Brewer's Dictionary of Names, 1992, p64)"),
  'D001632': NtO(MH='Bibenzyls', MS='Compounds that include 1,2-diphenylethane in their structure.'),
  'D001633': NtO(MH='Bible', MS='The book composed of writings generally accepted by Christians as inspired by God and of divine authority. (Webster, 3d ed)'),
  'D001634': NtO(MH='Bibliographies as Topic', MS='Works about lists of books, articles, and other publications.'),
  'D001635': NtO(MH='Bibliography of Medicine', MS='A list of works, documents, and other publications on medical subjects and topics of interest to the field of medicine.'),
  'D001636': NtO(MH='Bibliography, Descriptive', MS="The area of bibliography which makes known precisely the material conditions of books, i.e., the full name of the author, the exact title of the work, the date and place of publication, the publisher's and printer's names, the format, the pagination, typographical particulars, illustrations, and the price, and for old books, other characteristics such as the kind of paper, binding, etc. It is also called analytical bibliography and physical bibliography. (Harrod's Librarians' Glossary, 7th ed)"),
  'D001637': NtO(MH='Bibliography, National', MS="A bibliography which lists all the books and other publications published, or distributed in significant quantity, in a particular country. Sometimes the term is used with respect to the new publications published within a specific period, and sometimes with respect to all those published within a lengthy period of many years. It is also used to indicate a bibliography of publications about a country (whether written by its nationals or not) and those written in the language of the country as well as those published in it. (Harrod's Librarians' Glossary, 7th ed)"),
  'D001638': NtO(MH='Bibliotherapy', MS='A form of supportive psychotherapy in which the patient is given carefully selected material to read.'),
  'D001639': NtO(MH='Bicarbonates', MS='Inorganic salts that contain the -HCO3 radical. They are an important factor in determining the pH of the blood and the concentration of bicarbonate ions is regulated by the kidney. Levels in the blood are an index of the alkali reserve or buffering capacity.'),
  'D001640': NtO(MH='Bicuculline', MS='An isoquinoline alkaloid obtained from Dicentra cucullaria and other plants. It is a competitive antagonist for GABA-A receptors.'),
  'D001641': NtO(MH='Bicuspid', MS='One of the eight permanent teeth, two on either side in each jaw, between the canines (CUSPID) and the molars (MOLAR), serving for grinding and crushing food. The upper have two cusps (bicuspid) but the lower have one to three. (Jablonski, Dictionary of Dentistry, 1992, p822)'),
  'D001642': NtO(MH='Bicycling', MS="The use of a bicycle for transportation or recreation. It does not include the use of a bicycle in studying the body's response to physical exertion (BICYCLE ERGOMETRY TEST see EXERCISE TEST)."),
  'D001643': NtO(MH='Bridged Bicyclo Compounds', MS='Saturated alicyclic hydrocarbon molecules consisting of two rings that have two non-adjacent atoms in common.'),
  'D001644': NtO(MH='Bifidobacterium', MS='A rod-shaped, gram-positive, non-acid-fast, non-spore-forming, non-motile bacterium that is a genus of the family Bifidobacteriaceae, order Bifidobacteriales, class ACTINOBACTERIA. It inhabits the intestines and feces of humans as well as the human vagina.'),
  'D001645': NtO(MH='Biguanides', MS='Derivatives of biguanide (the structure formula HN(C(NH)NH2)2) that are primarily used as oral HYPOGLYCEMIC AGENTS for the treatment of DIABETES MELLITUS, TYPE 2 and PREDIABETES.'),
  'D001646': NtO(MH='Bile', MS='An emulsifying agent produced in the LIVER and secreted into the DUODENUM. Its composition includes BILE ACIDS AND SALTS; CHOLESTEROL; and ELECTROLYTES. It aids DIGESTION of fats in the duodenum.'),
  'D001647': NtO(MH='Bile Acids and Salts', MS='Steroid acids and salts. The primary bile acids are derived from cholesterol in the liver and usually conjugated with glycine or taurine. The secondary bile acids are further modified by bacteria in the intestine. They play an important role in the digestion and absorption of fat. They have also been used pharmacologically, especially in the treatment of gallstones.'),
  'D001648': NtO(MH='Bile Canaliculi', MS='Minute intercellular channels that occur between liver cells and carry bile towards interlobar bile ducts. Also called bile capillaries.'),
  'D001649': NtO(MH='Bile Duct Diseases', MS='Diseases in any part of the ductal system of the BILIARY TRACT from the smallest BILE CANALICULI to the largest COMMON BILE DUCT.'),
  'D001650': NtO(MH='Bile Duct Neoplasms', MS='Tumors or cancer of the BILE DUCTS.'),
  'D001651': NtO(MH='Cholestasis, Extrahepatic', MS='Impairment of bile flow in the large BILE DUCTS by mechanical obstruction or stricture due to benign or malignant processes.'),
  'D001652': NtO(MH='Bile Ducts', MS='The channels that collect and transport the bile secretion from the BILE CANALICULI, the smallest branch of the BILIARY TRACT in the LIVER, through the bile ductules, the bile ducts out the liver, and to the GALLBLADDER for storage.'),
  'D001653': NtO(MH='Bile Ducts, Intrahepatic', MS='Passages within the liver for the conveyance of bile. Includes right and left hepatic ducts even though these may join outside the liver to form the common hepatic duct.'),
  'D001654': NtO(MH='Bile Pigments', MS='Linear TETRAPYRROLES that give a characteristic color to BILE including: BILIRUBIN; BILIVERDIN; and bilicyanin.'),
  'D001655': NtO(MH='Bile Reflux', MS='Retrograde bile flow. Reflux of bile can be from the duodenum to the stomach (DUODENOGASTRIC REFLUX); to the esophagus (GASTROESOPHAGEAL REFLUX); or to the PANCREAS.'),
  'D001656': NtO(MH='Biliary Atresia', MS='Progressive destruction or the absence of all or part of the extrahepatic BILE DUCTS, resulting in the complete obstruction of BILE flow. Usually, biliary atresia is found in infants and accounts for one third of the neonatal cholestatic JAUNDICE.'),
  'D001657': NtO(MH='Biliary Dyskinesia', MS='A motility disorder characterized by biliary COLIC, absence of GALLSTONES, and an abnormal GALLBLADDER ejection fraction. It is caused by gallbladder dyskinesia and/or SPHINCTER OF ODDI DYSFUNCTION.'),
  'D001658': NtO(MH='Biliary Fistula', MS='Abnormal passage in any organ of the biliary tract or between biliary organs and other organs.'),
  'D001659': NtO(MH='Biliary Tract', MS='The BILE DUCTS and the GALLBLADDER.'),
  'D001660': NtO(MH='Biliary Tract Diseases', MS='Diseases in any part of the BILIARY TRACT including the BILE DUCTS and the GALLBLADDER.'),
  'D001661': NtO(MH='Biliary Tract Neoplasms', MS='Tumors or cancer in the BILIARY TRACT including the BILE DUCTS and the GALLBLADDER.'),
  'D001662': NtO(MH='Biliary Tract Surgical Procedures', MS='Any surgical procedure performed on the biliary tract.'),
  'D001663': NtO(MH='Bilirubin', MS='A bile pigment that is a degradation product of HEME.'),
  'D001664': NtO(MH='Biliverdine', MS='1,3,6,7-Tetramethyl-4,5-dicarboxyethyl-2,8-divinylbilenone. Biosynthesized from hemoglobin as a precursor of bilirubin. Occurs in the bile of AMPHIBIANS and of birds, but not in normal human bile or serum.'),
  'D001665': NtO(MH='Binding Sites', MS='The parts of a macromolecule that directly participate in its specific combination with another molecule.'),
  'D001666': NtO(MH='Binding Sites, Antibody', MS='Local surface sites on antibodies which react with antigen determinant sites on antigens (EPITOPES.) They are formed from parts of the variable regions of FAB FRAGMENTS.'),
  'D001667': NtO(MH='Binding, Competitive', MS='The interaction of two or more substrates or ligands with the same binding site. The displacement of one by the other is used in quantitative and selective affinity measurements.'),
  'D001669': NtO(MH='Biochemical Phenomena', MS='The chemical processes, enzymatic activities, and pathways of living things and related temporal, dimensional, qualitative, and quantitative concepts.'),
  'D001671': NtO(MH='Biochemistry', MS='The study of the composition, chemical structures, and chemical reactions of living things.'),
  'D001672': NtO(MH='Biocompatible Materials', MS='Synthetic or natural materials, other than DRUGS, that are used to replace or repair any body TISSUES or bodily function.'),
  'D001673': NtO(MH='Biodegradation, Environmental', MS='Elimination of ENVIRONMENTAL POLLUTANTS; PESTICIDES and other waste using living organisms, usually involving intervention of environmental or sanitation engineers.'),
  'D001674': NtO(MH='Bioelectric Energy Sources', MS='Electric power supply devices which convert biological energy, such as chemical energy of metabolism or mechanical energy of periodic movements, into electrical energy.'),
  'D001675': NtO(MH='Bioethics', MS='A branch of applied ethics that studies the value implications of practices and developments in life sciences, medicine, and health care.'),
  'D001676': NtO(MH='Biofeedback, Psychology', MS="The therapy technique of providing the status of one's own AUTONOMIC NERVOUS SYSTEM function (e.g., skin temperature, heartbeats, brain waves) as visual or auditory feedback in order to self-control related conditions (e.g., hypertension, migraine headaches)."),
  'D001678': NtO(MH='Organelle Biogenesis', MS='The natural growth and development within living CELLS.'),
  'D001679': NtO(MH='Biogenic Amines', MS='A group of naturally occurring amines derived by enzymatic decarboxylation of the natural amino acids. Many have powerful physiological effects (e.g., histamine, serotonin, epinephrine, tyramine). Those derived from aromatic amino acids, and also their synthetic analogs (e.g., amphetamine), are of use in pharmacology.'),
  'D001680': NtO(MH='Biographies as Topic', MS="Works about a written account of a person's life and the branch of literature concerned with the lives of people."),
  'D001681': NtO(MH='Biological Assay', MS='A method of measuring the effects of a biologically active substance using an intermediate in vivo or in vitro tissue or cell model under controlled conditions. It includes virulence studies in animal fetuses in utero, mouse convulsion bioassay of insulin, quantitation of tumor-initiator systems in mouse skin, calculation of potentiating effects of a hormonal factor in an isolated strip of contracting stomach muscle, etc.'),
  'D001682': NtO(MH='Biological Availability', MS='The extent to which the active ingredient of a drug dosage form becomes available at the site of drug action or in a biological medium believed to reflect accessibility to a site of action.'),
  'D001683': NtO(MH='Biological Clocks', MS='The physiological mechanisms that govern the rhythmic occurrence of certain biochemical, physiological, and behavioral phenomena.'),
  'D001684': NtO(MH='Biological Dressings', MS='Human or animal tissue used as temporary wound coverings.'),
  'D001685': NtO(MH='Biological Factors', MS='Endogenously-synthesized compounds that influence biological processes not otherwise classified under ENZYMES; HORMONES or HORMONE ANTAGONISTS.'),
  'D001686': NtO(MH='Biological Phenomena', MS='Biological processes, properties, and characteristics of the whole organism in human, animal, microorganisms, and plants, and of the biosphere.'),
  'D001688': NtO(MH='Biological Products', MS='Complex pharmaceutical substances, preparations, or matter derived from organisms usually obtained by biological methods or assay.'),
  'D001689': NtO(MH='Biological Psychiatry', MS='An interdisciplinary science concerned with studies of the biological bases of behavior - biochemical, genetic, physiological, and neurological - and applying these to the understanding and treatment of mental illness.'),
  'D001690': NtO(MH='Biological Science Disciplines', MS='All of the divisions of the natural sciences dealing with the various aspects of the phenomena of life and vital processes. The concept includes anatomy and physiology, biochemistry and biophysics, and the biology of animals, plants, and microorganisms. It should be differentiated from BIOLOGY, one of its subdivisions, concerned specifically with the origin and life processes of living organisms.'),
  'D001691': NtO(MH='Biological Therapy', MS='Treatment of diseases with biological materials or biological response modifiers, such as the use of GENES; CELLS; TISSUES; organs; SERUM; VACCINES; and humoral agents.'),
  'D001692': NtO(MH='Biological Transport', MS='The movement of materials (including biochemical substances and drugs) through a biological system at the cellular level. The transport can be across cell membranes and epithelial layers. It also can occur within intracellular compartments and extracellular compartments.'),
  'D001693': NtO(MH='Biological Transport, Active', MS='The movement of materials across cell membranes and epithelial layers against an electrochemical gradient, requiring the expenditure of metabolic energy.'),
  'D001694': NtO(MH='Biological Warfare', MS='Warfare involving the use of living organisms or their products as disease etiologic agents against people, animals, or plants.'),
  'D001695': NtO(MH='Biology', MS='One of the BIOLOGICAL SCIENCE DISCIPLINES concerned with the origin, structure, development, growth, function, genetics, and reproduction of animals, plants, and microorganisms.'),
  'D001696': NtO(MH='Biomechanical Phenomena', MS='The properties, processes, and behavior of biological systems under the action of mechanical forces.'),
  'D001697': NtO(MH='Biomedical and Dental Materials', MS='Substances used in biomedicine or dentistry predominantly for their physical, as opposed to chemical, properties.'),
  'D001698': NtO(MH='Biomedical Engineering', MS='Application of principles and practices of engineering science to biomedical research and health care.'),
  'D001699': NtO(MH='Biometry', MS='The use of statistical and mathematical methods to analyze biological observations and phenomena.'),
  'D001700': NtO(MH='Biomphalaria', MS='A genus of planorbid freshwater snails, species of which are intermediate hosts of Schistosoma mansoni.'),
  'D001701': NtO(MH='Bionics', MS='The study of systems, particularly electronic systems, which function after the manner of, in a manner characteristic of, or resembling living systems. Also, the science of applying biological techniques and principles to the design of electronic systems.'),
  'D001702': NtO(MH='Biopharmaceutics', MS='The study of the physical and chemical properties of a drug and its dosage form as related to the onset, duration, and intensity of its action.'),
  'D001703': NtO(MH='Biophysics', MS='The study of PHYSICAL PHENOMENA and PHYSICAL PROCESSES as applied to living things.'),
  'D001704': NtO(MH='Biopolymers', MS='Polymers synthesized by living organisms. They play a role in the formation of macromolecular structures and are synthesized via the covalent linkage of biological molecules, especially AMINO ACIDS; NUCLEOTIDES; and CARBOHYDRATES.'),
  'D001705': NtO(MH='Bioprosthesis', MS='Prosthesis, usually heart valve, composed of biological material and whose durability depends upon the stability of the material after pretreatment, rather than regeneration by host cell ingrowth. Durability is achieved 1, mechanically by the interposition of a cloth, usually polytetrafluoroethylene, between the host and the graft, and 2, chemically by stabilization of the tissue by intermolecular linking, usually with glutaraldehyde, after removal of antigenic components, or the use of reconstituted and restructured biopolymers.'),
  'D001706': NtO(MH='Biopsy', MS='Removal and pathologic examination of specimens from the living body.'),
  'D001707': NtO(MH='Biopsy, Needle', MS='Removal and examination of tissue obtained through a transdermal needle inserted into the specific region, organ, or tissue being analyzed.'),
  'D001708': NtO(MH='Biopterin', MS='A natural product that has been considered as a growth factor for some insects.'),
  'D001709': NtO(MH='Biotechnology', MS='Body of knowledge related to the use of organisms, cells or cell-derived constituents for the purpose of developing products which are technically, scientifically and clinically useful. Alteration of biologic function at the molecular level (i.e., GENETIC ENGINEERING) is a central focus; laboratory methods used include TRANSFECTION and CLONING technologies, sequence and structure analysis algorithms, computer databases, and gene and protein structure function analysis and prediction.'),
  'D001710': NtO(MH='Biotin', MS='A water-soluble, enzyme co-factor present in minute amounts in every living cell. It occurs mainly bound to proteins or polypeptides and is abundant in liver, kidney, pancreas, yeast, and milk.'),
  'D001711': NtO(MH='Biotransformation', MS='The chemical alteration of an exogenous substance by or in a biological system. The alteration may inactivate the compound or it may result in the production of an active metabolite of an inactive parent compound. The alterations may be divided into METABOLIC DETOXICATION, PHASE I and METABOLIC DETOXICATION, PHASE II.'),
  'D001712': NtO(MH='Biperiden', MS='A muscarinic antagonist that has effects in both the central and peripheral nervous systems. It has been used in the treatment of arteriosclerotic, idiopathic, and postencephalitic parkinsonism. It has also been used to alleviate extrapyramidal symptoms induced by phenothiazine derivatives and reserpine.'),
  'D001713': NtO(MH='Biphenyl Compounds', MS='Whitish aromatic crystalline organic compounds made up of two conjoined BENZENE rings.'),
  'D001714': NtO(MH='Bipolar Disorder', MS='A major affective disorder marked by severe mood swings (manic or major depressive episodes) and a tendency to remission and recurrence.'),
  'D001715': NtO(MH='Bird Diseases', MS='Diseases of birds not considered poultry, therefore usually found in zoos, parks, and the wild. The concept is differentiated from POULTRY DISEASES which is for birds raised as a source of meat or eggs for human consumption, and usually found in barnyards, hatcheries, etc.'),
  'D001716': NtO(MH="Bird Fancier's Lung", MS='A form of alveolitis or pneumonitis due to an acquired hypersensitivity to inhaled avian antigens, usually proteins in the dust of bird feathers and droppings.'),
  'D001717': NtO(MH='Birds', MS='Warm-blooded VERTEBRATES possessing FEATHERS and belonging to the class Aves.'),
  'D001718': NtO(MH='Birefringence', MS='The property of nonisotropic media, such as crystals, whereby a single incident beam of light traverses the medium as two beams, each plane-polarized, the planes being at right angles to each other. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D001719': NtO(MH='Birth Certificates', MS="Official certifications by a physician recording the individual's birth date, place of birth, parentage and other required identifying data which are filed with the local registrar of vital statistics."),
  'D001720': NtO(MH='Birth Injuries', MS='Mechanical or anoxic trauma incurred by the infant during labor or delivery.'),
  'D001721': NtO(MH='Birth Intervals', MS='The lengths of intervals between births to women in the population.'),
  'D001722': NtO(MH='Birth Order', MS='The sequence in which children are born into the family.'),
  'D001723': NtO(MH='Birth Rate', MS='The number of births in a given population per year or other unit of time.'),
  'D001724': NtO(MH='Birth Weight', MS='The mass or quantity of heaviness of an individual at BIRTH. It is expressed by units of pounds or kilograms.'),
  'D001725': NtO(MH='Bis(Chloromethyl) Ether', MS='A substance that is an irritant to the eyes and respiratory tract and may be carcinogenic.'),
  'D001726': NtO(MH='Bisacodyl', MS='A diphenylmethane stimulant laxative used for the treatment of CONSTIPATION and for bowel evacuation. (From Martindale, The Extra Pharmacopoeia, 30th ed, p871)'),
  'D001727': NtO(MH='Bisexuality', MS='The sexual attraction or relationship between members of both the same and the opposite SEX.'),
  'D001728': NtO(MH='Dicumarol', MS='An oral anticoagulant that interferes with the metabolism of vitamin K. It is also used in biochemical experiments as an inhibitor of reductases.'),
  'D001729': NtO(MH='Bismuth', MS='A metallic element that has the atomic symbol Bi, and atomic number 83. Its principal isotope is Bismuth 209.'),
  'D001731': NtO(MH='Bisphosphoglycerate Mutase', MS='An enzyme that catalyzes the transfer of phosphate from C-3 of 1,3-diphosphoglycerate to C-2 of 3-phosphoglycerate, forming 2,3-diphosphoglycerate. EC 5.4.2.4.'),
  'D001732': NtO(MH='Bite Force', MS='The force applied by the masticatory muscles in dental occlusion.'),
  'D001733': NtO(MH='Bites and Stings', MS='Injuries inflicted by the TEETH or poisoning caused by VENOMS of animals.'),
  'D001734': NtO(MH='Bites, Human', MS='Bites inflicted by humans.'),
  'D001735': NtO(MH='Bithionol', MS='Halogenated anti-infective agent that is used against trematode and cestode infestations.'),
  'D001736': NtO(MH='Biureas', MS=None),
  'D001737': NtO(MH='Biuret', MS='Used as feed supplement for sheep and cattle since it is a good non-protein nitrogen source. In strongly alkaline solution biuret gives a violet color with copper sulfate.'),
  'D001738': NtO(MH='Biuret Reaction', MS='A reaction characterized by a violet color upon the addition of copper sulfate to all compounds with two amide or peptide bonds linked directly or through an intermediate carbon atom. Used in the detection and estimation of proteins and peptides having more than two amino acids.'),
  'D001739': NtO(MH='BK Virus', MS='A species of POLYOMAVIRUS apparently infecting over 90% of children but not clearly associated with any clinical illness in childhood. The virus remains latent in the body throughout life and can be reactivated under certain circumstances.'),
  'D001740': NtO(MH='Black Widow Spider', MS='A venomous New World spider with an hourglass-shaped red mark on the abdomen.'),
  'D001741': NtO(MH='African Americans', MS='Persons living in the United States having origins in any of the black groups of Africa.'),
  'D001742': NtO(MH='Blackwater Fever', MS='A complication of MALARIA, FALCIPARUM characterized by the passage of dark red to black urine.'),
  'D001743': NtO(MH='Urinary Bladder', MS='A musculomembranous sac along the URINARY TRACT. URINE flows from the KIDNEYS into the bladder via the ureters (URETER), and is held there until URINATION.'),
  'D001744': NtO(MH='Urinary Bladder Calculi', MS='Stones in the URINARY BLADDER; also known as vesical calculi, bladder stones, or cystoliths.'),
  'D001745': NtO(MH='Urinary Bladder Diseases', MS='Pathological processes of the URINARY BLADDER.'),
  'D001746': NtO(MH='Bladder Exstrophy', MS='A birth defect in which the URINARY BLADDER is malformed and exposed, inside out, and protruded through the ABDOMINAL WALL. It is caused by closure defects involving the top front surface of the bladder, as well as the lower abdominal wall; SKIN; MUSCLES; and the pubic bone.'),
  'D001747': NtO(MH='Urinary Bladder Fistula', MS='An abnormal passage in the URINARY BLADDER or between the bladder and any surrounding organ.'),
  'D001748': NtO(MH='Urinary Bladder Neck Obstruction', MS='Blocked urine flow through the bladder neck, the narrow internal urethral opening at the base of the URINARY BLADDER. Narrowing or strictures of the URETHRA can be congenital or acquired. It is often observed in males with enlarged PROSTATE glands.'),
  'D001749': NtO(MH='Urinary Bladder Neoplasms', MS='Tumors or cancer of the URINARY BLADDER.'),
  'D001750': NtO(MH='Urinary Bladder, Neurogenic', MS='Dysfunction of the URINARY BLADDER due to disease of the central or peripheral nervous system pathways involved in the control of URINATION. This is often associated with SPINAL CORD DISEASES, but may also be caused by BRAIN DISEASES or PERIPHERAL NERVE DISEASES.'),
  'D001751': NtO(MH='Blade Implantation', MS='Insertion of an endosseous implant with a narrow wedge-shaped infrastructure extending through the oral mucosa into the mouth and bearing openings or vents through which tissue grows to obtain retention.'),
  'D001752': NtO(MH='Blast Crisis', MS='An advanced phase of chronic myelogenous leukemia, characterized by a rapid increase in the proportion of immature white blood cells (blasts) in the blood and bone marrow to greater than 30%.'),
  'D001753': NtO(MH='Blast Injuries', MS='Injuries resulting when a person is struck by particles impelled with violent force from an explosion. Blast causes pulmonary concussion and hemorrhage, laceration of other thoracic and abdominal viscera, ruptured ear drums, and minor effects in the central nervous system. (From Dorland, 27th ed)'),
  'D001754': NtO(MH='Blastocladiella', MS='A genus of aquatic fungi of the family Blastocladiaceae, order Blastocladiales, used in the study of zoospore formation.'),
  'D001755': NtO(MH='Blastocyst', MS='A post-MORULA preimplantation mammalian embryo that develops from a 32-cell stage into a fluid-filled hollow ball of over a hundred cells. A blastocyst has two distinctive tissues. The outer layer of trophoblasts gives rise to extra-embryonic tissues. The inner cell mass gives rise to the embryonic disc and eventual embryo proper.'),
  'D001756': NtO(MH='Blastoderm', MS='A layer of cells lining the fluid-filled cavity (blastocele) of a BLASTULA, usually developed from a fertilized insect, reptilian, or avian egg.'),
  'D001757': NtO(MH='Blastomeres', MS='Undifferentiated cells resulting from cleavage of a fertilized egg (ZYGOTE). Inside the intact ZONA PELLUCIDA, each cleavage yields two blastomeres of about half size of the parent cell. Up to the 8-cell stage, all of the blastomeres are totipotent. The 16-cell MORULA contains outer cells and inner cells.'),
  'D001758': NtO(MH='Blastomyces', MS='A genus of onygenacetous mitosporic fungi whose perfect state is Ajellomyces (see ONYGENALES). The species Blastomyces dermatitidis (perfect state Ajellomyces dermatitidis) causes blastomycosis.'),
  'D001759': NtO(MH='Blastomycosis', MS='A fungal infection that may appear in two forms: 1, a primary lesion characterized by the formation of a small cutaneous nodule and small nodules along the lymphatics that may heal within several months; and 2, chronic granulomatous lesions characterized by thick crusts, warty growths, and unusual vascularity and infection in the middle or upper lobes of the lung.'),
  'D001760': NtO(MH='Bleeding Time', MS='Duration of blood flow after skin puncture. This test is used as a measure of capillary and platelet function.'),
  'D001761': NtO(MH='Bleomycin', MS='A complex of related glycopeptide antibiotics from Streptomyces verticillus consisting of bleomycin A2 and B2. It inhibits DNA metabolism and is used as an antineoplastic, especially for solid tumors.'),
  'D001762': NtO(MH='Blepharitis', MS='Inflammation of the eyelids.'),
  'D001763': NtO(MH='Blepharoptosis', MS='Drooping of the upper lid due to deficient development or paralysis of the levator palpebrae muscle.'),
  'D001764': NtO(MH='Blepharospasm', MS='Excessive winking; tonic or clonic spasm of the orbicularis oculi muscle.'),
  'D001765': NtO(MH='Blind Loop Syndrome', MS='A malabsorption syndrome that is associated with a blind loop in the upper SMALL INTESTINE that is characterized by the lack of peristaltic movement, stasis of INTESTINAL CONTENTS, and the overgrowth of BACTERIA. Such bacterial overgrowth interferes with BILE SALTS action, FATTY ACIDS processing, MICROVILLI integrity, and the ABSORPTION of nutrients such as VITAMIN B12 and FOLIC ACID.'),
  'D001766': NtO(MH='Blindness', MS='The inability to see or the loss or absence of perception of visual stimuli. This condition may be the result of EYE DISEASES; OPTIC NERVE DISEASES; OPTIC CHIASM diseases; or BRAIN DISEASES affecting the VISUAL PATHWAYS or OCCIPITAL LOBE.'),
  'D001767': NtO(MH='Blinking', MS='Brief closing of the eyelids by involuntary normal periodic closing, as a protective measure, or by voluntary action.'),
  'D001768': NtO(MH='Blister', MS='Visible accumulations of fluid within or beneath the epidermis.'),
  'D001769': NtO(MH='Blood', MS='The body fluid that circulates in the vascular system (BLOOD VESSELS). Whole blood includes PLASMA and BLOOD CELLS.'),
  'D001770': NtO(MH='Blood Bactericidal Activity', MS="The natural bactericidal property of BLOOD due to normally occurring antibacterial substances such as beta lysin, leukin, etc. This activity needs to be distinguished from the bactericidal activity contained in a patient's serum as a result of antimicrobial therapy, which is measured by a SERUM BACTERICIDAL TEST."),
  'D001771': NtO(MH='Blood Banks', MS='Centers for collecting, characterizing and storing blood or plasma.'),
  'D001772': NtO(MH='Blood Cell Count', MS='The number of LEUKOCYTES and ERYTHROCYTES per unit volume in a sample of venous BLOOD. A complete blood count (CBC) also includes measurement of the HEMOGLOBIN; HEMATOCRIT; and ERYTHROCYTE INDICES.'),
  'D001773': NtO(MH='Blood Cells', MS='The cells found in the body fluid circulating throughout the CARDIOVASCULAR SYSTEM.'),
  'D001774': NtO(MH='Blood Chemical Analysis', MS='An examination of chemicals in the blood.'),
  'D001775': NtO(MH='Blood Circulation', MS='The movement of the BLOOD as it is pumped through the CARDIOVASCULAR SYSTEM.'),
  'D001776': NtO(MH='Blood Circulation Time', MS='Determination of the shortest time interval between the injection of a substance in the vein and its arrival at some distant site in sufficient concentration to produce a recognizable end result. It represents approximately the inverse of the average velocity of blood flow between two points.'),
  'D001777': NtO(MH='Blood Coagulation', MS='The process of the interaction of BLOOD COAGULATION FACTORS that results in an insoluble FIBRIN clot.'),
  'D001778': NtO(MH='Blood Coagulation Disorders', MS='Hemorrhagic and thrombotic disorders that occur as a consequence of abnormalities in blood coagulation due to a variety of factors such as COAGULATION PROTEIN DISORDERS; BLOOD PLATELET DISORDERS; BLOOD PROTEIN DISORDERS or nutritional conditions.'),
  'D001779': NtO(MH='Blood Coagulation Factors', MS='Endogenous substances, usually proteins, that are involved in the blood coagulation process.'),
  'D001780': NtO(MH='Blood Coagulation Tests', MS="Laboratory tests for evaluating the individual's clotting mechanism."),
  'D001781': NtO(MH='Blood Component Removal', MS='Any procedure in which blood is withdrawn from a donor, a portion is separated and retained, at the same time the remainder is returned to the donor.'),
  'D001782': NtO(MH='Blood Donors', MS=None),
  'D001783': NtO(MH='Blood Flow Velocity', MS='A value equal to the total volume flow divided by the cross-sectional area of the vascular bed.'),
  'D001784': NtO(MH='Blood Gas Analysis', MS='Measurement of oxygen and carbon dioxide in the blood.'),
  'D001785': NtO(MH='Blood Gas Monitoring, Transcutaneous', MS='The noninvasive measurement or determination of the partial pressure (tension) of oxygen and/or carbon dioxide locally in the capillaries of a tissue by the application to the skin of a special set of electrodes. These electrodes contain photoelectric sensors capable of picking up the specific wavelengths of radiation emitted by oxygenated versus reduced hemoglobin.'),
  'D001786': NtO(MH='Blood Glucose', MS='Glucose in blood.'),
  'D001787': NtO(MH='Blood Group Incompatibility', MS="An antigenic mismatch between donor and recipient blood. Antibodies present in the recipient's serum may be directed against antigens in the donor product. Such a mismatch may result in a transfusion reaction in which, for example, donor blood is hemolyzed. (From Saunders Dictionary & Encyclopedia of Laboratory Medicine and Technology, 1984)."),
  'D001788': NtO(MH='Blood Grouping and Crossmatching', MS='Testing erythrocytes to determine presence or absence of blood-group antigens, testing of serum to determine the presence or absence of antibodies to these antigens, and selecting biocompatible blood by crossmatching samples from the donor against samples from the recipient. Crossmatching is performed prior to transfusion.'),
  'D001789': NtO(MH='Blood Group Antigens', MS='Sets of cell surface antigens located on BLOOD CELLS. They are usually membrane GLYCOPROTEINS or GLYCOLIPIDS that are antigenically distinguished by their carbohydrate moieties.'),
  'D001790': NtO(MH='Blood Physiological Phenomena', MS='Physiological processes and properties of the BLOOD.'),
  'D001791': NtO(MH='Blood Platelet Disorders', MS='Disorders caused by abnormalities in platelet count or function.'),
  'D001792': NtO(MH='Blood Platelets', MS='Non-nucleated disk-shaped cells formed in the megakaryocyte and found in the blood of all mammals. They are mainly involved in blood coagulation.'),
  'D001793': NtO(MH='Blood Preservation', MS='The process by which blood or its components are kept viable outside of the organism from which they are derived (i.e., kept from decay by means of a chemical agent, cooling, or a fluid substitute that mimics the natural state within the organism).'),
  'D001794': NtO(MH='Blood Pressure', MS='PRESSURE of the BLOOD on the ARTERIES and other BLOOD VESSELS.'),
  'D001795': NtO(MH='Blood Pressure Determination', MS='Techniques used for measuring BLOOD PRESSURE.'),
  'D001796': NtO(MH='Blood Protein Disorders', MS='Hematologic diseases caused by structural or functional defects of BLOOD PROTEINS.'),
  'D001797': NtO(MH='Blood Protein Electrophoresis', MS='Electrophoresis applied to BLOOD PROTEINS.'),
  'D001798': NtO(MH='Blood Proteins', MS='Proteins that are present in blood serum, including SERUM ALBUMIN; BLOOD COAGULATION FACTORS; and many other types of proteins.'),
  'D001799': NtO(MH='Blood Sedimentation', MS='Measurement of rate of settling of erythrocytes in anticoagulated blood.'),
  'D001800': NtO(MH='Blood Specimen Collection', MS='The taking of a blood sample to determine its character as a whole, to identify levels of its component cells, chemicals, gases, or other constituents, to perform pathological examination, etc.'),
  'D001801': NtO(MH='Blood Stains', MS='Antigenic characteristics and DNA fingerprint patterns identified from blood stains. Their primary value is in criminal cases.'),
  'D001802': NtO(MH='Blood Substitutes', MS='Substances that are used in place of blood, for example, as an alternative to BLOOD TRANSFUSIONS after blood loss to restore BLOOD VOLUME and oxygen-carrying capacity to the blood circulation, or to perfuse isolated organs.'),
  'D001803': NtO(MH='Blood Transfusion', MS='The introduction of whole blood or blood component directly into the blood stream. (Dorland, 27th ed)'),
  'D001804': NtO(MH='Blood Transfusion, Autologous', MS="Reinfusion of blood or blood products derived from the patient's own circulation. (Dorland, 27th ed)"),
  'D001805': NtO(MH='Blood Transfusion, Intrauterine', MS='In utero transfusion of BLOOD into the FETUS for the treatment of FETAL DISEASES, such as fetal erythroblastosis (ERYTHROBLASTOSIS, FETAL).'),
  'D001806': NtO(MH='Blood Urea Nitrogen', MS='The urea concentration of the blood stated in terms of nitrogen content. Serum (plasma) urea nitrogen is approximately 12% higher than blood urea nitrogen concentration because of the greater protein content of red blood cells. Increases in blood or serum urea nitrogen are referred to as azotemia and may have prerenal, renal, or postrenal causes. (From Saunders Dictionary & Encyclopedia of Laboratory Medicine and Technology, 1984)'),
  'D001807': NtO(MH='Blood Vessel Prosthesis', MS='Device constructed of either synthetic or biological material that is used for the repair of injured or diseased blood vessels.'),
  'D001808': NtO(MH='Blood Vessels', MS='Any of the tubular vessels conveying the blood (arteries, arterioles, capillaries, venules, and veins).'),
  'D001809': NtO(MH='Blood Viscosity', MS='The internal resistance of the BLOOD to shear forces. The in vitro measure of whole blood viscosity is of limited clinical utility because it bears little relationship to the actual viscosity within the circulation, but an increase in the viscosity of circulating blood can contribute to morbidity in patients suffering from disorders such as SICKLE CELL ANEMIA and POLYCYTHEMIA.'),
  'D001810': NtO(MH='Blood Volume', MS='Volume of circulating BLOOD. It is the sum of the PLASMA VOLUME and ERYTHROCYTE VOLUME.'),
  'D001811': NtO(MH='Blood Volume Determination', MS='Method for determining the circulating blood volume by introducing a known quantity of foreign substance into the blood and determining its concentration some minutes later when thorough mixing has occurred. From these two values the blood volume can be calculated by dividing the quantity of injected material by its concentration in the blood at the time of uniform mixing. Generally expressed as cubic centimeters or liters per kilogram of body weight.'),
  'D001812': NtO(MH='Blood-Brain Barrier', MS='Specialized non-fenestrated tightly-joined ENDOTHELIAL CELLS with TIGHT JUNCTIONS that form a transport barrier for certain substances between the cerebral capillaries and the BRAIN tissue.'),
  'D001813': NtO(MH='Blood-Retinal Barrier', MS='A specialized transport barrier, in the EYE, formed by the retinal pigment EPITHELIUM, and the ENDOTHELIUM of the BLOOD VESSELS of the RETINA. TIGHT JUNCTIONS joining adjacent cells keep the barrier between cells continuous.'),
  'D001814': NtO(MH='Blood-Testis Barrier', MS='A specialized barrier, in the TESTIS, between the interstitial BLOOD compartment and the adluminal compartment of the SEMINIFEROUS TUBULES. The barrier is formed by layers of cells from the VASCULAR ENDOTHELIUM of the capillary BLOOD VESSELS, to the SEMINIFEROUS EPITHELIUM of the seminiferous tubules. TIGHT JUNCTIONS form between adjacent SERTOLI CELLS, as well as between the ENDOTHELIAL CELLS.'),
  'D001815': NtO(MH='Bloodletting', MS='Puncture of a vein to draw blood for therapeutic purposes. Bloodletting therapy has been used in Talmudic and Indian medicine since the medieval time, and was still practiced widely in the 18th and 19th centuries. Its modern counterpart is PHLEBOTOMY.'),
  'D001816': NtO(MH='Bloom Syndrome', MS='An autosomal recessive disorder characterized by telangiectatic ERYTHEMA of the face, photosensitivity, DWARFISM and other abnormalities, and a predisposition toward developing cancer. The Bloom syndrome gene (BLM) encodes a RecQ-like DNA helicase.'),
  'D001819': NtO(MH='Bluetongue', MS='A reovirus infection, chiefly of sheep, characterized by a swollen blue tongue, catarrhal inflammation of upper respiratory and gastrointestinal tracts, and often by inflammation of sensitive laminae of the feet and coronet.'),
  'D001820': NtO(MH='Bluetongue virus', MS='The type species of ORBIVIRUS causing a serious disease in sheep, especially lambs. It may also infect wild ruminants and other domestic animals.'),
  'D001821': NtO(MH='Blushing', MS='Involuntary reddening, especially of the face, associated with feelings of embarrassment, confusion or shame.'),
  'D001822': NtO(MH='Body Burden', MS='The total amount of a chemical, metal or radioactive substance present at any time after absorption in the body of man or animal.'),
  'D001823': NtO(MH='Body Composition', MS='The relative amounts of various components in the body, such as percentage of body fat.'),
  'D001824': NtO(MH='Body Constitution', MS='The physical characteristics of the body, including the mode of performance of functions, the activity of metabolic processes, the manner and degree of reactions to stimuli, and power of resistance to the attack of pathogenic organisms.'),
  'D001825': NtO(MH='Body Fluid Compartments', MS='The two types of spaces between which water and other body fluids are distributed: extracellular and intracellular.'),
  'D001826': NtO(MH='Body Fluids', MS='Liquid components of living organisms.'),
  'D001827': NtO(MH='Body Height', MS='The distance from the sole to the crown of the head with body standing on a flat surface and fully extended.'),
  'D001828': NtO(MH='Body Image', MS="Individuals' concept of their own bodies."),
  'D001829': NtO(MH='Body Regions', MS='Anatomical areas of the body.'),
  'D001830': NtO(MH='Body Surface Area', MS='The two dimensional measure of the outer layer of the body.'),
  'D001831': NtO(MH='Body Temperature', MS='The measure of the level of heat of a human or animal.'),
  'D001832': NtO(MH='Body Temperature Changes', MS='Significant alterations in temperature of the human body, above or below 98.6 degrees F. or 37 degrees C. when taken orally.'),
  'D001833': NtO(MH='Body Temperature Regulation', MS='The processes of heating and cooling that an organism uses to control its temperature.'),
  'D001834': NtO(MH='Body Water', MS='Fluids composed mainly of water found within the body.'),
  'D001835': NtO(MH='Body Weight', MS='The mass or quantity of heaviness of an individual. It is expressed by units of pounds or kilograms.'),
  'D001836': NtO(MH='Body Weight Changes', MS="A clinical manifestation consisting of alterations in an individual's weight from his or her norm."),
  'D001837': NtO(MH='Body Weights and Measures', MS='Measurements of the height, weight, length, area, etc., of the human and animal body or its parts.'),
  'D001838': NtO(MH='Bolivia', MS='A country in central South America, located southwest of Brazil.'),
  'D001839': NtO(MH='Bombesin', MS='A tetradecapeptide originally obtained from the skins of toads Bombina bombina and B. variegata. It is also an endogenous neurotransmitter in many animals including mammals. Bombesin affects vascular and other smooth muscle, gastric secretion, and renal circulation and function.'),
  'D001840': NtO(MH='Dental Bonding', MS='An adhesion procedure for orthodontic attachments, such as plastic DENTAL CROWNS. This process usually includes the application of an adhesive material (DENTAL CEMENTS) and letting it harden in-place by light or chemical curing.'),
  'D001841': NtO(MH='Bonding, Human-Pet', MS='The emotional attachment of individuals to PETS.'),
  'D001842': NtO(MH='Bone and Bones', MS='A specialized CONNECTIVE TISSUE that is the main constituent of the SKELETON. The principal cellular component of bone is comprised of OSTEOBLASTS; OSTEOCYTES; and OSTEOCLASTS, while FIBRILLAR COLLAGENS and hydroxyapatite crystals form the BONE MATRIX.'),
  'D001843': NtO(MH='Bone Cements', MS='Adhesives used to fix prosthetic devices to bones and to cement bone to bone in difficult fractures. Synthetic resins are commonly used as cements. A mixture of monocalcium phosphate, monohydrate, alpha-tricalcium phosphate, and calcium carbonate with a sodium phosphate solution is also a useful bone paste.'),
  'D001844': NtO(MH='Bone Conduction', MS='Transmission of sound waves through vibration of bones in the SKULL to the inner ear (COCHLEA). By using bone conduction stimulation and by bypassing any OUTER EAR or MIDDLE EAR abnormalities, hearing thresholds of the cochlea can be determined. Bone conduction hearing differs from normal hearing which is based on air conduction stimulation via the EAR CANAL and the TYMPANIC MEMBRANE.'),
  'D001845': NtO(MH='Bone Cysts', MS='Benign unilocular lytic areas in the proximal end of a long bone with well defined and narrow endosteal margins. The cysts contain fluid and the cyst walls may contain some giant cells. Bone cysts usually occur in males between the ages 3-15 years.'),
  'D001846': NtO(MH='Bone Development', MS='The growth and development of bones from fetus to adult. It includes two principal mechanisms of bone growth: growth in length of long bones at the epiphyseal cartilages and growth in thickness by depositing new bone (OSTEOGENESIS) with the actions of OSTEOBLASTS and OSTEOCLASTS.'),
  'D001847': NtO(MH='Bone Diseases', MS='Diseases of BONES.'),
  'D001848': NtO(MH='Bone Diseases, Developmental', MS='Diseases resulting in abnormal GROWTH or abnormal MORPHOGENESIS of BONES.'),
  'D001849': NtO(MH='Bone Diseases, Endocrine', MS='Diseases of the bones related to hyperfunction or hypofunction of the endocrine glands.'),
  'D001850': NtO(MH='Bone Diseases, Infectious', MS='Bone diseases caused by pathogenic microorganisms.'),
  'D001851': NtO(MH='Bone Diseases, Metabolic', MS='Diseases that affect the METABOLIC PROCESSES of BONE TISSUE.'),
  'D001852': NtO(MH='Bone Lengthening', MS='Increase in the longest dimension of a bone to correct anatomical deficiencies, congenital, traumatic, or as a result of disease. The lengthening is not restricted to long bones. The usual surgical methods are internal fixation and distraction.'),
  'D001853': NtO(MH='Bone Marrow', MS='The soft tissue filling the cavities of bones. Bone marrow exists in two types, yellow and red. Yellow marrow is found in the large cavities of large bones and consists mostly of fat cells and a few primitive blood cells. Red marrow is a hematopoietic tissue and is the site of production of erythrocytes and granular leukocytes. Bone marrow is made up of a framework of connective tissue containing branching fibers with the frame being filled with marrow cells.'),
  'D001854': NtO(MH='Bone Marrow Cells', MS='Cells contained in the bone marrow including fat cells (see ADIPOCYTES); STROMAL CELLS; MEGAKARYOCYTES; and the immediate precursors of most blood cells.'),
  'D001855': NtO(MH='Bone Marrow Diseases', MS='Diseases involving the BONE MARROW.'),
  'D001856': NtO(MH='Bone Marrow Examination', MS='Removal of bone marrow and evaluation of its histologic picture.'),
  'D001857': NtO(MH='Bone Matrix', MS='Extracellular substance of bone tissue consisting of COLLAGEN fibers, ground substance, and inorganic crystalline minerals and salts.'),
  'D001858': NtO(MH='Bone Nails', MS='Rods of bone, metal, or other material used for fixation of the fragments or ends of fractured bones.'),
  'D001859': NtO(MH='Bone Neoplasms', MS='Tumors or cancer located in bone tissue or specific BONES.'),
  'D001860': NtO(MH='Bone Plates', MS='Implantable fracture fixation devices attached to bone fragments with screws to bridge the fracture gap and shield the fracture site from stress as bone heals. (UMDNS, 1999)'),
  'D001861': NtO(MH='Bone Regeneration', MS='Renewal or repair of lost bone tissue. It excludes BONY CALLUS formed after BONE FRACTURES but not yet replaced by hard bone.'),
  'D001862': NtO(MH='Bone Resorption', MS='Bone loss due to osteoclastic activity.'),
  'D001863': NtO(MH='Bone Screws', MS='Specialized devices used in ORTHOPEDIC SURGERY to repair bone fractures.'),
  'D001864': NtO(MH='Bone Wires', MS='Steel wires, often threaded through the skin, soft tissues, and bone, used to fix broken bones. Kirschner wires or apparatus also includes the application of traction to the healing bones through the wires.'),
  'D001865': NtO(MH='Bongkrekic Acid', MS='An antibiotic produced by Pseudomonas cocovenenans. It is an inhibitor of MITOCHONDRIAL ADP, ATP TRANSLOCASES. Specifically, it blocks adenine nucleotide efflux from mitochondria by enhancing membrane binding.'),
  'D001867': NtO(MH='Book Classification', MS="A general term covering bibliographical and bibliothecal classifications. It mostly refers to library CLASSIFICATION for arrangement of books and documents on the shelves. (Harrod's Librarians' Glossary, 7th ed, p85)"),
  'D001868': NtO(MH='Book Collecting', MS='Seeking, locating, acquiring, organizing, cataloging, displaying, storing, and maintaining whatever books are of interest to a given individual collector.'),
  'D001869': NtO(MH='Book Imprints', MS='Information relating to the publication, printing, distribution, issue, release, or production of a book.'),
  'D001870': NtO(MH='Book Industry', MS='The aggregate enterprise of manufacturing and technically producing books. (From Random House Unabridged Dictionary, 2d ed)'),
  'D001871': NtO(MH='Book Ornamentation', MS=None),
  'D001872': NtO(MH='Book Prices', MS='Amount paid to purchase individual volumes or set from a bookseller.'),
  'D001873': NtO(MH='Book Reviews as Topic', MS='Works about critical analyses of books or other monographic works.'),
  'D001874': NtO(MH='Book Selection', MS=None),
  'D001875': NtO(MH='Bookbinding', MS='Joining together of a number of leaves or folios (most frequently of paper, parchment, or vellum) within covers to form a codex or book.'),
  'D001876': NtO(MH='Bookplates as Topic', MS='Works about labels pasted in books to mark their ownership and sometimes to indicate their location in a library.'),
  'D001877': NtO(MH='Books', MS='Non-periodical written or printed works consisting of sheets of pages fastened or bound together within covers.'),
  'D001878': NtO(MH='Books, Illustrated', MS='Books containing photographs, prints, drawings, portraits, plates, diagrams, facsimiles, maps, tables, or other representations or systematic arrangement of data designed to elucidate or decorate its contents. (From The ALA Glossary of Library and Information Science, 1983, p114)'),
  'D001879': NtO(MH='Bookselling', MS='Making books available for purchase.'),
  'D001880': NtO(MH='Boranes', MS="The collective name for the boron hydrides, which are analogous to the alkanes and silanes. Numerous boranes are known. Some have high calorific values and are used in high-energy fuels. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D001881': NtO(MH='Borates', MS='Inorganic or organic salts and esters of boric acid.'),
  'D001882': NtO(MH='Border Disease', MS='Congenital disorder of lambs caused by a virus closely related to or identical with certain strains of bovine viral diarrhea virus.'),
  'D001883': NtO(MH='Borderline Personality Disorder', MS='A personality disorder marked by a pattern of instability of interpersonal relationships, self-image, and affects, and marked impulsivity beginning by early adulthood and present in a variety of contexts. (DSM-IV)'),
  'D001884': NtO(MH='Bordetella', MS='A genus of gram-negative aerobic bacteria whose cells are minute coccobacilli. It consists of both parasitic and pathogenic species.'),
  'D001885': NtO(MH='Bordetella Infections', MS='Infections with bacteria of the genus BORDETELLA.'),
  'D001886': NtO(MH='Bordetella pertussis', MS='A species of gram-negative, aerobic bacteria that is the causative agent of WHOOPING COUGH. Its cells are minute coccobacilli that are surrounded by a slime sheath.'),
  'D001887': NtO(MH='Boredom', MS='A psychological state resulting from any activity that lacks motivation, or from enforced continuance in an uninteresting situation.'),
  'D001888': NtO(MH='Boric Acids', MS='Inorganic and organic derivatives of boric acid either B(OH)3 or, preferably H3BO3.'),
  'D001889': NtO(MH='Borinic Acids', MS='Inorganic or organic compounds that contain the general structure R2B(OH).'),
  'D001890': NtO(MH='Borna Disease', MS='An encephalomyelitis of horses, sheep and cattle caused by BORNA DISEASE VIRUS.'),
  'D001891': NtO(MH='Borna disease virus', MS='A species in the genus Bornavirus, family BORNAVIRIDAE, causing a rare and usually fatal encephalitic disease in horses and other domestic animals and possibly deer. Its name derives from the city in Saxony where the condition was first described in 1894, but the disease occurs in Europe, N. Africa, and the Near East.'),
  'D001892': NtO(MH='Camphanes', MS='Crystalline saturated MONOTERPENES, precursors of borneols and CAMPHENE.'),
  'D001893': NtO(MH='Borneo', MS="An island in the Malay Archipelago, east of Sumatra, north of Java, and west of Celebes. It is the third largest island in the world. Its name is a Portuguese alteration of BRUNEI, located on it. (From Webster's New Geographical Dictionary, 1988, p163; Room, Brewer's Dictionary of Names, 1992, p73)"),
  'D001894': NtO(MH='Borohydrides', MS='A class of inorganic or organic compounds that contain the borohydride (BH4-) anion.'),
  'D001895': NtO(MH='Boron', MS='A trace element with the atomic symbol B, atomic number 5, and atomic weight [10.806; 10.821]. Boron-10, an isotope of boron, is used as a neutron absorber in BORON NEUTRON CAPTURE THERAPY.'),
  'D001896': NtO(MH='Boron Compounds', MS='Inorganic or organic compounds that contain boron as an integral part of the molecule.'),
  'D001897': NtO(MH='Boronic Acids', MS='Inorganic or organic compounds that contain the basic structure RB(OH)2.'),
  'D001898': NtO(MH='Borrelia', MS='A genus of gram-negative, anaerobic, helical bacteria, various species of which produce RELAPSING FEVER in humans and other animals.'),
  'D001899': NtO(MH='Borrelia Infections', MS='Infections with bacteria of the genus BORRELIA.'),
  'D001900': NtO(MH='Boston', MS='City located in Massachusetts.'),
  'D001901': NtO(MH='Botany', MS='The study of the origin, structure, development, growth, function, genetics, and reproduction of plants.'),
  'D001902': NtO(MH='Botswana', MS='A republic in southern Africa, between NAMIBIA and ZAMBIA. It was formerly called Bechuanaland. Its capital is Gaborone. The Kalahari Desert is in the west and southwest.'),
  'D001903': NtO(MH='Bottle Feeding', MS='Use of nursing bottles for feeding. Applies to humans and animals.'),
  'D001904': NtO(MH='Botulinum Antitoxin', MS='Antiserum given therapeutically in BOTULISM.'),
  'D001905': NtO(MH='Botulinum Toxins', MS='Toxic proteins produced from the species CLOSTRIDIUM BOTULINUM. The toxins are synthesized as a single peptide chain which is processed into a mature protein consisting of a heavy chain and light chain joined via a disulfide bond. The botulinum toxin light chain is a zinc-dependent protease which is released from the heavy chain upon ENDOCYTOSIS into PRESYNAPTIC NERVE ENDINGS. Once inside the cell the botulinum toxin light chain cleaves specific SNARE proteins which are essential for secretion of ACETYLCHOLINE by SYNAPTIC VESICLES. This inhibition of acetylcholine release results in muscular PARALYSIS.'),
  'D001906': NtO(MH='Botulism', MS='A disease caused by potent protein NEUROTOXINS produced by CLOSTRIDIUM BOTULINUM which interfere with the presynaptic release of ACETYLCHOLINE at the NEUROMUSCULAR JUNCTION. Clinical features include abdominal pain, vomiting, acute PARALYSIS (including respiratory paralysis), blurred vision, and DIPLOPIA. Botulism may be classified into several subtypes (e.g., food-borne, infant, wound, and others). (From Adams et al., Principles of Neurology, 6th ed, p1208)'),
  'D001907': NtO(MH='Boutonneuse Fever', MS='A febrile disease of the Mediterranean area, the Crimea, Africa, and India, caused by infection with RICKETTSIA CONORII.'),
  'D001908': NtO(MH='Diarrhea Viruses, Bovine Viral', MS='A group of viruses in the genus PESTIVIRUS, causing diarrhea, fever, oral ulcerations, hemorrhagic syndrome, and various necrotic lesions among cattle and other domestic animals. The two species (genotypes), BVDV-1 and BVDV-2 , exhibit antigenic and pathological differences. The historical designation, BVDV, consisted of both (then unrecognized) genotypes.'),
  'D001909': NtO(MH='Leukemia Virus, Bovine', MS='The type species of DELTARETROVIRUS that causes a form of bovine lymphosarcoma (ENZOOTIC BOVINE LEUKOSIS) or persistent lymphocytosis.'),
  'D001911': NtO(MH='Herpesvirus 2, Bovine', MS='A species of SIMPLEXVIRUS causing mammillitis in cattle in Great Britain and South Africa.'),
  'D001912': NtO(MH='Bovine Virus Diarrhea-Mucosal Disease', MS='Acute disease of cattle caused by the bovine viral diarrhea viruses (DIARRHEA VIRUSES, BOVINE VIRAL). Often mouth ulcerations are the only sign but fever, diarrhea, drop in milk yield, and loss of appetite are also seen. Severity of clinical disease varies and is strain dependent. Outbreaks are characterized by low morbidity and high mortality.'),
  'D001913': NtO(MH="Bowen's Disease", MS='A persistent progressive non-elevated red scaly or crusted plaque which is due to an intradermal carcinoma and is potentially malignant. Atypical squamous cells proliferate through the whole thickness of the epidermis. The lesions may occur anywhere on the skin surface or on mucosal surfaces. The cause most frequently found is trivalent arsenic compounds. Freezing, cauterization or diathermy coagulation is often effective. (From Rook et al., Textbook of Dermatology, 4th ed, pp2428-9)'),
  'D001914': NtO(MH='Boxing', MS='A two-person sport in which the fists are skillfully used to attack and defend.'),
  'D001915': NtO(MH='Braces', MS='Orthopedic appliances used to support, align, or hold parts of the body in correct position. (Dorland, 28th ed)'),
  'D001916': NtO(MH='Brachial Artery', MS='The continuation of the axillary artery; it branches into the radial and ulnar arteries.'),
  'D001917': NtO(MH='Brachial Plexus', MS='The large network of nerve fibers which distributes the innervation of the upper extremity. The brachial plexus extends from the neck into the axilla. In humans, the nerves of the plexus usually originate from the lower cervical and the first thoracic spinal cord segments (C5-C8 and T1), but variations are not uncommon.'),
  'D001918': NtO(MH='Brachytherapy', MS='A collective term for interstitial, intracavity, and surface radiotherapy. It uses small sealed or partly-sealed sources that may be placed on or near the body surface or within a natural body cavity or implanted directly into the tissues.'),
  'D001919': NtO(MH='Bradycardia', MS='Cardiac arrhythmias that are characterized by excessively slow HEART RATE, usually below 50 beats per minute in human adults. They can be classified broadly into SINOATRIAL NODE dysfunction and ATRIOVENTRICULAR BLOCK.'),
  'D001920': NtO(MH='Bradykinin', MS='A nonapeptide messenger that is enzymatically produced from KALLIDIN in the blood where it is a potent but short-lived agent of arteriolar dilation and increased capillary permeability. Bradykinin is also released from MAST CELLS during asthma attacks, from gut walls as a gastrointestinal vasodilator, from damaged tissues as a pain signal, and may be a neurotransmitter.'),
  'D001921': NtO(MH='Brain', MS='The part of CENTRAL NERVOUS SYSTEM that is contained within the skull (CRANIUM). Arising from the NEURAL TUBE, the embryonic brain is comprised of three major parts including PROSENCEPHALON (the forebrain); MESENCEPHALON (the midbrain); and RHOMBENCEPHALON (the hindbrain). The developed brain consists of CEREBRUM; CEREBELLUM; and other structures in the BRAIN STEM.'),
  'D001922': NtO(MH='Brain Abscess', MS='A circumscribed collection of purulent exudate in the brain, due to bacterial and other infections. The majority are caused by spread of infected material from a focus of suppuration elsewhere in the body, notably the PARANASAL SINUSES, middle ear (see EAR, MIDDLE); HEART (see also ENDOCARDITIS, BACTERIAL), and LUNG. Penetrating CRANIOCEREBRAL TRAUMA and NEUROSURGICAL PROCEDURES may also be associated with this condition. Clinical manifestations include HEADACHE; SEIZURES; focal neurologic deficits; and alterations of consciousness. (Adams et al., Principles of Neurology, 6th ed, pp712-6)'),
  'D001923': NtO(MH='Brain Chemistry', MS='Changes in the amounts of various chemicals (neurotransmitters, receptors, enzymes, and other metabolites) specific to the area of the central nervous system contained within the head. These are monitored over time, during sensory stimulation, or under different disease states.'),
  'D001924': NtO(MH='Brain Concussion', MS="A nonspecific term used to describe transient alterations or loss of consciousness following closed head injuries. The duration of UNCONSCIOUSNESS generally lasts a few seconds, but may persist for several hours. Concussions may be classified as mild, intermediate, and severe. Prolonged periods of unconsciousness (often defined as greater than 6 hours in duration) may be referred to as post-traumatic coma (COMA, POST-HEAD INJURY). (From Rowland, Merritt's Textbook of Neurology, 9th ed, p418)"),
  'D001925': NtO(MH='Brain Damage, Chronic', MS='A condition characterized by long-standing brain dysfunction or damage, usually of three months duration or longer. Potential etiologies include BRAIN INFARCTION; certain NEURODEGENERATIVE DISORDERS; CRANIOCEREBRAL TRAUMA; ANOXIA, BRAIN; ENCEPHALITIS; certain NEUROTOXICITY SYNDROMES; metabolic disorders (see BRAIN DISEASES, METABOLIC); and other conditions.'),
  'D001926': NtO(MH='Brain Death', MS='A state of prolonged irreversible cessation of all brain activity, including lower brain stem function with the complete absence of voluntary movements, responses to stimuli, brain stem reflexes, and spontaneous respirations. Reversible conditions which mimic this clinical state (e.g., sedative overdose, hypothermia, etc.) are excluded prior to making the determination of brain death. (From Adams et al., Principles of Neurology, 6th ed, pp348-9)'),
  'D001927': NtO(MH='Brain Diseases', MS='Pathologic conditions affecting the BRAIN, which is composed of the intracranial components of the CENTRAL NERVOUS SYSTEM. This includes (but is not limited to) the CEREBRAL CORTEX; intracranial white matter; BASAL GANGLIA; THALAMUS; HYPOTHALAMUS; BRAIN STEM; and CEREBELLUM.'),
  'D001928': NtO(MH='Brain Diseases, Metabolic', MS='Acquired or inborn metabolic diseases that produce brain dysfunction or damage. These include primary (i.e., disorders intrinsic to the brain) and secondary (i.e., extracranial) metabolic conditions that adversely affect cerebral function.'),
  'D001929': NtO(MH='Brain Edema', MS='Increased intracellular or extracellular fluid in brain tissue. Cytotoxic brain edema (swelling due to increased intracellular fluid) is indicative of a disturbance in cell metabolism, and is commonly associated with hypoxic or ischemic injuries (see HYPOXIA, BRAIN). An increase in extracellular fluid may be caused by increased brain capillary permeability (vasogenic edema), an osmotic gradient, local blockages in interstitial fluid pathways, or by obstruction of CSF flow (e.g., obstructive HYDROCEPHALUS). (From Childs Nerv Syst 1992 Sep; 8(6):301-6)'),
  'D001930': NtO(MH='Brain Injuries', MS='Acute and chronic (see also BRAIN INJURIES, CHRONIC) injuries to the brain, including the cerebral hemispheres, CEREBELLUM, and BRAIN STEM. Clinical manifestations depend on the nature of injury. Diffuse trauma to the brain is frequently associated with DIFFUSE AXONAL INJURY or COMA, POST-TRAUMATIC. Localized injuries may be associated with NEUROBEHAVIORAL MANIFESTATIONS; HEMIPARESIS, or other focal neurologic deficits.'),
  'D001931': NtO(MH='Brain Mapping', MS='Imaging techniques used to colocalize sites of brain functions or physiological activity with brain structures.'),
  'D001932': NtO(MH='Brain Neoplasms', MS='Neoplasms of the intracranial components of the central nervous system, including the cerebral hemispheres, basal ganglia, hypothalamus, thalamus, brain stem, and cerebellum. Brain neoplasms are subdivided into primary (originating from brain tissue) and secondary (i.e., metastatic) forms. Primary neoplasms are subdivided into benign and malignant forms. In general, brain tumors may also be classified by age of onset, histologic type, or presenting location in the brain.'),
  'D001933': NtO(MH='Brain Stem', MS='The part of the brain that connects the CEREBRAL HEMISPHERES with the SPINAL CORD. It consists of the MESENCEPHALON; PONS; and MEDULLA OBLONGATA.'),
  'D001934': NtO(MH='Branchial Region', MS='A region, of SOMITE development period, that contains a number of paired arches, each with a mesodermal core lined by ectoderm and endoderm on the two sides. In lower aquatic vertebrates, branchial arches develop into GILLS. In higher vertebrates, the arches forms outpouchings and develop into structures of the head and neck. Separating the arches are the branchial clefts or grooves.'),
  'D001935': NtO(MH='Branchioma', MS='A tumor derived from branchial epithelium or branchial rests. (Dorland, 27th ed)'),
  'D001936': NtO(MH='Moraxella catarrhalis', MS='Gram-negative aerobic cocci of low virulence that colonize the nasopharynx and occasionally cause MENINGITIS; BACTEREMIA; EMPYEMA; PERICARDITIS; and PNEUMONIA.'),
  'D001937': NtO(MH='Brassica', MS='A plant genus of the family Cruciferae. It contains many species and cultivars used as food including cabbage, cauliflower, broccoli, Brussel sprouts, kale, collard greens, MUSTARD PLANT; (B. alba, B. junica, and B. nigra), turnips (BRASSICA NAPUS) and rapeseed (BRASSICA RAPA).'),
  'D001938': NtO(MH='Brazil', MS='A country located on the eastern coast of South America, located between Colombia and Peru, that borders the Atlantic Ocean. It is bordered on the north by Venezuela, Guyana, Suriname, and French Guiana, on the south by Uruguay, and on the west by Argentina. The capital is Brasilia.'),
  'D001939': NtO(MH='Bread', MS='Baked food product made of flour or meal that is moistened, kneaded, and sometimes fermented. A major food since prehistoric times, it has been made in various forms using a variety of ingredients and methods.'),
  'D001940': NtO(MH='Breast', MS='In humans, one of the paired regions in the anterior portion of the THORAX. The breasts consist of the MAMMARY GLANDS, the SKIN, the MUSCLES, the ADIPOSE TISSUE, and the CONNECTIVE TISSUES.'),
  'D001941': NtO(MH='Breast Diseases', MS='Pathological processes of the BREAST.'),
  'D001942': NtO(MH='Breast Feeding', MS='The nursing of an infant at the breast.'),
  'D001943': NtO(MH='Breast Neoplasms', MS='Tumors or cancer of the human BREAST.'),
  'D001944': NtO(MH='Breath Tests', MS='Any tests done on exhaled air.'),
  'D001945': NtO(MH='Breathing Exercises', MS='Therapeutic exercises aimed to deepen inspiration or expiration or even to alter the rate and rhythm of respiration.'),
  'D001946': NtO(MH='Breech Presentation', MS='A malpresentation of the FETUS at near term or during OBSTETRIC LABOR with the fetal cephalic pole in the fundus of the UTERUS. There are three types of breech: the complete breech with flexed hips and knees; the incomplete breech with one or both hips partially or fully extended; the frank breech with flexed hips and extended knees.'),
  'D001947': NtO(MH='Breeding', MS='The production of offspring by selective mating or HYBRIDIZATION, GENETIC in animals or plants.'),
  'D001948': NtO(MH='Brenner Tumor', MS='A smooth, solid or cystic fibroepithelial (FIBROEPITHELIAL NEOPLASMS) tumor, usually found in the OVARIES but can also be found in the adnexal region and the KIDNEYS. It consists of a fibrous stroma with nests of epithelial cells that sometimes resemble the transitional cells lining the urinary bladder. Brenner tumors generally are benign and asymptomatic. Malignant Brenner tumors have been reported.'),
  'D001949': NtO(MH='Bretylium Compounds', MS='Benzylammonium compounds with the formula Br-phenyl-CN+R3 that include BRETYLIUM TOSYLATE.'),
  'D001950': NtO(MH='Bretylium Tosylate', MS='An agent that blocks the release of adrenergic transmitters and may have other actions. It was formerly used as an antihypertensive agent, but is now proposed as an anti-arrhythmic.'),
  'D001951': NtO(MH='Brevibacterium', MS='A gram-positive organism found in dairy products, fresh and salt water, marine organisms, insects, and decaying organic matter.'),
  'D001952': NtO(MH='Bridged-Ring Compounds', MS='Cyclic hydrocarbons that contain multiple rings which share one or more bridgehead connections.'),
  'D001954': NtO(MH='Brinolase', MS='A fibrinolytic and thrombolytic enzyme from Aspergillus oryzae. EC 3.4.99.-.'),
  'D001955': NtO(MH='British Columbia', MS="A province of Canada on the Pacific coast. Its capital is Victoria. The name given in 1858 derives from the Columbia River which was named by the American captain Robert Gray for his ship Columbia which in turn was named for Columbus. (From Webster's New Geographical Dictionary, 1988, p178 & Room, Brewer's Dictionary of Names, 1992, p81-2)"),
  'D001956': NtO(MH='Broad Ligament', MS='A broad fold of peritoneum that extends from the side of the uterus to the wall of the pelvis.'),
  'D001957': NtO(MH='Broadsides as Topic', MS='Works about published pieces of paper or other material, usually printed on one side and intended to be read unfolded and usually intended to be posted, publicly distributed, or sold, e.g., proclamations, handbills, newssheets, etc.'),
  'D001958': NtO(MH='Brocresine', MS='A histidine decarboxylase inhibitor.'),
  'D001959': NtO(MH='Bromates', MS='Negative ions or salts derived from bromic acid, HBrO3.'),
  'D001960': NtO(MH='Bromazepam', MS='One of the BENZODIAZEPINES that is used in the treatment of ANXIETY DISORDERS.'),
  'D001961': NtO(MH='Bromcresol Green', MS='An indicator and reagent. It has been used in serum albumin determinations and as a pH indicator.'),
  'D001962': NtO(MH='Bromcresol Purple', MS='An indicator and reagent. It has been used for several purposes including the determination of serum albumin concentrations'),
  'D001963': NtO(MH='Bromelains', MS='Protein-digesting and milk-clotting enzymes found in PINEAPPLE fruit juice and stem tissue. Enzymes from the two sources are distinguished as fruit bromelain and stem bromelain. This enzyme was formerly listed as EC 3.4.22.4.'),
  'D001964': NtO(MH='Bromhexine', MS='A mucolytic agent used in the treatment of respiratory disorders associated with viscid or excessive mucus. (From Martindale, The Extra Pharmacopoeia, 30th ed, p744)'),
  'D001965': NtO(MH='Bromides', MS='Salts of hydrobromic acid, HBr, with the bromine atom in the 1- oxidation state. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D001966': NtO(MH='Bromine', MS='A halogen with the atomic symbol Br, atomic number 35, and atomic weight 79.904. It is a volatile reddish-brown liquid that gives off suffocating vapors, is corrosive to the skin, and may cause severe gastroenteritis if ingested.'),
  'D001967': NtO(MH='Bromine Radioisotopes', MS='Unstable isotopes of bromine that decay or disintegrate emitting radiation. Br atoms with atomic weights 74-78, 80, and 82-90 are radioactive bromine isotopes.'),
  'D001968': NtO(MH='Bromisovalum', MS='A sedative and mild hypnotic with potentially toxic effects.'),
  'D001969': NtO(MH='Bromobenzenes', MS='Derivatives of benzene in which one or more hydrogen atoms on the benzene ring are replaced by bromine atoms.'),
  'D001970': NtO(MH='Bromobenzoates', MS='Benzoic acid or benzoic acid esters substituted with one or more bromine atoms.'),
  'D001971': NtO(MH='Bromocriptine', MS='A semisynthetic ergotamine alkaloid that is a dopamine D2 agonist. It suppresses prolactin secretion.'),
  'D001972': NtO(MH='Bromodeoxycytidine', MS="5-Bromo-2'-deoxycytidine. Can be incorporated into DNA in the presence of DNA polymerase, replacing dCTP."),
  'D001973': NtO(MH='Bromodeoxyuridine', MS='A nucleoside that substitutes for thymidine in DNA and thus acts as an antimetabolite. It causes breaks in chromosomes and has been proposed as an antiviral and antineoplastic agent. It has been given orphan drug status for use in the treatment of primary brain tumors.'),
  'D001974': NtO(MH='Bromosuccinimide', MS="A brominating agent that replaces hydrogen atoms in benzylic or allylic positions. It is used in the oxidation of secondary alcohols to ketones and in controlled low-energy brominations. (From Miall's Dictionary of Chemistry, 5th ed; Hawley's Condensed Chemical Dictionary, 12th ed,)."),
  'D001975': NtO(MH='Bromotrichloromethane', MS='A potent liver poison. In rats, bromotrichloromethane produces about three times the degree of liver microsomal lipid peroxidation as does carbon tetrachloride.'),
  'D001976': NtO(MH='Bromouracil', MS='5-Bromo-2,4(1H,3H)-pyrimidinedione. Brominated derivative of uracil that acts as an antimetabolite, substituting for thymine in DNA. It is used mainly as an experimental mutagen, but its deoxyriboside (BROMODEOXYURIDINE) is used to treat neoplasms.'),
  'D001977': NtO(MH='Brompheniramine', MS='Histamine H1 antagonist used in treatment of allergies, rhinitis, and urticaria.'),
  'D001978': NtO(MH='Bromphenol Blue', MS='A dye that has been used as an industrial dye, a laboratory indicator, and a biological stain.'),
  'D001979': NtO(MH='Bromthymol Blue', MS='A pH sensitive dye that has been used as an indicator in many laboratory reactions.'),
  'D001980': NtO(MH='Bronchi', MS='The larger air passages of the lungs arising from the terminal bifurcation of the TRACHEA. They include the largest two primary bronchi which branch out into secondary bronchi, and tertiary bronchi which extend into BRONCHIOLES and PULMONARY ALVEOLI.'),
  'D001981': NtO(MH='Bronchial Arteries', MS='Left bronchial arteries arise from the thoracic aorta, the right from the first aortic intercostal or the upper left bronchial artery; they supply the bronchi and the lower trachea.'),
  'D001982': NtO(MH='Bronchial Diseases', MS='Diseases involving the BRONCHI.'),
  'D001983': NtO(MH='Bronchial Fistula', MS='An abnormal passage or communication between a bronchus and another part of the body.'),
  'D001984': NtO(MH='Bronchial Neoplasms', MS='Tumors or cancer of the BRONCHI.'),
  'D001985': NtO(MH='Bronchial Provocation Tests', MS='Tests involving inhalation of allergens (nebulized or in dust form), nebulized pharmacologically active solutions (e.g., histamine, methacholine), or control solutions, followed by assessment of respiratory function. These tests are used in the diagnosis of asthma.'),
  'D001986': NtO(MH='Bronchial Spasm', MS='Spasmodic contraction of the smooth muscle of the bronchi.'),
  'D001987': NtO(MH='Bronchiectasis', MS='Persistent abnormal dilatation of the bronchi.'),
  'D001988': NtO(MH='Bronchiolitis', MS='Inflammation of the BRONCHIOLES.'),
  'D001989': NtO(MH='Bronchiolitis Obliterans', MS='Inflammation of the BRONCHIOLES leading to an obstructive lung disease. Bronchioles are characterized by fibrous granulation tissue with bronchial exudates in the lumens. Clinical features include a nonproductive cough and DYSPNEA.'),
  'D001990': NtO(MH='Bronchiolitis, Viral', MS='An acute inflammatory disease of the lower RESPIRATORY TRACT, caused by paramyxoviruses, occurring primarily in infants and young children; the viruses most commonly implicated are PARAINFLUENZA VIRUS TYPE 3; RESPIRATORY SYNCYTIAL VIRUS, HUMAN; and METAPNEUMOVIRUS.'),
  'D001991': NtO(MH='Bronchitis', MS='Inflammation of the large airways in the lung including any part of the BRONCHI, from the PRIMARY BRONCHI to the TERTIARY BRONCHI.'),
  'D001992': NtO(MH='Bronchoalveolar Lavage Fluid', MS='Washing liquid obtained from irrigation of the lung, including the BRONCHI and the PULMONARY ALVEOLI. It is generally used to assess biochemical, inflammatory, or infection status of the lung.'),
  'D001993': NtO(MH='Bronchodilator Agents', MS='Agents that cause an increase in the expansion of a bronchus or bronchial tubes.'),
  'D001994': NtO(MH='Bronchogenic Cyst', MS='A usually spherical cyst, arising as an embryonic out-pouching of the foregut or trachea. It is generally found in the mediastinum or lung and is usually asymptomatic unless it becomes infected.'),
  'D001995': NtO(MH='Bronchography', MS='Radiography of the bronchial tree after injection of a contrast medium.'),
  'D001996': NtO(MH='Bronchopneumonia', MS='Inflammation of the lung parenchyma that is associated with BRONCHITIS, usually involving lobular areas from TERMINAL BRONCHIOLES to the PULMONARY ALVEOLI. The affected areas become filled with exudate that forms consolidated patches.'),
  'D001997': NtO(MH='Bronchopulmonary Dysplasia', MS='A chronic lung disease developed after OXYGEN INHALATION THERAPY or mechanical ventilation (VENTILATION, MECHANICAL) usually occurring in certain premature infants (INFANT, PREMATURE) or newborn infants with respiratory distress syndrome (RESPIRATORY DISTRESS SYNDROME, NEWBORN). Histologically, it is characterized by the unusual abnormalities of the bronchioles, such as METAPLASIA, decrease in alveolar number, and formation of CYSTS.'),
  'D001998': NtO(MH='Bronchopulmonary Sequestration', MS='A developmental anomaly in which a mass of nonfunctioning lung tissue lacks normal connection with the tracheobroncheal tree and receives an anomalous blood supply originating from the descending thoracic or abdominal aorta. The mass may be extralobar, i.e., completely separated from normally connected lung, or intralobar, i.e., partly surrounded by normal lung.'),
  'D001999': NtO(MH='Bronchoscopy', MS='Endoscopic examination, therapy or surgery of the bronchi.'),
  'D002000': NtO(MH='Bronchospirometry', MS='Spirometric technique in which the volume of air breathed in the right and left lung is recorded separately.'),
  'D002001': NtO(MH='Adipose Tissue, Brown', MS='A thermogenic form of adipose tissue composed of BROWN ADIPOCYTES. It is found in newborns of many species including humans, and in hibernating mammals. Brown fat is richly vascularized, innervated, and densely packed with MITOCHONDRIA which can generate heat directly from the stored lipids.'),
  'D002002': NtO(MH='Brucella', MS='A genus of gram-negative, aerobic bacteria that causes BRUCELLOSIS. Its cells are nonmotile coccobacilli and are animal parasites and pathogens. The bacterium is transmissible to humans through contact with infected dairy products or tissue.'),
  'D002003': NtO(MH='Brucella abortus', MS='A species of the genus BRUCELLA whose natural hosts are cattle and other bovidae. Abortion and placentitis are frequently produced in the pregnant animal. Other mammals, including humans, may be infected.'),
  'D002004': NtO(MH='Brucella Vaccine', MS='A bacterial vaccine for the prevention of brucellosis in man and animal. Brucella abortus vaccine is used for the immunization of cattle, sheep, and goats.'),
  'D002006': NtO(MH='Brucellosis', MS='Infection caused by bacteria of the genus BRUCELLA mainly involving the MONONUCLEAR PHAGOCYTE SYSTEM. This condition is characterized by fever, weakness, malaise, and weight loss.'),
  'D002007': NtO(MH='Brucellosis, Bovine', MS='A disease of cattle caused by bacteria of the genus BRUCELLA leading to abortion in late pregnancy. BRUCELLA ABORTUS is the primary infective agent.'),
  'D002009': NtO(MH='Brugia', MS='A filarial worm of Southeast Asia, producing filariasis and elephantiasis in various mammals including man. It was formerly included in the genus WUCHERERIA.'),
  'D002010': NtO(MH='Brunei', MS="An independent sultanate on the northeast coast of Borneo. Its chief products are oil and natural gas. Its name is Hindi, coming from the Sanskrit bhumi, land or region. It gave its name Brunei to Borneo. (From Webster's New Geographical Dictionary, 1988, p183 & Room, Brewer's Dictionary of Names, 1992, p82)"),
  'D002011': NtO(MH='Brunner Glands', MS='The abundant submucosal mucous glands in the DUODENUM. These glands secrete BICARBONATE IONS; GLYCOPROTEINS; and PEPSINOGEN II.'),
  'D002012': NtO(MH='Bruxism', MS='A disorder characterized by grinding and clenching of the teeth.'),
  'D002013': NtO(MH='Bryozoa', MS='A phylum of small sessile aquatic animals living as small tufted colonies. Some appear like hydroids or corals, but their internal structure is more advanced. Most bryozoans are matlike, forming thin encrustations on rocks, shells, or kelp. (Storer & Stebbins, General Zoology, 6th ed, p443)'),
  'D002015': NtO(MH='Bucrylate', MS='Cyanoacrylate tissue adhesive also used to occlude blood vessels supplying neoplastic or other diseased tissue.'),
  'D002016': NtO(MH='Buddhism', MS='The teaching ascribed to Gautama Buddha (ca. 483 B.C.) holding that suffering is inherent in life and that one can escape it into nirvana by mental and moral self-purification. (Webster, 3d ed)'),
  'D002017': NtO(MH='Budgets', MS='Detailed financial plans for carrying out specific activities for a certain period of time. They include proposed income and expenditures.'),
  'D002018': NtO(MH='Bufanolides', MS='Cyclopentanophenanthrenes with a 6-membered lactone ring attached at the 17-position and SUGARS attached at the 3-position. They are found in BUFONIDAE and often possess cardiotonic properties.'),
  'D002019': NtO(MH='Bufexamac', MS='A benzeneacetamide with anti-inflammatory, analgesic, and antipyretic action. It is administered topically, orally, or rectally.'),
  'D002020': NtO(MH='Buffaloes', MS='Ruminants of the family Bovidae consisting of Bubalus arnee and Syncerus caffer. This concept is differentiated from BISON, which refers to Bison bison and Bison bonasus.'),
  'D002021': NtO(MH='Buffers', MS='A chemical system that functions to control the levels of specific ions in solution. When the level of hydrogen ion in solution is controlled the system is called a pH buffer.'),
  'D002022': NtO(MH='Bufo arenarum', MS='A species of the true toads, Bufonidae, found in South America.'),
  'D002023': NtO(MH='Bufo bufo', MS='A species of the true toads, Bufonidae, widely distributed in the United States and Europe.'),
  'D002024': NtO(MH='Bufo marinus', MS='A species of the true toads, Bufonidae, becoming fairly common in the southern United States and almost pantropical. The secretions from the skin glands of this species are very toxic to animals.'),
  'D002025': NtO(MH='Bufonidae', MS='The family of true toads belonging to the order Anura. The genera include Bufo, Ansonia, Nectophrynoides, and Atelopus.'),
  'D002026': NtO(MH='Buformin', MS='An oral hypoglycemic agent that inhibits gluconeogenesis, increases glycolysis, and decreases glucose oxidation.'),
  'D002027': NtO(MH='Bufotenin', MS='A hallucinogenic serotonin analog found in frog or toad skins, mushrooms, higher plants, and mammals, especially in the brains, plasma, and urine of schizophrenics. Bufotenin has been used as a tool in CNS studies and misused as a psychedelic.'),
  'D002028': NtO(MH='Building Codes', MS='Standards or regulations for construction which are designed to ensure safety against electrical hazards, fires, etc.'),
  'D002030': NtO(MH='Bulbourethral Glands', MS='Glands situated on each side of the prostate that secrete a fluid component of the seminal fluid into the urethra.'),
  'D002031': NtO(MH='Bulgaria', MS='A country in Southeastern Europe, bordering the Black Sea, between Romania and Turkey. The capital is Sofia.'),
  'D002032': NtO(MH='Bulimia', MS='Eating an excess amount of food in a short period of time, as seen in the disorder of BULIMIA NERVOSA. It is caused by an abnormal craving for food, or insatiable hunger also known as "ox hunger".'),
  'D002033': NtO(MH='Bulinus', MS='A genus of small freshwater snails of the order Pulmonata, found throughout Africa and the Middle East, where it is a vector of SCHISTOSOMA HAEMATOBIUM.'),
  'D002034': NtO(MH='Bumetanide', MS='A sulfamyl diuretic.'),
  'D002035': NtO(MH='Bunaftine', MS='N-Butyl-N-(2-(diethylamino)ethyl)-1-naphthamide. A proposed antiarrhythmic that prolongs myocardial refractory period and stabilizes cell membranes.'),
  'D002036': NtO(MH='Bundle of His', MS='Small band of specialized CARDIAC MUSCLE fibers that originates in the ATRIOVENTRICULAR NODE and extends into the membranous part of the interventricular septum. The bundle of His, consisting of the left and the right bundle branches, conducts the electrical impulses to the HEART VENTRICLES in generation of MYOCARDIAL CONTRACTION.'),
  'D002037': NtO(MH='Bundle-Branch Block', MS='A form of heart block in which the electrical stimulation of HEART VENTRICLES is interrupted at either one of the branches of BUNDLE OF HIS thus preventing the simultaneous depolarization of the two ventricles.'),
  'D002038': NtO(MH='Bungarotoxins', MS='Neurotoxic proteins from the venom of the banded or Formosan krait (Bungarus multicinctus, an elapid snake). alpha-Bungarotoxin blocks nicotinic acetylcholine receptors and has been used to isolate and study them; beta- and gamma-bungarotoxins act presynaptically causing acetylcholine release and depletion. Both alpha and beta forms have been characterized, the alpha being similar to the large, long or Type II neurotoxins from other elapid venoms.'),
  'D002040': NtO(MH='Levobunolol', MS='The L-Isomer of bunolol.'),
  'D002042': NtO(MH='Bunyamwera virus', MS='A species in the ORTHOBUNYAVIRUS genus of the family BUNYAVIRIDAE. A large number of serotypes or strains exist in many parts of the world. They are transmitted by mosquitoes and infect humans in some areas.'),
  'D002043': NtO(MH='Bunyaviridae', MS='A family of viruses, mainly arboviruses, consisting of a single strand of RNA. Virions are enveloped particles 90-120 nm diameter. The complete family contains over 300 members arranged in five genera: ORTHOBUNYAVIRUS; HANTAVIRUS; NAIROVIRUS; PHLEBOVIRUS; and TOSPOVIRUS.'),
  'D002044': NtO(MH='Bunyaviridae Infections', MS='Virus diseases caused by the BUNYAVIRIDAE.'),
  'D002045': NtO(MH='Bupivacaine', MS='A widely used local anesthetic agent.'),
  'D002046': NtO(MH='Bupranolol', MS='An adrenergic-beta-2 antagonist that has been used for cardiac arrhythmia, angina pectoris, hypertension, glaucoma, and as an antithrombotic.'),
  'D002047': NtO(MH='Buprenorphine', MS='A derivative of the opioid alkaloid THEBAINE that is a more potent and longer lasting analgesic than MORPHINE. It appears to act as a partial agonist at mu and kappa opioid receptors and as an antagonist at delta receptors. The lack of delta-agonist activity has been suggested to account for the observation that buprenorphine tolerance may not develop with chronic use.'),
  'D002048': NtO(MH='Burial', MS='The act or ceremony of putting a corpse into the ground or a vault, or into the sea; or the inurnment of CREMAINS.'),
  'D002049': NtO(MH='Burimamide', MS='An antagonist of histamine that appears to block both H2 and H3 histamine receptors. It has been used in the treatment of ulcers.'),
  'D002050': NtO(MH='Burkina Faso', MS='A republic in western Africa, south and east of MALI and west of NIGER. Its capital is Ouagadougou. It was formerly called Upper Volta until 1984.'),
  'D002051': NtO(MH='Burkitt Lymphoma', MS='A form of undifferentiated malignant LYMPHOMA usually found in central Africa, but also reported in other parts of the world. It is commonly manifested as a large osteolytic lesion in the jaw or as an abdominal mass. B-cell antigens are expressed on the immature cells that make up the tumor in virtually all cases of Burkitt lymphoma. The Epstein-Barr virus (HERPESVIRUS 4, HUMAN) has been isolated from Burkitt lymphoma cases in Africa and it is implicated as the causative agent in these cases; however, most non-African cases are EBV-negative.'),
  'D002052': NtO(MH='Myanmar', MS="A republic of southeast Asia, northwest of Thailand, long familiar as Burma. Its capital is Yangon, formerly Rangoon. Inhabited by people of Mongolian stock and probably of Tibetan origin, by the 3d century A.D. it was settled by Hindus. The modern Burmese state was founded in the 18th century but was in conflict with the British during the 19th century. Made a crown colony of Great Britain in 1937, it was granted independence in 1947. In 1989 it became Myanmar. The name comes from myanma, meaning the strong, as applied to the Burmese people themselves. (From Webster's New Geographical Dictionary, 1988, p192 & Room, Brewer's Dictionary of Names, 1992, p367)"),
  'D002053': NtO(MH='Burn Units', MS='Specialized hospital facilities which provide intensive care for burn patients.'),
  'D002054': NtO(MH='Burning Mouth Syndrome', MS='A group of painful oral symptoms associated with a burning or similar sensation. There is usually a significant organic component with a degree of functional overlay; it is not limited to the psychophysiologic group of disorders.'),
  'D002055': NtO(MH='Burnout, Professional', MS="An excessive stress reaction to one's occupational or professional environment. It may be characterized by feelings of emotional and physical exhaustion, coupled with a sense of frustration and failure."),
  'D002056': NtO(MH='Burns', MS='Injuries to tissues caused by contact with heat, steam, chemicals (BURNS, CHEMICAL), electricity (BURNS, ELECTRIC), or the like.'),
  'D002057': NtO(MH='Burns, Chemical', MS='Burns caused by contact with or exposure to CAUSTICS or strong ACIDS.'),
  'D002058': NtO(MH='Burns, Electric', MS='Burns produced by contact with electric current or from a sudden discharge of electricity.'),
  'D002059': NtO(MH='Burns, Inhalation', MS='Burns of the respiratory tract caused by heat or inhaled chemicals.'),
  'D002060': NtO(MH='Bursa of Fabricius', MS='An epithelial outgrowth of the cloaca in birds similar to the thymus in mammals. It atrophies within 6 months after birth and remains as a fibrous remnant in adult birds. It is composed of lymphoid tissue and prior to involution, is the site of B-lymphocyte maturation.'),
  'D002061': NtO(MH='Bursa, Synovial', MS='A fluid-filled sac lined with SYNOVIAL MEMBRANE that provides a cushion between bones, tendons and/or muscles around a joint.'),
  'D002062': NtO(MH='Bursitis', MS='Inflammation or irritation of a SYNOVIAL BURSA, the fibrous sac that acts as a cushion between moving structures of bones, muscles, tendons or skin.'),
  'D002063': NtO(MH='Burundi', MS='A republic in eastern Africa bounded on the north by RWANDA and on the south by TANZANIA. Its capital is Bujumbura.'),
  'D002064': NtO(MH='Buserelin', MS='A potent synthetic analog of GONADOTROPIN-RELEASING HORMONE with D-serine substitution at residue 6, glycine10 deletion, and other modifications.'),
  'D002065': NtO(MH='Buspirone', MS='An anxiolytic agent and serotonin receptor agonist belonging to the azaspirodecanedione class of compounds. Its structure is unrelated to those of the BENZODIAZAPINES, but it has an efficacy comparable to DIAZEPAM.'),
  'D002066': NtO(MH='Busulfan', MS='An alkylating agent having a selective immunosuppressive effect on BONE MARROW. It has been used in the palliative treatment of chronic myeloid leukemia (MYELOID LEUKEMIA, CHRONIC), but although symptomatic relief is provided, no permanent remission is brought about. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), busulfan is listed as a known carcinogen.'),
  'D002069': NtO(MH='Butaclamol', MS='A benzocycloheptapyridoisoquinolinol that has been used as an antipsychotic, especially in schizophrenia.'),
  'D002070': NtO(MH='Butadienes', MS='Four carbon unsaturated hydrocarbons containing two double bonds.'),
  'D002072': NtO(MH='Butylene Glycols', MS='4-carbon straight chain aliphatic hydrocarbons substituted with two hydroxyl groups. The hydroxyl groups cannot be on the same carbon atom.'),
  'D002073': NtO(MH='Butanes', MS='Four-carbon alkanes with the formula C4H10.'),
  'D002074': NtO(MH='Butanones', MS='Derivatives of butanone, also known as methyl ethyl ketone (with structural formula CH3COC2H5).'),
  'D002076': NtO(MH='Butirosin Sulfate', MS='A water-soluble aminoglycosidic antibiotic complex isolated from fermentation filtrates of Bacillus circulans. Two components (A and B) have been separated from the complex. Both are active against many gram-positive and some gram-negative bacteria.'),
  'D002077': NtO(MH='Butorphanol', MS='A synthetic morphinan analgesic with narcotic antagonist action. It is used in the management of severe pain.'),
  'D002078': NtO(MH='Butoxamine', MS='A beta-2 selective adrenergic antagonist. It is used primarily in animal and tissue experiments to characterize BETA-2 ANDRENERGIC RECEPTORS.'),
  'D002079': NtO(MH='Butter', MS='The fatty portion of milk, separated as a soft yellowish solid when milk or cream is churned. It is processed for cooking and table use. (Random House Unabridged Dictionary, 2d ed)'),
  'D002080': NtO(MH='Butterflies', MS='Slender-bodies diurnal insects having large, broad wings often strikingly colored and patterned.'),
  'D002081': NtO(MH='Buttocks', MS='Either of two fleshy protuberances at the lower posterior section of the trunk or HIP in humans and primate on which a person or animal sits, consisting of gluteal MUSCLES and fat.'),
  'D002082': NtO(MH='Butylamines', MS='Isomeric amines of butane, where an amino group replaces a hydrogen on one of the four carbons. They include isobutylamine, n-Butylamine, sec-Butylamine, and tert-Butylamine.'),
  'D002083': NtO(MH='Butylated Hydroxyanisole', MS='Mixture of 2- and 3-tert-butyl-4-methoxyphenols that is used as an antioxidant in foods, cosmetics, and pharmaceuticals.'),
  'D002084': NtO(MH='Butylated Hydroxytoluene', MS='A di-tert-butyl PHENOL with antioxidant properties.'),
  'D002085': NtO(MH='Butylhydroxybutylnitrosamine', MS='A substituted carcinogenic nitrosamine.'),
  'D002086': NtO(MH='Butylscopolammonium Bromide', MS='Antimuscarinic quaternary ammonium derivative of scopolamine used to treat cramps in gastrointestinal, urinary, uterine, and biliary tracts, and to facilitate radiologic visualization of the gastrointestinal tract.'),
  'D002087': NtO(MH='Butyrates', MS='Derivatives of BUTYRIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the carboxypropane structure.'),
  'D002090': NtO(MH='Butyrophenones', MS='Compounds containing phenyl-1-butanone.'),
  'D002091': NtO(MH='Butyrylcholinesterase', MS='An aspect of cholinesterase (EC 3.1.1.8).'),
  'D002092': NtO(MH='Butyrylthiocholine', MS='A sulfur-containing analog of butyrylcholine which is hydrolyzed by butyrylcholinesterase to butyrate and thiocholine. It is used as a reagent in the determination of butyrylcholinesterase activity.'),
  'D002093': NtO(MH="Benzenaminium, 4,4'-(3-oxo-1,5-pentanediyl)bis(N,N-dimethyl-N-2-propenyl-), Dibromide", MS='Proposed cholinesterase inhibitor.'),
  'D002094': NtO(MH='Republic of Belarus', MS='A country in Eastern Europe, east of Poland. The capital is Minsk.'),
  'D002095': NtO(MH='Byssinosis', MS='A condition of BRONCHOCONSTRICTION resulting from hypersensitive reaction to inhaled dust during the initial processing of cotton, flax, or hemp in the textile industry. Symptoms include wheezing and tightness in the chest.'),
  'D002096': NtO(MH='C-Peptide', MS='The middle segment of proinsulin that is between the N-terminal B-chain and the C-terminal A-chain. It is a pancreatic peptide of about 31 residues, depending on the species. Upon proteolytic cleavage of proinsulin, equimolar INSULIN and C-peptide are released. C-peptide immunoassay has been used to assess pancreatic beta cell function in diabetic patients with circulating insulin antibodies or exogenous insulin. Half-life of C-peptide is 30 min, almost 8 times that of insulin.'),
  'D002097': NtO(MH='C-Reactive Protein', MS='A plasma protein that circulates in increased amounts during inflammation and after tissue damage.'),
  'D002099': NtO(MH='Cacao', MS='A tree of the family MALVACEAE, order MALVALES, whose seeds are processed to yield COCOA and CHOCOLATE.'),
  'D002100': NtO(MH='Cachexia', MS='General ill health, malnutrition, and weight loss, usually associated with chronic disease.'),
  'D002101': NtO(MH='Cacodylic Acid', MS='An arsenical that has been used as a dermatologic agent and as an herbicide.'),
  'D002102': NtO(MH='Cadaver', MS='A dead body, usually a human body.'),
  'D002103': NtO(MH='Cadaverine', MS='A foul-smelling diamine formed by bacterial decarboxylation of lysine.'),
  'D002104': NtO(MH='Cadmium', MS='An element with atomic symbol Cd, atomic number 48, and atomic weight 112.41. It is a metal and ingestion will lead to CADMIUM POISONING.'),
  'D002105': NtO(MH='Cadmium Poisoning', MS='Poisoning occurring after exposure to cadmium compounds or fumes. It may cause gastrointestinal syndromes, anemia, or pneumonitis.'),
  'D002106': NtO(MH='Cadmium Radioisotopes', MS='Unstable isotopes of cadmium that decay or disintegrate emitting radiation. Cd atoms with atomic weights 103-105, 107, 109, 115, and 117-119 are radioactive cadmium isotopes.'),
  'D002107': NtO(MH='Caenorhabditis', MS='A genus of small free-living nematodes. Two species, CAENORHABDITIS ELEGANS and C. briggsae are much used in studies of genetics, development, aging, muscle chemistry, and neuroanatomy.'),
  'D002108': NtO(MH='Ceruletide', MS='A specific decapeptide obtained from the skin of Hila caerulea, an Australian amphibian. Caerulein is similar in action and composition to CHOLECYSTOKININ. It stimulates gastric, biliary, and pancreatic secretion; and certain smooth muscle. It is used in paralytic ileus and as diagnostic aid in pancreatic malfunction.'),
  'D002109': NtO(MH='Caffeic Acids', MS='A class of phenolic acids related to chlorogenic acid, p-coumaric acid, vanillic acid, etc., which are found in plant tissues. It is involved in plant growth regulation.'),
  'D002110': NtO(MH='Caffeine', MS="A methylxanthine naturally occurring in some beverages and also used as a pharmacological agent. Caffeine's most notable pharmacological effect is as a central nervous system stimulant, increasing alertness and producing agitation. It also relaxes SMOOTH MUSCLE, stimulates CARDIAC MUSCLE, stimulates DIURESIS, and appears to be useful in the treatment of some types of headache. Several cellular actions of caffeine have been observed, but it is not entirely clear how each contributes to its pharmacological profile. Among the most important are inhibition of cyclic nucleotide PHOSPHODIESTERASES, antagonism of ADENOSINE RECEPTORS, and modulation of intracellular calcium handling."),
  'D002111': NtO(MH='Calcaneus', MS='The largest of the TARSAL BONES which is situated at the lower and back part of the FOOT, forming the HEEL.'),
  'D002112': NtO(MH='Calcifediol', MS="The major circulating metabolite of VITAMIN D3. It is produced in the LIVER and is the best indicator of the body's vitamin D stores. It is effective in the treatment of RICKETS and OSTEOMALACIA, both in azotemic and non-azotemic patients. Calcifediol also has mineralizing properties."),
  'D002113': NtO(MH='Calcification, Physiologic', MS='Process by which organic tissue becomes hardened by the physiologic deposit of calcium salts.'),
  'D002114': NtO(MH='Calcinosis', MS='Pathologic deposition of calcium salts in tissues.'),
  'D002115': NtO(MH='Calciphylaxis', MS='Condition of induced systemic hypersensitivity in which tissues respond to appropriate challenging agents with a sudden local calcification.'),
  'D002116': NtO(MH='Calcitonin', MS='A peptide hormone that lowers calcium concentration in the blood. In humans, it is released by thyroid cells and acts to decrease the formation and absorptive activity of osteoclasts. Its role in regulating plasma calcium is much greater in children and in certain diseases than in normal adults.'),
  'D002117': NtO(MH='Calcitriol', MS='The physiologically active form of vitamin D. It is formed primarily in the kidney by enzymatic hydroxylation of 25-hydroxycholecalciferol (CALCIFEDIOL). Its production is stimulated by low blood calcium levels and parathyroid hormone. Calcitriol increases intestinal absorption of calcium and phosphorus, and in concert with parathyroid hormone increases bone resorption.'),
  'D002118': NtO(MH='Calcium', MS='A basic element found in nearly all tissues. It is a member of the alkaline earth family of metals with the atomic symbol Ca, atomic number 20, and atomic weight 40. Calcium is the most abundant mineral in the body and combines with phosphorus to form calcium phosphate in the bones and teeth. It is essential for the normal functioning of nerves and muscles and plays a role in blood coagulation (as factor IV) and in many enzymatic processes.'),
  'D002119': NtO(MH='Calcium Carbonate', MS='Carbonic acid calcium salt (CaCO3). An odorless, tasteless powder or crystal that occurs in nature. It is used therapeutically as a phosphate buffer in hemodialysis patients and as a calcium supplement.'),
  'D002120': NtO(MH='Calcium Channel Agonists', MS='Agents that increase calcium influx into calcium channels of excitable tissues. This causes vasoconstriction in VASCULAR SMOOTH MUSCLE and/or CARDIAC MUSCLE cells as well as stimulation of insulin release from pancreatic islets. Therefore, tissue-selective calcium agonists have the potential to combat cardiac failure and endocrinological disorders. They have been used primarily in experimental studies in cell and tissue culture.'),
  'D002121': NtO(MH='Calcium Channel Blockers', MS='A class of drugs that act by selective inhibition of calcium influx through cellular membranes.'),
  'D002122': NtO(MH='Calcium Chloride', MS='A salt used to replenish calcium levels, as an acid-producing diuretic, and as an antidote for magnesium poisoning.'),
  'D002123': NtO(MH='Calcium Dobesilate', MS='A drug used to reduce hemorrhage in diabetic retinopathy.'),
  'D002124': NtO(MH='Calcium Fluoride', MS='Calcium fluoride. Occurring in nature as the mineral fluorite or fluorspar. It is the primary source of fluorine and its compounds. Pure calcium fluoride is used as a catalyst in dehydration and dehydrogenation and is used to fluoridate drinking water. (From Merck Index, 11th ed)'),
  'D002125': NtO(MH='Calcium Gluconate', MS='The calcium salt of gluconic acid. The compound has a variety of uses, including its use as a calcium replenisher in hypocalcemic states.'),
  'D002126': NtO(MH='Calcium Hydroxide', MS='A white powder prepared from lime that has many medical and industrial uses. It is in many dental formulations, especially for root canal filling.'),
  'D002127': NtO(MH='Calcium Isotopes', MS='Stable calcium atoms that have the same atomic number as the element calcium, but differ in atomic weight. Ca-42-44, 46, and 48 are stable calcium isotopes.'),
  'D002128': NtO(MH='Calcium Metabolism Disorders', MS='Disorders in the processing of calcium in the body: its absorption, transport, storage, and utilization.'),
  'D002129': NtO(MH='Calcium Oxalate', MS='The calcium salt of oxalic acid, occurring in the urine as crystals and in certain calculi.'),
  'D002130': NtO(MH='Calcium Phosphates', MS='Calcium salts of phosphoric acid. These compounds are frequently used as calcium supplements.'),
  'D002131': NtO(MH='Calcium Pyrophosphate', MS='An inorganic pyrophosphate which affects calcium metabolism in mammals. Abnormalities in its metabolism occur in some human diseases, notably HYPOPHOSPHATASIA and pseudogout (CHONDROCALCINOSIS).'),
  'D002132': NtO(MH='Calcium Radioisotopes', MS='Unstable isotopes of calcium that decay or disintegrate emitting radiation. Ca atoms with atomic weights 39, 41, 45, 47, 49, and 50 are radioactive calcium isotopes.'),
  'D002133': NtO(MH='Calcium Sulfate', MS='A calcium salt that is used for a variety of purposes including: building materials, as a desiccant, in dentistry as an impression material, cast, or die, and in medicine for immobilizing casts and as a tablet excipient. It exists in various forms and states of hydration. Plaster of Paris is a mixture of powdered and heat-treated gypsum.'),
  'D002135': NtO(MH='Calcium-Binding Proteins', MS='Proteins to which calcium ions are bound. They can act as transport proteins, regulator proteins, or activator proteins. They typically contain EF HAND MOTIFS.'),
  'D002136': NtO(MH='Calcium, Dietary', MS='Calcium compounds used as food supplements or in food to supply the body with calcium. Dietary calcium is needed during growth for bone development and for maintenance of skeletal integrity later in life to prevent osteoporosis.'),
  'D002137': NtO(MH='Calculi', MS='An abnormal concretion occurring mostly in the urinary and biliary tracts, usually composed of mineral salts. Also called stones.'),
  'D002138': NtO(MH='Calibration', MS='Determination, by measurement or comparison with a standard, of the correct value of each scale reading on a meter or other measuring instrument; or determination of the settings of a control device that correspond to particular values of voltage, current, frequency or other output.'),
  'D002139': NtO(MH='Caliciviridae', MS='A family of RNA viruses infecting a broad range of animals. Most individual species are restricted to their natural hosts. They possess a characteristic six-pointed starlike shape whose surfaces have cup-shaped (chalice) indentions. Transmission is by contaminated food, water, fomites, and occasionally aerosolization of secretions. Genera include LAGOVIRUS; NORWALK-LIKE VIRUSES; SAPPORO-LIKE VIRUSES; and VESIVIRUS.'),
  'D002140': NtO(MH='California', MS='State bounded on the east by Nevada and Arizona, on the south by Mexico and the Pacific Ocean on the south and west, and on the north by Oregon.'),
  'D002141': NtO(MH='Encephalitis Virus, California', MS='A species in the ORTHOBUNYAVIRUS genus of the family BUNYAVIRIDAE. Serotypes are found in temperate and arctic regions and each is closely associated with a single species of vector mosquito. The vertebrate hosts are usually small mammals but several serotypes infect humans.'),
  'D002142': NtO(MH='Californium', MS='A man-made radioactive actinide with atomic symbol Cf, atomic number 98, and atomic weight 251. Its valence can be +2 or +3. Californium has medical use as a radiation source for radiotherapy.'),
  'D002143': NtO(MH='Callitrichinae', MS="A subfamily in the family CEBIDAE that consists of four genera: CALLITHRIX (marmosets), CALLIMICO (Goeldi's monkey), LEONTOPITHECUS (lion tamarins), and SAGUINUS (long-tusked tamarins). The members of this family inhabit the tropical forests of South and Central America."),
  'D002144': NtO(MH='Callithrix', MS='A genus of the subfamily CALLITRICHINAE occurring in forests of Brazil and Bolivia and containing seventeen species.'),
  'D002145': NtO(MH='Callosities', MS='Localized hyperplasia of the horny layer of the epidermis due to pressure or friction. (Dorland, 27th ed)'),
  'D002146': NtO(MH='Bony Callus', MS='The bony deposit formed between and around the broken ends of BONE FRACTURES during normal healing.'),
  'D002147': NtO(MH='Calmodulin', MS='A heat-stable, low-molecular-weight activator protein found mainly in the brain and heart. The binding of calcium ions to this protein allows this protein to bind to cyclic nucleotide phosphodiesterases and to adenyl cyclase with subsequent activation. Thereby this protein modulates cyclic AMP and cyclic GMP levels.'),
  'D002148': NtO(MH='Calmodulin-Binding Proteins', MS='Proteins which bind calmodulin. They are found in many tissues and have a variety of functions including F-actin cross-linking properties, inhibition of cyclic nucleotide phosphodiesterase and calcium and magnesium ATPases.'),
  'D002149': NtO(MH='Energy Intake', MS='Total number of calories taken in daily whether ingested or by parenteral routes.'),
  'D002150': NtO(MH='Caloric Tests', MS='Elicitation of a rotatory nystagmus by stimulating the semicircular canals with water or air which is above or below body temperature. In warm caloric stimulation a rotatory nystagmus is developed toward the side of the stimulated ear; in cold, away from the stimulated side. Absence of nystagmus indicates the labyrinth is not functioning.'),
  'D002151': NtO(MH='Calorimetry', MS='The measurement of the quantity of heat involved in various processes, such as chemical reactions, changes of state, and formations of solutions, or in the determination of the heat capacities of substances. The fundamental unit of measurement is the joule or the calorie (4.184 joules). (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002152': NtO(MH='Calorimetry, Differential Scanning', MS='Differential thermal analysis in which the sample compartment of the apparatus is a differential calorimeter, allowing an exact measure of the heat of transition independent of the specific heat, thermal conductivity, and other variables of the sample.'),
  'D002153': NtO(MH='Calorimetry, Indirect', MS='Calculation of the energy expenditure in the form of heat production of the whole body or individual organs based on respiratory gas exchange.'),
  'D002154': NtO(MH='Calpain', MS='Cysteine proteinase found in many tissues. Hydrolyzes a variety of endogenous proteins including NEUROPEPTIDES; CYTOSKELETAL PROTEINS; proteins from SMOOTH MUSCLE; CARDIAC MUSCLE; liver; platelets; and erythrocytes. Two subclasses having high and low calcium sensitivity are known. Removes Z-discs and M-lines from myofibrils. Activates phosphorylase kinase and cyclic nucleotide-independent protein kinase. This enzyme was formerly listed as EC 3.4.22.4.'),
  'D002155': NtO(MH='Calsequestrin', MS='Acidic protein found in SARCOPLASMIC RETICULUM that binds calcium to the extent of 700-900 nmoles/mg. It plays the role of sequestering calcium transported to the interior of the intracellular vesicle.'),
  'D002157': NtO(MH='Calymmatobacterium', MS='A genus of bacteria causing GRANULOMA INGUINALE and other granulomatous lesions.'),
  'D002159': NtO(MH='Cambendazole', MS='A nematocide effective against a variety of gastrointestinal parasites in cattle, sheep, and horses.'),
  'D002160': NtO(MH='Cambodia', MS='A country located in Southeastern Asia, bordering the Gulf of Thailand, between Thailand, Vietnam, and Laos. The capital is Phnom Penh.'),
  'D002161': NtO(MH='Camelids, New World', MS='Camelidae of the Americas. The extant species are those originating from South America and include alpacas, llamas, guanicos, and vicunas.'),
  'D002162': NtO(MH='Camelus', MS='Two-toed, hoofed mammals with four legs, a big-lipped snout, and a humped back belonging to the family Camelidae. They are native to North Africa, and Western and Central Asia.'),
  'D002163': NtO(MH='Cameroon', MS='A republic in central Africa lying east of CHAD and the CENTRAL AFRICAN REPUBLIC and west of NIGERIA. The capital is Yaounde.'),
  'D002164': NtO(MH='Camphor', MS='A bicyclic monoterpene ketone found widely in plants, especially CINNAMOMUM CAMPHORA. It is used topically as a skin antipruritic and as an anti-infective agent.'),
  'D002165': NtO(MH='Camping', MS='Living outdoors as a recreational activity.'),
  'D002166': NtO(MH='Camptothecin', MS='An alkaloid isolated from the stem wood of the Chinese tree, Camptotheca acuminata. This compound selectively inhibits the nuclear enzyme DNA TOPOISOMERASES, TYPE I. Several semisynthetic analogs of camptothecin have demonstrated antitumor activity.'),
  'D002167': NtO(MH='Campylobacter', MS='A genus of bacteria found in the reproductive organs, intestinal tract, and oral cavity of animals and man. Some species are pathogenic.'),
  'D002168': NtO(MH='Campylobacter fetus', MS='A species of bacteria present in man and many kinds of animals and birds, often causing infertility and/or abortion.'),
  'D002169': NtO(MH='Campylobacter Infections', MS='Infections with bacteria of the genus CAMPYLOBACTER.'),
  'D002170': NtO(MH='Canada', MS='The largest country in North America, comprising 10 provinces and three territories. Its capital is Ottawa.'),
  'D002171': NtO(MH='Canaries', MS='Any of several Old World finches of the genus Serinus.'),
  'D002172': NtO(MH='Canavanine', MS=None),
  'D002173': NtO(MH='Cancer Care Facilities', MS='Institutions specializing in the care of cancer patients.'),
  'D002174': NtO(MH='Candicidin', MS='Mixture of antifungal heptaene macrolides from Streptomyces griseus or Actinomyces levoris used topically in candidiasis. The antibiotic complex is composed of candicidins A, B, C, and D, of which D is the major component.'),
  'D002175': NtO(MH='Candida', MS='A genus of yeast-like mitosporic Saccharomycetales fungi characterized by producing yeast cells, mycelia, pseudomycelia, and blastophores. It is commonly part of the normal flora of the skin, mouth, intestinal tract, and vagina, but can cause a variety of infections, including CANDIDIASIS; ONYCHOMYCOSIS; VULVOVAGINAL CANDIDIASIS; and CANDIDIASIS, ORAL (THRUSH).'),
  'D002176': NtO(MH='Candida albicans', MS='A unicellular budding fungus which is the principal pathogenic species causing CANDIDIASIS (moniliasis).'),
  'D002177': NtO(MH='Candidiasis', MS='Infection with a fungus of the genus CANDIDA. It is usually a superficial infection of the moist areas of the body and is generally caused by CANDIDA ALBICANS. (Dorland, 27th ed)'),
  'D002178': NtO(MH='Candidiasis, Chronic Mucocutaneous', MS='A clinical syndrome characterized by development, usually in infancy or childhood, of a chronic, often widespread candidiasis of skin, nails, and mucous membranes. It may be secondary to one of the immunodeficiency syndromes, inherited as an autosomal recessive trait, or associated with defects in cell-mediated immunity, endocrine disorders, dental stomatitis, or malignancy.'),
  'D002179': NtO(MH='Candidiasis, Cutaneous', MS='Candidiasis of the skin manifested as eczema-like lesions of the interdigital spaces, perleche, or chronic paronychia. (Dorland, 27th ed)'),
  'D002180': NtO(MH='Candidiasis, Oral', MS='Infection of the mucous membranes of the mouth by a fungus of the genus CANDIDA. (Dorland, 27th ed)'),
  'D002181': NtO(MH='Candidiasis, Vulvovaginal', MS='Infection of the VULVA and VAGINA with a fungus of the genus CANDIDA.'),
  'D002182': NtO(MH='Candy', MS='Sweet food products combining cane or beet sugars with other carbohydrates and chocolate, milk, eggs, and various flavorings. In the United States, candy refers to both sugar- and cocoa-based confections and is differentiated from sweetened baked goods; elsewhere the terms sugar confectionary, chocolate confectionary, and flour confectionary (meaning goods such as cakes and pastries) are used.'),
  'D002183': NtO(MH='Canes', MS='Sticks used as walking aids. The canes may have three or four prongs at the end of the shaft.'),
  'D002184': NtO(MH='Herpesvirus 1, Canid', MS='A species of VARICELLOVIRUS virus that causes a disease in newborn puppies.'),
  'D002185': NtO(MH='Cannabidiol', MS='Compound isolated from Cannabis sativa extract.'),
  'D002186': NtO(MH='Cannabinoids', MS='Compounds having the cannabinoid structure. They were originally extracted from Cannabis sativa L. The most pharmacologically active constituents are TETRAHYDROCANNABINOL; CANNABINOL; and CANNABIDIOL.'),
  'D002187': NtO(MH='Cannabinol', MS='A physiologically inactive constituent of Cannabis sativa L.'),
  'D002188': NtO(MH='Cannabis', MS='The plant genus in the Cannabaceae plant family, Urticales order, Hamamelidae subclass. The flowering tops are called many slang terms including pot, marijuana, hashish, bhang, and ganja. The stem is an important source of hemp fiber.'),
  'D002189': NtO(MH='Marijuana Abuse', MS='Use of marijuana associated with abnormal psychological, social, and or occupational functioning.'),
  'D002190': NtO(MH='Cannibalism', MS="Eating other individuals of one's own species."),
  'D002191': NtO(MH='Canrenoic Acid', MS='A synthetic pregnadiene derivative with anti-aldosterone activity.'),
  'D002192': NtO(MH='Canrenone', MS='A synthetic pregnadiene compound with anti-aldosterone activity.'),
  'D002193': NtO(MH='Cantharidin', MS='A toxic compound, isolated from the Spanish fly or blistering beetle (Lytta (Cantharis) vesicatoria) and other insects. It is a potent and specific inhibitor of protein phosphatases 1 (PP1) and 2A (PP2A). This compound can produce severe skin inflammation, and is extremely toxic if ingested orally.'),
  'D002194': NtO(MH='Capgras Syndrome', MS="A psychotic disorder characterized by the patient's belief that acquaintances or closely related persons have been replaced by doubles or imposters."),
  'D002195': NtO(MH='Capillaria', MS='A genus of trichuroid nematodes parasitic in the liver and intestines of many mammals and birds. Two species, C. hepatica and C. philippinensis, produce often fatal infections in man.'),
  'D002196': NtO(MH='Capillaries', MS='The minute vessels that connect arterioles and venules.'),
  'D002197': NtO(MH='Capillary Action', MS='A phenomenon in which the surface of a liquid where it contacts a solid is elevated or depressed, because of the relative attraction of the molecules of the liquid for each other and for those of the solid. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002198': NtO(MH='Capillary Fragility', MS='The susceptibility of CAPILLARIES, under conditions of increased stress, to leakage.'),
  'D002199': NtO(MH='Capillary Permeability', MS='The property of blood capillary ENDOTHELIUM that allows for the selective exchange of substances between the blood and surrounding tissues and through membranous barriers such as the BLOOD-AIR BARRIER; BLOOD-AQUEOUS BARRIER; BLOOD-BRAIN BARRIER; BLOOD-NERVE BARRIER; BLOOD-RETINAL BARRIER; and BLOOD-TESTIS BARRIER. Small lipid-soluble molecules such as carbon dioxide and oxygen move freely by diffusion. Water and water-soluble molecules cannot pass through the endothelial walls and are dependent on microscopic pores. These pores show narrow areas (TIGHT JUNCTIONS) which may limit large molecule movement.'),
  'D002200': NtO(MH='Capillary Resistance', MS='The vascular resistance to the flow of BLOOD through the CAPILLARIES portions of the peripheral vascular bed.'),
  'D002201': NtO(MH='Capital Expenditures', MS='Those funds disbursed for facilities and equipment, particularly those related to the delivery of health care.'),
  'D002202': NtO(MH='Capital Financing', MS='Institutional funding for facilities and for equipment which becomes a part of the assets of the institution.'),
  'D002203': NtO(MH='Capital Punishment', MS='The use of the death penalty for certain crimes.'),
  'D002204': NtO(MH='Capitation Fee', MS='A method of payment for health services in which an individual or institutional provider is paid a fixed, per capita amount without regard to the actual number or nature of services provided to each patient.'),
  'D002205': NtO(MH='Caplan Syndrome', MS='A condition characterized by the presence of RHEUMATOID ARTHRITIS associated with a specific form of pneumoconiosis, often in coal miners and asbestos workers.'),
  'D002206': NtO(MH='Capnocytophaga', MS='A gram-negative gliding bacterium isolated from the oral cavity. It is a pathogen often causing PERIODONTITIS.'),
  'D002207': NtO(MH='Capreomycin', MS='Cyclic peptide antibiotic similar to VIOMYCIN. It is produced by Streptomyces capreolus.'),
  'D002208': NtO(MH='Caproates', MS='Derivatives of caproic acid. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain a carboxy terminated six carbon aliphatic structure.'),
  'D002209': NtO(MH='Caprolactam', MS='Cyclic amide of caproic acid used in manufacture of synthetic fibers of the polyamide type. Can cause local irritation.'),
  'D002210': NtO(MH='Caprylates', MS='Derivatives of caprylic acid. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain a carboxy terminated eight carbon aliphatic structure.'),
  'D002211': NtO(MH='Capsaicin', MS='An alkylamide found in CAPSICUM that acts at TRPV CATION CHANNELS.'),
  'D002212': NtO(MH='Capsicum', MS='A plant genus of the family SOLANACEAE. The hot peppers yield CAPSAICIN, which activates VANILLOID RECEPTORS. Several varieties have sweet or pungent edible fruits that are used as vegetables when fresh and spices when the pods are dried.'),
  'D002213': NtO(MH='Capsid', MS='The outer protein protective shell of a virus, which protects the viral nucleic acid.'),
  'D002214': NtO(MH='Capsules', MS='Hard or soft soluble containers used for the oral administration of medicine.'),
  'D002215': NtO(MH='Captan', MS='One of the phthalimide fungicides.'),
  'D002216': NtO(MH='Captopril', MS='A potent and specific inhibitor of PEPTIDYL-DIPEPTIDASE A. It blocks the conversion of ANGIOTENSIN I to ANGIOTENSIN II, a vasoconstrictor and important regulator of arterial blood pressure. Captopril acts to suppress the RENIN-ANGIOTENSIN SYSTEM and inhibits pressure responses to exogenous angiotensin.'),
  'D002217': NtO(MH='Carbachol', MS='A slowly hydrolyzed CHOLINERGIC AGONIST that acts at both MUSCARINIC RECEPTORS and NICOTINIC RECEPTORS.'),
  'D002218': NtO(MH='Carbadox', MS='An antibacterial agent that has been used in veterinary practice for treating swine dysentery and enteritis and for promoting growth. However, its use has been prohibited in the UK following reports of carcinogenicity and mutagenicity. (From Martindale, The Extra Pharmacopoeia, 30th ed, p125)'),
  'D002219': NtO(MH='Carbamates', MS='Derivatives of carbamic acid, H2NC(=O)OH. Included under this heading are N-substituted and O-substituted carbamic acids. In general carbamate esters are referred to as urethanes, and polymers that include repeating units of carbamate are referred to as POLYURETHANES. Note however that polyurethanes are derived from the polymerization of ISOCYANATES and the singular term URETHANE refers to the ethyl ester of carbamic acid.'),
  'D002220': NtO(MH='Carbamazepine', MS='A dibenzazepine that acts as a sodium channel blocker. It is used as an anticonvulsant for the treatment of grand mal and psychomotor or focal SEIZURES. It may also be used in the management of BIPOLAR DISORDER, and has analgesic properties.'),
  'D002221': NtO(MH='Carbamyl Phosphate', MS='The monoanhydride of carbamic acid with PHOSPHORIC ACID. It is an important intermediate metabolite and is synthesized enzymatically by CARBAMYL-PHOSPHATE SYNTHASE (AMMONIA) and CARBAMOYL-PHOSPHATE SYNTHASE (GLUTAMINE-HYDROLYZING).'),
  'D002222': NtO(MH='Carbamoyl-Phosphate Synthase (Ammonia)', MS='An enzyme that catalyzes the formation of carbamoyl phosphate from ATP, carbon dioxide, and ammonia. This enzyme is specific for arginine biosynthesis or the urea cycle. Absence or lack of this enzyme may cause CARBAMOYL-PHOSPHATE SYNTHASE I DEFICIENCY DISEASE. EC 6.3.4.16.'),
  'D002223': NtO(MH='Carbamoyl-Phosphate Synthase (Glutamine-Hydrolyzing)', MS='An enzyme that catalyzes the formation of carbamoyl phosphate from ATP, carbon dioxide, and glutamine. This enzyme is important in the de novo biosynthesis of pyrimidines. EC 6.3.5.5.'),
  'D002224': NtO(MH='Carbanilides', MS='Compounds consisting of two phenyl groups joined by a urea. Derivatives may be used as ANTIPARASITIC AGENTS.'),
  'D002225': NtO(MH='Carbazilquinone', MS='An alkylating agent structurally similar to MITOMYCIN and found to be effective in the treatment of leukemia and various other neoplasms in mice. It causes leukemia and thrombocytopenia in almost all human patients.'),
  'D002227': NtO(MH='Carbazoles', MS='Benzo-indoles similar to CARBOLINES which are pyrido-indoles. In plants, carbazoles are derived from indole and form some of the INDOLE ALKALOIDS.'),
  'D002228': NtO(MH='Carbenicillin', MS='Broad-spectrum semisynthetic penicillin derivative used parenterally. It is susceptible to gastric juice and penicillinase and may damage platelet function.'),
  'D002229': NtO(MH='Carbenoxolone', MS='An agent derived from licorice root. It is used for the treatment of digestive tract ulcers, especially in the stomach. Antidiuretic side effects are frequent, but otherwise the drug is low in toxicity.'),
  'D002230': NtO(MH='Carbidopa', MS='An inhibitor of DOPA DECARBOXYLASE that prevents conversion of LEVODOPA to dopamine. It is used in PARKINSON DISEASE to reduce peripheral adverse effects of LEVODOPA. It has no anti-parkinson activity by itself.'),
  'D002231': NtO(MH='Carbimazole', MS='An imidazole antithyroid agent. Carbimazole is metabolized to METHIMAZOLE, which is responsible for the antithyroid activity.'),
  'D002232': NtO(MH='Carbocyanines', MS='Compounds that contain three methine groups. They are frequently used as cationic dyes used for differential staining of biological materials.'),
  'D002233': NtO(MH='Carbocysteine', MS='A compound formed when iodoacetic acid reacts with sulfhydryl groups in proteins. It has been used as an anti-infective nasal spray with mucolytic and expectorant action.'),
  'D002234': NtO(MH='Carbodiimides', MS='Compounds with the general formula RN=C=NR, where R is a hydrocarbyl group.'),
  'D002235': NtO(MH='Carbofuran', MS='A cholinesterase inhibitor that is used as a systemic insecticide, an acaricide, and nematocide. (From Merck Index, 11th ed)'),
  'D002236': NtO(MH='Carbohydrate Conformation', MS='The characteristic 3-dimensional shape of a carbohydrate.'),
  'D002237': NtO(MH='Carbohydrate Dehydrogenases', MS='Reversibly catalyze the oxidation of a hydroxyl group of carbohydrates to form a keto sugar, aldehyde or lactone. Any acceptor except molecular oxygen is permitted. Includes EC 1.1.1.; EC 1.1.2.; and 1.1.99.'),
  'D002238': NtO(MH='Carbohydrate Epimerases', MS='Enzymes that catalyze the epimerization of chiral centers within carbohydrates or their derivatives. EC 5.1.3.'),
  'D002239': NtO(MH='Carbohydrate Metabolism, Inborn Errors', MS='Dysfunctions of CARBOHYDRATE METABOLISM resulting from inborn genetic mutations that are inherited or acquired in utero.'),
  'D002240': NtO(MH='Carbohydrate Sequence', MS='The sequence of carbohydrates within POLYSACCHARIDES; GLYCOPROTEINS; and GLYCOLIPIDS.'),
  'D002241': NtO(MH='Carbohydrates', MS='A class of organic compounds composed of carbon, hydrogen, and oxygen in a ratio of Cn(H2O)n. The largest class of organic compounds, including STARCH; GLYCOGEN; CELLULOSE; POLYSACCHARIDES; and simple MONOSACCHARIDES.'),
  'D002243': NtO(MH='Carbolines', MS='A group of pyrido-indole compounds. Included are any points of fusion of pyridine with the five-membered ring of indole and any derivatives of these compounds. These are similar to CARBAZOLES which are benzo-indoles.'),
  'D002244': NtO(MH='Carbon', MS='A nonmetallic element with atomic symbol C, atomic number 6, and atomic weight [12.0096; 12.0116]. It may occur as several different allotropes including DIAMOND; CHARCOAL; and GRAPHITE; and as SOOT from incompletely burned fuel.'),
  'D002245': NtO(MH='Carbon Dioxide', MS='A colorless, odorless gas that can be formed by the body and is necessary for the respiration cycle of plants and animals.'),
  'D002246': NtO(MH='Carbon Disulfide', MS='A colorless, flammable, poisonous liquid, CS2. It is used as a solvent, and is a counterirritant and has local anesthetic properties but is not used as such. It is highly toxic with pronounced CNS, hematologic, and dermatologic effects.'),
  'D002247': NtO(MH='Carbon Isotopes', MS='Stable carbon atoms that have the same atomic number as the element carbon, but differ in atomic weight. C-13 is a stable carbon isotope.'),
  'D002248': NtO(MH='Carbon Monoxide', MS='Carbon monoxide (CO). A poisonous colorless, odorless, tasteless gas. It combines with hemoglobin to form carboxyhemoglobin, which has no oxygen carrying capacity. The resultant oxygen deprivation causes headache, dizziness, decreased pulse and respiratory rates, unconsciousness, and death. (From Merck Index, 11th ed)'),
  'D002249': NtO(MH='Carbon Monoxide Poisoning', MS='Toxic asphyxiation due to the displacement of oxygen from oxyhemoglobin by carbon monoxide.'),
  'D002250': NtO(MH='Carbon Radioisotopes', MS='Unstable isotopes of carbon that decay or disintegrate emitting radiation. C atoms with atomic weights 10, 11, and 14-16 are radioactive carbon isotopes.'),
  'D002251': NtO(MH='Carbon Tetrachloride', MS='A solvent for oils, fats, lacquers, varnishes, rubber waxes, and resins, and a starting material in the manufacturing of organic compounds. Poisoning by inhalation, ingestion or skin absorption is possible and may be fatal. (Merck Index, 11th ed)'),
  'D002252': NtO(MH='Carbon Tetrachloride Poisoning', MS='Poisoning that results from ingestion, injection, inhalation, or skin absorption of CARBON TETRACHLORIDE.'),
  'D002253': NtO(MH='Carbonated Beverages', MS='Drinkable liquids combined with or impregnated with CARBON DIOXIDE.'),
  'D002254': NtO(MH='Carbonates', MS="Salts or ions of the theoretical carbonic acid, containing the radical CO2(3-). Carbonates are readily decomposed by acids. The carbonates of the alkali metals are water-soluble; all others are insoluble. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D002255': NtO(MH='Carbonic Acid', MS="Carbonic acid (H2C03). The hypothetical acid of carbon dioxide and water. It exists only in the form of its salts (carbonates), acid salts (hydrogen carbonates), amines (carbamic acid), and acid chlorides (carbonyl chloride). (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D002256': NtO(MH='Carbonic Anhydrases', MS='A family of zinc-containing enzymes that catalyze the reversible hydration of carbon dioxide. They play an important role in the transport of CARBON DIOXIDE from the tissues to the LUNG. EC 4.2.1.1.'),
  'D002257': NtO(MH='Carbonic Anhydrase Inhibitors', MS='A class of compounds that reduces the secretion of H+ ions by the proximal kidney tubule through inhibition of CARBONIC ANHYDRASES.'),
  'D002258': NtO(MH='Carbonyl Cyanide m-Chlorophenyl Hydrazone', MS='A proton ionophore. It is commonly used as an uncoupling agent and inhibitor of photosynthesis because of its effects on mitochondrial and chloroplast membranes.'),
  'D002259': NtO(MH='Carbonyl Cyanide p-Trifluoromethoxyphenylhydrazone', MS='A proton ionophore that is commonly used as an uncoupling agent in biochemical studies.'),
  'D002260': NtO(MH='Carboprost', MS='A nonsteroidal abortifacient agent that is effective in both the first and second trimesters of PREGNANCY.'),
  'D002261': NtO(MH='Carboxin', MS='A systemic agricultural fungicide and seed treatment agent.'),
  'D002262': NtO(MH='Carboxy-Lyases', MS='Enzymes that catalyze the addition of a carboxyl group to a compound (carboxylases) or the removal of a carboxyl group from a compound (decarboxylases). EC 4.1.1.'),
  'D002263': NtO(MH='Carboxyhemoglobin', MS=None),
  'D002264': NtO(MH='Carboxylic Acids', MS='Organic compounds containing the carboxy group (-COOH). This group of compounds includes amino acids and fatty acids. Carboxylic acids can be saturated, unsaturated, or aromatic.'),
  'D002265': NtO(MH='Carboxylic Ester Hydrolases', MS='Enzymes which catalyze the hydrolysis of carboxylic acid esters with the formation of an alcohol and a carboxylic acid anion.'),
  'D002266': NtO(MH='Carboxymethylcellulose Sodium', MS='A cellulose derivative which is a beta-(1,4)-D-glucopyranose polymer. It is used as a bulk laxative and as an emulsifier and thickener in cosmetics and pharmaceuticals and as a stabilizer for reagents.'),
  'D002267': NtO(MH='Muramoylpentapeptide Carboxypeptidase', MS='Enzyme which catalyzes the peptide cross-linking of nascent CELL WALL; PEPTIDOGLYCAN.'),
  'D002268': NtO(MH='Carboxypeptidases', MS='Enzymes that act at a free C-terminus of a polypeptide to liberate a single amino acid residue.'),
  'D002270': NtO(MH='Carbuncle', MS='An infection of cutaneous and subcutaneous tissue that consists of a cluster of boils. Commonly, the causative agent is STAPHYLOCOCCUS AUREUS. Carbuncles produce fever, leukocytosis, extreme pain, and prostration.'),
  'D002271': NtO(MH='Carbutamide', MS='A sulfonylurea antidiabetic agent with similar actions and uses to CHLORPROPAMIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p277)'),
  'D002272': NtO(MH='Carcinoembryonic Antigen', MS='A glycoprotein that is secreted into the luminal surface of the epithelia in the gastrointestinal tract. It is found in the feces and pancreaticobiliary secretions and is used to monitor the response to colon cancer treatment.'),
  'D002273': NtO(MH='Carcinogens', MS='Substances that increase the risk of NEOPLASMS in humans or animals. Both genotoxic chemicals, which affect DNA directly, and nongenotoxic chemicals, which induce neoplasms by other mechanism, are included.'),
  'D002274': NtO(MH='Carcinogens, Environmental', MS='Carcinogenic substances that are found in the environment.'),
  'D002275': NtO(MH='Carcinoid Heart Disease', MS='Cardiac manifestation of gastrointestinal CARCINOID TUMOR that metastasizes to the liver. Substances secreted by the tumor cells, including SEROTONIN, promote fibrous plaque formation in ENDOCARDIUM and its underlying layers. These deposits cause distortion of the TRICUSPID VALVE and the PULMONARY VALVE eventually leading to STENOSIS and valve regurgitation.'),
  'D002276': NtO(MH='Carcinoid Tumor', MS='A usually small, slow-growing neoplasm composed of islands of rounded, oxyphilic, or spindle-shaped cells of medium size, with moderately small vesicular nuclei, and covered by intact mucosa with a yellow cut surface. The tumor can occur anywhere in the gastrointestinal tract (and in the lungs and other sites); approximately 90% arise in the appendix. It is now established that these tumors are of neuroendocrine origin and derive from a primitive stem cell. (From Stedman, 25th ed & Holland et al., Cancer Medicine, 3d ed, p1182)'),
  'D002277': NtO(MH='Carcinoma', MS='A malignant neoplasm made up of epithelial cells tending to infiltrate the surrounding tissues and give rise to metastases. It is a histological type of neoplasm and not a synonym for "cancer."'),
  'D002278': NtO(MH='Carcinoma in Situ', MS='A lesion with cytological characteristics associated with invasive carcinoma but the tumor cells are confined to the epithelium of origin, without invasion of the basement membrane.'),
  'D002279': NtO(MH='Carcinoma 256, Walker', MS='A transplantable carcinoma of the rat that originally appeared spontaneously in the mammary gland of a pregnant albino rat, and which now resembles a carcinoma in young transplants and a sarcoma in older transplants. (Stedman, 25th ed)'),
  'D002280': NtO(MH='Carcinoma, Basal Cell', MS='A malignant skin neoplasm that seldom metastasizes but has potentialities for local invasion and destruction. Clinically it is divided into types: nodular, cicatricial, morphaic, and erythematoid (pagetoid). They develop on hair-bearing skin, most commonly on sun-exposed areas. Approximately 85% are found on the head and neck area and the remaining 15% on the trunk and limbs. (From DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1471)'),
  'D002281': NtO(MH='Carcinoma, Basosquamous', MS='A skin carcinoma that histologically exhibits both basal and squamous elements. (From Dorland, 27th ed)'),
  'D002282': NtO(MH='Adenocarcinoma, Bronchiolo-Alveolar', MS='A carcinoma derived from epithelium of terminal bronchioles, in which the neoplastic tissue extends along the alveolar walls and grows in small masses within the alveoli. Involvement may be uniformly diffuse and massive, or nodular, or lobular. The neoplastic cells are cuboidal or columnar and form papillary structures. Mucin may be demonstrated in some of the cells and in the material in the alveoli, which also includes denuded cells. Metastases in regional lymph nodes, and in even more distant sites, are known to occur, but are infrequent. (From Stedman, 25th ed)'),
  'D002283': NtO(MH='Carcinoma, Bronchogenic', MS='Malignant neoplasm arising from the epithelium of the BRONCHI. It represents a large group of epithelial lung malignancies which can be divided into two clinical groups: SMALL CELL LUNG CANCER and NON-SMALL-CELL LUNG CARCINOMA.'),
  'D002284': NtO(MH='Carcinoma, Brown-Pearce', MS='A transplantable EPITHELIAL CELL neoplasm of rabbits.'),
  'D002285': NtO(MH='Carcinoma, Intraductal, Noninfiltrating', MS='A noninvasive (noninfiltrating) carcinoma of the breast characterized by a proliferation of malignant epithelial cells confined to the mammary ducts or lobules, without light-microscopy evidence of invasion through the basement membrane into the surrounding stroma.'),
  'D002286': NtO(MH='Carcinoma, Ehrlich Tumor', MS='A transplantable, poorly differentiated malignant tumor which appeared originally as a spontaneous breast carcinoma in a mouse. It grows in both solid and ascitic forms.'),
  'D002287': NtO(MH='Carcinoma, Krebs 2', MS='A transplantable neoplasm of mice.'),
  'D002288': NtO(MH='Adenocarcinoma, Mucinous', MS='An adenocarcinoma producing mucin in significant amounts. (From Dorland, 27th ed)'),
  'D002289': NtO(MH='Carcinoma, Non-Small-Cell Lung', MS='A heterogeneous aggregate of at least three distinct histological types of lung cancer, including SQUAMOUS CELL CARCINOMA; ADENOCARCINOMA; and LARGE CELL CARCINOMA. They are dealt with collectively because of their shared treatment strategy.'),
  'D002291': NtO(MH='Carcinoma, Papillary', MS='A malignant neoplasm characterized by the formation of numerous, irregular, finger-like projections of fibrous stroma that is covered with a surface layer of neoplastic epithelial cells. (Stedman, 25th ed)'),
  'D002292': NtO(MH='Carcinoma, Renal Cell', MS='A heterogeneous group of sporadic or hereditary carcinoma derived from cells of the KIDNEYS. There are several subtypes including the clear cells, the papillary, the chromophobe, the collecting duct, the spindle cells (sarcomatoid), or mixed cell-type carcinoma.'),
  'D002293': NtO(MH='Adenocarcinoma, Scirrhous', MS='An adenocarcinoma with a hard (Greek skirrhos, hard) structure owing to the formation of dense connective tissue in the stroma. (From Dorland, 27th ed)'),
  'D002294': NtO(MH='Carcinoma, Squamous Cell', MS='A carcinoma derived from stratified SQUAMOUS EPITHELIAL CELLS. It may also occur in sites where glandular or columnar epithelium is normally present. (From Stedman, 25th ed)'),
  'D002295': NtO(MH='Carcinoma, Transitional Cell', MS='A malignant neoplasm derived from TRANSITIONAL EPITHELIAL CELLS, occurring chiefly in the URINARY BLADDER; URETERS; or RENAL PELVIS.'),
  'D002296': NtO(MH='Carcinosarcoma', MS='A malignant neoplasm that contains elements of carcinoma and sarcoma so extensively intermixed as to indicate neoplasia of epithelial and mesenchymal tissue. (Stedman, 25th ed)'),
  'D002297': NtO(MH='Cardanolides', MS='The aglycone constituents of CARDIAC GLYCOSIDES. The ring structure is basically a cyclopentanoperhydrophenanthrene nucleus attached to a lactone ring at the C-17 position.'),
  'D002298': NtO(MH='Cardenolides', MS='C(23)-steroids with methyl groups at C-10 and C-13 and a five-membered lactone at C-17. They are aglycone constituents of CARDIAC GLYCOSIDES and must have at least one double bond in the molecule. The class includes cardadienolides and cardatrienolides. Members include DIGITOXIN and DIGOXIN and their derivatives and the STROPHANTHINS.'),
  'D002299': NtO(MH='Cardia', MS='That part of the STOMACH close to the opening from ESOPHAGUS into the stomach (cardiac orifice), the ESOPHAGOGASTRIC JUNCTION. The cardia is so named because of its closeness to the HEART. Cardia is characterized by the lack of acid-forming cells (GASTRIC PARIETAL CELLS).'),
  'D002300': NtO(MH='Cardiac Care Facilities', MS='Institutions specializing in the care of patients with heart disorders.'),
  'D002301': NtO(MH='Cardiac Glycosides', MS='Cyclopentanophenanthrenes with a 5- or 6-membered lactone ring attached at the 17-position and SUGARS attached at the 3-position. Plants they come from have long been used in congestive heart failure. They increase the force of cardiac contraction without significantly affecting other parameters, but are very toxic at larger doses. Their mechanism of action usually involves inhibition of the NA(+)-K(+)-EXCHANGING ATPASE and they are often used in cell biological studies for that purpose.'),
  'D002302': NtO(MH='Cardiac Output', MS='The volume of BLOOD passing through the HEART per unit of time. It is usually expressed as liters (volume) per minute so as not to be confused with STROKE VOLUME (volume per beat).'),
  'D002303': NtO(MH='Cardiac Output, Low', MS='A state of subnormal or depressed cardiac output at rest or during stress. It is a characteristic of CARDIOVASCULAR DISEASES, including congenital, valvular, rheumatic, hypertensive, coronary, and cardiomyopathic. The serious form of low cardiac output is characterized by marked reduction in STROKE VOLUME, and systemic vasoconstriction resulting in cold, pale, and sometimes cyanotic extremities.'),
  'D002304': NtO(MH='Cardiac Pacing, Artificial', MS='Regulation of the rate of contraction of the heart muscles by an artificial pacemaker.'),
  'D002305': NtO(MH='Cardiac Tamponade', MS='Compression of the heart by accumulated fluid (PERICARDIAL EFFUSION) or blood (HEMOPERICARDIUM) in the PERICARDIUM surrounding the heart. The affected cardiac functions and CARDIAC OUTPUT can range from minimal to total hemodynamic collapse.'),
  'D002306': NtO(MH='Cardiac Volume', MS='The volume of the HEART, usually relating to the volume of BLOOD contained within it at various periods of the cardiac cycle. The amount of blood ejected from a ventricle at each beat is STROKE VOLUME.'),
  'D002307': NtO(MH='Cardiography, Impedance', MS='A type of impedance plethysmography in which bioelectrical impedance is measured between electrodes positioned around the neck and around the lower thorax. It is used principally to calculate stroke volume and cardiac volume, but it is also related to myocardial contractility, thoracic fluid content, and circulation to the extremities.'),
  'D002308': NtO(MH='Cardiolipins', MS='Acidic phospholipids composed of two molecules of phosphatidic acid covalently linked to a molecule of glycerol. They occur primarily in mitochondrial inner membranes and in bacterial plasma membranes. They are the main antigenic components of the Wassermann-type antigen that is used in nontreponemal SYPHILIS SERODIAGNOSIS.'),
  'D002309': NtO(MH='Cardiology', MS='The study of the heart, its physiology, and its functions.'),
  'D002310': NtO(MH='Cardiomyopathy, Alcoholic', MS='Disease of CARDIAC MUSCLE resulting from chronic excessive alcohol consumption. Myocardial damage can be caused by: (1) a toxic effect of alcohol; (2) malnutrition in alcoholics such as THIAMINE DEFICIENCY; or (3) toxic effect of additives in alcoholic beverages such as COBALT. This disease is usually manifested by DYSPNEA and palpitations with CARDIOMEGALY and congestive heart failure (HEART FAILURE).'),
  'D002311': NtO(MH='Cardiomyopathy, Dilated', MS='A form of CARDIAC MUSCLE disease that is characterized by ventricular dilation, VENTRICULAR DYSFUNCTION, and HEART FAILURE. Risk factors include SMOKING; ALCOHOL DRINKING; HYPERTENSION; INFECTION; PREGNANCY; and mutations in the LMNA gene encoding LAMIN TYPE A, a NUCLEAR LAMINA protein.'),
  'D002312': NtO(MH='Cardiomyopathy, Hypertrophic', MS='A form of CARDIAC MUSCLE disease, characterized by left and/or right ventricular hypertrophy (HYPERTROPHY, LEFT VENTRICULAR; HYPERTROPHY, RIGHT VENTRICULAR), frequent asymmetrical involvement of the HEART SEPTUM, and normal or reduced left ventricular volume. Risk factors include HYPERTENSION; AORTIC STENOSIS; and gene MUTATION; (FAMILIAL HYPERTROPHIC CARDIOMYOPATHY).'),
  'D002313': NtO(MH='Cardiomyopathy, Restrictive', MS='A form of CARDIAC MUSCLE disease in which the ventricular walls are excessively rigid, impeding ventricular filling. It is marked by reduced diastolic volume of either or both ventricles but normal or nearly normal systolic function. It may be idiopathic or associated with other diseases (ENDOMYOCARDIAL FIBROSIS or AMYLOIDOSIS) causing interstitial fibrosis.'),
  'D002314': NtO(MH='Cardioplegic Solutions', MS='Solutions which, upon administration, will temporarily arrest cardiac activity. They are used in the performance of heart surgery.'),
  'D002315': NtO(MH='Cardiopulmonary Bypass', MS='Diversion of the flow of blood from the entrance of the right atrium directly to the aorta (or femoral artery) via an oxygenator thus bypassing both the heart and lungs.'),
  'D002316': NtO(MH='Cardiotonic Agents', MS='Agents that have a strengthening effect on the heart or that can increase cardiac output. They may be CARDIAC GLYCOSIDES; SYMPATHOMIMETICS; or other drugs. They are used after MYOCARDIAL INFARCT; CARDIAC SURGICAL PROCEDURES; in SHOCK; or in congestive heart failure (HEART FAILURE).'),
  'D002317': NtO(MH='Cardiovascular Agents', MS='Agents that affect the rate or intensity of cardiac contraction, blood vessel diameter, or blood volume.'),
  'D002318': NtO(MH='Cardiovascular Diseases', MS='Pathological conditions involving the CARDIOVASCULAR SYSTEM including the HEART; the BLOOD VESSELS; or the PERICARDIUM.'),
  'D002319': NtO(MH='Cardiovascular System', MS='The HEART and the BLOOD VESSELS by which BLOOD is pumped and circulated through the body.'),
  'D002320': NtO(MH='Cardiovascular Physiological Phenomena', MS='Processes and properties of the CARDIOVASCULAR SYSTEM as a whole or of any of its parts.'),
  'D002321': NtO(MH='Career Choice', MS='Selection of a type of occupation or profession.'),
  'D002322': NtO(MH='Career Mobility', MS='The upward or downward mobility in an occupation or the change from one occupation to another.'),
  'D002323': NtO(MH='Carfecillin', MS='The phenyl ester of CARBENICILLIN that, upon oral administration, is broken down in the intestinal mucosa to the active antibacterial. It is used for urinary tract infections.'),
  'D002325': NtO(MH='Caricatures as Topic', MS='Works about portraying in a critical or facetious way a real individual or group, or a figure representing a social, political, ethnic, or racial type. (Genre Terms: A Thesaurus for Use in Rare Book and Special Collections Cataloguing, 2d ed)'),
  'D002326': NtO(MH='Cariogenic Agents', MS='Substances that promote DENTAL CARIES.'),
  'D002327': NtO(MH='Cariostatic Agents', MS="Substances that inhibit or arrest DENTAL CARIES formation. (Boucher's Clinical Dental Terminology, 4th ed)"),
  'D002328': NtO(MH='Carisoprodol', MS='A centrally acting skeletal muscle relaxant whose mechanism of action is not completely understood but may be related to its sedative actions. It is used as an adjunct in the symptomatic treatment of musculoskeletal conditions associated with painful muscle spasm. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1202)'),
  'D002329': NtO(MH='Carmine', MS='Coloring matter from the insect Coccus cacti L. It is used in foods, pharmaceuticals, toiletries, etc., as a dye, and also has use as a microscopic stain and biological marker.'),
  'D002330': NtO(MH='Carmustine', MS='A cell-cycle phase nonspecific alkylating antineoplastic agent. It is used in the treatment of brain tumors and various other malignant neoplasms. (From Martindale, The Extra Pharmacopoeia, 30th ed, p462) This substance may reasonably be anticipated to be a carcinogen according to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985). (From Merck Index, 11th ed)'),
  'D002331': NtO(MH='Carnitine', MS='A constituent of STRIATED MUSCLE and LIVER. It is an amino acid derivative and an essential cofactor for fatty acid metabolism.'),
  'D002332': NtO(MH='Carnitine O-Acetyltransferase', MS='An enzyme that catalyzes the formation of O-acetylcarnitine from acetyl-CoA plus carnitine. EC 2.3.1.7.'),
  'D002333': NtO(MH='Carnitine Acyltransferases', MS='Acyltransferases in the inner mitochondrial membrane that catalyze the reversible transfer of acyl groups from acyl-CoA to L-carnitine and thereby mediate the transport of activated fatty acids through that membrane. EC 2.3.1.'),
  'D002334': NtO(MH='Carnitine O-Palmitoyltransferase', MS='An enzyme that catalyzes reversibly the conversion of palmitoyl-CoA to palmitoylcarnitine in the inner mitochondrial membrane. EC 2.3.1.21.'),
  'D002335': NtO(MH='Carnivora', MS='An order of MAMMALS, usually flesh eaters with appropriate dentition. Suborders include the terrestrial carnivores Fissipedia, and the aquatic carnivores CANIFORMIA.'),
  'D002336': NtO(MH='Carnosine', MS='A naturally occurring dipeptide neuropeptide found in muscles.'),
  'D002338': NtO(MH='Carotenoids', MS='The general name for a group of fat-soluble pigments found in green, yellow, and leafy vegetables, and yellow fruits. They are aliphatic hydrocarbons containing 4 terpene subunits.'),
  'D002339': NtO(MH='Carotid Arteries', MS='Either of the two principal arteries on both sides of the neck that supply blood to the head and neck; each divides into two branches, the internal carotid artery and the external carotid artery.'),
  'D002340': NtO(MH='Carotid Artery Diseases', MS='Pathological conditions involving the CAROTID ARTERIES, including the common, internal, and external carotid arteries. ATHEROSCLEROSIS and TRAUMA are relatively frequent causes of carotid artery pathology.'),
  'D002341': NtO(MH='Carotid Artery Thrombosis', MS='Blood clot formation in any part of the CAROTID ARTERIES. This may produce CAROTID STENOSIS or occlusion of the vessel, leading to TRANSIENT ISCHEMIC ATTACK; CEREBRAL INFARCTION; or AMAUROSIS FUGAX.'),
  'D002342': NtO(MH='Carotid Artery, External', MS='Branch of the common carotid artery which supplies the exterior of the head, the face, and the greater part of the neck.'),
  'D002343': NtO(MH='Carotid Artery, Internal', MS='Branch of the common carotid artery which supplies the anterior part of the brain, the eye and its appendages, the forehead and nose.'),
  'D002344': NtO(MH='Carotid Body', MS='A small cluster of chemoreceptive and supporting cells located near the bifurcation of the internal carotid artery. The carotid body, which is richly supplied with fenestrated capillaries, senses the pH, carbon dioxide, and oxygen concentrations in the blood and plays a crucial role in their homeostatic control.'),
  'D002345': NtO(MH='Carotid Body Tumor', MS='Benign paraganglioma at the bifurcation of the COMMON CAROTID ARTERIES. It can encroach on the parapharyngeal space and produce dysphagia, pain, and cranial nerve palsies.'),
  'D002346': NtO(MH='Carotid Sinus', MS='The dilated portion of the common carotid artery at its bifurcation into external and internal carotids. It contains baroreceptors which, when stimulated, cause slowing of the heart, vasodilatation, and a fall in blood pressure.'),
  'D002347': NtO(MH='Carps', MS='Common name for a number of different species of fish in the family Cyprinidae. This includes, among others, the common carp, crucian carp, grass carp, and silver carp.'),
  'D002348': NtO(MH='Carpal Bones', MS='The eight bones of the wrist: SCAPHOID BONE; LUNATE BONE; TRIQUETRUM BONE; PISIFORM BONE; TRAPEZIUM BONE; TRAPEZOID BONE; CAPITATE BONE; and HAMATE BONE.'),
  'D002349': NtO(MH='Carpal Tunnel Syndrome', MS='Entrapment of the MEDIAN NERVE in the carpal tunnel, which is formed by the flexor retinaculum and the CARPAL BONES. This syndrome may be associated with repetitive occupational trauma (CUMULATIVE TRAUMA DISORDERS); wrist injuries; AMYLOID NEUROPATHIES; rheumatoid arthritis (see ARTHRITIS, RHEUMATOID); ACROMEGALY; PREGNANCY; and other conditions. Symptoms include burning pain and paresthesias involving the ventral surface of the hand and fingers which may radiate proximally. Impairment of sensation in the distribution of the median nerve and thenar muscle atrophy may occur. (Joynt, Clinical Neurology, 1995, Ch51, p45)'),
  'D002350': NtO(MH='Carpus, Animal', MS='The region corresponding to the human WRIST in non-human ANIMALS.'),
  'D002351': NtO(MH='Carrageenan', MS='A water-soluble extractive mixture of sulfated polysaccharides from RED ALGAE. Chief sources are the Irish moss CHONDRUS CRISPUS (Carrageen), and Gigartina stellata. It is used as a stabilizer, for suspending COCOA in chocolate manufacture, and to clarify BEVERAGES.'),
  'D002352': NtO(MH='Carrier Proteins', MS='Proteins that bind or transport specific substances in the blood, within the cell, or across cell membranes.'),
  'D002353': NtO(MH='Carrier State', MS='The condition of harboring an infective organism without manifesting symptoms of infection. The organism must be readily transmissible to another susceptible host.'),
  'D002354': NtO(MH='Carteolol', MS='A beta-adrenergic antagonist used as an anti-arrhythmia agent, an anti-angina agent, an antihypertensive agent, and an antiglaucoma agent.'),
  'D002355': NtO(MH='Carticaine', MS='A thiophene-containing local anesthetic pharmacologically similar to MEPIVACAINE.'),
  'D002356': NtO(MH='Cartilage', MS='A non-vascular form of connective tissue composed of CHONDROCYTES embedded in a matrix that includes CHONDROITIN SULFATE and various types of FIBRILLAR COLLAGEN. There are three major types: HYALINE CARTILAGE; FIBROCARTILAGE; and ELASTIC CARTILAGE.'),
  'D002357': NtO(MH='Cartilage Diseases', MS='Pathological processes involving the chondral tissue (CARTILAGE).'),
  'D002358': NtO(MH='Cartilage, Articular', MS='A protective layer of firm, flexible cartilage over the articulating ends of bones. It provides a smooth surface for joint movement, protecting the ends of long bones from wear at points of contact.'),
  'D002359': NtO(MH='Cartoons as Topic', MS='Works about images used to comment on such things as contemporary events, social habits, or political trends; usually executed in a broad or abbreviated manner.'),
  'D002360': NtO(MH='Carubicin', MS='A very toxic anthracycline-type antineoplastic related to DAUNORUBICIN, obtained from Actinomadura carminata.'),
  'D002363': NtO(MH='Case Reports', MS='Clinical presentations that may be followed by evaluative studies that eventually lead to a diagnosis.'),
  'D002364': NtO(MH='Caseins', MS='A mixture of related phosphoproteins occurring in milk and cheese. The group is characterized as one of the most nutritive milk proteins, containing all of the common amino acids and rich in the essential ones.'),
  'D002365': NtO(MH='Manihot', MS='A plant genus of the family EUPHORBIACEAE that is perennial with conspicuous, almost palmate leaves like those of RICINUS but more deeply parted into five to nine lobes. It is a source of a starch after removal of the cyanogenic glucosides. The common name of Arrowroot is also used with Maranta (MARANTACEAE). The common name of yuca is also used for YUCCA.'),
  'D002366': NtO(MH='Cassia', MS='A plant genus of the family FABACEAE. Many species of this genus, including the medicinal C. senna and C. angustifolia, have been reclassified into the Senna genus (SENNA PLANT) and some to CHAMAECRISTA.'),
  'D002367': NtO(MH='Castor Bean', MS='Common name for Ricinus communis, a species in the family EUPHORBIACEAE. It is the source of CASTOR OIL.'),
  'D002368': NtO(MH='Castor Oil', MS='Oil obtained from seeds of Ricinus communis that is used as a cathartic and as a plasticizer.'),
  'D002369': NtO(MH='Castration', MS='Surgical removal or artificial destruction of gonads.'),
  'D002370': NtO(MH='Casts, Surgical', MS='Dressings made of fiberglass, plastic, or bandage impregnated with plaster of paris used for immobilization of various parts of the body in cases of fractures, dislocations, and infected wounds. In comparison with plaster casts, casts made of fiberglass or plastic are lightweight, radiolucent, able to withstand moisture, and less rigid.'),
  'D002371': NtO(MH='Cat Diseases', MS='Diseases of the domestic cat (Felis catus or F. domesticus). This term does not include diseases of the so-called big cats such as CHEETAHS; LIONS; tigers, cougars, panthers, leopards, and other Felidae for which the heading CARNIVORA is used.'),
  'D002372': NtO(MH='Cat-Scratch Disease', MS='A self-limiting bacterial infection of the regional lymph nodes caused by AFIPIA felis, a gram-negative bacterium recently identified by the Centers for Disease Control and Prevention and by BARTONELLA HENSELAE. It usually arises one or more weeks following a feline scratch, with raised inflammatory nodules at the site of the scratch being the primary symptom.'),
  'D002373': NtO(MH='Cyclic AMP Receptor Protein', MS='A transcriptional regulator in prokaryotes which, when activated by binding cyclic AMP, acts at several promoters. Cyclic AMP receptor protein was originally identified as a catabolite gene activator protein. It was subsequently shown to regulate several functions unrelated to catabolism, and to be both a negative and a positive regulator of transcription. Cell surface cyclic AMP receptors are not included (CYCLIC AMP RECEPTORS), nor are the eukaryotic cytoplasmic cyclic AMP receptor proteins, which are the regulatory subunits of CYCLIC AMP-DEPENDENT PROTEIN KINASES.'),
  'D002374': NtO(MH='Catalase', MS='An oxidoreductase that catalyzes the conversion of HYDROGEN PEROXIDE to water and oxygen. It is present in many animal cells. A deficiency of this enzyme results in ACATALASIA.'),
  'D002375': NtO(MH='Catalepsy', MS='A condition characterized by inactivity, decreased responsiveness to stimuli, and a tendency to maintain an immobile posture. The limbs tend to remain in whatever position they are placed (waxy flexibility). Catalepsy may be associated with PSYCHOTIC DISORDERS (e.g., SCHIZOPHRENIA, CATATONIC), nervous system drug toxicity, and other conditions.'),
  'D002376': NtO(MH='Cataloging', MS='Activities performed in the preparation of bibliographic records for CATALOG. It is carried out according to a set of rules and contains information enabling the user to know what is available and where items can be found.'),
  'D002377': NtO(MH='Catalogs as Topic', MS='Works about ordered compilations of item descriptions and sufficient information to afford access to them.'),
  'D002378': NtO(MH='Catalog, Bookseller', MS='Lists of books for sale by a specific seller.'),
  'D002379': NtO(MH='Catalogs, Commercial as Topic', MS='Works about lists of merchandise for sale.'),
  'D002380': NtO(MH='Catalogs, Drug as Topic', MS='Works about lists of drugs for sale.'),
  'D002381': NtO(MH='Catalogs, Library', MS='Lists of material available  in a library arranged in order with bibliographical METADATA for the purpose of identification and retrieval.'),
  'D002382': NtO(MH='Catalog, Publisher', MS='Lists of books or other titles produced by a specific entity.'),
  'D002383': NtO(MH='Catalogs, Union as Topic', MS='Works about lists of books and other information sources held among a cooperating group of LIBRARIES.'),
  'D002384': NtO(MH='Catalysis', MS='The facilitation of a chemical reaction by material (catalyst) that is not consumed by the reaction.'),
  'D002385': NtO(MH='Cataplexy', MS='A condition characterized by transient weakness or paralysis of somatic musculature triggered by an emotional stimulus or physical exertion. Cataplexy is frequently associated with NARCOLEPSY. During a cataplectic attack, there is a marked reduction in muscle tone similar to the normal physiologic hypotonia that accompanies rapid eye movement sleep (SLEEP, REM). (From Adams et al., Principles of Neurology, 6th ed, p396)'),
  'D002386': NtO(MH='Cataract', MS='Partial or complete opacity on or in the lens or capsule of one or both eyes, impairing vision or causing blindness. The many kinds of cataract are classified by their morphology (size, shape, location) or etiology (cause and time of occurrence). (Dorland, 27th ed)'),
  'D002387': NtO(MH='Cataract Extraction', MS='The removal of a cataractous CRYSTALLINE LENS from the eye.'),
  'D002388': NtO(MH='Catastrophic Illness', MS='An acute or prolonged illness usually considered to be life-threatening or with the threat of serious residual disability. Treatment may be radical and is frequently costly.'),
  'D002389': NtO(MH='Catatonia', MS='A neuropsychiatric disorder characterized by one or more of the following essential features: immobility, mutism, negativism (active or passive refusal to follow commands), mannerisms, stereotypies, posturing, grimacing, excitement, echolalia, echopraxia, muscular rigidity, and stupor; sometimes punctuated by sudden violent outbursts, panic, or hallucinations. This condition may be associated with psychiatric illnesses (e.g., SCHIZOPHRENIA; MOOD DISORDERS) or organic disorders (NEUROLEPTIC MALIGNANT SYNDROME; ENCEPHALITIS, etc.). (From DSM-IV, 4th ed, 1994; APA, Thesaurus of Psychological Index Terms, 1994)'),
  'D002391': NtO(MH='Catchment Area, Health', MS='A geographic area defined and served by a health program or institution.'),
  'D002392': NtO(MH='Catechin', MS='An antioxidant flavonoid, occurring especially in woody plants as both (+)-catechin and (-)-epicatechin (cis) forms.'),
  'D002393': NtO(MH='Estrogens, Catechol', MS='2- or 4-Hydroxyestrogens. Substances that are physiologically active in mammals, especially in the control of gonadotropin secretion. Physiological activity can be ascribed to either an estrogenic action or interaction with the catecholaminergic system.'),
  'D002394': NtO(MH='Catechol O-Methyltransferase', MS='Enzyme that catalyzes the movement of a methyl group from S-adenosylmethionone to a catechol or a catecholamine.'),
  'D002395': NtO(MH='Catecholamines', MS='A general class of ortho-dihydroxyphenylalkylamines derived from TYROSINE.'),
  'D002396': NtO(MH='Catechols', MS='A group of 1,2-benzenediols that contain the general formula R-C6H5O2.'),
  'D002397': NtO(MH='Catfishes', MS='Common name of the order Siluriformes. This order contains many families and over 2,000 species, including venomous species. Heteropneustes and Plotosus genera have dangerous stings and are aggressive. Most species are passive stingers.'),
  'D002398': NtO(MH='Catgut', MS='Sterile collagen strands obtained from healthy mammals. They are used as absorbable surgical ligatures and are frequently impregnated with chromium or silver for increased strength. They tend to cause tissue reaction.'),
  'D002399': NtO(MH='Catharsis', MS='The release of ideas, thoughts, and repressed material from the unconscious, accompanied by an emotional response and relief. (From Dorland, 28th ed)'),
  'D002400': NtO(MH='Cathartics', MS='Agents that are used to stimulate evacuation of the bowels.'),
  'D002401': NtO(MH='Cathepsin B', MS='A lysosomal cysteine proteinase with a specificity similar to that of PAPAIN. The enzyme is present in a variety of tissues and is important in many physiological and pathological processes. In pathology, cathepsin B has been found to be involved in DEMYELINATION; EMPHYSEMA; RHEUMATOID ARTHRITIS, and NEOPLASM INVASIVENESS.'),
  'D002402': NtO(MH='Cathepsin D', MS='An intracellular proteinase found in a variety of tissue. It has specificity similar to but narrower than that of pepsin A. The enzyme is involved in catabolism of cartilage and connective tissue. EC 3.4.23.5. (Formerly EC 3.4.4.23).'),
  'D002403': NtO(MH='Cathepsins', MS='A group of lysosomal proteinases or endopeptidases found in aqueous extracts of a variety of animal tissues. They function optimally within an acidic pH range. The cathepsins occur as a variety of enzyme subtypes including SERINE PROTEASES; ASPARTIC PROTEINASES; and CYSTEINE PROTEASES.'),
  'D002404': NtO(MH='Catheterization', MS='Use or insertion of a tubular device into a duct, blood vessel, hollow organ, or body cavity for injecting or withdrawing fluids for diagnostic or therapeutic purposes. It differs from INTUBATION in that the tube here is used to restore or maintain patency in obstructions.'),
  'D002405': NtO(MH='Catheterization, Central Venous', MS='Placement of an intravenous CATHETER in the subclavian, jugular, or other central vein.'),
  'D002406': NtO(MH='Catheterization, Peripheral', MS='Insertion of a catheter into a peripheral artery, vein, or airway for diagnostic or therapeutic purposes.'),
  'D002407': NtO(MH='Catheterization, Swan-Ganz', MS='Placement of a balloon-tipped catheter into the pulmonary artery through the antecubital, subclavian, and sometimes the femoral vein. It is used to measure pulmonary artery pressure and pulmonary artery wedge pressure which reflects left atrial pressure and left ventricular end-diastolic pressure. The catheter is threaded into the right atrium, the balloon is inflated and the catheter follows the blood flow through the tricuspid valve into the right ventricle and out into the pulmonary artery.'),
  'D002408': NtO(MH='Catheters, Indwelling', MS='Catheters designed to be left within an organ or passage for an extended period of time.'),
  'D002409': NtO(MH='Cathexis', MS='Attachment, conscious or unconscious, of emotional feeling and significance to an idea, object or most commonly a person.'),
  'D002410': NtO(MH='Catholicism', MS='The Christian faith, practice, or system of the Catholic Church, specifically the Roman Catholic, the Christian church that is characterized by a hierarchic structure of bishops and priests in which doctrinal and disciplinary authority are dependent upon apostolic succession, with the pope as head of the episcopal college. (From Webster, 3d ed; American Heritage Dictionary, 2d college ed)'),
  'D002411': NtO(MH='Cation Exchange Resins', MS='High molecular weight insoluble polymers which contain functional anionic groups that are capable of undergoing exchange reactions with cations.'),
  'D002412': NtO(MH='Cations', MS='Positively charged atoms, radicals or groups of atoms which travel to the cathode or negative pole during electrolysis.'),
  'D002413': NtO(MH='Cations, Divalent', MS='Positively charged atoms, radicals or groups of atoms with a valence of plus 2, which travel to the cathode or negative pole during electrolysis.'),
  'D002414': NtO(MH='Cations, Monovalent', MS='Positively charged atoms, radicals or group of atoms with a valence of plus 1, which travel to the cathode or negative pole during electrolysis.'),
  'D002415': NtO(MH='Cats', MS="The domestic cat, Felis catus, of the carnivore family FELIDAE, comprising over 30 different breeds. The domestic cat is descended primarily from the wild cat of Africa and extreme southwestern Asia. Though probably present in towns in Palestine as long ago as 7000 years, actual domestication occurred in Egypt about 4000 years ago. (From Walker's Mammals of the World, 6th ed, p801)"),
  'D002416': NtO(MH='Cattell Personality Factor Questionnaire', MS='Self report questionnaire which yields 16 scores on personality traits, such as reserved vs. outgoing, humble vs. assertive, etc.'),
  'D002417': NtO(MH='Cattle', MS='Domesticated bovine animals of the genus Bos, usually kept on a farm or ranch and used for the production of meat or dairy products or for heavy labor.'),
  'D002418': NtO(MH='Cattle Diseases', MS='Diseases of domestic cattle of the genus Bos. It includes diseases of cows, yaks, and zebus.'),
  'D002420': NtO(MH='Cauda Equina', MS='The lower part of the SPINAL CORD consisting of the lumbar, sacral, and coccygeal nerve roots.'),
  'D002421': NtO(MH='Caudate Nucleus', MS='Elongated gray mass of the neostriatum located adjacent to the lateral ventricle of the brain.'),
  'D002422': NtO(MH='Causalgia', MS='A complex regional pain syndrome characterized by burning pain and marked sensitivity to touch (HYPERESTHESIA) in the distribution of an injured peripheral nerve. Autonomic dysfunction in the form of sudomotor (i.e., sympathetic innervation to sweat glands), vasomotor, and trophic skin changes may also occur. (Adams et al., Principles of Neurology, 6th ed, p1359)'),
  'D002423': NtO(MH='Cause of Death', MS='Factors which produce cessation of all vital bodily functions. They can be analyzed from an epidemiologic viewpoint.'),
  'D002424': NtO(MH='Caustics', MS='Strong alkaline chemicals that destroy soft body tissues resulting in a deep, penetrating type of burn, in contrast to corrosives, that result in a more superficial type of damage via chemical means or inflammation. Caustics are usually hydroxides of light metals. SODIUM HYDROXIDE and potassium hydroxide are the most widely used caustic agents in industry. Medically, they have been used externally to remove diseased or dead tissues and destroy warts and small tumors. The accidental ingestion of products (household and industrial) containing caustic ingredients results in thousands of injuries per year.'),
  'D002425': NtO(MH='Cautery', MS='The application of a caustic substance, a hot instrument, an electric current, or other agent to control bleeding while removing or destroying tissue.'),
  'D002426': NtO(MH='Cavernous Sinus', MS='An irregularly shaped venous space in the dura mater at either side of the sphenoid bone.'),
  'D002427': NtO(MH='Cebidae', MS='A family of New World monkeys in the infraorder PLATYRRHINI, consisting of nine subfamilies: ALOUATTINAE; AOTINAE; Atelinae; Callicebinae; CALLIMICONINAE; CALLITRICHINAE; CEBINAE; Pithecinae; and SAIMIRINAE. They inhabit the forests of South and Central America, comprising the largest family of South American monkeys.'),
  'D002428': NtO(MH='Cebus', MS='A genus of the family CEBIDAE, subfamily CEBINAE, consisting of gracile or untufted capuchin species. Tufted capuchins belong to genus SAPAJUS. Members include C. capucinus, C. nigrivultatus, and C. albifrons. Cebus inhabits the forests of Central and South Americas.'),
  'D002429': NtO(MH='Cecal Diseases', MS='Pathological developments in the CECUM.'),
  'D002430': NtO(MH='Cecal Neoplasms', MS='Tumors or cancer of the CECUM.'),
  'D002431': NtO(MH='Cecostomy', MS='Surgical construction of an opening into the CECUM with a tube through the ABDOMINAL WALL (tube cecostomy) or by skin level approach, in which the cecum is sewn to the surrounding PERITONEUM. Its primary purpose is decompression of colonic obstruction.'),
  'D002432': NtO(MH='Cecum', MS='The blind sac or outpouching area of the LARGE INTESTINE that is below the entrance of the SMALL INTESTINE. It has a worm-like extension, the vermiform APPENDIX.'),
  'D002433': NtO(MH='Cefaclor', MS='Semisynthetic, broad-spectrum antibiotic derivative of CEPHALEXIN.'),
  'D002434': NtO(MH='Cefadroxil', MS='Long-acting, broad-spectrum, water-soluble, CEPHALEXIN derivative.'),
  'D002435': NtO(MH='Cefamandole', MS='Semisynthetic wide-spectrum cephalosporin with prolonged action, probably due to beta-lactamase resistance. It is used also as the nafate.'),
  'D002436': NtO(MH='Cefatrizine', MS='Orally active semisynthetic cephalosporin antibiotic with broad-spectrum activity.'),
  'D002437': NtO(MH='Cefazolin', MS='A semisynthetic cephalosporin analog with broad-spectrum antibiotic action due to inhibition of bacterial cell wall synthesis. It attains high serum levels and is excreted quickly via the urine.'),
  'D002438': NtO(MH='Cefoperazone', MS='Semisynthetic broad-spectrum cephalosporin with a tetrazolyl moiety that is resistant to beta-lactamase. It may be used to treat Pseudomonas infections.'),
  'D002439': NtO(MH='Cefotaxime', MS='Semisynthetic broad-spectrum cephalosporin.'),
  'D002440': NtO(MH='Cefoxitin', MS='A semisynthetic cephamycin antibiotic resistant to beta-lactamase.'),
  'D002441': NtO(MH='Cefsulodin', MS='A pyridinium-substituted semisynthetic, broad-spectrum antibacterial used especially for Pseudomonas infections in debilitated patients.'),
  'D002442': NtO(MH='Ceftazidime', MS='Semisynthetic, broad-spectrum antibacterial derived from CEPHALORIDINE and used especially for Pseudomonas and other gram-negative infections in debilitated patients.'),
  'D002443': NtO(MH='Ceftriaxone', MS='A broad-spectrum cephalosporin antibiotic and cefotaxime derivative with a very long half-life and high penetrability to meninges, eyes and inner ears.'),
  'D002444': NtO(MH='Cefuroxime', MS='Broad-spectrum cephalosporin antibiotic resistant to beta-lactamase. It has been proposed for infections with gram-negative and gram-positive organisms, GONORRHEA, and HAEMOPHILUS.'),
  'D002445': NtO(MH='Celiac Artery', MS='The arterial trunk that arises from the abdominal aorta and after a short course divides into the left gastric, common hepatic and splenic arteries.'),
  'D002446': NtO(MH='Celiac Disease', MS='A malabsorption syndrome that is precipitated by the ingestion of foods containing GLUTEN, such as wheat, rye, and barley. It is characterized by INFLAMMATION of the SMALL INTESTINE, loss of MICROVILLI structure, failed INTESTINAL ABSORPTION, and MALNUTRITION.'),
  'D002447': NtO(MH='Celiac Plexus', MS='A complex network of nerve fibers including sympathetic and parasympathetic efferents and visceral afferents. The celiac plexus is the largest of the autonomic plexuses and is located in the abdomen surrounding the celiac and superior mesenteric arteries.'),
  'D002448': NtO(MH='Cell Adhesion', MS='Adherence of cells to surfaces or to other cells.'),
  'D002449': NtO(MH='Cell Aggregation', MS='The phenomenon by which dissociated cells intermixed in vitro tend to group themselves with cells of their own type.'),
  'D002450': NtO(MH='Cell Communication', MS='Any of several ways in which living cells of an organism communicate with one another, whether by direct contact between cells or by means of chemical signals carried by neurotransmitter substances, hormones, and cyclic AMP.'),
  'D002451': NtO(MH='Cell Compartmentation', MS='A partitioning within cells due to the selectively permeable membranes which enclose each of the separate parts, e.g., mitochondria, lysosomes, etc.'),
  'D002452': NtO(MH='Cell Count', MS='The number of CELLS of a specific kind, usually measured per unit volume or area of sample.'),
  'D002453': NtO(MH='Cell Cycle', MS='The complex series of phenomena, occurring between the end of one CELL DIVISION and the end of the next, by which cellular material is duplicated and then divided between two daughter cells. The cell cycle includes INTERPHASE, which includes G0 PHASE; G1 PHASE; S PHASE; and G2 PHASE, and CELL DIVISION PHASE.'),
  'D002454': NtO(MH='Cell Differentiation', MS='Progressive restriction of the developmental potential and increasing specialization of function that leads to the formation of specialized cells, tissues, and organs.'),
  'D002455': NtO(MH='Cell Division', MS='The fission of a CELL. It includes CYTOKINESIS, when the CYTOPLASM of a cell is divided, and CELL NUCLEUS DIVISION.'),
  'D002457': NtO(MH='Cell Extracts', MS='Preparations of cell constituents or subcellular materials, isolates, or substances.'),
  'D002458': NtO(MH='Cell Fractionation', MS='Techniques to partition various components of the cell into SUBCELLULAR FRACTIONS.'),
  'D002459': NtO(MH='Cell Fusion', MS='Fusion of somatic cells in vitro or in vivo, which results in somatic cell hybridization.'),
  'D002460': NtO(MH='Cell Line', MS='Established cell cultures that have the potential to propagate indefinitely.'),
  'D002461': NtO(MH='Cell Line, Transformed', MS='Eukaryotic cell line obtained in a quiescent or stationary phase which undergoes conversion to a state of unregulated growth in culture, resembling an in vitro tumor. It occurs spontaneously or through interaction with viruses, oncogenes, radiation, or drugs/chemicals.'),
  'D002462': NtO(MH='Cell Membrane', MS='The lipid- and protein-containing, selectively permeable membrane that surrounds the cytoplasm in prokaryotic and eukaryotic cells.'),
  'D002463': NtO(MH='Cell Membrane Permeability', MS='A quality of cell membranes which permits the passage of solvents and solutes into and out of cells.'),
  'D002464': NtO(MH='Cell Migration Inhibition', MS='Phenomenon of cell-mediated immunity measured by in vitro inhibition of the migration or phagocytosis of antigen-stimulated LEUKOCYTES or MACROPHAGES. Specific CELL MIGRATION ASSAYS have been developed to estimate levels of migration inhibitory factors, immune reactivity against tumor-associated antigens, and immunosuppressive effects of infectious microorganisms.'),
  'D002465': NtO(MH='Cell Movement', MS='The movement of cells from one location to another. Distinguish from CYTOKINESIS which is the process of dividing the CYTOPLASM of a cell.'),
  'D002466': NtO(MH='Cell Nucleolus', MS='Within most types of eukaryotic CELL NUCLEUS, a distinct region, not delimited by a membrane, in which some species of rRNA (RNA, RIBOSOMAL) are synthesized and assembled into ribonucleoprotein subunits of ribosomes. In the nucleolus rRNA is transcribed from a nucleolar organizer, i.e., a group of tandemly repeated chromosomal genes which encode rRNA and which are transcribed by RNA polymerase I. (Singleton & Sainsbury, Dictionary of Microbiology & Molecular Biology, 2d ed)'),
  'D002467': NtO(MH='Cell Nucleus', MS='Within a eukaryotic cell, a membrane-limited body which contains chromosomes and one or more nucleoli (CELL NUCLEOLUS). The nuclear membrane consists of a double unit-type membrane which is perforated by a number of pores; the outermost membrane is continuous with the ENDOPLASMIC RETICULUM. A cell may contain more than one nucleus. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D002468': NtO(MH='Cell Physiological Phenomena', MS='Cellular processes, properties, and characteristics.'),
  'D002469': NtO(MH='Cell Separation', MS='Techniques for separating distinct populations of cells.'),
  'D002470': NtO(MH='Cell Survival', MS='The span of viability of a cell characterized by the capacity to perform certain functions such as metabolism, growth, reproduction, some form of responsiveness, and adaptability.'),
  'D002471': NtO(MH='Cell Transformation, Neoplastic', MS='Cell changes manifested by escape from control mechanisms, increased growth potential, alterations in the cell surface, karyotypic abnormalities, morphological and biochemical deviations from the norm, and other attributes conferring the ability to invade, metastasize, and kill.'),
  'D002472': NtO(MH='Cell Transformation, Viral', MS='An inheritable change in cells manifested by changes in cell division and growth and alterations in cell surface properties. It is induced by infection with a transforming virus.'),
  'D002473': NtO(MH='Cell Wall', MS='The outermost layer of a cell in most PLANTS; BACTERIA; FUNGI; and ALGAE. The cell wall is usually a rigid structure that lies external to the CELL MEMBRANE, and provides a protective barrier against physical or chemical agents.'),
  'D002474': NtO(MH='Cell-Free System', MS='A fractionated cell extract that maintains a biological function. A subcellular fraction isolated by ultracentrifugation or other separation techniques must first be isolated so that a process can be studied free from all of the complex side reactions that occur in a cell. The cell-free system is therefore widely used in cell biology. (From Alberts et al., Molecular Biology of the Cell, 2d ed, p166)'),
  'D002475': NtO(MH='Cellobiose', MS='A disaccharide consisting of two glucose units in beta (1-4) glycosidic linkage. Obtained from the partial hydrolysis of cellulose.'),
  'D002476': NtO(MH='Cellophane', MS="A generic name for film produced from wood pulp by the viscose process. It is a thin, transparent sheeting of regenerated cellulose, moisture-proof and sometimes dyed, and used chiefly as food wrapping or as bags for dialysis. (Grant & Hackh's Chemical Dictionary, 5th ed & McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)"),
  'D002477': NtO(MH='Cells', MS='The fundamental, structural, and functional units or subunits of living organisms. They are composed of CYTOPLASM containing various ORGANELLES and a CELL MEMBRANE boundary.'),
  'D002478': NtO(MH='Cells, Cultured', MS='Cells propagated in vitro in special media conducive to their growth. Cultured cells are used to study developmental, morphologic, metabolic, physiologic, and genetic processes, among others.'),
  'D002479': NtO(MH='Inclusion Bodies', MS='A generic term for any circumscribed mass of foreign (e.g., lead or viruses) or metabolically inactive materials (e.g., ceroid or MALLORY BODIES), within the cytoplasm or nucleus of a cell. Inclusion bodies are in cells infected with certain filtrable viruses, observed especially in nerve, epithelial, or endothelial cells. (Stedman, 25th ed)'),
  'D002480': NtO(MH='Cellulase', MS='An endocellulase with specificity for the hydrolysis of 1,4-beta-glucosidic linkages in CELLULOSE, lichenin, and cereal beta-glucans.'),
  'D002481': NtO(MH='Cellulitis', MS='An acute, diffuse, and suppurative inflammation of loose connective tissue, particularly the deep subcutaneous tissues, and sometimes muscle, which is most commonly seen as a result of infection of a wound, ulcer, or other skin lesions.'),
  'D002482': NtO(MH='Cellulose', MS='A polysaccharide with glucose units linked as in CELLOBIOSE. It is the chief constituent of plant fibers, cotton being the purest natural form of the substance. As a raw material, it forms the basis for many derivatives used in chromatography, ion exchange materials, explosives manufacturing, and pharmaceutical preparations.'),
  'D002483': NtO(MH='Cellulose, Oxidized', MS='A cellulose of varied carboxyl content retaining the fibrous structure. It is commonly used as a local hemostatic and as a matrix for normal blood coagulation.'),
  'D002484': NtO(MH='Cementation', MS='The joining of objects by means of a cement (e.g., in fracture fixation, such as in hip arthroplasty for joining of the acetabular component to the femoral component). In dentistry, it is used for the process of attaching parts of a tooth or restorative material to a natural tooth or for the attaching of orthodontic bands to teeth by means of an adhesive.'),
  'D002485': NtO(MH='Cementoma', MS='An odontogenic fibroma in which cells have developed into cementoblasts and which consists largely of cementum.'),
  'D002486': NtO(MH='Centchroman', MS='A non-steroidal anti-fertility agent with anti-hormonal properties.'),
  'D002487': NtO(MH='Centers for Disease Control and Prevention, U.S.', MS='An agency of the UNITED STATES PUBLIC HEALTH SERVICE that conducts and supports programs for the prevention and control of disease and provides consultation and assistance to health departments and other countries.'),
  'D002488': NtO(MH='Central African Republic', MS='A republic in central Africa south of CHAD and SUDAN, north of DEMOCRATIC REPUBLIC OF THE CONGO, and east of CAMEROON. The capital is Bangui.'),
  'D002489': NtO(MH='Central America', MS='A region of southern NORTH AMERICA extending from the southern border of MEXICO to the northern border of COLOMBIA. It separates the Caribbean Sea from the Pacific Ocean and is linked to SOUTH AMERICA by the Isthmus of Panama.'),
  'D002490': NtO(MH='Central Nervous System', MS='The main information-processing organs of the nervous system, consisting of the brain, spinal cord, and meninges.'),
  'D002491': NtO(MH='Central Nervous System Agents', MS='A class of drugs producing both physiological and psychological effects through a variety of mechanisms. They can be divided into "specific" agents, e.g., affecting an identifiable molecular mechanism unique to target cells bearing receptors for that agent, and "nonspecific" agents, those producing effects on different target cells and acting by diverse molecular mechanisms. Those with nonspecific mechanisms are generally further classed according to whether they produce behavioral depression or stimulation. Those with specific mechanisms are classed by locus of action or specific therapeutic use. (From Gilman AG, et al., Goodman and Gilman\'s The Pharmacological Basis of Therapeutics, 8th ed, p252)'),
  'D002492': NtO(MH='Central Nervous System Depressants', MS='A very loosely defined group of drugs that tend to reduce the activity of the central nervous system. The major groups included here are ethyl alcohol, anesthetics, hypnotics and sedatives, narcotics, and tranquilizing agents (antipsychotics and antianxiety agents).'),
  'D002493': NtO(MH='Central Nervous System Diseases', MS='Diseases of any component of the brain (including the cerebral hemispheres, diencephalon, brain stem, and cerebellum) or the spinal cord.'),
  'D002494': NtO(MH='Central Nervous System Infections', MS='Pathogenic infections of the brain, spinal cord, and meninges. DNA VIRUS INFECTIONS; RNA VIRUS INFECTIONS; BACTERIAL INFECTIONS; MYCOPLASMA INFECTIONS; SPIROCHAETALES INFECTIONS; fungal infections; PROTOZOAN INFECTIONS; HELMINTHIASIS; and PRION DISEASES may involve the central nervous system as a primary or secondary process.'),
  'D002495': NtO(MH='Central Supply, Hospital', MS='Hospital department responsible for receiving, storing, and distributing medical and surgical supplies and equipment.'),
  'D002496': NtO(MH='Central Venous Pressure', MS='The blood pressure in the central large VEINS of the body. It is distinguished from peripheral venous pressure which occurs in an extremity.'),
  'D002497': NtO(MH='Centralized Hospital Services', MS='The coordination of services in one area of a facility to improve efficiency.'),
  'D002498': NtO(MH='Centrifugation', MS='Process of using a rotating machine to generate centrifugal force to separate substances of different densities, remove moisture, or simulate gravitational effects. It employs a large motor-driven apparatus with a long arm, at the end of which human and animal subjects, biological specimens, or equipment can be revolved and rotated at various speeds to study gravitational effects. (From Websters, 10th ed; McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002499': NtO(MH='Centrifugation, Density Gradient', MS='Separation of particles according to density by employing a gradient of varying densities. At equilibrium each particle settles in the gradient at a point equal to its density. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002500': NtO(MH='Centrifugation, Isopycnic', MS='A technique used to separate particles according to their densities in a continuous density gradient. The sample is usually mixed with a solution of known gradient materials and subjected to centrifugation. Each particle sediments to the position at which the gradient density is equal to its own. The range of the density gradient is usually greater than that of the sample particles. It is used in purifying biological materials such as proteins, nucleic acids, organelles, and cell types.'),
  'D002501': NtO(MH='Centrifugation, Zonal', MS='Centrifugation using a rotating chamber of large capacity in which to separate cell organelles by density-gradient centrifugation. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002502': NtO(MH='Centrioles', MS='Self-replicating, short, fibrous, rod-shaped organelles. Each centriole is a short cylinder containing nine pairs of peripheral microtubules, arranged so as to form the wall of the cylinder.'),
  'D002503': NtO(MH='Centromere', MS='The clear constricted portion of the chromosome at which the chromatids are joined and by which the chromosome is attached to the spindle during cell division.'),
  'D002504': NtO(MH='Meclofenoxate', MS='An ester of DIMETHYLAMINOETHANOL and para-chlorophenoxyacetic acid.'),
  'D002505': NtO(MH='Cephacetrile', MS='A derivative of 7-aminocephalosporanic acid.'),
  'D002506': NtO(MH='Cephalexin', MS='A semisynthetic cephalosporin antibiotic with antimicrobial activity similar to that of CEPHALORIDINE or CEPHALOTHIN, but somewhat less potent. It is effective against both gram-positive and gram-negative organisms.'),
  'D002507': NtO(MH='Cephaloglycin', MS='A cephalorsporin antibiotic.'),
  'D002508': NtO(MH='Cephalometry', MS='The measurement of the dimensions of the HEAD.'),
  'D002509': NtO(MH='Cephaloridine', MS='A cephalosporin antibiotic.'),
  'D002510': NtO(MH='Cephalosporinase', MS=None),
  'D002511': NtO(MH='Cephalosporins', MS='A group of broad-spectrum antibiotics first isolated from the Mediterranean fungus ACREMONIUM. They contain the beta-lactam moiety thia-azabicyclo-octenecarboxylic acid also called 7-aminocephalosporanic acid.'),
  'D002512': NtO(MH='Cephalothin', MS='A cephalosporin antibiotic.'),
  'D002513': NtO(MH='Cephamycins', MS='Naturally occurring family of beta-lactam cephalosporin-type antibiotics having a 7-methoxy group and possessing marked resistance to the action of beta-lactamases from gram-positive and gram-negative organisms.'),
  'D002514': NtO(MH='Cephapirin', MS='Cephalosporin antibiotic, partly plasma-bound, that is effective against gram-negative and gram-positive organisms.'),
  'D002515': NtO(MH='Cephradine', MS='A semi-synthetic cephalosporin antibiotic.'),
  'D002516': NtO(MH='Ceramics', MS="Products made by baking or firing nonmetallic minerals (clay and similar materials). In making dental restorations or parts of restorations the material is fused porcelain. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed & Boucher's Clinical Dental Terminology, 4th ed)"),
  'D002517': NtO(MH='Galactosylgalactosylglucosylceramidase', MS='An enzyme that catalyzes the hydrolysis of a ceramidetrihexoside to a ceramidedihexoside plus galactose.'),
  'D002518': NtO(MH='Ceramides', MS='Members of the class of neutral glycosphingolipids. They are the basic units of SPHINGOLIPIDS. They are sphingoids attached via their amino groups to a long chain fatty acyl group. They abnormally accumulate in FABRY DISEASE.'),
  'D002519': NtO(MH='Ceratopogonidae', MS='A family of biting midges, in the order DIPTERA. It includes the genus Culicoides which transmits filarial parasites pathogenic to man and other primates.'),
  'D002520': NtO(MH='Cercopithecidae', MS='The family of Old World monkeys and baboons consisting of two subfamilies: CERCOPITHECINAE and COLOBINAE. They are found in Africa and part of Asia.'),
  'D002521': NtO(MH='Cercopithecus', MS='A genus of Old World monkeys found in Africa although some species have been introduced into the West Indies. This genus is composed of at least twenty species: C. AETHIOPS, C. ascanius, C. campbelli, C. cephus, C. denti, C. diana, C. dryas, C. erythrogaster, C. erythrotis, C. hamlyni, C. lhoesti, C. mitis, C. mona, C. neglectus, C. nictitans, C. petaurista, C. pogonias, C. preussi, C. salongo, and C. wolfi.'),
  'D002522': NtO(MH='Chlorocebus aethiops', MS='A species of CERCOPITHECUS containing three subspecies: C. tantalus, C. pygerythrus, and C. sabeus. They are found in the forests and savannah of Africa. The African green monkey (C. pygerythrus) is the natural host of SIMIAN IMMUNODEFICIENCY VIRUS and is used in AIDS research.'),
  'D002523': NtO(MH='Edible Grain', MS='SEEDS used as a major nutritional source, most often the grain from the POACEAE family.'),
  'D002524': NtO(MH='Cerebellar Ataxia', MS='Incoordination of voluntary movements that occur as a manifestation of CEREBELLAR DISEASES. Characteristic features include a tendency for limb movements to overshoot or undershoot a target (dysmetria), a tremor that occurs during attempted movements (intention TREMOR), impaired force and rhythm of diadochokinesis (rapidly alternating movements), and GAIT ATAXIA. (From Adams et al., Principles of Neurology, 6th ed, p90)'),
  'D002525': NtO(MH='Cerebellar Cortex', MS='The superficial GRAY MATTER of the CEREBELLUM. It consists of two main layers, the stratum moleculare and the stratum granulosum.'),
  'D002526': NtO(MH='Cerebellar Diseases', MS='Diseases that affect the structure or function of the cerebellum. Cardinal manifestations of cerebellar dysfunction include dysmetria, GAIT ATAXIA, and MUSCLE HYPOTONIA.'),
  'D002527': NtO(MH='Myoclonic Cerebellar Dyssynergia', MS='A condition marked by progressive CEREBELLAR ATAXIA combined with MYOCLONUS usually presenting in the third decade of life or later. Additional clinical features may include generalized and focal SEIZURES, spasticity, and DYSKINESIAS. Autosomal recessive and autosomal dominant patterns of inheritance have been reported. Pathologically, the dentate nucleus and brachium conjunctivum of the CEREBELLUM are atrophic, with variable involvement of the spinal cord, cerebellar cortex, and basal ganglia. (From Joynt, Clinical Neurology, 1991, Ch37, pp60-1)'),
  'D002528': NtO(MH='Cerebellar Neoplasms', MS='Primary or metastatic neoplasms of the CEREBELLUM. Tumors in this location frequently present with ATAXIA or signs of INTRACRANIAL HYPERTENSION due to obstruction of the fourth ventricle. Common primary cerebellar tumors include fibrillary ASTROCYTOMA and cerebellar HEMANGIOBLASTOMA. The cerebellum is a relatively common site for tumor metastases from the lung, breast, and other distant organs. (From Okazaki & Scheithauer, Atlas of Neuropathology, 1988, p86 and p141)'),
  'D002529': NtO(MH='Cerebellar Nuclei', MS='Four clusters of neurons located deep within the WHITE MATTER of the CEREBELLUM, which are the nucleus dentatus, nucleus emboliformis, nucleus globosus, and nucleus fastigii.'),
  'D002530': NtO(MH='Cerebellopontine Angle', MS='Junction between the cerebellum and the pons.'),
  'D002531': NtO(MH='Cerebellum', MS='The part of brain that lies behind the BRAIN STEM in the posterior base of skull (CRANIAL FOSSA, POSTERIOR). It is also known as the "little brain" with convolutions similar to those of CEREBRAL CORTEX, inner white matter, and deep cerebellar nuclei. Its function is to coordinate voluntary movements, maintain balance, and learn motor skills.'),
  'D002532': NtO(MH='Intracranial Aneurysm', MS='Abnormal outpouching in the wall of intracranial blood vessels. Most common are the saccular (berry) aneurysms located at branch points in CIRCLE OF WILLIS at the base of the brain. Vessel rupture results in SUBARACHNOID HEMORRHAGE or INTRACRANIAL HEMORRHAGES. Giant aneurysms (>2.5 cm in diameter) may compress adjacent structures, including the OCULOMOTOR NERVE. (From Adams et al., Principles of Neurology, 6th ed, p841)'),
  'D002533': NtO(MH='Cerebral Angiography', MS='Radiography of the vascular system of the brain after injection of a contrast medium.'),
  'D002534': NtO(MH='Hypoxia, Brain', MS='A reduction in brain oxygen supply due to ANOXEMIA (a reduced amount of oxygen being carried in the blood by HEMOGLOBIN), or to a restriction of the blood supply to the brain, or both. Severe hypoxia is referred to as anoxia, and is a relatively common cause of injury to the central nervous system. Prolonged brain anoxia may lead to BRAIN DEATH or a PERSISTENT VEGETATIVE STATE. Histologically, this condition is characterized by neuronal loss which is most prominent in the HIPPOCAMPUS; GLOBUS PALLIDUS; CEREBELLUM; and inferior olives.'),
  'D002535': NtO(MH='Cerebral Aqueduct', MS='Narrow channel in the MESENCEPHALON that connects the third and fourth CEREBRAL VENTRICLES.'),
  'D002536': NtO(MH='Cerebral Arteries', MS='The arterial blood vessels supplying the CEREBRUM.'),
  'D002537': NtO(MH='Intracranial Arteriosclerosis', MS="Vascular diseases characterized by thickening and hardening of the walls of ARTERIES inside the SKULL. There are three subtypes: (1) atherosclerosis with fatty deposits in the ARTERIAL INTIMA; (2) Monckeberg's sclerosis with calcium deposits in the media and (3) arteriolosclerosis involving the small caliber arteries. Clinical signs include HEADACHE; CONFUSION; transient blindness (AMAUROSIS FUGAX); speech impairment; and HEMIPARESIS."),
  'D002538': NtO(MH='Intracranial Arteriovenous Malformations', MS='Congenital vascular anomalies in the brain characterized by direct communication between an artery and a vein without passing through the CAPILLARIES. The locations and size of the shunts determine the symptoms including HEADACHES; SEIZURES; STROKE; INTRACRANIAL HEMORRHAGES; mass effect; and vascular steal effect.'),
  'D002539': NtO(MH='Cerebral Arterial Diseases', MS='Pathological conditions of intracranial ARTERIES supplying the CEREBRUM. These diseases often are due to abnormalities or pathological processes in the ANTERIOR CEREBRAL ARTERY; MIDDLE CEREBRAL ARTERY; and POSTERIOR CEREBRAL ARTERY.'),
  'D002540': NtO(MH='Cerebral Cortex', MS='The thin layer of GRAY MATTER on the surface of the CEREBRAL HEMISPHERES that develops from the TELENCEPHALON and folds into gyri and sulchi. It reaches its highest development in humans and is responsible for intellectual faculties and higher mental functions.'),
  'D002541': NtO(MH='Cerebral Decortication', MS='Partial or total removal, ablation, or destruction of the cerebral cortex; may be chemical. It is not used with animals that do not possess a cortex, i.e., it is used only with mammals.'),
  'D002542': NtO(MH='Intracranial Embolism and Thrombosis', MS='Embolism or thrombosis involving blood vessels which supply intracranial structures. Emboli may originate from extracranial or intracranial sources. Thrombosis may occur in arterial or venous structures.'),
  'D002543': NtO(MH='Cerebral Hemorrhage', MS='Bleeding into one or both CEREBRAL HEMISPHERES including the BASAL GANGLIA and the CEREBRAL CORTEX. It is often associated with HYPERTENSION and CRANIOCEREBRAL TRAUMA.'),
  'D002544': NtO(MH='Cerebral Infarction', MS='The formation of an area of NECROSIS in the CEREBRUM caused by an insufficiency of arterial or venous blood flow. Infarcts of the cerebrum are generally classified by hemisphere (i.e., left vs. right), lobe (e.g., frontal lobe infarction), arterial distribution (e.g., INFARCTION, ANTERIOR CEREBRAL ARTERY), and etiology (e.g., embolic infarction).'),
  'D002545': NtO(MH='Brain Ischemia', MS='Localized reduction of blood flow to brain tissue due to arterial obstruction or systemic hypoperfusion. This frequently occurs in conjunction with brain hypoxia (HYPOXIA, BRAIN). Prolonged ischemia is associated with BRAIN INFARCTION.'),
  'D002546': NtO(MH='Ischemic Attack, Transient', MS='Brief reversible episodes of focal, nonconvulsive ischemic dysfunction of the brain having a duration of less than 24 hours, and usually less than one hour, caused by transient thrombotic or embolic blood vessel occlusion or stenosis. Events may be classified by arterial distribution, temporal pattern, or etiology (e.g., embolic vs. thrombotic). (From Adams et al., Principles of Neurology, 6th ed, pp814-6)'),
  'D002547': NtO(MH='Cerebral Palsy', MS='A heterogeneous group of nonprogressive motor disorders caused by chronic brain injuries that originate in the prenatal period, perinatal period, or first few years of life. The four major subtypes are spastic, athetoid, ataxic, and mixed cerebral palsy, with spastic forms being the most common. The motor disorder may range from difficulties with fine motor control to severe spasticity (see MUSCLE SPASTICITY) in all limbs. Spastic diplegia (Little disease) is the most common subtype, and is characterized by spasticity that is more prominent in the legs than in the arms. Pathologically, this condition may be associated with LEUKOMALACIA, PERIVENTRICULAR. (From Dev Med Child Neurol 1998 Aug;40(8):520-7)'),
  'D002548': NtO(MH='Cerebral Revascularization', MS='Microsurgical revascularization to improve intracranial circulation. It usually involves joining the extracranial circulation to the intracranial circulation but may include extracranial revascularization (e.g., subclavian-vertebral artery bypass, subclavian-external carotid artery bypass). It is performed by joining two arteries (direct anastomosis or use of graft) or by free autologous transplantation of highly vascularized tissue to the surface of the brain.'),
  'D002549': NtO(MH='Diffuse Cerebral Sclerosis of Schilder', MS="A rare central nervous system demyelinating condition affecting children and young adults. Pathologic findings include a large, sharply defined, asymmetric focus of myelin destruction that may involve an entire lobe or cerebral hemisphere. The clinical course tends to be progressive and includes dementia, cortical blindness, cortical deafness, spastic hemiplegia, and pseudobulbar palsy. Concentric sclerosis of Balo is differentiated from diffuse cerebral sclerosis of Schilder by the pathologic finding of alternating bands of destruction and preservation of myelin in concentric rings. Alpers' Syndrome refers to a heterogeneous group of diseases that feature progressive cerebral deterioration and liver disease. (From Adams et al., Principles of Neurology, 6th ed, p914; Dev Neurosci 1991;13(4-5):267-73)"),
  'D002550': NtO(MH='Cerebral Veins', MS='Veins draining the cerebrum.'),
  'D002551': NtO(MH='Cerebral Ventricle Neoplasms', MS='Neoplasms located in the brain ventricles, including the two lateral, the third, and the fourth ventricle. Ventricular tumors may be primary (e.g., CHOROID PLEXUS NEOPLASMS and GLIOMA, SUBEPENDYMAL), metastasize from distant organs, or occur as extensions of locally invasive tumors from adjacent brain structures.'),
  'D002552': NtO(MH='Cerebral Ventricles', MS='Four CSF-filled (see CEREBROSPINAL FLUID) cavities within the cerebral hemispheres (LATERAL VENTRICLES), in the midline (THIRD VENTRICLE) and within the PONS and MEDULLA OBLONGATA (FOURTH VENTRICLE).'),
  'D002553': NtO(MH='Cerebroside-Sulfatase', MS='An enzyme that catalyzes the hydrolysis of cerebroside 3-sulfate (sulfatide) to yield a cerebroside and inorganic sulfate. A marked deficiency of arylsulfatase A, which is considered the heat-labile component of cerebroside sulfatase, has been demonstrated in all forms of metachromatic leukodystrophy (LEUKODYSTROPHY, METACHROMATIC). EC 3.1.6.8.'),
  'D002554': NtO(MH='Cerebrosides', MS='Neutral glycosphingolipids that contain a monosaccharide, normally glucose or galactose, in 1-ortho-beta-glycosidic linkage with the primary alcohol of an N-acyl sphingoid (ceramide). In plants the monosaccharide is normally glucose and the sphingoid usually phytosphingosine. In animals, the monosaccharide is usually galactose, though this may vary with the tissue and the sphingoid is usually sphingosine or dihydrosphingosine. (From Oxford Dictionary of Biochemistry and Molecular Biology, 1st ed)'),
  'D002555': NtO(MH='Cerebrospinal Fluid', MS='A watery fluid that is continuously produced in the CHOROID PLEXUS and circulates around the surface of the BRAIN; SPINAL CORD; and in the CEREBRAL VENTRICLES.'),
  'D002556': NtO(MH='Cerebrospinal Fluid Proteins', MS="Proteins in the cerebrospinal fluid, normally albumin and globulin present in the ratio of 8 to 1. Increases in protein levels are of diagnostic value in neurological diseases. (Brain and Bannister's Clinical Neurology, 7th ed, p221)"),
  'D002557': NtO(MH='Cerebrospinal Fluid Shunts', MS='Tubes inserted to create communication between a cerebral ventricle and the internal jugular vein. Their emplacement permits draining of cerebrospinal fluid for relief of hydrocephalus or other condition leading to fluid accumulation in the ventricles.'),
  'D002558': NtO(MH='Cerebrospinal Fluid Otorrhea', MS='Discharge of cerebrospinal fluid through the external auditory meatus or through the eustachian tube into the nasopharynx. This is usually associated with CRANIOCEREBRAL TRAUMA (e.g., SKULL FRACTURE involving the TEMPORAL BONE;), NEUROSURGICAL PROCEDURES; or other conditions, but may rarely occur spontaneously. (From Am J Otol 1995 Nov;16(6):765-71)'),
  'D002559': NtO(MH='Cerebrospinal Fluid Rhinorrhea', MS='Discharge of cerebrospinal fluid through the nose. Common etiologies include trauma, neoplasms, and prior surgery, although the condition may occur spontaneously. (Otolaryngol Head Neck Surg 1997 Apr;116(4):442-9)'),
  'D002560': NtO(MH='Cerebrovascular Circulation', MS='The circulation of blood through the BLOOD VESSELS of the BRAIN.'),
  'D002561': NtO(MH='Cerebrovascular Disorders', MS='A spectrum of pathological conditions of impaired blood flow in the brain. They can involve vessels (ARTERIES or VEINS) in the CEREBRUM, the CEREBELLUM, and the BRAIN STEM. Major categories include INTRACRANIAL ARTERIOVENOUS MALFORMATIONS; BRAIN ISCHEMIA; CEREBRAL HEMORRHAGE; and others.'),
  'D002562': NtO(MH='Ceremonial Behavior', MS='A series of actions, sometimes symbolic actions which may be associated with a behavior pattern, and are often indispensable to its performance.'),
  'D002563': NtO(MH='Cerium', MS='An element of the rare earth family of metals. It has the atomic symbol Ce, atomic number 58, and atomic weight 140.12. Cerium is a malleable metal used in industrial applications.'),
  'D002564': NtO(MH='Cerium Isotopes', MS='Stable cerium atoms that have the same atomic number as the element cerium, but differ in atomic weight. Ce-136, 138, and 142 are stable cerium isotopes.'),
  'D002565': NtO(MH='Cerium Radioisotopes', MS='Unstable isotopes of cerium that decay or disintegrate emitting radiation. Ce atoms with atomic weights 132-135, 137, 139, and 141-148 are radioactive cerium isotopes.'),
  'D002566': NtO(MH='Ceroid', MS='A naturally occurring lipid pigment with histochemical characteristics similar to lipofuscin. It accumulates in various tissues in certain experimental and pathological conditions.'),
  'D002567': NtO(MH='Certificate of Need', MS='A certificate issued by a governmental body to an individual or organization proposing to construct or modify a health facility, or to offer a new or different service. The process of issuing the certificate is also included.'),
  'D002568': NtO(MH='Certification', MS='Compliance with a set of standards defined by non-governmental organizations. Certification is applied for by individuals on a voluntary basis and represents a professional status when achieved, e.g., certification for a medical specialty.'),
  'D002569': NtO(MH='Cerulenin', MS='An epoxydodecadienamide isolated from several species, including ACREMONIUM, Acrocylindrum, and Helicoceras. It inhibits the biosynthesis of several lipids by interfering with enzyme function.'),
  'D002570': NtO(MH='Ceruloplasmin', MS='A multi-copper blood FERROXIDASE involved in iron and copper homeostasis and inflammation.'),
  'D002571': NtO(MH='Cerumen', MS='The yellow or brown waxy secretions produced by vestigial apocrine sweat glands in the external ear canal.'),
  'D002572': NtO(MH='Cervical Plexus', MS='A network of nerve fibers originating in the upper four CERVICAL SPINAL CORD segments. The cervical plexus distributes cutaneous nerves to parts of the neck, shoulders, and back of the head. It also distributes motor fibers to muscles of the cervical SPINAL COLUMN, infrahyoid muscles, and the DIAPHRAGM.'),
  'D002573': NtO(MH='Cervical Rib Syndrome', MS='A condition associated with compression of the BRACHIAL PLEXUS; SUBCLAVIAN ARTERY; and SUBCLAVIAN VEIN at the thoracic outlet and caused by a complete or incomplete anomalous CERVICAL RIB or fascial band connecting the tip of a cervical rib with the first thoracic rib. Clinical manifestations may include pain in the neck and shoulder which radiates into the upper extremity, PARESIS or PARALYSIS of brachial plexus innervated muscles; sensory loss; PARESTHESIAS; ISCHEMIA; and EDEMA. (Adams et al., Principles of Neurology, 6th ed, p214)'),
  'D002574': NtO(MH='Cervical Vertebrae', MS='The first seven VERTEBRAE of the SPINAL COLUMN, which correspond to the VERTEBRAE of the NECK.'),
  'D002575': NtO(MH='Uterine Cervicitis', MS='Inflammation of the UTERINE CERVIX.'),
  'D002577': NtO(MH='Uterine Cervical Diseases', MS='Pathological processes of the UTERINE CERVIX.'),
  'D002578': NtO(MH='Uterine Cervical Dysplasia', MS='Abnormal development of immature squamous EPITHELIAL CELLS of the UTERINE CERVIX, a term used to describe premalignant cytological changes in the cervical EPITHELIUM. These atypical cells do not penetrate the epithelial BASEMENT MEMBRANE.'),
  'D002579': NtO(MH='Uterine Cervical Erosion', MS='Loss or destruction of the epithelial lining of the UTERINE CERVIX.'),
  'D002581': NtO(MH='Uterine Cervical Incompetence', MS='Incompetent UTERINE CERVIX is usually diagnosed in the second trimester of PREGNANCY. It is characterized by passive painless cervical dilation in the absence of UTERINE CONTRACTION; BLEEDING; INFECTION; and sometimes with the amniotic sac (AMNIOTIC MEMBRANE) bulging through the partially dilated cervix. Left untreated, this condition may lead to premature pregnancy loss, such as HABITUAL ABORTION.'),
  'D002582': NtO(MH='Cervix Mucus', MS='A slightly alkaline secretion of the endocervical glands. The consistency and amount are dependent on the physiological hormone changes in the menstrual cycle. It contains the glycoprotein mucin, amino acids, sugar, enzymes, and electrolytes, with a water content up to 90%. The mucus is a useful protection against the ascent of bacteria and sperm into the uterus. (From Dictionary of Obstetrics and Gynecology, 1988)'),
  'D002583': NtO(MH='Uterine Cervical Neoplasms', MS='Tumors or cancer of the UTERINE CERVIX.'),
  'D002584': NtO(MH='Cervix Uteri', MS='The neck portion of the UTERUS between the lower isthmus and the VAGINA forming the cervical canal.'),
  'D002585': NtO(MH='Cesarean Section', MS='Extraction of the FETUS by means of abdominal HYSTEROTOMY.'),
  'D002586': NtO(MH='Cesium', MS='A member of the alkali metals. It has an atomic symbol Cs, atomic number 50, and atomic weight 132.91. Cesium has many industrial applications, including the construction of atomic clocks based on its atomic vibrational frequency.'),
  'D002587': NtO(MH='Cesium Isotopes', MS='Stable cesium atoms that have the same atomic number as the element cesium, but differ in atomic weight. Cs-133 is a naturally occurring isotope.'),
  'D002588': NtO(MH='Cesium Radioisotopes', MS='Unstable isotopes of cesium that decay or disintegrate emitting radiation. Cs atoms with atomic weights of 123, 125-132, and 134-145 are radioactive cesium isotopes.'),
  'D002589': NtO(MH='Cestoda', MS='A subclass of segmented worms comprising the tapeworms.'),
  'D002590': NtO(MH='Cestode Infections', MS='Infections with true tapeworms of the helminth subclass CESTODA.'),
  'D002591': NtO(MH='Cetacea', MS="An order of wholly aquatic MAMMALS occurring in all the OCEANS and adjoining seas of the world, as well as in certain river systems. They feed generally on FISHES, cephalopods, and crustaceans. Most are gregarious and most have a relatively long period of parental care and maturation. Included are DOLPHINS; PORPOISES; and WHALES. (From Walker's Mammals of the World, 5th ed, pp969-70)"),
  'D002592': NtO(MH='Cetomacrogol', MS='Non-ionic surfactant of the polyethylene glycol family. It is used as a solubilizer and emulsifying agent in foods, cosmetics, and pharmaceuticals, often as an ointment base, and also as a research tool.'),
  'D002593': NtO(MH='Cetrimonium Compounds', MS='Cetyltrimethylammonium compounds that have cationic detergent, antiseptic, and disinfectant activities. They are used in pharmaceuticals, foods, and cosmetics as preservatives; on skin, mucous membranes, etc., as antiseptics or cleansers, and also as emulsifiers. These compounds are toxic when used orally due to neuromuscular blockade.'),
  'D002594': NtO(MH='Cetylpyridinium', MS='Cationic bactericidal surfactant used as a topical antiseptic for skin, wounds, mucous membranes, instruments, etc.; and also as a component in mouthwash and lozenges.'),
  'D002595': NtO(MH='Cevanes', MS='Structurally-related alkaloids that contain the cevane carbon backbone.'),
  'D002596': NtO(MH='Chad', MS="A republic in central Africa, east of NIGER, west of SUDAN and south of LIBYA. Its capital is N'Djamena."),
  'D002597': NtO(MH='Chaetomium', MS='A genus of ascomycetous fungi of the family Chaetomiaceae, order SORDARIALES. Many members are cellulolytic and some mycotoxic. They occur naturally on paper and cotton fabric.'),
  'D002598': NtO(MH='Chagas Cardiomyopathy', MS='A disease of the CARDIAC MUSCLE developed subsequent to the initial protozoan infection by TRYPANOSOMA CRUZI. After infection, less than 10% develop acute illness such as MYOCARDITIS (mostly in children). The disease then enters a latent phase without clinical symptoms until about 20 years later. Myocardial symptoms of advanced CHAGAS DISEASE include conduction defects (HEART BLOCK) and CARDIOMEGALY.'),
  'D002599': NtO(MH='Chalcone', MS='An aromatic KETONE that forms the core molecule of CHALCONES.'),
  'D002601': NtO(MH='Chancre', MS='The primary sore of syphilis, a painless indurated, eroded papule, occurring at the site of entry of the infection.'),
  'D002602': NtO(MH='Chancroid', MS='Acute, localized autoinoculable infectious disease usually acquired through sexual contact. Caused by HAEMOPHILUS DUCREYI, it occurs endemically almost worldwide, especially in tropical and subtropical countries and more commonly in seaports and urban areas than in rural areas.'),
  'D002603': NtO(MH='Channel Islands', MS="A group of four British islands and several islets in the English Channel off the coast of France. They are known to have been occupied prehistorically. They were a part of Normandy in 933 but were united to the British crown at the time of the Norman Conquest in 1066. Guernsey and Jersey originated noted breeds of cattle. (From Webster's New Geographical Dictionary, 1988, p242)"),
  'D002604': NtO(MH='Chaplaincy Service, Hospital', MS='Hospital department which administers religious activities within the hospital, e.g., pastoral care, religious services.'),
  'D002605': NtO(MH='Character', MS='In current usage, approximately equivalent to personality. The sum of the relatively fixed personality traits and habitual modes of response of an individual.'),
  'D002606': NtO(MH='Charcoal', MS="An amorphous form of carbon prepared from the incomplete combustion of animal or vegetable matter, e.g., wood. The activated form of charcoal is used in the treatment of poisoning. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D002607': NtO(MH='Charcot-Marie-Tooth Disease', MS='A hereditary motor and sensory neuropathy transmitted most often as an autosomal dominant trait and characterized by progressive distal wasting and loss of reflexes in the muscles of the legs (and occasionally involving the arms). Onset is usually in the second to fourth decade of life. This condition has been divided into two subtypes, hereditary motor and sensory neuropathy (HMSN) types I and II. HMSN I is associated with abnormal nerve conduction velocities and nerve hypertrophy, features not seen in HMSN II. (Adams et al., Principles of Neurology, 6th ed, p1343)'),
  'D002608': NtO(MH='Charities', MS='Social welfare organizations with programs designed to assist individuals in need.'),
  'D002609': NtO(MH='Chediak-Higashi Syndrome', MS='A form of phagocyte bactericidal dysfunction characterized by unusual oculocutaneous albinism, high incidence of lymphoreticular neoplasms, and recurrent pyogenic infections. In many cell types, abnormal lysosomes are present leading to defective pigment distribution and abnormal neutrophil functions. The disease is transmitted by autosomal recessive inheritance and a similar disorder occurs in the beige mouse, the Aleutian mink, and albino Hereford cattle.'),
  'D002610': NtO(MH='Cheek', MS='The part of the face that is below the eye and to the side of the nose and mouth.'),
  'D002611': NtO(MH='Cheese', MS='A nutritious food consisting primarily of the curd or the semisolid substance formed when milk coagulates.'),
  'D002612': NtO(MH='Acinonyx', MS='A genus of long-legged, swift-moving felines (FELIDAE) from Africa (and formerly Asia) about the size of a small leopard.'),
  'D002613': NtO(MH='Cheilitis', MS='Inflammation of the lips. It is of various etiologies and degrees of pathology.'),
  'D002614': NtO(MH='Chelating Agents', MS='Chemicals that bind to and remove ions from solutions. Many chelating agents function through the formation of COORDINATION COMPLEXES with METALS.'),
  'D002615': NtO(MH='Chemexfoliation', MS='Application of a cauterant to the skin for the purpose of causing a superficial destruction of the epidermis and upper layers of the dermis. After healing, the treated area has new epithelium.'),
  'D002616': NtO(MH='Chemical Engineering', MS='Application of principles and practices of engineering science to the transformation, design, and manufacture of substances on an industrial scale.'),
  'D002617': NtO(MH='Chemical Industry', MS='The aggregate enterprise of manufacturing and technically producing chemicals. (From Random House Unabridged Dictionary, 2d ed)'),
  'D002618': NtO(MH='Chemical Warfare', MS='Tactical warfare using incendiary mixtures, smokes, or irritant, burning, or asphyxiating gases.'),
  'D002619': NtO(MH='Chemical Warfare Agents', MS='Chemicals that are used to cause the disturbance, disease, or death of humans during WARFARE.'),
  'D002620': NtO(MH='Pharmacological and Toxicological Phenomena', MS='Synthesis, metabolism, and poisoning associated with drugs, pharmaceuticals, and other active molecules. Toxicological phenomena also include the adverse effects of physical phenomena such as RADIATION and NOISE.'),
  'D002621': NtO(MH='Chemistry', MS='A basic science concerned with the composition, structure, and properties of matter; and the reactions that occur between substances and the associated energy exchange.'),
  'D002622': NtO(MH='Chemistry, Agricultural', MS='The science of the chemical composition and reactions of chemicals involved in the production, protection and use of crops and livestock. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D002623': NtO(MH='Chemistry Techniques, Analytical', MS='Methodologies used for the isolation, identification, detection, and quantitation of chemical substances.'),
  'D002624': NtO(MH='Chemistry, Clinical', MS='The specialty of ANALYTIC CHEMISTRY applied to assays of physiologically important substances found in blood, urine, tissues, and other biological fluids for the purpose of aiding the physician in making a diagnosis or following therapy.'),
  'D002625': NtO(MH='Chemistry, Organic', MS='The study of the structure, preparation, properties, and reactions of carbon compounds. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D002626': NtO(MH='Chemistry, Pharmaceutical', MS='Chemistry dealing with the composition and preparation of agents having PHARMACOLOGIC ACTIONS or diagnostic use.'),
  'D002627': NtO(MH='Chemistry, Physical', MS='The study of CHEMICAL PHENOMENA and processes in terms of the underlying PHYSICAL PHENOMENA and processes.'),
  'D002628': NtO(MH='Chemoreceptor Cells', MS='Cells specialized to detect chemical substances and relay that information centrally in the nervous system. Chemoreceptor cells may monitor external stimuli, as in TASTE and OLFACTION, or internal stimuli, such as the concentrations of OXYGEN and CARBON DIOXIDE in the blood.'),
  'D002629': NtO(MH='Chemosterilants', MS='Compounds that cause reproductive sterility in organisms. They are sometimes used to control pest populations by sterilizing males within the population.'),
  'D002630': NtO(MH='Chemotactic Factors', MS='Chemical substances that attract or repel cells. The concept denotes especially those factors released as a result of tissue injury, microbial invasion, or immunologic activity, that attract LEUKOCYTES; MACROPHAGES; or other cells to the site of infection or insult.'),
  'D002631': NtO(MH='Chemotactic Factors, Eosinophil', MS='Cytotaxins liberated from normal or invading cells that specifically attract eosinophils; they may be complement fragments, lymphokines, neutrophil products, histamine or other; the best known is the tetrapeptide ECF-A, released mainly by mast cells.'),
  'D002633': NtO(MH='Chemotaxis', MS='The movement of cells or organisms toward or away from a substance in response to its concentration gradient.'),
  'D002634': NtO(MH='Chemotaxis, Leukocyte', MS='The movement of leukocytes in response to a chemical concentration gradient or to products formed in an immunologic reaction.'),
  'D002635': NtO(MH='Chenodeoxycholic Acid', MS='A bile acid, usually conjugated with either glycine or taurine. It acts as a detergent to solubilize fats for intestinal absorption and is reabsorbed by the small intestine. It is used as cholagogue, a choleretic laxative, and to prevent or dissolve gallstones.'),
  'D002636': NtO(MH='Cherubism', MS='A fibro-osseous hereditary disease of the jaws. The swollen jaws and raised eyes give a cherubic appearance; multiple radiolucencies are evident upon radiographic examination.'),
  'D002637': NtO(MH='Chest Pain', MS='Pressure, burning, or numbness in the chest.'),
  'D002638': NtO(MH='Chewing Gum', MS='A preparation of chicle, sometimes mixed with other plastic substances, sweetened and flavored. It is masticated usually for pleasure as a candy substitute but it sometimes acts as a vehicle for the administration of medication.'),
  'D002639': NtO(MH='Cheyne-Stokes Respiration', MS='An abnormal pattern of breathing characterized by alternating periods of apnea and deep, rapid breathing. The cycle begins with slow, shallow breaths that gradually increase in depth and rate and is then followed by a period of apnea. The period of apnea can last 5 to 30 seconds, then the cycle repeats every 45 seconds to 3 minutes.'),
  'D002640': NtO(MH='Chiari-Frommel Syndrome', MS='A POSTPARTUM condition consists of persistent lactation (GALACTORRHEA) and AMENORRHEA in patients not BREAST FEEDING.'),
  'D002641': NtO(MH='Chicago', MS='A city in Illinois.'),
  'D002642': NtO(MH='Chick Embryo', MS='The developmental entity of a fertilized chicken egg (ZYGOTE). The developmental process begins about 24 h before the egg is laid at the BLASTODISC, a small whitish spot on the surface of the EGG YOLK. After 21 days of incubation, the embryo is fully developed before hatching.'),
  'D002644': NtO(MH='Chickenpox', MS='A highly contagious infectious disease caused by the varicella-zoster virus (HERPESVIRUS 3, HUMAN). It usually affects children, is spread by direct contact or respiratory route via droplet nuclei, and is characterized by the appearance on the skin and mucous membranes of successive crops of typical pruritic vesicular lesions that are easily broken and become scabbed. Chickenpox is relatively benign in children, but may be complicated by pneumonia and encephalitis in adults. (From Dorland, 27th ed)'),
  'D002645': NtO(MH='Chickens', MS='Common name for the species Gallus gallus, the domestic fowl, in the family Phasianidae, order GALLIFORMES. It is descended from the red jungle fowl of SOUTHEAST ASIA.'),
  'D002646': NtO(MH='Chikungunya virus', MS='A species of ALPHAVIRUS causing an acute dengue-like fever.'),
  'D002647': NtO(MH='Chilblains', MS='Recurrent localized itching, swelling and painful erythema on the fingers, toes or ears, produced by exposure to cold.'),
  'D002648': NtO(MH='Child', MS='A person 6 to 12 years of age. An individual 2 to 5 years old is CHILD, PRESCHOOL.'),
  'D002649': NtO(MH='Child Abuse', MS='Abuse of children in a family, institutional, or other setting. (APA, Thesaurus of Psychological Index Terms, 1994)'),
  'D002650': NtO(MH='Child Abuse, Sexual', MS='Sexual maltreatment of the child or minor.'),
  'D002651': NtO(MH='Child Advocacy', MS='Promotion and protection of the rights of children; frequently through a legal process.'),
  'D002652': NtO(MH='Child Behavior', MS='Any observable response or action of a child from 24 months through 12 years of age. For neonates or children younger than 24 months, INFANT BEHAVIOR is available.'),
  'D002653': NtO(MH='Child Behavior Disorders', MS='Disturbances considered to be pathological based on age and stage appropriateness, e.g., conduct disturbances and anaclitic depression. This concept does not include psychoneuroses, psychoses, or personality disorders with fixed patterns.'),
  'D002654': NtO(MH='Child Care', MS='Care of CHILDREN in the home or in an institution.'),
  'D002655': NtO(MH='Child Custody', MS='The formally authorized guardianship or care of a CHILD.'),
  'D002656': NtO(MH='Child Day Care Centers', MS='Facilities which provide care for pre-school and school-age children.'),
  'D002657': NtO(MH='Child Development', MS='The continuous sequential physiological and psychological maturing of an individual from birth up to but not including ADOLESCENCE.'),
  'D002658': NtO(MH='Developmental Disabilities', MS='Disorders in which there is a delay in development based on that expected for a given age level or stage of development. These impairments or disabilities originate before age 18, may be expected to continue indefinitely, and constitute a substantial impairment. Biological and nonbiological factors are involved in these disorders. (From American Psychiatric Glossary, 6th ed)'),
  'D002659': NtO(MH='Child Development Disorders, Pervasive', MS='Severe distortions in the development of many basic psychological functions that are not normal for any stage in development. These distortions are manifested in sustained social impairment, speech abnormalities, and peculiar motor movements.'),
  'D002660': NtO(MH='Child Guidance', MS='The psychiatric, sociological and psychological study and treatment of the developing child with emphasis on preventive or prophylactic measures focused on the familial, educational and socio-environmental milieu of the child.'),
  'D002661': NtO(MH='Child Guidance Clinics', MS='Facilities which administer the delivery of mental health counseling services to children.'),
  'D002662': NtO(MH='Child Health Services', MS='Organized services to provide health care for children.'),
  'D002663': NtO(MH='Child Language', MS='The language and sounds expressed by a child at a particular maturational stage in development.'),
  'D002664': NtO(MH='Child Nutritional Physiological Phenomena', MS='Nutritional physiology of children aged 2-12 years.'),
  'D002665': NtO(MH='Child Psychiatry', MS='The medical science that deals with the origin, diagnosis, prevention, and treatment of mental disorders in children.'),
  'D002666': NtO(MH='Psychology, Child', MS='Branch of psychology involving the study of normal and abnormal behavior of children.'),
  'D002667': NtO(MH='Child Reactive Disorders', MS='Reactions to an event or set of events which are considered to be of pathological degree, that have not developed into a neurosis, psychosis, or personality disorder with fixed patterns.'),
  'D002668': NtO(MH='Child Rearing', MS='The training or bringing-up of children by parents or parent-substitutes. It is used also for child rearing practices in different societies, at different economic levels, in different ethnic groups, etc. It differs from PARENTING in that in child rearing the emphasis is on the act of training or bringing up the child and the interaction between the parent and child, while parenting emphasizes the responsibility and qualities of exemplary behavior of the parent.'),
  'D002669': NtO(MH='Child Welfare', MS='Organized efforts by communities or organizations to improve the health and well-being of the child.'),
  'D002670': NtO(MH='Child, Abandoned', MS='A child or adolescent who is deserted by parents or parent substitutes without regard for its future care.'),
  'D002671': NtO(MH='Child, Exceptional', MS="A child whose needs, abilities, or other characteristics vary so much from the average in mental, physical, or social areas that a greater than usual level of services is needed to facilitate the child's maximum potential development."),
  'D002672': NtO(MH='Child, Gifted', MS='A child or adolescent who, when compared to others of the same age or experience, exhibits capability of high performance in intellectual, creative, or artistic areas, possesses an unusual capacity for leadership or excels in specific academic fields. (From PL 100-297, Sec. 4103, Definitions)'),
  'D002673': NtO(MH='Child, Hospitalized', MS='Child hospitalized for short term care.'),
  'D002674': NtO(MH='Child, Institutionalized', MS='A child who is receiving long-term in-patient services or who resides in an institutional setting.'),
  'D002675': NtO(MH='Child, Preschool', MS='A child between the ages of 2 and 5.'),
  'D002676': NtO(MH='Child, Unwanted', MS='The child who is not wanted by one or both parents.'),
  'D002677': NtO(MH='Chile', MS='A country in southern South America, bordering the South Pacific Ocean, between Argentina and Peru.'),
  'D002678': NtO(MH='Chimera', MS='An individual that contains cell populations derived from different zygotes.'),
  'D002679': NtO(MH='Pan troglodytes', MS='The common chimpanzee, a species of the genus Pan, family HOMINIDAE. It lives in Africa, primarily in the tropical rainforests. There are a number of recognized subspecies.'),
  'D002680': NtO(MH='Chin', MS='The anatomical frontal portion of the mandible, also known as the mentum, that contains the line of fusion of the two separate halves of the mandible (symphysis menti). This line of fusion divides inferiorly to enclose a triangular area called the mental protuberance. On each side, inferior to the second premolar tooth, is the mental foramen for the passage of blood vessels and a nerve.'),
  'D002681': NtO(MH='China', MS='A country spanning from central Asia to the Pacific Ocean.'),
  'D002682': NtO(MH='Chinchilla', MS='A genus of the family Chinchillidae which consists of three species: C. brevicaudata, C. lanigera, and C. villidera. They are used extensively in biomedical research.'),
  'D002683': NtO(MH='Chironomidae', MS='A family of nonbiting midges, in the order DIPTERA. Salivary glands of the genus Chironomus are used in studies of cellular genetics and biochemistry.'),
  'D002684': NtO(MH='Chiropractic', MS="An occupational discipline founded by D.D. Palmer in the 1890's based on the relationship of the spine to health and disease."),
  'D002685': NtO(MH='Chiroptera', MS='Order of mammals whose members are adapted for flight. It includes bats, flying foxes, and fruit bats.'),
  'D002686': NtO(MH='Chitin', MS='A linear polysaccharide of beta-1->4 linked units of ACETYLGLUCOSAMINE. It is the second most abundant biopolymer on earth, found especially in INSECTS and FUNGI. When deacetylated it is called CHITOSAN.'),
  'D002687': NtO(MH='Chitin Synthase', MS='An enzyme that converts UDP glucosamine into chitin and UDP. EC 2.4.1.16.'),
  'D002688': NtO(MH='Chitinases', MS='Glycoside hydrolases that break down glycosidic bonds in CHITIN. They are important for insect and worm morphogenesis and plant defense against pathogens. Human chitinases may play a role in the etiology of allergies and asthma.'),
  'D002689': NtO(MH='Chlamydia', MS='A genus of the family CHLAMYDIACEAE whose species cause a variety of diseases in vertebrates including humans, mice, and swine. Chlamydia species are gram-negative and produce glycogen. The type species is CHLAMYDIA TRACHOMATIS.'),
  'D002690': NtO(MH='Chlamydia Infections', MS='Infections with bacteria of the genus CHLAMYDIA.'),
  'D002691': NtO(MH='Chlamydophila psittaci', MS='A genus of CHLAMYDOPHILA infecting primarily birds. It contains eight known serovars, some of which infect more than one type of host, including humans.'),
  'D002692': NtO(MH='Chlamydia trachomatis', MS='Type species of CHLAMYDIA causing a variety of ocular and urogenital diseases.'),
  'D002693': NtO(MH='Chlamydiaceae', MS='A family of gram-negative, coccoid microorganisms, in the order CHLAMYDIALES, pathogenic for vertebrates. Genera include CHLAMYDIA and CHLAMYDOPHILA.'),
  'D002694': NtO(MH='Chlamydiaceae Infections', MS='Infections with bacteria of the family CHLAMYDIACEAE.'),
  'D002695': NtO(MH='Chlamydiales', MS='An order of obligately intracellular, gram-negative bacteria that have the chlamydia-like developmental cycle of replication. This is a two-stage cycle that includes a metabolically inactive infectious form, and a vegetative form that replicates by binary fission. Members of Chlamydiales are disseminated by aerosol or by contact. There are at least six recognized families: CHLAMYDIACEAE, Criblamydiaceae, Parachlamydiaceae, Rhabdochlamydia, Simkaniaceae, and Waddliaceae.'),
  'D002696': NtO(MH='Chlamydomonas', MS='A genus GREEN ALGAE in the order VOLVOCIDA. It consists of solitary biflagellated organisms common in fresh water and damp soil.'),
  'D002697': NtO(MH='Chloral Hydrate', MS='A hypnotic and sedative used in the treatment of INSOMNIA.'),
  'D002698': NtO(MH='Chloralose', MS='A derivative of CHLORAL HYDRATE that was used as a sedative but has been replaced by safer and more effective drugs. Its most common use is as a general anesthetic in animal experiments.'),
  'D002699': NtO(MH='Chlorambucil', MS="A nitrogen mustard alkylating agent used as antineoplastic for chronic lymphocytic leukemia, Hodgkin's disease, and others. Although it is less toxic than most other nitrogen mustards, it has been listed as a known carcinogen in the Fourth Annual Report on Carcinogens (NTP 85-002, 1985). (Merck Index, 11th ed)"),
  'D002700': NtO(MH='Chloramines', MS='Inorganic derivatives of ammonia by substitution of one or more hydrogen atoms with chlorine atoms or organic compounds with the general formulas R2NCl and RNCl2 (where R is an organic group).'),
  'D002701': NtO(MH='Chloramphenicol', MS='An antibiotic first isolated from cultures of Streptomyces venequelae in 1947 but now produced synthetically. It has a relatively simple structure and was the first broad-spectrum antibiotic to be discovered. It acts by interfering with bacterial protein synthesis and is mainly bacteriostatic. (From Martindale, The Extra Pharmacopoeia, 29th ed, p106)'),
  'D002702': NtO(MH='Chloramphenicol Resistance', MS='Nonsusceptibility of bacteria to the action of CHLORAMPHENICOL, a potent inhibitor of protein synthesis in the 50S ribosomal subunit where amino acids are added to nascent bacterial polypeptides.'),
  'D002703': NtO(MH='Chloranil', MS='A quinone fungicide used for treatment of seeds and foliage.'),
  'D002704': NtO(MH='Chlorates', MS='Inorganic salts of chloric acid that contain the ClO3- ion.'),
  'D002706': NtO(MH='Chlordan', MS='A highly poisonous organochlorine insecticide. The EPA has cancelled registrations of pesticides containing this compound with the exception of its use through subsurface ground insertion for termite control and the dipping of roots or tops of non-food plants. (From Merck Index, 11th ed)'),
  'D002707': NtO(MH='Chlordiazepoxide', MS='An anxiolytic benzodiazepine derivative with anticonvulsant, sedative, and amnesic properties. It has also been used in the symptomatic treatment of alcohol withdrawal.'),
  'D002708': NtO(MH='Chlorella', MS='Nonmotile unicellular green algae potentially valuable as a source of high-grade protein and B-complex vitamins.'),
  'D002709': NtO(MH='Chlorfenvinphos', MS='An organophosphorus cholinesterase inhibitor that is used as an insecticide and an acaricide.'),
  'D002710': NtO(MH='Chlorhexidine', MS='A disinfectant and topical anti-infective agent used also as mouthwash to prevent oral plaque.'),
  'D002711': NtO(MH='Chloride Peroxidase', MS='An enzyme that catalyzes the chlorination of a range of organic molecules, forming stable carbon-chloride bonds. EC 1.11.1.10.'),
  'D002712': NtO(MH='Chlorides', MS='Inorganic compounds derived from hydrochloric acid that contain the Cl- ion.'),
  'D002713': NtO(MH='Chlorine', MS='An element with atomic symbol Cl, atomic number 17, and atomic weight 35, and member of the halogen family.'),
  'D002714': NtO(MH='Chlorisondamine', MS='A nicotinic antagonist used primarily as a ganglionic blocker in animal research. It has been used as an antihypertensive agent but has been supplanted by more specific drugs in most clinical applications.'),
  'D002715': NtO(MH='Chlormadinone Acetate', MS='An orally active synthetic progestational hormone used often in combinations as an oral contraceptive (CONTRACEPTIVES, ORAL).'),
  'D002716': NtO(MH='Chlormequat', MS='A plant growth regulator that is commonly used on ornamental plants.'),
  'D002717': NtO(MH='Chlormerodrin', MS='A mercurial compound that has been used as a diuretic but is now superseded by more potent and less toxic drugs. The radiolabeled form has been used as a diagnostic and research tool.'),
  'D002719': NtO(MH='Chlormethiazole', MS='A sedative and anticonvulsant often used in the treatment of alcohol withdrawal. Chlormethiazole has also been proposed as a neuroprotective agent. The mechanism of its therapeutic activity is not entirely clear, but it does potentiate GAMMA-AMINOBUTYRIC ACID receptors response and it may also affect glycine receptors.'),
  'D002720': NtO(MH='Chlormezanone', MS='A non-benzodiazepine that is used in the management of anxiety. It has been suggested for use in the treatment of muscle spasm.'),
  'D002721': NtO(MH='omega-Chloroacetophenone', MS='A potent eye, throat, and skin irritant. One of its uses is as a riot control agent.'),
  'D002722': NtO(MH='Chlorobenzenes', MS='Aromatic organic compounds with the chemical formula C6H5Cln.'),
  'D002723': NtO(MH='Chlorobenzoates', MS='Benzoic acid or benzoic acid esters substituted with one or more chlorine atoms.'),
  'D002724': NtO(MH='Chlorobutanol', MS='A colorless to white crystalline compound with a camphoraceous odor and taste. It is a widely used preservative in various pharmaceutical solutions, especially injectables. Also, it is an active ingredient in certain oral sedatives and topical anesthetics.'),
  'D002725': NtO(MH='Chloroform', MS='A commonly used laboratory solvent. It was previously used as an anesthetic, but was banned from use in the U.S. due to its suspected carcinogenicity.'),
  'D002726': NtO(MH='Chlorogenic Acid', MS='A naturally occurring phenolic acid which is a carcinogenic inhibitor. It has also been shown to prevent paraquat-induced oxidative stress in rats. (From J Chromatogr A 1996;741(2):223-31; Biosci Biotechnol Biochem 1996;60(5):765-68).'),
  'D002727': NtO(MH='Proguanil', MS='A biguanide compound which metabolizes in the body to form cycloguanil, an anti-malaria agent.'),
  'D002728': NtO(MH='Chlorohydrins', MS='Any of the compounds derived from a group of glycols or polyhydroxy alcohols by chlorine substitution for part of the hydroxyl groups. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D002729': NtO(MH='Chloromercuribenzoates', MS='Chloride and mercury-containing derivatives of benzoic acid.'),
  'D002730': NtO(MH='Chloromercurinitrophenols', MS='Mercuriphenols substituted with one or more chlorine atoms and one or more nitro groups. Some of these are sulfhydryl reagents which act as chromophoric probes in enzymes and other proteins.'),
  'D002731': NtO(MH='4-Chloromercuribenzenesulfonate', MS='A cytotoxic sulfhydryl reagent that inhibits several subcellular metabolic systems and is used as a tool in cellular physiology.'),
  'D002733': NtO(MH='Chlorophenols', MS='Phenols substituted with one or more chlorine atoms in any position.'),
  'D002734': NtO(MH='Chlorophyll', MS='Porphyrin derivatives containing magnesium that act to convert light energy in photosynthetic organisms.'),
  'D002735': NtO(MH='Chlorophyllides', MS='Products of the hydrolysis of chlorophylls in which the phytic acid side chain has been removed and the carboxylic acids saponified.'),
  'D002736': NtO(MH='Chloroplasts', MS='Plant cell inclusion bodies that contain the photosynthetic pigment CHLOROPHYLL, which is associated with the membrane of THYLAKOIDS. Chloroplasts occur in cells of leaves and young stems of plants. They are also found in some forms of PHYTOPLANKTON such as HAPTOPHYTA; DINOFLAGELLATES; DIATOMS; and CRYPTOPHYTA.'),
  'D002737': NtO(MH='Chloroprene', MS='Toxic, possibly carcinogenic, monomer of neoprene, a synthetic rubber; causes damage to skin, lungs, CNS, kidneys, liver, blood cells and fetuses. Synonym: 2-chlorobutadiene.'),
  'D002738': NtO(MH='Chloroquine', MS='The prototypical antimalarial agent with a mechanism that is not well understood. It has also been used to treat rheumatoid arthritis, systemic lupus erythematosus, and in the systemic therapy of amebic liver abscesses.'),
  'D002739': NtO(MH='Chloroquinolinols', MS='8-Hydroxyquinolinols chlorinated on the number 5 and/or 7 carbon atom(s). They are antibacterial, antiprotozoal, and antidiarrheal, especially in amebiasis, and have also been used as antiseborrheics. The compounds are mostly used topically, but have been used also as animal feed additives. They may cause optic and other neuropathies and are most frequently administered in combination with other agents.'),
  'D002740': NtO(MH='Chlorothiazide', MS='A thiazide diuretic with actions and uses similar to those of HYDROCHLOROTHIAZIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p812)'),
  'D002741': NtO(MH='Chlorotrianisene', MS='A powerful synthetic, non-steroidal estrogen.'),
  'D002742': NtO(MH='Chlorphenamidine', MS='An acaricide used against many organophosphate and carbamate resistant pests. It acts as an uncoupling agent and monoamine oxidase inhibitor.'),
  'D002743': NtO(MH='Chlorphenesin', MS='A centrally acting muscle relaxant. Its mode of action is unknown. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1203)'),
  'D002744': NtO(MH='Chlorpheniramine', MS='A histamine H1 antagonist used in allergic reactions, hay fever, rhinitis, urticaria, and asthma. It has also been used in veterinary applications. One of the most widely used of the classical antihistaminics, it generally causes less drowsiness and sedation than PROMETHAZINE.'),
  'D002745': NtO(MH='Chlorphentermine', MS='A sympathomimetic agent that was formerly used as an anorectic. It has properties similar to those of DEXTROAMPHETAMINE. It has been implicated in lipid storage disorders and pulmonary hypertension. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1223)'),
  'D002746': NtO(MH='Chlorpromazine', MS="The prototypical phenothiazine antipsychotic drug. Like the other drugs in this class chlorpromazine's antipsychotic actions are thought to be due to long-term adaptation by the brain to blocking DOPAMINE RECEPTORS. Chlorpromazine has several other actions and therapeutic uses, including as an antiemetic and in the treatment of intractable hiccup."),
  'D002747': NtO(MH='Chlorpropamide', MS='A sulfonylurea hypoglycemic agent used in the treatment of non-insulin-dependent diabetes mellitus not responding to dietary modification. (From Martindale, The Extra Pharmacopoeia, 30th ed, p277)'),
  'D002748': NtO(MH='Chlorpropham', MS='A carbamate that is used as an herbicide and as a plant growth regulator.'),
  'D002749': NtO(MH='Chlorprothixene', MS='A thioxanthine with effects similar to the phenothiazine antipsychotics.'),
  'D002750': NtO(MH='Chlorquinaldol', MS='Local anti-infective agent used for skin, gastrointestinal, and vaginal infections with fungi, protozoa, and certain bacteria. In animals, it causes central nervous system damage and is not administered parenterally. It is also used as antiseptic, fungistat, or deodorant.'),
  'D002751': NtO(MH='Chlortetracycline', MS='A TETRACYCLINE with a 7-chloro substitution.'),
  'D002752': NtO(MH='Chlorthalidone', MS='A benzenesulfonamide-phthalimidine that tautomerizes to a BENZOPHENONES form. It is considered a thiazide-like diuretic.'),
  'D002753': NtO(MH='Chlorzoxazone', MS='A centrally acting central muscle relaxant with sedative properties. It is claimed to inhibit muscle spasm by exerting an effect primarily at the level of the spinal cord and subcortical areas of the brain. (From Martindale, The Extra Pharmacopoea, 30th ed, p1202)'),
  'D002754': NtO(MH='Choanal Atresia', MS='A congenital abnormality that is characterized by a blocked CHOANAE, the opening between the nose and the NASOPHARYNX. Blockage can be unilateral or bilateral; bony or membranous.'),
  'D002755': NtO(MH='Choice Behavior', MS='The act of making a selection among two or more alternatives, usually after a period of deliberation.'),
  'D002756': NtO(MH='Cholagogues and Choleretics', MS='Gastrointestinal agents that stimulate the flow of bile into the duodenum (cholagogues) or stimulate the production of bile by the liver (choleretic).'),
  'D002757': NtO(MH='Cholanes', MS=None),
  'D002758': NtO(MH='Cholangiography', MS='An imaging test of the BILIARY TRACT in which a contrast dye (RADIOPAQUE MEDIA) is injected into the BILE DUCT and x-ray pictures are taken.'),
  'D002759': NtO(MH='Adenoma, Bile Duct', MS='A benign tumor of the intrahepatic bile ducts.'),
  'D002760': NtO(MH='Cholangiopancreatography, Endoscopic Retrograde', MS="Fiberoptic endoscopy designed for duodenal observation and cannulation of VATER'S AMPULLA, in order to visualize the pancreatic and biliary duct system by retrograde injection of contrast media. Endoscopic (Vater) papillotomy (SPHINCTEROTOMY, ENDOSCOPIC) may be performed during this procedure."),
  'D002761': NtO(MH='Cholangitis', MS='Inflammation of the biliary ductal system (BILE DUCTS); intrahepatic, extrahepatic, or both.'),
  'D002762': NtO(MH='Cholecalciferol', MS='Derivative of 7-dehydroxycholesterol formed by ULTRAVIOLET RAYS breaking of the C9-C10 bond. It differs from ERGOCALCIFEROL in having a single bond between C22 and C23 and lacking a methyl group at C24.'),
  'D002763': NtO(MH='Cholecystectomy', MS='Surgical removal of the GALLBLADDER.'),
  'D002764': NtO(MH='Cholecystitis', MS='Inflammation of the GALLBLADDER; generally caused by impairment of BILE flow, GALLSTONES in the BILIARY TRACT, infections, or other diseases.'),
  'D002765': NtO(MH='Cholecystography', MS='Radiography of the gallbladder after ingestion of a contrast medium.'),
  'D002766': NtO(MH='Cholecystokinin', MS='A peptide, of about 33 amino acids, secreted by the upper INTESTINAL MUCOSA and also found in the central nervous system. It causes gallbladder contraction, release of pancreatic exocrine (or digestive) enzymes, and affects other gastrointestinal functions. Cholecystokinin may be the mediator of satiety.'),
  'D002767': NtO(MH='Cholecystostomy', MS='Establishment of an opening into the gallbladder either for drainage or surgical communication with another part of the digestive tract, usually the duodenum or jejunum.'),
  'D002768': NtO(MH='Choledochostomy', MS='Surgical formation of an opening (stoma) into the COMMON BILE DUCT for drainage or for direct communication with a site in the small intestine, primarily the DUODENUM or JEJUNUM.'),
  'D002769': NtO(MH='Cholelithiasis', MS='Presence or formation of GALLSTONES in the BILIARY TRACT, usually in the gallbladder (CHOLECYSTOLITHIASIS) or the common bile duct (CHOLEDOCHOLITHIASIS).'),
  'D002770': NtO(MH='Cholenes', MS='Unsaturated derivatives of cholane with methyl groups at C-10 and C-13 and a branched five-carbon chain at C-17. They must have at least one double bond in the ring system.'),
  'D002771': NtO(MH='Cholera', MS='An acute diarrheal disease endemic in India and Southeast Asia whose causative agent is VIBRIO CHOLERAE. This condition can lead to severe dehydration in a matter of hours unless quickly treated.'),
  'D002772': NtO(MH='Cholera Toxin', MS='An ENTEROTOXIN from VIBRIO CHOLERAE. It consists of two major protomers, the heavy (H) or A subunit and the B protomer which consists of 5 light (L) or B subunits. The catalytic A subunit is proteolytically cleaved into fragments A1 and A2. The A1 fragment is a MONO(ADP-RIBOSE) TRANSFERASE. The B protomer binds cholera toxin to intestinal epithelial cells, and facilitates the uptake of the A1 fragment. The A1 catalyzed transfer of ADP-RIBOSE to the alpha subunits of heterotrimeric G PROTEINS activates the production of CYCLIC AMP. Increased levels of cyclic AMP are thought to modulate release of fluid and electrolytes from intestinal crypt cells.'),
  'D002774': NtO(MH='Cholestadienes', MS='Cholene derivatives with methyl groups at C-10 and C-13 and a branched 8-carbon chain at C-17. They must have two double bonds in the ring system.'),
  'D002775': NtO(MH='Cholestadienols', MS='Cholestadiene derivatives containing a hydroxy group anywhere in the molecule.'),
  'D002776': NtO(MH='Cholestanes', MS='Derivatives of the saturated steroid cholestane with methyl groups at C-18 and C-19 and an iso-octyl side chain at C-17.'),
  'D002777': NtO(MH='Cholestanols', MS='Cholestanes substituted in any position with one or more hydroxy groups. They are found in feces and bile. In contrast to bile acids and salts, they are not reabsorbed.'),
  'D002778': NtO(MH='Cholestanones', MS='CHOLESTANES substituted with any number of keto groups.'),
  'D002779': NtO(MH='Cholestasis', MS='Impairment of bile flow due to obstruction in small bile ducts (INTRAHEPATIC CHOLESTASIS) or obstruction in large bile ducts (EXTRAHEPATIC CHOLESTASIS).'),
  'D002780': NtO(MH='Cholestasis, Intrahepatic', MS='Impairment of bile flow due to injury to the HEPATOCYTES; BILE CANALICULI; or the intrahepatic bile ducts (BILE DUCTS, INTRAHEPATIC).'),
  'D002781': NtO(MH='Cholesteatoma', MS='A non-neoplastic mass of keratin-producing squamous EPITHELIUM, frequently occurring in the MENINGES; bones of the skull, and most commonly in the MIDDLE EAR and MASTOID region. Cholesteatoma can be congenital or acquired. Cholesteatoma is not a tumor nor is it associated with high CHOLESTEROL.'),
  'D002782': NtO(MH='Cholestenes', MS='Steroids with methyl groups at C-10 and C-13 and a branched 8-carbon chain at C-17. Members include compounds with any degree of unsaturation; however, CHOLESTADIENES is available for derivatives containing two double bonds.'),
  'D002783': NtO(MH='Cholestenones', MS='CHOLESTENES with one or more double bonds and substituted by any number of keto groups.'),
  'D002784': NtO(MH='Cholesterol', MS='The principal sterol of all higher animals, distributed in body tissues, especially the brain and spinal cord, and in animal fats and oils.'),
  'D002785': NtO(MH='Sterol O-Acyltransferase', MS='An enzyme that catalyzes the formation of cholesterol esters by the direct transfer of the fatty acid group from a fatty acyl CoA derivative. This enzyme has been found in the adrenal gland, gonads, liver, intestinal mucosa, and aorta of many mammalian species. EC 2.3.1.26.'),
  'D002786': NtO(MH='Cholesterol Side-Chain Cleavage Enzyme', MS='A mitochondrial cytochrome P450 enzyme that catalyzes the side-chain cleavage of C27 cholesterol to C21 pregnenolone in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP11A1 gene, catalyzes the breakage between C20 and C22 which is the initial and rate-limiting step in the biosynthesis of various gonadal and adrenal steroid hormones.'),
  'D002787': NtO(MH='Sterol Esterase', MS='An enzyme that catalyzes the hydrolysis of CHOLESTEROL ESTERS and some other sterol esters, to liberate cholesterol plus a fatty acid anion.'),
  'D002788': NtO(MH='Cholesterol Esters', MS='Fatty acid esters of cholesterol which constitute about two-thirds of the cholesterol in the plasma. The accumulation of cholesterol esters in the arterial intima is a characteristic feature of atherosclerosis.'),
  'D002789': NtO(MH='Cholesterol Oxidase', MS='An enzyme that catalyzes the oxidation of cholesterol in the presence of molecular oxygen to 4-cholesten-3-one and hydrogen peroxide. The enzyme is not specific for cholesterol, but will also oxidize other 3-hydroxysteroids. EC 1.1.3.6.'),
  'D002790': NtO(MH='Cholesterol 7-alpha-Hydroxylase', MS='A membrane-bound cytochrome P450 enzyme that catalyzes the 7-alpha-hydroxylation of CHOLESTEROL in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP7, converts cholesterol to 7-alpha-hydroxycholesterol which is the first and rate-limiting step in the synthesis of BILE ACIDS.'),
  'D002791': NtO(MH='Cholesterol, Dietary', MS='Cholesterol present in food, especially in animal products.'),
  'D002792': NtO(MH='Cholestyramine Resin', MS='A strongly basic anion exchange resin whose main constituent is polystyrene trimethylbenzylammonium Cl(-) anion.'),
  'D002793': NtO(MH='Cholic Acids', MS='The 3 alpha,7 alpha,12 alpha-trihydroxy-5 beta-cholanic acid family of bile acids in man, usually conjugated with glycine or taurine. They act as detergents to solubilize fats for intestinal absorption, are reabsorbed by the small intestine, and are used as cholagogues and choleretics.'),
  'D002794': NtO(MH='Choline', MS='A basic constituent of lecithin that is found in many plants and animal organs. It is important as a precursor of acetylcholine, as a methyl donor in various metabolic processes, and in lipid metabolism.'),
  'D002795': NtO(MH='Choline O-Acetyltransferase', MS='An enzyme that catalyzes the formation of acetylcholine from acetyl-CoA and choline. EC 2.3.1.6.'),
  'D002796': NtO(MH='Choline Deficiency', MS='A condition produced by a deficiency of CHOLINE in animals. Choline is known as a lipotropic agent because it has been shown to promote the transport of excess fat from the liver under certain conditions in laboratory animals. Combined deficiency of choline (included in the B vitamin complex) and all other methyl group donors causes liver cirrhosis in some animals. Unlike compounds normally considered as vitamins, choline does not serve as a cofactor in enzymatic reactions. (From Saunders Dictionary & Encyclopedia of Laboratory Medicine and Technology, 1984)'),
  'D002797': NtO(MH='Choline Kinase', MS='An enzyme that is active in the first step of choline phosphoglyceride (lecithin) biosynthesis by catalyzing the phosphorylation of choline to phosphorylcholine in the presence of ATP. Ethanolamine and its methyl and ethyl derivatives can also act as acceptors. EC 2.7.1.32.'),
  'D002798': NtO(MH='Diacylglycerol Cholinephosphotransferase', MS='An enzyme that catalyzes the synthesis of phosphatidylcholines from CDPcholine and 1,2-diacylglycerols. EC 2.7.8.2.'),
  'D002799': NtO(MH='Cholinergic Fibers', MS='Nerve fibers liberating acetylcholine at the synapse after an impulse.'),
  'D002800': NtO(MH='Cholinesterase Inhibitors', MS='Drugs that inhibit cholinesterases. The neurotransmitter ACETYLCHOLINE is rapidly hydrolyzed, and thereby inactivated, by cholinesterases. When cholinesterases are inhibited, the action of endogenously released acetylcholine at cholinergic synapses is potentiated. Cholinesterase inhibitors are widely used clinically for their potentiation of cholinergic inputs to the gastrointestinal tract and urinary bladder, the eye, and skeletal muscles; they are also used for their effects on the heart and the central nervous system.'),
  'D002801': NtO(MH='Cholinesterase Reactivators', MS='Drugs used to reverse the inactivation of cholinesterase caused by organophosphates or sulfonates. They are an important component of therapy in agricultural, industrial, and military poisonings by organophosphates and sulfonates.'),
  'D002802': NtO(MH='Cholinesterases', MS=None),
  'D002803': NtO(MH='Chondro-4-Sulfatase', MS='An enzyme from the sulfuric ester hydrolase class that breaks down one of the products of the chondroitin lyase II reaction. EC 3.1.6.9.'),
  'D002804': NtO(MH='Chondroblastoma', MS='A usually benign tumor composed of cells which arise from chondroblasts or their precursors and which tend to differentiate into cartilage cells. It occurs primarily in the epiphyses of adolescents. It is relatively rare and represents less than 2% of all primary bone tumors. The peak incidence is in the second decade of life; it is about twice as common in males as in females. (From Dorland, 27th ed; Holland et al., Cancer Medicine, 3d ed, p1846)'),
  'D002805': NtO(MH='Chondrocalcinosis', MS='Presence of CALCIUM PYROPHOSPHATE in the connective tissues such as the cartilaginous structures of  joints. When accompanied by GOUT-like symptoms, it is referred to as pseudogout.'),
  'D002806': NtO(MH='Chondrodysplasia Punctata', MS='A heterogeneous group of bone dysplasias, the common character of which is stippling of the epiphyses in infancy. The group includes a severe autosomal recessive form (CHONDRODYSPLASIA PUNCTATA, RHIZOMELIC), an autosomal dominant form (Conradi-Hunermann syndrome), and a milder X-linked form. Metabolic defects associated with impaired peroxisomes are present only in the rhizomelic form.'),
  'D002807': NtO(MH='Chondroitin', MS="A mucopolysaccharide constituent of chondrin. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D002808': NtO(MH='Chondroitin Lyases', MS='Enzymes which catalyze the elimination of delta-4,5-D-glucuronate residues from polysaccharides containing 1,4-beta-hexosaminyl and 1,3-beta-D-glucuronosyl or 1,3-alpha-L-iduronosyl linkages thereby bringing about depolymerization. EC 4.2.2.4 acts on chondroitin sulfate A and C as well as on dermatan sulfate and slowly on hyaluronate. EC 4.2.2.5 acts on chondroitin sulfate A and C.'),
  'D002809': NtO(MH='Chondroitin Sulfates', MS='Derivatives of chondroitin which have a sulfate moiety esterified to the galactosamine moiety of chondroitin. Chondroitin sulfate A, or chondroitin 4-sulfate, and chondroitin sulfate C, or chondroitin 6-sulfate, have the sulfate esterified in the 4- and 6-positions, respectively. Chondroitin sulfate B (beta heparin; DERMATAN SULFATE) is a misnomer and this compound is not a true chondroitin sulfate.'),
  'D002810': NtO(MH='Chondroitinases and Chondroitin Lyases', MS='Enzymes which catalyze the elimination of glucuronate residues from chondroitin A,B, and C or which catalyze the hydrolysis of sulfate groups of the 2-acetamido-2-deoxy-D-galactose 6-sulfate units of chondroitin sulfate. EC 4.2.2.-.'),
  'D002811': NtO(MH='Chondroitinsulfatases', MS='A group of enzymes that catalyze the hydrolysis of various sulfate bonds of chondroitin sulfate. EC 3.1.6.-.'),
  'D002812': NtO(MH='Chondroma', MS='A benign neoplasm derived from mesodermal cells that form cartilage. It may remain within the substance of a cartilage or bone (true chondroma or enchondroma) or may develop on the surface of a cartilage (ecchondroma or ecchondrosis). (Dorland, 27th ed; Stedman, 25th ed)'),
  'D002813': NtO(MH='Chondrosarcoma', MS='A slowly growing malignant neoplasm derived from cartilage cells, occurring most frequently in pelvic bones or near the ends of long bones, in middle-aged and old people. Most chondrosarcomas arise de novo, but some may develop in a preexisting benign cartilaginous lesion or in patients with ENCHONDROMATOSIS. (Stedman, 25th ed)'),
  'D002814': NtO(MH='Chorda Tympani Nerve', MS='A branch of the facial (7th cranial) nerve which passes through the middle ear and continues through the petrotympanic fissure. The chorda tympani nerve carries taste sensation from the anterior two-thirds of the tongue and conveys parasympathetic efferents to the salivary glands.'),
  'D002815': NtO(MH='Chordae Tendineae', MS='The tendinous cords that connect each cusp of the two atrioventricular HEART VALVES to appropriate PAPILLARY MUSCLES in the HEART VENTRICLES, preventing the valves from reversing themselves when the ventricles contract.'),
  'D002816': NtO(MH='Chordata, Nonvertebrate', MS='A portion of the animal phylum Chordata comprised of the subphyla CEPHALOCHORDATA; UROCHORDATA, and HYPEROTRETI, but not including the Vertebrata (VERTEBRATES). It includes nonvertebrate animals having a NOTOCHORD during some developmental stage.'),
  'D002817': NtO(MH='Chordoma', MS='A malignant tumor arising from the embryonic remains of the notochord. It is also called chordocarcinoma, chordoepithelioma, and notochordoma. (Dorland, 27th ed)'),
  'D002818': NtO(MH='Cordotomy', MS='Any operation on the spinal cord. (Stedman, 26th ed)'),
  'D002819': NtO(MH='Chorea', MS='Involuntary, forcible, rapid, jerky movements that may be subtle or become confluent, markedly altering normal patterns of movement. Hypotonia and pendular reflexes are often associated. Conditions which feature recurrent or persistent episodes of chorea as a primary manifestation of disease are referred to as CHOREATIC DISORDERS. Chorea is also a frequent manifestation of BASAL GANGLIA DISEASES.'),
  'D002820': NtO(MH='Hydatidiform Mole, Invasive', MS='A uterine tumor derived from persistent gestational TROPHOBLASTS, most likely after a molar pregnancy (HYDATIDIFORM MOLE). Invasive hyadatiform mole develops in about 15% of patients after evacuation of a complete mole and less frequently after other types of gestation. It may perforate the MYOMETRIUM and erode uterine vessels causing hemorrhage.'),
  'D002821': NtO(MH='Chorioamnionitis', MS='INFLAMMATION of the placental membranes (CHORION; AMNION) and connected tissues such as fetal BLOOD VESSELS and UMBILICAL CORD. It is often associated with intrauterine ascending infections during PREGNANCY.'),
  'D002822': NtO(MH='Choriocarcinoma', MS='A malignant metastatic form of trophoblastic tumors. Unlike the HYDATIDIFORM MOLE, choriocarcinoma contains no CHORIONIC VILLI but rather sheets of undifferentiated cytotrophoblasts and syncytiotrophoblasts (TROPHOBLASTS). It is characterized by the large amounts of CHORIONIC GONADOTROPIN produced. Tissue origins can be determined by DNA analyses: placental (fetal) origin or non-placental origin (CHORIOCARCINOMA, NON-GESTATIONAL).'),
  'D002823': NtO(MH='Chorion', MS='The outermost extra-embryonic membrane surrounding the developing embryo. In REPTILES and BIRDS, it adheres to the shell and allows exchange of gases between the egg and its environment. In MAMMALS, the chorion evolves into the fetal contribution of the PLACENTA.'),
  'D002824': NtO(MH='Chorionic Villi', MS='Threadlike vascular projections of the chorion. Chorionic villi may be free or embedded within the DECIDUA forming the site for exchange of substances between fetal and maternal blood (PLACENTA).'),
  'D002825': NtO(MH='Chorioretinitis', MS='Inflammation of the choroid in which the sensory retina becomes edematous and opaque. The inflammatory cells and exudate may burst through the sensory retina to cloud the vitreous body.'),
  'D002826': NtO(MH='Chorismate Mutase', MS='An isomerase that catalyzes the conversion of chorismic acid to prephenic acid. EC 5.4.99.5.'),
  'D002827': NtO(MH='Chorismic Acid', MS='A cyclohexadiene carboxylic acid derived from SHIKIMIC ACID and a precursor for the biosynthesis of UBIQUINONE and the AROMATIC AMINO ACIDS.'),
  'D002828': NtO(MH='Choristoma', MS='A mass of histologically normal tissue present in an abnormal location.'),
  'D002829': NtO(MH='Choroid', MS='The thin, highly vascular membrane covering most of the posterior of the eye between the RETINA and SCLERA.'),
  'D002830': NtO(MH='Choroid Neoplasms', MS='Tumors of the choroid; most common intraocular tumors are malignant melanomas of the choroid. These usually occur after puberty and increase in incidence with advancing age. Most malignant melanomas of the uveal tract develop from benign melanomas (nevi).'),
  'D002831': NtO(MH='Choroid Plexus', MS='A villous structure of tangled masses of BLOOD VESSELS contained within the third, lateral, and fourth ventricles of the BRAIN. It regulates part of the production and composition of CEREBROSPINAL FLUID.'),
  'D002832': NtO(MH='Choroid Hemorrhage', MS='Hemorrhage from the vessels of the choroid.'),
  'D002833': NtO(MH='Choroiditis', MS='Inflammation of the choroid.'),
  'D002834': NtO(MH='Christian Science', MS='A religion founded by Mary Baker Eddy in 1866 that was organized under the official name of the Church of Christ, Scientist. It includes the practice of spiritual healing.'),
  'D002835': NtO(MH='Christianity', MS="The religion stemming from the life, teachings, and death of Jesus Christ: the religion that believes in God as the Father Almighty who works redemptively through the Holy Spirit for men's salvation and that affirms Jesus Christ as Lord and Savior who proclaimed to man the gospel of salvation. (From Webster, 3d ed)"),
  'D002836': NtO(MH='Hemophilia B', MS='A deficiency of blood coagulation factor IX inherited as an X-linked disorder. (Also known as Christmas Disease, after the first patient studied in detail, not the holy day.) Historical and clinical features resemble those in classic hemophilia (HEMOPHILIA A), but patients present with fewer symptoms. Severity of bleeding is usually similar in members of a single family. Many patients are asymptomatic until the hemostatic system is stressed by surgery or trauma. Treatment is similar to that for hemophilia A. (From Cecil Textbook of Medicine, 19th ed, p1008)'),
  'D002837': NtO(MH='Chromaffin Granules', MS='Organelles in CHROMAFFIN CELLS located in the adrenal glands and various other organs. These granules are the site of the synthesis, storage, metabolism, and secretion of EPINEPHRINE and NOREPINEPHRINE.'),
  'D002838': NtO(MH='Chromaffin System', MS='The cells of the body which stain with chromium salts. They occur along the sympathetic nerves, in the adrenal gland, and in various other organs.'),
  'D002839': NtO(MH='Chromans', MS='Benzopyrans saturated in the 2 and 3 positions.'),
  'D002840': NtO(MH='Chromates', MS='Salts of chromic acid containing the CrO(2-)4 radical.'),
  'D002841': NtO(MH='Chromatiaceae', MS='A family of phototrophic purple sulfur bacteria that deposit globules of elemental sulfur inside their cells. They are found in diverse aquatic environments.'),
  'D002842': NtO(MH='Chromatids', MS='Either of the two longitudinally adjacent threads formed when a eukaryotic chromosome replicates prior to mitosis. The chromatids are held together at the centromere. Sister chromatids are derived from the same chromosome. (Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D002843': NtO(MH='Chromatin', MS='The material of CHROMOSOMES. It is a complex of DNA; HISTONES; and nonhistone proteins (CHROMOSOMAL PROTEINS, NON-HISTONE) found within the nucleus of a cell.'),
  'D002844': NtO(MH='Chromatium', MS='A genus of gram-negative, ovoid to rod-shaped bacteria that is phototrophic. All species use ammonia as a nitrogen source. Some strains are found only in sulfide-containing freshwater habitats exposed to light while others may occur in marine, estuarine, and freshwater environments.'),
  'D002845': NtO(MH='Chromatography', MS='Techniques used to separate mixtures of substances based on differences in the relative affinities of the substances for mobile and stationary phases. A mobile phase (fluid or gas) passes through a column containing a stationary phase of porous solid or liquid coated on a solid support. Usage is both analytical for small amounts and preparative for bulk amounts.'),
  'D002846': NtO(MH='Chromatography, Affinity', MS='A chromatographic technique that utilizes the ability of biological molecules, often ANTIBODIES, to bind to certain ligands specifically and reversibly. It is used in protein biochemistry. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002847': NtO(MH='Chromatography, Agarose', MS='A method of gel filtration chromatography using agarose, the non-ionic component of agar, for the separation of compounds with molecular weights up to several million.'),
  'D002848': NtO(MH='Chromatography, DEAE-Cellulose', MS='A type of ion exchange chromatography using diethylaminoethyl cellulose (DEAE-CELLULOSE) as a positively charged resin. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002849': NtO(MH='Chromatography, Gas', MS='Fractionation of a vaporized sample as a consequence of partition between a mobile gaseous phase and a stationary phase held in a column. Two types are gas-solid chromatography, where the fixed phase is a solid, and gas-liquid, in which the stationary phase is a nonvolatile liquid supported on an inert solid matrix.'),
  'D002850': NtO(MH='Chromatography, Gel', MS='Chromatography on non-ionic gels without regard to the mechanism of solute discrimination.'),
  'D002851': NtO(MH='Chromatography, High Pressure Liquid', MS='Liquid chromatographic techniques which feature high inlet pressures, high sensitivity, and high speed.'),
  'D002852': NtO(MH='Chromatography, Ion Exchange', MS='Separation technique in which the stationary phase consists of ion exchange resins. The resins contain loosely held small ions that easily exchange places with other small ions of like charge present in solutions washed over the resins.'),
  'D002853': NtO(MH='Chromatography, Liquid', MS='Chromatographic techniques in which the mobile phase is a liquid.'),
  'D002854': NtO(MH='Chromatography, Paper', MS='An analytical technique for resolution of a chemical mixture into its component compounds. Compounds are separated on an adsorbent paper (stationary phase) by their varied degree of solubility/mobility in the eluting solvent (mobile phase).'),
  'D002855': NtO(MH='Chromatography, Thin Layer', MS='Chromatography on thin layers of adsorbents rather than in columns. The adsorbent can be alumina, silica gel, silicates, charcoals, or cellulose. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002856': NtO(MH='Chromatophores', MS='The large pigment cells of fish, amphibia, reptiles and many invertebrates which actively disperse and aggregate their pigment granules. These cells include MELANOPHORES, erythrophores, xanthophores, leucophores and iridiophores. (In algae, chromatophores refer to CHLOROPLASTS. In phototrophic bacteria chromatophores refer to membranous organelles (BACTERIAL CHROMATOPHORES).)'),
  'D002857': NtO(MH='Chromium', MS='A trace element that plays a role in glucose metabolism. It has the atomic symbol Cr, atomic number 24, and atomic weight 52. According to the Fourth Annual Report on Carcinogens (NTP85-002,1985), chromium and some of its compounds have been listed as known carcinogens.'),
  'D002858': NtO(MH='Chromium Alloys', MS='Specific alloys not less than 85% chromium and nickel or cobalt, with traces of either nickel or cobalt, molybdenum, and other substances. They are used in partial dentures, orthopedic implants, etc.'),
  'D002859': NtO(MH='Chromium Isotopes', MS='Stable chromium atoms that have the same atomic number as the element chromium, but differ in atomic weight. Cr-50, 53, and 54 are stable chromium isotopes.'),
  'D002860': NtO(MH='Chromium Radioisotopes', MS='Unstable isotopes of chromium that decay or disintegrate emitting radiation. Cr atoms with atomic weights of 46-49, 51, 55, and 56 are radioactive chromium isotopes.'),
  'D002861': NtO(MH='Chromobacterium', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria occurring in soil and water. Its organisms are generally nonpathogenic, but some species do cause infections of mammals, including humans.'),
  'D002862': NtO(MH='Chromoblastomycosis', MS="Scaly papule or warty growth, caused by five fungi, that spreads as a result of satellite lesions affecting the foot or leg. The extremity may become swollen and, at its distal portion, covered with various nodular, tumorous, verrucous lesions that resemble cauliflower. In rare instances, the disease may begin on the hand or wrist and involve the entire upper extremity. (Arnold, Odom, and James, Andrew's Diseases of the Skin, 8th ed, p362)"),
  'D002863': NtO(MH='Chromogenic Compounds', MS='Colorless, endogenous or exogenous pigment precursors that may be transformed by biological mechanisms into colored compounds; used in biochemical assays and in diagnosis as indicators, especially in the form of enzyme substrates. Synonym: chromogens (not to be confused with pigment-synthesizing bacteria also called chromogens).'),
  'D002864': NtO(MH='Chromogranins', MS='A group of acidic proteins that are major components of SECRETORY GRANULES in the endocrine and neuroendocrine cells. They play important roles in the aggregation, packaging, sorting, and processing of secretory protein prior to secretion. They are cleaved to release biologically active peptides. There are various types of granins, usually classified by their sources.'),
  'D002865': NtO(MH='Chromomycins', MS='A complex of several closely related glycosidic antibiotics from Streptomyces griseus. The major component, CHROMOMYCIN A3, is used as a fluorescent stain of DNA where it attaches and inhibits RNA synthesis. It is also used as an antineoplastic agent, especially for solid tumors.'),
  'D002866': NtO(MH='Chromonar', MS='A coronary vasodilator agent.'),
  'D002867': NtO(MH='Chromones', MS=None),
  'D002868': NtO(MH='Chromosomal Proteins, Non-Histone', MS='Nucleoproteins, which in contrast to HISTONES, are acid insoluble. They are involved in chromosomal functions; e.g. they bind selectively to DNA, stimulate transcription resulting in tissue-specific RNA synthesis and undergo specific changes in response to various hormones or phytomitogens.'),
  'D002869': NtO(MH='Chromosome Aberrations', MS='Abnormal number or structure of chromosomes. Chromosome aberrations may result in CHROMOSOME DISORDERS.'),
  'D002871': NtO(MH='Chromosome Banding', MS='Staining of bands, or chromosome segments, allowing the precise identification of individual chromosomes or parts of chromosomes. Applications include the determination of chromosome rearrangements in malformation syndromes and cancer, the chemistry of chromosome segments, chromosome changes during evolution, and, in conjunction with cell hybridization studies, chromosome mapping.'),
  'D002872': NtO(MH='Chromosome Deletion', MS='Actual loss of portion of a chromosome.'),
  'D002873': NtO(MH='Chromosome Fragility', MS='Susceptibility of chromosomes to breakage leading to translocation; CHROMOSOME INVERSION; SEQUENCE DELETION; or other CHROMOSOME BREAKAGE related aberrations.'),
  'D002874': NtO(MH='Chromosome Mapping', MS='Any method used for determining the location of and relative distances between genes on a chromosome.'),
  'D002875': NtO(MH='Chromosomes', MS='In a prokaryotic cell or in the nucleus of a eukaryotic cell, a structure consisting of or containing DNA which carries the genetic information essential to the cell. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D002876': NtO(MH='Chromosomes, Bacterial', MS='Structures within the nucleus of bacterial cells consisting of or containing DNA, which carry genetic information essential to the cell.'),
  'D002877': NtO(MH='Chromosomes, Human', MS='Very long DNA molecules and associated proteins, HISTONES, and non-histone chromosomal proteins (CHROMOSOMAL PROTEINS, NON-HISTONE). Normally 46 chromosomes, including two sex chromosomes are found in the nucleus of human cells. They carry the hereditary information of the individual.'),
  'D002878': NtO(MH='Chromosomes, Human, Pair 1', MS='A specific pair of human chromosomes in group A (CHROMOSOMES, HUMAN, 1-3) of the human chromosome classification.'),
  'D002879': NtO(MH='Chromosomes, Human, Pair 10', MS='A specific pair of GROUP C CHROMOSOMES of the human chromosome classification.'),
  'D002880': NtO(MH='Chromosomes, Human, Pair 11', MS='A specific pair of GROUP C CHROMOSOMES of the human chromosome classification.'),
  'D002881': NtO(MH='Chromosomes, Human, Pair 12', MS='A specific pair of GROUP C CHROMOSOMES of the human chromosome classification.'),
  'D002882': NtO(MH='Chromosomes, Human, Pair 13', MS='A specific pair of GROUP D CHROMOSOMES of the human chromosome classification.'),
  'D002883': NtO(MH='Chromosomes, Human, Pair 14', MS='A specific pair of GROUP D CHROMOSOMES of the human chromosome classification.'),
  'D002884': NtO(MH='Chromosomes, Human, Pair 15', MS='A specific pair of GROUP D CHROMOSOMES of the human chromosome classification.'),
  'D002885': NtO(MH='Chromosomes, Human, Pair 16', MS='A specific pair of GROUP E CHROMOSOMES of the human chromosome classification.'),
  'D002886': NtO(MH='Chromosomes, Human, Pair 17', MS='A specific pair of GROUP E CHROMOSOMES of the human chromosome classification.'),
  'D002887': NtO(MH='Chromosomes, Human, Pair 18', MS='A specific pair of GROUP E CHROMOSOMES of the human chromosome classification.'),
  'D002888': NtO(MH='Chromosomes, Human, Pair 19', MS='A specific pair of GROUP F CHROMOSOMES of the human chromosome classification.'),
  'D002889': NtO(MH='Chromosomes, Human, Pair 2', MS='A specific pair of human chromosomes in group A (CHROMOSOMES, HUMAN, 1-3) of the human chromosome classification.'),
  'D002890': NtO(MH='Chromosomes, Human, Pair 20', MS='A specific pair of GROUP F CHROMOSOMES of the human chromosome classification.'),
  'D002891': NtO(MH='Chromosomes, Human, Pair 21', MS='A specific pair of GROUP G CHROMOSOMES of the human chromosome classification.'),
  'D002892': NtO(MH='Chromosomes, Human, Pair 22', MS='A specific pair of GROUP G CHROMOSOMES of the human chromosome classification.'),
  'D002893': NtO(MH='Chromosomes, Human, Pair 3', MS='A specific pair of human chromosomes in group A (CHROMOSOMES, HUMAN, 1-3) of the human chromosome classification.'),
  'D002894': NtO(MH='Chromosomes, Human, Pair 4', MS='A specific pair of GROUP B CHROMOSOMES of the human chromosome classification.'),
  'D002895': NtO(MH='Chromosomes, Human, Pair 5', MS='One of the two pairs of human chromosomes in the group B class (CHROMOSOMES, HUMAN, 4-5).'),
  'D002896': NtO(MH='Chromosomes, Human, Pair 6', MS='A specific pair GROUP C CHROMSOMES of the human chromosome classification.'),
  'D002897': NtO(MH='Chromosomes, Human, Pair 7', MS='A specific pair of GROUP C CHROMOSOMES of the human chromosome classification.'),
  'D002898': NtO(MH='Chromosomes, Human, Pair 8', MS='A specific pair of GROUP C CHROMOSOMES of the human chromosome classification.'),
  'D002899': NtO(MH='Chromosomes, Human, Pair 9', MS='A specific pair of GROUP C CHROMSOMES of the human chromosome classification.'),
  'D002900': NtO(MH='Chromosomes, Human, 1-3', MS='The large, metacentric human chromosomes, called group A in the human chromosome classification. This group consists of chromosome pairs 1, 2, and 3.'),
  'D002901': NtO(MH='Chromosomes, Human, 13-15', MS='The medium-sized, acrocentric human chromosomes, called group D in the human chromosome classification. This group consists of chromosome pairs 13, 14, and 15.'),
  'D002902': NtO(MH='Chromosomes, Human, 16-18', MS='The short, submetacentric human chromosomes, called group E in the human chromosome classification. This group consists of chromosome pairs 16, 17, and 18.'),
  'D002903': NtO(MH='Chromosomes, Human, 19-20', MS='The short, metacentric human chromosomes, called group F in the human chromosome classification. This group consists of chromosome pairs 19 and 20.'),
  'D002904': NtO(MH='Chromosomes, Human, 21-22 and Y', MS='The short, acrocentric human chromosomes, called group G in the human chromosome classification. This group consists of chromosome pairs 21 and 22 and the Y chromosome.'),
  'D002905': NtO(MH='Chromosomes, Human, 4-5', MS='The large, submetacentric human chromosomes, called group B in the human chromosome classification. This group consists of chromosome pairs 4 and 5.'),
  'D002906': NtO(MH='Chromosomes, Human, 6-12 and X', MS='The medium-sized, submetacentric human chromosomes, called group C in the human chromosome classification. This group consists of chromosome pairs 6, 7, 8, 9, 10, 11, and 12 and the X chromosome.'),
  'D002907': NtO(MH='Chronaxy', MS='The shortest duration of an electrical stimulus where the threshold amplitude is twice the rheobase - the minimum required for eliciting an ACTION POTENTIAL at any time period. It is a measure of the excitability of nerve or muscle tissue, and is characteristic of types and/or condition of the nerve or muscle cells in the tissue.'),
  'D002908': NtO(MH='Chronic Disease', MS='Diseases which have one or more of the following characteristics: they are permanent, leave residual disability, are caused by nonreversible pathological alteration, require special training of the patient for rehabilitation, or may be expected to require a long period of supervision, observation, or care. (Dictionary of Health Services Management, 2d ed)'),
  'D002909': NtO(MH='Chronobiology Phenomena', MS='Biological systems as affected by time. Aging, biological rhythms, and cyclic phenomena are included. Statistical, computer-aided mathematical procedures are used to describe, in mathematical terminology, various biological functions over time.'),
  'D002910': NtO(MH='Chronology as Topic', MS='Works about the temporal sequence of events.'),
  'D002911': NtO(MH='Chrysenes', MS='1,2-Benzphenanthrenes. POLYCYCLIC COMPOUNDS obtained from coal tar.'),
  'D002912': NtO(MH='Chrysosporium', MS='A mitosporic Onygenaceae fungal genus which causes adiaspiromycosis, a pulmonary mycosis of man and rodents. One of its teleomorphs is Ajellomyces.'),
  'D002913': NtO(MH='Chyle', MS='An opaque, milky-white fluid consisting mainly of emulsified fats that passes through the lacteals of the small intestines into the lymphatic system.'),
  'D002914': NtO(MH='Chylomicrons', MS='A class of lipoproteins that carry dietary CHOLESTEROL and TRIGLYCERIDES from the SMALL INTESTINE to the tissues. Their density (0.93-1.006 g/ml) is the same as that of VERY-LOW-DENSITY LIPOPROTEINS.'),
  'D002915': NtO(MH='Chylous Ascites', MS='Presence of milky lymph (CHYLE) in the PERITONEAL CAVITY, with or without infection.'),
  'D002916': NtO(MH='Chylothorax', MS='The presence of chyle in the thoracic cavity. (Dorland, 27th ed)'),
  'D002917': NtO(MH='Chymopapain', MS='A cysteine endopeptidase isolated from papaya latex. Preferential cleavage at glutamic and aspartic acid residues. EC 3.4.22.6.'),
  'D002918': NtO(MH='Chymotrypsin', MS='A serine endopeptidase secreted by the pancreas as its zymogen, CHYMOTRYPSINOGEN and carried in the pancreatic juice to the duodenum where it is activated by TRYPSIN. It selectively cleaves aromatic amino acids on the carboxyl side.'),
  'D002919': NtO(MH='Chymotrypsinogen', MS=None),
  'D002920': NtO(MH='Cialit', MS='Tissue preservative.'),
  'D002921': NtO(MH='Cicatrix', MS='The fibrous tissue that replaces normal tissue during the process of WOUND HEALING.'),
  'D002922': NtO(MH='Ciguatoxins', MS='Polycyclic ethers produced by Gambierdiscus (DINOFLAGELLATES) from gambiertoxins, which are ingested by fish which in turn may be ingested by humans who are susceptible to the CIGUATERA POISONING.'),
  'D002923': NtO(MH='Cilia', MS='Populations of thin, motile processes found covering the surface of ciliates (CILIOPHORA) or the free surface of the cells making up ciliated EPITHELIUM. Each cilium arises from a basic granule in the superficial layer of CYTOPLASM. The movement of cilia propels ciliates through the liquid in which they live. The movement of cilia on a ciliated epithelium serves to propel a surface layer of mucus or fluid. (King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D002924': NtO(MH='Ciliary Body', MS='A ring of tissue extending from the scleral spur to the ora serrata of the RETINA. It consists of the uveal portion and the epithelial portion. The ciliary muscle is in the uveal portion and the ciliary processes are in the epithelial portion.'),
  'D002925': NtO(MH='Ciliary Motility Disorders', MS='Conditions caused by abnormal CILIA movement in the body, usually causing KARTAGENER SYNDROME, chronic respiratory disorders, chronic SINUSITIS, and chronic OTITIS. Abnormal ciliary beating is likely due to defects in any of the 200 plus ciliary proteins, such as missing motor enzyme DYNEIN arms.'),
  'D002927': NtO(MH='Cimetidine', MS='A histamine congener, it competitively inhibits HISTAMINE binding to HISTAMINE H2 RECEPTORS. Cimetidine has a range of pharmacological actions. It inhibits GASTRIC ACID secretion, as well as PEPSIN and GASTRIN output.'),
  'D002928': NtO(MH='Cinanserin', MS='A serotonin antagonist with limited antihistaminic, anticholinergic, and immunosuppressive activity.'),
  'D002929': NtO(MH='Cinchona', MS='A genus of rubiaceous South American trees that yields the toxic CINCHONA ALKALOIDS from their bark; QUININE; QUINIDINE; chinconine, cinchonidine and others are used to treat MALARIA and CARDIAC ARRHYTHMIAS.'),
  'D002930': NtO(MH='Cinchona Alkaloids', MS='Alkaloids extracted from various species of Cinchona.'),
  'D002932': NtO(MH='Cineangiography', MS='Motion pictures of the passage of contrast medium through blood vessels.'),
  'D002933': NtO(MH='Cineradiography', MS='Motion picture study of successive images appearing on a fluoroscopic screen.'),
  'D002934': NtO(MH='Cinnamates', MS='Derivatives of cinnamic acid (the structural formula: phenyl-HC=CH-COO-), including its salts and esters.'),
  'D002935': NtO(MH='Cinnamomum zeylanicum', MS='The tree which is known for its bark which is sold as cinnamon. The oil contains about 65-80% cinnamaldehyde and 10% EUGENOL and many TERPENES.'),
  'D002936': NtO(MH='Cinnarizine', MS='A piperazine derivative having histamine H1-receptor and calcium-channel blocking activity with vasodilating and antiemetic properties but it induces PARKINSONIAN DISORDERS.'),
  'D002937': NtO(MH='Cinoxacin', MS='Synthetic antimicrobial related to OXOLINIC ACID and NALIDIXIC ACID and used in URINARY TRACT INFECTIONS.'),
  'D002938': NtO(MH='Ciona intestinalis', MS='Vase or tube shaped TUNICATES with a cosmopolitan distribution.'),
  'D002939': NtO(MH='Ciprofloxacin', MS='A broad-spectrum antimicrobial carboxyfluoroquinoline.'),
  'D002940': NtO(MH='Circadian Rhythm', MS='The regular recurrence, in cycles of about 24 hours, of biological processes or activities, such as sensitivity to drugs or environmental and physiological stimuli.'),
  'D002941': NtO(MH='Circle of Willis', MS='A polygonal anastomosis at the base of the brain formed by the internal carotid (CAROTID ARTERY, INTERNAL), proximal parts of the anterior, middle, and posterior cerebral arteries (ANTERIOR CEREBRAL ARTERY; MIDDLE CEREBRAL ARTERY; POSTERIOR CEREBRAL ARTERY), the anterior communicating artery and the posterior communicating arteries.'),
  'D002942': NtO(MH='Circular Dichroism', MS='A change from planar to elliptic polarization when an initially plane-polarized light wave traverses an optically active medium. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D002943': NtO(MH='Circulatory and Respiratory Physiological Phenomena', MS='Functional processes and properties characteristic of the BLOOD; CARDIOVASCULAR SYSTEM; and RESPIRATORY SYSTEM.'),
  'D002944': NtO(MH='Circumcision, Male', MS='Excision of the prepuce of the penis (FORESKIN) or part of it.'),
  'D002945': NtO(MH='Cisplatin', MS='An inorganic and water-soluble platinum complex. After undergoing hydrolysis, it reacts with DNA to produce both intra and interstrand crosslinks. These crosslinks appear to impair replication and transcription of DNA. The cytotoxicity of cisplatin correlates with cellular arrest in the G2 phase of the cell cycle.'),
  'D002946': NtO(MH='Cisterna Magna', MS='One of three principal openings in the SUBARACHNOID SPACE. They are also known as cerebellomedullary cistern, and collectively as cisterns.'),
  'D002947': NtO(MH='Cities', MS='A large or important municipality of a country, usually a major metropolitan center.'),
  'D002949': NtO(MH='Citraconic Anhydrides', MS='Methylmaleic anhydrides.'),
  'D002950': NtO(MH='Citrate (si)-Synthase', MS='Enzyme that catalyzes the first step of the tricarboxylic acid cycle (CITRIC ACID CYCLE). It catalyzes the reaction of oxaloacetate and acetyl CoA to form citrate and coenzyme A. This enzyme was formerly listed as EC 4.1.3.7.'),
  'D002951': NtO(MH='Citrates', MS='Derivatives of CITRIC ACID.'),
  'D002952': NtO(MH='Citric Acid Cycle', MS='A series of oxidative reactions in the breakdown of acetyl units derived from GLUCOSE; FATTY ACIDS; or AMINO ACIDS by means of tricarboxylic acid intermediates. The end products are CARBON DIOXIDE, water, and energy in the form of phosphate bonds.'),
  'D002953': NtO(MH='Citrinin', MS='Antibiotic and mycotoxin from Aspergillus niveus and Penicillium citrinum.'),
  'D002954': NtO(MH='Citrobacter', MS='A genus of gram-negative, rod-shaped enterobacteria that can use citrate as the sole source of carbon.'),
  'D002955': NtO(MH='Leucovorin', MS='The active metabolite of FOLIC ACID. Leucovorin is used principally as an antidote to FOLIC ACID ANTAGONISTS.'),
  'D002956': NtO(MH='Citrulline', MS=None),
  'D002957': NtO(MH='Citrus', MS='A plant genus of the family RUTACEAE. They bear the familiar citrus fruits including oranges, grapefruit, lemons, and limes. There are many hybrids which makes the nomenclature confusing.'),
  'D002958': NtO(MH='City Planning', MS='Comprehensive planning for the physical development of the city.'),
  'D002959': NtO(MH='Civil Defense', MS='Preventive emergency measures and programs designed to protect the individual or community in times of hostile attack.'),
  'D002960': NtO(MH='Civil Disorders', MS='Deliberate and planned acts of unlawful behavior engaged in by aggrieved segments of the population in seeking social change.'),
  'D002961': NtO(MH='Civil Rights', MS='Legal guarantee protecting the individual from attack on personal liberties, right to fair trial, right to vote, and freedom from discrimination on the basis of race, color, religion, sex, age, disability, or national origin. (from http://www.usccr.gov/ accessed 1/31/2003)'),
  'D002962': NtO(MH='Civilization', MS='The distinctly human attributes and attainments of a particular society.'),
  'D002963': NtO(MH='Cladosporium', MS='A mitosporic Loculoascomycetes fungal genus including some economically important plant parasites. Teleomorphs include Mycosphaerella and Venturia.'),
  'D002965': NtO(MH='Classification', MS='The systematic arrangement of entities in any field into categories classes based on common characteristics such as properties, morphology, subject matter, etc.'),
  'D002966': NtO(MH='Clathrin', MS='The main structural coat protein of COATED VESICLES which play a key role in the intracellular transport between membranous organelles. Each molecule of clathrin consists of three light chains (CLATHRIN LIGHT CHAINS) and three heavy chains (CLATHRIN HEAVY CHAINS) that form a structure called a triskelion. Clathrin also interacts with cytoskeletal proteins.'),
  'D002967': NtO(MH='Claviceps', MS='A genus of ascomycetous fungi, family Clavicipitaceae, order Hypocreales, parasitic on various grasses (POACEAE). The sclerotia contain several toxic alkaloids. Claviceps purpurea on rye causes ergotism.'),
  'D002968': NtO(MH='Clavicle', MS='A bone on the ventral side of the shoulder girdle, which in humans is commonly called the collar bone.'),
  'D002969': NtO(MH='Clavulanic Acids', MS='Acids, salts, and derivatives of clavulanic acid (C8H9O5N). They consist of those beta-lactam compounds that differ from penicillin in having the sulfur of the thiazolidine ring replaced by an oxygen. They have limited antibacterial action, but block bacterial beta-lactamase irreversibly, so that similar antibiotics are not broken down by the bacterial enzymes and therefore can exert their antibacterial effects.'),
  'D002970': NtO(MH='Cleavage Stage, Ovum', MS='The earliest developmental stage of a fertilized ovum (ZYGOTE) during which there are several mitotic divisions within the ZONA PELLUCIDA. Each cleavage or segmentation yields two BLASTOMERES of about half size of the parent cell. This cleavage stage generally covers the period up to 16-cell MORULA.'),
  'D002971': NtO(MH='Cleft Lip', MS='Congenital defect in the upper lip where the maxillary prominence fails to merge with the merged medial nasal prominences. It is thought to be caused by faulty migration of the mesoderm in the head region.'),
  'D002972': NtO(MH='Cleft Palate', MS='Congenital fissure of the soft and/or hard palate, due to faulty fusion.'),
  'D002973': NtO(MH='Cleidocranial Dysplasia', MS='Autosomal dominant syndrome in which there is delayed closing of the CRANIAL FONTANELLES; complete or partial absence of the collarbones (CLAVICLES); wide PUBIC SYMPHYSIS; short middle phalanges of the fifth fingers; and dental and vertebral anomalies.'),
  'D002974': NtO(MH='Clemastine', MS='A histamine H1 antagonist used as the hydrogen fumarate in hay fever, rhinitis, allergic skin conditions, and pruritus. It causes drowsiness.'),
  'D002976': NtO(MH='Clenbuterol', MS='A substituted phenylaminoethanol that has beta-2 adrenomimetic properties at very low doses. It is used as a bronchodilator in asthma.'),
  'D002977': NtO(MH='Clergy', MS='Persons ordained for religious duties, who serve as leaders and perform religious services.'),
  'D002979': NtO(MH='Climacteric', MS='Physiologic period, characterized by endocrine, somatic, and psychic changes with the termination of ovarian function in the female. It may also accompany the normal diminution of sexual activity in the male.'),
  'D002980': NtO(MH='Climate', MS='The longterm manifestations of WEATHER. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D002981': NtO(MH='Clindamycin', MS='An antibacterial agent that is a semisynthetic analog of LINCOMYCIN.'),
  'D002982': NtO(MH='Clinical Clerkship', MS='Undergraduate education programs for second- , third- , and fourth-year students in health sciences in which the students receive clinical training and experience in teaching hospitals or affiliated health centers.'),
  'D002983': NtO(MH='Clinical Competence', MS='The capability to perform acceptably those duties directly related to patient care.'),
  'D002984': NtO(MH='Clinical Laboratory Information Systems', MS='Information systems, usually computer-assisted, designed to store, manipulate, and retrieve information for planning, organizing, directing, and controlling administrative and clinical activities associated with the provision and utilization of clinical laboratory services.'),
  'D002985': NtO(MH='Clinical Protocols', MS='Precise and detailed plans for the study of a medical or biomedical problem and/or plans for a regimen of therapy.'),
  'D002986': NtO(MH='Clinical Trials as Topic', MS='Works about pre-planned studies of the safety, efficacy, or optimum dosage schedule (if appropriate) of one or more diagnostic, therapeutic, or prophylactic drugs, devices, or techniques selected according to predetermined criteria of eligibility and observed for predefined evidence of favorable and unfavorable effects. This concept includes clinical trials conducted both in the U.S. and in other countries.'),
  'D002987': NtO(MH='Clitoris', MS='An erectile structure homologous with the penis, situated beneath the anterior labial commissure, partially hidden between the anterior ends of the labia minora.'),
  'D002988': NtO(MH='Cloaca', MS='A dilated cavity extended caudally from the hindgut. In adult birds, reptiles, amphibians, and many fishes but few mammals, cloaca is a common chamber into which the digestive, urinary and reproductive tracts discharge their contents. In most mammals, cloaca gives rise to LARGE INTESTINE; URINARY BLADDER; and GENITALIA.'),
  'D002989': NtO(MH='Cloacin', MS='A bacteriocin produced by a plasmid that can occur in several bacterial strains. It is a basic protein of molecular weight 56,000 and exists in a complex with its immunity protein which protects the host bacterium from its effects.'),
  'D002990': NtO(MH='Clobetasol', MS='A derivative of PREDNISOLONE with high glucocorticoid activity and low mineralocorticoid activity. Absorbed through the skin faster than FLUOCINONIDE, it is used topically in treatment of PSORIASIS but may cause marked adrenocortical suppression.'),
  'D002991': NtO(MH='Clofazimine', MS='A fat-soluble riminophenazine dye used for the treatment of leprosy. It has been used investigationally in combination with other antimycobacterial drugs to treat Mycobacterium avium infections in AIDS patients. Clofazimine also has a marked anti-inflammatory effect and is given to control the leprosy reaction, erythema nodosum leprosum. (From AMA Drug Evaluations Annual, 1993, p1619)'),
  'D002992': NtO(MH='Clofenapate', MS='An oral hypolipemic agent primarily used in DOGS and RATS.'),
  'D002994': NtO(MH='Clofibrate', MS='A fibric acid derivative used in the treatment of HYPERLIPOPROTEINEMIA TYPE III and severe HYPERTRIGLYCERIDEMIA. (From Martindale, The Extra Pharmacopoeia, 30th ed, p986)'),
  'D002995': NtO(MH='Clofibric Acid', MS='An antilipemic agent that is the biologically active metabolite of CLOFIBRATE.'),
  'D002996': NtO(MH='Clomiphene', MS='A triphenyl ethylene stilbene derivative which is an estrogen agonist or antagonist depending on the target tissue. Note that ENCLOMIPHENE and ZUCLOMIPHENE are the (E) and (Z) isomers of Clomiphene respectively.'),
  'D002997': NtO(MH='Clomipramine', MS='A tricyclic antidepressant similar to IMIPRAMINE that selectively inhibits the uptake of serotonin in the brain. It is readily absorbed from the gastrointestinal tract and demethylated in the liver to form its primary active metabolite, desmethylclomipramine.'),
  'D002998': NtO(MH='Clonazepam', MS='An anticonvulsant used for several types of seizures, including myotonic or atonic seizures, photosensitive epilepsy, and absence seizures, although tolerance may develop. It is seldom effective in generalized tonic-clonic or partial seizures. The mechanism of action appears to involve the enhancement of GAMMA-AMINOBUTYRIC ACID receptor responses.'),
  'D002999': NtO(MH='Clone Cells', MS='A group of genetically identical cells all descended from a single common ancestral cell by mitosis in eukaryotes or by binary fission in prokaryotes. Clone cells also include populations of recombinant DNA molecules all carrying the same inserted sequence. (From King & Stansfield, Dictionary of Genetics, 4th ed)'),
  'D003000': NtO(MH='Clonidine', MS='An imidazoline sympatholytic agent that stimulates ALPHA-2 ADRENERGIC RECEPTORS and central IMIDAZOLINE RECEPTORS. It is commonly used in the management of HYPERTENSION.'),
  'D003001': NtO(MH='Cloning, Molecular', MS='The insertion of recombinant DNA molecules from prokaryotic and/or eukaryotic sources into a replicating vehicle, such as a plasmid or virus vector, and the introduction of the resultant hybrid molecules into recipient cells without altering the viability of those cells.'),
  'D003002': NtO(MH='Clonixin', MS='Anti-inflammatory analgesic.'),
  'D003003': NtO(MH='Clonorchiasis', MS='Infection of the biliary passages with CLONORCHIS SINENSIS, also called Opisthorchis sinensis. It may lead to inflammation of the biliary tract, proliferation of biliary epithelium, progressive portal fibrosis, and sometimes bile duct carcinoma. Extension to the liver may lead to fatty changes and cirrhosis. (From Dorland, 27th ed)'),
  'D003004': NtO(MH='Clonorchis sinensis', MS='A species of trematode flukes of the family Opisthorchidae. Many authorities consider this genus belonging to Opisthorchis. It is common in China and other Asiatic countries. Snails and fish are the intermediate hosts.'),
  'D003005': NtO(MH='Clopamide', MS='A sulfamoylbenzamide piperidine. It is considered a thiazide-like diuretic.'),
  'D003006': NtO(MH='Clopenthixol', MS='A thioxanthene with therapeutic actions similar to the phenothiazine antipsychotics. It is an antagonist at D1 and D2 dopamine receptors.'),
  'D003007': NtO(MH='Clopidol', MS='A very effective anticoccidial agent used in poultry.'),
  'D003008': NtO(MH='Cloprostenol', MS='A synthetic prostaglandin F2alpha analog. The compound has luteolytic effects and is used for the synchronization of estrus in cattle.'),
  'D003009': NtO(MH='Clorazepate Dipotassium', MS='A water-soluble benzodiazepine derivative effective in the treatment of anxiety. It has also muscle relaxant and anticonvulsant actions.'),
  'D003010': NtO(MH='Clorgyline', MS='An antidepressive agent and monoamine oxidase inhibitor related to PARGYLINE.'),
  'D003011': NtO(MH='Closing Volume', MS='The lung volume at which the dependent lung zones cease to ventilate presumably as a result of airway closure.'),
  'D003012': NtO(MH='Microbial Collagenase', MS='A metalloproteinase which degrades helical regions of native collagen to small fragments. Preferred cleavage is -Gly in the sequence -Pro-Xaa-Gly-Pro-. Six forms (or 2 classes) have been isolated from Clostridium histolyticum that are immunologically cross-reactive but possess different sequences and different specificities. Other variants have been isolated from Bacillus cereus, Empedobacter collagenolyticum, Pseudomonas marinoglutinosa, and species of Vibrio and Streptomyces. EC 3.4.24.3.'),
  'D003013': NtO(MH='Clostridium', MS='A genus of motile or nonmotile gram-positive bacteria of the family Clostridiaceae. Many species have been identified with some being pathogenic. They occur in water, soil, and in the intestinal tract of humans and lower animals.'),
  'D003014': NtO(MH='Clostridium botulinum', MS='A species of anaerobic, gram-positive, rod-shaped bacteria in the family Clostridiaceae that produces proteins with characteristic neurotoxicity. It is the etiologic agent of BOTULISM in humans, wild fowl, HORSES; and CATTLE. Seven subtypes (sometimes called antigenic types, or strains) exist, each producing a different botulinum toxin (BOTULINUM TOXINS). The organism and its spores are widely distributed in nature.'),
  'D003015': NtO(MH='Clostridium Infections', MS='Infections with bacteria of the genus CLOSTRIDIUM.'),
  'D003016': NtO(MH='Clostridium perfringens', MS='The most common etiologic agent of GAS GANGRENE. It is differentiable into several distinct types based on the distribution of twelve different toxins.'),
  'D003017': NtO(MH='Clostridium tetani', MS='The cause of TETANUS in humans and domestic animals. It is a common inhabitant of human and horse intestines as well as soil. Two components make up its potent exotoxin activity, a neurotoxin and a hemolytic toxin.'),
  'D003018': NtO(MH='Clot Retraction', MS='Retraction of a clot resulting from contraction of PLATELET pseudopods attached to FIBRIN strands. The retraction is dependent on the contractile protein thrombosthenin. Clot retraction is used as a measure of platelet function.'),
  'D003020': NtO(MH='Clothing', MS='Fabric or other material used to cover the body.'),
  'D003022': NtO(MH='Clotrimazole', MS='An imidazole derivative with a broad spectrum of antimycotic activity. It inhibits biosynthesis of the sterol ergostol, an important component of fungal CELL MEMBRANES. Its action leads to increased membrane permeability and apparent disruption of enzyme systems bound to the membrane.'),
  'D003023': NtO(MH='Cloxacillin', MS='A semi-synthetic antibiotic that is a chlorinated derivative of OXACILLIN.'),
  'D003024': NtO(MH='Clozapine', MS='A tricylic dibenzodiazepine, classified as an atypical antipsychotic agent. It binds several types of central nervous system receptors, and displays a unique pharmacological profile. Clozapine is a serotonin antagonist, with strong binding to 5-HT 2A/2C receptor subtype. It also displays strong affinity to several dopaminergic receptors, but shows only weak antagonism at the dopamine D2 receptor, a receptor commonly thought to modulate neuroleptic activity. Agranulocytosis is a major adverse effect associated with administration of this agent.'),
  'D003025': NtO(MH='Clubfoot', MS='A deformed foot in which the foot is plantarflexed, inverted, and adducted.'),
  'D003026': NtO(MH='Clupeine', MS=None),
  'D003027': NtO(MH='Cluster Headache', MS='A primary headache disorder that is characterized by severe, strictly unilateral PAIN which is orbital, supraorbital, temporal or in any combination of these sites, lasting 15-180 min. occurring 1 to 8 times a day. The attacks are associated with one or more of the following, all of which are ipsilateral: conjunctival injection, lacrimation, nasal congestion, rhinorrhea, facial SWEATING, eyelid EDEMA, and miosis. (International Classification of Headache Disorders, 2nd ed. Cephalalgia 2004: suppl 1)'),
  'D003028': NtO(MH='CME-Carbodiimide', MS=None),
  'D003029': NtO(MH='Coagulants', MS='Agents that cause clotting.'),
  'D003030': NtO(MH='Coagulase', MS='Enzymes that cause coagulation in plasma by forming a complex with human PROTHROMBIN. Coagulases are produced by certain STAPHYLOCOCCUS and YERSINIA PESTIS. Staphylococci produce two types of coagulase: Staphylocoagulase, a free coagulase that produces true clotting of plasma, and Staphylococcal clumping factor, a bound coagulase in the cell wall that induces clumping of cells in the presence of fibrinogen.'),
  'D003031': NtO(MH='Coal', MS='A natural fuel formed by partial decomposition of vegetable matter under certain environmental conditions.'),
  'D003032': NtO(MH='Coal Mining', MS='The practice of extracting COAL from the earth.'),
  'D003033': NtO(MH='Coal Tar', MS='A by-product of the destructive distillation of coal used as a topical antieczematic. It is an antipruritic and keratoplastic agent used also in the treatment of psoriasis and other skin conditions. Occupational exposure to soots, tars, and certain mineral oils is known to be carcinogenic according to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985) (Merck Index, 11th ed).'),
  'D003034': NtO(MH='Coated Pits, Cell-Membrane', MS='Specialized regions of the cell membrane composed of pits coated with a bristle covering made of the protein CLATHRIN. These pits are the entry route for macromolecules bound by cell surface receptors. The pits are then internalized into the cytoplasm to form the COATED VESICLES.'),
  'D003035': NtO(MH='Cobalt', MS='A trace element that is a component of vitamin B12. It has the atomic symbol Co, atomic number 27, and atomic weight 58.93. It is used in nuclear weapons, alloys, and pigments. Deficiency in animals leads to anemia; its excess in humans can lead to erythrocytosis.'),
  'D003036': NtO(MH='Cobalt Isotopes', MS='Stable cobalt atoms that have the same atomic number as the element cobalt, but differ in atomic weight. Co-59 is a stable cobalt isotope.'),
  'D003037': NtO(MH='Cobalt Radioisotopes', MS='Unstable isotopes of cobalt that decay or disintegrate emitting radiation. Co atoms with atomic weights of 54-64, except 59, are radioactive cobalt isotopes.'),
  'D003038': NtO(MH='Cobamides', MS=None),
  'D003039': NtO(MH='Cobra Neurotoxin Proteins', MS='Toxins, contained in cobra (Naja) venom that block cholinergic receptors; two specific proteins have been described, the small (short, Type I) and the large (long, Type II) which also exist in other Elapid venoms.'),
  'D003041': NtO(MH='Coca', MS='Any of several South American shrubs of the Erythroxylon genus (and family) that yield COCAINE; the leaves are chewed with alum for CNS stimulation.'),
  'D003042': NtO(MH='Cocaine', MS='An alkaloid ester extracted from the leaves of plants including coca. It is a local anesthetic and vasoconstrictor and is clinically used for that purpose, particularly in the eye, ear, nose, and throat. It also has powerful central nervous system effects similar to the amphetamines and is a drug of abuse. Cocaine, like amphetamines, acts by multiple mechanisms on brain catecholaminergic neurons; the mechanism of its reinforcing effects is thought to involve inhibition of dopamine uptake.'),
  'D003043': NtO(MH='Cocarcinogenesis', MS='The combination of two or more different factors in the production of cancer.'),
  'D003044': NtO(MH='Coccidia', MS='A subclass of protozoans commonly parasitic in the epithelial cells of the intestinal tract but also found in the liver and other organs. Its organisms are found in both vertebrates and higher invertebrates and comprise two orders: EIMERIIDA and EUCOCCIDIIDA.'),
  'D003045': NtO(MH='Coccidioides', MS='A mitosporic fungal genus which causes COCCIDIOIDOMYCOSIS.'),
  'D003046': NtO(MH='Coccidioidin', MS='A sterile solution containing the by-products of growth products of COCCIDIOIDES IMMITIS, injected intracutaneously as a test for COCCIDIOIDOMYCOSIS.'),
  'D003047': NtO(MH='Coccidioidomycosis', MS='Infection with a fungus of the genus COCCIDIOIDES, endemic to the SOUTHWESTERN UNITED STATES. It is sometimes called valley fever but should not be confused with RIFT VALLEY FEVER. Infection is caused by inhalation of airborne, fungal particles known as arthroconidia, a form of FUNGAL SPORES. A primary form is an acute, benign, self-limited respiratory infection. A secondary form is a virulent, severe, chronic, progressive granulomatous disease with systemic involvement. It can be detected by use of COCCIDIOIDIN.'),
  'D003048': NtO(MH='Coccidiosis', MS='Protozoan infection found in animals and man. It is caused by several different genera of COCCIDIA.'),
  'D003049': NtO(MH='Coccidiostats', MS='Agents useful in the treatment or prevention of COCCIDIOSIS in man or animals.'),
  'D003050': NtO(MH='Coccyx', MS='The last bone in the VERTEBRAL COLUMN in tailless primates considered to be a vestigial tail-bone consisting of three to five fused VERTEBRAE.'),
  'D003051': NtO(MH='Cochlea', MS='The part of the inner ear (LABYRINTH) that is concerned with hearing. It forms the anterior part of the labyrinth, as a snail-like structure that is situated almost horizontally anterior to the VESTIBULAR LABYRINTH.'),
  'D003052': NtO(MH='Cochlear Aqueduct', MS='A fine channel that passes through the TEMPORAL BONE near the SCALA TYMPANI (the basilar turn of the cochlea). The cochlear aqueduct connects the PERILYMPH-filled bony labyrinth to the SUBARACHNOID SPACE.'),
  'D003053': NtO(MH='Cochlear Duct', MS='A spiral tube that is firmly suspended in the bony shell-shaped part of the cochlea. This ENDOLYMPH-filled cochlear duct begins at the vestibule and makes 2.5 turns around a core of spongy bone (the modiolus) thus dividing the PERILYMPH-filled spiral canal into two channels, the SCALA VESTIBULI and the SCALA TYMPANI.'),
  'D003054': NtO(MH='Cochlear Implants', MS='Electronic hearing devices typically used for patients with normal outer and middle ear function, but defective inner ear function. In the COCHLEA, the hair cells (HAIR CELLS, VESTIBULAR) may be absent or damaged but there are residual nerve fibers. The device electrically stimulates the COCHLEAR NERVE to create sound sensation.'),
  'D003055': NtO(MH='Cochlear Microphonic Potentials', MS='The electric response of the cochlear hair cells to acoustic stimulation.'),
  'D003056': NtO(MH='Cochlear Nerve', MS='The cochlear part of the 8th cranial nerve (VESTIBULOCOCHLEAR NERVE). The cochlear nerve fibers originate from neurons of the SPIRAL GANGLION and project peripherally to cochlear hair cells and centrally to the cochlear nuclei (COCHLEAR NUCLEUS) of the BRAIN STEM. They mediate the sense of hearing.'),
  'D003057': NtO(MH='Cockayne Syndrome', MS='A syndrome characterized by multiple system abnormalities including DWARFISM; PHOTOSENSITIVITY DISORDERS; PREMATURE AGING; and HEARING LOSS. It is caused by mutations of a number of autosomal recessive genes encoding proteins that involve transcriptional-coupled DNA REPAIR processes. Cockayne syndrome is classified by the severity and age of onset. Type I (classical; CSA) is early childhood onset in the second year of life; type II (congenital; CSB) is early onset at birth with severe symptoms; type III (xeroderma pigmentosum; XP) is late childhood onset with mild symptoms.'),
  'D003058': NtO(MH='Cockroaches', MS='Insects of the order Dictyoptera comprising several families including Blaberidae, BLATTELLIDAE, Blattidae (containing the American cockroach PERIPLANETA americana), Cryptocercidae, and Polyphagidae.'),
  'D003059': NtO(MH='Cocos', MS='A plant genus of the family ARECACEAE. It is a tropical palm tree that yields a large, edible hard-shelled fruit from which oil and fiber are also obtained.'),
  'D003060': NtO(MH='Cod Liver Oil', MS='Oil obtained from fresh livers of the cod family, Gadidae. It is a source of VITAMIN A and VITAMIN D.'),
  'D003061': NtO(MH='Codeine', MS='An opioid analgesic related to MORPHINE but with less potent analgesic properties and mild sedative effects. It also acts centrally to suppress cough.'),
  'D003062': NtO(MH='Codon', MS='A set of three nucleotides in a protein coding sequence that specifies individual amino acids or a termination signal (CODON, TERMINATOR). Most codons are universal, but some organisms do not produce the transfer RNAs (RNA, TRANSFER) complementary to all codons. These codons are referred to as unassigned codons (CODONS, NONSENSE).'),
  'D003063': NtO(MH='Cnidaria', MS='A phylum of radially symmetrical invertebrates characterized by possession of stinging cells called nematocysts. It includes the classes ANTHOZOA; CUBOZOA; HYDROZOA, and SCYPHOZOA. Members carry CNIDARIAN VENOMS.'),
  'D003064': NtO(MH='Cnidarian Venoms', MS='Venoms from jellyfish; CORALS; SEA ANEMONES; etc. They contain hemo-, cardio-, dermo- , and neuro-toxic substances and probably ENZYMES. They include palytoxin, sarcophine, and anthopleurine.'),
  'D003065': NtO(MH='Coenzyme A', MS=None),
  'D003066': NtO(MH='Coenzyme A Ligases', MS='Enzymes that catalyze the formation of acyl-CoA derivatives. EC 6.2.1.'),
  'D003067': NtO(MH='Coenzymes', MS='Small molecules that are required for the catalytic function of ENZYMES. Many VITAMINS are coenzymes.'),
  'D003068': NtO(MH='Coercion', MS='The use of force or intimidation to obtain compliance.'),
  'D003069': NtO(MH='Coffee', MS='A beverage made from ground COFFEA beans (SEEDS) infused in hot water. It generally contains CAFFEINE and THEOPHYLLINE unless it is decaffeinated.'),
  'D003070': NtO(MH='Coformycin', MS='A ribonucleoside antibiotic synergist and adenosine deaminase inhibitor isolated from Nocardia interforma and Streptomyces kaniharaensis. It is proposed as an antineoplastic synergist and immunosuppressant.'),
  'D003071': NtO(MH='Cognition', MS='Intellectual or mental process whereby an organism obtains knowledge.'),
  'D003072': NtO(MH='Cognition Disorders', MS='Disorders characterized by disturbances in mental processes related to learning, thinking, reasoning, and judgment.'),
  'D003073': NtO(MH='Cognitive Dissonance', MS='Motivational state produced by inconsistencies between simultaneously held cognitions or between a cognition and behavior; e.g., smoking enjoyment and believing smoking is harmful are dissonant.'),
  'D003074': NtO(MH='Solitary Pulmonary Nodule', MS='A single lung lesion that is characterized by a small round mass of tissue, usually less than 1 cm in diameter, and can be detected by chest radiography. A solitary pulmonary nodule can be associated with neoplasm, tuberculosis, cyst, or other anomalies in the lung, the CHEST WALL, or the PLEURA.'),
  'D003075': NtO(MH='Coitus', MS='The sexual union of a male and a female, a term used for human only.'),
  'D003076': NtO(MH='Coitus Interruptus', MS='A contraceptive method whereby coitus is purposely interrupted in order to prevent EJACULATION of SEMEN into the VAGINA.'),
  'D003077': NtO(MH='Coke', MS='A residue of coal, left after dry (destructive) distillation, used as a fuel.'),
  'D003078': NtO(MH='Colchicine', MS='A major alkaloid from Colchicum autumnale L. and found also in other Colchicum species. Its primary therapeutic use is in the treatment of gout, but it has been used also in the therapy of familial Mediterranean fever (PERIODIC DISEASE).'),
  'D003079': NtO(MH='Colchicum', MS='A genus of poisonous, liliaceous plants. The roots (corms) of Colchicum autumnale, the fall crocus or meadow saffron, yield COLCHICINE, which is used as a biochemical tool and to treat gout. Other members of this genus yield saffron dye, flavoring agents, and aromatics.'),
  'D003080': NtO(MH='Cold Temperature', MS='An absence of warmth or heat or a temperature notably below an accustomed norm.'),
  'D003081': NtO(MH='Cold Climate', MS='A climate characterized by COLD TEMPERATURE for a majority of the time during the year.'),
  'D003082': NtO(MH='Colectomy', MS='Surgical resection of a portion of or the entire colon.'),
  'D003084': NtO(MH='Colestipol', MS='Highly crosslinked and insoluble basic anion exchange resin used as anticholesteremic. It may also may reduce triglyceride levels.'),
  'D003085': NtO(MH='Colic', MS='A clinical syndrome with intermittent abdominal pain characterized by sudden onset and cessation that is commonly seen in infants. It is usually associated with obstruction of the INTESTINES; of the CYSTIC DUCT; or of the URINARY TRACT.'),
  'D003086': NtO(MH='Bacteriocin Plasmids', MS='Plasmids encoding bacterial exotoxins (BACTERIOCINS).'),
  'D003087': NtO(MH='Colicins', MS='Bacteriocins elaborated by strains of Escherichia coli and related species. They are proteins or protein-lipopolysaccharide complexes lethal to other strains of the same species.'),
  'D003088': NtO(MH='Colinus', MS='A genus of QUAIL, in the family Odontophoridae, comprised of at least four different species of bobwhites.'),
  'D003089': NtO(MH='Colipases', MS='Colipase I and II, consisting of 94-95 and 84-85 amino acid residues, respectively, have been isolated from porcine pancreas. Their role is to prevent the inhibitory effect of bile salts on the lipase-catalyzed intraduodenal hydrolysis of dietary long-chain triglycerides.'),
  'D003090': NtO(MH='Coliphages', MS='Viruses whose host is Escherichia coli.'),
  'D003091': NtO(MH='Colistin', MS='Cyclic polypeptide antibiotic from Bacillus colistinus. It is composed of Polymyxins E1 and E2 (or Colistins A, B, and C) which act as detergents on cell membranes. Colistin is less toxic than Polymyxin B, but otherwise similar; the methanesulfonate is used orally.'),
  'D003092': NtO(MH='Colitis', MS='Inflammation of the COLON section of the large intestine (INTESTINE, LARGE), usually with symptoms such as DIARRHEA (often with blood and mucus), ABDOMINAL PAIN, and FEVER.'),
  'D003093': NtO(MH='Colitis, Ulcerative', MS='Inflammation of the COLON that is predominantly confined to the MUCOSA. Its major symptoms include DIARRHEA, rectal BLEEDING, the passage of MUCUS, and ABDOMINAL PAIN.'),
  'D003094': NtO(MH='Collagen', MS='A polypeptide substance comprising about one third of the total protein in mammalian organisms. It is the main constituent of SKIN; CONNECTIVE TISSUE; and the organic substance of bones (BONE AND BONES) and teeth (TOOTH).'),
  'D003095': NtO(MH='Collagen Diseases', MS='Historically, a heterogeneous group of acute and chronic diseases, including rheumatoid arthritis, systemic lupus erythematosus, progressive systemic sclerosis, dermatomyositis, etc. This classification was based on the notion that "collagen" was equivalent to "connective tissue", but with the present recognition of the different types of collagen and the aggregates derived from them as distinct entities, the term "collagen diseases" now pertains exclusively to those inherited conditions in which the primary defect is at the gene level and affects collagen biosynthesis, post-translational modification, or extracellular processing directly. (From Cecil Textbook of Medicine, 19th ed, p1494)'),
  'D003096': NtO(MH='Collapse Therapy', MS='Surgical treatment of pulmonary tuberculosis whereby the lung is totally or partially, temporarily or permanently, immobilized. The procedure was based on the popular concept that collapsing the affected portion of a tuberculous lung allowed the infected area to rest and thereby recover. At the beginning of the 20th century artificially induced pneumothorax (PNEUMOTHORAX, ARTIFICIAL) was popular. Later a variety of other techniques was used to encourage collapse of the infected portion of the lung: unilateral phrenic nerve division, PNEUMONOLYSIS, pneumoperitoneum (PNEUMOPERITONEUM, ARTIFICIAL), and THORACOPLASTY. Collapse therapy has declined since the advent of antitubercular chemotherapy. (Stedman, 25th ed; from Sabiston Jr, Textbook of Surgery, 14th ed, p1733-4)'),
  'D003097': NtO(MH='Collateral Circulation', MS='Maintenance of blood flow to an organ despite obstruction of a principal vessel. Blood flow is maintained through small vessels.'),
  'D003098': NtO(MH='Collective Bargaining', MS='The process of negotiation between representatives of an employee organization, association or union, and representatives of the employer.'),
  'D003099': NtO(MH='College Admission Test', MS='Test designed to identify students suitable for admission into a graduate or undergraduate curriculum.'),
  'D003100': NtO(MH="Colles' Fracture", MS='Fracture of the lower end of the radius in which the lower fragment is displaced posteriorly.'),
  'D003101': NtO(MH='Collodion', MS='A nitrocellulose solution in ether and alcohol. Collodion has a wide range of uses in industry including applications in the manufacture of photographic film, in fibers, in lacquers, and in engraving and lithography. In medicine it is used as a drug solvent and a wound sealant.'),
  'D003102': NtO(MH='Colloids', MS='Two-phase systems in which one is uniformly dispersed in another as particles small enough so they cannot be filtered or will not settle out. The dispersing or continuous phase or medium envelops the particles of the discontinuous phase. All three states of matter can form colloids among each other.'),
  'D003103': NtO(MH='Coloboma', MS='Congenital anomaly in which some of the structures of the eye are absent due to incomplete fusion of the fetal intraocular fissure during gestation.'),
  'D003104': NtO(MH='Colobus', MS="A genus of Old World monkeys, subfamily COLOBINAE, family CERCOPITHECIDAE, that inhabits the forests of Africa. It consists of eight species: C. angolensis (Angolan colobus), C. badius or C. rufomitratus (Red or Bay colobus), C. guereza (Guereza or Eastern black-and-white colobus), C. kirkii (Kirk's colobus), C. polykomos (King colobus or Western black-and-white colobus), C. satanas (Black colobus), and C. verus (Olive colobus). Some authors recognize Procolobus as a separate genus and then the olive colobus is recognized as the species P. verus."),
  'D003105': NtO(MH='Colombia', MS='A country in northern South America, bordering the Caribbean Sea, between Panama and Venezuela, and bordering the north Pacific Ocean, between Ecuador and Panama. The capital is Bogota.'),
  'D003106': NtO(MH='Colon', MS='The segment of LARGE INTESTINE between the CECUM and the RECTUM. It includes the ASCENDING COLON; the TRANSVERSE COLON; the DESCENDING COLON; and the SIGMOID COLON.'),
  'D003107': NtO(MH='Colorectal Surgery', MS='A surgical specialty concerned with the diagnosis and treatment of disorders and abnormalities of the COLON; RECTUM; and ANAL CANAL.'),
  'D003108': NtO(MH='Colonic Diseases', MS='Pathological processes in the COLON region of the large intestine (INTESTINE, LARGE).'),
  'D003109': NtO(MH='Colonic Diseases, Functional', MS='Chronic or recurrent colonic disorders without an identifiable structural or biochemical explanation. The widely recognized IRRITABLE BOWEL SYNDROME falls into this category.'),
  'D003110': NtO(MH='Colonic Neoplasms', MS='Tumors or cancer of the COLON.'),
  'D003111': NtO(MH='Colonic Polyps', MS='Discrete tissue masses that protrude into the lumen of the COLON. These POLYPS are connected to the wall of the colon either by a stalk, pedunculus, or by a broad base.'),
  'D003112': NtO(MH='Colonic Pseudo-Obstruction', MS="Functional obstruction of the COLON leading to MEGACOLON in the absence of obvious COLONIC DISEASES or mechanical obstruction. When this condition is acquired, acute, and coexisting with another medical condition (trauma, surgery, serious injuries or illness, or medication), it is called Ogilvie's syndrome."),
  'D003113': NtO(MH='Colonoscopy', MS='Endoscopic examination, therapy or surgery of the luminal surface of the colon.'),
  'D003114': NtO(MH='Colony-Forming Units Assay', MS='A cytologic technique for measuring the functional capacity of stem cells by assaying their activity.'),
  'D003115': NtO(MH='Colony-Stimulating Factors', MS='Glycoproteins found in a subfraction of normal mammalian plasma and urine. They stimulate the proliferation of bone marrow cells in agar cultures and the formation of colonies of granulocytes and/or macrophages. The factors include INTERLEUKIN-3; (IL-3); GRANULOCYTE COLONY-STIMULATING FACTOR; (G-CSF); MACROPHAGE COLONY-STIMULATING FACTOR; (M-CSF); and GRANULOCYTE-MACROPHAGE COLONY-STIMULATING FACTOR; (GM-CSF).'),
  'D003116': NtO(MH='Color', MS='The visually perceived property of objects created by absorption or reflection of specific wavelengths of light.'),
  'D003117': NtO(MH='Color Vision Defects', MS='Defects of color vision are mainly hereditary traits but can be secondary to acquired or developmental abnormalities in the CONES (RETINA). Severity of hereditary defects of color vision depends on the degree of mutation of the ROD OPSINS genes (on X CHROMOSOME and CHROMOSOME 3) that code the photopigments for red, green and blue.'),
  'D003118': NtO(MH='Color Perception', MS='Mental processing of chromatic signals (COLOR VISION) from the eye by the VISUAL CORTEX where they are converted into symbolic representations. Color perception involves numerous neurons, and is influenced not only by the distribution of wavelengths from the viewed object, but also by its background color and brightness contrast at its boundary.'),
  'D003119': NtO(MH='Color Perception Tests', MS='Type of vision test used to determine COLOR VISION DEFECTS.'),
  'D003120': NtO(MH='Colorado', MS='State bounded on the north by Wyoming and Nebraska, on the east by Kansas and Nebraska, on the south by New Mexico and Oklahoma, and on the west by Utah.'),
  'D003121': NtO(MH='Colorado Tick Fever', MS='A febrile illness characterized by chills, aches, vomiting, leukopenia, and sometimes encephalitis. It is caused by the COLORADO TICK FEVER VIRUS, a reovirus transmitted by the tick Dermacentor andersoni.'),
  'D003122': NtO(MH='Colorado tick fever virus', MS='A species of COLTIVIRUS transmitted by the tick DERMACENTOR andersonii and causing fever, chills, aching head and limbs, and often vomiting. It occurs in the northwestern United States, except the Pacific Coast.'),
  'D003123': NtO(MH='Colorectal Neoplasms, Hereditary Nonpolyposis', MS='A group of autosomal-dominant inherited diseases in which COLON CANCER arises in discrete adenomas. Unlike FAMILIAL POLYPOSIS COLI with hundreds of polyps, hereditary nonpolyposis colorectal neoplasms occur much later, in the fourth and fifth decades. HNPCC has been associated with germline mutations in mismatch repair (MMR) genes. It has been subdivided into Lynch syndrome I or site-specific colonic cancer, and LYNCH SYNDROME II which includes extracolonic cancer.'),
  'D003124': NtO(MH='Colorimetry', MS='Any technique by which an unknown color is evaluated in terms of standard colors. The technique may be visual, photoelectric, or indirect by means of spectrophotometry. It is used in chemistry and physics. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003125': NtO(MH='Colostomy', MS='The surgical construction of an opening between the colon and the surface of the body.'),
  'D003126': NtO(MH='Colostrum', MS='The thin, yellow, serous fluid secreted by the mammary glands during pregnancy and immediately postpartum before lactation begins. It consists of immunologically active substances, white blood cells, water, protein, fat, and carbohydrates.'),
  'D003127': NtO(MH='Colposcopy', MS='The examination, therapy or surgery of the cervix and vagina by means of a specially designed endoscope introduced vaginally.'),
  'D003128': NtO(MH='Coma', MS='A profound state of unconsciousness associated with depressed cerebral activity from which the individual cannot be aroused. Coma generally occurs when there is dysfunction or injury involving both cerebral hemispheres or the brain stem RETICULAR FORMATION.'),
  'D003129': NtO(MH='Comb and Wattles', MS='Fleshy and reddish outgrowth of skin tissue found on top of the head, attached to the sides of the head, and hanging from the mandible of birds such as turkeys and chickens.'),
  'D003130': NtO(MH='Combat Disorders', MS='Neurotic reactions to unusual, severe, or overwhelming military stress.'),
  'D003131': NtO(MH='Combined Modality Therapy', MS='The treatment of a disease or condition by several different means simultaneously or sequentially. Chemoimmunotherapy, RADIOIMMUNOTHERAPY, chemoradiotherapy, cryochemotherapy, and SALVAGE THERAPY are seen most frequently, but their combinations with each other and surgery are also used.'),
  'D003132': NtO(MH='Commerce', MS='The interchange of goods or commodities, especially on a large scale, between different countries or between populations within the same country. It includes trade (the buying, selling, or exchanging of commodities, whether wholesale or retail) and business (the purchase and sale of goods to make a profit). (From Random House Unabridged Dictionary, 2d ed, p411, p2005 & p283)'),
  'D003133': NtO(MH='Commission on Professional and Hospital Activities', MS='The non-profit, non-governmental organization which collects, processes, and distributes data on hospital use. Two programs of the Commission are the Professional Activity Study and the Medical Audit Program.'),
  'D003134': NtO(MH='Commitment of Mentally Ill', MS='Legal process required for the institutionalization of a patient with severe mental problems.'),
  'D003135': NtO(MH='Common Bile Duct', MS='The largest bile duct. It is formed by the junction of the CYSTIC DUCT and the COMMON HEPATIC DUCT.'),
  'D003137': NtO(MH='Common Bile Duct Diseases', MS='Diseases of the COMMON BILE DUCT including the AMPULLA OF VATER and the SPHINCTER OF ODDI.'),
  'D003138': NtO(MH='Common Bile Duct Neoplasms', MS='Tumor or cancer of the COMMON BILE DUCT including the AMPULLA OF VATER and the SPHINCTER OF ODDI.'),
  'D003139': NtO(MH='Common Cold', MS='A catarrhal disorder of the upper respiratory tract, which may be viral or a mixed infection. It generally involves a runny nose, nasal congestion, and sneezing.'),
  'D003140': NtO(MH='Communicable Disease Control', MS='Programs of surveillance designed to prevent the transmission of disease by any means from person to person or from animal to man.'),
  'D003141': NtO(MH='Communicable Diseases', MS='An illness caused by an infectious agent or its toxins that occurs through the direct or indirect transmission of the infectious agent or its products from an infected individual or via an animal, vector or the inanimate environment to a susceptible animal or human host.'),
  'D003142': NtO(MH='Communication', MS='The exchange or transmission of ideas, attitudes, or beliefs between individuals or groups.'),
  'D003143': NtO(MH='Communication Aids for Disabled', MS='Equipment that provides mentally or physically disabled persons with a means of communication. The aids include display boards, typewriters, cathode ray tubes, computers, and speech synthesizers. The output of such aids includes written words, artificial speech, language signs, Morse code, and pictures.'),
  'D003144': NtO(MH='Communication Barriers', MS='Those factors, such as language or sociocultural relationships, which interfere in the meaningful interpretation and transmission of ideas between individuals or groups.'),
  'D003145': NtO(MH='Communication Methods, Total', MS='Utilization of all available receptive and expressive modes for the purpose of achieving communication with the hearing impaired, such as gestures, postures, facial expression, types of voice, formal speech and non-speech systems, and simultaneous communication.'),
  'D003146': NtO(MH='Communications Media', MS='The means of interchanging or transmitting and receiving information. Historically the media were written: books, journals, newspapers, and other publications; in the modern age the media include, in addition, radio, television, computers, and information networks.'),
  'D003147': NtO(MH='Communication Disorders', MS='Disorders of verbal and nonverbal communication caused by receptive or expressive LANGUAGE DISORDERS, cognitive dysfunction (e.g., MENTAL RETARDATION), psychiatric conditions, and HEARING DISORDERS.'),
  'D003148': NtO(MH='Communism', MS='A totalitarian system of government in which a single authoritarian party controls state-owned means of production with the professed aim of establishing a classless society.'),
  'D003149': NtO(MH='Community Dentistry', MS='The practice of dentistry concerned with preventive as well as diagnostic and treatment programs in a circumscribed population.'),
  'D003150': NtO(MH='Community Health Workers', MS='Persons trained to assist professional health personnel in communicating with residents in the community concerning needs and availability of health services.'),
  'D003151': NtO(MH='Community Health Centers', MS='Facilities which administer the delivery of health care services to people living in a community or neighborhood.'),
  'D003152': NtO(MH='Community Health Nursing', MS='General and comprehensive nursing practice directed to individuals, families, or groups as it relates to and contributes to the health of a population or community. This is not an official program of a Public Health Department.'),
  'D003153': NtO(MH='Community Health Services', MS='Diagnostic, therapeutic and preventive health services provided for individuals in the community.'),
  'D003154': NtO(MH='Community Medicine', MS='A branch of medicine concerned with the total health of the individual within the home environment and in the community, and with the application of comprehensive care to the prevention and treatment of illness in the entire community.'),
  'D003155': NtO(MH='Community Mental Health Centers', MS='Facilities which administer the delivery of psychologic and psychiatric services to people living in a neighborhood or community.'),
  'D003156': NtO(MH='Community Mental Health Services', MS='Diagnostic, therapeutic and preventive mental health services provided for individuals in the community.'),
  'D003157': NtO(MH='Community Pharmacy Services', MS='Total pharmaceutical services provided to the public through community pharmacies.'),
  'D003158': NtO(MH='Community Psychiatry', MS='Branch of psychiatry concerned with the provision and delivery of a coordinated program of mental health care to a specified population. The foci included in this concept are: all social, psychological and physical factors related to etiology, prevention, and maintaining positive mental health in the community.'),
  'D003159': NtO(MH='Community-Institutional Relations', MS='The interactions between members of a community and representatives of the institutions within that community.'),
  'D003160': NtO(MH='Comparative Study', MS='Comparison of outcomes, results, responses, etc for different techniques, therapeutic approaches or other inputs.'),
  'D003161': NtO(MH='Compartment Syndromes', MS='Conditions in which increased pressure within a limited space compromises the BLOOD CIRCULATION and function of tissue within that space. Some of the causes of increased pressure are TRAUMA, tight dressings, HEMORRHAGE, and exercise. Sequelae include nerve compression (NERVE COMPRESSION SYNDROMES); PARALYSIS; and ISCHEMIC CONTRACTURE.'),
  'D003162': NtO(MH='Competency-Based Education', MS='Educational programs designed to ensure that students attain prespecified levels of competence in a given field or training activity. Emphasis is on achievement or specified objectives.'),
  'D003163': NtO(MH='Competitive Behavior', MS='The direct struggle between individuals for environmental necessities or for a common goal.'),
  'D003164': NtO(MH='Competitive Bidding', MS='Pricing statements presented by more than one party for the purpose of securing a contract.'),
  'D003165': NtO(MH='Complement System Proteins', MS='Serum glycoproteins participating in the host defense mechanism of COMPLEMENT ACTIVATION that creates the COMPLEMENT MEMBRANE ATTACK COMPLEX. Included are glycoproteins in the various pathways of complement activation (CLASSICAL COMPLEMENT PATHWAY; ALTERNATIVE COMPLEMENT PATHWAY; and LECTIN COMPLEMENT PATHWAY).'),
  'D003166': NtO(MH='Complement Activating Enzymes', MS='Enzymes that activate one or more COMPLEMENT PROTEINS in the complement system leading to the formation of the COMPLEMENT MEMBRANE ATTACK COMPLEX, an important response in host defense. They are enzymes in the various COMPLEMENT ACTIVATION pathways.'),
  'D003167': NtO(MH='Complement Activation', MS='The sequential activation of serum COMPLEMENT PROTEINS to create the COMPLEMENT MEMBRANE ATTACK COMPLEX. Factors initiating complement activation include ANTIGEN-ANTIBODY COMPLEXES, microbial ANTIGENS, or cell surface POLYSACCHARIDES.'),
  'D003168': NtO(MH='Complement Fixation Tests', MS='Serologic tests based on inactivation of complement by the antigen-antibody complex (stage 1). Binding of free complement can be visualized by addition of a second antigen-antibody system such as red cells and appropriate red cell antibody (hemolysin) requiring complement for its completion (stage 2). Failure of the red cells to lyse indicates that a specific antigen-antibody reaction has taken place in stage 1. If red cells lyse, free complement is present indicating no antigen-antibody reaction occurred in stage 1.'),
  'D003169': NtO(MH='Complement Inactivator Proteins', MS='Serum proteins that negatively regulate the cascade process of COMPLEMENT ACTIVATION. Uncontrolled complement activation and resulting cell lysis is potentially dangerous for the host. The complement system is tightly regulated by inactivators that accelerate the decay of intermediates and certain cell surface receptors.'),
  'D003170': NtO(MH='Complement Pathway, Alternative', MS='Complement activation initiated by the interaction of microbial ANTIGENS with COMPLEMENT C3B. When COMPLEMENT FACTOR B binds to the membrane-bound C3b, COMPLEMENT FACTOR D cleaves it to form alternative C3 CONVERTASE (C3BBB) which, stabilized by COMPLEMENT FACTOR P, is able to cleave multiple COMPLEMENT C3 to form alternative C5 CONVERTASE (C3BBB3B) leading to cleavage of COMPLEMENT C5 and the assembly of COMPLEMENT MEMBRANE ATTACK COMPLEX.'),
  'D003171': NtO(MH='Complement Pathway, Classical', MS='Complement activation initiated by the binding of COMPLEMENT C1 to ANTIGEN-ANTIBODY COMPLEXES at the COMPLEMENT C1Q subunit. This leads to the sequential activation of COMPLEMENT C1R and COMPLEMENT C1S subunits. Activated C1s cleaves COMPLEMENT C4 and COMPLEMENT C2 forming the membrane-bound classical C3 CONVERTASE (C4B2A) and the subsequent C5 CONVERTASE (C4B2A3B) leading to cleavage of COMPLEMENT C5 and the assembly of COMPLEMENT MEMBRANE ATTACK COMPLEX.'),
  'D003172': NtO(MH='Complement C1', MS='The first complement component to act in the activation of CLASSICAL COMPLEMENT PATHWAY. It is a calcium-dependent trimolecular complex made up of three subcomponents: COMPLEMENT C1Q; COMPLEMENT C1R; and COMPLEMENT C1S at 1:2:2 ratios. When the intact C1 binds to at least two antibodies (involving C1q), C1r and C1s are sequentially activated, leading to subsequent steps in the cascade of COMPLEMENT ACTIVATION.'),
  'D003173': NtO(MH='Complement C1s', MS='A 77-kDa subcomponent of complement C1, encoded by gene C1S, is a SERINE PROTEASE existing as a proenzyme (homodimer) in the intact complement C1 complex. Upon the binding of COMPLEMENT C1Q to antibodies, the activated COMPLEMENT C1R cleaves C1s into two chains, A (heavy) and B (light, the serine protease), linked by disulfide bonds yielding the active C1s. The activated C1s, in turn, cleaves COMPLEMENT C2 and COMPLEMENT C4 to form C4b2a (CLASSICAL C3 CONVERTASE).'),
  'D003174': NtO(MH='Complement C1 Inactivator Proteins', MS='Serum proteins that inhibit, antagonize, or inactivate COMPLEMENT C1 or its subunits.'),
  'D003175': NtO(MH='Complement C2', MS='A component of the CLASSICAL COMPLEMENT PATHWAY. C2 is cleaved by activated COMPLEMENT C1S into COMPLEMENT C2B and COMPLEMENT C2A. C2a, the COOH-terminal fragment containing a SERINE PROTEASE, combines with COMPLEMENT C4B to form C4b2a (CLASSICAL PATHWAY C3 CONVERTASE) and subsequent C4b2a3b (CLASSICAL PATHWAY C5 CONVERTASE).'),
  'D003176': NtO(MH='Complement C3', MS='A glycoprotein that is central in both the classical and the alternative pathway of COMPLEMENT ACTIVATION. C3 can be cleaved into COMPLEMENT C3A and COMPLEMENT C3B, spontaneously at low level or by C3 CONVERTASE at high level. The smaller fragment C3a is an ANAPHYLATOXIN and mediator of local inflammatory process. The larger fragment C3b binds with C3 convertase to form C5 convertase.'),
  'D003178': NtO(MH='Complement C3 Nephritic Factor', MS='An IgG autoantibody against the ALTERNATIVE PATHWAY C3 CONVERTASE, found in serum of patients with MESANGIOCAPILLARY GLOMERULONEPHRITIS. The binding of this autoantibody to C3bBb stabilizes the enzyme thus reduces the actions of C3b inactivators (COMPLEMENT FACTOR H; COMPLEMENT FACTOR I). This abnormally stabilized enzyme induces a continuous COMPLEMENT ACTIVATION and generation of C3b thereby promoting the assembly of MEMBRANE ATTACK COMPLEX and cytolysis.'),
  'D003179': NtO(MH='Complement C3b', MS='The larger fragment generated from the cleavage of COMPLEMENT C3 by C3 CONVERTASE. It is a constituent of the ALTERNATIVE PATHWAY C3 CONVERTASE (C3bBb), and COMPLEMENT C5 CONVERTASES in both the classical (C4b2a3b) and the alternative (C3bBb3b) pathway. C3b participates in IMMUNE ADHERENCE REACTION and enhances PHAGOCYTOSIS. It can be inactivated (iC3b) or cleaved by various proteases to yield fragments such as COMPLEMENT C3C; COMPLEMENT C3D; C3e; C3f; and C3g.'),
  'D003180': NtO(MH='Complement C3b Inactivator Proteins', MS='Endogenous proteins that inhibit or inactivate COMPLEMENT C3B. They include COMPLEMENT FACTOR H and COMPLEMENT FACTOR I (C3b/C4b inactivator). They cleave or promote the cleavage of C3b into inactive fragments, and thus are important in the down-regulation of COMPLEMENT ACTIVATION and its cytolytic sequence.'),
  'D003181': NtO(MH='Complement C4', MS='A glycoprotein that is important in the activation of CLASSICAL COMPLEMENT PATHWAY. C4 is cleaved by the activated COMPLEMENT C1S into COMPLEMENT C4A and COMPLEMENT C4B.'),
  'D003182': NtO(MH='Complement C5', MS='C5 plays a central role in both the classical and the alternative pathway of COMPLEMENT ACTIVATION. C5 is cleaved by C5 CONVERTASE into COMPLEMENT C5A and COMPLEMENT C5B. The smaller fragment C5a is an ANAPHYLATOXIN and mediator of inflammatory process. The major fragment C5b binds to the membrane initiating the spontaneous assembly of the late complement components, C5-C9, into the MEMBRANE ATTACK COMPLEX.'),
  'D003183': NtO(MH='Complement C6', MS='A 105-kDa serum glycoprotein with significant homology to the other late complement components, C7-C9. It is a polypeptide chain cross-linked by 32 disulfide bonds. C6 is the next complement component to bind to the membrane-bound COMPLEMENT C5B in the assembly of MEMBRANE ATTACK COMPLEX. It is encoded by gene C6.'),
  'D003184': NtO(MH='Complement C7', MS='A 93-kDa serum glycoprotein encoded by C7 gene. It is a polypeptide chain with 28 disulfide bridges. In the formation of MEMBRANE ATTACK COMPLEX; C7 is the next component to bind the C5b-6 complex forming a trimolecular complex C5b-7 which is lipophilic, resembles an integral membrane protein, and serves as an anchor for the late complement components, C8 and C9.'),
  'D003185': NtO(MH='Complement C8', MS='A 150-kDa serum glycoprotein composed of three subunits with each encoded by a different gene (C8A; C8B; and C8G). This heterotrimer contains a disulfide-linked C8alpha-C8gamma heterodimer and a noncovalently associated C8beta chain. C8 is the next component to bind the C5-7 complex forming C5b-8 that binds COMPLEMENT C9 and acts as a catalyst in the polymerization of C9.'),
  'D003186': NtO(MH='Complement C9', MS='A 63-kDa serum glycoprotein encoded by gene C9. Monomeric C9 (mC9) binds the C5b-8 complex to form C5b-9 which catalyzes the polymerization of C9 forming C5b-p9 (MEMBRANE ATTACK COMPLEX) and transmembrane channels leading to lysis of the target cell. Patients with C9 deficiency suffer from recurrent bacterial infections.'),
  'D003187': NtO(MH='Compliance', MS='Distensibility measure of a chamber such as the lungs (LUNG COMPLIANCE) or bladder. Compliance is expressed as a change in volume per unit change in pressure.'),
  'D003188': NtO(MH='Composite Resins', MS='Synthetic resins, containing an inert filler, that are widely used in dentistry.'),
  'D003189': NtO(MH='p-Methoxy-N-methylphenethylamine', MS='A potent mast cell degranulator. It is involved in histamine release.'),
  'D003190': NtO(MH='Comprehensive Dental Care', MS='Providing for the full range of dental health services for diagnosis, treatment, follow-up, and rehabilitation of patients.'),
  'D003191': NtO(MH='Comprehensive Health Care', MS='Providing for the full range of personal health services for diagnosis, treatment, follow-up and rehabilitation of patients.'),
  'D003192': NtO(MH='Compulsive Behavior', MS='The behavior of performing an act persistently and repetitively without it leading to reward or pleasure. The act is usually a small, circumscribed behavior, almost ritualistic, yet not pathologically disturbing. Examples of compulsive behavior include twirling of hair, checking something constantly, not wanting pennies in change, straightening tilted pictures, etc.'),
  'D003193': NtO(MH='Compulsive Personality Disorder', MS="Disorder characterized by an emotionally constricted manner that is unduly conventional, serious, formal, and stingy, by preoccupation with trivial details, rules, order, organization, schedules, and lists, by stubborn insistence on having things one's own way without regard for the effects on others, by poor interpersonal relationships, and by indecisiveness due to fear of making mistakes."),
  'D003194': NtO(MH='Computer-Assisted Instruction', MS='A self-learning technique, usually online, involving interaction of the student with programmed instructional materials.'),
  'D003195': NtO(MH='Computer Communication Networks', MS='A system containing any combination of computers, computer terminals, printers, audio or visual display devices, or telephones interconnected by telecommunications equipment or cables: used to transmit or receive information. (Random House Unabridged Dictionary, 2d ed)'),
  'D003196': NtO(MH='Computer Graphics', MS='The process of pictorial communication, between human and computers, in which the computer input and output have the form of charts, drawings, or other appropriate pictorial representation.'),
  'D003197': NtO(MH='Computer Literacy', MS='Familiarity and comfort in using computers efficiently.'),
  'D003198': NtO(MH='Computer Simulation', MS='Computer-based representation of physical systems and phenomena such as chemical processes.'),
  'D003199': NtO(MH='Computer Systems', MS='Systems composed of a computer or computers, peripheral equipment, such as disks, printers, and terminals, and telecommunications capabilities.'),
  'D003200': NtO(MH='Computer User Training', MS='Process of teaching a person to interact and communicate with a computer.'),
  'D003201': NtO(MH='Computers', MS='Programmable electronic devices designed to accept data, perform prescribed mathematical and logical operations at high speed, and display the results of these operations.'),
  'D003202': NtO(MH='Computers, Analog', MS='Computers in which quantities are represented by physical variables; problem parameters are translated into equivalent mechanical or electrical circuits as an analog for the physical phenomenon being investigated. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003203': NtO(MH='Computers, Hybrid', MS='Computers that combine the functions of analog and digital computers. (Sippl, Computer Dictionary, 4th ed)'),
  'D003204': NtO(MH='Computers, Mainframe', MS='Large computers in both size and capacity.'),
  'D003205': NtO(MH='Computing Methodologies', MS='Computer-assisted analysis and processing of problems in a particular area.'),
  'D003207': NtO(MH='Conalbumin', MS="A glycoprotein albumin from hen's egg white with strong iron-binding affinity."),
  'D003208': NtO(MH='Concanavalin A', MS='A MANNOSE/GLUCOSE binding lectin isolated from the jack bean (Canavalia ensiformis). It is a potent mitogen used to stimulate cell proliferation in lymphocytes, primarily T-lymphocyte, cultures.'),
  'D003209': NtO(MH='Concentration Camps', MS='Facilities in which WARFARE or political prisoners are confined.'),
  'D003210': NtO(MH='Concept Formation', MS='A cognitive process involving the formation of ideas generalized from the knowledge of qualities, aspects, and relations of objects.'),
  'D003211': NtO(MH='Concurrent Review', MS='Review of the medical necessity of hospital or other health facility admissions, upon or within a short time following an admission, and periodic review of services provided during the course of treatment.'),
  'D003212': NtO(MH='Condiments', MS='Aromatic substances added to food before or after cooking to enhance its flavor. These are usually of vegetable origin.'),
  'D003213': NtO(MH='Conditioning, Psychological', MS='Simple form of learning involving the formation, strengthening, or weakening of an association between a stimulus and a response.'),
  'D003214': NtO(MH='Conditioning, Classical', MS='Learning that takes place when a conditioned stimulus is paired with an unconditioned stimulus.'),
  'D003215': NtO(MH='Conditioning, Eyelid', MS='Reflex closure of the eyelid occurring as a result of classical conditioning.'),
  'D003216': NtO(MH='Conditioning, Operant', MS='Learning situations in which the sequence responses of the subject are instrumental in producing reinforcement. When the correct response occurs, which involves the selection from among a repertoire of responses, the subject is immediately reinforced.'),
  'D003217': NtO(MH='Conductometry', MS='Determination of the quantity of a material present in a mixture by measurement of its effect on the electrical conductivity of the mixture. (Webster, 3d ed)'),
  'D003218': NtO(MH='Condylomata Acuminata', MS='Sexually transmitted form of anogenital warty growth caused by the human papillomaviruses.'),
  'D003219': NtO(MH='Confidentiality', MS='The privacy of information and its protection against unauthorized disclosure.'),
  'D003220': NtO(MH='Conflict, Psychological', MS="The internal individual struggle resulting from incompatible or opposing needs, drives, or external and internal demands. In group interactions, competitive or opposing action of incompatibles: antagonistic state or action (as of divergent ideas, interests, or persons). (from Merriam-Webster's Collegiate Dictionary, 10th ed)"),
  'D003221': NtO(MH='Confusion', MS='A mental state characterized by bewilderment, emotional disturbance, lack of clear thinking, and perceptual disorientation.'),
  'D003223': NtO(MH='Congo', MS='A republic in central Africa lying between GABON and DEMOCRATIC REPUBLIC OF THE CONGO and south of Cameroon. Its capital is Brazzaville.'),
  'D003224': NtO(MH='Congo Red', MS='An acid dye used in testing for hydrochloric acid in gastric contents. It is also used histologically to test for AMYLOIDOSIS.'),
  'D003225': NtO(MH='Hemorrhagic Fever Virus, Crimean-Congo', MS='A species of NAIROVIRUS of the family BUNYAVIRIDAE. It is primarily transmitted by ticks and causes a severe, often fatal disease in humans.'),
  'D003226': NtO(MH='Congresses as Topic', MS='Works about conferences, conventions or formal meetings usually attended by delegates representing a special field of interest.'),
  'D003227': NtO(MH='Conjugation, Genetic', MS='A parasexual process in BACTERIA; ALGAE; FUNGI; and ciliate EUKARYOTA for achieving exchange of chromosome material during fusion of two cells. In bacteria, this is a uni-directional transfer of genetic material; in protozoa it is a bi-directional exchange. In algae and fungi, it is a form of sexual reproduction, with the union of male and female gametes.'),
  'D003228': NtO(MH='Conjunctiva', MS='The mucous membrane that covers the posterior surface of the eyelids and the anterior pericorneal surface of the eyeball.'),
  'D003229': NtO(MH='Conjunctival Diseases', MS='Diseases involving the CONJUNCTIVA.'),
  'D003230': NtO(MH='Conjunctival Neoplasms', MS='Tumors or cancer of the CONJUNCTIVA.'),
  'D003231': NtO(MH='Conjunctivitis', MS='INFLAMMATION of the CONJUNCTIVA.'),
  'D003232': NtO(MH='Conjunctivitis, Acute Hemorrhagic', MS='A highly contagious disease characterized by subconjunctival hemorrhage, sudden swelling of the eyelids and congestion, redness, and pain in the eye. Epidemic conjunctivitis caused by Enterovirus 70 (EV-70) was first described in Africa in 1969. It is caused also by Coxsackievirus A24 variant (CA24v). Epidemics by this organism have appeared most frequently in Asia.'),
  'D003233': NtO(MH='Conjunctivitis, Allergic', MS='Conjunctivitis due to hypersensitivity to various allergens.'),
  'D003234': NtO(MH='Conjunctivitis, Bacterial', MS='Purulent infections of the conjunctiva by several species of gram-negative, gram-positive, or acid-fast organisms. Some of the more commonly found genera causing conjunctival infections are Haemophilus, Streptococcus, Neisseria, and Chlamydia.'),
  'D003235': NtO(MH='Conjunctivitis, Inclusion', MS='An infection of the eyes characterized by the presence in conjunctival epithelial cells of inclusion bodies indistinguishable from those of trachoma. It is acquired by infants during birth and by adults from swimming pools. The etiological agent is CHLAMYDIA TRACHOMATIS whose natural habitat appears to be the genito-urinary tract. Inclusion conjunctivitis is a less severe disease than trachoma and usually clears up spontaneously.'),
  'D003236': NtO(MH='Conjunctivitis, Viral', MS='Inflammation, often mild, of the conjunctiva caused by a variety of viral agents. Conjunctival involvement may be part of a systemic infection.'),
  'D003237': NtO(MH='Connecticut', MS='State bounded on the north by Massachusetts, on the east by Rhode Island, on the south by Long Island Sound, and on the west by New York.'),
  'D003238': NtO(MH='Connective Tissue', MS='Tissue that supports and binds other tissues. It consists of CONNECTIVE TISSUE CELLS embedded in a large amount of EXTRACELLULAR MATRIX.'),
  'D003239': NtO(MH='Connective Tissue Cells', MS='A group of cells that includes FIBROBLASTS, cartilage cells, ADIPOCYTES, smooth muscle cells, and bone cells.'),
  'D003240': NtO(MH='Connective Tissue Diseases', MS='A heterogeneous group of disorders, some hereditary, others acquired, characterized by abnormal structure or function of one or more of the elements of connective tissue, i.e., collagen, elastin, or the mucopolysaccharides.'),
  'D003241': NtO(MH='Consanguinity', MS='The magnitude of INBREEDING in humans.'),
  'D003242': NtO(MH='Conscience', MS="The cognitive and affective processes which constitute an internalized moral governor over an individual's moral conduct."),
  'D003243': NtO(MH='Consciousness', MS='Sense of awareness of self and of the environment.'),
  'D003244': NtO(MH='Consciousness Disorders', MS='Organic mental disorders in which there is impairment of the ability to maintain awareness of self and environment and to respond to environmental stimuli. Dysfunction of the cerebral hemispheres or brain stem RETICULAR FORMATION may result in this condition.'),
  'D003245': NtO(MH='Consensus Development Conferences as Topic', MS='Works that are about the concept of consensus development conferences, such as their general design or a means of scientific communication.'),
  'D003246': NtO(MH='Conservation of Energy Resources', MS='Planned management, use, and preservation of energy resources.'),
  'D003247': NtO(MH='Conservation of Natural Resources', MS='The protection, preservation, restoration, and rational use of all resources in the total environment.'),
  'D003248': NtO(MH='Constipation', MS='Infrequent or difficult evacuation of FECES. These symptoms are associated with a variety of causes, including low DIETARY FIBER intake, emotional or nervous disturbances, systemic and structural disorders, drug-induced aggravation, and infections.'),
  'D003249': NtO(MH='Constitution and Bylaws', MS='The fundamental principles and laws adopted by an organization for the regulation and governing of its affairs.'),
  'D003250': NtO(MH='Constriction', MS='The act of constricting.'),
  'D003251': NtO(MH='Constriction, Pathologic', MS="The condition of an anatomical structure's being constricted beyond normal dimensions."),
  'D003252': NtO(MH='Construction Materials', MS='Supplies used in building.'),
  'D003253': NtO(MH='Consultants', MS='Individuals referred to for expert or professional advice or services.'),
  'D003254': NtO(MH='Consumer Advocacy', MS="The promotion and support of consumers' rights and interests."),
  'D003255': NtO(MH='Consumer Organizations', MS='Organized groups of users of goods and services.'),
  'D003256': NtO(MH='Community Participation', MS='Involvement of members of the community in the affairs of that community.'),
  'D003257': NtO(MH='Consumer Product Safety', MS='The sum total of measures taken and regulatory policies enacted to ensure the safe use of consumer products.'),
  'D003258': NtO(MH='Consumer Behavior', MS='Behavior associated with the procurement of goods, services, or experiences.'),
  'D003259': NtO(MH='Consummatory Behavior', MS='An act which constitutes the termination of a given instinctive behavior pattern or sequence.'),
  'D003260': NtO(MH='Contact Inhibition', MS='Arrest of cell locomotion or cell division when two cells come into contact.'),
  'D003261': NtO(MH='Contact Lenses', MS='Lenses designed to be worn on the front surface of the eyeball. (UMDNS, 1999)'),
  'D003262': NtO(MH='Contact Lenses, Extended-Wear', MS='Hydrophilic contact lenses worn for an extended period or permanently.'),
  'D003263': NtO(MH='Contact Lenses, Hydrophilic', MS='Soft, supple contact lenses made of plastic polymers which interact readily with water molecules. Many types are available, including continuous and extended-wear versions, which are gas-permeable and easily sterilized.'),
  'D003264': NtO(MH='Containment of Biohazards', MS='Physical and biological barriers against the spread of potentially hazardous biologically active agents such as bacteria, viruses, recombinant nucleotides, contaminated bio-specimens, etc. Typically involves the use of specialized equipment, facilities, procedures, professional personnel and established protocols.'),
  'D003265': NtO(MH='Contingent Negative Variation', MS='A negative shift of the cortical electrical potentials that increases over time. It is associated with an anticipated response to an expected stimulus and is an electrical event indicative of a state of readiness or expectancy.'),
  'D003266': NtO(MH='Continuity of Patient Care', MS='Health care provided on a continuing basis from the initial contact, following the patient through all phases of medical care.'),
  'D003267': NtO(MH='Contraception', MS='Prevention of CONCEPTION by blocking fertility temporarily, or permanently (STERILIZATION, REPRODUCTIVE). Common means of reversible contraception include NATURAL FAMILY PLANNING METHODS; CONTRACEPTIVE AGENTS; or CONTRACEPTIVE DEVICES.'),
  'D003268': NtO(MH='Contraception Behavior', MS='Behavior patterns of those practicing CONTRACEPTION.'),
  'D003269': NtO(MH='Contraception, Immunologic', MS='Contraceptive methods based on immunological processes and techniques, such as the use of CONTRACEPTIVE VACCINES.'),
  'D003270': NtO(MH='Contraceptive Agents', MS='Chemical substances that prevent or reduce the probability of CONCEPTION.'),
  'D003271': NtO(MH='Contraceptive Agents, Female', MS='Chemical substances or agents with contraceptive activity in females. Use for female contraceptive agents in general or for which there is no specific heading.'),
  'D003272': NtO(MH='Contraceptive Agents, Male', MS='Chemical substances or agents with contraceptive activity in males. Use for male contraceptive agents in general or for which there is no specific heading.'),
  'D003273': NtO(MH='Contraceptive Devices', MS='Devices that diminish the likelihood of or prevent conception. (From Dorland, 28th ed)'),
  'D003274': NtO(MH='Contraceptive Devices, Female', MS='Contraceptive devices used by females.'),
  'D003275': NtO(MH='Contraceptive Devices, Male', MS='Contraceptive devices used by males.'),
  'D003276': NtO(MH='Contraceptives, Oral', MS='Compounds, usually hormonal, taken orally in order to block ovulation and prevent the occurrence of pregnancy. The hormones are generally estrogen or progesterone or both.'),
  'D003277': NtO(MH='Contraceptives, Oral, Combined', MS='Fixed drug combinations administered orally for contraceptive purposes.'),
  'D003278': NtO(MH='Contraceptives, Oral, Hormonal', MS='Oral contraceptives which owe their effectiveness to hormonal preparations.'),
  'D003279': NtO(MH='Contraceptives, Oral, Sequential', MS='Drugs administered orally and sequentially for contraceptive purposes.'),
  'D003280': NtO(MH='Contraceptives, Oral, Synthetic', MS='Oral contraceptives which owe their effectiveness to synthetic preparations.'),
  'D003281': NtO(MH='Contraceptives, Postcoital', MS='Contraceptive substances to be used after COITUS. These agents include high doses of estrogenic drugs; progesterone-receptor blockers; ANTIMETABOLITES; ALKALOIDS, and PROSTAGLANDINS.'),
  'D003282': NtO(MH='Contraceptives, Postcoital, Hormonal', MS='Postcoital contraceptives which owe their effectiveness to hormonal preparations.'),
  'D003283': NtO(MH='Contraceptives, Postcoital, Synthetic', MS='Postcoital contraceptives which owe their effectiveness to synthetic preparations.'),
  'D003284': NtO(MH='Contract Services', MS='Outside services provided to an institution under a formal financial agreement.'),
  'D003285': NtO(MH='Contractile Proteins', MS='Proteins which participate in contractile processes. They include MUSCLE PROTEINS as well as those found in other cells and tissues. In the latter, these proteins participate in localized contractile events in the cytoplasm, in motile activity, and in cell aggregation phenomena.'),
  'D003286': NtO(MH='Contracture', MS='Prolonged shortening of the muscle or other soft tissue around a joint, preventing movement of the joint.'),
  'D003287': NtO(MH='Contrast Media', MS='Substances used to allow enhanced visualization of tissues.'),
  'D003288': NtO(MH='Contusions', MS='Injuries resulting in hemorrhage, usually manifested in the skin.'),
  'D003289': NtO(MH='Convalescence', MS='The period of recovery following an illness.'),
  'D003290': NtO(MH='Convergence, Ocular', MS='The turning inward of the lines of sight toward each other.'),
  'D003291': NtO(MH='Conversion Disorder', MS='A disorder whose predominant feature is a loss or alteration in physical functioning that suggests a physical disorder but that is actually a direct expression of a psychological conflict or need.'),
  'D003292': NtO(MH='Convulsants', MS='Substances that act in the brain stem or spinal cord to produce tonic or clonic convulsions, often by removing normal inhibitory tone. They were formerly used to stimulate respiration or as antidotes to barbiturate overdose. They are now most commonly used as experimental tools.'),
  'D003294': NtO(MH='Seizures, Febrile', MS='Seizures that occur during a febrile episode. It is a common condition, affecting 2-5% of children aged 3 months to five years. An autosomal dominant pattern of inheritance has been identified in some families. The majority are simple febrile seizures (generally defined as generalized onset, single seizures with a duration of less than 30 minutes). Complex febrile seizures are characterized by focal onset, duration greater than 30 minutes, and/or more than one seizure in a 24 hour period. The likelihood of developing epilepsy (i.e., a nonfebrile seizure disorder) following simple febrile seizures is low. Complex febrile seizures are associated with a moderately increased incidence of epilepsy. (From Menkes, Textbook of Child Neurology, 5th ed, p784)'),
  'D003295': NtO(MH='Convulsive Therapy', MS='Convulsions induced in order to treat MENTAL DISORDERS. It is used primarily in the treatment of severe affective disorders and SCHIZOPHRENIA.'),
  'D003296': NtO(MH='Cooking', MS='The art or practice of preparing food. It includes the preparation of special foods for diets in various diseases.'),
  'D003297': NtO(MH='Cooking and Eating Utensils', MS='Tools used in COOKING or EATING such as cutlery, pots, pans, and dishes.'),
  'D003298': NtO(MH='Coombs Test', MS="A test to detect non-agglutinating ANTIBODIES against ERYTHROCYTES by use of anti-antibodies (the Coombs' reagent.) The direct test is applied to freshly drawn blood to detect antibody bound to circulating red cells. The indirect test is applied to serum to detect the presence of antibodies that can bind to red blood cells."),
  'D003299': NtO(MH='Cooperative Behavior', MS='The interaction of two or more persons or organizations directed toward a common goal which is mutually beneficial. An act or instance of working or acting together for a common purpose or benefit, i.e., joint action. (From Random House Dictionary Unabridged, 2d ed)'),
  'D003300': NtO(MH='Copper', MS='A heavy metal trace element with the atomic symbol Cu, atomic number 29, and atomic weight 63.55.'),
  'D003301': NtO(MH='Copper Radioisotopes', MS='Unstable isotopes of copper that decay or disintegrate emitting radiation. Cu atoms with atomic weights 58-62, 64, and 66-68 are radioactive copper isotopes.'),
  'D003302': NtO(MH='Coprinus', MS='A genus of black-spored basidiomycetous fungi of the family Coprinaceae, order Agaricales; some species are edible.'),
  'D003303': NtO(MH='Coprophagia', MS='Eating of excrement by animal species.'),
  'D003304': NtO(MH='Coproporphyrinogen Oxidase', MS='An enzyme that catalyzes the oxidative decarboxylation of coproporphyrinogen III to protoporphyrinogen IX by the conversion of two propionate groups to two vinyl groups. It is the sixth enzyme in the 8-enzyme biosynthetic pathway of HEME, and is encoded by CPO gene. Mutations of CPO gene result in HEREDITARY COPROPORPHYRIA.'),
  'D003305': NtO(MH='Coproporphyrinogens', MS='Porphyrinogens which are intermediates in the heme biosynthesis. They have four methyl and four propionic acid side chains attached to the pyrrole rings. Coproporphyrinogens I and III are formed in the presence of uroporphyrinogen decarboxylase from the corresponding uroporphyrinogen. They can yield coproporphyrins by autooxidation or protoporphyrin by oxidative decarboxylation.'),
  'D003306': NtO(MH='Coproporphyrins', MS='Porphyrins with four methyl and four propionic acid side chains attached to the pyrrole rings. Elevated levels of Coproporphyrin III in the urine and feces are major findings in patients with HEREDITARY COPROPORPHYRIA.'),
  'D003307': NtO(MH='Copulation', MS='Sexual union of a male and a female in non-human species.'),
  'D003308': NtO(MH='Copying Processes', MS='Reproduction of data in a new location or other destination, leaving the source data unchanged, although the physical form of the result may differ from that of the source.'),
  'D003309': NtO(MH='Copyright', MS='It is a form of protection provided by law. In the United States this protection is granted to authors of original works of authorship, including literary, dramatic, musical, artistic, and certain other intellectual works. This protection is available to both published and unpublished works. (from Circular of the United States Copyright Office, 6/30/2008)'),
  'D003310': NtO(MH='Cor Triatriatum', MS='A malformation of the heart in which the embryonic common PULMONARY VEIN was not incorporated into the LEFT ATRIUM leaving behind a perforated fibromuscular membrane bisecting the left atrium, a three-atrium heart. The opening between the two left atrium sections determines the degree of obstruction to pulmonary venous return, pulmonary venous and pulmonary arterial hypertension.'),
  'D003311': NtO(MH='Cord Factors', MS='Toxic glycolipids composed of trehalose dimycolate derivatives. They are produced by MYCOBACTERIUM TUBERCULOSIS and other species of MYCOBACTERIUM. They induce cellular dysfunction in animals.'),
  'D003312': NtO(MH='Coriolis Force', MS='The apparent deflection (Coriolis acceleration) of a body in motion with respect to the earth, as seen by an observer on the earth, attributed to a fictitious force (Coriolis force) but actually caused by the rotation of the earth. In a medical context it refers to the physiological effects (nausea, vertigo, dizziness, etc.) felt by a person moving radially in a rotating system, as a rotating space station. (From Random House Unabridged Dictionary, 2d ed & McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003313': NtO(MH='Zea mays', MS='A plant species of the family POACEAE. It is a tall grass grown for its EDIBLE GRAIN, corn, used as food and animal FODDER.'),
  'D003314': NtO(MH='Corn Oil', MS='Oil from ZEA MAYS or corn plant.'),
  'D003315': NtO(MH='Cornea', MS='The transparent anterior portion of the fibrous coat of the eye consisting of five layers: stratified squamous CORNEAL EPITHELIUM; BOWMAN MEMBRANE; CORNEAL STROMA; DESCEMET MEMBRANE; and mesenchymal CORNEAL ENDOTHELIUM. It serves as the first refracting medium of the eye. It is structurally continuous with the SCLERA, avascular, receiving its nourishment by permeation through spaces between the lamellae, and is innervated by the ophthalmic division of the TRIGEMINAL NERVE via the ciliary nerves and those of the surrounding conjunctiva which together form plexuses. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D003316': NtO(MH='Corneal Diseases', MS='Diseases of the cornea.'),
  'D003317': NtO(MH='Corneal Dystrophies, Hereditary', MS='Bilateral hereditary disorders of the cornea, usually autosomal dominant, which may be present at birth but more frequently develop during adolescence and progress slowly throughout life. Central macular dystrophy is transmitted as an autosomal recessive defect.'),
  'D003318': NtO(MH='Corneal Opacity', MS='Disorder occurring in the central or peripheral area of the cornea. The usual degree of transparency becomes relatively opaque.'),
  'D003319': NtO(MH='Corneal Stroma', MS='The lamellated connective tissue constituting the thickest layer of the cornea between the Bowman and Descemet membranes.'),
  'D003320': NtO(MH='Corneal Ulcer', MS='Loss of epithelial tissue from the surface of the cornea due to progressive erosion and necrosis of the tissue; usually caused by bacterial, fungal, or viral infection.'),
  'D003321': NtO(MH='Cornell Medical Index', MS='Self-administered health questionnaire developed to obtain details of the medical history as an adjunct to the medical interview. It consists of 195 questions divided into eighteen sections; the first twelve deal with somatic complaints and the last six with mood and feeling patterns. The Index is used also as a personality inventory or in epidemiologic studies.'),
  'D003323': NtO(MH='Coronary Aneurysm', MS='Abnormal balloon- or sac-like dilatation in the wall of CORONARY VESSELS. Most coronary aneurysms are due to CORONARY ATHEROSCLEROSIS, and the rest are due to inflammatory diseases, such as KAWASAKI DISEASE.'),
  'D003324': NtO(MH='Coronary Artery Disease', MS='Pathological processes of CORONARY ARTERIES that may derive from a congenital abnormality, atherosclerotic, or non-atherosclerotic cause.'),
  'D003325': NtO(MH='Coronary Care Units', MS='The hospital unit in which patients with acute cardiac disorders receive intensive care.'),
  'D003326': NtO(MH='Coronary Circulation', MS='The circulation of blood through the CORONARY VESSELS of the HEART.'),
  'D003327': NtO(MH='Coronary Disease', MS='An imbalance between myocardial functional requirements and the capacity of the CORONARY VESSELS to supply sufficient blood flow. It is a form of MYOCARDIAL ISCHEMIA (insufficient blood supply to the heart muscle) caused by a decreased capacity of the coronary vessels.'),
  'D003328': NtO(MH='Coronary Thrombosis', MS='Coagulation of blood in any of the CORONARY VESSELS. The presence of a blood clot (THROMBUS) often leads to MYOCARDIAL INFARCTION.'),
  'D003329': NtO(MH='Coronary Vasospasm', MS='Spasm of the large- or medium-sized coronary arteries.'),
  'D003330': NtO(MH='Coronary Vessel Anomalies', MS='Malformations of CORONARY VESSELS, either arteries or veins. Included are anomalous origins of coronary arteries; ARTERIOVENOUS FISTULA; CORONARY ANEURYSM; MYOCARDIAL BRIDGING; and others.'),
  'D003331': NtO(MH='Coronary Vessels', MS='The veins and arteries of the HEART.'),
  'D003332': NtO(MH='Coronaviridae', MS='Spherical RNA viruses, in the order NIDOVIRALES, infecting a wide range of animals including humans. Transmission is by fecal-oral and respiratory routes. Mechanical transmission is also common. There are two genera: CORONAVIRUS and TOROVIRUS.'),
  'D003333': NtO(MH='Coronaviridae Infections', MS='Virus diseases caused by CORONAVIRIDAE.'),
  'D003334': NtO(MH='Coroners and Medical Examiners', MS='Physicians appointed to investigate all cases of sudden or violent death.'),
  'D003335': NtO(MH='Corpora Allata', MS='Paired or fused ganglion-like bodies in the head of insects. The bodies secrete hormones important in the regulation of metamorphosis and the development of some adult tissues.'),
  'D003336': NtO(MH='Tectum Mesencephali', MS='The dorsal portion or roof of the midbrain which is composed of two pairs of bumps, the INFERIOR COLLICULI and the SUPERIOR COLLICULI. These four colliculi are also called the quadrigeminal bodies (TECTUM MESENCEPHALI). They are centers for visual sensorimotor integration.'),
  'D003337': NtO(MH='Corpus Callosum', MS='Broad plate of dense myelinated fibers that reciprocally interconnect regions of the cortex in all lobes with corresponding regions of the opposite hemisphere. The corpus callosum is located deep in the longitudinal fissure.'),
  'D003338': NtO(MH='Corpus Luteum', MS='The yellow body derived from the ruptured OVARIAN FOLLICLE after OVULATION. The process of corpus luteum formation, LUTEINIZATION, is regulated by LUTEINIZING HORMONE.'),
  'D003339': NtO(MH='Corpus Luteum Hormones', MS=None),
  'D003340': NtO(MH='Corpus Luteum Maintenance', MS='Process of maintaining the functions of CORPORA LUTEA, specifically PROGESTERONE production which is regulated primarily by pituitary LUTEINIZING HORMONE in cycling females, and by PLACENTAL HORMONES in pregnant females. The ability to maintain luteal functions is important in PREGNANCY MAINTENANCE.'),
  'D003341': NtO(MH='Luteolysis', MS='Degradation of CORPUS LUTEUM. In the absence of pregnancy and diminishing trophic hormones, the corpus luteum undergoes luteolysis which is characterized by the involution and cessation of its endocrine function.'),
  'D003342': NtO(MH='Corpus Striatum', MS='Striped GRAY MATTER and WHITE MATTER consisting of the NEOSTRIATUM and paleostriatum (GLOBUS PALLIDUS). It is located in front of and lateral to the THALAMUS in each cerebral hemisphere. The gray substance is made up of the CAUDATE NUCLEUS and the lentiform nucleus (the latter consisting of the GLOBUS PALLIDUS and PUTAMEN). The WHITE MATTER is the INTERNAL CAPSULE.'),
  'D003343': NtO(MH='Corrosion', MS='The gradual destruction of a metal or alloy due to oxidation or action of a chemical agent. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D003344': NtO(MH='Cortical Synchronization', MS='EEG phase synchronization of the cortical brain region (CEREBRAL CORTEX).'),
  'D003345': NtO(MH='Corticosterone', MS="An adrenocortical steroid that has modest but significant activities as a mineralocorticoid and a glucocorticoid. (From Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed, p1437)"),
  'D003346': NtO(MH='Corticotropin-Releasing Hormone', MS='A peptide of about 41 amino acids that stimulates the release of ADRENOCORTICOTROPIC HORMONE. CRH is synthesized by neurons in the PARAVENTRICULAR NUCLEUS of the HYPOTHALAMUS. After being released into the pituitary portal circulation, CRH stimulates the release of ACTH from the PITUITARY GLAND. CRH can also be synthesized in other tissues, such as PLACENTA; ADRENAL MEDULLA; and TESTIS.'),
  'D003348': NtO(MH='Cortisone', MS='A naturally occurring glucocorticoid that has been used in replacement therapy for ADRENAL INSUFFICIENCY and as an anti-inflammatory agent. Cortisone itself is inactive; it is converted in the liver to the active metabolite HYDROCORTISONE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p726)'),
  'D003349': NtO(MH='Cortisone Reductase', MS='An enzyme that catalyzes the interconversion of a ketone and hydroxy group at C-20 of cortisone and other 17,20,21-trihydroxy steroids. EC 1.1.1.53.'),
  'D003350': NtO(MH='Cortodoxone', MS='17,21-Dihydroxypregn-4-ene-3,20-dione. A 17-hydroxycorticosteroid with glucocorticoid and anti-inflammatory activities.'),
  'D003352': NtO(MH='Corynebacterium', MS='A genus of asporogenous bacteria that is widely distributed in nature. Its organisms appear as straight to slightly curved rods and are known to be human and animal parasites and pathogens.'),
  'D003353': NtO(MH='Corynebacterium diphtheriae', MS='A species of gram-positive, asporogenous bacteria in which three cultural types are recognized. These types (gravis, intermedius, and mitis) were originally given in accordance with the clinical severity of the cases from which the different strains were most frequently isolated. This species is the causative agent of DIPHTHERIA.'),
  'D003354': NtO(MH='Corynebacterium Infections', MS='Infections with bacteria of the genus CORYNEBACTERIUM.'),
  'D003355': NtO(MH='Corynebacterium pyogenes', MS='A species of CORYNEBACTERIUM isolated from abscesses of warm-blooded animals.'),
  'D003357': NtO(MH='Cosmetic Techniques', MS='Procedures for the improvement or enhancement of the appearance of the visible parts of the body.'),
  'D003358': NtO(MH='Cosmetics', MS="Substances intended to be applied to the human body for cleansing, beautifying, promoting attractiveness, or altering the appearance without affecting the body's structure or functions. Included in this definition are skin creams, lotions, perfumes, lipsticks, fingernail polishes, eye and facial makeup preparations, permanent waves, hair colors, toothpastes, and deodorants, as well as any material intended for use as a component of a cosmetic product. (U.S. Food & Drug Administration Center for Food Safety & Applied Nutrition Office of Cosmetics Fact Sheet (web page) Feb 1995)"),
  'D003359': NtO(MH='Cosmic Radiation', MS='High-energy radiation or particles from extraterrestrial space that strike the earth, its atmosphere, or spacecraft and may create secondary radiation as a result of collisions with the atmosphere or spacecraft.'),
  'D003360': NtO(MH='Cosmids', MS='Plasmids containing at least one cos (cohesive-end site) of PHAGE LAMBDA. They are used as cloning vehicles.'),
  'D003361': NtO(MH='Cost Allocation', MS='The assignment, to each of several particular cost-centers, of an equitable proportion of the costs of activities that serve all of them. Cost-center usually refers to institutional departments or services.'),
  'D003362': NtO(MH='Cost-Benefit Analysis', MS='A method of comparing the cost of a program with its expected benefits in dollars (or other currency). The benefit-to-cost ratio is a measure of total return expected per unit of money spent. This analysis generally excludes consideration of factors that are not measured ultimately in economic terms. Cost effectiveness compares alternative ways to achieve a specific set of results.'),
  'D003363': NtO(MH='Cost Control', MS='The containment, regulation, or restraint of costs. Costs are said to be contained when the value of resources committed to an activity is not considered excessive. This determination is frequently subjective and dependent upon the specific geographic area of the activity being measured. (From Dictionary of Health Services Management, 2d ed)'),
  'D003364': NtO(MH='Costa Rica', MS='A country in CENTRAL AMERICA, bordering both the Caribbean Sea and the North Pacific Ocean, between NICARAGUA and PANAMA.'),
  'D003365': NtO(MH='Costs and Cost Analysis', MS='Absolute, comparative, or differential costs pertaining to services, institutions, resources, etc., or the analysis and study of these costs.'),
  'D003366': NtO(MH='Cosyntropin', MS='A synthetic peptide that is identical to the 24-amino acid segment at the N-terminal of ADRENOCORTICOTROPIC HORMONE. ACTH (1-24), a segment similar in all species, contains the biological activity that stimulates production of CORTICOSTEROIDS in the ADRENAL CORTEX.'),
  'D003367': NtO(MH='Cotinine', MS='The N-glucuronide conjugate of cotinine is a major urinary metabolite of NICOTINE. It thus serves as a biomarker of exposure to tobacco SMOKING. It has CNS stimulating properties.'),
  'D003368': NtO(MH='Gossypium', MS='A plant genus of the family MALVACEAE. It is the source of COTTON FIBER; COTTONSEED OIL, which is used for cooking, and GOSSYPOL. The economically important cotton crop is a major user of agricultural PESTICIDES.'),
  'D003369': NtO(MH='Cottonseed Oil', MS='Oil obtained from the seeds of Gossypium herbaceum L., the cotton plant. It is used in dietary products such as oleomargarine and many cooking oils. Cottonseed oil is commonly used in soaps and cosmetics.'),
  'D003370': NtO(MH='Coturnix', MS='A genus of BIRDS in the family Phasianidae, order GALLIFORMES, containing the common European and other Old World QUAIL.'),
  'D003371': NtO(MH='Cough', MS='A sudden, audible expulsion of air from the lungs through a partially closed glottis, preceded by inhalation. It is a protective response that serves to clear the trachea, bronchi, and/or lungs of irritants and secretions, or to prevent aspiration of foreign materials into the lungs.'),
  'D003372': NtO(MH='Coumaphos', MS='A organothiophosphorus cholinesterase inhibitor that is used as an anthelmintic, insecticide, and as a nematocide.'),
  'D003373': NtO(MH='Coumaric Acids', MS='Hydroxycinnamic acid and its derivatives. Act as activators of the indoleacetic acid oxidizing system, thereby producing a decrease in the endogenous level of bound indoleacetic acid in plants.'),
  'D003374': NtO(MH='Coumarins', MS='Synthetic or naturally occurring substances related to coumarin, the delta-lactone of coumarinic acid.'),
  'D003375': NtO(MH='Coumestrol', MS='A daidzein derivative occurring naturally in forage crops which has some estrogenic activity.'),
  'D003376': NtO(MH='Counseling', MS='The giving of advice and assistance to individuals with educational or personal problems.'),
  'D003377': NtO(MH='Countercurrent Distribution', MS='A method of separation of two or more substances by repeated distribution between two immiscible liquid phases that move past each other in opposite directions. It is a form of liquid-liquid chromatography. (Stedman, 25th ed)'),
  'D003378': NtO(MH='Counterimmunoelectrophoresis', MS='Immunoelectrophoresis in which immunoprecipitation occurs when antigen at the cathode is caused to migrate in an electric field through a suitable medium of diffusion against a stream of antibody migrating from the anode as a result of endosmotic flow.'),
  'D003379': NtO(MH='Countertransference', MS='Conscious or unconscious emotional reaction of the therapist to the patient which may interfere with treatment. (APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D003380': NtO(MH='Courtship', MS='Activities designed to attract the attention or favors of another.'),
  'D003381': NtO(MH='Coxiella', MS='A genus of gram-negative, rod-shaped bacteria that is widely distributed in TICKS and various mammals throughout the world. Infection with this genus is particularly prevalent in CATTLE; SHEEP; and GOATS.'),
  'D003384': NtO(MH='Coxsackievirus Infections', MS='A heterogeneous group of infections produced by coxsackieviruses, including HERPANGINA, aseptic meningitis (MENINGITIS, ASEPTIC), a common-cold-like syndrome, a non-paralytic poliomyelitis-like syndrome, epidemic pleurodynia (PLEURODYNIA, EPIDEMIC) and a serious MYOCARDITIS.'),
  'D003386': NtO(MH='Brachyura', MS='An infraorder of chiefly marine, largely carnivorous CRUSTACEA, in the order DECAPODA, including the genera Cancer, Uca, and Callinectes.'),
  'D003387': NtO(MH='Cracked Tooth Syndrome', MS='Incomplete fracture of any part of a tooth, characterized by pain during mastication and sensitivity to heat, cold, sweet or sour tastes, and alcohol; it is often undiagnosed because the tooth is usually X-ray negative and normal to pulp vitality tests.'),
  'D003388': NtO(MH='Cranial Fossa, Posterior', MS='The infratentorial compartment that contains the CEREBELLUM and BRAIN STEM. It is formed by the posterior third of the superior surface of the body of the sphenoid (SPHENOID BONE), by the occipital, the petrous, and mastoid portions of the TEMPORAL BONE, and the posterior inferior angle of the PARIETAL BONE.'),
  'D003389': NtO(MH='Cranial Nerve Diseases', MS='Disorders of one or more of the twelve cranial nerves. With the exception of the optic and olfactory nerves, this includes disorders of the brain stem nuclei from which the cranial nerves originate or terminate.'),
  'D003390': NtO(MH='Cranial Nerve Neoplasms', MS='Benign and malignant neoplasms that arise from one or more of the twelve cranial nerves.'),
  'D003391': NtO(MH='Cranial Nerves', MS='Twelve pairs of nerves that carry general afferent, visceral afferent, special afferent, somatic efferent, and autonomic efferent fibers.'),
  'D003392': NtO(MH='Cranial Sinuses', MS='Large endothelium-lined venous channels situated between the two layers of DURA MATER, the endosteal and the meningeal layers. They are devoid of valves and are parts of the venous system of dura mater. Major cranial sinuses include a postero-superior group (such as superior sagittal, inferior sagittal, straight, transverse, and occipital) and an antero-inferior group (such as cavernous, petrosal, and basilar plexus).'),
  'D003393': NtO(MH='Cranial Sutures', MS='A type of fibrous joint between bones of the head.'),
  'D003394': NtO(MH='Craniofacial Dysostosis', MS='Autosomal dominant CRANIOSYNOSTOSIS with shallow ORBITS; EXOPHTHALMOS; and maxillary hypoplasia.'),
  'D003395': NtO(MH='Craniology', MS='The scientific study of variations in size, shape, and proportion of the cranium.'),
  'D003397': NtO(MH='Craniopharyngioma', MS="A benign pituitary-region neoplasm that originates from Rathke's pouch. The two major histologic and clinical subtypes are adamantinous (or classical) craniopharyngioma and papillary craniopharyngioma. The adamantinous form presents in children and adolescents as an expanding cystic lesion in the pituitary region. The cystic cavity is filled with a black viscous substance and histologically the tumor is composed of adamantinomatous epithelium and areas of calcification and necrosis. Papillary craniopharyngiomas occur in adults, and histologically feature a squamous epithelium with papillations. (From Joynt, Clinical Neurology, 1998, Ch14, p50)"),
  'D003398': NtO(MH='Craniosynostoses', MS='Premature closure of one or more CRANIAL SUTURES. It often results in plagiocephaly. Craniosynostoses that involve multiple sutures are sometimes associated with congenital syndromes such as ACROCEPHALOSYNDACTYLIA; and CRANIOFACIAL DYSOSTOSIS.'),
  'D003399': NtO(MH='Craniotomy', MS='Surgical incision into the cranium.'),
  'D003400': NtO(MH='Astacoidea', MS='A superfamily of various freshwater CRUSTACEA, in the infraorder Astacidea, comprising the crayfish. Common genera include Astacus and Procambarus. Crayfish resemble lobsters, but are usually much smaller.'),
  'D003401': NtO(MH='Creatine', MS='An amino acid that occurs in vertebrate tissues and in urine. In muscle tissue, creatine generally occurs as phosphocreatine. Creatine is excreted as CREATININE in the urine.'),
  'D003402': NtO(MH='Creatine Kinase', MS='A transferase that catalyzes formation of PHOSPHOCREATINE from ATP + CREATINE. The reaction stores ATP energy as phosphocreatine. Three cytoplasmic ISOENZYMES have been identified in human tissues: the MM type from SKELETAL MUSCLE, the MB type from myocardial tissue and the BB type from nervous tissue as well as a mitochondrial isoenzyme. Macro-creatine kinase refers to creatine kinase complexed with other serum proteins.'),
  'D003404': NtO(MH='Creatinine', MS=None),
  'D003405': NtO(MH='Creativity', MS='The ability to generate new ideas or images.'),
  'D003406': NtO(MH='Credentialing', MS='The recognition of professional or technical competence through registration, certification, licensure, admission to association membership, the award of a diploma or degree, etc.'),
  'D003407': NtO(MH='Creosote', MS='A greasy substance with a smoky odor and burned taste created by high temperature treatment of BEECH and other WOOD; COAL TAR; or resin of the CREOSOTE BUSH. It contains CRESOLS and POLYCYCLIC AROMATIC HYDROCARBONS which are CARCINOGENS. It has been widely used as wood preservative and in PESTICIDES and had former use medicinally in DISINFECTANTS; LAXATIVES; and DERMATOLOGIC AGENTS.'),
  'D003408': NtO(MH='Cresols', MS='Any of three isomeric crystalline aromatic methylphenols, also known as hydroxytoluenes.'),
  'D003409': NtO(MH='Congenital Hypothyroidism', MS='A condition in infancy or early childhood due to an in-utero deficiency of THYROID HORMONES that can be caused by genetic or environmental factors, such as thyroid dysgenesis or HYPOTHYROIDISM in infants of mothers treated with THIOURACIL during pregnancy. Endemic cretinism is the result of iodine deficiency. Clinical symptoms include severe MENTAL RETARDATION, impaired skeletal development, short stature, and MYXEDEMA.'),
  'D003410': NtO(MH='Cri-du-Chat Syndrome', MS='An infantile syndrome characterized by a cat-like cry, failure to thrive, microcephaly, MENTAL RETARDATION, spastic quadriparesis, micro- and retrognathia, glossoptosis, bilateral epicanthus, hypertelorism, and tiny external genitalia. It is caused by a deletion of the short arm of chromosome 5 (5p-).'),
  'D003411': NtO(MH='Arvicolinae', MS='A subfamily of MURIDAE found nearly world-wide and consisting of about 20 genera. Voles, lemmings, and muskrats are members.'),
  'D003412': NtO(MH='Cricetulus', MS='A genus of the family Muridae consisting of eleven species. C. migratorius, the grey or Armenian hamster, and C. griseus, the Chinese hamster, are the two species used in biomedical research.'),
  'D003413': NtO(MH='Cricoid Cartilage', MS='The small thick cartilage that forms the lower and posterior parts of the laryngeal wall.'),
  'D003414': NtO(MH='Crigler-Najjar Syndrome', MS='A familial form of congenital hyperbilirubinemia transmitted as an autosomal recessive trait. It is characterized by icterus and brain damage caused by a glucuronyl transferase deficiency in the liver and faulty bilirubin conjugation.'),
  'D003415': NtO(MH='Crime', MS='A violation of the criminal law, i.e., a breach of the conduct code specifically sanctioned by the state, which through its administrative agencies prosecutes offenders and imposes and administers punishments. The concept includes unacceptable actions whether prosecuted or going unpunished.'),
  'D003416': NtO(MH='Criminal Law', MS='A branch of law that defines criminal offenses, regulates the apprehension, charging and trial of suspected persons, and fixes the penalties and modes of treatment applicable to convicted offenders.'),
  'D003417': NtO(MH='Criminal Psychology', MS='The branch of psychology which investigates the psychology of crime with particular reference to the personality factors of the criminal.'),
  'D003418': NtO(MH='Criminology', MS='The study of crime and criminals with special reference to the personality factors and social conditions leading toward, or away from crime.'),
  'D003419': NtO(MH='Crisis Intervention', MS='Brief therapeutic approach which is ameliorative rather than curative of acute psychiatric emergencies. Used in contexts such as emergency rooms of psychiatric or general hospitals, or in the home or place of crisis occurrence, this treatment approach focuses on interpersonal and intrapsychic factors and environmental modification. (APA Thesaurus of Psychological Index Terms, 7th ed)'),
  'D003420': NtO(MH='Crisscross Heart', MS='A developmental malformation of the heart characterized by a twisted but not defective atrioventicular connection. The abnormal rotation of the ventricular mass around its long axis results in the crossing of the inflow streams of the two ventricles. Other features include hypoplasia of the TRICUSPID VALVE and RIGHT VENTRICLE.'),
  'D003421': NtO(MH='Crithidia', MS='A genus of parasitic protozoans found in the digestive tract of invertebrates, especially insects. Organisms of this genus have an amastigote and choanomastigote stage in their life cycle.'),
  'D003422': NtO(MH='Critical Care', MS='Health care provided to a critically ill patient during a medical emergency or crisis.'),
  'D003423': NtO(MH='Critical Period, Psychological', MS='A specific stage in animal and human development during which certain types of behavior normally are shaped and molded for life.'),
  'D003424': NtO(MH='Crohn Disease', MS='A chronic transmural inflammation that may involve any part of the DIGESTIVE TRACT from MOUTH to ANUS, mostly found in the ILEUM, the CECUM, and the COLON. In Crohn disease, the inflammation, extending through the intestinal wall from the MUCOSA to the serosa, is characteristically asymmetric and segmental. Epithelioid GRANULOMAS may be seen in some patients.'),
  'D003425': NtO(MH='Flurogestone Acetate', MS='A synthetic fluorinated steroid that is used as a progestational hormone.'),
  'D003426': NtO(MH='Crop, Avian', MS='A thin-walled distention of the alimentary tract protruding just outside the body cavity in the distal end of the neck (esophagus), used for the temporary storage of food and water.'),
  'D003427': NtO(MH='Cross Circulation', MS='The circulation in a portion of the body of one individual of blood supplied from another individual.'),
  'D003428': NtO(MH='Cross Infection', MS='Any infection which a patient contracts in a health-care institution.'),
  'D003429': NtO(MH='Cross Reactions', MS='Serological reactions in which an antiserum against one antigen reacts with a non-identical but closely related antigen.'),
  'D003430': NtO(MH='Cross-Sectional Studies', MS='Studies in which the presence or absence of disease or other health-related variables are determined in each member of the study population or in a representative sample at one particular time. This contrasts with LONGITUDINAL STUDIES which are followed over a period of time.'),
  'D003431': NtO(MH='Cross-Cultural Comparison', MS='Comparison of various psychological, sociological, or cultural factors in order to assess the similarities or diversities occurring in two or more different cultures or societies.'),
  'D003432': NtO(MH='Cross-Linking Reagents', MS='Reagents with two reactive groups, usually at opposite ends of the molecule, that are capable of reacting with and thereby forming bridges between side chains of amino acids in proteins; the locations of naturally reactive areas within proteins can thereby be identified; may also be used for other macromolecules, like glycoproteins, nucleic acids, or other.'),
  'D003433': NtO(MH='Crosses, Genetic', MS='Deliberate breeding of two different individuals that results in offspring that carry part of the genetic material of each parent. The parent organisms must be genetically compatible and may be from different varieties or closely related species.'),
  'D003434': NtO(MH='Crossing Over, Genetic', MS='The reciprocal exchange of segments at corresponding positions along pairs of homologous CHROMOSOMES by symmetrical breakage and crosswise rejoining forming cross-over sites (HOLLIDAY JUNCTIONS) that are resolved during CHROMOSOME SEGREGATION. Crossing-over typically occurs during MEIOSIS but it may also occur in the absence of meiosis, for example, with bacterial chromosomes, organelle chromosomes, or somatic cell nuclear chromosomes.'),
  'D003435': NtO(MH='Crotalid Venoms', MS='Venoms from snakes of the subfamily Crotalinae or pit vipers, found mostly in the Americas. They include the rattlesnake, cottonmouth, fer-de-lance, bushmaster, and American copperhead. Their venoms contain nontoxic proteins, cardio-, hemo-, cyto-, and neurotoxins, and many enzymes, especially phospholipases A. Many of the toxins have been characterized.'),
  'D003436': NtO(MH='Croton Oil', MS='Viscous, nauseating oil obtained from the shrub Croton tiglium (Euphorbaceae). It is a vesicant and skin irritant used as pharmacologic standard for skin inflammation and allergy and causes skin cancer. It was formerly used as an emetic and cathartic with frequent mortality.'),
  'D003437': NtO(MH='Crotonates', MS='Derivatives of BUTYRIC ACID that include a double bond between carbon 2 and 3 of the aliphatic structure. Included under this heading are a broad variety of acid forms, salts, esters, and amides that include the aminobutryrate structure.'),
  'D003439': NtO(MH='Crotoxin', MS='A specific complex of toxic proteins from the venom of Crotalus durissus terrificus (South American rattlesnake). It can be separated into a phospholipase A and crotapotin fragment; the latter consists of three different amino acid chains, potentiates the enzyme, and is specifically neurotoxic.'),
  'D003440': NtO(MH='Croup', MS='Inflammation involving the GLOTTIS or VOCAL CORDS and the subglottic larynx. Croup is characterized by a barking cough, HOARSENESS, and persistent inspiratory STRIDOR (a high-pitched breathing sound). It occurs chiefly in infants and children.'),
  'D003441': NtO(MH='Crowding', MS='An excessive number of individuals, human or animal, in relation to available space.'),
  'D003442': NtO(MH='Crowns', MS='A prosthetic restoration that reproduces the entire surface anatomy of the visible natural crown of a tooth. It may be partial (covering three or more surfaces of a tooth) or complete (covering all surfaces). It is made of gold or other metal, porcelain, or resin.'),
  'D003444': NtO(MH='Crush Syndrome', MS='Severe systemic manifestation of trauma and ischemia involving soft tissues, principally skeletal muscle, due to prolonged severe crushing. It leads to increased permeability of the cell membrane and to the release of potassium, enzymes, and myoglobin from within cells. Ischemic renal dysfunction secondary to hypotension and diminished renal perfusion results in acute tubular necrosis and uremia.'),
  'D003445': NtO(MH='Crustacea', MS='A large subphylum of mostly marine ARTHROPODS containing over 42,000 species. They include familiar arthropods such as lobsters (NEPHROPIDAE), crabs (BRACHYURA), shrimp (PENAEIDAE), and barnacles (THORACICA).'),
  'D003446': NtO(MH='Crutches', MS='Wooden or metal staffs designed to aid a person in walking. (UMDNS,1999)'),
  'D003448': NtO(MH='Crying', MS='To utter an inarticulate, characteristic sound in order to communicate or express a feeling, or desire for attention.'),
  'D003449': NtO(MH='Cryoglobulinemia', MS='A condition characterized by the presence of abnormal quantities of CRYOGLOBULINS in the blood. Upon cold exposure, these abnormal proteins precipitate into the microvasculature leading to restricted blood flow in the exposed areas.'),
  'D003450': NtO(MH='Cryoglobulins', MS='Abnormal immunoglobulins, especially IGG or IGM, that precipitate spontaneously when SERUM is cooled below 37 degrees Celsius. It is characteristic of CRYOGLOBULINEMIA.'),
  'D003451': NtO(MH='Cryoprotective Agents', MS='Substances that provide protection against the harmful effects of freezing temperatures.'),
  'D003452': NtO(MH='Cryosurgery', MS='The use of freezing as a special surgical technique to destroy or excise tissue.'),
  'D003453': NtO(MH='Cryptococcosis', MS='Infection with a fungus of the species CRYPTOCOCCUS NEOFORMANS.'),
  'D003454': NtO(MH='Cryptococcus', MS='A mitosporic Tremellales fungal genus whose species usually have a capsule and do not form pseudomycellium. Teleomorphs include Filobasidiella and Fidobasidium.'),
  'D003455': NtO(MH='Cryptococcus neoformans', MS='A species of the fungus CRYPTOCOCCUS. Its teleomorph is Filobasidiella neoformans.'),
  'D003456': NtO(MH='Cryptorchidism', MS='A developmental defect in which a TESTIS or both TESTES failed to descend from high in the ABDOMEN to the bottom of the SCROTUM. Testicular descent is essential to normal SPERMATOGENESIS which requires temperature lower than the BODY TEMPERATURE. Cryptorchidism can be subclassified by the location of the maldescended testis.'),
  'D003457': NtO(MH='Cryptosporidiosis', MS='Intestinal infection with organisms of the genus CRYPTOSPORIDIUM. It occurs in both animals and humans. Symptoms include severe DIARRHEA.'),
  'D003458': NtO(MH='Cryptosporidium', MS='A genus of coccidian parasites of the family CRYPTOSPORIDIIDAE, found in the intestinal epithelium of many vertebrates including humans.'),
  'D003459': NtO(MH='Crystallins', MS='A heterogeneous family of water-soluble structural proteins found in cells of the vertebrate lens. The presence of these proteins accounts for the transparency of the lens. The family is composed of four major groups, alpha, beta, gamma, and delta, and several minor groups, which are classed on the basis of size, charge, immunological properties, and vertebrate source. Alpha, beta, and delta crystallins occur in avian and reptilian lenses, while alpha, beta, and gamma crystallins occur in all other lenses.'),
  'D003460': NtO(MH='Crystallization', MS='The formation of crystalline substances from solutions or melts. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003461': NtO(MH='Crystallography', MS='The branch of science that deals with the geometric description of crystals and their internal arrangement. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003462': NtO(MH='Cuba', MS="An island in the Greater Antilles in the West Indies, south of Florida. With the adjacent islands it forms the Republic of Cuba. Its capital is Havana. It was discovered by Columbus on his first voyage in 1492 and conquered by Spain in 1511. It has a varied history under Spain, Great Britain, and the United States but has been independent since 1902. The name Cuba is said to be an Indian name of unknown origin but the language that gave the name is extinct, so the etymology is a conjecture. (From Webster's New Geographical Dictionary, 1988, p302 & Room, Brewer's Dictionary of Names, 1992, p132)"),
  'D003463': NtO(MH='Cues', MS='Signals for an action; that specific portion of a perceptual field or pattern of stimuli to which a subject has learned to respond.'),
  'D003464': NtO(MH='Culdoscopy', MS='Endoscopic examination, therapy or surgery of the female pelvic viscera by means of an endoscope introduced into the pelvic cavity through the posterior vaginal fornix.'),
  'D003465': NtO(MH='Culex', MS='A genus of mosquitoes (CULICIDAE) commonly found in tropical regions. Species of this genus are vectors for ST. LOUIS ENCEPHALITIS as well as many other diseases of man and domestic and wild animals.'),
  'D003466': NtO(MH='Cultural Characteristics', MS='Those aspects or characteristics which identify a culture.'),
  'D003467': NtO(MH='Cultural Deprivation', MS='The absence of certain expected and acceptable cultural phenomena in the environment which results in the failure of the individual to communicate and respond in the most appropriate manner within the context of society. Language acquisition and language use are commonly used in assessing this concept.'),
  'D003468': NtO(MH='Cultural Evolution', MS='The continuous developmental process of a culture from simple to complex forms and from homogeneous to heterogeneous qualities. It is the change of cultures over time. Research in cultural evolution tries to understand the causal relationships that shape and change human culture in both the long and the short term.'),
  'D003469': NtO(MH='Culture', MS='A collective expression for all behavior patterns acquired and socially transmitted through symbols. Culture includes customs, traditions, and language.'),
  'D003470': NtO(MH='Culture Media', MS='Any liquid or solid preparation made specifically for the growth, storage, or transport of microorganisms or other types of cells. The variety of media that exist allow for the culturing of specific microorganisms and cell types, such as differential media, selective media, test media, and defined media. Solid media consist of liquid media that have been solidified with an agent such as AGAR or GELATIN.'),
  'D003471': NtO(MH='Cuprizone', MS='Copper chelator that inhibits monoamine oxidase and causes liver and brain damage.'),
  'D003472': NtO(MH='Curare', MS='Plant extracts from several species, including genera STRYCHNOS and Chondodendron, which contain TETRAHYDROISOQUINOLINES that produce PARALYSIS of skeletal muscle. These extracts are toxic and must be used with the administration of artificial respiration.'),
  'D003473': NtO(MH='Neuromuscular Nondepolarizing Agents', MS='Drugs that interrupt transmission at the skeletal neuromuscular junction without causing depolarization of the motor end plate. They prevent acetylcholine from triggering muscle contraction and are used as muscle relaxants during electroshock treatments, in convulsive states, and as anesthesia adjuvants.'),
  'D003474': NtO(MH='Curcumin', MS='A yellow-orange dye obtained from tumeric, the powdered root of CURCUMA longa. It is used in the preparation of curcuma paper and the detection of boron. Curcumin appears to possess a spectrum of pharmacological properties, due primarily to its inhibitory effects on metabolic enzymes.'),
  'D003475': NtO(MH='Curettage', MS='A scraping, usually of the interior of a cavity or tract, for removal of new growth or other abnormal tissue, or to obtain material for tissue diagnosis. It is performed with a curet (curette), a spoon-shaped instrument designed for that purpose. (From Stedman, 25th ed & Dorland, 27th ed)'),
  'D003476': NtO(MH='Curium', MS='A radioactive actinide with atomic symbol Cm, and atomic number 96. Thirteen curium isotopes have been produced with mass numbers ranging from 238-250. Its valence can be +3 or +4. It is intensely radioactive and decays by alpha-emission.'),
  'D003479': NtO(MH='Curriculum', MS='A course of study offered by an educational institution.'),
  'D003480': NtO(MH='Cushing Syndrome', MS='A condition caused by prolonged exposure to excess levels of cortisol (HYDROCORTISONE) or other GLUCOCORTICOIDS from endogenous or exogenous sources. It is characterized by upper body OBESITY; OSTEOPOROSIS; HYPERTENSION; DIABETES MELLITUS; HIRSUTISM; AMENORRHEA; and excess body fluid. Endogenous Cushing syndrome or spontaneous hypercortisolism is divided into two groups, those due to an excess of ADRENOCORTICOTROPIN and those that are ACTH-independent.'),
  'D003481': NtO(MH='Cuspid', MS='The third tooth to the left and to the right of the midline of either jaw, situated between the second INCISOR and the premolar teeth (BICUSPID). (Jablonski, Dictionary of Dentistry, 1992, p817)'),
  'D003482': NtO(MH='Custodial Care', MS='Board, room, and other personal assistance services generally provided on a long term basis. It excludes regular medical care.'),
  'D003483': NtO(MH='Cutis Laxa', MS='A group of connective tissue diseases in which skin hangs in loose pendulous folds. It is believed to be associated with decreased elastic tissue formation as well as an abnormality in elastin formation. Cutis laxa is usually a genetic disease, but acquired cases have been reported. (From Dorland, 27th ed)'),
  'D003484': NtO(MH='Cyanamide', MS='A cyanide compound which has been used as a fertilizer, defoliant and in many manufacturing processes. It often occurs as the calcium salt, sometimes also referred to as cyanamide. The citrated calcium salt is used in the treatment of alcoholism.'),
  'D003485': NtO(MH='Cyanates', MS='Organic salts of cyanic acid containing the -OCN radical.'),
  'D003486': NtO(MH='Cyanides', MS='Inorganic salts of HYDROGEN CYANIDE containing the -CN radical. The concept also includes isocyanides. It is distinguished from NITRILES, which denotes organic compounds containing the -CN radical.'),
  'D003487': NtO(MH='Cyanoacrylates', MS='A group of compounds having the general formula CH2=C(CN)-COOR; it polymerizes on contact with moisture; used as tissue adhesive; higher homologs have hemostatic and antibacterial properties.'),
  'D003488': NtO(MH='Cyanogen Bromide', MS='Cyanogen bromide (CNBr). A compound used in molecular biology to digest some proteins and as a coupling reagent for phosphoroamidate or pyrophosphate internucleotide bonds in DNA duplexes.'),
  'D003489': NtO(MH='Cyanoketone', MS="2 alpha-Cyano-17 beta-hydroxy-4,4',17 alpha-trimethylandrost-5-ene-3-one. An androstenolone-nitrile compound with steroidogenesis-blocking activity."),
  'D003490': NtO(MH='Cyanosis', MS='A bluish or purplish discoloration of the skin and mucous membranes due to an increase in the amount of deoxygenated hemoglobin in the blood or a structural defect in the hemoglobin molecule.'),
  'D003491': NtO(MH='Cybernetics', MS='That branch of learning which brings together theories and studies on communication and control in living organisms and machines.'),
  'D003492': NtO(MH='Cycasin', MS='Carcinogenic and neurotoxic glycoside occurring in a number of plant species, including Cycas revoluta.'),
  'D003493': NtO(MH='Cyclacillin', MS='A cyclohexylamido analog of PENICILLANIC ACID.'),
  'D003494': NtO(MH='Cyclamates', MS='Salts and esters of cyclamic acid.'),
  'D003495': NtO(MH='Cyclandelate', MS='A direct-acting SMOOTH MUSCLE relaxant used to dilate BLOOD VESSELS.'),
  'D003496': NtO(MH='Cyclazocine', MS='An analgesic with mixed narcotic agonist-antagonist properties.'),
  'D003497': NtO(MH='Cyclic N-Oxides', MS='Heterocyclic compounds in which an oxygen is attached to a cyclic nitrogen.'),
  'D003498': NtO(MH='Cyclic P-Oxides', MS=None),
  'D003499': NtO(MH='Cyclic S-Oxides', MS=None),
  'D003500': NtO(MH='Cyclization', MS='Changing an open-chain hydrocarbon to a closed ring. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D003501': NtO(MH='Cyclizine', MS='A histamine H1 antagonist given by mouth or parenterally for the control of postoperative and drug-induced vomiting and in motion sickness. (From Martindale, The Extra Pharmacopoeia, 30th ed, p935)'),
  'D003503': NtO(MH='Cyclobutanes', MS='Four carbon cycloparaffin cyclobutane (the structural formula (CH2)4) and its derivatives.'),
  'D003504': NtO(MH='Ancitabine', MS='Congener of CYTARABINE that is metabolized to cytarabine and thereby maintains a more constant antineoplastic action.'),
  'D003505': NtO(MH='Cyclodextrins', MS='A homologous group of cyclic GLUCANS consisting of alpha-1,4 bound glucose units obtained by the action of cyclodextrin glucanotransferase on starch or similar substrates. The enzyme is produced by certain species of Bacillus. Cyclodextrins form inclusion complexes with a wide variety of substances.'),
  'D003506': NtO(MH='Cyclofenil', MS='A gonadal stimulant and inducer of ovulation. It is used in the treatment of infertility and amenorrhea, but is thought to be less effective than CLOMIPHENE.'),
  'D003508': NtO(MH='Cycloheptanes', MS='Seven-carbon cycloparaffin cycloheptane (the structural formula (CH2)7) and its derivatives.'),
  'D003509': NtO(MH='Cyclohexanecarboxylic Acids', MS='Carboxylic acid derivatives of cyclohexane.'),
  'D003510': NtO(MH='Cyclohexanes', MS='Six-carbon alicyclic hydrocarbons.'),
  'D003511': NtO(MH='Cyclohexanols', MS='Monohydroxy derivatives of cyclohexanes that contain the general formula R-C6H11O. They have a camphorlike odor and are used in making soaps, insecticides, germicides, dry cleaning, and plasticizers.'),
  'D003512': NtO(MH='Cyclohexanones', MS='Cyclohexane ring substituted by one or more ketones in any position.'),
  'D003513': NtO(MH='Cycloheximide', MS='Antibiotic substance isolated from streptomycin-producing strains of Streptomyces griseus. It acts by inhibiting elongation during protein synthesis.'),
  'D003514': NtO(MH='Cyclohexylamines', MS='A family of alicyclic hydrocarbons containing an amine group with the general formula R-C6H10NH2.'),
  'D003515': NtO(MH='Cycloleucine', MS='An amino acid formed by cyclization of leucine. It has cytostatic, immunosuppressive and antineoplastic activities.'),
  'D003516': NtO(MH='Cycloparaffins', MS="Alicyclic hydrocarbons in which three or more of the carbon atoms in each molecule are united in a ring structure and each of the ring carbon atoms is joined to two hydrogen atoms or alkyl groups. The simplest members are cyclopropane (C3H6), cyclobutane (C4H8), cyclohexane (C6H12), and derivatives of these such as methylcyclohexane (C6H11CH3). (From Sax, et al., Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D003517': NtO(MH='Cyclopentanes', MS='A group of alicyclic hydrocarbons with the general formula R-C5H9.'),
  'D003518': NtO(MH='Cyclopenthiazide', MS='Thiazide diuretic also used as an antihypertensive agent.'),
  'D003519': NtO(MH='Cyclopentolate', MS='A parasympatholytic anticholinergic used solely to obtain mydriasis or cycloplegia.'),
  'D003520': NtO(MH='Cyclophosphamide', MS='Precursor of an alkylating nitrogen mustard antineoplastic and immunosuppressive agent that must be activated in the LIVER to form the active aldophosphamide. It has been used in the treatment of LYMPHOMA and LEUKEMIA. Its side effect, ALOPECIA, has been used for defleecing sheep. Cyclophosphamide may also cause sterility, birth defects, mutations, and cancer.'),
  'D003521': NtO(MH='Cyclopropanes', MS='Three-carbon cycloparaffin cyclopropane (the structural formula (CH2)3) and its derivatives.'),
  'D003523': NtO(MH='Cycloserine', MS='Antibiotic substance produced by Streptomyces garyphalus.'),
  'D003524': NtO(MH='Cyclosporins', MS='A group of closely related cyclic undecapeptides from the fungi Trichoderma polysporum and Cylindocarpon lucidum. They have some antineoplastic and antifungal action and significant immunosuppressive effects. Cyclosporins have been proposed as adjuvants in tissue and organ transplantation to suppress graft rejection.'),
  'D003525': NtO(MH='Cyclosteroids', MS='Steroid derivatives in which one or more bridges is formed between carbon atoms at any locant in any of the rings.'),
  'D003527': NtO(MH='Cyclothymic Disorder', MS='An affective disorder characterized by periods of depression and hypomania. These may be separated by periods of normal mood.'),
  'D003528': NtO(MH='Carcinoma, Adenoid Cystic', MS='Carcinoma characterized by bands or cylinders of hyalinized or mucinous stroma separating or surrounded by nests or cords of small epithelial cells. When the cylinders occur within masses of epithelial cells, they give the tissue a perforated, sievelike, or cribriform appearance. Such tumors occur in the mammary glands, the mucous glands of the upper and lower respiratory tract, and the salivary glands. They are malignant but slow-growing, and tend to spread locally via the nerves. (Dorland, 27th ed)'),
  'D003529': NtO(MH='Cymarine', MS='A cardiotonic cardiac glycoside found in STROPHANTHUS. The aglycone is STROPHANTHIN.'),
  'D003530': NtO(MH='Cyprinidae', MS='A family of freshwater fish comprising the minnows or CARPS.'),
  'D003531': NtO(MH='Cypriniformes', MS='An order of fish with 26 families and over 3,000 species. This order includes the families CYPRINIDAE (minnows and CARPS), Cobitidae (loaches), and Catostomidae (suckers).'),
  'D003532': NtO(MH='Cyprinodontiformes', MS='An order of fish with eight families and numerous species of both egg-laying and livebearing fish. Families include Cyprinodontidae (egg-laying KILLIFISHES;), FUNDULIDAEl; (topminnows), Goodeidae (Mexican livebearers), Jenynsiidae (jenynsiids), Poeciliidae (livebearers), Profundulidae (Middle American killifishes), Aplocheilidae, and Rivulidae (rivulines). In the family Poeciliidae, the guppy and molly belong to the genus POECILIA.'),
  'D003533': NtO(MH='Cyproheptadine', MS='A serotonin antagonist and a histamine H1 blocker used as antipruritic, appetite stimulant, antiallergic, and for the post-gastrectomy dumping syndrome, etc.'),
  'D003534': NtO(MH='Cyproterone', MS='An anti-androgen that, in the form of its acetate (CYPROTERONE ACETATE), also has progestational properties. It is used in the treatment of hypersexuality in males, as a palliative in prostatic carcinoma, and, in combination with estrogen, for the therapy of severe acne and hirsutism in females.'),
  'D003535': NtO(MH='Cyprus', MS="An island republic in the eastern Mediterranean Sea. Its capital is Nicosia. It was colonized by the Phoenicians and ancient Greeks and ruled successively by the Assyrian, Persian, Ptolemaic, Roman, and Byzantine Empires. It was under various countries from the 12th to the 20th century but became independent in 1960. The name comes from the Greek Kupros, probably representing the Sumerian kabar or gabar, copper, famous in historic times for its copper mines. The cypress tree is also named after the island. (From Webster's New Geographical Dictionary, 1988, p308 & Room, Brewer's Dictionary of Names, 1992, p134)"),
  'D003536': NtO(MH='Cystadenocarcinoma', MS='A malignant neoplasm derived from glandular epithelium, in which cystic accumulations of retained secretions are formed. The neoplastic cells manifest varying degrees of anaplasia and invasiveness, and local extension and metastases occur. Cystadenocarcinomas develop frequently in the ovaries, where pseudomucinous and serous types are recognized. (Stedman, 25th ed)'),
  'D003537': NtO(MH='Cystadenoma', MS='A benign neoplasm derived from glandular epithelium, in which cystic accumulations of retained secretions are formed. In some instances, considerable portions of the neoplasm, or even the entire mass, may be cystic. (Stedman, 25th ed)'),
  'D003538': NtO(MH='Cystamine', MS='A radiation-protective agent that interferes with sulfhydryl enzymes. It may also protect against carbon tetrachloride liver damage.'),
  'D003539': NtO(MH='Cystaphos', MS='Proposed as an adjuvant to cancer chemotherapy; may have radiation protective properties.'),
  'D003540': NtO(MH='Cystathionine', MS='Sulfur-containing amino acid formed as an intermediate in the conversion of METHIONINE to CYSTEINE.'),
  'D003541': NtO(MH='Cystathionine beta-Synthase', MS='A multifunctional pyridoxal phosphate enzyme. In the second stage of cysteine biosynthesis it catalyzes the reaction of homocysteine with serine to form cystathionine with the elimination of water. Deficiency of this enzyme leads to HYPERHOMOCYSTEINEMIA and HOMOCYSTINURIA. EC 4.2.1.22.'),
  'D003542': NtO(MH='Cystathionine gamma-Lyase', MS='A multifunctional pyridoxal phosphate enzyme. In the final step in the biosynthesis of cysteine it catalyzes the cleavage of cystathionine to yield cysteine, ammonia, and 2-ketobutyrate. EC 4.4.1.1.'),
  'D003543': NtO(MH='Cysteamine', MS='A mercaptoethylamine compound that is endogenously derived from the COENZYME A degradative pathway. The fact that cysteamine is readily transported into LYSOSOMES where it reacts with CYSTINE to form cysteine-cysteamine disulfide and CYSTEINE has led to its use in CYSTINE DEPLETING AGENTS for the treatment of CYSTINOSIS.'),
  'D003544': NtO(MH='Cysteic Acid', MS="Beta-Sulfoalanine. An amino acid with a C-terminal sulfonic acid group which has been isolated from human hair oxidized with permanganate. It occurs normally in the outer part of the sheep's fleece, where the wool is exposed to light and weather."),
  'D003545': NtO(MH='Cysteine', MS='A thiol-containing non-essential amino acid that is oxidized to form CYSTINE.'),
  'D003546': NtO(MH='Cysteine Endopeptidases', MS='ENDOPEPTIDASES which have a cysteine involved in the catalytic process. This group of enzymes is inactivated by CYSTEINE PROTEINASE INHIBITORS such as CYSTATINS and SULFHYDRYL REAGENTS.'),
  'D003547': NtO(MH='Cysteine Synthase', MS='An enzyme that catalyzes the biosynthesis of cysteine in microorganisms and plants from O-acetyl-L-serine and hydrogen sulfide. This enzyme was formerly listed as EC 4.2.99.8.'),
  'D003548': NtO(MH='Cysteinyldopa', MS='Found in large amounts in the plasma and urine of patients with malignant melanoma. It is therefore used in the diagnosis of melanoma and for the detection of postoperative metastases. Cysteinyldopa is believed to be formed by the rapid enzymatic hydrolysis of 5-S-glutathionedopa found in melanin-producing cells.'),
  'D003549': NtO(MH='Cystic Duct', MS='The duct that is connected to the GALLBLADDER and allows the emptying of bile into the COMMON BILE DUCT.'),
  'D003550': NtO(MH='Cystic Fibrosis', MS='An autosomal recessive genetic disease of the EXOCRINE GLANDS. It is caused by mutations in the gene encoding the CYSTIC FIBROSIS TRANSMEMBRANE CONDUCTANCE REGULATOR expressed in several organs including the LUNG, the PANCREAS, the BILIARY SYSTEM, and the SWEAT GLANDS. Cystic fibrosis is characterized by epithelial secretory dysfunction associated with ductal obstruction resulting in AIRWAY OBSTRUCTION; chronic RESPIRATORY INFECTIONS; PANCREATIC INSUFFICIENCY; maldigestion; salt depletion; and HEAT PROSTRATION.'),
  'D003551': NtO(MH='Cysticercosis', MS='Infection with CYSTICERCUS, the larval form of the various tapeworms of the genus Taenia (usually T. solium in man). In humans they penetrate the intestinal wall and invade subcutaneous tissue, brain, eye, muscle, heart, liver, lung, and peritoneum. Brain involvement results in NEUROCYSTICERCOSIS.'),
  'D003552': NtO(MH='Cysticercus', MS='The larval form of various tapeworms of the genus Taenia.'),
  'D003553': NtO(MH='Cystine', MS='A covalently linked dimeric nonessential amino acid formed by the oxidation of CYSTEINE. Two molecules of cysteine are joined together by a disulfide bridge to form cystine.'),
  'D003554': NtO(MH='Cystinosis', MS='A metabolic disease characterized by the defective transport of CYSTINE across the lysosomal membrane due to mutation of a membrane protein cystinosin. This results in cystine accumulation and crystallization in the cells causing widespread tissue damage. In the KIDNEY, nephropathic cystinosis is a common cause of RENAL FANCONI SYNDROME.'),
  'D003555': NtO(MH='Cystinuria', MS='An inherited disorder due to defective reabsorption of CYSTINE and other BASIC AMINO ACIDS by the PROXIMAL RENAL TUBULES. This form of aminoaciduria is characterized by the abnormally high urinary levels of cystine; LYSINE; ARGININE; and ORNITHINE. Mutations involve the amino acid transport protein gene SLC3A1.'),
  'D003556': NtO(MH='Cystitis', MS='Inflammation of the URINARY BLADDER, either from bacterial or non-bacterial causes. Cystitis is usually associated with painful urination (dysuria), increased frequency, urgency, and suprapubic pain.'),
  'D003557': NtO(MH='Phyllodes Tumor', MS='A type of connective tissue neoplasm typically arising from intralobular stroma of the breast. It is characterized by the rapid enlargement of an asymmetric firm mobile mass. Histologically, its leaf-like stromal clefts are lined by EPITHELIAL CELLS. Rare phyllodes tumor of the prostate is also known.'),
  'D003558': NtO(MH='Cystoscopy', MS='Endoscopic examination, therapy or surgery of the urinary bladder.'),
  'D003559': NtO(MH='Cystostomy', MS='Surgical creation of an opening (stoma) in the URINARY BLADDER for drainage.'),
  'D003560': NtO(MH='Cysts', MS='Any fluid-filled closed cavity or sac that is lined by an EPITHELIUM. Cysts can be of normal, abnormal, non-neoplastic, or neoplastic tissues.'),
  'D003561': NtO(MH='Cytarabine', MS='A pyrimidine nucleoside analog that is used mainly in the treatment of leukemia, especially acute non-lymphoblastic leukemia. Cytarabine is an antimetabolite antineoplastic agent that inhibits the synthesis of DNA. Its actions are specific for the S phase of the cell cycle. It also has antiviral and immunosuppressant properties. (From Martindale, The Extra Pharmacopoeia, 30th ed, p472)'),
  'D003562': NtO(MH='Cytidine', MS='A pyrimidine nucleoside that is composed of the base CYTOSINE linked to the five-carbon sugar D-RIBOSE.'),
  'D003563': NtO(MH='Cyclic CMP', MS='A cyclic nucleotide formed from CYTIDINE TRIPHOSPHATE by the action of cytidylate cyclase. It is a potential cyclic nucleotide intracellular mediator of signal transductions.'),
  'D003564': NtO(MH='Cytidine Deaminase', MS='An enzyme that catalyzes the deamination of cytidine, forming uridine. EC 3.5.4.5.'),
  'D003565': NtO(MH='Cytidine Diphosphate', MS="Cytidine 5'-(trihydrogen diphosphate). A cytosine nucleotide containing two phosphate groups esterified to the sugar moiety. Synonyms: CRPP; cytidine pyrophosphate."),
  'D003566': NtO(MH='Cytidine Diphosphate Choline', MS='Donor of choline in biosynthesis of choline-containing phosphoglycerides.'),
  'D003567': NtO(MH='Cytidine Diphosphate Diglycerides', MS='The ester of diacylglycerol with the terminal phosphate of cytidine diphosphate. It serves as an intermediate in the biosynthesis of phosphatidylethanolamine and phosphatidylserine in bacteria.'),
  'D003568': NtO(MH='Cytidine Monophosphate', MS="Cytidine (dihydrogen phosphate). A cytosine nucleotide containing one phosphate group esterified to the sugar moiety in the 2', 3' or 5' position."),
  'D003569': NtO(MH='Cytidine Monophosphate N-Acetylneuraminic Acid', MS='A nucleoside monophosphate sugar which donates N-acetylneuraminic acid to the terminal sugar of a ganglioside or glycoprotein.'),
  'D003570': NtO(MH='Cytidine Triphosphate', MS="Cytidine 5'-(tetrahydrogen triphosphate). A cytosine nucleotide containing three phosphate groups esterified to the sugar moiety."),
  'D003571': NtO(MH='Cytochalasin B', MS='A cytotoxic member of the CYTOCHALASINS.'),
  'D003572': NtO(MH='Cytochalasins', MS='11- to 14-membered macrocyclic lactones with a fused isoindolone. Members with INDOLES attached at the C10 position are called chaetoglobosins. They are produced by various fungi. Some members interact with ACTIN and inhibit CYTOKINESIS.'),
  'D003573': NtO(MH='Cytochrome b Group', MS='Cytochromes (electron-transporting proteins) with protoheme (HEME B) as the prosthetic group.'),
  'D003574': NtO(MH='Cytochrome c Group', MS='A group of cytochromes with covalent thioether linkages between either or both of the vinyl side chains of protoheme and the protein. (Enzyme Nomenclature, 1992, p539)'),
  'D003575': NtO(MH='Cytochromes c1', MS='The 30-kDa membrane-bound c-type cytochrome protein of mitochondria that functions as an electron donor to CYTOCHROME C GROUP in the mitochondrial and bacterial RESPIRATORY CHAIN. (From Enzyme Nomenclature, 1992, p545)'),
  'D003576': NtO(MH='Electron Transport Complex IV', MS='A multisubunit enzyme complex containing CYTOCHROME A GROUP; CYTOCHROME A3; two copper atoms; and 13 different protein subunits. It is the terminal oxidase complex of the RESPIRATORY CHAIN and collects electrons that are transferred from the reduced CYTOCHROME C GROUP and donates them to molecular OXYGEN, which is then reduced to water. The redox reaction is simultaneously coupled to the transport of PROTONS across the inner mitochondrial membrane.'),
  'D003577': NtO(MH='Cytochrome P-450 Enzyme System', MS='A superfamily of hundreds of closely related HEMEPROTEINS found throughout the phylogenetic spectrum, from animals, plants, fungi, to bacteria. They include numerous complex monooxygenases (MIXED FUNCTION OXYGENASES). In animals, these P-450 enzymes serve two major functions: (1) biosynthesis of steroids, fatty acids, and bile acids; (2) metabolism of endogenous and a wide variety of exogenous substrates, such as toxins and drugs (BIOTRANSFORMATION). They are classified, according to their sequence similarities rather than functions, into CYP gene families (>40% homology) and subfamilies (>59% homology). For example, enzymes from the CYP1, CYP2, and CYP3 gene families are responsible for most drug metabolism.'),
  'D003578': NtO(MH='Cytochrome-c Peroxidase', MS='A hemeprotein which catalyzes the oxidation of ferrocytochrome c to ferricytochrome c in the presence of hydrogen peroxide. EC 1.11.1.5.'),
  'D003579': NtO(MH='Cytochrome Reductases', MS=None),
  'D003580': NtO(MH='Cytochromes', MS='Hemeproteins whose characteristic mode of action involves transfer of reducing equivalents which are associated with a reversible change in oxidation state of the prosthetic group. Formally, this redox change involves a single-electron, reversible equilibrium between the Fe(II) and Fe(III) states of the central iron atom (From Enzyme Nomenclature, 1992, p539). The various cytochrome subclasses are organized by the type of HEME and by the wavelength range of their reduced alpha-absorption bands.'),
  'D003581': NtO(MH='Cytodiagnosis', MS='Diagnosis of the type and, when feasible, the cause of a pathologic process by means of microscopic study of cells in an exudate or other form of body fluid. (Stedman, 26th ed)'),
  'D003582': NtO(MH='Cytogenetics', MS='A subdiscipline of genetics which deals with the cytological and molecular analysis of the CHROMOSOMES, and location of the GENES on chromosomes, and the movements of chromosomes during the CELL CYCLE.'),
  'D003583': NtO(MH='Cytokinins', MS='Plant hormones that promote the separation of daughter cells after mitotic division of a parent cell. Frequently they are purine derivatives.'),
  'D003584': NtO(MH='Cytological Techniques', MS='Methods used to study CELLS.'),
  'D003585': NtO(MH='Cell Biology', MS='The study of the structure, behavior, growth, reproduction, and pathology of cells; and the function and chemistry of cellular components.'),
  'D003586': NtO(MH='Cytomegalovirus Infections', MS='Infection with CYTOMEGALOVIRUS, characterized by enlarged cells bearing intranuclear inclusions. Infection may be in almost any organ, but the salivary glands are the most common site in children, as are the lungs in adults.'),
  'D003587': NtO(MH='Cytomegalovirus', MS='A genus of the family HERPESVIRIDAE, subfamily BETAHERPESVIRINAE, infecting the salivary glands, liver, spleen, lungs, eyes, and other organs, in which they produce characteristically enlarged cells with intranuclear inclusions. Infection with Cytomegalovirus is also seen as an opportunistic infection in AIDS.'),
  'D003588': NtO(MH='Cytopathogenic Effect, Viral', MS='Visible morphologic changes in cells infected with viruses. It includes shutdown of cellular RNA and protein synthesis, cell fusion, release of lysosomal enzymes, changes in cell membrane permeability, diffuse changes in intracellular structures, presence of viral inclusion bodies, and chromosomal aberrations. It excludes malignant transformation, which is CELL TRANSFORMATION, VIRAL. Viral cytopathogenic effects provide a valuable method for identifying and classifying the infecting viruses.'),
  'D003589': NtO(MH='Cytophaga', MS='A genus of gram-negative gliding bacteria found in SOIL; HUMUS; and FRESHWATER and marine habitats.'),
  'D003592': NtO(MH='Cytophotometry', MS='A method for the study of certain organic compounds within cells, in situ, by measuring the light intensities of the selectively stained areas of cytoplasm. The compounds studied and their locations in the cells are made to fluoresce and are observed under a microscope.'),
  'D003593': NtO(MH='Cytoplasm', MS='The part of a cell that contains the CYTOSOL and small structures excluding the CELL NUCLEUS; MITOCHONDRIA; and large VACUOLES. (Glick, Glossary of Biochemistry and Molecular Biology, 1990)'),
  'D003594': NtO(MH='Cytoplasmic Granules', MS='Condensed areas of cellular material that may be bounded by a membrane.'),
  'D003595': NtO(MH='Cytoplasmic Streaming', MS='The movement of CYTOPLASM within a CELL. It serves as an internal transport system for moving essential substances throughout the cell, and in single-celled organisms, such as the AMOEBA, it is responsible for the movement (CELL MOVEMENT) of the entire cell.'),
  'D003596': NtO(MH='Cytosine', MS='A pyrimidine base that is a fundamental unit of nucleic acids.'),
  'D003597': NtO(MH='Cytosine Nucleotides', MS='A group of pyrimidine NUCLEOTIDES which contain CYTOSINE.'),
  'D003598': NtO(MH='Cytoskeletal Proteins', MS='Major constituent of the cytoskeleton found in the cytoplasm of eukaryotic cells. They form a flexible framework for the cell, provide attachment points for organelles and formed bodies, and make communication between parts of the cell possible.'),
  'D003599': NtO(MH='Cytoskeleton', MS='The network of filaments, tubules, and interconnecting filamentous bridges which give shape, structure, and organization to the cytoplasm.'),
  'D003600': NtO(MH='Cytosol', MS='Intracellular fluid from the cytoplasm after removal of ORGANELLES and other insoluble cytoplasmic components.'),
  'D003601': NtO(MH='Cytotoxicity Tests, Immunologic', MS='The demonstration of the cytotoxic effect on a target cell of a lymphocyte, a mediator released by a sensitized lymphocyte, an antibody, or complement.'),
  'D003602': NtO(MH='Cytotoxicity, Immunologic', MS='The phenomenon of target cell destruction by immunologically active effector cells. It may be brought about directly by sensitized T-lymphocytes or by lymphoid or myeloid "killer" cells, or it may be mediated by cytotoxic antibody, cytotoxic factor released by lymphoid cells, or complement.'),
  'D003603': NtO(MH='Cytotoxins', MS='Substances that are toxic to cells; they may be involved in immunity or may be contained in venoms. These are distinguished from CYTOSTATIC AGENTS in degree of effect. Some of them are used as CYTOTOXIC ANTIBIOTICS. The mechanism of action of many of these are as ALKYLATING AGENTS or MITOSIS MODULATORS.'),
  'D003604': NtO(MH='Czechoslovakia', MS='Created as a republic in 1918 by Czechs and Slovaks from territories formerly part of the Austro-Hungarian Empire. The country split into the Czech Republic and Slovakia 1 January 1993.'),
  'D003605': NtO(MH='D-Amino-Acid Oxidase', MS=None),
  'D003606': NtO(MH='Dacarbazine', MS='An antineoplastic agent. It has significant activity against melanomas. (from Martindale, The Extra Pharmacopoeia, 31st ed, p564)'),
  'D003607': NtO(MH='Dacryocystitis', MS='Inflammation of the lacrimal sac. (Dorland, 27th ed)'),
  'D003608': NtO(MH='Dacryocystorhinostomy', MS='Surgical fistulization of the LACRIMAL SAC for external drainage of an obstructed nasolacrimal duct.'),
  'D003609': NtO(MH='Dactinomycin', MS='A compound composed of a two CYCLIC PEPTIDES attached to a phenoxazine that is derived from STREPTOMYCES parvullus. It binds to DNA and inhibits RNA synthesis (transcription), with chain elongation more sensitive than initiation, termination, or release. As a result of impaired mRNA production, protein synthesis also declines after dactinomycin therapy. (From AMA Drug Evaluations Annual, 1993, p2015)'),
  'D003610': NtO(MH='Dagestan', MS='One of the former Associated Soviet Socialist Republics, situated on the Caspian Sea in southwest Russia.'),
  'D003611': NtO(MH='Dairy Products', MS='Raw and processed or manufactured milk and milk-derived products. These are usually from cows (bovine) but are also from goats, sheep, reindeer, and water buffalo.'),
  'D003612': NtO(MH='Dairying', MS='Production, storage, and distribution of DAIRY PRODUCTS.'),
  'D003613': NtO(MH='Danazol', MS='A synthetic steroid with antigonadotropic and anti-estrogenic activities that acts as an anterior pituitary suppressant by inhibiting the pituitary output of gonadotropins. It possesses some androgenic properties. Danazol has been used in the treatment of endometriosis and some benign breast disorders.'),
  'D003614': NtO(MH='Dance Therapy', MS='The use of dancing for therapeutic purposes.'),
  'D003615': NtO(MH='Dancing', MS='Rhythmic and patterned body movements which are usually performed to music.'),
  'D003616': NtO(MH='Dandy-Walker Syndrome', MS='A congenital abnormality of the central nervous system marked by failure of the midline structures of the cerebellum to develop, dilation of the fourth ventricle, and upward displacement of the transverse sinuses, tentorium, and torcula. Clinical features include occipital bossing, progressive head enlargement, bulging of anterior fontanelle, papilledema, ataxia, gait disturbances, nystagmus, and intellectual compromise. (From Menkes, Textbook of Child Neurology, 5th ed, pp294-5)'),
  'D003617': NtO(MH='Dangerous Behavior', MS='Actions which have a high risk of being harmful or injurious to oneself or others.'),
  'D003619': NtO(MH='Dansyl Compounds', MS='Compounds that contain a 1-dimethylaminonaphthalene-5-sulfonyl group.'),
  'D003620': NtO(MH='Dantrolene', MS='Skeletal muscle relaxant that acts by interfering with excitation-contraction coupling in the muscle fiber. It is used in spasticity and other neuromuscular abnormalities. Although the mechanism of action is probably not central, dantrolene is usually grouped with the central muscle relaxants.'),
  'D003621': NtO(MH='Daphnia', MS='A diverse genus of minute freshwater CRUSTACEA, of the suborder CLADOCERA. They are a major food source for both young and adult freshwater fish.'),
  'D003622': NtO(MH='Dapsone', MS='A sulfone active against a wide range of bacteria but mainly employed for its actions against MYCOBACTERIUM LEPRAE. Its mechanism of action is probably similar to that of the SULFONAMIDES which involves inhibition of folic acid synthesis in susceptible organisms. It is also used with PYRIMETHAMINE in the treatment of malaria. (From Martindale, The Extra Pharmacopoeia, 30th ed, p157-8)'),
  'D003623': NtO(MH='Dark Adaptation', MS='Adjustment of the eyes under conditions of low light. The sensitivity of the eye to light is increased during dark adaptation.'),
  'D003624': NtO(MH='Darkness', MS='The absence of light.'),
  'D003625': NtO(MH='Data Collection', MS='Systematic gathering of data for a particular purpose from various sources, including questionnaires, interviews, observation, existing records, and electronic devices. The process is usually preliminary to statistical analysis of the data.'),
  'D003626': NtO(MH='Data Display', MS='The visual representation of data via a manufactured system.'),
  'D003627': NtO(MH='Data Interpretation, Statistical', MS='Application of statistical procedures to analyze specific observed or assumed facts from a particular study.'),
  'D003628': NtO(MH='Database Management Systems', MS='Software designed to store, manipulate, manage, and control data for specific uses.'),
  'D003629': NtO(MH='Modems', MS='Equipment that sends digital information over telephone lines. The term Modem is a short form of the phrase modulator-demodulator.'),
  'D003630': NtO(MH='Daunorubicin', MS='A very toxic anthracycline aminoglycoside antineoplastic isolated from Streptomyces peucetius and others, used in treatment of LEUKEMIA and other NEOPLASMS.'),
  'D003631': NtO(MH='Day Care, Medical', MS='Institutional health care of patients during the day. The patients return home at night.'),
  'D003632': NtO(MH='Dichlorodiphenyldichloroethane', MS='An organochlorine insecticide that is slightly irritating to the skin. (From Merck Index, 11th ed, p482)'),
  'D003633': NtO(MH='Dichlorodiphenyl Dichloroethylene', MS='An organochlorine pesticide, it is the ethylene metabolite of DDT.'),
  'D003634': NtO(MH='DDT', MS='A polychlorinated pesticide that is resistant to destruction by light and oxidation. Its unusual stability has resulted in difficulties in residue removal from water, soil, and foodstuffs. This substance may reasonably be anticipated to be a carcinogen: Fourth Annual Report on Carcinogens (NTP-85-002, 1985). (From Merck Index, 11th ed)'),
  'D003635': NtO(MH='De Lange Syndrome', MS='A syndrome characterized by growth retardation, severe MENTAL RETARDATION, short stature, a low-pitched growling cry, brachycephaly, low-set ears, webbed neck, carp mouth, depressed nasal bridge, bushy eyebrows meeting at the midline, hirsutism, and malformations of the hands. The condition may occur sporadically or be associated with an autosomal dominant pattern of inheritance or duplication of the long arm of chromosome 3. (Menkes, Textbook of Child Neurology, 5th ed, p231)'),
  'D003636': NtO(MH='DEAE-Cellulose', MS='Cellulose derivative used in chromatography, as ion-exchange material, and for various industrial applications.'),
  'D003637': NtO(MH='DEAE-Dextran', MS='Used as a support for ion-exchange chromatography.'),
  'D003638': NtO(MH='Deafness', MS='A general term for the complete loss of the ability to hear from both ears.'),
  'D003639': NtO(MH='Hearing Loss, Sudden', MS='Sensorineural hearing loss which develops suddenly over a period of hours or a few days. It varies in severity from mild to total deafness. Sudden deafness can be due to head trauma, vascular diseases, infections, or can appear without obvious cause or warning.'),
  'D003640': NtO(MH='Dealkylation', MS='The removing of alkyl groups from a compound. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D003641': NtO(MH='Deamination', MS='The removal of an amino group (NH2) from a chemical compound.'),
  'D003642': NtO(MH='Deanol', MS='An antidepressive agent that has also been used in the treatment of movement disorders. The mechanism of action is not well understood.'),
  'D003643': NtO(MH='Death', MS='Irreversible cessation of all bodily functions, manifested by absence of spontaneous breathing and total loss of cardiovascular and cerebral functions.'),
  'D003644': NtO(MH='Death Certificates', MS='Official records of individual deaths including the cause of death certified by a physician, and any other required identifying information.'),
  'D003645': NtO(MH='Death, Sudden', MS='The abrupt cessation of all vital bodily functions, manifested by the permanent loss of total cerebral, respiratory, and cardiovascular functions.'),
  'D003646': NtO(MH='Debridement', MS='The removal of foreign material and devitalized or contaminated tissue from or adjacent to a traumatic or infected lesion until surrounding healthy tissue is exposed. (Dorland, 27th ed)'),
  'D003647': NtO(MH='Debrisoquin', MS='An adrenergic neuron-blocking drug similar in effects to GUANETHIDINE. It is also noteworthy in being a substrate for a polymorphic cytochrome P-450 enzyme. Persons with certain isoforms of this enzyme are unable to properly metabolize this and many other clinically important drugs. They are commonly referred to as having a debrisoquin 4-hydroxylase polymorphism.'),
  'D003648': NtO(MH='Decalcification Technique', MS='Removal of minerals from bones during bone examination.'),
  'D003649': NtO(MH='Decalcification, Pathologic', MS='The loss of calcium salts from bones and teeth. Bacteria may be responsible for this occurrence in teeth. Old age may be a factor contributing to calcium loss, as is the presence of diseases such as rheumatoid arthritis.'),
  'D003650': NtO(MH='Decamethonium Compounds', MS='Compounds that contain the decamethylenebis(trimethyl)ammonium radical. These compounds frequently act as neuromuscular depolarizing agents.'),
  'D003651': NtO(MH='Decanoates', MS='Salts and esters of the 10-carbon monocarboxylic acid-decanoic acid.'),
  'D003652': NtO(MH='Decanoic Acids', MS='10-carbon saturated monocarboxylic acids.'),
  'D003653': NtO(MH='Decarboxylation', MS='The removal of a carboxyl group, usually in the form of carbon dioxide, from a chemical compound.'),
  'D003654': NtO(MH='Deceleration', MS='A decrease in the rate of speed.'),
  'D003655': NtO(MH='Decerebrate State', MS='A condition characterized by abnormal posturing of the limbs that is associated with injury to the brainstem. This may occur as a clinical manifestation or induced experimentally in animals. The extensor reflexes are exaggerated leading to rigid extension of the limbs accompanied by hyperreflexia and opisthotonus. This condition is usually caused by lesions which occur in the region of the brainstem that lies between the red nuclei and the vestibular nuclei. In contrast, decorticate rigidity is characterized by flexion of the elbows and wrists with extension of the legs and feet. The causative lesion for this condition is located above the red nuclei and usually consists of diffuse cerebral damage. (From Adams et al., Principles of Neurology, 6th ed, p358)'),
  'D003656': NtO(MH='Decidua', MS='The hormone-responsive glandular layer of ENDOMETRIUM that sloughs off at each menstrual flow (decidua menstrualis) or at the termination of pregnancy. During pregnancy, the thickest part of the decidua forms the maternal portion of the PLACENTA, thus named decidua placentalis. The thin portion of the decidua covering the rest of the embryo is the decidua capsularis.'),
  'D003657': NtO(MH='Decision Making', MS='The process of making a selective intellectual judgment when presented with several complex alternatives consisting of several variables, and usually defining a course of action or an idea.'),
  'D003658': NtO(MH='Decision Making, Computer-Assisted', MS='Use of an interactive computer system designed to assist the physician or other health professional in choosing between certain relationships or variables for the purpose of making a diagnostic or therapeutic decision.'),
  'D003659': NtO(MH='Decision Making, Organizational', MS='The process by which decisions are made in an institution or other organization.'),
  'D003660': NtO(MH='Decision Support Systems, Management', MS='Computer-based systems that enable management to interrogate the computer on an ad hoc basis for various kinds of information in the organization, which predict the effect of potential decisions.'),
  'D003661': NtO(MH='Decision Support Techniques', MS='Mathematical or statistical procedures used as aids in making a decision. They are frequently used in medical decision-making.'),
  'D003662': NtO(MH='Decision Theory', MS='A theoretical technique utilizing a group of related constructs to describe or prescribe how individuals or groups of people choose a course of action when faced with several alternatives and a variable amount of knowledge about the determinants of the outcomes of those alternatives.'),
  'D003663': NtO(MH='Decision Trees', MS='A graphic device used in decision analysis, series of decision options are represented as branches (hierarchical).'),
  'D003664': NtO(MH='Decompression', MS='Decompression external to the body, most often the slow lessening of external pressure on the whole body (especially in caisson workers, deep sea divers, and persons who ascend to great heights) to prevent DECOMPRESSION SICKNESS. It includes also sudden accidental decompression, but not surgical (local) decompression or decompression applied through body openings.'),
  'D003665': NtO(MH='Decompression Sickness', MS='A condition occurring as a result of exposure to a rapid fall in ambient pressure. Gases, nitrogen in particular, come out of solution and form bubbles in body fluid and blood. These gas bubbles accumulate in joint spaces and the peripheral circulation impairing tissue oxygenation causing disorientation, severe pain, and potentially death.'),
  'D003666': NtO(MH='Decontamination', MS='The removal of contaminating material, such as radioactive materials, biological materials, or CHEMICAL WARFARE AGENTS, from a person or object.'),
  'D003667': NtO(MH='Decoquinate', MS='A coccidiostat for poultry.'),
  'D003668': NtO(MH='Pressure Ulcer', MS='An ulceration caused by prolonged pressure on the SKIN and TISSUES when one stays in one position for a long period of time, such as lying in bed. The bony areas of the body are the most frequently affected sites which become ischemic (ISCHEMIA) under sustained and constant pressure.'),
  'D003669': NtO(MH='Deductibles and Coinsurance', MS='Cost-sharing mechanisms that provide for payment by the insured of some portion of covered expenses. Deductibles are the amounts paid by the insured under a health insurance contract before benefits become payable; coinsurance is the provision under which the insured pays part of the medical bill, usually according to a fixed percentage, when benefits become payable.'),
  'D003670': NtO(MH='Deer', MS="The family Cervidae of 17 genera and 45 species occurring nearly throughout North America, South America, and Eurasia, on most associated continental islands, and in northern Africa. Wild populations of deer have been established through introduction by people in Cuba, New Guinea, Australia, New Zealand, and other places where the family does not naturally occur. They are slim, long-legged and best characterized by the presence of antlers. Their habitat is forests, swamps, brush country, deserts, and arctic tundra. They are usually good swimmers; some migrate seasonally. (Walker's Mammals of the World, 5th ed, p1362)"),
  'D003671': NtO(MH='DEET', MS='A compound used as a topical insect repellent that may cause irritation to eyes and mucous membranes, but not to the skin.'),
  'D003672': NtO(MH='Defecation', MS='The normal process of elimination of fecal material from the RECTUM.'),
  'D003673': NtO(MH='Defective Viruses', MS='Viruses which lack a complete genome so that they cannot completely replicate or cannot form a protein coat. Some are host-dependent defectives, meaning they can replicate only in cell systems which provide the particular genetic function which they lack. Others, called SATELLITE VIRUSES, are able to replicate only when their genetic defect is complemented by a helper virus.'),
  'D003674': NtO(MH='Defense Mechanisms', MS='Unconscious process used by an individual or a group of individuals in order to cope with impulses, feelings or ideas which are not acceptable at their conscious level; various types include reaction formation, projection and self reversal.'),
  'D003675': NtO(MH='Defensive Medicine', MS='The alterations of modes of medical practice, induced by the threat of liability, for the principal purposes of forestalling lawsuits by patients as well as providing good legal defense in the event that such lawsuits are instituted.'),
  'D003676': NtO(MH='Deferoxamine', MS='Natural product isolated from Streptomyces pilosus. It forms iron complexes and is used as a chelating agent, particularly in the mesylate form.'),
  'D003677': NtO(MH='Deficiency Diseases', MS='A condition produced by dietary or metabolic deficiency. The term includes all diseases caused by an insufficient supply of essential nutrients, i.e., protein (or amino acids), vitamins, and minerals. It also includes an inadequacy of calories. (From Dorland, 27th ed; Stedman, 25th ed)'),
  'D003678': NtO(MH='Defoliants, Chemical', MS="Herbicides that remove leaves from trees and growing plants. They may be either organic or inorganic. Several of the more persistent types have been used in military operations and many are toxic. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D003679': NtO(MH='Deglutition', MS='The act of taking solids and liquids into the GASTROINTESTINAL TRACT through the mouth and throat.'),
  'D003680': NtO(MH='Deglutition Disorders', MS='Difficulty in SWALLOWING which may result from neuromuscular disorder or mechanical obstruction. Dysphagia is classified into two distinct types: oropharyngeal dysphagia due to malfunction of the PHARYNX and UPPER ESOPHAGEAL SPHINCTER; and esophageal dysphagia due to malfunction of the ESOPHAGUS.'),
  'D003681': NtO(MH='Dehydration', MS='The condition that results from excessive loss of water from a living organism.'),
  'D003682': NtO(MH='Dehydroascorbatase', MS='Catalyzes the hydrolysis of the gamma lactone, dehydroascorbate, to diketogulonate.'),
  'D003683': NtO(MH='Dehydroascorbic Acid', MS='The reversibly oxidized form of ascorbic acid. It is the lactone of 2,3-DIKETOGULONIC ACID and has antiscorbutic activity in man on oral ingestion.'),
  'D003684': NtO(MH='Dehydrocholesterols', MS='Cholesterol derivatives having an additional double bond in any position. The prevalent isomer 7-dehydrocholesterol is a precursor of cholesterol and of vitamin D3.'),
  'D003685': NtO(MH='Dehydrocholic Acid', MS='A semisynthetic bile acid made from cholic acid. It is used as a cholagogue, hydrocholeretic, diuretic, and as a diagnostic aid.'),
  'D003687': NtO(MH='Dehydroepiandrosterone', MS='A major C19 steroid produced by the ADRENAL CORTEX. It is also produced in small quantities in the TESTIS and the OVARY. Dehydroepiandrosterone (DHEA) can be converted to TESTOSTERONE; ANDROSTENEDIONE; ESTRADIOL; and ESTRONE. Most of DHEA is sulfated (DEHYDROEPIANDROSTERONE SULFATE) before secretion.'),
  'D003688': NtO(MH='Deinstitutionalization', MS="The practice of caring for individuals in the community, rather than in an institutional environment with resultant effects on the individual, the individual's family, the community, and the health care system."),
  'D003689': NtO(MH='Vestibular Nucleus, Lateral', MS='Vestibular nucleus lying immediately superior to the inferior vestibular nucleus and composed of large multipolar nerve cells. Its upper end becomes continuous with the superior vestibular nucleus.'),
  'D003690': NtO(MH='Deja Vu', MS='A subjective feeling that an experience which is occurring for the first time has been experienced before.'),
  'D003691': NtO(MH='Delaware', MS='State bounded on the north by Pennsylvania, on the west and south by Maryland, and on the east by Delaware Bay and the Atlantic Ocean.'),
  'D003692': NtO(MH='Delayed-Action Preparations', MS='Dosage forms of a drug that act over a period of time by controlled-release processes or technology.'),
  'D003693': NtO(MH='Delirium', MS='A disorder characterized by CONFUSION; inattentiveness; disorientation; ILLUSIONS; HALLUCINATIONS; agitation; and in some instances autonomic nervous system overactivity. It may result from toxic/metabolic conditions or structural brain lesions. (From Adams et al., Principles of Neurology, 6th ed, pp411-2)'),
  'D003695': NtO(MH='Delivery of Health Care', MS='The concept concerned with all aspects of providing and distributing health services to a patient population.'),
  'D003696': NtO(MH='Delivery Rooms', MS='Hospital units equipped for childbirth.'),
  'D003697': NtO(MH='Delphi Technique', MS='An iterative questionnaire designed to measure consensus among individual responses. In the classic Delphi approach, there is no interaction between responder and interviewer.'),
  'D003698': NtO(MH='Hepatitis Delta Virus', MS='A defective virus, containing particles of RNA nucleoprotein in virion-like form, present in patients with acute hepatitis B and chronic hepatitis. It requires the presence of a hepadnavirus for full replication. This is the lone species in the genus Deltavirus.'),
  'D003699': NtO(MH='Hepatitis D', MS='INFLAMMATION of the LIVER in humans caused by HEPATITIS DELTA VIRUS, a defective RNA virus that can only infect HEPATITIS B patients. For its viral coating, hepatitis delta virus requires the HEPATITIS B SURFACE ANTIGENS produced by these patients. Hepatitis D can occur either concomitantly with (coinfection) or subsequent to (superinfection) hepatitis B infection. Similar to hepatitis B, it is primarily transmitted by parenteral exposure, such as transfusion of contaminated blood or blood products, but can also be transmitted via sexual or intimate personal contact.'),
  'D003700': NtO(MH='Delta Rhythm', MS='Brain waves seen on EEG characterized by a high amplitude and a frequency of 4 Hz and below. They are considered the "deep sleep waves" observed during sleep in dreamless states, infancy, and in some brain disorders.'),
  'D003701': NtO(MH='Delta Sleep-Inducing Peptide', MS='A nonapeptide that is found in neurons, peripheral organs, and plasma. This neuropeptide induces mainly delta sleep in mammals. In addition to sleep, the peptide has been observed to affect electrophysiological activity, neurotransmitter levels in the brain, circadian and locomotor patterns, hormonal levels, psychological performance, and the activity of neuropharmacological drugs including their withdrawal.'),
  'D003702': NtO(MH='Delusions', MS="A false belief regarding the self or persons or objects outside the self that persists despite the facts, and is not considered tenable by one's associates."),
  'D003703': NtO(MH='Demecolcine', MS='An alkaloid isolated from Colchicum autumnale L. and used as an antineoplastic.'),
  'D003704': NtO(MH='Dementia', MS='An acquired organic mental disorder with loss of intellectual abilities of sufficient severity to interfere with social or occupational functioning. The dysfunction is multifaceted and involves memory, behavior, personality, judgment, attention, spatial relations, language, abstract thought, and other executive functions. The intellectual decline is usually progressive, and initially spares the level of consciousness.'),
  'D003707': NtO(MH='Demeclocycline', MS='A TETRACYCLINE analog having a 7-chloro and a 6-methyl. Because it is excreted more slowly than tetracycline, it maintains effective blood levels for longer periods of time.'),
  'D003708': NtO(MH='Nordazepam', MS='An intermediate in the metabolism of DIAZEPAM to OXAZEPAM. It may have actions similar to those of diazepam.'),
  'D003709': NtO(MH='Democracy', MS='A system of government in which there is free and equal participation by the people in the political decision-making process.'),
  'D003710': NtO(MH='Demography', MS='Statistical interpretation and description of a population with reference to distribution, composition, or structure.'),
  'D003711': NtO(MH='Demyelinating Diseases', MS='Diseases characterized by loss or dysfunction of myelin in the central or peripheral nervous system.'),
  'D003712': NtO(MH='Dendrites', MS='Extensions of the nerve cell body. They are short and branched and receive stimuli from other NEURONS.'),
  'D003713': NtO(MH='Dendritic Cells', MS='Specialized cells of the hematopoietic system that have branch-like extensions. They are found throughout the lymphatic system, and in non-lymphoid tissues such as SKIN and the epithelia of the intestinal, respiratory, and reproductive tracts. They trap and process ANTIGENS, and present them to T-CELLS, thereby stimulating CELL-MEDIATED IMMUNITY. They are different from the non-hematopoietic FOLLICULAR DENDRITIC CELLS, which have a similar morphology and immune system function, but with respect to humoral immunity (ANTIBODY PRODUCTION).'),
  'D003714': NtO(MH='Denervation', MS='The resection or removal of the nerve to an organ or part.'),
  'D003715': NtO(MH='Dengue', MS='An acute febrile disease transmitted by the bite of AEDES mosquitoes infected with DENGUE VIRUS. It is self-limiting and characterized by fever, myalgia, headache, and rash. SEVERE DENGUE is a more virulent form of dengue.'),
  'D003716': NtO(MH='Dengue Virus', MS='A species of the genus FLAVIVIRUS which causes an acute febrile and sometimes hemorrhagic disease in man. Dengue is mosquito-borne and four serotypes are known.'),
  'D003717': NtO(MH='Denial, Psychological', MS='Refusal to admit the truth or reality of a situation or experience.'),
  'D003718': NtO(MH='Denmark', MS='A country in northern Europe, bordering the Baltic Sea and the North Sea. The capital is Copenhagen.'),
  'D003719': NtO(MH='Dens in Dente', MS='Anomaly of the tooth, found chiefly in upper lateral incisors. It is characterized by invagination of the enamel at the incisal edge.'),
  'D003720': NtO(MH='Densitometry', MS='The measurement of the density of a material by measuring the amount of light or radiation passing through (or absorbed by) the material.'),
  'D003722': NtO(MH='Dental Alloys', MS='A mixture of metallic elements or compounds with other metallic or metalloid elements in varying proportions for use in restorative or prosthetic dentistry.'),
  'D003723': NtO(MH='Dental Amalgam', MS='An alloy used in restorative dentistry that contains mercury, silver, tin, copper, and possibly zinc.'),
  'D003724': NtO(MH='Dental Arch', MS='The curve formed by the row of TEETH in their normal position in the JAW. The inferior dental arch is formed by the mandibular teeth, and the superior dental arch by the maxillary teeth.'),
  'D003725': NtO(MH='Dental Articulators', MS='Mechanical devices that simulate the temporomandibular joints and jaws to which maxillary and mandibular casts are attached. The entire assembly attempts to reproduce the movements of the mandible and the various tooth-to-tooth relationships that accompany those movements.'),
  'D003726': NtO(MH='Dental Assistants', MS='Individuals who assist the dentist or the dental hygienist.'),
  'D003727': NtO(MH='Dental Auxiliaries', MS='Personnel whose work is prescribed and supervised by the dentist.'),
  'D003728': NtO(MH='Dental Calculus', MS='Abnormal concretion or calcified deposit that forms around the teeth or dental prostheses.'),
  'D003729': NtO(MH='Dental Care', MS='The total of dental diagnostic, preventive, and restorative services provided to meet the needs of a patient (from Illustrated Dictionary of Dentistry, 1982).'),
  'D003730': NtO(MH='Dental Care for Disabled', MS='Dental care for the emotionally, mentally, or physically disabled patient. It does not include dental care for the chronically ill ( = DENTAL CARE FOR CHRONICALLY ILL).'),
  'D003731': NtO(MH='Dental Caries', MS='Localized destruction of the tooth surface initiated by decalcification of the enamel followed by enzymatic lysis of organic structures and leading to cavity formation. If left unchecked, the cavity may penetrate the enamel and dentin and reach the pulp.'),
  'D003732': NtO(MH='Dental Caries Activity Tests', MS='Diagnostic tests conducted in order to measure the increment of active DENTAL CARIES over a period of time.'),
  'D003733': NtO(MH='Dental Caries Susceptibility', MS='The predisposition to tooth decay (DENTAL CARIES).'),
  'D003734': NtO(MH='Dental Casting Investment', MS="Material from which the casting mold is made in the fabrication of gold or cobalt-chromium castings. (Boucher's Clinical Dental Terminology, 4th ed, p168)"),
  'D003735': NtO(MH='Dental Casting Technique', MS='The process of producing a form or impression made of metal or plaster using a mold.'),
  'D003736': NtO(MH='Dental Cavity Lining', MS='An inner coating, as of varnish or other protective substance, to cover the dental cavity wall. It is usually a resinous film-forming agent dissolved in a volatile solvent, or a suspension of calcium hydroxide in a solution of a synthetic resin. The lining seals the dentinal tubules and protects the pulp before a restoration is inserted. (Jablonski, Illustrated Dictionary of Dentistry, 1982)'),
  'D003737': NtO(MH='Dental Cavity Preparation', MS="An operation in which carious material is removed from teeth and biomechanically correct forms are established in the teeth to receive and retain restorations. A constant requirement is provision for prevention of failure of the restoration through recurrence of decay or inadequate resistance to applied stresses. (Boucher's Clinical Dental Terminology, 4th ed, p239-40)"),
  'D003738': NtO(MH='Dental Cements', MS='Substances used to bond COMPOSITE RESINS to DENTAL ENAMEL and DENTIN. These bonding or luting agents are used in restorative dentistry, ROOT CANAL THERAPY; PROSTHODONTICS; and ORTHODONTICS.'),
  'D003739': NtO(MH='Dental Cementum', MS='The bonelike rigid connective tissue covering the root of a tooth from the cementoenamel junction to the apex and lining the apex of the root canal, also assisting in tooth support by serving as attachment structures for the periodontal ligament. (Jablonski, Dictionary of Dentistry, 1992)'),
  'D003740': NtO(MH='Dental Clinics', MS='Facilities where dental care is provided to patients.'),
  'D003741': NtO(MH='Dental Deposits', MS='Accumulations of microflora that lead to pathological plaque and calculus which cause PERIODONTAL DISEASES. It can be considered a type of BIOFILMS. It is subtly distinguished from the protective DENTAL PELLICLE.'),
  'D003742': NtO(MH='Dental Devices, Home Care', MS='Devices used in the home by persons to maintain dental and periodontal health. The devices include toothbrushes, dental flosses, water irrigators, gingival stimulators, etc.'),
  'D003743': NtO(MH='Dental Enamel', MS='A hard thin translucent layer of calcified substance which envelops and protects the dentin of the crown of the tooth. It is the hardest substance in the body and is almost entirely composed of calcium salts. Under the microscope, it is composed of thin rods (enamel prisms) held together by cementing substance, and surrounded by an enamel sheath. (From Jablonski, Dictionary of Dentistry, 1992, p286)'),
  'D003744': NtO(MH='Dental Enamel Hypoplasia', MS='An acquired or hereditary condition due to deficiency in the formation of tooth enamel (AMELOGENESIS). It is usually characterized by defective, thin, or malformed DENTAL ENAMEL. Risk factors for enamel hypoplasia include gene mutations, nutritional deficiencies, diseases, and environmental factors.'),
  'D003745': NtO(MH='Dental Enamel Permeability', MS='The property of dental enamel to permit passage of light, heat, gases, liquids, metabolites, mineral ions and other substances. It does not include the penetration of the dental enamel by microorganisms.'),
  'D003746': NtO(MH='Dental Enamel Proteins', MS='The proteins that are part of the dental enamel matrix.'),
  'D003747': NtO(MH='Dental Enamel Solubility', MS='The susceptibility of the DENTAL ENAMEL to dissolution.'),
  'D003748': NtO(MH='Dental Equipment', MS="The nonexpendable items used by the dentist or dental staff in the performance of professional duties. (From Boucher's Clinical Dental Terminology, 4th ed, p106)"),
  'D003749': NtO(MH='Dental Facilities', MS='Use for material on dental facilities in general or for which there is no specific heading.'),
  'D003750': NtO(MH='Dental Fissures', MS="Deep grooves or clefts in the surface of teeth equivalent to class 1 cavities in Black's classification of dental caries."),
  'D003751': NtO(MH='Dental Fistula', MS='An abnormal passage in the oral cavity on the gingiva.'),
  'D003752': NtO(MH='Dental Health Services', MS='Services designed to promote, maintain, or restore dental health.'),
  'D003753': NtO(MH='Dental Health Surveys', MS='A systematic collection of factual data pertaining to dental or oral health and disease in a human population within a given geographic area.'),
  'D003754': NtO(MH='Dental High-Speed Equipment', MS='Tools used in dentistry that operate at high rotation speeds.'),
  'D003755': NtO(MH='Dental High-Speed Technique', MS='Dental methods involving the use of DENTAL HIGH-SPEED EQUIPMENT.'),
  'D003756': NtO(MH='Dental Hygienists', MS='Persons trained in an accredited school or dental college and licensed by the state in which they reside to provide dental prophylaxis under the direction of a licensed dentist.'),
  'D003757': NtO(MH='Dental Implantation', MS='The grafting or inserting of a prosthetic device of alloplastic material into the oral tissue beneath the mucosal or periosteal layer or within the bone. Its purpose is to provide support and retention to a partial or complete denture.'),
  'D003758': NtO(MH='Dental Implantation, Endosseous', MS='Insertion of an implant into the bone of the mandible or maxilla. The implant has an exposed head which protrudes through the mucosa and is a prosthodontic abutment.'),
  'D003759': NtO(MH='Dental Implantation, Endosseous, Endodontic', MS='Insertion of a tapered rod through the root canal into the periapical osseous structure to lengthen the existing root and provide individual tooth stabilization.'),
  'D003760': NtO(MH='Dental Impression Materials', MS='Substances used to create an impression, or negative reproduction, of the teeth and dental arches. These materials include dental plasters and cements, metallic oxide pastes, silicone base materials, or elastomeric materials.'),
  'D003761': NtO(MH='Dental Impression Technique', MS='Procedure of producing an imprint or negative likeness of the teeth and/or edentulous areas. Impressions are made in plastic material which becomes hardened or set while in contact with the tissue. They are later filled with plaster of Paris or artificial stone to produce a facsimile of the oral structures present. Impressions may be made of a full complement of teeth, of areas where some teeth have been removed, or in a mouth from which all teeth have been extracted. (Illustrated Dictionary of Dentistry, 1982)'),
  'D003762': NtO(MH='Dental Instruments', MS='Hand-held tools or implements especially used by dental professionals for the performance of clinical tasks.'),
  'D003763': NtO(MH='Dental Leakage', MS='The seepage of fluids, debris, and micro-organisms between the walls of a prepared dental cavity and the restoration.'),
  'D003764': NtO(MH='Dental Materials', MS='Materials used in the production of dental bases, restorations, impressions, prostheses, etc.'),
  'D003765': NtO(MH='Dental Models', MS='Presentation devices used for patient education and technique training in dentistry.'),
  'D003766': NtO(MH='Dental Occlusion', MS='The relationship of all the components of the masticatory system in normal function. It has special reference to the position and contact of the maxillary and mandibular teeth for the highest efficiency during the excursive movements of the jaw that are essential for mastication. (From Jablonski, Dictionary of Dentistry, 1992, p556, p472)'),
  'D003767': NtO(MH='Dental Occlusion, Balanced', MS='Dental occlusion in which the occlusal contact of the teeth on the working side of the jaw is accompanied by the harmonious contact of the teeth on the opposite (balancing) side. (From Jablonski, Dictionary of Dentistry, 1992, p556)'),
  'D003768': NtO(MH='Dental Occlusion, Centric', MS="Contact between opposing teeth during a person's habitual bite."),
  'D003769': NtO(MH='Dental Occlusion, Traumatic', MS='An occlusion resulting in overstrain and injury to teeth, periodontal tissue, or other oral structures.'),
  'D003770': NtO(MH='Dental Offices', MS="The room or rooms in which the dentist and dental staff provide care. Offices include all rooms in the dentist's office suite."),
  'D003771': NtO(MH='Dental Papilla', MS='Mesodermal tissue enclosed in the invaginated portion of the epithelial enamel organ and giving rise to the dentin and pulp.'),
  'D003772': NtO(MH='Dental Pins', MS='Small cylindrical pieces of metal used to enhance retention.'),
  'D003773': NtO(MH='Dental Plaque', MS='A film that attaches to teeth, often causing DENTAL CARIES and GINGIVITIS. It is composed of MUCINS, secreted from salivary glands, and microorganisms.'),
  'D003774': NtO(MH='Dental Plaque Index', MS='An index which scores the degree of dental plaque accumulation.'),
  'D003775': NtO(MH='Dental Polishing', MS='Creation of a smooth and glossy surface finish on a denture or amalgam.'),
  'D003776': NtO(MH='Dental Porcelain', MS="A type of porcelain used in dental restorations, either jacket crowns or inlays, artificial teeth, or metal-ceramic crowns. It is essentially a mixture of particles of feldspar and quartz, the feldspar melting first and providing a glass matrix for the quartz. Dental porcelain is produced by mixing ceramic powder (a mixture of quartz, kaolin, pigments, opacifiers, a suitable flux, and other substances) with distilled water. (From Jablonski's Dictionary of Dentistry, 1992)"),
  'D003777': NtO(MH='Dental Prophylaxis', MS='Treatment for the prevention of periodontal diseases or other dental diseases by the cleaning of the teeth in the dental office using the procedures of DENTAL SCALING and DENTAL POLISHING. The treatment may include plaque detection, removal of supra- and subgingival plaque and calculus, application of caries-preventing agents, checking of restorations and prostheses and correcting overhanging margins and proximal contours of restorations, and checking for signs of food impaction.'),
  'D003778': NtO(MH='Dentures', MS='An appliance used as an artificial or prosthetic replacement for missing teeth and adjacent tissues. It does not include CROWNS; DENTAL ABUTMENTS; nor TOOTH, ARTIFICIAL.'),
  'D003779': NtO(MH='Denture Design', MS='The plan, delineation, and location of actual structural elements of dentures. The design can relate to retainers, stress-breakers, occlusal rests, flanges, framework, lingual or palatal bars, reciprocal arms, etc.'),
  'D003780': NtO(MH='Denture Repair', MS='The process of reuniting or replacing broken or worn parts of a denture.'),
  'D003781': NtO(MH='Denture Retention', MS='The retention of a denture in place by design, device, or adhesion.'),
  'D003782': NtO(MH='Dental Pulp', MS='A richly vascularized and innervated connective tissue of mesodermal origin, contained in the central cavity of a tooth and delimited by the dentin, and having formative, nutritive, sensory, and protective functions. (Jablonski, Dictionary of Dentistry, 1992)'),
  'D003784': NtO(MH='Dental Pulp Calcification', MS='CALCINOSIS of the DENTAL PULP or ROOT CANAL.'),
  'D003785': NtO(MH='Dental Pulp Capping', MS='Application of a protective agent to an exposed pulp (direct capping) or the remaining thin layer of dentin over a nearly exposed pulp (indirect capping) in order to allow the pulp to recover and maintain its normal vitality and function.'),
  'D003786': NtO(MH='Dental Pulp Cavity', MS='The space in a tooth bounded by the dentin and containing the dental pulp. The portion of the cavity within the crown of the tooth is the pulp chamber; the portion within the root is the pulp canal or root canal.'),
  'D003787': NtO(MH='Dental Pulp Devitalization', MS='The destruction of the vitality of the pulp of the tooth. (From Jablonski, Dictionary of Dentistry, 1992, p243)'),
  'D003788': NtO(MH='Dental Pulp Diseases', MS='Endodontic diseases of the DENTAL PULP inside the tooth, which is distinguished from PERIAPICAL DISEASES of the tissue surrounding the root.'),
  'D003789': NtO(MH='Dental Pulp Exposure', MS='The result of pathological changes in the hard tissue of a tooth caused by carious lesions, mechanical factors, or trauma, which render the pulp susceptible to bacterial invasion from the external environment.'),
  'D003790': NtO(MH='Dental Pulp Necrosis', MS='Death of pulp tissue with or without bacterial invasion. When the necrosis is due to ischemia with superimposed bacterial infection, it is referred to as pulp gangrene. When the necrosis is non-bacterial in origin, it is called pulp mummification.'),
  'D003791': NtO(MH='Dental Pulp Test', MS="Investigations conducted on the physical health of teeth involving use of a tool that transmits hot or cold electric currents on a tooth's surface that can determine problems with that tooth based on reactions to the currents."),
  'D003792': NtO(MH='Dental Records', MS='Data collected during dental examination for the purpose of study, diagnosis, or treatment planning.'),
  'D003793': NtO(MH='Dental Restoration, Permanent', MS='A restoration designed to remain in service for not less than 20 to 30 years, usually made of gold casting, cohesive gold, or amalgam. (Jablonski, Dictionary of Dentistry, 1992)'),
  'D003794': NtO(MH='Dental Restoration, Temporary', MS='A prosthesis or restoration placed for a limited period, from several days to several months, which is designed to seal the tooth and maintain its position until a permanent restoration (DENTAL RESTORATION, PERMANENT) will replace it. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D003795': NtO(MH='Dental Sac', MS='Dense fibrous layer formed from mesodermal tissue that surrounds the epithelial enamel organ. The cells eventually migrate to the external surface of the newly formed root dentin and give rise to the cementoblasts that deposit cementum on the developing root, fibroblasts of the developing periodontal ligament, and osteoblasts of the developing alveolar bone.'),
  'D003796': NtO(MH='Dental Service, Hospital', MS='Hospital department providing dental care.'),
  'D003797': NtO(MH='Dental Staff', MS='Personnel who provide dental service to patients in an organized facility, institution or agency.'),
  'D003798': NtO(MH='Dental Staff, Hospital', MS='Dental personnel practicing in hospitals.'),
  'D003799': NtO(MH='Dental Stress Analysis', MS='The description and measurement of the various factors that produce physical stress upon dental restorations, prostheses, or appliances, materials associated with them, or the natural oral structures.'),
  'D003800': NtO(MH='Dental Technicians', MS='Individuals responsible for fabrication of dental appliances.'),
  'D003801': NtO(MH='Dental Veneers', MS='The use of a layer of tooth-colored material, usually porcelain or acrylic resin, applied to the surface of natural teeth, crowns, or pontics by fusion, cementation, or mechanical retention.'),
  'D003802': NtO(MH='Dentifrices', MS='Any preparations used for cleansing teeth; they usually contain an abrasive, detergent, binder and flavoring agent and may exist in the form of liquid, paste or powder; may also contain medicaments and caries preventives.'),
  'D003803': NtO(MH='Dentigerous Cyst', MS='Most common follicular odontogenic cyst. Occurs in relation to a partially erupted or unerupted tooth with at least the crown of the tooth to which the cyst is attached protruding into the cystic cavity. May give rise to an ameloblastoma and, in rare instances, undergo malignant transformation.'),
  'D003804': NtO(MH='Dentin', MS='The hard portion of the tooth surrounding the pulp, covered by enamel on the crown and cementum on the root, which is harder and denser than bone but softer than enamel, and is thus readily abraded when left unprotected. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D003805': NtO(MH='Dentin Dysplasia', MS='An apparently hereditary disorder of dentin formation, marked by a normal appearance of coronal dentin associated with pulpal obliteration, faulty root formation, and a tendency for peripheral lesions without obvious cause. (From Dorland, 27th ed)'),
  'D003806': NtO(MH='Dentin Permeability', MS='The property of dentin that permits passage of light, heat, cold, and chemical substances. It does not include penetration by microorganisms.'),
  'D003807': NtO(MH='Dentin Sensitivity', MS='Pain associated with exposed DENTIN surfaces of the teeth.'),
  'D003808': NtO(MH='Dentin Solubility', MS='The susceptibility of the DENTIN to dissolution.'),
  'D003809': NtO(MH='Dentin, Secondary', MS='Dentin formed by normal pulp after completion of root end formation.'),
  'D003810': NtO(MH='Dentinogenesis', MS='The formation of dentin. Dentin first appears in the layer between the ameloblasts and odontoblasts and becomes calcified immediately. Formation progresses from the tip of the papilla over its slope to form a calcified cap becoming thicker by the apposition of new layers pulpward. A layer of uncalcified dentin intervenes between the calcified tissue and the odontoblast and its processes. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D003811': NtO(MH='Dentinogenesis Imperfecta', MS='An autosomal dominant disorder of tooth development characterized by opalescent dentin resulting in discoloration of the teeth. The dentin develops poorly with low mineral content while the pulp canal is obliterated.'),
  'D003812': NtO(MH='Dentist-Patient Relations', MS='The psychological relations between the dentist and patient.'),
  'D003813': NtO(MH='Dentistry', MS='The profession concerned with the teeth, oral cavity, and associated structures, and the diagnosis and treatment of their diseases including prevention and the restoration of defective and missing tissue.'),
  'D003814': NtO(MH='Dentistry, Operative', MS="That phase of clinical dentistry concerned with the restoration of parts of existing teeth that are defective through disease, trauma, or abnormal development, to the state of normal function, health, and esthetics, including preventive, diagnostic, biological, mechanical, and therapeutic techniques, as well as material and instrument science and application. (Jablonski's Dictionary of Dentistry, 2d ed, p237)"),
  'D003815': NtO(MH='Dentists', MS='Individuals licensed to practice DENTISTRY.'),
  'D003816': NtO(MH='Dentists, Women', MS='Female dentists.'),
  'D003817': NtO(MH='Dentition', MS='The teeth collectively in the dental arch. Dentition ordinarily refers to the natural teeth in position in their alveoli. Dentition referring to the deciduous teeth is DENTITION, PRIMARY; to the permanent teeth, DENTITION, PERMANENT. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D003818': NtO(MH='Dentition, Mixed', MS="The complement of teeth in the jaws after the eruption of some of the permanent teeth but before all the deciduous teeth are absent. (Boucher's Clinical Dental Terminology, 4th ed)"),
  'D003819': NtO(MH='Denture Bases', MS='The part of a denture that overlies the soft tissue and supports the supplied teeth and is supported in turn by abutment teeth or the residual alveolar ridge. It is usually made of resins or metal or their combination.'),
  'D003820': NtO(MH='Denture Cleansers', MS='Substances used to clean dentures; they are usually alkaline peroxides or hypochlorites, may contain enzymes and release oxygen. Use also for sonic action cleaners.'),
  'D003821': NtO(MH='Denture Identification Marking', MS='Any system of defining ownership of dentures or dental prostheses.'),
  'D003822': NtO(MH='Denture Liners', MS='Material applied to the tissue side of a denture to provide a soft lining to the parts of a denture coming in contact with soft tissue. It cushions contact of the denture with the tissues.'),
  'D003823': NtO(MH='Denture Rebasing', MS='The process of refitting a denture by replacing the denture base material without changing the occlusal relations of the teeth. Rebasing may include adding to the denture base to compensate for resorptive changes to subjacent structures.'),
  'D003824': NtO(MH='Denture, Complete', MS='A denture replacing all natural teeth and associated structures in both the maxilla and mandible.'),
  'D003825': NtO(MH='Denture, Complete, Immediate', MS='A complete denture constructed for replacement of natural teeth immediately after their removal. It does not fit the mouth perfectly and is intended only for functional and cosmetic purposes during the healing process after total extraction. It is to be replaced by the fitted permanent denture.'),
  'D003826': NtO(MH='Denture, Complete, Lower', MS='A complete denture replacing all the natural mandibular teeth and associated structures. It is completely supported by the oral tissue and underlying mandibular bone.'),
  'D003827': NtO(MH='Denture, Complete, Upper', MS='A complete denture replacing all the natural maxillary teeth and associated maxillary structures. It is completely supported by the oral tissue and underlying maxillary bone.'),
  'D003828': NtO(MH='Denture, Overlay', MS='Removable prosthesis constructed over natural teeth or implanted studs.'),
  'D003829': NtO(MH='Denture, Partial', MS='A denture replacing one or more (but not all) natural teeth. It is supported and retained by underlying tissue and some or all of the remaining teeth.'),
  'D003830': NtO(MH='Denture, Partial, Fixed', MS='A partial denture attached to prepared natural teeth, roots, or implants by cementation.'),
  'D003831': NtO(MH='Denture, Partial, Immediate', MS='A partial denture constructed before the teeth it replaces are removed. It is then inserted immediately after the removal of the natural teeth for functional and cosmetic reasons during the healing process. It is to be replaced later by the fitted partial denture.'),
  'D003832': NtO(MH='Denture, Partial, Removable', MS='A partial denture designed and constructed to be removed readily from the mouth.'),
  'D003833': NtO(MH='Denture, Partial, Temporary', MS='A partial denture intended for short-term use in a temporary or emergency situation.'),
  'D003834': NtO(MH='Denture Precision Attachment', MS="A precision device used for attaching a fixed or removable partial denture to the crown of an abutment tooth or a restoration. One type is the intracoronal attachment and the other type is the extracoronal attachment. It consists of a female portion within the coronal portion of the crown of an abutment and a fitted male portion attached to the denture proper. (Jablonski, Dictionary of Dentistry, 1992, p85; from Boucher's Clinical Dental Terminology, 4th ed, p264)"),
  'D003835': NtO(MH='Denturists', MS="Individuals who fabricate and fit DENTURES without the supervision of DENTISTS. (from Stedman's Medical Dictionary, 27th ed) They may or may not have formal education in health sciences, but are well versed in the art of constructing dentures."),
  'D003836': NtO(MH='Deodorants', MS="Agents that remove, correct, repress, or mask undesirable ODORANTS. In personal hygiene, deodorants often contain astringent preparations that reduce SWEATING, referred to as ANTIPERSPIRANTS. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D003837': NtO(MH='Deoxy Sugars', MS=None),
  'D003838': NtO(MH='Deoxyadenine Nucleotides', MS='Adenine nucleotides which contain deoxyribose as the sugar moiety.'),
  'D003839': NtO(MH='Deoxyadenosines', MS='Adenosine molecules which can be substituted in any position, but are lacking one hydroxyl group in the ribose part of the molecule.'),
  'D003840': NtO(MH='Deoxycholic Acid', MS='A bile acid formed by bacterial action from cholate. It is usually conjugated with glycine or taurine. Deoxycholic acid acts as a detergent to solubilize fats for intestinal absorption, is reabsorbed itself, and is used as a choleretic and detergent.'),
  'D003841': NtO(MH='Deoxycytidine', MS='A nucleoside component of DNA composed of CYTOSINE and DEOXYRIBOSE.'),
  'D003842': NtO(MH='Deoxycytidine Kinase', MS='An enzyme that catalyzes reversibly the phosphorylation of deoxycytidine with the formation of a nucleoside diphosphate and deoxycytidine monophosphate. Cytosine arabinoside can also act as an acceptor. All natural nucleoside triphosphates, except deoxycytidine triphosphate, can act as donors. The enzyme is induced by some viruses, particularly the herpes simplex virus (HERPESVIRUS HOMINIS). EC 2.7.1.74.'),
  'D003843': NtO(MH='Deoxycytidine Monophosphate', MS="Deoxycytidine (dihydrogen phosphate). A deoxycytosine nucleotide containing one phosphate group esterified to the deoxyribose moiety in the 2'-,3'- or 5- positions."),
  'D003844': NtO(MH='DCMP Deaminase', MS='An enzyme that catalyzes the hydrolytic deamination of deoxycytidylic acid to deoxyuridylic acid and ammonia. It plays an important role in the regulation of the pool of deoxynucleotides in higher organisms. The enzyme also acts on some 5-substituted deoxycytidylic acids. EC 3.5.4.12.'),
  'D003845': NtO(MH='Deoxycytosine Nucleotides', MS='Cytosine nucleotides which contain deoxyribose as the sugar moiety.'),
  'D003846': NtO(MH='Deoxyepinephrine', MS='Sympathomimetic, vasoconstrictor agent.'),
  'D003847': NtO(MH='Deoxyglucose', MS='2-Deoxy-D-arabino-hexose. An antimetabolite of glucose with antiviral activity.'),
  'D003848': NtO(MH='Deoxyguanine Nucleotides', MS='Guanine nucleotides which contain deoxyribose as the sugar moiety.'),
  'D003849': NtO(MH='Deoxyguanosine', MS='A nucleoside consisting of the base guanine and the sugar deoxyribose.'),
  'D003850': NtO(MH='Deoxyribonuclease I', MS="An enzyme capable of hydrolyzing highly polymerized DNA by splitting phosphodiester linkages, preferentially adjacent to a pyrimidine nucleotide. This catalyzes endonucleolytic cleavage of DNA yielding 5'-phosphodi- and oligonucleotide end-products. The enzyme has a preference for double-stranded DNA."),
  'D003851': NtO(MH='Deoxyribonucleases', MS='Enzymes which catalyze the hydrolases of ester bonds within DNA. EC 3.1.-.'),
  'D003852': NtO(MH='Deoxyribonucleoproteins', MS='Proteins conjugated with deoxyribonucleic acids (DNA) or specific DNA.'),
  'D003853': NtO(MH='Deoxyribonucleosides', MS='A purine or pyrimidine base bonded to DEOXYRIBOSE.'),
  'D003854': NtO(MH='Deoxyribonucleotides', MS='A purine or pyrimidine base bonded to a DEOXYRIBOSE containing a bond to a phosphate group.'),
  'D003855': NtO(MH='Deoxyribose', MS=None),
  'D003856': NtO(MH='Deoxyuracil Nucleotides', MS='Uracil nucleotides which contain deoxyribose as the sugar moiety.'),
  'D003857': NtO(MH='Deoxyuridine', MS="2'-Deoxyuridine. An antimetabolite that is converted to deoxyuridine triphosphate during DNA synthesis. Laboratory suppression of deoxyuridine is used to diagnose megaloblastic anemias due to vitamin B12 and folate deficiencies."),
  'D003858': NtO(MH='Dependency, Psychological', MS='The tendency of an individual or individuals to rely on others for advice, guidance, or support.'),
  'D003859': NtO(MH='Dependent Personality Disorder', MS='A personality disorder characterized by a pervasive and excessive need to be taken care of that leads to submissive and clinging behavior and fears of separation, beginning by early adulthood and present in a variety of contexts. (From DSM-IV, 1994)'),
  'D003861': NtO(MH='Depersonalization', MS='State in which an individual perceives or experiences a sensation of unreality concerning the self or the environment; it is seen in disorders such as schizophrenia, affection disorders, organic mental disorders, and personality disorders. (APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D003862': NtO(MH='Depreciation', MS='Decline in value of capital assets of a permanent or fixed nature over time with use.'),
  'D003863': NtO(MH='Depression', MS='Depressive states usually of moderate intensity in contrast with major depression present in neurotic and psychotic disorders.'),
  'D003864': NtO(MH='Depression, Chemical', MS='The decrease in a measurable parameter of a PHYSIOLOGICAL PROCESS, including cellular, microbial, and plant; immunological, cardiovascular, respiratory, reproductive, urinary, digestive, neural, musculoskeletal, ocular, and skin physiological processes; or METABOLIC PROCESS, including enzymatic and other pharmacological processes, by a drug or other chemical.'),
  'D003865': NtO(MH='Depressive Disorder, Major', MS='Disorder in which five (or more) of the following symptoms have been present during the same 2-week period and represent a change from previous functioning; at least one of the symptoms is either (1) depressed mood or (2) loss of interest or pleasure. Symptoms include: depressed mood most of the day, nearly every daily; markedly diminished interest or pleasure in activities most of the day, nearly every day; significant weight loss when not dieting or weight gain; Insomnia or hypersomnia nearly every day; psychomotor agitation or retardation nearly every day; fatigue or loss of energy nearly every day; feelings of worthlessness or excessive or inappropriate guilt; diminished ability to think or concentrate, or indecisiveness, nearly every day; or  recurrent thoughts of death, recurrent suicidal ideation without a specific plan, or a suicide attempt. (DSM-5)'),
  'D003866': NtO(MH='Depressive Disorder', MS='An affective disorder manifested by either a dysphoric mood or loss of interest or pleasure in usual activities. The mood disturbance is prominent and relatively persistent.'),
  'D003867': NtO(MH='Depth Perception', MS='Perception of three-dimensionality.'),
  'D003868': NtO(MH='Dequalinium', MS='A topical bacteriostat that is available as various salts. It is used in wound dressings and mouth infections and may also have antifungal action, but may cause skin ulceration.'),
  'D003869': NtO(MH='Dermabrasion', MS='The mechanical planing of the SKIN with sand paper, emery paper, or wire brushes, to promote reepithelialization and smoothing of skin disfigured by ACNE scars or dermal NEVI.'),
  'D003870': NtO(MH='Dermacentor', MS='A widely distributed genus of TICKS, in the family IXODIDAE, including a number that infest humans and other mammals. Several are vectors of diseases such as TULAREMIA; ROCKY MOUNTAIN SPOTTED FEVER; COLORADO TICK FEVER; and ANAPLASMOSIS.'),
  'D003871': NtO(MH='Dermatan Sulfate', MS='A naturally occurring glycosaminoglycan found mostly in the skin and in connective tissue. It differs from CHONDROITIN SULFATE A (see CHONDROITIN SULFATES) by containing IDURONIC ACID in place of glucuronic acid, its epimer, at carbon atom 5. (from Merck, 12th ed)'),
  'D003872': NtO(MH='Dermatitis', MS='Any inflammation of the skin.'),
  'D003873': NtO(MH='Dermatitis, Exfoliative', MS='The widespread involvement of the skin by a scaly, erythematous dermatitis occurring either as a secondary or reactive process to an underlying cutaneous disorder (e.g., atopic dermatitis, psoriasis, etc.), or as a primary or idiopathic disease. It is often associated with the loss of hair and nails, hyperkeratosis of the palms and soles, and pruritus. (From Dorland, 27th ed)'),
  'D003874': NtO(MH='Dermatitis Herpetiformis', MS='Rare, chronic, papulo-vesicular disease characterized by an intensely pruritic eruption consisting of various combinations of symmetrical, erythematous, papular, vesicular, or bullous lesions. The disease is strongly associated with the presence of HLA-B8 and HLA-DR3 antigens. A variety of different autoantibodies has been detected in small numbers in patients with dermatitis herpetiformis.'),
  'D003875': NtO(MH='Drug Eruptions', MS='Adverse cutaneous reactions caused by ingestion, parenteral use, or local application of a drug. These may assume various morphologic patterns and produce various types of lesions.'),
  'D003876': NtO(MH='Dermatitis, Atopic', MS='A chronic inflammatory genetically determined disease of the skin marked by increased ability to form reagin (IgE), with increased susceptibility to allergic rhinitis and asthma, and hereditary disposition to a lowered threshold for pruritus. It is manifested by lichenification, excoriation, and crusting, mainly on the flexural surfaces of the elbow and knee. In infants it is known as infantile eczema.'),
  'D003877': NtO(MH='Dermatitis, Contact', MS='A type of acute or chronic skin reaction in which sensitivity is manifested by reactivity to materials or substances coming in contact with the skin. It may involve allergic or non-allergic mechanisms.'),
  'D003878': NtO(MH='Dermatoglyphics', MS='The study of the patterns of ridges of the skin of the fingers, palms, toes, and soles.'),
  'D003879': NtO(MH='Dermatologic Agents', MS='Drugs used to treat or prevent skin disorders or for the routine care of skin.'),
  'D003880': NtO(MH='Dermatology', MS='A medical specialty concerned with the skin, its structure, functions, diseases, and treatment.'),
  'D003881': NtO(MH='Dermatomycoses', MS='Superficial infections of the skin or its appendages by any of various fungi.'),
  'D003882': NtO(MH='Dermatomyositis', MS='A subacute or chronic inflammatory disease of muscle and skin, marked by proximal muscle weakness and a characteristic skin rash. The illness occurs with approximately equal frequency in children and adults. The skin lesions usually take the form of a purplish rash (or less often an exfoliative dermatitis) involving the nose, cheeks, forehead, upper trunk, and arms. The disease is associated with a complement mediated intramuscular microangiopathy, leading to loss of capillaries, muscle ischemia, muscle-fiber necrosis, and perifascicular atrophy. The childhood form of this disease tends to evolve into a systemic vasculitis. Dermatomyositis may occur in association with malignant neoplasms. (From Adams et al., Principles of Neurology, 6th ed, pp1405-6)'),
  'D003883': NtO(MH='Arthrodermataceae', MS='A family of ascomycetous fungi, order Onygenales, characterized by smooth ascospores. Genera in the family include Arthroderma, Keratinomyces, and Ctenomyces. Several well-known anamorphic forms are parasitic upon the skin.'),
  'D003884': NtO(MH='Dermoid Cyst', MS='A tumor consisting of displaced ectodermal structures along the lines of embryonic fusion, the wall being formed of epithelium-lined connective tissue, including skin appendages, and containing keratin, sebum, and hair. (Stedman, 25th ed)'),
  'D003885': NtO(MH='Dermotoxins', MS='Specific substances elaborated by plants, microorganisms or animals that cause damage to the skin; they may be proteins or other specific factors or substances; constituents of spider, jellyfish or other venoms cause dermonecrosis and certain bacteria synthesize dermolytic agents.'),
  'D003886': NtO(MH='Descemet Membrane', MS='A layer of the cornea. It is the basal lamina of the CORNEAL ENDOTHELIUM (from which it is secreted) separating it from the CORNEAL STROMA. It is a homogeneous structure composed of fine collagenous filaments, and slowly increases in thickness with age.'),
  'D003887': NtO(MH='Desensitization, Psychologic', MS='A behavior therapy technique in which deep muscle relaxation is used to inhibit the effects of graded anxiety-evoking stimuli.'),
  'D003888': NtO(MH='Desensitization, Immunologic', MS='Immunosuppression by the administration of increasing doses of antigen. Though the exact mechanism is not clear, the therapy results in an increase in serum levels of allergen-specific IMMUNOGLOBULIN G, suppression of specific IgE, and an increase in suppressor T-cell activity.'),
  'D003889': NtO(MH='Desert Climate', MS='A type of climate characterized by insufficient moisture to support appreciable plant life. It is a climate of extreme aridity, usually of extreme heat, and of negligible rainfall. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003890': NtO(MH='Desiccation', MS='Removal of moisture from a substance (chemical, food, tissue, etc.).'),
  'D003891': NtO(MH='Desipramine', MS='A tricyclic dibenzazepine compound that potentiates neurotransmission. Desipramine selectively blocks reuptake of norepinephrine from the neural synapse, and also appears to impair serotonin transport. This compound also possesses minor anticholinergic activity, through its affinity to muscarinic receptors.'),
  'D003892': NtO(MH='Deslanoside', MS='Deacetyllanatoside C. A cardiotonic glycoside from the leaves of Digitalis lanata.'),
  'D003893': NtO(MH='Desmin', MS='An intermediate filament protein found predominantly in smooth, skeletal, and cardiac muscle cells. Localized at the Z line. MW 50,000 to 55,000 is species dependent.'),
  'D003894': NtO(MH='Deamino Arginine Vasopressin', MS='A synthetic analog of the pituitary hormone, ARGININE VASOPRESSIN. Its action is mediated by the VASOPRESSIN receptor V2. It has prolonged antidiuretic activity, but little pressor effects. It also modulates levels of circulating FACTOR VIII and VON WILLEBRAND FACTOR.'),
  'D003895': NtO(MH='Desmosine', MS='A rare amino acid found in elastin, formed by condensation of four molecules of lysine into a pyridinium ring.'),
  'D003896': NtO(MH='Desmosomes', MS='A type of junction that attaches one cell to its neighbor. One of a number of differentiated regions which occur, for example, where the cytoplasmic membranes of adjacent epithelial cells are closely apposed. It consists of a circular region of each membrane together with associated intracellular microfilaments and an intercellular material which may include, for example, mucopolysaccharides. (From Glick, Glossary of Biochemistry and Molecular Biology, 1990; Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D003897': NtO(MH='Desmosterol', MS='An intermediate in the synthesis of cholesterol.'),
  'D003898': NtO(MH='Desonide', MS='A nonfluorinated corticosteroid anti-inflammatory agent used topically for DERMATOSES.'),
  'D003899': NtO(MH='Desoximetasone', MS='A topical anti-inflammatory glucocorticoid used in DERMATOSES, skin allergies, PSORIASIS, etc.'),
  'D003900': NtO(MH='Desoxycorticosterone', MS='A steroid metabolite that is the 11-deoxy derivative of CORTICOSTERONE and the 21-hydroxy derivative of PROGESTERONE'),
  'D003901': NtO(MH='Desulfovibrio', MS='A genus of gram-negative, anaerobic, rod-shaped bacteria capable of reducing sulfur compounds to hydrogen sulfide. Organisms are isolated from anaerobic mud of fresh and salt water, animal intestines, manure, and feces.'),
  'D003902': NtO(MH='Detergents', MS='Purifying or cleansing agents, usually salts of long-chain aliphatic bases or acids, that exert cleansing (oil-dissolving) and antimicrobial effects through a surface action that depends on possessing both hydrophilic and hydrophobic properties.'),
  'D003903': NtO(MH='Deuterium', MS='The stable isotope of hydrogen. It has one neutron and one proton in the nucleus.'),
  'D003904': NtO(MH='Mitosporic Fungi', MS='A large and heterogenous group of fungi whose common characteristic is the absence of a sexual state. Many of the pathogenic fungi in humans belong to this group.'),
  'D003905': NtO(MH='Deuteroporphyrins', MS='Porphyrins with four methyl and two propionic acid side chains attached to the pyrrole rings.'),
  'D003906': NtO(MH='Developing Countries', MS='Countries in the process of change with economic growth, that is, an increase in production, per capita consumption, and income. The process of economic growth involves better utilization of natural and human resources, which results in a change in the social, political, and economic structures.'),
  'D003907': NtO(MH='Dexamethasone', MS='An anti-inflammatory 9-fluoro-glucocorticoid.'),
  'D003908': NtO(MH='Dexamethasone Isonicotinate', MS='An anti-inflammatory, anti-allergic glucocorticoid that can be administered orally, by inhalation, locally, and parenterally. It may cause water and salt retention.'),
  'D003909': NtO(MH='Dexetimide', MS='A muscarinic antagonist that has been used to treat neuroleptic-induced parkinsonism. Benzetimide is the (-)-enantimorph of dexetimide.'),
  'D003910': NtO(MH='Dextranase', MS='Enzyme that hydrolyzes DEXTRAN to OLIGOSACCHARIDES.'),
  'D003911': NtO(MH='Dextrans', MS='A group of glucose polymers made by certain bacteria. Dextrans are used therapeutically as plasma volume expanders and anticoagulants. They are also commonly used in biological experimentation and in industry for a wide variety of purposes.'),
  'D003912': NtO(MH='Dextrins', MS='A group of low-molecular-weight carbohydrates produced by the hydrolysis of STARCH or GLYCOGEN. They are mixtures of polymers of D-glucose units linked by alpha-(1->4) or alpha-(1->6) glycosidic bonds.'),
  'D003913': NtO(MH='Dextroamphetamine', MS='The d-form of AMPHETAMINE. It is a central nervous system stimulant and a sympathomimetic. It has also been used in the treatment of narcolepsy and of attention deficit disorders and hyperactivity in children. Dextroamphetamine has multiple mechanisms of action including blocking uptake of adrenergics and dopamine, stimulating release of monamines, and inhibiting monoamine oxidase. It is also a drug of abuse and a psychotomimetic.'),
  'D003914': NtO(MH='Dextrocardia', MS='A congenital defect in which the heart is located on the right side of the THORAX instead of on the left side (levocardia, the normal position). When dextrocardia is accompanied with inverted HEART ATRIA, a right-sided STOMACH, and a left-sided LIVER, the combination is called dextrocardia with SITUS INVERSUS. Dextrocardia may adversely affect other thoracic organs.'),
  'D003915': NtO(MH='Dextromethorphan', MS='Methyl analog of DEXTRORPHAN that shows high affinity binding to several regions of the brain, including the medullary cough center. This compound is an NMDA receptor antagonist (RECEPTORS, N-METHYL-D-ASPARTATE) and acts as a non-competitive channel blocker. It is one of the widely used ANTITUSSIVES, and is also used to study the involvement of glutamate receptors in neurotoxicity.'),
  'D003916': NtO(MH='Dextromoramide', MS='An opioid analgesic structurally related to METHADONE and used in the treatment of severe pain. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1070)'),
  'D003917': NtO(MH='Dextrorphan', MS='Dextro form of levorphanol. It acts as a noncompetitive NMDA receptor antagonist, among other effects, and has been proposed as a neuroprotective agent. It is also a metabolite of DEXTROMETHORPHAN.'),
  'D003918': NtO(MH='Dextrothyroxine', MS='The dextrorotary isomer of the synthetic THYROXINE.'),
  'D003919': NtO(MH='Diabetes Insipidus', MS='A disease that is characterized by frequent urination, excretion of large amounts of dilute URINE, and excessive THIRST. Etiologies of diabetes insipidus include deficiency of antidiuretic hormone (also known as ADH or VASOPRESSIN) secreted by the NEUROHYPOPHYSIS, impaired KIDNEY response to ADH, and impaired hypothalamic regulation of thirst.'),
  'D003920': NtO(MH='Diabetes Mellitus', MS='A heterogeneous group of disorders characterized by HYPERGLYCEMIA and GLUCOSE INTOLERANCE.'),
  'D003921': NtO(MH='Diabetes Mellitus, Experimental', MS='Diabetes mellitus induced experimentally by administration of various diabetogenic agents or by PANCREATECTOMY.'),
  'D003922': NtO(MH='Diabetes Mellitus, Type 1', MS='A subtype of DIABETES MELLITUS that is characterized by INSULIN deficiency. It is manifested by the sudden onset of severe HYPERGLYCEMIA, rapid progression to DIABETIC KETOACIDOSIS, and DEATH unless treated with insulin. The disease may occur at any age, but is most common in childhood or adolescence.'),
  'D003923': NtO(MH='Diabetes Mellitus, Lipoatrophic', MS='A type of diabetes mellitus that is characterized by severe INSULIN RESISTANCE and LIPODYSTROPHY. The latter may be generalized, partial, acquired, or congenital (LIPODYSTROPHY, CONGENITAL GENERALIZED).'),
  'D003924': NtO(MH='Diabetes Mellitus, Type 2', MS='A subclass of DIABETES MELLITUS that is not INSULIN-responsive or dependent (NIDDM). It is characterized initially by INSULIN RESISTANCE and HYPERINSULINEMIA; and eventually by GLUCOSE INTOLERANCE; HYPERGLYCEMIA; and overt diabetes. Type II diabetes mellitus is no longer considered a disease exclusively found in adults. Patients seldom develop KETOSIS but often exhibit OBESITY.'),
  'D003925': NtO(MH='Diabetic Angiopathies', MS='VASCULAR DISEASES that are associated with DIABETES MELLITUS.'),
  'D003926': NtO(MH='Diabetic Coma', MS='A state of unconsciousness as a complication of diabetes mellitus. It occurs in cases of extreme HYPERGLYCEMIA or extreme HYPOGLYCEMIA as a complication of INSULIN therapy.'),
  'D003927': NtO(MH='Diet, Diabetic', MS='A course of food intake prescribed for patients, that limits the amount of foods with a high GLYCEMIC INDEX.'),
  'D003928': NtO(MH='Diabetic Nephropathies', MS='KIDNEY injuries associated with diabetes mellitus and affecting KIDNEY GLOMERULUS; ARTERIOLES; KIDNEY TUBULES; and the interstitium. Clinical signs include persistent PROTEINURIA, from microalbuminuria progressing to ALBUMINURIA of greater than 300 mg/24 h, leading to reduced GLOMERULAR FILTRATION RATE and END-STAGE RENAL DISEASE.'),
  'D003929': NtO(MH='Diabetic Neuropathies', MS='Peripheral, autonomic, and cranial nerve disorders that are associated with DIABETES MELLITUS. These conditions usually result from diabetic microvascular injury involving small blood vessels that supply nerves (VASA NERVORUM). Relatively common conditions which may be associated with diabetic neuropathy include third nerve palsy (see OCULOMOTOR NERVE DISEASES); MONONEUROPATHY; mononeuropathy multiplex; diabetic amyotrophy; a painful POLYNEUROPATHY; autonomic neuropathy; and thoracoabdominal neuropathy. (From Adams et al., Principles of Neurology, 6th ed, p1325)'),
  'D003930': NtO(MH='Diabetic Retinopathy', MS='Disease of the RETINA as a complication of DIABETES MELLITUS. It is characterized by the progressive microvascular complications, such as ANEURYSM, interretinal EDEMA, and intraocular PATHOLOGIC NEOVASCULARIZATION.'),
  'D003931': NtO(MH='Diacetyl', MS='Carrier of aroma of butter, vinegar, coffee, and other foods.'),
  'D003932': NtO(MH='Heroin', MS='A narcotic analgesic that may be habit-forming. It is a controlled substance (opium derivative) listed in the U.S. Code of Federal Regulations, Title 21 Parts 329.1, 1308.11 (1987). Sale is forbidden in the United States by Federal statute. (Merck Index, 11th ed)'),
  'D003933': NtO(MH='Diagnosis', MS='The determination of the nature of a disease or condition, or the distinguishing of one disease or condition from another. Assessment may be made through physical examination, laboratory tests, or the likes. Computerized programs may be used to enhance the decision-making process.'),
  'D003935': NtO(MH='Diagnostic Techniques, Cardiovascular', MS='Methods and procedures for the diagnosis of diseases or dysfunction of the cardiovascular system or its organs or demonstration of their physiological processes.'),
  'D003936': NtO(MH='Diagnosis, Computer-Assisted', MS='Application of computer programs designed to assist the physician in solving a diagnostic problem.'),
  'D003937': NtO(MH='Diagnosis, Differential', MS='Determination of which one of two or more diseases or conditions a patient is suffering from by systematically comparing and contrasting results of diagnostic measures.'),
  'D003938': NtO(MH='Diagnostic Techniques, Digestive System', MS='Methods and procedures for the diagnosis of diseases or dysfunction of the digestive system or its organs or demonstration of their physiological processes.'),
  'D003939': NtO(MH='Diagnostic Techniques, Otological', MS='Methods and procedures for the diagnosis of diseases of the ear or of hearing disorders or demonstration of hearing acuity or loss.'),
  'D003940': NtO(MH='Diagnostic Techniques, Endocrine', MS='Methods and procedures for the diagnosis of diseases or dysfunction of the endocrine glands or demonstration of their physiological processes.'),
  'D003941': NtO(MH='Diagnostic Techniques, Ophthalmological', MS='Methods and procedures for the diagnosis of diseases of the eye or of vision disorders.'),
  'D003943': NtO(MH='Diagnostic Techniques, Neurological', MS='Methods and procedures for the diagnosis of diseases of the nervous system, central and peripheral, or demonstration of neurologic function or dysfunction.'),
  'D003944': NtO(MH='Diagnostic Techniques, Obstetrical and Gynecological', MS='Methods and procedures for the diagnosis of conditions related to pregnancy, labor, and the puerperium and of diseases of the female genitalia. It includes also demonstration of genital and pregnancy physiology.'),
  'D003945': NtO(MH='Diagnosis, Oral', MS='Examination of the mouth and teeth toward the identification and diagnosis of intraoral disease or manifestation of non-oral conditions.'),
  'D003947': NtO(MH='Diagnostic Techniques, Radioisotope', MS='Any diagnostic evaluation using radioactive (unstable) isotopes. This diagnosis includes many nuclear medicine procedures as well as radioimmunoassay tests.'),
  'D003948': NtO(MH='Diagnostic Techniques, Respiratory System', MS='Methods and procedures for the diagnosis of diseases of the respiratory tract or its organs. It includes RESPIRATORY FUNCTION TESTS.'),
  'D003949': NtO(MH='Diagnostic Techniques, Surgical', MS='Methods and procedures for the diagnosis of disease or dysfunction by examination of the pathological site or operative field during surgical intervention.'),
  'D003950': NtO(MH='Diagnostic Techniques, Urological', MS='Methods and procedures for the diagnosis of diseases or dysfunction of the urinary tract or its organs or demonstration of its physiological processes.'),
  'D003951': NtO(MH='Diagnostic Errors', MS='Incorrect diagnoses following clinical or technical diagnostic procedures.'),
  'D003952': NtO(MH='Diagnostic Imaging', MS='Any visual display of structural or functional patterns of organs or tissues for diagnostic evaluation. It includes measuring physiologic and metabolic responses to physical and chemical stimuli, as well as ultramicroscopy.'),
  'D003953': NtO(MH='Diagnosis-Related Groups', MS='A system for classifying patient care by relating common characteristics such as diagnosis, treatment, and age to an expected consumption of hospital resources and length of stay. Its purpose is to provide a framework for specifying case mix and to reduce hospital costs and reimbursements and it forms the cornerstone of the prospective payment system.'),
  'D003954': NtO(MH='Diagnostic Services', MS='Organized services for the purpose of providing diagnosis to promote and maintain health.'),
  'D003955': NtO(MH='Diagnostic Tests, Routine', MS='Diagnostic procedures, such as laboratory tests and x-rays, routinely performed on all individuals or specified categories of individuals in a specified situation, e.g., patients being admitted to the hospital. These include routine tests administered to neonates.'),
  'D003956': NtO(MH='Dialysis', MS='A process of selective diffusion through a membrane. It is usually used to separate low-molecular-weight solutes which diffuse through the membrane from the colloidal and high-molecular-weight solutes which do not. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D003957': NtO(MH='Diamfenetide', MS='Anthelmintic. It has been shown to be useful in fasciola infections in sheep.'),
  'D003958': NtO(MH='Diamide', MS='A sulfhydryl reagent which oxidizes sulfhydryl groups to the disulfide form. It is a radiation-sensitizing agent of anoxic bacterial and mammalian cells.'),
  'D003959': NtO(MH='Diamines', MS='Organic chemicals which have two amino groups in an aliphatic chain.'),
  'D003960': NtO(MH='Diaminopimelic Acid', MS='A diamino derivative of heptanedioic acid with amino groups at C-2 and C-6 and the general formula (COOH)CH(NH2)CH2CH2CH2CH(NH2)(COOH).'),
  'D003961': NtO(MH='Dianhydrogalactitol', MS='One of the cytotoxic dihalohexitols that alkylates and cross-links DNA via an epoxide group during all phases of the cell cycle, resulting in a disruption of DNA function and cell cycle arrest. It has antineoplastic activity and also causes bone marrow toxicity.'),
  'D003962': NtO(MH='Dianisidine', MS='Highly toxic compound which can cause skin irritation and sensitization. It is used in manufacture of azo dyes.'),
  'D003963': NtO(MH='Diaper Rash', MS='A type of irritant dermatitis localized to the area in contact with a diaper and occurring most often as a reaction to prolonged contact with urine, feces, or retained soap or detergent.'),
  'D003964': NtO(MH='Diaphragm', MS='The musculofibrous partition that separates the THORACIC CAVITY from the ABDOMINAL CAVITY. Contraction of the diaphragm increases the volume of the thoracic cavity aiding INHALATION.'),
  'D003965': NtO(MH='Diaphragmatic Eventration', MS='A congenital abnormality characterized by the elevation of the DIAPHRAGM dome. It is the result of a thinned diaphragmatic muscle and injured PHRENIC NERVE, allowing the intra-abdominal viscera to push the diaphragm upward against the LUNG.'),
  'D003966': NtO(MH='Camurati-Engelmann Syndrome', MS='An autosomal dominant form of dysplasia that is characterized by progressive thickening of diaphyseal cortex of long bones. Mutations in the gene that encodes TRANSFORMING GROWTH FACTOR BETA1 are one cause of this disorder.'),
  'D003967': NtO(MH='Diarrhea', MS='An increased liquidity or decreased consistency of FECES, such as running stool. Fecal consistency is related to the ratio of water-holding capacity of insoluble solids to total water, rather than the amount of water present. Diarrhea is not hyperdefecation or increased fecal weight.'),
  'D003968': NtO(MH='Diarrhea, Infantile', MS='DIARRHEA occurring in infants from newborn to 24-months old.'),
  'D003969': NtO(MH='Vipoma', MS='A tumor that secretes VASOACTIVE INTESTINAL PEPTIDE, a neuropeptide that causes VASODILATION; relaxation of smooth muscles; watery DIARRHEA; HYPOKALEMIA; and HYPOCHLORHYDRIA. Vipomas, derived from the pancreatic ISLET CELLS, generally are malignant and can secrete other hormones. In most cases, Vipomas are located in the PANCREAS but can be found in extrapancreatic sites.'),
  'D003970': NtO(MH='Diastema', MS='An abnormal opening or fissure between two adjacent teeth.'),
  'D003971': NtO(MH='Diastole', MS='Post-systolic relaxation of the HEART, especially the HEART VENTRICLES.'),
  'D003972': NtO(MH='Diathermy', MS='The induction of local hyperthermia by either short radio waves or high-frequency sound waves.'),
  'D003973': NtO(MH='Diatrizoate', MS='A commonly used x-ray contrast medium. As DIATRIZOATE MEGLUMINE and as Diatrizoate sodium, it is used for gastrointestinal studies, angiography, and urography.'),
  'D003974': NtO(MH='Diatrizoate Meglumine', MS='A versatile contrast medium used for DIAGNOSTIC X-RAY RADIOLOGY.'),
  'D003975': NtO(MH='Diazepam', MS='A benzodiazepine with anticonvulsant, anxiolytic, sedative, muscle relaxant, and amnesic properties and a long duration of action. Its actions are mediated by enhancement of GAMMA-AMINOBUTYRIC ACID activity.'),
  'D003976': NtO(MH='Diazinon', MS='A cholinesterase inhibitor that is used as an organothiophosphorus insecticide.'),
  'D003978': NtO(MH='Diazomethane', MS='A diazonium compound with the formula CH2N2.'),
  'D003979': NtO(MH='Diazonium Compounds', MS='Azo compounds consisting of an aryl or alkyl group that is joined through two nitrogen atoms to an anion (R-N2+X-).'),
  'D003980': NtO(MH='Diazooxonorleucine', MS='An amino acid that inhibits phosphate-activated glutaminase and interferes with glutamine metabolism. It is an antineoplastic antibiotic produced by an unidentified species of Streptomyces from Peruvian soil. (From Merck Index, 11th ed)'),
  'D003981': NtO(MH='Diazoxide', MS='A benzothiadiazine derivative that is a peripheral vasodilator used for hypertensive emergencies. It lacks diuretic effect, apparently because it lacks a sulfonamide group.'),
  'D003982': NtO(MH='Dibekacin', MS='Analog of KANAMYCIN with antitubercular as well as broad-spectrum antimicrobial properties.'),
  'D003983': NtO(MH='Dibenzylchlorethamine', MS='An alpha adrenergic antagonist.'),
  'D003984': NtO(MH='Dibenzazepines', MS='Compounds with two BENZENE rings fused to AZEPINES.'),
  'D003986': NtO(MH='Dibenzocycloheptenes', MS='A family of tricyclic hydrocarbons whose members include many of the commonly used tricyclic antidepressants (ANTIDEPRESSIVE AGENTS, TRICYCLIC).'),
  'D003987': NtO(MH='Dibenzothiazepines', MS=None),
  'D003988': NtO(MH='Dibenzothiepins', MS=None),
  'D003989': NtO(MH='Dibenzoxazepines', MS=None),
  'D003990': NtO(MH='Dibenzoxepins', MS=None),
  'D003991': NtO(MH='Dibromothymoquinone', MS='At low concentrations, this compound inhibits reduction of conventional hydrophilic electron acceptors, probably acting as a plastoquinone antagonist. At higher concentrations, it acts as an electron acceptor, intercepting electrons either before or at the site of its inhibitory activity.'),
  'D003992': NtO(MH='Dibucaine', MS='A local anesthetic of the amide type now generally used for surface anesthesia. It is one of the most potent and toxic of the long-acting local anesthetics and its parenteral use is restricted to spinal anesthesia. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1006)'),
  'D003993': NtO(MH='Dibutyl Phthalate', MS='A plasticizer used in most plastics and found in water, air, soil, plants and animals. It may have some adverse effects with long-term exposure.'),
  'D003994': NtO(MH='Bucladesine', MS='A cyclic nucleotide derivative that mimics the action of endogenous CYCLIC AMP and is capable of permeating the cell membrane. It has vasodilator properties and is used as a cardiac stimulant. (From Merck Index, 11th ed)'),
  'D003995': NtO(MH='Dibutyryl Cyclic GMP', MS="N-(1-Oxobutyl)-cyclic 3',5'-(hydrogen phosphate)-2'-butanoate guanosine. A derivative of cyclic GMP. It has a higher resistance to extracellular and intracellular phosphodiesterase than cyclic GMP."),
  'D003996': NtO(MH='Dicamba', MS='A chlorinated organic herbicide.'),
  'D003997': NtO(MH='Dicarbethoxydihydrocollidine', MS='1,4-Dihydro-2,4,6-trimethyl-3,5-pyridinedicarboxylic acid diethyl ester.'),
  'D003998': NtO(MH='Dicarboxylic Acids', MS='Acyclic acids that contain two carboxyl groups and have the formula HO2C-R-CO2H, where R may be an aromatic or aliphatic group.'),
  'D003999': NtO(MH='Dichloroacetic Acid', MS='A derivative of ACETIC ACID that contains two CHLORINE atoms attached to its methyl group.'),
  'D004000': NtO(MH='Dichloroethylenes', MS='Toxic chlorinated unsaturated hydrocarbons. Include both the 1,1- and 1,2-dichloro isomers. Both isomers are toxic, but 1,1-dichloroethylene is the more potent CNS depressant and hepatotoxin. It is used in the manufacture of thermoplastic polymers.'),
  'D004002': NtO(MH='Clodronic Acid', MS='A diphosphonate which affects calcium metabolism. It inhibits bone resorption and soft tissue calcification.'),
  'D004003': NtO(MH='Dichlorophen', MS='Nontoxic laxative vermicide effective for taenia infestation. It tends to produce colic and nausea. It is also used as a veterinary fungicide, anthelmintic, and antiprotozoan. (From Merck, 11th ed.)'),
  'D004004': NtO(MH='Dichlororibofuranosylbenzimidazole', MS='An RNA polymerase II transcriptional inhibitor. This compound terminates transcription prematurely by selective inhibition of RNA synthesis. It is used in research to study underlying mechanisms of cellular regulation.'),
  'D004005': NtO(MH='Dichlorphenamide', MS='A carbonic anhydrase inhibitor that is used in the treatment of glaucoma.'),
  'D004006': NtO(MH='Dichlorvos', MS='An organophosphorus insecticide that inhibits ACETYLCHOLINESTERASE.'),
  'D004007': NtO(MH='Dichotic Listening Tests', MS='Tests for central hearing disorders based on the competing message technique (binaural separation).'),
  'D004008': NtO(MH='Diclofenac', MS='A non-steroidal anti-inflammatory agent (NSAID) with antipyretic and analgesic actions. It is primarily available as the sodium salt.'),
  'D004009': NtO(MH='Dicloxacillin', MS='One of the PENICILLINS which is resistant to PENICILLINASE.'),
  'D004010': NtO(MH='Dicofol', MS='An organochlorine insecticide.'),
  'D004011': NtO(MH='Dicrocoeliasis', MS='Infection with flukes of the genus Dicrocoelium.'),
  'D004012': NtO(MH='Dicrocoeliidae', MS='A family of flattened, translucent flukes (TREMATODA) occurring in the gut, gallbladder, bile ducts, liver or pancreatic ducts of amphibians, reptiles, birds, and mammals. (Noble et al., Parasitology: the Biology of Animal Parasites, 6th ed, p188)'),
  'D004013': NtO(MH='Dicrocoelium', MS='A genus of trematode liver flukes of the family Dicrocoeliidae which includes the species dendriticum and hospes. It occurs in the biliary passages or liver of many vertebrates including man. The intermediate hosts are mainly mollusks but occasionally ants.'),
  'D004014': NtO(MH='Dictionaries as Topic', MS='Works about reference books containing words and their meaning.'),
  'D004015': NtO(MH='Dictionaries, Chemical as Topic', MS='Works about sources listing chemicals and their characteristics.'),
  'D004016': NtO(MH='Dictionaries, Classical as Topic', MS='Works about or subject matter related to reference works on the Greco-Roman world.'),
  'D004017': NtO(MH='Dictionaries, Dental as Topic', MS='Works about or subject matter related to reference works listing and describing concepts related to oral medicine and dentistry.'),
  'D004018': NtO(MH='Dictionaries, Medical as Topic', MS='Works about or subject matter related to reference works listing and describing concepts related to medicine.'),
  'D004019': NtO(MH='Dictionaries, Pharmaceutic as Topic', MS='Works about specialized reference lists and descriptions of drugs and related concepts.'),
  'D004020': NtO(MH='Dictionary, Polyglot', MS='Reference list of definitions of words repeated in several languages.'),
  'D004021': NtO(MH='Dictyocaulus', MS='Nematodes parasitic in the bronchi of herbivorous animals.'),
  'D004022': NtO(MH='Dictyocaulus Infections', MS='Infection with nematodes of the genus DICTYOCAULUS. In deer, cattle, sheep, and horses the bronchi are the site of infestation.'),
  'D004023': NtO(MH='Dictyostelium', MS='A genus of protozoa, formerly also considered a fungus. Its natural habitat is decaying forest leaves, where it feeds on bacteria. D. discoideum is the best-known species and is widely used in biomedical research.'),
  'D004024': NtO(MH='Dicyclohexylcarbodiimide', MS="A carbodiimide that is used as a chemical intermediate and coupling agent in peptide synthesis. (From Hawley's Condensed Chemical Dictionary, 12th ed)"),
  'D004025': NtO(MH='Dicyclomine', MS='A muscarinic antagonist used as an antispasmodic and in urinary incontinence. It has little effect on glandular secretion or the cardiovascular system. It does have some local anesthetic properties and is used in gastrointestinal, biliary, and urinary tract spasms.'),
  'D004026': NtO(MH='Dieldrin', MS='An organochlorine insecticide whose use has been cancelled or suspended in the United States. It has been used to control locusts, tropical disease vectors, in termite control by direct soil injection, and non-food seed and plant treatment. (From HSDB)'),
  'D004027': NtO(MH='Diencephalon', MS='The paired caudal parts of the PROSENCEPHALON from which the THALAMUS; HYPOTHALAMUS; EPITHALAMUS; and SUBTHALAMUS are derived.'),
  'D004028': NtO(MH='Dienestrol', MS='A synthetic, non-steroidal estrogen structurally related to stilbestrol. It is used, usually as the cream, in the treatment of menopausal and postmenopausal symptoms.'),
  'D004029': NtO(MH='Dientamoeba', MS='A genus of minute EUKARYOTES that are characterized by the preponderance of binucleate over uninucleate forms, the presence of several distinct granules in the karyosome, and the lack of a cystic stage. It is parasitic in the large intestine of humans and certain monkeys.'),
  'D004030': NtO(MH='Dientamoebiasis', MS='Gastrointestinal infection with organisms of the genus DIENTAMOEBA.'),
  'D004031': NtO(MH='Diestrus', MS='A phase of the ESTROUS CYCLES that follows METESTRUS. Diestrus is a period of sexual quiescence separating phases of ESTRUS in polyestrous animals.'),
  'D004032': NtO(MH='Diet', MS='Regular course of eating and drinking adopted by a person or animal.'),
  'D004033': NtO(MH='Diet Fads', MS='Diets which become fashionable, but which are not necessarily nutritious.(Lehninger 1982, page 484)'),
  'D004034': NtO(MH='Diet Surveys', MS='Systematic collections of factual data pertaining to the diet of a human population within a given geographic area.'),
  'D004035': NtO(MH='Diet Therapy', MS='Adjusting the quantity and quality of food intake to improve health status of an individual. This term does not include the methods of food intake (NUTRITIONAL SUPPORT).'),
  'D004036': NtO(MH='Diet, Atherogenic', MS='A diet that contributes to the development and acceleration of ATHEROGENESIS.'),
  'D004037': NtO(MH='Diet, Cariogenic', MS='A consistent pattern of food and drink consumption that contributes to the development and advancement of DENTAL CARIES.'),
  'D004038': NtO(MH='Diet, Reducing', MS='A diet designed to cause an individual to lose weight.'),
  'D004039': NtO(MH='Diet, Sodium-Restricted', MS='A diet which contains very little sodium chloride. It is prescribed by some for hypertension and for edematous states. (Dorland, 27th ed)'),
  'D004040': NtO(MH='Dietary Carbohydrates', MS='Carbohydrates present in food comprising digestible sugars and starches and indigestible cellulose and other dietary fibers. The former are the major source of energy. The sugars are in beet and cane sugar, fruits, honey, sweet corn, corn syrup, milk and milk products, etc.; the starches are in cereal grains, legumes (FABACEAE), tubers, etc. (From Claudio & Lagua, Nutrition and Diet Therapy Dictionary, 3d ed, p32, p277)'),
  'D004041': NtO(MH='Dietary Fats', MS='Fats present in food, especially in animal products such as meat, meat products, butter, ghee. They are present in lower amounts in nuts, seeds, and avocados.'),
  'D004042': NtO(MH='Dietary Fats, Unsaturated', MS='Unsaturated fats or oils used in foods or as a food.'),
  'D004043': NtO(MH='Dietary Fiber', MS='The remnants of plant cell walls that are resistant to digestion by the alimentary enzymes of man. It comprises various polysaccharides and lignins.'),
  'D004044': NtO(MH='Dietary Proteins', MS='Proteins obtained from foods. They are the main source of the ESSENTIAL AMINO ACIDS.'),
  'D004045': NtO(MH='Dietary Services', MS='Services provided by dietitians or nutritionists to meet the nutritional needs of individuals, including consultation with other professional personnel.'),
  'D004046': NtO(MH='Dietetics', MS='The application of nutritional principles to regulation of the diet and feeding persons or groups of persons.'),
  'D004047': NtO(MH='Diethyl Pyrocarbonate', MS='Preservative for wines, soft drinks, and fruit juices and a gentle esterifying agent.'),
  'D004048': NtO(MH='Diethylamines', MS='Diethylamine with the formula CH3CH2NHCH2CH3 and its derivatives.'),
  'D004049': NtO(MH='Diethylcarbamazine', MS='An anthelmintic used primarily as the citrate in the treatment of filariasis, particularly infestations with Wucheria bancrofti or Loa loa.'),
  'D004050': NtO(MH='Ditiocarb', MS='A chelating agent that has been used to mobilize toxic metals from the tissues of humans and experimental animals. It is the main metabolite of DISULFIRAM.'),
  'D004051': NtO(MH='Diethylhexyl Phthalate', MS='An ester of phthalic acid. It appears as a light-colored, odorless liquid and is used as a plasticizer for many resins and elastomers.'),
  'D004052': NtO(MH='Diethylnitrosamine', MS='A nitrosamine derivative with alkylating, carcinogenic, and mutagenic properties.'),
  'D004053': NtO(MH='Diethylpropion', MS='A appetite depressant considered to produce less central nervous system disturbance than most drugs in this therapeutic category. It is also considered to be among the safest for patients with hypertension. (From AMA Drug Evaluations Annual, 1994, p2290)'),
  'D004054': NtO(MH='Diethylstilbestrol', MS='A synthetic nonsteroidal estrogen used in the treatment of menopausal and postmenopausal disorders. It was also used formerly as a growth promoter in animals. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), diethylstilbestrol has been listed as a known carcinogen. (Merck, 11th ed)'),
  'D004055': NtO(MH='Differential Thermal Analysis', MS='Technique by which phase transitions of chemical reactions can be followed by observation of the heat absorbed or liberated.'),
  'D004056': NtO(MH='Differential Threshold', MS='The smallest difference which can be discriminated between two stimuli or one which is barely above the threshold.'),
  'D004057': NtO(MH='Hyperostosis, Diffuse Idiopathic Skeletal', MS='A disease of elderly men characterized by large osteophytes that bridge vertebrae and ossification of ligaments and tendon insertions.'),
  'D004058': NtO(MH='Diffusion', MS='The tendency of a gas or solute to pass from a point of higher pressure or concentration to a point of lower pressure or concentration and to distribute itself throughout the available space. Diffusion, especially FACILITATED DIFFUSION, is a major mechanism of BIOLOGICAL TRANSPORT.'),
  'D004059': NtO(MH='Diffusion of Innovation', MS='The broad dissemination of new ideas, procedures, techniques, materials, and devices and the degree to which these are accepted and used.'),
  'D004060': NtO(MH='Diflucortolone', MS='A topical glucocorticoid used in various DERMATOSES. It is absorbed through the skin, bound to plasma albumin, and may cause adrenal suppression. It is also administered as the valerate.'),
  'D004061': NtO(MH='Diflunisal', MS='A salicylate derivative and anti-inflammatory analgesic with actions and side effects similar to those of ASPIRIN.'),
  'D004062': NtO(MH='DiGeorge Syndrome', MS='Congenital syndrome characterized by a wide spectrum of characteristics including the absence of the THYMUS and PARATHYROID GLANDS resulting in T-cell immunodeficiency, HYPOCALCEMIA, defects in the outflow tract of the heart, and craniofacial anomalies.'),
  'D004063': NtO(MH='Digestion', MS='The process of breakdown of food for metabolism and use by the body.'),
  'D004064': NtO(MH='Digestive System', MS='A group of organs stretching from the MOUTH to the ANUS, serving to breakdown foods, assimilate nutrients, and eliminate waste. In humans, the digestive system includes the GASTROINTESTINAL TRACT and the accessory glands (LIVER; BILIARY TRACT; PANCREAS).'),
  'D004065': NtO(MH='Digestive System Abnormalities', MS='Congenital structural abnormalities of the DIGESTIVE SYSTEM.'),
  'D004066': NtO(MH='Digestive System Diseases', MS='Diseases in any part of the GASTROINTESTINAL TRACT or the accessory organs (LIVER; BILIARY TRACT; PANCREAS).'),
  'D004067': NtO(MH='Digestive System Neoplasms', MS='Tumors or cancer of the DIGESTIVE SYSTEM.'),
  'D004068': NtO(MH='Digestive System Physiological Phenomena', MS='Properties and processes of the DIGESTIVE SYSTEM as a whole or of any of its parts.'),
  'D004070': NtO(MH='Digitalis', MS='A genus of toxic herbaceous Eurasian plants of the Plantaginaceae which yield cardiotonic DIGITALIS GLYCOSIDES. The most useful species are Digitalis lanata and D. purpurea.'),
  'D004071': NtO(MH='Digitalis Glycosides', MS='Glycosides from plants of the genus DIGITALIS. Some of these are useful as cardiotonic and anti-arrhythmia agents. Included also are semi-synthetic derivatives of the naturally occurring glycosides. The term has sometimes been used more broadly to include all CARDIAC GLYCOSIDES, but here is restricted to those related to Digitalis.'),
  'D004072': NtO(MH='Digitonin', MS='A glycoside obtained from Digitalis purpurea; the aglycone is digitogenin which is bound to five sugars. Digitonin solubilizes lipids, especially in membranes and is used as a tool in cellular biochemistry, and reagent for precipitating cholesterol. It has no cardiac effects.'),
  'D004073': NtO(MH='Digitoxigenin', MS='3 beta,14-Dihydroxy-5 beta-card-20(22)enolide. A cardenolide which is the aglycon of digitoxin. Synonyms: Cerberigenin; Echujetin; Evonogenin; Thevetigenin.'),
  'D004074': NtO(MH='Digitoxin', MS='A cardiac glycoside sometimes used in place of DIGOXIN. It has a longer half-life than digoxin; toxic effects, which are similar to those of digoxin, are longer lasting. (From Martindale, The Extra Pharmacopoeia, 30th ed, p665)'),
  'D004075': NtO(MH='Diglycerides', MS=None),
  'D004076': NtO(MH='Digoxigenin', MS='3 beta,12 beta,14-Trihydroxy-5 beta-card-20(22)-enolide. A cardenolide which is the aglycon of digoxin. Can be obtained by hydrolysis of digoxin or from Digitalis orientalis L. and Digitalis lanata Ehrh.'),
  'D004077': NtO(MH='Digoxin', MS='A cardiotonic glycoside obtained mainly from Digitalis lanata; it consists of three sugars and the aglycone DIGOXIGENIN. Digoxin has positive inotropic and negative chronotropic activity. It is used to control ventricular rate in ATRIAL FIBRILLATION and in the management of congestive heart failure with atrial fibrillation. Its use in congestive heart failure and sinus rhythm is less certain. The margin between toxic and therapeutic doses is small. (From Martindale, The Extra Pharmacopoeia, 30th ed, p666)'),
  'D004078': NtO(MH='Dihydralazine', MS='1,4-Dihydrazinophthalazine. An antihypertensive agent with actions and uses similar to those of HYDRALAZINE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p354)'),
  'D004079': NtO(MH='Dihydro-beta-Erythroidine', MS='Dihydro analog of beta-erythroidine, which is isolated from the seeds and other plant parts of Erythrina sp. Leguminosae. It is an alkaloid with curarimimetic properties.'),
  'D004080': NtO(MH='Dihydroorotase', MS='An enzyme that, in the course of pyrimidine biosynthesis, catalyzes ring closure by removal of water from N-carbamoylaspartate to yield dihydro-orotic acid. EC 3.5.2.3.'),
  'D004081': NtO(MH='Dihydroorotate Oxidase', MS='An enzyme that in the course of pyrimidine biosynthesis, catalyzes the oxidation of dihydro-orotic acid to orotic acid utilizing oxygen as the electron acceptor. This enzyme is a flavoprotein which contains both FLAVIN-ADENINE DINUCLEOTIDE and FLAVIN MONONUCLEOTIDE as well as iron-sulfur centers. EC 1.3.3.1.'),
  'D004082': NtO(MH='Dihydroalprenolol', MS='Hydrogenated alprenolol derivative where the extra hydrogens are often tritiated. This radiolabeled form of ALPRENOLOL, a beta-adrenergic blocker, is used to label the beta-adrenergic receptor for isolation and study.'),
  'D004083': NtO(MH='Cholestanol', MS='A cholesterol derivative found in human feces, gallstones, eggs, and other biological matter.'),
  'D004087': NtO(MH='Dihydroergotamine', MS='A 9,10alpha-dihydro derivative of ERGOTAMINE. It is used as a vasoconstrictor, specifically for the therapy of MIGRAINE DISORDERS.'),
  'D004088': NtO(MH='Dihydroergotoxine', MS='A mixture of three different hydrogenated derivatives of ERGOTAMINE: DIHYDROERGOCORNINE; DIHYDROERGOCRISTINE; and DIHYDROERGOCRYPTINE. Dihydroergotoxine has been proposed to be a neuroprotective agent and a nootropic agent. The mechanism of its therapeutic actions is not clear, but it can act as an alpha-adrenergic antagonist and a dopamine agonist. The methanesulfonate salts of this mixture of alkaloids are called ERGOLOID MESYLATES.'),
  'D004090': NtO(MH='Dihydromorphine', MS='A semisynthetic analgesic used in the study of narcotic receptors.'),
  'D004091': NtO(MH='Hydromorphone', MS='An opioid analgesic made from MORPHINE and used mainly as an analgesic. It has a shorter duration of action than morphine.'),
  'D004092': NtO(MH='20-alpha-Dihydroprogesterone', MS='A biologically active 20-alpha-reduced metabolite of PROGESTERONE. It is converted from progesterone to 20-alpha-hydroxypregn-4-en-3-one by the 20-ALPHA-HYDROXYSTEROID DEHYDROGENASE in the CORPUS LUTEUM and the PLACENTA.'),
  'D004093': NtO(MH='Dihydropteridine Reductase', MS='An enzyme that catalyzes the reduction of 6,7-dihydropteridine to 5,6,7,8-tetrahydropteridine in the presence of NADP+. Defects in the enzyme are a cause of PHENYLKETONURIA II. Formerly listed as EC 1.6.99.7.'),
  'D004094': NtO(MH='Dihydropteroate Synthase', MS='An enzyme that catalyzes the formation of dihydropteroate from p-aminobenzoic acid and dihydropteridine-hydroxymethyl-pyrophosphate. EC 2.5.1.15.'),
  'D004095': NtO(MH='Dihydropyridines', MS='Pyridine moieties which are partially saturated by the addition of two hydrogen atoms in any position.'),
  'D004096': NtO(MH='Dihydrostreptomycin Sulfate', MS='A semi-synthetic aminoglycoside antibiotic that is used in the treatment of TUBERCULOSIS.'),
  'D004097': NtO(MH='Dihydrotachysterol', MS='A VITAMIN D that can be regarded as a reduction product of vitamin D2.'),
  'D004098': NtO(MH='Dihydroxyacetone', MS='A ketotriose compound. Its addition to blood preservation solutions results in better maintenance of 2,3-diphosphoglycerate levels during storage. It is readily phosphorylated to dihydroxyacetone phosphate by triokinase in erythrocytes. In combination with naphthoquinones it acts as a sunscreening agent.'),
  'D004099': NtO(MH='Dihydroxyacetone Phosphate', MS='An important intermediate in lipid biosynthesis and in glycolysis.'),
  'D004100': NtO(MH='Dihydroxycholecalciferols', MS='Cholecalciferols substituted with two hydroxy groups in any position.'),
  'D004101': NtO(MH='Dihydroxydihydrobenzopyrenes', MS='Benzopyrenes saturated in any two adjacent positions and substituted with two hydroxyl groups in any position. The majority of these compounds have carcinogenic or mutagenic activity.'),
  'D004102': NtO(MH='Dihydroxytryptamines', MS='Tryptamine substituted with two hydroxyl groups in any position. Some are cytotoxic serotonin analogs that are preferentially taken up by serotonergic neurons and then destroy those neurons.'),
  'D004103': NtO(MH='Iodoquinol', MS='One of the halogenated 8-quinolinols widely used as an intestinal antiseptic, especially as an antiamebic agent. It is also used topically in other infections and may cause CNS and eye damage. It is known by very many similar trade names world-wide.'),
  'D004104': NtO(MH='Diiodothyronines', MS='These metabolites of THYROXINE are formed by the deiodination of T3 or reverse T3.'),
  'D004105': NtO(MH='Diiodotyrosine', MS='A product from the iodination of MONOIODOTYROSINE. In the biosynthesis of thyroid hormones, diiodotyrosine residues are coupled with other monoiodotyrosine or diiodotyrosine residues to form T4 or T3 thyroid hormones (THYROXINE and TRIIODOTHYRONINE).'),
  'D004106': NtO(MH='Dilatation', MS='The act of dilating.'),
  'D004107': NtO(MH='Dilatation and Curettage', MS='Dilatation of the cervix uteri followed by a scraping of the endometrium with a curette.'),
  'D004108': NtO(MH='Dilatation, Pathologic', MS="The condition of an anatomical structure's being dilated beyond normal dimensions."),
  'D004109': NtO(MH='Dilazep', MS='Coronary vasodilator with some antiarrhythmic activity.'),
  'D004110': NtO(MH='Diltiazem', MS='A benzothiazepine derivative with vasodilating action due to its antagonism of the actions of CALCIUM ion on membrane functions.'),
  'D004111': NtO(MH='Dimenhydrinate', MS='A drug combination that contains diphenhydramine and theophylline. It is used for treating VERTIGO, MOTION SICKNESS, and NAUSEA associated with PREGNANCY.'),
  'D004112': NtO(MH='Dimercaprol', MS='An anti-gas warfare agent that is effective against Lewisite (dichloro(2-chlorovinyl)arsine) and formerly known as British Anti-Lewisite or BAL. It acts as a chelating agent and is used in the treatment of arsenic, gold, and other heavy metal poisoning.'),
  'D004113': NtO(MH='Succimer', MS='A mercaptodicarboxylic acid used as an antidote to heavy metal poisoning because it forms strong chelates with them.'),
  'D004114': NtO(MH='Dimethadione', MS='An anticonvulsant that is the active metabolite of TRIMETHADIONE.'),
  'D004115': NtO(MH='Dimethindene', MS='A histamine H1 antagonist. It is used in hypersensitivity reactions, in rhinitis, for pruritus, and in some common cold remedies.'),
  'D004116': NtO(MH='Dimethisterone', MS='A synthetic progestational hormone without significant estrogenic or androgenic properties. It was formerly used as the progestational component in SEQUENTIAL ORAL CONTRACEPTIVE AGENTS .'),
  'D004117': NtO(MH='Dimethoate', MS='An organothiophosphorus cholinesterase inhibitor that is used as a systemic and contact insecticide.'),
  'D004118': NtO(MH='Dimethoxyphenylethylamine', MS='A derivative of phenethylamine containing two substituent methoxy groups in the phenyl ring.'),
  'D004119': NtO(MH='Dimethyl Adipimidate', MS='Bifunctional cross-linking agent that links covalently free amino groups of proteins or polypeptides, including those in cell membranes. It is used as reagent or fixative in immunohistochemistry and is a proposed antisickling agent.'),
  'D004120': NtO(MH='Dimethyl Suberimidate', MS='The methyl imidoester of suberic acid used to produce cross links in proteins. Each end of the imidoester will react with an amino group in the protein molecule to form an amidine.'),
  'D004121': NtO(MH='Dimethyl Sulfoxide', MS='A highly polar organic liquid, that is used widely as a chemical solvent. Because of its ability to penetrate biological membranes, it is used as a vehicle for topical application of pharmaceuticals. It is also used to protect tissue during CRYOPRESERVATION. Dimethyl sulfoxide shows a range of pharmacological activity including analgesia and anti-inflammation.'),
  'D004122': NtO(MH='Dimethylallyltranstransferase', MS='An enzyme that, in the pathway of cholesterol biosynthesis, catalyzes the condensation of isopentenyl pyrophosphate and dimethylallylpyrophosphate to yield pyrophosphate and geranylpyrophosphate. The enzyme then catalyzes the condensation of the latter compound with another molecule of isopentenyl pyrophosphate to yield pyrophosphate and farnesylpyrophosphate. EC 2.5.1.1.'),
  'D004123': NtO(MH='Dimethylamines', MS='Derivatives of dimethylamine (the structural formula CH3NHCH3).'),
  'D004124': NtO(MH='p-Dimethylaminoazobenzene', MS='A reagent used mainly to induce experimental liver cancer. According to the Fourth Annual Report on Carcinogens (NTP 85-002, p. 89) published in 1985, this compound "may reasonably be anticipated to be a carcinogen." (Merck, 11th ed)'),
  'D004125': NtO(MH='Dimethyldithiocarbamate', MS='A chemical that acts as a dopamine beta-hydroxylase inhibitor. Its salts are agricultural fungicides. It is inferior to diethyldithiocarbamate as a chelating agent.'),
  'D004126': NtO(MH='Dimethylformamide', MS='A formamide in which the amino hydrogens are replaced by methyl groups.'),
  'D004127': NtO(MH='Dimethylhydrazines', MS='Hydrazines substituted with two methyl groups in any position.'),
  'D004128': NtO(MH='Dimethylnitrosamine', MS='A nitrosamine derivative with alkylating, carcinogenic, and mutagenic properties. It causes serious liver damage and is a hepatocarcinogen in rodents.'),
  'D004129': NtO(MH='Dimethylpolysiloxanes', MS='Silicone polymers which consist of silicon atoms substituted with methyl groups and linked by oxygen atoms. They comprise a series of biocompatible materials used as liquids, gels or solids; as film for artificial membranes, gels for implants, and liquids for drug vehicles; and as antifoaming agents.'),
  'D004130': NtO(MH='N,N-Dimethyltryptamine', MS='An N-methylated indoleamine derivative and serotonergic hallucinogen which occurs naturally and ubiquitously in several plant species including Psychotria veridis. It also occurs in trace amounts in mammalian brain, blood, and urine, and is known to act as an agonist or antagonist of certain SEROTONIN RECEPTORS.'),
  'D004131': NtO(MH='Dimetridazole', MS='A compound used to treat histomoniasis in poultry.'),
  'D004132': NtO(MH='Diflubenzuron', MS='An insect growth regulator which interferes with the formation of the insect cuticle. It is effective in the control of mosquitoes and flies.'),
  'D004133': NtO(MH='Diminazene', MS='An effective trypanocidal agent.'),
  'D004134': NtO(MH='Dimyristoylphosphatidylcholine', MS='A synthetic phospholipid used in liposomes and lipid bilayers for the study of biological membranes.'),
  'D004135': NtO(MH='Dinitolmide', MS='A coccidiostat for poultry.'),
  'D004136': NtO(MH='Dinitrobenzenes', MS='Benzene derivatives which are substituted with two nitro groups in the ortho, meta or para positions.'),
  'D004137': NtO(MH='Dinitrochlorobenzene', MS='A skin irritant that may cause dermatitis of both primary and allergic types. Contact sensitization with DNCB has been used as a measure of cellular immunity. DNCB is also used as a reagent for the detection and determination of pyridine compounds.'),
  'D004138': NtO(MH='Dinitrocresols', MS='Derivatives of CRESOLS containing two nitro groups.'),
  'D004139': NtO(MH='Dinitrofluorobenzene', MS='Irritants and reagents for labeling terminal amino acid groups.'),
  'D004140': NtO(MH='Dinitrophenols', MS='Organic compounds that contain two nitro groups attached to a phenol.'),
  'D004141': NtO(MH='Dinoflagellida', MS='Flagellate EUKARYOTES, found mainly in the oceans. They are characterized by the presence of transverse and longitudinal flagella which propel the organisms in a rotating manner through the water. Dinoflagellida were formerly members of the class Phytomastigophorea under the old five kingdom paradigm.'),
  'D004143': NtO(MH='Dioctyl Sulfosuccinic Acid', MS='All-purpose surfactant, wetting agent, and solubilizer used in the drug, cosmetics, and food industries. It has also been used in laxatives and as cerumenolytics. It is usually administered as either the calcium, potassium, or sodium salt.'),
  'D004144': NtO(MH='Diosgenin', MS='A spirostan found in DIOSCOREA and other plants. The 25S isomer is called yamogenin. Solasodine is a natural derivative formed by replacing the spiro-ring with a nitrogen, which can rearrange to SOLANINE.'),
  'D004145': NtO(MH='Diosmin', MS='A bioflavonoid that strengthens vascular walls.'),
  'D004146': NtO(MH='Dioxanes', MS='Compounds that contain the structure 1,4-dioxane.'),
  'D004147': NtO(MH='Dioxins', MS='A family of compounds that contain the 1,4-dioxin structure. Many specific dioxin derivatives are listed as CARCINOGENS; TERATOGENS; or MUTAGENS.'),
  'D004148': NtO(MH='Dioxolanes', MS=None),
  'D004149': NtO(MH='Dioxoles', MS=None),
  'D004150': NtO(MH='Dipeptidases', MS='EXOPEPTIDASES that specifically act on dipeptides. EC 3.4.13.'),
  'D004151': NtO(MH='Dipeptides', MS='Peptides composed of two amino acid units.'),
  'D004152': NtO(MH='Dipeptidyl-Peptidases and Tripeptidyl-Peptidases', MS='A subclass of exopeptidases that includes enzymes which cleave either two or three AMINO ACIDS from the end of a peptide chain.'),
  'D004153': NtO(MH='Dipetalonema', MS='A filarial nematode parasite of mammalian blood with the vector being a tick or small fly.'),
  'D004154': NtO(MH='Dipetalonema Infections', MS='Infections with nematodes of the genus DIPETALONEMA.'),
  'D004155': NtO(MH='Diphenhydramine', MS='A histamine H1 antagonist used as an antiemetic, antitussive, for dermatoses and pruritus, for hypersensitivity reactions, as a hypnotic, an antiparkinson, and as an ingredient in common cold preparations. It has some undesired antimuscarinic and sedative effects.'),
  'D004156': NtO(MH='Catechol Oxidase', MS='An enzyme of the oxidoreductase class that catalyzes the reaction between catechol and oxygen to yield benzoquinone and water. It is a complex of copper-containing proteins that acts also on a variety of substituted catechols. EC 1.10.3.1.'),
  'D004157': NtO(MH='Diphenoxylate', MS='A MEPERIDINE congener used as an antidiarrheal, usually in combination with ATROPINE. At high doses, it acts like morphine. Its unesterified metabolite difenoxin has similar properties and is used similarly. It has little or no analgesic activity.'),
  'D004158': NtO(MH='Diphenylacetic Acids', MS='Monocarboxylic acid derivatives of acetic acids, where the methyl hydrogens have been replaced by two phenyl groups.'),
  'D004159': NtO(MH='Diphenylamine', MS='In humans it may be irritating to mucous membranes. Methemoglobinemia has been produced experimentally. In veterinary use, it is one of active ingredients in topical agents for prevention and treatment of screwworm infestation. An indicator in tests for nitrate poisoning.'),
  'D004160': NtO(MH='Diphenylcarbazide', MS='Used as an indicator in titrating iron and for the colorimetric determination of chromium and the detection of cadmium, mercury, magnesium, aldehydes, and emetine.'),
  'D004161': NtO(MH='Diphenylhexatriene', MS='A fluorescent compound that emits light only in specific configurations in certain lipid media. It is used as a tool in the study of membrane lipids.'),
  'D004163': NtO(MH='Diphosphoglyceric Acids', MS='Glyceric acids where two of the hydroxyl groups have been replaced by phosphates.'),
  'D004164': NtO(MH='Diphosphonates', MS='Organic compounds which contain P-C-P bonds, where P stands for phosphonates or phosphonic acids. These compounds affect calcium metabolism. They inhibit ectopic calcification and slow down bone resorption and bone turnover. Technetium complexes of diphosphonates have been used successfully as bone scanning agents.'),
  'D004165': NtO(MH='Diphtheria', MS='A localized infection of mucous membranes or skin caused by toxigenic strains of CORYNEBACTERIUM DIPHTHERIAE. It is characterized by the presence of a pseudomembrane at the site of infection. DIPHTHERIA TOXIN, produced by C. diphtheriae, can cause myocarditis, polyneuritis, and other systemic toxic effects.'),
  'D004166': NtO(MH='Diphtheria Antitoxin', MS='An antitoxin produced against the toxin of CORYNEBACTERIUM DIPHTHERIAE that is used for the treatment of DIPHTHERIA.'),
  'D004167': NtO(MH='Diphtheria Toxin', MS='An ADP-ribosylating polypeptide produced by CORYNEBACTERIUM DIPHTHERIAE that causes the signs and symptoms of DIPHTHERIA. It can be broken into two unequal domains: the smaller, catalytic A domain is the lethal moiety and contains MONO(ADP-RIBOSE) TRANSFERASES which transfers ADP RIBOSE to PEPTIDE ELONGATION FACTOR 2 thereby inhibiting protein synthesis; and the larger B domain that is needed for entry into cells.'),
  'D004168': NtO(MH='Diphtheria Toxoid', MS='The formaldehyde-inactivated toxin of Corynebacterium diphtheriae. It is generally used in mixtures with TETANUS TOXOID and PERTUSSIS VACCINE; (DTP); or with tetanus toxoid alone (DT for pediatric use and Td, which contains 5- to 10-fold less diphtheria toxoid, for other use). Diphtheria toxoid is used for the prevention of diphtheria; DIPHTHERIA ANTITOXIN is for treatment.'),
  'D004169': NtO(MH='Diphyllobothriasis', MS='Infection with tapeworms of the genus Diphyllobothrium.'),
  'D004170': NtO(MH='Diphyllobothrium', MS='A genus of tapeworm containing several species which occurs in the intestine of fish, birds, and mammals including man. Infection in humans is usually by eating uncooked fish. The larval stage is known as SPARGANUM.'),
  'D004171': NtO(MH='Diploidy', MS='The chromosomal constitution of cells, in which each type of CHROMOSOME is represented twice. Symbol: 2N or 2X.'),
  'D004172': NtO(MH='Diplopia', MS='A visual symptom in which a single object is perceived by the visual cortex as two objects rather than one. Disorders associated with this condition include REFRACTIVE ERRORS; STRABISMUS; OCULOMOTOR NERVE DISEASES; TROCHLEAR NERVE DISEASES; ABDUCENS NERVE DISEASES; and diseases of the BRAIN STEM and OCCIPITAL LOBE.'),
  'D004173': NtO(MH='Dipodomys', MS='A genus of the family Heteromyidae which contains 22 species. Their physiology is adapted for the conservation of water, and they seldom drink water. They are found in arid or desert habitats and travel by hopping on their hind limbs.'),
  'D004174': NtO(MH='Diprenorphine', MS='A narcotic antagonist similar in action to NALOXONE. It is used to remobilize animals after ETORPHINE neuroleptanalgesia and is considered a specific antagonist to etorphine.'),
  'D004175': NtO(MH='Diptera', MS='An order of the class Insecta. Wings, when present, number two and distinguish Diptera from other so-called flies, while the halteres, or reduced hindwings, separate Diptera from other insects with one pair of wings. The order includes the families Calliphoridae, Oestridae, Phoridae, SARCOPHAGIDAE, Scatophagidae, Sciaridae, SIMULIIDAE, Tabanidae, Therevidae, Trypetidae, CERATOPOGONIDAE; CHIRONOMIDAE; CULICIDAE; DROSOPHILIDAE; GLOSSINIDAE; MUSCIDAE; TEPHRITIDAE; and PSYCHODIDAE. The larval form of Diptera species are called maggots (see LARVA).'),
  'D004176': NtO(MH='Dipyridamole', MS='A phosphodiesterase inhibitor that blocks uptake and metabolism of adenosine by erythrocytes and vascular endothelial cells. Dipyridamole also potentiates the antiaggregating action of prostacyclin. (From AMA Drug Evaluations Annual, 1994, p752)'),
  'D004177': NtO(MH='Dipyrone', MS='A drug that has analgesic, anti-inflammatory, and antipyretic properties. It is the sodium sulfonate of AMINOPYRINE.'),
  'D004178': NtO(MH='Diquat', MS='A contact herbicide used also to produce desiccation and defoliation. (From Merck Index, 11th ed)'),
  'D004179': NtO(MH='Cobra Cardiotoxin Proteins', MS='Most abundant proteins in COBRA venom; basic polypeptides of 57 to 62 amino acids with four disulfide bonds and a molecular weight of less than 7000; causes skeletal and cardiac muscle contraction, interferes with neuromuscular and ganglionic transmission, depolarizes nerve, muscle and blood cell membranes, thus causing hemolysis.'),
  'D004180': NtO(MH='Direct Service Costs', MS='Costs which are directly identifiable with a particular service.'),
  'D004181': NtO(MH='Directories as Topic', MS='Works about lists of persons or organizations, systematically arranged, usually in alphabetic or classed order, giving address, affiliations, etc., for individuals, and giving address, officers, functions, and similar data for organizations.'),
  'D004182': NtO(MH='Dirofilaria', MS='A genus of filarial nematodes. Various immature species have been found to infect the eyes or subcutaneous tissue in humans.'),
  'D004183': NtO(MH='Dirofilaria immitis', MS='A filarial parasite primarily of dogs but occurring also in foxes, wolves, and humans. The parasite is transmitted by mosquitoes.'),
  'D004184': NtO(MH='Dirofilariasis', MS='Infection with nematodes of the genus DIROFILARIA, usually in animals, especially dogs, but occasionally in man.'),
  'D004185': NtO(MH='Disability Evaluation', MS="Determination of the degree of a physical, mental, or emotional handicap. The diagnosis is applied to legal qualification for benefits and income under disability insurance and to eligibility for Social Security and workmen's compensation benefits."),
  'D004186': NtO(MH='Disaccharidases', MS='Enzymes belonging to the class of GLYCOSIDE HYDROLASES which break down DISACCHARIDES into simpler sugars, MONOSACCHARIDES.'),
  'D004187': NtO(MH='Disaccharides', MS='Oligosaccharides containing two monosaccharide units linked by a glycosidic bond.'),
  'D004188': NtO(MH='Disarticulation', MS='Amputation or separation at a joint. (Dorland, 28th ed)'),
  'D004189': NtO(MH='Disaster Planning', MS='Procedures outlined for the care of casualties and the maintenance of services in disasters.'),
  'D004190': NtO(MH='Disasters', MS='Calamities producing great damage, loss of life, and distress. They include results of natural phenomena and man-made phenomena. Normal conditions of existence are disrupted and the level of impact exceeds the capacity of the hazard-affected community.'),
  'D004191': NtO(MH='Behavioral Disciplines and Activities', MS='The specialties in psychiatry and psychology, their diagnostic techniques and tests, their therapeutic methods, and psychiatric and psychological services.'),
  'D004192': NtO(MH='Discrimination, Psychological', MS='Differential response to different stimuli.'),
  'D004193': NtO(MH='Discrimination Learning', MS='Learning that is manifested in the ability to respond differentially to various stimuli.'),
  'D004194': NtO(MH='Disease', MS='A definite pathologic process with a characteristic set of signs and symptoms. It may affect the whole body or any of its parts, and its etiology, pathology, and prognosis may be known or unknown.'),
  'D004195': NtO(MH='Disease Models, Animal', MS='Naturally-occurring or experimentally-induced animal diseases with pathological processes analogous to human diseases.'),
  'D004196': NtO(MH='Disease Outbreaks', MS='Sudden increase in the incidence of a disease. The concept includes EPIDEMICS and PANDEMICS.'),
  'D004197': NtO(MH='Disease Reservoirs', MS='Animate or inanimate sources which normally harbor disease-causing organisms and thus serve as potential sources of disease outbreaks. Reservoirs are distinguished from vectors (DISEASE VECTORS) and carriers, which are agents of disease transmission rather than continuing sources of potential disease outbreaks.'),
  'D004198': NtO(MH='Disease Susceptibility', MS='A constitution or condition of the body which makes the tissues react in special ways to certain extrinsic stimuli and thus tends to make the individual more than usually susceptible to certain diseases.'),
  'D004199': NtO(MH='Disease Vectors', MS='Invertebrates or non-human vertebrates which transmit infective organisms from one host to another.'),
  'D004200': NtO(MH='Diseases in Twins', MS='Disorders affecting TWINS, one or both, at any age.'),
  'D004202': NtO(MH='Disinfectants', MS="Substances used on inanimate objects that destroy harmful microorganisms or inhibit their activity. Disinfectants are classed as complete, destroying SPORES as well as vegetative forms of microorganisms, or incomplete, destroying only vegetative forms of the organisms. They are distinguished from ANTISEPTICS, which are local anti-infective agents used on humans and other animals. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D004203': NtO(MH='Disinfection', MS='Rendering pathogens harmless through the use of heat, antiseptics, antibacterial agents, etc.'),
  'D004204': NtO(MH='Joint Dislocations', MS='Displacement of bones from their normal positions at a joint.'),
  'D004205': NtO(MH='Cromolyn Sodium', MS='A chromone complex that acts by inhibiting the release of chemical mediators from sensitized mast cells. It is used in the prophylactic treatment of both allergic and exercise-induced asthma, but does not affect an established asthmatic attack.'),
  'D004206': NtO(MH='Disopyramide', MS='A class I anti-arrhythmic agent (one that interferes directly with the depolarization of the cardiac membrane and thus serves as a membrane-stabilizing agent) with a depressant action on the heart similar to that of guanidine. It also possesses some anticholinergic and local anesthetic properties.'),
  'D004207': NtO(MH='Dispensatories as Topic', MS='Works about systematic description of the drugs and preparations used generally or in a specific setting.'),
  'D004208': NtO(MH='Displacement, Psychological', MS='The process by which an emotional or behavioral response that is appropriate for one situation appears in another situation for which it is inappropriate.'),
  'D004209': NtO(MH='Disposable Equipment', MS='Apparatus, devices, or supplies intended for one-time or temporary use.'),
  'D004210': NtO(MH='Dissection', MS='The separation and isolation of tissues for surgical purposes, or for the analysis or study of their structures.'),
  'D004211': NtO(MH='Disseminated Intravascular Coagulation', MS='A disorder characterized by procoagulant substances entering the general circulation causing a systemic thrombotic process. The activation of the clotting mechanism may arise from any of a number of disorders. A majority of the patients manifest skin lesions, sometimes leading to PURPURA FULMINANS.'),
  'D004212': NtO(MH='Academic Dissertations as Topic', MS='Works about substantial papers written by candidates for an academic degree under the individual direction of a professor or papers written by undergraduates desirous of achieving honors or distinction.'),
  'D004213': NtO(MH='Dissociative Disorders', MS='Sudden temporary alterations in the normally integrative functions of consciousness.'),
  'D004214': NtO(MH='Distamycins', MS='Oligopeptide antibiotics from Streptomyces distallicus. Their binding to DNA inhibits synthesis of nucleic acids.'),
  'D004215': NtO(MH='Distance Perception', MS='The act of knowing or the recognition of a distance by recollective thought, or by means of a sensory process which is under the influence of a set of prior experiences.'),
  'D004216': NtO(MH='Distemper', MS='A name for several highly contagious viral diseases of animals, especially canine distemper. In dogs, it is caused by the canine distemper virus (DISTEMPER VIRUS, CANINE). It is characterized by a diphasic fever, leukopenia, gastrointestinal and respiratory inflammation and sometimes, neurologic complications. In cats it is known as FELINE PANLEUKOPENIA.'),
  'D004217': NtO(MH='Distemper Virus, Canine', MS='A species of MORBILLIVIRUS causing distemper in dogs, wolves, foxes, raccoons, and ferrets. Pinnipeds have also been known to contract Canine distemper virus from contact with domestic dogs.'),
  'D004219': NtO(MH='District of Columbia', MS='A federal area located between Maryland and Virginia on the Potomac river; it is coextensive with Washington, D.C., which is the capital of the United States.'),
  'D004220': NtO(MH='Disulfides', MS='Chemical groups containing the covalent disulfide bonds -S-S-. The sulfur atoms can be bound to inorganic or organic moieties.'),
  'D004221': NtO(MH='Disulfiram', MS='A carbamate derivative used as an alcohol deterrent. It is a relatively nontoxic substance when administered alone, but markedly alters the intermediary metabolism of alcohol. When alcohol is ingested after administration of disulfiram, blood acetaldehyde concentrations are increased, followed by flushing, systemic vasodilation, respiratory difficulties, nausea, hypotension, and other symptoms (acetaldehyde syndrome). It acts by inhibiting aldehyde dehydrogenase.'),
  'D004222': NtO(MH='Disulfoton', MS='An organothiophosphate insecticide.'),
  'D004224': NtO(MH='Diterpenes', MS='Twenty-carbon compounds derived from MEVALONIC ACID or deoxyxylulose phosphate.'),
  'D004225': NtO(MH='Dithiazanine', MS='3-Ethyl-2-(5-(3-ethyl-2-benzothiazolinylidene)-1,3- pentadienyl)benzothiazolium. A benzothiazole that was formerly used as an antinematodal agent and is currently used as a fluorescent dye.'),
  'D004226': NtO(MH='Dithioerythritol', MS="A compound that, along with its isomer, Cleland's reagent (DITHIOTHREITOL), is used for the protection of sulfhydryl groups against oxidation to disulfides and for the reduction of disulfides to sulfhydryl groups."),
  'D004227': NtO(MH='Dithionite', MS='Dithionite. The dithionous acid ion and its salts.'),
  'D004228': NtO(MH='Dithionitrobenzoic Acid', MS='A standard reagent for the determination of reactive sulfhydryl groups by absorbance measurements. It is used primarily for the determination of sulfhydryl and disulfide groups in proteins. The color produced is due to the formation of a thio anion, 3-carboxyl-4-nitrothiophenolate.'),
  'D004229': NtO(MH='Dithiothreitol', MS='A reagent commonly used in biochemical studies as a protective agent to prevent the oxidation of SH (thiol) groups and for reducing disulphides to dithiols.'),
  'D004230': NtO(MH='Dithizone', MS='Chelating agent used for heavy metal poisoning and assay. It causes diabetes.'),
  'D004231': NtO(MH='Diuresis', MS='An increase in the excretion of URINE. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D004232': NtO(MH='Diuretics', MS='Agents that promote the excretion of urine through their effects on kidney function.'),
  'D004234': NtO(MH='Diuretics, Osmotic', MS='Compounds that increase urine volume by increasing the amount of osmotically active solute in the urine. Osmotic diuretics also increase the osmolarity of plasma.'),
  'D004237': NtO(MH='Diuron', MS='A pre-emergent herbicide.'),
  'D004238': NtO(MH='Diverticulitis', MS='Inflammation of a DIVERTICULUM or diverticula.'),
  'D004239': NtO(MH='Diverticulitis, Colonic', MS='Inflammation of the COLONIC DIVERTICULA, generally with abscess formation and subsequent perforation.'),
  'D004240': NtO(MH='Diverticulum', MS='A pouch or sac developed from a tubular or saccular organ, such as the GASTROINTESTINAL TRACT.'),
  'D004241': NtO(MH='Diverticulum, Colon', MS='A pouch or sac opening from the COLON.'),
  'D004242': NtO(MH='Diving', MS='An activity in which the organism plunges into water. It includes scuba and bell diving. Diving as natural behavior of animals goes here, as well as diving in decompression experiments with humans or animals.'),
  'D004243': NtO(MH='Divorce', MS='Legal dissolution of an officially recognized marriage relationship.'),
  'D004244': NtO(MH='Dizziness', MS='An imprecise term which may refer to a sense of spatial disorientation, motion of the environment, or lightheadedness.'),
  'D004245': NtO(MH='DMF Index', MS='"Decayed, missing and filled teeth," a routinely used statistical concept in dentistry.'),
  'D004246': NtO(MH='Dimethylphenylpiperazinium Iodide', MS='A selective nicotinic cholinergic agonist used as a research tool. DMPP activates nicotinic receptors in autonomic ganglia but has little effect at the neuromuscular junction.'),
  'D004247': NtO(MH='DNA', MS='A deoxyribonucleotide polymer that is the primary genetic material of all cells. Eukaryotic and prokaryotic organisms normally contain DNA in a double-stranded state, yet several important biological processes transiently involve single-stranded regions. DNA, which consists of a polysugar-phosphate backbone possessing projections of purines (adenine and guanine) and pyrimidines (thymine and cytosine), forms a double helix that is held together by hydrogen bonds between these purines and pyrimidines (adenine to thymine and guanine to cytosine).'),
  'D004248': NtO(MH='DNA (Cytosine-5-)-Methyltransferases', MS='Enzymes that catalyzes the transfer of a methyl group from S-ADENOSYLMETHIONINE to the 5-position of CYTOSINE residues in DNA.'),
  'D004249': NtO(MH='DNA Damage', MS='Injuries to DNA that introduce deviations from its normal, intact structure and which may, if left unrepaired, result in a MUTATION or a block of DNA REPLICATION. These deviations may be caused by physical or chemical agents and occur by natural or unnatural, introduced circumstances. They include the introduction of illegitimate bases during replication or by deamination or other modification of bases; the loss of a base from the DNA backbone leaving an abasic site; single-strand breaks; double strand breaks; and intrastrand (PYRIMIDINE DIMERS) or interstrand crosslinking. Damage can often be repaired (DNA REPAIR). If the damage is extensive, it can induce APOPTOSIS.'),
  'D004250': NtO(MH='DNA Topoisomerases, Type II', MS='DNA TOPOISOMERASES that catalyze ATP-dependent breakage of both strands of DNA, passage of the unbroken strands through the breaks, and rejoining of the broken strands. These enzymes bring about relaxation of the supercoiled DNA and resolution of a knotted circular DNA duplex.'),
  'D004251': NtO(MH='DNA Transposable Elements', MS='Discrete segments of DNA which can excise and reintegrate to another site in the genome. Most are inactive, i.e., have not been found to exist outside the integrated state. DNA transposable elements include bacterial IS (insertion sequence) elements, Tn elements, the maize controlling elements Ac and Ds, Drosophila P, gypsy, and pogo elements, the human Tigger elements and the Tc and mariner elements which are found throughout the animal kingdom.'),
  'D004252': NtO(MH='DNA Mutational Analysis', MS='Biochemical identification of mutational changes in a nucleotide sequence.'),
  'D004253': NtO(MH='DNA Nucleotidylexotransferase', MS='A non-template-directed DNA polymerase normally found in vertebrate thymus and bone marrow. It catalyzes the elongation of oligo- or polydeoxynucleotide chains and is widely used as a tool in the differential diagnosis of acute leukemias in man. EC 2.7.7.31.'),
  'D004254': NtO(MH='DNA Nucleotidyltransferases', MS='Enzymes that catalyze the incorporation of deoxyribonucleotides into a chain of DNA. EC 2.7.7.-.'),
  'D004255': NtO(MH='Deoxyribodipyrimidine Photo-Lyase', MS='An enzyme that catalyzes the reactivation by light of UV-irradiated DNA. It breaks two carbon-carbon bonds in PYRIMIDINE DIMERS in DNA.'),
  'D004256': NtO(MH='DNA Polymerase I', MS="A DNA-dependent DNA polymerase characterized in prokaryotes and may be present in higher organisms. It has both 3'-5' and 5'-3' exonuclease activity, but cannot use native double-stranded DNA as template-primer. It is not inhibited by sulfhydryl reagents and is active in both DNA synthesis and repair. EC 2.7.7.7."),
  'D004257': NtO(MH='DNA Polymerase II', MS="A DNA-dependent DNA polymerase characterized in E. coli and other lower organisms. It may be present in higher organisms and has an intrinsic molecular activity only 5% of that of DNA Polymerase I. This polymerase has 3'-5' exonuclease activity, is effective only on duplex DNA with gaps or single-strand ends of less than 100 nucleotides as template, and is inhibited by sulfhydryl reagents. EC 2.7.7.7."),
  'D004258': NtO(MH='DNA Polymerase III', MS="A DNA-dependent DNA polymerase characterized in E. coli and other lower organisms but may be present in higher organisms. Use also for a more complex form of DNA polymerase III designated as DNA polymerase III* or pol III* which is 15 times more active biologically than DNA polymerase I in the synthesis of DNA. This polymerase has both 3'-5' and 5'-3' exonuclease activities, is inhibited by sulfhydryl reagents, and has the same template-primer dependence as pol II. EC 2.7.7.7."),
  'D004259': NtO(MH='DNA-Directed DNA Polymerase', MS='DNA-dependent DNA polymerases found in bacteria, animal and plant cells. During the replication process, these enzymes catalyze the addition of deoxyribonucleotide residues to the end of a DNA strand in the presence of DNA as template-primer. They also possess exonuclease activity and therefore function in DNA repair.'),
  'D004260': NtO(MH='DNA Repair', MS='The reconstruction of a continuous two-stranded DNA molecule without mismatch from a molecule which contained damaged regions. The major repair mechanisms are excision repair, in which defective regions in one strand are excised and resynthesized using the complementary base pairing information in the intact strand; photoreactivation repair, in which the lethal and mutagenic effects of ultraviolet light are eliminated; and post-replication repair, in which the primary lesions are not repaired, but the gaps in one daughter duplex are filled in by incorporation of portions of the other (undamaged) daughter duplex. Excision repair and post-replication repair are sometimes referred to as "dark repair" because they do not require light.'),
  'D004261': NtO(MH='DNA Replication', MS='The process by which a DNA molecule is duplicated.'),
  'D004262': NtO(MH='DNA Restriction Enzymes', MS="Enzymes that are part of the restriction-modification systems. They catalyze the endonucleolytic cleavage of DNA sequences which lack the species-specific methylation pattern in the host cell's DNA. Cleavage yields random or specific double-stranded fragments with terminal 5'-phosphates. The function of restriction enzymes is to destroy any foreign DNA that invades the host cell. Most have been studied in bacterial systems, but a few have been found in eukaryotic organisms. They are also used as tools for the systematic dissection and mapping of chromosomes, in the determination of base sequences of DNAs, and have made it possible to splice and recombine genes from one organism into the genome of another. EC 3.21.1."),
  'D004263': NtO(MH='DNA Tumor Viruses', MS='DNA viruses producing malignant tumors. Of the six major groupings of DNA viruses four contain members which are actually or potentially oncogenic: the Adenoviridae, the Herpesviridae, the Papovaviridae, and the Poxviridae.'),
  'D004264': NtO(MH='DNA Topoisomerases, Type I', MS='DNA TOPOISOMERASES that catalyze ATP-independent breakage of one of the two strands of DNA, passage of the unbroken strand through the break, and rejoining of the broken strand. DNA Topoisomerases, Type I enzymes reduce the topological stress in the DNA structure by relaxing the superhelical turns and knotted rings in the DNA helix.'),
  'D004265': NtO(MH='DNA Helicases', MS='Proteins that catalyze the unwinding of duplex DNA during replication by binding cooperatively to single-stranded regions of DNA or to short regions of duplex DNA that are undergoing transient opening. In addition DNA helicases are DNA-dependent ATPases that harness the free energy of ATP hydrolysis to translocate DNA strands.'),
  'D004266': NtO(MH='DNA Virus Infections', MS='Diseases caused by DNA VIRUSES.'),
  'D004267': NtO(MH='DNA Viruses', MS='Viruses whose nucleic acid is DNA.'),
  'D004268': NtO(MH='DNA-Binding Proteins', MS='Proteins which bind to DNA. The family includes proteins which bind to both double- and single-stranded DNA and also includes specific DNA binding proteins in serum which can be used as markers for malignant diseases.'),
  'D004269': NtO(MH='DNA, Bacterial', MS='Deoxyribonucleic acid that makes up the genetic material of bacteria.'),
  'D004270': NtO(MH='DNA, Circular', MS="Any of the covalently closed DNA molecules found in bacteria, many viruses, mitochondria, plastids, and plasmids. Small, polydisperse circular DNA's have also been observed in a number of eukaryotic organisms and are suggested to have homology with chromosomal DNA and the capacity to be inserted into, and excised from, chromosomal DNA. It is a fragment of DNA formed by a process of looping out and deletion, containing a constant region of the mu heavy chain and the 3'-part of the mu switch region. Circular DNA is a normal product of rearrangement among gene segments encoding the variable regions of immunoglobulin light and heavy chains, as well as the T-cell receptor. (Riger et al., Glossary of Genetics, 5th ed & Segen, Dictionary of Modern Medicine, 1992)"),
  'D004271': NtO(MH='DNA, Fungal', MS='Deoxyribonucleic acid that makes up the genetic material of fungi.'),
  'D004272': NtO(MH='DNA, Mitochondrial', MS='Double-stranded DNA of MITOCHONDRIA. In eukaryotes, the mitochondrial GENOME is circular and codes for ribosomal RNAs, transfer RNAs, and about 10 proteins.'),
  'D004273': NtO(MH='DNA, Neoplasm', MS='DNA present in neoplastic tissue.'),
  'D004274': NtO(MH='DNA, Recombinant', MS='Biologically active DNA which has been formed by the in vitro joining of segments of DNA from different sources. It includes the recombination joint or edge of a heteroduplex region where two recombining DNA molecules are connected.'),
  'D004275': NtO(MH='DNA, Ribosomal', MS='DNA sequences encoding RIBOSOMAL RNA and the segments of DNA separating the individual ribosomal RNA genes, referred to as RIBOSOMAL SPACER DNA.'),
  'D004276': NtO(MH='DNA, Satellite', MS='Highly repetitive DNA sequences found in HETEROCHROMATIN, mainly near centromeres. They are composed of simple sequences (very short) (see MINISATELLITE REPEATS) repeated in tandem many times to form large blocks of sequence. Additionally, following the accumulation of mutations, these blocks of repeats have been repeated in tandem themselves. The degree of repetition is on the order of 1000 to 10 million at each locus. Loci are few, usually one or two per chromosome. They were called satellites since in density gradients, they often sediment as distinct, satellite bands separate from the bulk of genomic DNA owing to a distinct BASE COMPOSITION.'),
  'D004277': NtO(MH='DNA, Single-Stranded', MS='A single chain of deoxyribonucleotides that occurs in some bacteria and viruses. It usually exists as a covalently closed circle.'),
  'D004278': NtO(MH='DNA, Superhelical', MS='Circular duplex DNA isolated from viruses, bacteria and mitochondria in supercoiled or supertwisted form. This superhelical DNA is endowed with free energy. During transcription, the magnitude of RNA initiation is proportional to the DNA superhelicity.'),
  'D004279': NtO(MH='DNA, Viral', MS='Deoxyribonucleic acid that makes up the genetic material of viruses.'),
  'D004280': NtO(MH='Dobutamine', MS='A catecholamine derivative with specificity for BETA-1 ADRENERGIC RECEPTORS. It is commonly used as a cardiotonic agent after CARDIAC SURGERY and during DOBUTAMINE STRESS ECHOCARDIOGRAPHY.'),
  'D004281': NtO(MH='Docosahexaenoic Acids', MS='C22-unsaturated fatty acids found predominantly in FISH OILS.'),
  'D004282': NtO(MH='Documentation', MS='Systematic organization, storage, retrieval, and dissemination of specialized information, especially of a scientific or technical nature (From ALA Glossary of Library and Information Science, 1983). It often involves authenticating or validating information.'),
  'D004283': NtO(MH='Dog Diseases', MS='Diseases of the domestic dog (Canis familiaris). This term does not include diseases of wild dogs, WOLVES; FOXES; and other Canidae for which the heading CARNIVORA is used.'),
  'D004284': NtO(MH='Dogfish', MS='Sharks of the family Squalidae, also called dogfish sharks. They comprise at least eight genera and 44 species. Their LIVER is valued for its oil and its flesh is often made into fertilizer.'),
  'D004285': NtO(MH='Dogs', MS="The domestic dog, Canis familiaris, comprising about 400 breeds, of the carnivore family CANIDAE. They are worldwide in distribution and live in association with people. (Walker's Mammals of the World, 5th ed, p1065)"),
  'D004286': NtO(MH='Dolichols', MS='A class of polyprenols which contain approximately 20 isoprene residues. Although considered ISOPRENOIDS, they terminate with an alpha-saturated isoprenoid group at the hydroxy end of the molecule.'),
  'D004287': NtO(MH='Dolichol Monophosphate Mannose', MS='A lipophilic glycosyl carrier of the monosaccharide mannose in the biosynthesis of oligosaccharide phospholipids and glycoproteins.'),
  'D004288': NtO(MH='Dolichol Phosphates', MS='Phosphoric acid esters of dolichol.'),
  'D004289': NtO(MH='Dolphins', MS="Mammals of the families Delphinidae (ocean dolphins), Iniidae, Lipotidae, Pontoporiidae, and Platanistidae (all river dolphins). Among the most well-known species are the BOTTLE-NOSED DOLPHIN and the KILLER WHALE (a dolphin). The common name dolphin is applied to small cetaceans having a beaklike snout and a slender, streamlined body, whereas PORPOISES are small cetaceans with a blunt snout and rather stocky body. (From Walker's Mammals of the World, 5th ed, pp978-9)"),
  'D004290': NtO(MH='2,5-Dimethoxy-4-Methylamphetamine', MS='A psychedelic phenyl isopropylamine derivative, commonly called DOM, whose mood-altering effects and mechanism of action may be similar to those of LSD.'),
  'D004291': NtO(MH='Dominance-Subordination', MS='Relationship between individuals when one individual threatens or becomes aggressive and the other individual remains passive or attempts to escape.'),
  'D004292': NtO(MH='Dominance, Cerebral', MS='Dominance of one cerebral hemisphere over the other in cerebral functions.'),
  'D004293': NtO(MH='Dominican Republic', MS="A republic in the Greater Antilles in the West Indies. Its capital is Santo Domingo. With Haiti, it forms the island of Hispaniola - the Dominican Republic occupying the eastern two thirds, and Haiti, the western third. It was created in 1844 after a revolt against the rule of President Boyer over the entire island of Hispaniola, itself visited by Columbus in 1492 and settled the next year. Except for a brief period of annexation to Spain (1861-65), it has been independent, though closely associated with the United States. Its name comes from the Spanish Santo Domingo, Holy Sunday, with reference to its discovery on a Sunday. (From Webster's New Geographical Dictionary, 1988, p338, 506 & Room, Brewer's Dictionary of Names, 1992, p151)"),
  'D004294': NtO(MH='Domperidone', MS='A specific blocker of dopamine receptors. It speeds gastrointestinal peristalsis, causes prolactin release, and is used as antiemetic and tool in the study of dopaminergic mechanisms.'),
  'D004295': NtO(MH='Dihydroxyphenylalanine', MS='A beta-hydroxylated derivative of phenylalanine. The D-form of dihydroxyphenylalanine has less physiologic activity than the L-form and is commonly used experimentally to determine whether the pharmacological effects of LEVODOPA are stereospecific.'),
  'D004296': NtO(MH='Dopa Decarboxylase', MS="One of the AROMATIC-L-AMINO-ACID DECARBOXYLASES, this enzyme is responsible for the conversion of DOPA to DOPAMINE. It is of clinical importance in the treatment of Parkinson's disease."),
  'D004298': NtO(MH='Dopamine', MS='One of the catecholamine NEUROTRANSMITTERS in the brain. It is derived from TYROSINE and is the precursor to NOREPINEPHRINE and EPINEPHRINE. Dopamine is a major transmitter in the extrapyramidal system of the brain, and important in regulating movement. A family of receptors (RECEPTORS, DOPAMINE) mediate its action.'),
  'D004299': NtO(MH='Dopamine beta-Hydroxylase', MS=None),
  'D004300': NtO(MH='Doping in Sports', MS='Illegitimate use of substances for a desired effect in competitive sports. It includes humans and animals.'),
  'D004301': NtO(MH='Doppler Effect', MS='Changes in the observed frequency of waves (as sound, light, or radio waves) due to the relative motion of source and observer. The effect was named for the 19th century Austrian physicist Johann Christian Doppler.'),
  'D004302': NtO(MH='Dorsomedial Hypothalamic Nucleus', MS='An aggregation of cells in the middle hypothalamus dorsal to the ventromedial nucleus and bordering the THIRD VENTRICLE.'),
  'D004303': NtO(MH='Dosage Compensation, Genetic', MS='Genetic mechanisms that allow GENES to be expressed at a similar level irrespective of their GENE DOSAGE. This term is usually used in discussing genes that lie on the SEX CHROMOSOMES. Because the sex chromosomes are only partially homologous, there is a different copy number, i.e., dosage, of these genes in males vs. females. In DROSOPHILA, dosage compensation is accomplished by hypertranscription of genes located on the X CHROMOSOME. In mammals, dosage compensation of X chromosome genes is accomplished by random X CHROMOSOME INACTIVATION of one of the two X chromosomes in the female.'),
  'D004304': NtO(MH='Dosage Forms', MS='Completed forms of the pharmaceutical preparation in which prescribed doses of medication are included. They are designed to resist action by gastric fluids, prevent vomiting and nausea, reduce or alleviate the undesirable taste and smells associated with oral administration, achieve a high concentration of drug at target site, or produce a delayed or long-acting drug effect.'),
  'D004305': NtO(MH='Dose-Response Relationship, Drug', MS='The relationship between the dose of an administered drug and the response of the organism to the drug.'),
  'D004306': NtO(MH='Dose-Response Relationship, Immunologic', MS='A specific immune response elicited by a specific dose of an immunologically active substance or cell in an organism, tissue, or cell.'),
  'D004307': NtO(MH='Dose-Response Relationship, Radiation', MS='The relationship between the dose of administered radiation and the response of the organism or tissue to the radiation.'),
  'D004308': NtO(MH='Dothiepin', MS='A tricyclic antidepressant with some tranquilizing action.'),
  'D004309': NtO(MH='Double Bind Interaction', MS='Simultaneous communication of conflicting messages in which the response to either message evokes rejection or disapproval. (APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D004310': NtO(MH='Double Outlet Right Ventricle', MS='Incomplete transposition of the great vessels in which both the AORTA and the PULMONARY ARTERY arise from the RIGHT VENTRICLE. The only outlet of the LEFT VENTRICLE is a large ventricular septal defect (VENTRICULAR SEPTAL DEFECTS or VSD). The various subtypes are classified by the location of the septal defect, such as subaortic, subpulmonary, or noncommitted.'),
  'D004311': NtO(MH='Double-Blind Method', MS='A method of studying a drug or procedure in which both the subjects and investigators are kept unaware of who is actually getting which specific treatment.'),
  'D004312': NtO(MH="Douglas' Pouch", MS='A sac or recess formed by a fold of the peritoneum.'),
  'D004313': NtO(MH='Dourine', MS='A disease of horses and donkeys caused by Trypanosoma equiperdum. The disease occurs in Africa, the Americas, and Asia.'),
  'D004314': NtO(MH='Down Syndrome', MS='A chromosome disorder associated either with an extra chromosome 21 or an effective trisomy for chromosome 21. Clinical manifestations include hypotonia, short stature, brachycephaly, upslanting palpebral fissures, epicanthus, Brushfield spots on the iris, protruding tongue, small ears, short, broad hands, fifth finger clinodactyly, Simian crease, and moderate to severe INTELLECTUAL DISABILITY. Cardiac and gastrointestinal malformations, a marked increase in the incidence of LEUKEMIA, and the early onset of ALZHEIMER DISEASE are also associated with this condition. Pathologic features include the development of NEUROFIBRILLARY TANGLES in neurons and the deposition of AMYLOID BETA-PROTEIN, similar to the pathology of ALZHEIMER DISEASE. (Menkes, Textbook of Child Neurology, 5th ed, p213)'),
  'D004315': NtO(MH='Doxapram', MS='A central respiratory stimulant with a brief duration of action. (From Martindale, The Extra Pharmocopoeia, 30th ed, p1225)'),
  'D004316': NtO(MH='Doxepin', MS='A dibenzoxepin tricyclic compound. It displays a range of pharmacological actions including maintaining adrenergic innervation. Its mechanism of action is not fully understood, but it appears to block reuptake of monoaminergic neurotransmitters into presynaptic terminals. It also possesses anticholinergic activity and modulates antagonism of histamine H(1)- and H(2)-receptors.'),
  'D004317': NtO(MH='Doxorubicin', MS='Antineoplastic antibiotic obtained from Streptomyces peucetius. It is a hydroxy derivative of DAUNORUBICIN.'),
  'D004318': NtO(MH='Doxycycline', MS='A synthetic tetracycline derivative with similar antimicrobial activity.'),
  'D004319': NtO(MH='Doxylamine', MS='Histamine H1 antagonist with pronounced sedative properties. It is used in allergies and as an antitussive, antiemetic, and hypnotic. Doxylamine has also been administered in veterinary applications and was formerly used in PARKINSONISM.'),
  'D004320': NtO(MH='Dracunculiasis', MS='Infection with nematodes of the genus Dracunculus. One or more worms may be seen at a time, with the legs and feet being the most commonly infected areas. Symptoms include pruritus, nausea, vomiting, diarrhea, or asthmatic attacks.'),
  'D004321': NtO(MH='Dracunculus Nematode', MS='A genus of nematode parasites which inhabit the body cavity, serous membranes, and connective tissues of vertebrates. The parasitic species in humans is Dracunculus medinensis.'),
  'D004322': NtO(MH='Drainage', MS='The removal of fluids or discharges from the body, such as from a wound, sore, or cavity.'),
  'D004323': NtO(MH='Drainage, Sanitary', MS='A system of artificial or natural drains, generally used for the disposal of liquid wastes.'),
  'D004324': NtO(MH='Drama', MS='A composition in prose or verse presenting in dialogue or pantomime a story involving various characters, usually intended to be acted on a stage and to be regarded as a form of entertainment. (From Random House Unabridged Dictionary, 2d ed)'),
  'D004325': NtO(MH='Dreams', MS='A series of thoughts, images, or emotions occurring during sleep which are dissociated from the usual stream of consciousness of the waking state.'),
  'D004326': NtO(MH='Drinking', MS='The consumption of liquids.'),
  'D004327': NtO(MH='Drinking Behavior', MS='Behaviors associated with the ingesting of water and other liquids; includes rhythmic patterns of drinking (time intervals - onset and duration), frequency and satiety.'),
  'D004328': NtO(MH='Drive', MS='A state of internal activity of an organism that is a necessary condition before a given stimulus will elicit a class of responses; e.g., a certain level of hunger (drive) must be present before food will elicit an eating response.'),
  'D004329': NtO(MH='Droperidol', MS='A butyrophenone with general properties similar to those of HALOPERIDOL. It is used in conjunction with an opioid analgesic such as FENTANYL to maintain the patient in a calm state of neuroleptanalgesia with indifference to surroundings but still able to cooperate with the surgeon. It is also used as a premedicant, as an antiemetic, and for the control of agitation in acute psychoses. (From Martindale, The Extra Pharmacopoeia, 29th ed, p593)'),
  'D004330': NtO(MH='Drosophila', MS='A genus of small, two-winged flies containing approximately 900 described species. These organisms are the most extensively studied of all genera from the standpoint of genetics and cytology.'),
  'D004331': NtO(MH='Drosophila melanogaster', MS='A species of fruit fly frequently used in genetics because of the large size of its chromosomes.'),
  'D004332': NtO(MH='Drowning', MS='Death that occurs as a result of anoxia or heart arrest, associated with immersion in liquid.'),
  'D004333': NtO(MH='Drug Administration Routes', MS='The various ways of administering a drug or other chemical to a site in a patient or animal from where the chemical is absorbed into the blood and delivered to the target tissue.'),
  'D004334': NtO(MH='Drug Administration Schedule', MS='Time schedule for administration of a drug in order to achieve optimum effectiveness and convenience.'),
  'D004335': NtO(MH='Drug and Narcotic Control', MS='Control of drug and narcotic use by international agreement, or by institutional systems for handling prescribed drugs. This includes regulations concerned with the manufacturing, dispensing, approval (DRUG APPROVAL), and marketing of drugs.'),
  'D004336': NtO(MH='Drug Antagonism', MS='Phenomena and pharmaceutics of compounds that inhibit the function of agonists (DRUG AGONISM) and inverse agonists (DRUG INVERSE AGONISM) for a specific receptor. On their own, antagonists produce no effect by themselves to a receptor, and are said to have neither intrinsic activity nor efficacy.'),
  'D004337': NtO(MH='Drug Carriers', MS='Forms to which substances are incorporated to improve the delivery and the effectiveness of drugs. Drug carriers are used in drug-delivery systems such as the controlled-release technology to prolong in vivo drug actions, decrease drug metabolism, and reduce drug toxicity. Carriers are also used in designs to increase the effectiveness of drug delivery to the target sites of pharmacological actions. Liposomes, albumin microspheres, soluble synthetic polymers, DNA complexes, protein-drug conjugates, and carrier erythrocytes among others have been employed as biodegradable drug carriers.'),
  'D004338': NtO(MH='Drug Combinations', MS='Single preparations containing two or more active agents, for the purpose of their concurrent administration as a fixed dose mixture.'),
  'D004339': NtO(MH='Drug Compounding', MS='The preparation, mixing, and assembly of a drug. (From Remington, The Science and Practice of Pharmacy, 19th ed, p1814).'),
  'D004340': NtO(MH='Drug Contamination', MS='The presence of organisms, or any foreign material that makes a drug preparation impure.'),
  'D004341': NtO(MH='Drug Evaluation', MS='Any process by which toxicity, metabolism, absorption, elimination, preferred route of administration, safe dosage range, etc., for a drug or group of drugs is determined through clinical assessment in humans or veterinary animals.'),
  'D004342': NtO(MH='Drug Hypersensitivity', MS='Immunologically mediated adverse reactions to medicinal substances used legally or illegally.'),
  'D004343': NtO(MH='Drug Implants', MS='Small containers or pellets of a solid drug implanted in the body to achieve sustained release of the drug.'),
  'D004344': NtO(MH='Drug Incompatibility', MS='The quality of not being miscible with another given substance without a chemical change. One drug is not of suitable composition to be combined or mixed with another agent or substance. The incompatibility usually results in an undesirable reaction, including chemical alteration or destruction. (Dorland, 27th ed; Stedman, 25th ed)'),
  'D004345': NtO(MH='Drug Industry', MS='That segment of commercial enterprise devoted to the design, development, and manufacture of chemical products for use in the diagnosis and treatment of disease, disability, or other dysfunction, or to improve function.'),
  'D004346': NtO(MH='Drug Information Services', MS='Services providing pharmaceutic and therapeutic drug information and consultation.'),
  'D004347': NtO(MH='Drug Interactions', MS='The action of a drug that may affect the activity, metabolism, or toxicity of another drug.'),
  'D004348': NtO(MH='Drug Labeling', MS='Use of written, printed, or graphic materials upon or accompanying a drug container or wrapper. It includes contents, indications, effects, dosages, routes, methods, frequency and duration of administration, warnings, hazards, contraindications, side effects, precautions, and other relevant information.'),
  'D004349': NtO(MH='Drug Packaging', MS='Containers, packaging, and packaging materials for drugs and BIOLOGICAL PRODUCTS. These include those in ampule, capsule, tablet, solution or other forms. Packaging includes immediate-containers, secondary-containers, and cartons. In the United States, such packaging is controlled under the Federal Food, Drug, and Cosmetic Act which also stipulates requirements for tamper-resistance and child-resistance. Similar laws govern use elsewhere. (From Code of Federal Regulations, 21 CFR 1 Section 210, 1993) DRUG LABELING is also available.'),
  'D004350': NtO(MH='Drug Residues', MS='Drugs and their metabolites which are found in the edible tissues and milk of animals after their medication with specific drugs. This term can also apply to drugs found in adipose tissue of humans after drug treatment.'),
  'D004351': NtO(MH='Drug Resistance', MS='Diminished or failed response of an organism, disease or tissue to the intended effectiveness of a chemical or drug. It should be differentiated from DRUG TOLERANCE which is the progressive diminution of the susceptibility of a human or animal to the effects of a drug, as a result of continued administration.'),
  'D004352': NtO(MH='Drug Resistance, Microbial', MS='The ability of microorganisms, especially bacteria, to resist or to become tolerant to chemotherapeutic agents, antimicrobial agents, or antibiotics. This resistance may be acquired through gene mutation or foreign DNA in transmissible plasmids (R FACTORS).'),
  'D004353': NtO(MH='Drug Evaluation, Preclinical', MS='Preclinical testing of drugs in experimental animals or in vitro for their biological and toxic effects and potential clinical applications.'),
  'D004354': NtO(MH='Drug Screening Assays, Antitumor', MS='Methods of investigating the effectiveness of anticancer cytotoxic drugs and biologic inhibitors. These include in vitro cell-kill models and cytostatic dye exclusion tests as well as in vivo measurement of tumor growth parameters in laboratory animals.'),
  'D004355': NtO(MH='Drug Stability', MS='The chemical and physical integrity of a pharmaceutical product.'),
  'D004356': NtO(MH='Drug Storage', MS='The process of keeping pharmaceutical products in an appropriate location.'),
  'D004357': NtO(MH='Drug Synergism', MS='The action of a drug in promoting or enhancing the effectiveness of another drug.'),
  'D004358': NtO(MH='Drug Therapy', MS='The use of DRUGS to treat a DISEASE or its symptoms. One example is the use of ANTINEOPLASTIC AGENTS to treat CANCER.'),
  'D004359': NtO(MH='Drug Therapy, Combination', MS='Therapy with two or more separate preparations given for a combined effect.'),
  'D004360': NtO(MH='Drug Therapy, Computer-Assisted', MS='Adjunctive computer programs in providing drug treatment to patients.'),
  'D004361': NtO(MH='Drug Tolerance', MS='Progressive diminution of the susceptibility of a human or animal to the effects of a drug, resulting from its continued administration. It should be differentiated from DRUG RESISTANCE wherein an organism, disease, or tissue fails to respond to the intended effectiveness of a chemical or drug. It should also be differentiated from MAXIMUM TOLERATED DOSE and NO-OBSERVED-ADVERSE-EFFECT LEVEL.'),
  'D004363': NtO(MH='Drug Utilization', MS='The utilization of drugs as reported in individual hospital studies, FDA studies, marketing, or consumption, etc. This includes drug stockpiling, and patient drug profiles.'),
  'D004364': NtO(MH='Pharmaceutical Preparations', MS='Drugs intended for human or veterinary use, presented in their finished dosage form. Included here are materials used in the preparation and/or formulation of the finished dosage form.'),
  'D004365': NtO(MH='Drugs, Chinese Herbal', MS='Chinese herbal or plant extracts which are used as drugs to treat diseases or promote general well-being. The concept does not include synthesized compounds manufactured in China.'),
  'D004366': NtO(MH='Nonprescription Drugs', MS='Medicines that can be sold legally without a DRUG PRESCRIPTION.'),
  'D004367': NtO(MH='Dry Ice', MS='A solid form of carbon dioxide used as a refrigerant.'),
  'D004368': NtO(MH='Dry Socket', MS='A condition sometimes occurring after tooth extraction, particularly after traumatic extraction, resulting in a dry appearance of the exposed bone in the socket, due to disintegration or loss of the blood clot. It is basically a focal osteomyelitis without suppuration and is accompanied by severe pain (alveolalgia) and foul odor. (Dorland, 28th ed)'),
  'D004369': NtO(MH='Pentetic Acid', MS='An iron chelating agent with properties like EDETIC ACID. DTPA has also been used as a chelator for other metals, such as plutonium.'),
  'D004370': NtO(MH='Duane Retraction Syndrome', MS='A syndrome characterized by marked limitation of abduction of the eye, variable limitation of adduction and retraction of the globe, and narrowing of the palpebral fissure on attempted adduction. The condition is caused by aberrant innervation of the lateral rectus by fibers of the OCULOMOTOR NERVE.'),
  'D004371': NtO(MH='Hepatitis Virus, Duck', MS='Unassigned species, in the family PICORNAVIRIDAE, causing high mortality in ducklings 3 days to 3 weeks old.'),
  'D004372': NtO(MH='Ducks', MS='A water bird in the order Anseriformes (subfamily Anatinae (true ducks)) with a broad blunt bill, short legs, webbed feet, and a waddling gait.'),
  'D004373': NtO(MH='Ductus Arteriosus', MS='A fetal blood vessel connecting the pulmonary artery with the descending aorta.'),
  'D004374': NtO(MH='Ductus Arteriosus, Patent', MS='A congenital heart defect characterized by the persistent opening of fetal DUCTUS ARTERIOSUS that connects the PULMONARY ARTERY to the descending aorta (AORTA, DESCENDING) allowing unoxygenated blood to bypass the lung and flow to the PLACENTA. Normally, the ductus is closed shortly after birth.'),
  'D004375': NtO(MH='Duffy Blood-Group System', MS='A blood group consisting mainly of the antigens Fy(a) and Fy(b), determined by allelic genes, the frequency of which varies profoundly in different human groups; amorphic genes are common.'),
  'D004376': NtO(MH='Galactitol', MS='A naturally occurring product of plants obtained following reduction of GALACTOSE. It appears as a white crystalline powder with a slight sweet taste. It may form in excess in the lens of the eye in GALACTOSEMIAS, a deficiency of GALACTOKINASE.'),
  'D004377': NtO(MH='Dumping Syndrome', MS='Gastrointestinal symptoms resulting from an absent or nonfunctioning pylorus.'),
  'D004378': NtO(MH='Duodenal Diseases', MS='Pathological conditions in the DUODENUM region of the small intestine (INTESTINE, SMALL).'),
  'D004379': NtO(MH='Duodenal Neoplasms', MS='Tumors or cancer of the DUODENUM.'),
  'D004380': NtO(MH='Duodenal Obstruction', MS='Hindrance of the passage of luminal contents in the DUODENUM. Duodenal obstruction can be partial or complete, and caused by intrinsic or extrinsic factors. Simple obstruction is associated with diminished or stopped flow of luminal contents. Strangulating obstruction is associated with impaired blood flow to the duodenum in addition to obstructed flow of luminal contents.'),
  'D004381': NtO(MH='Duodenal Ulcer', MS='A PEPTIC ULCER located in the DUODENUM.'),
  'D004382': NtO(MH='Duodenitis', MS='Inflammation of the DUODENUM section of the small intestine (INTESTINE, SMALL). Erosive duodenitis may cause bleeding in the UPPER GI TRACT and PEPTIC ULCER.'),
  'D004383': NtO(MH='Duodenogastric Reflux', MS='Retrograde flow of duodenal contents (BILE ACIDS; PANCREATIC JUICE) into the STOMACH.'),
  'D004384': NtO(MH='Duodenoscopy', MS='Endoscopic examination, therapy or surgery of the luminal surface of the duodenum.'),
  'D004385': NtO(MH='Duodenostomy', MS='Surgical formation of an opening into the DUODENUM.'),
  'D004386': NtO(MH='Duodenum', MS='The shortest and widest portion of the SMALL INTESTINE adjacent to the PYLORUS of the STOMACH. It is named for having the length equal to about the width of 12 fingers.'),
  'D004387': NtO(MH='Dupuytren Contracture', MS='A fibromatosis of the palmar fascia characterized by thickening and contracture of the fibrous bands on the palmar surfaces of the hand and fingers. It arises most commonly in men between the ages of 30 and 50.'),
  'D004388': NtO(MH='Dura Mater', MS='The outermost of the three MENINGES, a fibrous membrane of connective tissue that covers the brain and the spinal cord.'),
  'D004389': NtO(MH='Durable Medical Equipment', MS='Devices which are very resistant to wear and may be used over a long period of time. They include items such as wheelchairs, hospital beds, artificial limbs, etc.'),
  'D004390': NtO(MH='Chlorpyrifos', MS='An organothiophosphate cholinesterase inhibitor that is used as an insecticide and as an acaricide.'),
  'D004391': NtO(MH='Dust', MS='Earth or other matter in fine, dry particles. (Random House Unabridged Dictionary, 2d ed)'),
  'D004392': NtO(MH='Dwarfism', MS='A genetic or pathological condition that is characterized by short stature and undersize. Abnormal skeletal growth usually results in an adult who is significantly below the average height.'),
  'D004393': NtO(MH='Dwarfism, Pituitary', MS='A form of dwarfism caused by complete or partial GROWTH HORMONE deficiency, resulting from either the lack of GROWTH HORMONE-RELEASING FACTOR from the HYPOTHALAMUS or from the mutations in the growth hormone gene (GH1) in the PITUITARY GLAND. It is also known as Type I pituitary dwarfism. Human hypophysial dwarf is caused by a deficiency of HUMAN GROWTH HORMONE during development.'),
  'D004394': NtO(MH='Dydrogesterone', MS='A synthetic progestational hormone with no androgenic or estrogenic properties. Unlike many other progestational compounds, dydrogesterone produces no increase in temperature and does not inhibit OVULATION.'),
  'D004395': NtO(MH='Dye Dilution Technique', MS='Method for assessing flow through a system by injection of a known quantity of dye into the system and monitoring its concentration over time at a specific point in the system. (From Dorland, 28th ed)'),
  'D004396': NtO(MH='Coloring Agents', MS='Chemicals and substances that impart color including soluble dyes and insoluble pigments. They are used in INKS; PAINTS; and as INDICATORS AND REAGENTS.'),
  'D004397': NtO(MH='Fonofos', MS='An organothiophosphorus cholinesterase inhibitor that is used as an insecticide.'),
  'D004398': NtO(MH='Dyneins', MS='A family of multisubunit cytoskeletal motor proteins that use the energy of ATP hydrolysis, generated by a ring of AAA ATPASES in the dynein heavy chain, to power a variety of cellular functions. Dyneins fall into two major classes based upon structural and functional criteria.'),
  'D004399': NtO(MH='Dynorphins', MS='A class of opioid peptides including dynorphin A, dynorphin B, and smaller fragments of these peptides. Dynorphins prefer kappa-opioid receptors (RECEPTORS, OPIOID, KAPPA) and have been shown to play a role as central nervous system transmitters.'),
  'D004400': NtO(MH='Dyphylline', MS='A THEOPHYLLINE derivative with broncho- and vasodilator properties. It is used in the treatment of asthma, cardiac dyspnea, and bronchitis.'),
  'D004401': NtO(MH='Dysarthria', MS='Disorders of speech articulation caused by imperfect coordination of pharynx, larynx, tongue, or face muscles. This may result from CRANIAL NERVE DISEASES; NEUROMUSCULAR DISEASES; CEREBELLAR DISEASES; BASAL GANGLIA DISEASES; BRAIN STEM diseases; or diseases of the corticobulbar tracts (see PYRAMIDAL TRACTS). The cortical language centers are intact in this condition. (From Adams et al., Principles of Neurology, 6th ed, p489)'),
  'D004402': NtO(MH='Dysautonomia, Familial', MS='An autosomal disorder of the peripheral and autonomic nervous systems limited to individuals of Ashkenazic Jewish descent. Clinical manifestations are present at birth and include diminished lacrimation, defective thermoregulation, orthostatic hypotension (HYPOTENSION, ORTHOSTATIC), fixed pupils, excessive SWEATING, loss of pain and temperature sensation, and absent reflexes. Pathologic features include reduced numbers of small diameter peripheral nerve fibers and autonomic ganglion neurons. (From Adams et al., Principles of Neurology, 6th ed, p1348; Nat Genet 1993;4(2):160-4)'),
  'D004403': NtO(MH='Dysentery', MS='Acute inflammation of the intestine associated with infectious DIARRHEA of various etiologies, generally acquired by eating contaminated food containing TOXINS, BIOLOGICAL derived from BACTERIA or other microorganisms. Dysentery is characterized initially by watery FECES then by bloody mucoid stools. It is often associated with ABDOMINAL PAIN; FEVER; and DEHYDRATION.'),
  'D004404': NtO(MH='Dysentery, Amebic', MS='DYSENTERY caused by intestinal amebic infection, chiefly with ENTAMOEBA HISTOLYTICA. This condition may be associated with amebic infection of the LIVER and other distant sites.'),
  'D004405': NtO(MH='Dysentery, Bacillary', MS='DYSENTERY caused by gram-negative rod-shaped enteric bacteria (ENTEROBACTERIACEAE), most often by the genus SHIGELLA. Shigella dysentery, Shigellosis, is classified into subgroups according to syndrome severity and the infectious species. Group A: SHIGELLA DYSENTERIAE (severest); Group B: SHIGELLA FLEXNERI; Group C: SHIGELLA BOYDII; and Group D: SHIGELLA SONNEI (mildest).'),
  'D004406': NtO(MH='Dysgammaglobulinemia', MS='An immunologic deficiency state characterized by selective deficiencies of one or more, but not all, classes of immunoglobulins.'),
  'D004407': NtO(MH='Dysgerminoma', MS='A malignant ovarian neoplasm, thought to be derived from primordial germ cells of the sexually undifferentiated embryonic gonad. It is the counterpart of the classical seminoma of the testis, to which it is both grossly and histologically identical. Dysgerminomas comprise 16% of all germ cell tumors but are rare before the age of 10, although nearly 50% occur before the age of 20. They are generally considered of low-grade malignancy but may spread if the tumor extends through its capsule and involves lymph nodes or blood vessels. (Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1646)'),
  'D004408': NtO(MH='Dysgeusia', MS='A condition characterized by alterations of the sense of taste which may range from mild to severe, including gross distortions of taste quality.'),
  'D004409': NtO(MH='Dyskinesia, Drug-Induced', MS='Abnormal movements, including HYPERKINESIS; HYPOKINESIA; TREMOR; and DYSTONIA, associated with the use of certain medications or drugs. Muscles of the face, trunk, neck, and extremities are most commonly affected. Tardive dyskinesia refers to abnormal hyperkinetic movements of the muscles of the face, tongue, and neck associated with the use of neuroleptic agents (see ANTIPSYCHOTIC AGENTS). (Adams et al., Principles of Neurology, 6th ed, p1199)'),
  'D004410': NtO(MH='Dyslexia', MS="A cognitive disorder characterized by an impaired ability to comprehend written and printed words or phrases despite intact vision. This condition may be developmental or acquired. Developmental dyslexia is marked by reading achievement that falls substantially below that expected given the individual's chronological age, measured intelligence, and age-appropriate education. The disturbance in reading significantly interferes with academic achievement or with activities of daily living that require reading skills. (From DSM-IV)"),
  'D004411': NtO(MH='Dyslexia, Acquired', MS='A receptive visual aphasia characterized by the loss of a previously possessed ability to comprehend the meaning or significance of handwritten words, despite intact vision. This condition may be associated with posterior cerebral artery infarction (INFARCTION, POSTERIOR CEREBRAL ARTERY) and other BRAIN DISEASES.'),
  'D004412': NtO(MH='Dysmenorrhea', MS='Painful menstruation.'),
  'D004413': NtO(MH='Dysostoses', MS='Defective bone formation involving individual bones, singly or in combination.'),
  'D004414': NtO(MH='Dyspareunia', MS='Recurrent genital pain occurring during, before, or after SEXUAL INTERCOURSE in either the male or the female.'),
  'D004415': NtO(MH='Dyspepsia', MS='Impaired digestion, especially after eating.'),
  'D004416': NtO(MH='Dysplastic Nevus Syndrome', MS='Clinically atypical nevi (usually exceeding 5 mm in diameter and having variable pigmentation and ill defined borders) with an increased risk for development of non-familial cutaneous malignant melanoma. Biopsies show melanocytic dysplasia. Nevi are clinically and histologically identical to the precursor lesions for melanoma in the B-K mole syndrome. (Stedman, 25th ed)'),
  'D004417': NtO(MH='Dyspnea', MS='Difficult or labored breathing.'),
  'D004418': NtO(MH='Dyspnea, Paroxysmal', MS='A disorder characterized by sudden attacks of respiratory distress in at rest patients with HEART FAILURE and PULMONARY EDEMA. It usually occurs at night after several hours of sleep in a reclining position. Patients awaken with a feeling of suffocation, coughing, a cold sweat, and TACHYCARDIA. When there is significant WHEEZING, it is called cardiac asthma.'),
  'D004419': NtO(MH='Dysprosium', MS='An element of the rare earth family that has the atomic symbol Dy, atomic number 66, and atomic weight 162.50. Dysprosium is a silvery metal used primarily in the form of various salts.'),
  'D004420': NtO(MH='Dystocia', MS='Slow or difficult OBSTETRIC LABOR or CHILDBIRTH.'),
  'D004421': NtO(MH='Dystonia', MS='An attitude or posture due to the co-contraction of agonists and antagonist muscles in one region of the body. It most often affects the large axial muscles of the trunk and limb girdles. Conditions which feature persistent or recurrent episodes of dystonia as a primary manifestation of disease are referred to as DYSTONIC DISORDERS. (Adams et al., Principles of Neurology, 6th ed, p77)'),
  'D004422': NtO(MH='Dystonia Musculorum Deformans', MS='A condition characterized by focal DYSTONIA that progresses to involuntary spasmodic contractions of the muscles of the legs, trunk, arms, and face. The hands are often spared, however, sustained axial and limb contractions may lead to a state where the body is grossly contorted. Onset is usually in the first or second decade. Familial patterns of inheritance, primarily autosomal dominant with incomplete penetrance, have been identified. (Adams et al., Principles of Neurology, 6th ed, p1078)'),
  'D004423': NtO(MH='Ear', MS='The hearing and equilibrium system of the body. It consists of three parts: the EXTERNAL EAR, the MIDDLE EAR, and the INNER EAR. Sound waves are transmitted through this organ where vibration is transduced to nerve signals that pass through the ACOUSTIC NERVE to the CENTRAL NERVOUS SYSTEM. The inner ear also contains the vestibular organ that maintains equilibrium by transducing signals to the VESTIBULAR NERVE.'),
  'D004424': NtO(MH='Ear Canal', MS='The narrow passage way that conducts the sound collected by the EAR AURICLE to the TYMPANIC MEMBRANE.'),
  'D004425': NtO(MH='Ear Cartilage', MS='Cartilage of the EAR AURICLE and the EXTERNAL EAR CANAL.'),
  'D004426': NtO(MH='Ear Deformities, Acquired', MS='Distortion or disfigurement of the ear caused by disease or injury after birth.'),
  'D004427': NtO(MH='Ear Diseases', MS='Pathological processes of the ear, the hearing, and the equilibrium system of the body.'),
  'D004428': NtO(MH='Ear Neoplasms', MS='Tumors or cancer of any part of the hearing and equilibrium system of the body (the EXTERNAL EAR, the MIDDLE EAR, and the INNER EAR).'),
  'D004429': NtO(MH='Ear Ossicles', MS='A mobile chain of three small bones (INCUS; MALLEUS; STAPES) in the TYMPANIC CAVITY between the TYMPANIC MEMBRANE and the oval window on the wall of INNER EAR. Sound waves are converted to vibration by the tympanic membrane then transmitted via these ear ossicles to the inner ear.'),
  'D004430': NtO(MH='Ear Protective Devices', MS='Personal devices for protection of the ears from loud or high intensity noise, water, or cold. These include earmuffs and earplugs.'),
  'D004431': NtO(MH='Ear, External', MS='The outer part of the hearing system of the body. It includes the shell-like EAR AURICLE which collects sound, and the EXTERNAL EAR CANAL, the TYMPANIC MEMBRANE, and the EXTERNAL EAR CARTILAGES.'),
  'D004432': NtO(MH='Ear, Middle', MS='The space and structures directly internal to the TYMPANIC MEMBRANE and external to the inner ear (LABYRINTH). Its major components include the AUDITORY OSSICLES and the EUSTACHIAN TUBE that connects the cavity of middle ear (tympanic cavity) to the upper part of the throat.'),
  'D004433': NtO(MH='Earache', MS='Pain in the ear.'),
  'D004434': NtO(MH='Early Ambulation', MS='Procedure to accelerate the ability of a patient to walk or move about by reducing the time to AMBULATION. It is characterized by a shorter period of hospitalization or recumbency than is normally practiced.'),
  'D004435': NtO(MH='Eating', MS='The consumption of edible substances.'),
  'D004437': NtO(MH='Ebstein Anomaly', MS='A congenital heart defect characterized by downward or apical displacement of the TRICUSPID VALVE, usually with the septal and posterior leaflets being attached to the wall of the RIGHT VENTRICLE. It is characterized by a huge RIGHT ATRIUM and a small and less effective right ventricle.'),
  'D004438': NtO(MH='Ecchymosis', MS='Extravasation of blood into the skin, resulting in a nonelevated, rounded or irregular, blue or purplish patch, larger than a petechia.'),
  'D004439': NtO(MH='Eccrine Glands', MS='Simple sweat glands that secrete sweat directly onto the SKIN.'),
  'D004440': NtO(MH='Ecdysone', MS='A steroid hormone that regulates the processes of MOLTING or ecdysis in insects.'),
  'D004441': NtO(MH='Ecdysterone', MS='A steroid hormone that regulates the processes of MOLTING or ecdysis in insects. Ecdysterone is the 20-hydroxylated ECDYSONE.'),
  'D004442': NtO(MH='Echidna', MS='An oviparous burrowing mammal of the order Monotremata native to Australia, Tasmania, and New Guinea. It has hair mingled with spines on the upper part of the body and is adapted for feeding on ants.'),
  'D004443': NtO(MH='Echinococcosis', MS='An infection caused by the infestation of the larval form of tapeworms of the genus Echinococcus. The liver, lungs, and kidney are the most common areas of infestation.'),
  'D004444': NtO(MH='Echinococcosis, Hepatic', MS='Liver disease caused by infections with parasitic tapeworms of the genus ECHINOCOCCUS, such as Echinococcus granulosus or Echinococcus multilocularis. Ingested Echinococcus ova burrow into the intestinal mucosa. The larval migration to the liver via the PORTAL VEIN leads to watery vesicles (HYDATID CYST).'),
  'D004445': NtO(MH='Echinococcosis, Pulmonary', MS='Helminth infection of the lung caused by Echinococcus granulosus or Echinococcus multilocularis.'),
  'D004446': NtO(MH='Echinococcus', MS='A genus of very small TAPEWORMS, in the family Taeniidae. The adult form is found in various CARNIVORA but not humans. The larval form is seen in humans under certain epidemiologic circumstances.'),
  'D004447': NtO(MH='Echinodermata', MS='A phylum of the most familiar marine invertebrates. Its class Stelleroidea contains two subclasses, the Asteroidea (the STARFISH or sea stars) and the Ophiuroidea (the brittle stars, also called basket stars and serpent stars). There are 1500 described species of STARFISH found throughout the world. The second class, Echinoidea, contains about 950 species of SEA URCHINS, heart urchins, and sand dollars. A third class, Holothuroidea, comprises about 900 echinoderms known as SEA CUCUMBERS. Echinoderms are used extensively in biological research. (From Barnes, Invertebrate Zoology, 5th ed, pp773-826)'),
  'D004448': NtO(MH='Echinomycin', MS='A cytotoxic polypeptide quinoxaline antibiotic isolated from Streptomyces echinatus that binds to DNA and inhibits RNA synthesis.'),
  'D004449': NtO(MH='Echinostoma', MS='A genus of intestinal flukes of the family Echinostomatidae which consists of many species. They occur in man and other vertebrates. The intermediate hosts are frequently mollusks.'),
  'D004450': NtO(MH='Echinostomatidae', MS='A family of flukes (TREMATODA) characterized by a collar of spines at their anterior end. The body is elongated and is covered with spines, and the two suckers are usually close together. (Noble et al., Parasitology: the Biology of Animal Parasites, 6th ed, p183)'),
  'D004451': NtO(MH='Echinostomiasis', MS='Infection by flukes of the genus Echinostoma.'),
  'D004452': NtO(MH='Echocardiography', MS='Ultrasonic recording of the size, motion, and composition of the heart and surrounding tissues. The standard approach is transthoracic.'),
  'D004453': NtO(MH='Echoencephalography', MS='Use of reflected ultrasound in the diagnosis of intracranial pathologic processes.'),
  'D004454': NtO(MH='Echolalia', MS='Involuntary ("parrot-like"), meaningless repetition of a recently heard word, phrase, or song. This condition may be associated with transcortical APHASIA; SCHIZOPHRENIA; or other disorders. (From Adams et al., Principles of Neurology, 6th ed, p485)'),
  'D004455': NtO(MH='Echolocation', MS='An auditory orientation mechanism involving the emission of high frequency sounds which are reflected back to the emitter (animal).'),
  'D004456': NtO(MH='Echothiophate Iodide', MS='A potent, long-acting cholinesterase inhibitor used as a miotic in the treatment of glaucoma.'),
  'D004457': NtO(MH='Echovirus Infections', MS='Infectious disease processes, including meningitis, diarrhea, and respiratory disorders, caused by echoviruses.'),
  'D004458': NtO(MH='Echovirus 6, Human', MS='A species of ENTEROVIRUS that has caused outbreaks of aseptic meningitis in children and adults.'),
  'D004459': NtO(MH='Echovirus 9', MS='A species of ENTEROVIRUS associated with outbreaks of aseptic meningitis (MENINGITIS, ASEPTIC).'),
  'D004461': NtO(MH='Eclampsia', MS='Onset of HYPERREFLEXIA; SEIZURES; or COMA in a previously diagnosed pre-eclamptic patient (PRE-ECLAMPSIA).'),
  'D004463': NtO(MH='Ecology', MS="The branch of science concerned with the interrelationship of organisms and their ENVIRONMENT, especially as manifested by natural cycles and rhythms, community development and structure, interactions between different kinds of organisms, geographic distributions, and population alterations. (Webster's, 3d ed)"),
  'D004464': NtO(MH='Econazole', MS='An imidazole derivative that is commonly used as a topical antifungal agent.'),
  'D004465': NtO(MH='Economic Competition', MS='The effort of two or more parties to secure the business of a third party by offering, usually under fair or equitable rules of business practice, the most favorable terms.'),
  'D004467': NtO(MH='Economics', MS='The science of utilization, distribution, and consumption of services and materials.'),
  'D004468': NtO(MH='Economics, Dental', MS='Economic aspects of the dental profession and dental care.'),
  'D004469': NtO(MH='Economics, Hospital', MS='Economic aspects related to the management and operation of a hospital.'),
  'D004470': NtO(MH='Economics, Medical', MS='Economic aspects of the field of medicine, the medical profession, and health care. It includes the economic and financial impact of disease in general on the patient, the physician, society, or government.'),
  'D004471': NtO(MH='Economics, Nursing', MS='Economic aspects of the nursing profession.'),
  'D004472': NtO(MH='Health Care Economics and Organizations', MS='The economic aspects of health care, its planning, and delivery. It includes government agencies and organizations in the private sector.'),
  'D004473': NtO(MH='Ecthyma', MS='An ulcerative pyoderma usually caused by group A beta-hemolytic streptococcal infection at the site of minor trauma. (Dorland, 27th ed)'),
  'D004474': NtO(MH='Ecthyma, Contagious', MS='An infectious dermatitis of sheep and goats, affecting primarily the muzzle and lips. It is caused by a poxvirus and may be transmitted to man.'),
  'D004475': NtO(MH='Ectoderm', MS='The outer of the three germ layers of an embryo.'),
  'D004476': NtO(MH='Ectodermal Dysplasia', MS='A group of hereditary disorders involving tissues and structures derived from the embryonic ectoderm. They are characterized by the presence of abnormalities at birth and involvement of both the epidermis and skin appendages. They are generally nonprogressive and diffuse. Various forms exist, including anhidrotic and hidrotic dysplasias, FOCAL DERMAL HYPOPLASIA, and aplasia cutis congenita.'),
  'D004477': NtO(MH='Ectogenesis', MS='Embryonic and fetal development that takes place in an artificial environment in vitro.'),
  'D004478': NtO(MH='Ectoparasitic Infestations', MS="Infestations by PARASITES which live on, or burrow into, the surface of their host's EPIDERMIS. Most ectoparasites are ARTHROPODS."),
  'D004479': NtO(MH='Ectopia Lentis', MS='Congenital displacement of the lens resulting from defective zonule formation.'),
  'D004480': NtO(MH='Ectromelia', MS='Gross hypo- or aplasia of one or more long bones of one or more limbs. The concept includes amelia, hemimelia, phocomelia, and sirenomelia.'),
  'D004481': NtO(MH='Ectromelia virus', MS='A species of ORTHOPOXVIRUS infecting mice and causing a disease that involves internal organs and produces characteristic skin lesions.'),
  'D004482': NtO(MH='Ectromelia, Infectious', MS='A viral infection of mice, causing edema and necrosis followed by limb loss.'),
  'D004483': NtO(MH='Ectropion', MS='The turning outward (eversion) of the edge of the eyelid, resulting in the exposure of the palpebral conjunctiva. (Dorland, 27th ed)'),
  'D004484': NtO(MH='Ecuador', MS='A country in western South America, bordering the Pacific Ocean at the equator, between Colombia and Peru.'),
  'D004485': NtO(MH='Eczema', MS='A pruritic papulovesicular dermatitis occurring as a reaction to many endogenous and exogenous agents (Dorland, 27th ed).'),
  'D004486': NtO(MH='Edeine', MS='Basic peptide antibiotic from Bacillus brevis. It exhibits broad-spectrum antibacterial activity and inhibits bacterial DNA synthesis.'),
  'D004487': NtO(MH='Edema', MS='Abnormal fluid accumulation in TISSUES or body cavities. Most cases of edema are present under the SKIN in SUBCUTANEOUS TISSUE.'),
  'D004488': NtO(MH='Edema Disease of Swine', MS='An acute disease of young pigs that is usually associated with weaning. It is characterized clinically by paresis and subcutaneous edema.'),
  'D004489': NtO(MH='Edema, Cardiac', MS='Abnormal fluid retention by the body due to impaired cardiac function or heart failure. It is usually characterized by increase in venous and capillary pressure, and swollen legs when standing. It is different from the generalized edema caused by renal dysfunction (NEPHROTIC SYNDROME).'),
  'D004490': NtO(MH='Cingulata', MS="An order of New World mammals characterized by the absence of incisors and canines from among their teeth, and comprising the ARMADILLOS, the SLOTHS, and the anteaters. The order is distinguished from all others by what are known as xenarthrous vertebrae (xenos, strange; arthron, joint): there are secondary, and sometimes even more, articulations between the vertebrae of the lumbar series. The order was formerly called Edentata. (From Random House Unabridged Dictionary, 2d ed; Walker's Mammals of the World, 5th ed, vol. I, p515)"),
  'D004491': NtO(MH='Edrophonium', MS='A rapid-onset, short-acting cholinesterase inhibitor used in cardiac arrhythmias and in the diagnosis of myasthenia gravis. It has also been used as an antidote to curare principles.'),
  'D004492': NtO(MH='Edetic Acid', MS='A chelating agent that sequesters a variety of polyvalent cations such as CALCIUM. It is used in pharmaceutical manufacturing and as a food additive.'),
  'D004493': NtO(MH='Education', MS='Acquisition of knowledge as a result of instruction in a formal course of study.'),
  'D004494': NtO(MH='Education Department, Hospital', MS='Hospital department responsible for administering educational and training activities pertaining to health for patients and staff.'),
  'D004495': NtO(MH='Education of Intellectually Disabled', MS='The teaching or training of those individuals with subnormal intellectual functioning.'),
  'D004496': NtO(MH='Education, Continuing', MS='Educational programs designed to inform individuals of recent advances in their particular field of interest. They do not lead to any formal advanced standing.'),
  'D004497': NtO(MH='Education, Dental', MS='Use for articles concerning dental education in general.'),
  'D004498': NtO(MH='Education, Dental, Continuing', MS='Educational programs designed to inform dentists of recent advances in their fields.'),
  'D004499': NtO(MH='Education, Dental, Graduate', MS='Educational programs for dental graduates entering a specialty. They include formal specialty training as well as academic work in the clinical and basic dental sciences, and may lead to board certification or an advanced dental degree.'),
  'D004500': NtO(MH='Education, Graduate', MS="Studies beyond the bachelor's degree at an institution having graduate programs for the purpose of preparing for entrance into a specific field, and obtaining a higher degree."),
  'D004501': NtO(MH='Education, Medical', MS='Use for general articles concerning medical education.'),
  'D004502': NtO(MH='Education, Medical, Continuing', MS='Educational programs designed to inform physicians of recent advances in their field.'),
  'D004503': NtO(MH='Education, Medical, Graduate', MS='Educational programs for medical graduates entering a specialty. They include formal specialty training as well as academic work in the clinical and basic medical sciences, and may lead to board certification or an advanced medical degree.'),
  'D004504': NtO(MH='Education, Medical, Undergraduate', MS='The period of medical education in a medical school. In the United States it follows the baccalaureate degree and precedes the granting of the M.D.'),
  'D004505': NtO(MH='Education, Nonprofessional', MS='Education and training outside that for the professions.'),
  'D004506': NtO(MH='Education, Nursing', MS='Use for general articles concerning nursing education.'),
  'D004507': NtO(MH='Education, Nursing, Associate', MS='A two-year program in nursing education in a community or junior college leading to an A.D. (Associate Degree). Graduates of this program are eligible for state examination for licensure as RN (Registered Nurse).'),
  'D004508': NtO(MH='Education, Nursing, Baccalaureate', MS='A four-year program in nursing education in a college or university leading to a B.S.N. (Bachelor of Science in Nursing). Graduates are eligible for state examination for licensure as RN (Registered Nurse).'),
  'D004509': NtO(MH='Education, Nursing, Continuing', MS='Educational programs designed to inform nurses of recent advances in their fields.'),
  'D004510': NtO(MH='Education, Nursing, Diploma Programs', MS='Programs usually offered in hospital schools of nursing leading to a registered nurse diploma (RN). Graduates are eligible for state examination for licensure as RN (Registered Nurse).'),
  'D004511': NtO(MH='Education, Nursing, Graduate', MS="Those educational activities engaged in by holders of a bachelor's degree in nursing, which are primarily designed to prepare them for entrance into a specific field of nursing, and may lead to board certification or a more advanced degree."),
  'D004512': NtO(MH='Education, Pharmacy', MS='Formal instruction, learning, or training in the preparation, dispensing, and proper utilization of drugs in the field of medicine.'),
  'D004513': NtO(MH='Education, Pharmacy, Continuing', MS='Educational programs designed to inform graduate pharmacists of recent advances in their particular field.'),
  'D004514': NtO(MH='Education, Pharmacy, Graduate', MS="Educational programs for pharmacists who have a bachelor's degree or a Doctor of Pharmacy degree entering a specific field of pharmacy. They may lead to an advanced degree."),
  'D004515': NtO(MH='Education, Predental', MS='Preparatory education meeting the requirements for admission to dental school.'),
  'D004516': NtO(MH='Education, Premedical', MS='Introductory courses leading to an advance study in medicine.'),
  'D004517': NtO(MH='Education, Professional', MS='Formal education and training in preparation for the practice of a profession.'),
  'D004518': NtO(MH='Education, Professional, Retraining', MS='Educational programs for individuals who have been inactive in their profession, or who wish to regain unused skills.'),
  'D004519': NtO(MH='Education, Special', MS='Education of the individual who markedly deviates intellectually, physically, socially, or emotionally from those considered to be normal, thus requiring special instruction.'),
  'D004520': NtO(MH='Education, Veterinary', MS='Use for general articles concerning veterinary medical education.'),
  'D004521': NtO(MH='Educational Measurement', MS='The assessing of academic or educational achievement. It includes all aspects of testing and test construction.'),
  'D004522': NtO(MH='Educational Status', MS='Educational attainment or level of education of an individual.'),
  'D004524': NtO(MH='Eels', MS='Common name for an order (Anguilliformes) of voracious, elongate, snakelike teleost fishes.'),
  'D004525': NtO(MH='Efferent Pathways', MS='Nerve structures through which impulses are conducted from a nerve center toward a peripheral site. Such impulses are conducted via efferent neurons (NEURONS, EFFERENT), such as MOTOR NEURONS, autonomic neurons, and hypophyseal neurons.'),
  'D004526': NtO(MH='Efficiency', MS='Ratio of output to effort and or resources, or the ratio of effort and or resources produced to energy expended.'),
  'D004527': NtO(MH='Egg Proteins', MS='Proteins which are found in eggs (OVA) from any species.'),
  'D004528': NtO(MH='Egg Shell', MS='A hard or leathery calciferous exterior covering of an egg.'),
  'D004529': NtO(MH='Egg White', MS="The white of an egg, especially a chicken's egg, used in cooking. It contains albumin. (Random House Unabridged Dictionary, 2d ed)"),
  'D004530': NtO(MH='Egg Yolk', MS='Cytoplasm stored in an egg that contains nutritional reserves for the developing embryo. It is rich in polysaccharides, lipids, and proteins.'),
  'D004531': NtO(MH='Eggs', MS='Animal reproductive bodies, or the contents thereof, used as food. The concept is differentiated from OVUM, the anatomic or physiologic entity.'),
  'D004532': NtO(MH='Ego', MS='The conscious portion of the personality structure which serves to mediate between the demands of the primitive instinctual drives, (the id), of internalized parental and social prohibitions or the conscience, (the superego), and of reality.'),
  'D004533': NtO(MH='Egtazic Acid', MS='A chelating agent relatively more specific for calcium and less toxic than EDETIC ACID.'),
  'D004534': NtO(MH='Egypt', MS='A country in northern Africa, bordering the Mediterranean Sea, between Libya and the Gaza Strip, and the Red Sea north of Sudan, and includes the Asian Sinai Peninsula Its capital is Cairo.'),
  'D004535': NtO(MH='Ehlers-Danlos Syndrome', MS='A heterogeneous group of autosomally inherited COLLAGEN DISEASES caused by defects in the synthesis or structure of FIBRILLAR COLLAGEN. There are numerous subtypes: classical, hypermobility, vascular, and others. Common clinical features include hyperextensible skin and joints, skin fragility and reduced wound healing capability.'),
  'D004536': NtO(MH='Ehrlichia', MS='Small, often pleomorphic, coccoid to ellipsoidal organisms occurring intracytoplasmically in circulating LYMPHOCYTES. They are the etiologic agents of tick-borne diseases of humans; DOGS; CATTLE; SHEEP; GOATS; and HORSES.'),
  'D004537': NtO(MH='Eicosanoic Acids', MS='20-carbon saturated monocarboxylic acids.'),
  'D004538': NtO(MH='Eidetic Imagery', MS='A visual image which is recalled in accurate detail. It is a sort of projection of an image on a mental screen.'),
  'D004539': NtO(MH='Eimeria', MS='A genus of protozoan parasites of the subclass COCCIDIA. Various species are parasitic in the epithelial cells of the liver and intestines of man and other animals.'),
  'D004540': NtO(MH='Einsteinium', MS='A man-made radioactive actinide with atomic symbol Es, and atomic number 99. Its known isotopes range in mass number from 240-258. Its valence can be +2 or +3. Einsteinium was originally discovered in the debris from a thermonuclear explosion in 1952.'),
  'D004541': NtO(MH='Eisenmenger Complex', MS='A condition associated with VENTRICULAR SEPTAL DEFECT and other congenital heart defects that allow the mixing of pulmonary and systemic circulation, increase blood flow into the lung, and subsequent responses to low oxygen in blood. This complex is characterized by progressive PULMONARY HYPERTENSION; HYPERTROPHY of the RIGHT VENTRICLE; CYANOSIS; and ERYTHROCYTOSIS.'),
  'D004542': NtO(MH='Ejaculation', MS='The emission of SEMEN to the exterior, resulting from the contraction of muscles surrounding the male internal urogenital ducts.'),
  'D004543': NtO(MH='Ejaculatory Ducts', MS='Paired ducts in the human male through which semen is ejaculated into the urethra.'),
  'D004544': NtO(MH='El Salvador', MS='A country in Central America, bordering the north Pacific Ocean, between Guatemala and Honduras. The capital is San Salvador.'),
  'D004546': NtO(MH='Elapid Venoms', MS='Venoms from snakes of the family Elapidae, including cobras, kraits, mambas, coral, tiger, and Australian snakes. The venoms contain polypeptide toxins of various kinds, cytolytic, hemolytic, and neurotoxic factors, but fewer enzymes than viper or crotalid venoms. Many of the toxins have been characterized.'),
  'D004547': NtO(MH='Elastic Tissue', MS='Connective tissue comprised chiefly of elastic fibers. Elastic fibers have two components: ELASTIN and MICROFIBRILS.'),
  'D004548': NtO(MH='Elasticity', MS='Resistance and recovery from distortion of shape.'),
  'D004549': NtO(MH='Elastin', MS=None),
  'D004550': NtO(MH='Elbow', MS='Region of the body immediately surrounding and including the ELBOW JOINT.'),
  'D004551': NtO(MH='Elbow Joint', MS='A hinge joint connecting the FOREARM to the ARM.'),
  'D004552': NtO(MH='Elder Abuse', MS='Emotional, nutritional, financial, or physical maltreatment, exploitation, or abandonment of the older person generally by family members or by institutional personnel.'),
  'D004553': NtO(MH='Electric Conductivity', MS='The ability of a substrate to allow the passage of ELECTRONS.'),
  'D004554': NtO(MH='Electric Countershock', MS='An electrical current applied to the HEART to terminate a CARDIAC ARRHYTHMIA.'),
  'D004555': NtO(MH='Electric Fish', MS='Fishes which generate an electric discharge. The voltage of the discharge varies from weak to strong in various groups of fish. The ELECTRIC ORGAN and electroplax are of prime interest in this group. They occur in more than one family.'),
  'D004556': NtO(MH='Electric Injuries', MS='Injuries caused by electric currents. The concept excludes electric burns (BURNS, ELECTRIC), but includes accidental electrocution and electric shock.'),
  'D004557': NtO(MH='Electric Organ', MS='In about 250 species of electric fishes, modified muscle fibers forming disklike multinucleate plates arranged in stacks like batteries in series and embedded in a gelatinous matrix. A large torpedo ray may have half a million plates. Muscles in different parts of the body may be modified, i.e., the trunk and tail in the electric eel, the hyobranchial apparatus in the electric ray, and extrinsic eye muscles in the stargazers. Powerful electric organs emit pulses in brief bursts several times a second. They serve to stun prey and ward off predators. A large torpedo ray can produce of shock of more than 200 volts, capable of stunning a human. (Storer et al., General Zoology, 6th ed, p672)'),
  'D004558': NtO(MH='Electric Stimulation', MS='Use of electric potential or currents to elicit biological responses.'),
  'D004559': NtO(MH='Electric Wiring', MS='An arrangement of wires distributing electricity.'),
  'D004560': NtO(MH='Electricity', MS='The physical effects involving the presence of electric charges at rest and in motion.'),
  'D004561': NtO(MH='Transcutaneous Electric Nerve Stimulation', MS='The use of specifically placed small electrodes to deliver electrical impulses across the SKIN to relieve PAIN. It is used less frequently to produce ANESTHESIA.'),
  'D004562': NtO(MH='Electrocardiography', MS="Recording of the moment-to-moment electromotive forces of the HEART as projected onto various sites on the body's surface, delineated as a scalar function of time. The recording is monitored by a tracing on slow moving chart paper or by observing it on a cardioscope, which is a CATHODE RAY TUBE DISPLAY."),
  'D004563': NtO(MH='Electrochemistry', MS='The study of chemical changes resulting from electrical action and electrical activity resulting from chemical changes.'),
  'D004564': NtO(MH='Electrocoagulation', MS='Procedures using an electrically heated wire or scalpel to treat hemorrhage (e.g., bleeding ulcers) and to ablate tumors, mucosal lesions, and refractory arrhythmias. It is different from ELECTROSURGERY which is used more for cutting tissue than destroying and in which the patient is part of the electric circuit.'),
  'D004565': NtO(MH='Electroconvulsive Therapy', MS='Electrically induced CONVULSIONS primarily used in the treatment of severe AFFECTIVE DISORDERS and SCHIZOPHRENIA.'),
  'D004566': NtO(MH='Electrodes', MS='Electric conductors through which electric currents enter or leave a medium, whether it be an electrolytic solution, solid, molten mass, gas, or vacuum.'),
  'D004567': NtO(MH='Electrodes, Implanted', MS='Surgically placed electric conductors through which ELECTRIC STIMULATION is delivered to or electrical activity is recorded from a specific point inside the body.'),
  'D004568': NtO(MH='Electrodiagnosis', MS='Diagnosis of disease states by recording the spontaneous electrical activity of tissues or organs or by the response to stimulation of electrically excitable tissue.'),
  'D004569': NtO(MH='Electroencephalography', MS='Recording of electric currents developed in the brain by means of electrodes applied to the scalp, to the surface of the brain, or placed within the substance of the brain.'),
  'D004570': NtO(MH='Electrogalvanism, Intraoral', MS='Chemical reaction that occurs when saliva comes into contact with the metals in dental fillings. This reaction can send electric currents through the brain causing negative symptoms.'),
  'D004571': NtO(MH='Electrokymography', MS='Radiographic technique combining a photoelectric recording system with fluoroscopy. It is used especially with electrocardiography to study heart motion.'),
  'D004572': NtO(MH='Electrolysis', MS='Destruction by passage of a galvanic electric current, as in disintegration of a chemical compound in solution.'),
  'D004573': NtO(MH='Electrolytes', MS="Substances that dissociate into two or more ions, to some extent, in water. Solutions of electrolytes thus conduct an electric current and can be decomposed by it (ELECTROLYSIS). (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D004574': NtO(MH='Electromagnetic Fields', MS='Fields representing the joint interplay of electric and magnetic forces.'),
  'D004576': NtO(MH='Electromyography', MS='Recording of the changes in electric potential of muscle by means of surface or needle electrodes.'),
  'D004577': NtO(MH='Electron Probe Microanalysis', MS='Identification and measurement of ELEMENTS and their location based on the fact that X-RAYS emitted by an element excited by an electron beam have a wavelength characteristic of that element and an intensity related to its concentration. It is performed with an electron microscope fitted with an x-ray spectrometer, in scanning or transmission mode.'),
  'D004578': NtO(MH='Electron Spin Resonance Spectroscopy', MS='A technique applicable to the wide variety of substances which exhibit paramagnetism because of the magnetic moments of unpaired electrons. The spectra are useful for detection and identification, for determination of electron structure, for study of interactions between molecules, and for measurement of nuclear spins and moments. (From McGraw-Hill Encyclopedia of Science and Technology, 7th edition) Electron nuclear double resonance (ENDOR) spectroscopy is a variant of the technique which can give enhanced resolution. Electron spin resonance analysis can now be used in vivo, including imaging applications such as MAGNETIC RESONANCE IMAGING.'),
  'D004579': NtO(MH='Electron Transport', MS='The process by which ELECTRONS are transported from a reduced substrate to molecular OXYGEN. (From Bennington, Saunders Dictionary and Encyclopedia of Laboratory Medicine and Technology, 1984, p270)'),
  'D004580': NtO(MH='Electronarcosis', MS='Profound stupor produced by passing an electric current through the brain.'),
  'D004581': NtO(MH='Electronics', MS='The study, control, and application of the conduction of ELECTRICITY through gases or vacuum, or through semiconducting or conducting materials. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D004582': NtO(MH='Electronics, Medical', MS='The research and development of ELECTRICAL EQUIPMENT AND SUPPLIES for such medical applications as diagnosis, therapy, research, anesthesia control, cardiac control, and surgery. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D004583': NtO(MH='Electrons', MS='Stable elementary particles having the smallest known negative charge, present in all elements; also called negatrons. Positively charged electrons are called positrons. The numbers, energies and arrangement of electrons around atomic nuclei determine the chemical identities of elements. Beams of electrons are called CATHODE RAYS.'),
  'D004584': NtO(MH='Electronystagmography', MS='Recording of nystagmus based on changes in the electrical field surrounding the eye produced by the difference in potential between the cornea and the retina.'),
  'D004585': NtO(MH='Electrooculography', MS='Recording of the average amplitude of the resting potential arising between the cornea and the retina in light and dark adaptation as the eyes turn a standard distance to the right and the left. The increase in potential with light adaptation is used to evaluate the condition of the retinal pigment epithelium.'),
  'D004586': NtO(MH='Electrophoresis', MS='An electrochemical process in which macromolecules or colloidal particles with a net electric charge migrate in a solution under the influence of an electric current.'),
  'D004587': NtO(MH='Electrophoresis, Agar Gel', MS='Electrophoresis in which agar or agarose gel is used as the diffusion medium.'),
  'D004588': NtO(MH='Electrophoresis, Cellulose Acetate', MS='Electrophoresis in which cellulose acetate is the diffusion medium.'),
  'D004589': NtO(MH='Electrophoresis, Disc', MS="Electrophoresis in which discontinuities in both the voltage and pH gradients are introduced by using buffers of different composition and pH in the different parts of the gel column. The term 'disc' was originally used as an abbreviation for 'discontinuous' referring to the buffers employed, and does not have anything to do with the shape of the separated zones."),
  'D004590': NtO(MH='Electrophoresis, Paper', MS='Electrophoresis in which paper is used as the diffusion medium. This technique is confined almost entirely to separations of small molecules such as amino acids, peptides, and nucleotides, and relatively high voltages are nearly always used.'),
  'D004591': NtO(MH='Electrophoresis, Polyacrylamide Gel', MS='Electrophoresis in which a polyacrylamide gel is used as the diffusion medium.'),
  'D004592': NtO(MH='Electrophoresis, Starch Gel', MS='Electrophoresis in which a starch gel (a mixture of amylose and amylopectin) is used as the diffusion medium.'),
  'D004593': NtO(MH='Electrophorus', MS='A genus of fish, in the family GYMNOTIFORMES, capable of producing an electric shock that immobilizes fish and other prey. The species Electrophorus electricus is also known as the electric eel, though it is not a true eel.'),
  'D004594': NtO(MH='Electrophysiology', MS='The study of the generation and behavior of electrical charges in living organisms particularly the nervous system and the effects of electricity on living organisms.'),
  'D004595': NtO(MH='Electroplating', MS='Coating with a metal or alloy by electrolysis.'),
  'D004596': NtO(MH='Electroretinography', MS='Recording of electric potentials in the retina after stimulation by light.'),
  'D004597': NtO(MH='Electroshock', MS='Induction of a stress reaction in experimental subjects by means of an electrical shock; applies to either convulsive or non-convulsive states.'),
  'D004598': NtO(MH='Electrosurgery', MS='Division of tissues by a high-frequency current applied locally with a metal instrument or needle. (Stedman, 25th ed)'),
  'D004599': NtO(MH='Electric Stimulation Therapy', MS='Application of electric current in treatment without the generation of perceptible heat. It includes electric stimulation of nerves or muscles, passage of current into the body, or use of interrupted current of low intensity to raise the detection threshold of the skin to pain.'),
  'D004600': NtO(MH='Eledoisin', MS='A peptide extracted from the posterior salivary glands of certain small octopi (Eledone spp., Mollusca), or obtained by synthesis. Its actions resemble those of SUBSTANCE P; it is a potent vasodilator and increases capillary permeability. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1364)'),
  'D004601': NtO(MH='Elementary Particles', MS='Individual components of atoms, usually subatomic; subnuclear particles are usually detected only when the atomic nucleus decays and then only transiently, as most of them are unstable, often yielding pure energy without substance, i.e., radiation.'),
  'D004602': NtO(MH='Elements', MS='Substances that comprise all matter. Each element is made up of atoms that are identical in number of electrons and protons and in nuclear charge, but may differ in mass or number of neutrons.'),
  'D004603': NtO(MH='Elements, Radioactive', MS='Unstable elements with atomic numbers greater than 83 that exist in nature solely as radioisotopes.'),
  'D004604': NtO(MH='Elephantiasis', MS='Hypertrophy and thickening of tissues from causes other than filarial infection, the latter being described as ELEPHANTIASIS, FILARIAL.'),
  'D004605': NtO(MH='Elephantiasis, Filarial', MS='Parasitic infestation of the human lymphatic system by WUCHERERIA BANCROFTI or BRUGIA MALAYI. It is also called lymphatic filariasis.'),
  'D004606': NtO(MH='Elephants', MS='Large mammals in the family Elephantidae, with columnar limbs, bulky bodies, and elongated snouts. They are the only surviving members of the PROBOSCIDEA MAMMALS.'),
  'D004607': NtO(MH='Elevators and Escalators', MS='Mechanical ascending and descending devices which convey objects and/or people.'),
  'D004608': NtO(MH='Eligibility Determination', MS='Criteria to determine eligibility of patients for medical care programs and services.'),
  'D004609': NtO(MH='Eliminative Behavior, Animal', MS='Behavior associated with the elimination of feces and urine from the body.'),
  'D004610': NtO(MH='Ellagic Acid', MS='A fused four ring compound occurring free or combined in galls. Isolated from the kino of Eucalyptus maculata Hook and E. Hemipholia F. Muell. Activates Factor XII of the blood clotting system which also causes kinin release; used in research and as a dye.'),
  'D004611': NtO(MH='Ellipticines', MS='Pyrido-CARBAZOLES originally discovered in the bark of OCHROSIA ELLIPTICA. They inhibit DNA and RNA synthesis and have immunosuppressive properties.'),
  'D004612': NtO(MH='Elliptocytosis, Hereditary', MS='An intrinsic defect of erythrocytes inherited as an autosomal dominant trait. The erythrocytes assume an oval or elliptical shape.'),
  'D004613': NtO(MH='Ellis-Van Creveld Syndrome', MS='Dwarfism occurring in association with defective development of skin, hair, and teeth, polydactyly, and defect of the cardiac septum. (Dorland, 27th ed)'),
  'D004614': NtO(MH='Emaciation', MS='Clinical manifestation of excessive LEANNESS usually caused by disease or a lack of nutrition (MALNUTRITION).'),
  'D004615': NtO(MH='Embalming', MS='Process of preserving a dead body to protect it from decay.'),
  'D004616': NtO(MH='Emblems and Insignia', MS='Figures or symbols identifying or representing organizations or societies.'),
  'D004617': NtO(MH='Embolism', MS='Blocking of a blood vessel by an embolus which can be a blood clot or other undissolved material in the blood stream.'),
  'D004618': NtO(MH='Embolism, Air', MS='Blocking of a blood vessel by air bubbles that enter the circulatory system, usually after TRAUMA; surgical procedures, or changes in atmospheric pressure.'),
  'D004619': NtO(MH='Embolism, Amniotic Fluid', MS='Blocking of maternal circulation by AMNIOTIC FLUID that is forced into uterine VEINS by strong UTERINE CONTRACTION near the end of pregnancy. It is characterized by the sudden onset of severe respiratory distress and HYPOTENSION that can lead to maternal DEATH.'),
  'D004620': NtO(MH='Embolism, Fat', MS='Blocking of a blood vessel by fat deposits in the circulation. It is often seen after fractures of large bones or after administration of CORTICOSTEROIDS.'),
  'D004621': NtO(MH='Embolization, Therapeutic', MS='A method of hemostasis utilizing various agents such as Gelfoam, silastic, metal, glass, or plastic pellets, autologous clot, fat, and muscle as emboli. It has been used in the treatment of spinal cord and INTRACRANIAL ARTERIOVENOUS MALFORMATIONS, renal arteriovenous fistulas, gastrointestinal bleeding, epistaxis, hypersplenism, certain highly vascular tumors, traumatic rupture of blood vessels, and control of operative hemorrhage.'),
  'D004622': NtO(MH='Embryo, Mammalian', MS='The entity of a developing mammal (MAMMALS), generally from the cleavage of a ZYGOTE to the end of embryonic differentiation of basic structures. For the human embryo, this represents the first two months of intrauterine development preceding the stages of the FETUS.'),
  'D004624': NtO(MH='Embryo Transfer', MS='The transfer of mammalian embryos from an in vivo or in vitro environment to a suitable host to improve pregnancy or gestational outcome in human or animal. In human fertility treatment programs, preimplantation embryos ranging from the 4-cell stage to the blastocyst stage are transferred to the uterine cavity between 3-5 days after FERTILIZATION IN VITRO.'),
  'D004625': NtO(MH='Embryo, Nonmammalian', MS='The developmental entity of a fertilized egg (ZYGOTE) in animal species other than MAMMALS. For chickens, use CHICK EMBRYO.'),
  'D004626': NtO(MH='Embryology', MS='The study of the development of an organism during the embryonic and fetal stages of life.'),
  'D004627': NtO(MH='Embryonic Induction', MS='The complex processes of initiating CELL DIFFERENTIATION in the embryo. The precise regulation by cell interactions leads to diversity of cell types and specific pattern of organization (EMBRYOGENESIS).'),
  'D004628': NtO(MH='Embryonic Structures', MS='The anatomical parts that make up an organism in the early stages of development.'),
  'D004629': NtO(MH='Emepronium', MS='A muscarinic antagonist used mainly in the treatment of urinary syndromes. It is incompletely absorbed from the gastrointestinal tract and does not cross the blood-brain barrier.'),
  'D004630': NtO(MH='Emergencies', MS='Situations or conditions requiring immediate intervention to avoid serious adverse results.'),
  'D004631': NtO(MH='Emergency Medical Service Communication Systems', MS='The use of communication systems, such as telecommunication, to transmit emergency information to appropriate providers of health services.'),
  'D004632': NtO(MH='Emergency Medical Services', MS='Services specifically designed, staffed, and equipped for the emergency care of patients.'),
  'D004633': NtO(MH='Emergency Medical Tags', MS='A bracelet or necklace worn by an individual that alerts emergency personnel of medical information for that individual which could affect their condition or treatment.'),
  'D004634': NtO(MH='Emergency Medical Technicians', MS='Paramedical personnel trained to provide basic emergency care and life support under the supervision of physicians and/or nurses. These services may be carried out at the site of the emergency, in the ambulance, or in a health care institution.'),
  'D004635': NtO(MH='Emergency Medicine', MS='The branch of medicine concerned with the evaluation and initial treatment of urgent and emergent medical problems, such as those caused by accidents, trauma, sudden illness, poisoning, or disasters. Emergency medical care can be provided at the hospital or at sites outside the medical facility.'),
  'D004636': NtO(MH='Emergency Service, Hospital', MS='Hospital department responsible for the administration and provision of immediate medical or surgical care to the emergency patient.'),
  'D004637': NtO(MH='Emergency Services, Psychiatric', MS='Organized services to provide immediate psychiatric care to patients with acute psychological disturbances.'),
  'D004638': NtO(MH='Emergency Treatment', MS='First aid or other immediate intervention for accidents or medical conditions requiring immediate care and treatment before definitive medical and surgical management can be procured.'),
  'D004639': NtO(MH='Emetics', MS='Agents that cause vomiting. They may act directly on the gastrointestinal tract, bringing about emesis through local irritant effects, or indirectly, through their effects on the chemoreceptor trigger zone in the postremal area near the medulla.'),
  'D004640': NtO(MH='Emetine', MS='The principal alkaloid of ipecac, from the ground roots of Uragoga (or Cephaelis) ipecacuanha or U. acuminata, of the Rubiaceae. It is used as an amebicide in many different preparations and may cause serious cardiac, hepatic, or renal damage and violent diarrhea and vomiting. Emetine inhibits protein synthesis in EUKARYOTIC CELLS but not PROKARYOTIC CELLS.'),
  'D004641': NtO(MH='Emigration and Immigration', MS="The process of leaving one's country to establish residence in a foreign country."),
  'D004642': NtO(MH='Emodin', MS='Purgative anthraquinone found in several plants, especially RHAMNUS PURSHIANA. It was formerly used as a laxative, but is now used mainly as a tool in toxicity studies.'),
  'D004643': NtO(MH='Emollients', MS='Oleagenous substances used topically to soothe, soften or protect skin or mucous membranes. They are used also as vehicles for other dermatologic agents.'),
  'D004644': NtO(MH='Emotions', MS='Those affective states which can be experienced and have arousing and motivational properties.'),
  'D004645': NtO(MH='Empathy', MS="An individual's objective and insightful awareness of the feelings and behavior of another person. It should be distinguished from sympathy, which is usually nonobjective and noncritical. It includes caring, which is the demonstration of an awareness of and a concern for the good of others. (From Bioethics Thesaurus, 1992)"),
  'D004646': NtO(MH='Emphysema', MS='A pathological accumulation of air in tissues or organs.'),
  'D004647': NtO(MH='Employee Discipline', MS='Regulations or conditions imposed on employees by management in order to correct or prevent behaviors which are counterproductive to the organization.'),
  'D004648': NtO(MH='Employee Grievances', MS='Formal procedures whereby the employee expresses any dissatisfaction or feeling of injustice regarding the work situation.'),
  'D004649': NtO(MH='Employee Incentive Plans', MS='Programs designed by management to motivate employees to work more efficiently with increased productivity, and greater employee satisfaction.'),
  'D004650': NtO(MH='Employee Performance Appraisal', MS='The assessment of the functioning of an employee in relation to work.'),
  'D004651': NtO(MH='Employment', MS='The state of being engaged in an activity or service for wages or salary.'),
  'D004652': NtO(MH='Empty Sella Syndrome', MS='A condition when the SELLA TURCICA is not filled with pituitary tissue. The pituitary gland is either compressed, atrophied, or removed. There are two types: (1) primary empty sella is due a defect in the sella diaphragm leading to arachnoid herniation into the sellar space; (2) secondary empty sella is associated with the removal or treatment of PITUITARY NEOPLASMS.'),
  'D004653': NtO(MH='Empyema', MS='Presence of pus in a hollow organ or body cavity.'),
  'D004654': NtO(MH='Empyema, Tuberculous', MS='Empyema due to MYCOBACTERIUM TUBERCULOSIS.'),
  'D004655': NtO(MH='Emulsions', MS='Colloids formed by the combination of two immiscible liquids such as oil and water. Lipid-in-water emulsions are usually liquid, like milk or lotion. Water-in-lipid emulsions tend to be creams. The formation of emulsions may be aided by amphiphatic molecules that surround one component of the system to form MICELLES.'),
  'D004656': NtO(MH='Enalapril', MS='An angiotensin-converting enzyme inhibitor that is used to treat HYPERTENSION and HEART FAILURE.'),
  'D004658': NtO(MH='Enamel Organ', MS='Epithelial cells surrounding the dental papilla and differentiated into three layers: the inner enamel epithelium, consisting of ameloblasts which eventually form the enamel, and the enamel pulp and external enamel epithelium, both of which atrophy and disappear before and upon eruption of the tooth, respectively.'),
  'D004659': NtO(MH='Enbucrilate', MS='A tissue adhesive that is applied as a monomer to moist tissue and polymerizes to form a bond. It is slowly biodegradable and used in all kinds of surgery, including dental.'),
  'D004660': NtO(MH='Encephalitis', MS='Inflammation of the BRAIN due to infection, autoimmune processes, toxins, and other conditions. Viral infections (see ENCEPHALITIS, VIRAL) are a relatively frequent cause of this condition.'),
  'D004663': NtO(MH='Encephalitis Virus, Eastern Equine', MS='A species of ALPHAVIRUS causing encephalomyelitis in Equidae and humans. The virus ranges along the Atlantic seaboard of the United States and Canada and as far south as the Caribbean, Mexico, and parts of Central and South America. Infections in horses show a mortality of up to 90 percent and in humans as high as 80 percent in epidemics.'),
  'D004664': NtO(MH='Encephalitis Virus, Japanese', MS='A species of FLAVIVIRUS, one of the Japanese encephalitis virus group (ENCEPHALITIS VIRUSES, JAPANESE), which is the etiological agent of Japanese encephalitis found in Asia, southeast Asia, and the Indian subcontinent.'),
  'D004665': NtO(MH='Encephalitis Virus, St. Louis', MS='A species of FLAVIVIRUS, one of the Japanese encephalitis virus group (ENCEPHALITIS VIRUSES, JAPANESE), which is the etiologic agent of ST. LOUIS ENCEPHALITIS in the United States, the Caribbean, and Central and South America.'),
  'D004666': NtO(MH='Encephalitis Virus, Venezuelan Equine', MS='A species of ALPHAVIRUS that is the etiologic agent of encephalomyelitis in humans and equines. It is seen most commonly in parts of Central and South America.'),
  'D004667': NtO(MH='Encephalitis Virus, Western Equine', MS='A species of ALPHAVIRUS that is the etiologic agent of encephalomyelitis in humans and equines in the United States, southern Canada, and parts of South America.'),
  'D004668': NtO(MH='Encephalitis Viruses', MS='A collection of single-stranded RNA viruses scattered across the Bunyaviridae, Flaviviridae, and Togaviridae families whose common property is the ability to induce encephalitic conditions in infected hosts.'),
  'D004669': NtO(MH='Encephalitis Viruses, Tick-Borne', MS='A subgroup of the genus FLAVIVIRUS that causes encephalitis and hemorrhagic fevers and is found in eastern and western Europe and the former Soviet Union. It is transmitted by TICKS and there is an associated milk-borne transmission from viremic cattle, goats, and sheep.'),
  'D004670': NtO(MH='Encephalitis, California', MS='A viral infection of the brain caused by serotypes of California encephalitis virus (ENCEPHALITIS VIRUS, CALIFORNIA) transmitted to humans by the mosquito AEDES triseriatus. The majority of cases are caused by the LA CROSSE VIRUS. This condition is endemic to the midwestern United States and primarily affects children between 5-10 years of age. Clinical manifestations include FEVER; VOMITING; HEADACHE; and abdominal pain followed by SEIZURES, altered mentation, and focal neurologic deficits. (From Joynt, Clinical Neurology, 1996, Ch26, p13)'),
  'D004671': NtO(MH='Encephalitis, Arbovirus', MS="Infections of the brain caused by arthropod-borne viruses (i.e., arboviruses) primarily from the families TOGAVIRIDAE; FLAVIVIRIDAE; BUNYAVIRIDAE; REOVIRIDAE; and RHABDOVIRIDAE. Life cycles of these viruses are characterized by ZOONOSES, with birds and lower mammals serving as intermediate hosts. The virus is transmitted to humans by the bite of mosquitoes (CULICIDAE) or TICKS. Clinical manifestations include fever, headache, alterations of mentation, focal neurologic deficits, and COMA. (From Clin Microbiol Rev 1994 Jan;7(1):89-116; Walton, Brain's Diseases of the Nervous System, 10th ed, p321)"),
  'D004672': NtO(MH='Encephalitis, Japanese', MS='A mosquito-borne encephalitis caused by the Japanese B encephalitis virus (ENCEPHALITIS VIRUS, JAPANESE) occurring throughout Eastern Asia and Australia. The majority of infections occur in children and are subclinical or have features limited to transient fever and gastrointestinal symptoms. Inflammation of the brain, spinal cord, and meninges may occur and lead to transient or permanent neurologic deficits (including a POLIOMYELITIS-like presentation); SEIZURES; COMA; and death. (From Adams et al., Principles of Neurology, 6th ed, p751; Lancet 1998 Apr 11;351(9109):1094-7)'),
  'D004673': NtO(MH='Encephalomyelitis, Acute Disseminated', MS='An acute or subacute inflammatory process of the CENTRAL NERVOUS SYSTEM characterized histologically by multiple foci of perivascular demyelination. Symptom onset usually occurs several days after an acute viral infection or immunization, but it may coincide with the onset of infection or rarely no antecedent event can be identified. Clinical manifestations include CONFUSION, somnolence, FEVER, nuchal rigidity, and involuntary movements. The illness may progress to COMA and eventually be fatal. (Adams et al., Principles of Neurology, 6th ed, p921)'),
  'D004674': NtO(MH='Encephalitis, St. Louis', MS='A viral encephalitis caused by the St. Louis encephalitis virus (ENCEPHALITIS VIRUS, ST. LOUIS), a FLAVIVIRUS. It is transmitted to humans and other vertebrates primarily by mosquitoes of the genus CULEX. The primary animal vectors are wild birds and the disorder is endemic to the midwestern and southeastern United States. Infections may be limited to an influenza-like illness or present as an ASEPTIC MENINGITIS or ENCEPHALITIS. Clinical manifestations of the encephalitic presentation may include SEIZURES, lethargy, MYOCLONUS, focal neurologic signs, COMA, and DEATH. (From Adams et al., Principles of Neurology, 6th ed, p750)'),
  'D004675': NtO(MH='Encephalitis, Tick-Borne', MS='Encephalitis caused by neurotropic viruses that are transmitted via the bite of TICKS. In Europe, the diseases are caused by ENCEPHALITIS VIRUSES, TICK-BORNE, which give rise to Russian spring-summer encephalitis, central European encephalitis, louping ill encephalitis, and related disorders. Powassan encephalitis occurs in North America and Russia and is caused by the Powassan virus. ASEPTIC MENINGITIS and rarely encephalitis may complicate COLORADO TICK FEVER which is endemic to mountainous regions of the western United States. (From Joynt, Clinical Neurology, 1996, Ch26, pp14-5)'),
  'D004676': NtO(MH='Myelin Basic Protein', MS='An abundant cytosolic protein that plays a critical role in the structure of multilamellar myelin. Myelin basic protein binds to the cytosolic sides of myelin cell membranes and causes a tight adhesion between opposing cell membranes.'),
  'D004677': NtO(MH='Encephalocele', MS='Brain tissue herniation through a congenital or acquired defect in the skull. The majority of congenital encephaloceles occur in the occipital or frontal regions. Clinical features include a protuberant mass that may be pulsatile. The quantity and location of protruding neural tissue determines the type and degree of neurologic deficit. Visual defects, psychomotor developmental delay, and persistent motor deficits frequently occur.'),
  'D004678': NtO(MH='Encephalomalacia', MS='Softening or loss of brain tissue following CEREBRAL INFARCTION; cerebral ischemia (see BRAIN ISCHEMIA), infection, CRANIOCEREBRAL TRAUMA, or other injury. The term is often used during gross pathologic inspection to describe blurred cortical margins and decreased consistency of brain tissue following infarction. Multicystic encephalomalacia refers to the formation of multiple cystic cavities of various sizes in the cerebral cortex of neonates and infants following injury, most notably perinatal hypoxia-ischemic events. (From Davis et al., Textbook of Neuropathology, 2nd ed, p665; J Neuropathol Exp Neurol, 1995 Mar;54(2):268-75)'),
  'D004679': NtO(MH='Encephalomyelitis', MS='A general term indicating inflammation of the BRAIN and SPINAL CORD, often used to indicate an infectious process, but also applicable to a variety of autoimmune and toxic-metabolic conditions. There is significant overlap regarding the usage of this term and ENCEPHALITIS in the literature.'),
  'D004680': NtO(MH='Maus Elberfeld virus', MS='A strain of ENCEPHALOMYOCARDITIS VIRUS, a species of CARDIOVIRUS, usually causing an inapparent intestinal infection in mice. A small number of mice may show signs of flaccid paralysis.'),
  'D004681': NtO(MH='Encephalomyelitis, Autoimmune, Experimental', MS="An experimental animal model for central nervous system demyelinating disease. Inoculation with a white matter emulsion combined with FREUND'S ADJUVANT, myelin basic protein, or purified central myelin triggers a T cell-mediated immune response directed towards central myelin. The pathologic features are similar to MULTIPLE SCLEROSIS, including perivascular and periventricular foci of inflammation and demyelination. Subpial demyelination underlying meningeal infiltrations also occurs, which is also a feature of ENCEPHALOMYELITIS, ACUTE DISSEMINATED. Passive immunization with T-cells from an afflicted animal to a normal animal also induces this condition. (From Immunol Res 1998;17(1-2):217-27; Raine CS, Textbook of Neuropathology, 2nd ed, p604-5)"),
  'D004682': NtO(MH='Encephalomyelitis, Enzootic Porcine', MS='A picornavirus infection producing symptoms similar to poliomyelitis in pigs.'),
  'D004683': NtO(MH='Encephalomyelitis, Equine', MS='A group of ALPHAVIRUS INFECTIONS which affect horses and man, transmitted via the bites of mosquitoes. Disorders in this category are endemic to regions of South America and North America. In humans, clinical manifestations vary with the type of infection, and range from a mild influenza-like syndrome to a fulminant encephalitis. (From Joynt, Clinical Neurology, 1996, Ch26, pp8-10)'),
  'D004684': NtO(MH='Leukoencephalitis, Acute Hemorrhagic', MS='A fulminant and often fatal demyelinating disease of the brain which primarily affects young adults and children. Clinical features include the rapid onset of weakness, SEIZURES, and COMA. It may follow a viral illness or MYCOPLASMA PNEUMONIAE infections but in most instances there is no precipitating event. Pathologic examination reveals marked perivascular demyelination and necrosis of white matter with microhemorrhages. (Adams et al., Principles of Neurology, 6th ed, pp924-5)'),
  'D004685': NtO(MH='Encephalomyelitis, Venezuelan Equine', MS='A form of arboviral encephalitis endemic to Central America and the northern latitudes of South America. The causative organism (ENCEPHALITIS VIRUS, VENEZUELAN EQUINE) is transmitted to humans and horses via the bite of several mosquito species. Human viral infection may be asymptomatic or remain restricted to a mild influenza-like illness. Encephalitis, usually not severe, occurs in a small percentage of cases and may rarely feature SEIZURES and COMA. (From Joynt, Clinical Neurology, 1996, Ch26, pp9-10)'),
  'D004686': NtO(MH='Encephalomyocarditis virus', MS='The type species of CARDIOVIRUS causing encephalomyelitis and myocarditis in rodents, pigs, and monkeys. Infection in man has been reported with CNS involvement but without myocarditis.'),
  'D004687': NtO(MH='Enchondromatosis', MS='Benign growths of cartilage in the metaphyses of several bones.'),
  'D004688': NtO(MH='Encopresis', MS='Incontinence of feces not due to organic defect or illness.'),
  'D004690': NtO(MH='Encyclopedias as Topic', MS='Discussion of works containing information articles on subjects in various fields of knowledge, usually arranged in alphabetical order, or a similar work limited to a special field or subject. (From The ALA Glossary of Library and Information Science, 1983)'),
  'D004691': NtO(MH='Endarterectomy', MS='Surgical excision, performed under general anesthesia, of the atheromatous tunica intima of an artery. When reconstruction of an artery is performed as an endovascular procedure through a catheter, it is called ATHERECTOMY.'),
  'D004692': NtO(MH='Endarteritis', MS='Inflammation of the inner endothelial lining (TUNICA INTIMA) of an artery.'),
  'D004693': NtO(MH='Glucan Endo-1,3-beta-D-Glucosidase', MS='An endocellulase with specificity for the hydrolysis of 1,3-beta-D-glucosidic linkages in 1,3-beta-D-glucans including laminarin, paramylon, and pachyman.'),
  'D004694': NtO(MH='Endocardial Cushion Defects', MS='A spectrum of septal defects involving the ATRIAL SEPTUM; VENTRICULAR SEPTUM; and the atrioventricular valves (TRICUSPID VALVE; BICUSPID VALVE). These defects are due to incomplete growth and fusion of the ENDOCARDIAL CUSHIONS which are important in the formation of two atrioventricular canals, site of future atrioventricular valves.'),
  'D004695': NtO(MH='Endocardial Fibroelastosis', MS='A condition characterized by the thickening of ENDOCARDIUM due to proliferation of fibrous and elastic tissue, usually in the left ventricle leading to impaired cardiac function (CARDIOMYOPATHY, RESTRICTIVE). It is most commonly seen in young children and rarely in adults. It is often associated with congenital heart anomalies (HEART DEFECTS CONGENITAL;) INFECTION; or gene mutation. Defects in the tafazzin protein, encoded by TAZ gene, result in a form of autosomal dominant familial endocardial fibroelastosis.'),
  'D004696': NtO(MH='Endocarditis', MS='Inflammation of the inner lining of the heart (ENDOCARDIUM), the continuous membrane lining the four chambers and HEART VALVES. It is often caused by microorganisms including bacteria, viruses, fungi, and rickettsiae. Left untreated, endocarditis can damage heart valves and become life-threatening.'),
  'D004697': NtO(MH='Endocarditis, Bacterial', MS='Inflammation of the ENDOCARDIUM caused by BACTERIA that entered the bloodstream. The strains of bacteria vary with predisposing factors, such as CONGENITAL HEART DEFECTS; HEART VALVE DISEASES; HEART VALVE PROSTHESIS IMPLANTATION; or intravenous drug use.'),
  'D004698': NtO(MH='Endocarditis, Subacute Bacterial', MS='ENDOCARDIUM infection that is usually caused by STREPTOCOCCUS. Subacute infective endocarditis evolves over weeks and months with modest toxicity and rare metastatic infection.'),
  'D004699': NtO(MH='Endocardium', MS='The innermost layer of the heart, comprised of endothelial cells.'),
  'D004700': NtO(MH='Endocrine System Diseases', MS='Pathological processes of the ENDOCRINE GLANDS, and diseases resulting from abnormal level of available HORMONES.'),
  'D004701': NtO(MH='Endocrine Gland Neoplasms', MS='Tumors or cancer of the ENDOCRINE GLANDS.'),
  'D004702': NtO(MH='Endocrine Glands', MS='Ductless glands that secrete HORMONES directly into the BLOOD CIRCULATION. These hormones influence the METABOLISM and other functions of cells in the body.'),
  'D004703': NtO(MH='Endocrine System', MS='The system of glands that release their secretions (hormones) directly into the circulatory system. In addition to the ENDOCRINE GLANDS, included are the CHROMAFFIN SYSTEM and the NEUROSECRETORY SYSTEMS.'),
  'D004704': NtO(MH='Endocrinology', MS='A subspecialty of internal medicine concerned with the metabolism, physiology, and disorders of the ENDOCRINE SYSTEM.'),
  'D004705': NtO(MH='Endocytosis', MS='Cellular uptake of extracellular materials within membrane-limited vacuoles or microvesicles. ENDOSOMES play a central role in endocytosis.'),
  'D004706': NtO(MH='Endodeoxyribonucleases', MS='A group of enzymes catalyzing the endonucleolytic cleavage of DNA. They include members of EC 3.1.21.-, EC 3.1.22.-, EC 3.1.23.- (DNA RESTRICTION ENZYMES), EC 3.1.24.- (DNA RESTRICTION ENZYMES), and EC 3.1.25.-.'),
  'D004707': NtO(MH='Endoderm', MS='The inner of the three germ layers of an embryo.'),
  'D004708': NtO(MH='Endodontics', MS='A dental specialty concerned with the maintenance of the dental pulp in a state of health and the treatment of the pulp cavity (pulp chamber and pulp canal).'),
  'D004709': NtO(MH='Endolimax', MS='A genus of ameboid protozoa found in the intestines of vertebrates and invertebrates.'),
  'D004710': NtO(MH='Endolymph', MS='The lymph fluid found in the membranous labyrinth of the ear. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D004711': NtO(MH='Endolymphatic Duct', MS='The part of the membranous labyrinth that traverses the bony vestibular aqueduct and emerges through the bone of posterior cranial fossa (CRANIAL FOSSA, POSTERIOR) where it expands into a blind pouch called the endolymphatic sac.'),
  'D004712': NtO(MH='Endolymphatic Sac', MS='The blind pouch at the end of the endolymphatic duct. It is a storage reservoir for excess ENDOLYMPH, formed by the blood vessels in the membranous labyrinth.'),
  'D004713': NtO(MH='Endolymphatic Shunt', MS='Surgical fistulization of the membranous labyrinth of the inner ear with mastoid, subarachnoid or cochlear shunt. This procedure is used in the treatment of MENIERE DISEASE.'),
  'D004714': NtO(MH='Endometrial Hyperplasia', MS='Benign proliferation of the ENDOMETRIUM in the UTERUS. Endometrial hyperplasia is classified by its cytology and glandular tissue. There are simple, complex (adenomatous without atypia), and atypical hyperplasia representing also the ascending risk of becoming malignant.'),
  'D004715': NtO(MH='Endometriosis', MS='A condition in which functional endometrial tissue is present outside the UTERUS. It is often confined to the PELVIS involving the OVARY, the ligaments, cul-de-sac, and the uterovesical peritoneum.'),
  'D004716': NtO(MH='Endometritis', MS='Inflammation of the ENDOMETRIUM, usually caused by intrauterine infections. Endometritis is the most common cause of postpartum fever.'),
  'D004717': NtO(MH='Endometrium', MS='The mucous membrane lining of the uterine cavity that is hormonally responsive during the MENSTRUAL CYCLE and PREGNANCY. The endometrium undergoes cyclic changes that characterize MENSTRUATION. After successful FERTILIZATION, it serves to sustain the developing embryo.'),
  'D004718': NtO(MH='Saccharomycetales', MS='An order of fungi in the phylum Ascomycota that multiply by budding. They include the telomorphic ascomycetous yeasts which are found in a very wide range of habitats.'),
  'D004719': NtO(MH='Endomyocardial Fibrosis', MS='A condition characterized by the thickening of the ventricular ENDOCARDIUM and subendocardium (MYOCARDIUM), seen mostly in children and young adults in the TROPICAL CLIMATE. The fibrous tissue extends from the apex toward and often involves the HEART VALVES causing restrictive blood flow into the respective ventricles (CARDIOMYOPATHY, RESTRICTIVE).'),
  'D004720': NtO(MH='Endonucleases', MS='Enzymes that catalyze the hydrolysis of the internal bonds and thereby the formation of polynucleotides or oligonucleotides from ribo- or deoxyribonucleotide chains. EC 3.1.-.'),
  'D004721': NtO(MH='Endoplasmic Reticulum', MS='A system of cisternae in the CYTOPLASM of many cells. In places the endoplasmic reticulum is continuous with the plasma membrane (CELL MEMBRANE) or outer membrane of the nuclear envelope. If the outer surfaces of the endoplasmic reticulum membranes are coated with ribosomes, the endoplasmic reticulum is said to be rough-surfaced (ENDOPLASMIC RETICULUM, ROUGH); otherwise it is said to be smooth-surfaced (ENDOPLASMIC RETICULUM, SMOOTH). (King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D004722': NtO(MH='Endoribonucleases', MS='A family of enzymes that catalyze the endonucleolytic cleavage of RNA. It includes EC 3.1.26.-, EC 3.1.27.-, EC 3.1.30.-, and EC 3.1.31.-.'),
  'D004723': NtO(MH='Endorphins', MS='One of the three major groups of endogenous opioid peptides. They are large peptides derived from the PRO-OPIOMELANOCORTIN precursor. The known members of this group are alpha-, beta-, and gamma-endorphin. The term endorphin is also sometimes used to refer to all opioid peptides, but the narrower sense is used here; OPIOID PEPTIDES is used for the broader group.'),
  'D004724': NtO(MH='Endoscopy', MS='Procedures of applying ENDOSCOPES for disease diagnosis and treatment. Endoscopy involves passing an optical instrument through a small incision in the skin i.e., percutaneous; or through a natural orifice and along natural body pathways such as the digestive tract; and/or through an incision in the wall of a tubular structure or organ, i.e. transluminal, to examine or perform surgery on the interior parts of the body.'),
  'D004725': NtO(MH='Endospore-Forming Bacteria', MS='A group of rods or cocci whose taxonomic affinities are uncertain. They form endospores, thick-walled bodies formed within the vegetative cells of certain bacteria, able to withstand adverse environmental conditions for prolonged periods.'),
  'D004726': NtO(MH='Endosulfan', MS='A polychlorinated compound used for controlling a variety of insects. It is practically water-insoluble, but readily adheres to clay particles and persists in soil and water for several years. Its mode of action involves repetitive nerve-discharges positively correlated to increase in temperature. This compound is extremely toxic to most fish. (From Comp Biochem Physiol (C) 1993 Jul;105(3):347-61)'),
  'D004727': NtO(MH='Endothelium', MS='A layer of epithelium that lines the heart, blood vessels (ENDOTHELIUM, VASCULAR), lymph vessels (ENDOTHELIUM, LYMPHATIC), and the serous cavities of the body.'),
  'D004728': NtO(MH='Endothelium, Corneal', MS='Single layer of large flattened cells covering the surface of the cornea.'),
  'D004729': NtO(MH='Endothelium, Lymphatic', MS='Unbroken cellular lining (intima) of the lymph vessels (e.g., the high endothelial lymphatic venules). It is more permeable than vascular endothelium, lacking selective absorption and functioning mainly to remove plasma proteins that have filtered through the capillaries into the tissue spaces.'),
  'D004730': NtO(MH='Endothelium, Vascular', MS='Single pavement layer of cells which line the luminal surface of the entire vascular system and regulate the transport of macromolecules and blood components.'),
  'D004731': NtO(MH='Endotoxins', MS='Toxins closely associated with the living cytoplasm or cell wall of certain microorganisms, which do not readily diffuse into the culture medium, but are released upon lysis of the cells.'),
  'D004732': NtO(MH='Endrin', MS='An organochlorine compound that was formerly used as an insecticide. Its manufacture and use has been discontinued in the United States. (From Merck Index, 11th ed)'),
  'D004733': NtO(MH='Enema', MS='Insertion of a solution or compound through the RECTUM with the purpose of cleansing the COLON or for diagnostic procedures.'),
  'D004734': NtO(MH='Energy Metabolism', MS='The chemical reactions involved in the production and utilization of various forms of energy in cells.'),
  'D004735': NtO(MH='Energy Transfer', MS='The transfer of energy of a given form among different scales of motion. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed). It includes the transfer of kinetic energy and the transfer of chemical energy. The transfer of chemical energy from one molecule to another depends on proximity of molecules so it is often used as in techniques to measure distance such as the use of FORSTER RESONANCE ENERGY TRANSFER.'),
  'D004736': NtO(MH='Energy-Generating Resources', MS='Materials or phenomena which can provide energy directly or via conversion.'),
  'D004737': NtO(MH='Enflurane', MS='An extremely stable inhalation anesthetic that allows rapid adjustments of anesthesia depth with little change in pulse or respiratory rate.'),
  'D004738': NtO(MH='Engineering', MS='The practical application of physical, mechanical, and mathematical principles. (Stedman, 25th ed)'),
  'D004739': NtO(MH='England', MS='A part of Great Britain within the United Kingdom.'),
  'D004740': NtO(MH='English Abstract', MS='Identifier for English language abstracts provided with non-English language works.'),
  'D004741': NtO(MH='Engraving and Engravings', MS='Creating marks on a hard surface by incising with a sharp tool. Resulting images are created for their own sake or for printing processes that involve this technique. (Getty ATT).'),
  'D004742': NtO(MH='Enhancer Elements, Genetic', MS='Cis-acting DNA sequences which can increase transcription of genes. Enhancers can usually function in either orientation and at various distances from a promoter.'),
  'D004743': NtO(MH='Enkephalin, Leucine', MS='One of the endogenous pentapeptides with morphine-like activity. It differs from MET-ENKEPHALIN in the LEUCINE at position 5. Its first four amino acid sequence is identical to the tetrapeptide sequence at the N-terminal of BETA-ENDORPHIN.'),
  'D004744': NtO(MH='Enkephalin, Methionine', MS='One of the endogenous pentapeptides with morphine-like activity. It differs from LEU-ENKEPHALIN by the amino acid METHIONINE in position 5. Its first four amino acid sequence is identical to the tetrapeptide sequence at the N-terminal of BETA-ENDORPHIN.'),
  'D004745': NtO(MH='Enkephalins', MS='One of the three major families of endogenous opioid peptides. The enkephalins are pentapeptides that are widespread in the central and peripheral nervous systems and in the adrenal medulla.'),
  'D004746': NtO(MH='Enoyl-CoA Hydratase', MS='An enzyme that catalyzes reversibly the hydration of unsaturated fatty acyl-CoA to yield beta-hydroxyacyl-CoA. It plays a role in the oxidation of fatty acids and in mitochondrial fatty acid synthesis, has broad specificity, and is most active with crotonyl-CoA. EC 4.2.1.17.'),
  'D004747': NtO(MH='Entamoeba', MS='A genus of ameboid protozoa characterized by the presence of beaded chromatin on the inner surface of the nuclear membrane. Its organisms are parasitic in invertebrates and vertebrates, including humans.'),
  'D004748': NtO(MH='Entamoeba histolytica', MS='A species of parasitic protozoa causing ENTAMOEBIASIS and amebic dysentery (DYSENTERY, AMEBIC). Characteristics include a single nucleus containing a small central karyosome and peripheral chromatin that is finely and regularly beaded.'),
  'D004749': NtO(MH='Entamoebiasis', MS='Infection with amoebae of the genus ENTAMOEBA. Infection with E. histolytica causes DYSENTERY, AMEBIC and LIVER ABSCESS, AMEBIC.'),
  'D004750': NtO(MH='Enteral Nutrition', MS='Nutritional support given via the alimentary canal or any route connected to the gastrointestinal system (i.e., the enteral route). This includes oral feeding, sip feeding, and tube feeding using nasogastric, gastrostomy, and jejunostomy tubes.'),
  'D004751': NtO(MH='Enteritis', MS='Inflammation of any segment of the SMALL INTESTINE.'),
  'D004752': NtO(MH='Coronavirus, Turkey', MS='A species of CORONAVIRUS causing enteritis in turkeys and pullets.'),
  'D004753': NtO(MH='Enteritis, Transmissible, of Turkeys', MS='An acute, highly contagious virus disease of turkeys characterized by chilling, anorexia, decreased water intake, diarrhea, dehydration and weight loss. The infectious agent is a CORONAVIRUS.'),
  'D004754': NtO(MH='Enterobacter', MS='Gram-negative gas-producing rods found in feces of humans and other animals, sewage, soil, water, and dairy products.'),
  'D004755': NtO(MH='Enterobacteriaceae', MS='A family of gram-negative, facultatively anaerobic, rod-shaped bacteria that do not form endospores. Its organisms are distributed worldwide with some being saprophytes and others being plant and animal parasites. Many species are of considerable economic importance due to their pathogenic effects on agriculture and livestock.'),
  'D004756': NtO(MH='Enterobacteriaceae Infections', MS='Infections with bacteria of the family ENTEROBACTERIACEAE.'),
  'D004757': NtO(MH='Enterobius', MS='A genus of intestinal nematode worms which includes the pinworm or threadworm Enterobius vermicularis.'),
  'D004758': NtO(MH='Enterobactin', MS='An iron-binding cyclic trimer of 2,3-dihydroxy-N-benzoyl-L-serine. It is produced by E COLI and other enteric bacteria.'),
  'D004759': NtO(MH='Enterochromaffin Cells', MS='A subtype of enteroendocrine cells found in the gastrointestinal MUCOSA, particularly in the glands of PYLORIC ANTRUM; DUODENUM; and ILEUM. These cells secrete mainly SEROTONIN and some neuropeptides. Their secretory granules stain readily with silver (argentaffin stain).'),
  'D004760': NtO(MH='Enterocolitis', MS='Inflammation of the MUCOSA of both the SMALL INTESTINE and the LARGE INTESTINE. Etiology includes ISCHEMIA, infections, allergic, and immune responses.'),
  'D004761': NtO(MH='Enterocolitis, Pseudomembranous', MS='An acute inflammation of the INTESTINAL MUCOSA that is characterized by the presence of pseudomembranes or plaques in the SMALL INTESTINE (pseudomembranous enteritis) and the LARGE INTESTINE (pseudomembranous colitis). It is commonly associated with antibiotic therapy and CLOSTRIDIUM DIFFICILE colonization.'),
  'D004763': NtO(MH='Glucagon-Like Peptides', MS='Peptides derived from proglucagon which is also the precursor of pancreatic GLUCAGON. Despite expression of proglucagon in multiple tissues, the major production site of glucagon-like peptides (GLPs) is the INTESTINAL L CELLS. GLPs include glucagon-like peptide 1, glucagon-like peptide 2, and the various truncated forms.'),
  'D004764': NtO(MH='Enterohepatic Circulation', MS='Recycling through liver by excretion in bile, reabsorption from intestines (INTESTINAL REABSORPTION) into portal circulation, passage back into liver, and re-excretion in bile.'),
  'D004765': NtO(MH='Enteropeptidase', MS='A specialized proteolytic enzyme secreted by intestinal cells. It converts TRYPSINOGEN into its active form TRYPSIN by removing the N-terminal peptide. EC 3.4.21.9.'),
  'D004766': NtO(MH='Enterostomy', MS='Creation of an artificial external opening or fistula in the intestines.'),
  'D004767': NtO(MH='Enterotoxemia', MS='Disease caused by the liberation of exotoxins of CLOSTRIDIUM PERFRINGENS in the intestines of sheep, goats, cattle, foals, and piglets. Type B enterotoxemia in lambs is lamb dysentery; type C enterotoxemia in mature sheep produces "struck", and in calves, lambs and piglets it produces hemorrhagic enterotoxemia; type D enterotoxemia in sheep and goats is pulpy-kidney disease or overeating disease.'),
  'D004768': NtO(MH='Enterotoxins', MS='Substances that are toxic to the intestinal tract causing vomiting, diarrhea, etc.; most common enterotoxins are produced by bacteria.'),
  'D004769': NtO(MH='Enterovirus Infections', MS='Diseases caused by ENTEROVIRUS.'),
  'D004770': NtO(MH='Enterovirus', MS='A genus of the family PICORNAVIRIDAE whose members preferentially inhabit the intestinal tract of a variety of hosts. The genus contains many species. Newly described members of human enteroviruses are assigned continuous numbers with the species designated "human enterovirus".'),
  'D004771': NtO(MH='Enteroviruses, Porcine', MS='Species of ENTEROVIRUS causing mild to severe neurological diseases among pigs especially in Eastern Europe. Mild strains are also present in Canada, U.S., and Australia. Specific species include Porcine enterovirus A and Porcine enterovirus B.'),
  'D004772': NtO(MH='Entomology', MS='A discipline or occupation concerned with the study of INSECTS, including the biology and the control of insects.'),
  'D004773': NtO(MH='Entomophthora', MS='A genus of fungi in the family Entomophthoraceae, order Entomorphthorales. They are primarily parasites of insects and spiders, but have been found to cause mycotic infections of the nose in man and horses.'),
  'D004774': NtO(MH='Entropion', MS='The turning inward (inversion) of the edge of the eyelid, with the tarsal cartilage turned inward toward the eyeball. (Dorland, 27th ed)'),
  'D004775': NtO(MH='Enuresis', MS='Involuntary discharge of URINE after expected age of completed development of urinary control. This can happen during the daytime (DIURNAL ENURESIS) while one is awake or during sleep (NOCTURNAL ENURESIS). Enuresis can be in children or in adults (as persistent primary enuresis and secondary adult-onset enuresis).'),
  'D004776': NtO(MH='Enviomycin', MS='Cyclic basic peptide related to VIOMYCIN. It is isolated from an induced mutant of Streptomyces griseoverticillatus var. tuberacticus and acts as an antitubercular agent with less ototoxicity than tuberactinomycin.'),
  'D004777': NtO(MH='Environment', MS='The external elements and conditions which surround, influence, and affect the life and development of an organism or population.'),
  'D004778': NtO(MH='Environment and Public Health', MS='Natural and man-made environments and their impact on public health.'),
  'D004779': NtO(MH='Environment Design', MS='The structuring of the environment to permit or promote specific patterns of behavior.'),
  'D004780': NtO(MH='Environment, Controlled', MS="A state in which the environs of hospitals, laboratories, domestic and animal housing, work places, spacecraft, and other surroundings are under technological control with regard to air conditioning, heating, lighting, humidity, ventilation, and other ambient features. The concept includes control of atmospheric composition. (From Jane's Aerospace Dictionary, 3d ed)"),
  'D004781': NtO(MH='Environmental Exposure', MS='The exposure to potentially harmful chemical, physical, or biological agents in the environment or to environmental factors that may include ionizing radiation, pathogenic organisms, or toxic chemicals.'),
  'D004782': NtO(MH='Environmental Health', MS='The science of controlling or modifying those conditions, influences, or forces surrounding man which relate to promoting, establishing, and maintaining health.'),
  'D004783': NtO(MH='Environmental Microbiology', MS='The study of microorganisms living in a variety of environments (air, soil, water, etc.) and their pathogenic relationship to other organisms including man.'),
  'D004784': NtO(MH='Environmental Monitoring', MS='The monitoring of the level of toxins, chemical pollutants, microbial contaminants, or other harmful substances in the environment (soil, air, and water), workplace, or in the bodies of people and animals present in that environment.'),
  'D004785': NtO(MH='Environmental Pollutants', MS='Substances or energies, for example heat or light, which when introduced into the air, water, or land threaten life or health of individuals or ECOSYSTEMS.'),
  'D004786': NtO(MH='Toxic Actions', MS='A broad category of chemical actions with harmful or deadly effect on living organisms.'),
  'D004787': NtO(MH='Environmental Pollution', MS='Contamination of the air, bodies of water, or land with substances that are harmful to human health and the environment.'),
  'D004789': NtO(MH='Enzyme Activation', MS='Conversion of an inactive form of an enzyme to one possessing metabolic activity. It includes 1, activation by ions (activators); 2, activation by cofactors (coenzymes); and 3, conversion of an enzyme precursor (proenzyme or zymogen) to an active enzyme.'),
  'D004790': NtO(MH='Enzyme Induction', MS='An increase in the rate of synthesis of an enzyme due to the presence of an inducer which acts to derepress the gene responsible for enzyme synthesis.'),
  'D004791': NtO(MH='Enzyme Inhibitors', MS='Compounds or agents that combine with an enzyme in such a manner as to prevent the normal substrate-enzyme combination and the catalytic reaction.'),
  'D004792': NtO(MH='Enzyme Precursors', MS='Physiologically inactive substances that can be converted to active enzymes.'),
  'D004793': NtO(MH='Enzyme Reactivators', MS='Compounds which restore enzymatic activity by removing an inhibitory group bound to the reactive site of the enzyme.'),
  'D004794': NtO(MH='Enzyme Repression', MS='The interference in synthesis of an enzyme due to the elevated level of an effector substance, usually a metabolite, whose presence would cause depression of the gene responsible for enzyme synthesis.'),
  'D004795': NtO(MH='Enzyme Stability', MS='The extent to which an enzyme retains its structural conformation or its activity when subjected to storage, isolation, and purification or various other physical or chemical manipulations, including proteolytic enzymes and heat.'),
  'D004796': NtO(MH='Clinical Enzyme Tests', MS='Analyses for a specific enzyme activity, or of the level of a specific enzyme that is used to assess health and disease risk, for early detection of disease or disease prediction, diagnosis, and change in disease status.'),
  'D004797': NtO(MH='Enzyme-Linked Immunosorbent Assay', MS='An immunoassay utilizing an antibody labeled with an enzyme marker such as horseradish peroxidase. While either the enzyme or the antibody is bound to an immunosorbent substrate, they both retain their biologic activity; the change in enzyme activity as a result of the enzyme-antibody-antigen reaction is proportional to the concentration of the antigen and can be measured spectrophotometrically or with the naked eye. Many variations of the method have been developed.'),
  'D004798': NtO(MH='Enzymes', MS='Biological molecules that possess catalytic activity. They may occur naturally or be synthetically created. Enzymes are usually proteins, however CATALYTIC RNA and CATALYTIC DNA molecules have also been identified.'),
  'D004800': NtO(MH='Enzymes, Immobilized', MS='Enzymes which are immobilized on or in a variety of water-soluble or water-insoluble matrices with little or no loss of their catalytic activity. Since they can be reused continuously, immobilized enzymes have found wide application in the industrial, medical and research fields.'),
  'D004801': NtO(MH='Eosine Yellowish-(YS)', MS='A versatile red dye used in cosmetics, pharmaceuticals, textiles, etc., and as tissue stain, vital stain, and counterstain with HEMATOXYLIN. It is also used in special culture media.'),
  'D004802': NtO(MH='Eosinophilia', MS='Abnormal increase of EOSINOPHILS in the blood, tissues or organs.'),
  'D004803': NtO(MH='Eosinophilic Granuloma', MS='The most benign and common form of Langerhans-cell histiocytosis which involves localized nodular lesions predominantly of the bones but also of the gastric mucosa, small intestine, lungs, or skin, with infiltration by EOSINOPHILS.'),
  'D004804': NtO(MH='Eosinophils', MS='Granular leukocytes with a nucleus that usually has two lobes connected by a slender thread of chromatin, and cytoplasm containing coarse, round granules that are uniform in size and stainable by eosin.'),
  'D004805': NtO(MH='Ependyma', MS='A thin membrane that lines the CEREBRAL VENTRICLES and the central canal of the SPINAL CORD.'),
  'D004806': NtO(MH='Ependymoma', MS='Glioma derived from EPENDYMOGLIAL CELLS that tend to present as malignant intracranial tumors in children and as benign intraspinal neoplasms in adults. It may arise from any level of the ventricular system or central canal of the spinal cord. Intracranial ependymomas most frequently originate in the FOURTH VENTRICLE and histologically are densely cellular tumors which may contain ependymal tubules and perivascular pseudorosettes. Spinal ependymomas are usually benign papillary or myxopapillary tumors. (From DeVita et al., Principles and Practice of Oncology, 5th ed, p2018; Escourolle et al., Manual of Basic Neuropathology, 2nd ed, pp28-9)'),
  'D004809': NtO(MH='Ephedrine', MS='A phenethylamine found in EPHEDRA SINICA. PSEUDOEPHEDRINE is an isomer. It is an alpha- and beta-adrenergic agonist that may also enhance release of norepinephrine. It has been used for asthma, heart failure, rhinitis, and urinary incontinence, and for its central nervous system stimulatory effects in the treatment of narcolepsy and depression. It has become less extensively used with the advent of more selective agonists.'),
  'D004810': NtO(MH='Ephemeral Fever', MS='An Ephemerovirus infection of cattle caused by bovine ephemeral fever virus (EPHEMERAL FEVER VIRUS, BOVINE). It is characterized by respiratory symptoms, increased oropharyngeal secretions and lacrimation, joint pains, tremor, and stiffness.'),
  'D004811': NtO(MH='Epichlorohydrin', MS='A chlorinated epoxy compound used as an industrial solvent. It is a strong skin irritant and carcinogen.'),
  'D004812': NtO(MH='Epidemiologic Methods', MS='Research techniques that focus on study designs and data gathering methods in human and animal populations.'),
  'D004813': NtO(MH='Epidemiology', MS='Field of medicine concerned with the determination of causes, incidence, and characteristic behavior of disease outbreaks affecting human populations. It includes the interrelationships of host, agent, and environment as related to the distribution and control of disease.'),
  'D004814': NtO(MH='Epidermal Cyst', MS='Intradermal or subcutaneous saclike structure, the wall of which is stratified epithelium containing keratohyalin granules.'),
  'D004815': NtO(MH='Epidermal Growth Factor', MS='A 6-kDa polypeptide growth factor initially discovered in mouse submaxillary glands. Human epidermal growth factor was originally isolated from urine based on its ability to inhibit gastric secretion and called urogastrone. Epidermal growth factor exerts a wide variety of biological effects including the promotion of proliferation and differentiation of mesenchymal and EPITHELIAL CELLS. It is synthesized as a transmembrane protein which can be cleaved to release a soluble active form.'),
  'D004817': NtO(MH='Epidermis', MS='The external, nonvascular layer of the skin. It is made up, from within outward, of five layers of EPITHELIUM: (1) basal layer (stratum basale epidermidis); (2) spinous layer (stratum spinosum epidermidis); (3) granular layer (stratum granulosum epidermidis); (4) clear layer (stratum lucidum epidermidis); and (5) horny layer (stratum corneum epidermidis).'),
  'D004818': NtO(MH='Epidermitis, Exudative, of Swine', MS='An acute generalized dermatitis of pigs which occurs from 5 to 35 days of age, characterized by sudden onset, with morbidity of 10 to 90% and mortality of 5 to 90%. The lesions are caused by Staphylococcus hyos but the bacterial agent is unable to penetrate the intact skin. Abrasions on the feet and legs or lacerations on the body frequently precede infection. In acute cases, a vesicular-type virus may be the predisposing factor. The causative organism is inhibited by most antibiotics. (Merck Veterinary Manual, 5th ed)'),
  'D004819': NtO(MH='Epidermodysplasia Verruciformis', MS='An autosomal recessive trait with impaired cell-mediated immunity. About 15 human papillomaviruses are implicated in associated infection, four of which lead to skin neoplasms. The disease begins in childhood with red papules and later spreads over the body as gray or yellow scales.'),
  'D004820': NtO(MH='Epidermolysis Bullosa', MS='Group of genetically determined disorders characterized by the blistering of skin and mucosae. There are four major forms: acquired, simple, junctional, and dystrophic. Each of the latter three has several varieties.'),
  'D004821': NtO(MH='Epidermophyton', MS='A fungal genus which grows in the epidermis and is the cause of TINEA.'),
  'D004822': NtO(MH='Epididymis', MS='The convoluted cordlike structure attached to the posterior of the TESTIS. Epididymis consists of the head (caput), the body (corpus), and the tail (cauda). A network of ducts leaving the testis joins into a common epididymal tubule proper which provides the transport, storage, and maturation of SPERMATOZOA.'),
  'D004823': NtO(MH='Epididymitis', MS='Inflammation of the EPIDIDYMIS. Its clinical features include enlarged epididymis, a swollen SCROTUM; PAIN; PYURIA; and FEVER. It is usually related to infections in the URINARY TRACT, which likely spread to the EPIDIDYMIS through either the VAS DEFERENS or the lymphatics of the SPERMATIC CORD.'),
  'D004824': NtO(MH='Epidural Space', MS='Space between the dura mater and the walls of the vertebral canal.'),
  'D004825': NtO(MH='Epiglottis', MS='A thin leaf-shaped cartilage that is covered with LARYNGEAL MUCOSA and situated posterior to the root of the tongue and HYOID BONE. During swallowing, the epiglottis folds back over the larynx inlet thus prevents foods from entering the airway.'),
  'D004826': NtO(MH='Epiglottitis', MS='Inflammation of the EPIGLOTTIS.'),
  'D004827': NtO(MH='Epilepsy', MS='A disorder characterized by recurrent episodes of paroxysmal brain dysfunction due to a sudden, disorderly, and excessive neuronal discharge. Epilepsy classification systems are generally based upon: (1) clinical features of the seizure episodes (e.g., motor seizure), (2) etiology (e.g., post-traumatic), (3) anatomic site of seizure origin (e.g., frontal lobe seizure), (4) tendency to spread to other structures in the brain, and (5) temporal patterns (e.g., nocturnal epilepsy). (From Adams et al., Principles of Neurology, 6th ed, p313)'),
  'D004828': NtO(MH='Epilepsies, Partial', MS='Conditions characterized by recurrent paroxysmal neuronal discharges which arise from a focal region of the brain. Partial seizures are divided into simple and complex, depending on whether consciousness is unaltered (simple partial seizure) or disturbed (complex partial seizure). Both types may feature a wide variety of motor, sensory, and autonomic symptoms. Partial seizures may be classified by associated clinical features or anatomic location of the seizure focus. A secondary generalized seizure refers to a partial seizure that spreads to involve the brain diffusely. (From Adams et al., Principles of Neurology, 6th ed, pp317)'),
  'D004829': NtO(MH='Epilepsy, Generalized', MS='Recurrent conditions characterized by epileptic seizures which arise diffusely and simultaneously from both hemispheres of the brain. Classification is generally based upon motor manifestations of the seizure (e.g., convulsive, nonconvulsive, akinetic, atonic, etc.) or etiology (e.g., idiopathic, cryptogenic, and symptomatic). (From Mayo Clin Proc, 1996 Apr;71(4):405-14)'),
  'D004830': NtO(MH='Epilepsy, Tonic-Clonic', MS='A generalized seizure disorder characterized by recurrent major motor seizures. The initial brief tonic phase is marked by trunk flexion followed by diffuse extension of the trunk and extremities. The clonic phase features rhythmic flexor contractions of the trunk and limbs, pupillary dilation, elevations of blood pressure and pulse, urinary incontinence, and tongue biting. This is followed by a profound state of depressed consciousness (post-ictal state) which gradually improves over minutes to hours. The disorder may be cryptogenic, familial, or symptomatic (caused by an identified disease process). (From Adams et al., Principles of Neurology, 6th ed, p329)'),
  'D004831': NtO(MH='Epilepsies, Myoclonic', MS='A clinically diverse group of epilepsy syndromes characterized either by myoclonic seizures or by myoclonus in association with other seizure types. Myoclonic epilepsy syndromes are divided into three subtypes based on etiology: familial, cryptogenic, and symptomatic.'),
  'D004832': NtO(MH='Epilepsy, Absence', MS='A seizure disorder usually occurring in childhood characterized by rhythmic electrical brain discharges of generalized onset. Clinical features include a sudden cessation of ongoing activity usually without loss of postural tone. Rhythmic blinking of the eyelids or lip smacking frequently accompanies the SEIZURES. The usual duration is 5-10 seconds, and multiple episodes may occur daily. Juvenile absence epilepsy is characterized by the juvenile onset of absence seizures and an increased incidence of myoclonus and tonic-clonic seizures. (Menkes, Textbook of Child Neurology, 5th ed, p736)'),
  'D004833': NtO(MH='Epilepsy, Temporal Lobe', MS='A localization-related (focal) form of epilepsy characterized by recurrent seizures that arise from foci within the TEMPORAL LOBE, most commonly from its mesial aspect. A wide variety of psychic phenomena may be associated, including illusions, hallucinations, dyscognitive states, and affective experiences. The majority of complex partial seizures (see EPILEPSY, COMPLEX PARTIAL) originate from the temporal lobes. Temporal lobe seizures may be classified by etiology as cryptogenic, familial, or symptomatic. (From Adams et al., Principles of Neurology, 6th ed, p321).'),
  'D004834': NtO(MH='Epilepsy, Post-Traumatic', MS='Recurrent seizures causally related to CRANIOCEREBRAL TRAUMA. Seizure onset may be immediate but is typically delayed for several days after the injury and may not occur for up to two years. The majority of seizures have a focal onset that correlates clinically with the site of brain injury. Cerebral cortex injuries caused by a penetrating foreign object (CRANIOCEREBRAL TRAUMA, PENETRATING) are more likely than closed head injuries (HEAD INJURIES, CLOSED) to be associated with epilepsy. Concussive convulsions are nonepileptic phenomena that occur immediately after head injury and are characterized by tonic and clonic movements. (From Rev Neurol 1998 Feb;26(150):256-261; Sports Med 1998 Feb;25(2):131-6)'),
  'D004836': NtO(MH='Epimestrol', MS='A synthetic steroid with estrogenic activity.'),
  'D004837': NtO(MH='Epinephrine', MS='The active sympathomimetic hormone from the ADRENAL MEDULLA. It stimulates both the alpha- and beta- adrenergic systems, causes systemic VASOCONSTRICTION and gastrointestinal relaxation, stimulates the HEART, and dilates BRONCHI and cerebral vessels. It is used in ASTHMA and CARDIAC FAILURE and to delay absorption of local ANESTHETICS.'),
  'D004838': NtO(MH='Epiphyses', MS='The head of a long bone that is separated from the shaft by the epiphyseal plate until bone growth stops. At that time, the plate disappears and the head and shaft are united.'),
  'D004839': NtO(MH='Epiphyses, Slipped', MS='A complete or partial separation of the EPIPHYSES from the DIAPHYSES.'),
  'D004840': NtO(MH='Epirizole', MS='4-Methoxy-2-(5-methoxy-3-methylpyrazol-1-yl)-6-methylpyrimidine. A pyrimidinyl pyrazole with antipyretic, analgesic, and anti-inflammatory activity.'),
  'D004841': NtO(MH='Episiotomy', MS='An incision of the posterior vaginal wall and a portion of the pudenda which enlarges the vaginal introitus to facilitate delivery and prevent lacerations.'),
  'D004842': NtO(MH='Epispadias', MS='A birth defect due to malformation of the URETHRA in which the urethral opening is above its normal location. In the male, the malformed urethra generally opens on the top or the side of the PENIS, but the urethra can also be open the entire length of the penis. In the female, the malformed urethral opening is often between the CLITORIS and the labia, or in the ABDOMEN.'),
  'D004843': NtO(MH='Epistasis, Genetic', MS='A form of gene interaction whereby the expression of one gene interferes with or masks the expression of a different gene or genes. Genes whose expression interferes with or masks the effects of other genes are said to be epistatic to the effected genes. Genes whose expression is affected (blocked or masked) are hypostatic to the interfering genes.'),
  'D004844': NtO(MH='Epistaxis', MS='Bleeding from the nose.'),
  'D004845': NtO(MH='Epitestosterone', MS='The 17-alpha isomer of TESTOSTERONE, derived from PREGNENOLONE via the delta5-steroid pathway, and via 5-androstene-3-beta,17-alpha-diol. Epitestosterone acts as an antiandrogen in various target tissues. The ratio between testosterone/epitestosterone is used to monitor anabolic drug abuse.'),
  'D004846': NtO(MH='Epithelial Attachment', MS='A wedge-shaped collar of epithelial cells which form the attachment of the gingiva to the tooth surface at the base of the gingival crevice.'),
  'D004847': NtO(MH='Epithelial Cells', MS='Cells that line the inner and outer surfaces of the body by forming cellular layers (EPITHELIUM) or masses. Epithelial cells lining the SKIN; the MOUTH; the NOSE; and the ANAL CANAL derive from ectoderm; those lining the RESPIRATORY SYSTEM and the DIGESTIVE SYSTEM derive from endoderm; others (CARDIOVASCULAR SYSTEM and LYMPHATIC SYSTEM) derive from mesoderm. Epithelial cells can be classified mainly by cell shape and function into squamous, glandular and transitional epithelial cells.'),
  'D004848': NtO(MH='Epithelium', MS='The layers of EPITHELIAL CELLS which cover the inner and outer surfaces of the cutaneous, mucus, and serous tissues and glands of the body.'),
  'D004849': NtO(MH='Phenylphosphonothioic Acid, 2-Ethyl 2-(4-Nitrophenyl) Ester', MS='An organothiophosphorus cholinesterase inhibitor that is used as an insecticide and as a acaricide.'),
  'D004850': NtO(MH='Eponyms', MS='Names based on or derived from the name of a person or a place.'),
  'D004851': NtO(MH='Epoxide Hydrolases', MS='Enzymes that catalyze reversibly the formation of an epoxide or arene oxide from a glycol or aromatic diol, respectively.'),
  'D004852': NtO(MH='Epoxy Compounds', MS='Organic compounds that include a cyclic ether with three ring atoms in their structure. They are commonly used as precursors for POLYMERS such as EPOXY RESINS.'),
  'D004853': NtO(MH='Epoxy Resins', MS='Polymeric resins derived from OXIRANES and characterized by strength and thermosetting properties. Epoxy resins are often used as dental materials.'),
  'D004854': NtO(MH='Herpesvirus 4, Human', MS='The type species of LYMPHOCRYPTOVIRUS, subfamily GAMMAHERPESVIRINAE, infecting B-cells in humans. It is thought to be the causative agent of INFECTIOUS MONONUCLEOSIS and is strongly associated with oral hairy leukoplakia (LEUKOPLAKIA, HAIRY;), BURKITT LYMPHOMA; and other malignancies.'),
  'D004855': NtO(MH='Equilenin', MS='An estrogenic steroid produced by HORSES. It has a total of five double bonds in the A- and B-ring. High concentration of equilenin is found in the URINE of pregnant mares.'),
  'D004856': NtO(MH='Postural Balance', MS='A POSTURE in which an ideal body mass distribution is achieved. Postural balance provides the body carriage stability and conditions for normal functions in stationary position or in movement, such as sitting, standing, or walking.'),
  'D004857': NtO(MH='Equilin', MS='An estrogenic steroid produced by HORSES. It has a total of four double bonds in the A- and B-ring. High concentration of euilin is found in the URINE of pregnant mares.'),
  'D004858': NtO(MH='Herpesvirus 3, Equid', MS='A species of VARICELLOVIRUS causing coital exanthema in horses.'),
  'D004859': NtO(MH='Equine Infectious Anemia', MS='Viral disease of horses caused by the equine infectious anemia virus (EIAV; INFECTIOUS ANEMIA VIRUS, EQUINE). It is characterized by intermittent fever, weakness, and anemia. Chronic infection consists of acute episodes with remissions.'),
  'D004860': NtO(MH='Infectious Anemia Virus, Equine', MS='A species of LENTIVIRUS, subgenus equine lentiviruses (LENTIVIRUSES, EQUINE), causing acute and chronic infection in horses. It is transmitted mechanically by biting flies, mosquitoes, and midges, and iatrogenically through unsterilized equipment. Chronic infection often consists of acute episodes with remissions.'),
  'D004861': NtO(MH='Herpesvirus 1, Equid', MS='A species of VARICELLOVIRUS causing abortion and respiratory disease in horses.'),
  'D004863': NtO(MH='Equinus Deformity', MS='Plantar declination of the foot.'),
  'D004864': NtO(MH='Equipment and Supplies', MS='Expendable and nonexpendable equipment, supplies, apparatus, and instruments that are used in diagnostic, surgical, therapeutic, scientific, and experimental procedures.'),
  'D004865': NtO(MH='Equipment and Supplies, Hospital', MS='Any materials used in providing care specifically in the hospital.'),
  'D004866': NtO(MH='Equipment Contamination', MS='The presence of an infectious agent on instruments, prostheses, or other inanimate articles.'),
  'D004867': NtO(MH='Equipment Design', MS='Methods and patterns of fabricating machines and related hardware.'),
  'D004868': NtO(MH='Equipment Failure', MS='Failure of equipment to perform to standard. The failure may be due to defects or improper use.'),
  'D004869': NtO(MH='Equipment Safety', MS='Freedom of equipment from actual or potential hazards.'),
  'D004870': NtO(MH='Erabutoxins', MS='Toxins isolated from the venom of Laticauda semifasciata, a sea snake (Hydrophid); immunogenic, basic polypeptides of 62 amino acids, folded by four disulfide bonds, block neuromuscular end-plates irreversibly, thus causing paralysis and severe muscle damage; they are similar to Elapid neurotoxins.'),
  'D004871': NtO(MH='Erbium', MS='Erbium. An element of the rare earth family of metals. It has the atomic symbol Er, atomic number 68, and atomic weight 167.26.'),
  'D004872': NtO(MH='Ergocalciferols', MS='Derivatives of ERGOSTEROL formed by ULTRAVIOLET RAYS breaking of the C9-C10 bond. They differ from CHOLECALCIFEROL in having a double bond between C22 and C23 and a methyl group at C24.'),
  'D004873': NtO(MH='Ergolines', MS='A series of structurally-related alkaloids that contain the ergoline backbone structure.'),
  'D004874': NtO(MH='Ergonovine', MS='An ergot alkaloid (ERGOT ALKALOIDS) with uterine and VASCULAR SMOOTH MUSCLE contractile properties.'),
  'D004875': NtO(MH='Ergosterol', MS='A steroid occurring in FUNGI. Irradiation with ULTRAVIOLET RAYS results in formation of ERGOCALCIFEROL (vitamin D2).'),
  'D004876': NtO(MH='Ergot Alkaloids', MS='Alkaloids originally isolated from the ergot fungus Claviceps purpurea (Hypocreaceae). They include compounds that are structurally related to ergoline (ERGOLINES) and ergotamine (ERGOTAMINES). Many of the ergot alkaloids act as alpha-adrenergic antagonists.'),
  'D004877': NtO(MH='Ergoloid Mesylates', MS='A mixture of the mesylates (methane sulfonates) of DIHYDROERGOCORNINE; DIHYDROERGOCRISTINE; and the alpha- and beta-isomers of DIHYDROERGOCRYPTINE. The substance produces a generalized peripheral vasodilation and a fall in arterial pressure and has been used to treat symptoms of mild to moderate impairment of mental function in the elderly.'),
  'D004878': NtO(MH='Ergotamine', MS='A vasoconstrictor found in ergot of Central Europe. It is a serotonin agonist that has been used as an oxytocic agent and in the treatment of MIGRAINE DISORDERS.'),
  'D004879': NtO(MH='Ergotamines', MS='A series of structurally-related alkaloids containing the ergotaman backbone structure.'),
  'D004880': NtO(MH='Ergothioneine', MS='A naturally occurring metabolite of HISTIDINE that has antioxidant properties.'),
  'D004881': NtO(MH='Ergotism', MS='Poisoning caused by ingesting ergotized grain or by the misdirected or excessive use of ergot as a medicine.'),
  'D004882': NtO(MH='Erotica', MS='Literary or artistic items having an erotic theme. It refers especially to books treating sexual love in a sensuous or voluptuous manner. (Webster, 3d ed)'),
  'D004883': NtO(MH='Erucic Acids', MS='cis-13-Docosenoic Acids. 22-Carbon monounsaturated, monocarboxylic acids.'),
  'D004884': NtO(MH='Eructation', MS='The ejection of gas or air through the mouth from the stomach.'),
  'D004885': NtO(MH='Erwinia', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria whose organisms are associated with plants as pathogens, saprophytes, or as constituents of the epiphytic flora.'),
  'D004886': NtO(MH='Erysipelas', MS='An acute infection of the skin caused by species of STREPTOCOCCUS. This disease most frequently affects infants, young children, and the elderly. Characteristics include pink-to-red lesions that spread rapidly and are warm to the touch. The commonest site of involvement is the face.'),
  'D004887': NtO(MH='Erysipeloid', MS='An infection caused by Erysipelothrix rhusiopathiae that is almost wholly restricted to persons who in their occupation handle infected fish, shellfish, poultry, or meat. Three forms of this condition exist: a mild localized form manifested by local swelling and redness of the skin; a diffuse form that might present with fever; and a rare systemic form associated with endocarditis.'),
  'D004888': NtO(MH='Erysipelothrix', MS='A genus of gram-positive, facultatively anaerobic, rod-shaped bacteria that has a tendency to form long filaments. Its organisms are widely distributed in nature and are found in MAMMALS; BIRDS; and FISHES. Erysipelothrix may appear gram-negative because they decolorize easily.'),
  'D004889': NtO(MH='Erysipelothrix Infections', MS='Infections with bacteria of the genus ERYSIPELOTHRIX.'),
  'D004890': NtO(MH='Erythema', MS='Redness of the skin produced by congestion of the capillaries. This condition may result from a variety of disease processes.'),
  'D004891': NtO(MH='Erythema Induratum', MS='A type of panniculitis characterized histologically by the presence of granulomas, vasculitis, and necrosis. It is traditionally considered to be the tuberculous counterpart of nodular vasculitis, but is now known to occur without tuberculous precedent. It is seen most commonly in adolescent and menopausal women, is initiated or exacerbated by cold weather, and typically presents as one or more recurrent erythrocyanotic nodules or plaques on the calves. The nodules may progress to form indurations, ulcerations, and scars.'),
  'D004892': NtO(MH='Erythema Multiforme', MS='A skin and mucous membrane disease characterized by an eruption of macules, papules, nodules, vesicles, and/or bullae with characteristic "bull\'s-eye" lesions usually occurring on the dorsal aspect of the hands and forearms.'),
  'D004893': NtO(MH='Erythema Nodosum', MS='An erythematous eruption commonly associated with drug reactions or infection and characterized by inflammatory nodules that are usually tender, multiple, and bilateral. These nodules are located predominantly on the shins with less common occurrence on the thighs and forearms. They undergo characteristic color changes ending in temporary bruise-like areas. This condition usually subsides in 3-6 weeks without scarring or atrophy.'),
  'D004894': NtO(MH='Erythrasma', MS='A chronic bacterial infection of major folds of the skin, caused by Corynebacterium minutissimum.'),
  'D004895': NtO(MH='Erythrina', MS='A genus of leguminous shrubs or trees, mainly tropical, yielding useful compounds such as ALKALOIDS and  PLANT LECTINS.'),
  'D004896': NtO(MH='Erythritol', MS='A four-carbon sugar that is found in algae, fungi, and lichens. It is twice as sweet as sucrose and can be used as a coronary vasodilator.'),
  'D004897': NtO(MH='Erythrityl Tetranitrate', MS='A vasodilator with general properties similar to NITROGLYCERIN. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1020)'),
  'D004899': NtO(MH='Erythroblastosis, Fetal', MS='A condition characterized by the abnormal presence of ERYTHROBLASTS in the circulation of the FETUS or NEWBORNS. It is a disorder due to BLOOD GROUP INCOMPATIBILITY, such as the maternal alloimmunization by fetal antigen RH FACTORS leading to HEMOLYSIS of ERYTHROCYTES, hemolytic anemia (ANEMIA, HEMOLYTIC), general edema (HYDROPS FETALIS), and SEVERE JAUNDICE IN NEWBORN.'),
  'D004900': NtO(MH='Erythroblasts', MS='Immature, nucleated ERYTHROCYTES occupying the stage of ERYTHROPOIESIS that follows formation of ERYTHROID PRECURSOR CELLS and precedes formation of RETICULOCYTES. The normal series is called normoblasts. Cells called MEGALOBLASTS are a pathologic series of erythroblasts.'),
  'D004901': NtO(MH='Erythrocebus patas', MS='A species of the genus ERYTHROCEBUS, subfamily CERCOPITHECINAE, family CERCOPITHECIDAE. It inhabits the flat open arid country of Africa. It is also known as the patas monkey or the red monkey.'),
  'D004902': NtO(MH='Erythrocruorins', MS='High molecular weight (1,500,000 to 3,000,000) hemoglobins found in the plasma of many polychete and oligochete annelid worms, crustaceans, insects, and mollusks. They bind one mole of oxygen per heme and function as oxygen carriers.'),
  'D004903': NtO(MH='Erythrocyte Aggregation', MS='The formation of clumps of RED BLOOD CELLS under low or non-flow conditions, resulting from the attraction forces between the red blood cells. The cells adhere to each other in rouleaux aggregates. Slight mechanical force, such as occurs in the circulation, is enough to disperse these aggregates. Stronger or weaker than normal aggregation may result from a variety of effects in the ERYTHROCYTE MEMBRANE or in BLOOD PLASMA. The degree of aggregation is affected by ERYTHROCYTE DEFORMABILITY, erythrocyte membrane sialylation, masking of negative surface charge by plasma proteins, etc. BLOOD VISCOSITY and the ERYTHROCYTE SEDIMENTATION RATE are affected by the amount of erythrocyte aggregation and are parameters used to measure the aggregation.'),
  'D004905': NtO(MH='Erythrocyte Aging', MS='The senescence of RED BLOOD CELLS. Lacking the organelles that make protein synthesis possible, the mature erythrocyte is incapable of self-repair, reproduction, and carrying out certain functions performed by other cells. This limits the average life span of an erythrocyte to 120 days.'),
  'D004906': NtO(MH='Erythrocyte Count', MS='The number of RED BLOOD CELLS per unit volume in a sample of venous BLOOD.'),
  'D004907': NtO(MH='Erythrocyte Deformability', MS='Ability of ERYTHROCYTES to change shape as they pass through narrow spaces, such as the microvasculature.'),
  'D004908': NtO(MH='Erythrocyte Inclusions', MS='Pathologic inclusions occurring in erythrocytes.'),
  'D004909': NtO(MH='Erythrocyte Indices', MS='ERYTHROCYTE size and HEMOGLOBIN content or concentration, usually derived from ERYTHROCYTE COUNT; BLOOD hemoglobin concentration; and HEMATOCRIT. The indices include the mean corpuscular volume (MCV), the mean corpuscular hemoglobin (MCH), and the mean corpuscular hemoglobin concentration (MCHC).'),
  'D004910': NtO(MH='Erythrocyte Membrane', MS="The semi-permeable outer structure of a red blood cell. It is known as a red cell 'ghost' after HEMOLYSIS."),
  'D004911': NtO(MH='Erythrocyte Volume', MS='Volume of circulating ERYTHROCYTES . It is usually measured by RADIOISOTOPE DILUTION TECHNIQUE.'),
  'D004912': NtO(MH='Erythrocytes', MS='Red blood cells. Mature erythrocytes are non-nucleated, biconcave disks containing HEMOGLOBIN whose function is to transport OXYGEN.'),
  'D004913': NtO(MH='Erythrocytes, Abnormal', MS='Oxygen-carrying RED BLOOD CELLS in mammalian blood that are abnormal in structure or function.'),
  'D004915': NtO(MH='Leukemia, Erythroblastic, Acute', MS='A myeloproliferative disorder characterized by neoplastic proliferation of erythroblastic and myeloblastic elements with atypical erythroblasts and myeloblasts in the peripheral blood.'),
  'D004916': NtO(MH='Erythromelalgia', MS='A peripheral arterial disease that is characterized by the triad of ERYTHEMA, burning PAIN, and increased SKIN TEMPERATURE of the extremities (or red, painful extremities). Erythromelalgia may be classified as primary or idiopathic, familial or non-familial. Secondary erythromelalgia is associated with other diseases, the most common being MYELOPROLIFERATIVE DISORDERS.'),
  'D004917': NtO(MH='Erythromycin', MS='A bacteriostatic antibiotic macrolide produced by Streptomyces erythreus. Erythromycin A is considered its major active component. In sensitive organisms, it inhibits protein synthesis by binding to 50S ribosomal subunits. This binding process inhibits peptidyl transferase activity and interferes with translocation of amino acids during translation and assembly of proteins.'),
  'D004918': NtO(MH='Erythromycin Estolate', MS='A macrolide antibiotic, produced by Streptomyces erythreus. It is the lauryl sulfate salt of the propionic ester of erythromycin. This erythromycin salt acts primarily as a bacteriostatic agent. In sensitive organisms, it inhibits protein synthesis by binding to 50S ribosomal subunits. This binding process inhibits peptidyl transferase activity and interferes with translocation of amino acids during translation and assembly of proteins.'),
  'D004919': NtO(MH='Erythroplasia', MS='A condition of the mucous membrane characterized by erythematous papular lesions.'),
  'D004920': NtO(MH='Erythropoiesis', MS='The production of red blood cells (ERYTHROCYTES). In humans, erythrocytes are produced by the YOLK SAC in the first trimester; by the liver in the second trimester; by the BONE MARROW in the third trimester and after birth. In normal individuals, the erythrocyte count in the peripheral blood remains relatively constant implying a balance between the rate of erythrocyte production and rate of destruction.'),
  'D004921': NtO(MH='Erythropoietin', MS='Glycoprotein hormone, secreted chiefly by the KIDNEY in the adult and the LIVER in the FETUS, that acts on erythroid stem cells of the BONE MARROW to stimulate proliferation and differentiation.'),
  'D004923': NtO(MH='Erythrosine', MS='A tetraiodofluorescein used as a red coloring in some foods (cherries, fish), as a disclosure of DENTAL PLAQUE, and as a stain of some cell types. It has structural similarity to THYROXINE.'),
  'D004924': NtO(MH='Escape Reaction', MS='Innate response elicited by sensory stimuli associated with a threatening situation, or actual confrontation with an enemy.'),
  'D004925': NtO(MH='Escherichia', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria whose organisms occur in the lower part of the intestine of warm-blooded animals. The species are either nonpathogenic or opportunistic pathogens.'),
  'D004926': NtO(MH='Escherichia coli', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria (GRAM-NEGATIVE FACULTATIVELY ANAEROBIC RODS) commonly found in the lower part of the intestine of warm-blooded animals. It is usually nonpathogenic, but some strains are known to produce DIARRHEA and pyogenic infections. Pathogenic strains (virotypes) are classified by their specific pathogenic mechanisms such as toxins (ENTEROTOXIGENIC ESCHERICHIA COLI), etc.'),
  'D004927': NtO(MH='Escherichia coli Infections', MS='Infections with bacteria of the species ESCHERICHIA COLI.'),
  'D004928': NtO(MH='Escin', MS='Pentacyclic triterpene saponins, biosynthesized from protoaescigenin and barringtogenol, occurring in the seeds of AESCULUS. It inhibits edema formation and decreases vascular fragility.'),
  'D004929': NtO(MH='Esculin', MS='A derivative of COUMARIN with molecular formula C15H16O9.'),
  'D004930': NtO(MH='Inuits', MS='Inuktitut-speakers generally associated with the northern polar region.'),
  'D004931': NtO(MH='Esophageal Achalasia', MS='A motility disorder of the ESOPHAGUS in which the LOWER ESOPHAGEAL SPHINCTER (near the CARDIA) fails to relax resulting in functional obstruction of the esophagus, and DYSPHAGIA. Achalasia is characterized by a grossly contorted and dilated esophagus (megaesophagus).'),
  'D004932': NtO(MH='Esophageal and Gastric Varices', MS='Dilated blood vessels in the ESOPHAGUS or GASTRIC FUNDUS that shunt blood from the portal circulation (PORTAL SYSTEM) to the systemic venous circulation. Often they are observed in individuals with portal hypertension (HYPERTENSION, PORTAL).'),
  'D004933': NtO(MH='Esophageal Atresia', MS='Congenital abnormality characterized by the lack of full development of the ESOPHAGUS that commonly occurs with TRACHEOESOPHAGEAL FISTULA. Symptoms include excessive SALIVATION; GAGGING; CYANOSIS; and DYSPNEA.'),
  'D004934': NtO(MH='Esophageal Cyst', MS='Any fluid-filled closed cavity or sac (CYSTS) that is lined by an EPITHELIUM and found in the ESOPHAGUS region.'),
  'D004935': NtO(MH='Esophageal Diseases', MS='Pathological processes in the ESOPHAGUS.'),
  'D004936': NtO(MH='Diverticulum, Esophageal', MS='Saccular protrusion beyond the wall of the ESOPHAGUS.'),
  'D004937': NtO(MH='Esophageal Fistula', MS='Abnormal passage communicating with the ESOPHAGUS. The most common type is TRACHEOESOPHAGEAL FISTULA between the esophagus and the TRACHEA.'),
  'D004938': NtO(MH='Esophageal Neoplasms', MS='Tumors or cancer of the ESOPHAGUS.'),
  'D004939': NtO(MH='Esophageal Perforation', MS='An opening or hole in the ESOPHAGUS that is caused by TRAUMA, injury, or pathological process.'),
  'D004940': NtO(MH='Esophageal Stenosis', MS='A stricture of the ESOPHAGUS. Most are acquired but can be congenital.'),
  'D004941': NtO(MH='Esophagitis', MS='INFLAMMATION, acute or chronic, of the ESOPHAGUS caused by BACTERIA, chemicals, or TRAUMA.'),
  'D004942': NtO(MH='Esophagitis, Peptic', MS='INFLAMMATION of the ESOPHAGUS that is caused by the reflux of GASTRIC JUICE with contents of the STOMACH and DUODENUM.'),
  'D004943': NtO(MH='Esophagogastric Junction', MS='The area covering the terminal portion of ESOPHAGUS and the beginning of STOMACH at the cardiac orifice.'),
  'D004944': NtO(MH='Esophagoplasty', MS='A plastic operation on the esophagus. (Dorland, 28th ed)'),
  'D004945': NtO(MH='Esophagoscopy', MS='Endoscopic examination, therapy or surgery of the esophagus.'),
  'D004946': NtO(MH='Esophagostomy', MS='Surgical formation of an external opening (stoma) into the esophagus.'),
  'D004947': NtO(MH='Esophagus', MS='The muscular membranous segment between the PHARYNX and the STOMACH in the UPPER GASTROINTESTINAL TRACT.'),
  'D004948': NtO(MH='Esotropia', MS='A form of ocular misalignment characterized by an excessive convergence of the visual axes, resulting in a "cross-eye" appearance. An example of this condition occurs when paralysis of the lateral rectus muscle causes an abnormal inward deviation of one eye on attempted gaze.'),
  'D004949': NtO(MH='Estazolam', MS='A benzodiazepine with anticonvulsant, hypnotic, and muscle relaxant properties. It has been shown in some cases to be more potent than DIAZEPAM or NITRAZEPAM.'),
  'D004950': NtO(MH='Esterases', MS=None),
  'D004951': NtO(MH='Esterification', MS='The process of converting an acid into an alkyl or aryl derivative. Most frequently the process consists of the reaction of an acid with an alcohol in the presence of a trace of mineral acid as catalyst or the reaction of an acyl chloride with an alcohol. Esterification can also be accomplished by enzymatic processes.'),
  'D004952': NtO(MH='Esters', MS='Compounds derived from organic or inorganic acids in which at least one hydroxyl group is replaced by an \xe2\x80\x93O-alkyl or other organic group. They can be represented by the structure formula RCOOR\xe2\x80\x99 and are usually formed by the reaction between an acid and an alcohol with elimination of water.'),
  'D004953': NtO(MH='Estetrol', MS='A metabolite of ESTRIOL with a 15-alpha-hydroxyl group. Estetrol can be converted from estriol sulfate or DEHYDROEPIANDROSTERONE SULFATE by the fetal-placental unit.'),
  'D004954': NtO(MH='Esthetics', MS='The branch of philosophy dealing with the nature of the beautiful. It includes beauty, esthetic experience, esthetic judgment, esthetic aspects of medicine, etc.'),
  'D004955': NtO(MH='Esthetics, Dental', MS="Skills, techniques, standards, and principles used to improve the art and symmetry of the teeth and face to improve the appearance as well as the function of the teeth, mouth, and face. (From Boucher's Clinical Dental Terminology, 4th ed, p108)"),
  'D004956': NtO(MH='Estivation', MS='In certain living species, a period of dormancy during the summer months marked by decreased metabolism.'),
  'D004957': NtO(MH='Estonia', MS='A country in Eastern Europe, bordering the Baltic Sea and Gulf of Finland, between Latvia and Russia. The capital is Tallinn.'),
  'D004958': NtO(MH='Estradiol', MS='The 17-beta-isomer of estradiol, an aromatized C18 steroid with hydroxyl group at 3-beta- and 17-beta-position. Estradiol-17-beta is the most potent form of mammalian estrogenic steroids.'),
  'D004960': NtO(MH='Estradiol Dehydrogenases', MS='Enzymes that catalyze the oxidation of estradiol at the 17-hydroxyl group in the presence of NAD+ or NADP+ to yield estrone and NADH or NADPH. The 17-hydroxyl group can be in the alpha- or beta-configuration. EC 1.1.1.62'),
  'D004961': NtO(MH='Estramustine', MS='A nitrogen mustard linked to estradiol, usually as phosphate; used to treat prostatic neoplasms; also has radiation protective properties.'),
  'D004962': NtO(MH='Estranes', MS='A group of compounds forming the nucleus of the estrogenic steroid family.'),
  'D004963': NtO(MH='Estrenes', MS='Unsaturated derivatives of the ESTRANES with methyl groups at carbon-13, with no carbon at carbon-10, and with no more than one carbon at carbon-17. They must contain one or more double bonds.'),
  'D004964': NtO(MH='Estriol', MS='A hydroxylated metabolite of ESTRADIOL or ESTRONE that has a hydroxyl group at C3, 16-alpha, and 17-beta position. Estriol is a major urinary estrogen. During PREGNANCY, a large amount of estriol is produced by the PLACENTA. Isomers with inversion of the hydroxyl group or groups are called epiestriol.'),
  'D004965': NtO(MH='Estrogen Antagonists', MS='Compounds which inhibit or antagonize the action or biosynthesis of estrogenic compounds.'),
  'D004966': NtO(MH='Estrogens, Conjugated (USP)', MS='A pharmaceutical preparation containing a mixture of water-soluble, conjugated estrogens derived wholly or in part from URINE of pregnant mares or synthetically from ESTRONE and EQUILIN. It contains a sodium-salt mixture of estrone sulfate (52-62%) and equilin sulfate (22-30%) with a total of the two between 80-88%. Other concomitant conjugates include 17-alpha-dihydroequilin, 17-alpha-estradiol, and 17-beta-dihydroequilin. The potency of the preparation is expressed in terms of an equivalent quantity of sodium estrone sulfate.'),
  'D004967': NtO(MH='Estrogens', MS='Compounds that interact with ESTROGEN RECEPTORS in target tissues to bring about the effects similar to those of ESTRADIOL. Estrogens stimulate the female reproductive organs, and the development of secondary female SEX CHARACTERISTICS. Estrogenic chemicals include natural, synthetic, steroidal, or non-steroidal compounds.'),
  'D004968': NtO(MH='Estrogens, Non-Steroidal', MS='Non-steroidal compounds with estrogenic activity.'),
  'D004970': NtO(MH='Estrone', MS='An aromatized C18 steroid with a 3-hydroxyl group and a 17-ketone, a major mammalian estrogen. It is converted from ANDROSTENEDIONE directly, or from TESTOSTERONE via ESTRADIOL. In humans, it is produced primarily by the cyclic ovaries, PLACENTA, and the ADIPOSE TISSUE of men and postmenopausal women.'),
  'D004971': NtO(MH='Estrus', MS='The period in the ESTROUS CYCLE associated with maximum sexual receptivity and fertility in non-primate female mammals.'),
  'D004972': NtO(MH='Estrus Detection', MS='Methods for recognizing the state of ESTRUS.'),
  'D004973': NtO(MH='Estrus Synchronization', MS='Occurrence or induction of ESTRUS in all of the females in a group at the same time, applies only to non-primate mammals with ESTROUS CYCLE.'),
  'D004974': NtO(MH='Etazolate', MS='A potent phosphodiesterase inhibitor proposed as an antipsychotic agent.'),
  'D004975': NtO(MH='Ethacridine', MS='A topically applied anti-infective agent.'),
  'D004976': NtO(MH='Ethacrynic Acid', MS='A compound that inhibits symport of sodium, potassium, and chloride primarily in the ascending limb of Henle, but also in the proximal and distal tubules. This pharmacological action results in excretion of these ions, increased urinary output, and reduction in extracellular fluid. This compound has been classified as a loop or high ceiling diuretic.'),
  'D004977': NtO(MH='Ethambutol', MS='An antitubercular agent that inhibits the transfer of mycolic acids into the cell wall of the tubercle bacillus. It may also inhibit the synthesis of spermidine in mycobacteria. The action is usually bactericidal, and the drug can penetrate human cell membranes to exert its lethal effect. (From Smith and Reynard, Textbook of Pharmacology, 1992, p863)'),
  'D004978': NtO(MH='Ethamoxytriphetol', MS='A non-steroidal estrogen antagonist.'),
  'D004979': NtO(MH='Ethamsylate', MS='Benzenesulfonate derivative used as a systemic hemostatic.'),
  'D004980': NtO(MH='Ethane', MS='A two carbon alkane with the formula H3C-CH3.'),
  'D004981': NtO(MH='Ethanolamine Ammonia-Lyase', MS='An enzyme that catalyzes the deamination of ethanolamine to acetaldehyde. EC 4.3.1.7.'),
  'D004982': NtO(MH='Ethanolaminephosphotransferase', MS='An enzyme that catalyzes reversibly the transfer of phosphoethanolamine from CDP-ethanolamine to diacylglycerol to yield phosphatidylethanolamine (cephalin) and CMP. The enzyme is found in the endoplasmic reticulum. EC 2.7.8.1.'),
  'D004983': NtO(MH='Ethanolamines', MS='AMINO ALCOHOLS containing the ETHANOLAMINE; (-NH2CH2CHOH) group and its derivatives.'),
  'D004984': NtO(MH='Ethchlorvynol', MS='A sedative and hypnotic that has been used in the short-term management of INSOMNIA. Its use has been superseded by other drugs.'),
  'D004985': NtO(MH='Ethenoadenosine Triphosphate', MS='1,N-6-Ethenoadenosine triphosphate. A fluorescent analog of adenosine triphosphate.'),
  'D004986': NtO(MH='Ether', MS='A mobile, very volatile, highly flammable liquid used as an inhalation anesthetic and as a solvent for waxes, fats, oils, perfumes, alkaloids, and gums. It is mildly irritating to skin and mucous membranes.'),
  'D004987': NtO(MH='Ethers', MS='Organic compounds having two alkyl or aryl groups bonded to an oxygen atom, as in the formula R1\xe2\x80\x93O\xe2\x80\x93R2.'),
  'D004988': NtO(MH='Ethers, Cyclic', MS='Compounds of the general formula R-O-R arranged in a ring or crown formation.'),
  'D004989': NtO(MH='Ethics', MS='The philosophy or code pertaining to what is ideal in human character and conduct. Also, the field of study dealing with the principles of morality.'),
  'D004990': NtO(MH='Ethics, Dental', MS='The principles of proper professional conduct concerning the rights and duties of the dentist, relations with patients and fellow practitioners, as well as actions of the dentist in patient care and interpersonal relations with patient families. (From Stedman, 25th ed)'),
  'D004991': NtO(MH='Ethics, Institutional', MS='The moral and ethical obligations or responsibilities of institutions.'),
  'D004992': NtO(MH='Ethics, Medical', MS='The principles of professional conduct concerning the rights and duties of the physician, relations with patients and fellow practitioners, as well as actions of the physician in patient care and interpersonal relations with patient families.'),
  'D004993': NtO(MH='Ethics, Nursing', MS='The principles of proper professional conduct concerning the rights and duties of nurses themselves, their patients, and their fellow practitioners, as well as their actions in the care of patients and in relations with their families.'),
  'D004994': NtO(MH='Ethics, Pharmacy', MS='The principles of proper professional conduct concerning the rights and duties of the pharmacist, relations with patients and fellow practitioners, as well as actions of the pharmacist in health care and interpersonal relations with patient families. (From Stedman, 25th ed)'),
  'D004995': NtO(MH='Ethics, Professional', MS='The principles of proper conduct concerning the rights and duties of the professional, relations with patients or consumers and fellow practitioners, as well as actions of the professional and interpersonal relations with patient or consumer families. (From Stedman, 25th ed)'),
  'D004996': NtO(MH='Ethidium', MS='A trypanocidal agent and possible antiviral agent that is widely used in experimental cell biology and biochemistry. Ethidium has several experimentally useful properties including binding to nucleic acids, noncompetitive inhibition of nicotinic acetylcholine receptors, and fluorescence among others. It is most commonly used as the bromide.'),
  'D004997': NtO(MH='Ethinyl Estradiol', MS='A semisynthetic alkylated ESTRADIOL with a 17-alpha-ethinyl substitution. It has high estrogenic potency when administered orally, and is often used as the estrogenic component in ORAL CONTRACEPTIVES.'),
  'D004998': NtO(MH='Ethiodized Oil', MS='Ethyl ester of iodinated fatty acid of poppyseed oil. It contains 37% organically bound iodine and has been used as a diagnostic aid (radiopaque medium) and as an antineoplastic agent when part of the iodine is 131-I. (From Merck Index, 11th ed)'),
  'D004999': NtO(MH='Amifostine', MS='A phosphorothioate proposed as a radiation-protective agent. It causes splenic vasodilation and may block autonomic ganglia.'),
  'D005000': NtO(MH='Ethionamide', MS='A second-line antitubercular agent that inhibits mycolic acid synthesis.'),
  'D005001': NtO(MH='Ethionine', MS='2-Amino-4-(ethylthio)butyric acid. An antimetabolite and methionine antagonist that interferes with amino acid incorporation into proteins and with cellular ATP utilization. It also produces liver neoplasms.'),
  'D005002': NtO(MH='Ethiopia', MS='An independent state in eastern Africa. Ethiopia is located in the Horn of Africa and is bordered on the north and northeast by Eritrea, on the east by Djibouti and Somalia, on the south by Kenya, and on the west and southwest by Sudan. Its capital is Addis Ababa.'),
  'D005003': NtO(MH='Ethisterone', MS='17 alpha-Hydroxypregn-4-en-20-yn-3-one. A synthetic steroid hormone with progestational effects.'),
  'D005004': NtO(MH='Ethmoid Bone', MS='A light and spongy (pneumatized) bone that lies between the orbital part of FRONTAL BONE and the anterior of SPHENOID BONE. Ethmoid bone separates the ORBIT from the ETHMOID SINUS. It consists of a horizontal plate, a perpendicular plate, and two lateral labyrinths.'),
  'D005005': NtO(MH='Ethmoid Sinus', MS='The numerous (6-12) small thin-walled spaces or air cells in the ETHMOID BONE located between the eyes. These air cells form an ethmoidal labyrinth.'),
  'D005006': NtO(MH='Ethnic Groups', MS='A group of people with a common cultural heritage that sets them apart from others in a variety of social relationships.'),
  'D005007': NtO(MH='Ethnology', MS='The comparative and theoretical study of culture, often synonymous with cultural anthropology.'),
  'D005008': NtO(MH='Ethnopsychology', MS='Comparative PSYCHOLOGY of different ethnic and cultural groups.'),
  'D005009': NtO(MH='Ethoglucid', MS='Alkylating antineoplastic agent used especially in bladder neoplasms. It is toxic to hair follicles, gastro-intestinal tract, and vasculature.'),
  'D005010': NtO(MH='Ethology', MS='The discipline pertaining to the study of animal behavior.'),
  'D005011': NtO(MH='Ethopabate', MS='An inhibitor of folate metabolism. It is used as a coccidiostat in poultry.'),
  'D005013': NtO(MH='Ethosuximide', MS='An anticonvulsant especially useful in the treatment of absence seizures unaccompanied by other types of seizures.'),
  'D005015': NtO(MH='Ethoxyquin', MS='Antioxidant; also a post-harvest dip to prevent scald on apples and pears.'),
  'D005016': NtO(MH='Ethoxzolamide', MS='A carbonic anhydrase inhibitor used as diuretic and in glaucoma. It may cause hypokalemia.'),
  'D005017': NtO(MH='Ethyl Biscoumacetate', MS='A coumarin that is used as an anticoagulant. It has actions similar to those of WARFARIN. (From Martindale, The Extra Pharmacopoeia, 30th ed, p226)'),
  'D005018': NtO(MH='Ethyl Chloride', MS='A gas that condenses under slight pressure. Because of its low boiling point ethyl chloride sprayed on skin produces an intense cold by evaporation. Cold blocks nerve conduction. Ethyl chloride has been used in surgery but is primarily used to relieve local pain in sports medicine.'),
  'D005019': NtO(MH='Ethyl Ethers', MS='Organic compounds having ethyl groups bound to an oxygen atom.'),
  'D005020': NtO(MH='Ethyl Methanesulfonate', MS='An antineoplastic agent with alkylating properties. It also acts as a mutagen by damaging DNA and is used experimentally for that effect.'),
  'D005021': NtO(MH='Ethylamines', MS='Derivatives of ethylamine (the structural formula CH3CH2NH2).'),
  'D005022': NtO(MH='Ethyldimethylaminopropyl Carbodiimide', MS='Carbodiimide cross-linking reagent.'),
  'D005023': NtO(MH='Ethylene Chlorohydrin', MS='Used as a solvent, in the manufacture of insecticides, and for treating sweet potatoes before planting. May cause nausea, vomiting, pains in head and chest, stupefaction. Irritates mucous membranes and causes kidney and liver degeneration.'),
  'D005025': NtO(MH='Ethylene Dichlorides', MS='Toxic, chlorinated, saturated hydrocarbons. Include both the 1,1- and 1,2-dichloro isomers. The latter is considerably more toxic. It has a sweet taste, ethereal odor and has been used as a fumigant and intoxicant among sniffers. Has many household and industrial uses.'),
  'D005026': NtO(MH='Ethylene Glycols', MS='An ethylene compound with two hydroxy groups (-OH) located on adjacent carbons. They are viscous and colorless liquids. Some are used as anesthetics or hypnotics. However, the class is best known for their use as a coolant or antifreeze.'),
  'D005027': NtO(MH='Ethylene Oxide', MS='A colorless and flammable gas at room temperature and pressure. Ethylene oxide is a bactericidal, fungicidal, and sporicidal disinfectant. It is effective against most micro-organisms, including viruses. It is used as a fumigant for foodstuffs and textiles and as an agent for the gaseous sterilization of heat-labile pharmaceutical and surgical materials. (From Reynolds, Martindale The Extra Pharmacopoeia, 30th ed, p794)'),
  'D005028': NtO(MH='Ethylenebis(dithiocarbamates)', MS='A class of thiocarbamate derivatives whose salts possess fungicidal activity.'),
  'D005029': NtO(MH='Ethylenediamines', MS='Derivatives of ethylenediamine (the structural formula NH2CH2CH2NH2).'),
  'D005030': NtO(MH='Ethylenes', MS='Derivatives of ethylene, a simple organic gas of biological origin with many industrial and biological use.'),
  'D005031': NtO(MH='Ethylenethiourea', MS='A degradation product of ethylenebis(dithiocarbamate) fungicides. It has been found to be carcinogenic and to cause THYROID hyperplasia.'),
  'D005032': NtO(MH='Ethylestrenol', MS='An anabolic steroid with some progestational activity and little androgenic effect.'),
  'D005033': NtO(MH='Ethylmaleimide', MS='A sulfhydryl reagent that is widely used in experimental biochemical studies.'),
  'D005034': NtO(MH='Ethylmercuric Chloride', MS='A highly toxic compound used as a fungicide for treating seeds.'),
  'D005035': NtO(MH='Ethylmercury Compounds', MS='Organic mercury compounds in which the mercury is attached to an ethyl group.'),
  'D005036': NtO(MH='Ethylmorphine', MS='A narcotic analgesic and antitussive. It is metabolized in the liver by ETHYLMORPHINE-N-DEMETHYLASE and used as an indicator of liver function.'),
  'D005037': NtO(MH='Ethylmorphine-N-Demethylase', MS='A drug-metabolizing enzyme of the hepatic microsomal oxidase system which catalyzes the oxidation of the N-methyl group of ethylmorphine with the formation of formaldehyde.'),
  'D005038': NtO(MH='Ethylnitrosourea', MS='A nitrosourea compound with alkylating, carcinogenic, and mutagenic properties.'),
  'D005039': NtO(MH='Etilefrine', MS='A phenylephrine-related beta-1 adrenergic and alpha adrenergic agonist used as a cardiotonic and antihypotensive agent.'),
  'D005040': NtO(MH='Ethynodiol Diacetate', MS='A synthetic progestational hormone used alone or in combination with estrogens as an oral contraceptive (CONTRACEPTIVES, ORAL).'),
  'D005041': NtO(MH='Etidocaine', MS='A local anesthetic with rapid onset and long action, similar to BUPIVACAINE.'),
  'D005042': NtO(MH='Etimizol', MS='A xanthine-related, putative nootropic drug.'),
  'D005043': NtO(MH='Etiocholanolone', MS='The 5-beta-reduced isomer of ANDROSTERONE. Etiocholanolone is a major metabolite of TESTOSTERONE and ANDROSTENEDIONE in many mammalian species including humans. It is excreted in the URINE.'),
  'D005044': NtO(MH='Etioporphyrins', MS='Porphyrins with four methyl and four ethyl side chains attached to the pyrrole rings.'),
  'D005045': NtO(MH='Etomidate', MS='Imidazole derivative anesthetic and hypnotic with little effect on blood gases, ventilation, or the cardiovascular system. It has been proposed as an induction anesthetic.'),
  'D005047': NtO(MH='Etoposide', MS='A semisynthetic derivative of PODOPHYLLOTOXIN that exhibits antitumor activity. Etoposide inhibits DNA synthesis by forming a complex with topoisomerase II and DNA. This complex induces breaks in double stranded DNA and prevents repair by topoisomerase II binding. Accumulated breaks in DNA prevent entry into the mitotic phase of cell division, and lead to cell death. Etoposide acts primarily in the G2 and S phases of the cell cycle.'),
  'D005048': NtO(MH='Etorphine', MS='A narcotic analgesic morphinan used as a sedative in veterinary practice.'),
  'D005050': NtO(MH='Etretinate', MS='An oral retinoid used in the treatment of keratotic genodermatosis, lichen planus, and psoriasis. Beneficial effects have also been claimed in the prophylaxis of epithelial neoplasia. The compound may be teratogenic.'),
  'D005051': NtO(MH='Eubacterium', MS='A genus of gram-positive, rod-shaped bacteria found in cavities of man and animals, animal and plant products, infections of soft tissue, and soil. Some species may be pathogenic. No endospores are produced. The genus Eubacterium should not be confused with EUBACTERIA, one of the three domains of life.'),
  'D005052': NtO(MH='Eucalyptus', MS='A genus of trees of the Myrtaceae family, native to Australia, that yields gums, oils, and resins which are used as flavoring agents, astringents, and aromatics.'),
  'D005053': NtO(MH='Eugenics', MS='The attempt to improve the PHENOTYPES of future generations of the human population by fostering the reproduction of those with favorable phenotypes and GENOTYPES and hampering or preventing BREEDING by those with "undesirable" phenotypes and genotypes. The concept is largely discredited. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D005054': NtO(MH='Eugenol', MS='A cinnamate derivative of the shikamate pathway found in CLOVE OIL and other PLANTS.'),
  'D005055': NtO(MH='Euglena', MS='A genus of EUKARYOTES, in the phylum EUGLENIDA, found mostly in stagnant water. Characteristics include a pellicle usually marked by spiral or longitudinal striations.'),
  'D005056': NtO(MH='Euglena gracilis', MS='A species of fresh-water, flagellated EUKARYOTES in the phylum EUGLENIDA.'),
  'D005057': NtO(MH='Eukaryotic Cells', MS='Cells of the higher organisms, containing a true nucleus bounded by a nuclear membrane.'),
  'D005058': NtO(MH='Eunuchism', MS='The state of being a eunuch, a male without TESTES or whose testes failed to develop. It is characterized by the lack of mature male GERM CELLS and TESTICULAR HORMONES.'),
  'D005059': NtO(MH='Euphoria', MS='An exaggerated feeling of physical and emotional well-being not consonant with apparent stimuli or events; usually of psychologic origin, but also seen in organic brain disease and toxic states.'),
  'D005060': NtO(MH='Europe', MS='The continent north of AFRICA, west of ASIA and east of the ATLANTIC OCEAN.'),
  'D005061': NtO(MH='Europe, Eastern', MS='The region of Europe generally from the area of the Baltic Sea to the Ural Mountains of Russia.'),
  'D005062': NtO(MH='European Union', MS='The collective designation of three organizations with common membership: the European Economic Community (Common Market), the European Coal and Steel Community, and the European Atomic Energy Community (Euratom). It was known as the European Community until 1994. It is primarily an economic union with the principal objectives of free movement of goods, capital, and labor. Professional services, social, medical and paramedical, are subsumed under labor. The constituent countries are Austria, Belgium, Denmark, Finland, France, Germany, Greece, Ireland, Italy, Luxembourg, Netherlands, Portugal, Spain, Sweden, and the United Kingdom. (The World Almanac and Book of Facts 1997, p842)'),
  'D005063': NtO(MH='Europium', MS='An element of the rare earth family of metals. It has the atomic symbol Eu, atomic number 63, and atomic weight 152. Europium is used in the form of its salts as coatings for cathode ray tubes and in the form of its organic derivatives as shift reagents in NMR spectroscopy.'),
  'D005064': NtO(MH='Eustachian Tube', MS='A narrow passageway that connects the upper part of the throat to the TYMPANIC CAVITY.'),
  'D005065': NtO(MH='Euthanasia', MS='The act or practice of killing or allowing death from natural causes, for reasons of mercy, i.e., in order to release a person from incurable disease, intolerable suffering, or undignified death. (from Beauchamp and Walters, Contemporary Issues in Bioethics, 5th ed)'),
  'D005066': NtO(MH='Euthanasia, Passive', MS='Failing to prevent death from natural causes, for reasons of mercy by the withdrawal or withholding of life-prolonging treatment.'),
  'D005067': NtO(MH='Euthyroid Sick Syndromes', MS='Conditions of abnormal THYROID HORMONES release in patients with apparently normal THYROID GLAND during severe systemic illness, physical TRAUMA, and psychiatric disturbances. It can be caused by the loss of endogenous hypothalamic input or by exogenous drug effects. The most common abnormality results in low T3 THYROID HORMONE with progressive decrease in THYROXINE; (T4) and TSH. Elevated T4 with normal T3 may be seen in diseases in which THYROXINE-BINDING GLOBULIN synthesis and release are increased.'),
  'D005068': NtO(MH='Eutrophication', MS='The enrichment of a terrestrial or aquatic ECOSYSTEM by the addition of nutrients, especially nitrogen and phosphorus, that results in a superabundant growth of plants, ALGAE, or other primary producers. It can be a natural process or result from human activity such as agriculture runoff or sewage pollution. In aquatic ecosystems, an increase in the algae population is termed an algal bloom.'),
  'D005069': NtO(MH='Evaluation Studies as Topic', MS='Works about studies that determine the effectiveness or value of processes, personnel, and equipment, or the material on conducting such studies.'),
  'D005070': NtO(MH='Evans Blue', MS='An azo dye used in blood volume and cardiac output measurement by the dye dilution method. It is very soluble, strongly bound to plasma albumin, and disappears very slowly.'),
  'D005071': NtO(MH='Evoked Potentials', MS='Electrical responses recorded from nerve, muscle, SENSORY RECEPTOR, or area of the CENTRAL NERVOUS SYSTEM following stimulation. They range from less than a microvolt to several microvolts. The evoked potential can be auditory (EVOKED POTENTIALS, AUDITORY), somatosensory (EVOKED POTENTIALS, SOMATOSENSORY), visual (EVOKED POTENTIALS, VISUAL), or motor (EVOKED POTENTIALS, MOTOR), or other modalities that have been reported.'),
  'D005072': NtO(MH='Evoked Potentials, Auditory', MS='The electric response evoked in the CEREBRAL CORTEX by ACOUSTIC STIMULATION or stimulation of the AUDITORY PATHWAYS.'),
  'D005073': NtO(MH='Evoked Potentials, Somatosensory', MS='The electric response evoked in the CEREBRAL CORTEX by stimulation along AFFERENT PATHWAYS from PERIPHERAL NERVES to CEREBRUM.'),
  'D005074': NtO(MH='Evoked Potentials, Visual', MS='The electric response evoked in the cerebral cortex by visual stimulation or stimulation of the visual pathways.'),
  'D005075': NtO(MH='Biological Evolution', MS='The process of cumulative change over successive generations through which organisms acquire their distinguishing morphological and physiological characteristics.'),
  'D005076': NtO(MH='Exanthema', MS="Diseases in which skin eruptions or rashes are a prominent manifestation. Classically, six such diseases were described with similar rashes; they were numbered in the order in which they were reported. Only the fourth (Duke's disease), fifth (ERYTHEMA INFECTIOSUM), and sixth (EXANTHEMA SUBITUM) numeric designations survive as occasional synonyms in current terminology."),
  'D005077': NtO(MH='Exanthema Subitum', MS='An acute, short-lived, viral disease of infants and young children characterized by a high fever at onset that drops to normal after 3-4 days and the concomitant appearance of a macular or maculopapular rash that appears first on the trunk and then spreads to other areas. It is the sixth of the classical exanthematous diseases and is caused by HHV-6; (HERPESVIRUS 6, HUMAN). (From Dorland, 27th ed)'),
  'D005078': NtO(MH='Exchange Transfusion, Whole Blood', MS='Repetitive withdrawal of small amounts of blood and replacement with donor blood until a large proportion of the blood volume has been exchanged. Used in treatment of fetal erythroblastosis, hepatic coma, sickle cell anemia, disseminated intravascular coagulation, septicemia, burns, thrombotic thrombopenic purpura, and fulminant malaria.'),
  'D005079': NtO(MH='Excipients', MS='Usually inert substances added to a prescription in order to provide suitable consistency to the dosage form. These include binders, matrix, base or diluent in pills, tablets, creams, salves, etc.'),
  'D005080': NtO(MH='Exercise Test', MS='Controlled physical activity which is performed in order to allow assessment of physiological functions, particularly cardiovascular and pulmonary, but also aerobic capacity. Maximal (most intense) exercise is usually required but submaximal exercise is also used.'),
  'D005081': NtO(MH='Exercise Therapy', MS='A regimen or plan of physical activities designed and prescribed for specific therapeutic goals. Its purpose is to restore normal musculoskeletal function or to reduce pain caused by diseases or injuries.'),
  'D005082': NtO(MH='Physical Exertion', MS='Expenditure of energy during PHYSICAL ACTIVITY. Intensity of exertion may be measured by rate of OXYGEN CONSUMPTION; HEAT produced, or HEART RATE. Perceived exertion, a psychological measure of exertion, is included.'),
  'D005083': NtO(MH='Exfoliatins', MS='Protein exotoxins from Staphylococcus aureus, phage type II, which cause epidermal necrolysis. They are proteins with a molecular weight of 26,000 to 32,000. They cause a condition variously called scaled skin, Lyell or Ritter syndrome, epidermal exfoliative disease, toxic epidermal necrolysis, etc.'),
  'D005084': NtO(MH='Exhibitionism', MS='A disorder in which fantasies about or the act of exposing the genitals to an unsuspecting stranger produces sexual excitement with no attempt at further sexual activity with the stranger.'),
  'D005085': NtO(MH='Exhibitions as Topic', MS='Works about discussions, descriptions or catalogs of public displays or items representative of a given subject.'),
  'D005086': NtO(MH='Existentialism', MS="Philosophy based on the analysis of the individual's existence in the world which holds that human existence cannot be completely described in scientific terms. Existentialism also stresses the freedom and responsibility of the individual as well as the uniqueness of religious and ethical experiences and the analysis of subjective phenomena such as anxiety, guilt, and suffering. (APA, Thesaurus of Psychological Index Terms, 8th ed.)"),
  'D005087': NtO(MH='Glucan 1,4-alpha-Glucosidase', MS='An enzyme that catalyzes the hydrolysis of terminal 1,4-linked alpha-D-glucose residues successively from non-reducing ends of polysaccharide chains with the release of beta-glucose. It is also able to hydrolyze 1,6-alpha-glucosidic bonds when the next bond in sequence is 1,4.'),
  'D005088': NtO(MH='Exocrine Glands', MS="Glands of external secretion that release its secretions to the body's cavities, organs, or surface, through a duct."),
  'D005089': NtO(MH='Exocytosis', MS='Cellular release of material within membrane-limited vesicles by fusion of the vesicles with the CELL MEMBRANE.'),
  'D005090': NtO(MH='Exodeoxyribonucleases', MS="A family of enzymes that catalyze the exonucleolytic cleavage of DNA. It includes members of the class EC 3.1.11 that produce 5'-phosphomonoesters as cleavage products."),
  'D005091': NtO(MH='Exons', MS='The parts of a transcript of a split GENE remaining after the INTRONS are removed. They are spliced together to become a MESSENGER RNA or other functional RNA.'),
  'D005092': NtO(MH='Exonucleases', MS='Enzymes that catalyze the release of mononucleotides by the hydrolysis of the terminal bond of deoxyribonucleotide or ribonucleotide chains.'),
  'D005093': NtO(MH='Exophiala', MS='A normally saprophytic mitosporic Chaetothyriales fungal genus. Infections in humans include PHAEOHYPHOMYCOSIS; and PERITONITIS.. Exophiala jeanselmei (previously Phialophora jeanselmei) is an etiological agent of MYCETOMA.'),
  'D005094': NtO(MH='Exophthalmos', MS='Abnormal protrusion of both eyes; may be caused by endocrine gland malfunction, malignancy, injury, or paralysis of the extrinsic muscles of the eye.'),
  'D005095': NtO(MH='Exoribonucleases', MS='A family of enzymes that catalyze the exonucleolytic cleavage of RNA. It includes EC 3.1.13.-, EC 3.1.14.-, EC 3.1.15.-, and EC 3.1.16.-. EC 3.1.-'),
  'D005096': NtO(MH='Exostoses', MS='Benign hypertrophy that projects outward from the surface of bone, often containing a cartilaginous component.'),
  'D005097': NtO(MH='Exostoses, Multiple Hereditary', MS='Hereditary disorder transmitted by an autosomal dominant gene and characterized by multiple exostoses (multiple osteochondromas) near the ends of long bones. The genetic abnormality results in a defect in the osteoclastic activity at the metaphyseal ends of the bone during the remodeling process in childhood or early adolescence. The metaphyses develop benign, bony outgrowths often capped by cartilage. A small number undergo neoplastic transformation.'),
  'D005098': NtO(MH='Exotoxins', MS='Toxins produced, especially by bacterial or fungal cells, and released into the culture medium or environment.'),
  'D005099': NtO(MH='Exotropia', MS='A form of ocular misalignment where the visual axes diverge inappropriately. For example, medial rectus muscle weakness may produce this condition as the affected eye will deviate laterally upon attempted forward gaze. An exotropia occurs due to the relatively unopposed force exerted on the eye by the lateral rectus muscle, which pulls the eye in an outward direction.'),
  'D005100': NtO(MH='Expectorants', MS='Agents that increase mucous excretion. Mucolytic agents, that is drugs that liquefy mucous secretions, are also included here.'),
  'D005101': NtO(MH='Expeditions', MS='Usually refers to planned scientific data-gathering excursions.'),
  'D005102': NtO(MH='Health Expenditures', MS='The amounts spent by individuals, groups, nations, or private or public organizations for total health care and/or its various components. These amounts may or may not be equivalent to the actual costs (HEALTH CARE COSTS) and may or may not be shared among the patient, insurers, and/or employers.'),
  'D005103': NtO(MH='Expert Systems', MS='Computer programs based on knowledge developed from consultation with experts on a problem, and the processing and/or formalizing of this knowledge using these programs in such a manner that the problems may be solved.'),
  'D005104': NtO(MH='Expert Testimony', MS='Presentation of pertinent data by one with special skill or knowledge representing mastery of a particular subject.'),
  'D005105': NtO(MH='Expiratory Reserve Volume', MS='The extra volume of air that can be expired with maximum effort beyond the level reached at the end of a normal, quiet expiration. Common abbreviation is ERV.'),
  'D005106': NtO(MH='Exploratory Behavior', MS='The tendency to explore or investigate a novel environment. It is considered a motivation not clearly distinguishable from curiosity.'),
  'D005107': NtO(MH='Explosions', MS='Sudden, violent, and often destructive expansion of gases which propagates energy outward, such as a shock wave, ejecting fragments and debris at high velocities.'),
  'D005108': NtO(MH='Extinction, Psychological', MS='The procedure of presenting the conditioned stimulus without REINFORCEMENT to an organism previously conditioned. It refers also to the diminution of a conditioned response resulting from this procedure.'),
  'D005109': NtO(MH='Extracellular Matrix', MS='A meshwork-like substance found within the extracellular space and in association with the basement membrane of the cell surface. It promotes cellular proliferation and provides a supporting structure to which cells or cell lysates in culture dishes adhere.'),
  'D005110': NtO(MH='Extracellular Space', MS='Interstitial space between cells, occupied by INTERSTITIAL FLUID as well as amorphous and fibrous substances. For organisms with a CELL WALL, the extracellular space includes everything outside of the CELL MEMBRANE including the PERIPLASM and the cell wall.'),
  'D005111': NtO(MH='Extrachromosomal Inheritance', MS='Vertical transmission of hereditary characters by DNA from cytoplasmic organelles such as MITOCHONDRIA; CHLOROPLASTS; and PLASTIDS, or from PLASMIDS or viral episomal DNA.'),
  'D005112': NtO(MH='Extracorporeal Circulation', MS='Diversion of blood flow through a circuit located outside the body but continuous with the bodily circulation.'),
  'D005113': NtO(MH='Extraction, Obstetrical', MS='Extraction of the fetus by means of obstetrical instruments.'),
  'D005114': NtO(MH='Extramarital Relations', MS='Voluntary SEXUAL INTERCOURSE between a married person and someone other than the SPOUSE.'),
  'D005115': NtO(MH='Extraoral Traction Appliances', MS='Extraoral devices for applying force to the dentition in order to avoid some of the problems in anchorage control met with in intermaxillary traction and to apply force in directions not otherwise possible.'),
  'D005116': NtO(MH='Extrapyramidal Tracts', MS='Uncrossed tracts of motor nerves from the brain to the anterior horns of the spinal cord, involved in reflexes, locomotion, complex movements, and postural control.'),
  'D005117': NtO(MH='Cardiac Complexes, Premature', MS='A group of cardiac arrhythmias in which the cardiac contractions are not initiated at the SINOATRIAL NODE. They include both atrial and ventricular premature beats, and are also known as extra or ectopic heartbeats. Their frequency is increased in heart diseases.'),
  'D005118': NtO(MH='Extraterrestrial Environment', MS='The environment outside the earth or its atmosphere. The environment may refer to a closed cabin (such as a space shuttle or space station) or to space itself, the moon, or other planets.'),
  'D005119': NtO(MH='Extravasation of Diagnostic and Therapeutic Materials', MS='The escape of diagnostic or therapeutic material from the vessel into which it is introduced into the surrounding tissue or body cavity.'),
  'D005120': NtO(MH='Extraversion, Psychological', MS='A state in which attention is largely directed outward from the self.'),
  'D005121': NtO(MH='Extremities', MS='The farthest or outermost projections of the body, such as the HAND and FOOT.'),
  'D005122': NtO(MH='Exudates and Transudates', MS='Exudates are fluids, CELLS, or other cellular substances that are slowly discharged from BLOOD VESSELS usually from inflamed tissues. Transudates are fluids that pass through a membrane or squeeze through tissue or into the EXTRACELLULAR SPACE of TISSUES. Transudates are thin and watery and contain few cells or PROTEINS.'),
  'D005123': NtO(MH='Eye', MS='The organ of sight constituting a pair of globular organs made up of a three-layered roughly spherical structure specialized for receiving and responding to light.'),
  'D005124': NtO(MH='Eye Abnormalities', MS='Congenital absence of or defects in structures of the eye; may also be hereditary.'),
  'D005125': NtO(MH='Eye Banks', MS='Centers for storing various parts of the eye for future use.'),
  'D005126': NtO(MH='Eye Burns', MS='Injury to any part of the eye by extreme heat, chemical agents, or ultraviolet radiation.'),
  'D005127': NtO(MH='Eye Color', MS='Color of the iris.'),
  'D005128': NtO(MH='Eye Diseases', MS='Diseases affecting the eye.'),
  'D005129': NtO(MH='Eye Foreign Bodies', MS='Inanimate objects that become enclosed in the eye.'),
  'D005130': NtO(MH='Eye Hemorrhage', MS='Intraocular hemorrhage from the vessels of various tissues of the eye.'),
  'D005131': NtO(MH='Eye Injuries', MS='Damage or trauma inflicted to the eye by external means. The concept includes both surface injuries and intraocular injuries.'),
  'D005132': NtO(MH='Eye Manifestations', MS='Ocular disorders attendant upon non-ocular disease or injury.'),
  'D005133': NtO(MH='Eye Movements', MS='Voluntary or reflex-controlled movements of the eye.'),
  'D005134': NtO(MH='Eye Neoplasms', MS='Tumors or cancer of the EYE.'),
  'D005135': NtO(MH='Eye Protective Devices', MS='Personal devices for protection of the eyes from impact, flying objects, glare, liquids, or injurious radiation.'),
  'D005136': NtO(MH='Eye Proteins', MS='PROTEINS derived from TISSUES of the EYE.'),
  'D005137': NtO(MH='Eye, Artificial', MS='A ready-made or custom-made prosthesis of glass or plastic shaped and colored to resemble the anterior portion of a normal eye and used for cosmetic reasons. It is attached to the anterior portion of an orbital implant (ORBITAL IMPLANTS) which is placed in the socket of an enucleated or eviscerated eye. (From Dorland, 28th ed)'),
  'D005138': NtO(MH='Eyebrows', MS='Curved rows of HAIR located on the upper edges of the eye sockets.'),
  'D005139': NtO(MH='Eyeglasses', MS='A pair of ophthalmic lenses in a frame or mounting which is supported by the nose and ears. The purpose is to aid or improve vision. It does not include goggles or nonprescription sun glasses for which EYE PROTECTIVE DEVICES is available.'),
  'D005140': NtO(MH='Eyelashes', MS='The hairs which project from the edges of the EYELIDS.'),
  'D005141': NtO(MH='Eyelid Diseases', MS='Diseases involving the EYELIDS.'),
  'D005142': NtO(MH='Eyelid Neoplasms', MS='Tumors of cancer of the EYELIDS.'),
  'D005143': NtO(MH='Eyelids', MS='Each of the upper and lower folds of SKIN which cover the EYE when closed.'),
  'D005144': NtO(MH='F Factor', MS='A plasmid whose presence in the cell, either extrachromosomal or integrated into the BACTERIAL CHROMOSOME, determines the "sex" of the bacterium, host chromosome mobilization, transfer via conjugation (CONJUGATION, GENETIC) of genetic material, and the formation of SEX PILI.'),
  'D005145': NtO(MH='Face', MS='The anterior portion of the head that includes the skin, muscles, and structures of the forehead, eyes, nose, mouth, cheeks, and jaw.'),
  'D005146': NtO(MH='Facial Asymmetry', MS='Congenital or acquired asymmetry of the face.'),
  'D005147': NtO(MH='Facial Bones', MS='The facial skeleton, consisting of bones situated between the cranial base and the mandibular region. While some consider the facial bones to comprise the hyoid (HYOID BONE), palatine (HARD PALATE), and zygomatic (ZYGOMA) bones, MANDIBLE, and MAXILLA, others include also the lacrimal and nasal bones, inferior nasal concha, and vomer but exclude the hyoid bone. (Jablonski, Dictionary of Dentistry, 1992, p113)'),
  'D005148': NtO(MH='Facial Dermatoses', MS='Skin diseases involving the FACE.'),
  'D005149': NtO(MH='Facial Expression', MS='Observable changes of expression in the face in response to emotional stimuli.'),
  'D005150': NtO(MH='Facial Hemiatrophy', MS='A syndrome characterized by slowly progressive unilateral atrophy of facial subcutaneous fat, muscle tissue, skin, cartilage, and bone. The condition typically progresses over a period of 2-10 years and then stabilizes.'),
  'D005151': NtO(MH='Facial Injuries', MS='General or unspecified injuries to the soft tissue or bony portions of the face.'),
  'D005152': NtO(MH='Facial Muscles', MS='Muscles of facial expression or mimetic muscles that include the numerous muscles supplied by the facial nerve that are attached to and move the skin of the face. (From Stedman, 25th ed)'),
  'D005153': NtO(MH='Facial Neoplasms', MS='New abnormal growth of tissue in the FACE.'),
  'D005154': NtO(MH='Facial Nerve', MS='The 7th cranial nerve. The facial nerve has two parts, the larger motor root which may be called the facial nerve proper, and the smaller intermediate or sensory root. Together they provide efferent innervation to the muscles of facial expression and to the lacrimal and SALIVARY GLANDS, and convey afferent information for TASTE from the anterior two-thirds of the TONGUE and for TOUCH from the EXTERNAL EAR.'),
  'D005155': NtO(MH='Facial Nerve Diseases', MS='Diseases of the facial nerve or nuclei. Pontine disorders may affect the facial nuclei or nerve fascicle. The nerve may be involved intracranially, along its course through the petrous portion of the temporal bone, or along its extracranial course. Clinical manifestations include facial muscle weakness, loss of taste from the anterior tongue, hyperacusis, and decreased lacrimation.'),
  'D005156': NtO(MH='Facial Neuralgia', MS='Neuralgic syndromes which feature chronic or recurrent FACIAL PAIN as the primary manifestation of disease. Disorders of the trigeminal and facial nerves are frequently associated with these conditions.'),
  'D005157': NtO(MH='Facial Pain', MS='Pain in the facial region including orofacial pain and craniofacial pain. Associated conditions include local inflammatory and neoplastic disorders and neuralgic syndromes involving the trigeminal, facial, and glossopharyngeal nerves. Conditions which feature recurrent or persistent facial pain as the primary manifestation of disease are referred to as FACIAL PAIN SYNDROMES.'),
  'D005158': NtO(MH='Facial Paralysis', MS='Severe or complete loss of facial muscle motor function. This condition may result from central or peripheral lesions. Damage to CNS motor pathways from the cerebral cortex to the facial nuclei in the pons leads to facial weakness that generally spares the forehead muscles. FACIAL NERVE DISEASES generally results in generalized hemifacial weakness. NEUROMUSCULAR JUNCTION DISEASES and MUSCULAR DISEASES may also cause facial paralysis or paresis.'),
  'D005159': NtO(MH='Health Care Facilities, Manpower, and Services', MS='The services provided in the delivery of health care, associated facilities in health care, and attendant manpower required or available.'),
  'D005160': NtO(MH='Facility Design and Construction', MS='Architecture, exterior and interior design, and construction of facilities other than hospitals, e.g., dental schools, medical schools, ambulatory care clinics, and specified units of health care facilities. The concept also includes architecture, design, and construction of specialized contained, controlled, or closed research environments including those of space labs and stations.'),
  'D005161': NtO(MH='Facility Regulation and Control', MS='Formal voluntary or governmental procedures and standards required of hospitals and health or other facilities to improve operating efficiency, and for the protection of the consumer.'),
  'D005162': NtO(MH='Factitious Disorders', MS='Disorders characterized by physical or psychological symptoms that are not real, genuine, or natural.'),
  'D005163': NtO(MH='Factor Analysis, Statistical', MS='A set of statistical methods for analyzing the correlations among several variables in order to estimate the number of fundamental dimensions that underlie the observed data and to describe and measure those dimensions. It is used frequently in the development of scoring systems for rating scales and questionnaires.'),
  'D005164': NtO(MH='Factor IX', MS='Storage-stable blood coagulation factor acting in the intrinsic pathway of blood coagulation. Its activated form, IXa, forms a complex with factor VIII and calcium on platelet factor 3 to activate factor X to Xa. Deficiency of factor IX results in HEMOPHILIA B (Christmas Disease).'),
  'D005165': NtO(MH='Factor V', MS="Heat- and storage-labile plasma glycoprotein which accelerates the conversion of prothrombin to thrombin in blood coagulation. Factor V accomplishes this by forming a complex with factor Xa, phospholipid, and calcium (prothrombinase complex). Deficiency of factor V leads to Owren's disease."),
  'D005166': NtO(MH='Factor V Deficiency', MS="A deficiency of blood coagulation factor V (known as proaccelerin or accelerator globulin or labile factor) leading to a rare hemorrhagic tendency known as Owren's disease or parahemophilia. It varies greatly in severity. Factor V deficiency is an autosomal recessive trait. (Dorland, 27th ed)"),
  'D005167': NtO(MH='Factor VII', MS='Heat- and storage-stable plasma protein that is activated by tissue thromboplastin to form factor VIIa in the extrinsic pathway of blood coagulation. The activated form then catalyzes the activation of factor X to factor Xa.'),
  'D005168': NtO(MH='Factor VII Deficiency', MS='An autosomal recessive characteristic or a coagulation disorder acquired in association with VITAMIN K DEFICIENCY. FACTOR VII is a Vitamin K dependent glycoprotein essential to the extrinsic pathway of coagulation.'),
  'D005169': NtO(MH='Factor VIII', MS='Factor VIII of blood coagulation. Antihemophilic factor that is part of the factor VIII/von Willebrand factor complex. Factor VIII is produced in the liver and acts in the intrinsic pathway of blood coagulation. It serves as a cofactor in factor X activation and this action is markedly enhanced by small amounts of thrombin.'),
  'D005170': NtO(MH='Factor X', MS='Storage-stable glycoprotein blood coagulation factor that can be activated to factor Xa by both the intrinsic and extrinsic pathways. A deficiency of factor X, sometimes called Stuart-Prower factor deficiency, may lead to a systemic coagulation disorder.'),
  'D005171': NtO(MH='Factor X Deficiency', MS='Blood coagulation disorder usually inherited as an autosomal recessive trait, though it can be acquired. It is characterized by defective activity in both the intrinsic and extrinsic pathways, impaired thromboplastin time, and impaired prothrombin consumption.'),
  'D005172': NtO(MH='Factor XI', MS='Stable blood coagulation factor involved in the intrinsic pathway. The activated form XIa activates factor IX to IXa. Deficiency of factor XI is often called hemophilia C.'),
  'D005173': NtO(MH='Factor XI Deficiency', MS="A hereditary deficiency of blood coagulation factor XI (also known as plasma thromboplastin antecedent or PTA or antihemophilic factor C) resulting in a systemic blood-clotting defect called hemophilia C or Rosenthal's syndrome, that may resemble classical hemophilia."),
  'D005174': NtO(MH='Factor XII', MS='Stable blood coagulation factor activated by contact with the subendothelial surface of an injured vessel. Along with prekallikrein, it serves as the contact factor that initiates the intrinsic pathway of blood coagulation. Kallikrein activates factor XII to XIIa. Deficiency of factor XII, also called the Hageman trait, leads to increased incidence of thromboembolic disease. Mutations in the gene for factor XII that appear to increase factor XII amidolytic activity are associated with HEREDITARY ANGIOEDEMA TYPE III.'),
  'D005175': NtO(MH='Factor XII Deficiency', MS='An absence or reduced level of blood coagulation factor XII. It normally occurs in the absence of patient or family history of hemorrhagic disorders and is marked by prolonged clotting time.'),
  'D005176': NtO(MH='Factor XIII', MS='A fibrin-stabilizing plasma enzyme (TRANSGLUTAMINASES) that is activated by THROMBIN and CALCIUM to form FACTOR XIIIA. It is important for stabilizing the formation of the fibrin polymer (clot) which culminates the coagulation cascade.'),
  'D005177': NtO(MH='Factor XIII Deficiency', MS='A deficiency of blood coagulation FACTOR XIII or fibrin stabilizing factor (FSF) that prevents blood clot formation and results in a clinical hemorrhagic diathesis.'),
  'D005178': NtO(MH='Faculty', MS='TEACHING and administrative staff having academic rank in a post-secondary educational institution.'),
  'D005179': NtO(MH='Faculty, Dental', MS='The teaching staff and members of the administrative staff having academic rank in a dental school.'),
  'D005180': NtO(MH='Faculty, Medical', MS='Teaching and administrative staff having academic rank in a medical school.'),
  'D005181': NtO(MH='Faculty, Nursing', MS='The teaching staff and members of the administrative staff having academic rank in a nursing school.'),
  'D005182': NtO(MH='Flavin-Adenine Dinucleotide', MS='A condensation product of riboflavin and adenosine diphosphate. The coenzyme of various aerobic dehydrogenases, e.g., D-amino acid oxidase and L-amino acid oxidase. (Lehninger, Principles of Biochemistry, 1982, p972)'),
  'D005183': NtO(MH='Failure to Thrive', MS='A condition of substandard growth or diminished capacity to maintain normal function.'),
  'D005184': NtO(MH='Fallopian Tube Diseases', MS='Diseases involving the FALLOPIAN TUBES including neoplasms (FALLOPIAN TUBE NEOPLASMS); SALPINGITIS; tubo-ovarian abscess; and blockage.'),
  'D005185': NtO(MH='Fallopian Tube Neoplasms', MS='Benign or malignant neoplasms of the FALLOPIAN TUBES. They are uncommon. If they develop, they may be located in the wall or within the lumen as a growth attached to the wall by a stalk.'),
  'D005186': NtO(MH='Fallopian Tube Patency Tests', MS='Methods for assessing the patency of the fallopian tubes.'),
  'D005187': NtO(MH='Fallopian Tubes', MS='A pair of highly specialized muscular canals extending from the UTERUS to its corresponding OVARY. They provide the means for OVUM collection, and the site for the final maturation of gametes and FERTILIZATION. The fallopian tube consists of an interstitium, an isthmus, an ampulla, an infundibulum, and fimbriae. Its wall consists of three histologic layers: serous, muscular, and an internal mucosal layer lined with both ciliated and secretory cells.'),
  'D005188': NtO(MH='False Negative Reactions', MS='Negative test results in subjects who possess the attribute for which the test is conducted. The labeling of diseased persons as healthy when screening in the detection of disease. (Last, A Dictionary of Epidemiology, 2d ed)'),
  'D005189': NtO(MH='False Positive Reactions', MS='Positive test results in subjects who do not possess the attribute for which the test is conducted. The labeling of healthy persons as diseased when screening in the detection of disease. (Last, A Dictionary of Epidemiology, 2d ed)'),
  'D005190': NtO(MH='Family', MS='A social group consisting of parents or parent substitutes and children.'),
  'D005191': NtO(MH='Family Characteristics', MS='Size and composition of the family.'),
  'D005192': NtO(MH='Family Health', MS='The health status of the family as a unit including the impact of the health of one member of the family on the family as a unit and on individual family members; also, the impact of family organization or disorganization on the health status of its members.'),
  'D005193': NtO(MH='Family Planning Services', MS='Health care programs or services designed to assist individuals in the planning of family size. Various methods of CONTRACEPTION can be used to control the number and timing of childbirths.'),
  'D005194': NtO(MH='Family Practice', MS='A medical specialty concerned with the provision of continuing, comprehensive primary health care for the entire family.'),
  'D005195': NtO(MH='Family Relations', MS='Behavioral, psychological, and social relations among various members of the nuclear family and the extended family.'),
  'D005196': NtO(MH='Family Therapy', MS='A form of group psychotherapy. It involves treatment of more than one member of the family simultaneously in the same session.'),
  'D005197': NtO(MH='Famous Persons', MS='Individuals widely known or recognized.'),
  'D005198': NtO(MH='Fanconi Syndrome', MS='A hereditary or acquired form of generalized dysfunction of the PROXIMAL KIDNEY TUBULE without primary involvement of the KIDNEY GLOMERULUS. It is usually characterized by the tubular wasting of nutrients and salts (GLUCOSE; AMINO ACIDS; PHOSPHATES; and BICARBONATES) resulting in HYPOKALEMIA; ACIDOSIS; HYPERCALCIURIA; and PROTEINURIA.'),
  'D005199': NtO(MH='Fanconi Anemia', MS='Congenital disorder affecting all bone marrow elements, resulting in ANEMIA; LEUKOPENIA; and THROMBOPENIA, and associated with cardiac, renal, and limb malformations as well as dermal pigmentary changes. Spontaneous CHROMOSOME BREAKAGE is a feature of this disease along with predisposition to LEUKEMIA. There are at least 7 complementation groups in Fanconi anemia: FANCA, FANCB, FANCC, FANCD1, FANCD2, FANCE, FANCF, FANCG, and FANCL. (from Online Mendelian Inheritance in Man, http://www.ncbi.nlm.nih.gov/entrez/dispomim.cgi?id=227650, August 20, 2004)'),
  'D005200': NtO(MH='FANFT', MS='A potent nitrofuran derivative tumor initiator. It causes bladder tumors in all animals studied and is mutagenic to many bacteria.'),
  'D005201': NtO(MH='Fantasy', MS='An imagined sequence of events or mental images, e.g., daydreams.'),
  'D005202': NtO(MH='Far East', MS='A geographic area of east and southeast Asia encompassing CHINA; HONG KONG; JAPAN; KOREA; MACAO; MONGOLIA; and TAIWAN.'),
  'D005203': NtO(MH="Farmer's Lung", MS='A form of alveolitis or pneumonitis due to an acquired hypersensitivity to inhaled antigens associated with farm environment. Antigens in the farm dust are commonly from bacteria actinomycetes (SACCHAROPOLYSPORA and THERMOACTINOMYCES), fungi, and animal proteins in the soil, straw, crops, pelts, serum, and excreta.'),
  'D005204': NtO(MH='Farnesol', MS='A colorless liquid extracted from oils of plants such as citronella, neroli, cyclamen, and tuberose. It is an intermediate step in the biological synthesis of cholesterol from mevalonic acid in vertebrates. It has a delicate odor and is used in perfumery. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D005205': NtO(MH='Fascia', MS='Layers of connective tissue of variable thickness. The superficial fascia is found immediately below the skin; the deep fascia invests MUSCLES, nerves, and other organs.'),
  'D005206': NtO(MH='Fascia Lata', MS='CONNECTIVE TISSUE of the anterior compartment of the THIGH that has its origins on the anterior aspect of the iliac crest and anterior superior iliac spine, and its insertion point on the iliotibial tract. It plays a role in medial rotation of the THIGH, steadying the trunk, and in KNEE extension.'),
  'D005207': NtO(MH='Fasciculation', MS='Involuntary contraction of the muscle fibers innervated by a motor unit. Fasciculations may be visualized as a muscle twitch or dimpling under the skin, but usually do not generate sufficient force to move a limb. They may represent a benign condition or occur as a manifestation of MOTOR NEURON DISEASE or PERIPHERAL NERVOUS SYSTEM DISEASES. (Adams et al., Principles of Neurology, 6th ed, p1294)'),
  'D005208': NtO(MH='Fasciitis', MS='Inflammation of the fascia. There are three major types: 1, Eosinophilic fasciitis, an inflammatory reaction with eosinophilia, producing hard thickened skin with an orange-peel configuration suggestive of scleroderma and considered by some a variant of scleroderma; 2, Necrotizing fasciitis (FASCIITIS, NECROTIZING), a serious fulminating infection (usually by a beta hemolytic streptococcus) causing extensive necrosis of superficial fascia; 3, Nodular/Pseudosarcomatous /Proliferative fasciitis, characterized by a rapid growth of fibroblasts with mononuclear inflammatory cells and proliferating capillaries in soft tissue, often the forearm; it is not malignant but is sometimes mistaken for fibrosarcoma.'),
  'D005209': NtO(MH='Fasciola', MS='A genus of trematode liver flukes of the family Fasciolidae. Two species of this genus are F. hepatica and F. gigantica. The parasites are found in the liver and gallbladder and associated ducts in mammals and occasionally man. F. gigantica occurs rarely in man.'),
  'D005210': NtO(MH='Fasciola hepatica', MS='A species of helminth commonly called the sheep liver fluke. It occurs in the biliary passages, liver, and gallbladder during various stages of development. Snails and aquatic vegetation are the intermediate hosts. Occasionally seen in man, it is most common in sheep and cattle.'),
  'D005211': NtO(MH='Fascioliasis', MS='Liver disease caused by infections with parasitic flukes of the genus FASCIOLA, such as FASCIOLA HEPATICA.'),
  'D005212': NtO(MH='Fasciolidae', MS='A family of flukes of the class Trematoda occurring primarily in the liver of animals and man. There are six genera: Fasciola, Fasciolopsis, Fascioloides, Tenuifasciola, Parafasciolopsis, and Protofasciola. The adult form of Fasciolopsis occurs in the intestines of pigs and man.'),
  'D005213': NtO(MH='Fascioloidiasis', MS='Infection of cattle and other herbivores with the giant liver fluke Fascioloides magna. It is characterized by extensive destruction of the liver parenchyma.'),
  'D005214': NtO(MH='Fast Neutrons', MS='Neutrons, the energy of which exceeds some arbitrary level, usually around one million electron volts.'),
  'D005215': NtO(MH='Fasting', MS='Abstaining from FOOD.'),
  'D005216': NtO(MH='Fat Body', MS='A nutritional reservoir of fatty tissue found mainly in insects and amphibians.'),
  'D005217': NtO(MH='Fat Emulsions, Intravenous', MS='Emulsions of fats or lipids used primarily in parenteral feeding.'),
  'D005218': NtO(MH='Fat Necrosis', MS='A condition in which the death of adipose tissue results in neutral fats being split into fatty acids and glycerol.'),
  'D005219': NtO(MH='Father-Child Relations', MS='Interaction between the father and the child.'),
  'D005220': NtO(MH='Fathers', MS='Male parents, human or animal.'),
  'D005221': NtO(MH='Fatigue', MS='The state of weariness following a period of exertion, mental or physical, characterized by a decreased capacity for work and reduced efficiency to respond to stimuli.'),
  'D005222': NtO(MH='Mental Fatigue', MS='A condition of low alertness or cognitive impairment, usually associated with prolonged mental activities or stress.'),
  'D005223': NtO(MH='Fats', MS="The glyceryl esters of a fatty acid, or of a mixture of fatty acids. They are generally odorless, colorless, and tasteless if pure, but they may be flavored according to origin. Fats are insoluble in water, soluble in most organic solvents. They occur in animal and vegetable tissue and are generally obtained by boiling or by extraction under pressure. They are important in the diet (DIETARY FATS) as a source of energy. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D005224': NtO(MH='Fats, Unsaturated', MS='Fats containing one or more double bonds, as from oleic acid, an unsaturated fatty acid.'),
  'D005227': NtO(MH='Fatty Acids', MS="Organic, monobasic acids derived from hydrocarbons by the equivalent of oxidation of a methyl group to an alcohol, aldehyde, and then acid. Fatty acids are saturated and unsaturated (FATTY ACIDS, UNSATURATED). (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D005228': NtO(MH='Fatty Acids, Essential', MS='Long chain organic acid molecules that must be obtained from the diet. Examples are LINOLEIC ACIDS and LINOLENIC ACIDS.'),
  'D005229': NtO(MH='Fatty Acids, Monounsaturated', MS='Fatty acids which are unsaturated in only one position.'),
  'D005230': NtO(MH='Fatty Acids, Nonesterified', MS='FATTY ACIDS found in the plasma that are complexed with SERUM ALBUMIN for transport. These fatty acids are not in glycerol ester form.'),
  'D005231': NtO(MH='Fatty Acids, Unsaturated', MS='FATTY ACIDS in which the carbon chain contains one or more double or triple carbon-carbon bonds.'),
  'D005232': NtO(MH='Fatty Acids, Volatile', MS='Short-chain fatty acids of up to six carbon atoms in length. They are the major end products of microbial fermentation in the ruminant digestive tract and have also been implicated in the causation of neurological diseases in humans.'),
  'D005233': NtO(MH='Fatty Alcohols', MS='Usually high-molecular-weight, straight-chain primary alcohols, but can also range from as few as 4 carbons, derived from natural fats and oils, including lauryl, stearyl, oleyl, and linoleyl alcohols. They are used in pharmaceuticals, cosmetics, detergents, plastics, and lube oils and in textile manufacture. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D005234': NtO(MH='Fatty Liver', MS='Lipid infiltration of the hepatic parenchymal cells resulting in a yellow-colored liver. The abnormal lipid accumulation is usually in the form of TRIGLYCERIDES, either as a single large droplet or multiple small droplets. Fatty liver is caused by an imbalance in the metabolism of FATTY ACIDS.'),
  'D005235': NtO(MH='Fatty Liver, Alcoholic', MS='Lipid infiltration of the hepatic parenchymal cells that is due to ALCOHOL ABUSE. The fatty changes in the alcoholic fatty liver may be reversible, depending on the amounts of TRIGLYCERIDES accumulated.'),
  'D005236': NtO(MH='Favism', MS='Hemolytic anemia due to the ingestion of fava beans or after inhalation of pollen from the Vicia fava plant by persons with glucose-6-phosphate dehydrogenase deficient erythrocytes.'),
  'D005239': NtO(MH='Fear', MS='The affective response to an actual current external danger which subsides with the elimination of the threatening condition.'),
  'D005240': NtO(MH='Feasibility Studies', MS='Studies to determine the advantages or disadvantages, practicability, or capability of accomplishing a projected plan, study, or project.'),
  'D005241': NtO(MH='Feathers', MS='Flat keratinous structures found on the skin surface of birds. Feathers are made partly of a hollow shaft fringed with barbs. They constitute the plumage.'),
  'D005242': NtO(MH='Fecal Incontinence', MS='Failure of voluntary control of the anal sphincters, with involuntary passage of feces and flatus.'),
  'D005243': NtO(MH='Feces', MS='Excrement from the INTESTINES, containing unabsorbed solids, waste products, secretions, and BACTERIA of the DIGESTIVE SYSTEM.'),
  'D005244': NtO(MH='Fecal Impaction', MS='Formation of a firm impassable mass of stool in the RECTUM or distal COLON.'),
  'D005245': NtO(MH='Fee Schedules', MS='A listing of established professional service charges, for specified dental and medical procedures.'),
  'D005246': NtO(MH='Feedback', MS='A mechanism of communication within a system in that the input signal generates an output response which returns to influence the continued activity or productivity of that system.'),
  'D005247': NtO(MH='Feeding Behavior', MS='Behavioral responses or sequences associated with eating including modes of feeding, rhythmic patterns of eating, and time intervals.'),
  'D005248': NtO(MH='Feeding Methods', MS='Methods of giving food to humans or animals.'),
  'D005249': NtO(MH='Fees and Charges', MS='Amounts charged to the patient as payer for health care services.'),
  'D005250': NtO(MH='Fees, Dental', MS='Amounts charged to the patient as payer for dental services.'),
  'D005251': NtO(MH='Fees, Medical', MS='Amounts charged to the patient as payer for medical services.'),
  'D005252': NtO(MH='Fees, Pharmaceutical', MS="Amounts charged to the patient or third-party payer for medication. It includes the pharmacist's professional fee and cost of ingredients, containers, etc."),
  'D005253': NtO(MH='Sarcoma Viruses, Feline', MS='Species of GAMMARETROVIRUS isolated from fibrosarcoma in cats. The viruses are actually recombinant feline leukemia viruses (FeLV) where part of the genome has been replaced by cellular oncogenes. It is unique to individuals and not transmitted naturally to other cats. FeSVs are replication defective and require FeLV to reproduce.'),
  'D005254': NtO(MH='Feline Panleukopenia', MS='A highly contagious DNA virus infection of the cat family, characterized by fever, enteritis and bone marrow changes. It is also called feline ataxia, feline agranulocytosis, feline infectious enteritis, cat fever, cat plague, and show fever. It is caused by FELINE PANLEUKOPENIA VIRUS or the closely related MINK ENTERITIS VIRUS or CANINE PARVOVIRUS.'),
  'D005256': NtO(MH='Leukemia Virus, Feline', MS='A species of GAMMARETROVIRUS causing leukemia, lymphosarcoma, immune deficiency, or other degenerative diseases in cats. Several cellular oncogenes confer on FeLV the ability to induce sarcomas (see also SARCOMA VIRUSES, FELINE).'),
  'D005257': NtO(MH='Fellowships and Scholarships', MS='Stipends or grants-in-aid granted by foundations or institutions to individuals for study.'),
  'D005258': NtO(MH='Felty Syndrome', MS='A rare complication of rheumatoid arthritis with autoimmune NEUTROPENIA; and SPLENOMEGALY.'),
  'D005259': NtO(MH='Felypressin', MS='A synthetic analog of LYPRESSIN with a PHENYLALANINE substitution at residue 2. Felypressin is a vasoconstrictor with reduced antidiuretic activity.'),
  'D005260': NtO(MH='Female', MS=None),
  'D005261': NtO(MH='Female Urogenital Diseases and Pregnancy Complications', MS='Pathological processes of the female URINARY TRACT, the reproductive system (GENITALIA, FEMALE), and disorders related to PREGNANCY.'),
  'D005262': NtO(MH='Feminization', MS='Development of female secondary SEX CHARACTERISTICS in the MALE. It is due to the effects of estrogenic metabolites of precursors from endogenous or exogenous sources, such as ADRENAL GLANDS or therapeutic drugs.'),
  'D005263': NtO(MH='Femoral Artery', MS='The main artery of the thigh, a continuation of the external iliac artery.'),
  'D005264': NtO(MH='Femoral Fractures', MS='Fractures of the femur.'),
  'D005265': NtO(MH='Femoral Neck Fractures', MS='Fractures of the short, constricted portion of the thigh bone between the femur head and the trochanters. It excludes intertrochanteric fractures which are HIP FRACTURES.'),
  'D005266': NtO(MH='Femoral Neoplasms', MS='New abnormal growth of tissue in the FEMUR.'),
  'D005267': NtO(MH='Femoral Nerve', MS='A nerve originating in the lumbar spinal cord (usually L2 to L4) and traveling through the lumbar plexus to provide motor innervation to extensors of the thigh and sensory innervation to parts of the thigh, lower leg, and foot, and to the hip and knee joints.'),
  'D005268': NtO(MH='Femoral Vein', MS='The vein accompanying the femoral artery in the same sheath; it is a continuation of the popliteal vein and becomes the external iliac vein.'),
  'D005269': NtO(MH='Femur', MS='The longest and largest bone of the skeleton, it is situated between the hip and the knee.'),
  'D005270': NtO(MH='Femur Head', MS='The hemispheric articular surface at the upper extremity of the thigh bone. (Stedman, 26th ed)'),
  'D005271': NtO(MH='Femur Head Necrosis', MS='Aseptic or avascular necrosis of the femoral head. The major types are idiopathic (primary), as a complication of fractures or dislocations, and LEGG-CALVE-PERTHES DISEASE.'),
  'D005272': NtO(MH='Femur Neck', MS='The constricted portion of the thigh bone between the femur head and the trochanters.'),
  'D005273': NtO(MH='Fenbendazole', MS='Antinematodal benzimidazole used in veterinary medicine.'),
  'D005275': NtO(MH='Fendiline', MS='Coronary vasodilator; inhibits calcium function in muscle cells in excitation-contraction coupling; proposed as antiarrhythmic and antianginal agents.'),
  'D005276': NtO(MH='Fenestration, Labyrinth', MS='The surgical creation of a new opening in the labyrinth of the ear for the restoration of hearing in cases of OTOSCLEROSIS. (Dorland, 27th ed)'),
  'D005277': NtO(MH='Fenfluramine', MS='A centrally active drug that apparently both blocks serotonin uptake and provokes transport-mediated serotonin release.'),
  'D005278': NtO(MH='Fenitrothion', MS='An organothiophosphate cholinesterase inhibitor that is used as an insecticide.'),
  'D005279': NtO(MH='Fenoprofen', MS='A propionic acid derivative that is used as a non-steroidal anti-inflammatory agent.'),
  'D005280': NtO(MH='Fenoterol', MS='A synthetic adrenergic beta-2 agonist that is used as a bronchodilator and tocolytic.'),
  'D005283': NtO(MH='Fentanyl', MS='A potent narcotic analgesic, abuse of which leads to habituation or addiction. It is primarily a mu-opioid agonist. Fentanyl is also used as an adjunct to general anesthetics, and as an anesthetic for induction and maintenance. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1078)'),
  'D005284': NtO(MH='Fenthion', MS='Potent cholinesterase inhibitor used as an insecticide and acaricide.'),
  'D005285': NtO(MH='Fermentation', MS='Anaerobic degradation of GLUCOSE or other organic nutrients to gain energy in the form of ATP. End products vary depending on organisms, substrates, and enzymatic pathways. Common fermentation products include ETHANOL and LACTIC ACID.'),
  'D005286': NtO(MH='Fermium', MS='A man-made radioactive actinide with atomic symbol Fm, and atomic number 100. Its known isotopes range from 244-254 and 256-258. Its valence can be +2 or +3. Like einsteinium, it was discovered in 1952 in the debris from a thermonuclear explosion.'),
  'D005287': NtO(MH='Ferredoxin-NADP Reductase', MS='An enzyme that catalyzes the oxidation and reduction of FERREDOXIN or ADRENODOXIN in the presence of NADP. EC 1.18.1.2 was formerly listed as EC 1.6.7.1 and EC 1.6.99.4.'),
  'D005288': NtO(MH='Ferredoxins', MS='Iron-containing proteins that transfer electrons, usually at a low potential, to flavoproteins; the iron is not present as in heme. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D005289': NtO(MH='Ferrets', MS='Semidomesticated variety of European polecat much used for hunting RODENTS and/or RABBITS and as a laboratory animal. It is in the subfamily Mustelinae, family MUSTELIDAE.'),
  'D005290': NtO(MH='Ferric Compounds', MS='Inorganic or organic compounds containing trivalent iron.'),
  'D005291': NtO(MH='Ferrichrome', MS='A cyclic peptide consisting of three residues of delta-N-hydroxy-delta-N-acetylornithine. It acts as an iron transport agent in Ustilago sphaerogena.'),
  'D005292': NtO(MH='Ferricyanides', MS='Inorganic salts of the hypothetical acid, H3Fe(CN)6.'),
  'D005293': NtO(MH='Ferritins', MS='Iron-containing proteins that are widely distributed in animals, plants, and microorganisms. Their major function is to store IRON in a nontoxic bioavailable form. Each ferritin molecule consists of ferric iron in a hollow protein shell (APOFERRITINS) made of 24 subunits of various sequences depending on the species and tissue types.'),
  'D005294': NtO(MH='Ferrochelatase', MS='A mitochondrial enzyme found in a wide variety of cells and tissues. It is the final enzyme in the 8-enzyme biosynthetic pathway of HEME. Ferrochelatase catalyzes ferrous insertion into protoporphyrin IX to form protoheme or heme. Deficiency in this enzyme results in ERYTHROPOIETIC PROTOPORPHYRIA.'),
  'D005295': NtO(MH='Ferrocyanides', MS='Inorganic salts of the hypothetical acid ferrocyanic acid (H4Fe(CN)6).'),
  'D005296': NtO(MH='Ferrous Compounds', MS='Inorganic or organic compounds that contain divalent iron.'),
  'D005297': NtO(MH='Ferrozine', MS='A ferroin compound that forms a stable magenta-colored solution with the ferrous ion. The complex has an absorption peak at 562 nm and is used as a reagent and indicator for iron.'),
  'D005298': NtO(MH='Fertility', MS='The capacity to conceive or to induce conception. It may refer to either the male or female.'),
  'D005299': NtO(MH='Fertility Agents', MS='Drugs used to increase fertility or to treat infertility.'),
  'D005300': NtO(MH='Fertility Agents, Female', MS='Compounds which increase the capacity to conceive in females.'),
  'D005303': NtO(MH='Fertility Agents, Male', MS='Compounds which increase the capacity of the male to induce conception.'),
  'D005306': NtO(MH='Fertilization', MS='The fusion of a spermatozoon (SPERMATOZOA) with an OVUM thus resulting in the formation of a ZYGOTE.'),
  'D005307': NtO(MH='Fertilization in Vitro', MS='An assisted reproductive technique that includes the direct handling and manipulation of oocytes and sperm to achieve fertilization in vitro.'),
  'D005308': NtO(MH='Fertilizers', MS='Substances or mixtures that are added to the soil to supply nutrients or to make available nutrients already present in the soil, in order to increase plant growth and productivity.'),
  'D005309': NtO(MH='Ferula', MS="A plant genus of the family APIACEAE. It contains pungent oils and resins. It is used to flavor curries, as a carminative, and as cat and dog repellent. The occasionally used common name of 'giant fennel' should not be confused with true fennel (FOENICULUM)."),
  'D005311': NtO(MH='Fetal Hypoxia', MS='Deficient oxygenation of FETAL BLOOD.'),
  'D005312': NtO(MH='Fetal Blood', MS='Blood of the fetus. Exchange of nutrients and waste between the fetal and maternal blood occurs via the PLACENTA. The cord blood is blood contained in the umbilical vessels (UMBILICAL CORD) at the time of delivery.'),
  'D005313': NtO(MH='Fetal Death', MS='Death of the developing young in utero. BIRTH of a dead FETUS is STILLBIRTH.'),
  'D005314': NtO(MH='Embryonic and Fetal Development', MS='Morphological and physiological development of EMBRYOS or FETUSES.'),
  'D005315': NtO(MH='Fetal Diseases', MS='Pathophysiological conditions of the FETUS in the UTERUS. Some fetal diseases may be treated with FETAL THERAPIES.'),
  'D005316': NtO(MH='Fetal Distress', MS='A nonreassuring fetal status (NRFS) indicating that the FETUS is compromised (American College of Obstetricians and Gynecologists 1988). It can be identified by sub-optimal values in FETAL HEART RATE; oxygenation of FETAL BLOOD; and other parameters.'),
  'D005317': NtO(MH='Fetal Growth Retardation', MS='Failure of a FETUS to attain expected GROWTH.'),
  'D005318': NtO(MH='Fetal Heart', MS='The heart of the fetus of any viviparous animal. It refers to the heart in the postembryonic period and is differentiated from the embryonic heart (HEART/embryology) only on the basis of time.'),
  'D005319': NtO(MH='Fetal Hemoglobin', MS='The major component of hemoglobin in the fetus. This HEMOGLOBIN has two alpha and two gamma polypeptide subunits in comparison to normal adult hemoglobin, which has two alpha and two beta polypeptide subunits. Fetal hemoglobin concentrations can be elevated (usually above 0.5%) in children and adults affected by LEUKEMIA and several types of ANEMIA.'),
  'D005320': NtO(MH='Fetal Macrosomia', MS='A condition of fetal overgrowth leading to a large-for-gestational-age FETUS. It is defined as BIRTH WEIGHT greater than 4,000 grams or above the 90th percentile for population and sex-specific growth curves. It is commonly seen in GESTATIONAL DIABETES; PROLONGED PREGNANCY; and pregnancies complicated by pre-existing diabetes mellitus.'),
  'D005321': NtO(MH='Extraembryonic Membranes', MS='The thin layers of tissue that surround the developing embryo. There are four extra-embryonic membranes commonly found in VERTEBRATES, such as REPTILES; BIRDS; and MAMMALS. They are the YOLK SAC, the ALLANTOIS, the AMNION, and the CHORION. These membranes provide protection and means to transport nutrients and wastes.'),
  'D005322': NtO(MH='Fetal Membranes, Premature Rupture', MS='Spontaneous tearing of the membranes surrounding the FETUS any time before the onset of OBSTETRIC LABOR. Preterm PROM is membrane rupture before 37 weeks of GESTATION.'),
  'D005323': NtO(MH='Fetal Monitoring', MS='Physiologic or biochemical monitoring of the fetus. It is usually done during LABOR, OBSTETRIC and may be performed in conjunction with the monitoring of uterine activity. It may also be performed prenatally as when the mother is undergoing surgery.'),
  'D005324': NtO(MH='Fetal Movement', MS='Physical activity of the FETUS in utero. Gross or fine fetal body movement can be monitored by the mother, PALPATION, or ULTRASONOGRAPHY.'),
  'D005325': NtO(MH='Fetal Organ Maturity', MS='Functional competence of specific organs or body systems of the FETUS in utero.'),
  'D005326': NtO(MH='Fetal Proteins', MS='Proteins that are preferentially expressed or upregulated during FETAL DEVELOPMENT.'),
  'D005327': NtO(MH='Fetal Resorption', MS='The disintegration and assimilation of the dead FETUS in the UTERUS at any stage after the completion of organogenesis which, in humans, is after the 9th week of GESTATION. It does not include embryo resorption (see EMBRYO LOSS).'),
  'D005328': NtO(MH='Fetal Viability', MS='The potential of the FETUS to survive outside the UTERUS after birth, natural or induced. Fetal viability depends largely on the FETAL ORGAN MATURITY, and environmental conditions.'),
  'D005329': NtO(MH='Fetishism, Psychiatric', MS='A condition in which inanimate objects are utilized as a preferred or exclusive method of stimulating erotic arousal.'),
  'D005330': NtO(MH='Fetofetal Transfusion', MS="Passage of blood from one fetus to another via an arteriovenous communication or other shunt, in a monozygotic twin pregnancy. It results in anemia in one twin and polycythemia in the other. (Lee et al., Wintrobe's Clinical Hematology, 9th ed, p737-8)"),
  'D005331': NtO(MH='Fetomaternal Transfusion', MS='Transplacental passage of fetal blood into the circulation of the maternal organism. (Dorland, 27th ed)'),
  'D005332': NtO(MH='Fetoscopy', MS='Endoscopic examination, therapy or surgery of the fetus and amniotic cavity through abdominal or uterine entry.'),
  'D005333': NtO(MH='Fetus', MS='The unborn young of a viviparous mammal, in the postembryonic period, after the major structures have been outlined. In humans, the unborn young from the end of the eighth week after CONCEPTION until BIRTH, as distinguished from the earlier EMBRYO, MAMMALIAN.'),
  'D005334': NtO(MH='Fever', MS='An abnormal elevation of body temperature, usually as a result of a pathologic process.'),
  'D005335': NtO(MH='Fever of Unknown Origin', MS='Fever in which the etiology cannot be ascertained.'),
  'D005336': NtO(MH='Fiber Optic Technology', MS='The technology of transmitting light over long distances through strands of glass or other transparent material.'),
  'D005337': NtO(MH='Fibrin', MS='A protein derived from FIBRINOGEN in the presence of THROMBIN, which forms part of the blood clot.'),
  'D005338': NtO(MH='Fibrin Fibrinogen Degradation Products', MS='Soluble protein fragments formed by the proteolytic action of plasmin on fibrin or fibrinogen. FDP and their complexes profoundly impair the hemostatic process and are a major cause of hemorrhage in intravascular coagulation and fibrinolysis.'),
  'D005339': NtO(MH='Fibrin Foam', MS='A dry artificial sterile sponge of fibrin prepared by clotting with thrombin a foam or solution of fibrinogen. It is used in conjunction with thrombin as a hemostatic in surgery at sites where bleeding cannot be controlled by more common methods. (From Martindale, The Extra Pharmacopoeia, 30th ed, p648)'),
  'D005340': NtO(MH='Fibrinogen', MS='Plasma glycoprotein clotted by thrombin, composed of a dimer of three non-identical pairs of polypeptide chains (alpha, beta, gamma) held together by disulfide bonds. Fibrinogen clotting is a sol-gel change involving complex molecular arrangements: whereas fibrinogen is cleaved by thrombin to form polypeptides A and B, the proteolytic action of other enzymes yields different fibrinogen degradation products.'),
  'D005341': NtO(MH='Fibrinolysin', MS='A product of the lysis of plasminogen (profibrinolysin) by PLASMINOGEN activators. It is composed of two polypeptide chains, light (B) and heavy (A), with a molecular weight of 75,000. It is the major proteolytic enzyme involved in blood clot retraction or the lysis of fibrin and quickly inactivated by antiplasmins.'),
  'D005342': NtO(MH='Fibrinolysis', MS='The natural enzymatic dissolution of FIBRIN.'),
  'D005343': NtO(MH='Fibrinolytic Agents', MS='Fibrinolysin or agents that convert plasminogen to FIBRINOLYSIN.'),
  'D005344': NtO(MH='Fibrinopeptide A', MS='Two small peptide chains removed from the N-terminal segment of the alpha chains of fibrinogen by the action of thrombin during the blood coagulation process. Each peptide chain contains 18 amino acid residues. In vivo, fibrinopeptide A is used as a marker to determine the rate of conversion of fibrinogen to fibrin by thrombin.'),
  'D005345': NtO(MH='Fibrinopeptide B', MS='Two small peptide chains removed from the N-terminal segment of the beta chains of fibrinogen by the action of thrombin. Each peptide chain contains 20 amino acid residues. The removal of fibrinopeptides B is not required for coagulation.'),
  'D005346': NtO(MH='Fibroblast Growth Factors', MS='A family of small polypeptide growth factors that share several common features including a strong affinity for HEPARIN, and a central barrel-shaped core region of 140 amino acids that is highly homologous between family members. Although originally studied as proteins that stimulate the growth of fibroblasts this distinction is no longer a requirement for membership in the fibroblast growth factor family.'),
  'D005347': NtO(MH='Fibroblasts', MS='Connective tissue cells which secrete an extracellular matrix rich in collagen and other macromolecules.'),
  'D005348': NtO(MH='Fibrocystic Breast Disease', MS='A common and benign breast disease characterized by varying degree of fibrocystic changes in the breast tissue. There are three major patterns of morphological changes, including FIBROSIS, formation of CYSTS, and proliferation of glandular tissue (adenosis). The fibrocystic breast has a dense irregular, lumpy, bumpy consistency.'),
  'D005349': NtO(MH='Fibroins', MS='Fibrous proteins secreted by INSECTS and SPIDERS. Generally, the term refers to silkworm fibroin secreted by the silk gland cells of SILKWORMS, Bombyx mori. Spider fibroins are called spidroins or dragline silk fibroins.'),
  'D005350': NtO(MH='Fibroma', MS='A benign tumor of fibrous or fully developed connective tissue.'),
  'D005351': NtO(MH='Fibromatosis, Gingival', MS='Generalized or localized diffuse fibrous overgrowth of the gingival tissue, usually transmitted as an autosomal dominant trait, but some cases are idiopathic and others produced by drugs. The enlarged gingiva is pink, firm, and has a leather-like consistency with a minutely pebbled surface and in severe cases the teeth are almost completely covered and the enlargement projects into the oral vestibule. (Dorland, 28th ed)'),
  'D005352': NtO(MH='Fibromuscular Dysplasia', MS='An idiopathic, segmental, nonatheromatous disease of the musculature of arterial walls, leading to STENOSIS of small and medium-sized arteries. There is true proliferation of SMOOTH MUSCLE CELLS and fibrous tissue. Fibromuscular dysplasia lesions are smooth stenosis and occur most often in the renal and carotid arteries. They may also occur in other peripheral arteries of the extremity.'),
  'D005353': NtO(MH='Fibronectins', MS='Glycoproteins found on the surfaces of cells, particularly in fibrillar structures. The proteins are lost or reduced when these cells undergo viral or chemical transformation. They are highly susceptible to proteolysis and are substrates for activated blood coagulation factor VIII. The forms present in plasma are called cold-insoluble globulins.'),
  'D005354': NtO(MH='Fibrosarcoma', MS='A sarcoma derived from deep fibrous tissue, characterized by bundles of immature proliferating fibroblasts with variable collagen formation, which tends to invade locally and metastasize by the bloodstream. (Stedman, 25th ed)'),
  'D005355': NtO(MH='Fibrosis', MS='Any pathological condition where fibrous connective tissue invades any organ, usually as a consequence of inflammation or other injury.'),
  'D005356': NtO(MH='Fibromyalgia', MS='A common nonarticular rheumatic syndrome characterized by myalgia and multiple points of focal muscle tenderness to palpation (trigger points). Muscle pain is typically aggravated by inactivity or exposure to cold. This condition is often associated with general symptoms, such as sleep disturbances, fatigue, stiffness, HEADACHES, and occasionally DEPRESSION. There is significant overlap between fibromyalgia and the chronic fatigue syndrome (FATIGUE SYNDROME, CHRONIC). Fibromyalgia may arise as a primary or secondary disease process. It is most frequent in females aged 20 to 50 years. (From Adams et al., Principles of Neurology, 6th ed, p1494-95)'),
  'D005357': NtO(MH='Fibrous Dysplasia of Bone', MS='A disease of bone marked by thinning of the cortex by fibrous tissue containing bony spicules, producing pain, disability, and gradually increasing deformity. Only one bone may be involved (FIBROUS DYSPLASIA, MONOSTOTIC) or several (FIBROUS DYSPLASIA, POLYOSTOTIC).'),
  'D005358': NtO(MH='Fibrous Dysplasia, Monostotic', MS='FIBROUS DYSPLASIA OF BONE involving only one bone.'),
  'D005359': NtO(MH='Fibrous Dysplasia, Polyostotic', MS='FIBROUS DYSPLASIA OF BONE affecting several bones. When melanotic pigmentation (CAFE-AU-LAIT SPOTS) and multiple endocrine hyperfunction are additionally associated it is referred to as Albright syndrome.'),
  'D005360': NtO(MH='Fibula', MS='The bone of the lower leg lateral to and smaller than the tibia. In proportion to its length, it is the most slender of the long bones.'),
  'D005361': NtO(MH='Ficain', MS='A sulfhydryl proteinase with cysteine at the active site from ficus latex. Preferential cleavage is at tyrosine and phenylalanine residues. EC 3.4.22.3.'),
  'D005362': NtO(MH='Ficoll', MS='A sucrose polymer of high molecular weight.'),
  'D005363': NtO(MH='Ficusin', MS='A naturally occurring furocoumarin, found in PSORALEA. After photoactivation with UV radiation, it binds DNA via single and double-stranded cross-linking.'),
  'D005364': NtO(MH='Field Dependence-Independence', MS='The ability to respond to segments of the perceptual experience rather than to the whole.'),
  'D005365': NtO(MH='FIGLU Test', MS='A urine test for formiminoglutamic acid, an intermediate metabolite in L-histidine catabolism in the conversion of L-histidine to L-glutamic acid. It may be an indicator of vitamin B12 or folic acid deficiency or liver disease.'),
  'D005366': NtO(MH='Figural Aftereffect', MS='A perceptual phenomenon used by Gestalt psychologists to demonstrate that events in one part of the perceptual field may affect perception in another part.'),
  'D005367': NtO(MH='Fiji', MS="A republic consisting of an island group in Melanesia, in the southwest Pacific Ocean. Its capital is Suva. It was discovered by Abel Tasman in 1643 and was visited by Captain Cook in 1774. It was used by escaped convicts from Australia as early as 1804. It was annexed by Great Britain in 1874 but achieved independence in 1970. The name Fiji is of uncertain origin. In its present form it may represent that of Viti, the main island in the group. (From Webster's New Geographical Dictionary, 1988, p396 & Room, Brewer's Dictionary of Names, 1992, p186)"),
  'D005368': NtO(MH='Filariasis', MS='Infections with nematodes of the superfamily FILARIOIDEA. The presence of living worms in the body is mainly asymptomatic but the death of adult worms leads to granulomatous inflammation and permanent fibrosis. Organisms of the genus Elaeophora infect wild elk and domestic sheep causing ischemic necrosis of the brain, blindness, and dermatosis of the face.'),
  'D005369': NtO(MH='Filaricides', MS='Pharmacological agents destructive to nematodes in the superfamily Filarioidea.'),
  'D005370': NtO(MH='Filarioidea', MS='A superfamily of nematodes of the order SPIRURIDA. Members possess a filiform body and a mouth surrounded by papillae.'),
  'D005371': NtO(MH='Filing', MS='Collections of related records treated as a unit; ordering of such files.'),
  'D005372': NtO(MH='Filipin', MS='A complex of polyene antibiotics obtained from Streptomyces filipinensis. Filipin III alters membrane function by interfering with membrane sterols, inhibits mitochondrial respiration, and is proposed as an antifungal agent. Filipins I, II, and IV are less important.'),
  'D005373': NtO(MH='Film Dosimetry', MS='Use of a device (film badge) for measuring exposure of individuals to radiation. It is usually made of metal, plastic, or paper and loaded with one or more pieces of x-ray film.'),
  'D005374': NtO(MH='Filtration', MS='A process of separating particulate matter from a fluid, such as air or a liquid, by passing the fluid carrier through a medium that will not pass the particulates. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D005375': NtO(MH='Financial Audit', MS='An examination, review and verification of all financial accounts.'),
  'D005376': NtO(MH='Financial Management', MS='The obtaining and management of funds for institutional needs and responsibility for fiscal affairs.'),
  'D005377': NtO(MH='Financial Management, Hospital', MS='The obtaining and management of funds for hospital needs and responsibility for fiscal affairs.'),
  'D005378': NtO(MH='Financial Support', MS='The provision of monetary resources including money or capital and credit; obtaining or furnishing money or capital for a purchase or enterprise and the funds so obtained. (From Random House Unabridged Dictionary, 2d ed.)'),
  'D005379': NtO(MH='Financing, Construction', MS='Funding resources and procedures for capital improvement or the construction of facilities.'),
  'D005380': NtO(MH='Financing, Government', MS='Federal, state, or local government organized methods of financial assistance.'),
  'D005381': NtO(MH='Financing, Organized', MS='All organized methods of funding.'),
  'D005382': NtO(MH='Financing, Personal', MS='Payment by individuals or their family for health care services which are not covered by a third-party payer, either insurance or medical assistance.'),
  'D005383': NtO(MH='Finger Injuries', MS='General or unspecified injuries involving the fingers.'),
  'D005384': NtO(MH='Finger Joint', MS='The articulation between the head of one phalanx and the base of the one distal to it, in each finger.'),
  'D005385': NtO(MH='Fingers', MS='Four or five slender jointed digits in humans and primates, attached to each HAND.'),
  'D005386': NtO(MH='Fingersucking', MS='Sucking of the finger. This is one of the most common manipulations of the body found in young children.'),
  'D005387': NtO(MH='Finland', MS='A country in northern Europe, bordering the Baltic Sea, Gulf of Bothnia, and Gulf of Finland, between Sweden and Russia. The capital is Helsinki.'),
  'D005388': NtO(MH='Fire Extinguishing Systems', MS='Automatic or hand operated equipment used to control and extinguish fires.'),
  'D005389': NtO(MH='Firearms', MS='Small-arms weapons, including handguns, pistols, revolvers, rifles, shotguns, etc.'),
  'D005390': NtO(MH='Fires', MS='Combustion or burning in which substances combine chemically with oxygen typically giving out bright light, heat, and smoke.'),
  'D005391': NtO(MH='Firesetting Behavior', MS='A compulsion to set fires.'),
  'D005392': NtO(MH='First Aid', MS='Emergency care or treatment given to a person who suddenly becomes ill or injured before full medical services become available.'),
  'D005393': NtO(MH='Fish Diseases', MS='Diseases of freshwater, marine, hatchery or aquarium fish. This term includes diseases of both teleosts (true fish) and elasmobranchs (sharks, rays and skates).'),
  'D005394': NtO(MH='Fish Flour', MS='A flour made of pulverized, dried fish or fish parts.'),
  'D005395': NtO(MH='Fish Oils', MS='Oils high in unsaturated fats extracted from the bodies of fish or fish parts, especially the LIVER. Those from the liver are usually high in VITAMIN A. The oils are used as DIETARY SUPPLEMENTS. They are also used in soaps and detergents and as protective coatings.'),
  'D005396': NtO(MH='Fish Products', MS='Food products manufactured from fish (e.g., FISH FLOUR, fish meal).'),
  'D005397': NtO(MH='Fish Venoms', MS='Venoms produced by FISHES, including SHARKS and sting rays, usually delivered by spines. They contain various substances, including very labile toxins that affect the HEART specifically and all MUSCLES generally.'),
  'D005398': NtO(MH='Fisheries', MS='Places for cultivation and harvesting of fish, particularly in sea waters. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D005399': NtO(MH='Fishes', MS='A group of cold-blooded, aquatic vertebrates having gills, fins, a cartilaginous or bony endoskeleton, and elongated bodies covered with scales.'),
  'D005400': NtO(MH='Fishes, Poisonous', MS='Fishes that produce FISH VENOMS, as well as members of the TETRAODONTIFORMES order, which produce TETRODOTOXIN.'),
  'D005401': NtO(MH='Fissure in Ano', MS='A painful linear ulcer at the margin of the anus. It appears as a crack or slit in the mucous membrane of the anus and is very painful and difficult to heal. (Dorland, 27th ed & Stedman, 25th ed)'),
  'D005402': NtO(MH='Fistula', MS='Abnormal communication most commonly seen between two internal organs, or between an internal organ and the surface of the body.'),
  'D005403': NtO(MH='Fixation, Ocular', MS='Positioning and accommodation of eyes that allows the image to be brought into place on the FOVEA CENTRALIS of each eye.'),
  'D005404': NtO(MH='Fixatives', MS='Agents employed in the preparation of histologic or pathologic specimens for the purpose of maintaining the existing form and structure of all of the constituent elements. Great numbers of different agents are used; some are also decalcifying and hardening agents. They must quickly kill and coagulate living tissue.'),
  'D005405': NtO(MH='D-Ala(2),MePhe(4),Met(0)-ol-enkephalin', MS='A stable synthetic analog of methionine enkephalin (ENKEPHALIN, METHIONINE). Actions are similar to those of methionine enkephalin. Its effects can be reversed by narcotic antagonists such as naloxone.'),
  'D005406': NtO(MH='Bis(4-Methyl-1-Homopiperazinylthiocarbonyl)disulfide', MS='An inhibitor of the last step of noradrenaline biosynthesis.'),
  'D005407': NtO(MH='Flagella', MS='A whiplike motility appendage present on the surface cells. Prokaryote flagella are composed of a protein called FLAGELLIN. Bacteria can have a single flagellum, a tuft at one pole, or multiple flagella covering the entire surface. In eukaryotes, flagella are threadlike protoplasmic extensions used to propel flagellates and sperm. Flagella have the same basic structure as CILIA but are longer in proportion to the cell bearing them and present in much smaller numbers. (From King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D005408': NtO(MH='Flagellin', MS='A protein with a molecular weight of 40,000 isolated from bacterial flagella. At appropriate pH and salt concentration, three flagellin monomers can spontaneously reaggregate to form structures which appear identical to intact flagella.'),
  'D005409': NtO(MH='Flail Chest', MS='A complication of multiple RIB FRACTURES; RIB and STERNUM fractures, or thoracic surgery. A portion of the THORACIC WALL becomes isolated from the RIB CAGE and exhibits paradoxical respiration.'),
  'D005410': NtO(MH='Flame Ionization', MS='Pyrolysis of organic compounds at the temperature of a hydrogen-air flame to produce ionic intermediates which can be collected and the resulting ion current measured by gas chromatography.'),
  'D005411': NtO(MH='Flame Retardants', MS='Materials applied to fabrics, bedding, furniture, plastics, etc. to retard their burning; many may leach out and cause allergies or other harm.'),
  'D005412': NtO(MH='Flatfishes', MS='Common name for the order Pleuronectiformes. A very distinctive group in that during development they become asymmetrical, i.e., one eye migrates to lie adjacent to the other. They swim on the eyeless side. FLOUNDER, sole, and turbot, along with several others, are included in this order.'),
  'D005413': NtO(MH='Flatfoot', MS='Anomaly in which one or more of the arches of the feet are flat.'),
  'D005414': NtO(MH='Flatulence', MS='Production or presence of gas in the gastrointestinal tract which may be expelled through the anus.'),
  'D005415': NtO(MH='Flavins', MS='Derivatives of the dimethylisoalloxazine (7,8-dimethylbenzo[g]pteridine-2,4(3H,10H)-dione) skeleton. Flavin derivatives serve an electron transfer function as ENZYME COFACTORS in FLAVOPROTEINS.'),
  'D005416': NtO(MH='Flavivirus', MS='A genus of FLAVIVIRIDAE containing several subgroups and many species. Most are arboviruses transmitted by mosquitoes or ticks. The type species is YELLOW FEVER VIRUS.'),
  'D005417': NtO(MH='Flavobacterium', MS='A genus of gram-negative, aerobic, rod-shaped bacteria widely distributed in SOIL and WATER. Its organisms are also found in raw meats, MILK and other FOOD, hospital environments, and human clinical specimens. Some species are pathogenic in humans.'),
  'D005418': NtO(MH='Flavodoxin', MS='A low-molecular-weight (16,000) iron-free flavoprotein containing one molecule of flavin mononucleotide (FMN) and isolated from bacteria grown on an iron-deficient medium. It can replace ferredoxin in all the electron-transfer functions in which the latter is known to serve in bacterial cells.'),
  'D005419': NtO(MH='Flavonoids', MS='A group of phenyl benzopyrans named for having structures like FLAVONES.'),
  'D005420': NtO(MH='Flavoproteins', MS=None),
  'D005421': NtO(MH='Flavoring Agents', MS='Substances added to foods and medicine to improve the taste.'),
  'D005422': NtO(MH='Flavoxate', MS='A drug that has been used in various urinary syndromes and as an antispasmodic. Its therapeutic usefulness and its mechanism of action are not clear. It may have local anesthetic activity and direct relaxing effects on smooth muscle as well as some activity as a muscarinic antagonist.'),
  'D005423': NtO(MH='Siphonaptera', MS='An order of parasitic, blood-sucking, wingless INSECTS with the common name of fleas.'),
  'D005424': NtO(MH='Flecainide', MS='A potent anti-arrhythmia agent, effective in a wide range of ventricular and atrial ARRHYTHMIAS and TACHYCARDIAS.'),
  'D005425': NtO(MH='Flicker Fusion', MS='The point or frequency at which all flicker of an intermittent light stimulus disappears.'),
  'D005426': NtO(MH='Flight, Animal', MS='The use of wings or wing-like appendages to remain aloft and move through the air.'),
  'D005427': NtO(MH='Flocculation', MS='The aggregation of suspended solids into larger clumps.'),
  'D005428': NtO(MH='Flocculation Tests', MS='Precipitin tests which occur over a narrow range of antigen-antibody ratio, due chiefly to peculiarities of the antibody (precipitin). (From Stedman, 26th ed)'),
  'D005430': NtO(MH='Floors and Floorcoverings', MS='The surface of a structure upon which one stands or walks.'),
  'D005431': NtO(MH='Florida', MS='State bounded on east by the Atlantic Ocean, on the south by the Gulf of Mexico, on the west by Alabama and on the north by Alabama and Georgia.'),
  'D005432': NtO(MH='Flounder', MS='Common name for two families of FLATFISHES belonging to the order Pleuronectiformes: left-eye flounders (Bothidae) and right-eye flounders (Pleuronectidae). The latter is more commonly used in research.'),
  'D005433': NtO(MH='Flour', MS='Ground up seed of WHEAT.'),
  'D005434': NtO(MH='Flow Cytometry', MS='Technique using an instrument system for making, processing, and displaying one or more measurements on individual cells obtained from a cell suspension. Cells are usually stained with one or more fluorescent dyes specific to cell components of interest, e.g., DNA, and fluorescence of each cell is measured as it rapidly transverses the excitation beam (laser or mercury arc lamp). Fluorescence provides a quantitative measure of various biochemical and biophysical properties of the cell, as well as a basis for cell sorting. Other measurable optical parameters include light absorption and light scattering, the latter being applicable to the measurement of cell size, shape, density, granularity, and stain uptake.'),
  'D005436': NtO(MH='Floxacillin', MS='Antibiotic analog of CLOXACILLIN.'),
  'D005437': NtO(MH='Flucytosine', MS='A fluorinated cytosine analog that is used as an antifungal agent.'),
  'D005438': NtO(MH='Fludrocortisone', MS='A synthetic mineralocorticoid with anti-inflammatory activity.'),
  'D005439': NtO(MH='Flufenamic Acid', MS='An anthranilic acid derivative with analgesic, anti-inflammatory, and antipyretic properties. It is used in musculoskeletal and joint disorders and administered by mouth and topically. (From Martindale, The Extra Pharmacopoeia, 30th ed, p16)'),
  'D005440': NtO(MH='Fluid Therapy', MS='Therapy whose basic objective is to restore the volume and composition of the body fluids to normal with respect to WATER-ELECTROLYTE BALANCE. Fluids may be administered intravenously, orally, by intermittent gavage, or by HYPODERMOCLYSIS.'),
  'D005441': NtO(MH='Fluids and Secretions', MS='Liquid substances produced by living organisms to fulfill specific functions or excreted as waste.'),
  'D005442': NtO(MH='Flumazenil', MS='A potent benzodiazepine receptor antagonist. Since it reverses the sedative and other actions of benzodiazepines, it has been suggested as an antidote to benzodiazepine overdoses.'),
  'D005443': NtO(MH='Flumethasone', MS='An anti-inflammatory glucocorticoid used in veterinary practice.'),
  'D005444': NtO(MH='Flunarizine', MS='Flunarizine is a selective calcium entry blocker with calmodulin binding properties and histamine H1 blocking activity. It is effective in the prophylaxis of migraine, occlusive peripheral vascular disease, vertigo of central and peripheral origin, and as an adjuvant in the therapy of epilepsy.'),
  'D005445': NtO(MH='Flunitrazepam', MS='A benzodiazepine with pharmacologic actions similar to those of DIAZEPAM that can cause ANTEROGRADE AMNESIA. Some reports indicate that it is used as a date rape drug and suggest that it may precipitate violent behavior. The United States Government has banned the importation of this drug.'),
  'D005446': NtO(MH='Fluocinolone Acetonide', MS='A glucocorticoid derivative used topically in the treatment of various skin disorders. It is usually employed as a cream, gel, lotion, or ointment. It has also been used topically in the treatment of inflammatory eye, ear, and nose disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p732)'),
  'D005447': NtO(MH='Fluocinonide', MS='A topical glucocorticoid used in the treatment of ECZEMA.'),
  'D005448': NtO(MH='Fluocortolone', MS='A glucocorticoid with anti-inflammatory activity used topically for various skin disorders.'),
  'D005449': NtO(MH='Fluorenes', MS='A family of diphenylenemethane derivatives.'),
  'D005450': NtO(MH='Fluorescamine', MS='A nonfluorescent reagent for the detection of primary amines, peptides and proteins. The reaction products are highly fluorescent.'),
  'D005451': NtO(MH='Fluorescein Angiography', MS='Visualization of a vascular system after intravenous injection of a fluorescein solution. The images may be photographed or televised. It is used especially in studying the retinal and uveal vasculature.'),
  'D005452': NtO(MH='Fluoresceins', MS="A family of spiro(isobenzofuran-1(3H),9'-(9H)xanthen)-3-one derivatives. These are used as dyes, as indicators for various metals, and as fluorescent labels in immunoassays."),
  'D005453': NtO(MH='Fluorescence', MS='The property of emitting radiation while being irradiated. The radiation emitted is usually of longer wavelength than that incident or absorbed, e.g., a substance can be irradiated with invisible radiation and emit visible light. X-ray fluorescence is used in diagnosis.'),
  'D005454': NtO(MH='Fluorescence Polarization', MS='Measurement of the polarization of fluorescent light from solutions or microscopic specimens. It is used to provide information concerning molecular size, shape, and conformation, molecular anisotropy, electronic energy transfer, molecular interaction, including dye and coenzyme binding, and the antigen-antibody reaction.'),
  'D005455': NtO(MH='Fluorescent Antibody Technique', MS='Test for tissue antigen using either a direct method, by conjugation of antibody with fluorescent dye (FLUORESCENT ANTIBODY TECHNIQUE, DIRECT) or an indirect method, by formation of antigen-antibody complex which is then labeled with fluorescein-conjugated anti-immunoglobulin antibody (FLUORESCENT ANTIBODY TECHNIQUE, INDIRECT). The tissue is then examined by fluorescence microscopy.'),
  'D005456': NtO(MH='Fluorescent Dyes', MS='Agents that emit light after excitation by light. The wave length of the emitted light is usually longer than that of the incident light. Fluorochromes are substances that cause fluorescence in other substances, i.e., dyes used to mark or label other compounds with fluorescent tags.'),
  'D005457': NtO(MH='Fluoridation', MS='Practice of adding fluoride to water, and other food or beverages, for the purpose of preventing tooth decay and tooth cavities.'),
  'D005458': NtO(MH='Fluoride Poisoning', MS='Poisoning that results from chronic or acute ingestion, injection, inhalation, or skin absorption of FLUORIDE compounds.'),
  'D005459': NtO(MH='Fluorides', MS='Inorganic salts of hydrofluoric acid, HF, in which the fluorine atom is in the -1 oxidation state. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed) Sodium and stannous salts are commonly used in dentifrices.'),
  'D005460': NtO(MH='Fluorides, Topical', MS='Fluorides, usually in pastes or gels, used for topical application to reduce the incidence of DENTAL CARIES.'),
  'D005461': NtO(MH='Fluorine', MS='A nonmetallic, diatomic gas that is a trace element and member of the halogen family. It is used in dentistry as fluoride (FLUORIDES) to prevent dental caries.'),
  'D005462': NtO(MH='Fluorine Radioisotopes', MS='Unstable isotopes of fluorine that decay or disintegrate emitting radiation. F atoms with atomic weights 17, 18, and 20-22 are radioactive fluorine isotopes.'),
  'D005463': NtO(MH='Fluoroacetates', MS="Derivatives of acetic acid with one or more fluorines attached. They are almost odorless, difficult to detect chemically, and very stable. The acid itself, as well as the derivatives that are broken down in the body to the acid, are highly toxic substances, behaving as convulsant poisons with a delayed action. (From Miall's Dictionary of Chemistry, 5th ed)"),
  'D005464': NtO(MH='Fluorobenzenes', MS='Derivatives of BENZENE that contain FLUORINE.'),
  'D005465': NtO(MH='Fluorocarbon Polymers', MS=None),
  'D005466': NtO(MH='Fluorocarbons', MS='Liquid perfluorinated carbon compounds which may or may not contain a hetero atom such as nitrogen, oxygen or sulfur, but do not contain another halogen or hydrogen atom. This concept includes fluorocarbon emulsions and fluorocarbon blood substitutes.'),
  'D005467': NtO(MH='Floxuridine', MS='An antineoplastic antimetabolite that is metabolized to fluorouracil when administered by rapid injection; when administered by slow, continuous, intra-arterial infusion, it is converted to floxuridine monophosphate. It has been used to treat hepatic metastases of gastrointestinal adenocarcinomas and for palliation in malignant neoplasms of the liver and gastrointestinal tract.'),
  'D005468': NtO(MH='Fluorodeoxyuridylate', MS="5-Fluoro-2'-deoxyuridylate. An inhibitor of thymidylate synthetase. Formed from 5-fluorouracil or 5-fluorodeoxyuridine."),
  'D005469': NtO(MH='Fluorometholone', MS='A glucocorticoid employed, usually as eye drops, in the treatment of allergic and inflammatory conditions of the eye. It has also been used topically in the treatment of various skin disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p732)'),
  'D005470': NtO(MH='Fluorometry', MS='An analytical method for detecting and measuring FLUORESCENCE in compounds or targets such as cells, proteins, or nucleotides, or targets previously labeled with FLUORESCENCE AGENTS.'),
  'D005471': NtO(MH='Fluoroscopy', MS='Production of an image when x-rays strike a fluorescent screen.'),
  'D005472': NtO(MH='Fluorouracil', MS='A pyrimidine analog that is an antineoplastic antimetabolite. It interferes with DNA synthesis by blocking the THYMIDYLATE SYNTHETASE conversion of deoxyuridylic acid to thymidylic acid.'),
  'D005473': NtO(MH='Fluoxetine', MS='The first highly specific serotonin uptake inhibitor. It is used as an antidepressant and often has a more acceptable side-effects profile than traditional antidepressants.'),
  'D005474': NtO(MH='Fluoxymesterone', MS='An anabolic steroid that has been used in the treatment of male HYPOGONADISM, delayed puberty in males, and in the treatment of breast neoplasms in women.'),
  'D005475': NtO(MH='Flupenthixol', MS='A thioxanthene neuroleptic that, unlike CHLORPROMAZINE, is claimed to have CNS-activating properties. It is used in the treatment of psychoses although not in excited or manic patients. (From Martindale, The Extra Pharmacopoeia, 30th ed, p595)'),
  'D005476': NtO(MH='Fluphenazine', MS='A phenothiazine used in the treatment of PSYCHOSES. Its properties and uses are generally similar to those of CHLORPROMAZINE.'),
  'D005477': NtO(MH='Fluprednisolone', MS='A synthetic glucocorticoid with anti-inflammatory properties.'),
  'D005478': NtO(MH='Flurandrenolone', MS='A corticosteroid used topically in the treatment of various skin disorders. It is usually employed as a cream or an ointment, and is also used as a polyethylene tape with an adhesive. (From Martindale, The Extra Pharmacopoeia, 30th ed, p733)'),
  'D005479': NtO(MH='Flurazepam', MS='A benzodiazepine derivative used mainly as a hypnotic.'),
  'D005480': NtO(MH='Flurbiprofen', MS='An anti-inflammatory analgesic and antipyretic of the phenylalkynoic acid series. It has been shown to reduce bone resorption in periodontal disease by inhibiting CARBONIC ANHYDRASE.'),
  'D005481': NtO(MH='Flurothyl', MS='A convulsant primarily used in experimental animals. It was formerly used to induce convulsions as a alternative to electroshock therapy.'),
  'D005483': NtO(MH='Flushing', MS='A transient reddening of the face that may be due to fever, certain drugs, exertion, or stress.'),
  'D005484': NtO(MH='Fluspirilene', MS='A long-acting injectable antipsychotic agent used for chronic schizophrenia.'),
  'D005485': NtO(MH='Flutamide', MS='An antiandrogen with about the same potency as cyproterone in rodent and canine species.'),
  'D005486': NtO(MH='Flavin Mononucleotide', MS='A coenzyme for a number of oxidative enzymes including NADH DEHYDROGENASE. It is the principal form in which RIBOFLAVIN is found in cells and tissues.'),
  'D005487': NtO(MH='Foam Cells', MS='Lipid-laden macrophages originating from monocytes or from smooth muscle cells.'),
  'D005489': NtO(MH='Focal Dermal Hypoplasia', MS='A genetic skin disease characterized by hypoplasia of the dermis, herniations of fat, and hand anomalies. It is found exclusively in females and transmitted as an X-linked dominant trait.'),
  'D005490': NtO(MH='Focal Infection', MS='An infection at a specific location that may spread to another region of the body.'),
  'D005491': NtO(MH='Focal Infection, Dental', MS='Secondary or systemic infections due to dissemination throughout the body of microorganisms whose primary focus of infection lies in the periodontal tissues.'),
  'D005492': NtO(MH='Folic Acid', MS='A member of the vitamin B family that stimulates the hematopoietic system. It is present in the liver and kidney and is found in mushrooms, spinach, yeast, green leaves, and grasses (POACEAE). Folic acid is used in the treatment and prevention of folate deficiencies and megaloblastic anemia.'),
  'D005493': NtO(MH='Folic Acid Antagonists', MS='Inhibitors of the enzyme, dihydrofolate reductase (TETRAHYDROFOLATE DEHYDROGENASE), which converts dihydrofolate (FH2) to tetrahydrofolate (FH4). They are frequently used in cancer chemotherapy. (From AMA, Drug Evaluations Annual, 1994, p2033)'),
  'D005494': NtO(MH='Folic Acid Deficiency', MS='A nutritional condition produced by a deficiency of FOLIC ACID in the diet. Many plant and animal tissues contain folic acid, abundant in green leafy vegetables, yeast, liver, and mushrooms but destroyed by long-term cooking. Alcohol interferes with its intermediate metabolism and absorption. Folic acid deficiency may develop in long-term anticonvulsant therapy or with use of oral contraceptives. This deficiency causes anemia, macrocytic anemia, and megaloblastic anemia. It is indistinguishable from vitamin B 12 deficiency in peripheral blood and bone marrow findings, but the neurologic lesions seen in B 12 deficiency do not occur. (Merck Manual, 16th ed)'),
  'D005495': NtO(MH='Folklore', MS='The common orally transmitted traditions, myths, festivals, songs, superstitions, and stories of all peoples.'),
  'D005496': NtO(MH='Follicular Atresia', MS='The degeneration and resorption of an OVARIAN FOLLICLE before it reaches maturity and ruptures.'),
  'D005497': NtO(MH='Follicular Cyst', MS='Cyst due to the occlusion of the duct of a follicle or small gland.'),
  'D005498': NtO(MH='Follicular Phase', MS='The period of the MENSTRUAL CYCLE representing follicular growth, increase in ovarian estrogen (ESTROGENS) production, and epithelial proliferation of the ENDOMETRIUM. Follicular phase begins with the onset of MENSTRUATION and ends with OVULATION.'),
  'D005499': NtO(MH='Folliculitis', MS='Inflammation of follicles, primarily hair follicles.'),
  'D005500': NtO(MH='Follow-Up Studies', MS='Studies in which individuals or populations are followed to assess the outcome of exposures, procedures, or effects of a characteristic, e.g., occurrence of disease.'),
  'D005502': NtO(MH='Food', MS='Substances taken in by the body to provide nourishment.'),
  'D005503': NtO(MH='Food Additives', MS='Substances used in the processing or storage of foods or animal feed including ANTIOXIDANTS; FOOD PRESERVATIVES; FOOD COLORING AGENTS; FLAVORING AGENTS; ANTI-INFECTIVE AGENTS; EXCIPIENTS and other similarly used substances. Many of the same substances are used as PHARMACEUTIC AIDS.'),
  'D005504': NtO(MH='Food Analysis', MS='Measurement and evaluation of the components of substances to be taken as FOOD.'),
  'D005505': NtO(MH='Food Coloring Agents', MS='Natural or synthetic dyes used as coloring agents in processed foods.'),
  'D005506': NtO(MH='Food Contamination', MS='The presence in food of harmful, unpalatable, or otherwise objectionable foreign substances, e.g. chemicals, microorganisms or diluents, before, during, or after processing or storage.'),
  'D005507': NtO(MH='Food Contamination, Radioactive', MS='Exposure of FOOD to radioactive substances, distinguishable from FOOD IRRADIATION, a food preservation technique.'),
  'D005508': NtO(MH='Food Deprivation', MS='The withholding of food in a structured experimental situation.'),
  'D005509': NtO(MH='Food Dispensers, Automatic', MS='Mechanical food dispensing machines.'),
  'D005511': NtO(MH='Food Handling', MS='Any aspect of the operations in the preparation, processing, transport, storage, packaging, wrapping, exposure for sale, service, or delivery of food.'),
  'D005512': NtO(MH='Food Hypersensitivity', MS='Gastrointestinal disturbances, skin eruptions, or shock due to allergic reactions to allergens in food.'),
  'D005513': NtO(MH='Food Inspection', MS='Examination of foods to assure wholesome and clean products free from unsafe microbes or chemical contamination, natural or added deleterious substances, and decomposition during production, processing, packaging, etc.'),
  'D005514': NtO(MH='Food Irradiation', MS='Treatment of food with RADIATION.'),
  'D005515': NtO(MH='Food Labeling', MS='Use of written, printed, or graphic materials upon or accompanying a food or its container or wrapper. The concept includes ingredients, NUTRITIONAL VALUE, directions, warnings, and other relevant information.'),
  'D005516': NtO(MH='Food Microbiology', MS='The presence of bacteria, viruses, and fungi in food and food products. This term is not restricted to pathogenic organisms: the presence of various non-pathogenic bacteria and fungi in cheeses and wines, for example, is included in this concept.'),
  'D005517': NtO(MH='Foodborne Diseases', MS='Acute illnesses, usually affecting the GASTROINTESTINAL TRACT, brought on by consuming contaminated food or beverages. Most of these diseases are infectious, caused by a variety of bacteria, viruses, or parasites that can be foodborne. Sometimes the diseases are caused by harmful toxins from the microbes or other chemicals present in the food. Especially in the latter case, the condition is often called food poisoning.'),
  'D005518': NtO(MH='Food Preferences', MS='The selection of one food over another.'),
  'D005519': NtO(MH='Food Preservation', MS='Procedures or techniques used to keep food from spoiling.'),
  'D005520': NtO(MH='Food Preservatives', MS='Substances capable of inhibiting, retarding or arresting the process of fermentation, acidification or other deterioration of foods.'),
  'D005521': NtO(MH='Food Service, Hospital', MS="Hospital department that manages and supervises the dietary program in accordance with the patients' requirements."),
  'D005522': NtO(MH='Food Services', MS='Functions, equipment, and facilities concerned with the preparation and distribution of ready-to-eat food.'),
  'D005523': NtO(MH='Food Supply', MS='The production and movement of food items from point of origin to use or consumption.'),
  'D005524': NtO(MH='Food Technology', MS='The application of knowledge to the food industry.'),
  'D005525': NtO(MH='Food-Processing Industry', MS='The productive enterprises concerned with food processing.'),
  'D005526': NtO(MH='Food, Formulated', MS='Food and dietary formulations including elemental (chemically defined formula) diets, synthetic and semisynthetic diets, space diets, weight-reduction formulas, tube-feeding diets, complete liquid diets, and supplemental liquid and solid diets.'),
  'D005527': NtO(MH='Food, Fortified', MS='Any food that has been supplemented with essential nutrients either in quantities that are greater than those present normally, or which are not present in the food normally. Fortified food includes also food to which various nutrients have been added to compensate for those removed by refinement or processing. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D005528': NtO(MH='Foot', MS='The distal extremity of the leg in vertebrates, consisting of the tarsus (ANKLE); METATARSUS; phalanges; and the soft tissues surrounding these bones.'),
  'D005529': NtO(MH='Foot Bones', MS='The TARSAL BONES; METATARSAL BONES; and PHALANGES OF TOES. The tarsal bones consists of seven bones: CALCANEUS; TALUS; cuboid; navicular; internal; middle; and external cuneiform bones. The five metatarsal bones are numbered one through five, running medial to lateral. There are 14 phalanges in each foot, the great toe has two while the other toes have three each.'),
  'D005530': NtO(MH='Foot Deformities', MS='Alterations or deviations from normal shape or size which result in a disfigurement of the foot.'),
  'D005531': NtO(MH='Foot Deformities, Acquired', MS='Distortion or disfigurement of the foot, or a part of the foot, acquired through disease or injury after birth.'),
  'D005532': NtO(MH='Foot Deformities, Congenital', MS='Alterations or deviations from normal shape or size which result in a disfigurement of the foot occurring at or before birth.'),
  'D005533': NtO(MH='Foot Dermatoses', MS='Skin diseases of the foot, general or unspecified.'),
  'D005534': NtO(MH='Foot Diseases', MS='Anatomical and functional disorders affecting the foot.'),
  'D005535': NtO(MH='Foot Rot', MS="A disease of the horny parts and of the adjacent soft structures of the feet of cattle, swine, and sheep. It is usually caused by Corynebacterium pyogenes or Bacteroides nodosus (see DICHELOBACTER NODOSUS). It is also known as interdigital necrobacillosis. (From Black's Veterinary Dictionary, 18th ed)"),
  'D005536': NtO(MH='Foot-and-Mouth Disease', MS='A highly infectious disease caused by FOOT AND MOUTH DISEASE VIRUS that affects mammals of the ARTIODACTYLA order (CATTLE; SHEEP; GOATS; and PIGS) and is characterized by high fever followed by the appearance of blisters inside the mouth and on the feet.'),
  'D005537': NtO(MH='Aphthovirus', MS='A genus of the family PICORNAVIRIDAE infecting mainly cloven-hoofed animals. They cause vesicular lesions and upper respiratory tract infections. FOOT AND MOUTH DISEASE VIRUS is the type species.'),
  'D005538': NtO(MH='Football', MS='A competitive team sport played on a rectangular field. This is the American or Canadian version of the game and also includes the form known as rugby. It does not include non-North American football (= SOCCER).'),
  'D005539': NtO(MH='Foramen Magnum', MS='The large hole at the base of the skull through which the SPINAL CORD passes.'),
  'D005540': NtO(MH='Forced Expiratory Flow Rates', MS='The rate of airflow measured during a FORCED VITAL CAPACITY determination.'),
  'D005541': NtO(MH='Forced Expiratory Volume', MS='Measure of the maximum amount of air that can be expelled in a given number of seconds during a FORCED VITAL CAPACITY determination . It is usually given as FEV followed by a subscript indicating the number of seconds over which the measurement is made, although it is sometimes given as a percentage of forced vital capacity.'),
  'D005542': NtO(MH='Forearm', MS='Part of the upper extremity in humans and primates extending from the ELBOW to the WRIST.'),
  'D005543': NtO(MH='Forearm Injuries', MS='Injuries to the part of the upper limb of the body between the wrist and elbow.'),
  'D005544': NtO(MH='Forecasting', MS='The prediction or projection of the nature of future problems or existing conditions based upon the extrapolation or interpretation of existing scientific data or by the application of scientific methodology.'),
  'D005545': NtO(MH='Forefoot, Human', MS='The forepart of the foot including the metatarsals and the TOES.'),
  'D005546': NtO(MH='Forehead', MS='The part of the face above the eyes.'),
  'D005547': NtO(MH='Foreign Bodies', MS='Inanimate objects that become enclosed in the body.'),
  'D005548': NtO(MH='Foreign-Body Migration', MS='Migration of a foreign body from its original location to some other location in the body.'),
  'D005549': NtO(MH='Foreign-Body Reaction', MS='Chronic inflammation and granuloma formation around irritating foreign bodies.'),
  'D005550': NtO(MH='Foreign Medical Graduates', MS='Physicians who hold degrees from medical schools in countries other than the ones in which they practice.'),
  'D005551': NtO(MH='Foreign Professional Personnel', MS='Persons who have acquired academic or specialized training in countries other than that in which they are working. The concept excludes physicians for which FOREIGN MEDICAL GRADUATES is the likely heading.'),
  'D005552': NtO(MH='Forelimb', MS='A front limb of a quadruped. (The Random House College Dictionary, 1980)'),
  'D005553': NtO(MH='Forensic Dentistry', MS='The application of dental knowledge to questions of law.'),
  'D005554': NtO(MH='Forensic Medicine', MS='The application of medical knowledge to questions of law.'),
  'D005555': NtO(MH='Forensic Psychiatry', MS="Psychiatry in its legal aspects. This includes criminology, penology, commitment of mentally ill, the psychiatrist's role in compensation cases, the problems of releasing information to the court, and of expert testimony."),
  'D005556': NtO(MH='Form Perception', MS='The sensory discrimination of a pattern shape or outline.'),
  'D005557': NtO(MH='Formaldehyde', MS='A highly reactive aldehyde gas formed by oxidation or incomplete combustion of hydrocarbons. In solution, it has a wide range of uses: in the manufacture of resins and textiles, as a disinfectant, and as a laboratory fixative or preservative. Formaldehyde solution (formalin) is considered a hazardous compound, and its vapor toxic. (From Reynolds, Martindale The Extra Pharmacopoeia, 30th ed, p717)'),
  'D005558': NtO(MH='Arylformamidase', MS='An enzyme that catalyzes the conversion of N-formyl-L-kynurenine and water to formate and L-kynurenine. It also acts on other aromatic formylamines. (Enzyme Nomenclature, 1992) EC 3.5.1.9.'),
  'D005559': NtO(MH='Formamides', MS='A group of amides with the general formula of R-CONH2.'),
  'D005560': NtO(MH='Formate Dehydrogenases', MS='Flavoproteins that catalyze reversibly the reduction of carbon dioxide to formate. Many compounds can act as acceptors, but the only physiologically active acceptor is NAD. The enzymes are active in the fermentation of sugars and other compounds to carbon dioxide and are the key enzymes in obtaining energy when bacteria are grown on formate as the main carbon source. They have been purified from bovine blood. EC 1.2.1.2.'),
  'D005561': NtO(MH='Formates', MS='Derivatives of formic acids. Included under this heading are a broad variety of acid forms, salts, esters, and amides that are formed with a single carbon carboxy group.'),
  'D005562': NtO(MH='Formazans', MS='Colored azo compounds formed by the reduction of tetrazolium salts. Employing this reaction, oxidoreductase activity can be determined quantitatively in tissue sections by allowing the enzymes to act on their specific substrates in the presence of tetrazolium salts.'),
  'D005563': NtO(MH='Formic Acid Esters', MS='ESTER derivatives of formic acid with the formula H-COO-R.'),
  'D005565': NtO(MH='Formiminoglutamic Acid', MS='Measurement of this acid in the urine after oral administration of histidine provides the basis for the diagnostic test of folic acid deficiency and of megaloblastic anemia of pregnancy.'),
  'D005566': NtO(MH='Formocresols', MS='Compounds consisting of formaldehyde, cresol, GLYCERIN, and water. They are  used in the PULPOTOMY of PRIMARY TEETH and as ROOT CANAL IRRIGANTS.'),
  'D005568': NtO(MH='Forms and Records Control', MS='A management function in which standards and guidelines are developed for the development, maintenance, and handling of forms and records.'),
  'D005569': NtO(MH='Formularies as Topic', MS='Works about lists of drugs or collections of recipes, formulas, and prescriptions for the compounding of medicinal preparations. Formularies differ from PHARMACOPEIA in that they are less complete, lacking full descriptions of the drugs, their formulations, analytic composition, chemical properties, etc. In hospitals, formularies list all drugs commonly stocked in the hospital pharmacy.'),
  'D005570': NtO(MH='Formularies, Dental as Topic', MS='Works about reference lists of descriptions and uses of drugs related to oral medicine and dentistry.'),
  'D005571': NtO(MH='Formularies, Homeopathic as Topic', MS='Works about formularies concerned with HOMEOPATHIC REMEDIES.'),
  'D005572': NtO(MH='Formularies, Hospital as Topic', MS='Works about formularies concerned with PHARMACEUTICAL PREPARATIONS prescribed in hospitals.'),
  'D005573': NtO(MH='Formycins', MS='Pyrazolopyrimidine ribonucleosides isolated from Nocardia interforma. They are antineoplastic antibiotics with cytostatic properties.'),
  'D005574': NtO(MH='Formate-Tetrahydrofolate Ligase', MS='A carbon-nitrogen ligase that catalyzes the formation of 10-formyltetrahydrofolate from formate and tetrahydrofolate in the presence of ATP. In higher eukaryotes the enzyme also contains METHYLENETETRAHYDROFOLATE DEHYDROGENASE (NADP+) and METHENYLTETRAHYDROFOLATE CYCLOHYDROLASE activity.'),
  'D005575': NtO(MH='Formyltetrahydrofolates', MS='Tetrahydrofolates which are substituted by a formyl group at either the nitrogen atom in the 5 position or the nitrogen atom in the 10 position. N(5)-Formyltetrahydrofolate is leucovorin (citrovorum factor) while N(10)-formyltetrahydrofolate is an active coenzyme which functions as a carrier of the formyl group in a number of enzymatic reactions.'),
  'D005576': NtO(MH='Colforsin', MS='Potent activator of the adenylate cyclase system and the biosynthesis of cyclic AMP. From the plant COLEUS FORSKOHLII. Has antihypertensive, positive inotropic, platelet aggregation inhibitory, and smooth muscle relaxant activities; also lowers intraocular pressure and promotes release of hormones from the pituitary gland.'),
  'D005577': NtO(MH='Forssman Antigen', MS='A glycolipid, cross-species antigen that induces production of antisheep hemolysin. It is present on the tissue cells of many species but absent in humans. It is found in many infectious agents.'),
  'D005578': NtO(MH='Fosfomycin', MS='An antibiotic produced by Streptomyces fradiae.'),
  'D005579': NtO(MH='Fossil Fuels', MS='Any combustible hydrocarbon deposit formed from the remains of prehistoric organisms. Examples are petroleum, coal, and natural gas.'),
  'D005580': NtO(MH='Fossils', MS="Remains, impressions, or traces of animals or plants of past geological times which have been preserved in the earth's crust."),
  'D005581': NtO(MH='Foster Home Care', MS='Families who care for neglected children or patients unable to care for themselves.'),
  'D005582': NtO(MH='Foundations', MS='Organizations established by endowments with provision for future maintenance.'),
  'D005583': NtO(MH='Fourier Analysis', MS='Analysis based on the mathematical function first formulated by Jean-Baptiste-Joseph Fourier in 1807. The function, known as the Fourier transform, describes the sinusoidal pattern of any fluctuating pattern in the physical world in terms of its amplitude and its phase. It has broad applications in biomedicine, e.g., analysis of the x-ray crystallography data pivotal in identifying the double helical nature of DNA and in analysis of other molecules, including viruses, and the modified back-projection algorithm universally used in computerized tomography imaging, etc. (From Segen, The Dictionary of Modern Medicine, 1992)'),
  'D005584': NtO(MH='Fovea Centralis', MS='An area approximately 1.5 millimeters in diameter within the macula lutea where the retina thins out greatly because of the oblique shifting of all layers except the pigment epithelium layer. It includes the sloping walls of the fovea (clivus) and contains a few rods in its periphery. In its center (foveola) are the cones most adapted to yield high visual acuity, each cone being connected to only one ganglion cell. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D005585': NtO(MH='Influenza in Birds', MS='Infection of domestic and wild fowl and other BIRDS with INFLUENZA A VIRUS. Avian influenza usually does not sicken birds, but can be highly pathogenic and fatal in domestic POULTRY.'),
  'D005586': NtO(MH='Fowlpox', MS='A poxvirus infection of poultry and other birds characterized by the formation of wart-like nodules on the skin and diphtheritic necrotic masses (cankers) in the upper digestive and respiratory tracts.'),
  'D005587': NtO(MH='Fowlpox virus', MS='The type species of the genus AVIPOXVIRUS. It is the etiologic agent of FOWLPOX.'),
  'D005588': NtO(MH='Fox-Fordyce Disease', MS='Chronic pruritic disease, usually in women, characterized by small follicular papular eruptions in APOCRINE GLANDS areas. It is caused by obstruction and rupture of intraepidermal apocrine ducts.'),
  'D005589': NtO(MH='Foxes', MS='Any of several carnivores in the family CANIDAE, that possess erect ears and long bushy tails and are smaller than WOLVES. They are classified in several genera and found on all continents except Antarctica.'),
  'D005590': NtO(MH='Fractional Precipitation', MS='A method which uses specific precipitation reactions to separate or collect substances from a solution.'),
  'D005591': NtO(MH='Chemical Fractionation', MS='Separation of a mixture in successive stages, each stage removing from the mixture some proportion of one of the substances, for example by differential solubility in water-solvent mixtures. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D005592': NtO(MH='Fracture Fixation', MS='The use of metallic devices inserted into or through bone to hold a fracture in a set position and alignment while it heals.'),
  'D005593': NtO(MH='Fracture Fixation, Internal', MS='The use of internal devices (metal plates, nails, rods, etc.) to hold the position of a fracture in proper alignment.'),
  'D005594': NtO(MH='Fracture Fixation, Intramedullary', MS='The use of nails that are inserted into bone cavities in order to keep fractured bones together.'),
  'D005596': NtO(MH='Fractures, Closed', MS='Fractures in which the break in bone is not accompanied by an external wound.'),
  'D005597': NtO(MH='Fractures, Open', MS='Fractures in which there is an external wound communicating with the break of the bone.'),
  'D005598': NtO(MH='Fractures, Spontaneous', MS='Fractures occurring as a result of disease of a bone or from some undiscoverable cause, and not due to trauma. (Dorland, 27th ed)'),
  'D005599': NtO(MH='Fractures, Ununited', MS='A fracture in which union fails to occur, the ends of the bone becoming rounded and eburnated, and a false joint occurs. (Stedman, 25th ed)'),
  'D005600': NtO(MH='Fragile X Syndrome', MS='A condition characterized genotypically by mutation of the distal end of the long arm of the X chromosome (at gene loci FRAXA or FRAXE) and phenotypically by cognitive impairment, hyperactivity, SEIZURES, language delay, and enlargement of the ears, head, and testes. INTELLECTUAL DISABILITY occurs in nearly all males and roughly 50% of females with the full mutation of FRAXA. (From Menkes, Textbook of Child Neurology, 5th ed, p226)'),
  'D005601': NtO(MH='Framycetin', MS='A component of NEOMYCIN that is produced by Streptomyces fradiae. On hydrolysis it yields neamine and neobiosamine B. (From Merck Index, 11th ed)'),
  'D005602': NtO(MH='France', MS='A country in western Europe bordered by the Atlantic Ocean, the English Channel, the Mediterranean Sea, and the countries of Belgium, Germany, Italy, Spain, Switzerland, the principalities of Andorra and Monaco, and by the duchy of Luxembourg. Its capital is Paris.'),
  'D005603': NtO(MH='Francisella', MS='The lone genus of bacteria in the family Francisellaceae, frequently found in natural waters. It can be parasitic in humans, other MAMMALS; BIRDS; and ARTHROPODS.'),
  'D005604': NtO(MH='Francisella tularensis', MS='The etiologic agent of TULAREMIA in man and other warm-blooded animals.'),
  'D005605': NtO(MH='Francium', MS='A radioactive alkali metal with the atomic symbol Fr, and atomic number 87. The mass numbers of known isotopes are 204-213, 217-224. Its valence is +1.'),
  'D005607': NtO(MH='Fraud', MS='Exploitation through misrepresentation of the facts or concealment of the purposes of the exploiter.'),
  'D005608': NtO(MH='Free Association', MS='Spontaneous verbalization of whatever comes to mind.'),
  'D005609': NtO(MH='Free Radicals', MS='Highly reactive molecules with an unsatisfied electron valence pair. Free radicals are produced in both normal and pathological processes. They are proven or suspected agents of tissue damage in a wide variety of circumstances including radiation, damage from environment chemicals, and aging. Natural and pharmacological prevention of free radical damage is being actively investigated.'),
  'D005610': NtO(MH='Freedom', MS='The rights of individuals to act and make decisions without external constraints.'),
  'D005611': NtO(MH='Freemartinism', MS='A condition occurring in the female offspring of dizygotic twins (TWIN, DIZYGOTIC) in a mixed-sex pregnancy, usually in CATTLE. Freemartinism can occur in other mammals. When placental fusion between the male and the female FETUSES permits the exchange of fetal cells and fetal hormones, TESTICULAR HORMONES from the male fetus can androgenize the female fetus producing a sterile XX/XY chimeric "female"(CHIMERISM).'),
  'D005612': NtO(MH='Freeze Drying', MS='Method of tissue preparation in which the tissue specimen is frozen and then dehydrated at low temperature in a high vacuum. This method is also used for dehydrating pharmaceutical and food products.'),
  'D005613': NtO(MH='Freeze Etching', MS='A replica technique in which cells are frozen to a very low temperature and cracked with a knife blade to expose the interior surfaces of the cells or cell membranes. The cracked cell surfaces are then freeze-dried to expose their constituents. The surfaces are now ready for shadowing to be viewed using an electron microscope. This method differs from freeze-fracturing in that no cryoprotectant is used and, thus, allows for the sublimation of water during the freeze-drying process to etch the surfaces.'),
  'D005614': NtO(MH='Freeze Fracturing', MS='Preparation for electron microscopy of minute replicas of exposed surfaces of the cell which have been ruptured in the frozen state. The specimen is frozen, then cleaved under high vacuum at the same temperature. The exposed surface is shadowed with carbon and platinum and coated with carbon to obtain a carbon replica.'),
  'D005615': NtO(MH='Freezing', MS='Liquids transforming into solids by the removal of heat.'),
  'D005616': NtO(MH='French Guiana', MS="A French overseas department on the northeast coast of South America. Its capital is Cayenne. It was first settled by the French in 1604. Early development was hindered because of the presence of a penal colony. The name of the country and the capital are variants of Guyana, possibly from the native Indian Guarani guai (born) + ana (kin), implying a united and interrelated race of people. (From Webster's New Geographical Dictionary, 1988, p418 & Room, Brewer's Dictionary of Names, 1992, p195)"),
  'D005617': NtO(MH='Chlorofluorocarbons, Methane', MS='A group of methane-based halogenated hydrocarbons containing one or more fluorine and chlorine atoms.'),
  'D005618': NtO(MH='Fresh Water', MS='Water containing no significant amounts of salts, such as water from RIVERS and LAKES.'),
  'D005619': NtO(MH='Freudian Theory', MS='Philosophic formulations which are basic to psychoanalysis. Some of the conceptual theories developed were of the libido, repression, regression, transference, id, ego, superego, Oedipus Complex, etc.'),
  'D005620': NtO(MH="Freund's Adjuvant", MS='An antigen solution emulsified in mineral oil. The complete form is made up of killed, dried mycobacteria, usually M. tuberculosis, suspended in the oil phase. It is effective in stimulating cell-mediated immunity (IMMUNITY, CELLULAR) and potentiates the production of certain IMMUNOGLOBULINS in some animals. The incomplete form does not contain mycobacteria.'),
  'D005621': NtO(MH='Friedreich Ataxia', MS='An autosomal recessive disease, usually of childhood onset, characterized pathologically by degeneration of the spinocerebellar tracts, posterior columns, and to a lesser extent the corticospinal tracts. Clinical manifestations include GAIT ATAXIA, pes cavus, speech impairment, lateral curvature of spine, rhythmic head tremor, kyphoscoliosis, congestive heart failure (secondary to a cardiomyopathy), and lower extremity weakness. Most forms of this condition are associated with a mutation in a gene on chromosome 9, at band q13, which codes for the mitochondrial protein frataxin. (From Adams et al., Principles of Neurology, 6th ed, p1081; N Engl J Med 1996 Oct 17;335(16):1169-75) The severity of Friedreich ataxia associated with expansion of GAA repeats in the first intron of the frataxin gene correlates with the number of trinucleotide repeats. (From Durr et al, N Engl J Med 1996 Oct 17;335(16):1169-75)'),
  'D005622': NtO(MH='Friend murine leukemia virus', MS='A strain of Murine leukemia virus (LEUKEMIA VIRUS, MURINE) producing leukemia of the reticulum-cell type with massive infiltration of liver, spleen, and bone marrow. It infects DBA/2 and Swiss mice.'),
  'D005624': NtO(MH='Frontal Bone', MS='The bone that forms the frontal aspect of the skull. Its flat part forms the forehead, articulating inferiorly with the NASAL BONE and the CHEEK BONE on each side of the face.'),
  'D005625': NtO(MH='Frontal Lobe', MS='The part of the cerebral hemisphere anterior to the central sulcus, and anterior and superior to the lateral sulcus.'),
  'D005626': NtO(MH='Frontal Sinus', MS='One of the paired, but seldom symmetrical, air spaces located between the inner and outer compact layers of the FRONTAL BONE in the forehead.'),
  'D005627': NtO(MH='Frostbite', MS='Damage to tissues as the result of low environmental temperatures.'),
  'D005628': NtO(MH='Frozen Foods', MS='Foods that are preserved by lowering their storage temperature below the freezing point.'),
  'D005629': NtO(MH='Frozen Sections', MS='Thinly cut sections of frozen tissue specimens prepared with a cryostat or freezing microtome.'),
  'D005630': NtO(MH='Fructans', MS='Polysaccharides composed of D-fructose units.'),
  'D005631': NtO(MH='Fructokinases', MS='A class of enzymes that catalyzes the phosphorylation of fructose in the presence of ATP. EC 2.7.1.-.'),
  'D005632': NtO(MH='Fructose', MS='A monosaccharide in sweet fruits and honey that is soluble in water, alcohol, or ether. It is used as a preservative and an intravenous infusion in parenteral feeding.'),
  'D005633': NtO(MH='Fructose Intolerance', MS='An autosomal recessive fructose metabolism disorder due to deficient fructose-1-phosphate aldolase (EC 2.1.2.13) activity, resulting in accumulation of fructose-1-phosphate. The accumulated fructose-1-phosphate inhibits glycogenolysis and gluconeogenesis, causing severe hypoglycemia following ingestion of fructose. Prolonged fructose ingestion in infants leads ultimately to hepatic failure and death. Patients develop a strong distaste for sweet food, and avoid a chronic course of the disease by remaining on a fructose- and sucrose-free diet.'),
  'D005634': NtO(MH='Fructose-Bisphosphate Aldolase', MS='An enzyme of the lyase class that catalyzes the cleavage of fructose 1,6-biphosphate to form dihydroxyacetone phosphate and glyceraldehyde 3-phosphate. The enzyme also acts on (3S,4R)-ketose 1-phosphates. The yeast and bacterial enzymes are zinc proteins. (Enzyme Nomenclature, 1992) E.C. 4.1.2.13.'),
  'D005635': NtO(MH='Fructosediphosphates', MS='Diphosphoric acid esters of fructose. The fructose-1,6- diphosphate isomer is most prevalent. It is an important intermediate in the glycolysis process.'),
  'D005636': NtO(MH='Fructosephosphates', MS=None),
  'D005637': NtO(MH='Fructuronate Reductase', MS='An enzyme that catalyzes the reversible oxidation of mannonate to fructuronate in the presence of NAD. Also reduces D-tagaturonate to D-altronate. EC 1.1.1.57.'),
  'D005638': NtO(MH='Fruit', MS='The fleshy or dry ripened ovary of a plant, enclosing the seed or seeds.'),
  'D005639': NtO(MH='Frustration', MS='The motivational and/or affective state resulting from being blocked, thwarted, disappointed or defeated.'),
  'D005640': NtO(MH='Follicle Stimulating Hormone', MS='A major gonadotropin secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR). Follicle-stimulating hormone stimulates GAMETOGENESIS and the supporting cells such as the ovarian GRANULOSA CELLS, the testicular SERTOLI CELLS, and LEYDIG CELLS. FSH consists of two noncovalently linked subunits, alpha and beta. Within a species, the alpha subunit is common in the three pituitary glycoprotein hormones (TSH, LH, and FSH), but the beta subunit is unique and confers its biological specificity.'),
  'D005641': NtO(MH='Tegafur', MS='Congener of FLUOROURACIL with comparable antineoplastic action. It has been suggested especially for the treatment of breast neoplasms.'),
  'D005642': NtO(MH="Fuchs' Endothelial Dystrophy", MS="Disorder caused by loss of endothelium of the central cornea. It is characterized by hyaline endothelial outgrowths on Descemet's membrane, epithelial blisters, reduced vision, and pain."),
  'D005643': NtO(MH='Fucose', MS='A six-member ring deoxysugar with the chemical formula C6H12O5. It lacks a hydroxyl group on the carbon at position 6 of the molecule.'),
  'D005644': NtO(MH='alpha-L-Fucosidase', MS='An enzyme that catalyzes the hydrolysis of an alpha L-fucoside to yield an alcohol and L-fucose. Deficiency of this enzyme can cause FUCOSIDOSIS. EC 3.2.1.51.'),
  'D005645': NtO(MH='Fucosidosis', MS='An autosomal recessive lysosomal storage disease caused by a deficiency of ALPHA-L-FUCOSIDASE activity resulting in an accumulation of fucose containing SPHINGOLIPIDS; GLYCOPROTEINS, and mucopolysaccharides (GLYCOSAMINOGLYCANS) in lysosomes. The infantile form (type I) features psychomotor deterioration, MUSCLE SPASTICITY, coarse facial features, growth retardation, skeletal abnormalities, visceromegaly, SEIZURES, recurrent infections, and MACROGLOSSIA, with death occurring in the first decade of life. Juvenile fucosidosis (type II) is the more common variant and features a slowly progressive decline in neurologic function and angiokeratoma corporis diffusum. Type II survival may be through the fourth decade of life. (From Menkes, Textbook of Child Neurology, 5th ed, p87; Am J Med Genet 1991 Jan;38(1):111-31)'),
  'D005646': NtO(MH='Fucosyl Galactose alpha-N-Acetylgalactosaminyltransferase', MS='An enzyme that catalyzes the transfer of acetylgalactosamine from UDP N-acetylgalactosamine to various 2-fucosylgalactosides as acceptors. EC 2.4.1.40.'),
  'D005647': NtO(MH='Fucosyltransferases', MS='Enzymes catalyzing the transfer of fucose from a nucleoside diphosphate fucose to an acceptor molecule which is frequently another carbohydrate, a glycoprotein, or a glycolipid molecule. Elevated activity of some fucosyltransferases in human serum may serve as an indicator of malignancy. The class includes EC 2.4.1.65; EC 2.4.1.68; EC 2.4.1.69; EC 2.4.1.89.'),
  'D005648': NtO(MH='Fuel Oils', MS='Complex petroleum hydrocarbons consisting mainly of residues from crude oil distillation. These liquid products include heating oils, stove oils, and furnace oils and are burned to generate energy.'),
  'D005649': NtO(MH='Fumarate Hydratase', MS='An enzyme that catalyzes the reversible hydration of fumaric acid to yield L-malic acid. It is one of the citric acid cycle enzymes. EC 4.2.1.2.'),
  'D005650': NtO(MH='Fumarates', MS='Compounds based on fumaric acid.'),
  'D005651': NtO(MH='Fumigation', MS='The application of smoke, vapor, or gas for the purpose of disinfecting or destroying pests or microorganisms.'),
  'D005652': NtO(MH='Functional Residual Capacity', MS='The volume of air remaining in the LUNGS at the end of a normal, quiet expiration. It is the sum of the RESIDUAL VOLUME and the EXPIRATORY RESERVE VOLUME. Common abbreviation is FRC.'),
  'D005653': NtO(MH='Fund Raising', MS='Usually organized community efforts to raise money to promote financial programs of institutions. The funds may include individual gifts.'),
  'D005654': NtO(MH='Fundus Oculi', MS='The concave interior of the eye, consisting of the retina, the choroid, the sclera, the optic disk, and blood vessels, seen by means of the ophthalmoscope. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D005655': NtO(MH='Funeral Rites', MS='Those customs and ceremonies pertaining to the dead.'),
  'D005656': NtO(MH='Fungal Proteins', MS='Proteins found in any species of fungus.'),
  'D005657': NtO(MH='Fungal Vaccines', MS='Suspensions of attenuated or killed fungi administered for the prevention or treatment of infectious fungal disease.'),
  'D005658': NtO(MH='Fungi', MS='A kingdom of eukaryotic, heterotrophic organisms that live parasitically as saprobes, including MUSHROOMS; YEASTS; smuts, molds, etc. They reproduce either sexually or asexually, and have life cycles that range from simple to complex. Filamentous fungi, commonly known as molds, refer to those that grow as multicellular colonies.'),
  'D005659': NtO(MH='Fungicides, Industrial', MS='Chemicals that kill or inhibit the growth of fungi in agricultural applications, on wood, plastics, or other materials, in swimming pools, etc.'),
  'D005660': NtO(MH='Funnel Chest', MS='A developmental anomaly in which the lower sternum is posteriorly dislocated and concavely deformed, resulting in a funnel-shaped thorax.'),
  'D005661': NtO(MH='Furagin', MS='Nitrofuran derivative anti-infective agent used for urinary tract infections.'),
  'D005662': NtO(MH='Furaldehyde', MS='A heterocyclic compound consisting of a furan where the hydrogen at position 2 is substituted by a formyl group.'),
  'D005663': NtO(MH='Furans', MS='Compounds with a 5-membered ring of four carbons and an oxygen. They are aromatic heterocycles. The reduced form is tetrahydrofuran.'),
  'D005664': NtO(MH='Furazolidone', MS='A nitrofuran derivative with antiprotozoal and antibacterial activity. Furazolidone acts by gradual inhibition of monoamine oxidase. (From Martindale, The Extra Pharmacopoeia, 30th ed, p514)'),
  'D005665': NtO(MH='Furosemide', MS='A benzoic-sulfonamide-furan. It is a diuretic with fast onset and short duration that is used for EDEMA and chronic RENAL INSUFFICIENCY.'),
  'D005666': NtO(MH='Fursultiamin', MS='Compound used for therapy of thiamine deficiency. It has also been suggested for several non-deficiency disorders but has not yet proven useful.'),
  'D005667': NtO(MH='Furunculosis', MS='A persistent skin infection marked by the presence of furuncles, often chronic and recurrent. In humans, the causative agent is various species of STAPHYLOCOCCUS. In salmonid fish (SALMONIDS), the pathogen is AEROMONAS SALMONICIDA.'),
  'D005668': NtO(MH='Furylfuramide', MS='Used formerly as antimicrobial food additive. It causes mutations in many cell cultures and may be carcinogenic.'),
  'D005669': NtO(MH='Fusaric Acid', MS='A picolinic acid derivative isolated from various Fusarium species. It has been proposed for a variety of therapeutic applications but is primarily used as a research tool. Its mechanisms of action are poorly understood. It probably inhibits DOPAMINE BETA-HYDROXYLASE, the enzyme that converts dopamine to norepinephrine. It may also have other actions, including the inhibition of cell proliferation and DNA synthesis.'),
  'D005670': NtO(MH='Fusarium', MS='A mitosporic Hypocreales fungal genus, various species of which are important parasitic pathogens of plants and a variety of vertebrates. Teleomorphs include GIBBERELLA.'),
  'D005671': NtO(MH='Fused Teeth', MS='Two teeth united during development by the union of their tooth germs; the teeth may be joined by the enamel of their crowns, by their root dentin, or by both.'),
  'D005672': NtO(MH='Fusidic Acid', MS='An antibiotic isolated from the fermentation broth of Fusidium coccineum. (From Merck Index, 11th ed). It acts by inhibiting translocation during protein synthesis.'),
  'D005673': NtO(MH='Fusobacterium', MS='A genus of gram-negative, anaerobic, rod-shaped bacteria found in cavities of humans and other animals. No endospores are formed. Some species are pathogenic and occur in various purulent or gangrenous infections.'),
  'D005674': NtO(MH='Fusobacterium Infections', MS='Infections with bacteria of the genus FUSOBACTERIUM.'),
  'D005675': NtO(MH='Fusobacterium necrophorum', MS='A species of gram-negative, non-spore-forming bacteria isolated from the natural cavities of man and other animals and from necrotic lesions, abscesses, and blood.'),
  'D005676': NtO(MH='Gravity Suits', MS="Double-layered inflatable suits which, when inflated, exert pressure on the lower part of the wearer's body. The suits are used to improve or stabilize the circulatory state, i.e., to prevent hypotension, control hemorrhage, and regulate blood pressure. The suits are also used by pilots under positive acceleration."),
  'D005677': NtO(MH='G(M1) Ganglioside', MS='A specific monosialoganglioside that accumulates abnormally within the nervous system due to a deficiency of GM1-b-galactosidase, resulting in GM1 gangliosidosis.'),
  'D005678': NtO(MH='G(M2) Ganglioside', MS='A glycosphingolipid that accumulates due to a deficiency of hexosaminidase A or B (BETA-N-ACETYLHEXOSAMINIDASES), or GM2 activator protein, resulting in GANGLIOSIDOSES, heredity metabolic disorders that include TAY-SACHS DISEASE and SANDHOFF DISEASE.'),
  'D005679': NtO(MH='G(M3) Ganglioside', MS='A ganglioside present in abnormally large amounts in the brain and liver due to a deficient biosynthetic enzyme, G(M3):UDP-N-acetylgalactosaminyltransferase. Deficiency of this enzyme prevents the formation of G(M2) ganglioside from G(M3) ganglioside and is the cause of an anabolic sphingolipidosis.'),
  'D005680': NtO(MH='gamma-Aminobutyric Acid', MS='The most common inhibitory neurotransmitter in the central nervous system.'),
  'D005681': NtO(MH='Gabon', MS='A republic in west equatorial Africa, south of CAMEROON and west of the CONGO. Its capital is Libreville.'),
  'D005682': NtO(MH='Gadolinium', MS='An element of the rare earth family of metals. It has the atomic symbol Gd, atomic number 64, and atomic weight 157.25. Its oxide is used in the control rods of some nuclear reactors.'),
  'D005683': NtO(MH='Gagging', MS='Contraction of the muscle of the PHARYNX caused by stimulation of sensory receptors on the SOFT PALATE, by psychic stimuli, or systemically by drugs.'),
  'D005684': NtO(MH='Gait', MS='Manner or style of walking.'),
  'D005685': NtO(MH='Galactans', MS='Polysaccharides composed of repeating galactose units. They can consist of branched or unbranched chains in any linkages.'),
  'D005686': NtO(MH='Galactokinase', MS='An enzyme that catalyzes reversibly the formation of galactose 1-phosphate and ADP from ATP and D-galactose. Galactosamine can also act as the acceptor. A deficiency of this enzyme results in GALACTOSEMIA. EC 2.7.1.6.'),
  'D005687': NtO(MH='Galactorrhea', MS='Excessive or inappropriate LACTATION in females or males, and not necessarily related to PREGNANCY. Galactorrhea can occur either unilaterally or bilaterally, and be profuse or sparse. Its most common cause is HYPERPROLACTINEMIA.'),
  'D005688': NtO(MH='Galactosamine', MS=None),
  'D005690': NtO(MH='Galactose', MS='An aldohexose that occurs naturally in the D-form in lactose, cerebrosides, gangliosides, and mucoproteins. Deficiency of galactosyl-1-phosphate uridyltransferase (GALACTOSE-1-PHOSPHATE URIDYL-TRANSFERASE DEFICIENCY DISEASE) causes an error in galactose metabolism called GALACTOSEMIA, resulting in elevations of galactose in the blood.'),
  'D005691': NtO(MH='Galactose Dehydrogenases', MS='D-Galactose:NAD(P)+ 1-oxidoreductases. Catalyzes the oxidation of D-galactose in the presence of NAD+ or NADP+ to D-galactono-gamma-lactone and NADH or NADPH. Includes EC 1.1.1.48 and EC 1.1.1.120.'),
  'D005692': NtO(MH='Galactose Oxidase', MS='An enzyme that oxidizes galactose in the presence of molecular oxygen to D-galacto-hexodialdose. It is a copper protein. EC 1.1.3.9.'),
  'D005693': NtO(MH='Galactosemias', MS='A group of inherited enzyme deficiencies which feature elevations of GALACTOSE in the blood. This condition may be associated with deficiencies of GALACTOKINASE; UDPGLUCOSE-HEXOSE-1-PHOSPHATE URIDYLYLTRANSFERASE; or UDPGLUCOSE 4-EPIMERASE. The classic form is caused by UDPglucose-Hexose-1-Phosphate Uridylyltransferase deficiency, and presents in infancy with FAILURE TO THRIVE; VOMITING; and INTRACRANIAL HYPERTENSION. Affected individuals also may develop MENTAL RETARDATION; JAUNDICE; hepatosplenomegaly; ovarian failure (PRIMARY OVARIAN INSUFFICIENCY); and cataracts. (From Menkes, Textbook of Child Neurology, 5th ed, pp61-3)'),
  'D005694': NtO(MH='UTP-Hexose-1-Phosphate Uridylyltransferase', MS='An enzyme that catalyzes the synthesis of UDPgalactose from UTP and galactose-1-phosphate. It is present in low levels in fetal and infant liver, but increases with age, thereby enabling galactosemic infants who survive to develop the capacity to metabolize galactose. EC 2.7.7.10.'),
  'D005695': NtO(MH='Galactosephosphates', MS='Phosphoric acid esters of galactose.'),
  'D005696': NtO(MH='Galactosidases', MS='A family of galactoside hydrolases that hydrolyze compounds with an O-galactosyl linkage. EC 3.2.1.-.'),
  'D005697': NtO(MH='Galactosides', MS='Glycosides formed by the reaction of the hydroxyl group on the anomeric carbon atom of galactose with an alcohol to form an acetal. They include both alpha- and beta-galactosides.'),
  'D005698': NtO(MH='Galactosylceramidase', MS='An enzyme that hydrolyzes galactose from ceramide monohexosides. Deficiency of this enzyme may cause globoid cell leukodystrophy (LEUKODYSTROPHY, GLOBOID CELL). EC 3.2.1.46.'),
  'D005699': NtO(MH='Galactosylceramides', MS='Cerebrosides which contain as their polar head group a galactose moiety bound in glycosidic linkage to the hydroxyl group of ceramide. Their accumulation in tissue, due to a defect in beta-galactosidase, is the cause of galactosylceramide lipidosis or globoid cell leukodystrophy.'),
  'D005700': NtO(MH='Galactosyltransferases', MS='Enzymes that catalyze the transfer of galactose from a nucleoside diphosphate galactose to an acceptor molecule which is frequently another carbohydrate. EC 2.4.1.-.'),
  'D005701': NtO(MH='Galago', MS='A genus of the family Lorisidae having four species which inhabit the forests and bush regions of Africa south of the Sahara and some nearby islands. The four species are G. alleni, G. crassicaudatus, G. demidovii, and G. senegalensis. There is another genus, Euoticus, containing two species which some authors have included in the Galago genus.'),
  'D005702': NtO(MH='Galantamine', MS='A benzazepine derived from norbelladine. It is found in GALANTHUS and other AMARYLLIDACEAE. It is a cholinesterase inhibitor that has been used to reverse the muscular effects of GALLAMINE TRIETHIODIDE and TUBOCURARINE and has been studied as a treatment for ALZHEIMER DISEASE and other central nervous system disorders.'),
  'D005703': NtO(MH='Gallamine Triethiodide', MS='A synthetic nondepolarizing blocking drug. The actions of gallamine triethiodide are similar to those of TUBOCURARINE, but this agent blocks the cardiac vagus and may cause sinus tachycardia and, occasionally, hypertension and increased cardiac output. It should be used cautiously in patients at risk from increased heart rate but may be preferred for patients with bradycardia. (From AMA Drug Evaluations Annual, 1992, p198)'),
  'D005704': NtO(MH='Gallbladder', MS='A storage reservoir for BILE secretion. Gallbladder allows the delivery of bile acids at a high concentration and in a controlled manner, via the CYSTIC DUCT to the DUODENUM, for degradation of dietary lipid.'),
  'D005705': NtO(MH='Gallbladder Diseases', MS='Diseases of the GALLBLADDER. They generally involve the impairment of BILE flow, GALLSTONES in the BILIARY TRACT, infections, neoplasms, or other diseases.'),
  'D005706': NtO(MH='Gallbladder Neoplasms', MS='Tumors or cancer of the gallbladder.'),
  'D005707': NtO(MH='Gallic Acid', MS='A colorless or slightly yellow crystalline compound obtained from nutgalls. It is used in photography, pharmaceuticals, and as an analytical reagent.'),
  'D005708': NtO(MH='Gallium', MS='A rare, metallic element designated by the symbol, Ga, atomic number 31, and atomic weight 69.72.'),
  'D005709': NtO(MH='Gallium Isotopes', MS='Stable gallium atoms that have the same atomic number as the element gallium, but differ in atomic weight. Ga-71 is a stable gallium isotope.'),
  'D005710': NtO(MH='Gallium Radioisotopes', MS='Unstable isotopes of gallium that decay or disintegrate emitting radiation. Ga atoms with atomic weights 63-68, 70 and 72-76 are radioactive gallium isotopes.'),
  'D005711': NtO(MH='Gallopamil', MS='Coronary vasodilator that is an analog of iproveratril (VERAPAMIL) with one more methoxy group on the benzene ring.'),
  'D005712': NtO(MH='Galvanic Skin Response', MS='A change in electrical resistance of the skin, occurring in emotion and in certain other conditions.'),
  'D005714': NtO(MH='Gambia', MS='A republic in western Africa, constituting an enclave within SENEGAL extending on both sides of the Gambia River. Its capital is Banjul, formerly Bathurst.'),
  'D005715': NtO(MH='Gambling', MS='An activity distinguished primarily by an element of risk in trying to obtain a desired goal, e.g., playing a game of chance for money.'),
  'D005716': NtO(MH='Game Theory', MS='Theoretical construct used in applied mathematics to analyze certain situations in which there is an interplay between parties that may have similar, opposed, or mixed interests. In a typical game, decision-making "players," who each have their own goals, try to gain advantage over the other parties by anticipating each other\'s decisions; the game is finally resolved as a consequence of the players\' decisions.'),
  'D005717': NtO(MH='Games, Experimental', MS='Games designed to provide information on hypotheses, policies, procedures, or strategies.'),
  'D005718': NtO(MH='Gametogenesis', MS='The process of germ cell development from the primordial GERM CELLS to the mature haploid GAMETES: ova in the female (OOGENESIS) or sperm in the male (SPERMATOGENESIS).'),
  'D005719': NtO(MH='gamma-Globulins', MS='Serum globulins that migrate to the gamma region (most positively charged) upon ELECTROPHORESIS. At one time, gamma-globulins came to be used as a synonym for immunoglobulins since most immunoglobulins are gamma globulins and conversely most gamma globulins are immunoglobulins. But since some immunoglobulins exhibit an alpha or beta electrophoretic mobility, that usage is in decline.'),
  'D005720': NtO(MH='Gamma Rays', MS='Penetrating, high-energy electromagnetic radiation emitted from atomic nuclei during NUCLEAR DECAY. The range of wavelengths of emitted radiation is between 0.1 - 100 pm which overlaps the shorter, more energetic hard X-RAYS wavelengths. The distinction between gamma rays and X-rays is based on their radiation source.'),
  'D005721': NtO(MH='Glutamate-Cysteine Ligase', MS='One of the enzymes active in the gamma-glutamyl cycle. It catalyzes the synthesis of gamma-glutamylcysteine from glutamate and cysteine in the presence of ATP with the formation of ADP and orthophosphate. EC 6.3.2.2.'),
  'D005722': NtO(MH='gamma-Glutamylcyclotransferase', MS='An enzyme that catalyzes the synthesis of pyroglutamate from a gamma-glutamyl-amino acid, also releasing the free amino acid. The enzyme acts on derivatives of glutamate, 2-aminobutyrate, alanine and glycine. The enzyme has been proposed to have a role in a gamma-glutamyl cycle for amino acid transport into cells in the intestines. EC 2.3.2.4.'),
  'D005723': NtO(MH='gamma-Glutamyltransferase', MS='An enzyme, sometimes called GGT, with a key role in the synthesis and degradation of GLUTATHIONE; (GSH, a tripeptide that protects cells from many toxins). It catalyzes the transfer of the gamma-glutamyl moiety to an acceptor amino acid.'),
  'D005724': NtO(MH='Ganglia', MS='Clusters of multipolar neurons surrounded by a capsule of loosely organized CONNECTIVE TISSUE located outside the CENTRAL NERVOUS SYSTEM.'),
  'D005725': NtO(MH='Ganglia, Autonomic', MS='Clusters of neurons and their processes in the autonomic nervous system. In the autonomic ganglia, the preganglionic fibers from the central nervous system synapse onto the neurons whose axons are the postganglionic fibers innervating target organs. The ganglia also contain intrinsic neurons and supporting cells and preganglionic fibers passing through to other ganglia.'),
  'D005726': NtO(MH='Ganglia, Parasympathetic', MS='Ganglia of the parasympathetic nervous system, including the ciliary, pterygopalatine, submandibular, and otic ganglia in the cranial region and intrinsic (terminal) ganglia associated with target organs in the thorax and abdomen.'),
  'D005727': NtO(MH='Ganglia, Spinal', MS='Sensory ganglia located on the dorsal spinal roots within the vertebral column. The spinal ganglion cells are pseudounipolar. The single primary branch bifurcates sending a peripheral process to carry sensory information from the periphery and a central branch which relays that information to the spinal cord or brain.'),
  'D005728': NtO(MH='Ganglia, Sympathetic', MS='Ganglia of the sympathetic nervous system including the paravertebral and the prevertebral ganglia. Among these are the sympathetic chain ganglia, the superior, middle, and inferior cervical ganglia, and the aorticorenal, celiac, and stellate ganglia.'),
  'D005729': NtO(MH='Ganglioneuroma', MS='A benign neoplasm that usually arises from the sympathetic trunk in the mediastinum. Histologic features include spindle cell proliferation (resembling a neurofibroma) and the presence of large ganglion cells. The tumor may present clinically with HORNER SYNDROME or diarrhea due to ectopic production of vasoactive intestinal peptide. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, p966)'),
  'D005730': NtO(MH='Ganglionic Blockers', MS='Agents having as their major action the interruption of neural transmission at nicotinic receptors on postganglionic autonomic neurons. Because their actions are so broad, including blocking of sympathetic and parasympathetic systems, their therapeutic use has been largely supplanted by more specific drugs. They may still be used in the control of blood pressure in patients with acute dissecting aortic aneurysm and for the induction of hypotension in surgery.'),
  'D005731': NtO(MH='Ganglionic Stimulants', MS='Agents that mimic neural transmission by stimulation of the nicotinic receptors on postganglionic autonomic neurons. Drugs that indirectly augment ganglionic transmission by increasing the release or slowing the breakdown of acetylcholine or by non-nicotinic effects on postganglionic neurons are not included here nor are the nonspecific cholinergic agonists.'),
  'D005732': NtO(MH='Gangliosides', MS='A subclass of ACIDIC GLYCOSPHINGOLIPIDS. They contain one or more sialic acid (N-ACETYLNEURAMINIC ACID) residues. Using the Svennerholm system of abbrevations, gangliosides are designated G for ganglioside, plus subscript M, D, or T for mono-, di-, or trisialo, respectively, the subscript letter being followed by a subscript arabic numeral to indicated sequence of migration in thin-layer chromatograms. (From Oxford Dictionary of Biochemistry and Molecular Biology, 1997)'),
  'D005733': NtO(MH='Gangliosidoses', MS='A group of autosomal recessive lysosomal storage disorders marked by the accumulation of GANGLIOSIDES. They are caused by impaired enzymes or defective cofactors required for normal ganglioside degradation in the LYSOSOMES. Gangliosidoses are classified by the specific ganglioside accumulated in the defective degradation pathway.'),
  'D005734': NtO(MH='Gangrene', MS='Death and putrefaction of tissue usually due to a loss of blood supply.'),
  'D005735': NtO(MH='Garbage', MS='Discarded animal and vegetable matter from a kitchen or the refuse from food preparation. (From Random House College Dictionary, 1982)'),
  'D005736': NtO(MH='Gardner Syndrome', MS='A variant of ADENOMATOUS POLYPOSIS COLI caused by mutation in the APC gene (GENES, APC) on CHROMOSOME 5. It is characterized by not only the presence of multiple colonic polyposis but also extracolonic ADENOMATOUS POLYPS in the UPPER GASTROINTESTINAL TRACT; the EYE; the SKIN; the SKULL; and the FACIAL BONES; as well as malignancy in organs other than the GI tract.'),
  'D005737': NtO(MH='Garlic', MS='A species of Allium used as a spice (SPICES) and traditional remedy. It contains alliin lyase and alliin, which is converted by alliin lyase to allicin, the pungent ingredient responsible for the aroma of fresh cut garlic.'),
  'D005738': NtO(MH='Gas Gangrene', MS='A severe condition resulting from bacteria invading healthy muscle from adjacent traumatized muscle or soft tissue. The infection originates in a wound contaminated with bacteria of the genus CLOSTRIDIUM. C. perfringens accounts for the majority of cases (over eighty percent), while C. noyvi, C. septicum, and C. histolyticum cause most of the other cases.'),
  'D005739': NtO(MH='Gas Poisoning', MS='Poisoning that results from exposure to gases such as CARBON MONOXIDE; NOBLE GASES; OXYGEN; or NATURAL GAS.'),
  'D005740': NtO(MH='Gases', MS="The vapor state of matter; nonelastic fluids in which the molecules are in free movement and their mean positions far apart. Gases tend to expand indefinitely, to diffuse and mix readily with other gases, to have definite relations of volume, temperature, and pressure, and to condense or liquefy at low temperatures or under sufficient pressure. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D005741': NtO(MH='Noble Gases', MS='Elements that constitute group 18 (formerly the zero group) of the periodic table. They are gases that generally do not react chemically.'),
  'D005742': NtO(MH='Gasoline', MS='Volative flammable fuel (liquid hydrocarbons) derived from crude petroleum by processes such as distillation reforming, polymerization, etc.'),
  'D005743': NtO(MH='Gastrectomy', MS='Excision of the whole (total gastrectomy) or part (subtotal gastrectomy, partial gastrectomy, gastric resection) of the stomach. (Dorland, 28th ed)'),
  'D005744': NtO(MH='Gastric Acid', MS='Hydrochloric acid present in GASTRIC JUICE.'),
  'D005745': NtO(MH='Gastric Acidity Determination', MS='Gastric analysis for determination of free acid or total acid.'),
  'D005746': NtO(MH='Gastric Emptying', MS='The evacuation of food from the stomach into the duodenum.'),
  'D005747': NtO(MH='Gastric Fistula', MS='Abnormal passage communicating with the STOMACH.'),
  'D005748': NtO(MH='Gastric Fundus', MS='The superior portion of the body of the stomach above the level of the cardiac notch.'),
  'D005749': NtO(MH='Gastric Inhibitory Polypeptide', MS='A gastrointestinal peptide hormone of about 43-amino acids. It is found to be a potent stimulator of INSULIN secretion and a relatively poor inhibitor of GASTRIC ACID secretion.'),
  'D005750': NtO(MH='Gastric Juice', MS="The liquid secretion of the stomach mucosa consisting of hydrochloric acid (GASTRIC ACID); PEPSINOGENS; INTRINSIC FACTOR; GASTRIN; MUCUS; and the bicarbonate ion (BICARBONATES). (From Best & Taylor's Physiological Basis of Medical Practice, 12th ed, p651)"),
  'D005751': NtO(MH='Gastric Lavage', MS='Medical procedure involving the emptying of contents in the stomach through the use of a tube inserted through the nose or mouth. It is performed to remove poisons or relieve pressure due to intestinal blockages or during surgery.'),
  'D005752': NtO(MH='Gastric Mucins', MS='Mucins that are found on the surface of the gastric epithelium. They play a role in protecting the epithelial layer from mechanical and chemical damage.'),
  'D005753': NtO(MH='Gastric Mucosa', MS='Lining of the STOMACH, consisting of an inner EPITHELIUM, a middle LAMINA PROPRIA, and an outer MUSCULARIS MUCOSAE. The surface cells produce MUCUS that protects the stomach from attack by digestive acid and enzymes. When the epithelium invaginates into the LAMINA PROPRIA at various region of the stomach (CARDIA; GASTRIC FUNDUS; and PYLORUS), different tubular gastric glands are formed. These glands consist of cells that secrete mucus, enzymes, HYDROCHLORIC ACID, or hormones.'),
  'D005755': NtO(MH='Gastrins', MS='A family of gastrointestinal peptide hormones that excite the secretion of GASTRIC JUICE. They may also occur in the central nervous system where they are presumed to be neurotransmitters.'),
  'D005756': NtO(MH='Gastritis', MS='Inflammation of the GASTRIC MUCOSA, a lesion observed in a number of unrelated disorders.'),
  'D005757': NtO(MH='Gastritis, Atrophic', MS='GASTRITIS with atrophy of the GASTRIC MUCOSA, the GASTRIC PARIETAL CELLS, and the mucosal glands leading to ACHLORHYDRIA. Atrophic gastritis usually progresses from chronic gastritis.'),
  'D005758': NtO(MH='Gastritis, Hypertrophic', MS='GASTRITIS with HYPERTROPHY of the GASTRIC MUCOSA. It is characterized by giant gastric folds, diminished acid secretion, excessive MUCUS secretion, and HYPOPROTEINEMIA. Symptoms include VOMITING; DIARRHEA; and WEIGHT LOSS.'),
  'D005759': NtO(MH='Gastroenteritis', MS='INFLAMMATION of any segment of the GASTROINTESTINAL TRACT from ESOPHAGUS to RECTUM. Causes of gastroenteritis are many including genetic, infection, HYPERSENSITIVITY, drug effects, and CANCER.'),
  'D005760': NtO(MH='Transmissible gastroenteritis virus', MS='A species of CORONAVIRUS causing a fatal disease to pigs under 3 weeks old.'),
  'D005761': NtO(MH='Gastroenteritis, Transmissible, of Swine', MS='A condition of chronic gastroenteritis in adult pigs and fatal gastroenteritis in piglets caused by a CORONAVIRUS.'),
  'D005762': NtO(MH='Gastroenterology', MS='A subspecialty of internal medicine concerned with the study of the physiology and diseases of the digestive system and related structures (esophagus, liver, gallbladder, and pancreas).'),
  'D005763': NtO(MH='Gastroenterostomy', MS='A variety of surgical reconstructive procedures devised to restore gastrointestinal continuity, The two major classes of reconstruction are the Billroth I (gastroduodenostomy) and Billroth II (gastrojejunostomy) procedures.'),
  'D005764': NtO(MH='Gastroesophageal Reflux', MS='Retrograde flow of gastric juice (GASTRIC ACID) and/or duodenal contents (BILE ACIDS; PANCREATIC JUICE) into the distal ESOPHAGUS, commonly due to incompetence of the LOWER ESOPHAGEAL SPHINCTER.'),
  'D005765': NtO(MH='Gastrointestinal Agents', MS='Drugs used for their effects on the gastrointestinal system, as to control gastric acidity, regulate gastrointestinal motility and water flow, and improve digestion.'),
  'D005766': NtO(MH='Gastrointestinal Contents', MS='The contents included in all or any segment of the GASTROINTESTINAL TRACT.'),
  'D005767': NtO(MH='Gastrointestinal Diseases', MS='Diseases in any segment of the GASTROINTESTINAL TRACT from ESOPHAGUS to RECTUM.'),
  'D005768': NtO(MH='Gastrointestinal Hormones', MS='HORMONES secreted by the gastrointestinal mucosa that affect the timing or the quality of secretion of digestive enzymes, and regulate the motor activity of the digestive system organs.'),
  'D005769': NtO(MH='Gastrointestinal Motility', MS='The motor activity of the GASTROINTESTINAL TRACT.'),
  'D005770': NtO(MH='Gastrointestinal Neoplasms', MS='Tumors or cancer of the GASTROINTESTINAL TRACT, from the MOUTH to the ANAL CANAL.'),
  'D005772': NtO(MH='Gastrointestinal Transit', MS='Passage of food (sometimes in the form of a test meal) through the gastrointestinal tract as measured in minutes or hours. The rate of passage through the intestine is an indicator of small bowel function.'),
  'D005773': NtO(MH='Gastroscopy', MS='Endoscopic examination, therapy or surgery of the interior of the stomach.'),
  'D005774': NtO(MH='Gastrostomy', MS='Creation of an artificial external opening into the stomach for nutritional support or gastrointestinal compression.'),
  'D005775': NtO(MH='Gastrula', MS='The developmental stage that follows BLASTULA or BLASTOCYST. It is characterized by the morphogenetic cell movements including invagination, ingression, and involution. Gastrulation begins with the formation of the PRIMITIVE STREAK, and ends with the formation of three GERM LAYERS, the body plan of the mature organism.'),
  'D005776': NtO(MH='Gaucher Disease', MS='An autosomal recessive disorder caused by a deficiency of acid beta-glucosidase (GLUCOSYLCERAMIDASE) leading to intralysosomal accumulation of glycosylceramide mainly in cells of the MONONUCLEAR PHAGOCYTE SYSTEM. The characteristic Gaucher cells, glycosphingolipid-filled HISTIOCYTES, displace normal cells in BONE MARROW and visceral organs causing skeletal deterioration, hepatosplenomegaly, and organ dysfunction. There are several subtypes based on the presence and severity of neurological involvement.'),
  'D005777': NtO(MH='Geese', MS='Any of various large waterfowl in the order Anseriformes, especially those of the genera Anser (gray geese) and Branta (black geese). They are larger than ducks but smaller than swans, prefer FRESH WATER, and occur primarily in the northern hemisphere.'),
  'D005778': NtO(MH='Gefarnate', MS='A water insoluble terpene fatty acid used in the treatment of gastrointestinal ulcers; it facilitates the healing and function of mucosal tissue.'),
  'D005779': NtO(MH='Immunodiffusion', MS='Technique involving the diffusion of antigen or antibody through a semisolid medium, usually agar or agarose gel, with the result being a precipitin reaction.'),
  'D005780': NtO(MH='Gelatin', MS='A product formed from skin, white connective tissue, or bone COLLAGEN. It is used as a protein food adjuvant, plasma substitute, hemostatic, suspending agent in pharmaceutical preparations, and in the manufacturing of capsules and suppositories.'),
  'D005781': NtO(MH='Gelatin Sponge, Absorbable', MS='Sterile, gelatin-base surgical sponge applied topically as an adjunct to hemostasis when the control of bleeding by conventional procedures is ineffective to reduce capillary ooze or is impractical. (From AMA Drug Evaluations Annual, 1994, p797)'),
  'D005782': NtO(MH='Gels', MS='Colloids with a solid continuous phase and liquid as the dispersed phase; gels may be unstable when, due to temperature or other cause, the solid phase liquefies; the resulting colloid is called a sol.'),
  'D005783': NtO(MH='Gender Identity', MS="A person's concept of self as being male and masculine or female and feminine, or ambivalent, based in part on physical characteristics, parental responses, and psychological and social pressures. It is the internal experience of gender role."),
  'D005784': NtO(MH='Gene Amplification', MS='A selective increase in the number of copies of a gene coding for a specific protein without a proportional increase in other genes. It occurs naturally via the excision of a copy of the repeating sequence from the chromosome and its extrachromosomal replication in a plasmid, or via the production of an RNA transcript of the entire repeating sequence of ribosomal RNA followed by the reverse transcription of the molecule to produce an additional copy of the original DNA sequence. Laboratory techniques have been introduced for inducing disproportional replication by unequal crossing over, uptake of DNA from lysed cells, or generation of extrachromosomal sequences from rolling circle replication.'),
  'D005785': NtO(MH='Gene Conversion', MS='The asymmetrical segregation of genes during replication which leads to the production of non-reciprocal recombinant strands and the apparent conversion of one allele into another. Thus, e.g., the meiotic products of an Aa individual may be AAAa or aaaA instead of AAaa, i.e., the A allele has been converted into the a allele or vice versa.'),
  'D005786': NtO(MH='Gene Expression Regulation', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control (induction or repression) of gene action at the level of transcription or translation.'),
  'D005787': NtO(MH='Gene Frequency', MS='The proportion of one particular in the total of all ALLELES for one genetic locus in a breeding POPULATION.'),
  'D005788': NtO(MH='Gene Pool', MS='The total genetic information possessed by the reproductive members of a POPULATION of sexually reproducing organisms.'),
  'D005789': NtO(MH='Genealogy and Heraldry', MS='Descent of a person, family, or group from an ancestor and coats of arms and other armorial bearings associated with an ancestor.'),
  'D005790': NtO(MH='General Adaptation Syndrome', MS='The sum of all nonspecific systemic reactions of the body to long-continued exposure to systemic stress.'),
  'D005791': NtO(MH='Patient Care', MS='The services rendered by members of the health profession and non-professionals under their supervision.'),
  'D005792': NtO(MH='General Practice, Dental', MS='Nonspecialized dental practice which is concerned with providing primary and continuing dental care.'),
  'D005793': NtO(MH='Generalization, Psychological', MS="The phenomenon of an organism's responding to all situations similar to one in which it has been conditioned."),
  'D005794': NtO(MH='Generalization, Response', MS='The principle that after an organism learns to respond in a particular manner to a stimulus, that stimulus is effective in eliciting similar responses.'),
  'D005795': NtO(MH='Generalization, Stimulus', MS='The tendency to react to stimuli that are different from, but somewhat similar to, the stimulus used as a conditioned stimulus.'),
  'D005796': NtO(MH='Genes', MS='A category of nucleic acid sequences that function as units of heredity and which code for the basic instructions for the development, reproduction, and maintenance of organisms.'),
  'D005797': NtO(MH='Genes, araC', MS='Regulatory genes which encode a cyclic AMP receptor protein required for L-arabinose utilization in E. coli. It is an example of positive control or regulation of gene expression in the bacterial operon.'),
  'D005798': NtO(MH='Genes, Bacterial', MS='The functional hereditary units of BACTERIA.'),
  'D005799': NtO(MH='Genes, Dominant', MS='Genes that influence the PHENOTYPE both in the homozygous and the heterozygous state.'),
  'D005800': NtO(MH='Genes, Fungal', MS='The functional hereditary units of FUNGI.'),
  'D005801': NtO(MH='Genes, Homeobox', MS='Genes that encode highly conserved TRANSCRIPTION FACTORS that control positional identity of cells (BODY PATTERNING) and MORPHOGENESIS throughout development. Their sequences contain a 180 nucleotide sequence designated the homeobox, so called because mutations of these genes often results in homeotic transformations, in which one body structure replaces another. The proteins encoded by homeobox genes are called HOMEODOMAIN PROTEINS.'),
  'D005802': NtO(MH='Genes, MHC Class II', MS='Genetic loci in the vertebrate major histocompatibility complex that encode polymorphic products which control the immune response to specific antigens. The genes are found in the HLA-D region in humans and include H-2M, I-A, and I-E loci in mice.'),
  'D005803': NtO(MH='Genes, Immunoglobulin', MS='Genes encoding the different subunits of the IMMUNOGLOBULINS, for example the IMMUNOGLOBULIN LIGHT CHAIN GENES and the IMMUNOGLOBULIN HEAVY CHAIN GENES. The heavy and light immunoglobulin genes are present as gene segments in the germline cells. The completed genes are created when the segments are shuffled and assembled (B-LYMPHOCYTE GENE REARRANGEMENT) during B-LYMPHOCYTE maturation. The gene segments of the human light and heavy chain germline genes are symbolized V (variable), J (joining) and C (constant). The heavy chain germline genes have an additional segment D (diversity).'),
  'D005804': NtO(MH='Genes, Lethal', MS='Genes whose loss of function or gain of function MUTATION leads to the death of the carrier prior to maturity. They may be essential genes (GENES, ESSENTIAL) required for viability, or genes which cause a block of function of an essential gene at a time when the essential gene function is required for viability.'),
  'D005805': NtO(MH='Genes, MHC Class I', MS='Genetic loci in the vertebrate major histocompatibility complex which encode polymorphic characteristics not related to immune responsiveness or complement activity, e.g., B loci (chicken), DLA (dog), GPLA (guinea pig), H-2 (mouse), RT-1 (rat), HLA-A, -B, and -C class I genes of man.'),
  'D005807': NtO(MH='Genes, Overlapping', MS='Genes whose nucleotide sequences overlap to some degree. The overlapped sequences may involve structural or regulatory genes of eukaryotic or prokaryotic cells.'),
  'D005808': NtO(MH='Genes, Recessive', MS='Genes that influence the PHENOTYPE only in the homozygous state.'),
  'D005809': NtO(MH='Genes, Regulator', MS='Genes which regulate or circumscribe the activity of other genes; specifically, genes which code for PROTEINS or RNAs which have GENE EXPRESSION REGULATION functions.'),
  'D005810': NtO(MH='Multigene Family', MS='A set of genes descended by duplication and variation from some ancestral gene. Such genes may be clustered together on the same chromosome or dispersed on different chromosomes. Examples of multigene families include those that encode the hemoglobins, immunoglobulins, histocompatibility antigens, actins, tubulins, keratins, collagens, heat shock proteins, salivary glue proteins, chorion proteins, cuticle proteins, yolk proteins, and phaseolins, as well as histones, ribosomal RNA, and transfer RNA genes. The latter three are examples of reiterated genes, where hundreds of identical genes are present in a tandem array. (King & Stanfield, A Dictionary of Genetics, 4th ed)'),
  'D005812': NtO(MH='Genes, Switch', MS='Genes that cause the epigenotype (i.e., the interrelated developmental pathways through which the adult organism is realized) to switch to an alternate cell lineage-related pathway. Switch complexes control the expression of normal functional development as well as oncogenic transformation.'),
  'D005813': NtO(MH='Genes, Synthetic', MS='Biologically functional sequences of DNA chemically synthesized in vitro.'),
  'D005814': NtO(MH='Genes, Viral', MS='The functional hereditary units of VIRUSES.'),
  'D005815': NtO(MH='Genetic Code', MS='The meaning ascribed to the BASE SEQUENCE with respect to how it is translated into AMINO ACID SEQUENCE. The start, stop, and order of amino acids of a protein is specified by consecutive triplets of nucleotides called codons (CODON).'),
  'D005816': NtO(MH='Genetic Complementation Test', MS='A test used to determine whether or not complementation (compensation in the form of dominance) will occur in a cell with a given mutant phenotype when another mutant genome, encoding the same mutant phenotype, is introduced into that cell.'),
  'D005817': NtO(MH='Genetic Counseling', MS='An educational process that provides information and advice to individuals or families about a genetic condition that may affect them. The purpose is to help individuals make informed decisions about marriage, reproduction, and other health management issues based on information about the genetic disease, the available diagnostic tests, and management programs. Psychosocial support is usually offered.'),
  'D005818': NtO(MH='Genetic Engineering', MS='Directed modification of the gene complement of a living organism by such techniques as altering the DNA, substituting genetic material by means of a virus, transplanting whole nuclei, transplanting cell hybrids, etc.'),
  'D005819': NtO(MH='Genetic Markers', MS='A phenotypically recognizable genetic trait which can be used to identify a genetic locus, a linkage group, or a recombination event.'),
  'D005820': NtO(MH='Genetic Testing', MS='Detection of a MUTATION; GENOTYPE; KARYOTYPE; or specific ALLELES associated with genetic traits, heritable diseases, or predisposition to a disease, or that may lead to the disease in descendants. It includes prenatal genetic testing.'),
  'D005821': NtO(MH='Genetic Techniques', MS='Chromosomal, biochemical, intracellular, and other methods used in the study of genetics.'),
  'D005822': NtO(MH='Genetic Vectors', MS='DNA molecules capable of autonomous replication within a host cell and into which other DNA sequences can be inserted and thus amplified. Many are derived from PLASMIDS; BACTERIOPHAGES; or VIRUSES. They are used for transporting foreign genes into recipient cells. Genetic vectors possess a functional replicator site and contain GENETIC MARKERS to facilitate their selective recognition.'),
  'D005823': NtO(MH='Genetics', MS='The branch of science concerned with the means and consequences of transmission and generation of the components of biological inheritance. (Stedman, 26th ed)'),
  'D005824': NtO(MH='Genetics, Behavioral', MS='The experimental study of the relationship between the genotype of an organism and its behavior. The scope includes the effects of genes on simple sensory processes to complex organization of the nervous system.'),
  'D005826': NtO(MH='Genetics, Medical', MS='A subdiscipline of human genetics which entails the reliable prediction of certain human disorders as a function of the lineage and/or genetic makeup of an individual or of any two parents or potential parents.'),
  'D005827': NtO(MH='Genetics, Microbial', MS='A subdiscipline of genetics which deals with the genetic mechanisms and processes of microorganisms.'),
  'D005828': NtO(MH='Genetics, Population', MS='The discipline studying genetic composition of populations and effects of factors such as GENETIC SELECTION, population size, MUTATION, migration, and GENETIC DRIFT on the frequencies of various GENOTYPES and PHENOTYPES using a variety of GENETIC TECHNIQUES.'),
  'D005829': NtO(MH='Geniculate Bodies', MS='Part of the DIENCEPHALON inferior to the caudal end of the dorsal THALAMUS. Includes the lateral geniculate body which relays visual impulses from the OPTIC TRACT to the calcarine cortex, and the medial geniculate body which relays auditory impulses from the lateral lemniscus to the AUDITORY CORTEX.'),
  'D005830': NtO(MH='Geniculate Ganglion', MS='The sensory ganglion of the facial (7th cranial) nerve. The geniculate ganglion cells send central processes to the brain stem and peripheral processes to the taste buds in the anterior tongue, the soft palate, and the skin of the external auditory meatus and the mastoid process.'),
  'D005831': NtO(MH='Genital Diseases, Female', MS='Pathological processes involving the female reproductive tract (GENITALIA, FEMALE).'),
  'D005832': NtO(MH='Genital Diseases, Male', MS='Pathological processes involving the male reproductive tract (GENITALIA, MALE).'),
  'D005833': NtO(MH='Genital Neoplasms, Female', MS='Tumor or cancer of the female reproductive tract (GENITALIA, FEMALE).'),
  'D005834': NtO(MH='Genital Neoplasms, Male', MS='Tumor or cancer of the MALE GENITALIA.'),
  'D005835': NtO(MH='Genitalia', MS='The external and internal organs involved in the functions of REPRODUCTION.'),
  'D005836': NtO(MH='Genitalia, Female', MS="The female reproductive organs. The external organs include the VULVA; BARTHOLIN'S GLANDS; and CLITORIS. The internal organs include the VAGINA; UTERUS; OVARY; and FALLOPIAN TUBES."),
  'D005837': NtO(MH='Genitalia, Male', MS='The male reproductive organs. They are divided into the external organs (PENIS; SCROTUM; and URETHRA) and the internal organs (TESTIS; EPIDIDYMIS; VAS DEFERENS; SEMINAL VESICLES; EJACULATORY DUCTS; PROSTATE; and BULBOURETHRAL GLANDS).'),
  'D005838': NtO(MH='Genotype', MS='The genetic constitution of the individual, comprising the ALLELES present at each GENETIC LOCUS.'),
  'D005839': NtO(MH='Gentamicins', MS='A complex of closely related aminoglycosides obtained from MICROMONOSPORA purpurea and related species. They are broad-spectrum antibiotics, but may cause ear and kidney damage. They act to inhibit PROTEIN BIOSYNTHESIS.'),
  'D005840': NtO(MH='Gentian Violet', MS='A dye that is a mixture of violet rosanilinis with antibacterial, antifungal, and anthelmintic properties.'),
  'D005841': NtO(MH='Gentisates', MS='Salts and esters of gentisic acid.'),
  'D005842': NtO(MH='Geographic Locations', MS='The continents and countries situated on those continents; the UNITED STATES and each of the constituent states arranged by region; CANADA and each of its provinces; AUSTRALIA and each of its states; the major bodies of water and major islands on both hemispheres; and selected major cities.'),
  'D005843': NtO(MH='Geography', MS='The science dealing with the earth and its life, especially the description of land, sea, and air and the distribution of plant and animal life, including humanity and human industries with reference to the mutual relations of these elements. (From Webster, 3d ed)'),
  'D005844': NtO(MH='Geology', MS='The science of the earth and other celestial bodies and their history as recorded in the rocks. It includes the study of geologic processes of an area such as rock formations, weathering and erosion, and sedimentation. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D005845': NtO(MH='Georgia', MS='A state located in the southeastern United States, The capital is Atlanta.'),
  'D005846': NtO(MH='Georgia (Republic)', MS='A country in southwestern Asia, bordering the Black Sea, between Turkey and Russia. The capital is Tbilisi.'),
  'D005847': NtO(MH='Geotrichosis', MS='Infection due to the fungus Geotrichum.'),
  'D005848': NtO(MH='Geotrichum', MS='A mitosporic Saccharomycetales fungal genus, various species of which have been isolated from pulmonary lesions. Teleomorphs include Dipodascus and Galactomyces.'),
  'D005849': NtO(MH='Gerbillinae', MS='A subfamily of the Muridae consisting of several genera including Gerbillus, Rhombomys, Tatera, Meriones, and Psammomys.'),
  'D005850': NtO(MH='Geriatric Dentistry', MS='The branch of dentistry concerned with the dental problems of older people.'),
  'D005851': NtO(MH='Geriatric Nursing', MS='Nursing care of the aged patient given in the home, the hospital, or special institutions such as nursing homes, psychiatric institutions, etc.'),
  'D005852': NtO(MH='Geriatric Psychiatry', MS='A subspecialty of psychiatry concerned with the mental health of the aged.'),
  'D005853': NtO(MH='Geriatrics', MS='The branch of medicine concerned with the physiological and pathological aspects of the aged, including the clinical problems of senescence and senility.'),
  'D005854': NtO(MH='Germ Cells', MS='The reproductive cells in multicellular organisms at various stages during GAMETOGENESIS.'),
  'D005855': NtO(MH='Germ Layers', MS='The three primary germinal layers (ECTODERM; ENDODERM; and MESODERM) developed during GASTRULATION that provide tissues and body plan of a mature organism. They derive from two early layers, hypoblast and epiblast.'),
  'D005856': NtO(MH='Germ-Free Life', MS='Animals not contaminated by or associated with any foreign organisms.'),
  'D005857': NtO(MH='Germanium', MS='A rare metal element with a blue-gray appearance and atomic symbol Ge, atomic number 32, and atomic weight 72.63.'),
  'D005858': NtO(MH='Germany', MS='A country in central Europe, bordering the Baltic Sea and the North Sea, between the Netherlands and Poland, south of Denmark. The capital is Berlin.'),
  'D005859': NtO(MH='Germany, East', MS='The former German Democratic Republic which was reunified with the former Federal Republic of Germany in 1990.'),
  'D005860': NtO(MH='Germany, West', MS='The former Federal Republic of Germany which was reunified with the former German Democratic Republic in 1990.'),
  'D005861': NtO(MH='Germine Acetates', MS='Germine derivatives acetylated on any one or more of the hydroxy groups. These compounds are present in many polyester alkaloids which occur in Veratrum and Zygadenus species. They are used as antihypertensive agents, and in some cases, exhibit curare-like activity.'),
  'D005862': NtO(MH='Gerstmann Syndrome', MS='A disorder of cognition characterized by the tetrad of finger agnosia, dysgraphia, DYSCALCULIA, and right-left disorientation. The syndrome may be developmental or acquired. Acquired Gerstmann syndrome is associated with lesions in the dominant (usually left) PARIETAL LOBE which involve the angular gyrus or subjacent white matter. (From Adams et al., Principles of Neurology, 6th ed, p457)'),
  'D005863': NtO(MH='Gestalt Theory', MS='A system which emphasizes that experience and behavior contain basic patterns and relationships which cannot be reduced to simpler components; that is, the whole is greater than the sum of its parts.'),
  'D005864': NtO(MH='Gestalt Therapy', MS='A form of psychotherapy with emphasis on the interplay of organism and environment. Basic to this therapy is the development of awareness and maturity, as well as self-confidence.'),
  'D005865': NtO(MH='Gestational Age', MS='The age of the conceptus, beginning from the time of FERTILIZATION. In clinical obstetrics, the gestational age is often estimated as the time from the last day of the last MENSTRUATION which is about 2 weeks before OVULATION and fertilization.'),
  'D005866': NtO(MH='Gestonorone Caproate', MS='A long-acting potent progestogen structurally related to PROGESTERONE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1185)'),
  'D005867': NtO(MH='Gestrinone', MS='A non-estrogenic contraceptive which is a weak progestin with strong anti-progesterone properties. It is effective if used once a week orally or can also be used in intravaginal devices.'),
  'D005868': NtO(MH='Gestures', MS='Movement of a part of the body for the purpose of communication.'),
  'D005869': NtO(MH='Ghana', MS='A republic in western Africa, south of BURKINA FASO and west of TOGO. Its capital is Accra.'),
  'D005870': NtO(MH='Giant Cell Tumors', MS='Tumors of bone tissue or synovial or other soft tissue characterized by the presence of giant cells. The most common are giant cell tumor of tendon sheath and GIANT CELL TUMOR OF BONE.'),
  'D005871': NtO(MH='Castleman Disease', MS='Large benign, hyperplastic lymph nodes. The more common hyaline vascular subtype is characterized by small hyaline vascular follicles and interfollicular capillary proliferations. Plasma cells are often present and represent another subtype with the plasma cells containing IgM and IMMUNOGLOBULIN A.'),
  'D005872': NtO(MH='Giardia', MS='A genus of flagellate intestinal EUKARYOTES parasitic in various vertebrates, including humans. Characteristics include the presence of four pairs of flagella arising from a complicated system of axonemes and cysts that are ellipsoidal to ovoidal in shape.'),
  'D005873': NtO(MH='Giardiasis', MS='An infection of the SMALL INTESTINE caused by the flagellated protozoan GIARDIA. It is spread via contaminated food and water and by direct person-to-person contact.'),
  'D005874': NtO(MH='Gibberella', MS='A genus of ascomycetous fungi of the family Hypocreaceae, order Hypocreales including several pathogens of grains and cereals. It is also the source of plant growth regulators such as gibberellin and gibberellic acid.'),
  'D005875': NtO(MH='Gibberellins', MS='A class of plant growth hormone isolated from cultures of Gibberella fujikuroi, a fungus causing Bakanae disease in rice. There are many different members of the family as well as mixtures of multiple members; all are diterpenoid acids based on the gibberellane skeleton.'),
  'D005876': NtO(MH='Gibraltar', MS='An overseas territory of the United Kingdom in southwestern Europe, bordering the Strait of Gibraltar.'),
  'D005877': NtO(MH='Gigantism', MS='The condition of accelerated and excessive GROWTH in children or adolescents who are exposed to excess HUMAN GROWTH HORMONE before the closure of EPIPHYSES. It is usually caused by somatotroph hyperplasia or a GROWTH HORMONE-SECRETING PITUITARY ADENOMA. These patients are of abnormally tall stature, more than 3 standard deviations above normal mean height for age.'),
  'D005878': NtO(MH='Gilbert Disease', MS='A benign familial disorder, transmitted as an autosomal dominant trait. It is characterized by low-grade chronic hyperbilirubinemia with considerable daily fluctuations of the bilirubin level.'),
  'D005879': NtO(MH='Tourette Syndrome', MS='A neuropsychological disorder related to alterations in DOPAMINE metabolism and neurotransmission involving frontal-subcortical neuronal circuits. Both multiple motor and one or more vocal tics need to be present with TICS occurring many times a day, nearly daily, over a period of more than one year. The onset is before age 18 and the disturbance is not due to direct physiological effects of a substance or a another medical condition. The disturbance causes marked distress or significant impairment in social, occupational, or other important areas of functioning. (From DSM-IV, 1994; Neurol Clin 1997 May;15(2):357-79)'),
  'D005880': NtO(MH='Gills', MS='Paired respiratory organs of fishes and some amphibians that are analogous to lungs. They are richly supplied with blood vessels by which oxygen and carbon dioxide are exchanged directly with the environment.'),
  'D005881': NtO(MH='Gingiva', MS='Oral tissue surrounding and attached to TEETH.'),
  'D005882': NtO(MH='Gingival Diseases', MS='Diseases involving the GINGIVA.'),
  'D005883': NtO(MH='Gingival Crevicular Fluid', MS='A fluid occurring in minute amounts in the gingival crevice, believed by some authorities to be an inflammatory exudate and by others to cleanse material from the crevice, containing sticky plasma proteins which improve adhesions of the epithelial attachment, have antimicrobial properties, and exert antibody activity. (From Jablonski, Illustrated Dictionary of Dentistry, 1982)'),
  'D005884': NtO(MH='Gingival Hemorrhage', MS='The flowing of blood from the marginal gingival area, particularly the sulcus, seen in such conditions as GINGIVITIS, marginal PERIODONTITIS, injury, and ASCORBIC ACID DEFICIENCY.'),
  'D005885': NtO(MH='Gingival Hyperplasia', MS="Non-inflammatory enlargement of the gingivae produced by factors other than local irritation. It is characteristically due to an increase in the number of cells. (From Jablonski's Dictionary of Dentistry, 1992, p400)"),
  'D005886': NtO(MH='Gingival Hypertrophy', MS='Abnormal enlargement or overgrowth of the gingivae brought about by enlargement of existing cells.'),
  'D005887': NtO(MH='Gingival Neoplasms', MS='New abnormal growth of tissue in the GINGIVA.'),
  'D005888': NtO(MH='Gingival Pocket', MS='An abnormal extension of a gingival sulcus not accompanied by the apical migration of the epithelial attachment.'),
  'D005889': NtO(MH='Gingival Recession', MS='Exposure of the root surface when the edge of the gum (GINGIVA) moves apically away from the crown of the tooth. This is common with advancing age, vigorous tooth brushing, diseases, or tissue loss of the gingiva, the PERIODONTAL LIGAMENT and the supporting bone (ALVEOLAR PROCESS).'),
  'D005890': NtO(MH='Gingivectomy', MS='Surgical excision of the gingiva at the level of its attachment, thus creating new marginal gingiva. This procedure is used to eliminate gingival or periodontal pockets or to provide an approach for extensive surgical interventions, and to gain access necessary to remove calculus within the pocket. (Dorland, 28th ed)'),
  'D005891': NtO(MH='Gingivitis', MS='Inflammation of gum tissue (GINGIVA) without loss of connective tissue.'),
  'D005892': NtO(MH='Gingivitis, Necrotizing Ulcerative', MS='An acute or chronic GINGIVITIS characterized by redness and swelling, NECROSIS extending from the interdental papillae along the gingival margins, PAIN; HEMORRHAGE, necrotic odor, and often a pseudomembrane. The condition may extend to the ORAL MUCOSA; TONGUE; PALATE; or PHARYNX. The etiology is somewhat unclear, but may involve a complex of FUSOBACTERIUM NUCLEATUM along with spirochetes BORRELIA or TREPONEMA.'),
  'D005893': NtO(MH='Gingivoplasty', MS='Surgical reshaping of the gingivae and papillae for correction of deformities (particularly enlargements) and to provide the gingivae with a normal and functional form, the incision creating an external bevel. (Dorland, 28th ed)'),
  'D005894': NtO(MH='Panax', MS='An araliaceous genus of plants that contains a number of pharmacologically active agents used as stimulants, sedatives, and tonics, especially in traditional medicine. Sometimes confused with Siberian ginseng (ELEUTHEROCOCCUS).'),
  'D005895': NtO(MH='Gizzard, Avian', MS='The posterior portion of the avian stomach located between the PROVENTRICULUS and INTESTINE. It consists of a thick muscular wall and small stones (gastroliths) that function to mechanically grind SEEDS and other ingested objects.'),
  'D005896': NtO(MH='Glanders', MS='A contagious disease of horses that can be transmitted to humans. It is caused by BURKHOLDERIA MALLEI and characterized by ulceration of the respiratory mucosa and an eruption of nodules on the skin.'),
  'D005897': NtO(MH='Glafenine', MS='An anthranilic acid derivative with analgesic properties used for the relief of all types of pain.'),
  'D005898': NtO(MH='Glass', MS='Hard, amorphous, brittle, inorganic, usually transparent, polymerous silicate of basic oxides, usually potassium or sodium. It is used in the form of hard sheets, vessels, tubing, fibers, ceramics, beads, etc.'),
  'D005899': NtO(MH='Glass Ionomer Cements', MS='A polymer obtained by reacting polyacrylic acid with a special anion-leachable glass (alumino-silicate). The resulting cement is more durable and tougher than others in that the materials comprising the polymer backbone do not leach out.'),
  'D005900': NtO(MH='Glaucarubin', MS='(1 beta,2 alpha,11 beta,12 alpha,15 beta(S))-11,20-Epoxy-1,2,11,12-tetrahydroxy-15-(2-hydroxy-2-methyl-1-oxobutoxy)picras-3-en-16-one. A quassinoid (Simaroubolide) from Simaruba glauca, a tropical shrub. It has been used as an antiamebic agent and is found to be cytotoxic. It may be of use in cancer chemotherapy.'),
  'D005901': NtO(MH='Glaucoma', MS='An ocular disease, occurring in many forms, having as its primary characteristics an unstable or a sustained increase in the intraocular pressure which the eye cannot withstand without damage to its structure or impairment of its function. The consequences of the increased pressure may be manifested in a variety of symptoms, depending upon type and severity, such as excavation of the optic disk, hardness of the eyeball, corneal anesthesia, reduced visual acuity, seeing of colored halos around lights, disturbed dark adaptation, visual field defects, and headaches. (Dictionary of Visual Science, 4th ed)'),
  'D005902': NtO(MH='Glaucoma, Open-Angle', MS='Glaucoma in which the angle of the anterior chamber is open and the trabecular meshwork does not encroach on the base of the iris.'),
  'D005903': NtO(MH='Gliadin', MS='Simple protein, one of the prolamines, derived from the gluten of wheat, rye, etc. May be separated into 4 discrete electrophoretic fractions. It is the toxic factor associated with CELIAC DISEASE.'),
  'D005904': NtO(MH='Glial Fibrillary Acidic Protein', MS='An intermediate filament protein found only in glial cells or cells of glial origin. MW 51,000.'),
  'D005905': NtO(MH='Glyburide', MS='An antidiabetic sulfonylurea derivative with actions similar to those of chlorpropamide.'),
  'D005907': NtO(MH='Gliclazide', MS='An oral sulfonylurea hypoglycemic agent which stimulates insulin secretion.'),
  'D005909': NtO(MH='Glioblastoma', MS='A malignant form of astrocytoma histologically characterized by pleomorphism of cells, nuclear atypia, microhemorrhage, and necrosis. They may arise in any region of the central nervous system, with a predilection for the cerebral hemispheres, basal ganglia, and commissural pathways. Clinical presentation most frequently occurs in the fifth or sixth decade of life with focal neurologic signs or seizures.'),
  'D005910': NtO(MH='Glioma', MS='Benign and malignant central nervous system neoplasms derived from glial cells (i.e., astrocytes, oligodendrocytes, and ependymocytes). Astrocytes may give rise to astrocytomas (ASTROCYTOMA) or glioblastoma multiforme (see GLIOBLASTOMA). Oligodendrocytes give rise to oligodendrogliomas (OLIGODENDROGLIOMA) and ependymocytes may undergo transformation to become EPENDYMOMA; CHOROID PLEXUS NEOPLASMS; or colloid cysts of the third ventricle. (From Escourolle et al., Manual of Basic Neuropathology, 2nd ed, p21)'),
  'D005911': NtO(MH='Gliosis', MS='The production of a dense fibrous network of neuroglia; includes astrocytosis, which is a proliferation of astrocytes in the area of a degenerative lesion.'),
  'D005912': NtO(MH='Gliotoxin', MS='A fungal toxin produced by various species of Trichoderma, Gladiocladium fimbriatum, Aspergillus fumigatus, and Penicillium. It is used as an immunosuppressive agent.'),
  'D005913': NtO(MH='Glipizide', MS='An oral hypoglycemic agent which is rapidly absorbed and completely metabolized.'),
  'D005914': NtO(MH='Globins', MS='A superfamily of proteins containing the globin fold which is composed of 6-8 alpha helices arranged in a characterstic HEME enclosing structure.'),
  'D005915': NtO(MH='Globosides', MS='Glycosphingolipids containing N-acetylglucosamine (paragloboside) or N-acetylgalactosamine (globoside). Globoside is the P antigen on erythrocytes and paragloboside is an intermediate in the biosynthesis of erythrocyte blood group ABH and P 1 glycosphingolipid antigens. The accumulation of globoside in tissue, due to a defect in hexosaminidases A and B, is the cause of Sandhoff disease.'),
  'D005916': NtO(MH='Globulins', MS=None),
  'D005917': NtO(MH='Globus Pallidus', MS='The representation of the phylogenetically oldest part of the corpus striatum called the paleostriatum. It forms the smaller, more medial part of the lentiform nucleus.'),
  'D005918': NtO(MH='Glomus Tumor', MS='A blue-red, extremely painful vascular neoplasm involving a glomeriform arteriovenous anastomosis (glomus body), which may be found anywhere in the skin, most often in the distal portion of the fingers and toes, especially beneath the nail. It is composed of specialized pericytes (sometimes termed glomus cells), usually in single encapsulated nodular masses which may be several millimeters in diameter (From Stedman, 27th ed). CHEMODECTOMA, a tumor of NEURAL CREST origin, is also sometimes called a glomus tumor.'),
  'D005919': NtO(MH='Glomerular Filtration Rate', MS="The volume of water filtered out of plasma through glomerular capillary walls into Bowman's capsules per unit of time. It is considered to be equivalent to INULIN clearance."),
  'D005920': NtO(MH='Glomerular Mesangium', MS='The thin membranous structure supporting the adjoining glomerular capillaries. It is composed of GLOMERULAR MESANGIAL CELLS and their EXTRACELLULAR MATRIX.'),
  'D005921': NtO(MH='Glomerulonephritis', MS='Inflammation of the renal glomeruli (KIDNEY GLOMERULUS) that can be classified by the type of glomerular injuries including antibody deposition, complement activation, cellular proliferation, and glomerulosclerosis. These structural and functional abnormalities usually lead to HEMATURIA; PROTEINURIA; HYPERTENSION; and RENAL INSUFFICIENCY.'),
  'D005922': NtO(MH='Glomerulonephritis, IGA', MS='A chronic form of glomerulonephritis characterized by deposits of predominantly IMMUNOGLOBULIN A in the mesangial area (GLOMERULAR MESANGIUM). Deposits of COMPLEMENT C3 and IMMUNOGLOBULIN G are also often found. Clinical features may progress from asymptomatic HEMATURIA to END-STAGE KIDNEY DISEASE.'),
  'D005923': NtO(MH='Glomerulosclerosis, Focal Segmental', MS='A clinicopathological syndrome or diagnostic term for a type of glomerular injury that has multiple causes, primary or secondary. Clinical features include PROTEINURIA, reduced GLOMERULAR FILTRATION RATE, and EDEMA. Kidney biopsy initially indicates focal segmental glomerular consolidation (hyalinosis) or scarring which can progress to globally sclerotic glomeruli leading to eventual KIDNEY FAILURE.'),
  'D005924': NtO(MH='Glomus Jugulare', MS='A nonchromaffin paraganglion located in the wall of the jugular bulb. The most common tumors of the middle ear arise from this tissue. (Lockard, Desk Reference for Neuroscience, 1992, p114)'),
  'D005925': NtO(MH='Glomus Jugulare Tumor', MS='A paraganglioma involving the glomus jugulare, a microscopic collection of chemoreceptor tissue in the adventitia of the bulb of the jugular vein. It may cause paralysis of the vocal cords, attacks of dizziness, blackouts, and nystagmus. It is not resectable but radiation therapy is effective. It regresses slowly, but permanent control is regularly achieved. (From Dorland, 27th ed; Stedman, 25th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, pp1603-4)'),
  'D005926': NtO(MH='Glossalgia', MS='Painful sensations in the tongue, including a sensation of burning.'),
  'D005927': NtO(MH='Glossectomy', MS='Partial or total surgical excision of the tongue. (Dorland, 28th ed)'),
  'D005928': NtO(MH='Glossitis', MS='Inflammation of the tongue.'),
  'D005929': NtO(MH='Glossitis, Benign Migratory', MS='An idiopathic disorder of the tongue characterized by the loss of filiform papillae leaving reddened areas of circinate macules bound by a white band. The lesions heal, then others erupt.'),
  'D005930': NtO(MH='Glossopharyngeal Nerve', MS='The 9th cranial nerve. The glossopharyngeal nerve is a mixed motor and sensory nerve; it conveys somatic and autonomic efferents as well as general, special, and visceral afferents. Among the connections are motor fibers to the stylopharyngeus muscle, parasympathetic fibers to the parotid glands, general and taste afferents from the posterior third of the tongue, the nasopharynx, and the palate, and afferents from baroreceptors and CHEMORECEPTOR CELLS of the carotid sinus.'),
  'D005931': NtO(MH='Glottis', MS='The vocal apparatus of the larynx, situated in the middle section of the larynx. Glottis consists of the VOCAL FOLDS and an opening (rima glottidis) between the folds.'),
  'D005932': NtO(MH='Gloves, Surgical', MS='Gloves, usually rubber, worn by surgeons, examining physicians, dentists, and other health personnel for the mutual protection of personnel and patient.'),
  'D005934': NtO(MH='Glucagon', MS="A 29-amino acid pancreatic peptide derived from proglucagon which is also the precursor of intestinal GLUCAGON-LIKE PEPTIDES. Glucagon is secreted by PANCREATIC ALPHA CELLS and plays an important role in regulation of BLOOD GLUCOSE concentration, ketone metabolism, and several other biochemical and physiological processes. (From Gilman et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 9th ed, p1511)"),
  'D005935': NtO(MH='Glucagonoma', MS='An almost always malignant GLUCAGON-secreting tumor derived from the PANCREATIC ALPHA CELLS. It is characterized by a distinctive migratory ERYTHEMA; WEIGHT LOSS; STOMATITIS; GLOSSITIS; DIABETES MELLITUS; hypoaminoacidemia; and normochromic normocytic ANEMIA.'),
  'D005936': NtO(MH='Glucans', MS='Polysaccharides composed of repeating glucose units. They can consist of branched or unbranched chains in any linkages.'),
  'D005937': NtO(MH='Glucaric Acid', MS='A sugar acid derived from D-glucose in which both the aldehydic carbon atom and the carbon atom bearing the primary hydroxyl group are oxidized to carboxylic acid groups.'),
  'D005938': NtO(MH='Glucocorticoids', MS='A group of CORTICOSTEROIDS that affect carbohydrate metabolism (GLUCONEOGENESIS, liver glycogen deposition, elevation of BLOOD SUGAR), inhibit ADRENOCORTICOTROPIC HORMONE secretion, and possess pronounced anti-inflammatory activity. They also play a role in fat and protein metabolism, maintenance of arterial blood pressure, alteration of the connective tissue response to injury, reduction in the number of circulating lymphocytes, and functioning of the central nervous system.'),
  'D005941': NtO(MH='Glucokinase', MS='A group of enzymes that catalyzes the conversion of ATP and D-glucose to ADP and D-glucose 6-phosphate. They are found in invertebrates and microorganisms, and are highly specific for glucose. (Enzyme Nomenclature, 1992) EC 2.7.1.2.'),
  'D005942': NtO(MH='Gluconates', MS='Derivatives of gluconic acid (the structural formula HOCH2(CHOH)4COOH), including its salts and esters.'),
  'D005943': NtO(MH='Gluconeogenesis', MS='Biosynthesis of GLUCOSE from nonhexose or non-carbohydrate precursors, such as LACTATE; PYRUVATE; ALANINE; and GLYCEROL.'),
  'D005944': NtO(MH='Glucosamine', MS=None),
  'D005945': NtO(MH='Glutamine-Fructose-6-Phosphate Transaminase (Isomerizing)', MS='An enzyme that catalyzes the synthesis of fructose-6-phosphate plus GLUTAMINE from GLUTAMATE plus glucosamine-6-phosphate.'),
  'D005947': NtO(MH='Glucose', MS='A primary source of energy for living organisms. It is naturally occurring and is found in fruits and other parts of plants in its free state. It is used therapeutically in fluid and nutrient replacement.'),
  'D005948': NtO(MH='Glucose Dehydrogenases', MS='D-Glucose:1-oxidoreductases. Catalyzes the oxidation of D-glucose to D-glucono-gamma-lactone and reduced acceptor. Any acceptor except molecular oxygen is permitted. Includes EC 1.1.1.47; EC 1.1.1.118; EC 1.1.1.119 and EC 1.1.99.10.'),
  'D005949': NtO(MH='Glucose Oxidase', MS='An enzyme of the oxidoreductase class that catalyzes the conversion of beta-D-glucose and oxygen to D-glucono-1,5-lactone and peroxide. It is a flavoprotein, highly specific for beta-D-glucose. The enzyme is produced by Penicillium notatum and other fungi and has antibacterial activity in the presence of glucose and oxygen. It is used to estimate glucose concentration in blood or urine samples through the formation of colored dyes by the hydrogen peroxide produced in the reaction. (From Enzyme Nomenclature, 1992) EC 1.1.3.4.'),
  'D005950': NtO(MH='Glucose Solution, Hypertonic', MS='Solution that is usually 10 percent glucose but may be higher. An isotonic solution of glucose is 5 percent.'),
  'D005951': NtO(MH='Glucose Tolerance Test', MS='A test to determine the ability of an individual to maintain HOMEOSTASIS of BLOOD GLUCOSE. It includes measuring blood glucose levels in a fasting state, and at prescribed intervals before and after oral glucose intake (75 or 100 g) or intravenous infusion (0.5 g/kg).'),
  'D005952': NtO(MH='Glucose-6-Phosphatase', MS='An enzyme that catalyzes the conversion of D-glucose 6-phosphate and water to D-glucose and orthophosphate. EC 3.1.3.9.'),
  'D005953': NtO(MH='Glycogen Storage Disease Type I', MS='An autosomal recessive disease in which gene expression of glucose-6-phosphatase is absent, resulting in hypoglycemia due to lack of glucose production. Accumulation of glycogen in liver and kidney leads to organomegaly, particularly massive hepatomegaly. Increased concentrations of lactic acid and hyperlipidemia appear in the plasma. Clinical gout often appears in early childhood.'),
  'D005954': NtO(MH='Glucosephosphate Dehydrogenase', MS=None),
  'D005955': NtO(MH='Glucosephosphate Dehydrogenase Deficiency', MS='A disease-producing enzyme deficiency subject to many variants, some of which cause a deficiency of GLUCOSE-6-PHOSPHATE DEHYDROGENASE activity in erythrocytes, leading to hemolytic anemia.'),
  'D005956': NtO(MH='Glucose-6-Phosphate Isomerase', MS='An aldose-ketose isomerase that catalyzes the reversible interconversion of glucose 6-phosphate and fructose 6-phosphate. In prokaryotic and eukaryotic organisms it plays an essential role in glycolytic and gluconeogenic pathways. In mammalian systems the enzyme is found in the cytoplasm and as a secreted protein. This secreted form of glucose-6-phosphate isomerase has been referred to as autocrine motility factor or neuroleukin, and acts as a cytokine which binds to the AUTOCRINE MOTILITY FACTOR RECEPTOR. Deficiency of the enzyme in humans is an autosomal recessive trait, which results in CONGENITAL NONSPHEROCYTIC HEMOLYTIC ANEMIA.'),
  'D005957': NtO(MH='UTP-Glucose-1-Phosphate Uridylyltransferase', MS='An enzyme that catalyzes the formation of UDPglucose from UTP plus glucose 1-phosphate. EC 2.7.7.9.'),
  'D005958': NtO(MH='Glucosephosphates', MS=None),
  'D005959': NtO(MH='Glucosidases', MS='Enzymes that hydrolyze O-glucosyl-compounds. (Enzyme Nomenclature, 1992) EC 3.2.1.-.'),
  'D005960': NtO(MH='Glucosides', MS=None),
  'D005961': NtO(MH='Glucosinolates', MS='Substituted thioglucosides. They are found in rapeseed (Brassica campestris) products and related cruciferae. They are metabolized to a variety of toxic products which are most likely the cause of hepatocytic necrosis in animals and humans.'),
  'D005962': NtO(MH='Glucosylceramidase', MS='A glycosidase that hydrolyzes a glucosylceramide to yield free ceramide plus glucose. Deficiency of this enzyme leads to abnormally high concentrations of glucosylceramide in the brain in GAUCHER DISEASE. EC 3.2.1.45.'),
  'D005963': NtO(MH='Glucosylceramides', MS="Cerebrosides which contain as their polar head group a glucose moiety bound in glycosidic linkage to the hydroxyl group of ceramides. Their accumulation in tissue, due to a defect in beta-glucosidase, is the cause of Gaucher's disease."),
  'D005964': NtO(MH='Glucosyltransferases', MS='Enzymes that catalyze the transfer of glucose from a nucleoside diphosphate glucose to an acceptor molecule which is frequently another carbohydrate. EC 2.4.1.-.'),
  'D005965': NtO(MH='Glucuronates', MS='Derivatives of GLUCURONIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that include the 6-carboxy glucose structure.'),
  'D005966': NtO(MH='Glucuronidase', MS=None),
  'D005967': NtO(MH='Glue Proteins, Drosophila', MS='Glycosylated proteins which are part of the salivary glue that Drosophila larvae secrete as a means of fixing themselves to an external substrate for the duration of the pre-pupal and pupal period.'),
  'D005968': NtO(MH='Glutamate Decarboxylase', MS='A pyridoxal-phosphate protein that catalyzes the alpha-decarboxylation of L-glutamic acid to form gamma-aminobutyric acid and carbon dioxide. The enzyme is found in bacteria and in invertebrate and vertebrate nervous systems. It is the rate-limiting enzyme in determining GAMMA-AMINOBUTYRIC ACID levels in normal nervous tissues. The brain enzyme also acts on L-cysteate, L-cysteine sulfinate, and L-aspartate. EC 4.1.1.15.'),
  'D005969': NtO(MH='Glutamate Dehydrogenase', MS='An enzyme that catalyzes the conversion of L-glutamate and water to 2-oxoglutarate and NH3 in the presence of NAD+. (From Enzyme Nomenclature, 1992) EC 1.4.1.2.'),
  'D005970': NtO(MH='Glutamate Synthase', MS='An enzyme that catalyzes the formation of 2 molecules of glutamate from glutamine plus alpha-ketoglutarate in the presence of NADPH. EC 1.4.1.13.'),
  'D005971': NtO(MH='Glutamates', MS='Derivatives of GLUTAMIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the 2-aminopentanedioic acid structure.'),
  'D005972': NtO(MH='Glutaminase', MS=None),
  'D005973': NtO(MH='Glutamine', MS='A non-essential amino acid present abundantly throughout the body and is involved in many metabolic processes. It is synthesized from GLUTAMIC ACID and AMMONIA. It is the principal carrier of NITROGEN in the body and is an important energy source for many cells.'),
  'D005974': NtO(MH='Glutamate-Ammonia Ligase', MS='An enzyme that catalyzes the conversion of ATP, L-glutamate, and NH3 to ADP, orthophosphate, and L-glutamine. It also acts more slowly on 4-methylene-L-glutamate. (From Enzyme Nomenclature, 1992) EC 6.3.1.2.'),
  'D005975': NtO(MH='Glutamate-tRNA Ligase', MS='An enzyme that activates glutamic acid with its specific transfer RNA. EC 6.1.1.17.'),
  'D005976': NtO(MH='Glutaral', MS='One of the protein CROSS-LINKING REAGENTS that is used as a disinfectant for sterilization of heat-sensitive equipment and as a laboratory reagent, especially as a fixative.'),
  'D005977': NtO(MH='Glutarates', MS='Derivatives of glutaric acid (the structural formula (COO-)2C3H6), including its salts and esters.'),
  'D005978': NtO(MH='Glutathione', MS='A tripeptide with many roles in cells. It conjugates to drugs to make them more soluble for excretion, is a cofactor for some enzymes, is involved in protein disulfide bond rearrangement and reduces peroxides.'),
  'D005979': NtO(MH='Glutathione Peroxidase', MS='An enzyme catalyzing the oxidation of 2 moles of GLUTATHIONE in the presence of HYDROGEN PEROXIDE to yield oxidized glutathione and water.'),
  'D005980': NtO(MH='Glutathione Reductase', MS='Catalyzes the oxidation of GLUTATHIONE to GLUTATHIONE DISULFIDE in the presence of NADP+. Deficiency in the enzyme is associated with HEMOLYTIC ANEMIA. Formerly listed as EC 1.6.4.2.'),
  'D005981': NtO(MH='Glutathione Synthase', MS='One of the enzymes active in the gamma-glutamyl cycle. It catalyzes the synthesis of glutathione from gamma-glutamylcysteine and glycine in the presence of ATP with the formation of ADP and orthophosphate. EC 6.3.2.3.'),
  'D005982': NtO(MH='Glutathione Transferase', MS='A transferase that catalyzes the addition of aliphatic, aromatic, or heterocyclic FREE RADICALS as well as EPOXIDES and arene oxides to GLUTATHIONE. Addition takes place at the SULFUR. It also catalyzes the reduction of polyol nitrate by glutathione to polyol and nitrite.'),
  'D005983': NtO(MH='Glutens', MS='Prolamins in the endosperm of SEEDS from the Triticeae tribe which includes species of WHEAT; BARLEY; and RYE.'),
  'D005984': NtO(MH='Glutethimide', MS='A hypnotic and sedative. Its use has been largely superseded by other drugs.'),
  'D005985': NtO(MH='Glyceraldehyde', MS='An aldotriose containing the propionaldehyde structure with hydroxy groups at the 2- and 3-positions. It is involved in the formation of ADVANCED GLYCOSYLATION END PRODUCTS.'),
  'D005986': NtO(MH='Glyceraldehyde 3-Phosphate', MS='An aldotriose which is an important intermediate in glycolysis and in tryptophan biosynthesis.'),
  'D005987': NtO(MH='Glyceraldehyde-3-Phosphate Dehydrogenases', MS='Enzymes that catalyze the dehydrogenation of GLYCERALDEHYDE 3-PHOSPHATE. Several types of glyceraldehyde-3-phosphate-dehydrogenase exist including phosphorylating and non-phosphorylating varieties and ones that transfer hydrogen to NADP and ones that transfer hydrogen to NAD.'),
  'D005988': NtO(MH='Glyceric Acids', MS='Three-carbon SUGAR ACIDS with the formula HOCH2(CHOH)COOH.'),
  'D005989': NtO(MH='Glycerides', MS='GLYCEROL esterified with FATTY ACIDS.'),
  'D005990': NtO(MH='Glycerol', MS='A trihydroxy sugar alcohol that is an intermediate in carbohydrate and lipid metabolism. It is used as a solvent, emollient, pharmaceutical agent, or sweetening agent.'),
  'D005991': NtO(MH='Glycerol Kinase', MS='An enzyme that catalyzes the formation of glycerol 3-phosphate from ATP and glycerol. Dihydroxyacetone and L-glyceraldehyde can also act as acceptors; UTP and, in the case of the yeast enzyme, ITP and GTP can act as donors. It provides a way for glycerol derived from fats or glycerides to enter the glycolytic pathway. EC 2.7.1.30.'),
  'D005992': NtO(MH='Glycerol-3-Phosphate O-Acyltransferase', MS='An enzyme that transfers acyl groups from acyl-CoA to glycerol-3-phosphate to form monoglyceride phosphates. It acts only with CoA derivatives of fatty acids of chain length above C-10. Also forms diglyceride phosphates. EC 2.3.1.15.'),
  'D005993': NtO(MH='Glycerolphosphate Dehydrogenase', MS=None),
  'D005994': NtO(MH='Glycerophosphates', MS='Any salt or ester of glycerophosphoric acid.'),
  'D005995': NtO(MH='Glyceryl Ethers', MS='Compounds in which one or more of the three hydroxyl groups of glycerol are in ethereal linkage with a saturated or unsaturated aliphatic alcohol; one or two of the hydroxyl groups of glycerol may be esterified. These compounds have been found in various animal tissue.'),
  'D005996': NtO(MH='Nitroglycerin', MS='A volatile vasodilator which relieves ANGINA PECTORIS by stimulating GUANYLATE CYCLASE and lowering cytosolic calcium. It is also sometimes used for TOCOLYSIS and explosives.'),
  'D005997': NtO(MH='Glycerylphosphorylcholine', MS='A component of PHOSPHATIDYLCHOLINES or LECITHINS, in which the two hydroxy groups of GLYCEROL are esterified with fatty acids. (From Stedman, 26th ed)'),
  'D005998': NtO(MH='Glycine', MS='A non-essential amino acid. It is found primarily in gelatin and silk fibroin and used therapeutically as a nutrient. It is also a fast inhibitory neurotransmitter.'),
  'D005999': NtO(MH='Glycochenodeoxycholic Acid', MS='A bile salt formed in the liver from chenodeoxycholate and glycine, usually as the sodium salt. It acts as a detergent to solubilize fats for absorption and is itself absorbed. It is a cholagogue and choleretic.'),
  'D006000': NtO(MH='Glycocholic Acid', MS='The glycine conjugate of CHOLIC ACID. It acts as a detergent to solubilize fats for absorption and is itself absorbed.'),
  'D006001': NtO(MH='Glycoconjugates', MS='Carbohydrates covalently linked to a nonsugar moiety (lipids or proteins). The major glycoconjugates are glycoproteins, glycopeptides, peptidoglycans, glycolipids, and lipopolysaccharides. (From Biochemical Nomenclature and Related Documents, 2d ed; From Principles of Biochemistry, 2d ed)'),
  'D006002': NtO(MH='Glycodeoxycholic Acid', MS='A bile salt formed in the liver by conjugation of deoxycholate with glycine, usually as the sodium salt. It acts as a detergent to solubilize fats for absorption and is itself absorbed. It is used as a cholagogue and choleretic.'),
  'D006003': NtO(MH='Glycogen', MS=None),
  'D006004': NtO(MH='Glycogen Debranching Enzyme System', MS='1,4-alpha-D-Glucan-1,4-alpha-D-glucan 4-alpha-D-glucosyltransferase/dextrin 6 alpha-D-glucanohydrolase. An enzyme system having both 4-alpha-glucanotransferase (EC 2.4.1.25) and amylo-1,6-glucosidase (EC 3.2.1.33) activities. As a transferase it transfers a segment of a 1,4-alpha-D-glucan to a new 4-position in an acceptor, which may be glucose or another 1,4-alpha-D-glucan. As a glucosidase it catalyzes the endohydrolysis of 1,6-alpha-D-glucoside linkages at points of branching in chains of 1,4-linked alpha-D-glucose residues. Amylo-1,6-glucosidase activity is deficient in glycogen storage disease type III.'),
  'D006005': NtO(MH='Phosphorylases', MS='A class of glucosyltransferases that catalyzes the degradation of storage polysaccharides, such as glucose polymers, by phosphorolysis in animals (GLYCOGEN PHOSPHORYLASE) and in plants (STARCH PHOSPHORYLASE).'),
  'D006006': NtO(MH='Glycogen Synthase', MS='An enzyme that catalyzes the transfer of D-glucose from UDPglucose into 1,4-alpha-D-glucosyl chains. EC 2.4.1.11.'),
  'D006007': NtO(MH='Glycogen-Synthase-D Phosphatase', MS='An enzyme that catalyzes the conversion of phosphorylated, inactive glycogen synthase D to active dephosphoglycogen synthase I. EC 3.1.3.42.'),
  'D006008': NtO(MH='Glycogen Storage Disease', MS='A group of inherited metabolic disorders involving the enzymes responsible for the synthesis and degradation of glycogen. In some patients, prominent liver involvement is presented. In others, more generalized storage of glycogen occurs, sometimes with prominent cardiac involvement.'),
  'D006009': NtO(MH='Glycogen Storage Disease Type II', MS='An autosomal recessively inherited glycogen storage disease caused by GLUCAN 1,4-ALPHA-GLUCOSIDASE deficiency. Large amounts of GLYCOGEN accumulate in the LYSOSOMES of skeletal muscle (MUSCLE, SKELETAL); HEART; LIVER; SPINAL CORD; and BRAIN. Three forms have been described: infantile, childhood, and adult. The infantile form is fatal in infancy and presents with hypotonia and a hypertrophic cardiomyopathy (CARDIOMYOPATHY, HYPERTROPHIC). The childhood form usually presents in the second year of life with proximal weakness and respiratory symptoms. The adult form consists of a slowly progressive proximal myopathy. (From Muscle Nerve 1995;3:S61-9; Menkes, Textbook of Child Neurology, 5th ed, pp73-4)'),
  'D006010': NtO(MH='Glycogen Storage Disease Type III', MS='An autosomal recessive metabolic disorder due to deficient expression of amylo-1,6-glucosidase (one part of the glycogen debranching enzyme system). The clinical course of the disease is similar to that of glycogen storage disease type I, but milder. Massive hepatomegaly, which is present in young children, diminishes and occasionally disappears with age. Levels of glycogen with short outer branches are elevated in muscle, liver, and erythrocytes. Six subgroups have been identified, with subgroups Type IIIa and Type IIIb being the most prevalent.'),
  'D006011': NtO(MH='Glycogen Storage Disease Type IV', MS='An autosomal recessive metabolic disorder due to a deficiency in expression of glycogen branching enzyme 1 (alpha-1,4-glucan-6-alpha-glucosyltransferase), resulting in an accumulation of abnormal GLYCOGEN with long outer branches. Clinical features are MUSCLE HYPOTONIA and CIRRHOSIS. Death from liver disease usually occurs before age 2.'),
  'D006012': NtO(MH='Glycogen Storage Disease Type V', MS='Glycogenosis due to muscle phosphorylase deficiency. Characterized by painful cramps following sustained exercise.'),
  'D006013': NtO(MH='Glycogen Storage Disease Type VI', MS='A hepatic GLYCOGEN STORAGE DISEASE in which there is an apparent deficiency of hepatic phosphorylase (GLYCOGEN PHOSPHORYLASE, LIVER FORM) activity.'),
  'D006014': NtO(MH='Glycogen Storage Disease Type VII', MS='An autosomal recessive glycogen storage disease in which there is deficient expression of 6-phosphofructose 1-kinase in muscle (PHOSPHOFRUCTOKINASE-1, MUSCLE TYPE) resulting in abnormal deposition of glycogen in muscle tissue. These patients have severe congenital muscular dystrophy and are exercise intolerant.'),
  'D006015': NtO(MH='Glycogen Storage Disease Type VIII', MS='An x-linked recessive hepatic glycogen storage disease resulting from lack of expression of phosphorylase-b-kinase activity. Symptoms are relatively mild; hepatomegaly, increased liver glycogen, and decreased leukocyte phosphorylase are present. Liver shrinkage occurs in response to glucagon.'),
  'D006016': NtO(MH='Glycolates', MS='Derivatives of ACETIC ACID which contain an hydroxy group attached to the methyl carbon.'),
  'D006017': NtO(MH='Glycolipids', MS="Any compound containing one or more monosaccharide residues bound by a glycosidic linkage to a hydrophobic moiety such as an acylglycerol (see GLYCERIDES), a sphingoid, a ceramide (CERAMIDES) (N-acylsphingoid) or a prenyl phosphate. (From IUPAC's webpage)"),
  'D006018': NtO(MH='Glycols', MS='A generic grouping for dihydric alcohols with the hydroxy groups (-OH) located on different carbon atoms. They are viscous liquids with high boiling points for their molecular weights.'),
  'D006019': NtO(MH='Glycolysis', MS='A metabolic process that converts GLUCOSE into two molecules of PYRUVIC ACID through a series of enzymatic reactions. Energy generated by this process is conserved in two molecules of ATP. Glycolysis is the universal catabolic pathway for glucose, free glucose, or glucose derived from complex CARBOHYDRATES, such as GLYCOGEN and STARCH.'),
  'D006020': NtO(MH='Glycopeptides', MS='Proteins which contain carbohydrate groups attached covalently to the polypeptide chain. The protein moiety is the predominant group with the carbohydrate making up only a small percentage of the total weight.'),
  'D006021': NtO(MH='Glycophorins', MS='The major sialoglycoprotein of human erythrocyte membranes. It consists of at least two sialoglycopeptides and is composed of 60% carbohydrate including sialic acid and 40% protein. It is involved in a number of different biological activities including the binding of MN blood groups, influenza viruses, kidney bean phytohemagglutinin, and wheat germ agglutinin.'),
  'D006022': NtO(MH='beta-N-Acetylglucosaminylglycopeptide beta-1,4-Galactosyltransferase', MS='An enzyme that catalyzes the transfer of galactose from UDP-galactose to a specific glycoprotein receptor, 2-acetamido-2-deoxy-D-glucosyl-glycopeptide, during glycopeptide synthesis. EC 2.4.1.38.'),
  'D006023': NtO(MH='Glycoproteins', MS='Conjugated protein-carbohydrate compounds including mucins, mucoid, and amyloid glycoproteins.'),
  'D006024': NtO(MH='Glycopyrrolate', MS='A muscarinic antagonist used as an antispasmodic, in some disorders of the gastrointestinal tract, and to reduce salivation with some anesthetics.'),
  'D006025': NtO(MH='Glycosaminoglycans', MS='Heteropolysaccharides which contain an N-acetylated hexosamine in a characteristic repeating disaccharide unit. The repeating structure of each disaccharide involves alternate 1,4- and 1,3-linkages consisting of either N-acetylglucosamine (see ACETYLGLUCOSAMINE) or N-acetylgalactosamine (see ACETYLGALACTOSAMINE).'),
  'D006026': NtO(MH='Glycoside Hydrolases', MS=None),
  'D006027': NtO(MH='Glycosides', MS="Any compound that contains a constituent sugar, in which the hydroxyl group attached to the first carbon is substituted by an alcoholic, phenolic, or other group. They are named specifically for the sugar contained, such as glucoside (glucose), pentoside (pentose), fructoside (fructose), etc. Upon hydrolysis, a sugar and nonsugar component (aglycone) are formed. (From Dorland, 28th ed; From Miall's Dictionary of Chemistry, 5th ed)"),
  'D006028': NtO(MH='Glycosphingolipids', MS="Lipids containing at least one monosaccharide residue and either a sphingoid or a ceramide (CERAMIDES). They are subdivided into NEUTRAL GLYCOSPHINGOLIPIDS comprising monoglycosyl- and oligoglycosylsphingoids and monoglycosyl- and oligoglycosylceramides; and ACIDIC GLYCOSPHINGOLIPIDS which comprises sialosylglycosylsphingolipids (GANGLIOSIDES); SULFOGLYCOSPHINGOLIPIDS (formerly known as sulfatides), glycuronoglycosphingolipids, and phospho- and phosphonoglycosphingolipids. (From IUPAC's webpage)"),
  'D006029': NtO(MH='Glycosuria', MS='The appearance of an abnormally large amount of GLUCOSE in the urine, such as more than 500 mg/day in adults. It can be due to HYPERGLYCEMIA or genetic defects in renal reabsorption (RENAL GLYCOSURIA).'),
  'D006030': NtO(MH='Glycosuria, Renal', MS='An autosomal inherited disorder due to defective reabsorption of GLUCOSE by the PROXIMAL RENAL TUBULES. The urinary loss of glucose can reach beyond 50 g/day. It is attributed to the mutations in the SODIUM-GLUCOSE TRANSPORTER 2 encoded by the SLC5A2 gene.'),
  'D006031': NtO(MH='Glycosylation', MS='The chemical or biochemical addition of carbohydrate or glycosyl groups to other chemicals, especially peptides or proteins. Glycosyl transferases are used in this biochemical reaction.'),
  'D006032': NtO(MH='Glycine-tRNA Ligase', MS='An enzyme that activates glycine with its specific transfer RNA. EC 6.1.1.14.'),
  'D006033': NtO(MH='Glycylglycine', MS='The simplest of all peptides. It functions as a gamma-glutamyl acceptor.'),
  'D006034': NtO(MH='Glycyrrhetinic Acid', MS='An oleanolic acid from GLYCYRRHIZA that has some antiallergic, antibacterial, and antiviral properties. It is used topically for allergic or infectious skin inflammation and orally for its aldosterone effects in electrolyte regulation.'),
  'D006035': NtO(MH='Glycyrrhiza', MS='A genus of leguminous herbs or shrubs whose roots yield GLYCYRRHETINIC ACID and its derivative, CARBENOXOLONE.'),
  'D006037': NtO(MH='Glyoxal', MS='A 2-carbon aldehyde with carbonyl groups on both carbons.'),
  'D006038': NtO(MH='Glyoxylates', MS='Derivatives of glyoxylic acid (the structural formula C2H2O3), including its salts and esters.'),
  'D006039': NtO(MH='Gnathostoma', MS='A genus of parasitic nematodes that occurs in mammals including man. Infection in humans is either by larvae penetrating the skin or by ingestion of uncooked fish.'),
  'D006040': NtO(MH='Goals', MS='The end-result or objective, which may be specified or required in advance.'),
  'D006041': NtO(MH='Goats', MS='Any of numerous agile, hollow-horned RUMINANTS of the genus Capra, in the family Bovidae, closely related to the SHEEP.'),
  'D006042': NtO(MH='Goiter', MS='Enlargement of the THYROID GLAND that may increase from about 20 grams to hundreds of grams in human adults. Goiter is observed in individuals with normal thyroid function (euthyroidism), thyroid deficiency (HYPOTHYROIDISM), or hormone overproduction (HYPERTHYROIDISM). Goiter may be congenital or acquired, sporadic or endemic (GOITER, ENDEMIC).'),
  'D006043': NtO(MH='Goiter, Endemic', MS='A form of IODINE deficiency disorders characterized by an enlargement of the THYROID GLAND in a significantly large fraction of a POPULATION GROUP. Endemic goiter is common in mountainous and iodine-deficient areas of the world where the DIET contains insufficient amount of iodine.'),
  'D006044': NtO(MH='Goiter, Nodular', MS='An enlarged THYROID GLAND containing multiple nodules (THYROID NODULE), usually resulting from recurrent thyroid HYPERPLASIA and involution over many years to produce the irregular enlargement. Multinodular goiters may be nontoxic or may induce THYROTOXICOSIS.'),
  'D006045': NtO(MH='Goiter, Substernal', MS='An enlarged THYROID GLAND with at least 50% of the gland situated behind the STERNUM. It is an unusual presentation of an intrathoracic goiter. Substernal goiters frequently cause compression on the TRACHEA leading to deviation, narrowing, and respiratory symptoms.'),
  'D006046': NtO(MH='Gold', MS='A yellow metallic element with the atomic symbol Au, atomic number 79, and atomic weight 197. It is used in jewelry, goldplating of other metals, as currency, and in dental restoration. Many of its clinical applications, such as ANTIRHEUMATIC AGENTS, are in the form of its salts.'),
  'D006047': NtO(MH='Gold Alloys', MS='Alloys that contain a high percentage of gold. They are used in restorative or prosthetic dentistry.'),
  'D006048': NtO(MH='Gold Colloid, Radioactive', MS='A suspension of radioactive gold particles emitting negative beta particles and gamma irradiation. It was formerly used for liver scans and irradiation treatment of some metastatic malignancies.'),
  'D006049': NtO(MH='Gold Isotopes', MS='Stable gold atoms that have the same atomic number as the element gold, but differ in atomic weight. Au-197 is a stable isotope.'),
  'D006050': NtO(MH='Gold Radioisotopes', MS='Unstable isotopes of gold that decay or disintegrate emitting radiation. Au 185-196, 198-201, and 203 are radioactive gold isotopes.'),
  'D006051': NtO(MH='Aurothioglucose', MS='A thioglucose derivative used as an antirheumatic and experimentally to produce obesity in animals.'),
  'D006052': NtO(MH='Gold Sodium Thiomalate', MS='A variable mixture of the mono- and disodium salts of gold thiomalic acid used mainly for its anti-inflammatory action in the treatment of rheumatoid arthritis. It is most effective in active progressive rheumatoid arthritis and of little or no value in the presence of extensive deformities or in the treatment of other forms of arthritis.'),
  'D006053': NtO(MH='Goldenhar Syndrome', MS='Mandibulofacial dysostosis with congenital eyelid dermoids.'),
  'D006054': NtO(MH='Goldfish', MS='Common name for Carassius auratus, a type of carp (CARPS).'),
  'D006055': NtO(MH='Golf', MS='A game whose object is to sink a ball into each of 9 or 18 successive holes on a golf course using as few strokes as possible.'),
  'D006056': NtO(MH='Golgi Apparatus', MS='A stack of flattened vesicles that functions in posttranslational processing and sorting of proteins, receiving them from the rough ENDOPLASMIC RETICULUM and directing them to secretory vesicles, LYSOSOMES, or the CELL MEMBRANE. The movement of proteins takes place by transfer vesicles that bud off from the rough endoplasmic reticulum or Golgi apparatus and fuse with the Golgi, lysosomes or cell membrane. (From Glick, Glossary of Biochemistry and Molecular Biology, 1990)'),
  'D006057': NtO(MH='Golgi-Mazzoni Corpuscles', MS='Thin capsules enveloping end nerve fibrils in the subcutaneous tissue of fingers.'),
  'D006058': NtO(MH='Gonadal Disorders', MS='Pathological processes of the OVARIES or the TESTES.'),
  'D006059': NtO(MH='Gonadal Dysgenesis', MS='A number of syndromes with defective gonadal developments such as streak GONADS and dysgenetic testes or ovaries. The spectrum of gonadal and sexual abnormalities is reflected in their varied sex chromosome (SEX CHROMOSOMES) constitution as shown by the karyotypes of 45,X monosomy (TURNER SYNDROME); 46,XX (GONADAL DYSGENESIS, 46XX); 46,XY (GONADAL DYSGENESIS, 46,XY); and sex chromosome MOSAICISM; (GONADAL DYSGENESIS, MIXED). Their phenotypes range from female, through ambiguous, to male. This concept includes gonadal agenesis.'),
  'D006060': NtO(MH='Gonadal Dysgenesis, Mixed', MS='A type of defective gonadal development in patients with a wide spectrum of chromosomal mosaic variants. Their karyotypes are of partial sex chromosome monosomy resulting from an absence or an abnormal second sex chromosome (X or Y). Karyotypes include 45,X/46,XX; 45,X/46,XX/47,XXX; 46,XXp-; 45,X/46,XY; 45,X/47,XYY; 46,XYpi; etc. The spectrum of phenotypes may range from phenotypic female to phenotypic male including variations in gonads and internal and external genitalia, depending on the ratio in each gonad of 45,X primordial germ cells to those with normal 46,XX or 46,XY constitution.'),
  'D006061': NtO(MH='Gonadal Dysgenesis, 46,XY', MS='Defects in the SEX DETERMINATION PROCESS in 46, XY individuals that result in abnormal gonadal development and deficiencies in TESTOSTERONE and subsequently ANTIMULLERIAN HORMONE or other factors required for normal male sex development. This leads to the development of female phenotypes (male to female sex reversal), normal to tall stature, and bilateral streak or dysgenic gonads which are susceptible to GONADAL TISSUE NEOPLASMS. An XY gonadal dysgenesis is associated with structural abnormalities on the Y CHROMOSOME, a mutation in the GENE, SRY, or a mutation in other autosomal genes that are involved in sex determination.'),
  'D006062': NtO(MH='Gonadotropins', MS='Hormones that stimulate gonadal functions such as GAMETOGENESIS and sex steroid hormone production in the OVARY and the TESTIS. Major gonadotropins are glycoproteins produced primarily by the adenohypophysis (GONADOTROPINS, PITUITARY) and the placenta (CHORIONIC GONADOTROPIN). In some species, pituitary PROLACTIN and PLACENTAL LACTOGEN exert some luteotropic activities.'),
  'D006063': NtO(MH='Chorionic Gonadotropin', MS='A gonadotropic glycoprotein hormone produced primarily by the PLACENTA. Similar to the pituitary LUTEINIZING HORMONE in structure and function, chorionic gonadotropin is involved in maintaining the CORPUS LUTEUM during pregnancy. CG consists of two noncovalently linked subunits, alpha and beta. Within a species, the alpha subunit is virtually identical to the alpha subunits of the three pituitary glycoprotein hormones (TSH, LH, and FSH), but the beta subunit is unique and confers its biological specificity (CHORIONIC GONADOTROPIN, BETA SUBUNIT, HUMAN).'),
  'D006064': NtO(MH='Gonadotropins, Equine', MS='Gonadotropins secreted by the pituitary or the placenta in horses. This term generally refers to the gonadotropins found in the pregnant mare serum, a rich source of equine CHORIONIC GONADOTROPIN; LUTEINIZING HORMONE; and FOLLICLE STIMULATING HORMONE. Unlike that in humans, the equine LUTEINIZING HORMONE, BETA SUBUNIT is identical to the equine choronic gonadotropin, beta. Equine gonadotropins prepared from pregnant mare serum are used in reproductive studies.'),
  'D006065': NtO(MH='Gonadotropins, Pituitary', MS='Hormones secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR) that stimulate gonadal functions in both males and females. They include FOLLICLE STIMULATING HORMONE that stimulates germ cell maturation (OOGENESIS; SPERMATOGENESIS), and LUTEINIZING HORMONE that stimulates the production of sex steroids (ESTROGENS; PROGESTERONE; ANDROGENS).'),
  'D006066': NtO(MH='Gonads', MS='The gamete-producing glands, OVARY or TESTIS.'),
  'D006067': NtO(MH='Gonanes', MS='Steroids containing the fundamental tetracyclic unit with no methyl groups at C-10 and C-13 and with no side chain at C-17. The concept includes both saturated and unsaturated derivatives.'),
  'D006068': NtO(MH='Gonioscopy', MS='Examination of the angle of the anterior chamber of the eye with a specialized optical instrument (gonioscope) or a contact prism lens.'),
  'D006069': NtO(MH='Gonorrhea', MS='Acute infectious disease characterized by primary invasion of the urogenital tract. The etiologic agent, NEISSERIA GONORRHOEAE, was isolated by Neisser in 1879.'),
  'D006071': NtO(MH='Gorilla gorilla', MS='This single species of Gorilla, which is a member of the HOMINIDAE family, is the largest and most powerful of the PRIMATES. It is distributed in isolated scattered populations throughout forests of equatorial Africa.'),
  'D006072': NtO(MH='Gossypol', MS='A dimeric sesquiterpene found in cottonseed (GOSSYPIUM). The (-) isomer is active as a male contraceptive (CONTRACEPTIVE AGENTS, MALE) whereas toxic symptoms are associated with the (+) isomer.'),
  'D006073': NtO(MH='Gout', MS='Metabolic disorder characterized by recurrent acute arthritis, hyperuricemia and deposition of sodium urate in and around the joints, sometimes with formation of URIC ACID calculi.'),
  'D006074': NtO(MH='Gout Suppressants', MS='Agents that increase uric acid excretion by the kidney (URICOSURIC AGENTS), decrease uric acid production (antihyperuricemics), or alleviate the pain and inflammation of acute attacks of gout.'),
  'D006075': NtO(MH='Governing Board', MS='The group in which legal authority is vested for the control of health-related institutions and organizations.'),
  'D006076': NtO(MH='Government', MS='The complex of political institutions, laws, and customs through which the function of governing is carried out in a specific political unit.'),
  'D006077': NtO(MH='Government Agencies', MS='Administrative units of government responsible for policy making and management of governmental activities.'),
  'D006078': NtO(MH='Government Programs', MS='Programs and activities sponsored or administered by local, state, or national governments.'),
  'D006079': NtO(MH='Government Publications as Topic', MS='Works about publications issued by local, regional, or national governments or by their agencies or subdivisions.'),
  'D006080': NtO(MH='Ovarian Follicle', MS='An OOCYTE-containing structure in the cortex of the OVARY. The oocyte is enclosed by a layer of GRANULOSA CELLS providing a nourishing microenvironment (FOLLICULAR FLUID). The number and size of follicles vary depending on the age and reproductive state of the female. The growing follicles are divided into five stages: primary, secondary, tertiary, Graafian, and atretic. Follicular growth and steroidogenesis depend on the presence of GONADOTROPINS.'),
  'D006082': NtO(MH='Graft Enhancement, Immunologic', MS="The induction of prolonged survival and growth of allografts of either tumors or normal tissues which would ordinarily be rejected. It may be induced passively by introducing graft-specific antibodies from previously immunized donors, which bind to the graft's surface antigens, masking them from recognition by T-cells; or actively by prior immunization of the recipient with graft antigens which evoke specific antibodies and form antigen-antibody complexes which bind to the antigen receptor sites of the T-cells and block their cytotoxic activity."),
  'D006083': NtO(MH='Graft Occlusion, Vascular', MS='Obstruction of flow in biological or prosthetic vascular grafts.'),
  'D006084': NtO(MH='Graft Rejection', MS='An immune response with both cellular and humoral components, directed against an allogeneic transplant, whose tissue antigens are not compatible with those of the recipient.'),
  'D006085': NtO(MH='Graft Survival', MS='The survival of a graft in a host, the factors responsible for the survival and the changes occurring within the graft during growth in the host.'),
  'D006086': NtO(MH='Graft vs Host Disease', MS='The clinical entity characterized by anorexia, diarrhea, loss of hair, leukopenia, thrombocytopenia, growth retardation, and eventual death brought about by the GRAFT VS HOST REACTION.'),
  'D006087': NtO(MH='Graft vs Host Reaction', MS='An immunological attack mounted by a graft against the host because of tissue incompatibility when immunologically competent cells are transplanted to an immunologically incompetent host; the resulting clinical picture is that of GRAFT VS HOST DISEASE.'),
  'D006088': NtO(MH='Gram-Negative Aerobic Bacteria', MS='A large group of aerobic bacteria which show up as pink (negative) when treated by the gram-staining method. This is because the cell walls of gram-negative bacteria are low in peptidoglycan and thus have low affinity for violet stain and high affinity for the pink dye safranine.'),
  'D006089': NtO(MH='Gram-Negative Anaerobic Bacteria', MS='A large group of anaerobic bacteria which show up as pink (negative) when treated by the Gram-staining method.'),
  'D006090': NtO(MH='Gram-Negative Bacteria', MS="Bacteria which lose crystal violet stain but are stained pink when treated by Gram's method."),
  'D006091': NtO(MH='Gram-Negative Chemolithotrophic Bacteria', MS='A large group of bacteria including those which oxidize ammonia or nitrite, metabolize sulfur and sulfur compounds, or deposit iron and/or manganese oxides.'),
  'D006092': NtO(MH='Gram-Negative Facultatively Anaerobic Rods', MS='A large group of facultatively anaerobic, rod-shaped bacteria that show up as pink (negative) when treated by the gram-staining method.'),
  'D006093': NtO(MH='Gram-Positive Asporogenous Rods', MS='A gram-positive, non-spore-forming group of bacteria comprising organisms that have morphological and physiological characteristics in common.'),
  'D006094': NtO(MH='Gram-Positive Bacteria', MS="Bacteria which retain the crystal violet stain when treated by Gram's method."),
  'D006095': NtO(MH='Gram-Positive Cocci', MS="Coccus-shaped bacteria that retain the crystal violet stain when treated by Gram's method."),
  'D006096': NtO(MH='Gramicidin', MS='A group of peptide antibiotics from BACILLUS brevis. Gramicidin C or S is a cyclic, ten-amino acid polypeptide and gramicidins A, B, D are linear. Gramicidin is one of the two principal components of TYROTHRICIN.'),
  'D006097': NtO(MH='Granulation Tissue', MS='A vascular connective tissue formed on the surface of a healing wound, ulcer, or inflamed tissue. It consists of new capillaries and an infiltrate containing lymphoid cells, macrophages, and plasma cells.'),
  'D006098': NtO(MH='Granulocytes', MS='Leukocytes with abundant granules in the cytoplasm. They are divided into three groups according to the staining properties of the granules: neutrophilic, eosinophilic, and basophilic. Mature granulocytes are the NEUTROPHILS; EOSINOPHILS; and BASOPHILS.'),
  'D006099': NtO(MH='Granuloma', MS='A relatively small nodular inflammatory lesion containing grouped mononuclear phagocytes, caused by infectious and noninfectious agents.'),
  'D006100': NtO(MH='Granuloma Inguinale', MS='Anogenital ulcers caused by Calymmatobacterium granulomatis as distinguished from lymphogranuloma inguinale (see LYMPHOGRANULOMA VENEREUM) caused by CHLAMYDIA TRACHOMATIS. Diagnosis is made by demonstration of typical intracellular Donovan bodies in crushed-tissue smears.'),
  'D006101': NtO(MH='Granuloma, Giant Cell', MS='A non-neoplastic inflammatory lesion, usually of the jaw or gingiva, containing large, multinucleated cells. It includes reparative giant cell granuloma. Peripheral giant cell granuloma refers to the gingiva (giant cell epulis); central refers to the jaw.'),
  'D006102': NtO(MH='Granuloma, Laryngeal', MS='A tumor-like nodule or mass of inflammatory granulation tissue projecting into the lumen of the LARYNX.'),
  'D006103': NtO(MH='Granuloma, Lethal Midline', MS='A condition that is characterized by inflammation, ulceration, and perforation of the nose and the PALATE with progressive destruction of midline facial structures. This syndrome can be manifested in several diseases including the nasal type of EXTRANODAL NK-T-CELL LYMPHOMA and GRANULOMATOSIS WITH POLYANGIITIS.'),
  'D006104': NtO(MH='Granuloma, Plasma Cell', MS='A slow-growing benign pseudotumor in which plasma cells greatly outnumber the inflammatory cells.'),
  'D006105': NtO(MH='Granulomatous Disease, Chronic', MS='A defect of leukocyte function in which phagocytic cells ingest but fail to digest bacteria, resulting in recurring bacterial infections with granuloma formation. When chronic granulomatous disease is caused by mutations in the CYBB gene, the condition is inherited in an X-linked recessive pattern. When chronic granulomatous disease is caused by CYBA, NCF1, NCF2, or NCF4 gene mutations, the condition is inherited in an autosomal recessive pattern.'),
  'D006106': NtO(MH='Granulosa Cell Tumor', MS='A neoplasm composed entirely of GRANULOSA CELLS, occurring mostly in the OVARY. In the adult form, it may contain some THECA CELLS. This tumor often produces ESTRADIOL and INHIBIN. The excess estrogen exposure can lead to other malignancies in women and PRECOCIOUS PUBERTY in girls. In rare cases, granulosa cell tumors have been identified in the TESTES.'),
  'D006107': NtO(MH='Granulosa Cells', MS='Supporting cells for the developing female gamete in the OVARY. They are derived from the coelomic epithelial cells of the gonadal ridge. Granulosa cells form a single layer around the OOCYTE in the primordial ovarian follicle and advance to form a multilayered cumulus oophorus surrounding the OVUM in the Graafian follicle. The major functions of granulosa cells include the production of steroids and LH receptors (RECEPTORS, LH).'),
  'D006108': NtO(MH='Graphite', MS='An allotropic form of carbon that is used in pencils, as a lubricant, and in matches and explosives. It is obtained by mining and its dust can cause lung irritation.'),
  'D006109': NtO(MH='Poaceae', MS='A large family of narrow-leaved herbaceous grasses of the order Cyperales, subclass Commelinidae, class Liliopsida (monocotyledons). Food grains (EDIBLE GRAIN) come from members of this family. RHINITIS, ALLERGIC, SEASONAL can be induced by POLLEN of many of the grasses.'),
  'D006110': NtO(MH='Grasshoppers', MS='Plant-eating orthopterans having hindlegs adapted for jumping. There are two main families: Acrididae and Romaleidae. Some of the more common genera are: Melanoplus, the most common grasshopper; Conocephalus, the eastern meadow grasshopper; and Pterophylla, the true katydid.'),
  'D006111': NtO(MH='Graves Disease', MS='A common form of hyperthyroidism with a diffuse hyperplastic GOITER. It is an autoimmune disorder that produces antibodies against the THYROID STIMULATING HORMONE RECEPTOR. These autoantibodies activate the TSH receptor, thereby stimulating the THYROID GLAND and hypersecretion of THYROID HORMONES. These autoantibodies can also affect the eyes (GRAVES OPHTHALMOPATHY) and the skin (Graves dermopathy).'),
  'D006112': NtO(MH='Gravitation', MS='Acceleration produced by the mutual attraction of two masses, and of magnitude inversely proportional to the square of the distance between the two centers of mass. It is also the force imparted by the earth, moon, or a planet to an object near its surface. (From NASA Thesaurus, 1988)'),
  'D006113': NtO(MH='United Kingdom', MS='Country in northwestern Europe including Great Britain and the northern one-sixth of the island of Ireland, located between the North Sea and north Atlantic Ocean. The capital is London.'),
  'D006114': NtO(MH='Great Lakes Region', MS='The geographic area of the Great Lakes in general and when the specific state or states are not indicated. It usually includes Illinois, Indiana, Michigan, Minnesota, New York, Ohio, Pennsylvania, and Wisconsin.'),
  'D006115': NtO(MH='Greece', MS='A country in southern Europe, bordering the Aegean Sea, Ionian Sea, and the Mediterranean Sea, between Albania and Turkey. The capital is Athens.'),
  'D006116': NtO(MH='Greenland', MS='An island in Northern North America, between the Arctic Ocean and the North Atlantic Ocean, northeast of Canada.'),
  'D006117': NtO(MH='Grief', MS='Normal, appropriate sorrowful response to an immediate cause. It is self-limiting and gradually subsides within a reasonable time.'),
  'D006118': NtO(MH='Griseofulvin', MS='An antifungal agent used in the treatment of TINEA infections.'),
  'D006119': NtO(MH='Groin', MS='The external junctural region between the lower part of the abdomen and the thigh.'),
  'D006120': NtO(MH='Grooming', MS="An animal's cleaning and caring for the body surface. This includes preening, the cleaning and oiling of feathers with the bill or of hair with the tongue."),
  'D006123': NtO(MH='Group Practice', MS='Any group of three or more full-time physicians organized in a legally recognized entity for the provision of health care services, sharing space, equipment, personnel and records for both patient care and business management, and who have a predetermined arrangement for the distribution of income.'),
  'D006124': NtO(MH='Group Practice, Dental', MS='Any group of three or more full-time dentists, organized in a legally recognized entity for the provision of dental care, sharing space, equipment, personnel and records for both patient care and business management, and who have a predetermined arrangement for the distribution of income.'),
  'D006125': NtO(MH='Group Practice, Prepaid', MS='An organized group of three or more full-time physicians rendering services for a fixed prepayment.'),
  'D006126': NtO(MH='Group Processes', MS='The procedures through which a group approaches, attacks, and solves a common problem.'),
  'D006127': NtO(MH='Group Structure', MS='The informal or formal organization of a group of people based on a network of personal relationships which is influenced by the size and composition, etc., of the group.'),
  'D006128': NtO(MH='Growth', MS='Gradual increase in the number, the size, and the complexity of cells of an individual. Growth generally results in increase in ORGAN WEIGHT; BODY WEIGHT; and BODY HEIGHT.'),
  'D006130': NtO(MH='Growth Disorders', MS='Deviations from the average values for a specific age and sex in any or all of the following: height, weight, skeletal proportions, osseous development, or maturation of features. Included here are both acceleration and retardation of growth.'),
  'D006131': NtO(MH='Growth Inhibitors', MS='Endogenous or exogenous substances which inhibit the normal growth of human and animal cells or micro-organisms, as distinguished from those affecting plant growth (= PLANT GROWTH REGULATORS).'),
  'D006132': NtO(MH='Growth Plate', MS='The area between the EPIPHYSIS and the DIAPHYSIS within which bone growth occurs.'),
  'D006133': NtO(MH='Growth Substances', MS='Signal molecules that are involved in the control of cell growth and differentiation.'),
  'D006135': NtO(MH='Gryllidae', MS='The family Gryllidae consists of the common house cricket, Acheta domesticus, which is used in neurological and physiological studies. Other genera include Gryllotalpa (mole cricket); Gryllus (field cricket); and Oecanthus (tree cricket).'),
  'D006136': NtO(MH='GTP Cyclohydrolase', MS='(GTP cyclohydrolase I) or GTP 7,8-8,9-dihydrolase (pyrophosphate-forming) (GTP cyclohydrolase II). An enzyme group that hydrolyzes the imidazole ring of GTP, releasing carbon-8 as formate. Two C-N bonds are hydrolyzed and the pentase unit is isomerized. This is the first step in the synthesis of folic acid from GTP. EC 3.5.4.16 (GTP cyclohydrolase I) and EC 3.5.4.25 (GTP cyclohydrolase II).'),
  'D006138': NtO(MH='Guaiac', MS='Resin from wood of certain species of GUAIACUM. It is used as clinical reagent for occult blood.'),
  'D006139': NtO(MH='Guaiacol', MS='An agent thought to have disinfectant properties and used as an expectorant. (From Martindale, The Extra Pharmacopoeia, 30th ed, p747)'),
  'D006140': NtO(MH='Guaifenesin', MS='An expectorant that also has some muscle relaxing action. It is used in many cough preparations.'),
  'D006141': NtO(MH='Guam', MS="An island in Micronesia, east of the Philippines, the largest and southernmost of the Marianas. Its capital is Agana. It was discovered by Magellan in 1521 and occupied by Spain in 1565. They ceded it to the United States in 1898. It is an unincorporated territory of the United States, administered by the Department of the Interior since 1950. The derivation of the name Guam is in dispute. (From Webster's New Geographical Dictionary, 1988, p471)"),
  'D006143': NtO(MH='Guanabenz', MS='An alpha-2 selective adrenergic agonist used as an antihypertensive agent.'),
  'D006144': NtO(MH='Guanazole', MS='A cytostatic triazole derivative which is not to be confused with guanazolo, the generic name for 8-azaguanine.'),
  'D006145': NtO(MH='Guanethidine', MS='An antihypertensive agent that acts by inhibiting selectively transmission in post-ganglionic adrenergic nerves. It is believed to act mainly by preventing the release of norepinephrine at nerve endings and causes depletion of norepinephrine in peripheral sympathetic nerve terminals as well as in tissues.'),
  'D006146': NtO(MH='Guanidines', MS='A family of iminourea derivatives. The parent compound has been isolated from mushrooms, corn germ, rice hulls, mussels, earthworms, and turnip juice. Derivatives may have antiviral and antifungal properties.'),
  'D006147': NtO(MH='Guanine', MS=None),
  'D006148': NtO(MH='Guanine Deaminase', MS='An enzyme that catalyzes the deamination of guanine to form xanthine. EC 3.5.4.3.'),
  'D006150': NtO(MH='Guanine Nucleotides', MS=None),
  'D006151': NtO(MH='Guanosine', MS='A purine nucleoside that has guanine linked by its N9 nitrogen to the C1 carbon of ribose. It is a component of ribonucleic acid and its nucleotides play important roles in metabolism. (From Dorland, 28th ed)'),
  'D006152': NtO(MH='Cyclic GMP', MS="Guanosine cyclic 3',5'-(hydrogen phosphate). A guanine nucleotide containing one phosphate group which is esterified to the sugar moiety in both the 3'- and 5'-positions. It is a cellular regulatory agent and has been described as a second messenger. Its levels increase in response to a variety of hormones, including acetylcholine, insulin, and oxytocin and it has been found to activate specific protein kinases. (From Merck Index, 11th ed)"),
  'D006153': NtO(MH='Guanosine Diphosphate', MS='A guanine nucleotide containing two phosphate groups esterified to the sugar moiety.'),
  'D006154': NtO(MH='Guanosine Diphosphate Fucose', MS='A nucleoside diphosphate sugar formed from GDPmannose, which provides fucose for lipopolysaccharides of bacterial cell walls, and for blood group substances and other glycoproteins.'),
  'D006155': NtO(MH='Guanosine Diphosphate Mannose', MS='A nucleoside diphosphate sugar which can be converted to the deoxy sugar GDPfucose, which provides fucose for lipopolysaccharides of bacterial cell walls. Also acts as mannose donor for glycolipid synthesis.'),
  'D006156': NtO(MH='Guanosine Diphosphate Sugars', MS='Esters formed between the aldehydic carbon of sugars and the terminal phosphate of guanosine diphosphate.'),
  'D006157': NtO(MH='Guanosine Monophosphate', MS='A guanine nucleotide containing one phosphate group esterified to the sugar moiety and found widely in nature.'),
  'D006158': NtO(MH='Guanosine Pentaphosphate', MS="Guanosine 5'-triphosphate 2'(3')-diphosphate. A guanine nucleotide containing five phosphate groups. Three phosphate groups are esterified to the sugar moiety in the 5' position and the other two in the 2' or 3' position. This nucleotide serves as a messenger to turn off the synthesis of ribosomal RNA when amino acids are not available for protein synthesis. Synonym: magic spot II."),
  'D006159': NtO(MH='Guanosine Tetraphosphate', MS="Guanosine 5'-diphosphate 2'(3')-diphosphate. A guanine nucleotide containing four phosphate groups. Two phosphate groups are esterified to the sugar moiety in the 5' position and the other two in the 2' or 3' position. This nucleotide serves as a messenger to turn off the synthesis of ribosomal RNA when amino acids are not available for protein synthesis. Synonym: magic spot I."),
  'D006160': NtO(MH='Guanosine Triphosphate', MS="Guanosine 5'-(tetrahydrogen triphosphate). A guanine nucleotide containing three phosphate groups esterified to the sugar moiety."),
  'D006162': NtO(MH='Guanylate Cyclase', MS="An enzyme that catalyzes the conversion of GTP to 3',5'-cyclic GMP and pyrophosphate. It also acts on ITP and dGTP. (From Enzyme Nomenclature, 1992) EC 4.6.1.2."),
  'D006163': NtO(MH='Ribonuclease T1', MS="An enzyme catalyzing the endonucleolytic cleavage of RNA at the 3'-position of a guanylate residue. EC 3.1.27.3."),
  'D006164': NtO(MH='Guanylthiourea', MS='May protect against hypoxic damage; proposed for treatment of shock due to trauma or blood loss; also stimulates paretic gastrointestinal system.'),
  'D006165': NtO(MH='Guanylyl Imidodiphosphate', MS='A non-hydrolyzable analog of GTP, in which the oxygen atom bridging the beta to the gamma phosphate is replaced by a nitrogen atom. It binds tightly to G-protein in the presence of Mg2+. The nucleotide is a potent stimulator of ADENYLYL CYCLASES.'),
  'D006166': NtO(MH='Guatemala', MS='A country in CENTRAL AMERICA bordering the North Pacific Ocean, between EL SALVADOR and MEXICO, and bordering the Gulf of Honduras (Caribbean Sea) between HONDURAS and BELIZE.'),
  'D006167': NtO(MH='Guilt', MS="Subjective feeling of having committed an error, offense or sin; unpleasant feeling of self-criticism. These result from acts, impulses, or thoughts contrary to one's personal conscience."),
  'D006168': NtO(MH='Guinea Pigs', MS='A common name used for the genus Cavia. The most common species is Cavia porcellus which is the domesticated guinea pig used for pets and biomedical research.'),
  'D006169': NtO(MH='Guinea-Bissau', MS='A republic in western Africa, south of SENEGAL and west of GUINEA. Its capital is Bissau.'),
  'D006170': NtO(MH='Gum Arabic', MS='Powdered exudate from various Acacia species, especially A. senegal (Leguminosae). It forms mucilage or syrup in water. Gum arabic is used as a suspending agent, excipient, and emulsifier in foods and pharmaceuticals.'),
  'D006171': NtO(MH='Gutta-Percha', MS='Coagulated exudate isolated from several species of the tropical tree Palaquium (Sapotaceae). It is the trans-isomer of natural rubber and is used as a filling and impression material in dentistry and orthopedics and as an insulator in electronics. It has also been used as a rubber substitute.'),
  'D006172': NtO(MH='Guyana', MS='A republic in the north of South America, east of VENEZUELA and west of SURINAME. Its capital is Georgetown.'),
  'D006173': NtO(MH='Gymnastics', MS='A system of exercise using the weight of the body and specific apparatus to promote strength and grace of carriage.'),
  'D006175': NtO(MH='Gynatresia', MS='Absence of a normal opening in the lumen of the female genital tract, from the FALLOPIAN TUBES to the VAGINA. This anomaly may be congenital or acquired due to injuries, diseases, or TISSUE ADHESIONS.'),
  'D006176': NtO(MH='Gynecology', MS='A medical-surgical specialty concerned with the physiology and disorders primarily of the female genital tract, as well as female endocrinology and reproductive physiology.'),
  'D006177': NtO(MH='Gynecomastia', MS='Enlargement of the BREAST in the males, caused by an excess of ESTROGENS. Physiological gynecomastia is normally observed in NEWBORNS; ADOLESCENT; and AGING males.'),
  'D006178': NtO(MH='Roma', MS='Ethnic group originating in India and entering Europe in the 14th or 15th century.'),
  'D006179': NtO(MH='Gyrus Cinguli', MS='One of the convolutions on the medial surface of the CEREBRAL HEMISPHERES. It surrounds the rostral part of the brain and CORPUS CALLOSUM and forms part of the LIMBIC SYSTEM.'),
  'D006180': NtO(MH='Proton-Translocating ATPases', MS='Multisubunit enzymes that reversibly synthesize ADENOSINE TRIPHOSPHATE. They are coupled to the transport of protons across a membrane.'),
  'D006181': NtO(MH='H-Reflex', MS='A monosynaptic reflex elicited by stimulating a nerve, particularly the tibial nerve, with an electric shock.'),
  'D006182': NtO(MH='H-Y Antigen', MS='A sex-specific cell surface antigen produced by the sex-determining gene of the Y chromosome in mammals. It causes syngeneic grafts from males to females to be rejected and interacts with somatic elements of the embryologic undifferentiated gonad to produce testicular organogenesis.'),
  'D006183': NtO(MH='H-2 Antigens', MS='The major group of transplantation antigens in the mouse.'),
  'D006184': NtO(MH='Habits', MS='Acquired or learned responses which are regularly manifested.'),
  'D006185': NtO(MH='Habituation, Psychophysiologic', MS='The disappearance of responsiveness to a repeated stimulation. It does not include drug habituation.'),
  'D006188': NtO(MH='Haemonchiasis', MS='Infection with nematodes of the genus HAEMONCHUS, characterized by digestive abnormalities and anemia similar to that from hookworm infestation.'),
  'D006189': NtO(MH='Haemonchus', MS='A genus of parasitic nematode worms which infest the duodenum and stomach of domestic and wild herbivores, which ingest it with the grasses (POACEAE) they eat. Infestation of man is accidental.'),
  'D006190': NtO(MH='Haemophilus', MS='A genus of PASTEURELLACEAE that consists of several species occurring in animals and humans. Its organisms are described as gram-negative, facultatively anaerobic, coccobacillus or rod-shaped, and nonmotile.'),
  'D006191': NtO(MH='Haemophilus ducreyi', MS='A species of HAEMOPHILUS that appears to be the pathogen or causative agent of the sexually transmitted disease, CHANCROID.'),
  'D006192': NtO(MH='Haemophilus Infections', MS='Infections with bacteria of the genus HAEMOPHILUS.'),
  'D006193': NtO(MH='Haemophilus influenzae', MS='A species of HAEMOPHILUS found on the mucous membranes of humans and a variety of animals. The species is further divided into biotypes I through VIII.'),
  'D006194': NtO(MH='Gardnerella vaginalis', MS='A species in the genus GARDNERELLA previously classified as Haemophilus vaginalis. This bacterium, also isolated from the female genital tract of healthy women, is implicated in the cause of bacterial vaginosis (VAGINOSIS, BACTERIAL).'),
  'D006195': NtO(MH='Hafnium', MS='A metal element of atomic number 72 and atomic weight 178.49, symbol Hf.'),
  'D006196': NtO(MH='Hagfishes', MS='Common name for a family of eel-shaped jawless fishes (Myxinidae), the only family in the order MYXINIFORMES. They are not true vertebrates.'),
  'D006197': NtO(MH='Hair', MS='A filament-like structure consisting of a shaft which projects to the surface of the SKIN from a root which is softer than the shaft and lodges in the cavity of a HAIR FOLLICLE. It is found on most surfaces of the body.'),
  'D006198': NtO(MH='Hair Cells, Auditory', MS='Sensory cells in the organ of Corti, characterized by their apical stereocilia (hair-like projections). The inner and outer hair cells, as defined by their proximity to the core of spongy bone (the modiolus), change morphologically along the COCHLEA. Towards the cochlear apex, the length of hair cell bodies and their apical STEREOCILIA increase, allowing differential responses to various frequencies of sound.'),
  'D006199': NtO(MH='Hair Cells, Auditory, Inner', MS='Auditory sensory cells of organ of Corti, usually placed in one row medially to the core of spongy bone (the modiolus). Inner hair cells are in fewer numbers than the OUTER AUDITORY HAIR CELLS, and their STEREOCILIA are approximately twice as thick as those of the outer hair cells.'),
  'D006200': NtO(MH='Hair Color', MS='Color of hair or fur.'),
  'D006201': NtO(MH='Hair Diseases', MS='Diseases affecting the orderly growth and persistence of hair.'),
  'D006202': NtO(MH='Hair Dyes', MS='Dyes used as cosmetics to change hair color either permanently or temporarily.'),
  'D006203': NtO(MH='Hair Preparations', MS='Hair grooming, cleansing and modifying products meant for topical application to hair, usually human. They include sprays, bleaches, dyes, conditioners, rinses, shampoos, nutrient lotions, etc.'),
  'D006204': NtO(MH='Hair Removal', MS='Methods used to remove unwanted facial and body hair.'),
  'D006205': NtO(MH='Haiti', MS="A republic in the Greater Antilles in the West Indies. Its capital is Port-au-Prince. With the Dominican Republic it forms the island of Hispaniola - Haiti occupying the western third and the Dominican Republic, the eastern two thirds. Haiti belonged to France from 1697 until its rule was challenged by slave insurrections from 1791. It became a republic in 1820. It was virtually an American protectorate from 1915 to 1934. It adopted its present constitution in 1964 and amended it in 1971. The name may represent either of two Caribbean words, haiti, mountain land, or jhaiti, nest. (From Webster's New Geographical Dictionary, 1988, p481 & Room, Brewer's Dictionary of Names, 1992, p225)"),
  'D006206': NtO(MH='Halcinonide', MS='A glucocorticoid used topically in the treatment of DERMATITIS; ECZEMA; or PSORIASIS. It may cause skin irritation.'),
  'D006207': NtO(MH='Half-Life', MS='The time it takes for a substance (drug, radioactive nuclide, or other) to lose half of its pharmacologic, physiologic, or radiologic activity.'),
  'D006208': NtO(MH='Halfway Houses', MS='Specialized residences for persons who do not require full hospitalization, and are not well enough to function completely within the community without professional supervision, protection and support.'),
  'D006209': NtO(MH='Halitosis', MS='An offensive, foul breath odor resulting from a variety of causes such as poor oral hygiene, dental or oral infections, or the ingestion of certain foods.'),
  'D006210': NtO(MH="Hallermann's Syndrome", MS='An oculomandibulofacial syndrome principally characterized by dyscephaly (usually brachycephaly), parrot nose, mandibular hypoplasia, proportionate nanism, hypotrichosis, bilateral congenital cataracts, and microphthalmia. (Dorland, 27th ed)'),
  'D006211': NtO(MH='Pantothenate Kinase-Associated Neurodegeneration', MS='A rare autosomal recessive degenerative disorder which usually presents in late childhood or adolescence. Clinical manifestations include progressive MUSCLE SPASTICITY; hyperreflexia; MUSCLE RIGIDITY; DYSTONIA; DYSARTHRIA; and intellectual deterioration which progresses to severe dementia over several years. (From Adams et al., Principles of Neurology, 6th ed, p972; Davis & Robertson, Textbook of Neuropathology, 2nd ed, pp972-929)'),
  'D006212': NtO(MH='Hallucinations', MS='Subjectively experienced sensations in the absence of an appropriate stimulus, but which are regarded by the individual as real. They may be of organic origin or associated with MENTAL DISORDERS.'),
  'D006213': NtO(MH='Hallucinogens', MS='Drugs capable of inducing illusions, hallucinations, delusions, paranoid ideations, and other alterations of mood and thinking. Despite the name, the feature that distinguishes these agents from other classes of drugs is their capacity to induce states of altered perception, thought, and feeling that are not experienced otherwise.'),
  'D006214': NtO(MH='Hallux', MS='The innermost digit of the foot in PRIMATES.'),
  'D006215': NtO(MH='Hallux Valgus', MS='Lateral displacement of the great toe (HALLUX), producing deformity of the first METATARSOPHALANGEAL JOINT with callous, bursa, or BUNION formation over the bony prominence.'),
  'D006216': NtO(MH='Halobacteriaceae', MS='A family of extremely halophilic archaea found in environments with high salt concentrations, such as salt lakes, evaporated brines, or salted fish. Halobacteriaceae are either obligate aerobes or facultative anaerobes and are divided into at least twenty-six genera including: HALOARCULA; HALOBACTERIUM; HALOCOCCUS; HALOFERAX; HALORUBRUM; NATRONOBACTERIUM; and NATRONOCOCCUS.'),
  'D006217': NtO(MH='Halobacterium', MS='A genus of HALOBACTERIACEAE whose growth requires a high concentration of salt. Binary fission is by constriction.'),
  'D006218': NtO(MH='Halofenate', MS='An antihyperlipoproteinemic agent and uricosuric agent.'),
  'D006219': NtO(MH='Halogens', MS='A family of nonmetallic, generally electronegative, elements that form group 17 (formerly group VIIa) of the periodic table.'),
  'D006220': NtO(MH='Haloperidol', MS='A phenyl-piperidinyl-butyrophenone that is used primarily to treat SCHIZOPHRENIA and other PSYCHOSES. It is also used in schizoaffective disorder, DELUSIONAL DISORDERS, ballism, and TOURETTE SYNDROME (a drug of choice) and occasionally as adjunctive therapy in INTELLECTUAL DISABILITY and the chorea of HUNTINGTON DISEASE. It is a potent antiemetic and is used in the treatment of intractable HICCUPS. (From AMA Drug Evaluations Annual, 1994, p279)'),
  'D006221': NtO(MH='Halothane', MS='A nonflammable, halogenated, hydrocarbon anesthetic that provides relatively rapid induction with little or no excitement. Analgesia may not be adequate. NITROUS OXIDE is often given concomitantly. Because halothane may not produce sufficient muscle relaxation, supplemental neuromuscular blocking agents may be required. (From AMA Drug Evaluations Annual, 1994, p178)'),
  'D006222': NtO(MH='Hamartoma', MS='A focal malformation resembling a neoplasm, composed of an overgrowth of mature cells and tissues that normally occur in the affected area.'),
  'D006223': NtO(MH='Hamartoma Syndrome, Multiple', MS='A hereditary disease characterized by multiple ectodermal, mesodermal, and endodermal nevoid and neoplastic anomalies. Facial trichilemmomas and papillomatous papules of the oral mucosa are the most characteristic lesions. Individuals with this syndrome have a high risk of BREAST CANCER; THYROID CANCER; and ENDOMETRIAL CANCER. This syndrome is associated with mutations in the gene for PTEN PHOSPHATASE.'),
  'D006224': NtO(MH='Cricetinae', MS='A subfamily in the family MURIDAE, comprising the hamsters. Four of the more common genera are Cricetus, CRICETULUS; MESOCRICETUS; and PHODOPUS.'),
  'D006225': NtO(MH='Hand', MS='The distal part of the arm beyond the wrist in humans and primates, that includes the palm, fingers, and thumb.'),
  'D006226': NtO(MH='Hand Deformities', MS='Alterations or deviations from normal shape or size which result in a disfigurement of the hand.'),
  'D006227': NtO(MH='Hand Deformities, Acquired', MS='Deformities of the hand, or a part of the hand, acquired after birth as the result of injury or disease.'),
  'D006228': NtO(MH='Hand Deformities, Congenital', MS='Alterations or deviations from normal shape or size which result in a disfigurement of the hand occurring at or before birth.'),
  'D006229': NtO(MH='Hand Dermatoses', MS='Skin diseases involving the HANDS.'),
  'D006230': NtO(MH='Hand Injuries', MS='General or unspecified injuries to the hand.'),
  'D006232': NtO(MH='Hand, Foot and Mouth Disease', MS='A mild, highly infectious viral disease of children, characterized by vesicular lesions in the mouth and on the hands and feet. It is caused by coxsackieviruses A.'),
  'D006233': NtO(MH='Disabled Persons', MS='Persons with physical or mental disabilities that affect or limit their activities of daily living and that may require special accommodations.'),
  'D006234': NtO(MH='Handling, Psychological', MS='Physical manipulation of animals and humans to induce a behavioral or other psychological reaction. In experimental psychology, the animal is handled to induce a stress situation or to study the effects of "gentling" or "mothering".'),
  'D006235': NtO(MH='Hand Disinfection', MS='The act of cleansing the hands with water or other liquid, with or without the inclusion of soap or other detergent, for the purpose of destroying infectious microorganisms.'),
  'D006236': NtO(MH='Handwriting', MS='Manual writing with a pen or pencil.'),
  'D006238': NtO(MH='Haploidy', MS='The chromosomal constitution of cells, in which each type of CHROMOSOME is represented once. Symbol: N.'),
  'D006239': NtO(MH='Haplotypes', MS='The genetic constitution of individuals with respect to one member of a pair of allelic genes, or sets of genes that are closely linked and tend to be inherited together such as those of the MAJOR HISTOCOMPATIBILITY COMPLEX.'),
  'D006240': NtO(MH='Happiness', MS='Highly pleasant emotion characterized by outward manifestations of gratification; joy.'),
  'D006241': NtO(MH='Haptens', MS='Small antigenic determinants capable of eliciting an immune response only when coupled to a carrier. Haptens bind to antibodies but by themselves cannot elicit an antibody response.'),
  'D006242': NtO(MH='Haptoglobins', MS='Plasma glycoproteins that form a stable complex with hemoglobin to aid the recycling of heme iron. They are encoded in man by a gene on the short arm of chromosome 16.'),
  'D006243': NtO(MH='Harderian Gland', MS='A sebaceous gland that, in some animals, acts as an accessory to the lacrimal gland. The harderian gland excretes fluid that facilitates movement of the third eyelid.'),
  'D006244': NtO(MH='Hardness', MS='The mechanical property of material that determines its resistance to force. HARDNESS TESTS measure this property.'),
  'D006245': NtO(MH='Hardness Tests', MS='A test to determine the relative hardness of a metal, mineral, or other material according to one of several scales, such as Brinell, Mohs, Rockwell, Vickers, or Shore. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D006246': NtO(MH='Harmaline', MS='A beta-carboline alkaloid isolated from seeds of PEGANUM.'),
  'D006247': NtO(MH='Harmine', MS="Alkaloid isolated from seeds of PEGANUM HARMALA; ZYGOPHYLLACEAE. It is identical to banisterine, or telepathine, from Banisteria caapi and is one of the active ingredients of hallucinogenic drinks made in the western Amazon region from related plants. It has no therapeutic use, but (as banisterine) was hailed as a cure for postencephalitic PARKINSON DISEASE in the 1920's."),
  'D006248': NtO(MH='Harringtonines', MS='Tetracyclic spiro-BENZAZEPINES isolated from the seeds of CEPHALOTAXUS. They are esters of the alkaloid cephalotaxine and may be effective as antineoplastic agents.'),
  'D006249': NtO(MH='Hartmannella', MS='A genus of free-living amoebae found in fresh water. The cysts usually pass harmlessly through the intestinal tract of man and may thus be found in feces. Occasionally, these organisms cause respiratory tract infections or generalized fatal meningoencephalitis.'),
  'D006250': NtO(MH='Hartnup Disease', MS='An autosomal recessive disorder due to defective absorption of NEUTRAL AMINO ACIDS by both the intestine and the PROXIMAL RENAL TUBULES. The abnormal urinary loss of TRYPTOPHAN, a precursor of NIACIN, leads to a NICOTINAMIDE deficiency, PELLAGRA-like light-sensitive rash, CEREBELLAR ATAXIA, emotional instability, and aminoaciduria. Mutations involve the neurotransmitter transporter gene SLC6A19.'),
  'D006251': NtO(MH='Harvey murine sarcoma virus', MS='A replication-defective mouse sarcoma virus (SARCOMA VIRUSES, MURINE) first described by J.J. Harvey in 1964.'),
  'D006252': NtO(MH='Hate', MS='An enduring attitude or sentiment toward persons or objects manifested by anger, aversion and desire for the misfortune of others.'),
  'D006253': NtO(MH='Haversian System', MS='A circular structural unit of bone tissue. It consists of a central hole, the Haversian canal through which blood vessels run, surrounded by concentric rings, called lamellae.'),
  'D006254': NtO(MH='Hawaii', MS="A group of islands in Polynesia, in the north central Pacific Ocean, comprising eight major and 114 minor islands, largely volcanic and coral. Its capital is Honolulu. It was first reached by Polynesians about 500 A.D. It was discovered and named the Sandwich Islands in 1778 by Captain Cook. The islands were united under the rule of King Kamehameha 1795-1819 and requested annexation to the United States in 1893 when a provisional government was set up. Hawaii was established as a territory in 1900 and admitted as a state in 1959. The name is from the Polynesian Owhyhii, place of the gods, with reference to the two volcanoes Mauna Kea and Mauna Loa, regarded as the abode of the gods. (From Webster's New Geographical Dictionary, 1988, p493 & Room, Brewer's Dictionary of Names, 1992, p2330)"),
  'D006255': NtO(MH='Rhinitis, Allergic, Seasonal', MS='Allergic rhinitis that occurs at the same time every year. It is characterized by acute CONJUNCTIVITIS with lacrimation and ITCHING, and regarded as an allergic condition triggered by specific ALLERGENS.'),
  'D006256': NtO(MH='Hazardous Waste', MS='Waste products which threaten life, health, or the environment when improperly treated, stored, transported, disposed of, or otherwise managed.'),
  'D006257': NtO(MH='Head', MS='The upper part of the human body, or the front or upper part of the body of an animal, typically separated from the rest of the body by a neck, and containing the brain, mouth, and sense organs.'),
  'D006258': NtO(MH='Head and Neck Neoplasms', MS='Soft tissue tumors or cancer arising from the mucosal surfaces of the LIP; oral cavity; PHARYNX; LARYNX; and cervical esophagus. Other sites included are the NOSE and PARANASAL SINUSES; SALIVARY GLANDS; THYROID GLAND and PARATHYROID GLANDS; and MELANOMA and non-melanoma skin cancers of the head and neck. (from Holland et al., Cancer Medicine, 4th ed, p1651)'),
  'D006259': NtO(MH='Craniocerebral Trauma', MS='Traumatic injuries involving the cranium and intracranial structures (i.e., BRAIN; CRANIAL NERVES; MENINGES; and other structures). Injuries may be classified by whether or not the skull is penetrated (i.e., penetrating vs. nonpenetrating) or whether there is an associated hemorrhage.'),
  'D006260': NtO(MH='Head Protective Devices', MS='Personal devices for protection of heads from impact, penetration from falling and flying objects, and from limited electric shock and burn.'),
  'D006261': NtO(MH='Headache', MS='The symptom of PAIN in the cranial region. It may be an isolated benign occurrence or manifestation of a wide variety of HEADACHE DISORDERS.'),
  'D006262': NtO(MH='Health', MS='The state of the organism when it functions optimally without evidence of disease.'),
  'D006264': NtO(MH='Health Benefit Plans, Employee', MS='Health insurance plans for employees, and generally including their dependents, usually on a cost-sharing basis with the employer paying a percentage of the premium.'),
  'D006265': NtO(MH='Health Care Coalitions', MS='Voluntary groups of people representing diverse interests in the community such as hospitals, businesses, physicians, and insurers, with the principal objective to improve health care cost effectiveness.'),
  'D006266': NtO(MH='Health Education', MS='Education that increases the awareness and favorably influences the attitudes and knowledge relating to the improvement of health on a personal or community basis.'),
  'D006267': NtO(MH='Health Education, Dental', MS='Education which increases the awareness and favorably influences the attitudes and knowledge relating to the improvement of dental health on a personal or community basis.'),
  'D006268': NtO(MH='Health Facilities', MS='Institutions which provide medical or health-related services.'),
  'D006269': NtO(MH='Health Facilities, Proprietary', MS='Health care institutions operated by private groups or corporations for a profit.'),
  'D006270': NtO(MH='Health Facility Administrators', MS='Managerial personnel responsible for implementing policy and directing the activities of health care facilities such as nursing homes.'),
  'D006271': NtO(MH='Health Facility Closure', MS='The closing of any health facility, e.g., health centers, residential facilities, and hospitals.'),
  'D006272': NtO(MH='Health Facility Environment', MS='Physical surroundings or conditions of a hospital or other health facility and influence of these factors on patients and staff.'),
  'D006273': NtO(MH='Health Facility Merger', MS='The combining of administrative and organizational resources of two or more health care facilities.'),
  'D006274': NtO(MH='Health Facility Moving', MS='The relocation of health care institutions or units thereof. The concept includes equipment relocation.'),
  'D006275': NtO(MH='Health Facility Planning', MS='Areawide planning for health care institutions on the basis of projected consumer need.'),
  'D006276': NtO(MH='Health Facility Size', MS='The physical space or dimensions of a facility. Size may be indicated by bed capacity.'),
  'D006277': NtO(MH='Health Fairs', MS='Community health education events focused on prevention of disease and promotion of health through audiovisual exhibits.'),
  'D006278': NtO(MH='Medicare', MS='Federal program, created by Public Law 89-97, Title XVIII-Health Insurance for the Aged, a 1965 amendment to the Social Security Act, that provides health insurance benefits to persons over the age of 65 and others eligible for Social Security benefits. It consists of two separate but coordinated programs: hospital insurance (MEDICARE PART A) and supplementary medical insurance (MEDICARE PART B). (Hospital Administration Terminology, AHA, 2d ed and A Discursive Dictionary of Health Care, US House of Representatives, 1976)'),
  'D006279': NtO(MH='Health Maintenance Organizations', MS='Organized systems for providing comprehensive prepaid health care that have five basic attributes: (1) provide care in a defined geographic area; (2) provide or ensure delivery of an agreed-upon set of basic and supplemental health maintenance and treatment services; (3) provide care to a voluntarily enrolled group of persons; (4) require their enrollees to use the services of designated providers; and (5) receive reimbursement through a predetermined, fixed, periodic prepayment made by the enrollee without regard to the degree of services provided. (From Facts on File Dictionary of Health Care Management, 1988)'),
  'D006280': NtO(MH='Health Workforce', MS='The availability of HEALTH PERSONNEL. It includes the demand and recruitment of both professional and allied health personnel, their present and future supply and distribution, and their assignment and utilization.'),
  'D006281': NtO(MH='Health Occupations', MS='Professions or other business activities directed to the cure and prevention of disease. For occupations of medical personnel who are not physicians but who are working in the fields of medical technology, physical therapy, etc., ALLIED HEALTH OCCUPATIONS is available.'),
  'D006282': NtO(MH='Health Personnel', MS='Men and women working in the provision of health services, whether as individual practitioners or employees of health institutions and programs, whether or not professionally trained, and whether or not subject to public regulation. (From A Discursive Dictionary of Health Care, 1976)'),
  'D006283': NtO(MH='Health Physics', MS='The science concerned with problems of radiation protection relevant to reducing or preventing radiation exposure, and the effects of ionizing radiation on humans and their environment.'),
  'D006284': NtO(MH='Health Plan Implementation', MS='Those actions designed to carry out recommendations pertaining to health plans or programs.'),
  'D006285': NtO(MH='Health Planning', MS='Planning for needed health and/or welfare services and facilities.'),
  'D006286': NtO(MH='Health Planning Councils', MS='Organized groups serving in advisory capacities related to health planning activities.'),
  'D006287': NtO(MH='Health Planning Guidelines', MS='Recommendations for directing health planning functions and policies. These may be mandated by PL93-641 and issued by the Department of Health and Human Services for use by state and local planning agencies.'),
  'D006288': NtO(MH='Health Planning Organizations', MS='Organizations involved in all aspects of health planning activities.'),
  'D006289': NtO(MH='Health Planning Support', MS='Financial resources provided for activities related to health planning and development.'),
  'D006290': NtO(MH='Health Planning Technical Assistance', MS='The provision of expert assistance in developing health planning programs, plans as technical materials, etc., as requested by Health Systems Agencies or other health planning organizations.'),
  'D006291': NtO(MH='Health Policy', MS='Decisions, usually developed by government policymakers, for determining present and future objectives pertaining to the health care system.'),
  'D006292': NtO(MH='Health Priorities', MS='Preferentially rated health-related activities or functions to be used in establishing health planning goals. This may refer specifically to PL93-641.'),
  'D006293': NtO(MH='Health Promotion', MS='Encouraging consumer behaviors most likely to optimize health potentials (physical and psychosocial) through health information, preventive programs, and access to medical care.'),
  'D006294': NtO(MH='Health Resorts', MS='Destinations frequented as vacation or recreational retreats, equipped with specialized amenities, treatments and facilities, for the promotion of general well-being and health.'),
  'D006295': NtO(MH='Health Resources', MS='Available manpower, facilities, revenue, equipment, and supplies to produce requisite health care and services.'),
  'D006296': NtO(MH='Health Services', MS='Services for the diagnosis and treatment of disease and the maintenance of health.'),
  'D006297': NtO(MH='Health Services Accessibility', MS='The degree to which individuals are inhibited or facilitated in their ability to gain entry to and to receive care and services from the health care system. Factors influencing this ability include geographic, architectural, transportational, and financial considerations, among others.'),
  'D006298': NtO(MH='Health Services Administration', MS='The organization and administration of health services dedicated to the delivery of health care.'),
  'D006299': NtO(MH='Health Services for the Aged', MS='Services for the diagnosis and treatment of diseases in the aged and the maintenance of health in the elderly.'),
  'D006300': NtO(MH='Health Services Misuse', MS='Excessive, under or unnecessary utilization of health services by patients or physicians.'),
  'D006301': NtO(MH='Health Services Needs and Demand', MS='Health services required by a population or community as well as the health services that the population or community is able and willing to pay for.'),
  'D006302': NtO(MH='Health Services Research', MS='The integration of epidemiologic, sociological, economic, and other analytic sciences in the study of health services. Health services research is usually concerned with relationships between need, demand, supply, use, and outcome of health services. The aim of the research is evaluation, particularly in terms of structure, process, output, and outcome. (From Last, Dictionary of Epidemiology, 2d ed)'),
  'D006303': NtO(MH='Health Services, Indigenous', MS='Health care provided to specific cultural or tribal peoples which incorporates local customs, beliefs, and taboos.'),
  'D006304': NtO(MH='Health Status', MS='The level of health of the individual, group, or population as subjectively assessed by the individual or by more objective measures.'),
  'D006305': NtO(MH='Health Status Indicators', MS='The measurement of the health status for a given population using a variety of indices, including morbidity, mortality, and available health resources.'),
  'D006306': NtO(MH='Health Surveys', MS='A systematic collection of factual data pertaining to health and disease in a human population within a given geographic area.'),
  'D006307': NtO(MH='Health Systems Agencies', MS='Health planning and resources development agencies which function in each health service area of the United States (PL 93-641).'),
  'D006308': NtO(MH='Health Systems Plans', MS='Statements of goals for the delivery of health services pertaining to the Health Systems Agency service area, established under PL 93-641, and consistent with national guidelines for health planning.'),
  'D006309': NtO(MH='Hearing', MS='The ability or act of sensing and transducing ACOUSTIC STIMULATION to the CENTRAL NERVOUS SYSTEM. It is also called audition.'),
  'D006310': NtO(MH='Hearing Aids', MS='Wearable sound-amplifying devices that are intended to compensate for impaired hearing. These generic devices include air-conduction hearing aids and bone-conduction hearing aids. (UMDNS, 1999)'),
  'D006311': NtO(MH='Hearing Disorders', MS='Conditions that impair the transmission of auditory impulses and information from the level of the ear to the temporal cortices, including the sensorineural pathways.'),
  'D006312': NtO(MH='Hearing Loss, Bilateral', MS='Partial hearing loss in both ears.'),
  'D006313': NtO(MH='Hearing Loss, Central', MS='Hearing loss due to disease of the AUDITORY PATHWAYS (in the CENTRAL NERVOUS SYSTEM) which originate in the COCHLEAR NUCLEI of the PONS and then ascend bilaterally to the MIDBRAIN, the THALAMUS, and then the AUDITORY CORTEX in the TEMPORAL LOBE. Bilateral lesions of the auditory pathways are usually required to cause central hearing loss. Cortical deafness refers to loss of hearing due to bilateral auditory cortex lesions. Unilateral BRAIN STEM lesions involving the cochlear nuclei may result in unilateral hearing loss.'),
  'D006314': NtO(MH='Hearing Loss, Conductive', MS='Hearing loss due to interference with the mechanical reception or amplification of sound to the COCHLEA. The interference is in the outer or middle ear involving the EAR CANAL; TYMPANIC MEMBRANE; or EAR OSSICLES.'),
  'D006315': NtO(MH='Hearing Loss, Functional', MS='Hearing loss without a physical basis. Often observed in patients with psychological or behavioral disorders.'),
  'D006316': NtO(MH='Hearing Loss, High-Frequency', MS='Hearing loss in frequencies above 1000 hertz.'),
  'D006317': NtO(MH='Hearing Loss, Noise-Induced', MS='Hearing loss due to exposure to explosive loud noise or chronic exposure to sound level greater than 85 dB. The hearing loss is often in the frequency range 4000-6000 hertz.'),
  'D006319': NtO(MH='Hearing Loss, Sensorineural', MS='Hearing loss resulting from damage to the COCHLEA and the sensorineural elements which lie internally beyond the oval and round windows. These elements include the AUDITORY NERVE and its connections in the BRAINSTEM.'),
  'D006320': NtO(MH='Hearing Tests', MS='Part of an ear examination that measures the ability of sound to reach the brain.'),
  'D006321': NtO(MH='Heart', MS='The hollow, muscular organ that maintains the circulation of the blood.'),
  'D006322': NtO(MH='Heart Aneurysm', MS='A localized bulging or dilatation in the muscle wall of a heart (MYOCARDIUM), usually in the LEFT VENTRICLE. Blood-filled aneurysms are dangerous because they may burst. Fibrous aneurysms interfere with the heart function through the loss of contractility. True aneurysm is bound by the vessel wall or cardiac wall. False aneurysms are HEMATOMA caused by myocardial rupture.'),
  'D006323': NtO(MH='Heart Arrest', MS='Cessation of heart beat or MYOCARDIAL CONTRACTION. If it is treated within a few minutes, heart arrest can be reversed in most cases to normal cardiac rhythm and effective circulation.'),
  'D006324': NtO(MH='Heart Arrest, Induced', MS='A procedure to stop the contraction of MYOCARDIUM during HEART SURGERY. It is usually achieved with the use of chemicals (CARDIOPLEGIC SOLUTIONS) or cold temperature (such as chilled perfusate).'),
  'D006325': NtO(MH='Heart Atria', MS='The chambers of the heart, to which the BLOOD returns from the circulation.'),
  'D006326': NtO(MH='Heart Auscultation', MS='Act of listening for sounds within the heart.'),
  'D006327': NtO(MH='Heart Block', MS='Impaired conduction of cardiac impulse that can occur anywhere along the conduction pathway, such as between the SINOATRIAL NODE and the right atrium (SA block) or between atria and ventricles (AV block). Heart blocks can be classified by the duration, frequency, or completeness of conduction block. Reversibility depends on the degree of structural or functional defects.'),
  'D006328': NtO(MH='Cardiac Catheterization', MS='Procedures in which placement of CARDIAC CATHETERS is performed for therapeutic or diagnostic procedures.'),
  'D006329': NtO(MH='Heart Conduction System', MS='An impulse-conducting system composed of modified cardiac muscle, having the power of spontaneous rhythmicity and conduction more highly developed than the rest of the heart.'),
  'D006330': NtO(MH='Heart Defects, Congenital', MS='Developmental abnormalities involving structures of the heart. These defects are present at birth but may be discovered later in life.'),
  'D006331': NtO(MH='Heart Diseases', MS='Pathological conditions involving the HEART including its structural and functional abnormalities.'),
  'D006332': NtO(MH='Cardiomegaly', MS='Enlargement of the HEART, usually indicated by a cardiothoracic ratio above 0.50. Heart enlargement may involve the right, the left, or both HEART VENTRICLES or HEART ATRIA. Cardiomegaly is a nonspecific symptom seen in patients with chronic systolic heart failure (HEART FAILURE) or several forms of CARDIOMYOPATHIES.'),
  'D006333': NtO(MH='Heart Failure', MS='A heterogeneous condition in which the heart is unable to pump out sufficient blood to meet the metabolic need of the body. Heart failure can be caused by structural defects, functional abnormalities (VENTRICULAR DYSFUNCTION), or a sudden overload beyond its capacity. Chronic heart failure is more common than acute heart failure which results from sudden insult to cardiac function, such as MYOCARDIAL INFARCTION.'),
  'D006334': NtO(MH='Heart Function Tests', MS='Examinations used to diagnose and treat heart conditions.'),
  'D006335': NtO(MH='Heart Injuries', MS='General or unspecified injuries to the heart.'),
  'D006336': NtO(MH='Heart Massage', MS='Rhythmic compression of the heart by pressure applied manually over the sternum (closed heart massage) or directly to the heart through an opening in the chest wall (open heart massage). It is done to reinstate and maintain circulation. (Dorland, 28th ed)'),
  'D006337': NtO(MH='Heart Murmurs', MS='Heart sounds caused by vibrations resulting from the flow of blood through the heart. Heart murmurs can be examined by HEART AUSCULTATION, and analyzed by their intensity (6 grades), duration, timing (systolic, diastolic, or continuous), location, transmission, and quality (musical, vibratory, blowing, etc).'),
  'D006338': NtO(MH='Heart Neoplasms', MS='Tumors in any part of the heart. They include primary cardiac tumors and metastatic tumors to the heart. Their interference with normal cardiac functions can cause a wide variety of symptoms including HEART FAILURE; CARDIAC ARRHYTHMIAS; or EMBOLISM.'),
  'D006339': NtO(MH='Heart Rate', MS='The number of times the HEART VENTRICLES contract per unit of time, usually per minute.'),
  'D006340': NtO(MH='Heart Rate, Fetal', MS='The heart rate of the FETUS. The normal range at term is between 120 and 160 beats per minute.'),
  'D006341': NtO(MH='Heart Rupture', MS='Disease-related laceration or tearing of tissues of the heart, including the free-wall MYOCARDIUM; HEART SEPTUM; PAPILLARY MUSCLES; CHORDAE TENDINEAE; and any of the HEART VALVES. Pathological rupture usually results from myocardial infarction (HEART RUPTURE, POST-INFARCTION).'),
  'D006342': NtO(MH='Heart Rupture, Post-Infarction', MS='Laceration or tearing of cardiac tissues appearing after MYOCARDIAL INFARCTION.'),
  'D006343': NtO(MH='Heart Septal Defects', MS='Abnormalities in any part of the HEART SEPTUM resulting in abnormal communication between the left and the right chambers of the heart. The abnormal blood flow inside the heart may be caused by defects in the ATRIAL SEPTUM, the VENTRICULAR SEPTUM, or both.'),
  'D006344': NtO(MH='Heart Septal Defects, Atrial', MS='Developmental abnormalities in any portion of the ATRIAL SEPTUM resulting in abnormal communications between the two upper chambers of the heart. Classification of atrial septal defects is based on location of the communication and types of incomplete fusion of atrial septa with the ENDOCARDIAL CUSHIONS in the fetal heart. They include ostium primum, ostium secundum, sinus venosus, and coronary sinus defects.'),
  'D006345': NtO(MH='Heart Septal Defects, Ventricular', MS='Developmental abnormalities in any portion of the VENTRICULAR SEPTUM resulting in abnormal communications between the two lower chambers of the heart. Classification of ventricular septal defects is based on location of the communication, such as perimembranous, inlet, outlet (infundibular), central muscular, marginal muscular, or apical muscular defect.'),
  'D006346': NtO(MH='Heart Septum', MS='This structure includes the thin muscular atrial septum between the two HEART ATRIA, and the thick muscular ventricular septum between the two HEART VENTRICLES.'),
  'D006347': NtO(MH='Heart Sounds', MS='The sounds heard over the cardiac region produced by the functioning of the heart. There are four distinct sounds: the first occurs at the beginning of SYSTOLE and is heard as a "lubb" sound; the second is produced by the closing of the AORTIC VALVE and PULMONARY VALVE and is heard as a "dupp" sound; the third is produced by vibrations of the ventricular walls when suddenly distended by the rush of blood from the HEART ATRIA; and the fourth is produced by atrial contraction and ventricular filling.'),
  'D006348': NtO(MH='Cardiac Surgical Procedures', MS='Surgery performed on the heart.'),
  'D006349': NtO(MH='Heart Valve Diseases', MS='Pathological conditions involving any of the various HEART VALVES and the associated structures (PAPILLARY MUSCLES and CHORDAE TENDINEAE).'),
  'D006350': NtO(MH='Heart Valve Prosthesis', MS='A device that substitutes for a heart valve. It may be composed of biological material (BIOPROSTHESIS) and/or synthetic material.'),
  'D006351': NtO(MH='Heart Valves', MS='Flaps of tissue that prevent regurgitation of BLOOD from the HEART VENTRICLES to the HEART ATRIA or from the PULMONARY ARTERIES or AORTA to the ventricles.'),
  'D006352': NtO(MH='Heart Ventricles', MS='The lower right and left chambers of the heart. The right ventricle pumps venous BLOOD into the LUNGS and the left ventricle pumps oxygenated blood into the systemic arterial circulation.'),
  'D006353': NtO(MH='Heart-Assist Devices', MS='Small pumps, often implantable, designed for temporarily assisting the heart, usually the LEFT VENTRICLE, to pump blood. They consist of a pumping chamber and a power source, which may be partially or totally external to the body and activated by electromagnetic motors.'),
  'D006354': NtO(MH='Heart, Artificial', MS='A pumping mechanism that duplicates the output, rate, and blood pressure of the natural heart. It may replace the function of the entire heart or a portion of it, and may be an intracorporeal, extracorporeal, or paracorporeal heart. (Dorland, 28th ed)'),
  'D006355': NtO(MH='Heart-Lung Machine', MS="Apparatus that provides mechanical circulatory support during open-heart surgery, by passing the heart to facilitate surgery on the organ. The basic function of the machine is to oxygenate the body's venous supply of blood and then pump it back into the arterial system. The machine also provides intracardiac suction, filtration, and temperature control. Some of the more important components of these machines include pumps, oxygenators, temperature regulators, and filters. (UMDNS, 1999)"),
  'D006356': NtO(MH='Heartburn', MS='Substernal pain or burning sensation, usually associated with regurgitation of gastric juice into the esophagus.'),
  'D006357': NtO(MH='Heartwater Disease', MS='A tick-borne septicemic disease of domestic and wild ruminants caused by EHRLICHIA RUMINANTIUM.'),
  'D006358': NtO(MH='Hot Temperature', MS='Presence of warmth or heat or a temperature notably higher than an accustomed norm.'),
  'D006359': NtO(MH='Heat Exhaustion', MS='A clinical syndrome caused by heat stress, such as over-exertion in a hot environment or excessive exposure to sun. It is characterized by SWEATING, water (volume) depletion, salt depletion, cool clammy skin, NAUSEA, and HEADACHE.'),
  'D006360': NtO(MH='Heat-Shock Proteins', MS='Proteins which are synthesized in eukaryotic organisms and bacteria in response to hyperthermia and other environmental stresses. They increase thermal tolerance and perform functions essential to cell survival under these conditions.'),
  'D006361': NtO(MH='Heating', MS='The application of heat to raise the temperature of the environment, ambient or local, or the systems for accomplishing this effect. It is distinguished from HEAT, the physical property and principle of physics.'),
  'D006362': NtO(MH='Heavy Chain Disease', MS='A disorder of immunoglobulin synthesis in which large quantities of abnormal heavy chains are excreted in the urine. The amino acid sequences of the N-(amino-) terminal regions of these chains are normal, but they have a deletion extending from part of the variable domain through the first domain of the constant region, so that they cannot form cross-links to the light chains. The defect arises through faulty coupling of the variable (V) and constant (C) region genes.'),
  'D006363': NtO(MH='Hebrides', MS='A group of islands in the Atlantic Ocean west of Scotland, comprising the Outer Hebrides and the Inner Hebrides.'),
  'D006364': NtO(MH='Hedgehogs', MS='The family Erinaceidae, in the order INSECTIVORA. Most are true hedgehogs possessing a coat of spines and a very short tail. Those members of the family found in Southeast Asia (moonrats or gymnures) have normal body hair and a long tail.'),
  'D006365': NtO(MH='Heel', MS='The back (or posterior) of the foot in PRIMATES, found behind the ankle and distal to the toes.'),
  'D006366': NtO(MH='Heinz Bodies', MS='Abnormal intracellular inclusions, composed of denatured hemoglobin, found on the membrane of red blood cells. They are seen in thalassemias, enzymopathies, hemoglobinopathies, and after splenectomy.'),
  'D006367': NtO(MH='HeLa Cells', MS='The first continuously cultured human malignant CELL LINE, derived from the cervical carcinoma of Henrietta Lacks. These cells are used for, among other things, VIRUS CULTIVATION and PRECLINICAL DRUG EVALUATION assays.'),
  'D006368': NtO(MH='Helianthus', MS='A genus herbs of the Asteraceae family. The SEEDS yield oil and are used as food and animal feed; the roots of Helianthus tuberosum (Jerusalem artichoke) are edible.'),
  'D006369': NtO(MH='Heligmosomatoidea', MS='A superfamily of parasitic nematodes which were formerly considered a part of TRICHOSTRONGYLOIDEA. It includes the following genera: Heligmosomum, NEMATOSPIROIDES, and NIPPOSTRONGYLUS. The hosts are rodents.'),
  'D006370': NtO(MH='Heliotherapy', MS='The treatment of disease by exposing the body to SUNLIGHT, a therapeutic use of SUNBATHING.'),
  'D006371': NtO(MH='Helium', MS='A noble gas with the atomic symbol He, atomic number 2, and atomic weight 4.003. It is a colorless, odorless, tasteless gas that is not combustible and does not support combustion. It was first detected in the sun and is now obtained from natural gas. Medically it is used as a diluent for other gases, being especially useful with oxygen in the treatment of certain cases of respiratory obstruction, and as a vehicle for general anesthetics.'),
  'D006372': NtO(MH='Helix, Snails', MS='A genus of chiefly Eurasian and African land snails including the principal edible snails as well as several pests of cultivated plants.'),
  'D006373': NtO(MH='Helminthiasis', MS='Infestation with parasitic worms of the helminth class.'),
  'D006374': NtO(MH='Helminthiasis, Animal', MS='Infestation of animals with parasitic worms of the helminth class. The infestation may be experimental or veterinary.'),
  'D006375': NtO(MH='Helminthosporium', MS='A mitosporic fungal genus including both saprophytes and plant parasites.'),
  'D006376': NtO(MH='Helminths', MS='Commonly known as parasitic worms, this group includes the ACANTHOCEPHALA; NEMATODA; and PLATYHELMINTHS. Some authors consider certain species of LEECHES that can become temporarily parasitic as helminths.'),
  'D006377': NtO(MH='T-Lymphocytes, Helper-Inducer', MS='Subpopulation of CD4+ lymphocytes that cooperate with other lymphocytes (either T or B) to initiate a variety of immune functions. For example, helper-inducer T-cells cooperate with B-cells to produce antibodies to thymus-dependent antigens and with other subpopulations of T-cells to initiate a variety of cell-mediated immune functions.'),
  'D006378': NtO(MH='Helper Viruses', MS='Viruses which enable defective viruses to replicate or to form a protein coat by complementing the missing gene function of the defective (satellite) virus. Helper and satellite may be of the same or different genus.'),
  'D006379': NtO(MH='Helping Behavior', MS='Behaviors associated with the giving of assistance or aid to individuals.'),
  'D006380': NtO(MH='Helplessness, Learned', MS="Learned expectation that one's responses are independent of reward and, hence, do not predict or control the occurrence of rewards. Learned helplessness derives from a history, experimentally induced or naturally occurring, of having received punishment/aversive stimulation regardless of responses made. Such circumstances result in an impaired ability to learn. Used for human or animal populations. (APA, Thesaurus of Psychological Index Terms, 1994)"),
  'D006381': NtO(MH='Helsinki Declaration', MS='An international agreement of the World Medical Association which offers guidelines for conducting experiments using human subjects. It was adopted in 1962 and revised by the 18th World Medical Assembly at Helsinki, Finland in 1964. Subsequent revisions were made in 1975, 1983, 1989, and 1996. (From Encyclopedia of Bioethics, rev ed, 1995)'),
  'D006382': NtO(MH='Hemadsorption', MS='A phenomenon manifested by an agent or substance adhering to or being adsorbed on the surface of a red blood cell, as tuberculin can be adsorbed on red blood cells under certain conditions. (Stedman, 25th ed)'),
  'D006383': NtO(MH='Hemadsorption Inhibition Tests', MS='Serological tests that measure anti-hemadsorption agents such as antiviral ANTIBODIES that block VIRAL HEMAGGLUNININS from adhering to the surface of red blood cells.'),
  'D006384': NtO(MH='Hemagglutination', MS='The aggregation of ERYTHROCYTES by AGGLUTININS, including antibodies, lectins, and viral proteins (HEMAGGLUTINATION, VIRAL).'),
  'D006385': NtO(MH='Hemagglutination Inhibition Tests', MS='Serologic tests in which a known quantity of antigen is added to the serum prior to the addition of a red cell suspension. Reaction result is expressed as the smallest amount of antigen which causes complete inhibition of hemagglutination.'),
  'D006386': NtO(MH='Hemagglutination Tests', MS='Sensitive tests to measure certain antigens, antibodies, or viruses, using their ability to agglutinate certain erythrocytes. (From Stedman, 26th ed)'),
  'D006387': NtO(MH='Hemagglutination, Viral', MS='Agglutination of ERYTHROCYTES by a virus.'),
  'D006388': NtO(MH='Hemagglutinins', MS='Agents that cause agglutination of red blood cells. They include antibodies, blood group antigens, lectins, autoimmune factors, bacterial, viral, or parasitic blood agglutinins, etc.'),
  'D006389': NtO(MH='Hemagglutinins, Viral', MS='Specific hemagglutinin subtypes encoded by VIRUSES.'),
  'D006390': NtO(MH='Hemangioendothelioma', MS='A neoplasm derived from blood vessels, characterized by numerous prominent endothelial cells that occur singly, in aggregates, and as the lining of congeries of vascular tubes or channels. Hemangioendotheliomas are relatively rare and are of intermediate malignancy (between benign hemangiomas and conventional angiosarcomas). They affect men and women about equally and rarely develop in childhood. (From Stedman, 25th ed; Holland et al., Cancer Medicine, 3d ed, p1866)'),
  'D006391': NtO(MH='Hemangioma', MS='A vascular anomaly due to proliferation of BLOOD VESSELS that forms a tumor-like mass. The common types involve CAPILLARIES and VEINS. It can occur anywhere in the body but is most frequently noticed in the SKIN and SUBCUTANEOUS TISSUE. (from Stedman, 27th ed, 2000)'),
  'D006392': NtO(MH='Hemangioma, Cavernous', MS='A vascular anomaly that is a collection of tortuous BLOOD VESSELS and connective tissue. This tumor-like mass with the large vascular space is filled with blood and usually appears as a strawberry-like lesion in the subcutaneous areas of the face, extremities, or other regions of the body including the central nervous system.'),
  'D006393': NtO(MH='Hemangiopericytoma', MS='A tumor composed of spindle cells with a rich vascular network, which apparently arises from pericytes, cells of smooth muscle origin that lie around small vessels. Benign and malignant hemangiopericytomas exist, and the rarity of these lesions has led to considerable confusion in distinguishing between benign and malignant variants. (From Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1364)'),
  'D006394': NtO(MH='Hemangiosarcoma', MS='A rare malignant neoplasm characterized by rapidly proliferating, extensively infiltrating, anaplastic cells derived from blood vessels and lining irregular blood-filled or lumpy spaces. (Stedman, 25th ed)'),
  'D006395': NtO(MH='Hemarthrosis', MS='Bleeding into the joints. It may arise from trauma or spontaneously in patients with hemophilia.'),
  'D006396': NtO(MH='Hematemesis', MS='Vomiting of blood that is either fresh bright red, or older "coffee-ground" in character. It generally indicates bleeding of the UPPER GASTROINTESTINAL TRACT.'),
  'D006397': NtO(MH='Hematinics', MS='Agents which improve the quality of the blood, increasing the hemoglobin level and the number of erythrocytes. They are used in the treatment of anemias.'),
  'D006398': NtO(MH='Hematocele', MS='Hemorrhage into a canal or cavity of the body, such as the space covered by the serous membrane (tunica vaginalis) around the TESTIS leading to testicular hematocele or scrotal hematocele.'),
  'D006399': NtO(MH='Hematocolpos', MS='A blood-filled VAGINA that is obstructed.'),
  'D006400': NtO(MH='Hematocrit', MS='The volume of packed RED BLOOD CELLS in a blood specimen. The volume is measured by centrifugation in a tube with graduated markings, or with automated blood cell counters. It is an indicator of erythrocyte status in disease. For example, ANEMIA shows a low value; POLYCYTHEMIA, a high value.'),
  'D006401': NtO(MH='Hematologic Agents', MS='Drugs that act on blood and blood-forming organs and those that affect the hemostatic system.'),
  'D006402': NtO(MH='Hematologic Diseases', MS='Disorders of the blood and blood forming tissues.'),
  'D006403': NtO(MH='Hematologic Tests', MS='Tests used in the analysis of the hemic system.'),
  'D006405': NtO(MH='Hematology', MS='A subspecialty of internal medicine concerned with morphology, physiology, and pathology of the blood and blood-forming tissues.'),
  'D006406': NtO(MH='Hematoma', MS='A collection of blood outside the BLOOD VESSELS. Hematoma can be localized in an organ, space, or tissue.'),
  'D006407': NtO(MH='Hematoma, Epidural, Cranial', MS='Accumulation of blood in the EPIDURAL SPACE between the SKULL and the DURA MATER, often as a result of bleeding from the MENINGEAL ARTERIES associated with a temporal or parietal bone fracture. Epidural hematoma tends to expand rapidly, compressing the dura and underlying brain. Clinical features may include HEADACHE; VOMITING; HEMIPARESIS; and impaired mental function.'),
  'D006408': NtO(MH='Hematoma, Subdural', MS='Accumulation of blood in the SUBDURAL SPACE between the DURA MATER and the arachnoidal layer of the MENINGES. This condition primarily occurs over the surface of a CEREBRAL HEMISPHERE, but may develop in the spinal canal (HEMATOMA, SUBDURAL, SPINAL). Subdural hematoma can be classified as the acute or the chronic form, with immediate or delayed symptom onset, respectively. Symptoms may include loss of consciousness, severe HEADACHE, and deteriorating mental status.'),
  'D006409': NtO(MH='Hematometra', MS='Blood-filled UTERUS.'),
  'D006410': NtO(MH='Hematopoiesis', MS='The development and formation of various types of BLOOD CELLS. Hematopoiesis can take place in the BONE MARROW (medullary) or outside the bone marrow (HEMATOPOIESIS, EXTRAMEDULLARY).'),
  'D006411': NtO(MH='Hematopoiesis, Extramedullary', MS='The formation and development of blood cells outside the BONE MARROW, as in the SPLEEN; LIVER; or LYMPH NODES.'),
  'D006412': NtO(MH='Hematopoietic Stem Cells', MS='Progenitor cells from which all blood cells derived. They are found primarily in the bone marrow and also in small numbers in the peripheral blood.'),
  'D006413': NtO(MH='Hematopoietic System', MS='The blood-making organs and tissues, principally the bone marrow and lymph nodes.'),
  'D006414': NtO(MH='Hematoporphyrin Photoradiation', MS='Photochemotherapy using visible light, usually red, topically or delivered locally by fiberoptic probe to tissues sensitized with hematoporphyrins.'),
  'D006415': NtO(MH='Hematoporphyrins', MS='Iron-free derivatives of heme with 4 methyl groups, 2 hydroxyethyl groups and 2 propionic acid groups attached to the pyrrole rings. Some of these PHOTOSENSITIZING AGENTS are used in the PHOTOTHERAPY of malignant NEOPLASMS.'),
  'D006416': NtO(MH='Hematoxylin', MS='A dye obtained from the heartwood of logwood (Haematoxylon campechianum Linn., Leguminosae) used as a stain in microscopy and in the manufacture of ink.'),
  'D006417': NtO(MH='Hematuria', MS='Presence of blood in the urine.'),
  'D006418': NtO(MH='Heme', MS='The color-furnishing portion of hemoglobin. It is found free in tissues and as the prosthetic group in many hemeproteins.'),
  'D006419': NtO(MH='Heme Oxygenase (Decyclizing)', MS='A mixed function oxidase enzyme which during hemoglobin catabolism catalyzes the degradation of heme to ferrous iron, carbon monoxide and biliverdin in the presence of molecular oxygen and reduced NADPH. The enzyme is induced by metals, particularly cobalt.'),
  'D006420': NtO(MH='Hemeproteins', MS='Proteins that contain an iron-porphyrin, or heme, prosthetic group resembling that of hemoglobin. (From Lehninger, Principles of Biochemistry, 1982, p480)'),
  'D006422': NtO(MH='Hemerythrin', MS='A non-heme iron protein consisting of eight apparently identical subunits each containing 2 iron atoms. It binds one molecule of oxygen per pair of iron atoms and functions as a respiratory protein.'),
  'D006423': NtO(MH='Hemianopsia', MS='Partial or complete loss of vision in one half of the visual field(s) of one or both eyes. Subtypes include altitudinal hemianopsia, characterized by a visual defect above or below the horizontal meridian of the visual field. Homonymous hemianopsia refers to a visual defect that affects both eyes equally, and occurs either to the left or right of the midline of the visual field. Binasal hemianopsia consists of loss of vision in the nasal hemifields of both eyes. Bitemporal hemianopsia is the bilateral loss of vision in the temporal fields. Quadrantanopsia refers to loss of vision in one quarter of the visual field in one or both eyes.'),
  'D006424': NtO(MH='Hemic and Immune Systems', MS='Organs involved in the production of BLOOD, including the cellular and the molecular components essential in providing defense against foreign organisms or substances.'),
  'D006425': NtO(MH='Hemic and Lymphatic Diseases', MS='Hematologic diseases and diseases of the lymphatic system collectively. Hemic diseases include disorders involving the formed elements (e.g., ERYTHROCYTE AGGREGATION, INTRAVASCULAR) and chemical components (e.g., BLOOD PROTEIN DISORDERS); lymphatic diseases include disorders relating to lymph, lymph nodes, and lymphocytes.'),
  'D006426': NtO(MH='Hemicholinium 3', MS='A potent inhibitor of the high affinity uptake system for CHOLINE. It has less effect on the low affinity uptake system. Since choline is one of the components of ACETYLCHOLINE, treatment with hemicholinium can deplete acetylcholine from cholinergic terminals. Hemicholinium 3 is commonly used as a research tool in animal and in vitro experiments.'),
  'D006427': NtO(MH='Hemin', MS='Chloro(7,12-diethenyl-3,8,13,17-tetramethyl-21H,23H-porphine-2,18-dipropanoato(4-)-N(21),N(22),N(23),N(24)) ferrate(2-) dihydrogen.'),
  'D006428': NtO(MH='Hemipelvectomy', MS='Amputation of a lower limb through the sacroiliac joint.'),
  'D006429': NtO(MH='Hemiplegia', MS='Severe or complete loss of motor function on one side of the body. This condition is usually caused by BRAIN DISEASES that are localized to the cerebral hemisphere opposite to the side of weakness. Less frequently, BRAIN STEM lesions; cervical SPINAL CORD DISEASES; PERIPHERAL NERVOUS SYSTEM DISEASES; and other conditions may manifest as hemiplegia. The term hemiparesis (see PARESIS) refers to mild to moderate weakness involving one side of the body.'),
  'D006430': NtO(MH='Hemiptera', MS='A large order of insects characterized by having the mouth parts adapted to piercing or sucking. It is comprised of four suborders: HETEROPTERA, Auchenorrhyncha, Sternorrhyncha, and Coleorrhyncha.'),
  'D006431': NtO(MH='Hemobilia', MS='Hemorrhage in or through the BILIARY TRACT due to trauma, inflammation, CHOLELITHIASIS, vascular disease, or neoplasms.'),
  'D006432': NtO(MH='Hemochromatosis', MS="A disorder of iron metabolism characterized by a triad of HEMOSIDEROSIS; LIVER CIRRHOSIS; and DIABETES MELLITUS. It is caused by massive iron deposits in parenchymal cells that may develop after a prolonged increase of iron absorption. (Jablonski's Dictionary of Syndromes & Eponymic Diseases, 2d ed)"),
  'D006433': NtO(MH='Hemocyanins', MS='Metalloproteins that function as oxygen transport proteins in the HEMOLYMPH of MOLLUSKS and ARTHROPODS. They are characterized by two copper atoms, coordinated with HISTIDINE residues, that reversibly bind a single oxygen molecule; they do not contain HEME groups.'),
  'D006434': NtO(MH='Hemocytes', MS='Any blood or formed element especially in invertebrates.'),
  'D006435': NtO(MH='Renal Dialysis', MS='Therapy for the insufficient cleansing of the BLOOD by the kidneys based on dialysis and including hemodialysis, PERITONEAL DIALYSIS, and HEMODIAFILTRATION.'),
  'D006436': NtO(MH='Hemodialysis Units, Hospital', MS='Hospital units in which care is provided the hemodialysis patient. This includes hemodialysis centers in hospitals.'),
  'D006437': NtO(MH='Hemodialysis, Home', MS='Long-term maintenance hemodialysis in the home.'),
  'D006438': NtO(MH='Hemodilution', MS='Reduction of blood viscosity usually by the addition of cell free solutions. Used clinically (1) in states of impaired microcirculation, (2) for replacement of intraoperative blood loss without homologous blood transfusion, and (3) in cardiopulmonary bypass and hypothermia.'),
  'D006439': NtO(MH='Hemodynamics', MS='The movement and the forces involved in the movement of the blood through the CARDIOVASCULAR SYSTEM.'),
  'D006440': NtO(MH='Hemofiltration', MS='Extracorporeal ULTRAFILTRATION technique without HEMODIALYSIS for treatment of fluid overload and electrolyte disturbances affecting renal, cardiac, or pulmonary function.'),
  'D006441': NtO(MH='Hemoglobin A', MS='Normal adult human hemoglobin. The globin moiety consists of two alpha and two beta chains.'),
  'D006442': NtO(MH='Glycated Hemoglobin A', MS='Products of non-enzymatic reactions between GLUCOSE and HEMOGLOBIN A, occurring as a minor fraction of the hemoglobin components of human erythrocytes. Hemoglobin A1c is hemoglobin A with glucose covalently bound to the terminal VALINE of the beta chain. Glycated hemoglobin A is used as an index of the average blood sugar level over a lifetime of erythrocytes.'),
  'D006443': NtO(MH='Hemoglobin A2', MS='An adult hemoglobin component normally present in hemolysates from human erythrocytes in concentrations of about 3%. The hemoglobin is composed of two alpha chains and two delta chains. The percentage of HbA2 varies in some hematologic disorders, but is about double in beta-thalassemia.'),
  'D006444': NtO(MH='Hemoglobin C', MS='A commonly occurring abnormal hemoglobin in which lysine replaces a glutamic acid residue at the sixth position of the beta chains. It results in reduced plasticity of erythrocytes.'),
  'D006445': NtO(MH='Hemoglobin C Disease', MS='A disease characterized by compensated hemolysis with a normal hemoglobin level or a mild to moderate anemia. There may be intermittent abdominal discomfort, splenomegaly, and slight jaundice.'),
  'D006446': NtO(MH='Hemoglobin E', MS='An abnormal hemoglobin that results from the substitution of lysine for glutamic acid at position 26 of the beta chain. It is most frequently observed in southeast Asian populations.'),
  'D006447': NtO(MH='Hemoglobin H', MS='An abnormal hemoglobin composed of four beta chains. It is caused by the reduced synthesis of the alpha chain. This abnormality results in ALPHA-THALASSEMIA.'),
  'D006448': NtO(MH='Hemoglobin J', MS='A group of abnormal hemoglobins with similar electrophoretic characteristics. They have faster electrophoretic mobility and different amino acid substitutions in either the alpha or beta chains than normal adult hemoglobin. Some of the variants produce hematologic abnormalities, others result in no clinical disorders.'),
  'D006449': NtO(MH='Hemoglobin M', MS='A group of abnormal hemoglobins in which amino acid substitutions take place in either the alpha or beta chains but near the heme iron. This results in facilitated oxidation of the hemoglobin to yield excess methemoglobin which leads to cyanosis.'),
  'D006450': NtO(MH='Hemoglobin SC Disease', MS='One of the sickle cell disorders characterized by the presence of both hemoglobin S and hemoglobin C. It is similar to, but less severe than sickle cell anemia.'),
  'D006451': NtO(MH='Hemoglobin, Sickle', MS='An abnormal hemoglobin resulting from the substitution of valine for glutamic acid at position 6 of the beta chain of the globin moiety. The heterozygous state results in sickle cell trait, the homozygous in sickle cell anemia.'),
  'D006452': NtO(MH='Hemoglobinometry', MS='Measurement of hemoglobin concentration in blood.'),
  'D006453': NtO(MH='Hemoglobinopathies', MS='A group of inherited disorders characterized by structural alterations within the hemoglobin molecule.'),
  'D006454': NtO(MH='Hemoglobins', MS='The oxygen-carrying proteins of ERYTHROCYTES. They are found in all vertebrates and some invertebrates. The number of globin subunits in the hemoglobin quaternary structure differs between species. Structures range from monomeric to a variety of multimeric arrangements.'),
  'D006455': NtO(MH='Hemoglobins, Abnormal', MS='Hemoglobins characterized by structural alterations within the molecule. The alteration can be either absence, addition or substitution of one or more amino acids in the globin part of the molecule at selected positions in the polypeptide chains.'),
  'D006456': NtO(MH='Hemoglobinuria', MS='The presence of free HEMOGLOBIN in the URINE, indicating hemolysis of ERYTHROCYTES within the vascular system. After saturating the hemoglobin-binding proteins (HAPTOGLOBINS), free hemoglobin begins to appear in the urine.'),
  'D006457': NtO(MH='Hemoglobinuria, Paroxysmal', MS='A condition characterized by the recurrence of HEMOGLOBINURIA caused by intravascular HEMOLYSIS. In cases occurring upon cold exposure (paroxysmal cold hemoglobinuria), usually after infections, there is a circulating antibody which is also a cold hemolysin. In cases occurring during or after sleep (paroxysmal nocturnal hemoglobinuria), the clonal hematopoietic stem cells exhibit a global deficiency of cell membrane proteins.'),
  'D006458': NtO(MH='Hemolymph', MS='The blood/lymphlike nutrient fluid of some invertebrates.'),
  'D006459': NtO(MH='Hemolysin Factors', MS='Plasmids controlling the synthesis of hemolysin by bacteria.'),
  'D006460': NtO(MH='Hemolysin Proteins', MS='Proteins from BACTERIA and FUNGI that are soluble enough to be secreted to target ERYTHROCYTES and insert into the membrane to form beta-barrel pores. Biosynthesis may be regulated by HEMOLYSIN FACTORS.'),
  'D006461': NtO(MH='Hemolysis', MS='The destruction of ERYTHROCYTES by many different causal agents such as antibodies, bacteria, chemicals, temperature, and changes in tonicity.'),
  'D006462': NtO(MH='Hemolytic Plaque Technique', MS='A method to identify and enumerate cells that are synthesizing ANTIBODIES against ANTIGENS or HAPTENS conjugated to sheep RED BLOOD CELLS. The sheep red blood cells surrounding cells secreting antibody are lysed by added COMPLEMENT producing a clear zone of HEMOLYSIS. (From Illustrated Dictionary of Immunology, 3rd ed)'),
  'D006463': NtO(MH='Hemolytic-Uremic Syndrome', MS='A syndrome that is associated with microvascular diseases of the KIDNEY, such as RENAL CORTICAL NECROSIS. It is characterized by hemolytic anemia (ANEMIA, HEMOLYTIC); THROMBOCYTOPENIA; and ACUTE RENAL FAILURE.'),
  'D006464': NtO(MH='Hemoperfusion', MS='Removal of toxins or metabolites from the circulation by the passing of blood, within a suitable extracorporeal circuit, over semipermeable microcapsules containing adsorbents (e.g., activated charcoal) or enzymes, other enzyme preparations (e.g., gel-entrapped microsomes, membrane-free enzymes bound to artificial carriers), or other adsorbents (e.g., various resins, albumin-conjugated agarose).'),
  'D006465': NtO(MH='Hemoperitoneum', MS='Accumulations of blood in the PERITONEAL CAVITY due to internal HEMORRHAGE.'),
  'D006466': NtO(MH='Hemopexin', MS=None),
  'D006467': NtO(MH='Hemophilia A', MS='The classic hemophilia resulting from a deficiency of factor VIII. It is an inherited disorder of blood coagulation characterized by a permanent tendency to hemorrhage.'),
  'D006468': NtO(MH='Hemopneumothorax', MS='Collection of air and blood in the pleural cavity.'),
  'D006469': NtO(MH='Hemoptysis', MS='Expectoration or spitting of blood originating from any part of the RESPIRATORY TRACT, usually from hemorrhage in the lung parenchyma (PULMONARY ALVEOLI) and the BRONCHIAL ARTERIES.'),
  'D006470': NtO(MH='Hemorrhage', MS='Bleeding or escape of blood from a vessel.'),
  'D006471': NtO(MH='Gastrointestinal Hemorrhage', MS='Bleeding in any segment of the GASTROINTESTINAL TRACT from ESOPHAGUS to RECTUM.'),
  'D006472': NtO(MH='Oral Hemorrhage', MS='Bleeding from the blood vessels of the mouth, which may occur as a result of injuries to the mouth, accidents in oral surgery, or diseases of the gums.'),
  'D006473': NtO(MH='Postpartum Hemorrhage', MS='Excess blood loss from uterine bleeding associated with OBSTETRIC LABOR or CHILDBIRTH. It is defined as blood loss greater than 500 ml or of the amount that adversely affects the maternal physiology, such as BLOOD PRESSURE and HEMATOCRIT. Postpartum hemorrhage is divided into two categories, immediate (within first 24 hours after birth) or delayed (after 24 hours postpartum).'),
  'D006474': NtO(MH='Hemorrhagic Disorders', MS='Spontaneous or near spontaneous bleeding caused by a defect in clotting mechanisms (BLOOD COAGULATION DISORDERS) or another abnormality causing a structural flaw in the blood vessels (HEMOSTATIC DISORDERS).'),
  'D006475': NtO(MH='Vitamin K Deficiency Bleeding', MS='Hemorrhage caused by vitamin K deficiency.'),
  'D006476': NtO(MH='Hantavirus', MS='A genus of the family BUNYAVIRIDAE causing HANTAVIRUS INFECTIONS, first identified during the Korean war. Infection is found primarily in rodents and humans. Transmission does not appear to involve arthropods. HANTAAN VIRUS is the type species.'),
  'D006477': NtO(MH='Arenaviruses, New World', MS='One of two groups of viruses in the ARENAVIRUS genus and considered part of the New World complex. It includes JUNIN VIRUS; PICHINDE VIRUS; Amapari virus, and Machupo virus among others. They are the cause of human hemorrhagic fevers mostly in Central and South America.'),
  'D006478': NtO(MH='Hemorrhagic Fever, American', MS='Diseases caused by American hemorrhagic fever viruses (ARENAVIRUSES, NEW WORLD).'),
  'D006479': NtO(MH='Hemorrhagic Fever, Crimean', MS='A severe, often fatal disease in humans caused by the Crimean-Congo hemorrhagic fever virus (HEMORRHAGIC FEVER VIRUS, CRIMEAN-CONGO).'),
  'D006480': NtO(MH='Hemorrhagic Fever with Renal Syndrome', MS='An acute febrile disease occurring predominately in Asia. It is characterized by fever, prostration, vomiting, hemorrhagic phenonema, shock, and renal failure. It is caused by any one of several closely related species of the genus Hantavirus. The most severe form is caused by HANTAAN VIRUS whose natural host is the rodent Apodemus agrarius. Milder forms are caused by SEOUL VIRUS and transmitted by the rodents Rattus rattus and R. norvegicus, and the PUUMALA VIRUS with transmission by Clethrionomys galreolus.'),
  'D006481': NtO(MH='Hemorrhagic Fever, Omsk', MS='Infection with the Omsk hemorrhagic fever virus, a Flavivirus.'),
  'D006482': NtO(MH='Hemorrhagic Fevers, Viral', MS='A group of viral diseases of diverse etiology but having many similar clinical characteristics; increased capillary permeability, leukopenia, and thrombocytopenia are common to all. Hemorrhagic fevers are characterized by sudden onset, fever, headache, generalized myalgia, backache, conjunctivitis, and severe prostration, followed by various hemorrhagic symptoms. Hemorrhagic fever with kidney involvement is HEMORRHAGIC FEVER WITH RENAL SYNDROME.'),
  'D006483': NtO(MH='Hemorrhagic Septicemia', MS='Any of several bacterial diseases, usually caused by PASTEURELLA MULTOCIDA, marked by the presence of hemorrhagic areas in the subcutaneous tissues, serous membranes, muscles, lymph glands, and throughout the internal organs. The diseases primarily affect animals and rarely humans.'),
  'D006484': NtO(MH='Hemorrhoids', MS='Swollen veins in the lower part of the RECTUM or ANUS. Hemorrhoids can be inside the anus (internal), under the skin around the anus (external), or protruding from inside to outside of the anus. People with hemorrhoids may or may not exhibit symptoms which include bleeding, itching, and pain.'),
  'D006485': NtO(MH='Hemosiderin', MS=None),
  'D006486': NtO(MH='Hemosiderosis', MS='Conditions in which there is a generalized increase in the iron stores of body tissues, particularly of liver and the MONONUCLEAR PHAGOCYTE SYSTEM, without demonstrable tissue damage. The name refers to the presence of stainable iron in the tissue in the form of hemosiderin.'),
  'D006487': NtO(MH='Hemostasis', MS='The process which spontaneously arrests the flow of BLOOD from vessels carrying blood under pressure. It is accomplished by contraction of the vessels, adhesion and aggregation of formed blood elements (eg. ERYTHROCYTE AGGREGATION), and the process of BLOOD COAGULATION.'),
  'D006488': NtO(MH='Hemostasis, Surgical', MS='Control of bleeding during or after surgery.'),
  'D006489': NtO(MH='Hemostatic Techniques', MS='Techniques for controlling bleeding.'),
  'D006490': NtO(MH='Hemostatics', MS='Agents acting to arrest the flow of blood. Absorbable hemostatics arrest bleeding either by the formation of an artificial clot or by providing a mechanical matrix that facilitates clotting when applied directly to the bleeding surface. These agents function more at the capillary level and are not effective at stemming arterial or venous bleeding under any significant intravascular pressure.'),
  'D006491': NtO(MH='Hemothorax', MS='Hemorrhage within the pleural cavity.'),
  'D006492': NtO(MH='Hempa', MS='A chemosterilant agent that is anticipated to be a carcinogen.'),
  'D006493': NtO(MH='Heparin', MS='A highly acidic mucopolysaccharide formed of equal parts of sulfated D-glucosamine and D-glucuronic acid with sulfaminic bridges. The molecular weight ranges from six to twenty thousand. Heparin occurs in and is obtained from liver, lung, mast cells, etc., of vertebrates. Its function is unknown, but it is used to prevent blood clotting in vivo and vitro, in the form of many different salts.'),
  'D006494': NtO(MH='Heparin Antagonists', MS='Coagulant substances inhibiting the anticoagulant action of heparin.'),
  'D006495': NtO(MH='Heparin, Low-Molecular-Weight', MS='Heparin fractions with a molecular weight usually between 4000 and 6000 kD. These low-molecular-weight fractions are effective antithrombotic agents. Their administration reduces the risk of hemorrhage, they have a longer half-life, and their platelet interactions are reduced in comparison to unfractionated heparin. They also provide an effective prophylaxis against postoperative major pulmonary embolism.'),
  'D006496': NtO(MH='Heparinoids', MS='Heparin derivatives. The term has also been used more loosely to include naturally occurring and synthetic highly-sulphated polysaccharides of similar structure. Heparinoid preparations have been used for a wide range of applications including as anticoagulants and anti-inflammatories and they have been claimed to have hypolipidemic properties. (From Martindale, The Extra Pharmacopoeia, 30th, p232)'),
  'D006497': NtO(MH='Heparitin Sulfate', MS='A heteropolysaccharide that is similar in structure to HEPARIN. It accumulates in individuals with MUCOPOLYSACCHARIDOSIS.'),
  'D006498': NtO(MH='Hepatectomy', MS='Excision of all or part of the liver. (Dorland, 28th ed)'),
  'D006499': NtO(MH='Hepatic Artery', MS='A branch of the celiac artery that distributes to the stomach, pancreas, duodenum, liver, gallbladder, and greater omentum.'),
  'D006500': NtO(MH='Hepatic Duct, Common', MS='Predominantly extrahepatic bile duct which is formed by the junction of the right and left hepatic ducts, which are predominantly intrahepatic, and, in turn, joins the cystic duct to form the common bile duct.'),
  'D006501': NtO(MH='Hepatic Encephalopathy', MS='A syndrome characterized by central nervous system dysfunction in association with LIVER FAILURE, including portal-systemic shunts. Clinical features include lethargy and CONFUSION (frequently progressing to COMA); ASTERIXIS; NYSTAGMUS, PATHOLOGIC; brisk oculovestibular reflexes; decorticate and decerebrate posturing; MUSCLE SPASTICITY; and bilateral extensor plantar reflexes (see REFLEX, BABINSKI). ELECTROENCEPHALOGRAPHY may demonstrate triphasic waves. (From Adams et al., Principles of Neurology, 6th ed, pp1117-20; Plum & Posner, Diagnosis of Stupor and Coma, 3rd ed, p222-5)'),
  'D006502': NtO(MH='Budd-Chiari Syndrome', MS='A condition in which the hepatic venous outflow is obstructed anywhere from the small HEPATIC VEINS to the junction of the INFERIOR VENA CAVA and the RIGHT ATRIUM. Usually the blockage is extrahepatic and caused by blood clots (THROMBUS) or fibrous webs. Parenchymal FIBROSIS is uncommon.'),
  'D006503': NtO(MH='Hepatic Veins', MS='Veins which drain the liver.'),
  'D006504': NtO(MH='Hepatic Veno-Occlusive Disease', MS='Liver disease that is caused by injuries to the ENDOTHELIAL CELLS of the vessels and subendothelial EDEMA, but not by THROMBOSIS. Extracellular matrix, rich in FIBRONECTINS, is usually deposited around the HEPATIC VEINS leading to venous outflow occlusion and sinusoidal obstruction.'),
  'D006505': NtO(MH='Hepatitis', MS='INFLAMMATION of the LIVER.'),
  'D006506': NtO(MH='Hepatitis A', MS='INFLAMMATION of the LIVER in humans caused by a member of the HEPATOVIRUS genus, HUMAN HEPATITIS A VIRUS. It can be transmitted through fecal contamination of food or water.'),
  'D006507': NtO(MH='Hepatovirus', MS='A genus of PICORNAVIRIDAE causing infectious hepatitis naturally in humans and experimentally in other primates. It is transmitted through fecal contamination of food or water. HEPATITIS A VIRUS is the type species.'),
  'D006508': NtO(MH='Hepatitis Antibodies', MS='Immunoglobulins raised by any form of viral hepatitis; some of these antibodies are used to diagnose the specific kind of hepatitis.'),
  'D006509': NtO(MH='Hepatitis B', MS='INFLAMMATION of the LIVER in humans caused by a member of the ORTHOHEPADNAVIRUS genus, HEPATITIS B VIRUS. It is primarily transmitted by parenteral exposure, such as transfusion of contaminated blood or blood products, but can also be transmitted via sexual or intimate personal contact.'),
  'D006510': NtO(MH='Hepatitis B Antibodies', MS='Antibodies to the HEPATITIS B ANTIGENS, including antibodies to the surface (Australia) and core of the Dane particle and those to the "e" antigens.'),
  'D006511': NtO(MH='Hepatitis B Antigens', MS='Antigens of the virion of the HEPATITIS B VIRUS or the Dane particle, its surface (HEPATITIS B SURFACE ANTIGENS), core (HEPATITIS B CORE ANTIGENS), and other associated antigens, including the HEPATITIS B E ANTIGENS.'),
  'D006512': NtO(MH='Hepatitis B Core Antigens', MS='The hepatitis B antigen within the core of the Dane particle, the infectious hepatitis virion.'),
  'D006513': NtO(MH='Hepatitis B e Antigens', MS='A closely related group of antigens found in the plasma only during the infective phase of hepatitis B or in virulent chronic hepatitis B, probably indicating active virus replication; there are three subtypes which may exist in a complex with immunoglobulins G.'),
  'D006514': NtO(MH='Hepatitis B Surface Antigens', MS='Those hepatitis B antigens found on the surface of the Dane particle and on the 20 nm spherical and tubular particles. Several subspecificities of the surface antigen are known. These were formerly called the Australia antigen.'),
  'D006515': NtO(MH='Hepatitis B virus', MS='The type species of the genus ORTHOHEPADNAVIRUS which causes human HEPATITIS B and is also apparently a causal agent in human HEPATOCELLULAR CARCINOMA. The Dane particle is an intact hepatitis virion, named after its discoverer. Non-infectious spherical and tubular particles are also seen in the serum.'),
  'D006516': NtO(MH='Adenoviruses, Canine', MS='Species of the genus MASTADENOVIRUS that causes fever, edema, vomiting, and diarrhea in dogs and encephalitis in foxes. Epizootics have also been caused in bears, wolves, coyotes, and skunks. The official species name is Canine adenovirus and it contains two serotypes.'),
  'D006517': NtO(MH='Murine hepatitis virus', MS='A species of the CORONAVIRUS genus causing hepatitis in mice. Four strains have been identified as MHV 1, MHV 2, MHV 3, and MHV 4 (also known as MHV-JHM, which is neurotropic and causes disseminated encephalomyelitis with demyelination as well as focal liver necrosis).'),
  'D006518': NtO(MH='Hepatitis Viruses', MS='Any of the viruses that cause inflammation of the liver. They include both DNA and RNA viruses as well viruses from humans and animals.'),
  'D006519': NtO(MH='Hepatitis, Alcoholic', MS='INFLAMMATION of the LIVER due to ALCOHOL ABUSE. It is characterized by NECROSIS of HEPATOCYTES, infiltration by NEUTROPHILS, and deposit of MALLORY BODIES. Depending on its severity, the inflammatory lesion may be reversible or progress to LIVER CIRRHOSIS.'),
  'D006520': NtO(MH='Hepatitis, Animal', MS='INFLAMMATION of the LIVER in non-human animals.'),
  'D006521': NtO(MH='Hepatitis, Chronic', MS='INFLAMMATION of the LIVER with ongoing hepatocellular injury for 6 months or more, characterized by NECROSIS of HEPATOCYTES and inflammatory cell (LEUKOCYTES) infiltration. Chronic hepatitis can be caused by viruses, medications, autoimmune diseases, and other unknown factors.'),
  'D006522': NtO(MH='Hepatitis, Infectious Canine', MS='A contagious disease caused by canine adenovirus (ADENOVIRUSES, CANINE) infecting the LIVER, the EYE, the KIDNEY, and other organs in dogs, other canids, and bears. Symptoms include FEVER; EDEMA; VOMITING; and DIARRHEA.'),
  'D006524': NtO(MH='Hepatitis, Viral, Animal', MS='INFLAMMATION of the LIVER in animals due to viral infection.'),
  'D006525': NtO(MH='Hepatitis, Viral, Human', MS='INFLAMMATION of the LIVER in humans due to infection by VIRUSES. There are several significant types of human viral hepatitis with infection caused by enteric-transmission (HEPATITIS A; HEPATITIS E) or blood transfusion (HEPATITIS B; HEPATITIS C; and HEPATITIS D).'),
  'D006526': NtO(MH='Hepatitis C', MS='INFLAMMATION of the LIVER in humans caused by HEPATITIS C VIRUS, a single-stranded RNA virus. Its incubation period is 30-90 days. Hepatitis C is transmitted primarily by contaminated blood parenterally, and is often associated with transfusion and intravenous drug abuse. However, in a significant number of cases, the source of hepatitis C infection is unknown.'),
  'D006527': NtO(MH='Hepatolenticular Degeneration', MS='A rare autosomal recessive disease characterized by the deposition of copper in the BRAIN; LIVER; CORNEA; and other organs. It is caused by defects in the ATP7B gene encoding copper-transporting ATPase 2 (EC 3.6.3.4), also known as the Wilson disease protein. The overload of copper inevitably leads to progressive liver and neurological dysfunction such as LIVER CIRRHOSIS; TREMOR; ATAXIA and intellectual deterioration. Hepatic dysfunction may precede neurologic dysfunction by several years.'),
  'D006528': NtO(MH='Carcinoma, Hepatocellular', MS='A primary malignant neoplasm of epithelial liver cells. It ranges from a well-differentiated tumor with EPITHELIAL CELLS indistinguishable from normal HEPATOCYTES to a poorly differentiated neoplasm. The cells may be uniform or markedly pleomorphic, or form GIANT CELLS. Several classification schemes have been suggested.'),
  'D006529': NtO(MH='Hepatomegaly', MS='Enlargement of the liver.'),
  'D006530': NtO(MH='Hepatorenal Syndrome', MS='Functional KIDNEY FAILURE in patients with liver disease, usually LIVER CIRRHOSIS or portal hypertension (HYPERTENSION, PORTAL), and in the absence of intrinsic renal disease or kidney abnormality. It is characterized by intense renal vasculature constriction, reduced renal blood flow, OLIGURIA, and sodium retention.'),
  'D006531': NtO(MH='HEPES', MS='A dipolar ionic buffer.'),
  'D006533': NtO(MH='Heptachlor', MS="A man-made compound previously used to control termites and other insects. Even though production of heptachlor was phased out of use in the United States during the late 1980's it remains in soil and hazardous waste sites. It is clearly toxic to animals and humans but, the International Agency for Research on Cancer (IARC) has determined that heptachlor is not classifiable as to its carcinogenicity to humans. (From ATSDR Public Heath Statement, April 1989)"),
  'D006534': NtO(MH='Heptachlor Epoxide', MS='An oxidation product of HEPTACHLOR formed by many plants and animals, including humans, after exposure to HEPTACHLOR. It has been shown to remain in soil treated with HEPTACHLOR for over fifteen years and is toxic to animals and humans. (From ATSDR Public Heath Statement, April 1989)'),
  'D006535': NtO(MH='Heptaminol', MS="An amino alcohol that has been used as a myocardial stimulant and vasodilator and to relieve bronchospasm. Its most common therapeutic use is in orthostatic hypotension. The mechanism of heptaminol's therapeutic actions is not well understood although it has been suggested to affect catecholamine release or calcium metabolism."),
  'D006536': NtO(MH='Heptanes', MS='Seven-carbon alkanes with the formula C7H16.'),
  'D006537': NtO(MH='Heptanoates', MS='Salts and esters of the 7-carbon saturated monocarboxylic acid heptanoic acid.'),
  'D006538': NtO(MH='Heptanoic Acids', MS='7-carbon saturated monocarboxylic acids.'),
  'D006539': NtO(MH='Heptoses', MS=None),
  'D006540': NtO(MH='Herbicides', MS='Pesticides used to destroy unwanted vegetation, especially various types of weeds, grasses (POACEAE), and woody plants. Some plants develop HERBICIDE RESISTANCE.'),
  'D006547': NtO(MH='Hernia', MS='Protrusion of tissue, structure, or part of an organ through the bone, muscular tissue, or the membrane by which it is normally contained. Hernia may involve tissues such as the ABDOMINAL WALL or the respiratory DIAPHRAGM. Hernias may be internal, external, congenital, or acquired.'),
  'D006548': NtO(MH='Hernia, Diaphragmatic', MS='Protrusion of abdominal structures into the THORAX as a result of congenital or traumatic defects in the respiratory DIAPHRAGM.'),
  'D006549': NtO(MH='Hernia, Diaphragmatic, Traumatic', MS='The type of DIAPHRAGMATIC HERNIA caused by TRAUMA or injury, usually to the ABDOMEN.'),
  'D006550': NtO(MH='Hernia, Femoral', MS='A groin hernia occurring inferior to the inguinal ligament and medial to the FEMORAL VEIN and FEMORAL ARTERY. The femoral hernia sac has a small neck but may enlarge considerably when it enters the subcutaneous tissue of the thigh. It is caused by defects in the ABDOMINAL WALL.'),
  'D006551': NtO(MH='Hernia, Hiatal', MS='STOMACH herniation located at or near the diaphragmatic opening for the ESOPHAGUS, the esophageal hiatus.'),
  'D006552': NtO(MH='Hernia, Inguinal', MS="An abdominal hernia with an external bulge in the GROIN region. It can be classified by the location of herniation. Indirect inguinal hernias occur through the internal inguinal ring. Direct inguinal hernias occur through defects in the ABDOMINAL WALL (transversalis fascia) in Hesselbach's triangle. The former type is commonly seen in children and young adults; the latter in adults."),
  'D006553': NtO(MH='Hernia, Obturator', MS='A pelvic hernia through the obturator foramen, a large aperture in the hip bone normally covered by a membrane. Obturator hernia can lead to intestinal incarceration and INTESTINAL OBSTRUCTION.'),
  'D006554': NtO(MH='Hernia, Umbilical', MS='A HERNIA due to an imperfect closure or weakness of the umbilical ring. It appears as a skin-covered protrusion at the UMBILICUS during crying, coughing, or straining. The hernia generally consists of OMENTUM or SMALL INTESTINE. The vast majority of umbilical hernias are congenital but can be acquired due to severe abdominal distention.'),
  'D006555': NtO(MH='Hernia, Ventral', MS='A hernia caused by weakness of the anterior ABDOMINAL WALL due to midline defects, previous incisions, or increased intra-abdominal pressure. Ventral hernias include UMBILICAL HERNIA, incisional, epigastric, and spigelian hernias.'),
  'D006556': NtO(MH='Heroin Dependence', MS='Strong dependence or addiction, both physiological and emotional, upon HEROIN.'),
  'D006557': NtO(MH='Herpangina', MS='Acute types of coxsackievirus infections or ECHOVIRUS INFECTIONS that usually affect children during the summer and are characterized by vesiculoulcerative lesions on the MUCOUS MEMBRANES of the THROAT; DYSPHAGIA; VOMITING, and FEVER.'),
  'D006558': NtO(MH='Herpes Genitalis', MS='Infection of the genitals (GENITALIA) with HERPES SIMPLEX VIRUS in either the males or the females.'),
  'D006559': NtO(MH='Pemphigoid Gestationis', MS='An itching, autoimmune, bullous SKIN disease that occurs during the last two trimesters of PREGNANCY and PUERPERIUM. It also affects non-pregnant females with tissue of PLACENTA origin, such as CHORIOCARCINOMA; or HYDATIDIFORM MOLE. It exhibits antigenic and clinical similarity to bullous pemphigoid (PEMPHIGOID, BULLOUS). This disease does not involve herpes viruses (old name, herpes gestationis).'),
  'D006560': NtO(MH='Herpes Labialis', MS='Herpes simplex, caused by type 1 virus, primarily spread by oral secretions and usually occurring as a concomitant of fever. It may also develop in the absence of fever or prior illness. It commonly involves the facial region, especially the lips and the nares. (Dorland, 27th ed.)'),
  'D006561': NtO(MH='Herpes Simplex', MS='A group of acute infections caused by herpes simplex virus type 1 or type 2 that is characterized by the development of one or more small fluid-filled vesicles with a raised erythematous base on the skin or mucous membrane. It occurs as a primary infection or recurs due to a reactivation of a latent infection. (Dorland, 27th ed.)'),
  'D006562': NtO(MH='Herpes Zoster', MS='An acute infectious, usually self-limited, disease believed to represent activation of latent varicella-zoster virus (HERPESVIRUS 3, HUMAN) in those who have been rendered partially immune after a previous attack of CHICKENPOX. It involves the SENSORY GANGLIA and their areas of innervation and is characterized by severe neuralgic pain along the distribution of the affected nerve and crops of clustered vesicles over the area. (From Dorland, 27th ed)'),
  'D006563': NtO(MH='Herpes Zoster Ophthalmicus', MS='Virus infection of the Gasserian ganglion and its nerve branches characterized by pain and vesicular eruptions with much swelling. Ocular involvement is usually heralded by a vesicle on the tip of the nose. This area is innervated by the nasociliary nerve.'),
  'D006564': NtO(MH='Herpesviridae', MS='A family of enveloped, linear, double-stranded DNA viruses infecting a wide variety of animals. Subfamilies, based on biological characteristics, include: ALPHAHERPESVIRINAE; BETAHERPESVIRINAE; and GAMMAHERPESVIRINAE.'),
  'D006566': NtO(MH='Herpesviridae Infections', MS='Virus diseases caused by the HERPESVIRIDAE.'),
  'D006567': NtO(MH='Herpesvirus 2, Saimiriine', MS='The type species of RHADINOVIRUS, in the subfamily GAMMAHERPESVIRINAE, isolated from squirrel monkeys. It produces malignant lymphomas (LYMPHOMA, MALIGNANT) in inoculated marmosets or owl monkeys.'),
  'D006568': NtO(MH='Herpesvirus 1, Cercopithecine', MS='A species of SIMPLEXVIRUS that causes vesicular lesions of the mouth in monkeys. When the virus is transmitted to man it causes an acute encephalitis or encephalomyelitis, which is nearly always fatal.'),
  'D006569': NtO(MH='Hesperidin', MS='A flavanone glycoside found in CITRUS fruit peels.'),
  'D006570': NtO(MH='Heterochromatin', MS='The portion of chromosome material that remains condensed and is transcriptionally inactive during INTERPHASE.'),
  'D006571': NtO(MH='Heterocyclic Compounds', MS='Cyclic compounds that include atoms other than carbon in their ring structure.'),
  'D006572': NtO(MH='Heterocyclic Compounds, Bridged-Ring', MS='A class of organic compounds which contain two rings that share a pair of bridgehead carbon atoms.'),
  'D006573': NtO(MH='Heterocyclic Compounds, 1-Ring', MS='Organic compounds that contain a ring structure made up of carbon and one or more additional elements such as nitrogen and oxygen.'),
  'D006574': NtO(MH='Heterocyclic Compounds, 2-Ring', MS='A class of heterocyclic compounds that include a two-ring fused structure. Both aromatic and non-aromatic ring structures are included in this category.'),
  'D006575': NtO(MH='Heterocyclic Compounds, 3-Ring', MS='A class of heterocyclic compounds that include a three-ring fused structure. Both aromatic or non-aromatic ring structures are included in this category.'),
  'D006576': NtO(MH='Heterocyclic Compounds, 4 or More Rings', MS='A class of heterocyclic compounds that include four or more fused rings. Both aromatic or non-aromatic ring structures are included in this category.'),
  'D006577': NtO(MH='Heterocyclic Oxides', MS='Oxides of a ring compound having atoms other than carbon in its ring.'),
  'D006578': NtO(MH='Heterophyidae', MS='A family of intestinal flukes of the class Trematoda which occurs in animals and man. Some of the genera are Heterophyes, Metagonimus, Cryptocotyle, Stellantchasmus, and Euryhelmis.'),
  'D006579': NtO(MH='Heterozygote', MS='An individual having different alleles at one or more loci regarding a specific character.'),
  'D006580': NtO(MH='Genetic Carrier Screening', MS='Identification of individuals who are heterozygous at a GENETIC LOCUS for a recessive PHENOTYPE.'),
  'D006581': NtO(MH='Hexachlorobenzene', MS='An agricultural fungicide and seed treatment agent.'),
  'D006582': NtO(MH='Hexachlorophene', MS='A chlorinated bisphenol antiseptic with a bacteriostatic action against Gram-positive organisms, but much less effective against Gram-negative organisms. It is mainly used in soaps and creams and is an ingredient of various preparations used for skin disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p797)'),
  'D006583': NtO(MH='Hexadimethrine Bromide', MS='A synthetic polymer which agglutinates red blood cells. It is used as a heparin antagonist.'),
  'D006584': NtO(MH='Hexamethonium Compounds', MS='Compounds containing the hexamethylenebis(trimethylammonium) cation. Members of this group frequently act as antihypertensive agents and selective ganglionic blocking agents.'),
  'D006585': NtO(MH='Altretamine', MS='A hexamethyl-2,4,6-triamine derivative of 1,3,5-triazine.'),
  'D006586': NtO(MH='Hexanes', MS='Six-carbon saturated hydrocarbon group of the methane series. Include isomers and derivatives. Various polyneuropathies are caused by hexane poisoning.'),
  'D006588': NtO(MH='Hexanones', MS='6-carbon straight-chain or branched ketones.'),
  'D006589': NtO(MH='Hexestrol', MS='A synthetic estrogen that has been used as a hormonal antineoplastic agent.'),
  'D006590': NtO(MH='Hexetidine', MS='A bactericidal and fungicidal antiseptic. It is used as a 0.1% mouthwash for local infections and oral hygiene. (From Martindale, The Extra Pharmacopoeia, 30th ed, p797)'),
  'D006591': NtO(MH='Hexobarbital', MS='A barbiturate that is effective as a hypnotic and sedative.'),
  'D006592': NtO(MH='Hexobendine', MS='A potent vasoactive agent that dilates cerebral and coronary arteries, but slightly constricts femoral arteries, without any effects on heart rate, blood pressure or cardiac output.'),
  'D006593': NtO(MH='Hexokinase', MS='An enzyme that catalyzes the conversion of ATP and a D-hexose to ADP and a D-hexose 6-phosphate. D-Glucose, D-mannose, D-fructose, sorbitol, and D-glucosamine can act as acceptors; ITP and dATP can act as donors. The liver isoenzyme has sometimes been called glucokinase. (From Enzyme Nomenclature, 1992) EC 2.7.1.1.'),
  'D006594': NtO(MH='Hexoprenaline', MS='Stimulant of adrenergic beta 2 receptors. It is used as a bronchodilator, antiasthmatic agent, and tocolytic agent.'),
  'D006595': NtO(MH='Hexosamines', MS=None),
  'D006596': NtO(MH='Hexosaminidases', MS='Enzymes that catalyze the hydrolysis of N-acylhexosamine residues in N-acylhexosamides. Hexosaminidases also act on GLUCOSIDES; GALACTOSIDES; and several OLIGOSACCHARIDES.'),
  'D006597': NtO(MH='Fructose-Bisphosphatase', MS='An enzyme that catalyzes the conversion of D-fructose 1,6-bisphosphate and water to D-fructose 6-phosphate and orthophosphate. EC 3.1.3.11.'),
  'D006598': NtO(MH='Hexosediphosphates', MS=None),
  'D006599': NtO(MH='UDPglucose-Hexose-1-Phosphate Uridylyltransferase', MS='An enzyme that catalyzes the transfer of UMP from UDPglucose to galactose 1-phosphate, forming UDPgalactose and glucose 1-phosphate. Deficiency in this enzyme is the major cause of GALACTOSEMIA. EC 2.7.7.12.'),
  'D006600': NtO(MH='Hexosephosphates', MS=None),
  'D006601': NtO(MH='Hexoses', MS=None),
  'D006602': NtO(MH='Hexosyltransferases', MS='Enzymes that catalyze the transfer of hexose groups. EC 2.4.1.-.'),
  'D006603': NtO(MH='Hexuronic Acids', MS='Term used to designate tetrahydroxy aldehydic acids obtained by oxidation of hexose sugars, i.e. glucuronic acid, galacturonic acid, etc. Historically, the name hexuronic acid was originally given to ascorbic acid.'),
  'D006604': NtO(MH='Hexylresorcinol', MS='A substituted dihydroxybenzene used topically as an antiseptic for the treatment of minor skin infections.'),
  'D006605': NtO(MH='Hibernation', MS='The dormant state in which some warm-blooded animal species pass the winter. It is characterized by narcosis and by sharp reduction in body temperature and metabolic activity and by a depression of vital signs.'),
  'D006606': NtO(MH='Hiccup', MS='A spasm of the diaphragm that causes a sudden inhalation followed by rapid closure of the glottis which produces a sound.'),
  'D006607': NtO(MH='Adenoma, Sweat Gland', MS='A benign neoplasm derived from epithelial cells of sweat glands. (Stedman, 25th ed)'),
  'D006608': NtO(MH='Hierarchy, Social', MS='Social rank-order established by certain behavioral patterns.'),
  'D006609': NtO(MH='High Mobility Group Proteins', MS='A family of low-molecular weight, non-histone proteins found in chromatin.'),
  'D006610': NtO(MH='High Pressure Neurological Syndrome', MS='A syndrome related to increased atmospheric pressure and characterized by tremors, nausea, dizziness, decreased motor and mental performance, and SEIZURES. This condition may occur in those who dive deeply (c. 1000 ft) usually while breathing a mixture of oxygen and helium. The condition is associated with a neuroexcitatory effect of helium.'),
  'D006611': NtO(MH='High-Frequency Jet Ventilation', MS='Respiratory support system used primarily with rates of about 100 to 200/min with volumes of from about one to three times predicted anatomic dead space. Used to treat respiratory failure and maintain ventilation under severe circumstances.'),
  'D006612': NtO(MH='High-Frequency Ventilation', MS='Ventilatory support system using frequencies from 60-900 cycles/min or more. Three types of systems have been distinguished on the basis of rates, volumes, and the system used. They are high frequency positive-pressure ventilation (HFPPV); HIGH-FREQUENCY JET VENTILATION; (HFJV); and high-frequency oscillation (HFO).'),
  'D006613': NtO(MH='Higher Nervous Activity', MS='A term used in Eastern European research literature on brain and behavior physiology for cortical functions. It refers to the highest level of integrative function of the brain, centered in the CEREBRAL CORTEX, regulating language, thought, and behavior via sensory, motor, and cognitive processes.'),
  'D006614': NtO(MH='Hindlimb', MS='Either of two extremities of four-footed non-primate land animals. It usually consists of a FEMUR; TIBIA; and FIBULA; tarsals; METATARSALS; and TOES. (From Storer et al., General Zoology, 6th ed, p73)'),
  'D006615': NtO(MH='Hip', MS='The projecting part on each side of the body, formed by the side of the pelvis and the top portion of the femur.'),
  'D006616': NtO(MH='Hip Contracture', MS='Permanent fixation of the hip in primary positions, with limited passive or active motion at the hip joint. Locomotion is difficult and pain is sometimes present when the hip is in motion. It may be caused by trauma, infection, or poliomyelitis. (From Current Medical Information & Technology, 5th ed)'),
  'D006617': NtO(MH='Hip Dislocation', MS='Displacement of the femur bone from its normal position at the HIP JOINT.'),
  'D006618': NtO(MH='Hip Dislocation, Congenital', MS='Congenital dislocation of the hip generally includes subluxation of the femoral head, acetabular dysplasia, and complete dislocation of the femoral head from the true acetabulum. This condition occurs in approximately 1 in 1000 live births and is more common in females than in males.'),
  'D006619': NtO(MH='Hip Dysplasia, Canine', MS='A hereditary disease of the hip joints in dogs. Signs of the disease may be evident any time after 4 weeks of age.'),
  'D006620': NtO(MH='Hip Fractures', MS='Fractures of the FEMUR HEAD; the FEMUR NECK; (FEMORAL NECK FRACTURES); the trochanters; or the inter- or subtrochanteric region. Excludes fractures of the acetabulum and fractures of the femoral shaft below the subtrochanteric region (FEMORAL FRACTURES).'),
  'D006621': NtO(MH='Hip Joint', MS='The joint that is formed by the articulation of the head of FEMUR and the ACETABULUM of the PELVIS.'),
  'D006622': NtO(MH='Hip Prosthesis', MS='Replacement for a hip joint.'),
  'D006623': NtO(MH='von Hippel-Lindau Disease', MS='An autosomal dominant disorder caused by mutations in a tumor suppressor gene. This syndrome is characterized by abnormal growth of small blood vessels leading to a host of neoplasms. They include HEMANGIOBLASTOMA in the RETINA; CEREBELLUM; and SPINAL CORD; PHEOCHROMOCYTOMA; pancreatic tumors; and renal cell carcinoma (see CARCINOMA, RENAL CELL). Common clinical signs include HYPERTENSION and neurological dysfunctions.'),
  'D006624': NtO(MH='Hippocampus', MS='A curved elevation of GRAY MATTER extending the entire length of the floor of the TEMPORAL HORN of the LATERAL VENTRICLE (see also TEMPORAL LOBE). The hippocampus proper, subiculum, and DENTATE GYRUS constitute the hippocampal formation. Sometimes authors include the ENTORHINAL CORTEX in the hippocampal formation.'),
  'D006625': NtO(MH='Hippocratic Oath', MS='An oath, attributed to Hippocrates, that serves as an ethical guide for the medical profession.'),
  'D006626': NtO(MH='Hippurates', MS='Salts and esters of hippuric acid.'),
  'D006627': NtO(MH='Hirschsprung Disease', MS='Congenital MEGACOLON resulting from the absence of ganglion cells (aganglionosis) in a distal segment of the LARGE INTESTINE. The aganglionic segment is permanently contracted thus causing dilatation proximal to it. In most cases, the aganglionic segment is within the RECTUM and SIGMOID COLON.'),
  'D006628': NtO(MH='Hirsutism', MS='A condition observed in WOMEN and CHILDREN when there is excess coarse body hair of an adult male distribution pattern, such as facial and chest areas. It is the result of elevated ANDROGENS from the OVARIES, the ADRENAL GLANDS, or exogenous sources. The concept does not include HYPERTRICHOSIS, which is an androgen-independent excessive hair growth.'),
  'D006629': NtO(MH='Hirudins', MS="Single-chain polypeptides of about 65 amino acids (7 kDa) from LEECHES that have a neutral hydrophobic N terminus, an acidic hydrophilic C terminus, and a compact, hydrophobic core region. Recombinant hirudins lack tyr-63 sulfation and are referred to as 'desulfato-hirudins'. They form a stable non-covalent complex with ALPHA-THROMBIN, thereby abolishing its ability to cleave FIBRINOGEN."),
  'D006630': NtO(MH='Hispanic Americans', MS='Persons living in the United States of Mexican (MEXICAN AMERICANS), Puerto Rican, Cuban, Central or South American, or other Spanish culture or origin. The concept does not include Brazilian Americans or Portuguese Americans.'),
  'D006631': NtO(MH='Amine Oxidase (Copper-Containing)', MS='A group of enzymes including those oxidizing primary monoamines, diamines, and histamine. They are copper proteins, and, as their action depends on a carbonyl group, they are sensitive to inhibition by semicarbazide.'),
  'D006632': NtO(MH='Histamine', MS='An amine derived by enzymatic decarboxylation of HISTIDINE. It is a powerful stimulant of gastric secretion, a constrictor of bronchial smooth muscle, a vasodilator, and also a centrally acting neurotransmitter.'),
  'D006633': NtO(MH='Histamine Antagonists', MS='Drugs that bind to but do not activate histamine receptors, thereby blocking the actions of histamine or histamine agonists. Classical antihistaminics block the histamine H1 receptors only.'),
  'D006634': NtO(MH='Histamine H1 Antagonists', MS='Drugs that selectively bind to but do not activate histamine H1 receptors, thereby blocking the actions of endogenous histamine. Included here are the classical antihistaminics that antagonize or prevent the action of histamine mainly in immediate hypersensitivity. They act in the bronchi, capillaries, and some other smooth muscles, and are used to prevent or allay motion sickness, seasonal rhinitis, and allergic dermatitis and to induce somnolence. The effects of blocking central nervous system H1 receptors are not as well understood.'),
  'D006635': NtO(MH='Histamine H2 Antagonists', MS='Drugs that selectively bind to but do not activate histamine H2 receptors, thereby blocking the actions of histamine. Their clinically most important action is the inhibition of acid secretion in the treatment of gastrointestinal ulcers. Smooth muscle may also be affected. Some drugs in this class have strong effects in the central nervous system, but these actions are not well understood.'),
  'D006636': NtO(MH='Histamine Release', MS="The secretion of histamine from mast cell and basophil granules by exocytosis. This can be initiated by a number of factors, all of which involve binding of IgE, cross-linked by antigen, to the mast cell or basophil's Fc receptors. Once released, histamine binds to a number of different target cell receptors and exerts a wide variety of effects."),
  'D006637': NtO(MH='Histamine N-Methyltransferase', MS='An enzyme that catalyzes the transfer of a methyl group from S-adenosylmethionine to histamine, forming N-methylhistamine, the major metabolite of histamine in man. EC 2.1.1.8.'),
  'D006638': NtO(MH='Histidine Ammonia-Lyase', MS='An enzyme that catalyzes the first step of histidine catabolism, forming UROCANIC ACID and AMMONIA from HISTIDINE. Deficiency of this enzyme is associated with elevated levels of serum histidine and is called histidinemia (AMINO ACID METABOLISM, INBORN ERRORS).'),
  'D006639': NtO(MH='Histidine', MS='An essential amino acid that is required for the production of HISTAMINE.'),
  'D006640': NtO(MH='Histidine Decarboxylase', MS='An enzyme that catalyzes the decarboxylation of histidine to histamine and carbon dioxide. It requires pyridoxal phosphate in animal tissues, but not in microorganisms. EC 4.1.1.22.'),
  'D006641': NtO(MH='Histidinol', MS='The penultimate step in the pathway of histidine biosynthesis. Oxidation of the alcohol group on the side chain gives the acid group forming histidine. Histidinol has also been used as an inhibitor of protein synthesis.'),
  'D006642': NtO(MH='Histidinol-Phosphatase', MS='An enzyme that catalyzes the hydrolysis of histidinol-phosphate to histidinol. One of the regulatory enzymes in histidine biosynthesis. EC 3.1.3.15.'),
  'D006643': NtO(MH='Histidine-tRNA Ligase', MS='An enzyme that activates histidine with its specific transfer RNA. EC 6.1.1.21.'),
  'D006644': NtO(MH='Histiocytes', MS='Macrophages found in the TISSUES, as opposed to those found in the blood (MONOCYTES) or serous cavities (SEROUS MEMBRANE).'),
  'D006646': NtO(MH='Histiocytosis, Langerhans-Cell', MS='A group of disorders resulting from the abnormal proliferation of and tissue infiltration by LANGERHANS CELLS which can be detected by their characteristic Birbeck granules (X bodies), or by monoclonal antibody staining for their surface CD1 ANTIGENS. Langerhans-cell granulomatosis can involve a single organ, or can be a systemic disorder.'),
  'D006648': NtO(MH='Histocompatibility', MS='The degree of antigenic similarity between the tissues of different individuals, which determines the acceptance or rejection of allografts.'),
  'D006649': NtO(MH='Histocompatibility Antigens', MS='A group of antigens that includes both the major and minor histocompatibility antigens. The former are genetically determined by the major histocompatibility complex. They determine tissue type for transplantation and cause allograft rejections. The latter are systems of allelic alloantigens that can cause weak transplant rejection.'),
  'D006650': NtO(MH='Histocompatibility Testing', MS='Identification of the major histocompatibility antigens of transplant DONORS and potential recipients, usually by serological tests. Donor and recipient pairs should be of identical ABO blood group, and in addition should be matched as closely as possible for HISTOCOMPATIBILITY ANTIGENS in order to minimize the likelihood of allograft rejection. (King, Dictionary of Genetics, 4th ed)'),
  'D006651': NtO(MH='Histocytochemistry', MS='Study of intracellular distribution of chemicals, reaction sites, enzymes, etc., by means of staining reactions, radioactive isotope uptake, selective metal distribution in electron microscopy, or other methods.'),
  'D006652': NtO(MH='Histological Techniques', MS='Methods of preparing tissue for examination and study of the origin, structure, function, or pathology.'),
  'D006653': NtO(MH='Histology', MS='The study of the structure of various TISSUES of organisms on a microscopic level.'),
  'D006654': NtO(MH='Histology, Comparative', MS='The study of the similarities and differences in the structures of homologous tissues across various species.'),
  'D006655': NtO(MH='Histone Deacetylases', MS='Deacetylases that remove N-acetyl groups from amino side chains of the amino acids of HISTONES. The enzyme family can be divided into at least three structurally-defined subclasses. Class I and class II deacetylases utilize a zinc-dependent mechanism. The sirtuin histone deacetylases belong to class III and are NAD-dependent enzymes.'),
  'D006656': NtO(MH='Protamine Kinase', MS='An aspect of protein kinase (EC 2.7.1.37) in which serine residues in protamines and histones are phosphorylated in the presence of ATP.'),
  'D006657': NtO(MH='Histones', MS='Small chromosomal proteins (approx 12-20 kD) possessing an open, unfolded structure and attached to the DNA in cell nuclei by ionic linkages. Classification into the various types (designated histone I, histone II, etc.) is based on the relative amounts of arginine and lysine in each.'),
  'D006658': NtO(MH='Histoplasma', MS='A mitosporic Onygenales fungal genus causing HISTOPLASMOSIS in humans and animals. Its single species is Histoplasma capsulatum which has two varieties: H. capsulatum var. capsulatum and H. capsulatum var. duboisii. Its teleomorph is AJELLOMYCES capsulatus.'),
  'D006659': NtO(MH='Histoplasmin', MS=None),
  'D006660': NtO(MH='Histoplasmosis', MS='Infection resulting from exposure to the fungus HISTOPLASMA. It is worldwide in distribution and particularly common in the central and eastern states, especially areas around the Ohio and Mississippi River valleys.'),
  'D006663': NtO(MH='Historiography', MS='The writing of history; the principles, theory, and history of historical writing; the product of historical writing. (Webster, 3d ed)'),
  'D006664': NtO(MH='History', MS='Record and study of past events.'),
  'D006665': NtO(MH='History of Dentistry', MS='Study of past events and development in the field of DENTISTRY.'),
  'D006666': NtO(MH='History of Medicine', MS='Study of past events and development in the field of Medicine.'),
  'D006676': NtO(MH='History of Nursing', MS='Study of past events and development in the field of Nursing.'),
  'D006677': NtO(MH='Histrionic Personality Disorder', MS='A personality disorder characterized by overly reactive and intensely expressed or overly dramatic behavior, proneness to exaggeration, emotional excitability, and disturbances in interpersonal relationships.'),
  'D006678': NtO(MH='HIV', MS='Human immunodeficiency virus. A non-taxonomic and historical term referring to any of two species, specifically HIV-1 and/or HIV-2. Prior to 1986, this was called human T-lymphotropic virus type III/lymphadenopathy-associated virus (HTLV-III/LAV). From 1986-1990, it was an official species called HIV. Since 1991, HIV was no longer considered an official species name; the two species were designated HIV-1 and HIV-2.'),
  'D006679': NtO(MH='HIV Seropositivity', MS='Development of neutralizing antibodies in individuals who have been exposed to the human immunodeficiency virus (HIV/HTLV-III/LAV).'),
  'D006680': NtO(MH='HLA Antigens', MS='Antigens determined by leukocyte loci found on chromosome 6, the major histocompatibility loci in humans. They are polypeptides or glycoproteins found on most nucleated cells and platelets, determine tissue types for transplantation, and are associated with certain diseases.'),
  'D006681': NtO(MH='HLA-D Antigens', MS='Human immune-response or Class II antigens found mainly, but not exclusively, on B-lymphocytes and produced from genes of the HLA-D locus. They are extremely polymorphic families of glycopeptides, each consisting of two chains, alpha and beta. This group of antigens includes the -DR, -DQ and -DP designations, of which HLA-DR is most studied; some of these glycoproteins are associated with certain diseases, possibly of immune etiology.'),
  'D006682': NtO(MH='HLA-DP Antigens', MS='A group of the D-related HLA antigens (human) found to differ from the DR antigens in genetic locus and therefore inheritance. These antigens are polymorphic glycoproteins comprising alpha and beta chains and are found on lymphoid and other cells, often associated with certain diseases.'),
  'D006683': NtO(MH='HLA-DQ Antigens', MS='A group of the D-related HLA antigens found to differ from the DR antigens in genetic locus and therefore inheritance. These antigens are polymorphic glycoproteins comprising alpha and beta chains and are found on lymphoid and other cells, often associated with certain diseases.'),
  'D006684': NtO(MH='HLA-DR Antigens', MS='A subclass of HLA-D antigens that consist of alpha and beta chains. The inheritance of HLA-DR antigens differs from that of the HLA-DQ ANTIGENS and HLA-DP ANTIGENS.'),
  'D006685': NtO(MH='Hoarseness', MS='An unnaturally deep or rough quality of voice.'),
  'D006686': NtO(MH='Hobbies', MS='Leisure activities engaged in for pleasure.'),
  'D006688': NtO(MH='Hockey', MS='A game in which two parties of players provided with curved or hooked sticks seek to drive a ball or puck through opposite goals. This applies to either ice hockey or field hockey.'),
  'D006689': NtO(MH='Hodgkin Disease', MS="A malignant disease characterized by progressive enlargement of the lymph nodes, spleen, and general lymphoid tissue. In the classical variant, giant usually multinucleate Hodgkin's and REED-STERNBERG CELLS are present; in the nodular lymphocyte predominant variant, lymphocytic and histiocytic cells are seen."),
  'D006690': NtO(MH='Bisbenzimidazole', MS='A benzimidazole antifilarial agent; it is fluorescent when it binds to certain nucleotides in DNA, thus providing a tool for the study of DNA replication; it also interferes with mitosis.'),
  'D006691': NtO(MH='Classical Swine Fever', MS='An acute, highly contagious disease affecting swine of all ages and caused by the CLASSICAL SWINE FEVER VIRUS. It has a sudden onset with high morbidity and mortality.'),
  'D006692': NtO(MH='Classical Swine Fever Virus', MS='A species of the PESTIVIRUS genus causing exceedingly contagious and fatal hemorrhagic disease of swine.'),
  'D006693': NtO(MH='Holidays', MS='Days commemorating events. Holidays also include vacation periods.'),
  'D006694': NtO(MH='Holistic Health', MS='Health as viewed from the perspective that humans function as complete, integrated units rather than as aggregates of separate parts.'),
  'D006695': NtO(MH='Holmium', MS='An element of the rare earth family of metals. It has the atomic symbol Ho, atomic number 67, and atomic weight 164.93.'),
  'D006696': NtO(MH='Holography', MS='The recording of images in three-dimensional form on a photographic film by exposing it to a laser beam reflected from the object under study.'),
  'D006697': NtO(MH='Holothurin', MS='A highly toxic saponin occurring in HOLOTHUROIDEA. This marine toxin is an anionic surfactant, hemolyzing ERYTHROCYTES.'),
  'D006698': NtO(MH='Holtzman Inkblot Test', MS="Personality evaluation based on the scoring of several variables as a result of the subject's responses in perceiving 47 inkblot plates."),
  'D006699': NtO(MH='Home Care Services', MS="Community health and NURSING SERVICES providing coordinated multiple services to the patient at the patient's homes. These home-care services are provided by a visiting nurse, home health agencies, HOSPITALS, or organized community groups using professional staff for care delivery. It differs from HOME NURSING which is provided by non-professionals."),
  'D006700': NtO(MH='Home Childbirth', MS='Childbirth taking place in the home.'),
  'D006701': NtO(MH='Home Nursing', MS='Nursing care given to an individual in the home. The care may be provided by a family member or a friend. Home nursing as care by a non-professional is differentiated from HOME CARE SERVICES provided by professionals: visiting nurse, home health agencies, hospital, or other organized community group.'),
  'D006702': NtO(MH='Homing Behavior', MS='Instinctual patterns of activity related to a specific area including ability of certain animals to return to a given place when displaced from it, often over great distances using navigational clues such as those used in migration (ANIMAL MIGRATION).'),
  'D006703': NtO(MH='Homeless Persons', MS='Persons who have no permanent residence. The concept excludes nomadic peoples.'),
  'D006704': NtO(MH='Homemaker Services', MS='Non-medical support services, such as food preparation and bathing, given by trained personnel to disabled, sick or convalescent individuals in their home.'),
  'D006705': NtO(MH='Homeopathy', MS='A system of therapeutics founded by Samuel Hahnemann (1755-1843), based on the Law of Similars where "like cures like". Diseases are treated by highly diluted substances that cause, in healthy persons, symptoms like those of the disease to be treated.'),
  'D006706': NtO(MH='Homeostasis', MS='The processes whereby the internal environment of an organism tends to remain balanced and stable.'),
  'D006707': NtO(MH='Homes for the Aged', MS='Geriatric long-term care facilities which provide supervision and assistance in activities of daily living with medical and nursing services when required.'),
  'D006708': NtO(MH='Homicide', MS='The killing of one person by another.'),
  'D006709': NtO(MH='Homoarginine', MS=None),
  'D006710': NtO(MH='Homocysteine', MS='A thiol-containing amino acid formed by a demethylation of METHIONINE.'),
  'D006711': NtO(MH='Homocystine', MS=None),
  'D006712': NtO(MH='Homocystinuria', MS='Autosomal recessive inborn error of methionine metabolism usually caused by a deficiency of CYSTATHIONINE BETA-SYNTHASE and associated with elevations of homocysteine in plasma and urine. Clinical features include a tall slender habitus, SCOLIOSIS, arachnodactyly, MUSCLE WEAKNESS, genu varus, thin blond hair, malar flush, lens dislocations, an increased incidence of MENTAL RETARDATION, and a tendency to develop fibrosis of arteries, frequently complicated by CEREBROVASCULAR ACCIDENTS and MYOCARDIAL INFARCTION. (From Adams et al., Principles of Neurology, 6th ed, p979)'),
  'D006713': NtO(MH='Homogentisic Acid', MS='Dihydroxyphenylacetic acid with hydroxyls at the 2 and 5 positions of the phenyl ring.'),
  'D006714': NtO(MH='Homoserine', MS=None),
  'D006715': NtO(MH='Homoserine Dehydrogenase', MS='An enzyme that catalyzes the reduction of aspartic beta-semialdehyde to homoserine, which is the branch point in biosynthesis of methionine, lysine, threonine and leucine from aspartic acid. EC 1.1.1.3.'),
  'D006716': NtO(MH='Homosexuality', MS='The sexual attraction or relationship between members of the same SEX.'),
  'D006718': NtO(MH='Homosteroids', MS='Steroids whose structure has been expanded by the addition of one or more carbon atoms to the ring skeleton in any of the four rings.'),
  'D006719': NtO(MH='Homovanillic Acid', MS='A 3-O-methyl ETHER of (3,4-dihydroxyphenyl)acetic acid.'),
  'D006720': NtO(MH='Homozygote', MS='An individual in which both alleles at a given locus are identical.'),
  'D006721': NtO(MH='Honduras', MS='A country in CENTRAL AMERICA, bordering the Caribbean Sea, between GUATEMALA and NICARAGUA and bordering the Gulf of Fonseca (North Pacific Ocean), between EL SALVADOR and NICARAGUA.'),
  'D006722': NtO(MH='Honey', MS='A sweet viscous liquid food, produced in the honey sacs of various bees from nectar collected from flowers. The nectar is ripened into honey by inversion of its sucrose sugar into fructose and glucose. It is somewhat acidic and has mild antiseptic properties, being sometimes used in the treatment of burns and lacerations.'),
  'D006723': NtO(MH='Hong Kong', MS='The former British crown colony located off the southeast coast of China, comprised of Hong Kong Island, Kowloon Peninsula, and New Territories. The three sites were ceded to the British by the Chinese respectively in 1841, 1860, and 1898. Hong Kong reverted to China in July 1997. The name represents the Cantonese pronunciation of the Chinese xianggang, fragrant port, from xiang, perfume and gang, port or harbor, with reference to its currents sweetened by fresh water from a river west of it.'),
  'D006724': NtO(MH='Hoof and Claw', MS='Highly keratinized processes that are sharp and curved, or flat with pointed margins. They are found especially at the end of the limbs in certain animals.'),
  'D006725': NtO(MH='Hookworm Infections', MS='Infection of humans or animals with hookworms other than those caused by the genus Ancylostoma or Necator, for which the specific terms ANCYLOSTOMIASIS and NECATORIASIS are available.'),
  'D006726': NtO(MH='Hordeolum', MS='Purulent infection of one of the sebaceous glands of Zeis along the eyelid margin (external) or of the meibomian gland on the conjunctival side of the eyelid (internal).'),
  'D006727': NtO(MH='Hormone Antagonists', MS='Chemical substances which inhibit the function of the endocrine glands, the biosynthesis of their secreted hormones, or the action of hormones upon their specific sites.'),
  'D006728': NtO(MH='Hormones', MS='Chemical substances having a specific regulatory effect on the activity of a certain organ or organs. The term was originally applied to substances secreted by various ENDOCRINE GLANDS and transported in the bloodstream to the target organs. It is sometimes extended to include those substances that are not produced by the endocrine glands but that have similar effects.'),
  'D006729': NtO(MH='Hormones, Ectopic', MS='Hormones released from neoplasms or from other cells that are not the usual sources of hormones.'),
  'D006730': NtO(MH='Hormones, Hormone Substitutes, and Hormone Antagonists', MS='A collective grouping for both naturally occurring and synthetic hormones, substitutes, and antagonists.'),
  'D006732': NtO(MH='Horner Syndrome', MS='A syndrome associated with defective sympathetic innervation to one side of the face, including the eye. Clinical features include MIOSIS; mild BLEPHAROPTOSIS; and hemifacial ANHIDROSIS (decreased sweating)(see HYPOHIDROSIS). Lesions of the BRAIN STEM; cervical SPINAL CORD; first thoracic nerve root; apex of the LUNG; CAROTID ARTERY; CAVERNOUS SINUS; and apex of the ORBIT may cause this condition. (From Miller et al., Clinical Neuro-Ophthalmology, 4th ed, pp500-11)'),
  'D006733': NtO(MH='Horns', MS='Either of the pair of hard processes, including ANTLERS, that consist of a BONE core surrounded by a middle KERATIN layer which is in turn covered by EPIDERMIS. They grow from the upper portion of the head of many RUMINANTS.'),
  'D006734': NtO(MH='Horse Diseases', MS='Diseases of domestic and wild horses of the species Equus caballus.'),
  'D006735': NtO(MH='Horseradish Peroxidase', MS='An enzyme isolated from horseradish which is able to act as an antigen. It is frequently used as a histochemical tracer for light and electron microscopy. Its antigenicity has permitted its use as a combined antigen and marker in experimental immunology.'),
  'D006736': NtO(MH='Horses', MS='Large, hoofed mammals of the family EQUIDAE. Horses are active day and night with most of the day spent seeking and consuming food. Feeding peaks occur in the early morning and late afternoon, and there are several daily periods of rest.'),
  'D006737': NtO(MH='Horseshoe Crabs', MS='An arthropod subclass (Xiphosura) comprising the North American (Limulus) and Asiatic (Tachypleus) genera of horseshoe crabs.'),
  'D006738': NtO(MH='Hospices', MS="Facilities or services which are especially devoted to providing palliative and supportive care to the patient with a terminal illness and to the patient's family."),
  'D006739': NtO(MH='Hospital Administration', MS='Management of the internal organization of the hospital.'),
  'D006740': NtO(MH='Hospital Administrators', MS='Managerial personnel responsible for implementing policy and directing the activities of hospitals.'),
  'D006741': NtO(MH='Hospital Auxiliaries', MS='Volunteer organizations whose members perform work for the hospital without compensation.'),
  'D006742': NtO(MH='Hospital Bed Capacity', MS='The number of beds which a hospital has been designed and constructed to contain. It may also refer to the number of beds set up and staffed for use.'),
  'D006743': NtO(MH='Hospital Bed Capacity, under 100', MS='The number of beds under 100 which a hospital has been designed and constructed to contain. It may also refer to the number of beds set up and staffed for use.'),
  'D006744': NtO(MH='Hospital Bed Capacity, 100 to 299', MS=None),
  'D006745': NtO(MH='Hospital Bed Capacity, 300 to 499', MS='The number of beds from 300 to 499 which a hospital has been designed and constructed to contain. It may also refer to the number of beds set up and staffed for use.'),
  'D006746': NtO(MH='Hospital Bed Capacity, 500 and over', MS='The number of beds 500 or more which a hospital has been designed and constructed to contain. It may also refer to the number of beds set up and staffed for use.'),
  'D006747': NtO(MH='Hospital Communication Systems', MS='The transmission of messages to staff and patients within a hospital.'),
  'D006748': NtO(MH='Hospital Departments', MS='Major administrative divisions of the hospital.'),
  'D006749': NtO(MH='Hospital Design and Construction', MS='The architecture, functional design, and construction of hospitals.'),
  'D006750': NtO(MH='Hospital Distribution Systems', MS='Systems for delivering hospital supplies, food, laundry, etc., to patient care areas.'),
  'D006751': NtO(MH='Hospital Information Systems', MS='Integrated, computer-assisted systems designed to store, manipulate, and retrieve information concerned with the administrative and clinical aspects of providing medical services within the hospital.'),
  'D006752': NtO(MH='Hospital Planning', MS='Areawide planning for hospitals or planning of a particular hospital unit on the basis of projected consumer need. This does not include hospital design and construction or architectural plans.'),
  'D006753': NtO(MH='Hospital Records', MS='Compilations of data on hospital activities and programs; excludes patient medical records.'),
  'D006754': NtO(MH='Hospital Restructuring', MS='Reorganization of the hospital corporate structure.'),
  'D006755': NtO(MH='Hospital Shared Services', MS='Cooperation among hospitals for the purpose of sharing various departmental services, e.g., pharmacy, laundry, data processing, etc.'),
  'D006756': NtO(MH='Hospital Shops', MS='Stores located in hospitals selling merchandise or services for the convenience of patients, staff, and visitors.'),
  'D006757': NtO(MH='Hospital Units', MS='Those areas of the hospital organization not considered departments which provide specialized patient care. They include various hospital special care wards.'),
  'D006758': NtO(MH='Hospital Volunteers', MS='Individuals who donate their services to the hospital.'),
  'D006759': NtO(MH='Hospital-Physician Joint Ventures', MS='A formal financial agreement made between one or more physicians and a hospital to provide ambulatory alternative services to those patients who do not require hospitalization.'),
  'D006760': NtO(MH='Hospitalization', MS='The confinement of a patient in a hospital.'),
  'D006761': NtO(MH='Hospitals', MS='Institutions with an organized medical staff which provide medical care to patients.'),
  'D006762': NtO(MH='Hospitals, Animal', MS='Facility for in-house medical care of animals.'),
  'D006763': NtO(MH='Hospitals, Chronic Disease', MS='Hospitals which provide care to patients with long-term illnesses.'),
  'D006764': NtO(MH='Hospitals, Community', MS='Institutions with permanent facilities and organized medical staff which provide the full range of hospital services primarily to a neighborhood area.'),
  'D006765': NtO(MH='Hospitals, Convalescent', MS='Hospitals which provide care to the patient for the period following an acute illness until health is restored.'),
  'D006766': NtO(MH='Hospitals, County', MS='Hospitals controlled by the county government.'),
  'D006767': NtO(MH='Hospitals, District', MS='Government-controlled hospitals which represent the major health facility for a designated geographic area.'),
  'D006768': NtO(MH='Hospitals, Federal', MS='Hospitals controlled by agencies and departments of the U.S. federal government.'),
  'D006769': NtO(MH='Hospitals, General', MS='Large hospitals with a resident medical staff which provides continuous care to maternity, surgical and medical patients.'),
  'D006770': NtO(MH='Hospitals, Group Practice', MS='Hospitals organized and controlled by a group of physicians who practice together and provide each other with mutual support.'),
  'D006771': NtO(MH='Hospitals, Maternity', MS='Special hospitals which provide care to women during pregnancy and parturition.'),
  'D006772': NtO(MH='Hospitals, Military', MS='Hospitals which provide care for the military personnel and usually for their dependents.'),
  'D006773': NtO(MH='Hospitals, Municipal', MS='Hospitals controlled by the city government.'),
  'D006774': NtO(MH='Hospitals, Osteopathic', MS='Hospitals providing care utilizing the generally accepted medical and surgical methods but with emphasis on the osteopathic system of therapy.'),
  'D006775': NtO(MH='Hospitals, Packaged', MS='Hospital equipment and supplies, packaged for long-term storage, sufficient to set up a general hospital in an emergency situation. They are also called Packaged Disaster Hospitals and formerly Civil Defense Emergency Hospitals.'),
  'D006776': NtO(MH='Hospitals, Pediatric', MS='Special hospitals which provide care for ill children.'),
  'D006777': NtO(MH='Hospitals, Proprietary', MS='Hospitals owned and operated by a corporation or an individual that operate on a for-profit basis, also referred to as investor-owned hospitals.'),
  'D006778': NtO(MH='Hospitals, Psychiatric', MS='Special hospitals which provide care to the mentally ill patient.'),
  'D006779': NtO(MH='Hospitals, Public', MS='Hospitals controlled by various types of government, i.e., city, county, district, state or federal.'),
  'D006780': NtO(MH='Hospitals, Rural', MS='Hospitals located in a rural area.'),
  'D006781': NtO(MH='Hospitals, Satellite', MS='Those hospitals which are extensions of a main hospital and are wholly or partly administered by that hospital.'),
  'D006782': NtO(MH='Hospitals, Special', MS='Hospitals which provide care for a single category of illness with facilities and staff directed toward a specific service.'),
  'D006783': NtO(MH='Hospitals, State', MS='Hospitals controlled by agencies and departments of the state government.'),
  'D006784': NtO(MH='Hospitals, Teaching', MS='Hospitals engaged in educational and research programs, as well as providing medical care to the patients.'),
  'D006785': NtO(MH='Hospitals, University', MS='Hospitals maintained by a university for the teaching of medical students, postgraduate training programs, and clinical research.'),
  'D006786': NtO(MH='Hospitals, Urban', MS='Hospitals located in metropolitan areas.'),
  'D006787': NtO(MH='Hospitals, Veterans', MS='Hospitals providing medical care to veterans of wars.'),
  'D006788': NtO(MH='Hospitals, Voluntary', MS='Private, not-for-profit hospitals that are autonomous, self-established, and self-supported.'),
  'D006789': NtO(MH='Host vs Graft Reaction', MS='The immune responses of a host to a graft. A specific response is GRAFT REJECTION.'),
  'D006790': NtO(MH='Host-Parasite Interactions', MS='The relationship between an invertebrate and another organism (the host), one of which lives at the expense of the other. Traditionally excluded from definition of parasites are pathogenic BACTERIA; FUNGI; VIRUSES; and PLANTS; though they may live parasitically.'),
  'D006791': NtO(MH='Hostility', MS='Tendency to feel anger toward and to seek to inflict harm upon a person or group.'),
  'D006792': NtO(MH='House Calls', MS="Visits to the patient's home by professional personnel for the purpose of diagnosis and/or treatment."),
  'D006793': NtO(MH='Houseflies', MS='Flies of the species Musca domestica (family MUSCIDAE), which infest human habitations throughout the world and often act as carriers of pathogenic organisms.'),
  'D006794': NtO(MH='Household Articles', MS='Various material objects and items in the home. It includes temporary or permanent machinery and appliances. It does not include furniture or interior furnishings (FURNITURE see INTERIOR DESIGN AND FURNISHINGS; INTERIOR FURNISHINGS see INTERIOR DESIGN AND FURNISHINGS).'),
  'D006795': NtO(MH='Household Products', MS='Substances or materials used in the course of housekeeping or personal routine.'),
  'D006796': NtO(MH='Housekeeping', MS='The care and management of property.'),
  'D006797': NtO(MH='Housekeeping, Hospital', MS='Hospital department which manages and provides the required housekeeping functions in all areas of the hospital.'),
  'D006798': NtO(MH='Housing', MS='Living facilities for humans.'),
  'D006799': NtO(MH='Housing, Animal', MS='The physical environment in which animals are maintained.'),
  'D006800': NtO(MH='Deltaretrovirus Infections', MS='Infections caused by the HTLV or BLV deltaretroviruses. They include human T-cell leukemia-lymphoma (LEUKEMIA-LYMPHOMA, T-CELL, ACUTE, HTLV-I-ASSOCIATED).'),
  'D006801': NtO(MH='Humans', MS='Members of the species Homo sapiens.'),
  'D006802': NtO(MH='Human Activities', MS='Activities performed by humans.'),
  'D006803': NtO(MH='Human Development', MS='Continuous sequential changes which occur in the physiological and psychological functions during the life-time of an individual.'),
  'D006804': NtO(MH='Ergonomics', MS='The science of designing, building or equipping mechanical devices or artificial environments to the anthropometric, physiological, or psychological requirements of the people who will use them.'),
  'D006805': NtO(MH='Human Experimentation', MS='The use of humans as investigational subjects.'),
  'D006806': NtO(MH='Human Rights', MS='The right of the individual to cultural, social, economic, and educational opportunities as provided by society, e.g., right to work, right to education, and right to social security.'),
  'D006808': NtO(MH='Humanism', MS='An ethical system which emphasizes human values and the personal worth of each individual, as well as concern for the dignity and freedom of humankind.'),
  'D006809': NtO(MH='Humanities', MS='Fields of inquiry in human constructs and concerns as opposed to natural processes and social relations. These are traditionally the study of literature, philosophy, and religion.'),
  'D006810': NtO(MH='Humeral Fractures', MS='Fractures of the HUMERUS.'),
  'D006811': NtO(MH='Humerus', MS='Bone in humans and primates extending from the SHOULDER JOINT to the ELBOW JOINT.'),
  'D006812': NtO(MH='Humic Substances', MS='Organic matter in a state of advanced decay, after passing through the stages of COMPOST and PEAT and before becoming lignite (COAL). It is composed of a heterogenous mixture of compounds including phenolic radicals and acids that polymerize and are not easily separated nor analyzed. (E.A. Ghabbour & G. Davies, eds. Humic Substances, 2001).'),
  'D006813': NtO(MH='Humidity', MS='A measure of the amount of WATER VAPOR in the air.'),
  'D006814': NtO(MH='Hungary', MS='A country in Europe, northwest of Romania, south of Slovakia, and east of Austria. The capital is Budapest.'),
  'D006815': NtO(MH='Hunger', MS='The desire for FOOD generated by a sensation arising from the lack of food in the STOMACH.'),
  'D006816': NtO(MH='Huntington Disease', MS='A familial disorder inherited as an autosomal dominant trait and characterized by the onset of progressive CHOREA and DEMENTIA in the fourth or fifth decade of life. Common initial manifestations include paranoia; poor impulse control; DEPRESSION; HALLUCINATIONS; and DELUSIONS. Eventually intellectual impairment; loss of fine motor control; ATHETOSIS; and diffuse chorea involving axial and limb musculature develops, leading to a vegetative state within 10-15 years of disease onset. The juvenile variant has a more fulminant course including SEIZURES; ATAXIA; dementia; and chorea. (From Adams et al., Principles of Neurology, 6th ed, pp1060-4)'),
  'D006818': NtO(MH='Hyalin', MS='A clear, homogenous, structureless, eosinophilic substance occurring in pathological degeneration of tissues.'),
  'D006819': NtO(MH='Hyaline Membrane Disease', MS='A respiratory distress syndrome in newborn infants, usually premature infants with insufficient PULMONARY SURFACTANTS. The disease is characterized by the formation of a HYALINE-like membrane lining the terminal respiratory airspaces (PULMONARY ALVEOLI) and subsequent collapse of the lung (PULMONARY ATELECTASIS).'),
  'D006820': NtO(MH='Hyaluronic Acid', MS='A natural high-viscosity mucopolysaccharide with alternating beta (1-3) glucuronide and beta (1-4) glucosaminidic bonds. It is found in the UMBILICAL CORD, in VITREOUS BODY and in SYNOVIAL FLUID. A high urinary level is found in PROGERIA.'),
  'D006821': NtO(MH='Hyaluronoglucosaminidase', MS='An enzyme that catalyzes the random hydrolysis of 1,4-linkages between N-acetyl-beta-D-glucosamine and D-glucuronate residues in hyaluronate. (From Enzyme Nomenclature, 1992) There has been use as ANTINEOPLASTIC AGENTS to limit NEOPLASM METASTASIS.'),
  'D006822': NtO(MH='Hybrid Cells', MS='Any cell, other than a ZYGOTE, that contains elements (such as NUCLEI and CYTOPLASM) from two or more different cells, usually produced by artificial CELL FUSION.'),
  'D006823': NtO(MH='Hybrid Vigor', MS='The adaptive superiority of the heterozygous GENOTYPE with respect to one or more characters in comparison with the corresponding HOMOZYGOTE.'),
  'D006824': NtO(MH='Hybridization, Genetic', MS='The genetic process of crossbreeding between genetically dissimilar parents to produce a hybrid.'),
  'D006825': NtO(MH='Hybridomas', MS='Cells artificially created by fusion of activated lymphocytes with neoplastic cells. The resulting hybrid cells are cloned and produce pure MONOCLONAL ANTIBODIES or T-cell products, identical to those produced by the immunologically competent parent cell.'),
  'D006826': NtO(MH='Hycanthone', MS='Potentially toxic, but effective antischistosomal agent, it is a metabolite of LUCANTHONE.'),
  'D006827': NtO(MH='Hydantoins', MS='Compounds based on imidazolidine dione. Some derivatives are ANTICONVULSANTS.'),
  'D006828': NtO(MH='Hydatidiform Mole', MS='Trophoblastic hyperplasia associated with normal gestation, or molar pregnancy. It is characterized by the swelling of the CHORIONIC VILLI and elevated human CHORIONIC GONADOTROPIN. Hydatidiform moles or molar pregnancy may be categorized as complete or partial based on their gross morphology, histopathology, and karyotype.'),
  'D006829': NtO(MH='Hydra', MS='A genus of freshwater polyps in the family Hydridae, order Hydroida, class HYDROZOA. They are of special interest because of their complex organization and because their adult organization corresponds roughly to the gastrula of higher animals.'),
  'D006830': NtO(MH='Hydralazine', MS='A direct-acting vasodilator that is used as an antihypertensive agent.'),
  'D006831': NtO(MH='Polyhydramnios', MS='A condition of abnormally high AMNIOTIC FLUID volume, such as greater than 2,000 ml in the LAST TRIMESTER and usually diagnosed by ultrasonographic criteria (AMNIOTIC FLUID INDEX). It is associated with maternal DIABETES MELLITUS; MULTIPLE PREGNANCY; CHROMOSOMAL DISORDERS; and congenital abnormalities.'),
  'D006832': NtO(MH='Hydranencephaly', MS='A congenital condition where the greater portions of the cerebral hemispheres and CORPUS STRIATUM are replaced by CSF and glial tissue. The meninges and the skull are well formed, which is consistent with earlier normal embryogenesis of the telencephalon. Bilateral occlusions of the internal carotid arteries in utero is a potential mechanism. Clinical features include intact brainstem reflexes without evidence of higher cortical activity. (Menkes, Textbook of Child Neurology, 5th ed, p307)'),
  'D006833': NtO(MH='Hydrarthrosis', MS='Accumulation of watery fluid in the cavity of a joint. (Dorland, 27th ed)'),
  'D006834': NtO(MH='Hydrazines', MS='Substituted derivatives of hydrazine (formula H2N-NH2).'),
  'D006835': NtO(MH='Hydrazones', MS="Compounds of the general formula R:N.NR2, as resulting from the action of hydrazines with aldehydes or ketones. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D006836': NtO(MH='Hydro-Lyases', MS='Enzymes that catalyze the breakage of a carbon-oxygen bond leading to unsaturated products via the removal of water. EC 4.2.1.'),
  'D006837': NtO(MH='Hydroa Vacciniforme', MS='A vesicular and bullous eruption having a tendency to recur in summer during childhood and commonly appearing on sun-exposed skin. The lesions are surrounded by an erythematous zone and resemble a vaccination. (From Dorland, 27th ed)'),
  'D006838': NtO(MH='Hydrocarbons', MS='Organic compounds that primarily contain carbon and hydrogen atoms with the carbon atoms forming a linear or circular structure.'),
  'D006839': NtO(MH='Hydrocarbons, Acyclic', MS='Organic compounds composed exclusively of carbon and hydrogen where no carbon atoms join to form a ring structure.'),
  'D006840': NtO(MH='Hydrocarbons, Alicyclic', MS='Organic compounds composed exclusively of carbon and hydrogen. Three or more carbon atoms are arranged in a cyclic structure and they possess aliphatic properties.'),
  'D006841': NtO(MH='Hydrocarbons, Aromatic', MS='Organic compounds containing carbon and hydrogen in the form of an unsaturated, usually hexagonal ring structure. The compounds can be single ring, or double, triple, or multiple fused rings.'),
  'D006842': NtO(MH='Hydrocarbons, Brominated', MS='Hydrocarbon compounds with one or more HYDROGEN atoms substituted with BROMINE.'),
  'D006843': NtO(MH='Hydrocarbons, Chlorinated', MS='Hydrocarbon compounds with one or more of the hydrogens replaced by CHLORINE.'),
  'D006844': NtO(MH='Hydrocarbons, Cyclic', MS='Organic compounds composed exclusively of carbon and hydrogen forming a closed ring that may be either alicyclic or aromatic.'),
  'D006845': NtO(MH='Hydrocarbons, Fluorinated', MS='Inert liquid or gaseous halocarbon compounds in which FLUORINE replaces some or all HYDROGEN atoms.'),
  'D006846': NtO(MH='Hydrocarbons, Halogenated', MS='Hydrocarbon compounds with one or more HYDROGEN atoms substituted with HALOGENS.'),
  'D006847': NtO(MH='Hydrocarbons, Iodinated', MS='Hydrocarbon compounds with one or more HYDROGEN atoms substituted with IODINE.'),
  'D006848': NtO(MH='Testicular Hydrocele', MS='Accumulation of serous fluid between the layers of membrane (tunica vaginalis) covering the TESTIS in the SCROTUM.'),
  'D006849': NtO(MH='Hydrocephalus', MS='Excessive accumulation of cerebrospinal fluid within the cranium which may be associated with dilation of cerebral ventricles, INTRACRANIAL HYPERTENSION; HEADACHE; lethargy; URINARY INCONTINENCE; and ATAXIA.'),
  'D006850': NtO(MH='Hydrocephalus, Normal Pressure', MS='A form of compensated hydrocephalus characterized clinically by a slowly progressive gait disorder (see GAIT DISORDERS, NEUROLOGIC), progressive intellectual decline, and URINARY INCONTINENCE. Spinal fluid pressure tends to be in the high normal range. This condition may result from processes which interfere with the absorption of CSF including SUBARACHNOID HEMORRHAGE, chronic MENINGITIS, and other conditions. (From Adams et al., Principles of Neurology, 6th ed, pp631-3)'),
  'D006851': NtO(MH='Hydrochloric Acid', MS='A strong corrosive acid that is commonly used as a laboratory reagent. It is formed by dissolving hydrogen chloride in water. GASTRIC ACID is the hydrochloric acid component of GASTRIC JUICE.'),
  'D006852': NtO(MH='Hydrochlorothiazide', MS='A thiazide diuretic often considered the prototypical member of this class. It reduces the reabsorption of electrolytes from the renal tubules. This results in increased excretion of water and electrolytes, including sodium, potassium, chloride, and magnesium. It is used in the treatment of several disorders including edema, hypertension, diabetes insipidus, and hypoparathyroidism.'),
  'D006853': NtO(MH='Hydrocodone', MS='Narcotic analgesic related to CODEINE, but more potent and more addicting by weight. It is used also as cough suppressant.'),
  'D006854': NtO(MH='Hydrocortisone', MS='The main glucocorticoid secreted by the ADRENAL CORTEX. Its synthetic counterpart is used, either as an injection or topically, in the treatment of inflammation, allergy, collagen diseases, asthma, adrenocortical deficiency, shock, and some neoplastic conditions.'),
  'D006856': NtO(MH='Hydrogen Cyanide', MS='Hydrogen cyanide (HCN); A toxic liquid or colorless gas. It is found in the smoke of various tobacco products and released by combustion of nitrogen-containing organic materials.'),
  'D006857': NtO(MH='Hydroflumethiazide', MS='A thiazide diuretic with actions and uses similar to those of HYDROCHLOROTHIAZIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p822)'),
  'D006858': NtO(MH='Hydrofluoric Acid', MS='Hydrofluoric acid. A solution of hydrogen fluoride in water. It is a colorless fuming liquid which can cause painful burns.'),
  'D006859': NtO(MH='Hydrogen', MS='The first chemical element in the periodic table with atomic symbol H, and atomic number 1. Protium (atomic weight 1) is by far the most common hydrogen isotope. Hydrogen also exists as the stable isotope DEUTERIUM (atomic weight 2) and the radioactive isotope TRITIUM (atomic weight 3). Hydrogen forms into a diatomic molecule at room temperature and appears as a highly flammable colorless and odorless gas.'),
  'D006860': NtO(MH='Hydrogen Bonding', MS='A low-energy attractive force between hydrogen and another element. It plays a major role in determining the properties of water, proteins, and other compounds.'),
  'D006861': NtO(MH='Hydrogen Peroxide', MS='A strong oxidizing agent used in aqueous solution as a ripening agent, bleach, and topical anti-infective. It is relatively unstable and solutions deteriorate over time unless stabilized by the addition of acetanilide or similar organic materials.'),
  'D006862': NtO(MH='Hydrogen Sulfide', MS='A flammable, poisonous gas with a characteristic odor of rotten eggs. It is used in the manufacture of chemicals, in metallurgy, and as an analytical reagent. (From Merck Index, 11th ed)'),
  'D006863': NtO(MH='Hydrogen-Ion Concentration', MS='The normality of a solution with respect to HYDROGEN ions; H+. It is related to acidity measurements in most cases by pH = log 1/2[1/(H+)], where (H+) is the hydrogen ion concentration in gram equivalents per liter of solution. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D006864': NtO(MH='Hydrogenase', MS='An enzyme found in bacteria. It catalyzes the reduction of FERREDOXIN and other substances in the presence of molecular hydrogen and is involved in the electron transport of bacterial photosynthesis.'),
  'D006865': NtO(MH='Hydrogenation', MS='Addition of hydrogen to a compound, especially to an unsaturated fat or fatty acid. (From Stedman, 26th ed)'),
  'D006867': NtO(MH='Hydrolases', MS='Any member of the class of enzymes that catalyze the cleavage of the substrate and the addition of water to the resulting molecules, e.g., ESTERASES, glycosidases (GLYCOSIDE HYDROLASES), lipases, NUCLEOTIDASES, peptidases (PEPTIDE HYDROLASES), and phosphatases (PHOSPHORIC MONOESTER HYDROLASES). EC 3.'),
  'D006868': NtO(MH='Hydrolysis', MS='The process of cleaving a chemical compound by the addition of a molecule of water.'),
  'D006869': NtO(MH='Hydronephrosis', MS='Abnormal enlargement or swelling of a KIDNEY due to dilation of the KIDNEY CALICES and the KIDNEY PELVIS. It is often associated with obstruction of the URETER or chronic kidney diseases that prevents normal drainage of urine into the URINARY BLADDER.'),
  'D006871': NtO(MH='Hydrophthalmos', MS='Congenital open-angle glaucoma that results from dysgenesis of the angle structures accompanied by increased intraocular pressure and enlargement of the eye. Treatment is both medical and surgical.'),
  'D006872': NtO(MH='Hydropneumothorax', MS='A collection of fluid and gas within the pleural cavity. (Dorland, 27th ed)'),
  'D006873': NtO(MH='Hydroquinones', MS='Derivatives of hydroquinone (1,4-dihydrobenzene) made by reduction of BENZOQUINONES.'),
  'D006874': NtO(MH='Hydrostatic Pressure', MS='The pressure due to the weight of fluid.'),
  'D006875': NtO(MH='Hydrotherapy', MS='External application of water for therapeutic purposes.'),
  'D006876': NtO(MH='Hydrothorax', MS='A collection of watery fluid in the pleural cavity. (Dorland, 27th ed)'),
  'D006877': NtO(MH='Hydroxamic Acids', MS='A class of weak acids with the general formula R-CONHOH.'),
  'D006878': NtO(MH='Hydroxides', MS='Inorganic compounds that contain the OH- group.'),
  'D006879': NtO(MH='Hydroxocobalamin', MS='Injectable form of VITAMIN B 12 that has been used therapeutically to treat VITAMIN B 12 DEFICIENCY.'),
  'D006880': NtO(MH='Hydroxy Acids', MS='Organic compounds containing both the hydroxyl and carboxyl radicals.'),
  'D006881': NtO(MH='Hydroxyacetylaminofluorene', MS='A N-hydroxylated derivative of 2-ACETYLAMINOFLUORENE that has demonstrated carcinogenic action.'),
  'D006882': NtO(MH='Hydroxyapatites', MS="A group of compounds with the general formula M10(PO4)6(OH)2, where M is barium, strontium, or calcium. The compounds are the principal mineral in phosphorite deposits, biological tissue, human bones, and teeth. They are also used as an anticaking agent and polymer catalysts. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D006884': NtO(MH='Hydroxybutyrate Dehydrogenase', MS=None),
  'D006885': NtO(MH='Hydroxybutyrates', MS='Salts and esters of hydroxybutyric acid.'),
  'D006886': NtO(MH='Hydroxychloroquine', MS="A chemotherapeutic agent that acts against erythrocytic forms of malarial parasites. Hydroxychloroquine appears to concentrate in food vacuoles of affected protozoa. It inhibits plasmodial heme polymerase. (From Gilman et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 9th ed, p970)"),
  'D006887': NtO(MH='Hydroxycholecalciferols', MS='Hydroxy analogs of vitamin D 3; (CHOLECALCIFEROL); including CALCIFEDIOL; CALCITRIOL; and 24,25-DIHYDROXYVITAMIN D 3.'),
  'D006888': NtO(MH='Hydroxycholesterols', MS='Cholesterol which is substituted by a hydroxy group in any position.'),
  'D006889': NtO(MH='Hydroxycorticosteroids', MS='A group of corticosteroids carrying hydroxy groups, usually in the 11- or 17-positions. They comprise the bulk of the corticosteroids used systemically. As they are relatively insoluble in water, salts of various esterified forms are often used for injections or solutions.'),
  'D006892': NtO(MH='Hydroxydopamines', MS='Dopamines with a hydroxy group substituted in one or more positions.'),
  'D006893': NtO(MH='Hydroxyeicosatetraenoic Acids', MS='Eicosatetraenoic acids substituted in any position by one or more hydroxy groups. They are important intermediates in a series of biosynthetic processes leading from arachidonic acid to a number of biologically active compounds such as prostaglandins, thromboxanes, and leukotrienes.'),
  'D006894': NtO(MH='Hydroxyestrones', MS='Estrone derivatives substituted with one or more hydroxyl groups in any position. They are important metabolites of estrone and other estrogens.'),
  'D006895': NtO(MH='Hydroxyethyl Starch Derivatives', MS='Starches that have been chemically modified so that a percentage of OH groups are substituted with 2-hydroxyethyl ether groups.'),
  'D006896': NtO(MH='Hydroxyethylrutoside', MS='Monohydroxyethyl derivative of rutin. Peripheral circulation stimulant used in treatment of venous disorders.'),
  'D006897': NtO(MH='Hydroxyindoleacetic Acid', MS=None),
  'D006898': NtO(MH='Hydroxylamines', MS='Organic compounds that contain the (-NH2OH) radical.'),
  'D006899': NtO(MH='Mixed Function Oxygenases', MS='Widely distributed enzymes that carry out oxidation-reduction reactions in which one atom of the oxygen molecule is incorporated into the organic substrate; the other oxygen atom is reduced and combined with hydrogen ions to form water. They are also known as monooxygenases or hydroxylases. These reactions require two substrates as reductants for each of the two oxygen atoms. There are different classes of monooxygenases depending on the type of hydrogen-providing cosubstrate (COENZYMES) required in the mixed-function oxidation.'),
  'D006900': NtO(MH='Hydroxylation', MS='Placing of a hydroxyl group on a compound in a position where one did not exist before. (Stedman, 26th ed)'),
  'D006901': NtO(MH='Hydroxylysine', MS='A hydroxylated derivative of the amino acid LYSINE that is present in certain collagens.'),
  'D006902': NtO(MH='Hydroxymercuribenzoates', MS='Hydroxylated benzoic acid derivatives that contain mercury. Some of these are used as sulfhydryl reagents in biochemical studies.'),
  'D006903': NtO(MH='Hydroxymethylglutaryl CoA Reductases', MS='Enzymes that catalyze the reversible reduction of alpha-carboxyl group of 3-hydroxy-3-methylglutaryl-coenzyme A to yield MEVALONIC ACID.'),
  'D006904': NtO(MH='Hydroxymethylglutaryl-CoA Synthase', MS='An enzyme that catalyzes the synthesis of hydroxymethylglutaryl-CoA from acetyl-CoA and acetoacetyl-CoA. This is a key enzyme in steroid biosynthesis. This enzyme was formerly listed as EC 4.1.3.5.'),
  'D006906': NtO(MH='Hydroxyphenylazouracil', MS='Inhibitor of DNA replication in gram-positive bacteria.'),
  'D006907': NtO(MH='17-alpha-Hydroxypregnenolone', MS='A 21-carbon steroid that is converted from PREGNENOLONE by STEROID 17-ALPHA-HYDROXYLASE. It is an intermediate in the delta-5 pathway of biosynthesis of GONADAL STEROID HORMONES and the adrenal CORTICOSTEROIDS.'),
  'D006908': NtO(MH='Hydroxyprogesterones', MS='Metabolites or derivatives of PROGESTERONE with hydroxyl group substitution at various sites.'),
  'D006909': NtO(MH='Hydroxyproline', MS='A hydroxylated form of the imino acid proline. A deficiency in ASCORBIC ACID can result in impaired hydroxyproline formation.'),
  'D006910': NtO(MH='Hydroxypropiophenone', MS=None),
  'D006911': NtO(MH='Hydroxyprostaglandin Dehydrogenases', MS='Catalyzes reversibly the oxidation of hydroxyl groups of prostaglandins.'),
  'D006912': NtO(MH='Hydroxyquinolines', MS='The 8-hydroxy derivatives inhibit various enzymes and their halogenated derivatives, though neurotoxic, are used as topical anti-infective agents, among other uses.'),
  'D006913': NtO(MH='Hydroxysteroid Dehydrogenases', MS='Enzymes of the oxidoreductase class that catalyze the dehydrogenation of hydroxysteroids. (From Enzyme Nomenclature, 1992) EC 1.1.-.'),
  'D006914': NtO(MH='Hydroxysteroids', MS='Steroids in which one or more hydroxy groups have been substituted for hydrogen atoms either within the ring skeleton or on any of the side chains.'),
  'D006915': NtO(MH='Hydroxytestosterones', MS='17 beta-Hydroxy-4-androsten-3-ones. Testosterone derivatives formed by the substitution of one or more hydroxyl groups in any position.'),
  'D006916': NtO(MH='5-Hydroxytryptophan', MS='The immediate precursor in the biosynthesis of SEROTONIN from tryptophan. It is used as an antiepileptic and antidepressant.'),
  'D006917': NtO(MH='Hydroxytryptophol', MS='5-Hydroxy-indole-3-ethanol.'),
  'D006918': NtO(MH='Hydroxyurea', MS='An antineoplastic agent that inhibits DNA synthesis through the inhibition of ribonucleoside diphosphate reductase.'),
  'D006919': NtO(MH='Hydroxyzine', MS='A histamine H1 receptor antagonist that is effective in the treatment of chronic urticaria, dermatitis, and histamine-mediated pruritus. Unlike its major metabolite CETIRIZINE, it does cause drowsiness. It is also effective as an antiemetic, for relief of anxiety and tension, and as a sedative.'),
  'D006920': NtO(MH='Hygiene', MS='The science dealing with the establishment and maintenance of health in the individual and the group. It includes the conditions and practices conducive to health. (Webster, 3d ed)'),
  'D006921': NtO(MH='Hygromycin B', MS='Aminoglycoside produced by Streptomyces hygroscopicus. It is used as an anthelmintic against swine infections by large roundworms, nodular worms, and whipworms.'),
  'D006922': NtO(MH='Hylobates', MS='A genus of the family HYLOBATIDAE consisting of nine species. The members of this genus inhabit rain forests in southeast Asia. They are arboreal and differ from other anthropoids in the great length of their arms and very slender bodies and limbs. Their major means of locomotion is by swinging from branch to branch by their arms. Hylobates means dweller in the trees.'),
  'D006923': NtO(MH='Hymecromone', MS='A coumarin derivative possessing properties as a spasmolytic, choleretic and light-protective agent. It is also used in ANALYTICAL CHEMISTRY TECHNIQUES for the determination of NITRIC ACID.'),
  'D006924': NtO(MH='Hymen', MS='A thin fold of MUCOUS MEMBRANE situated at the orifice of the vagina.'),
  'D006925': NtO(MH='Hymenolepiasis', MS='Infection with tapeworms of the genus Hymenolepis.'),
  'D006926': NtO(MH='Hymenolepis', MS='A genus of small tapeworms of birds and mammals.'),
  'D006927': NtO(MH='Hymenoptera', MS='An extensive order of highly specialized insects including bees, wasps, and ants.'),
  'D006928': NtO(MH='Hyoid Bone', MS='A mobile U-shaped bone that lies in the anterior part of the neck at the level of the third CERVICAL VERTEBRAE. The hyoid bone is suspended from the processes of the TEMPORAL BONES by ligaments, and is firmly bound to the THYROID CARTILAGE by muscles.'),
  'D006929': NtO(MH='Hyperaldosteronism', MS='A condition caused by the overproduction of ALDOSTERONE. It is characterized by sodium retention and potassium excretion with resultant HYPERTENSION and HYPOKALEMIA.'),
  'D006930': NtO(MH='Hyperalgesia', MS='An increased sensation of pain or discomfort produced by mimimally noxious stimuli due to damage to soft tissue containing NOCICEPTORS or injury to a peripheral nerve.'),
  'D006931': NtO(MH='Hyperbaric Oxygenation', MS='The therapeutic intermittent administration of oxygen in a chamber at greater than sea-level atmospheric pressures (three atmospheres). It is considered effective treatment for air and gas embolisms, smoke inhalation, acute carbon monoxide poisoning, caisson disease, clostridial gangrene, etc. (From Segen, Dictionary of Modern Medicine, 1992). The list of treatment modalities includes stroke.'),
  'D006932': NtO(MH='Hyperbilirubinemia', MS='A condition characterized by an abnormal increase of BILIRUBIN in the blood, which may result in JAUNDICE. Bilirubin, a breakdown product of HEME, is normally excreted in the BILE or further catabolized before excretion in the urine.'),
  'D006933': NtO(MH='Hyperbilirubinemia, Hereditary', MS='Inborn errors of bilirubin metabolism resulting in excessive amounts of bilirubin in the circulating blood, either because of increased bilirubin production or because of delayed clearance of bilirubin from the blood.'),
  'D006934': NtO(MH='Hypercalcemia', MS='Abnormally high level of calcium in the blood.'),
  'D006935': NtO(MH='Hypercapnia', MS='A clinical manifestation of abnormal increase in the amount of carbon dioxide in arterial blood.'),
  'D006936': NtO(MH='Hypercementosis', MS='A regressive change of teeth characterized by excessive development of secondary cementum on the tooth surface. It may occur on any part of the root, but the apical two-thirds are most commonly affected. (Dorland, 27th ed)'),
  'D006937': NtO(MH='Hypercholesterolemia', MS='A condition with abnormally high levels of CHOLESTEROL in the blood. It is defined as a cholesterol value exceeding the 95th percentile for the population.'),
  'D006938': NtO(MH='Hyperlipoproteinemia Type II', MS='A group of familial disorders characterized by elevated circulating cholesterol contained in either LOW-DENSITY LIPOPROTEINS alone or also in VERY-LOW-DENSITY LIPOPROTEINS (pre-beta lipoproteins).'),
  'D006939': NtO(MH='Hyperemesis Gravidarum', MS='Intractable VOMITING that develops in early PREGNANCY and persists. This can lead to DEHYDRATION and WEIGHT LOSS.'),
  'D006940': NtO(MH='Hyperemia', MS='The presence of an increased amount of blood in a body part or an organ leading to congestion or engorgement of blood vessels. Hyperemia can be due to increase of blood flow into the area (active or arterial), or due to obstruction of outflow of blood from the area (passive or venous).'),
  'D006941': NtO(MH='Hyperesthesia', MS='Increased sensitivity to cutaneous stimulation due to a diminished threshold or an increased response to stimuli.'),
  'D006942': NtO(MH='Hypergammaglobulinemia', MS='An excess of GAMMA-GLOBULINS in the serum due to chronic infections or PARAPROTEINEMIAS.'),
  'D006943': NtO(MH='Hyperglycemia', MS='Abnormally high BLOOD GLUCOSE level.'),
  'D006944': NtO(MH='Hyperglycemic Hyperosmolar Nonketotic Coma', MS='A serious complication of TYPE 2 DIABETES MELLITUS. It is characterized by extreme HYPERGLYCEMIA; DEHYDRATION; serum hyperosmolarity; and depressed consciousness leading to COMA in the absence of KETOSIS and ACIDOSIS.'),
  'D006945': NtO(MH='Hyperhidrosis', MS='Excessive sweating. In the localized type, the most frequent sites are the palms, soles, axillae, inguinal folds, and the perineal area. Its chief cause is thought to be emotional. Generalized hyperhidrosis may be induced by a hot, humid environment, by fever, or by vigorous exercise.'),
  'D006946': NtO(MH='Hyperinsulinism', MS='A syndrome with excessively high INSULIN levels in the BLOOD. It may cause HYPOGLYCEMIA. Etiology of hyperinsulinism varies, including hypersecretion of a beta cell tumor (INSULINOMA); autoantibodies against insulin (INSULIN ANTIBODIES); defective insulin receptor (INSULIN RESISTANCE); or overuse of exogenous insulin or HYPOGLYCEMIC AGENTS.'),
  'D006947': NtO(MH='Hyperkalemia', MS='Abnormally high potassium concentration in the blood, most often due to defective renal excretion. It is characterized clinically by electrocardiographic abnormalities (elevated T waves and depressed P waves, and eventually by atrial asystole). In severe cases, weakness and flaccid paralysis may occur. (Dorland, 27th ed)'),
  'D006948': NtO(MH='Hyperkinesis', MS='Excessive movement of muscles of the body as a whole, which may be associated with organic or psychological disorders.'),
  'D006949': NtO(MH='Hyperlipidemias', MS='Conditions with excess LIPIDS in the blood.'),
  'D006950': NtO(MH='Hyperlipidemia, Familial Combined', MS='A type of familial lipid metabolism disorder characterized by a variable pattern of elevated plasma CHOLESTEROL and/or TRIGLYCERIDES. Multiple genes on different chromosomes may be involved, such as the major late transcription factor (UPSTREAM STIMULATORY FACTORS) on CHROMOSOME 1.'),
  'D006951': NtO(MH='Hyperlipoproteinemias', MS='Conditions with abnormally elevated levels of LIPOPROTEINS in the blood. They may be inherited, acquired, primary, or secondary. Hyperlipoproteinemias are classified according to the pattern of lipoproteins on electrophoresis or ultracentrifugation.'),
  'D006952': NtO(MH='Hyperlipoproteinemia Type III', MS='An autosomal recessively inherited disorder characterized by the accumulation of intermediate-density lipoprotein (IDL or broad-beta-lipoprotein). IDL has a CHOLESTEROL to TRIGLYCERIDES ratio greater than that of VERY-LOW-DENSITY LIPOPROTEINS. This disorder is due to mutation of APOLIPOPROTEINS E, a receptor-binding component of VLDL and CHYLOMICRONS, resulting in their reduced clearance and high plasma levels of both cholesterol and triglycerides.'),
  'D006953': NtO(MH='Hyperlipoproteinemia Type IV', MS='A hypertriglyceridemia disorder, often with autosomal dominant inheritance. It is characterized by the persistent elevations of plasma TRIGLYCERIDES, endogenously synthesized and contained predominantly in VERY-LOW-DENSITY LIPOPROTEINS (pre-beta lipoproteins). In contrast, the plasma CHOLESTEROL and PHOSPHOLIPIDS usually remain within normal limits.'),
  'D006954': NtO(MH='Hyperlipoproteinemia Type V', MS='A severe type of hyperlipidemia, sometimes familial, that is characterized by the elevation of both plasma CHYLOMICRONS and TRIGLYCERIDES contained in VERY-LOW-DENSITY LIPOPROTEINS. Type V hyperlipoproteinemia is often associated with DIABETES MELLITUS and is not caused by reduced LIPOPROTEIN LIPASE activity as in HYPERLIPOPROTEINEMIA TYPE I .'),
  'D006955': NtO(MH='Hypernatremia', MS='Excessive amount of sodium in the blood. (Dorland, 27th ed)'),
  'D006956': NtO(MH='Hyperopia', MS='A refractive error in which rays of light entering the eye parallel to the optic axis are brought to a focus behind the retina, as a result of the eyeball being too short from front to back. It is also called farsightedness because the near point is more distant than it is in emmetropia with an equal amplitude of accommodation. (Dorland, 27th ed)'),
  'D006957': NtO(MH='Hyperostosis Frontalis Interna', MS='Thickening of the inner table of the frontal bone, which may be associated with hypertrichosis and obesity. It most commonly affects women near menopause.'),
  'D006958': NtO(MH='Hyperostosis, Cortical, Congenital', MS='A disease of young infants characterized by soft tissue swellings over the affected bones, fever, and irritability, and marked by periods of remission and exacerbation. (Dorland, 27th ed)'),
  'D006959': NtO(MH='Hyperoxaluria', MS='Excretion of an excessive amount of OXALATES in the urine.'),
  'D006960': NtO(MH='Hyperoxaluria, Primary', MS='A genetic disorder characterized by excretion of large amounts of OXALATES in urine; NEPHROLITHIASIS; NEPHROCALCINOSIS; early onset of RENAL FAILURE; and often a generalized deposit of CALCIUM OXALATE. There are subtypes classified by the enzyme defects in glyoxylate metabolism.'),
  'D006961': NtO(MH='Hyperparathyroidism', MS='A condition of abnormally elevated output of PARATHYROID HORMONE (or PTH) triggering responses that increase blood CALCIUM. It is characterized by HYPERCALCEMIA and BONE RESORPTION, eventually leading to bone diseases. PRIMARY HYPERPARATHYROIDISM is caused by parathyroid HYPERPLASIA or PARATHYROID NEOPLASMS. SECONDARY HYPERPARATHYROIDISM is increased PTH secretion in response to HYPOCALCEMIA, usually caused by chronic KIDNEY DISEASES.'),
  'D006962': NtO(MH='Hyperparathyroidism, Secondary', MS='Abnormally elevated PARATHYROID HORMONE secretion as a response to HYPOCALCEMIA. It is caused by chronic KIDNEY FAILURE or other abnormalities in the controls of bone and mineral metabolism, leading to various BONE DISEASES, such as RENAL OSTEODYSTROPHY.'),
  'D006963': NtO(MH='Hyperphagia', MS='Ingestion of a greater than optimal quantity of food.'),
  'D006964': NtO(MH='Hyperpituitarism', MS='Disease of the glandular, anterior portion of the pituitary (PITUITARY GLAND, ANTERIOR) resulting in hypersecretion of ADENOHYPOPHYSEAL HORMONES such as GROWTH HORMONE; PROLACTIN; THYROTROPIN; LUTEINIZING HORMONE; FOLLICLE STIMULATING HORMONE ; and ADRENOCORTICOTROPIC HORMONE. Hyperpituitarism usually is caused by a functional ADENOMA.'),
  'D006965': NtO(MH='Hyperplasia', MS='An increase in the number of cells in a tissue or organ without tumor formation. It differs from HYPERTROPHY, which is an increase in bulk without an increase in the number of cells.'),
  'D006966': NtO(MH='Hyperprolactinemia', MS='Increased levels of PROLACTIN in the BLOOD, which may be associated with AMENORRHEA and GALACTORRHEA. Relatively common etiologies include PROLACTINOMA, medication effect, KIDNEY FAILURE, granulomatous diseases of the PITUITARY GLAND, and disorders which interfere with the hypothalamic inhibition of prolactin release. Ectopic (non-pituitary) production of prolactin may also occur. (From Joynt, Clinical Neurology, 1992, Ch36, pp77-8)'),
  'D006967': NtO(MH='Hypersensitivity', MS='Altered reactivity to an antigen, which can result in pathologic reactions upon subsequent exposure to that particular antigen.'),
  'D006968': NtO(MH='Hypersensitivity, Delayed', MS='An increased reactivity to specific antigens mediated not by antibodies but by cells.'),
  'D006969': NtO(MH='Hypersensitivity, Immediate', MS='Hypersensitivity reactions which occur within minutes of exposure to challenging antigen due to the release of histamine which follows the antigen-antibody reaction and causes smooth muscle contraction and increased vascular permeability.'),
  'D006970': NtO(MH='Disorders of Excessive Somnolence', MS='Disorders characterized by hypersomnolence during normal waking hours that may impair cognitive functioning. Subtypes include primary hypersomnia disorders (e.g., IDIOPATHIC HYPERSOMNOLENCE; NARCOLEPSY; and KLEINE-LEVIN SYNDROME) and secondary hypersomnia disorders where excessive somnolence can be attributed to a known cause (e.g., drug affect, MENTAL DISORDERS, and SLEEP APNEA SYNDROME). (From J Neurol Sci 1998 Jan 8;153(2):192-202; Thorpy, Principles and Practice of Sleep Medicine, 2nd ed, p320)'),
  'D006971': NtO(MH='Hypersplenism', MS='Condition characterized by splenomegaly, some reduction in the number of circulating blood cells in the presence of a normal or hyperactive bone marrow, and the potential for reversal by splenectomy.'),
  'D006972': NtO(MH='Hypertelorism', MS='Abnormal increase in the interorbital distance due to overdevelopment of the lesser wings of the sphenoid.'),
  'D006973': NtO(MH='Hypertension', MS='Persistently high systemic arterial BLOOD PRESSURE. Based on multiple readings (BLOOD PRESSURE DETERMINATION), hypertension is currently defined as when SYSTOLIC PRESSURE is consistently greater than 140 mm Hg or when DIASTOLIC PRESSURE is consistently 90 mm Hg or more.'),
  'D006974': NtO(MH='Hypertension, Malignant', MS='A condition of markedly elevated BLOOD PRESSURE with DIASTOLIC PRESSURE usually greater than 120 mm Hg. Malignant hypertension is characterized by widespread vascular damage, PAPILLEDEMA, retinopathy, HYPERTENSIVE ENCEPHALOPATHY, and renal dysfunction.'),
  'D006975': NtO(MH='Hypertension, Portal', MS='Abnormal increase of resistance to blood flow within the hepatic PORTAL SYSTEM, frequently seen in LIVER CIRRHOSIS and conditions with obstruction of the PORTAL VEIN.'),
  'D006976': NtO(MH='Hypertension, Pulmonary', MS='Increased VASCULAR RESISTANCE in the PULMONARY CIRCULATION, usually secondary to HEART DISEASES or LUNG DISEASES.'),
  'D006977': NtO(MH='Hypertension, Renal', MS='Persistent high BLOOD PRESSURE due to KIDNEY DISEASES, such as those involving the renal parenchyma, the renal vasculature, or tumors that secrete RENIN.'),
  'D006978': NtO(MH='Hypertension, Renovascular', MS='Hypertension due to RENAL ARTERY OBSTRUCTION or compression.'),
  'D006979': NtO(MH='Hyperthermia, Induced', MS='Abnormally high temperature intentionally induced in living things regionally or whole body. It is most often induced by radiation (heat waves, infra-red), ultrasound, or drugs.'),
  'D006980': NtO(MH='Hyperthyroidism', MS='Hypersecretion of THYROID HORMONES from the THYROID GLAND. Elevated levels of thyroid hormones increase BASAL METABOLIC RATE.'),
  'D006981': NtO(MH='Hyperthyroxinemia', MS='Abnormally elevated THYROXINE level in the BLOOD.'),
  'D006982': NtO(MH='Hypertonic Solutions', MS='Solutions that have a greater osmotic pressure than a reference solution such as blood, plasma, or interstitial fluid.'),
  'D006983': NtO(MH='Hypertrichosis', MS='Excessive hair growth at inappropriate locations, such as on the extremities, the head, and the back. It is caused by genetic or acquired factors, and is an androgen-independent process. This concept does not include HIRSUTISM which is an androgen-dependent excess hair growth in WOMEN and CHILDREN.'),
  'D006984': NtO(MH='Hypertrophy', MS='General increase in bulk of a part or organ due to CELL ENLARGEMENT and accumulation of FLUIDS AND SECRETIONS, not due to tumor formation, nor to an increase in the number of cells (HYPERPLASIA).'),
  'D006985': NtO(MH='Hyperventilation', MS='A pulmonary ventilation rate faster than is metabolically necessary for the exchange of gases. It is the result of an increased frequency of breathing, an increased tidal volume, or a combination of both. It causes an excess intake of oxygen and the blowing off of carbon dioxide.'),
  'D006986': NtO(MH='Hypervitaminosis A', MS='A symptom complex resulting from ingesting excessive amounts of VITAMIN A.'),
  'D006987': NtO(MH='Hypesthesia', MS='Absent or reduced sensitivity to cutaneous stimulation.'),
  'D006988': NtO(MH='Hyphema', MS='Bleeding in the anterior chamber of the eye.'),
  'D006990': NtO(MH='Hypnosis', MS='A state of increased receptivity to suggestion and direction, initially induced by the influence of another person.'),
  'D006991': NtO(MH='Hypnosis, Anesthetic', MS='Procedure in which an individual is induced into a trance-like state to relieve pain. This procedure is frequently performed with local but not general ANESTHESIA.'),
  'D006992': NtO(MH='Hypnosis, Dental', MS='Process in which a patient is induced into a trance-like state in order to relieve anxiety during a dental procedure.'),
  'D006993': NtO(MH='Hypnotics and Sedatives', MS='Drugs used to induce drowsiness or sleep or to reduce psychological excitement or anxiety.'),
  'D006994': NtO(MH='Hypoaldosteronism', MS='A congenital or acquired condition of insufficient production of ALDOSTERONE by the ADRENAL CORTEX leading to diminished aldosterone-mediated synthesis of Na(+)-K(+)-EXCHANGING ATPASE in renal tubular cells. Clinical symptoms include HYPERKALEMIA, sodium-wasting, HYPOTENSION, and sometimes metabolic ACIDOSIS.'),
  'D006995': NtO(MH='Hypobetalipoproteinemias', MS='Conditions with abnormally low levels of BETA-LIPOPROTEINS (low density lipoproteins or LDL) in the blood. It is defined as LDL values equal to or less than the 5th percentile for the population. They include the autosomal dominant form involving mutation of the APOLIPOPROTEINS B gene, and the autosomal recessive form involving mutation of the microsomal triglyceride transfer protein. All are characterized by low LDL and dietary fat malabsorption.'),
  'D006996': NtO(MH='Hypocalcemia', MS="Reduction of the blood calcium below normal. Manifestations include hyperactive deep tendon reflexes, Chvostek's sign, muscle and abdominal cramps, and carpopedal spasm. (Dorland, 27th ed)"),
  'D006997': NtO(MH='Hypochlorous Acid', MS='An oxyacid of chlorine (HClO) containing monovalent chlorine that acts as an oxidizing or reducing agent.'),
  'D006998': NtO(MH='Hypochondriasis', MS="Preoccupation with the fear of having, or the idea that one has, a serious disease based on the person's misinterpretation of bodily symptoms. (APA, DSM-IV)"),
  'D006999': NtO(MH='Hypocreales', MS='An order of fungi in the phylum ASCOMYCOTA that includes a number of species which are parasitic on higher plants, insects, or fungi. Other species are saprotrophic.'),
  'D007000': NtO(MH='Hypodermyiasis', MS='Infestation with larvae of the genus Hypoderma, the warble fly.'),
  'D007001': NtO(MH='Hypogastric Plexus', MS='A complex network of nerve fibers in the pelvic region. The hypogastric plexus distributes sympathetic fibers from the lumbar paravertebral ganglia and the aortic plexus, parasympathetic fibers from the pelvic nerve, and visceral afferents. The bilateral pelvic plexus is in its lateral extent.'),
  'D007002': NtO(MH='Hypoglossal Nerve', MS='The 12th cranial nerve. The hypoglossal nerve originates in the hypoglossal nucleus of the medulla and supplies motor innervation to all of the muscles of the tongue except the palatoglossus (which is supplied by the vagus). This nerve also contains proprioceptive afferents from the tongue muscles.'),
  'D007003': NtO(MH='Hypoglycemia', MS='A syndrome of abnormally low BLOOD GLUCOSE level. Clinical hypoglycemia has diverse etiologies. Severe hypoglycemia eventually lead to glucose deprivation of the CENTRAL NERVOUS SYSTEM resulting in HUNGER; SWEATING; PARESTHESIA; impaired mental function; SEIZURES; COMA; and even DEATH.'),
  'D007004': NtO(MH='Hypoglycemic Agents', MS='Substances which lower blood glucose levels.'),
  'D007005': NtO(MH='Hypoglycins', MS='Methylene cyclopropyl alanine and congeners isolated from the unripe edible fruit of the AKEE plant (BLIGHIA SAPIDA). Hypoglycin B is the gamma-glutamyl congener of hypoglycin A. They are very toxic and teratogenic, causing a syndrome called Jamaican vomiting sickness that includes a fall in blood glucose due to the interference of FATTY ACIDS and LEUCINE metabolism which leads to VOMITING, liver damage, CONVULSIONS and DEATH.'),
  'D007006': NtO(MH='Hypogonadism', MS='Condition resulting from deficient gonadal functions, such as GAMETOGENESIS and the production of GONADAL STEROID HORMONES. It is characterized by delay in GROWTH, germ cell maturation, and development of secondary sex characteristics. Hypogonadism can be due to a deficiency of GONADOTROPINS (hypogonadotropic hypogonadism) or due to primary gonadal failure (hypergonadotropic hypogonadism).'),
  'D007007': NtO(MH='Hypohidrosis', MS='Abnormally diminished or absent perspiration. Both generalized and segmented (reduced or absent sweating in circumscribed locations) forms of the disease are usually associated with other underlying conditions.'),
  'D007008': NtO(MH='Hypokalemia', MS='Abnormally low potassium concentration in the blood. It may result from potassium loss by renal secretion or by the gastrointestinal route, as by vomiting or diarrhea. It may be manifested clinically by neuromuscular disorders ranging from weakness to paralysis, by electrocardiographic abnormalities (depression of the T wave and elevation of the U wave), by renal disease, and by gastrointestinal disorders. (Dorland, 27th ed)'),
  'D007009': NtO(MH='Hypolipoproteinemias', MS='Conditions with abnormally low levels of LIPOPROTEINS in the blood. This may involve any of the lipoprotein subclasses, including ALPHA-LIPOPROTEINS (high-density lipoproteins); BETA-LIPOPROTEINS (low-density lipoproteins); and PREBETA-LIPOPROTEINS (very-low-density lipoproteins).'),
  'D007010': NtO(MH='Hyponatremia', MS='Deficiency of sodium in the blood; salt depletion. (Dorland, 27th ed)'),
  'D007011': NtO(MH='Hypoparathyroidism', MS='A condition caused by a deficiency of PARATHYROID HORMONE (or PTH). It is characterized by HYPOCALCEMIA and hyperphosphatemia. Hypocalcemia leads to TETANY. The acquired form is due to removal or injuries to the PARATHYROID GLANDS. The congenital form is due to mutations of genes, such as TBX1; (see DIGEORGE SYNDROME); CASR encoding CALCIUM-SENSING RECEPTOR; or PTH encoding parathyroid hormone.'),
  'D007012': NtO(MH='Hypopharyngeal Neoplasms', MS='Tumors or cancer of the HYPOPHARYNX.'),
  'D007013': NtO(MH='Hypopharynx', MS='The bottom portion of the pharynx situated below the OROPHARYNX and posterior to the LARYNX. The hypopharynx communicates with the larynx through the laryngeal inlet, and is also called laryngopharynx.'),
  'D007014': NtO(MH='Hypophosphatasia', MS='A genetic metabolic disorder resulting from serum and bone alkaline phosphatase deficiency leading to hypercalcemia, ethanolamine phosphatemia, and ethanolamine phosphaturia. Clinical manifestations include severe skeletal defects resembling vitamin D-resistant rickets, failure of the calvarium to calcify, dyspnea, cyanosis, vomiting, constipation, renal calcinosis, failure to thrive, disorders of movement, beading of the costochondral junction, and rachitic bone changes. (From Dorland, 27th ed)'),
  'D007015': NtO(MH='Hypophosphatemia, Familial', MS='An inherited condition of abnormally low serum levels of PHOSPHATES (below 1 mg/liter) which can occur in a number of genetic diseases with defective reabsorption of inorganic phosphorus by the PROXIMAL RENAL TUBULES. This leads to phosphaturia, HYPOPHOSPHATEMIA, and disturbances of cellular and organ functions such as those in X-LINKED HYPOPHOSPHATEMIC RICKETS; OSTEOMALACIA; and FANCONI SYNDROME.'),
  'D007016': NtO(MH='Hypophysectomy', MS='Surgical removal or destruction of the hypophysis, or pituitary gland. (Dorland, 28th ed)'),
  'D007017': NtO(MH='Hypophysectomy, Chemical', MS='Total or subtotal destruction of the pituitary gland by chemical injection. It is usually achieved by injection of ethyl alcohol via trans-sphenoidal cannulation under stereotaxic control. It is usually performed for the treatment of intractable pain.'),
  'D007018': NtO(MH='Hypopituitarism', MS='Diminution or cessation of secretion of one or more hormones from the anterior pituitary gland (including LH; FOLLICLE STIMULATING HORMONE; SOMATOTROPIN; and CORTICOTROPIN). This may result from surgical or radiation ablation, non-secretory PITUITARY NEOPLASMS, metastatic tumors, infarction, PITUITARY APOPLEXY, infiltrative or granulomatous processes, and other conditions.'),
  'D007019': NtO(MH='Hypoproteinemia', MS='A condition in which total serum protein level is below the normal range. Hypoproteinemia can be caused by protein malabsorption in the gastrointestinal tract, EDEMA, or PROTEINURIA.'),
  'D007020': NtO(MH='Hypoprothrombinemias', MS='Absence or reduced levels of PROTHROMBIN in the blood.'),
  'D007021': NtO(MH='Hypospadias', MS='A birth defect due to malformation of the URETHRA in which the urethral opening is below its normal location. In the male, the malformed urethra generally opens on the ventral surface of the PENIS or on the PERINEUM. In the female, the malformed urethral opening is in the VAGINA.'),
  'D007022': NtO(MH='Hypotension', MS='Abnormally low BLOOD PRESSURE that can result in inadequate blood flow to the brain and other vital organs. Common symptom is DIZZINESS but greater negative impacts on the body occur when there is prolonged depravation of oxygen and nutrients.'),
  'D007023': NtO(MH='Hypotension, Controlled', MS='Procedure in which arterial blood pressure is intentionally reduced in order to control blood loss during surgery. This procedure is performed either pharmacologically or by pre-surgical removal of blood.'),
  'D007024': NtO(MH='Hypotension, Orthostatic', MS='A significant drop in BLOOD PRESSURE after assuming a standing position. Orthostatic hypotension is a finding, and defined as a 20-mm Hg decrease in systolic pressure or a 10-mm Hg decrease in diastolic pressure 3 minutes after the person has risen from supine to standing. Symptoms generally include DIZZINESS, blurred vision, and SYNCOPE.'),
  'D007025': NtO(MH='Anterior Hypothalamic Nucleus', MS='Loose heterogeneous collection of cells in the anterior hypothalamus, continuous rostrally with the medial and lateral PREOPTIC AREAS and caudally with the TUBER CINEREUM.'),
  'D007026': NtO(MH='Hypothalamic Area, Lateral', MS='Area in the hypothalamus bounded medially by the mammillothalamic tract and the anterior column of the FORNIX (BRAIN). The medial edge of the INTERNAL CAPSULE and the subthalamic region form its lateral boundary. It contains the lateral hypothalamic nucleus, tuberomammillary nucleus, lateral tuberal nuclei, and fibers of the MEDIAL FOREBRAIN BUNDLE.'),
  'D007027': NtO(MH='Hypothalamic Diseases', MS='Neoplastic, inflammatory, infectious, and other diseases of the hypothalamus. Clinical manifestations include appetite disorders; AUTONOMIC NERVOUS SYSTEM DISEASES; SLEEP DISORDERS; behavioral symptoms related to dysfunction of the LIMBIC SYSTEM; and neuroendocrine disorders.'),
  'D007028': NtO(MH='Hypothalamic Hormones', MS='Peptide hormones produced by NEURONS of various regions in the HYPOTHALAMUS. They are released into the pituitary portal circulation to stimulate or inhibit PITUITARY GLAND functions. VASOPRESSIN and OXYTOCIN, though produced in the hypothalamus, are not included here for they are transported down the AXONS to the POSTERIOR LOBE OF PITUITARY before being released into the portal circulation.'),
  'D007029': NtO(MH='Hypothalamic Neoplasms', MS='Benign and malignant tumors of the HYPOTHALAMUS. Pilocytic astrocytomas and hamartomas are relatively frequent histologic types. Neoplasms of the hypothalamus frequently originate from adjacent structures, including the OPTIC CHIASM, optic nerve (see OPTIC NERVE NEOPLASMS), and pituitary gland (see PITUITARY NEOPLASMS). Relatively frequent clinical manifestations include visual loss, developmental delay, macrocephaly, and precocious puberty. (From Devita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2051)'),
  'D007030': NtO(MH='Hypothalamo-Hypophyseal System', MS='A collection of NEURONS, tracts of NERVE FIBERS, endocrine tissue, and blood vessels in the HYPOTHALAMUS and the PITUITARY GLAND. This hypothalamo-hypophyseal portal circulation provides the mechanism for hypothalamic neuroendocrine (HYPOTHALAMIC HORMONES) regulation of pituitary function and the release of various PITUITARY HORMONES into the systemic circulation to maintain HOMEOSTASIS.'),
  'D007031': NtO(MH='Hypothalamus', MS='Ventral part of the DIENCEPHALON extending from the region of the OPTIC CHIASM to the caudal border of the MAMMILLARY BODIES and forming the inferior and lateral walls of the THIRD VENTRICLE.'),
  'D007032': NtO(MH='Hypothalamus, Anterior', MS='The front portion of the HYPOTHALAMUS separated into the preoptic region and the supraoptic region. The preoptic region is made up of the periventricular GRAY MATTER of the rostral portion of the THIRD VENTRICLE and contains the preoptic ventricular nucleus and the medial preoptic nucleus. The supraoptic region contains the PARAVENTRICULAR HYPOTHALAMIC NUCLEUS, the SUPRAOPTIC NUCLEUS, the ANTERIOR HYPOTHALAMIC NUCLEUS, and the SUPRACHIASMATIC NUCLEUS.'),
  'D007033': NtO(MH='Hypothalamus, Middle', MS='Middle portion of the hypothalamus containing the arcuate, dorsomedial, ventromedial nuclei, the TUBER CINEREUM and the PITUITARY GLAND.'),
  'D007034': NtO(MH='Hypothalamus, Posterior', MS='The part of the hypothalamus posterior to the middle region consisting of several nuclei including the medial maxillary nucleus, lateral mammillary nucleus, and posterior hypothalamic nucleus (posterior hypothalamic area). The posterior hypothalamic area is concerned with control of sympathetic responses and is sensitive to conditions of decreasing temperature and controls the mechanisms for the conservation and increased production of heat.'),
  'D007035': NtO(MH='Hypothermia', MS='Lower than normal body temperature, especially in warm-blooded animals.'),
  'D007036': NtO(MH='Hypothermia, Induced', MS='Abnormally low BODY TEMPERATURE that is intentionally induced in warm-blooded animals by artificial means. In humans, mild or moderate hypothermia has been used to reduce tissue damages, particularly after cardiac or spinal cord injuries and during subsequent surgeries.'),
  'D007037': NtO(MH='Hypothyroidism', MS='A syndrome that results from abnormally low secretion of THYROID HORMONES from the THYROID GLAND, leading to a decrease in BASAL METABOLIC RATE. In its most severe form, there is accumulation of MUCOPOLYSACCHARIDES in the SKIN and EDEMA, known as MYXEDEMA. It may be primary or secondary due to other pituitary disease, or hypothalamic dysfunction.'),
  'D007038': NtO(MH='Hypotonic Solutions', MS='Solutions that have a lesser osmotic pressure than a reference solution such as blood, plasma, or interstitial fluid.'),
  'D007039': NtO(MH='Hypotrichosis', MS='Presence of less than the normal amount of hair. (Dorland, 27th ed)'),
  'D007040': NtO(MH='Hypoventilation', MS='A reduction in the amount of air entering the pulmonary alveoli.'),
  'D007041': NtO(MH='Hypoxanthine Phosphoribosyltransferase', MS="An enzyme that catalyzes the conversion of 5-phosphoribosyl-1-pyrophosphate and hypoxanthine, guanine, or MERCAPTOPURINE to the corresponding 5'-mononucleotides and pyrophosphate. The enzyme is important in purine biosynthesis as well as central nervous system functions. Complete lack of enzyme activity is associated with the LESCH-NYHAN SYNDROME, while partial deficiency results in overproduction of uric acid. EC 2.4.2.8."),
  'D007042': NtO(MH='Hypoxanthines', MS='Purine bases related to hypoxanthine, an intermediate product of uric acid synthesis and a breakdown product of adenine catabolism.'),
  'D007043': NtO(MH='Hyraxes', MS='Any of certain small hoofed mammals of the order Hyracoidea native to Africa and southwestern Asia.'),
  'D007044': NtO(MH='Hysterectomy', MS='Excision of the uterus.'),
  'D007045': NtO(MH='Hysterectomy, Vaginal', MS='Removal of the uterus through the vagina.'),
  'D007046': NtO(MH='Hysteria', MS='Historical term for a chronic, but fluctuating, disorder beginning in early life and characterized by recurrent and multiple somatic complaints not apparently due to physical illness. This diagnosis is not used in contemporary practice.'),
  'D007047': NtO(MH='Hysterosalpingography', MS='Radiography of the uterus and fallopian tubes after the injection of a contrast medium.'),
  'D007048': NtO(MH='I Blood-Group System', MS='A blood group related both to the ABO and P systems that includes several different antigens found in most people on erythrocytes, in milk, and in saliva. The antibodies react only at low temperatures.'),
  'D007049': NtO(MH='Iatrogenic Disease', MS='Any adverse condition in a patient occurring as the result of treatment by a physician, surgeon, or other health professional, especially infections acquired by a patient during the course of treatment.'),
  'D007050': NtO(MH='Ibogaine', MS='One of several indole alkaloids extracted from Tabernanthe iboga, Baill. It has a complex pharmacological profile, and interacts with multiple systems of neurotransmission. Ibogaine has psychoactive properties and appears to modulate tolerance to opiates.'),
  'D007051': NtO(MH='Ibotenic Acid', MS='A neurotoxic isoxazole (similar to KAINIC ACID and MUSCIMOL) found in AMANITA mushrooms. It causes motor depression, ataxia, and changes in mood, perceptions and feelings, and is a potent excitatory amino acid agonist.'),
  'D007052': NtO(MH='Ibuprofen', MS='A nonsteroidal anti-inflammatory agent with analgesic properties used in the treatment of RHEUMATISM and ARTHRITIS.'),
  'D007053': NtO(MH='Ice', MS='The solid substance formed by the FREEZING of water.'),
  'D007054': NtO(MH='Ice Cream', MS='A frozen dairy food made from cream or butterfat, milk, sugar, and flavorings. Frozen custard and French-type ice creams also contain eggs.'),
  'D007055': NtO(MH='Iceland', MS='A country in northern Europe, an island between the Greenland Sea and the North Atlantic Ocean, northwest of the United Kingdom. The capital is Reykjavik.'),
  'D007057': NtO(MH='Ichthyosis', MS='Any of several generalized skin disorders characterized by dryness, roughness, and scaliness, due to hypertrophy of the stratum corneum epidermis. Most are genetic, but some are acquired, developing in association with other systemic disease or genetic syndrome.'),
  'D007059': NtO(MH='Ictaluridae', MS='A family of North American freshwater CATFISHES. It consists of four genera (Ameiurus, Ictalurus, Noturus, Pylodictis,) comprising several species, two of which are eyeless.'),
  'D007060': NtO(MH='Id', MS='The part of the personality structure which harbors the unconscious instinctive desires and strivings of the individual.'),
  'D007061': NtO(MH='Idaho', MS='State bounded on the north by Canada, on the east by Montana and Wyoming, on the south by Nevada and Utah, and on the west by Oregon and Washington.'),
  'D007062': NtO(MH='Identification, Psychological', MS='A process by which an individual unconsciously endeavors to pattern himself after another. This process is also important in the development of the personality, particularly the superego or conscience, which is modeled largely on the behavior of adult significant others.'),
  'D007063': NtO(MH='Identity Crisis', MS="Chaotic concept of self wherein one's role in life appears to be an insoluble dilemma often expressed by isolation, withdrawal, rebellion and extremism."),
  'D007064': NtO(MH='L-Iditol 2-Dehydrogenase', MS='An alcohol oxidoreductase which catalyzes the oxidation of L-iditol to L-sorbose in the presence of NAD. It also acts on D-glucitol to form D-fructose. It also acts on other closely related sugar alcohols to form the corresponding sugar. EC 1.1.1.14'),
  'D007065': NtO(MH='Idoxuridine', MS='An analog of DEOXYURIDINE that inhibits viral DNA synthesis. The drug is used as an antiviral agent.'),
  'D007066': NtO(MH='Iduronate Sulfatase', MS="An enzyme that specifically cleaves the ester sulfate of iduronic acid. Its deficiency has been demonstrated in Hunter's syndrome, which is characterized by an excess of dermatan sulfate and heparan sulfate. EC 3.1.6.13."),
  'D007067': NtO(MH='Iduronic Acid', MS='Component of dermatan sulfate. Differs in configuration from glucuronic acid only at the C-5 position.'),
  'D007068': NtO(MH='Iduronidase', MS="An enzyme that hydrolyzes iduronosidic linkages in desulfated dermatan. Deficiency of this enzyme produces Hurler's syndrome. EC 3.2.1.76."),
  'D007069': NtO(MH='Ifosfamide', MS='Positional isomer of CYCLOPHOSPHAMIDE which is active as an alkylating agent and an immunosuppressive agent.'),
  'D007070': NtO(MH='Immunoglobulin A', MS='Represents 15-20% of the human serum immunoglobulins, mostly as the 4-chain polymer in humans or dimer in other mammals. Secretory IgA (IMMUNOGLOBULIN A, SECRETORY) is the main immunoglobulin in secretions.'),
  'D007071': NtO(MH='Immunoglobulin A, Secretory', MS='The principle immunoglobulin in exocrine secretions such as milk, respiratory and intestinal mucin, saliva and tears. The complete molecule (around 400 kD) is composed of two four-chain units of IMMUNOGLOBULIN A, one SECRETORY COMPONENT and one J chain (IMMUNOGLOBULIN J-CHAINS).'),
  'D007072': NtO(MH='Immunoglobulin D', MS='An immunoglobulin which accounts for less than 1% of plasma immunoglobulin. It is found on the membrane of many circulating B LYMPHOCYTES.'),
  'D007073': NtO(MH='Immunoglobulin E', MS='An immunoglobulin associated with MAST CELLS. Overexpression has been associated with allergic hypersensitivity (HYPERSENSITIVITY, IMMEDIATE).'),
  'D007074': NtO(MH='Immunoglobulin G', MS='The major immunoglobulin isotype class in normal human serum. There are several isotype subclasses of IgG, for example, IgG1, IgG2A, and IgG2B.'),
  'D007075': NtO(MH='Immunoglobulin M', MS='A class of immunoglobulin bearing mu chains (IMMUNOGLOBULIN MU-CHAINS). IgM can fix COMPLEMENT. The name comes from its high molecular weight and originally being called a macroglobulin.'),
  'D007076': NtO(MH='Iguanas', MS='Large herbivorous tropical American lizards.'),
  'D007077': NtO(MH='Ileal Diseases', MS='Pathological development in the ILEUM including the ILEOCECAL VALVE.'),
  'D007078': NtO(MH='Ileal Neoplasms', MS='Tumors or cancer in the ILEUM region of the small intestine (INTESTINE, SMALL).'),
  'D007079': NtO(MH='Ileitis', MS='Inflammation of any segment of the ILEUM and the ILEOCECAL VALVE.'),
  'D007080': NtO(MH='Ileocecal Valve', MS='The valve, at the junction of the CECUM with the COLON, that guards the opening where the ILEUM enters the LARGE INTESTINE.'),
  'D007081': NtO(MH='Ileostomy', MS='Surgical creation of an external opening into the ILEUM for fecal diversion or drainage. This replacement for the RECTUM is usually created in patients with severe INFLAMMATORY BOWEL DISEASES. Loop (continent) or tube (incontinent) procedures are most often employed.'),
  'D007082': NtO(MH='Ileum', MS='The distal and narrowest portion of the SMALL INTESTINE, between the JEJUNUM and the ILEOCECAL VALVE of the LARGE INTESTINE.'),
  'D007083': NtO(MH='Iliac Artery', MS='Either of two large arteries originating from the abdominal aorta; they supply blood to the pelvis, abdominal wall and legs.'),
  'D007084': NtO(MH='Iliac Vein', MS='A vein on either side of the body which is formed by the union of the external and internal iliac veins and passes upward to join with its fellow of the opposite side to form the inferior vena cava.'),
  'D007085': NtO(MH='Ilium', MS='The largest of three bones that make up each half of the pelvic girdle.'),
  'D007086': NtO(MH='Illegitimacy', MS='The state of birth outside of wedlock. It may refer to the offspring or the parents.'),
  'D007087': NtO(MH='Illinois', MS='State bounded on the north by Wisconsin, on the east by Lake Michigan, Indiana, and Kentucky, on the west by Missouri and Iowa, and on the south by Kentucky.'),
  'D007088': NtO(MH='Illusions', MS='The misinterpretation of a real external, sensory experience.'),
  'D007089': NtO(MH='Image Enhancement', MS='Improvement of the quality of a picture by various techniques, including computer processing, digital filtering, echocardiographic techniques, light and ultrastructural MICROSCOPY, fluorescence spectrometry and microscopy, scintigraphy, and in vitro image processing at the molecular level.'),
  'D007090': NtO(MH='Image Interpretation, Computer-Assisted', MS='Methods developed to aid in the interpretation of ultrasound, radiographic images, etc., for diagnosis of disease.'),
  'D007091': NtO(MH='Image Processing, Computer-Assisted', MS='A technique of inputting two-dimensional images into a computer and then enhancing or analyzing the imagery into a form that is more useful to the human observer.'),
  'D007092': NtO(MH='Imagination', MS='A new pattern of perceptual or ideational material derived from past experience.'),
  'D007093': NtO(MH='Imidazoles', MS='Compounds containing 1,3-diazole, a five membered aromatic ring containing two nitrogen atoms separated by one of the carbons. Chemically reduced ones include IMIDAZOLINES and IMIDAZOLIDINES. Distinguish from 1,2-diazole (PYRAZOLES).'),
  'D007094': NtO(MH='Imides', MS='Organic compounds containing two acyl groups bound to NITROGEN.'),
  'D007095': NtO(MH='Imidocarb', MS='One of ANTIPROTOZOAL AGENTS used especially against BABESIA in livestock. Toxicity has been reported.'),
  'D007096': NtO(MH='Imidoesters', MS='Esters of the hypothetical imidic acids. They react with amines or amino acids to form amidines and are therefore used to modify protein structures and as cross-linking agents.'),
  'D007097': NtO(MH='Imines', MS='Organic compounds containing a carbon-nitrogen double bond where a NITROGEN atom can be attached to HYDROGEN or an alkyl or aryl group.'),
  'D007098': NtO(MH='Imino Acids', MS='Carboxylic acids that contain an imino group (C=NH).'),
  'D007099': NtO(MH='Imipramine', MS='The prototypical tricyclic antidepressant. It has been used in major depression, dysthymia, bipolar depression, attention-deficit disorders, agoraphobia, and panic disorders. It has less sedative effect than some other members of this therapeutic group.'),
  'D007100': NtO(MH='Imitative Behavior', MS='The mimicking of the behavior of one individual by another.'),
  'D007101': NtO(MH='Immersion', MS='The placing of a body or a part thereof into a liquid.'),
  'D007102': NtO(MH='Immersion Foot', MS="A condition of the feet produced by prolonged exposure of the feet to water. Exposure for 48 hours or more to warm water causes tropical immersion foot or warm-water immersion foot common in Vietnam where troops were exposed to prolonged or repeated wading in paddy fields or streams. Trench foot results from prolonged exposure to cold, without actual freezing. It was common in trench warfare during World War I, when soldiers stood, sometimes for hours, in trenches with a few inches of cold water in them. (Andrews' Diseases of the Skin, 8th ed, p27)"),
  'D007103': NtO(MH='Immobilization', MS='The restriction of the MOVEMENT of whole or part of the body by physical means (RESTRAINT, PHYSICAL) or chemically by ANALGESIA, or the use of TRANQUILIZING AGENTS or NEUROMUSCULAR NONDEPOLARIZING AGENTS. It includes experimental protocols used to evaluate the physiologic effects of immobility.'),
  'D007104': NtO(MH='Immune Adherence Reaction', MS='A method for the detection of very small quantities of antibody in which the antigen-antibody-complement complex adheres to indicator cells, usually primate erythrocytes or nonprimate blood platelets. The reaction is dependent on the number of bound C3 molecules on the C3b receptor sites of the indicator cell.'),
  'D007105': NtO(MH='Immune Complex Diseases', MS='Group of diseases mediated by the deposition of large soluble complexes of antigen and antibody with resultant damage to tissue. Besides SERUM SICKNESS and the ARTHUS REACTION, evidence supports a pathogenic role for immune complexes in many other IMMUNE SYSTEM DISEASES including GLOMERULONEPHRITIS, systemic lupus erythematosus (LUPUS ERYTHEMATOSUS, SYSTEMIC) and POLYARTERITIS NODOSA.'),
  'D007106': NtO(MH='Immune Sera', MS='Serum that contains antibodies. It is obtained from an animal that has been immunized either by ANTIGEN injection or infection with microorganisms containing the antigen.'),
  'D007107': NtO(MH='Immune System', MS="The body's defense mechanism against foreign organisms or substances and deviant native cells. It includes the humoral immune response and the cell-mediated response and consists of a complex of interrelated cellular, molecular, and genetic components."),
  'D007108': NtO(MH='Immune Tolerance', MS='The specific failure of a normally responsive individual to make an immune response to a known antigen. It results from previous contact with the antigen by an immunologically immature individual (fetus or neonate) or by an adult exposed to extreme high-dose or low-dose antigen, or by exposure to radiation, antimetabolites, antilymphocytic serum, etc.'),
  'D007109': NtO(MH='Immunity', MS='Nonsusceptibility to the invasive or pathogenic effects of foreign microorganisms or to the toxic effect of antigenic substances.'),
  'D007110': NtO(MH='Immunity, Active', MS='Resistance to a disease agent resulting from the production of specific antibodies by the host, either after exposure to the disease or after vaccination.'),
  'D007111': NtO(MH='Immunity, Cellular', MS='Manifestations of the immune response which are mediated by antigen-sensitized T-lymphocytes via lymphokines or direct cytotoxicity. This takes place in the absence of circulating antibody or where antibody plays a subordinate role.'),
  'D007112': NtO(MH='Immunity, Maternally-Acquired', MS='Resistance to a disease-causing agent induced by the introduction of maternal immunity into the fetus by transplacental transfer or into the neonate through colostrum and milk.'),
  'D007113': NtO(MH='Immunity, Innate', MS='The capacity of a normal organism to remain unaffected by microorganisms and their toxins. It results from the presence of naturally occurring ANTI-INFECTIVE AGENTS, constitutional factors such as BODY TEMPERATURE and immediate acting immune cells such as NATURAL KILLER CELLS.'),
  'D007114': NtO(MH='Immunization', MS="Deliberate stimulation of the host's immune response. ACTIVE IMMUNIZATION involves administration of ANTIGENS or IMMUNOLOGIC ADJUVANTS. PASSIVE IMMUNIZATION involves administration of IMMUNE SERA or LYMPHOCYTES or their extracts (e.g., transfer factor, immune RNA) or transplantation of immunocompetent cell producing tissue (thymus or bone marrow)."),
  'D007115': NtO(MH='Immunization Schedule', MS='Schedule giving optimum times usually for primary and/or secondary immunization.'),
  'D007116': NtO(MH='Immunization, Passive', MS='Transfer of immunity from immunized to non-immune host by administration of serum antibodies, or transplantation of lymphocytes (ADOPTIVE TRANSFER).'),
  'D007117': NtO(MH='Immunization, Secondary', MS='Any immunization following a primary immunization and involving exposure to the same or a closely related antigen.'),
  'D007118': NtO(MH='Immunoassay', MS='A technique using antibodies for identifying or quantifying a substance. Usually the substance being studied serves as antigen both in antibody production and in measurement of antibody by the test substance.'),
  'D007119': NtO(MH='Immunoblastic Lymphadenopathy', MS='A disorder characterized by proliferation of arborizing small vessels, prominent immunoblastic proliferations and amorphous acidophilic interstitial material. Clinical manifestations include fever, sweats, weight loss, generalized lymphadenopathy and frequently hepatosplenomegaly.'),
  'D007120': NtO(MH='Immunochemistry', MS='Field of chemistry that pertains to immunological phenomena and the study of chemical reactions related to antigen stimulation of tissues. It includes physicochemical interactions between antigens and antibodies.'),
  'D007121': NtO(MH='Immunocompetence', MS='The ability of lymphoid cells to mount a humoral or cellular immune response when challenged by antigen.'),
  'D007122': NtO(MH='Immunoelectrophoresis', MS='A technique that combines protein electrophoresis and double immunodiffusion. In this procedure proteins are first separated by gel electrophoresis (usually agarose), then made visible by immunodiffusion of specific antibodies. A distinct elliptical precipitin arc results for each protein detectable by the antisera.'),
  'D007123': NtO(MH='Immunoelectrophoresis, Two-Dimensional', MS='Immunoelectrophoresis in which a second electrophoretic transport is performed on the initially separated antigen fragments into an antibody-containing medium in a direction perpendicular to the first electrophoresis.'),
  'D007124': NtO(MH='Immunoenzyme Techniques', MS='Immunologic techniques based on the use of: (1) enzyme-antibody conjugates; (2) enzyme-antigen conjugates; (3) antienzyme antibody followed by its homologous enzyme; or (4) enzyme-antienzyme complexes. These are used histologically for visualizing or labeling tissue specimens.'),
  'D007125': NtO(MH='Immunogenetics', MS='A subdiscipline of genetics which deals with the genetic basis of the immune response (IMMUNITY).'),
  'D007126': NtO(MH='Immunoglobulin Allotypes', MS='Allelic variants of the immunoglobulin light chains (IMMUNOGLOBULIN LIGHT CHAINS) or heavy chains (IMMUNOGLOBULIN HEAVY CHAINS) encoded by ALLELES of IMMUNOGLOBULIN GENES.'),
  'D007127': NtO(MH='Immunoglobulin Constant Regions', MS='The domains of the immunoglobulin molecules that are invariable in their amino acid sequence within any class or subclass of immunoglobulin. They confer biological as well as structural functions to immunoglobulins. One each on both the light chains and the heavy chains comprises the C-terminus half of the IMMUNOGLOBULIN FAB FRAGMENT and two or three of them make up the rest of the heavy chains (all of the IMMUNOGLOBULIN FC FRAGMENT)'),
  'D007128': NtO(MH='Immunoglobulin Fragments', MS='Partial immunoglobulin molecules resulting from selective cleavage by proteolytic enzymes or generated through PROTEIN ENGINEERING techniques.'),
  'D007129': NtO(MH='Immunoglobulin Gm Allotypes', MS='Allelic variants of the gamma-immunoglobulin heavy chain (IMMUNOGLOBULIN GAMMA-CHAINS) encoded by ALLELES of IMMUNOGLOBULIN HEAVY CHAIN GENES.'),
  'D007130': NtO(MH='Immunoglobulin Idiotypes', MS='Unique genetically-controlled determinants present on ANTIBODIES whose specificity is limited to a single group of proteins (e.g., another antibody molecule or an individual myeloma protein). The idiotype appears to represent the antigenicity of the antigen-binding site of the antibody and to be genetically codetermined with it. The idiotypic determinants have been precisely located to the IMMUNOGLOBULIN VARIABLE REGION of both immunoglobin polypeptide chains.'),
  'D007131': NtO(MH='Immunoglobulin Km Allotypes', MS='Allelic variants of the kappa light chains (IMMUNOGLOBULIN KAPPA-CHAINS) encoded by ALLELES of IMMUNOGLOBULIN LIGHT CHAIN GENES.'),
  'D007132': NtO(MH='Immunoglobulin Isotypes', MS='The classes of immunoglobulins found in any species of animal. In man there are nine classes that migrate in five different groups in electrophoresis; they each consist of two light and two heavy protein chains, and each group has distinguishing structural and functional properties.'),
  'D007133': NtO(MH='Immunoglobulin Joining Region', MS='A segment of the immunoglobulin heavy chains, encoded by the IMMUNOGLOBULIN HEAVY CHAIN GENES in the J segment where, during the maturation of B-LYMPHOCYTES; the gene segment for the variable region upstream is joined to a constant region gene segment downstream. The exact position of joining of the two gene segments is variable and contributes to ANTIBODY DIVERSITY. It is distinguished from the IMMUNOGLOBULIN J CHAINS; a separate polypeptide that serves as a linkage piece in polymeric IGA or IGM.'),
  'D007134': NtO(MH='Immunoglobulin Switch Region', MS="A site located in the INTRONS at the 5' end of each constant region segment of a immunoglobulin heavy-chain gene where recombination (or rearrangement) occur during IMMUNOGLOBULIN CLASS SWITCHING. Ig switch regions are found on genes encoding all five classes (IMMUNOGLOBULIN ISOTYPES) of IMMUNOGLOBULIN HEAVY CHAINS."),
  'D007135': NtO(MH='Immunoglobulin Variable Region', MS='That region of the immunoglobulin molecule that varies in its amino acid sequence and composition, and comprises the binding site for a specific antigen. It is located at the N-terminus of the Fab fragment of the immunoglobulin. It includes hypervariable regions (COMPLEMENTARITY DETERMINING REGIONS) and framework regions.'),
  'D007136': NtO(MH='Immunoglobulins', MS='Multi-subunit proteins which function in IMMUNITY. They are produced by B LYMPHOCYTES from the IMMUNOGLOBULIN GENES. They are comprised of two heavy (IMMUNOGLOBULIN HEAVY CHAINS) and two light chains (IMMUNOGLOBULIN LIGHT CHAINS) with additional ancillary polypeptide chains depending on their isoforms. The variety of isoforms include monomeric or polymeric forms, and transmembrane forms (B-CELL ANTIGEN RECEPTORS) or secreted forms (ANTIBODIES). They are divided by the amino acid sequence of their heavy chains into five classes (IMMUNOGLOBULIN A; IMMUNOGLOBULIN D; IMMUNOGLOBULIN E; IMMUNOGLOBULIN G; IMMUNOGLOBULIN M) and various subclasses.'),
  'D007137': NtO(MH='Immunoglobulin alpha-Chains', MS='The class of heavy chains found in IMMUNOGLOBULIN A. They have a molecular weight of approximately 58 kDa and contain about 470 amino acid residues arranged in four domains and an oligosaccharide component bound covalently to their Fc fragment constant region.'),
  'D007138': NtO(MH='Immunoglobulin delta-Chains', MS='The class of heavy chains found in IMMUNOGLOBULIN D. They have a molecular weight of approximately 64 kDa and they contain about 500 amino acid residues arranged in four domains and an oligosaccharide component covalently bound to the Fc fragment constant region.'),
  'D007139': NtO(MH='Immunoglobulin epsilon-Chains', MS='The class of heavy chains found in IMMUNOGLOBULIN E. They have a molecular weight of approximately 72 kDa and they contain about 550 amino acid residues arranged in five domains and about three times more carbohydrate than the heavy chains of IMMUNOGLOBULIN A; IMMUNOGLOBULIN D; and IMMUNOGLOBULIN G.'),
  'D007140': NtO(MH='Immunoglobulin Fab Fragments', MS='Univalent antigen-binding fragments composed of one entire IMMUNOGLOBULIN LIGHT CHAIN and the amino terminal end of one of the IMMUNOGLOBULIN HEAVY CHAINS from the hinge region, linked to each other by disulfide bonds. Fab contains the IMMUNOGLOBULIN VARIABLE REGIONS, which are part of the antigen-binding site, and the first IMMUNOGLOBULIN CONSTANT REGIONS. This fragment can be obtained by digestion of immunoglobulins with the proteolytic enzyme PAPAIN.'),
  'D007141': NtO(MH='Immunoglobulin Fc Fragments', MS='Crystallizable fragments composed of the carboxy-terminal halves of both IMMUNOGLOBULIN HEAVY CHAINS linked to each other by disulfide bonds. Fc fragments contain the carboxy-terminal parts of the heavy chain constant regions that are responsible for the effector functions of an immunoglobulin (COMPLEMENT fixation, binding to the cell membrane via FC RECEPTORS, and placental transport). This fragment can be obtained by digestion of immunoglobulins with the proteolytic enzyme PAPAIN.'),
  'D007142': NtO(MH='Immunoglobulin gamma-Chains', MS='Heavy chains of IMMUNOGLOBULIN G having a molecular weight of approximately 51 kDa. They contain about 450 amino acid residues arranged in four domains and an oligosaccharide component covalently bound to the Fc fragment constant region. The gamma heavy chain subclasses (for example, gamma 1, gamma 2a, and gamma 2b) of the IMMUNOGLOBULIN G isotype subclasses (IgG1, IgG2A, and IgG2B) resemble each other more closely than the heavy chains of the other IMMUNOGLOBULIN ISOTYPES.'),
  'D007143': NtO(MH='Immunoglobulin Heavy Chains', MS='The largest of polypeptide chains comprising immunoglobulins. They contain 450 to 600 amino acid residues per chain, and have molecular weights of 51-72 kDa.'),
  'D007144': NtO(MH='Immunoglobulin J-Chains', MS='A 15 kDa "joining" peptide that forms one of the linkages between monomers of IMMUNOGLOBULIN A or IMMUNOGLOBULIN M in the formation of polymeric immunoglobulins. There is one J chain per one IgA dimer or one IgM pentamer. It is also involved in binding the polymeric immunoglobulins to POLYMERIC IMMUNOGLOBULIN RECEPTOR which is necessary for their transcytosis to the lumen. It is distinguished from the IMMUNOGLOBULIN JOINING REGION which is part of the IMMUNOGLOBULIN VARIABLE REGION of the immunoglobulin light and heavy chains.'),
  'D007145': NtO(MH='Immunoglobulin kappa-Chains', MS='One of the types of light chains of the immunoglobulins with a molecular weight of approximately 22 kDa.'),
  'D007146': NtO(MH='Immunoglobulin lambda-Chains', MS='One of the types of light chain subunits of the immunoglobulins with a molecular weight of approximately 22 kDa.'),
  'D007147': NtO(MH='Immunoglobulin Light Chains', MS='Polypeptide chains, consisting of 211 to 217 amino acid residues and having a molecular weight of approximately 22 kDa. There are two major types of light chains, kappa and lambda. Two Ig light chains and two Ig heavy chains (IMMUNOGLOBULIN HEAVY CHAINS) make one immunoglobulin molecule.'),
  'D007148': NtO(MH='Immunoglobulin mu-Chains', MS='The class of heavy chains found in IMMUNOGLOBULIN M. They have a molecular weight of approximately 72 kDa and they contain about 57 amino acid residues arranged in five domains and have more oligosaccharide branches and a higher carbohydrate content than the heavy chains of IMMUNOGLOBULIN G.'),
  'D007150': NtO(MH='Immunohistochemistry', MS='Histochemical localization of immunoreactive substances using labeled antibodies as reagents.'),
  'D007152': NtO(MH='Immunologic Capping', MS='An energy dependent process following the crosslinking of B CELL ANTIGEN RECEPTORS by multivalent ligands (bivalent anti-antibodies, LECTINS or ANTIGENS), on the B-cell surface. The crosslinked ligand-antigen receptor complexes collect in patches which flow to and aggregate at one pole of the cell to form a large mass - the cap. The caps may then be endocytosed or shed into the environment.'),
  'D007153': NtO(MH='Immunologic Deficiency Syndromes', MS='Syndromes in which there is a deficiency or defect in the mechanisms of immunity, either cellular or humoral.'),
  'D007154': NtO(MH='Immune System Diseases', MS='Disorders caused by abnormal or absent immunologic mechanisms, whether humoral, cell-mediated, or both.'),
  'D007155': NtO(MH='Immunologic Factors', MS='Biologically active substances whose activities affect or play a role in the functioning of the immune system.'),
  'D007156': NtO(MH='Immunologic Memory', MS='The altered state of immunologic responsiveness resulting from initial contact with antigen, which enables the individual to produce antibodies more rapidly and in greater quantity in response to secondary antigenic stimulus.'),
  'D007157': NtO(MH='Immunologic Surveillance', MS='The theory that T-cells monitor cell surfaces and detect structural changes in the plasma membrane and/or surface antigens of virally or neoplastically transformed cells.'),
  'D007158': NtO(MH='Immunologic Techniques', MS='Techniques used to demonstrate or measure an immune response, and to identify or measure antigens using antibodies.'),
  'D007159': NtO(MH='Immunologic Tests', MS='Immunologic techniques involved in diagnosis.'),
  'D007160': NtO(MH='Immunoproliferative Disorders', MS='Disorders characterized by abnormal proliferation of primary cells of the immune system or by excessive production of immunoglobulins.'),
  'D007161': NtO(MH='Immunoproliferative Small Intestinal Disease', MS='A condition that is caused by HYPERPLASIA of LYMPHOCYTES in the small intestine (INTESTINE, SMALL) and the mesenteric LYMPH NODES. These lymphocytes produce an anomalous alpha heavy chain protein. Generally, these IPSID patients have either concurrent LYMPHOMA or develop lymphoma within a few years. The disease was first described in the Mediterranean region and is characterized by malabsorption; WEIGHT LOSS; DIARRHEA; and STEATORRHEA.'),
  'D007162': NtO(MH='Immunoproteins', MS='Blood proteins whose activities affect or play a role in the functioning of the immune system.'),
  'D007163': NtO(MH='Immunosorbent Techniques', MS='Techniques for removal by adsorption and subsequent elution of a specific antibody or antigen using an immunosorbent containing the homologous antigen or antibody.'),
  'D007164': NtO(MH='Immunosorbents', MS='An insoluble support for an ANTIGEN or ANTIBODIES that is used in AFFINITY CHROMATOGRAPHY to adsorb the homologous antibody or antigen from a mixture. Many different substances are used, among them SEPHAROSE; GLUTARALDEHYDE; copolymers of ANHYDRIDES; polyacrylamides, etc.'),
  'D007165': NtO(MH='Immunosuppression', MS="Deliberate prevention or diminution of the host's immune response. It may be nonspecific as in the administration of immunosuppressive agents (drugs or radiation) or by lymphocyte depletion or may be specific as in desensitization or the simultaneous administration of antigen and immunosuppressive drugs."),
  'D007166': NtO(MH='Immunosuppressive Agents', MS='Agents that suppress immune function by one of several mechanisms of action. Classical cytotoxic immunosuppressants act by inhibiting DNA synthesis. Others may act through activation of T-CELLS or by inhibiting the activation of HELPER CELLS. While immunosuppression has been brought about in the past primarily to prevent rejection of transplanted organs, new applications involving mediation of the effects of INTERLEUKINS and other CYTOKINES are emerging.'),
  'D007167': NtO(MH='Immunotherapy', MS="Manipulation of the host's immune system in treatment of disease. It includes both active and passive immunization as well as immunosuppressive therapy to prevent graft rejection."),
  'D007168': NtO(MH='IMP Dehydrogenase', MS="An enzyme that catalyzes the dehydrogenation of inosine 5'-phosphate to xanthosine 5'-phosphate in the presence of NAD. EC 1.1.1.205."),
  'D007169': NtO(MH='Impetigo', MS='A common superficial bacterial infection caused by STAPHYLOCOCCUS AUREUS or group A beta-hemolytic streptococci. Characteristics include pustular lesions that rupture and discharge a thin, amber-colored fluid that dries and forms a crust. This condition is commonly located on the face, especially about the mouth and nose.'),
  'D007171': NtO(MH='Implosive Therapy', MS='A method for extinguishing anxiety by a saturation exposure to the feared stimulus situation or its substitute.'),
  'D007172': NtO(MH='Erectile Dysfunction', MS='The inability in the male to have a PENILE ERECTION due to psychological or organ dysfunction.'),
  'D007173': NtO(MH='Imprinting, Psychological', MS='A particular kind of learning characterized by occurrence in very early life, rapidity of acquisition, and relative insusceptibility to forgetting or extinction. Imprinted behavior includes most (or all) behavior commonly called instinctive, but imprinting is used purely descriptively.'),
  'D007174': NtO(MH='Disruptive, Impulse Control, and Conduct Disorders', MS='Disorders whose essential features are the failure to resist an impulse, drive, or temptation to perform an act that is harmful to the individual or to others. Individuals experience an increased sense of tension prior to the act and pleasure, gratification or release of tension at the time of committing the act.'),
  'D007175': NtO(MH='Impulsive Behavior', MS='An act performed without delay, reflection, voluntary direction or obvious control in response to a stimulus.'),
  'D007177': NtO(MH='Inappropriate ADH Syndrome', MS='A condition of HYPONATREMIA and renal salt loss attributed to overexpansion of BODY FLUIDS resulting from sustained release of ANTIDIURETIC HORMONES which stimulates renal resorption of water. It is characterized by normal KIDNEY function, high urine OSMOLALITY, low serum osmolality, and neurological dysfunction. Etiologies include ADH-producing neoplasms, injuries or diseases involving the HYPOTHALAMUS, the PITUITARY GLAND, and the LUNG. This syndrome can also be drug-induced.'),
  'D007178': NtO(MH='Inbreeding', MS='The mating of plants or non-human animals which are closely related genetically.'),
  'D007179': NtO(MH='Incest', MS='Sexual intercourse between persons so closely related that they are forbidden by law to marry.'),
  'D007180': NtO(MH='Incisor', MS='Any of the eight frontal teeth (four maxillary and four mandibular) having a sharp incisal edge for cutting food and a single root, which occurs in man both as a deciduous and a permanent tooth. (Jablonski, Dictionary of Dentistry, 1992, p820)'),
  'D007181': NtO(MH='Inclusion Bodies, Viral', MS='An area showing altered staining behavior in the nucleus or cytoplasm of a virus-infected cell. Some inclusion bodies represent "virus factories" in which viral nucleic acid or protein is being synthesized; others are merely artifacts of fixation and staining. One example, Negri bodies, are found in the cytoplasm or processes of nerve cells in animals that have died from rabies.'),
  'D007182': NtO(MH='Income', MS='Revenues or receipts accruing from business enterprise, labor, or invested capital.'),
  'D007183': NtO(MH='Income Tax', MS='Tax on the net income of an individual, organization, or business.'),
  'D007184': NtO(MH='Incontinentia Pigmenti', MS='A genodermatosis occurring mostly in females and characterized by skin changes in three phases - vesiculobullous, verrucous papillomatous, and macular melanodermic. Hyperpigmentation is bizarre and irregular. Sixty percent of patients have abnormalities of eyes, teeth, central nervous system, and skin appendages.'),
  'D007185': NtO(MH='Incubators', MS='Insulated enclosures in which temperature, humidity, and other environmental conditions can be regulated at levels optimal for growth, hatching, reproduction, or metabolic reactions.'),
  'D007186': NtO(MH='Incubators, Infant', MS='Electrically powered devices that are intended to assist in the maintenance of the thermal balance of infants, principally by controlling the air temperature and humidity in an enclosure. (from UMDNS, 1999)'),
  'D007187': NtO(MH='Incunabula as Topic', MS='Works about books printed before the year before 1501.'),
  'D007188': NtO(MH='Incus', MS='One of three ossicles of the middle ear. It conducts sound vibrations from the MALLEUS to the STAPES.'),
  'D007189': NtO(MH='Indans', MS='Aryl CYCLOPENTANES that are a reduced (protonated) form of INDENES.'),
  'D007190': NtO(MH='Indapamide', MS='A benzamide-sulfonamide-indole derived DIURETIC that functions by inhibiting SODIUM CHLORIDE SYMPORTERS.'),
  'D007191': NtO(MH='Indazoles', MS=None),
  'D007192': NtO(MH='Indenes', MS='A family of fused-ring hydrocarbons isolated from coal tar that act as intermediates in various chemical reactions and are used in the production of coumarone-indene resins.'),
  'D007193': NtO(MH='Independent Practice Associations', MS='A partnership, corporation, association, or other legal entity that enters into an arrangement for the provision of services with persons who are licensed to practice medicine, osteopathy, and dentistry, and with other care personnel. Under an IPA arrangement, licensed professional persons provide services through the entity in accordance with a mutually accepted compensation arrangement, while retaining their private practices. Services under the IPA are marketed through a prepaid health plan. (From Facts on File Dictionary of Health Care Management, 1988)'),
  'D007194': NtO(MH='India', MS='A country in southern Asia, bordering the Arabian Sea and the Bay of Bengal, between Burma and Pakistan. The capitol is New Delhi.'),
  'D007195': NtO(MH='Indian Ocean Islands', MS='Numerous islands in the Indian Ocean situated east of Madagascar, north to the Arabian Sea and east to Sri Lanka. Included are COMOROS (republic), MADAGASCAR (republic), Maldives (republic), MAURITIUS (parliamentary democracy), Pemba (administered by Tanzania), REUNION (a department of France), and SEYCHELLES (republic).'),
  'D007196': NtO(MH='Indiana', MS='State bounded on the north by Lake Michigan and Michigan, on the east by Ohio, on the south by Kentucky, and on the west by Illinois.'),
  'D007197': NtO(MH='Indians, Central American', MS='Individual members of Central American ethnic groups with ancient historic ancestral origins in Asia. Mexican Indians are not included.'),
  'D007198': NtO(MH='Indians, North American', MS='Individual members of North American ethnic groups with ancient historic ancestral origins in Asia.'),
  'D007199': NtO(MH='Indians, South American', MS='Individual members of South American ethnic groups with historic ancestral origins in Asia.'),
  'D007200': NtO(MH='Indican', MS='A substance occurring in the urine of mammals and also in blood plasma as the normal metabolite of tryptophan. An increased urinary excretion of indican is seen in Hartnup disease from the bacterial degradation of unabsorbed tryptophan.'),
  'D007201': NtO(MH='Indicator Dilution Techniques', MS='Methods for assessing flow through a system by injection of a known quantity of an indicator, such as a dye, radionuclide, or chilled liquid, into the system and monitoring its concentration over time at a specific point in the system. (From Dorland, 28th ed)'),
  'D007202': NtO(MH='Indicators and Reagents', MS="Substances used for the detection, identification, analysis, etc. of chemical, biological, or pathologic processes or conditions. Indicators are substances that change in physical appearance, e.g., color, at or approaching the endpoint of a chemical titration, e.g., on the passage between acidity and alkalinity. Reagents are substances used for the detection or determination of another substance by chemical or microscopical means, especially analysis. Types of reagents are precipitants, solvents, oxidizers, reducers, fluxes, and colorimetric reagents. (From Grant & Hackh's Chemical Dictionary, 5th ed, p301, p499)"),
  'D007203': NtO(MH='Indigo Carmine', MS='Indolesulfonic acid used as a dye in renal function testing for the detection of nitrates and chlorates, and in the testing of milk.'),
  'D007204': NtO(MH='Indium', MS='A metallic element, atomic number 49, atomic weight 114.818, symbol In. It is named from its blue line in the spectrum.'),
  'D007205': NtO(MH='Indium Radioisotopes', MS='Unstable isotopes of indium that decay or disintegrate emitting radiation. In atoms with atomic weights 106-112, 113m, 114, and 116-124 are radioactive indium isotopes.'),
  'D007206': NtO(MH='Individuality', MS='Those psychological characteristics which differentiate individuals from one another.'),
  'D007207': NtO(MH='Individuation', MS='A process of differentiation having for its goal the development of the individual personality.'),
  'D007208': NtO(MH='Indocyanine Green', MS='A tricarbocyanine dye that is used diagnostically in liver function tests and to determine blood volume and cardiac output.'),
  'D007209': NtO(MH='Indole-3-Glycerol-Phosphate Synthase', MS='An enzyme in the tryptophan biosynthetic pathway. EC 4.1.1.48.'),
  'D007210': NtO(MH='Indoleacetic Acids', MS='Acetic acid derivatives of the heterocyclic compound indole. (Merck Index, 11th ed)'),
  'D007211': NtO(MH='Indoles', MS='Benzopyrroles with the nitrogen at the number one carbon adjacent to the benzyl portion, in contrast to ISOINDOLES which have the nitrogen away from the six-membered ring.'),
  'D007212': NtO(MH='Indolizines', MS=None),
  'D007213': NtO(MH='Indomethacin', MS='A non-steroidal anti-inflammatory agent (NSAID) that inhibits CYCLOOXYGENASE, which is necessary for the formation of PROSTAGLANDINS and other AUTACOIDS. It also inhibits the motility of POLYMORPHONUCLEAR LEUKOCYTES.'),
  'D007214': NtO(MH='Indonesia', MS='A republic stretching from the Indian Ocean east to New Guinea, comprising six main islands: Java, Sumatra, Bali, Kalimantan (the Indonesian portion of the island of Borneo), Sulawesi (formerly known as the Celebes) and Irian Jaya (the western part of New Guinea). Its capital is Djakarta. The ethnic groups living there are largely Chinese, Arab, Eurasian, Indian, and Pakistani; 85% of the peoples are of the Islamic faith.'),
  'D007215': NtO(MH='Indophenol', MS="A deep blue dye (with the formula OC6H4NC6H4OH) used to detect AMMONIA in a common test called the Berthelot's reaction and to detect PARACETAMOL by spectrophotometry."),
  'D007216': NtO(MH='Indoprofen', MS='A drug that has analgesic and anti-inflammatory properties. Following reports of adverse reactions including reports of carcinogenicity in animal studies it was withdrawn from the market worldwide. (From Martindale, The Extra Pharmacopoeia, 30th ed, p21)'),
  'D007217': NtO(MH='Indoramin', MS='An alpha-1 adrenergic antagonist that is commonly used as an antihypertensive agent.'),
  'D007218': NtO(MH='Industrial Microbiology', MS='The study, utilization, and manipulation of those microorganisms capable of economically producing desirable substances or changes in substances, and the control of undesirable microorganisms.'),
  'D007219': NtO(MH='Industrial Oils', MS='Oils which are used in industrial or commercial applications.'),
  'D007220': NtO(MH='Industrial Waste', MS='Worthless, damaged, defective, superfluous or effluent material from industrial operations.'),
  'D007221': NtO(MH='Industry', MS='Any enterprise centered on the processing, assembly, production, or marketing of a line of products, services, commodities, or merchandise, in a particular field often named after its principal product. Examples include the automobile, fishing, music, publishing, insurance, and textile industries.'),
  'D007222': NtO(MH='Inert Gas Narcosis', MS='Progressive mental disturbances and unconsciousness due to breathing mixtures of oxygen and inert gases (argon, helium, xenon, krypton, and atmospheric nitrogen) at high pressure.'),
  'D007223': NtO(MH='Infant', MS='A child between 1 and 23 months of age.'),
  'D007224': NtO(MH='Infant Care', MS='Care of infants in the home or institution.'),
  'D007225': NtO(MH='Infant Food', MS='Food processed and manufactured for the nutritional health of children in their first year of life.'),
  'D007226': NtO(MH='Infant Mortality', MS='Postnatal deaths from BIRTH to 365 days after birth in a given population. Postneonatal mortality represents deaths between 28 days and 365 days after birth (as defined by National Center for Health Statistics). Neonatal mortality represents deaths from birth to 27 days after birth.'),
  'D007227': NtO(MH='Infant Nutritional Physiological Phenomena', MS='Nutritional physiology of children from birth to 2 years of age.'),
  'D007228': NtO(MH='Infant Nutrition Disorders', MS='Disorders caused by nutritional imbalance, either overnutrition or undernutrition, occurring in infants ages 1 month to 24 months.'),
  'D007229': NtO(MH='Infant Welfare', MS='Organized efforts by communities or organizations to improve the health and well-being of infants.'),
  'D007230': NtO(MH='Infant, Low Birth Weight', MS='An infant having a birth weight of 2500 gm. (5.5 lb.) or less but INFANT, VERY LOW BIRTH WEIGHT is available for infants having a birth weight of 1500 grams (3.3 lb.) or less.'),
  'D007231': NtO(MH='Infant, Newborn', MS='An infant during the first 28 days after birth.'),
  'D007232': NtO(MH='Infant, Newborn, Diseases', MS='Diseases of newborn infants present at birth (congenital) or developing within the first month of birth. It does not include hereditary diseases not manifesting at birth or within the first 30 days of life nor does it include inborn errors of metabolism. Both HEREDITARY DISEASES and METABOLISM, INBORN ERRORS are available as general concepts.'),
  'D007233': NtO(MH='Infant, Postmature', MS='An infant born at or after 42 weeks of gestation.'),
  'D007234': NtO(MH='Infant, Premature', MS='A human infant born before 37 weeks of GESTATION.'),
  'D007235': NtO(MH='Infant, Premature, Diseases', MS='Diseases that occur in PREMATURE INFANTS.'),
  'D007236': NtO(MH='Infant, Small for Gestational Age', MS='An infant having a birth weight lower than expected for its gestational age.'),
  'D007237': NtO(MH='Infanticide', MS='The killing of infants at birth or soon after.'),
  'D007238': NtO(MH='Infarction', MS='Formation of an infarct, which is NECROSIS in tissue due to local ISCHEMIA resulting from obstruction of BLOOD CIRCULATION, most commonly by a THROMBUS or EMBOLUS.'),
  'D007239': NtO(MH='Infections', MS='Invasion of the host organism by microorganisms or their toxins or by parasites that can cause pathological conditions or diseases.'),
  'D007240': NtO(MH='Equartevirus', MS=None),
  'D007241': NtO(MH='Infectious Bovine Rhinotracheitis', MS='A herpesvirus infection of CATTLE characterized by INFLAMMATION and NECROSIS of the mucous membranes of the upper RESPIRATORY TRACT.'),
  'D007242': NtO(MH='Herpesvirus 1, Bovine', MS='A species of VARICELLOVIRUS that causes INFECTIOUS BOVINE RHINOTRACHEITIS and other associated syndromes in CATTLE.'),
  'D007243': NtO(MH='Infectious bursal disease virus', MS='A species of AVIBIRNAVIRUS causing severe inflammation of the bursa of Fabricius in chickens and other fowl. Transmission is thought to be through contaminated feed or water. Vaccines have been used with varying degrees of success.'),
  'D007244': NtO(MH='Infectious Mononucleosis', MS='A common, acute infection usually caused by the Epstein-Barr virus (HERPESVIRUS 4, HUMAN). There is an increase in mononuclear white blood cells and other atypical lymphocytes, generalized lymphadenopathy, splenomegaly, and occasionally hepatomegaly with hepatitis.'),
  'D007245': NtO(MH='Inferior Colliculi', MS='The posterior pair of the quadrigeminal bodies which contain centers for auditory function.'),
  'D007246': NtO(MH='Infertility', MS='A reduced or absent capacity to reproduce.'),
  'D007247': NtO(MH='Infertility, Female', MS='Diminished or absent ability of a female to achieve conception.'),
  'D007248': NtO(MH='Infertility, Male', MS='The inability of the male to effect FERTILIZATION of an OVUM after a specified period of unprotected intercourse. Male sterility is permanent infertility.'),
  'D007249': NtO(MH='Inflammation', MS='A pathological process characterized by injury or destruction of tissues caused by a variety of cytologic and chemical reactions. It is usually manifested by typical signs of pain, heat, redness, swelling, and loss of function.'),
  'D007250': NtO(MH='Inflation, Economic', MS='An increase in the volume of money and credit relative to available goods resulting in a substantial and continuing rise in the general price level.'),
  'D007251': NtO(MH='Influenza, Human', MS='An acute viral infection in humans involving the respiratory tract. It is marked by inflammation of the NASAL MUCOSA; the PHARYNX; and conjunctiva, and by headache and severe, often generalized, myalgia.'),
  'D007252': NtO(MH='Influenza Vaccines', MS='Vaccines used to prevent infection by viruses in the family ORTHOMYXOVIRIDAE. It includes both killed and attenuated vaccines. The composition of the vaccines is changed each year in response to antigenic shifts and changes in prevalence of influenza virus strains. The flu vaccines may be mono- or multi-valent, which contains one or more INFLUENZAVIRUS A and INFLUENZAVIRUS B strains.'),
  'D007253': NtO(MH='Information Centers', MS='Facilities for collecting and organizing information. They may be specialized by subject field, type of source material, persons served, location, or type of services.'),
  'D007254': NtO(MH='Information Science', MS='The field of knowledge, theory, and technology dealing with the collection of facts and figures, and the processes and methods involved in their manipulation, storage, dissemination, publication, and retrieval. It includes the fields of COMMUNICATION; PUBLISHING; LIBRARY SCIENCE; and informatics.'),
  'D007255': NtO(MH='Information Services', MS='Organized services to provide information on any questions an individual might have using databases and other sources. (From Random House Unabridged Dictionary, 2d ed)'),
  'D007256': NtO(MH='Information Systems', MS='Integrated set of files, procedures, and equipment for the storage, manipulation, and retrieval of information.'),
  'D007257': NtO(MH='Information Theory', MS='An interdisciplinary study dealing with the transmission of messages or signals, or the communication of information. Information theory does not directly deal with meaning or content, but with physical representations that have meaning or content. It overlaps considerably with communication theory and CYBERNETICS.'),
  'D007258': NtO(MH='Informed Consent', MS='Voluntary authorization, by a patient or research subject, with full comprehension of the risks involved, for diagnostic or investigative procedures, and for medical and surgical treatment.'),
  'D007259': NtO(MH='Infrared Rays', MS='That portion of the electromagnetic spectrum usually sensed as heat. Infrared wavelengths are longer than those of visible light, extending into the microwave frequencies. They are used therapeutically as heat, and also to warm food in restaurants.'),
  'D007260': NtO(MH='Infusion Pumps', MS='Fluid propulsion systems driven mechanically, electrically, or osmotically that are used to inject (or infuse) over time agents into a patient or experimental animal; used routinely in hospitals to maintain a patent intravenous line, to administer antineoplastic agents and other drugs in thromboembolism, heart disease, diabetes mellitus (INSULIN INFUSION SYSTEMS is also available), and other disorders.'),
  'D007261': NtO(MH='Infusions, Intra-Arterial', MS='Regional infusion of drugs via an arterial catheter. Often a pump is used to impel the drug through the catheter. Used in therapy of cancer, upper gastrointestinal hemorrhage, infection, and peripheral vascular disease.'),
  'D007262': NtO(MH='Infusions, Intravenous', MS='The long-term (minutes to hours) administration of a fluid into the vein through venipuncture, either by letting the fluid flow by gravity or by pumping it.'),
  'D007263': NtO(MH='Infusions, Parenteral', MS='The administration of liquid medication, nutrient, or other fluid through some other route than the alimentary canal, usually over minutes or hours, either by gravity flow or often by infusion pumping.'),
  'D007264': NtO(MH='Inguinal Canal', MS='The tunnel in the lower anterior ABDOMINAL WALL through which the SPERMATIC CORD, in the male; ROUND LIGAMENT, in the female; nerves; and vessels pass. Its internal end is at the deep inguinal ring and its external end is at the superficial inguinal ring.'),
  'D007265': NtO(MH='Inhibins', MS='Glycoproteins that inhibit pituitary FOLLICLE STIMULATING HORMONE secretion. Inhibins are secreted by the Sertoli cells of the testes, the granulosa cells of the ovarian follicles, the placenta, and other tissues. Inhibins and ACTIVINS are modulators of FOLLICLE STIMULATING HORMONE secretions; both groups belong to the TGF-beta superfamily, as the TRANSFORMING GROWTH FACTOR BETA. Inhibins consist of a disulfide-linked heterodimer with a unique alpha linked to either a beta A or a beta B subunit to form inhibin A or inhibin B, respectively'),
  'D007266': NtO(MH='Inhibition, Psychological', MS='The interference with or prevention of a behavioral or verbal response even though the stimulus for that response is present; in psychoanalysis the unconscious restraining of an instinctual process.'),
  'D007267': NtO(MH='Injections', MS='Introduction of substances into the body using a needle and syringe.'),
  'D007268': NtO(MH='Injections, Epidural', MS='The injection of drugs, most often analgesics, into the spinal canal without puncturing the dura mater.'),
  'D007269': NtO(MH='Injections, Intra-Arterial', MS='Delivery of drugs into an artery.'),
  'D007270': NtO(MH='Injections, Intra-Articular', MS='Methods of delivering drugs into a joint space.'),
  'D007271': NtO(MH='Injections, Intradermal', MS='The forcing into the skin of liquid medication, nutrient, or other fluid through a hollow needle, piercing the top skin layer.'),
  'D007272': NtO(MH='Injections, Intralymphatic', MS='Injections into the lymph nodes or the lymphatic system.'),
  'D007273': NtO(MH='Injections, Intramuscular', MS='Forceful administration into a muscle of liquid medication, nutrient, or other fluid through a hollow needle piercing the muscle and any tissue covering it.'),
  'D007274': NtO(MH='Injections, Intraperitoneal', MS='Forceful administration into the peritoneal cavity of liquid medication, nutrient, or other fluid through a hollow needle piercing the abdominal wall.'),
  'D007275': NtO(MH='Injections, Intravenous', MS='Injections made into a vein for therapeutic or experimental purposes.'),
  'D007276': NtO(MH='Injections, Intraventricular', MS='Injections into the cerebral ventricles.'),
  'D007277': NtO(MH='Injections, Jet', MS='The injection of solutions into the skin by compressed air devices so that only the solution pierces the skin.'),
  'D007278': NtO(MH='Injections, Spinal', MS='Introduction of therapeutic agents into the spinal region using a needle and syringe.'),
  'D007279': NtO(MH='Injections, Subcutaneous', MS='Forceful administration under the skin of liquid medication, nutrient, or other fluid through a hollow needle piercing the skin.'),
  'D007280': NtO(MH='Disorders of Environmental Origin', MS='Disorders caused by external forces rather than by physiologic dysfunction or by pathogens.'),
  'D007281': NtO(MH='Ink', MS='Pigmented or colored fluid used for writing, staining, painting, drawing, printing, or copying images.'),
  'D007282': NtO(MH='Ink Blot Tests', MS='Projective tests utilizing ink blots to which a subject responds. They are used in personality diagnosis.'),
  'D007283': NtO(MH='Inlay Casting Wax', MS="A mixture of several dental waxes, usually containing paraffin wax, ceresin, beeswax, resins, and other natural and synthetic waxes. It is used for making patterns to determine the shape of the metallic framework and other parts of removable partial dentures. (From Jablonski's Dictionary of Dentistry, 1992, p868)"),
  'D007284': NtO(MH='Inlays', MS='Restorations of metal, porcelain, or plastic made to fit a cavity preparation, then cemented into the tooth. Onlays are restorations which fit into cavity preparations and overlay the occlusal surface of a tooth or teeth. Onlays are retained by frictional or mechanical factors.'),
  'D007287': NtO(MH='Inorganic Chemicals', MS='A broad class of substances encompassing all those that do not include carbon and its derivatives as their principal elements. However, carbides, carbonates, cyanides, cyanates, and carbon disulfide are included in this class.'),
  'D007288': NtO(MH='Inosine', MS='A purine nucleoside that has hypoxanthine linked by the N9 nitrogen to the C1 carbon of ribose. It is an intermediate in the degradation of purines and purine nucleosides to uric acid and in pathways of purine salvage. It also occurs in the anticodon of certain transfer RNA molecules. (Dorland, 28th ed)'),
  'D007289': NtO(MH='Cyclic IMP', MS="Inosine cyclic 3',5'-(hydrogen phosphate). An inosine nucleotide which acts as a mild inhibitor of the hydrolysis of cyclic AMP and cyclic GMP and as an inhibitor of cat heart cyclic AMP phosphodiesterase."),
  'D007290': NtO(MH='Inosine Diphosphate', MS='An inosine nucleotide containing a pyrophosphate group esterified to C5 of the sugar moiety.'),
  'D007291': NtO(MH='Inosine Monophosphate', MS="Inosine 5'-Monophosphate. A purine nucleotide which has hypoxanthine as the base and one phosphate group esterified to the sugar moiety."),
  'D007292': NtO(MH='Inosine Nucleotides', MS=None),
  'D007293': NtO(MH='Inosine Triphosphate', MS="Inosine 5'-(tetrahydrogen triphosphate). An inosine nucleotide containing three phosphate groups esterified to the sugar moiety. Synonym: IRPPP."),
  'D007294': NtO(MH='Inositol', MS='An isomer of glucose that has traditionally been considered to be a B vitamin although it has an uncertain status as a vitamin and a deficiency syndrome has not been identified in man. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1379) Inositol phospholipids are important in signal transduction.'),
  'D007295': NtO(MH='Inositol Phosphates', MS='Phosphoric acid esters of inositol. They include mono- and polyphosphoric acid esters, with the exception of inositol hexaphosphate which is PHYTIC ACID.'),
  'D007296': NtO(MH='Myo-Inositol-1-Phosphate Synthase', MS='An enzyme that catalyzes the formation of myo-inositol-1-phosphate from glucose-6-phosphate in the presence of NAD. EC 5.5.1.4.'),
  'D007297': NtO(MH='Inpatients', MS='Persons admitted to health facilities which provide board and room, for the purpose of observation, care, diagnosis or treatment.'),
  'D007298': NtO(MH='Insanity Defense', MS="A legal concept that an accused is not criminally responsible if, at the time of committing the act, the person was laboring under such a defect of reason from disease of the mind as not to know the nature and quality of the act done or if the act was known, to not have known that what was done was wrong. (From Black's Law Dictionary, 6th ed)"),
  'D007299': NtO(MH='Insect Bites and Stings', MS='Bites and stings inflicted by insects.'),
  'D007300': NtO(MH='Insect Control', MS='The reduction or regulation of the population of noxious, destructive, or dangerous insects through chemical, biological, or other means.'),
  'D007301': NtO(MH='Insect Hormones', MS='Hormones secreted by insects. They influence their growth and development. Also synthetic substances that act like insect hormones.'),
  'D007302': NtO(MH='Insect Repellents', MS='Substances causing insects to turn away from them or reject them as food.'),
  'D007303': NtO(MH='Insect Vectors', MS='Insects that transmit infective organisms from one host to another or from an inanimate reservoir to an animate host.'),
  'D007304': NtO(MH='Insect Viruses', MS='Viruses infecting insects, the largest family being BACULOVIRIDAE.'),
  'D007305': NtO(MH='Insecticide Resistance', MS='The development by insects of resistance to insecticides.'),
  'D007306': NtO(MH='Insecticides', MS='Pesticides designed to control insects that are harmful to man. The insects may be directly harmful, as those acting as disease vectors, or indirectly harmful, as destroyers of crops, food products, or textile fabrics.'),
  'D007312': NtO(MH='Insectivora', MS='An order of insect eating MAMMALS including MOLES; SHREWS; HEDGEHOGS and tenrecs.'),
  'D007313': NtO(MH='Insecta', MS='Members of the phylum ARTHROPODA composed or organisms characterized by division into three parts: head, thorax, and abdomen. They are the dominant group of animals on earth with several hundred thousand different kinds. Three orders, HEMIPTERA; DIPTERA; and SIPHONAPTERA; are of medical interest in that they cause disease in humans and animals. (From Borror et al., An Introduction to the Study of Insects, 4th ed, p1).'),
  'D007314': NtO(MH='Insemination', MS='The deposit of SEMEN or SPERMATOZOA into the VAGINA to facilitate FERTILIZATION.'),
  'D007315': NtO(MH='Insemination, Artificial', MS='Artificial introduction of SEMEN or SPERMATOZOA into the VAGINA to facilitate FERTILIZATION.'),
  'D007316': NtO(MH='Insemination, Artificial, Heterologous', MS="Human artificial insemination in which the semen used is that of a man other than the woman's husband."),
  'D007317': NtO(MH='Insemination, Artificial, Homologous', MS="Human artificial insemination in which the husband's semen is used."),
  'D007318': NtO(MH='Inservice Training', MS='On the job training programs for personnel carried out within an institution or agency. It includes orientation programs.'),
  'D007319': NtO(MH='Sleep Initiation and Maintenance Disorders', MS='Disorders characterized by impairment of the ability to initiate or maintain sleep. This may occur as a primary disorder or in association with another medical or psychiatric condition.'),
  'D007320': NtO(MH='Inspiratory Capacity', MS='The maximum volume of air that can be inspired after reaching the end of a normal, quiet expiration. It is the sum of the TIDAL VOLUME and the INSPIRATORY RESERVE VOLUME. Common abbreviation is IC.'),
  'D007321': NtO(MH='Inspiratory Reserve Volume', MS='The extra volume of air that can be inspired with maximal effort after reaching the end of a normal, quiet inspiration. Common abbreviation is IRV.'),
  'D007322': NtO(MH='Instillation, Drug', MS='The administration of therapeutic agents drop by drop, as eye drops, ear drops, or nose drops. It is also administered into a body space or cavity through a catheter. It differs from THERAPEUTIC IRRIGATION in that the irrigate is removed within minutes, but the instillate is left in place.'),
  'D007323': NtO(MH='Instinct', MS='Stereotyped patterns of response, characteristic of a given species, that have been phylogenetically adapted to a specific type of situation.'),
  'D007324': NtO(MH='Institutional Management Teams', MS='Administrator-selected management groups who are responsible for making decisions pertaining to the provision of integrated direction for various institutional functions.'),
  'D007325': NtO(MH='Institutional Practice', MS='Professional practice as an employee or contractee of a health care institution.'),
  'D007326': NtO(MH='Institutionalization', MS='The caring for individuals in institutions and their adaptation to routines characteristic of the institutional environment, and/or their loss of adaptation to life outside the institution.'),
  'D007327': NtO(MH='Insufflation', MS='The act of blowing a powder, vapor, or gas into any body cavity for experimental, diagnostic, or therapeutic purposes.'),
  'D007328': NtO(MH='Insulin', MS='A 51-amino acid pancreatic hormone that plays a major role in the regulation of glucose metabolism, directly by suppressing endogenous glucose production (GLYCOGENOLYSIS; GLUCONEOGENESIS) and indirectly by suppressing GLUCAGON secretion and LIPOLYSIS. Native insulin is a globular protein comprised of a zinc-coordinated hexamer. Each insulin monomer containing two chains, A (21 residues) and B (30 residues), linked by two disulfide bonds. Insulin is used as a drug to control insulin-dependent diabetes mellitus (DIABETES MELLITUS, TYPE 1).'),
  'D007329': NtO(MH='Insulin Antagonists', MS='Compounds which inhibit or antagonize the biosynthesis or action of insulin.'),
  'D007330': NtO(MH='Insulin Antibodies', MS='Antibodies specific to INSULIN.'),
  'D007331': NtO(MH='Insulin Coma', MS='Severe HYPOGLYCEMIA induced by a large dose of exogenous INSULIN resulting in a COMA or profound state of unconsciousness from which the individual cannot be aroused.'),
  'D007332': NtO(MH='Insulin Infusion Systems', MS='Portable or implantable devices for infusion of insulin. Includes open-loop systems which may be patient-operated or controlled by a pre-set program and are designed for constant delivery of small quantities of insulin, increased during food ingestion, and closed-loop systems which deliver quantities of insulin automatically based on an electronic glucose sensor.'),
  'D007333': NtO(MH='Insulin Resistance', MS='Diminished effectiveness of INSULIN in lowering blood sugar levels: requiring the use of 200 units or more of insulin per day to prevent HYPERGLYCEMIA or KETOSIS.'),
  'D007334': NtO(MH='Insulin-Like Growth Factor I', MS='A well-characterized basic peptide believed to be secreted by the liver and to circulate in the blood. It has growth-regulating, insulin-like, and mitogenic activities. This growth factor has a major, but not absolute, dependence on GROWTH HORMONE. It is believed to be mainly active in adults in contrast to INSULIN-LIKE GROWTH FACTOR II, which is a major fetal growth factor.'),
  'D007335': NtO(MH='Insulin-Like Growth Factor II', MS='A well-characterized neutral peptide believed to be secreted by the LIVER and to circulate in the BLOOD. It has growth-regulating, insulin-like and mitogenic activities. The growth factor has a major, but not absolute, dependence on SOMATOTROPIN. It is believed to be a major fetal growth factor in contrast to INSULIN-LIKE GROWTH FACTOR I, which is a major growth factor in adults.'),
  'D007336': NtO(MH='Insulin, Isophane', MS='An intermediate-acting INSULIN preparation with onset time of 2 hours and duration of 24 hours. It is produced by crystallizing ZINC-insulin-PROTAMINES at neutral pH 7. Thus it is called neutral protamine Hagedorn for inventor Hans Christian Hagedorn.'),
  'D007339': NtO(MH='Insulysin', MS='An enzyme the catalyzes the degradation of insulin, glucagon and other polypeptides. It is inhibited by bacitracin, chelating agents EDTA and 1,10-phenanthroline, and by thiol-blocking reagents such as N-ethylmaleimide, but not phosphoramidon. (Eur J Biochem 1994;223:1-5) EC 3.4.24.56.'),
  'D007340': NtO(MH='Insulinoma', MS='A benign tumor of the PANCREATIC BETA CELLS. Insulinoma secretes excess INSULIN resulting in HYPOGLYCEMIA.'),
  'D007341': NtO(MH='Insurance', MS='Coverage by contract whereby one part indemnifies or guarantees another against loss by a specified contingency.'),
  'D007342': NtO(MH='Insurance Benefits', MS='Payments or services provided under stated circumstances under the terms of an insurance policy. In prepayment programs, benefits are the services the programs will provide at defined locations and to the extent needed.'),
  'D007343': NtO(MH='Insurance Carriers', MS='Organizations which assume the financial responsibility for the risks of policyholders.'),
  'D007344': NtO(MH='Insurance Claim Reporting', MS='The design, completion, and filing of forms with the insurer.'),
  'D007345': NtO(MH='Insurance Claim Review', MS='Review of claims by insurance companies to determine liability and amount of payment for various services. The review may also include determination of eligibility of the claimant or beneficiary or of the provider of the benefit; determination that the benefit is covered or not payable under another policy; or determination that the service was necessary and of reasonable cost and quality.'),
  'D007346': NtO(MH='Insurance, Accident', MS='Insurance providing coverage for physical injury suffered as a result of unavoidable circumstances.'),
  'D007347': NtO(MH='Insurance, Dental', MS='Insurance providing coverage for dental care.'),
  'D007348': NtO(MH='Insurance, Health', MS='Insurance providing coverage of medical, surgical, or hospital care in general or for which there is no specific heading.'),
  'D007349': NtO(MH='Insurance, Health, Reimbursement', MS='Payment by a third-party payer in a sum equal to the amount expended by a health care provider or facility for health services rendered to an insured or program beneficiary. (From Facts on File Dictionary of Health Care Management, 1988)'),
  'D007350': NtO(MH='Insurance, Hospitalization', MS='Health insurance providing benefits to cover or partly cover hospital expenses.'),
  'D007351': NtO(MH='Insurance, Liability', MS='Insurance against loss resulting from liability for injury or damage to the persons or property of others.'),
  'D007352': NtO(MH='Insurance, Life', MS='Insurance providing for payment of a stipulated sum to a designated beneficiary upon death of the insured.'),
  'D007353': NtO(MH='Insurance, Long-Term Care', MS='Health insurance to provide full or partial coverage for long-term home care services or for long-term nursing care provided in a residential facility such as a nursing home.'),
  'D007354': NtO(MH='Insurance, Major Medical', MS='Insurance providing a broad range of medical services and supplies, when prescribed by a physician, whether or not the patient is hospitalized. It frequently is an extension of a basic policy and benefits will not begin until the basic policy is exhausted.'),
  'D007355': NtO(MH='Insurance, Nursing Services', MS='Insurance providing benefits for the costs of care provided by nurses, especially nurse practitioners and nurse clinicians.'),
  'D007356': NtO(MH='Insurance, Pharmaceutical Services', MS='Insurance providing for payment of services rendered by the pharmacist. Services include the preparation and distribution of medical products.'),
  'D007357': NtO(MH='Insurance, Physician Services', MS='Insurance providing benefits for the costs of care by a physician which can be comprehensive or limited to surgical expenses or for care provided only in the hospital. It is frequently called "regular medical expense" or "surgical expense".'),
  'D007358': NtO(MH='Insurance, Psychiatric', MS='Insurance providing benefits to cover part or all of the psychiatric care.'),
  'D007359': NtO(MH='Insurance, Surgical', MS="A specific type of health insurance which provides surgeons' fees for specified amounts according to the type of surgery listed in the policy."),
  'D007360': NtO(MH='Intelligence', MS='The ability to learn and to deal with new situations and to deal effectively with tasks involving abstractions.'),
  'D007361': NtO(MH='Intelligence Tests', MS='Standardized tests that measure the present general ability or aptitude for intellectual performance.'),
  'D007362': NtO(MH='Intensive Care Units', MS='Hospital units providing continuous surveillance and care to acutely ill patients.'),
  'D007363': NtO(MH='Intensive Care Units, Neonatal', MS='Hospital units providing continuing surveillance and care to acutely ill newborn infants.'),
  'D007364': NtO(MH='Intercalating Agents', MS='Agents that are capable of inserting themselves between the successive bases in DNA, thus kinking, uncoiling or otherwise deforming it and therefore preventing its proper functioning. They are used in the study of DNA.'),
  'D007365': NtO(MH='Intercellular Junctions', MS='Direct contact of a cell with a neighboring cell. Most such junctions are too small to be resolved by light microscopy, but they can be visualized by conventional or freeze-fracture electron microscopy, both of which show that the interacting CELL MEMBRANE and often the underlying CYTOPLASM and the intervening EXTRACELLULAR SPACE are highly specialized in these regions. (From Alberts et al., Molecular Biology of the Cell, 2d ed, p792)'),
  'D007366': NtO(MH='Intercostal Muscles', MS='Respiratory muscles that arise from the lower border of one rib and insert into the upper border of the adjoining rib, and contract during inspiration or respiration. (From Stedman, 25th ed)'),
  'D007367': NtO(MH='Intercostal Nerves', MS='The ventral rami of the thoracic nerves from segments T1 through T11. The intercostal nerves supply motor and sensory innervation to the thorax and abdomen. The skin and muscles supplied by a given pair are called, respectively, a dermatome and a myotome.'),
  'D007368': NtO(MH='Interferometry', MS='Measurement of distances or movements by means of the phenomena caused by the interference of two rays of light (optical interferometry) or of sound (acoustic interferometry).'),
  'D007369': NtO(MH='Interferon Inducers', MS='Agents that promote the production and release of interferons. They include mitogens, lipopolysaccharides, and the synthetic polymers Poly A-U and Poly I-C. Viruses, bacteria, and protozoa have been also known to induce interferons.'),
  'D007370': NtO(MH='Interferon Type I', MS='Interferon secreted by leukocytes, fibroblasts, or lymphoblasts in response to viruses or interferon inducers other than mitogens, antigens, or allo-antigens. They include alpha- and beta-interferons (INTERFERON-ALPHA and INTERFERON-BETA).'),
  'D007371': NtO(MH='Interferon-gamma', MS='The major interferon produced by mitogenically or antigenically stimulated LYMPHOCYTES. It is structurally different from TYPE I INTERFERON and its major activity is immunoregulation. It has been implicated in the expression of CLASS II HISTOCOMPATIBILITY ANTIGENS in cells that do not normally produce them, leading to AUTOIMMUNE DISEASES.'),
  'D007372': NtO(MH='Interferons', MS='Proteins secreted by vertebrate cells in response to a wide variety of inducers. They confer resistance against many different viruses, inhibit proliferation of normal and malignant cells, impede multiplication of intracellular parasites, enhance macrophage and granulocyte phagocytosis, augment natural killer cell activity, and show several other immunomodulatory functions.'),
  'D007373': NtO(MH='Interinstitutional Relations', MS='The interactions between representatives of institutions, agencies, or organizations.'),
  'D007374': NtO(MH='Interior Design and Furnishings', MS='The planning of the furnishings and decorations of an architectural interior.'),
  'D007375': NtO(MH='Interleukin-1', MS='A soluble factor produced by MONOCYTES; MACROPHAGES, and other cells which activates T-lymphocytes and potentiates their response to mitogens or antigens. Interleukin-1 is a general term refers to either of the two distinct proteins, INTERLEUKIN-1ALPHA and INTERLEUKIN-1BETA. The biological effects of IL-1 include the ability to replace macrophage requirements for T-cell activation.'),
  'D007376': NtO(MH='Interleukin-2', MS='A soluble substance elaborated by antigen- or mitogen-stimulated T-LYMPHOCYTES which induces DNA synthesis in naive lymphocytes.'),
  'D007377': NtO(MH='Interleukin-3', MS='A multilineage cell growth factor secreted by LYMPHOCYTES; EPITHELIAL CELLS; and ASTROCYTES which stimulates clonal proliferation and differentiation of various types of blood and tissue cells.'),
  'D007378': NtO(MH='Interleukins', MS='Soluble factors which stimulate growth-related activities of leukocytes as well as other cell types. They enhance cell proliferation and differentiation, DNA synthesis, secretion of other biologically active molecules and responses to immune and inflammatory stimuli.'),
  'D007379': NtO(MH='Interlibrary Loans', MS='Copies or original works acquired from cooperating institutions on request.'),
  'D007380': NtO(MH='Intermediate Care Facilities', MS='Institutions which provide health-related care and services to individuals who do not require the degree of care which hospitals or skilled nursing facilities provide, but because of their physical or mental condition require care and services above the level of room and board.'),
  'D007381': NtO(MH='Intermediate Filament Proteins', MS='Filaments 7-11 nm in diameter found in the cytoplasm of all cells. Many specific proteins belong to this group, e.g., desmin, vimentin, prekeratin, decamin, skeletin, neurofilin, neurofilament protein, and glial fibrillary acid protein.'),
  'D007382': NtO(MH='Intermediate Filaments', MS='Cytoplasmic filaments intermediate in diameter (about 10 nanometers) between the microfilaments and the microtubules. They may be composed of any of a number of different proteins and form a ring around the cell nucleus.'),
  'D007383': NtO(MH='Intermittent Claudication', MS='A symptom complex characterized by pain and weakness in SKELETAL MUSCLE group associated with exercise, such as leg pain and weakness brought on by walking. Such muscle limpness disappears after a brief rest and is often relates to arterial STENOSIS; muscle ISCHEMIA; and accumulation of LACTATE.'),
  'D007384': NtO(MH='Intermittent Positive-Pressure Breathing', MS='Application of positive pressure to the inspiratory phase of spontaneous respiration.'),
  'D007385': NtO(MH='Intermittent Positive-Pressure Ventilation', MS='Application of positive pressure to the inspiratory phase when the patient has an artificial airway in place and is connected to a ventilator.'),
  'D007387': NtO(MH='Internal Mammary-Coronary Artery Anastomosis', MS='Direct myocardial revascularization in which the internal mammary artery is anastomosed to the right coronary artery, circumflex artery, or anterior descending coronary artery. The internal mammary artery is the most frequent choice, especially for a single graft, for coronary artery bypass surgery.'),
  'D007388': NtO(MH='Internal Medicine', MS='A medical specialty concerned with the diagnosis and treatment of diseases of the internal organ systems of adults.'),
  'D007389': NtO(MH='Internal-External Control', MS="Personality construct referring to an individual's perception of the locus of events as determined internally by his or her own behavior versus fate, luck, or external forces. (ERIC Thesaurus, 1996)."),
  'D007390': NtO(MH='International Agencies', MS='International organizations which provide health-related or other cooperative services.'),
  'D007391': NtO(MH='International Cooperation', MS='The interaction of persons or groups of persons representing various nations in the pursuit of a common goal or interest.'),
  'D007392': NtO(MH='International Council of Nurses', MS="An international professional organization composed of one association per country for the purpose of improving and developing nursing's contribution to the promotion of health and care of the sick."),
  'D007393': NtO(MH='International Educational Exchange', MS='The exchange of students or professional personnel between countries done under the auspices of an organization for the purpose of further education.'),
  'D007394': NtO(MH='International System of Units', MS='A system of physical units in which the fundamental quantities are length, time, mass, electric current, temperature, luminous intensity, and amount of substance, and the corresponding units are the meter, second, kilogram, ampere, kelvin, candela, and mole. The system has been given official status and recommended for universal use by the General Conference on Weights and Measures.'),
  'D007395': NtO(MH='Interneurons', MS='Most generally any NEURONS which are not motor or sensory. Interneurons may also refer to neurons whose AXONS remain within a particular brain region in contrast to projection neurons, which have axons projecting to other brain regions.'),
  'D007396': NtO(MH='Internship and Residency', MS='Programs of EDUCATION, MEDICAL, GRADUATE training to meet the requirements established by accrediting authorities.'),
  'D007397': NtO(MH='Internship, Nonmedical', MS='Advanced programs of training to meet certain professional requirements in fields other than medicine or dentistry, e.g., pharmacology, nutrition, nursing, etc.'),
  'D007398': NtO(MH='Interpersonal Relations', MS='The reciprocal interaction of two or more persons.'),
  'D007399': NtO(MH='Interphase', MS='The interval between two successive CELL DIVISIONS during which the CHROMOSOMES are not individually distinguishable. It is composed of the G phases (G1 PHASE; G0 PHASE; G2 PHASE) and S PHASE (when DNA replication occurs).'),
  'D007400': NtO(MH='Interprofessional Relations', MS='The reciprocal interaction of two or more professional individuals.'),
  'D007401': NtO(MH='Interrenal Gland', MS='Structures in fishes homologous to the cortical tissue of the mammalian adrenal gland; they are in close proximity to or imbedded in the kidney.'),
  'D007402': NtO(MH='Intertrigo', MS='A superficial dermatitis occurring on skin surfaces in contact with each other, such as the axillae, neck creases, intergluteal fold, between the toes, etc. Obesity is a predisposing factor. The condition is caused by moisture and friction and is characterized by erythema, maceration, burning, and exudation.'),
  'D007403': NtO(MH='Intervertebral Disc', MS='Any of the 23 plates of fibrocartilage found between the bodies of adjacent VERTEBRAE.'),
  'D007404': NtO(MH='Intervertebral Disc Chemolysis', MS='The dissolving of the NUCLEUS PULPOSUS, the semi-gelatinous tissue of a displaced INTERVERTEBRAL DISC. It is usually achieved by the direct injection of a proteolytic enzyme, especially CHYMOPAPAIN, into the herniated disc.'),
  'D007405': NtO(MH='Intervertebral Disc Displacement', MS='An INTERVERTEBRAL DISC in which the NUCLEUS PULPOSUS has protruded through surrounding ANNULUS FIBROSUS. This occurs most frequently in the lower lumbar region.'),
  'D007406': NtO(MH='Interview, Psychological', MS='A directed conversation aimed at eliciting information for psychiatric diagnosis, evaluation, treatment planning, etc. The interview may be conducted by a social worker or psychologist.'),
  'D007407': NtO(MH='Interviews as Topic', MS='Works about conversations with an individual or individuals held in order to obtain information about their background and other personal biographical data, their attitudes and opinions, etc. It includes works about school admission or job interviews.'),
  'D007408': NtO(MH='Intestinal Absorption', MS='Uptake of substances through the lining of the INTESTINES.'),
  'D007409': NtO(MH='Intestinal Atresia', MS='Congenital obliteration of the lumen of the intestine, with the ILEUM involved in 50% of the cases and the JEJUNUM and DUODENUM following in frequency. It is the most frequent cause of INTESTINAL OBSTRUCTION in NEWBORNS. (From Stedman, 25th ed)'),
  'D007410': NtO(MH='Intestinal Diseases', MS='Pathological processes in any segment of the INTESTINE from DUODENUM to RECTUM.'),
  'D007411': NtO(MH='Intestinal Diseases, Parasitic', MS='Infections of the INTESTINES with PARASITES, commonly involving PARASITIC WORMS. Infections with roundworms (NEMATODE INFECTIONS) and tapeworms (CESTODE INFECTIONS) are also known as HELMINTHIASIS.'),
  'D007412': NtO(MH='Intestinal Fistula', MS='An abnormal anatomical passage between the INTESTINE, and another segment of the intestine or other organs. External intestinal fistula is connected to the SKIN (enterocutaneous fistula). Internal intestinal fistula can be connected to a number of organs, such as STOMACH (gastrocolic fistula), the BILIARY TRACT (cholecystoduodenal fistula), or the URINARY BLADDER of the URINARY TRACT (colovesical fistula). Risk factors include inflammatory processes, cancer, radiation treatment, and surgical misadventures (MEDICAL ERRORS).'),
  'D007413': NtO(MH='Intestinal Mucosa', MS='Lining of the INTESTINES, consisting of an inner EPITHELIUM, a middle LAMINA PROPRIA, and an outer MUSCULARIS MUCOSAE. In the SMALL INTESTINE, the mucosa is characterized by a series of folds and abundance of absorptive cells (ENTEROCYTES) with MICROVILLI.'),
  'D007414': NtO(MH='Intestinal Neoplasms', MS='Tumors or cancer of the INTESTINES.'),
  'D007415': NtO(MH='Intestinal Obstruction', MS='Any impairment, arrest, or reversal of the normal flow of INTESTINAL CONTENTS toward the ANAL CANAL.'),
  'D007416': NtO(MH='Intestinal Perforation', MS='Opening or penetration through the wall of the INTESTINES.'),
  'D007417': NtO(MH='Intestinal Polyps', MS='Discrete abnormal tissue masses that protrude into the lumen of the INTESTINE. A polyp is attached to the intestinal wall either by a stalk, pedunculus, or by a broad base.'),
  'D007418': NtO(MH='Intestinal Pseudo-Obstruction', MS='A type of ILEUS, a functional not mechanical obstruction of the INTESTINES. This syndrome is caused by a large number of disorders involving the smooth muscles (MUSCLE, SMOOTH) or the NERVOUS SYSTEM.'),
  'D007419': NtO(MH='Intestinal Secretions', MS='Fluids originating from the epithelial lining of the intestines, adjoining exocrine glands and from organs such as the liver, which empty into the cavity of the intestines.'),
  'D007420': NtO(MH='Intestine, Large', MS='A segment of the LOWER GASTROINTESTINAL TRACT that includes the CECUM; the COLON; and the RECTUM.'),
  'D007421': NtO(MH='Intestine, Small', MS='The portion of the GASTROINTESTINAL TRACT between the PYLORUS of the STOMACH and the ILEOCECAL VALVE of the LARGE INTESTINE. It is divisible into three portions: the DUODENUM, the JEJUNUM, and the ILEUM.'),
  'D007422': NtO(MH='Intestines', MS='The section of the alimentary canal from the STOMACH to the ANAL CANAL. It includes the LARGE INTESTINE and SMALL INTESTINE.'),
  'D007423': NtO(MH='Intra-Aortic Balloon Pumping', MS="Counterpulsation in which a pumping unit synchronized with the patient's electrocardiogram rapidly fills a balloon in the aorta with helium or carbon dioxide in early diastole and evacuates the balloon at the onset of systole. As the balloon inflates, it raises aortic diastolic pressure, and as it deflates, it lowers aortic systolic pressure. The result is a decrease in left ventricular work and increased myocardial and peripheral perfusion."),
  'D007424': NtO(MH='Intracellular Fluid', MS='The fluid inside CELLS.'),
  'D007425': NtO(MH='Intracellular Membranes', MS='Thin structures that encapsulate subcellular structures or ORGANELLES in EUKARYOTIC CELLS. They include a variety of membranes associated with the CELL NUCLEUS; the MITOCHONDRIA; the GOLGI APPARATUS; the ENDOPLASMIC RETICULUM; LYSOSOMES; PLASTIDS; and VACUOLES.'),
  'D007426': NtO(MH='Genes, Intracisternal A-Particle', MS='Genes of IAP elements (a family of retrovirus-like genetic elements) which code for virus-like particles (IAPs) found regularly in rodent early embryos. ("Intracisternal" refers to the cisternae of the endoplasmic reticulum.) Under certain circumstances, such as DNA hypomethylation they are transcribed. Their transcripts are found in a variety of neoplasms, including plasmacytomas, neuroblastoma, rhabdomyosarcomas, teratocarcinomas, and colon carcinomas.'),
  'D007427': NtO(MH='Intracranial Pressure', MS='Pressure within the cranial cavity. It is influenced by brain mass, the circulatory system, CSF dynamics, and skull rigidity.'),
  'D007428': NtO(MH='Intradermal Tests', MS='Skin tests in which the sensitizer is injected.'),
  'D007429': NtO(MH='Intraocular Pressure', MS='The pressure of the fluids in the eye.'),
  'D007430': NtO(MH='Intraoperative Care', MS='Patient care procedures performed during the operation that are ancillary to the actual surgery. It includes monitoring, fluid therapy, medication, transfusion, anesthesia, radiography, and laboratory tests.'),
  'D007431': NtO(MH='Intraoperative Complications', MS='Complications that affect patients during surgery. They may or may not be associated with the disease for which the surgery is done, or within the same surgical procedure.'),
  'D007432': NtO(MH='Intraoperative Period', MS='The period during a surgical operation.'),
  'D007433': NtO(MH='Intrauterine Device Expulsion', MS='Spontaneous loss of INTRAUTERINE DEVICES from the UTERUS.'),
  'D007434': NtO(MH='Intrauterine Devices', MS='Contraceptive devices placed high in the uterine fundus.'),
  'D007435': NtO(MH='Intrauterine Devices, Copper', MS='Intrauterine contraceptive devices that depend on the release of metallic copper.'),
  'D007436': NtO(MH='Intrauterine Devices, Medicated', MS='Intrauterine devices that release contraceptive agents.'),
  'D007437': NtO(MH='Intrinsic Factor', MS='A glycoprotein secreted by the cells of the GASTRIC GLANDS that is required for the absorption of VITAMIN B 12 (cyanocobalamin). Deficiency of intrinsic factor leads to VITAMIN B 12 DEFICIENCY and ANEMIA, PERNICIOUS.'),
  'D007438': NtO(MH='Introns', MS='Sequences of DNA in the genes that are located between the EXONS. They are transcribed along with the exons but are removed from the primary gene transcript by RNA SPLICING to leave mature RNA. Some introns code for separate genes.'),
  'D007439': NtO(MH='Introversion, Psychological', MS="A state in which attention is largely directed inward upon one's self."),
  'D007440': NtO(MH='Intubation', MS='Introduction of a tube into a hollow organ to restore or maintain patency if obstructed. It is differentiated from CATHETERIZATION in that the insertion of a catheter is usually performed for the introducing or withdrawing of fluids from the body.'),
  'D007441': NtO(MH='Intubation, Gastrointestinal', MS='The insertion of a tube into the stomach, intestines, or other portion of the gastrointestinal tract to allow for the passage of food products, etc.'),
  'D007442': NtO(MH='Intubation, Intratracheal', MS='A procedure involving placement of a tube into the trachea through the mouth or nose in order to provide a patient with oxygen and anesthesia.'),
  'D007443': NtO(MH='Intussusception', MS='A form of intestinal obstruction caused by the PROLAPSE of a part of the intestine into the adjoining intestinal lumen. There are four types: colic, involving segments of the LARGE INTESTINE; enteric, involving only the SMALL INTESTINE; ileocecal, in which the ILEOCECAL VALVE prolapses into the CECUM, drawing the ILEUM along with it; and ileocolic, in which the ileum prolapses through the ileocecal valve into the COLON.'),
  'D007444': NtO(MH='Inulin', MS='A starch found in the tubers and roots of many plants. Since it is hydrolyzable to FRUCTOSE, it is classified as a fructosan. It has been used in physiologic investigation for determination of the rate of glomerular function.'),
  'D007445': NtO(MH='Inventories, Hospital', MS='Materials and equipment in stock; includes drugs in pharmacies, blood in blood banks, etc.'),
  'D007446': NtO(MH='Chromosome Inversion', MS='An aberration in which a chromosomal segment is deleted and reinserted in the same place but turned 180 degrees from its original orientation, so that the gene sequence for the segment is reversed with respect to that of the rest of the chromosome.'),
  'D007447': NtO(MH='Invertebrate Hormones', MS='Hormones produced by invertebrates, usually insects, mollusks, annelids, and helminths.'),
  'D007448': NtO(MH='Invertebrates', MS='Animals that have no spinal column.'),
  'D007449': NtO(MH='Investments', MS='Use for articles on the investing of funds for income or profit.'),
  'D007451': NtO(MH='Iodamide', MS='An ionic monomeric contrast medium. (From Martindale, The Extra Pharmacopoeia, 30th ed, p706)'),
  'D007452': NtO(MH='Iodates', MS='Inorganic salts of iodic acid (HIO3).'),
  'D007453': NtO(MH='Iodide Peroxidase', MS='A hemeprotein that catalyzes the oxidation of the iodide radical to iodine with the subsequent iodination of many organic compounds, particularly proteins. EC 1.11.1.8.'),
  'D007454': NtO(MH='Iodides', MS='Inorganic binary compounds of iodine or the I- ion.'),
  'D007455': NtO(MH='Iodine', MS='A nonmetallic element of the halogen group that is represented by the atomic symbol I, atomic number 53, and atomic weight of 126.90. It is a nutritionally essential element, especially important in thyroid hormone synthesis. In solution, it has anti-infective properties and is used topically.'),
  'D007456': NtO(MH='Iodine Isotopes', MS='Stable iodine atoms that have the same atomic number as the element iodine, but differ in atomic weight. I-127 is the only naturally occurring stable iodine isotope.'),
  'D007457': NtO(MH='Iodine Radioisotopes', MS='Unstable isotopes of iodine that decay or disintegrate emitting radiation. I atoms with atomic weights 117-139, except I 127, are radioactive iodine isotopes.'),
  'D007458': NtO(MH='Iodipamide', MS='A water-soluble radiographic contrast media for cholecystography and intravenous cholangiography.'),
  'D007459': NtO(MH='Iodized Oil', MS='A preparation of oil that contains covalently bound IODINE. It is commonly used as a RADIOCONTRAST AGENT and as a suspension medium for CHEMOTHERAPEUTIC AGENTS.'),
  'D007460': NtO(MH='Iodoacetamide', MS='An alkylating sulfhydryl reagent. Its actions are similar to those of iodoacetate.'),
  'D007461': NtO(MH='Iodoacetates', MS='Iodinated derivatives of acetic acid. Iodoacetates are commonly used as alkylating sulfhydryl reagents and enzyme inhibitors in biochemical research.'),
  'D007462': NtO(MH='Iodobenzenes', MS='Any derivative of BENZENE that contains IODINE.'),
  'D007463': NtO(MH='Iodobenzoates', MS='Benzoic acid esters or salts substituted with one or more iodine atoms.'),
  'D007464': NtO(MH='Clioquinol', MS='A potentially neurotoxic 8-hydroxyquinoline derivative long used as a topical anti-infective, intestinal antiamebic, and vaginal trichomonacide. The oral preparation has been shown to cause subacute myelo-optic neuropathy and has been banned worldwide.'),
  'D007465': NtO(MH='Iodohippuric Acid', MS='An iodine-containing compound used in pyelography as a radiopaque medium. If labeled with radioiodine, it can be used for studies of renal function.'),
  'D007466': NtO(MH='Iodophors', MS='Complexes of iodine and non-ionic SURFACE-ACTIVE AGENTS acting as carrier and solubilizing agent for the iodine in water. Iodophors usually enhance bactericidal activity of iodine, reduce vapor pressure and odor, minimize staining, and allow wide dilution with water. (From Merck Index, 11th ed)'),
  'D007467': NtO(MH='Iodoproteins', MS=None),
  'D007468': NtO(MH='Iodopyracet', MS='An ionic monomeric contrast medium that was formerly used for a variety of diagnostic procedures. (From Martindale, The Extra Pharmacopoeia, 30th ed, p706)'),
  'D007469': NtO(MH='Iodopyridones', MS='Iodinated pyridine derivatives that are often used as contrast media.'),
  'D007470': NtO(MH='Monoiodotyrosine', MS='A product from the iodination of tyrosine. In the biosynthesis of thyroid hormones (THYROXINE and TRIIODOTHYRONINE), tyrosine is first iodized to monoiodotyrosine.'),
  'D007471': NtO(MH='Ioglycamic Acid', MS='A radiopaque medium. It is a mixture of its meglumine and sodium salts and is used to visualize the biliary tract.'),
  'D007472': NtO(MH='Iohexol', MS='An effective non-ionic, water-soluble contrast agent which is used in myelography, arthrography, nephroangiography, arteriography, and other radiographic procedures. Its low systemic toxicity is the combined result of low chemotoxicity and low osmolality.'),
  'D007473': NtO(MH='Ion Channels', MS='Gated, ion-selective glycoproteins that traverse membranes. The stimulus for ION CHANNEL GATING can be due to a variety of stimuli such as LIGANDS, a TRANSMEMBRANE POTENTIAL DIFFERENCE, mechanical deformation or through INTRACELLULAR SIGNALING PEPTIDES AND PROTEINS.'),
  'D007474': NtO(MH='Ion Exchange', MS='Reversible chemical reaction between a solid, often one of the ION EXCHANGE RESINS, and a fluid whereby ions may be exchanged from one substance to another. This technique is used in water purification, in research, and in industry.'),
  'D007475': NtO(MH='Ion Exchange Resins', MS='High molecular weight, insoluble polymers which contain functional groups that are capable of undergoing exchange reactions (ION EXCHANGE) with either cations or anions.'),
  'D007476': NtO(MH='Ionophores', MS='Chemical agents that increase the permeability of biological or artificial lipid membranes to specific ions. Most ionophores are relatively small organic molecules that act as mobile carriers within membranes or coalesce to form ion permeable channels across membranes. Many are antibiotics, and many act as uncoupling agents by short-circuiting the proton gradient across mitochondrial membranes.'),
  'D007477': NtO(MH='Ions', MS='An atom or group of atoms that have a positive or negative electric charge due to a gain (negative charge) or loss (positive charge) of one or more electrons. Atoms with a positive charge are known as CATIONS; those with a negative charge are ANIONS.'),
  'D007478': NtO(MH='Iontophoresis', MS='Therapeutic introduction of ions of soluble salts into tissues by means of electric current. In medical literature it is commonly used to indicate the process of increasing the penetration of drugs into surface tissues by the application of electric current. It has nothing to do with ION EXCHANGE; AIR IONIZATION nor PHONOPHORESIS, none of which requires current.'),
  'D007479': NtO(MH='Iopamidol', MS='A non-ionic, water-soluble contrast agent which is used in myelography, arthrography, nephroangiography, arteriography, and other radiological procedures.'),
  'D007480': NtO(MH='Iopanoic Acid', MS='Radiopaque medium used as diagnostic aid.'),
  'D007481': NtO(MH='Iophendylate', MS='An inert iodine-containing agent which is opaque to X-RAYS. It is used mainly for BRAIN and SPINAL CORD visualization.'),
  'D007482': NtO(MH='Iothalamate Meglumine', MS='A radiopaque medium used for urography, angiography, venography, and myelography. It is highly viscous and binds to plasma proteins.'),
  'D007483': NtO(MH='Iothalamic Acid', MS='A contrast medium in diagnostic radiology with properties similar to those of diatrizoic acid. It is used primarily as its sodium and meglumine (IOTHALAMATE MEGLUMINE) salts.'),
  'D007484': NtO(MH='Iowa', MS='State bounded on the north by Minnesota, on the east by Wisconsin and Illinois, on the south by Missouri, and on the west by Nebraska and South Dakota.'),
  'D007485': NtO(MH='Ioxaglic Acid', MS='A low-osmolar, ionic contrast medium used in various radiographic procedures.'),
  'D007486': NtO(MH='Ipecac', MS='A syrup made from the dried rhizomes of two different species, CEPHAELIS ipecacuanha and C. acuminata. They contain EMETINE, cephaeline, psychotrine and other ISOQUINOLINES. Ipecac syrup is used widely as an emetic acting both locally on the gastric mucosa and centrally on the chemoreceptor trigger zone.'),
  'D007487': NtO(MH='Ipodate', MS='Ionic monomeric contrast media. Usually the sodium or calcium salts are used for examination of the gall bladder and biliary tract. (From Martindale, The Extra Pharmacopoeia, 30th ed, p704)'),
  'D007488': NtO(MH='Iprindole', MS='A tricyclic antidepressant that has actions and uses similar to those of AMITRIPTYLINE, but has only weak antimuscarinic and sedative effects. (From Martindale, The Extra Pharmacopoeia, 30th ed, p257)'),
  'D007490': NtO(MH='Iproniazid', MS='An irreversible inhibitor of monoamine oxidase types A and B that is used as an antidepressive agent. It has also been used as an antitubercular agent, but its use is limited by its toxicity.'),
  'D007491': NtO(MH='Ipronidazole', MS='An antihistomonal agent with low toxicity. It also promotes growth and feed utilization in poultry.'),
  'D007492': NtO(MH='Iran', MS='A country bordering the Gulf of Oman, the Persian Gulf, and the Caspian Sea, between Iraq and Pakistan. The capital is Tehran.'),
  'D007493': NtO(MH='Iraq', MS='A country bordering the Persian Gulf, between Iran and Kuwait. The capital is Baghdad.'),
  'D007494': NtO(MH='Ireland', MS='A country in western Europe, occupying five-sixths of the island of Ireland in the North Atlantic Ocean, west of Great Britain. The capital is Dublin.'),
  'D007495': NtO(MH='Iridium', MS='A metallic element with the atomic symbol Ir, atomic number 77, and atomic weight 192.22.'),
  'D007496': NtO(MH='Iridium Radioisotopes', MS='Unstable isotopes of iridium that decay or disintegrate emitting radiation. Ir atoms with atomic weights 182-190, 192, and 194-198 are radioactive iridium isotopes.'),
  'D007497': NtO(MH='Iridoviridae', MS='A family of large icosahedral DNA viruses infecting insects and poikilothermic vertebrates. Genera include IRIDOVIRUS; RANAVIRUS; Chloriridovirus; Megalocytivirus; and Lymphocystivirus.'),
  'D007498': NtO(MH='Iris', MS='The most anterior portion of the uveal layer, separating the anterior chamber from the posterior. It consists of two layers - the stroma and the pigmented epithelium. Color of the iris depends on the amount of melanin in the stroma on reflection from the pigmented epithelium.'),
  'D007499': NtO(MH='Iris Diseases', MS='Diseases, dysfunctions, or disorders of or located in the iris.'),
  'D007500': NtO(MH='Iritis', MS='Inflammation of the iris characterized by circumcorneal injection, aqueous flare, keratotic precipitates, and constricted and sluggish pupil along with discoloration of the iris.'),
  'D007501': NtO(MH='Iron', MS='A metallic element with atomic symbol Fe, atomic number 26, and atomic weight 55.85. It is an essential constituent of HEMOGLOBINS; CYTOCHROMES; and IRON-BINDING PROTEINS. It plays a role in cellular redox reactions and in the transport of OXYGEN.'),
  'D007502': NtO(MH='Iron Chelating Agents', MS='Organic chemicals that form two or more coordination links with an iron ion. Once coordination has occurred, the complex formed is called a chelate. The iron-binding porphyrin group of hemoglobin is an example of a metal chelate found in biological systems.'),
  'D007503': NtO(MH='Iron Isotopes', MS='Stable iron atoms that have the same atomic number as the element iron, but differ in atomic weight. Fe-54, 57, and 58 are stable iron isotopes.'),
  'D007504': NtO(MH='Iron Radioisotopes', MS='Unstable isotopes of iron that decay or disintegrate emitting radiation. Fe atoms with atomic weights 52, 53, 55, and 59-61 are radioactive iron isotopes.'),
  'D007505': NtO(MH='Iron-Dextran Complex', MS="A complex of ferric oxyhydroxide with dextrans of 5000 to 7000 daltons in a viscous solution containing 50 mg/ml of iron. It is supplied as a parenteral preparation and is used as a hematinic. (Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed, p1292)"),
  'D007506': NtO(MH='Iron-Sulfur Proteins', MS='A group of proteins possessing only the iron-sulfur complex as the prosthetic group. These proteins participate in all major pathways of electron transport: photosynthesis, respiration, hydroxylation and bacterial hydrogen and nitrogen fixation.'),
  'D007507': NtO(MH='Therapeutic Irrigation', MS='The washing of a body cavity or surface by flowing water or solution for therapy or diagnosis.'),
  'D007508': NtO(MH='Irritable Mood', MS='Abnormal or excessive excitability with easily triggered anger, annoyance, or impatience.'),
  'D007509': NtO(MH='Irritants', MS='Drugs that act locally on cutaneous or mucosal surfaces to produce inflammation; those that cause redness due to hyperemia are rubefacients; those that raise blisters are vesicants and those that penetrate sebaceous glands and cause abscesses are pustulants; tear gases and mustard gases are also irritants.'),
  'D007510': NtO(MH='Isatin', MS='An indole-dione that is obtained by oxidation of indigo blue. It is a MONOAMINE OXIDASE INHIBITOR and high levels have been found in urine of PARKINSONISM patients.'),
  'D007511': NtO(MH='Ischemia', MS='A hypoperfusion of the BLOOD through an organ or tissue caused by a PATHOLOGIC CONSTRICTION or obstruction of its BLOOD VESSELS, or an absence of BLOOD CIRCULATION.'),
  'D007512': NtO(MH='Ischium', MS='One of three bones that make up the coxal bone of the pelvic girdle. In tetrapods, it is the part of the pelvis that projects backward on the ventral side, and in primates, it bears the weight of the sitting animal.'),
  'D007513': NtO(MH='Isethionic Acid', MS='A colorless, syrupy, strongly acidic liquid that can form detergents with oleic acid.'),
  'D007514': NtO(MH='Islam', MS='A monotheistic religion promulgated by the Prophet Mohammed with Allah as the deity.'),
  'D007515': NtO(MH='Islets of Langerhans', MS='Irregular microscopic structures consisting of cords of endocrine cells that are scattered throughout the PANCREAS among the exocrine acini. Each islet is surrounded by connective tissue fibers and penetrated by a network of capillaries. There are four major cell types. The most abundant beta cells (50-80%) secrete INSULIN. Alpha cells (5-20%) secrete GLUCAGON. PP cells (10-35%) secrete PANCREATIC POLYPEPTIDE. Delta cells (~5%) secrete SOMATOSTATIN.'),
  'D007516': NtO(MH='Adenoma, Islet Cell', MS='A benign tumor of the pancreatic ISLET CELLS. Usually it involves the INSULIN-producing PANCREATIC BETA CELLS, as in INSULINOMA, resulting in HYPERINSULINISM.'),
  'D007517': NtO(MH='Isoamylase', MS='An enzyme that hydrolyzes 1,6-alpha-glucosidic branch linkages in glycogen, amylopectin, and their beta-limit dextrins. It is distinguished from pullulanase (EC 3.2.1.41) by its inability to attack pullulan and by the feeble action of alpha-limit dextrins. It is distinguished from amylopectin 6-glucanohydrolase (EC 3.2.1.69) by its action on glycogen. With EC 3.2.1.69, it produces the activity called "debranching enzyme". EC 3.2.1.68.'),
  'D007518': NtO(MH='Isoantibodies', MS='Antibodies from an individual that react with ISOANTIGENS of another individual of the same species.'),
  'D007519': NtO(MH='Isoantigens', MS='Antigens that exist in alternative (allelic) forms in a single species. When an isoantigen is encountered by species members who lack it, an immune response is induced. Typical isoantigens are the BLOOD GROUP ANTIGENS.'),
  'D007520': NtO(MH='Isocarboxazid', MS='An MAO inhibitor that is effective in the treatment of major depression, dysthymic disorder, and atypical depression. It also is useful in the treatment of panic disorder and the phobic disorders. (From AMA, Drug Evaluations Annual, 1994, p311)'),
  'D007521': NtO(MH='Isocitrate Dehydrogenase', MS='An enzyme of the oxidoreductase class that catalyzes the conversion of isocitrate and NAD+ to yield 2-ketoglutarate, carbon dioxide, and NADH. It occurs in cell mitochondria. The enzyme requires Mg2+, Mn2+; it is activated by ADP, citrate, and Ca2+, and inhibited by NADH, NADPH, and ATP. The reaction is the key rate-limiting step of the citric acid (tricarboxylic) cycle. (From Dorland, 27th ed) (The NADP+ enzyme is EC 1.1.1.42.) EC 1.1.1.41.'),
  'D007522': NtO(MH='Isocitrate Lyase', MS='A key enzyme in the glyoxylate cycle. It catalyzes the conversion of isocitrate to succinate and glyoxylate. EC 4.1.3.1.'),
  'D007523': NtO(MH='Isocitrates', MS='Derivatives of isocitric acid, a structural isomer of CITRIC ACID, including its salts and esters.'),
  'D007524': NtO(MH='Isodesmosine', MS='2-(4-Amino-4-carboxybutyl)-1-(5-amino-5-carboxypentyl)-3,5-bis(3-amino-3-carboxypropyl)pyridinium. A rare amino acid found in elastin, formed by condensation of four molecules of lysine into a pyridinium ring.'),
  'D007525': NtO(MH='Isoelectric Focusing', MS='Electrophoresis in which a pH gradient is established in a gel medium and proteins migrate until they reach the site (or focus) at which the pH is equal to their isoelectric point.'),
  'D007526': NtO(MH='Isoelectric Point', MS='The pH in solutions of proteins and related compounds at which the dipolar ions are at a maximum.'),
  'D007527': NtO(MH='Isoenzymes', MS='Structurally related forms of an enzyme. Each isoenzyme has the same mechanism and classification, but differs in its chemical, physical, or immunological characteristics.'),
  'D007528': NtO(MH='Isoetharine', MS='Adrenergic beta-2 agonist used as bronchodilator for emphysema, bronchitis and asthma.'),
  'D007529': NtO(MH='Isoflavones', MS='3-Phenylchromones. Isomeric form of FLAVONOIDS in which the benzene group is attached to the 3 position of the benzopyran ring instead of the 2 position.'),
  'D007530': NtO(MH='Isoflurane', MS='A stable, non-explosive inhalation anesthetic, relatively free from significant side effects.'),
  'D007531': NtO(MH='Isoflurophate', MS='A di-isopropyl-fluorophosphate which is an irreversible cholinesterase inhibitor used to investigate the NERVOUS SYSTEM.'),
  'D007532': NtO(MH='Isoleucine', MS='An essential branched-chain aliphatic amino acid found in many proteins. It is an isomer of LEUCINE. It is important in hemoglobin synthesis and regulation of blood sugar and energy levels.'),
  'D007533': NtO(MH='Isoleucine-tRNA Ligase', MS='An enzyme that activates isoleucine with its specific transfer RNA. EC 6.1.1.5.'),
  'D007534': NtO(MH='Isomaltose', MS='A disaccharide consisting of two glucose units in an alpha (1-6) glycosidic linkage.'),
  'D007535': NtO(MH='Isomerases', MS='A class of enzymes that catalyze geometric or structural changes within a molecule to form a single product. The reactions do not involve a net change in the concentrations of compounds other than the substrate and the product.(from Dorland, 28th ed) EC 5.'),
  'D007536': NtO(MH='Isomerism', MS='The phenomenon whereby certain chemical compounds have structures that are different although the compounds possess the same elemental composition. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D007537': NtO(MH='Isometric Contraction', MS='Muscular contractions characterized by increase in tension without change in length.'),
  'D007538': NtO(MH='Isoniazid', MS='Antibacterial agent used primarily as a tuberculostatic. It remains the treatment of choice for tuberculosis.'),
  'D007539': NtO(MH='Isonicotinic Acids', MS='Heterocyclic acids that are derivatives of 4-pyridinecarboxylic acid (isonicotinic acid).'),
  'D007540': NtO(MH='Isonipecotic Acids', MS=None),
  'D007541': NtO(MH='Isopentenyladenosine', MS='N(6)-[delta(3)-isopentenyl]adenosine. Isopentenyl derivative of adenosine which is a member of the cytokinin family of plant growth regulators.'),
  'D007542': NtO(MH='Inosine Pranobex', MS='An alkylamino-alcohol complex of inosine used in the treatment of a variety of viral infections. Unlike other antiviral agents, it acts by modifying or stimulating cell-mediated immune processes rather than acting on the virus directly.'),
  'D007544': NtO(MH='Isopropyl Thiogalactoside', MS='A non-metabolizable galactose analog that induces expression of the LAC OPERON.'),
  'D007545': NtO(MH='Isoproterenol', MS='Isopropyl analog of EPINEPHRINE; beta-sympathomimetic that acts on the heart, bronchi, skeletal muscle, alimentary tract, etc. It is used mainly as bronchodilator and heart stimulant.'),
  'D007546': NtO(MH='Isoquinolines', MS='A group of compounds with the heterocyclic ring structure of benzo(c)pyridine. The ring structure is characteristic of the group of opium alkaloids such as papaverine. (From Stedman, 25th ed)'),
  'D007547': NtO(MH='Isosorbide', MS='1,4:3,6-Dianhydro D-glucitol. Chemically inert osmotic diuretic used mainly to treat hydrocephalus; also used in glaucoma.'),
  'D007548': NtO(MH='Isosorbide Dinitrate', MS='A vasodilator used in the treatment of ANGINA PECTORIS. Its actions are similar to NITROGLYCERIN but with a slower onset of action.'),
  'D007549': NtO(MH='Isospora', MS='A genus of protozoan parasites found in the intestines of birds, amphibians, reptiles, and mammals, including man. The oocysts produce two sporocysts, each with four sporozoites. Many species are parasitic in wild and domestic animals.'),
  'D007550': NtO(MH='Isothiuronium', MS='An undecenyl THIOUREA which may have topical anti-inflammatory activity.'),
  'D007551': NtO(MH='Isotonic Contraction', MS='Muscle contraction with negligible change in the force of contraction but shortening of the distance between the origin and insertion.'),
  'D007552': NtO(MH='Isotonic Solutions', MS="Solutions having the same osmotic pressure as blood serum, or another solution with which they are compared. (From Grant & Hackh's Chemical Dictionary, 5th ed & Dorland, 28th ed)"),
  'D007553': NtO(MH='Isotope Labeling', MS='Techniques for labeling a substance with a stable or radioactive isotope. It is not used for articles involving labeled substances unless the methods of labeling are substantively discussed. Tracers that may be labeled include chemical substances, cells, or microorganisms.'),
  'D007554': NtO(MH='Isotopes', MS="Atomic species differing in mass number but having the same atomic number. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D007555': NtO(MH='Isoxazoles', MS='Azoles with an OXYGEN and a NITROGEN next to each other at the 1,2 positions, in contrast to OXAZOLES that have nitrogens at the 1,3 positions.'),
  'D007556': NtO(MH='Isoxsuprine', MS='A beta-adrenergic agonist that causes direct relaxation of uterine and vascular smooth muscle. Its vasodilating actions are greater on the arteries supplying skeletal muscle than on those supplying skin. It is used in the treatment of peripheral vascular disease and in premature labor.'),
  'D007557': NtO(MH='Israel', MS='A country in the Middle East, bordering the Mediterranean Sea, between Egypt and Lebanon. The capital is Jerusalem.'),
  'D007558': NtO(MH='Italy', MS='A country in southern Europe, a peninsula extending into the central Mediterranean Sea, northeast of Tunisia. The capital is Rome.'),
  'D007559': NtO(MH='Ivermectin', MS='A mixture of mostly avermectin H2B1a (RN 71827-03-7) with some avermectin H2B1b (RN 70209-81-3), which are macrolides from STREPTOMYCES avermitilis. It binds glutamate-gated chloride channel to cause increased permeability and hyperpolarization of nerve and muscle cells. It also interacts with other CHLORIDE CHANNELS. It is a broad spectrum antiparasitic that is active against microfilariae of ONCHOCERCA VOLVULUS but not the adult form.'),
  'D007560': NtO(MH="Cote d'Ivoire", MS='A republic in western Africa, south of MALI and BURKINA FASO, bordered by GHANA on the east. Its administrative capital is Abidjan and Yamoussoukro has been the official capital since 1983. The country was formerly called Ivory Coast.'),
  'D007562': NtO(MH='Creutzfeldt-Jakob Syndrome', MS='A rare transmissible encephalopathy most prevalent between the ages of 50 and 70 years. Affected individuals may present with sleep disturbances, personality changes, ATAXIA; APHASIA, visual loss, weakness, muscle atrophy, MYOCLONUS, progressive dementia, and death within one year of disease onset. A familial form exhibiting autosomal dominant inheritance and a new variant CJD (potentially associated with ENCEPHALOPATHY, BOVINE SPONGIFORM) have been described. Pathological features include prominent cerebellar and cerebral cortical spongiform degeneration and the presence of PRIONS. (From N Engl J Med, 1998 Dec 31;339(27))'),
  'D007563': NtO(MH='Jamaica', MS="An island in the Greater Antilles in the West Indies. Its capital is Kingston. It was discovered in 1494 by Columbus and was a Spanish colony 1509-1655 until captured by the English. Its flourishing slave trade was abolished in the 19th century. It was a British colony 1655-1958 and a territory of the West Indies Federation 1958-62. It achieved full independence in 1962. The name is from the Arawak Xaymaca, rich in springs or land of springs. (From Webster's New Geographical Dictionary, 1988, p564 & Room, Brewer's Dictionary of Names, 1992, p267)"),
  'D007564': NtO(MH='Japan', MS='A country in eastern Asia, island chain between the North Pacific Ocean and the Sea of Japan, east of the Korean Peninsula. The capital is Tokyo.'),
  'D007565': NtO(MH='Jaundice', MS='A clinical manifestation of HYPERBILIRUBINEMIA, characterized by the yellowish staining of the SKIN; MUCOUS MEMBRANE; and SCLERA. Clinical jaundice usually is a sign of LIVER dysfunction.'),
  'D007566': NtO(MH='Jaundice, Chronic Idiopathic', MS='A benign, autosomally recessive inherited hyperbilirubinemia characterized by the presence of a dark pigment in the centrilobular region of the liver cells. There is a functional defect in biliary excretion of bilirubin, cholephilic dyes, and porphyrins. Affected persons may be asymptomatic or have vague constitutional or gastrointestinal symptoms. The liver may be slightly enlarged, and oral and intravenous cholangiography fails to visualize the biliary tract.'),
  'D007567': NtO(MH='Jaundice, Neonatal', MS='Yellow discoloration of the SKIN; MUCOUS MEMBRANE; and SCLERA in the NEWBORN. It is a sign of NEONATAL HYPERBILIRUBINEMIA. Most cases are transient self-limiting (PHYSIOLOGICAL NEONATAL JAUNDICE) occurring in the first week of life, but some can be a sign of pathological disorders, particularly LIVER DISEASES.'),
  'D007568': NtO(MH='Jaw', MS='Bony structure of the mouth that holds the teeth. It consists of the MANDIBLE and the MAXILLA.'),
  'D007569': NtO(MH='Jaw Abnormalities', MS='Congenital absence of or defects in structures of the jaw.'),
  'D007570': NtO(MH='Jaw Cysts', MS='Saccular lesions lined with epithelium and contained within pathologically formed cavities in the jaw; also nonepithelial cysts (pseudocysts) as they apply to the jaw, e.g., traumatic or solitary cyst, static bone cavity, and aneurysmal bone cyst. True jaw cysts are classified as odontogenic or nonodontogenic.'),
  'D007571': NtO(MH='Jaw Diseases', MS='Diseases involving the JAW.'),
  'D007572': NtO(MH='Jaw Fractures', MS='Fractures of the upper or lower jaw.'),
  'D007573': NtO(MH='Jaw Neoplasms', MS='Cancers or tumors of the MAXILLA or MANDIBLE unspecified. For neoplasms of the maxilla, MAXILLARY NEOPLASMS is available and of the mandible, MANDIBULAR NEOPLASMS is available.'),
  'D007574': NtO(MH='Jaw Relation Record', MS='A registration of any positional relationship of the mandible in reference to the maxillae. These records may be any of the many vertical, horizontal, or orientation relations. (Jablonski, Illustrated Dictionary of Dentistry)'),
  'D007575': NtO(MH='Jaw, Edentulous', MS='The total absence of teeth from either the mandible or the maxilla, but not both. Total absence of teeth from both is MOUTH, EDENTULOUS. Partial absence of teeth in either is JAW, EDENTULOUS, PARTIALLY.'),
  'D007576': NtO(MH='Jaw, Edentulous, Partially', MS='Absence of teeth from a portion of the mandible and/or maxilla.'),
  'D007577': NtO(MH='JC Virus', MS="A species of POLYOMAVIRUS, originally isolated from the brain of a patient with progressive multifocal leukoencephalopathy. The patient's initials J.C. gave the virus its name. Infection is not accompanied by any apparent illness but serious demyelinating disease can appear later, probably following reactivation of latent virus."),
  'D007578': NtO(MH='Jealousy', MS='An irrational reaction compounded of grief, loss of self-esteem, enmity against the rival and self criticism.'),
  'D007579': NtO(MH='Jejunal Diseases', MS='Pathological development in the JEJUNUM region of the SMALL INTESTINE.'),
  'D007580': NtO(MH='Jejunal Neoplasms', MS='Tumors or cancer in the JEJUNUM region of the small intestine (INTESTINE, SMALL).'),
  'D007581': NtO(MH='Jejunoileal Bypass', MS='A procedure consisting of the SURGICAL ANASTOMOSIS of the proximal part of the JEJUNUM to the distal portion of the ILEUM, so as to bypass the nutrient-absorptive segment of the SMALL INTESTINE. Due to the severe malnutrition and life-threatening metabolic complications, this method is no longer used to treat MORBID OBESITY.'),
  'D007582': NtO(MH='Jejunostomy', MS='Surgical formation of an opening through the ABDOMINAL WALL into the JEJUNUM, usually for enteral hyperalimentation.'),
  'D007583': NtO(MH='Jejunum', MS='The middle portion of the SMALL INTESTINE, between DUODENUM and ILEUM. It represents about 2/5 of the remaining portion of the small intestine below duodenum.'),
  'D007585': NtO(MH='Jews', MS='An ethnic group with historical ties to the land of ISRAEL and the religion of JUDAISM.'),
  'D007586': NtO(MH='Job Application', MS='Process of applying for employment. It includes written application for employment or personal appearance.'),
  'D007587': NtO(MH='Job Description', MS='Statement of the position requirements, qualifications for the position, wage range, and any special conditions expected of the employee.'),
  'D007588': NtO(MH='Job Satisfaction', MS='Personal satisfaction relative to the work situation.'),
  'D007589': NtO(MH='Job Syndrome', MS='Primary immunodeficiency syndrome characterized by recurrent infections and hyperimmunoglobulinemia E. Most cases are sporadic. Of the rare familial forms, the dominantly inherited subtype has additional connective tissue, dental and skeletal involvement that the recessive type does not share.'),
  'D007590': NtO(MH='Jogging', MS='Running at a low rate of speed. It can be done as a means of conditioning or for general health and well being.'),
  'D007591': NtO(MH='Joint Commission on Accreditation of Healthcare Organizations', MS='A private, voluntary, not-for-profit organization which establishes standards for the operation of health facilities and services, conducts surveys, and awards accreditation.'),
  'D007592': NtO(MH='Joint Diseases', MS='Diseases involving the JOINTS.'),
  'D007593': NtO(MH='Joint Instability', MS='Lack of stability of a joint or joint prosthesis.'),
  'D007594': NtO(MH='Joint Loose Bodies', MS='Fibrous, bony, cartilaginous and osteocartilaginous fragments in a synovial joint. Major causes are osteochondritis dissecans, synovial chondromatosis, osteophytes, fractured articular surfaces and damaged menisci.'),
  'D007595': NtO(MH='Joint Prosthesis', MS='Prostheses used to partially or totally replace a human or animal joint. (from UMDNS, 1999)'),
  'D007596': NtO(MH='Joints', MS='Also known as articulations, these are points of connection between the ends of certain separate bones, or where the borders of other bones are juxtaposed.'),
  'D007597': NtO(MH='Jordan', MS='A country in the MIDDLE EAST, northwest of Saudi Arabia, between Israel and Iraq. The capital is Amman.'),
  'D007598': NtO(MH='Journalism, Dental', MS='Content, management, editing, policies, and printing of dental periodicals such as journals, newsletters, tabloids, and bulletins.'),
  'D007599': NtO(MH='Judaism', MS='The monotheistic religion of the Jews based on the teachings of the Hebrew Scriptures (Torah).'),
  'D007600': NtO(MH='Judgment', MS='The process of discovering or asserting an objective or intrinsic relation between two objects or concepts; a faculty or power that enables a person to make judgments; the process of bringing to light and asserting the implicit meaning of a concept; a critical evaluation of a person or situation.'),
  'D007601': NtO(MH='Jugular Veins', MS='Veins in the neck which drain the brain, face, and neck into the brachiocephalic or subclavian veins.'),
  'D007602': NtO(MH='Jungian Theory', MS='A theoretical psychoanalytical system centered around symbols of the unconscious with the unconscious material derived from two sources - the personal unconscious (repressed or forgotten experiences, thoughts and feelings) and the collective or objective unconscious (the universal inherited qualities which dispose individuals to behave in ways similar to their ancestors).'),
  'D007603': NtO(MH='Jurisprudence', MS='The science or philosophy of law. Also, the application of the principles of law and justice to health and medicine.'),
  'D007604': NtO(MH='Juvenile Delinquency', MS='The antisocial acts of children or persons under age which are illegal or lawfully interpreted as constituting delinquency.'),
  'D007605': NtO(MH='Juvenile Hormones', MS='Compounds, either natural or synthetic, which block development of the growing insect.'),
  'D007606': NtO(MH='Juxtaglomerular Apparatus', MS='A complex of cells consisting of juxtaglomerular cells, extraglomerular mesangium lacis cells, the macula densa of the distal convoluted tubule, and granular epithelial peripolar cells. Juxtaglomerular cells are modified SMOOTH MUSCLE CELLS found in the walls of afferent glomerular arterioles and sometimes the efferent arterioles. Extraglomerular mesangium lacis cells are located in the angle between the afferent and efferent glomerular arterioles. Granular epithelial peripolar cells are located at the angle of reflection of the parietal to visceral angle of the renal corpuscle.'),
  'D007608': NtO(MH='Kainic Acid', MS='(2S-(2 alpha,3 beta,4 beta))-2-Carboxy-4-(1-methylethenyl)-3-pyrrolidineacetic acid. Ascaricide obtained from the red alga Digenea simplex. It is a potent excitatory amino acid agonist at some types of excitatory amino acid receptors and has been used to discriminate among receptor types. Like many excitatory amino acid agonists it can cause neurotoxicity and has been used experimentally for that purpose.'),
  'D007609': NtO(MH='Kallidin', MS='A decapeptide bradykinin homolog cleaved from kininogen by kallikreins. It is a smooth-muscle stimulant and hypotensive agent that acts by vasodilatation.'),
  'D007610': NtO(MH='Kallikreins', MS='Proteolytic enzymes from the serine endopeptidase family found in normal blood and urine. Specifically, Kallikreins are potent vasodilators and hypotensives and increase vascular permeability and affect smooth muscle. They act as infertility agents in men. Three forms are recognized, PLASMA KALLIKREIN (EC 3.4.21.34), TISSUE KALLIKREIN (EC 3.4.21.35), and PROSTATE-SPECIFIC ANTIGEN (EC 3.4.21.77).'),
  'D007611': NtO(MH='Aprotinin', MS='A single-chain polypeptide derived from bovine tissues consisting of 58 amino-acid residues. It is an inhibitor of proteolytic enzymes including CHYMOTRYPSIN; KALLIKREIN; PLASMIN; and TRYPSIN. It is used in the treatment of HEMORRHAGE associated with raised plasma concentrations of plasmin. It is also used to reduce blood loss and transfusion requirements in patients at high risk of major blood loss during and following open heart surgery with EXTRACORPOREAL CIRCULATION. (Reynolds JEF(Ed): Martindale: The Extra Pharmacopoeia (electronic version). Micromedex, Inc, Englewood, CO, 1995)'),
  'D007612': NtO(MH='Kanamycin', MS='Antibiotic complex produced by Streptomyces kanamyceticus from Japanese soil. Comprises 3 components: kanamycin A, the major component, and kanamycins B and C, the minor components.'),
  'D007613': NtO(MH='Kanamycin Resistance', MS='Nonsusceptibility of bacteria to the antibiotic KANAMYCIN, which can bind to their 70S ribosomes and cause misreading of messenger RNA.'),
  'D007614': NtO(MH='Macropodidae', MS='A family of herbivorous leaping MAMMALS of Australia, New Guinea, and adjacent islands. Members include kangaroos, wallabies, quokkas, and wallaroos.'),
  'D007615': NtO(MH='Kansas', MS='State bounded on the north by Nebraska on the east by Missouri, on the south by Oklahoma, and on the west by Colorado.'),
  'D007616': NtO(MH='Kaolin', MS='The most common mineral of a group of hydrated aluminum silicates, approximately H2Al2Si2O8-H2O. It is prepared for pharmaceutical and medicinal purposes by levigating with water to remove sand, etc. (From Merck Index, 11th ed) The name is derived from Kao-ling (Chinese: "high ridge"), the original site. (From Grant & Hackh\'s Chemical Dictionary, 5th ed)'),
  'D007617': NtO(MH='Kaposi Varicelliform Eruption', MS='A disseminated vesicular-pustular eruption caused by the herpes simplex virus (HERPESVIRUS HOMINIS), the VACCINIA VIRUS, or Varicella zoster (HERPESVIRUS 3, HUMAN). It is usually superimposed on a preexisting, inactive or active, atopic dermatitis (DERMATITIS, ATOPIC).'),
  'D007618': NtO(MH='Karaya Gum', MS='Polysaccharide gum from Sterculia urens (STERCULIA). It is used as a suspending or stabilizing agent in foods, cosmetics and pharmaceuticals; a bulk-forming laxative; a surgical lubricant and adhesive; and in the treatment of skin ulcers.'),
  'D007619': NtO(MH='Kartagener Syndrome', MS='An autosomal recessive disorder characterized by a triad of DEXTROCARDIA; INFERTILITY; and SINUSITIS. The syndrome is caused by mutations of DYNEIN genes encoding motility proteins which are components of sperm tails, and CILIA in the respiratory and the reproductive tracts.'),
  'D007620': NtO(MH='Karyometry', MS='Microscopic study of the spacial distribution pattern of CHROMATIN in CELL NUCLEI and CELL NUCLEOLI.'),
  'D007621': NtO(MH='Karyotyping', MS='Mapping of the KARYOTYPE of a cell.'),
  'D007623': NtO(MH='Kazakhstan', MS='A country in central Asia, northwest of China. The capital is Astana.'),
  'D007624': NtO(MH='KB Cells', MS='This line KB is now known to be a subline of the ubiquitous KERATIN-forming tumor cell line HeLa. It was originally thought to be derived from an epidermal carcinoma of the mouth, but was subsequently found, based on isoenzyme analysis, HeLa marker chromosomes, and DNA fingerprinting, to have been established via contamination by HELA CELLS. The cells are positive for keratin by immunoperoxidase staining. KB cells have been reported to contain human papillomavirus18 (HPV-18) sequences.'),
  'D007625': NtO(MH='Kearns-Sayre Syndrome', MS='A mitochondrial disorder featuring the triad of chronic progressive EXTERNAL OPHTHALMOPLEGIA, cardiomyopathy (CARDIOMYOPATHIES) with conduction block (HEART BLOCK), and RETINITIS PIGMENTOSA. Disease onset is in the first or second decade. Elevated CSF protein, sensorineural deafness, seizures, and pyramidal signs may also be present. Ragged-red fibers are found on muscle biopsy. (Adams et al., Principles of Neurology, 6th ed, p984)'),
  'D007626': NtO(MH='Kell Blood-Group System', MS='Multiple erythrocytic antigens that comprise at least three pairs of alternates and amorphs, determined by one complex gene or possibly several genes at closely linked loci. The system is important in transfusion reactions. Its expression involves the X-chromosome.'),
  'D007627': NtO(MH='Keloid', MS='A sharply elevated, irregularly shaped, progressively enlarging scar resulting from formation of excessive amounts of collagen in the dermis during connective tissue repair. It is differentiated from a hypertrophic scar (CICATRIX, HYPERTROPHIC) in that the former does not spread to surrounding tissues.'),
  'D007629': NtO(MH='Kentucky', MS='A state bounded on the north by Illinois, Indiana, and Ohio; on the east by Virginia and west Virginia; on the south by Tennessee, and on the west by Missouri.'),
  'D007630': NtO(MH='Kenya', MS='A republic in eastern Africa, south of ETHIOPIA, west of SOMALIA with TANZANIA to its south, and coastline on the Indian Ocean. Its capital is Nairobi.'),
  'D007631': NtO(MH='Chlordecone', MS='A highly chlorinated polycyclic hydrocarbon insecticide whose large number of chlorine atoms makes it resistant to degradation. It has been shown to be toxic to mammals and causes abnormal cellular changes in laboratory animals.'),
  'D007632': NtO(MH='Keratan Sulfate', MS='A sulfated mucopolysaccharide initially isolated from bovine cornea. At least two types are known. Type I, found mostly in the cornea, contains D-galactose and D-glucosamine-6-O-sulfate as the repeating unit; type II, found in skeletal tissues, contains D-galactose and D-galactosamine-6-O-sulfate as the repeating unit.'),
  'D007633': NtO(MH='Keratins', MS='A class of fibrous proteins or scleroproteins that represents the principal constituent of EPIDERMIS; HAIR; NAILS; horny tissues, and the organic matrix of tooth ENAMEL. Two major conformational groups have been characterized, alpha-keratin, whose peptide backbone forms a coiled-coil alpha helical structure consisting of TYPE I KERATIN and a TYPE II KERATIN, and beta-keratin, whose backbone forms a zigzag or pleated sheet structure. alpha-Keratins have been classified into at least 20 subtypes. In addition multiple isoforms of subtypes have been found which may be due to GENE DUPLICATION.'),
  'D007634': NtO(MH='Keratitis', MS='Inflammation of the cornea.'),
  'D007635': NtO(MH='Keratitis, Dendritic', MS='A form of herpetic keratitis characterized by the formation of small vesicles which break down and coalesce to form recurring dendritic ulcers, characteristically irregular, linear, branching, and ending in knoblike extremities. (Dictionary of Visual Science, 3d ed)'),
  'D007636': NtO(MH='Keratoacanthoma', MS='A benign, non-neoplastic, usually self-limiting epithelial lesion closely resembling squamous cell carcinoma clinically and histopathologically. It occurs in solitary, multiple, and eruptive forms. The solitary and multiple forms occur on sunlight exposed areas and are identical histologically; they affect primarily white males. The eruptive form usually involves both sexes and appears as a generalized papular eruption.'),
  'D007637': NtO(MH='Keratoconjunctivitis', MS='Simultaneous inflammation of the cornea and conjunctiva.'),
  'D007638': NtO(MH='Keratoconjunctivitis Sicca', MS="Drying and inflammation of the conjunctiva as a result of insufficient lacrimal secretion. When found in association with XEROSTOMIA and polyarthritis, it is called SJOGREN'S SYNDROME."),
  'D007639': NtO(MH='Keratoconjunctivitis, Infectious', MS='Infectious diseases of cattle, sheep, and goats, characterized by blepharospasm, lacrimation, conjunctivitis, and varying degrees of corneal opacity and ulceration. In cattle the causative agent is MORAXELLA (MORAXELLA) BOVIS; in sheep, MYCOPLASMA; RICKETTSIA; CHLAMYDIA; or ACHOLEPLASMA; in goats, RICKETTSIA.'),
  'D007640': NtO(MH='Keratoconus', MS='A noninflammatory, usually bilateral protrusion of the cornea, the apex being displaced downward and nasally. It occurs most commonly in females at about puberty. The cause is unknown but hereditary factors may play a role. The -conus refers to the cone shape of the corneal protrusion. (From Dorland, 27th ed)'),
  'D007641': NtO(MH='Keratolytic Agents', MS='Agents that soften, separate, and cause desquamation of the cornified epithelium or horny layer of skin. They are used to expose mycelia of infecting fungi or to treat corns, warts, and certain other skin diseases.'),
  'D007642': NtO(MH='Keratosis', MS='Any horny growth such as a wart or callus.'),
  'D007644': NtO(MH='Darier Disease', MS='An autosomal dominantly inherited skin disorder characterized by warty malodorous papules that coalesce into plaques. It is caused by mutations in the ATP2A2 gene encoding SERCA2 protein, one of the SARCOPLASMIC RETICULUM CALCIUM-TRANSPORTING ATPASES. The condition is similar, clinically and histologically, to BENIGN FAMILIAL PEMPHIGUS, another autosomal dominant skin disorder. Both diseases have defective calcium pumps (CALCIUM-TRANSPORTING ATPASES) and unstable desmosomal adhesion junctions (DESMOSOMES) between KERATINOCYTES.'),
  'D007645': NtO(MH='Keratoderma, Palmoplantar', MS='Group of mostly hereditary disorders characterized by thickening of the palms and soles as a result of excessive keratin formation leading to hypertrophy of the stratum corneum (hyperkeratosis).'),
  'D007646': NtO(MH='Keratotomy, Radial', MS='A procedure to surgically correct REFRACTIVE ERRORS by cutting radial slits into the CORNEA to change its refractive properties.'),
  'D007647': NtO(MH='Kernicterus', MS='A term used pathologically to describe BILIRUBIN staining of the BASAL GANGLIA; BRAIN STEM; and CEREBELLUM and clinically to describe a syndrome associated with HYPERBILIRUBINEMIA. Clinical features include athetosis, MUSCLE SPASTICITY or hypotonia, impaired vertical gaze, and DEAFNESS. Nonconjugated bilirubin enters the brain and acts as a neurotoxin, often in association with conditions that impair the BLOOD-BRAIN BARRIER (e.g., SEPSIS). This condition occurs primarily in neonates (INFANT, NEWBORN), but may rarely occur in adults. (Menkes, Textbook of Child Neurology, 5th ed, p613)'),
  'D007648': NtO(MH='Kerosene', MS='A refined petroleum fraction used as a fuel as well as a solvent.'),
  'D007649': NtO(MH='Ketamine', MS='A cyclohexanone derivative used for induction of anesthesia. Its mechanism of action is not well understood, but ketamine can block NMDA receptors (RECEPTORS, N-METHYL-D-ASPARTATE) and may interact with sigma receptors.'),
  'D007650': NtO(MH='Ketanserin', MS='A selective serotonin receptor antagonist with weak adrenergic receptor blocking properties. The drug is effective in lowering blood pressure in essential hypertension. It also inhibits platelet aggregation. It is well tolerated and is particularly effective in older patients.'),
  'D007651': NtO(MH='Keto Acids', MS='Carboxylic acids that contain a KETONE group.'),
  'D007652': NtO(MH='Oxo-Acid-Lyases', MS='Enzymes that catalyze the cleavage of a carbon-carbon bond of a 3-hydroxy acid. (Dorland, 28th ed) EC 4.1.3.'),
  'D007653': NtO(MH='Ketocholesterols', MS='Cholesterol substituted in any position by a keto moiety. The 7-keto isomer inhibits cholesterol uptake in the coronary arteries and aorta by blocking 3-hydroxy-3-methylglutaryl-CoA reductase activity.'),
  'D007654': NtO(MH='Ketoconazole', MS='Broad spectrum antifungal agent used for long periods at high doses, especially in immunosuppressed patients.'),
  'D007655': NtO(MH='Ketoglutarate Dehydrogenase Complex', MS=None),
  'D007656': NtO(MH='Ketoglutaric Acids', MS='A family of compounds containing an oxo group with the general structure of 1,5-pentanedioic acid. (From Lehninger, Principles of Biochemistry, 1982, p442)'),
  'D007657': NtO(MH='Ketone Bodies', MS='The metabolic substances ACETONE; 3-HYDROXYBUTYRIC ACID; and acetoacetic acid (ACETOACETATES). They are produced in the liver and kidney during FATTY ACIDS oxidation and used as a source of energy by the heart, muscle and brain.'),
  'D007658': NtO(MH='Ketone Oxidoreductases', MS='Oxidoreductases that are specific for KETONES.'),
  'D007659': NtO(MH='Ketones', MS='Organic compounds containing a carbonyl group =C=O bonded to two hydrocarbon groups.'),
  'D007660': NtO(MH='Ketoprofen', MS='An IBUPROFEN-type anti-inflammatory analgesic and antipyretic. It is used in the treatment of rheumatoid arthritis and osteoarthritis.'),
  'D007661': NtO(MH='Ketoses', MS=None),
  'D007662': NtO(MH='Ketosis', MS='A condition characterized by an abnormally elevated concentration of KETONE BODIES in the blood (acetonemia) or urine (acetonuria). It is a sign of DIABETES COMPLICATION, starvation, alcoholism or a mitochondrial metabolic disturbance (e.g., MAPLE SYRUP URINE DISEASE).'),
  'D007664': NtO(MH='Ketosteroids', MS='Steroid derivatives formed by oxidation of a methyl group on the side chain or a methylene group in the ring skeleton to form a ketone.'),
  'D007665': NtO(MH='Ketotifen', MS='A cycloheptathiophene blocker of histamine H1 receptors and release of inflammatory mediators. It has been proposed for the treatment of asthma, rhinitis, skin allergies, and anaphylaxis.'),
  'D007666': NtO(MH='Khellin', MS='A vasodilator that also has bronchodilatory action. It has been employed in the treatment of angina pectoris, in the treatment of asthma, and in conjunction with ultraviolet light A, has been tried in the treatment of vitiligo. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1024)'),
  'D007667': NtO(MH='Kidd Blood-Group System', MS='A group of antigens consisting principally of Jk(a) and Jk(b), determined by allelic genes. Amorphs are encountered. Antibodies of these substances are usually weak and quite labile, stimulated by erythrocytes.'),
  'D007668': NtO(MH='Kidney', MS='Body organ that filters blood for the secretion of URINE and that regulates ion concentrations.'),
  'D007669': NtO(MH='Kidney Calculi', MS='Stones in the KIDNEY, usually formed in the urine-collecting area of the kidney (KIDNEY PELVIS). Their sizes vary and most contains CALCIUM OXALATE.'),
  'D007670': NtO(MH='Kidney Calices', MS='Recesses of the kidney pelvis which divides into two wide, cup-shaped major renal calices, with each major calix subdivided into 7 to 14 minor calices. Urine empties into a minor calix from collecting tubules, then passes through the major calix, renal pelvis, and ureter to enter the urinary bladder. (From Moore, Clinically Oriented Anatomy, 3d ed, p211)'),
  'D007671': NtO(MH='Kidney Concentrating Ability', MS='The ability of the kidney to excrete in the urine high concentrations of solutes from the blood plasma.'),
  'D007672': NtO(MH='Kidney Cortex', MS='The outer zone of the KIDNEY, beneath the capsule, consisting of KIDNEY GLOMERULUS; KIDNEY TUBULES, DISTAL; and KIDNEY TUBULES, PROXIMAL.'),
  'D007673': NtO(MH='Kidney Cortex Necrosis', MS='Death of cells in the KIDNEY CORTEX, a common final result of various renal injuries including HYPOXIA; ISCHEMIA; and drug toxicity.'),
  'D007674': NtO(MH='Kidney Diseases', MS='Pathological processes of the KIDNEY or its component tissues.'),
  'D007676': NtO(MH='Kidney Failure, Chronic', MS='The end-stage of CHRONIC RENAL INSUFFICIENCY. It is characterized by the severe irreversible kidney damage (as measured by the level of PROTEINURIA) and the reduction in GLOMERULAR FILTRATION RATE to less than 15 ml per min (Kidney Foundation: Kidney Disease Outcome Quality Initiative, 2002). These patients generally require HEMODIALYSIS or KIDNEY TRANSPLANTATION.'),
  'D007677': NtO(MH='Kidney Function Tests', MS='Laboratory tests used to evaluate how well the kidneys are working through examination of blood and urine.'),
  'D007678': NtO(MH='Kidney Glomerulus', MS='A cluster of convoluted capillaries beginning at each nephric tubule in the kidney and held together by connective tissue.'),
  'D007679': NtO(MH='Kidney Medulla', MS='The internal portion of the kidney, consisting of striated conical masses, the renal pyramids, whose bases are adjacent to the cortex and whose apices form prominent papillae projecting into the lumen of the minor calyces.'),
  'D007680': NtO(MH='Kidney Neoplasms', MS='Tumors or cancers of the KIDNEY.'),
  'D007681': NtO(MH='Kidney Papillary Necrosis', MS='A complication of kidney diseases characterized by cell death involving KIDNEY PAPILLA in the KIDNEY MEDULLA. Damages to this area may hinder the kidney to concentrate urine resulting in POLYURIA. Sloughed off necrotic tissue may block KIDNEY PELVIS or URETER. Necrosis of multiple renal papillae can lead to KIDNEY FAILURE.'),
  'D007682': NtO(MH='Kidney Pelvis', MS='The flattened, funnel-shaped expansion connecting the URETER to the KIDNEY CALICES.'),
  'D007683': NtO(MH='Kidney Tubular Necrosis, Acute', MS='Acute kidney failure resulting from destruction of EPITHELIAL CELLS of the KIDNEY TUBULES. It is commonly attributed to exposure to toxic agents or renal ISCHEMIA following severe TRAUMA.'),
  'D007684': NtO(MH='Kidney Tubules', MS='Long convoluted tubules in the nephrons. They collect filtrate from blood passing through the KIDNEY GLOMERULUS and process this filtrate into URINE. Each renal tubule consists of a BOWMAN CAPSULE; PROXIMAL KIDNEY TUBULE; LOOP OF HENLE; DISTAL KIDNEY TUBULE; and KIDNEY COLLECTING DUCT leading to the central cavity of the kidney (KIDNEY PELVIS) that connects to the URETER.'),
  'D007685': NtO(MH='Kidney Tubules, Collecting', MS='Straight tubes commencing in the radiate part of the kidney cortex where they receive the curved ends of the distal convoluted tubules. In the medulla the collecting tubules of each pyramid converge to join a central tube (duct of Bellini) which opens on the summit of the papilla.'),
  'D007686': NtO(MH='Kidney Tubules, Distal', MS='The portion of renal tubule that begins from the enlarged segment of the ascending limb of the LOOP OF HENLE. It reenters the KIDNEY CORTEX and forms the convoluted segments of the distal tubule.'),
  'D007687': NtO(MH='Kidney Tubules, Proximal', MS='The renal tubule portion that extends from the BOWMAN CAPSULE in the KIDNEY CORTEX into the KIDNEY MEDULLA. The proximal tubule consists of a convoluted proximal segment in the cortex, and a distal straight segment descending into the medulla where it forms the U-shaped LOOP OF HENLE.'),
  'D007688': NtO(MH='Kidneys, Artificial', MS='Devices which can substitute for normally functioning KIDNEYS in removing components from the blood by DIALYSIS that are normally eliminated in the URINE.'),
  'D007690': NtO(MH='Polycystic Kidney Diseases', MS='Hereditary diseases that are characterized by the progressive expansion of a large number of tightly packed CYSTS within the KIDNEYS. They include diseases with autosomal dominant and autosomal recessive inheritance.'),
  'D007691': NtO(MH='Medullary Sponge Kidney', MS='A non-hereditary KIDNEY disorder characterized by the abnormally dilated (ECTASIA) medullary and inner papillary portions of the collecting ducts. These collecting ducts usually contain CYSTS or DIVERTICULA filled with jelly-like material or small calculi (KIDNEY STONES) leading to infections or obstruction. It should be distinguished from congenital or hereditary POLYCYSTIC KIDNEY DISEASES.'),
  'D007692': NtO(MH='Diatomaceous Earth', MS='A form of SILICON DIOXIDE composed of skeletons of prehistoric aquatic plants which is used for its ABSORPTION quality, taking up 1.5-4 times its weight in water. The microscopic sharp edges are useful for insect control but can also be an inhalation hazard. It has been used in baked goods and animal feed. Kieselguhr is German for flint + earthy sediment.'),
  'D007694': NtO(MH='Killer Cells, Natural', MS='Bone marrow-derived lymphocytes that possess cytotoxic properties, classically directed against transformed and virus-infected cells. Unlike T CELLS; and B CELLS; NK CELLS are not antigen specific. The cytotoxicity of natural killer cells is determined by the collective signaling of an array of inhibitory and stimulatory CELL SURFACE RECEPTORS. A subset of T-LYMPHOCYTES referred to as NATURAL KILLER T CELLS shares some of the properties of this cell type.'),
  'D007695': NtO(MH='Killifishes', MS='Small oviparous fishes in the family Cyprinodontidae, usually striped or barred black. They are much used in mosquito control.'),
  'D007696': NtO(MH='Kindling, Neurologic', MS='The repeated weak excitation of brain structures, that progressively increases sensitivity to the same stimulation. Over time, this can lower the threshold required to trigger seizures.'),
  'D007697': NtO(MH='Kinesics', MS='Systematic study of the body and the use of its static and dynamic position as a means of communication.'),
  'D007698': NtO(MH='Kinesis', MS='Locomotor behavior not involving a steering reaction, but in which there may be a turning random in direction. It includes orthokinesis, the rate of movement and klinokinesis, the amount of turning, which are related to the intensity of stimulation.'),
  'D007699': NtO(MH='Kinesthesis', MS='Sense of movement of a part of the body, such as movement of fingers, elbows, knees, limbs, or weights.'),
  'D007700': NtO(MH='Kinetics', MS='The rate dynamics in chemical or physical systems.'),
  'D007701': NtO(MH='Kinetin', MS='A furanyl adenine found in PLANTS and FUNGI. It has plant growth regulation effects.'),
  'D007702': NtO(MH='Kinetocardiography', MS='The graphic recording of chest wall movement due to cardiac impulses.'),
  'D007703': NtO(MH='Peptidyl-Dipeptidase A', MS='A peptidyl-dipeptidase that catalyzes the release of a C-terminal dipeptide, -Xaa-*-Xbb-Xcc, when neither Xaa nor Xbb is Pro. It is a Cl(-)-dependent, zinc glycoprotein that is generally membrane-bound and active at neutral pH. It may also have endopeptidase activity on some substrates. (From Enzyme Nomenclature, 1992)'),
  'D007704': NtO(MH='Kininogens', MS='Endogenous peptides present in most body fluids. Certain enzymes convert them to active KININS which are involved in inflammation, blood clotting, complement reactions, etc. Kininogens belong to the cystatin superfamily. They are cysteine proteinase inhibitors. HIGH-MOLECULAR-WEIGHT KININOGEN; (HMWK); is split by plasma kallikrein to produce BRADYKININ. LOW-MOLECULAR-WEIGHT KININOGEN; (LMWK); is split by tissue kallikrein to produce KALLIDIN.'),
  'D007705': NtO(MH='Kinins', MS="A generic term used to describe a group of polypeptides with related chemical structures and pharmacological properties that are widely distributed in nature. These peptides are AUTACOIDS that act locally to produce pain, vasodilatation, increased vascular permeability, and the synthesis of prostaglandins. Thus, they comprise a subset of the large number of mediators that contribute to the inflammatory response. (From Goodman and Gilman's The Pharmacologic Basis of Therapeutics, 8th ed, p588)"),
  'D007706': NtO(MH='Menkes Kinky Hair Syndrome', MS='An inherited disorder of copper metabolism transmitted as an X-linked trait and characterized by the infantile onset of HYPOTHERMIA, feeding difficulties, hypotonia, SEIZURES, bony deformities, pili torti (twisted hair), and severely impaired intellectual development. Defective copper transport across plasma and endoplasmic reticulum membranes results in copper being unavailable for the synthesis of several copper containing enzymes, including PROTEIN-LYSINE 6-OXIDASE; CERULOPLASMIN; and SUPEROXIDE DISMUTASE. Pathologic changes include defects in arterial elastin, neuronal loss, and gliosis. (From Menkes, Textbook of Child Neurology, 5th ed, p125)'),
  'D007707': NtO(MH='Kyrgyzstan', MS='A country in located Central Asia, west of China, south of Kazakhstan.'),
  'D007708': NtO(MH='Kirsten murine sarcoma virus', MS='A replication-defective murine sarcoma virus (SARCOMA VIRUSES, MURINE) capable of transforming mouse lymphoid cells and producing erythroid leukemia after superinfection with murine leukemia viruses (LEUKEMIA VIRUS, MURINE). It has also been found to transform cultured human fibroblasts, rat liver epithelial cells, and rat adrenocortical cells.'),
  'D007709': NtO(MH='Klebsiella', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria whose organisms arrange singly, in pairs, or short chains. This genus is commonly found in the intestinal tract and is an opportunistic pathogen that can give rise to bacteremia, pneumonia, urinary tract and several other types of human infection.'),
  'D007710': NtO(MH='Klebsiella Infections', MS='Infections with bacteria of the genus KLEBSIELLA.'),
  'D007711': NtO(MH='Klebsiella pneumoniae', MS='Gram-negative, non-motile, capsulated, gas-producing rods found widely in nature and associated with urinary and respiratory infections in humans.'),
  'D007713': NtO(MH='Klinefelter Syndrome', MS='A form of male HYPOGONADISM, characterized by the presence of an extra X CHROMOSOME, small TESTES, seminiferous tubule dysgenesis, elevated levels of GONADOTROPINS, low serum TESTOSTERONE, underdeveloped secondary sex characteristics, and male infertility (INFERTILITY, MALE). Patients tend to have long legs and a slim, tall stature. GYNECOMASTIA is present in many of the patients. The classic form has the karyotype 47,XXY. Several karyotype variants include 48,XXYY; 48,XXXY; 49,XXXXY, and mosaic patterns ( 46,XY/47,XXY; 47,XXY/48,XXXY, etc.).'),
  'D007714': NtO(MH='Klippel-Feil Syndrome', MS='A syndrome characterised by a low hairline and a shortened neck resulting from a reduced number of vertebrae or the fusion of multiple hemivertebrae into one osseous mass.'),
  'D007715': NtO(MH='Klippel-Trenaunay-Weber Syndrome', MS='A congenital disorder that is characterized by a triad of capillary malformations (HEMANGIOMA), venous malformations (ARTERIOVENOUS FISTULA), and soft tissue or bony hypertrophy of the limb. This syndrome is caused by mutations in the VG5Q gene which encodes a strong angiogenesis stimulator.'),
  'D007716': NtO(MH='Kluyveromyces', MS='An ascomycetous yeast of the fungal family Saccharomycetaceae, order SACCHAROMYCETALES.'),
  'D007717': NtO(MH='Knee', MS='A region of the lower extremity immediately surrounding and including the KNEE JOINT.'),
  'D007718': NtO(MH='Knee Injuries', MS='Injuries to the knee or the knee joint.'),
  'D007719': NtO(MH='Knee Joint', MS='A synovial hinge connection formed between the bones of the FEMUR; TIBIA; and PATELLA.'),
  'D007720': NtO(MH='Knee Prosthesis', MS='Replacement for a knee joint.'),
  'D007721': NtO(MH='Knowledge of Results, Psychological', MS='A principle that learning is facilitated when the learner receives immediate evaluation of learning performance. The concept also hypothesizes that learning is facilitated when the learner is promptly informed whether a response is correct, and, if incorrect, of the direction of error.'),
  'D007722': NtO(MH='Health Knowledge, Attitudes, Practice', MS='Knowledge, attitudes, and associated behaviors which pertain to health-related topics such as PATHOLOGIC PROCESSES or diseases, their prevention, and treatment. This term refers to non-health workers and health workers (HEALTH PERSONNEL).'),
  'D007723': NtO(MH='Korea', MS='Former kingdom, located on Korea Peninsula between Sea of Japan and Yellow Sea on east coast of Asia. In 1948, the kingdom ceased and two independent countries were formed, divided by the 38th parallel.'),
  'D007724': NtO(MH='Vulvar Lichen Sclerosus', MS='Atrophy and shriveling of the SKIN of the VULVA that is characterized by the whitish LICHEN SCLEROSUS appearance, inflammation, and PRURITUS.'),
  'D007725': NtO(MH='Krukenberg Tumor', MS='Mucocellular carcinoma of the ovary, usually metastatic from the gastrointestinal tract, characterized by areas of mucoid degeneration and the presence of signet-ring-like cells. It accounts for 30%-40% of metastatic cancers to the ovaries and possibly 1%-2% of all malignant ovarian tumors. The lesions may not be discovered until the primary disease is advanced, and most patients die of their disease within a year. In some cases, a primary tumor is not found. (From Dorland, 27th ed; Holland et al., Cancer Medicine, 3d ed, p1685)'),
  'D007726': NtO(MH='Krypton', MS='A noble gas that is found in the atmosphere. It has the atomic symbol Kr, atomic number 36, atomic weight 83.80, and has been used in electric bulbs.'),
  'D007727': NtO(MH='Krypton Radioisotopes', MS='Unstable isotopes of krypton that decay or disintegrate emitting radiation. Kr atoms with atomic weights 74-77, 79, 81, 85, and 87-94 are radioactive krypton isotopes.'),
  'D007728': NtO(MH='Kupffer Cells', MS='Specialized phagocytic cells of the MONONUCLEAR PHAGOCYTE SYSTEM found on the luminal surface of the hepatic sinusoids. They filter bacteria and small foreign proteins out of the blood, and dispose of worn out red blood cells.'),
  'D007729': NtO(MH='Kuru', MS='A prion disease found exclusively among the Fore linguistic group natives of the highlands of NEW GUINEA. The illness is primarily restricted to adult females and children of both sexes. It is marked by the subacute onset of tremor and ataxia followed by motor weakness and incontinence. Death occurs within 3-6 months of disease onset. The condition is associated with ritual cannibalism, and has become rare since this practice has been discontinued. Pathologic features include a noninflammatory loss of neurons that is most prominent in the cerebellum, glial proliferation, and amyloid plaques. (From Adams et al., Principles of Neurology, 6th ed, p773)'),
  'D007730': NtO(MH='Kuwait', MS='A country located in the MIDDLE EAST, bordering the Persian Gulf, between IRAQ and SAUDI ARABIA. The capital is Kuwait City.'),
  'D007731': NtO(MH='Kveim Test', MS='Intradermal injection of a heated (pasteurized) saline suspension of sarcoid tissue obtained from a sarcoid spleen or lymph node. In patients with active sarcoidosis a dusky red nodule develops slowly over the next few weeks at the injection site. Histologic examination, an essential part of the complete test, reveals sarcoid tissue.'),
  'D007732': NtO(MH='Kwashiorkor', MS='A syndrome produced by severe protein deficiency, characterized by retarded growth, changes in skin and hair pigment, edema, and pathologic changes in the liver, including fatty infiltration, necrosis, and fibrosis. The word is a local name in Gold Coast, Africa, meaning "displaced child". Although first reported from Africa, kwashiorkor is now known throughout the world, but mainly in the tropics and subtropics. It is considered to be related to marasmus. (From Dorland, 27th ed)'),
  'D007733': NtO(MH='Kyasanur Forest Disease', MS='Tick-borne flavivirus infection occurring in the Kyasanur Forest in India.'),
  'D007734': NtO(MH='Kymography', MS='The recording of wavelike motions or undulations. It is usually used on arteries to detect variations in blood pressure.'),
  'D007735': NtO(MH='Kynuramine', MS='An aromatic ketone containing the aniline structure (ANILINE COMPOUNDS).'),
  'D007736': NtO(MH='Kynurenic Acid', MS='A broad-spectrum excitatory amino acid antagonist used as a research tool.'),
  'D007737': NtO(MH='Kynurenine', MS='A metabolite of the essential amino acid tryptophan metabolized via the tryptophan-kynurenine pathway.'),
  'D007738': NtO(MH='Kyphosis', MS='Deformities of the SPINE characterized by an exaggerated convexity of the vertebral column. The forward bending of the thoracic region usually is more than 40 degrees. This deformity sometimes is called round back or hunchback.'),
  'D007739': NtO(MH='L Cells', MS='A cultured line of C3H mouse FIBROBLASTS that do not adhere to one another and do not express CADHERINS.'),
  'D007740': NtO(MH='L Forms', MS='Bacterial variants, unable to form a complete cell wall, which are formed in cultures by various bacteria; granules (L bodies) appear, unite, and grow into amorphous bodies which multiply and give rise to bacterial cells morphologically indistinguishable from the parent strain.'),
  'D007741': NtO(MH='Labetalol', MS='A salicylamide derivative that is a non-cardioselective blocker of BETA-ADRENERGIC RECEPTORS and ALPHA-1 ADRENERGIC RECEPTORS.'),
  'D007742': NtO(MH='Labial Frenum', MS='ORAL MUCOSA that connect the GINGIVA to the midline of the upper or lower lips.'),
  'D007743': NtO(MH='Labor, Obstetric', MS='The repetitive uterine contraction during childbirth which is associated with the progressive dilation of the uterine cervix (CERVIX UTERI). Successful labor results in the expulsion of the FETUS and PLACENTA. Obstetric labor can be spontaneous or induced (LABOR, INDUCED).'),
  'D007744': NtO(MH='Obstetric Labor Complications', MS='Medical problems associated with OBSTETRIC LABOR, such as BREECH PRESENTATION; PREMATURE OBSTETRIC LABOR; HEMORRHAGE; or others. These complications can affect the well-being of the mother, the FETUS, or both.'),
  'D007745': NtO(MH='Labor Onset', MS='The beginning of true OBSTETRIC LABOR which is characterized by the cyclic uterine contractions of increasing frequency, duration, and strength causing CERVICAL DILATATION to begin (LABOR STAGE, FIRST ).'),
  'D007746': NtO(MH='Labor Presentation', MS='The position or orientation of the FETUS at near term or during OBSTETRIC LABOR, determined by its relation to the SPINE of the mother and the birth canal. The normal position is a vertical, cephalic presentation with the fetal vertex flexed on the NECK.'),
  'D007747': NtO(MH='Labor Stage, First', MS='Period from the onset of true OBSTETRIC LABOR to the complete dilatation of the CERVIX UTERI.'),
  'D007748': NtO(MH='Labor Stage, Second', MS='The period of OBSTETRIC LABOR that is from the complete dilatation of the CERVIX UTERI to the expulsion of the FETUS.'),
  'D007749': NtO(MH='Labor Stage, Third', MS='The final period of OBSTETRIC LABOR that is from the expulsion of the FETUS to the expulsion of the PLACENTA.'),
  'D007750': NtO(MH='Labor Unions', MS='Organizations comprising wage and salary workers in health-related fields for the purpose of improving their status and conditions. The concept includes labor union activities toward providing health services to members.'),
  'D007751': NtO(MH='Labor, Induced', MS='Artificially induced UTERINE CONTRACTION.'),
  'D007752': NtO(MH='Obstetric Labor, Premature', MS='Onset of OBSTETRIC LABOR before term (TERM BIRTH) but usually after the FETUS has become viable. In humans, it occurs sometime during the 29th through 38th week of PREGNANCY. TOCOLYSIS inhibits premature labor and can prevent the BIRTH of premature infants (INFANT, PREMATURE).'),
  'D007753': NtO(MH='Laboratories', MS='Facilities equipped to carry out investigative procedures.'),
  'D007754': NtO(MH='Laboratories, Dental', MS="Facilities for the performance of services related to dental treatment but not done directly in the patient's mouth."),
  'D007755': NtO(MH='Laboratories, Hospital', MS='Hospital facilities equipped to carry out investigative procedures.'),
  'D007756': NtO(MH='Laboratory Animal Science', MS='The science and technology dealing with the procurement, breeding, care, health, and selection of animals used in biomedical research and testing.'),
  'D007757': NtO(MH='Laboratory Infection', MS='Accidentally acquired infection in laboratory workers.'),
  'D007758': NtO(MH='Ear, Inner', MS='The essential part of the hearing organ consists of two labyrinthine compartments: the bony labyrinthine and the membranous labyrinth. The bony labyrinth is a complex of three interconnecting cavities or spaces (COCHLEA; VESTIBULAR LABYRINTH; and SEMICIRCULAR CANALS) in the TEMPORAL BONE. Within the bony labyrinth lies the membranous labyrinth which is a complex of sacs and tubules (COCHLEAR DUCT; SACCULE AND UTRICLE; and SEMICIRCULAR DUCTS) forming a continuous space enclosed by EPITHELIUM and connective tissue. These spaces are filled with LABYRINTHINE FLUIDS of various compositions.'),
  'D007759': NtO(MH='Labyrinth Diseases', MS='Pathological processes of the inner ear (LABYRINTH) which contains the essential apparatus of hearing (COCHLEA) and balance (SEMICIRCULAR CANALS).'),
  'D007760': NtO(MH='Labyrinth Supporting Cells', MS='Cells forming a framework supporting the sensory AUDITORY HAIR CELLS in the organ of Corti. Lateral to the medial inner hair cells, there are inner pillar cells, outer pillar cells, Deiters cells, Hensens cells, Claudius cells, Boettchers cells, and others.'),
  'D007761': NtO(MH='Labyrinthine Fluids', MS="Fluids found within the osseous labyrinth (PERILYMPH) and the membranous labyrinth (ENDOLYMPH) of the inner ear. (From Gray's Anatomy, 30th American ed, p1328, 1332)"),
  'D007762': NtO(MH='Labyrinthitis', MS='Inflammation of the inner ear (LABYRINTH).'),
  'D007763': NtO(MH='Lac Operon', MS='The genetic unit consisting of three structural genes, an operator and a regulatory gene. The regulatory gene controls the synthesis of the three structural genes: BETA-GALACTOSIDASE and beta-galactoside permease (involved with the metabolism of lactose), and beta-thiogalactoside acetyltransferase.'),
  'D007764': NtO(MH='Lacquer', MS='A liquid resinous varnish solubilized in a volatile solvent which may be combined with a pigment. It forms a hard, glossy, and protective coating when applied to wood or metal.'),
  'D007765': NtO(MH='Lacrimal Apparatus', MS='The tear-forming and tear-conducting system which includes the lacrimal glands, eyelid margins, conjunctival sac, and the tear drainage system.'),
  'D007766': NtO(MH='Lacrimal Apparatus Diseases', MS='Diseases of the lacrimal apparatus.'),
  'D007767': NtO(MH='Lacrimal Duct Obstruction', MS='Interference with the secretion of tears by the lacrimal glands. Obstruction of the LACRIMAL SAC or NASOLACRIMAL DUCT causing acute or chronic inflammation of the lacrimal sac (DACRYOCYSTITIS). It is caused also in infants by failure of the nasolacrimal duct to open into the inferior meatus and occurs about the third week of life. In adults occlusion may occur spontaneously or after injury or nasal disease. (Newell, Ophthalmology: Principles and Concepts, 7th ed, p250)'),
  'D007768': NtO(MH='Lactalbumin', MS='A major protein fraction of milk obtained from the WHEY.'),
  'D007769': NtO(MH='Lactams', MS='Cyclic AMIDES formed from aminocarboxylic acids by the elimination of water. Lactims are the enol forms of lactams.'),
  'D007770': NtO(MH='L-Lactate Dehydrogenase', MS='A tetrameric enzyme that, along with the coenzyme NAD+, catalyzes the interconversion of LACTATE and PYRUVATE. In vertebrates, genes for three different subunits (LDH-A, LDH-B and LDH-C) exist.'),
  'D007772': NtO(MH='Lactate dehydrogenase-elevating virus', MS='A species ARTERIVIRUS, occurring in a number of transplantable mouse tumors. Infected mice have permanently elevated serum levels of lactate dehydrogenase.'),
  'D007773': NtO(MH='Lactates', MS='Salts or esters of LACTIC ACID containing the general formula CH3CHOHCOOR.'),
  'D007774': NtO(MH='Lactation', MS='The processes of milk secretion by the maternal MAMMARY GLANDS after PARTURITION. The proliferation of the mammary glandular tissue, milk synthesis, and milk expulsion or let down are regulated by the interactions of several hormones including ESTRADIOL; PROGESTERONE; PROLACTIN; and OXYTOCIN.'),
  'D007775': NtO(MH='Lactation Disorders', MS='Disturbances of MILK secretion in either SEX, not necessarily related to PREGNANCY.'),
  'D007777': NtO(MH='Lactobacillaceae', MS='A family of gram-positive bacteria found regularly in the mouth and intestinal tract of man and other animals, in food and dairy products, and in fermenting vegetable juices. A few species are highly pathogenic.'),
  'D007778': NtO(MH='Lactobacillus', MS='A genus of gram-positive, microaerophilic, rod-shaped bacteria occurring widely in nature. Its species are also part of the many normal flora of the mouth, intestinal tract, and vagina of many mammals, including humans. Pathogenicity from this genus is rare.'),
  'D007779': NtO(MH='Lactobacillus acidophilus', MS='A species of gram-positive, rod-shaped bacteria isolated from the intestinal tract of humans and animals, the human mouth, and vagina. This organism produces the fermented product, acidophilus milk.'),
  'D007780': NtO(MH='Lactobacillus casei', MS='A rod-shaped bacterium isolated from milk and cheese, dairy products and dairy environments, sour dough, cow dung, silage, and human mouth, human intestinal contents and stools, and the human vagina.'),
  'D007781': NtO(MH='Lactoferrin', MS='An iron-binding protein that was originally characterized as a milk protein. It is widely distributed in secretory fluids and is found in the neutrophilic granules of LEUKOCYTES. The N-terminal part of lactoferrin possesses a serine protease which functions to inactivate the TYPE III SECRETION SYSTEM used by bacteria to export virulence proteins for host cell invasion.'),
  'D007782': NtO(MH='Lactoglobulins', MS='Globulins of milk obtained from the WHEY.'),
  'D007783': NtO(MH='Lactones', MS='Cyclic esters of hydroxy carboxylic acids, containing a 1-oxacycloalkan-2-one structure. Large cyclic lactones of over a dozen atoms are MACROLIDES.'),
  'D007784': NtO(MH='Lactoperoxidase', MS="An enzyme derived from cow's milk. It catalyzes the radioiodination of tyrosine and its derivatives and of peptides containing tyrosine."),
  'D007785': NtO(MH='Lactose', MS='A disaccharide of GLUCOSE and GALACTOSE in human and cow milk. It is used in pharmacy for tablets, in medicine as a nutrient, and in industry.'),
  'D007786': NtO(MH='Lactose Factors', MS='Plasmids which determine the ability of a bacterium to ferment lactose.'),
  'D007787': NtO(MH='Lactose Intolerance', MS='The condition resulting from the absence or deficiency of LACTASE in the MUCOSA cells of the GASTROINTESTINAL TRACT, and the inability to break down LACTOSE in milk for ABSORPTION. Bacterial fermentation of the unabsorbed lactose leads to symptoms that range from a mild indigestion (DYSPEPSIA) to severe DIARRHEA. Lactose intolerance may be an inborn error or acquired.'),
  'D007788': NtO(MH='Lactose Synthase', MS='An enzyme complex that catalyzes the transfer of GALACTOSE from UDP GALACTOSE to GLUCOSE, forming LACTOSE. The enzyme complex is composed of a B subunit, ALPHA-LACTALBUMIN, which changes the substrate specificity of the A subunit, N-ACETYLLACTOSAMINE SYNTHASE, from N-ACETYLGLUCOSAMINE to glucose making lactose synthesis the preferred reaction.'),
  'D007789': NtO(MH='Lactose Tolerance Test', MS="A measure of a patient's ability to break down lactose."),
  'D007790': NtO(MH='Lactosylceramides', MS='Glycosphingolipids which contain as their polar head group a lactose moiety bound in glycosidic linkage to the hydroxyl group of ceramide. Their accumulation in tissue, due to a defect in lactosylceramide beta-galactosidase, is the cause of lactosylceramidosis.'),
  'D007791': NtO(MH='Lactoylglutathione Lyase', MS='An enzyme that catalyzes the interconversion of methylglyoxal and lactate, with glutathione serving as a coenzyme. EC 4.4.1.5.'),
  'D007792': NtO(MH='Lactulose', MS='A synthetic disaccharide used in the treatment of constipation and hepatic encephalopathy. It has also been used in the diagnosis of gastrointestinal disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p887)'),
  'D007793': NtO(MH='Lagomorpha', MS="An order of small mammals comprising two families, Ochotonidae (pikas) and Leporidae (RABBITS and HARES). Head and body length ranges from about 125 mm to 750 mm. Hares and rabbits have a short tail, and the pikas lack a tail. Rabbits are born furless and with both eyes and ears closed. HARES are born fully haired with eyes and ears open. All are vegetarians. (From Nowak, Walker's Mammals of the World, 5th ed, p539-41)"),
  'D007794': NtO(MH='Lameness, Animal', MS='A departure from the normal gait in animals.'),
  'D007795': NtO(MH='Laminaria', MS='A genus of BROWN ALGAE in the family Laminariaceae. Dried pencil-like pieces may be inserted in the cervix where they swell as they absorb moisture, serving as osmotic dilators.'),
  'D007796': NtO(MH='Laminectomy', MS='A surgical procedure that entails removing all (laminectomy) or part (laminotomy) of selected vertebral lamina to relieve pressure on the SPINAL CORD and/or SPINAL NERVE ROOTS. Vertebral lamina is the thin flattened posterior wall of vertebral arch that forms the vertebral foramen through which pass the spinal cord and nerve roots.'),
  'D007797': NtO(MH='Laminin', MS='Large, noncollagenous glycoprotein with antigenic properties. It is localized in the basement membrane lamina lucida and functions to bind epithelial cells to the basement membrane. Evidence suggests that the protein plays a role in tumor invasion.'),
  'D007798': NtO(MH='Lampreys', MS='Common name for the only family (Petromyzontidae) of eellike fish in the order Petromyzontiformes. They are jawless but have a sucking mouth with horny teeth.'),
  'D007799': NtO(MH='Lanatosides', MS='Glycosides from DIGITALIS lanata leaf. Lanatoside C has actions similar to DIGOXIN. Mixtures of lanatosides A, B, and C have also been used. (From Martindale, The Extra Pharmacopoeia, 30th ed, p670)'),
  'D007801': NtO(MH='Langerhans Cells', MS='Recirculating, dendritic, antigen-presenting cells containing characteristic racket-shaped granules (Birbeck granules). They are found principally in the stratum spinosum of the EPIDERMIS and are rich in Class II MAJOR HISTOCOMPATIBILITY COMPLEX molecules. Langerhans cells were the first dendritic cell to be described and have been a model of study for other dendritic cells (DCs), especially other migrating DCs such as dermal DCs and INTERSTITIAL DENDRITIC CELLS.'),
  'D007802': NtO(MH='Language', MS='A verbal or nonverbal means of communicating ideas or feelings.'),
  'D007803': NtO(MH='Language Arts', MS='Skills in the use of language which lead to proficiency in written or spoken communication.'),
  'D007804': NtO(MH='Language Development', MS='The gradual expansion in complexity and meaning of symbols and sounds as perceived and interpreted by the individual through a maturational and learning process. Stages in development include babbling, cooing, word imitation with cognition, and use of short sentences.'),
  'D007805': NtO(MH='Language Development Disorders', MS='Conditions characterized by language abilities (comprehension and expression of speech and writing) that are below the expected level for a given age, generally in the absence of an intellectual impairment. These conditions may be associated with DEAFNESS; BRAIN DISEASES; MENTAL DISORDERS; or environmental factors.'),
  'D007806': NtO(MH='Language Disorders', MS='Conditions characterized by deficiencies of comprehension or expression of written and spoken forms of language. These include acquired and developmental disorders.'),
  'D007807': NtO(MH='Language Tests', MS="Tests designed to assess language behavior and abilities. They include tests of vocabulary, comprehension, grammar and functional use of language, e.g., Development Sentence Scoring, Receptive-Expressive Emergent Language Scale, Parsons Language Sample, Utah Test of Language Development, Michigan Language Inventory and Verbal Language Development Scale, Illinois Test of Psycholinguistic Abilities, Northwestern Syntax Screening Test, Peabody Picture Vocabulary Test, Ammons Full-Range Picture Vocabulary Test, and Assessment of Children's Language Comprehension."),
  'D007808': NtO(MH='Language Therapy', MS='Rehabilitation of persons with language disorders or training of children with language development disorders.'),
  'D007809': NtO(MH='Lanolin', MS="A yellow fat obtained from sheep's wool. It is used as an emollient, cosmetic, and pharmaceutic aid."),
  'D007810': NtO(MH='Lanosterol', MS='A triterpene that derives from the chair-boat-chair-boat folding of 2,3-oxidosqualene. It is metabolized to CHOLESTEROL and CUCURBITACINS.'),
  'D007811': NtO(MH='Lanthanum', MS='The prototypical element in the rare earth family of metals. It has the atomic symbol La, atomic number 57, and atomic weight 138.91. Lanthanide ion is used in experimental biology as a calcium antagonist; lanthanum oxide improves the optical properties of glass.'),
  'D007812': NtO(MH='Laos', MS='A country in Southeastern Asia, northeast of Thailand, west of Vietnam. The capital is Vientiane.'),
  'D007813': NtO(MH='Laparotomy', MS='Incision into the side of the abdomen between the ribs and pelvis.'),
  'D007814': NtO(MH='Larva', MS='Wormlike or grublike stage, following the egg in the life cycle of insects, worms, and other metamorphosing animals.'),
  'D007815': NtO(MH='Larva Migrans', MS='Infections caused by nematode larvae which never develop into the adult stage and migrate through various body tissues. They commonly infect the skin, eyes, and viscera in man. Ancylostoma brasiliensis causes cutaneous larva migrans. Toxocara causes visceral larva migrans.'),
  'D007816': NtO(MH='Larva Migrans, Visceral', MS='A condition produced in man by the prolonged migration of animal nematode larvae in extraintestinal tissues other than skin; characterized by persistent hypereosinophilia, hepatomegaly, and frequently pneumonitis, commonly caused by Toxocara canis and Toxocara cati.'),
  'D007817': NtO(MH='Laryngeal Cartilages', MS='The nine cartilages of the larynx, including the cricoid, thyroid and epiglottic, and two each of arytenoid, corniculate and cuneiform.'),
  'D007818': NtO(MH='Laryngeal Diseases', MS='Pathological processes involving any part of the LARYNX which coordinates many functions such as voice production, breathing, swallowing, and coughing.'),
  'D007819': NtO(MH='Laryngeal Edema', MS='Abnormal accumulation of fluid in tissues of any part of the LARYNX, commonly associated with laryngeal injuries and allergic reactions.'),
  'D007820': NtO(MH='Laryngeal Mucosa', MS='The mucous lining of the LARYNX, consisting of various types of epithelial cells ranging from stratified squamous EPITHELIUM in the upper larynx to ciliated columnar epithelium in the rest of the larynx, mucous GOBLET CELLS, and glands containing both mucous and serous cells.'),
  'D007821': NtO(MH='Laryngeal Muscles', MS='The striated muscle groups which move the LARYNX as a whole or its parts, such as altering tension of the VOCAL CORDS, or size of the slit (RIMA GLOTTIDIS).'),
  'D007822': NtO(MH='Laryngeal Neoplasms', MS='Cancers or tumors of the LARYNX or any of its parts: the GLOTTIS; EPIGLOTTIS; LARYNGEAL CARTILAGES; LARYNGEAL MUSCLES; and VOCAL CORDS.'),
  'D007823': NtO(MH='Laryngeal Nerves', MS='Branches of the VAGUS NERVE. The superior laryngeal nerves originate near the nodose ganglion and separate into external branches, which supply motor fibers to the cricothyroid muscles, and internal branches, which carry sensory fibers. The RECURRENT LARYNGEAL NERVE originates more caudally and carries efferents to all muscles of the larynx except the cricothyroid. The laryngeal nerves and their various branches also carry sensory and autonomic fibers to the laryngeal, pharyngeal, tracheal, and cardiac regions.'),
  'D007825': NtO(MH='Laryngectomy', MS='Total or partial excision of the larynx.'),
  'D007826': NtO(MH='Laryngismus', MS='A disorder in which the adductor muscles of the VOCAL CORDS exhibit increased activity leading to laryngeal spasm. Laryngismus causes closure of the VOCAL FOLDS and airflow obstruction during inspiration.'),
  'D007827': NtO(MH='Laryngitis', MS='Inflammation of the LARYNGEAL MUCOSA, including the VOCAL CORDS. Laryngitis is characterized by irritation, edema, and reduced pliability of the mucosa leading to VOICE DISORDERS such as APHONIA and HOARSENESS.'),
  'D007828': NtO(MH='Laryngoscopy', MS='Examination, therapy or surgery of the interior of the larynx performed with a specially designed endoscope.'),
  'D007829': NtO(MH='Laryngostenosis', MS='Developmental or acquired stricture or narrowing of the LARYNX. Symptoms of respiratory difficulty depend on the degree of laryngeal narrowing.'),
  'D007830': NtO(MH='Larynx', MS='A tubular organ of VOICE production. It is located in the anterior neck, superior to the TRACHEA and inferior to the tongue and HYOID BONE.'),
  'D007831': NtO(MH='Larynx, Artificial', MS='A device, activated electronically or by expired pulmonary air, which simulates laryngeal activity and enables a laryngectomized person to speak. Examples of the pneumatic mechanical device are the Tokyo and Van Hunen artificial larynges. Electronic devices include the Western Electric electrolarynx, Tait oral vibrator, Cooper-Rand electrolarynx and the Ticchioni pipe.'),
  'D007832': NtO(MH='Lasalocid', MS='Cationic ionophore antibiotic obtained from Streptomyces lasaliensis that, among other effects, dissociates the calcium fluxes in muscle fibers. It is used as a coccidiostat, especially in poultry.'),
  'D007834': NtO(MH='Lasers', MS='An optical source that emits photons in a coherent beam. Light Amplification by Stimulated Emission of Radiation (LASER) is brought about using devices that transform light of varying frequencies into a single intense, nearly nondivergent beam of monochromatic radiation. Lasers operate in the infrared, visible, ultraviolet, or X-ray regions of the spectrum.'),
  'D007835': NtO(MH='Lassa Fever', MS='An acute febrile human disease caused by the LASSA VIRUS.'),
  'D007836': NtO(MH='Lassa virus', MS='A species of ARENAVIRUS, part of the Old World Arenaviruses (ARENAVIRUSES, OLD WORLD), and the etiologic agent of LASSA FEVER. LASSA VIRUS is a common infective agent in humans in West Africa. Its natural host is the multimammate mouse Mastomys natalensis.'),
  'D007837': NtO(MH='Latency Period, Psychological', MS='The period from about 5 to 7 years to adolescence when there is an apparent cessation of psychosexual development.'),
  'D007839': NtO(MH='Functional Laterality', MS='Behavioral manifestations of cerebral dominance in which there is preferential use and superior functioning of either the left or the right side, as in the preferred use of the right hand or right foot.'),
  'D007840': NtO(MH='Latex', MS='A milky, product excreted from the latex canals of a variety of plant species that contain cauotchouc. Latex is composed of 25-35% caoutchouc, 60-75% water, 2% protein, 2% resin, 1.5% sugar & 1% ash. RUBBER is made by the removal of water from latex.(From Concise Encyclopedia Biochemistry and Molecular Biology, 3rd ed). Hevein proteins are responsible for LATEX HYPERSENSITIVITY. Latexes are used as inert vehicles to carry antibodies or antigens in LATEX FIXATION TESTS.'),
  'D007841': NtO(MH='Latex Fixation Tests', MS='Passive agglutination tests in which antigen is adsorbed onto latex particles which then clump in the presence of antibody specific for the adsorbed antigen. (From Stedman, 26th ed)'),
  'D007842': NtO(MH='Lathyrism', MS='A paralytic condition of the legs caused by ingestion of lathyrogens, especially BETA-AMINOPROPIONITRILE or beta-N-oxalyl amino-L-alanine, which are found in the seeds of plants of the genus LATHYRUS.'),
  'D007843': NtO(MH='Latin America', MS='The geographic area of Latin America in general and when the specific country or countries are not indicated. It usually includes Central America, South America, Mexico, and the islands of the Caribbean.'),
  'D007844': NtO(MH='Latvia', MS='A country located in Eastern Europe, bordering the Baltic Sea, between Estonia and Lithuania. The capital is Riga.'),
  'D007845': NtO(MH='Laughter', MS='An involuntary expression of merriment and pleasure; it includes the patterned motor responses as well as the inarticulate vocalization.'),
  'D007846': NtO(MH='Laundering', MS='The process of washing clothes, linens or other fabric.'),
  'D007847': NtO(MH='Laundry Service, Hospital', MS='Hospital department which administers all activities pertaining to the hospital laundry service.'),
  'D007848': NtO(MH='Laurates', MS='Salts and esters of the 12-carbon saturated monocarboxylic acid--lauric acid.'),
  'D007849': NtO(MH='Laurence-Moon Syndrome', MS='An autosomal recessive condition characterized by hypogonadism; spinocerebellar degeneration; MENTAL RETARDATION; RETINITIS PIGMENTOSA; and OBESITY. This syndrome was previously referred to as Laurence-Moon-Biedl syndrome until BARDET-BIEDL SYNDROME was identified as a distinct entity. (From N Engl J Med. 1989 Oct 12;321(15):1002-9)'),
  'D007850': NtO(MH='Lauric Acids', MS='12-Carbon saturated monocarboxylic acids.'),
  'D007851': NtO(MH='Dodecanol', MS='A saturated 12-carbon fatty alcohol obtained from coconut oil fatty acids. It has a floral odor and is used in detergents, lubricating oils, and pharmaceuticals. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D007852': NtO(MH='Lawrencium', MS='A radioactive actinide discovered in 1961. It has the atomic symbol Lr, and atomic number 103. There are twelve isotopes with mass numbers 252\xe2\x80\x93262 and 266.'),
  'D007854': NtO(MH='Lead', MS='A soft, grayish metal with poisonous salts; atomic number 82, atomic weight 207.2, symbol Pb.'),
  'D007855': NtO(MH='Lead Poisoning', MS='Poisoning that results from chronic or acute ingestion, injection, inhalation, or skin absorption of LEAD or lead compounds.'),
  'D007856': NtO(MH='Lead Radioisotopes', MS='Unstable isotopes of lead that decay or disintegrate emitting radiation. Pb atoms with atomic weights 194-203, 205, and 209-214 are radioactive lead isotopes.'),
  'D007857': NtO(MH='Leadership', MS='The function of directing or controlling the actions or attitudes of an individual or group with more or less willing acquiescence of the followers.'),
  'D007858': NtO(MH='Learning', MS='Relatively permanent change in behavior that is the result of past experience or practice. The concept includes the acquisition of knowledge.'),
  'D007859': NtO(MH='Learning Disabilities', MS="Conditions characterized by a significant discrepancy between an individual's perceived level of intellect and their ability to acquire new language and other cognitive skills. These may result from organic or psychological conditions. Relatively common subtypes include DYSLEXIA, DYSCALCULIA, and DYSGRAPHIA."),
  'D007860': NtO(MH='Leasing, Property', MS='Contractual arrangement between the lessor (owner) and the lessee in which the use of equipment or facilities is granted to the lessee for a period of time and at a specified rate.'),
  'D007861': NtO(MH='Lebanon', MS='A country located in the Middle East, bordering the Mediterranean Sea, between Israel and Syria. The capital is Beirut.'),
  'D007862': NtO(MH='Phosphatidylcholine-Sterol O-Acyltransferase', MS='An enzyme secreted from the liver into the plasma of many mammalian species. It catalyzes the esterification of the hydroxyl group of lipoprotein cholesterol by the transfer of a fatty acid from the C-2 position of lecithin. In familial lecithin:cholesterol acyltransferase deficiency disease, the absence of the enzyme results in an excess of unesterified cholesterol in plasma.'),
  'D007863': NtO(MH='Lecithin Cholesterol Acyltransferase Deficiency', MS='An autosomal recessive disorder of lipoprotein metabolism caused by mutation of LECITHIN CHOLESTEROL ACYLTRANSFERASE gene. It is characterized by low HDL-cholesterol levels, and the triad of CORNEAL OPACITIES; HEMOLYTIC ANEMIA; and PROTEINURIA with renal failure.'),
  'D007865': NtO(MH='Leeches', MS='Annelids of the class Hirudinea. Some species, the bloodsuckers, may become temporarily parasitic upon animals, including man. Medicinal leeches (HIRUDO MEDICINALIS) have been used therapeutically for drawing blood since ancient times.'),
  'D007866': NtO(MH='Leg', MS='The inferior part of the lower extremity between the KNEE and the ANKLE.'),
  'D007867': NtO(MH='Leg Bones', MS='The bones of the free part of the lower extremity in humans and of any of the four extremities in animals. It includes the PATELLA; TIBIA; and FIBULA.'),
  'D007868': NtO(MH='Leg Dermatoses', MS='A nonspecific term used to denote any cutaneous lesion or group of lesions, or eruptions of any type on the leg. (From Stedman, 25th ed)'),
  'D007869': NtO(MH='Leg Injuries', MS='General or unspecified injuries involving the leg.'),
  'D007870': NtO(MH='Leg Length Inequality', MS='A condition in which one of a pair of legs fails to grow as long as the other, which could result from injury or surgery.'),
  'D007871': NtO(MH='Leg Ulcer', MS='Ulceration of the skin and underlying structures of the lower extremity. About 90% of the cases are due to venous insufficiency (VARICOSE ULCER), 5% to arterial disease, and the remaining 5% to other causes.'),
  'D007872': NtO(MH='Legal Guardians', MS='A legal concept for individuals who are designated to act on behalf of persons who are considered incapable of acting in their own behalf, e.g., minors and persons found to be not mentally competent.'),
  'D007873': NtO(MH='Legg-Calve-Perthes Disease', MS='A particular type of FEMUR HEAD NECROSIS occurring in children, mainly male, with a course of four years or so.'),
  'D007874': NtO(MH='Leghemoglobin', MS='A hemoglobin-like oxygen-binding hemeprotein present in the nitrogen-fixing root nodules of leguminous plants. The red pigment has a molecular weight approximately 1/4 that of hemoglobin and has been suggested to act as an oxido-reduction catalyst in symbiotic nitrogen fixation.'),
  'D007875': NtO(MH='Legionella', MS="Gram-negative aerobic rods, isolated from surface water or thermally polluted lakes or streams. Member are pathogenic for man. Legionella pneumophila is the causative agent for LEGIONNAIRES' DISEASE."),
  'D007876': NtO(MH='Legionellosis', MS='Infections with bacteria of the genus LEGIONELLA.'),
  'D007877': NtO(MH="Legionnaires' Disease", MS='An acute, sometimes fatal, pneumonia-like bacterial infection characterized by high fever, malaise, muscle aches, respiratory disorders and headache. It is named for an outbreak at the 1976 Philadelphia convention of the American Legion.'),
  'D007878': NtO(MH='Legislation as Topic', MS='Works about the enactment of laws and ordinances and their regulation by official organs of a nation, state, or other legislative organization. It refers also to health-related laws and regulations in general or for which there is no specific descriptor.'),
  'D007879': NtO(MH='Legislation, Dental', MS='Laws and regulations pertaining to the field of dentistry, proposed for enactment or recently enacted by a legislative body.'),
  'D007880': NtO(MH='Legislation, Drug', MS='Laws concerned with manufacturing, dispensing, and marketing of drugs.'),
  'D007881': NtO(MH='Legislation, Food', MS='Laws and regulations concerned with industrial processing and marketing of foods and beverages.'),
  'D007882': NtO(MH='Legislation, Hospital', MS='Laws and regulations concerning hospitals, which are proposed for enactment or enacted by a legislative body.'),
  'D007883': NtO(MH='Legislation, Medical', MS='Laws and regulations, pertaining to the field of medicine, proposed for enactment or enacted by a legislative body.'),
  'D007884': NtO(MH='Legislation, Nursing', MS='Laws and regulations, pertaining to the field of nursing, proposed for enactment by a legislative body.'),
  'D007885': NtO(MH='Legislation, Pharmacy', MS='Laws and regulations, pertaining to the field of pharmacy, proposed for enactment or enacted by a legislative body.'),
  'D007886': NtO(MH='Legislation, Veterinary', MS='Laws and regulations, pertaining to the field of veterinary medicine, proposed for enactment or enacted by a legislative body.'),
  'D007887': NtO(MH='Fabaceae', MS='The large family of plants characterized by pods. Some are edible and some cause LATHYRISM or FAVISM and other forms of poisoning. Other species yield useful materials like gums from ACACIA and various LECTINS like PHYTOHEMAGGLUTININS from PHASEOLUS. Many of them harbor NITROGEN FIXATION bacteria on their roots. Many but not all species of "beans" belong to this family.'),
  'D007888': NtO(MH='Leigh Disease', MS='A group of metabolic disorders primarily of infancy characterized by the subacute onset of psychomotor retardation, hypotonia, ataxia, weakness, vision loss, eye movement abnormalities, seizures, dysphagia, and lactic acidosis. Pathological features include spongy degeneration of the neuropile of the basal ganglia, thalamus, brain stem, and spinal cord. Patterns of inheritance include X-linked recessive, autosomal recessive, and mitochondrial. Leigh disease has been associated with mutations in genes for the PYRUVATE DEHYDROGENASE COMPLEX; CYTOCHROME-C OXIDASE; ATP synthase subunit 6; and subunits of mitochondrial complex I. (From Menkes, Textbook of Child Neurology, 5th ed, p850).'),
  'D007889': NtO(MH='Leiomyoma', MS='A benign tumor derived from smooth muscle tissue, also known as a fibroid tumor. They rarely occur outside of the UTERUS and the GASTROINTESTINAL TRACT but can occur in the SKIN and SUBCUTANEOUS TISSUE, probably arising from the smooth muscle of small blood vessels in these tissues.'),
  'D007890': NtO(MH='Leiomyosarcoma', MS='A sarcoma containing large spindle cells of smooth muscle. Although it rarely occurs in soft tissue, it is common in the viscera. It is the most common soft tissue sarcoma of the gastrointestinal tract and uterus. The median age of patients is 60 years. (From Dorland, 27th ed; Holland et al., Cancer Medicine, 3d ed, p1865)'),
  'D007891': NtO(MH='Leishmania', MS='A genus of flagellate protozoa comprising several species that are pathogenic for humans. Organisms of this genus have an amastigote and a promastigote stage in their life cycles. As a result of enzymatic studies this single genus has been divided into two subgenera: Leishmania leishmania and Leishmania viannia. Species within the Leishmania leishmania subgenus include: L. aethiopica, L. arabica, L. donovani, L. enrietti, L. gerbilli, L. hertigi, L. infantum, L. major, L. mexicana, and L. tropica. The following species are those that compose the Leishmania viannia subgenus: L. braziliensis, L. guyanensis, L. lainsoni, L. naiffi, and L. shawi.'),
  'D007892': NtO(MH='Leishmania braziliensis', MS='A parasitic hemoflagellate of the subgenus Leishmania viannia that infects man and animals. It causes cutaneous (LEISHMANIASIS, CUTANEOUS), diffuse cutaneous (LEISHMANIASIS, DIFFUSE CUTANEOUS), and mucocutaneous leishmaniasis (LEISHMANIASIS, MUCOCUTANEOUS) depending on the subspecies of this organism. The sandfly, Lutzomyia, is the vector. The Leishmania braziliensis complex includes the subspecies braziliensis and peruviana. Uta, a form of cutaneous leishmaniasis in the New World, is caused by the subspecies peruviana.'),
  'D007893': NtO(MH='Leishmania donovani', MS='A parasitic hemoflagellate of the subgenus Leishmania leishmania that infects man and animals and causes visceral leishmaniasis (LEISHMANIASIS, VISCERAL). The sandfly genera Phlebotomus and Lutzomyia are the vectors.'),
  'D007894': NtO(MH='Leishmania mexicana', MS='A parasitic hemoflagellate of the subgenus Leishmania leishmania that infects man and animals including rodents. The Leishmania mexicana complex causes both cutaneous (LEISHMANIASIS, CUTANEOUS) and diffuse cutaneous leishmaniasis (LEISHMANIASIS, DIFFUSE CUTANEOUS) and includes the subspecies amazonensis, garnhami, mexicana, pifanoi, and venezuelensis. L. m. mexicana causes chiclero ulcer, a form of cutaneous leishmaniasis (LEISHMANIASIS, CUTANEOUS) in the New World. The sandfly, Lutzomyia, appears to be the vector.'),
  'D007895': NtO(MH='Leishmania tropica', MS='A parasitic hemoflagellate of the subgenus Leishmania leishmania that infects man and rodents. This taxonomic complex includes species which cause a disease called Oriental sore which is a form of cutaneous leishmaniasis (LEISHMANIASIS, CUTANEOUS) of the Old World.'),
  'D007896': NtO(MH='Leishmaniasis', MS='A disease caused by any of a number of species of protozoa in the genus LEISHMANIA. There are four major clinical types of this infection: cutaneous (Old and New World) (LEISHMANIASIS, CUTANEOUS), diffuse cutaneous (LEISHMANIASIS, DIFFUSE CUTANEOUS), mucocutaneous (LEISHMANIASIS, MUCOCUTANEOUS), and visceral (LEISHMANIASIS, VISCERAL).'),
  'D007897': NtO(MH='Leishmaniasis, Mucocutaneous', MS='A disease characterized by the chronic, progressive spread of lesions from New World cutaneous leishmaniasis caused by species of the L. braziliensis complex to the nasal, pharyngeal, and buccal mucosa some time after the appearance of the initial cutaneous lesion. Nasal obstruction and epistaxis are frequent presenting symptoms.'),
  'D007898': NtO(MH='Leishmaniasis, Visceral', MS='A chronic disease caused by LEISHMANIA DONOVANI and transmitted by the bite of several sandflies of the genera Phlebotomus and Lutzomyia. It is commonly characterized by fever, chills, vomiting, anemia, hepatosplenomegaly, leukopenia, hypergammaglobulinemia, emaciation, and an earth-gray color of the skin. The disease is classified into three main types according to geographic distribution: Indian, Mediterranean (or infantile), and African.'),
  'D007899': NtO(MH='Leisure Activities', MS='Voluntary use of free time for activities outside the daily routine.'),
  'D007900': NtO(MH='Lemur', MS='A genus of the family Lemuridae consisting of five species: L. catta (ring-tailed lemur), L. fulvus, L. macaco (acoumba or black lemur), L. mongoz (mongoose lemur), and L. variegatus (white lemur). Most members of this genus occur in forested areas on Madagascar and the Comoro Islands.'),
  'D007901': NtO(MH='Lemuridae', MS='A family of the order PRIMATES, suborder STREPSIRHINI, containing four genera which inhabit Madagascar and the Comoro Island. Most of the lemurs prefer wooded areas. The four genera are Hapalemur, LEMUR, Lepilemur, and Varecia.'),
  'D007902': NtO(MH='Length of Stay', MS='The period of confinement of a patient to a hospital or other health facility.'),
  'D007903': NtO(MH='Lens Capsule, Crystalline', MS='The thin noncellular outer covering of the CRYSTALLINE LENS composed mainly of COLLAGEN TYPE IV and GLYCOSAMINOGLYCANS. It is secreted by the embryonic anterior and posterior epithelium. The embryonic posterior epithelium later disappears.'),
  'D007904': NtO(MH='Lens Cortex, Crystalline', MS='The portion of the crystalline lens surrounding the nucleus and bound anteriorly by the epithelium and posteriorly by the capsule. It contains lens fibers and amorphous, intercellular substance.'),
  'D007905': NtO(MH='Lens Diseases', MS='Diseases involving the CRYSTALLINE LENS.'),
  'D007906': NtO(MH='Lens Subluxation', MS='Incomplete rupture of the zonule with the displaced lens remaining behind the pupil. In dislocation, or complete rupture, the lens is displaced forward into the anterior chamber or backward into the vitreous body. When congenital, this condition is known as ECTOPIA LENTIS.'),
  'D007907': NtO(MH='Lens Nucleus, Crystalline', MS='The core of the crystalline lens, surrounded by the cortex.'),
  'D007908': NtO(MH='Lens, Crystalline', MS='A transparent, biconvex structure of the EYE, enclosed in a capsule and situated behind the IRIS and in front of the vitreous humor (VITREOUS BODY). It is slightly overlapped at its margin by the ciliary processes. Adaptation by the CILIARY BODY is crucial for OCULAR ACCOMMODATION.'),
  'D007909': NtO(MH='Lenses', MS='Pieces of glass or other transparent materials used for magnification or increased visual acuity.'),
  'D007910': NtO(MH='Lenses, Intraocular', MS='Artificial implanted lenses.'),
  'D007911': NtO(MH='Lentigo', MS="Small circumscribed melanoses resembling, but differing histologically from, freckles. The concept includes senile lentigo ('liver spots') and nevoid lentigo (nevus spilus, lentigo simplex) and may also occur in association with multiple congenital defects or congenital syndromes (e.g., Peutz-Jeghers syndrome)."),
  'D007912': NtO(MH='Lentinan', MS='Polysaccharide isolated from the edible mushroom LENTINULA EDODES. The exact composition is unknown.'),
  'D007913': NtO(MH='Herpesvirus 1, Ranid', MS='A species of the family HERPESVIRIDAE, whose genus is so far unassigned. It is probably the causative agent of ADENOMA or ADENOCARCINOMA in the kidneys of certain frogs.'),
  'D007914': NtO(MH='Leper Colonies', MS='Residential treatment centers for individuals with leprosy.'),
  'D007915': NtO(MH='Lepidoptera', MS='A large order of insects comprising the butterflies and moths.'),
  'D007916': NtO(MH='Lepromin', MS=None),
  'D007917': NtO(MH='Leprostatic Agents', MS='Substances that suppress Mycobacterium leprae, ameliorate the clinical manifestations of leprosy, and/or reduce the incidence and severity of leprous reactions.'),
  'D007918': NtO(MH='Leprosy', MS='A chronic granulomatous infection caused by MYCOBACTERIUM LEPRAE. The granulomatous lesions are manifested in the skin, the mucous membranes, and the peripheral nerves. Two polar or principal types are lepromatous and tuberculoid.'),
  'D007919': NtO(MH='Leptospira', MS='A genus of aerobic, helical spirochetes, some species of which are pathogenic, others free-living or saprophytic.'),
  'D007921': NtO(MH='Leptospira interrogans', MS='A genus of question mark-shaped bacteria spirochetes which is found in fresh water that is contaminated by animal urine. It causes LEPTOSPIROSIS.'),
  'D007922': NtO(MH='Leptospirosis', MS='Infections with bacteria of the genus LEPTOSPIRA.'),
  'D007925': NtO(MH='Leriche Syndrome', MS='A condition caused by occlusion of terminal aorta, the primary branches of the ABDOMINAL AORTA, as in aortoiliac obstruction. Leriche syndrome usually occurs in males and is characterized by IMPOTENCE, absence of a pulse in the femoral arteries, weakness and numbness in the lower back, buttocks, hips, and lower limbs.'),
  'D007926': NtO(MH='Lesch-Nyhan Syndrome', MS='An inherited disorder transmitted as a sex-linked trait and caused by a deficiency of an enzyme of purine metabolism; HYPOXANTHINE PHOSPHORIBOSYLTRANSFERASE. Affected individuals are normal in the first year of life and then develop psychomotor retardation, extrapyramidal movement disorders, progressive spasticity, and seizures. Self-destructive behaviors such as biting of fingers and lips are seen frequently. Intellectual impairment may also occur but is typically not severe. Elevation of uric acid in the serum leads to the development of renal calculi and gouty arthritis. (Menkes, Textbook of Child Neurology, 5th ed, pp127)'),
  'D007927': NtO(MH='Lesotho', MS='A kingdom in southern Africa, within the republic of SOUTH AFRICA. Its capital is Maseru.'),
  'D007928': NtO(MH='Lethal Dose 50', MS='The dose amount of poisonous or toxic substance or dose of ionizing radiation required to kill 50% of the tested population.'),
  'D007930': NtO(MH='Leucine', MS='An essential branched-chain amino acid important for hemoglobin formation.'),
  'D007931': NtO(MH='Leucyl Aminopeptidase', MS='A zinc containing enzyme of the hydrolase class that catalyzes the removal of the N-terminal amino acid from most L-peptides, particularly those with N-terminal leucine residues but not those with N-terminal lysine or arginine residues. This occurs in tissue cell cytosol, with high activity in the duodenum, liver, and kidney. The activity of this enzyme is commonly assayed using a leucine arylamide chromogenic substrate such as leucyl beta-naphthylamide.'),
  'D007932': NtO(MH='Leucogenenol', MS='A heterocyclic enolic thymothyroid hormone synthesized by the THYMUS and/or THYROID.'),
  'D007933': NtO(MH='Leucomycins', MS='An antibiotic complex produced by Streptomyces kitasatoensis. The complex consists of a mixture of at least eight biologically active components, A1 and A3 to A9. Leucomycins have both antibacterial and antimycoplasmal activities.'),
  'D007934': NtO(MH='Leuconostoc', MS='A genus of gram-positive, facultatively anaerobic bacteria whose growth is dependent on the presence of a fermentable carbohydrate. It is nonpathogenic to plants and animals, including humans.'),
  'D007935': NtO(MH='Leucine-tRNA Ligase', MS='An enzyme that activates leucine with its specific transfer RNA. EC 6.1.1.4.'),
  'D007937': NtO(MH='Leukapheresis', MS='The preparation of leukocyte concentrates with the return of red cells and leukocyte-poor plasma to the donor.'),
  'D007938': NtO(MH='Leukemia', MS='A progressive, malignant disease of the blood-forming organs, characterized by distorted proliferation and development of leukocytes and their precursors in the blood and bone marrow. Leukemias were originally termed acute or chronic based on life expectancy but now are classified according to cellular maturity. Acute leukemias consist of predominately immature cells; chronic leukemias are composed of more mature cells. (From The Merck Manual, 2006)'),
  'D007939': NtO(MH='Leukemia L1210', MS='An experimental LYMPHOCYTIC LEUKEMIA of mice.'),
  'D007940': NtO(MH='Leukemia L5178', MS='An experimental lymphocytic leukemia of mice.'),
  'D007941': NtO(MH='Leukemia P388', MS='An experimental lymphocytic leukemia originally induced in DBA/2 mice by painting with methylcholanthrene.'),
  'D007942': NtO(MH='Leukemia, Experimental', MS='Leukemia induced experimentally in animals by exposure to leukemogenic agents, such as VIRUSES; RADIATION; or by TRANSPLANTATION of leukemic tissues.'),
  'D007943': NtO(MH='Leukemia, Hairy Cell', MS='A neoplastic disease of the lymphoreticular cells which is considered to be a rare type of chronic leukemia; it is characterized by an insidious onset, splenomegaly, anemia, granulocytopenia, thrombocytopenia, little or no lymphadenopathy, and the presence of "hairy" or "flagellated" cells in the blood and bone marrow.'),
  'D007945': NtO(MH='Leukemia, Lymphoid', MS='Leukemia associated with HYPERPLASIA of the lymphoid tissues and increased numbers of circulating malignant LYMPHOCYTES and lymphoblasts.'),
  'D007946': NtO(MH='Leukemia, Mast-Cell', MS='A form of systemic mastocytosis (MASTOCYTOSIS, SYSTEMIC) characterized by the presence of large numbers of tissue MAST CELLS in the peripheral blood without skin lesions. It is a high-grade LEUKEMIA disease with bone marrow smear of >20% MAST CELLS, multi-organ failure and a short survival.'),
  'D007947': NtO(MH='Leukemia, Megakaryoblastic, Acute', MS='An acute myeloid leukemia in which 20-30% of the bone marrow or peripheral blood cells are of megakaryocyte lineage. MYELOFIBROSIS or increased bone marrow RETICULIN is common.'),
  'D007948': NtO(MH='Leukemia, Monocytic, Acute', MS='An acute myeloid leukemia in which 80% or more of the leukemic cells are of monocytic lineage including monoblasts, promonocytes, and MONOCYTES.'),
  'D007951': NtO(MH='Leukemia, Myeloid', MS='Form of leukemia characterized by an uncontrolled proliferation of the myeloid lineage and their precursors (MYELOID PROGENITOR CELLS) in the bone marrow and other sites.'),
  'D007952': NtO(MH='Leukemia, Plasma Cell', MS='A rare, aggressive variant of MULTIPLE MYELOMA characterized by the circulation of excessive PLASMA CELLS in the peripheral blood. It can be a primary manifestation of multiple myeloma or develop as a terminal complication during the disease.'),
  'D007953': NtO(MH='Leukemia, Radiation-Induced', MS='Leukemia produced by exposure to IONIZING RADIATION or NON-IONIZING RADIATION.'),
  'D007955': NtO(MH='Leukemoid Reaction', MS='A peripheral blood picture resembling that of leukemia or indistinguishable from it on the basis of morphologic appearance alone. (Dorland, 27th ed)'),
  'D007956': NtO(MH='Leukocidins', MS='Pore forming proteins originally discovered for toxic activity to LEUKOCYTES. They are EXOTOXINS produced by some pathogenic STAPHYLOCOCCUS and STREPTOCOCCUS that destroy leukocytes by lysis of the cytoplasmic granules and are partially responsible for the pathogenicity of the organisms.'),
  'D007957': NtO(MH='Leukocyte Adherence Inhibition Test', MS='Test for cell-mediated antitumor immunity and related serum blocking factors based on the finding that leukocytes from cancer patients, but not from controls, when mixed in vitro with antigenic extracts of tumors of the same histological type, undergo a diminution in their normal adherence to glass surfaces. Sera from tumor-bearing patients block the LAI reaction of their own leukocytes or those of other patients with the same type of tumor.'),
  'D007958': NtO(MH='Leukocyte Count', MS='The number of WHITE BLOOD CELLS per unit volume in venous BLOOD. A differential leukocyte count measures the relative numbers of the different types of white cells.'),
  'D007959': NtO(MH='Lymphocyte Culture Test, Mixed', MS='Measure of histocompatibility at the HL-A locus. Peripheral blood lymphocytes from two individuals are mixed together in tissue culture for several days. Lymphocytes from incompatible individuals will stimulate each other to proliferate significantly (measured by tritiated thymidine uptake) whereas those from compatible individuals will not. In the one-way MLC test, the lymphocytes from one of the individuals are inactivated (usually by treatment with MITOMYCIN or radiation) thereby allowing only the untreated remaining population of cells to proliferate in response to foreign histocompatibility antigens.'),
  'D007960': NtO(MH='Leukocyte Disorders', MS='Disordered formation of various types of leukocytes or an abnormal accumulation or deficiency of these cells.'),
  'D007961': NtO(MH='Leukocyte Migration-Inhibitory Factors', MS='Protein factor(s) released by sensitized lymphocytes (and possibly other cells) that inhibit the movement of LEUKOCYTES, especially polymorphonuclear cells, away from their site of release. Assays for these factors are used as tests for cellular immunity. Two of the common assays are the LEUKOCYTE MIGRATION CAPILLARY TUBE TECHNIQUE (LMCT) and the LEUKOCYTE MIGRATION AGAROSE TEST (LMAT).'),
  'D007962': NtO(MH='Leukocytes', MS='White blood cells. These include granular leukocytes (BASOPHILS; EOSINOPHILS; and NEUTROPHILS) as well as non-granular leukocytes (LYMPHOCYTES and MONOCYTES).'),
  'D007963': NtO(MH='Leukocytes, Mononuclear', MS="Mature LYMPHOCYTES and MONOCYTES transported by the blood to the body's extravascular space. They are morphologically distinguishable from mature granulocytic leukocytes by their large, non-lobed nuclei and lack of coarse, heavily stained cytoplasmic granules."),
  'D007964': NtO(MH='Leukocytosis', MS='A transient increase in the number of leukocytes in a body fluid.'),
  'D007965': NtO(MH='Leukodystrophy, Globoid Cell', MS='An autosomal recessive metabolic disorder caused by a deficiency of GALACTOSYLCERAMIDASE leading to intralysosomal accumulation of galactolipids such as GALACTOSYLCERAMIDES and PSYCHOSINE. It is characterized by demyelination associated with large multinucleated globoid cells, predominantly involving the white matter of the central nervous system. The loss of MYELIN disrupts normal conduction of nerve impulses.'),
  'D007966': NtO(MH='Leukodystrophy, Metachromatic', MS='An autosomal recessive metabolic disease caused by a deficiency of CEREBROSIDE-SULFATASE leading to intralysosomal accumulation of cerebroside sulfate (SULFOGLYCOSPHINGOLIPIDS) in the nervous system and other organs. Pathological features include diffuse demyelination, and metachromatically-staining granules in many cell types such as the GLIAL CELLS. There are several allelic and nonallelic forms with a variety of neurological symptoms.'),
  'D007967': NtO(MH='Leukoedema, Oral', MS='A disorder of the buccal mucosa resembling early leukoplakia, characterized by the presence of filmy opalescence of the mucosa in the early stages to a whitish gray cast with a coarsely wrinkled surface in the later stages, associated with intracellular edema of the spinous or malpighian layer. (Dorland, 27th ed)'),
  'D007968': NtO(MH='Leukoencephalopathy, Progressive Multifocal', MS='An opportunistic viral infection of the central nervous system associated with conditions that impair cell-mediated immunity (e.g., ACQUIRED IMMUNODEFICIENCY SYNDROME and other IMMUNOLOGIC DEFICIENCY SYNDROMES; HEMATOLOGIC NEOPLASMS; IMMUNOSUPPRESSION; and COLLAGEN DISEASES). The causative organism is JC Polyomavirus (JC VIRUS) which primarily affects oligodendrocytes, resulting in multiple areas of demyelination. Clinical manifestations include DEMENTIA; ATAXIA; visual disturbances; and other focal neurologic deficits, generally progressing to a vegetative state within 6 months. (From Joynt, Clinical Neurology, 1996, Ch26, pp36-7)'),
  'D007969': NtO(MH='Leukomalacia, Periventricular', MS='Degeneration of white matter adjacent to the CEREBRAL VENTRICLES following cerebral hypoxia or BRAIN ISCHEMIA in neonates. The condition primarily affects white matter in the perfusion zone between superficial and deep branches of the MIDDLE CEREBRAL ARTERY. Clinical manifestations include VISION DISORDERS; CEREBRAL PALSY; PARAPLEGIA; SEIZURES; and cognitive disorders. (From Adams et al., Principles of Neurology, 6th ed, p1021; Joynt, Clinical Neurology, 1997, Ch4, pp30-1)'),
  'D007970': NtO(MH='Leukopenia', MS='A decrease in the number of LEUKOCYTES in a blood sample below the normal range (LEUKOCYTE COUNT less than 4000).'),
  'D007971': NtO(MH='Leukoplakia', MS='A white patch lesion found on a MUCOUS MEMBRANE that cannot be scraped off. Leukoplakia is generally considered a precancerous condition, however its appearance may also result from a variety of HEREDITARY DISEASES.'),
  'D007972': NtO(MH='Leukoplakia, Oral', MS='A white patch seen on the oral mucosa. It is considered a premalignant condition and is often tobacco-induced. When evidence of Epstein-Barr virus is present, the condition is called hairy leukoplakia (LEUKOPLAKIA, HAIRY).'),
  'D007973': NtO(MH='Leukorrhea', MS='A clear or white discharge from the VAGINA, consisting mainly of MUCUS.'),
  'D007975': NtO(MH='Leukotriene B4', MS='The major metabolite in neutrophil polymorphonuclear leukocytes. It stimulates polymorphonuclear cell function (degranulation, formation of oxygen-centered free radicals, arachidonic acid release, and metabolism). (From Dictionary of Prostaglandins and Related Compounds, 1990)'),
  'D007976': NtO(MH='Leupeptins', MS='A group of acylated oligopeptides produced by Actinomycetes that function as protease inhibitors. They have been known to inhibit to varying degrees trypsin, plasmin, KALLIKREINS, papain and the cathepsins.'),
  'D007977': NtO(MH='Levallorphan', MS='An opioid antagonist with properties similar to those of NALOXONE; in addition it also possesses some agonist properties. It should be used cautiously; levallorphan reverses severe opioid-induced respiratory depression but may exacerbate respiratory depression such as that induced by alcohol or other non-opioid central depressants. (From Martindale, The Extra Pharmacopoeia, 30th ed, p683)'),
  'D007978': NtO(MH='Levamisole', MS='An antihelminthic drug that has been tried experimentally in rheumatic disorders where it apparently restores the immune response by increasing macrophage chemotaxis and T-lymphocyte function. Paradoxically, this immune enhancement appears to be beneficial in rheumatoid arthritis where dermatitis, leukopenia, and thrombocytopenia, and nausea and vomiting have been reported as side effects. (From Smith and Reynard, Textbook of Pharmacology, 1991, p435-6)'),
  'D007979': NtO(MH='Levocardia', MS='Congenital abnormalities in which the HEART is in the normal position (levocardia) in the left side of the chest but some or all of the THORAX or ABDOMEN viscera are transposed laterally (SITUS INVERSUS). It is also known as situs inversus with levocardia, or isolated levocardia. This condition is often associated with severe heart defects and splenic abnormalities such as asplenia or polysplenia.'),
  'D007980': NtO(MH='Levodopa', MS='The naturally occurring form of DIHYDROXYPHENYLALANINE and the immediate precursor of DOPAMINE. Unlike dopamine itself, it can be taken orally and crosses the blood-brain barrier. It is rapidly taken up by dopaminergic neurons and converted to DOPAMINE. It is used for the treatment of PARKINSONIAN DISORDERS and is usually given with agents that inhibit its conversion to dopamine outside of the central nervous system.'),
  'D007981': NtO(MH='Levorphanol', MS='A narcotic analgesic that may be habit-forming. It is nearly as effective orally as by injection.'),
  'D007982': NtO(MH='Levulinic Acids', MS='Keto acids that are derivatives of 4-oxopentanoic acids (levulinic acid).'),
  'D007983': NtO(MH='Lewis Blood Group Antigens', MS='Carbohydrate antigens structurally related to the ABH BLOOD-GROUP SYSTEM. They may occur as a modification of saccharide chains on glycolipids or glycoproteins on cell surfaces or in plasma, or as free oligosaccharides in secretions. Lewis  antigens are not synthesized in blood cells. Instead Lewis glycolipids present in plasma are absorbed onto the surface of ERYTHROCYTES; LYMPHOCYTES; and PLATELETS. The phenotypes Le(a) and Le(b) are the result of the actions of two genes the Le gene (fucosyltransferase FUT3) and the Se gene (fucosyltransferase FUT2) on the precursor carbohydrate, glycolipid or glycoprotein. Other FUCOSYLTRANSFERASES can also synthesize the Lewis antigens.'),
  'D007984': NtO(MH='Leydig Cell Tumor', MS='Gonadal interstitial or stromal cell neoplasm composed of only LEYDIG CELLS. These tumors may produce one or more of the steroid hormones such as ANDROGENS; ESTROGENS; and CORTICOSTEROIDS. Clinical symptoms include testicular swelling, GYNECOMASTIA, sexual precocity in children, or virilization (VIRILISM) in females.'),
  'D007985': NtO(MH='Leydig Cells', MS='Steroid-producing cells in the interstitial tissue of the TESTIS. They are under the regulation of PITUITARY HORMONES; LUTEINIZING HORMONE; or interstitial cell-stimulating hormone. TESTOSTERONE is the major androgen (ANDROGENS) produced.'),
  'D007986': NtO(MH='Luteinizing Hormone', MS='A major gonadotropin secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR). Luteinizing hormone regulates steroid production by the interstitial cells of the TESTIS and the OVARY. The preovulatory LUTEINIZING HORMONE surge in females induces OVULATION, and subsequent LUTEINIZATION of the follicle. LUTEINIZING HORMONE consists of two noncovalently linked subunits, alpha and beta. Within a species, the alpha subunit is common in the three pituitary glycoprotein hormones (TSH, LH and FSH), but the beta subunit is unique and confers its biological specificity.'),
  'D007987': NtO(MH='Gonadotropin-Releasing Hormone', MS='A decapeptide that stimulates the synthesis and secretion of both pituitary gonadotropins, LUTEINIZING HORMONE and FOLLICLE STIMULATING HORMONE. GnRH is produced by neurons in the septum PREOPTIC AREA of the HYPOTHALAMUS and released into the pituitary portal blood, leading to stimulation of GONADOTROPHS in the ANTERIOR PITUITARY GLAND.'),
  'D007988': NtO(MH='Liberia', MS="A republic in western Africa, south of GUINEA and east of COTE D'IVOIRE. Its capital is Monrovia."),
  'D007989': NtO(MH='Libido', MS='The psychic drive or energy associated with sexual instinct in the broad sense (pleasure and love-object seeking). It may also connote the psychic energy associated with instincts in general that motivate behavior.'),
  'D007990': NtO(MH='Libraries', MS='Collections of systematically acquired and organized information resources, and usually providing assistance to users. (ERIC Thesaurus, http://www.eric.ed.gov/ accessed 2/1/2008)'),
  'D007991': NtO(MH='Libraries, Dental', MS='Collections of information resources relevant to DENTISTRY.'),
  'D007992': NtO(MH='Libraries, Hospital', MS='Information centers primarily serving the needs of hospital medical staff and sometimes also providing patient education and other services.'),
  'D007993': NtO(MH='Libraries, Medical', MS='Collection of information resources relevant to MEDICINE.'),
  'D007994': NtO(MH='Libraries, Nursing', MS='Collections of information resources relevant to NURSING.'),
  'D007995': NtO(MH='Library Administration', MS='Planning, organizing, staffing, direction, and control of libraries.'),
  'D007996': NtO(MH='Library Associations', MS='Organizations of persons and institutions associated with a type of library or region.'),
  'D007997': NtO(MH='Library Schools', MS='Educational institutions for individuals specializing in the field of library science or information.'),
  'D007998': NtO(MH='Library Science', MS='Study of the principles and practices of library administration and services.'),
  'D007999': NtO(MH='Library Services', MS='Services offered to the library user. They include reference and circulation.'),
  'D008000': NtO(MH='Library Surveys', MS='Collection and analysis of data pertaining to operations of a particular library, library system, or group of independent libraries, with recommendations for improvement and/or ordered plans for further development.'),
  'D008001': NtO(MH='Library Technical Services', MS='Acquisition, organization, and preparation of library materials for use, including selection, weeding, cataloging, classification, and preservation.'),
  'D008002': NtO(MH='Libya', MS='A country in northern Africa, bordering the Mediterranean Sea, between Egypt, Tunisia, and Algeria, having southern border with Chad, Niger, and Sudan. Its capital is Tripoli.'),
  'D008003': NtO(MH='Phthiraptera', MS='An order of small, wingless parasitic insects, commonly known as lice. The suborders include ANOPLURA (sucking lice); AMBLYCERA; ISCHNOCERA; and Rhynchophthirina (elephant and warthog lice).'),
  'D008004': NtO(MH='Licensure', MS='The legal authority or formal permission from authorities to carry on certain activities which by law or regulation require such permission. It may be applied to licensure of institutions as well as individuals.'),
  'D008005': NtO(MH='Licensure, Dental', MS='The granting of a license to practice dentistry.'),
  'D008006': NtO(MH='Licensure, Hospital', MS='The granting of a license to a hospital.'),
  'D008007': NtO(MH='Licensure, Medical', MS='The granting of a license to practice medicine.'),
  'D008008': NtO(MH='Licensure, Nursing', MS='The granting of a license to practice the profession of nursing.'),
  'D008009': NtO(MH='Licensure, Pharmacy', MS='The granting of a license to practice pharmacy.'),
  'D008010': NtO(MH='Lichen Planus', MS='An inflammatory, pruritic disease of the skin and mucous membranes, which can be either generalized or localized. It is characterized by distinctive purplish, flat-topped papules having a predilection for the trunk and flexor surfaces. The lesions may be discrete or coalesce to form plaques. Histologically, there is a "saw-tooth" pattern of epidermal hyperplasia and vacuolar alteration of the basal layer of the epidermis along with an intense upper dermal inflammatory infiltrate composed predominantly of T-cells. Etiology is unknown.'),
  'D008011': NtO(MH='Lichens', MS='Any of a group of plants formed by a symbiotic combination of a fungus with an algae or CYANOBACTERIA, and sometimes both. The fungal component makes up the bulk of the lichen and forms the basis for its name.'),
  'D008012': NtO(MH='Lidocaine', MS='A local anesthetic and cardiac depressant used as an antiarrhythmia agent. Its actions are more intense and its effects more prolonged than those of PROCAINE but its duration of action is shorter than that of BUPIVACAINE or PRILOCAINE.'),
  'D008013': NtO(MH='Lidoflazine', MS='Coronary vasodilator with some antiarrhythmic action.'),
  'D008014': NtO(MH='Lie Detection', MS='Ascertaining of deception through detection of emotional disturbance as manifested by changes in physiologic processes usually using a polygraph.'),
  'D008015': NtO(MH='Liechtenstein', MS='Principality in central Europe, located between Austria and Switzerland. The capital is Vaduz.'),
  'D008016': NtO(MH='Life Change Events', MS="Those occurrences, including social, psychological, and environmental, which require an adjustment or effect a change in an individual's pattern of living."),
  'D008017': NtO(MH='Life Expectancy', MS='Based on known statistical data, the number of years which any person of a given age may reasonably expected to live.'),
  'D008018': NtO(MH='Life Cycle Stages', MS='The continuous sequence of changes undergone by living organisms during the post-embryonic developmental process, such as metamorphosis in insects and amphibians. This includes the developmental stages of apicomplexans such as the malarial parasite, PLASMODIUM FALCIPARUM.'),
  'D008019': NtO(MH='Life Style', MS='Typical way of life or manner of living characteristic of an individual or group. (From APA, Thesaurus of Psychological Index Terms, 8th ed)'),
  'D008020': NtO(MH='Life Support Care', MS='Care provided patients requiring extraordinary therapeutic measures in order to sustain and prolong life.'),
  'D008021': NtO(MH='Life Support Systems', MS="Systems that provide all or most of the items necessary for maintaining life and health. Provisions are made for the supplying of oxygen, food, water, temperature and pressure control, disposition of carbon dioxide and body waste. The milieu may be a spacecraft, a submarine, or the surface of the moon. In medical care, usually under hospital conditions, LIFE SUPPORT CARE is available. (From Webster's New Collegiate Dictionary)"),
  'D008022': NtO(MH='Ligaments', MS='Shiny, flexible bands of fibrous tissue connecting together articular extremities of bones. They are pliant, tough, and inextensile.'),
  'D008023': NtO(MH='Ligaments, Articular', MS='Fibrous cords of CONNECTIVE TISSUE that attach bones to each other and hold together the many types of joints in the body. Articular ligaments are strong, elastic, and allow movement in only specific directions, depending on the individual joint.'),
  'D008024': NtO(MH='Ligands', MS='A molecule that binds to another molecule, used especially to refer to a small molecule that binds specifically to a larger molecule, e.g., an antigen binding to an antibody, a hormone or neurotransmitter binding to a receptor, or a substrate or allosteric effector binding to an enzyme. Ligands are also molecules that donate or accept a pair of electrons to form a coordinate covalent bond with the central metal atom of a coordination complex. (From Dorland, 27th ed)'),
  'D008025': NtO(MH='Ligases', MS='A class of enzymes that catalyze the formation of a bond between two substrate molecules, coupled with the hydrolysis of a pyrophosphate bond in ATP or a similar energy donor. (Dorland, 28th ed) EC 6.'),
  'D008026': NtO(MH='Ligation', MS='Application of a ligature to tie a vessel or strangulate a part.'),
  'D008027': NtO(MH='Light', MS='That portion of the electromagnetic spectrum in the visible, ultraviolet, and infrared range.'),
  'D008028': NtO(MH='Light Coagulation', MS='The coagulation of tissue by an intense beam of light, including laser (LASER COAGULATION). In the eye it is used in the treatment of retinal detachments, retinal holes, aneurysms, hemorrhages, and malignant and benign neoplasms. (Dictionary of Visual Science, 3d ed)'),
  'D008029': NtO(MH='Lighting', MS='The illumination of an environment and the arrangement of lights to achieve an effect or optimal visibility. Its application is in domestic or in public settings and in medical and non-medical environments.'),
  'D008030': NtO(MH='Lightning', MS='An abrupt high-current electric discharge that occurs in the ATMOSPHERE and that has a path length ranging from hundreds of feet to tens of miles. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D008031': NtO(MH='Lignin', MS='The most abundant natural aromatic organic polymer found in all vascular plants. Lignin together with cellulose and hemicellulose are the major cell wall components of the fibers of all wood and grass species. Lignin is composed of coniferyl, p-coumaryl, and sinapyl alcohols in varying ratios in different plant species. (From Merck Index, 11th ed)'),
  'D008032': NtO(MH='Limbic System', MS="A set of forebrain structures common to all mammals that is defined functionally and anatomically. It is implicated in the higher integration of visceral, olfactory, and somatic information as well as homeostatic responses including fundamental survival behaviors (feeding, mating, emotion). For most authors, it includes the AMYGDALA; EPITHALAMUS; GYRUS CINGULI; hippocampal formation (see HIPPOCAMPUS); HYPOTHALAMUS; PARAHIPPOCAMPAL GYRUS; SEPTAL NUCLEI; anterior nuclear group of thalamus, and portions of the basal ganglia. (Parent, Carpenter's Human Neuroanatomy, 9th ed, p744; NeuroNames, http://rprcsgi.rprc.washington.edu/neuronames/index.html (September 2, 1998))."),
  'D008033': NtO(MH='Limulus Test', MS='Sensitive method for detection of bacterial endotoxins and endotoxin-like substances that depends on the in vitro gelation of Limulus amebocyte lysate (LAL), prepared from the circulating blood (amebocytes) of the horseshoe crab, by the endotoxin or related compound. Used for detection of endotoxin in body fluids and parenteral pharmaceuticals.'),
  'D008034': NtO(MH='Lincomycin', MS='An antibiotic produced by Streptomyces lincolnensis var. lincolnensis. It has been used in the treatment of staphylococcal, streptococcal, and Bacteroides fragilis infections.'),
  'D008035': NtO(MH='Lingual Frenum', MS='MUCOUS MEMBRANE extending from floor of mouth to the under-surface of the tongue.'),
  'D008036': NtO(MH='Lingual Nerve', MS='A sensory branch of the MANDIBULAR NERVE, which is part of the trigeminal (5th cranial) nerve. The lingual nerve carries general afferent fibers from the anterior two-thirds of the tongue, the floor of the mouth, and the mandibular gingivae.'),
  'D008037': NtO(MH='Linguistics', MS='The science of language, including phonetics, phonology, morphology, syntax, semantics, pragmatics, and historical linguistics. (Random House Unabridged Dictionary, 2d ed)'),
  'D008038': NtO(MH='Liniments', MS='Heat-generating liquids that are thinner than ointments and are applied to the skin with friction. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D008039': NtO(MH='Linitis Plastica', MS='A condition where the stomach wall becomes thickened, rubbery and loses its ability to distend. The stomach assumes a "leather bottle" shape. It is most often seen in adenocarcinoma of the stomach. The term is often used synonymously with diffuse adenocarcinoma of the stomach.'),
  'D008040': NtO(MH='Genetic Linkage', MS='The co-inheritance of two or more non-allelic GENES due to their being located more or less closely on the same CHROMOSOME.'),
  'D008041': NtO(MH='Linoleic Acids', MS='Eighteen-carbon essential fatty acids that contain two double bonds.'),
  'D008042': NtO(MH='Linolenic Acids', MS='Eighteen-carbon essential fatty acids that contain three double bonds.'),
  'D008043': NtO(MH='Linseed Oil', MS='The fixed oil obtained from the dried ripe seed of linseed, Linum usitatissimum (L. Linaceae). It is used as an emollient in liniments, pastes, and medicinal soaps, and in veterinary medicine as a laxative. It is also called flaxseed oil. (Dorland, 28th ed)'),
  'D008044': NtO(MH='Linuron', MS='A selective pre- and post-emergence herbicide. (From Merck Index, 11th ed)'),
  'D008045': NtO(MH='Lions', MS='Large, chiefly nocturnal mammals of the cat family FELIDAE, species Panthera leo. They are found in Africa and southern Asia.'),
  'D008046': NtO(MH='Lip', MS='Either of the two fleshy, full-blooded margins of the mouth.'),
  'D008047': NtO(MH='Lip Diseases', MS='Diseases involving the LIP.'),
  'D008048': NtO(MH='Lip Neoplasms', MS='Tumors or cancer of the LIP.'),
  'D008049': NtO(MH='Lipase', MS='An enzyme of the hydrolase class that catalyzes the reaction of triacylglycerol and water to yield diacylglycerol and a fatty acid anion. It is produced by glands on the tongue and by the pancreas and initiates the digestion of dietary fats. (From Dorland, 27th ed) EC 3.1.1.3.'),
  'D008050': NtO(MH='Lipid A', MS='Lipid A is the biologically active component of lipopolysaccharides. It shows strong endotoxic activity and exhibits immunogenic properties.'),
  'D008051': NtO(MH='Lipid Bilayers', MS='Layers of lipid molecules which are two molecules thick. Bilayer systems are frequently studied as models of biological membranes.'),
  'D008052': NtO(MH='Lipid Metabolism, Inborn Errors', MS='Errors in the metabolism of LIPIDS resulting from inborn genetic MUTATIONS that are heritable.'),
  'D008053': NtO(MH='Lipid Mobilization', MS='LIPOLYSIS of stored LIPIDS in the ADIPOSE TISSUE to release FREE FATTY ACIDS. Mobilization of stored lipids is under the regulation of lipolytic signals (CATECHOLAMINES) or anti-lipolytic signals (INSULIN) via their actions on the hormone-sensitive LIPASE. This concept does not include lipid transport.'),
  'D008054': NtO(MH='Lipid Peroxides', MS='Peroxides produced in the presence of a free radical by the oxidation of unsaturated fatty acids in the cell in the presence of molecular oxygen. The formation of lipid peroxides results in the destruction of the original lipid leading to the loss of integrity of the membranes. They therefore cause a variety of toxic effects in vivo and their formation is considered a pathological process in biological systems. Their formation can be inhibited by antioxidants, such as vitamin E, structural separation or low oxygen tension.'),
  'D008055': NtO(MH='Lipids', MS="A generic term for fats and lipoids, the alcohol-ether-soluble constituents of protoplasm, which are insoluble in water. They comprise the fats, fatty oils, essential oils, waxes, phospholipids, glycolipids, sulfolipids, aminolipids, chromolipids (lipochromes), and fatty acids. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D008058': NtO(MH='Dihydrolipoamide Dehydrogenase', MS='A flavoprotein containing oxidoreductase that catalyzes the reduction of lipoamide by NADH to yield dihydrolipoamide and NAD+. The enzyme is a component of several MULTIENZYME COMPLEXES.'),
  'D008059': NtO(MH='Mucopolysaccharidosis I', MS='Systemic lysosomal storage disease caused by a deficiency of alpha-L-iduronidase (IDURONIDASE) and characterized by progressive physical deterioration with urinary excretion of DERMATAN SULFATE and HEPARAN SULFATE. There are three recognized phenotypes representing a spectrum of clinical severity from severe to mild: Hurler syndrome, Hurler-Scheie syndrome and Scheie syndrome (formerly mucopolysaccharidosis V). Symptoms may include DWARFISM; hepatosplenomegaly; thick, coarse facial features with low nasal bridge; corneal clouding; cardiac complications; and noisy breathing.'),
  'D008060': NtO(MH='Lipodystrophy', MS='A collection of heterogenous conditions resulting from defective LIPID METABOLISM and characterized by ADIPOSE TISSUE atrophy. Often there is redistribution of body fat resulting in peripheral fat wasting and central adiposity. They include generalized, localized, congenital, and acquired lipodystrophy.'),
  'D008061': NtO(MH='Whipple Disease', MS='A chronic systemic infection by a gram-positive bacterium, Tropheryma whippelii, mainly affecting the SMALL INTESTINE but also the JOINTS; CARDIOVASCULAR SYSTEM; and the CENTRAL NERVOUS SYSTEM. The disease is characterized by fat deposits in the INTESTINAL MUCOSA and LYMPH NODES, malabsorption, DIARRHEA with fatty stools, MALNUTRITION, and ARTHRITIS.'),
  'D008062': NtO(MH='Lipofuscin', MS='A naturally occurring lipid pigment with histochemical characteristics similar to ceroid. It accumulates in various normal tissues and apparently increases in quantity with age.'),
  'D008063': NtO(MH='Thioctic Acid', MS='An octanoic acid bridged with two sulfurs so that it is sometimes also called a pentanoic acid in some naming schemes. It is biosynthesized by cleavage of LINOLEIC ACID and is a coenzyme of oxoglutarate dehydrogenase (KETOGLUTARATE DEHYDROGENASE COMPLEX). It is used in DIETARY SUPPLEMENTS.'),
  'D008064': NtO(MH='Lipidoses', MS='Conditions characterized by abnormal lipid deposition due to disturbance in lipid metabolism, such as hereditary diseases involving lysosomal enzymes required for lipid breakdown. They are classified either by the enzyme defect or by the type of lipid involved.'),
  'D008065': NtO(MH='Lipoid Proteinosis of Urbach and Wiethe', MS='An autosomal recessive disorder characterized by glassy degenerative thickening (hyalinosis) of SKIN; MUCOSA; and certain VISCERA. This disorder is caused by mutation in the extracellular matrix protein 1 gene (ECM1). Clinical features include hoarseness and skin eruption due to widespread deposition of HYALIN.'),
  'D008066': NtO(MH='Lipolysis', MS='The metabolic process of breaking down LIPIDS to release FREE FATTY ACIDS, the major oxidative fuel for the body. Lipolysis may involve dietary lipids in the DIGESTIVE TRACT, circulating lipids in the BLOOD, and stored lipids in the ADIPOSE TISSUE or the LIVER. A number of enzymes are involved in such lipid hydrolysis, such as LIPASE and LIPOPROTEIN LIPASE from various tissues.'),
  'D008067': NtO(MH='Lipoma', MS='A benign tumor composed of fat cells (ADIPOCYTES). It can be surrounded by a thin layer of connective tissue (encapsulated), or diffuse without the capsule.'),
  'D008068': NtO(MH='Lipomatosis', MS='A disorder characterized by the accumulation of encapsulated or unencapsulated tumor-like fatty tissue resembling LIPOMA.'),
  'D008069': NtO(MH='Lipomatosis, Multiple Symmetrical', MS='A condition characterized by the growth of unencapsulated masses of ADIPOSE TISSUE symmetrically deposited around the neck, shoulders, or other sites around the body.'),
  'D008070': NtO(MH='Lipopolysaccharides', MS='Lipid-containing polysaccharides which are endotoxins and important group-specific antigens. They are often derived from the cell wall of gram-negative bacteria and induce immunoglobulin secretion. The lipopolysaccharide molecule consists of three parts: LIPID A, core polysaccharide, and O-specific chains (O ANTIGENS). When derived from Escherichia coli, lipopolysaccharides serve as polyclonal B-cell mitogens commonly used in laboratory immunology. (From Dorland, 28th ed)'),
  'D008071': NtO(MH='Lipoprotein Lipase', MS='An enzyme of the hydrolase class that catalyzes the reaction of triacylglycerol and water to yield diacylglycerol and a fatty acid anion. The enzyme hydrolyzes triacylglycerols in chylomicrons, very-low-density lipoproteins, low-density lipoproteins, and diacylglycerols. It occurs on capillary endothelial surfaces, especially in mammary, muscle, and adipose tissue. Genetic deficiency of the enzyme causes familial hyperlipoproteinemia Type I. (Dorland, 27th ed) EC 3.1.1.34.'),
  'D008072': NtO(MH='Hyperlipoproteinemia Type I', MS='An inherited condition due to a deficiency of either LIPOPROTEIN LIPASE or APOLIPOPROTEIN C-II (a lipase-activating protein). The lack of lipase activities results in inability to remove CHYLOMICRONS and TRIGLYCERIDES from the blood which has a creamy top layer after standing.'),
  'D008073': NtO(MH='Lipoprotein-X', MS='An abnormal lipoprotein present in large amounts in patients with obstructive liver diseases such as INTRAHEPATIC CHOLESTASIS. LP-X derives from the reflux of BILE lipoproteins into the bloodstream. LP-X is a low-density lipoprotein rich in free CHOLESTEROL and PHOSPHOLIPIDS but poor in TRIGLYCERIDES; CHOLESTEROL ESTERS; and protein.'),
  'D008074': NtO(MH='Lipoproteins', MS='Lipid-protein complexes involved in the transportation and metabolism of lipids in the body. They are spherical particles consisting of a hydrophobic core of TRIGLYCERIDES and CHOLESTEROL ESTERS surrounded by a layer of hydrophilic free CHOLESTEROL; PHOSPHOLIPIDS; and APOLIPOPROTEINS. Lipoproteins are classified by their varying buoyant density and sizes.'),
  'D008075': NtO(MH='Lipoproteins, HDL', MS='A class of lipoproteins of small size (4-13 nm) and dense (greater than 1.063 g/ml) particles. HDL lipoproteins, synthesized in the liver without a lipid core, accumulate cholesterol esters from peripheral tissues and transport them to the liver for re-utilization or elimination from the body (the reverse cholesterol transport). Their major protein component is APOLIPOPROTEIN A-I. HDL also shuttle APOLIPOPROTEINS C and APOLIPOPROTEINS E to and from triglyceride-rich lipoproteins during their catabolism. HDL plasma level has been inversely correlated with the risk of cardiovascular diseases.'),
  'D008076': NtO(MH='Cholesterol, HDL', MS='Cholesterol which is contained in or bound to high-density lipoproteins (HDL), including CHOLESTEROL ESTERS and free cholesterol.'),
  'D008077': NtO(MH='Lipoproteins, LDL', MS='A class of lipoproteins of small size (18-25 nm) and light (1.019-1.063 g/ml) particles with a core composed mainly of CHOLESTEROL ESTERS and smaller amounts of TRIGLYCERIDES. The surface monolayer consists mostly of PHOSPHOLIPIDS, a single copy of APOLIPOPROTEIN B-100, and free cholesterol molecules. The main LDL function is to transport cholesterol and cholesterol esters to extrahepatic tissues.'),
  'D008078': NtO(MH='Cholesterol, LDL', MS='Cholesterol which is contained in or bound to low density lipoproteins (LDL), including CHOLESTEROL ESTERS and free cholesterol.'),
  'D008079': NtO(MH='Lipoproteins, VLDL', MS='A class of lipoproteins of very light (0.93-1.006 g/ml) large size (30-80 nm) particles with a core composed mainly of TRIGLYCERIDES and a surface monolayer of PHOSPHOLIPIDS and CHOLESTEROL into which are imbedded the apolipoproteins B, E, and C. VLDL facilitates the transport of endogenously made triglycerides to extrahepatic tissues. As triglycerides and Apo C are removed, VLDL is converted to INTERMEDIATE-DENSITY LIPOPROTEINS, then to LOW-DENSITY LIPOPROTEINS from which cholesterol is delivered to the extrahepatic tissues.'),
  'D008080': NtO(MH='Liposarcoma', MS='A malignant tumor derived from primitive or embryonal lipoblastic cells. It may be composed of well-differentiated fat cells or may be dedifferentiated: myxoid (LIPOSARCOMA, MYXOID), round-celled, or pleomorphic, usually in association with a rich network of capillaries. Recurrences are common and dedifferentiated liposarcomas metastasize to the lungs or serosal surfaces. (From Dorland, 27th ed; Stedman, 25th ed)'),
  'D008081': NtO(MH='Liposomes', MS='Artificial, single or multilaminar vesicles (made from lecithins or other lipids) that are used for the delivery of a variety of biological molecules or molecular complexes to cells, for example, drug delivery and gene transfer. They are also used to study membranes and membrane proteins.'),
  'D008082': NtO(MH='Lipotropic Agents', MS='Endogenous factors or drugs that increase the transport and metabolism of LIPIDS including the synthesis of LIPOPROTEINS by the LIVER and their uptake by extrahepatic tissues.'),
  'D008083': NtO(MH='beta-Lipotropin', MS='A 90-amino acid peptide derived from post-translational processing of pro-opiomelanocortin (POMC) in the PITUITARY GLAND and the HYPOTHALAMUS. It is the C-terminal fragment of POMC with lipid-mobilizing activities, such as LIPOLYSIS and steroidogenesis. Depending on the species and the tissue sites, beta-LPH may be further processed to yield active peptides including GAMMA-LIPOTROPIN; BETA-MSH; and ENDORPHINS.'),
  'D008084': NtO(MH='Lipoxygenase', MS='An enzyme of the oxidoreductase class primarily found in PLANTS. It catalyzes reactions between linoleate and other fatty acids and oxygen to form hydroperoxy-fatty acid derivatives.'),
  'D008085': NtO(MH='Lipreading', MS="The process by which an observer comprehends speech by watching the movements of the speaker's lips without hearing the speaker's voice."),
  'D008086': NtO(MH='Lissamine Green Dyes', MS='Green dyes containing ammonium and aryl sulfonate moieties that facilitate the visualization of tissues, if given intravenously. They have mostly been used in the study of kidney physiology.'),
  'D008087': NtO(MH='Listeria', MS='A genus of bacteria which may be found in the feces of animals and man, on vegetation, and in silage. Its species are parasitic on cold-blooded and warm-blooded animals, including man.'),
  'D008088': NtO(MH='Listeriosis', MS='Infections with bacteria of the genus LISTERIA.'),
  'D008089': NtO(MH='Listeria monocytogenes', MS='A species of gram-positive, rod-shaped bacteria widely distributed in nature. It has been isolated from sewage, soil, silage, and from feces of healthy animals and man. Infection with this bacterium leads to encephalitis, meningitis, endocarditis, and abortion.'),
  'D008090': NtO(MH='Lisuride', MS='An ergot derivative that acts as an agonist at dopamine D2 receptors (DOPAMINE AGONISTS). It may also act as an antagonist at dopamine D1 receptors, and as an agonist at some serotonin receptors (SEROTONIN RECEPTOR AGONISTS).'),
  'D008091': NtO(MH='Literature', MS='Writings having excellence of form or expression and expressing ideas of permanent or universal interest. The body of written works produced in a particular language, country, or age. (Webster, 3d ed)'),
  'D008092': NtO(MH='Literature, Medieval', MS='Writings originating in the 5th century A.D. through the 15h century.'),
  'D008093': NtO(MH='Literature, Modern', MS='Writings originating from the 15th century A.D. to the present'),
  'D008094': NtO(MH='Lithium', MS='An element in the alkali metals family. It has the atomic symbol Li, atomic number 3, and atomic weight [6.938; 6.997]. Salts of lithium are used in treating BIPOLAR DISORDER.'),
  'D008095': NtO(MH='Lithocholic Acid', MS='A bile acid formed from chenodeoxycholate by bacterial action, usually conjugated with glycine or taurine. It acts as a detergent to solubilize fats for absorption and is itself absorbed. It is used as cholagogue and choleretic.'),
  'D008096': NtO(MH='Lithotripsy', MS='The destruction of a calculus of the kidney, ureter, bladder, or gallbladder by physical forces, including crushing with a lithotriptor through a catheter. Focused percutaneous ultrasound and focused hydraulic shock waves may be used without surgery. Lithotripsy does not include the dissolving of stones by acids or litholysis. Lithotripsy by laser is LITHOTRIPSY, LASER.'),
  'D008097': NtO(MH='Lithuania', MS='Country in eastern Europe, bordering the Baltic Sea, between Latvia and Russia, west of Belarus. The capital is Vilnius.'),
  'D008098': NtO(MH='Litter Size', MS='The number of offspring produced at one birth by a viviparous animal.'),
  'D008099': NtO(MH='Liver', MS='A large lobed glandular organ in the abdomen of vertebrates that is responsible for detoxification, metabolism, synthesis and storage of various substances.'),
  'D008100': NtO(MH='Liver Abscess', MS='Solitary or multiple collections of PUS within the liver as a result of infection by bacteria, protozoa, or other agents.'),
  'D008101': NtO(MH='Liver Abscess, Amebic', MS='Single or multiple areas of PUS due to infection by any ameboid protozoa (AMEBIASIS). A common form is caused by the ingestion of ENTAMOEBA HISTOLYTICA.'),
  'D008102': NtO(MH='Liver Circulation', MS='The circulation of BLOOD through the LIVER.'),
  'D008103': NtO(MH='Liver Cirrhosis', MS='Liver disease in which the normal microcirculation, the gross vascular anatomy, and the hepatic architecture have been variably destroyed and altered with fibrous septa surrounding regenerated or regenerating parenchymal nodules.'),
  'D008104': NtO(MH='Liver Cirrhosis, Alcoholic', MS='FIBROSIS of the hepatic parenchyma due to chronic excess ALCOHOL DRINKING.'),
  'D008105': NtO(MH='Liver Cirrhosis, Biliary', MS='FIBROSIS of the hepatic parenchyma due to obstruction of BILE flow (CHOLESTASIS) in the intrahepatic or extrahepatic bile ducts (BILE DUCTS, INTRAHEPATIC; BILE DUCTS, EXTRAHEPATIC). Primary biliary cholangitis involves the destruction of small intra-hepatic bile ducts and decreased bile secretion. Secondary biliary cholangitis is produced by prolonged obstruction of large intrahepatic or extrahepatic bile ducts from a variety of causes.'),
  'D008106': NtO(MH='Liver Cirrhosis, Experimental', MS='Experimentally induced chronic injuries to the parenchymal cells in the liver to achieve a model for LIVER CIRRHOSIS.'),
  'D008107': NtO(MH='Liver Diseases', MS='Pathological processes of the LIVER.'),
  'D008108': NtO(MH='Liver Diseases, Alcoholic', MS='Liver diseases associated with ALCOHOLISM. It usually refers to the coexistence of two or more subentities, i.e., ALCOHOLIC FATTY LIVER; ALCOHOLIC HEPATITIS; and ALCOHOLIC CIRRHOSIS.'),
  'D008109': NtO(MH='Liver Diseases, Parasitic', MS='Liver diseases caused by infections with PARASITES, such as tapeworms (CESTODA) and flukes (TREMATODA).'),
  'D008110': NtO(MH='Liver Extracts', MS='Extracts of liver tissue containing uncharacterized specific factors with specific activities; a soluble thermostable fraction of mammalian liver is used in the treatment of pernicious anemia.'),
  'D008111': NtO(MH='Liver Function Tests', MS="Blood tests that are used to evaluate how well a patient's liver is working and also to help diagnose liver conditions."),
  'D008112': NtO(MH='Liver Glycogen', MS='Glycogen stored in the liver. (Dorland, 28th ed)'),
  'D008113': NtO(MH='Liver Neoplasms', MS='Tumors or cancer of the LIVER.'),
  'D008114': NtO(MH='Liver Neoplasms, Experimental', MS='Experimentally induced tumors of the LIVER.'),
  'D008115': NtO(MH='Liver Regeneration', MS='Repair or renewal of hepatic tissue.'),
  'D008116': NtO(MH='Lizards', MS='Reptiles within the order Squamata that generally possess limbs, moveable EYELIDS, and EXTERNAL EAR openings, although there are some species which lack one or more of these structures.'),
  'D008117': NtO(MH='Loa', MS='A genus of parasitic nematodes found throughout the rain-forest areas of the Sudan and the basin of the Congo. L. loa inhabits the subcutaneous tissues, which it traverses freely.'),
  'D008118': NtO(MH='Loiasis', MS='A parasitic infection caused by the nematode Loa loa. The vector in the transmission of this infection is the horsefly (Tabanus) or the deerfly or mango fly (Chrysops). The larvae may be seen just beneath the skin or passing through the conjunctiva. Eye lesions are not uncommon. The disease is generally mild and painless.'),
  'D008119': NtO(MH='Lobbying', MS='A process whereby representatives of a particular interest group attempt to influence governmental decision makers to accept the policy desires of the lobbying organization.'),
  'D008120': NtO(MH='Lobeline', MS='An alkaloid that has actions similar to NICOTINE on nicotinic cholinergic receptors but is less potent. It has been proposed for a variety of therapeutic uses including in respiratory disorders, peripheral vascular disorders, insomnia, and smoking cessation.'),
  'D008121': NtO(MH='Nephropidae', MS='Family of large marine CRUSTACEA, in the order DECAPODA. These are called clawed lobsters because they bear pincers on the first three pairs of legs. The American lobster and Cape lobster in the genus Homarus are commonly used for food.'),
  'D008122': NtO(MH='Local Area Networks', MS='Communications networks connecting various hardware devices together within or between buildings by means of a continuous cable or voice data telephone system.'),
  'D008123': NtO(MH='Location Directories and Signs', MS='Directory signs or listings of designated areas within or without a facility.'),
  'D008124': NtO(MH='Locomotion', MS='Movement or the ability to move from one place or another. It can refer to humans, vertebrate or invertebrate animals, and microorganisms.'),
  'D008125': NtO(MH='Locus Coeruleus', MS='Bluish-colored region in the superior angle of the FOURTH VENTRICLE floor, corresponding to melanin-like pigmented nerve cells which lie lateral to the PERIAQUEDUCTAL GRAY.'),
  'D008126': NtO(MH='Lod Score', MS='The total relative probability, expressed on a logarithmic scale, that a linkage relationship exists among selected loci. Lod is an acronym for "logarithmic odds."'),
  'D008127': NtO(MH='Lofepramine', MS='A psychotropic IMIPRAMINE derivative that acts as a tricyclic antidepressant and possesses few anticholinergic properties. It is metabolized to DESIPRAMINE.'),
  'D008128': NtO(MH='Logic', MS='The science that investigates the principles governing correct or reliable inference and deals with the canons and criteria of validity in thought and demonstration. This system of reasoning is applicable to any branch of knowledge or study. (Random House Unabridged Dictionary, 2d ed & Sippl, Computer Dictionary, 4th ed)'),
  'D008129': NtO(MH='Lolium', MS='Common member of the Gramineae family used as cattle FODDER. It harbors several fungi and other parasites toxic to livestock and people and produces allergenic compounds, especially in its pollen. The most commonly seen varieties are L. perenne, L. multiflorum, and L. rigidum.'),
  'D008130': NtO(MH='Lomustine', MS='An alkylating agent of value against both hematologic malignancies and solid tumors.'),
  'D008131': NtO(MH='London', MS='The capital of the United Kingdom. It is located in England.'),
  'D008132': NtO(MH='Loneliness', MS='The state of feeling sad or dejected as a result of lack of companionship or being separated from others.'),
  'D008133': NtO(MH='Long QT Syndrome', MS='A condition that is characterized by episodes of fainting (SYNCOPE) and varying degree of ventricular arrhythmia as indicated by the prolonged QT interval. The inherited forms are caused by mutation of genes encoding cardiac ion channel proteins. The two major forms are ROMANO-WARD SYNDROME and JERVELL-LANGE NIELSEN SYNDROME.'),
  'D008134': NtO(MH='Long-Term Care', MS='Care over an extended period, usually for a chronic condition or disability, requiring periodic, intermittent, or continuous care.'),
  'D008135': NtO(MH='Long-Acting Thyroid Stimulator', MS='An immunoglobulin G, often found in the blood of hyperthyroid individuals. It stimulates the thyroid for a longer duration than does thyrotoxin and may cause hyperthyroidism in newborns due to placental transmission.'),
  'D008136': NtO(MH='Longevity', MS="The normal length of time of an organism's life."),
  'D008137': NtO(MH='Longitudinal Studies', MS='Studies in which variables relating to an individual or group of individuals are assessed over a period of time.'),
  'D008138': NtO(MH='Loop of Henle', MS='The U-shaped portion of the renal tubule in the KIDNEY MEDULLA, consisting of a descending limb and an ascending limb. It is situated between the PROXIMAL KIDNEY TUBULE and the DISTAL KIDNEY TUBULE.'),
  'D008139': NtO(MH='Loperamide', MS='One of the long-acting synthetic ANTIDIARRHEALS; it is not significantly absorbed from the gut, and has no effect on the adrenergic system or central nervous system, but may antagonize histamine and interfere with acetylcholine release locally.'),
  'D008140': NtO(MH='Lorazepam', MS='A benzodiazepine used as an anti-anxiety agent with few side effects. It also has hypnotic, anticonvulsant, and considerable sedative properties and has been proposed as a preanesthetic agent.'),
  'D008141': NtO(MH='Lordosis', MS='The anterior concavity in the curvature of the lumbar and cervical spine as viewed from the side. The term usually refers to abnormally increased curvature (hollow back, saddle back, swayback). It does not include lordosis as normal mating posture in certain animals ( = POSTURE + SEX BEHAVIOR, ANIMAL).'),
  'D008142': NtO(MH='Lorisidae', MS='A family of Primates of the suborder Strepsirhini containing four genera. The family is distributed in parts of Africa and Southeast Asian. The four genera are: Arctocebus (angwantibos) and Perodicticus (pottos) in the subfamily Perodicticinae and Loris (slender lorises) and Nycticebus (slow lorises) in the subfamily Lorisinae. All are arboreal and nocturnal. Genus GALAGO is now classified in the family GALAGIDAE.'),
  'D008144': NtO(MH='Loudness Perception', MS='The perceived attribute of a sound which corresponds to the physical attribute of intensity.'),
  'D008145': NtO(MH='Louisiana', MS='State bounded by Arkansas on the north, by Mississippi on the east, the Gulf of Mexico on the south and Texas on the west.'),
  'D008146': NtO(MH='Louping Ill', MS='An acute tick-borne arbovirus infection causing meningoencephalomyelitis of sheep.'),
  'D008148': NtO(MH='Lovastatin', MS='A fungal metabolite isolated from cultures of Aspergillus terreus. The compound is a potent anticholesteremic agent. It inhibits 3-hydroxy-3-methylglutaryl coenzyme A reductase (HYDROXYMETHYLGLUTARYL COA REDUCTASES), which is the rate-limiting enzyme in cholesterol biosynthesis. It also stimulates the production of low-density lipoprotein receptors in the liver.'),
  'D008149': NtO(MH='Love', MS='Affection; in psychiatry commonly refers to pleasure, particularly as it applies to gratifying experiences between individuals.'),
  'D008150': NtO(MH='Lower Body Negative Pressure', MS='External decompression applied to the lower body. It is used to study orthostatic intolerance and the effects of gravitation and acceleration, to produce simulated hemorrhage in physiologic research, to assess cardiovascular function, and to reduce abdominal stress during childbirth.'),
  'D008151': NtO(MH='Lown-Ganong-Levine Syndrome', MS='A form of ventricular pre-excitation characterized by a short PR interval and a normal QRS complex. In this syndrome, the atrial impulse conducts via the JAMES FIBERS which connect the atrium to BUNDLE OF HIS bypassing the upper ATRIOVENTRICULAR NODE. HEART VENTRICLES are depolarized normally through the His-Purkinje system.'),
  'D008152': NtO(MH='Loxapine', MS='An antipsychotic agent used in SCHIZOPHRENIA.'),
  'D008153': NtO(MH='Lubrication', MS='The application of LUBRICANTS to diminish FRICTION between two surfaces.'),
  'D008154': NtO(MH='Lucanthone', MS='One of the SCHISTOSOMICIDES, it has been replaced largely by HYCANTHONE and more recently PRAZIQUANTEL. (From Martindale The Extrapharmacopoeia, 30th ed., p46)'),
  'D008155': NtO(MH='Lucensomycin', MS='A macrolide antibiotic isolated from cultures of Streptomyces lucensis.'),
  'D008156': NtO(MH='Luciferases', MS='Enzymes that oxidize certain LUMINESCENT AGENTS to emit light (PHYSICAL LUMINESCENCE). The luciferases from different organisms have evolved differently so have different structures and substrates.'),
  'D008158': NtO(MH="Ludwig's Angina", MS='Severe cellulitis of the submaxillary space with secondary involvement of the perimandibular spaces. It usually results from infection in the lower molar area or from an infection following a penetrating injury to the MOUTH FLOOR.'),
  'D008159': NtO(MH='Lumbar Vertebrae', MS='VERTEBRAE in the region of the lower BACK below the THORACIC VERTEBRAE and above the SACRAL VERTEBRAE.'),
  'D008160': NtO(MH='Lumbosacral Plexus', MS='The lumbar and sacral plexuses taken together. The fibers of the lumbosacral plexus originate in the lumbar and upper sacral spinal cord (L1 to S3) and innervate the lower extremities.'),
  'D008161': NtO(MH='Lumbosacral Region', MS='Region of the back including the LUMBAR VERTEBRAE, SACRUM, and nearby structures.'),
  'D008162': NtO(MH='Lumicolchicines', MS='Three, alpha, beta, and gamma isomers of ultraviolet degradation products of colchicine that lack many of the physiological actions of the parent; used as experimental control for colchicine actions.'),
  'D008163': NtO(MH='Luminescent Measurements', MS='Techniques used for determining the values of photometric parameters of light resulting from LUMINESCENCE.'),
  'D008164': NtO(MH='Luminescent Proteins', MS='Proteins which are involved in the phenomenon of light emission in living systems. Included are the "enzymatic" and "non-enzymatic" types of system with or without the presence of oxygen or co-factors.'),
  'D008165': NtO(MH='Luminol', MS='5-Amino-2,3-dihydro-1,4-phthalazinedione. Substance that emits light on oxidation. It is used in chemical determinations.'),
  'D008166': NtO(MH='Lumpy Skin Disease', MS='A poxvirus infection of cattle characterized by the appearance of nodules on all parts of the skin.'),
  'D008167': NtO(MH='Lumpy skin disease virus', MS='A species of CAPRIPOXVIRUS causing a cattle disease occurring in Africa.'),
  'D008168': NtO(MH='Lung', MS='Either of the pair of organs occupying the cavity of the thorax that effect the aeration of the blood.'),
  'D008169': NtO(MH='Lung Abscess', MS='Solitary or multiple collections of PUS within the lung parenchyma as a result of infection by bacteria, protozoa, or other agents.'),
  'D008170': NtO(MH='Lung Compliance', MS="The capability of the LUNGS to distend under pressure as measured by pulmonary volume change per unit pressure change. While not a complete description of the pressure-volume properties of the lung, it is nevertheless useful in practice as a measure of the comparative stiffness of the lung. (From Best & Taylor's Physiological Basis of Medical Practice, 12th ed, p562)"),
  'D008171': NtO(MH='Lung Diseases', MS='Pathological processes involving any part of the LUNG.'),
  'D008172': NtO(MH='Lung Diseases, Fungal', MS='Pulmonary diseases caused by fungal infections, usually through hematogenous spread.'),
  'D008173': NtO(MH='Lung Diseases, Obstructive', MS='Any disorder marked by obstruction of conducting airways of the lung. AIRWAY OBSTRUCTION may be acute, chronic, intermittent, or persistent.'),
  'D008174': NtO(MH='Lung Diseases, Parasitic', MS='Infections of the lungs with parasites, most commonly by parasitic worms (HELMINTHS).'),
  'D008175': NtO(MH='Lung Neoplasms', MS='Tumors or cancer of the LUNG.'),
  'D008176': NtO(MH='Lung Volume Measurements', MS='Measurement of the amount of air that the lungs may contain at various points in the respiratory cycle.'),
  'D008177': NtO(MH='Lupus Vulgaris', MS='A form of cutaneous tuberculosis. It is seen predominantly in women and typically involves the NASAL MUCOSA; BUCCAL MUCOSA; and conjunctival mucosa.'),
  'D008178': NtO(MH='Lupus Erythematosus, Cutaneous', MS='A form of lupus erythematosus in which the skin may be the only organ involved or in which skin involvement precedes the spread into other body systems. It has been classified into three forms - acute (= LUPUS ERYTHEMATOSUS, SYSTEMIC with skin lesions), subacute, and chronic (= LUPUS ERYTHEMATOSUS, DISCOID).'),
  'D008179': NtO(MH='Lupus Erythematosus, Discoid', MS='A chronic form of cutaneous lupus erythematosus (LUPUS ERYTHEMATOSUS, CUTANEOUS) in which the skin lesions mimic those of the systemic form but in which systemic signs are rare. It is characterized by the presence of discoid skin plaques showing varying degrees of edema, erythema, scaliness, follicular plugging, and skin atrophy. Lesions are surrounded by an elevated erythematous border. The condition typically involves the face and scalp, but widespread dissemination may occur.'),
  'D008180': NtO(MH='Lupus Erythematosus, Systemic', MS='A chronic, relapsing, inflammatory, and often febrile multisystemic disorder of connective tissue, characterized principally by involvement of the skin, joints, kidneys, and serosal membranes. It is of unknown etiology, but is thought to represent a failure of the regulatory mechanisms of the autoimmune system. The disease is marked by a wide range of system dysfunctions, an elevated erythrocyte sedimentation rate, and the formation of LE cells in the blood or bone marrow.'),
  'D008181': NtO(MH='Lupus Nephritis', MS='Glomerulonephritis associated with autoimmune disease SYSTEMIC LUPUS ERYTHEMATOSUS. Lupus nephritis is histologically classified into 6 classes: class I - normal glomeruli, class II - pure mesangial alterations, class III - focal segmental glomerulonephritis, class IV - diffuse glomerulonephritis, class V - diffuse membranous glomerulonephritis, and class VI - advanced sclerosing glomerulonephritis (The World Health Organization classification 1982).'),
  'D008182': NtO(MH='Luria-Nebraska Neuropsychological Battery', MS='A series of tests designed to assess neuropsychological function. The battery is used to diagnose specific cerebral dysfunction and also to determine lateralization.'),
  'D008183': NtO(MH='Luteal Phase', MS='The period in the MENSTRUAL CYCLE that follows OVULATION, characterized by the development of CORPUS LUTEUM, increase in PROGESTERONE production by the OVARY and secretion by the glandular epithelium of the ENDOMETRIUM. The luteal phase begins with ovulation and ends with the onset of MENSTRUATION.'),
  'D008184': NtO(MH='Luteal Cells', MS='PROGESTERONE-producing cells in the CORPUS LUTEUM. The large luteal cells derive from the GRANULOSA CELLS. The small luteal cells derive from the THECA CELLS.'),
  'D008185': NtO(MH='Lutembacher Syndrome', MS='A condition characterized by a combination of OSTIUM SECUNDUM ATRIAL SEPTAL DEFECT and an acquired MITRAL VALVE STENOSIS.'),
  'D008186': NtO(MH='Luteolytic Agents', MS='Chemical compounds that cause LUTEOLYSIS or degeneration of the CORPUS LUTEUM.'),
  'D008187': NtO(MH='Lutetium', MS='An element of the rare earth family of metals. It has the atomic symbol Lu, atomic number 71, and atomic weight 175.'),
  'D008188': NtO(MH='Lutheran Blood-Group System', MS='A complex blood group system having pairs of alternate antigens and amorphic genes, but also subject to a dominant independently segregating repressor.'),
  'D008189': NtO(MH='Luxembourg', MS='Country in Europe, between France and Germany.'),
  'D008190': NtO(MH='Lyases', MS='A class of enzymes that catalyze the cleavage of C-C, C-O, and C-N, and other bonds by other means than by hydrolysis or oxidation. (Enzyme Nomenclature, 1992) EC 4.'),
  'D008191': NtO(MH='Lye', MS='Generally speaking, it is the alkaline substance obtained from wood ashes by percolation. Preparations of lye can be solutions of either potassium or sodium hydroxide. The term lye, is also used to refer to the household product which is a mixture of sodium hydroxide and sodium carbonate.'),
  'D008192': NtO(MH='Deception', MS='The act of deceiving or the fact of being deceived.'),
  'D008193': NtO(MH='Lyme Disease', MS='An infectious disease caused by a spirochete, BORRELIA BURGDORFERI, which is transmitted chiefly by Ixodes dammini (see IXODES) and pacificus ticks in the United States and Ixodes ricinis (see IXODES) in Europe. It is a disease with early and late cutaneous manifestations plus involvement of the nervous system, heart, eye, and joints in variable combinations. The disease was formerly known as Lyme arthritis and first discovered at Old Lyme, Connecticut.'),
  'D008194': NtO(MH='Lymecycline', MS='A semisynthetic antibiotic related to TETRACYCLINE. It is more readily absorbed than TETRACYCLINE and can be used in lower doses.'),
  'D008195': NtO(MH='Lymnaea', MS='A genus of dextrally coiled freshwater snails that includes some species of importance as intermediate hosts of parasitic flukes.'),
  'D008196': NtO(MH='Lymph', MS='The interstitial fluid that is in the LYMPHATIC SYSTEM.'),
  'D008197': NtO(MH='Lymph Node Excision', MS='Surgical excision of one or more lymph nodes. Its most common use is in cancer surgery. (From Dorland, 28th ed, p966)'),
  'D008198': NtO(MH='Lymph Nodes', MS='They are oval or bean shaped bodies (1 - 30 mm in diameter) located along the lymphatic system.'),
  'D008199': NtO(MH='Lymphadenitis', MS='Inflammation of the lymph nodes.'),
  'D008200': NtO(MH='Lymphangiectasis', MS='A transient dilatation of the lymphatic vessels.'),
  'D008201': NtO(MH='Lymphangiectasis, Intestinal', MS='Dilatation of the intestinal lymphatic system usually caused by an obstruction in the intestinal wall. It may be congenital or acquired and is characterized by DIARRHEA; HYPOPROTEINEMIA; peripheral and/or abdominal EDEMA; and PROTEIN-LOSING ENTEROPATHIES.'),
  'D008202': NtO(MH='Lymphangioma', MS='A benign tumor resulting from a congenital malformation of the lymphatic system. Lymphangioendothelioma is a type of lymphangioma in which endothelial cells are the dominant component.'),
  'D008203': NtO(MH='Lymphangiomyoma', MS='A tumorlike condition characterized by SMOOTH MUSCLE and ENDOTHELIUM proliferation of LYMPHATIC VESSELS and LYMPH NODES in the MEDIASTINUM and retroperitoneum, also in the lung. It may be manifested by chylous PLEURAL EFFUSION and ASCITES.'),
  'D008204': NtO(MH='Lymphangiosarcoma', MS='A malignant tumor originating from the endothelial cells of lymphatic vessels. Most lymphangiosarcomas arise in an arm secondary to radical mastectomy but they sometimes complicate idiopathic lymphedema. The lymphedema has usually been present for 6 to 10 years before malignant changes develop. (From Dorland, 27th ed; Holland et al., Cancer Medicine, 3d ed, p1866)'),
  'D008205': NtO(MH='Lymphangitis', MS='A lymphatic disease characterized by INFLAMMATION of LYMPHATIC VESSELS.'),
  'D008206': NtO(MH='Lymphatic Diseases', MS='Diseases of LYMPH; LYMPH NODES; or LYMPHATIC VESSELS.'),
  'D008207': NtO(MH='Lymphatic Metastasis', MS='Transfer of a neoplasm from its primary site to lymph nodes or to distant parts of the body by way of the lymphatic system.'),
  'D008208': NtO(MH='Lymphatic System', MS='A system of organs and tissues that process and transport immune cells and LYMPH.'),
  'D008209': NtO(MH='Lymphedema', MS='Edema due to obstruction of lymph vessels or disorders of the lymph nodes.'),
  'D008210': NtO(MH='Lymphocele', MS='Cystic mass containing lymph from diseased lymphatic channels or following surgical trauma or other injury.'),
  'D008211': NtO(MH='Lymphocyte Cooperation', MS='T-cell enhancement of the B-cell response to thymic-dependent antigens.'),
  'D008212': NtO(MH='Lymphocyte Depletion', MS='Immunosuppression by reduction of circulating lymphocytes or by T-cell depletion of bone marrow. The former may be accomplished in vivo by thoracic duct drainage or administration of antilymphocyte serum. The latter is performed ex vivo on bone marrow before its transplantation.'),
  'D008213': NtO(MH='Lymphocyte Activation', MS='Morphologic alteration of small B LYMPHOCYTES or T LYMPHOCYTES in culture into large blast-like cells able to synthesize DNA and RNA and to divide mitotically. It is induced by INTERLEUKINS; MITOGENS such as PHYTOHEMAGGLUTININS, and by specific ANTIGENS. It may also occur in vivo as in GRAFT REJECTION.'),
  'D008214': NtO(MH='Lymphocytes', MS="White blood cells formed in the body's lymphoid tissue. The nucleus is round or ovoid with coarse, irregularly clumped chromatin while the cytoplasm is typically pale blue with azurophilic (if any) granules. Most lymphocytes can be classified as either T or B (with subpopulations of each), or NATURAL KILLER CELLS."),
  'D008215': NtO(MH='Lymphocytes, Null', MS='A class of lymphocytes characterized by the lack of surface markers specific for either T or B lymphocytes.'),
  'D008216': NtO(MH='Lymphocytic Choriomeningitis', MS='A form of meningitis caused by LYMPHOCYTIC CHORIOMENINGITIS VIRUS. MICE and other rodents serve as the natural hosts, and infection in humans usually occurs through inhalation or ingestion of infectious particles. Clinical manifestations include an influenza-like syndrome followed by stiff neck, alterations of mentation, ATAXIA, and incontinence. Maternal infections may result in fetal malformations and injury, including neonatal HYDROCEPHALUS, aqueductal stenosis, CHORIORETINITIS, and MICROCEPHALY. (From Joynt, Clinical Neurology, 1996, Ch26, pp1-3)'),
  'D008217': NtO(MH='Lymphocytic choriomeningitis virus', MS='The type species of ARENAVIRUS, part of the Old World Arenaviruses (ARENAVIRUSES, OLD WORLD), producing a silent infection in house and laboratory mice. In humans, infection with LCMV can be inapparent, or can present with an influenza-like illness, a benign aseptic meningitis, or a severe meningoencephalomyelitis. The virus can also infect monkeys, dogs, field mice, guinea pigs, and hamsters, the latter an epidemiologically important host.'),
  'D008218': NtO(MH='Lymphocytosis', MS='Excess of normal lymphocytes in the blood or in any effusion.'),
  'D008219': NtO(MH='Lymphogranuloma Venereum', MS='Subacute inflammation of the inguinal lymph glands caused by certain immunotypes of CHLAMYDIA TRACHOMATIS. It is a sexually transmitted disease in the U.S. but is more widespread in developing countries. It is distinguished from granuloma venereum (see GRANULOMA INGUINALE), which is caused by Calymmatobacterium granulomatis.'),
  'D008220': NtO(MH='Lymphography', MS='Radiographic study of the lymphatic system following injection of dye or contrast medium.'),
  'D008221': NtO(MH='Lymphoid Tissue', MS='Specialized tissues that are components of the lymphatic system. They provide fixed locations within the body where a variety of LYMPHOCYTES can form, mature and multiply. The lymphoid tissues are connected by a network of LYMPHATIC VESSELS.'),
  'D008222': NtO(MH='Lymphokines', MS='Soluble protein factors generated by activated lymphocytes that affect other cells, primarily those involved in cellular immunity.'),
  'D008223': NtO(MH='Lymphoma', MS='A general term for various neoplastic diseases of the lymphoid tissue.'),
  'D008224': NtO(MH='Lymphoma, Follicular', MS='Malignant lymphoma in which the lymphomatous cells are clustered into identifiable nodules within the LYMPH NODES. The nodules resemble to some extent the GERMINAL CENTER of lymph node follicles and most likely represent neoplastic proliferation of lymph node-derived follicular center B-LYMPHOCYTES.'),
  'D008228': NtO(MH='Lymphoma, Non-Hodgkin', MS="Any of a group of malignant tumors of lymphoid tissue that differ from HODGKIN DISEASE, being more heterogeneous with respect to malignant cell lineage, clinical course, prognosis, and therapy. The only common feature among these tumors is the absence of giant REED-STERNBERG CELLS, a characteristic of Hodgkin's disease."),
  'D008230': NtO(MH='Lymphomatoid Granulomatosis', MS='An angiocentric and angiodestructive lymphoproliferative disorder primarily involving the lungs. It is caused by an Epstein-Barr virus-induced transformation of the B-cells, in a T-cell rich environment. Clinically and pathologically it resembles EXTRANODAL NK-T-CELL LYMPHOMA.'),
  'D008231': NtO(MH='Lymphopenia', MS='Reduction in the number of lymphocytes.'),
  'D008232': NtO(MH='Lymphoproliferative Disorders', MS='Disorders characterized by proliferation of lymphoid tissue, general or unspecified.'),
  'D008233': NtO(MH='Lymphotoxin-alpha', MS='A tumor necrosis factor family member that is released by activated LYMPHOCYTES. Soluble lymphotoxin is specific for TUMOR NECROSIS FACTOR RECEPTOR TYPE I; TUMOR NECROSIS FACTOR RECEPTOR TYPE II; and TUMOR NECROSIS FACTOR RECEPTOR SUPERFAMILY, MEMBER 14. Lymphotoxin-alpha can form a membrane-bound heterodimer with LYMPHOTOXIN-BETA that has specificity for the LYMPHOTOXIN BETA RECEPTOR.'),
  'D008234': NtO(MH='Lynestrenol', MS='A synthetic progestational hormone used often in mixtures with estrogens as an oral contraceptive (CONTRACEPTIVES, ORAL).'),
  'D008235': NtO(MH='Lyngbya Toxins', MS='Toxins isolated from any species of the seaweed Lyngbya or similar chemicals from other sources, including mollusks and micro-organisms. These have been found to be potent tumor promoters. They are biosynthesized from TRYPTOPHAN; VALINE; and METHIONINE nonribosomally (PEPTIDE BIOSYNTHESIS, NUCLEIC ACID-INDEPENDENT).'),
  'D008236': NtO(MH='Lypressin', MS='The porcine antidiuretic hormone (VASOPRESSINS). It is a cyclic nonapeptide that differs from ARG-VASOPRESSIN by one amino acid, containing a LYSINE at residue 8 instead of an ARGININE. Lys-vasopressin is used to treat DIABETES INSIPIDUS or to improve vasomotor tone and BLOOD PRESSURE.'),
  'D008237': NtO(MH='Lysergic Acid', MS=None),
  'D008238': NtO(MH='Lysergic Acid Diethylamide', MS='Semisynthetic derivative of ergot (Claviceps purpurea). It has complex effects on serotonergic systems including antagonism at some peripheral serotonin receptors, both agonist and antagonist actions at central nervous system serotonin receptors, and possibly effects on serotonin turnover. It is a potent hallucinogen, but the mechanisms of that effect are not well understood.'),
  'D008239': NtO(MH='Lysine', MS='An essential amino acid. It is often added to animal feed.'),
  'D008240': NtO(MH='Procollagen-Lysine, 2-Oxoglutarate 5-Dioxygenase', MS='A mixed-function oxygenase that catalyzes the hydroxylation of peptidyllysine, usually in protocollagen, to peptidylhydroxylysine. The enzyme utilizes molecular oxygen with concomitant oxidative decarboxylation of the cosubstrate 2-oxoglutarate to succinate. EC 1.14.11.4.'),
  'D008241': NtO(MH='Lysinoalanine', MS='N(6)-(2-Amino-2-carboxyethyl)-L-lysine. An unusual amino acid, not a dipeptide, which has been found in proteins of cooked foods. It is formed in food that is heated or treated with alkali. Has been implicated in nephrocytomegalia in rats.'),
  'D008242': NtO(MH='Lysogeny', MS='The phenomenon by which a temperate phage incorporates itself into the DNA of a bacterial host, establishing a kind of symbiotic relation between PROPHAGE and bacterium which results in the perpetuation of the prophage in all the descendants of the bacterium. Upon induction (VIRUS ACTIVATION) by various agents, such as ultraviolet radiation, the phage is released, which then becomes virulent and lyses the bacterium.'),
  'D008243': NtO(MH='1-Acylglycerophosphocholine O-Acyltransferase', MS='An enzyme localized predominantly within the plasma membrane of lymphocytes. It catalyzes the transfer of long-chain fatty acids, preferentially unsaturated fatty acids, to lysophosphatides with the formation of 1,2-diacylglycero-3-phosphocholine and CoA. EC 2.3.1.23.'),
  'D008244': NtO(MH='Lysophosphatidylcholines', MS='Derivatives of PHOSPHATIDYLCHOLINES obtained by their partial hydrolysis which removes one of the fatty acid moieties.'),
  'D008245': NtO(MH='Lysophospholipase', MS='An enzyme that catalyzes the hydrolysis of a single fatty acid ester bond in lysoglycerophosphatidates with the formation of glyceryl phosphatidates and a fatty acid. EC 3.1.1.5.'),
  'D008246': NtO(MH='Lysophospholipids', MS='Derivatives of PHOSPHATIDIC ACIDS that lack one of its fatty acyl chains due to its hydrolytic removal.'),
  'D008247': NtO(MH='Lysosomes', MS='A class of morphologically heterogeneous cytoplasmic particles in animal and plant tissues characterized by their content of hydrolytic enzymes and the structure-linked latency of these enzymes. The intracellular functions of lysosomes depend on their lytic potential. The single unit membrane of the lysosome acts as a barrier between the enzymes enclosed in the lysosome and the external substrate. The activity of the enzymes contained in lysosomes is limited or nil unless the vesicle in which they are enclosed is ruptured. Such rupture is supposed to be under metabolic (hormonal) control. (From Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D008248': NtO(MH='Lysostaphin', MS='A 25-kDa peptidase produced by Staphylococcus simulans which cleaves a glycine-glcyine bond unique to an inter-peptide cross-bridge of the STAPHYLOCOCCUS AUREUS cell wall.'),
  'D008249': NtO(MH='Protein-Lysine 6-Oxidase', MS='An enzyme oxidizing peptidyl-lysyl-peptide in the presence of water & molecular oxygen to yield peptidyl-allysyl-peptide plus ammonia & hydrogen peroxide. EC 1.4.3.13.'),
  'D008250': NtO(MH='Lysine-tRNA Ligase', MS='An enzyme that activates lysine with its specific transfer RNA. EC 6.1.1.6.'),
  'D008251': NtO(MH='Macaca', MS='A genus of the subfamily CERCOPITHECINAE, family CERCOPITHECIDAE, consisting of 16 species inhabiting forests of Africa, Asia, and the islands of Borneo, Philippines, and Celebes.'),
  'D008252': NtO(MH='Macaca fascicularis', MS='A species of the genus MACACA which typically lives near the coast in tidal creeks and mangrove swamps primarily on the islands of the Malay peninsula.'),
  'D008253': NtO(MH='Macaca mulatta', MS='A species of the genus MACACA inhabiting India, China, and other parts of Asia. The species is used extensively in biomedical research and adapts very well to living with humans.'),
  'D008254': NtO(MH='Macaca nemestrina', MS='A species of the genus MACACA which inhabits Malaya, Sumatra, and Borneo. It is one of the most arboreal species of Macaca. The tail is short and untwisted.'),
  'D008255': NtO(MH='Macaca radiata', MS='A species of macaque monkey that mainly inhabits the forest of southern India. They are also called bonnet macaques or bonnet monkeys.'),
  'D008256': NtO(MH='Macau', MS="Special Administrative Region of the People's Republic of China since December 20, 1999 with its own constitution. The island of Macau and adjacent islands are located off the southeast coast of China."),
  'D008257': NtO(MH='Machiavellianism', MS='A personality dimension characterized by the manipulation of others.'),
  'D008258': NtO(MH='Waldenstrom Macroglobulinemia', MS='A lymphoproliferative disorder characterized by pleomorphic B-LYMPHOCYTES including PLASMA CELLS, with increased levels of monoclonal serum IMMUNOGLOBULIN M. There is lymphoplasmacytic cells infiltration into bone marrow and often other tissues, also known as lymphoplasmacytic lymphoma. Clinical features include ANEMIA; HEMORRHAGES; and hyperviscosity.'),
  'D008259': NtO(MH='Macroglobulins', MS='Serum globulins with high molecular weight. (Dorland, 28th ed)'),
  'D008260': NtO(MH='Macroglossia', MS='The presence of an excessively large tongue, which may be congenital or may develop as a result of a tumor or edema due to obstruction of lymphatic vessels, or it may occur in association with hyperpituitarism or acromegaly. It also may be associated with malocclusion because of pressure of the tongue on the teeth. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D008262': NtO(MH='Macrophage Activation', MS='The process of altering the morphology and functional activity of macrophages so that they become avidly phagocytic. It is initiated by lymphokines, such as the macrophage activation factor (MAF) and the macrophage migration-inhibitory factor (MMIF), immune complexes, C3b, and various peptides, polysaccharides, and immunologic adjuvants.'),
  'D008263': NtO(MH='Macrophage Migration-Inhibitory Factors', MS='Proteins released by sensitized LYMPHOCYTES and possibly other cells that inhibit the migration of MACROPHAGES away from the release site. The structure and chemical properties may vary with the species and type of releasing cell.'),
  'D008264': NtO(MH='Macrophages', MS='The relatively long-lived phagocytic cell of mammalian tissues that are derived from blood MONOCYTES. Main types are PERITONEAL MACROPHAGES; ALVEOLAR MACROPHAGES; HISTIOCYTES; KUPFFER CELLS of the liver; and OSTEOCLASTS. They may further differentiate within chronic inflammatory lesions to EPITHELIOID CELLS or may fuse to form FOREIGN BODY GIANT CELLS or LANGHANS GIANT CELLS. (from The Dictionary of Cell Biology, Lackie and Dow, 3rd ed.)'),
  'D008265': NtO(MH='Macrostomia', MS='Greatly exaggerated width of the mouth, resulting from failure of union of the maxillary and mandibular processes, with extension of the oral orifice toward the ear. The defect may be unilateral or bilateral. (Dorland, 27th ed)'),
  'D008266': NtO(MH='Macula Lutea', MS='An oval area in the retina, 3 to 5 mm in diameter, usually located temporal to the posterior pole of the eye and slightly below the level of the optic disk. It is characterized by the presence of a yellow pigment diffusely permeating the inner layers, contains the fovea centralis in its center, and provides the best phototropic visual acuity. It is devoid of retinal blood vessels, except in its periphery, and receives nourishment from the choriocapillaris of the choroid. (From Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D008267': NtO(MH='Acoustic Maculae', MS='The sensory areas on the vertical wall of the saccule and in the floor of the utricle. The hair cells in the maculae are innervated by fibers of the VESTIBULAR NERVE.'),
  'D008268': NtO(MH='Macular Degeneration', MS='Degenerative changes in the RETINA usually of older adults which results in a loss of vision in the center of the visual field (the MACULA LUTEA) because of damage to the retina. It occurs in dry and wet forms.'),
  'D008269': NtO(MH='Macular Edema', MS='Fluid accumulation in the outer layer of the MACULA LUTEA that results from intraocular or systemic insults. It may develop in a diffuse pattern where the macula appears thickened or it may acquire the characteristic petaloid appearance referred to as cystoid macular edema. Although macular edema may be associated with various underlying conditions, it is most commonly seen following intraocular surgery, venous occlusive disease, DIABETIC RETINOPATHY, and posterior segment inflammatory disease. (From Survey of Ophthalmology 2004; 49(5) 470-90)'),
  'D008270': NtO(MH='Madagascar', MS="One of the Indian Ocean Islands off the southeast coast of Africa. Its capital is Antananarivo. It was formerly called the Malagasy Republic. Discovered by the Portuguese in 1500, its history has been tied predominantly to the French, becoming a French protectorate in 1882, a French colony in 1896, and a territory within the French union in 1946. The Malagasy Republic was established in the French Community in 1958 but it achieved independence in 1960. Its name was changed to Madagascar in 1975. (From Webster's New Geographical Dictionary, 1988, p714)"),
  'D008271': NtO(MH='Mycetoma', MS='A chronic progressive subcutaneous infection caused by species of fungi (eumycetoma), or actinomycetes (actinomycetoma). It is characterized by tumefaction, abscesses, and tumor-like granules representing microcolonies of pathogens, such as MADURELLA fungi and bacteria ACTINOMYCETES, with different grain colors.'),
  'D008272': NtO(MH='Mafenide', MS='A sulfonamide that inhibits the enzyme CARBONIC ANHYDRASE and is used as a topical anti-bacterial agent, especially in burn therapy.'),
  'D008273': NtO(MH='Magic', MS='Beliefs and practices concerned with producing desired results through supernatural forces or agents as with the manipulation of fetishes or rituals.'),
  'D008274': NtO(MH='Magnesium', MS='A metallic element that has the atomic symbol Mg, atomic number 12, and atomic weight 24.31. It is important for the activity of many enzymes, especially those involved in OXIDATIVE PHOSPHORYLATION.'),
  'D008275': NtO(MH='Magnesium Deficiency', MS="A nutritional condition produced by a deficiency of magnesium in the diet, characterized by anorexia, nausea, vomiting, lethargy, and weakness. Symptoms are paresthesias, muscle cramps, irritability, decreased attention span, and mental confusion, possibly requiring months to appear. Deficiency of body magnesium can exist even when serum values are normal. In addition, magnesium deficiency may be organ-selective, since certain tissues become deficient before others. (Harrison's Principles of Internal Medicine, 12th ed, p1936)"),
  'D008276': NtO(MH='Magnesium Hydroxide', MS='An inorganic compound that occurs in nature as the mineral brucite. It acts as an antacid with cathartic effects.'),
  'D008277': NtO(MH='Magnesium Oxide', MS='Magnesium oxide (MgO). An inorganic compound that occurs in nature as the mineral periclase. In aqueous media combines quickly with water to form magnesium hydroxide. It is used as an antacid and mild laxative and has many nonmedicinal uses.'),
  'D008278': NtO(MH='Magnesium Sulfate', MS='A small colorless crystal used as an anticonvulsant, a cathartic, and an electrolyte replenisher in the treatment of pre-eclampsia and eclampsia. It causes direct inhibition of action potentials in myometrial muscle cells. Excitation and contraction are uncoupled, which decreases the frequency and force of contractions. (From AMA Drug Evaluations Annual, 1992, p1083)'),
  'D008279': NtO(MH='Magnetic Resonance Imaging', MS='Non-invasive method of demonstrating internal anatomy based on the principle that atomic nuclei in a strong magnetic field absorb pulses of radiofrequency energy and emit them as radiowaves which can be reconstructed into computerized images. The concept includes proton spin tomographic techniques.'),
  'D008280': NtO(MH='Magnetics', MS='The study of MAGNETIC PHENOMENA.'),
  'D008281': NtO(MH='Maine', MS='A state bounded on the east by the Atlantic Ocean, on the north by Canada, and on the south and west by New Hampshire.'),
  'D008282': NtO(MH='Mainstreaming, Education', MS='Most frequently refers to the integration of a physically or mentally disabled child into the regular class of normal peers and provision of the appropriately determined educational program.'),
  'D008283': NtO(MH='Maintenance', MS='The upkeep of property or equipment.'),
  'D008284': NtO(MH='Maintenance and Engineering, Hospital', MS='Hospital department whose primary function is the upkeep and supervision of the buildings and grounds and the maintenance of hospital physical plant and equipment which requires engineering expertise.'),
  'D008285': NtO(MH='Major Histocompatibility Complex', MS='The genetic region which contains the loci of genes which determine the structure of the serologically defined (SD) and lymphocyte-defined (LD) TRANSPLANTATION ANTIGENS, genes which control the structure of the IMMUNE RESPONSE-ASSOCIATED ANTIGENS, HUMAN; the IMMUNE RESPONSE GENES which control the ability of an animal to respond immunologically to antigenic stimuli, and genes which determine the structure and/or level of the first four components of complement.'),
  'D008286': NtO(MH='Malabsorption Syndromes', MS='General term for a group of MALNUTRITION syndromes caused by failure of normal INTESTINAL ABSORPTION of nutrients.'),
  'D008287': NtO(MH='Malacoplakia', MS='The formation of soft patches on the mucous membrane of a hollow organ, such as the urogenital tract or digestive tract.'),
  'D008288': NtO(MH='Malaria', MS='A protozoan disease caused in humans by four species of the PLASMODIUM genus: PLASMODIUM FALCIPARUM; PLASMODIUM VIVAX; PLASMODIUM OVALE; and PLASMODIUM MALARIAE; and transmitted by the bite of an infected female mosquito of the genus ANOPHELES. Malaria is endemic in parts of Asia, Africa, Central and South America, Oceania, and certain Caribbean islands. It is characterized by extreme exhaustion associated with paroxysms of high FEVER; SWEATING; shaking CHILLS; and ANEMIA. Malaria in ANIMALS is caused by other species of plasmodia.'),
  'D008289': NtO(MH='Malaria, Avian', MS='Any of a group of infections of fowl caused by protozoa of the genera PLASMODIUM, Leucocytozoon, and Haemoproteus. The life cycles of these parasites and the disease produced bears strong resemblance to those observed in human malaria.'),
  'D008290': NtO(MH='Malassezia', MS='A mitosporic fungal genus that causes a variety of skin disorders. Malassezia furfur (Pityrosporum orbiculare) causes TINEA VERSICOLOR.'),
  'D008291': NtO(MH='Malate Dehydrogenase', MS='An enzyme that catalyzes the conversion of (S)-malate and NAD+ to oxaloacetate and NADH. EC 1.1.1.37.'),
  'D008292': NtO(MH='Malate Synthase', MS='An important enzyme in the glyoxylic acid cycle which reversibly catalyzes the synthesis of L-malate from acetyl-CoA and glyoxylate. This enzyme was formerly listed as EC 4.1.3.2.'),
  'D008293': NtO(MH='Malates', MS='Derivatives of malic acid (the structural formula: (COO-)2CH2CHOH), including its salts and esters.'),
  'D008294': NtO(MH='Malathion', MS='A wide spectrum aliphatic organophosphate insecticide widely used for both domestic and commercial agricultural purposes.'),
  'D008295': NtO(MH='Malawi', MS='A republic in southern Africa east of ZAMBIA and MOZAMBIQUE. Its capital is Lilongwe. It was formerly called Nyasaland.'),
  'D008296': NtO(MH='Malaysia', MS="A parliamentary democracy with a constitutional monarch in southeast Asia, consisting of 11 states (West Malaysia) on the Malay Peninsula and two states (East Malaysia) on the island of BORNEO. It is also called the Federation of Malaysia. Its capital is Kuala Lumpur. Before 1963 it was the Union of Malaya. It reorganized in 1948 as the Federation of Malaya, becoming independent from British Malaya in 1957 and becoming Malaysia in 1963 as a federation of Malaya, Sabah, Sarawak, and Singapore (which seceded in 1965). The form Malay- probably derives from the Tamil malay, mountain, with reference to its geography. (From Webster's New Geographical Dictionary, 1988, p715 & Room, Brewer's Dictionary of Names, 1992, p329)"),
  'D008297': NtO(MH='Male', MS=None),
  'D008298': NtO(MH='Maleates', MS='Derivatives of maleic acid (the structural formula (COO-)-C=C-(COO-)), including its salts and esters.'),
  'D008299': NtO(MH='Maleic Anhydrides', MS='Used in copolymerization reactions, in the Diels-Alder(diene)synthesis, in the preparation of resins, pharmaceuticals and agricultural chemicals. It is a powerful irritant and causes burns.'),
  'D008300': NtO(MH='Maleic Hydrazide', MS='1,2-Dihydro-3,6-pyridazinedione. A herbicide and plant growth regulator; also used to control suckering of tobacco. Its residue in food and tobacco is highly toxic, causing CNS disturbances and liver damage.'),
  'D008301': NtO(MH='Maleimides', MS='Derivatives of maleimide (the structural formula H2C2(CO)2NH) containing a pyrroledione ring where the hydrogen atom of the NH group is replaced with aliphatic or aromatic groups.'),
  'D008302': NtO(MH='Mali', MS='A country in western Africa, east of MAURITANIA and south of ALGERIA. Its capital is Bamako. From 1904-1920 it was known as Upper Senegal-Niger; prior to 1958, as French Sudan; 1958-1960 as the Sudanese Republic and 1959-1960 it joined Senegal in the Mali Federation. It became an independent republic in 1960.'),
  'D008303': NtO(MH='Malignant Carcinoid Syndrome', MS='A symptom complex associated with CARCINOID TUMOR and characterized by attacks of severe flushing of the skin, diarrheal watery stools, bronchoconstriction, sudden drops in blood pressure, edema, and ascites. The carcinoid tumors are usually located in the gastrointestinal tract and metastasize to the liver. Symptoms are caused by tumor secretion of serotonin, prostaglandins, and other biologically active substances. Cardiac manifestations constitute CARCINOID HEART DISEASE. (Dorland, 27th ed; Stedman, 25th ed)'),
  'D008304': NtO(MH='Malignant Catarrh', MS='A herpesvirus infection of cattle characterized by catarrhal inflammation of the upper respiratory and alimentary epithelia, keratoconjunctivitis, encephalitis and lymph node enlargement. Syn: bovine epitheliosis, snotsiekte.'),
  'D008305': NtO(MH='Malignant Hyperthermia', MS='Rapid and excessive rise of temperature accompanied by muscular rigidity following general anesthesia.'),
  'D008306': NtO(MH='Malingering', MS='Simulation of symptoms of illness or injury with intent to deceive in order to obtain a goal, e.g., a claim of physical illness to avoid jury duty.'),
  'D008307': NtO(MH='Malleus', MS='The largest of the auditory ossicles, and the one attached to the membrana tympani (TYMPANIC MEMBRANE). Its club-shaped head articulates with the INCUS.'),
  'D008309': NtO(MH='Mallory-Weiss Syndrome', MS='A condition characterized by mucosal tears at the ESOPHAGOGASTRIC JUNCTION, sometimes with HEMATEMESIS. Typically it is caused by forceful bouts of retching or VOMITING.'),
  'D008310': NtO(MH='Malocclusion', MS='Such malposition and contact of the maxillary and mandibular teeth as to interfere with the highest efficiency during the excursive movements of the jaw that are essential for mastication. (Jablonski, Illustrated Dictionary of Dentistry, 1982)'),
  'D008311': NtO(MH='Malocclusion, Angle Class I', MS='Malocclusion in which the mandible and maxilla are anteroposteriorly normal as reflected by the relationship of the first permanent molar (i.e., in neutroclusion), but in which individual teeth are abnormally related to each other.'),
  'D008312': NtO(MH='Malocclusion, Angle Class II', MS='Malocclusion in which the mandible is posterior to the maxilla as reflected by the relationship of the first permanent molar (distoclusion).'),
  'D008313': NtO(MH='Malocclusion, Angle Class III', MS='Malocclusion in which the mandible is anterior to the maxilla as reflected by the first relationship of the first permanent molar (mesioclusion).'),
  'D008314': NtO(MH='Malonates', MS='Derivatives of malonic acid (the structural formula CH2(COOH)2), including its salts and esters.'),
  'D008315': NtO(MH='Malondialdehyde', MS='The dialdehyde of malonic acid.'),
  'D008316': NtO(MH='Malonyl Coenzyme A', MS='A coenzyme A derivative which plays a key role in the fatty acid synthesis in the cytoplasmic and microsomal systems.'),
  'D008317': NtO(MH='Malpighian Tubules', MS='Slender tubular or hairlike excretory structures found in insects. They emerge from the alimentary canal between the mesenteron (midgut) and the proctodeum (hindgut).'),
  'D008318': NtO(MH='Malpractice', MS='Failure of a professional person, a physician or lawyer, to render proper services through reprehensible ignorance or negligence or through criminal intent, especially when injury or loss follows. (Random House Unabridged Dictionary, 2d ed)'),
  'D008319': NtO(MH='Malta', MS='An independent state consisting of three islands: Malta, Comino and Gozo, located in the Mediterranean Sea, south of Sicily. Its capital is Valetta.'),
  'D008320': NtO(MH='Maltose', MS="A dextrodisaccharide from malt and starch. It is used as a sweetening agent and fermentable intermediate in brewing. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D008321': NtO(MH='Mammary Glands, Animal', MS='MAMMARY GLANDS in the non-human MAMMALS.'),
  'D008322': NtO(MH='Mammals', MS='Warm-blooded vertebrate animals belonging to the class Mammalia, including all that possess hair and suckle their young.'),
  'D008323': NtO(MH='Mammary Arteries', MS='Arteries originating from the subclavian or axillary arteries and distributing to the anterior thoracic wall, mediastinal structures, diaphragm, pectoral muscles and mammary gland.'),
  'D008324': NtO(MH='Mammary Tumor Virus, Mouse', MS='The type species of BETARETROVIRUS commonly latent in mice. It causes mammary adenocarcinoma in a genetically susceptible strain of mice when the appropriate hormonal influences operate.'),
  'D008325': NtO(MH='Mammary Neoplasms, Experimental', MS='Experimentally induced mammary neoplasms in animals to provide a model for studying human BREAST NEOPLASMS.'),
  'D008326': NtO(MH='Mammillary Bodies', MS='A pair of nuclei and associated GRAY MATTER in the interpeduncular space rostral to the posterior perforated substance in the POSTERIOR HYPOTHALAMUS.'),
  'D008327': NtO(MH='Mammography', MS='Radiographic examination of the breast.'),
  'D008328': NtO(MH='Man-Machine Systems', MS='A system in which the functions of the man and the machine are interrelated and necessary for the operation of the system.'),
  'D008329': NtO(MH='Managed Care Programs', MS='Health insurance plans intended to reduce unnecessary health care costs through a variety of mechanisms, including: economic incentives for physicians and patients to select less costly forms of care; programs for reviewing the medical necessity of specific services; increased beneficiary cost sharing; controls on inpatient admissions and lengths of stay; the establishment of cost-sharing incentives for outpatient surgery; selective contracting with health care providers; and the intensive management of high-cost health care cases. The programs may be provided in a variety of settings, such as HEALTH MAINTENANCE ORGANIZATIONS and PREFERRED PROVIDER ORGANIZATIONS.'),
  'D008330': NtO(MH='Management Audit', MS='Management review designed to evaluate efficiency and to identify areas in need of management improvement within the institution in order to ensure effectiveness in meeting organizational goals.'),
  'D008331': NtO(MH='Management Information Systems', MS='Systems designed to provide information primarily concerned with the administrative functions associated with the provision and utilization of services; also includes program planning, etc.'),
  'D008332': NtO(MH='Management Quality Circles', MS='Participation of employees with management as a labor-management team, in decisions pertaining to the operational activities of the organization or industry.'),
  'D008333': NtO(MH='Mandelic Acids', MS='Analogs or derivatives of mandelic acid (alpha-hydroxybenzeneacetic acid).'),
  'D008334': NtO(MH='Mandible', MS='The largest and strongest bone of the FACE constituting the lower jaw. It supports the lower teeth.'),
  'D008335': NtO(MH='Mandibular Condyle', MS='The posterior process on the ramus of the mandible composed of two parts: a superior part, the articular portion, and an inferior part, the condylar neck.'),
  'D008336': NtO(MH='Mandibular Diseases', MS='Diseases involving the MANDIBLE.'),
  'D008337': NtO(MH='Mandibular Fractures', MS='Fractures of the lower jaw.'),
  'D008338': NtO(MH='Mandibular Injuries', MS='Injuries to the lower jaw bone.'),
  'D008339': NtO(MH='Mandibular Neoplasms', MS='Tumors or cancer of the MANDIBLE.'),
  'D008340': NtO(MH='Mandibular Nerve', MS='A branch of the trigeminal (5th cranial) nerve. The mandibular nerve carries motor fibers to the muscles of mastication and sensory fibers to the teeth and gingivae, the face in the region of the mandible, and parts of the dura.'),
  'D008341': NtO(MH='Mandibular Prosthesis', MS='A prosthetic appliance for the replacement of areas of the mandible missing or defective as a result of deformity, disease, injury, or surgery.'),
  'D008342': NtO(MH='Mandibulofacial Dysostosis', MS="A hereditary disorder occurring in two forms: the complete form (Franceschetti's syndrome) is characterized by antimongoloid slant of the palpebral fissures, COLOBOMA of the lower lid, MICROGNATHIA and hypoplasia of the ZYGOMATIC ARCHES, and CONGENITAL MICROTIA. It is transmitted as an autosomal trait. The incomplete form (Treacher Collins syndrome) is characterized by the same anomalies in less pronounced degree. It occurs sporadically, but an autosomal dominant mode of transmission is suspected. (Dorland, 27th ed)"),
  'D008344': NtO(MH='Maneb', MS='Manganese derivative of ethylenebisdithiocarbamate. It is used in agriculture as a fungicide and has been shown to cause irritation to the eyes, nose, skin, and throat.'),
  'D008345': NtO(MH='Manganese', MS='A trace element with atomic symbol Mn, atomic number 25, and atomic weight 54.94. It is concentrated in cell mitochondria, mostly in the pituitary gland, liver, pancreas, kidney, and bone, influences the synthesis of mucopolysaccharides, stimulates hepatic synthesis of cholesterol and fatty acids, and is a cofactor in many enzymes, including arginase and alkaline phosphatase in the liver. (From AMA Drug Evaluations Annual 1992, p2035)'),
  'D008347': NtO(MH='Manifest Anxiety Scale', MS='True-false questionnaire made up of items believed to indicate anxiety, in which the subject answers verbally the statement that describes him.'),
  'D008348': NtO(MH='Manikins', MS='Inanimate physical models which bear likeness to human beings and are used for various purposes in place of humans.'),
  'D008349': NtO(MH='Manipulation, Orthopedic', MS='The planned and carefully managed manual movement of the musculoskeletal system, extremities, and spine to produce increased motion. The term is sometimes used to denote a precise sequence of movements of a joint to determine the presence of disease or to reduce a dislocation. In the case of fractures, orthopedic manipulation can produce better position and alignment of the fracture. (From Blauvelt & Nelson, A Manual of Orthopaedic Terminology, 5th ed, p264)'),
  'D008350': NtO(MH='Manitoba', MS="A province of Canada, lying between the provinces of Saskatchewan and Ontario. Its capital is Winnipeg. Taking its name from Lake Manitoba, itself named for one of its islands, the name derived from Algonquian Manitou, great spirit. (From Webster's New Geographical Dictionary, 1988, p724 & Room, Brewer's Dictionary of Names, 1992, p332)"),
  'D008351': NtO(MH='Mannans', MS='Polysaccharides consisting of mannose units.'),
  'D008352': NtO(MH='Mannich Bases', MS='Ketonic amines prepared from the condensation of a ketone with formaldehyde and ammonia or a primary or secondary amine. A Mannich base can act as the equivalent of an alpha,beta unsaturated ketone in synthesis or can be reduced to form physiologically active amino alcohols.'),
  'D008353': NtO(MH='Mannitol', MS='A diuretic and renal diagnostic aid related to sorbitol. It has little significant energy value as it is largely eliminated from the body before any metabolism can take place. It can be used to treat oliguria associated with kidney failure or other manifestations of inadequate renal function and has been used for determination of glomerular filtration rate. Mannitol is also commonly used as a research tool in cell biological studies, usually to control osmolarity.'),
  'D008354': NtO(MH='Mannitol Dehydrogenases', MS='Sugar alcohol dehydrogenases that have specificity for MANNITOL. Enzymes in this category are generally classified according to their preference for a specific reducing cofactor.'),
  'D008355': NtO(MH='Mannitol Phosphates', MS='Phosphoric acid esters of mannitol.'),
  'D008356': NtO(MH='Mannoheptulose', MS='A 7-carbon keto sugar having the mannose configuration.'),
  'D008357': NtO(MH='Mannomustine', MS='Nitrogen mustard derivative alkylating agent used as antineoplastic. It causes severe bone marrow depression and is a powerful vesicant.'),
  'D008358': NtO(MH='Mannose', MS="A hexose or fermentable monosaccharide and isomer of glucose from manna, the ash Fraxinus ornus and related plants. (From Grant & Hackh's Chemical Dictionary, 5th ed & Random House Unabridged Dictionary, 2d ed)"),
  'D008359': NtO(MH='Mannose-6-Phosphate Isomerase', MS='An enzyme that catalyzes the reversible isomerization of D-mannose-6-phosphate to form D-fructose-6-phosphate, an important step in glycolysis. EC 5.3.1.8.'),
  'D008360': NtO(MH='Mannosephosphates', MS='Phosphoric acid esters of mannose.'),
  'D008361': NtO(MH='Mannosidases', MS='Glycoside hydrolases that catalyze the hydrolysis of alpha or beta linked MANNOSE.'),
  'D008362': NtO(MH='Mannosides', MS='Glycosides formed by the reaction of the hydroxyl group on the anomeric carbon atom of mannose with an alcohol to form an acetal. They include both alpha- and beta-mannosides.'),
  'D008363': NtO(MH='alpha-Mannosidosis', MS='An inborn error of metabolism marked by a defect in the lysosomal isoform of ALPHA-MANNOSIDASE activity that results in lysosomal accumulation of mannose-rich intermediate metabolites. Virtually all patients have psychomotor retardation, facial coarsening, and some degree of dysostosis multiplex. It is thought to be an autosomal recessive disorder.'),
  'D008364': NtO(MH='Mannosyltransferases', MS='Enzymes that catalyze the transfer of mannose from a nucleoside diphosphate mannose to an acceptor molecule which is frequently another carbohydrate. The group includes EC 2.4.1.32, EC 2.4.1.48, EC 2.4.1.54, and EC 2.4.1.57.'),
  'D008365': NtO(MH='Manometry', MS='Measurement of the pressure or tension of liquids or gases with a manometer.'),
  'D008366': NtO(MH='Mansonella', MS='A genus of parasitic nematodes whose organisms are distributed in Central and South America. Characteristics include a smooth cuticle and an enlarged anterior end.'),
  'D008368': NtO(MH='Mansonelliasis', MS='Infection with nematodes of the genus MANSONELLA. Symptoms include pruritus, headache, and articular swelling.'),
  'D008369': NtO(MH='Manual Communication', MS='Method of nonverbal communication utilizing hand movements as speech equivalents.'),
  'D008370': NtO(MH='Manuals as Topic', MS='Works about presentation designed to give factual information or instructions.'),
  'D008371': NtO(MH='Manubrium', MS='The upper or most anterior segment of the STERNUM which articulates with the CLAVICLE and first two pairs of RIBS.'),
  'D008372': NtO(MH='Manure', MS="Accumulations of solid or liquid animal excreta usually from stables and barnyards with or without litter material. Its chief application is as a fertilizer. (From Webster's 3d ed)"),
  'D008373': NtO(MH='Manuscripts as Topic', MS='Works about compositions written by hand, as one written before the invention or adoption of printing.'),
  'D008374': NtO(MH='Manuscripts, Medical as Topic', MS='Works about unpublished medical works.'),
  'D008375': NtO(MH='Maple Syrup Urine Disease', MS='An autosomal recessive inherited disorder with multiple forms of phenotypic expression, caused by a defect in the oxidative decarboxylation of branched-chain amino acids (AMINO ACIDS, BRANCHED-CHAIN). These metabolites accumulate in body fluids and render a "maple syrup" odor. The disease is divided into classic, intermediate, intermittent, and thiamine responsive subtypes. The classic form presents in the first week of life with ketoacidosis, hypoglycemia, emesis, neonatal seizures, and hypertonia. The intermediate and intermittent forms present in childhood or later with acute episodes of ataxia and vomiting. (From Adams et al., Principles of Neurology, 6th ed, p936)'),
  'D008376': NtO(MH='Maprotiline', MS='A bridged-ring tetracyclic antidepressant that is both mechanistically and functionally similar to the tricyclic antidepressants, including side effects associated with its use.'),
  'D008377': NtO(MH='Maps as Topic', MS='Works about representations, normally to scale and on a flat medium, of a selection of material or abstract features on the surface of the earth, the heavens, or celestial bodies.'),
  'D008379': NtO(MH='Marburg Virus Disease', MS='An RNA virus infection of rhesus, vervet, and squirrel monkeys transmissible to man.'),
  'D008380': NtO(MH='Marek Disease', MS='A transmissible viral disease of birds caused by avian herpesvirus 2 (HERPESVIRUS 2, GALLID) and other MARDIVIRUS. There is lymphoid cell infiltration or lymphomatous tumor formation in the peripheral nerves and gonads, but may also involve visceral organs, skin, muscle, and the eye.'),
  'D008381': NtO(MH='Herpesvirus 2, Gallid', MS='The type species of the genus MARDIVIRUS in the family HERPESVIRIDAE. It is the etiologic agent of MAREK DISEASE, infecting domestic fowl and wild birds.'),
  'D008382': NtO(MH='Marfan Syndrome', MS='An autosomal dominant disorder of CONNECTIVE TISSUE with abnormal features in the heart, the eye, and the skeleton. Cardiovascular manifestations include MITRAL VALVE PROLAPSE, dilation of the AORTA, and aortic dissection. Other features include lens displacement (ectopia lentis), disproportioned long limbs and enlarged DURA MATER (dural ectasia). Marfan syndrome is associated with mutations in the gene encoding fibrillin, a major element of extracellular microfibrils of connective tissue.'),
  'D008383': NtO(MH='Margarine', MS='A butterlike product made of refined vegetable oils, sometimes blended with animal fats, and emulsified usually with water or milk. It is used as a butter substitute. (From Random House Unabridged Dictionary, 2d ed)'),
  'D008385': NtO(MH='Marijuana Smoking', MS='Inhaling and exhaling SMOKE from CANNABIS.'),
  'D008386': NtO(MH='Marine Biology', MS='The study of the origin, structure, development, growth, function, genetics, and reproduction of organisms which inhabit the OCEANS AND SEAS.'),
  'D008387': NtO(MH='Marine Toxins', MS='Toxic or poisonous substances elaborated by marine flora or fauna. They include also specific, characterized poisons or toxins for which there is no more specific heading, like those from poisonous FISHES.'),
  'D008388': NtO(MH='Marital Therapy', MS='A form of psychotherapy involving both spouses and directed to improving the marital relationship.'),
  'D008389': NtO(MH='Marketing of Health Services', MS='Application of marketing principles and techniques to maximize the use of health care resources.'),
  'D008390': NtO(MH='Markov Chains', MS='A stochastic process such that the conditional probability distribution for a state at any future instant, given the present state, is unaffected by any additional knowledge of the past history of the system.'),
  'D008392': NtO(MH='Marmota', MS='A genus of Sciuridae consisting of 14 species. They are shortlegged, burrowing rodents which hibernate in winter.'),
  'D008393': NtO(MH='Marriage', MS='The social institution involving legal and/or religious sanction whereby individuals are joined together.'),
  'D008394': NtO(MH='Marsupialia', MS='An infraclass of MAMMALS, also called Metatheria, where the young are born at an early stage of development and continue to develop in a pouch (marsupium). In contrast to Eutheria (placentals), marsupials have an incomplete PLACENTA.'),
  'D008395': NtO(MH='Martinique', MS='An island in the Lesser Antilles, one of the Windward Islands. Its capital is Fort-de-France. It was discovered by Columbus in 1502 and from its settlement in 1635 by the French it passed into and out of Dutch and British hands. It was made a French overseas department in 1946. One account of the name tells of native women on the shore calling "Madinina" as Columbus approached the island. The meaning was never discovered but was entered on early charts as Martinique, influenced by the name of St. Martin. (From Webster\'s New Geographical Dictionary, 1988, p734 & Room, Brewer\'s Dictionary of Names, 1992, p339)'),
  'D008396': NtO(MH='Maryland', MS='A state bounded on the north by Pennsylvania, on the east by Delaware and the Atlantic Ocean, on the south by Virginia and West Virginia, and on the west by West Virginia.'),
  'D008397': NtO(MH='Masks', MS='Devices that cover the nose and mouth to maintain aseptic conditions or to administer inhaled anesthetics or other gases. (UMDNS, 1999)'),
  'D008398': NtO(MH='Masochism', MS='Pleasure derived from being physically or psychologically abused, whether inflicted by oneself or by others. Masochism includes sexual masochism.'),
  'D008399': NtO(MH='Mass Behavior', MS='Collective behavior of an aggregate of individuals giving the appearance of unity of attitude, feeling, and motivation.'),
  'D008400': NtO(MH='Mass Chest X-Ray', MS='X-ray screening of large groups of persons for diseases of the lung and heart by means of radiography of the chest.'),
  'D008401': NtO(MH='Gas Chromatography-Mass Spectrometry', MS='A microanalytical technique combining mass spectrometry and gas chromatography for the qualitative as well as quantitative determinations of compounds.'),
  'D008402': NtO(MH='Mass Media', MS='Instruments or technological means of communication that reach large numbers of people with a common message: press, radio, television, etc.'),
  'D008403': NtO(MH='Mass Screening', MS='Organized periodic procedures performed on large groups of people for the purpose of detecting disease.'),
  'D008404': NtO(MH='Massachusetts', MS='State bounded on the north by New Hampshire and Vermont, on the east by the Atlantic Ocean, on the south by Connecticut and Rhode Island, and on the west by New York.'),
  'D008405': NtO(MH='Massage', MS='The systematic and methodical manipulations of body tissues best performed with the hands for the purpose of affecting the nervous and muscular systems and the general circulation.'),
  'D008406': NtO(MH='Masseter Muscle', MS='A masticatory muscle whose action is closing the jaws.'),
  'D008407': NtO(MH='Mast Cells', MS='Granulated cells that are found in almost all tissues, most abundantly in the skin and the gastrointestinal tract. Like the BASOPHILS, mast cells contain large amounts of HISTAMINE and HEPARIN. Unlike basophils, mast cells normally remain in the tissues and do not circulate in the blood. Mast cells, derived from the bone marrow stem cells, are regulated by the STEM CELL FACTOR.'),
  'D008408': NtO(MH='Mastectomy', MS='Surgical procedure to remove one or both breasts.'),
  'D008409': NtO(MH='Mastication', MS='The act and process of chewing and grinding food in the mouth.'),
  'D008410': NtO(MH='Masticatory Muscles', MS='Muscles arising in the zygomatic arch that close the jaw. Their nerve supply is masseteric from the mandibular division of the trigeminal nerve. (From Stedman, 25th ed)'),
  'D008411': NtO(MH='Chytridiomycota', MS='A phylum of fungi that was formerly considered a subdivision of Phycomycetes. They are the only fungi that produce motile spores (zoospores) at some stage in their life cycle. Most are saprobes but they also include examples of plant, animal, and fungal pathogens.'),
  'D008413': NtO(MH='Mastitis', MS='INFLAMMATION of the BREAST, or MAMMARY GLAND.'),
  'D008414': NtO(MH='Mastitis, Bovine', MS='INFLAMMATION of the UDDER in cows.'),
  'D008415': NtO(MH='Mastocytosis', MS='A heterogenous group of disorders characterized by the abnormal increase of MAST CELLS in only the skin (MASTOCYTOSIS, CUTANEOUS), in extracutaneous tissues involving multiple organs (MASTOCYTOSIS, SYSTEMIC), or in solid tumors (MASTOCYTOMA).'),
  'D008416': NtO(MH='Mastoid', MS='The posterior part of the temporal bone. It is a projection of the petrous bone.'),
  'D008417': NtO(MH='Mastoiditis', MS='Inflammation of the honeycomb-like MASTOID BONE in the skull just behind the ear. It is usually a complication of OTITIS MEDIA.'),
  'D008418': NtO(MH='Masturbation', MS='Sexual stimulation or gratification of the self.'),
  'D008419': NtO(MH='Materia Medica', MS='Materials or substances used in the composition of traditional medical remedies. The use of this term in MeSH was formerly restricted to historical articles or those concerned with traditional medicine, but it can also refer to homeopathic remedies. Nosodes are specific types of homeopathic remedies prepared from causal agents or disease products.'),
  'D008420': NtO(MH='Manufactured Materials', MS='Substances and materials manufactured for use in various technologies and industries and for domestic use.'),
  'D008421': NtO(MH='Materials Management, Hospital', MS='The management of all procurement, distribution, and storage of equipment and supplies, as well as logistics management including laundry, processing of reusables, etc.'),
  'D008422': NtO(MH='Materials Testing', MS='The testing of materials and devices, especially those used for PROSTHESES AND IMPLANTS; SUTURES; TISSUE ADHESIVES; etc., for hardness, strength, durability, safety, efficacy, and biocompatibility.'),
  'D008423': NtO(MH='Maternal Age', MS='The age of the mother in PREGNANCY.'),
  'D008425': NtO(MH='Maternal Behavior', MS='The behavior patterns associated with or characteristic of a mother.'),
  'D008426': NtO(MH='Maternal Deprivation', MS='Prolonged separation of the offspring from the mother.'),
  'D008427': NtO(MH='Maternal Health Services', MS='Organized services to provide health care to expectant and nursing mothers.'),
  'D008428': NtO(MH='Maternal Mortality', MS='Maternal deaths resulting from complications of pregnancy and childbirth in a given population.'),
  'D008429': NtO(MH='Maternal Welfare', MS='Organized efforts by communities or organizations to improve the health and well-being of the mother.'),
  'D008430': NtO(MH='Maternal-Child Health Centers', MS='Facilities which administer the delivery of health care services to mothers and children.'),
  'D008431': NtO(MH='Maternal-Fetal Exchange', MS='Exchange of substances between the maternal blood and the fetal blood at the PLACENTA via PLACENTAL CIRCULATION. The placental barrier excludes microbial or viral transmission.'),
  'D008432': NtO(MH='Mathematical Computing', MS='Computer-assisted interpretation and analysis of various mathematical functions related to a particular problem.'),
  'D008433': NtO(MH='Mathematics', MS='The deductive study of shape, quantity, and dependence. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D008434': NtO(MH='Matrix Bands', MS='Devices which provide an artificial temporary wall, or matrix, used in filling a prepared cavity.'),
  'D008435': NtO(MH='Mauritania', MS='A republic in western Africa, southwest of ALGERIA and west of MALI. Its capital is Nouakchott.'),
  'D008436': NtO(MH='Mauritius', MS="One of the Indian Ocean Islands, east of Madagascar. Its capital is Port Louis. It was discovered by the Portuguese in 1505, occupied by the Dutch 1598-1710, held by the French 1715-1810 when the British captured it, formally ceded to the British in 1814, and became independent in 1968. It was named by the Dutch in honor of Maurice of Nassau, Prince of Orange (1567-1625). (From Webster's New Geographical Dictionary, 1988, p742 & Room, Brewer's Dictionary of Names, 1992, p341)"),
  'D008437': NtO(MH='Maxilla', MS='One of a pair of irregularly shaped bones that form the upper jaw. A maxillary bone provides tooth sockets for the superior teeth, forms part of the ORBIT, and contains the MAXILLARY SINUS.'),
  'D008438': NtO(MH='Maxillary Artery', MS='A branch of the external carotid artery which distributes to the deep structures of the face (internal maxillary) and to the side of the face and nose (external maxillary).'),
  'D008439': NtO(MH='Maxillary Diseases', MS='Diseases involving the MAXILLA.'),
  'D008440': NtO(MH='Maxillary Fractures', MS='Fractures of the upper jaw.'),
  'D008441': NtO(MH='Maxillary Neoplasms', MS='Cancer or tumors of the MAXILLA or upper jaw.'),
  'D008442': NtO(MH='Maxillary Nerve', MS='The intermediate sensory division of the trigeminal (5th cranial) nerve. The maxillary nerve carries general afferents from the intermediate region of the face including the lower eyelid, nose and upper lip, the maxillary teeth, and parts of the dura.'),
  'D008443': NtO(MH='Maxillary Sinus', MS='The air space located in the body of the MAXILLARY BONE near each cheek. Each maxillary sinus communicates with the middle passage (meatus) of the NASAL CAVITY on the same side.'),
  'D008444': NtO(MH='Maxillary Sinus Neoplasms', MS='Tumors or cancer of the MAXILLARY SINUS. They represent the majority of paranasal neoplasms.'),
  'D008445': NtO(MH='Maxillofacial Development', MS='The process of growth and differentiation of the jaws and face.'),
  'D008446': NtO(MH='Maxillofacial Injuries', MS='General or unspecified injuries involving the face and jaw (either upper, lower, or both).'),
  'D008447': NtO(MH='Maxillofacial Prosthesis', MS='A prosthetic appliance for the replacement of areas of the maxilla, mandible, and face, missing as a result of deformity, disease, injury, or surgery. When the prosthesis replaces portions of the mandible only, it is referred to as MANDIBULAR PROSTHESIS.'),
  'D008448': NtO(MH='Maximal Expiratory Flow Rate', MS='The airflow rate measured during the first liter expired after the first 200 ml have been exhausted during a FORCED VITAL CAPACITY determination. Common abbreviations are MEFR, FEF 200-1200, and FEF 0.2-1.2.'),
  'D008449': NtO(MH='Maximal Expiratory Flow-Volume Curves', MS='Curves depicting MAXIMAL EXPIRATORY FLOW RATE, in liters/second, versus lung inflation, in liters or percentage of lung capacity, during a FORCED VITAL CAPACITY determination. Common abbreviation is MEFV.'),
  'D008450': NtO(MH='Maximal Midexpiratory Flow Rate', MS='Measurement of rate of airflow over the middle half of a FORCED VITAL CAPACITY determination (from the 25 percent level to the 75 percent level). Common abbreviations are MMFR and FEF 25%-75%.'),
  'D008451': NtO(MH='Maximal Voluntary Ventilation', MS='Measure of the maximum amount of air that can be breathed in and blown out over a sustained interval such as 15 or 20 seconds. Common abbreviations are MVV and MBC.'),
  'D008452': NtO(MH='Maximum Allowable Concentration', MS='The maximum exposure to a biologically active physical or chemical agent that is allowed during an 8-hour period (a workday) in a population of workers, or during a 24-hour period in the general population, which does not appear to cause appreciable harm, whether immediate or delayed for any period, in the target population. (From Lewis Dictionary of Toxicology, 1st ed)'),
  'D008453': NtO(MH='Maytansine', MS='An ansa macrolide isolated from the MAYTENUS genus of East African shrubs.'),
  'D008454': NtO(MH='Mazindol', MS='Tricyclic anorexigenic agent unrelated to and less toxic than AMPHETAMINE, but with some similar side effects. It inhibits uptake of catecholamines and blocks the binding of cocaine to the dopamine uptake transporter.'),
  'D008455': NtO(MH='(4-(m-Chlorophenylcarbamoyloxy)-2-butynyl)trimethylammonium Chloride', MS='A drug that selectively activates certain subclasses of muscarinic receptors and also activates postganglionic nicotinic receptors. It is commonly used experimentally to distinguish muscarinic receptor subtypes.'),
  'D008456': NtO(MH='2-Methyl-4-chlorophenoxyacetic Acid', MS='A powerful herbicide used as a selective weed killer.'),
  'D008457': NtO(MH='Measles', MS='A highly contagious infectious disease caused by MORBILLIVIRUS, common among children but also seen in the nonimmune of any age, in which the virus enters the respiratory tract via droplet nuclei and multiplies in the epithelial cells, spreading throughout the MONONUCLEAR PHAGOCYTE SYSTEM.'),
  'D008458': NtO(MH='Measles Vaccine', MS='A live attenuated virus vaccine of chick embryo origin, used for routine immunization of children and for immunization of adolescents and adults who have not had measles or been immunized with live measles vaccine and have no serum antibodies against measles. Children are usually immunized with measles-mumps-rubella combination vaccine. (From Dorland, 28th ed)'),
  'D008459': NtO(MH='Measles virus', MS='The type species of MORBILLIVIRUS and the cause of the highly infectious human disease MEASLES, which affects mostly children.'),
  'D008460': NtO(MH='Meat', MS='The edible portions of any animal used for food including domestic mammals (the major ones being cattle, swine, and sheep) along with poultry, fish, shellfish, and game.'),
  'D008461': NtO(MH='Meat Products', MS='Articles of food which are derived by a process of manufacture from any portion of carcasses of any animal used for food (e.g., head cheese, sausage, scrapple).'),
  'D008462': NtO(MH='Meat-Packing Industry', MS='The aggregate enterprise of technically producing packaged meat.'),
  'D008463': NtO(MH='Mebendazole', MS='A benzimidazole that acts by interfering with CARBOHYDRATE METABOLISM and inhibiting polymerization of MICROTUBULES.'),
  'D008464': NtO(MH='Mecamylamine', MS='A nicotinic antagonist that is well absorbed from the gastrointestinal tract and crosses the blood-brain barrier. Mecamylamine has been used as a ganglionic blocker in treating hypertension, but, like most ganglionic blockers, is more often used now as a research tool.'),
  'D008465': NtO(MH='Mechanoreceptors', MS='Cells specialized to transduce mechanical stimuli and relay that information centrally in the nervous system. Mechanoreceptor cells include the INNER EAR hair cells, which mediate hearing and balance, and the various somatosensory receptors, often with non-neural accessory structures.'),
  'D008466': NtO(MH='Mechlorethamine', MS='A biologic alkylating agent that exerts its cytotoxic effects by forming DNA ADDUCTS and DNA interstrand crosslinks, thereby inhibiting rapidly proliferating cells. The hydrochloride is an antineoplastic agent used to treat HODGKIN DISEASE and LYMPHOMA.'),
  'D008467': NtO(MH='Meckel Diverticulum', MS='A congenital abnormality characterized by the outpouching or sac formation in the ILEUM. It is a remnant of the embryonic YOLK SAC in which the VITELLINE DUCT failed to close.'),
  'D008468': NtO(MH='Meclizine', MS='A histamine H1 antagonist used in the treatment of motion sickness, vertigo, and nausea during pregnancy and radiation sickness.'),
  'D008469': NtO(MH='Meclofenamic Acid', MS='A non-steroidal anti-inflammatory agent with antipyretic and antigranulation activities. It also inhibits prostaglandin biosynthesis.'),
  'D008470': NtO(MH='Meconium', MS='The thick green-to-black mucilaginous material found in the intestines of a full-term fetus. It consists of secretions of the INTESTINAL GLANDS; BILE PIGMENTS; FATTY ACIDS; AMNIOTIC FLUID; and intrauterine debris. It constitutes the first stools passed by a newborn.'),
  'D008471': NtO(MH='Meconium Aspiration Syndrome', MS='A condition caused by inhalation of MECONIUM into the LUNG of FETUS or NEWBORN, usually due to vigorous respiratory movements during difficult PARTURITION or respiratory system abnormalities. Meconium aspirate may block small airways leading to difficulties in PULMONARY GAS EXCHANGE and ASPIRATION PNEUMONIA.'),
  'D008472': NtO(MH='Medazepam', MS='A benzodiazepine derivative used in the treatment of anxiety. It has sedative, muscle relaxant, and anticonvulsant properties. One of its metabolites is DIAZEPAM and one of its excretion products is OXAZEPAM.'),
  'D008473': NtO(MH='Median Eminence', MS='Raised area at the infundibular region of the HYPOTHALAMUS at the floor of the BRAIN, ventral to the THIRD VENTRICLE and adjacent to the ARCUATE NUCLEUS OF HYPOTHALAMUS. It contains the terminals of hypothalamic neurons and the capillary network of hypophyseal portal system, thus serving as a neuroendocrine link between the brain and the PITUITARY GLAND.'),
  'D008474': NtO(MH='Medial Forebrain Bundle', MS='A complex group of fibers arising from the basal olfactory regions, the periamygdaloid region, and the septal nuclei, and passing to the lateral hypothalamus. Some fibers continue into the tegmentum.'),
  'D008475': NtO(MH='Median Nerve', MS='A major nerve of the upper extremity. In humans, the fibers of the median nerve originate in the lower cervical and upper thoracic spinal cord (usually C6 to T1), travel via the brachial plexus, and supply sensory and motor innervation to parts of the forearm and hand.'),
  'D008476': NtO(MH='Mediastinal Cyst', MS='Cysts of one of the parts of the mediastinum: the superior part, containing the trachea, esophagus, thoracic duct and thymus organs; the inferior middle part, containing the pericardium; the inferior anterior part containing some lymph nodes; and the inferior posterior part, containing the thoracic duct and esophagus.'),
  'D008477': NtO(MH='Mediastinal Diseases', MS='Disorders of the mediastinum, general or unspecified.'),
  'D008478': NtO(MH='Mediastinal Emphysema', MS='Presence of air in the mediastinal tissues due to leakage of air from the tracheobronchial tree, usually as a result of trauma.'),
  'D008479': NtO(MH='Mediastinal Neoplasms', MS='Tumors or cancer of the MEDIASTINUM.'),
  'D008480': NtO(MH='Mediastinitis', MS='Inflammation of the mediastinum, the area between the pleural sacs.'),
  'D008481': NtO(MH='Mediastinoscopy', MS='Endoscopic examination, therapy or surgery of the anterior superior mediastinum of the thorax.'),
  'D008482': NtO(MH='Mediastinum', MS='A membrane in the midline of the THORAX of mammals. It separates the lungs between the STERNUM in front and the VERTEBRAL COLUMN behind. It also surrounds the HEART, TRACHEA, ESOPHAGUS, THYMUS, and LYMPH NODES.'),
  'D008483': NtO(MH='Medical Assistance', MS='Financing of medical care provided to public assistance recipients.'),
  'D008484': NtO(MH='Medicaid', MS='Federal program, created by Public Law 89-97, Title XIX, a 1965 amendment to the Social Security Act, administered by the states, that provides health care benefits to indigent and medically indigent persons.'),
  'D008485': NtO(MH='Medical Audit', MS='A detailed review and evaluation of selected clinical records by qualified professional personnel for evaluating quality of medical care.'),
  'D008486': NtO(MH='Physician Executives', MS='Physicians who serve in a medical and administrative capacity as head of an organized medical staff and who also may serve as liaison for the medical staff with the administration and governing board.'),
  'D008487': NtO(MH='Medical History Taking', MS='Acquiring information from a patient on past medical conditions and treatments.'),
  'D008488': NtO(MH='Medical Illustration', MS='The field which deals with illustrative clarification of biomedical concepts, as in the use of diagrams and drawings. The illustration may be produced by hand, photography, computer, or other electronic or mechanical methods.'),
  'D008489': NtO(MH='Medical Indigency', MS='The condition in which individuals are financially unable to access adequate medical care without depriving themselves and their dependents of food, clothing, shelter, and other essentials of living.'),
  'D008490': NtO(MH='Medical Informatics', MS='The field of information science concerned with the analysis and dissemination of medical data through the application of computers to various aspects of health care and medicine.'),
  'D008491': NtO(MH='Medical Informatics Applications', MS='Automated systems applied to the patient care process including diagnosis, therapy, and systems of communicating medical data within the health care setting.'),
  'D008492': NtO(MH='Medical Informatics Computing', MS='Precise procedural mathematical and logical operations utilized in the study of medical information pertaining to health care.'),
  'D008493': NtO(MH='Medical Missions', MS='Travel by a group for the purpose of undertaking a health-care related project of short-term duration.'),
  'D008494': NtO(MH='Medical Office Buildings', MS='Office and laboratory facilities constructed for the use of physicians and other health personnel.'),
  'D008495': NtO(MH='Medical Oncology', MS='A subspecialty of internal medicine concerned with the study of neoplasms.'),
  'D008496': NtO(MH='Medical Receptionists', MS='Individuals who receive patients in a medical office.'),
  'D008497': NtO(MH='Medical Record Administrators', MS="Individuals professionally qualified in the management of patients' records. Duties may include planning, designing, and managing systems for patient administrative and clinical data, as well as patient medical records. The concept includes medical record technicians."),
  'D008498': NtO(MH='Medical Record Linkage', MS='The creation and maintenance of medical and vital records in multiple institutions in a manner that will facilitate the combined use of the records of identified individuals.'),
  'D008499': NtO(MH='Medical Records', MS="Recording of pertinent information concerning patient's illness or illnesses."),
  'D008500': NtO(MH='Medical Records Department, Hospital', MS='Hospital department responsible for the creating, care, storage and retrieval of medical records. It also provides statistical information for the medical and administrative staff.'),
  'D008501': NtO(MH='Medical Records, Problem-Oriented', MS="A system of record keeping in which a list of the patient's problems is made and all history, physical findings, laboratory data, etc. pertinent to each problem are placed under that heading."),
  'D008502': NtO(MH='Medical Secretaries', MS='Individuals responsible for various duties pertaining to the medical office routine.'),
  'D008503': NtO(MH='Medical Staff', MS='Professional medical personnel who provide care to patients in an organized facility, institution or agency.'),
  'D008504': NtO(MH='Medical Staff Privileges', MS="Those rights or activities which are specific to members of the institution's medical staff, including the right to admit private patients."),
  'D008505': NtO(MH='Medical Staff, Hospital', MS='Professional medical personnel approved to provide care to patients in a hospital.'),
  'D008506': NtO(MH='Medical Waste', MS='Blood, mucus, tissue removed at surgery or autopsy, soiled surgical dressings, and other materials requiring special disposal procedures.'),
  'D008507': NtO(MH='Medically Underserved Area', MS='A geographic location which has insufficient health resources (manpower and/or facilities) to meet the medical needs of the resident population.'),
  'D008508': NtO(MH='Medication Errors', MS='Errors in prescribing, dispensing, or administering medication with the result that the patient fails to receive the correct drug or the indicated proper drug dosage.'),
  'D008509': NtO(MH='Medication Systems', MS='Overall systems, traditional or automated, to provide medication to patients.'),
  'D008510': NtO(MH='Medication Systems, Hospital', MS="Overall systems, traditional or automated, to provide medication to patients in hospitals. Elements of the system are: handling the physician's order, transcription of the order by nurse and/or pharmacist, filling the medication order, transfer to the nursing unit, and administration to the patient."),
  'D008511': NtO(MH='Medicine', MS='The art and science of studying, performing research on, preventing, diagnosing, and treating disease, as well as the maintenance of health.'),
  'D008512': NtO(MH='Medicine in the Arts', MS='Depiction of medical issues or themes via the arts including visual, oral, or written forms of expression.'),
  'D008513': NtO(MH='Medicine in Literature', MS='Written or other literary works whose subject matter is medical or about the profession of medicine and related areas.'),
  'D008514': NtO(MH='Medicine, Arabic', MS='Traditional concepts of medicine and associated methods reflected in medicine in the ARAB WORLD.'),
  'D008515': NtO(MH='Medicine, Ayurvedic', MS='The traditional medicine which is based on customs, beliefs, and practices of the Hindu culture.'),
  'D008516': NtO(MH='Medicine, Chinese Traditional', MS='A system of traditional medicine which is based on the beliefs and practices of the Chinese culture.'),
  'D008517': NtO(MH='Phytotherapy', MS='Use of plants or herbs to treat diseases or to alleviate pain.'),
  'D008518': NtO(MH='Medicine, East Asian Traditional', MS='Medical practice or discipline that is based on the knowledge, cultures, and beliefs of the people in EAST ASIA.'),
  'D008519': NtO(MH='Medicine, Traditional', MS='Systems of medicine based on cultural beliefs and practices handed down from generation to generation. The concept includes mystical and magical rituals (SPIRITUAL THERAPIES); PHYTOTHERAPY; and other treatments which may not be explained by modern medicine.'),
  'D008520': NtO(MH='Medigoxin', MS='A semisynthetic digitalis glycoside with the general properties of DIGOXIN but more rapid onset of action. Its cardiotonic action is prolonged by its demethylation to DIGOXIN in the liver. It has been used in the treatment of congestive heart failure (HEART FAILURE).'),
  'D008521': NtO(MH='Mediterranean Islands', MS='Islands in the Mediterranean Sea. The chief islands are the Balearic Islands, Corsica, Crete, Cyprus, the Cyclades, Dodecanese and Ionian Islands, Malta, Sardinia and Sicily.'),
  'D008522': NtO(MH='Mediterranean Sea', MS='An inland sea enclosed by Europe on the west and north, by Asia on the east, and Africa on the south.'),
  'D008523': NtO(MH='MEDLARS', MS='A computerized biomedical bibliographic storage and retrieval system operated by the NATIONAL LIBRARY OF MEDICINE. MEDLARS stands for Medical Literature Analysis and Retrieval System, which was first introduced in 1964 and evolved into an online system in 1971 called MEDLINE (MEDLARS Online). As other online databases were developed, MEDLARS became the name of the entire NLM information system while MEDLINE became the name of the premier database. MEDLARS was used to produce the former printed Cumulated Index Medicus, and the printed monthly Index Medicus, until that publication ceased in December 2004.'),
  'D008524': NtO(MH='Medrogestone', MS='6,17-Dimethylpregna-4,6-diene-3,20-dione. A synthetic progestational hormone with actions similar to those of progesterone. It is used in the treatment of menstrual irregularities and has also been employed in the treatment of prostatic hypertrophy and endometrial carcinoma.'),
  'D008525': NtO(MH='Medroxyprogesterone', MS='A synthetic progestational hormone used in veterinary practice as an estrus regulator.'),
  'D008526': NtO(MH='Medulla Oblongata', MS='The lower portion of the BRAIN STEM. It is inferior to the PONS and anterior to the CEREBELLUM. Medulla oblongata serves as a relay station between the brain and the spinal cord, and contains centers for regulating respiratory, vasomotor, cardiac, and reflex activities.'),
  'D008527': NtO(MH='Medulloblastoma', MS='A malignant neoplasm that may be classified either as a glioma or as a primitive neuroectodermal tumor of childhood (see NEUROECTODERMAL TUMOR, PRIMITIVE). The tumor occurs most frequently in the first decade of life with the most typical location being the cerebellar vermis. Histologic features include a high degree of cellularity, frequent mitotic figures, and a tendency for the cells to organize into sheets or form rosettes. Medulloblastoma have a high propensity to spread throughout the craniospinal intradural axis. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, pp2060-1)'),
  'D008528': NtO(MH='Mefenamic Acid', MS='A non-steroidal anti-inflammatory agent with analgesic, anti-inflammatory, and antipyretic properties. It is an inhibitor of cyclooxygenase.'),
  'D008529': NtO(MH='Mefruside', MS='A benzene-sulfonamide-furan. It is used as a diuretic that affects the concentrating ability of the KIDNEY, increases SODIUM CHLORIDE excretion, but may not spare POTASSIUM. It inhibits CARBONIC ANHYDRASES and may increase the blood URIC ACID level.'),
  'D008530': NtO(MH='Megacins', MS='Bacteriocins elaborated by mutant strains of Bacillus megaterium. They are protein or protein-lipopolysaccharide complexes lethal to other strains of the same species.'),
  'D008531': NtO(MH='Megacolon', MS='Dilatation of the COLON, often to alarming dimensions. There are various types of megacolon including congenital megacolon in HIRSCHSPRUNG DISEASE, idiopathic megacolon in CONSTIPATION, and TOXIC MEGACOLON.'),
  'D008532': NtO(MH='Megacolon, Toxic', MS='An acute form of MEGACOLON, severe pathological dilatation of the COLON. It is associated with clinical conditions such as ULCERATIVE COLITIS; CROHN DISEASE; AMEBIC DYSENTERY; or CLOSTRIDIUM ENTEROCOLITIS.'),
  'D008533': NtO(MH='Megakaryocytes', MS='Very large BONE MARROW CELLS which release mature BLOOD PLATELETS.'),
  'D008534': NtO(MH='Megaloblasts', MS='Red blood cell precursors, corresponding to ERYTHROBLASTS, that are larger than normal, usually resulting from a FOLIC ACID DEFICIENCY or VITAMIN B 12 DEFICIENCY.'),
  'D008535': NtO(MH='Megestrol', MS='A progestational hormone used most commonly as the acetate ester. As the acetate, it is more potent than progesterone both as a progestagen and as an ovulation inhibitor. It has also been used in the palliative treatment of breast cancer.'),
  'D008536': NtO(MH='Meglumine', MS='1-Deoxy-1-(methylamino)-D-glucitol. A derivative of sorbitol in which the hydroxyl group in position 1 is replaced by a methylamino group. Often used in conjunction with iodinated organic compounds as contrast medium.'),
  'D008537': NtO(MH='Meibomian Glands', MS='The sebaceous glands situated on the inner surface of the eyelids between the tarsal plates and CONJUNCTIVA.'),
  'D008538': NtO(MH='Meige Syndrome', MS='A syndrome characterized by orofacial DYSTONIA; including BLEPHAROSPASM; forceful jaw opening; lip retraction; platysma muscle spasm; and tongue protrusion. It primarily affects older adults, with an incidence peak in the seventh decade of life. (From Adams et al., Principles of Neurology, 6th ed, p108)'),
  'D008539': NtO(MH='Meigs Syndrome', MS='The triad of benign FIBROMA or other ovarian tumors with ASCITES, and HYDROTHORAX due to large PLEURAL EFFUSIONS.'),
  'D008540': NtO(MH='Meiosis', MS='A type of CELL NUCLEUS division, occurring during maturation of the GERM CELLS. Two successive cell nucleus divisions following a single chromosome duplication (S PHASE) result in daughter cells with half the number of CHROMOSOMES as the parent cells.'),
  'D008541': NtO(MH='Mekong Valley', MS='The geographic area of the Mekong Valley in general or when the specific country or countries are not indicated.'),
  'D008542': NtO(MH='Melanesia', MS="The collective name for the islands of the Pacific Ocean northeast of Australia, including NEW CALEDONIA; VANUATU; New Hebrides, Solomon Islands, Admiralty Islands, Bismarck Archipelago, FIJI, etc. Melanesia (from the Greek melas, black + nesos, island) is so called from the black color of the natives who are generally considered to be descended originally from the Negroid Papuans and the Polynesians or Malays. (From Webster's New Geographical Dictionary, 1988, p748 & Room, Brewer's Dictionary of Names, 1992, p344)"),
  'D008543': NtO(MH='Melanins', MS='Insoluble polymers of TYROSINE derivatives found in and causing darkness in skin (SKIN PIGMENTATION), hair, and feathers providing protection against SUNBURN induced by SUNLIGHT. CAROTENES contribute yellow and red coloration.'),
  'D008544': NtO(MH='Melanocytes', MS='Mammalian pigment cells that produce MELANINS, pigments found mainly in the EPIDERMIS, but also in the eyes and the hair, by a process called melanogenesis. Coloration can be altered by the number of melanocytes or the amount of pigment produced and stored in the organelles called MELANOSOMES. The large non-mammalian melanin-containing cells are called MELANOPHORES.'),
  'D008545': NtO(MH='Melanoma', MS='A malignant neoplasm derived from cells that are capable of forming melanin, which may occur in the skin of any part of the body, in the eye, or, rarely, in the mucous membranes of the genitalia, anus, oral cavity, or other sites. It occurs mostly in adults and may originate de novo or from a pigmented nevus or malignant lentigo. Melanomas frequently metastasize widely, and the regional lymph nodes, liver, lungs, and brain are likely to be involved. The incidence of malignant skin melanomas is rising rapidly in all parts of the world. (Stedman, 25th ed; from Rook et al., Textbook of Dermatology, 4th ed, p2445)'),
  'D008546': NtO(MH='Melanoma, Experimental', MS='Experimentally induced tumor that produces MELANIN in animals to provide a model for studying human MELANOMA.'),
  'D008547': NtO(MH='Melanophores', MS='Chromatophores (large pigment cells of fish, amphibia, reptiles and many invertebrates) which contain melanin. Short term color changes are brought about by an active redistribution of the melanophores pigment containing organelles (MELANOSOMES). Mammals do not have melanophores; however they have retained smaller pigment cells known as MELANOCYTES.'),
  'D008548': NtO(MH='Melanosis', MS='Disorders of increased melanin pigmentation that develop without preceding inflammatory disease.'),
  'D008549': NtO(MH='Melarsoprol', MS='Arsenical used in trypanosomiases. It may cause fatal encephalopathy and other undesirable side effects.'),
  'D008550': NtO(MH='Melatonin', MS='A biogenic amine that is found in animals and plants. In mammals, melatonin is produced by the PINEAL GLAND. Its secretion increases in darkness and decreases during exposure to light. Melatonin is implicated in the regulation of SLEEP, mood, and REPRODUCTION. Melatonin is also an effective antioxidant.'),
  'D008551': NtO(MH='Melena', MS='The black, tarry, foul-smelling FECES that contain degraded blood.'),
  'D008552': NtO(MH='Melengestrol Acetate', MS='A 6-methyl PROGESTERONE acetate with reported glucocorticoid activity and effect on ESTRUS.'),
  'D008553': NtO(MH='Melibiose', MS='A disaccharide consisting of one galactose and one glucose moiety in an alpha (1-6) glycosidic linkage.'),
  'D008554': NtO(MH='Melioidosis', MS='A disease of humans and animals that resembles GLANDERS. It is caused by BURKHOLDERIA PSEUDOMALLEI and may range from a dormant infection to a condition that causes multiple abscesses, pneumonia, and bacteremia.'),
  'D008555': NtO(MH='Melitten', MS='Basic polypeptide from the venom of the honey bee (Apis mellifera). It contains 26 amino acids, has cytolytic properties, causes contracture of muscle, releases histamine, and disrupts surface tension, probably due to lysis of cell and mitochondrial membranes.'),
  'D008556': NtO(MH='Melkersson-Rosenthal Syndrome', MS='An idiopathic syndrome characterized by one or more of the following; recurrent orofacial swelling, relapsing facial paralysis, and fissured tongue (lingua plicata). The onset is usually in childhood and relapses are common. Cheilitis granulomatosa is a monosymptomatic variant of this condition. (Dermatol Clin 1996 Apr;14(2):371-9; Magalini & Magalini, Dictionary of Medical Syndromes, 4th ed, p531)'),
  'D008557': NtO(MH='Melorheostosis', MS='A form of osteosclerosis extending in a linear track mainly through one of the long bones of the upper and lower limbs.'),
  'D008558': NtO(MH='Melphalan', MS='An alkylating nitrogen mustard that is used as an antineoplastic in the form of the levo isomer - MELPHALAN, the racemic mixture - MERPHALAN, and the dextro isomer - MEDPHALAN; toxic to bone marrow, but little vesicant action; potential carcinogen.'),
  'D008559': NtO(MH='Memantine', MS='AMANTADINE derivative that has some dopaminergic effects. It has been proposed as an antiparkinson agent.'),
  'D008560': NtO(MH='Membrane Fluidity', MS='The motion of phospholipid molecules within the lipid bilayer, dependent on the classes of phospholipids present, their fatty acid composition and degree of unsaturation of the acyl chains, the cholesterol concentration, and temperature.'),
  'D008561': NtO(MH='Membrane Fusion', MS='The adherence and merging of cell membranes, intracellular membranes, or artificial membranes to each other or to viruses, parasites, or interstitial particles through a variety of chemical and physical processes.'),
  'D008562': NtO(MH='Membrane Glycoproteins', MS='Glycoproteins found on the membrane or surface of cells.'),
  'D008563': NtO(MH='Membrane Lipids', MS='Lipids, predominantly phospholipids, cholesterol and small amounts of glycolipids found in membranes including cellular and intracellular membranes. These lipids may be arranged in bilayers in the membranes with integral proteins between the layers and peripheral proteins attached to the outside. Membrane lipids are required for active transport, several enzymatic activities and membrane formation.'),
  'D008564': NtO(MH='Membrane Potentials', MS="The voltage differences across a membrane. For cellular membranes they are computed by subtracting the voltage measured outside the membrane from the voltage measured inside the membrane. They result from differences of inside versus outside concentration of potassium, sodium, chloride, and other ions across cells' or ORGANELLES membranes. For excitable cells, the resting membrane potentials range between -30 and -100 millivolts. Physical, chemical, or electrical stimuli can make a membrane potential more negative (hyperpolarization), or less negative (depolarization)."),
  'D008565': NtO(MH='Membrane Proteins', MS='Proteins which are found in membranes including cellular and intracellular membranes. They consist of two types, peripheral and integral proteins. They include most membrane-associated enzymes, antigenic proteins, transport proteins, and drug, hormone, and lectin receptors.'),
  'D008566': NtO(MH='Membranes', MS='Thin layers of tissue which cover parts of the body, separate adjacent cavities, or connect adjacent structures.'),
  'D008567': NtO(MH='Membranes, Artificial', MS='Artificially produced membranes, such as semipermeable membranes used in artificial kidney dialysis (RENAL DIALYSIS), monomolecular and bimolecular membranes used as models to simulate biological CELL MEMBRANES. These membranes are also used in the process of GUIDED TISSUE REGENERATION.'),
  'D008568': NtO(MH='Memory', MS='Complex mental function having four distinct phases: (1) memorizing or learning, (2) retention, (3) recall, and (4) recognition. Clinically, it is usually subdivided into immediate, recent, and remote memory.'),
  'D008569': NtO(MH='Memory Disorders', MS='Disturbances in registering an impression, in the retention of an acquired impression, or in the recall of an impression. Memory impairments are associated with DEMENTIA; CRANIOCEREBRAL TRAUMA; ENCEPHALITIS; ALCOHOLISM (see also ALCOHOL AMNESTIC DISORDER); SCHIZOPHRENIA; and other conditions.'),
  'D008570': NtO(MH='Memory, Short-Term', MS='Remembrance of information for a few seconds to hours.'),
  'D008571': NtO(MH='Men', MS='Human males as cultural, psychological, sociological, political, and economic entities.'),
  'D008572': NtO(MH='Menarche', MS='The first MENSTRUAL CYCLE marked by the initiation of MENSTRUATION.'),
  'D008573': NtO(MH='Mendelevium', MS='A man-made radioactive element of the actinide family with atomic symbol Md, and atomic number 101.'),
  'D008574': NtO(MH='Mengovirus', MS='A strain of ENCEPHALOMYOCARDITIS VIRUS, a species of CARDIOVIRUS, isolated from rodents and lagomorphs and occasionally causing febrile illness in man.'),
  'D008575': NtO(MH='Meniere Disease', MS='A disease of the inner ear (LABYRINTH) that is characterized by fluctuating SENSORINEURAL HEARING LOSS; TINNITUS; episodic VERTIGO; and aural fullness. It is the most common form of endolymphatic hydrops.'),
  'D008576': NtO(MH='Meningeal Arteries', MS='Arteries which supply the dura mater.'),
  'D008577': NtO(MH='Meningeal Neoplasms', MS='Benign and malignant neoplastic processes that arise from or secondarily involve the meningeal coverings of the brain and spinal cord.'),
  'D008578': NtO(MH='Meninges', MS='The three membranes that cover the BRAIN and the SPINAL CORD. They are the dura mater, the arachnoid, and the pia mater.'),
  'D008579': NtO(MH='Meningioma', MS='A relatively common neoplasm of the CENTRAL NERVOUS SYSTEM that arises from arachnoidal cells. The majority are well differentiated vascular tumors which grow slowly and have a low potential to be invasive, although malignant subtypes occur. Meningiomas have a predilection to arise from the parasagittal region, cerebral convexity, sphenoidal ridge, olfactory groove, and SPINAL CANAL. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, pp2056-7)'),
  'D008580': NtO(MH='Meningism', MS='A condition characterized by neck stiffness, headache, and other symptoms suggestive of meningeal irritation, but without actual inflammation of the meninges (MENINGITIS). Spinal fluid pressure may be elevated but spinal fluid is normal. (DeJong, The Neurologic Examination, 4th ed, p673)'),
  'D008581': NtO(MH='Meningitis', MS='Inflammation of the coverings of the brain and/or spinal cord, which consist of the PIA MATER; ARACHNOID; and DURA MATER. Infections (viral, bacterial, and fungal) are the most common causes of this condition, but subarachnoid hemorrhage (HEMORRHAGES, SUBARACHNOID), chemical irritation (chemical MENINGITIS), granulomatous conditions, neoplastic conditions (CARCINOMATOUS MENINGITIS), and other inflammatory conditions may produce this syndrome. (From Joynt, Clinical Neurology, 1994, Ch24, p6)'),
  'D008582': NtO(MH='Meningitis, Aseptic', MS='A syndrome characterized by headache, neck stiffness, low grade fever, and CSF lymphocytic pleocytosis in the absence of an acute bacterial pathogen. Viral meningitis is the most frequent cause although MYCOPLASMA INFECTIONS; RICKETTSIA INFECTIONS; diagnostic or therapeutic procedures; NEOPLASTIC PROCESSES; septic perimeningeal foci; and other conditions may result in this syndrome. (From Adams et al., Principles of Neurology, 6th ed, p745)'),
  'D008583': NtO(MH='Meningitis, Haemophilus', MS='Infections of the nervous system caused by bacteria of the genus HAEMOPHILUS, and marked by prominent inflammation of the MENINGES. HAEMOPHILUS INFLUENZAE TYPE B is the most common causative organism. The condition primarily affects children under 6 years of age but may occur in adults.'),
  'D008584': NtO(MH='Meningitis, Listeria', MS='Inflammation of the meninges caused by LISTERIA MONOCYTOGENES infection, usually occurring in individuals under the age of 3 years or over the age of 50 years. It may occur at any age in individuals with IMMUNOLOGIC DEFICIENCY SYNDROMES. Clinical manifestations include FEVER, altered mentation, HEADACHE, meningeal signs, focal neurologic signs, and SEIZURES. (From Medicine 1998 Sep;77(5):313-36)'),
  'D008585': NtO(MH='Meningitis, Meningococcal', MS='A fulminant infection of the meninges and subarachnoid fluid by the bacterium NEISSERIA MENINGITIDIS, producing diffuse inflammation and peri-meningeal venous thromboses. Clinical manifestations include FEVER, nuchal rigidity, SEIZURES, severe HEADACHE, petechial rash, stupor, focal neurologic deficits, HYDROCEPHALUS, and COMA. The organism is usually transmitted via nasopharyngeal secretions and is a leading cause of meningitis in children and young adults. Organisms from Neisseria meningitidis serogroups A, B, C, Y, and W-135 have been reported to cause meningitis. (From Adams et al., Principles of Neurology, 6th ed, pp689-701; Curr Opin Pediatr 1998 Feb;10(1):13-8)'),
  'D008586': NtO(MH='Meningitis, Pneumococcal', MS="An acute purulent infection of the meninges and subarachnoid space caused by Streptococcus pneumoniae, most prevalent in children and adults over the age of 60. This illness may be associated with OTITIS MEDIA; MASTOIDITIS; SINUSITIS; RESPIRATORY TRACT INFECTIONS; sickle cell disease (ANEMIA, SICKLE CELL); skull fractures; and other disorders. Clinical manifestations include FEVER; HEADACHE; neck stiffness; and somnolence followed by SEIZURES; focal neurologic deficits (notably DEAFNESS); and COMA. (From Miller et al., Merritt's Textbook of Neurology, 9th ed, p111)"),
  'D008587': NtO(MH='Meningitis, Viral', MS='Viral infections of the leptomeninges and subarachnoid space. TOGAVIRIDAE INFECTIONS; FLAVIVIRIDAE INFECTIONS; RUBELLA; BUNYAVIRIDAE INFECTIONS; ORBIVIRUS infections; PICORNAVIRIDAE INFECTIONS; ORTHOMYXOVIRIDAE INFECTIONS; RHABDOVIRIDAE INFECTIONS; ARENAVIRIDAE INFECTIONS; HERPESVIRIDAE INFECTIONS; ADENOVIRIDAE INFECTIONS; JC VIRUS infections; and RETROVIRIDAE INFECTIONS may cause this form of meningitis. Clinical manifestations include fever, headache, neck pain, vomiting, PHOTOPHOBIA, and signs of meningeal irritation. (From Joynt, Clinical Neurology, 1996, Ch26, pp1-3)'),
  'D008588': NtO(MH='Meningocele', MS='A congenital or acquired protrusion of the meninges, unaccompanied by neural tissue, through a bony defect in the skull or vertebral column.'),
  'D008589': NtO(MH='Meningococcal Infections', MS='Infections with bacteria of the species NEISSERIA MENINGITIDIS.'),
  'D008590': NtO(MH='Meningoencephalitis', MS='An inflammatory process involving the brain (ENCEPHALITIS) and meninges (MENINGITIS), most often produced by pathogenic organisms which invade the central nervous system, and occasionally by toxins, autoimmune disorders, and other conditions.'),
  'D008591': NtO(MH='Meningomyelocele', MS='Congenital, or rarely acquired, herniation of meningeal and spinal cord tissue through a bony defect in the vertebral column. The majority of these defects occur in the lumbosacral region. Clinical features include PARAPLEGIA, loss of sensation in the lower body, and incontinence. This condition may be associated with the ARNOLD-CHIARI MALFORMATION and HYDROCEPHALUS. (From Joynt, Clinical Neurology, 1992, Ch55, pp35-6)'),
  'D008592': NtO(MH='Menisci, Tibial', MS='The interarticular fibrocartilages of the superior surface of the tibia.'),
  'D008593': NtO(MH='Menopause', MS='The last menstrual period. Permanent cessation of menses (MENSTRUATION) is usually defined after 6 to 12 months of AMENORRHEA in a woman over 45 years of age. In the United States, menopause generally occurs in women between 48 and 55 years of age.'),
  'D008594': NtO(MH='Menopause, Premature', MS='The premature cessation of menses (MENSTRUATION) when the last menstrual period occurs in a woman under the age of 40. It is due to the depletion of OVARIAN FOLLICLES. Premature MENOPAUSE can be caused by diseases; OVARIECTOMY; RADIATION; chemicals; and chromosomal abnormalities.'),
  'D008595': NtO(MH='Menorrhagia', MS='Excessive uterine bleeding during MENSTRUATION.'),
  'D008596': NtO(MH='Menotropins', MS='Extracts of urine from menopausal women that contain high concentrations of pituitary gonadotropins, FOLLICLE STIMULATING HORMONE and LUTEINIZING HORMONE. Menotropins are used to treat infertility. The FSH:LH ratio and degree of purity vary in different preparations.'),
  'D008597': NtO(MH='Menstrual Cycle', MS='The period from onset of one menstrual bleeding (MENSTRUATION) to the next in an ovulating woman or female primate. The menstrual cycle is regulated by endocrine interactions of the HYPOTHALAMUS; the PITUITARY GLAND; the ovaries; and the genital tract. The menstrual cycle is divided by OVULATION into two phases. Based on the endocrine status of the OVARY, there is a FOLLICULAR PHASE and a LUTEAL PHASE. Based on the response in the ENDOMETRIUM, the menstrual cycle is divided into a proliferative and a secretory phase.'),
  'D008598': NtO(MH='Menstruation', MS='The periodic shedding of the ENDOMETRIUM and associated menstrual bleeding in the MENSTRUAL CYCLE of humans and primates. Menstruation is due to the decline in circulating PROGESTERONE, and occurs at the late LUTEAL PHASE when LUTEOLYSIS of the CORPUS LUTEUM takes place.'),
  'D008599': NtO(MH='Menstruation Disturbances', MS='Variations of MENSTRUATION which may be indicative of disease.'),
  'D008600': NtO(MH='Menstruation-Inducing Agents', MS='Chemical compounds that induce menstruation either through direct action on the reproductive organs or through indirect action by relieving another condition of which amenorrhea is a secondary result. (From Dorland, 27th ed)'),
  'D008602': NtO(MH='Mental Healing', MS='The use of mind to cure disease, particularly physical illness.'),
  'D008603': NtO(MH='Mental Health', MS='Emotional, psychological, and social well-being of an individual or group.'),
  'D008604': NtO(MH='Mental Health Associations', MS='Voluntary organizations which support educational programs and research in psychiatry with the objective of the promotion of mental health. An early association in the United States was founded as the National Committee for Mental Hygiene in 1909, became the Mental Health Association in 1976 and later the National Mental Health Association in 1980. State and local mental health associations in this country are chartered by the national organization and affiliated with it.'),
  'D008605': NtO(MH='Mental Health Services', MS='Organized services to provide mental health care.'),
  'D008606': NtO(MH='Mental Processes', MS='Conceptual functions or thinking in all its forms.'),
  'D008607': NtO(MH='Intellectual Disability', MS='Subnormal intellectual functioning which originates during the developmental period. This has multiple potential etiologies, including genetic defects and perinatal insults. Intelligence quotient (IQ) scores are commonly used to determine whether an individual has an intellectual disability. IQ scores between 70 and 79 are in the borderline range. Scores below 67 are in the disabled range. (from Joynt, Clinical Neurology, 1992, Ch55, p28)'),
  'D008609': NtO(MH='Mental Status Schedule', MS='Standardized clinical interview used to assess current psychopathology by scaling patient responses to the questions.'),
  'D008610': NtO(MH='Menthol', MS='A monoterpene cyclohexanol produced from mint oils.'),
  'D008611': NtO(MH='Mentors', MS='Senior professionals who provide guidance, direction and support to those persons desirous of improvement in academic positions, administrative positions or other career development situations.'),
  'D008612': NtO(MH='Menu Planning', MS='A food service control process involving scheduling of meals in advance.'),
  'D008613': NtO(MH='Mepartricin', MS='Polyene macrolide antibiotic with unknown composition. It is obtained from Streptomyces aureofaciens. It is used as an antifungal agent, an antiprotozoal agent, and in the treatment of BENIGN PROSTATIC HYPERTROPHY.'),
  'D008614': NtO(MH='Meperidine', MS='A narcotic analgesic that can be used for the relief of most types of moderate to severe pain, including postoperative pain and the pain of labor. Prolonged use may lead to dependence of the morphine type; withdrawal symptoms appear more rapidly than with morphine and are of shorter duration.'),
  'D008615': NtO(MH='Mephenesin', MS='A centrally acting muscle relaxant with a short duration of action.'),
  'D008616': NtO(MH='Mephentermine', MS='A sympathomimetic agent with specificity for alpha-1 adrenergic receptors. It is used to maintain BLOOD PRESSURE in hypotensive states such as following SPINAL ANESTHESIA.'),
  'D008617': NtO(MH='Mephenytoin', MS='An anticonvulsant effective in tonic-clonic epilepsy (EPILEPSY, TONIC-CLONIC). It may cause blood dyscrasias.'),
  'D008618': NtO(MH='Mephobarbital', MS='A barbiturate that is metabolized to PHENOBARBITAL. It has been used for similar purposes, especially in EPILEPSY, but there is no evidence mephobarbital offers any advantage over PHENOBARBITAL.'),
  'D008619': NtO(MH='Mepivacaine', MS='A local anesthetic that is chemically related to BUPIVACAINE but pharmacologically related to LIDOCAINE. It is indicated for infiltration, nerve block, and epidural anesthesia. Mepivacaine is effective topically only in large doses and therefore should not be used by this route. (From AMA Drug Evaluations, 1994, p168)'),
  'D008620': NtO(MH='Meprobamate', MS='A carbamate with hypnotic, sedative, and some muscle relaxant properties, although in therapeutic doses reduction of anxiety rather than a direct effect may be responsible for muscle relaxation. Meprobamate has been reported to have anticonvulsant actions against petit mal seizures, but not against grand mal seizures (which may be exacerbated). It is used in the treatment of ANXIETY DISORDERS, and also for the short-term management of INSOMNIA but has largely been superseded by the BENZODIAZEPINES. (From Martindale, The Extra Pharmacopoeia, 30th ed, p603)'),
  'D008621': NtO(MH='Meptazinol', MS='A narcotic antagonist with analgesic properties. It is used for the control of moderate to severe pain.'),
  'D008622': NtO(MH='Merbromin', MS='A once-popular mercury containing topical antiseptic.'),
  'D008623': NtO(MH='Mercaptoethanol', MS='A water-soluble thiol derived from hydrogen sulfide and ethanol. It is used as a reducing agent for disulfide bonds and to protect sulfhydryl groups from oxidation.'),
  'D008624': NtO(MH='Mercaptoethylamines', MS='Ethylamines, including CYSTEAMINE, that contain a sulfhydryl group in their structure.'),
  'D008625': NtO(MH='Tiopronin', MS='Sulfhydryl acylated derivative of GLYCINE.'),
  'D008626': NtO(MH='Mercuribenzoates', MS='Mercury-containing benzoic acid derivatives.'),
  'D008627': NtO(MH='Mercuric Chloride', MS='Mercury chloride (HgCl2). A highly toxic compound that volatizes slightly at ordinary temperature and appreciably at 100 degrees C. It is corrosive to mucous membranes and used as a topical antiseptic and disinfectant.'),
  'D008628': NtO(MH='Mercury', MS='A silver metallic element that exists as a liquid at room temperature. It has the atomic symbol Hg (from hydrargyrum, liquid silver), atomic number 80, and atomic weight 200.59. Mercury is used in many industrial applications and its salts have been employed therapeutically as purgatives, antisyphilitics, disinfectants, and astringents. It can be absorbed through the skin and mucous membranes which leads to MERCURY POISONING. Because of its toxicity, the clinical use of mercury and mercurials is diminishing.'),
  'D008629': NtO(MH='Mercury Isotopes', MS='Stable mercury atoms that have the same atomic number as the element mercury, but differ in atomic weight. Hg-196, 198-201, and 204 are stable mercury isotopes.'),
  'D008630': NtO(MH='Mercury Poisoning', MS='Poisoning that results from chronic or acute ingestion, injection, inhalation, or skin absorption of MERCURY or MERCURY COMPOUNDS.'),
  'D008631': NtO(MH='Mercury Radioisotopes', MS='Unstable isotopes of mercury that decay or disintegrate emitting radiation. Hg atoms with atomic weights 185-195, 197, 203, 205, and 206 are radioactive mercury isotopes.'),
  'D008632': NtO(MH='Mermithoidea', MS='A superfamily of nematodes of the order ENOPLIDA. Characteristics include a reduced alimentary tract and the presence of a trophosome. Its organisms can be present in the human intestine through ingestion of unwashed or contaminated raw vegetables.'),
  'D008634': NtO(MH='Mersalyl', MS='A toxic thiol mercury salt formerly used as a diuretic. It inhibits various biochemical functions, especially in mitochondria, and is used to study those functions.'),
  'D008635': NtO(MH='Mescaline', MS='Hallucinogenic alkaloid isolated from the flowering heads (peyote) of Lophophora (formerly Anhalonium) williamsii, a Mexican cactus used in Indian religious rites and as an experimental psychotomimetic. Among its cellular effects are agonist actions at some types of serotonin receptors. It has no accepted therapeutic uses although it is legal for religious use by members of the Native American Church.'),
  'D008636': NtO(MH='Mesencephalon', MS='The middle of the three primitive cerebral vesicles of the embryonic brain. Without further subdivision, midbrain develops into a short, constricted portion connecting the PONS and the DIENCEPHALON. Midbrain contains two major parts, the dorsal TECTUM MESENCEPHALI and the ventral TEGMENTUM MESENCEPHALI, housing components of auditory, visual, and other sensorimoter systems.'),
  'D008637': NtO(MH='Mesenchymoma', MS='A mixed mesenchymal tumor composed of two or more mesodermal cellular elements not commonly associated, not counting fibrous tissue as one of the elements. Mesenchymomas are widely distributed in the body and about 75% are malignant. (Dorland, 27th ed; Holland et al., Cancer Medicine, 3d ed, p1866)'),
  'D008638': NtO(MH='Mesenteric Arteries', MS='Arteries which arise from the abdominal aorta and distribute to most of the intestines.'),
  'D008639': NtO(MH='Mesenteric Cyst', MS='A rare intra-abdominal tumor in the MESENTERY. Mesenteric cysts are usually benign and can be very large fluid-filled (2000 mL) lesions.'),
  'D008640': NtO(MH='Mesenteric Lymphadenitis', MS='INFLAMMATION of LYMPH NODES in the MESENTERY.'),
  'D008641': NtO(MH='Mesenteric Vascular Occlusion', MS='Obstruction of the flow in the SPLANCHNIC CIRCULATION by ATHEROSCLEROSIS; EMBOLISM; THROMBOSIS; STENOSIS; TRAUMA; and compression or intrinsic pressure from adjacent tumors. Rare causes are drugs, intestinal parasites, and vascular immunoinflammatory diseases such as PERIARTERITIS NODOSA and THROMBOANGIITIS OBLITERANS. (From Juergens et al., Peripheral Vascular Diseases, 5th ed, pp295-6)'),
  'D008642': NtO(MH='Mesenteric Veins', MS='Veins which return blood from the intestines; the inferior mesenteric vein empties into the splenic vein, the superior mesenteric vein joins the splenic vein to form the portal vein.'),
  'D008643': NtO(MH='Mesentery', MS='A layer of the peritoneum which attaches the abdominal viscera to the ABDOMINAL WALL and conveys their blood vessels and nerves.'),
  'D008644': NtO(MH='Mesial Movement of Teeth', MS="Migration of the teeth toward the midline or forward in the DENTAL ARCH. (From Boucher's Clinical Dental Terminology, 4th ed)"),
  'D008645': NtO(MH='Mesocestoides', MS='A genus of tapeworm, containing several species, found as adults in birds and mammals. The larvae or cysticercoid stage develop in invertebrates. Human infection has been reported and is probably acquired from eating inadequately cooked meat of animals infected with the second larval stage known as the tetrahythridium.'),
  'D008646': NtO(MH='Mesocolon', MS='The fold of peritoneum by which the COLON is attached to the posterior ABDOMINAL WALL.'),
  'D008647': NtO(MH='Mesocricetus', MS='A genus of the family Muridae having three species. The present domesticated strains were developed from individuals brought from Syria. They are widely used in biomedical research.'),
  'D008648': NtO(MH='Mesoderm', MS='The middle germ layer of an embryo derived from three paired mesenchymal aggregates along the neural tube.'),
  'D008649': NtO(MH='Mesonephroma', MS='A rare tumor of the female genital tract, most often the ovary, formerly considered to be derived from mesonephric rests. Two varieties are recognized: (1) clear cell carcinoma, so called because of its histologic resemblance to renal cell carcinoma, and now considered to be of muellerian duct derivation and (2) an embryonal tumor (called also ENDODERMAL SINUS TUMOR and yolk sac tumor), occurring chiefly in children. The latter variety may also arise in the testis. (Dorland, 27th ed)'),
  'D008650': NtO(MH='Mesonephros', MS='One of a pair of excretory organs (mesonephroi) which grows caudally to the first pair (PRONEPHROI) during development. Mesonephroi are the permanent kidneys in adult amphibians and fish. In higher vertebrates, proneprhoi and most of mesonephroi degenerate with the appearance of metanephroi. The remaining ducts become WOLFFIAN DUCTS.'),
  'D008651': NtO(MH='Mesons', MS='Short-lived elementary particles found in cosmic radiation or produced from nuclear disintegration. Their mass is between that of protons and electrons and they can be negative, positive, or neutral. pi-Mesons (pions) are heavier than mu-mesons (muons) and are proposed for cancer radiotherapy because their capture and disintegration by matter produces powerful, but short-lived, secondary radiation.'),
  'D008652': NtO(MH='Mesoporphyrins', MS='Porphyrins with four methyl, two ethyl, and two propionic acid side chains attached to the pyrrole rings.'),
  'D008653': NtO(MH='Mesoridazine', MS='A phenothiazine antipsychotic with effects similar to CHLORPROMAZINE.'),
  'D008654': NtO(MH='Mesothelioma', MS='A tumor derived from mesothelial tissue (peritoneum, pleura, pericardium). It appears as broad sheets of cells, with some regions containing spindle-shaped, sarcoma-like cells and other regions showing adenomatous patterns. Pleural mesotheliomas have been linked to exposure to asbestos. (Dorland, 27th ed)'),
  'D008655': NtO(MH='Mesterolone', MS='17 beta-Hydroxy-1 alpha-methyl-5 alpha-androstan-3-one. A synthetic steroid with anabolic and androgenic activities.'),
  'D008656': NtO(MH='Mestranol', MS='The 3-methyl ether of ETHINYL ESTRADIOL. It must be demethylated to be biologically active. It is used as the estrogen component of many combination ORAL CONTRACEPTIVES.'),
  'D008657': NtO(MH='Metabolic Clearance Rate', MS='Volume of biological fluid completely cleared of drug metabolites as measured in unit time. Elimination occurs as a result of metabolic processes in the kidney, liver, saliva, sweat, intestine, heart, brain, or other site.'),
  'D008658': NtO(MH='Inactivation, Metabolic', MS='Reduction of pharmacologic activity or toxicity of a drug or other foreign substance by a living system, usually by enzymatic action. It includes those metabolic transformations that make the substance more soluble for faster renal excretion.'),
  'D008659': NtO(MH='Metabolic Diseases', MS='Generic term for diseases caused by an abnormal metabolic process. It can be congenital due to inherited enzyme abnormality (METABOLISM, INBORN ERRORS) or acquired due to disease of an endocrine organ or failure of a metabolically important organ such as the liver. (Stedman, 26th ed)'),
  'D008660': NtO(MH='Metabolism', MS='The chemical reactions in living organisms by which energy is provided for vital processes and activities and new material is assimilated.'),
  'D008661': NtO(MH='Metabolism, Inborn Errors', MS='Errors in metabolic processes resulting from inborn genetic mutations that are inherited or acquired in utero.'),
  'D008662': NtO(MH='Metacarpophalangeal Joint', MS='The articulation between a metacarpal bone and a phalanx.'),
  'D008663': NtO(MH='Metacarpus', MS='The region of the HAND between the WRIST and the FINGERS.'),
  'D008664': NtO(MH='Metal Metabolism, Inborn Errors', MS='Dysfunctions in the metabolism of metals resulting from inborn genetic mutations that are inherited or acquired in utero.'),
  'D008665': NtO(MH='Metalloporphyrins', MS='Porphyrins which are combined with a metal ion. The metal is bound equally to all four nitrogen atoms of the pyrrole rings. They possess characteristic absorption spectra which can be utilized for identification or quantitative estimation of porphyrins and porphyrin-bound compounds.'),
  'D008666': NtO(MH='Metalloendopeptidases', MS='ENDOPEPTIDASES which use a metal such as ZINC in the catalytic mechanism.'),
  'D008667': NtO(MH='Metalloproteins', MS='Proteins that have one or more tightly bound metal ions forming part of their structure. (Dorland, 28th ed)'),
  'D008668': NtO(MH='Metallothionein', MS='A low-molecular-weight (approx. 10 kD) protein occurring in the cytoplasm of kidney cortex and liver. It is rich in cysteinyl residues and contains no aromatic amino acids. Metallothionein shows high affinity for bivalent heavy metals.'),
  'D008669': NtO(MH='Metallurgy', MS='The science, art, or technology dealing with processes involved in the separation of metals from their ores, the technique of making or compounding the alloys, the techniques of working or heat-treating metals, and the mining of metals. It includes industrial metallurgy as well as metallurgical techniques employed in the preparation and working of metals used in dentistry, with special reference to orthodontic and prosthodontic appliances. (From Jablonski, Dictionary of Dentistry, 1992, p494)'),
  'D008670': NtO(MH='Metals', MS="Electropositive chemical elements characterized by ductility, malleability, luster, and conductance of heat and electricity. They can replace the hydrogen of an acid and form bases with hydroxyl radicals. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D008671': NtO(MH='Actinoid Series Elements', MS='A series of radioactive elements from ACTINIUM, atomic number 89, to and including LAWRENCIUM, atomic number 103.'),
  'D008672': NtO(MH='Metals, Alkali', MS='Metals that constitute group 1(formerly group Ia) of the periodic table. They are the most strongly electropositive of the metals. Note that HYDROGEN is not considered an alkali metal even though it falls under the group 1 heading in the periodic table.'),
  'D008673': NtO(MH='Metals, Alkaline Earth', MS='Metals that constitute the group 2 (formerly group IIa) of the periodic table.'),
  'D008674': NtO(MH='Metals, Rare Earth', MS="A group of elements that include SCANDIUM; YTTRIUM; and the LANTHANOID SERIES ELEMENTS. Historically, the rare earth metals got their name from the fact that they were never found in their pure elemental form, but as an oxide. In addition they were very difficult to purify. They are not truly rare and comprise about 25% of the metals in the earth's crust."),
  'D008675': NtO(MH='Metamorphosis, Biological', MS='Profound physical changes during maturation of living organisms from the immature forms to the adult forms, such as from TADPOLES to frogs; caterpillars to BUTTERFLIES.'),
  'D008676': NtO(MH='Metanephrine', MS='Product of epinephrine O-methylation. It is a commonly occurring, pharmacologically and physiologically inactive metabolite of epinephrine.'),
  'D008677': NtO(MH='Metaphase', MS='The phase of cell nucleus division following PROMETAPHASE, in which the CHROMOSOMES line up across the equatorial plane of the SPINDLE APPARATUS prior to separation.'),
  'D008678': NtO(MH='Metaphysics', MS='The branch of philosophy that treats of first principles, including ontology (the nature of existence or being) and cosmology (the origin and structure of the universe). (From Random House Unabridged Dictionary, 2d ed)'),
  'D008679': NtO(MH='Metaplasia', MS='A condition in which there is a change of one adult cell type to another similar adult cell type.'),
  'D008680': NtO(MH='Metaraminol', MS='A sympathomimetic agent that acts predominantly at alpha-1 adrenergic receptors. It has been used primarily as a vasoconstrictor in the treatment of HYPOTENSION.'),
  'D008681': NtO(MH='Metastrongyloidea', MS='A superfamily of nematodes of the order STRONGYLIDA. Characteristics include a fluid-filled outer layer of cuticle and a reduced mouth and bursa.'),
  'D008682': NtO(MH='Metatarsal Bones', MS='The five long bones of the METATARSUS, articulating with the TARSAL BONES proximally and the PHALANGES OF TOES distally.'),
  'D008683': NtO(MH='Metatarsophalangeal Joint', MS='The articulation between a metatarsal bone (METATARSAL BONES) and a phalanx.'),
  'D008684': NtO(MH='Metatarsus', MS='The part of the foot between the tarsa and the TOES.'),
  'D008685': NtO(MH='Meteorological Concepts', MS='The atmospheric properties, characteristics and other atmospheric phenomena especially pertaining to WEATHER or CLIMATE.'),
  'D008686': NtO(MH='Metestrus', MS='The period following ESTRUS during which the phenomena of estrus subside in those animals in which pregnancy or pseudopregnancy does not occur.'),
  'D008687': NtO(MH='Metformin', MS='A biguanide hypoglycemic agent used in the treatment of non-insulin-dependent diabetes mellitus not responding to dietary modification. Metformin improves glycemic control by improving insulin sensitivity and decreasing intestinal absorption of glucose. (From Martindale, The Extra Pharmacopoeia, 30th ed, p289)'),
  'D008688': NtO(MH='Methacholine Compounds', MS='A group of compounds that are derivatives of beta-methylacetylcholine (methacholine).'),
  'D008689': NtO(MH='Methacrylates', MS='Acrylic acids or acrylates which are substituted in the C-2 position with a methyl group.'),
  'D008690': NtO(MH='Methacycline', MS='A broad-spectrum semisynthetic antibiotic related to TETRACYCLINE but excreted more slowly and maintaining effective blood levels for a more extended period.'),
  'D008691': NtO(MH='Methadone', MS='A synthetic opioid that is used as the hydrochloride. It is an opioid analgesic that is primarily a mu-opioid agonist. It has actions and uses similar to those of MORPHINE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1082-3)'),
  'D008692': NtO(MH='Methadyl Acetate', MS='A narcotic analgesic with a long onset and duration of action.'),
  'D008693': NtO(MH='Methallibure', MS='A dithiobiurea compound with anti-gonadotropic activity.'),
  'D008694': NtO(MH='Methamphetamine', MS='A central nervous system stimulant and sympathomimetic with actions and uses similar to DEXTROAMPHETAMINE. The smokable form is a drug of abuse and is referred to as crank, crystal, crystal meth, ice, and speed.'),
  'D008695': NtO(MH='Methandriol', MS='A synthetic steroid with anabolic and androgenic properties. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1188)'),
  'D008696': NtO(MH='Methandrostenolone', MS='A synthetic steroid with anabolic properties that are more pronounced than its androgenic effects. It has little progestational activity. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1188)'),
  'D008697': NtO(MH='Methane', MS="The simplest saturated hydrocarbon. It is a colorless, flammable gas, slightly soluble in water. It is one of the chief constituents of natural gas and is formed in the decomposition of organic matter. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D008698': NtO(MH='Mesylates', MS='Organic salts or esters of methanesulfonic acid.'),
  'D008701': NtO(MH='Methapyrilene', MS='Histamine H1 antagonist with sedative action used as a hypnotic and in allergies.'),
  'D008702': NtO(MH='Methaqualone', MS='A quinazoline derivative with hypnotic and sedative properties. It has been withdrawn from the market in many countries because of problems with abuse. (From Martindale, The Extra Pharmacopoeia, 30th ed, p604)'),
  'D008704': NtO(MH='Methazolamide', MS='A carbonic anhydrase inhibitor that is used as a diuretic and in the treatment of glaucoma.'),
  'D008705': NtO(MH='Methemalbumin', MS='A 1:1 molar complex of heme or hematin and albumin formed after the dissociation of methemoglobin into heme or hematin and globin in plasma. This complex, which imparts a coffee-brown color to plasma, occurs in hemolytic and hemorrhagic disorders. Its presence in plasma is used as a test to differentiate between hemorrhagic and edematous pancreatitis.'),
  'D008706': NtO(MH='Methemoglobin', MS=None),
  'D008708': NtO(MH='Methemoglobinemia', MS='The presence of methemoglobin in the blood, resulting in cyanosis. A small amount of methemoglobin is present in the blood normally, but injury or toxic agents convert a larger proportion of hemoglobin into methemoglobin, which does not function reversibly as an oxygen carrier. Methemoglobinemia may be due to a defect in the enzyme NADH methemoglobin reductase (an autosomal recessive trait) or to an abnormality in hemoglobin M (an autosomal dominant trait). (Dorland, 27th ed)'),
  'D008709': NtO(MH='Methenamine', MS='An anti-infective agent most commonly used in the treatment of urinary tract infections. Its anti-infective action derives from the slow release of formaldehyde by hydrolysis at acidic pH. (From Martindale, The Extra Pharmacopoeia, 30th ed, p173)'),
  'D008710': NtO(MH='Methenolone', MS='A synthetic steroid that has been used for its anabolic action.'),
  'D008711': NtO(MH='Metergoline', MS='A dopamine agonist and serotonin antagonist. It has been used similarly to BROMOCRIPTINE as a dopamine agonist and also for MIGRAINE DISORDERS therapy.'),
  'D008712': NtO(MH='Methicillin', MS='One of the PENICILLINS which is resistant to PENICILLINASE but susceptible to a penicillin-binding protein. It is inactivated by gastric acid so administered by injection.'),
  'D008713': NtO(MH='Methimazole', MS='A thioureylene antithyroid agent that inhibits the formation of thyroid hormones by interfering with the incorporation of iodine into tyrosyl residues of thyroglobulin. This is done by interfering with the oxidation of iodide ion and iodotyrosyl groups through inhibition of the peroxidase enzyme.'),
  'D008714': NtO(MH='Methiocarb', MS='Insecticide, molluscacide, acaricide.'),
  'D008715': NtO(MH='Methionine', MS='A sulfur-containing essential L-amino acid that is important in many body functions.'),
  'D008716': NtO(MH='Methionine Adenosyltransferase', MS='An enzyme that catalyzes the synthesis of S-adenosylmethionine from methionine and ATP. EC 2.5.1.6.'),
  'D008717': NtO(MH='Methionine Sulfoximine', MS=None),
  'D008718': NtO(MH='Methionine-tRNA Ligase', MS='An enzyme that activates methionine with its specific transfer RNA. EC 6.1.1.10.'),
  'D008719': NtO(MH='Methiothepin', MS='A serotonin receptor antagonist in the CENTRAL NERVOUS SYSTEM used as an antipsychotic.'),
  'D008720': NtO(MH='Methisazone', MS='An antiviral agent effective against pox viruses.'),
  'D008721': NtO(MH='Methocarbamol', MS='A centrally acting muscle relaxant whose mode of action has not been established. It is used as an adjunct in the symptomatic treatment of musculoskeletal conditions associated with painful muscle spasm. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1206)'),
  'D008722': NtO(MH='Methods', MS='A series of steps taken in order to conduct research.'),
  'D008723': NtO(MH='Methohexital', MS='An intravenous anesthetic with a short duration of action that may be used for induction of anesthesia.'),
  'D008724': NtO(MH='Methomyl', MS='A carbamate insecticide with anticholinesterase activity.'),
  'D008725': NtO(MH='Bis-Trimethylammonium Compounds', MS='Compounds containing polymethylene bis-trimethylammonium cations. Members of this group frequently act as ganglionic blockers and neuromuscular depolarizing agents.'),
  'D008726': NtO(MH='Methoprene', MS='Juvenile hormone analog and insect growth regulator used to control insects by disrupting metamorphosis. Has been effective in controlling mosquito larvae.'),
  'D008727': NtO(MH='Methotrexate', MS='An antineoplastic antimetabolite with immunosuppressant properties. It is an inhibitor of TETRAHYDROFOLATE DEHYDROGENASE and prevents the formation of tetrahydrofolate, necessary for synthesis of thymidylate, an essential component of DNA.'),
  'D008728': NtO(MH='Methotrimeprazine', MS='A phenothiazine with pharmacological activity similar to that of both CHLORPROMAZINE and PROMETHAZINE. It has the histamine-antagonist properties of the antihistamines together with CENTRAL NERVOUS SYSTEM effects resembling those of chlorpromazine. (From Martindale, The Extra Pharmacopoeia, 30th ed, p604)'),
  'D008729': NtO(MH='Methoxamine', MS='An alpha-1 adrenergic agonist that causes prolonged peripheral VASOCONSTRICTION.'),
  'D008730': NtO(MH='Methoxsalen', MS='A naturally occurring furocoumarin compound found in several species of plants, including Psoralea corylifolia. It is a photoactive substance that forms DNA ADDUCTS in the presence of ultraviolet A irradiation.'),
  'D008731': NtO(MH='Methoxychlor', MS='An insecticide. Methoxychlor has estrogenic effects in mammals, among other effects.'),
  'D008732': NtO(MH='Methoxydimethyltryptamines', MS='Compounds that contain the biogenic monoamine tryptamine and are substituted with one methoxy group and two methyl groups. Members of this group include several potent serotonergic hallucinogens found in several unrelated plants, skins of certain toads, and in mammalian brains. They are possibly involved in the etiology of schizophrenia.'),
  'D008733': NtO(MH='Methoxyflurane', MS='An inhalation anesthetic. Currently, methoxyflurane is rarely used for surgical, obstetric, or dental anesthesia. If so employed, it should be administered with NITROUS OXIDE to achieve a relatively light level of anesthesia, and a neuromuscular blocking agent given concurrently to obtain the desired degree of muscular relaxation. (From AMA Drug Evaluations Annual, 1994, p180)'),
  'D008734': NtO(MH='Methoxyhydroxyphenylglycol', MS='Synthesized from endogenous epinephrine and norepinephrine in vivo. It is found in brain, blood, CSF, and urine, where its concentrations are used to measure catecholamine turnover.'),
  'D008735': NtO(MH='5-Methoxytryptamine', MS='Serotonin derivative proposed as potentiator for hypnotics and sedatives.'),
  'D008736': NtO(MH='Methyclothiazide', MS='A thiazide diuretic with properties similar to those of HYDROCHLOROTHIAZIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p825)'),
  'D008737': NtO(MH='Methyl Chloride', MS='A hydrocarbon used as an industrial solvent. It has been used as an aerosal propellent, as a refrigerant and as a local anesthetic. (From Martindale, The Extra Pharmacopoeia, 31st ed, p1403)'),
  'D008738': NtO(MH='Methyl Ethers', MS='A group of compounds that contain the general formula R-OCH3.'),
  'D008739': NtO(MH='Methyl Green', MS='A tri-benzene-ammonium usually compounded with zinc chloride. It is used as a biological stain and for the dyeing and printing of textiles.'),
  'D008741': NtO(MH='Methyl Methanesulfonate', MS='An alkylating agent in cancer therapy that may also act as a mutagen by interfering with and causing damage to DNA.'),
  'D008742': NtO(MH='Methyl n-Butyl Ketone', MS='An industrial solvent which causes nervous system degeneration. MBK is an acronym often used to refer to it.'),
  'D008743': NtO(MH='Methyl Parathion', MS='The methyl homolog of parathion. An effective, but highly toxic, organothiophosphate insecticide and cholinesterase inhibitor.'),
  'D008744': NtO(MH='Methylamines', MS='Derivatives of methylamine (the structural formula CH3NH2).'),
  'D008745': NtO(MH='Methylation', MS='Addition of methyl groups. In histo-chemistry methylation is used to esterify carboxyl groups and remove sulfate groups by treating tissue sections with hot methanol in the presence of hydrochloric acid. (From Stedman, 25th ed)'),
  'D008746': NtO(MH='Methylazoxymethanol Acetate', MS='The aglycone of CYCASIN. It acts as a potent carcinogen and neurotoxin and inhibits hepatic DNA, RNA, and protein synthesis.'),
  'D008747': NtO(MH='Methylcellulose', MS='Methylester of cellulose. Methylcellulose is used as an emulsifying and suspending agent in cosmetics, pharmaceutics and the chemical industry. It is used therapeutically as a bulk laxative.'),
  'D008748': NtO(MH='Methylcholanthrene', MS='A carcinogen that is often used in experimental cancer studies.'),
  'D008749': NtO(MH='Methyldimethylaminoazobenzene', MS='A very potent liver carcinogen.'),
  'D008750': NtO(MH='Methyldopa', MS='An alpha-2 adrenergic agonist that has both central and peripheral nervous system effects. Its primary clinical use is as an antihypertensive agent.'),
  'D008751': NtO(MH='Methylene Blue', MS='A compound consisting of dark green crystals or crystalline powder, having a bronze-like luster. Solutions in water or alcohol have a deep blue color. Methylene blue is used as a bacteriologic stain and as an indicator. It inhibits GUANYLATE CYCLASE, and has been used to treat cyanide poisoning and to lower levels of METHEMOGLOBIN.'),
  'D008752': NtO(MH='Methylene Chloride', MS='A chlorinated hydrocarbon that has been used as an inhalation anesthetic and acts as a narcotic in high concentrations. Its primary use is as a solvent in manufacturing and food technology.'),
  'D008753': NtO(MH='Methylenebis(chloroaniline)', MS='Aromatic diamine used in the plastics industry as curing agent for epoxy resins and urethane rubbers. It causes bladder, liver, lung, and other neoplasms.'),
  'D008754': NtO(MH='Methylenetetrahydrofolate Dehydrogenase (NADP)', MS='An NADP-dependent oxidoreductase that catalyses the conversion of 5,10-methyleneterahydrofolate to 5,10-methenyl-tetrahydrofolate. In higher eukaryotes a trifunctional enzyme exists with additional METHENYLTETRAHYDROFOLATE CYCLOHYDROLASE and FORMATE-TETRAHYDROFOLATE LIGASE activity. The enzyme plays an important role in the synthesis of 5-methyltetrahydrofolate, the methyl donor for the VITAMIN B12-dependent remethylation of HOMOCYSTEINE to METHIONINE via METHIONINE SYNTHETASE.'),
  'D008755': NtO(MH='Methylergonovine', MS='A homolog of ERGONOVINE containing one more CH2 group. (Merck Index, 11th ed)'),
  'D008756': NtO(MH='Methylgalactosides', MS=None),
  'D008757': NtO(MH='Methylglucosides', MS=None),
  'D008759': NtO(MH='Methylglycosides', MS=None),
  'D008760': NtO(MH='Methylguanidine', MS='A product of putrefaction. Poisonous.'),
  'D008761': NtO(MH='Methylhistamines', MS='Histamine substituted in any position with one or more methyl groups. Many of these are agonists for the H1, H2, or both histamine receptors.'),
  'D008762': NtO(MH='Methylhistidines', MS='Histidine substituted in any position with one or more methyl groups.'),
  'D008763': NtO(MH='Methylhydrazines', MS='Hydrazines substituted by one or more methyl groups in any position.'),
  'D008764': NtO(MH='Methylmalonic Acid', MS='A malonic acid derivative which is a vital intermediate in the metabolism of fat and protein. Abnormalities in methylmalonic acid metabolism lead to methylmalonic aciduria. This metabolic disease is attributed to a block in the enzymatic conversion of methylmalonyl CoA to succinyl CoA.'),
  'D008765': NtO(MH='Methylmalonyl-CoA Mutase', MS='An enzyme that catalyzes the conversion of methylmalonyl-CoA to succinyl-CoA by transfer of the carbonyl group. It requires a cobamide coenzyme. A block in this enzymatic conversion leads to the metabolic disease, methylmalonic aciduria. EC 5.4.99.2.'),
  'D008766': NtO(MH='Methylmannosides', MS='Mannosides formed by the reaction of the hydroxyl group on the anomeric carbon atom of mannose with methyl alcohol. They include both alpha- and beta-methylmannosides.'),
  'D008767': NtO(MH='Methylmercury Compounds', MS='Organic compounds in which mercury is attached to a methyl group.'),
  'D008768': NtO(MH='Methylmethacrylates', MS='The methyl esters of methacrylic acid that polymerize easily and are used as tissue cements, dental materials, and absorbent for biological substances.'),
  'D008769': NtO(MH='Methylnitronitrosoguanidine', MS='A nitrosoguanidine derivative with potent mutagenic and carcinogenic properties.'),
  'D008770': NtO(MH='Methylnitrosourea', MS='A nitrosourea compound with alkylating, carcinogenic, and mutagenic properties.'),
  'D008771': NtO(MH='Nordefrin', MS='A norepinephrine derivative used as a vasoconstrictor agent.'),
  'D008772': NtO(MH='Methylococcaceae', MS='A family of gram-negative, aerobic bacteria utilizing only one-carbon organic compounds and isolated from in soil and water.'),
  'D008773': NtO(MH='Methylphenazonium Methosulfate', MS='Used as an electron carrier in place of the flavine enzyme of Warburg in the hexosemonophosphate system and also in the preparation of SUCCINIC DEHYDROGENASE.'),
  'D008774': NtO(MH='Methylphenidate', MS='A central nervous system stimulant used most commonly in the treatment of ATTENTION DEFICIT DISORDER in children and for NARCOLEPSY. Its mechanisms appear to be similar to those of DEXTROAMPHETAMINE. The d-isomer of this drug is referred to as DEXMETHYLPHENIDATE HYDROCHLORIDE.'),
  'D008775': NtO(MH='Methylprednisolone', MS='A PREDNISOLONE derivative with similar anti-inflammatory action.'),
  'D008776': NtO(MH='Methylprednisolone Hemisuccinate', MS='A water-soluble ester of METHYLPREDNISOLONE used for cardiac, allergic, and hypoxic emergencies.'),
  'D008777': NtO(MH='Methyltestosterone', MS='A synthetic hormone used for androgen replacement therapy and as an hormonal antineoplastic agent (ANTINEOPLASTIC AGENTS, HORMONAL).'),
  'D008778': NtO(MH='Methylthioinosine', MS='6-(Methylthio)-9-beta-D-ribofuranosylpurine. An analog of inosine with a methylthio group replacing the hydroxyl group in the 6-position.'),
  'D008779': NtO(MH='Methylthiouracil', MS='A thiourea antithyroid agent that inhibits the synthesis of thyroid hormone. It is used in the treatment of hyperthyroidism.'),
  'D008780': NtO(MH='Methyltransferases', MS='A subclass of enzymes of the transferase class that catalyze the transfer of a methyl group from one compound to another. (Dorland, 28th ed) EC 2.1.1.'),
  'D008781': NtO(MH='Methyltyrosines', MS='A group of compounds that are methyl derivatives of the amino acid TYROSINE.'),
  'D008782': NtO(MH='Methylurea Compounds', MS='Urea compounds which are substituted with one or more methyl groups.'),
  'D008784': NtO(MH='Methysergide', MS='An ergot derivative that is a congener of LYSERGIC ACID DIETHYLAMIDE. It antagonizes the effects of serotonin in blood vessels and gastrointestinal smooth muscle, but has few of the properties of other ergot alkaloids. Methysergide is used prophylactically in migraine and other vascular headaches and to antagonize serotonin in the carcinoid syndrome.'),
  'D008785': NtO(MH='Metiamide', MS='A histamine H2 receptor antagonist that is used as an anti-ulcer agent.'),
  'D008786': NtO(MH='Metmyoglobin', MS='Myoglobin which is in the oxidized ferric or hemin form. The oxidation causes a change in color from red to brown.'),
  'D008787': NtO(MH='Metoclopramide', MS='A dopamine D2 antagonist that is used as an antiemetic.'),
  'D008788': NtO(MH='Metolazone', MS='A quinazoline-sulfonamide derived DIURETIC that functions by inhibiting SODIUM CHLORIDE SYMPORTERS.'),
  'D008790': NtO(MH='Metoprolol', MS='A selective adrenergic beta-1 blocking agent that is commonly used to treat ANGINA PECTORIS; HYPERTENSION; and CARDIAC ARRHYTHMIAS.'),
  'D008791': NtO(MH='Metrial Gland', MS='Collection of granular epithelial cells in the uterine muscle beneath the placenta that develop during pregnancy in certain species of animals.'),
  'D008792': NtO(MH='Metric System', MS='A system of units used in scientific work throughout the world and employed in general commercial transactions and engineering applications. Its units of length, time, and mass are the meter, second, and kilogram respectively, or decimal multiples and submultiples thereof. ( McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D008793': NtO(MH='Metrizamide', MS='A solute for density gradient centrifugation offering higher maximum solution density without the problems of increased viscosity. It is also used as a resorbable, non-ionic contrast medium.'),
  'D008794': NtO(MH='Metrizoic Acid', MS='A diagnostic radiopaque that usually occurs as the sodium salt.'),
  'D008795': NtO(MH='Metronidazole', MS='A nitroimidazole used to treat AMEBIASIS; VAGINITIS; TRICHOMONAS INFECTIONS; GIARDIASIS; ANAEROBIC BACTERIA; and TREPONEMAL INFECTIONS.'),
  'D008796': NtO(MH='Metrorrhagia', MS='Abnormal uterine bleeding that is not related to MENSTRUATION, usually in females without regular MENSTRUAL CYCLE. The irregular and unpredictable bleeding usually comes from a dysfunctional ENDOMETRIUM.'),
  'D008797': NtO(MH='Metyrapone', MS='An inhibitor of the enzyme STEROID 11-BETA-MONOOXYGENASE. It is used as a test of the feedback hypothalamic-pituitary mechanism in the diagnosis of CUSHING SYNDROME.'),
  'D008798': NtO(MH='Mevalonic Acid', MS='A dihydroxy monocarboxylic acid and precursor in the biosynthetic pathway known as the mevalonate pathway, which produces terpenes and steroids that are vital for diverse cellular functions.'),
  'D008799': NtO(MH='Mevinphos', MS='An organophosphate cholinesterase inhibitor that is used as an insecticide.'),
  'D008800': NtO(MH='Mexico', MS='A country in NORTH AMERICA, bordering the Caribbean Sea and the Gulf of Mexico, between BELIZE and the UNITED STATES, and bordering the North Pacific Ocean, between Guatemala and the UNITED STATES.'),
  'D008801': NtO(MH='Mexiletine', MS='Antiarrhythmic agent pharmacologically similar to LIDOCAINE. It may have some anticonvulsant properties.'),
  'D008802': NtO(MH='Mezlocillin', MS='Semisynthetic ampicillin-derived acylureido penicillin. It has been proposed for infections with certain anaerobes and may be useful in inner ear, bile, and CNS infections.'),
  'D008803': NtO(MH='Mianserin', MS='A tetracyclic compound with antidepressant effects. It may cause drowsiness and hematological problems. Its mechanism of therapeutic action is not well understood, although it apparently blocks alpha-adrenergic, histamine H1, and some types of serotonin receptors.'),
  'D008805': NtO(MH='Mice, Inbred A', MS='An inbred strain of mouse that is widely used in IMMUNOLOGY studies and cancer research.'),
  'D008806': NtO(MH='Mice, Inbred AKR', MS='An inbred strain of mouse that is widely used in IMMUNOLOGY studies and cancer research.'),
  'D008807': NtO(MH='Mice, Inbred BALB C', MS='An inbred strain of mouse that is widely used in IMMUNOLOGY studies and cancer research.'),
  'D008808': NtO(MH='Mice, Inbred CBA', MS='An inbred strain of mouse that is widely used in BIOMEDICAL RESEARCH.'),
  'D008809': NtO(MH='Mice, Inbred C3H', MS='An inbred strain of mouse that is used as a general purpose strain in a wide variety of RESEARCH areas including CANCER; INFECTIOUS DISEASES; sensorineural, and cardiovascular biology research.'),
  'D008810': NtO(MH='Mice, Inbred C57BL', MS='One of the first INBRED MOUSE STRAINS to be sequenced. This strain is commonly used as genetic background for transgenic mouse models. Refractory to many tumors, this strain is also preferred model for studying role of genetic variations in development of diseases.'),
  'D008811': NtO(MH='Mice, Inbred DBA', MS='An inbred strain of mouse. Specific substrains are used in a variety of areas of BIOMEDICAL RESEARCH such as DBA/1J, which is used as a model for RHEUMATOID ARTHRITIS.'),
  'D008812': NtO(MH='Mice, Hairless', MS='Mutant strains of mice that produce little or no hair.'),
  'D008813': NtO(MH='Mice, Inbred ICR', MS='An inbred strain of mouse that is used as a general purpose research strain, for therapeutic drug testing, and for the genetic analysis of CARCINOGEN-induced COLON CANCER.'),
  'D008814': NtO(MH='Mice, Inbred NZB', MS='An inbred strain of mouse that is widely used as a model for AUTOIMMUNE DISEASES such as SYSTEMIC LUPUS ERYTHEMATOSUS.'),
  'D008815': NtO(MH='Mice, Inbred Strains', MS='Genetically identical individuals developed from brother and sister matings which have been carried out for twenty or more generations, or by parent x offspring matings carried out with certain restrictions. All animals within an inbred strain trace back to a common ancestor in the twentieth generation.'),
  'D008816': NtO(MH='Mice, Jimpy', MS='Myelin-deficient mutants which are from the inbred Tabby-Jimpy strain.'),
  'D008817': NtO(MH='Mice, Mutant Strains', MS='Mice bearing mutant genes which are phenotypically expressed in the animals.'),
  'D008818': NtO(MH='Mice, Neurologic Mutants', MS='Mice which carry mutant genes for neurologic defects or abnormalities.'),
  'D008819': NtO(MH='Mice, Nude', MS='Mutant mice homozygous for the recessive gene "nude" which fail to develop a thymus. They are useful in tumor studies and studies on immune responses.'),
  'D008820': NtO(MH='Mice, Obese', MS='Mutant mice exhibiting a marked obesity coupled with overeating, hyperglycemia, hyperinsulinemia, marked insulin resistance, and infertility when in a homozygous state. They may be inbred or hybrid.'),
  'D008821': NtO(MH='Mice, Quaking', MS='Mice homozygous for the mutant autosomal recessive gene, quaking (qk), associated with disorder in myelin formation and manifested by axial tremors.'),
  'D008822': NtO(MH='Mice, Transgenic', MS='Laboratory mice that have been produced from a genetically manipulated EGG or EMBRYO, MAMMALIAN.'),
  'D008823': NtO(MH='Micelles', MS='Particles consisting of aggregates of molecules held loosely together by secondary bonds. The surface of micelles are usually comprised of amphiphatic compounds that are oriented in a way that minimizes the energy of interaction between the micelle and its environment. Liquids that contain large numbers of suspended micelles are referred to as EMULSIONS.'),
  'D008824': NtO(MH='Michigan', MS='State bounded on the north by the Great Lakes, on the east by Canada, on the south by Wisconsin, Indiana, and Ohio, and on the west by Lake Michigan and Wisconsin.'),
  'D008825': NtO(MH='Miconazole', MS='An imidazole antifungal agent that is used topically and by intravenous infusion.'),
  'D008826': NtO(MH='Microbial Sensitivity Tests', MS='Any tests that demonstrate the relative efficacy of different chemotherapeutic agents against specific microorganisms (i.e., bacteria, fungi, viruses).'),
  'D008827': NtO(MH='Microbiological Phenomena', MS='Physiological processes and properties of microorganisms, such as ARCHAEA; BACTERIA; RICKETTSIA; VIRUSES; and FUNGI.'),
  'D008828': NtO(MH='Microbiological Techniques', MS='Techniques used in microbiology.'),
  'D008829': NtO(MH='Microbiology', MS='The study of microorganisms such as fungi, bacteria, algae, archaea, and viruses.'),
  'D008830': NtO(MH='Microbodies', MS='Electron-dense cytoplasmic particles bounded by a single membrane, such as PEROXISOMES; GLYOXYSOMES; and glycosomes.'),
  'D008831': NtO(MH='Microcephaly', MS='A congenital abnormality in which the CEREBRUM is underdeveloped, the fontanels close prematurely, and, as a result, the head is small. (Desk Reference for Neuroscience, 2nd ed.)'),
  'D008832': NtO(MH='Microchemistry', MS='The development and use of techniques and equipment to study or perform chemical reactions, with small quantities of materials, frequently less than a milligram or a milliliter.'),
  'D008833': NtO(MH='Microcirculation', MS='The circulation of the BLOOD through the MICROVASCULAR NETWORK.'),
  'D008834': NtO(MH='Microclimate', MS='The climate of a very small area.'),
  'D008835': NtO(MH='Micrococcaceae', MS='A family of bacteria ranging from free living and saprophytic to parasitic and pathogenic forms.'),
  'D008836': NtO(MH='Micrococcal Nuclease', MS="An enzyme that catalyzes the endonucleolytic cleavage to 3'-phosphomononucleotide and 3'-phospholigonucleotide end-products. It can cause hydrolysis of double- or single-stranded DNA or RNA. (From Enzyme Nomenclature, 1992) EC 3.1.31.1."),
  'D008837': NtO(MH='Micrococcus', MS='A genus of gram-positive, spherical bacteria found in soils and fresh water, and frequently on the skin of man and other animals.'),
  'D008838': NtO(MH='Microcomputers', MS='Small computers using LSI (large-scale integration) microprocessor chips as the CPU (central processing unit) and semiconductor memories for compact, inexpensive storage of program instructions and data. They are smaller and less expensive than minicomputers and are usually built into a dedicated system where they are optimized for a particular application. "Microprocessor" may refer to just the CPU or the entire microcomputer.'),
  'D008839': NtO(MH='Microelectrodes', MS='Electrodes with an extremely small tip, used in a voltage clamp or other apparatus to stimulate or record bioelectric potentials of single cells intracellularly or extracellularly. (Dorland, 28th ed)'),
  'D008840': NtO(MH='Microfilament Proteins', MS='Monomeric subunits of primarily globular ACTIN and found in the cytoplasmic matrix of almost all cells. They are often associated with microtubules and may play a role in cytoskeletal function and/or mediate movement of the cell or the organelles within the cell.'),
  'D008841': NtO(MH='Actin Cytoskeleton', MS='Fibers composed of MICROFILAMENT PROTEINS, which are predominately ACTIN. They are the smallest of the cytoskeletal filaments.'),
  'D008842': NtO(MH='Microfilariae', MS='The prelarval stage of Filarioidea found in the blood and tissues of mammals, birds and intermediate hosts (vector).'),
  'D008843': NtO(MH='Microfilming', MS='Process of capturing very reduced images for preservation and storage.'),
  'D008844': NtO(MH='Micrognathism', MS='Abnormally small jaw.'),
  'D008845': NtO(MH='Microinjections', MS='The injection of very small amounts of fluid, often with the aid of a microscope and microsyringes.'),
  'D008846': NtO(MH='Micromanipulation', MS='The performance of dissections, injections, surgery, etc., by the use of micromanipulators (attachments to a microscope) that manipulate tiny instruments.'),
  'D008847': NtO(MH='Micromonospora', MS='A genus of gram-positive bacteria that forms a branched mycelium. It commonly occurs as a saprophytic form in soil and aquatic environments.'),
  'D008848': NtO(MH='Micromonosporaceae', MS='A family of gram-positive, saprophytic bacteria occurring in soil and aquatic environments.'),
  'D008849': NtO(MH='Micronesia', MS="The collective name for islands of the Pacific Ocean east of the Philippines, including the Mariana, PALAU, Caroline, Marshall, and Kiribati Islands. (From Webster's New Geographical Dictionary, 1988, p761 & Room, Brewer's Dictionary of Names, 1992, p350)"),
  'D008850': NtO(MH='Microphthalmos', MS='Congenital or developmental anomaly in which the eyeballs are abnormally small.'),
  'D008851': NtO(MH='Micropore Filters', MS='A membrane or barrier with micrometer sized pores used for separation purification processes.'),
  'D008852': NtO(MH='Microradiography', MS='Production of a radiographic image of a small or very thin object on fine-grained photographic film under conditions which permit subsequent microscopic examination or enlargement of the radiograph at linear magnifications of up to several hundred and with a resolution approaching the resolving power of the photographic emulsion (about 1000 lines per millimeter).'),
  'D008853': NtO(MH='Microscopy', MS='The use of instrumentation and techniques for visualizing material and details that cannot be seen by the unaided eye. It is usually done by enlarging images, transmitted by light or electron beams, with optical or magnetic lenses that magnify the entire image field. With scanning microscopy, images are generated by collecting output from the specimen in a point-by-point fashion, on a magnified scale, as it is scanned by a narrow beam of light or electrons, a laser, a conductive probe, or a topographical probe.'),
  'D008854': NtO(MH='Microscopy, Electron', MS='Microscopy using an electron beam, instead of light, to visualize the sample, thereby allowing much greater magnification. The interactions of ELECTRONS with specimens are used to provide information about the fine structure of that specimen. In TRANSMISSION ELECTRON MICROSCOPY the reactions of the electrons that are transmitted through the specimen are imaged. In SCANNING ELECTRON MICROSCOPY an electron beam falls at a non-normal angle on the specimen and the image is derived from the reactions occurring above the plane of the specimen.'),
  'D008855': NtO(MH='Microscopy, Electron, Scanning', MS='Microscopy in which the object is examined directly by an electron beam scanning the specimen point-by-point. The image is constructed by detecting the products of specimen interactions that are projected above the plane of the sample, such as backscattered electrons. Although SCANNING TRANSMISSION ELECTRON MICROSCOPY also scans the specimen point by point with the electron beam, the image is constructed by detecting the electrons, or their interaction products that are transmitted through the sample plane, so that is a form of TRANSMISSION ELECTRON MICROSCOPY.'),
  'D008856': NtO(MH='Microscopy, Fluorescence', MS='Microscopy of specimens stained with fluorescent dye (usually fluorescein isothiocyanate) or of naturally fluorescent materials, which emit light when exposed to ultraviolet or blue light. Immunofluorescence microscopy utilizes antibodies that are labeled with fluorescent dye.'),
  'D008857': NtO(MH='Microscopy, Interference', MS='The science and application of a double-beam transmission interference microscope in which the illuminating light beam is split into two paths. One beam passes through the specimen while the other beam reflects off a reference mirror before joining and interfering with the other. The observed optical path difference between the two beams can be measured and used to discriminate minute differences in thickness and refraction of non-stained transparent specimens, such as living cells in culture.'),
  'D008858': NtO(MH='Microscopy, Phase-Contrast', MS='A form of interference microscopy in which variations of the refracting index in the object are converted into variations of intensity in the image. This is achieved by the action of a phase plate.'),
  'D008859': NtO(MH='Microscopy, Polarization', MS='Microscopy using polarized light in which phenomena due to the preferential orientation of optical properties with respect to the vibration plane of the polarized light are made visible and correlated parameters are made measurable.'),
  'D008860': NtO(MH='Microscopy, Ultraviolet', MS='Microscopy in which the image is formed by ultraviolet radiation and is displayed and recorded by means of photographic film.'),
  'D008861': NtO(MH='Microsomes', MS='Artifactual vesicles formed from the endoplasmic reticulum when cells are disrupted. They are isolated by differential centrifugation and are composed of three structural features: rough vesicles, smooth vesicles, and ribosomes. Numerous enzyme activities are associated with the microsomal fraction. (Glick, Glossary of Biochemistry and Molecular Biology, 1990; from Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D008862': NtO(MH='Microsomes, Liver', MS='Closed vesicles of fragmented endoplasmic reticulum created when liver cells or tissue are disrupted by homogenization. They may be smooth or rough.'),
  'D008863': NtO(MH='Microspheres', MS='Small uniformly-sized spherical particles, of micrometer dimensions, frequently labeled with radioisotopes or various reagents acting as tags or markers.'),
  'D008864': NtO(MH='Microsporum', MS='A mitosporic Oxygenales fungal genus causing various diseases of the skin and hair. The species Microsporum canis produces TINEA CAPITIS and tinea corporis, which usually are acquired from domestic cats and dogs. Teleomorphs includes Arthroderma (Nannizzia). (Alexopoulos et al., Introductory Mycology, 4th edition, p305)'),
  'D008865': NtO(MH='Microstomia', MS='A congenital defect in which the mouth is unusually small. (Dorland, 27th ed)'),
  'D008866': NtO(MH='Microsurgery', MS='The performance of surgical procedures with the aid of a microscope.'),
  'D008867': NtO(MH='Microtomy', MS='The technique of using a microtome to cut thin or ultrathin sections of tissues embedded in a supporting substance. The microtome is an instrument that hold a steel, glass or diamond knife in clamps at an angle to the blocks of prepared tissues, which it cuts in sections of equal thickness.'),
  'D008868': NtO(MH='Microtubule Proteins', MS='Proteins found in the microtubules.'),
  'D008869': NtO(MH='Microtubule-Associated Proteins', MS='High molecular weight proteins found in the MICROTUBULES of the cytoskeletal system. Under certain conditions they are required for TUBULIN assembly into the microtubules and stabilize the assembled microtubules.'),
  'D008870': NtO(MH='Microtubules', MS='Slender, cylindrical filaments found in the cytoskeleton of plant and animal cells. They are composed of the protein TUBULIN and are influenced by TUBULIN MODULATORS.'),
  'D008871': NtO(MH='Microvilli', MS='Minute projections of cell membranes which greatly increase the surface area of the cell.'),
  'D008872': NtO(MH='Microwaves', MS='That portion of the electromagnetic spectrum from the UHF (ultrahigh frequency) radio waves and extending into the INFRARED RAYS frequencies.'),
  'D008873': NtO(MH='Mid-Atlantic Region', MS='A geographical area of the United States comprising the District of Columbia, Delaware, Maryland, New Jersey, New York, and Pennsylvania.'),
  'D008874': NtO(MH='Midazolam', MS='A short-acting hypnotic-sedative drug with anxiolytic and amnestic properties. It is used in dentistry, cardiac surgery, endoscopic procedures, as preanesthetic medication, and as an adjunct to local anesthesia. The short duration and cardiorespiratory stability makes it useful in poor-risk, elderly, and cardiac patients. It is water-soluble at pH less than 4 and lipid-soluble at physiological pH.'),
  'D008875': NtO(MH='Middle Aged', MS='An adult aged 45 - 64 years.'),
  'D008876': NtO(MH='Middle Ear Ventilation', MS='Ventilation of the middle ear in the treatment of secretory (serous) OTITIS MEDIA, usually by placement of tubes or grommets which pierce the TYMPANIC MEMBRANE.'),
  'D008877': NtO(MH='Middle East', MS="The region of southwest Asia and northeastern Africa usually considered as extending from Libya on the west to Afghanistan on the east. (From Webster's New Geographical Dictionary, 1988)"),
  'D008878': NtO(MH='Middle Lobe Syndrome', MS='Atelectasis of the right middle pulmonary lobe, with chronic pneumonitis. (Dorland, 27th ed)'),
  'D008879': NtO(MH='Midodrine', MS='An ethanolamine derivative that is an adrenergic alpha-1 agonist. It is used as a vasoconstrictor agent in the treatment of HYPOTENSION.'),
  'D008880': NtO(MH='Midwifery', MS='The practice of assisting women in childbirth.'),
  'D008881': NtO(MH='Migraine Disorders', MS='A class of disabling primary headache disorders, characterized by recurrent unilateral pulsatile headaches. The two major subtypes are common migraine (without aura) and classic migraine (with aura or neurological symptoms). (International Classification of Headache Disorders, 2nd ed. Cephalalgia 2004: suppl 1)'),
  'D008882': NtO(MH="Mikulicz' Disease", MS="A chronic, benign, and usually painless inflammatory swelling of the lacrimal and salivary glands. It is considered by some to include the glandular enlargement associated with other diseases, such as Sjogren's syndrome, sarcoidosis, lupus erythematosus, etc."),
  'D008883': NtO(MH='Miliaria', MS='A syndrome of cutaneous changes associated with sweat retention and extravasation of sweat at different levels in the skin. Miliaria rubra, or prickly heat, results from apocrine duct obstruction. The sweat then seeps into the epidermis, producing pruritic erythematous papulovesicles. (From Dorland, 27th ed)'),
  'D008884': NtO(MH='Milieu Therapy', MS="A treatment program based on manipulation of the patient's environment by the medical staff. The patient does not participate in planning the treatment regimen."),
  'D008885': NtO(MH='Military Dentistry', MS='The practice of dentistry as applied to special circumstances associated with military operations.'),
  'D008886': NtO(MH='Military Hygiene', MS='Science dealing with the maintenance of health in servicemen.'),
  'D008887': NtO(MH='Military Medicine', MS='The practice of medicine as applied to special circumstances associated with military operations.'),
  'D008888': NtO(MH='Military Nursing', MS='The practice of nursing in military environments.'),
  'D008889': NtO(MH='Military Personnel', MS='Persons including soldiers involved with the armed forces.'),
  'D008890': NtO(MH='Military Psychiatry', MS='Branch of psychiatry concerned with problems related to the prevention, diagnosis, etiology, and treatment of mental or emotional disorders of Armed Forces personnel.'),
  'D008891': NtO(MH='Military Science', MS='Study of the causes as well as tactical and strategic principles of WARFARE.'),
  'D008892': NtO(MH='Milk', MS='The white liquid secreted by the mammary glands. It contains proteins, sugar, lipids, vitamins, and minerals.'),
  'D008893': NtO(MH='Milk Ejection', MS='Expulsion of milk from the mammary alveolar lumen, which is surrounded by a layer of milk-secreting EPITHELIAL CELLS and a network of myoepithelial cells. Contraction of the myoepithelial cells is regulated by neuroendocrine signals.'),
  'D008894': NtO(MH='Milk Proteins', MS='The major protein constituents of milk are CASEINS and whey proteins such as LACTALBUMIN and LACTOGLOBULINS. IMMUNOGLOBULINS occur in high concentrations in COLOSTRUM and in relatively lower concentrations in milk. (Singleton and Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed, p554)'),
  'D008895': NtO(MH='Milk, Human', MS='Milk that is produced by HUMAN MAMMARY GLANDS.'),
  'D008897': NtO(MH='Panicum', MS='A plant genus of the family POACEAE. The seed is one of the EDIBLE GRAINS used in millet cereals and in feed for birds and livestock (ANIMAL FEED). It contains diosgenin (SAPONINS).'),
  'D008898': NtO(MH='Mimosine', MS='3-Hydroxy-4-oxo-1(4H)-pyridinealanine. An antineoplastic alanine-substituted pyridine derivative isolated from Leucena glauca.'),
  'D008899': NtO(MH='Mineral Oil', MS='A mixture of liquid hydrocarbons obtained from petroleum. It is used as laxative, lubricant, ointment base, and emollient.'),
  'D008900': NtO(MH='Mineral Waters', MS='Water naturally or artificially infused with mineral salts or gases.'),
  'D008901': NtO(MH='Mineralocorticoids', MS='A group of CORTICOSTEROIDS primarily associated with water and electrolyte balance. This is accomplished through the effect on ION TRANSPORT in renal tubules, resulting in retention of sodium and loss of potassium. Mineralocorticoid secretion is itself regulated by PLASMA VOLUME, serum potassium, and ANGIOTENSIN II.'),
  'D008903': NtO(MH='Minerals', MS="Native, inorganic or fossilized organic substances having a definite chemical composition and formed by inorganic reactions. They may occur as individual crystals or may be disseminated in some other mineral or rock. (Grant & Hackh's Chemical Dictionary, 5th ed; McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)"),
  'D008904': NtO(MH='Miniaturization', MS='The design or construction of objects greatly reduced in scale.'),
  'D008905': NtO(MH='Minicomputers', MS='Small computers that lack the speed, memory capacity, and instructional capability of the full-size computer but usually retain its programmable flexibility. They are larger, faster, and more flexible, powerful, and expensive than microcomputers.'),
  'D008906': NtO(MH='Mining', MS='The process of extracting mineral deposits from the earth.'),
  'D008907': NtO(MH='Mink', MS='Carnivores of genus Mustela of the family MUSTELIDAE. The European mink, which has white upper and lower lips, was widely trapped for commercial purposes and is classified as endangered. The American mink, lacking a white upper lip, is farmed commercially.'),
  'D008908': NtO(MH='Mink Cell Focus-Inducing Viruses', MS='Strains of MURINE LEUKEMIA VIRUS discovered in 1976 by Hartley, Wolford, Old, and Rowe and so named because the viruses originally isolated had the capacity to transform cell foci in mink cell cultures. MCF viruses are generated by recombination with ecotropic murine leukemia viruses including AKR, Friend, Moloney, and Rauscher, causing ERYTHROLEUKEMIA and severe anemia in mice.'),
  'D008910': NtO(MH='Minnesota', MS='State bordered on the north by Canada, on the east by Lake Superior and Wisconsin, on the south by Iowa, and on the west by North Dakota and South Dakota.'),
  'D008911': NtO(MH='Minocycline', MS='A TETRACYCLINE analog, having a 7-dimethylamino and lacking the 5 methyl and hydroxyl groups, which is effective against tetracycline-resistant STAPHYLOCOCCUS infections.'),
  'D008912': NtO(MH='Minor Histocompatibility Loci', MS='Genetic loci responsible for the encoding of histocompatibility antigens other than those encoded by the MAJOR HISTOCOMPATIBILITY COMPLEX. The antigens encoded by these genes are often responsible for graft rejection in cases where histocompatibility has been established by standard tests. The location of some of these loci on the X and Y chromosomes explains why grafts from males to females may be rejected while grafts from females to males are accepted. In the mouse roughly 30 minor histocompatibility loci have been recognized, comprising more than 500 genes.'),
  'D008913': NtO(MH='Minority Groups', MS='A subgroup having special characteristics within a larger group, often bound together by special ties which distinguish it from the larger group.'),
  'D008914': NtO(MH='Minoxidil', MS='A potent direct-acting peripheral vasodilator (VASODILATOR AGENTS) that reduces peripheral resistance and produces a fall in BLOOD PRESSURE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p371)'),
  'D008915': NtO(MH='Minute Virus of Mice', MS='The type species of PARVOVIRUS prevalent in mouse colonies and found as a contaminant of many transplanted tumors or leukemias.'),
  'D008916': NtO(MH='Miotics', MS='Agents causing contraction of the pupil of the eye. Some sources use the term miotics only for the parasympathomimetics but any drug used to induce miosis is included here.'),
  'D008917': NtO(MH='Mirex', MS='An organochlorine insecticide that is carcinogenic.'),
  'D008919': NtO(MH='Investigative Techniques', MS='Investigative techniques used in pre-clinical and clinical research, epidemiology, chemistry, immunology, genetics, etc. They do not include techniques specifically applied to DIAGNOSIS; THERAPEUTICS; anesthesia and analgesia; SURGICAL PROCEDURES, OPERATIVE; and DENTISTRY.'),
  'D008920': NtO(MH='Misonidazole', MS='A nitroimidazole that sensitizes normally radio-resistant hypoxic cells to radiation. It may also be directly cytotoxic to hypoxic cells and has been proposed as an antineoplastic.'),
  'D008921': NtO(MH='Religious Missions', MS='Programs or projects established by religious groups or institution to support and provide missionary activities, including sharing faith or providing educational, medical, or other assistance.'),
  'D008922': NtO(MH='Mississippi', MS='State bounded on the north by Tennessee, on the east by Alabama, on the south by Louisiana and Gulf of Mexico, and on the west by Arkansas and Louisiana.'),
  'D008923': NtO(MH='Missouri', MS='State bounded on the east by Illinois and Kentucky, on the south by Arkansas, on the west by Kansas, and on the north by Iowa.'),
  'D008924': NtO(MH='Mite Infestations', MS='Infestations with arthropods of the subclass ACARI, superorder Acariformes.'),
  'D008925': NtO(MH='Mites', MS='Any arthropod of the subclass ACARI except the TICKS. They are minute animals related to the spiders, usually having transparent or semitransparent bodies. They may be parasitic on humans and domestic animals, producing various irritations of the skin (MITE INFESTATIONS). Many mite species are important to human and veterinary medicine as both parasite and vector. Mites also infest plants.'),
  'D008926': NtO(MH='Plicamycin', MS='A tricyclic pentaglycosidic antibiotic from Streptomyces strains that inhibits RNA and protein synthesis by adhering to DNA. It is used as a fluorescent dye and as an antineoplastic agent, especially in bone and testicular tumors. Plicamycin is also used to reduce hypercalcemia, especially that due to malignancies.'),
  'D008927': NtO(MH='Mitobronitol', MS='Brominated analog of MANNITOL which is an antineoplastic agent appearing to act as an alkylating agent.'),
  'D008928': NtO(MH='Mitochondria', MS='Semiautonomous, self-reproducing organelles that occur in the cytoplasm of all cells of most, but not all, eukaryotes. Each mitochondrion is surrounded by a double limiting membrane. The inner membrane is highly invaginated, and its projections are called cristae. Mitochondria are the sites of the reactions of oxidative phosphorylation, which result in the formation of ATP. They contain distinctive RIBOSOMES, transfer RNAs (RNA, TRANSFER); AMINO ACYL T RNA SYNTHETASES; and elongation and termination factors. Mitochondria depend upon genes within the nucleus of the cells in which they reside for many essential messenger RNAs (RNA, MESSENGER). Mitochondria are believed to have arisen from aerobic bacteria that established a symbiotic relationship with primitive protoeukaryotes. (King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D008929': NtO(MH='Mitochondria, Heart', MS='The mitochondria of the myocardium.'),
  'D008930': NtO(MH='Mitochondria, Liver', MS='Mitochondria in hepatocytes. As in all mitochondria, there are an outer membrane and an inner membrane, together creating two separate mitochondrial compartments: the internal matrix space and a much narrower intermembrane space. In the liver mitochondrion, an estimated 67% of the total mitochondrial proteins is located in the matrix. (From Alberts et al., Molecular Biology of the Cell, 2d ed, p343-4)'),
  'D008931': NtO(MH='Mitochondria, Muscle', MS='Mitochondria of skeletal and smooth muscle. It does not include myocardial mitochondria for which MITOCHONDRIA, HEART is available.'),
  'D008933': NtO(MH='Mitochondrial Swelling', MS='An increase in MITOCHONDRIAL VOLUME due to an influx of fluid; it occurs in hypotonic solutions due to osmotic pressure and in isotonic solutions as a result of altered permeability of the membranes of respiring mitochondria.'),
  'D008934': NtO(MH='Mitogens', MS='Substances that stimulate mitosis and lymphocyte transformation. They include not only substances associated with LECTINS, but also substances from streptococci (associated with streptolysin S) and from strains of alpha-toxin-producing staphylococci. (Stedman, 25th ed)'),
  'D008935': NtO(MH='Mitoguazone', MS='Antineoplastic agent effective against myelogenous leukemia in experimental animals. Also acts as an inhibitor of animal S-adenosylmethionine decarboxylase.'),
  'D008936': NtO(MH='Mitolactol', MS='Alkylating antineoplastic toxic to bone marrow; used in breast cancer, also in combination with other drugs.'),
  'D008937': NtO(MH='Mitomycins', MS='A group of methylazirinopyrroloindolediones obtained from certain Streptomyces strains. They are very toxic antibiotics used as ANTINEOPLASTIC AGENTS in some solid tumors. PORFIROMYCIN and MITOMYCIN are the most useful members of the group.'),
  'D008938': NtO(MH='Mitosis', MS='A type of CELL NUCLEUS division by means of which the two daughter nuclei normally receive identical complements of the number of CHROMOSOMES of the somatic cells of the species.'),
  'D008939': NtO(MH='Mitotane', MS='A derivative of the insecticide DICHLORODIPHENYLDICHLOROETHANE that specifically inhibits cells of the adrenal cortex and their production of hormones. It is used to treat adrenocortical tumors and causes CNS damage, but no bone marrow depression.'),
  'D008940': NtO(MH='Mitotic Index', MS='An expression of the number of mitoses found in a stated number of cells.'),
  'D008941': NtO(MH='Spindle Apparatus', MS='A microtubule structure that forms during CELL DIVISION. It consists of two SPINDLE POLES, and sets of MICROTUBULES that may include the astral microtubules, the polar microtubules, and the kinetochore microtubules.'),
  'D008942': NtO(MH='Mitoxantrone', MS='An anthracenedione-derived antineoplastic agent.'),
  'D008943': NtO(MH='Mitral Valve', MS='The valve between the left atrium and left ventricle of the heart.'),
  'D008944': NtO(MH='Mitral Valve Insufficiency', MS='Backflow of blood from the LEFT VENTRICLE into the LEFT ATRIUM due to imperfect closure of the MITRAL VALVE. This can lead to mitral valve regurgitation.'),
  'D008945': NtO(MH='Mitral Valve Prolapse', MS='Abnormal protrusion or billowing of one or both of the leaflets of MITRAL VALVE into the LEFT ATRIUM during SYSTOLE. This allows the backflow of blood into left atrium leading to MITRAL VALVE INSUFFICIENCY; SYSTOLIC MURMURS; or CARDIAC ARRHYTHMIA.'),
  'D008946': NtO(MH='Mitral Valve Stenosis', MS='Narrowing of the passage through the MITRAL VALVE due to FIBROSIS, and CALCINOSIS in the leaflets and chordal areas. This elevates the left atrial pressure which, in turn, raises pulmonary venous and capillary pressure leading to bouts of DYSPNEA and TACHYCARDIA during physical exertion. RHEUMATIC FEVER is its primary cause.'),
  'D008947': NtO(MH='Mixed Connective Tissue Disease', MS='A syndrome with overlapping clinical features of systemic lupus erythematosus, scleroderma, polymyositis, and Raynaud\'s phenomenon. The disease is differentially characterized by high serum titers of antibodies to ribonuclease-sensitive extractable (saline soluble) nuclear antigen and a "speckled" epidermal nuclear staining pattern on direct immunofluorescence.'),
  'D008949': NtO(MH='Adenoma, Pleomorphic', MS='A benign, slow-growing tumor, most commonly of the salivary gland, occurring as a small, painless, firm nodule, usually of the parotid gland, but also found in any major or accessory salivary gland anywhere in the oral cavity. It is most often seen in women in the fifth decade. Histologically, the tumor presents a variety of cells: cuboidal, columnar, and squamous cells, showing all forms of epithelial growth. (Dorland, 27th ed)'),
  'D008950': NtO(MH='MMPI', MS='A personality inventory consisting of statements to be asserted or denied by the individual. The patterns of response are characteristic of certain personality attributes.'),
  'D008951': NtO(MH='MNSs Blood-Group System', MS='A system of universal human blood group isoantigens with many associated subgroups. The M and N traits are codominant and the S and s traits are probably very closely linked alleles, including the U antigen. This system is most frequently used in paternity studies.'),
  'D008952': NtO(MH='Mobile Health Units', MS='Movable or portable facilities in which diagnostic and therapeutic services are provided to the community.'),
  'D008953': NtO(MH='Models, Anatomic', MS='Three-dimensional representation to show anatomic structures. Models may be used in place of intact animals or organisms for teaching, practice, and study.'),
  'D008954': NtO(MH='Models, Biological', MS='Theoretical representations that simulate the behavior or activity of biological processes or diseases. For disease models in living animals, DISEASE MODELS, ANIMAL is available. Biological models include the use of mathematical equations, computers, and other electronic equipment.'),
  'D008955': NtO(MH='Models, Cardiovascular', MS='Theoretical representations that simulate the behavior or activity of the cardiovascular system, processes, or phenomena; includes the use of mathematical equations, computers and other electronic equipment.'),
  'D008956': NtO(MH='Models, Chemical', MS='Theoretical representations that simulate the behavior or activity of chemical processes or phenomena; includes the use of mathematical equations, computers, and other electronic equipment.'),
  'D008957': NtO(MH='Models, Genetic', MS='Theoretical representations that simulate the behavior or activity of genetic processes or phenomena. They include the use of mathematical equations, computers, and other electronic equipment.'),
  'D008958': NtO(MH='Models, Molecular', MS='Models used experimentally or theoretically to study molecular shape, electronic properties, or interactions; includes analogous molecules, computer-generated graphics, and mechanical structures.'),
  'D008959': NtO(MH='Models, Neurological', MS='Theoretical representations that simulate the behavior or activity of the neurological system, processes or phenomena; includes the use of mathematical equations, computers, and other electronic equipment.'),
  'D008960': NtO(MH='Models, Psychological', MS='Theoretical representations that simulate psychological processes and/or social processes. These include the use of mathematical equations, computers, and other electronic equipment.'),
  'D008961': NtO(MH='Models, Structural', MS='A representation, generally small in scale, to show the structure, construction, or appearance of something. (From Random House Unabridged Dictionary, 2d ed)'),
  'D008962': NtO(MH='Models, Theoretical', MS='Theoretical representations that simulate the behavior or activity of systems, processes, or phenomena. They include the use of mathematical equations, computers, and other electronic equipment.'),
  'D008963': NtO(MH='Molar', MS='The most posterior teeth on either side of the jaw, totaling eight in the deciduous dentition (2 on each side, upper and lower), and usually 12 in the permanent dentition (three on each side, upper and lower). They are grinding teeth, having large crowns and broad chewing surfaces. (Jablonski, Dictionary of Dentistry, 1992, p821)'),
  'D008964': NtO(MH='Molar, Third', MS='The aftermost permanent tooth on each side in the maxilla and mandible.'),
  'D008965': NtO(MH='Molasses', MS='The syrup remaining after sugar is crystallized out of SUGARCANE or sugar beet juice. It is also used in ANIMAL FEED, and in a fermented form, is used to make industrial ETHYL ALCOHOL and ALCOHOLIC BEVERAGES.'),
  'D008966': NtO(MH='Moldova', MS='Country in Europe south and west of Ukraine and east of Romania. The capital is Chisinau.'),
  'D008967': NtO(MH='Molecular Biology', MS='A discipline concerned with studying biological phenomena in terms of the chemical and physical interactions of molecules.'),
  'D008968': NtO(MH='Molecular Conformation', MS='The characteristic three-dimensional shape of a molecule.'),
  'D008969': NtO(MH='Molecular Sequence Data', MS='Descriptions of specific amino acid, carbohydrate, or nucleotide sequences which have appeared in the published literature and/or are deposited in and maintained by databanks such as GENBANK, European Molecular Biology Laboratory (EMBL), National Biomedical Research Foundation (NBRF), or other sequence repositories.'),
  'D008970': NtO(MH='Molecular Weight', MS='The sum of the weight of all the atoms in a molecule.'),
  'D008971': NtO(MH='Moles', MS='Any of numerous burrowing mammals found in temperate regions and having minute eyes often covered with skin.'),
  'D008972': NtO(MH='Molindone', MS='An indole derivative effective in schizophrenia and other psychoses and possibly useful in the treatment of the aggressive type of undersocialized conduct disorder. Molindone has much lower affinity for D2 receptors than most antipsychotic agents and has a relatively low affinity for D1 receptors. It has only low to moderate affinity for cholinergic and alpha-adrenergic receptors. Some electrophysiologic data from animals indicate that molindone has certain characteristics that resemble those of CLOZAPINE. (From AMA Drug Evaluations Annual, 1994, p283)'),
  'D008973': NtO(MH='Tenericutes', MS='A phylum of gram-negative bacteria consisting of cells bounded by a plasma membrane. Its organisms differ from other bacteria in that they are devoid of cell walls. This phylum was formerly the class Mollicutes. Mollicutes is now the sole class in the phylum Tenericutes.'),
  'D008974': NtO(MH='Mollusca', MS='A phylum of the kingdom Metazoa. Mollusca have soft, unsegmented bodies with an anterior head, a dorsal visceral mass, and a ventral foot. Most are encased in a protective calcareous shell. It includes the classes GASTROPODA; BIVALVIA; CEPHALOPODA; Aplacophora; Scaphopoda; Polyplacophora; and Monoplacophora.'),
  'D008975': NtO(MH='Molluscacides', MS='Agents destructive to snails and other mollusks.'),
  'D008976': NtO(MH='Molluscum Contagiosum', MS='A common, benign, usually self-limited viral infection of the skin and occasionally the conjunctivae by a poxvirus (MOLLUSCUM CONTAGIOSUM VIRUS). (Dorland, 27th ed)'),
  'D008977': NtO(MH='Molluscum contagiosum virus', MS='A species of MOLLUSCIPOXVIRUS causing skin lesions in humans. It is transmitted by direct contact or from non-living reservoirs (fomites), such as books or clothing.'),
  'D008978': NtO(MH='Mollusk Venoms', MS='Venoms from mollusks, including CONUS and OCTOPUS species. The venoms contain proteins, enzymes, choline derivatives, slow-reacting substances, and several characterized polypeptide toxins that affect the nervous system. Mollusk venoms include cephalotoxin, venerupin, maculotoxin, surugatoxin, conotoxins, and murexine.'),
  'D008979': NtO(MH='Moloney murine leukemia virus', MS="A strain of Murine leukemia virus (LEUKEMIA VIRUS, MURINE) arising during the propagation of S37 mouse sarcoma, and causing lymphoid leukemia in mice. It also infects rats and newborn hamsters. It is apparently transmitted to embryos in utero and to newborns through mother's milk."),
  'D008980': NtO(MH='Moloney murine sarcoma virus', MS='A replication-defective murine sarcoma virus (SARCOMA VIRUSES, MURINE) isolated from a rhabdomyosarcoma by Moloney in 1966.'),
  'D008981': NtO(MH='Molsidomine', MS='A morpholinyl sydnone imine ethyl ester, having a nitrogen in place of the keto oxygen. It acts as NITRIC OXIDE DONORS and is a vasodilator that has been used in ANGINA PECTORIS.'),
  'D008982': NtO(MH='Molybdenum', MS='A metallic element with the atomic symbol Mo, atomic number 42, and atomic weight 95.95. It is an essential trace element, being a component of the enzymes xanthine oxidase, aldehyde oxidase, and nitrate reductase.'),
  'D008983': NtO(MH='Molybdoferredoxin', MS='A non-heme iron-sulfur protein isolated from Clostridium pasteurianum and other bacteria. It is a component of NITROGENASE, which is active in nitrogen fixation, and consists of two subunits with molecular weights of 59.5 kDa and 50.7 kDa, respectively.'),
  'D008984': NtO(MH='Monaco', MS='European Principality bordering the Mediterranean Sea on the southern coast of France, near the border with Italy.'),
  'D008985': NtO(MH='Monensin', MS="An antiprotozoal agent produced by Streptomyces cinnamonensis. It exerts its effect during the development of first-generation trophozoites into first-generation schizonts within the intestinal epithelial cells. It does not interfere with hosts' development of acquired immunity to the majority of coccidial species. Monensin is a sodium and proton selective ionophore and is widely used as such in biochemical studies."),
  'D008986': NtO(MH='Mongolia', MS='The country is bordered by RUSSIA on the north and CHINA on the west, south, and east. The capita is Ulaanbaatar.'),
  'D008988': NtO(MH='Herpestidae', MS='The family of agile, keen-sighted mongooses of Asia and Africa that feed on RODENTS and SNAKES.'),
  'D008989': NtO(MH='Monieziasis', MS='Infection of ruminants with tapeworms of the genus Moniezia.'),
  'D008990': NtO(MH='Moniliformis', MS='A genus of roundworms of the phylum Acanthocephala, parasitic in rats, mice, hamsters, dogs and cats. Occasional infection in man produces inflammation and ulceration of the intestinal mucosa.'),
  'D008991': NtO(MH='Monitoring, Physiologic', MS='The continuous measurement of physiological processes, blood pressure, heart rate, renal output, reflexes, respiration, etc., in a patient or experimental animal; includes pharmacologic monitoring, the measurement of administered drugs or their metabolites in the blood, tissues, or urine.'),
  'D008992': NtO(MH='Monkey Diseases', MS='Diseases of Old World and New World monkeys. This term includes diseases of baboons but not of chimpanzees or gorillas (= APE DISEASES).'),
  'D008993': NtO(MH='Monkeypox virus', MS='A species of ORTHOPOXVIRUS causing an epidemic disease among captive primates.'),
  'D008994': NtO(MH='Monoacylglycerol Lipases', MS='An enzyme that catalyzes the hydrolysis of glycerol monoesters of long-chain fatty acids EC 3.1.1.23.'),
  'D008995': NtO(MH='Monoamine Oxidase', MS="An enzyme that catalyzes the oxidative deamination of naturally occurring monoamines. It is a flavin-containing enzyme that is localized in mitochondrial membranes, whether in nerve terminals, the liver, or other organs. Monoamine oxidase is important in regulating the metabolic degradation of catecholamines and serotonin in neural or target tissues. Hepatic monoamine oxidase has a crucial defensive role in inactivating circulating monoamines or those, such as tyramine, that originate in the gut and are absorbed into the portal circulation. (From Goodman and Gilman's, The Pharmacological Basis of Therapeutics, 8th ed, p415) EC 1.4.3.4."),
  'D008996': NtO(MH='Monoamine Oxidase Inhibitors', MS="A chemically heterogeneous group of drugs that have in common the ability to block oxidative deamination of naturally occurring monoamines. (From Gilman, et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed, p414)"),
  'D008997': NtO(MH='Monobactams', MS='Monocyclic, bacterially produced or semisynthetic beta-lactam antibiotics. They lack the double ring construction of the traditional beta-lactam antibiotics and can be easily synthesized.'),
  'D008998': NtO(MH='Monoclonal Gammopathy of Undetermined Significance', MS='Conditions characterized by the presence of M protein (Monoclonal protein) in serum or urine without clinical manifestations of plasma cell dyscrasia.'),
  'D008999': NtO(MH='Monocrotophos', MS='An organophosphate insecticide that inhibits monoamine oxidase and acetylcholinesterase. It has been shown to be genotoxic.'),
  'D009000': NtO(MH='Monocytes', MS='Large, phagocytic mononuclear leukocytes produced in the vertebrate BONE MARROW and released into the BLOOD; contain a large, oval or somewhat indented nucleus surrounded by voluminous cytoplasm and numerous organelles.'),
  'D009002': NtO(MH='Monomethylhydrazine', MS='Hydrazine substituted by one methyl group.'),
  'D009004': NtO(MH='Monosaccharide Transport Proteins', MS='A large group of membrane transport proteins that shuttle MONOSACCHARIDES across CELL MEMBRANES.'),
  'D009005': NtO(MH='Monosaccharides', MS='Single chain carbohydrates that are the most basic units of CARBOHYDRATES. They are typically colorless crystalline substances with a sweet taste and have the same general formula CnH2nOn.'),
  'D009006': NtO(MH='Monosomy', MS='The condition in which one chromosome of a pair is missing. In a normally diploid cell it is represented symbolically as 2N-1.'),
  'D009007': NtO(MH='Monotremata', MS='An order of egg-laying mammals that includes the PLATYPUS and ECHIDNA.'),
  'D009008': NtO(MH='Abnormalities, Severe Teratoid', MS='Marked developmental anomalies of a fetus or infant.'),
  'D009009': NtO(MH='Montana', MS='State bounded on the north by Canada, on the east by North Dakota and South Dakota, on the south by Idaho and Wyoming, and on the west by Idaho and Utah.'),
  'D009010': NtO(MH='Monte Carlo Method', MS='In statistics, a technique for numerically approximating the solution of a mathematical problem by studying the distribution of some random variable, often generated by a computer. The name alludes to the randomness characteristic of the games of chance played at the gambling casinos in Monte Carlo. (From Random House Unabridged Dictionary, 2d ed, 1993)'),
  'D009011': NtO(MH="Monteggia's Fracture", MS='Fracture in the proximal half of the shaft of the ulna, with dislocation of the head of the radius.'),
  'D009012': NtO(MH='Mopidamol', MS='A phosphodiesterase inhibitor which inhibits platelet aggregation. Formerly used as an antineoplastic.'),
  'D009013': NtO(MH='Morale', MS='The prevailing temper or spirit of an individual or group in relation to the tasks or functions which are expected.'),
  'D009014': NtO(MH='Morals', MS='Standards of conduct that distinguish right from wrong.'),
  'D009015': NtO(MH='Morantel', MS='Antinematodal agent used mainly for livestock.'),
  'D009016': NtO(MH='Moraxella', MS='A genus of gram-negative, aerobic bacteria occurring as rods (subgenus Moraxella) or cocci (subgenus Branhamella). Its organisms are parasitic on the mucous membranes of humans and other warm-blooded animals.'),
  'D009017': NtO(MH='Morbidity', MS='The proportion of patients with a particular disease during a given year per given unit of population.'),
  'D009018': NtO(MH='Morocco', MS='A country located in north Africa, bordering the Atlantic Ocean and the Mediterranean Sea, with a southern border with Western Sahara, eastern border with Algeria. The capital is Rabat.'),
  'D009019': NtO(MH='Morphinans', MS='Compounds based on a partially saturated iminoethanophenanthrene, which can be described as ethylimino-bridged benzo-decahydronaphthalenes. They include some of the OPIOIDS found in PAPAVER that are used as ANALGESICS.'),
  'D009020': NtO(MH='Morphine', MS='The principal alkaloid in opium and the prototype opiate analgesic and narcotic. Morphine has widespread effects in the central nervous system and on smooth muscle.'),
  'D009021': NtO(MH='Morphine Dependence', MS='Strong dependence, both physiological and emotional, upon morphine.'),
  'D009022': NtO(MH='Morphine Derivatives', MS='Analogs or derivatives of morphine.'),
  'D009024': NtO(MH='Morphogenesis', MS='The development of anatomical structures to create the form of a single- or multi-cell organism. Morphogenesis provides form changes of a part, parts, or the whole organism.'),
  'D009025': NtO(MH='Morpholines', MS=None),
  'D009026': NtO(MH='Mortality', MS='All deaths reported in a given population.'),
  'D009027': NtO(MH='Mortuary Practice', MS='Activities associated with the disposition of the dead. It excludes cultural practices such as FUNERAL RITES.'),
  'D009028': NtO(MH='Morula', MS='An early embryo that is a compact mass of about 16 BLASTOMERES. It resembles a cluster of mulberries with two types of cells, outer cells and inner cells. Morula is the stage before BLASTULA in non-mammalian animals or a BLASTOCYST in mammals.'),
  'D009029': NtO(MH='Mosaic Viruses', MS='Viruses which produce a mottled appearance of the leaves of plants.'),
  'D009030': NtO(MH='Mosaicism', MS='The occurrence in an individual of two or more cell populations of different chromosomal constitutions, derived from a single ZYGOTE, as opposed to CHIMERISM in which the different cell populations are derived from more than one zygote.'),
  'D009031': NtO(MH='Moscow', MS='The capital of Russia.'),
  'D009032': NtO(MH='Mosquito Control', MS='The reduction or regulation of the population of mosquitoes through chemical, biological, or other means.'),
  'D009033': NtO(MH='Culicidae', MS='A family of the order DIPTERA that comprises the mosquitoes. The larval stages are aquatic, and the adults can be recognized by the characteristic WINGS, ANIMAL venation, the scales along the wing veins, and the long proboscis. Many species are of particular medical importance.'),
  'D009034': NtO(MH='Mother-Child Relations', MS='Interaction between a mother and child.'),
  'D009035': NtO(MH='Mothers', MS='Female parents, human or animal.'),
  'D009036': NtO(MH='Moths', MS='Insects of the suborder Heterocera of the order LEPIDOPTERA.'),
  'D009037': NtO(MH='Motilin', MS='A peptide of about 22-amino acids isolated from the DUODENUM. At low pH it inhibits gastric motor activity, whereas at high pH it has a stimulating effect.'),
  'D009038': NtO(MH='Motion', MS='Physical motion, i.e., a change in position of a body or subject as a result of an external force. It is distinguished from MOVEMENT, a process resulting from biological activity.'),
  'D009039': NtO(MH='Motion Perception', MS='The real or apparent movement of objects through the visual field.'),
  'D009040': NtO(MH='Motion Pictures', MS='The art, technique, or business of producing motion pictures for entertainment, propaganda, or instruction.'),
  'D009041': NtO(MH='Motion Sickness', MS='Disorder caused by motion. It includes sea sickness, train sickness, roller coaster rides, rocking chair, hammock swing, car sickness, air sickness, or SPACE MOTION SICKNESS. Symptoms include nausea, vomiting and/or dizziness.'),
  'D009042': NtO(MH='Motivation', MS='Those factors which cause an organism to behave or act in either a goal-seeking or satisfying manner. They may be influenced by physiological drives or by external stimuli.'),
  'D009043': NtO(MH='Motor Activity', MS='Body movements of a human or an animal as a behavioral phenomenon.'),
  'D009044': NtO(MH='Motor Cortex', MS='Area of the FRONTAL LOBE concerned with primary motor control located in the dorsal PRECENTRAL GYRUS immediately anterior to the central sulcus. It is comprised of three areas: the primary motor cortex located on the anterior paracentral lobule on the medial surface of the brain; the premotor cortex located anterior to the primary motor cortex; and the supplementary motor area located on the midline surface of the hemisphere anterior to the primary motor cortex.'),
  'D009045': NtO(MH='Motor Endplate', MS='The specialized postsynaptic region of a muscle cell. The motor endplate is immediately across the synaptic cleft from the presynaptic axon terminal. Among its anatomical specializations are junctional folds which harbor a high density of cholinergic receptors.'),
  'D009046': NtO(MH='Motor Neurons', MS='Neurons which activate MUSCLE CELLS.'),
  'D009047': NtO(MH='Motor Neurons, Gamma', MS='Motor neurons which activate the contractile regions of intrafusal SKELETAL MUSCLE FIBERS, thus adjusting the sensitivity of the MUSCLE SPINDLES to stretch. Gamma motor neurons may be "static" or "dynamic" according to which aspect of responsiveness (or which fiber types) they regulate. The alpha and gamma motor neurons are often activated together (alpha gamma coactivation) which allows the spindles to contribute to the control of movement trajectories despite changes in muscle length.'),
  'D009048': NtO(MH='Motor Skills', MS='Performance of complex motor acts.'),
  'D009049': NtO(MH='Motorcycles', MS='Two-wheeled, engine-driven vehicles.'),
  'D009050': NtO(MH='Fluorosis, Dental', MS="A chronic endemic form of hypoplasia of the dental enamel caused by drinking water with a high fluorine content during the time of tooth formation, and characterized by defective calcification that gives a white chalky appearance to the enamel, which gradually undergoes brown discoloration. (Jablonski's Dictionary of Dentistry, 1992, p286)"),
  'D009051': NtO(MH='Mountaineering', MS='A sport involving mountain climbing techniques.'),
  'D009052': NtO(MH='Leukemia Virus, Murine', MS='Species of GAMMARETROVIRUS, containing many well-defined strains, producing leukemia in mice. Disease is commonly induced by injecting filtrates of propagable tumors into newborn mice.'),
  'D009053': NtO(MH='Sarcoma Viruses, Murine', MS='A group of replication-defective viruses, in the genus GAMMARETROVIRUS, which are capable of transforming cells, but which replicate and produce tumors only in the presence of Murine leukemia viruses (LEUKEMIA VIRUS, MURINE).'),
  'D009055': NtO(MH='Mouth', MS='The oval-shaped oral cavity located at the apex of the digestive tract and consisting of two parts: the vestibule and the oral cavity proper.'),
  'D009056': NtO(MH='Mouth Abnormalities', MS='Congenital absence of or defects in structures of the mouth.'),
  'D009057': NtO(MH='Stomatognathic Diseases', MS='General or unspecified diseases of the stomatognathic system, comprising the mouth, teeth, jaws, and pharynx.'),
  'D009058': NtO(MH='Mouth Breathing', MS='Abnormal breathing through the mouth, usually associated with obstructive disorders of the nasal passages.'),
  'D009059': NtO(MH='Mouth Diseases', MS='Diseases involving the MOUTH.'),
  'D009060': NtO(MH='Mouth Floor', MS='The area of the mouth beneath the TONGUE.'),
  'D009061': NtO(MH='Mouth Mucosa', MS='Lining of the ORAL CAVITY, including mucosa on the GUMS; the PALATE; the LIP; the CHEEK; floor of the mouth; and other structures. The mucosa is generally a nonkeratinized stratified squamous EPITHELIUM covering muscle, bone, or glands but can show varying degree of keratinization at specific locations.'),
  'D009062': NtO(MH='Mouth Neoplasms', MS='Tumors or cancer of the MOUTH.'),
  'D009063': NtO(MH='Dental Physiological Phenomena', MS='Physiological processes and properties of the DENTITION.'),
  'D009064': NtO(MH='Mouth Protectors', MS='Devices or pieces of equipment placed in or around the mouth or attached to instruments to protect the external or internal tissues of the mouth and the teeth.'),
  'D009065': NtO(MH='Mouth Rehabilitation', MS='Process of restoring damaged or decayed teeth using various restorative and non-cosmetic materials so that oral health is improved.'),
  'D009066': NtO(MH='Mouth, Edentulous', MS='Total lack of teeth through disease or extraction.'),
  'D009067': NtO(MH='Mouthwashes', MS="Solutions for rinsing the mouth, possessing cleansing, germicidal, or palliative properties. (From Boucher's Clinical Dental Terminology, 4th ed)"),
  'D009068': NtO(MH='Movement', MS='The act, process, or result of passing from one place or position to another. It differs from LOCOMOTION in that locomotion is restricted to the passing of the whole body from one place to another, while movement encompasses both locomotion but also a change of the position of the whole body or any of its parts. Movement may be used with reference to humans, vertebrate and invertebrate animals, and microorganisms. Differentiate also from MOTOR ACTIVITY, movement associated with behavior.'),
  'D009069': NtO(MH='Movement Disorders', MS='Syndromes which feature DYSKINESIAS as a cardinal manifestation of the disease process. Included in this category are degenerative, hereditary, post-infectious, medication-induced, post-inflammatory, and post-traumatic conditions.'),
  'D009070': NtO(MH='Moxalactam', MS='Broad- spectrum beta-lactam antibiotic similar in structure to the CEPHALOSPORINS except for the substitution of an oxaazabicyclo moiety for the thiaazabicyclo moiety of certain CEPHALOSPORINS. It has been proposed especially for the meningitides because it passes the blood-brain barrier and for anaerobic infections.'),
  'D009071': NtO(MH='Moxibustion', MS='The burning of a small, thimble sized, smoldering plug of dried leaves on the SKIN at an ACUPUNCTURE point. Usually the plugs contain leaves of MUGWORT or moxa.'),
  'D009072': NtO(MH='Moyamoya Disease', MS='A noninflammatory, progressive occlusion of the intracranial CAROTID ARTERIES and the formation of netlike collateral arteries arising from the CIRCLE OF WILLIS. Cerebral angiogram shows the puff-of-smoke (moyamoya) collaterals at the base of the brain. It is characterized by endothelial HYPERPLASIA and FIBROSIS with thickening of arterial walls. This disease primarily affects children but can also occur in adults.'),
  'D009073': NtO(MH='Mozambique', MS='A republic in southern Africa, south of TANZANIA, east of ZAMBIA and ZIMBABWE, bordered on the west by the Indian Ocean. Its capital is Maputo. It was formerly called Portuguese East Africa.'),
  'D009074': NtO(MH='Melanocyte-Stimulating Hormones', MS='Peptides with the ability to stimulate pigmented cells MELANOCYTES in mammals and MELANOPHORES in lower vertebrates. By stimulating the synthesis and distribution of MELANIN in these pigmented cells, they increase coloration of skin and other tissue. MSHs, derived from pro-opiomelanocortin (POMC), are produced by MELANOTROPHS in the INTERMEDIATE LOBE OF PITUITARY; CORTICOTROPHS in the ANTERIOR LOBE OF PITUITARY, and the hypothalamic neurons in the ARCUATE NUCLEUS OF HYPOTHALAMUS.'),
  'D009075': NtO(MH='MSH Release-Inhibiting Hormone', MS='A hypothalamic tripeptide, enzymatic degradation product of OXYTOCIN, that inhibits the release of MELANOCYTE-STIMULATING HORMONES.'),
  'D009077': NtO(MH='Mucins', MS='High molecular weight mucoproteins that protect the surface of EPITHELIAL CELLS by providing a barrier to particulate matter and microorganisms. Membrane-anchored mucins may have additional roles concerned with protein interactions at the cell surface.'),
  'D009078': NtO(MH='Mucocele', MS='A retention cyst of the salivary gland, lacrimal sac, paranasal sinuses, appendix, or gallbladder. (Stedman, 26th ed)'),
  'D009079': NtO(MH='Mucociliary Clearance', MS='A non-specific host defense mechanism that removes MUCUS and other material from the LUNGS by ciliary and secretory activity of the tracheobronchial submucosal glands. It is measured in vivo as mucus transfer, ciliary beat frequency, and clearance of radioactive tracers.'),
  'D009080': NtO(MH='Mucocutaneous Lymph Node Syndrome', MS='An acute, febrile, mucocutaneous condition accompanied by swelling of cervical lymph nodes in infants and young children. The principal symptoms are fever, congestion of the ocular conjunctivae, reddening of the lips and oral cavity, protuberance of tongue papillae, and edema or erythema of the extremities.'),
  'D009081': NtO(MH='Mucolipidoses', MS='A group of inherited metabolic diseases characterized by the accumulation of excessive amounts of acid mucopolysaccharides, sphingolipids, and/or glycolipids in visceral and mesenchymal cells. Abnormal amounts of sphingolipids or glycolipids are present in neural tissue. INTELLECTUAL DISABILITY and skeletal changes, most notably dysostosis multiplex, occur frequently. (From Joynt, Clinical Neurology, 1992, Ch56, pp36-7)'),
  'D009083': NtO(MH='Mucopolysaccharidoses', MS='Group of lysosomal storage diseases each caused by an inherited deficiency of an enzyme involved in the degradation of glycosaminoglycans (mucopolysaccharides). The diseases are progressive and often display a wide spectrum of clinical severity within one enzyme deficiency.'),
  'D009084': NtO(MH='Mucopolysaccharidosis III', MS='Mucopolysaccharidosis characterized by heparitin sulfate in the urine, progressive mental retardation, mild dwarfism, and other skeletal disorders. There are four clinically indistinguishable but biochemically distinct forms, each due to a deficiency of a different enzyme.'),
  'D009085': NtO(MH='Mucopolysaccharidosis IV', MS='Genetic disorder of mucopolysaccharide metabolism characterized by skeletal abnormalities, joint instability, development of cervical myelopathy, and excessive urinary keratan sulfate. There are two biochemically distinct forms, each due to a deficiency of a different enzyme.'),
  'D009087': NtO(MH='Mucopolysaccharidosis VI', MS='Mucopolysaccharidosis with excessive CHONDROITIN SULFATE B in urine, characterized by dwarfism and deafness. It is caused by a deficiency of N-ACETYLGALACTOSAMINE-4-SULFATASE (arylsulfatase B).'),
  'D009088': NtO(MH='Mucoproteins', MS='Conjugated proteins in which mucopolysaccharides are combined with proteins. The mucopolysaccharide moiety is the predominant group with the protein making up only a small percentage of the total weight.'),
  'D009089': NtO(MH='Mucor', MS='A genus of zygomycetous fungi of the family Mucoraceae, order Mucorales. It is primarily saprophytic, but may cause MUCORMYCOSIS in man from spores germinating in the lungs.'),
  'D009090': NtO(MH='Mucorales', MS='An order of zygomycetous fungi, usually saprophytic, causing damage to food in storage, but which may cause respiratory infection or MUCORMYCOSIS in persons suffering from other debilitating diseases.'),
  'D009091': NtO(MH='Mucormycosis', MS='Infection in humans and animals caused by any fungus in the order Mucorales (e.g., Absidia, Mucor, Rhizopus etc.) There are many clinical types associated with infection of the central nervous system, lung, gastrointestinal tract, skin, orbit and paranasal sinuses. In humans, it usually occurs as an opportunistic infection in patients with a chronic debilitating disease, particularly uncontrolled diabetes, or who are receiving immunosuppressive agents. (From Dorland, 28th ed)'),
  'D009092': NtO(MH='Mucous Membrane', MS='An EPITHELIUM with MUCUS-secreting cells, such as GOBLET CELLS. It forms the lining of many body cavities, such as the DIGESTIVE TRACT, the RESPIRATORY TRACT, and the reproductive tract. Mucosa, rich in blood and lymph vessels, comprises an inner epithelium, a middle layer (lamina propria) of loose CONNECTIVE TISSUE, and an outer layer (muscularis mucosae) of SMOOTH MUSCLE CELLS that separates the mucosa from submucosa.'),
  'D009093': NtO(MH='Mucus', MS='The viscous secretion of mucous membranes. It contains mucin, white blood cells, water, inorganic salts, and exfoliated cells.'),
  'D009094': NtO(MH='Mud Therapy', MS='The therapeutic use of mud in packs or baths taking advantage of the absorptive qualities of the mud. It has been used for rheumatism and skin problems.'),
  'D009095': NtO(MH='Mullerian Ducts', MS='A pair of ducts near the WOLFFIAN DUCTS in a developing embryo. In the male embryo, they degenerate with the appearance of testicular ANTI-MULLERIAN HORMONE. In the absence of anti-mullerian hormone, mullerian ducts give rise to the female reproductive tract, including the OVIDUCTS; UTERUS; CERVIX; and VAGINA.'),
  'D009096': NtO(MH='Multi-Institutional Systems', MS='Institutional systems consisting of more than one health facility which have cooperative administrative arrangements through merger, affiliation, shared services, or other collective ventures.'),
  'D009097': NtO(MH='Multienzyme Complexes', MS='Systems of enzymes which function sequentially by catalyzing consecutive reactions linked by common metabolic intermediates. They may involve simply a transfer of water molecules or hydrogen atoms and may be associated with large supramolecular structures such as MITOCHONDRIA or RIBOSOMES.'),
  'D009098': NtO(MH='Multiphasic Screening', MS='The simultaneous use of multiple laboratory procedures for the detection of various diseases. These are usually performed on groups of people.'),
  'D009099': NtO(MH='Multiple Birth Offspring', MS='The offspring in multiple pregnancies (PREGNANCY, MULTIPLE): TWINS; TRIPLETS; QUADRUPLETS; QUINTUPLETS; etc.'),
  'D009100': NtO(MH='Multiple Carboxylase Deficiency', MS='A deficiency in the activities of biotin-dependent enzymes (propionyl-CoA carboxylase, methylcrotonyl-CoA carboxylase, and PYRUVATE CARBOXYLASE) due to one of two defects in BIOTIN metabolism. The neonatal form is due to HOLOCARBOXYLASE SYNTHETASE DEFICIENCY. The late-onset form is due to BIOTINIDASE DEFICIENCY.'),
  'D009101': NtO(MH='Multiple Myeloma', MS='A malignancy of mature PLASMA CELLS engaging in monoclonal immunoglobulin production. It is characterized by hyperglobulinemia, excess Bence-Jones proteins (free monoclonal IMMUNOGLOBULIN LIGHT CHAINS) in the urine, skeletal destruction, bone pain, and fractures. Other features include ANEMIA; HYPERCALCEMIA; and RENAL INSUFFICIENCY.'),
  'D009102': NtO(MH='Multiple Organ Failure', MS='A progressive condition usually characterized by combined failure of several organs such as the lungs, liver, kidney, along with some clotting mechanisms, usually postinjury or postoperative.'),
  'D009103': NtO(MH='Multiple Sclerosis', MS='An autoimmune disorder mainly affecting young adults and characterized by destruction of myelin in the central nervous system. Pathologic findings include multiple sharply demarcated areas of demyelination throughout the white matter of the central nervous system. Clinical manifestations include visual loss, extra-ocular movement disorders, paresthesias, loss of sensation, weakness, dysarthria, spasticity, ataxia, and bladder dysfunction. The usual pattern is one of recurrent attacks followed by partial recovery (see MULTIPLE SCLEROSIS, RELAPSING-REMITTING), but acute fulminating and chronic progressive forms (see MULTIPLE SCLEROSIS, CHRONIC PROGRESSIVE) also occur. (Adams et al., Principles of Neurology, 6th ed, p903)'),
  'D009104': NtO(MH='Multiple Trauma', MS='Multiple physical insults or injuries occurring simultaneously.'),
  'D009105': NtO(MH='Dissociative Identity Disorder', MS='A dissociative disorder in which the individual adopts two or more distinct personalities. Each personality is a fully integrated and complex unit with memories, behavior patterns and social friendships. Transition from one personality to another is sudden.'),
  'D009106': NtO(MH='Mummies', MS='Bodies preserved either by the ancient Egyptian technique or due to chance under favorable climatic conditions.'),
  'D009107': NtO(MH='Mumps', MS='An acute infectious disease caused by RUBULAVIRUS, spread by direct contact, airborne droplet nuclei, fomites contaminated by infectious saliva, and perhaps urine, and usually seen in children under the age of 15, although adults may also be affected. (From Dorland, 28th ed)'),
  'D009108': NtO(MH='Mumps Vaccine', MS='Vaccines used to prevent infection by MUMPS VIRUS. Best known is the live attenuated virus vaccine of chick embryo origin, used for routine immunization of children and for immunization of adolescents and adults who have not had mumps or been immunized with live mumps vaccine. Children are usually immunized with measles-mumps-rubella combination vaccine.'),
  'D009109': NtO(MH='Mumps virus', MS='The type species of RUBULAVIRUS that causes an acute infectious disease in humans, affecting mainly children. Transmission occurs by droplet infection.'),
  'D009110': NtO(MH='Munchausen Syndrome', MS='A factitious disorder characterized by habitual presentation for hospital treatment of an apparent acute illness, the patient giving a plausible and dramatic history, all of which is false.'),
  'D009111': NtO(MH='Local Government', MS='Smallest political subdivisions within a country at which general governmental functions are carried-out.'),
  'D009112': NtO(MH='Muramic Acids', MS='Compounds consisting of glucosamine and lactate joined by an ether linkage. They occur naturally as N-acetyl derivatives in peptidoglycan, the characteristic polysaccharide composing bacterial cell walls. (From Dorland, 28th ed)'),
  'D009113': NtO(MH='Muramidase', MS='A basic enzyme that is present in saliva, tears, egg white, and many animal fluids. It functions as an antibacterial agent. The enzyme catalyzes the hydrolysis of 1,4-beta-linkages between N-acetylmuramic acid and N-acetyl-D-glucosamine residues in peptidoglycan and between N-acetyl-D-glucosamine residues in chitodextrin. EC 3.2.1.17.'),
  'D009114': NtO(MH='Murexide', MS="5,5'-Nitrilodibarbituric acid ammonium derivative. Used as an indicator for complexometric titrations."),
  'D009115': NtO(MH='Muridae', MS='A family of the order Rodentia containing 250 genera including the two genera Mus (MICE) and Rattus (RATS), from which the laboratory inbred strains are developed. The fifteen subfamilies are SIGMODONTINAE (New World mice and rats), CRICETINAE, Spalacinae, Myospalacinae, Lophiomyinae, ARVICOLINAE, Platacanthomyinae, Nesomyinae, Otomyinae, Rhizomyinae, GERBILLINAE, Dendromurinae, Cricetomyinae, MURINAE (Old World mice and rats), and Hydromyinae.'),
  'D009116': NtO(MH='Muscarine', MS='A toxic alkaloid found in Amanita muscaria (fly fungus) and other fungi of the Inocybe species. It is the first parasympathomimetic substance ever studied and causes profound parasympathetic activation that may end in convulsions and death. The specific antidote is atropine.'),
  'D009117': NtO(MH='Muscidae', MS='A family of the order DIPTERA with over 700 species. Important species that may be mechanical vectors of disease include Musca domesticus (HOUSEFLIES), Musca autumnalis (face fly), Stomoxys calcitrans (stable fly), Haematobia irritans (horn fly) and Fannia spp.'),
  'D009118': NtO(MH='Muscimol', MS='A neurotoxic isoxazole isolated from species of AMANITA. It is obtained by decarboxylation of IBOTENIC ACID. Muscimol is a potent agonist of GABA-A RECEPTORS and is used mainly as an experimental tool in animal and tissue studies.'),
  'D009119': NtO(MH='Muscle Contraction', MS='A process leading to shortening and/or development of tension in muscle tissue. Muscle contraction occurs by a sliding filament mechanism whereby actin filaments slide inward among the myosin filaments.'),
  'D009120': NtO(MH='Muscle Cramp', MS='A sustained and usually painful contraction of muscle fibers. This may occur as an isolated phenomenon or as a manifestation of an underlying disease process (e.g., UREMIA; HYPOTHYROIDISM; MOTOR NEURON DISEASE; etc.). (From Adams et al., Principles of Neurology, 6th ed, p1398)'),
  'D009121': NtO(MH='Muscle Denervation', MS='The resection or removal of the innervation of a muscle or muscle tissue.'),
  'D009122': NtO(MH='Muscle Hypertonia', MS='Abnormal increase in skeletal or smooth muscle tone. Skeletal muscle hypertonicity may be associated with PYRAMIDAL TRACT lesions or BASAL GANGLIA DISEASES.'),
  'D009123': NtO(MH='Muscle Hypotonia', MS='A diminution of the skeletal muscle tone marked by a diminished resistance to passive stretching.'),
  'D009124': NtO(MH='Muscle Proteins', MS='The protein constituents of muscle, the major ones being ACTINS and MYOSINS. More than a dozen accessory proteins exist including TROPONIN; TROPOMYOSIN; and DYSTROPHIN.'),
  'D009125': NtO(MH='Muscle Relaxants, Central', MS='A heterogeneous group of drugs used to produce muscle relaxation, excepting the neuromuscular blocking agents. They have their primary clinical and therapeutic uses in the treatment of muscle spasm and immobility associated with strains, sprains, and injuries of the back and, to a lesser degree, injuries to the neck. They have been used also for the treatment of a variety of clinical conditions that have in common only the presence of skeletal muscle hyperactivity, for example, the muscle spasms that can occur in MULTIPLE SCLEROSIS. (From Smith and Reynard, Textbook of Pharmacology, 1991, p358)'),
  'D009126': NtO(MH='Muscle Relaxation', MS='That phase of a muscle twitch during which a muscle returns to a resting position.'),
  'D009127': NtO(MH='Muscle Rigidity', MS='Continuous involuntary sustained muscle contraction which is often a manifestation of BASAL GANGLIA DISEASES. When an affected muscle is passively stretched, the degree of resistance remains constant regardless of the rate at which the muscle is stretched. This feature helps to distinguish rigidity from MUSCLE SPASTICITY. (From Adams et al., Principles of Neurology, 6th ed, p73)'),
  'D009128': NtO(MH='Muscle Spasticity', MS='A form of muscle hypertonia associated with upper MOTOR NEURON DISEASE. Resistance to passive stretch of a spastic muscle results in minimal initial resistance (a "free interval") followed by an incremental increase in muscle tone. Tone increases in proportion to the velocity of stretch. Spasticity is usually accompanied by HYPERREFLEXIA and variable degrees of MUSCLE WEAKNESS. (From Adams et al., Principles of Neurology, 6th ed, p54)'),
  'D009129': NtO(MH='Muscle Tonus', MS='The state of activity or tension of a muscle beyond that related to its physical properties, that is, its active resistance to stretch. In skeletal muscle, tonus is dependent upon efferent innervation. (Stedman, 25th ed)'),
  'D009130': NtO(MH='Muscle, Smooth', MS='Unstriated and unstriped muscle, one of the muscles of the internal organs, blood vessels, hair follicles, etc. Contractile elements are elongated, usually spindle-shaped cells with centrally located nuclei. Smooth muscle fibers are bound together into sheets or bundles by reticular fibers and frequently elastic nets are also abundant. (From Stedman, 25th ed)'),
  'D009131': NtO(MH='Muscle, Smooth, Vascular', MS='The nonstriated involuntary muscle tissue of blood vessels.'),
  'D009132': NtO(MH='Muscles', MS='Contractile tissue that produces movement in animals.'),
  'D009133': NtO(MH='Muscular Atrophy', MS='Derangement in size and number of muscle fibers occurring with aging, reduction in blood supply, or following immobilization, prolonged weightlessness, malnutrition, and particularly in denervation.'),
  'D009134': NtO(MH='Muscular Atrophy, Spinal', MS='A group of disorders marked by progressive degeneration of motor neurons in the spinal cord resulting in weakness and muscular atrophy, usually without evidence of injury to the corticospinal tracts. Diseases in this category include Werdnig-Hoffmann disease and later onset SPINAL MUSCULAR ATROPHIES OF CHILDHOOD, most of which are hereditary. (Adams et al., Principles of Neurology, 6th ed, p1089)'),
  'D009135': NtO(MH='Muscular Diseases', MS='Acquired, familial, and congenital disorders of SKELETAL MUSCLE and SMOOTH MUSCLE.'),
  'D009136': NtO(MH='Muscular Dystrophies', MS='A heterogeneous group of inherited MYOPATHIES, characterized by wasting and weakness of the SKELETAL MUSCLE. They are categorized by the sites of MUSCLE WEAKNESS; AGE OF ONSET; and INHERITANCE PATTERNS.'),
  'D009137': NtO(MH='Muscular Dystrophy, Animal', MS='MUSCULAR DYSTROPHY that occurs in VERTEBRATE animals.'),
  'D009138': NtO(MH='Musculocutaneous Nerve', MS='A major nerve of the upper extremity. The fibers of the musculocutaneous nerve originate in the lower cervical spinal cord (usually C5 to C7), travel via the lateral cord of the brachial plexus, and supply sensory and motor innervation to the upper arm, elbow, and forearm.'),
  'D009139': NtO(MH='Musculoskeletal Abnormalities', MS='Congenital structural abnormalities and deformities of the musculoskeletal system.'),
  'D009140': NtO(MH='Musculoskeletal Diseases', MS='Diseases of the muscles and their associated ligaments and other connective tissue and of the bones and cartilage viewed collectively.'),
  'D009141': NtO(MH='Musculoskeletal System', MS='The MUSCLES, bones (BONE AND BONES), and CARTILAGE of the body.'),
  'D009142': NtO(MH='Musculoskeletal Physiological Phenomena', MS='Processes and properties of the MUSCULOSKELETAL SYSTEM.'),
  'D009144': NtO(MH='Museums', MS='Facility for the procurement, care, study, and display of objects of scholarly interest or value.'),
  'D009145': NtO(MH='Mushroom Poisoning', MS='Poisoning from ingestion of mushrooms, primarily from, but not restricted to, toxic varieties.'),
  'D009146': NtO(MH='Music', MS='Sound that expresses emotion through rhythm, melody, and harmony.'),
  'D009147': NtO(MH='Music Therapy', MS='The use of music as an adjunctive therapy in the treatment of neurological, mental, or behavioral disorders.'),
  'D009149': NtO(MH='Mustard Plant', MS='Any of several BRASSICA species that are commonly called mustard. Brassica juncea is brown or Chinese mustard, and Brassica nigra is black, brown, or red mustard. The plant is grown both for mustard seed from which oil is extracted or used as SPICES, and for its greens used as VEGETABLES or ANIMAL FEED. There is no relationship to MUSTARD COMPOUNDS. A species from a related genus SINAPIS ALBA is white mustard.'),
  'D009150': NtO(MH='Mustard Compounds', MS='Strong alkylating and immunosuppressive agents whose biological activity is based on the presence of bis(2-chloroethyl)- groups. Although otherwise structurally diverse, the compounds have in common the capacity to contribute alkyl groups to DNA. They are generally highly toxic but include among their number many widely used and effective antineoplastic agents.'),
  'D009151': NtO(MH='Mustard Gas', MS='Severe irritant and vesicant of skin, eyes, and lungs. It may cause blindness and lethal lung edema and was formerly used as a war gas. The substance has been proposed as a cytostatic and for treatment of psoriasis. It has been listed as a known carcinogen in the Fourth Annual Report on Carcinogens (NTP-85-002, 1985) (Merck, 11th ed).'),
  'D009152': NtO(MH='Mutagenicity Tests', MS='Tests of chemical substances and physical agents for mutagenic potential. They include microbial, insect, mammalian cell, and whole animal tests.'),
  'D009153': NtO(MH='Mutagens', MS='Chemical agents that increase the rate of genetic mutation by interfering with the function of nucleic acids. A clastogen is a specific mutagen that causes breaks in chromosomes.'),
  'D009154': NtO(MH='Mutation', MS='Any detectable and heritable change in the genetic material that causes a change in the GENOTYPE and which is transmitted to daughter cells and to succeeding generations.'),
  'D009155': NtO(MH='Mutism', MS='The inability to generate oral-verbal expression, despite normal comprehension of speech. This may be associated with BRAIN DISEASES or MENTAL DISORDERS. Organic mutism may be associated with damage to the FRONTAL LOBE; BRAIN STEM; THALAMUS; and CEREBELLUM. Selective mutism is a psychological condition that usually affects children characterized by continuous refusal to speak in social situations by a child who is able and willing to speak to selected persons. Kussmal aphasia refers to mutism in psychosis. (From Fortschr Neurol Psychiatr 1994; 62(9):337-44)'),
  'D009156': NtO(MH='Muzolimine', MS='A pyrazole diuretic with long duration and high capacity of action. It was proposed for kidney failure and hypertension but was withdrawn worldwide because of severe neurological effects.'),
  'D009157': NtO(MH='Myasthenia Gravis', MS='A disorder of neuromuscular transmission characterized by fatigable weakness of cranial and skeletal muscles with elevated titers of ACETYLCHOLINE RECEPTORS or muscle-specific receptor tyrosine kinase (MuSK) autoantibodies. Clinical manifestations may include ocular muscle weakness (fluctuating, asymmetric, external ophthalmoplegia; diplopia; ptosis; and weakness of eye closure) and extraocular fatigable weakness of facial, bulbar, respiratory, and proximal limb muscles. The disease may remain limited to the ocular muscles (ocular myasthenia). THYMOMA is commonly associated with this condition.'),
  'D009158': NtO(MH='Mycobacillin', MS='A cyclic polypeptide antibiotic isolated from culture filtrates of Bacillus subtilis that acts as an antifungal agent.'),
  'D009159': NtO(MH='Mycobacteriaceae', MS='A family of gram-positive bacteria found in soil and dairy products and as parasites on animals and man. Several are important pathogens.'),
  'D009160': NtO(MH='Mycobacteriophages', MS='Viruses whose host is one or more Mycobacterium species. They include both temperate and virulent types.'),
  'D009161': NtO(MH='Mycobacterium', MS='A genus of gram-positive, aerobic bacteria. Most species are free-living in soil and water, but the major habitat for some is the diseased tissue of warm-blooded hosts.'),
  'D009162': NtO(MH='Mycobacterium avium', MS='A bacterium causing tuberculosis in domestic fowl and other birds. In pigs, it may cause localized and sometimes disseminated disease. The organism occurs occasionally in sheep and cattle. It should be distinguished from the M. avium complex, which infects primarily humans.'),
  'D009163': NtO(MH='Mycobacterium bovis', MS='The bovine variety of the tubercle bacillus. It is called also Mycobacterium tuberculosis var. bovis.'),
  'D009164': NtO(MH='Mycobacterium Infections', MS='Infections with bacteria of the genus MYCOBACTERIUM.'),
  'D009165': NtO(MH='Mycobacterium Infections, Nontuberculous', MS='Infections with nontuberculous mycobacteria (atypical mycobacteria): M. kansasii, M. marinum, M. scrofulaceum, M. flavescens, M. gordonae, M. obuense, M. gilvum, M. duvali, M. szulgai, M. intracellulare (see MYCOBACTERIUM AVIUM COMPLEX;), M. xenopi (littorale), M. ulcerans, M. buruli, M. terrae, M. fortuitum (minetti, giae), M. chelonae, M. leprae.'),
  'D009166': NtO(MH='Mycobacterium leprae', MS='A species of gram-positive, aerobic bacteria that causes LEPROSY in man. Its organisms are generally arranged in clumps, rounded masses, or in groups of bacilli side by side.'),
  'D009167': NtO(MH='Mycobacterium lepraemurium', MS='The etiologic agent of rat leprosy, also known as murine leprosy.'),
  'D009168': NtO(MH='Mycobacterium phlei', MS='A saprophytic bacterium widely distributed in soil and dust and on plants.'),
  'D009169': NtO(MH='Mycobacterium tuberculosis', MS='A species of gram-positive, aerobic bacteria that produces TUBERCULOSIS in humans, other primates, CATTLE; DOGS; and some other animals which have contact with humans. Growth tends to be in serpentine, cordlike masses in which the bacilli show a parallel orientation.'),
  'D009170': NtO(MH='Nontuberculous Mycobacteria', MS='So-called atypical species of the genus MYCOBACTERIUM that do not cause tuberculosis. They are also called tuberculoid bacilli, i.e.: M. abscessus, M. buruli, M. chelonae, M. duvalii, M. flavescens, M. fortuitum, M. gilvum, M. gordonae, M. intracellulare (see MYCOBACTERIUM AVIUM COMPLEX;), M. kansasii, M. marinum, M. obuense, M. scrofulaceum, M. szulgai, M. terrae, M. ulcerans, M. xenopi.'),
  'D009171': NtO(MH='Mycolic Acids', MS='Long fatty acids found in the CELL WALLS of some ACTINOBACTERIA, including MYCOBACTERIUM TUBERCULOSIS.'),
  'D009172': NtO(MH='Mycology', MS='The study of the structure, growth, function, genetics, and reproduction of fungi, and MYCOSES.'),
  'D009173': NtO(MH='Mycophenolic Acid', MS='Compound derived from Penicillium stoloniferum and related species. It blocks de novo biosynthesis of purine nucleotides by inhibition of the enzyme inosine monophosphate dehydrogenase (IMP DEHYDROGENASE). Mycophenolic acid exerts selective effects on the immune system in which it prevents the proliferation of T-CELLS, LYMPHOCYTES, and the formation of antibodies from B-CELLS. It may also inhibit recruitment of LEUKOCYTES to sites of INFLAMMATION.'),
  'D009174': NtO(MH='Mycoplasma', MS='A genus of gram-negative, mostly facultatively anaerobic bacteria in the family MYCOPLASMATACEAE. The cells are bounded by a PLASMA MEMBRANE and lack a true CELL WALL. Its organisms are pathogens found on the MUCOUS MEMBRANES of humans, ANIMALS, and BIRDS.'),
  'D009175': NtO(MH='Mycoplasma Infections', MS='Infections with species of the genus MYCOPLASMA.'),
  'D009176': NtO(MH='Mycoplasma mycoides', MS='The etiological agent of contagious pleuropneumonia (PLEUROPNEUMONIA, CONTAGIOUS) of cattle and goats.'),
  'D009177': NtO(MH='Mycoplasma pneumoniae', MS='Short filamentous organism of the genus Mycoplasma, which binds firmly to the cells of the respiratory epithelium. It is one of the etiologic agents of non-viral primary atypical pneumonia in man.'),
  'D009178': NtO(MH='Mycoplasmataceae', MS='A family of gram-negative, non-motile bacteria from human and animal sources. One saprophytic species is known.'),
  'D009179': NtO(MH='Mycoplasmatales', MS='An order of highly pleomorphic, gram-negative bacteria including both pathogenic and saprophytic species.'),
  'D009180': NtO(MH='Mycoplasmatales Infections', MS='Infections with bacteria of the order MYCOPLASMATALES.'),
  'D009181': NtO(MH='Mycoses', MS='Diseases caused by FUNGI.'),
  'D009182': NtO(MH='Mycosis Fungoides', MS='A chronic, malignant T-cell lymphoma of the skin. In the late stages, the LYMPH NODES and viscera are affected.'),
  'D009183': NtO(MH='Mycotoxins', MS='Toxic compounds produced by FUNGI.'),
  'D009184': NtO(MH='Mydriatics', MS='Agents that dilate the pupil. They may be either sympathomimetics or parasympatholytics.'),
  'D009185': NtO(MH='Myelin Proteins', MS='MYELIN-specific proteins that play a structural or regulatory role in the genesis and maintenance of the lamellar MYELIN SHEATH structure.'),
  'D009186': NtO(MH='Myelin Sheath', MS='The lipid-rich sheath surrounding AXONS in both the CENTRAL NERVOUS SYSTEMS and PERIPHERAL NERVOUS SYSTEM. The myelin sheath is an electrical insulator and allows faster and more energetically efficient conduction of impulses. The sheath is formed by the cell membranes of glial cells (SCHWANN CELLS in the peripheral and OLIGODENDROGLIA in the central nervous system). Deterioration of the sheath in DEMYELINATING DISEASES is a serious clinical problem.'),
  'D009187': NtO(MH='Myelitis', MS='Inflammation of the spinal cord. Relatively common etiologies include infections; AUTOIMMUNE DISEASES; SPINAL CORD; and ischemia (see also SPINAL CORD VASCULAR DISEASES). Clinical features generally include weakness, sensory loss, localized pain, incontinence, and other signs of autonomic dysfunction.'),
  'D009188': NtO(MH='Myelitis, Transverse', MS='Inflammation of a transverse portion of the spinal cord characterized by acute or subacute segmental demyelination or necrosis. The condition may occur sporadically, follow an infection or vaccination, or present as a paraneoplastic syndrome (see also ENCEPHALOMYELITIS, ACUTE DISSEMINATED). Clinical manifestations include motor weakness, sensory loss, and incontinence. (Adams et al., Principles of Neurology, 6th ed, pp1242-6)'),
  'D009189': NtO(MH='Avian Myeloblastosis Virus', MS='A species of ALPHARETROVIRUS causing anemia in fowl.'),
  'D009190': NtO(MH='Myelodysplastic Syndromes', MS='Clonal hematopoietic stem cell disorders characterized by dysplasia in one or more hematopoietic cell lineages. They predominantly affect patients over 60, are considered preleukemic conditions, and have high probability of transformation into ACUTE MYELOID LEUKEMIA.'),
  'D009192': NtO(MH='Myelography', MS='X-ray visualization of the spinal cord following injection of contrast medium into the spinal arachnoid space.'),
  'D009194': NtO(MH='Myeloma Proteins', MS='Abnormal immunoglobulins characteristic of MULTIPLE MYELOMA.'),
  'D009195': NtO(MH='Peroxidase', MS='A hemeprotein from leukocytes. Deficiency of this enzyme leads to a hereditary disorder coupled with disseminated moniliasis. It catalyzes the conversion of a donor and peroxide to an oxidized donor and water. EC 1.11.1.7.'),
  'D009196': NtO(MH='Myeloproliferative Disorders', MS='Conditions which cause proliferation of hemopoietically active tissue or of tissue which has embryonic hemopoietic potential. They all involve dysregulation of multipotent MYELOID PROGENITOR CELLS, most often caused by a mutation in the JAK2 PROTEIN TYROSINE KINASE.'),
  'D009197': NtO(MH='Myenteric Plexus', MS="One of two ganglionated neural networks which together form the ENTERIC NERVOUS SYSTEM. The myenteric (Auerbach's) plexus is located between the longitudinal and circular muscle layers of the gut. Its neurons project to the circular muscle, to other myenteric ganglia, to submucosal ganglia, or directly to the epithelium, and play an important role in regulating and patterning gut motility. (From FASEB J 1989;3:127-38)"),
  'D009198': NtO(MH='Myiasis', MS='The invasion of living tissues of man and other mammals by dipterous larvae.'),
  'D009200': NtO(MH='Myocardial Contraction', MS='Contractile activity of the MYOCARDIUM.'),
  'D009201': NtO(MH='Myocardial Depressant Factor', MS='A low molecular weight peptide of about 800-1000 having a negative inotropic effect. It is released into the circulation during experimental hemorrhagic pancreatitis, severe ischemia, and postoligemic shock.'),
  'D009202': NtO(MH='Cardiomyopathies', MS='A group of diseases in which the dominant feature is the involvement of the CARDIAC MUSCLE itself. Cardiomyopathies are classified according to their predominant pathophysiological features (DILATED CARDIOMYOPATHY; HYPERTROPHIC CARDIOMYOPATHY; RESTRICTIVE CARDIOMYOPATHY) or their etiological/pathological factors (CARDIOMYOPATHY, ALCOHOLIC; ENDOCARDIAL FIBROELASTOSIS).'),
  'D009203': NtO(MH='Myocardial Infarction', MS='NECROSIS of the MYOCARDIUM caused by an obstruction of the blood supply to the heart (CORONARY CIRCULATION).'),
  'D009204': NtO(MH='Myocardial Revascularization', MS='The restoration of blood supply to the myocardium. (From Dorland, 28th ed)'),
  'D009205': NtO(MH='Myocarditis', MS='Inflammatory processes of the muscular walls of the heart (MYOCARDIUM) which result in injury to the cardiac muscle cells (MYOCYTES, CARDIAC). Manifestations range from subclinical to sudden death (DEATH, SUDDEN). Myocarditis in association with cardiac dysfunction is classified as inflammatory CARDIOMYOPATHY usually caused by INFECTION, autoimmune diseases, or responses to toxic substances. Myocarditis is also a common cause of DILATED CARDIOMYOPATHY and other cardiomyopathies.'),
  'D009206': NtO(MH='Myocardium', MS='The muscle tissue of the HEART. It is composed of striated, involuntary muscle cells (MYOCYTES, CARDIAC) connected to form the contractile pump to generate blood flow.'),
  'D009207': NtO(MH='Myoclonus', MS='Involuntary shock-like contractions, irregular in rhythm and amplitude, followed by relaxation, of a muscle or a group of muscles. This condition may be a feature of some CENTRAL NERVOUS SYSTEM DISEASES; (e.g., EPILEPSY, MYOCLONIC). Nocturnal myoclonus is the principal feature of the NOCTURNAL MYOCLONUS SYNDROME. (From Adams et al., Principles of Neurology, 6th ed, pp102-3).'),
  'D009208': NtO(MH='Myoepithelioma', MS='A usually benign tumor made up predominantly of myoepithelial cells.'),
  'D009209': NtO(MH='Myofascial Pain Syndromes', MS='Muscular pain in numerous body regions that can be reproduced by pressure on TRIGGER POINTS, localized hardenings in skeletal muscle tissue. Pain is referred to a location distant from the trigger points. A prime example is the TEMPOROMANDIBULAR JOINT DYSFUNCTION SYNDROME.'),
  'D009210': NtO(MH='Myofibrils', MS='The long cylindrical contractile organelles of STRIATED MUSCLE cells composed of ACTIN FILAMENTS; MYOSIN filaments; and other proteins organized in arrays of repeating units called SARCOMERES .'),
  'D009211': NtO(MH='Myoglobin', MS='A conjugated protein which is the oxygen-transporting pigment of muscle. It is made up of one globin polypeptide chain and one heme group.'),
  'D009212': NtO(MH='Myoglobinuria', MS='The presence of MYOGLOBIN in URINE usually as a result of rhabdomyolysis.'),
  'D009213': NtO(MH='Myography', MS='The recording of muscular movements. The apparatus is called a myograph, the record or tracing, a myogram. (From Stedman, 25th ed)'),
  'D009214': NtO(MH='Myoma', MS='A benign neoplasm of muscular tissue. (Stedman, 25th ed)'),
  'D009215': NtO(MH='Myometrium', MS='The smooth muscle coat of the uterus, which forms the main mass of the organ.'),
  'D009216': NtO(MH='Myopia', MS='A refractive error in which rays of light entering the EYE parallel to the optic axis are brought to a focus in front of the RETINA when accommodation (ACCOMMODATION, OCULAR) is relaxed. This results from an overly curved CORNEA or from the eyeball being too long from front to back. It is also called nearsightedness.'),
  'D009217': NtO(MH='Myosarcoma', MS='A general term for a malignant neoplasm derived from muscular tissue. (Stedman, 25th ed)'),
  'D009218': NtO(MH='Myosins', MS='A diverse superfamily of proteins that function as translocating proteins. They share the common characteristics of being able to bind ACTINS and hydrolyze MgATP. Myosins generally consist of heavy chains which are involved in locomotion, and light chains which are involved in regulation. Within the structure of myosin heavy chain are three domains: the head, the neck and the tail. The head region of the heavy chain contains the actin binding domain and MgATPase domain which provides energy for locomotion. The neck region is involved in binding the light-chains. The tail region provides the anchoring point that maintains the position of the heavy chain. The superfamily of myosins is organized into structural classes based upon the type and arrangement of the subunits they contain.'),
  'D009219': NtO(MH='Myosin-Light-Chain Kinase', MS='An enzyme that phosphorylates myosin light chains in the presence of ATP to yield myosin-light chain phosphate and ADP, and requires calcium and CALMODULIN. The 20-kDa light chain is phosphorylated more rapidly than any other acceptor, but light chains from other myosins and myosin itself can act as acceptors. The enzyme plays a central role in the regulation of smooth muscle contraction.'),
  'D009220': NtO(MH='Myositis', MS='Inflammation of a muscle or muscle tissue.'),
  'D009221': NtO(MH='Myositis Ossificans', MS='A disease characterized by bony deposits or the ossification of muscle tissue.'),
  'D009222': NtO(MH='Myotonia', MS='Prolonged failure of muscle relaxation after contraction. This may occur after voluntary contractions, muscle percussion, or electrical stimulation of the muscle. Myotonia is a characteristic feature of MYOTONIC DISORDERS.'),
  'D009223': NtO(MH='Myotonic Dystrophy', MS="Neuromuscular disorder characterized by PROGRESSIVE MUSCULAR ATROPHY; MYOTONIA, and various multisystem atrophies. Mild INTELLECTUAL DISABILITY may also occur. Abnormal TRINUCLEOTIDE REPEAT EXPANSION in the 3' UNTRANSLATED REGIONS of DMPK PROTEIN gene is associated with Myotonic Dystrophy 1. DNA REPEAT EXPANSION of zinc finger protein-9 gene intron is associated with Myotonic Dystrophy 2."),
  'D009224': NtO(MH='Myotonia Congenita', MS='Inherited myotonic disorders with early childhood onset MYOTONIA. Muscular hypertrophy is common and myotonia may impair ambulation and other movements. It is classified as Thomsen (autosomal dominant) or Becker (autosomal recessive) generalized myotonia mainly based on the inheritance pattern. Becker type is also clinically more severe. An autosomal dominant variant with milder symptoms and later onset is known as myotonia levior. Mutations in the voltage-dependent skeletal muscle chloride channel are associated with the disorders.'),
  'D009225': NtO(MH='Myringoplasty', MS='Surgical restoration of a perforated tympanic membrane by grafting. (Dorland, 28th ed.)'),
  'D009226': NtO(MH='Myristates', MS='Salts and esters of the 14-carbon saturated monocarboxylic acid--myristic acid.'),
  'D009227': NtO(MH='Myristic Acids', MS='14-carbon saturated monocarboxylic acids.'),
  'D009228': NtO(MH='Mysticism', MS='A philosophy based upon spiritual intuition that is believed to transcend ordinary sensory experiences or understanding.'),
  'D009229': NtO(MH='Mythology', MS="A body of stories, the origins of which may be unknown or forgotten, that serve to explain practices, beliefs, institutions or natural phenomena. Mythology includes legends and folk tales. It may refer to classical mythology or to a body of modern thought and modern life. (From Webster's 1st ed)"),
  'D009230': NtO(MH='Myxedema', MS='A condition characterized by a dry, waxy type of swelling (EDEMA) with abnormal deposits of MUCOPOLYSACCHARIDES in the SKIN and other tissues. It is caused by a deficiency of THYROID HORMONES. The skin becomes puffy around the eyes and on the cheeks. The face is dull and expressionless with thickened nose and lips.'),
  'D009231': NtO(MH='Myxococcales', MS='An order of rod-shaped, gram-negative fruiting gliding bacteria found in SOIL; WATER; and HUMUS.'),
  'D009232': NtO(MH='Myxoma', MS='A benign neoplasm derived from connective tissue, consisting chiefly of polyhedral and stellate cells that are loosely embedded in a soft mucoid matrix, thereby resembling primitive mesenchymal tissue. It occurs frequently intramuscularly where it may be mistaken for a sarcoma. It appears also in the jaws and the skin. (From Stedman, 25th ed)'),
  'D009233': NtO(MH='Myxoma virus', MS='The type species of LEPORIPOXVIRUS causing infectious myxomatosis, a severe generalized disease, in rabbits. Tumors are not always present.'),
  'D009234': NtO(MH='Myxomatosis, Infectious', MS='Diseases caused by MYXOMA VIRUS.'),
  'D009235': NtO(MH='Myxomycetes', MS='A division of organisms that exist vegetatively as complex mobile plasmodia, reproduce by means of spores, and have complex life cycles. They are now classed as protozoa but formerly were considered fungi.'),
  'D009236': NtO(MH='Myxosarcoma', MS='A sarcoma, usually a liposarcoma or malignant fibrous histiocytoma, with an abundant component of myxoid tissue resembling primitive mesenchyme containing connective tissue mucin. (Stedman, 25th ed)'),
  'D009237': NtO(MH='N-Acetyllactosamine Synthase', MS='The A protein of the lactose synthase complex. In the presence of the B protein (LACTALBUMIN) specificity is changed from N-acetylglucosamine to glucose. EC 2.4.1.90.'),
  'D009238': NtO(MH='N-Acetylmuramoyl-L-alanine Amidase', MS='An autolytic enzyme bound to the surface of bacterial cell walls. It catalyzes the hydrolysis of the link between N-acetylmuramoyl residues and L-amino acid residues in certain cell wall glycopeptides, particularly peptidoglycan. EC 3.5.1.28.'),
  'D009239': NtO(MH='N-Formylmethionine', MS='Effective in the initiation of protein synthesis. The initiating methionine residue enters the ribosome as N-formylmethionyl tRNA. This process occurs in Escherichia coli and other bacteria as well as in the mitochondria of eucaryotic cells.'),
  'D009240': NtO(MH='N-Formylmethionine Leucyl-Phenylalanine', MS='A formylated tripeptide originally isolated from bacterial filtrates that is positively chemotactic to polymorphonuclear leucocytes, and causes them to release lysosomal enzymes and become metabolically activated.'),
  'D009241': NtO(MH='Ipratropium', MS='A muscarinic antagonist structurally related to ATROPINE but often considered safer and more effective for inhalation use. It is used for various bronchial disorders, in rhinitis, and as an antiarrhythmic.'),
  'D009242': NtO(MH='N-Nitrosopyrrolidine', MS='Carcinogenic nitrosamine that may be formed from preservatives in meats during their preparation or in the liver during metabolism.'),
  'D009243': NtO(MH='NAD', MS="A coenzyme composed of ribosylnicotinamide 5'-diphosphate coupled to adenosine 5'-phosphate by pyrophosphate linkage. It is found widely in nature and is involved in numerous enzymatic reactions in which it serves as an electron carrier by being alternately oxidized (NAD+) and reduced (NADH). (Dorland, 27th ed)"),
  'D009244': NtO(MH='NAD+ Nucleosidase', MS='An enzyme that catalyzes the hydrolysis of nicotinamide adenine dinucleotide (NAD) to NICOTINAMIDE and ADENOSINE DIPHOSPHATE RIBOSE. Some are extracellular (ectoenzymes).The enzyme from some sources also catalyses the hydrolysis of nicotinamide adenine dinucleotide phosphate (NADP).'),
  'D009245': NtO(MH='NADH Dehydrogenase', MS='A flavoprotein and iron sulfur-containing oxidoreductase that catalyzes the oxidation of NADH to NAD. In eukaryotes the enzyme can be found as a component of mitochondrial electron transport complex I. Under experimental conditions the enzyme can use CYTOCHROME C GROUP as the reducing cofactor. The enzyme was formerly listed as EC 1.6.2.1.'),
  'D009246': NtO(MH='NADH Tetrazolium Reductase', MS='Catalyzes the reduction of tetrazolium compounds in the presence of NADH.'),
  'D009247': NtO(MH='NADH, NADPH Oxidoreductases', MS='A group of oxidoreductases that act on NADH or NADPH. In general, enzymes using NADH or NADPH to reduce a substrate are classified according to the reverse reaction, in which NAD+ or NADP+ is formally regarded as an acceptor. This subclass includes only those enzymes in which some other redox carrier is the acceptor. (Enzyme Nomenclature, 1992, p100) EC 1.6.'),
  'D009248': NtO(MH='Nadolol', MS='A non-selective beta-adrenergic antagonist with a long half-life, used in cardiovascular disease to treat arrhythmias, angina pectoris, and hypertension. Nadolol is also used for MIGRAINE DISORDERS and for tremor.'),
  'D009249': NtO(MH='NADP', MS="Nicotinamide adenine dinucleotide phosphate. A coenzyme composed of ribosylnicotinamide 5'-phosphate (NMN) coupled by pyrophosphate linkage to the 5'-phosphate adenosine 2',5'-bisphosphate. It serves as an electron carrier in a number of reactions, being alternately oxidized (NADP+) and reduced (NADPH). (Dorland, 27th ed)"),
  'D009250': NtO(MH='NADP Transhydrogenases', MS='Enzymes that catalyze the reversible reduction of NAD by NADPH to yield NADP and NADH. This reaction permits the utilization of the reducing properties of NADPH by the respiratory chain and in the reverse direction it allows the reduction of NADP for biosynthetic purposes.'),
  'D009251': NtO(MH='NADPH-Ferrihemoprotein Reductase', MS='A flavoprotein that catalyzes the reduction of heme-thiolate-dependent monooxygenases and is part of the microsomal hydroxylating system. EC 1.6.2.4.'),
  'D009252': NtO(MH='NADPH Dehydrogenase', MS='A flavoprotein that reversibly oxidizes NADPH to NADP and a reduced acceptor. EC 1.6.99.1.'),
  'D009253': NtO(MH='Naegleria', MS='A free-living soil amoeba pathogenic to humans and animals. It occurs also in water and sewage. The most commonly found species in man is NAEGLERIA FOWLERI which is the pathogen for primary amebic meningoencephalitis in primates.'),
  'D009254': NtO(MH='Nafcillin', MS='A semi-synthetic antibiotic related to penicillin.'),
  'D009255': NtO(MH='Nafenopin', MS='A peroxisome proliferator that is used experimentally to promote liver tumors. It has been used as an antihyperlipoproteinemic agent.'),
  'D009256': NtO(MH='Nafoxidine', MS='An estrogen antagonist that has been used in the treatment of breast cancer.'),
  'D009257': NtO(MH='Nafronyl', MS='A drug used in the management of peripheral and cerebral vascular disorders. It is claimed to enhance cellular oxidative capacity and to be a spasmolytic. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1310) It may also be an antagonist at 5HT-2 serotonin receptors.'),
  'D009259': NtO(MH='Nail Biting', MS='Common form of habitual body manipulation which is an expression of tension.'),
  'D009260': NtO(MH='Nail Diseases', MS='Diseases of the nail plate and tissues surrounding it. The concept is limited to primates.'),
  'D009261': NtO(MH='Nail-Patella Syndrome', MS='A syndrome of multiple abnormalities characterized by the absence or hypoplasia of the PATELLA and congenital nail dystrophy. It is a genetically determined autosomal dominant trait.'),
  'D009262': NtO(MH='Nails', MS='The thin, horny plates that cover the dorsal surfaces of the distal phalanges of the fingers and toes of primates.'),
  'D009263': NtO(MH='Nails, Ingrown', MS='Excessive lateral nail growth into the nail fold. Because the lateral margin of the nail acts as a foreign body, inflammation and granulation may result. It is caused by improperly fitting shoes and by improper trimming of the nail.'),
  'D009264': NtO(MH='Nails, Malformed', MS='Deformities in nail structure or appearance, including hypertrophy, splitting, clubbing, furrowing, etc. Genetic diseases such as PACHYONYCHIA CONGENITA can result in malformed nails.'),
  'D009265': NtO(MH='Nairobi Sheep Disease', MS='An arbovirus infection of sheep and goats transmitted by ticks. It is characterized by high fever and hemorrhagic gastroenteritis.'),
  'D009266': NtO(MH='Nalbuphine', MS='A narcotic used as a pain medication. It appears to be an agonist at KAPPA RECEPTORS and an antagonist or partial agonist at MU RECEPTORS.'),
  'D009267': NtO(MH='Naled', MS='An organophosphate cholinesterase inhibitor that is used as an insecticide and as an acaricide.'),
  'D009268': NtO(MH='Nalidixic Acid', MS='A synthetic 1,8-naphthyridine antimicrobial agent with a limited bacteriocidal spectrum. It is an inhibitor of the A subunit of bacterial DNA GYRASE.'),
  'D009269': NtO(MH='Nalorphine', MS='A narcotic antagonist with some agonist properties. It is an antagonist at mu opioid receptors and an agonist at kappa opioid receptors. Given alone it produces a broad spectrum of unpleasant effects and it is considered to be clinically obsolete.'),
  'D009270': NtO(MH='Naloxone', MS='A specific opiate antagonist that has no agonist activity. It is a competitive antagonist at mu, delta, and kappa opioid receptors.'),
  'D009271': NtO(MH='Naltrexone', MS='Derivative of noroxymorphone that is the N-cyclopropylmethyl congener of NALOXONE. It is a narcotic antagonist that is effective orally, longer lasting and more potent than naloxone, and has been proposed for the treatment of heroin addiction. The FDA has approved naltrexone for the treatment of alcohol dependence.'),
  'D009272': NtO(MH='Persons', MS='Persons as individuals (e.g., ABORTION APPLICANTS) or as members of a group (e.g., HISPANIC AMERICANS). It is not used for members of the various professions (e.g., PHYSICIANS) or occupations (e.g., LIBRARIANS) for which OCCUPATIONAL GROUPS is available.'),
  'D009273': NtO(MH='Age Groups', MS='Persons classified by age from birth (INFANT, NEWBORN) to octogenarians and older (AGED, 80 AND OVER).'),
  'D009274': NtO(MH='Occupational Groups', MS='Members of the various professions (e.g., PHYSICIANS) or occupations (e.g., POLICE).'),
  'D009275': NtO(MH='Names', MS='Personal names, given or surname, as cultural characteristics, as ethnological or religious patterns, as indications of the geographic distribution of families and inbreeding, etc. Analysis of isonymy, the quality of having the same or similar names, is useful in the study of population genetics. NAMES is used also for the history of names or name changes of corporate bodies, such as medical societies, universities, hospitals, government agencies, etc.'),
  'D009276': NtO(MH='Namibia', MS='A republic in southern Africa, south of ANGOLA and west of BOTSWANA. Its capital is Windhoek.'),
  'D009277': NtO(MH='Nandrolone', MS='C18 steroid with androgenic and anabolic properties. It is generally prepared from alkyl ethers of ESTRADIOL to resemble TESTOSTERONE but less one carbon at the 19 position.'),
  'D009278': NtO(MH='Naphazoline', MS='An adrenergic vasoconstrictor agent used as a decongestant.'),
  'D009279': NtO(MH='Naphthacenes', MS='Polyacenes with four ortho-fused benzene rings in a straight linear arrangement. This group is best known for the subclass called TETRACYCLINES.'),
  'D009280': NtO(MH='Naphthaleneacetic Acids', MS='Naphthalene derivatives containing the -CH2CCO2H radical at the 1-position, the 2-position, or both. Compounds are used as plant growth regulators to delay sprouting, exert weed control, thin fruit, etc.'),
  'D009281': NtO(MH='Naphthalenes', MS='Two-ring crystalline hydrocarbons isolated from coal tar. They are used as intermediates in chemical synthesis, as insect repellents, fungicides, lubricants, preservatives, and, formerly, as topical antiseptics.'),
  'D009282': NtO(MH='Naphthalenesulfonates', MS='A class of organic compounds that contains a naphthalene moiety linked to a sulfonic acid salt or ester.'),
  'D009283': NtO(MH='Naphthol AS D Esterase', MS="Hydrolytic enzyme activity used as a histocytochemical test for the presence of esterases in tissue. Substrate used is 3-hydroxy-4'-nitro-2-naphthanilide chloroacetate (naphthol AS-D)."),
  'D009284': NtO(MH='Naphthols', MS='Naphthalene derivatives carrying one or more hydroxyl (-OH) groups at any ring position. They are often used in dyes and pigments, as antioxidants for rubber, fats, and oils, as insecticides, in pharmaceuticals, and in numerous other applications.'),
  'D009285': NtO(MH='Naphthoquinones', MS='Naphthalene rings which contain two ketone moieties in any position. They can be substituted in any position except at the ketone groups.'),
  'D009286': NtO(MH='Naphthylvinylpyridine', MS='4(1-Naphthylvinyl)pyridine hydrochloride. Cholinesterase inhibitor. Synonym: YuB 25.'),
  'D009287': NtO(MH='Naphthyridines', MS='A group of two-ring heterocyclic compounds consisting of a NAPHTHALENES double ring in which two carbon atoms, one per each ring, are replaced with nitrogens.'),
  'D009288': NtO(MH='Naproxen', MS='An anti-inflammatory agent with analgesic and antipyretic properties. Both the acid and its sodium salt are used in the treatment of rheumatoid arthritis and other rheumatic or musculoskeletal disorders, dysmenorrhea, and acute gout.'),
  'D009289': NtO(MH='Narcissism', MS='A psychoanalytic term meaning self-love.'),
  'D009290': NtO(MH='Narcolepsy', MS='A condition characterized by recurrent episodes of daytime somnolence and lapses in consciousness (microsomnias) that may be associated with automatic behaviors and AMNESIA. CATAPLEXY; SLEEP PARALYSIS, and hypnagogic HALLUCINATIONS frequently accompany narcolepsy. The pathophysiology of this disorder includes sleep-onset rapid eye movement (REM) sleep, which normally follows stage III or IV sleep. (From Neurology 1998 Feb;50(2 Suppl 1):S2-S7)'),
  'D009291': NtO(MH='Narcotherapy', MS='Intravenous injections of sodium amytal or sodium pentothal to induce a state in which the patient is more relaxed and communicative. Narcosuggestion, narcosynthesis, and narcoanalysis are therapeutic processes using these drug adjuncts.'),
  'D009292': NtO(MH='Narcotic Antagonists', MS='Agents inhibiting the effect of narcotics on the central nervous system.'),
  'D009293': NtO(MH='Opioid-Related Disorders', MS='Disorders related to or resulting from abuse or misuse of OPIOIDS.'),
  'D009294': NtO(MH='Narcotics', MS='Agents that induce NARCOSIS. Narcotics include agents that cause somnolence or induced sleep (STUPOR); natural or synthetic derivatives of OPIUM or MORPHINE or any substance that has such effects. They are potent inducers of ANALGESIA and OPIOID-RELATED DISORDERS.'),
  'D009295': NtO(MH='Nasal Bone', MS='Either one of the two small elongated rectangular bones that together form the bridge of the nose.'),
  'D009296': NtO(MH='Nasal Cavity', MS='The proximal portion of the respiratory passages on either side of the NASAL SEPTUM. Nasal cavities, extending from the nares to the NASOPHARYNX, are lined with ciliated NASAL MUCOSA.'),
  'D009297': NtO(MH='Nasal Mucosa', MS='The mucous lining of the NASAL CAVITY, including lining of the nostril (vestibule) and the OLFACTORY MUCOSA. Nasal mucosa consists of ciliated cells, GOBLET CELLS, brush cells, small granule cells, basal cells (STEM CELLS) and glands containing both mucous and serous cells.'),
  'D009298': NtO(MH='Nasal Polyps', MS='Focal accumulations of EDEMA fluid in the NASAL MUCOSA accompanied by HYPERPLASIA of the associated submucosal connective tissue. Polyps may be NEOPLASMS, foci of INFLAMMATION, degenerative lesions, or malformations.'),
  'D009299': NtO(MH='Nasal Provocation Tests', MS='Application of allergens to the nasal mucosa. Interpretation includes observation of nasal symptoms, rhinoscopy, and rhinomanometry. Nasal provocation tests are used in the diagnosis of nasal hypersensitivity, including RHINITIS, ALLERGIC, SEASONAL.'),
  'D009300': NtO(MH='Nasal Septum', MS='The partition separating the two NASAL CAVITIES in the midplane. It is formed by the SEPTAL NASAL CARTILAGE, parts of skull bones (ETHMOID BONE; VOMER), and membranous parts.'),
  'D009301': NtO(MH='Nasolacrimal Duct', MS='A tubular duct that conveys TEARS from the LACRIMAL GLAND to the nose.'),
  'D009302': NtO(MH='Nasopharyngeal Diseases', MS='Pathological processes involving the NASOPHARYNX.'),
  'D009303': NtO(MH='Nasopharyngeal Neoplasms', MS='Tumors or cancer of the NASOPHARYNX.'),
  'D009304': NtO(MH='Nasopharyngitis', MS='Inflammation of the NASOPHARYNX, usually including its mucosa, related lymphoid structure, and glands.'),
  'D009305': NtO(MH='Nasopharynx', MS='The top portion of the pharynx situated posterior to the nose and superior to the SOFT PALATE. The nasopharynx is the posterior extension of the nasal cavities and has a respiratory function.'),
  'D009306': NtO(MH='Natal Teeth', MS='Predeciduous teeth present at birth. They may be well formed and normal or may represent hornified epithelial structures without roots. They are found on the gingivae over the crest of the ridge and arise from accessory buds of the dental lamina ahead of the deciduous buds or from buds of the accessory dental lamina. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D009307': NtO(MH='National Academy of Sciences, U.S.', MS='A United States organization of distinguished scientists and engineers established for the purpose of investigating and reporting upon any subject of art or science as requested by any department of government. The National Research Council organized by NAS serves as the principal operating agency to stimulate and support research.'),
  'D009308': NtO(MH='National Center for Health Care Technology, U.S.', MS='A center in the PUBLIC HEALTH SERVICE which coordinates and administers a program of research, demonstrations, and evaluations of medical technologies and assessments of health care technology.'),
  'D009309': NtO(MH='National Center for Health Statistics, U.S.', MS='A unit of the CENTERS FOR DISEASE CONTROL AND PREVENTION. It is primarily concerned with the collection, analysis, and dissemination of health statistics on vital events and health activities to reflect the health status of people, health needs, and health resources.'),
  'D009310': NtO(MH='National Health Insurance, United States', MS='Federal system administering national health insurance in the United States.'),
  'D009312': NtO(MH='National Health Planning Information Center, U.S.', MS='A center in the HEALTH RESOURCES ADMINISTRATION Division of Planning Methods and Technology which provides access to current information on health planning and resources development.'),
  'D009313': NtO(MH='National Health Programs', MS='Components of a national health care system which administer specific services, e.g., national health insurance.'),
  'D009314': NtO(MH='National Institute for Occupational Safety and Health, U.S.', MS='An institute of the CENTERS FOR DISEASE CONTROL AND PREVENTION which is responsible for assuring safe and healthful working conditions and for developing standards of safety and health. Research activities are carried out pertinent to these goals.'),
  'D009315': NtO(MH='National Institute of Mental Health (U.S.)', MS='A component of the NATIONAL INSTITUTES OF HEALTH concerned with research, overall planning, promoting, and administering mental health programs and research. It was established in 1949.'),
  'D009316': NtO(MH='National Institutes of Health (U.S.)', MS='An operating division of the US Department of Health and Human Services. It is concerned with the overall planning, promoting, and administering of programs pertaining to health and medical research.'),
  'D009317': NtO(MH='National Library of Medicine (U.S.)', MS='An agency of the NATIONAL INSTITUTES OF HEALTH concerned with overall planning, promoting, and administering programs pertaining to advancement of medical and related sciences. Major activities of this institute include the collection, dissemination, and exchange of information important to the progress of medicine and health, research in medical informatics and support for medical library development.'),
  'D009318': NtO(MH='Natriuresis', MS='Sodium excretion by URINATION.'),
  'D009320': NtO(MH='Atrial Natriuretic Factor', MS='A potent natriuretic and vasodilatory peptide or mixture of different-sized low molecular weight PEPTIDES derived from a common precursor and secreted mainly by the HEART ATRIUM. All these peptides share a sequence of about 20 AMINO ACIDS.'),
  'D009321': NtO(MH='Natural Childbirth', MS='Labor and delivery without medical intervention, usually involving RELAXATION THERAPY.'),
  'D009323': NtO(MH='Natural Language Processing', MS='Computer processing of a language with rules that reflect and describe current usage rather than prescribed usage.'),
  'D009324': NtO(MH='Naturopathy', MS="A drugless system of therapy, making use of physical forces such as air, light, water, heat, massage. Treatments are often diet- and nutrition-oriented with attention given to the patient's personal history and lifestyle. (From Cassileth, Alternative Medicine Handbook, 1998, p329)"),
  'D009325': NtO(MH='Nausea', MS='An unpleasant sensation in the stomach usually accompanied by the urge to vomit. Common causes are early pregnancy, sea and motion sickness, emotional stress, intense pain, food poisoning, and various enteroviruses.'),
  'D009326': NtO(MH='Naval Medicine', MS='The practice of medicine concerned with conditions affecting the health of individuals associated with the marine environment.'),
  'D009327': NtO(MH='4-Chloro-7-nitrobenzofurazan', MS='A benzofuran derivative used as a protein reagent since the terminal N-NBD-protein conjugate possesses interesting fluorescence and spectral properties. It has also been used as a covalent inhibitor of both beef heart mitochondrial ATPase and bacterial ATPase.'),
  'D009328': NtO(MH='Nebramycin', MS='A complex of antibiotic substances produced by Streptomyces tenebrarius.'),
  'D009329': NtO(MH='Nebraska', MS='State bounded on the north by South Dakota, on the east by Iowa and Missouri, on the south by Kansas and Colorado, and on the west by Colorado and Wyoming.'),
  'D009330': NtO(MH='Nebulizers and Vaporizers', MS='Devices that cause a liquid or solid to be converted into an aerosol (spray) or a vapor. It is used in drug administration by inhalation, humidification of ambient air, and in certain analytical instruments.'),
  'D009331': NtO(MH='Necator', MS='A genus of intestinal parasite worms which includes one of the most important hookworms of man, NECATOR AMERICANUS. The only other known species, N. suillus, has been recovered from pigs.'),
  'D009332': NtO(MH='Necatoriasis', MS='Infection of humans or animals with hookworms of the genus NECATOR. The resulting anemia from this condition is less severe than that from ANCYLOSTOMIASIS.'),
  'D009333': NtO(MH='Neck', MS='The part of a human or animal body connecting the HEAD to the rest of the body.'),
  'D009334': NtO(MH='Neck Muscles', MS='The neck muscles consist of the platysma, splenius cervicis, sternocleidomastoid(eus), longus colli, the anterior, medius, and posterior scalenes, digastric(us), stylohyoid(eus), mylohyoid(eus), geniohyoid(eus), sternohyoid(eus), omohyoid(eus), sternothyroid(eus), and thyrohyoid(eus).'),
  'D009335': NtO(MH='Necrobiosis Lipoidica', MS='A degenerative disease of the dermal connective tissue characterized by the development of erythematous papules or nodules in the pretibial area. The papules form plaques covered with telangiectatic vessels. More than half of the affected patients have diabetes.'),
  'D009336': NtO(MH='Necrosis', MS='The death of cells in an organ or tissue due to disease, injury or failure of the blood supply.'),
  'D009337': NtO(MH='Necturus', MS='A genus of the Proteidae family with five recognized species, which inhabit the Atlantic and Gulf drainages.'),
  'D009338': NtO(MH='Necturus maculosus', MS='A neotenic aquatic species of mudpuppy (Necturus) occurring from Manitoba to Louisiana and Texas.'),
  'D009339': NtO(MH='Needles', MS='Sharp instruments used for puncturing or suturing.'),
  'D009340': NtO(MH='Nefopam', MS='Non-narcotic analgesic chemically similar to ORPHENADRINE. Its mechanism of action is unclear. It is used for the relief of acute and chronic pain. (From Martindale, The Extra Pharmacopoeia, 30th ed, p26)'),
  'D009341': NtO(MH='Negativism', MS='State of mind or behavior characterized by extreme skepticism and persistent opposition or resistance to outside suggestions or advice. (APA, Thesaurus of Psychological Index Terms, 1994)'),
  'D009343': NtO(MH='Neisseria', MS='A genus of gram-negative, aerobic, coccoid bacteria whose organisms are part of the normal flora of the oropharynx, nasopharynx, and genitourinary tract. Some species are primary pathogens for humans.'),
  'D009344': NtO(MH='Neisseria gonorrhoeae', MS='A species of gram-negative, aerobic bacteria primarily found in purulent venereal discharges. It is the causative agent of GONORRHEA.'),
  'D009345': NtO(MH='Neisseria meningitidis', MS='A species of gram-negative, aerobic BACTERIA. It is a commensal and pathogen only of humans, and can be carried asymptomatically in the NASOPHARYNX. When found in cerebrospinal fluid it is the causative agent of cerebrospinal meningitis (MENINGITIS, MENINGOCOCCAL). It is also found in venereal discharges and blood. There are at least 13 serogroups based on antigenic differences in the capsular polysaccharides; the ones causing most meningitis infections being A, B, C, Y, and W-135. Each serogroup can be further classified by serotype, serosubtype, and immunotype.'),
  'D009346': NtO(MH='Neisseriaceae', MS='A family of gram-negative, parasitic bacteria including several important pathogens of man.'),
  'D009347': NtO(MH='Nelson Syndrome', MS='A syndrome characterized by HYPERPIGMENTATION, enlarging pituitary mass, visual defects secondary to compression of the OPTIC CHIASM, and elevated serum ACTH. It is caused by the expansion of an underlying ACTH-SECRETING PITUITARY ADENOMA that grows in the absence of feedback inhibition by adrenal CORTICOSTEROIDS, usually after ADRENALECTOMY.'),
  'D009348': NtO(MH='Nematoda', MS='A phylum of unsegmented helminths with fundamental bilateral symmetry and secondary triradiate symmetry of the oral and esophageal structures. Many species are parasites.'),
  'D009349': NtO(MH='Nematode Infections', MS='Infections by nematodes, general or unspecified.'),
  'D009351': NtO(MH='Nematospiroides dubius', MS='A species of intestinal nematode parasites which occur most commonly in mice. Infection is by ingesting larvae. This particular species is used extensively in immunological research.'),
  'D009353': NtO(MH='Zinostatin', MS='An enediyne that alkylates DNA and RNA like MITOMYCIN does, so it is cytotoxic.'),
  'D009354': NtO(MH='Neodymium', MS='An element of the rare earth family of metals. It has the atomic symbol Nd, atomic number 60, and atomic weight 144.24, and is used in industrial applications.'),
  'D009355': NtO(MH='Neomycin', MS='Aminoglycoside antibiotic complex produced by Streptomyces fradiae. It is composed of neomycins A, B, and C, and acts by inhibiting translation during protein synthesis.'),
  'D009356': NtO(MH='Neon', MS="A noble gas with the atomic symbol Ne, atomic number 10, and atomic weight 20.18. It is found in the earth's crust and atmosphere as an inert, odorless gas and is used in vacuum tubes and incandescent lamps."),
  'D009357': NtO(MH='Neonatal Abstinence Syndrome', MS="Fetal and neonatal addiction and withdrawal as a result of the mother's dependence on drugs during pregnancy. Withdrawal or abstinence symptoms develop shortly after birth. Symptoms exhibited are loud, high-pitched crying, sweating, yawning and gastrointestinal disturbances."),
  'D009358': NtO(MH='Congenital, Hereditary, and Neonatal Diseases and Abnormalities', MS='Diseases existing at birth and often before birth, or that develop during the first month of life (INFANT, NEWBORN, DISEASES), regardless of causation. Of these diseases, those characterized by structural deformities are termed CONGENITAL ABNORMALITIES.'),
  'D009359': NtO(MH='Neonatology', MS='A subspecialty of Pediatrics concerned with the newborn infant.'),
  'D009360': NtO(MH='Neoplastic Cells, Circulating', MS='Exfoliate neoplastic cells circulating in the blood and associated with metastasizing tumors.'),
  'D009361': NtO(MH='Neoplasm Invasiveness', MS='Ability of neoplasms to infiltrate and actively destroy surrounding tissue.'),
  'D009362': NtO(MH='Neoplasm Metastasis', MS='The transfer of a neoplasm from one organ or part of the body to another remote from the primary site.'),
  'D009363': NtO(MH='Neoplasm Proteins', MS='Proteins whose abnormal expression (gain or loss) are associated with the development, growth, or progression of NEOPLASMS. Some neoplasm proteins are tumor antigens (ANTIGENS, NEOPLASM), i.e. they induce an immune reaction to their tumor. Many neoplasm proteins have been characterized and are used as tumor markers (BIOMARKERS, TUMOR) when they are detectable in cells and body fluids as monitors for the presence or growth of tumors. Abnormal expression of ONCOGENE PROTEINS is involved in neoplastic transformation, whereas the loss of expression of TUMOR SUPPRESSOR PROTEINS is involved with the loss of growth control and progression of the neoplasm.'),
  'D009364': NtO(MH='Neoplasm Recurrence, Local', MS='The local recurrence of a neoplasm following treatment. It arises from microscopic cells of the original neoplasm that have escaped therapeutic intervention and later become clinically visible at the original site.'),
  'D009365': NtO(MH='Neoplasm Regression, Spontaneous', MS='Disappearance of a neoplasm or neoplastic state without the intervention of therapy.'),
  'D009366': NtO(MH='Neoplasm Seeding', MS='The local implantation of tumor cells by contamination of instruments and surgical equipment during and after surgical resection, resulting in local growth of the cells and tumor formation.'),
  'D009367': NtO(MH='Neoplasm Staging', MS='Methods which attempt to express in replicable terms the extent of the neoplasm in the patient.'),
  'D009368': NtO(MH='Neoplasm Transplantation', MS='Experimental transplantation of neoplasms in laboratory animals for research purposes.'),
  'D009369': NtO(MH='Neoplasms', MS='New abnormal growth of tissue. Malignant neoplasms show a greater degree of anaplasia and have the properties of invasion and metastasis, compared to benign neoplasms.'),
  'D009370': NtO(MH='Neoplasms by Histologic Type', MS='A collective term for the various histological types of NEOPLASMS. It is more likely to be used by searchers than by indexers and catalogers.'),
  'D009371': NtO(MH='Neoplasms by Site', MS='A collective term for precoordinated organ/neoplasm headings locating neoplasms by organ, as BRAIN NEOPLASMS; DUODENAL NEOPLASMS; LIVER NEOPLASMS; etc.'),
  'D009372': NtO(MH='Neoplasms, Connective Tissue', MS='Neoplasms composed of connective tissue, including elastic, mucous, reticular, osseous, and cartilaginous tissue. The concept does not refer to neoplasms located in connective tissue.'),
  'D009373': NtO(MH='Neoplasms, Germ Cell and Embryonal', MS='Neoplasms composed of primordial GERM CELLS of embryonic GONADS or of elements of the germ layers of the EMBRYO, MAMMALIAN. The concept does not refer to neoplasms located in the gonads or present in an embryo or FETUS.'),
  'D009374': NtO(MH='Neoplasms, Experimental', MS='Experimentally induced new abnormal growth of TISSUES in animals to provide models for studying human neoplasms.'),
  'D009375': NtO(MH='Neoplasms, Glandular and Epithelial', MS='Neoplasms composed of glandular tissue, an aggregation of epithelial cells that elaborate secretions, and of any type of epithelium itself. The concept does not refer to neoplasms located in the various glands or in epithelial tissue.'),
  'D009376': NtO(MH='Neoplasms, Hormone-Dependent', MS='Certain tumors that 1, arise in organs that are normally dependent on specific hormones and 2, are stimulated or caused to regress by manipulation of the endocrine environment.'),
  'D009377': NtO(MH='Multiple Endocrine Neoplasia', MS='A group of autosomal dominant diseases characterized by the combined occurrence of tumors involving two or more ENDOCRINE GLANDS that secrete PEPTIDE HORMONES or AMINES. These neoplasias are often benign but can be malignant. They are classified by the endocrine glands involved and the degree of aggressiveness. The two major forms are MEN1 and MEN2 with gene mutations on CHROMOSOME 11 and CHROMOSOME 10, respectively.'),
  'D009378': NtO(MH='Neoplasms, Multiple Primary', MS='Two or more abnormal growths of tissue occurring simultaneously and presumed to be of separate origin. The neoplasms may be histologically the same or different, and may be found in the same or different sites.'),
  'D009379': NtO(MH='Neoplasms, Muscle Tissue', MS='Neoplasms composed of muscle tissue: skeletal, cardiac, or smooth. The concept does not refer to neoplasms located in muscles.'),
  'D009380': NtO(MH='Neoplasms, Nerve Tissue', MS='Neoplasms composed of nerve tissue. This concept does not refer to neoplasms located in the nervous system or its component nerves.'),
  'D009381': NtO(MH='Neoplasms, Radiation-Induced', MS='Tumors, cancer or other neoplasms produced by exposure to ionizing or non-ionizing radiation.'),
  'D009382': NtO(MH='Neoplasms, Unknown Primary', MS='Metastases in which the tissue of origin is unknown.'),
  'D009383': NtO(MH='Neoplasms, Vascular Tissue', MS='Neoplasms composed of vascular tissue. This concept does not refer to neoplasms located in blood vessels.'),
  'D009384': NtO(MH='Paraneoplastic Endocrine Syndromes', MS='Syndromes resulting from inappropriate production of HORMONES or hormone-like materials by NEOPLASMS in non-endocrine tissues or not by the usual ENDOCRINE GLANDS. Such hormone outputs are called ectopic hormone (HORMONES, ECTOPIC) secretion.'),
  'D009385': NtO(MH='Neoplastic Processes', MS='The pathological mechanisms and forms taken by tissue during degeneration into a neoplasm and its subsequent activity.'),
  'D009386': NtO(MH='Neoplastic Syndromes, Hereditary', MS="The condition of a pattern of malignancies within a family, but not every individual's necessarily having the same neoplasm. Characteristically the tumor tends to occur at an earlier than average age, individuals may have more than one primary tumor, the tumors may be multicentric, usually more than 25 percent of the individuals in direct lineal descent from the proband are affected, and the cancer predisposition in these families behaves as an autosomal dominant trait with about 60 percent penetrance."),
  'D009387': NtO(MH='Neoprene', MS='An oil-resistant synthetic rubber made by the polymerization of chloroprene.'),
  'D009388': NtO(MH='Neostigmine', MS='A cholinesterase inhibitor used in the treatment of myasthenia gravis and to reverse the effects of muscle relaxants such as gallamine and tubocurarine. Neostigmine, unlike PHYSOSTIGMINE, does not cross the blood-brain barrier.'),
  'D009389': NtO(MH='Neovascularization, Pathologic', MS='A pathologic process consisting of the proliferation of blood vessels in abnormal tissues or in abnormal positions.'),
  'D009390': NtO(MH='Nepal', MS='Country located in southern Asia, between China and India. The capital is Kathmandu.'),
  'D009391': NtO(MH='Nephelometry and Turbidimetry', MS='Chemical analysis based on the phenomenon whereby light, passing through a medium with dispersed particles of a different refractive index from that of the medium, is attenuated in intensity by scattering. In turbidimetry, the intensity of light transmitted through the medium, the unscattered light, is measured. In nephelometry, the intensity of the scattered light is measured, usually, but not necessarily, at right angles to the incident light beam.'),
  'D009392': NtO(MH='Nephrectomy', MS='Excision of kidney.'),
  'D009393': NtO(MH='Nephritis', MS='Inflammation of any part of the KIDNEY.'),
  'D009394': NtO(MH='Nephritis, Hereditary', MS='A group of inherited conditions characterized initially by HEMATURIA and slowly progressing to RENAL INSUFFICIENCY. The most common form is the Alport syndrome (hereditary nephritis with HEARING LOSS) which is caused by mutations in genes for TYPE IV COLLAGEN and defective GLOMERULAR BASEMENT MEMBRANE.'),
  'D009395': NtO(MH='Nephritis, Interstitial', MS='Inflammation of the interstitial tissue of the kidney. This term is generally used for primary inflammation of KIDNEY TUBULES and/or surrounding interstitium. For primary inflammation of glomerular interstitium, see GLOMERULONEPHRITIS. Infiltration of the inflammatory cells into the interstitial compartment results in EDEMA, increased spaces between the tubules, and tubular renal dysfunction.'),
  'D009396': NtO(MH='Wilms Tumor', MS="A malignant kidney tumor, caused by the uncontrolled multiplication of renal stem (blastemal), stromal (STROMAL CELLS), and epithelial (EPITHELIAL CELLS) elements. However, not all three are present in every case. Several genes or chromosomal areas have been associated with Wilms tumor which is usually found in childhood as a firm lump in a child's side or ABDOMEN."),
  'D009397': NtO(MH='Nephrocalcinosis', MS='A condition characterized by calcification of the renal tissue itself. It is usually seen in distal RENAL TUBULAR ACIDOSIS with calcium deposition in the DISTAL KIDNEY TUBULES and the surrounding interstitium. Nephrocalcinosis causes RENAL INSUFFICIENCY.'),
  'D009398': NtO(MH='Nephrology', MS='A subspecialty of internal medicine concerned with the anatomy, physiology, and pathology of the kidney.'),
  'D009399': NtO(MH='Nephrons', MS='The functional units of the kidney, consisting of the glomerulus and the attached tubule.'),
  'D009400': NtO(MH='Nephrosclerosis', MS='Hardening of the KIDNEY due to infiltration by fibrous connective tissue (FIBROSIS), usually caused by renovascular diseases or chronic HYPERTENSION. Nephrosclerosis leads to renal ISCHEMIA.'),
  'D009401': NtO(MH='Nephrosis', MS='Pathological processes of the KIDNEY without inflammatory or neoplastic components. Nephrosis may be a primary disorder or secondary complication of other diseases. It is characterized by the NEPHROTIC SYNDROME indicating the presence of PROTEINURIA and HYPOALBUMINEMIA with accompanying EDEMA.'),
  'D009402': NtO(MH='Nephrosis, Lipoid', MS='A kidney disease with no or minimal histological glomerular changes on light microscopy and with no immune deposits. It is characterized by lipid accumulation in the epithelial cells of KIDNEY TUBULES and in the URINE. Patients usually show NEPHROTIC SYNDROME indicating the presence of PROTEINURIA with accompanying EDEMA.'),
  'D009403': NtO(MH='Nephrostomy, Percutaneous', MS='The insertion of a catheter through the skin and body wall into the kidney pelvis, mainly to provide urine drainage where the ureter is not functional. It is used also to remove or dissolve renal calculi and to diagnose ureteral obstruction.'),
  'D009404': NtO(MH='Nephrotic Syndrome', MS='A condition characterized by severe PROTEINURIA, greater than 3.5 g/day in an average adult. The substantial loss of protein in the urine results in complications such as HYPOPROTEINEMIA; generalized EDEMA; HYPERTENSION; and HYPERLIPIDEMIAS. Diseases associated with nephrotic syndrome generally cause chronic kidney dysfunction.'),
  'D009405': NtO(MH='Neptunium', MS='A radioactive element of the actinide metals family. It has the atomic symbol Np, and atomic number 93.'),
  'D009407': NtO(MH='Nerve Block', MS='Interruption of NEURAL CONDUCTION in peripheral nerves or nerve trunks by the injection of a local anesthetic agent (e.g., LIDOCAINE; PHENOL; BOTULINUM TOXINS) to manage or treat pain.'),
  'D009408': NtO(MH='Nerve Compression Syndromes', MS='Mechanical compression of nerves or nerve roots from internal or external causes. These may result in a conduction block to nerve impulses (due to MYELIN SHEATH dysfunction) or axonal loss. The nerve and nerve sheath injuries may be caused by ISCHEMIA; INFLAMMATION; or a direct mechanical effect.'),
  'D009409': NtO(MH='Nerve Crush', MS='Treatment of muscles and nerves under pressure as a result of crush injuries.'),
  'D009410': NtO(MH='Nerve Degeneration', MS='Loss of functional activity and trophic degeneration of nerve axons and their terminal arborizations following the destruction of their cells of origin or interruption of their continuity with these cells. The pathology is characteristic of neurodegenerative diseases. Often the process of nerve degeneration is studied in research on neuroanatomical localization and correlation of the neurophysiology of neural pathways.'),
  'D009411': NtO(MH='Nerve Endings', MS='Branch-like terminations of NERVE FIBERS, sensory or motor NEURONS. Endings of sensory neurons are the beginnings of afferent pathway to the CENTRAL NERVOUS SYSTEM. Endings of motor neurons are the terminals of axons at the muscle cells. Nerve endings which release neurotransmitters are called PRESYNAPTIC TERMINALS.'),
  'D009412': NtO(MH='Nerve Fibers', MS='Slender processes of NEURONS, including the AXONS and their glial envelopes (MYELIN SHEATH). Nerve fibers conduct nerve impulses to and from the CENTRAL NERVOUS SYSTEM.'),
  'D009413': NtO(MH='Nerve Fibers, Myelinated', MS='A class of nerve fibers as defined by their structure, specifically the nerve sheath arrangement. The AXONS of the myelinated nerve fibers are completely encased in a MYELIN SHEATH. They are fibers of relatively large and varied diameters. Their NEURAL CONDUCTION rates are faster than those of the unmyelinated nerve fibers (NERVE FIBERS, UNMYELINATED). Myelinated nerve fibers are present in somatic and autonomic nerves.'),
  'D009414': NtO(MH='Nerve Growth Factors', MS='Factors which enhance the growth potentialities of sensory and sympathetic nerve cells.'),
  'D009415': NtO(MH='Nerve Net', MS='A meshlike structure composed of interconnecting nerve cells that are separated at the synaptic junction or joined to one another by cytoplasmic processes. In invertebrates, for example, the nerve net allows nerve impulses to spread over a wide area of the net because synapses can pass information in any direction.'),
  'D009416': NtO(MH='Nerve Regeneration', MS='Renewal or physiological repair of damaged nerve tissue.'),
  'D009417': NtO(MH='Nerve Tissue', MS='Differentiated tissue of the central nervous system composed of NERVE CELLS, fibers, DENDRITES, and specialized supporting cells.'),
  'D009418': NtO(MH='S100 Proteins', MS='A family of highly acidic calcium-binding proteins found in large concentration in the brain and believed to be glial in origin. They are also found in other organs in the body. They have in common the EF-hand motif (EF HAND MOTIFS) found on a number of calcium binding proteins. The name of this family derives from the property of being soluble in a 100% saturated ammonium sulfate solution.'),
  'D009419': NtO(MH='Nerve Tissue Proteins', MS=None),
  'D009420': NtO(MH='Nervous System', MS='The entire nerve apparatus, composed of a central part, the brain and spinal cord, and a peripheral part, the cranial and spinal nerves, autonomic ganglia, and plexuses. (Stedman, 26th ed)'),
  'D009421': NtO(MH='Nervous System Malformations', MS='Structural abnormalities of the central or peripheral nervous system resulting primarily from defects of embryogenesis.'),
  'D009422': NtO(MH='Nervous System Diseases', MS='Diseases of the central and peripheral nervous system. This includes disorders of the brain, spinal cord, cranial nerves, peripheral nerves, nerve roots, autonomic nervous system, neuromuscular junction, and muscle.'),
  'D009423': NtO(MH='Nervous System Neoplasms', MS='Benign and malignant neoplastic processes arising from or involving components of the central, peripheral, and autonomic nervous systems, cranial nerves, and meninges. Included in this category are primary and metastatic nervous system neoplasms.'),
  'D009424': NtO(MH='Nervous System Physiological Phenomena', MS='Characteristic properties and processes of the NERVOUS SYSTEM as a whole or with reference to the peripheral or the CENTRAL NERVOUS SYSTEM.'),
  'D009425': NtO(MH='Nesting Behavior', MS='Animal behavior associated with the nest; includes construction, effects of size and material; behavior of the adult during the nesting period and the effect of the nest on the behavior of the young.'),
  'D009426': NtO(MH='Netherlands', MS='Country located in EUROPE. It is bordered by the NORTH SEA, BELGIUM, and GERMANY. Constituent areas are Aruba, Curacao, and Sint Maarten, formerly included in the NETHERLANDS ANTILLES.'),
  'D009427': NtO(MH='Netherlands Antilles', MS='Former Netherlands overseas territory in the Lesser Antilles in the West Indies. It had included the islands of Aruba, Bonaire, Curacao, Saba, St. Eustatius, and the southern part of St. Martin. The Netherlands Antilles dissolved on October 10, 2010.'),
  'D009428': NtO(MH='Netilmicin', MS='Semisynthetic 1-N-ethyl derivative of SISOMYCIN, an aminoglycoside antibiotic with action similar to gentamicin, but less ear and kidney toxicity.'),
  'D009429': NtO(MH='Netropsin', MS='A basic polypeptide isolated from Streptomyces netropsis. It is cytotoxic and its strong, specific binding to A-T areas of DNA is useful to genetics research.'),
  'D009430': NtO(MH='Neural Analyzers', MS="A term used in Eastern European research literature for the functional neural unit that provides the basis for differential sensitivity; the analyzer consists of receptor, afferent nerves, and their central connections. (From Campbell's Psychiatric Dictionary, 8th ed.)"),
  'D009431': NtO(MH='Neural Conduction', MS='The propagation of the NERVE IMPULSE along the nerve away from the site of an excitation stimulus.'),
  'D009432': NtO(MH='Neural Crest', MS='The two longitudinal ridges along the PRIMITIVE STREAK appearing near the end of GASTRULATION during development of nervous system (NEURULATION). The ridges are formed by folding of NEURAL PLATE. Between the ridges is a neural groove which deepens as the fold become elevated. When the folds meet at midline, the groove becomes a closed tube, the NEURAL TUBE.'),
  'D009433': NtO(MH='Neural Inhibition', MS='The function of opposing or restraining the excitation of neurons or their target excitable cells.'),
  'D009434': NtO(MH='Neural Pathways', MS='Neural tracts connecting one part of the nervous system with another.'),
  'D009435': NtO(MH='Synaptic Transmission', MS='The communication from a NEURON to a target (neuron, muscle, or secretory cell) across a SYNAPSE. In chemical synaptic transmission, the presynaptic neuron releases a NEUROTRANSMITTER that diffuses across the synaptic cleft and binds to specific synaptic receptors, activating them. The activated receptors modulate specific ion channels and/or second-messenger systems in the postsynaptic cell. In electrical synaptic transmission, electrical signals are communicated as an ionic current flow across ELECTRICAL SYNAPSES.'),
  'D009436': NtO(MH='Neural Tube Defects', MS='Congenital malformations of the central nervous system and adjacent structures related to defective neural tube closure during the first trimester of pregnancy generally occurring between days 18-29 of gestation. Ectodermal and mesodermal malformations (mainly involving the skull and vertebrae) may occur as a result of defects of neural tube closure. (From Joynt, Clinical Neurology, 1992, Ch55, pp31-41)'),
  'D009437': NtO(MH='Neuralgia', MS='Intense or aching pain that occurs along the course or distribution of a peripheral or cranial nerve.'),
  'D009438': NtO(MH='Neuraminic Acids', MS='Nine-carbon deoxyamino SUGAR ACIDS that are substrates for NEURAMINIDASE. Their derivatives include SIALIC ACIDS.'),
  'D009439': NtO(MH='Neuraminidase', MS='An enzyme that catalyzes the hydrolysis of alpha-2,3, alpha-2,6-, and alpha-2,8-glycosidic linkages (at a decreasing rate, respectively) of terminal sialic residues in oligosaccharides, glycoproteins, glycolipids, colominic acid, and synthetic substrate. (From Enzyme Nomenclature, 1992)'),
  'D009440': NtO(MH='Neurasthenia', MS='A mental disorder characterized by chronic fatigue and concomitant physiologic symptoms.'),
  'D009441': NtO(MH='Neurilemma', MS='The outermost cytoplasmic layer of the SCHWANN CELLS covering NERVE FIBERS.'),
  'D009442': NtO(MH='Neurilemmoma', MS='A neoplasm that arises from SCHWANN CELLS of the cranial, peripheral, and autonomic nerves. Clinically, these tumors may present as a cranial neuropathy, abdominal or soft tissue mass, intracranial lesion, or with spinal cord compression. Histologically, these tumors are encapsulated, highly vascular, and composed of a homogenous pattern of biphasic fusiform-shaped cells that may have a palisaded appearance. (From DeVita Jr et al., Cancer: Principles and Practice of Oncology, 5th ed, pp964-5)'),
  'D009443': NtO(MH='Neuritis', MS='A general term indicating inflammation of a peripheral or cranial nerve. Clinical manifestation may include PAIN; PARESTHESIAS; PARESIS; or HYPESTHESIA.'),
  'D009444': NtO(MH='Neuritis, Autoimmune, Experimental', MS='An experimental animal model for the demyelinating disease of GUILLAINE-BARRE SYNDROME. In the most frequently used protocol, animals are injected with a peripheral nerve tissue protein homogenate. After approximately 2 weeks the animals develop a neuropathy secondary to a T cell-mediated autoimmune response directed towards the MYELIN P2 PROTEIN in peripheral nerves. Pathologic findings include a perivascular accumulation of macrophages and T lymphocytes in the peripheral nervous system, similar to that seen in the Guillaine-Barre syndrome. (From Adams et al., Principles of Neurology, 6th ed, p1314; J Neuroimmunol 1998 Apr 1;84(1):40-52)'),
  'D009445': NtO(MH='Neuroanatomy', MS='Study of the anatomy of the nervous system as a specialty or discipline.'),
  'D009446': NtO(MH='Neurobiology', MS='The study of the structure, growth, activities, and functions of NEURONS and the NERVOUS SYSTEM.'),
  'D009447': NtO(MH='Neuroblastoma', MS='A common neoplasm of early childhood arising from neural crest cells in the sympathetic nervous system, and characterized by diverse clinical behavior, ranging from spontaneous remission to rapid metastatic progression and death. This tumor is the most common intraabdominal malignancy of childhood, but it may also arise from thorax, neck, or rarely occur in the central nervous system. Histologic features include uniform round cells with hyperchromatic nuclei arranged in nests and separated by fibrovascular septa. Neuroblastomas may be associated with the opsoclonus-myoclonus syndrome. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, pp2099-2101; Curr Opin Oncol 1998 Jan;10(1):43-51)'),
  'D009448': NtO(MH='Neurochemistry', MS='The study of the composition, chemical structures, and chemical reactions of the NERVOUS SYSTEM or its components.'),
  'D009449': NtO(MH='Neurocirculatory Asthenia', MS='A clinical syndrome characterized by palpitation, SHORTNESS OF BREATH, labored breathing, subjective complaints of effort and discomfort, all following slight PHYSICAL EXERTION. Other symptoms may be DIZZINESS, tremulousness, SWEATING, and INSOMNIA. Neurocirculatory asthenia is most typically seen as a form of anxiety disorder.'),
  'D009450': NtO(MH='Neurodermatitis', MS='An extremely variable eczematous skin disease that is presumed to be a response to prolonged vigorous scratching, rubbing, or pinching to relieve intense pruritus. It varies in intensity, severity, course, and morphologic expression in different individuals. Neurodermatitis is believed by some to be psychogenic. The circumscribed or localized form is often referred to as lichen simplex chronicus.'),
  'D009451': NtO(MH='Neuroeffector Junction', MS='The synapse between a neuron (presynaptic) and an effector cell other than another neuron (postsynaptic). Neuroeffector junctions include synapses onto muscles and onto secretory cells.'),
  'D009452': NtO(MH='Neuroendocrinology', MS='The study of the anatomical and functional relationships between the nervous system and the endocrine system.'),
  'D009454': NtO(MH='Neurofibrils', MS='The delicate interlacing threads, formed by aggregations of neurofilaments and neurotubules, coursing through the CYTOPLASM of the body of a NEURON and extending from one DENDRITE into another or into the AXON.'),
  'D009455': NtO(MH='Neurofibroma', MS='A moderately firm, benign, encapsulated tumor resulting from proliferation of SCHWANN CELLS and FIBROBLASTS that includes portions of nerve fibers. The tumors usually develop along peripheral or cranial nerves and are a central feature of NEUROFIBROMATOSIS 1, where they may occur intracranially or involve spinal roots. Pathologic features include fusiform enlargement of the involved nerve. Microscopic examination reveals a disorganized and loose cellular pattern with elongated nuclei intermixed with fibrous strands. (From Adams et al., Principles of Neurology, 6th ed, p1016)'),
  'D009456': NtO(MH='Neurofibromatosis 1', MS='An autosomal dominant inherited disorder (with a high frequency of spontaneous mutations) that features developmental changes in the nervous system, muscles, bones, and skin, most notably in tissue derived from the embryonic NEURAL CREST. Multiple hyperpigmented skin lesions and subcutaneous tumors are the hallmark of this disease. Peripheral and central nervous system neoplasms occur frequently, especially OPTIC NERVE GLIOMA and NEUROFIBROSARCOMA. NF1 is caused by mutations which inactivate the NF1 gene (GENES, NEUROFIBROMATOSIS 1) on chromosome 17q. The incidence of learning disabilities is also elevated in this condition. (From Adams et al., Principles of Neurology, 6th ed, pp1014-18) There is overlap of clinical features with NOONAN SYNDROME in a syndrome called neurofibromatosis-Noonan syndrome. Both the PTPN11 and NF1 gene products are involved in the SIGNAL TRANSDUCTION pathway of Ras (RAS PROTEINS).'),
  'D009457': NtO(MH='Neuroglia', MS='The non-neuronal cells of the nervous system. They not only provide physical support, but also respond to injury, regulate the ionic and chemical composition of the extracellular milieu, participate in the BLOOD-BRAIN BARRIER and BLOOD-RETINAL BARRIER, form the myelin insulation of nervous pathways, guide neuronal migration during development, and exchange metabolites with neurons. Neuroglia have high-affinity transmitter uptake systems, voltage-dependent and transmitter-gated ion channels, and can release transmitters, but their role in signaling (as in many other functions) is unclear.'),
  'D009458': NtO(MH='Neuroleptanalgesia', MS='A form of analgesia accompanied by general quiescence and psychic indifference to environmental stimuli, without loss of consciousness, and produced by the combined administration of a major tranquilizer (neuroleptic) and a narcotic.'),
  'D009459': NtO(MH='Neuroleptic Malignant Syndrome', MS='A potentially fatal syndrome associated primarily with the use of neuroleptic agents (see ANTIPSYCHOTIC AGENTS) which are in turn associated with dopaminergic receptor blockade (see RECEPTORS, DOPAMINE) in the BASAL GANGLIA and HYPOTHALAMUS, and sympathetic dysregulation. Clinical features include diffuse MUSCLE RIGIDITY; TREMOR; high FEVER; diaphoresis; labile blood pressure; cognitive dysfunction; and autonomic disturbances. Serum CPK level elevation and a leukocytosis may also be present. (From Adams et al., Principles of Neurology, 6th ed, p1199; Psychiatr Serv 1998 Sep;49(9):1163-72)'),
  'D009460': NtO(MH='Neurologic Examination', MS='Assessment of sensory and motor responses and reflexes that is used to determine impairment of the nervous system.'),
  'D009461': NtO(MH='Neurologic Manifestations', MS='Clinical signs and symptoms caused by nervous system injury or dysfunction.'),
  'D009462': NtO(MH='Neurology', MS='A medical specialty concerned with the study of the structures, functions, and diseases of the nervous system.'),
  'D009463': NtO(MH='Neuroma', MS='A tumor made up of nerve cells and nerve fibers. (Dorland, 27th ed)'),
  'D009464': NtO(MH='Neuroma, Acoustic', MS='A benign SCHWANNOMA of the eighth cranial nerve (VESTIBULOCOCHLEAR NERVE), mostly arising from the vestibular branch (VESTIBULAR NERVE) during the fifth or sixth decade of life. Clinical manifestations include HEARING LOSS; HEADACHE; VERTIGO; TINNITUS; and FACIAL PAIN. Bilateral acoustic neuromas are associated with NEUROFIBROMATOSIS 2. (From Adams et al., Principles of Neurology, 6th ed, p673)'),
  'D009465': NtO(MH='Neuromuscular Agents', MS='Drugs used for their actions on skeletal muscle. Included are agents that act directly on skeletal muscle, those that alter neuromuscular transmission (NEUROMUSCULAR BLOCKING AGENTS), and drugs that act centrally as skeletal muscle relaxants (MUSCLE RELAXANTS, CENTRAL). Drugs used in the treatment of movement disorders are ANTI-DYSKINESIA AGENTS.'),
  'D009466': NtO(MH='Neuromuscular Blocking Agents', MS='Drugs that interrupt transmission of nerve impulses at the skeletal neuromuscular junction. They can be of two types, competitive, stabilizing blockers (NEUROMUSCULAR NONDEPOLARIZING AGENTS) or noncompetitive, depolarizing agents (NEUROMUSCULAR DEPOLARIZING AGENTS). Both prevent acetylcholine from triggering the muscle contraction and they are used as anesthesia adjuvants, as relaxants during electroshock, in convulsive states, etc.'),
  'D009467': NtO(MH='Neuromuscular Depolarizing Agents', MS='Drugs that interrupt transmission at the skeletal neuromuscular junction by causing sustained depolarization of the motor end plate. These agents are primarily used as adjuvants in surgical anesthesia to cause skeletal muscle relaxation.'),
  'D009468': NtO(MH='Neuromuscular Diseases', MS='A general term encompassing lower MOTOR NEURON DISEASE; PERIPHERAL NERVOUS SYSTEM DISEASES; and certain MUSCULAR DISEASES. Manifestations include MUSCLE WEAKNESS; FASCICULATION; muscle ATROPHY; SPASM; MYOKYMIA; MUSCLE HYPERTONIA, myalgias, and MUSCLE HYPOTONIA.'),
  'D009469': NtO(MH='Neuromuscular Junction', MS='The synapse between a neuron and a muscle.'),
  'D009470': NtO(MH='Muscle Spindles', MS='Skeletal muscle structures that function as the MECHANORECEPTORS responsible for the stretch or myotactic reflex (REFLEX, STRETCH). They are composed of a bundle of encapsulated SKELETAL MUSCLE FIBERS, i.e., the intrafusal fibers (nuclear bag 1 fibers, nuclear bag 2 fibers, and nuclear chain fibers) innervated by SENSORY NEURONS.'),
  'D009471': NtO(MH='Neuromyelitis Optica', MS='A syndrome characterized by acute OPTIC NEURITIS; MYELITIS, TRANSVERSE; demyelinating and/or necrotizing lesions in the OPTIC NERVES and SPINAL CORD; and presence of specific autoantibodies to AQUAPORIN 4.'),
  'D009472': NtO(MH='Neuronal Ceroid-Lipofuscinoses', MS='A group of severe neurodegenerative diseases characterized by intracellular accumulation of autofluorescent wax-like lipid materials (CEROID; LIPOFUSCIN) in neurons. There are several subtypes based on mutations of the various genes, time of disease onset, and severity of the neurological defects such as progressive DEMENTIA; SEIZURES; and visual failure.'),
  'D009473': NtO(MH='Neuronal Plasticity', MS='The capacity of the NERVOUS SYSTEM to change its reactivity as the result of successive activations.'),
  'D009474': NtO(MH='Neurons', MS='The basic cellular units of nervous tissue. Each neuron consists of a body, an axon, and dendrites. Their purpose is to receive, conduct, and transmit impulses in the NERVOUS SYSTEM.'),
  'D009475': NtO(MH='Neurons, Afferent', MS='Neurons which conduct NERVE IMPULSES to the CENTRAL NERVOUS SYSTEM.'),
  'D009476': NtO(MH='Neurons, Efferent', MS='Neurons which send impulses peripherally to activate muscles or secretory cells.'),
  'D009477': NtO(MH='Hereditary Sensory and Autonomic Neuropathies', MS='A group of inherited disorders characterized by degeneration of dorsal root and autonomic ganglion cells, and clinically by loss of sensation and autonomic dysfunction. There are five subtypes. Type I features autosomal dominant inheritance and distal sensory involvement. Type II is characterized by autosomal inheritance and distal and proximal sensory loss. Type III is DYSAUTONOMIA, FAMILIAL. Type IV features insensitivity to pain, heat intolerance, and mental deficiency. Type V is characterized by a selective loss of pain with intact light touch and vibratory sensation. (From Joynt, Clinical Neurology, 1995, Ch51, pp142-4)'),
  'D009478': NtO(MH='Neuropeptide Y', MS='A 36-amino acid peptide present in many organs and in many sympathetic noradrenergic neurons. It has vasoconstrictor and natriuretic activity and regulates local blood flow, glandular secretion, and smooth muscle activity. The peptide also stimulates feeding and drinking behavior and influences secretion of pituitary hormones.'),
  'D009479': NtO(MH='Neuropeptides', MS='Peptides released by NEURONS as intercellular messengers. Many neuropeptides are also hormones released by non-neuronal cells.'),
  'D009480': NtO(MH='Neuropharmacology', MS='The branch of pharmacology dealing especially with the action of drugs upon various parts of the nervous system.'),
  'D009481': NtO(MH='Neurophysins', MS='Carrier proteins for OXYTOCIN and VASOPRESSIN. They are polypeptides of about 10-kDa, synthesized in the HYPOTHALAMUS. Neurophysin I is associated with oxytocin and neurophysin II is associated with vasopressin in their respective precursors and during transportation down the axons to the neurohypophysis (PITUITARY GLAND, POSTERIOR).'),
  'D009482': NtO(MH='Neurophysiology', MS='The scientific discipline concerned with the physiology of the nervous system.'),
  'D009483': NtO(MH='Neuropsychological Tests', MS='Tests designed to assess neurological function associated with certain behaviors. They are used in diagnosing brain dysfunction or damage and central nervous system disorders or injury.'),
  'D009484': NtO(MH='Neuropsychology', MS='A branch of psychology which investigates the correlation between experience or behavior and the basic neurophysiological processes. The term neuropsychology stresses the dominant role of the nervous system. It is a more narrowly defined field than physiological psychology or psychophysiology.'),
  'D009485': NtO(MH='Neuroradiography', MS='Radiography of the central nervous system.'),
  'D009488': NtO(MH='Neurosciences', MS='The scientific disciplines concerned with the embryology, anatomy, physiology, biochemistry, pharmacology, etc., of the nervous system.'),
  'D009489': NtO(MH='Neurosecretion', MS='The production and release of substances such as NEUROTRANSMITTERS or HORMONES from nerve cells.'),
  'D009490': NtO(MH='Neurosecretory Systems', MS='A system of NEURONS that has the specialized function to produce and secrete HORMONES, and that constitutes, in whole or in part, an ENDOCRINE SYSTEM or organ.'),
  'D009491': NtO(MH='Neurospora', MS='A genus of ascomycetous fungi, family Sordariaceae, order SORDARIALES, comprising bread molds. They are capable of converting tryptophan to nicotinic acid and are used extensively in genetic and enzyme research. (Dorland, 27th ed)'),
  'D009492': NtO(MH='Neurospora crassa', MS='A species of ascomycetous fungi of the family Sordariaceae, order SORDARIALES, much used in biochemical, genetic, and physiologic studies.'),
  'D009493': NtO(MH='Neurosurgery', MS='A surgical specialty concerned with the treatment of diseases and disorders of the brain, spinal cord, and peripheral and sympathetic nervous system.'),
  'D009494': NtO(MH='Neurosyphilis', MS='Infections of the central nervous system caused by TREPONEMA PALLIDUM which present with a variety of clinical syndromes. The initial phase of infection usually causes a mild or asymptomatic meningeal reaction. The meningovascular form may present acutely as BRAIN INFARCTION. The infection may also remain subclinical for several years. Late syndromes include general paresis; TABES DORSALIS; meningeal syphilis; syphilitic OPTIC ATROPHY; and spinal syphilis. General paresis is characterized by progressive DEMENTIA; DYSARTHRIA; TREMOR; MYOCLONUS; SEIZURES; and Argyll-Robertson pupils. (Adams et al., Principles of Neurology, 6th ed, pp722-8)'),
  'D009496': NtO(MH='Neurotensin', MS='A biologically active tridecapeptide isolated from the hypothalamus. It has been shown to induce hypotension in the rat, to stimulate contraction of guinea pig ileum and rat uterus, and to cause relaxation of rat duodenum. There is also evidence that it acts as both a peripheral and a central nervous system neurotransmitter.'),
  'D009497': NtO(MH='Neurotic Disorders', MS='Disorders in which the symptoms are distressing to the individual and recognized by him or her as being unacceptable. Social relationships may be greatly affected but usually remain within acceptable limits. The disturbance is relatively enduring or recurrent without treatment.'),
  'D009498': NtO(MH='Neurotoxins', MS='Toxic substances from microorganisms, plants or animals that interfere with the functions of the nervous system. Most venoms contain neurotoxic substances. Myotoxins are included in this concept.'),
  'D009499': NtO(MH='Neutral Red', MS='A vital dye used as an indicator and biological stain. Various adverse effects have been observed in biological systems.'),
  'D009500': NtO(MH='Neutralization Tests', MS='The measurement of infection-blocking titer of ANTISERA by testing a series of dilutions for a given virus-antiserum interaction end-point, which is generally the dilution at which tissue cultures inoculated with the serum-virus mixtures demonstrate cytopathology (CPE) or the dilution at which 50% of test animals injected with serum-virus mixtures show infectivity (ID50) or die (LD50).'),
  'D009501': NtO(MH='Neutron Activation Analysis', MS='Activation analysis in which the specimen is bombarded with neutrons. Identification is made by measuring the resulting radioisotopes. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D009502': NtO(MH='Neutrons', MS='Electrically neutral elementary particles found in all atomic nuclei except light hydrogen; the mass is equal to that of the proton and electron combined and they are unstable when isolated from the nucleus, undergoing beta decay. Slow, thermal, epithermal, and fast neutrons refer to the energy levels with which the neutrons are ejected from heavier nuclei during their decay.'),
  'D009503': NtO(MH='Neutropenia', MS='A decrease in the number of NEUTROPHILS found in the blood.'),
  'D009504': NtO(MH='Neutrophils', MS='Granular leukocytes having a nucleus with three to five lobes connected by slender threads of chromatin, and cytoplasm containing fine inconspicuous granules and stainable by neutral dyes.'),
  'D009505': NtO(MH='Nevada', MS='State bounded on the north by Oregon and Idaho, on the east by Utah, on the south and west by California.'),
  'D009506': NtO(MH='Nevus', MS='A circumscribed stable malformation of the skin and occasionally of the oral mucosa, which is not due to external causes and therefore presumed to be of hereditary origin.'),
  'D009507': NtO(MH='Nevus of Ota', MS='A macular lesion on the side of the FACE, involving the CONJUNCTIVA and EYELIDS, as well as the adjacent facial skin, SCLERA; OCULOMOTOR MUSCLES; and PERIOSTEUM. Histological features vary from those of a MONGOLIAN SPOT to those of a BLUE NEVUS.'),
  'D009508': NtO(MH='Nevus, Pigmented', MS='A nevus containing melanin. The term is usually restricted to nevocytic nevi (round or oval collections of melanin-containing nevus cells occurring at the dermoepidermal junction of the skin or in the dermis proper) or moles, but may be applied to other pigmented nevi.'),
  'D009509': NtO(MH='New Brunswick', MS="A province of eastern Canada, one of the Maritime Provinces with NOVA SCOTIA; PRINCE EDWARD ISLAND; and sometimes NEWFOUNDLAND AND LABRADOR. Its capital is Fredericton. It was named in honor of King George III, of the House of Hanover, also called Brunswick. (From Webster's New Geographical Dictionary, 1988, p828 & Room, Brewer's Dictionary of Names, 1992, p375)"),
  'D009510': NtO(MH='New Caledonia', MS="A group of islands in Melanesia constituting a French overseas territory. The group includes New Caledonia (the main island), Ile des Pins, Loyalty Island, and several other islet groups. The capital is Noumea. It was discovered by Captain Cook in 1774 and visited by various navigators, explorers, and traders from 1792 to 1840. Occupied by the French in 1853, it was set up as a penal colony 1864-94. In 1946 it was made a French overseas territory. It was named by Captain Cook with the 5th and 6th century A.D. Latin name for Scotland, Caledonia. (From Webster's New Geographical Dictionary, 1988, p830 & Room, Brewer's Dictionary of Names, 1992, p375)"),
  'D009511': NtO(MH='New England', MS='The geographic area of New England in general and when the specific state or states are not indicated. States usually included in this region are Maine, New Hampshire, Vermont, Massachusetts, Connecticut, and Rhode Island.'),
  'D009512': NtO(MH='New Guinea', MS='Originally an island of the Malay Archipelago, the second largest island in the world. It divided, West New Guinea becoming part of Indonesia and East New Guinea becoming Papua New Guinea.'),
  'D009513': NtO(MH='New Hampshire', MS='State bounded on the north by Canada, on the east by Maine, on the west by Vermont, and on the south by Massachusetts.'),
  'D009514': NtO(MH='Vanuatu', MS="A republic consisting of an island group in Melanesia, in the southwest Pacific Ocean. Its capital is Port-Vila. It was called New Hebrides until 1980. It was discovered in 1606 by the Portuguese, forgotten for 160 years, then visited by Bougainville in 1768 and Captain Cook in 1774. It was under joint British and French administration from 1906 until it became independent in 1980 under the name of Vanuatu. The name is native, meaning our land. (From Webster's New Geographical Dictionary, 1988, p833 & Room, Brewer's Dictionary of Names, 1992, p570)"),
  'D009515': NtO(MH='New Jersey', MS='State bounded on the north by New York and Pennsylvania, on the east by New York and the Atlantic Ocean, on the south by Delaware Bay, and on the west by Pennsylvania.'),
  'D009516': NtO(MH='New Mexico', MS='State bounded on the north by Colorado, on the east by Oklahoma and Texas, on the south by Texas and Mexico, and on the west by Arizona.'),
  'D009517': NtO(MH='New South Wales', MS="A state in southeastern Australia. Its capital is Sydney. It was discovered by Captain Cook in 1770 and first settled at Botany Bay by marines and convicts in 1788. It was named by Captain Cook who thought its coastline resembled that of South Wales. (From Webster's New Geographical Dictionary, 1988, p840 & Room, Brewer's Dictionary of Names, 1992, p377)"),
  'D009518': NtO(MH='New York', MS='State bounded on the north by Lake Ontario and Canada, on the east by Vermont, Massachusetts, and Connecticut, on the south by the Atlantic Ocean, New Jersey, and Pennsylvania, and on the west by Pennsylvania, Lake Erie, and Canada.'),
  'D009519': NtO(MH='New York City', MS='City located at the mouth of the Hudson River, in New York State.'),
  'D009520': NtO(MH='New Zealand', MS="A group of islands in the southwest Pacific. Its capital is Wellington. It was discovered by the Dutch explorer Abel Tasman in 1642 and circumnavigated by Cook in 1769. Colonized in 1840 by the New Zealand Company, it became a British crown colony in 1840 until 1907 when colonial status was terminated. New Zealand is a partly anglicized form of the original Dutch name Nieuw Zeeland, new sea land, possibly with reference to the Dutch province of Zeeland. (From Webster's New Geographical Dictionary, 1988, p842 & Room, Brewer's Dictionary of Names, 1992, p378)"),
  'D009521': NtO(MH='Newcastle Disease', MS='An acute febrile, contagious, viral disease of birds caused by an AVULAVIRUS called NEWCASTLE DISEASE VIRUS. It is characterized by respiratory and nervous symptoms in fowl and is transmissible to man causing a severe, but transient conjunctivitis.'),
  'D009522': NtO(MH='Newcastle disease virus', MS='The most well known avian paramyxovirus in the genus AVULAVIRUS and the cause of a highly infectious pneumoencephalitis in fowl. It is also reported to cause CONJUNCTIVITIS in humans. Transmission is by droplet inhalation or ingestion of contaminated water or food.'),
  'D009523': NtO(MH='Newfoundland and Labrador', MS="Province of Canada consisting of the island of Newfoundland and an area of Labrador. Its capital is St. John's."),
  'D009524': NtO(MH='Newspapers as Topic', MS="Publications printed and distributed daily, weekly, or at some other regular and usually short interval, containing news, articles of opinion (as editorials and letters), features, advertising, and announcements of current interest. (Webster's 3d ed)"),
  'D009525': NtO(MH='Niacin', MS='A water-soluble vitamin of the B complex occurring in various animal and plant tissues. It is required by the body for the formation of coenzymes NAD and NADP. It has PELLAGRA-curative, vasodilating, and antilipemic properties.'),
  'D009526': NtO(MH='Nialamide', MS='An MAO inhibitor that is used as an antidepressive agent.'),
  'D009527': NtO(MH='Nicaragua', MS='A country in CENTRAL AMERICA, bordering both the Caribbean Sea and the North Pacific Ocean, between COSTA RICA and HONDURAS.'),
  'D009528': NtO(MH='Nicarbazin', MS="An equimolar complex of 4,4'-Dinitrocarbanilide and 2-Hydroxy-4,6-dimethylpyrimidine. A coccidiostat for poultry."),
  'D009529': NtO(MH='Nicardipine', MS='A potent calcium channel blockader with marked vasodilator action. It has antihypertensive properties and is effective in the treatment of angina and coronary spasms without showing cardiodepressant effects. It has also been used in the treatment of asthma and enhances the action of specific antineoplastic agents.'),
  'D009530': NtO(MH='Nicergoline', MS='An ergot derivative that has been used as a cerebral vasodilator and in peripheral vascular disease. It may ameliorate cognitive deficits in CEREBROVASCULAR DISORDERS.'),
  'D009531': NtO(MH='Niceritrol', MS='An ester of nicotinic acid that lowers cholesterol and triglycerides in total plasma and in the VLD- and LD-lipoprotein fractions.'),
  'D009532': NtO(MH='Nickel', MS='A trace element with the atomic symbol Ni, atomic number 28, and atomic weight 58.69. It is a cofactor of the enzyme UREASE.'),
  'D009533': NtO(MH='Niclofolan', MS='Proposed fasciolacide. Synonyms: Dertil; Bilevon; Bay 9015.'),
  'D009534': NtO(MH='Niclosamide', MS='An antihelmintic that is active against most tapeworms. (From Martindale, The Extra Pharmacopoeia, 30th ed, p48)'),
  'D009535': NtO(MH='Nicotinamidase', MS='An enzyme that catalyzes the hydrolysis of nicotinamide to nicotinate and ammonia. EC 3.5.1.19.'),
  'D009536': NtO(MH='Niacinamide', MS='An important compound functioning as a component of the coenzyme NAD. Its primary significance is in the prevention and/or cure of blacktongue and PELLAGRA. Most animals cannot manufacture this compound in amounts sufficient to prevent nutritional deficiency and it therefore must be supplemented through dietary intake.'),
  'D009537': NtO(MH='Nicotinamide Mononucleotide', MS="3-Carbamoyl-1-beta-D-ribofuranosyl pyridinium hydroxide-5'phosphate, inner salt. A nucleotide in which the nitrogenous base, nicotinamide, is in beta-N-glycosidic linkage with the C-1 position of D-ribose. Synonyms: Nicotinamide Ribonucleotide; NMN."),
  'D009538': NtO(MH='Nicotine', MS='Nicotine is highly toxic alkaloid. It is the prototypical agonist at nicotinic cholinergic receptors where it dramatically stimulates neurons and ultimately blocks synaptic transmission. Nicotine is also important medically because of its presence in tobacco smoke.'),
  'D009539': NtO(MH='Nicotinic Acids', MS='2-, 3-, or 4-Pyridinecarboxylic acids. Pyridine derivatives substituted with a carboxy group at the 2-, 3-, or 4-position. The 3-carboxy derivative (NIACIN) is active as a vitamin.'),
  'D009540': NtO(MH='Nicotinyl Alcohol', MS='Alcohol analog of NICOTINIC ACID which is a direct-acting peripheral vasodilator that causes flushing and may decrease blood pressure. It is used in vasospasm and threatened GANGRENE.'),
  'D009541': NtO(MH='Nictitating Membrane', MS='A fold of the mucous membrane of the CONJUNCTIVA in many animals. At rest, it is hidden in the medial canthus. It can extend to cover part or all of the cornea to help clean the CORNEA.'),
  'D009542': NtO(MH='Niemann-Pick Diseases', MS='A group of autosomal recessive disorders in which harmful quantities of lipids accumulate in the viscera and the central nervous system. They can be caused by deficiencies of enzyme activities (SPHINGOMYELIN PHOSPHODIESTERASE) or defects in intracellular transport, resulting in the accumulation of SPHINGOMYELINS and CHOLESTEROL. There are various subtypes based on their clinical and genetic differences.'),
  'D009543': NtO(MH='Nifedipine', MS='A potent vasodilator agent with calcium antagonistic action. It is a useful anti-anginal agent that also lowers blood pressure.'),
  'D009544': NtO(MH='Niflumic Acid', MS='An analgesic and anti-inflammatory agent used in the treatment of rheumatoid arthritis.'),
  'D009545': NtO(MH='Nifuratel', MS='Local antiprotozoal and antifungal agent that may also be given orally.'),
  'D009547': NtO(MH='Nifurtimox', MS='A nitrofuran thiazine that has been used against TRYPANOSOMIASIS.'),
  'D009548': NtO(MH='Niger', MS='A republic in western Africa, north of NIGERIA and west of CHAD. Its capital is Niamey.'),
  'D009549': NtO(MH='Nigeria', MS='A republic in western Africa, south of NIGER between BENIN and CAMEROON. Its capital is Abuja.'),
  'D009550': NtO(MH='Nigericin', MS='A polyether antibiotic which affects ion transport and ATPase activity in mitochondria. It is produced by Streptomyces hygroscopicus. (From Merck Index, 11th ed)'),
  'D009551': NtO(MH='Night Care', MS='Institutional night care of patients.'),
  'D009552': NtO(MH='Nikethamide', MS='A central nervous system stimulant. It was formerly used in the treatment of barbiturate overdose but is now considered to be of no value for such purposes and may be dangerous. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1229)'),
  'D009553': NtO(MH='Nimodipine', MS='A calcium channel blockader with preferential cerebrovascular activity. It has marked cerebrovascular dilating effects and lowers blood pressure.'),
  'D009554': NtO(MH='Nimorazole', MS='An antitrichomonal agent which is effective either topically or orally and whose urinary metabolites are also trichomonicidal.'),
  'D009555': NtO(MH='Ninhydrin', MS='2,2-Dihydroxy-1H-indene-1,3-(2H)-dione. Reagent toxic to skin and mucus membranes. It is used in chemical assay for peptide bonds, i.e., protein determinations and has radiosensitizing properties.'),
  'D009556': NtO(MH='Niobium', MS='A metal element atomic number 41, atomic weight 92.906, symbol Nb.'),
  'D009557': NtO(MH='Nipecotic Acids', MS=None),
  'D009558': NtO(MH='Nipples', MS='The conic organs which usually give outlet to milk from the mammary glands.'),
  'D009559': NtO(MH='Nippostrongylus', MS='A genus of intestinal nematode parasites belonging to the superfamily HELIGMOSOMATOIDEA, which commonly occurs in rats but has been experimentally transmitted to other rodents and rabbits. Infection is usually through the skin.'),
  'D009560': NtO(MH='Niridazole', MS='An antischistosomal agent that has become obsolete.'),
  'D009561': NtO(MH='Nisin', MS='A 34-amino acid polypeptide antibiotic produced by Streptococcus lactis. It has been used as a food preservative in canned fruits and vegetables, and cheese.'),
  'D009562': NtO(MH='Nissl Bodies', MS='Subcellular structures found in nerve cell bodies and DENDRITES. They consist of granular endoplasmic reticulum (ENDOPLASMIC RETICULUM, ROUGH) and RIBOSOMES.'),
  'D009564': NtO(MH='Nitracrine', MS='Acridine antineoplastic agent used in mammary and ovarian tumors. It inhibits RNA synthesis.'),
  'D009565': NtO(MH='Nitrate Reductases', MS='Oxidoreductases that are specific for the reduction of NITRATES.'),
  'D009566': NtO(MH='Nitrates', MS='Inorganic or organic salts and esters of nitric acid. These compounds contain the NO3- radical.'),
  'D009567': NtO(MH='Nitrazepam', MS='A benzodiazepine derivative used as an anticonvulsant and hypnotic.'),
  'D009568': NtO(MH='Nitrendipine', MS='A calcium channel blocker with marked vasodilator action. It is an effective antihypertensive agent and differs from other calcium channel blockers in that it does not reduce glomerular filtration rate and is mildly natriuretic, rather than sodium retentive.'),
  'D009569': NtO(MH='Nitric Oxide', MS='A free radical gas produced endogenously by a variety of mammalian cells, synthesized from ARGININE by NITRIC OXIDE SYNTHASE. Nitric oxide is one of the ENDOTHELIUM-DEPENDENT RELAXING FACTORS released by the vascular endothelium and mediates VASODILATION. It also inhibits platelet aggregation, induces disaggregation of aggregated platelets, and inhibits platelet adhesion to the vascular endothelium. Nitric oxide activates cytosolic GUANYLATE CYCLASE and thus elevates intracellular levels of CYCLIC GMP.'),
  'D009570': NtO(MH='Nitriles', MS='Organic compounds containing the -CN radical. The concept is distinguished from CYANIDES, which denotes inorganic salts of HYDROGEN CYANIDE.'),
  'D009571': NtO(MH='Nitrilotriacetic Acid', MS="A derivative of acetic acid, N(CH2COOH)3. It is a complexing (sequestering) agent that forms stable complexes with Zn2+. (From Miall's Dictionary of Chemistry, 5th ed.)"),
  'D009572': NtO(MH='Nitrite Reductases', MS='A group of enzymes that oxidize diverse nitrogenous substances to yield nitrite. (Enzyme Nomenclature, 1992) EC 1.'),
  'D009573': NtO(MH='Nitrites', MS="Salts of nitrous acid or compounds containing the group NO2-. The inorganic nitrites of the type MNO2 (where M=metal) are all insoluble, except the alkali nitrites. The organic nitrites may be isomeric, but not identical with the corresponding nitro compounds. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D009574': NtO(MH='Nitro Compounds', MS='Compounds having the nitro group, -NO2, attached to carbon. When attached to nitrogen they are nitramines and attached to oxygen they are NITRATES.'),
  'D009575': NtO(MH='Nitroanisole O-Demethylase', MS='Oxidative enzyme which transforms p-nitroanisole into p-nitrophenol.'),
  'D009576': NtO(MH='Nitrobacter', MS='A genus of gram-negative, rod-shaped bacteria that oxidizes nitrites to nitrates. Its organisms occur in aerobic environments where organic matter is being mineralized, including soil, fresh water, and sea water.'),
  'D009578': NtO(MH='Nitrobenzenes', MS='BENZENE derivatives carrying nitro group substituents.'),
  'D009579': NtO(MH='Nitrobenzoates', MS='Benzoic acid or benzoic acid esters substituted with one or more nitro groups.'),
  'D009580': NtO(MH='Nitroblue Tetrazolium', MS='Colorless to yellow dye that is reducible to blue or black formazan crystals by certain cells; formerly used to distinguish between nonbacterial and bacterial diseases, the latter causing neutrophils to reduce the dye; used to confirm diagnosis of chronic granulomatous disease.'),
  'D009581': NtO(MH='Nitrofurans', MS='Compounds containing FURANS attached to a nitro group.'),
  'D009582': NtO(MH='Nitrofurantoin', MS='A urinary anti-infective agent effective against most gram-positive and gram-negative organisms. Although sulfonamides and antibiotics are usually the agents of choice for urinary tract infections, nitrofurantoin is widely used for prophylaxis and long-term suppression.'),
  'D009583': NtO(MH='Nitrofurazone', MS='A topical anti-infective agent effective against gram-negative and gram-positive bacteria. It is used for superficial WOUNDS AND INJURIES and skin infections. Nitrofurazone has also been administered orally in the treatment of TRYPANOSOMIASIS.'),
  'D009584': NtO(MH='Nitrogen', MS="An element with the atomic symbol N, atomic number 7, and atomic weight [14.00643; 14.00728]. Nitrogen exists as a diatomic gas and makes up about 78% of the earth's atmosphere by volume. It is a constituent of proteins and nucleic acids and found in all living cells."),
  'D009585': NtO(MH='Nitrogen Dioxide', MS="Nitrogen oxide (NO2). A highly poisonous gas. Exposure produces inflammation of lungs that may only cause slight pain or pass unnoticed, but resulting edema several days later may cause death. (From Merck, 11th ed) It is a major atmospheric pollutant that is able to absorb UV light that does not reach the earth's surface."),
  'D009586': NtO(MH='Nitrogen Fixation', MS='The process in certain BACTERIA; FUNGI; and CYANOBACTERIA converting free atmospheric NITROGEN to biologically usable forms of nitrogen, such as AMMONIA; NITRATES; and amino compounds.'),
  'D009587': NtO(MH='Nitrogen Isotopes', MS='Stable nitrogen atoms that have the same atomic number as the element nitrogen, but differ in atomic weight. N-15 is a stable nitrogen isotope.'),
  'D009588': NtO(MH='Nitrogen Mustard Compounds', MS='A group of alkylating agents derived from mustard gas, with the sulfur replaced by nitrogen. They were formerly used as toxicants and vesicants, but now function as antineoplastic agents. These compounds are also powerful mutagens, teratogens, immunosuppressants, and carcinogens.'),
  'D009589': NtO(MH='Nitrogen Oxides', MS='Inorganic oxides that contain nitrogen.'),
  'D009590': NtO(MH='Nitrogen Radioisotopes', MS='Unstable isotopes of nitrogen that decay or disintegrate emitting radiation. N atoms with atomic weights 12, 13, 16, 17, and 18 are radioactive nitrogen isotopes.'),
  'D009591': NtO(MH='Nitrogenase', MS='An enzyme system that catalyzes the fixing of nitrogen in soil bacteria and blue-green algae (CYANOBACTERIA). EC 1.18.6.1.'),
  'D009592': NtO(MH='Nitrohydroxyiodophenylacetate', MS='Also called 4-hydroxy-3-iodo-5-nitrophenylacetate. A haptenic determinant that can be radiolabeled and used as salts and derivatives for investigations of immunogenic specificity studies.'),
  'D009593': NtO(MH='Nitroimidazoles', MS='IMIDAZOLES having a nitro moiety.'),
  'D009594': NtO(MH='Nitromifene', MS='A non-steroidal estrogen antagonist (as the 1:1 citrate) most commonly used as a research tool in animal studies.'),
  'D009595': NtO(MH='Nitroparaffins', MS='Alkanes having one or more nitro (NO2) groups attached to carbon atoms.'),
  'D009596': NtO(MH='Nitrophenols', MS='PHENOLS carrying nitro group substituents.'),
  'D009597': NtO(MH='4-Nitrophenylphosphatase', MS='An enzyme that catalyzes the hydrolysis of nitrophenyl phosphates to nitrophenols. At acid pH it is probably ACID PHOSPHATASE (EC 3.1.3.2); at alkaline pH it is probably ALKALINE PHOSPHATASE (EC 3.1.3.1). EC 3.1.3.41.'),
  'D009598': NtO(MH='Nitrophenylgalactosides', MS='Includes ortho-, meta-, and para-nitrophenylgalactosides.'),
  'D009599': NtO(MH='Nitroprusside', MS='A powerful vasodilator used in emergencies to lower blood pressure or to improve cardiac function. It is also an indicator for free sulfhydryl groups in proteins.'),
  'D009600': NtO(MH='Nitroquinolines', MS='Quinolines substituted in any position by one or more nitro groups.'),
  'D009601': NtO(MH='Nitroreductases', MS='Enzymes which reduce nitro groups (NITRO COMPOUNDS) and other nitrogenous compounds.'),
  'D009602': NtO(MH='Nitrosamines', MS='A class of compounds that contain a -NH2 and a -NO radical. Many members of this group have carcinogenic and mutagenic properties.'),
  'D009603': NtO(MH='Nitroso Compounds', MS='Organic compounds containing the nitroso (-N=O) group attached to an organic moiety.'),
  'D009604': NtO(MH='Nitrosoguanidines', MS='Nitrosylated derivatives of guanidine. They are used as MUTAGENS in MOLECULAR BIOLOGY research.'),
  'D009605': NtO(MH='Nitrosomethylurethane', MS='An alkylating carcinogen that produces gastrointestinal and probably lung and nervous system tumors.'),
  'D009606': NtO(MH='Nitrosomonas', MS='A genus of gram-negative, ellipsoidal or rod-shaped bacteria whose major source of energy and reducing power is from the oxidation of ammonia to nitrite. Its species occur in soils, oceans, lakes, rivers, and sewage disposal systems.'),
  'D009607': NtO(MH='Nitrosourea Compounds', MS='A class of compounds in which the core molecule is R-NO, where R is UREA.'),
  'D009608': NtO(MH='Nitrous Acid', MS='Nitrous acid (HNO2). A weak acid that exists only in solution. It can form water-soluble nitrites and stable esters. (From Merck Index, 11th ed)'),
  'D009609': NtO(MH='Nitrous Oxide', MS='Nitrogen oxide (N2O). A colorless, odorless gas that is used as an anesthetic and analgesic. High concentrations cause a narcotic effect and may replace oxygen, causing death by asphyxia. It is also used as a food aerosol in the preparation of whipping cream.'),
  'D009610': NtO(MH='Nitrovin', MS='An antibacterial growth promoter used in animal feeds.'),
  'D009611': NtO(MH='Nitroxinil', MS='Proposed anthelmintic for fasciola and liver fluke infestations.'),
  'D009612': NtO(MH='Nicotinamide-Nucleotide Adenylyltransferase', MS='An enzyme that catalyzes reversibly the transfer of the adenylyl moiety of ATP to the phosphoryl group of NMN to form NAD+ and pyrophosphate. The enzyme is found predominantly in the nuclei and catalyzes the final reaction in the major pathway for the biosynthesis of NAD in mammals.'),
  'D009613': NtO(MH='Nobel Prize', MS='Any of six international prizes awarded annually for outstanding work in physics, chemistry, physiology or medicine, literature, economics and the promotion of peace.'),
  'D009614': NtO(MH='Nobelium', MS='A man-made radioactive element of the actinide metal series. It has the atomic symbol No, and atomic number 102.'),
  'D009615': NtO(MH='Nocardia', MS='A genus of gram-positive, aerobic bacteria whose species are widely distributed and are abundant in soil. Some strains are pathogenic opportunists for humans and animals.'),
  'D009616': NtO(MH='Nocardia asteroides', MS='A species of bacterium of the family NOCARDIACEAE, producing pulmonary infections in man.'),
  'D009617': NtO(MH='Nocardia Infections', MS='Infections with bacteria of the genus NOCARDIA.'),
  'D009618': NtO(MH='Nocardiaceae', MS='A family of gram-positive, aerobic actinomycetes found in soil and animal tissue. Some species are the cause of infection in man and animals.'),
  'D009619': NtO(MH='Nociceptors', MS='Peripheral AFFERENT NEURONS which are sensitive to injuries or pain, usually caused by extreme thermal exposures, mechanical forces, or other noxious stimuli. Their cell bodies reside in the DORSAL ROOT GANGLIA. Their peripheral terminals (NERVE ENDINGS) innervate target tissues and transduce noxious stimuli via axons to the CENTRAL NERVOUS SYSTEM.'),
  'D009620': NtO(MH='Nodose Ganglion', MS='The inferior (caudal) ganglion of the vagus (10th cranial) nerve. The unipolar nodose ganglion cells are sensory cells with central projections to the medulla and peripheral processes traveling in various branches of the vagus nerve.'),
  'D009621': NtO(MH='Nogalamycin', MS='An anthrocycline from a Streptomyces nogalater variant. It is a cytolytic antineoplastic that inhibits DNA-dependent RNA synthesis by binding to DNA.'),
  'D009622': NtO(MH='Noise', MS='Any sound which is unwanted or interferes with HEARING other sounds.'),
  'D009623': NtO(MH='Noise, Occupational', MS='Noise present in occupational, industrial, and factory situations.'),
  'D009624': NtO(MH='Noise, Transportation', MS='Noise associated with transportation, particularly aircraft and automobiles.'),
  'D009625': NtO(MH='Noma', MS='A severe gangrenous process occurring predominantly in debilitated and malnourished children, especially in underdeveloped countries. It typically begins as a small vesicle or ulcer on the gingiva that rapidly becomes necrotic and spreads to produce extensive destruction of the buccal and labial mucosa and tissues of the face, which may result in severe disfigurement and even death. Various bacteria have been implicated in the etiology. (Dorland, 27th ed)'),
  'D009626': NtO(MH='Terminology as Topic', MS='Works about the terms, expressions, designations, or symbols used in a particular science, discipline, or specialized subject area.'),
  'D009627': NtO(MH='Nomifensine', MS='An isoquinoline derivative that prevents dopamine reuptake into synaptosomes. The maleate was formerly used in the treatment of depression. It was withdrawn worldwide in 1986 due to the risk of acute hemolytic anemia with intravascular hemolysis resulting from its use. In some cases, renal failure also developed. (From Martindale, The Extra Pharmacopoeia, 30th ed, p266)'),
  'D009628': NtO(MH='Nonachlazine', MS='Coronary vasodilator with a novel mechanism of action; proposed as antianginal agent.'),
  'D009629': NtO(MH='Person-Centered Psychotherapy', MS='A nondirective psychotherapy approach originated by Carl Rogers. The goals of therapy are to promote the client\xe2\x80\x99s congruence, self awareness, and self acceptance. This approach views the client as naturally directed toward self actualization, and only needing facilitative conditions in order to promote this tendency.'),
  'D009630': NtO(MH='Nondisjunction, Genetic', MS='The failure of homologous CHROMOSOMES or CHROMATIDS to segregate during MITOSIS or MEIOSIS with the result that one daughter cell has both of a pair of parental chromosomes or chromatids and the other has none.'),
  'D009631': NtO(MH='Nonodontogenic Cysts', MS='Cysts formed from epithelial inclusions in the lines of fusion of the embryonic processes which form the jaws. They include nasopalatine or incisive canal cyst, incisive papilla cyst, globulomaxillary cyst, median palatal cyst, median alveolar cyst, median mandibular cyst, and nasoalveolar cyst.'),
  'D009632': NtO(MH='Nonsuppressible Insulin-Like Activity', MS='A blood protein (NSILA) which mimics the biological activity of insulin in serum, but is not suppressed by insulin antibodies. During acid-ethanol extraction of Cohn fraction III, 10% of the activity is found in the supernatant (NSILA-S) and the remaining activity in the precipitate (NSILA-P). The latter is a large molecular compound, much less stable than the soluble fraction. NSILA-S is a more potent growth factor than insulin and exhibits sulfation activity.'),
  'D009633': NtO(MH='Nonverbal Communication', MS='Transmission of emotions, ideas, and attitudes between individuals in ways other than the spoken language.'),
  'D009634': NtO(MH='Noonan Syndrome', MS='A genetically heterogeneous, multifaceted disorder characterized by short stature, webbed neck, ptosis, skeletal malformations, hypertelorism, hormonal imbalance, CRYPTORCHIDISM, multiple cardiac abnormalities (most commonly including PULMONARY VALVE STENOSIS), and some degree of INTELLECTUAL DISABILITY. The phenotype bears similarities to that of TURNER SYNDROME that occurs only in females and has its basis in a 45, X karyotype abnormality. Noonan syndrome occurs in both males and females with a normal karyotype (46,XX and 46,XY). Mutations in a several genes (PTPN11, KRAS, SOS1, NF1 and RAF1) have been associated the NS phenotype. Mutations in PTPN11 are the most common. LEOPARD SYNDROME, a disorder that has clinical features overlapping those of Noonan Syndrome, is also due to mutations in PTPN11. In addition, there is overlap with the syndrome called neurofibromatosis-Noonan syndrome due to mutations in NF1.'),
  'D009635': NtO(MH='Norandrostanes', MS='Androstanes in which ring contractions have occurred or which are lacking carbon-18. Compounds with any degree of unsaturation are included. Androstanes which are lacking carbon-19 are ESTRANES.'),
  'D009636': NtO(MH='Norbornanes', MS='Compounds that include or are derivatives of norbornane(bicyclo[2.2.1]heptane).'),
  'D009637': NtO(MH='Masoprocol', MS='A potent lipoxygenase inhibitor that interferes with arachidonic acid metabolism. The compound also inhibits formyltetrahydrofolate synthetase, carboxylesterase, and cyclooxygenase to a lesser extent. It also serves as an antioxidant in fats and oils.'),
  'D009638': NtO(MH='Norepinephrine', MS='Precursor of epinephrine that is secreted by the ADRENAL MEDULLA and is a widespread central and autonomic neurotransmitter. Norepinephrine is the principal transmitter of most postganglionic sympathetic fibers, and of the diffuse projection system in the brain that arises from the LOCUS CERULEUS. It is also found in plants and is used pharmacologically as a sympathomimetic.'),
  'D009639': NtO(MH='Norethandrolone', MS='A synthetic hormone with anabolic and androgenic properties and moderate progestational activity.'),
  'D009640': NtO(MH='Norethindrone', MS='A synthetic progestational hormone with actions similar to those of PROGESTERONE but functioning as a more potent inhibitor of ovulation. It has weak estrogenic and androgenic properties. The hormone has been used in treating amenorrhea, functional uterine bleeding, endometriosis, and for CONTRACEPTION.'),
  'D009641': NtO(MH='Norethynodrel', MS='A synthetic progestational hormone with actions and uses similar to those of PROGESTERONE. It has been used in the treatment of functional uterine bleeding and ENDOMETRIOSIS. As a contraceptive (CONTRACEPTIVE AGENTS), it has usually been administered in combination with MESTRANOL.'),
  'D009642': NtO(MH='Norfenfluramine', MS='A FENFLURAMINE analog that inhibits serotonin uptake and may provoke release of serotonin. It is used as an appetite depressant and an experimental tool in animal studies.'),
  'D009643': NtO(MH='Norfloxacin', MS='A synthetic fluoroquinolone (FLUOROQUINOLONES) with broad-spectrum antibacterial activity against most gram-negative and gram-positive bacteria. Norfloxacin inhibits bacterial DNA GYRASE.'),
  'D009644': NtO(MH='Norgestrel', MS='A synthetic progestational agent with actions similar to those of PROGESTERONE. This racemic or (+-)-form has about half the potency of the levo form (LEVONORGESTREL). Norgestrel is used as a contraceptive, ovulation inhibitor, and for the control of menstrual disorders and endometriosis.'),
  'D009645': NtO(MH='Norgestrienone', MS='A synthetic steroid with progestational and contraceptive activities.'),
  'D009646': NtO(MH='Norleucine', MS='An unnatural amino acid that is used experimentally to study protein structure and function. It is structurally similar to METHIONINE, however it does not contain SULFUR.'),
  'D009647': NtO(MH='Normetanephrine', MS='A methylated metabolite of norepinephrine that is excreted in the urine and found in certain tissues. It is a marker for tumors.'),
  'D009649': NtO(MH='Norpregnadienes', MS='Pregnadienes which have undergone ring contractions or are lacking carbon-18 or carbon-19.'),
  'D009650': NtO(MH='Norpregnanes', MS='Pregnanes which have undergone ring contractions or are lacking carbon-18 or carbon-19. Unsaturation in the steroid ring system is not included.'),
  'D009651': NtO(MH='Norpregnatrienes', MS='Pregnatrienes which have undergone ring contractions or are lacking carbon-18 or carbon-19.'),
  'D009652': NtO(MH='Norpregnenes', MS='Pregnenes with one double bond or more than three double bonds which have undergone ring contractions or are lacking carbon-18 or carbon-19..'),
  'D009653': NtO(MH='Norprogesterones', MS='Progesterones which have undergone ring contraction or which are lacking carbon 18 or 19.'),
  'D009654': NtO(MH='Norsteroids', MS='Steroids which have undergone contraction in ring size or reduction in side chains.'),
  'D009655': NtO(MH='Octopamine', MS='An alpha-adrenergic sympathomimetic amine, biosynthesized from tyramine in the CNS and platelets and also in invertebrate nervous systems. It is used to treat hypotension and as a cardiotonic. The natural D(-) form is more potent than the L(+) form in producing cardiovascular adrenergic responses. It is also a neurotransmitter in some invertebrates.'),
  'D009656': NtO(MH='North America', MS='The northern continent of the Western Hemisphere, extending northward from the Colombia-Panama border and including CENTRAL AMERICA, MEXICO, Caribbean area, the UNITED STATES, CANADA and GREENLAND. The term often refers more narrowly to MEXICO, continental UNITED STATES, AND CANADA.'),
  'D009657': NtO(MH='North Carolina', MS='State bounded on the north by Virginia, on the east and Southeast by the Atlantic Ocean, on the south by Georgia and South Carolina, and on the west by Tennessee.'),
  'D009658': NtO(MH='North Dakota', MS='State bounded on the north by Canada, on the east by Minnesota, on the south by South Dakota, and on the west by Montana.'),
  'D009659': NtO(MH='North Sea', MS='An arm of the Atlantic Ocean extending between Europe on the south and east, Great Britain on the west.'),
  'D009660': NtO(MH='Northern Ireland', MS='The northern one-sixth of the island of Ireland - between the North Atlantic Ocean and the North Sea; northwest of France. The capital is Belfast.'),
  'D009661': NtO(MH='Nortriptyline', MS='A metabolite of AMITRIPTYLINE that is also used as an antidepressive agent. Nortriptyline is used in major depression, dysthymia, and atypical depressions.'),
  'D009662': NtO(MH='Nortropanes', MS='Derivatives of 8-azabicyclo[3.2.1]octane.'),
  'D009663': NtO(MH='Norwalk virus', MS='The type species in the genus NOROVIRUS, first isolated in 1968 from the stools of school children in Norwalk, Ohio, who were suffering from GASTROENTERITIS. The virions are non-enveloped spherical particles containing a single protein. Multiple strains are named after the places where outbreaks have occurred.'),
  'D009664': NtO(MH='Norway', MS='A country located in northern Europe, bordering the North Sea and the Atlantic Ocean, west of Sweden. The capital is Oslo.'),
  'D009665': NtO(MH='Noscapine', MS='A naturally occurring opium alkaloid that is a centrally acting antitussive agent.'),
  'D009666': NtO(MH='Nose', MS='A part of the upper respiratory tract. It contains the organ of SMELL. The term includes the external nose, the nasal cavity, and the PARANASAL SINUSES.'),
  'D009667': NtO(MH='Nose Deformities, Acquired', MS='Abnormalities of the nose acquired after birth from injury or disease.'),
  'D009668': NtO(MH='Nose Diseases', MS='Disorders of the nose, general or unspecified.'),
  'D009669': NtO(MH='Nose Neoplasms', MS='Tumors or cancer of the NOSE.'),
  'D009670': NtO(MH='Encephalitozoon cuniculi', MS='A species of parasitic FUNGI. This intracellular parasite is found in the BRAIN; HEART; and KIDNEYS of several MAMMALS. Transmission is probably by ingestion of the spores (SPORES, FUNGAL).'),
  'D009671': NtO(MH='Nostrums', MS='Medicines whose effectiveness is unproven and whose ingredients are often secret.'),
  'D009672': NtO(MH='Notochord', MS='A cartilaginous rod of mesodermal cells at the dorsal midline of all CHORDATE embryos. In lower vertebrates, notochord is the backbone of support. In the higher vertebrates, notochord is a transient structure, and segments of the vertebral column will develop around it. Notochord is also a source of midline signals that pattern surrounding tissues including the NEURAL TUBE development.'),
  'D009673': NtO(MH='Notophthalmus viridescens', MS='A species of newt in the Salamandridae family in which the larvae transform into terrestrial eft stage and later into an aquatic adult. They occur from Canada to southern United States. Viridescens refers to the greenish color often found in this species.'),
  'D009674': NtO(MH='Nova Scotia', MS="A province of eastern Canada, one of the Maritime Provinces with NEW BRUNSWICK; PRINCE EDWARD ISLAND; and sometimes NEWFOUNDLAND AND LABRADOR. Its capital is Halifax. The territory was granted in 1621 by James I to the Scotsman Sir William Alexander and was called Nova Scotia, the Latin for New Scotland. The territory had earlier belonged to the French, under the name of Acadia. (From Webster's New Geographical Dictionary, 1988, p871 & Room, Brewer's Dictionary of Names, 1992, p384)"),
  'D009675': NtO(MH='Novobiocin', MS='An antibiotic compound derived from Streptomyces niveus. It has a chemical structure similar to coumarin. Novobiocin binds to DNA gyrase, and blocks adenosine triphosphatase (ATPase) activity. (From Reynolds, Martindale The Extra Pharmacopoeia, 30th ed, p189)'),
  'D009676': NtO(MH='Noxae', MS='Agents capable of exerting a harmful effect on the body.'),
  'D009677': NtO(MH='Noxythiolin', MS='Local antibacterial that probably acts by releasing formaldehyde in aqueous solutions. It is used for THERAPEUTIC IRRIGATION of infected body cavities - bladder, peritoneum, etc. and as a spray for burns.'),
  'D009678': NtO(MH='Nuclear Energy', MS='Energy released by nuclear fission or nuclear fusion.'),
  'D009679': NtO(MH='Nuclear Family', MS='A family composed of spouses and their children.'),
  'D009680': NtO(MH='Nuclear Fission', MS='Nuclear reaction in which the nucleus of a heavy atom such as uranium or plutonium is split into two approximately equal parts by a neutron, charged particle, or photon.'),
  'D009681': NtO(MH='Nuclear Fusion', MS='Thermonuclear reaction in which the nuclei of an element of low atomic weight unite under extremely high temperature and pressure to form a nucleus of a heavier atom.'),
  'D009682': NtO(MH='Magnetic Resonance Spectroscopy', MS='Spectroscopic method of measuring the magnetic moment of elementary particles such as atomic nuclei, protons or electrons. It is employed in clinical applications such as NMR Tomography (MAGNETIC RESONANCE IMAGING).'),
  'D009683': NtO(MH='Nuclear Medicine', MS='A specialty field of radiology concerned with diagnostic, therapeutic, and investigative use of radioactive compounds.'),
  'D009684': NtO(MH='Nuclear Medicine Department, Hospital', MS='Hospital department responsible for the administration and management of nuclear medicine services.'),
  'D009685': NtO(MH='Nuclear Envelope', MS='The membrane system of the CELL NUCLEUS that surrounds the nucleoplasm. It consists of two concentric membranes separated by the perinuclear space. The structures of the envelope where it opens to the cytoplasm are called the nuclear pores (NUCLEAR PORE).'),
  'D009686': NtO(MH='Nuclear Physics', MS='The study of the characteristics, behavior, and internal structures of the atomic nucleus and its interactions with other nuclei. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D009687': NtO(MH='Nuclear Proteins', MS='Proteins found in the nucleus of a cell. Do not confuse with NUCLEOPROTEINS which are proteins conjugated with nucleic acids, that are not necessarily present in the nucleus.'),
  'D009688': NtO(MH='Nuclear Reactors', MS='Devices containing fissionable material in sufficient quantity and so arranged as to be capable of maintaining a controlled, self-sustaining NUCLEAR FISSION chain reaction. They are also known as atomic piles, atomic reactors, fission reactors, and nuclear piles, although such names are deprecated. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D009689': NtO(MH='Nuclear Warfare', MS='Warfare involving the use of NUCLEAR WEAPONS.'),
  'D009690': NtO(MH='Nucleic Acid Conformation', MS='The spatial arrangement of the atoms of a nucleic acid or polynucleotide that results in its characteristic 3-dimensional shape.'),
  'D009691': NtO(MH='Nucleic Acid Denaturation', MS='Disruption of the secondary structure of nucleic acids by heat, extreme pH or chemical treatment. Double strand DNA is "melted" by dissociation of the non-covalent hydrogen bonds and hydrophobic interactions. Denatured DNA appears to be a single-stranded flexible structure. The effects of denaturation on RNA are similar though less pronounced and largely reversible.'),
  'D009692': NtO(MH='Nucleic Acid Heteroduplexes', MS='Double-stranded nucleic acid molecules (DNA-DNA or DNA-RNA) which contain regions of nucleotide mismatches (non-complementary). In vivo, these heteroduplexes can result from mutation or genetic recombination; in vitro, they are formed by nucleic acid hybridization. Electron microscopic analysis of the resulting heteroduplexes facilitates the mapping of regions of base sequence homology of nucleic acids.'),
  'D009693': NtO(MH='Nucleic Acid Hybridization', MS='Widely used technique which exploits the ability of complementary sequences in single-stranded DNAs or RNAs to pair with each other to form a double helix. Hybridization can take place between two complimentary DNA sequences, between a single-stranded DNA and a complementary RNA, or between two RNA sequences. The technique is used to detect and isolate specific sequences, measure homology, or define other characteristics of one or both strands. (Kendrew, Encyclopedia of Molecular Biology, 1994, p503)'),
  'D009694': NtO(MH='Nucleic Acid Precursors', MS='Use for nucleic acid precursors in general or for which there is no specific heading.'),
  'D009695': NtO(MH='Nucleic Acid Renaturation', MS='The reformation of all, or part of, the native conformation of a nucleic acid molecule after the molecule has undergone denaturation.'),
  'D009696': NtO(MH='Nucleic Acids', MS='High molecular weight polymers containing a mixture of purine and pyrimidine nucleotides chained together by ribose or deoxyribose linkages.'),
  'D009697': NtO(MH='Nucleolus Organizer Region', MS='The chromosome region which is active in nucleolus formation and which functions in the synthesis of ribosomal RNA.'),
  'D009698': NtO(MH='Nucleoproteins', MS='Proteins conjugated with nucleic acids.'),
  'D009699': NtO(MH='N-Glycosyl Hydrolases', MS='A class of enzymes involved in the hydrolysis of the N-glycosidic bond of nitrogen-linked sugars.'),
  'D009700': NtO(MH='Nucleoside Deaminases', MS='Catalyze the hydrolysis of nucleosides with the elimination of ammonia.'),
  'D009701': NtO(MH='Nucleoside-Diphosphate Kinase', MS='An enzyme that is found in mitochondria and in the soluble cytoplasm of cells. It catalyzes reversible reactions of a nucleoside triphosphate, e.g., ATP, with a nucleoside diphosphate, e.g., UDP, to form ADP and UTP. Many nucleoside diphosphates can act as acceptor, while many ribo- and deoxyribonucleoside triphosphates can act as donor. EC 2.7.4.6.'),
  'D009702': NtO(MH='Nucleoside Diphosphate Sugars', MS=None),
  'D009703': NtO(MH='Nucleoside-Phosphate Kinase', MS='An enzyme that catalyzes reversible reactions of a nucleoside triphosphate, e.g., ATP, with a nucleoside monophosphate, e.g., UMP, to form ADP and UDP. Many nucleoside monophosphates can act as acceptor while many ribo- and deoxyribonucleoside triphosphates can act as donor. EC 2.7.4.4.'),
  'D009704': NtO(MH='Nucleoside Q', MS='A modified nucleoside which is present in the first position of the anticodon of tRNA-tyrosine, tRNA-histidine, tRNA-asparagine and tRNA-aspartic acid of many organisms. It is believed to play a role in the regulatory function of tRNA. Nucleoside Q can be further modified to nucleoside Q*, which has a mannose or galactose moiety linked to position 4 of its cyclopentenediol moiety.'),
  'D009705': NtO(MH='Nucleosides', MS='Purine or pyrimidine bases attached to a ribose or deoxyribose. (From King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D009706': NtO(MH='Nucleic Acids, Nucleotides, and Nucleosides', MS='Complex compounds of high molecular weight occurring in living cells. These are basically of two types, ribonucleic (RNA) and deoxyribonucleic (DNA) acids, both of which consist of nucleotides (nucleoside phosphates linked together by phosphate bridges).'),
  'D009707': NtO(MH='Nucleosomes', MS='The repeating structural units of chromatin, each consisting of approximately 200 base pairs of DNA wound around a protein core. This core is composed of the histones H2A, H2B, H3, and H4.'),
  'D009708': NtO(MH='Nucleotidases', MS='A class of enzymes that catalyze the conversion of a nucleotide and water to a nucleoside and orthophosphate. EC 3.1.3.-.'),
  'D009709': NtO(MH='Nucleotide Deaminases', MS='Catalyze the hydrolysis of nucleotides with the elimination of ammonia.'),
  'D009710': NtO(MH='Nucleotide Mapping', MS='Two-dimensional separation and analysis of nucleotides.'),
  'D009711': NtO(MH='Nucleotides', MS='The monomeric units from which DNA or RNA polymers are constructed. They consist of a purine or pyrimidine base, a pentose sugar, and a phosphate group. (From King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D009712': NtO(MH='Nucleotides, Cyclic', MS=None),
  'D009713': NtO(MH='Nucleotidyltransferases', MS='A class of enzymes that transfers nucleotidyl residues. EC 2.7.7.'),
  'D009714': NtO(MH='Nucleus Accumbens', MS='Collection of pleomorphic cells in the caudal part of the anterior horn of the LATERAL VENTRICLE, in the region of the OLFACTORY TUBERCLE, lying between the head of the CAUDATE NUCLEUS and the ANTERIOR PERFORATED SUBSTANCE. It is part of the so-called VENTRAL STRIATUM, a composite structure considered part of the BASAL GANGLIA.'),
  'D009715': NtO(MH='Nudism', MS='The practice of living unclothed for reasons of comfort or health.'),
  'D009716': NtO(MH='Numerical Analysis, Computer-Assisted', MS='Computer-assisted study of methods for obtaining useful quantitative solutions to problems that have been expressed mathematically.'),
  'D009717': NtO(MH='Numismatics', MS='Study of coins, tokens, medals, etc. However, it usually refers to medals pertaining to the history of medicine.'),
  'D009718': NtO(MH='Nurse Administrators', MS='Nurses professionally qualified in administration.'),
  'D009719': NtO(MH='Nurse Anesthetists', MS='Professional nurses who have completed postgraduate training in the administration of anesthetics.'),
  'D009720': NtO(MH='Nurse Clinicians', MS="Registered nurses who hold Master's degrees in nursing with an emphasis in clinical nursing and who function independently in coordinating plans for patient care."),
  'D009721': NtO(MH='Nurse Midwives', MS='Professional nurses who have received postgraduate training in MIDWIFERY.'),
  'D009722': NtO(MH='Nurse Practitioners', MS='Nurses who are specially trained to assume an expanded role in providing medical care under the supervision of a physician.'),
  'D009723': NtO(MH='Nurse-Patient Relations', MS='Interaction between the patient and nurse.'),
  'D009724': NtO(MH='Nurseries, Infant', MS='Facilities which provide care for infants.'),
  'D009725': NtO(MH='Nurseries, Hospital', MS='Hospital facilities which provide care for newborn infants.'),
  'D009726': NtO(MH='Nurses', MS='Professionals qualified by graduation from an accredited school of nursing and by passage of a national licensing examination to practice nursing. They provide services to patients requiring assistance in recovering or maintaining their physical or mental health.'),
  'D009727': NtO(MH='Nurses, Male', MS='Nurses of the male sex.'),
  'D009728': NtO(MH='Nursing Assistants', MS=None),
  'D009729': NtO(MH='Nursing', MS='The field of nursing care concerned with the promotion, maintenance, and restoration of health.'),
  'D009730': NtO(MH='Nursing Assessment', MS='Evaluation of the nature and extent of nursing problems presented by a patient for the purpose of patient care planning.'),
  'D009731': NtO(MH='Nursing Audit', MS='A detailed review and evaluation of selected clinical records by qualified professional personnel for evaluating quality of nursing care.'),
  'D009732': NtO(MH='Nursing Care', MS='Care given to patients by nursing service personnel.'),
  'D009733': NtO(MH='Nursing Diagnosis', MS='Conclusions derived from the nursing assessment that establish a health status profile for the patient and from which nursing interventions may be ordered.'),
  'D009734': NtO(MH='Nursing Faculty Practice', MS='Clinical practice by members of the nursing faculty in order to maintain a balance in their nursing activities--clinical, education, and research.'),
  'D009735': NtO(MH='Nursing Homes', MS='Facilities which provide nursing supervision and limited medical care to persons who do not require hospitalization.'),
  'D009736': NtO(MH='Nursing Process', MS='The sum total of nursing activities which includes assessment (identifying needs), intervention (ministering to needs), and evaluation (validating the effectiveness of the help given).'),
  'D009737': NtO(MH='Nursing Records', MS="Data recorded by nurses concerning the nursing care given to the patient, including judgment of the patient's progress."),
  'D009738': NtO(MH='Nursing Service, Hospital', MS='The hospital department which is responsible for the organization and administration of nursing activities.'),
  'D009739': NtO(MH='Nursing Services', MS='A general concept referring to the organization and administration of nursing activities.'),
  'D009740': NtO(MH='Nursing Staff', MS='Personnel who provide nursing service to patients in an organized facility, institution, or agency.'),
  'D009741': NtO(MH='Nursing Staff, Hospital', MS='Personnel who provide nursing service to patients in a hospital.'),
  'D009742': NtO(MH='Nursing Theory', MS='Concepts, definitions, and propositions applied to the study of various phenomena which pertain to nursing and nursing research.'),
  'D009743': NtO(MH='Nursing, Practical', MS='The practice of nursing by licensed, non-registered persons qualified to provide routine care to the sick.'),
  'D009744': NtO(MH='Nursing, Private Duty', MS='The practice of nursing by a registered or licensed nurse to care for a specific patient in a health facility or in the home.'),
  'D009745': NtO(MH='Nursing, Supervisory', MS='Administration of nursing services for one or more clinical units.'),
  'D009746': NtO(MH='Nursing, Team', MS="Coordination of nursing services by various nursing care personnel under the leadership of a professional nurse. The team may consist of a professional nurse, nurses' aides, and the practical nurse."),
  'D009747': NtO(MH='Nutritional Physiological Phenomena', MS='The processes and properties of living organisms by which they take in and balance the use of nutritive materials for energy, heat production, or building material for the growth, maintenance, or repair of tissues and the nutritive properties of FOOD.'),
  'D009748': NtO(MH='Nutrition Disorders', MS='Disorders caused by nutritional imbalance, either overnutrition or undernutrition.'),
  'D009749': NtO(MH='Nutrition Surveys', MS='A systematic collection of factual data pertaining to the nutritional status of a human population within a given geographic area. Data from these surveys are used in preparing NUTRITION ASSESSMENTS.'),
  'D009750': NtO(MH='Nutritional and Metabolic Diseases', MS='A collective term for nutritional disorders resulting from poor absorption or nutritional imbalance, and metabolic disorders resulting from defects in biosynthesis (ANABOLISM) or breakdown (CATABOLISM) of endogenous substances.'),
  'D009751': NtO(MH='Nutritional Requirements', MS='The amounts of various substances in food needed by an organism to sustain healthy life.'),
  'D009752': NtO(MH='Nutritional Status', MS='State of the body in relation to the consumption and utilization of nutrients.'),
  'D009753': NtO(MH='Nutritive Value', MS='An indication of the contribution of a food to the nutrient content of the diet. This value depends on the quantity of a food which is digested and absorbed and the amounts of the essential nutrients (protein, fat, carbohydrate, minerals, vitamins) which it contains. This value can be affected by soil and growing conditions, handling and storage, and processing.'),
  'D009754': NtO(MH='Nuts', MS='Botanically, a type of single-seeded fruit in which the pericarp enclosing the seed is a hard woody shell. In common usage the term is used loosely for any hard, oil-rich kernel. Of those commonly eaten, only hazel, filbert, and chestnut are strictly nuts. Walnuts, pecans, almonds, and coconuts are really drupes. Brazil nuts, pistachios, macadamias, and cashews are really seeds with a hard shell derived from the testa rather than the pericarp.'),
  'D009755': NtO(MH='Night Blindness', MS='Failure or imperfection of vision at night or in dim light, with good vision only on bright days. (Dorland, 27th ed)'),
  'D009756': NtO(MH='Nylidrin', MS='A beta-adrenergic agonist. Nylidrin causes peripheral vasodilation, a positive inotropic effect, and increased gastric volume of gastric juice. It is used in the treatment of peripheral vascular disorders and premature labor.'),
  'D009757': NtO(MH='Nylons', MS='Polymers where the main polymer chain comprises recurring amide groups. These compounds are generally formed from combinations of diamines, diacids, and amino acids and yield fibers, sheeting, or extruded forms used in textiles, gels, filters, sutures, contact lenses, and other biomaterials.'),
  'D009758': NtO(MH='Nymph', MS='The immature stage in the life cycle of those orders of insects characterized by gradual metamorphosis, in which the young resemble the imago in general form of body, including compound eyes and external wings; also the 8-legged stage of mites and ticks that follows the first moult.'),
  'D009759': NtO(MH='Nystagmus, Pathologic', MS='Involuntary movements of the eye that are divided into two types, jerk and pendular. Jerk nystagmus has a slow phase in one direction followed by a corrective fast phase in the opposite direction, and is usually caused by central or peripheral vestibular dysfunction. Pendular nystagmus features oscillations that are of equal velocity in both directions and this condition is often associated with visual loss early in life. (Adams et al., Principles of Neurology, 6th ed, p272)'),
  'D009760': NtO(MH='Nystagmus, Physiologic', MS='Involuntary rhythmical movements of the eyes in the normal person. These can be naturally occurring as in end-position (end-point, end-stage, or deviational) nystagmus or induced by the optokinetic drum (NYSTAGMUS, OPTOKINETIC), caloric test, or a rotating chair.'),
  'D009761': NtO(MH='Nystatin', MS='Macrolide antifungal antibiotic complex produced by Streptomyces noursei, S. aureus, and other Streptomyces species. The biologically active components of the complex are nystatin A1, A2, and A3.'),
  'D009762': NtO(MH='o-Aminoazotoluene', MS='An azo dye with carcinogenic properties.'),
  'D009763': NtO(MH='o-Chlorobenzylidenemalonitrile', MS='A riot control agent which causes temporary irritation of the eyes and the mucosal surface of the respiratory tract. It is a more potent irritant than OMEGA-CHLOROACETOPHENONE, but less incapacitating.'),
  'D009764': NtO(MH='o-Phthalaldehyde', MS='A reagent that forms fluorescent conjugation products with primary amines. It is used for the detection of many biogenic amines, peptides, and proteins in nanogram quantities in body fluids.'),
  'D009765': NtO(MH='Obesity', MS='A status with BODY WEIGHT that is grossly above the acceptable or desirable weight, usually due to accumulation of excess FATS in the body. The standards may vary with age, sex, genetic or cultural background. In the BODY MASS INDEX, a BMI greater than 30.0 kg/m2 is considered obese, and a BMI greater than 40.0 kg/m2 is considered morbidly obese (MORBID OBESITY).'),
  'D009767': NtO(MH='Obesity, Morbid', MS='The condition of weighing two, three, or more times the ideal weight, so called because it is associated with many serious and life-threatening disorders. In the BODY MASS INDEX, morbid obesity is defined as having a BMI greater than 40.0 kg/m2.'),
  'D009768': NtO(MH='Obidoxime Chloride', MS='Cholinesterase reactivator occurring in two interchangeable isomeric forms, syn and anti.'),
  'D009769': NtO(MH='Object Attachment', MS='Emotional attachment to someone or something in the environment.'),
  'D009770': NtO(MH='Obsessive Behavior', MS='Persistent, unwanted idea or impulse which is considered normal when it does not markedly interfere with mental processes or emotional adjustment.'),
  'D009771': NtO(MH='Obsessive-Compulsive Disorder', MS='An anxiety disorder characterized by recurrent, persistent obsessions or compulsions. Obsessions are the intrusive ideas, thoughts, or images that are experienced as senseless or repugnant. Compulsions are repetitive and seemingly purposeful behavior which the individual generally recognizes as senseless and from which the individual does not derive pleasure although it may provide a release from tension.'),
  'D009772': NtO(MH='Obstetrical Forceps', MS='Surgical instrument designed to extract the newborn by the head from the maternal passages without injury to it or the mother.'),
  'D009773': NtO(MH='Obstetric Nursing', MS='A nursing specialty involving nursing care given to the pregnant patient before, after, or during childbirth.'),
  'D009774': NtO(MH='Obstetrics', MS='A medical-surgical specialty concerned with management and care of women during pregnancy, parturition, and the puerperium.'),
  'D009775': NtO(MH='Obstetrics and Gynecology Department, Hospital', MS='Hospital department responsible for the administration and management of services provided for obstetric and gynecologic patients.'),
  'D009776': NtO(MH='Obturator Nerve', MS='A nerve originating in the lumbar spinal cord (L2 to L4) and traveling through the lumbar plexus to the lower extremity. The obturator nerve provides motor innervation to the adductor muscles of the thigh and cutaneous sensory innervation of the inner thigh.'),
  'D009777': NtO(MH='Occipital Bone', MS='Part of the back and base of the CRANIUM that encloses the FORAMEN MAGNUM.'),
  'D009778': NtO(MH='Occipital Lobe', MS='Posterior portion of the CEREBRAL HEMISPHERES responsible for processing visual sensory information. It is located posterior to the parieto-occipital sulcus and extends to the preoccipital notch.'),
  'D009779': NtO(MH='Occlusive Dressings', MS='Material, usually gauze or absorbent cotton, used to cover and protect wounds, to seal them from contact with air or bacteria. (From Dorland, 27th ed)'),
  'D009780': NtO(MH='Occult Blood', MS='Chemical, spectroscopic, or microscopic detection of extremely small amounts of blood.'),
  'D009781': NtO(MH='Occultism', MS='The belief in or study of practices and knowledge of magical, mystical, or supernatural powers. ALCHEMY, astrology, and many RELIGIOUS PHILOSOPHIES are based upon occult principles.'),
  'D009782': NtO(MH='Occupational Dentistry', MS='The branch of dentistry concerned with maintaining oral health of employees in occupational environments.'),
  'D009783': NtO(MH='Dermatitis, Occupational', MS='A recurrent contact dermatitis caused by substances found in the work place.'),
  'D009784': NtO(MH='Occupational Diseases', MS="Diseases caused by factors involved in one's employment."),
  'D009785': NtO(MH='Occupational Health Nursing', MS='The practice of nursing in the work environment.'),
  'D009786': NtO(MH='Occupational Health Services', MS='Health services for employees, usually provided by the employer at the place of work.'),
  'D009787': NtO(MH='Occupational Medicine', MS='Medical specialty concerned with the promotion and maintenance of the physical and mental health of employees in occupational settings.'),
  'D009788': NtO(MH='Occupational Therapy', MS='Skilled treatment that helps individuals achieve independence in all facets of their lives. It assists in the development of skills needed for independent living.'),
  'D009789': NtO(MH='Occupational Therapy Department, Hospital', MS='Hospital department responsible for the administration and provision of any occupational or work activity for remedial purposes.'),
  'D009790': NtO(MH='Occupations', MS='Crafts, trades, professions, or other means of earning a living.'),
  'D009791': NtO(MH='Oceanography', MS='The science that deals with the ocean and its phenomena. (Webster, 3d ed)'),
  'D009792': NtO(MH='Oceans and Seas', MS="A great expanse of continuous bodies of salt water which together cover more than 70 percent of the earth's surface. Seas may be partially or entirely enclosed by land, and are smaller than the five oceans (Atlantic, Pacific, Indian, Arctic, and Antarctic)."),
  'D009793': NtO(MH='Ochratoxins', MS='Isocoumarins found in ASPERGILLUS OCHRACEUS and other FUNGI. Ochratoxin contaminated FOOD has been responsible for cases of FOODBORNE DISEASES.'),
  'D009794': NtO(MH='Ochronosis', MS='The yellowish discoloration of connective tissue due to deposition of HOMOGENTISIC ACID (a brown-black pigment). This is due to defects in the metabolism of PHENYLALANINE and TYROSINE. Ochronosis occurs in ALKAPTONURIA, but has also been associated with exposure to certain chemicals (e.g., PHENOL, trinitrophenol, BENZENE DERIVATIVES).'),
  'D009795': NtO(MH='Octanes', MS='Eight-carbon saturated hydrocarbon group of the methane series. Include isomers and derivatives.'),
  'D009798': NtO(MH='Ocular Hypertension', MS='A condition in which the intraocular pressure is elevated above normal and which may lead to glaucoma.'),
  'D009799': NtO(MH='Ocular Physiological Phenomena', MS='Processes and properties of the EYE as a whole or of any of its parts.'),
  'D009800': NtO(MH='Oculocerebrorenal Syndrome', MS='A sex-linked recessive disorder affecting multiple systems including the EYE, the NERVOUS SYSTEM, and the KIDNEY. Clinical features include congenital CATARACT; MENTAL RETARDATION; and renal tubular dysfunction (FANCONI SYNDROME; RENAL TUBULAR ACIDOSIS; X-LINKED HYPOPHOSPHATEMIA or vitamin-D-resistant rickets) and SCOLIOSIS. This condition is due to a deficiency of phosphatidylinositol 4,5-bisphosphate-5-phosphatase leading to defects in PHOSPHATIDYLINOSITOL metabolism and INOSITOL signaling pathway. (from Menkes, Textbook of Child Neurology, 5th ed, p60; Am J Hum Genet 1997 Jun;60(6):1384-8)'),
  'D009801': NtO(MH='Oculomotor Muscles', MS='The muscles that move the eye. Included in this group are the medial rectus, lateral rectus, superior rectus, inferior rectus, inferior oblique, superior oblique, musculus orbitalis, and levator palpebrae superioris.'),
  'D009802': NtO(MH='Oculomotor Nerve', MS='The 3d cranial nerve. The oculomotor nerve sends motor fibers to the levator muscles of the eyelid and to the superior rectus, inferior rectus, and inferior oblique muscles of the eye. It also sends parasympathetic efferents (via the ciliary ganglion) to the muscles controlling pupillary constriction and accommodation. The motor fibers originate in the oculomotor nuclei of the midbrain.'),
  'D009803': NtO(MH='Sphincter of Oddi', MS='The sphincter of the hepatopancreatic ampulla within the duodenal papilla. The COMMON BILE DUCT and main pancreatic duct pass through this sphincter.'),
  'D009804': NtO(MH='Odontoblasts', MS='The mesenchymal cells which line the DENTAL PULP CAVITY and produce DENTIN. They have a columnar morphology in the coronal pulp but are cuboidal in the root pulp, or when adjacent to tertiary dentin.'),
  'D009805': NtO(MH='Odontogenesis', MS='The process of TOOTH formation. It is divided into several stages including: the dental lamina stage, the bud stage, the cap stage, and the bell stage. Odontogenesis includes the production of tooth enamel (AMELOGENESIS), dentin (DENTINOGENESIS), and dental cementum (CEMENTOGENESIS).'),
  'D009807': NtO(MH='Odontogenic Cysts', MS='Cysts found in the jaws and arising from epithelium involved in tooth formation. They include follicular cysts (e.g., primordial cyst, dentigerous cyst, multilocular cyst), lateral periodontal cysts, and radicular cysts. They may become keratinized (odontogenic keratocysts). Follicular cysts may give rise to ameloblastomas and, in rare cases, undergo malignant transformation.'),
  'D009808': NtO(MH='Odontogenic Tumors', MS='Neoplasms produced from tooth-forming tissues.'),
  'D009809': NtO(MH='Odontoid Process', MS='The toothlike process on the upper surface of the axis, which articulates with the CERVICAL ATLAS above.'),
  'D009810': NtO(MH='Odontoma', MS='A mixed tumor of odontogenic origin, in which both the epithelial and mesenchymal cells exhibit complete differentiation, resulting in the formation of tooth structures. (Jablonski, Illustrated Dictionary of Dentistry, 1982)'),
  'D009811': NtO(MH='Odontometry', MS='Measurement of tooth characteristics.'),
  'D009812': NtO(MH='Odorants', MS='The volatile portions of chemical substances perceptible by the sense of smell.'),
  'D009813': NtO(MH='Oedipus Complex', MS='Attachment of the child to the parent of the opposite sex, accompanied by envious and aggressive feelings toward the parent of the same sex.'),
  'D009814': NtO(MH='Oesophagostomiasis', MS='Infection of the intestinal tract with worms of the genus OESOPHAGOSTOMUM. This condition occurs mainly in animals other than man.'),
  'D009815': NtO(MH='Oesophagostomum', MS='A genus of nematodes of the superfamily STRONGYLOIDEA, parasitic in the intestines of animals. The adults are usually free in the intestinal lumen; the larvae encyst in the wall.'),
  'D009816': NtO(MH='Office Automation', MS='Use of computers or computer systems for doing routine clerical work, e.g., billing, records pertaining to the administration of the office, etc.'),
  'D009817': NtO(MH='Office Management', MS='Planning, organizing, and administering activities in an office.'),
  'D009818': NtO(MH='Office Nursing', MS='Nursing practice limited to an office setting.'),
  'D009819': NtO(MH='Office Visits', MS="Visits made by patients to health service providers' offices for diagnosis, treatment, and follow-up."),
  'D009820': NtO(MH='Ohio', MS='State bounded on the north by Michigan and Lake Erie, on the east by Ohio River and Pennsylvania, on the south by Ohio River, and on the west by Indiana.'),
  'D009821': NtO(MH='Oils', MS='Unctuous combustible substances that are liquid or easily liquefiable on warming, and are soluble in ether but insoluble in water. Such substances, depending on their origin, are classified as animal, mineral, or vegetable oils. Depending on their behavior on heating, they are volatile or fixed. (Dorland, 28th ed)'),
  'D009822': NtO(MH='Oils, Volatile', MS='Oils which evaporate readily. The volatile oils occur in aromatic plants, to which they give odor and other characteristics. Most volatile oils consist of a mixture of two or more TERPENES or of a mixture of an eleoptene (the more volatile constituent of a volatile oil) with a stearopten (the more solid constituent). The synonym essential oils refers to the essence of a plant, as its perfume or scent, and not to its indispensability.'),
  'D009823': NtO(MH='Ointment Bases', MS='Various mixtures of fats, waxes, animal and plant oils and solid and liquid hydrocarbons; vehicles for medicinal substances intended for external application; there are four classes: hydrocarbon base, absorption base, water-removable base and water-soluble base; several are also emollients.'),
  'D009824': NtO(MH='Ointments', MS='Semisolid preparations used topically for protective emollient effects or as a vehicle for local administration of medications. Ointment bases are various mixtures of fats, waxes, animal and plant oils and solid and liquid hydrocarbons.'),
  'D009825': NtO(MH='Oklahoma', MS='State bounded on the north by Kansas, on the east by Missouri and Arkansas, on the south by Texas, and on the west by New Mexico.'),
  'D009826': NtO(MH='Old Age Assistance', MS='Financial assistance for the impoverished elderly through public funding of programs, services, and individual income supplements.'),
  'D009827': NtO(MH='Oleandomycin', MS='Antibiotic macrolide produced by Streptomyces antibioticus.'),
  'D009828': NtO(MH='Oleanolic Acid', MS='A pentacyclic triterpene that occurs widely in many PLANTS as the free acid or the aglycone for many SAPONINS. It is biosynthesized from lupane. It can rearrange to the isomer, ursolic acid, or be oxidized to taraxasterol and amyrin.'),
  'D009829': NtO(MH='Oleic Acids', MS='A group of fatty acids that contain 18 carbon atoms and a double bond at the omega 9 carbon.'),
  'D009830': NtO(MH='Olfactory Bulb', MS='Ovoid body resting on the CRIBRIFORM PLATE of the ethmoid bone where the OLFACTORY NERVE terminates. The olfactory bulb contains several types of nerve cells including the mitral cells, on whose DENDRITES the olfactory nerve synapses, forming the olfactory glomeruli. The accessory olfactory bulb, which receives the projection from the VOMERONASAL ORGAN via the vomeronasal nerve, is also included here.'),
  'D009831': NtO(MH='Olfactory Mucosa', MS='That portion of the nasal mucosa containing the sensory nerve endings for SMELL, located at the dome of each NASAL CAVITY. The yellow-brownish olfactory epithelium consists of OLFACTORY RECEPTOR NEURONS; brush cells; STEM CELLS; and the associated olfactory glands.'),
  'D009832': NtO(MH='Olfactory Nerve', MS='The 1st cranial nerve. The olfactory nerve conveys the sense of smell. It is formed by the axons of OLFACTORY RECEPTOR NEURONS which project from the olfactory epithelium (in the nasal epithelium) to the OLFACTORY BULB.'),
  'D009833': NtO(MH='Olfactory Pathways', MS='Set of nerve fibers conducting impulses from olfactory receptors to the cerebral cortex. It includes the OLFACTORY NERVE; OLFACTORY BULB; OLFACTORY TRACT; OLFACTORY TUBERCLE; ANTERIOR PERFORATED SUBSTANCE; and OLFACTORY CORTEX.'),
  'D009834': NtO(MH='Oligo-1,6-Glucosidase', MS='An enzyme that catalyzes the endohydrolysis of 1,6-alpha-glucosidic linkages in isomaltose and dextrins produced from starch and glycogen by ALPHA-AMYLASES. EC 3.2.1.10.'),
  'D009835': NtO(MH='Oligochaeta', MS='A class of annelid worms with few setae per segment. It includes the earthworms such as Lumbricus and Eisenia.'),
  'D009836': NtO(MH='Oligodendroglia', MS='A class of large neuroglial (macroglial) cells in the central nervous system. Oligodendroglia may be called interfascicular, perivascular, or perineuronal (not the same as SATELLITE CELLS, PERINEURONAL of GANGLIA) according to their location. They form the insulating MYELIN SHEATH of axons in the central nervous system.'),
  'D009837': NtO(MH='Oligodendroglioma', MS='A relatively slow-growing glioma that is derived from oligodendrocytes and tends to occur in the cerebral hemispheres, thalamus, or lateral ventricle. They may present at any age, but are most frequent in the third to fifth decades, with an earlier incidence peak in the first decade. Histologically, these tumors are encapsulated, relatively avascular, and tend to form cysts and microcalcifications. Neoplastic cells tend to have small round nuclei surrounded by unstained nuclei. The tumors may vary from well-differentiated to highly anaplastic forms. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2052; Adams et al., Principles of Neurology, 6th ed, p655)'),
  'D009838': NtO(MH='Oligodeoxyribonucleotides', MS='A group of deoxyribonucleotides (up to 12) in which the phosphate residues of each deoxyribonucleotide act as bridges in forming diester linkages between the deoxyribose moieties.'),
  'D009839': NtO(MH='Oligomenorrhea', MS='Abnormally infrequent menstruation.'),
  'D009840': NtO(MH='Oligomycins', MS='A closely related group of toxic substances elaborated by various strains of Streptomyces. They are 26-membered macrolides with lactone moieties and double bonds and inhibit various ATPases, causing uncoupling of phosphorylation from mitochondrial respiration. Used as tools in cytochemistry. Some specific oligomycins are RUTAMYCIN, peliomycin, and botrycidin (formerly venturicidin X).'),
  'D009841': NtO(MH='Oligonucleotides', MS='Polymers made up of a few (2-20) nucleotides. In molecular genetics, they refer to a short sequence synthesized to match a region where a mutation is known to occur, and then used as a probe (OLIGONUCLEOTIDE PROBES). (Dorland, 28th ed)'),
  'D009842': NtO(MH='Oligopeptides', MS='Peptides composed of between two and twelve amino acids.'),
  'D009843': NtO(MH='Oligoribonucleotides', MS='A group of ribonucleotides (up to 12) in which the phosphate residues of each ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D009844': NtO(MH='Oligosaccharides', MS='Carbohydrates consisting of between two (DISACCHARIDES) and ten MONOSACCHARIDES connected by either an alpha- or beta-glycosidic link. They are found throughout nature in both the free and bound form.'),
  'D009845': NtO(MH='Oligospermia', MS='A condition of suboptimal concentration of SPERMATOZOA in the ejaculated SEMEN to ensure successful FERTILIZATION of an OVUM. In humans, oligospermia is defined as a sperm count below 20 million per milliliter semen.'),
  'D009846': NtO(MH='Oliguria', MS='Decreased URINE output that is below the normal range. Oliguria can be defined as urine output of less than or equal to 0.5 or 1 ml/kg/hr depending on the age.'),
  'D009847': NtO(MH='Olivary Nucleus', MS='A part of the MEDULLA OBLONGATA situated in the olivary body. It is involved with motor control and is a major source of sensory input to the CEREBELLUM.'),
  'D009848': NtO(MH='Olivomycins', MS='A mixture of several closely related glycosidic antibiotics obtained from Actinomyces (or Streptomyces) olivoreticuli. They are used as fluorescent dyes that bind to DNA and prevent both RNA and protein synthesis and are also used as antineoplastic agents.'),
  'D009849': NtO(MH='Olivopontocerebellar Atrophies', MS='A group of inherited and sporadic disorders which share progressive ataxia in combination with atrophy of the CEREBELLUM; PONS; and inferior olivary nuclei. Additional clinical features may include MUSCLE RIGIDITY; NYSTAGMUS, PATHOLOGIC; RETINAL DEGENERATION; MUSCLE SPASTICITY; DEMENTIA; URINARY INCONTINENCE; and OPHTHALMOPLEGIA. The familial form has an earlier onset (second decade) and may feature spinal cord atrophy. The sporadic form tends to present in the fifth or sixth decade, and is considered a clinical subtype of MULTIPLE SYSTEM ATROPHY. (From Adams et al., Principles of Neurology, 6th ed, p1085)'),
  'D009850': NtO(MH='Oman', MS="A sultanate on the southeast coast of the Arabian peninsula. Its capital is Masqat. Before the 16th century it was ruled by independent emirs but was captured and controlled by the Portuguese 1508-1648. In 1741 it was recovered by a descendent of Yemen's imam. After its decline in the 19th century, it became virtually a political and economic dependency within the British Government of India, retaining close ties with Great Britain by treaty from 1939 to 1970 when it achieved autonomy. The name was recorded by Pliny in the 1st century A.D. as Omana, said to be derived from the founder of the state, Oman ben Ibrahim al-Khalil. (From Webster's New Geographical Dictionary, 1988, p890; Oman Embassy, Washington; Room, Brewer's Dictionary of Names, 1992, p391)"),
  'D009851': NtO(MH='Omasum', MS="The third stomach of ruminants, situated on the right side of the abdomen at a higher level than the fourth stomach and between this latter and the second stomach, with both of which it communicates. From its inner surface project large numbers of leaves or folia, each of which possesses roughened surfaces. In the center of each folium is a band of muscle fibers which produces a rasping movement of the leaf when it contracts. One leaf rubs against those on either side of it, and large particles of food material are ground down between the rough surfaces, preparatory to further digestion in the succeeding parts of the alimentary canal. (Black's Veterinary Dictionary, 17th ed)"),
  'D009852': NtO(MH='Omentum', MS='A double-layered fold of peritoneum that attaches the STOMACH to other organs in the ABDOMINAL CAVITY.'),
  'D009853': NtO(MH='Omeprazole', MS='A 4-methoxy-3,5-dimethylpyridyl, 5-methoxybenzimidazole derivative of timoprazole that is used in the therapy of STOMACH ULCERS and ZOLLINGER-ELLISON SYNDROME. The drug inhibits an H(+)-K(+)-EXCHANGING ATPASE which is found in GASTRIC PARIETAL CELLS.'),
  'D009854': NtO(MH='Onchocerca', MS='A genus of parasitic nematodes whose organisms live and breed in skin and subcutaneous tissues. Onchocercal microfilariae may also be found in the urine, blood, or sputum.'),
  'D009855': NtO(MH='Onchocerciasis', MS='Infection with nematodes of the genus ONCHOCERCA. Characteristics include the presence of firm subcutaneous nodules filled with adult worms, PRURITUS, and ocular lesions.'),
  'D009856': NtO(MH='Oncogene Proteins, Viral', MS='Products of viral oncogenes, most commonly retroviral oncogenes. They usually have transforming and often protein kinase activities.'),
  'D009857': NtO(MH='Oncogenes', MS='Genes whose gain-of-function alterations lead to NEOPLASTIC CELL TRANSFORMATION. They include, for example, genes for activators or stimulators of CELL PROLIFERATION such as growth factors, growth factor receptors, protein kinases, signal transducers, nuclear phosphoproteins, and transcription factors. A prefix of "v-" before oncogene symbols indicates oncogenes captured and transmitted by RETROVIRUSES; the prefix "c-" before the gene symbol of an oncogene indicates it is the cellular homolog (PROTO-ONCOGENES) of a v-oncogene.'),
  'D009858': NtO(MH='Oncogenic Viruses', MS='Viruses that produce tumors.'),
  'D009859': NtO(MH='Oncology Nursing', MS='A nursing specialty concerned with the care provided to cancer patients. It includes aspects of family functioning through education of both patient and family.'),
  'D009861': NtO(MH='Onium Compounds', MS='Ions with the suffix -onium, indicating cations with coordination number 4 of the type RxA+ which are analogous to QUATERNARY AMMONIUM COMPOUNDS (H4N+). Ions include phosphonium R4P+, oxonium R3O+, sulfonium R3S+, chloronium R2Cl+'),
  'D009862': NtO(MH='Online Systems', MS='Computer-based Information systems having real-time remote access to information or processes.'),
  'D009863': NtO(MH='Only Child', MS='Child who has no siblings.'),
  'D009864': NtO(MH='Ontario', MS="A province of Canada lying between the provinces of Manitoba and Quebec. Its capital is Toronto. It takes its name from Lake Ontario which is said to represent the Iroquois oniatariio, beautiful lake. (From Webster's New Geographical Dictionary, 1988, p892 & Room, Brewer's Dictionary of Names, 1992, p391)"),
  'D009865': NtO(MH='Oocytes', MS='Female germ cells derived from OOGONIA and termed OOCYTES when they enter MEIOSIS. The primary oocytes begin meiosis but are arrested at the diplotene state until OVULATION at PUBERTY to give rise to haploid secondary oocytes or ova (OVUM).'),
  'D009866': NtO(MH='Oogenesis', MS='The process of germ cell development in the female from the primordial germ cells through OOGONIA to the mature haploid ova (OVUM).'),
  'D009867': NtO(MH='Oogonia', MS='Euploid female germ cells of an early stage of OOGENESIS, derived from primordial germ cells during ovarian differentiation. Oogonia undergo MEIOSIS and give rise to haploid OOCYTES'),
  'D009868': NtO(MH='Oomycetes', MS='Eukaryotes in the group STRAMENOPILES, formerly considered FUNGI, whose exact taxonomic level is unsettled. Many consider Oomycetes (Oomycota) a phylum in the kingdom Stramenopila, or alternatively, as Pseudofungi in the phylum Heterokonta of the kingdom Chromista. They are morphologically similar to fungi but have no close phylogenetic relationship to them. Oomycetes are found in both fresh and salt water as well as in terrestrial environments. (Alexopoulos et al., Introductory Mycology, 4th ed, pp683-4). They produce flagellated, actively motile spores (zoospores) that are pathogenic to many crop plants and FISHES.'),
  'D009869': NtO(MH='Oophoritis', MS='Inflammation of the OVARY, generally caused by an ascending infection of organisms from the endocervix.'),
  'D009870': NtO(MH='Operating Room Information Systems', MS='Information systems, usually computer-assisted, designed to store, manipulate, and retrieve information for planning, organizing, directing, and controlling administrative activities associated with the provision and utilization of operating room services and facilities.'),
  'D009871': NtO(MH='Operating Room Nursing', MS='The functions of the professional nurse in the operating room.'),
  'D009872': NtO(MH='Operating Room Technicians', MS='Specially trained personnel to assist in routine technical procedures in the operating room.'),
  'D009873': NtO(MH='Operating Rooms', MS='Facilities equipped for performing surgery.'),
  'D009874': NtO(MH='Operations Research', MS='A group of techniques developed to apply scientific methods and tools to solve the problems of DECISION MAKING in complex organizations and systems. Operations research searches for optimal solutions in situations of conflicting GOALS and makes use of mathematical models from which solutions for actual problems may be derived. (From Psychiatric Dictionary, 6th ed)'),
  'D009875': NtO(MH='Operator Regions, Genetic', MS='The regulatory elements of an OPERON to which activators or repressors bind thereby effecting the transcription of GENES in the operon.'),
  'D009876': NtO(MH='Operon', MS='In bacteria, a group of metabolically related genes, with a common promoter, whose transcription into a single polycistronic MESSENGER RNA is under the control of an OPERATOR REGION.'),
  'D009877': NtO(MH='Endophthalmitis', MS='Suppurative inflammation of the tissues of the internal structures of the eye frequently associated with an infection.'),
  'D009878': NtO(MH='Ophthalmia Neonatorum', MS="Acute conjunctival inflammation in the newborn, usually caused by maternal gonococcal infection. The causative agent is NEISSERIA GONORRHOEAE. The baby's eyes are contaminated during passage through the birth canal."),
  'D009879': NtO(MH='Ophthalmia, Sympathetic', MS='Granulomatous uveitis which follows in one eye after a penetrating injury to the other eye; the secondarily affected eye is called the sympathizing eye, and the injured eye is called the exciting or activating eye.'),
  'D009880': NtO(MH='Ophthalmic Artery', MS='Artery originating from the internal carotid artery and distributing to the eye, orbit and adjacent facial structures.'),
  'D009881': NtO(MH='Ophthalmic Assistants', MS='Persons academically trained to care for patients with eye diseases or structural defects of the eye, under the supervision of an ophthalmologist.'),
  'D009882': NtO(MH='Ophthalmic Nerve', MS='A sensory branch of the trigeminal (5th cranial) nerve. The ophthalmic nerve carries general afferents from the superficial division of the face including the eyeball, conjunctiva, upper eyelid, upper nose, nasal mucosa, and scalp.'),
  'D009883': NtO(MH='Ophthalmic Solutions', MS='Sterile solutions that are intended for instillation into the eye. It does not include solutions for cleaning eyeglasses or CONTACT LENS SOLUTIONS.'),
  'D009884': NtO(MH='Ophthalmodynamometry', MS='Measurement of the blood pressure of the retinal vessels. It is used also for the determination of the near point of convergence (CONVERGENCE, OCULAR). (From Cline, et al., Dictionary of Visual Science, 4th ed)'),
  'D009885': NtO(MH='Ophthalmology', MS='A surgical specialty concerned with the structure and function of the eye and the medical and surgical treatment of its defects and diseases.'),
  'D009886': NtO(MH='Ophthalmoplegia', MS='Paralysis of one or more of the ocular muscles due to disorders of the eye muscles, neuromuscular junction, supporting soft tissue, tendons, or innervation to the muscles.'),
  'D009887': NtO(MH='Ophthalmoscopy', MS='Examination of the interior of the eye with an ophthalmoscope.'),
  'D009888': NtO(MH='Opipramol', MS='A tricyclic antidepressant with actions similar to AMITRIPTYLINE.'),
  'D009889': NtO(MH='Opisthorchiasis', MS='Infection with flukes of the genus Opisthorchis.'),
  'D009890': NtO(MH='Opisthorchidae', MS='A family of lanceolate liver flukes of the class Trematoda which occurs in animals and man. There are several genera including Amphimerus, Cyclorchis, Delphinicola, Metorchis, Parametorchis, Phocitrema, Clonorchis, and Opisthorchis.'),
  'D009891': NtO(MH='Opisthorchis', MS='A genus of trematode liver flukes of the family Opisthorchidae. It consists of the following species: O. felineus, O. noverca (Amphimerus noverca), and O. viverrini. The intermediate hosts are snails, fish, and AMPHIBIANS.'),
  'D009892': NtO(MH='Opium', MS='The air-dried exudate from the unripe seed capsule of the opium poppy, Papaver somniferum, or its variant, P. album. It contains a number of alkaloids, but only a few - MORPHINE; CODEINE; and PAPAVERINE - have clinical significance. Opium has been used as an analgesic, antitussive, antidiarrheal, and antispasmodic.'),
  'D009893': NtO(MH='Opossums', MS='New World marsupials of the family Didelphidae. Opossums are omnivorous, largely nocturnal and arboreal MAMMALS, grow to about three feet in length, including the scaly prehensile tail, and have an abdominal pouch in which the young are carried at birth.'),
  'D009894': NtO(MH='Opportunistic Infections', MS='An infection caused by an organism which becomes pathogenic under certain conditions, e.g., during immunosuppression.'),
  'D009895': NtO(MH='Opsonin Proteins', MS='Proteins that bind to particles and cells to increase susceptibility to PHAGOCYTOSIS, especially ANTIBODIES bound to EPITOPES that attach to FC RECEPTORS. COMPLEMENT C3B may also participate.'),
  'D009896': NtO(MH='Optic Atrophy', MS='Atrophy of the optic disk which may be congenital or acquired. This condition indicates a deficiency in the number of nerve fibers which arise in the RETINA and converge to form the OPTIC DISK; OPTIC NERVE; OPTIC CHIASM; and optic tracts. GLAUCOMA; ISCHEMIA; inflammation, a chronic elevation of intracranial pressure, toxins, optic nerve compression, and inherited conditions (see OPTIC ATROPHIES, HEREDITARY) are relatively common causes of this condition.'),
  'D009897': NtO(MH='Optic Chiasm', MS='The X-shaped structure formed by the meeting of the two optic nerves. At the optic chiasm the fibers from the medial part of each retina cross to project to the other side of the brain while the lateral retinal fibers continue on the same side. As a result each half of the brain receives information about the contralateral visual field from both eyes.'),
  'D009898': NtO(MH='Optic Disk', MS='The portion of the optic nerve seen in the fundus with the ophthalmoscope. It is formed by the meeting of all the retinal ganglion cell axons as they enter the optic nerve.'),
  'D009899': NtO(MH='Optic Lobe, Nonmammalian', MS='In invertebrate zoology, a lateral lobe of the FOREBRAIN in certain ARTHROPODS. In vertebrate zoology, either of the corpora bigemina of non-mammalian VERTEBRATES. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed, p1329)'),
  'D009900': NtO(MH='Optic Nerve', MS='The 2nd cranial nerve which conveys visual information from the RETINA to the brain. The nerve carries the axons of the RETINAL GANGLION CELLS which sort at the OPTIC CHIASM and continue via the OPTIC TRACTS to the brain. The largest projection is to the lateral geniculate nuclei; other targets include the SUPERIOR COLLICULI and the SUPRACHIASMATIC NUCLEI. Though known as the second cranial nerve, it is considered part of the CENTRAL NERVOUS SYSTEM.'),
  'D009901': NtO(MH='Optic Nerve Diseases', MS='Conditions which produce injury or dysfunction of the second cranial or optic nerve, which is generally considered a component of the central nervous system. Damage to optic nerve fibers may occur at or near their origin in the retina, at the optic disk, or in the nerve, optic chiasm, optic tract, or lateral geniculate nuclei. Clinical manifestations may include decreased visual acuity and contrast sensitivity, impaired color vision, and an afferent pupillary defect.'),
  'D009902': NtO(MH='Optic Neuritis', MS='Inflammation of the optic nerve. Commonly associated conditions include autoimmune disorders such as MULTIPLE SCLEROSIS, infections, and granulomatous diseases. Clinical features include retro-orbital pain that is aggravated by eye movement, loss of color vision, and contrast sensitivity that may progress to severe visual loss, an afferent pupillary defect (Marcus-Gunn pupil), and in some instances optic disc hyperemia and swelling. Inflammation may occur in the portion of the nerve within the globe (neuropapillitis or anterior optic neuritis) or the portion behind the globe (retrobulbar neuritis or posterior optic neuritis).'),
  'D009903': NtO(MH='Optical Illusions', MS='An illusion of vision usually affecting spatial relations.'),
  'D009904': NtO(MH='Optical Rotation', MS='The rotation of linearly polarized light as it passes through various media.'),
  'D009905': NtO(MH='Optical Rotatory Dispersion', MS='The method of measuring the dispersion of an optically active molecule to determine the relative magnitude of right- or left-handed components and sometimes structural features of the molecule.'),
  'D009907': NtO(MH='Optometry', MS='The professional practice of primary eye and vision care that includes the measurement of visual refractive power and the correction of visual defects with lenses or glasses.'),
  'D009909': NtO(MH='Oral Health', MS='The optimal state of the mouth and normal functioning of the organs of the mouth without evidence of disease.'),
  'D009910': NtO(MH='Oral Hygiene', MS='The practice of personal hygiene of the mouth. It includes the maintenance of oral cleanliness, tissue tone, and general preservation of oral health.'),
  'D009911': NtO(MH='Oral Hygiene Index', MS='A combination of the debris index and the dental calculus index to determine the status of oral hygiene.'),
  'D009912': NtO(MH='Oral Manifestations', MS='Disorders of the mouth attendant upon non-oral disease or injury.'),
  'D009913': NtO(MH='Oral Stage', MS='The earliest of the stages of infantile psychosexual development, lasting from birth to 12 months or longer.'),
  'D009914': NtO(MH='Oral Submucous Fibrosis', MS='Irreversible FIBROSIS of the submucosal tissue of the MOUTH.'),
  'D009915': NtO(MH='Orbit', MS='Bony cavity that holds the eyeball and its associated tissues and appendages.'),
  'D009916': NtO(MH='Orbital Diseases', MS='Diseases of the bony orbit and contents except the eyeball.'),
  'D009917': NtO(MH='Orbital Fractures', MS='Fractures of the bones in the orbit, which include parts of the frontal, ethmoidal, lacrimal, and sphenoid bones and the maxilla and zygoma.'),
  'D009918': NtO(MH='Orbital Neoplasms', MS='Neoplasms of the bony orbit and contents except the eyeball.'),
  'D009919': NtO(MH='Orchiectomy', MS='The surgical removal of one or both testicles.'),
  'D009920': NtO(MH='Orchitis', MS='Inflammation of a TESTIS. It has many features of EPIDIDYMITIS, such as swollen SCROTUM; PAIN; PYURIA; and FEVER. It is usually related to infections in the URINARY TRACT, which likely spread to the EPIDIDYMIS and then the TESTIS through either the VAS DEFERENS or the lymphatics of the SPERMATIC CORD.'),
  'D009921': NtO(MH='Metaproterenol', MS='A beta-2 adrenergic agonist used in the treatment of ASTHMA and BRONCHIAL SPASM.'),
  'D009922': NtO(MH='Oregon', MS='State bounded on the north by Washington, on the east by Idaho, on the south by California and Nevada, and on the west by the Pacific Ocean.'),
  'D009923': NtO(MH='Orf virus', MS='The type species of PARAPOXVIRUS which causes a skin infection in natural hosts, usually young sheep. Humans may contract local skin lesions by contact. The virus apparently persists in soil.'),
  'D009924': NtO(MH='Organ Culture Techniques', MS='A technique for maintenance or growth of animal organs in vitro. It refers to three-dimensional cultures of undisaggregated tissue retaining some or all of the histological features of the tissue in vivo. (Freshney, Culture of Animal Cells, 3d ed, p1)'),
  'D009925': NtO(MH='Organ of Corti', MS='The spiral EPITHELIUM containing sensory AUDITORY HAIR CELLS and supporting cells in the cochlea. Organ of Corti, situated on the BASILAR MEMBRANE and overlaid by a gelatinous TECTORIAL MEMBRANE, converts sound-induced mechanical waves to neural impulses to the brain.'),
  'D009926': NtO(MH='Organ Preservation', MS='The process by which organs are kept viable outside of the organism from which they were removed (i.e., kept from decay by means of a chemical agent, cooling, or a fluid substitute that mimics the natural state within the organism).'),
  'D009927': NtO(MH='Tissue and Organ Procurement', MS='The administrative procedures involved with acquiring TISSUES or organs for TRANSPLANTATION through various programs, systems, or organizations. These procedures include obtaining consent from TISSUE DONORS and arranging for transportation of donated tissues and organs, after TISSUE HARVESTING, to HOSPITALS for processing and transplantation.'),
  'D009928': NtO(MH='Organ Specificity', MS='Characteristic restricted to a particular organ of the body, such as a cell type, metabolic response or expression of a particular protein or antigen.'),
  'D009929': NtO(MH='Organ Size', MS='The measurement of an organ in volume, mass, or heaviness.'),
  'D009930': NtO(MH='Organic Chemicals', MS='A broad class of substances containing carbon and its derivatives. Many of these chemicals will frequently contain hydrogen with or without oxygen, nitrogen, sulfur, phosphorus, and other elements. They exist in either carbon chain or carbon ring form.'),
  'D009934': NtO(MH='Organization and Administration', MS='The planning and managing of programs, services, and resources.'),
  'D009935': NtO(MH='Organizational Affiliation', MS='Formal relationships established between otherwise independent organizations. These include affiliation agreements, interlocking boards, common controls, hospital medical school affiliations, etc.'),
  'D009936': NtO(MH='Organizational Innovation', MS='Introduction of changes which are new to the organization and are created by management.'),
  'D009937': NtO(MH='Organizational Objectives', MS="The purposes, missions, and goals of an individual organization or its units, established through administrative processes. It includes an organization's long-range plans and administrative philosophy."),
  'D009938': NtO(MH='Organizations', MS='Groups of people working together in a structured manner to pursue common goals and objectives.'),
  'D009939': NtO(MH='Organizations, Nonprofit', MS='Organizations which are not operated for a profit and may be supported by endowments or private contributions.'),
  'D009940': NtO(MH='Organoids', MS='An organization of cells into an organ-like structure. Organoids can be generated in culture. They are also found in certain neoplasms.'),
  'D009941': NtO(MH='Organomercury Compounds', MS='Organic compounds which contain mercury as an integral part of the molecule.'),
  'D009942': NtO(MH='Organometallic Compounds', MS="A class of compounds of the type R-M, where a C atom is joined directly to any other element except H, C, N, O, F, Cl, Br, I, or At. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D009943': NtO(MH='Organophosphorus Compounds', MS='Organic compounds that contain phosphorus as an integral part of the molecule. Included under this heading is broad array of synthetic compounds that are used as PESTICIDES and DRUGS.'),
  'D009944': NtO(MH='Organoplatinum Compounds', MS='Organic compounds which contain platinum as an integral part of the molecule.'),
  'D009945': NtO(MH='Organotherapy', MS='Historically, the treatment of disease by the administration of animal organs or their extracts (after Brown-Sequard). At present synthetic preparations substitute for the extracts of a gland. (From Stedman, 26th ed)'),
  'D009946': NtO(MH='Organothiophosphorus Compounds', MS='Compounds containing carbon-phosphorus bonds in which the phosphorus component is also bonded to one or more sulfur atoms. Many of these compounds function as CHOLINERGIC AGENTS and as INSECTICIDES.'),
  'D009947': NtO(MH='Organotin Compounds', MS='Organic compounds which contain tin in the molecule. Used widely in industry and agriculture.'),
  'D009948': NtO(MH='Orgasm', MS='The climax of sexual excitement in either humans or animals.'),
  'D009949': NtO(MH='Orientation', MS='Awareness of oneself in relation to time, place and person.'),
  'D009950': NtO(MH='Ornidazole', MS='A nitroimidazole antiprotozoal agent used in ameba and trichomonas infections. It is partially plasma-bound and also has radiation-sensitizing action.'),
  'D009951': NtO(MH='Ornipressin', MS='A synthetic analog of vasopressin with ORNITHINE substitution at residue 8 of the cyclic nonapeptide. It is used as a local vasoconstrictor and hemostatic.'),
  'D009952': NtO(MH='Ornithine', MS='An amino acid produced in the urea cycle by the splitting off of urea from arginine.'),
  'D009953': NtO(MH='Ornithine-Oxo-Acid Transaminase', MS='A pyridoxal phosphate enzyme that catalyzes the formation of glutamate gamma-semialdehyde and an L-amino acid from L-ornithine and a 2-keto-acid. EC 2.6.1.13.'),
  'D009954': NtO(MH='Ornithine Carbamoyltransferase', MS='A urea cycle enzyme that catalyzes the formation of orthophosphate and L-citrulline (CITRULLINE) from CARBAMOYL PHOSPHATE and L-ornithine (ORNITHINE). Deficiency of this enzyme may be transmitted as an X-linked trait. EC 2.1.3.3.'),
  'D009955': NtO(MH='Ornithine Decarboxylase', MS='A pyridoxal-phosphate protein, believed to be the rate-limiting compound in the biosynthesis of polyamines. It catalyzes the decarboxylation of ornithine to form putrescine, which is then linked to a propylamine moiety of decarboxylated S-adenosylmethionine to form spermidine.'),
  'D009956': NtO(MH='Psittacosis', MS='Infection with CHLAMYDOPHILA PSITTACI (formerly Chlamydia psittaci), transmitted to humans by inhalation of dust-borne contaminated nasal secretions or excreta of infected BIRDS. This infection results in a febrile illness characterized by PNEUMONITIS and systemic manifestations.'),
  'D009957': NtO(MH='Oroantral Fistula', MS='A fistula between the maxillary sinus and the oral cavity.'),
  'D009958': NtO(MH='Orofaciodigital Syndromes', MS='Two syndromes of oral, facial, and digital malformations. Type I (Papillon-Leage and Psaume syndrome, Gorlin-Psaume syndrome) is inherited as an X-linked dominant trait and is found only in females and XXY males. Type II (Mohr syndrome) is inherited as an autosomal recessive trait.'),
  'D009959': NtO(MH='Oropharyngeal Neoplasms', MS='Tumors or cancer of the OROPHARYNX.'),
  'D009960': NtO(MH='Oropharynx', MS='The middle portion of the pharynx that lies posterior to the mouth, inferior to the SOFT PALATE, and superior to the base of the tongue and EPIGLOTTIS. It has a digestive function as food passes from the mouth into the oropharynx before entering ESOPHAGUS.'),
  'D009961': NtO(MH='Orosomucoid', MS=None),
  'D009962': NtO(MH='Orotate Phosphoribosyltransferase', MS="The enzyme catalyzing the formation of orotidine-5'-phosphoric acid (orotidylic acid) from orotic acid and 5-phosphoribosyl-1-pyrophosphate in the course of pyrimidine nucleotide biosynthesis. EC 2.4.2.10."),
  'D009963': NtO(MH='Orotic Acid', MS='An intermediate product in PYRIMIDINE synthesis which plays a role in chemical conversions between DIHYDROFOLATE and TETRAHYDROFOLATE.'),
  'D009964': NtO(MH="Orotidine-5'-Phosphate Decarboxylase", MS="Orotidine-5'-phosphate carboxy-lyase. Catalyzes the decarboxylation of orotidylic acid to yield uridylic acid in the final step of the pyrimidine nucleotide biosynthesis pathway. EC 4.1.1.23."),
  'D009965': NtO(MH='Orphan Drug Production', MS='Production of drugs or biologicals which are unlikely to be manufactured by private industry unless special incentives are provided by others.'),
  'D009966': NtO(MH='Orphenadrine', MS='A muscarinic antagonist used to treat drug-induced parkinsonism and to relieve pain from muscle spasm.'),
  'D009967': NtO(MH='Orthodontic Appliances', MS="Devices used for influencing tooth position. Orthodontic appliances may be classified as fixed or removable, active or retaining, and intraoral or extraoral. (Boucher's Clinical Dental Terminology, 4th ed, p19)"),
  'D009968': NtO(MH='Orthodontic Appliances, Removable', MS='Dental devices such as RETAINERS, ORTHODONTIC used to improve gaps in teeth and structure of the jaws. These devices can be removed and reinserted at will.'),
  'D009969': NtO(MH='Orthodontic Wires', MS='Wires of various dimensions and grades made of stainless steel or precious metal. They are used in orthodontic treatment.'),
  'D009970': NtO(MH='Orthodontics', MS='A dental specialty concerned with the prevention and correction of dental and oral anomalies (malocclusion).'),
  'D009971': NtO(MH='Orthodontics, Corrective', MS="The phase of orthodontics concerned with the correction of malocclusion with proper appliances and prevention of its sequelae (Jablonski's Illus. Dictionary of Dentistry)."),
  'D009972': NtO(MH='Orthodontics, Interceptive', MS='Recognition and elimination of potential irregularities and malpositions in the developing dentofacial complex.'),
  'D009973': NtO(MH='Orthodontics, Preventive', MS="Treatment methods intended to prevent the development of crooked teeth that involve the use of space maintainers for BABY TEETH lost early or removal of PRIMARY TEETH that don't come out on their own. Commonly used in pediatric dentistry."),
  'D009974': NtO(MH='Orthomolecular Therapy', MS='The use of very large doses of vitamins or other naturally occurring substances normally present in the body, frequently for the treatment of mental disorders.'),
  'D009975': NtO(MH='Orthomyxoviridae', MS='A family of RNA viruses causing INFLUENZA and other diseases. There are five recognized genera: INFLUENZAVIRUS A; INFLUENZAVIRUS B; INFLUENZAVIRUS C; ISAVIRUS; and THOGOTOVIRUS.'),
  'D009976': NtO(MH='Orthomyxoviridae Infections', MS='Virus diseases caused by the ORTHOMYXOVIRIDAE.'),
  'D009980': NtO(MH='Influenza A virus', MS='The type species of the genus INFLUENZAVIRUS A that causes influenza and other diseases in humans and animals. Antigenic variation occurs frequently between strains, allowing classification into subtypes and variants. Transmission is usually by aerosol (human and most non-aquatic hosts) or waterborne (ducks). Infected birds shed the virus in their saliva, nasal secretions, and feces.'),
  'D009981': NtO(MH='Influenza B virus', MS='Species of the genus INFLUENZAVIRUS B that cause HUMAN INFLUENZA and other diseases primarily in humans. Antigenic variation is less extensive than in type A viruses (INFLUENZA A VIRUS) and consequently there is no basis for distinct subtypes or variants. Epidemics are less likely than with INFLUENZA A VIRUS and there have been no pandemics. Previously only found in humans, Influenza B virus has been isolated from seals which may constitute the animal reservoir from which humans are exposed.'),
  'D009982': NtO(MH='Influenzavirus C', MS='A genus of the family ORTHOMYXOVIRIDAE comprising viruses similar to types A and B but less common, more stable, more homogeneous, and lacking the neuraminidase protein. They have not been associated with epidemics but may cause mild influenza. Influenza C virus is the type species.'),
  'D009983': NtO(MH='Orthopedic Equipment', MS='Nonexpendable items used in the performance of orthopedic surgery and related therapy. They are differentiated from ORTHOTIC DEVICES, apparatus used to prevent or correct deformities in patients.'),
  'D009984': NtO(MH='Orthopedic Fixation Devices', MS='Devices which are used in the treatment of orthopedic injuries and diseases.'),
  'D009985': NtO(MH='Orthopedics', MS='A specialty which utilizes medical, surgical, and physical methods to treat and correct deformities, diseases, and injuries to the skeletal system, its articulations, and associated structures.'),
  'D009986': NtO(MH='Orthopsychiatry', MS='The study and treatment of human behavior problems utilizing principles derived from the behavioral, medical, and social science with emphasis on promotion of mental health.'),
  'D009987': NtO(MH='Orthoptera', MS='An order of insects comprising two suborders: Caelifera and Ensifera. They consist of GRASSHOPPERS, locusts, and crickets (GRYLLIDAE).'),
  'D009988': NtO(MH='Orthoptics', MS='The study and treatment of defects in binocular vision resulting from defects in the optic musculature or of faulty visual habits. It involves a technique of eye exercises designed to correct the visual axes of eyes not properly coordinated for binocular vision.'),
  'D009989': NtO(MH='Orthotic Devices', MS='Apparatus used to support, align, or augment the functioning of parts of the body.'),
  'D009990': NtO(MH='Oryzias', MS='The only genus in the family Oryziinae, order BELONIFORMES. Oryzias are egg-layers; other fish of the same order are livebearers. Oryzias are used extensively in testing carcinogens.'),
  'D009991': NtO(MH='Oscillometry', MS='The measurement of frequency or oscillation changes.'),
  'D009992': NtO(MH='Osmium', MS='A very hard, gray, toxic, and nearly infusible metal element, atomic number 76, atomic weight 190.2, symbol Os.'),
  'D009993': NtO(MH='Osmium Tetroxide', MS='(T-4)-Osmium oxide (OsO4). A highly toxic and volatile oxide of osmium used in industry as an oxidizing agent. It is also used as a histological fixative and stain and as a synovectomy agent in arthritic joints. Its vapor can cause eye, skin, and lung damage.'),
  'D009994': NtO(MH='Osmolar Concentration', MS='The concentration of osmotically active particles in solution expressed in terms of osmoles of solute per liter of solution. Osmolality is expressed in terms of osmoles of solute per kilogram of solvent.'),
  'D009995': NtO(MH='Osmosis', MS='Tendency of fluids (e.g., water) to move from the less concentrated to the more concentrated side of a semipermeable membrane.'),
  'D009996': NtO(MH='Osmotic Fragility', MS='RED BLOOD CELL sensitivity to change in OSMOTIC PRESSURE. When exposed to a hypotonic concentration of sodium in a solution, red cells take in more water, swell until the capacity of the cell membrane is exceeded, and burst.'),
  'D009997': NtO(MH='Osmotic Pressure', MS='The pressure required to prevent the passage of solvent through a semipermeable membrane that separates a pure solvent from a solution of the solvent and solute or that separates different concentrations of a solution. It is proportional to the osmolality of the solution.'),
  'D009998': NtO(MH='Ossicular Prosthesis', MS='An implant used to replace one or more of the ear ossicles. They are usually made of plastic, Gelfoam, ceramic, or stainless steel.'),
  'D009999': NtO(MH='Ossification, Heterotopic', MS='The development of bony substance in normally soft structures.'),
  'D010000': NtO(MH='Osteitis', MS='Inflammation of the bone.'),
  'D010001': NtO(MH='Osteitis Deformans', MS='A disease marked by repeated episodes of increased bone resorption followed by excessive attempts at repair, resulting in weakened, deformed bones of increased mass. The resultant architecture of the bone assumes a mosaic pattern in which the fibers take on a haphazard pattern instead of the normal parallel symmetry.'),
  'D010002': NtO(MH='Osteitis Fibrosa Cystica', MS='A fibrous degeneration, cyst formation, and the presence of fibrous nodules in bone, usually due to HYPERPARATHYROIDISM.'),
  'D010003': NtO(MH='Osteoarthritis', MS='A progressive, degenerative joint disease, the most common form of arthritis, especially in older persons. The disease is thought to result not from the aging process but from biochemical changes and biomechanical stresses affecting articular cartilage. In the foreign literature it is often called osteoarthrosis deformans.'),
  'D010004': NtO(MH='Osteoarthropathy, Primary Hypertrophic', MS='A condition chiefly characterized by thickening of the skin of the head and distal extremities, deep folds and furrows of the skin of the forehead, cheeks, and scalp, SEBORRHEA; HYPERHIDROSIS; periostosis of the long bones, digital clubbing, and spadelike enlargement of the hands and feet. It is more prevalent in the male, and is usually first evident during adolescence. Inheritance is primarily autosomal recessive, but an autosomal dominant form exists.'),
  'D010005': NtO(MH='Osteoarthropathy, Secondary Hypertrophic', MS='Symmetrical osteitis of the four limbs, chiefly localized to the phalanges and the terminal epiphyses of the long bones of the forearm and leg, sometimes extending to the proximal ends of the limbs and the flat bones, and accompanied by dorsal kyphosis and joint involvement. It is often secondary to chronic conditions of the lungs and heart. (Dorland, 27th ed)'),
  'D010006': NtO(MH='Osteoblasts', MS='Bone-forming cells which secrete an EXTRACELLULAR MATRIX. HYDROXYAPATITE crystals are then deposited into the matrix to form bone.'),
  'D010007': NtO(MH='Osteochondritis', MS='Inflammation of a bone and its overlaying CARTILAGE.'),
  'D010008': NtO(MH='Osteochondritis Dissecans', MS='A type of osteochondritis in which articular cartilage and associated bone becomes partially or totally detached to form joint loose bodies. Affects mainly the knee, ankle, and elbow joints.'),
  'D010009': NtO(MH='Osteochondrodysplasias', MS='Abnormal development of cartilage and bone.'),
  'D010010': NtO(MH='Osteoclasts', MS='A large multinuclear cell associated with the BONE RESORPTION. An odontoclast, also called cementoclast, is cytomorphologically the same as an osteoclast and is involved in CEMENTUM resorption.'),
  'D010011': NtO(MH='Osteocytes', MS='Mature osteoblasts that have become embedded in the BONE MATRIX. They occupy a small cavity, called lacuna, in the matrix and are connected to adjacent osteocytes via protoplasmic projections called canaliculi.'),
  'D010012': NtO(MH='Osteogenesis', MS='The process of bone formation. Histogenesis of bone including ossification.'),
  'D010013': NtO(MH='Osteogenesis Imperfecta', MS='COLLAGEN DISEASES characterized by brittle, osteoporotic, and easily fractured bones. It may also present with blue sclerae, loose joints, and imperfect dentin formation. Most types are autosomal dominant and are associated with mutations in COLLAGEN TYPE I.'),
  'D010014': NtO(MH='Osteolysis', MS='Dissolution of bone that particularly involves the removal or loss of calcium.'),
  'D010015': NtO(MH='Osteolysis, Essential', MS='Syndromes of bone destruction where the cause is not obvious such as neoplasia, infection, or trauma. The destruction follows various patterns: massive (Gorham disease), multicentric (HAJDU-CHENEY SYNDROME), or carpal/tarsal.'),
  'D010016': NtO(MH='Osteoma', MS='A benign tumor composed of bone tissue or a hard tumor of bonelike structure developing on a bone (homoplastic osteoma) or on other structures (heteroplastic osteoma). (From Dorland, 27th ed)'),
  'D010017': NtO(MH='Osteoma, Osteoid', MS='A benign osteoblastic tumor with central vascularized nidus surrounded by normal reactive bone. It occurs especially in second decade of life most commonly in the femoral neck but can occur in any bone and any site within a bone.'),
  'D010018': NtO(MH='Osteomalacia', MS='Disorder caused by an interruption of the mineralization of organic bone matrix leading to bone softening, bone pain, and weakness. It is the adult form of rickets resulting from disruption of VITAMIN D; PHOSPHORUS; or CALCIUM homeostasis.'),
  'D010019': NtO(MH='Osteomyelitis', MS='INFLAMMATION of the bone as a result of infection. It may be caused by a variety of infectious agents, especially pyogenic (PUS - producing) BACTERIA.'),
  'D010020': NtO(MH='Osteonecrosis', MS='Death of a bone or part of a bone, either atraumatic or posttraumatic.'),
  'D010021': NtO(MH='Osteopathic Medicine', MS='A medical discipline that is based on the philosophy that all body systems are interrelated and dependent upon one another for good health. This philosophy, developed in 1874 by Dr. Andrew Taylor Still, recognizes the concept of "wellness" and the importance of treating illness within the context of the whole body. Special attention is placed on the MUSCULOSKELETAL SYSTEM.'),
  'D010022': NtO(MH='Osteopetrosis', MS='Excessive formation of dense trabecular bone leading to pathological fractures; OSTEITIS; SPLENOMEGALY with infarct; ANEMIA; and extramedullary hemopoiesis (HEMATOPOIESIS, EXTRAMEDULLARY).'),
  'D010023': NtO(MH='Osteopoikilosis', MS='An asymptomatic, autosomal dominant trait in which pea-sized sclerotic spots, prominent in the metaphyseal area, are accompanied by unique cutaneous lesions. These are yellowish papules or plaques with increased elastin content. (From Cecil Textbook of Medicine, 19th ed, pp1434-35)'),
  'D010024': NtO(MH='Osteoporosis', MS='Reduction of bone mass without alteration in the composition of bone, leading to fractures. Primary osteoporosis can be of two major types: postmenopausal osteoporosis (OSTEOPOROSIS, POSTMENOPAUSAL) and age-related or senile osteoporosis.'),
  'D010025': NtO(MH='Osteoradionecrosis', MS='Necrosis of bone following radiation injury.'),
  'D010026': NtO(MH='Osteosclerosis', MS='An abnormal hardening or increased density of bone tissue.'),
  'D010027': NtO(MH='Osteotomy', MS='The surgical cutting of a bone. (Dorland, 28th ed)'),
  'D010028': NtO(MH='Ostertagia', MS='A genus of parasitic nematodes occurring in the stomach of ruminants.'),
  'D010029': NtO(MH='Ostertagiasis', MS='A disease of herbivorous mammals, particularly cattle and sheep, caused by stomach worms of the genus OSTERTAGIA.'),
  'D010030': NtO(MH='Ostomy', MS='Surgical construction of an artificial opening (stoma) for external fistulization of a duct or vessel by insertion of a tube with or without a supportive stent.'),
  'D010031': NtO(MH='Otitis', MS='Inflammation of the ear, which may be marked by pain (EARACHE), fever, HEARING DISORDERS, and VERTIGO. Inflammation of the external ear is OTITIS EXTERNA; of the middle ear, OTITIS MEDIA; of the inner ear, LABYRINTHITIS.'),
  'D010032': NtO(MH='Otitis Externa', MS='Inflammation of the OUTER EAR including the external EAR CANAL, cartilages of the auricle (EAR CARTILAGE), and the TYMPANIC MEMBRANE.'),
  'D010033': NtO(MH='Otitis Media', MS='Inflammation of the MIDDLE EAR including the AUDITORY OSSICLES and the EUSTACHIAN TUBE.'),
  'D010034': NtO(MH='Otitis Media with Effusion', MS='Inflammation of the middle ear with a clear pale yellow-colored transudate.'),
  'D010035': NtO(MH='Otitis Media, Suppurative', MS='Inflammation of the middle ear with purulent discharge.'),
  'D010036': NtO(MH='Otolaryngology', MS='A surgical specialty concerned with the study and treatment of disorders of the ear, nose, and throat.'),
  'D010037': NtO(MH='Otolithic Membrane', MS='A gelatinous membrane overlying the acoustic maculae of SACCULE AND UTRICLE. It contains minute crystalline particles (otoliths) of CALCIUM CARBONATE and protein on its outer surface. In response to head movement, the otoliths shift causing distortion of the vestibular hair cells which transduce nerve signals to the BRAIN for interpretation of equilibrium.'),
  'D010038': NtO(MH='Otorhinolaryngologic Diseases', MS='Pathological processes of the ear, the nose, and the throat, also known as the ENT diseases.'),
  'D010039': NtO(MH='Otorhinolaryngologic Neoplasms', MS='A general concept for tumors or cancer of any part of the EAR; the NOSE; the THROAT; and the PHARYNX. It is used when there is no specific heading.'),
  'D010040': NtO(MH='Otosclerosis', MS='Formation of spongy bone in the labyrinth capsule which can progress toward the STAPES (stapedial fixation) or anteriorly toward the COCHLEA leading to conductive, sensorineural, or mixed HEARING LOSS. Several genes are associated with familial otosclerosis with varied clinical signs.'),
  'D010041': NtO(MH='Otters', MS='Fish-eating carnivores of the family MUSTELIDAE, found on both hemispheres.'),
  'D010042': NtO(MH='Ouabain', MS='A cardioactive glycoside consisting of rhamnose and ouabagenin, obtained from the seeds of Strophanthus gratus and other plants of the Apocynaceae; used like DIGITALIS. It is commonly used in cell biological studies as an inhibitor of the NA(+)-K(+)-EXCHANGING ATPASE.'),
  'D010043': NtO(MH='Outcome and Process Assessment, Health Care', MS='Evaluation procedures that focus on both the outcome or status (OUTCOMES ASSESSMENT) of the patient at the end of an episode of care - presence of symptoms, level of activity, and mortality; and the process (ASSESSMENT, PROCESS) - what is done for the patient diagnostically and therapeutically.'),
  'D010044': NtO(MH='Outpatient Clinics, Hospital', MS='Organized services in a hospital which provide medical care on an outpatient basis.'),
  'D010045': NtO(MH='Outpatients', MS='Persons who receive ambulatory care at an outpatient department or clinic without room and board being provided.'),
  'D010046': NtO(MH='Oval Window, Ear', MS='Fenestra or oval opening on the lateral wall of the vestibular labyrinth adjacent to the MIDDLE EAR. It is located above the cochlear round window and normally covered by the base of the STAPES.'),
  'D010047': NtO(MH='Ovalbumin', MS='An albumin obtained from the white of eggs. It is a member of the serpin superfamily.'),
  'D010048': NtO(MH='Ovarian Cysts', MS='General term for CYSTS and cystic diseases of the OVARY.'),
  'D010049': NtO(MH='Ovarian Diseases', MS='Pathological processes of the OVARY.'),
  'D010050': NtO(MH='Ovarian Function Tests', MS='Methods used for assessment of ovarian function.'),
  'D010051': NtO(MH='Ovarian Neoplasms', MS='Tumors or cancer of the OVARY. These neoplasms can be benign or malignant. They are classified according to the tissue of origin, such as the surface EPITHELIUM, the stromal endocrine cells, and the totipotent GERM CELLS.'),
  'D010052': NtO(MH='Ovariectomy', MS='The surgical removal of one or both ovaries.'),
  'D010053': NtO(MH='Ovary', MS='The reproductive organ (GONADS) in female animals. In vertebrates, the ovary contains two functional parts: the OVARIAN FOLLICLE for the production of female germ cells (OOGENESIS); and the endocrine cells (GRANULOSA CELLS; THECA CELLS; and LUTEAL CELLS) for the production of ESTROGENS and PROGESTERONE.'),
  'D010054': NtO(MH='Brief Psychiatric Rating Scale', MS='A scale comprising 18 symptom constructs chosen to represent relatively independent dimensions of manifest psychopathology. The initial intended use was to provide more efficient assessment of treatment response in clinical psychopharmacology research; however, the scale was readily adapted to other uses. (From Hersen, M. and Bellack, A.S., Dictionary of Behavioral Assessment Techniques, p. 87)'),
  'D010056': NtO(MH='Overlearning', MS='Learning in which practice proceeds beyond the point where the act can just be performed with the required degree of excellence.'),
  'D010057': NtO(MH='Oviducts', MS='Ducts that serve exclusively for the passage of eggs from the ovaries to the exterior of the body. In non-mammals, they are termed oviducts. In mammals, they are highly specialized and known as FALLOPIAN TUBES.'),
  'D010058': NtO(MH='Oviposition', MS='The process of laying or shedding fully developed eggs (OVA) from the female body. The term is usually used for certain INSECTS or FISHES with an organ called ovipositor where eggs are stored or deposited before expulsion from the body.'),
  'D010059': NtO(MH='Ovomucin', MS='A heterogeneous mixture of glycoproteins responsible for the gel structure of egg white. It has trypsin-inhibiting activity.'),
  'D010060': NtO(MH='Ovulation', MS='The discharge of an OVUM from a rupturing follicle in the OVARY.'),
  'D010061': NtO(MH='Ovulation Detection', MS='Method to determine the occurrence of OVULATION by direct or indirect means. Indirect methods examine the effects of PROGESTERONE on cervical mucus (CERVIX MUCUS), or basal body temperature. Direct ovulation detection, generally used in fertility treatment, involves analyses of circulating hormones in blood and ULTRASONOGRAPHY.'),
  'D010062': NtO(MH='Ovulation Induction', MS='Techniques for the artifical induction of ovulation, the rupture of the follicle and release of the ovum.'),
  'D010063': NtO(MH='Ovum', MS='A mature haploid female germ cell extruded from the OVARY at OVULATION.'),
  'D010064': NtO(MH='Embryo Implantation', MS='Endometrial implantation of EMBRYO, MAMMALIAN at the BLASTOCYST stage.'),
  'D010065': NtO(MH='Embryo Implantation, Delayed', MS='Delay in the attachment and implantation of BLASTOCYST to the uterine ENDOMETRIUM. The blastocyst remains unattached beyond the normal duration thus delaying embryonic development.'),
  'D010066': NtO(MH='Ovum Transport', MS='Transport of the OVUM or fertilized ovum (ZYGOTE) from the mammalian oviduct (FALLOPIAN TUBES) to the site of EMBRYO IMPLANTATION in the UTERUS.'),
  'D010067': NtO(MH='Ownership', MS='The legal relation between an entity (individual, group, corporation, or-profit, secular, government) and an object. The object may be corporeal, such as equipment, or completely a creature of law, such as a patent; it may be movable, such as an animal, or immovable, such as a building.'),
  'D010068': NtO(MH='Oxacillin', MS='An antibiotic similar to FLUCLOXACILLIN used in resistant staphylococci infections.'),
  'D010069': NtO(MH='Oxadiazoles', MS=None),
  'D010070': NtO(MH='Oxalates', MS='Derivatives of OXALIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that are derived from the ethanedioic acid structure.'),
  'D010071': NtO(MH='Oxaloacetates', MS='Derivatives of OXALOACETIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that include a 2-keto-1,4-carboxy aliphatic structure.'),
  'D010072': NtO(MH='Oxamic Acid', MS='Amino-substituted glyoxylic acid derivative.'),
  'D010073': NtO(MH='Oxamniquine', MS='An anthelmintic with schistosomicidal activity against Schistosoma mansoni, but not against other Schistosoma spp. Oxamniquine causes worms to shift from the mesenteric veins to the liver where the male worms are retained; the female worms return to the mesentery, but can no longer release eggs. (From Martindale, The Extra Pharmacopoeia, 31st ed, p121)'),
  'D010074': NtO(MH='Oxandrolone', MS='A synthetic hormone with anabolic and androgenic properties.'),
  'D010075': NtO(MH='Oxathiins', MS=None),
  'D010076': NtO(MH='Oxazepam', MS='A benzodiazepine used in the treatment of anxiety, alcohol withdrawal, and insomnia.'),
  'D010077': NtO(MH='Oxazepines', MS=None),
  'D010078': NtO(MH='Oxazines', MS='Six-membered heterocycles containing an oxygen and a nitrogen.'),
  'D010079': NtO(MH='Oxazocines', MS='One ring heterocyclic compounds defined by C6H7NO. Permitted are any degree of hydrogenation, any substituents and any ortho-fused or ortho-peri-fused ring systems.'),
  'D010080': NtO(MH='Oxazoles', MS='Five-membered heterocyclic ring structures containing an oxygen in the 1-position and a nitrogen in the 3-position, in distinction from ISOXAZOLES where they are at the 1,2 positions.'),
  'D010081': NtO(MH='Oxazolone', MS='Immunologic adjuvant and sensitizing agent.'),
  'D010082': NtO(MH='Oxepins', MS='Compounds based on a 7-membered heterocyclic ring including an oxygen. They can be considered a medium ring ether. A natural source is the MONTANOA plant genus. Some dibenzo-dioxepins, called depsidones, are found in GARCINIA plants.'),
  'D010083': NtO(MH='Oxidants, Photochemical', MS='Compounds that accept electrons in an oxidation-reduction reaction. The reaction is induced by or accelerated by exposure to electromagnetic radiation in the spectrum of visible or ultraviolet light.'),
  'D010084': NtO(MH='Oxidation-Reduction', MS='A chemical reaction in which an electron is transferred from one molecule to another. The electron-donating molecule is the reducing agent or reductant; the electron-accepting molecule is the oxidizing agent or oxidant. Reducing and oxidizing agents function as conjugate reductant-oxidant pairs or redox pairs (Lehninger, Principles of Biochemistry, 1982, p471).'),
  'D010085': NtO(MH='Oxidative Phosphorylation', MS='Electron transfer through the cytochrome system liberating free energy which is transformed into high-energy phosphate bonds.'),
  'D010086': NtO(MH='Oxidative Phosphorylation Coupling Factors', MS=None),
  'D010087': NtO(MH='Oxides', MS='Binary compounds of oxygen containing the anion O(2-). The anion combines with metals to form alkaline oxides and non-metals to form acidic oxides.'),
  'D010088': NtO(MH='Oxidoreductases', MS='The class of all enzymes catalyzing oxidoreduction reactions. The substrate that is oxidized is regarded as a hydrogen donor. The systematic name is based on donor:acceptor oxidoreductase. The recommended name will be dehydrogenase, wherever this is possible; as an alternative, reductase can be used. Oxidase is only used in cases where O2 is the acceptor. (Enzyme Nomenclature, 1992, p9)'),
  'D010089': NtO(MH='Oxidoreductases, N-Demethylating', MS=None),
  'D010090': NtO(MH='Oxidoreductases, O-Demethylating', MS='Drug metabolizing enzymes which oxidize methyl ethers. Usually found in liver microsomes.'),
  'D010091': NtO(MH='Oximes', MS='Compounds that contain the radical R2C=N.OH derived from condensation of ALDEHYDES or KETONES with HYDROXYLAMINE. Members of this group are CHOLINESTERASE REACTIVATORS.'),
  'D010092': NtO(MH='Oximetry', MS='The determination of oxygen-hemoglobin saturation of blood either by withdrawing a sample and passing it through a classical photoelectric oximeter or by electrodes attached to some translucent part of the body like finger, earlobe, or skin fold. It includes non-invasive oxygen monitoring by pulse oximetry.'),
  'D010093': NtO(MH='Oxolinic Acid', MS='Synthetic antimicrobial related to NALIDIXIC ACID and used in URINARY TRACT INFECTIONS.'),
  'D010094': NtO(MH='Oxonic Acid', MS='Antagonist of urate oxidase.'),
  'D010095': NtO(MH='Oxotremorine', MS='A non-hydrolyzed muscarinic agonist used as a research tool.'),
  'D010096': NtO(MH='Oxprenolol', MS='A beta-adrenergic antagonist used in the treatment of hypertension, angina pectoris, arrhythmias, and anxiety.'),
  'D010097': NtO(MH='Oxyclozanide', MS='Anthelmintic used in grazing animals for fasciola and cestode infestations.'),
  'D010098': NtO(MH='Oxycodone', MS='A semisynthetic derivative of CODEINE.'),
  'D010099': NtO(MH='Oxyfedrine', MS='A drug used in the treatment of angina pectoris, heart failure, conduction defects, and myocardial infarction. It is a partial agonist at beta adrenergic receptors and acts as a coronary vasodilator and cardiotonic agent.'),
  'D010100': NtO(MH='Oxygen', MS='An element with atomic symbol O, atomic number 8, and atomic weight [15.99903; 15.99977]. It is the most abundant element on earth and essential for respiration.'),
  'D010101': NtO(MH='Oxygen Consumption', MS='The rate at which oxygen is used by a tissue; microliters of oxygen STPD used per milligram of tissue per hour; the rate at which oxygen enters the blood from alveolar gas, equal in the steady state to the consumption of oxygen by tissue metabolism throughout the body. (Stedman, 25th ed, p346)'),
  'D010102': NtO(MH='Oxygen Inhalation Therapy', MS='Inhalation of oxygen aimed at restoring toward normal any pathophysiologic alterations of gas exchange in the cardiopulmonary system, as by the use of a respirator, nasal catheter, tent, chamber, or mask. (From Dorland, 27th ed & Stedman, 25th ed)'),
  'D010103': NtO(MH='Oxygen Isotopes', MS='Stable oxygen atoms that have the same atomic number as the element oxygen, but differ in atomic weight. O-17 and 18 are stable oxygen isotopes.'),
  'D010104': NtO(MH='Oxygen Radioisotopes', MS='Unstable isotopes of oxygen that decay or disintegrate emitting radiation. O atoms with atomic weights 13, 14, 15, 19, and 20 are radioactive oxygen isotopes.'),
  'D010105': NtO(MH='Oxygenases', MS='Oxidases that specifically introduce DIOXYGEN-derived oxygen atoms into a variety of organic molecules.'),
  'D010106': NtO(MH='Oxygenators', MS='Devices which mechanically oxygenate venous blood extracorporeally. They are used in combination with one or more pumps for maintaining circulation during open heart surgery and for assisting the circulation in patients seriously ill with some cardiac and pulmonary disorders. (UMDNS, 1999)'),
  'D010107': NtO(MH='Oxygenators, Membrane', MS='Devices in which blood and oxygen are separated by a semipermeable membrane, generally of Teflon or polypropylene, across which gas exchange occurs. The membrane may be arranged as a series of parallel plates or as a number of hollow fibers; in the latter arrangement, the blood may flow inside the fibers, which are surrounded by gas, or the blood may flow outside the fibers and the gas inside the fibers. (Dorland, 28th ed)'),
  'D010108': NtO(MH='Oxyhemoglobins', MS='A compound formed by the combination of hemoglobin and oxygen. It is a complex in which the oxygen is bound directly to the iron without causing a change from the ferrous to the ferric state.'),
  'D010109': NtO(MH='Oxymetazoline', MS='A direct acting sympathomimetic used as a vasoconstrictor to relieve nasal congestion. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1251)'),
  'D010110': NtO(MH='Oxymetholone', MS='A synthetic hormone with anabolic and androgenic properties. It is used mainly in the treatment of anemias. According to the Fourth Annual Report on Carcinogens (NTP 85-002), this compound may reasonably be anticipated to be a carcinogen. (From Merck Index, 11th ed)'),
  'D010111': NtO(MH='Oxymorphone', MS='An opioid analgesic with actions and uses similar to those of MORPHINE, apart from an absence of cough suppressant activity. It is used in the treatment of moderate to severe pain, including pain in obstetrics. It may also be used as an adjunct to anesthesia. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1092)'),
  'D010113': NtO(MH='Oxyphenbutazone', MS='A non-steroidal anti-inflammatory drug. Oxyphenbutazone eyedrops have been used abroad in the management of postoperative ocular inflammation, superficial eye injuries, and episcleritis. (From AMA, Drug Evaluations Annual, 1994, p2000) It had been used by mouth in rheumatic disorders such as ankylosing spondylitis, osteoarthritis, and rheumatoid arthritis but such use is no longer considered justified owing to the risk of severe hematological adverse effects. (From Martindale, The Extra Pharmacopoeia, 30th ed, p27)'),
  'D010114': NtO(MH='Oxyphenisatin Acetate', MS='A laxative that undergoes enterohepatic circulation. It may cause jaundice.'),
  'D010115': NtO(MH='Oxyphenonium', MS='A quaternary ammonium anticholinergic agent with peripheral side effects similar to those of ATROPINE. It is used as an adjunct in the treatment of gastric and duodenal ulcer, and to relieve visceral spasms. The drug has also been used in the form of eye drops for mydriatic effect.'),
  'D010117': NtO(MH='Oxypurinol', MS='A xanthine oxidase inhibitor.'),
  'D010118': NtO(MH='Oxytetracycline', MS='A TETRACYCLINE analog isolated from the actinomycete STREPTOMYCES RIMOSUS and used in a wide variety of clinical conditions.'),
  'D010119': NtO(MH='Oxythiamine', MS='Thiamine antagonist, antimetabolite.'),
  'D010120': NtO(MH='Oxytocics', MS='Drugs that stimulate contraction of the myometrium. They are used to induce LABOR, OBSTETRIC at term, to prevent or control postpartum or postabortion hemorrhage, and to assess fetal status in high risk pregnancies. They may also be used alone or with other drugs to induce abortions (ABORTIFACIENTS). Oxytocics used clinically include the neurohypophyseal hormone OXYTOCIN and certain prostaglandins and ergot alkaloids. (From AMA Drug Evaluations, 1994, p1157)'),
  'D010121': NtO(MH='Oxytocin', MS='A nonapeptide hormone released from the neurohypophysis (PITUITARY GLAND, POSTERIOR). It differs from VASOPRESSIN by two amino acids at residues 3 and 8. Oxytocin acts on SMOOTH MUSCLE CELLS, such as causing UTERINE CONTRACTIONS and MILK EJECTION.'),
  'D010122': NtO(MH='Cystinyl Aminopeptidase', MS='A zinc-containing sialoglycoprotein that is used to study aminopeptidase activity in the pathogenesis of hypertension. EC 3.4.11.3.'),
  'D010123': NtO(MH='Oxyuriasis', MS='Infection with nematodes of the superfamily OXYUROIDEA.'),
  'D010124': NtO(MH='Oxyuroidea', MS='A superfamily of parasitic nematodes consisting of several genera. ENTEROBIUS, which occurs in humans, and Oxyuris, which occurs in horses, are two of the most common. Other genera are: Skrjabinema, Passalurus, Dermatoxys, and Probstmayria.'),
  'D010125': NtO(MH='Ostreidae', MS='A family of marine mollusks in the class BIVALVIA, commonly known as oysters. They have a rough irregular shell closed by a single adductor muscle.'),
  'D010126': NtO(MH='Ozone', MS='The unstable triatomic form of oxygen, O3. It is a powerful oxidant that is produced for various chemical and industrial uses. Its production is also catalyzed in the ATMOSPHERE by ULTRAVIOLET RAY irradiation of oxygen or other ozone precursors such as VOLATILE ORGANIC COMPOUNDS and NITROGEN OXIDES. About 90% of the ozone in the atmosphere exists in the stratosphere (STRATOSPHERIC OZONE).'),
  'D010127': NtO(MH='P Blood-Group System', MS='A blood group related to the ABO, Lewis and I systems. At least five different erythrocyte antigens are possible, some very rare, others almost universal. Multiple alleles are involved in this blood group.'),
  'D010128': NtO(MH='p-Aminoazobenzene', MS='Used in the form of its salts as a dye and as an intermediate in manufacture of Acid Yellow, diazo dyes, and indulines.'),
  'D010129': NtO(MH='4-Aminobenzoic Acid', MS='An aminobenzoic acid isomer that combines with pteridine and GLUTAMIC ACID to form FOLIC ACID. The fact that 4-aminobenzoic acid absorbs light throughout the UVB range has also resulted in its use as an ingredient in SUNSCREENS.'),
  'D010130': NtO(MH='p-Aminohippuric Acid', MS='The glycine amide of 4-aminobenzoic acid. Its sodium salt is used as a diagnostic aid to measure effective renal plasma flow (ERPF) and excretory capacity.'),
  'D010131': NtO(MH='Aminosalicylic Acid', MS='An antitubercular agent often administered in association with ISONIAZID. The sodium salt of the drug is better tolerated than the free acid.'),
  'D010132': NtO(MH='p-Azobenzenearsonate', MS='A hapten capable of eliciting both antibody formation and delayed hypersensitivity when bound to aromatic amino acids, polypeptides or proteins. It is used as an immunologic research tool.'),
  'D010133': NtO(MH='p-Chloroamphetamine', MS='Chlorinated analog of AMPHETAMINE. Potent neurotoxin that causes release and eventually depletion of serotonin in the CNS. It is used as a research tool.'),
  'D010134': NtO(MH='Fenclonine', MS='A selective and irreversible inhibitor of tryptophan hydroxylase, a rate-limiting enzyme in the biosynthesis of serotonin (5-HYDROXYTRYPTAMINE). Fenclonine acts pharmacologically to deplete endogenous levels of serotonin.'),
  'D010135': NtO(MH='p-Fluorophenylalanine', MS='3-(p-Fluorophenyl)-alanine.'),
  'D010136': NtO(MH='p-Hydroxyamphetamine', MS='Amphetamine metabolite with sympathomimetic effects. It is sometimes called alpha-methyltyramine, which may also refer to the meta isomer, gepefrine.'),
  'D010137': NtO(MH='p-Hydroxynorephedrine', MS='Parahydroxy analog of phenylpropanolamine with properties as a sympathomimetic.'),
  'D010138': NtO(MH='Pacemaker, Artificial', MS='A device designed to stimulate, by electric impulses, contraction of the heart muscles. It may be temporary (external) or permanent (internal or internal-external).'),
  'D010139': NtO(MH='Pacific Islands', MS="The islands of the Pacific Ocean divided into MICRONESIA; MELANESIA; and POLYNESIA (including NEW ZEALAND). The collective name Oceania includes the aforenamed islands, adding AUSTRALIA; NEW ZEALAND; and the Malay Archipelago (INDONESIA). (Webster's New Geographical Dictionary, 1988, p910, 880)"),
  'D010140': NtO(MH='Pacific Ocean', MS='Body of water between Asia and South America, extending from the Arctic circle to the Equator and from the Equator to the Antarctic.'),
  'D010141': NtO(MH='Pacinian Corpuscles', MS='Rapidly adapting mechanoreceptors found in subcutaneous tissue beneath both hairy and glabrous skin. Pacinian corpuscles contain an afferent nerve fiber surrounded by a capsule with multiple concentric layers. They have large receptive fields and are most sensitive to high-frequency stimuli, such as vibration.'),
  'D010142': NtO(MH='Pactamycin', MS='Antibiotic produced by Streptomyces pactum used as an antineoplastic agent. It is also used as a tool in biochemistry because it inhibits certain steps in protein synthesis.'),
  'D010143': NtO(MH='Paecilomyces', MS='A mitosporic fungal genus occasionally causing human diseases such as pulmonary infections, mycotic keratitis, endocarditis, and opportunistic infections. Its teleomorph is BYSSOCHLAMYS.'),
  'D010144': NtO(MH="Paget's Disease, Mammary", MS="An intraductal in situ carcinoma of the breast, characterized clinically by eczema-like inflammatory skin changes and histologically by infiltration of the dermis by malignant cells (Paget's cells)."),
  'D010145': NtO(MH='Paget Disease, Extramammary', MS='A rare cutaneous neoplasm that occurs in the elderly. It develops more frequently in women and predominantly involves apocrine gland-bearing areas, especially the vulva, scrotum, and perianal areas. The lesions develop as erythematous scaly patches that progress to crusted, pruritic, erythematous plaques. The clinical differential diagnosis includes squamous cell carcinoma in situ and superficial fungal infection. It is generally thought to be an adenocarcinoma of the epidermis, from which it extends into the contiguous epithelium of hair follicles and eccrine sweat ducts. (DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1478)'),
  'D010146': NtO(MH='Pain', MS='An unpleasant sensation induced by noxious stimuli which are detected by NERVE ENDINGS of NOCICEPTIVE NEURONS.'),
  'D010147': NtO(MH='Pain Measurement', MS='Scales, questionnaires, tests, and other methods used to assess pain severity and duration in patients or experimental animals to aid in diagnosis, therapy, and physiological studies.'),
  'D010148': NtO(MH='Pain, Intractable', MS='Persistent pain that is refractory to some or all forms of treatment.'),
  'D010149': NtO(MH='Pain, Postoperative', MS='Pain during the period after surgery.'),
  'D010150': NtO(MH='Paint', MS='An emulsion of solid color which when spread over a surface leaves a thin decorative and or protective coating.'),
  'D010151': NtO(MH='Paintings', MS='Works of art formed primarily by the direct application of pigments suspended in oil, water, egg yolk, molten wax, or other liquid, arranged in masses of color, onto a generally two-dimensional surface. (Getty ATT)'),
  'D010152': NtO(MH='Pair Bond', MS='In animals, the social relationship established between a male and female for reproduction. It may include raising of young.'),
  'D010153': NtO(MH='Paired-Associate Learning', MS='Learning in which the subject must respond with one word or syllable when presented with another word or syllable.'),
  'D010154': NtO(MH='Pakistan', MS='A country located in southern Asia, bordering the Arabian Sea, between India on the east and Iran and Afghanistan on the west and China in the north. The capital is Islamabad.'),
  'D010155': NtO(MH='Palatal Expansion Technique', MS="An orthodontic method used for correcting narrow or collapsed maxillary arches and functional cross-bite. (From Jablonski's Dictionary of Dentistry),"),
  'D010156': NtO(MH='Palatal Muscles', MS='The muscles of the palate are the glossopalatine, palatoglossus, levator palati(ni), musculus uvulae, palatopharyngeus, and tensor palati(ni).'),
  'D010157': NtO(MH='Palatal Neoplasms', MS='Tumors or cancer of the PALATE, including those of the hard palate, soft palate and UVULA.'),
  'D010158': NtO(MH='Palatal Obturators', MS='Appliances that close a cleft or fissure of the palate.'),
  'D010159': NtO(MH='Palate', MS='The structure that forms the roof of the mouth. It consists of the anterior hard palate (PALATE, HARD) and the posterior soft palate (PALATE, SOFT).'),
  'D010160': NtO(MH='Palate, Soft', MS='A movable fold suspended from the posterior border of the hard palate. The uvula hangs from the middle of the lower border.'),
  'D010161': NtO(MH='Paleodontology', MS='The study of the teeth of early forms of life through fossil remains.'),
  'D010162': NtO(MH='Paleography', MS="The study of ancient inscriptions and modes of writing. It includes the deciphering of manuscripts and other forms to determine their date, provenance, etc. (Webster's 1st ed)"),
  'D010163': NtO(MH='Paleontology', MS='The study of early forms of life through fossil remains.'),
  'D010164': NtO(MH='Paleopathology', MS='The study of disease in prehistoric times as revealed in bones, mummies, and archaeologic artifacts.'),
  'D010165': NtO(MH='Palladium', MS='A chemical element having an atomic weight of 106.4, atomic number of 46, and the symbol Pd. It is a white, ductile metal resembling platinum, and following it in abundance and importance of applications. It is used in dentistry in the form of gold, silver, and copper alloys.'),
  'D010166': NtO(MH='Palliative Care', MS='Care alleviating symptoms without curing the underlying disease. (Stedman, 25th ed)'),
  'D010167': NtO(MH='Pallor', MS='A clinical manifestation consisting of an unnatural paleness of the skin.'),
  'D010168': NtO(MH='Palmitates', MS='Salts and esters of the 16-carbon saturated monocarboxylic acid--palmitic acid.'),
  'D010169': NtO(MH='Palmitic Acids', MS='A group of 16-carbon fatty acids that contain no double bonds.'),
  'D010170': NtO(MH='Palmitoyl-CoA Hydrolase', MS='Enzyme catalyzing reversibly the hydrolysis of palmitoyl-CoA or other long-chain acyl coenzyme A compounds to yield CoA and palmitate or other acyl esters. The enzyme is involved in the esterification of fatty acids to form triglycerides. EC 3.1.2.2.'),
  'D010171': NtO(MH='Palmitoyl Coenzyme A', MS='A fatty acid coenzyme derivative which plays a key role in fatty acid oxidation and biosynthesis.'),
  'D010172': NtO(MH='Palmitoylcarnitine', MS='A long-chain fatty acid ester of carnitine which facilitates the transfer of long-chain fatty acids from cytoplasm into mitochondria during the oxidation of fatty acids.'),
  'D010173': NtO(MH='Palpation', MS='Application of fingers with light pressure to the surface of the body to determine consistence of parts beneath in physical diagnosis; includes palpation for determining the outlines of organs.'),
  'D010174': NtO(MH='Pamphlets', MS='Printed publications usually having a format with no binding and no cover and having fewer than some set number of pages. They are often devoted to a single subject.'),
  'D010175': NtO(MH='Pan American Health Organization', MS='WHO regional office for the Americas acting as a coordinating agency for the improvement of health conditions in the hemisphere. The four main functions are: control or eradication of communicable diseases, strengthening of national and local health services, education and training, and research.'),
  'D010176': NtO(MH='Panama', MS='A country in CENTRAL AMERICA, bordering both the Caribbean Sea and the North Pacific Ocean, between COLOMBIA and COSTA RICA.'),
  'D010177': NtO(MH='Panama Canal Zone', MS='The area across the Isthmus of Panama formerly administered by the UNITED STATES from 1903 to 1979 for the operation of the Panama Canal. Transfer to PANAMA was completed in 1999.'),
  'D010178': NtO(MH='Pancoast Syndrome', MS='A condition caused by an apical lung tumor (Pancoast tumor) with involvement of the nearby vertebral column and the BRACHIAL PLEXUS. Symptoms include pain in the shoulder and the arm, and atrophy of the hand.'),
  'D010179': NtO(MH='Pancreas', MS='A nodular organ in the ABDOMEN that contains a mixture of ENDOCRINE GLANDS and EXOCRINE GLANDS. The small endocrine portion consists of the ISLETS OF LANGERHANS secreting a number of hormones into the blood stream. The large exocrine portion (EXOCRINE PANCREAS) is a compound acinar gland that secretes several digestive enzymes into the pancreatic ductal system that empties into the DUODENUM.'),
  'D010180': NtO(MH='Pancreatectomy', MS='Surgical removal of the pancreas. (Dorland, 28th ed)'),
  'D010181': NtO(MH='Pancreatic Cyst', MS='A true cyst of the PANCREAS, distinguished from the much more common PANCREATIC PSEUDOCYST by possessing a lining of mucous EPITHELIUM. Pancreatic cysts are categorized as congenital, retention, neoplastic, parasitic, enterogenous, or dermoid. Congenital cysts occur more frequently as solitary cysts but may be multiple. Retention cysts are gross enlargements of PANCREATIC DUCTS secondary to ductal obstruction. (From Bockus Gastroenterology, 4th ed, p4145)'),
  'D010182': NtO(MH='Pancreatic Diseases', MS='Pathological processes of the PANCREAS.'),
  'D010183': NtO(MH='Pancreatic Ducts', MS='Ducts that collect PANCREATIC JUICE from the PANCREAS and supply it to the DUODENUM.'),
  'D010184': NtO(MH='Pancreatic Extracts', MS='Extracts prepared from pancreatic tissue that may contain the pancreatic enzymes or other specific uncharacterized factors or proteins with specific activities. PANCREATIN is a specific extract containing digestive enzymes and used to treat pancreatic insufficiency.'),
  'D010185': NtO(MH='Pancreatic Fistula', MS='Abnormal passage communicating with the PANCREAS.'),
  'D010186': NtO(MH='Pancreatic Function Tests', MS='Tests based on the biochemistry and physiology of the exocrine pancreas and involving analysis of blood, duodenal contents, feces, or urine for products of pancreatic secretion.'),
  'D010187': NtO(MH='Pancreatic Hormones', MS='Peptide hormones secreted into the blood by cells in the ISLETS OF LANGERHANS of the pancreas. The alpha cells secrete glucagon; the beta cells secrete insulin; the delta cells secrete somatostatin; and the PP cells secrete pancreatic polypeptide.'),
  'D010188': NtO(MH='Exocrine Pancreatic Insufficiency', MS='A malabsorption condition resulting from greater than 10% reduction in the secretion of pancreatic digestive enzymes (LIPASE; PROTEASES; and AMYLASE) by the EXOCRINE PANCREAS into the DUODENUM. This condition is often associated with CYSTIC FIBROSIS and with chronic PANCREATITIS.'),
  'D010189': NtO(MH='Pancreatic Juice', MS='The fluid containing digestive enzymes secreted by the pancreas in response to food in the duodenum.'),
  'D010190': NtO(MH='Pancreatic Neoplasms', MS='Tumors or cancer of the PANCREAS. Depending on the types of ISLET CELLS present in the tumors, various hormones can be secreted: GLUCAGON from PANCREATIC ALPHA CELLS; INSULIN from PANCREATIC BETA CELLS; and SOMATOSTATIN from the SOMATOSTATIN-SECRETING CELLS. Most are malignant except the insulin-producing tumors (INSULINOMA).'),
  'D010191': NtO(MH='Pancreatic Polypeptide', MS='A 36-amino acid pancreatic hormone that is secreted mainly by endocrine cells found at the periphery of the ISLETS OF LANGERHANS and adjacent to cells containing SOMATOSTATIN and GLUCAGON. Pancreatic polypeptide (PP), when administered peripherally, can suppress gastric secretion, gastric emptying, pancreatic enzyme secretion, and appetite. A lack of pancreatic polypeptide (PP) has been associated with OBESITY in rats and mice.'),
  'D010192': NtO(MH='Pancreatic Pseudocyst', MS='Cyst-like space not lined by EPITHELIUM and contained within the PANCREAS. Pancreatic pseudocysts account for most of the cystic collections in the pancreas and are often associated with chronic PANCREATITIS.'),
  'D010193': NtO(MH='Pancreaticojejunostomy', MS='Surgical anastomosis of the pancreatic duct, or the divided end of the transected pancreas, with the jejunum. (Dorland, 28th ed)'),
  'D010194': NtO(MH='Pancreatin', MS='A mammalian pancreatic extract composed of enzymes with protease, amylase and lipase activities. It is used as a digestant in pancreatic malfunction.'),
  'D010195': NtO(MH='Pancreatitis', MS='INFLAMMATION of the PANCREAS. Pancreatitis is classified as acute unless there are computed tomographic or endoscopic retrograde cholangiopancreatographic findings of CHRONIC PANCREATITIS (International Symposium on Acute Pancreatitis, Atlanta, 1992). The two most common forms of acute pancreatitis are ALCOHOLIC PANCREATITIS and gallstone pancreatitis.'),
  'D010196': NtO(MH='Pancreatic Elastase', MS='A protease of broad specificity, obtained from dried pancreas. Molecular weight is approximately 25,000. The enzyme breaks down elastin, the specific protein of elastic fibers, and digests other proteins such as fibrin, hemoglobin, and albumin. EC 3.4.21.36.'),
  'D010197': NtO(MH='Pancuronium', MS='A bis-quaternary steroid that is a competitive nicotinic antagonist. As a neuromuscular blocking agent it is more potent than CURARE but has less effect on the circulatory system and on histamine release.'),
  'D010198': NtO(MH='Pancytopenia', MS='Deficiency of all three cell elements of the blood, erythrocytes, leukocytes and platelets.'),
  'D010200': NtO(MH='Panic', MS='A state of extreme acute, intense anxiety and unreasoning fear accompanied by disorganization of personality function.'),
  'D010201': NtO(MH='Panniculitis, Nodular Nonsuppurative', MS='A form of panniculitis characterized by recurrent episodes of fever accompanied by the eruption of single or multiple erythematous subcutaneous nodules on the lower extremities. They normally resolve, but tend to leave depressions in the skin. The condition is most often seen in women, alone or in association with other disorders.'),
  'D010202': NtO(MH='Panophthalmitis', MS='Acute suppurative inflammation of the inner eye with necrosis of the sclera (and sometimes the cornea) and extension of the inflammation into the orbit. Pain may be severe and the globe may rupture. In endophthalmitis the globe does not rupture.'),
  'D010203': NtO(MH='Panstrongylus', MS='A genus of cone-nosed bugs of the subfamily TRIATOMINAE. Its species are vectors of TRYPANOSOMA CRUZI.'),
  'D010204': NtO(MH='Pantetheine', MS='An intermediate in the pathway of coenzyme A formation in mammalian liver and some microorganisms.'),
  'D010205': NtO(MH='Pantothenic Acid', MS='A butyryl-beta-alanine that can also be viewed as pantoic acid complexed with BETA ALANINE. It is incorporated into COENZYME A and protects cells against peroxidative damage by increasing the level of GLUTATHIONE.'),
  'D010206': NtO(MH='Papain', MS='A proteolytic enzyme obtained from Carica papaya. It is also the name used for a purified mixture of papain and CHYMOPAPAIN that is used as a topical enzymatic debriding agent. EC 3.4.22.2.'),
  'D010207': NtO(MH='Papaver', MS='A genus of Eurasian herbaceous plants, the poppies (family PAPAVERACEAE of the dicotyledon class Magnoliopsida), that yield OPIUM from the latex of the unripe seed pods.'),
  'D010208': NtO(MH='Papaverine', MS='An alkaloid found in opium but not closely related to the other opium alkaloids in its structure or pharmacological actions. It is a direct-acting smooth muscle relaxant used in the treatment of impotence and as a vasodilator, especially for cerebral vasodilation. The mechanism of its pharmacological actions is not clear, but it apparently can inhibit phosphodiesterases and it may have direct actions on calcium channels.'),
  'D010209': NtO(MH='Paper', MS='Thin sheets made from wood pulp and other fibrous substances, used for writing, drawing, printing, image duplication or wrapping.'),
  'D010210': NtO(MH='Papillary Muscles', MS='Conical muscular projections from the walls of the cardiac ventricles, attached to the cusps of the atrioventricular valves by the chordae tendineae.'),
  'D010211': NtO(MH='Papilledema', MS='Swelling of the OPTIC DISK, usually in association with increased intracranial pressure, characterized by hyperemia, blurring of the disk margins, microhemorrhages, blind spot enlargement, and engorgement of retinal veins. Chronic papilledema may cause OPTIC ATROPHY and visual loss. (Miller et al., Clinical Neuro-Ophthalmology, 4th ed, p175)'),
  'D010212': NtO(MH='Papilloma', MS='A circumscribed benign epithelial tumor projecting from the surrounding surface; more precisely, a benign epithelial neoplasm consisting of villous or arborescent outgrowths of fibrovascular stroma covered by neoplastic cells. (Stedman, 25th ed)'),
  'D010214': NtO(MH='Papillon-Lefevre Disease', MS='Rare, autosomal recessive disorder occurring between the first and fifth years of life. It is characterized by palmoplantar keratoderma with periodontitis followed by the premature shedding of both deciduous and permanent teeth. Mutations in the gene for CATHEPSIN C have been associated with this disease.'),
  'D010215': NtO(MH='Papio', MS='A genus of the subfamily CERCOPITHECINAE, family CERCOPITHECIDAE, consisting of five named species: PAPIO URSINUS (chacma baboon), PAPIO CYNOCEPHALUS (yellow baboon), PAPIO PAPIO (western baboon), PAPIO ANUBIS (or olive baboon), and PAPIO HAMADRYAS (hamadryas baboon). Members of the Papio genus inhabit open woodland, savannahs, grassland, and rocky hill country. Some authors consider MANDRILLUS a subgenus of Papio.'),
  'D010217': NtO(MH='Phlebotomus Fever', MS='Influenza-like febrile viral disease caused by several members of the BUNYAVIRIDAE family and transmitted mostly by the bloodsucking sandfly Phlebotomus papatasii.'),
  'D010219': NtO(MH='Papua New Guinea', MS="A country consisting of the eastern half of the island of New Guinea and adjacent islands, including New Britain, New Ireland, the Admiralty Islands, and New Hanover in the Bismarck Archipelago; Bougainville and Buka in the northern Solomon Islands; the D'Entrecasteaux and Trobriand Islands; Woodlark (Murua) Island; and the Louisiade Archipelago. It became independent on September 16, 1975. Formerly, the southern part was the Australian Territory of Papua, and the northern part was the UN Trust Territory of New Guinea, administered by Australia. They were administratively merged in 1949 and named Papua and New Guinea, and renamed Papua New Guinea in 1971."),
  'D010220': NtO(MH='Para-Aortic Bodies', MS='Small masses of chromaffin cells found near the SYMPATHETIC GANGLIA along the ABDOMINAL AORTA, beginning cranial to the superior mesenteric artery (MESENTERIC ARTERY, SUPERIOR) or renal arteries and extending to the level of the aortic bifurcation or just beyond. They are also called the organs of Zuckerkandl and sometimes called aortic bodies (not to be confused with AORTIC BODIES in the THORAX). The para-aortic bodies are the dominant source of CATECHOLAMINES in the FETUS and normally regress after BIRTH.'),
  'D010222': NtO(MH='Parainfluenza Virus 1, Human', MS='A species of RESPIROVIRUS also called hemadsorption virus 2 (HA2), which causes laryngotracheitis in humans, especially children.'),
  'D010223': NtO(MH='Parainfluenza Virus 2, Human', MS='A species of RUBULAVIRUS associated particularly with acute laryngotracheitis (CROUP) in children aged 6 months to 3 years.'),
  'D010224': NtO(MH='Parainfluenza Virus 3, Human', MS='A species of RESPIROVIRUS frequently isolated from small children with pharyngitis, bronchitis, and pneumonia.'),
  'D010226': NtO(MH='Parabens', MS="Methyl, propyl, butyl, and ethyl esters of p-hydroxybenzoic acid. They have been approved by the FDA as antimicrobial agents for foods and pharmaceuticals. (From Hawley's Condensed Chemical Dictionary, 11th ed, p872)"),
  'D010227': NtO(MH='Parabiosis', MS='The experimental joining of two individuals for the purpose of studying the effects of one on the other.'),
  'D010228': NtO(MH='Paracoccidioides', MS='A mitosporic fungal genus. P. brasiliensis (previously Blastomyces brasiliensis) is the etiologic agent of PARACOCCIDIOIDOMYCOSIS.'),
  'D010229': NtO(MH='Paracoccidioidomycosis', MS='A mycosis affecting the skin, mucous membranes, lymph nodes, and internal organs. It is caused by Paracoccidioides brasiliensis. It is also called paracoccidioidal granuloma. Superficial resemblance of P. brasiliensis to Blastomyces brasiliensis (BLASTOMYCES) may cause misdiagnosis.'),
  'D010230': NtO(MH='Paracoccus', MS='Gram-negative non-motile bacteria found in soil or brines.'),
  'D010231': NtO(MH='Paracoccus denitrificans', MS='A species of bacteria isolated from soil.'),
  'D010232': NtO(MH='Paraffin', MS='A mixture of solid hydrocarbons obtained from petroleum. It has a wide range of uses including as a stiffening agent in ointments, as a lubricant, and as a topical anti-inflammatory. It is also commonly used as an embedding material in histology.'),
  'D010233': NtO(MH='Paraganglia, Chromaffin', MS='Small bodies containing chromaffin cells occurring outside of the adrenal medulla, most commonly near the sympathetic ganglia and in organs such as the kidney, liver, heart and gonads.'),
  'D010234': NtO(MH='Paraganglia, Nonchromaffin', MS='Several clusters of chemoreceptive and supporting cells associated with blood vessels and nerves (especially the glossopharyngeal and vagus). The nonchromaffin paraganglia sense pH, carbon dioxide, and oxygen concentrations in the blood and participate in respiratory, and perhaps circulatory, control. They include the CAROTID BODY; AORTIC BODIES; the GLOMUS JUGULARE; and the GLOMUS TYMPANICUM.'),
  'D010235': NtO(MH='Paraganglioma', MS='A neural crest tumor usually derived from the chromoreceptor tissue of a paraganglion, such as the carotid body, or medulla of the adrenal gland (usually called a chromaffinoma or pheochromocytoma). It is more common in women than in men. (Stedman, 25th ed; from Segen, Dictionary of Modern Medicine, 1992)'),
  'D010236': NtO(MH='Paraganglioma, Extra-Adrenal', MS='A relatively rare, usually benign neoplasm originating in the chemoreceptor tissue of the CAROTID BODY; GLOMUS JUGULARE; GLOMUS TYMPANICUM; AORTIC BODIES; and the female genital tract. It consists histologically of rounded or ovoid hyperchromatic cells that tend to be grouped in an alveolus-like pattern within a scant to moderate amount of fibrous stroma and a few large thin-walled vascular channels. (From Stedman, 27th ed)'),
  'D010237': NtO(MH='Paragonimiasis', MS='Infection with TREMATODA of the genus PARAGONIMUS.'),
  'D010238': NtO(MH='Paragonimus', MS='A genus of lung flukes of the family Troglotrematidae infecting humans and animals. This genus consists of several species one of which is PARAGONIMUS WESTERMANI, a common lung fluke in humans.'),
  'D010239': NtO(MH='Paraguay', MS='A country in central South America, northeast of Argentina, southwest of Brazil. The capital is Asuncion.'),
  'D010240': NtO(MH='Parakeets', MS='Common name for one of five species of small PARROTS, containing long tails.'),
  'D010241': NtO(MH='Parakeratosis', MS='Persistence of the nuclei of the keratinocytes into the stratum corneum of the skin. This is a normal state only in the epithelium of true mucous membranes in the mouth and vagina. (Dorland, 27th ed)'),
  'D010242': NtO(MH='Paraldehyde', MS='A hypnotic and sedative with anticonvulsant effects. However, because of the hazards associated with its administration, its tendency to react with plastic, and the risks associated with its deterioration, it has largely been superseded by other agents. It is still occasionally used to control status epilepticus resistant to conventional treatment. (From Martindale, The Extra Pharmacopoeia, 30th ed, p608-9)'),
  'D010243': NtO(MH='Paralysis', MS='A general term most often used to describe severe or complete loss of muscle strength due to motor system disease from the level of the cerebral cortex to the muscle fiber. This term may also occasionally refer to a loss of sensory function. (From Adams et al., Principles of Neurology, 6th ed, p45)'),
  'D010244': NtO(MH='Bulbar Palsy, Progressive', MS='A motor neuron disease marked by progressive weakness of the muscles innervated by cranial nerves of the lower brain stem. Clinical manifestations include dysarthria, dysphagia, facial weakness, tongue weakness, and fasciculations of the tongue and facial muscles. The adult form of the disease is marked initially by bulbar weakness which progresses to involve motor neurons throughout the neuroaxis. Eventually this condition may become indistinguishable from AMYOTROPHIC LATERAL SCLEROSIS. Fazio-Londe syndrome is an inherited form of this illness which occurs in children and young adults. (Adams et al., Principles of Neurology, 6th ed, p1091; Brain 1992 Dec;115(Pt 6):1889-1900)'),
  'D010245': NtO(MH='Paralyses, Familial Periodic', MS='A heterogenous group of inherited disorders characterized by recurring attacks of rapidly progressive flaccid paralysis or myotonia. These conditions have in common a mutation of the gene encoding the alpha subunit of the sodium channel in skeletal muscle. They are frequently associated with fluctuations in serum potassium levels. Periodic paralysis may also occur as a non-familial process secondary to THYROTOXICOSIS and other conditions. (From Adams et al., Principles of Neurology, 6th ed, p1481)'),
  'D010246': NtO(MH='Paralysis, Obstetric', MS='Paralysis of an infant resulting from injury received at birth. (From Dorland, 27th ed)'),
  'D010247': NtO(MH='Paramecium', MS='A genus of ciliate protozoa that is often large enough to be seen by the naked eye. Paramecia are commonly used in genetic, cytological, and other research.'),
  'D010248': NtO(MH='Paramethasone', MS='A glucocorticoid with the general properties of corticosteroids. It has been used by mouth in the treatment of all conditions in which corticosteroid therapy is indicated except adrenal-deficiency states for which its lack of sodium-retaining properties makes it less suitable than HYDROCORTISONE with supplementary FLUDROCORTISONE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p737)'),
  'D010249': NtO(MH='Parametritis', MS='Inflammation of the parametrium, the connective tissue of the pelvic floor, extending from the subserous coat of the uterus laterally between the layers of the BROAD LIGAMENT.'),
  'D010250': NtO(MH='Paramphistomatidae', MS='A family of flukes of the class Trematoda found in the intestinal tract and liver of animals and man. Some of the genera are Homalagaster, Gastrodiscus, Paramphistomum, Watsonius, Nilocotyle, Gigantocotyle, Gastrothylax, Macropotrema, Ceylonocotyle, Zygocotyle, Cotylophoron, and Calicophoron.'),
  'D010252': NtO(MH='Paramyxoviridae', MS='A family of spherical viruses, of the order MONONEGAVIRALES, somewhat larger than the orthomyxoviruses, and containing single-stranded RNA. Subfamilies include PARAMYXOVIRINAE and PNEUMOVIRINAE.'),
  'D010253': NtO(MH='Respirovirus Infections', MS='Infections with viruses of the genus RESPIROVIRUS, family PARAMYXOVIRIDAE. Host cell infection occurs by adsorption, via HEMAGGLUTININ, to the cell surface.'),
  'D010254': NtO(MH='Paranasal Sinus Diseases', MS='Diseases affecting or involving the PARANASAL SINUSES and generally manifesting as inflammation, abscesses, cysts, or tumors.'),
  'D010255': NtO(MH='Paranasal Sinus Neoplasms', MS='Tumors or cancer of the PARANASAL SINUSES.'),
  'D010256': NtO(MH='Paranasal Sinuses', MS='Air-filled spaces located within the bones around the NASAL CAVITY. They are extensions of the nasal cavity and lined by the ciliated NASAL MUCOSA. Each sinus is named for the cranial bone in which it is located, such as the ETHMOID SINUS; the FRONTAL SINUS; the MAXILLARY SINUS; and the SPHENOID SINUS.'),
  'D010257': NtO(MH='Paraneoplastic Syndromes', MS='In patients with neoplastic diseases a wide variety of clinical pictures which are indirect and usually remote effects produced by tumor cell metabolites or other products.'),
  'D010258': NtO(MH='Paranoid Behavior', MS='Behavior exhibited by individuals who are overly suspicious, but without the constellation of symptoms characteristic of paranoid personality disorder or paranoid type of schizophrenia.'),
  'D010259': NtO(MH='Paranoid Disorders', MS='Chronic mental disorders in which there has been an insidious development of a permanent and unshakeable delusional system (persecutory delusions or delusions of jealousy), accompanied by preservation of clear and orderly thinking. Emotional responses and behavior are consistent with the delusional state.'),
  'D010260': NtO(MH='Paranoid Personality Disorder', MS='A personality disorder characterized by the avoidance of accepting deserved blame and an unwarranted view of others as malevolent. The latter is expressed as suspiciousness, hypersensitivity, and mistrust.'),
  'D010261': NtO(MH='Paraoxon', MS='An organophosphate cholinesterase inhibitor that is used as a pesticide.'),
  'D010262': NtO(MH='Paraphilic Disorders', MS='Disorders that include recurrent, intense sexually arousing fantasies, sexual urges, or behaviors generally involving nonhuman objects, suffering of oneself or partners, or children or other nonconsenting partners. (from DSM-V)'),
  'D010263': NtO(MH='Paraphimosis', MS='A condition in which the FORESKIN, once retracted, cannot return to its original position. If this condition persists, it can lead to painful constriction of GLANS PENIS, swelling, and impaired blood flow to the penis.'),
  'D010264': NtO(MH='Paraplegia', MS='Severe or complete loss of motor function in the lower extremities and lower portions of the trunk. This condition is most often associated with SPINAL CORD DISEASES, although BRAIN DISEASES; PERIPHERAL NERVOUS SYSTEM DISEASES; NEUROMUSCULAR DISEASES; and MUSCULAR DISEASES may also cause bilateral leg weakness.'),
  'D010265': NtO(MH='Paraproteinemias', MS='A group of related diseases characterized by an unbalanced or disproportionate proliferation of immunoglobulin-producing cells, usually from a single clone. These cells frequently secrete a structurally homogeneous immunoglobulin (M-component) and/or an abnormal immunoglobulin.'),
  'D010266': NtO(MH='Paraproteins', MS='Abnormal immunoglobulins synthesized by atypical cells of the MONONUCLEAR PHAGOCYTE SYSTEM. Paraproteins containing only light chains lead to Bence Jones paraproteinemia, while the presence of only atypical heavy chains leads to heavy chain disease. Most of the paraproteins show themselves as an M-component (monoclonal gammopathy) in electrophoresis. Diclonal and polyclonal paraproteins are much less frequently encountered.'),
  'D010267': NtO(MH='Parapsoriasis', MS='The term applied to a group of relatively uncommon inflammatory, maculopapular, scaly eruptions of unknown etiology and resistant to conventional treatment. Eruptions are both psoriatic and lichenoid in appearance, but the diseases are distinct from psoriasis, lichen planus, or other recognized dermatoses. Proposed nomenclature divides parapsoriasis into two distinct subgroups, PITYRIASIS LICHENOIDES and parapsoriasis en plaques (small- and large-plaque parapsoriasis).'),
  'D010268': NtO(MH='Parapsychology', MS='Branch of psychology that deals with paranormal behavior and events such as telepathy, precognition, and clairvoyance, which are not explicable by present day "natural laws".'),
  'D010269': NtO(MH='Paraquat', MS='A poisonous dipyridilium compound used as contact herbicide. Contact with concentrated solutions causes irritation of the skin, cracking and shedding of the nails, and delayed healing of cuts and wounds.'),
  'D010270': NtO(MH='Parasite Egg Count', MS='Determination of parasite eggs in feces.'),
  'D010271': NtO(MH='Parasites', MS='Invertebrate organisms that live on or in another organism (the host), and benefit at the expense of the other. Traditionally excluded from definition of parasites are pathogenic BACTERIA; FUNGI; VIRUSES; and PLANTS; though they may live parasitically.'),
  'D010272': NtO(MH='Parasitic Diseases', MS='Infections or infestations with PARASITES. They are often contracted through contact with an intermediate vector, but may occur as the result of direct exposure.'),
  'D010273': NtO(MH='Parasitic Diseases, Animal', MS='Animal diseases caused by PARASITES.'),
  'D010274': NtO(MH='Parasitology', MS='The study of parasites and PARASITIC DISEASES.'),
  'D010275': NtO(MH='Parasympathetic Nervous System', MS='The craniosacral division of the autonomic nervous system. The cell bodies of the parasympathetic preganglionic fibers are in brain stem nuclei and in the sacral spinal cord. They synapse in cranial autonomic ganglia or in terminal ganglia near target organs. The parasympathetic nervous system generally acts to conserve resources and restore homeostasis, often with effects reciprocal to the sympathetic nervous system.'),
  'D010276': NtO(MH='Parasympatholytics', MS='Agents that inhibit the actions of the parasympathetic nervous system. The major group of drugs used therapeutically for this purpose is the MUSCARINIC ANTAGONISTS.'),
  'D010277': NtO(MH='Parasympathomimetics', MS='Drugs that mimic the effects of parasympathetic nervous system activity. Included here are drugs that directly stimulate muscarinic receptors and drugs that potentiate cholinergic activity, usually by slowing the breakdown of acetylcholine (CHOLINESTERASE INHIBITORS). Drugs that stimulate both sympathetic and parasympathetic postganglionic neurons (GANGLIONIC STIMULANTS) are not included here.'),
  'D010278': NtO(MH='Parathion', MS='A highly toxic cholinesterase inhibitor that is used as an acaricide and as an insecticide.'),
  'D010279': NtO(MH='Parathyroid Diseases', MS='Pathological processes of the PARATHYROID GLANDS. They usually manifest as hypersecretion or hyposecretion of PARATHYROID HORMONE that regulates the balance of CALCIUM; PHOSPHORUS; and MAGNESIUM in the body.'),
  'D010280': NtO(MH='Parathyroid Glands', MS='Two pairs of small oval-shaped glands located in the front and the base of the NECK and adjacent to the two lobes of THYROID GLAND. They secrete PARATHYROID HORMONE that regulates the balance of CALCIUM; PHOSPHORUS; and MAGNESIUM in the body.'),
  'D010281': NtO(MH='Parathyroid Hormone', MS='A polypeptide hormone (84 amino acid residues) secreted by the PARATHYROID GLANDS which performs the essential role of maintaining intracellular CALCIUM levels in the body. Parathyroid hormone increases intracellular calcium by promoting the release of CALCIUM from BONE, increases the intestinal absorption of calcium, increases the renal tubular reabsorption of calcium, and increases the renal excretion of phosphates.'),
  'D010282': NtO(MH='Parathyroid Neoplasms', MS='Tumors or cancer of the PARATHYROID GLANDS.'),
  'D010283': NtO(MH='Paratuberculosis', MS='A chronic GASTROENTERITIS in RUMINANTS caused by MYCOBACTERIUM AVIUM SUBSPECIES PARATUBERCULOSIS.'),
  'D010284': NtO(MH='Paratyphoid Fever', MS='A prolonged febrile illness commonly caused by several Paratyphi serotypes of SALMONELLA ENTERICA. It is similar to TYPHOID FEVER but less severe.'),
  'D010285': NtO(MH='Pseudocowpox Virus', MS='A species of PARAPOXVIRUS causing a pox-like disease on udders of cows that may spread to milkers.'),
  'D010286': NtO(MH='Paraventricular Hypothalamic Nucleus', MS='Nucleus in the anterior part of the HYPOTHALAMUS.'),
  'D010287': NtO(MH='Parent-Child Relations', MS='The interactions between parent and child.'),
  'D010288': NtO(MH='Parenteral Nutrition', MS='The administering of nutrients for assimilation and utilization by a patient who cannot maintain adequate nutrition by enteral feeding alone. Nutrients are administered by a route other than the alimentary canal (e.g., intravenously, subcutaneously).'),
  'D010289': NtO(MH='Parenteral Nutrition, Total', MS='The delivery of nutrients for assimilation and utilization by a patient whose sole source of nutrients is via solutions administered intravenously, subcutaneously, or by some other non-alimentary route. The basic components of TPN solutions are protein hydrolysates or free amino acid mixtures, monosaccharides, and electrolytes. Components are selected for their ability to reverse catabolism, promote anabolism, and build structural proteins.'),
  'D010290': NtO(MH='Parents', MS='Persons functioning as natural, adoptive, or substitute parents. The heading includes the concept of parenthood as well as preparation for becoming a parent.'),
  'D010291': NtO(MH='Paresis', MS='A general term referring to a mild to moderate degree of muscular weakness, occasionally used as a synonym for PARALYSIS (severe or complete loss of motor function). In the older literature, paresis often referred specifically to paretic neurosyphilis (see NEUROSYPHILIS). "General paresis" and "general paralysis" may still carry that connotation. Bilateral lower extremity paresis is referred to as PARAPARESIS.'),
  'D010292': NtO(MH='Paresthesia', MS='Subjective cutaneous sensations (e.g., cold, warmth, tingling, pressure, etc.) that are experienced spontaneously in the absence of stimulation.'),
  'D010293': NtO(MH='Pargyline', MS='A monoamine oxidase inhibitor with antihypertensive properties.'),
  'D010294': NtO(MH='Parietal Bone', MS='One of a pair of irregularly shaped quadrilateral bones situated between the FRONTAL BONE and OCCIPITAL BONE, which together form the sides of the CRANIUM.'),
  'D010295': NtO(MH='Parietal Cells, Gastric', MS='Rounded or pyramidal cells of the GASTRIC GLANDS. They secrete HYDROCHLORIC ACID and produce gastric intrinsic factor, a glycoprotein that binds VITAMIN B12.'),
  'D010296': NtO(MH='Parietal Lobe', MS='Upper central part of the cerebral hemisphere. It is located posterior to central sulcus, anterior to the OCCIPITAL LOBE, and superior to the TEMPORAL LOBES.'),
  'D010297': NtO(MH='Paris', MS='The capital city of France.'),
  'D010298': NtO(MH='Parity', MS='The number of offspring a female has borne. It is contrasted with GRAVIDITY, which refers to the number of pregnancies, regardless of outcome.'),
  'D010299': NtO(MH='Parking Facilities', MS='Indoor or outdoor areas designated for the parking of vehicles.'),
  'D010300': NtO(MH='Parkinson Disease', MS='A progressive, degenerative neurologic disease characterized by a TREMOR that is maximal at rest, retropulsion (i.e. a tendency to fall backwards), rigidity, stooped posture, slowness of voluntary movements, and a masklike facial expression. Pathologic features include loss of melanin containing neurons in the substantia nigra and other pigmented nuclei of the brainstem. LEWY BODIES are present in the substantia nigra and locus coeruleus but may also be found in a related condition (LEWY BODY DISEASE, DIFFUSE) characterized by dementia in combination with varying degrees of parkinsonism. (Adams et al., Principles of Neurology, 6th ed, p1059, pp1067-75)'),
  'D010301': NtO(MH='Parkinson Disease, Postencephalitic', MS='Parkinsonism following encephalitis, historically seen as a sequella of encephalitis lethargica (Von Economo Encephalitis). The early age of onset, the rapid progression of symptoms followed by stabilization, and the presence of a variety of other neurological disorders (e.g., sociopathic behavior; TICS; MUSCLE SPASMS; oculogyric crises; hyperphagia; and bizarre movements) distinguish this condition from primary PARKINSON DISEASE. Pathologic features include neuronal loss and gliosis concentrated in the MESENCEPHALON; SUBTHALAMUS; and HYPOTHALAMUS. (From Adams et al., Principles of Neurology, 6th ed, p754)'),
  'D010302': NtO(MH='Parkinson Disease, Secondary', MS='Conditions which feature clinical manifestations resembling primary Parkinson disease that are caused by a known or suspected condition. Examples include parkinsonism caused by vascular injury, drugs, trauma, toxin exposure, neoplasms, infections and degenerative or hereditary conditions. Clinical features may include bradykinesia, rigidity, parkinsonian gait, and masked facies. In general, tremor is less prominent in secondary parkinsonism than in the primary form. (From Joynt, Clinical Neurology, 1998, Ch38, pp39-42)'),
  'D010303': NtO(MH='Paromomycin', MS='An aminoglycoside antibacterial and antiprotozoal agent produced by species of STREPTOMYCES.'),
  'D010304': NtO(MH='Paronychia', MS="An inflammatory reaction involving the folds of the skin surrounding the fingernail. It is characterized by acute or chronic purulent, tender, and painful swellings of the tissues around the nail, caused by an abscess of the nail fold. The pathogenic yeast causing paronychia is most frequently Candida albicans. Saprophytic fungi may also be involved. The causative bacteria are usually Staphylococcus, Pseudomonas aeruginosa, or Streptococcus. (Andrews' Diseases of the Skin, 8th ed, p271)"),
  'D010305': NtO(MH='Parotid Diseases', MS='Diseases involving the PAROTID GLAND.'),
  'D010306': NtO(MH='Parotid Gland', MS='The largest of the three pairs of SALIVARY GLANDS. They lie on the sides of the FACE immediately below and in front of the EAR.'),
  'D010307': NtO(MH='Parotid Neoplasms', MS='Tumors or cancer of the PAROTID GLAND.'),
  'D010309': NtO(MH='Parotitis', MS='INFLAMMATION of the PAROTID GLAND.'),
  'D010310': NtO(MH='Parovarian Cyst', MS='A cyst (CYSTS) near the OVARY, derived from anomalies of the FALLOPIAN TUBES or the BROAD LIGAMENT. The paramesonephric type consists of ciliated cells similar to the oviduct epithelium. The mesonephric type consisted of an epithelium with minimally surface structures. They can be found on the thin oviduct (paratubal cysts) or near its fimbriated end (hydatid of Morgagni).'),
  'D010311': NtO(MH='Parrots', MS='BIRDS of the large family Psittacidae, widely distributed in tropical regions and having a distinctive stout, curved hooked bill. The family includes LOVEBIRDS; AMAZON PARROTS; conures; PARAKEETS; and many other kinds of parrots.'),
  'D010312': NtO(MH='Parthenogenesis', MS='A unisexual reproduction without the fusion of a male and a female gamete (FERTILIZATION). In parthenogenesis, an individual is formed from an unfertilized OVUM that did not complete MEIOSIS. Parthenogenesis occurs in nature and can be artificially induced.'),
  'D010313': NtO(MH='Partial Pressure', MS='The pressure that would be exerted by one component of a mixture of gases if it were present alone in a container. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D010314': NtO(MH='Partial Thromboplastin Time', MS='The time required for the appearance of FIBRIN strands following the mixing of PLASMA with phospholipid platelet substitute (e.g., crude cephalins, soybean phosphatides). It is a test of the intrinsic pathway (factors VIII, IX, XI, and XII) and the common pathway (fibrinogen, prothrombin, factors V and X) of BLOOD COAGULATION. It is used as a screening test and to monitor HEPARIN therapy.'),
  'D010315': NtO(MH='Particle Accelerators', MS='Devices which accelerate electrically charged atomic or subatomic particles, such as electrons, protons or ions, to high velocities so they have high kinetic energy.'),
  'D010316': NtO(MH='Particle Size', MS='Relating to the size of solids.'),
  'D010317': NtO(MH='Partnership Practice', MS='A voluntary contract between two or more doctors who may or may not share responsibility for the care of patients, with proportional sharing of profits and losses.'),
  'D010318': NtO(MH='Partnership Practice, Dental', MS='A voluntary contract between two or more dentists who may or may not share responsibility for the care of patients, with proportional sharing of profits and losses.'),
  'D010319': NtO(MH='Parturient Paresis', MS='A disease of pregnant and lactating cows and ewes leading to generalized paresis and death. The disease, which is characterized by hypocalcemia, occurs at or shortly after parturition in cows and within weeks before or after parturition in ewes.'),
  'D010320': NtO(MH='Parvalbumins', MS='Low molecular weight, calcium binding muscle proteins. Their physiological function is possibly related to the contractile process.'),
  'D010321': NtO(MH='Parvoviridae', MS='A family of very small DNA viruses containing a single molecule of single-stranded DNA and consisting of two subfamilies: PARVOVIRINAE and DENSOVIRINAE. They infect both vertebrates and invertebrates.'),
  'D010322': NtO(MH='Parvoviridae Infections', MS='Virus infections caused by the PARVOVIRIDAE.'),
  'D010323': NtO(MH='Passive Cutaneous Anaphylaxis', MS='An evanescent cutaneous reaction occurring when antibody is injected into a local area on the skin and antigen is subsequently injected intravenously along with a dye. The dye makes the rapidly occurring capillary dilatation and increased vascular permeability readily visible by leakage into the reaction site. PCA is a sensitive reaction for detecting very small quantities of antibodies and is also a method for studying the mechanisms of immediate hypersensitivity.'),
  'D010324': NtO(MH='Passive-Aggressive Personality Disorder', MS='A personality disorder characterized by an indirect resistance to demands for adequate social and occupational performance; anger and opposition to authority and the expectations of others that is expressed covertly by obstructionism, procrastination, stubbornness, dawdling, forgetfulness, and intentional inefficiency. (Dorland, 27th ed)'),
  'D010325': NtO(MH='Pasteurella', MS='The oldest recognized genus of the family PASTEURELLACEAE. It consists of several species. Its organisms occur most frequently as coccobacillus or rod-shaped and are gram-negative, nonmotile, facultative anaerobes. Species of this genus are found in both animals and humans.'),
  'D010326': NtO(MH='Pasteurella Infections', MS='Infections with bacteria of the genus PASTEURELLA.'),
  'D010327': NtO(MH='Pastoral Care', MS='Counseling or comfort given by ministers, priests, rabbis, etc., to those in need of help with emotional problems or stressful situations.'),
  'D010328': NtO(MH='Patch Tests', MS='Skin tests in which the sensitizer is applied to a patch of cotton cloth or gauze held in place for approximately 48-72 hours. It is used for the elicitation of a contact hypersensitivity reaction.'),
  'D010329': NtO(MH='Patella', MS='The flat, triangular bone situated at the anterior part of the KNEE.'),
  'D010330': NtO(MH='Patents as Topic', MS='Works about exclusive legal rights or privileges applied to inventions, plants, etc.'),
  'D010331': NtO(MH='Paternal Age', MS='Age of the biological father.'),
  'D010332': NtO(MH='Paternal Behavior', MS='The behavior patterns associated with or characteristic of a father.'),
  'D010333': NtO(MH='Paternal Deprivation', MS='Prolonged separation of the offspring from the father.'),
  'D010334': NtO(MH='Paternity', MS='Establishing the father relationship of a man and a child.'),
  'D010335': NtO(MH='Pathologic Processes', MS='The abnormal mechanisms and forms involved in the dysfunctions of tissues and organs.'),
  'D010336': NtO(MH='Pathology', MS='A specialty concerned with the nature and cause of disease as expressed by changes in cellular or tissue structure and function caused by the disease process.'),
  'D010337': NtO(MH='Pathology Department, Hospital', MS='Hospital department which administers and provides pathology services.'),
  'D010338': NtO(MH='Pathology, Clinical', MS='A subspecialty of pathology applied to the solution of clinical problems, especially the use of laboratory methods in clinical diagnosis. (Dorland, 28th ed.)'),
  'D010339': NtO(MH='Pathology, Oral', MS='A dental specialty concerned with pathology of the oral cavity.'),
  'D010340': NtO(MH='Pathology, Surgical', MS='A field of anatomical pathology in which living tissue is surgically removed for the purpose of diagnosis and treatment.'),
  'D010341': NtO(MH='Pathology, Veterinary', MS='The field of veterinary medicine concerned with the causes of and changes produced in the body by disease.'),
  'D010342': NtO(MH='Patient Acceptance of Health Care', MS="Patients' willingness to receive health care."),
  'D010343': NtO(MH='Patient Admission', MS='The process of accepting patients. The concept includes patients accepted for medical and nursing care in a hospital or other health care institution.'),
  'D010344': NtO(MH='Patient Advocacy', MS='Promotion and protection of the rights of patients, frequently through a legal process.'),
  'D010346': NtO(MH='Patient Care Management', MS='Generating, planning, organizing, and administering medical and nursing care and services for patients.'),
  'D010347': NtO(MH='Patient Care Planning', MS='Usually a written medical and nursing care program designed for a particular patient.'),
  'D010348': NtO(MH='Patient Care Team', MS='Care of patients by a multidisciplinary team usually organized under the leadership of a physician; each member of the team has specific responsibilities and the whole team contributes to the care of the patient.'),
  'D010349': NtO(MH='Patient Compliance', MS='Voluntary cooperation of the patient in following a prescribed regimen.'),
  'D010350': NtO(MH='Patient Credit and Collection', MS='Accounting procedures for determining credit status and methods of obtaining payment.'),
  'D010351': NtO(MH='Patient Discharge', MS='The administrative process of discharging the patient, alive or dead, from hospitals or other health facilities.'),
  'D010352': NtO(MH='Patient Dropouts', MS='Discontinuance of care received by patient(s) due to reasons other than full recovery from the disease.'),
  'D010353': NtO(MH='Patient Education as Topic', MS='The teaching or training of patients concerning their own health needs.'),
  'D010354': NtO(MH='Patient Escort Service', MS='A special service provided by volunteers to accompany patients who need help in moving about the health facility.'),
  'D010355': NtO(MH='Patient Identification Systems', MS='Organized procedures for establishing patient identity, including use of bracelets, etc.'),
  'D010356': NtO(MH='Patient Isolation', MS='The segregation of patients with communicable or other diseases for a specified time. Isolation may be strict, in which movement and social contacts are limited; modified, where an effort to control specified aspects of care is made in order to prevent cross infection; or reverse, where the patient is secluded in a controlled or germ-free environment in order to protect him or her from cross infection.'),
  'D010357': NtO(MH='Patient Isolators', MS='Equipment used to prevent contamination of and by patients, especially those with bacterial infections. This includes plastic surgical isolators and isolators used to protect immunocompromised patients.'),
  'D010358': NtO(MH='Patient Participation', MS='Patient involvement in the decision-making process in matters pertaining to health.'),
  'D010359': NtO(MH='Patient Readmission', MS='Subsequent admissions of a patient to a hospital or other health care institution for treatment.'),
  'D010360': NtO(MH='Patient Transfer', MS='Interfacility or intrahospital transfer of patients. Intrahospital transfer is usually to obtain a specific kind of care and interfacility transfer is usually for economic reasons as well as for the type of care provided.'),
  'D010361': NtO(MH='Patients', MS='Individuals participating in the health care system for the purpose of receiving therapeutic, diagnostic, or preventive procedures.'),
  'D010362': NtO(MH="Patients' Rooms", MS='Rooms occupied by one or more individuals during a stay in a health facility. The concept includes aspects of environment, design, care, or economics.'),
  'D010363': NtO(MH='Pattern Recognition, Automated', MS="In INFORMATION RETRIEVAL, machine-sensing or identification of visible patterns (shapes, forms, and configurations). (Harrod's Librarians' Glossary, 7th ed)"),
  'D010364': NtO(MH='Pattern Recognition, Visual', MS='Mental process to visually perceive a critical number of facts (the pattern), such as characters, shapes, displays, or designs.'),
  'D010365': NtO(MH='Patulin', MS='4-Hydroxy-4H-furo(3,2-c)pyran-2(6H)-one. A mycotoxin produced by several species of Aspergillus and Penicillium. It is found in unfermented apple and grape juice and field crops. It has antibiotic properties and has been shown to be carcinogenic and mutagenic and causes chromosome damage in biological systems.'),
  'D010366': NtO(MH='Peak Expiratory Flow Rate', MS='Measurement of the maximum rate of airflow attained during a FORCED VITAL CAPACITY determination. Common abbreviations are PEFR and PFR.'),
  'D010367': NtO(MH='Arachis', MS='A plant genus of the family FABACEAE that includes peanuts.'),
  'D010368': NtO(MH='Pectins', MS='High molecular weight polysaccharides present in the cell walls of all plants. Pectins cement cell walls together. They are used as emulsifiers and stabilizers in the food industry. They have been tried for a variety of therapeutic uses including as antidiarrheals, where they are now generally considered ineffective, and in the treatment of hypercholesterolemia.'),
  'D010369': NtO(MH='Pectoralis Muscles', MS='The pectoralis major and pectoralis minor muscles that make up the upper and fore part of the chest in front of the AXILLA.'),
  'D010370': NtO(MH='Pediatric Assistants', MS='Persons academically trained to provide medical care, under the supervision of a physician, to infants and children.'),
  'D010371': NtO(MH='Pediatric Nursing', MS='The nursing specialty concerning care of children from birth to adolescence. It includes the clinical and psychological aspects of nursing care.'),
  'D010372': NtO(MH='Pediatrics', MS='A medical specialty concerned with maintaining health and providing medical care to children from birth to adolescence.'),
  'D010373': NtO(MH='Lice Infestations', MS='Parasitic attack or subsistence on the skin by members of the order Phthiraptera, especially on humans by Pediculus humanus of the family Pediculidae. The hair of the head, eyelashes, and pubis is a frequent site of infestation. (From Dorland, 28th ed; Stedman, 26th ed)'),
  'D010374': NtO(MH='Pediculus', MS='Lice of the genus Pediculus, family Pediculidae. Pediculus humanus corporus is the human body louse and Pediculus humanus capitis is the human head louse.'),
  'D010375': NtO(MH='Pedigree', MS='The record of descent or ancestry, particularly of a particular condition or trait, indicating individual family members, their relationships, and their status with respect to the trait or condition.'),
  'D010376': NtO(MH='Pediococcus', MS='A genus of gram-positive, facultatively anaerobic bacteria whose growth is dependent on the presence of a fermentable carbohydrate. No endospores are produced. Its organisms are found in fermenting plant products and are nonpathogenic to plants and animals, including humans.'),
  'D010377': NtO(MH='Pediatric Dentistry', MS='The practice of dentistry concerned with the dental problems of children, proper maintenance, and treatment. The dental care may include the services provided by dental specialists.'),
  'D010378': NtO(MH='Pedophilia', MS='A sexual disorder occurring in a person 16 years or older and that is recurrent with intense sexually arousing fantasies, sexual urges, or behaviors involving sexual activity with a prepubescent child (generally age 13 or younger). (from APA, DSM-IV, 1994).'),
  'D010379': NtO(MH='Peer Group', MS='Group composed of associates of same species, approximately the same age, and usually of similar rank or social status.'),
  'D010380': NtO(MH='Peer Review', MS='An organized procedure carried out by a select committee of professionals in evaluating the performance of other professionals in meeting the standards of their specialty. Review by peers is used by editors in the evaluation of articles and other papers submitted for publication. Peer review is used also in the evaluation of grant applications. It is applied also in evaluating the quality of health care provided to patients.'),
  'D010381': NtO(MH='Pelger-Huet Anomaly', MS='Autosomal dominant anomaly characterized by abnormal ovoid shape GRANULOCYTE nuclei and their clumping chromatin. Mutations in the LAMIN B receptor gene that results in reduced protein levels are associated with the disorder. Heterozygote individuals are healthy with normal granulocyte function while homozygote individuals occasionally have skeletal anomalies, developmental delay, and seizures.'),
  'D010382': NtO(MH='Peliosis Hepatis', MS='A vascular disease of the LIVER characterized by the occurrence of multiple blood-filled CYSTS or cavities. The cysts are lined with ENDOTHELIAL CELLS; the cavities lined with hepatic parenchymal cells (HEPATOCYTES). Peliosis hepatis has been associated with use of anabolic steroids (ANABOLIC AGENTS) and certain drugs.'),
  'D010383': NtO(MH='Pellagra', MS="A disease due to deficiency of NIACIN, a B-complex vitamin, or its precursor TRYPTOPHAN. It is characterized by scaly DERMATITIS which is often associated with DIARRHEA and DEMENTIA (the three D's)."),
  'D010384': NtO(MH='Pelvic Bones', MS='Bones that constitute each half of the pelvic girdle in VERTEBRATES, formed by fusion of the ILIUM; ISCHIUM; and PUBIC BONE.'),
  'D010385': NtO(MH='Pelvic Exenteration', MS='Removal of all of the organs and adjacent structures of the pelvis. It is usually performed to surgically remove cancer involving the bladder, uterine cervix, or rectum. (Stedman, 25th ed)'),
  'D010386': NtO(MH='Pelvic Neoplasms', MS='Tumors or cancer of the pelvic region.'),
  'D010387': NtO(MH='Pelvimetry', MS='Measurement of the dimensions and capacity of the pelvis. It includes cephalopelvimetry (measurement of fetal head size in relation to maternal pelvic capacity), a prognostic guide to the management of LABOR, OBSTETRIC associated with disproportion.'),
  'D010388': NtO(MH='Pelvis', MS='The space or compartment surrounded by the pelvic girdle (bony pelvis). It is subdivided into the greater pelvis and LESSER PELVIS. The pelvic girdle is formed by the PELVIC BONES and SACRUM.'),
  'D010389': NtO(MH='Pemoline', MS='A central nervous system stimulant used in fatigue and depressive states and to treat hyperkinetic disorders in children.'),
  'D010390': NtO(MH='Pemphigoid, Benign Mucous Membrane', MS='A chronic blistering disease with predilection for mucous membranes and less frequently the skin, and with a tendency to scarring. It is sometimes called ocular pemphigoid because of conjunctival mucous membrane involvement.'),
  'D010391': NtO(MH='Pemphigoid, Bullous', MS='A chronic and relatively benign subepidermal blistering disease usually of the elderly and without histopathologic acantholysis.'),
  'D010392': NtO(MH='Pemphigus', MS='Group of chronic blistering diseases characterized histologically by ACANTHOLYSIS and blister formation within the EPIDERMIS.'),
  'D010393': NtO(MH='Pempidine', MS='A nicotinic antagonist most commonly used as an experimental tool. It has been used as a ganglionic blocker in the treatment of hypertension but has largely been supplanted for that purpose by more specific drugs.'),
  'D010394': NtO(MH='Penbutolol', MS='A nonselective beta-blocker used as an antihypertensive and an antianginal agent.'),
  'D010395': NtO(MH='Penfluridol', MS='One of the long-acting ANTIPSYCHOTIC AGENTS used for maintenance or long-term therapy of SCHIZOPHRENIA and other PSYCHOTIC DISORDERS.'),
  'D010396': NtO(MH='Penicillamine', MS="3-Mercapto-D-valine. The most characteristic degradation product of the penicillin antibiotics. It is used as an antirheumatic and as a chelating agent in Wilson's disease."),
  'D010397': NtO(MH='Penicillanic Acid', MS='A building block of penicillin, devoid of significant antibacterial activity. (From Merck Index, 11th ed)'),
  'D010398': NtO(MH='Penicillic Acid', MS='A mycotoxin with antibiotic and carcinogenic activity produced by various strains of PENICILLIUM and ASPERGILLUS. It has been found in tobacco, sausages, and corn.'),
  'D010399': NtO(MH='Penicillin Amidase', MS='An enzyme catalyzing the hydrolysis of penicillin to penicin and a carboxylic acid anion. EC 3.5.1.11.'),
  'D010400': NtO(MH='Penicillin G', MS='A penicillin derivative commonly used in the form of its sodium or potassium salts in the treatment of a variety of infections. It is effective against most gram-positive bacteria and against gram-negative cocci. It has also been used as an experimental convulsant because of its actions on GAMMA-AMINOBUTYRIC ACID mediated synaptic transmission.'),
  'D010401': NtO(MH='Penicillin G Benzathine', MS="Semisynthetic antibiotic prepared by combining the sodium salt of penicillin G with N,N'-dibenzylethylenediamine."),
  'D010402': NtO(MH='Penicillin G Procaine', MS='Semisynthetic antibiotic prepared by combining penicillin G with PROCAINE.'),
  'D010403': NtO(MH='Penicillin Resistance', MS='Nonsusceptibility of an organism to the action of penicillins.'),
  'D010404': NtO(MH='Penicillin V', MS='A broad-spectrum penicillin antibiotic used orally in the treatment of mild to moderate infections by susceptible gram-positive organisms.'),
  'D010405': NtO(MH='Penicillinase', MS='A beta-lactamase preferentially cleaving penicillins. (Dorland, 28th ed) EC 3.5.2.-.'),
  'D010406': NtO(MH='Penicillins', MS="A group of antibiotics that contain 6-aminopenicillanic acid with a side chain attached to the 6-amino group. The penicillin nucleus is the chief structural requirement for biological activity. The side-chain structure determines many of the antibacterial and pharmacological characteristics. (Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed, p1065)"),
  'D010407': NtO(MH='Penicillium', MS='A mitosporic Trichocomaceae fungal genus that develops fruiting organs resembling a broom. When identified, teleomorphs include EUPENICILLIUM and TALAROMYCES. Several species (but especially PENICILLIUM CHRYSOGENUM) are sources of the antibiotic penicillin.'),
  'D010408': NtO(MH='Penicillium chrysogenum', MS='A mitosporic fungal species used in the production of penicillin.'),
  'D010409': NtO(MH='Penile Diseases', MS='Pathological processes involving the PENIS or its component tissues.'),
  'D010410': NtO(MH='Penile Erection', MS='The state of the PENIS when the erectile tissue becomes filled or swollen (tumid) with BLOOD and causes the penis to become rigid and elevated. It is a complex process involving CENTRAL NERVOUS SYSTEM; PERIPHERAL NERVOUS SYSTEMS; HORMONES; SMOOTH MUSCLES; and vascular functions.'),
  'D010411': NtO(MH='Penile Induration', MS='A condition characterized by hardening of the PENIS due to the formation of fibrous plaques on the dorsolateral aspect of the PENIS, usually involving the membrane (tunica albuginea) surrounding the erectile tissue (corpus cavernosum penis). This may eventually cause a painful deformity of the shaft or constriction of the urethra, or both.'),
  'D010412': NtO(MH='Penile Neoplasms', MS='Cancers or tumors of the PENIS or of its component tissues.'),
  'D010413': NtO(MH='Penis', MS='The external reproductive organ of males. It is composed of a mass of erectile tissue enclosed in three cylindrical fibrous compartments. Two of the three compartments, the corpus cavernosa, are placed side-by-side along the upper part of the organ. The third compartment below, the corpus spongiosum, houses the urethra.'),
  'D010414': NtO(MH='Pennsylvania', MS='State bounded on the north by New York and Lake Erie, on the east by Delaware and New Jersey, on the south by Delaware and Maryland, and on the west by Ohio and West Virginia.'),
  'D010415': NtO(MH='Pensions', MS='Fixed sums paid regularly to individuals.'),
  'D010416': NtO(MH='Pentachlorophenol', MS='An insecticide and herbicide that has also been used as a wood preservative. Pentachlorphenol is a widespread environmental pollutant. Both chronic and acute pentachlorophenol poisoning are medical concerns. The range of its biological actions is still being actively explored, but it is clearly a potent enzyme inhibitor and has been used as such as an experimental tool.'),
  'D010417': NtO(MH='Pentaerythritol Tetranitrate', MS='A vasodilator with general properties similar to NITROGLYCERIN but with a more prolonged duration of action. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1025)'),
  'D010418': NtO(MH='Pentagastrin', MS='A synthetic pentapeptide that has effects like gastrin when given parenterally. It stimulates the secretion of gastric acid, pepsin, and intrinsic factor, and has been used as a diagnostic aid.'),
  'D010419': NtO(MH='Pentamidine', MS='Antiprotozoal agent effective in trypanosomiasis, leishmaniasis, and some fungal infections; used in treatment of PNEUMOCYSTIS pneumonia in HIV-infected patients. It may cause diabetes mellitus, central nervous system damage, and other toxic effects.'),
  'D010420': NtO(MH='Pentanes', MS='Five-carbon saturated hydrocarbon group of the methane series. Include isomers and derivatives.'),
  'D010421': NtO(MH='Pentanoic Acids', MS='Straight-chain CARBOXYLIC ACIDS with the general formula C5H10O2.'),
  'D010422': NtO(MH='Pentanones', MS='5-carbon straight-chain or branched-chain ketones.'),
  'D010423': NtO(MH='Pentazocine', MS='The first mixed agonist-antagonist analgesic to be marketed. It is an agonist at the kappa and sigma opioid receptors and has a weak antagonist action at the mu receptor. (From AMA Drug Evaluations Annual, 1991, p97)'),
  'D010424': NtO(MH='Pentobarbital', MS='A short-acting barbiturate that is effective as a sedative and hypnotic (but not as an anti-anxiety) agent and is usually given orally. It is prescribed more frequently for sleep induction than for sedation but, like similar agents, may lose its effectiveness by the second week of continued administration. (From AMA Drug Evaluations Annual, 1994, p236)'),
  'D010425': NtO(MH='Pentolinium Tartrate', MS='A nicotinic antagonist that has been used as a ganglionic blocking agent in hypertension.'),
  'D010426': NtO(MH='Pentosan Sulfuric Polyester', MS='A sulfated pentosyl polysaccharide with heparin-like properties.'),
  'D010427': NtO(MH='Pentose Phosphate Pathway', MS='An oxidative decarboxylation process that converts GLUCOSE-6-PHOSPHATE to D-ribose-5-phosphate via 6-phosphogluconate. The pentose product is used in the biosynthesis of NUCLEIC ACIDS. The generated energy is stored in the form of NADPH. This pathway is prominent in tissues which are active in the synthesis of FATTY ACIDS and STEROIDS.'),
  'D010428': NtO(MH='Pentosephosphates', MS=None),
  'D010429': NtO(MH='Pentoses', MS='Monosaccharide sugar molecules that contain a five carbon backbone.'),
  'D010430': NtO(MH='Pentosyltransferases', MS='Enzymes of the transferase class that catalyze the transfer of a pentose group from one compound to another.'),
  'D010431': NtO(MH='Pentoxifylline', MS='A METHYLXANTHINE derivative that inhibits phosphodiesterase and affects blood rheology. It improves blood flow by increasing erythrocyte and leukocyte flexibility. It also inhibits platelet aggregation. Pentoxifylline modulates immunologic activity by stimulating cytokine production.'),
  'D010432': NtO(MH='Pentoxyl', MS='5-Hydroxymethyl-6-methyl- 2,4-(1H,3H)-pyrimidinedione. Uracil derivative used in combination with toxic antibiotics to lessen their toxicity; also to stimulate leukopoiesis and immunity. Synonyms: pentoksil; hydroxymethylmethyluracil.'),
  'D010433': NtO(MH='Pentylenetetrazole', MS='A pharmaceutical agent that displays activity as a central nervous system and respiratory stimulant. It is considered a non-competitive GAMMA-AMINOBUTYRIC ACID antagonist. Pentylenetetrazole has been used experimentally to study seizure phenomenon and to identify pharmaceuticals that may control seizure susceptibility.'),
  'D010434': NtO(MH='Pepsin A', MS='Formed from pig pepsinogen by cleavage of one peptide bond. The enzyme is a single polypeptide chain and is inhibited by methyl 2-diaazoacetamidohexanoate. It cleaves peptides preferentially at the carbonyl linkages of phenylalanine or leucine and acts as the principal digestive enzyme of gastric juice.'),
  'D010435': NtO(MH='Pepsinogens', MS='Proenzymes secreted by chief cells, mucous neck cells, and pyloric gland cells, which are converted into pepsin in the presence of gastric acid or pepsin itself. (Dorland, 28th ed) In humans there are 2 related pepsinogen systems: PEPSINOGEN A (formerly pepsinogen I or pepsinogen) and PEPSINOGEN C (formerly pepsinogen II or progastricsin). Pepsinogen B is the name of a pepsinogen from pigs.'),
  'D010436': NtO(MH='Pepstatins', MS='N-acylated oligopeptides isolated from culture filtrates of Actinomycetes, which act specifically to inhibit acid proteases such as pepsin and renin.'),
  'D010437': NtO(MH='Peptic Ulcer', MS='Ulcer that occurs in the regions of the GASTROINTESTINAL TRACT which come into contact with GASTRIC JUICE containing PEPSIN and GASTRIC ACID. It occurs when there are defects in the MUCOSA barrier. The common forms of peptic ulcers are associated with HELICOBACTER PYLORI and the consumption of nonsteroidal anti-inflammatory drugs (NSAIDS).'),
  'D010438': NtO(MH='Peptic Ulcer Hemorrhage', MS='Bleeding from a PEPTIC ULCER that can be located in any segment of the GASTROINTESTINAL TRACT.'),
  'D010439': NtO(MH='Peptic Ulcer Perforation', MS='Penetration of a PEPTIC ULCER through the wall of DUODENUM or STOMACH allowing the leakage of luminal contents into the PERITONEAL CAVITY.'),
  'D010440': NtO(MH='Peptichemio', MS='A mixture of six synthetic oligopeptides, each containing MELPHALAN. It is used as a broad-spectrum antineoplastic due to its alkylating and antimetabolic actions but, is toxic to bone marrow, gastrointestinal system and vasculature.'),
  'D010441': NtO(MH='Peptide Chain Elongation, Translational', MS='A process of GENETIC TRANSLATION, when an amino acid is transferred from its cognate TRANSFER RNA to the lengthening chain of PEPTIDES.'),
  'D010442': NtO(MH='Peptide Chain Initiation, Translational', MS='A process of GENETIC TRANSLATION whereby the formation of a peptide chain is started. It includes assembly of the RIBOSOME components, the MESSENGER RNA coding for the polypeptide to be made, INITIATOR TRNA, and PEPTIDE INITIATION FACTORS; and placement of the first amino acid in the peptide chain. The details and components of this process are unique for prokaryotic protein biosynthesis and eukaryotic protein biosynthesis.'),
  'D010443': NtO(MH='Peptide Chain Termination, Translational', MS='A process of GENETIC TRANSLATION whereby the terminal amino acid is added to a lengthening polypeptide. This termination process is signaled from the MESSENGER RNA, by one of three termination codons (CODON, TERMINATOR) that immediately follows the last amino acid-specifying CODON.'),
  'D010444': NtO(MH='Peptide Elongation Factor Tu', MS="A protein found in bacteria and eukaryotic mitochondria which delivers aminoacyl-tRNA's to the A site of the ribosome. The aminoacyl-tRNA is first bound to a complex of elongation factor Tu containing a molecule of bound GTP. The resulting complex is then bound to the 70S initiation complex. Simultaneously the GTP is hydrolyzed and a Tu-GDP complex is released from the 70S ribosome. The Tu-GTP complex is regenerated from the Tu-GDP complex by the Ts elongation factor and GTP."),
  'D010445': NtO(MH='Peptide Elongation Factors', MS='Protein factors uniquely required during the elongation phase of protein synthesis.'),
  'D010446': NtO(MH='Peptide Fragments', MS='Partial proteins formed by partial hydrolysis of complete proteins or generated through PROTEIN ENGINEERING techniques.'),
  'D010447': NtO(MH='Peptide Hydrolases', MS='Hydrolases that specifically cleave the peptide bonds found in PROTEINS and PEPTIDES. Examples of sub-subclasses for this group include EXOPEPTIDASES and ENDOPEPTIDASES.'),
  'D010448': NtO(MH='Peptide Initiation Factors', MS='Protein factors uniquely required during the initiation phase of protein synthesis in GENETIC TRANSLATION.'),
  'D010449': NtO(MH='Peptide Mapping', MS='Analysis of PEPTIDES that are generated from the digestion or fragmentation of a protein or mixture of PROTEINS, by ELECTROPHORESIS; CHROMATOGRAPHY; or MASS SPECTROMETRY. The resulting peptide fingerprints are analyzed for a variety of purposes including the identification of the proteins in a sample, GENETIC POLYMORPHISMS, patterns of gene expression, and patterns diagnostic for diseases.'),
  'D010450': NtO(MH='Endopeptidases', MS='A subclass of PEPTIDE HYDROLASES that catalyze the internal cleavage of PEPTIDES or PROTEINS.'),
  'D010451': NtO(MH='Peptide PHI', MS='A 27-amino acid peptide with histidine at the N-terminal and isoleucine amide at the C-terminal. The exact amino acid composition of the peptide is species dependent. The peptide is secreted in the intestine, but is found in the nervous system, many organs, and in the majority of peripheral tissues. It has a wide range of biological actions, affecting the cardiovascular, gastrointestinal, respiratory, and central nervous systems.'),
  'D010452': NtO(MH='Peptide Biosynthesis', MS='The production of PEPTIDES or PROTEINS by the constituents of a living organism. The biosynthesis of proteins on RIBOSOMES following an RNA template is termed translation (TRANSLATION, GENETIC). There are other, non-ribosomal peptide biosynthesis (PEPTIDE BIOSYNTHESIS, NUCLEIC ACID-INDEPENDENT) mechanisms carried out by PEPTIDE SYNTHASES and PEPTIDYLTRANSFERASES. Further modifications of peptide chains yield functional peptide and protein molecules.'),
  'D010453': NtO(MH='Peptide Synthases', MS='Ligases that catalyze the joining of adjacent AMINO ACIDS by the formation of carbon-nitrogen bonds between their carboxylic acid groups and amine groups.'),
  'D010454': NtO(MH='Peptide Termination Factors', MS='Proteins that are involved in the peptide chain termination reaction (PEPTIDE CHAIN TERMINATION, TRANSLATIONAL) on RIBOSOMES. They include codon-specific class-I release factors, which recognize stop signals (TERMINATOR CODON) in the MESSENGER RNA; and codon-nonspecific class-II release factors.'),
  'D010455': NtO(MH='Peptides', MS='Members of the class of compounds composed of AMINO ACIDS joined together by peptide bonds between adjacent amino acids into linear, branched or cyclical structures. OLIGOPEPTIDES are composed of approximately 2-12 amino acids. Polypeptides are composed of approximately 13 or more amino acids. PROTEINS are linear polypeptides that are normally synthesized on RIBOSOMES.'),
  'D010456': NtO(MH='Peptides, Cyclic', MS='Peptides whose amino acid residues are linked together forming a circular chain. Some of them are ANTI-INFECTIVE AGENTS; some are biosynthesized non-ribosomally (PEPTIDE BIOSYNTHESIS, NON-RIBOSOMAL).'),
  'D010457': NtO(MH='Peptidoglycan', MS='A structural polymer of the bacterial cell envelope consisting of sugars and amino acids which is responsible for both shape determination and cellular integrity under osmotic stress in virtually all bacteria.'),
  'D010458': NtO(MH='Peptidyl Transferases', MS='Acyltransferases that use AMINO ACYL TRNA as the amino acid donor in formation of a peptide bond. There are ribosomal and non-ribosomal peptidyltransferases.'),
  'D010459': NtO(MH='Peptococcaceae', MS='A family of bacteria found in the mouth and intestinal and respiratory tracts of man and other animals as well as in the human female urogenital tract. Its organisms are also found in soil and on cereal grains.'),
  'D010460': NtO(MH='Peptococcus', MS='A genus of gram-positive, anaerobic, coccoid bacteria that is part of the normal flora of the mouth, upper respiratory tract, and large intestine in humans. Its organisms cause infections of soft tissues and bacteremias.'),
  'D010461': NtO(MH='Peptones', MS='Derived proteins or mixtures of cleavage products produced by the partial hydrolysis of a native protein either by an acid or by an enzyme. Peptones are readily soluble in water, and are not precipitable by heat, by alkalis, or by saturation with ammonium sulfate. (Dorland, 28th ed)'),
  'D010462': NtO(MH='Peptostreptococcus', MS='A genus of gram-positive, anaerobic, coccoid bacteria that is part of the normal flora of humans. Its organisms are opportunistic pathogens causing bacteremias and soft tissue infections.'),
  'D010463': NtO(MH='Peracetic Acid', MS='A liquid that functions as a strong oxidizing agent. It has an acrid odor and is used as a disinfectant.'),
  'D010464': NtO(MH='Perazine', MS='A phenothiazine antipsychotic with actions and uses similar to those of CHLORPROMAZINE. Extrapyramidal symptoms may be more common than other side effects.'),
  'D010465': NtO(MH='Perception', MS='The process by which the nature and meaning of sensory stimuli are recognized and interpreted.'),
  'D010466': NtO(MH='Perceptual Closure', MS='The tendency to perceive an incomplete pattern or object as complete or whole. This includes the Gestalt Law of Closure.'),
  'D010467': NtO(MH='Perceptual Defense', MS='Selective perceiving such that the individual protects himself from becoming aware of something unpleasant or threatening, e.g., obscene words are not heard correctly, or violent acts are not seen accurately.'),
  'D010468': NtO(MH='Perceptual Disorders', MS='Cognitive disorders characterized by an impaired ability to perceive the nature of objects or concepts through use of the sense organs. These include spatial neglect syndromes, where an individual does not attend to visual, auditory, or sensory stimuli presented from one side of the body.'),
  'D010469': NtO(MH='Perceptual Distortion', MS='Lack of correspondence between the way a stimulus is commonly perceived and the way an individual perceives it under given conditions.'),
  'D010470': NtO(MH='Perceptual Masking', MS='The interference of one perceptual stimulus with another causing a decrease or lessening in perceptual effectiveness.'),
  'D010471': NtO(MH='Perches', MS='A common name for fish of the family Percidae, belonging to the suborder Percoidei, order PERCIFORMES.'),
  'D010472': NtO(MH='Perchlorates', MS='Compounds that contain the Cl(=O)(=O)(=O)O- structure. Included under this heading is perchloric acid and the salts and ester forms of perchlorate.'),
  'D010473': NtO(MH='Perciformes', MS='The most diversified of all fish orders and the largest vertebrate order. It includes many of the commonly known fish such as porgies, croakers, sunfishes, dolphin fish, mackerels, TUNA, etc.'),
  'D010474': NtO(MH='Percussion', MS='Act of striking a part with short, sharp blows as an aid in diagnosing the condition beneath the sound obtained.'),
  'D010476': NtO(MH='Perfume', MS='A substance, extract, or preparation for diffusing or imparting an agreeable or attractive smell, especially a fluid containing fragrant natural oils extracted from flowers, woods, etc., or similar synthetic oils. (Random House Unabridged Dictionary, 2d ed)'),
  'D010477': NtO(MH='Perfusion', MS='Treatment process involving the injection of fluid into an organ or tissue.'),
  'D010478': NtO(MH='Chemotherapy, Cancer, Regional Perfusion', MS='Neoplasm drug therapy involving an extracorporeal circuit with temporary exclusion of the tumor-bearing area from the general circulation during which high concentrations of the drug are perfused to the isolated part.'),
  'D010479': NtO(MH='Pergolide', MS='A long-acting dopamine agonist which has been used to treat PARKINSON DISEASE and HYPERPROLACTINEMIA but withdrawn from some markets due to potential for HEART VALVE DISEASES.'),
  'D010480': NtO(MH='Perhexiline', MS='2-(2,2-Dicyclohexylethyl)piperidine. Coronary vasodilator used especially for angina of effort. It may cause neuropathy and hepatitis.'),
  'D010481': NtO(MH='Perianal Glands', MS='A type of SEBACEOUS GLAND located in the area surrounding the ANUS of some mammals such as dogs, cats, opossums, and guinea pigs.'),
  'D010482': NtO(MH='Periapical Abscess', MS='Acute or chronic inflammation of tissues surrounding the apical portion of a tooth, associated with the collection of pus, resulting from infection following pulp infection through a carious lesion or as a result of an injury causing pulp necrosis. (Dorland, 27th ed)'),
  'D010483': NtO(MH='Periapical Diseases', MS='Diseases of the PERIAPICAL TISSUE surrounding the root of the tooth, which is distinguished from DENTAL PULP DISEASES inside the TOOTH ROOT.'),
  'D010484': NtO(MH='Periapical Granuloma', MS='Chronic nonsuppurative inflammation of periapical tissue resulting from irritation following pulp disease or endodontic treatment.'),
  'D010485': NtO(MH='Periapical Periodontitis', MS='Inflammation of the PERIAPICAL TISSUE. It includes general, unspecified, or acute nonsuppurative inflammation. Chronic nonsuppurative inflammation is PERIAPICAL GRANULOMA. Suppurative inflammation is PERIAPICAL ABSCESS.'),
  'D010486': NtO(MH='Periapical Tissue', MS='Tissue surrounding the apex of a tooth, including the apical portion of the periodontal membrane and alveolar bone.'),
  'D010487': NtO(MH='Periaqueductal Gray', MS='Central gray matter surrounding the CEREBRAL AQUEDUCT in the MESENCEPHALON. Physiologically it is probably involved in RAGE reactions, the LORDOSIS REFLEX; FEEDING responses, bladder tonus, and pain.'),
  'D010488': NtO(MH='Polyarteritis Nodosa', MS='A form of necrotizing non-granulomatous inflammation occurring primarily in medium-sized ARTERIES, often with microaneurysms. It is characterized by muscle, joint, and abdominal pain resulting from arterial infarction and scarring in affected organs. Polyarteritis nodosa with lung involvement is called CHURG-STRAUSS SYNDROME.'),
  'D010489': NtO(MH='Periarthritis', MS='Inflammation of the tissues around a joint. (Dorland, 27th ed)'),
  'D010490': NtO(MH='Pericardial Effusion', MS='Fluid accumulation within the PERICARDIUM. Serous effusions are associated with pericardial diseases. Hemopericardium is associated with trauma. Lipid-containing effusion (chylopericardium) results from leakage of THORACIC DUCT. Severe cases can lead to CARDIAC TAMPONADE.'),
  'D010491': NtO(MH='Pericardial Window Techniques', MS='Surgical construction of an opening or window in the pericardium. It is often called subxiphoid pericardial window technique.'),
  'D010492': NtO(MH='Pericardiectomy', MS='Surgical excision (total or partial) of a portion of the pericardium. Pericardiotomy refers to incision of the pericardium.'),
  'D010493': NtO(MH='Pericarditis', MS='Inflammation of the PERICARDIUM from various origins, such as infection, neoplasm, autoimmune process, injuries, or drug-induced. Pericarditis usually leads to PERICARDIAL EFFUSION, or CONSTRICTIVE PERICARDITIS.'),
  'D010494': NtO(MH='Pericarditis, Constrictive', MS='Inflammation of the PERICARDIUM that is characterized by the fibrous scarring and adhesion of both serous layers, the VISCERAL PERICARDIUM and the PARIETAL PERICARDIUM leading to the loss of pericardial cavity. The thickened pericardium severely restricts cardiac filling. Clinical signs include FATIGUE, muscle wasting, and WEIGHT LOSS.'),
  'D010495': NtO(MH='Pericarditis, Tuberculous', MS='INFLAMMATION of the sac surrounding the heart (PERICARDIUM) due to MYCOBACTERIUM TUBERCULOSIS infection. Pericarditis can lead to swelling (PERICARDIAL EFFUSION), compression of the heart (CARDIAC TAMPONADE), and preventing normal beating of the heart.'),
  'D010496': NtO(MH='Pericardium', MS='A conical fibro-serous sac surrounding the HEART and the roots of the great vessels (AORTA; VENAE CAVAE; PULMONARY ARTERY). Pericardium consists of two sacs: the outer fibrous pericardium and the inner serous pericardium. The latter consists of an outer parietal layer facing the fibrous pericardium, and an inner visceral layer (epicardium) resting next to the heart, and a pericardial cavity between these two layers.'),
  'D010497': NtO(MH='Pericoronitis', MS='Inflammation of the gingiva surrounding the crown of a tooth.'),
  'D010498': NtO(MH='Perilymph', MS='The fluid separating the membranous labyrinth from the osseous labyrinth of the ear. It is entirely separate from the ENDOLYMPH which is contained in the membranous labyrinth. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed, p1396, 642)'),
  'D010500': NtO(MH='Perinatology', MS='The branch of medicine dealing with the fetus and infant during the perinatal period. The perinatal period begins with the twenty-eighth week of gestation and ends twenty-eight days after birth. (From Dorland, 27th ed)'),
  'D010501': NtO(MH='Perinephritis', MS='Inflammation of the connective and adipose tissues surrounding the KIDNEY.'),
  'D010502': NtO(MH='Perineum', MS='The body region lying between the genital area and the ANUS on the surface of the trunk, and to the shallow compartment lying deep to this area that is inferior to the PELVIC DIAPHRAGM. The surface area is between the VULVA and the anus in the female, and between the SCROTUM and the anus in the male.'),
  'D010503': NtO(MH='Periodic Acid-Schiff Reaction', MS='A histochemical technique for staining carbohydrates. It is based on PERIODIC ACID oxidation of a substance containing adjacent hydroxyl groups. The resulting aldehydes react with Schiff reagent to form a colored product.'),
  'D010504': NtO(MH='Periodic Acid', MS='A strong oxidizing agent.'),
  'D010505': NtO(MH='Familial Mediterranean Fever', MS='A group of HEREDITARY AUTOINFLAMMATION DISEASES, characterized by recurrent fever, abdominal pain, headache, rash, PLEURISY; and ARTHRITIS. ORCHITIS; benign MENINGITIS; and AMYLOIDOSIS may also occur. Homozygous or compound heterozygous mutations in marenostrin gene result in autosomal recessive transmission; simple heterozygous, autosomal dominant form of the disease.'),
  'D010506': NtO(MH='Periodicals as Topic', MS='Works about publications issued at stated, more or less regular, intervals.'),
  'D010507': NtO(MH='Periodicity', MS='The tendency of a phenomenon to recur at regular intervals; in biological systems, the recurrence of certain activities (including hormonal, cellular, neural) may be annual, seasonal, monthly, daily, or more frequently (ultradian).'),
  'D010508': NtO(MH='Periodontal Abscess', MS='Localized circumscribed purulent area of inflammation in the periodontal tissue. It is a derivative of marginal periodontitis and commonly associated with suprabony and infrabony pockets and interradicular involvements, in contrast to periapical abscess which is attributable to pulp necrosis.'),
  'D010509': NtO(MH='Periodontal Cyst', MS='An epithelium-lined sac containing fluid; usually found at the apex of a pulp-involved tooth. The lateral type occurs less frequently along the side of the root.'),
  'D010510': NtO(MH='Periodontal Diseases', MS='Pathological processes involving the PERIODONTIUM including the gum (GINGIVA), the alveolar bone (ALVEOLAR PROCESS), the DENTAL CEMENTUM, and the PERIODONTAL LIGAMENT.'),
  'D010511': NtO(MH='Periodontal Dressings', MS="Surgical dressings, after periodontal surgery, applied to the necks of teeth and the adjacent tissue to cover and protect the surgical wound. (Boucher's Clinical Dental Terminology, 4th ed, p218)"),
  'D010512': NtO(MH='Periodontal Index', MS='A numerical rating scale for classifying the periodontal status of a person or population with a single figure which takes into consideration prevalence as well as severity of the condition. It is based upon probe measurement of periodontal pockets and on gingival tissue status.'),
  'D010513': NtO(MH='Periodontal Ligament', MS='The fibrous CONNECTIVE TISSUE surrounding the TOOTH ROOT, separating it from and attaching it to the alveolar bone (ALVEOLAR PROCESS).'),
  'D010514': NtO(MH='Periodontal Pocket', MS='An abnormal extension of a gingival sulcus accompanied by the apical migration of the epithelial attachment and bone resorption.'),
  'D010515': NtO(MH='Periodontal Prosthesis', MS="Any restorative and replacement device that is used as a therapeutic aid in the treatment of periodontal disease. It is an adjunct to other forms of periodontal therapy and does not cure periodontal disease by itself. (Boucher's Clinical Dental Terminology, 3d ed)"),
  'D010516': NtO(MH='Periodontal Splints', MS='Fixed or removable devices that join teeth together. They are used to repair teeth that are mobile as a result of PERIODONTITIS.'),
  'D010517': NtO(MH='Periodontics', MS='A dental specialty concerned with the histology, physiology, and pathology of the tissues that support, attach, and surround the teeth.'),
  'D010518': NtO(MH='Periodontitis', MS='Inflammation and loss of connective tissues supporting or surrounding the teeth. This may involve any part of the PERIODONTIUM. Periodontitis is currently classified by disease progression (CHRONIC PERIODONTITIS; AGGRESSIVE PERIODONTITIS) instead of age of onset. (From 1999 International Workshop for a Classification of Periodontal Diseases and Conditions, American Academy of Periodontology)'),
  'D010519': NtO(MH='Periodontium', MS='The structures surrounding and supporting the tooth. Periodontium includes the gum (GINGIVA), the alveolar bone (ALVEOLAR PROCESS), the DENTAL CEMENTUM, and the PERIODONTAL LIGAMENT.'),
  'D010520': NtO(MH='Aggressive Periodontitis', MS='Inflammation and loss of PERIODONTIUM that is characterized by rapid attachment loss and bone destruction in the presence of little local factors such as DENTAL PLAQUE and DENTAL CALCULUS. This highly destructive form of periodontitis often occurs in young people and was called early-onset periodontitis, but this disease also appears in old people.'),
  'D010521': NtO(MH='Periosteum', MS='Thin outer membrane that surrounds a bone. It contains CONNECTIVE TISSUE, CAPILLARIES, nerves, and a number of cell types.'),
  'D010522': NtO(MH='Periostitis', MS='Inflammation of the periosteum. The condition is generally chronic, and is marked by tenderness and swelling of the bone and an aching pain. Acute periostitis is due to infection, is characterized by diffuse suppuration, severe pain, and constitutional symptoms, and usually results in necrosis. (Dorland, 27th ed)'),
  'D010523': NtO(MH='Peripheral Nervous System Diseases', MS='Diseases of the peripheral nerves external to the brain and spinal cord, which includes diseases of the nerve roots, ganglia, plexi, autonomic nerves, sensory nerves, and motor nerves.'),
  'D010524': NtO(MH='Peripheral Nervous System Neoplasms', MS='Neoplasms which arise from peripheral nerve tissue. This includes NEUROFIBROMAS; SCHWANNOMAS; GRANULAR CELL TUMORS; and malignant peripheral NERVE SHEATH NEOPLASMS. (From DeVita Jr et al., Cancer: Principles and Practice of Oncology, 5th ed, pp1750-1)'),
  'D010525': NtO(MH='Peripheral Nerves', MS='The nerves outside of the brain and spinal cord, including the autonomic, cranial, and spinal nerves. Peripheral nerves contain non-neuronal cells and connective tissue as well as axons. The connective tissue layers include, from the outside to the inside, the epineurium, the perineurium, and the endoneurium.'),
  'D010526': NtO(MH='Periplaneta', MS='A genus in the family Blattidae containing several species, the most common being P. americana, the American cockroach.'),
  'D010527': NtO(MH='Perissodactyla', MS='An order of ungulates having an odd number of toes, including the horse, tapir, and rhinoceros. (Dorland, 27th ed)'),
  'D010528': NtO(MH='Peristalsis', MS='A movement, caused by sequential muscle contraction, that pushes the contents of the intestines or other tubular organs in one direction.'),
  'D010529': NtO(MH='Peritoneal Cavity', MS='The space enclosed by the peritoneum. It is divided into two portions, the greater sac and the lesser sac or omental bursa, which lies behind the STOMACH. The two sacs are connected by the foramen of Winslow, or epiploic foramen.'),
  'D010530': NtO(MH='Peritoneal Dialysis', MS='Dialysis fluid being introduced into and removed from the peritoneal cavity as either a continuous or an intermittent procedure.'),
  'D010531': NtO(MH='Peritoneal Dialysis, Continuous Ambulatory', MS='Portable peritoneal dialysis using the continuous (24 hours a day, 7 days a week) presence of peritoneal dialysis solution in the peritoneal cavity except for periods of drainage and instillation of fresh solution.'),
  'D010532': NtO(MH='Peritoneal Diseases', MS='Pathological processes involving the PERITONEUM.'),
  'D010533': NtO(MH='Peritoneal Lavage', MS='Washing out of the peritoneal cavity. The procedure is a diagnostic as well as a therapeutic technique following abdominal trauma or inflammation.'),
  'D010534': NtO(MH='Peritoneal Neoplasms', MS='Tumors or cancer of the PERITONEUM.'),
  'D010535': NtO(MH='Laparoscopy', MS='A procedure in which a laparoscope (LAPAROSCOPES) is inserted through a small incision near the navel to examine the abdominal and pelvic organs in the PERITONEAL CAVITY. If appropriate, biopsy or surgery can be performed during laparoscopy.'),
  'D010536': NtO(MH='Peritoneovenous Shunt', MS='An operation for the continuous emptying of ascitic fluid into the venous system. Fluid removal is based on intraperitoneal and intrathoracic superior vena cava pressure differentials and is performed via a pressure-sensitive one-way valve connected to a tube traversing the subcutaneous tissue of the chest wall to the neck where it enters the internal jugular vein and terminates in the superior vena cava. It is used in the treatment of intractable ascites.'),
  'D010537': NtO(MH='Peritoneum', MS='A membrane of squamous EPITHELIAL CELLS, the mesothelial cells, covered by apical MICROVILLI that allow rapid absorption of fluid and particles in the PERITONEAL CAVITY. The peritoneum is divided into parietal and visceral components. The parietal peritoneum covers the inside of the ABDOMINAL WALL. The visceral peritoneum covers the intraperitoneal organs. The double-layered peritoneum forms the MESENTERY that suspends these organs from the abdominal wall.'),
  'D010538': NtO(MH='Peritonitis', MS='INFLAMMATION of the PERITONEUM lining the ABDOMINAL CAVITY as the result of infectious, autoimmune, or chemical processes. Primary peritonitis is due to infection of the PERITONEAL CAVITY via hematogenous or lymphatic spread and without intra-abdominal source. Secondary peritonitis arises from the ABDOMINAL CAVITY itself through RUPTURE or ABSCESS of intra-abdominal organs.'),
  'D010539': NtO(MH='Permeability', MS='Property of membranes and other structures to permit passage of light, heat, gases, liquids, metabolites, and mineral ions.'),
  'D010541': NtO(MH='Permissiveness', MS='The attitude that grants freedom of expression and activity to another individual, but not necessarily with sanction or approval.'),
  'D010542': NtO(MH='Peromyscus', MS='A genus of the subfamily SIGMODONTINAE consisting of 49 species. Two of these are widely used in medical research. They are P. leucopus, or the white-footed mouse, and P. maniculatus, or the deer mouse.'),
  'D010543': NtO(MH='Peroneal Nerve', MS='The lateral of the two terminal branches of the sciatic nerve. The peroneal (or fibular) nerve provides motor and sensory innervation to parts of the leg and foot.'),
  'D010544': NtO(MH='Peroxidases', MS=None),
  'D010545': NtO(MH='Peroxides', MS="A group of compounds that contain a bivalent O-O group, i.e., the oxygen atoms are univalent. They can either be inorganic or organic in nature. Such compounds release atomic (nascent) oxygen readily. Thus they are strong oxidizing agents and fire hazards when in contact with combustible materials, especially under high-temperature conditions. The chief industrial uses of peroxides are as oxidizing agents, bleaching agents, and initiators of polymerization. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D010546': NtO(MH='Perphenazine', MS='An antipsychotic phenothiazine derivative with actions and uses similar to those of CHLORPROMAZINE.'),
  'D010547': NtO(MH='Persistent Fetal Circulation Syndrome', MS='A syndrome of persistent PULMONARY HYPERTENSION in the newborn infant (INFANT, NEWBORN) without demonstrable HEART DISEASES. This neonatal condition can be caused by severe pulmonary vasoconstriction (reactive type), hypertrophy of pulmonary arterial muscle (hypertrophic type), or abnormally developed pulmonary arterioles (hypoplastic type). The newborn patient exhibits CYANOSIS and ACIDOSIS due to the persistence of fetal circulatory pattern of right-to-left shunting of blood through a patent ductus arteriosus (DUCTUS ARTERIOSUS, PATENT) and at times a patent foramen ovale (FORAMEN OVALE, PATENT).'),
  'D010548': NtO(MH='Personal Health Services', MS='Health care provided to individuals.'),
  'D010549': NtO(MH='Personal Satisfaction', MS="The individual's experience of a sense of fulfillment of a need or want and the quality or state of being satisfied."),
  'D010550': NtO(MH='Personal Space', MS="Invisible boundaries surrounding the individual's body which are maintained in relation to others."),
  'D010551': NtO(MH='Personality', MS='Behavior-response patterns that characterize the individual.'),
  'D010552': NtO(MH='Personality Assessment', MS='The determination and evaluation of personality attributes by interviews, observations, tests, or scales. Articles concerning personality measurement are considered to be within scope of this term.'),
  'D010553': NtO(MH='Personality Development', MS='Growth of habitual patterns of behavior in childhood and adolescence.'),
  'D010554': NtO(MH='Personality Disorders', MS='A major deviation from normal patterns of behavior.'),
  'D010555': NtO(MH='Personality Inventory', MS='Check list, usually to be filled out by a person about himself, consisting of many statements about personal characteristics which the subject checks.'),
  'D010556': NtO(MH='Personality Tests', MS='Standardized objective tests designed to facilitate the evaluation of personality.'),
  'D010557': NtO(MH='Personnel Administration, Hospital', MS='Management activities concerned with hospital employees.'),
  'D010558': NtO(MH='Personnel Loyalty', MS='Dedication or commitment shown by employees to organizations or institutions where they work.'),
  'D010559': NtO(MH='Personnel Management', MS='Planning, organizing, and administering all activities related to personnel.'),
  'D010560': NtO(MH='Personnel Selection', MS='The process of choosing employees for specific types of employment. The concept includes recruitment.'),
  'D010561': NtO(MH='Personnel Staffing and Scheduling', MS='The selection, appointing, and scheduling of personnel.'),
  'D010562': NtO(MH='Personnel Staffing and Scheduling Information Systems', MS='Computer-based systems for use in personnel management in a facility, e.g., distribution of caregivers with relation to patient needs.'),
  'D010563': NtO(MH='Personnel Turnover', MS='A change or shift in personnel due to reorganization, resignation, or discharge.'),
  'D010564': NtO(MH='Personnel, Hospital', MS='The individuals employed by the hospital.'),
  'D010565': NtO(MH='Persuasive Communication', MS='A mode of communication concerned with inducing or urging the adoption of certain beliefs, theories, or lines of action by others.'),
  'D010566': NtO(MH='Virulence Factors, Bordetella', MS='A set of BACTERIAL ADHESINS and TOXINS, BIOLOGICAL produced by BORDETELLA organisms that determine the pathogenesis of BORDETELLA INFECTIONS, such as WHOOPING COUGH. They include filamentous hemagglutinin; FIMBRIAE PROTEINS; pertactin; PERTUSSIS TOXIN; ADENYLATE CYCLASE TOXIN; dermonecrotic toxin; tracheal cytotoxin; Bordetella LIPOPOLYSACCHARIDES; and tracheal colonization factor.'),
  'D010567': NtO(MH='Pertussis Vaccine', MS='A suspension of killed Bordetella pertussis organisms, used for immunization against pertussis (WHOOPING COUGH). It is generally used in a mixture with diphtheria and tetanus toxoids (DTP). There is an acellular pertussis vaccine prepared from the purified antigenic components of Bordetella pertussis, which causes fewer adverse reactions than whole-cell vaccine and, like the whole-cell vaccine, is generally used in a mixture with diphtheria and tetanus toxoids. (From Dorland, 28th ed)'),
  'D010568': NtO(MH='Peru', MS='A country in western South America, bordering the South Pacific Ocean, and located between Chile and Ecuador.'),
  'D010569': NtO(MH='Perylene', MS='A 20-carbon dibenz(de,kl)anthracene that can be viewed as a naphthalene fused to a phenalene or as dinaphthalene. It is used as fluorescent lipid probe in the cytochemistry of membranes and is a polycyclic hydrocarbon pollutant in soil and water. Derivatives may be carcinogenic.'),
  'D010570': NtO(MH='Pessaries', MS='Devices worn in the vagina to provide support to displaced uterus or rectum. Pessaries are used in conditions such as UTERINE PROLAPSE; CYSTOCELE; or RECTOCELE.'),
  'D010571': NtO(MH='Pest Control', MS='The reduction or regulation of the population of noxious, destructive, or dangerous plants, insects, or other animals.  This includes control of plants that serve as habitats or food sources for animal pests.'),
  'D010572': NtO(MH='Pest Control, Biological', MS='Use of naturally-occuring or genetically-engineered organisms to reduce or eliminate populations of pests.'),
  'D010573': NtO(MH='Pesticide Residues', MS='Pesticides or their breakdown products remaining in the environment following their normal use or accidental contamination.'),
  'D010574': NtO(MH='Pesticide Synergists', MS='Chemicals that, while not possessing inherent pesticidal activity, nonetheless promote or enhance the effectiveness of other pesticides when combined.'),
  'D010575': NtO(MH='Pesticides', MS='Chemicals used to destroy pests of any sort. The concept includes fungicides (FUNGICIDES, INDUSTRIAL); INSECTICIDES; RODENTICIDES; etc.'),
  'D010576': NtO(MH='Pestivirus', MS='A genus of FLAVIVIRIDAE, also known as mucosal disease virus group, which is not arthropod-borne. Transmission is by direct and indirect contact, and by transplacental and congenital transmission. Species include BORDER DISEASE VIRUS, bovine viral diarrhea virus (DIARRHEA VIRUS, BOVINE VIRAL), and CLASSICAL SWINE FEVER VIRUS.'),
  'D010577': NtO(MH='Petrolatum', MS='A colloidal system of semisolid hydrocarbons obtained from PETROLEUM. It is used as an ointment base, topical protectant, and lubricant.'),
  'D010578': NtO(MH='Petroleum', MS='Naturally occurring complex liquid hydrocarbons which, after distillation, yield combustible fuels, petrochemicals, and lubricants.'),
  'D010579': NtO(MH='Petrous Bone', MS='The dense rock-like part of temporal bone that contains the INNER EAR. Petrous bone is located at the base of the skull. Sometimes it is combined with the MASTOID PROCESS and called petromastoid part of temporal bone.'),
  'D010580': NtO(MH='Peutz-Jeghers Syndrome', MS='A hereditary disease caused by autosomal dominant mutations involving CHROMOSOME 19. It is characterized by the presence of INTESTINAL POLYPS, consistently in the JEJUNUM, and mucocutaneous pigmentation with MELANIN spots of the lips, buccal MUCOSA, and digits.'),
  'D010581': NtO(MH="Peyer's Patches", MS='Lymphoid tissue on the mucosa of the small intestine.'),
  'D010582': NtO(MH='Bacteriophage lambda', MS="A temperate inducible phage and type species of the genus lambda-like viruses, in the family SIPHOVIRIDAE. Its natural host is E. coli K12. Its VIRION contains linear double-stranded DNA with single-stranded 12-base 5' sticky ends. The DNA circularizes on infection."),
  'D010583': NtO(MH='Bacteriophage mu', MS='A temperate coliphage, in the genus Mu-like viruses, family MYOVIRIDAE, composed of a linear, double-stranded molecule of DNA, which is able to insert itself randomly at any point on the host chromosome. It frequently causes a mutation by interrupting the continuity of the bacterial OPERON at the site of insertion.'),
  'D010584': NtO(MH='Bacteriophage phi X 174', MS='The type species of the genus MICROVIRUS. A prototype of the small virulent DNA coliphages, it is composed of a single strand of supercoiled circular DNA, which on infection, is converted to a double-stranded replicative form by a host enzyme.'),
  'D010585': NtO(MH='Phagocyte Bactericidal Dysfunction', MS='Disorders in which phagocytic cells cannot kill ingested bacteria; characterized by frequent recurring infection with formulation of granulomas.'),
  'D010586': NtO(MH='Phagocytes', MS='Cells that can carry out the process of PHAGOCYTOSIS.'),
  'D010587': NtO(MH='Phagocytosis', MS='The engulfing and degradation of microorganisms; other cells that are dead, dying, or pathogenic; and foreign particles by phagocytic cells (PHAGOCYTES).'),
  'D010588': NtO(MH='Phagosomes', MS='Membrane-bound cytoplasmic vesicles formed by invagination of phagocytized material. They fuse with lysosomes to form phagolysosomes in which the hydrolytic enzymes of the lysosome digest the phagocytized material.'),
  'D010590': NtO(MH='Phalloidine', MS='Very toxic polypeptide isolated mainly from AMANITA phalloides (Agaricaceae) or death cup; causes fatal liver, kidney and CNS damage in mushroom poisoning; used in the study of liver damage.'),
  'D010591': NtO(MH='Phantom Limb', MS='Perception of painful and nonpainful phantom sensations that occur following the complete or partial loss of a limb. The majority of individuals with an amputated extremity will experience the impression that the limb is still present, and in many cases, painful. (From Neurol Clin 1998 Nov;16(4):919-36; Brain 1998 Sep;121(Pt 9):1603-30)'),
  'D010592': NtO(MH='Pharmaceutic Aids', MS='Substances which are of little or no therapeutic value, but are necessary in the manufacture, compounding, storage, etc., of pharmaceutical preparations or drug dosage forms. They include SOLVENTS, diluting agents, and suspending agents, and emulsifying agents. Also, ANTIOXIDANTS; PRESERVATIVES, PHARMACEUTICAL; COLORING AGENTS; FLAVORING AGENTS; VEHICLES; EXCIPIENTS; OINTMENT BASES.'),
  'D010593': NtO(MH='Pharmaceutical Services', MS='Total pharmaceutical services provided by qualified PHARMACISTS. In addition to the preparation and distribution of medical products, they may include consultative services provided to agencies and institutions which do not have a qualified pharmacist.'),
  'D010594': NtO(MH='Pharmacies', MS='Facilities for the preparation and dispensing of drugs.'),
  'D010595': NtO(MH='Pharmacists', MS='Those persons legally qualified by education and training to engage in the practice of pharmacy.'),
  'D010596': NtO(MH='Pharmacy Technicians', MS='Persons who perform certain functions under the supervision of a pharmacist.'),
  'D010597': NtO(MH='Pharmacogenetics', MS='A branch of genetics which deals with the genetic variability in individual responses to drugs and drug metabolism (BIOTRANSFORMATION).'),
  'D010598': NtO(MH='Pharmacognosy', MS='The science of drugs prepared from natural-sources including preparations from PLANTS, animals, and other organisms as well as MINERALS and other substances included in MATERIA MEDICA. The therapeutic usage of plants is PHYTOTHERAPY.'),
  'D010599': NtO(MH='Pharmacokinetics', MS='Dynamic and kinetic mechanisms of exogenous chemical and DRUG LIBERATION; ABSORPTION; BIOLOGICAL TRANSPORT; TISSUE DISTRIBUTION; BIOTRANSFORMATION; elimination; and DRUG TOXICITY as a function of dosage, and rate of METABOLISM. LADMER, ADME and ADMET are abbreviations for liberation, absorption, distribution, metabolism, elimination, and toxicology.'),
  'D010600': NtO(MH='Pharmacology', MS='The study of the origin, nature, properties, and actions of drugs and their effects on living organisms.'),
  'D010601': NtO(MH='Pharmacology, Clinical', MS='The branch of pharmacology that deals directly with the effectiveness and safety of drugs in humans.'),
  'D010602': NtO(MH='Pharmacopoeias as Topic', MS='Works about authoritative treatises on drugs and preparations, their description, formulation, analytic composition, physical constants, main chemical properties used in identification, standards for strength, purity, and dosage, chemical tests for determining identity and purity, etc.'),
  'D010603': NtO(MH='Pharmacopoeias, Homeopathic as Topic', MS='Works about authoritative resources describing the composition, properties, manufacture, and quality control of HOMEOPATHIC REMEDIES.'),
  'D010604': NtO(MH='Pharmacy', MS='The practice of compounding and dispensing medicinal preparations.'),
  'D010605': NtO(MH='Pharmacy Administration', MS='The business and managerial aspects of pharmacy in its broadest sense.'),
  'D010606': NtO(MH='Pharmacy and Therapeutics Committee', MS='An advisory group composed primarily of staff physicians and the pharmacist which serves as the communication link between the medical staff and the pharmacy department.'),
  'D010607': NtO(MH='Pharmacy Service, Hospital', MS='Hospital department responsible for the receiving, storing, and distribution of pharmaceutical supplies.'),
  'D010608': NtO(MH='Pharyngeal Diseases', MS='Pathological processes involving the PHARYNX.'),
  'D010609': NtO(MH='Pharyngeal Muscles', MS='The muscles of the PHARYNX are voluntary muscles arranged in two layers. The external circular layer consists of three constrictors (superior, middle, and inferior). The internal longitudinal layer consists of the palatopharyngeus, the salpingopharyngeus, and the stylopharyngeus. During swallowing, the outer layer constricts the pharyngeal wall and the inner layer elevates pharynx and LARYNX.'),
  'D010610': NtO(MH='Pharyngeal Neoplasms', MS='Tumors or cancer of the PHARYNX.'),
  'D010611': NtO(MH='Pharyngectomy', MS='Surgical removal of a part of the pharynx. (Dorland, 28th ed)'),
  'D010612': NtO(MH='Pharyngitis', MS='Inflammation of the throat (PHARYNX).'),
  'D010613': NtO(MH='Pharyngostomy', MS='External surgical fistulization of the pharynx for feeding.'),
  'D010614': NtO(MH='Pharynx', MS='A funnel-shaped fibromuscular tube that conducts food to the ESOPHAGUS, and air to the LARYNX and LUNGS. It is located posterior to the NASAL CAVITY; ORAL CAVITY; and LARYNX, and extends from the SKULL BASE to the inferior border of the CRICOID CARTILAGE anteriorly and to the inferior border of the C6 vertebra posteriorly. It is divided into the NASOPHARYNX; OROPHARYNX; and HYPOPHARYNX (laryngopharynx).'),
  'D010615': NtO(MH='Phenacetin', MS='A phenylacetamide that was formerly used in ANALGESICS but nephropathy and METHEMOGLOBINEMIA led to its withdrawal from the market. (From Smith and Reynard, Textbook of Pharmacology,1991, p431)'),
  'D010616': NtO(MH='Phenanthrenes', MS='POLYCYCLIC AROMATIC HYDROCARBONS composed of three fused BENZENE rings.'),
  'D010617': NtO(MH='Phenanthridines', MS=None),
  'D010618': NtO(MH='Phenanthrolines', MS=None),
  'D010619': NtO(MH='Phenazines', MS=None),
  'D010620': NtO(MH='Phenazocine', MS='An opioid analgesic with actions and uses similar to MORPHINE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1095)'),
  'D010621': NtO(MH='Phenazopyridine', MS='A local anesthetic that has been used in urinary tract disorders. Its use is limited by problems with toxicity (primarily blood disorders) and potential carcinogenicity.'),
  'D010622': NtO(MH='Phencyclidine', MS='A hallucinogen formerly used as a veterinary anesthetic, and briefly as a general anesthetic for humans. Phencyclidine is similar to KETAMINE in structure and in many of its effects. Like ketamine, it can produce a dissociative state. It exerts its pharmacological action through inhibition of NMDA receptors (RECEPTORS, N-METHYL-D-ASPARTATE). As a drug of abuse, it is known as PCP and Angel Dust.'),
  'D010623': NtO(MH='Phencyclidine Abuse', MS='The misuse of phencyclidine with associated psychological symptoms and impairment in social or occupational functioning.'),
  'D010624': NtO(MH='Phenelzine', MS='One of the MONOAMINE OXIDASE INHIBITORS used to treat DEPRESSION; PHOBIC DISORDERS; and PANIC.'),
  'D010625': NtO(MH='Phenylethanolamine N-Methyltransferase', MS='A methyltransferase that catalyzes the reaction of S-adenosyl-L-methionine and phenylethanolamine to yield S-adenosyl-L-homocysteine and N-methylphenylethanolamine. It can act on various phenylethanolamines and converts norepinephrine into epinephrine. (From Enzyme Nomenclature, 1992) EC 2.1.1.28.'),
  'D010626': NtO(MH='Phenylethyl Alcohol', MS='An antimicrobial, antiseptic, and disinfectant that is used also as an aromatic essence and preservative in pharmaceutics and perfumery.'),
  'D010627': NtO(MH='Phenethylamines', MS='A group of compounds that are derivatives of beta- aminoethylbenzene which is structurally and pharmacologically related to amphetamine. (From Merck Index, 11th ed)'),
  'D010628': NtO(MH='Phenetidine', MS='Used in the manufacture of acetophenetidin.'),
  'D010629': NtO(MH='Phenformin', MS='A biguanide hypoglycemic agent with actions and uses similar to those of METFORMIN. Although it is generally considered to be associated with an unacceptably high incidence of lactic acidosis, often fatal, it is still available in some countries. (From Martindale, The Extra Pharmacopoeia, 30th ed, p290)'),
  'D010630': NtO(MH='Phenindione', MS='An indandione that has been used as an anticoagulant. Phenindione has actions similar to WARFARIN, but it is now rarely employed because of its higher incidence of severe adverse effects. (From Martindale, The Extra Pharmacopoeia, 30th ed, p234)'),
  'D010632': NtO(MH='Pheniramine', MS='One of the HISTAMINE H1 ANTAGONISTS with little sedative action. It is used in treatment of hay fever, rhinitis, allergic dermatoses, and pruritus.'),
  'D010633': NtO(MH='Phenmetrazine', MS='A sympathomimetic drug used primarily as an appetite depressant. Its actions and mechanisms are similar to DEXTROAMPHETAMINE.'),
  'D010634': NtO(MH='Phenobarbital', MS='A barbituric acid derivative that acts as a nonselective central nervous system depressant. It potentiates GAMMA-AMINOBUTYRIC ACID action on GABA-A RECEPTORS, and modulates chloride currents through receptor channels. It also inhibits glutamate induced depolarizations.'),
  'D010635': NtO(MH='Phenolphthaleins', MS='A family of 3,3-bis(p-hydroxyphenyl)phthalides. They are used as CATHARTICS, indicators, and COLORING AGENTS.'),
  'D010636': NtO(MH='Phenols', MS='Benzene derivatives that include one or more hydroxyl groups attached to the ring structure.'),
  'D010637': NtO(MH='Phenolsulfonphthalein', MS='Red dye, pH indicator, and diagnostic aid for determination of renal function. It is used also for studies of the gastrointestinal and other systems.'),
  'D010638': NtO(MH='Phenoperidine', MS='A narcotic analgesic partly metabolized to meperidine in the liver. It is similar to morphine in action and used for neuroleptanalgesia, usually with droperidol.'),
  'D010640': NtO(MH='Phenothiazines', MS='Compounds containing dibenzo-1,4-thiazine. Some of them are neuroactive.'),
  'D010641': NtO(MH='Phenotype', MS='The outward appearance of the individual. It is the product of interactions between genes, and between the GENOTYPE and the environment.'),
  'D010642': NtO(MH='Phenoxyacetates', MS='Derivatives of phenoxyacetic acid, including its salts and esters.'),
  'D010643': NtO(MH='Phenoxybenzamine', MS='An alpha-adrenergic antagonist with long duration of action. It has been used to treat hypertension and as a peripheral vasodilator.'),
  'D010644': NtO(MH='Phenprocoumon', MS='Coumarin derivative that acts as a long acting oral anticoagulant.'),
  'D010645': NtO(MH='Phentermine', MS='A central nervous system stimulant and sympathomimetic with actions and uses similar to those of DEXTROAMPHETAMINE. It has been used most frequently in the treatment of obesity.'),
  'D010646': NtO(MH='Phentolamine', MS='A nonselective alpha-adrenergic antagonist. It is used in the treatment of hypertension and hypertensive emergencies, pheochromocytoma, vasospasm of RAYNAUD DISEASE and frostbite, clonidine withdrawal syndrome, impotence, and peripheral vascular disease.'),
  'D010647': NtO(MH='Phenyl Ethers', MS='Ethers that are linked to a benzene ring structure.'),
  'D010648': NtO(MH='Phenylacetates', MS='Derivatives of phenylacetic acid. Included under this heading are a variety of acid forms, salts, esters, and amides that contain the benzeneacetic acid structure. Note that this class of compounds should not be confused with derivatives of phenyl acetate, which contain the PHENOL ester of ACETIC ACID.'),
  'D010649': NtO(MH='Phenylalanine', MS='An essential aromatic amino acid that is a precursor of MELANIN; DOPAMINE; noradrenalin (NOREPINEPHRINE), and THYROXINE.'),
  'D010650': NtO(MH='Phenylalanine Ammonia-Lyase', MS='An enzyme that catalyzes the deamination of PHENYLALANINE to form trans-cinnamate and ammonia.'),
  'D010651': NtO(MH='Phenylalanine Hydroxylase', MS='An enzyme of the oxidoreductase class that catalyzes the formation of L-TYROSINE, dihydrobiopterin, and water from L-PHENYLALANINE, tetrahydrobiopterin, and oxygen. Deficiency of this enzyme may cause PHENYLKETONURIAS and PHENYLKETONURIA, MATERNAL. EC 1.14.16.1.'),
  'D010652': NtO(MH='Phenylalanine-tRNA Ligase', MS='An enzyme that activates phenylalanine with its specific transfer RNA. EC 6.1.1.20.'),
  'D010653': NtO(MH='Phenylbutazone', MS='A butyl-diphenyl-pyrazolidinedione that has anti-inflammatory, antipyretic, and analgesic activities. It has been used in ANKYLOSING SPONDYLITIS; RHEUMATOID ARTHRITIS; and REACTIVE ARTHRITIS.'),
  'D010654': NtO(MH='Phenylbutyrates', MS='Derivatives of 4-phenylbutyric acid, including its salts and esters.'),
  'D010655': NtO(MH='Phenylenediamines', MS='Aniline compounds that contain two amino groups. They are used as a precursor in the synthesis of HETEROCYCLIC COMPOUNDS and POLYMERS. p-Phenylenediamine is used in the manufacture of HAIR DYES and is an ALLERGEN.'),
  'D010656': NtO(MH='Phenylephrine', MS='An alpha-1 adrenergic agonist used as a mydriatic, nasal decongestant, and cardiotonic agent.'),
  'D010657': NtO(MH='Phenylethylmalonamide', MS='A metabolite of primidone.'),
  'D010658': NtO(MH='Phenylglyoxal', MS='A reagent that is highly selective for the modification of arginyl residues. It is used to selectively inhibit various enzymes and acts as an energy transfer inhibitor in photophosphorylation.'),
  'D010659': NtO(MH='Phenylhydrazines', MS='Diazo derivatives of aniline, used as a reagent for sugars, ketones, and aldehydes. (Dorland, 28th ed)'),
  'D010660': NtO(MH='Phenylisopropyladenosine', MS='N-Isopropyl-N-phenyl-adenosine. Antilipemic agent. Synonym: TH 162.'),
  'D010661': NtO(MH='Phenylketonurias', MS='A group of autosomal recessive disorders marked by a deficiency of the hepatic enzyme PHENYLALANINE HYDROXYLASE or less frequently by reduced activity of DIHYDROPTERIDINE REDUCTASE (i.e., atypical phenylketonuria). Classical phenylketonuria is caused by a severe deficiency of phenylalanine hydroxylase and presents in infancy with developmental delay; SEIZURES; skin HYPOPIGMENTATION; ECZEMA; and demyelination in the central nervous system. (From Adams et al., Principles of Neurology, 6th ed, p952).'),
  'D010662': NtO(MH='Phenylmercuric Acetate', MS='A phenyl mercury compound used mainly as a fungicide. Has also been used as a herbicide, slimicide, and bacteriocide.'),
  'D010663': NtO(MH='Phenylmercury Compounds', MS='Organic mercury compounds in which the mercury is attached to a phenyl group. Often used as fungicides and seed treatment agents.'),
  'D010664': NtO(MH='Phenylmethylsulfonyl Fluoride', MS='An enzyme inhibitor that inactivates IRC-50 arvin, subtilisin, and the fatty acid synthetase complex.'),
  'D010665': NtO(MH='Phenylpropanolamine', MS='A sympathomimetic that acts mainly by causing release of NOREPINEPHRINE but also has direct agonist activity at some adrenergic receptors. It is most commonly used as a nasal vasoconstrictor and an appetite depressant.'),
  'D010666': NtO(MH='Phenylpropionates', MS='Derivatives of 3-phenylpropionic acid, including its salts and esters.'),
  'D010667': NtO(MH='Phenylpyruvic Acids', MS='A group of compounds that are derivatives of phenylpyruvic acid which has the general formula C6H5CH2COCOOH, and is a metabolite of phenylalanine. (From Dorland, 28th ed)'),
  'D010668': NtO(MH='Phenylthiazolylthiourea', MS='A dopamine-beta-hydroxylase inhibitor.'),
  'D010669': NtO(MH='Phenylthiohydantoin', MS='Thiohydantoin benzene derivative.'),
  'D010670': NtO(MH='Phenylthiourea', MS=None),
  'D010671': NtO(MH='Phenylurea Compounds', MS='Compounds that include the amino-N-phenylamide structure.'),
  'D010672': NtO(MH='Phenytoin', MS='An anticonvulsant that is used to treat a wide variety of seizures. It is also an anti-arrhythmic and a muscle relaxant. The mechanism of therapeutic action is not clear, although several cellular actions have been described including effects on ion channels, active transport, and general membrane stabilization. The mechanism of its muscle relaxant effect appears to involve a reduction in the sensitivity of muscle spindles to stretch. Phenytoin has been proposed for several other therapeutic uses, but its use has been limited by its many adverse effects and interactions with other drugs.'),
  'D010673': NtO(MH='Pheochromocytoma', MS='A usually benign, well-encapsulated, lobular, vascular tumor of chromaffin tissue of the ADRENAL MEDULLA or sympathetic paraganglia. The cardinal symptom, reflecting the increased secretion of EPINEPHRINE and NOREPINEPHRINE, is HYPERTENSION, which may be persistent or intermittent. During severe attacks, there may be HEADACHE; SWEATING, palpitation, apprehension, TREMOR; PALLOR or FLUSHING of the face, NAUSEA and VOMITING, pain in the CHEST and ABDOMEN, and paresthesias of the extremities. The incidence of malignancy is as low as 5% but the pathologic distinction between benign and malignant pheochromocytomas is not clear. (Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1298)'),
  'D010674': NtO(MH='Pheophytins', MS='Chlorophylls from which the magnesium has been removed by treatment with weak acid.'),
  'D010675': NtO(MH='Pheromones', MS='Chemical substances, excreted by an organism into the environment, that elicit behavioral or physiological responses from other organisms of the same species. Perception of these chemical signals may be olfactory or by contact.'),
  'D010676': NtO(MH='Phialophora', MS='A mitosporic fungal genus. Phialophora verrucosa is a cause of chromomycosis (CHROMOBLASTOMYCOSIS). Ophiobolus is the teleomorph of Phialophora.'),
  'D010677': NtO(MH='Philadelphia Chromosome', MS='An aberrant form of human CHROMOSOME 22 characterized by translocation of the distal end of chromosome 9 from 9q34, to the long arm of chromosome 22 at 22q11. It is present in the bone marrow cells of 80 to 90 per cent of patients with chronic myelocytic leukemia (LEUKEMIA, MYELOGENOUS, CHRONIC, BCR-ABL POSITIVE).'),
  'D010678': NtO(MH='Philately', MS='Study of stamps or postal markings. It usually refers to the design and commemorative aspects of the stamp.'),
  'D010679': NtO(MH='Philippines', MS='A country in southeastern Asia, archipelago between the Philippine Sea and the South China Sea, and east of Vietnam. The capital is Manila.'),
  'D010680': NtO(MH='Philology', MS='The study of literature in its widest sense, including grammar, etymology, criticism, literary history, language history, linguistic history, systems of writing, and anything else that is relevant to literature or language viewed as literature. Philology as a discipline has both philosophical and scientific overtones.'),
  'D010681': NtO(MH='Philology, Classical', MS='The study of ancient Greek and Roman literature, including grammar, etymology, criticism, literary history, and language and linguistic history.'),
  'D010682': NtO(MH='Philology, Oriental', MS='The study of literature written in languages of Asia and the Far East, including grammar, etymology, criticism, literary history, and language and linguistic history.'),
  'D010683': NtO(MH='Philology, Romance', MS='The study of literature written in the Romance languages (French, Spanish, Italian, and others descended from Latin), including grammar, etymology, criticism, literary history, and language and linguistic history.'),
  'D010684': NtO(MH='Philosophy', MS='A love or pursuit of wisdom. A search for the underlying causes and principles of reality. (Webster, 3d ed)'),
  'D010685': NtO(MH='Philosophy, Dental', MS='The underlying rationale or theoretical basis for the principles of DENTISTRY.'),
  'D010686': NtO(MH='Philosophy, Medical', MS='The underlying rationale or theoretical basis for the principles of MEDICINE.'),
  'D010687': NtO(MH='Philosophy, Nursing', MS='The underlying rationale or theoretical basis for the principles of NURSING.'),
  'D010688': NtO(MH='Phimosis', MS='A condition in which the FORESKIN cannot be retracted to reveal the GLANS PENIS. It is due to tightness or narrowing of the foreskin opening.'),
  'D010689': NtO(MH='Phlebitis', MS='Inflammation of a vein, often a vein in the leg. Phlebitis associated with a blood clot is called (THROMBOPHLEBITIS).'),
  'D010690': NtO(MH='Phlebography', MS='Radiographic visualization or recording of a vein after the injection of contrast medium.'),
  'D010691': NtO(MH='Phlebotomus', MS='A genus of PSYCHODIDAE which functions as the vector of a number of pathogenic organisms, including LEISHMANIA DONOVANI; LEISHMANIA TROPICA; Bartonella bacilliformis, and the Pappataci fever virus (SANDFLY FEVER NAPLES VIRUS).'),
  'D010692': NtO(MH='Phleomycins', MS='Water-soluble, copper-containing low molecular weight polypeptides obtained from the culture medium of Streptomyces verticillus. They are specific inhibitors of DNA synthesis in bacteria and have been found to act as antitumor agents. They have also been used against rust fungi of plants.'),
  'D010693': NtO(MH='Phloretin', MS='A natural dihydrochalcone found in apples and many other fruits.'),
  'D010694': NtO(MH='Lactase-Phlorizin Hydrolase', MS='A multifunctional protein that contains two enzyme domains. The first domain (EC 3.2.1.62) hydrolyzes glycosyl-N-acylsphingosine to a sugar and N-acylsphingosine. The second domain (EC 3.2.1.108) hydrolyzes LACTOSE and is found in the intestinal brush border membrane. Loss of activity for this enzyme in humans results in LACTOSE INTOLERANCE.'),
  'D010695': NtO(MH='Phlorhizin', MS=None),
  'D010696': NtO(MH='Phloroglucinol', MS='A trinitrobenzene derivative with antispasmodic properties that is used primarily as a laboratory reagent.'),
  'D010697': NtO(MH='Eosine I Bluish', MS='A red fluorescein dye used as a histologic stain. It may be cytotoxic, mutagenic, and inhibit certain mitochondrial functions.'),
  'D010698': NtO(MH='Phobic Disorders', MS='Anxiety disorders in which the essential feature is persistent and irrational fear of a specific object, activity, or situation that the individual feels compelled to avoid. The individual recognizes the fear as excessive or unreasonable.'),
  'D010699': NtO(MH='Phonation', MS='The process of producing vocal sounds by means of VOCAL CORDS vibrating in an expiratory blast of air.'),
  'D010700': NtO(MH='Phonetics', MS='The science or study of speech sounds and their production, transmission, and reception, and their analysis, classification, and transcription. (Random House Unabridged Dictionary, 2d ed)'),
  'D010701': NtO(MH='Phonocardiography', MS='Graphic registration of the heart sounds picked up as vibrations and transformed by a piezoelectric crystal microphone into a varying electrical output according to the stresses imposed by the sound waves. The electrical output is amplified by a stethograph amplifier and recorded by a device incorporated into the electrocardiograph or by a multichannel recording machine.'),
  'D010702': NtO(MH='Phorate', MS='A cholinesterase inhibitor that is used as an insecticide.'),
  'D010703': NtO(MH='Phorbol Esters', MS='Tumor-promoting compounds obtained from CROTON OIL (Croton tiglium). Some of these are used in cell biological experiments as activators of protein kinase C.'),
  'D010704': NtO(MH='Phorbols', MS='The parent alcohol of the tumor promoting compounds from CROTON OIL (Croton tiglium).'),
  'D010705': NtO(MH='Phosgene', MS='A highly toxic gas that has been used as a chemical warfare agent. It is an insidious poison as it is not irritating immediately, even when fatal concentrations are inhaled. (From The Merck Index, 11th ed, p7304)'),
  'D010706': NtO(MH='Phosmet', MS='An organothiophosphorus insecticide that has been used to control pig mange.'),
  'D010707': NtO(MH='Phosphamidon', MS='An organophosphate cholinesterase inhibitor that is used as an insecticide.'),
  'D010709': NtO(MH='Phosphate Acetyltransferase', MS='An enzyme that catalyzes the synthesis of acetylphosphate from acetyl-CoA and inorganic phosphate. Acetylphosphate serves as a high-energy phosphate compound. EC 2.3.1.8.'),
  'D010710': NtO(MH='Phosphates', MS='Inorganic salts of phosphoric acid.'),
  'D010711': NtO(MH='Phosphatidate Phosphatase', MS='A phosphomonoesterase involved in the synthesis of triacylglycerols. It catalyzes the hydrolysis of phosphatidates with the formation of diacylglycerols and orthophosphate. EC 3.1.3.4.'),
  'D010712': NtO(MH='Phosphatidic Acids', MS='Fatty acid derivatives of glycerophosphates. They are composed of glycerol bound in ester linkage with 1 mole of phosphoric acid at the terminal 3-hydroxyl group and with 2 moles of fatty acids at the other two hydroxyl groups.'),
  'D010713': NtO(MH='Phosphatidylcholines', MS='Derivatives of PHOSPHATIDIC ACIDS in which the phosphoric acid is bound in ester linkage to a CHOLINE moiety.'),
  'D010714': NtO(MH='Phosphatidylethanolamines', MS='Derivatives of phosphatidic acids in which the phosphoric acid is bound in ester linkage to an ethanolamine moiety. Complete hydrolysis yields 1 mole of glycerol, phosphoric acid and ethanolamine and 2 moles of fatty acids.'),
  'D010715': NtO(MH='Phosphatidylglycerols', MS='A nitrogen-free class of lipids present in animal and particularly plant tissues and composed of one mole of glycerol and 1 or 2 moles of phosphatidic acid. Members of this group differ from one another in the nature of the fatty acids released on hydrolysis.'),
  'D010716': NtO(MH='Phosphatidylinositols', MS='Derivatives of phosphatidic acids in which the phosphoric acid is bound in ester linkage to the hexahydroxy alcohol, myo-inositol. Complete hydrolysis yields 1 mole of glycerol, phosphoric acid, myo-inositol, and 2 moles of fatty acids.'),
  'D010717': NtO(MH='CDPdiacylglycerol-Serine O-Phosphatidyltransferase', MS='An enzyme that catalyzes the formation of phosphatidylserine and CMP from CDPdiglyceride plus serine. EC 2.7.8.8.'),
  'D010718': NtO(MH='Phosphatidylserines', MS='Derivatives of PHOSPHATIDIC ACIDS in which the phosphoric acid is bound in ester linkage to a SERINE moiety.'),
  'D010719': NtO(MH='Phosphenes', MS='A subjective visual sensation with the eyes closed and in the absence of light. Phosphenes can be spontaneous, or induced by chemical, electrical, or mechanical (pressure) stimuli which cause the visual field to light up without optical inputs.'),
  'D010720': NtO(MH='Phosphines', MS="Inorganic or organic compounds derived from phosphine (PH3) by the replacement of H atoms. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D010721': NtO(MH='Phosphinic Acids', MS='Inorganic or organic derivatives of phosphinic acid, H2PO(OH). They include phosphinates and phosphinic acid esters.'),
  'D010723': NtO(MH='3-Deoxy-7-Phosphoheptulonate Synthase', MS='An enzyme that catalyzes the formation of 7-phospho-2-keto-3-deoxy-D-arabinoheptonate from phosphoenolpyruvate and D-erythrose-4-phosphate. It is one of the first enzymes in the biosynthesis of TYROSINE and PHENYLALANINE. This enzyme was formerly listed as EC 4.1.2.15.'),
  'D010724': NtO(MH='Phosphoadenosine Phosphosulfate', MS="3'-Phosphoadenosine-5'-phosphosulfate. Key intermediate in the formation by living cells of sulfate esters of phenols, alcohols, steroids, sulfated polysaccharides, and simple esters, such as choline sulfate. It is formed from sulfate ion and ATP in a two-step process. This compound also is an important step in the process of sulfur fixation in plants and microorganisms."),
  'D010725': NtO(MH='Phosphocreatine', MS='An endogenous substance found mainly in skeletal muscle of vertebrates. It has been tried in the treatment of cardiac disorders and has been added to cardioplegic solutions. (Reynolds JEF(Ed): Martindale: The Extra Pharmacopoeia (electronic version). Micromedex, Inc, Englewood, CO, 1996)'),
  'D010726': NtO(MH='Phosphodiesterase Inhibitors', MS='Compounds which inhibit or antagonize the biosynthesis or actions of phosphodiesterases.'),
  'D010727': NtO(MH='Phosphoric Diester Hydrolases', MS='A class of enzymes that catalyze the hydrolysis of one of the two ester bonds in a phosphodiester compound. EC 3.1.4.'),
  'D010728': NtO(MH='Phosphoenolpyruvate', MS='A monocarboxylic acid anion derived from selective deprotonation of the carboxy group of phosphoenolpyruvic acid. It is a metabolic intermediate in GLYCOLYSIS; GLUCONEOGENESIS; and other pathways.'),
  'D010729': NtO(MH='Phosphoenolpyruvate Carboxykinase (GTP)', MS='An enzyme of the lyase class that catalyzes the conversion of GTP and oxaloacetate to GDP, phosphoenolpyruvate, and carbon dioxide. This reaction is part of gluconeogenesis in the liver. The enzyme occurs in both the mitochondria and cytosol of mammalian liver. (From Dorland, 27th ed) EC 4.1.1.32.'),
  'D010730': NtO(MH='Phosphoenolpyruvate Carboxylase', MS='An enzyme with high affinity for carbon dioxide. It catalyzes irreversibly the formation of oxaloacetate from phosphoenolpyruvate and carbon dioxide. This fixation of carbon dioxide in several bacteria and some plants is the first step in the biosynthesis of glucose. EC 4.1.1.31.'),
  'D010731': NtO(MH='Phosphoenolpyruvate Sugar Phosphotransferase System', MS='The bacterial sugar phosphotransferase system (PTS) that catalyzes the transfer of the phosphoryl group from phosphoenolpyruvate to its sugar substrates (the PTS sugars) concomitant with the translocation of these sugars across the bacterial membrane. The phosphorylation of a given sugar requires four proteins, two general proteins, Enzyme I and HPr and a pair of sugar-specific proteins designated as the Enzyme II complex. The PTS has also been implicated in the induction of synthesis of some catabolic enzyme systems required for the utilization of sugars that are not substrates of the PTS as well as the regulation of the activity of ADENYLYL CYCLASES. EC 2.7.1.-.'),
  'D010732': NtO(MH='Phosphofructokinase-1', MS='An allosteric enzyme that regulates glycolysis by catalyzing the transfer of a phosphate group from ATP to fructose-6-phosphate to yield fructose-1,6-bisphosphate. D-tagatose- 6-phosphate and sedoheptulose-7-phosphate also are acceptors. UTP, CTP, and ITP also are donors. In human phosphofructokinase-1, three types of subunits have been identified. They are PHOSPHOFRUCTOKINASE-1, MUSCLE TYPE; PHOSPHOFRUCTOKINASE-1, LIVER TYPE; and PHOSPHOFRUCTOKINASE-1, TYPE C; found in platelets, brain, and other tissues.'),
  'D010733': NtO(MH='Phosphoglucomutase', MS='An enzyme that catalyzes the conversion of alpha D-glucose 1-phosphate to alpha D-glucose 6-phosphate. EC 5.4.2.2.'),
  'D010734': NtO(MH='Phosphogluconate Dehydrogenase', MS='An enzyme of the oxidoreductase class that catalyzes the reaction 6-phospho-D-gluconate and NADP+ to yield D-ribulose 5-phosphate, carbon dioxide, and NADPH. The reaction is a step in the pentose phosphate pathway of glucose metabolism. (From Dorland, 27th ed) EC 1.1.1.43.'),
  'D010735': NtO(MH='Phosphoglycerate Kinase', MS='An enzyme catalyzing the transfer of a phosphate group from 3-phospho-D-glycerate in the presence of ATP to yield 3-phospho-D-glyceroyl phosphate and ADP. EC 2.7.2.3.'),
  'D010736': NtO(MH='Phosphoglycerate Mutase', MS='An enzyme that catalyzes the conversion of 2-phospho-D-glycerate to 3-phospho-D-glycerate.'),
  'D010738': NtO(MH='Type C Phospholipases', MS='A subclass of phospholipases that hydrolyze the phosphoester bond found in the third position of GLYCEROPHOSPHOLIPIDS. Although the singular term phospholipase C specifically refers to an enzyme that catalyzes the hydrolysis of PHOSPHATIDYLCHOLINE (EC 3.1.4.3), it is commonly used in the literature to refer to broad variety of enzymes that specifically catalyze the hydrolysis of PHOSPHATIDYLINOSITOLS.'),
  'D010739': NtO(MH='Phospholipase D', MS='An enzyme found mostly in plant tissue. It hydrolyzes glycerophosphatidates with the formation of a phosphatidic acid and a nitrogenous base such as choline. This enzyme also catalyzes transphosphatidylation reactions. EC 3.1.4.4.'),
  'D010740': NtO(MH='Phospholipases', MS='A class of enzymes that catalyze the hydrolysis of phosphoglycerides or glycerophosphatidates. EC 3.1.-.'),
  'D010741': NtO(MH='Phospholipases A', MS='Phospholipases that hydrolyze one of the acyl groups of phosphoglycerides or glycerophosphatidates.'),
  'D010742': NtO(MH='Phospholipid Ethers', MS='Phospholipids which have an alcohol moiety in ethereal linkage with a saturated or unsaturated aliphatic alcohol. They are usually derivatives of phosphoglycerols or phosphatidates. The other two alcohol groups of the glycerol backbone are usually in ester linkage. These compounds are widely distributed in animal tissues.'),
  'D010743': NtO(MH='Phospholipids', MS='Lipids containing one or more phosphate groups, particularly those derived from either glycerol (phosphoglycerides see GLYCEROPHOSPHOLIPIDS) or sphingosine (SPHINGOLIPIDS). They are polar lipids that are of great importance for the structure and function of cell membranes and are the most abundant of membrane lipids, although not stored in large amounts in the system.'),
  'D010744': NtO(MH='Phosphoric Monoester Hydrolases', MS='A group of hydrolases which catalyze the hydrolysis of monophosphoric esters with the production of one mole of orthophosphate.'),
  'D010746': NtO(MH='Phosphonoacetic Acid', MS='A simple organophosphorus compound that inhibits DNA polymerase, especially in viruses and is used as an antiviral agent.'),
  'D010748': NtO(MH='Phosphopeptides', MS='PEPTIDES that incorporate a phosphate group via PHOSPHORYLATION.'),
  'D010749': NtO(MH='Phosphoprotein Phosphatases', MS='A group of enzymes removing the SERINE- or THREONINE-bound phosphate groups from a wide range of phosphoproteins, including a number of enzymes which have been phosphorylated under the action of a kinase. (Enzyme Nomenclature, 1992)'),
  'D010750': NtO(MH='Phosphoproteins', MS=None),
  'D010751': NtO(MH='Phosphopyruvate Hydratase', MS='A hydro-lyase that catalyzes the dehydration of 2-phosphoglycerate to form PHOSPHOENOLPYRUVATE. Several different isoforms of this enzyme exist, each with its own tissue specificity.'),
  'D010752': NtO(MH='Phosphoramide Mustards', MS='A group of nitrogen mustard compounds which are substituted with a phosphoramide group or its derivatives. They are usually cytotoxic and used as antineoplastic agents.'),
  'D010753': NtO(MH='Phosphoranes', MS='Inorganic or organic compounds that contain pentavalent phosphorus and the general formula R5P.'),
  'D010754': NtO(MH='Phosphoribosyl Pyrophosphate', MS='The key substance in the biosynthesis of histidine, tryptophan, and purine and pyrimidine nucleotides.'),
  'D010755': NtO(MH='Organophosphates', MS='Carbon-containing phosphoric acid derivatives. Included under this heading are compounds that have CARBON atoms bound to one or more OXYGEN atoms of the P(=O)(O)3 structure. Note that several specific classes of endogenous phosphorus-containing compounds such as NUCLEOTIDES; PHOSPHOLIPIDS; and PHOSPHOPROTEINS are listed elsewhere.'),
  'D010756': NtO(MH='Phosphoric Acids', MS='Inorganic derivatives of phosphoric acid (H3PO4). Note that organic derivatives of phosphoric acids are listed under ORGANOPHOSPHATES.'),
  'D010757': NtO(MH='Phosphorous Acids', MS='Inorganic derivatives of phosphorus trihydroxide (P(OH)3) and its tautomeric form dihydroxyphosphine oxide (HP=O(OH)2). Note that organic derivatives of phosphonic acids are listed under are ORGANOPHOSPHONATES.'),
  'D010758': NtO(MH='Phosphorus', MS='A non-metal element that has the atomic symbol P, atomic number 15, and atomic weight 31. It is an essential element that takes part in a broad variety of biochemical reactions.'),
  'D010759': NtO(MH='Phosphorus Isotopes', MS='Stable phosphorus atoms that have the same atomic number as the element phosphorus, but differ in atomic weight. P-31 is a stable phosphorus isotope.'),
  'D010760': NtO(MH='Phosphorus Metabolism Disorders', MS='Disorders in the processing of phosphorus in the body: its absorption, transport, storage, and utilization.'),
  'D010761': NtO(MH='Phosphorus Radioisotopes', MS='Unstable isotopes of phosphorus that decay or disintegrate emitting radiation. P atoms with atomic weights 28-34 except 31 are radioactive phosphorus isotopes.'),
  'D010762': NtO(MH='Phosphorylase a', MS='The active form of GLYCOGEN PHOSPHORYLASE that is derived from the phosphorylation of PHOSPHORYLASE B. Phosphorylase a is deactivated via hydrolysis of phosphoserine by PHOSPHORYLASE PHOSPHATASE to form PHOSPHORYLASE B.'),
  'D010763': NtO(MH='Phosphorylase b', MS='The inactive form of GLYCOGEN PHOSPHORYLASE that is converted to the active form PHOSPHORYLASE A via phosphorylation by PHOSPHORYLASE KINASE and ATP.'),
  'D010764': NtO(MH='Phosphorylase Kinase', MS='An enzyme that catalyzes the conversion of ATP and PHOSPHORYLASE B to ADP and PHOSPHORYLASE A.'),
  'D010765': NtO(MH='Phosphorylase Phosphatase', MS='An enzyme that deactivates glycogen phosphorylase a by releasing inorganic phosphate and phosphorylase b, the inactive form. EC 3.1.3.17.'),
  'D010766': NtO(MH='Phosphorylation', MS='The introduction of a phosphoryl group into a compound through the formation of an ester bond between the compound and a phosphorus moiety.'),
  'D010767': NtO(MH='Phosphorylcholine', MS='Calcium and magnesium salts used therapeutically in hepatobiliary dysfunction.'),
  'D010768': NtO(MH='Phosphoserine', MS='The phosphoric acid ester of serine.'),
  'D010769': NtO(MH='Phosphothreonine', MS='The phosphoric acid ester of threonine. Used as an identifier in the analysis of peptides, proteins, and enzymes.'),
  'D010770': NtO(MH='Phosphotransferases', MS='A rather large group of enzymes comprising not only those transferring phosphate but also diphosphate, nucleotidyl residues, and others. These have also been subdivided according to the acceptor group. (From Enzyme Nomenclature, 1992) EC 2.7.'),
  'D010772': NtO(MH='Phosphotungstic Acid', MS='Tungsten hydroxide oxide phosphate. A white or slightly yellowish-green, slightly efflorescent crystal or crystalline powder. It is used as a reagent for alkaloids and many other nitrogen bases, for phenols, albumin, peptone, amino acids, uric acid, urea, blood, and carbohydrates. (From Merck Index, 11th ed)'),
  'D010773': NtO(MH='Leptophos', MS='An organothiophosphate insecticide.'),
  'D010774': NtO(MH='Phosvitin', MS='An egg yolk phosphoglycoprotein which contains about 90% of the yolk protein phosphorus. It is synthesized in the liver of the hen and transferred to the developing oocyte, where it is bound to lipoproteins within the yolk granules.'),
  'D010775': NtO(MH='Photic Stimulation', MS='Investigative technique commonly used during ELECTROENCEPHALOGRAPHY in which a series of bright light flashes or visual patterns are used to elicit brain activity.'),
  'D010776': NtO(MH='Photobacterium', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that are common in the marine environment and on the surfaces and in the intestinal contents of marine animals. Some species are bioluminescent and are found as symbionts in specialized luminous organs of fish.'),
  'D010777': NtO(MH='Photochemistry', MS='A branch of physical chemistry which studies chemical reactions, isomerization and physical behavior that may occur under the influence of visible and/or ultraviolet light.'),
  'D010778': NtO(MH='Photochemotherapy', MS='Therapy using oral or topical photosensitizing agents with subsequent exposure to light.'),
  'D010779': NtO(MH='Photofluorography', MS='The photography of images produced on a fluorescent screen by X-rays.'),
  'D010780': NtO(MH='Photogrammetry', MS='Making measurements by the use of stereoscopic photographs.'),
  'D010781': NtO(MH='Photography', MS='Method of making images on a sensitized surface by exposure to light or other radiant energy.'),
  'D010782': NtO(MH='Photolysis', MS='Chemical bond cleavage reactions resulting from absorption of radiant energy.'),
  'D010783': NtO(MH='Photometry', MS='Measurement of the various properties of light.'),
  'D010784': NtO(MH='Photomicrography', MS='Photography of objects viewed under a microscope using ordinary photographic methods.'),
  'D010785': NtO(MH='Photophosphorylation', MS='The use of light to convert ADP to ATP without the concomitant reduction of dioxygen to water as occurs during OXIDATIVE PHOSPHORYLATION in MITOCHONDRIA.'),
  'D010786': NtO(MH='Photoreceptor Cells', MS='Specialized cells that detect and transduce light. They are classified into two types based on their light reception structure, the ciliary photoreceptors and the rhabdomeric photoreceptors with MICROVILLI. Ciliary photoreceptor cells use OPSINS that activate a PHOSPHODIESTERASE phosphodiesterase cascade. Rhabdomeric photoreceptor cells use opsins that activate a PHOSPHOLIPASE C cascade.'),
  'D010787': NtO(MH='Photosensitivity Disorders', MS='Abnormal responses to sunlight or artificial light due to extreme reactivity of light-absorbing molecules in tissues. It refers almost exclusively to skin photosensitivity, including sunburn, reactions due to repeated prolonged exposure in the absence of photosensitizing factors, and reactions requiring photosensitizing factors such as photosensitizing agents and certain diseases. With restricted reference to skin tissue, it does not include photosensitivity of the eye to light, as in photophobia or photosensitive epilepsy.'),
  'D010788': NtO(MH='Photosynthesis', MS='The synthesis by organisms of organic chemical compounds, especially carbohydrates, from carbon dioxide using energy obtained from light rather than from the oxidation of chemical compounds. Photosynthesis comprises two separate processes: the light reactions and the dark reactions. In higher plants; GREEN ALGAE; and CYANOBACTERIA; NADPH and ATP formed by the light reactions drive the dark reactions which result in the fixation of carbon dioxide. (from Oxford Dictionary of Biochemistry and Molecular Biology, 2001)'),
  'D010789': NtO(MH='Phototherapy', MS='Treatment of disease by exposure to light, especially by variously concentrated light rays or specific wavelengths.'),
  'D010791': NtO(MH='Phrenic Nerve', MS='The motor nerve of the diaphragm. The phrenic nerve fibers originate in the cervical spinal column (mostly C4) and travel through the cervical plexus to the diaphragm.'),
  'D010792': NtO(MH='Phrenology', MS='The study of the conformation of the skull as indicative of mental faculties and traits of character, especially according to the hypothesis of F. J. Gall (1758-1828). (Webster 3d ed)'),
  'D010793': NtO(MH='Phthalazines', MS='Bicyclic heterocyclic compounds containing a BENZENE ring fused to PYRIDAZINE.'),
  'D010795': NtO(MH='Phthalic Acids', MS='A group of compounds that has the general structure of a dicarboxylic acid-substituted benzene ring. The ortho-isomer is used in dye manufacture. (Dorland, 28th ed)'),
  'D010796': NtO(MH='Phthalic Anhydrides', MS='Phthalic acid anhydrides. Can be substituted on any carbon atom. Used extensively in industry and as a reagent in the acylation of amino- and hydroxyl groups.'),
  'D010797': NtO(MH='Phthalimides', MS='The imide of phthalic acids.'),
  'D010798': NtO(MH='Phycocyanin', MS='The metal-free blue phycobilin pigment in a conjugated chromoprotein of blue-green algae. It functions as light-absorbing substance together with chlorophylls.'),
  'D010799': NtO(MH='Phycoerythrin', MS='The metal-free red phycobilin pigment in a conjugated chromoprotein of red algae. It functions as a light-absorbing substance together with chlorophylls.'),
  'D010800': NtO(MH='Phycomyces', MS='A genus of zygomycetous fungi in the family Mucoraceae, order MUCORALES, forming mycelia having a metallic sheen. It has been used for research on phototropism.'),
  'D010802': NtO(MH='Phylogeny', MS='The relationships of groups of organisms as reflected by their genetic makeup.'),
  'D010803': NtO(MH='Physalaemin', MS='An oligopeptide isolated from the skin of Physalaemus fuscumaculatus, a South American frog. It is a typical kinin, resembling SUBSTANCE P in structure and action and has been proposed as a sialagogue, antihypertensive, and vasodilator.'),
  'D010804': NtO(MH='Physarum', MS='A genus of protozoa, formerly also considered a fungus. Characteristics include the presence of violet to brown spores.'),
  'D010805': NtO(MH='Physical Conditioning, Animal', MS='Diet modification and physical exercise to improve the ability of animals to perform physical activities.'),
  'D010806': NtO(MH='Physical Education and Training', MS='Instructional programs in the care and development of the body, often in schools. The concept does not include prescribed exercises, which is EXERCISE THERAPY.'),
  'D010807': NtO(MH='Physical Endurance', MS='The time span between the beginning of physical activity by an individual and the termination because of exhaustion.'),
  'D010808': NtO(MH='Physical Examination', MS='Systematic and thorough inspection of the patient for physical signs of disease or abnormality.'),
  'D010809': NtO(MH='Physical Fitness', MS='The ability to carry out daily tasks and perform physical activities in a highly functional state, often as a result of physical conditioning.'),
  'D010810': NtO(MH='Physical and Rehabilitation Medicine', MS='A medical specialty concerned with the use of physical agents, mechanical apparatus, and manipulation in rehabilitating physically diseased or injured patients.'),
  'D010811': NtO(MH='Natural Science Disciplines', MS='The sciences dealing with processes observable in nature.'),
  'D010812': NtO(MH='Physical Stimulation', MS='Act of eliciting a response from a person or organism through physical contact.'),
  'D010814': NtO(MH='Physical Therapy Department, Hospital', MS='Hospital department which is responsible for the administration and provision of diagnostic and medical rehabilitation services to restore or improve the functional capacity of the patient.'),
  'D010815': NtO(MH='Physician Impairment', MS="The physician's inability to practice medicine with reasonable skill and safety to the patient due to the physician's disability. Common causes include alcohol and drug abuse, mental illness, physical disability, and senility."),
  'D010816': NtO(MH='Physician Incentive Plans', MS='Compensatory plans designed to motivate physicians in relation to patient referral, physician recruitment, and efficient use of the health facility.'),
  'D010817': NtO(MH='Physician-Patient Relations', MS='The interactions between physician and patient.'),
  'D010818': NtO(MH="Practice Patterns, Physicians'", MS='Patterns of practice related to diagnosis and treatment as especially influenced by cost of the service requested and provided.'),
  'D010819': NtO(MH="Physician's Role", MS='The expected function of a member of the medical profession.'),
  'D010820': NtO(MH='Physicians', MS='Individuals licensed to practice medicine.'),
  'D010821': NtO(MH='Physicians, Family', MS='Those physicians who have completed the education requirements specified by the American Academy of Family Physicians.'),
  'D010822': NtO(MH='Physicians, Women', MS='Women licensed to practice medicine.'),
  'D010823': NtO(MH='Physician Assistants', MS='Health professionals who practice medicine as members of a team with their supervising physicians. They deliver a broad range of medical and surgical services to diverse populations in rural and urban settings. Duties may include physical exams, diagnosis and treatment of disease, interpretation of tests, assist in surgery, and prescribe medications. (from http://www.aapa.orglabout-pas accessed 2114/2011)'),
  'D010824': NtO(MH="Physicians' Offices", MS="The room or rooms in which the physician and staff provide patient care. The offices include all rooms in the physician's office suite."),
  'D010825': NtO(MH='Physics', MS='The study of those aspects of energy and matter in terms of elementary principles and laws. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D010826': NtO(MH='Physiognomy', MS="An assessment of a person's personality based on their facial and other external features."),
  'D010827': NtO(MH='Physiology', MS='The biological science concerned with the life-supporting properties, functions, and processes of living organisms or their parts.'),
  'D010828': NtO(MH='Physiology, Comparative', MS='The biological science concerned with similarities or differences in the life-supporting functions and processes of different species.'),
  'D010829': NtO(MH='Physiological Phenomena', MS='The functions and properties of living organisms, including both the physical and chemical factors and processes, supporting life in single- or multi-cell organisms from their origin through the progression of life.'),
  'D010830': NtO(MH='Physostigmine', MS='A cholinesterase inhibitor that is rapidly absorbed through membranes. It can be applied topically to the conjunctiva. It also can cross the blood-brain barrier and is used when central nervous system effects are desired, as in the treatment of severe anticholinergic toxicity.'),
  'D010831': NtO(MH='Phytanic Acid', MS='A 20-carbon branched chain fatty acid. In phytanic acid storage disease (REFSUM DISEASE) this lipid may comprise as much as 30% of the total fatty acids of the plasma. This is due to a phytanic acid alpha-hydroxylase deficiency.'),
  'D010832': NtO(MH='6-Phytase', MS='An enzyme that catalyzes the conversion of myo-inositol hexakisphosphate and water to 1L-myo-inositol 1,2,3,4,5-pentakisphosphate and orthophosphate. EC 3.1.3.26.'),
  'D010833': NtO(MH='Phytic Acid', MS='Complexing agent for removal of traces of heavy metal ions. It acts also as a hypocalcemic agent.'),
  'D010834': NtO(MH='Phytochrome', MS='A blue-green biliprotein widely distributed in the plant kingdom.'),
  'D010835': NtO(MH='Phytohemagglutinins', MS='Mucoproteins isolated from the kidney bean (Phaseolus vulgaris); some of them are mitogenic to lymphocytes, others agglutinate all or certain types of erythrocytes or lymphocytes. They are used mainly in the study of immune mechanisms and in cell culture.'),
  'D010836': NtO(MH='Phytol', MS='Acyclic diterpene used in making synthetic forms of vitamin E and vitamin K1.'),
  'D010837': NtO(MH='Vitamin K 1', MS='A family of phylloquinones that contains a ring of 2-methyl-1,4-naphthoquinone and an isoprenoid side chain. Members of this group of vitamin K 1 have only one double bond on the proximal isoprene unit. Rich sources of vitamin K 1 include green plants, algae, and photosynthetic bacteria. Vitamin K1 has antihemorrhagic and prothrombogenic activity.'),
  'D010838': NtO(MH='Phytophthora', MS='A genus of destructive parasitic OOMYCETES in the family Peronosporaceae, order Peronosporales, affecting numerous fruit, vegetable, and other crops. Differentiation of zoospores usually takes place in the sporangium and no vesicle is formed. It was previously considered a fungus.'),
  'D010839': NtO(MH='Phytoplankton', MS='Free-floating minute organisms that are photosynthetic. The term is non-taxonomic and refers to a lifestyle (energy utilization and motility), rather than a particular type of organism. Most, but not all, are unicellular algae. Important groups include DIATOMS; DINOFLAGELLATES; CYANOBACTERIA; CHLOROPHYTA; HAPTOPHYTA; CRYPTOMONADS; and silicoflagellates.'),
  'D010840': NtO(MH='Phytosterols', MS='A class of organic compounds known as sterols or STEROIDS derived from plants.'),
  'D010841': NtO(MH='Pia Mater', MS='The innermost layer of the three meninges covering the brain and spinal cord. It is the fine vascular membrane that lies under the ARACHNOID and the DURA MATER.'),
  'D010842': NtO(MH='Pica', MS='The persistent eating of non-nutritive substances for a period of at least one month.'),
  'D010843': NtO(MH='Pichia', MS='Yeast-like ascomycetous fungi of the family Saccharomycetaceae, order SACCHAROMYCETALES isolated from exuded tree sap.'),
  'D010844': NtO(MH='Picibanil', MS='A lyophilized preparation of a low-virulence strain (SU) of Streptococcus pyogenes (S. hemolyticus), inactivated by heating with penicillin G. It has been proposed as a noncytotoxic antineoplastic agent because of its immune system-stimulating activity.'),
  'D010845': NtO(MH='Obesity Hypoventilation Syndrome', MS='HYPOVENTILATION syndrome in very obese persons with excessive ADIPOSE TISSUE around the ABDOMEN and DIAPHRAGM. It is characterized by diminished to absent ventilatory chemoresponsiveness; chronic HYPOXIA; HYPERCAPNIA; POLYCYTHEMIA; and long periods of sleep during day and night (HYPERSOMNOLENCE). It is a condition often related to OBSTRUCTIVE SLEEP APNEA but can occur separately.'),
  'D010846': NtO(MH='Picloram', MS='A picolinic acid derivative that is used as a herbicide.'),
  'D010847': NtO(MH='Picolines', MS='A group of compounds that are monomethyl derivatives of pyridines. (From Dorland, 28th ed)'),
  'D010848': NtO(MH='Picolinic Acids', MS='Compounds with general formula C5H4N(CO2H) derived from PYRIDINE, having a carboxylic acid substituent at the 2-position.'),
  'D010849': NtO(MH='Picornaviridae', MS='A family of small RNA viruses comprising some important pathogens of humans and animals. Transmission usually occurs mechanically. There are nine genera: APHTHOVIRUS; CARDIOVIRUS; ENTEROVIRUS; ERBOVIRUS; HEPATOVIRUS; KOBUVIRUS; PARECHOVIRUS; RHINOVIRUS; and TESCHOVIRUS.'),
  'D010850': NtO(MH='Picornaviridae Infections', MS='Virus diseases caused by the PICORNAVIRIDAE.'),
  'D010851': NtO(MH='Picrates', MS='Salts or esters of PICRIC ACID.'),
  'D010852': NtO(MH='Picrotoxin', MS='A noncompetitive antagonist at GABA-A receptors and thus a convulsant. Picrotoxin blocks the GAMMA-AMINOBUTYRIC ACID-activated chloride ionophore. Although it is most often used as a research tool, it has been used as a CNS stimulant and an antidote in poisoning by CNS depressants, especially the barbiturates.'),
  'D010853': NtO(MH='Picryl Chloride', MS='A hapten that generates suppressor cells capable of down-regulating the efferent phase of trinitrophenol-specific contact hypersensitivity. (Arthritis Rheum 1991 Feb;34(2):180).'),
  'D010854': NtO(MH='Piedra', MS='Either of two diseases resulting from fungal infection of the hair shafts. Black piedra occurs mainly in and on the hairs of the scalp and is caused by Piedraia hortae; white piedra occurs in and on the hairs of the scalp, beard, moustache and genital areas and is caused by Trichosporon species.'),
  'D010855': NtO(MH='Pierre Robin Syndrome', MS='Congenital malformation characterized by MICROGNATHIA or RETROGNATHIA; GLOSSOPTOSIS and CLEFT PALATE. The mandibular abnormalities often result in difficulties in sucking and swallowing. The syndrome may be isolated or associated with other syndromes (e.g., ANDERSEN SYNDROME; CAMPOMELIC DYSPLASIA). Developmental mis-expression of SOX9 TRANSCRIPTION FACTOR gene on chromosome 17q and its surrounding region is associated with the syndrome.'),
  'D010856': NtO(MH='Columbidae', MS='Family in the order COLUMBIFORMES, comprised of pigeons or doves. They are BIRDS with short legs, stout bodies, small heads, and slender bills. Some sources call the smaller species doves and the larger pigeons, but the names are interchangeable.'),
  'D010857': NtO(MH='Pigment Epithelium of Eye', MS='The layer of pigment-containing epithelial cells in the RETINA; the CILIARY BODY; and the IRIS in the eye.'),
  'D010858': NtO(MH='Pigmentation', MS='Coloration or discoloration of a part by a pigment.'),
  'D010859': NtO(MH='Pigmentation Disorders', MS='Diseases affecting PIGMENTATION, including SKIN PIGMENTATION.'),
  'D010860': NtO(MH='Pigments, Biological', MS='Any normal or abnormal coloring matter in PLANTS; ANIMALS or micro-organisms.'),
  'D010861': NtO(MH='Fimbriae, Bacterial', MS='Thin, hairlike appendages, 1 to 20 microns in length and often occurring in large numbers, present on the cells of gram-negative bacteria, particularly Enterobacteriaceae and Neisseria. Unlike flagella, they do not possess motility, but being protein (pilin) in nature, they possess antigenic and hemagglutinating properties. They are of medical importance because some fimbriae mediate the attachment of bacteria to cells via adhesins (ADHESINS, BACTERIAL). Bacterial fimbriae refer to common pili, to be distinguished from the preferred use of "pili", which is confined to sex pili (PILI, SEX).'),
  'D010862': NtO(MH='Pilocarpine', MS='A slowly hydrolyzed muscarinic agonist with no nicotinic effects. Pilocarpine is used as a miotic and in the treatment of glaucoma.'),
  'D010863': NtO(MH='Piloerection', MS='Involuntary erection or bristling of hairs.'),
  'D010864': NtO(MH='Pilonidal Sinus', MS='A hair-containing cyst or sinus, occurring chiefly in the coccygeal region.'),
  'D010865': NtO(MH='Pilot Projects', MS='Small-scale tests of methods and procedures to be used on a larger scale if the pilot study demonstrates that these methods and procedures can work.'),
  'D010866': NtO(MH='Natamycin', MS='Amphoteric macrolide antifungal antibiotic from Streptomyces natalensis or S. chattanoogensis. It is used for a variety of fungal infections, mainly topically.'),
  'D010867': NtO(MH='Pimelic Acids', MS='A group of compounds that are derivatives of heptanedioic acid with the general formula R-C7H11O4.'),
  'D010868': NtO(MH='Pimozide', MS='A diphenylbutylpiperidine that is effective as an antipsychotic agent and as an alternative to HALOPERIDOL for the suppression of vocal and motor tics in patients with Tourette syndrome. Although the precise mechanism of action is unknown, blockade of postsynaptic dopamine receptors has been postulated. (From AMA Drug Evaluations Annual, 1994, p403)'),
  'D010869': NtO(MH='Pindolol', MS='A moderately lipophilic beta blocker (ADRENERGIC BETA-ANTAGONISTS). It is non-cardioselective and has intrinsic sympathomimetic actions, but little membrane-stabilizing activity. (From Martindale, The Extra Pharmocopoeia, 30th ed, p638)'),
  'D010870': NtO(MH='Pineal Gland', MS='A light-sensitive neuroendocrine organ attached to the roof of the THIRD VENTRICLE of the brain. The pineal gland secretes MELATONIN, other BIOGENIC AMINES and NEUROPEPTIDES.'),
  'D010871': NtO(MH='Pinealoma', MS="Neoplasms which originate from pineal parenchymal cells that tend to enlarge the gland and be locally invasive. The two major forms are pineocytoma and the more malignant pineoblastoma. Pineocytomas have moderate cellularity and tend to form rosette patterns. Pineoblastomas are highly cellular tumors containing small, poorly differentiated cells. These tumors occasionally seed the neuroaxis or cause obstructive HYDROCEPHALUS or Parinaud's syndrome. GERMINOMA; CARCINOMA, EMBRYONAL; GLIOMA; and other neoplasms may arise in the pineal region with germinoma being the most common pineal region tumor. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2064; Adams et al., Principles of Neurology, 6th ed, p670)"),
  'D010872': NtO(MH='Caniformia', MS='Suborder of dog-like carnivores which include dogs, coyotes, skunks, mustelids, walruses, seals, bears, and procyonids.'),
  'D010873': NtO(MH='Pinocytosis', MS='The engulfing of liquids by cells by a process of invagination and closure of the cell membrane to form fluid-filled vacuoles.'),
  'D010874': NtO(MH='Pinta', MS='An infectious disease of the skin caused by Treponema carateum that occurs only in the western hemisphere. Age of onset is between 10 and 20 years of age. This condition is characterized by marked changes in the skin color and is believed to be transmitted by direct person-to-person contact.'),
  'D010875': NtO(MH='Pipecolic Acids', MS=None),
  'D010876': NtO(MH='Pipemidic Acid', MS='Antimicrobial against Gram negative and some Gram positive bacteria. It is protein bound and concentrated in bile and urine and used for gastrointestinal, biliary, and urinary infections.'),
  'D010878': NtO(MH='Piperacillin', MS='Semisynthetic, broad-spectrum, AMPICILLIN derived ureidopenicillin antibiotic proposed for PSEUDOMONAS infections. It is also used in combination with other antibiotics.'),
  'D010879': NtO(MH='Piperazines', MS='Compounds that are derived from PIPERAZINE.'),
  'D010880': NtO(MH='Piperidines', MS='A family of hexahydropyridines.'),
  'D010881': NtO(MH='Piperidones', MS=None),
  'D010882': NtO(MH='Piperonyl Butoxide', MS='An insecticide synergist, especially for pyrethroids and ROTENONE.'),
  'D010883': NtO(MH='Piperoxan', MS='A benzodioxane alpha-adrenergic blocking agent with considerable stimulatory action. It has been used to diagnose PHEOCHROMOCYTOMA and as an antihypertensive agent.'),
  'D010884': NtO(MH='Pipidae', MS='A family of the order Anura, distinguished by the lack of a tongue. It includes four living genera of aquatic "toads". Two of the most familiar pipids are the popularly called Surinam "toad" (Pipa pipa) and XENOPUS LAEVIS.'),
  'D010885': NtO(MH='Pipobroman', MS='An antineoplastic agent that acts by alkylation.'),
  'D010889': NtO(MH='Piracetam', MS='A compound suggested to be both a nootropic and a neuroprotective agent.'),
  'D010890': NtO(MH='Pirenzepine', MS='An antimuscarinic agent that inhibits gastric secretion at lower doses than are required to affect gastrointestinal motility, salivary, central nervous system, cardiovascular, ocular, and urinary function. It promotes the healing of duodenal ulcers and due to its cytoprotective action is beneficial in the prevention of duodenal ulcer recurrence. It also potentiates the effect of other antiulcer agents such as CIMETIDINE and RANITIDINE. It is generally well tolerated by patients.'),
  'D010891': NtO(MH='Piribedil', MS='A dopamine D2 agonist. It is used in the treatment of parkinson disease, particularly for alleviation of tremor. It has also been used for circulatory disorders and in other applications as a D2 agonist.'),
  'D010892': NtO(MH='Pirinitramide', MS='A diphenylpropylamine with intense narcotic analgesic activity of long duration. It is a derivative of MEPERIDINE with similar activity and usage.'),
  'D010893': NtO(MH='Piromidic Acid', MS='Antibacterial against mainly gram negative organisms. It is used for urinary tract and intestinal infections.'),
  'D010894': NtO(MH='Piroxicam', MS='A cyclooxygenase inhibiting, non-steroidal anti-inflammatory agent (NSAID) that is well established in treating rheumatoid arthritis and osteoarthritis and used for musculoskeletal disorders, dysmenorrhea, and postoperative pain. Its long half-life enables it to be administered once daily.'),
  'D010895': NtO(MH='Pit and Fissure Sealants', MS='Agents used to occlude dental enamel pits and fissures in the prevention of dental caries.'),
  'D010896': NtO(MH='Pitcairn Island', MS="An island in Polynesia, in the south Pacific Ocean. It was discovered in 1767 by Philip Carteret, uninhabited until 1790 when settled by mutineers from the English ship, Bounty. The settlement was discovered in 1808; the population was removed temporarily to Tahiti in 1831 and to Norfolk Island (between New Caledonia and New Zealand) in 1856. Some later returned to Pitcairn and their descendents constitute the present population of this British colony. The island is named for the midshipman who first sighted it from the ship. (From Webster's New Geographical Dictionary, 1988, p958 & Room, Brewer's Dictionary of Names, 1992, p422)"),
  'D010897': NtO(MH='Pitch Discrimination', MS='The ability to differentiate tones.'),
  'D010898': NtO(MH='Pitch Perception', MS='A dimension of auditory sensation varying with cycles per second of the sound stimulus.'),
  'D010899': NtO(MH='Pituitary Apoplexy', MS='The sudden loss of blood supply to the PITUITARY GLAND, leading to tissue NECROSIS and loss of function (PANHYPOPITUITARISM). The most common cause is hemorrhage or INFARCTION of a PITUITARY ADENOMA. It can also result from acute hemorrhage into SELLA TURCICA due to HEAD TRAUMA; INTRACRANIAL HYPERTENSION; or other acute effects of central nervous system hemorrhage. Clinical signs include severe HEADACHE; HYPOTENSION; bilateral visual disturbances; UNCONSCIOUSNESS; and COMA.'),
  'D010900': NtO(MH='Pituitary Diseases', MS='Disorders involving either the ADENOHYPOPHYSIS or the NEUROHYPOPHYSIS. These diseases usually manifest as hypersecretion or hyposecretion of PITUITARY HORMONES. Neoplastic pituitary masses can also cause compression of the OPTIC CHIASM and other adjacent structures.'),
  'D010901': NtO(MH='Pituitary Function Tests', MS='Examinations that evaluate functions of the pituitary gland.'),
  'D010902': NtO(MH='Pituitary Gland', MS='A small, unpaired gland situated in the SELLA TURCICA. It is connected to the HYPOTHALAMUS by a short stalk which is called the INFUNDIBULUM.'),
  'D010903': NtO(MH='Pituitary Gland, Anterior', MS='The anterior glandular lobe of the pituitary gland, also known as the adenohypophysis. It secretes the ADENOHYPOPHYSEAL HORMONES that regulate vital functions such as GROWTH; METABOLISM; and REPRODUCTION.'),
  'D010904': NtO(MH='Pituitary Gland, Posterior', MS='Neural tissue of the pituitary gland, also known as the neurohypophysis. It consists of the distal AXONS of neurons that produce VASOPRESSIN and OXYTOCIN in the SUPRAOPTIC NUCLEUS and the PARAVENTRICULAR NUCLEUS. These axons travel down through the MEDIAN EMINENCE, the hypothalamic infundibulum of the PITUITARY STALK, to the posterior lobe of the pituitary gland.'),
  'D010905': NtO(MH='Pituitary Hormone Release Inhibiting Hormones', MS='Polypeptide hormones produced in the hypothalamus which inhibit the release of pituitary hormones. Used for PHRIH in general or for which there is no specific heading.'),
  'D010906': NtO(MH='Pituitary Hormone-Releasing Hormones', MS='Peptides, natural or synthetic, that stimulate the release of PITUITARY HORMONES. They were first isolated from the extracts of the HYPOTHALAMUS; MEDIAN EMINENCE; PITUITARY STALK; and NEUROHYPOPHYSIS. In addition, some hypophysiotropic hormones control pituitary cell differentiation, cell proliferation, and hormone synthesis. Some can act on more than one pituitary hormone.'),
  'D010907': NtO(MH='Pituitary Hormones', MS='Hormones secreted by the PITUITARY GLAND including those from the anterior lobe (adenohypophysis), the posterior lobe (neurohypophysis), and the ill-defined intermediate lobe. Structurally, they include small peptides, proteins, and glycoproteins. They are under the regulation of neural signals (NEUROTRANSMITTERS) or neuroendocrine signals (HYPOTHALAMIC HORMONES) from the hypothalamus as well as feedback from their targets such as ADRENAL CORTEX HORMONES; ANDROGENS; ESTROGENS.'),
  'D010908': NtO(MH='Pituitary Hormones, Anterior', MS='Hormones secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR). Structurally, they include polypeptide, protein, and glycoprotein molecules.'),
  'D010909': NtO(MH='Pituitary Hormones, Posterior', MS='Hormones released from the neurohypophysis (PITUITARY GLAND, POSTERIOR). They include a number of peptides which are formed in the NEURONS in the HYPOTHALAMUS, bound to NEUROPHYSINS, and stored in the nerve terminals in the posterior pituitary. Upon stimulation, these peptides are released into the hypophysial portal vessel blood.'),
  'D010910': NtO(MH='Pituitary Irradiation', MS='Radiation therapy used to treat the PITUITARY GLAND.'),
  'D010911': NtO(MH='Pituitary Neoplasms', MS='Neoplasms which arise from or metastasize to the PITUITARY GLAND. The majority of pituitary neoplasms are adenomas, which are divided into non-secreting and secreting forms. Hormone producing forms are further classified by the type of hormone they secrete. Pituitary adenomas may also be characterized by their staining properties (see ADENOMA, BASOPHIL; ADENOMA, ACIDOPHIL; and ADENOMA, CHROMOPHOBE). Pituitary tumors may compress adjacent structures, including the HYPOTHALAMUS, several CRANIAL NERVES, and the OPTIC CHIASM. Chiasmal compression may result in bitemporal HEMIANOPSIA.'),
  'D010912': NtO(MH='Pituitary-Adrenal Function Tests', MS='Tests that evaluate the adrenal glands controlled by pituitary hormones.'),
  'D010913': NtO(MH='Pituitary-Adrenal System', MS='The interactions between the anterior pituitary and adrenal glands, in which corticotropin (ACTH) stimulates the adrenal cortex and adrenal cortical hormones suppress the production of corticotropin by the anterior pituitary.'),
  'D010915': NtO(MH='Pityriasis', MS='A name originally applied to a group of skin diseases characterized by the formation of fine, branny scales, but now used only with a modifier. (Dorland, 27th ed)'),
  'D010916': NtO(MH='Pityriasis Rubra Pilaris', MS='A chronic skin disease characterized by small follicular papules, disseminated reddish-brown scaly patches, and often, palmoplantar hyperkeratosis. The papules are about the size of a pin and topped by a horny plug.'),
  'D010917': NtO(MH='Pivampicillin', MS='Pivalate ester analog of AMPICILLIN.'),
  'D010918': NtO(MH='Pizotyline', MS='Serotonin antagonist used against MIGRAINE DISORDERS and vascular headaches.'),
  'D010919': NtO(MH='Placebos', MS='Any dummy medication or treatment. Although placebos originally were medicinal preparations having no specific pharmacological activity against a targeted condition, the concept has been extended to include treatments or procedures, especially those administered to control groups in clinical trials in order to provide baseline measurements for the experimental protocol.'),
  'D010920': NtO(MH='Placenta', MS='A highly vascularized mammalian fetal-maternal organ and major site of transport of oxygen, nutrients, and fetal waste products. It includes a fetal portion (CHORIONIC VILLI) derived from TROPHOBLASTS and a maternal portion (DECIDUA) derived from the uterine ENDOMETRIUM. The placenta produces an array of steroid, protein and peptide hormones (PLACENTAL HORMONES).'),
  'D010921': NtO(MH='Placenta Accreta', MS='Abnormal placentation in which all or parts of the PLACENTA are attached directly to the MYOMETRIUM due to a complete or partial absence of DECIDUA. It is associated with POSTPARTUM HEMORRHAGE because of the failure of placental separation.'),
  'D010922': NtO(MH='Placenta Diseases', MS='Pathological processes or abnormal functions of the PLACENTA.'),
  'D010923': NtO(MH='Placenta Previa', MS='Abnormal placentation in which the PLACENTA implants in the lower segment of the UTERUS (the zone of dilation) and may cover part or all of the opening of the CERVIX. It is often associated with serious antepartum bleeding and PREMATURE LABOR.'),
  'D010924': NtO(MH='Placental Extracts', MS='Extracts prepared from placental tissue; they may contain specific but uncharacterized factors or proteins with specific activities.'),
  'D010925': NtO(MH='Placental Function Tests', MS='Methods used for the assessment of placental function.'),
  'D010926': NtO(MH='Placental Hormones', MS='Hormones produced by the placenta include CHORIONIC GONADOTROPIN, and PLACENTAL LACTOGEN as well as steroids (ESTROGENS; PROGESTERONE), and neuropeptide hormones similar to those found in the hypothalamus (HYPOTHALAMIC HORMONES).'),
  'D010927': NtO(MH='Placental Insufficiency', MS='Failure of the PLACENTA to deliver an adequate supply of nutrients and OXYGEN to the FETUS.'),
  'D010928': NtO(MH='Placental Lactogen', MS='A polypeptide hormone of approximately 25 kDa that is produced by the SYNCYTIOTROPHOBLASTS of the PLACENTA, also known as chorionic somatomammotropin. It has both GROWTH HORMONE and PROLACTIN activities on growth, lactation, and luteal steroid production. In women, placental lactogen secretion begins soon after implantation and increases to 1 g or more a day in late pregnancy. Placental lactogen is also an insulin antagonist.'),
  'D010929': NtO(MH='Placentation', MS='The development of the PLACENTA, a highly vascularized mammalian fetal-maternal organ and major site of transport of oxygen, nutrients, and fetal waste products between mother and FETUS. The process begins at FERTILIZATION, through the development of CYTOTROPHOBLASTS and SYNCYTIOTROPHOBLASTS, the formation of CHORIONIC VILLI, to the progressive increase in BLOOD VESSELS to support the growing fetus.'),
  'D010930': NtO(MH='Plague', MS='An acute infectious disease caused by YERSINIA PESTIS that affects humans, wild rodents, and their ectoparasites. This condition persists due to its firm entrenchment in sylvatic rodent-flea ecosystems throughout the world. Bubonic plague is the most common form.'),
  'D010931': NtO(MH='Plague Vaccine', MS='A suspension of killed Yersinia pestis used for immunizing people in enzootic plague areas.'),
  'D010932': NtO(MH='Planarians', MS='Nonparasitic free-living flatworms of the class Turbellaria. The most common genera are Dugesia, formerly Planaria, which lives in water, and Bipalium, which lives on land. Geoplana occurs in South America and California.'),
  'D010933': NtO(MH='Plankton', MS='Community of tiny aquatic PLANTS and ANIMALS, and photosynthetic BACTERIA, that are either free-floating or suspended in the water, with little or no power of locomotion. They are divided into PHYTOPLANKTON and ZOOPLANKTON.'),
  'D010934': NtO(MH='Planning Techniques', MS='Procedures, strategies, and theories of planning.'),
  'D010935': NtO(MH='Plant Diseases', MS='Diseases of plants.'),
  'D010936': NtO(MH='Plant Extracts', MS='Concentrated pharmaceutical preparations of plants obtained by removing active constituents with a suitable solvent, which is evaporated away, and adjusting the residue to a prescribed standard.'),
  'D010937': NtO(MH='Plant Growth Regulators', MS='Any of the hormones produced naturally in plants and active in controlling growth and other functions. There are three primary classes: auxins, cytokinins, and gibberellins.'),
  'D010938': NtO(MH='Plant Oils', MS='Oils derived from plants or plant products.'),
  'D010939': NtO(MH='Plant Poisoning', MS='Poisoning by the ingestion of plants or its leaves, berries, roots or stalks. The manifestations in both humans and animals vary in severity from mild to life threatening. In animals, especially domestic animals, it is usually the result of ingesting moldy or fermented forage.'),
  'D010940': NtO(MH='Plant Proteins', MS='Proteins found in plants (flowers, herbs, shrubs, trees, etc.). The concept does not include proteins found in vegetables for which PLANT PROTEINS, DIETARY is available.'),
  'D010941': NtO(MH='Plant Tumors', MS='A localized proliferation of plant tissue forming a swelling or outgrowth, commonly with a characteristic shape and unlike any organ of the normal plant. Plant tumors or galls usually form in response to the action of a pathogen or a pest. (Holliday, P., A Dictionary of Plant Pathology, 1989, p330)'),
  'D010942': NtO(MH='Plant Viruses', MS='Viruses parasitic on plants.'),
  'D010943': NtO(MH='Plantago', MS='A plant genus of the family Plantaginaceae. The small plants usually have a dense tuft of basal leaves and long, leafless stalks bearing a terminal spike of small flowers. The seeds, known as PSYLLIUM, swell in water and are used as laxatives. The leaves have been used medicinally.'),
  'D010944': NtO(MH='Plants', MS='Multicellular, eukaryotic life forms of kingdom Plantae (sensu lato), comprising the VIRIDIPLANTAE; RHODOPHYTA; and GLAUCOPHYTA; all of which acquired chloroplasts by direct endosymbiosis of CYANOBACTERIA. They are characterized by a mainly photosynthetic mode of nutrition; essentially unlimited growth at localized regions of cell divisions (MERISTEMS); cellulose within cells providing rigidity; the absence of organs of locomotion; absence of nervous and sensory systems; and an alternation of haploid and diploid generations.'),
  'D010945': NtO(MH='Plants, Edible', MS='An organism of the vegetable kingdom suitable by nature for use as a food, especially by human beings. Not all parts of any given plant are edible but all parts of edible plants have been known to figure as raw or cooked food: leaves, roots, tubers, stems, seeds, buds, fruits, and flowers. The most commonly edible parts of plants are FRUIT, usually sweet, fleshy, and succulent. Most edible plants are commonly cultivated for their nutritional value and are referred to as VEGETABLES.'),
  'D010946': NtO(MH='Plants, Medicinal', MS='Plants whose roots, leaves, seeds, bark, or other constituent parts possess therapeutic, tonic, purgative, curative or other pharmacologic attributes, when administered to man or animals.'),
  'D010947': NtO(MH='Plants, Toxic', MS='Plants or plant parts which are harmful to man or other animals.'),
  'D010948': NtO(MH='Viral Plaque Assay', MS='Method for measuring viral infectivity and multiplication in CULTURED CELLS. Clear lysed areas or plaques develop as the VIRAL PARTICLES are released from the infected cells during incubation. With some VIRUSES, the cells are killed by a cytopathic effect; with others, the infected cells are not killed but can be detected by their hemadsorptive ability. Sometimes the plaque cells contain VIRAL ANTIGENS which can be measured by IMMUNOFLUORESCENCE.'),
  'D010949': NtO(MH='Plasma', MS='The residual portion of BLOOD that is left after removal of BLOOD CELLS by CENTRIFUGATION without prior BLOOD COAGULATION.'),
  'D010950': NtO(MH='Plasma Cells', MS='Specialized forms of antibody-producing B-LYMPHOCYTES. They synthesize and secrete immunoglobulin. They are found only in lymphoid organs and at sites of immune responses and normally do not circulate in the blood or lymph. (Rosen et al., Dictionary of Immunology, 1989, p169 & Abbas et al., Cellular and Molecular Immunology, 2d ed, p20)'),
  'D010951': NtO(MH='Plasma Exchange', MS='Removal of plasma and replacement with various fluids, e.g., fresh frozen plasma, plasma protein fractions (PPF), albumin preparations, dextran solutions, saline. Used in treatment of autoimmune diseases, immune complex diseases, diseases of excess plasma factors, and other conditions.'),
  'D010952': NtO(MH='Plasma Substitutes', MS='Any liquid used to replace blood plasma, usually a saline solution, often with serum albumins, dextrans or other preparations. These substances do not enhance the oxygen- carrying capacity of blood, but merely replace the volume. They are also used to treat dehydration.'),
  'D010953': NtO(MH='Plasma Volume', MS='Volume of PLASMA in the circulation. It is usually measured by INDICATOR DILUTION TECHNIQUES.'),
  'D010954': NtO(MH='Plasmacytoma', MS='Any discrete, presumably solitary, mass of neoplastic PLASMA CELLS either in BONE MARROW or various extramedullary sites.'),
  'D010955': NtO(MH='Plasmalogens', MS='GLYCEROPHOSPHOLIPIDS in which one of the two acyl chains is attached to glycerol with an ether alkenyl linkage instead of an ester as with the other glycerophospholipids.'),
  'D010956': NtO(MH='Plasmapheresis', MS='Procedure whereby plasma is separated and extracted from anticoagulated whole blood and the red cells retransfused to the donor. Plasmapheresis is also employed for therapeutic use.'),
  'D010957': NtO(MH='Plasmids', MS='Extrachromosomal, usually CIRCULAR DNA molecules that are self-replicating and transferable from one organism to another. They are found in a variety of bacterial, archaeal, fungal, algal, and plant species. They are used in GENETIC ENGINEERING as CLONING VECTORS.'),
  'D010958': NtO(MH='Plasminogen', MS='Precursor of plasmin (FIBRINOLYSIN). It is a single-chain beta-globulin of molecular weight 80-90,000 found mostly in association with fibrinogen in plasma; plasminogen activators change it to fibrinolysin. It is used in wound debriding and has been investigated as a thrombolytic agent.'),
  'D010959': NtO(MH='Tissue Plasminogen Activator', MS='A proteolytic enzyme in the serine protease family found in many tissues which converts PLASMINOGEN to FIBRINOLYSIN. It has fibrin-binding activity and is immunologically different from UROKINASE-TYPE PLASMINOGEN ACTIVATOR. The primary sequence, composed of 527 amino acids, is identical in both the naturally occurring and synthetic proteases.'),
  'D010960': NtO(MH='Plasminogen Activators', MS='A heterogeneous group of proteolytic enzymes that convert PLASMINOGEN to FIBRINOLYSIN. They are concentrated in the lysosomes of most cells and in the vascular endothelium, particularly in the vessels of the microcirculation.'),
  'D010961': NtO(MH='Plasmodium', MS='A genus of protozoa that comprise the malaria parasites of mammals. Four species infect humans (although occasional infections with primate malarias may occur). These are PLASMODIUM FALCIPARUM; PLASMODIUM MALARIAE; PLASMODIUM OVALE, and PLASMODIUM VIVAX. Species causing infection in vertebrates other than man include: PLASMODIUM BERGHEI; PLASMODIUM CHABAUDI; P. vinckei, and PLASMODIUM YOELII in rodents; P. brasilianum, PLASMODIUM CYNOMOLGI; and PLASMODIUM KNOWLESI in monkeys; and PLASMODIUM GALLINACEUM in chickens.'),
  'D010962': NtO(MH='Plasmodium berghei', MS='A protozoan parasite of rodents transmitted by the mosquito Anopheles dureni.'),
  'D010963': NtO(MH='Plasmodium falciparum', MS='A species of protozoa that is the causal agent of falciparum malaria (MALARIA, FALCIPARUM). It is most prevalent in the tropics and subtropics.'),
  'D010964': NtO(MH='Plasmodium gallinaceum', MS='A protozoan parasite that causes avian malaria (MALARIA, AVIAN), primarily in chickens, and is transmitted by the Aedes mosquito.'),
  'D010965': NtO(MH='Plasmodium malariae', MS='A protozoan parasite that occurs primarily in subtropical and temperate areas. It is the causal agent of quartan malaria. As the parasite grows it exhibits little ameboid activity.'),
  'D010966': NtO(MH='Plasmodium vivax', MS='A protozoan parasite that causes vivax malaria (MALARIA, VIVAX). This species is found almost everywhere malaria is endemic and is the only one that has a range extending into the temperate regions.'),
  'D010967': NtO(MH='Plasmodium yoelii', MS='A species of PLASMODIUM causing malaria in rodents.'),
  'D010968': NtO(MH='Plasticizers', MS='Materials incorporated mechanically in plastics (usually PVC) to increase flexibility, workability or distensibility; due to the non-chemical inclusion, plasticizers leach out from the plastic and are found in body fluids and the general environment.'),
  'D010969': NtO(MH='Plastics', MS='Polymeric materials (usually organic) of large molecular weight which can be shaped by flow. Plastic usually refers to the final product with fillers, plasticizers, pigments, and stabilizers included (versus the resin, the homogeneous polymeric starting material). (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D010970': NtO(MH='Plastocyanin', MS='A copper-containing plant protein that is a fundamental link in the electron transport chain of green plants during the photosynthetic conversion of light energy by photophosphorylation into the potential energy of chemical bonds.'),
  'D010971': NtO(MH='Plastoquinone', MS='Polyunsaturated side-chain quinone derivative which is an important link in the electron transport chain of green plants during the photosynthetic conversion of light energy by photophosphorylation into the potential energy of chemical bonds.'),
  'D010972': NtO(MH='Platelet Activating Factor', MS='A phospholipid derivative formed by PLATELETS; BASOPHILS; NEUTROPHILS; MONOCYTES; and MACROPHAGES. It is a potent platelet aggregating agent and inducer of systemic anaphylactic symptoms, including HYPOTENSION; THROMBOCYTOPENIA; NEUTROPENIA; and BRONCHOCONSTRICTION.'),
  'D010973': NtO(MH='Platelet Adhesiveness', MS='The process whereby PLATELETS adhere to something other than platelets, e.g., COLLAGEN; BASEMENT MEMBRANE; MICROFIBRILS; or other "foreign" surfaces.'),
  'D010974': NtO(MH='Platelet Aggregation', MS='The attachment of PLATELETS to one another. This clumping together can be induced by a number of agents (e.g., THROMBIN; COLLAGEN) and is part of the mechanism leading to the formation of a THROMBUS.'),
  'D010975': NtO(MH='Platelet Aggregation Inhibitors', MS='Drugs or agents which antagonize or impair any mechanism leading to blood platelet aggregation, whether during the phases of activation and shape change or following the dense-granule release reaction and stimulation of the prostaglandin-thromboxane system.'),
  'D010976': NtO(MH='Platelet Count', MS='The number of PLATELETS per unit volume in a sample of venous BLOOD.'),
  'D010977': NtO(MH='Platelet Factor 3', MS='A phospholipid from the platelet membrane that contributes to the blood clotting cascade by forming a phospholipid-protein complex (THROMBOPLASTIN) which serves as a cofactor with FACTOR VIIA to activate FACTOR X in the extrinsic pathway of BLOOD COAGULATION.'),
  'D010978': NtO(MH='Platelet Factor 4', MS='A CXC chemokine that is found in the alpha granules of PLATELETS. The protein has a molecular size of 7800 kDa and can occur as a monomer, a dimer or a tetramer depending upon its concentration in solution. Platelet factor 4 has a high affinity for HEPARIN and is often found complexed with GLYCOPROTEINS such as PROTEIN C.'),
  'D010979': NtO(MH='Platelet Function Tests', MS="Laboratory examination used to monitor and evaluate platelet function in a patient's blood."),
  'D010980': NtO(MH='Platelet Membrane Glycoproteins', MS='Surface glycoproteins on platelets which have a key role in hemostasis and thrombosis such as platelet adhesion and aggregation. Many of these are receptors.'),
  'D010981': NtO(MH='Platelet Storage Pool Deficiency', MS='Disorder characterized by a decrease or lack of platelet dense bodies in which the releasable pool of adenine nucleotides and 5HT are normally stored.'),
  'D010982': NtO(MH='Platelet-Derived Growth Factor', MS='Mitogenic peptide growth hormone carried in the alpha-granules of platelets. It is released when platelets adhere to traumatized tissues. Connective tissue cells near the traumatized region respond by initiating the process of replication.'),
  'D010983': NtO(MH='Plateletpheresis', MS='The preparation of platelet concentrates with the return of red cells and platelet-poor plasma to the donor.'),
  'D010984': NtO(MH='Platinum', MS='A heavy, soft, whitish metal, resembling tin, with atomic number 78, atomic weight 195.084, symbol Pt. It is used in manufacturing equipment for laboratory and industrial use. It occurs as a black powder (platinum black) and as a spongy substance (spongy platinum) and may have been known in Pliny\'s time as "alutiae".'),
  'D010985': NtO(MH='Platybasia', MS='A developmental deformity of the occipital bone and upper end of the cervical spine, in which the latter appears to have pushed the floor of the occipital bone upward. (Dorland, 27th ed)'),
  'D010986': NtO(MH='Platyhelminths', MS='A phylum of acoelomate, bilaterally symmetrical flatworms, without a definite anus. It includes three classes: Cestoda, Turbellaria, and Trematoda.'),
  'D010987': NtO(MH='Platypus', MS='A small aquatic oviparous mammal of the order Monotremata found in Australia and Tasmania.'),
  'D010988': NtO(MH='Play and Playthings', MS='Spontaneous or voluntary recreational activities pursued for enjoyment and accessories or equipment used in the activities; includes games, toys, etc.'),
  'D010989': NtO(MH='Play Therapy', MS='A treatment technique utilizing play as a medium for expression and communication between patient and therapist.'),
  'D010990': NtO(MH='Pleasure-Pain Principle', MS='The psychoanalytic concept that man instinctively seeks to avoid pain and discomfort and strives for gratification and pleasure.'),
  'D010991': NtO(MH='Plethysmography', MS='Recording of change in the size of a part as modified by the circulation in it.'),
  'D010992': NtO(MH='Plethysmography, Impedance', MS='Recording changes in electrical impedance between electrodes placed on opposite sides of a part of the body, as a measure of volume changes in the path of the current. (Stedman, 25th ed)'),
  'D010993': NtO(MH='Plethysmography, Whole Body', MS='Measurement of the volume of gas in the lungs, including that which is trapped in poorly communicating air spaces. It is of particular use in chronic obstructive pulmonary disease and emphysema. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D010994': NtO(MH='Pleura', MS='The thin serous membrane enveloping the lungs (LUNG) and lining the THORACIC CAVITY. Pleura consist of two layers, the inner visceral pleura lying next to the pulmonary parenchyma and the outer parietal pleura. Between the two layers is the PLEURAL CAVITY which contains a thin film of liquid.'),
  'D010995': NtO(MH='Pleural Diseases', MS='Diseases involving the PLEURA.'),
  'D010996': NtO(MH='Pleural Effusion', MS='Presence of fluid in the pleural cavity resulting from excessive transudation or exudation from the pleural surfaces. It is a sign of disease and not a diagnosis in itself.'),
  'D010997': NtO(MH='Pleural Neoplasms', MS='Neoplasms of the thin serous membrane that envelopes the lungs and lines the thoracic cavity. Pleural neoplasms are exceedingly rare and are usually not diagnosed until they are advanced because in the early stages they produce no symptoms.'),
  'D010998': NtO(MH='Pleurisy', MS='INFLAMMATION of PLEURA, the lining of the LUNG. When PARIETAL PLEURA is involved, there is pleuritic CHEST PAIN.'),
  'D010999': NtO(MH='Pleurodeles', MS='A genus of aquatic newts belonging to the family Salamandridae and sometimes referred to as "spiny" tritons. There are two species P. waltlii and P. poireti. P. waltlii is commonly used in the laboratory. Since this genus adapts to aquarium living, it is easy to maintain in laboratories.'),
  'D011000': NtO(MH='Pleurodynia, Epidemic', MS='An acute, febrile, infectious disease generally occurring in epidemics. It is usually caused by coxsackieviruses B and sometimes by coxsackieviruses A; echoviruses; or other enteroviruses.'),
  'D011001': NtO(MH='Pleuropneumonia', MS='Inflammation of the lung parenchyma that is associated with PLEURISY, inflammation of the PLEURA.'),
  'D011002': NtO(MH='Pleuropneumonia, Contagious', MS='A pleuropneumonia of cattle and goats caused by species of MYCOPLASMA.'),
  'D011003': NtO(MH='Ploidies', MS='The degree of replication of the chromosome set in the karyotype.'),
  'D011004': NtO(MH='Plummer-Vinson Syndrome', MS='A syndrome of DYSPHAGIA with IRON-DEFICIENCY ANEMIA that is due to congenital anomalies in the ESOPHAGUS (such as cervical esophageal webs). It is known as Patterson-Kelly syndrome in the United Kingdom.'),
  'D011005': NtO(MH='Plutonium', MS='A naturally radioactive element of the actinide metals series. It has the atomic symbol Pu, and atomic number 94. Plutonium is used as a nuclear fuel, to produce radioisotopes for research, in radionuclide batteries for pacemakers, and as the agent of fission in nuclear weapons.'),
  'D011006': NtO(MH='Pneumatosis Cystoides Intestinalis', MS='A condition characterized by the presence of multiple gas-filled cysts in the intestinal wall, the submucosa and/or subserosa of the INTESTINE. The majority of the cysts are found in the JEJUNUM and the ILEUM.'),
  'D011007': NtO(MH='Pneumocephalus', MS='Presence of air or gas within the intracranial cavity (e.g., epidural space, subdural space, intracerebral, etc.) which may result from traumatic injuries, fistulous tract formation, erosions of the skull from NEOPLASMS or infection, NEUROSURGICAL PROCEDURES, and other conditions.'),
  'D011008': NtO(MH='Pneumococcal Infections', MS='Infections with bacteria of the species STREPTOCOCCUS PNEUMONIAE.'),
  'D011009': NtO(MH='Pneumoconiosis', MS='A diffuse parenchymal lung disease caused by inhalation of dust and by tissue reaction to their presence. These inorganic, organic, particulate, or vaporized matters usually are inhaled by workers in their occupational environment, leading to the various forms (ASBESTOSIS; BYSSINOSIS; and others). Similar air pollution can also have deleterious effects on the general population.'),
  'D011010': NtO(MH='Pneumocystis', MS='A genus of ascomycetous FUNGI, family Pneumocystidaceae, order Pneumocystidales. It includes various host-specific species causing PNEUMOCYSTIS PNEUMONIA in humans and other MAMMALS.'),
  'D011011': NtO(MH='Pneumoencephalography', MS='Radiographic visualization of the cerebral ventricles by injection of air or other gas.'),
  'D011012': NtO(MH='Pneumomediastinum, Diagnostic', MS='Deliberate introduction of air or gas into the mediastinum as an aid to examination and diagnosis.'),
  'D011013': NtO(MH='Pneumonectomy', MS='The excision of lung tissue including partial or total lung lobectomy.'),
  'D011014': NtO(MH='Pneumonia', MS='Infection of the lung often accompanied by inflammation.'),
  'D011015': NtO(MH='Pneumonia, Aspiration', MS='A type of lung inflammation resulting from the aspiration of food, liquid, or gastric contents into the upper RESPIRATORY TRACT.'),
  'D011016': NtO(MH='Pneumonia, Atypical Interstitial, of Cattle', MS='A cattle disease of uncertain cause, probably an allergic reaction.'),
  'D011017': NtO(MH='Pneumonia, Lipid', MS='Pneumonia due to aspiration or inhalation of various oily or fatty substances.'),
  'D011018': NtO(MH='Pneumonia, Pneumococcal', MS='A febrile disease caused by STREPTOCOCCUS PNEUMONIAE.'),
  'D011019': NtO(MH='Pneumonia, Mycoplasma', MS='Interstitial pneumonia caused by extensive infection of the lungs (LUNG) and BRONCHI, particularly the lower lobes of the lungs, by MYCOPLASMA PNEUMONIAE in humans. In SHEEP, it is caused by MYCOPLASMA OVIPNEUMONIAE. In CATTLE, it may be caused by MYCOPLASMA DISPAR.'),
  'D011020': NtO(MH='Pneumonia, Pneumocystis', MS='A pulmonary disease in humans occurring in immunodeficient or malnourished patients or infants, characterized by DYSPNEA, tachypnea, and HYPOXEMIA. Pneumocystis pneumonia is a frequently seen opportunistic infection in AIDS. It is caused by the fungus PNEUMOCYSTIS JIROVECII. The disease is also found in other MAMMALS where it is caused by related species of Pneumocystis.'),
  'D011021': NtO(MH='Pneumonia, Progressive Interstitial, of Sheep', MS='Chronic respiratory disease caused by the VISNA-MAEDI VIRUS. It was formerly believed to be identical with jaagsiekte (PULMONARY ADENOMATOSIS, OVINE) but is now recognized as a separate entity.'),
  'D011022': NtO(MH='Pneumonia, Rickettsial', MS='Pneumonia caused by infection with bacteria of the family RICKETTSIACEAE.'),
  'D011023': NtO(MH='Pneumonia, Staphylococcal', MS='Pneumonia caused by infections with bacteria of the genus STAPHYLOCOCCUS, usually with STAPHYLOCOCCUS AUREUS.'),
  'D011024': NtO(MH='Pneumonia, Viral', MS='Inflammation of the lung parenchyma that is caused by a viral infection.'),
  'D011025': NtO(MH='Pneumonolysis', MS='Division of the tissues attaching the lung to the wall of the chest cavity, to permit collapse of the lung. It was formerly used to treat tuberculosis. (Dorland, 28th ed)'),
  'D011026': NtO(MH='Pneumopericardium', MS='Presence of air or gas in the space between the heart and the PERICARDIUM. The degree of respiratory distress depends on the amount of trapped air and circulation blocked in the systemic and pulmonary veins.'),
  'D011027': NtO(MH='Pneumoperitoneum', MS='A condition with trapped gas or air in the PERITONEAL CAVITY, usually secondary to perforation of the internal organs such as the LUNG and the GASTROINTESTINAL TRACT, or to recent surgery. Pneumoperitoneum may be purposely introduced to aid radiological examination.'),
  'D011028': NtO(MH='Pneumoperitoneum, Artificial', MS='Deliberate introduction of air into the peritoneal cavity.'),
  'D011029': NtO(MH='Pneumoradiography', MS='Radiography using air, oxygen, or some other gas as a contrast medium.'),
  'D011030': NtO(MH='Pneumothorax', MS='An accumulation of air or gas in the PLEURAL CAVITY, which may occur spontaneously or as a result of trauma or a pathological process. The gas may also be introduced deliberately during PNEUMOTHORAX, ARTIFICIAL.'),
  'D011031': NtO(MH='Pneumothorax, Artificial', MS='Injection of air or a more slowly absorbed gas such as nitrogen, into the PLEURAL CAVITY to collapse the lung.'),
  'D011032': NtO(MH='Podiatry', MS='A specialty concerned with the diagnosis and treatment of foot disorders and injuries and anatomic defects of the foot.'),
  'D011033': NtO(MH='Podophyllin', MS='Caustic extract from the roots of Podophyllum peltatum and P. emodi. It contains PODOPHYLLOTOXIN and its congeners and is very irritating to mucous membranes and skin. Podophyllin is a violent purgative that may cause CNS damage and teratogenesis. It is used as a paint for warts, skin neoplasms, and senile keratoses.'),
  'D011034': NtO(MH='Podophyllotoxin', MS='A lignan (LIGNANS) found in PODOPHYLLIN resin from the roots of PODOPHYLLUM plants. It is a potent spindle poison, toxic if taken internally, and has been used as a cathartic. It is very irritating to skin and mucous membranes, has keratolytic actions, has been used to treat warts and keratoses, and may have antineoplastic properties, as do some of its congeners and derivatives.'),
  'D011035': NtO(MH='Podophyllum', MS='A genus of poisonous American herbs, family BERBERIDACEAE. The roots yield PODOPHYLLOTOXIN and other pharmacologically important agents. The plant was formerly used as a cholagogue and cathartic. It is different from the European mandrake, MANDRAGORA.'),
  'D011036': NtO(MH='Poecilia', MS='A genus of livebearing cyprinodont fish comprising the guppy and molly. Some species are virtually all female and depend on sperm from other species to stimulate egg development. Poecilia is used in carcinogenicity studies as well as neurologic and physiologic research.'),
  'D011037': NtO(MH='Poetry as Topic', MS='Works about the literary and oral genre expressing meaning via symbolism and following formal or informal patterns.'),
  'D011038': NtO(MH='Rothmund-Thomson Syndrome', MS='An autosomal recessive syndrome occurring principally in females, characterized by the presence of reticulated, atrophic, hyperpigmented, telangiectatic cutaneous plaques, often accompanied by juvenile cataracts, saddle nose, congenital bone defects, disturbances in the growth of HAIR; NAILS; and TEETH; and HYPOGONADISM.'),
  'D011039': NtO(MH='Poison Control Centers', MS='Facilities which provide information concerning poisons and treatment of poisoning in emergencies.'),
  'D011040': NtO(MH='Dermatitis, Toxicodendron', MS='An allergic contact dermatitis caused by exposure to plants of the genus Toxicodendron (formerly Rhus). These include poison ivy, poison oak, and poison sumac, all plants that contain the substance urushiol, a potent skin sensitizing agent. (From Dorland, 27th ed)'),
  'D011041': NtO(MH='Poisoning', MS='A condition or physical state produced by the ingestion, injection, inhalation of or exposure to a deleterious agent.'),
  'D011042': NtO(MH='Poisons', MS='Substances which, when ingested, inhaled, or absorbed, or when applied to, injected into, or developed within the body in relatively small amounts may, by their chemical action, cause damage to structure or disturbance of function. (From Dorland, 27th ed)'),
  'D011043': NtO(MH='Pokeweed Mitogens', MS='Proteins isolated from the roots of the pokeweed, Phytolacca americana, that agglutinate some erythrocytes, stimulate mitosis and antibody synthesis in lymphocytes, and induce activation of plasma cells.'),
  'D011044': NtO(MH='Poland', MS='A country in central Europe, east of Germany. The capital is Warsaw.'),
  'D011045': NtO(MH='Poland Syndrome', MS='A syndrome which is characterized by symbrachydactyly and aplasia of the sternal head of pectoralis major.'),
  'D011048': NtO(MH='Polarography', MS='An electrochemical technique for measuring the current that flows in solution as a function of an applied voltage. The observed polarographic wave, resulting from the electrochemical response, depends on the way voltage is applied (linear sweep or differential pulse) and the type of electrode used. Usually a mercury drop electrode is used.'),
  'D011049': NtO(MH='Social Control Policies', MS='Decisions for determining and guiding present and future objectives from among alternatives.'),
  'D011050': NtO(MH='Policy Making', MS='The decision process by which individuals, groups or institutions establish policies pertaining to plans, programs or procedures.'),
  'D011051': NtO(MH='Poliomyelitis', MS='An acute infectious disease of humans, particularly children, caused by any of three serotypes of human poliovirus (POLIOVIRUS). Usually the infection is limited to the gastrointestinal tract and nasopharynx, and is often asymptomatic. The central nervous system, primarily the spinal cord, may be affected, leading to rapidly progressive paralysis, coarse FASCICULATION and hyporeflexia. Motor neurons are primarily affected. Encephalitis may also occur. The virus replicates in the nervous system, and may cause significant neuronal loss, most notably in the spinal cord. A rare related condition, nonpoliovirus poliomyelitis, may result from infections with nonpoliovirus enteroviruses. (From Adams et al., Principles of Neurology, 6th ed, pp764-5)'),
  'D011052': NtO(MH='Poliomyelitis, Bulbar', MS='A form of paralytic poliomyelitis affecting neurons of the MEDULLA OBLONGATA of the brain stem. Clinical features include impaired respiration, HYPERTENSION, alterations of vasomotor control, and dysphagia. Weakness and atrophy of the limbs and trunk due to spinal cord involvement is usually associated. (From Adams et al., Principles of Neurology, 6th ed, p765)'),
  'D011054': NtO(MH='Poliovirus Vaccine, Inactivated', MS='A suspension of formalin-inactivated poliovirus grown in monkey kidney cell tissue culture and used to prevent POLIOMYELITIS.'),
  'D011055': NtO(MH='Poliovirus Vaccine, Oral', MS='A live vaccine containing attenuated poliovirus, types I, II, and III, grown in monkey kidney cell tissue culture, used for routine immunization of children against polio. This vaccine induces long-lasting intestinal and humoral immunity. Killed vaccine induces only humoral immunity. Oral poliovirus vaccine should not be administered to immunocompromised individuals or their household contacts. (Dorland, 28th ed)'),
  'D011056': NtO(MH='Political Systems', MS='The units based on political theory and chosen by countries under which their governmental power is organized and administered to their citizens.'),
  'D011057': NtO(MH='Politics', MS='Activities concerned with governmental policies, functions, etc.'),
  'D011058': NtO(MH='Pollen', MS='The fertilizing element of plants that contains the male GAMETOPHYTES.'),
  'D011059': NtO(MH='Polonium', MS='A radioactive element that is a member of the chalcogen family. It has the atomic symbol Po, atomic number 84, and the atomic weight of the isotope with the longest half-life (209Po) is 208.98. It decays by alpha-emission.'),
  'D011060': NtO(MH='Poloxalene', MS='A copolymer of polyethylene and polypropylene ether glycol. It is a non-ionic polyol surface-active agent used medically as a fecal softener and in cattle for prevention of bloat.'),
  'D011061': NtO(MH='Poly A', MS='A group of adenine ribonucleotides in which the phosphate residues of each adenine ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D011062': NtO(MH='Polynucleotide Adenylyltransferase', MS='An enzyme that catalyzes the synthesis of polyadenylic acid from ATP. May be due to the action of RNA polymerase (EC 2.7.7.6) or polynucleotide adenylyltransferase (EC 2.7.7.19). EC 2.7.7.19.'),
  'D011063': NtO(MH='Poly A-U', MS='A double-stranded polyribonucleotide comprising polyadenylic and polyuridylic acids.'),
  'D011064': NtO(MH='Poly Adenosine Diphosphate Ribose', MS='A polynucleotide formed from the ADP-RIBOSE moiety of nicotinamide-adenine dinucleotide (NAD) by POLY(ADP-RIBOSE) POLYMERASES.'),
  'D011065': NtO(MH='Poly(ADP-ribose) Polymerases', MS='Enzymes that catalyze the transfer of multiple ADP-RIBOSE groups from nicotinamide-adenine dinucleotide (NAD) onto protein targets, thus building up a linear or branched homopolymer of repeating ADP-ribose units i.e., POLY ADENOSINE DIPHOSPHATE RIBOSE.'),
  'D011066': NtO(MH='Poly C', MS='A group of cytosine ribonucleotides in which the phosphate residues of each cytosine ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D011067': NtO(MH='Poly dA-dT', MS='Polydeoxyribonucleotides made up of deoxyadenine nucleotides and thymine nucleotides. Present in DNA preparations isolated from crab species. Synthetic preparations have been used extensively in the study of DNA.'),
  'D011068': NtO(MH='Poly G', MS='A group of guanine ribonucleotides in which the phosphate residues of each guanine ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D011069': NtO(MH='Poly I', MS='A group of inosine ribonucleotides in which the phosphate residues of each inosine ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D011070': NtO(MH='Poly I-C', MS='Interferon inducer consisting of a synthetic, mismatched double-stranded RNA. The polymer is made of one strand each of polyinosinic acid and polycytidylic acid.'),
  'D011071': NtO(MH='Poly T', MS='A group of thymine nucleotides in which the phosphate residues of each thymine nucleotide act as bridges in forming diester linkages between the deoxyribose moieties.'),
  'D011072': NtO(MH='Poly U', MS='A group of uridine ribonucleotides in which the phosphate residues of each uridine ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D011073': NtO(MH='Polyamines', MS='Amine compounds that consist of carbon chains or rings containing two or more primary amino groups.'),
  'D011074': NtO(MH='Polyanetholesulfonate', MS='A compound originally developed as an anticoagulant, but possessing anticomplement action and lowering the bactericidal action of blood. It is used in vitro to inhibit blood coagulation and as a diagnostic reagent to encourage the growth of pathogens in the blood. It is also used to stabilize colloidal solutions such as milk and gelatin. (From Merck Index, 11th ed)'),
  'D011075': NtO(MH='Polybrominated Biphenyls', MS='Biphenyl compounds which are extensively brominated. Many of these compounds are toxic environmental pollutants.'),
  'D011076': NtO(MH='Polycarboxylate Cement', MS='Water-soluble low-molecular-weight polymers of acrylic or methacrylic acid that form solid, insoluble products when mixed with specially prepared ZnO powder. The resulting cement adheres to dental enamel and is also used as a luting agent.'),
  'D011077': NtO(MH='Polychaeta', MS='A class of marine annelids including sandworms, tube worms, clamworms, and fire worms. It includes also the genus Myxicola infundibulum.'),
  'D011078': NtO(MH='Polychlorinated Biphenyls', MS='Industrial products consisting of a mixture of chlorinated biphenyl congeners and isomers. These compounds are highly lipophilic and tend to accumulate in fat stores of animals. Many of these compounds are considered toxic and potential environmental pollutants.'),
  'D011080': NtO(MH='Polychloroterphenyl Compounds', MS='Compounds consisting of three benzene rings linked to each other in either ortho, meta or para positions and substituted with chlorine atoms.'),
  'D011081': NtO(MH='Polychondritis, Relapsing', MS='An acquired disease of unknown etiology, chronic course, and tendency to recur. It is characterized by inflammation and degeneration of cartilage and can result in deformities such as floppy ear and saddle nose. Loss of cartilage in the respiratory tract can lead to respiratory obstruction.'),
  'D011083': NtO(MH='Polycyclic Compounds', MS='Compounds which contain two or more rings in their structure.'),
  'D011084': NtO(MH='Polycyclic Aromatic Hydrocarbons', MS='Aromatic hydrocarbons that contain extended fused-ring structures.'),
  'D011085': NtO(MH='Polycystic Ovary Syndrome', MS='A complex disorder characterized by infertility, HIRSUTISM; OBESITY; and various menstrual disturbances such as OLIGOMENORRHEA; AMENORRHEA; ANOVULATION. Polycystic ovary syndrome is usually associated with bilateral enlarged ovaries studded with atretic follicles, not with cysts. The term, polycystic ovary, is misleading.'),
  'D011086': NtO(MH='Polycythemia', MS='An increase in the total red cell mass of the blood. (Dorland, 27th ed)'),
  'D011087': NtO(MH='Polycythemia Vera', MS='A myeloproliferative disorder of unknown etiology, characterized by abnormal proliferation of all hematopoietic bone marrow elements and an absolute increase in red cell mass and total blood volume, associated frequently with splenomegaly, leukocytosis, and thrombocythemia. Hematopoiesis is also reactive in extramedullary sites (liver and spleen). In time myelofibrosis occurs.'),
  'D011088': NtO(MH='DNA Ligases', MS='Poly(deoxyribonucleotide):poly(deoxyribonucleotide)ligases. Enzymes that catalyze the joining of preformed deoxyribonucleotides in phosphodiester linkage during genetic processes during repair of a single-stranded break in duplex DNA. The class includes both EC 6.5.1.1 (ATP) and EC 6.5.1.2 (NAD).'),
  'D011089': NtO(MH='Polydeoxyribonucleotides', MS='A group of 13 or more deoxyribonucleotides in which the phosphate residues of each deoxyribonucleotide act as bridges in forming diester linkages between the deoxyribose moieties.'),
  'D011090': NtO(MH='Polyenes', MS='Hydrocarbons with more than one double bond. They are a reduced form of POLYYNES.'),
  'D011091': NtO(MH='Polyesters', MS='Polymers of organic acids and alcohols, with ester linkages--usually polyethylene terephthalate; can be cured into hard plastic, films or tapes, or fibers which can be woven into fabrics, meshes or velours.'),
  'D011092': NtO(MH='Polyethylene Glycols', MS='Polymers of ETHYLENE OXIDE and water, and their ethers. They vary in consistency from liquid to solid depending on the molecular weight indicated by a number following the name. They are used as SURFACTANTS, dispersing agents, solvents, ointment and suppository bases, vehicles, and tablet excipients. Some specific groups are NONOXYNOLS, OCTOXYNOLS, and POLOXAMERS.'),
  'D011093': NtO(MH='Polyethylene Terephthalates', MS='Polyester polymers formed from terephthalic acid or its esters and ethylene glycol. They can be formed into tapes, films or pulled into fibers that are pressed into meshes or woven into fabrics.'),
  'D011094': NtO(MH='Polyethyleneimine', MS='Strongly cationic polymer that binds to certain proteins; used as a marker in immunology, to precipitate and purify enzymes and lipids. Synonyms: aziridine polymer; Epamine; Epomine; ethylenimine polymer; Montrek; PEI; Polymin(e).'),
  'D011095': NtO(MH='Polyethylenes', MS='Synthetic thermoplastics that are tough, flexible, inert, and resistant to chemicals and electrical current. They are often used as biocompatible materials for prostheses and implants.'),
  'D011096': NtO(MH='Polygalacturonase', MS='A cell wall-degrading enzyme found in microorganisms and higher plants. It catalyzes the random hydrolysis of 1,4-alpha-D-galactosiduronic linkages in pectate and other galacturonans. EC 3.2.1.15.'),
  'D011097': NtO(MH='Polygeline', MS='A 3.5 per cent colloidal solution containing urea-cross-linked polymerized peptides. It has a molecular weight of approximately 35,000 and is prepared from gelatin and electrolytes. The polymeric solution is used as a plasma expander.'),
  'D011098': NtO(MH='Polyglactin 910', MS='A polyester used for absorbable sutures & surgical mesh, especially in ophthalmic surgery. 2-Hydroxy-propanoic acid polymer with polymerized hydroxyacetic acid, which forms 3,6-dimethyl-1,4-dioxane-dione polymer with 1,4-dioxane-2,5-dione copolymer of molecular weight about 80,000 daltons.'),
  'D011099': NtO(MH='Polyglutamic Acid', MS='A peptide that is a homopolymer of glutamic acid.'),
  'D011100': NtO(MH='Polyglycolic Acid', MS='A biocompatible polymer used as a surgical suture material.'),
  'D011102': NtO(MH='Polyhydroxyethyl Methacrylate', MS='A biocompatible, hydrophilic, inert gel that is permeable to tissue fluids. It is used as an embedding medium for microscopy, as a coating for implants and prostheses, for contact lenses, as microspheres in adsorption research, etc.'),
  'D011103': NtO(MH='Polyisoprenyl Phosphate Monosaccharides', MS='These compounds function as activated monosaccharide carriers in the biosynthesis of glycoproteins and oligosaccharide phospholipids. Obtained from a nucleoside diphosphate sugar and a polyisoprenyl phosphate.'),
  'D011104': NtO(MH='Polyisoprenyl Phosphate Oligosaccharides', MS='These compounds function as activated glycosyl carriers in the biosynthesis of glycoproteins and glycophospholipids. Include the pyrophosphates.'),
  'D011105': NtO(MH='Polyisoprenyl Phosphate Sugars', MS='Compounds functioning as activated glycosyl carriers in the biosynthesis of glycoproteins and glycophospholipids. They include the polyisoprenyl pyrophosphates.'),
  'D011106': NtO(MH='Polyisoprenyl Phosphates', MS='Phosphoric or pyrophosphoric acid esters of polyisoprenoids.'),
  'D011107': NtO(MH='Polylysine', MS='A peptide which is a homopolymer of lysine.'),
  'D011108': NtO(MH='Polymers', MS='Compounds formed by the joining of smaller, usually repeating, units linked by covalent bonds. These compounds often form large macromolecules (e.g., BIOPOLYMERS; PLASTICS).'),
  'D011109': NtO(MH='Polymethacrylic Acids', MS='Poly-2-methylpropenoic acids. Used in the manufacture of methacrylate resins and plastics in the form of pellets and granules, as absorbent for biological materials and as filters; also as biological membranes and as hydrogens. Synonyms: methylacrylate polymer; poly(methylacrylate); acrylic acid methyl ester polymer.'),
  'D011110': NtO(MH='Polymorphism, Genetic', MS='The regular and simultaneous occurrence in a single interbreeding population of two or more discontinuous genotypes. The concept includes differences in genotypes ranging in size from a single nucleotide site (POLYMORPHISM, SINGLE NUCLEOTIDE) to large nucleotide sequences visible at a chromosomal level.'),
  'D011111': NtO(MH='Polymyalgia Rheumatica', MS='A syndrome in the elderly characterized by proximal joint and muscle pain, high erythrocyte sedimentation rate, and a self-limiting course. Pain is usually accompanied by evidence of an inflammatory reaction. Women are affected twice as commonly as men and Caucasians more frequently than other groups. The condition is frequently associated with GIANT CELL ARTERITIS and some theories pose the possibility that the two diseases arise from a single etiology or even that they are the same entity.'),
  'D011112': NtO(MH='Polymyxin B', MS='A mixture of polymyxins B1 and B2, obtained from BACILLUS POLYMYXA strains. They are basic polypeptides of about eight amino acids and have cationic detergent action on cell membranes. Polymyxin B is used for treatment of infections with gram-negative bacteria, but may be neurotoxic and nephrotoxic.'),
  'D011113': NtO(MH='Polymyxins', MS='Basic lipopeptide antibiotic group obtained from Bacillus polymyxa. They affect the cell membrane by detergent action and may cause neuromuscular and kidney damage. At least eleven different members of the polymyxin group have been identified, each designated by a letter.'),
  'D011114': NtO(MH='Polynesia', MS="The collective name for the islands of the central Pacific Ocean, including the Austral Islands, Cook Islands, Easter Island, HAWAII; NEW ZEALAND; Phoenix Islands, PITCAIRN ISLAND; SAMOA; TONGA; Tuamotu Archipelago, Wake Island, and Wallis and Futuna Islands. Polynesians are of the Caucasoid race, but many are of mixed origin. Polynesia is from the Greek poly, many + nesos, island, with reference to the many islands in the group. (From Webster's New Geographical Dictionary, 1988, p966 & Room, Brewer's Dictionary of Names, 1992, p426)"),
  'D011115': NtO(MH='Polyneuropathies', MS='Diseases of multiple peripheral nerves simultaneously. Polyneuropathies usually are characterized by symmetrical, bilateral distal motor and sensory impairment with a graded increase in severity distally. The pathological processes affecting peripheral nerves include degeneration of the axon, myelin or both. The various forms of polyneuropathy are categorized by the type of nerve affected (e.g., sensory, motor, or autonomic), by the distribution of nerve injury (e.g., distal vs. proximal), by nerve component primarily affected (e.g., demyelinating vs. axonal), by etiology, or by pattern of inheritance.'),
  'D011116': NtO(MH="Polynucleotide 5'-Hydroxyl-Kinase", MS="An enzyme that catalyzes the transfer of a phosphate group to the 5'-terminal hydroxyl groups of DNA and RNA. EC 2.7.1.78."),
  'D011117': NtO(MH='Polyribonucleotide Nucleotidyltransferase', MS='An enzyme of the transferase class that catalyzes the reaction RNA(n+1) and orthophosphate to yield RNA(n) and a nucleoside diphosphate, or the reverse reaction. ADP, IDP, GDP, UDP, and CDP can act as donors in the latter case. (From Dorland, 27th ed) EC 2.7.7.8.'),
  'D011118': NtO(MH='Polynucleotide Ligases', MS='Catalyze the joining of preformed ribonucleotides or deoxyribonucleotides in phosphodiester linkage during genetic processes. EC 6.5.1.'),
  'D011119': NtO(MH='Polynucleotides', MS=None),
  'D011120': NtO(MH='Polyomavirus', MS='A genus of potentially oncogenic viruses of the family POLYOMAVIRIDAE. These viruses are normally present in their natural hosts as latent infections. The virus is oncogenic in hosts different from the species of origin.'),
  'D011121': NtO(MH='Polyphloretin Phosphate', MS='A polymeric mixture of polyesters of phosphoric acid and phloretin. It blocks some cellular responses to prostaglandins.'),
  'D011122': NtO(MH='Polyphosphates', MS='Linear polymers in which orthophosphate residues are linked with energy-rich phosphoanhydride bonds. They are found in plants, animals, and microorganisms.'),
  'D011123': NtO(MH='Polyploidy', MS='The chromosomal constitution of a cell containing multiples of the normal number of CHROMOSOMES; includes triploidy (symbol: 3N), tetraploidy (symbol: 4N), etc.'),
  'D011124': NtO(MH='Polyporaceae', MS='A family of bracket fungi, order POLYPORALES, living in decaying plant matter and timber.'),
  'D011125': NtO(MH='Adenomatous Polyposis Coli', MS='A polyposis syndrome due to an autosomal dominant mutation of the APC genes (GENES, APC) on CHROMOSOME 5. The syndrome is characterized by the development of hundreds of ADENOMATOUS POLYPS in the COLON and RECTUM of affected individuals by early adulthood.'),
  'D011126': NtO(MH='Polypropylenes', MS='Propylene or propene polymers. Thermoplastics that can be extruded into fibers, films or solid forms. They are used as a copolymer in plastics, especially polyethylene. The fibers are used for fabrics, filters and surgical sutures.'),
  'D011127': NtO(MH='Polyps', MS='Discrete abnormal tissue masses that protrude into the lumen of the DIGESTIVE TRACT or the RESPIRATORY TRACT. Polyps can be spheroidal, hemispheroidal, or irregular mound-shaped structures attached to the MUCOUS MEMBRANE of the lumen wall either by a stalk, pedunculus, or by a broad base.'),
  'D011128': NtO(MH='Polyradiculopathy', MS='Disease or injury involving multiple SPINAL NERVE ROOTS. Polyradiculitis refers to inflammation of multiple spinal nerve roots.'),
  'D011129': NtO(MH='Polyradiculoneuropathy', MS='Diseases characterized by injury or dysfunction involving multiple peripheral nerves and nerve roots. The process may primarily affect myelin or nerve axons. Two of the more common demyelinating forms are acute inflammatory polyradiculopathy (GUILLAIN-BARRE SYNDROME) and POLYRADICULONEUROPATHY, CHRONIC INFLAMMATORY DEMYELINATING. Polyradiculoneuritis refers to inflammation of multiple peripheral nerves and spinal nerve roots.'),
  'D011130': NtO(MH='RNA Ligase (ATP)', MS="An enzyme that catalyzes the conversion of linear RNA to a circular form by the transfer of the 5'-phosphate to the 3'-hydroxyl terminus. It also catalyzes the covalent joining of two polyribonucleotides in phosphodiester linkage. EC 6.5.1.3."),
  'D011131': NtO(MH='Polyribonucleotides', MS='A group of 13 or more ribonucleotides in which the phosphate residues of each ribonucleotide act as bridges in forming diester linkages between the ribose moieties.'),
  'D011132': NtO(MH='Polyribosomes', MS='A multiribosomal structure representing a linear array of RIBOSOMES held together by messenger RNA; (RNA, MESSENGER); They represent the active complexes in cellular protein synthesis and are able to incorporate amino acids into polypeptides both in vivo and in vitro. (From Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D011133': NtO(MH='Polysaccharide-Lyases', MS='A group of carbon-oxygen lyases. These enzymes catalyze the breakage of a carbon-oxygen bond in polysaccharides leading to an unsaturated product and the elimination of an alcohol. EC 4.2.2.'),
  'D011134': NtO(MH='Polysaccharides', MS=None),
  'D011135': NtO(MH='Polysaccharides, Bacterial', MS='Polysaccharides found in bacteria and in capsules thereof.'),
  'D011136': NtO(MH='Polysorbates', MS='Sorbitan mono-9-octadecanoate poly(oxy-1,2-ethanediyl) derivatives; complex mixtures of polyoxyethylene ethers used as emulsifiers or dispersing agents in pharmaceuticals.'),
  'D011137': NtO(MH='Polystyrenes', MS='Polymerized forms of styrene used as a biocompatible material, especially in dentistry. They are thermoplastic and are used as insulators, for injection molding and casting, as sheets, plates, rods, rigid forms and beads.'),
  'D011138': NtO(MH='Polytetrafluoroethylene', MS='Homopolymer of tetrafluoroethylene. Nonflammable, tough, inert plastic tubing or sheeting; used to line vessels, insulate, protect or lubricate apparatus; also as filter, coating for surgical implants or as prosthetic material. Synonyms: Fluoroflex; Fluoroplast; Ftoroplast; Halon; Polyfene; PTFE; Tetron.'),
  'D011139': NtO(MH='Polythiazide', MS='A thiazide diuretic with actions and uses similar to those of HYDROCHLOROTHIAZIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p826)'),
  'D011140': NtO(MH='Polyurethanes', MS='A group of thermoplastic or thermosetting polymers containing polyisocyanate. They are used as ELASTOMERS, as coatings, as fibers and as foams.'),
  'D011141': NtO(MH='Polyuria', MS='Urination of a large volume of urine with an increase in urinary frequency, commonly seen in diabetes (DIABETES MELLITUS; DIABETES INSIPIDUS).'),
  'D011142': NtO(MH='Polyvinyl Alcohol', MS='A polymer prepared from polyvinyl acetates by replacement of the acetate groups with hydroxyl groups. It is used as a pharmaceutic aid and ophthalmic lubricant as well as in the manufacture of surface coatings artificial sponges, cosmetics, and other products.'),
  'D011143': NtO(MH='Polyvinyl Chloride', MS='A polyvinyl resin used extensively in the manufacture of plastics, including medical devices, tubing, and other packaging. It is also used as a rubber substitute.'),
  'D011144': NtO(MH='Polyvinylpyridine N-Oxide', MS='Inhibits the cytotoxic action of quartz and is reported to have delayed the development of experimental silicosis in rats.'),
  'D011145': NtO(MH='Polyvinyls', MS='POLYMERS derived from the monomer VINYL COMPOUNDS.'),
  'D011146': NtO(MH='Eczema, Dyshidrotic', MS='A recurrent eczematous reaction characterized by the development of vesicular eruptions on the palms and soles, particularly along the sides and between the digits. It is accompanied by pruritus, a burning sensation, and hyperhidrosis. The disease is self-limiting, lasting only a few weeks. (Dorland, 27th ed)'),
  'D011148': NtO(MH='Pongo pygmaeus', MS='A species of orangutan, family HOMINIDAE, found in the forests on the island of Borneo.'),
  'D011149': NtO(MH='Pons', MS='The front part of the hindbrain (RHOMBENCEPHALON) that lies between the MEDULLA and the midbrain (MESENCEPHALON) ventral to the cerebellum. It is composed of two parts, the dorsal and the ventral. The pons serves as a relay station for neural pathways between the CEREBELLUM to the CEREBRUM.'),
  'D011150': NtO(MH='Popliteal Artery', MS='The continuation of the femoral artery coursing through the popliteal fossa; it divides into the anterior and posterior tibial arteries.'),
  'D011151': NtO(MH='Popliteal Cyst', MS='A SYNOVIAL CYST located in the back of the knee, in the popliteal space arising from the semimembranous bursa or the knee joint.'),
  'D011152': NtO(MH='Popliteal Vein', MS='The vein formed by the union of the anterior and posterior tibial veins; it courses through the popliteal space and becomes the femoral vein.'),
  'D011153': NtO(MH='Population', MS='The total number of individuals inhabiting a particular region or area.'),
  'D011154': NtO(MH='Population Characteristics', MS='Qualities and characterization of various types of populations within a social or geographic group, with emphasis on demography, health status, and socioeconomic factors.'),
  'D011155': NtO(MH='Population Control', MS='Includes mechanisms or programs which control the numbers of individuals in a population of humans or animals.'),
  'D011156': NtO(MH='Population Density', MS='Number of individuals in a population relative to space.'),
  'D011157': NtO(MH='Population Dynamics', MS='The pattern of any process, or the interrelationship of phenomena, which affects growth or change within a population.'),
  'D011158': NtO(MH='Population Growth', MS='Increase, over a specific period of time, in the number of individuals living in a country or region.'),
  'D011159': NtO(MH='Population Surveillance', MS='Ongoing scrutiny of a population (general population, study population, target population, etc.), generally using methods distinguished by their practicability, uniformity, and frequently their rapidity, rather than by complete accuracy.'),
  'D011160': NtO(MH='Porfiromycin', MS='Toxic antibiotic of the mitomycin group, obtained from MITOMYCIN and also from Streptomyces ardus and other species. It is proposed as an antineoplastic agent, with some antibiotic properties.'),
  'D011161': NtO(MH='Porifera', MS='The phylum of sponges which are sessile, suspension-feeding, multicellular animals that utilize flagellated cells called choanocytes to circulate water. Most are hermaphroditic. They are probably an early evolutionary side branch that gave rise to no other group of animals. Except for about 150 freshwater species, sponges are marine animals. They are a source of ALKALOIDS; STEROLS; and other complex molecules useful in medicine and biological research.'),
  'D011162': NtO(MH='Porphobilinogen', MS=None),
  'D011163': NtO(MH='Hydroxymethylbilane Synthase', MS='An enzyme that catalyzes the tetrapolymerization of the monopyrrole PORPHOBILINOGEN into the hydroxymethylbilane preuroporphyrinogen (UROPORPHYRINOGENS) in several discrete steps. It is the third enzyme in the 8-enzyme biosynthetic pathway of HEME. In humans, deficiency in this enzyme encoded by HMBS (or PBGD) gene results in a form of neurological porphyria (PORPHYRIA, ACUTE INTERMITTENT). This enzyme was formerly listed as EC 4.3.1.8'),
  'D011164': NtO(MH='Porphyrias', MS='A diverse group of metabolic diseases characterized by errors in the biosynthetic pathway of HEME in the LIVER, the BONE MARROW, or both. They are classified by the deficiency of specific enzymes, the tissue site of enzyme defect, or the clinical features that include neurological (acute) or cutaneous (skin lesions). Porphyrias can be hereditary or acquired as a result of toxicity to the hepatic or erythropoietic marrow tissues.'),
  'D011165': NtO(MH='Porphyrinogens', MS='Colorless reduced precursors of porphyrins in which the pyrrole rings are linked by methylene (-CH2-) bridges.'),
  'D011166': NtO(MH='Porphyrins', MS='A group of compounds containing the porphin structure, four pyrrole rings connected by methine bridges in a cyclic configuration to which a variety of side chains are attached. The nature of the side chain is indicated by a prefix, as uroporphyrin, hematoporphyrin, etc. The porphyrins, in combination with iron, form the heme component in biologically significant compounds such as hemoglobin and myoglobin.'),
  'D011167': NtO(MH='Portacaval Shunt, Surgical', MS='Surgical portasystemic shunt between the portal vein and inferior vena cava.'),
  'D011168': NtO(MH='Portal System', MS='A system of vessels in which blood, after passing through one CAPILLARY BED, is conveyed through a second set of capillaries before it returns to the systemic circulation. It pertains especially to the hepatic portal system.'),
  'D011169': NtO(MH='Portal Vein', MS='A short thick vein formed by union of the superior mesenteric vein and the splenic vein.'),
  'D011170': NtO(MH='Portasystemic Shunt, Surgical', MS='Surgical venous shunt between the portal and systemic circulation to effect decompression of the portal circulation. It is performed primarily in the treatment of bleeding esophageal varices resulting from portal hypertension. Types of shunt include portacaval, splenorenal, mesocaval, splenocaval, left gastric-caval (coronary-caval), portarenal, umbilicorenal, and umbilicocaval.'),
  'D011171': NtO(MH='Portoenterostomy, Hepatic', MS='Operation for BILIARY ATRESIA by anastomosis of the BILE DUCTS into the JEJUNUM or DUODENUM.'),
  'D011172': NtO(MH='Portography', MS='Examination of the portal circulation by the use of X-ray films after injection of radiopaque material.'),
  'D011173': NtO(MH='Portraits as Topic', MS='Works about graphic representations, especially of the face, of real persons, usually posed, living or dead. (From Thesaurus for Graphic Materials II, p540, 1995)'),
  'D011174': NtO(MH='Portugal', MS='A country in southwestern Europe, bordering the North Atlantic Ocean, west of Spain. The capital is Lisbon.'),
  'D011175': NtO(MH='Positive-Pressure Respiration', MS='A method of mechanical ventilation in which pressure is maintained to increase the volume of gas remaining in the lungs at the end of expiration, thus reducing the shunting of blood through the lungs and improving gas exchange.'),
  'D011176': NtO(MH='Post and Core Technique', MS='Use of a metal casting, usually with a post in the pulp or root canal, designed to support and retain an artificial crown.'),
  'D011177': NtO(MH='Postal Service', MS='The functions and activities carried out by the U.S. Postal Service, foreign postal services, and private postal services such as Federal Express.'),
  'D011178': NtO(MH='Postgastrectomy Syndromes', MS='Sequelae of gastrectomy from the second week after operation on. Include recurrent or anastomotic ulcer, postprandial syndromes (DUMPING SYNDROME and late postprandial hypoglycemia), disordered bowel action, and nutritional deficiencies.'),
  'D011180': NtO(MH='Postmortem Changes', MS='Physiological changes that occur in bodies after death.'),
  'D011181': NtO(MH='Postnatal Care', MS='The care provided to women and their NEWBORNS for the first few months following CHILDBIRTH.'),
  'D011182': NtO(MH='Postoperative Care', MS="The period of care beginning when the patient is removed from surgery and aimed at meeting the patient's psychological and physical needs directly after surgery. (From Dictionary of Health Services Management, 2d ed)"),
  'D011183': NtO(MH='Postoperative Complications', MS='Pathologic processes that affect patients after a surgical procedure. They may or may not be related to the disease for which the surgery was done, and they may or may not be direct results of the surgery.'),
  'D011184': NtO(MH='Postoperative Period', MS='The period following a surgical operation.'),
  'D011185': NtO(MH='Postpericardiotomy Syndrome', MS='A nonspecific hypersensitivity reaction caused by TRAUMA to the PERICARDIUM, often following PERICARDIOTOMY. It is characterized by PERICARDIAL EFFUSION; high titers of anti-heart antibodies; low-grade FEVER; LETHARGY; loss of APPETITE; or ABDOMINAL PAIN.'),
  'D011186': NtO(MH='Postphlebitic Syndrome', MS='A condition characterized by a chronically swollen limb, often a leg with stasis dermatitis and ulcerations. This syndrome can appear soon after phlebitis or years later. Postphlebitic syndrome is the result of damaged or incompetent venous valves in the limbs. Distended, tortuous VARICOSE VEINS are usually present. Leg pain may occur after long period of standing.'),
  'D011187': NtO(MH='Posture', MS='The position or physical attitude of the body.'),
  'D011188': NtO(MH='Potassium', MS='An element in the alkali group of metals with an atomic symbol K, atomic number 19, and atomic weight 39.10. It is the chief cation in the intracellular fluid of muscle and other cells. Potassium ion is a strong electrolyte that plays a significant role in the regulation of fluid volume and maintenance of the WATER-ELECTROLYTE BALANCE.'),
  'D011189': NtO(MH='Potassium Chloride', MS='A white crystal or crystalline powder used in BUFFERS; FERTILIZERS; and EXPLOSIVES. It can be used to replenish ELECTROLYTES and restore WATER-ELECTROLYTE BALANCE in treating HYPOKALEMIA.'),
  'D011190': NtO(MH='Potassium Cyanide', MS='A highly poisonous compound that is an inhibitor of many metabolic processes, but has been shown to be an especially potent inhibitor of heme enzymes and hemeproteins. It is used in many industrial processes.'),
  'D011191': NtO(MH='Potassium Deficiency', MS='A condition due to decreased dietary intake of potassium, as in starvation or failure to administer in intravenous solutions, or to gastrointestinal loss in diarrhea, chronic laxative abuse, vomiting, gastric suction, or bowel diversion. Severe potassium deficiency may produce muscular weakness and lead to paralysis and respiratory failure. Muscular malfunction may result in hypoventilation, paralytic ileus, hypotension, muscle twitches, tetany, and rhabomyolysis. Nephropathy from potassium deficit impairs the concentrating mechanism, producing POLYURIA and decreased maximal urinary concentrating ability with secondary POLYDIPSIA. (Merck Manual, 16th ed)'),
  'D011192': NtO(MH='Potassium Dichromate', MS='Chromic acid (H2Cr2O7), dipotassium salt. A compound having bright orange-red crystals and used in dyeing, staining, tanning leather, as bleach, oxidizer, depolarizer for dry cells, etc. Medically it has been used externally as an astringent, antiseptic, and caustic. When taken internally, it is a corrosive poison.'),
  'D011193': NtO(MH='Potassium Iodide', MS='An inorganic compound that is used as a source of iodine in thyrotoxic crisis and in the preparation of thyrotoxic patients for thyroidectomy. (From Dorland, 27th ed)'),
  'D011194': NtO(MH='Potassium Isotopes', MS='Stable potassium atoms that have the same atomic number as the element potassium, but differ in atomic weight. K-41 is a stable potassium isotope.'),
  'D011195': NtO(MH='Potassium Magnesium Aspartate', MS='An intra-extracellular electrolyte exchange agent with a variety of effects.'),
  'D011196': NtO(MH='Potassium Permanganate', MS='Permanganic acid (HMnO4), potassium salt. A highly oxidative, water-soluble compound with purple crystals, and a sweet taste. (From McGraw-Hill Dictionary of Scientific and Technical Information, 4th ed)'),
  'D011197': NtO(MH='Potassium Radioisotopes', MS='Unstable isotopes of potassium that decay or disintegrate emitting radiation. K atoms with atomic weights 37, 38, 40, and 42-45 are radioactive potassium isotopes.'),
  'D011198': NtO(MH='Solanum tuberosum', MS='A plant species of the genus SOLANUM, family SOLANACEAE. The starchy roots are used as food. SOLANINE is found in green parts.'),
  'D011199': NtO(MH='Potentiometry', MS='Solution titration in which the end point is read from the electrode-potential variations with the concentrations of potential determining ions. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D011200': NtO(MH='Poultry', MS='Domesticated birds raised for food. It typically includes CHICKENS; TURKEYS, DUCKS; GEESE; and others.'),
  'D011201': NtO(MH='Poultry Diseases', MS='Diseases of birds which are raised as a source of meat or eggs for human consumption and are usually found in barnyards, hatcheries, etc. The concept is differentiated from BIRD DISEASES which is for diseases of birds not considered poultry and usually found in zoos, parks, and the wild.'),
  'D011202': NtO(MH='Poultry Products', MS='Food products manufactured from poultry.'),
  'D011203': NtO(MH='Poverty', MS='A situation in which the level of living of an individual, family, or group is below the standard of the community. It is often related to a specific income level.'),
  'D011204': NtO(MH='Poverty Areas', MS='City, urban, rural, or suburban areas which are characterized by severe economic deprivation and by accompanying physical and social decay.'),
  'D011205': NtO(MH='Povidone', MS='A polyvinyl polymer of variable molecular weight; used as suspending and dispersing agent and vehicle for pharmaceuticals; also used as blood volume expander.'),
  'D011206': NtO(MH='Povidone-Iodine', MS='An iodinated polyvinyl polymer used as topical antiseptic in surgery and for skin and mucous membrane infections, also as aerosol. The iodine may be radiolabeled for research purposes.'),
  'D011208': NtO(MH='Powders', MS='Substances made up of an aggregation of small particles, as that obtained by grinding or trituration of a solid drug. In pharmacy it is a form in which substances are administered. (From Dorland, 28th ed)'),
  'D011209': NtO(MH='Power, Psychological', MS='The exertion of a strong influence or control over others in a variety of settings--administrative, social, academic, etc.'),
  'D011210': NtO(MH='Power Plants', MS='Units that convert some other form of energy into electrical energy.'),
  'D011211': NtO(MH='Electric Power Supplies', MS='Devices that control the supply of electric current for running electrical equipment.'),
  'D011212': NtO(MH='Poxviridae', MS='A family of double-stranded DNA viruses infecting mammals (including humans), birds and insects. There are two subfamilies: CHORDOPOXVIRINAE, poxviruses of vertebrates, and ENTOMOPOXVIRINAE, poxviruses of insects.'),
  'D011213': NtO(MH='Poxviridae Infections', MS='Virus diseases caused by the POXVIRIDAE.'),
  'D011214': NtO(MH='Practice, Psychological', MS='Performance of an act one or more times, with a view to its fixation or improvement; any performance of an act or behavior that leads to learning.'),
  'D011215': NtO(MH='Practice Management, Dental', MS='The organization and operation of the business aspects of a dental practice.'),
  'D011216': NtO(MH='Practice Management, Medical', MS="The organization and operation of the business aspects of a physician's practice."),
  'D011217': NtO(MH='Practolol', MS='A beta-1 adrenergic antagonist that has been used in the emergency treatment of CARDIAC ARRYTHMIAS.'),
  'D011218': NtO(MH='Prader-Willi Syndrome', MS='An autosomal dominant disorder caused by deletion of the proximal long arm of the paternal chromosome 15 (15q11-q13) or by inheritance of both of the pair of chromosomes 15 from the mother (UNIPARENTAL DISOMY) which are imprinted (GENETIC IMPRINTING) and hence silenced. Clinical manifestations include MENTAL RETARDATION; MUSCULAR HYPOTONIA; HYPERPHAGIA; OBESITY; short stature; HYPOGONADISM; STRABISMUS; and HYPERSOMNOLENCE. (Menkes, Textbook of Child Neurology, 5th ed, p229)'),
  'D011219': NtO(MH='Prajmaline', MS='A derivative of the rauwolfia alkaloid AJMALINE. It is an anti-arrhythmia agent, but may cause liver damage.'),
  'D011220': NtO(MH='Pralidoxime Compounds', MS='Various salts of a quaternary ammonium oxime that reconstitute inactivated acetylcholinesterase, especially at the neuromuscular junction, and may cause neuromuscular blockade. They are used as antidotes to organophosphorus poisoning as chlorides, iodides, methanesulfonates (mesylates), or other salts.'),
  'D011221': NtO(MH='Praseodymium', MS='An element of the rare earth family of metals. It has the atomic symbol Pr, atomic number 59, and atomic weight 140.91.'),
  'D011222': NtO(MH='Prazepam', MS='A benzodiazepine that is used in the treatment of ANXIETY DISORDERS.'),
  'D011223': NtO(MH='Praziquantel', MS='An anthelmintic used in most schistosome and many cestode infestations.'),
  'D011224': NtO(MH='Prazosin', MS='A selective adrenergic alpha-1 antagonist used in the treatment of HEART FAILURE; HYPERTENSION; PHEOCHROMOCYTOMA; RAYNAUD DISEASE; PROSTATIC HYPERTROPHY; and URINARY RETENTION.'),
  'D011225': NtO(MH='Pre-Eclampsia', MS='A complication of PREGNANCY, characterized by a complex of symptoms including maternal HYPERTENSION and PROTEINURIA with or without pathological EDEMA. Symptoms may range between mild and severe. Pre-eclampsia usually occurs after the 20th week of gestation, but may develop before this time in the presence of trophoblastic disease.'),
  'D011226': NtO(MH='Pre-Excitation Syndromes', MS='A group of conditions in which HEART VENTRICLE activation by the atrial impulse is faster than the normal impulse conduction from the SINOATRIAL NODE. In these pre-excitation syndromes, atrial impulses often bypass the ATRIOVENTRICULAR NODE delay and travel via ACCESSORY CONDUCTING PATHWAYS connecting the atrium directly to the BUNDLE OF HIS.'),
  'D011227': NtO(MH='Pre-Excitation, Mahaim-Type', MS='A form of ventricular pre-excitation characterized by a normal PR interval and a long QRS interval with an initial slow deflection (delta wave). In this syndrome, the atrial impulse travel to the ventricle via the MAHAIM FIBERS which connect ATRIOVENTRICULAR NODE directly to the right ventricle wall (NODOVENTRICULAR ACCESSORY PATHWAY) or to the RIGHT BUNDLE BRANCH OF HIS (nodofascicular accessory pathway).'),
  'D011228': NtO(MH='Prealbumin', MS='A tetrameric protein, molecular weight between 50,000 and 70,000, consisting of 4 equal chains, and migrating on electrophoresis in 3 fractions more mobile than serum albumin. Its concentration ranges from 7 to 33 per cent in the serum, but levels decrease in liver disease.'),
  'D011229': NtO(MH='Preanesthetic Medication', MS="Drugs administered before an anesthetic to decrease a patient's anxiety and control the effects of that anesthetic."),
  'D011230': NtO(MH='Precancerous Conditions', MS='Pathological conditions that tend eventually to become malignant.'),
  'D011231': NtO(MH='Preceptorship', MS='Practical experience in medical and health-related services that occurs as part of an educational program wherein the professionally-trained student works outside the academic environment under the supervision of an established professional in the particular field.'),
  'D011232': NtO(MH='Chemical Precipitation', MS='The formation of a solid in a solution as a result of a chemical reaction or the aggregation of soluble substances into complexes large enough to fall out of solution.'),
  'D011233': NtO(MH='Precipitin Tests', MS='Serologic tests in which a positive reaction manifested by visible CHEMICAL PRECIPITATION occurs when a soluble ANTIGEN reacts with its precipitins, i.e., ANTIBODIES that can form a precipitate.'),
  'D011234': NtO(MH='Precipitins', MS='Antibodies which elicit IMMUNOPRECIPITATION when combined with antigen.'),
  'D011235': NtO(MH='Predatory Behavior', MS='Instinctual behavior pattern in which food is obtained by killing and consuming other species.'),
  'D011236': NtO(MH='Prediabetic State', MS='The time period before the development of symptomatic diabetes. For example, certain risk factors can be observed in subjects who subsequently develop INSULIN RESISTANCE as in type 2 diabetes (DIABETES MELLITUS, TYPE 2).'),
  'D011237': NtO(MH='Predictive Value of Tests', MS='In screening and diagnostic tests, the probability that a person with a positive test is a true positive (i.e., has the disease), is referred to as the predictive value of a positive test; whereas, the predictive value of a negative test is the probability that the person with a negative test does not have the disease. Predictive value is related to the sensitivity and specificity of the test.'),
  'D011238': NtO(MH='Prednimustine', MS='Ester of CHLORAMBUCIL and PREDNISOLONE used as a combination alkylating agent and synthetic steroid to treat various leukemias and other neoplasms. It causes gastrointestinal and bone marrow toxicity.'),
  'D011239': NtO(MH='Prednisolone', MS='A glucocorticoid with the general properties of the corticosteroids. It is the drug of choice for all conditions in which routine systemic corticosteroid therapy is indicated, except adrenal deficiency states.'),
  'D011241': NtO(MH='Prednisone', MS='A synthetic anti-inflammatory glucocorticoid derived from CORTISONE. It is biologically inert and converted to PREDNISOLONE in the liver.'),
  'D011243': NtO(MH='Preferred Provider Organizations', MS='Arrangements negotiated between a third-party payer (often a self-insured company or union trust fund) and a group of health-care providers (hospitals and physicians) who furnish services at lower than usual fees, and, in return, receive prompt payment and an expectation of an increased volume of patients.'),
  'D011244': NtO(MH='Pregnadienediols', MS='Doubly unsaturated pregnane derivatives with two hydroxy groups substituted anywhere on the rings or side chains.'),
  'D011245': NtO(MH='Pregnadienes', MS='Pregnane derivatives containing two double bonds anywhere within the ring structures.'),
  'D011246': NtO(MH='Pregnadienetriols', MS='Doubly unsaturated pregnane derivatives substituted with three hydroxy groups anywhere within the ring structure or side chains.'),
  'D011247': NtO(MH='Pregnancy', MS='The status during which female mammals carry their developing young (EMBRYOS or FETUSES) in utero before birth, beginning from FERTILIZATION to BIRTH.'),
  'D011248': NtO(MH='Pregnancy Complications', MS='Conditions or pathological processes associated with pregnancy. They can occur during or after pregnancy, and range from minor discomforts to serious diseases that require medical interventions. They include diseases in pregnant females, and pregnancies in females with diseases.'),
  'D011249': NtO(MH='Pregnancy Complications, Cardiovascular', MS='The co-occurrence of pregnancy and a cardiovascular disease. The disease may precede or follow FERTILIZATION and it may or may not have a deleterious effect on the pregnant woman or FETUS.'),
  'D011250': NtO(MH='Pregnancy Complications, Hematologic', MS='The co-occurrence of pregnancy and a blood disease (HEMATOLOGIC DISEASES) which involves BLOOD CELLS or COAGULATION FACTORS. The hematologic disease may precede or follow FERTILIZATION and it may or may not have a deleterious effect on the pregnant woman or FETUS.'),
  'D011251': NtO(MH='Pregnancy Complications, Infectious', MS='The co-occurrence of pregnancy and an INFECTION. The infection may precede or follow FERTILIZATION.'),
  'D011252': NtO(MH='Pregnancy Complications, Neoplastic', MS='The co-occurrence of pregnancy and NEOPLASMS. The neoplastic disease may precede or follow FERTILIZATION.'),
  'D011253': NtO(MH='Pregnancy in Adolescence', MS='Pregnancy in human adolescent females under the age of 19.'),
  'D011254': NtO(MH='Pregnancy in Diabetics', MS='The state of PREGNANCY in women with DIABETES MELLITUS. This does not include either symptomatic diabetes or GLUCOSE INTOLERANCE induced by pregnancy (DIABETES, GESTATIONAL) which resolves at the end of pregnancy.'),
  'D011255': NtO(MH='Pregnancy Maintenance', MS='Physiological mechanisms that sustain the state of PREGNANCY.'),
  'D011256': NtO(MH='Pregnancy Outcome', MS='Results of conception and ensuing pregnancy, including LIVE BIRTH; STILLBIRTH; or SPONTANEOUS ABORTION. The outcome may follow natural or artificial insemination or any of the various ASSISTED REPRODUCTIVE TECHNIQUES, such as EMBRYO TRANSFER or FERTILIZATION IN VITRO.'),
  'D011257': NtO(MH='Pregnancy Proteins', MS='Proteins produced by organs of the mother or the PLACENTA during PREGNANCY. These proteins may be pregnancy-specific (present only during pregnancy) or pregnancy-associated (present during pregnancy or under other conditions such as hormone therapy or certain malignancies.)'),
  'D011258': NtO(MH='Pregnancy Tests', MS='Tests to determine whether or not an individual is pregnant.'),
  'D011259': NtO(MH='Pregnancy Tests, Immunologic', MS='Methods of detecting pregnancy by examining the levels of human chorionic gonadotropin (HCG) in plasma or urine.'),
  'D011261': NtO(MH='Pregnancy Trimester, First', MS='The beginning third of a human PREGNANCY, from the first day of the last normal menstrual period (MENSTRUATION) through the completion of 14 weeks (98 days) of gestation.'),
  'D011262': NtO(MH='Pregnancy Trimester, Second', MS='The middle third of a human PREGNANCY, from the beginning of the 15th through the 28th completed week (99 to 196 days) of gestation.'),
  'D011263': NtO(MH='Pregnancy Trimester, Third', MS='The last third of a human PREGNANCY, from the beginning of the 29th through the 42nd completed week (197 to 294 days) of gestation.'),
  'D011264': NtO(MH='Pregnancy Trimesters', MS='The three approximately equal periods of a normal human PREGNANCY. Each trimester is about three months or 13 to 14 weeks in duration depending on the designation of the first day of gestation.'),
  'D011266': NtO(MH='Pregnancy-Associated Plasma Protein-A', MS='A product of the PLACENTA, and DECIDUA, secreted into the maternal circulation during PREGNANCY. It has been identified as an IGF binding protein (IGFBP)-4 protease that proteolyzes IGFBP-4 and thus increases IGF bioavailability. It is found also in human FIBROBLASTS, ovarian FOLLICULAR FLUID, and GRANULOSA CELLS. The enzyme is a heterotetramer of about 500-kDa.'),
  'D011268': NtO(MH='Pregnancy-Specific beta 1-Glycoproteins', MS='Glycoproteins with the electrophoretic mobility of BETA-GLOBULINS, secreted by the placental TROPHOBLASTS into the maternal bloodstream during PREGNANCY. They can be detected 18 days after OVULATION and reach 200 mg/ml at the end of pregnancy. They are associated with fetal well-being.'),
  'D011269': NtO(MH='Pregnancy, Abdominal', MS='A type of ectopic pregnancy in which the EMBRYO, MAMMALIAN implants in the ABDOMINAL CAVITY instead of in the ENDOMETRIUM of the UTERUS.'),
  'D011270': NtO(MH='Pregnancy, Animal', MS='The process of bearing developing young (EMBRYOS or FETUSES) in utero in non-human mammals, beginning from FERTILIZATION to BIRTH.'),
  'D011271': NtO(MH='Pregnancy, Ectopic', MS='A potentially life-threatening condition in which EMBRYO IMPLANTATION occurs outside the cavity of the UTERUS. Most ectopic pregnancies (>96%) occur in the FALLOPIAN TUBES, known as TUBAL PREGNANCY. They can be in other locations, such as UTERINE CERVIX; OVARY; and abdominal cavity (PREGNANCY, ABDOMINAL).'),
  'D011272': NtO(MH='Pregnancy, Multiple', MS='The condition of carrying two or more FETUSES simultaneously.'),
  'D011273': NtO(MH='Pregnancy, Prolonged', MS='A term used to describe pregnancies that exceed the upper limit of a normal gestational period. In humans, a prolonged pregnancy is defined as one that extends beyond 42 weeks (294 days) after the first day of the last menstrual period (MENSTRUATION), or birth with gestational age of 41 weeks or more.'),
  'D011274': NtO(MH='Pregnancy, Tubal', MS='The most common (>96%) type of ectopic pregnancy in which the extrauterine EMBRYO IMPLANTATION occurs in the FALLOPIAN TUBE, usually in the ampullary region where FERTILIZATION takes place.'),
  'D011275': NtO(MH='Pregnancy, Unwanted', MS='Pregnancy, usually accidental, that is not desired by the parent or parents.'),
  'D011276': NtO(MH='Pregnanediol', MS='An inactive metabolite of PROGESTERONE by reduction at C5, C3, and C20 position. Pregnanediol has two hydroxyl groups, at 3-alpha and 20-alpha. It is detectable in URINE after OVULATION and is found in great quantities in the pregnancy urine.'),
  'D011277': NtO(MH='Pregnanediones', MS='Pregnane derivatives in which two side-chain methyl groups or two methylene groups in the ring skeleton (or a combination thereof) have been oxidized to keto groups.'),
  'D011278': NtO(MH='Pregnanes', MS='Saturated derivatives of the steroid pregnane. The 5-beta series includes PROGESTERONE and related hormones; the 5-alpha series includes forms generally excreted in the urine.'),
  'D011279': NtO(MH='Pregnanetriol', MS='A metabolite of 17-ALPHA-HYDROXYPROGESTERONE, normally produced in small quantities by the GONADS and the ADRENAL GLANDS, found in URINE. An elevated urinary pregnanetriol is associated with CONGENITAL ADRENAL HYPERPLASIA with a deficiency of STEROID 21-HYDROXYLASE.'),
  'D011280': NtO(MH='Pregnanolone', MS='A pregnane found in the urine of pregnant women and sows. It has anesthetic, hypnotic, and sedative properties.'),
  'D011281': NtO(MH='Pregnatrienes', MS='Pregnane derivatives containing three double bonds in the ring structures.'),
  'D011282': NtO(MH='Pregnenediones', MS='Unsaturated pregnane derivatives containing two keto groups on side chains or ring structures.'),
  'D011283': NtO(MH='Pregnenes', MS='Unsaturated derivatives of PREGNANES.'),
  'D011284': NtO(MH='Pregnenolone', MS='A 21-carbon steroid, derived from CHOLESTEROL and found in steroid hormone-producing tissues. Pregnenolone is the precursor to GONADAL STEROID HORMONES and the adrenal CORTICOSTEROIDS.'),
  'D011285': NtO(MH='Pregnenolone Carbonitrile', MS='A catatoxic steroid and microsomal enzyme inducer having significant effects on the induction of cytochrome P450. It has also demonstrated the potential for protective capability against acetaminophen-induced liver damage.'),
  'D011287': NtO(MH='Prejudice', MS='A preconceived judgment made without factual basis.'),
  'D011288': NtO(MH='Prekallikrein', MS='A plasma protein which is the precursor of kallikrein. Plasma that is deficient in prekallikrein has been found to be abnormal in thromboplastin formation, kinin generation, evolution of a permeability globulin, and plasmin formation. The absence of prekallikrein in plasma leads to Fletcher factor deficiency, a congenital disease.'),
  'D011289': NtO(MH='Preleukemia', MS='Conditions in which the abnormalities in the peripheral blood or bone marrow represent the early manifestations of acute leukemia, but in which the changes are not of sufficient magnitude or specificity to permit a diagnosis of acute leukemia by the usual clinical criteria.'),
  'D011291': NtO(MH='Premarital Examinations', MS='Medical tests taken by couples planning to be married in order to determine presence of genetic and contagious diseases.'),
  'D011292': NtO(MH='Premedication', MS='Preliminary administration of a drug preceding a diagnostic, therapeutic, or surgical procedure. The commonest types of premedication are antibiotics (ANTIBIOTIC PROPHYLAXIS) and anti-anxiety agents. It does not include PREANESTHETIC MEDICATION.'),
  'D011293': NtO(MH='Premenstrual Syndrome', MS='A combination of distressing physical, psychologic, or behavioral changes that occur during the luteal phase of the menstrual cycle. Symptoms of PMS are diverse (such as pain, water-retention, anxiety, cravings, and depression) and they diminish markedly 2 or 3 days after the initiation of menses.'),
  'D011294': NtO(MH='Prenalterol', MS='A partial adrenergic agonist with functional beta 1-receptor specificity and inotropic effect. It is effective in the treatment of acute CARDIAC FAILURE, postmyocardial infarction low-output syndrome, SHOCK, and reducing ORTHOSTATIC HYPOTENSION in the SHY-RAGER SYNDROME.'),
  'D011295': NtO(MH='Prenatal Care', MS='Care provided the pregnant woman in order to prevent complications, and decrease the incidence of maternal and prenatal mortality.'),
  'D011296': NtO(MH='Prenatal Diagnosis', MS='Determination of the nature of a pathological condition or disease in the postimplantation EMBRYO; FETUS; or pregnant female before birth.'),
  'D011297': NtO(MH='Prenatal Exposure Delayed Effects', MS='The consequences of exposing the FETUS in utero to certain factors, such as NUTRITION PHYSIOLOGICAL PHENOMENA; PHYSIOLOGICAL STRESS; DRUGS; RADIATION; and other physical or chemical factors. These consequences are observed later in the offspring after BIRTH.'),
  'D011298': NtO(MH='Feprazone', MS='A pyrazole that has analgesic, anti-inflammatory, and antipyretic properties. It has been used in mild to moderate pain, fever, and inflammation associated with musculoskeletal and joint disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p15)'),
  'D011299': NtO(MH='Prenylamine', MS='A drug formerly used in the treatment of angina pectoris but superseded by less hazardous drugs. Prenylamine depletes myocardial catecholamine stores and has some calcium channel blocking activity. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1406)'),
  'D011300': NtO(MH='Preoperative Care', MS='Care given during the period prior to undergoing surgery when psychological and physical preparations are made according to the special needs of the individual patient. This period spans the time between admission to the hospital to the time the surgery begins. (From Dictionary of Health Services Management, 2d ed)'),
  'D011301': NtO(MH='Preoptic Area', MS='Region of hypothalamus between the ANTERIOR COMMISSURE and OPTIC CHIASM.'),
  'D011302': NtO(MH='Prephenate Dehydratase', MS='An enzyme that catalyzes the conversion of prephenate to phenylpyruvate with the elimination of water and carbon dioxide. In the enteric bacteria this enzyme also possesses chorismate mutase activity, thereby catalyzing the first two steps in the biosynthesis of phenylalanine. EC 4.2.1.51.'),
  'D011303': NtO(MH='Prephenate Dehydrogenase', MS='An enzyme that catalyzes the conversion of prephenate to p-hydroxyphenylpyruvate in the presence of NAD. In the enteric bacteria, this enzyme also possesses chorismate mutase activity, thereby catalyzing the first two steps in the biosynthesis of tyrosine. EC 1.3.1.12.'),
  'D011304': NtO(MH='Presbycusis', MS='Gradual bilateral hearing loss associated with aging that is due to progressive degeneration of cochlear structures and central auditory pathways. Hearing loss usually begins with the high frequencies then progresses to sounds of middle and low frequencies.'),
  'D011305': NtO(MH='Presbyopia', MS='The normal decreasing elasticity of the crystalline lens that leads to loss of accommodation.'),
  'D011306': NtO(MH='Prescription Fees', MS='The charge levied on the consumer for drugs or therapy prescribed under written order of a physician or other health professional.'),
  'D011307': NtO(MH='Drug Prescriptions', MS='Directions written for the obtaining and use of DRUGS.'),
  'D011309': NtO(MH='Preservation, Biological', MS='The process of protecting various samples of biological material.'),
  'D011310': NtO(MH='Preservatives, Pharmaceutical', MS='Substances added to pharmaceutical preparations to protect them from chemical change or microbial action. They include ANTI-BACTERIAL AGENTS and antioxidants.'),
  'D011311': NtO(MH='Pressoreceptors', MS='Receptors in the vascular system, particularly the aorta and carotid sinus, which are sensitive to stretch of the vessel walls.'),
  'D011312': NtO(MH='Pressure', MS='A type of stress exerted uniformly in all directions. Its measure is the force exerted per unit area. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D011313': NtO(MH='Preventive Dentistry', MS='The branch of dentistry concerned with the prevention of disease and the maintenance and promotion of oral health.'),
  'D011314': NtO(MH='Preventive Health Services', MS='Services designed for HEALTH PROMOTION.'),
  'D011315': NtO(MH='Preventive Medicine', MS='A medical specialty primarily concerned with prevention of disease (PRIMARY PREVENTION) and the promotion and preservation of health in the individual.'),
  'D011316': NtO(MH='Preventive Psychiatry', MS='A discipline concerned with the prevention of mental illness and the promotion of mental health.'),
  'D011317': NtO(MH='Priapism', MS='A prolonged painful erection that may lasts hours and is not associated with sexual activity. It is seen in patients with SICKLE CELL ANEMIA, advanced malignancy, spinal trauma; and certain drug treatments.'),
  'D011318': NtO(MH='Prilocaine', MS='A local anesthetic that is similar pharmacologically to LIDOCAINE. Currently, it is used most often for infiltration anesthesia in dentistry.'),
  'D011319': NtO(MH='Primaquine', MS='An aminoquinoline that is given by mouth to produce a radical cure and prevent relapse of vivax and ovale malarias following treatment with a blood schizontocide. It has also been used to prevent transmission of falciparum malaria by those returning to areas where there is a potential for re-introduction of malaria. Adverse effects include anemias and GI disturbances. (From Martindale, The Extra Pharmacopeia, 30th ed, p404)'),
  'D011320': NtO(MH='Primary Health Care', MS='Care which provides integrated, accessible health care services by clinicians who are accountable for addressing a large majority of personal health care needs, developing a sustained partnership with patients, and practicing in the context of family and community. (JAMA 1995;273(3):192)'),
  'D011321': NtO(MH='Primary Nursing', MS='The primary responsibility of one nurse for the planning, evaluation, and care of a patient throughout the course of illness, convalescence, and recovery.'),
  'D011322': NtO(MH='Primary Prevention', MS='Specific practices for the prevention of disease or mental disorders in susceptible individuals or populations. These include HEALTH PROMOTION, including mental health; protective procedures, such as COMMUNICABLE DISEASE CONTROL; and monitoring and regulation of ENVIRONMENTAL POLLUTANTS. Primary prevention is to be distinguished from SECONDARY PREVENTION and TERTIARY PREVENTION.'),
  'D011323': NtO(MH='Primates', MS='An order of mammals consisting of more than 300 species that include LEMURS; LORISIDAE; TARSIERS; MONKEYS; and HOMINIDS. They are characterized by a relatively large brain when compared with other terrestrial mammals, forward-facing eyes, the presence of a CALCARINE SULCUS, and specialized MECHANORECEPTORS in the hands and feet which allow the perception of light touch.'),
  'D011324': NtO(MH='Primidone', MS='A barbiturate derivative that acts as a GABA modulator and anti-epileptic agent. It is partly metabolized to PHENOBARBITAL in the body and owes some of its actions to this metabolite.'),
  'D011325': NtO(MH='Prince Edward Island', MS="An island in the Gulf of St. Lawrence constituting a province of Canada in the eastern part of the country. It is very irregular in shape with many deep inlets. Its capital is Charlottetown. Discovered by the French in 1534 and originally named Ile Saint-Jean, it was renamed in 1799 in honor of Prince Edward, fourth son of George III and future father of Queen Victoria. (From Webster's New Geographical Dictionary, 1988, p981 & Room, Brewer's Dictionary of Names, 1992, p433)"),
  'D011326': NtO(MH="Printers' Marks", MS='Symbols or text that identifies a book as the work of a specific printer.'),
  'D011327': NtO(MH='Printing', MS='Process of reproducing words or images on a physical media.'),
  'D011328': NtO(MH='Prions', MS='Small proteinaceous infectious particles which resist inactivation by procedures that modify NUCLEIC ACIDS and contain an abnormal isoform of a cellular protein which is a major and necessary component. The abnormal (scrapie) isoform is PrPSc (PRPSC PROTEINS) and the cellular isoform PrPC (PRPC PROTEINS). The primary amino acid sequence of the two isoforms is identical. Human diseases caused by prions include CREUTZFELDT-JAKOB SYNDROME; GERSTMANN-STRAUSSLER SYNDROME; and INSOMNIA, FATAL FAMILIAL.'),
  'D011329': NtO(MH='Prisoners', MS=None),
  'D011330': NtO(MH='Prisons', MS='Penal institutions, or places of confinement for prisoners.'),
  'D011331': NtO(MH='Private Practice', MS='Practice of a health profession by an individual, offering services on a person-to-person basis, as opposed to group or partnership practice.'),
  'D011332': NtO(MH='Privatization', MS='Process of shifting publicly controlled services and/or facilities to the private sector.'),
  'D011333': NtO(MH='Pro-Opiomelanocortin', MS='A 30-kDa protein synthesized primarily in the ANTERIOR PITUITARY GLAND and the HYPOTHALAMUS. It is also found in the skin and other peripheral tissues. Depending on species and tissues, POMC is cleaved by PROHORMONE CONVERTASES yielding various active peptides including ACTH; BETA-LIPOTROPIN; ENDORPHINS; MELANOCYTE-STIMULATING HORMONES; and others (GAMMA-LPH; CORTICOTROPIN-LIKE INTERMEDIATE LOBE PEPTIDE; N-terminal peptide of POMC or NPP).'),
  'D011334': NtO(MH='Proactive Inhibition', MS='The state or process hypothesized to account for poorer learning rate for elements later in a series as compared to the learning rate for elements coming earlier in a series.'),
  'D011335': NtO(MH='Proadifen', MS='An inhibitor of drug metabolism and CYTOCHROME P-450 ENZYME SYSTEM activity.'),
  'D011336': NtO(MH='Probability', MS='The study of chance processes or the relative frequency characterizing a chance process.'),
  'D011337': NtO(MH='Probability Learning', MS='Usually refers to the use of mathematical models in the prediction of learning to perform tasks based on the theory of probability applied to responses; it may also refer to the frequency of occurrence of the responses observed in the particular study.'),
  'D011338': NtO(MH='Probability Theory', MS='The branch of mathematics dealing with the purely logical properties of probability. Its theorems underlie most statistical methods. (Last, A Dictionary of Epidemiology, 2d ed)'),
  'D011339': NtO(MH='Probenecid', MS='The prototypical uricosuric agent. It inhibits the renal excretion of organic anions and reduces tubular reabsorption of urate. Probenecid has also been used to treat patients with renal impairment, and, because it reduces the renal tubular excretion of other drugs, has been used as an adjunct to antibacterial therapy.'),
  'D011340': NtO(MH='Problem Solving', MS='A learning situation involving more than one alternative from which a selection is made in order to attain a specific goal.'),
  'D011341': NtO(MH='Probucol', MS='A drug used to lower LDL and HDL cholesterol yet has little effect on serum-triglyceride or VLDL cholesterol. (From Martindale, The Extra Pharmacopoeia, 30th ed, p993).'),
  'D011342': NtO(MH='Procainamide', MS='A class Ia antiarrhythmic drug that is structurally-related to PROCAINE.'),
  'D011343': NtO(MH='Procaine', MS='A local anesthetic of the ester type that has a slow onset and a short duration of action. It is mainly used for infiltration anesthesia, peripheral nerve block, and spinal block. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1016).'),
  'D011344': NtO(MH='Procarbazine', MS="An antineoplastic agent used primarily in combination with mechlorethamine, vincristine, and prednisone (the MOPP protocol) in the treatment of Hodgkin's disease."),
  'D011345': NtO(MH='Fenofibrate', MS='An antilipemic agent which reduces both CHOLESTEROL and TRIGLYCERIDES in the blood.'),
  'D011346': NtO(MH='Prochlorperazine', MS='A phenothiazine antipsychotic used principally in the treatment of NAUSEA; VOMITING; and VERTIGO. It is more likely than CHLORPROMAZINE to cause EXTRAPYRAMIDAL DISORDERS. (From Martindale, The Extra Pharmacopoeia, 30th ed, p612)'),
  'D011347': NtO(MH='Procollagen', MS='A biosynthetic precursor of collagen containing additional amino acid sequences at the amino-terminal and carboxyl-terminal ends of the polypeptide chains.'),
  'D011348': NtO(MH='Procollagen N-Endopeptidase', MS='An extracellular endopeptidase which excises a block of peptides at the amino terminal, nonhelical region of the procollagen molecule with the formation of collagen. Absence or deficiency of the enzyme causes accumulation of procollagen which results in the inherited connective tissue disorder--dermatosparaxis. EC 3.4.24.14.'),
  'D011349': NtO(MH='Proctitis', MS='INFLAMMATION of the MUCOUS MEMBRANE of the RECTUM, the distal end of the large intestine (INTESTINE, LARGE).'),
  'D011350': NtO(MH='Proctocolitis', MS='Inflammation of the RECTUM and the distal portion of the COLON.'),
  'D011351': NtO(MH='Proctoscopy', MS='Endoscopic examination, therapy or surgery of the rectum.'),
  'D011352': NtO(MH='Procyclidine', MS='A muscarinic antagonist that crosses the blood-brain barrier and is used in the treatment of drug-induced extrapyramidal disorders and in parkinsonism.'),
  'D011353': NtO(MH='Prodigiosin', MS="4-Methoxy-5-((5-methyl-4-pentyl-2H-pyrrol-2-ylidene)methyl)- 2,2'-bi-1H-pyrrole. A toxic, bright red tripyrrole pigment from Serratia marcescens and others. It has antibacterial, anticoccidial, antimalarial, and antifungal activities, but is used mainly as a biochemical tool."),
  'D011354': NtO(MH='Prodigiozan', MS='A polysaccharide extracted from Serratia marcescens and other bacteria. It activates enzymatic activity of macrophages and stimulates phagocytic processes.'),
  'D011355': NtO(MH='Prodrugs', MS='A compound that, on administration, must undergo chemical conversion by metabolic processes before becoming the pharmacologically active drug for which it is a prodrug.'),
  'D011356': NtO(MH='Product Labeling', MS='Use of written, printed, or graphic materials upon or accompanying a product or its container or wrapper. It includes purpose, effect, description, directions, hazards, warnings, and other relevant information.'),
  'D011357': NtO(MH='Product Line Management', MS='Management control systems for structuring health care delivery strategies around case types, as in DRGs, or specific clinical services.'),
  'D011358': NtO(MH='Product Surveillance, Postmarketing', MS='Surveillance of drugs, devices, appliances, etc., for efficacy or adverse effects, after they have been released for general sale.'),
  'D011359': NtO(MH='Proestrus', MS='A phase of the ESTROUS CYCLE that precedes ESTRUS. During proestrus, the Graafian follicles undergo maturation.'),
  'D011361': NtO(MH='Professional Competence', MS="The capability to perform the duties of one's profession generally, or to perform a particular professional task, with skill of an acceptable quality."),
  'D011362': NtO(MH='Professional Corporations', MS='Legally authorized corporations owned and managed by one or more professionals (medical, dental, legal) in which the income is ascribed primarily to the professional activities of the owners or stockholders.'),
  'D011363': NtO(MH='Professional Impairment', MS='The inability of a health professional to provide proper professional care of patients due to his or her physical and/or mental disability.'),
  'D011364': NtO(MH='Professional Practice', MS="The use of one's knowledge in a particular profession. It includes, in the case of the field of biomedicine, professional activities related to health care and the actual performance of the duties related to the provision of health care."),
  'D011365': NtO(MH='Professional Practice Location', MS='Geographic area in which a professional person practices; includes primarily physicians and dentists.'),
  'D011366': NtO(MH='Professional Review Organizations', MS='Organizations representing designated geographic areas which have contracts under the PRO program to review the medical necessity, appropriateness, quality, and cost-effectiveness of care received by Medicare beneficiaries. Peer Review Improvement Act, PL 97-248, 1982.'),
  'D011367': NtO(MH='Professional Staff Committees', MS='Committees of professional personnel who have responsibility for determining policies, procedures, and controls related to professional matters in health facilities.'),
  'D011368': NtO(MH='Professional-Family Relations', MS='The interactions between the professional person and the family.'),
  'D011369': NtO(MH='Professional-Patient Relations', MS='Interactions between health personnel and patients.'),
  'D011370': NtO(MH='Proflavine', MS='Topical antiseptic used mainly in wound dressings.'),
  'D011371': NtO(MH='Progeria', MS='An abnormal congenital condition, associated with defects in the LAMIN TYPE A gene, which is characterized by premature aging in children, where all the changes of cell senescence occur. It is manifested by premature greying; hair loss; hearing loss (DEAFNESS); cataracts (CATARACT); ARTHRITIS; OSTEOPOROSIS; DIABETES MELLITUS; atrophy of subcutaneous fat; skeletal hypoplasia; elevated urinary HYALURONIC ACID; and accelerated ATHEROSCLEROSIS. Many affected individuals develop malignant tumors, especially SARCOMA.'),
  'D011372': NtO(MH='Progestins', MS='Compounds that interact with PROGESTERONE RECEPTORS in target tissues to bring about the effects similar to those of PROGESTERONE. Primary actions of progestins, including natural and synthetic steroids, are on the UTERUS and the MAMMARY GLAND in preparation for and in maintenance of PREGNANCY.'),
  'D011374': NtO(MH='Progesterone', MS='The major progestational steroid that is secreted primarily by the CORPUS LUTEUM and the PLACENTA. Progesterone acts on the UTERUS, the MAMMARY GLANDS and the BRAIN. It is required in EMBRYO IMPLANTATION; PREGNANCY maintenance, and the development of mammary tissue for MILK production. Progesterone, converted from PREGNENOLONE, also serves as an intermediate in the biosynthesis of GONADAL STEROID HORMONES and adrenal CORTICOSTEROIDS.'),
  'D011375': NtO(MH='Progesterone Reductase', MS='An enzyme that catalyzes the reduction of a 3 beta-hydroxy-delta(5)-steroid to 3-oxo-delta(4)-steroid in the presence of NAD. It converts pregnenolone to progesterone and dehydroepiandrosterone to androstenedione. EC 1.1.1.145.'),
  'D011376': NtO(MH='Progesterone-Binding Globulin', MS='A glycoprotein migrating as alpha 1-globulin, molecular weight 70,000 to 120,000. The protein, which is present in increased amounts in the plasma during pregnancy, binds mainly progesterone, with other steroids including testosterone competing weakly.'),
  'D011377': NtO(MH='Proglumide', MS='A drug that exerts an inhibitory effect on gastric secretion and reduces gastrointestinal motility. It is used clinically in the drug therapy of gastrointestinal ulcers.'),
  'D011378': NtO(MH='Prognathism', MS='A condition marked by abnormal protrusion of the mandible. (Dorland, 27th ed)'),
  'D011379': NtO(MH='Prognosis', MS="A prediction of the probable outcome of a disease based on a individual's condition and the usual course of the disease as seen in similar situations."),
  'D011380': NtO(MH='Programmed Instructions as Topic', MS='Works about instruction in which learners progress at their own rate using workbooks, textbooks, or electromechanical devices that provide information in discrete steps, test learning at each step, and provide immediate feedback about achievement. (ERIC, Thesaurus of ERIC Descriptors, 1996).'),
  'D011381': NtO(MH='Programming Languages', MS='Specific languages used to prepare computer programs.'),
  'D011382': NtO(MH='Programming, Linear', MS='A technique of operations research for solving certain kinds of problems involving many variables where a best value or set of best values is to be found. It is most likely to be feasible when the quantity to be optimized, sometimes called the objective function, can be stated as a mathematical expression in terms of the various activities within the system, and when this expression is simply proportional to the measure of the activities, i.e., is linear, and when all the restrictions are also linear. It is different from computer programming, although problems using linear programming techniques may be programmed on a computer.'),
  'D011383': NtO(MH='Progressive Patient Care', MS='Organization of medical and nursing care according to the degree of illness and care requirements in the hospital. The elements are intensive care, intermediate care, self-care, long-term care, and organized home care.'),
  'D011384': NtO(MH='Proinsulin', MS='A pancreatic polypeptide of about 110 amino acids, depending on the species, that is the precursor of insulin. Proinsulin, produced by the PANCREATIC BETA CELLS, is comprised sequentially of the N-terminal B-chain, the proteolytically removable connecting C-peptide, and the C-terminal A-chain. It also contains three disulfide bonds, two between A-chain and B-chain. After cleavage at two locations, insulin and C-peptide are the secreted products. Intact proinsulin with low bioactivity also is secreted in small amounts.'),
  'D011385': NtO(MH='Projection', MS='A defense mechanism, operating unconsciously, whereby that which is emotionally unacceptable in the self is rejected and attributed (projected) to others.'),
  'D011386': NtO(MH='Projective Techniques', MS='Techniques to reveal personality attributes by responses to relatively unstructured or ambiguous stimuli.'),
  'D011387': NtO(MH='Prokaryotic Cells', MS='Cells lacking a nuclear membrane so that the nuclear material is either scattered in the cytoplasm or collected in a nucleoid region.'),
  'D011388': NtO(MH='Prolactin', MS='A lactogenic hormone secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR). It is a polypeptide of approximately 23 kD. Besides its major action on lactation, in some species prolactin exerts effects on reproduction, maternal behavior, fat metabolism, immunomodulation and osmoregulation. Prolactin receptors are present in the mammary gland, hypothalamus, liver, ovary, testis, and prostate.'),
  'D011389': NtO(MH='Prolactin Release-Inhibiting Factors', MS='A number of peptides with inhibitory activities on PROLACTIN release have been isolated from the HYPOTHALAMUS, the peripheral nervous system, and the gut. These include SOMATOSTATIN, and peptides derived from POMC and precursor for VASOPRESSIN-ASSOCIATED NEUROPHYSIN. Biogenic amine DOPAMINE is also a potent PIF.'),
  'D011391': NtO(MH='Prolapse', MS='The protrusion of an organ or part of an organ into a natural or artificial orifice.'),
  'D011392': NtO(MH='Proline', MS='A non-essential amino acid that is synthesized from GLUTAMIC ACID. It is an essential component of COLLAGEN and is important for proper functioning of joints and tendons.'),
  'D011393': NtO(MH='Procollagen-Proline Dioxygenase', MS='A mixed-function oxygenase that catalyzes the hydroxylation of a prolyl-glycyl containing peptide, usually in PROTOCOLLAGEN, to a hydroxyprolylglycyl-containing-peptide. The enzyme utilizes molecular OXYGEN with a concomitant oxidative decarboxylation of 2-oxoglutarate to SUCCINATE. The enzyme occurs as a tetramer of two alpha and two beta subunits. The beta subunit of procollagen-proline dioxygenase is identical to the enzyme PROTEIN DISULFIDE-ISOMERASES.'),
  'D011394': NtO(MH='Proline Oxidase', MS='The first enzyme of the proline degradative pathway. It catalyzes the oxidation of proline to pyrroline-5-carboxylic acid in the presence of oxygen and water. The action is not reversible. The specific activity of proline oxidase increases with age. EC 1.5.3.-.'),
  'D011395': NtO(MH='Promazine', MS='A phenothiazine with actions similar to CHLORPROMAZINE but with less antipsychotic activity. It is primarily used in short-term treatment of disturbed behavior and as an antiemetic.'),
  'D011396': NtO(MH='Promedol', MS='A narcotic analgesic similar to MEPERIDINE; it exists in four stereoisomers, two of which, the beta (isopromedol) and the gamma (trimeperidine) are active.'),
  'D011397': NtO(MH='Promegestone', MS='A synthetic progestin which is useful for the study of progestin distribution and progestin tissue receptors, as it is not bound by transcortin and binds to progesterone receptors with a higher association constant than progesterone.'),
  'D011398': NtO(MH='Promethazine', MS='A phenothiazine derivative with histamine H1-blocking, antimuscarinic, and sedative properties. It is used as an antiallergic, in pruritus, for motion sickness and sedation, and also in animals.'),
  'D011399': NtO(MH='Promethium', MS='A radioactive element of the rare earth family of metals. It has the atomic symbol Pm, and atomic number 61. It has been used in the construction of atomic batteries, in the preparation of self-luminous compounds, and as a beta-particle source for thickness gauges.'),
  'D011400': NtO(MH='Prometryne', MS='A triazine used as a selective pre- and post-emergence herbicide. (From Merck Index, 11th ed)'),
  'D011401': NtO(MH='Promoter Regions, Genetic', MS='DNA sequences which are recognized (directly or indirectly) and bound by a DNA-dependent RNA polymerase during the initiation of transcription. Highly conserved sequences within the promoter include the Pribnow box in bacteria and the TATA BOX in eukaryotes.'),
  'D011402': NtO(MH='Pronase', MS='A proteolytic enzyme obtained from Streptomyces griseus.'),
  'D011403': NtO(MH='Pronation', MS='Applies to movements of the forearm in turning the palm backward or downward. When referring to the foot, a combination of eversion and abduction movements in the tarsal and metatarsal joints (turning the foot up and in toward the midline of the body).'),
  'D011405': NtO(MH='Propafenone', MS='An antiarrhythmia agent that is particularly effective in ventricular arrhythmias. It also has weak beta-blocking activity.'),
  'D011406': NtO(MH='Propaganda', MS="The deliberate attempt to influence attitudes and beliefs for furthering one's cause or damaging an opponent's cause."),
  'D011407': NtO(MH='Propane', MS='A three carbon alkane with the formula H3CCH2CH3.'),
  'D011408': NtO(MH='Propanediol Dehydratase', MS='An enzyme that catalyzes the dehydration of 1,2-propanediol to propionaldehyde. EC 4.2.1.28.'),
  'D011409': NtO(MH='Propylene Glycols', MS='Derivatives of propylene glycol (1,2-propanediol). They are used as humectants and solvents in pharmaceutical preparations.'),
  'D011410': NtO(MH='Propanidid', MS='An intravenous anesthetic that has been used for rapid induction of anesthesia and for maintenance of anesthesia of short duration. (From Martindale, The Extra Pharmacopoeia, 30th ed, p918)'),
  'D011411': NtO(MH='Propanil', MS='A chlorinated anilide that is used as an herbicide.'),
  'D011412': NtO(MH='Propanolamines', MS='AMINO ALCOHOLS containing the propanolamine (NH2CH2CHOHCH2) group and its derivatives.'),
  'D011413': NtO(MH='Propantheline', MS='A muscarinic antagonist used as an antispasmodic, in rhinitis, in urinary incontinence, and in the treatment of ulcers. At high doses it has nicotinic effects resulting in neuromuscular blocking.'),
  'D011414': NtO(MH='Properdin', MS='A 53-kDa protein that is a positive regulator of the alternate pathway of complement activation (COMPLEMENT ACTIVATION PATHWAY, ALTERNATIVE). It stabilizes the ALTERNATIVE PATHWAY C3 CONVERTASE (C3bBb) and protects it from rapid inactivation, thus facilitating the cascade of COMPLEMENT ACTIVATION and the formation of MEMBRANE ATTACK COMPLEX. Individuals with mutation in the PFC gene exhibit properdin deficiency and have a high susceptibility to infections.'),
  'D011415': NtO(MH='Complement Factor B', MS='A glycine-rich, heat-labile serum glycoprotein that contains a component of the C3 CONVERTASE ALTERNATE PATHWAY (C3bBb). Bb, a serine protease, is generated when factor B is cleaved by COMPLEMENT FACTOR D into Ba and Bb.'),
  'D011416': NtO(MH='Complement Factor D', MS='A serum protein which is important in the ALTERNATIVE COMPLEMENT ACTIVATION PATHWAY. This enzyme cleaves the COMPLEMENT C3B-bound COMPLEMENT FACTOR B to form C3bBb which is ALTERNATIVE PATHWAY C3 CONVERTASE.'),
  'D011418': NtO(MH='Prophase', MS='The first phase of cell nucleus division, in which the CHROMOSOMES become visible, the CELL NUCLEUS starts to lose its identity, the SPINDLE APPARATUS appears, and the CENTRIOLES migrate toward opposite poles.'),
  'D011419': NtO(MH='Propidium', MS='Quaternary ammonium analog of ethidium; an intercalating dye with a specific affinity to certain forms of DNA and, used as diiodide, to separate them in density gradients; also forms fluorescent complexes with cholinesterase which it inhibits.'),
  'D011420': NtO(MH='Propiolactone', MS='Disinfectant used in vapor form to sterilize vaccines, grafts, etc. The vapor is very irritating and the liquid form is carcinogenic.'),
  'D011422': NtO(MH='Propionates', MS='Derivatives of propionic acid. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the carboxyethane structure.'),
  'D011423': NtO(MH='Propionibacteriaceae', MS='A family of gram-positive bacteria found in dairy products or in the intestinal tracts of animals.'),
  'D011424': NtO(MH='Propionibacterium', MS='A genus of gram-positive, rod-shaped bacteria whose cells occur singly, in pairs or short chains, in V or Y configurations, or in clumps resembling letters of the Chinese alphabet. Its organisms are found in cheese and dairy products as well as on human skin and can occasionally cause soft tissue infections.'),
  'D011425': NtO(MH='Propionibacterium acnes', MS='A bacteria isolated from normal skin, intestinal contents, wounds, blood, pus, and soft tissue abscesses. It is a common contaminant of clinical specimens, presumably from the skin of patients or attendants.'),
  'D011427': NtO(MH='Propiophenones', MS='Propiophenone (ethyl phenyl ketone, structural formula C6H5COCH2CH3) and its derivatives. They are commonly used in perfumes and pharmaceuticals.'),
  'D011428': NtO(MH='Proplast', MS='Polymer of polytetrafluoroethylene and carbon filaments; porous biocompatible material used in orofacial and middle ear reconstruction and as coating for metal implants.'),
  'D011429': NtO(MH='Propolis', MS='A resinous substance obtained from beehives that is used traditionally as an antimicrobial. It is a heterogeneous mixture of many substances.'),
  'D011430': NtO(MH='Propoxycaine', MS='A local anesthetic of the ester type that has a rapid onset of action and a longer duration of action than procaine hydrochloride. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1017)'),
  'D011431': NtO(MH='Dextropropoxyphene', MS='A narcotic analgesic structurally related to METHADONE. Only the dextro-isomer has an analgesic effect; the levo-isomer appears to exert an antitussive effect.'),
  'D011433': NtO(MH='Propranolol', MS='A widely used non-cardioselective beta-adrenergic antagonist. Propranolol has been used for MYOCARDIAL INFARCTION; ARRHYTHMIA; ANGINA PECTORIS; HYPERTENSION; HYPERTHYROIDISM; MIGRAINE; PHEOCHROMOCYTOMA; and ANXIETY but adverse effects instigate replacement by newer drugs.'),
  'D011434': NtO(MH='Proprioception', MS="Sensory functions that transduce stimuli received by proprioceptive receptors in joints, tendons, muscles, and the INNER EAR into neural impulses to be transmitted to the CENTRAL NERVOUS SYSTEM. Proprioception provides sense of stationary positions and movements of one's body parts, and is important in maintaining KINESTHESIA and POSTURAL BALANCE."),
  'D011435': NtO(MH='Propyl Gallate', MS='Antioxidant for foods, fats, oils, ethers, emulsions, waxes, and transformer oils.'),
  'D011436': NtO(MH='Spermidine Synthase', MS="An enzyme that catalyzes the transfer of the propylamine moiety from 5'-deoxy-5'-S-(3-methylthiopropylamine)sulfonium adenosine to putrescine in the biosynthesis of spermidine. The enzyme has a molecular weight of approximately 73,000 kDa and is composed of two subunits of equal size."),
  'D011437': NtO(MH='Propylamines', MS='Derivatives of propylamine (the structural formula NH2CH2CH2CH3).'),
  'D011438': NtO(MH='Propylbenzilylcholine Mustard', MS='An analog of benzilylcholine mustard. It is an alkylating nitrogen mustard analog that binds specifically and irreversibly to cholinergic muscarinic receptors and is used as an affinity label to isolate and study the receptors.'),
  'D011440': NtO(MH='Propyliodone', MS='Radiopaque medium usually in oil; used in bronchography.'),
  'D011441': NtO(MH='Propylthiouracil', MS='A thiourea antithyroid agent. Propythiouracil inhibits the synthesis of thyroxine and inhibits the peripheral conversion of throxine to tri-iodothyronine. It is used in the treatment of hyperthyroidism. (From Martindale, The Extra Pharmacopeoia, 30th ed, p534)'),
  'D011442': NtO(MH='Proscillaridin', MS='A cardiotonic glycoside isolated from Scilla maritima var. alba (Squill).'),
  'D011443': NtO(MH='Strepsirhini', MS='A suborder of PRIMATES consisting of the following five families: CHEIROGALEIDAE; Daubentoniidae; Indriidae; LEMURIDAE; and LORISIDAE.'),
  'D011444': NtO(MH='Prospective Payment Assessment Commission', MS='The commission charged with evaluating issues and factors which affect the implementation of the PROSPECTIVE PAYMENT SYSTEM.'),
  'D011445': NtO(MH='Prospective Payment System', MS='A system wherein reimbursement rates are set, for a given period of time, prior to the circumstances giving rise to actual reimbursement claims.'),
  'D011446': NtO(MH='Prospective Studies', MS='Observation of a population for a sufficient number of persons over a sufficient number of years to generate incidence or mortality rates subsequent to the selection of the study group.'),
  'D011447': NtO(MH='Prospidium', MS='An antineoplastic dispiropiperazine derivative.'),
  'D011448': NtO(MH='Prostaglandin Antagonists', MS='Compounds that inhibit the action of prostaglandins.'),
  'D011449': NtO(MH='Prostaglandin Endoperoxides', MS='Precursors in the biosynthesis of prostaglandins and thromboxanes from arachidonic acid. They are physiologically active compounds, having effect on vascular and airway smooth muscles, platelet aggregation, etc.'),
  'D011450': NtO(MH='Prostaglandin Endoperoxides, Synthetic', MS='Synthetic compounds that are analogs of the naturally occurring prostaglandin endoperoxides and that mimic their pharmacologic and physiologic activities. They are usually more stable than the naturally occurring compounds.'),
  'D011451': NtO(MH='Prostaglandin-Endoperoxide Synthases', MS='Enzyme complexes that catalyze the formation of PROSTAGLANDINS from the appropriate unsaturated FATTY ACIDS, molecular OXYGEN, and a reduced acceptor.'),
  'D011452': NtO(MH='15-Oxoprostaglandin 13-Reductase', MS='(5Z)-(15S)-11 alpha-Hydroxy-9,15-dioxoprostanoate:NAD(P)+ delta(13)-oxidoreductase. An enzyme active in prostaglandin E and F catabolism. It catalyzes the reduction of the double bond at the 13-14 position of the 15-ketoprostaglandins and uses NADPH as cofactor. EC 1.3.1.48.'),
  'D011453': NtO(MH='Prostaglandins', MS='A group of compounds derived from unsaturated 20-carbon fatty acids, primarily arachidonic acid, via the cyclooxygenase pathway. They are extremely potent mediators of a diverse group of physiological processes.'),
  'D011454': NtO(MH='Prostaglandins A', MS='(13E,15S)-15-Hydroxy-9-oxoprosta-10,13-dien-1-oic acid (PGA(1)); (5Z,13E,15S)-15-hydroxy-9-oxoprosta-5,10,13-trien-1-oic acid (PGA(2)); (5Z,13E,15S,17Z)-15-hydroxy-9-oxoprosta-5,10,13,17-tetraen-1-oic acid (PGA(3)). A group of naturally occurring secondary prostaglandins derived from PGE; PGA(1) and PGA(2) as well as their 19-hydroxy derivatives are found in many organs and tissues.'),
  'D011455': NtO(MH='Prostaglandins A, Synthetic', MS='Analogs or derivatives of prostaglandin A that do not occur naturally in the body. They do not include the product of the chemical synthesis of hormonal PGA.'),
  'D011456': NtO(MH='Prostaglandins B', MS='Physiologically active prostaglandins found in many tissues and organs. They are potent pressor substances and have many other physiological activities.'),
  'D011457': NtO(MH='Prostaglandins D', MS='Physiologically active prostaglandins found in many tissues and organs. They show pressor activity, are mediators of inflammation, and have potential antithrombotic effects.'),
  'D011458': NtO(MH='Prostaglandins E', MS='(11 alpha,13E,15S)-11,15-Dihydroxy-9-oxoprost-13-en-1-oic acid (PGE(1)); (5Z,11 alpha,13E,15S)-11,15-dihydroxy-9-oxoprosta-5,13-dien-1-oic acid (PGE(2)); and (5Z,11 alpha,13E,15S,17Z)-11,15-dihydroxy-9-oxoprosta-5,13,17-trien-1-oic acid (PGE(3)). Three of the six naturally occurring prostaglandins. They are considered primary in that no one is derived from another in living organisms. Originally isolated from sheep seminal fluid and vesicles, they are found in many organs and tissues and play a major role in mediating various physiological activities.'),
  'D011459': NtO(MH='Prostaglandins E, Synthetic', MS='Analogs or derivatives of prostaglandins E that do not occur naturally in the body. They do not include the product of the chemical synthesis of hormonal PGE.'),
  'D011460': NtO(MH='Prostaglandins F', MS='(9 alpha,11 alpha,13E,15S)-9,11,15-Trihydroxyprost-13-en-1-oic acid (PGF(1 alpha)); (5Z,9 alpha,11,alpha,13E,15S)-9,11,15-trihydroxyprosta-5,13-dien-1-oic acid (PGF(2 alpha)); (5Z,9 alpha,11 alpha,13E,15S,17Z)-9,11,15-trihydroxyprosta-5,13,17-trien-1-oic acid (PGF(3 alpha)). A family of prostaglandins that includes three of the six naturally occurring prostaglandins. All naturally occurring PGF have an alpha configuration at the 9-carbon position. They stimulate uterine and bronchial smooth muscle and are often used as oxytocics.'),
  'D011461': NtO(MH='Prostaglandins F, Synthetic', MS='Analogs or derivatives of prostaglandins F that do not occur naturally in the body. They do not include the product of the chemical synthesis of hormonal PGF.'),
  'D011462': NtO(MH='Prostaglandins G', MS='A group of physiologically active prostaglandin endoperoxides. They are precursors in the biosynthesis of prostaglandins and thromboxanes. Most frequently encountered member of this group is the prostaglandin G2.'),
  'D011463': NtO(MH='Prostaglandins H', MS='A group of physiologically active prostaglandin endoperoxides. They are precursors in the biosynthesis of prostaglandins and thromboxanes. The most frequently encountered member of this group is the prostaglandin H2.'),
  'D011464': NtO(MH='Epoprostenol', MS='A prostaglandin that is a powerful vasodilator and inhibits platelet aggregation. It is biosynthesized enzymatically from PROSTAGLANDIN ENDOPEROXIDES in human vascular tissue. The sodium salt has been also used to treat primary pulmonary hypertension (HYPERTENSION, PULMONARY).'),
  'D011465': NtO(MH='Prostaglandins, Synthetic', MS='Compounds obtained by chemical synthesis that are analogs or derivatives of naturally occurring prostaglandins and that have similar activity.'),
  'D011466': NtO(MH='Prostanoic Acids', MS='2-Octylcyclopentaneheptanoic acids. The family of saturated carbon-20 cyclic fatty acids that represent the parent compounds of the prostaglandins.'),
  'D011467': NtO(MH='Prostate', MS='A gland in males that surrounds the neck of the URINARY BLADDER and the URETHRA. It secretes a substance that liquefies coagulated semen. It is situated in the pelvic cavity behind the lower part of the PUBIC SYMPHYSIS, above the deep layer of the triangular ligament, and rests upon the RECTUM.'),
  'D011468': NtO(MH='Prostatectomy', MS='Complete or partial surgical removal of the prostate. Three primary approaches are commonly employed: suprapubic - removal through an incision above the pubis and through the urinary bladder; retropubic - as for suprapubic but without entering the urinary bladder; and transurethral (TRANSURETHRAL RESECTION OF PROSTATE).'),
  'D011469': NtO(MH='Prostatic Diseases', MS='Pathological processes involving the PROSTATE or its component tissues.'),
  'D011470': NtO(MH='Prostatic Hyperplasia', MS='Increase in constituent cells in the PROSTATE, leading to enlargement of the organ (hypertrophy) and adverse impact on the lower urinary tract function. This can be caused by increased rate of cell proliferation, reduced rate of cell death, or both.'),
  'D011471': NtO(MH='Prostatic Neoplasms', MS='Tumors or cancer of the PROSTATE.'),
  'D011472': NtO(MH='Prostatitis', MS='Infiltration of inflammatory cells into the parenchyma of PROSTATE. The subtypes are classified by their varied laboratory analysis, clinical presentation and response to treatment.'),
  'D011474': NtO(MH='Prosthesis Design', MS='The plan and delineation of prostheses in general or a specific prosthesis.'),
  'D011475': NtO(MH='Prosthesis Failure', MS='Malfunction of implantation shunts, valves, etc., and prosthesis loosening, migration, and breaking.'),
  'D011476': NtO(MH='Prosthodontics', MS='A dental specialty concerned with the restoration and maintenance of oral function by the replacement of missing TEETH and related structures by artificial devices or DENTAL PROSTHESES.'),
  'D011477': NtO(MH='Sex Work', MS='The act or practice of engaging in sexual relations for money or other benefit.'),
  'D011478': NtO(MH='Protactinium', MS='A radioactive element of the actinide group of metals. It has the atomic symbol Pa, atomic number 91, and atomic weight 231. It decays by alpha-emission.'),
  'D011479': NtO(MH='Protamines', MS='A group of simple proteins that yield basic amino acids on hydrolysis and that occur combined with nucleic acid in the sperm of fish. Protamines contain very few kinds of amino acids. Protamine sulfate combines with heparin to form a stable inactive complex; it is used to neutralize the anticoagulant action of heparin in the treatment of heparin overdose. (From Merck Index, 11th ed; Martindale, The Extra Pharmacopoeia, 30th ed, p692)'),
  'D011480': NtO(MH='Protease Inhibitors', MS='Compounds which inhibit or antagonize biosynthesis or actions of proteases (ENDOPEPTIDASES).'),
  'D011481': NtO(MH='Protective Clothing', MS='Clothing designed to protect the individual against possible exposure to known hazards.'),
  'D011482': NtO(MH='Protective Devices', MS='Devices designed to provide protection against injury.'),
  'D011483': NtO(MH='Proteidae', MS='The Proteidae family of permanently larval aquatic salamanders. It consists of two living genera - Necturus (mudpuppy) of the eastern United States and Proteus (the European olm).'),
  'D011484': NtO(MH='Protein-Arginine N-Methyltransferases', MS='Enzymes that catalyze the methylation of arginine residues of proteins to yield N-mono- and N,N-dimethylarginine. This enzyme is found in many organs, primarily brain and spleen.'),
  'D011485': NtO(MH='Protein Binding', MS='The process in which substances, either endogenous or exogenous, bind to proteins, peptides, enzymes, protein precursors, or allied compounds. Specific protein-binding measures are often used as assays in diagnostic assessments.'),
  'D011486': NtO(MH='Protein C', MS='A vitamin-K dependent zymogen present in the blood, which, upon activation by thrombin and thrombomodulin exerts anticoagulant properties by inactivating factors Va and VIIIa at the rate-limiting steps of thrombin formation.'),
  'D011487': NtO(MH='Protein Conformation', MS='The characteristic 3-dimensional shape of a protein, including the secondary, supersecondary (motifs), tertiary (domains) and quaternary structure of the peptide chain. PROTEIN STRUCTURE, QUATERNARY describes the conformation assumed by multimeric proteins (aggregates of more than one polypeptide chain).'),
  'D011488': NtO(MH='Protein Deficiency', MS="A nutritional condition produced by a deficiency of proteins in the diet, characterized by adaptive enzyme changes in the liver, increase in amino acid synthetases, and diminution of urea formation, thus conserving nitrogen and reducing its loss in the urine. Growth, immune response, repair, and production of enzymes and hormones are all impaired in severe protein deficiency. Protein deficiency may also arise in the face of adequate protein intake if the protein is of poor quality (i.e., the content of one or more amino acids is inadequate and thus becomes the limiting factor in protein utilization). (From Merck Manual, 16th ed; Harrison's Principles of Internal Medicine, 12th ed, p406)"),
  'D011489': NtO(MH='Protein Denaturation', MS='Disruption of the non-covalent bonds and/or disulfide bonds responsible for maintaining the three-dimensional shape and activity of the native protein.'),
  'D011490': NtO(MH='Protein Disulfide Reductase (Glutathione)', MS='An enzyme that catalyzes the reduction of a protein-disulfide in the presence of glutathione, forming a protein-dithiol. Insulin is one of its substrates. EC 1.8.4.2.'),
  'D011492': NtO(MH='Protein Hydrolysates', MS=None),
  'D011493': NtO(MH='Protein Kinase C', MS='An serine-threonine protein kinase that requires the presence of physiological concentrations of CALCIUM and membrane PHOSPHOLIPIDS. The additional presence of DIACYLGLYCEROLS markedly increases its sensitivity to both calcium and phospholipids. The sensitivity of the enzyme can also be increased by PHORBOL ESTERS and it is believed that protein kinase C is the receptor protein of tumor-promoting phorbol esters.'),
  'D011494': NtO(MH='Protein Kinases', MS='A family of enzymes that catalyze the conversion of ATP and a protein to ADP and a phosphoprotein.'),
  'D011495': NtO(MH='Histone-Lysine N-Methyltransferase', MS='An enzyme that catalyzes the methylation of the epsilon-amino group of lysine residues in proteins to yield epsilon mono-, di-, and trimethyllysine.'),
  'D011496': NtO(MH='Protein Methyltransferases', MS='Enzymes that catalyze the methylation of amino acids after their incorporation into a polypeptide chain. S-Adenosyl-L-methionine acts as the methylating agent. EC 2.1.1.'),
  'D011497': NtO(MH='Protein O-Methyltransferase', MS='An enzyme that catalyzes the transfer of methyl groups from S-adenosylmethionine to free carboxyl groups of a protein molecule forming methyl esters. EC 2.1.1.-.'),
  'D011498': NtO(MH='Protein Precursors', MS=None),
  'D011499': NtO(MH='Protein Processing, Post-Translational', MS='Any of various enzymatically catalyzed post-translational modifications of PEPTIDES or PROTEINS in the cell of origin. These modifications include carboxylation; HYDROXYLATION; ACETYLATION; PHOSPHORYLATION; METHYLATION; GLYCOSYLATION; ubiquitination; oxidation; proteolysis; and crosslinking and result in changes in molecular weight and electrophoretic motility.'),
  'D011500': NtO(MH='Protein Synthesis Inhibitors', MS='Compounds which inhibit the synthesis of proteins. They are usually ANTI-BACTERIAL AGENTS or toxins. Mechanism of the action of inhibition includes the interruption of peptide-chain elongation, the blocking the A site of ribosomes, the misreading of the genetic code or the prevention of the attachment of oligosaccharide side chains to glycoproteins.'),
  'D011502': NtO(MH='Protein-Energy Malnutrition', MS="The lack of sufficient energy or protein to meet the body's metabolic demands, as a result of either an inadequate dietary intake of protein, intake of poor quality dietary protein, increased demands due to disease, or increased nutrient losses."),
  'D011503': NtO(MH='Transglutaminases', MS='Transglutaminases catalyze cross-linking of proteins at a GLUTAMINE in one chain with LYSINE in another chain. They include keratinocyte transglutaminase (TGM1 or TGK), tissue transglutaminase (TGM2 or TGC), plasma transglutaminase involved with coagulation (FACTOR XIII and FACTOR XIIIa), hair follicle transglutaminase, and prostate transglutaminase. Although structures differ, they share an active site (YGQCW) and strict CALCIUM dependence.'),
  'D011504': NtO(MH='Protein-Losing Enteropathies', MS="Pathological conditions in the INTESTINES that are characterized by the gastrointestinal loss of serum proteins, including SERUM ALBUMIN; IMMUNOGLOBULINS; and at times LYMPHOCYTES. Severe condition can result in HYPOGAMMAGLOBULINEMIA or LYMPHOPENIA. Protein-losing enteropathies are associated with a number of diseases including INTESTINAL LYMPHANGIECTASIS; WHIPPLE'S DISEASE; and NEOPLASMS of the SMALL INTESTINE."),
  'D011505': NtO(MH='Protein-Tyrosine Kinases', MS='Protein kinases that catalyze the PHOSPHORYLATION of TYROSINE residues in proteins with ATP or other nucleotides as phosphate donors.'),
  'D011506': NtO(MH='Proteins', MS='Linear POLYPEPTIDES that are synthesized on RIBOSOMES and may be further modified, crosslinked, cleaved, or assembled into complex proteins with several subunits. The specific sequence of AMINO ACIDS determines the shape the polypeptide will take, during PROTEIN FOLDING, and the function of the protein.'),
  'D011507': NtO(MH='Proteinuria', MS='The presence of proteins in the urine, an indicator of KIDNEY DISEASES.'),
  'D011508': NtO(MH='Chondroitin Sulfate Proteoglycans', MS='Proteoglycans consisting of proteins linked to one or more CHONDROITIN SULFATE-containing oligosaccharide chains.'),
  'D011509': NtO(MH='Proteoglycans', MS='Glycoproteins which have a very high polysaccharide content.'),
  'D011510': NtO(MH='Proteolipids', MS='Protein-lipid combinations abundant in brain tissue, but also present in a wide variety of animal and plant tissues. In contrast to lipoproteins, they are insoluble in water, but soluble in a chloroform-methanol mixture. The protein moiety has a high content of hydrophobic amino acids. The associated lipids consist of a mixture of GLYCEROPHOSPHATES; CEREBROSIDES; and SULFOGLYCOSPHINGOLIPIDS; while lipoproteins contain PHOSPHOLIPIDS; CHOLESTEROL; and TRIGLYCERIDES.'),
  'D011511': NtO(MH='Proteus', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs in the intestines of humans and a wide variety of animals, as well as in manure, soil, and polluted waters. Its species are pathogenic, causing urinary tract infections and are also considered secondary invaders, causing septic lesions at other sites of the body.'),
  'D011512': NtO(MH='Proteus Infections', MS='Infections with bacteria of the genus PROTEUS.'),
  'D011513': NtO(MH='Proteus mirabilis', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that is frequently isolated from clinical specimens. Its most common site of infection is the urinary tract.'),
  'D011514': NtO(MH='Proteus vulgaris', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs in soil, fecal matter, and sewage. It is an opportunistic pathogen and causes cystitis and pyelonephritis.'),
  'D011515': NtO(MH='Prothionamide', MS='Antitubercular agent similar in action and side effects to ETHIONAMIDE. It is used mostly in combination with other agents.'),
  'D011516': NtO(MH='Prothrombin', MS='A plasma protein that is the inactive precursor of thrombin. It is converted to thrombin by a prothrombin activator complex consisting of factor Xa, factor V, phospholipid, and calcium ions. Deficiency of prothrombin leads to hypoprothrombinemia.'),
  'D011517': NtO(MH='Prothrombin Time', MS='Clotting time of PLASMA recalcified in the presence of excess TISSUE THROMBOPLASTIN. Factors measured are FIBRINOGEN; PROTHROMBIN; FACTOR V; FACTOR VII; and FACTOR X. It is used for monitoring anticoagulant therapy with COUMARINS.'),
  'D011518': NtO(MH='Proto-Oncogene Proteins', MS='Products of proto-oncogenes. Normally they do not have oncogenic or transforming properties, but are involved in the regulation or differentiation of cell growth. They often have protein kinase activity.'),
  'D011519': NtO(MH='Proto-Oncogenes', MS='Normal cellular genes homologous to viral oncogenes. The products of proto-oncogenes are important regulators of biological processes and appear to be involved in the events that serve to maintain the ordered procession through the cell cycle. Proto-oncogenes have names of the form c-onc.'),
  'D011520': NtO(MH='Protocatechuate-3,4-Dioxygenase', MS='An enzyme that catalyzes the oxidation of protocatechuate to 3-carboxy-cis-cis-muconate in the presence of molecular oxygen. It contains ferric ion. EC 1.13.11.3.'),
  'D011521': NtO(MH='Protochlorophyllide', MS='A photo-active pigment localized in prolamellar bodies occurring within the proplastids of dark-grown bean leaves. In the process of photoconversion, the highly fluorescent protochlorophyllide is converted to chlorophyll.'),
  'D011522': NtO(MH='Protons', MS='Stable elementary particles having the smallest known positive charge, found in the nuclei of all elements. The proton mass is less than that of a neutron. A proton is the nucleus of the light hydrogen atom, i.e., the hydrogen ion.'),
  'D011523': NtO(MH='Protoplasts', MS='The protoplasm and plasma membrane of plant, fungal, bacterial or archaeon cells without the CELL WALL.'),
  'D011524': NtO(MH='Protoporphyrins', MS='Porphyrins with four methyl, two vinyl, and two propionic acid side chains attached to the pyrrole rings. Protoporphyrin IX occurs in hemoglobin, myoglobin, and most of the cytochromes.'),
  'D011525': NtO(MH='Prototheca', MS='A genus of achlorophyllic algae in the family Chlorellaceae, and closely related to CHLORELLA. It is found in decayed matter; WATER; SEWAGE; and SOIL; and produces cutaneous and disseminated infections in various VERTEBRATES including humans.'),
  'D011526': NtO(MH='Protoveratrines', MS='Mixtures of closely related hypotensive alkaloids from Veratrum album (Liliaceae). They have been used in the treatment of hypertension but have largely been replaced by drugs with fewer adverse effects.'),
  'D011528': NtO(MH='Protozoan Infections', MS='Infections with unicellular organisms formerly members of the subkingdom Protozoa.'),
  'D011529': NtO(MH='Protozoan Infections, Animal', MS='Infections with unicellular organisms formerly members of the subkingdom Protozoa. The infections may be experimental or veterinary.'),
  'D011530': NtO(MH='Protriptyline', MS='Tricyclic antidepressant similar in action and side effects to IMIPRAMINE. It may produce excitation.'),
  'D011531': NtO(MH='Proventriculus', MS='A thin-walled, glandular stomach found in birds. It precedes the gizzard.'),
  'D011532': NtO(MH='Providencia', MS='Gram-negative rods isolated from human urine and feces.'),
  'D011533': NtO(MH='Proviruses', MS='Duplex DNA sequences in eukaryotic chromosomes, corresponding to the genome of a virus, that are transmitted from one cell generation to the next without causing lysis of the host. Proviruses are often associated with neoplastic cell transformation and are key features of retrovirus biology.'),
  'D011535': NtO(MH='Prune Belly Syndrome', MS='A syndrome characterized by abdominal wall musculature deficiency, cryptorchism, and urinary tract abnormalities. The syndrome derives its name from its characteristic distended abdomen with wrinkled skin.'),
  'D011536': NtO(MH='Prurigo', MS='A name applied to several itchy skin eruptions of unknown cause. The characteristic course is the formation of a dome-shaped papule with a small transient vesicle on top, followed by crusting over or lichenification. (From Dorland, 27th ed)'),
  'D011537': NtO(MH='Pruritus', MS='An intense itching sensation that produces the urge to rub or scratch the skin to obtain relief.'),
  'D011538': NtO(MH='Pruritus Ani', MS='Intense chronic itching in the anal area.'),
  'D011539': NtO(MH='Pruritus Vulvae', MS='Intense itching of the external female genitals.'),
  'D011540': NtO(MH='Prussian Blue Reaction', MS='The reaction of potassium ferrocyanide with ferric iron to yield a dark blue precipitate at the sites of the ferric iron. Used to determine ferric iron in tissues, particularly in the diagnosis of disorders of iron metabolism.'),
  'D011541': NtO(MH='Pseudallescheria', MS='Ascomycetous fungi, family Microascaceae, order Microascales, commonly found in the soil. They are causative agents of mycetoma, maduromycosis, and other infections in humans.'),
  'D011542': NtO(MH='Pseudarthrosis', MS='A pathologic entity characterized by deossification of a weight-bearing long bone, followed by bending and pathologic fracture, with inability to form normal BONY CALLUS leading to existence of the "false joint" that gives the condition its name. (Dorland, 27th ed)'),
  'D011544': NtO(MH='Pseudogenes', MS='Genes bearing close resemblance to known genes at different loci, but rendered non-functional by additions or deletions in structure that prevent normal transcription or translation. When lacking introns and containing a poly-A segment near the downstream end (as a result of reverse copying from processed nuclear RNA into double-stranded DNA), they are called processed genes.'),
  'D011546': NtO(MH='Pseudohypoaldosteronism', MS='A heterogeneous group of disorders characterized by renal electrolyte transport dysfunctions. Congenital forms are rare autosomal disorders characterized by neonatal hypertension, HYPERKALEMIA, increased RENIN activity and ALDOSTERONE concentration. The Type I features HYPERKALEMIA with sodium wasting; Type II, HYPERKALEMIA without sodium wasting. Pseudohypoaldosteronism can be the result of a defective renal electrolyte transport protein or acquired after KIDNEY TRANSPLANTATION.'),
  'D011547': NtO(MH='Pseudohypoparathyroidism', MS='A hereditary syndrome clinically similar to HYPOPARATHYROIDISM. It is characterized by HYPOCALCEMIA; HYPERPHOSPHATEMIA; and associated skeletal development impairment and caused by failure of response to PARATHYROID HORMONE rather than deficiencies. A severe form with resistance to multiple hormones is referred to as Type 1a and is associated with maternal mutant allele of the ALPHA CHAIN OF STIMULATORY G PROTEIN.'),
  'D011548': NtO(MH='Pseudomonadaceae', MS='A family of gram-negative bacteria usually found in soil or water and including many plant pathogens and a few animal pathogens.'),
  'D011549': NtO(MH='Pseudomonas', MS='A genus of gram-negative, aerobic, rod-shaped bacteria widely distributed in nature. Some species are pathogenic for humans, animals, and plants.'),
  'D011550': NtO(MH='Pseudomonas aeruginosa', MS='A species of gram-negative, aerobic, rod-shaped bacteria commonly isolated from clinical specimens (wound, burn, and urinary tract infections). It is also found widely distributed in soil and water. P. aeruginosa is a major agent of nosocomial infection.'),
  'D011551': NtO(MH='Pseudomonas fluorescens', MS='A species of nonpathogenic fluorescent bacteria found in feces, sewage, soil, and water, and which liquefy gelatin.'),
  'D011552': NtO(MH='Pseudomonas Infections', MS='Infections with bacteria of the genus PSEUDOMONAS.'),
  'D011553': NtO(MH='Pseudomyxoma Peritonei', MS='A condition characterized by poorly-circumscribed gelatinous masses filled with malignant mucin-secreting cells. Forty-five percent of pseudomyxomas arise from the ovary, usually in a mucinous cystadenocarcinoma (CYSTADENOCARCINOMA, MUCINOUS), which has prognostic significance. Pseudomyxoma peritonei must be differentiated from mucinous spillage into the peritoneum by a benign mucocele of the appendix. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D011554': NtO(MH='Pseudopodia', MS='A dynamic actin-rich extension of the surface of an animal cell used for locomotion or prehension of food.'),
  'D011555': NtO(MH='Pseudopregnancy', MS='An acyclic state that resembles PREGNANCY in that there is no ovarian cycle, ESTROUS CYCLE, or MENSTRUAL CYCLE. Unlike pregnancy, there is no EMBRYO IMPLANTATION. Pseudopregnancy can be experimentally induced to form DECIDUOMA in the UTERUS.'),
  'D011556': NtO(MH='Pseudopseudohypoparathyroidism', MS='A form of PSEUDOHYPOPARATHYROIDISM characterized by the same features except for the abnormal response to hormones such as PARATHYROID HORMONE. It is associated with paternally inherited mutant alleles of the ALPHA CHAIN OF STIMULATORY G PROTEIN.'),
  'D011557': NtO(MH='Pseudorabies', MS='A highly contagious herpesvirus infection affecting the central nervous system of swine, cattle, dogs, cats, rats, and other animals.'),
  'D011558': NtO(MH='Herpesvirus 1, Suid', MS='A species of VARICELLOVIRUS producing a respiratory infection (PSEUDORABIES) in swine, its natural host. It also produces an usually fatal ENCEPHALOMYELITIS in cattle, sheep, dogs, cats, foxes, and mink.'),
  'D011559': NtO(MH='Pseudotumor Cerebri', MS='A condition marked by raised intracranial pressure and characterized clinically by HEADACHES; NAUSEA; PAPILLEDEMA, peripheral constriction of the visual fields, transient visual obscurations, and pulsatile TINNITUS. OBESITY is frequently associated with this condition, which primarily affects women between 20 and 44 years of age. Chronic PAPILLEDEMA may lead to optic nerve injury (see OPTIC NERVE DISEASES) and visual loss (see BLINDNESS).'),
  'D011560': NtO(MH='Pseudouridine', MS='A naturally-occurring isomer of URIDINE found in RNA, in which ribosyl is attached to a carbon instead of a nitrogen atom.'),
  'D011561': NtO(MH='Pseudoxanthoma Elasticum', MS='An inherited disorder of connective tissue with extensive degeneration and calcification of ELASTIC TISSUE primarily in the skin, eye, and vasculature. At least two forms exist, autosomal recessive and autosomal dominant. This disorder is caused by mutations of one of the ATP-BINDING CASSETTE TRANSPORTERS. Patients are predisposed to MYOCARDIAL INFARCTION and GASTROINTESTINAL HEMORRHAGE.'),
  'D011562': NtO(MH='Psilocybin', MS='The major of two hallucinogenic components of Teonanacatl, the sacred mushroom of Mexico, the other component being psilocin. (From Merck Index, 11th ed)'),
  'D011563': NtO(MH='Psittaciformes', MS='An order of BIRDS comprised of several families and more than 300 species. It includes COCKATOOS; PARROTS; PARAKEETS; macaws; and BUDGERIGARS.'),
  'D011564': NtO(MH='Furocoumarins', MS='Polycyclic compounds consisting of a furan ring fused with coumarin. They commonly occur in PLANTS, especially UMBELLIFERAE and RUTACEAE, as well as PSORALEA.'),
  'D011565': NtO(MH='Psoriasis', MS='A common genetically determined, chronic, inflammatory skin disease characterized by rounded erythematous, dry, scaling patches. The lesions have a predilection for nails, scalp, genitalia, extensor surfaces, and the lumbosacral region. Accelerated epidermopoiesis is considered to be the fundamental pathologic feature in psoriasis.'),
  'D011566': NtO(MH='Psychiatric Aides', MS='Persons who assist in the routine care of psychiatric persons, usually under the supervision of the nursing department.'),
  'D011567': NtO(MH='Psychiatric Department, Hospital', MS='Hospital department responsible for the organization and administration of psychiatric services.'),
  'D011568': NtO(MH='Psychiatric Nursing', MS='A specialty concerned with the application of psychiatric principles in caring for the mentally ill. It also includes the nursing care provided the mentally ill patient.'),
  'D011569': NtO(MH='Psychiatric Status Rating Scales', MS='Standardized procedures utilizing rating scales or interview schedules carried out by health personnel for evaluating the degree of mental illness.'),
  'D011570': NtO(MH='Psychiatry', MS='The medical science that deals with the origin, diagnosis, prevention, and treatment of mental disorders.'),
  'D011571': NtO(MH='Psychoacoustics', MS="The science pertaining to the interrelationship of psychologic phenomena and the individual's response to the physical properties of sound."),
  'D011572': NtO(MH='Psychoanalysis', MS='The separation or resolution of the psyche into its constituent elements. The term has two separate meanings: 1. a procedure devised by Sigmund Freud, for investigating mental processes by means of free association, dream interpretation and interpretation of resistance and transference manifestations; and 2. a theory of psychology developed by Freud from his clinical experience with hysterical patients. (From Campbell, Psychiatric Dictionary, 1996).'),
  'D011573': NtO(MH='Psychoanalytic Interpretation', MS='Utilization of Freudian theories to explain various psychologic aspects of art, literature, biographical material, etc.'),
  'D011574': NtO(MH='Psychoanalytic Theory', MS='Conceptual system developed by Freud and his followers in which unconscious motivations are considered to shape normal and abnormal personality development and behavior.'),
  'D011575': NtO(MH='Psychoanalytic Therapy', MS='A form of psychiatric treatment, based on Freudian principles, which seeks to eliminate or diminish the undesirable effects of unconscious conflicts by making the patient aware of their existence, origin, and inappropriate expression in current emotions and behavior.'),
  'D011576': NtO(MH='Psychodidae', MS='Small, hairy, moth-like flies which are of considerable public health importance as vectors of certain pathogenic organisms. Important disease-related genera are PHLEBOTOMUS, Lutzomyia, and Sergentomyia.'),
  'D011577': NtO(MH='Psychodrama', MS="Primarily a technique of group psychotherapy which involves a structured, directed, and dramatized acting out of the patient's personal and emotional problems."),
  'D011578': NtO(MH='Psycholinguistics', MS='A discipline concerned with relations between messages and the characteristics of individuals who select and interpret them; it deals directly with the processes of encoding (phonetics) and decoding (psychoacoustics) as they relate states of messages to states of communicators.'),
  'D011579': NtO(MH='Psychological Phenomena', MS='Mechanisms and underlying psychological principles of mental processes and their applications.'),
  'D011580': NtO(MH='Psychological Techniques', MS='Methods used in the diagnosis and treatment of behavioral, personality, and mental disorders.'),
  'D011581': NtO(MH='Psychological Tests', MS='Standardized tests designed to measure abilities, as in intelligence, aptitude, and achievement tests, or to evaluate personality traits.'),
  'D011582': NtO(MH='Psychological Theory', MS='Principles applied to the analysis and explanation of psychological or behavioral phenomena.'),
  'D011583': NtO(MH='Psychological Warfare', MS="The manipulation of psychological influences, primarily concerned with morale, to strengthen the ability of one's own country and weaken the enemy."),
  'D011584': NtO(MH='Psychology', MS='The science dealing with the study of mental processes and behavior in man and animals.'),
  'D011585': NtO(MH='Psychology, Applied', MS='The science which utilizes psychologic principles to derive more effective means in dealing with practical problems.'),
  'D011586': NtO(MH='Psychology, Clinical', MS='The branch of psychology concerned with psychological methods of recognizing and treating behavior disorders.'),
  'D011587': NtO(MH='Psychology, Comparative', MS='The branch of psychology concerned with similarities or differences in the behavior of different animal species or of different races or peoples.'),
  'D011588': NtO(MH='Psychology, Educational', MS='The branch of psychology concerned with psychological aspects of teaching and the formal learning process in school.'),
  'D011589': NtO(MH='Psychology, Experimental', MS='The branch of psychology which seeks to learn more about the fundamental causes of behavior by studying various psychologic phenomena in controlled experimental situations.'),
  'D011590': NtO(MH='Psychology, Industrial', MS='The branch of applied psychology concerned with the application of psychologic principles and methods to industrial problems including selection and training of workers, working conditions, etc.'),
  'D011591': NtO(MH='Psychology, Medical', MS='A branch of psychology in which there is collaboration between psychologists and physicians in the management of medical problems. It differs from clinical psychology, which is concerned with the diagnosis and treatment of behavior disorders.'),
  'D011592': NtO(MH='Psychology, Military', MS='The branch of applied psychology concerned with psychological aspects of selection, assignment, training, morale, etc., of Armed Forces personnel.'),
  'D011593': NtO(MH='Psychology, Social', MS='The branch of psychology concerned with the effects of group membership upon the behavior, attitudes, and beliefs of an individual.'),
  'D011594': NtO(MH='Psychometrics', MS='Assessment of psychological variables by the application of mathematical procedures.'),
  'D011595': NtO(MH='Psychomotor Agitation', MS='A feeling of restlessness associated with increased motor activity. This may occur as a manifestation of nervous system drug toxicity or other conditions.'),
  'D011596': NtO(MH='Psychomotor Disorders', MS='Abnormalities of motor function that are associated with organic and non-organic cognitive disorders.'),
  'D011597': NtO(MH='Psychomotor Performance', MS='The coordination of a sensory or ideational (cognitive) process and a motor activity.'),
  'D011598': NtO(MH='Psychoneuroimmunology', MS='The field concerned with the interrelationship between the brain, behavior and the immune system. Neuropsychologic, neuroanatomic and psychosocial studies have demonstrated their role in accentuating or diminishing immune/allergic responses.'),
  'D011599': NtO(MH='Psychopathology', MS='The study of significant causes and processes in the development of mental illness.'),
  'D011600': NtO(MH='Psychopharmacology', MS='The study of the effects of drugs on mental and behavioral activity.'),
  'D011601': NtO(MH='Psychophysics', MS='The science dealing with the correlation of the physical characteristics of a stimulus, e.g., frequency or intensity, with the response to the stimulus, in order to assess the psychologic factors involved in the relationship.'),
  'D011602': NtO(MH='Psychophysiologic Disorders', MS='A group of disorders characterized by physical symptoms that are affected by emotional factors and involve a single organ system, usually under AUTONOMIC NERVOUS SYSTEM control. (American Psychiatric Glossary, 1988)'),
  'D011603': NtO(MH='Psychophysiology', MS='The study of the physiological basis of human and animal behavior.'),
  'D011604': NtO(MH='Psychoses, Alcoholic', MS='A group of mental disorders associated with organic brain damage and caused by poisoning from alcohol.'),
  'D011605': NtO(MH='Psychoses, Substance-Induced', MS='Psychotic organic mental disorders resulting from the toxic effect of drugs and chemicals or other harmful substance.'),
  'D011606': NtO(MH='Psychosexual Development', MS='The stages of development of the psychological aspects of sexuality from birth to adulthood; i.e., oral, anal, genital, and latent periods.'),
  'D011609': NtO(MH='Psychosine', MS='An intermediate in the biosynthesis of cerebrosides. It is formed by reaction of sphingosine with UDP-galactose and then itself reacts with fatty acid-Coenzyme A to form the cerebroside.'),
  'D011610': NtO(MH='Psychosocial Deprivation', MS='The absence of appropriate stimuli in the physical or social environment which are necessary for the emotional, social, and intellectual development of the individual.'),
  'D011611': NtO(MH='Psychosomatic Medicine', MS='A system of medicine which aims at discovering the exact nature of the relationship between the emotions and bodily function, affirming the principle that the mind and body are one.'),
  'D011612': NtO(MH='Psychosurgery', MS='Treatment of chronic, severe and intractable psychiatric disorders by surgical removal or interruption of certain areas or pathways in the brain, especially in the prefrontal lobes.'),
  'D011613': NtO(MH='Psychotherapy', MS='A generic term for the treatment of mental illness or emotional disturbances primarily by verbal or nonverbal communication.'),
  'D011614': NtO(MH='Psychotherapy, Brief', MS='Any form of psychotherapy designed to produce therapeutic change within a minimal amount of time, generally not more than 20 sessions.'),
  'D011615': NtO(MH='Psychotherapy, Group', MS='A form of therapy in which two or more patients participate under the guidance of one or more psychotherapists for the purpose of treating emotional disturbances, social maladjustments, and psychotic states.'),
  'D011616': NtO(MH='Psychotherapy, Multiple', MS='The use of more than one therapist at one time in individual or group psychotherapy.'),
  'D011617': NtO(MH='Psychotherapy, Rational-Emotive', MS='The replacement of illogical and unrealistic ideas with more realistic and adaptive ones through direct intervention and confrontation by the therapist.'),
  'D011618': NtO(MH='Psychotic Disorders', MS='Disorders in which there is a loss of ego boundaries or a gross impairment in reality testing with delusions or prominent hallucinations. (From DSM-IV, 1994)'),
  'D011619': NtO(MH='Psychotropic Drugs', MS='A loosely defined grouping of drugs that have effects on psychological function. Here the psychotropic agents include the antidepressive agents, hallucinogens, and tranquilizing agents (including the antipsychotics and anti-anxiety agents).'),
  'D011620': NtO(MH='Psyllium', MS='Dried, ripe seeds of PLANTAGO PSYLLIUM; PLANTAGO INDICA; and PLANTAGO OVATA. Plantain seeds swell in water and are used as demulcents and bulk laxatives.'),
  'D011621': NtO(MH='Pteridines', MS='Compounds based on pyrazino[2,3-d]pyrimidine which is a pyrimidine fused to a pyrazine, containing four NITROGEN atoms.'),
  'D011622': NtO(MH='Pterins', MS='Compounds based on 2-amino-4-hydroxypteridine.'),
  'D011623': NtO(MH='gamma-Glutamyl Hydrolase', MS='Catalyzes the hydrolysis of pteroylpolyglutamic acids in gamma linkage to pterolylmonoglutamic acid and free glutamic acid. EC 3.4.19.9.'),
  'D011624': NtO(MH='Pteroylpolyglutamic Acids', MS='Derivatives of folic acid (pteroylglutamic acid). In gamma-glutamyl linkage they are found in many tissues. They are converted to folic acid by the action of pteroylpolyglutamate hydrolase or synthesized from folic acid by the action of folate polyglutamate synthetase. Synthetic pteroylpolyglutamic acids, which are in alpha-glutamyl linkage, are active in bacterial growth assays.'),
  'D011625': NtO(MH='Pterygium', MS='An abnormal triangular fold of membrane in the interpalpebral fissure, extending from the conjunctiva to the cornea, being immovably united to the cornea at its apex, firmly attached to the sclera throughout its middle portion, and merged with the conjunctiva at its base. (Dorland, 27th ed)'),
  'D011626': NtO(MH='Pterygoid Muscles', MS='Two of the masticatory muscles: the internal, or medial, pterygoid muscle and external, or lateral, pterygoid muscle. Action of the former is closing the jaws and that of the latter is opening the jaws, protruding the mandible, and moving the mandible from side to side.'),
  'D011627': NtO(MH='Puberty', MS='A period in the human life in which the development of the hypothalamic-pituitary-gonadal system takes place and reaches full maturity. The onset of synchronized endocrine events in puberty lead to the capacity for reproduction (FERTILITY), development of secondary SEX CHARACTERISTICS, and other changes seen in ADOLESCENT DEVELOPMENT.'),
  'D011628': NtO(MH='Puberty, Delayed', MS='The lack of development of SEXUAL MATURATION in boys and girls at a chronological age that is 2.5 standard deviations above the mean age at onset of PUBERTY in a population. Delayed puberty can be classified by defects in the hypothalamic LHRH pulse generator, the PITUITARY GLAND, or the GONADS. These patients will undergo spontaneous but delayed puberty whereas patients with SEXUAL INFANTILISM will not.'),
  'D011629': NtO(MH='Puberty, Precocious', MS='Development of SEXUAL MATURATION in boys and girls at a chronological age that is 2.5 standard deviations below the mean age at onset of PUBERTY in the population. This early maturation of the hypothalamic-pituitary-gonadal axis results in sexual precocity, elevated serum levels of GONADOTROPINS and GONADAL STEROID HORMONES such as ESTRADIOL and TESTOSTERONE.'),
  'D011630': NtO(MH='Pubic Bone', MS='A bone that forms the lower and anterior part of each side of the hip bone.'),
  'D011631': NtO(MH='Pubic Symphysis', MS='A slightly movable cartilaginous joint which occurs between the pubic bones.'),
  'D011632': NtO(MH='Public Assistance', MS='Financial assistance to impoverished persons for the essentials of living through federal, state or local government programs.'),
  'D011633': NtO(MH='Public Facilities', MS='Any facility  owned, leased, or otherwise operated, or funded by a governmental body or public entity.'),
  'D011634': NtO(MH='Public Health', MS='Branch of medicine concerned with the prevention and control of disease and disability, and the promotion of physical and mental health of the population on the international, national, state, or municipal level.'),
  'D011635': NtO(MH='Public Health Administration', MS='Management of public health organizations or agencies.'),
  'D011636': NtO(MH='Public Health Dentistry', MS='A dental specialty concerned with the prevention of disease and the maintenance of oral health through promoting organized dental health programs at a community, state, or federal level.'),
  'D011637': NtO(MH='Public Health Nursing', MS='A nursing specialty concerned with promoting and protecting the health of populations, using knowledge from nursing, social, and public health sciences to develop local, regional, state, and national health policy and research. It is population-focused and community-oriented, aimed at health promotion and disease prevention through educational, diagnostic, and preventive programs.'),
  'D011638': NtO(MH='Public Housing', MS='Housing subsidized by tax funds, usually intended for low income persons or families.'),
  'D011639': NtO(MH='Public Opinion', MS='The attitude of a significant portion of a population toward any given proposition, based upon a measurable amount of factual evidence, and involving some degree of reflection, analysis, and reasoning.'),
  'D011640': NtO(MH='Public Policy', MS='A course or method of action selected, usually by a government, from among alternatives to guide and determine present and future decisions.'),
  'D011641': NtO(MH='Public Relations', MS='Relations of an individual, association, organization, hospital, or corporation with the publics which it must take into consideration in carrying out its functions. Publics may include consumers, patients, pressure groups, departments, etc.'),
  'D011642': NtO(MH='Publications', MS='Copies of a work or document distributed to the public by sale, rental, lease, or lending. (From ALA Glossary of Library and Information Science, 1983, p181)'),
  'D011643': NtO(MH='Publishing', MS='"The business or profession of the commercial production and issuance of literature" (Webster\'s 3d). It includes the publisher, publication processes, editing and editors. Production may be by conventional printing methods or by electronic publishing.'),
  'D011644': NtO(MH='Puerperal Disorders', MS='Disorders or diseases associated with PUERPERIUM, the six-to-eight-week period immediately after PARTURITION in humans.'),
  'D011645': NtO(MH='Puerperal Infection', MS='An infection occurring in PUERPERIUM, the period of 6-8 weeks after giving birth.'),
  'D011647': NtO(MH='Puerto Rico', MS="An island in the Greater Antilles in the West Indies. Its capital is San Juan. It is a self-governing commonwealth in union with the United States. It was discovered by Columbus in 1493 but no colonization was attempted until 1508. It belonged to Spain until ceded to the United States in 1898. It became a commonwealth with autonomy in internal affairs in 1952. Columbus named the island San Juan for St. John's Day, the Monday he arrived, and the bay Puerto Rico, rich harbor. The island became Puerto Rico officially in 1932. (From Webster's New Geographical Dictionary, 1988, p987 & Room, Brewer's Dictionary of Names, 1992, p436)"),
  'D011648': NtO(MH='Pulmonary Adenomatosis, Ovine', MS='A contagious, neoplastic, pulmonary disease of sheep characterized by hyperplasia and hypertrophy of pneumocytes and epithelial cells of the lung. It is caused by JAAGSIEKTE SHEEP RETROVIRUS.'),
  'D011649': NtO(MH='Pulmonary Alveolar Proteinosis', MS='A PULMONARY ALVEOLI-filling disease, characterized by dense phospholipoproteinaceous deposits in the alveoli, cough, and DYSPNEA. This disease is often related to, congenital or acquired, impaired processing of PULMONARY SURFACTANTS by alveolar macrophages, a process dependent on GRANULOCYTE-MACROPHAGE COLONY-STIMULATING FACTOR.'),
  'D011650': NtO(MH='Pulmonary Alveoli', MS='Small polyhedral outpouchings along the walls of the alveolar sacs, alveolar ducts and terminal bronchioles through the walls of which gas exchange between alveolar air and pulmonary capillary blood takes place.'),
  'D011651': NtO(MH='Pulmonary Artery', MS='The short wide vessel arising from the conus arteriosus of the right ventricle and conveying unaerated blood to the lungs.'),
  'D011652': NtO(MH='Pulmonary Circulation', MS='The circulation of the BLOOD through the LUNGS.'),
  'D011653': NtO(MH='Pulmonary Diffusing Capacity', MS='The amount of a gas taken up, by the pulmonary capillary blood from the alveolar gas, per minute per unit of average pressure of the gradient of the gas across the BLOOD-AIR BARRIER.'),
  'D011654': NtO(MH='Pulmonary Edema', MS='Excessive accumulation of extravascular fluid in the lung, an indication of a serious underlying disease or disorder. Pulmonary edema prevents efficient PULMONARY GAS EXCHANGE in the PULMONARY ALVEOLI, and can be life-threatening.'),
  'D011655': NtO(MH='Pulmonary Embolism', MS='Blocking of the PULMONARY ARTERY or one of its branches by an EMBOLUS.'),
  'D011656': NtO(MH='Pulmonary Emphysema', MS='Enlargement of air spaces distal to the TERMINAL BRONCHIOLES where gas-exchange normally takes place. This is usually due to destruction of the alveolar wall. Pulmonary emphysema can be classified by the location and distribution of the lesions.'),
  'D011657': NtO(MH='Pulmonary Eosinophilia', MS='A condition characterized by infiltration of the lung with EOSINOPHILS due to inflammation or other disease processes. Major eosinophilic lung diseases are the eosinophilic pneumonias caused by infections, allergens, or toxic agents.'),
  'D011658': NtO(MH='Pulmonary Fibrosis', MS='A process in which normal lung tissues are progressively replaced by FIBROBLASTS and COLLAGEN causing an irreversible loss of the ability to transfer oxygen into the bloodstream via PULMONARY ALVEOLI. Patients show progressive DYSPNEA finally resulting in death.'),
  'D011659': NtO(MH='Pulmonary Gas Exchange', MS='The exchange of OXYGEN and CARBON DIOXIDE between alveolar air and pulmonary capillary blood that occurs across the BLOOD-AIR BARRIER.'),
  'D011660': NtO(MH='Pulmonary Heart Disease', MS='Hypertrophy and dilation of the RIGHT VENTRICLE of the heart that is caused by PULMONARY HYPERTENSION. This condition is often associated with pulmonary parenchymal or vascular diseases, such as CHRONIC OBSTRUCTIVE PULMONARY DISEASE and PULMONARY EMBOLISM.'),
  'D011661': NtO(MH='Pulmonary Stretch Receptors', MS='Stretch receptors found in the bronchi and bronchioles. Pulmonary stretch receptors are sensors for a reflex which stops inspiration. In humans, the reflex is protective and is probably not activated during normal respiration.'),
  'D011662': NtO(MH='Pulmonary Subvalvular Stenosis', MS='Narrowing below the PULMONARY VALVE or well below it in the infundibuluar chamber where the pulmonary artery originates, usually caused by a defective VENTRICULAR SEPTUM or presence of fibrous tissues. It is characterized by restricted blood outflow from the RIGHT VENTRICLE into the PULMONARY ARTERY, exertional fatigue, DYSPNEA, and chest discomfort.'),
  'D011663': NtO(MH='Pulmonary Surfactants', MS='Substances and drugs that lower the SURFACE TENSION of the mucoid layer lining the PULMONARY ALVEOLI.'),
  'D011664': NtO(MH='Pulmonary Valve', MS='A valve situated at the entrance to the pulmonary trunk from the right ventricle.'),
  'D011665': NtO(MH='Pulmonary Valve Insufficiency', MS='Backflow of blood from the PULMONARY ARTERY into the RIGHT VENTRICLE due to imperfect closure of the PULMONARY VALVE.'),
  'D011666': NtO(MH='Pulmonary Valve Stenosis', MS='The pathologic narrowing of the orifice of the PULMONARY VALVE. This lesion restricts blood outflow from the RIGHT VENTRICLE to the PULMONARY ARTERY. When the trileaflet valve is fused into an imperforate membrane, the blockage is complete.'),
  'D011667': NtO(MH='Pulmonary Veins', MS='The veins that return the oxygenated blood from the lungs to the left atrium of the heart.'),
  'D011668': NtO(MH='Pulmonary Veno-Occlusive Disease', MS='Pathological process resulting in the fibrous obstruction of the small- and medium-sized PULMONARY VEINS and PULMONARY HYPERTENSION. Veno-occlusion can arise from fibrous proliferation of the VASCULAR INTIMA and VASCULAR MEDIA; THROMBOSIS; or a combination of both.'),
  'D011669': NtO(MH='Pulmonary Wedge Pressure', MS='The blood pressure as recorded after wedging a CATHETER in a small PULMONARY ARTERY; believed to reflect the PRESSURE in the pulmonary CAPILLARIES.'),
  'D011670': NtO(MH='Pulpectomy', MS='Dental procedure in which the entire pulp chamber is removed from the crown and roots of a tooth.'),
  'D011671': NtO(MH='Pulpitis', MS='Inflammation of the DENTAL PULP, usually due to bacterial infection in dental caries, tooth fracture, or other conditions causing exposure of the pulp to bacterial invasion. Chemical irritants, thermal factors, hyperemic changes, and other factors may also cause pulpitis.'),
  'D011672': NtO(MH='Pulpotomy', MS='Dental procedure in which part of the pulp chamber is removed from the crown of a tooth.'),
  'D011673': NtO(MH='Pulsatile Flow', MS='Rhythmic, intermittent propagation of a fluid through a BLOOD VESSEL or piping system, in contrast to constant, smooth propagation, which produces laminar flow.'),
  'D011674': NtO(MH='Pulse', MS='The rhythmical expansion and contraction of an ARTERY produced by waves of pressure caused by the ejection of BLOOD from the left ventricle of the HEART as it contracts.'),
  'D011675': NtO(MH='Pulse Radiolysis', MS='Use of a pulse of X-rays or fast electrons to generate free radicals for spectroscopic examination.'),
  'D011676': NtO(MH='Punched-Card Systems', MS='Data processing using paper cards with punched holes that represent data.'),
  'D011677': NtO(MH='Punctures', MS='Incision of tissues for injection of medication or for other diagnostic or therapeutic procedures. Punctures of the skin, for example may be used for diagnostic drainage; of blood vessels for diagnostic imaging procedures.'),
  'D011678': NtO(MH='Punishment', MS='The application of an unpleasant stimulus or penalty for the purpose of eliminating or correcting undesirable behavior.'),
  'D011679': NtO(MH='Pupa', MS='An inactive stage between the larval and adult stages in the life cycle of INSECTA.'),
  'D011680': NtO(MH='Pupil', MS='The aperture in the iris through which light passes.'),
  'D011681': NtO(MH='Pupil Disorders', MS='Conditions which affect the structure or function of the pupil of the eye, including disorders of innervation to the pupillary constrictor or dilator muscles, and disorders of pupillary reflexes.'),
  'D011682': NtO(MH='Purchasing, Hospital', MS='Hospital department responsible for the purchasing of supplies and equipment.'),
  'D011683': NtO(MH='Purine-Nucleoside Phosphorylase', MS='An enzyme that catalyzes the reaction between a purine nucleoside and orthophosphate to form a free purine plus ribose-5-phosphate. EC 2.4.2.1.'),
  'D011684': NtO(MH='Purine Nucleosides', MS='Purines with a RIBOSE attached that can be phosphorylated to PURINE NUCLEOTIDES.'),
  'D011685': NtO(MH='Purine Nucleotides', MS='Purines attached to a RIBOSE and a phosphate that can polymerize to form DNA and RNA.'),
  'D011686': NtO(MH='Purine-Pyrimidine Metabolism, Inborn Errors', MS='Dysfunctions in the metabolism of PURINES or PYRIMIDINES resulting from inborn genetic mutations that are inherited or acquired in utero.'),
  'D011687': NtO(MH='Purines', MS='A series of heterocyclic compounds that are variously substituted in nature and are known also as purine bases. They include ADENINE and GUANINE, constituents of nucleic acids, as well as many alkaloids such as CAFFEINE and THEOPHYLLINE. Uric acid is the metabolic end product of purine metabolism.'),
  'D011688': NtO(MH='Purinones', MS=None),
  'D011689': NtO(MH='Purkinje Cells', MS='The output neurons of the cerebellar cortex.'),
  'D011690': NtO(MH='Purkinje Fibers', MS='Modified cardiac muscle fibers composing the terminal portion of the heart conduction system.'),
  'D011691': NtO(MH='Puromycin', MS='A cinnamamido ADENOSINE found in STREPTOMYCES alboniger. It inhibits protein synthesis by binding to RNA. It is an antineoplastic and antitrypanosomal agent and is used in research as an inhibitor of protein synthesis.'),
  'D011692': NtO(MH='Puromycin Aminonucleoside', MS='PUROMYCIN derivative that lacks the methoxyphenylalanyl group on the amine of the sugar ring. It is an antibiotic with antineoplastic properties and can cause nephrosis.'),
  'D011693': NtO(MH='Purpura', MS='Purplish or brownish red discoloration, easily visible through the epidermis, caused by hemorrhage into the tissues. When the size of the discolorization is >2-3 cm it is generally called Ecchymoses (ECCHYMOSIS).'),
  'D011694': NtO(MH='Purpura, Hyperglobulinemic', MS='Purplish or brownish red discoloration of the skin associated with increase in circulating polyclonal globulins, usually GAMMA-GLOBULINS. This syndrome often occurs on the legs of women aged 20 to 40 years.'),
  'D011695': NtO(MH='Purpura, Schoenlein-Henoch', MS='A systemic non-thrombocytopenic purpura caused by HYPERSENSITIVITY VASCULITIS and deposition of IGA-containing IMMUNE COMPLEXES within the blood vessels throughout the body, including those in the kidney (KIDNEY GLOMERULUS). Clinical symptoms include URTICARIA; ERYTHEMA; ARTHRITIS; GASTROINTESTINAL HEMORRHAGE; and renal involvement. Most cases are seen in children after acute upper respiratory infections.'),
  'D011696': NtO(MH='Purpura, Thrombocytopenic', MS='Any form of purpura in which the PLATELET COUNT is decreased. Many forms are thought to be caused by immunological mechanisms.'),
  'D011697': NtO(MH='Purpura, Thrombotic Thrombocytopenic', MS='An acquired, congenital, or familial disorder caused by PLATELET AGGREGATION with THROMBOSIS in terminal arterioles and capillaries. Clinical features include THROMBOCYTOPENIA; HEMOLYTIC ANEMIA; AZOTEMIA; FEVER; and thrombotic microangiopathy. The classical form also includes neurological symptoms and end-organ damage, such as RENAL FAILURE. Mutations in the ADAMTS13 PROTEIN gene have been identified in familial cases.'),
  'D011698': NtO(MH='Pursuit, Smooth', MS='Eye movements that are slow, continuous, and conjugate and occur when a fixed object is moved slowly.'),
  'D011699': NtO(MH='Putamen', MS='The largest and most lateral of the BASAL GANGLIA lying between the lateral medullary lamina of the GLOBUS PALLIDUS and the EXTERNAL CAPSULE. It is part of the neostriatum and forms part of the LENTIFORM NUCLEUS along with the GLOBUS PALLIDUS.'),
  'D011700': NtO(MH='Putrescine', MS='A toxic diamine formed by putrefaction from the decarboxylation of arginine and ornithine.'),
  'D011701': NtO(MH='PUVA Therapy', MS='Photochemotherapy using PSORALENS as the photosensitizing agent and ultraviolet light type A (UVA).'),
  'D011702': NtO(MH='Pyelitis', MS='Inflammation of the KIDNEY PELVIS and KIDNEY CALICES where urine is collected before discharge, but does not involve the renal parenchyma (the NEPHRONS) where urine is processed.'),
  'D011704': NtO(MH='Pyelonephritis', MS='Inflammation of the KIDNEY involving the renal parenchyma (the NEPHRONS); KIDNEY PELVIS; and KIDNEY CALICES. It is characterized by ABDOMINAL PAIN; FEVER; NAUSEA; VOMITING; and occasionally DIARRHEA.'),
  'D011705': NtO(MH='Pyelonephritis, Xanthogranulomatous', MS='A chronic inflammatory condition of the KIDNEY resulting in diffuse renal destruction, a grossly enlarged and nonfunctioning kidney associated with NEPHROLITHIASIS and KIDNEY STONES.'),
  'D011706': NtO(MH='Pyloric Antrum', MS='The region between the sharp indentation at the lower third of the STOMACH (incisura angularis) and the junction of the PYLORUS with the DUODENUM. Pyloric antral glands contain mucus-secreting cells and gastrin-secreting endocrine cells (G CELLS).'),
  'D011707': NtO(MH='Pyloric Stenosis', MS='Narrowing of the pyloric canal with varied etiology. A common form is due to muscle hypertrophy (PYLORIC STENOSIS, HYPERTROPHIC) seen in infants.'),
  'D011708': NtO(MH='Pylorus', MS='The region of the STOMACH at the junction with the DUODENUM. It is marked by the thickening of circular muscle layers forming the pyloric sphincter to control the opening and closure of the lumen.'),
  'D011709': NtO(MH='Pyocins', MS='Bacteriocins elaborated by mutant strains of Pseudomonas aeruginosa. They are protein or protein-lipopolysaccharide complexes lethal to other strains of the same or related species.'),
  'D011710': NtO(MH='Pyocyanine', MS='Antibiotic pigment produced by Pseudomonas aeruginosa.'),
  'D011711': NtO(MH='Pyoderma', MS='Any purulent skin disease (Dorland, 27th ed).'),
  'D011712': NtO(MH='Pyramidal Tracts', MS='Fibers that arise from cells within the cerebral cortex, pass through the medullary pyramid, and descend in the spinal cord. Many authorities say the pyramidal tracts include both the corticospinal and corticobulbar tracts.'),
  'D011713': NtO(MH='Pyran Copolymer', MS='Copolymer of divinyl ether and maleic anhydride that acts as an immunostimulant with antineoplastic and anti-infective properties. It is used in combination with other antineoplastic agents.'),
  'D011714': NtO(MH='Pyrans', MS=None),
  'D011715': NtO(MH='Pyrantel', MS='A depolarizing neuromuscular-blocking agent, that causes persistent nicotinic activation resulting in spastic paralysis of susceptible nematodes. It is a drug of second-choice after benzimidazoles for treatment of ascariasis, hookworm, and pinworm infections, being effective after a single dose. (From Smith and Reynard, Textbook of Pharmacology, 1992, p920)'),
  'D011716': NtO(MH='Pyrantel Pamoate', MS='Broad spectrum antinematodal anthelmintic used also in veterinary medicine.'),
  'D011717': NtO(MH='Pyrantel Tartrate', MS='Broad spectrum anthelmintic for livestock.'),
  'D011718': NtO(MH='Pyrazinamide', MS='A pyrazine that is used therapeutically as an antitubercular agent.'),
  'D011719': NtO(MH='Pyrazines', MS='A heterocyclic aromatic organic compounds with the chemical formula C4H4N2.'),
  'D011720': NtO(MH='Pyrazoles', MS='Azoles of two nitrogens at the 1,2 positions, next to each other, in contrast with IMIDAZOLES in which they are at the 1,3 positions.'),
  'D011721': NtO(MH='Pyrenes', MS='A group of condensed ring hydrocarbons.'),
  'D011722': NtO(MH='Pyrethrins', MS='The active insecticidal constituent of CHRYSANTHEMUM CINERARIIFOLIUM flowers. Pyrethrin I is the pyretholone ester of chrysanthemummonocarboxylic acid and pyrethrin II is the pyretholone ester of chrysanthemumdicarboxylic acid monomethyl ester.'),
  'D011723': NtO(MH='Chrysanthemum cinerariifolium', MS='A plant species of the genus CHRYSANTHEMUM, family ASTERACEAE. The flowers contain PYRETHRINS, cinerolones, and chrysanthemines which are powerful contact insecticides. Most in the old Pyrethrum genus are reclassified to TANACETUM; some to other ASTERACEAE genera.'),
  'D011724': NtO(MH='Pyridazines', MS='Six-membered rings with two adjacent nitrogen atoms also called 1,2-diazine.'),
  'D011725': NtO(MH='Pyridines', MS='Compounds with a six membered aromatic ring containing NITROGEN. The saturated version is PIPERIDINES.'),
  'D011726': NtO(MH='Pyridinium Compounds', MS='Derivatives of PYRIDINE containing a cation C5H5NH or radical C5H6N.'),
  'D011727': NtO(MH='Pyridinolcarbamate', MS='A drug that has been given by mouth in the treatment of atherosclerosis and other vascular disorders, hyperlipidemias, and thrombo-embolic disorders. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1408)'),
  'D011728': NtO(MH='Pyridones', MS='Pyridine derivatives with one or more keto groups on the ring.'),
  'D011729': NtO(MH='Pyridostigmine Bromide', MS='A cholinesterase inhibitor with a slightly longer duration of action than NEOSTIGMINE. It is used in the treatment of myasthenia gravis and to reverse the actions of muscle relaxants.'),
  'D011730': NtO(MH='Pyridoxal', MS='The 4-carboxyaldehyde form of VITAMIN B 6 which is converted to PYRIDOXAL PHOSPHATE which is a coenzyme for synthesis of amino acids, neurotransmitters (serotonin, norepinephrine), sphingolipids, aminolevulinic acid.'),
  'D011731': NtO(MH='Pyridoxal Kinase', MS='An enzyme that catalyzes reversibly the phosphorylation of pyridoxal in the presence of ATP with the formation of pyridoxal 5-phosphate and ADP. Pyridoxine, pyridoxamine and various derivatives can also act as acceptors. EC 2.7.1.35.'),
  'D011732': NtO(MH='Pyridoxal Phosphate', MS='This is the active form of VITAMIN B 6 serving as a coenzyme for synthesis of amino acids, neurotransmitters (serotonin, norepinephrine), sphingolipids, aminolevulinic acid. During transamination of amino acids, pyridoxal phosphate is transiently converted into pyridoxamine phosphate (PYRIDOXAMINE).'),
  'D011733': NtO(MH='Pyridoxamine', MS='The 4-aminomethyl form of VITAMIN B 6. During transamination of amino acids, PYRIDOXAL PHOSPHATE is transiently converted into pyridoxamine phosphate.'),
  'D011734': NtO(MH='Pyridoxaminephosphate Oxidase', MS='An enzyme catalyzing the deamination of pyridoxaminephosphate to pyridoxal phosphate. It is a flavoprotein that also oxidizes pyridoxine-5-phosphate and pyridoxine. EC 1.4.3.5.'),
  'D011735': NtO(MH='Pyridoxic Acid', MS='The catabolic product of most of VITAMIN B 6; (PYRIDOXINE; PYRIDOXAL; and PYRIDOXAMINE) which is excreted in the urine.'),
  'D011736': NtO(MH='Pyridoxine', MS='The 4-methanol form of VITAMIN B 6 which is converted to PYRIDOXAL PHOSPHATE which is a coenzyme for synthesis of amino acids, neurotransmitters (serotonin, norepinephrine), sphingolipids, aminolevulinic acid. Although pyridoxine and Vitamin B 6 are still frequently used as synonyms, especially by medical researchers, this practice is erroneous and sometimes misleading (EE Snell; Ann NY Acad Sci, vol 585 pg 1, 1990).'),
  'D011738': NtO(MH='Pyrilamine', MS='A histamine H1 antagonist. It has mild hypnotic properties and some local anesthetic action and is used for allergies (including skin eruptions) both parenterally and locally. It is a common ingredient of cold remedies.'),
  'D011739': NtO(MH='Pyrimethamine', MS='One of the FOLIC ACID ANTAGONISTS that is used as an antimalarial or with a sulfonamide to treat toxoplasmosis.'),
  'D011740': NtO(MH='Pyrimidine Dimers', MS='Dimers found in DNA chains damaged by ULTRAVIOLET RAYS. They consist of two adjacent PYRIMIDINE NUCLEOTIDES, usually THYMINE nucleotides, in which the pyrimidine residues are covalently joined by a cyclobutane ring. These dimers block DNA REPLICATION.'),
  'D011741': NtO(MH='Pyrimidine Nucleosides', MS='Pyrimidines with a RIBOSE attached that can be phosphorylated to PYRIMIDINE NUCLEOTIDES.'),
  'D011742': NtO(MH='Pyrimidine Nucleotides', MS='Pyrimidines with a RIBOSE and phosphate attached that can polymerize to form DNA and RNA.'),
  'D011743': NtO(MH='Pyrimidines', MS='A family of 6-membered heterocyclic compounds occurring in nature in a wide variety of forms. They include several nucleic acid constituents (CYTOSINE; THYMINE; and URACIL) and form the basic structure of the barbiturates.'),
  'D011744': NtO(MH='Pyrimidinones', MS='Heterocyclic compounds known as 2-pyrimidones (or 2-hydroxypyrimidines) and 4-pyrimidones (or 4-hydroxypyrimidines) with the general formula C4H4N2O.'),
  'D011745': NtO(MH='Pyrithiamine', MS='A thiamine antagonist due to its inhibition of thiamine pyrophosphorylation. It is used to produce thiamine deficiency.'),
  'D011746': NtO(MH='Pyrithioxin', MS='A neurotropic agent which reduces permeability of blood-brain barrier to phosphate. It has no vitamin B6 activity.'),
  'D011748': NtO(MH='Pyrogallol', MS='A trihydroxybenzene or dihydroxy phenol that can be prepared by heating GALLIC ACID.'),
  'D011749': NtO(MH='Pyrogens', MS='Substances capable of increasing BODY TEMPERATURE and cause FEVER and may be used for FEVER THERAPY. They may be of microbial origin, often POLYSACCHARIDES, and may contaminate distilled water.'),
  'D011750': NtO(MH='Pyroglobulins', MS='Atypical gamma immunoglobulins characterized by their irreversible heat denaturation at 56-degrees C. Pyroprecipitation is inhibited at pH below 3 and above 9. The presence of pyroglobulins in the serum is the cause of pyroglobulinemia. They are frequently present in multiple myeloma and the pyroglobulin precipitate binds complement, reacts with rheumatoid factor, produces passive cutaneous anaphylaxis, generalized passive anaphylaxis and passive Arthus-type phenomena.'),
  'D011751': NtO(MH='Pyroglutamyl-Peptidase I', MS='An enzyme that catalyzes the release of a N-terminal pyroglutamyl group from a polypeptide provided the next residue is not proline. It is inhibited by thiol-blocking reagents and occurs in mammalian tissues, microorganisms, and plants. (From Enzyme Nomenclature, 1992) EC 3.4.19.3.'),
  'D011752': NtO(MH='Pyroglutamate Hydrolase', MS='Hydrolyzes pyroglutamic acid in the presence of ATP to glutamate plus ADP and inorganic phosphate. Deficiency leads to pyroglutamic acidurea.'),
  'D011753': NtO(MH='Pyrones', MS='Keto-pyrans.'),
  'D011754': NtO(MH='Pyronine', MS='Xanthene dye used as a bacterial and biological stain. Synonyms: Pyronin; Pyronine G; Pyronine Y. Use also for Pyronine B. which is diethyl-rather than dimethylamino-.'),
  'D011755': NtO(MH='Pyrophosphatases', MS='A group of enzymes within the class EC 3.6.1.- that catalyze the hydrolysis of diphosphate bonds, chiefly in nucleoside di- and triphosphates. They may liberate either a mono- or diphosphate. EC 3.6.1.-.'),
  'D011756': NtO(MH='Diphosphates', MS='Inorganic salts of phosphoric acid that contain two phosphate groups.'),
  'D011758': NtO(MH='Pyrroles', MS='Azoles of one NITROGEN and two double bonds that have aromatic chemical properties.'),
  'D011759': NtO(MH='Pyrrolidines', MS='Compounds also known as tetrahydropyridines with general molecular formula (CH2)4NH.'),
  'D011760': NtO(MH='Pyrrolidinones', MS='A group of compounds that are derivatives of oxo-pyrrolidines. A member of this group is 2-oxo pyrrolidine, which is an intermediate in the manufacture of polyvinylpyrrolidone. (From Merck Index, 11th ed)'),
  'D011761': NtO(MH='Pyrrolidonecarboxylic Acid', MS='A cyclized derivative of L-GLUTAMIC ACID. Elevated blood levels may be associated with problems of GLUTAMINE or GLUTATHIONE metabolism.'),
  'D011762': NtO(MH='Pyrroline Carboxylate Reductases', MS='A group of enzymes that catalyze the reduction of 1-pyrroline carboxylate to proline in the presence of NAD(P)H. Includes both the 2-oxidoreductase (EC 1.5.1.1) and the 5-oxidoreductase (EC 1.5.1.2). The former also reduces 1-piperidine-2-carboxylate to pipecolate and the latter also reduces 1-pyrroline-3-hydroxy-5-carboxylate to hydroxyproline.'),
  'D011763': NtO(MH='Pyrrolizidine Alkaloids', MS='A group of ALKALOIDS, characterized by a nitrogen-containing necine, occurring mainly in plants of the BORAGINACEAE; COMPOSITAE; and LEGUMINOSAE plant families. They can be activated in the liver by hydrolysis of the ester and desaturation of the necine base to reactive electrophilic pyrrolic CYTOTOXINS.'),
  'D011764': NtO(MH='Pyrrolnitrin', MS='3-Chloro-4-(3-chloro-2-nitrophenyl)pyrrole. Antifungal antibiotic isolated from Pseudomonas pyrrocinia. It is effective mainly against Trichophyton, Microsporium, Epidermophyton, and Penicillium.'),
  'D011765': NtO(MH='Pyruvaldehyde', MS='An organic compound used often as a reagent in organic synthesis, as a flavoring agent, and in tanning. It has been demonstrated as an intermediate in the metabolism of acetone and its derivatives in isolated cell preparations, in various culture media, and in vivo in certain animals.'),
  'D011766': NtO(MH='Pyruvate Carboxylase', MS='A biotin-dependent enzyme belonging to the ligase family that catalyzes the addition of CARBON DIOXIDE to pyruvate. It is occurs in both plants and animals. Deficiency of this enzyme causes severe psychomotor retardation and ACIDOSIS, LACTIC in infants. EC 6.4.1.1.'),
  'D011767': NtO(MH='Pyruvate Decarboxylase', MS='Catalyzes the decarboxylation of an alpha keto acid to an aldehyde and carbon dioxide. Thiamine pyrophosphate is an essential cofactor. In lower organisms, which ferment glucose to ethanol and carbon dioxide, the enzyme irreversibly decarboxylates pyruvate to acetaldehyde. EC 4.1.1.1.'),
  'D011768': NtO(MH='Pyruvate Dehydrogenase Complex', MS='A multienzyme complex responsible for the formation of ACETYL COENZYME A from pyruvate. The enzyme components are PYRUVATE DEHYDROGENASE (LIPOAMIDE); dihydrolipoamide acetyltransferase; and LIPOAMIDE DEHYDROGENASE. Pyruvate dehydrogenase complex is subject to three types of control: inhibited by acetyl-CoA and NADH; influenced by the energy state of the cell; and inhibited when a specific serine residue in the pyruvate decarboxylase is phosphorylated by ATP. PYRUVATE DEHYDROGENASE (LIPOAMIDE)-PHOSPHATASE catalyzes reactivation of the complex. (From Concise Encyclopedia Biochemistry and Molecular Biology, 3rd ed)'),
  'D011769': NtO(MH='Pyruvate Dehydrogenase (Lipoamide)-Phosphatase', MS='(Pyruvate dehydrogenase (lipoamide))-phosphate phosphohydrolase. A mitochondrial enzyme that catalyzes the hydrolytic removal of a phosphate on a specific seryl hydroxyl group of pyruvate dehydrogenase, reactivating the enzyme complex. EC 3.1.3.43.'),
  'D011770': NtO(MH='Pyruvate Kinase', MS='ATP:pyruvate 2-O-phosphotransferase. A phosphotransferase that catalyzes reversibly the phosphorylation of pyruvate to phosphoenolpyruvate in the presence of ATP. It has four isozymes (L, R, M1, and M2). Deficiency of the enzyme results in hemolytic anemia. EC 2.7.1.40.'),
  'D011771': NtO(MH='Pyruvate Oxidase', MS=None),
  'D011772': NtO(MH='Pyruvate, Orthophosphate Dikinase', MS='An enzyme that catalyzes the reaction of ATP, pyruvate, and orthophosphate to form AMP plus phosphoenolpyruvate plus pyrophosphate. EC 2.7.9.1.'),
  'D011773': NtO(MH='Pyruvates', MS='Derivatives of PYRUVIC ACID, including its salts and esters.'),
  'D011774': NtO(MH='Pyrvinium Compounds', MS=None),
  'D011775': NtO(MH='Pythium', MS='A genus of destructive root-parasitic OOMYCETES in the family Pythiaceae, order Peronosporales, commonly found in cultivated soils all over the world. Differentiation of zoospores takes place in a vesicle.'),
  'D011776': NtO(MH='Pyuria', MS='The presence of white blood cells (LEUKOCYTES) in the urine. It is often associated with bacterial infections of the urinary tract. Pyuria without BACTERIURIA can be caused by TUBERCULOSIS, stones, or cancer.'),
  'D011777': NtO(MH='Q beta Replicase', MS='An enzyme that catalyzes the replication of the RNA of coliphage Q beta. EC 2.7.7.-.'),
  'D011778': NtO(MH='Q Fever', MS='An acute infectious disease caused by COXIELLA BURNETII. It is characterized by a sudden onset of FEVER; HEADACHE; malaise; and weakness. In humans, it is commonly contracted by inhalation of infected dusts derived from infected domestic animals (ANIMALS, DOMESTIC).'),
  'D011779': NtO(MH='Q-Sort', MS='A personality assessment technique in which the subject or observer indicates the degree to which a standardized set of descriptive statements actually describes the subject. The term reflects "sorting" procedures occasionally used with this technique.'),
  'D011780': NtO(MH='Qatar', MS='A country in the Middle East, a peninsula bordering the Persian Gulf and Saudi Arabia. The capital is Doha.'),
  'D011781': NtO(MH='Quackery', MS='The fraudulent misrepresentation of the diagnosis and treatment of disease.'),
  'D011782': NtO(MH='Quadriplegia', MS='Severe or complete loss of motor function in all four limbs which may result from BRAIN DISEASES; SPINAL CORD DISEASES; PERIPHERAL NERVOUS SYSTEM DISEASES; NEUROMUSCULAR DISEASES; or rarely MUSCULAR DISEASES. The locked-in syndrome is characterized by quadriplegia in combination with cranial muscle paralysis. Consciousness is spared and the only retained voluntary motor activity may be limited eye movements. This condition is usually caused by a lesion in the upper BRAIN STEM which injures the descending cortico-spinal and cortico-bulbar tracts.'),
  'D011783': NtO(MH='Quadruplets', MS='Four individuals derived from four FETUSES that were fertilized at or about the same time, developed in the UTERUS simultaneously, and born to the same mother.'),
  'D011784': NtO(MH='Quail', MS='Common name for two distinct groups of BIRDS in the order GALLIFORMES: the New World or American quails of the family Odontophoridae and the Old World quails in the genus COTURNIX, family Phasianidae.'),
  'D011785': NtO(MH='Quality Assurance, Health Care', MS='Activities and programs intended to assure or improve the quality of care in either a defined medical setting or a program. The concept includes the assessment or evaluation of the quality of care; identification of problems or shortcomings in the delivery of care; designing activities to overcome these deficiencies; and follow-up monitoring to ensure effectiveness of corrective steps.'),
  'D011786': NtO(MH='Quality Control', MS='A system for verifying and maintaining a desired level of quality in a product or process by careful planning, use of proper equipment, continued inspection, and corrective action as required. (Random House Unabridged Dictionary, 2d ed)'),
  'D011787': NtO(MH='Quality of Health Care', MS='The levels of excellence which characterize the health service or health care provided based on accepted standards of quality.'),
  'D011788': NtO(MH='Quality of Life', MS='A generic concept reflecting concern with the modification and enhancement of life attributes, e.g., physical, political, moral, social environment as well as health and disease.'),
  'D011789': NtO(MH='Quantum Theory', MS="The theory that the radiation and absorption of energy take place in definite quantities called quanta (E) which vary in size and are defined by the equation E=hv in which h is Planck's constant and v is the frequency of the radiation."),
  'D011790': NtO(MH='Quarantine', MS="Restriction of freedom of movement of individuals who have been exposed to infectious or communicable disease in order to prevent its spread; a period of detention of vessels, vehicles, or travelers coming from infected or suspected places; and detention or isolation on account of suspected contagion. It includes government regulations on the detention of animals at frontiers or ports of entrance for the prevention of infectious disease, through a period of isolation before being allowed to enter a country. (From Dorland, 28th ed & Black's Veterinary Dictionary, 17th ed)"),
  'D011791': NtO(MH='Quartz', MS="Quartz (SiO2). A glassy or crystalline form of silicon dioxide. Many colored varieties are semiprecious stones. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D011792': NtO(MH='Quebec', MS="A province of eastern Canada. Its capital is Quebec. The region belonged to France from 1627 to 1763 when it was lost to the British. The name is from the Algonquian quilibek meaning the place where waters narrow, referring to the gradually narrowing channel of the St. Lawrence or to the narrows of the river at Cape Diamond. (From Webster's New Geographical Dictionary, 1988, p993 & Room, Brewer's Dictionary of Names, 1992, p440)"),
  'D011793': NtO(MH='Queensland', MS="A state in northeastern Australia. Its capital is Brisbane. Its coast was first visited by Captain Cook in 1770 and its first settlement (penal) was located on Moreton Bay in 1824. The name Cooksland was first proposed but honor to Queen Victoria prevailed. (From Webster's New Geographical Dictionary, 1988, p996 & Room, Brewer's Dictionary of Names, 1992, p441)"),
  'D011794': NtO(MH='Quercetin', MS='A flavonol widely distributed in plants. It is an antioxidant, like many other phenolic heterocyclic compounds. Glycosylated forms include RUTIN and quercetrin.'),
  'D011795': NtO(MH='Surveys and Questionnaires', MS='Collections of data obtained from voluntary subjects. The information usually takes the form of answers to questions, or suggestions.'),
  'D011796': NtO(MH='Quinacrine', MS='An acridine derivative formerly widely used as an antimalarial but superseded by chloroquine in recent years. It has also been used as an anthelmintic and in the treatment of giardiasis and malignant effusions. It is used in cell biological experiments as an inhibitor of phospholipase A2.'),
  'D011797': NtO(MH='Quinacrine Mustard', MS='Nitrogen mustard analog of quinacrine used primarily as a stain in the studies of chromosomes and chromatin. Fluoresces by reaction with nucleic acids in chromosomes.'),
  'D011798': NtO(MH='Quinaldines', MS=None),
  'D011799': NtO(MH='Quinazolines', MS=None),
  'D011800': NtO(MH='Quinestrol', MS='The 3-cyclopentyl ether of ETHINYL ESTRADIOL. After gastrointestinal absorption, it is stored in ADIPOSE TISSUE, slowly released, and metabolized principally to the parent compound. It has been used in ESTROGEN REPLACEMENT THERAPY. (From AMA Drug Evaluations Annual, 1992, p1011)'),
  'D011801': NtO(MH='Quinic Acid', MS='An acid which is found in cinchona bark and elsewhere in plants. (From Stedman, 26th ed)'),
  'D011802': NtO(MH='Quinidine', MS='An optical isomer of quinine, extracted from the bark of the CHINCHONA tree and similar plant species. This alkaloid dampens the excitability of cardiac and skeletal muscles by blocking sodium and potassium currents across cellular membranes. It prolongs cellular ACTION POTENTIALS, and decreases automaticity. Quinidine also blocks muscarinic and alpha-adrenergic neurotransmission.'),
  'D011803': NtO(MH='Quinine', MS='An alkaloid derived from the bark of the cinchona tree. It is used as an antimalarial drug, and is the active ingredient in extracts of the cinchona that have been used for that purpose since before 1633. Quinine is also a mild antipyretic and analgesic and has been used in common cold preparations for that purpose. It was used commonly and as a bitter and flavoring agent, and is still useful for the treatment of babesiosis. Quinine is also useful in some muscular disorders, especially nocturnal leg cramps and myotonia congenita, because of its direct effects on muscle membrane and sodium channels. The mechanisms of its antimalarial effects are not well understood.'),
  'D011804': NtO(MH='Quinolines', MS=None),
  'D011805': NtO(MH='Quinolinic Acids', MS='Dicarboxylic acids with a PYRIDINE backbone. Quinolinic Acids are downstream products of the KYNURENINE pathway which metabolize amino acid TRYPTOPHAN.'),
  'D011806': NtO(MH='Quinolinium Compounds', MS=None),
  'D011807': NtO(MH='Quinolizines', MS=None),
  'D011808': NtO(MH='Quinone Reductases', MS='NAD(P)H:(quinone acceptor) oxidoreductases. A family that includes three enzymes which are distinguished by their sensitivity to various inhibitors. EC 1.6.99.2 (NAD(P)H DEHYDROGENASE (QUINONE);) is a flavoprotein which reduces various quinones in the presence of NADH or NADPH and is inhibited by dicoumarol. EC 1.6.99.5 (NADH dehydrogenase (quinone)) requires NADH, is inhibited by AMP and 2,4-dinitrophenol but not by dicoumarol or folic acid derivatives. EC 1.6.99.6 (NADPH dehydrogenase (quinone)) requires NADPH and is inhibited by dicoumarol and folic acid derivatives but not by 2,4-dinitrophenol.'),
  'D011809': NtO(MH='Quinones', MS='Hydrocarbon rings which contain two ketone moieties in any position. They can be substituted in any position except at the ketone groups.'),
  'D011810': NtO(MH='Quinoxalines', MS=None),
  'D011811': NtO(MH='Quintuplets', MS='Five individuals derived from five FETUSES that were fertilized at or about the same time, developed in the UTERUS simultaneously, and born to the same mother.'),
  'D011812': NtO(MH='Quinuclidines', MS=None),
  'D011813': NtO(MH='Quinuclidinyl Benzilate', MS='A high-affinity muscarinic antagonist commonly used as a tool in animal and tissue studies.'),
  'D011814': NtO(MH='Quipazine', MS='A pharmacologic congener of serotonin that contracts smooth muscle and has actions similar to those of tricyclic antidepressants. It has been proposed as an oxytocic.'),
  'D011815': NtO(MH='R Factors', MS='A class of plasmids that transfer antibiotic resistance from one bacterium to another by conjugation.'),
  'D011817': NtO(MH='Rabbits', MS='A burrowing plant-eating mammal with hind limbs that are longer than its fore limbs. It belongs to the family Leporidae of the order Lagomorpha, and in contrast to hares, possesses 22 instead of 24 pairs of chromosomes.'),
  'D011818': NtO(MH='Rabies', MS='Acute VIRAL CNS INFECTION affecting mammals, including humans. It is caused by RABIES VIRUS and usually spread by contamination with virus-laden saliva of bites inflicted by rabid animals. Important animal vectors include the dog, cat, bat, fox, raccoon, skunk, and wolf.'),
  'D011819': NtO(MH='Rabies Vaccines', MS='Vaccines or candidate vaccines used to prevent and treat RABIES. The inactivated virus vaccine is used for preexposure immunization to persons at high risk of exposure, and in conjunction with rabies immunoglobulin, for postexposure prophylaxis.'),
  'D011820': NtO(MH='Rabies virus', MS='The type species of LYSSAVIRUS causing rabies in humans and other animals. Transmission is mostly by animal bites through saliva. The virus is neurotropic multiplying in neurons and myotubes of vertebrates.'),
  'D011821': NtO(MH='Raccoons', MS='Carnivores of the genus Procyon of the family PROCYONIDAE. Two subgenera and seven species are currently recognized. They range from southern Canada to Panama and are found in several of the Caribbean Islands.'),
  'D011822': NtO(MH='Race Relations', MS='Cultural contacts between people of different races.'),
  'D011825': NtO(MH='Radar', MS='A system using beamed and reflected radio signals to and from an object in such a way that range, bearing, and other characteristics of the object may be determined.'),
  'D011826': NtO(MH='Radial Nerve', MS='A major nerve of the upper extremity. In humans the fibers of the radial nerve originate in the lower cervical and upper thoracic spinal cord (usually C5 to T1), travel via the posterior cord of the brachial plexus, and supply motor innervation to extensor muscles of the arm and cutaneous sensory fibers to extensor regions of the arm and hand.'),
  'D011827': NtO(MH='Radiation', MS='Emission or propagation of acoustic waves (SOUND), ELECTROMAGNETIC ENERGY waves (such as LIGHT; RADIO WAVES; GAMMA RAYS; or X-RAYS), or a stream of subatomic particles (such as ELECTRONS; NEUTRONS; PROTONS; or ALPHA PARTICLES).'),
  'D011828': NtO(MH='Radiation Chimera', MS="An organism whose body contains cell populations of different genotypes as a result of the TRANSPLANTATION of donor cells after sufficient ionizing radiation to destroy the mature recipient's cells which would otherwise reject the donor cells."),
  'D011829': NtO(MH='Radiation Dosage', MS='The amount of radiation energy that is deposited in a unit mass of material, such as tissues of plants or animal. In RADIOTHERAPY, radiation dosage is expressed in gray units (Gy). In RADIOLOGIC HEALTH, the dosage is expressed by the product of absorbed dose (Gy) and quality factor (a function of linear energy transfer), and is called radiation dose equivalent in sievert units (Sv).'),
  'D011830': NtO(MH='Radiation Effects', MS='The effects of ionizing and nonionizing radiation upon living organisms, organs and tissues, and their constituents, and upon physiologic processes. It includes the effect of irradiation on food, drugs, and chemicals.'),
  'D011831': NtO(MH='Radiation Genetics', MS='A subdiscipline of genetics that studies RADIATION EFFECTS on the components and processes of biological inheritance.'),
  'D011832': NtO(MH='Radiation Injuries', MS='Harmful effects of non-experimental exposure to ionizing or non-ionizing radiation in VERTEBRATES.'),
  'D011833': NtO(MH='Radiation Injuries, Experimental', MS='Experimentally produced harmful effects of ionizing or non-ionizing RADIATION in CHORDATA animals.'),
  'D011834': NtO(MH='Radiation Monitoring', MS='The observation, either continuously or at intervals, of the levels of radiation in a given area, generally for the purpose of assuring that they have not exceeded prescribed amounts or, in case of radiation already present in the area, assuring that the levels have returned to those meeting acceptable safety standards.'),
  'D011835': NtO(MH='Radiation Protection', MS='Methods and practices adopted to protect against RADIATION.'),
  'D011836': NtO(MH='Radiation Tolerance', MS='The ability of some cells or tissues to survive lethal doses of IONIZING RADIATION. Tolerance depends on the species, cell type, and physical and chemical variables, including RADIATION-PROTECTIVE AGENTS and RADIATION-SENSITIZING AGENTS.'),
  'D011837': NtO(MH='Radiation-Protective Agents', MS='Drugs used to protect against ionizing radiation. They are usually of interest for use in radiation therapy but have been considered for other, e.g. military, purposes.'),
  'D011838': NtO(MH='Radiation-Sensitizing Agents', MS='Drugs used to potentiate the effectiveness of radiation therapy in destroying unwanted cells.'),
  'D011839': NtO(MH='Radiation, Ionizing', MS='ELECTROMAGNETIC RADIATION or particle radiation (high energy ELEMENTARY PARTICLES) capable of directly or indirectly producing IONS in its passage through matter. The wavelengths of ionizing electromagnetic radiation are equal to or smaller than those of short (far) ultraviolet radiation and include gamma and X-rays.'),
  'D011840': NtO(MH='Radiation, Nonionizing', MS='ELECTROMAGNETIC RADIATION or sonic radiation (SOUND WAVES) which does not produce IONS in matter through which it passes. The wavelengths of non-ionizing electromagentic radiation are generally longer than those of far ultraviolet radiation and range through the longest RADIO WAVES.'),
  'D011842': NtO(MH='Radicular Cyst', MS='Slow-growing fluid-filled epithelial sac at the apex of a tooth with a nonvital pulp or defective root canal filling.'),
  'D011843': NtO(MH='Radiculopathy', MS='Disease involving a spinal nerve root (see SPINAL NERVE ROOTS) which may result from compression related to INTERVERTEBRAL DISK DISPLACEMENT; SPINAL CORD INJURIES; SPINAL DISEASES; and other conditions. Clinical manifestations include radicular pain, weakness, and sensory loss referable to structures innervated by the involved nerve root.'),
  'D011844': NtO(MH='Radiesthesia', MS='Therapeutic cult concerned with intangible energies surrounding the living body and based on the detection of these intrinsic radiations by dowsing, or divining, or the use of more elaborate instruments (radionics).'),
  'D011845': NtO(MH='Radio', MS="The transmission and reception of electric impulses or signals by means of electric waves without a connecting wire, or the use of these waves for the wireless transmission of electric impulses into which sound is converted. (From Webster's 3d)"),
  'D011846': NtO(MH='Radio Waves', MS='Electromagnetic waves with frequencies between about 3 kilohertz (very low frequency - VLF) and 300,000 megahertz (extremely high frequency - EHF). They are used in television and radio broadcasting, land and satellite communications systems, radionavigation, radiolocation, and DIATHERMY. The highest frequency radio waves are MICROWAVES.'),
  'D011847': NtO(MH='Radioactive Fallout', MS='The material that descends to the earth or water well beyond the site of a surface or subsurface nuclear explosion. (McGraw-Hill Dictionary of Chemical and Technical Terms, 4th ed)'),
  'D011848': NtO(MH='Radioactive Pollutants', MS='Radioactive substances which act as pollutants. They include chemicals whose radiation is released via radioactive waste, nuclear accidents, fallout from nuclear explosions, and the like.'),
  'D011849': NtO(MH='Radioactive Tracers', MS="Radioactive substances added in minute amounts to the reacting elements or compounds in a chemical process and traced through the process by appropriate detection methods, e.g., Geiger counter. Compounds containing tracers are often said to be tagged or labeled. (Hawley's Condensed Chemical Dictionary, 12th ed)"),
  'D011850': NtO(MH='Radioactive Waste', MS='Liquid, solid, or gaseous waste resulting from mining of radioactive ore, production of reactor fuel materials, reactor operation, processing of irradiated reactor fuels, and related operations, and from use of radioactive materials in research, industry, and medicine. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D011851': NtO(MH='Radioactivity', MS='The spontaneous transformation of a nuclide into one or more different nuclides, accompanied by either the emission of particles from the nucleus, nuclear capture or ejection of orbital electrons, or fission. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D011852': NtO(MH='Radioallergosorbent Test', MS='An in vitro allergen radioimmunoassay in which allergens are coupled to an immunosorbent. The coupled allergens bind the IgE in the sera of patients which in turn binds radioisotope-labeled anti-IMMUNOGLOBULIN E antibodies.'),
  'D011853': NtO(MH='Radiobiology', MS='Study of the scientific principles, mechanisms, and effects of the interaction of ionizing radiation with living matter. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D011854': NtO(MH='Radiochemistry', MS='The study of the chemical and physical phenomena of radioactive substances.'),
  'D011855': NtO(MH='Radiodermatitis', MS='A cutaneous inflammatory reaction occurring as a result of exposure to ionizing radiation.'),
  'D011856': NtO(MH='Radiographic Image Enhancement', MS='Improvement in the quality of an x-ray image by use of an intensifying screen, tube, or filter and by optimum exposure techniques. Digital processing methods are often employed.'),
  'D011857': NtO(MH='Radiographic Image Interpretation, Computer-Assisted', MS='Computer systems or networks designed to provide radiographic interpretive information.'),
  'D011858': NtO(MH='Radiographic Magnification', MS='Use of optic and geometric techniques to enhance radiographic image quality and interpretation. It includes use of microfocal X-ray tubes and intensifying fluoroscopic screens.'),
  'D011859': NtO(MH='Radiography', MS='Examination of any part of the body for diagnostic purposes by means of X-RAYS or GAMMA RAYS, recording the image on a sensitized surface (such as photographic film).'),
  'D011860': NtO(MH='Radiography, Abdominal', MS='Radiographic visualization of the body between the thorax and the pelvis, i.e., within the peritoneal cavity.'),
  'D011861': NtO(MH='Radiography, Dental', MS='Radiographic techniques used in dentistry.'),
  'D011862': NtO(MH='Radiography, Panoramic', MS='Extraoral body-section radiography depicting an entire maxilla, or both maxilla and mandible, on a single film.'),
  'D011863': NtO(MH='Radioimmunoassay', MS='Classic quantitative assay for detection of antigen-antibody reactions using a radioactively labeled substance (radioligand) either directly or indirectly to measure the binding of the unlabeled substance to a specific antibody or other receptor system. Non-immunogenic substances (e.g., haptens) can be measured if coupled to larger carrier proteins (e.g., bovine gamma-globulin or human serum albumin) capable of inducing antibody formation.'),
  'D011864': NtO(MH='Radioimmunosorbent Test', MS='Radioimmunoassay of proteins using antibody coupled to an immunosorbent.'),
  'D011865': NtO(MH='Radioisotope Dilution Technique', MS='Method for assessing flow through a system by injection of a known quantity of radionuclide into the system and monitoring its concentration over time at a specific point in the system. (From Dorland, 28th ed)'),
  'D011866': NtO(MH='Radioisotope Renography', MS='Graphic tracing over a time period of radioactivity measured externally over the kidneys following intravenous injection of a radionuclide which is taken up and excreted by the kidneys.'),
  'D011867': NtO(MH='Radioisotope Teletherapy', MS='A type of high-energy radiotherapy using a beam of gamma-radiation produced by a radioisotope source encapsulated within a teletherapy unit.'),
  'D011868': NtO(MH='Radioisotopes', MS="Isotopes that exhibit radioactivity and undergo radioactive decay. (From Grant & Hackh's Chemical Dictionary, 5th ed & McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)"),
  'D011869': NtO(MH='Radioligand Assay', MS='Quantitative determination of receptor (binding) proteins in body fluids or tissue using radioactively labeled binding reagents (e.g., antibodies, intracellular receptors, plasma binders).'),
  'D011870': NtO(MH='Radiologic Health', MS='Health concerns associated with the effects of radiation on the environment and on public and personal health.'),
  'D011871': NtO(MH='Radiology', MS='A specialty concerned with the use of x-ray and other forms of radiant energy in the diagnosis and treatment of disease.'),
  'D011872': NtO(MH='Radiology Department, Hospital', MS='Hospital department which is responsible for the administration and provision of x-ray diagnostic and therapeutic services.'),
  'D011873': NtO(MH='Radiology Information Systems', MS='Information systems, usually computer-assisted, designed to store, manipulate, and retrieve information for planning, organizing, directing, and controlling administrative activities associated with the provision and utilization of radiology services and facilities.'),
  'D011874': NtO(MH='Radiometry', MS='The measurement of radiation by photography, as in x-ray film and film badge, by Geiger-Mueller tube, and by SCINTILLATION COUNTING.'),
  'D011875': NtO(MH='Radionuclide Angiography', MS='The measurement of visualization by radiation of any organ after a radionuclide has been injected into its blood supply. It is used to diagnose heart, liver, lung, and other diseases and to measure the function of those organs, except renography, for which RADIOISOTOPE RENOGRAPHY is available.'),
  'D011876': NtO(MH='Radionuclide Generators', MS='Separation systems containing a relatively long-lived parent radionuclide which produces a short-lived daughter in its decay scheme. The daughter can be periodically extracted (milked) by means of an appropriate eluting agent.'),
  'D011877': NtO(MH='Radionuclide Imaging', MS='The production of an image obtained by cameras that detect the radioactive emissions of an injected radionuclide as it has distributed differentially throughout tissues in the body. The image obtained from a moving detector is called a scan, while the image obtained from a stationary camera device is called a scintiphotograph.'),
  'D011878': NtO(MH='Radiotherapy', MS='The use of IONIZING RADIATION to treat malignant NEOPLASMS and some benign conditions.'),
  'D011879': NtO(MH='Radiotherapy Dosage', MS='The total amount of radiation absorbed by tissues as a result of radiotherapy.'),
  'D011880': NtO(MH='Radiotherapy Planning, Computer-Assisted', MS='Computer-assisted mathematical calculations of beam angles, intensities of radiation, and duration of irradiation in radiotherapy.'),
  'D011881': NtO(MH='Radiotherapy, Computer-Assisted', MS='Computer systems or programs used in accurate computations for providing radiation dosage treatment to patients.'),
  'D011882': NtO(MH='Radiotherapy, High-Energy', MS='Radiotherapy using high-energy (megavolt or higher) ionizing radiation. Types of radiation include gamma rays, produced by a radioisotope within a teletherapy unit; x-rays, electrons, protons, alpha particles (helium ions) and heavy charged ions, produced by particle acceleration; and neutrons and pi-mesons (pions), produced as secondary particles following bombardment of a target with a primary particle.'),
  'D011883': NtO(MH='Radium', MS='A radioactive element of the alkaline earth series of metals. It has the atomic symbol Ra and atomic number 88. Radium is the product of the disintegration of URANIUM and is present in pitchblende and all ores containing uranium. It is used clinically as a source of beta and gamma-rays in radiotherapy, particularly BRACHYTHERAPY.'),
  'D011884': NtO(MH='Radius', MS='The outer shorter of the two bones of the FOREARM, lying parallel to the ULNA and partially revolving around it.'),
  'D011885': NtO(MH='Radius Fractures', MS='Fractures of the RADIUS.'),
  'D011886': NtO(MH='Radon', MS='A naturally radioactive element with atomic symbol Rn, and atomic number 86. It is a member of the noble gas family found in soil, and is released during the decay of RADIUM.'),
  'D011887': NtO(MH='Raffinose', MS='A trisaccharide occurring in Australian manna (from Eucalyptus spp, Myrtaceae) and in cottonseed meal.'),
  'D011888': NtO(MH='Rafoxanide', MS='Veterinary anthelmintic for grazing animals; used to treat fluke, hookworm and other infestations.'),
  'D011889': NtO(MH='Rage', MS='Fury; violent, intense anger.'),
  'D011890': NtO(MH='Railroads', MS="Permanent roads having a line of rails fixed to ties and laid to gage, usually on a leveled or graded ballasted roadbed and providing a track for freight cars, passenger cars, and other rolling stock. Cars are designed to be drawn by locomotives or sometimes propelled by self-contained motors. (From Webster's 3d) The concept includes the organizational and administrative aspects of railroads as well."),
  'D011891': NtO(MH='Rain', MS='Water particles that fall from the ATMOSPHERE.'),
  'D011892': NtO(MH='Rana catesbeiana', MS='A species of the family Ranidae (true frogs). The only anuran properly referred to by the common name "bullfrog", it is the largest native anuran in North America.'),
  'D011893': NtO(MH='Rana esculenta', MS='An edible species of the family Ranidae, occurring in Europe and used extensively in biomedical research. Commonly referred to as "edible frog".'),
  'D011894': NtO(MH='Rana pipiens', MS='A highly variable species of the family Ranidae in Canada, the United States and Central America. It is the most widely used Anuran in biomedical research.'),
  'D011895': NtO(MH='Rana ridibunda', MS='A species of the family Ranidae which occurs primarily in Europe and is used widely in biomedical research.'),
  'D011896': NtO(MH='Rana temporaria', MS='A species of the family Ranidae occurring in a wide variety of habitats from within the Arctic Circle to South Africa, Australia, etc.'),
  'D011897': NtO(MH='Random Allocation', MS='A process involving chance used in therapeutic trials or other research endeavor for allocating experimental subjects, human or animal, between treatment and control groups, or among treatment groups. It may also apply to experiments on inanimate objects.'),
  'D011898': NtO(MH='Ranidae', MS='The family of true frogs of the order Anura. The family occurs worldwide except in Antarctica.'),
  'D011899': NtO(MH='Ranitidine', MS='A non-imidazole blocker of those histamine receptors that mediate gastric secretion (H2 receptors). It is used to treat gastrointestinal ulcers.'),
  'D011900': NtO(MH='Ranula', MS='A form of retention cyst of the floor of the mouth, usually due to obstruction of the ducts of the submaxillary or sublingual glands, presenting a slowly enlarging painless deep burrowing mucocele of one side of the mouth. It is also called sublingual cyst and sublingual ptyalocele.'),
  'D011901': NtO(MH="Ranvier's Nodes", MS="Regularly spaced gaps in the myelin sheaths of peripheral axons. Ranvier's nodes allow saltatory conduction, that is, jumping of impulses from node to node, which is faster and more energetically favorable than continuous conduction."),
  'D011902': NtO(MH='Rape', MS='Sexual intercourse without consent of the victim.'),
  'D011903': NtO(MH='Raphe Nuclei', MS='Collections of small neurons centrally scattered among many fibers from the level of the TROCHLEAR NUCLEUS in the midbrain to the hypoglossal area in the MEDULLA OBLONGATA.'),
  'D011904': NtO(MH='Rare Books', MS='Books that are limited in availability, have early printing dates or special characteristics that increase the value of an individual copy, print run, or edition.'),
  'D011905': NtO(MH='Genes, ras', MS='Family of retrovirus-associated DNA sequences (ras) originally isolated from Harvey (H-ras, Ha-ras, rasH) and Kirsten (K-ras, Ki-ras, rasK) murine sarcoma viruses. Ras genes are widely conserved among animal species and sequences corresponding to both H-ras and K-ras genes have been detected in human, avian, murine, and non-vertebrate genomes. The closely related N-ras gene has been detected in human neuroblastoma and sarcoma cell lines. All genes of the family have a similar exon-intron structure and each encodes a p21 protein.'),
  'D011906': NtO(MH='Rat-Bite Fever', MS='A syndrome characterized by recurring fever, rash, and arthralgias occurring days to weeks after a rat bite. The causative agents are either Streptobacillus moniliformis or Spirillum minus.'),
  'D011907': NtO(MH='Rate Setting and Review', MS='A method of examining and setting levels of payments.'),
  'D011908': NtO(MH='Rationalization', MS='A defense mechanism operating unconsciously, in which the individual attempts to justify or make consciously tolerable, by plausible means, feelings, behavior, and motives that would otherwise be intolerable.'),
  'D011910': NtO(MH='Rats, Brattleboro', MS='A mutant strain of Rattus norvegicus used in research on renal function and hypertension and as a disease model for diabetes insipidus.'),
  'D011911': NtO(MH='Rats, Gunn', MS='Mutant strain of Rattus norvegicus which is used as a disease model of kernicterus.'),
  'D011912': NtO(MH='Rats, Inbred ACI', MS='An inbred strain of rat that is widely used in BIOMEDICAL RESEARCH. Applications include the study of spontaneous NEOPLASMS; CHRONIC KIDNEY DISEASES, and CONGENITAL ABNORMALITIES.'),
  'D011913': NtO(MH='Rats, Inbred BB', MS='A strain of Rattus norvegicus which is a model for spontaneous insulin-dependent diabetes mellitus (DIABETES MELLITUS, INSULIN-DEPENDENT).'),
  'D011914': NtO(MH='Rats, Inbred BN', MS='An inbred strain of rat that is widely used in a variety of research areas such as the study of ASTHMA; CARCINOGENESIS; AGING; and LEUKEMIA.'),
  'D011915': NtO(MH='Rats, Inbred BUF', MS='An inbred strain of rat that is used for cancer research, particularly  the study of  CARCINOGENESIS'),
  'D011916': NtO(MH='Rats, Inbred F344', MS='An inbred strain of rat that is used for general BIOMEDICAL RESEARCH purposes.'),
  'D011917': NtO(MH='Rats, Inbred Lew', MS='An inbred strain of rat that is used in BIOMEDICAL RESEARCH.'),
  'D011918': NtO(MH='Rats, Inbred SHR', MS='A strain of Rattus norvegicus with elevated blood pressure used as a model for studying hypertension and stroke.'),
  'D011919': NtO(MH='Rats, Inbred Strains', MS='Genetically identical individuals developed from brother and sister matings which have been carried out for twenty or more generations or by parent x offspring matings carried out with certain restrictions. This also includes animals with a long history of closed colony breeding.'),
  'D011920': NtO(MH='Rats, Inbred WF', MS='An inbred strain of rat that is used in BIOMEDICAL RESEARCH.'),
  'D011921': NtO(MH='Rats, Inbred WKY', MS='A strain of Rattus norvegicus used as a normotensive control for the spontaneous hypertensive rats (SHR).'),
  'D011922': NtO(MH='Rats, Mutant Strains', MS='Rats bearing mutant genes which are phenotypically expressed in the animals.'),
  'D011923': NtO(MH='Rats, Nude', MS='A mutant strain of Rattus norvegicus without a thymus and with depressed or absent T-cell function. This strain of rats may have a small amount of hair at times, but then lose it.'),
  'D011924': NtO(MH='Rats, Zucker', MS='Two populations of Zucker rats have been cited in research--the "fatty" or obese and the lean. The "fatty" rat (Rattus norvegicus) appeared as a spontaneous mutant. The obese condition appears to be due to a single recessive gene.'),
  'D011925': NtO(MH='Rauscher Virus', MS='A strain of MURINE LEUKEMIA VIRUS associated with mouse tumors similar to those caused by the FRIEND MURINE LEUKEMIA VIRUS. It is a replication-competent murine leukemia virus. It can act as a helper virus when complexing with a defective transforming component, RAUSCHER SPLEEN FOCUS-FORMING VIRUS.'),
  'D011926': NtO(MH='Rauwolfia', MS='A plant genus of the APOCYNACEAE or dogbane family. Alkaloids from plants in this genus have been used as tranquilizers and antihypertensive agents. RESERPINE is derived from R. serpentina.'),
  'D011928': NtO(MH='Raynaud Disease', MS='An idiopathic vascular disorder characterized by bilateral Raynaud phenomenon, the abrupt onset of digital paleness or CYANOSIS in response to cold exposure or stress.'),
  'D011929': NtO(MH='Razoxane', MS='An antimitotic agent with immunosuppressive properties.'),
  'D011930': NtO(MH='Reaction Time', MS='The time from the onset of a stimulus until a response is observed.'),
  'D011931': NtO(MH='Reactive Inhibition', MS='Tendency toward a lessened strength of response due to practice or activity. It is independent of the effect of reward and is a direct function of time interval since the last response and the number of preceding responses.'),
  'D011932': NtO(MH='Reading', MS='Acquiring information from text.'),
  'D011933': NtO(MH='Reagent Kits, Diagnostic', MS='Commercially prepared reagent sets, with accessory devices, containing all of the major components and literature necessary to perform one or more designated diagnostic tests or procedures. They may be for laboratory or personal use.'),
  'D011934': NtO(MH='Reagent Strips', MS='Narrow pieces of material impregnated or covered with a substance used to produce a chemical reaction. The strips are used in detecting, measuring, producing, etc., other substances. (From Dorland, 28th ed)'),
  'D011935': NtO(MH='Reagins', MS='Antibodies, especially IGE, that bind to tissue of the same species so that ANTIGENS induce release of HISTAMINE and other vasoactive agents. HYPERSENSITIVITY is the clinical manifestation.'),
  'D011936': NtO(MH='Reality Testing', MS="The individual's objective evaluation of the external world and the ability to differentiate adequately between it and the internal world; considered to be a primary ego function."),
  'D011937': NtO(MH='Reality Therapy', MS="Method of psychotherapeutic treatment based on assumption of patients' personal responsibility for their own behavior. The therapist actively guides patients to accurate self-perception for fulfillment of needs of self-worth and respect for others. (From APA, Thesaurus of Psychological Index Terms, 8th ed.)"),
  'D011938': NtO(MH='Rec A Recombinases', MS='A family of recombinases initially identified in BACTERIA. They catalyze the ATP-driven exchange of DNA strands in GENETIC RECOMBINATION. The product of the reaction consists of a duplex and a displaced single-stranded loop, which has the shape of the letter D and is therefore called a D-loop structure.'),
  'D011939': NtO(MH='Mental Recall', MS='The process whereby a representation of past experience is elicited.'),
  'D011940': NtO(MH='Receptor Aggregation', MS='Chemically stimulated aggregation of cell surface receptors, which potentiates the action of the effector cell.'),
  'D011941': NtO(MH='Receptors, Adrenergic', MS='Cell-surface proteins that bind epinephrine and/or norepinephrine with high affinity and trigger intracellular changes. The two major classes of adrenergic receptors, alpha and beta, were originally discriminated based on their cellular actions but now are distinguished by their relative affinity for characteristic synthetic ligands. Adrenergic receptors may also be classified according to the subtypes of G-proteins with which they bind; this scheme does not respect the alpha-beta distinction.'),
  'D011942': NtO(MH='Receptors, Adrenergic, alpha', MS='One of the two major pharmacological subdivisions of adrenergic receptors that were originally defined by the relative potencies of various adrenergic compounds. The alpha receptors were initially described as excitatory receptors that post-junctionally stimulate SMOOTH MUSCLE contraction. However, further analysis has revealed a more complex picture involving several alpha receptor subtypes and their involvement in feedback regulation.'),
  'D011943': NtO(MH='Receptors, Adrenergic, beta', MS='One of two major pharmacologically defined classes of adrenergic receptors. The beta adrenergic receptors play an important role in regulating CARDIAC MUSCLE contraction, SMOOTH MUSCLE relaxation, and GLYCOGENOLYSIS.'),
  'D011944': NtO(MH='Receptors, Androgen', MS='Proteins, generally found in the CYTOPLASM, that specifically bind ANDROGENS and mediate their cellular actions. The complex of the androgen and receptor migrates to the CELL NUCLEUS where it induces transcription of specific segments of DNA.'),
  'D011945': NtO(MH='Receptors, Angiotensin', MS='Cell surface proteins that bind ANGIOTENSINS and trigger intracellular changes influencing the behavior of cells.'),
  'D011946': NtO(MH='Receptors, Antigen', MS='Molecules on the surface of B- and T-lymphocytes that recognize and combine with specific antigens.'),
  'D011947': NtO(MH='Receptors, Antigen, B-Cell', MS='IMMUNOGLOBULINS on the surface of B-LYMPHOCYTES. Their MESSENGER RNA contains an EXON with a membrane spanning sequence, producing immunoglobulins in the form of type I transmembrane proteins as opposed to secreted immunoglobulins (ANTIBODIES) which do not contain the membrane spanning segment.'),
  'D011948': NtO(MH='Receptors, Antigen, T-Cell', MS='Molecules on the surface of T-lymphocytes that recognize and combine with antigens. The receptors are non-covalently associated with a complex of several polypeptides collectively called CD3 antigens (CD3 COMPLEX). Recognition of foreign antigen and the major histocompatibility complex is accomplished by a single heterodimeric antigen-receptor structure, composed of either alpha-beta (RECEPTORS, ANTIGEN, T-CELL, ALPHA-BETA) or gamma-delta (RECEPTORS, ANTIGEN, T-CELL, GAMMA-DELTA) chains.'),
  'D011949': NtO(MH='Receptors, Cholecystokinin', MS='Cell surface proteins that bind cholecystokinin (CCK) with high affinity and trigger intracellular changes influencing the behavior of cells. Cholecystokinin receptors are activated by GASTRIN as well as by CCK-4; CCK-8; and CCK-33. Activation of these receptors evokes secretion of AMYLASE by pancreatic acinar cells, acid and PEPSIN by stomach mucosal cells, and contraction of the PYLORUS and GALLBLADDER. The role of the widespread CCK receptors in the central nervous system is not well understood.'),
  'D011950': NtO(MH='Receptors, Cholinergic', MS='Cell surface proteins that bind acetylcholine with high affinity and trigger intracellular changes influencing the behavior of cells. Cholinergic receptors are divided into two major classes, muscarinic and nicotinic, based originally on their affinity for nicotine and muscarine. Each group is further subdivided based on pharmacology, location, mode of action, and/or molecular biology.'),
  'D011951': NtO(MH='Receptors, Complement', MS='Molecules on the surface of some B-lymphocytes and macrophages, that recognize and combine with the C3b, C3d, C1q, and C4b components of complement.'),
  'D011952': NtO(MH='Receptors, Concanavalin A', MS='Glycoprotein moieties on the surfaces of cell membranes that bind concanavalin A selectively; the number and location of the sites depends on the type and condition of the cell.'),
  'D011953': NtO(MH='Receptors, Cyclic AMP', MS='Cell surface proteins that bind cyclic AMP with high affinity and trigger intracellular changes which influence the behavior of cells. The best characterized cyclic AMP receptors are those of the slime mold Dictyostelium discoideum. The transcription regulator CYCLIC AMP RECEPTOR PROTEIN of prokaryotes is not included nor are the eukaryotic cytoplasmic cyclic AMP receptor proteins which are the regulatory subunits of CYCLIC AMP-DEPENDENT PROTEIN KINASES.'),
  'D011954': NtO(MH='Receptors, Dopamine', MS='Cell-surface proteins that bind dopamine with high affinity and trigger intracellular changes influencing the behavior of cells.'),
  'D011955': NtO(MH='Receptors, Drug', MS='Proteins that bind specific drugs with high affinity and trigger intracellular changes influencing the behavior of cells. Drug receptors are generally thought to be receptors for some endogenous substance not otherwise specified.'),
  'D011956': NtO(MH='Receptors, Cell Surface', MS='Cell surface proteins that bind signalling molecules external to the cell with high affinity and convert this extracellular event into one or more intracellular signals that alter the behavior of the target cell (From Alberts, Molecular Biology of the Cell, 2nd ed, pp693-5). Cell surface receptors, unlike enzymes, do not chemically alter their ligands.'),
  'D011957': NtO(MH='Receptors, Opioid', MS='Cell membrane proteins that bind opioids and trigger intracellular changes which influence the behavior of cells. The endogenous ligands for opioid receptors in mammals include three families of peptides, the enkephalins, endorphins, and dynorphins. The receptor classes include mu, delta, and kappa receptors. Sigma receptors bind several psychoactive substances, including certain opioids, but their endogenous ligands are not known.'),
  'D011959': NtO(MH='Receptors, Estradiol', MS='Cytoplasmic proteins that bind estradiol, migrate to the nucleus, and regulate DNA transcription.'),
  'D011960': NtO(MH='Receptors, Estrogen', MS='Cytoplasmic proteins that bind estrogens and migrate to the nucleus where they regulate DNA transcription. Evaluation of the state of estrogen receptors in breast cancer patients has become clinically important.'),
  'D011961': NtO(MH='Receptors, Fc', MS='Molecules found on the surface of some, but not all, B-lymphocytes, T-lymphocytes, and macrophages, which recognize and combine with the Fc (crystallizable) portion of immunoglobulin molecules.'),
  'D011962': NtO(MH='Receptors, FSH', MS='Cell surface proteins that bind FOLLICLE STIMULATING HORMONE with high affinity and trigger intracellular changes influencing the behavior of cells.'),
  'D011963': NtO(MH='Receptors, GABA-A', MS='Cell surface proteins which bind GAMMA-AMINOBUTYRIC ACID and contain an integral membrane chloride channel. Each receptor is assembled as a pentamer from a pool of at least 19 different possible subunits. The receptors belong to a superfamily that share a common CYSTEINE loop.'),
  'D011964': NtO(MH='Receptors, Gastrointestinal Hormone', MS='Cell surface proteins that bind gastrointestinal hormones with high affinity and trigger intracellular changes influencing the behavior of cells. Most gastrointestinal hormones also act as neurotransmitters so these receptors are also present in the central and peripheral nervous systems.'),
  'D011965': NtO(MH='Receptors, Glucocorticoid', MS='Cytoplasmic proteins that specifically bind glucocorticoids and mediate their cellular effects. The glucocorticoid receptor-glucocorticoid complex acts in the nucleus to induce transcription of DNA. Glucocorticoids were named for their actions on blood glucose concentration, but they have equally important effects on protein and fat metabolism. Cortisol is the most important example.'),
  'D011966': NtO(MH='Receptors, LHRH', MS='Receptors with a 6-kDa protein on the surfaces of cells that secrete LUTEINIZING HORMONE or FOLLICLE STIMULATING HORMONE, usually in the adenohypophysis. LUTEINIZING HORMONE-RELEASING HORMONE binds to these receptors, is endocytosed with the receptor and, in the cell, triggers the release of LUTEINIZING HORMONE or FOLLICLE STIMULATING HORMONE by the cell. These receptors are also found in rat gonads. INHIBINS prevent the binding of GnRH to its receptors.'),
  'D011967': NtO(MH='Receptors, Gonadotropin', MS='Those protein complexes or molecular sites on the surfaces of gonadal and other sensitive cells that bind gonadotropins and thereby modify the functions of those cells; hCG, LH, and FOLLICLE STIMULATING HORMONE are the major specific gonadotropins.'),
  'D011968': NtO(MH='Receptors, Histamine', MS='Cell-surface proteins that bind histamine and trigger intracellular changes influencing the behavior of cells. Histamine receptors are widespread in the central nervous system and in peripheral tissues. Three types have been recognized and designated H1, H2, and H3. They differ in pharmacology, distribution, and mode of action.'),
  'D011969': NtO(MH='Receptors, Histamine H1', MS='A class of histamine receptors discriminated by their pharmacology and mode of action. Most histamine H1 receptors operate through the inositol phosphate/diacylglycerol second messenger system. Among the many responses mediated by these receptors are smooth muscle contraction, increased vascular permeability, hormone release, and cerebral glyconeogenesis. (From Biochem Soc Trans 1992 Feb;20(1):122-5)'),
  'D011970': NtO(MH='Receptors, Histamine H2', MS='A class of histamine receptors discriminated by their pharmacology and mode of action. Histamine H2 receptors act via G-proteins to stimulate ADENYLYL CYCLASES. Among the many responses mediated by these receptors are gastric acid secretion, smooth muscle relaxation, inotropic and chronotropic effects on heart muscle, and inhibition of lymphocyte function. (From Biochem Soc Trans 1992 Feb;20(1):122-5)'),
  'D011971': NtO(MH='Receptors, Immunologic', MS='Cell surface molecules on cells of the immune system that specifically bind surface molecules or messenger molecules and trigger changes in the behavior of cells. Although these receptors were first identified in the immune system, many have important functions elsewhere.'),
  'D011972': NtO(MH='Receptor, Insulin', MS='A cell surface receptor for INSULIN. It comprises a tetramer of two alpha and two beta subunits which are derived from cleavage of a single precursor protein. The receptor contains an intrinsic TYROSINE KINASE domain that is located within the beta subunit. Activation of the receptor by INSULIN results in numerous metabolic changes including increased uptake of GLUCOSE into the liver, muscle, and ADIPOSE TISSUE.'),
  'D011973': NtO(MH='Receptors, LDL', MS='Receptors on the plasma membrane of nonhepatic cells that specifically bind LDL. The receptors are localized in specialized regions called coated pits. Hypercholesteremia is caused by an allelic genetic defect of three types: 1, receptors do not bind to LDL; 2, there is reduced binding of LDL; and 3, there is normal binding but no internalization of LDL. In consequence, entry of cholesterol esters into the cell is impaired and the intracellular feedback by cholesterol on 3-hydroxy-3-methylglutaryl CoA reductase is lacking.'),
  'D011974': NtO(MH='Receptors, LH', MS='Those protein complexes or molecular sites on the surfaces and cytoplasm of gonadal cells that bind luteinizing or chorionic gonadotropic hormones and thereby cause the gonadal cells to synthesize and secrete sex steroids. The hormone-receptor complex is internalized from the plasma membrane and initiates steroid synthesis.'),
  'D011975': NtO(MH='Receptors, Mitogen', MS='Glycoprotein molecules on the surface of B- and T-lymphocytes, that react with molecules of antilymphocyte sera, lectins, and other agents which induce blast transformation of lymphocytes.'),
  'D011976': NtO(MH='Receptors, Muscarinic', MS='One of the two major classes of cholinergic receptors. Muscarinic receptors were originally defined by their preference for MUSCARINE over NICOTINE. There are several subtypes (usually M1, M2, M3....) that are characterized by their cellular actions, pharmacology, and molecular biology.'),
  'D011978': NtO(MH='Receptors, Nicotinic', MS='One of the two major classes of cholinergic receptors. Nicotinic receptors were originally distinguished by their preference for NICOTINE over MUSCARINE. They are generally divided into muscle-type and neuronal-type (previously ganglionic) based on pharmacology, and subunit composition of the receptors.'),
  'D011979': NtO(MH='Receptors, Pituitary Hormone', MS='Cell surface proteins that bind pituitary hormones with high affinity and trigger intracellular changes influencing the behavior of cells. Since many pituitary hormones are also released by neurons as neurotransmitters, these receptors are also found in the nervous system.'),
  'D011980': NtO(MH='Receptors, Progesterone', MS='Specific proteins found in or on cells of progesterone target tissues that specifically combine with progesterone. The cytosol progesterone-receptor complex then associates with the nucleic acids to initiate protein synthesis. There are two kinds of progesterone receptors, A and B. Both are induced by estrogen and have short half-lives.'),
  'D011981': NtO(MH='Receptors, Prolactin', MS="Labile proteins on or in prolactin-sensitive cells that bind prolactin initiating the cells' physiological response to that hormone. Mammary casein synthesis is one of the responses. The receptors are also found in placenta, liver, testes, kidneys, ovaries, and other organs and bind and respond to certain other hormones and their analogs and antagonists. This receptor is related to the growth hormone receptor."),
  'D011982': NtO(MH='Receptors, Prostaglandin', MS='Cell surface receptors that bind prostaglandins with high affinity and trigger intracellular changes which influence the behavior of cells. Prostaglandin receptor subtypes have been tentatively named according to their relative affinities for the endogenous prostaglandins. They include those which prefer prostaglandin D2 (DP receptors), prostaglandin E2 (EP1, EP2, and EP3 receptors), prostaglandin F2-alpha (FP receptors), and prostacyclin (IP receptors).'),
  'D011983': NtO(MH='Receptors, Purinergic', MS='Cell surface proteins that bind PURINES with high affinity and trigger intracellular changes which influence the behavior of cells. The best characterized classes of purinergic receptors in mammals are the P1 receptors, which prefer ADENOSINE, and the P2 receptors, which prefer ATP or ADP.'),
  'D011984': NtO(MH='Sensory Receptor Cells', MS='Specialized afferent neurons capable of transducing sensory stimuli into NERVE IMPULSES to be transmitted to the CENTRAL NERVOUS SYSTEM. Sometimes sensory receptors for external stimuli are called exteroceptors; for internal stimuli are called interoceptors and proprioceptors.'),
  'D011985': NtO(MH='Receptors, Serotonin', MS='Cell-surface proteins that bind SEROTONIN and trigger intracellular changes which influence the behavior of cells. Several types of serotonin receptors have been recognized which differ in their pharmacology, molecular biology, and mode of action.'),
  'D011986': NtO(MH='Receptors, Somatotropin', MS='Cell surface proteins that bind GROWTH HORMONE with high affinity and trigger intracellular changes influencing the behavior of cells. Activation of growth hormone receptors regulates amino acid transport through cell membranes, RNA translation to protein, DNA transcription, and protein and amino acid catabolism in many cell types. Many of these effects are mediated indirectly through stimulation of the release of somatomedins.'),
  'D011987': NtO(MH='Receptors, Steroid', MS='Proteins found usually in the cytoplasm or nucleus that specifically bind steroid hormones and trigger changes influencing the behavior of cells. The steroid receptor-steroid hormone complex regulates the transcription of specific genes.'),
  'D011988': NtO(MH='Receptors, Thyroid Hormone', MS='Specific high affinity binding proteins for THYROID HORMONES in target cells. They are usually found in the nucleus and regulate DNA transcription. These receptors are activated by hormones that leads to transcription, cell differentiation, and growth suppression. Thyroid hormone receptors are encoded by two genes (GENES, ERBA): erbA-alpha and erbA-beta for alpha and beta thyroid hormone receptors, respectively.'),
  'D011989': NtO(MH='Receptors, Thyrotropin', MS='Cell surface proteins that bind pituitary THYROTROPIN (also named thyroid stimulating hormone or TSH) and trigger intracellular changes of the target cells. TSH receptors are present in the nervous system and on target cells in the thyroid gland. Autoantibodies to TSH receptors are implicated in thyroid diseases such as GRAVES DISEASE and Hashimoto disease (THYROIDITIS, AUTOIMMUNE).'),
  'D011990': NtO(MH='Receptors, Transferrin', MS='Membrane glycoproteins found in high concentrations on iron-utilizing cells. They specifically bind iron-bearing transferrin, are endocytosed with its ligand and then returned to the cell surface where transferrin without its iron is released.'),
  'D011991': NtO(MH='Receptors, Virus', MS='Specific molecular components of the cell capable of recognizing and interacting with a virus, and which, after binding it, are capable of generating some signal that initiates the chain of events leading to the biological response.'),
  'D011992': NtO(MH='Endosomes', MS='Cytoplasmic vesicles formed when COATED VESICLES shed their CLATHRIN coat. Endosomes internalize macromolecules bound by receptors on the cell surface.'),
  'D011993': NtO(MH='Recombinant Fusion Proteins', MS='Recombinant proteins produced by the GENETIC TRANSLATION of fused genes formed by the combination of NUCLEIC ACID REGULATORY SEQUENCES of one or more genes with the protein coding sequences of one or more genes.'),
  'D011994': NtO(MH='Recombinant Proteins', MS='Proteins prepared by recombinant DNA technology.'),
  'D011995': NtO(MH='Recombination, Genetic', MS='Production of new arrangements of DNA by various mechanisms such as assortment and segregation, CROSSING OVER; GENE CONVERSION; GENETIC TRANSFORMATION; GENETIC CONJUGATION; GENETIC TRANSDUCTION; or mixed infection of viruses.'),
  'D011996': NtO(MH='Records', MS='Works on authentic evidence, of something having legal importance. The concept includes certificates of birth, death, etc., as well as hospital, medical, and other institutional records.'),
  'D011997': NtO(MH='Recovery Room', MS='Hospital unit providing continuous monitoring of the patient following anesthesia.'),
  'D011998': NtO(MH='Recreation', MS='Activity engaged in for pleasure.'),
  'D011999': NtO(MH='Recruitment, Neurophysiological', MS="The spread of response if stimulation is prolonged. (Campbell's Psychiatric Dictionary, 8th ed.)"),
  'D012000': NtO(MH='Recruitment Detection, Audiologic', MS='Tests for measuring loudness recruitment.'),
  'D012001': NtO(MH='Hyperacusis', MS='An abnormally disproportionate increase in the sensation of loudness in response to auditory stimuli of normal volume. COCHLEAR DISEASES; VESTIBULOCOCHLEAR NERVE DISEASES; FACIAL NERVE DISEASES; STAPES SURGERY; and other disorders may be associated with this condition.'),
  'D012002': NtO(MH='Rectal Diseases', MS='Pathological developments in the RECTUM region of the large intestine (INTESTINE, LARGE).'),
  'D012003': NtO(MH='Rectal Fistula', MS='An abnormal anatomical passage connecting the RECTUM to the outside, with an orifice at the site of drainage.'),
  'D012004': NtO(MH='Rectal Neoplasms', MS='Tumors or cancer of the RECTUM.'),
  'D012005': NtO(MH='Rectal Prolapse', MS='Protrusion of the rectal mucous membrane through the anus. There are various degrees: incomplete with no displacement of the anal sphincter muscle; complete with displacement of the anal sphincter muscle; complete with no displacement of the anal sphincter muscle but with herniation of the bowel; and internal complete with rectosigmoid or upper rectum intussusception into the lower rectum.'),
  'D012006': NtO(MH='Rectovaginal Fistula', MS='An abnormal anatomical passage between the RECTUM and the VAGINA.'),
  'D012007': NtO(MH='Rectum', MS='The distal segment of the LARGE INTESTINE, between the SIGMOID COLON and the ANAL CANAL.'),
  'D012008': NtO(MH='Recurrence', MS='The return of a sign, symptom, or disease after a remission.'),
  'D012009': NtO(MH='Recurrent Laryngeal Nerve', MS='Branches of the vagus (tenth cranial) nerve. The recurrent laryngeal nerves originate more caudally than the superior laryngeal nerves and follow different paths on the right and left sides. They carry efferents to all muscles of the larynx except the cricothyroid and carry sensory and autonomic fibers to the laryngeal, pharyngeal, tracheal, and cardiac regions.'),
  'D012010': NtO(MH='Red-Cell Aplasia, Pure', MS='Suppression of erythropoiesis with little or no abnormality of leukocyte or platelet production.'),
  'D012011': NtO(MH='Red Cross', MS='International collective of humanitarian organizations led by volunteers and guided by its Congressional Charter and the Fundamental Principles of the International Red Cross Movement, to provide relief to victims of disaster and help people prevent, prepare for, and respond to emergencies.'),
  'D012012': NtO(MH='Red Nucleus', MS='A pinkish-yellow portion of the midbrain situated in the rostral mesencephalic tegmentum. It receives a large projection from the contralateral half of the CEREBELLUM via the superior cerebellar peduncle and a projection from the ipsilateral MOTOR CORTEX.'),
  'D012013': NtO(MH='Reference Books', MS='Books designed by the arrangement and treatment of their subject matter to be consulted for definite terms of information rather than to be read consecutively. Reference books include DICTIONARIES; ENCYCLOPEDIAS; ATLASES; etc. (From the ALA Glossary of Library and Information Science, 1983)'),
  'D012014': NtO(MH='Reference Books, Medical', MS='Books in the field of medicine intended primarily for consultation.'),
  'D012015': NtO(MH='Reference Standards', MS='A basis of value established for the measure of quantity, weight, extent or quality, e.g. weight standards, standard solutions, methods, techniques, and procedures used in diagnosis and therapy.'),
  'D012016': NtO(MH='Reference Values', MS='The range or frequency distribution of a measurement in a population (of organisms, organs or things) that has not been selected for the presence of disease or abnormality.'),
  'D012017': NtO(MH='Referral and Consultation', MS='The practice of sending a patient to another program or practitioner for services or advice which the referring source is not prepared to provide.'),
  'D012018': NtO(MH='Reflex', MS='An involuntary movement or exercise of function in a part, excited in response to a stimulus applied to the periphery and transmitted to the brain or spinal cord.'),
  'D012019': NtO(MH='Reflex Sympathetic Dystrophy', MS='A syndrome characterized by severe burning pain in an extremity accompanied by sudomotor, vasomotor, and trophic changes in bone without an associated specific nerve injury. This condition is most often precipitated by trauma to soft tissue or nerve complexes. The skin over the affected region is usually erythematous and demonstrates hypersensitivity to tactile stimuli and erythema. (Adams et al., Principles of Neurology, 6th ed, p1360; Pain 1995 Oct;63(1):127-33)'),
  'D012020': NtO(MH='Reflex, Abdominal', MS="Contractions of the abdominal muscles upon stimulation of the skin (superficial abdominal reflex) or tapping neighboring bony structures (deep abdominal reflex). The superficial reflex may be weak or absent, for example, after a stroke, a sign of upper (suprasegmental) motor neuron lesions. (Stedman, 25th ed & Best & Taylor's Physiological Basis of Medical Practice, 12th ed, p1073)"),
  'D012021': NtO(MH='Reflex, Abnormal', MS='An abnormal response to a stimulus applied to the sensory components of the nervous system. This may take the form of increased, decreased, or absent reflexes.'),
  'D012022': NtO(MH='Reflex, Acoustic', MS='Intra-aural contraction of tensor tympani and stapedius in response to sound.'),
  'D012023': NtO(MH='Reflex, Monosynaptic', MS='A reflex in which the AFFERENT NEURONS synapse directly on the EFFERENT NEURONS, without any INTERCALATED NEURONS. (Lockard, Desk Reference for Neuroscience, 2nd ed.)'),
  'D012024': NtO(MH='Reflex, Oculocardiac', MS='Change of heartbeat induced by pressure on the eyeball, manipulation of extraocular muscles, or pressure upon the tissue remaining in the orbital apex after enucleation.'),
  'D012025': NtO(MH='Reflex, Pupillary', MS='Constriction of the pupil in response to light stimulation of the retina. It refers also to any reflex involving the iris, with resultant alteration of the diameter of the pupil. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D012026': NtO(MH='Reflex, Stretch', MS='Reflex contraction of a muscle in response to stretching, which stimulates muscle proprioceptors.'),
  'D012027': NtO(MH='Reflex, Vestibulo-Ocular', MS='A reflex wherein impulses are conveyed from the cupulas of the SEMICIRCULAR CANALS and from the OTOLITHIC MEMBRANE of the SACCULE AND UTRICLE via the VESTIBULAR NUCLEI of the BRAIN STEM and the median longitudinal fasciculus to the OCULOMOTOR NERVE nuclei. It functions to maintain a stable retinal image during head rotation by generating appropriate compensatory EYE MOVEMENTS.'),
  'D012028': NtO(MH='Reflexotherapy', MS='Treatment of some morbid condition by producing a reflex action, as in the household treatment of nosebleed by a piece of ice applied to the cervical spine. (Stedman, 27th ed)'),
  'D012029': NtO(MH='Refraction, Ocular', MS='Refraction of LIGHT effected by the media of the EYE.'),
  'D012030': NtO(MH='Refractive Errors', MS='Deviations from the average or standard indices of refraction of the eye through its dioptric or refractive apparatus.'),
  'D012031': NtO(MH='Refractometry', MS='Measurement of the index of refraction (the ratio of the velocity of light or other radiation in the first of two media to its velocity in the second as it passes from one into the other).'),
  'D012032': NtO(MH='Refractory Period, Electrophysiological', MS='The period of time following the triggering of an ACTION POTENTIAL when the CELL MEMBRANE has changed to an unexcitable state and is gradually restored to the resting (excitable) state. During the absolute refractory period no other stimulus can trigger a response. This is followed by the relative refractory period during which the cell gradually becomes more excitable and the stronger impulse that is required to illicit a response gradually lessens to that required during the resting state.'),
  'D012033': NtO(MH='Refractory Period, Psychological', MS='A delayed response interval occurring when two stimuli are presented in close succession.'),
  'D012034': NtO(MH='Refrigeration', MS='The mechanical process of cooling.'),
  'D012035': NtO(MH='Refsum Disease', MS='An autosomal recessive familial disorder that usually presents in childhood with POLYNEUROPATHY; SENSORINEURAL HEARING LOSS; ICHTHYOSIS; ATAXIA; RETINITIS PIGMENTOSA; and CARDIOMYOPATHIES. (From Joynt, Clinical Neurology, 1991, Ch37, p58-9; Rev Med Interne 1996;17(5):391-8) This condition can be caused by mutation in the genes encoding peroxisomal phytanoyl-CoA hydroxylase or proteins associated peroxisomal membrane, leading to impaired catabolism of PHYTANIC ACID in PEROXISOMES.'),
  'D012036': NtO(MH='Refugees', MS='Persons fleeing to a place of safety, especially those who flee to a foreign country or power to escape danger, persecution, or economic distress in their own country or habitual residence.'),
  'D012037': NtO(MH='Refuse Disposal', MS='The discarding or destroying of garbage, sewage, or other waste matter or its transformation into something useful or innocuous.'),
  'D012038': NtO(MH='Regeneration', MS='The physiological renewal, repair, or replacement of tissue.'),
  'D012039': NtO(MH='Regional Blood Flow', MS='The flow of BLOOD through or around an organ or region of the body.'),
  'D012040': NtO(MH='Regional Health Planning', MS='Planning for health resources at a regional or multi-state level.'),
  'D012041': NtO(MH='Regional Medical Programs', MS='Coordination of activities and programs among health care institutions within defined geographic areas for the purpose of improving delivery and quality of medical care to the patients. These programs are mandated under U.S. Public Law 89-239.'),
  'D012042': NtO(MH='Registries', MS='The systems and processes involved in the establishment, support, management, and operation of registers, e.g., disease registers.'),
  'D012043': NtO(MH='Regression, Psychology', MS='A return to earlier, especially to infantile, patterns of thought or behavior, or stage of functioning, e.g., feelings of helplessness and dependency in a patient with a serious physical illness. (From APA, Thesaurus of Psychological Index Terms, 1994).'),
  'D012044': NtO(MH='Regression Analysis', MS='Procedures for finding the mathematical function which best describes the relationship between a dependent variable and one or more independent variables. In linear regression (see LINEAR MODELS) the relationship is constrained to be a straight line and LEAST-SQUARES ANALYSIS is used to determine the best fit. In logistic regression (see LOGISTIC MODELS) the dependent variable is qualitative rather than continuously variable and LIKELIHOOD FUNCTIONS are used to find the best relationship. In multiple regression, the dependent variable is considered to depend on more than a single independent variable.'),
  'D012045': NtO(MH='Regulatory Sequences, Nucleic Acid', MS='Nucleic acid sequences involved in regulating the expression of genes.'),
  'D012046': NtO(MH='Rehabilitation', MS='Restoration of human functions to the maximum degree possible in a person or persons suffering from disease or injury.'),
  'D012047': NtO(MH='Rehabilitation Centers', MS='Facilities which provide programs for rehabilitating the mentally or physically disabled individuals.'),
  'D012048': NtO(MH='Correction of Hearing Impairment', MS='Procedures for correcting HEARING DISORDERS.'),
  'D012049': NtO(MH='Rehabilitation of Speech and Language Disorders', MS='Procedures for assisting a person with a speech or language disorder to communicate with maximum efficiency.'),
  'D012050': NtO(MH='Rehabilitation, Vocational', MS='Training of the mentally or physically disabled in work skills so they may be returned to regular employment utilizing these skills.'),
  'D012051': NtO(MH='Reimbursement Mechanisms', MS='Processes or methods of reimbursement for services rendered or equipment.'),
  'D012052': NtO(MH='Reimbursement, Incentive', MS='A scheme which provides reimbursement for the health services rendered, generally by an institution, and which provides added financial rewards if certain conditions are met. Such a scheme is intended to promote and reward increased efficiency and cost containment, with better care, or at least without adverse effect on the quality of the care rendered.'),
  'D012053': NtO(MH='Reindeer', MS="A genus of deer, Rangifer, that inhabits the northern parts of Europe, Asia, and America. Caribou is the North American name; reindeer, the European. They are often domesticated and used, especially in Lapland, for drawing sleds and as a source of food. Rangifer is the only genus of the deer family in which both sexes are antlered. Most caribou inhabit arctic tundra and surrounding arboreal coniferous forests and most have seasonal shifts in migration. They are hunted extensively for their meat, skin, antlers, and other parts. (From Webster, 3d ed; Walker's Mammals of the World, 5th ed, p1397)"),
  'D012054': NtO(MH='Reinforcement, Psychology', MS='The strengthening of a conditioned response.'),
  'D012055': NtO(MH='Reinforcement Schedule', MS='A schedule prescribing when the subject is to be reinforced or rewarded in terms of temporal interval in psychological experiments. The schedule may be continuous or intermittent.'),
  'D012056': NtO(MH='Reinforcement, Social', MS="The strengthening of a response with a social reward such as a nod of approval, a parent's love or attention."),
  'D012057': NtO(MH='Reinforcement, Verbal', MS='Use of word stimulus to strengthen a response during learning.'),
  'D012059': NtO(MH='Rejection, Psychology', MS='Non-acceptance, negative attitudes, hostility or excessive criticism of the individual which may precipitate feelings of rejection.'),
  'D012060': NtO(MH='Rejuvenation', MS='The phenomenon of youthfulness, vitality, and freshness being restored. This can apply to appearance, TISSUES, organ functions, or other areas.'),
  'D012061': NtO(MH='Relapsing Fever', MS='An acute infection characterized by recurrent episodes of PYREXIA alternating with asymptomatic intervals of apparent recovery. This condition is caused by SPIROCHETES of the genus BORRELIA. It is transmitted by the BITES of either the body louse (PEDICULUS humanus corporis), for which humans are the reservoir, or by soft ticks of the genus ORNITHODOROS, for which rodents and other animals are the principal reservoirs.'),
  'D012062': NtO(MH='Relative Biological Effectiveness', MS='The ratio of radiation dosages required to produce identical change based on a formula comparing other types of radiation with that of gamma or roentgen rays.'),
  'D012063': NtO(MH='Relaxation', MS='Activity which reduces the feelings of tension and the effects of STRESS, PHYSIOLOGICAL.'),
  'D012064': NtO(MH='Relaxation Therapy', MS="Treatment to improve one's health condition by using techniques that can reduce PHYSIOLOGICAL STRESS; PSYCHOLOGICAL STRESS; or both."),
  'D012065': NtO(MH='Relaxin', MS='A water-soluble polypeptide (molecular weight approximately 8,000) extractable from the corpus luteum of pregnancy. It produces relaxation of the pubic symphysis and dilation of the uterine cervix in certain animal species. Its role in the human pregnant female is uncertain. (Dorland, 28th ed)'),
  'D012066': NtO(MH='Relief Work', MS='Assistance, such as money, food, or shelter, given to the needy, aged, or victims of disaster. It is usually granted on a temporary basis. (From The American Heritage Dictionary, 2d college ed)'),
  'D012067': NtO(MH='Religion', MS='A set of beliefs concerning the nature, cause, and purpose of the universe, especially when considered as the creation of a superhuman agency. It usually involves devotional and ritual observances and often a moral code for the conduct of human affairs. (Random House Collegiate Dictionary, rev. ed.)'),
  'D012068': NtO(MH='Religion and Medicine', MS='The interrelationship of medicine and religion.'),
  'D012069': NtO(MH='Religion and Psychology', MS='The interrelationship of psychology and religion.'),
  'D012070': NtO(MH='Religion and Science', MS='The interrelationship of science and religion.'),
  'D012071': NtO(MH='Religion and Sex', MS='The interrelationship of religion and sexual practices and beliefs.'),
  'D012072': NtO(MH='Religious Philosophies', MS='Sets of beliefs on the nature of the universe or Man.'),
  'D012073': NtO(MH='Remedial Teaching', MS='Specialized instruction for students deviating from the expected norm.'),
  'D012074': NtO(MH='Remission Induction', MS='Therapeutic act or process that initiates a response to a complete or partial remission level.'),
  'D012075': NtO(MH='Remission, Spontaneous', MS='A spontaneous diminution or abatement of a disease over time, without formal treatment.'),
  'D012076': NtO(MH='Renal Agents', MS="Drugs used for their effects on the kidneys' regulation of body fluid composition and volume. The most commonly used are the diuretics. Also included are drugs used for their antidiuretic and uricosuric actions, for their effects on the kidneys' clearance of other drugs, and for diagnosis of renal function."),
  'D012077': NtO(MH='Renal Artery', MS='A branch of the abdominal aorta which supplies the kidneys, adrenal glands and ureters.'),
  'D012078': NtO(MH='Renal Artery Obstruction', MS='Narrowing or occlusion of the RENAL ARTERY or arteries. It is due usually to ATHEROSCLEROSIS; FIBROMUSCULAR DYSPLASIA; THROMBOSIS; EMBOLISM, or external pressure. The reduced renal perfusion can lead to renovascular hypertension (HYPERTENSION, RENOVASCULAR).'),
  'D012079': NtO(MH='Renal Circulation', MS='The circulation of the BLOOD through the vessels of the KIDNEY.'),
  'D012080': NtO(MH='Chronic Kidney Disease-Mineral and Bone Disorder', MS='Decalcification of bone or abnormal bone development due to chronic KIDNEY DISEASES, in which 1,25-DIHYDROXYVITAMIN D3 synthesis by the kidneys is impaired, leading to reduced negative feedback on PARATHYROID HORMONE. The resulting SECONDARY HYPERPARATHYROIDISM eventually leads to bone disorders.'),
  'D012082': NtO(MH='Renal Veins', MS='Short thick veins which return blood from the kidneys to the vena cava.'),
  'D012083': NtO(MH='Renin', MS='A highly specific (Leu-Leu) endopeptidase that generates ANGIOTENSIN I from its precursor ANGIOTENSINOGEN, leading to a cascade of reactions which elevate BLOOD PRESSURE and increase sodium retention by the kidney in the RENIN-ANGIOTENSIN SYSTEM. The enzyme was formerly listed as EC 3.4.99.19.'),
  'D012084': NtO(MH='Renin-Angiotensin System', MS='A BLOOD PRESSURE regulating system of interacting components that include RENIN; ANGIOTENSINOGEN; ANGIOTENSIN CONVERTING ENZYME; ANGIOTENSIN I; ANGIOTENSIN II; and angiotensinase. Renin, an enzyme produced in the kidney, acts on angiotensinogen, an alpha-2 globulin produced by the liver, forming ANGIOTENSIN I. Angiotensin-converting enzyme, contained in the lung, acts on angiotensin I in the plasma converting it to ANGIOTENSIN II, an extremely powerful vasoconstrictor. Angiotensin II causes contraction of the arteriolar and renal VASCULAR SMOOTH MUSCLE, leading to retention of salt and water in the KIDNEY and increased arterial blood pressure. In addition, angiotensin II stimulates the release of ALDOSTERONE from the ADRENAL CORTEX, which in turn also increases salt and water retention in the kidney. Angiotensin-converting enzyme also breaks down BRADYKININ, a powerful vasodilator and component of the KALLIKREIN-KININ SYSTEM.'),
  'D012085': NtO(MH='Chymosin', MS='The predominant milk-clotting enzyme from the true stomach or abomasum of the suckling calf. It is secreted as an inactive precursor called prorennin and converted in the acid environment of the stomach to the active enzyme. EC 3.4.23.4.'),
  'D012086': NtO(MH='Reoperation', MS='A repeat operation for the same condition in the same patient due to disease progression or recurrence, or as followup to failed previous surgery.'),
  'D012087': NtO(MH='Reoviridae', MS='A family of unenveloped RNA viruses with cubic symmetry. The twelve genera include ORTHOREOVIRUS; ORBIVIRUS; COLTIVIRUS; ROTAVIRUS; Aquareovirus, Cypovirus, Phytoreovirus, Fijivirus, Seadornavirus, Idnoreovirus, Mycoreovirus, and Oryzavirus.'),
  'D012088': NtO(MH='Reoviridae Infections', MS='Infections produced by reoviruses, general or unspecified.'),
  'D012089': NtO(MH='Mammalian orthoreovirus 3', MS='A serotype of ORTHOREOVIRUS, MAMMALIAN causing serious pathology in laboratory rodents, characterized by diarrhea, oily coat, jaundice, and multiple organ involvement.'),
  'D012090': NtO(MH='Cumulative Trauma Disorders', MS='Harmful and painful condition caused by overuse or overexertion of some part of the musculoskeletal system, often resulting from work-related physical activities. It is characterized by inflammation, pain, or dysfunction of the involved joints, bones, ligaments, and nerves.'),
  'D012091': NtO(MH='Repetitive Sequences, Nucleic Acid', MS='Sequences of DNA or RNA that occur in multiple copies. There are several types: INTERSPERSED REPETITIVE SEQUENCES are copies of transposable elements (DNA TRANSPOSABLE ELEMENTS or RETROELEMENTS) dispersed throughout the genome. TERMINAL REPEAT SEQUENCES flank both ends of another sequence, for example, the long terminal repeats (LTRs) on RETROVIRUSES. Variations may be direct repeats, those occurring in the same direction, or inverted repeats, those opposite to each other in direction. TANDEM REPEAT SEQUENCES are copies which lie adjacent to each other, direct or inverted (INVERTED REPEAT SEQUENCES).'),
  'D012092': NtO(MH='Replantation', MS='Restoration of an organ or other structure to its original site.'),
  'D012093': NtO(MH='Replicon', MS='Any DNA sequence capable of independent replication or a molecule that possesses a REPLICATION ORIGIN and which is therefore potentially capable of being replicated in a suitable cell. (Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D012094': NtO(MH='Repression, Psychology', MS='The active mental process of keeping out and ejecting, banishing from consciousness, ideas or impulses that are unacceptable to it.'),
  'D012095': NtO(MH='Repression-Sensitization', MS='Defense mechanisms involving approach and avoidance responses to threatening stimuli. The sensitizing process involves intellectualization in approaching or controlling the stimulus whereas repression involves unconscious denial in avoiding the stimulus.'),
  'D012097': NtO(MH='Repressor Proteins', MS='Proteins which maintain the transcriptional quiescence of specific GENES or OPERONS. Classical repressor proteins are DNA-binding proteins that are normally bound to the OPERATOR REGION of an operon, or the ENHANCER SEQUENCES of a gene until a signal occurs that causes their release.'),
  'D012098': NtO(MH='Reproduction', MS='The total process by which organisms produce offspring. (Stedman, 25th ed)'),
  'D012099': NtO(MH='Reproductive Techniques', MS='Methods pertaining to the generation of new individuals, including techniques used in selective BREEDING, cloning (CLONING, ORGANISM), and assisted reproduction (REPRODUCTIVE TECHNIQUES, ASSISTED).'),
  'D012100': NtO(MH='Reproduction, Asexual', MS='Reproduction without fusion of two types of cells, mostly found in ALGAE; FUNGI; and PLANTS. Asexual reproduction occurs in several ways, such as budding, fission, or splitting from "parent" cells. Only few groups of ANIMALS reproduce asexually or unisexually (PARTHENOGENESIS).'),
  'D012101': NtO(MH='Reproductive and Urinary Physiological Phenomena', MS='Physiology of the human and animal body, male or female, in the processes and characteristics of REPRODUCTION and the URINARY TRACT.'),
  'D012102': NtO(MH='Reproductive Control Agents', MS='Substances used either in the prevention or facilitation of pregnancy.'),
  'D012103': NtO(MH='Batroxobin', MS='A proteolytic enzyme obtained from the venom of fer-de-lance (Bothrops atrox). It is used as a plasma clotting agent for fibrinogen and for the detection of fibrinogen degradation products. The presence of heparin does not interfere with the clotting test. Hemocoagulase is a mixture containing batroxobin and factor X activator. EC 3.4.21.-.'),
  'D012104': NtO(MH='Reptiles', MS='Cold-blooded, air-breathing VERTEBRATES belonging to the class Reptilia, usually covered with external scales or bony plates.'),
  'D012106': NtO(MH='Research', MS='Critical and exhaustive investigation or experimentation, having for its aim the discovery of new facts and their correct interpretation, the revision of accepted conclusions, theories, or laws in the light of newly discovered facts, or the practical application of such new or revised conclusions, theories, or laws. (Webster, 3d ed)'),
  'D012107': NtO(MH='Research Design', MS='A plan for collecting and utilizing data so that desired information can be obtained with sufficient precision or so that an hypothesis can be tested properly.'),
  'D012108': NtO(MH='Research Personnel', MS='Those individuals engaged in research.'),
  'D012109': NtO(MH='Research Support as Topic', MS='Work about financial support of research activities.'),
  'D012110': NtO(MH='Reserpine', MS='An alkaloid found in the roots of Rauwolfia serpentina and R. vomitoria. Reserpine inhibits the uptake of norepinephrine into storage vesicles resulting in depletion of catecholamines and serotonin from central and peripheral axon terminals. It has been used as an antihypertensive and an antipsychotic as well as a research tool, but its adverse effects limit its clinical use.'),
  'D012111': NtO(MH='Residence Characteristics', MS='Elements of residence that characterize a population. They are applicable in determining need for and utilization of health services.'),
  'D012112': NtO(MH='Residential Facilities', MS='Long-term care facilities which provide supervision and assistance in activities of daily living with medical and nursing services when required.'),
  'D012114': NtO(MH='Residential Treatment', MS='A specialized residential treatment program for behavior disorders including substance abuse. It may include therapeutically planned group living and learning situations including teaching of adaptive skills to help patient functioning in the community. (From Kahn, A. P. and Fawcett, J. Encyclopedia of Mental Health, 1993, p320.)'),
  'D012115': NtO(MH='Residual Volume', MS='The volume of air remaining in the LUNGS at the end of a maximal expiration. Common abbreviation is RV.'),
  'D012116': NtO(MH='Resins, Plant', MS="Flammable, amorphous, vegetable products of secretion or disintegration, usually formed in special cavities of plants. They are generally insoluble in water and soluble in alcohol, carbon tetrachloride, ether, or volatile oils. They are fusible and have a conchoidal fracture. They are the oxidation or polymerization products of the terpenes, and are mixtures of aromatic acids and esters. Most are soft and sticky, but harden after exposure to cold. (From Grant & Hackh's Chemical Dictionary, 5th ed & Dorland, 28th ed)"),
  'D012117': NtO(MH='Resins, Synthetic', MS='Polymers of high molecular weight which at some stage are capable of being molded and then harden to form useful components.'),
  'D012118': NtO(MH='Resorcinols', MS='A water-soluble crystalline benzene-1,3-diol (resorcinol) and its derivatives.'),
  'D012119': NtO(MH='Respiration', MS="The act of breathing with the LUNGS, consisting of INHALATION, or the taking into the lungs of the ambient air, and of EXHALATION, or the expelling of the modified air which contains more CARBON DIOXIDE than the air taken in (Blakiston's Gould Medical Dictionary, 4th ed.). This does not include tissue respiration (= OXYGEN CONSUMPTION) or cell respiration (= CELL RESPIRATION)."),
  'D012120': NtO(MH='Respiration Disorders', MS='Diseases of the respiratory system in general or unspecified or for a specific respiratory disease not available.'),
  'D012121': NtO(MH='Respiration, Artificial', MS='Any method of artificial breathing that employs mechanical or non-mechanical means to force the air into and out of the lungs. Artificial respiration or ventilation is used in individuals who have stopped breathing or have RESPIRATORY INSUFFICIENCY to increase their intake of oxygen (O2) and excretion of carbon dioxide (CO2).'),
  'D012122': NtO(MH='Ventilators, Mechanical', MS='Mechanical devices used to produce or assist pulmonary ventilation.'),
  'D012123': NtO(MH='Pulmonary Ventilation', MS='The total volume of gas inspired or expired per unit of time, usually measured in liters per minute.'),
  'D012124': NtO(MH='Respiratory Care Units', MS='The hospital unit in which patients with respiratory conditions requiring special attention receive intensive medical care and surveillance.'),
  'D012125': NtO(MH='Respiratory Center', MS='Part of the brain located in the MEDULLA OBLONGATA and PONS. It receives neural, chemical and hormonal signals, and controls the rate and depth of respiratory movements of the DIAPHRAGM and other respiratory muscles.'),
  'D012126': NtO(MH='Respiratory Dead Space', MS='That part of the RESPIRATORY TRACT or the air within the respiratory tract that does not exchange OXYGEN and CARBON DIOXIDE with pulmonary capillary blood.'),
  'D012127': NtO(MH='Respiratory Distress Syndrome, Newborn', MS='A condition of the newborn marked by DYSPNEA with CYANOSIS, heralded by such prodromal signs as dilatation of the alae nasi, expiratory grunt, and retraction of the suprasternal notch or costal margins, mostly frequently occurring in premature infants, children of diabetic mothers, and infants delivered by cesarean section, and sometimes with no apparent predisposing cause.'),
  'D012128': NtO(MH='Respiratory Distress Syndrome, Adult', MS='A syndrome characterized by progressive life-threatening RESPIRATORY INSUFFICIENCY in the absence of known LUNG DISEASES, usually following a systemic insult such as surgery or major TRAUMA.'),
  'D012129': NtO(MH='Respiratory Function Tests', MS='Measurement of the various processes involved in the act of respiration: inspiration, expiration, oxygen and carbon dioxide exchange, lung volume and compliance, etc.'),
  'D012130': NtO(MH='Respiratory Hypersensitivity', MS='A form of hypersensitivity affecting the respiratory tract. It includes ASTHMA and RHINITIS, ALLERGIC, SEASONAL.'),
  'D012131': NtO(MH='Respiratory Insufficiency', MS='Failure to adequately provide oxygen to cells of the body and to remove excess carbon dioxide from them. (Stedman, 25th ed)'),
  'D012132': NtO(MH='Respiratory Muscles', MS='These include the muscles of the DIAPHRAGM and the INTERCOSTAL MUSCLES.'),
  'D012133': NtO(MH='Respiratory Paralysis', MS='Complete or severe weakness of the muscles of respiration. This condition may be associated with MOTOR NEURON DISEASES; PERIPHERAL NERVE DISEASES; NEUROMUSCULAR JUNCTION DISEASES; SPINAL CORD DISEASES; injury to the PHRENIC NERVE; and other disorders.'),
  'D012134': NtO(MH='Respiratory Protective Devices', MS='Respirators to protect individuals from breathing air contaminated with harmful dusts, fogs, fumes, mists, gases, smokes, sprays, or vapors.'),
  'D012135': NtO(MH='Respiratory Sounds', MS='Noises, normal and abnormal, heard on auscultation over any part of the RESPIRATORY TRACT.'),
  'D012136': NtO(MH='Respiratory Syncytial Viruses', MS='A group of viruses in the PNEUMOVIRUS genus causing respiratory infections in various mammals. Humans and cattle are most affected but infections in goats and sheep have also been reported.'),
  'D012137': NtO(MH='Respiratory System', MS='The tubular and cavernous organs and structures, by means of which pulmonary ventilation and gas exchange between ambient air and the blood are brought about.'),
  'D012138': NtO(MH='Respiratory Therapy', MS='Care of patients with deficiencies and abnormalities associated with the cardiopulmonary system. It includes the therapeutic use of medical gases and their administrative apparatus, environmental control systems, humidification, aerosols, ventilatory support, bronchopulmonary drainage and exercise, respiratory rehabilitation, assistance with cardiopulmonary resuscitation, and maintenance of natural, artificial, and mechanical airways.'),
  'D012139': NtO(MH='Respiratory Therapy Department, Hospital', MS='Hospital department which is responsible for the administration of diagnostic pulmonary function tests and of procedures to restore optimum pulmonary ventilation.'),
  'D012140': NtO(MH='Respiratory Tract Diseases', MS='Diseases involving the RESPIRATORY SYSTEM.'),
  'D012141': NtO(MH='Respiratory Tract Infections', MS='Invasion of the host RESPIRATORY SYSTEM by microorganisms, usually leading to pathological processes or diseases.'),
  'D012142': NtO(MH='Respiratory Tract Neoplasms', MS='New abnormal growth of tissue in the RESPIRATORY SYSTEM.'),
  'D012143': NtO(MH='Respiratory Physiological Phenomena', MS='Physiological processes and properties of the RESPIRATORY SYSTEM as a whole or of any of its parts.'),
  'D012144': NtO(MH='Respite Care', MS='Patient care provided in the home or institution intermittently in order to provide temporary relief to the family home care giver.'),
  'D012146': NtO(MH='Rest', MS='Freedom from activity.'),
  'D012147': NtO(MH='Restaurants', MS='An establishment where meals are cooked, sold and served to customers.'),
  'D012148': NtO(MH='Restless Legs Syndrome', MS='A disorder characterized by aching or burning sensations in the lower and rarely the upper extremities that occur prior to sleep or may awaken the patient from sleep.'),
  'D012149': NtO(MH='Restraint, Physical', MS='Use of a device for the purpose of controlling movement of all or part of the body. Splinting and casting are FRACTURE FIXATION.'),
  'D012150': NtO(MH='Polymorphism, Restriction Fragment Length', MS='Variation occurring within a species in the presence or length of DNA fragment generated by a specific endonuclease at a specific site in the genome. Such variations are generated by mutations that create or abolish recognition sites for these enzymes or change the length of the fragment.'),
  'D012151': NtO(MH='Resuscitation', MS='The restoration to life or consciousness of one apparently dead. (Dorland, 27th ed)'),
  'D012152': NtO(MH='Rete Testis', MS='The network of channels formed at the termination of the straight SEMINIFEROUS TUBULES in the mediastinum testis. Rete testis channels drain into the efferent ductules that pass into the caput EPIDIDYMIS.'),
  'D012153': NtO(MH='Retention, Psychology', MS='The persistence to perform a learned behavior (facts or experiences) after an interval has elapsed in which there has been no performance or practice of the behavior.'),
  'D012154': NtO(MH='Reticular Formation', MS='A region extending from the PONS & MEDULLA OBLONGATA through the MESENCEPHALON, characterized by a diversity of neurons of various sizes and shapes, arranged in different aggregations and enmeshed in a complicated fiber network.'),
  'D012155': NtO(MH='Reticulin', MS='A scleroprotein fibril consisting mostly of type III collagen. Reticulin fibrils are extremely thin, with a diameter of between 0.5 and 2 um. They are involved in maintaining the structural integrity in a variety of organs.'),
  'D012156': NtO(MH='Reticulocytes', MS='Immature ERYTHROCYTES. In humans, these are ERYTHROID CELLS that have just undergone extrusion of their CELL NUCLEUS. They still contain some organelles that gradually decrease in number as the cells mature. RIBOSOMES are last to disappear. Certain staining techniques cause components of the ribosomes to precipitate into characteristic "reticulum" (not the same as the ENDOPLASMIC RETICULUM), hence the name reticulocytes.'),
  'D012157': NtO(MH='Mononuclear Phagocyte System', MS='Mononuclear cells with pronounced phagocytic ability that are distributed extensively in lymphoid and other organs. It includes MACROPHAGES and their precursors; PHAGOCYTES; KUPFFER CELLS; HISTIOCYTES; DENDRITIC CELLS; LANGERHANS CELLS; and MICROGLIA. The term mononuclear phagocyte system has replaced the former reticuloendothelial system, which also included less active phagocytic cells such as fibroblasts and endothelial cells. (From Illustrated Dictionary of Immunology, 2d ed.)'),
  'D012159': NtO(MH='Reticulum', MS='The second stomach of ruminants. It lies almost in the midline in the front of the abdomen, in contact with the liver and diaphragm and communicates freely with the RUMEN via the ruminoreticular orifice. The lining of the reticulum is raised into folds forming a honeycomb pattern over the surface. (From Concise Veterinary Dictionary, 1988)'),
  'D012160': NtO(MH='Retina', MS='The ten-layered nervous tissue membrane of the eye. It is continuous with the OPTIC NERVE and receives images of external objects and transmits visual impulses to the brain. Its outer surface is in contact with the CHOROID and the inner surface with the VITREOUS BODY. The outer-most layer is pigmented, whereas the inner nine layers are transparent.'),
  'D012161': NtO(MH='Retinal Artery', MS='Central retinal artery and its branches. It arises from the ophthalmic artery, pierces the optic nerve and runs through its center, enters the eye through the porus opticus and branches to supply the retina.'),
  'D012162': NtO(MH='Retinal Degeneration', MS='A retrogressive pathological change in the retina, focal or generalized, caused by genetic defects, inflammation, trauma, vascular disease, or aging. Degeneration affecting predominantly the macula lutea of the retina is MACULAR DEGENERATION. (Newell, Ophthalmology: Principles and Concepts, 7th ed, p304)'),
  'D012163': NtO(MH='Retinal Detachment', MS='Separation of the inner layers of the retina (neural retina) from the pigment epithelium. Retinal detachment occurs more commonly in men than in women, in eyes with degenerative myopia, in aging and in aphakia. It may occur after an uncomplicated cataract extraction, but it is seen more often if vitreous humor has been lost during surgery. (Dorland, 27th ed; Newell, Ophthalmology: Principles and Concepts, 7th ed, p310-12).'),
  'D012164': NtO(MH='Retinal Diseases', MS='Diseases involving the RETINA.'),
  'D012165': NtO(MH='Retinal Ganglion Cells', MS='Neurons of the innermost layer of the retina, the internal plexiform layer. They are of variable sizes and shapes, and their axons project via the OPTIC NERVE to the brain. A small subset of these cells act as photoreceptors with projections to the SUPRACHIASMATIC NUCLEUS, the center for regulating CIRCADIAN RHYTHM.'),
  'D012166': NtO(MH='Retinal Hemorrhage', MS='Bleeding from the vessels of the retina.'),
  'D012167': NtO(MH='Retinal Perforations', MS='Perforations through the whole thickness of the retina including the macula as the result of inflammation, trauma, degeneration, etc. The concept includes retinal breaks, tears, dialyses, and holes.'),
  'D012168': NtO(MH='Retinal Pigments', MS='Photosensitive protein complexes of varied light absorption properties which are expressed in the PHOTORECEPTOR CELLS. They are OPSINS conjugated with VITAMIN A-based chromophores. Chromophores capture photons of light, leading to the activation of opsins and a biochemical cascade that ultimately excites the photoreceptor cells.'),
  'D012169': NtO(MH='Retinal Vein', MS='Central retinal vein and its tributaries. It runs a short course within the optic nerve and then leaves and empties into the superior ophthalmic vein or cavernous sinus.'),
  'D012170': NtO(MH='Retinal Vein Occlusion', MS='Blockage of the RETINAL VEIN. Those at high risk for this condition include patients with HYPERTENSION; DIABETES MELLITUS; ATHEROSCLEROSIS; and other CARDIOVASCULAR DISEASES.'),
  'D012171': NtO(MH='Retinal Vessels', MS='The blood vessels which supply and drain the RETINA.'),
  'D012172': NtO(MH='Retinaldehyde', MS='A carotenoid constituent of visual pigments. It is the oxidized form of retinol which functions as the active component of the visual cycle. It is bound to the protein opsin forming the complex rhodopsin. When stimulated by visible light, the retinal component of the rhodopsin complex undergoes isomerization at the 11-position of the double bond to the cis-form; this is reversed in "dark" reactions to return to the native trans-configuration.'),
  'D012173': NtO(MH='Retinitis', MS='Inflammation of the RETINA. It is rarely limited to the retina, but is commonly associated with diseases of the choroid (CHORIORETINITIS) and of the OPTIC DISK (neuroretinitis).'),
  'D012174': NtO(MH='Retinitis Pigmentosa', MS='Hereditary, progressive degeneration of the retina due to death of ROD PHOTORECEPTORS initially and subsequent death of CONE PHOTORECEPTORS. It is characterized by deposition of pigment in the retina.'),
  'D012175': NtO(MH='Retinoblastoma', MS='A malignant tumor arising from the nuclear layer of the retina that is the most common primary tumor of the eye in children. The tumor tends to occur in early childhood or infancy and may be present at birth. The majority are sporadic, but the condition may be transmitted as an autosomal dominant trait. Histologic features include dense cellularity, small round polygonal cells, and areas of calcification and necrosis. An abnormal pupil reflex (leukokoria); NYSTAGMUS, PATHOLOGIC; STRABISMUS; and visual loss represent common clinical characteristics of this condition. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2104)'),
  'D012176': NtO(MH='Retinoids', MS='A group of tetraterpenes, with four terpene units joined head-to-tail. Biologically active members of this class are used clinically in the treatment of severe cystic ACNE; PSORIASIS; and other disorders of keratinization.'),
  'D012177': NtO(MH='Retinol-Binding Proteins', MS='Proteins which bind with RETINOL. The retinol-binding protein found in plasma has an alpha-1 mobility on electrophoresis and a molecular weight of about 21 kDa. The retinol-protein complex (MW=80-90 kDa) circulates in plasma in the form of a protein-protein complex with prealbumin. The retinol-binding protein found in tissue has a molecular weight of 14 kDa and carries retinol as a non-covalently-bound ligand.'),
  'D012178': NtO(MH='Retinopathy of Prematurity', MS='A bilateral retinopathy occurring in premature infants treated with excessively high concentrations of oxygen, characterized by vascular dilatation, proliferation, and tortuosity, edema, and retinal detachment, with ultimate conversion of the retina into a fibrous mass that can be seen as a dense retrolental membrane. Usually growth of the eye is arrested and may result in microophthalmia, and blindness may occur. (Dorland, 27th ed)'),
  'D012179': NtO(MH='Retirement', MS="The state of being retired from one's position or occupation."),
  'D012180': NtO(MH='Retraction of Publication as Topic', MS="Works about authors' withdrawal or disavowal of their participation in performing research or writing the results of their study."),
  'D012181': NtO(MH='Retrocochlear Diseases', MS='Pathological processes involving the VESTIBULOCOCHLEAR NERVE; BRAINSTEM; or CENTRAL NERVOUS SYSTEM. When hearing loss is due to retrocochlear pathology, it is called retrocochlear hearing loss.'),
  'D012183': NtO(MH='Retrograde Degeneration', MS='Pathologic changes that occur in the axon and cell body of a neuron proximal to an axonal lesion. The process is characterized by central chromatolysis which features flattening and displacement of the nucleus, loss of Nissl bodies, and cellular edema. Central chromatolysis primarily occurs in lower motor neurons.'),
  'D012184': NtO(MH='Retrograde Obturation', MS='Procedure that involves the removal of infectious products from a root canal space through use of special instruments and fillings. This procedure is performed when root canal treatment fails.'),
  'D012185': NtO(MH='Retroperitoneal Fibrosis', MS='A slowly progressive condition of unknown etiology, characterized by deposition of fibrous tissue in the retroperitoneal space compressing the ureters, great vessels, bile duct, and other structures. When associated with abdominal aortic aneurysm, it may be called chronic periaortitis or inflammatory perianeurysmal fibrosis.'),
  'D012186': NtO(MH='Retroperitoneal Neoplasms', MS='New abnormal growth of tissue in the RETROPERITONEAL SPACE.'),
  'D012187': NtO(MH='Retroperitoneal Space', MS='An area occupying the most posterior aspect of the ABDOMINAL CAVITY. It is bounded laterally by the borders of the quadratus lumborum muscles and extends from the DIAPHRAGM to the brim of the true PELVIS, where it continues as the pelvic extraperitoneal space.'),
  'D012188': NtO(MH='Retropneumoperitoneum', MS='Pathological or accidental introduction of air into the retroperitoneal space.'),
  'D012189': NtO(MH='Retrospective Studies', MS='Studies used to test etiologic hypotheses in which inferences about an exposure to putative causal factors are derived from data relating to characteristics of persons under study or to events or experiences in their past. The essential feature is that some of the persons under study have the disease or outcome of interest and their characteristics are compared with those of unaffected persons.'),
  'D012190': NtO(MH='Retroviridae', MS='Family of RNA viruses that infects birds and mammals and encodes the enzyme reverse transcriptase. The family contains seven genera: DELTARETROVIRUS; LENTIVIRUS; RETROVIRUSES TYPE B, MAMMALIAN; ALPHARETROVIRUS; GAMMARETROVIRUS; RETROVIRUSES TYPE D; and SPUMAVIRUS. A key feature of retrovirus biology is the synthesis of a DNA copy of the genome which is integrated into cellular DNA. After integration it is sometimes not expressed but maintained in a latent state (PROVIRUSES).'),
  'D012191': NtO(MH='Retroviridae Proteins', MS='Proteins from the family Retroviridae. The most frequently encountered member of this family is the Rous sarcoma virus protein.'),
  'D012192': NtO(MH='Retroviridae Infections', MS='Virus diseases caused by the RETROVIRIDAE.'),
  'D012193': NtO(MH='Reversal Learning', MS='Any situation where an animal or human is trained to respond differentially to two stimuli (e.g., approach and avoidance) under reward and punishment conditions and subsequently trained under reversed reward values (i.e., the approach which was previously rewarded is punished and vice versa).'),
  'D012194': NtO(MH='RNA-Directed DNA Polymerase', MS='An enzyme that synthesizes DNA on an RNA template. It is encoded by the pol gene of retroviruses and by certain retrovirus-like elements. EC 2.7.7.49.'),
  'D012196': NtO(MH='Review Literature as Topic', MS='Works about published materials which provide an examination of recent or current literature. These articles can cover a wide range of subject matter at various levels of completeness and comprehensiveness based on analyses of literature that may include research findings. The review may reflect the state of the art and may also include reviews as a literary form.'),
  'D012201': NtO(MH='Reward', MS='An object or a situation that can serve to reinforce a response, to satisfy a motive, or to afford pleasure.'),
  'D012202': NtO(MH='Reye Syndrome', MS='A form of encephalopathy with fatty infiltration of the LIVER, characterized by brain EDEMA and VOMITING that may rapidly progress to SEIZURES; COMA; and DEATH. It is caused by a generalized loss of mitochondrial function leading to disturbances in fatty acid and CARNITINE metabolism.'),
  'D012203': NtO(MH='Rh Isoimmunization', MS='The process by which fetal Rh+ erythrocytes enter the circulation of an Rh- mother, causing her to produce IMMUNOGLOBULIN G antibodies, which can cross the placenta and destroy the erythrocytes of Rh+ fetuses. Rh isoimmunization can also be caused by BLOOD TRANSFUSION with mismatched blood.'),
  'D012204': NtO(MH='Rh-Hr Blood-Group System', MS='Erythrocyte isoantigens of the Rh (Rhesus) blood group system, the most complex of all human blood groups. The major antigen Rh or D is the most common cause of erythroblastosis fetalis.'),
  'D012205': NtO(MH='Rhabdiasoidea', MS='A superfamily of intestinal nematode parasites containing one genus, STRONGYLOIDES, and several species. It is transmitted through fecal material onto the skin and occurs in animals and humans.'),
  'D012206': NtO(MH='Rhabdomyolysis', MS='Necrosis or disintegration of skeletal muscle often followed by myoglobinuria.'),
  'D012207': NtO(MH='Rhabdomyoma', MS='A benign tumor derived from striated muscle. It is extremely rare, generally occurring in the tongue, neck muscles, larynx, uvula, nasal cavity, axilla, vulva, and heart. These tumors are treated by simple excision. (Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1354)'),
  'D012208': NtO(MH='Rhabdomyosarcoma', MS='A malignant solid tumor arising from mesenchymal tissues which normally differentiate to form striated muscle. It can occur in a wide variety of sites. It is divided into four distinct types: pleomorphic, predominantly in male adults; alveolar (RHABDOMYOSARCOMA, ALVEOLAR), mainly in adolescents and young adults; embryonal (RHABDOMYOSARCOMA, EMBRYONAL), predominantly in infants and children; and botryoidal, also in young children. It is one of the most frequently occurring soft tissue sarcomas and the most common in children under 15. (From Dorland, 27th ed; Holland et al., Cancer Medicine, 3d ed, p2186; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, pp1647-9)'),
  'D012209': NtO(MH='Rhabdoviridae', MS='A family of bullet-shaped viruses of the order MONONEGAVIRALES, infecting vertebrates, arthropods, protozoa, and plants. Genera include VESICULOVIRUS; LYSSAVIRUS; EPHEMEROVIRUS; NOVIRHABDOVIRUS; Cytorhabdovirus; and Nucleorhabdovirus.'),
  'D012210': NtO(MH='Rhamnose', MS='A methylpentose whose L- isomer is found naturally in many plant glycosides and some gram-negative bacterial lipopolysaccharides.'),
  'D012211': NtO(MH='Rhenium', MS='A metal, atomic number 75, atomic weight 186.207, symbol Re.'),
  'D012212': NtO(MH='Rheology', MS='The study of the deformation and flow of matter, usually liquids or fluids, and of the plastic flow of solids. The concept covers consistency, dilatancy, liquefaction, resistance to flow, shearing, thixotrophy, and VISCOSITY.'),
  'D012213': NtO(MH='Rheumatic Fever', MS='A febrile disease occurring as a delayed sequela of infections with STREPTOCOCCUS PYOGENES. It is characterized by multiple focal inflammatory lesions of the connective tissue structures, such as the heart, blood vessels, and joints (POLYARTHRITIS) and brain, and by the presence of ASCHOFF BODIES in the myocardium and skin.'),
  'D012214': NtO(MH='Rheumatic Heart Disease', MS='Cardiac manifestation of systemic rheumatological conditions, such as RHEUMATIC FEVER. Rheumatic heart disease can involve any part the heart, most often the HEART VALVES and the ENDOCARDIUM.'),
  'D012215': NtO(MH='Rheumatic Nodule', MS='A small round or oval, mostly subcutaneous nodule made up chiefly of a mass of Aschoff bodies and seen in cases of rheumatic fever. It is differentiated from the RHEUMATOID NODULE which appears in rheumatoid arthritis, most frequently over bony prominences. (From Dorland, 27th ed)'),
  'D012216': NtO(MH='Rheumatic Diseases', MS='Disorders of connective tissue, especially the joints and related structures, characterized by inflammation, degeneration, or metabolic derangement.'),
  'D012217': NtO(MH='Rheumatoid Factor', MS='Antibodies found in adult RHEUMATOID ARTHRITIS patients that are directed against GAMMA-CHAIN IMMUNOGLOBULINS.'),
  'D012218': NtO(MH='Rheumatoid Nodule', MS='Subcutaneous nodules seen in 20-30% of rheumatoid arthritis patients. They may arise anywhere on the body, but are most frequently found over the bony prominences. The nodules are characterized histologically by dense areas of fibrinoid necrosis with basophilic streaks and granules, surrounded by a palisade of cells, mainly fibroblasts and histiocytes.'),
  'D012219': NtO(MH='Rheumatology', MS='A subspecialty of internal medicine concerned with the study of inflammatory or degenerative processes and metabolic derangement of connective tissue structures which pertain to a variety of musculoskeletal disorders, such as arthritis.'),
  'D012220': NtO(MH='Rhinitis', MS='Inflammation of the NASAL MUCOSA, the mucous membrane lining the NASAL CAVITIES.'),
  'D012221': NtO(MH='Rhinitis, Allergic, Perennial', MS='Inflammation of the mucous membrane of the nose similar to that found in hay fever except that symptoms persist throughout the year. The causes are usually air-borne allergens, particularly dusts, feathers, molds, animal fur, etc.'),
  'D012222': NtO(MH='Rhinitis, Atrophic', MS='A chronic inflammation in which the NASAL MUCOSA gradually changes from a functional to a non-functional lining without mucociliary clearance. It is often accompanied by degradation of the bony TURBINATES, and the foul-smelling mucus which forms a greenish crust (ozena).'),
  'D012223': NtO(MH='Rhinitis, Vasomotor', MS='A form of non-allergic rhinitis that is characterized by nasal congestion and posterior pharyngeal drainage.'),
  'D012224': NtO(MH='Rhinophyma', MS='A manifestation of severe ROSACEA resulting in significant enlargement of the NOSE and occurring primarily in men. It is caused by hypertrophy of the SEBACEOUS GLANDS and surrounding CONNECTIVE TISSUE. The nose is reddened and marked with TELANGIECTASIS.'),
  'D012225': NtO(MH='Rhinoplasty', MS='A plastic surgical operation on the nose, either reconstructive, restorative, or cosmetic. (Dorland, 28th ed)'),
  'D012226': NtO(MH='Rhinoscleroma', MS='A granulomatous disease caused by KLEBSIELLA RHINOSCLEROMATIS infection. Despite its name, this disease is not limited to the nose and NASOPHARYNX but may affect any part of the RESPIRATORY TRACT, sometimes with extension to the lip and the skin.'),
  'D012227': NtO(MH='Rhinosporidiosis', MS='Chronic, localized granulomatous infection of mucocutaneous tissues, especially the NOSE, and characterized by HYPERPLASIA and the development of POLYPS. It is found in humans and other animals and is caused by the mesomycetozoean organism RHINOSPORIDIUM SEEBERI.'),
  'D012228': NtO(MH='Rhinosporidium', MS='A genus in the order Dermocystidium, class MESOMYCETOZOEA. It causes RHINOSPORIDIOSIS in MAMMALS and BIRDS.'),
  'D012229': NtO(MH='Rhinovirus', MS='A genus of PICORNAVIRIDAE inhabiting primarily the respiratory tract of mammalian hosts. It includes over 100 human serotypes associated with the COMMON COLD.'),
  'D012230': NtO(MH='Rhizobiaceae', MS='A family of gram-negative bacteria which are saprophytes, symbionts, or plant pathogens.'),
  'D012231': NtO(MH='Rhizobium', MS='A genus of gram-negative, aerobic, rod-shaped bacteria that activate PLANT ROOT NODULATION in leguminous plants. Members of this genus are nitrogen-fixing and common soil inhabitants.'),
  'D012232': NtO(MH='Rhizoctonia', MS='A mitosporic Ceratobasidiaceae fungal genus that is an important plant pathogen affecting potatoes and other plants. There are numerous teleomorphs.'),
  'D012233': NtO(MH='Rhizopus', MS='A genus of zygomycetous fungi of the family Mucoraceae, order MUCORALES, a common saprophyte and facultative parasite of mature fruits and vegetables. It may cause cerebral mycoses in diabetes and cutaneous infection in severely burned patients.'),
  'D012234': NtO(MH='Rho Factor', MS='A protein which effects termination of RNA synthesis during the genetic transcription process by dissociating the ternary transcription complex RNA;-RNA POLYMERASE DNA at the termination of a gene.'),
  'D012235': NtO(MH='Rhodamines', MS='A family of 3,6-di(substituted-amino)-9-benzoate derivatives of xanthene that are used as dyes and as indicators for various metals; also used as fluorescent tracers in histochemistry.'),
  'D012236': NtO(MH='Rhodanine', MS=None),
  'D012237': NtO(MH='Rhode Island', MS='State bounded by Massachusetts, the Atlantic Ocean, and Connecticut. The capital is Providence.'),
  'D012238': NtO(MH='Rhodium', MS='A hard and rare metal of the platinum group, atomic number 45, atomic weight 102.905, symbol Rh.'),
  'D012239': NtO(MH='Rhodnius', MS='A genus of the subfamily TRIATOMINAE. Rhodnius prolixus is a vector for TRYPANOSOMA CRUZI.'),
  'D012240': NtO(MH='Rhodococcus', MS='A bacterial genus of the order ACTINOMYCETALES.'),
  'D012241': NtO(MH='Rhodopseudomonas', MS='A genus of gram-negative, rod-shaped, phototrophic bacteria found in aquatic environments. Internal photosynthetic membranes are present as lamellae underlying the cytoplasmic membrane.'),
  'D012242': NtO(MH='Rhodobacter sphaeroides', MS='Spherical phototrophic bacteria found in mud and stagnant water exposed to light.'),
  'D012243': NtO(MH='Rhodopsin', MS='A purplish-red, light-sensitive pigment found in RETINAL ROD CELLS of most vertebrates. It is a complex consisting of a molecule of ROD OPSIN and a molecule of 11-cis retinal (RETINALDEHYDE). Rhodopsin exhibits peak absorption wavelength at about 500 nm.'),
  'D012244': NtO(MH='Rhodospirillaceae', MS='A family of phototrophic bacteria, in the order Rhodospirillales, isolated from stagnant water and mud.'),
  'D012245': NtO(MH='Rhodospirillales', MS='An order of photosynthetic bacteria representing a physiological community of predominantly aquatic bacteria.'),
  'D012246': NtO(MH='Rhodospirillum', MS='A genus of gram-negative, spiral bacteria that possesses internal photosynthetic membranes. Its organisms divide by binary fission, are motile by means of polar flagella, and are found in aquatic environments.'),
  'D012247': NtO(MH='Rhodospirillum rubrum', MS='Vibrio- to spiral-shaped phototrophic bacteria found in stagnant water and mud exposed to light.'),
  'D012248': NtO(MH='Rhodotorula', MS='A red yeast-like mitosporic fungal genus generally regarded as nonpathogenic. It is cultured from numerous sources in human patients.'),
  'D012249': NtO(MH='Rhombencephalon', MS='The posterior of the three primitive cerebral vesicles of an embryonic brain. It consists of myelencephalon, metencephalon, and isthmus rhombencephali from which develop the major BRAIN STEM components, such as MEDULLA OBLONGATA from the myelencephalon, CEREBELLUM and PONS from the metencephalon, with the expanded cavity forming the FOURTH VENTRICLE.'),
  'D012250': NtO(MH='Rheum', MS='A plant genus of the family POLYGONACEAE. Members contain chrysophanic acid, rhein, EMODIN, and other ANTHRAQUINONES. The roots were formerly used as PURGATIVES.'),
  'D012251': NtO(MH='Toxicodendron', MS='A genus (formerly part of Rhus genus) of shrubs, vines, or trees that yields a highly allergenic oleoresin which causes a severe contact dermatitis (DERMATITIS, TOXICODENDRON). The most toxic species are Toxicodendron vernix (poison sumac), T. diversilobum (poison oak), and T. radicans (poison ivy). T. vernicifera yields a useful varnish from which certain enzymes (laccases) are obtained.'),
  'D012252': NtO(MH='Natural Family Planning Methods', MS='A class of natural contraceptive methods in which SEXUAL ABSTINENCE is practiced a few days before and after the estimated day of ovulation, during the fertile phase. Methods for determining the fertile period or OVULATION DETECTION are based on various physiological indicators, such as circulating hormones, changes in cervical mucus (CERVIX MUCUS), and the basal body temperature.'),
  'D012253': NtO(MH='Rib Fractures', MS='Fractures of any of the RIBS.'),
  'D012254': NtO(MH='Ribavirin', MS='A nucleoside antimetabolite antiviral agent that blocks nucleic acid synthesis and is used against both RNA and DNA viruses.'),
  'D012255': NtO(MH='Ribitol', MS='A sugar alcohol formed by the reduction of ribose.'),
  'D012256': NtO(MH='Riboflavin', MS='Nutritional factor found in milk, eggs, malted barley, liver, kidney, heart, and leafy vegetables. The richest natural source is yeast. It occurs in the free form only in the retina of the eye, in whey, and in urine; its principal forms in tissues and cells are as FLAVIN MONONUCLEOTIDE and FLAVIN-ADENINE DINUCLEOTIDE.'),
  'D012257': NtO(MH='Riboflavin Deficiency', MS='A dietary deficiency of riboflavin causing a syndrome chiefly marked by cheilitis, angular stomatitis, glossitis associated with a purplish red or magenta-colored tongue that may show fissures, corneal vascularization, dyssebacia, and anemia. (Dorland, 27th ed)'),
  'D012258': NtO(MH='Riboflavin Synthase', MS='An enzyme that catalyzes the formation of riboflavin from two molecules of 6,7-dimethyl-8-ribityllumazine, utilizing a four-carbon fragment from one molecule which is transferred to the second molecule. EC 2.5.1.9.'),
  'D012259': NtO(MH='Ribonuclease, Pancreatic', MS="An enzyme that catalyzes the endonucleolytic cleavage of pancreatic ribonucleic acids to 3'-phosphomono- and oligonucleotides ending in cytidylic or uridylic acids with 2',3'-cyclic phosphate intermediates. EC 3.1.27.5."),
  'D012260': NtO(MH='Ribonucleases', MS='Enzymes that catalyze the hydrolysis of ester bonds within RNA. EC 3.1.-.'),
  'D012261': NtO(MH='Ribonucleoproteins', MS='Complexes of RNA-binding proteins with ribonucleic acids (RNA).'),
  'D012262': NtO(MH='Ribonucleoside Diphosphate Reductase', MS="An enzyme of the oxidoreductase class that catalyzes the formation of 2'-deoxyribonucleotides from the corresponding ribonucleotides using NADPH as the ultimate electron donor. The deoxyribonucleoside diphosphates are used in DNA synthesis. (From Dorland, 27th ed) EC 1.17.4.1."),
  'D012263': NtO(MH='Ribonucleosides', MS='Nucleosides in which the purine or pyrimidine base is combined with ribose. (Dorland, 28th ed)'),
  'D012264': NtO(MH='Ribonucleotide Reductases', MS=None),
  'D012265': NtO(MH='Ribonucleotides', MS='Nucleotides in which the purine or pyrimidine base is combined with ribose. (Dorland, 28th ed)'),
  'D012266': NtO(MH='Ribose', MS='A pentose active in biological systems usually in its D-form.'),
  'D012267': NtO(MH='Ribosemonophosphates', MS='Ribose substituted in the 1-, 3-, or 5-position by a phosphoric acid moiety.'),
  'D012268': NtO(MH='Ribose-Phosphate Pyrophosphokinase', MS='An enzyme that catalyzes the formation of phosphoribosyl pyrophosphate from ATP and ribose-5-phosphate. EC 2.7.6.1.'),
  'D012269': NtO(MH='Ribosomal Proteins', MS='Proteins found in ribosomes. They are believed to have a catalytic function in reconstituting biologically active ribosomal subunits.'),
  'D012270': NtO(MH='Ribosomes', MS='Multicomponent ribonucleoprotein structures found in the CYTOPLASM of all cells, and in MITOCHONDRIA, and PLASTIDS. They function in PROTEIN BIOSYNTHESIS via GENETIC TRANSLATION.'),
  'D012271': NtO(MH='Ribostamycin', MS='A broad-spectrum antimicrobial isolated from Streptomyces ribosifidicus.'),
  'D012272': NtO(MH='Ribs', MS='A set of twelve curved bones which connect to the vertebral column posteriorly, and terminate anteriorly as costal cartilage. Together, they form a protective cage around the internal thoracic organs.'),
  'D012273': NtO(MH='Ribulose-Bisphosphate Carboxylase', MS='A carboxy-lyase that plays a key role in photosynthetic carbon assimilation in the CALVIN-BENSON CYCLE by catalyzing the formation of 3-phosphoglycerate from ribulose 1,5-biphosphate and CARBON DIOXIDE. It can also utilize OXYGEN as a substrate to catalyze the synthesis of 2-phosphoglycolate and 3-phosphoglycerate in a process referred to as photorespiration.'),
  'D012274': NtO(MH='Ribulosephosphates', MS='Ribulose substituted by one or more phosphoric acid moieties.'),
  'D012275': NtO(MH='Oryza', MS='A genus of grass family (Poaceae) that include several rice species.'),
  'D012276': NtO(MH='Ricin', MS='A protein phytotoxin from the seeds of Ricinus communis, the castor oil plant. It agglutinates cells, is proteolytic, and causes lethal inflammation and hemorrhage if taken internally.'),
  'D012277': NtO(MH='Ricinoleic Acids', MS='Eighteen carbon fatty acids that comprise the great majority of CASTOR OIL, which is from the seed of RICINUS.'),
  'D012278': NtO(MH='Ricinus', MS='A plant genus of the family Euphorbiaceae. The seed of Ricinus communis is the CASTOR BEAN which is the source of CASTOR OIL; RICIN; and other LECTINS.'),
  'D012279': NtO(MH='Rickets', MS='Disorders caused by interruption of BONE MINERALIZATION manifesting as OSTEOMALACIA in adults and characteristic deformities in infancy and childhood due to disturbances in normal BONE FORMATION. The mineralization process may be interrupted by disruption of VITAMIN D; PHOSPHORUS; or CALCIUM homeostasis, resulting from dietary deficiencies, or acquired, or inherited metabolic, or hormonal disturbances.'),
  'D012281': NtO(MH='Rickettsia', MS='A genus of gram-negative, aerobic, rod-shaped bacteria often surrounded by a protein microcapsular layer and slime layer. The natural cycle of its organisms generally involves a vertebrate and an invertebrate host. Species of the genus are the etiological agents of human diseases, such as typhus.'),
  'D012282': NtO(MH='Rickettsia Infections', MS='Infections by the genus RICKETTSIA.'),
  'D012283': NtO(MH='Rickettsia prowazekii', MS="A species of gram-negative, aerobic bacteria that is the etiologic agent of epidemic typhus fever acquired through contact with lice (TYPHUS, EPIDEMIC LOUSE-BORNE) as well as Brill's disease."),
  'D012284': NtO(MH='Rickettsia rickettsii', MS='A species of gram-negative, aerobic bacteria that is the etiologic agent of ROCKY MOUNTAIN SPOTTED FEVER. Its cells are slightly smaller and more uniform in size than those of RICKETTSIA PROWAZEKII.'),
  'D012285': NtO(MH='Orientia tsutsugamushi', MS='A gram-negative, rod-shaped to coccoid bacterium. It is the etiologic agent of SCRUB TYPHUS in humans and is transmitted by mites from rodent reservoirs.'),
  'D012286': NtO(MH='Rickettsia typhi', MS='The etiologic agent of murine typhus (see TYPHUS, ENDEMIC FLEA-BORNE).'),
  'D012287': NtO(MH='Rickettsiaceae', MS='A family of gram-negative bacteria belonging to the order Rickettsiales.'),
  'D012288': NtO(MH='Rickettsiaceae Infections', MS='Infections with bacteria of the family RICKETTSIACEAE.'),
  'D012290': NtO(MH='Rickettsial Vaccines', MS='Vaccines for the prevention of diseases caused by various species of Rickettsia.'),
  'D012293': NtO(MH='Rifampin', MS="A semisynthetic antibiotic produced from Streptomyces mediterranei. It has a broad antibacterial spectrum, including activity against several forms of Mycobacterium. In susceptible organisms it inhibits DNA-dependent RNA polymerase activity by forming a stable complex with the enzyme. It thus suppresses the initiation of RNA synthesis. Rifampin is bactericidal, and acts on both intracellular and extracellular organisms. (From Gilman et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 9th ed, p1160)"),
  'D012294': NtO(MH='Rifamycins', MS='A group of ANTI-BACTERIAL AGENTS characterized by a chromophoric naphthohydroquinone group spanned by an aliphatic bridge not previously found in other known ANTI-BACTERIAL AGENTS. They have been isolated from fermentation broths of Streptomyces mediterranei.'),
  'D012295': NtO(MH='Rift Valley Fever', MS='An acute infection caused by the RIFT VALLEY FEVER VIRUS, an RNA arthropod-borne virus, affecting domestic animals and humans. In animals, symptoms include HEPATITIS; abortion (ABORTION, VETERINARY); and DEATH. In humans, symptoms range from those of a flu-like disease to hemorrhagic fever, ENCEPHALITIS, or BLINDNESS.'),
  'D012296': NtO(MH='Rift Valley fever virus', MS='A mosquito-borne species of the PHLEBOVIRUS genus found in eastern, central, and southern Africa, producing massive hepatitis, abortion, and death in sheep, goats, cattle, and other animals. It also has caused disease in humans.'),
  'D012297': NtO(MH='Right to Die', MS="The right of the patient or the patient's representative to make decisions with regard to the patient's dying."),
  'D012298': NtO(MH='Rigor Mortis', MS='Muscular rigidity which develops in the cadaver usually from 4 to 10 hours after death and lasts 3 or 4 days.'),
  'D012299': NtO(MH='Rimantadine', MS='An RNA synthesis inhibitor that is used as an antiviral agent in the prophylaxis and treatment of influenza.'),
  'D012301': NtO(MH='Rinderpest', MS='A viral disease of cloven-hoofed animals caused by MORBILLIVIRUS. It may be acute, subacute, or chronic with the major lesions characterized by inflammation and ulceration of the entire digestive tract. The disease was declared successfully eradicated worldwide in 2010.'),
  'D012302': NtO(MH='Rinderpest virus', MS='A species of MORBILLIVIRUS causing cattle plague, a disease with high mortality. Sheep, goats, pigs, and other animals of the order Artiodactyla can also be infected.'),
  'D012303': NtO(MH='Ring Chromosomes', MS='Aberrant chromosomes with no ends, i.e., circular.'),
  'D012304': NtO(MH='Riot Control Agents, Chemical', MS='Chemical substances which are employed during a riot in order to control or disperse the rioting parties.'),
  'D012305': NtO(MH='Riots', MS='A form of violent crowd behavior which expresses the emotional release of resentments and prejudices, usually relevant to grievances toward the social system.'),
  'D012306': NtO(MH='Risk', MS='The probability that an event will occur. It encompasses a variety of measures of the probability of a generally unfavorable outcome.'),
  'D012307': NtO(MH='Risk Factors', MS='An aspect of personal behavior or lifestyle, environmental exposure, inborn or inherited characteristic, which, on the basis of epidemiological evidence, is known to be associated with a health-related condition considered important to prevent.'),
  'D012308': NtO(MH='Risk Management', MS='The process of minimizing risk to an organization by developing systems to identify and analyze potential hazards to prevent accidents, injuries, and other adverse occurrences, and by attempting to handle events and incidents which do occur in such a manner that their effect and cost are minimized. Effective risk management has its greatest benefits in application to insurance in order to avert or minimize financial liability. (From Slee & Slee: Health care terms, 2d ed)'),
  'D012309': NtO(MH='Risk-Taking', MS='Undertaking a task involving a challenge for achievement or a desirable goal in which there is a lack of certainty or a fear of failure. It may also include the exhibiting of certain behaviors whose outcomes may present a risk to the individual or to those associated with him or her.'),
  'D012310': NtO(MH='Ristocetin', MS='An antibiotic mixture of two components, A and B, obtained from Nocardia lurida (or the same substance produced by any other means). It is no longer used clinically because of its toxicity. It causes platelet agglutination and blood coagulation and is used to assay those functions in vitro.'),
  'D012312': NtO(MH='Ritodrine', MS='An adrenergic beta-2 agonist used to control PREMATURE LABOR.'),
  'D012313': NtO(MH='RNA', MS='A polynucleotide consisting essentially of chains with a repeating backbone of phosphate and ribose units to which nitrogenous bases are attached. RNA is unique among biological macromolecules in that it can encode genetic information, serve as an abundant structural component of cells, and also possesses catalytic activity. (Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D012314': NtO(MH='RNA Cap Analogs', MS='Analogs of RNA cap compounds which do not have a positive charge. These compounds inhibit the initiation of translation of both capped and uncapped messenger RNA.'),
  'D012315': NtO(MH='RNA Caps', MS="Nucleic acid structures found on the 5' end of eukaryotic cellular and viral messenger RNA and some heterogeneous nuclear RNAs. These structures, which are positively charged, protect the above specified RNAs at their termini against attack by phosphatases and other nucleases and promote mRNA function at the level of initiation of translation. Analogs of the RNA caps (RNA CAP ANALOGS), which lack the positive charge, inhibit the initiation of protein synthesis."),
  'D012316': NtO(MH='RNA Nucleotidyltransferases', MS='Enzymes that catalyze the template-directed incorporation of ribonucleotides into an RNA chain. EC 2.7.7.-.'),
  'D012317': NtO(MH='RNA Phages', MS="Bacteriophages whose genetic material is RNA, which is single-stranded in all except the Pseudomonas phage phi 6 (BACTERIOPHAGE PHI 6). All RNA phages infect their host bacteria via the host's surface pili. Some frequently encountered RNA phages are: BF23, F2, R17, fr, PhiCb5, PhiCb12r, PhiCb8r, PhiCb23r, 7s, PP7, Q beta phage, MS2 phage, and BACTERIOPHAGE PHI 6."),
  'D012318': NtO(MH='RNA Polymerase I', MS='A DNA-dependent RNA polymerase present in bacterial, plant, and animal cells. The enzyme functions in the nucleolar structure and transcribes DNA into RNA. It has different requirements for cations and salts than RNA polymerase II and III and is not inhibited by alpha-amanitin.'),
  'D012319': NtO(MH='RNA Polymerase II', MS='A DNA-dependent RNA polymerase present in bacterial, plant, and animal cells. It functions in the nucleoplasmic structure and transcribes DNA into RNA. It has different requirements for cations and salt than RNA polymerase I and is strongly inhibited by alpha-amanitin. EC 2.7.7.6.'),
  'D012320': NtO(MH='RNA Polymerase III', MS='A DNA-dependent RNA polymerase present in bacterial, plant, and animal cells. It functions in the nucleoplasmic structure where it transcribes DNA into RNA. It has specific requirements for cations and salt and has shown an intermediate sensitivity to alpha-amanitin in comparison to RNA polymerase I and II.'),
  'D012321': NtO(MH='DNA-Directed RNA Polymerases', MS="Enzymes that catalyze DNA template-directed extension of the 3'-end of an RNA strand one nucleotide at a time. They can initiate a chain de novo. In eukaryotes, three forms of the enzyme have been distinguished on the basis of sensitivity to alpha-amanitin, and the type of RNA synthesized. (From Enzyme Nomenclature, 1992)."),
  'D012322': NtO(MH='RNA Precursors', MS='RNA transcripts of the DNA that are in some unfinished stage of post-transcriptional processing (RNA PROCESSING, POST-TRANSCRIPTIONAL) required for function. RNA precursors may undergo several steps of RNA SPLICING during which the phosphodiester bonds at exon-intron boundaries are cleaved and the introns are excised. Consequently a new bond is formed between the ends of the exons. Resulting mature RNAs can then be used; for example, mature mRNA (RNA, MESSENGER) is used as a template for protein production.'),
  'D012323': NtO(MH='RNA Processing, Post-Transcriptional', MS='Post-transcriptional biological modification of messenger, transfer, or ribosomal RNAs or their precursors. It includes cleavage, methylation, thiolation, isopentenylation, pseudouridine formation, conformational changes, and association with ribosomal protein.'),
  'D012324': NtO(MH='RNA Replicase', MS="An enzyme that catalyses RNA-template-directed extension of the 3'- end of an RNA strand by one nucleotide at a time, and can initiate a chain de novo. (Enzyme Nomenclature, 1992, p293)"),
  'D012326': NtO(MH='RNA Splicing', MS='The ultimate exclusion of nonsense sequences or intervening sequences (introns) before the final RNA transcript is sent to the cytoplasm.'),
  'D012327': NtO(MH='RNA Virus Infections', MS='Diseases caused by RNA VIRUSES.'),
  'D012328': NtO(MH='RNA Viruses', MS='Viruses whose genetic material is RNA.'),
  'D012329': NtO(MH='RNA, Bacterial', MS='Ribonucleic acid in bacteria having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D012330': NtO(MH='RNA, Double-Stranded', MS='RNA consisting of two strands as opposed to the more prevalent single-stranded RNA. Most of the double-stranded segments are formed from transcription of DNA by intramolecular base-pairing of inverted complementary sequences separated by a single-stranded loop. Some double-stranded segments of RNA are normal in all organisms.'),
  'D012331': NtO(MH='RNA, Fungal', MS='Ribonucleic acid in fungi having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D012332': NtO(MH='RNA, Heterogeneous Nuclear', MS='Nuclear nonribosomal RNA larger than about 1000 nucleotides, the mass of which is rapidly synthesized and degraded within the cell nucleus. Some heterogeneous nuclear RNA may be a precursor to mRNA. However, the great bulk of total hnRNA hybridizes with nuclear DNA rather than with mRNA.'),
  'D012333': NtO(MH='RNA, Messenger', MS="RNA sequences that serve as templates for protein synthesis. Bacterial mRNAs are generally primary transcripts in that they do not require post-transcriptional processing. Eukaryotic mRNA is synthesized in the nucleus and must be exported to the cytoplasm for translation. Most eukaryotic mRNAs have a sequence of polyadenylic acid at the 3' end, referred to as the poly(A) tail. The function of this tail is not known for certain, but it may play a role in the export of mature mRNA from the nucleus as well as in helping stabilize some mRNA molecules by retarding their degradation in the cytoplasm."),
  'D012334': NtO(MH='RNA, Neoplasm', MS='RNA present in neoplastic tissue.'),
  'D012335': NtO(MH='RNA, Ribosomal', MS='The most abundant form of RNA. Together with proteins, it forms the ribosomes, playing a structural role and also a role in ribosomal binding of mRNA and tRNAs. Individual chains are conventionally designated by their sedimentation coefficients. In eukaryotes, four large chains exist, synthesized in the nucleolus and constituting about 50% of the ribosome. (Dorland, 28th ed)'),
  'D012336': NtO(MH='RNA, Ribosomal, 16S', MS='Constituent of 30S subunit prokaryotic ribosomes containing 1600 nucleotides and 21 proteins. 16S rRNA is involved in initiation of polypeptide synthesis.'),
  'D012337': NtO(MH='RNA, Ribosomal, 18S', MS='Constituent of the 40S subunit of eukaryotic ribosomes. 18S rRNA is involved in the initiation of polypeptide synthesis in eukaryotes.'),
  'D012338': NtO(MH='RNA, Ribosomal, 23S', MS='Constituent of 50S subunit of prokaryotic ribosomes containing about 3200 nucleotides. 23S rRNA is involved in the initiation of polypeptide synthesis.'),
  'D012339': NtO(MH='RNA, Ribosomal, 28S', MS='Constituent of the 60S subunit of eukaryotic ribosomes. 28S rRNA is involved in the initiation of polypeptide synthesis in eukaryotes.'),
  'D012340': NtO(MH='RNA, Ribosomal, 5.8S', MS='Constituent of the 60S subunit of eukaryotic ribosomes. 5.8S rRNA is involved in the initiation of polypeptide synthesis in eukaryotes.'),
  'D012341': NtO(MH='RNA, Ribosomal, 5S', MS='Constituent of the 50S subunit of prokaryotic ribosomes containing about 120 nucleotides and 34 proteins. It is also a constituent of the 60S subunit of eukaryotic ribosomes. 5S rRNA is involved in initiation of polypeptide synthesis.'),
  'D012342': NtO(MH='RNA, Small Nuclear', MS='Short chains of RNA (100-300 nucleotides long) that are abundant in the nucleus and usually complexed with proteins in snRNPs (RIBONUCLEOPROTEINS, SMALL NUCLEAR). Many function in the processing of messenger RNA precursors. Others, the snoRNAs (RNA, SMALL NUCLEOLAR), are involved with the processing of ribosomal RNA precursors.'),
  'D012343': NtO(MH='RNA, Transfer', MS='The small RNA molecules, 73-80 nucleotides long, that function during translation (TRANSLATION, GENETIC) to align AMINO ACIDS at the RIBOSOMES in a sequence determined by the mRNA (RNA, MESSENGER). There are about 30 different transfer RNAs. Each recognizes a specific CODON set on the mRNA through its own ANTICODON and as aminoacyl tRNAs (RNA, TRANSFER, AMINO ACYL), each carries a specific amino acid to the ribosome to add to the elongating peptide chains.'),
  'D012344': NtO(MH='RNA, Transfer, Ala', MS='A transfer RNA which is specific for carrying alanine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012345': NtO(MH='RNA, Transfer, Amino Acid-Specific', MS='A group of transfer RNAs which are specific for carrying each one of the 20 amino acids to the ribosome in preparation for protein synthesis.'),
  'D012346': NtO(MH='RNA, Transfer, Amino Acyl', MS='Intermediates in protein biosynthesis. The compounds are formed from amino acids, ATP and transfer RNA, a reaction catalyzed by aminoacyl tRNA synthetase. They are key compounds in the genetic translation process.'),
  'D012347': NtO(MH='RNA, Transfer, Arg', MS='A transfer RNA which is specific for carrying arginine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012348': NtO(MH='RNA, Transfer, Asn', MS='A transfer RNA which is specific for carrying asparagine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012349': NtO(MH='RNA, Transfer, Asp', MS='A transfer RNA which is specific for carrying aspartic acid to sites on the ribosomes in preparation for protein synthesis.'),
  'D012350': NtO(MH='RNA, Transfer, Cys', MS='A transfer RNA which is specific for carrying cysteine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012351': NtO(MH='RNA, Transfer, Gln', MS='A transfer RNA which is specific for carrying glutamine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012352': NtO(MH='RNA, Transfer, Glu', MS='A transfer RNA which is specific for carrying glutamic acid to sites on the ribosomes in preparation for protein synthesis.'),
  'D012353': NtO(MH='RNA, Transfer, Gly', MS='A transfer RNA which is specific for carrying glycine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012354': NtO(MH='RNA, Transfer, His', MS='A transfer RNA which is specific for carrying histidine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012355': NtO(MH='RNA, Transfer, Ile', MS='A transfer RNA which is specific for carrying isoleucine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012356': NtO(MH='RNA, Transfer, Leu', MS='A transfer RNA which is specific for carrying leucine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012357': NtO(MH='RNA, Transfer, Lys', MS='A transfer RNA which is specific for carrying lysine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012358': NtO(MH='RNA, Transfer, Met', MS='A transfer RNA which is specific for carrying methionine to sites on the ribosomes. During initiation of protein synthesis, tRNA(f)Met in prokaryotic cells and tRNA(i)Met in eukaryotic cells binds to the start codon (CODON, INITIATOR).'),
  'D012359': NtO(MH='tRNA Methyltransferases', MS='Enzymes that catalyze the S-adenosyl-L-methionine-dependent methylation of ribonucleotide bases within a transfer RNA molecule. EC 2.1.1.'),
  'D012360': NtO(MH='RNA, Transfer, Phe', MS='A transfer RNA which is specific for carrying phenylalanine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012361': NtO(MH='RNA, Transfer, Pro', MS='A transfer RNA which is specific for carrying proline to sites on the ribosomes in preparation for protein synthesis.'),
  'D012362': NtO(MH='RNA, Transfer, Ser', MS='A transfer RNA which is specific for carrying serine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012363': NtO(MH='RNA, Transfer, Thr', MS='A transfer RNA which is specific for carrying threonine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012364': NtO(MH='RNA, Transfer, Trp', MS='A transfer RNA which is specific for carrying tryptophan to sites on the ribosomes in preparation for protein synthesis.'),
  'D012365': NtO(MH='RNA, Transfer, Tyr', MS='A transfer RNA which is specific for carrying tyrosine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012366': NtO(MH='RNA, Transfer, Val', MS='A transfer RNA which is specific for carrying valine to sites on the ribosomes in preparation for protein synthesis.'),
  'D012367': NtO(MH='RNA, Viral', MS='Ribonucleic acid that makes up the genetic material of viruses.'),
  'D012368': NtO(MH='4-(3-Butoxy-4-methoxybenzyl)-2-imidazolidinone', MS='Inhibitor of phosphodiesterases.'),
  'D012369': NtO(MH='2H-Benzo(a)quinolizin-2-ol, 2-Ethyl-1,3,4,6,7,11b-hexahydro-3-isobutyl-9,10-dimethoxy-', MS='Proposed catecholamine depletor.'),
  'D012370': NtO(MH='Robenidine', MS='An anticoccidial agent mainly for poultry.'),
  'D012371': NtO(MH='Robotics', MS='The application of electronic, computerized control systems to mechanical devices designed to perform human functions. Formerly restricted to industry, but nowadays applied to artificial organs controlled by bionic (bioelectronic) devices, like automated insulin pumps and other prostheses.'),
  'D012372': NtO(MH='ROC Curve', MS='A graphic means for assessing the ability of a screening test to discriminate between healthy and diseased persons; may also be used in other studies, e.g., distinguishing stimuli responses as to a faint stimuli or nonstimuli.'),
  'D012373': NtO(MH='Rocky Mountain Spotted Fever', MS='An acute febrile illness caused by RICKETTSIA RICKETTSII. It is transmitted to humans by bites of infected ticks and occurs only in North and South America. Characteristics include a sudden onset with headache and chills and fever lasting about two to three weeks. A cutaneous rash commonly appears on the extremities and trunk about the fourth day of illness.'),
  'D012374': NtO(MH='Rod Cell Outer Segment', MS='The portion of a retinal rod cell situated between the ROD INNER SEGMENT and the RETINAL PIGMENT EPITHELIUM. It contains a stack of photosensitive disk membranes laden with RHODOPSIN.'),
  'D012375': NtO(MH='Rodent Control', MS='The reduction or regulation of the population of noxious, destructive, or dangerous rodents through chemical, biological, or other means.'),
  'D012376': NtO(MH='Rodent Diseases', MS='Diseases of rodents of the order RODENTIA. This term includes diseases of Sciuridae (squirrels), Geomyidae (gophers), Heteromyidae (pouched mice), Castoridae (beavers), Cricetidae (rats and mice), Muridae (Old World rats and mice), Erethizontidae (porcupines), and Caviidae (guinea pigs).'),
  'D012377': NtO(MH='Rodentia', MS='A mammalian order which consists of 29 families and many genera.'),
  'D012378': NtO(MH='Rodenticides', MS='Substances used to destroy or inhibit the action of rats, mice, or other rodents.'),
  'D012380': NtO(MH='Role', MS='The expected and characteristic pattern of behavior exhibited by an individual as a member of a particular social group.'),
  'D012381': NtO(MH='Role Playing', MS='The adopting or performing the role of another significant individual in order to gain insight into the behavior of that person.'),
  'D012382': NtO(MH='Rolitetracycline', MS='A pyrrolidinylmethyl TETRACYCLINE.'),
  'D012383': NtO(MH='Romania', MS='A country in southeastern Europe, bordering the Black Sea, between Bulgaria and Ukraine. The capital is Bucharest.'),
  'D012384': NtO(MH='Rome', MS='The capital city of Italy.'),
  'D012385': NtO(MH='Ronidazole', MS='Antiprotozoal and antimicrobial agent used mainly in veterinary practice.'),
  'D012386': NtO(MH='Rooming-in Care', MS="Care of the newborn infant in a crib near the mother's bed, instead of in a nursery, during the hospital stay."),
  'D012387': NtO(MH='Root Canal Filling Materials', MS="Materials placed inside a root canal for the purpose of obturating or sealing it. The materials may be gutta-percha, silver cones, paste mixtures, or other substances. (Dorland, 28th ed, p631 & Boucher's Clinical Dental Terminology, 4th ed, p187)"),
  'D012388': NtO(MH='Root Canal Irrigants', MS='Chemicals used mainly to disinfect root canals after pulpectomy and before obturation. The major ones are camphorated monochlorophenol, EDTA, formocresol, hydrogen peroxide, metacresylacetate, and sodium hypochlorite. Root canal irrigants include also rinsing solutions of distilled water, sodium chloride, etc.'),
  'D012389': NtO(MH='Root Canal Obturation', MS='Phase of endodontic treatment in which a root canal system that has been cleaned is filled through use of special materials and techniques in order to prevent reinfection.'),
  'D012390': NtO(MH='Root Canal Therapy', MS='A treatment modality in endodontics concerned with the therapy of diseases of the dental pulp. For preparatory procedures, ROOT CANAL PREPARATION is available.'),
  'D012391': NtO(MH='Root Resorption', MS='Resorption in which cementum or dentin is lost from the root of a tooth owing to cementoclastic or osteoclastic activity in conditions such as trauma of occlusion or neoplasms. (Dorland, 27th ed)'),
  'D012392': NtO(MH='Rorschach Test', MS="A projective test used to evaluate a broad range of personality variables including pathology of thought and perception. The subject's responses to inkblot prints are scored along with subjective interpretation by the test administrator."),
  'D012393': NtO(MH='Rosacea', MS="A cutaneous disorder primarily of convexities of the central part of the FACE, such as FOREHEAD; CHEEK; NOSE; and CHIN. It is characterized by FLUSHING; ERYTHEMA; EDEMA; RHINOPHYMA; papules; and ocular symptoms. It may occur at any age but typically after age 30. There are various subtypes of rosacea: erythematotelangiectatic, papulopustular, phymatous, and ocular (National Rosacea Society's Expert Committee on the Classification and Staging of Rosacea, J Am Acad Dermatol 2002; 46:584-7)."),
  'D012394': NtO(MH='Rosaniline Dyes', MS='Compounds that contain the triphenylmethane aniline structure found in rosaniline. Many of them have a characteristic magenta color and are used as COLORING AGENTS.'),
  'D012395': NtO(MH='Rose Bengal', MS='A bright bluish pink compound that has been used as a dye, biological stain, and diagnostic aid.'),
  'D012397': NtO(MH='Rosette Formation', MS='The in vitro formation of clusters consisting of a cell (usually a lymphocyte) surrounded by antigenic cells or antigen-bearing particles (usually erythrocytes, which may or may not be coated with antibody or antibody and complement). The rosette-forming cell may be an antibody-forming cell, a memory cell, a T-cell, a cell bearing surface cytophilic antibodies, or a monocyte possessing Fc receptors. Rosette formation can be used to identify specific populations of these cells.'),
  'D012398': NtO(MH='Ross River virus', MS='A species of ALPHAVIRUS associated with epidemic EXANTHEMA and polyarthritis in Australia.'),
  'D012399': NtO(MH='Rotation', MS='Motion of an object in which either one or more points on a line are fixed. It is also the motion of a particle about a fixed point. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D012400': NtO(MH='Rotavirus Infections', MS='Infection with any of the rotaviruses. Specific infections include human infantile diarrhea, neonatal calf diarrhea, and epidemic diarrhea of infant mice.'),
  'D012401': NtO(MH='Rotavirus', MS='A genus of REOVIRIDAE, causing acute gastroenteritis in BIRDS and MAMMALS, including humans. Transmission is horizontal and by environmental contamination. Seven species (Rotaviruses A thru G) are recognized.'),
  'D012402': NtO(MH='Rotenone', MS='A botanical insecticide that is an inhibitor of mitochondrial electron transport.'),
  'D012403': NtO(MH='Rotifera', MS='A class of minute animals of the phylum Aschelminthes.'),
  'D012404': NtO(MH='Round Ligament of Uterus', MS='A fibromuscular band that attaches to the UTERUS and then passes along the BROAD LIGAMENT, out through the INGUINAL RING, and into the labium majus.'),
  'D012405': NtO(MH='Round Window, Ear', MS='Fenestra of the cochlea, an opening in the basal wall between the MIDDLE EAR and the INNER EAR, leading to the cochlea. It is closed by a secondary tympanic membrane.'),
  'D012406': NtO(MH='Roxarsone', MS='An arsenic derivative which has anticoccidial action and promotes growth in animals.'),
  'D012407': NtO(MH='rRNA Operon', MS='Genetic loci which direct transcription of ribosomal RNA in bacterial operons. They are designated rrnB, rrnC, rrnD, etc. according to the structural position of the transcription unit in the DNA sequence.'),
  'D012408': NtO(MH='Rubber', MS='A high-molecular-weight polymeric elastomer derived from the milk juice (LATEX) of HEVEA brasiliensis and other trees and plants. It is a substance that can be stretched at room temperature to at least twice its original length and after releasing the stress, retract rapidly, and recover its original dimensions fully.'),
  'D012409': NtO(MH='Rubella', MS='An acute infectious disease caused by the RUBELLA VIRUS. The virus enters the respiratory tract via airborne droplet and spreads to the LYMPHATIC SYSTEM.'),
  'D012410': NtO(MH='Rubella Syndrome, Congenital', MS='Transplacental infection of the fetus with rubella usually in the first trimester of pregnancy, as a consequence of maternal infection, resulting in various developmental abnormalities in the newborn infant. They include cardiac and ocular lesions, deafness, microcephaly, mental retardation, and generalized growth retardation. (From Dorland, 27th ed)'),
  'D012411': NtO(MH='Rubella Vaccine', MS='A live attenuated virus vaccine of duck embryo or human diploid cell tissue culture origin, used for routine immunization of children and for immunization of nonpregnant adolescent and adult females of childbearing age who are unimmunized and do not have serum antibodies to rubella. Children are usually immunized with measles-mumps-rubella combination vaccine. (Dorland, 28th ed)'),
  'D012412': NtO(MH='Rubella virus', MS='The type (and only) species of RUBIVIRUS causing acute infection in humans, primarily children and young adults. Humans are the only natural host. A live, attenuated vaccine is available for prophylaxis.'),
  'D012413': NtO(MH='Rubidium', MS='An element that is an alkali metal. It has an atomic symbol Rb, atomic number 37, and atomic weight 85.47. It is used as a chemical reagent and in the manufacture of photoelectric cells.'),
  'D012414': NtO(MH='Rubidium Radioisotopes', MS='Unstable isotopes of rubidium that decay or disintegrate emitting radiation. Rb atoms with atomic weights 79-84, and 86-95 are radioactive rubidium isotopes.'),
  'D012415': NtO(MH='Rubinstein-Taybi Syndrome', MS='A chromosomal disorder characterized by MENTAL RETARDATION, broad thumbs, webbing of fingers and toes, beaked nose, short upper lip, pouting lower lip, agenesis of corpus callosum, large foramen magnum, keloid formation, pulmonary stenosis, vertebral anomalies, chest wall anomalies, sleep apnea, and megacolon. The disease has an autosomal dominant pattern of inheritance and is associated with deletions of the short arm of chromosome 16 (16p13.3).'),
  'D012416': NtO(MH='Rubredoxins', MS='A class of iron-sulfur proteins that contains one iron coordinated to the sulfur atom of four cysteine residues. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D012417': NtO(MH='Rumen', MS="The first stomach of ruminants. It lies on the left side of the body, occupying the whole of the left side of the abdomen and even stretching across the median plane of the body to the right side. It is capacious, divided into an upper and a lower sac, each of which has a blind sac at its posterior extremity. The rumen is lined by mucous membrane containing no digestive glands, but mucus-secreting glands are present in large numbers. Coarse, partially chewed food is stored and churned in the rumen until the animal finds circumstances convenient for rumination. When this occurs, little balls of food are regurgitated through the esophagus into the mouth, and are subjected to a second more thorough mastication, swallowed, and passed on into other parts of the compound stomach. (From Black's Veterinary Dictionary, 17th ed)"),
  'D012418': NtO(MH='Ruminants', MS='A suborder of the order ARTIODACTYLA whose members have the distinguishing feature of a four-chambered stomach, including the capacious RUMEN. Horns or antlers are usually present, at least in males.'),
  'D012419': NtO(MH='Runaway Behavior', MS='A behavioral response manifested by leaving home in order to escape from threatening situations. Children or adolescents leaving home without permission is usually implied.'),
  'D012420': NtO(MH='Running', MS='An activity in which the body is propelled by moving the legs rapidly. Running is performed at a moderate to rapid pace and should be differentiated from JOGGING, which is performed at a much slower pace.'),
  'D012421': NtO(MH='Rupture', MS='Forcible or traumatic tear or break of an organ or other soft part of the body.'),
  'D012422': NtO(MH='Rupture, Spontaneous', MS='Tear or break of an organ, vessel or other soft part of the body, occurring in the absence of external force.'),
  'D012423': NtO(MH='Rural Health', MS='The status of health in rural populations.'),
  'D012424': NtO(MH='Rural Population', MS='The inhabitants of rural areas or of small towns classified as rural.'),
  'D012425': NtO(MH='Russia (Pre-1917)', MS='A monarchy located in north Asia bordering the Arctic Ocean, extending from Europe (the portion west of the Urals) to the North Pacific Ocean. St Petersburg was the capital 1712\xe2\x80\x931918. Moscow was the capital before St. Petersburg and following it.'),
  'D012426': NtO(MH='Russia', MS='A country located in north Asia bordering the Arctic Ocean, extending from Europe (the portion west of the Urals) to the North Pacific Ocean. The capital is Moscow.'),
  'D012427': NtO(MH='Rutamycin', MS='A macrolide antibiotic of the oligomycin group, obtained from Streptomyces rutgersensis. It is used in cytochemistry as a tool to inhibit various ATPases and to uncouple oxidative phosphorylation from electron transport and also clinically as an antifungal agent.'),
  'D012428': NtO(MH='Ruthenium', MS='A hard, brittle, grayish-white rare earth metal with an atomic symbol Ru, atomic number 44, and atomic weight 101.07. It is used as a catalyst and hardener for PLATINUM and PALLADIUM.'),
  'D012429': NtO(MH='Ruthenium Radioisotopes', MS='Unstable isotopes of ruthenium that decay or disintegrate emitting radiation. Ru atoms with atomic weights 93-95, 97, 103, and 105-108 are radioactive ruthenium isotopes.'),
  'D012430': NtO(MH='Ruthenium Red', MS='An inorganic dye used in microscopy for differential staining and as a diagnostic reagent. In research this compound is used to study changes in cytoplasmic concentrations of calcium. Ruthenium red inhibits calcium transport through membrane channels.'),
  'D012431': NtO(MH='Rutin', MS='A flavonol glycoside found in many plants, including BUCKWHEAT; TOBACCO; FORSYTHIA; HYDRANGEA; VIOLA, etc. It has been used therapeutically to decrease capillary fragility.'),
  'D012432': NtO(MH='Rwanda', MS='A republic in eastern Africa, south of UGANDA, east of DEMOCRATIC REPUBLIC OF THE CONGO, west of TANZANIA. Its capital is Kigali. It was formerly part of the Belgian trust territory of Ruanda-Urund.'),
  'D012433': NtO(MH='Ryanodine', MS='A methylpyrrole-carboxylate from RYANIA that disrupts the RYANODINE RECEPTOR CALCIUM RELEASE CHANNEL to modify CALCIUM release from SARCOPLASMIC RETICULUM resulting in alteration of MUSCLE CONTRACTION. It was previously used in INSECTICIDES. It is used experimentally in conjunction with THAPSIGARGIN and other inhibitors of CALCIUM ATPASE uptake of calcium into SARCOPLASMIC RETICULUM.'),
  'D012434': NtO(MH='Secale', MS='Genus in the grass family (Poaceae) including well known species Secale cereale (rye).'),
  'D012435': NtO(MH='S-Adenosylhomocysteine', MS="5'-S-(3-Amino-3-carboxypropyl)-5'-thioadenosine. Formed from S-adenosylmethionine after transmethylation reactions."),
  'D012436': NtO(MH='S-Adenosylmethionine', MS='Physiologic methyl radical donor involved in enzymatic transmethylation reactions and present in all living organisms. It possesses anti-inflammatory activity and has been used in treatment of chronic liver disease. (From Merck, 11th ed)'),
  'D012437': NtO(MH='Adenosylmethionine Decarboxylase', MS="An enzyme that catalyzes the decarboxylation of S-adenosyl-L-methionine to yield 5'-deoxy-(5'-),3-aminopropyl-(1), methylsulfonium salt. It is one of the enzymes responsible for the synthesis of spermidine from putrescine. EC 4.1.1.50."),
  'D012438': NtO(MH='Saccades', MS='An abrupt voluntary shift in ocular fixation from one point to another, as occurs in reading.'),
  'D012439': NtO(MH='Saccharin', MS='Flavoring agent and non-nutritive sweetener.'),
  'D012440': NtO(MH='Saccharomyces', MS='A genus of ascomycetous fungi of the family Saccharomycetaceae, order SACCHAROMYCETALES.'),
  'D012441': NtO(MH='Saccharomyces cerevisiae', MS='A species of the genus SACCHAROMYCES, family Saccharomycetaceae, order Saccharomycetales, known as "baker\'s" or "brewer\'s" yeast. The dried form is used as a dietary supplement.'),
  'D012442': NtO(MH='Saccharomycopsis', MS='Yeast-like ascomycetous fungi of the family Saccharomycopsidaceae, order SACCHAROMYCETALES, isolated from the stomach of rabbits and some other animals.'),
  'D012443': NtO(MH='Saccharopine Dehydrogenases', MS='Amine oxidoreductases that use either NAD+ (EC 1.5.1.7) or NADP+ (EC 1.5.1.8) as an acceptor to form L-LYSINE or NAD+ (EC 1.5.1.9) or NADP+ (EC 1.5.1.10) as an acceptor to form L-GLUTAMATE. Deficiency of this enzyme causes HYPERLYSINEMIAS.'),
  'D012444': NtO(MH='Saccule and Utricle', MS='Two membranous sacs within the vestibular labyrinth of the INNER EAR. The saccule communicates with COCHLEAR DUCT through the ductus reuniens, and communicates with utricle through the utriculosaccular duct from which the ENDOLYMPHATIC DUCT arises. The utricle and saccule have sensory areas (acoustic maculae) which are innervated by the VESTIBULAR NERVE.'),
  'D012445': NtO(MH='Sacrococcygeal Region', MS='The body region between (and flanking) the SACRUM and COCCYX.'),
  'D012446': NtO(MH='Sacroiliac Joint', MS='The immovable joint formed by the lateral surfaces of the SACRUM and ILIUM.'),
  'D012447': NtO(MH='Sacrum', MS='Five fused VERTEBRAE forming a triangle-shaped structure at the back of the PELVIS. It articulates superiorly with the LUMBAR VERTEBRAE, inferiorly with the COCCYX, and anteriorly with the ILIUM of the PELVIS. The sacrum strengthens and stabilizes the PELVIS.'),
  'D012448': NtO(MH='Sadism', MS='A condition in which there is a derivation of pleasure from inflicting pain, discomfort or humiliation on another person or persons. The sexual significance of sadistic wishes or behavior may be conscious or unconscious.'),
  'D012449': NtO(MH='Safety', MS='Freedom from exposure to danger and protection from the occurrence or risk of injury or loss. It suggests optimal precautions in the workplace, on the street, in the home, etc., and includes personal safety as well as the safety of property.'),
  'D012450': NtO(MH='Safflower Oil', MS='An oily liquid extracted from the seeds of the safflower, Carthamus tinctorius. It is used as a dietary supplement in the management of HYPERCHOLESTEROLEMIA. It is used also in cooking, as a salad oil, and as a vehicle for medicines, paints, varnishes, etc. (Dorland, 28th ed & Random House Unabridged Dictionary, 2d ed)'),
  'D012451': NtO(MH='Safrole', MS='A member of the BENZODIOXOLES that is a constituent of several VOLATILE OILS, notably SASSAFRAS oil. It is a precursor in the synthesis of the insecticide PIPERONYL BUTOXIDE and the drug N-methyl-3,4-methylenedioxyamphetamine (MDMA).'),
  'D012452': NtO(MH='Saguinus', MS='A genus in the subfamily CALLITRICHINAE consisting of 12 species and found in Panama as well as South America. Species seen most frequently in the literature are S. oedipus (cotton-top marmoset), S. nigricollis, and S. fusicollis.'),
  'D012453': NtO(MH='Saimiri', MS='A genus of the family CEBIDAE consisting of four species: S. boliviensis, S. orstedii (red-backed squirrel monkey), S. sciureus (common squirrel monkey), and S. ustus. They inhabit tropical rain forests in Central and South America. S. sciureus is used extensively in research studies.'),
  'D012454': NtO(MH='Salamandra', MS='A genus of European newts in the Salamandridae family. The two species of this genus are Salamandra salamandra (European "fire" salamander) and Salamandra atra (European alpine salamander).'),
  'D012455': NtO(MH='Salamandridae', MS='A family of Urodela consisting of 15 living genera and about 42 species and occurring in North America, Europe, Asia, and North Africa.'),
  'D012456': NtO(MH='Salaries and Fringe Benefits', MS='The remuneration paid or benefits granted to an employee.'),
  'D012457': NtO(MH='Salicylamides', MS='Amides of salicylic acid.'),
  'D012458': NtO(MH='Salicylanilides', MS='2-Hydroxy-N-phenylbenzamides. N-phenyl substituted salicylamides. Derivatives have been used as fungicides, anti-mildew agents and topical antifungal agents. In concentrated form may cause irritation of skin and mucous membranes.'),
  'D012459': NtO(MH='Salicylates', MS='The salts or esters of salicylic acids, or salicylate esters of an organic acid. Some of these have analgesic, antipyretic, and anti-inflammatory activities by inhibiting prostaglandin synthesis.'),
  'D012460': NtO(MH='Sulfasalazine', MS='A drug that is used in the management of inflammatory bowel diseases. Its activity is generally considered to lie in its metabolic breakdown product, 5-aminosalicylic acid (see MESALAMINE) released in the colon. (From Martindale, The Extra Pharmacopoeia, 30th ed, p907)'),
  'D012462': NtO(MH='Saline Solution, Hypertonic', MS='Hypertonic sodium chloride solution. A solution having an osmotic pressure greater than that of physiologic salt solution (0.9 g NaCl in 100 ml purified water).'),
  'D012463': NtO(MH='Saliva', MS='The clear, viscous fluid secreted by the SALIVARY GLANDS and mucous glands of the mouth. It contains MUCINS, water, organic salts, and ptylin.'),
  'D012464': NtO(MH='Saliva, Artificial', MS='A solution used for irrigating the mouth in xerostomia and as a substitute for saliva.'),
  'D012465': NtO(MH='Salivary Duct Calculi', MS='Presence of small calculi in the terminal salivary ducts (salivary sand), or stones (larger calculi) found in the larger ducts.'),
  'D012466': NtO(MH='Salivary Gland Diseases', MS='Diseases involving the SALIVARY GLANDS.'),
  'D012467': NtO(MH='Salivary Gland Fistula', MS='A fistula between a salivary duct or gland and the cutaneous surface of the oral cavity.'),
  'D012468': NtO(MH='Salivary Gland Neoplasms', MS='Tumors or cancer of the SALIVARY GLANDS.'),
  'D012469': NtO(MH='Salivary Glands', MS='Glands that secrete SALIVA in the MOUTH. There are three pairs of salivary glands (PAROTID GLAND; SUBLINGUAL GLAND; SUBMANDIBULAR GLAND).'),
  'D012470': NtO(MH='Salivary Glands, Minor', MS='Accessory salivary glands located in the lip, cheek, tongue, floor of mouth, palate and intramaxillary.'),
  'D012471': NtO(MH='Salivary Proteins and Peptides', MS='Proteins and peptides found in SALIVA and the SALIVARY GLANDS. Some salivary proteins such as ALPHA-AMYLASES are enzymes, but their composition varies in different individuals.'),
  'D012472': NtO(MH='Salivation', MS='The discharge of saliva from the SALIVARY GLANDS that keeps the mouth tissues moist and aids in digestion.'),
  'D012473': NtO(MH='Salmine', MS=None),
  'D012474': NtO(MH='Salmon', MS='Fish of the genera ONCORHYNCHUS and Salmo in the family SALMONIDAE. They are anadromous game fish, frequenting the coastal waters of both the North Atlantic and Pacific. They are known for their gameness as a sport fish and for the quality of their flesh as a table fish. (Webster, 3d ed).'),
  'D012475': NtO(MH='Salmonella', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that utilizes citrate as a sole carbon source. It is pathogenic for humans, causing enteric fevers, gastroenteritis, and bacteremia. Food poisoning is the most common clinical manifestation. Organisms within this genus are separated on the basis of antigenic characteristics, sugar fermentation patterns, and bacteriophage susceptibility.'),
  'D012476': NtO(MH='Salmonella arizonae', MS='Gram-negative rods widely distributed in LIZARDS and SNAKES, and implicated in enteric, bone (BONE DISEASES), and joint infections (JOINT DISEASES) in humans.'),
  'D012477': NtO(MH='Salmonella enteritidis', MS='A serotype of Salmonella enterica which is an etiologic agent of gastroenteritis in man and other animals.'),
  'D012478': NtO(MH='Salmonella Food Poisoning', MS='Poisoning caused by ingestion of food harboring species of SALMONELLA. Conditions of raising, shipping, slaughtering, and marketing of domestic animals contribute to the spread of this bacterium in the food supply.'),
  'D012479': NtO(MH='Salmonella paratyphi C', MS='A serotype of SALMONELLA ENTERICA which is an agent of PARATYPHOID FEVER in Asia, Africa, and southern Europe.'),
  'D012480': NtO(MH='Salmonella Infections', MS='Infections with bacteria of the genus SALMONELLA.'),
  'D012481': NtO(MH='Salmonella Infections, Animal', MS='Infections in animals with bacteria of the genus SALMONELLA.'),
  'D012482': NtO(MH='Salmonella paratyphi A', MS='A serotype of SALMONELLA ENTERICA that causes mild PARATYPHOID FEVER in humans.'),
  'D012483': NtO(MH='Salmonella Phages', MS='Viruses whose host is Salmonella. A frequently encountered Salmonella phage is BACTERIOPHAGE P22.'),
  'D012484': NtO(MH='Salmonella paratyphi B', MS='A serotype of SALMONELLA ENTERICA which is an agent of PARATYPHOID FEVER in humans.'),
  'D012485': NtO(MH='Salmonella typhi', MS='A serotype of SALMONELLA ENTERICA which is the etiologic agent of TYPHOID FEVER.'),
  'D012486': NtO(MH='Salmonella typhimurium', MS='A serotype of Salmonella enterica that is a frequent agent of Salmonella gastroenteritis in humans. It also causes PARATYPHOID FEVER.'),
  'D012487': NtO(MH='Salmonidae', MS='A family of anadromous fish comprising SALMON; TROUT; whitefish; and graylings. They are the most important food and game fishes. Their habitat is the northern Atlantic and Pacific, both marine and inland, and the Great Lakes. (Nelson: Fishes of the World, 1976, p97)'),
  'D012488': NtO(MH='Salpingitis', MS='Inflammation of the uterine salpinx, the trumpet-shaped FALLOPIAN TUBES, usually caused by ascending infections of organisms from the lower reproductive tract. Salpingitis can lead to tubal scarring, hydrosalpinx, tubal occlusion, INFERTILITY, and ectopic pregnancy (PREGNANCY, ECTOPIC)'),
  'D012489': NtO(MH='Salpingostomy', MS='Formation of an artificial opening in a fallopian tube.'),
  'D012490': NtO(MH='Salsoline Alkaloids', MS='Tetrahydroisoquinolinol alkaloids in both dextro and levo forms, originally found in SALSOLA plants.'),
  'D012491': NtO(MH='Salt Gland', MS='A compound tubular gland, located around the eyes and nasal passages in marine animals and birds, the physiology of which figures in water-electrolyte balance. The Pekin duck serves as a common research animal in salt gland studies. A rectal gland or rectal salt gland in the dogfish shark is attached at the junction of the intestine and cloaca and aids the kidneys in removing excess salts from the blood. (Storer, Usinger, Stebbins & Nybakken: General Zoology, 6th ed, p658)'),
  'D012492': NtO(MH='Salts', MS="Substances produced from the reaction between acids and bases; compounds consisting of a metal (positive) and nonmetal (negative) radical. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D012493': NtO(MH='Samarium', MS='An element of the rare earth family of metals. It has the atomic symbol Sm, atomic number 62, and atomic weight 150.36. The oxide is used in the control rods of some nuclear reactors.'),
  'D012494': NtO(MH='Sampling Studies', MS='Studies in which a number of subjects are selected from all subjects in a defined population. Conclusions based on sample results may be attributed only to the population sampled.'),
  'D012495': NtO(MH='San Francisco', MS='A city in northern California.'),
  'D012496': NtO(MH='San Marino', MS='An enclave in central Italy.'),
  'D012497': NtO(MH='Sandhoff Disease', MS='An autosomal recessive neurodegenerative disorder characterized by an accumulation of G(M2) GANGLIOSIDE in neurons and other tissues. It is caused by mutation in the common beta subunit of HEXOSAMINIDASE A and HEXOSAMINIDASE B. Thus this disease is also known as the O variant since both hexosaminidase A and B are missing. Clinically, it is indistinguishable from TAY-SACHS DISEASE.'),
  'D012498': NtO(MH='Sanitary Engineering', MS='A branch of engineering concerned with the design, construction, and maintenance of environmental facilities conducive to public health, such as water supply and waste disposal.'),
  'D012499': NtO(MH='Sanitation', MS='The development and establishment of environmental conditions favorable to the health of the public.'),
  'D012500': NtO(MH='Santonin', MS='Anthelmintic isolated from the dried unexpanded flower heads of Artemisia maritima and other species of Artemisia found principally in Russian and Chinese Turkestan and the Southern Ural region. (From Merck, 11th ed.)'),
  'D012501': NtO(MH='Saphenous Vein', MS='The vein which drains the foot and leg.'),
  'D012502': NtO(MH='Sapogenins', MS='The aglucon moiety of a saponin molecule. It may be triterpenoid or steroid, usually spirostan, in nature.'),
  'D012503': NtO(MH='Saponins', MS='A type of glycoside widely distributed in plants. Each consists of a sapogenin as the aglycone moiety, and a sugar. The sapogenin may be a steroid or a triterpene and the sugar may be glucose, galactose, a pentose, or a methylpentose.'),
  'D012504': NtO(MH='Saralasin', MS='An octapeptide analog of angiotensin II (bovine) with amino acids 1 and 8 replaced with sarcosine and alanine, respectively. It is a highly specific competitive inhibitor of angiotensin II that is used in the diagnosis of HYPERTENSION.'),
  'D012505': NtO(MH='Sarcina', MS='A genus of gram-positive, anaerobic bacteria whose organisms divide in three perpendicular planes and occur in packets of eight or more cells. It has been isolated from soil, grains, and clinical specimens.'),
  'D012507': NtO(MH='Sarcoidosis', MS='An idiopathic systemic inflammatory granulomatous disorder comprised of epithelioid and multinucleated giant cells with little necrosis. It usually invades the lungs with fibrosis and may also involve lymph nodes, skin, liver, spleen, eyes, phalangeal bones, and parotid glands.'),
  'D012508': NtO(MH='Sarcolemma', MS='The excitable plasma membrane of a muscle cell. (Glick, Glossary of Biochemistry and Molecular Biology, 1990)'),
  'D012509': NtO(MH='Sarcoma', MS='A connective tissue neoplasm formed by proliferation of mesodermal cells; it is usually highly malignant.'),
  'D012510': NtO(MH='Sarcoma 180', MS='An experimental sarcoma of mice.'),
  'D012511': NtO(MH='Sarcoma 37', MS='An experimental sarcoma of mice.'),
  'D012512': NtO(MH='Sarcoma, Ewing', MS='A malignant tumor of the bone which always arises in the medullary tissue, occurring more often in cylindrical bones. The tumor occurs usually before the age of 20, about twice as frequently in males as in females.'),
  'D012513': NtO(MH='Sarcoma, Experimental', MS='Experimentally induced neoplasms of CONNECTIVE TISSUE in animals to provide a model for studying human SARCOMA.'),
  'D012514': NtO(MH='Sarcoma, Kaposi', MS="A multicentric, malignant neoplastic vascular proliferation characterized by the development of bluish-red cutaneous nodules, usually on the lower extremities, most often on the toes or feet, and slowly increasing in size and number and spreading to more proximal areas. The tumors have endothelium-lined channels and vascular spaces admixed with variably sized aggregates of spindle-shaped cells, and often remain confined to the skin and subcutaneous tissue, but widespread visceral involvement may occur. Kaposi's sarcoma occurs spontaneously in Jewish and Italian males in Europe and the United States. An aggressive variant in young children is endemic in some areas of Africa. A third form occurs in about 0.04% of kidney transplant patients. There is also a high incidence in AIDS patients. (From Dorland, 27th ed & Holland et al., Cancer Medicine, 3d ed, pp2105-7) HHV-8 is the suspected cause."),
  'D012515': NtO(MH='Mast-Cell Sarcoma', MS='A unifocal malignant tumor that consists of atypical pathological MAST CELLS without systemic involvement. It causes local destructive growth in organs other than in skin or bone marrow.'),
  'D012516': NtO(MH='Osteosarcoma', MS='A sarcoma originating in bone-forming cells, affecting the ends of long bones. It is the most common and most malignant of sarcomas of the bones, and occurs chiefly among 10- to 25-year-old youths. (From Stedman, 25th ed)'),
  'D012517': NtO(MH='Sarcoma, Yoshida', MS='An experimental sarcoma of rats.'),
  'D012518': NtO(MH='Sarcomeres', MS='The repeating contractile units of the MYOFIBRIL, delimited by Z bands along its length.'),
  'D012519': NtO(MH='Sarcoplasmic Reticulum', MS='A network of tubules and sacs in the cytoplasm of SKELETAL MUSCLE FIBERS that assist with muscle contraction and relaxation by releasing and storing calcium ions.'),
  'D012520': NtO(MH='Sarcoptes scabiei', MS='A species of mite that causes SCABIES in humans and sarcoptic mange in other animals. Specific variants of S. scabiei exist for humans and animals, but many have the ability to cross species and cause disease.'),
  'D012521': NtO(MH='Sarcosine', MS='An amino acid intermediate in the metabolism of choline.'),
  'D012522': NtO(MH='Sarcocystis', MS='A genus of protozoa found in reptiles, birds, and mammals, including humans. This heteroxenous parasite produces muscle cysts in intermediate hosts such as domestic herbivores (cattle, sheep, pigs) and rodents. Final hosts are predators such as dogs, cats, and man.'),
  'D012523': NtO(MH='Sarcocystosis', MS='Infection of the striated muscle of mammals by parasites of the genus SARCOCYSTIS. Disease symptoms such as vomiting, diarrhea, muscle weakness, and paralysis are produced by sarcocystin, a toxin produced by the organism.'),
  'D012524': NtO(MH='Sarin', MS='An organophosphorus ester compound that produces potent and irreversible inhibition of cholinesterase. It is toxic to the nervous system and is a chemical warfare agent.'),
  'D012525': NtO(MH='Saskatchewan', MS="A province of Canada, lying between the provinces of Alberta and Manitoba. Its capital is Regina. It is entirely a plains region with prairie in the south and wooded country with many lakes and swamps in the north. The name was taken from the Saskatchewan River from the Cree name Kisiskatchewani Sipi, meaning rapid-flowing river. (From Webster's New Geographical Dictionary, 1988, p1083 & Room, Brewer's Dictionary of Names, 1992, p486)"),
  'D012526': NtO(MH='Satellite Viruses', MS='Defective viruses which can multiply only by association with a helper virus which complements the defective gene. Satellite viruses may be associated with certain plant viruses, animal viruses, or bacteriophages. They differ from satellite RNA; (RNA, SATELLITE) in that satellite viruses encode their own coat protein.'),
  'D012527': NtO(MH='Satiation', MS='Full gratification of a need or desire followed by a state of relative insensitivity to that particular need or desire.'),
  'D012528': NtO(MH='Satiety Response', MS='Behavioral response associated with the achieving of gratification.'),
  'D012529': NtO(MH='Saudi Arabia', MS='A country located in the Middle East, bordering the Persian Gulf and the Red Sea, north of Yemen. The capital is Riyadh.'),
  'D012530': NtO(MH='Saxitoxin', MS='A compound that contains a reduced purine ring system but is not biosynthetically related to the purine alkaloids. It is a poison found in certain edible mollusks at certain times; elaborated by GONYAULAX and consumed by mollusks, fishes, etc. without ill effects. It is neurotoxic and causes RESPIRATORY PARALYSIS and other effects in MAMMALS, known as paralytic SHELLFISH poisoning.'),
  'D012531': NtO(MH='Dibenz(b,f)(1,4)oxazepine-10(11H)-carboxylic acid, 8-chloro-, 2-acetylhydrazide', MS='Inhibits the activity of prostaglandins.'),
  'D012532': NtO(MH='Scabies', MS='A contagious cutaneous inflammation caused by the bite of the mite SARCOPTES SCABIEI. It is characterized by pruritic papular eruptions and burrows and affects primarily the axillae, elbows, wrists, and genitalia, although it can spread to cover the entire body.'),
  'D012533': NtO(MH='Scala Tympani', MS='The lower chamber of the COCHLEA, extending from the round window to the helicotrema (the opening at the apex that connects the PERILYMPH-filled spaces of scala tympani and SCALA VESTIBULI).'),
  'D012534': NtO(MH='Dental Scaling', MS='Removal of dental plaque and dental calculus from the surface of a tooth, from the surface of a tooth apical to the gingival margin accumulated in periodontal pockets, or from the surface coronal to the gingival margin.'),
  'D012535': NtO(MH='Scalp', MS='The outer covering of the calvaria. It is composed of several layers: SKIN; subcutaneous connective tissue; the occipitofrontal muscle which includes the tendinous galea aponeurotica; loose connective tissue; and the pericranium (the PERIOSTEUM of the SKULL).'),
  'D012536': NtO(MH='Scalp Dermatoses', MS='Skin diseases involving the SCALP.'),
  'D012537': NtO(MH='Scandinavian and Nordic Countries', MS='NORWAY; SWEDEN; DENMARK; ICELAND; and FINLAND.'),
  'D012538': NtO(MH='Scandium', MS='An element of the rare earth family of metals. It has the atomic symbol Sc, atomic number 21, and atomic weight 45.'),
  'D012539': NtO(MH='Scapegoating', MS='Process in which the mechanisms of projection or displacement are utilized in focusing feelings of aggression, hostility, frustration, etc., upon another individual or group; the amount of blame being unwarranted.'),
  'D012540': NtO(MH='Scapula', MS='Also called the shoulder blade, it is a flat triangular bone, a pair of which form the back part of the shoulder girdle.'),
  'D012541': NtO(MH='Scarlet Fever', MS='Infection with group A streptococci that is characterized by tonsillitis and pharyngitis. An erythematous rash is commonly present.'),
  'D012542': NtO(MH='Scattering, Radiation', MS='The diversion of RADIATION (thermal, electromagnetic, or nuclear) from its original path as a result of interactions or collisions with atoms, molecules, or larger particles in the atmosphere or other media. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D012543': NtO(MH='Scent Glands', MS='Exocrine glands in animals which secrete scents which either repel or attract other animals, e.g. perianal glands of skunks, anal glands of weasels, musk glands of foxes, ventral glands of wood rats, and dorsal glands of peccaries.'),
  'D012544': NtO(MH='Scheuermann Disease', MS='A type of juvenile osteochondrosis affecting the fibrocartilaginous disc (INTERVERTEBRAL DISC) in the thoracic or thoracolumbar region of the SPINE. It is characterized by a forward concave SPINAL CURVATURE or KYPHOSIS.'),
  'D012545': NtO(MH='Schiff Bases', MS="Condensation products of aromatic amines and aldehydes forming azomethines substituted on the N atom, containing the general formula R-N:CHR. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D012546': NtO(MH='Schilling Test', MS='A diagnostic test in which vitamin B12 is tagged with radioactive cobalt, taken orally, and gastrointestinal absorption is determined via measurement of the amount of radioactivity in a 24-hour urine collection.'),
  'D012547': NtO(MH='Schistosoma', MS='A genus of trematode flukes belonging to the family Schistosomatidae. There are over a dozen species. These parasites are found in man and other mammals. Snails are the intermediate hosts.'),
  'D012548': NtO(MH='Schistosoma haematobium', MS='A species of trematode blood flukes of the family Schistosomatidae which occurs at different stages in development in veins of the pulmonary and hepatic system and finally the bladder lumen. This parasite causes urinary schistosomiasis.'),
  'D012549': NtO(MH='Schistosoma japonicum', MS='A species of trematode blood flukes belonging to the family Schistosomatidae whose distribution is confined to areas of the Far East. The intermediate host is a snail. It occurs in man and other mammals.'),
  'D012550': NtO(MH='Schistosoma mansoni', MS='A species of trematode blood flukes of the family Schistosomatidae. It is common in the Nile delta. The intermediate host is the planorbid snail. This parasite causes schistosomiasis mansoni and intestinal bilharziasis.'),
  'D012551': NtO(MH='Schistosomatidae', MS='A family of blood flukes of the class Trematoda which is found in animals and man. It Includes the genera Heterobilharzia, Schistosomatium, Schistosoma, Ornithobilharzia, Bilharziella, Trichobilharzia, Pseudobilharzia, and Austrobilharzia.'),
  'D012552': NtO(MH='Schistosomiasis', MS='Infection with flukes (trematodes) of the genus SCHISTOSOMA. Three species produce the most frequent clinical diseases: SCHISTOSOMA HAEMATOBIUM (endemic in Africa and the Middle East), SCHISTOSOMA MANSONI (in Egypt, northern and southern Africa, some West Indies islands, northern 2/3 of South America), and SCHISTOSOMA JAPONICUM (in Japan, China, the Philippines, Celebes, Thailand, Laos). S. mansoni is often seen in Puerto Ricans living in the United States.'),
  'D012553': NtO(MH='Schistosomiasis haematobia', MS='A human disease caused by the infection of parasitic worms SCHISTOSOMA HAEMATOBIUM. It is endemic in AFRICA and parts of the MIDDLE EAST. Tissue damages most often occur in the URINARY TRACT, specifically the URINARY BLADDER.'),
  'D012554': NtO(MH='Schistosomiasis japonica', MS='Schistosomiasis caused by Schistosoma japonicum. It is endemic in the Far East and affects the bowel, liver, and spleen.'),
  'D012555': NtO(MH='Schistosomiasis mansoni', MS='Schistosomiasis caused by Schistosoma mansoni. It is endemic in Africa, the Middle East, South America, and the Caribbean and affects mainly the bowel, spleen, and liver.'),
  'D012556': NtO(MH='Schistosomicides', MS='Agents that act systemically to kill adult schistosomes.'),
  'D012557': NtO(MH='Schizoid Personality Disorder', MS='A personality disorder manifested by a profound defect in the ability to form social relationships, no desire for social involvement, and an indifference to praise or criticism.'),
  'D012559': NtO(MH='Schizophrenia', MS='A severe emotional disorder of psychotic depth characteristically marked by a retreat from reality with delusion formation, HALLUCINATIONS, emotional disharmony, and regressive behavior.'),
  'D012560': NtO(MH='Schizophrenia, Catatonic', MS='A type of schizophrenia characterized by abnormality of motor behavior which may involve particular forms of stupor, rigidity, excitement or inappropriate posture.'),
  'D012561': NtO(MH='Schizophrenia, Childhood', MS='An obsolete concept, historically used for childhood mental disorders thought to be a form of schizophrenia. It was in earlier versions of DSM but is now included within the broad concept of PERVASIVE DEVELOPMENT DISORDERS.'),
  'D012562': NtO(MH='Schizophrenia, Disorganized', MS='A type of schizophrenia characterized by frequent incoherence; marked loosening of associations, or grossly disorganized behavior and flat or grossly inappropriate affect that does not meet the criteria for the catatonic type; associated features include extreme social withdrawal, grimacing, mannerisms, mirror gazing, inappropriate giggling, and other odd behavior. (Dorland, 27th ed)'),
  'D012563': NtO(MH='Schizophrenia, Paranoid', MS='A chronic form of schizophrenia characterized primarily by the presence of persecutory or grandiose delusions, often associated with hallucination.'),
  'D012564': NtO(MH='Schizophrenic Language', MS="The artificial language of schizophrenic patients - neologisms (words of the patient's own making with new meanings)."),
  'D012565': NtO(MH='Schizophrenic Psychology', MS='Study of mental processes and behavior of schizophrenics.'),
  'D012566': NtO(MH='Sizofiran', MS='A beta-D-glucan obtained from the Aphyllophoral fungus Schizophyllum commune. It is used as an immunoadjuvant in the treatment of neoplasms, especially tumors found in the stomach.'),
  'D012567': NtO(MH='Schizophyllum', MS='A genus of fleshy shelf basidiomycetous fungi, family Schizophyllaceae, order POLYPORALES, growing on woody substrata. It is pathogenic in humans.'),
  'D012568': NtO(MH='Schizosaccharomyces', MS='A genus of ascomycetous fungi of the family Schizosaccharomycetaceae, order Schizosaccharomycetales.'),
  'D012569': NtO(MH='Schizotypal Personality Disorder', MS='A personality disorder in which there are oddities of thought (magical thinking, paranoid ideation, suspiciousness), perception (illusions, depersonalization), speech (digressive, vague, overelaborate), and behavior (inappropriate affect in social interactions, frequently social isolation) that are not severe enough to characterize schizophrenia.'),
  'D012570': NtO(MH='School Admission Criteria', MS='Requirements for the selection of students for admission to academic institutions.'),
  'D012571': NtO(MH='School Dentistry', MS='Preventive dental services provided for students in primary and secondary schools.'),
  'D012572': NtO(MH='School Health Services', MS='Preventive health services provided for students. It excludes college or university students.'),
  'D012573': NtO(MH='School Nursing', MS='A nursing specialty concerned with health and nursing care given to primary and secondary school students by a registered nurse.'),
  'D012574': NtO(MH='Schools', MS='Educational institutions.'),
  'D012575': NtO(MH='Schools, Dental', MS='Educational institutions for individuals specializing in the field of dentistry.'),
  'D012576': NtO(MH='Schools, Health Occupations', MS='Schools which offer training in the area of health.'),
  'D012577': NtO(MH='Schools, Medical', MS='Educational institutions for individuals specializing in the field of medicine.'),
  'D012578': NtO(MH='Schools, Nursery', MS='Schools for children usually under five years of age.'),
  'D012579': NtO(MH='Schools, Nursing', MS='Educational institutions for individuals specializing in the field of nursing.'),
  'D012580': NtO(MH='Schools, Pharmacy', MS='Educational institutions for individuals specializing in the field of pharmacy.'),
  'D012581': NtO(MH='Schools, Public Health', MS='Educational institutions for individuals specializing in the field of public health.'),
  'D012582': NtO(MH='Schools, Veterinary', MS='Educational institutions for individuals specializing in the field of veterinary medicine.'),
  'D012583': NtO(MH='Schwann Cells', MS='Neuroglial cells of the peripheral nervous system which form the insulating myelin sheaths of peripheral axons.'),
  'D012584': NtO(MH='Sciatic Nerve', MS='A nerve which originates in the lumbar and sacral spinal cord (L4 to S3) and supplies motor and sensory innervation to the lower extremity. The sciatic nerve, which is the main continuation of the sacral plexus, is the largest nerve in the body. It has two major branches, the TIBIAL NERVE and the PERONEAL NERVE.'),
  'D012585': NtO(MH='Sciatica', MS='A condition characterized by pain radiating from the back into the buttock and posterior/lateral aspects of the leg. Sciatica may be a manifestation of SCIATIC NEUROPATHY; RADICULOPATHY (involving the SPINAL NERVE ROOTS; L4, L5, S1, or S2, often associated with INTERVERTEBRAL DISK DISPLACEMENT); or lesions of the CAUDA EQUINA.'),
  'D012586': NtO(MH='Science', MS='The study of natural phenomena by observation, measurement, and experimentation.'),
  'D012587': NtO(MH='Scimitar Syndrome', MS='An anomalous pulmonary venous return in which the right PULMONARY VEIN is not connected to the LEFT ATRIUM but to the INFERIOR VENA CAVA. Scimitar syndrome is named for the crescent- or Turkish sword-like shadow in the chest radiography and is often associated with hypoplasia of the right lung and right pulmonary artery, and dextroposition of the heart.'),
  'D012588': NtO(MH='Scintillation Counting', MS='Detection and counting of scintillations produced in a fluorescent material by ionizing radiation.'),
  'D012589': NtO(MH='Sciuridae', MS='A family of the order Rodentia which contains 49 genera. Some of the more common genera are MARMOTA, which includes the marmot and woodchuck; Sciurus, the gray squirrel, S. carolinensis, and the fox squirrel, S. niger; Tamias, the eastern and western chipmunk; and Tamiasciurus, the red squirrel. The flying squirrels, except the scaly-tailed Anomaluridae, also belong to this family.'),
  'D012590': NtO(MH='Sclera', MS='The white, opaque, fibrous, outer tunic of the eyeball, covering it entirely excepting the segment covered anteriorly by the cornea. It is essentially avascular but contains apertures for vessels, lymphatics, and nerves. It receives the tendons of insertion of the extraocular muscles and at the corneoscleral junction contains the canal of Schlemm. (From Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D012591': NtO(MH='Scleral Buckling', MS='An operation for retinal detachment which reduces the size of the globe by indenting the sclera so that it approximates the retina.'),
  'D012592': NtO(MH='Scleredema Adultorum', MS='A diffuse, non-pitting induration of the skin of unknown etiology that occurs most commonly in association with diabetes mellitus, predominantly in females. It typically begins on the face or head and spreads to other areas of the body, sometimes involving noncutaneous tissues. Often it is preceded by any of various infections, notably staphylococcal infections. The condition resolves spontaneously, usually within two years of onset. (From Dorland, 27th ed)'),
  'D012593': NtO(MH='Sclerema Neonatorum', MS='A severe, sometimes fatal, disorder of adipose tissue occurring chiefly in preterm or debilitated infants suffering from an underlying illness and manifested by a diffuse, nonpitting induration of the affected tissue. The skin becomes cold, yellowish, mottled, and inflexible.'),
  'D012594': NtO(MH='Scleroderma, Localized', MS='A term used to describe a variety of localized asymmetrical SKIN thickening that is similar to those of SYSTEMIC SCLERODERMA but without the disease features in the multiple internal organs and BLOOD VESSELS. Lesions may be characterized as patches or plaques (morphea), bands (linear), or nodules.'),
  'D012595': NtO(MH='Scleroderma, Systemic', MS='A chronic multi-system disorder of CONNECTIVE TISSUE. It is characterized by SCLEROSIS in the SKIN, the LUNGS, the HEART, the GASTROINTESTINAL TRACT, the KIDNEYS, and the MUSCULOSKELETAL SYSTEM. Other important features include diseased small BLOOD VESSELS and AUTOANTIBODIES. The disorder is named for its most prominent feature (hard skin), and classified into subsets by the extent of skin thickening: LIMITED SCLERODERMA and DIFFUSE SCLERODERMA.'),
  'D012596': NtO(MH='Scleroproteins', MS='Simple proteins characterized by their insolubility and fibrous structure. Within the body, they perform a supportive or protective function.'),
  'D012597': NtO(MH='Sclerosing Solutions', MS='Chemical agents injected into blood vessels and lymphatic sinuses to shrink or cause localized THROMBOSIS; FIBROSIS, and obliteration of the vessels. This treatment is applied in a number of conditions such as VARICOSE VEINS; HEMORRHOIDS; GASTRIC VARICES; ESOPHAGEAL VARICES; PEPTIC ULCER HEMORRHAGE.'),
  'D012598': NtO(MH='Sclerosis', MS='A pathological process consisting of hardening or fibrosis of an anatomical structure, often a vessel or a nerve.'),
  'D012599': NtO(MH='Sclerostomy', MS='Surgical formation of an external opening in the sclera, primarily in the treatment of glaucoma.'),
  'D012600': NtO(MH='Scoliosis', MS='An appreciable lateral deviation in the normally straight vertical line of the spine. (Dorland, 27th ed)'),
  'D012601': NtO(MH='Scopolamine', MS='An alkaloid from SOLANACEAE, especially DATURA and SCOPOLIA. Scopolamine and its quaternary derivatives act as antimuscarinics like ATROPINE, but may have more central nervous system effects. Its many uses include an anesthetic premedication, the treatment of URINARY INCONTINENCE and MOTION SICKNESS, an antispasmodic, and a mydriatic and cycloplegic.'),
  'D012602': NtO(MH='Scopolamine Derivatives', MS='Analogs or derivatives of scopolamine.'),
  'D012603': NtO(MH='Scopoletin', MS='Plant growth factor derived from the root of Scopolia carniolica or Scopolia japonica.'),
  'D012604': NtO(MH='Scorpion Venoms', MS='Venoms from animals of the order Scorpionida of the class Arachnida. They contain neuro- and hemotoxins, enzymes, and various other factors that may release acetylcholine and catecholamines from nerve endings. Of the several protein toxins that have been characterized, most are immunogenic.'),
  'D012605': NtO(MH='Scorpions', MS='Arthropods of the order Scorpiones, of which 1500 to 2000 species have been described. The most common live in tropical or subtropical areas. They are nocturnal and feed principally on insects and other arthropods. They are large arachnids but do not attack man spontaneously. They have a venomous sting. Their medical significance varies considerably and is dependent on their habits and venom potency rather than on their size. At most, the sting is equivalent to that of a hornet but certain species possess a highly toxic venom potentially fatal to humans. (From Dorland, 27th ed; Smith, Insects and Other Arthropods of Medical Importance, 1973, p417; Barnes, Invertebrate Zoology, 5th ed, p503)'),
  'D012606': NtO(MH='Scotland', MS='The most northerly of the four countries of the United Kingdom, occupying about one-third of the island of Great Britain. The capital is Edinburgh.'),
  'D012607': NtO(MH='Scotoma', MS='A localized defect in the visual field bordered by an area of normal vision. This occurs with a variety of EYE DISEASES (e.g., RETINAL DISEASES and GLAUCOMA); OPTIC NERVE DISEASES, and other conditions.'),
  'D012608': NtO(MH='Scrapie', MS='A fatal disease of the nervous system in sheep and goats, characterized by pruritus, debility, and locomotor incoordination. It is caused by proteinaceous infectious particles called PRIONS.'),
  'D012610': NtO(MH='Screw Worm Infection', MS='Infection with larvae of the blow fly Cochliomyia hominivorax (Callitroga americanum), a common cause of disease in livestock in the southern and southwestern U.S.A.'),
  'D012611': NtO(MH='Scrotum', MS='A cutaneous pouch of skin containing the testicles and spermatic cords.'),
  'D012612': NtO(MH='Scrub Typhus', MS='An acute infectious disease caused by ORIENTIA TSUTSUGAMUSHI. It is limited to eastern and southeastern Asia, India, northern Australia, and the adjacent islands. Characteristics include the formation of a primary cutaneous lesion at the site of the bite of an infected mite, fever lasting about two weeks, and a maculopapular rash.'),
  'D012613': NtO(MH='Sculpture', MS=None),
  'D012614': NtO(MH='Scurvy', MS='An acquired blood vessel disorder caused by severe deficiency of vitamin C (ASCORBIC ACID) in the diet leading to defective collagen formation in small blood vessels. Scurvy is characterized by bleeding in any tissue, weakness, ANEMIA, spongy gums, and a brawny induration of the muscles of the calves and legs.'),
  'D012615': NtO(MH='Sea Anemones', MS='The order Actiniaria, in the class ANTHOZOA, comprised of large, solitary polyps. All species are carnivorous.'),
  'D012616': NtO(MH='Sea Cucumbers', MS='A class of Echinodermata characterized by long, slender bodies.'),
  'D012617': NtO(MH='Sea Urchins', MS='Somewhat flattened, globular echinoderms, having thin, brittle shells of calcareous plates. They are useful models for studying FERTILIZATION and EMBRYO DEVELOPMENT.'),
  'D012618': NtO(MH='Sea-Blue Histiocyte Syndrome', MS='A congenital disease caused by an inborn error involving APOLIPOPROTEINS E leading to abnormal LIPID METABOLISM and the accumulation of GLYCOSPHINGOLIPIDS, particularly SPHINGOMYELINS in the HISTIOCYTES. This disorder is characterized by SPLENOMEGALY and the sea-blue histiocytes in the spleen and bone marrow after May Grunwald staining.'),
  'D012621': NtO(MH='Seasons', MS='Divisions of the year according to some regularly recurrent phenomena usually astronomical or climatic. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D012622': NtO(MH='Seat Belts', MS='Restraining belts fastened to the frame of automobiles, aircraft, or other vehicles, and strapped around the person occupying the seat in the car or plane, intended to prevent the person from being thrown forward or out of the vehicle in case of sudden deceleration.'),
  'D012623': NtO(MH='Seawater', MS='The salinated water of OCEANS AND SEAS that provides habitat for marine organisms.'),
  'D012624': NtO(MH='Seaweed', MS='Multicellular marine macroalgae including some members of red (RHODOPHYTA), green (CHLOROPHYTA), and brown (PHAEOPHYTA) algae. They are widely distributed in the ocean, occurring from the tide level to considerable depths, free-floating (planktonic) or anchored to the substratum (benthic). They lack a specialized vascular system but take up fluids, nutrients, and gases directly from the water. They contain CHLOROPHYLL and are photosynthetic, but some also contain other light-absorbing pigments. Many are of economic importance as FOOD, fertilizer, AGAR, potash, or source of IODINE.'),
  'D012625': NtO(MH='Sebaceous Gland Diseases', MS='Diseases of the sebaceous glands such as sebaceous hyperplasia and sebaceous cell carcinoma (SEBACEOUS GLAND NEOPLASMS).'),
  'D012626': NtO(MH='Sebaceous Gland Neoplasms', MS='New abnormal growth of tissue in the SEBACEOUS GLANDS.'),
  'D012627': NtO(MH='Sebaceous Glands', MS='Small, sacculated organs found within the DERMIS. Each gland has a single duct that emerges from a cluster of oval alveoli. Each alveolus consists of a transparent BASEMENT MEMBRANE enclosing epithelial cells. The ducts from most sebaceous glands open into a HAIR FOLLICLE, but some open on the general surface of the SKIN. Sebaceous glands secrete SEBUM.'),
  'D012628': NtO(MH='Dermatitis, Seborrheic', MS='A chronic inflammatory disease of the skin with unknown etiology. It is characterized by moderate ERYTHEMA, dry, moist, or greasy (SEBACEOUS GLAND) scaling and yellow crusted patches on various areas, especially the scalp, that exfoliate as dandruff. Seborrheic dermatitis is common in children and adolescents with HIV INFECTIONS.'),
  'D012629': NtO(MH='Sebum', MS='The oily substance secreted by SEBACEOUS GLANDS. It is composed of KERATIN, fat, and cellular debris.'),
  'D012631': NtO(MH='Secobarbital', MS='A barbiturate that is used as a sedative. Secobarbital is reported to have no anti-anxiety activity.'),
  'D012632': NtO(MH='Secosteroids', MS='Steroids in which fission of one or more ring structures and concomitant addition of a hydrogen atom at each terminal group has occurred.'),
  'D012633': NtO(MH='Secretin', MS='A peptide hormone of about 27 amino acids from the duodenal mucosa that activates pancreatic secretion and lowers the blood sugar level. (USAN and the USP Dictionary of Drug Names, 1994, p597)'),
  'D012634': NtO(MH='Bodily Secretions', MS='Endogenous substances produced through the activity of intact cells of glands, tissues, or organs.'),
  'D012635': NtO(MH='Secretory Component', MS='The extracellular moiety of the POLYMERIC IMMUNOGLOBULIN RECEPTOR found alone or complexed with IGA or IGM, in a variety of external secretions (tears, bile, colostrum.) Secretory component is derived by proteolytic cleavage of the receptor during transcytosis. When immunoglobulins IgA and IgM are bound to the receptor, during their transcytosis secretory component becomes covalently attached to them generating SECRETORY IMMUNOGLOBULIN A or secretory IMMUNOGLOBULIN M.'),
  'D012636': NtO(MH='Secretory Rate', MS='The amount of a substance secreted by cells or by a specific organ or organism over a given period of time; usually applies to those substances which are formed by glandular tissues and are released by them into biological fluids, e.g., secretory rate of corticosteroids by the adrenal cortex, secretory rate of gastric acid by the gastric mucosa.'),
  'D012637': NtO(MH='Security Measures', MS='Regulations to assure protection of property and equipment.'),
  'D012639': NtO(MH='Seeds', MS='The encapsulated embryos of flowering plants. They are used as is or for animal feed because of the high content of concentrated nutrients like starches, proteins, and fats. Rapeseed, cottonseed, and sunflower seed are also produced for the oils (fats) they yield.'),
  'D012640': NtO(MH='Seizures', MS='Clinical or subclinical disturbances of cortical function due to a sudden, abnormal, excessive, and disorganized discharge of brain cells. Clinical manifestations include abnormal motor, sensory and psychic phenomena. Recurrent seizures are usually referred to as EPILEPSY or "seizure disorder."'),
  'D012641': NtO(MH='Selection, Genetic', MS='Differential and non-random reproduction of different genotypes, operating to alter the gene frequencies within a population.'),
  'D012642': NtO(MH='Selegiline', MS='A selective, irreversible inhibitor of Type B monoamine oxidase that is used for the treatment of newly diagnosed patients with PARKINSON DISEASE, and for the treatment of depressive disorders. The compound without isomeric designation is Deprenyl.'),
  'D012643': NtO(MH='Selenium', MS='An element with the atomic symbol Se, atomic number 34, and atomic weight 78.97. It is an essential micronutrient for mammals and other animals but is toxic in large amounts. Selenium protects intracellular structures against oxidative damage. It is an essential component of GLUTATHIONE PEROXIDASE.'),
  'D012644': NtO(MH='Selenium Radioisotopes', MS='Unstable isotopes of selenium that decay or disintegrate emitting radiation. Se atoms with atomic weights 70-73, 75, 79, 81, and 83-85 are radioactive selenium isotopes.'),
  'D012645': NtO(MH='Selenomethionine', MS='Diagnostic aid in pancreas function determination.'),
  'D012646': NtO(MH='Self Administration', MS='Administration of a drug or chemical by the individual under the direction of a physician. It includes administration clinically or experimentally, by human or animal.'),
  'D012647': NtO(MH='Self-Assessment', MS="Appraisal of one's own personal qualities or traits."),
  'D012648': NtO(MH='Self Care', MS='Caring for self when ill or positive actions and adopting behaviors to prevent illness.'),
  'D012649': NtO(MH='Self Concept', MS="Persons' view of themselves."),
  'D012650': NtO(MH='Self Disclosure', MS='A willingness to reveal information about oneself to others.'),
  'D012651': NtO(MH='Self Medication', MS='The self administration of medication not prescribed by a physician or in a manner not directed by a physician.'),
  'D012652': NtO(MH='Self Mutilation', MS="The act of injuring one's own body to the extent of cutting off or permanently destroying a limb or other essential part of a body."),
  'D012653': NtO(MH='Self Stimulation', MS='Stimulation of the brain, which is self-administered. The stimulation may result in negative or positive reinforcement.'),
  'D012654': NtO(MH='Self-Care Units', MS='Rooms in health care facilities for patients who require a minimal level of care.'),
  'D012655': NtO(MH='Self-Evaluation Programs', MS='Educational programs structured in such a manner that the participating professionals, physicians, or students develop an increased awareness of their performance, usually on the basis of self-evaluation questionnaires.'),
  'D012656': NtO(MH='Self-Help Devices', MS='Devices, not affixed to the body, designed to help persons having musculoskeletal or neuromuscular disabilities to perform activities involving movement.'),
  'D012657': NtO(MH='Self-Help Groups', MS='Organizations which provide an environment encouraging social interactions through group activities or individual relationships especially for the purpose of rehabilitating or supporting patients, individuals with common health problems, or the elderly. They include therapeutic social clubs.'),
  'D012658': NtO(MH='Sella Turcica', MS='A bony prominence situated on the upper surface of the body of the sphenoid bone. It houses the PITUITARY GLAND.'),
  'D012659': NtO(MH='Semantic Differential', MS="Analysis of word concepts by the association of polar adjectives, e.g., good-bad, with the concept, father. The adjectives are usually scaled in 7 steps. The subject's placement of the concept on the adjectival scale indicates the connotative meaning of the concept."),
  'D012660': NtO(MH='Semantics', MS='The relationships between symbols and their meanings.'),
  'D012661': NtO(MH='Semen', MS='The thick, yellowish-white, viscid fluid secretion of male reproductive organs discharged upon ejaculation. In addition to reproductive organ secretions, it contains SPERMATOZOA and their nutrient plasma.'),
  'D012662': NtO(MH='Semen Preservation', MS='The process by which semen is kept viable outside of the organism from which it was derived (i.e., kept from decay by means of a chemical agent, cooling, or a fluid substitute that mimics the natural state within the organism).'),
  'D012663': NtO(MH='Semicarbazides', MS='Derivatives of UREA where one of the amino groups is replaced with hydrazine.'),
  'D012664': NtO(MH='Semicarbazones', MS=None),
  'D012665': NtO(MH='Semicircular Canals', MS='Three long canals (anterior, posterior, and lateral) of the bony labyrinth. They are set at right angles to each other and are situated posterosuperior to the vestibule of the bony labyrinth (VESTIBULAR LABYRINTH). The semicircular canals have five openings into the vestibule with one shared by the anterior and the posterior canals. Within the canals are the SEMICIRCULAR DUCTS.'),
  'D012666': NtO(MH='Semiconductors', MS='Materials that have a limited and usually variable electrical conductivity. They are particularly useful for the production of solid-state electronic devices.'),
  'D012667': NtO(MH='Lunate Bone', MS='A moon-shaped carpal bone which is located between the SCAPHOID BONE and TRIQUETRUM BONE.'),
  'D012668': NtO(MH='Trigeminal Ganglion', MS='The semilunar-shaped ganglion containing the cells of origin of most of the sensory fibers of the trigeminal nerve. It is situated within the dural cleft on the cerebral surface of the petrous portion of the temporal bone and gives off the ophthalmic, maxillary, and part of the mandibular nerves.'),
  'D012669': NtO(MH='Seminal Vesicles', MS='A saclike, glandular diverticulum on each ductus deferens in male vertebrates. It is united with the excretory duct and serves for temporary storage of semen. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D012670': NtO(MH='Seminiferous Epithelium', MS='The epithelium lining the seminiferous tubules composed of primary male germ cells (SPERMATOGONIA) and supporting SERTOLI CELLS. As SPERMATOGENESIS proceeds, the developing germ cells migrate toward the lumen. The adluminal compartment, the inner two thirds of the tubules, contains SPERMATOCYTES and the more advanced germ cells.'),
  'D012671': NtO(MH='Seminiferous Tubules', MS='The convoluted tubules in the TESTIS where sperm are produced (SPERMATOGENESIS) and conveyed to the RETE TESTIS. Spermatogenic tubules are composed of developing germ cells and the supporting SERTOLI CELLS.'),
  'D012672': NtO(MH='Semliki forest virus', MS='A species of ALPHAVIRUS isolated in central, eastern, and southern Africa.'),
  'D012673': NtO(MH='Semustine', MS='4-Methyl derivative of LOMUSTINE; (CCNU). An antineoplastic agent which functions as an alkylating agent.'),
  'D012674': NtO(MH='Senecio', MS='A species of toxic plants of the Compositae. The poisonous compounds are alkaloids which cause cattle diseases, neoplasms, and liver damage and are used to produce cancers in experimental animals.'),
  'D012675': NtO(MH='Senegal', MS='A republic in western Africa, southwest of MAURITANIA and east of MALI. Its capital is Dakar.'),
  'D012676': NtO(MH='Senna Extract', MS='Preparations of Cassia senna and C. angustifolia (see SENNA PLANT). They contain sennosides, which are anthraquinone type CATHARTICS and are used in many different preparations as laxatives.'),
  'D012677': NtO(MH='Sensation', MS='The process in which specialized SENSORY RECEPTOR CELLS transduce peripheral stimuli (physical or chemical) into NERVE IMPULSES which are then transmitted to the various sensory centers in the CENTRAL NERVOUS SYSTEM.'),
  'D012678': NtO(MH='Sensation Disorders', MS='Disorders of the special senses (i.e., VISION; HEARING; TASTE; and SMELL) or somatosensory system (i.e., afferent components of the PERIPHERAL NERVOUS SYSTEM).'),
  'D012679': NtO(MH='Sense Organs', MS='Specialized organs adapted for the reception of stimuli by the NERVOUS SYSTEM.'),
  'D012680': NtO(MH='Sensitivity and Specificity', MS='Binary classification measures to assess test results. Sensitivity or recall rate is the proportion of true positives. Specificity is the probability of correctly determining the absence of a condition. (From Last, Dictionary of Epidemiology, 2d ed)'),
  'D012681': NtO(MH='Sensitivity Training Groups', MS='A group of people who meet in an unstructured setting to learn about themselves, interpersonal relationships, and group processes and about larger social systems.'),
  'D012682': NtO(MH='Sensory Aids', MS='Devices that help people with impaired sensory responses.'),
  'D012683': NtO(MH='Sensory Deprivation', MS='The absence or restriction of the usual external sensory stimuli to which the individual responds.'),
  'D012684': NtO(MH='Sensory Thresholds', MS='The minimum amount of stimulus energy necessary to elicit a sensory response.'),
  'D012685': NtO(MH='Sepharose', MS=None),
  'D012686': NtO(MH='Septal Nuclei', MS='Neural nuclei situated in the septal region. They have afferent and cholinergic efferent connections with a variety of FOREBRAIN and BRAIN STEM areas including the HIPPOCAMPAL FORMATION, the LATERAL HYPOTHALAMUS, the tegmentum, and the AMYGDALA. Included are the dorsal, lateral, medial, and triangular septal nuclei, septofimbrial nucleus, nucleus of diagonal band, nucleus of anterior commissure, and the nucleus of stria terminalis.'),
  'D012688': NtO(MH='Septum Pellucidum', MS='A triangular double membrane separating the anterior horns of the LATERAL VENTRICLES of the brain. It is situated in the median plane and bounded by the CORPUS CALLOSUM and the body and columns of the FORNIX (BRAIN).'),
  'D012689': NtO(MH='Sequence Homology, Nucleic Acid', MS='The sequential correspondence of nucleotides in one nucleic acid molecule with those of another nucleic acid molecule. Sequence homology is an indication of the genetic relatedness of different organisms and gene function.'),
  'D012690': NtO(MH='Serial Extraction', MS='The selective extraction of deciduous teeth during the stage of mixed dentition in accordance with the shedding and eruption of the teeth. It is done over an extended period to allow autonomous adjustment to relieve crowding of the dental arches during the eruption of the lateral incisors, canines, and premolars, eventually involving the extraction of the first premolar teeth. (Dorland, 28th ed)'),
  'D012691': NtO(MH='Serial Learning', MS='Learning to make a series of responses in exact order.'),
  'D012692': NtO(MH='Serial Passage', MS='Inoculation of a series of animals or in vitro tissue with an infectious bacterium or virus, as in VIRULENCE studies and the development of vaccines.'),
  'D012693': NtO(MH='Serial Publications', MS='Publications in any medium issued in successive parts bearing numerical or chronological designations and intended to be continued indefinitely. (ALA Glossary of Library and Information Science, 1983, p203)'),
  'D012694': NtO(MH='Serine', MS='A non-essential amino acid occurring in natural form as the L-isomer. It is synthesized from GLYCINE or THREONINE. It is involved in the biosynthesis of PURINES; PYRIMIDINES; and other amino acids.'),
  'D012695': NtO(MH='L-Serine Dehydratase', MS='A PYRIDOXAL-phosphate containing enzyme that catalyzes the dehydration and deamination of L-serine to form pyruvate. This enzyme was formerly listed as EC 4.2.1.13.'),
  'D012696': NtO(MH='Glycine Hydroxymethyltransferase', MS='A pyridoxal phosphate enzyme that catalyzes the reaction of glycine and 5,10-methylene-tetrahydrofolate to form serine. It also catalyzes the reaction of glycine with acetaldehyde to form L-threonine. EC 2.1.2.1.'),
  'D012697': NtO(MH='Serine Endopeptidases', MS='Any member of the group of ENDOPEPTIDASES containing at the active site a serine residue involved in catalysis.'),
  'D012698': NtO(MH='Serologic Tests', MS='Diagnostic procedures involving immunoglobulin reactions.'),
  'D012699': NtO(MH='Serology', MS='The study of serum, especially of antigen-antibody reactions in vitro.'),
  'D012700': NtO(MH='Serositis', MS='Inflammation of a serous membrane.'),
  'D012701': NtO(MH='Serotonin', MS='A biochemical messenger and regulator, synthesized from the essential amino acid L-TRYPTOPHAN. In humans it is found primarily in the central nervous system, gastrointestinal tract, and blood platelets. Serotonin mediates several important physiological functions including neurotransmission, gastrointestinal motility, hemostasis, and cardiovascular integrity. Multiple receptor families (RECEPTORS, SEROTONIN) explain the broad physiological actions and distribution of this biochemical mediator.'),
  'D012702': NtO(MH='Serotonin Antagonists', MS='Drugs that bind to but do not activate serotonin receptors, thereby blocking the actions of serotonin or SEROTONIN RECEPTOR AGONISTS.'),
  'D012703': NtO(MH='Serotyping', MS='Process of determining and distinguishing species of bacteria or viruses based on antigens they share.'),
  'D012704': NtO(MH='Serous Membrane', MS='A thin lining of closed cavities of the body, consisting of a single layer of squamous epithelial cells (MESOTHELIUM) resting on a thin layer of CONNECTIVE TISSUE, and covered with secreted clear fluid from blood and lymph vessels. Major serous membranes in the body include PERICARDIUM; PERITONEUM; and PLEURA.'),
  'D012705': NtO(MH='Serratia', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs in the natural environment (soil, water, and plant surfaces) or as an opportunistic human pathogen.'),
  'D012706': NtO(MH='Serratia marcescens', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria found in soil, water, food, and clinical specimens. It is a prominent opportunistic pathogen for hospitalized patients.'),
  'D012707': NtO(MH='Sertoli Cell Tumor', MS='Gonadal neoplasm composed entirely of SERTOLI CELLS or may have a component of GRANULOSA CELLS. Some of the Sertoli cell tumors produce ESTROGEN or ANDROGENS, but seldom in sufficient quantity to cause clinical symptoms such as FEMINIZATION or masculinization (VIRILISM).'),
  'D012708': NtO(MH='Sertoli Cells', MS='Supporting cells projecting inward from the basement membrane of SEMINIFEROUS TUBULES. They surround and nourish the developing male germ cells and secrete ANDROGEN-BINDING PROTEIN and hormones such as ANTI-MULLERIAN HORMONE. The tight junctions of Sertoli cells with the SPERMATOGONIA and SPERMATOCYTES provide a BLOOD-TESTIS BARRIER.'),
  'D012709': NtO(MH='Serum Albumin', MS='A major protein in the BLOOD. It is important in maintaining the colloidal osmotic pressure and transporting large organic molecules.'),
  'D012710': NtO(MH='Serum Albumin, Bovine', MS='Serum albumin from cows, commonly used in in vitro biological studies. (From Stedman, 25th ed)'),
  'D012711': NtO(MH='Serum Albumin, Radio-Iodinated', MS='Normal human serum albumin mildly iodinated with radioactive iodine (131-I) which has a half-life of 8 days, and emits beta and gamma rays. It is used as a diagnostic aid in blood volume determination. (from Merck Index, 11th ed)'),
  'D012712': NtO(MH='Serum Globulins', MS='All blood proteins except albumin ( = SERUM ALBUMIN, which is not a globulin) and FIBRINOGEN (which is not in the serum). The serum globulins are subdivided into ALPHA-GLOBULINS; BETA-GLOBULINS; and GAMMA-GLOBULINS on the basis of their electrophoretic mobilities. (From Dorland, 28th ed)'),
  'D012713': NtO(MH='Serum Sickness', MS='Immune complex disease caused by the administration of foreign serum or serum proteins and characterized by fever, lymphadenopathy, arthralgia, and urticaria. When they are complexed to protein carriers, some drugs can also cause serum sickness when they act as haptens inducing antibody responses.'),
  'D012714': NtO(MH='Serine-tRNA Ligase', MS='An enzyme that activates serine with its specific transfer RNA. EC 6.1.1.11.'),
  'D012715': NtO(MH='Sesame Oil', MS='The refined fixed oil obtained from the seed of one or more cultivated varieties of Sesamum indicum. It is used as a solvent and oleaginous vehicle for drugs and has been used internally as a laxative and externally as a skin softener. It is used also in the manufacture of margarine, soap, and cosmetics. (Dorland, 28th ed & Random House Unabridged Dictionary, 2d ed)'),
  'D012716': NtO(MH='Sesamoid Bones', MS='Nodular bones which lie within a tendon and slide over another bony surface. The kneecap (see PATELLA) is a sesamoid bone.'),
  'D012717': NtO(MH='Sesquiterpenes', MS='Fifteen-carbon compounds formed from three isoprenoid units with general formula C15H24.'),
  'D012718': NtO(MH='Set, Psychology', MS='Readiness to think or respond in a predetermined way when confronted with a problem or stimulus situation.'),
  'D012719': NtO(MH='Setariasis', MS='Infection with nematodes of the genus Setaria. This condition is usually seen in cattle and equines and is of little pathogenic significance, although migration of the worm to the eye may lead to blindness.'),
  'D012720': NtO(MH='Severity of Illness Index', MS="Levels within a diagnostic group which are established by various measurement criteria applied to the seriousness of a patient's disorder."),
  'D012721': NtO(MH='Carbaryl', MS='A carbamate insecticide and parasiticide. It is a potent anticholinesterase agent belonging to the carbamate group of reversible cholinesterase inhibitors. It has a particularly low toxicity from dermal absorption and is used for control of head lice in some countries.'),
  'D012722': NtO(MH='Sewage', MS='Refuse liquid or waste matter carried off by sewers.'),
  'D012723': NtO(MH='Sex', MS='The totality of characteristics of reproductive structure, functions, PHENOTYPE, and GENOTYPE, differentiating the MALE from the FEMALE organism.'),
  'D012724': NtO(MH='Sex Attractants', MS='Pheromones that elicit sexual attraction or mating behavior usually in members of the opposite sex in the same species.'),
  'D012725': NtO(MH='Sexual Behavior', MS='Sexual activities of humans.'),
  'D012726': NtO(MH='Sexual Behavior, Animal', MS='Sexual activities of animals.'),
  'D012727': NtO(MH='Sex Characteristics', MS='Those characteristics that distinguish one SEX from the other. The primary sex characteristics are the OVARIES and TESTES and their related hormones. Secondary sex characteristics are those which are masculine or feminine but not directly related to reproduction.'),
  'D012728': NtO(MH='Sex Chromatin', MS='In the interphase nucleus, a condensed mass of chromatin representing an inactivated X chromosome. Each X CHROMOSOME, in excess of one, forms sex chromatin (Barr body) in the mammalian nucleus. (from King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D012729': NtO(MH='Sex Chromosome Aberrations', MS='Abnormal number or structure of the SEX CHROMOSOMES. Some sex chromosome aberrations are associated with SEX CHROMOSOME DISORDERS and SEX CHROMOSOME DISORDERS OF SEX DEVELOPMENT.'),
  'D012730': NtO(MH='Sex Chromosomes', MS='The homologous chromosomes that are dissimilar in the heterogametic sex. There are the X CHROMOSOME, the Y CHROMOSOME, and the W, Z chromosomes (in animals in which the female is the heterogametic sex (the silkworm moth Bombyx mori, for example)). In such cases the W chromosome is the female-determining and the male is ZZ. (From King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D012731': NtO(MH='Sex Counseling', MS='Advice and support given to individuals to help them understand and resolve their sexual adjustment problems. It excludes treatment for PSYCHOSEXUAL DISORDERS or PSYCHOSEXUAL DYSFUNCTION.'),
  'D012732': NtO(MH='Sex Determination Analysis', MS='Validation of the SEX of an individual by inspection of the GONADS and/or by genetic tests.'),
  'D012733': NtO(MH='Sex Differentiation', MS='The process in developing sex- or gender-specific tissue, organ, or function after SEX DETERMINATION PROCESSES have set the sex of the GONADS. Major areas of sex differentiation occur in the reproductive tract (GENITALIA) and the brain.'),
  'D012734': NtO(MH='Disorders of Sex Development', MS='In gonochoristic organisms, congenital conditions in which development of chromosomal, gonadal, or anatomical sex is atypical. Effects from exposure to abnormal levels of GONADAL HORMONES in the maternal environment, or disruption of the function of those hormones by ENDOCRINE DISRUPTORS are included.'),
  'D012735': NtO(MH='Sexual Dysfunction, Physiological', MS='Physiological disturbances in normal sexual performance in either the male or the female.'),
  'D012736': NtO(MH='Sex Education', MS='Education which increases the knowledge of the functional, structural, and behavioral aspects of human reproduction.'),
  'D012737': NtO(MH='Sex Factors', MS='Maleness or femaleness as a constituent element or influence contributing to the production of a result. It may be applicable to the cause or effect of a circumstance. It is used with human or animal concepts but should be differentiated from SEX CHARACTERISTICS, anatomical or physiological manifestations of sex, and from SEX DISTRIBUTION, the number of males and females in given circumstances.'),
  'D012738': NtO(MH='Sex Hormone-Binding Globulin', MS='A glycoprotein migrating as a beta-globulin. Its molecular weight, 52,000 or 95,000-115,000, indicates that it exists as a dimer. The protein binds testosterone, dihydrotestosterone, and estradiol in the plasma. Sex hormone-binding protein has the same amino acid sequence as ANDROGEN-BINDING PROTEIN. They differ by their sites of synthesis and post-translational oligosaccharide modifications.'),
  'D012739': NtO(MH='Gonadal Steroid Hormones', MS='Steroid hormones produced by the GONADS. They stimulate reproductive organs, germ cell maturation, and the secondary sex characteristics in the males and the females. The major sex steroid hormones include ESTRADIOL; PROGESTERONE; and TESTOSTERONE.'),
  'D012740': NtO(MH='Sex Manuals', MS='Books used for sex education and procedural instruction.'),
  'D012741': NtO(MH='Sexual Maturation', MS='Achievement of full sexual capacity in animals and in humans.'),
  'D012742': NtO(MH='Sex Offenses', MS='Any violation of established legal or moral codes in respect to sexual behavior.'),
  'D012743': NtO(MH='Sex Preselection', MS='Methods for controlling genetic SEX of offspring.'),
  'D012744': NtO(MH='Sex Ratio', MS='The number of males per 100 females.'),
  'D012746': NtO(MH='Sexual Abstinence', MS='Refraining from SEXUAL INTERCOURSE.'),
  'D012747': NtO(MH='Sexual Partners', MS='Married or single individuals who share sexual relations.'),
  'D012749': NtO(MH='Sexually Transmitted Diseases', MS='Diseases due to or propagated by sexual contact.'),
  'D012750': NtO(MH='Seychelles', MS="A group of Indian Ocean Islands, east of Tanzania. Their capital is Victoria. They were first claimed by the French in 1744 but taken by the English in 1794 and made a dependency of MAURITIUS in 1810. They became a crown colony in 1903 and a republic within the Commonwealth in 1976. They were named for the French finance minister, Jean Moreau de Sechelles, but respelled by the English in 1794. (From Webster's New Geographical Dictionary, 1988, p1102 & Room, Brewer's Dictionary of Names, 1992, p496)"),
  'D012751': NtO(MH='Sezary Syndrome', MS='A form of cutaneous T-cell lymphoma manifested by generalized exfoliative ERYTHRODERMA; PRURITUS; peripheral lymphadenopathy, and abnormal hyperchromatic mononuclear (cerebriform) cells in the skin, LYMPH NODES, and peripheral blood (Sezary cells).'),
  'D012752': NtO(MH='Shame', MS='An emotional attitude excited by realization of a shortcoming or impropriety.'),
  'D012753': NtO(MH='Shared Paranoid Disorder', MS='A condition in which closely related persons, usually in the same family, share the same delusions.'),
  'D012754': NtO(MH='Sharks', MS='A group of elongate elasmobranchs. Sharks are mostly marine fish, with certain species large and voracious.'),
  'D012756': NtO(MH='Sheep', MS='Any of the ruminant mammals with curved horns in the genus Ovis, family Bovidae. They possess lachrymal grooves and interdigital glands, which are absent in GOATS.'),
  'D012757': NtO(MH='Sheep Diseases', MS='Diseases of domestic and mountain sheep of the genus Ovis.'),
  'D012758': NtO(MH='Shellfish', MS='Aquatic invertebrates belonging to the phylum MOLLUSCA or the subphylum CRUSTACEA, and used as food.'),
  'D012759': NtO(MH='Sheltered Workshops', MS='Protective places of employment for disabled persons which provide training and employment on a temporary or permanent basis.'),
  'D012760': NtO(MH='Shigella', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that ferments sugar without gas production. Its organisms are intestinal pathogens of man and other primates and cause bacillary dysentery (DYSENTERY, BACILLARY).'),
  'D012761': NtO(MH='Shigella boydii', MS='One of the SHIGELLA species that produces bacillary dysentery (DYSENTERY, BACILLARY).'),
  'D012762': NtO(MH='Shigella dysenteriae', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that is extremely pathogenic and causes severe dysentery. Infection with this organism often leads to ulceration of the intestinal epithelium.'),
  'D012763': NtO(MH='Shigella flexneri', MS='A bacterium which is one of the etiologic agents of bacillary dysentery (DYSENTERY, BACILLARY) and sometimes of infantile gastroenteritis.'),
  'D012764': NtO(MH='Shigella sonnei', MS='A lactose-fermenting bacterium causing dysentery.'),
  'D012765': NtO(MH='Shikimic Acid', MS='A tri-hydroxy cyclohexene carboxylic acid important in biosynthesis of so many compounds that the shikimate pathway is named after it.'),
  'D012766': NtO(MH='Pasteurellosis, Pneumonic', MS='Bovine respiratory disease found in animals that have been shipped or exposed to CATTLE recently transported. The major agent responsible for the disease is MANNHEIMIA HAEMOLYTICA and less commonly, PASTEURELLA MULTOCIDA or HAEMOPHILUS SOMNUS. All three agents are normal inhabitants of the bovine nasal pharyngeal mucosa but not the LUNG. They are considered opportunistic pathogens following STRESS, PHYSIOLOGICAL and/or a viral infection. The resulting bacterial fibrinous BRONCHOPNEUMONIA is often fatal.'),
  'D012767': NtO(MH='Ships', MS='Large vessels propelled by power or sail used for transportation on rivers, seas, oceans, or other navigable waters. Boats are smaller vessels propelled by oars, paddles, sail, or power; they may or may not have a deck.'),
  'D012768': NtO(MH='Shivering', MS='Involuntary contraction or twitching of the muscles. It is a physiologic method of heat production in man and other mammals.'),
  'D012769': NtO(MH='Shock', MS='A pathological condition manifested by failure to perfuse or oxygenate vital organs.'),
  'D012770': NtO(MH='Shock, Cardiogenic', MS='Shock resulting from diminution of cardiac output in heart disease.'),
  'D012771': NtO(MH='Shock, Hemorrhagic', MS='Acute hemorrhage or excessive fluid loss resulting in HYPOVOLEMIA.'),
  'D012772': NtO(MH='Shock, Septic', MS='Sepsis associated with HYPOTENSION or hypoperfusion despite adequate fluid resuscitation. Perfusion abnormalities may include, but are not limited to LACTIC ACIDOSIS; OLIGURIA; or acute alteration in mental status.'),
  'D012773': NtO(MH='Shock, Surgical', MS='A type of shock that occurs as a result of a surgical procedure.'),
  'D012774': NtO(MH='Shock, Traumatic', MS='Shock produced as a result of trauma.'),
  'D012775': NtO(MH='Shoes', MS='Footwear made of a flat sole shaped to the contour of the foot, connected to an upper portion which may or may not completely cover or wrap around the foot.'),
  'D012776': NtO(MH='Fibroma Virus, Rabbit', MS='A species of LEPORIPOXVIRUS causing subcutaneous localized swellings in rabbits, usually on the feet.'),
  'D012777': NtO(MH='Cottontail rabbit papillomavirus', MS='The type species of KAPPAPAPILLOMAVIRUS. It is reported to occur naturally in cottontail rabbits in North America.'),
  'D012778': NtO(MH='Short Bowel Syndrome', MS='A malabsorption syndrome resulting from extensive operative resection of the SMALL INTESTINE, the absorptive region of the GASTROINTESTINAL TRACT.'),
  'D012779': NtO(MH='Short Rib-Polydactyly Syndrome', MS='A syndrome inherited as an autosomal recessive trait and incompatible with life. The main features are narrow thorax, short ribs, scapular and pelvic dysplasia, and polydactyly.'),
  'D012780': NtO(MH='Short-Wave Therapy', MS='The use of focused short radio waves to produce local hyperthermia in an injured person or diseased body area.'),
  'D012781': NtO(MH='Shorthand', MS='Rapid handwriting using simple strokes, abbreviations, or symbols that designate letters, words, or phrases.'),
  'D012782': NtO(MH='Shoulder', MS='Part of the body in humans and primates where the arms connect to the trunk. The shoulder has five joints; ACROMIOCLAVICULAR joint, CORACOCLAVICULAR joint, GLENOHUMERAL joint, scapulathoracic joint, and STERNOCLAVICULAR joint.'),
  'D012783': NtO(MH='Shoulder Dislocation', MS='Displacement of the HUMERUS from the SCAPULA.'),
  'D012784': NtO(MH='Shoulder Fractures', MS='Fractures of the proximal humerus, including the head, anatomic and surgical necks, and tuberosities.'),
  'D012785': NtO(MH='Shoulder Joint', MS='The articulation between the head of the HUMERUS and the glenoid cavity of the SCAPULA.'),
  'D012787': NtO(MH='Showdomycin', MS='3-beta-D-Ribofuranosylmaleimide. Antineoplastic antibiotic isolated from Streptomyces showdoensis. It is possibly active also as a sulfhydryl reagent.'),
  'D012788': NtO(MH='Shrews', MS='Small mammals in the family Soricidae, order Soricomorpha (formerly Insectivora). They resemble moles and are characterized by a mobile snout, sharp INCISOR teeth, and are found in tropical and temperate regions worldwide.'),
  'D012790': NtO(MH='Shwartzman Phenomenon', MS='Hemorrhagic necrosis that was first demonstrated in rabbits with a two-step reaction, an initial local (intradermal) or general (intravenous) injection of a priming endotoxin (ENDOTOXINS) followed by a second intravenous endotoxin injection (provoking agent) 24 h later. The acute inflammation damages the small blood vessels. The following intravascular coagulation leads to capillary and venous THROMBOSIS and NECROSIS. Shwartzman phenomenon can also occur in other species with a single injection of a provoking agent, and during infections or pregnancy. Its susceptibility depends on the status of IMMUNE SYSTEM, coagulation, FIBRINOLYSIS, and blood flow.'),
  'D012791': NtO(MH='Shy-Drager Syndrome', MS='A progressive neurodegenerative condition of the central and autonomic nervous systems characterized by atrophy of the preganglionic lateral horn neurons of the thoracic spinal cord. This disease is generally considered a clinical variant of MULTIPLE SYSTEM ATROPHY. Affected individuals present in the fifth or sixth decade with ORTHOSTASIS and bladder dysfunction; and later develop FECAL INCONTINENCE; anhidrosis; ATAXIA; IMPOTENCE; and alterations of tone suggestive of basal ganglia dysfunction. (From Adams et al., Principles of Neurology, 6th ed, p536)'),
  'D012792': NtO(MH='Shyness', MS='Discomfort and partial inhibition of the usual forms of behavior when in the presence of others.'),
  'D012793': NtO(MH='Sialadenitis', MS='INFLAMMATION of salivary tissue (SALIVARY GLANDS), usually due to INFECTION or injuries.'),
  'D012794': NtO(MH='Sialic Acids', MS='A group of naturally occurring N-and O-acyl derivatives of the deoxyamino sugar neuraminic acid. They are ubiquitously distributed in many tissues.'),
  'D012795': NtO(MH='Sialoglycoproteins', MS='Glycoproteins which contain sialic acid as one of their carbohydrates. They are often found on or in the cell or tissue membranes and participate in a variety of biological activities.'),
  'D012796': NtO(MH='Sialography', MS='Radiography of the SALIVARY GLANDS or ducts following injection of contrast medium.'),
  'D012797': NtO(MH='Sialometaplasia, Necrotizing', MS='A benign, inflammatory, variably ulcerated, occasionally bilateral, self-healing lesion of the minor salivary glands that is often confused clinically and histologically with carcinoma.'),
  'D012798': NtO(MH='Sialorrhea', MS='Increased salivary flow.'),
  'D012799': NtO(MH='Sialyltransferases', MS='A group of enzymes with the general formula CMP-N-acetylneuraminate:acceptor N-acetylneuraminyl transferase. They catalyze the transfer of N-acetylneuraminic acid from CMP-N-acetylneuraminic acid to an acceptor, which is usually the terminal sugar residue of an oligosaccharide, a glycoprotein, or a glycolipid. EC 2.4.99.-.'),
  'D012800': NtO(MH='Siberia', MS='A region, north-central Asia, largely in Russia. It extends from the Ural Mountains to the Pacific Ocean and from the Arctic Ocean to central Kazakhstan and the borders of China and Mongolia.'),
  'D012801': NtO(MH='Sibling Relations', MS='Interactions and relationships between sisters and/or brothers. The concept also applies to animal studies.'),
  'D012802': NtO(MH='Sicily', MS='An island located in the Mediterranean Sea and governed by Italy,'),
  'D012803': NtO(MH='Sick Role', MS='Set of expectations that exempt persons from responsibility for their illness and exempt them from usual responsibilities.'),
  'D012804': NtO(MH='Sick Sinus Syndrome', MS='A condition caused by dysfunctions related to the SINOATRIAL NODE including impulse generation (CARDIAC SINUS ARREST) and impulse conduction (SINOATRIAL EXIT BLOCK). It is characterized by persistent BRADYCARDIA, chronic ATRIAL FIBRILLATION, and failure to resume sinus rhythm following CARDIOVERSION. This syndrome can be congenital or acquired, particularly after surgical correction for heart defects.'),
  'D012805': NtO(MH='Sickle Cell Trait', MS='The condition of being heterozygous for hemoglobin S.'),
  'D012806': NtO(MH='Siderosis', MS='A form of pneumoconiosis resulting from inhalation of iron in the mining dust or welding fumes.'),
  'D012807': NtO(MH='Sierra Leone', MS='A republic in western Africa, south of GUINEA and west of LIBERIA. Its capital is Freetown.'),
  'D012808': NtO(MH='Sigma Factor', MS='A protein which is a subunit of RNA polymerase. It effects initiation of specific RNA chains from DNA.'),
  'D012809': NtO(MH='Colon, Sigmoid', MS='A segment of the COLON between the RECTUM and the descending colon.'),
  'D012810': NtO(MH='Sigmoid Diseases', MS='Pathological processes in the SIGMOID COLON region of the large intestine (INTESTINE, LARGE).'),
  'D012811': NtO(MH='Sigmoid Neoplasms', MS='Tumors or cancer of the SIGMOID COLON.'),
  'D012812': NtO(MH='Sigmoidoscopy', MS='Endoscopic examination, therapy or surgery of the sigmoid flexure.'),
  'D012813': NtO(MH='Sign Language', MS='A system of hand gestures used for communication by the deaf or by people speaking different languages.'),
  'D012815': NtO(MH='Signal Processing, Computer-Assisted', MS='Computer-assisted processing of electric, ultrasonic, or electronic signals to interpret function and activity.'),
  'D012816': NtO(MH='Signs and Symptoms', MS='Clinical manifestations that can be either objective when observed by a physician, or subjective when perceived by the patient.'),
  'D012817': NtO(MH='Signs and Symptoms, Digestive', MS='Digestive system manifestations of diseases of the gastrointestinal system or of other organs.'),
  'D012818': NtO(MH='Signs and Symptoms, Respiratory', MS='Respiratory system manifestations of diseases of the respiratory tract or of other organs.'),
  'D012819': NtO(MH='Sikkim', MS='A state within northeast India in the Eastern Himalayas. The capital is Gangtok.'),
  'D012820': NtO(MH='Silage', MS='Fodder converted into succulent feed for livestock through processes of anaerobic fermentation (as in a silo).'),
  'D012821': NtO(MH='Silanes', MS='Compounds similar to hydrocarbons in which a tetravalent silicon atom replaces the carbon atom. They are very reactive, ignite in air, and form useful derivatives.'),
  'D012822': NtO(MH='Silicon Dioxide', MS='Transparent, tasteless crystals found in nature as agate, amethyst, chalcedony, cristobalite, flint, sand, QUARTZ, and tridymite. The compound is insoluble in water or acids except hydrofluoric acid.'),
  'D012823': NtO(MH='Silicate Cement', MS="A relatively hard, translucent, restorative material used primarily in anterior teeth. (From Boucher's Clinical Dental Terminology, 4th ed, p50)"),
  'D012824': NtO(MH='Silicic Acid', MS='A hydrated form of silicon dioxide. It is commonly used in the manufacture of TOOTHPASTES and as a stationary phase for CHROMATOGRAPHY.'),
  'D012825': NtO(MH='Silicon', MS="A trace element that constitutes about 27.6% of the earth's crust in the form of SILICON DIOXIDE. It does not occur free in nature. Silicon has the atomic symbol Si, atomic number 14, and atomic weight [28.084; 28.086]."),
  'D012826': NtO(MH='Silicone Elastomers', MS='Polymers of silicone that are formed by crosslinking and treatment with amorphous silica to increase strength. They have properties similar to vulcanized natural rubber, in that they stretch under tension, retract rapidly, and fully recover to their original dimensions upon release. They are used in the encapsulation of surgical membranes and implants.'),
  'D012827': NtO(MH='Silicone Oils', MS='Organic siloxanes which are polymerized to the oily stage. The oils have low surface tension and density less than 1. They are used in industrial applications and in the treatment of retinal detachment, complicated by proliferative vitreoretinopathy.'),
  'D012828': NtO(MH='Silicones', MS='A broad family of synthetic organosiloxane polymers containing a repeating silicon-oxygen backbone with organic side groups attached via carbon-silicon bonds. Depending on their structure, they are classified as liquids, gels, and elastomers. (From Merck Index, 12th ed)'),
  'D012829': NtO(MH='Silicosis', MS='A form of pneumoconiosis resulting from inhalation of dust containing crystalline form of SILICON DIOXIDE, usually in the form of quartz. Amorphous silica is relatively nontoxic.'),
  'D012830': NtO(MH='Silicotuberculosis', MS='Pulmonary or extrapulmonary infection caused by MYCOBACTERIUM TUBERCULOSIS or nontuberculous mycobacteria in a patient with silicosis.'),
  'D012831': NtO(MH='Bombyx', MS='A genus of silkworm MOTHS in the family Bombycidae of the order LEPIDOPTERA. The family contains a single species, Bombyx mori from the Greek for silkworm + mulberry tree (on which it feeds). A native of Asia, it is sometimes reared in this country. It has long been raised for its SILK and after centuries of domestication it probably does not exist in nature. It is used extensively in experimental GENETICS. (From Borror et al., An Introduction to the Study of Insects, 4th ed, p519)'),
  'D012832': NtO(MH="Silo Filler's Disease", MS='A form of alveolitis or PNEUMONITIS caused by hypersensitivity to high level of inhaled nitrogen oxides, decomposition products of silage.'),
  'D012833': NtO(MH='Siloxanes', MS='Silicon polymers that contain alternate silicon and oxygen atoms in linear or cyclic molecular structures.'),
  'D012834': NtO(MH='Silver', MS='An element with the atomic symbol Ag, atomic number 47, and atomic weight 107.87. It is a soft metal that is used medically in surgical instruments, dental prostheses, and alloys. Long-continued use of silver salts can lead to a form of poisoning known as ARGYRIA.'),
  'D012835': NtO(MH='Silver Nitrate', MS='A silver salt with powerful germicidal activity. It has been used topically to prevent OPHTHALMIA NEONATORUM.'),
  'D012836': NtO(MH='Silver Proteins', MS='Compounds of silver and proteins used as topical anti-infective agents.'),
  'D012837': NtO(MH='Silver Sulfadiazine', MS='Antibacterial used topically in burn therapy.'),
  'D012838': NtO(MH='Silymarin', MS='A mixture of flavonoids extracted from seeds of the MILK THISTLE, Silybum marianum. It consists primarily of silybin and its isomers, silicristin and silidianin. Silymarin displays antioxidant and membrane stabilizing activity. It protects various tissues and organs against chemical injury, and shows potential as an antihepatoxic agent.'),
  'D012839': NtO(MH='Simazine', MS='A triazine herbicide.'),
  'D012840': NtO(MH='Simbu virus', MS='A species in the ORTHOBUNYAVIRUS genus of the family BUNYAVIRIDAE family. Previously a large group of serotypes, most are now considered separate species.'),
  'D012841': NtO(MH='Simethicone', MS='A poly(dimethylsiloxane) which is a polymer of 200-350 units of dimethylsiloxane, along with added silica gel. It is used as an antiflatulent, surfactant, and ointment base.'),
  'D012842': NtO(MH='Sarcoma Virus, Woolly Monkey', MS='A species of GAMMARETROVIRUS producing tumors in primates. Originally isolated from a fibrosarcoma in a woolly monkey, WMSV is a replication-defective v-onc virus which carries the sis oncogene. In order to propagate, WMSV requires a replication-competent helper virus.'),
  'D012843': NtO(MH='Simuliidae', MS='A family of insects in the order DIPTERA, which include black flies, buffalo gnats and simulies. Several species are intermediate hosts (vectors) for the parasitic disease ONCHOCERCIASIS.'),
  'D012844': NtO(MH='Sincalide', MS='An octapeptide hormone present in the intestine and brain. When secreted from the gastric mucosa, it stimulates the release of bile from the gallbladder and digestive enzymes from the pancreas.'),
  'D012845': NtO(MH='Sindbis Virus', MS='The type species of ALPHAVIRUS normally transmitted to birds by CULEX mosquitoes in Egypt, South Africa, India, Malaya, the Philippines, and Australia. It may be associated with fever in humans. Serotypes (differing by less than 17% in nucleotide sequence) include Babanki, Kyzylagach, and Ockelbo viruses.'),
  'D012846': NtO(MH='Singapore', MS='Country of islands in Southeastern Asia, islands between Malaysia and Indonesia. The capital is Singapore.'),
  'D012847': NtO(MH='Single Person', MS='The unmarried man or woman.'),
  'D012848': NtO(MH='Sinoatrial Block', MS='Disturbance in the atrial activation that is caused by transient failure of impulse conduction from the SINOATRIAL NODE to the HEART ATRIA. It is characterized by a delayed in heartbeat and pauses between P waves in an ELECTROCARDIOGRAM.'),
  'D012849': NtO(MH='Sinoatrial Node', MS='The small mass of modified cardiac muscle fibers located at the junction of the superior vena cava (VENA CAVA, SUPERIOR) and right atrium. Contraction impulses probably start in this node, spread over the atrium (HEART ATRIUM) and are then transmitted by the atrioventricular bundle (BUNDLE OF HIS) to the ventricle (HEART VENTRICLE).'),
  'D012850': NtO(MH='Sinus of Valsalva', MS='The dilatation of the aortic wall behind each of the cusps of the aortic valve.'),
  'D012851': NtO(MH='Sinus Thrombosis, Intracranial', MS='Formation or presence of a blood clot (THROMBUS) in the CRANIAL SINUSES, large endothelium-lined venous channels situated within the SKULL. Intracranial sinuses, also called cranial venous sinuses, include the superior sagittal, cavernous, lateral, petrous sinuses, and many others. Cranial sinus thrombosis can lead to severe HEADACHE; SEIZURE; and other neurological defects.'),
  'D012852': NtO(MH='Sinusitis', MS='Inflammation of the NASAL MUCOSA in one or more of the PARANASAL SINUSES.'),
  'D012853': NtO(MH='Sisomicin', MS='Antibiotic produced by Micromonospora inyoensis. It is closely related to gentamicin C1A, one of the components of the gentamicin complex (GENTAMICINS).'),
  'D012854': NtO(MH='Sister Chromatid Exchange', MS='An exchange of segments between the sister chromatids of a chromosome, either between the sister chromatids of a meiotic tetrad or between the sister chromatids of a duplicated somatic chromosome. Its frequency is increased by ultraviolet and ionizing radiation and other mutagenic agents and is particularly high in BLOOM SYNDROME.'),
  'D012855': NtO(MH='Sitosterols', MS='A family of sterols commonly found in plants and plant oils. Alpha-, beta-, and gamma-isomers have been characterized.'),
  'D012856': NtO(MH="4-Acetamido-4'-isothiocyanatostilbene-2,2'-disulfonic Acid", MS='A non-penetrating amino reagent (commonly called SITS) which acts as an inhibitor of anion transport in erythrocytes and other cells.'),
  'D012857': NtO(MH='Situs Inversus', MS='A congenital abnormality in which organs in the THORAX and the ABDOMEN are opposite to their normal positions (situs solitus) due to lateral transposition. Normally the STOMACH and SPLEEN are on the left, LIVER on the right, the three-lobed right lung is on the right, and the two-lobed left lung on the left. Situs inversus has a familial pattern and has been associated with a number of genes related to microtubule-associated proteins.'),
  'D012858': NtO(MH='Size Perception', MS='The sensory interpretation of the dimensions of objects.'),
  'D012859': NtO(MH="Sjogren's Syndrome", MS='Chronic inflammatory and autoimmune disease in which the salivary and lacrimal glands undergo progressive destruction by lymphocytes and plasma cells resulting in decreased production of saliva and tears. The primary form, often called sicca syndrome, involves both KERATOCONJUNCTIVITIS SICCA and XEROSTOMIA. The secondary form includes, in addition, the presence of a connective tissue disease, usually rheumatoid arthritis.'),
  'D012860': NtO(MH='Skates, Fish', MS='The common name for all members of the Rajidae family. Skates and rays are members of the same order (Rajiformes). Skates have weak electric organs.'),
  'D012861': NtO(MH='Skating', MS='Using ice skates, roller skates, or skateboards in racing or other competition or for recreation.'),
  'D012862': NtO(MH='Skatole', MS=None),
  'D012863': NtO(MH='Skeleton', MS='The rigid framework of connected bones that gives form to the body, protects and supports its soft organs and tissues, and provides attachments for MUSCLES.'),
  'D012865': NtO(MH='Skiing', MS='A snow sport which uses skis to glide over the snow. It does not include water-skiing.'),
  'D012866': NtO(MH='Skilled Nursing Facilities', MS='Extended care facilities which provide skilled nursing care or rehabilitation services for inpatients on a daily basis.'),
  'D012867': NtO(MH='Skin', MS='The outer covering of the body that protects it from the environment. It is composed of the DERMIS and the EPIDERMIS.'),
  'D012868': NtO(MH='Skin Abnormalities', MS='Congenital structural abnormalities of the skin.'),
  'D012869': NtO(MH='Skin Absorption', MS='Uptake of substances through the SKIN.'),
  'D012871': NtO(MH='Skin Diseases', MS='Diseases involving the DERMIS or EPIDERMIS.'),
  'D012872': NtO(MH='Skin Diseases, Vesiculobullous', MS='Skin diseases characterized by local or general distributions of blisters. They are classified according to the site and mode of blister formation. Lesions can appear spontaneously or be precipitated by infection, trauma, or sunlight. Etiologies include immunologic and genetic factors. (From Scientific American Medicine, 1990)'),
  'D012873': NtO(MH='Skin Diseases, Genetic', MS='Diseases of the skin with a genetic component, usually the result of various inborn errors of metabolism.'),
  'D012874': NtO(MH='Skin Diseases, Infectious', MS='Skin diseases caused by bacteria, fungi, parasites, or viruses.'),
  'D012875': NtO(MH='Skin Diseases, Metabolic', MS='Diseases of the skin associated with underlying metabolic disorders.'),
  'D012876': NtO(MH='Skin Diseases, Parasitic', MS='Skin diseases caused by ARTHROPODS; HELMINTHS; or other parasites.'),
  'D012877': NtO(MH='Skin Manifestations', MS='Dermatologic disorders attendant upon non-dermatologic disease or injury.'),
  'D012878': NtO(MH='Skin Neoplasms', MS='Tumors or cancer of the SKIN.'),
  'D012879': NtO(MH='Skin Physiological Phenomena', MS='The functions of the skin in the human and animal body. It includes the pigmentation of the skin.'),
  'D012880': NtO(MH='Skin Pigmentation', MS='Coloration of the skin.'),
  'D012881': NtO(MH='Skin Temperature', MS='The TEMPERATURE at the outer surface of the body.'),
  'D012882': NtO(MH='Skin Tests', MS='Epicutaneous or intradermal application of a sensitizer for demonstration of either delayed or immediate hypersensitivity. Used in diagnosis of hypersensitivity or as a test for cellular immunity.'),
  'D012883': NtO(MH='Skin Ulcer', MS='An ULCER of the skin and underlying tissues.'),
  'D012884': NtO(MH='Skin Window Technique', MS='A technique to study CELL MIGRATION in the INFLAMMATION process or during immune reactions. After an area on the skin is abraded, the movement of cells in the area is followed via microscopic observation of the exudate through a coverslip or tissue culture chamber placed over the area.'),
  'D012885': NtO(MH='Skinfold Thickness', MS='The measurement of subcutaneous fat located directly beneath the skin by grasping a fold of skin and subcutaneous fat between the thumb and forefinger and pulling it away from the underlying muscle tissue. The thickness of the double layer of skin and subcutaneous tissue is then read with a caliper. The five most frequently measured sites are the upper arm, below the scapula, above the hip bone, the abdomen, and the thigh. Its application is the determination of relative fatness, of changes in physical conditioning programs, and of the percentage of body fat in desirable body weight. (From McArdle, et al., Exercise Physiology, 2d ed, p496-8)'),
  'D012886': NtO(MH='Skull', MS='The SKELETON of the HEAD including the FACIAL BONES and the bones enclosing the BRAIN.'),
  'D012887': NtO(MH='Skull Fractures', MS='Fractures of the skull which may result from penetrating or nonpenetrating head injuries or rarely BONE DISEASES (see also FRACTURES, SPONTANEOUS). Skull fractures may be classified by location (e.g., SKULL FRACTURE, BASILAR), radiographic appearance (e.g., linear), or based upon cranial integrity (e.g., SKULL FRACTURE, DEPRESSED).'),
  'D012888': NtO(MH='Skull Neoplasms', MS='Neoplasms of the bony part of the skull.'),
  'D012889': NtO(MH='Mephitidae', MS='The family of omnivorous New World skunks, showing typical warning coloration of patterned black and white and able to eject a malodorous secretion when the animal is startled or in danger.'),
  'D012890': NtO(MH='Sleep', MS='A readily reversible suspension of sensorimotor interaction with the environment, usually associated with recumbency and immobility.'),
  'D012891': NtO(MH='Sleep Apnea Syndromes', MS='Disorders characterized by multiple cessations of respirations during sleep that induce partial arousals and interfere with the maintenance of sleep. Sleep apnea syndromes are divided into central (see SLEEP APNEA, CENTRAL), obstructive (see SLEEP APNEA, OBSTRUCTIVE), and mixed central-obstructive types.'),
  'D012892': NtO(MH='Sleep Deprivation', MS='The state of being deprived of sleep under experimental conditions, due to life events, or from a wide variety of pathophysiologic causes such as medication effect, chronic illness, psychiatric illness, or sleep disorder.'),
  'D012893': NtO(MH='Sleep Wake Disorders', MS='Abnormal sleep-wake schedule or pattern associated with the CIRCADIAN RHYTHM which affect the length, timing, and/or rigidity of the sleep-wake cycle relative to the day-night cycle.'),
  'D012894': NtO(MH='Sleep Stages', MS='Periods of sleep manifested by changes in EEG activity and certain behavioral correlates; they formerly included Stage 1: sleep onset, drowsy sleep; Stage 2: light sleep; Stages 3 and 4: delta sleep, light sleep, deep sleep, telencephalic sleep. In 2007, sleep stages were redefined by The American Academy of Sleep Medicine (AASM) as: N1-N2 (sleep onset - light sleep), N3 (SLOW-WAVE SLEEP), and REM SLEEP.'),
  'D012895': NtO(MH='Sleep, REM', MS='A stage of sleep characterized by rapid movements of the eye and low voltage fast pattern EEG. It is usually associated with dreaming.'),
  'D012896': NtO(MH='Sloths', MS='Slow-moving exclusively arboreal mammals that inhabit the tropical forests of South and Central America.'),
  'D012897': NtO(MH='Slow Virus Diseases', MS='Diseases of viral origin, characterized by incubation periods of months to years, insidious onset of clinical manifestations, and protracted clinical course. Though the disease process is protracted, viral multiplication may not be unusually slow. Conventional viruses produce slow virus diseases such as SUBACUTE SCLEROSING PANENCEPHALITIS, progressive multifocal leukoencephalopathy (LEUKOENCEPHALOPATHY, PROGRESSIVE MULTIFOCAL), and AIDS. Diseases produced by unconventional agents were originally considered part of this group. They are now called PRION DISEASES.'),
  'D012898': NtO(MH='Autacoids', MS='A chemically diverse group of substances produced by various tissues in the body that cause slow contraction of smooth muscle; they have other intense but varied pharmacologic activities.'),
  'D012899': NtO(MH='Smallpox', MS='An acute, highly contagious, often fatal infectious disease caused by an orthopoxvirus characterized by a biphasic febrile course and distinctive progressive skin eruptions. Vaccination has succeeded in eradicating smallpox worldwide. (Dorland, 28th ed)'),
  'D012900': NtO(MH='Smallpox Vaccine', MS='A live VACCINIA VIRUS vaccine of calf lymph or chick embryo origin, used for immunization against smallpox. It is now recommended only for laboratory workers exposed to smallpox virus. Certain countries continue to vaccinate those in the military service. Complications that result from smallpox vaccination include vaccinia, secondary bacterial infections, and encephalomyelitis. (Dorland, 28th ed)'),
  'D012901': NtO(MH='Variola virus', MS='A species of ORTHOPOXVIRUS causing infections in humans. No infections have been reported since 1977 and the virus is now believed to be virtually extinct.'),
  'D012902': NtO(MH='Smegma', MS='A foul-smelling accumulation of SEBUM and desquaminated epidermal cells, especially the cheesy substance found under the foreskin of the penis and at the base of the labia minor near the clitoris.'),
  'D012903': NtO(MH='Smell', MS='The ability to detect scents or odors, such as the function of OLFACTORY RECEPTOR NEURONS.'),
  'D012904': NtO(MH='Smiling', MS='A facial expression which may denote feelings of pleasure, affection, amusement, etc.'),
  'D012905': NtO(MH='Smog', MS='A mixture of smoke and fog polluting the atmosphere. (Dorland, 27th ed)'),
  'D012906': NtO(MH='Smoke', MS='Visible gaseous suspension of carbon and other particulate matter emitted from burning substances.'),
  'D012907': NtO(MH='Smoking', MS='Willful or deliberate act of inhaling and exhaling SMOKE from burning substances or agents held by hand.'),
  'D012908': NtO(MH='Snails', MS='Marine, freshwater, or terrestrial mollusks of the class Gastropoda. Most have an enclosing spiral shell, and several genera harbor parasites pathogenic to man.'),
  'D012909': NtO(MH='Snake Bites', MS="Bites by snakes. Bite by a venomous snake is characterized by stinging pain at the wound puncture. The venom injected at the site of the bite is capable of producing a deleterious effect on the blood or on the nervous system. (Webster's 3d ed; from Dorland, 27th ed, at snake, venomous)"),
  'D012910': NtO(MH='Snake Venoms', MS='Solutions or mixtures of toxic and nontoxic substances elaborated by snake (Ophidia) salivary glands for the purpose of killing prey or disabling predators and delivered by grooved or hollow fangs. They usually contain enzymes, toxins, and other factors.'),
  'D012911': NtO(MH='Snakes', MS='Limbless REPTILES of the suborder Serpentes.'),
  'D012912': NtO(MH='Sneezing', MS='The sudden, forceful, involuntary expulsion of air from the NOSE and MOUTH caused by irritation to the MUCOUS MEMBRANES of the upper RESPIRATORY TRACT.'),
  'D012913': NtO(MH='Snoring', MS='Rough, noisy breathing during sleep, due to vibration of the uvula and soft palate.'),
  'D012914': NtO(MH='Snow', MS='Frozen water crystals that fall from the ATMOSPHERE.'),
  'D012915': NtO(MH='Soaps', MS='Sodium or potassium salts of long chain fatty acids. These detergent substances are obtained by boiling natural oils or fats with caustic alkali. Sodium soaps are harder and are used as topical anti-infectives and vehicles in pills and liniments; potassium soaps are soft, used as vehicles for ointments and also as topical antimicrobials.'),
  'D012916': NtO(MH='Soccer', MS='A game in which a round inflated ball is advanced by kicking or propelling with any part of the body except the hands or arms. The object of the game is to place the ball in opposite goals.'),
  'D012917': NtO(MH='Social Adjustment', MS='Adaptation of the person to the social environment. Adjustment may take place by adapting the self to the environment or by changing the environment. (From Campbell, Psychiatric Dictionary, 1996)'),
  'D012918': NtO(MH='Social Alienation', MS='The state of estrangement individuals feel in cultural settings that they view as foreign, unpredictable, or unacceptable.'),
  'D012919': NtO(MH='Social Behavior', MS='Any behavior caused by or affecting another individual or group usually of the same species.'),
  'D012920': NtO(MH='Social Behavior Disorders', MS='Behaviors which are at variance with the expected social norm and which affect other individuals.'),
  'D012922': NtO(MH='Social Change', MS='A shift, alteration, modification, or evolution of human behavior and culture, which over time, results in measurable consequences on societal values and norms.'),
  'D012923': NtO(MH='Social Class', MS='A stratum of people with similar position and prestige; includes social stratification. Social class is measured by criteria such as education, occupation, and income.'),
  'D012924': NtO(MH='Social Conditions', MS='The state of society as it exists or in flux. While it usually refers to society as a whole in a specified geographical or political region, it is applicable also to restricted strata of a society.'),
  'D012925': NtO(MH='Social Conformity', MS='Behavioral or attitudinal compliance with recognized social patterns or standards.'),
  'D012926': NtO(MH='Social Control, Formal', MS='Control which is exerted by the more stable organizations of society, such as established institutions and the law. They are ordinarily embodied in definite codes, usually written.'),
  'D012927': NtO(MH='Social Control, Informal', MS='Those forms of control which are exerted in less concrete and tangible ways, as through folkways, mores, conventions, and public sentiment.'),
  'D012928': NtO(MH='Social Desirability', MS='A personality trait rendering the individual acceptable in social or interpersonal relations. It is related to social acceptance, social approval, popularity, social status, leadership qualities, or any quality making him a socially desirable companion.'),
  'D012929': NtO(MH='Social Distance', MS='The degree of closeness or acceptance an individual or group feels toward another individual or group.'),
  'D012930': NtO(MH='Social Dominance', MS='Social structure of a group as it relates to the relative social rank of dominance status of its members. (APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D012931': NtO(MH='Social Environment', MS='The aggregate of social and cultural institutions, forms, patterns, and processes that influence the life of an individual or community.'),
  'D012932': NtO(MH='Social Facilitation', MS='Any enhancement of a motivated behavior in which individuals do the same thing with some degree of mutual stimulation and consequent coordination.'),
  'D012933': NtO(MH='Social Identification', MS='The process by which an aspect of self image is developed based on in-group preference or ethnocentrism and a perception of belonging to a social or cultural group. (From APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D012934': NtO(MH='Social Isolation', MS='The separation of individuals or groups resulting in the lack of or minimizing of social contact and/or communication. This separation may be accomplished by physical separation, by social barriers and by psychological mechanisms. In the latter, there may be interaction but no real communication.'),
  'D012935': NtO(MH='Social Justice', MS='An interactive process whereby members of a community are concerned for the equality and rights of all.'),
  'D012936': NtO(MH='Social Medicine', MS='A branch of medicine concerned with the role of socio-environmental factors in the occurrence, prevention and treatment of disease.'),
  'D012937': NtO(MH='Social Mobility', MS='The movement or shifting of membership between or within social classes by individuals or by groups.'),
  'D012938': NtO(MH='Social Perception', MS="The perceiving of attributes, characteristics, and behaviors of one's associates or social groups."),
  'D012939': NtO(MH='Social Planning', MS='A systematic process for evaluating and identifying specific needs in the community and the implementation of practical steps for meeting those needs and improving quality of life.'),
  'D012940': NtO(MH='Social Problems', MS='Situations affecting a significant number of people, that are believed to be sources of difficulty or threaten the stability of the community, and that require programs of amelioration.'),
  'D012941': NtO(MH='Social Responsibility', MS='The obligations and accountability assumed in carrying out actions or ideas on behalf of others.'),
  'D012942': NtO(MH='Social Sciences', MS='Disciplines concerned with the interrelationships of individuals in a social environment including social organizations and institutions. Includes Sociology and Anthropology.'),
  'D012943': NtO(MH='Social Security', MS='Government sponsored social insurance programs.'),
  'D012944': NtO(MH='Social Support', MS='Support systems that provide assistance and encouragement to individuals with physical or emotional disabilities in order that they may better cope. Informal social support is usually provided by friends, relatives, or peers, while formal assistance is provided by churches, groups, etc.'),
  'D012945': NtO(MH='Social Values', MS='Abstract standards or empirical variables in social life which are believed to be important and/or desirable.'),
  'D012946': NtO(MH='Social Welfare', MS='Organized institutions which provide services to ameliorate conditions of need or social pathology in the community.'),
  'D012947': NtO(MH='Social Work', MS='The use of community resources, individual case work, or group work to promote the adaptive capacities of individuals in relation to their social and economic environments. It includes social service agencies.'),
  'D012948': NtO(MH='Social Work Department, Hospital', MS='Hospital department responsible for administering and providing social services to patients and their families.'),
  'D012949': NtO(MH='Social Work, Psychiatric', MS='Use of all social work processes in the treatment of patients in a psychiatric or mental health setting.'),
  'D012950': NtO(MH='Socialism', MS='A system of government in which means of production and distribution of goods are controlled by the state.'),
  'D012951': NtO(MH='Socialization', MS='The training or molding of an individual through various relationships, educational agencies, and social controls, which enables him to become a member of a particular society.'),
  'D012952': NtO(MH='Societies', MS='Organizations composed of members with common interests and whose professions may be similar.'),
  'D012953': NtO(MH='Societies, Dental', MS='Societies whose membership is limited to dentists.'),
  'D012954': NtO(MH='Societies, Hospital', MS='Societies having institutional membership limited to hospitals and other health care institutions.'),
  'D012955': NtO(MH='Societies, Medical', MS='Societies whose membership is limited to physicians.'),
  'D012956': NtO(MH='Societies, Nursing', MS='Societies whose membership is limited to nurses.'),
  'D012957': NtO(MH='Societies, Pharmaceutical', MS='Societies whose membership is limited to pharmacists.'),
  'D012958': NtO(MH='Societies, Scientific', MS='Societies whose membership is limited to scientists.'),
  'D012959': NtO(MH='Socioeconomic Factors', MS='Social and economic factors that characterize the individual or group within the social structure.'),
  'D012960': NtO(MH='Socioenvironmental Therapy', MS='Therapy whose primary emphasis is on the physical and social structuring of the environment to promote interpersonal relationships which will be influential in reducing behavioral disturbances of patients.'),
  'D012961': NtO(MH='Sociology', MS='A social science dealing with group relationships, patterns of collective behavior, and social organization.'),
  'D012962': NtO(MH='Sociology, Medical', MS='The study of the social determinants and social effects of health and disease, and of the social structure of medical institutions or professions.'),
  'D012963': NtO(MH='Sociometric Techniques', MS='Methods for quantitatively assessing and measuring interpersonal and group relationships.'),
  'D012964': NtO(MH='Sodium', MS='A member of the alkali group of metals. It has the atomic symbol Na, atomic number 11, and atomic weight 23.'),
  'D012965': NtO(MH='Sodium Chloride', MS='A ubiquitous sodium salt that is commonly used to season food.'),
  'D012966': NtO(MH='Sodium Cyanide', MS='A highly poisonous compound that is an inhibitor of many metabolic processes and is used as a test reagent for the function of chemoreceptors. It is also used in many industrial processes.'),
  'D012967': NtO(MH='Sodium Dodecyl Sulfate', MS='An anionic surfactant, usually a mixture of sodium alkyl sulfates, mainly the lauryl; lowers surface tension of aqueous solutions; used as fat emulsifier, wetting agent, detergent in cosmetics, pharmaceuticals and toothpastes; also as research tool in protein biochemistry.'),
  'D012968': NtO(MH='Etidronic Acid', MS='A diphosphonate which affects calcium metabolism. It inhibits ectopic calcification and slows down bone resorption and bone turnover.'),
  'D012969': NtO(MH='Sodium Fluoride', MS='A source of inorganic fluoride which is used topically to prevent dental caries.'),
  'D012970': NtO(MH='Sodium Glutamate', MS='One of the FLAVORING AGENTS used to impart a meat-like flavor.'),
  'D012971': NtO(MH='Gold Sodium Thiosulfate', MS='A sodium salt of gold thiosulfate. It has uses like the ORGANOGOLD COMPOUNDS.'),
  'D012972': NtO(MH='Sodium Hydroxide', MS='A highly caustic substance that is used to neutralize acids and make sodium salts. (From Merck Index, 11th ed)'),
  'D012973': NtO(MH='Sodium Hypochlorite', MS="It is used as an oxidizing and bleaching agent and as a disinfectant. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D012974': NtO(MH='Sodium Iodide', MS='A compound forming white, odorless deliquescent crystals and used as iodine supplement, expectorant or in its radioactive (I-131) form as an diagnostic aid, particularly for thyroid function tests.'),
  'D012975': NtO(MH='Sodium Isotopes', MS='Stable sodium atoms that have the same atomic number as the element sodium, but differ in atomic weight. Na-23 is a stable sodium isotope.'),
  'D012976': NtO(MH='Sodium Morrhuate', MS='The sodium salts of the fatty acids in cod liver oil; an irritant and sclerosing agent used to treat varicose veins and arthritic joints.'),
  'D012977': NtO(MH='Sodium Nitrite', MS='Nitrous acid sodium salt. Used in many industrial processes, in meat curing, coloring, and preserving, and as a reagent in ANALYTICAL CHEMISTRY TECHNIQUES. It is used therapeutically as an antidote in cyanide poisoning. The compound is toxic and mutagenic and will react in vivo with secondary or tertiary amines thereby producing highly carcinogenic nitrosamines.'),
  'D012978': NtO(MH='Sodium Oxybate', MS='The sodium salt of 4-hydroxybutyric acid. It is used for both induction and maintenance of ANESTHESIA.'),
  'D012979': NtO(MH='Sodium Radioisotopes', MS='Unstable isotopes of sodium that decay or disintegrate emitting radiation. Na atoms with atomic weights 20-22 and 24-26 are radioactive sodium isotopes.'),
  'D012980': NtO(MH='Sodium Salicylate', MS='A non-steroidal anti-inflammatory agent that is less effective than equal doses of ASPIRIN in relieving pain and reducing fever. However, individuals who are hypersensitive to ASPIRIN may tolerate sodium salicylate. In general, this salicylate produces the same adverse reactions as ASPIRIN, but there is less occult gastrointestinal bleeding. (From AMA Drug Evaluations Annual, 1992, p120)'),
  'D012981': NtO(MH='Sodium Tetradecyl Sulfate', MS='An anionic surface-active agent used for its wetting properties in industry and used in medicine as an irritant and sclerosing agent for hemorrhoids and varicose veins.'),
  'D012982': NtO(MH='Sodium, Dietary', MS='Sodium or sodium compounds used in foods or as a food. The most frequently used compounds are sodium chloride or sodium glutamate.'),
  'D012983': NtO(MH='Soft Tissue Neoplasms', MS='Neoplasms of whatever cell type or origin, occurring in the extraskeletal connective tissue framework of the body including the organs of locomotion and their various component structures, such as nerves, blood vessels, lymphatics, etc.'),
  'D012984': NtO(MH='Software', MS='Sequential operating programs and data which instruct the functioning of a digital computer.'),
  'D012985': NtO(MH='Software Design', MS='Specifications and instructions applied to the software.'),
  'D012986': NtO(MH='Software Validation', MS='The act of testing the software for compliance with a standard.'),
  'D012987': NtO(MH='Soil', MS='The unconsolidated mineral or organic matter on the surface of the earth that serves as a natural medium for the growth of land plants.'),
  'D012988': NtO(MH='Soil Microbiology', MS='The presence of bacteria, viruses, and fungi in the soil. This term is not restricted to pathogenic organisms.'),
  'D012989': NtO(MH='Soil Pollutants', MS='Substances which pollute the soil. Use for soil pollutants in general or for which there is no specific heading.'),
  'D012990': NtO(MH='Soil Pollutants, Radioactive', MS='Pollutants, present in soil, which exhibit radioactivity.'),
  'D012991': NtO(MH='Solanaceous Alkaloids', MS='Alkaloids, mainly tropanes, elaborated by plants of the family Solanaceae, including Atropa, Hyoscyamus, Mandragora, Nicotiana, Solanum, etc. Some act as cholinergic antagonists; most are very toxic; many are used medicinally.'),
  'D012992': NtO(MH='Solanine', MS='A mixture of alpha-chaconine and alpha-solanine, found in SOLANACEAE plants.'),
  'D012993': NtO(MH='Solar Energy', MS='Energy transmitted from the sun in the form of electromagnetic radiation.'),
  'D012994': NtO(MH='Dental Soldering', MS='The joining of pieces of metal through the use of an alloy which has a lower melting point, usually at least 100 degrees Celsius below the fusion temperature of the parts being soldered. In dentistry, soldering is used for joining components of a dental appliance, as in assembling a bridge, joining metals to orthodontic bands, or adding to the bulk of certain structures, such as the establishment of proper contact areas on inlays and crowns with adjacent teeth. (Illustrated Dictionary of Dentistry, 1982)'),
  'D012995': NtO(MH='Solubility', MS='The ability of a substance to be dissolved, i.e. to form a solution with another substance. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D012996': NtO(MH='Solutions', MS="The homogeneous mixtures formed by the mixing of a solid, liquid, or gaseous substance (solute) with a liquid (the solvent), from which the dissolved substances can be recovered by physical processes. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D012997': NtO(MH='Solvents', MS="Liquids that dissolve other substances (solutes), generally solids, without any change in chemical composition, as, water containing sugar. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D012998': NtO(MH='Somalia', MS="Somalia is located on the east coast of Africa on and north of the Equator and, with Ethiopia, Eritrea, Djibouti, and Kenya, is often referred to as the Horn of Africa. It comprises Italy's former Trust Territory of Somalia and the former British Protectorate of Somaliland. The capital is Mogadishu."),
  'D012999': NtO(MH='Soman', MS='An organophosphorus compound that inhibits cholinesterase. It causes seizures and has been used as a chemical warfare agent.'),
  'D013000': NtO(MH='Psychiatric Somatic Therapies', MS='The biologic treatment of mental disorders (e.g., ELECTROCONVULSIVE THERAPY), in contrast with psychotherapy. (Stone, American Psychiatric Glossary, 1988, p159)'),
  'D013001': NtO(MH='Somatoform Disorders', MS='Disorders having the presence of physical symptoms that suggest a general medical condition but that are not fully explained by another medical condition, by the direct effects of a substance, or by another mental disorder. The MEDICALLY UNEXPLAINED SYMPTOMS must cause clinically significant distress or impairment in social, occupational, or other areas of functioning. In contrast to FACTITIOUS DISORDERS and MALINGERING, the physical symptoms are not under voluntary control. (APA, DSM-V)'),
  'D013002': NtO(MH='Somatomedins', MS='Insulin-like polypeptides made by the liver and some fibroblasts and released into the blood when stimulated by SOMATOTROPIN. They cause sulfate incorporation into collagen, RNA, and DNA synthesis, which are prerequisites to cell division and growth of the organism.'),
  'D013003': NtO(MH='Somatosensory Cortex', MS='Area of the parietal lobe concerned with receiving sensations such as movement, pain, pressure, position, temperature, touch, and vibration. It lies posterior to the central sulcus.'),
  'D013004': NtO(MH='Somatostatin', MS='A 14-amino acid peptide named for its ability to inhibit pituitary GROWTH HORMONE release, also called somatotropin release-inhibiting factor. It is expressed in the central and peripheral nervous systems, the gut, and other organs. SRIF can also inhibit the release of THYROID-STIMULATING HORMONE; PROLACTIN; INSULIN; and GLUCAGON besides acting as a neurotransmitter and neuromodulator. In a number of species including humans, there is an additional form of somatostatin, SRIF-28 with a 14-amino acid extension at the N-terminal.'),
  'D013005': NtO(MH='Somatostatinoma', MS='A SOMATOSTATIN-secreting tumor derived from the pancreatic delta cells (SOMATOSTATIN-SECRETING CELLS). It is also found in the INTESTINE. Somatostatinomas are associated with DIABETES MELLITUS; CHOLELITHIASIS; STEATORRHEA; and HYPOCHLORHYDRIA. The majority of somatostatinomas have the potential for METASTASIS.'),
  'D013006': NtO(MH='Growth Hormone', MS='A polypeptide that is secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR). Growth hormone, also known as somatotropin, stimulates mitosis, cell differentiation and cell growth. Species-specific growth hormones have been synthesized.'),
  'D013007': NtO(MH='Growth Hormone-Releasing Hormone', MS='A peptide of 44 amino acids in most species that stimulates the release and synthesis of GROWTH HORMONE. GHRF (or GRF) is synthesized by neurons in the ARCUATE NUCLEUS of the HYPOTHALAMUS. After being released into the pituitary portal circulation, GHRF stimulates GH release by the SOMATOTROPHS in the PITUITARY GLAND.'),
  'D013008': NtO(MH='Somatotypes', MS='Particular categories of body build, determined on the basis of certain physical characteristics. The three basic body types are ectomorph (thin physique), endomorph (rounded physique), and mesomorph (athletic physique).'),
  'D013009': NtO(MH='Somnambulism', MS='A parasomnia characterized by a partial arousal that occurs during stage IV of non-REM sleep. Affected individuals exhibit semipurposeful behaviors such as ambulation and are difficult to fully awaken. Children are primarily affected, with a peak age range of 4-6 years.'),
  'D013010': NtO(MH='Sonication', MS='The application of high intensity ultrasound to liquids.'),
  'D013011': NtO(MH='Sorbic Acid', MS='Mold and yeast inhibitor. Used as a fungistatic agent for foods, especially cheeses.'),
  'D013012': NtO(MH='Sorbitol', MS='A polyhydric alcohol with about half the sweetness of sucrose. Sorbitol occurs naturally and is also produced synthetically from glucose. It was formerly used as a diuretic and may still be used as a laxative and in irrigating solutions for some surgical procedures. It is also used in many manufacturing processes, as a pharmaceutical aid, and in several research applications.'),
  'D013013': NtO(MH='Sorbose', MS='A ketose sugar that is commonly used in the commercial synthesis of ASCORBIC ACID.'),
  'D013014': NtO(MH='SOS Response, Genetics', MS='An error-prone mechanism or set of functions for repairing damaged microbial DNA. SOS functions (a concept reputedly derived from the SOS of the international distress signal) are involved in DNA repair and mutagenesis, in cell division inhibition, in recovery of normal physiological conditions after DNA repair, and possibly in cell death when DNA damage is extensive.'),
  'D013015': NtO(MH='Sotalol', MS='An adrenergic beta-antagonist that is used in the treatment of life-threatening arrhythmias.'),
  'D013016': NtO(MH='Sound', MS='A type of non-ionizing radiation in which energy is transmitted through solid, liquid, or gas as compression waves. Sound (acoustic or sonic) radiation with frequencies above the audible range is classified as ultrasonic. Sound radiation below the audible range is classified as infrasonic.'),
  'D013017': NtO(MH='Sound Localization', MS='Ability to determine the specific location of a sound source.'),
  'D013018': NtO(MH='Sound Spectrography', MS='The graphic registration of the frequency and intensity of sounds, such as speech, infant crying, and animal vocalizations.'),
  'D013019': NtO(MH='South Africa', MS='A republic in southern Africa, the southernmost part of Africa. It has three capitals: Pretoria (administrative), Cape Town (legislative), and Bloemfontein (judicial). Officially the Republic of South Africa since 1960, it was called the Union of South Africa 1910-1960.'),
  'D013020': NtO(MH='South America', MS='The southern continent of the Western Hemisphere, extending southward from the Colombia-Panama border.'),
  'D013021': NtO(MH='South Australia', MS="A state in south central Australia. Its capital is Adelaide. It was probably first visited by F. Thyssen in 1627. Later discoveries in 1802 and 1830 opened up the southern part. It became a British province in 1836 with this self-descriptive name and became a state in 1901. (From Webster's New Geographical Dictionary, 1988, p1135)"),
  'D013022': NtO(MH='South Carolina', MS='State bounded on the north by North Carolina, on the east and south by the Atlantic Ocean, and on the west by Georgia.'),
  'D013023': NtO(MH='South Dakota', MS='State bounded on the north by North Dakota, on the east by Minnesota, on the south by Nebraska, and on the west by Montana and Wyoming.'),
  'D013024': NtO(MH='Soybean Oil', MS='Oil from soybean or soybean plant.'),
  'D013025': NtO(MH='Soybeans', MS='An annual legume. The SEEDS of this plant are edible and used to produce a variety of SOY FOODS.'),
  'D013026': NtO(MH='Space Flight', MS="Travel beyond the earth's atmosphere."),
  'D013027': NtO(MH='Space Maintenance, Orthodontic', MS='Common treatment method used in pediatric dentistry that involves the use of fixed or removable space maintainers in order to keep the space left by a baby tooth safe in the event it is lost early. Space maintainers also help permanent teeth to grow safely.'),
  'D013028': NtO(MH='Space Perception', MS='The awareness of the spatial properties of objects; includes physical space.'),
  'D013029': NtO(MH='Space-Time Clustering', MS='A statistically significant excess of cases of a disease, occurring within a limited space-time continuum.'),
  'D013030': NtO(MH='Spain', MS='Country located between France on the northeast and Portugal on the west and bordered by the Atlantic Ocean and the Mediterranean Sea. The capital is Madrid.'),
  'D013031': NtO(MH='Sparganosis', MS='Infection of animals, including fish and man, with a developmental stage of Diphyllobothrium. This stage has recently been referred to as a plerocercoid but the name sparganum has persisted. Therefore, infection of fish or other animals with the plerocercoid larvae is sparganosis. Fish-eating mammals, including man, are the final hosts.'),
  'D013032': NtO(MH='Sparganum', MS='The larval form of the diphyllobothriid tapeworms of the genus DIPHYLLOBOTHRIUM and SPIROMETRA. Fish-eating mammals and man are the final hosts.'),
  'D013033': NtO(MH='Sparsomycin', MS='An antitumor antibiotic produced by Streptomyces sparsogenes. It inhibits protein synthesis in 70S and 80S ribosomal systems.'),
  'D013034': NtO(MH='Sparteine', MS='A quinolizidine alkaloid isolated from several FABACEAE including LUPINUS; SPARTIUM; and CYTISUS. It has been used as an oxytocic and an anti-arrhythmia agent. It has also been of interest as an indicator of CYP2D6 genotype.'),
  'D013035': NtO(MH='Spasm', MS='An involuntary contraction of a muscle or group of muscles. Spasms may involve SKELETAL MUSCLE or SMOOTH MUSCLE.'),
  'D013036': NtO(MH='Spasms, Infantile', MS='An epileptic syndrome characterized by the triad of infantile spasms, hypsarrhythmia, and arrest of psychomotor development at seizure onset. The majority present between 3-12 months of age, with spasms consisting of combinations of brief flexor or extensor movements of the head, trunk, and limbs. The condition is divided into two forms: cryptogenic (idiopathic) and symptomatic (secondary to a known disease process such as intrauterine infections; nervous system abnormalities; BRAIN DISEASES, METABOLIC, INBORN; prematurity; perinatal asphyxia; TUBEROUS SCLEROSIS; etc.). (From Menkes, Textbook of Child Neurology, 5th ed, pp744-8)'),
  'D013037': NtO(MH='Spatial Behavior', MS='Reactions of an individual or groups of individuals with relation to the immediate surrounding area including the animate or inanimate objects within that area.'),
  'D013038': NtO(MH='Specialization', MS='An occupation limited in scope to a subsection of a broader field.'),
  'D013039': NtO(MH='Specialties, Dental', MS='Various branches of dental practice limited to specialized areas.'),
  'D013041': NtO(MH='Specialties, Nursing', MS='Various branches of nursing practice limited to specialized areas.'),
  'D013043': NtO(MH='Specialties, Surgical', MS='Various branches of surgical practice limited to specialized areas.'),
  'D013044': NtO(MH='Specialty Boards', MS='Organizations which certify physicians and dentists as specialists in various fields of medical and dental practice.'),
  'D013045': NtO(MH='Species Specificity', MS='The restriction of a characteristic behavior, anatomical structure or physical system, such as immune response; metabolic response, or gene or gene variant to the members of one species. It refers to that property which differentiates one species from another but it is also used for phylogenetic levels higher or lower than the species.'),
  'D013046': NtO(MH='Specific Gravity', MS='The ratio of the density of a material to the density of some standard material, such as water or air, at a specified temperature.'),
  'D013047': NtO(MH='Specific Pathogen-Free Organisms', MS='Animals or humans raised in the absence of a particular disease-causing virus or other microorganism. Less frequently plants are cultivated pathogen-free.'),
  'D013048': NtO(MH='Specimen Handling', MS='Procedures for collecting, preserving, and transporting of specimens sufficiently stable to provide accurate and precise results suitable for clinical interpretation.'),
  'D013049': NtO(MH='Spectrin', MS='A high molecular weight (220-250 kDa) water-soluble protein which can be extracted from erythrocyte ghosts in low ionic strength buffers. The protein contains no lipids or carbohydrates, is the predominant species of peripheral erythrocyte membrane proteins, and exists as a fibrous coating on the inner, cytoplasmic surface of the membrane.'),
  'D013050': NtO(MH='Spectrometry, Fluorescence', MS='Measurement of the intensity and quality of fluorescence.'),
  'D013051': NtO(MH='Spectrometry, Gamma', MS='Determination of the energy distribution of gamma rays emitted by nuclei. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013052': NtO(MH='Spectrometry, X-Ray Emission', MS='The spectrometric analysis of fluorescent X-RAYS, i.e. X-rays emitted after bombarding matter with high energy particles such as PROTONS; ELECTRONS; or higher energy X-rays. Identification of ELEMENTS by this technique is based on the specific type of X-rays that are emitted which are characteristic of the specific elements in the material being analyzed. The characteristic X-rays are distinguished and/or quantified by either wavelength dispersive or energy dispersive methods.'),
  'D013053': NtO(MH='Spectrophotometry', MS='The art or process of comparing photometrically the relative intensities of the light in different parts of the spectrum.'),
  'D013054': NtO(MH='Spectrophotometry, Atomic', MS='Spectrophotometric techniques by which the absorption or emmision spectra of radiation from atoms are produced and analyzed.'),
  'D013055': NtO(MH='Spectrophotometry, Infrared', MS='Spectrophotometry in the infrared region, usually for the purpose of chemical analysis through measurement of absorption spectra associated with rotational and vibrational energy levels of molecules. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013056': NtO(MH='Spectrophotometry, Ultraviolet', MS='Determination of the spectra of ultraviolet absorption by specific molecules in gases or liquids, for example Cl2, SO2, NO2, CS2, ozone, mercury vapor, and various unsaturated compounds. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013057': NtO(MH='Spectrum Analysis', MS='The measurement of the amplitude of the components of a complex waveform throughout the frequency range of the waveform. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D013058': NtO(MH='Mass Spectrometry', MS='An analytical method used in determining the identity of a chemical based on its mass using mass analyzers/mass spectrometers.'),
  'D013059': NtO(MH='Spectrum Analysis, Raman', MS='Analysis of the intensity of Raman scattering of monochromatic light as a function of frequency of the scattered light.'),
  'D013060': NtO(MH='Speech', MS='Communication through a system of conventional vocal symbols.'),
  'D013061': NtO(MH='Speech Acoustics', MS='The acoustic aspects of speech in terms of frequency, intensity, and time.'),
  'D013062': NtO(MH='Speech Articulation Tests', MS='Tests of accuracy in pronouncing speech sounds, e.g., Iowa Pressure Articulation Test, Deep Test of Articulation, Templin-Darley Tests of Articulation, Goldman-Fristoe Test of Articulation, Screening Speech Articulation Test, Arizona Articulation Proficiency Scale.'),
  'D013063': NtO(MH='Speech Discrimination Tests', MS='Tests of the ability to hear and understand speech as determined by scoring the number of words in a word list repeated correctly.'),
  'D013064': NtO(MH='Speech Disorders', MS='Acquired or developmental conditions marked by an impaired ability to comprehend or generate spoken forms of language.'),
  'D013065': NtO(MH='Speech Intelligibility', MS='Ability to make speech sounds that are recognizable.'),
  'D013066': NtO(MH='Speech-Language Pathology', MS='The study of speech or language disorders and their diagnosis and correction.'),
  'D013067': NtO(MH='Speech Perception', MS='The process whereby an utterance is decoded into a representation in terms of linguistic units (sequences of phonetic segments which combine to form lexical and grammatical morphemes).'),
  'D013068': NtO(MH='Speech Production Measurement', MS='Measurement of parameters of the speech product such as vocal tone, loudness, pitch, voice quality, articulation, resonance, phonation, phonetic structure and prosody.'),
  'D013069': NtO(MH='Speech Reception Threshold Test', MS='A test to determine the lowest sound intensity level at which fifty percent or more of the spondaic test words (words of two syllables having equal stress) are repeated correctly.'),
  'D013070': NtO(MH='Speech Therapy', MS='Treatment for individuals with speech defects and disorders that involves counseling and use of various exercises and aids to help the development of new speech habits.'),
  'D013071': NtO(MH='Speech, Alaryngeal', MS='Methods of enabling a patient without a larynx or with a non-functional larynx to produce voice or speech. The methods may be pneumatic or electronic.'),
  'D013072': NtO(MH='Speech, Esophageal', MS='A method of speech used after laryngectomy, with sound produced by vibration of the column of air in the esophagus against the contracting cricopharyngeal sphincter. (Dorland, 27th ed)'),
  'D013073': NtO(MH='Sperm Agglutination', MS='Agglutination of spermatozoa by antibodies or autoantibodies.'),
  'D013074': NtO(MH='Sperm Banks', MS='Centers for acquiring and storing semen.'),
  'D013075': NtO(MH='Sperm Capacitation', MS='The structural and functional changes by which SPERMATOZOA become capable of oocyte FERTILIZATION. It normally requires exposing the sperm to the female genital tract for a period of time to bring about increased SPERM MOTILITY and the ACROSOME REACTION before fertilization in the FALLOPIAN TUBES can take place.'),
  'D013076': NtO(MH='Sperm Count', MS='A count of SPERM in the ejaculum, expressed as number per milliliter.'),
  'D013077': NtO(MH='Sperm Head', MS='The anterior portion of the spermatozoon (SPERMATOZOA) that contains mainly the nucleus with highly compact CHROMATIN material.'),
  'D013078': NtO(MH='Sperm Immobilizing Agents', MS='Chemical substances with sperm immobilizing activity used as topically administered vaginal contraceptives.'),
  'D013079': NtO(MH='Sperm Maturation', MS='The maturing process of SPERMATOZOA after leaving the testicular SEMINIFEROUS TUBULES. Maturation in SPERM MOTILITY and FERTILITY takes place in the EPIDIDYMIS as the sperm migrate from caput epididymis to cauda epididymis.'),
  'D013081': NtO(MH='Sperm Motility', MS='Movement characteristics of SPERMATOZOA in a fresh specimen. It is measured as the percentage of sperms that are moving, and as the percentage of sperms with productive flagellar motion such as rapid, linear, and forward progression.'),
  'D013082': NtO(MH='Sperm Tail', MS='The posterior filiform portion of the spermatozoon (SPERMATOZOA) that provides sperm motility.'),
  'D013083': NtO(MH='Sperm Transport', MS='Passive or active movement of SPERMATOZOA from the testicular SEMINIFEROUS TUBULES through the male reproductive tract as well as within the female reproductive tract.'),
  'D013084': NtO(MH='Sperm-Ovum Interactions', MS='Interactive processes between the oocyte (OVUM) and the sperm (SPERMATOZOA) including sperm adhesion, ACROSOME REACTION, sperm penetration of the ZONA PELLUCIDA, and events leading to FERTILIZATION.'),
  'D013085': NtO(MH='Spermatic Cord', MS='Either of a pair of tubular structures formed by DUCTUS DEFERENS; ARTERIES; VEINS; LYMPHATIC VESSELS; and nerves. The spermatic cord extends from the deep inguinal ring through the INGUINAL CANAL to the TESTIS in the SCROTUM.'),
  'D013086': NtO(MH='Spermatic Cord Torsion', MS='The twisting of the SPERMATIC CORD due to an anatomical abnormality that left the TESTIS mobile and dangling in the SCROTUM. The initial effect of testicular torsion is obstruction of venous return. Depending on the duration and degree of cord rotation, testicular symptoms range from EDEMA to interrupted arterial flow and testicular pain. If blood flow to testis is absent for 4 to 6 h, SPERMATOGENESIS may be permanently lost.'),
  'D013087': NtO(MH='Spermatids', MS='Male germ cells derived from the haploid secondary SPERMATOCYTES. Without further division, spermatids undergo structural changes and give rise to SPERMATOZOA.'),
  'D013088': NtO(MH='Spermatocele', MS='A cystic dilation of the EPIDIDYMIS, usually in the head portion (caput epididymis). The cyst fluid contains dead SPERMATOZOA and can be easily differentiated from TESTICULAR HYDROCELE and other testicular lesions.'),
  'D013089': NtO(MH='Spermatocidal Agents', MS='Chemical substances that are destructive to spermatozoa used as topically administered vaginal contraceptives.'),
  'D013090': NtO(MH='Spermatocytes', MS='Male germ cells derived from SPERMATOGONIA. The euploid primary spermatocytes undergo MEIOSIS and give rise to the haploid secondary spermatocytes which in turn give rise to SPERMATIDS.'),
  'D013091': NtO(MH='Spermatogenesis', MS='The process of germ cell development in the male from the primordial germ cells, through SPERMATOGONIA; SPERMATOCYTES; SPERMATIDS; to the mature haploid SPERMATOZOA.'),
  'D013092': NtO(MH='Spermatogenesis-Blocking Agents', MS='Chemical substances which inhibit the process of spermatozoa formation at either the first stage, in which spermatogonia develop into spermatocytes and then into spermatids, or the second stage, in which spermatids transform into spermatozoa.'),
  'D013093': NtO(MH='Spermatogonia', MS='Euploid male germ cells of an early stage of SPERMATOGENESIS, derived from prespermatogonia. With the onset of puberty, spermatogonia at the basement membrane of the seminiferous tubule proliferate by mitotic then meiotic divisions and give rise to the haploid SPERMATOCYTES.'),
  'D013094': NtO(MH='Spermatozoa', MS='Mature male germ cells derived from SPERMATIDS. As spermatids move toward the lumen of the SEMINIFEROUS TUBULES, they undergo extensive structural changes including the loss of cytoplasm, condensation of CHROMATIN into the SPERM HEAD, formation of the ACROSOME cap, the SPERM MIDPIECE and the SPERM TAIL that provides motility.'),
  'D013095': NtO(MH='Spermidine', MS='A polyamine formed from putrescine. It is found in almost all tissues in association with nucleic acids. It is found as a cation at all pH values, and is thought to help stabilize some membranes and nucleic acid structures. It is a precursor of spermine.'),
  'D013096': NtO(MH='Spermine', MS='A biogenic polyamine formed from spermidine. It is found in a wide variety of organisms and tissues and is an essential growth factor in some bacteria. It is found as a polycation at all pH values. Spermine is associated with nucleic acids, particularly in viruses, and is thought to stabilize the helical structure.'),
  'D013097': NtO(MH='Spermine Synthase', MS="An enzyme that catalyzes the transfer of the propylamine moiety from 5'-deoxy-5'-S-(3-methylthiopropylamine)sulfonium adenosine to spermidine in the biosynthesis of spermine. It has an acidic isoelectric point at pH 5.0. EC 2.5.1.22."),
  'D013098': NtO(MH='Xylariales', MS='An order of ascomycetous FUNGI which includes many economically important plant parasites as well as saprophytes.'),
  'D013100': NtO(MH='Sphenoid Bone', MS='An irregular unpaired bone situated at the SKULL BASE and wedged between the frontal, temporal, and occipital bones (FRONTAL BONE; TEMPORAL BONE; OCCIPITAL BONE). Sphenoid bone consists of a median body and three pairs of processes resembling a bat with spread wings. The body is hollowed out in its inferior to form two large cavities (SPHENOID SINUS).'),
  'D013101': NtO(MH='Sphenoid Sinus', MS='One of the paired air spaces located in the body of the SPHENOID BONE behind the ETHMOID BONE in the middle of the skull. Sphenoid sinus communicates with the posterosuperior part of NASAL CAVITY on the same side.'),
  'D013102': NtO(MH='Spherocytes', MS='Small, abnormal spherical red blood cells with more than the normal amount of hemoglobin.'),
  'D013103': NtO(MH='Spherocytosis, Hereditary', MS='A group of familial congenital hemolytic anemias characterized by numerous abnormally shaped erythrocytes which are generally spheroidal. The erythrocytes have increased osmotic fragility and are abnormally permeable to sodium ions.'),
  'D013104': NtO(MH='Spheroplasts', MS='Cells, usually bacteria or yeast, which have partially lost their cell wall, lost their characteristic shape and become round.'),
  'D013105': NtO(MH='Sphincterotomy, Transduodenal', MS='Surgical incision of the SPHINCTER OF ODDI in the management of COMMON BILE DUCT DISEASES (e.g., CHOLEDOCHOLITHIASIS; and SPHINCTER OF ODDI DYSFUNCTION).'),
  'D013106': NtO(MH='Sphingolipidoses', MS='A group of inherited metabolic disorders characterized by the intralysosomal accumulation of SPHINGOLIPIDS primarily in the CENTRAL NERVOUS SYSTEM and to a variable degree in the visceral organs. They are classified by the enzyme defect in the degradation pathway and the substrate accumulation (or storage). Clinical features vary in subtypes but neurodegeneration is a common sign.'),
  'D013107': NtO(MH='Sphingolipids', MS='A class of membrane lipids that have a polar head and two nonpolar tails. They are composed of one molecule of the long-chain amino alcohol sphingosine (4-sphingenine) or one of its derivatives, one molecule of a long-chain acid, a polar head alcohol and sometimes phosphoric acid in diester linkage at the polar head group. (Lehninger et al, Principles of Biochemistry, 2nd ed)'),
  'D013108': NtO(MH='Sphingomyelin Phosphodiesterase', MS='An enzyme that catalyzes the hydrolysis of sphingomyelin to ceramide (N-acylsphingosine) plus choline phosphate. A defect in this enzyme leads to NIEMANN-PICK DISEASE. EC 3.1.4.12.'),
  'D013109': NtO(MH='Sphingomyelins', MS='A class of sphingolipids found largely in the brain and other nervous tissue. They contain phosphocholine or phosphoethanolamine as their polar head group so therefore are the only sphingolipids classified as PHOSPHOLIPIDS.'),
  'D013110': NtO(MH='Sphingosine', MS='An amino alcohol with a long unsaturated hydrocarbon chain. Sphingosine and its derivative sphinganine are the major bases of the sphingolipids in mammals. (Dorland, 28th ed)'),
  'D013111': NtO(MH='Spider Venoms', MS='Venoms of arthropods of the order Araneida of the ARACHNIDA. The venoms usually contain several protein fractions, including ENZYMES, hemolytic, neurolytic, and other TOXINS, BIOLOGICAL.'),
  'D013112': NtO(MH='Spiders', MS='Arthropods of the class ARACHNIDA, order Araneae. Except for mites and ticks, spiders constitute the largest order of arachnids, with approximately 37,000 species having been described. The majority of spiders are harmless, although some species can be regarded as moderately harmful since their bites can lead to quite severe local symptoms. (From Barnes, Invertebrate Zoology, 5th ed, p508; Smith, Insects and Other Arthropods of Medical Importance, 1973, pp424-430)'),
  'D013113': NtO(MH='Spin Labels', MS='Molecules which contain an atom or a group of atoms exhibiting an unpaired electron spin that can be detected by electron spin resonance spectroscopy and can be bonded to another molecule. (McGraw-Hill Dictionary of Chemical and Technical Terms, 4th ed)'),
  'D013115': NtO(MH='Spinal Canal', MS='The cavity within the SPINAL COLUMN through which the SPINAL CORD passes.'),
  'D013116': NtO(MH='Spinal Cord', MS='A cylindrical column of tissue that lies within the vertebral canal. It is composed of WHITE MATTER and GRAY MATTER.'),
  'D013117': NtO(MH='Spinal Cord Compression', MS='Acute and chronic conditions characterized by external mechanical compression of the SPINAL CORD due to extramedullary neoplasm; EPIDURAL ABSCESS; SPINAL FRACTURES; bony deformities of the vertebral bodies; and other conditions. Clinical manifestations vary with the anatomic site of the lesion and may include localized pain, weakness, sensory loss, incontinence, and impotence.'),
  'D013118': NtO(MH='Spinal Cord Diseases', MS='Pathologic conditions which feature SPINAL CORD damage or dysfunction, including disorders involving the meninges and perimeningeal spaces surrounding the spinal cord. Traumatic injuries, vascular diseases, infections, and inflammatory/autoimmune processes may affect the spinal cord.'),
  'D013119': NtO(MH='Spinal Cord Injuries', MS='Penetrating and non-penetrating injuries to the spinal cord resulting from traumatic external forces (e.g., WOUNDS, GUNSHOT; WHIPLASH INJURIES; etc.).'),
  'D013120': NtO(MH='Spinal Cord Neoplasms', MS='Benign and malignant neoplasms which occur within the substance of the spinal cord (intramedullary neoplasms) or in the space between the dura and spinal cord (intradural extramedullary neoplasms). The majority of intramedullary spinal tumors are primary CNS neoplasms including ASTROCYTOMA; EPENDYMOMA; and LIPOMA. Intramedullary neoplasms are often associated with SYRINGOMYELIA. The most frequent histologic types of intradural-extramedullary tumors are MENINGIOMA and NEUROFIBROMA.'),
  'D013121': NtO(MH='Spinal Curvatures', MS='Deformities of the SPINE characterized by abnormal bending or flexure in the vertebral column. They may be bending forward (KYPHOSIS), backward (LORDOSIS), or sideway (SCOLIOSIS).'),
  'D013122': NtO(MH='Spinal Diseases', MS='Diseases involving the SPINE.'),
  'D013123': NtO(MH='Spinal Fusion', MS='Operative immobilization or ankylosis of two or more vertebrae by fusion of the vertebral bodies with a short bone graft or often with diskectomy or laminectomy. (From Blauvelt & Nelson, A Manual of Orthopaedic Terminology, 5th ed, p236; Dorland, 28th ed)'),
  'D013124': NtO(MH='Spinal Injuries', MS='Injuries involving the vertebral column.'),
  'D013125': NtO(MH='Spinal Neoplasms', MS='New abnormal growth of tissue in the SPINE.'),
  'D013126': NtO(MH='Spinal Nerve Roots', MS='Paired bundles of NERVE FIBERS entering and leaving the SPINAL CORD at each segment. The dorsal and ventral nerve roots join to form the mixed segmental spinal nerves. The dorsal roots are generally afferent, formed by the central projections of the spinal (dorsal root) ganglia sensory cells, and the ventral roots are efferent, comprising the axons of spinal motor and PREGANGLIONIC AUTONOMIC FIBERS.'),
  'D013127': NtO(MH='Spinal Nerves', MS='The 31 paired peripheral nerves formed by the union of the dorsal and ventral spinal roots from each spinal cord segment. The spinal nerve plexuses and the spinal roots are also included.'),
  'D013128': NtO(MH='Spinal Osteophytosis', MS='Outgrowth of immature bony processes or bone spurs (OSTEOPHYTE) from the VERTEBRAE, reflecting the presence of degenerative disease and calcification. It commonly occurs in cervical and lumbar SPONDYLOSIS.'),
  'D013129': NtO(MH='Spinal Puncture', MS='Tapping fluid from the subarachnoid space in the lumbar region, usually between the third and fourth lumbar vertebrae.'),
  'D013130': NtO(MH='Spinal Stenosis', MS='Narrowing of the spinal canal.'),
  'D013131': NtO(MH='Spine', MS='The spinal or vertebral column.'),
  'D013132': NtO(MH='Spinocerebellar Degenerations', MS='A heterogenous group of degenerative syndromes marked by progressive cerebellar dysfunction either in isolation or combined with other neurologic manifestations. Sporadic and inherited subtypes occur. Inheritance patterns include autosomal dominant, autosomal recessive, and X-linked.'),
  'D013133': NtO(MH='Spinothalamic Tracts', MS='A bundle of NERVE FIBERS connecting each posterior horn of the spinal cord to the opposite side of the THALAMUS, carrying information about pain, temperature, and touch. It is one of two major routes by which afferent spinal NERVE FIBERS carrying sensations of somaesthesis are transmitted to the THALAMUS.'),
  'D013134': NtO(MH='Spiperone', MS='A spiro butyrophenone analog similar to HALOPERIDOL and other related compounds. It has been recommended in the treatment of SCHIZOPHRENIA.'),
  'D013136': NtO(MH='Spiral Ganglion', MS='The sensory ganglion of the COCHLEAR NERVE. The cells of the spiral ganglion send fibers peripherally to the cochlear hair cells and centrally to the COCHLEAR NUCLEI of the BRAIN STEM.'),
  'D013137': NtO(MH='Spiral Lamina', MS='The bony plate which extends outwards from the modiolus into the spiral canal of the cochlea, forming part of the structure that divides the upper SCALA VESTIBULI and the lower SCALA TYMPANI.'),
  'D013139': NtO(MH='Spirillum', MS='A genus of gram-negative, curved and spiral-shaped bacteria found in stagnant, freshwater environments. These organisms are motile by bipolar tufts of flagella having a long wavelength and about one helical turn. Some species of Spirillum cause a form of RAT-BITE FEVER.'),
  'D013140': NtO(MH='Spiritualism', MS='Religious philosophy expressing the fundamental belief that departed spirits may be contacted by the living through a medium.'),
  'D013141': NtO(MH='Spiro Compounds', MS='Cyclic compounds that include two rings which share a single atom (usually a carbon). The simplest example of this type of compound is Spiro[2.2]pentane, which looks like a bow tie.'),
  'D013142': NtO(MH='Spirochaeta', MS='A genus of flexible, spiral rods found in hydrogen sulfide-containing mud, sewage, and polluted water. None of the species properly referred to in this genus are pathogenic.'),
  'D013143': NtO(MH='Spirochaetaceae', MS='A family of spiral bacteria of the order SPIROCHAETALES.'),
  'D013144': NtO(MH='Spirochaetales', MS='An order of slender, flexuous, helically coiled bacteria, with one or more complete turns in the helix.'),
  'D013145': NtO(MH='Spirochaetales Infections', MS='Infections with bacteria of the order SPIROCHAETALES.'),
  'D013146': NtO(MH='Spirometra', MS='A genus of tapeworms of the family Diphyllobothriidae, which are parasites of fish-eating cats, dogs, and birds. Infection in man is caused by eating undercooked fish. The larval form is called SPARGANUM.'),
  'D013147': NtO(MH='Spirometry', MS='Measurement of volume of air inhaled or exhaled by the lung.'),
  'D013148': NtO(MH='Spironolactone', MS='A potassium sparing diuretic that acts by antagonism of aldosterone in the distal renal tubules. It is used mainly in the treatment of refractory edema in patients with congestive heart failure, nephrotic syndrome, or hepatic cirrhosis. Its effects on the endocrine system are utilized in the treatments of hirsutism and acne but they can lead to adverse effects. (From Martindale, The Extra Pharmacopoeia, 30th ed, p827)'),
  'D013149': NtO(MH='Spiroplasma', MS='A genus of gram-negative, helical bacteria, in the family SPIROPLASMATACEAE, order Entomoplasmatales, causing disease in PLANTS. It has been isolated from TICKS; INSECTS; and PLANTS.'),
  'D013150': NtO(MH='Spirostans', MS='Cholestane derivatives containing a fused lactone ring at the 16,17-position and a spiroglycosidic linkage at C-22. Members include sarsaponin, DIOSGENIN and yamogenin.'),
  'D013151': NtO(MH='Spiruroidea', MS='A superfamily of parasitic nematodes which requires one or two intermediate arthropod hosts before finally being eaten by the final host. Its organisms occur rarely in man.'),
  'D013152': NtO(MH='Splanchnic Circulation', MS='The circulation of blood through the BLOOD VESSELS supplying the abdominal VISCERA.'),
  'D013153': NtO(MH='Splanchnic Nerves', MS='The major nerves supplying sympathetic innervation to the abdomen. The greater, lesser, and lowest (or smallest) splanchnic nerves are formed by preganglionic fibers from the spinal cord which pass through the paravertebral ganglia and then to the celiac ganglia and plexuses. The lumbar splanchnic nerves carry fibers which pass through the lumbar paravertebral ganglia to the mesenteric and hypogastric ganglia.'),
  'D013154': NtO(MH='Spleen', MS='An encapsulated lymphatic organ through which venous blood filters.'),
  'D013155': NtO(MH='Spleen Focus-Forming Viruses', MS='Strains of MURINE LEUKEMIA VIRUS that are replication-defective and rapidly transforming. The envelope gene plays an essential role in initiating erythroleukemia (LEUKEMIA, ERYTHROBLASTIC, ACUTE), manifested by splenic foci, SPLENOMEGALY, and POLYCYTHEMIA. Spleen focus-forming viruses are generated by recombination with endogenous retroviral sequences.'),
  'D013156': NtO(MH='Splenectomy', MS='Surgical procedure involving either partial or entire removal of the spleen.'),
  'D013157': NtO(MH='Splenic Artery', MS='The largest branch of the celiac trunk with distribution to the spleen, pancreas, stomach and greater omentum.'),
  'D013158': NtO(MH='Splenic Diseases', MS='Diseases involving the SPLEEN.'),
  'D013159': NtO(MH='Splenic Infarction', MS='Insufficiency of arterial or venous blood supply to the spleen due to emboli, thrombi, vascular torsion, or pressure that produces a macroscopic area of necrosis. (From Stedman, 25th ed)'),
  'D013160': NtO(MH='Splenic Neoplasms', MS='Tumors or cancer of the SPLEEN.'),
  'D013161': NtO(MH='Splenic Rupture', MS='Rupture of the SPLEEN due to trauma or disease.'),
  'D013162': NtO(MH='Splenic Vein', MS='Vein formed by the union (at the hilus of the spleen) of several small veins from the stomach, pancreas, spleen and mesentery.'),
  'D013163': NtO(MH='Splenomegaly', MS='Enlargement of the spleen.'),
  'D013164': NtO(MH='Splenorenal Shunt, Surgical', MS='Anastomosis of splenic vein to renal vein to relieve portal hypertension.'),
  'D013165': NtO(MH='Splints', MS='Rigid or flexible appliances that are used to maintain a displaced or movable part in position, or to maintain the position of and protect an injured part.'),
  'D013166': NtO(MH='Spondylitis', MS='Inflammation of the SPINE. This includes both arthritic and non-arthritic conditions.'),
  'D013167': NtO(MH='Spondylitis, Ankylosing', MS='A chronic inflammatory condition affecting the axial joints, such as the SACROILIAC JOINT and other intervertebral or costovertebral joints. It occurs predominantly in young males and is characterized by pain and stiffness of joints (ANKYLOSIS) with inflammation at tendon insertions.'),
  'D013168': NtO(MH='Spondylolisthesis', MS='Forward displacement of a superior vertebral body over the vertebral body below.'),
  'D013169': NtO(MH='Spondylolysis', MS='Deficient development or degeneration of a portion of the VERTEBRA, usually in the pars interarticularis (the bone bridge between the superior and inferior facet joints of the LUMBAR VERTEBRAE) leading to SPONDYLOLISTHESIS.'),
  'D013170': NtO(MH='Spores', MS='The reproductive elements of lower organisms, such as BACTERIA; FUNGI; and cryptogamic plants.'),
  'D013171': NtO(MH='Spores, Bacterial', MS='Heat and stain resistant, metabolically inactive bodies formed within the vegetative cells of bacteria of the genera Bacillus and Clostridium.'),
  'D013172': NtO(MH='Spores, Fungal', MS='Reproductive bodies produced by fungi.'),
  'D013173': NtO(MH='Sporidesmins', MS='A group of related mycotoxins produced by the pasture fungus Pithomyces chartarum (formerly Sporidesmium bakeri); causes liver damage and facial eczema in cattle and sheep.'),
  'D013174': NtO(MH='Sporotrichosis', MS='The commonest and least serious of the deep mycoses, characterized by nodular lesions of the cutaneous and subcutaneous tissues. It is caused by inhalation of contaminated dust or by infection of a wound with SPOROTHRIX.'),
  'D013175': NtO(MH='Sporothrix', MS='A mitosporic Ophiostomataceae fungal genus, whose species Sporothrix schenckii is a well-known animal pathogen. The conidia of this soil fungus may be inhaled causing a primary lung infection, or may infect independently via skin punctures.'),
  'D013177': NtO(MH='Sports', MS='Activities or games, usually involving physical effort or skill. Reasons for engagement in sports include pleasure, competition, and/or financial reward.'),
  'D013178': NtO(MH='Sports Medicine', MS='The field of medicine concerned with physical fitness and the diagnosis and treatment of injuries sustained in exercise and sports activities.'),
  'D013179': NtO(MH='Spouse Abuse', MS='Deliberate severe and repeated injury to one domestic partner by the other.'),
  'D013180': NtO(MH='Sprains and Strains', MS='A collective term for muscle and ligament injuries without dislocation or fracture. A sprain is a joint injury in which some of the fibers of a supporting ligament are ruptured but the continuity of the ligament remains intact. A strain is an overstretching or overexertion of some part of the musculature.'),
  'D013181': NtO(MH='Cortical Spreading Depression', MS="The decrease in neuronal activity (related to a decrease in metabolic demand) extending from the site of cortical stimulation. It is believed to be responsible for the decrease in cerebral blood flow that accompanies the aura of MIGRAINE WITH AURA. (Campbell's Psychiatric Dictionary, 8th ed.)"),
  'D013182': NtO(MH='Sprue, Tropical', MS='A chronic malabsorption syndrome, occurring mainly in residents of or visitors to the tropics or subtropics. The failed INTESTINAL ABSORPTION of nutrients from the SMALL INTESTINE results in MALNUTRITION and ANEMIA that is due to FOLIC ACID deficiency.'),
  'D013183': NtO(MH='Sputum', MS='Material coughed up from the lungs and expectorated via the mouth. It contains MUCUS, cellular debris, and microorganisms. It may also contain blood or pus.'),
  'D013184': NtO(MH='5-Amino-3-((5-nitro-2-furyl)vinyl)-1,2,4-oxadiazole', MS='A nitrovinyl furan used as a schistosomicidal agent and proposed for trypanosomiasis, especially Chagas disease.'),
  'D013185': NtO(MH='Squalene', MS='A natural 30-carbon triterpene.'),
  'D013186': NtO(MH='Farnesyl-Diphosphate Farnesyltransferase', MS='The first committed enzyme of the biosynthesis pathway that leads to the production of STEROLS. it catalyzes the synthesis of SQUALENE from farnesyl pyrophosphate via the intermediate PRESQUALENE PYROPHOSPHATE. This enzyme is also a critical branch point enzyme in the biosynthesis of ISOPRENOIDS that is thought to regulate the flux of isoprene intermediates through the sterol pathway.'),
  'D013188': NtO(MH='Sri Lanka', MS='Country in southern Asia, an island in the Indian Ocean, south of India. The capital is Colombo.'),
  'D013189': NtO(MH='SRS-A', MS='A group of LEUKOTRIENES; (LTC4; LTD4; and LTE4) that is the major mediator of BRONCHOCONSTRICTION; HYPERSENSITIVITY; and other allergic reactions. Earlier studies described a "slow-reacting substance of ANAPHYLAXIS" released from lung by cobra venom or after anaphylactic shock. The relationship between SRS-A leukotrienes was established by UV which showed the presence of the conjugated triene. (From Merck Index, 11th ed)'),
  'D013190': NtO(MH='SSPE Virus', MS='A defective variant of MEASLES VIRUS that has been isolated from the brain tissue of patients with subacute sclerosing panencephalitis.'),
  'D013191': NtO(MH='Stachybotrys', MS='A mitosporic fungal genus including one species which forms a toxin in moldy hay that may cause a serious illness in horses.'),
  'D013192': NtO(MH='Staff Development', MS='The process by which the employer promotes staff performance and efficiency consistent with management goals and objectives.'),
  'D013193': NtO(MH='Stainless Steel', MS="Stainless steel. A steel containing Ni, Cr, or both. It does not tarnish on exposure and is used in corrosive environments. (Grant & Hack's Chemical Dictionary, 5th ed)"),
  'D013194': NtO(MH='Staining and Labeling', MS='The marking of biological material with a dye or other reagent for the purpose of identifying and quantitating components of tissues, cells or their extracts.'),
  'D013195': NtO(MH='Stanford-Binet Test', MS='An individual intelligence test designed primarily for school children to predict school performance and the ability to adjust to everyday demands.'),
  'D013196': NtO(MH='Dihydrotestosterone', MS='A potent androgenic metabolite of TESTOSTERONE. It is produced by the action of the enzyme 3-OXO-5-ALPHA-STEROID 4-DEHYDROGENASE.'),
  'D013197': NtO(MH='Stanozolol', MS='A synthetic steroid that has anabolic and androgenic properties. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1194)'),
  'D013198': NtO(MH='Stapedius', MS='A tiny muscle that arises from the posterior wall of the TYMPANIC CAVITY of MIDDLE EAR with its tendon inserted onto the neck of the STAPES. Stapedius pulls the stapes posteriorly and controls its movement.'),
  'D013199': NtO(MH='Stapes', MS='One of the three ossicles of the middle ear. It transmits sound vibrations from the INCUS to the internal ear (Ear, Internal see LABYRINTH).'),
  'D013200': NtO(MH='Stapes Mobilization', MS='An operation to remobilize the footplate of the stapes to relieve conductive hearing impairment caused by its immobilization through otosclerosis or middle ear disease. (Stedman, 26th ed)'),
  'D013201': NtO(MH='Stapes Surgery', MS='Surgery performed in which part of the STAPES, a bone in the middle ear, is removed and a prosthesis is placed to help transmit sound between the middle ear and inner ear.'),
  'D013202': NtO(MH='Staphylococcal Food Poisoning', MS='Poisoning by staphylococcal toxins present in contaminated food.'),
  'D013203': NtO(MH='Staphylococcal Infections', MS='Infections with bacteria of the genus STAPHYLOCOCCUS.'),
  'D013204': NtO(MH='Staphylococcus Phages', MS='Viruses whose host is Staphylococcus.'),
  'D013205': NtO(MH='Staphylococcal Protein A', MS='A protein present in the cell wall of most Staphylococcus aureus strains. The protein selectively binds to the Fc region of human normal and myeloma-derived IMMUNOGLOBULIN G. It elicits antibody activity and may cause hypersensitivity reactions due to histamine release; has also been used as cell surface antigen marker and in the clinical assessment of B lymphocyte function.'),
  'D013206': NtO(MH='Staphylococcal Scalded Skin Syndrome', MS='A disease of infants due to group 2 phage type 17 staphylococci that produce an epidermolytic exotoxin. Superficial fine vesicles and bullae form and rupture easily, resulting in loss of large sheets of epidermis.'),
  'D013207': NtO(MH='Staphylococcal Skin Infections', MS='Infections to the skin caused by bacteria of the genus STAPHYLOCOCCUS.'),
  'D013208': NtO(MH='Staphylococcal Toxoid', MS=None),
  'D013209': NtO(MH='Staphylococcal Vaccines', MS=None),
  'D013210': NtO(MH='Staphylococcus', MS='A genus of gram-positive, facultatively anaerobic, coccoid bacteria. Its organisms occur singly, in pairs, and in tetrads and characteristically divide in more than one plane to form irregular clusters. Natural populations of Staphylococcus are found on the skin and mucous membranes of warm-blooded animals. Some species are opportunistic pathogens of humans and animals.'),
  'D013211': NtO(MH='Staphylococcus aureus', MS='Potentially pathogenic bacteria found in nasal membranes, skin, hair follicles, and perineum of warm-blooded animals. They may cause a wide range of infections and intoxications.'),
  'D013212': NtO(MH='Staphylococcus epidermidis', MS='A species of STAPHYLOCOCCUS that is a spherical, non-motile, gram-positive, chemoorganotrophic, facultative anaerobe. Mainly found on the skin and mucous membrane of warm-blooded animals, it can be primary pathogen or secondary invader.'),
  'D013213': NtO(MH='Starch', MS='Any of a group of polysaccharides of the general formula (C6-H10-O5)n, composed of a long-chain polymer of glucose in the form of amylose and amylopectin. It is the chief storage form of energy reserve (carbohydrates) in plants.'),
  'D013214': NtO(MH='Starch Synthase', MS='An enzyme that catalyzes the transfer of glucose from ADPglucose to glucose-containing polysaccharides in 1,4-alpha-linkages. EC 2.4.1.21.'),
  'D013215': NtO(MH='Starfish', MS='Echinoderms having bodies of usually five radially disposed arms coalescing at the center.'),
  'D013216': NtO(MH='Reflex, Startle', MS='A complex involuntary response to an unexpected strong stimulus. The reaction involves physical movement away from the stimulus, MUSCLE CONTRACTION and limb flexion, BLINKING, and changes in HEART RATE, BLOOD PRESSURE, and RESPIRATION.'),
  'D013217': NtO(MH='Starvation', MS='Lengthy and continuous deprivation of food. (Stedman, 25th ed)'),
  'D013218': NtO(MH='State Dentistry', MS='Control, direction and financing of the total dental care of the population by a national government.'),
  'D013219': NtO(MH='State Government', MS='The level of governmental organization and function below that of the national or country-wide government.'),
  'D013220': NtO(MH='State Health Planning and Development Agencies', MS='Agencies established under PL93-641 to coordinate, conduct, and implement state health planning activities. Two primary responsibilities are the preparation of an annual State Health Plan and giving assistance to the Statewide Health Coordinating Council.'),
  'D013221': NtO(MH='State Health Plans', MS='State plans prepared by the State Health Planning and Development Agencies which are made up from plans submitted by the Health Systems Agencies and subject to review and revision by the Statewide Health Coordinating Council.'),
  'D013222': NtO(MH='State Medicine', MS='A system of medical care regulated, controlled and financed by the government, in which the government assumes responsibility for the health needs of the population.'),
  'D013223': NtO(MH='Statistics as Topic', MS='Works about the science and art of collecting, summarizing, and analyzing data that are subject to random variation.'),
  'D013224': NtO(MH='Status Asthmaticus', MS='A sudden intense and continuous aggravation of a state of asthma, marked by dyspnea to the point of exhaustion and collapse and not responding to the usual therapeutic efforts.'),
  'D013226': NtO(MH='Status Epilepticus', MS='A prolonged seizure or seizures repeated frequently enough to prevent recovery between episodes occurring over a period of 20-30 minutes. The most common subtype is generalized tonic-clonic status epilepticus, a potentially fatal condition associated with neuronal injury and respiratory and metabolic dysfunction. Nonconvulsive forms include petit mal status and complex partial status, which may manifest as behavioral disturbances. Simple partial status epilepticus consists of persistent motor, sensory, or autonomic seizures that do not impair cognition (see also EPILEPSIA PARTIALIS CONTINUA). Subclinical status epilepticus generally refers to seizures occurring in an unresponsive or comatose individual in the absence of overt signs of seizure activity. (From N Engl J Med 1998 Apr 2;338(14):970-6; Neurologia 1997 Dec;12 Suppl 6:25-30)'),
  'D013227': NtO(MH='Steam', MS='Water in its gaseous state. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D013228': NtO(MH='Stearates', MS='Salts and esters of the 18-carbon saturated, monocarboxylic acid--stearic acid.'),
  'D013229': NtO(MH='Stearic Acids', MS='A group of compounds that are derivatives of octadecanoic acid which is one of the most abundant fatty acids found in animal lipids. (Stedman, 25th ed)'),
  'D013230': NtO(MH='Stearoyl-CoA Desaturase', MS='An enzyme that catalyzes the formation of oleoyl-CoA, A, and water from stearoyl-CoA, AH2, and oxygen where AH2 is an unspecified hydrogen donor.'),
  'D013231': NtO(MH='Steatitis', MS='A disease of cats and mink characterized by a marked inflammation of adipose tissue and the deposition of "ceroid" pigment in the interstices of the adipose cells. It is believed to be caused by feeding diets containing too much unsaturated fatty acid and too little vitamin E. (Merck Veterinary Manual, 5th ed; Stedman, 25th ed)'),
  'D013232': NtO(MH='Steel', MS='A tough, malleable, iron-based alloy containing up to, but no more than, two percent carbon and often other metals. It is used in medicine and dentistry in implants and instrumentation.'),
  'D013233': NtO(MH='Stellate Ganglion', MS='A paravertebral sympathetic ganglion formed by the fusion of the inferior cervical and first thoracic ganglia.'),
  'D013234': NtO(MH='Stem Cells', MS='Relatively undifferentiated cells that retain the ability to divide and proliferate throughout postnatal life to provide progenitor cells that can differentiate into specialized cells.'),
  'D013236': NtO(MH='Stereognosis', MS='Perception of shape and form of objects by TOUCH, via tactile stimuli.'),
  'D013237': NtO(MH='Stereoisomerism', MS='The phenomenon whereby compounds whose molecules have the same number and kind of atoms and the same atomic arrangement, but differ in their spatial relationships. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D013238': NtO(MH='Stereotaxic Techniques', MS='Techniques used mostly during brain surgery which use a system of three-dimensional coordinates to locate the site to be operated on.'),
  'D013239': NtO(MH='Stereotyped Behavior', MS='Relatively invariant mode of behavior elicited or determined by a particular situation; may be verbal, postural, or expressive.'),
  'D013240': NtO(MH='Stereotyping', MS='An oversimplified perception or conception especially of persons, social groups, etc.'),
  'D013241': NtO(MH='Sterigmatocystin', MS='A carcinogenic mycotoxin produced in high yields by strains of the common molds, Aspergillus versicolor, A. nidulans, and an unidentified species of Bipolaris. It causes necrosis of the liver and kidney and has an inhibitory effect on orotic acid incorporation into nuclear RNA.'),
  'D013242': NtO(MH='Sterilization', MS='The destroying of all forms of life, especially microorganisms, by heat, chemical, or other means.'),
  'D013243': NtO(MH='Sterilization Reversal', MS='Procedures to reverse the effect of REPRODUCTIVE STERILIZATION and to regain fertility. Reversal procedures include those used to restore the flow in the FALLOPIAN TUBE or the VAS DEFERENS.'),
  'D013244': NtO(MH='Sterilization, Involuntary', MS="Interference of a person's reproductive ability without the consent of the person or guardian."),
  'D013245': NtO(MH='Sterilization, Reproductive', MS='Procedures to block or remove all or part of the genital tract for the purpose of rendering individuals sterile, incapable of reproduction. Surgical sterilization procedures are the most commonly used. There are also sterilization procedures involving chemical or physical means.'),
  'D013246': NtO(MH='Sterilization, Tubal', MS='Procedures that render the female sterile by interrupting the flow in the FALLOPIAN TUBE. These procedures generally are surgical, and may also use chemicals or physical means.'),
  'D013247': NtO(MH='Sternoclavicular Joint', MS='A double gliding joint formed by the CLAVICLE, superior and lateral parts of the manubrium sterni at the clavicular notch, and the cartilage of the first rib.'),
  'D013248': NtO(MH='Sternocostal Joints', MS='An articulation where the costal cartilage of each rib fit with slight concavities along the lateral borders of the STERNUM.'),
  'D013249': NtO(MH='Sternum', MS='A long, narrow, and flat bone commonly known as BREASTBONE occurring in the midsection of the anterior thoracic segment or chest region, which stabilizes the rib cage and serves as the point of origin for several muscles that move the arms, head, and neck.'),
  'D013250': NtO(MH='Steroid Hydroxylases', MS='Cytochrome P-450 monooxygenases (MIXED FUNCTION OXYGENASES) that are important in steroid biosynthesis and metabolism.'),
  'D013251': NtO(MH='Steroid Isomerases', MS='Enzymes that catalyze the transposition of double bond(s) in a steroid molecule. EC 5.3.3.'),
  'D013252': NtO(MH='Steroid 11-beta-Hydroxylase', MS='A mitochondrial cytochrome P450 enzyme that catalyzes the 11-beta-hydroxylation of steroids in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP11B1 gene, is important in the synthesis of CORTICOSTERONE and HYDROCORTISONE. Defects in CYP11B1 cause congenital adrenal hyperplasia (ADRENAL HYPERPLASIA, CONGENITAL).'),
  'D013253': NtO(MH='Steroid 12-alpha-Hydroxylase', MS='A liver microsomal cytochrome P450 enzyme that catalyzes the 12-alpha-hydroxylation of a broad spectrum of sterols in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP8B1gene, converts 7-alpha-hydroxy-4-cholesten-3-one to 7-alpha-12-alpha-dihydroxy-4-cholesten-3-one and is required in the synthesis of BILE ACIDS from cholesterol.'),
  'D013254': NtO(MH='Steroid 17-alpha-Hydroxylase', MS='A microsomal cytochrome P450 enzyme that catalyzes the 17-alpha-hydroxylation of progesterone or pregnenolone and subsequent cleavage of the residual two carbons at C17 in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP17 gene, generates precursors for glucocorticoid, androgen, and estrogen synthesis. Defects in CYP17 gene cause congenital adrenal hyperplasia (ADRENAL HYPERPLASIA, CONGENITAL) and abnormal sexual differentiation.'),
  'D013255': NtO(MH='Steroid 21-Hydroxylase', MS='An adrenal microsomal cytochrome P450 enzyme that catalyzes the 21-hydroxylation of steroids in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP21 gene, converts progesterones to precursors of adrenal steroid hormones (CORTICOSTERONE; HYDROCORTISONE). Defects in CYP21 cause congenital adrenal hyperplasia (ADRENAL HYPERPLASIA, CONGENITAL).'),
  'D013256': NtO(MH='Steroids', MS="A group of polycyclic compounds closely related biochemically to TERPENES. They include cholesterol, numerous hormones, precursors of certain vitamins, bile acids, alcohols (STEROLS), and certain natural drugs and poisons. Steroids have a common nucleus, a fused, reduced 17-carbon atom ring system, cyclopentanoperhydrophenanthrene. Most steroids also have two methyl groups and an aliphatic side-chain attached to the nucleus. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D013257': NtO(MH='Steroids, Brominated', MS='Steroids which are substituted with one or more bromine atoms in any position.'),
  'D013258': NtO(MH='Steroids, Chlorinated', MS='Steroids which are substituted with one or more chlorine atoms in any position.'),
  'D013259': NtO(MH='Steroids, Fluorinated', MS='Steroids which are substituted with one or more fluorine atoms in any position.'),
  'D013260': NtO(MH='Steroids, Heterocyclic', MS='Steroidal compounds in which one or more carbon atoms in the steroid ring system have been substituted with non-carbon atoms.'),
  'D013261': NtO(MH='Sterols', MS='Steroids with a hydroxyl group at C-3 and most of the skeleton of cholestane. Additional carbon atoms may be present in the side chain. (IUPAC Steroid Nomenclature, 1987)'),
  'D013262': NtO(MH='Stevens-Johnson Syndrome', MS='Rare cutaneous eruption characterized by extensive KERATINOCYTE apoptosis resulting in skin detachment with mucosal involvement. It is often provoked by the use of drugs (e.g., antibiotics and anticonvulsants) or associated with PNEUMONIA, MYCOPLASMA. It is considered a continuum of Toxic Epidermal Necrolysis.'),
  'D013264': NtO(MH='Stifle', MS='In horses, cattle, and other quadrupeds, the joint between the femur and the tibia, corresponding to the human knee.'),
  'D013265': NtO(MH='Stigmasterol', MS=None),
  'D013266': NtO(MH='Stilbamidines', MS='STILBENES with AMIDINES attached.'),
  'D013267': NtO(MH='Stilbenes', MS='Organic compounds that contain 1,2-diphenylethylene as a functional group.'),
  'D013268': NtO(MH='Stimulation, Chemical', MS='The increase in a measurable parameter of a PHYSIOLOGICAL PROCESS, including cellular, microbial, and plant; immunological, cardiovascular, respiratory, reproductive, urinary, digestive, neural, musculoskeletal, ocular, and skin physiological processes; or METABOLIC PROCESS, including enzymatic and other pharmacological processes, by a drug or other chemical.'),
  'D013269': NtO(MH='Stochastic Processes', MS='Processes that incorporate some element of randomness, used particularly to refer to a time series of random variables.'),
  'D013270': NtO(MH='Stomach', MS='An organ of digestion situated in the left upper quadrant of the abdomen between the termination of the ESOPHAGUS and the beginning of the DUODENUM.'),
  'D013271': NtO(MH='Gastric Dilatation', MS='Abnormal distention of the STOMACH due to accumulation of gastric contents that may reach 10 to 15 liters. Gastric dilatation may be the result of GASTRIC OUTLET OBSTRUCTION; ILEUS; GASTROPARESIS; or denervation.'),
  'D013272': NtO(MH='Stomach Diseases', MS='Pathological processes involving the STOMACH.'),
  'D013273': NtO(MH='Diverticulum, Stomach', MS='Saccular, outward protrusion of all or a portion of the wall of the STOMACH.'),
  'D013274': NtO(MH='Stomach Neoplasms', MS='Tumors or cancer of the STOMACH.'),
  'D013275': NtO(MH='Stomach Rupture', MS='Bursting of the STOMACH.'),
  'D013276': NtO(MH='Stomach Ulcer', MS='Ulceration of the GASTRIC MUCOSA due to contact with GASTRIC JUICE. It is often associated with HELICOBACTER PYLORI infection or consumption of nonsteroidal anti-inflammatory drugs (NSAIDS).'),
  'D013277': NtO(MH='Stomach Volvulus', MS='Twisting of the STOMACH that may result in gastric ISCHEMIA and GASTRIC OUTLET OBSTRUCTION. It is often associated with DIAPHRAGMATIC HERNIA.'),
  'D013278': NtO(MH='Stomach, Avian', MS='A component of the digestive system of birds which consists of the GIZZARD and PROVENTRICULUS.'),
  'D013279': NtO(MH='Stomach, Ruminant', MS='A component of the DIGESTIVE SYSTEM of RUMINANTS which consists of the ABOMASUM; OMASUM; RETICULUM; and RUMEN.'),
  'D013280': NtO(MH='Stomatitis', MS='INFLAMMATION of the soft tissues of the MOUTH, such as MUCOSA; PALATE; GINGIVA; and LIP.'),
  'D013281': NtO(MH='Stomatitis, Aphthous', MS="A recurrent disease of the oral mucosa of unknown etiology. It is characterized by small white ulcerative lesions, single or multiple, round or oval. Two to eight crops of lesions occur per year, lasting for 7 to 14 days and then heal without scarring. (From Jablonski's Dictionary of Dentistry, 1992, p742)"),
  'D013282': NtO(MH='Stomatitis, Denture', MS='Inflammation of the mouth due to denture irritation.'),
  'D013283': NtO(MH='Stomatitis, Herpetic', MS='Stomatitis caused by Herpesvirus hominis. It usually occurs as acute herpetic stomatitis (or gingivostomatitis), an oral manifestation of primary herpes simplex seen primarily in children and adolescents.'),
  'D013284': NtO(MH='Stomatognathic System', MS='The mouth, teeth, jaws, pharynx, and related structures as they relate to mastication, deglutition, and speech.'),
  'D013285': NtO(MH='Strabismus', MS="Misalignment of the visual axes of the eyes. In comitant strabismus the degree of ocular misalignment does not vary with the direction of gaze. In noncomitant strabismus the degree of misalignment varies depending on direction of gaze or which eye is fixating on the target. (Miller, Walsh & Hoyt's Clinical Neuro-Ophthalmology, 4th ed, p641)"),
  'D013286': NtO(MH='Datura stramonium', MS='A plant species of the genus DATURA, family SOLANACEAE, that contains TROPANES and other SOLANACEOUS ALKALOIDS.'),
  'D013287': NtO(MH='Illicit Drugs', MS='Drugs that are manufactured, obtained, or sold illegally. They include prescription drugs obtained or sold without prescription and non-prescription drugs. Illicit drugs are widely distributed, tend to be grossly impure and may cause unexpected toxicity.'),
  'D013288': NtO(MH='Streptobacillus', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that require SERUM; ASCITIC FLUID; or BLOOD for growth. Its organisms inhabit the THROAT; and NASOPHARYNX of wild and laboratory rats and cause one form of RAT-BITE FEVER in man.'),
  'D013289': NtO(MH='Streptococcaceae', MS='A family of gram-positive non-sporing bacteria including many parasitic, pathogenic, and saprophytic forms.'),
  'D013290': NtO(MH='Streptococcal Infections', MS='Infections with bacteria of the genus STREPTOCOCCUS.'),
  'D013291': NtO(MH='Streptococcus', MS='A genus of gram-positive, coccoid bacteria whose organisms occur in pairs or chains. No endospores are produced. Many species exist as commensals or parasites on man or animals with some being highly pathogenic. A few species are saprophytes and occur in the natural environment.'),
  'D013292': NtO(MH='Streptococcus agalactiae', MS='A bacterium which causes mastitis in cattle and occasionally in man.'),
  'D013293': NtO(MH='Enterococcus faecalis', MS='A species of gram-positive, coccoid bacteria commonly isolated from clinical specimens and the human intestinal tract. Most strains are nonhemolytic.'),
  'D013294': NtO(MH='Lactococcus lactis', MS='A non-pathogenic species of LACTOCOCCUS found in DAIRY PRODUCTS and responsible for the souring of MILK and the production of LACTIC ACID.'),
  'D013295': NtO(MH='Streptococcus mutans', MS='A polysaccharide-producing species of STREPTOCOCCUS isolated from human dental plaque.'),
  'D013296': NtO(MH='Streptococcus pneumoniae', MS='A gram-positive organism found in the upper respiratory tract, inflammatory exudates, and various body fluids of normal and/or diseased humans and, rarely, domestic animals.'),
  'D013297': NtO(MH='Streptococcus pyogenes', MS='A species of gram-positive, coccoid bacteria isolated from skin lesions, blood, inflammatory exudates, and the upper respiratory tract of humans. It is a group A hemolytic Streptococcus that can cause SCARLET FEVER and RHEUMATIC FEVER.'),
  'D013298': NtO(MH='Streptococcus sanguis', MS='A gram-positive organism found in dental plaque, in blood, on heart valves in subacute endocarditis, and infrequently in saliva and throat specimens. L-forms are associated with recurrent aphthous stomatitis.'),
  'D013299': NtO(MH='Streptodornase and Streptokinase', MS='A mixture of the enzymes (streptokinase and streptodornase) produced by hemolytic streptococci. It is used topically on surface lesions and by instillation in closed body cavities to remove clotted blood or fibrinous or purulent accumulations. It is also used as a skin test antigen in evaluating generalized cell-mediated immunodeficiency. (Dorland, 27th ed) EC 3.-.'),
  'D013300': NtO(MH='Streptokinase', MS='Streptococcal fibrinolysin . An enzyme produced by hemolytic streptococci. It hydrolyzes amide linkages and serves as an activator of plasminogen. It is used in thrombolytic therapy and is used also in mixtures with streptodornase (STREPTODORNASE AND STREPTOKINASE). EC 3.4.-.'),
  'D013301': NtO(MH='Streptolysins', MS='Exotoxins produced by certain strains of streptococci, particularly those of group A (STREPTOCOCCUS PYOGENES), that cause HEMOLYSIS.'),
  'D013302': NtO(MH='Streptomyces', MS='A genus of bacteria that form a nonfragmented aerial mycelium. Many species have been identified with some being pathogenic. This genus is responsible for producing a majority of the ANTI-BACTERIAL AGENTS of practical value.'),
  'D013303': NtO(MH='Streptomyces antibioticus', MS='An actinomycete from which the antibiotic OLEANDOMYCIN is obtained.'),
  'D013304': NtO(MH='Streptomyces aureofaciens', MS='An actinomycete from which the antibiotic CHLORTETRACYCLINE is obtained.'),
  'D013305': NtO(MH='Streptomyces griseus', MS='An actinomycete from which the antibiotics STREPTOMYCIN, grisein, and CANDICIDIN are obtained.'),
  'D013306': NtO(MH='Streptomycetaceae', MS='A family of soil bacteria. It also includes some parasitic forms.'),
  'D013307': NtO(MH='Streptomycin', MS='An antibiotic produced by the soil actinomycete Streptomyces griseus. It acts by inhibiting the initiation and elongation processes during protein synthesis.'),
  'D013308': NtO(MH='Streptonigrin', MS='Complex cytotoxic antibiotic obtained from Streptomyces flocculus or S. rufochronmogenus. It is used in advanced carcinoma and causes leukopenia.'),
  'D013309': NtO(MH='Streptothricins', MS='A group of antibiotic aminoglycosides differing only in the number of repeating residues in the peptide side chain. They are produced by Streptomyces and Actinomyces and may have broad spectrum antimicrobial and some antiviral properties.'),
  'D013310': NtO(MH='Streptovaricin', MS='A complex of ansamycin antibiotics consisting of streptovaricins A, B, C, D, E, F, G, J, and K of which streptovaricin C is the major component. (From Merck Index, 11th ed)'),
  'D013311': NtO(MH='Streptozocin', MS='An antibiotic that is produced by Stretomyces achromogenes. It is used as an antineoplastic agent and to induce diabetes in experimental animals.'),
  'D013312': NtO(MH='Stress, Physiological', MS='The unfavorable effect of environmental factors (stressors) on the physiological functions of an organism. Prolonged unresolved physiological stress can affect HOMEOSTASIS of the organism, and may lead to damaging or pathological conditions.'),
  'D013313': NtO(MH='Stress Disorders, Post-Traumatic', MS='A class of traumatic stress disorders with symptoms that last more than one month.'),
  'D013314': NtO(MH='Stress, Mechanical', MS='A purely physical condition which exists within any material because of strain or deformation by external forces or by non-uniform thermal expansion; expressed quantitatively in units of force per unit area.'),
  'D013315': NtO(MH='Stress, Psychological', MS='Stress wherein emotional factors predominate.'),
  'D013316': NtO(MH='Stria Vascularis', MS='A layer of stratified EPITHELIUM forming the endolymphatic border of the cochlear duct at the lateral wall of the cochlea. Stria vascularis contains primarily three cell types (marginal, intermediate, and basal), and capillaries. The marginal cells directly facing the ENDOLYMPH are important in producing ion gradients and endochoclear potential.'),
  'D013317': NtO(MH='Strikes, Employee', MS='Work-related situations in which the employees as a group refuse to work until certain conditions of employment are granted by the employer.'),
  'D013318': NtO(MH='Stroke Volume', MS='The amount of BLOOD pumped out of the HEART per beat, not to be confused with cardiac output (volume/time). It is calculated as the difference between the end-diastolic volume and the end-systolic volume.'),
  'D013319': NtO(MH='Strongyle Infections, Equine', MS='Infection of horses with parasitic nematodes of the superfamily STRONGYLOIDEA. Characteristics include the development of hemorrhagic nodules on the abdominal peritoneum.'),
  'D013320': NtO(MH='Strongyloidea', MS='A superfamily of strongyles or roundworms which are parasites in the intestinal tract of equines, pigs, rodents, and primates (including man). It includes the genera Cyasthostomum, Ransomus, Globocephalus, OESOPHAGOSTOMUM, and STRONGYLUS.'),
  'D013321': NtO(MH='Strongyloides', MS='A genus of parasitic nematodes widely distributed as intestinal parasites of mammals.'),
  'D013322': NtO(MH='Strongyloidiasis', MS='Infection with nematodes of the genus STRONGYLOIDES. The presence of larvae may produce pneumonitis and the presence of adult worms in the intestine could lead to moderate to severe diarrhea.'),
  'D013323': NtO(MH='Strongylus', MS='A genus of intestinal parasitic nematodes occurring in animals and man.'),
  'D013324': NtO(MH='Strontium', MS='An element of the alkaline earth family of metals. It has the atomic symbol Sr, atomic number 38, and atomic weight 87.62.'),
  'D013325': NtO(MH='Strontium Isotopes', MS='Stable strontium atoms that have the same atomic number as the element strontium, but differ in the atomic weight. Sr-84, 86, 87, and 88 are the stable strontium isotopes.'),
  'D013326': NtO(MH='Strontium Radioisotopes', MS='Unstable isotopes of strontium that decay or disintegrate spontaneously emitting radiation. Sr 80-83, 85, and 89-95 are radioactive strontium isotopes.'),
  'D013327': NtO(MH='Strophanthidin', MS='3 beta,5,14-Trihydroxy-19-oxo-5 beta-card-20(22)-enolide. The aglycone cardioactive agent isolated from Strophanthus Kombe, S. gratus and other species; it is a very toxic material formerly used as digitalis. Synonyms: Apocymarin; Corchorin; Cynotoxin; Corchorgenin.'),
  'D013328': NtO(MH='Strophanthins', MS='A number of different cardioactive glycosides obtained from Strophanthus species. OUABAIN is from S. gratus and CYMARINE from S. kombe. They are used like the digitalis glycosides.'),
  'D013329': NtO(MH='Structure-Activity Relationship', MS='The relationship between the chemical structure of a compound and its biological or pharmacological activity. Compounds are often classed together because they have structural characteristics in common including shape, size, stereochemical arrangement, and distribution of functional groups.'),
  'D013330': NtO(MH='Struma Ovarii', MS='A rare teratoid tumor of the ovary composed almost entirely of thyroid tissue, with large follicles containing abundant colloid. Occasionally there are symptoms of hyperthyroidism. 5-10% of struma ovarii become malignant, the only absolute criterion for which is the presence of metastasis. (Dorland, 27th ed; Segen, Dictionary of Modern Medicine, 1992)'),
  'D013331': NtO(MH='Strychnine', MS='An alkaloid found in the seeds of STRYCHNOS NUX-VOMICA. It is a competitive antagonist at glycine receptors and thus a convulsant. It has been used as an analeptic, in the treatment of nonketotic hyperglycinemia and sleep apnea, and as a rat poison.'),
  'D013332': NtO(MH='Student Dropouts', MS='Individuals who leave school, secondary or college, prior to completion of specified curriculum requirements.'),
  'D013333': NtO(MH='Student Health Services', MS='Health services for college and university students usually provided by the educational institution.'),
  'D013334': NtO(MH='Students', MS='Individuals enrolled in a school or formal educational program.'),
  'D013335': NtO(MH='Students, Dental', MS='Individuals enrolled a school of dentistry or a formal educational program in leading to a degree in dentistry.'),
  'D013336': NtO(MH='Students, Health Occupations', MS='Individuals enrolled in a school or formal educational program in the health occupations.'),
  'D013337': NtO(MH='Students, Medical', MS='Individuals enrolled in a school of medicine or a formal educational program in medicine.'),
  'D013338': NtO(MH='Students, Nursing', MS='Individuals enrolled in a school of nursing or a formal educational program leading to a degree in nursing.'),
  'D013339': NtO(MH='Students, Pharmacy', MS='Individuals enrolled in a school of pharmacy or a formal educational program leading to a degree in pharmacy.'),
  'D013340': NtO(MH='Students, Premedical', MS='Individuals enrolled in a preparatory course for medical school.'),
  'D013341': NtO(MH='Sturge-Weber Syndrome', MS='A non-inherited congenital condition with vascular and neurological abnormalities. It is characterized by facial vascular nevi (PORT-WINE STAIN), and capillary angiomatosis of intracranial membranes (MENINGES; CHOROID). Neurological features include EPILEPSY; cognitive deficits; GLAUCOMA; and visual defects.'),
  'D013342': NtO(MH='Stuttering', MS="A disturbance in the normal fluency and time patterning of speech that is inappropriate for the individual's age. This disturbance is characterized by frequent repetitions or prolongations of sounds or syllables. Various other types of speech dysfluencies may also be involved including interjections, broken words, audible or silent blocking, circumlocutions, words produced with an excess of physical tension, and monosyllabic whole word repetitions. Stuttering may occur as a developmental condition in childhood or as an acquired disorder which may be associated with BRAIN INFARCTIONS and other BRAIN DISEASES. (From DSM-IV, 1994)"),
  'D013343': NtO(MH='Styrenes', MS='Derivatives and polymers of styrene. They are used in the manufacturing of synthetic rubber, plastics, and resins. Some of the polymers form the skeletal structures for ion exchange resin beads.'),
  'D013344': NtO(MH='Subacute Sclerosing Panencephalitis', MS='A rare, slowly progressive encephalitis caused by chronic infection with the MEASLES VIRUS. The condition occurs primarily in children and young adults, approximately 2-8 years after the initial infection. A gradual decline in intellectual abilities and behavioral alterations are followed by progressive MYOCLONUS; MUSCLE SPASTICITY; SEIZURES; DEMENTIA; autonomic dysfunction; and ATAXIA. DEATH usually occurs 1-3 years after disease onset. Pathologic features include perivascular cuffing, eosinophilic cytoplasmic inclusions, neurophagia, and fibrous gliosis. It is caused by the SSPE virus, which is a defective variant of MEASLES VIRUS. (From Adams et al., Principles of Neurology, 6th ed, pp767-8)'),
  'D013345': NtO(MH='Subarachnoid Hemorrhage', MS='Bleeding into the intracranial or spinal SUBARACHNOID SPACE, most resulting from INTRACRANIAL ANEURYSM rupture. It can occur after traumatic injuries (SUBARACHNOID HEMORRHAGE, TRAUMATIC). Clinical features include HEADACHE; NAUSEA; VOMITING, nuchal rigidity, variable neurological deficits and reduced mental status.'),
  'D013346': NtO(MH='Subarachnoid Space', MS='The space between the arachnoid membrane and PIA MATER, filled with CEREBROSPINAL FLUID. It contains large blood vessels that supply the BRAIN and SPINAL CORD.'),
  'D013347': NtO(MH='Subcellular Fractions', MS='Components of a cell produced by various separation techniques which, though they disrupt the delicate anatomy of a cell, preserve the structure and physiology of its functioning constituents for biochemical and ultrastructural analysis. (From Alberts et al., Molecular Biology of the Cell, 2d ed, p163)'),
  'D013348': NtO(MH='Subclavian Artery', MS='Artery arising from the brachiocephalic trunk on the right side and from the arch of the aorta on the left side. It distributes to the neck, thoracic wall, spinal cord, brain, meninges, and upper limb.'),
  'D013349': NtO(MH='Subclavian Steal Syndrome', MS='A clinically significant reduction in blood supply to the BRAIN STEM and CEREBELLUM (i.e., VERTEBROBASILAR INSUFFICIENCY) resulting from reversal of blood flow through the VERTEBRAL ARTERY from occlusion or stenosis of the proximal subclavian or brachiocephalic artery. Common symptoms include VERTIGO; SYNCOPE; and INTERMITTENT CLAUDICATION of the involved upper extremity. Subclavian steal may also occur in asymptomatic individuals. (From J Cardiovasc Surg 1994;35(1):11-4; Acta Neurol Scand 1994;90(3):174-8)'),
  'D013350': NtO(MH='Subclavian Vein', MS='The continuation of the axillary vein which follows the subclavian artery and then joins the internal jugular vein to form the brachiocephalic vein.'),
  'D013351': NtO(MH='Subcommissural Organ', MS='Ependymal derivative located at the junction of the THIRD VENTRICLE and the CEREBRAL AQUEDUCT; and the SOMATOSTATIN SECRETING CELLS.'),
  'D013352': NtO(MH='Subcutaneous Emphysema', MS='Presence of air or gas in the subcutaneous tissues of the body.'),
  'D013353': NtO(MH='Subdural Effusion', MS='Leakage and accumulation of CEREBROSPINAL FLUID in the subdural space which may be associated with an infectious process; CRANIOCEREBRAL TRAUMA; BRAIN NEOPLASMS; INTRACRANIAL HYPOTENSION; and other conditions.'),
  'D013354': NtO(MH='Empyema, Subdural', MS='An intracranial or rarely intraspinal suppurative process invading the space between the inner surface of the DURA MATER and the outer surface of the ARACHNOID.'),
  'D013355': NtO(MH='Subdural Space', MS='Potential cavity which separates the ARACHNOID MATER from the DURA MATER.'),
  'D013356': NtO(MH='Subfornical Organ', MS='A structure, situated close to the intraventricular foramen, which induces DRINKING BEHAVIOR after stimulation with ANGIOTENSIN II.'),
  'D013357': NtO(MH='Subgingival Curettage', MS='Removal of degenerated and necrotic epithelium and underlying connective tissue of a periodontal pocket in an effort to convert a chronic ulcerated wound to an acute surgical wound, thereby insuring wound healing and attachment or epithelial adhesion, and shrinkage of the marginal gingiva. The term is sometimes used in connection with smoothing of a root surface or ROOT PLANING. (Jablonski; Illustrated Dictionary of Dentistry, 1982)'),
  'D013358': NtO(MH='Subject Headings', MS='Terms or expressions which provide the major means of access by subject to the bibliographic unit.'),
  'D013359': NtO(MH='Sublimation, Psychological', MS='A defense mechanism through which unacceptable impulses and instinctive urges are diverted into personally and socially acceptable channels; e.g., aggression may be diverted through sports activities.'),
  'D013360': NtO(MH='Subliminal Stimulation', MS='Stimulation at an intensity below that where a differentiated response can be elicited.'),
  'D013361': NtO(MH='Sublingual Gland', MS='A salivary gland on each side of the mouth below the TONGUE.'),
  'D013362': NtO(MH='Sublingual Gland Neoplasms', MS='Neoplasms of the sublingual glands.'),
  'D013363': NtO(MH='Submandibular Gland', MS='One of two salivary glands in the neck, located in the space bound by the two bellies of the digastric muscle and the angle of the mandible. It discharges through the submandibular duct. The secretory units are predominantly serous although a few mucous alveoli, some with serous demilunes, occur. (Stedman, 25th ed)'),
  'D013364': NtO(MH='Submandibular Gland Diseases', MS='Diseases involving the SUBMANDIBULAR GLAND.'),
  'D013365': NtO(MH='Submandibular Gland Neoplasms', MS='New abnormal growth of tissue in the SUBMANDIBULAR GLAND.'),
  'D013366': NtO(MH='Submarine Medicine', MS='The field of medicine concerned with conditions affecting the health of people in submarines or sealabs.'),
  'D013367': NtO(MH='Submitochondrial Particles', MS='The various filaments, granules, tubules or other inclusions within mitochondria.'),
  'D013368': NtO(MH='Submucous Plexus', MS="One of two ganglionated neural networks which together form the enteric nervous system. The submucous (Meissner's) plexus is in the connective tissue of the submucosa. Its neurons innervate the epithelium, blood vessels, endocrine cells, other submucosal ganglia, and myenteric ganglia, and play an important role in regulating ion and water transport. (From FASEB J 1989;3:127-38)"),
  'D013369': NtO(MH='Subphrenic Abscess', MS='Accumulation of purulent EXUDATES beneath the DIAPHRAGM, also known as upper abdominal abscess. It is usually associated with PERITONITIS or postoperative infections.'),
  'D013370': NtO(MH='Subrenal Capsule Assay', MS='In vivo method of screening investigative anticancer drugs and biologic response modifiers for individual cancer patients. Fresh tumor tissue is implanted under the kidney capsule of immunocompetent mice or rats; gross and histological assessments follow several days after tumor treatment in situ.'),
  'D013373': NtO(MH='Substance P', MS='An eleven-amino acid neurotransmitter that appears in both the central and peripheral nervous systems. It is involved in transmission of PAIN, causes rapid contractions of the gastrointestinal smooth muscle, and modulates inflammatory and immune responses.'),
  'D013375': NtO(MH='Substance Withdrawal Syndrome', MS='Physiological and psychological symptoms associated with withdrawal from the use of a drug after prolonged administration or habituation. The concept includes withdrawal from smoking or drinking, as well as withdrawal from an administered drug.'),
  'D013376': NtO(MH='Substantia Gelatinosa', MS='Gelatinous-appearing material in the dorsal horn of the spinal cord, consisting chiefly of Golgi type II neurons and some larger nerve cells.'),
  'D013377': NtO(MH='Substantia Innominata', MS='Tissue in the BASAL FOREBRAIN inferior to the anterior perforated substance, and anterior to the GLOBUS PALLIDUS and ansa lenticularis. It contains the BASAL NUCLEUS OF MEYNERT.'),
  'D013378': NtO(MH='Substantia Nigra', MS='The black substance in the ventral midbrain or the nucleus of cells containing the black substance. These cells produce DOPAMINE, an important neurotransmitter in regulation of the sensorimotor system and mood. The dark colored MELANIN is a by-product of dopamine synthesis.'),
  'D013379': NtO(MH='Substrate Specificity', MS='A characteristic feature of enzyme activity in relation to the kind of substrate on which the enzyme or catalytic molecule reacts.'),
  'D013380': NtO(MH='Subtalar Joint', MS='Formed by the articulation of the talus with the calcaneus.'),
  'D013381': NtO(MH='Subtilisins', MS='A family of SERINE ENDOPEPTIDASES isolated from Bacillus subtilis. EC 3.4.21.-'),
  'D013382': NtO(MH='Subtraction Technique', MS='Combination or superimposition of two images for demonstrating differences between them (e.g., radiograph with contrast vs. one without, radionuclide images using different radionuclides, radiograph vs. radionuclide image) and in the preparation of audiovisual materials (e.g., offsetting identical images, coloring of vessels in angiograms).'),
  'D013383': NtO(MH='Suburban Population', MS='The inhabitants of peripheral or adjacent areas of a city or town.'),
  'D013384': NtO(MH='Succinate Cytochrome c Oxidoreductase', MS='An electron transport chain complex that catalyzes the transfer of electrons from SUCCINATE to CYTOCHROME C. It includes ELECTRON TRANSPORT COMPLEX II and ELECTRON TRANSPORT COMPLEX III.'),
  'D013385': NtO(MH='Succinate Dehydrogenase', MS='A flavoprotein containing oxidoreductase that catalyzes the dehydrogenation of SUCCINATE to fumarate. In most eukaryotic organisms this enzyme is a component of mitochondrial electron transport complex II.'),
  'D013386': NtO(MH='Succinates', MS='Derivatives of SUCCINIC ACID. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain a 1,4-carboxy terminated aliphatic structure.'),
  'D013387': NtO(MH='Succinic Anhydrides', MS='A subclass of anhydrides with the general structure of dihydrofurandione. They can be substituted on any carbon atom. They modify and inhibit proteins and enzymes and are used in the acylation of amino- and hydroxyl groups.'),
  'D013388': NtO(MH='Succinimides', MS='A subclass of IMIDES with the general structure of pyrrolidinedione. They are prepared by the distillation of ammonium succinate. They are sweet-tasting compounds that are used as chemical intermediates and plant growth stimulants.'),
  'D013389': NtO(MH='Succinate-CoA Ligases', MS='Enzymes that catalyze the first step leading to the oxidation of succinic acid by the reversible formation of succinyl-CoA from succinate and CoA with the concomitant cleavage of ATP to ADP (EC 6.2.1.5) or GTP to GDP (EC 6.2.1.4) and orthophosphate. Itaconate can act instead of succinate and ITP instead of GTP.EC 6.2.1.-.'),
  'D013390': NtO(MH='Succinylcholine', MS='A quaternary skeletal muscle relaxant usually used in the form of its bromide, chloride, or iodide. It is a depolarizing relaxant, acting in about 30 seconds and with a duration of effect averaging three to five minutes. Succinylcholine is used in surgical, anesthetic, and other procedures in which a brief period of muscle relaxation is called for.'),
  'D013391': NtO(MH='Sucking Behavior', MS='Any suction exerted by the mouth; response of the mammalian infant to draw milk from the breast. Includes sucking on inanimate objects. Not to be used for thumb sucking, which is indexed under fingersucking.'),
  'D013392': NtO(MH='Sucralfate', MS='A basic aluminum complex of sulfated sucrose.'),
  'D013393': NtO(MH='Sucrase', MS='Digestive enzyme secreted in the INTESTINES. It catalyzes hydrolysis of SUCROSE to FRUCTOSE and GLUCOSE.'),
  'D013394': NtO(MH='Sucrase-Isomaltase Complex', MS='An enzyme complex found in the brush border membranes of the small intestine. It is believed to be an enzyme complex with different catalytic sites. Its absence is manifested by an inherited disease called sucrase-isomaltase deficiency.'),
  'D013395': NtO(MH='Sucrose', MS='A nonreducing disaccharide composed of GLUCOSE and FRUCTOSE linked via their anomeric carbons. It is obtained commercially from SUGARCANE, sugar beet (BETA VULGARIS), and other plants and used extensively as a food and a sweetener.'),
  'D013396': NtO(MH='Suction', MS='The removal of secretions, gas or fluid from hollow or tubular organs or cavities by means of a tube and a device that acts on negative pressure.'),
  'D013397': NtO(MH='Sudan', MS='A country in northeastern Africa. The capital is Khartoum.'),
  'D013398': NtO(MH='Sudden Infant Death', MS='The abrupt and unexplained death of an apparently healthy infant under one year of age, remaining unexplained after a thorough case investigation, including performance of a complete autopsy, examination of the death scene, and review of the clinical history. (Pediatr Pathol 1991 Sep-Oct;11(5):677-84)'),
  'D013400': NtO(MH='Sugar Acids', MS='MONOSACCHARIDES and other sugars that contain one or more carboxylic acid moieties.'),
  'D013401': NtO(MH='Sugar Alcohol Dehydrogenases', MS='Reversibly catalyzes the oxidation of a hydroxyl group of sugar alcohols to form a keto sugar, aldehyde or lactone. Any acceptor except molecular oxygen is permitted. Includes EC 1.1.1.; EC 1.1.2. and EC 1.1.99.'),
  'D013402': NtO(MH='Sugar Alcohols', MS='Polyhydric alcohols having no more than one hydroxy group attached to each carbon atom. They are formed by the reduction of the carbonyl group of a sugar to a hydroxyl group.(From Dorland, 28th ed)'),
  'D013403': NtO(MH='Sugar Phosphates', MS=None),
  'D013404': NtO(MH='Suggestion', MS='The uncritical acceptance of an idea or plan of action.'),
  'D013405': NtO(MH='Suicide', MS='The act of killing oneself.'),
  'D013406': NtO(MH='Suicide, Attempted', MS='The unsuccessful attempt to kill oneself.'),
  'D013407': NtO(MH='Sulbactam', MS='A beta-lactamase inhibitor with very weak antibacterial action. The compound prevents antibiotic destruction of beta-lactam antibiotics by inhibiting beta-lactamases, thus extending their spectrum activity. Combinations of sulbactam with beta-lactam antibiotics have been used successfully for the therapy of infections caused by organisms resistant to the antibiotic alone.'),
  'D013408': NtO(MH='Sulbenicillin', MS='Semisynthetic penicillin-type antibiotic.'),
  'D013409': NtO(MH='Sulfacetamide', MS='An anti-bacterial agent that is used topically to treat skin infections and orally for urinary tract infections.'),
  'D013410': NtO(MH='Sulfachlorpyridazine', MS='A sulfonamide antimicrobial used for urinary tract infections and in veterinary medicine.'),
  'D013411': NtO(MH='Sulfadiazine', MS='One of the short-acting SULFONAMIDES used in combination with PYRIMETHAMINE to treat toxoplasmosis in patients with acquired immunodeficiency syndrome and in newborns with congenital infections.'),
  'D013412': NtO(MH='Sulfadimethoxine', MS='A sulfanilamide that is used as an anti-infective agent.'),
  'D013413': NtO(MH='Sulfadoxine', MS='A long acting sulfonamide that is used, usually in combination with other drugs, for respiratory, urinary tract, and malarial infections.'),
  'D013414': NtO(MH='Sulfaguanidine', MS='A sulfanilamide antimicrobial agent that is used to treat enteric infections.'),
  'D013415': NtO(MH='Sulfalene', MS='Long-acting plasma-bound sulfonamide used for respiratory and urinary tract infections and also for malaria.'),
  'D013416': NtO(MH='Sulfamerazine', MS='A sulfanilamide that is used as an antibacterial agent.'),
  'D013417': NtO(MH='Sulfameter', MS='Long acting sulfonamide used in leprosy, urinary, and respiratory tract infections.'),
  'D013418': NtO(MH='Sulfamethazine', MS='A sulfanilamide anti-infective agent. It has a spectrum of antimicrobial action similar to other sulfonamides.'),
  'D013419': NtO(MH='Sulfamethizole', MS='A sulfathiazole antibacterial agent.'),
  'D013420': NtO(MH='Sulfamethoxazole', MS='A bacteriostatic antibacterial agent that interferes with folic acid synthesis in susceptible bacteria. Its broad spectrum of activity has been limited by the development of resistance. (From Martindale, The Extra Pharmacopoeia, 30th ed, p208)'),
  'D013421': NtO(MH='Sulfamethoxypyridazine', MS='A sulfanilamide antibacterial agent.'),
  'D013422': NtO(MH='Sulfamonomethoxine', MS='Long acting sulfonamide antibacterial agent.'),
  'D013423': NtO(MH='Sulfamoxole', MS='A sulfanilamide antibacterial agent.'),
  'D013424': NtO(MH='Sulfanilamides', MS="Compounds based on 4-aminobenzenesulfonamide. The '-anil-' part of the name refers to aniline."),
  'D013425': NtO(MH='Sulfanilic Acids', MS='Aminobenzenesulfonic acids. Organic acids that are used in the manufacture of dyes and organic chemicals and as reagents.'),
  'D013426': NtO(MH='Sulfaphenazole', MS='A sulfonilamide anti-infective agent.'),
  'D013427': NtO(MH='Sulfapyridine', MS='Antibacterial, potentially toxic, used to treat certain skin diseases.'),
  'D013428': NtO(MH='Sulfaquinoxaline', MS='An antiprotozoal agent used to combat coccidial infections of swine, cattle, fowl, and other veterinary animals. Also used in controlling outbreaks of fowl typhoid and fowl cholera and in treatment of infectious enteritis.'),
  'D013429': NtO(MH='Sulfatases', MS=None),
  'D013430': NtO(MH='Sulfate Adenylyltransferase', MS="An enzyme that catalyzes the activation of sulfate ions by ATP to form adenosine-5'-phosphosulfate and pyrophosphate. This reaction constitutes the first enzymatic step in sulfate utilization following the uptake of sulfate. EC 2.7.7.4."),
  'D013431': NtO(MH='Sulfates', MS='Inorganic salts of sulfuric acid.'),
  'D013432': NtO(MH='Sulfathiazoles', MS='Sulfanilamides consisting of  a 4-aminobenzenesulfonamido group at the 2-position of 1,3-thiazole. They are often used as ANTI-INFECTIVE AGENTS.'),
  'D013433': NtO(MH='Sulfoglycosphingolipids', MS='GLYCOSPHINGOLIPIDS with a sulfate group esterified to one of the sugar groups.'),
  'D013434': NtO(MH='Sulfenic Acids', MS='Oxy acids of sulfur with the general formula RSOH, where R is an alkyl or aryl group such as CH3. They are often encountered as esters and halides. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013435': NtO(MH='Sulfhemoglobin', MS=None),
  'D013436': NtO(MH='Sulfhemoglobinemia', MS='A morbid condition due to the presence of sulfmethemoglobin in the blood. It is marked by persistent cyanosis, but the blood count does not reveal any special abnormality in the blood. It is thought to be caused by the action of hydrogen sulfide absorbed from the intestine. (Stedman, 25th ed)'),
  'D013437': NtO(MH='Carbon-Sulfur Lyases', MS='Enzymes that catalyze the cleavage of a carbon-sulfur bond by means other than hydrolysis or oxidation. EC 4.4.'),
  'D013438': NtO(MH='Sulfhydryl Compounds', MS='Compounds containing the -SH radical.'),
  'D013439': NtO(MH='Sulfhydryl Reagents', MS='Chemical agents that react with SH groups. This is a chemically diverse group that is used for a variety of purposes. Among these are enzyme inhibition, enzyme reactivation or protection, and labelling.'),
  'D013440': NtO(MH='Sulfides', MS='Chemical groups containing the covalent sulfur bonds -S-. The sulfur atom can be bound to inorganic or organic moieties.'),
  'D013441': NtO(MH='Sulfinic Acids', MS='Any of the monobasic inorganic or organic acids of sulfur with the general formula RSO(OH). (From McGraw Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013442': NtO(MH='Sulfinpyrazone', MS='A uricosuric drug that is used to reduce the serum urate levels in gout therapy. It lacks anti-inflammatory, analgesic, and diuretic properties.'),
  'D013443': NtO(MH='Sulfisomidine', MS='A sulfanilamide antibacterial agent.'),
  'D013444': NtO(MH='Sulfisoxazole', MS='A short-acting sulfonamide antibacterial with activity against a wide range of gram- negative and gram-positive organisms.'),
  'D013447': NtO(MH='Sulfites', MS='Inorganic salts of sulfurous acid.'),
  'D013448': NtO(MH='Sulfobromophthalein', MS='A phenolphthalein that is used as a diagnostic aid in hepatic function determination.'),
  'D013449': NtO(MH='Sulfonamides', MS='A group of compounds that contain the structure SO2NH2.'),
  'D013450': NtO(MH='Sulfones', MS=None),
  'D013451': NtO(MH='Sulfonic Acids', MS='Inorganic or organic oxy acids of sulfur which contain the RSO2(OH) radical.'),
  'D013452': NtO(MH='Sulfonium Compounds', MS='Sulfur compounds in which the sulfur atom is attached to three organic radicals and an electronegative element or radical.'),
  'D013453': NtO(MH='Sulfonylurea Compounds', MS=None),
  'D013454': NtO(MH='Sulfoxides', MS="Organic compounds that have the general formula R-SO-R. They are obtained by oxidation of mercaptans (analogous to the ketones). (From Hackh's Chemical Dictionary, 4th ed)"),
  'D013455': NtO(MH='Sulfur', MS='An element that is a member of the chalcogen family. It has an atomic symbol S, atomic number 16, and atomic weight [32.059; 32.076]. It is found in the amino acids cysteine and methionine.'),
  'D013456': NtO(MH='Sulfur Acids', MS='Inorganic or organic acids that contain sulfur as an integral part of the molecule.'),
  'D013457': NtO(MH='Sulfur Compounds', MS='Inorganic or organic compounds that contain sulfur as an integral part of the molecule.'),
  'D013458': NtO(MH='Sulfur Dioxide', MS='A highly toxic, colorless, nonflammable gas. It is used as a pharmaceutical aid and antioxidant. It is also an environmental air pollutant.'),
  'D013459': NtO(MH='Sulfur Hexafluoride', MS='Sulfur hexafluoride. An inert gas used mainly as a test gas in respiratory physiology. Other uses include its injection in vitreoretinal surgery to restore the vitreous chamber and as a tracer in monitoring the dispersion and deposition of air pollutants.'),
  'D013460': NtO(MH='Sulfur Isotopes', MS='Stable sulfur atoms that have the same atomic number as the element sulfur, but differ in atomic weight. S-33, 34, and 36 are stable sulfur isotopes.'),
  'D013461': NtO(MH='Sulfur Oxides', MS='Inorganic oxides of sulfur.'),
  'D013462': NtO(MH='Sulfur Radioisotopes', MS='Unstable isotopes of sulfur that decay or disintegrate spontaneously emitting radiation. S 29-31, 35, 37, and 38 are radioactive sulfur isotopes.'),
  'D013463': NtO(MH='Sulfuric Acid Esters', MS='Organic esters of sulfuric acid.'),
  'D013464': NtO(MH='Sulfuric Acids', MS='Inorganic and organic derivatives of sulfuric acid (H2SO4). The salts and esters of sulfuric acid are known as SULFATES and SULFURIC ACID ESTERS respectively.'),
  'D013466': NtO(MH='Sulfurtransferases', MS='Enzymes which transfer sulfur atoms to various acceptor molecules. EC 2.8.1.'),
  'D013467': NtO(MH='Sulindac', MS='A sulfinylindene derivative prodrug whose sulfinyl moiety is converted in vivo to an active NSAID analgesic. Specifically, the prodrug is converted by liver enzymes to a sulfide which is excreted in the bile and then reabsorbed from the intestine. This helps to maintain constant blood levels with reduced gastrointestinal side effects.'),
  'D013468': NtO(MH='Suloctidil', MS='A peripheral vasodilator that was formerly used in the management of peripheral and cerebral vascular disorders. It is hepatotoxic and fatalities have occurred. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1312)'),
  'D013469': NtO(MH='Sulpiride', MS='A dopamine D2-receptor antagonist. It has been used therapeutically as an antidepressant, antipsychotic, and as a digestive aid. (From Merck Index, 11th ed)'),
  'D013471': NtO(MH='Sunburn', MS='An injury to the skin causing erythema, tenderness, and sometimes blistering and resulting from excessive exposure to the sun. The reaction is produced by the ultraviolet radiation in sunlight.'),
  'D013472': NtO(MH='Sunlight', MS='Irradiation directly from the sun.'),
  'D013473': NtO(MH='Sunscreening Agents', MS='Chemical or physical agents that protect the skin from sunburn and erythema by absorbing or blocking ultraviolet radiation.'),
  'D013474': NtO(MH='Sunstroke', MS='Heat stroke caused by exposure to the sun. It is characterized by dangerously high BODY TEMPERATURE; red, hot skin; DELUSIONS; CONVULSIONS; or COMA. It can be a life-threatening emergency and is most common in infants and the elderly.'),
  'D013475': NtO(MH='Superego', MS='The component of the personality associated with ethics, standards, and self-criticism - the "conscience". It is derived mainly from identification with parents and parent substitutes.'),
  'D013476': NtO(MH='Superfetation', MS='The FERTILIZATION of an OVUM that takes place when a FETUS is already present in the UTERUS. Superfetation results in an unusual PREGNANCY with fetuses of different ages and sizes developing in utero simultaneously.'),
  'D013477': NtO(MH='Superior Colliculi', MS='The anterior pair of the quadrigeminal bodies which coordinate the general behavioral orienting responses to visual stimuli, such as whole-body turning, and reaching.'),
  'D013478': NtO(MH='Superior Mesenteric Artery Syndrome', MS='DUODENAL OBSTRUCTION by the superior mesenteric artery (MESENTERIC ARTERY, SUPERIOR) which travels in the root of the MESENTERY and crosses over the DUODENUM. The syndrome is characterized by the dilated proximal duodenum and STOMACH, bloating, ABDOMINAL CRAMPS, and VOMITING. Often it is observed in patient with body casts after spinal surgery.'),
  'D013479': NtO(MH='Superior Vena Cava Syndrome', MS='A condition that occurs when the obstruction of the thin-walled SUPERIOR VENA CAVA interrupts blood flow from the head, upper extremities, and thorax to the RIGHT ATRIUM. Obstruction can be caused by NEOPLASMS; THROMBOSIS; ANEURYSM; or external compression. The syndrome is characterized by swelling and/or CYANOSIS of the face, neck, and upper arms.'),
  'D013480': NtO(MH='Superovulation', MS='Occurrence or induction of release of more ova than are normally released at the same time in a given species. The term applies to both animals and humans.'),
  'D013481': NtO(MH='Superoxides', MS='Highly reactive compounds produced when oxygen is reduced by a single electron. In biological systems, they may be generated during the normal catalytic function of a number of enzymes and during the oxidation of hemoglobin to METHEMOGLOBIN. In living organisms, SUPEROXIDE DISMUTASE protects the cell from the deleterious effects of superoxides.'),
  'D013482': NtO(MH='Superoxide Dismutase', MS='An oxidoreductase that catalyzes the reaction between SUPEROXIDES and hydrogen to yield molecular oxygen and hydrogen peroxide. The enzyme protects the cell against dangerous levels of superoxide.'),
  'D013483': NtO(MH='Superstitions', MS='A belief or practice which lacks adequate basis for proof; an embodiment of fear of the unknown, magic, and ignorance.'),
  'D013484': NtO(MH='Supination', MS='Applies to movements of the forearm in turning the palm forward or upward. When referring to the foot, a combination of adduction and inversion movements of the foot.'),
  'D013485': NtO(MH="Research Support, Non-U.S. Gov't", MS='Acknowledgement that funding support is from any non-US government agency, for example state and local governments, foreign governments, and private organizations.'),
  'D013486': NtO(MH="Research Support, U.S. Gov't, Non-P.H.S.", MS='Acknowledgment that funding support is from any US government agency other than the Public Health Service, such as the National Science Foundation, NASA, Department of Energy, etc.'),
  'D013487': NtO(MH="Research Support, U.S. Gov't, P.H.S.", MS='Acknowledgement that funding support is from any component of the Public Health Service.'),
  'D013488': NtO(MH='Suppositories', MS='Medicated dosage forms that are designed to be inserted into the rectal, vaginal, or urethral orifice of the body for absorption. Generally, the active ingredients are packaged in dosage forms containing fatty bases such as cocoa butter, hydrogenated oil, or glycerogelatin that are solid at room temperature but melt or dissolve at body temperature.'),
  'D013489': NtO(MH='Suppression, Genetic', MS='Mutation process that restores the wild-type PHENOTYPE in an organism possessing a mutationally altered GENOTYPE. The second "suppressor" mutation may be on a different gene, on the same gene but located at a distance from the site of the primary mutation, or in extrachromosomal genes (EXTRACHROMOSOMAL INHERITANCE).'),
  'D013491': NtO(MH='Suppressor Factors, Immunologic', MS='Proteins, protein complexes, or glycoproteins secreted by suppressor T-cells that inhibit either subsequent T-cells, B-cells, or other immunologic phenomena. Some of these factors have both histocompatibility (I-J) and antigen-specific domains which may be linked by disulfide bridges. They can be elicited by haptens or other antigens and may be mass-produced by hybridomas or monoclones in the laboratory.'),
  'D013492': NtO(MH='Suppuration', MS='A pathologic process consisting in the formation of pus.'),
  'D013493': NtO(MH='Suprachiasmatic Nucleus', MS='An ovoid densely packed collection of small cells of the anterior hypothalamus lying close to the midline in a shallow impression of the OPTIC CHIASM.'),
  'D013494': NtO(MH='Supranuclear Palsy, Progressive', MS='A degenerative disease of the central nervous system characterized by balance difficulties; OCULAR MOTILITY DISORDERS (supranuclear ophthalmoplegia); DYSARTHRIA; swallowing difficulties; and axial DYSTONIA. Onset is usually in the fifth decade and disease progression occurs over several years. Pathologic findings include neurofibrillary degeneration and neuronal loss in the dorsal MESENCEPHALON; SUBTHALAMIC NUCLEUS; RED NUCLEUS; pallidum; dentate nucleus; and vestibular nuclei. (From Adams et al., Principles of Neurology, 6th ed, pp1076-7)'),
  'D013495': NtO(MH='Supraoptic Nucleus', MS='Hypothalamic nucleus overlying the beginning of the OPTIC TRACT.'),
  'D013496': NtO(MH='Suprofen', MS='An IBUPROFEN-type anti-inflammatory analgesic and antipyretic. It inhibits prostaglandin synthesis and has been proposed as an anti-arthritic.'),
  'D013497': NtO(MH='Sural Nerve', MS='A branch of the tibial nerve which supplies sensory innervation to parts of the lower leg and foot.'),
  'D013498': NtO(MH='Suramin', MS='A polyanionic compound with an unknown mechanism of action. It is used parenterally in the treatment of African trypanosomiasis and it has been used clinically with diethylcarbamazine to kill the adult Onchocerca. (From AMA Drug Evaluations Annual, 1992, p1643) It has also been shown to have potent antineoplastic properties.'),
  'D013499': NtO(MH='Surface Properties', MS='Characteristics or attributes of the outer boundaries of objects, including molecules.'),
  'D013500': NtO(MH='Surface Tension', MS='The force acting on the surface of a liquid, tending to minimize the area of the surface. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D013501': NtO(MH='Surface-Active Agents', MS='Agents that modify interfacial tension of water; usually substances that have one lipophilic and one hydrophilic group in the molecule; includes soaps, detergents, emulsifiers, dispersing and wetting agents, and several groups of antiseptics.'),
  'D013502': NtO(MH='General Surgery', MS='A specialty in which manual or operative procedures are used in the treatment of disease, injuries, or deformities.'),
  'D013503': NtO(MH='Surgery Department, Hospital', MS='Hospital department which administers all departmental functions and the provision of surgical diagnostic and therapeutic services.'),
  'D013504': NtO(MH='Cardiovascular Surgical Procedures', MS='Surgery performed on the heart or blood vessels.'),
  'D013505': NtO(MH='Digestive System Surgical Procedures', MS='Surgery performed on the digestive system or its parts.'),
  'D013506': NtO(MH='Otologic Surgical Procedures', MS='Surgery performed on the external, middle, or internal ear.'),
  'D013507': NtO(MH='Endocrine Surgical Procedures', MS='Surgery performed on any endocrine gland.'),
  'D013508': NtO(MH='Ophthalmologic Surgical Procedures', MS='Surgery performed on the eye or any of its parts.'),
  'D013509': NtO(MH='Gynecologic Surgical Procedures', MS='Surgery performed on the female genitalia.'),
  'D013510': NtO(MH='Pulmonary Surgical Procedures', MS='Surgery performed on the lung.'),
  'D013511': NtO(MH='Minor Surgical Procedures', MS='Surgery for problems and injuries that are of relatively slight extent, and with a low risk for life threatening complications.'),
  'D013513': NtO(MH='Obstetric Surgical Procedures', MS='Surgery performed on the pregnant woman for conditions associated with pregnancy, labor, or the puerperium. It does not include surgery of the newborn infant.'),
  'D013514': NtO(MH='Surgical Procedures, Operative', MS='Operations carried out for the correction of deformities and defects, repair of injuries, and diagnosis and cure of certain diseases. (Taber, 18th ed.)'),
  'D013515': NtO(MH='Surgery, Oral', MS='A dental specialty concerned with the diagnosis and surgical treatment of disease, injuries, and defects of the human oral and maxillofacial region.'),
  'D013516': NtO(MH='Oral Surgical Procedures, Preprosthetic', MS='Surgery necessary for a denture to rest on a firm base, free from marked osseous protuberances or undercuts, and devoid of interfering muscle attachments, excess mucoperiosteum, hyperplasias, and fibrous or papillary growths.'),
  'D013517': NtO(MH='Otorhinolaryngologic Surgical Procedures', MS='Surgery performed on the ear and its parts, the nose and nasal cavity, or the throat, including surgery of the adenoids, tonsils, pharynx, and trachea.'),
  'D013518': NtO(MH='Surgery, Plastic', MS='The branch of surgery concerned with restoration, reconstruction, or improvement of defective, damaged, or missing structures.'),
  'D013519': NtO(MH='Urogenital Surgical Procedures', MS='Surgery performed on the urinary tract or its organs and on the male or female genitalia.'),
  'D013520': NtO(MH='Urologic Surgical Procedures', MS='Surgery performed on the urinary tract or its parts in the male or female. For surgery of the male genitalia, UROLOGIC SURGICAL PROCEDURES, MALE is available.'),
  'D013521': NtO(MH='Urologic Surgical Procedures, Male', MS='Surgery performed on the male genitalia.'),
  'D013522': NtO(MH='Surgery, Veterinary', MS='A board-certified specialty of VETERINARY MEDICINE, requiring at least four years of special education, training, and practice of veterinary surgery after graduation from veterinary school. In the written, oral, and practical examinations candidates may choose either large or small animal surgery. (From AVMA Directory, 43d ed, p278)'),
  'D013523': NtO(MH='Surgical Equipment', MS='Nonexpendable apparatus used during surgical procedures. They are differentiated from SURGICAL INSTRUMENTS, usually hand-held and used in the immediate operative field.'),
  'D013524': NtO(MH='Surgical Flaps', MS='Tongues of skin and subcutaneous tissue, sometimes including muscle, cut away from the underlying parts but often still attached at one end. They retain their own microvasculature which is also transferred to the new site. They are often used in plastic surgery for filling a defect in a neighboring region.'),
  'D013525': NtO(MH='Surgical Instruments', MS='Hand-held tools or implements used by health professionals for the performance of surgical tasks.'),
  'D013526': NtO(MH='Surgical Mesh', MS='Any woven or knit material of open texture used in surgery for the repair, reconstruction, or substitution of tissue. The mesh is usually a synthetic fabric made of various polymers. It is occasionally made of metal.'),
  'D013527': NtO(MH='Perioperative Nursing', MS='Nursing care of the surgical patient before, during, and after surgery.'),
  'D013528': NtO(MH='Surgical Staplers', MS='Fastening devices composed of steel-tantalum alloys used to close operative wounds, especially of the skin, which minimizes infection by not introducing a foreign body that would connect external and internal regions of the body. (From Segen, Current Med Talk, 1995)'),
  'D013529': NtO(MH='Surgical Wound Dehiscence', MS='Pathologic process consisting of a partial or complete disruption of the layers of a surgical wound.'),
  'D013530': NtO(MH='Surgical Wound Infection', MS='Infection occurring at the site of a surgical incision.'),
  'D013531': NtO(MH='Surgicenters', MS="Facilities designed to serve patients who require surgical treatment exceeding the capabilities of usual physician's office yet not of such proportion as to require hospitalization."),
  'D013532': NtO(MH='Suriname', MS="A republic in the north of South America, bordered on the west by GUYANA (British Guiana) and on the east by FRENCH GUIANA. Its capital is Paramaribo. It was formerly called Netherlands Guiana or Dutch Guiana or Surinam. Suriname was first settled by the English in 1651 but was ceded to the Dutch by treaty in 1667. It became an autonomous territory under the Dutch crown in 1954 and gained independence in 1975. The country was named for the Surinam River but the meaning of that name is uncertain. (From Webster's New Geographical Dictionary, 1988, p1167 & Room, Brewer's Dictionary of Names, 1992, p526)"),
  'D013533': NtO(MH='Surrogate Mothers', MS='Women who allow themselves to be impregnated with the understanding that the offspring are to be given over to the parents who have commissioned the surrogate.'),
  'D013534': NtO(MH='Survival', MS='Continuance of life or existence especially under adverse conditions; includes methods and philosophy of survival.'),
  'D013535': NtO(MH='Suspensions', MS='Colloids with liquid continuous phase and solid dispersed phase; the term is used loosely also for solid-in-gas (AEROSOLS) and other colloidal systems; water-insoluble drugs may be given as suspensions.'),
  'D013536': NtO(MH='Suture Techniques', MS='Techniques for securing together the edges of a wound, with loops of thread or similar materials (SUTURES).'),
  'D013537': NtO(MH='Sutures', MS='Materials used in closing a surgical or traumatic wound. (From Dorland, 28th ed)'),
  'D013538': NtO(MH='Svalbard', MS='Norwegian islands located in the Arctic Ocean, halfway between Norway and the North Pole'),
  'D013539': NtO(MH='Simian virus 40', MS='A species of POLYOMAVIRUS originally isolated from Rhesus monkey kidney tissue. It produces malignancy in human and newborn hamster kidney cell cultures.'),
  'D013540': NtO(MH='Swayback', MS='Congenital locomotor ataxia of lambs, thought to be associated with copper deficiency. It is characterized clinically by progressive incoordination of the hind limbs and pathologically by disruption of neuron and myelin development in the central nervous system. It is caused by a deficiency of metabolizable copper in the ewe during the last half of her pregnancy. (Dorland, 28th ed; Stedman, 26th ed)'),
  'D013541': NtO(MH='Eswatini', MS='A kingdom in southern Africa, west of MOZAMBIQUE. Its capital is Mbabane. It became independent from Britain in 1968. The former country name Swaziland was changed in 2018. (from CIA World Factbook, 2019)'),
  'D013542': NtO(MH='Sweat', MS='The fluid excreted by the SWEAT GLANDS. It consists of water containing sodium chloride, phosphate, urea, ammonia, and other waste products.'),
  'D013543': NtO(MH='Sweat Gland Diseases', MS='Diseases of the SWEAT GLANDS.'),
  'D013544': NtO(MH='Sweat Gland Neoplasms', MS='New abnormal growth of tissue in the SWEAT GLANDS.'),
  'D013545': NtO(MH='Sweat Glands', MS='Sweat-producing structures that are embedded in the DERMIS. Each gland consists of a single tube, a coiled body, and a superficial duct.'),
  'D013546': NtO(MH='Sweating', MS='The process of exocrine secretion of the SWEAT GLANDS, including the aqueous sweat from the ECCRINE GLANDS and the complex viscous fluids of the APOCRINE GLANDS.'),
  'D013547': NtO(MH='Sweating, Gustatory', MS='An autonomic disorder characterized by excessive sweating of the forehead, upper lip, perioral region, or sternum subsequent to gustatory stimuli. The auriculotemporal syndrome features facial flushing or sweating limited to the distribution of the auriculotemporal nerve and may develop after trauma to the parotid gland, in association with PAROTID NEOPLASMS, or following their surgical removal. (From Ann Neurol 1997 Dec;42(6):973-5)'),
  'D013548': NtO(MH='Sweden', MS='Country in northern Europe, bordering the Baltic Sea, between Finland and Norway. The capital is Stockholm.'),
  'D013549': NtO(MH='Sweetening Agents', MS='Substances that sweeten food, beverages, medications, etc., such as sugar, saccharine or other low-calorie synthetic products. (From Random House Unabridged Dictionary, 2d ed)'),
  'D013550': NtO(MH='Swimming', MS='An activity in which the body is propelled through water by specific movement of the arms and/or the legs. Swimming as propulsion through water by the movement of limbs, tail, or fins of animals is often studied as a form of PHYSICAL EXERTION or endurance.'),
  'D013551': NtO(MH='Swimming Pools', MS='Excavations or containment structures filled with water and used for swimming.'),
  'D013552': NtO(MH='Swine', MS='Any of various animals that constitute the family Suidae and comprise stout-bodied, short-legged omnivorous mammals with thick skin, usually covered with coarse bristles, a rather long mobile snout, and small tail. Included are the genera Babyrousa, Phacochoerus (wart hogs), and Sus, the latter containing the domestic pig (see SUS SCROFA).'),
  'D013553': NtO(MH='Swine Diseases', MS='Diseases of domestic swine and of the wild boar of the genus Sus.'),
  'D013554': NtO(MH='Swine Erysipelas', MS='An acute and chronic contagious disease of young pigs caused by Erysipelothrix insidiosa.'),
  'D013555': NtO(MH='Swine Vesicular Disease', MS='An enterovirus infection of swine clinically indistinguishable from FOOT-AND-MOUTH DISEASE, vesicular stomatitis, and VESICULAR EXANTHEMA OF SWINE. It is caused by a strain of HUMAN ENTEROVIRUS B.'),
  'D013556': NtO(MH='Swine, Miniature', MS='Genetically developed small pigs for use in biomedical research. There are several strains - Yucatan miniature, Sinclair miniature, and Minnesota miniature.'),
  'D013557': NtO(MH='Switzerland', MS='A country in Europe. It is bordered by Austria, France, Italy, Liechtenstein, and Germany. The capital is Bern.'),
  'D013558': NtO(MH='Sydnones', MS='OXADIAZOLES bearing an oxygen at the 5-position. They are mesoionic, with delocalized positive and negative charges.'),
  'D013559': NtO(MH='Symbiosis', MS='The relationship between two different species of organisms that are interdependent; each gains benefits from the other or a relationship between different species where both of the organisms in question benefit from the presence of the other.'),
  'D013561': NtO(MH='Symbolism', MS='A concept that stands for or suggests something else by reason of its relationship, association, convention, or resemblance. The symbolism may be mental or a visible sign or representation. (From Webster, 3d ed)'),
  'D013562': NtO(MH='Sympathectomy', MS='The removal or interruption of some part of the sympathetic nervous system for therapeutic or research purposes.'),
  'D013563': NtO(MH='Sympathectomy, Chemical', MS='Sympathectomy using chemicals (e.g., 6-hydroxydopamine or guanethidine) which selectively and reversibly destroy adrenergic nerve endings while leaving cholinergic nerve endings intact.'),
  'D013564': NtO(MH='Sympathetic Nervous System', MS="The thoracolumbar division of the autonomic nervous system. Sympathetic preganglionic fibers originate in neurons of the intermediolateral column of the spinal cord and project to the paravertebral and prevertebral ganglia, which in turn project to target organs. The sympathetic nervous system mediates the body's response to stressful situations, i.e., the fight or flight reactions. It often acts reciprocally to the parasympathetic system."),
  'D013565': NtO(MH='Sympatholytics', MS='Drugs that inhibit the actions of the sympathetic nervous system by any mechanism. The most common of these are the ADRENERGIC ANTAGONISTS and drugs that deplete norepinephrine or reduce the release of transmitters from adrenergic postganglionic terminals (see ADRENERGIC AGENTS). Drugs that act in the central nervous system to reduce sympathetic activity (e.g., centrally acting alpha-2 adrenergic agonists, see ADRENERGIC ALPHA-AGONISTS) are included here.'),
  'D013566': NtO(MH='Sympathomimetics', MS='Drugs that mimic the effects of stimulating postganglionic adrenergic sympathetic nerves. Included here are drugs that directly stimulate adrenergic receptors and drugs that act indirectly by provoking the release of adrenergic transmitters.'),
  'D013567': NtO(MH='Symphysiotomy', MS="Surgical procedure to widen a woman's pelvis during childbirth."),
  'D013568': NtO(MH='Pathological Conditions, Signs and Symptoms', MS='Abnormal anatomical or physiological conditions and objective or subjective manifestations of disease, not classified as disease or syndrome.'),
  'D013569': NtO(MH='Synapses', MS="Specialized junctions at which a neuron communicates with a target cell. At classical synapses, a neuron's presynaptic terminal releases a chemical transmitter stored in synaptic vesicles which diffuses across a narrow synaptic cleft and activates receptors on the postsynaptic membrane of the target cell. The target may be a dendrite, cell body, or axon of another neuron, or a specialized region of a muscle or secretory cell. Neurons may also communicate via direct electrical coupling with ELECTRICAL SYNAPSES. Several other non-synaptic chemical or electric signal transmitting processes occur via extracellular mediated interactions."),
  'D013570': NtO(MH='Synaptic Membranes', MS='Cell membranes associated with synapses. Both presynaptic and postsynaptic membranes are included along with their integral or tightly associated specializations for the release or reception of transmitters.'),
  'D013572': NtO(MH='Synaptic Vesicles', MS='Membrane-bound compartments which contain transmitter molecules. Synaptic vesicles are concentrated at presynaptic terminals. They actively sequester transmitter molecules from the cytoplasm. In at least some synapses, transmitter release occurs by fusion of these vesicles with the presynaptic membrane, followed by exocytosis of their contents.'),
  'D013573': NtO(MH='Synaptonemal Complex', MS='The three-part structure of ribbon-like proteinaceous material that serves to align and join the paired homologous CHROMOSOMES. It is formed during the ZYGOTENE STAGE of the first meiotic division. It is a prerequisite for CROSSING OVER.'),
  'D013574': NtO(MH='Synaptosomes', MS='Pinched-off nerve endings and their contents of vesicles and cytoplasm together with the attached subsynaptic area of the membrane of the post-synaptic cell. They are largely artificial structures produced by fractionation after selective centrifugation of nervous tissue homogenates.'),
  'D013575': NtO(MH='Syncope', MS='A transient loss of consciousness and postural tone caused by diminished blood flow to the brain (i.e., BRAIN ISCHEMIA). Presyncope refers to the sensation of lightheadedness and loss of strength that precedes a syncopal event or accompanies an incomplete syncope. (From Adams et al., Principles of Neurology, 6th ed, pp367-9)'),
  'D013576': NtO(MH='Syndactyly', MS='A congenital anomaly of the hand or foot, marked by the webbing between adjacent fingers or toes. Syndactylies are classified as complete or incomplete by the degree of joining. Syndactylies can also be simple or complex. Simple syndactyly indicates joining of only skin or soft tissue; complex syndactyly marks joining of bony elements.'),
  'D013577': NtO(MH='Syndrome', MS='A characteristic symptom complex.'),
  'D013578': NtO(MH='Synephrine', MS='Sympathetic alpha-adrenergic agonist with actions like PHENYLEPHRINE. It is used as a vasoconstrictor in circulatory failure, asthma, nasal congestion, and glaucoma.'),
  'D013580': NtO(MH='Synostosis', MS='A union between adjacent bones or parts of a single bone formed by osseous material, such as ossified connecting cartilage or fibrous tissue. (Dorland, 27th ed)'),
  'D013581': NtO(MH='Synovial Cyst', MS='Non-neoplastic tumor-like lesions at joints, developed from the SYNOVIAL MEMBRANE of a joint through the JOINT CAPSULE into the periarticular tissues. They are filled with SYNOVIAL FLUID with a smooth and translucent appearance. A synovial cyst can develop from any joint, but most commonly at the back of the knee, where it is known as POPLITEAL CYST.'),
  'D013582': NtO(MH='Synovial Fluid', MS='The clear, viscous fluid secreted by the SYNOVIAL MEMBRANE. It contains mucin, albumin, fat, and mineral salts and serves to lubricate joints.'),
  'D013583': NtO(MH='Synovial Membrane', MS='The inner membrane of a joint capsule surrounding a freely movable joint. It is loosely attached to the external fibrous capsule and secretes SYNOVIAL FLUID.'),
  'D013584': NtO(MH='Sarcoma, Synovial', MS='A malignant neoplasm arising from tenosynovial tissue of the joints and in synovial cells of tendons and bursae. The legs are the most common site, but the tumor can occur in the abdominal wall and other trunk muscles. There are two recognized types: the monophasic (characterized by sheaths of monotonous spindle cells) and the biphasic (characterized by slit-like spaces or clefts within the tumor, lined by cuboidal or tall columnar epithelial cells). These sarcomas occur most commonly in the second and fourth decades of life. (From Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1363)'),
  'D013585': NtO(MH='Synovitis', MS='Inflammation of the SYNOVIAL MEMBRANE.'),
  'D013586': NtO(MH='Synovitis, Pigmented Villonodular', MS='Diffuse outgrowth arising from the SYNOVIAL MEMBRANE; SYNOVIAL BURSA; or TENDON sheath around the joint cavity, with extension to surrounding soft tissue. It is characterized by pigmented HEMOSIDERIN-containing MACROPHAGES; FOAM CELLS; and multinucleated GIANT CELLS. It usually occurs in the hands and feet, and around large joints, such as in the ankle and knee joints.'),
  'D013587': NtO(MH='Syphilis', MS='A contagious venereal disease caused by the spirochete TREPONEMA PALLIDUM.'),
  'D013588': NtO(MH='Syphilis Serodiagnosis', MS='Serologic tests for syphilis.'),
  'D013589': NtO(MH='Syphilis, Cardiovascular', MS='Cardiovascular manifestations of SYPHILIS, an infection of TREPONEMA PALLIDUM. In the late stage of syphilis, sometimes 20-30 years after the initial infection, damages are often seen in the blood vessels including the AORTA and the AORTIC VALVE. Clinical signs include syphilitic aortitis, aortic insufficiency, or aortic ANEURYSM.'),
  'D013590': NtO(MH='Syphilis, Congenital', MS="Syphilis acquired in utero and manifested by any of several characteristic tooth (Hutchinson's teeth) or bone malformations and by active mucocutaneous syphilis at birth or shortly thereafter. Ocular and neurologic changes may also occur."),
  'D013591': NtO(MH='Syphilis, Cutaneous', MS="Cutaneous lesions arising from infection with Treponema pallidum. In the primary stage, 18-21 days following infection, one or more chancres appear. If untreated, the subsequent stages of the disease appear as syphilids. These eruptions are superficial, nondestructive, exanthematic, transient, macular roseolas that may later be maculopapular or papular polymorphous or scaly, pustular, pigmented eruptions.(Arnold, Odom, and James, Andrew's Diseases of the Skin, 8th ed, p409)"),
  'D013592': NtO(MH='Syphilis, Latent', MS='The stage of syphilis that occurs following the primary (CHANCRE) and secondary stages. The patient is asymptomatic at the latent stage but remains seropositive for the SPIROCHETE.'),
  'D013593': NtO(MH='Syria', MS='Country in the Middle East, bordering the Mediterranean Sea, between Lebanon and Turkey. The capital is Damascus.'),
  'D013594': NtO(MH='Syringes', MS='Instruments used for injecting or withdrawing fluids. (Stedman, 25th ed)'),
  'D013595': NtO(MH='Syringomyelia', MS='Longitudinal cavities in the spinal cord, most often in the cervical region, which may extend for multiple spinal levels. The cavities are lined by dense, gliogenous tissue and may be associated with SPINAL CORD NEOPLASMS; spinal cord traumatic injuries; and vascular malformations. Syringomyelia is marked clinically by pain and PARESTHESIA, muscular atrophy of the hands, and analgesia with thermoanesthesia of the hands and arms, but with the tactile sense preserved (sensory dissociation). Lower extremity spasticity and incontinence may also develop. (From Adams et al., Principles of Neurology, 6th ed, p1269)'),
  'D013597': NtO(MH='Systems Analysis', MS='The analysis of an activity, procedure, method, technique, or business to determine what must be accomplished and how the necessary operations may best be accomplished.'),
  'D013598': NtO(MH='Systems Theory', MS='Principles, models, and laws that apply to complex interrelationships and interdependencies of sets of linked components which form a functioning whole, a system. Any system may be composed of components which are systems in their own right (sub-systems), such as several organs within an individual organism.'),
  'D013599': NtO(MH='Systole', MS='Period of contraction of the HEART, especially of the HEART VENTRICLES.'),
  'D013601': NtO(MH='T-Lymphocytes', MS='Lymphocytes responsible for cell-mediated immunity. Two types have been identified - cytotoxic (T-LYMPHOCYTES, CYTOTOXIC) and helper T-lymphocytes (T-LYMPHOCYTES, HELPER-INDUCER). They are formed when lymphocytes circulate through the THYMUS GLAND and differentiate to thymocytes. When exposed to an antigen, they divide rapidly and produce large numbers of new T cells sensitized to that antigen.'),
  'D013602': NtO(MH='T-Lymphocytes, Cytotoxic', MS='Immunized T-lymphocytes which can directly destroy appropriate target cells. These cytotoxic lymphocytes may be generated in vitro in mixed lymphocyte cultures (MLC), in vivo during a graft-versus-host (GVH) reaction, or after immunization with an allograft, tumor cell or virally transformed or chemically modified target cell. The lytic phenomenon is sometimes referred to as cell-mediated lympholysis (CML). These CD8-positive cells are distinct from NATURAL KILLER CELLS and NATURAL KILLER T-CELLS. There are two effector phenotypes: TC1 and TC2.'),
  'D013604': NtO(MH='T-Phages', MS='A series of 7 virulent phages which infect E. coli. The T-even phages T2, T4; (BACTERIOPHAGE T4), and T6, and the phage T5 are called "autonomously virulent" because they cause cessation of all bacterial metabolism on infection. Phages T1, T3; (BACTERIOPHAGE T3), and T7; (BACTERIOPHAGE T7) are called "dependent virulent" because they depend on continued bacterial metabolism during the lytic cycle. The T-even phages contain 5-hydroxymethylcytosine in place of ordinary cytosine in their DNA.'),
  'D013605': NtO(MH='T-2 Toxin', MS='A potent mycotoxin produced in feedstuffs by several species of the genus FUSARIUM. It elicits a severe inflammatory reaction in animals and has teratogenic effects.'),
  'D013606': NtO(MH='Tabes Dorsalis', MS="Parenchymatous NEUROSYPHILIS marked by slowly progressive degeneration of the posterior columns, posterior roots, and ganglia of the spinal cord. The condition tends to present 15 to 20 years after the initial infection and is characterized by lightening-like pains in the lower extremities, URINARY INCONTINENCE; ATAXIA; severely impaired position and vibratory sense, abnormal gait (see GAIT DISORDERS, NEUROLOGIC), OPTIC ATROPHY; Argyll-Robertson pupils, hypotonia, hyperreflexia, and trophic joint degeneration (Charcot's Joint; see ARTHROPATHY, NEUROGENIC). (From Adams et al., Principles of Neurology, 6th ed, p726)"),
  'D013607': NtO(MH='Tablets', MS='Solid dosage forms, of varying weight, size, and shape, which may be molded or compressed, and which contain a medicinal substance in pure or diluted form. (Dorland, 28th ed)'),
  'D013608': NtO(MH='Tablets, Enteric-Coated', MS='Tablets coated with material that delays release of the medication until after they leave the stomach. (Dorland, 28th ed)'),
  'D013609': NtO(MH='Taboo', MS='Any negative tradition or behavior that is generally regarded as harmful to social welfare and forbidden within a cultural or social group.'),
  'D013610': NtO(MH='Tachycardia', MS='Abnormally rapid heartbeat, usually with a HEART RATE above 100 beats per minute for adults. Tachycardia accompanied by disturbance in the cardiac depolarization (cardiac arrhythmia) is called tachyarrhythmia.'),
  'D013611': NtO(MH='Tachycardia, Atrioventricular Nodal Reentry', MS='Abnormally rapid heartbeats caused by reentry of atrial impulse into the dual (fast and slow) pathways of ATRIOVENTRICULAR NODE. The common type involves a blocked atrial impulse in the slow pathway which reenters the fast pathway in a retrograde direction and simultaneously conducts to the atria and the ventricles leading to rapid HEART RATE of 150-250 beats per minute.'),
  'D013612': NtO(MH='Tachycardia, Ectopic Atrial', MS='Abnormally rapid heartbeats originating from one or more automatic foci (nonsinus pacemakers) in the HEART ATRIUM but away from the SINOATRIAL NODE. Unlike the reentry mechanism, automatic tachycardia speeds up and slows down gradually. The episode is characterized by a HEART RATE between 135 to less than 200 beats per minute and lasting 30 seconds or longer.'),
  'D013613': NtO(MH='Tachycardia, Ectopic Junctional', MS='A rare form of supraventricular tachycardia caused by automatic, not reentrant, conduction initiated from sites at the atrioventricular junction, but not the ATRIOVENTRICULAR NODE. It usually occurs during myocardial infarction, after heart surgery, or in digitalis intoxication with a HEART RATE ranging from 140 to 250 beats per minute.'),
  'D013614': NtO(MH='Tachycardia, Paroxysmal', MS='Abnormally rapid heartbeats with sudden onset and cessation.'),
  'D013615': NtO(MH='Tachycardia, Sinoatrial Nodal Reentry', MS='Abnormally rapid heartbeats caused by reentry circuit in or around the SINOATRIAL NODE. It is characterized by sudden onset and offset episodes of tachycardia with a HEART RATE of 100-150 beats per minute. The P wave is identical to the sinus P wave but with a longer PR interval.'),
  'D013616': NtO(MH='Tachycardia, Sinus', MS='Simple rapid heartbeats caused by rapid discharge of impulses from the SINOATRIAL NODE, usually between 100 and 180 beats/min in adults. It is characterized by a gradual onset and termination. Sinus tachycardia is common in infants, young children, and adults during strenuous physical activities.'),
  'D013617': NtO(MH='Tachycardia, Supraventricular', MS='A generic expression for any tachycardia that originates above the BUNDLE OF HIS.'),
  'D013618': NtO(MH='Tachyphylaxis', MS='Rapidly decreasing response to a drug or physiologically active agent after administration of a few doses. In immunology, it is the rapid immunization against the effect of toxic doses of an extract or serum by previous injection of small doses. (Dorland, 28th ed)'),
  'D013619': NtO(MH='Tacrine', MS="A cholinesterase inhibitor that crosses the blood-brain barrier. Tacrine has been used to counter the effects of muscle relaxants, as a respiratory stimulant, and in the treatment of Alzheimer's disease and other central nervous system disorders."),
  'D013620': NtO(MH='Tajikistan', MS='Country in central Asia, west of China, south of Kyrgyzstan. The capital is Dushanbe.'),
  'D013621': NtO(MH='Taenia', MS='A genus of large tapeworms.'),
  'D013622': NtO(MH='Taeniasis', MS='Infection with tapeworms of the genus Taenia.'),
  'D013623': NtO(MH='Tail', MS='An extension of the posterior of an animal body beyond the TORSO.'),
  'D013624': NtO(MH='Taiwan', MS='Country in eastern Asia, islands bordering the East China Sea, Philippine Sea, South China Sea, and Taiwan Strait, north of the Philippines, off the southeastern coast of China. The capital is Taipei. The alternate country name is Republic of China.'),
  'D013625': NtO(MH='Takayasu Arteritis', MS='A chronic inflammatory process that affects the AORTA and its primary branches, such as the brachiocephalic artery (BRACHIOCEPHALIC TRUNK) and CAROTID ARTERIES. It results in progressive arterial stenosis, occlusion, and aneurysm formation. The pulse in the arm is hard to detect. Patients with aortitis syndrome often exhibit retinopathy.'),
  'D013626': NtO(MH='Talampicillin', MS='An ester of AMPICILLIN which is readily hydrolyzed on absorption to release ampicillin. It is well absorbed from the gastrointestinal tract resulting in a greater bioavailability of ampicillin than can be achieved with equivalent doses of ampicillin.'),
  'D013627': NtO(MH='Talc', MS='Finely powdered native hydrous magnesium silicate. It is used as a dusting powder, either alone or with starch or boric acid, for medicinal and toilet preparations. It is also an excipient and filler for pills, tablets, and for dusting tablet molds. (From Merck Index, 11th ed)'),
  'D013628': NtO(MH='Talus', MS='The second largest of the TARSAL BONES. It articulates with the TIBIA and FIBULA to form the ANKLE JOINT.'),
  'D013629': NtO(MH='Tamoxifen', MS='One of the SELECTIVE ESTROGEN RECEPTOR MODULATORS with tissue-specific activities. Tamoxifen acts as an anti-estrogen (inhibiting agent) in the mammary tissue, but as an estrogen (stimulating agent) in cholesterol metabolism, bone density, and cell proliferation in the ENDOMETRIUM.'),
  'D013630': NtO(MH='Tampons, Surgical', MS='Plugs or cylinders made of cotton, sponge, or other absorbent material. They are used in surgery to absorb fluids such as blood or drainage.'),
  'D013631': NtO(MH='Tangier Disease', MS='An autosomal recessively inherited disorder caused by mutation of ATP-BINDING CASSETTE TRANSPORTERS involved in cellular cholesterol removal (reverse-cholesterol transport). It is characterized by near absence of ALPHA-LIPOPROTEINS (high-density lipoproteins) in blood. The massive tissue deposition of cholesterol esters results in HEPATOMEGALY; SPLENOMEGALY; RETINITIS PIGMENTOSA; large orange tonsils; and often sensory POLYNEUROPATHY. The disorder was first found among inhabitants of Tangier Island in the Chesapeake Bay, MD.'),
  'D013633': NtO(MH='Tanning', MS='A process of preserving animal hides by chemical treatment (using vegetable tannins, metallic sulfates, and sulfurized phenol compounds, or syntans) to make them immune to bacterial attack, and subsequent treatments with fats and greases to make them pliable. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D013634': NtO(MH='Tannins', MS='Polyphenolic compounds with molecular weights of around 500-3000 daltons and containing enough hydroxyl groups (1-2 per 100 MW) for effective cross linking of other compounds (ASTRINGENTS). The two main types are HYDROLYZABLE TANNINS and CONDENSED TANNINS. Historically, the term has applied to many compounds and plant extracts able to render skin COLLAGEN impervious to degradation. The word tannin derives from the Celtic word for OAK TREE which was used for leather processing.'),
  'D013635': NtO(MH='Tantalum', MS='A rare metallic element, atomic number 73, atomic weight 180.948, symbol Ta. It is a noncorrosive and malleable metal that has been used for plates or disks to replace cranial defects, for wire sutures, and for making prosthetic devices.'),
  'D013636': NtO(MH='Tanzania', MS='A republic in eastern Africa, south of UGANDA and north of MOZAMBIQUE. Its capital is Dar es Salaam. It was formed in 1964 by a merger of the countries of TANGANYIKA and ZANZIBAR.'),
  'D013637': NtO(MH='Tape Recording', MS='Recording of information on magnetic or punched paper tape.'),
  'D013638': NtO(MH='Tars', MS='Viscous materials composed of complex, high-molecular-weight compounds derived from the distillation of petroleum or the destructive distillation of wood or coal. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013639': NtO(MH='Tarsal Bones', MS='The seven bones which form the tarsus - namely, CALCANEUS; TALUS; cuboid, navicular, and the internal, middle, and external cuneiforms.'),
  'D013640': NtO(MH='Tarsal Joints', MS='The articulations between the various TARSAL BONES. This does not include the ANKLE JOINT which consists of the articulations between the TIBIA; FIBULA; and TALUS.'),
  'D013641': NtO(MH='Tarsal Tunnel Syndrome', MS="Entrapment of the distal branches of the posterior TIBIAL NERVE (which divides into the medial plantar, lateral plantar, and calcanial nerves) in the tarsal tunnel, which lies posterior to the internal malleolus and beneath the retinaculum of the flexor muscles of the foot. Symptoms include ankle pain radiating into the foot which tends to be aggravated by walking. Examination may reveal Tinel's sign (radiating pain following nerve percussion) over the tibial nerve at the ankle, weakness and atrophy of the small foot muscles, or loss of sensation in the foot. (From Foot Ankle 1990;11(1):47-52)"),
  'D013643': NtO(MH='Tarsus, Animal', MS='The region in the hindlimb of a quadruped, corresponding to the human ANKLE.'),
  'D013644': NtO(MH='Tartrates', MS='Derivatives of tartronic acid (the structural formula (COOH)CHOH(COOH)) including its salts and esters.'),
  'D013645': NtO(MH='Tartrazine', MS='An anionic, hydrophilic azo dye with an orange-yellow color used in fabrics, foods and cosmetics, and as a biological stain.'),
  'D013646': NtO(MH='Tartronates', MS='Derivatives of tartronic acid (the structural formula (COOH)CHOH(COOH)) including its salts and esters.'),
  'D013647': NtO(MH='Task Performance and Analysis', MS='The detailed examination of observable activity or behavior associated with the execution or completion of a required function or unit of work.'),
  'D013648': NtO(MH='Tasmania', MS="An island south of Australia and the smallest state of the Commonwealth. Its capital is Hobart. It was discovered and named Van Diemen's Island in 1642 by Abel Tasman, a Dutch navigator, in honor of the Dutch governor-general of the Dutch East Indian colonies. It was renamed for the discoverer in 1853. In 1803 it was taken over by Great Britain and was used as a penal colony. It was granted government in 1856 and federated as a state in 1901. (From Webster's New Geographical Dictionary, 1988, p1190 & Room, Brewer's Dictionary of Names, p535)"),
  'D013649': NtO(MH='Taste', MS='The ability to detect chemicals through gustatory receptors in the mouth, including those on the TONGUE; the PALATE; the PHARYNX; and the EPIGLOTTIS.'),
  'D013650': NtO(MH='Taste Buds', MS='Small sensory organs which contain gustatory receptor cells, basal cells, and supporting cells. Taste buds in humans are found in the epithelia of the tongue, palate, and pharynx. They are innervated by the CHORDA TYMPANI NERVE (a branch of the facial nerve) and the GLOSSOPHARYNGEAL NERVE.'),
  'D013651': NtO(MH='Taste Disorders', MS='Conditions characterized by an alteration in gustatory function or perception. Taste disorders are frequently associated with OLFACTION DISORDERS. Additional potential etiologies include METABOLIC DISEASES; DRUG TOXICITY; and taste pathway disorders (e.g., TASTE BUD diseases; FACIAL NERVE DISEASES; GLOSSOPHARYNGEAL NERVE DISEASES; and BRAIN STEM diseases).'),
  'D013652': NtO(MH='Taste Threshold', MS='The minimum concentration at which taste sensitivity to a particular substance or food can be perceived.'),
  'D013653': NtO(MH='Tattooing', MS='The indelible marking of TISSUES, primarily SKIN, by pricking it with NEEDLES to imbed various COLORING AGENTS. Tattooing of the CORNEA is done to colorize LEUKOMA spots.'),
  'D013654': NtO(MH='Taurine', MS='A conditionally essential nutrient, important during mammalian development. It is present in milk but is isolated mostly from ox bile and strongly conjugates bile acids.'),
  'D013655': NtO(MH='Taurochenodeoxycholic Acid', MS='A bile salt formed in the liver by conjugation of chenodeoxycholate with taurine, usually as the sodium salt. It acts as detergent to solubilize fats in the small intestine and is itself absorbed. It is used as a cholagogue and choleretic.'),
  'D013656': NtO(MH='Taurocholic Acid', MS='The product of conjugation of cholic acid with taurine. Its sodium salt is the chief ingredient of the bile of carnivorous animals. It acts as a detergent to solubilize fats for absorption and is itself absorbed. It is used as a cholagogue and cholerectic.'),
  'D013657': NtO(MH='Taurodeoxycholic Acid', MS='A bile salt formed in the liver by conjugation of deoxycholate with taurine, usually as the sodium salt. It is used as a cholagogue and choleretic, also industrially as a fat emulsifier.'),
  'D013658': NtO(MH='Taurolithocholic Acid', MS='A bile salt formed in the liver from lithocholic acid conjugation with taurine, usually as the sodium salt. It solubilizes fats for absorption and is itself absorbed. It is a cholagogue and choleretic.'),
  'D013659': NtO(MH='Tax Equity and Fiscal Responsibility Act', MS='PL97-248. Title II of the Act specifies "provisions relating to savings in health and income security programs." This includes changes in payment for services, benefits and premiums of Medicare as well as changes in provisions under Medicaid and other specific programs covered by Social Security. Title II includes various revenue measures.'),
  'D013660': NtO(MH='Taxes', MS='Governmental levies on property, inheritance, gifts, etc.'),
  'D013661': NtO(MH='Tay-Sachs Disease', MS='An autosomal recessive neurodegenerative disorder characterized by the onset in infancy of an exaggerated startle response, followed by paralysis, dementia, and blindness. It is caused by mutation in the alpha subunit of the HEXOSAMINIDASE A resulting in lipid-laden ganglion cells. It is also known as the B variant (with increased HEXOSAMINIDASE B but absence of hexosaminidase A) and is strongly associated with Ashkenazic Jewish ancestry.'),
  'D013662': NtO(MH='Tea', MS='The infusion of leaves of CAMELLIA SINENSIS (formerly Thea sinensis) as a beverage, the familiar Asian tea, which contains CATECHIN (especially epigallocatechin gallate) and CAFFEINE.'),
  'D013663': NtO(MH='Teaching', MS='A formal and organized process of transmitting knowledge to a person or group.'),
  'D013664': NtO(MH='Teaching Materials', MS='Instructional materials used in teaching.'),
  'D013665': NtO(MH='Tear Gases', MS='Gases that irritate the eyes, throat, or skin. Severe lacrimation develops upon irritation of the eyes.'),
  'D013666': NtO(MH='Tears', MS='The fluid secreted by the lacrimal glands. This fluid moistens the CONJUNCTIVA and CORNEA.'),
  'D013667': NtO(MH='Technetium', MS='The first artificially produced element and a radioactive fission product of URANIUM. Technetium has the atomic symbol Tc, and atomic number 43. All technetium isotopes are radioactive. Technetium 99m (m=metastable) which is the decay product of Molybdenum 99, has a half-life of about 6 hours and is used diagnostically as a radioactive imaging agent. Technetium 99 which is a decay product of technetium 99m, has a half-life of 210,000 years.'),
  'D013668': NtO(MH='Technetium Tc 99m Aggregated Albumin', MS='A gamma-emitting radionuclide imaging agent used for the diagnosis of diseases in many tissues, particularly in cardiovascular and cerebral circulation.'),
  'D013669': NtO(MH='Technetium Tc 99m Medronate', MS='A gamma-emitting radionuclide imaging agent used primarily in skeletal scintigraphy. Because of its absorption by a variety of tumors, it is useful for the detection of neoplasms.'),
  'D013670': NtO(MH='Sodium Pertechnetate Tc 99m', MS='A gamma-emitting radionuclide imaging agent used for the diagnosis of diseases in many tissues, particularly in the gastrointestinal system, cardiovascular and cerebral circulation, brain, thyroid, and joints.'),
  'D013671': NtO(MH='Technetium Tc 99m Sulfur Colloid', MS='A gamma-emitting radionuclide imaging agent used for the diagnosis of diseases in many tissues, particularly in the gastrointestinal system, liver, and spleen.'),
  'D013672': NtO(MH='Technology', MS='The application of scientific knowledge to practical purposes in any field. It includes methods, techniques, and instrumentation.'),
  'D013673': NtO(MH='Technology Assessment, Biomedical', MS='Evaluation of biomedical technology in relation to cost, efficacy, utilization, etc., and its future impact on social, ethical, and legal systems.'),
  'D013674': NtO(MH='Technology, Dental', MS='The field of dentistry involved in procedures for designing and constructing dental appliances. It includes also the application of any technology to the field of dentistry.'),
  'D013675': NtO(MH='Technology, High-Cost', MS='Advanced technology that is costly, requires highly skilled personnel, and is unique in its particular application. Includes innovative, specialized medical/surgical procedures as well as advanced diagnostic and therapeutic equipment.'),
  'D013676': NtO(MH='Technology, Industry, and Agriculture', MS='The science and application of techniques for the extraction of raw materials, provision of services, manufacture of goods, cultivation of crops, and raising of livestock.'),
  'D013677': NtO(MH='Medical Laboratory Science', MS='The specialty related to the performance of techniques in clinical pathology such as those in hematology, microbiology, and other general clinical laboratory applications.'),
  'D013678': NtO(MH='Technology, Pharmaceutical', MS='The application of scientific knowledge or technology to pharmacy and the pharmaceutical industry. It includes methods, techniques, and instrumentation in the manufacture, preparation, compounding, dispensing, packaging, and storing of drugs and other preparations used in diagnostic and determinative procedures, and in the treatment of patients.'),
  'D013679': NtO(MH='Technology, Radiologic', MS='The application of scientific knowledge or technology to the field of radiology. The applications center mostly around x-ray or radioisotopes for diagnostic and therapeutic purposes but the technological applications of any radiation or radiologic procedure is within the scope of radiologic technology.'),
  'D013680': NtO(MH='Tectorial Membrane', MS='A membrane, attached to the bony SPIRAL LAMINA, overlying and coupling with the hair cells of the ORGAN OF CORTI in the inner ear. It is a glycoprotein-rich keratin-like layer containing fibrils embedded in a dense amorphous substance.'),
  'D013681': NtO(MH='Tegmentum Mesencephali', MS='Portion of midbrain situated under the dorsal TECTUM MESENCEPHALI. The two ventrolateral cylindrical masses or peduncles are large nerve fiber bundles providing a tract of passage between the FOREBRAIN with the HINDBRAIN. Ventral MIDBRAIN also contains three colorful structures: the GRAY MATTER (PERIAQUEDUCTAL GRAY), the black substance (SUBSTANTIA NIGRA), and the RED NUCLEUS.'),
  'D013682': NtO(MH='Teichoic Acids', MS='Bacterial polysaccharides that are rich in phosphodiester linkages. They are the major components of the cell walls and membranes of many bacteria.'),
  'D013683': NtO(MH='Telangiectasia, Hereditary Hemorrhagic', MS='An autosomal dominant vascular anomaly characterized by telangiectases of the skin and mucous membranes and by recurrent gastrointestinal bleeding. This disorder is caused by mutations of a gene (on chromosome 9q3) which encodes endoglin, a membrane glycoprotein that binds TRANSFORMING GROWTH FACTOR BETA.'),
  'D013684': NtO(MH='Telangiectasis', MS='Permanent dilation of preexisting blood vessels (CAPILLARIES; ARTERIOLES; VENULES) creating small focal red lesions, most commonly in the skin or mucous membranes. It is characterized by the prominence of skin blood vessels, such as vascular spiders.'),
  'D013685': NtO(MH='Telecommunications', MS='Transmission of information over distances via electronic means.'),
  'D013686': NtO(MH='Telemetry', MS='Transmission of the readings of instruments to a remote location by means of wires, radio waves, or other means. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D013687': NtO(MH='Telencephalon', MS='The anterior subdivision of the embryonic PROSENCEPHALON or the corresponding part of the adult prosencephalon that includes the cerebrum and associated structures.'),
  'D013688': NtO(MH='Telepathy', MS='The knowledge or communication by one person with the mental processes of another through channels other than known physical or perceptual processes.'),
  'D013689': NtO(MH='Telephone', MS='An instrument for reproducing sounds especially articulate speech at a distance. (Webster, 3rd ed)'),
  'D013690': NtO(MH='Television', MS='The transmission and reproduction of transient images of fixed or moving objects. An electronic system of transmitting such images together with sound over a wire or through space by apparatus that converts light and sound into electrical waves and reconverts them into visible light rays and audible sound. (From Webster, 3rd ed)'),
  'D013691': NtO(MH='Tellurium', MS='An element that is a member of the chalcogen family. It has the atomic symbol Te, atomic number 52, and atomic weight 127.60. It has been used as a coloring agent and in the manufacture of electrical equipment. Exposure may cause nausea, vomiting, and CNS depression.'),
  'D013692': NtO(MH='Telophase', MS='The final phase of cell nucleus division following ANAPHASE, in which two daughter nuclei are formed, the CYTOPLASM completes division, and the CHROMOSOMES lose their distinctness and are transformed into CHROMATIN threads.'),
  'D013693': NtO(MH='Temazepam', MS='A benzodiazepine that acts as a GAMMA-AMINOBUTYRIC ACID modulator and anti-anxiety agent.'),
  'D013694': NtO(MH='Temperament', MS="Predisposition to react to one's environment in a certain way; usually refers to mood changes."),
  'D013695': NtO(MH='Temperance', MS='Habitual moderation in the indulgence of a natural appetite, especially but not exclusively the consumption of alcohol.'),
  'D013696': NtO(MH='Temperature', MS='The property of objects that determines the direction of heat flow when they are placed in direct thermal contact. The temperature is the energy of microscopic motions (vibrational and translational) of the particles of atoms.'),
  'D013697': NtO(MH='Thermosensing', MS='The sensation of cold, heat, coolness, and warmth as detected by THERMORECEPTORS.'),
  'D013698': NtO(MH='Templates, Genetic', MS='Macromolecular molds for the synthesis of complementary macromolecules, as in DNA REPLICATION; GENETIC TRANSCRIPTION of DNA to RNA, and GENETIC TRANSLATION of RNA into POLYPEPTIDES.'),
  'D013699': NtO(MH='Temporal Arteries', MS='Arteries arising from the external carotid or the maxillary artery and distributing to the temporal region.'),
  'D013700': NtO(MH='Giant Cell Arteritis', MS='A systemic autoimmune disorder that typically affects medium and large ARTERIES, usually leading to occlusive granulomatous vasculitis with transmural infiltrate containing multinucleated GIANT CELLS. The TEMPORAL ARTERY is commonly involved. This disorder appears primarily in people over the age of 50. Symptoms include FEVER; FATIGUE; HEADACHE; visual impairment; pain in the jaw and tongue; and aggravation of pain by cold temperatures. (From Adams et al., Principles of Neurology, 6th ed)'),
  'D013701': NtO(MH='Temporal Bone', MS='Either of a pair of compound bones forming the lateral (left and right) surfaces and base of the skull which contains the organs of hearing. It is a large bone formed by the fusion of parts: the squamous (the flattened anterior-superior part), the tympanic (the curved anterior-inferior part), the mastoid (the irregular posterior portion), and the petrous (the part at the base of the skull).'),
  'D013702': NtO(MH='Temporal Lobe', MS='Lower lateral part of the cerebral hemisphere responsible for auditory, olfactory, and semantic processing. It is located inferior to the lateral fissure and anterior to the OCCIPITAL LOBE.'),
  'D013703': NtO(MH='Temporal Muscle', MS='A masticatory muscle whose action is closing the jaws; its posterior portion retracts the mandible.'),
  'D013704': NtO(MH='Temporomandibular Joint', MS='An articulation between the condyle of the mandible and the articular tubercle of the temporal bone.'),
  'D013705': NtO(MH='Temporomandibular Joint Disorders', MS="A variety of conditions affecting the anatomic and functional characteristics of the temporomandibular joint. Factors contributing to the complexity of temporomandibular diseases are its relation to dentition and mastication and the symptomatic effects in other areas which account for referred pain to the joint and the difficulties in applying traditional diagnostic procedures to temporomandibular joint pathology where tissue is rarely obtained and x-rays are often inadequate or nonspecific. Common diseases are developmental abnormalities, trauma, subluxation, luxation, arthritis, and neoplasia. (From Thoma's Oral Pathology, 6th ed, pp577-600)"),
  'D013706': NtO(MH='Temporomandibular Joint Dysfunction Syndrome', MS='A symptom complex consisting of pain, muscle tenderness, clicking in the joint, and limitation or alteration of mandibular movement. The symptoms are subjective and manifested primarily in the masticatory muscles rather than the temporomandibular joint itself. Etiologic factors are uncertain but include occlusal dysharmony and psychophysiologic factors.'),
  'D013708': NtO(MH='Tendon Injuries', MS='Injuries to the fibrous cords of connective tissue which attach muscles to bones or other structures.'),
  'D013709': NtO(MH='Tendon Transfer', MS='Surgical procedure by which a tendon is incised at its insertion and placed at an anatomical site distant from the original insertion. The tendon remains attached at the point of origin and takes over the function of a muscle inactivated by trauma or disease.'),
  'D013710': NtO(MH='Tendons', MS='Fibrous bands or cords of CONNECTIVE TISSUE at the ends of SKELETAL MUSCLE FIBERS that serve to attach the MUSCLES to bones and other structures.'),
  'D013712': NtO(MH='Tenebrio', MS='A genus of beetles which infests grain products. Its larva is called mealworm.'),
  'D013713': NtO(MH='Teniposide', MS='A semisynthetic derivative of PODOPHYLLOTOXIN that exhibits antitumor activity. Teniposide inhibits DNA synthesis by forming a complex with topoisomerase II and DNA. This complex induces breaks in double stranded DNA and prevents repair by topoisomerase II binding. Accumulated breaks in DNA prevent cells from entering into the mitotic phase of the cell cycle, and lead to cell death. Teniposide acts primarily in the G2 and S phases of the cycle.'),
  'D013714': NtO(MH='Tennessee', MS='State bounded on the north by Kentucky and Virginia, on the east by North Carolina, on the south by Georgia, Alabama and Mississippi, and on the west by Arkansas and Missouri.'),
  'D013715': NtO(MH='Tennis', MS='A game played by two or four players with rackets and an elastic ball on a level court divided by a low net.'),
  'D013716': NtO(MH='Tennis Elbow', MS='A condition characterized by pain in or near the lateral humeral epicondyle or in the forearm extensor muscle mass as a result of unusual strain. It occurs due repetitive stresses on the elbow from activities such as tennis playing.'),
  'D013717': NtO(MH='Tenosynovitis', MS='Inflammation of the synovial lining of a tendon sheath. Causes include trauma, tendon stress, bacterial disease (gonorrhea, tuberculosis), rheumatic disease, and gout. Common sites are the hand, wrist, shoulder capsule, hip capsule, hamstring muscles, and Achilles tendon. The tendon sheaths become inflamed and painful, and accumulate fluid. Joint mobility is usually reduced.'),
  'D013718': NtO(MH='Tensile Strength', MS='The maximum stress a material subjected to a stretching load can withstand without tearing. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed, p2001)'),
  'D013719': NtO(MH='Tensor Tympani', MS='A short muscle that arises from the pharyngotympanic tube (EUSTACHIAN TUBE) and inserts into the handle of the MALLEUS. This muscle pulls the handle medially thus controlling the tension and movement of TYMPANIC MEMBRANE.'),
  'D013720': NtO(MH='Tenuazonic Acid', MS='3-Acetyl-5-sec-butyl-4-hydroxy-3-pyrrolin-2-one. A metabolite found in a strain of the fungus Alternaria tenuis Auct. which functions as an antibiotic with antiviral and antineoplastic properties, and may also act as a mycotoxin.'),
  'D013721': NtO(MH='Triethylenephosphoramide', MS='An insect chemosterilant and an antineoplastic agent.'),
  'D013722': NtO(MH='Teprotide', MS='A synthetic nonapeptide (Pyr-Trp-Pro-Arg-Pro-Gln-Ile-Pro-Pro) which is identical to the peptide from the venom of the snake, Bothrops jararaca. It inhibits kininase II and ANGIOTENSIN I and has been proposed as an antihypertensive agent.'),
  'D013723': NtO(MH='Teratogens', MS='An agent that causes the production of physical defects in the developing embryo.'),
  'D013724': NtO(MH='Teratoma', MS='A true neoplasm composed of a number of different types of tissue, none of which is native to the area in which it occurs. It is composed of tissues that are derived from three germinal layers, the endoderm, mesoderm, and ectoderm. They are classified histologically as mature (benign) or immature (malignant). (From DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1642)'),
  'D013725': NtO(MH='Terbium', MS='An element of the rare earth family of metals. It has the atomic symbol Tb, atomic number 65, and atomic weight 158.92.'),
  'D013726': NtO(MH='Terbutaline', MS='A selective beta-2 adrenergic agonist used as a bronchodilator and tocolytic.'),
  'D013727': NtO(MH='Terminal Care', MS='Medical and nursing care of patients in the terminal stage of an illness.'),
  'D013728': NtO(MH='Terminator Regions, Genetic', MS='DNA sequences recognized as signals to end GENETIC TRANSCRIPTION.'),
  'D013729': NtO(MH='Terpenes', MS='A class of compounds composed of repeating 5-carbon units of HEMITERPENES.'),
  'D013730': NtO(MH='Terphenyl Compounds', MS='Compounds consisting of benzene rings linked to each other in either ortho, meta or para positions. Permitted are any substitutions, but ring fusion to any of the benzene rings is not allowed.'),
  'D013731': NtO(MH='Territoriality', MS='Behavior in defense of an area against another individual or individuals primarily of the same species.'),
  'D013732': NtO(MH='Test Anxiety Scale', MS='A self-reporting test consisting of items concerning fear and worry about taking tests and physiological activity, such as heart rate, sweating, etc., before, during, and after tests.'),
  'D013733': NtO(MH='Testicular Diseases', MS='Pathological processes of the TESTIS.'),
  'D013734': NtO(MH='Androgen-Insensitivity Syndrome', MS='A disorder of sexual development transmitted as an X-linked recessive trait. These patients have a karyotype of 46,XY with end-organ resistance to androgen due to mutations in the androgen receptor (RECEPTORS, ANDROGEN) gene. Severity of the defect in receptor quantity or quality correlates with their phenotypes. In these genetic males, the phenotypic spectrum ranges from those with normal female external genitalia, through those with genital ambiguity as in Reifenstein Syndrome, to that of a normal male with INFERTILITY.'),
  'D013735': NtO(MH='Testicular Hormones', MS='Hormones produced in the testis.'),
  'D013736': NtO(MH='Testicular Neoplasms', MS='Tumors or cancer of the TESTIS. Germ cell tumors (GERMINOMA) of the testis constitute 95% of all testicular neoplasms.'),
  'D013737': NtO(MH='Testis', MS='The male gonad containing two functional parts: the SEMINIFEROUS TUBULES for the production and transport of male germ cells (SPERMATOGENESIS) and the interstitial compartment containing LEYDIG CELLS that produce ANDROGENS.'),
  'D013738': NtO(MH='Testolactone', MS='An antineoplastic agent that is a derivative of progesterone and used to treat advanced breast cancer.'),
  'D013739': NtO(MH='Testosterone', MS='A potent androgenic steroid and major product secreted by the LEYDIG CELLS of the TESTIS. Its production is stimulated by LUTEINIZING HORMONE from the PITUITARY GLAND. In turn, testosterone exerts feedback control of the pituitary LH and FSH secretion. Depending on the tissues, testosterone can be further converted to DIHYDROTESTOSTERONE or ESTRADIOL.'),
  'D013741': NtO(MH='3-Oxo-5-alpha-Steroid 4-Dehydrogenase', MS='An enzyme that catalyzes the reduction of TESTOSTERONE to 5-ALPHA DIHYDROTESTOSTERONE.'),
  'D013742': NtO(MH='Tetanus', MS='A disease caused by tetanospasmin, a powerful protein toxin produced by CLOSTRIDIUM TETANI. Tetanus usually occurs after an acute injury, such as a puncture wound or laceration. Generalized tetanus, the most common form, is characterized by tetanic muscular contractions and hyperreflexia. Localized tetanus presents itself as a mild condition with manifestations restricted to muscles near the wound. It may progress to the generalized form.'),
  'D013743': NtO(MH='Tetanus Antitoxin', MS='An antitoxin used for the treatment of TETANUS.'),
  'D013744': NtO(MH='Tetanus Toxin', MS='Protein synthesized by CLOSTRIDIUM TETANI as a single chain of ~150 kDa with 35% sequence identity to BOTULINUM TOXIN that is cleaved to a light and a heavy chain that are linked by a single disulfide bond. Tetanolysin is the hemolytic and tetanospasmin is the neurotoxic principle. The toxin causes disruption of the inhibitory mechanisms of the CNS, thus permitting uncontrolled nervous activity, leading to fatal CONVULSIONS.'),
  'D013745': NtO(MH='Tetanus Toxoid', MS=None),
  'D013746': NtO(MH='Tetany', MS='A disorder characterized by muscle twitches, cramps, and carpopedal spasm, and when severe, laryngospasm and seizures. This condition is associated with unstable depolarization of axonal membranes, primarily in the peripheral nervous system. Tetany usually results from HYPOCALCEMIA or reduced serum levels of MAGNESIUM that may be associated with HYPERVENTILATION; HYPOPARATHYROIDISM; RICKETS; UREMIA; or other conditions. (From Adams et al., Principles of Neurology, 6th ed, p1490)'),
  'D013747': NtO(MH='Tetrabenazine', MS='A drug formerly used as an antipsychotic and treatment of various movement disorders. Tetrabenazine blocks neurotransmitter uptake into adrenergic storage vesicles and has been used as a high affinity label for the vesicle transport system.'),
  'D013748': NtO(MH='Tetracaine', MS='A potent local anesthetic of the ester type used for surface and spinal anesthesia.'),
  'D013750': NtO(MH='Tetrachloroethylene', MS='A chlorinated hydrocarbon used as an industrial solvent and cooling liquid in electrical transformers. It is a potential carcinogen.'),
  'D013751': NtO(MH='Tetrachlorvinphos', MS='An organophosphate cholinesterase inhibitor that is used as an insecticide. It has low mammalian toxicity. (From Merck Index, 11th ed)'),
  'D013752': NtO(MH='Tetracycline', MS='A naphthacene antibiotic that inhibits AMINO ACYL TRNA binding during protein synthesis.'),
  'D013753': NtO(MH='Tetracycline Resistance', MS='Nonsusceptibility of bacteria to the action of TETRACYCLINE which inhibits aminoacyl-tRNA binding to the 30S ribosomal subunit during protein synthesis.'),
  'D013754': NtO(MH='Tetracyclines', MS="Closely congeneric derivatives of the polycyclic naphthacenecarboxamide. (Gilman et al., Goodman and Gilman's The Pharmacological Basis of Therapeutics, 8th ed, p1117)"),
  'D013755': NtO(MH='Tetradecanoylphorbol Acetate', MS='A phorbol ester found in CROTON OIL with very effective tumor promoting activity. It stimulates the synthesis of both DNA and RNA.'),
  'D013756': NtO(MH='Tetraethyl Lead', MS='A highly toxic compound used as a gasoline additive. It causes acute toxic psychosis or chronic poisoning if inhaled or absorbed through the skin.'),
  'D013757': NtO(MH='Tetraethylammonium Compounds', MS='Quaternary ammonium compounds that consist of an ammonium cation where the central nitrogen atom is bonded to four ethyl groups.'),
  'D013758': NtO(MH='Tetragastrin', MS='L-Tryptophyl-L-methionyl-L-aspartyl-L-phenylalaninamide. The C-terminal tetrapeptide of gastrin. It is the smallest peptide fragment of gastrin which has the same physiological and pharmacological activity as gastrin.'),
  'D013759': NtO(MH='Dronabinol', MS='A psychoactive compound extracted from the resin of Cannabis sativa (marihuana, hashish). The isomer delta-9-tetrahydrocannabinol (THC) is considered the most active form, producing characteristic mood and perceptual changes associated with this compound.'),
  'D013760': NtO(MH='Tetrahydrocortisol', MS=None),
  'D013761': NtO(MH='Tetrahydrocortisone', MS=None),
  'D013762': NtO(MH='Tetrahydrofolate Dehydrogenase', MS='An enzyme of the oxidoreductase class that catalyzes the reaction 7,8-dihyrofolate and NADPH to yield 5,6,7,8-tetrahydrofolate and NADPH+, producing reduced folate for amino acid metabolism, purine ring synthesis, and the formation of deoxythymidine monophosphate. Methotrexate and other folic acid antagonists used as chemotherapeutic drugs act by inhibiting this enzyme. (Dorland, 27th ed) EC 1.5.1.3.'),
  'D013763': NtO(MH='Tetrahydrofolates', MS='Compounds based on 5,6,7,8-tetrahydrofolate.'),
  'D013764': NtO(MH='Tetrahydronaphthalenes', MS='Partially saturated 1,2,3,4-tetrahydronaphthalene compounds.'),
  'D013765': NtO(MH='Tetrahydropapaveroline', MS='A leukomaine (animal alkaloid) formed in brain and liver from dopamine and L-dopa; it may be implicated in psychiatric problems.'),
  'D013766': NtO(MH='5-Methyltetrahydrofolate-Homocysteine S-Methyltransferase', MS='An enzyme that catalyzes the formation of methionine by transfer of a methyl group from 5-methyltetrahydrofolate to homocysteine. It requires a cobamide coenzyme. The enzyme can act on mono- or triglutamate derivatives. EC 2.1.1.13.'),
  'D013767': NtO(MH='Tetrahydrouridine', MS='An inhibitor of nucleotide metabolism.'),
  'D013768': NtO(MH='Tetrahymena', MS='A genus of ciliate protozoa commonly used in genetic, cytological, and other research.'),
  'D013769': NtO(MH='Tetrahymena pyriformis', MS='A species of ciliate protozoa used extensively in genetic research.'),
  'D013770': NtO(MH='Tetraisopropylpyrophosphamide', MS="N,N',N'',N'''-Tetraisopropylpyrophosphamide. A specific inhibitor of pseudocholinesterases. It is commonly used experimentally to determine whether pseudo- or acetylcholinesterases are involved in an enzymatic process."),
  'D013771': NtO(MH='Tetralogy of Fallot', MS='A combination of congenital heart defects consisting of four key features including VENTRICULAR SEPTAL DEFECTS; PULMONARY STENOSIS; RIGHT VENTRICULAR HYPERTROPHY; and a dextro-positioned AORTA. In this condition, blood from both ventricles (oxygen-rich and oxygen-poor) is pumped into the body often causing CYANOSIS.'),
  'D013772': NtO(MH='Tetramethylphenylenediamine', MS='Used in the form of the hydrochloride as a reagent in ANALYTICAL CHEMISTRY TECHNIQUES.'),
  'D013773': NtO(MH='Tetramisole', MS=None),
  'D013774': NtO(MH='Tetranitromethane', MS='Corrosive oxidant, explosive; additive to diesel and rocket fuels; causes skin and lung irritation; proposed war gas. A useful reagent for studying the modification of specific amino acids, particularly tyrosine residues in proteins. Has also been used for studying carbanion formation and for detecting the presence of double bonds in organic compounds.'),
  'D013775': NtO(MH='Tetraphenylborate', MS='An anionic compound that is used as a reagent for determination of potassium, ammonium, rubidium, and cesium ions. It also uncouples oxidative phosphorylation and forms complexes with biological materials, and is used in biological assays.'),
  'D013776': NtO(MH='Tetrathionic Acid', MS='A sulfuric acid dimer, formed by disulfide linkage. This compound has been used to prolong coagulation time and as an antidote in cyanide poisoning.'),
  'D013777': NtO(MH='Tetrazoles', MS=None),
  'D013778': NtO(MH='Tetrazolium Salts', MS="Quaternary salts derived from tetrazoles. They are used in tests to distinguish between reducing sugars and simple aldehydes, for detection of dehydrogenase in tissues, cells, and bacteria, for determination of corticosteroids, and in color photography. (From Mall's Dictionary of Chemistry, 5th ed, p455)"),
  'D013779': NtO(MH='Tetrodotoxin', MS='An aminoperhydroquinazoline poison found mainly in the liver and ovaries of fishes in the order TETRAODONTIFORMES, which are eaten. The toxin causes paresthesia and paralysis through interference with neuromuscular conduction.'),
  'D013780': NtO(MH='Tetroses', MS='Monosaccharide sugar molecules that contain a four carbon backbone.'),
  'D013781': NtO(MH='Texas', MS='State bounded on the north by Oklahoma, on the east by Arkansas and Louisiana, on the south by Mexico, and on the west by New Mexico.'),
  'D013782': NtO(MH='Textbooks as Topic', MS='Works about books used in the study of a subject that contain a systematic presentation of the principles and vocabulary of a subject.'),
  'D013783': NtO(MH='Textile Industry', MS='The aggregate business enterprise of manufacturing textiles. (From Random House Unabridged Dictionary, 2d ed)'),
  'D013784': NtO(MH='Textiles', MS='Filament or fiber, or the materials generated from them by weaving, embroidery, knitting, crocheting, tufting, or felting such as yarn, thread, cording, rope, netting, lace, or sheets of cloth or fabric.'),
  'D013785': NtO(MH='Thailand', MS='Formerly known as Siam, this is a Southeast Asian nation at the center of the Indochina peninsula. Bangkok is the capital city.'),
  'D013786': NtO(MH='Thalamic Diseases', MS='Disorders of the centrally located thalamus, which integrates a wide range of cortical and subcortical information. Manifestations include sensory loss, MOVEMENT DISORDERS; ATAXIA, pain syndromes, visual disorders, a variety of neuropsychological conditions, and COMA. Relatively common etiologies include CEREBROVASCULAR DISORDERS; CRANIOCEREBRAL TRAUMA; BRAIN NEOPLASMS; BRAIN HYPOXIA; INTRACRANIAL HEMORRHAGES; and infectious processes.'),
  'D013787': NtO(MH='Thalamic Nuclei', MS='Several groups of nuclei in the thalamus that serve as the major relay centers for sensory impulses in the brain.'),
  'D013788': NtO(MH='Thalamus', MS='Paired bodies containing mostly GRAY MATTER and forming part of the lateral wall of the THIRD VENTRICLE of the brain.'),
  'D013789': NtO(MH='Thalassemia', MS='A group of hereditary hemolytic anemias in which there is decreased synthesis of one or more hemoglobin polypeptide chains. There are several genetic types with clinical pictures ranging from barely detectable hematologic abnormality to severe and fatal anemia.'),
  'D013790': NtO(MH='Climatotherapy', MS='Relocating a patient to a CLIMATE more suitable for health or for management of a health condition.'),
  'D013792': NtO(MH='Thalidomide', MS='A piperidinyl isoindole originally introduced as a non-barbiturate hypnotic, but withdrawn from the market due to teratogenic effects. It has been reintroduced and used for a number of immunological and inflammatory disorders. Thalidomide displays immunosuppressive and anti-angiogenic activity. It inhibits release of TUMOR NECROSIS FACTOR-ALPHA from monocytes, and modulates other cytokine action.'),
  'D013793': NtO(MH='Thallium', MS='A heavy, bluish white metal, atomic number 81, atomic weight [204.382; 204.385], symbol Tl.'),
  'D013794': NtO(MH='Thallium Radioisotopes', MS='Unstable isotopes of thallium that decay or disintegrate emitting radiation. Tl atoms with atomic weights 198-202, 204, and 206-210 are thallium radioisotopes.'),
  'D013795': NtO(MH='Thanatology', MS='The study of the theory, philosophy, and doctrine of death.'),
  'D013796': NtO(MH='Thanatophoric Dysplasia', MS='A severe form of neonatal dwarfism with very short limbs. All cases have died at birth or later in the neonatal period.'),
  'D013797': NtO(MH='Thebaine', MS='A drug that is derived from opium, which contains from 0.3-1.5% thebaine depending on its origin. It produces strychnine-like convulsions rather than narcosis. It may be habit-forming and is a controlled substance (opiate) listed in the U.S. Code of Federal Regulations, Title 21 Part 1308.12 (1985). (From Merck Index, 11th ed)'),
  'D013798': NtO(MH='Thecoma', MS='A gonadal stromal neoplasm composed only of THECA CELLS, occurring mostly in the postmenopausal OVARY. It is filled with lipid-containing spindle cells and produces ESTROGENS that can lead to ENDOMETRIAL HYPERPLASIA; UTERINE HEMORRHAGE; or other malignancies in postmenopausal women and sexual precocity in girls. When tumors containing theca cells also contain FIBROBLASTS, they are identified as thecoma-fibroma tumors with less active hormone production.'),
  'D013799': NtO(MH='Theca Cells', MS='The flattened stroma cells forming a sheath or theca outside the basal lamina lining the mature OVARIAN FOLLICLE. Thecal interstitial or stromal cells are steroidogenic, and produce primarily ANDROGENS which serve as precusors of ESTROGENS in the GRANULOSA CELLS.'),
  'D013800': NtO(MH='Theft', MS='Unlawful act of taking property.'),
  'D013801': NtO(MH='Theileriasis', MS='Infection of cattle, sheep, or goats with protozoa of the genus THEILERIA. This infection results in an acute or chronic febrile condition.'),
  'D013802': NtO(MH='Thelazioidea', MS='A superfamily of parasitic nematodes which includes three genera: Thelazia, Spirocerca, and GNATHOSTOMA. Only Thelazia and GNATHOSTOMA occasionally occur in man.'),
  'D013803': NtO(MH='Thematic Apperception Test', MS="A projective technique which focuses primarily on the dynamics of interpersonal relationships. It consists of a series of 31 pictures that depict various social situations and interpersonal relations. A subset is selected by the examiner and presented to the subject who is asked to tell a story about each picture. The stories are interpreted in terms of the subject's relations to authority figures, to contemporaries of both sexes, and in terms of the compromises between external demands and the needs of the id, the ego, and the superego. (From Campbell, Psychiatric Dictionary, 1996)"),
  'D013804': NtO(MH='Thenoyltrifluoroacetone', MS='Chelating agent and inhibitor of cellular respiration.'),
  'D013805': NtO(MH='Theobromine', MS='3,7-Dimethylxanthine. The principle alkaloid in Theobroma cacao (the cacao bean) and other plants. A xanthine alkaloid that is used as a bronchodilator and as a vasodilator. It has a weaker diuretic activity than THEOPHYLLINE and is also a less powerful stimulant of smooth muscle. It has practically no stimulant effect on the central nervous system. It was formerly used as a diuretic and in the treatment of angina pectoris and hypertension. (From Martindale, The Extra Pharmacopoeia, 30th ed, pp1318-9)'),
  'D013806': NtO(MH='Theophylline', MS="A methyl xanthine derivative from tea with diuretic, smooth muscle relaxant, bronchial dilation, cardiac and central nervous system stimulant activities. Theophylline inhibits the 3',5'-CYCLIC NUCLEOTIDE PHOSPHODIESTERASE that degrades CYCLIC AMP thus potentiates the actions of agents that act through ADENYLYL CYCLASES and cyclic AMP."),
  'D013808': NtO(MH='Therapeutic Community', MS='Psychotherapeutic technique which emphasizes socioenvironmental and interpersonal influences in the resocialization and rehabilitation of the patient. The setting is usually a hospital unit or ward in which professional and nonprofessional staff interact with the patients.'),
  'D013810': NtO(MH='Therapeutic Equivalency', MS='The relative equivalency in the efficacy of different modes of treatment of a disease, most often used to compare the efficacy of different pharmaceuticals to treat a given disease.'),
  'D013811': NtO(MH='Psychotherapeutic Processes', MS='Experiential, attitudinal, emotional, or behavioral phenomena occurring during the course of treatment. They apply to the patient or therapist (i.e., nurse, doctor, etc.) individually or to their interaction. (American Psychological Association: Thesaurus of Psychological Index Terms, 1994)'),
  'D013812': NtO(MH='Therapeutics', MS='Procedures concerned with the remedial treatment or prevention of diseases.'),
  'D013813': NtO(MH='Therapy, Computer-Assisted', MS='Computer systems utilized as adjuncts in the treatment of disease.'),
  'D013814': NtO(MH='Thermal Conductivity', MS='The heat flow across a surface per unit area per unit time, divided by the negative of the rate of change of temperature with distance in a direction perpendicular to the surface. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D013815': NtO(MH='Thermodilution', MS='Measurement of blood flow based on induction at one point of the circulation of a known change in the intravascular heat content of flowing blood and detection of the resultant change in temperature at a point downstream.'),
  'D013816': NtO(MH='Thermodynamics', MS="A rigorously mathematical analysis of energy relationships (heat, work, temperature, and equilibrium). It describes systems whose states are determined by thermal parameters, such as temperature, in addition to mechanical and electromagnetic parameters. (From Hawley's Condensed Chemical Dictionary, 12th ed)"),
  'D013817': NtO(MH='Thermography', MS='Imaging the temperatures in a material, or in the body or an organ. Imaging is based on self-emanating infrared radiation (HEAT WAVES), or on changes in properties of the material or tissue that vary with temperature, such as ELASTICITY; MAGNETIC FIELD; or LUMINESCENCE.'),
  'D013818': NtO(MH='Thermogravimetry', MS='Technique whereby the weight of a sample can be followed over a period of time while its temperature is being changed (usually increased at a constant rate).'),
  'D013819': NtO(MH='Thermoluminescent Dosimetry', MS='The use of a device composed of thermoluminescent material for measuring exposure to IONIZING RADIATION. The thermoluminescent material emits light when heated. The amount of light emitted is proportional to the amount of ionizing radiation to which the material has been exposed.'),
  'D013820': NtO(MH='Thermolysin', MS='A thermostable extracellular metalloendopeptidase containing four calcium ions. (Enzyme Nomenclature, 1992) 3.4.24.27.'),
  'D013821': NtO(MH='Thermometers', MS='Measuring instruments for determining the temperature of matter. Most thermometers used in the field of medicine are designed for measuring body temperature or for use in the clinical laboratory. (From UMDNS, 1999)'),
  'D013822': NtO(MH='Thermoplasma', MS='A genus of facultatively anaerobic heterotrophic archaea, in the order THERMOPLASMALES, isolated from self-heating coal refuse piles and acid hot springs. They are thermophilic and can grow both with and without sulfur.'),
  'D013823': NtO(MH='Thermoreceptors', MS='Cellular receptors which mediate the sense of temperature. Thermoreceptors in vertebrates are mostly located under the skin. In mammals there are separate types of thermoreceptors for cold and for warmth and NOCICEPTORS which detect cold or heat extreme enough to cause pain.'),
  'D013824': NtO(MH='Thermus', MS='Gram-negative aerobic rods found in warm water (40-79 degrees C) such as hot springs, hot water tanks, and thermally polluted rivers.'),
  'D013826': NtO(MH='Theta Rhythm', MS='Brain waves characterized by a frequency of 4-7 Hz, usually observed in the temporal lobes when the individual is awake, but relaxed and sleepy.'),
  'D013827': NtO(MH='Thiabendazole', MS='2-Substituted benzimidazole first introduced in 1962. It is active against a variety of nematodes and is the drug of choice for STRONGYLOIDIASIS. It has CENTRAL NERVOUS SYSTEM side effects and hepatototoxic potential. (From Smith and Reynard, Textbook of Pharmacology, 1992, p919)'),
  'D013828': NtO(MH='Thioacetazone', MS='A thiosemicarbazone that is used in association with other antimycobacterial agents in the initial and continuation phases of antituberculosis regimens. Thiacetazone containing regimens are less effective than the short-course regimen recommended by the International Union Against Tuberculosis and are used in some developing countries to reduce drug costs. (From Martindale, The Extra Pharmacopoeia, 30th ed, p217)'),
  'D013829': NtO(MH='Thiadiazines', MS=None),
  'D013830': NtO(MH='Thiadiazoles', MS=None),
  'D013831': NtO(MH='Thiamine', MS='3-((4-Amino-2-methyl-5-pyrimidinyl)methyl)-5-(2- hydroxyethyl)-4-methylthiazolium chloride.'),
  'D013832': NtO(MH='Thiamine Deficiency', MS='A nutritional condition produced by a deficiency of THIAMINE in the diet, characterized by anorexia, irritability, and weight loss. Later, patients experience weakness, peripheral neuropathy, headache, and tachycardia. In addition to being caused by a poor diet, thiamine deficiency in the United States most commonly occurs as a result of alcoholism, since ethanol interferes with thiamine absorption. In countries relying on polished rice as a dietary staple, BERIBERI prevalence is very high. (From Cecil Textbook of Medicine, 19th ed, p1171)'),
  'D013833': NtO(MH='Thiamine Monophosphate', MS='Thiamine dihydrogen phosphate ester. The monophosphate ester of thiamine. Synonyms: monophosphothiamine; vitamin B1 monophosphate.'),
  'D013834': NtO(MH='Thiamine Pyrophosphatase', MS='An enzyme that hydrolyzes thiamine pyrophosphate to thiamine monophosphate plus inorganic phosphate. EC 3.6.1.-.'),
  'D013835': NtO(MH='Thiamine Pyrophosphate', MS='The coenzyme form of Vitamin B1 present in many animal tissues. It is a required intermediate in the PYRUVATE DEHYDROGENASE COMPLEX and the KETOGLUTARATE DEHYDROGENASE COMPLEX.'),
  'D013836': NtO(MH='Thiamin Pyrophosphokinase', MS='An enzyme that catalyzes the formation of thiamine pyrophosphate from ATP and thiamine. EC 2.7.6.2.'),
  'D013837': NtO(MH='Thiamin-Triphosphatase', MS='An enzyme present in nerve tissue. It catalyzes reversibly the formation of thiamine diphosphate and orthophosphate from thiamine triphosphate. EC 3.6.1.28.'),
  'D013838': NtO(MH='Thiamine Triphosphate', MS="3-((4-Amino-2-methyl-5-pyrimidinyl)methyl)-4-methyl-5-(4,6,8,8-tetrahydroxy-3,5,7-trioxa-4,6,8-triphosphaoct-1-yl)thiazolium hydroxide, inner salt, P,P',P''-trioxide. The triphosphate ester of thiamine. In Leigh's disease, this compound is present in decreased amounts in the brain due to a metabolic block in its formation."),
  'D013839': NtO(MH='Thiamphenicol', MS='A methylsulfonyl analog of CHLORAMPHENICOL. It is an antibiotic and immunosuppressive agent.'),
  'D013840': NtO(MH='Thiamylal', MS='A barbiturate that is administered intravenously for the production of complete anesthesia of short duration, for the induction of general anesthesia, or for inducing a hypnotic state. (From Martindale, The Extra Pharmacopoeia, 30th ed, p919)'),
  'D013841': NtO(MH='Thiazepines', MS=None),
  'D013843': NtO(MH='Thiazines', MS=None),
  'D013844': NtO(MH='Thiazoles', MS='Heterocyclic compounds where the ring system is composed of three CARBON atoms, a SULFUR and NITROGEN atoms.'),
  'D013845': NtO(MH='Thienamycins', MS='Beta-lactam antibiotics that differ from PENICILLINS in having the thiazolidine sulfur atom replaced by carbon, the sulfur then becoming the first atom in the side chain. They are unstable chemically, but have a very broad antibacterial spectrum. Thienamycin and its more stable derivatives are proposed for use in combinations with enzyme inhibitors.'),
  'D013846': NtO(MH='Thiepins', MS=None),
  'D013847': NtO(MH='Thiethylperazine', MS='A dopamine antagonist that is particularly useful in treating the nausea and vomiting associated with anesthesia, mildly emetic cancer chemotherapy agents, radiation therapy, and toxins. This piperazine phenothiazine does not prevent vertigo or motion sickness. (From AMA Drug Evaluations Annual, 1994, p457)'),
  'D013848': NtO(MH='Thigh', MS='The superior part of the lower extremity between the HIP and the KNEE.'),
  'D013849': NtO(MH='Thimerosal', MS='An ethylmercury-sulfidobenzoate that has been used as a preservative in VACCINES; ANTIVENINS; and OINTMENTS. It was formerly used as a topical antiseptic. It degrades to ethylmercury and thiosalicylate.'),
  'D013850': NtO(MH='Thinking', MS='Mental activity that is not predominantly perceptual by which one apprehends some aspect of an object or situation based on past learning and experience.'),
  'D013851': NtO(MH='Thinness', MS='A state of insufficient flesh on the body usually defined as having a body weight less than skeletal and physical standards. Depending on age, sex, and genetic background, a BODY MASS INDEX of less than 18.5 is considered as underweight.'),
  'D013852': NtO(MH='Thiotepa', MS='A very toxic alkylating antineoplastic agent also used as an insect sterilant. It causes skin, gastrointestinal, CNS, and bone marrow damage. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), thiotepa may reasonably be anticipated to be a carcinogen (Merck Index, 11th ed).'),
  'D013853': NtO(MH='Thioacetamide', MS='A crystalline compound used as a laboratory reagent in place of HYDROGEN SULFIDE. It is a potent hepatocarcinogen.'),
  'D013854': NtO(MH='Thioamides', MS='Organic compounds containing the radical -CSNH2.'),
  'D013855': NtO(MH='Thiobacillus', MS='A genus of gram-negative, rod-shaped bacteria that derives energy from the oxidation of one or more reduced sulfur compounds. Many former species have been reclassified to other classes of PROTEOBACTERIA.'),
  'D013856': NtO(MH='Acidithiobacillus thiooxidans', MS='A strictly autotrophic species of bacteria that oxidizes sulfur and thiosulfate to sulfuric acid. It was formerly called Thiobacillus thiooxidans.'),
  'D013858': NtO(MH='Thiobarbiturates', MS='Compounds in which one or more of the ketone groups on the pyrimidine ring of barbituric acid are replaced by thione groups.'),
  'D013859': NtO(MH='Thiocarbamates', MS='Carbamates in which the -CO- group has been replaced by a -CS- group.'),
  'D013860': NtO(MH='Thiocholine', MS='A mercaptocholine used as a reagent for the determination of CHOLINESTERASES. It also serves as a highly selective nerve stain.'),
  'D013861': NtO(MH='Thiocyanates', MS='Organic derivatives of thiocyanic acid which contain the general formula R-SCN.'),
  'D013862': NtO(MH='Thiogalactosides', MS='Galactosides in which the oxygen atom linking the sugar and aglycone is replaced by a sulfur atom.'),
  'D013863': NtO(MH='Thioglucosides', MS=None),
  'D013864': NtO(MH='Thioglycolates', MS='Organic esters of thioglycolic acid (HS-CH2COOH).'),
  'D013865': NtO(MH='Thioglycosides', MS=None),
  'D013866': NtO(MH='Thioguanine', MS='An antineoplastic compound which also has antimetabolite action. The drug is used in the therapy of acute leukemia.'),
  'D013867': NtO(MH='Thiohydantoins', MS=None),
  'D013868': NtO(MH='Thioinosine', MS='Sulfhydryl analog of INOSINE that inhibits nucleoside transport across erythrocyte plasma membranes, and has immunosuppressive properties. It has been used similarly to MERCAPTOPURINE in the treatment of leukemia. (From Martindale, The Extra Pharmacopoeia, 30th ed, p503)'),
  'D013869': NtO(MH='Thiolester Hydrolases', MS=None),
  'D013870': NtO(MH='Thiomalates', MS='Derivatives of thiomalic acid (the structural formula (COOH)CHSH(COOH)), including its salts and esters.'),
  'D013871': NtO(MH='Thiones', MS='Sulfur-containing compounds also known as thioketones of general formula R2C=S.'),
  'D013872': NtO(MH='Thionucleosides', MS='Nucleosides in which the base moiety is substituted with one or more sulfur atoms.'),
  'D013873': NtO(MH='Thionucleotides', MS='Nucleotides in which the base moiety is substituted with one or more sulfur atoms.'),
  'D013874': NtO(MH='Thiopental', MS='A barbiturate that is administered intravenously for the induction of general anesthesia or for the production of complete anesthesia of short duration.'),
  'D013875': NtO(MH='Thiophanate', MS='Nematocide used in livestock; also has fungicidal properties.'),
  'D013876': NtO(MH='Thiophenes', MS=None),
  'D013879': NtO(MH='Thioredoxins', MS='Hydrogen-donating proteins that participates in a variety of biochemical reactions including ribonucleotide reduction and reduction of PEROXIREDOXINS. Thioredoxin is oxidized from a dithiol to a disulfide when acting as a reducing cofactor. The disulfide form is then reduced by NADPH in a reaction catalyzed by THIOREDOXIN REDUCTASE.'),
  'D013880': NtO(MH='Thioredoxin-Disulfide Reductase', MS='A FLAVOPROTEIN enzyme that catalyzes the oxidation of THIOREDOXINS to thioredoxin disulfide in the presence of NADP+. It was formerly listed as EC 1.6.4.5'),
  'D013881': NtO(MH='Thioridazine', MS='A phenothiazine antipsychotic used in the management of PHYCOSES, including SCHIZOPHRENIA.'),
  'D013882': NtO(MH='Thiosemicarbazones', MS=None),
  'D013883': NtO(MH='Thiostrepton', MS='One of the CYCLIC PEPTIDES from Streptomyces that is active against gram-positive bacteria. In veterinary medicine, it has been used in mastitis caused by gram-negative organisms and in dermatologic disorders.'),
  'D013884': NtO(MH='Thiosulfate Sulfurtransferase', MS='An enzyme that catalyzes the transfer of the planetary sulfur atom of thiosulfate ion to cyanide ion to form thiocyanate ion. EC 2.8.1.1.'),
  'D013885': NtO(MH='Thiosulfates', MS='Inorganic salts of thiosulfuric acid possessing the general formula R2S2O3.'),
  'D013886': NtO(MH='Thiosulfonic Acids', MS='Inorganic or organic oxy acids of sulfur which contain the general formula RS2O2H.'),
  'D013888': NtO(MH='Thiothixene', MS='A thioxanthine used as an antipsychotic agent. Its effects are similar to the phenothiazine antipsychotics.'),
  'D013889': NtO(MH='Thiouracil', MS='Occurs in seeds of Brassica and Crucifera species. Thiouracil has been used as antithyroid, coronary vasodilator, and in congestive heart failure although its use has been largely supplanted by other drugs. It is known to cause blood dyscrasias and suspected of terato- and carcinogenesis.'),
  'D013890': NtO(MH='Thiourea', MS='A photographic fixative used also in the manufacture of resins. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), this substance may reasonably be anticipated to be a carcinogen (Merck Index, 9th ed). Many of its derivatives are ANTITHYROID AGENTS and/or FREE RADICAL SCAVENGERS.'),
  'D013891': NtO(MH='Thiouridine', MS='A photoactivable URIDINE analog that is used as an affinity label.'),
  'D013892': NtO(MH='Thioxanthenes', MS='Compounds with three aromatic rings in linear arrangement with a SULFUR in the center ring.'),
  'D013893': NtO(MH='Thiram', MS='A dithiocarbamate chemical, used commercially in the rubber processing industry and as a fungicide. In vivo studies indicate that it inactivates the enzyme GLUTATHIONE REDUCTASE. It has mutagenic activity and may induce chromosomal aberrations.'),
  'D013894': NtO(MH='Thirst', MS='A drive stemming from a physiological need for WATER.'),
  'D013895': NtO(MH='Thoracic Arteries', MS='Arteries originating from the subclavian or axillary arteries and distributing to the anterior thoracic wall, mediastinal structures, diaphragm, pectoral muscles, mammary gland and the axillary aspect of the chest wall.'),
  'D013896': NtO(MH='Thoracic Diseases', MS='Disorders affecting the organs of the thorax.'),
  'D013897': NtO(MH='Thoracic Duct', MS='The largest lymphatic vessel that passes through the chest and drains into the SUBCLAVIAN VEIN.'),
  'D013898': NtO(MH='Thoracic Injuries', MS='General or unspecified injuries to the chest area.'),
  'D013899': NtO(MH='Thoracic Neoplasms', MS='New abnormal growth of tissue in the THORAX.'),
  'D013900': NtO(MH='Thoracic Nerves', MS='The twelve spinal nerves on each side of the thorax. They include eleven INTERCOSTAL NERVES and one subcostal nerve. Both sensory and motor, they supply the muscles and skin of the thoracic and abdominal walls.'),
  'D013901': NtO(MH='Thoracic Outlet Syndrome', MS='A neurovascular syndrome associated with compression of the BRACHIAL PLEXUS; SUBCLAVIAN ARTERY; and SUBCLAVIAN VEIN at the superior thoracic outlet. This may result from a variety of anomalies such as a CERVICAL RIB, anomalous fascial bands, and abnormalities of the origin or insertion of the anterior or medial scalene muscles. Clinical features may include pain in the shoulder and neck region which radiates into the arm, PARESIS or PARALYSIS of brachial plexus innervated muscles, PARESTHESIA, loss of sensation, reduction of arterial pulses in the affected extremity, ISCHEMIA, and EDEMA. (Adams et al., Principles of Neurology, 6th ed, pp214-5).'),
  'D013902': NtO(MH='Radiography, Thoracic', MS='X-ray visualization of the chest and organs of the thoracic cavity. It is not restricted to visualization of the lungs.'),
  'D013903': NtO(MH='Thoracic Surgery', MS='A surgical specialty concerned with diagnosis and treatment of disorders of the heart, lungs, and esophagus. Two major types of thoracic surgery are classified as pulmonary and cardiovascular.'),
  'D013904': NtO(MH='Thoracic Vertebrae', MS='A group of twelve VERTEBRAE connected to the ribs that support the upper trunk region.'),
  'D013905': NtO(MH='Thoracoplasty', MS='Surgical removal of ribs, allowing the chest wall to move inward and collapse a diseased lung. (Dorland, 28th ed)'),
  'D013906': NtO(MH='Thoracoscopy', MS='Endoscopic examination, therapy or surgery of the pleural cavity.'),
  'D013907': NtO(MH='Thoracostomy', MS='Surgical procedure involving the creation of an opening (stoma) into the chest cavity for drainage; used in the treatment of PLEURAL EFFUSION; PNEUMOTHORAX; HEMOTHORAX; and EMPYEMA.'),
  'D013908': NtO(MH='Thoracotomy', MS='Surgical incision into the chest wall.'),
  'D013909': NtO(MH='Thorax', MS='The upper part of the trunk between the NECK and the ABDOMEN. It contains the chief organs of the circulatory and respiratory systems. (From Stedman, 25th ed)'),
  'D013910': NtO(MH='Thorium', MS='A radioactive element of the actinide series of metals. It has an atomic symbol Th, atomic number 90, and atomic weight 232.04. It is used as fuel in nuclear reactors to produce fissionable uranium isotopes. Because of its radioopacity, various thorium compounds are used to facilitate visualization in roentgenography.'),
  'D013911': NtO(MH='Thorium Dioxide', MS='Thorium oxide (ThO2). A radiographic contrast agent that was used in the early 1930s through about 1954. High rates of mortality have been linked to its use and it has been shown to cause liver cancer.'),
  'D013912': NtO(MH='Threonine', MS='An essential amino acid occurring naturally in the L-form, which is the active form. It is found in eggs, milk, gelatin, and other proteins.'),
  'D013913': NtO(MH='Threonine Dehydratase', MS='A pyridoxal-phosphate protein that catalyzes the deamination of THREONINE to 2-ketobutyrate and AMMONIA. The role of this enzyme can be biosynthetic or biodegradative. In the former role it supplies 2-ketobutyrate required for ISOLEUCINE biosynthesis, while in the latter it is only involved in the breakdown of threonine to supply energy. This enzyme was formerly listed as EC 4.2.1.16.'),
  'D013914': NtO(MH='Threonine-tRNA Ligase', MS='An enzyme that activates threonine with its specific transfer RNA. EC 6.1.1.3.'),
  'D013915': NtO(MH='Thrombasthenia', MS='A congenital bleeding disorder with prolonged bleeding time, absence of aggregation of platelets in response to most agents, especially ADP, and impaired or absent clot retraction. Platelet membranes are deficient in or have a defect in the glycoprotein IIb-IIIa complex (PLATELET GLYCOPROTEIN GPIIB-IIIA COMPLEX).'),
  'D013916': NtO(MH='Thrombelastography', MS='Use of a thrombelastograph, which provides a continuous graphic record of the physical shape of a clot during fibrin formation and subsequent lysis.'),
  'D013917': NtO(MH='Thrombin', MS='An enzyme formed from PROTHROMBIN that converts FIBRINOGEN to FIBRIN.'),
  'D013918': NtO(MH='Thrombin Time', MS='Clotting time of PLASMA mixed with a THROMBIN solution. It is a measure of the conversion of FIBRINOGEN to FIBRIN, which is prolonged by AFIBRINOGENEMIA, abnormal fibrinogen, or the presence of inhibitory substances, e.g., fibrin-fibrinogen degradation products, or HEPARIN. BATROXOBIN, a thrombin-like enzyme unaffected by the presence of heparin, may be used in place of thrombin.'),
  'D013919': NtO(MH='Thromboangiitis Obliterans', MS='A non-atherosclerotic, inflammatory thrombotic disease that commonly involves small and medium-sized arteries or veins in the extremities. It is characterized by occlusive THROMBOSIS and FIBROSIS in the vascular wall leading to digital and limb ISCHEMIA and ulcerations. Thromboangiitis obliterans is highly associated with tobacco smoking.'),
  'D013920': NtO(MH='Thrombocythemia, Essential', MS='A clinical syndrome characterized by repeated spontaneous hemorrhages and a remarkable increase in the number of circulating platelets.'),
  'D013921': NtO(MH='Thrombocytopenia', MS='A subnormal level of BLOOD PLATELETS.'),
  'D013922': NtO(MH='Thrombocytosis', MS='Increased numbers of platelets in the peripheral blood. (Dorland, 27th ed)'),
  'D013923': NtO(MH='Thromboembolism', MS='Obstruction of a blood vessel (embolism) by a blood clot (THROMBUS) in the blood stream.'),
  'D013924': NtO(MH='Thrombophlebitis', MS='Inflammation of a vein associated with a blood clot (THROMBUS).'),
  'D013925': NtO(MH='Thromboplastin', MS='Constituent composed of protein and phospholipid that is widely distributed in many tissues. It serves as a cofactor with factor VIIa to activate factor X in the extrinsic pathway of blood coagulation.'),
  'D013926': NtO(MH='Thrombopoietin', MS='A humoral factor that stimulates the production of thrombocytes (BLOOD PLATELETS). Thrombopoietin stimulates the proliferation of bone marrow MEGAKARYOCYTES and their release of blood platelets. The process is called THROMBOPOIESIS.'),
  'D013927': NtO(MH='Thrombosis', MS='Formation and development of a thrombus or blood clot in the blood vessel.'),
  'D013928': NtO(MH='Thromboxane A2', MS='An unstable intermediate between the prostaglandin endoperoxides and thromboxane B2. The compound has a bicyclic oxaneoxetane structure. It is a potent inducer of platelet aggregation and causes vasoconstriction. It is the principal component of rabbit aorta contracting substance (RCS).'),
  'D013929': NtO(MH='Thromboxane B2', MS='A stable, physiologically active compound formed in vivo from the prostaglandin endoperoxides. It is important in the platelet-release reaction (release of ADP and serotonin).'),
  'D013930': NtO(MH='Thromboxane-A Synthase', MS='An enzyme found predominantly in platelet microsomes. It catalyzes the conversion of PGG(2) and PGH(2) (prostaglandin endoperoxides) to thromboxane A2. EC 5.3.99.5.'),
  'D013931': NtO(MH='Thromboxanes', MS='Physiologically active compounds found in many organs of the body. They are formed in vivo from the prostaglandin endoperoxides and cause platelet aggregation, contraction of arteries, and other biological effects. Thromboxanes are important mediators of the actions of polyunsaturated fatty acids transformed by cyclooxygenase.'),
  'D013932': NtO(MH='Thulium', MS='An element of the rare earth family of metals. It has the atomic symbol Tm, atomic number 69, and atomic weight 168.93.'),
  'D013933': NtO(MH='Thumb', MS='The first digit on the radial side of the hand which in humans lies opposite the other four.'),
  'D013934': NtO(MH='Thymectomy', MS='Surgical removal of the thymus gland. (Dorland, 28th ed)'),
  'D013935': NtO(MH='Thymic Factor, Circulating', MS='A thymus-dependent nonapeptide found in normal blood. Stimulates the formation of E rosettes and is believed to be involved in T-cell differentiation.'),
  'D013936': NtO(MH='Thymidine', MS='A nucleoside in which THYMINE is linked to DEOXYRIBOSE.'),
  'D013937': NtO(MH='Thymidine Kinase', MS="An enzyme that catalyzes the conversion of ATP and thymidine to ADP and thymidine 5'-phosphate. Deoxyuridine can also act as an acceptor and dGTP as a donor. (From Enzyme Nomenclature, 1992) EC 2.7.1.21."),
  'D013938': NtO(MH='Thymidine Monophosphate', MS='5-Thymidylic acid. A thymine nucleotide containing one phosphate group esterified to the deoxyribose moiety.'),
  'D013939': NtO(MH='Thymidine Phosphorylase', MS='An enzyme that catalyzes the transfer of 2-deoxy-D-ribose from THYMIDINE to orthophosphate, thereby liberating thymidine.'),
  'D013940': NtO(MH='Thymidylate Synthase', MS='An enzyme of the transferase class that catalyzes the reaction 5,10-methylenetetrahydrofolate and dUMP to dihydrofolate and dTMP in the synthesis of thymidine triphosphate. (From Dorland, 27th ed) EC 2.1.1.45.'),
  'D013941': NtO(MH='Thymine', MS='One of four constituent bases of DNA.'),
  'D013942': NtO(MH='Thymine Nucleotides', MS='Phosphate esters of THYMIDINE in N-glycosidic linkage with ribose or deoxyribose, as occurs in nucleic acids. (From Dorland, 28th ed, p1154)'),
  'D013943': NtO(MH='Thymol', MS='A phenol obtained from thyme oil or other volatile oils used as a stabilizer in pharmaceutical preparations, and as an antiseptic (antibacterial or antifungal) agent.'),
  'D013944': NtO(MH='Thymolphthalein', MS='Used as a pH indicator and as a reagent for blood after decolorizing the alkaline solution by boiling with zinc dust.'),
  'D013945': NtO(MH='Thymoma', MS="A neoplasm originating from thymic tissue, usually benign, and frequently encapsulated. Although it is occasionally invasive, metastases are extremely rare. It consists of any type of thymic epithelial cell as well as lymphocytes that are usually abundant. Malignant lymphomas that involve the thymus, e.g., lymphosarcoma, Hodgkin's disease (previously termed granulomatous thymoma), should not be regarded as thymoma. (From Stedman, 25th ed)"),
  'D013946': NtO(MH='Thymopoietins', MS='Two closely related polypeptides (molecular weight 7,000) isolated from the thymus gland. These hormones induce the differentiation of prothymocytes to thymocytes within the thymus. They also cause a delayed impairment of neuromuscular transmission in vivo and are therefore believed to be the agent responsible for myasthenia gravis.'),
  'D013947': NtO(MH='Thymosin', MS='Thymosin. A family of heat-stable, polypeptide hormones secreted by the thymus gland. Their biological activities include lymphocytopoiesis, restoration of immunological competence and enhancement of expression of T-cell characteristics and function. They have therapeutic potential in patients having primary or secondary immunodeficiency diseases, cancer or diseases related to aging.'),
  'D013948': NtO(MH='Moxisylyte', MS="An alpha-adrenergic blocking agent that is used in Raynaud's disease. It is also used locally in the eye to reverse the mydriasis caused by phenylephrine and other sympathomimetic agents. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1312)"),
  'D013949': NtO(MH='Thymus Extracts', MS='Extracts of the THYMUS GLAND that contain specific factors, peptides, and proteins with specific biological activities.'),
  'D013950': NtO(MH='Thymus Gland', MS='A single, unpaired primary lymphoid organ situated in the MEDIASTINUM, extending superiorly into the neck to the lower edge of the THYROID GLAND and inferiorly to the fourth costal cartilage. It is necessary for normal development of immunologic function early in life. By puberty, it begins to involute and much of the tissue is replaced by fat.'),
  'D013951': NtO(MH='Thymus Hormones', MS='Humoral factors secreted by the thymus gland. They participate in the development of the lymphoid system and the maturation of the cellular immune response.'),
  'D013952': NtO(MH='Thymus Hyperplasia', MS="Enlargement of the thymus. A condition described in the late 1940's and 1950's as pathological thymic hypertrophy was status thymolymphaticus and was treated with radiotherapy. Unnecessary removal of the thymus was also practiced. It later became apparent that the thymus undergoes normal physiological hypertrophy, reaching a maximum at puberty and involuting thereafter. The concept of status thymolymphaticus has been abandoned. Thymus hyperplasia is present in two thirds of all patients with myasthenia gravis. (From Segen, Dictionary of Modern Medicine, 1992; Cecil Textbook of Medicine, 19th ed, p1486)"),
  'D013953': NtO(MH='Thymus Neoplasms', MS='Tumors or cancer of the THYMUS GLAND.'),
  'D013954': NtO(MH='Thyroglobulin', MS=None),
  'D013955': NtO(MH='Thyroglossal Cyst', MS='A cyst in the neck caused by persistence of portions of, or by lack of closure of, the primitive thyroglossal duct. (Dorland, 27th ed)'),
  'D013956': NtO(MH='Antithyroid Agents', MS='Agents that are used to treat hyperthyroidism by reducing the excessive production of thyroid hormones.'),
  'D013957': NtO(MH='Thyroid Cartilage', MS="The largest cartilage of the larynx consisting of two laminae fusing anteriorly at an acute angle in the midline of the neck. The point of fusion forms a subcutaneous projection known as the Adam's apple."),
  'D013958': NtO(MH='Thyroid Crisis', MS='A dangerous life-threatening hypermetabolic condition characterized by high FEVER and dysfunction of the cardiovascular, the nervous, and the gastrointestinal systems.'),
  'D013959': NtO(MH='Thyroid Diseases', MS='Pathological processes involving the THYROID GLAND.'),
  'D013960': NtO(MH='Thyroid Function Tests', MS='Blood tests used to evaluate the functioning of the thyroid gland.'),
  'D013961': NtO(MH='Thyroid Gland', MS='A highly vascularized endocrine gland consisting of two lobes joined by a thin band of tissue with one lobe on each side of the TRACHEA. It secretes THYROID HORMONES from the follicular cells and CALCITONIN from the parafollicular cells thereby regulating METABOLISM and CALCIUM level in blood, respectively.'),
  'D013962': NtO(MH='Thyroid (USP)', MS='A dehydrated extract of thyroid glands from domesticated animals. After the removal of fat and connective tissue, the extract is dried or lyophilized to yield a yellowish to buff-colored amorphous powder containing 0.17-0.23% of iodine.'),
  'D013963': NtO(MH='Thyroid Hormones', MS='Natural hormones secreted by the THYROID GLAND, such as THYROXINE, and their synthetic analogs.'),
  'D013964': NtO(MH='Thyroid Neoplasms', MS='Tumors or cancer of the THYROID GLAND.'),
  'D013965': NtO(MH='Thyroidectomy', MS='Surgical removal of the thyroid gland. (Dorland, 28th ed)'),
  'D013966': NtO(MH='Thyroiditis', MS="Inflammatory diseases of the THYROID GLAND. Thyroiditis can be classified into acute (THYROIDITIS, SUPPURATIVE), subacute (granulomatous and lymphocytic), chronic fibrous (Riedel's), chronic lymphocytic (HASHIMOTO DISEASE), transient (POSTPARTUM THYROIDITIS), and other AUTOIMMUNE THYROIDITIS subtypes."),
  'D013967': NtO(MH='Thyroiditis, Autoimmune', MS='Inflammatory disease of the THYROID GLAND due to autoimmune responses leading to lymphocytic infiltration of the gland. It is characterized by the presence of circulating thyroid antigen-specific T-CELLS and thyroid AUTOANTIBODIES. The clinical signs can range from HYPOTHYROIDISM to THYROTOXICOSIS depending on the type of autoimmune thyroiditis.'),
  'D013968': NtO(MH='Thyroiditis, Subacute', MS='Spontaneously remitting inflammatory condition of the THYROID GLAND, characterized by FEVER; MUSCLE WEAKNESS; SORE THROAT; severe thyroid PAIN; and an enlarged damaged gland containing GIANT CELLS. The disease frequently follows a viral infection.'),
  'D013969': NtO(MH='Thyroiditis, Suppurative', MS='Acute inflammatory disease of the THYROID GLAND due to infections by BACTERIA; FUNGI; or other microorganisms. Symptoms include tender swelling, FEVER, and often with LEUKOCYTOSIS.'),
  'D013970': NtO(MH='Thyronines', MS='A group of metabolites derived from THYROXINE and TRIIODOTHYRONINE via the peripheral enzymatic removal of iodines from the thyroxine nucleus. Thyronine is the thyroxine nucleus devoid of its four iodine atoms.'),
  'D013971': NtO(MH='Thyrotoxicosis', MS='A hypermetabolic syndrome caused by excess THYROID HORMONES which may come from endogenous or exogenous sources. The endogenous source of hormone may be thyroid HYPERPLASIA; THYROID NEOPLASMS; or hormone-producing extrathyroidal tissue. Thyrotoxicosis is characterized by NERVOUSNESS; TACHYCARDIA; FATIGUE; WEIGHT LOSS; heat intolerance; and excessive SWEATING.'),
  'D013972': NtO(MH='Thyrotropin', MS='A glycoprotein hormone secreted by the adenohypophysis (PITUITARY GLAND, ANTERIOR). Thyrotropin stimulates THYROID GLAND by increasing the iodide transport, synthesis and release of thyroid hormones (THYROXINE and TRIIODOTHYRONINE). Thyrotropin consists of two noncovalently linked subunits, alpha and beta. Within a species, the alpha subunit is common in the pituitary glycoprotein hormones (TSH; LUTEINIZING HORMONE and FSH), but the beta subunit is unique and confers its biological specificity.'),
  'D013973': NtO(MH='Thyrotropin-Releasing Hormone', MS='A tripeptide that stimulates the release of THYROTROPIN and PROLACTIN. It is synthesized by the neurons in the PARAVENTRICULAR NUCLEUS of the HYPOTHALAMUS. After being released into the pituitary portal circulation, TRH (was called TRF) stimulates the release of TSH and PRL from the ANTERIOR PITUITARY GLAND.'),
  'D013974': NtO(MH='Thyroxine', MS='The major hormone derived from the thyroid gland. Thyroxine is synthesized via the iodination of tyrosines (MONOIODOTYROSINE) and the coupling of iodotyrosines (DIIODOTYROSINE) in the THYROGLOBULIN. Thyroxine is released from thyroglobulin by proteolysis and secreted into the blood. Thyroxine is peripherally deiodinated to form TRIIODOTHYRONINE which exerts a broad spectrum of stimulatory effects on cell metabolism.'),
  'D013975': NtO(MH='Thyroxine-Binding Proteins', MS='Blood proteins that bind to THYROID HORMONES such as THYROXINE and transport them throughout the circulatory system.'),
  'D013976': NtO(MH='Tiapamil Hydrochloride', MS='A phenylethylamine derivative that acts as a calcium antagonist showing hemodynamic effects in patients with acute myocardial infarction.'),
  'D013977': NtO(MH='Tibia', MS='The second longest bone of the skeleton. It is located on the medial side of the lower leg, articulating with the FIBULA laterally, the TALUS distally, and the FEMUR proximally.'),
  'D013978': NtO(MH='Tibial Fractures', MS='Fractures of the TIBIA.'),
  'D013979': NtO(MH='Tibial Nerve', MS='The medial terminal branch of the sciatic nerve. The tibial nerve fibers originate in lumbar and sacral spinal segments (L4 to S2). They supply motor and sensory innervation to parts of the calf and foot.'),
  'D013981': NtO(MH='Tic Disorders', MS='Disorders characterized by recurrent TICS that may interfere with speech and other activities. Tics are sudden, rapid, nonrhythmic, stereotyped motor movements or vocalizations which may be exacerbated by stress and are generally attenuated during absorbing activities. Tic disorders are distinguished from conditions which feature other types of abnormal movements that may accompany another another condition. (From DSM-IV, 1994)'),
  'D013982': NtO(MH='Ticarcillin', MS='An antibiotic derived from penicillin similar to CARBENICILLIN in action.'),
  'D013983': NtO(MH='Tick Control', MS='Chemical, biological, or medical measures designed to prevent the spread of ticks or the concomitant infestations which result in tick-borne diseases. It includes the veterinary as well as the public health aspects of tick and mite control.'),
  'D013984': NtO(MH='Tick Infestations', MS='Infestations with soft-bodied (Argasidae) or hard-bodied (Ixodidae) ticks.'),
  'D013985': NtO(MH='Tick Paralysis', MS='Paralysis caused by a neurotropic toxin secreted by the salivary glands of ticks.'),
  'D013986': NtO(MH='Tick Toxicoses', MS='Toxicoses caused by toxic substances secreted by the salivary glands of ticks; include tick paralysis (neurotropic toxin), sweating sickness (dermotropic toxin), and Rhipicephalus appendiculatus toxicosis (leukotropic toxin).'),
  'D013987': NtO(MH='Ticks', MS='Blood-sucking acarid parasites of the order Ixodida comprising two families: the softbacked ticks (ARGASIDAE) and hardbacked ticks (IXODIDAE). Ticks are larger than their relatives, the MITES. They penetrate the skin of their host by means of highly specialized, hooked mouth parts and feed on its blood. Ticks attack all groups of terrestrial vertebrates. In humans they are responsible for many TICK-BORNE DISEASES, including the transmission of ROCKY MOUNTAIN SPOTTED FEVER; TULAREMIA; BABESIOSIS; AFRICAN SWINE FEVER; and RELAPSING FEVER. (From Barnes, Invertebrate Zoology, 5th ed, pp543-44)'),
  'D013988': NtO(MH='Ticlopidine', MS='An effective inhibitor of platelet aggregation commonly used in the placement of STENTS in CORONARY ARTERIES.'),
  'D013989': NtO(MH='Ticrynafen', MS='A novel diuretic with uricosuric action. It has been proposed as an antihypertensive agent.'),
  'D013990': NtO(MH='Tidal Volume', MS='The volume of air inspired or expired during each normal, quiet respiratory cycle. Common abbreviations are TV or V with subscript T.'),
  'D013991': NtO(MH="Tietze's Syndrome", MS='Idiopathic painful nonsuppurative swellings of one or more costal cartilages, especially of the second rib. The anterior chest pain may mimic that of coronary artery disease. (Dorland, 27th ed.)'),
  'D013992': NtO(MH='Tiletamine', MS='Proposed anesthetic with possible anticonvulsant and sedative properties.'),
  'D013993': NtO(MH='Tilidine', MS='An opioid analgesic used similarly to MORPHINE in the control of moderate to severe pain. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1097)'),
  'D013994': NtO(MH='Tilorone', MS='An antiviral agent used as its hydrochloride. It is the first recognized synthetic, low-molecular-weight compound that is an orally active interferon inducer, and is also reported to have antineoplastic and anti-inflammatory actions.'),
  'D013995': NtO(MH='Time', MS='The dimension of the physical universe which, at a given place, orders the sequence of events. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D013996': NtO(MH='Time and Motion Studies', MS='The observation and analysis of movements in a task with an emphasis on the amount of time required to perform the task.'),
  'D013997': NtO(MH='Time Factors', MS='Elements of limited time intervals, contributing to particular results or situations.'),
  'D013998': NtO(MH='Time Perception', MS='The ability to estimate periods of time lapsed or duration of time.'),
  'D013999': NtO(MH='Timolol', MS='A beta-adrenergic antagonist that is similar in action to PROPRANOLOL; the levo-isomer is more active. Timolol has been proposed as an anti-hypertensive, anti-arrhythmic, anti-angina, and anti-glaucoma agent. It is also used in the treatment of MIGRAINE DISORDERS and tremor.'),
  'D014001': NtO(MH='Tin', MS='A trace element that is required in bone formation. It has the atomic symbol Sn, atomic number 50, and atomic weight 118.71.'),
  'D014002': NtO(MH='Tin Fluorides', MS='Inorganic fluorides of tin. They include both stannic fluoride (tin tetrafluoride) and stannous fluoride (tin difluoride). The latter is used in the prevention of dental caries.'),
  'D014003': NtO(MH='Tin Polyphosphates', MS='Poly or pyrophosphates of tin. In conjunction with radioactive technetium these compounds are used as bone-scanning agents and in scintigraphy to diagnose myocardial and cerebral infarction.'),
  'D014004': NtO(MH='Tin Radioisotopes', MS='Unstable isotopes of tin that decay or disintegrate emitting radiation. Sn atoms with atomic weights 108-111, 113, 120-121, 123 and 125-128 are tin radioisotopes.'),
  'D014005': NtO(MH='Tinea', MS='Fungal infection of keratinized tissues such as hair, skin and nails. The main causative fungi include MICROSPORUM; TRICHOPHYTON; and EPIDERMOPHYTON.'),
  'D014006': NtO(MH='Tinea Capitis', MS='Ringworm of the scalp and associated hair mainly caused by species of MICROSPORUM; TRICHOPHYTON; and EPIDERMOPHYTON, which may occasionally involve the eyebrows and eyelashes.'),
  'D014007': NtO(MH='Tinea Favosa', MS="A disease of the scalp that may affect the glabrous skin and the nails and is recognized by the concave sulfur-yellow crusts that form around loose, wiry hairs. Atrophy ensues, leaving a smooth, glossy, thin, paper-white patch. This type of disease is rare in the United States and more frequently seen in the Middle East, Africa, Southeastern Europe, and other countries bordering the Mediterranean Sea. (Arnold, Odom, and James, Andrew's Diseases of the Skin, 8th ed, p319)"),
  'D014008': NtO(MH='Tinea Pedis', MS='Dermatological pruritic lesion in the feet, caused by Trichophyton rubrum, T. mentagrophytes, or Epidermophyton floccosum.'),
  'D014009': NtO(MH='Onychomycosis', MS='A fungal infection of the nail, usually caused by DERMATOPHYTES; YEASTS; or nondermatophyte MOLDS.'),
  'D014010': NtO(MH='Tinea Versicolor', MS='A common chronic, noninflammatory and usually symptomless disorder, characterized by the occurrence of multiple macular patches of all sizes and shapes, and varying in pigmentation from fawn-colored to brown. It is seen most frequently in hot, humid, tropical regions, and is caused by Pityrosporon orbiculare. (Dorland, 27th ed)'),
  'D014011': NtO(MH='Tinidazole', MS='A nitroimidazole alkylating agent that is used as an antitrichomonal agent against TRICHOMONAS VAGINALIS; ENTAMOEBA HISTOLYTICA; and GIARDIA LAMBLIA infections. It also acts as an antibacterial agent for the treatment of BACTERIAL VAGINOSIS and anaerobic bacterial infections.'),
  'D014012': NtO(MH='Tinnitus', MS='A nonspecific symptom of hearing disorder characterized by the sensation of buzzing, ringing, clicking, pulsations, and other noises in the ear. Objective tinnitus refers to noises generated from within the ear or adjacent structures that can be heard by other individuals. The term subjective tinnitus is used when the sound is audible only to the affected individual. Tinnitus may occur as a manifestation of COCHLEAR DISEASES; VESTIBULOCOCHLEAR NERVE DISEASES; INTRACRANIAL HYPERTENSION; CRANIOCEREBRAL TRAUMA; and other conditions.'),
  'D014013': NtO(MH='1,2-Dihydroxybenzene-3,5-Disulfonic Acid Disodium Salt', MS='A colorimetric reagent for iron, manganese, titanium, molybdenum, and complexes of zirconium. (From Merck Index, 11th ed)'),
  'D014014': NtO(MH='Tissue Adhesives', MS='Substances used to cause adherence of tissue to tissue or tissue to non-tissue surfaces, as for prostheses.'),
  'D014015': NtO(MH='Tissue Banks', MS='Centers for acquiring, characterizing, and storing organs or tissue for future use.'),
  'D014016': NtO(MH='Tissue Conditioning, Dental', MS='The use of a treatment material (tissue conditioner) to re-establish tone and health to irritated oral soft tissue, usually applied to the edentulous alveolar ridge.'),
  'D014018': NtO(MH='Tissue Distribution', MS='Accumulation of a drug or chemical substance in various organs (including those not relevant to its pharmacologic or therapeutic action). This distribution depends on the blood flow or perfusion rate of the organ, the ability of the drug to penetrate organ membranes, tissue specificity, protein binding. The distribution is usually expressed as tissue to plasma ratios.'),
  'D014019': NtO(MH='Tissue Donors', MS='Individuals supplying living tissue, organs, cells, blood or blood components for transfer or transplantation to histocompatible recipients.'),
  'D014020': NtO(MH='Tissue Extracts', MS='Preparations made from animal tissues or organs (ANIMAL STRUCTURES). They usually contain many components, any one of which may be pharmacologically or physiologically active. Tissue extracts may contain specific, but uncharacterized factors or proteins with specific actions.'),
  'D014021': NtO(MH='Tissue Preservation', MS='The process by which a tissue or aggregate of cells is kept alive outside of the organism from which it was derived (i.e., kept from decay by means of a chemical agent, cooling, or a fluid substitute that mimics the natural state within the organism).'),
  'D014022': NtO(MH='Tissue Survival', MS='The span of viability of a tissue or an organ.'),
  'D014023': NtO(MH='Tissue Therapy, Historical', MS='Historically, tissue transplantation, especially of refrigerated tissue (after Filatov). It was theorized that nonspecific substances, capable of initiating restorative processes, formed in tissues when refrigerated. Cell therapy (after Niehans) refers to implantation of tissue by injection. Originally this involved fresh cells but later frozen or lyophilized cells.'),
  'D014024': NtO(MH='Tissues', MS='Collections of differentiated CELLS, such as EPITHELIUM; CONNECTIVE TISSUE; MUSCLES; and NERVE TISSUE. Tissues are cooperatively arranged to form organs with specialized functions such as RESPIRATION; DIGESTION; REPRODUCTION; MOVEMENT; and others.'),
  'D014025': NtO(MH='Titanium', MS='A dark-gray, metallic element of widespread distribution but occurring in small amounts with atomic number, 22, atomic weight, 47.867 and symbol, Ti; specific gravity, 4.5; used for fixation of fractures.'),
  'D014026': NtO(MH='Tobacco', MS='A plant genus of the family SOLANACEAE. Members contain NICOTINE and other biologically active chemicals; its dried leaves are used for SMOKING.'),
  'D014027': NtO(MH='Tobacco Mosaic Virus', MS='The type species of TOBAMOVIRUS which causes mosaic disease of tobacco. Transmission occurs by mechanical inoculation.'),
  'D014028': NtO(MH='Tobacco Smoke Pollution', MS='Contamination of the air by tobacco smoke.'),
  'D014029': NtO(MH='Tobacco Use Disorder', MS="Tobacco used to the detriment of a person's health or social functioning. Tobacco dependence is included."),
  'D014030': NtO(MH='Tobacco, Smokeless', MS='Any product of NICOTIANA TABACUM that is not smoked but is either inhaled through the nose, chewed, or stored in cheek pouches.'),
  'D014031': NtO(MH='Tobramycin', MS='An aminoglycoside, broad-spectrum antibiotic produced by Streptomyces tenebrarius. It is effective against gram-negative bacteria, especially the PSEUDOMONAS species. It is a 10% component of the antibiotic complex, NEBRAMYCIN, produced by the same species.'),
  'D014032': NtO(MH='Todralazine', MS='An antihypertensive agent with both central and peripheral action; it has some central nervous system depressant effects.'),
  'D014033': NtO(MH='Toe Joint', MS='The articulation between the head of one phalanx and the base of the one distal to it, in each toe.'),
  'D014034': NtO(MH='Toes', MS='Any one of five terminal digits of the vertebrate FOOT.'),
  'D014035': NtO(MH='Togaviridae', MS='A family of RNA viruses, mainly arboviruses, consisting of two genera: ALPHAVIRUS (group A arboviruses), and RUBIVIRUS. Virions are spherical, 60-70 nm in diameter, with a lipoprotein envelope tightly applied to the icosahedral nucleocapsid.'),
  'D014036': NtO(MH='Togaviridae Infections', MS='Virus diseases caused by the TOGAVIRIDAE.'),
  'D014037': NtO(MH='Togo', MS='A republic in western Africa, lying between GHANA on its west and BENIN on its east. Its capital is Lome.'),
  'D014038': NtO(MH='Toilet Facilities', MS='Facilities containing equipment for disposal of human excrement, often with an accompanying hand-washing station.'),
  'D014039': NtO(MH='Toilet Training', MS='Conditioning to defecate and urinate in culturally acceptable places.'),
  'D014040': NtO(MH='Token Economy', MS='A practice whereby tokens representing money, toys, candy, etc., are given as secondary reinforcers contingent upon certain desired behaviors or performances.'),
  'D014041': NtO(MH='Tokyo', MS='Capital of the country of JAPAN.'),
  'D014042': NtO(MH='Tolazamide', MS='A sulphonylurea hypoglycemic agent with actions and uses similar to those of CHLORPROPAMIDE.'),
  'D014043': NtO(MH='Tolazoline', MS='A vasodilator that apparently has direct actions on blood vessels and also increases cardiac output. Tolazoline can interact to some degree with histamine, adrenergic, and cholinergic receptors, but the mechanisms of its therapeutic effects are not clear. It is used in treatment of persistent pulmonary hypertension of the newborn.'),
  'D014044': NtO(MH='Tolbutamide', MS='A sulphonylurea hypoglycemic agent with actions and uses similar to those of CHLORPROPAMIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p290)'),
  'D014046': NtO(MH='Tolmetin', MS='A non-steroidal anti-inflammatory agent (ANTI-INFLAMMATORY AGENTS, NON-STEROIDAL) similar in mode of action to INDOMETHACIN.'),
  'D014047': NtO(MH='Tolnaftate', MS='A synthetic antifungal agent.'),
  'D014048': NtO(MH='Tolonium Chloride', MS='A phenothiazine that has been used as a hemostatic, a biological stain, and a dye for wool and silk. Tolonium chloride has also been used as a diagnostic aid for oral and gastric neoplasms and in the identification of the parathyroid gland in thyroid surgery.'),
  'D014049': NtO(MH='Tolperisone', MS='A centrally acting muscle relaxant that has been used for the symptomatic treatment of spasticity and muscle spasm. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1211)'),
  'D014050': NtO(MH='Toluene', MS='A widely used industrial solvent.'),
  'D014051': NtO(MH='Toluene 2,4-Diisocyanate', MS='Skin irritant and allergen used in the manufacture of polyurethane foams and other elastomers.'),
  'D014052': NtO(MH='Toluidines', MS='Any of three isomeric compounds (ortho-, meta-, and para-) containing a benzene ring with a methyl and amino group attached to it. They are suspected CARCINOGENS.'),
  'D014053': NtO(MH='Tomatine', MS='An alkaloid that occurs in the extract of leaves of wild tomato plants. It has been found to inhibit the growth of various fungi and bacteria. It is used as a precipitating agent for steroids. (From The Merck Index, 11th ed)'),
  'D014054': NtO(MH='Tomography', MS='Imaging methods that result in sharp images of objects located on a chosen plane and blurred images located above or below the plane.'),
  'D014055': NtO(MH='Tomography, Emission-Computed', MS='Tomography using radioactive emissions from injected RADIONUCLIDES and computer ALGORITHMS to reconstruct an image.'),
  'D014056': NtO(MH='Tomography, X-Ray', MS='Tomography using x-ray transmission.'),
  'D014057': NtO(MH='Tomography, X-Ray Computed', MS='Tomography using x-ray transmission and a computer algorithm to reconstruct the image.'),
  'D014058': NtO(MH='Tonga', MS="An archipelago in Polynesia in the southwest Pacific Ocean, comprising about 150 islands. It is a kingdom whose capital is Nukualofa. It was discovered by the Dutch in 1616, visited by Tasman in 1643, and by Captain Cook in 1773 and 1777. The modern kingdom was established during the reign of King George Tupou I, 1845-93. It became a British protectorate in 1900 and gained independence in 1970. The name Tonga may be of local origin, meaning either island or holy. Its other name, Friendly Islands, was given by Captain Cook from the welcome given him by the natives. (From Webster's New Geographical Dictionary, 1988, p1219 & Room, Brewer's Dictionary of Names, 1992, p549)"),
  'D014059': NtO(MH='Tongue', MS='A muscular organ in the mouth that is covered with pink tissue called mucosa, tiny bumps called papillae, and thousands of taste buds. The tongue is anchored to the mouth and is vital for chewing, swallowing, and for speech.'),
  'D014060': NtO(MH='Tongue Diseases', MS='Diseases involving the TONGUE.'),
  'D014061': NtO(MH='Tongue Habits', MS='Acquired responses regularly manifested by tongue movement or positioning.'),
  'D014062': NtO(MH='Tongue Neoplasms', MS='Tumors or cancer of the TONGUE.'),
  'D014063': NtO(MH='Tongue, Fissured', MS='The occurrence of of breaks or slits in the tissue of the dorsal surface of the TONGUE.'),
  'D014064': NtO(MH='Tongue, Hairy', MS='A benign condition of the tongue characterized by hypertrophy of the filiform papillae that give the dorsum of the tongue a furry appearance. The color of the elongated papillae varies from yellowish white to brown or black, depending upon staining by substances such as tobacco, food, or drugs. (Dorland, 27th ed)'),
  'D014065': NtO(MH='Tonometry, Ocular', MS='Measurement of ocular tension (INTRAOCULAR PRESSURE) with a tonometer. (Cline, et al., Dictionary of Visual Science, 4th ed)'),
  'D014066': NtO(MH='Palatine Tonsil', MS='A round-to-oval mass of lymphoid tissue embedded in the lateral wall of the PHARYNX. There is one on each side of the oropharynx in the fauces between the anterior and posterior pillars of the SOFT PALATE.'),
  'D014067': NtO(MH='Tonsillar Neoplasms', MS='Tumors or cancer of the PALATINE TONSIL.'),
  'D014068': NtO(MH='Tonsillectomy', MS='Surgical removal of a tonsil or tonsils. (Dorland, 28th ed)'),
  'D014069': NtO(MH='Tonsillitis', MS='Inflammation of the tonsils, especially the PALATINE TONSILS but the ADENOIDS (pharyngeal tonsils) and lingual tonsils may also be involved. Tonsillitis usually is caused by bacterial infection. Tonsillitis may be acute, chronic, or recurrent.'),
  'D014070': NtO(MH='Tooth', MS='One of a set of bone-like structures in the mouth used for biting and chewing.'),
  'D014071': NtO(MH='Tooth Abnormalities', MS='Congenital absence of or defects in structures of the teeth.'),
  'D014072': NtO(MH='Tooth Abrasion', MS='The pathologic wearing away of the tooth substance by brushing, bruxism, clenching, and other mechanical causes. It is differentiated from TOOTH ATTRITION in that this type of wearing away is the result of tooth-to-tooth contact, as in mastication, occurring only on the occlusal, incisal, and proximal surfaces. It differs also from TOOTH EROSION, the progressive loss of the hard substance of a tooth by chemical processes not involving bacterial action. (From Jablonski, Dictionary of Dentistry, 1992, p2)'),
  'D014073': NtO(MH='Tooth Bleaching', MS='The use of a chemical oxidizing agent to whiten TEETH. In some procedures the oxidation process is activated by the use of heat or light.'),
  'D014074': NtO(MH='Tooth Calcification', MS="The process whereby calcium salts are deposited in the dental enamel. The process is normal in the development of bones and teeth. (Boucher's Clinical Dental Terminology, 4th ed, p43)"),
  'D014075': NtO(MH='Tooth Discoloration', MS='Any change in the hue, color, or translucency of a tooth due to any cause. Restorative filling materials, drugs (both topical and systemic), pulpal necrosis, or hemorrhage may be responsible. (Jablonski, Dictionary of Dentistry, 1992, p253)'),
  'D014076': NtO(MH='Tooth Diseases', MS='Diseases involving the TEETH.'),
  'D014077': NtO(MH='Tooth Erosion', MS='Progressive loss of the hard substance of a tooth by chemical processes that do not involve bacterial action. (Jablonski, Dictionary of Dentistry, 1992, p296)'),
  'D014078': NtO(MH='Tooth Eruption', MS="The emergence of a tooth from within its follicle in the ALVEOLAR PROCESS of the MAXILLA or MANDIBLE into the ORAL CAVITY. (Boucher's Clinical Dental Terminology, 4th ed)"),
  'D014079': NtO(MH='Tooth Eruption, Ectopic', MS='An abnormality in the direction of a TOOTH ERUPTION.'),
  'D014080': NtO(MH='Tooth Exfoliation', MS="Physiologic loss of the primary dentition. (Zwemer, Boucher's Clinical Dental Terminology, 4th ed)"),
  'D014081': NtO(MH='Tooth Extraction', MS='The surgical removal of a tooth. (Dorland, 28th ed)'),
  'D014082': NtO(MH='Tooth Fractures', MS='Break or rupture of a tooth or tooth root.'),
  'D014083': NtO(MH='Tooth Germ', MS='The collective tissues from which an entire tooth is formed, including the DENTAL SAC; ENAMEL ORGAN; and DENTAL PAPILLA. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D014084': NtO(MH='Tooth Avulsion', MS="Partial or complete displacement of a tooth from its alveolar support. It is commonly the result of trauma. (From Boucher's Clinical Dental Terminology, 4th ed, p312)"),
  'D014085': NtO(MH='Tooth Migration', MS="The movement of teeth into altered positions in relationship to the basal bone of the ALVEOLAR PROCESS and to adjoining and opposing teeth as a result of loss of approximating or opposing teeth, occlusal interferences, habits, inflammatory and dystrophic disease of the attaching and supporting structures of the teeth. (From Boucher's Clinical Dental Terminology, 4th ed)"),
  'D014086': NtO(MH='Tooth Mobility', MS="Horizontal and, to a lesser degree, axial movement of a tooth in response to normal forces, as in occlusion. It refers also to the movability of a tooth resulting from loss of all or a portion of its attachment and supportive apparatus, as seen in periodontitis, occlusal trauma, and periodontosis. (From Jablonski, Dictionary of Dentistry, 1992, p507 & Boucher's Clinical Dental Terminology, 4th ed, p313)"),
  'D014087': NtO(MH='Tooth Movement Techniques', MS='Orthodontic techniques used to correct the malposition of a tooth or teeth.'),
  'D014088': NtO(MH='Tooth Permeability', MS='The property of TEETH to permit passage of light, heat, gases, liquids, metabolites, mineral ions and other substances. It does not include the penetration of the teeth by microorganisms.'),
  'D014089': NtO(MH='Tooth Remineralization', MS='Therapeutic technique for replacement of minerals in partially decalcified teeth.'),
  'D014090': NtO(MH='Tooth Replantation', MS='Reinsertion of a tooth into the alveolus from which it was removed or otherwise lost.'),
  'D014091': NtO(MH='Tooth Resorption', MS='Resorption of calcified dental tissue, involving demineralization due to reversal of the cation exchange and lacunar resorption by osteoclasts. There are two types: external (as a result of tooth pathology) and internal (apparently initiated by a peculiar inflammatory hyperplasia of the pulp). (From Jablonski, Dictionary of Dentistry, 1992, p676)'),
  'D014092': NtO(MH='Tooth Root', MS='The part of a tooth from the neck to the apex, embedded in the alveolar process and covered with cementum. A root may be single or divided into several branches, usually identified by their relative position, e.g., lingual root or buccal root. Single-rooted teeth include mandibular first and second premolars and the maxillary second premolar teeth. The maxillary first premolar has two roots in most cases. Maxillary molars have three roots. (Jablonski, Dictionary of Dentistry, 1992, p690)'),
  'D014093': NtO(MH='Tooth, Artificial', MS='A fabricated tooth substituting for a natural tooth in a prosthesis. It is usually made of porcelain or plastic.'),
  'D014094': NtO(MH='Tooth, Deciduous', MS='The teeth of the first dentition, which are shed and replaced by the permanent teeth.'),
  'D014095': NtO(MH='Tooth, Impacted', MS='A tooth that is prevented from erupting by a physical barrier, usually other teeth. Impaction may also result from orientation of the tooth in an other than vertical position in the periodontal structures.'),
  'D014096': NtO(MH='Tooth, Supernumerary', MS='An extra tooth, erupted or unerupted, resembling or unlike the other teeth in the group to which it belongs. Its presence may cause malposition of adjacent teeth or prevent their eruption.'),
  'D014097': NtO(MH='Tooth, Unerupted', MS='A normal developing tooth which has not yet perforated the oral mucosa or one that fails to erupt in the normal sequence or time interval expected for the type of tooth in a given gender, age, or population group.'),
  'D014098': NtO(MH='Toothache', MS='Pain in the adjacent areas of the teeth.'),
  'D014099': NtO(MH='Toothbrushing', MS='The act of cleaning teeth with a brush to remove plaque and prevent tooth decay. (From Webster, 3d ed)'),
  'D014100': NtO(MH='Toothpastes', MS='Dentifrices that are formulated into a paste form. They typically contain abrasives, HUMECTANTS; DETERGENTS; FLAVORING AGENTS; and CARIOSTATIC AGENTS.'),
  'D014101': NtO(MH='Torpedo', MS='A genus of the Torpedinidae family consisting of several species. Members of this family have powerful electric organs and are commonly called electric rays.'),
  'D014102': NtO(MH='Torsion Abnormality', MS='An abnormal twisting or rotation of a bodily part or member on its axis.'),
  'D014103': NtO(MH='Torticollis', MS='A symptom, not a disease, of a twisted neck. In most instances, the head is tipped toward one side and the chin rotated toward the other. The involuntary muscle contractions in the neck region of patients with torticollis can be due to congenital defects, trauma, inflammation, tumors, and neurological or other factors.'),
  'D014104': NtO(MH='Torture', MS='The intentional infliction of physical or mental suffering upon an individual or individuals, including the torture of animals.'),
  'D014105': NtO(MH='Tosyl Compounds', MS='Toluenesulfonyl compounds with general formula CH3C6H4S(O2)R\xe2\x80\x9d widely used to block amino groups in the course of syntheses of drugs and other biologically active compounds.'),
  'D014106': NtO(MH='Tosylarginine Methyl Ester', MS='Arginine derivative which is a substrate for many proteolytic enzymes. As a substrate for the esterase from the first component of complement, it inhibits the action of C(l) on C(4).'),
  'D014107': NtO(MH='Tosyllysine Chloromethyl Ketone', MS='An inhibitor of SERINE ENDOPEPTIDASES. Acts as an alkylating agent and is known to interfere with the translation process.'),
  'D014108': NtO(MH='Tosylphenylalanyl Chloromethyl Ketone', MS='An inhibitor of Serine Endopeptidases. Acts as alkylating agent and is known to interfere with the translation process.'),
  'D014109': NtO(MH='Total Lung Capacity', MS='The volume of air contained in the lungs at the end of a maximal inspiration. It is the equivalent to each of the following sums: VITAL CAPACITY plus RESIDUAL VOLUME; INSPIRATORY CAPACITY plus FUNCTIONAL RESIDUAL CAPACITY; TIDAL VOLUME plus INSPIRATORY RESERVE VOLUME plus functional residual capacity; or tidal volume plus inspiratory reserve volume plus EXPIRATORY RESERVE VOLUME plus residual volume.'),
  'D014110': NtO(MH='Touch', MS='Sensation of making physical contact with objects, animate or inanimate. Tactile stimuli are detected by MECHANORECEPTORS in the skin and mucous membranes.'),
  'D014111': NtO(MH='Tourniquets', MS='Devices for the compression of a blood vessel by application around an extremity to control the circulation and prevent the flow of blood to or from the distal area. (From Dorland, 28th ed)'),
  'D014112': NtO(MH='Toxaphene', MS='A very complex, but reproducible mixture of at least 177 C10 polychloro derivatives, having an approximate overall empirical formula of C10-H10-Cl8. It is used as an insecticide and may reasonably be anticipated to be a carcinogen: Fourth Annual Report on Carcinogens (NTP 85-002, 1985). (From Merck Index, 11th ed)'),
  'D014114': NtO(MH='Toxascaris', MS='An ascarid nematode found primarily in the small intestine of the larger Felidae as well as dogs and cats. It differs from TOXOCARA in that the larvae do not migrate through the lungs. It does occasionally produce visceral larva migrans (LARVA MIGRANS, VISCERAL) in man, although more rarely than does Toxocara.'),
  'D014115': NtO(MH='Toxemia', MS='A condition produced by the presence of toxins or other harmful substances in the BLOOD.'),
  'D014116': NtO(MH='Toxicology', MS='The science concerned with the detection, chemical composition, and biological action of toxic substances or poisons and the treatment and prevention of toxic manifestations.'),
  'D014117': NtO(MH='Toxiferine', MS='A curare alkaloid that is a very potent competitive nicotinic antagonist at the neuromuscular junction.'),
  'D014118': NtO(MH='Toxins, Biological', MS='Specific, characterizable, poisonous chemicals, often PROTEINS, with specific biological properties, including immunogenicity, produced by microbes, higher plants (PLANTS, TOXIC), or ANIMALS.'),
  'D014119': NtO(MH='Toxocara', MS='A genus of ascarid nematodes commonly parasitic in the intestines of cats and dogs.'),
  'D014120': NtO(MH='Toxocariasis', MS='Infection by round worms of the genus TOXOCARA, usually found in wild and domesticated cats and dogs and foxes, except for the larvae, which may produce visceral and ocular larva migrans in man.'),
  'D014121': NtO(MH='Toxoids', MS='Preparations of pathogenic organisms or their derivatives made nontoxic and intended for active immunologic prophylaxis. They include deactivated toxins. Anatoxin toxoids are distinct from anatoxins that are TROPANES found in CYANOBACTERIA.'),
  'D014122': NtO(MH='Toxoplasma', MS='A genus of protozoa parasitic to birds and mammals. T. gondii is one of the most common infectious pathogenic animal parasites of man.'),
  'D014123': NtO(MH='Toxoplasmosis', MS='The acquired form of infection by Toxoplasma gondii in animals and man.'),
  'D014124': NtO(MH='Toxoplasmosis, Animal', MS='Acquired infection of non-human animals by organisms of the genus TOXOPLASMA.'),
  'D014125': NtO(MH='Toxoplasmosis, Congenital', MS='Prenatal protozoal infection with TOXOPLASMA gondii which is associated with injury to the developing fetal nervous system. The severity of this condition is related to the stage of pregnancy during which the infection occurs; first trimester infections are associated with a greater degree of neurologic dysfunction. Clinical features include HYDROCEPHALUS; MICROCEPHALY; deafness; cerebral calcifications; SEIZURES; and psychomotor retardation. Signs of a systemic infection may also be present at birth, including fever, rash, and hepatosplenomegaly. (From Adams et al., Principles of Neurology, 6th ed, p735)'),
  'D014126': NtO(MH='Toxoplasmosis, Ocular', MS='Infection caused by the protozoan parasite TOXOPLASMA in which there is extensive connective tissue proliferation, the retina surrounding the lesions remains normal, and the ocular media remain clear. Chorioretinitis may be associated with all forms of toxoplasmosis, but is usually a late sequel of congenital toxoplasmosis. The severe ocular lesions in infants may lead to blindness.'),
  'D014127': NtO(MH='Toyocamycin', MS='4-Amino-5-cyano-7-(D-ribofuranosyl)-7H- pyrrolo(2,3-d)pyrimidine. Antibiotic antimetabolite isolated from Streptomyces toyocaensis cultures. It is an analog of adenosine, blocks RNA synthesis and ribosome function, and is used mainly as a tool in biochemistry.'),
  'D014128': NtO(MH='Chromomycin A3', MS='Glycosidic antibiotic from Streptomyces griseus used as a fluorescent stain of DNA and as an antineoplastic agent.'),
  'D014129': NtO(MH='Trabecular Meshwork', MS='A porelike structure surrounding the entire circumference of the anterior chamber through which aqueous humor circulates to the canal of Schlemm.'),
  'D014130': NtO(MH='Trabeculectomy', MS='Any surgical procedure for treatment of glaucoma by means of puncture or reshaping of the trabecular meshwork. It includes goniotomy, trabeculectomy, and laser perforation.'),
  'D014131': NtO(MH='Trace Elements', MS='A group of chemical elements that are needed in minute quantities for the proper growth, development, and physiology of an organism. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D014132': NtO(MH='Trachea', MS='The cartilaginous and membranous tube descending from the larynx and branching into the right and left main bronchi.'),
  'D014133': NtO(MH='Tracheal Diseases', MS='Diseases involving the TRACHEA.'),
  'D014134': NtO(MH='Tracheal Neoplasms', MS='New abnormal growth of tissue in the TRACHEA.'),
  'D014135': NtO(MH='Tracheal Stenosis', MS='A pathological narrowing of the TRACHEA.'),
  'D014136': NtO(MH='Tracheitis', MS='INFLAMMATION of the TRACHEA that is usually associated with RESPIRATORY TRACT INFECTIONS.'),
  'D014137': NtO(MH='Tracheobronchomegaly', MS='A rare and probably congenital condition characterized by great enlargement of the lumen of the trachea and the larger bronchi.'),
  'D014138': NtO(MH='Tracheoesophageal Fistula', MS='Abnormal passage between the ESOPHAGUS and the TRACHEA, acquired or congenital, often associated with ESOPHAGEAL ATRESIA.'),
  'D014139': NtO(MH='Tracheostomy', MS='Surgical formation of an opening into the trachea through the neck, or the opening so created.'),
  'D014140': NtO(MH='Tracheotomy', MS='Surgical incision of the trachea.'),
  'D014141': NtO(MH='Trachoma', MS='A chronic infection of the CONJUNCTIVA and CORNEA caused by CHLAMYDIA TRACHOMATIS.'),
  'D014142': NtO(MH='Track and Field', MS='Sports performed on a track, field, or arena and including running events and other competitions, such as the pole vault, shot put, etc.'),
  'D014143': NtO(MH='Traction', MS='The pull on a limb or a part thereof. Skin traction (indirect traction) is applied by using a bandage to pull on the skin and fascia where light traction is required. Skeletal traction (direct traction), however, uses pins or wires inserted through bone and is attached to weights, pulleys, and ropes. (From Blauvelt & Nelson, A Manual of Orthopaedic Terminology, 5th ed)'),
  'D014144': NtO(MH='Tragacanth', MS='Powdered exudate from Astragalus gummifer and related plants. It forms gelatinous mass in water. Tragacanth is used as suspending agent, excipient or emulsifier in foods, cosmetics and pharmaceuticals. It has also been used as a bulk-forming laxative.'),
  'D014145': NtO(MH='Trail Making Test', MS="The subject's ability to connect 25 numbered and lettered circles in sequence in a specific length of time. A score of 12 or below is suggestive of organic brain damage."),
  'D014146': NtO(MH='Training Support', MS='Financial support for training including both student stipends and loans and training grants to institutions.'),
  'D014147': NtO(MH='Tramadol', MS='A narcotic analgesic proposed for severe pain. It may be habituating.'),
  'D014148': NtO(MH='Tranexamic Acid', MS='Antifibrinolytic hemostatic used in severe hemorrhage.'),
  'D014149': NtO(MH='Tranquilizing Agents', MS='A traditional grouping of drugs said to have a soothing or calming effect on mood, thought, or behavior. Included here are the ANTI-ANXIETY AGENTS (minor tranquilizers), ANTIMANIC AGENTS, and the ANTIPSYCHOTIC AGENTS (major tranquilizers). These drugs act by different mechanisms and are used for different therapeutic purposes.'),
  'D014150': NtO(MH='Antipsychotic Agents', MS='Agents that control agitated psychotic behavior, alleviate acute psychotic states, reduce psychotic symptoms, and exert a quieting effect. They are used in SCHIZOPHRENIA; senile dementia; transient psychosis following surgery; or MYOCARDIAL INFARCTION; etc. These drugs are often referred to as neuroleptics alluding to the tendency to produce neurological side effects, but not all antipsychotics are likely to produce such effects. Many of these drugs may also be effective against nausea, emesis, and pruritus.'),
  'D014151': NtO(MH='Anti-Anxiety Agents', MS='Agents that alleviate ANXIETY, tension, and ANXIETY DISORDERS, promote sedation, and have a calming effect without affecting clarity of consciousness or neurologic conditions. ADRENERGIC BETA-ANTAGONISTS are commonly used in the symptomatic treatment of anxiety but are not included here.'),
  'D014152': NtO(MH='Transactional Analysis', MS='A psychoanalytic therapy wherein each social transaction is analyzed to determine the involved ego state (whether parent-like, child-like, or adult-like) as a basis for understanding behavior.'),
  'D014153': NtO(MH='Transaldolase', MS='An enzyme of the transferase class that catalyzes the reaction sedoheptulose 7-phosphate and D-glyceraldehyde 3-phosphate to yield D-erythrose 4-phosphate and D-fructose phosphate in the PENTOSE PHOSPHATE PATHWAY. (Dorland, 27th ed) EC 2.2.1.2.'),
  'D014154': NtO(MH='Transcaucasia', MS='Area of Europe that includes ARMENIA,; AZERBAIJAN; and the Republic of GEORGIA.'),
  'D014155': NtO(MH='Transcobalamins', MS='A group of carrier proteins which bind with VITAMIN B12 in the BLOOD and aid in its transport. Transcobalamin I migrates electrophoretically as a beta-globulin, while transcobalamins II and III migrate as alpha-globulins.'),
  'D014156': NtO(MH='Transcortin', MS='A serpin family member that binds to and transports GLUCOCORTICOIDS in the BLOOD.'),
  'D014157': NtO(MH='Transcription Factors', MS='Endogenous substances, usually proteins, which are effective in the initiation, stimulation, or termination of the genetic transcription process.'),
  'D014158': NtO(MH='Transcription, Genetic', MS='The biosynthesis of RNA carried out on a template of DNA. The biosynthesis of DNA from an RNA template is called REVERSE TRANSCRIPTION.'),
  'D014159': NtO(MH='Transducers', MS='Any device or element which converts an input signal into an output signal of a different form. Examples include the microphone, phonographic pickup, loudspeaker, barometer, photoelectric cell, automobile horn, doorbell, and underwater sound transducer. (McGraw Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D014160': NtO(MH='Transducers, Pressure', MS='Transducers that are activated by pressure changes, e.g., blood pressure.'),
  'D014161': NtO(MH='Transduction, Genetic', MS='The transfer of bacterial DNA by phages from an infected bacterium to another bacterium. This also refers to the transfer of genes into eukaryotic cells by viruses. This naturally occurring process is routinely employed as a GENE TRANSFER TECHNIQUE.'),
  'D014162': NtO(MH='Transfection', MS='The uptake of naked or purified DNA by CELLS, usually meaning the process as it occurs in eukaryotic cells. It is analogous to bacterial transformation (TRANSFORMATION, BACTERIAL) and both are routinely employed in GENE TRANSFER TECHNIQUES.'),
  'D014163': NtO(MH='Transfer, Psychology', MS='Change in learning in one situation due to prior learning in another situation. The transfer can be positive (with second learning improved by first) or negative (where the reverse holds).'),
  'D014164': NtO(MH='Transfer Agreement', MS='A written agreement for the transfer of patients and their medical records from one health care institution to another.'),
  'D014165': NtO(MH='Transfer Factor', MS='Factor derived from leukocyte lysates of immune donors which can transfer both local and systemic cellular immunity to nonimmune recipients.'),
  'D014166': NtO(MH='Transferases', MS='Transferases are enzymes transferring a group, for example, the methyl group or a glycosyl group, from one compound (generally regarded as donor) to another compound (generally regarded as acceptor). The classification is based on the scheme "donor:acceptor group transferase". (Enzyme Nomenclature, 1992) EC 2.'),
  'D014167': NtO(MH='Transference, Psychology', MS="The unconscious transfer to others (including psychotherapists) of feelings and attitudes which were originally associated with important figures (parents, siblings, etc.) in one's early life."),
  'D014168': NtO(MH='Transferrin', MS='An iron-binding beta1-globulin that is synthesized in the LIVER and secreted into the blood. It plays a central role in the transport of IRON throughout the circulation. A variety of transferrin isoforms exist in humans, including some that are considered markers for specific disease states.'),
  'D014169': NtO(MH='Transformation, Bacterial', MS='The heritable modification of the properties of a competent bacterium by naked DNA from another source. The uptake of naked DNA is a naturally occuring phenomenon in some bacteria. It is often used as a GENE TRANSFER TECHNIQUE.'),
  'D014170': NtO(MH='Transformation, Genetic', MS="Change brought about to an organisms genetic composition by unidirectional transfer (TRANSFECTION; TRANSDUCTION, GENETIC; CONJUGATION, GENETIC, etc.) and incorporation of foreign DNA into prokaryotic or eukaryotic cells by recombination of part or all of that DNA into the cell's genome."),
  'D014171': NtO(MH='Transients and Migrants', MS='People who frequently change their place of residence.'),
  'D014172': NtO(MH='Transillumination', MS='Passage of light through body tissues or cavities for examination of internal structures.'),
  'D014173': NtO(MH='Transistors, Electronic', MS='Electrical devices that are composed of semiconductor material, with at least three connections to an external electronic circuit. They are used to amplify electrical signals, detect signals, or as switches.'),
  'D014174': NtO(MH='Transketolase', MS='An enzyme of the transferase class that catalyzes the conversion of sedoheptulose 7-phosphate and D-glyceraldehyde 3-phosphate to D-ribose 5-phosphate and D-xylulose 5-phosphate in the PENTOSE PHOSPHATE PATHWAY. (Dorland, 27th ed) EC 2.2.1.1.'),
  'D014175': NtO(MH='Translating', MS='Conversion from one language to another language.'),
  'D014176': NtO(MH='Protein Biosynthesis', MS='The biosynthesis of PEPTIDES and PROTEINS on RIBOSOMES, directed by MESSENGER RNA, via TRANSFER RNA that is charged with standard proteinogenic AMINO ACIDS.'),
  'D014177': NtO(MH='Translations', MS='Products resulting from the conversion of one language to another.'),
  'D014178': NtO(MH='Translocation, Genetic', MS='A type of chromosome aberration characterized by CHROMOSOME BREAKAGE and transfer of the broken-off portion to another location, often to a different chromosome.'),
  'D014179': NtO(MH='Neurotransmitter Uptake Inhibitors', MS='Drugs that inhibit the transport of neurotransmitters into axon terminals or into storage vesicles within terminals. For many transmitters, uptake determines the time course of transmitter action so inhibiting uptake prolongs the activity of the transmitter. Blocking uptake may also deplete available transmitter stores. Many clinically important drugs are uptake inhibitors although the indirect reactions of the brain rather than the acute block of uptake itself is often responsible for the therapeutic effects.'),
  'D014180': NtO(MH='Transplantation', MS='Transference of a tissue or organ from either an alive or deceased donor, within an individual, between individuals of the same species, or between individuals of different species.'),
  'D014181': NtO(MH='Transplantation Immunology', MS='A general term for the complex phenomena involved in allo- and xenograft rejection by a host and graft vs host reaction. Although the reactions involved in transplantation immunology are primarily thymus-dependent phenomena of cellular immunity, humoral factors also play a part in late rejection.'),
  'D014182': NtO(MH='Transplantation, Autologous', MS="Transplantation of an individual's own tissue from one site to another site."),
  'D014183': NtO(MH='Transplantation, Heterologous', MS='Transplantation between animals of different species.'),
  'D014184': NtO(MH='Transplantation, Homologous', MS='Transplantation between individuals of the same species. Usually refers to genetically disparate individuals in contradistinction to isogeneic transplantation for genetically identical individuals.'),
  'D014185': NtO(MH='Transplantation, Isogeneic', MS='Transplantation between genetically identical individuals, i.e., members of the same species with identical histocompatibility antigens, such as monozygotic twins, members of the same inbred strain, or members of a hybrid population produced by crossing certain inbred strains.'),
  'D014186': NtO(MH='Transportation', MS='The means of moving persons, animals, goods, or materials from one place to another.'),
  'D014187': NtO(MH='Transportation of Patients', MS='Conveying ill or injured individuals from one place to another.'),
  'D014188': NtO(MH='Transposition of Great Vessels', MS='A congenital cardiovascular malformation in which the AORTA arises entirely from the RIGHT VENTRICLE, and the PULMONARY ARTERY arises from the LEFT VENTRICLE. Consequently, the pulmonary and the systemic circulations are parallel and not sequential, so that the venous return from the peripheral circulation is re-circulated by the right ventricle via aorta to the systemic circulation without being oxygenated in the lungs. This is a potentially lethal form of heart disease in newborns and infants.'),
  'D014189': NtO(MH='Transsexualism', MS='Expression of a GENDER IDENTITY inconsistent with, or not culturally-associated with the gender assigned to an individual at birth, combined with the desire to permanently transition to the gender with which they identify.'),
  'D014190': NtO(MH='Transvestism', MS='Disorder characterized by recurrent, intense sexually arousing fantasies, sexual urges, or behaviors involving cross-dressing in a heterosexual male. The fantasies, urges, or behaviors cause clinically significant distress or impairment in social, occupational or other areas of functioning. (from APA, DSM-IV, 1994)'),
  'D014191': NtO(MH='Tranylcypromine', MS='A propylamine formed from the cyclization of the side chain of amphetamine. This monoamine oxidase inhibitor is effective in the treatment of major depression, dysthymic disorder, and atypical depression. It also is useful in panic and phobic disorders. (From AMA Drug Evaluations Annual, 1994, p311)'),
  'D014192': NtO(MH='Trapidil', MS='A coronary vasodilator agent.'),
  'D014193': NtO(MH='Trauma Centers', MS='Specialized hospital facilities which provide diagnostic and therapeutic services for trauma patients.'),
  'D014194': NtO(MH='Traumatology', MS='The medical specialty which deals with WOUNDS and INJURIES as well as resulting disability and disorders from physical traumas.'),
  'D014195': NtO(MH='Travel', MS='MOVEMENT across different GEOGRAPHIC LOCATIONS.'),
  'D014196': NtO(MH='Trazodone', MS='A serotonin uptake inhibitor that is used as an antidepressive agent. It has been shown to be effective in patients with major depressive disorders and other subsets of depressive disorders. It is generally more useful in depressive disorders associated with insomnia and anxiety. This drug does not aggravate psychotic symptoms in patients with schizophrenia or schizoaffective disorders. (From AMA Drug Evaluations Annual, 1994, p309)'),
  'D014197': NtO(MH='Trees', MS='Woody, usually tall, perennial higher plants (Angiosperms, Gymnosperms, and some Pterophyta) having usually a main stem and numerous branches.'),
  'D014198': NtO(MH='Trehalase', MS='An enzyme that catalyzes the conversion of alpha,alpha-trehalose and water to D-glucose. EC 3.2.1.28.'),
  'D014199': NtO(MH='Trehalose', MS=None),
  'D014200': NtO(MH='Trematoda', MS='Class of parasitic flukes consisting of three subclasses, Monogenea, Aspidogastrea, and Digenea. The digenetic trematodes are the only ones found in man. They are endoparasites and require two hosts to complete their life cycle.'),
  'D014201': NtO(MH='Trematode Infections', MS='Infections caused by infestation with worms of the class Trematoda.'),
  'D014202': NtO(MH='Tremor', MS='Cyclical movement of a body part that can represent either a physiologic process or a manifestation of disease. Intention or action tremor, a common manifestation of CEREBELLAR DISEASES, is aggravated by movement. In contrast, resting tremor is maximal when there is no attempt at voluntary movement, and occurs as a relatively frequent manifestation of PARKINSON DISEASE.'),
  'D014203': NtO(MH='Tremorine', MS=None),
  'D014204': NtO(MH='Trenbolone Acetate', MS='An anabolic steroid used mainly as an anabolic agent in veterinary practice.'),
  'D014205': NtO(MH='Trench Fever', MS='An intermittent fever characterized by intervals of chills, fever, and splenomegaly each of which may last as long as 40 hours. It is caused by BARTONELLA QUINTANA and transmitted by the human louse.'),
  'D014207': NtO(MH='Trephining', MS='The removal of a circular disk of the cranium.'),
  'D014208': NtO(MH='Treponema', MS='A genus of microorganisms of the order SPIROCHAETALES, many of which are pathogenic and parasitic for man and animals.'),
  'D014209': NtO(MH='Treponema Immobilization Test', MS='Syphilis serodiagnosis employing as the antigen Treponema pallidum obtained from rabbit syphilis orchitis. Treponemes are kept alive for a few hours in a special medium. When syphilitic serum and complement are added and incubated, the treponemes are immobilized, i.e., stop moving.'),
  'D014210': NtO(MH='Treponema pallidum', MS='The causative agent of venereal and non-venereal syphilis as well as yaws.'),
  'D014211': NtO(MH='Treponemal Infections', MS='Infections with bacteria of the genus TREPONEMA.'),
  'D014212': NtO(MH='Tretinoin', MS='An important regulator of GENE EXPRESSION during growth and development, and in NEOPLASMS. Tretinoin, also known as retinoic acid and derived from maternal VITAMIN A, is essential for normal GROWTH; and EMBRYONIC DEVELOPMENT. An excess of tretinoin can be teratogenic. It is used in the treatment of PSORIASIS; ACNE VULGARIS; and several other SKIN DISEASES. It has also been approved for use in promyelocytic leukemia (LEUKEMIA, PROMYELOCYTIC, ACUTE).'),
  'D014213': NtO(MH='Tretoquinol', MS='An adrenergic beta-agonist used as a bronchodilator agent in asthma therapy.'),
  'D014214': NtO(MH='Triallate', MS='A pre-emergence, selective herbicide for the control of wild oats in various crops.'),
  'D014215': NtO(MH='Triacetin', MS='A triglyceride that is used as an antifungal agent.'),
  'D014216': NtO(MH='Triacetoneamine-N-Oxyl', MS='Cyclic N-oxide radical functioning as a spin label and radiation-sensitizing agent.'),
  'D014217': NtO(MH='Troleandomycin', MS='A macrolide antibiotic that is similar to ERYTHROMYCIN.'),
  'D014218': NtO(MH='Triage', MS='The sorting out and classification of patients or casualties to determine priority of need and proper place of treatment.'),
  'D014219': NtO(MH='Trial of Labor', MS='Allowing a woman to be in LABOR, OBSTETRIC long enough to determine if vaginal birth may be anticipated.'),
  'D014220': NtO(MH='Trialkyltin Compounds', MS='Organometallic compounds which contain tin and three alkyl groups.'),
  'D014221': NtO(MH='Triamcinolone', MS='A glucocorticoid given, as the free alcohol or in esterified form, orally, intramuscularly, by local injection, by inhalation, or applied topically in the management of various disorders in which corticosteroids are indicated. (From Martindale, The Extra Pharmacopoeia, 30th ed, p739)'),
  'D014222': NtO(MH='Triamcinolone Acetonide', MS='An esterified form of TRIAMCINOLONE. It is an anti-inflammatory glucocorticoid used topically in the treatment of various skin disorders. Intralesional, intramuscular, and intra-articular injections are also administered under certain conditions.'),
  'D014223': NtO(MH='Triamterene', MS='A pteridinetriamine compound that inhibits SODIUM reabsorption through SODIUM CHANNELS in renal EPITHELIAL CELLS.'),
  'D014224': NtO(MH='Triatoma', MS='A genus of the subfamily TRIATOMINAE. Several species are vectors of TRYPANOSOMA CRUZI.'),
  'D014225': NtO(MH='Triatominae', MS='A subfamily of assassin bugs (REDUVIIDAE) that are obligate blood-suckers of vertebrates. Included are the genera TRIATOMA; RHODNIUS; and PANSTRONGYLUS, which are vectors of TRYPANOSOMA CRUZI, the agent of CHAGAS DISEASE in humans.'),
  'D014226': NtO(MH='Triazenes', MS='Compounds with three contiguous nitrogen atoms in linear format, H2N-N=NH, and hydrocarbyl derivatives.'),
  'D014227': NtO(MH='Triazines', MS='Heterocyclic rings containing three nitrogen atoms, commonly in 1,2,4 or 1,3,5 or 2,4,6 formats. Some are used as HERBICIDES.'),
  'D014228': NtO(MH='Triaziquone', MS='Alkylating antineoplastic agent used mainly for ovarian tumors. It is toxic to skin, gastrointestinal tract, bone marrow and kidneys.'),
  'D014229': NtO(MH='Triazolam', MS='A short-acting benzodiazepine used in the treatment of insomnia. Some countries temporarily withdrew triazolam from the market because of concerns about adverse reactions, mostly psychological, associated with higher dose ranges. Its use at lower doses with appropriate care and labeling has been reaffirmed by the FDA and most other countries.'),
  'D014230': NtO(MH='Triazoles', MS=None),
  'D014231': NtO(MH='Tribolium', MS='A genus of small beetles of the family Tenebrionidae; T. confusum is the "confused flour beetle".'),
  'D014233': NtO(MH='Tricarboxylic Acids', MS='Organic compounds that are acyclic and contain three acid groups. A member of this class is citric acid which is the first product formed by reaction of pyruvate and oxaloacetate. (From Lehninger, Principles of Biochemistry, 1982, p443)'),
  'D014234': NtO(MH='Trichinella', MS='A genus of parasitic nematodes that causes TRICHINELLOSIS in man and other animal.'),
  'D014235': NtO(MH='Trichinellosis', MS='An infection with TRICHINELLA. It is caused by eating raw or undercooked meat that is infected with larvae of nematode worms TRICHINELLA genus. All members of the TRICHINELLA genus can infect human in addition to TRICHINELLA SPIRALIS, the traditional etiological agent. It is distributed throughout much of the world and is re-emerging in some parts as a public health hazard and a food safety problem.'),
  'D014236': NtO(MH='Trichlorfon', MS='An organochlorophosphate cholinesterase inhibitor that is used as an insecticide for the control of flies and roaches. It is also used in anthelmintic compositions for animals. (From Merck, 11th ed)'),
  'D014237': NtO(MH='Trichlormethiazide', MS='A thiazide diuretic with properties similar to those of HYDROCHLOROTHIAZIDE. (From Martindale, The Extra Pharmacopoeia, 30th ed, p830)'),
  'D014238': NtO(MH='Trichloroacetic Acid', MS='A strong acid used as a protein precipitant in clinical chemistry and also as a caustic for removing warts.'),
  'D014239': NtO(MH='Trichloroepoxypropane', MS='A potent epoxide hydrase and aryl hydrocarbon hydroxylase inhibitor. It enhances the tumor-initiating ability of certain carcinogens.'),
  'D014240': NtO(MH='Trichloroethanes', MS='Chlorinated ethanes which are used extensively as industrial solvents. They have been utilized in numerous home-use products including spot remover preparations and inhalant decongestant sprays. These compounds cause central nervous system and cardiovascular depression and are hepatotoxic. Include 1,1,1- and 1,1,2-isomers.'),
  'D014241': NtO(MH='Trichloroethylene', MS='A highly volatile inhalation anesthetic used mainly in short surgical procedures where light anesthesia with good analgesia is required. It is also used as an industrial solvent. Prolonged exposure to high concentrations of the vapor can lead to cardiotoxicity and neurological impairment.'),
  'D014242': NtO(MH='Trichoderma', MS='A mitosporic fungal genus frequently found in soil and on wood. It is sometimes used for controlling pathogenic fungi. Its teleomorph is HYPOCREA.'),
  'D014243': NtO(MH='Trichodermin', MS='Antifungal metabolite from several fungi, mainly Trichoderma viride; inhibits protein synthesis by binding to ribosomes; proposed as antifungal and antineoplastic; used as tool in cellular biochemistry.'),
  'D014244': NtO(MH='Trichomonas', MS='A genus of parasitic flagellate EUKARYOTES distinguished by the presence of four anterior flagella, an undulating membrane, and a trailing flagellum.'),
  'D014245': NtO(MH='Trichomonas Infections', MS='Infections in birds and mammals produced by various species of Trichomonas.'),
  'D014246': NtO(MH='Trichomonas vaginalis', MS='A species of TRICHOMONAS that produces a refractory vaginal discharge in females, as well as bladder and urethral infections in males.'),
  'D014247': NtO(MH='Trichomonas Vaginitis', MS='Inflammation of the vagina, marked by a purulent discharge. This disease is caused by the protozoan TRICHOMONAS VAGINALIS.'),
  'D014248': NtO(MH='Trichophytin', MS='Antigens from the genus TRICHOPHYTON. Trichophytin has been used in studies of immunological responses to these dermatophytes.'),
  'D014249': NtO(MH='Trichophyton', MS='A mitosporic fungal genus and an anamorphic form of Arthroderma. Various species attack the skin, nails, and hair.'),
  'D014250': NtO(MH='Trichosporon', MS='A mitosporic fungal genus causing opportunistic infections, endocarditis, fungemia, a hypersensitivity pneumonitis (see TRICHOSPORONOSIS) and white PIEDRA.'),
  'D014251': NtO(MH='Trichostrongyloidea', MS='A superfamily of nematodes. Most are intestinal parasites of ruminants and accidentally in humans. This superfamily includes seven genera: DICTYOCAULUS; HAEMONCHUS; Cooperia, OSTERTAGIA; Nematodirus, TRICHOSTRONGYLUS; and Hyostrongylus.'),
  'D014252': NtO(MH='Trichostrongyloidiasis', MS='Infection by roundworms of the superfamily TRICHOSTRONGYLOIDEA, including the genera TRICHOSTRONGYLUS; OSTERTAGIA; Cooperia, HAEMONCHUS; Nematodirus, Hyostrongylus, and DICTYOCAULUS.'),
  'D014253': NtO(MH='Trichostrongylosis', MS='Infestation with nematode worms of the genus TRICHOSTRONGYLUS. Man and animals become infected by swallowing larvae, usually with contaminated food or drink, although the larvae may penetrate human skin.'),
  'D014254': NtO(MH='Trichostrongylus', MS='A genus of parasitic nematodes found in the digestive tract of herbivorous animals. They cause incidental infections in humans from the following species: Trichostrongylus colubriformis, T. orientalis, T. axei, and T. probolurus.'),
  'D014255': NtO(MH='Trichothecenes', MS='Usually 12,13-epoxytrichothecenes, produced by Fusaria, Stachybotrys, Trichoderma and other fungi, and some higher plants. They may contaminate food or feed grains, induce emesis and hemorrhage in lungs and brain, and damage bone marrow due to protein and DNA synthesis inhibition.'),
  'D014256': NtO(MH='Trichotillomania', MS="Compulsion to pull out one's hair."),
  'D014257': NtO(MH='Trichuriasis', MS='Infection with nematodes of the genus TRICHURIS, formerly called Trichocephalus.'),
  'D014258': NtO(MH='Trichuris', MS='A genus of nematode worms comprising the whipworms.'),
  'D014259': NtO(MH='Trichuroidea', MS='A superfamily of nematodes of the order ENOPLIDA. Its organisms have a well developed intestine and rectum.'),
  'D014260': NtO(MH='Triclosan', MS='A diphenyl ether derivative used in cosmetics and toilet soaps as an antiseptic. It has some bacteriostatic and fungistatic action.'),
  'D014261': NtO(MH='Tricuspid Valve', MS='The valve consisting of three cusps situated between the right atrium and right ventricle of the heart.'),
  'D014262': NtO(MH='Tricuspid Valve Insufficiency', MS='Backflow of blood from the RIGHT VENTRICLE into the RIGHT ATRIUM due to imperfect closure of the TRICUSPID VALVE.'),
  'D014263': NtO(MH='Tricuspid Valve Prolapse', MS='Abnormal protrusion of one or more of the leaflets of TRICUSPID VALVE into the RIGHT ATRIUM during SYSTOLE. This allows the backflow of blood into right atrium leading to TRICUSPID VALVE INSUFFICIENCY; SYSTOLIC MURMURS. Its most common cause is not primary valve abnormality but rather the dilation of the RIGHT VENTRICLE and the tricuspid annulus.'),
  'D014264': NtO(MH='Tricuspid Valve Stenosis', MS='The pathologic narrowing of the orifice of the TRICUSPID VALVE. This hinders the emptying of RIGHT ATRIUM leading to elevated right atrial pressure and systemic venous congestion. Tricuspid valve stenosis is almost always due to RHEUMATIC FEVER.'),
  'D014265': NtO(MH='Triethylenemelamine', MS='Toxic alkylating agent used in industry; also as antineoplastic and research tool to produce chromosome aberrations and cancers.'),
  'D014266': NtO(MH='Trientine', MS='An ethylenediamine derivative used as stabilizer for EPOXY RESINS, as ampholyte for ISOELECTRIC FOCUSING and as chelating agent for copper in HEPATOLENTICULAR DEGENERATION.'),
  'D014267': NtO(MH='Triethyltin Compounds', MS='Organic compounds composed of tin and three ethyl groups. Affect mitochondrial metabolism and inhibit oxidative phosphorylation by acting directly on the energy conserving processes.'),
  'D014268': NtO(MH='Trifluoperazine', MS='A phenothiazine with actions similar to CHLORPROMAZINE. It is used as an antipsychotic and an antiemetic.'),
  'D014269': NtO(MH='Trifluoroacetic Acid', MS='A very strong halogenated derivative of acetic acid. It is used in acid catalyzed reactions, especially those where an ester is cleaved in peptide synthesis.'),
  'D014270': NtO(MH='Trifluoroethanol', MS='A non-aqueous co-solvent that serves as tool to study protein folding. It is also used in various pharmaceutical, chemical and engineering applications.'),
  'D014271': NtO(MH='Trifluridine', MS='An antiviral derivative of THYMIDINE used mainly in the treatment of primary keratoconjunctivitis and recurrent epithelial keratitis due to HERPES SIMPLEX virus. (From Martindale, The Extra Pharmacopoeia, 30th ed, p557)'),
  'D014272': NtO(MH='Trifluperidol', MS='A butyrophenone with general properties similar to those of HALOPERIDOL. It is used in the treatment of PSYCHOSES including MANIA and SCHIZOPHRENIA. (From Martindale, The Extra Pharmacopoeia, 30th ed, p621)'),
  'D014273': NtO(MH='Triflupromazine', MS='A phenothiazine used as an antipsychotic agent and as an antiemetic.'),
  'D014274': NtO(MH='Trifluralin', MS='A microtubule-disrupting pre-emergence herbicide.'),
  'D014275': NtO(MH='Trigeminal Caudal Nucleus', MS='The caudal portion of the nucleus of the spinal trigeminal tract (TRIGEMINAL NUCLEUS, SPINAL), a nucleus involved with pain and temperature sensation.'),
  'D014276': NtO(MH='Trigeminal Nerve', MS='The 5th and largest cranial nerve. The trigeminal nerve is a mixed motor and sensory nerve. The larger sensory part forms the ophthalmic, mandibular, and maxillary nerves which carry afferents sensitive to external or internal stimuli from the skin, muscles, and joints of the face and mouth and from the teeth. Most of these fibers originate from cells of the TRIGEMINAL GANGLION and project to the TRIGEMINAL NUCLEUS of the brain stem. The smaller motor part arises from the brain stem trigeminal motor nucleus and innervates the muscles of mastication.'),
  'D014277': NtO(MH='Trigeminal Neuralgia', MS='A syndrome characterized by recurrent episodes of excruciating pain lasting several seconds or longer in the sensory distribution of the TRIGEMINAL NERVE. Pain may be initiated by stimulation of trigger points on the face, lips, or gums or by movement of facial muscles or chewing. Associated conditions include MULTIPLE SCLEROSIS, vascular anomalies, ANEURYSMS, and neoplasms. (Adams et al., Principles of Neurology, 6th ed, p187)'),
  'D014278': NtO(MH='Trigeminal Nuclei', MS='Nuclei of the trigeminal nerve situated in the brain stem. They include the nucleus of the spinal trigeminal tract (TRIGEMINAL NUCLEUS, SPINAL), the principal sensory nucleus, the mesencephalic nucleus, and the motor nucleus.'),
  'D014279': NtO(MH='Trigeminal Nucleus, Spinal', MS='Nucleus of the spinal tract of the trigeminal nerve. It is divided cytoarchitectonically into three parts: oralis, caudalis (TRIGEMINAL CAUDAL NUCLEUS), and interpolaris.'),
  'D014280': NtO(MH='Triglycerides', MS=None),
  'D014281': NtO(MH='Trihexosylceramides', MS='Glycosphingolipids which contain as their polar head group a trisaccharide (galactose-galactose-glucose) moiety bound in glycosidic linkage to the hydroxyl group of ceramide. Their accumulation in tissue, due to a defect in ceramide trihexosidase, is the cause of angiokeratoma corporis diffusum (FABRY DISEASE).'),
  'D014282': NtO(MH='Trihexyphenidyl', MS='One of the centrally acting MUSCARINIC ANTAGONISTS used for treatment of PARKINSONIAN DISORDERS and drug-induced extrapyramidal movement disorders and as an antispasmodic.'),
  'D014283': NtO(MH='Triiodobenzoic Acids', MS='Triiodo-substituted derivatives of BENZOIC ACID.'),
  'D014284': NtO(MH='Triiodothyronine', MS="A T3 thyroid hormone normally synthesized and secreted by the thyroid gland in much smaller quantities than thyroxine (T4). Most T3 is derived from peripheral monodeiodination of T4 at the 5' position of the outer ring of the iodothyronine nucleus. The hormone finally delivered and used by the tissues is mainly T3."),
  'D014285': NtO(MH='Triiodothyronine, Reverse', MS='A metabolite of THYROXINE, formed by the peripheral enzymatic monodeiodination of T4 at the 5 position of the inner ring of the iodothyronine nucleus.'),
  'D014286': NtO(MH='Trilogy of Fallot', MS='A combination of congenital heart defects consisting of three key features including ATRIAL SEPTAL DEFECTS; PULMONARY STENOSIS; and RIGHT VENTRICULAR HYPERTROPHY.'),
  'D014287': NtO(MH='Trimebutine', MS='Proposed spasmolytic with possible local anesthetic action used in gastrointestinal disorders.'),
  'D014288': NtO(MH='Trimecaine', MS='Acetanilide derivative used as a local anesthetic.'),
  'D014289': NtO(MH='Trimedoxime', MS='Cholinesterase reactivator used as an antidote in alkyl phosphate poisoning.'),
  'D014290': NtO(MH='Metipranolol', MS='A beta-adrenergic antagonist effective for both beta-1 and beta-2 receptors. It is used as an antiarrhythmic, antihypertensive, and antiglaucoma agent.'),
  'D014291': NtO(MH='Trimeprazine', MS='A phenothiazine derivative that is used as an antipruritic.'),
  'D014292': NtO(MH='Trimetazidine', MS='A vasodilator used in angina of effort or ischemic heart disease.'),
  'D014293': NtO(MH='Trimethadione', MS='An anticonvulsant effective in absence seizures, but generally reserved for refractory cases because of its toxicity. (From AMA Drug Evaluations Annual, 1994, p378)'),
  'D014294': NtO(MH='Trimethaphan', MS='A nicotinic antagonist that has been used as a ganglionic blocker in hypertension, as an adjunct to anesthesia, and to induce hypotension during surgery.'),
  'D014295': NtO(MH='Trimethoprim', MS='A pyrimidine inhibitor of dihydrofolate reductase, it is an antibacterial related to PYRIMETHAMINE. It is potentiated by SULFONAMIDES and the TRIMETHOPRIM, SULFAMETHOXAZOLE DRUG COMBINATION is the form most often used. It is sometimes used alone as an antimalarial. TRIMETHOPRIM RESISTANCE has been reported.'),
  'D014296': NtO(MH='Trimethoprim Resistance', MS='Nonsusceptibility of bacteria to the action of TRIMETHOPRIM.'),
  'D014297': NtO(MH='Trimethylsilyl Compounds', MS='Organic silicon derivatives used to characterize hydroxysteroids, nucleosides, and related compounds. Trimethylsilyl esters of amino acids are used in peptide synthesis.'),
  'D014298': NtO(MH='Trimethyltin Compounds', MS='Organic compounds composed of tin and three methyl groups. Affect mitochondrial metabolism and inhibit oxidative phosphorylation by acting directly on the energy conserving processes.'),
  'D014299': NtO(MH='Trimipramine', MS='Tricyclic antidepressant similar to IMIPRAMINE, but with more antihistaminic and sedative properties.'),
  'D014300': NtO(MH='Trinidad and Tobago', MS="An independent state in the Lesser Antilles in the West Indies, north of Venezuela, comprising the islands of Trinidad and Tobago. Its capital is Port of Spain. Both islands were discovered by Columbus in 1498. The Spanish, English, Dutch, and French figure in their history over four centuries. Trinidad and Tobago united in 1898 and were made part of the British colony of Trinidad and Tobago in 1899. The colony became an independent state in 1962. Trinidad was so named by Columbus either because he arrived on Trinity Sunday or because three mountain peaks suggested the Holy Trinity. Tobago was given the name by Columbus from the Haitian tambaku, pipe, from the natives' habit of smoking tobacco leaves. (Webster's New Geographical Dictionary, 1988, p1228, 1216 & Room, Brewer's Dictionary of Names, 1992, p555, 547)"),
  'D014301': NtO(MH='Trinitrobenzenes', MS='Benzene derivatives which are substituted with three nitro groups in any position.'),
  'D014302': NtO(MH='Trinitrobenzenesulfonic Acid', MS='A reagent that is used to neutralize peptide terminal amino groups.'),
  'D014303': NtO(MH='Trinitrotoluene', MS='A 2,4,6-trinitrotoluene, which is an explosive chemical that can cause skin irritation and other toxic consequences.'),
  'D014304': NtO(MH='Triolein', MS='(Z)-9-Octadecenoic acid 1,2,3-propanetriyl ester.'),
  'D014305': NtO(MH='Triose-Phosphate Isomerase', MS='An enzyme that catalyzes reversibly the conversion of D-glyceraldehyde 3-phosphate to dihydroxyacetone phosphate. A deficiency in humans causes nonspherocytic hemolytic disease (ANEMIA, HEMOLYTIC, CONGENITAL NONSPHEROCYTIC). EC 5.3.1.1.'),
  'D014306': NtO(MH='Trioses', MS='Monosaccharide molecules that contain a three carbon backbone.'),
  'D014307': NtO(MH='Trioxsalen', MS='Pigmenting photosensitizing agent obtained from several plants, mainly Psoralea corylifolia. It is administered either topically or orally in conjunction with ultraviolet light in the treatment of vitiligo.'),
  'D014308': NtO(MH='Triparanol', MS='Antilipemic agent with high ophthalmic toxicity. According to Merck Index, 11th ed, the compound was withdrawn from the market in 1962 because of its association with the formation of irreversible cataracts.'),
  'D014309': NtO(MH='Tripelennamine', MS='A histamine H1 antagonist with low sedative action but frequent gastrointestinal irritation. It is used to treat ASTHMA; HAY FEVER; URTICARIA; and RHINITIS; and also in veterinary applications. Tripelennamine is administered by various routes, including topically.'),
  'D014310': NtO(MH='Triplets', MS='Three individuals derived from three FETUSES that were fertilized at or about the same time, developed in the UTERUS simultaneously, and born to the same mother.'),
  'D014311': NtO(MH='Triprolidine', MS='Histamine H1 antagonist used in allergic rhinitis; ASTHMA; and URTICARIA. It is a component of COUGH and COLD medicines. It may cause drowsiness.'),
  'D014312': NtO(MH='Trisaccharides', MS='Oligosaccharides containing three monosaccharide units linked by glycosidic bonds.'),
  'D014313': NtO(MH='Trismus', MS='Spasmodic contraction of the masseter muscle resulting in forceful jaw closure. This may be seen with a variety of diseases, including TETANUS, as a complication of radiation therapy, trauma, or in association with neoplastic conditions.'),
  'D014314': NtO(MH='Trisomy', MS='The possession of a third chromosome of any one type in an otherwise diploid cell.'),
  'D014315': NtO(MH='Triterpenes', MS='A class of terpenes (the general formula C30H48) formed by the condensation of six isoprene units, equivalent to three terpene units.'),
  'D014316': NtO(MH='Tritium', MS='The radioactive isotope of hydrogen also known as hydrogen-3. It contains two NEUTRONS and one PROTON in its nucleus and decays to produce low energy BETA PARTICLES.'),
  'D014317': NtO(MH='Tritolyl Phosphates', MS='A mixture of isomeric tritolyl phosphates. Used in the sterilization of certain surgical instruments and in many industrial processes.'),
  'D014318': NtO(MH='Tritrichomonas', MS='A genus of flagellate EUKARYOTES possessing three long anterior flagella.'),
  'D014319': NtO(MH='Triturus', MS='A genus of aquatic newts in the Salamandridae family. During breeding season many Triturus males have a dorsal crest which also serves as an accessory respiratory organ. One of the common Triturus species is Triturus cristatus (crested newt).'),
  'D014320': NtO(MH='Trityl Compounds', MS='Derivatives of triphenylmethane.'),
  'D014321': NtO(MH='Trochlear Nerve', MS='The 4th cranial nerve. The trochlear nerve carries the motor innervation of the superior oblique muscles of the eye.'),
  'D014322': NtO(MH='Troglotrematidae', MS='A family of flukes (TREMATODA) usually under 10 mm in length, some being only 1 or 2 mm long. In general they are thick, spinous, or scaly flukes having an oval outline and living in various organs and sinuses of birds and mammals. The genus with the greatest interest is PARAGONIMUS. (From Noble et al., Parasitology: the Biology of Animal Parasites, 6th ed, p191)'),
  'D014323': NtO(MH='Trombiculiasis', MS='Infestation with mites of the genus Trombicula, whose larvae carry the rickettsial agent of scrub typhus.'),
  'D014324': NtO(MH='Trombiculidae', MS="Family of MITES in the superfamily Trombiculoidea, suborder Prostigmata, which attack humans and other vertebrates, causing DERMATITIS and severe allergic reactions. Chiggers, red bugs, and harvest mites commonly refer to the larval stage of Trombiculid mites, the only parasitic stage of the mite's life cycle."),
  'D014325': NtO(MH='Tromethamine', MS='An organic amine proton acceptor. It is used in the synthesis of surface-active agents and pharmaceuticals; as an emulsifying agent for cosmetic creams and lotions, mineral oil and paraffin wax emulsions, as a biological buffer, and used as an alkalizer. (From Merck, 11th ed; Martindale, The Extra Pharmacopoeia, 30th ed, p1424)'),
  'D014326': NtO(MH='Tropanes', MS='N-methyl-8-azabicyclo[3.2.1]octanes best known for the ones found in PLANTS.'),
  'D014327': NtO(MH='Trophoblasts', MS='Cells lining the outside of the BLASTOCYST. After binding to the ENDOMETRIUM, trophoblasts develop into two distinct layers, an inner layer of mononuclear cytotrophoblasts and an outer layer of continuous multinuclear cytoplasm, the syncytiotrophoblasts, which form the early fetal-maternal interface (PLACENTA).'),
  'D014328': NtO(MH='Trophoblastic Neoplasms', MS='Trophoblastic growth, which may be gestational or nongestational in origin. Trophoblastic neoplasia resulting from pregnancy is often described as gestational trophoblastic disease to distinguish it from germ cell tumors which frequently show trophoblastic elements, and from the trophoblastic differentiation which sometimes occurs in a wide variety of epithelial cancers. Gestational trophoblastic growth has several forms, including HYDATIDIFORM MOLE and CHORIOCARCINOMA. (From Holland et al., Cancer Medicine, 3d ed, p1691)'),
  'D014329': NtO(MH='Tropical Climate', MS='A climate which is typical of equatorial and tropical regions, i.e., one with continually high temperatures with considerable precipitation, at least during part of the year. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D014330': NtO(MH='Tropical Medicine', MS='The branch of medicine concerned with diseases, mainly of parasitic origin, common in tropical and subtropical regions.'),
  'D014331': NtO(MH='Tropicamide', MS='One of the MUSCARINIC ANTAGONISTS with pharmacologic action similar to ATROPINE and used mainly as an ophthalmic parasympatholytic or mydriatic.'),
  'D014332': NtO(MH='Tropocollagen', MS='The molecular unit of collagen fibrils that consist of repeating three-stranded polypeptide units arranged head to tail in parallel bundles. It is a right-handed triple helix composed of 2 polypeptide chains. It is rich in glycine, proline, hydroxyproline, and hydroxylysine.'),
  'D014333': NtO(MH='Tropoelastin', MS='A salt-soluble precursor of elastin. Lysyl oxidase is instrumental in converting it to elastin in connective tissue.'),
  'D014334': NtO(MH='Tropolone', MS='A seven-membered aromatic ring compound. It is structurally related to a number of naturally occurring antifungal compounds (ANTIFUNGAL AGENTS).'),
  'D014335': NtO(MH='Tropomyosin', MS='A protein found in the thin filaments of muscle fibers. It inhibits contraction of the muscle unless its position is modified by TROPONIN.'),
  'D014336': NtO(MH='Troponin', MS='One of the minor protein components of skeletal and cardiac muscles. It functions as the calcium-binding component in a complex with BETA-TROPOMYOSIN; ACTIN; and MYOSIN and confers calcium sensitivity to the cross-linked actin and myosin filaments. Troponin itself is a complex of three regulatory proteins (TROPONIN C; TROPONIN I; and TROPONIN T).'),
  'D014337': NtO(MH='Trout', MS='Various fish of the family SALMONIDAE, usually smaller than salmon. They are mostly restricted to cool clear freshwater. Some are anadromous. They are highly regarded for their handsome colors, rich well-flavored flesh, and gameness as an angling fish. The genera Salvelinus, Salmo, and ONCORHYNCHUS have been introduced virtually throughout the world.'),
  'D014338': NtO(MH='Truncus Arteriosus', MS='The arterial trunk arising from the fetal heart. During development, it divides into AORTA and the PULMONARY ARTERY.'),
  'D014339': NtO(MH='Truncus Arteriosus, Persistent', MS='A congenital anomaly caused by the failed development of TRUNCUS ARTERIOSUS into separate AORTA and PULMONARY ARTERY. It is characterized by a single arterial trunk that forms the outlet for both HEART VENTRICLES and gives rise to the systemic, pulmonary, and coronary arteries. It is always accompanied by a ventricular septal defect.'),
  'D014340': NtO(MH='Trusses', MS='A surgical device designed for retaining a hernia in a reduced state within the abdominal cavity.'),
  'D014341': NtO(MH='Trustees', MS='Board members of an institution or organization who are entrusted with the administering of funds and the directing of policy.'),
  'D014342': NtO(MH='Truth Disclosure', MS='Truthful revelation of information, specifically when the information disclosed is likely to be psychologically painful ("bad news") to the recipient (e.g., revelation to a patient or a patient\'s family of the patient\'s DIAGNOSIS or PROGNOSIS) or embarrassing to the teller (e.g., revelation of medical errors).'),
  'D014343': NtO(MH='Trypan Blue', MS='A diazo-naphthalene sulfonate that is widely used as a stain.'),
  'D014344': NtO(MH='Trypanocidal Agents', MS='Agents destructive to the protozoal organisms belonging to the suborder TRYPANOSOMATINA.'),
  'D014345': NtO(MH='Trypanosoma', MS='A genus of flagellate protozoans found in the BLOOD and LYMPH of vertebrates and invertebrates, both hosts being required to complete the life cycle.'),
  'D014346': NtO(MH='Trypanosoma brucei brucei', MS='A hemoflagellate subspecies of parasitic protozoa that causes nagana in domestic and game animals in Africa. It apparently does not infect humans. It is transmitted by bites of tsetse flies (Glossina).'),
  'D014347': NtO(MH='Trypanosoma brucei gambiense', MS='A hemoflagellate subspecies of parasitic protozoa that causes Gambian or West African sleeping sickness in humans. The vector host is usually the tsetse fly (Glossina).'),
  'D014348': NtO(MH='Trypanosoma congolense', MS='A species of Trypanosome hemoflagellates that is carried by tsetse flies and causes severe anemia in cattle. These parasites are also found in horses, sheep, goats, and camels.'),
  'D014349': NtO(MH='Trypanosoma cruzi', MS='The agent of South American trypanosomiasis or CHAGAS DISEASE. Its vertebrate hosts are man and various domestic and wild animals. Insects of several species are vectors.'),
  'D014350': NtO(MH='Trypanosoma lewisi', MS='A trypanosome found in the blood of adult rats and transmitted by the rat flea. It is generally non-pathogenic in adult rats but can cause lethal infection in suckling rats.'),
  'D014351': NtO(MH='Trypanosomatina', MS='A suborder of monoflagellate parasitic protozoa that lives in the blood and tissues of man and animals. Representative genera include: Blastocrithidia, Leptomonas, CRITHIDIA, Herpetomonas, LEISHMANIA, Phytomonas, and TRYPANOSOMA. Species of this suborder may exist in two or more morphologic stages formerly named after genera exemplifying these forms - amastigote (LEISHMANIA), choanomastigote (CRITHIDIA), promastigote (Leptomonas), opisthomastigote (Herpetomonas), epimastigote (Blastocrithidia), and trypomastigote (TRYPANOSOMA).'),
  'D014352': NtO(MH='Trypanosomiasis', MS='Infection with protozoa of the genus TRYPANOSOMA.'),
  'D014353': NtO(MH='Trypanosomiasis, African', MS='A disease endemic among people and animals in Central Africa. It is caused by various species of trypanosomes, particularly T. gambiense and T. rhodesiense. Its second host is the TSETSE FLY. Involvement of the central nervous system produces "African sleeping sickness." Nagana is a rapidly fatal trypanosomiasis of horses and other animals.'),
  'D014354': NtO(MH='Trypanosomiasis, Bovine', MS='Infection in cattle caused by various species of trypanosomes.'),
  'D014355': NtO(MH='Chagas Disease', MS='Infection with the protozoan parasite TRYPANOSOMA CRUZI, a form of TRYPANOSOMIASIS endemic in Central and South America. It is named after the Brazilian physician Carlos Chagas, who discovered the parasite. Infection by the parasite (positive serologic result only) is distinguished from the clinical manifestations that develop years later, such as destruction of PARASYMPATHETIC GANGLIA; CHAGAS CARDIOMYOPATHY; and dysfunction of the ESOPHAGUS or COLON.'),
  'D014357': NtO(MH='Trypsin', MS='A serine endopeptidase that is formed from TRYPSINOGEN in the pancreas. It is converted into its active form by ENTEROPEPTIDASE in the small intestine. It catalyzes hydrolysis of the carboxyl group of either arginine or lysine. EC 3.4.21.4.'),
  'D014358': NtO(MH='Trypsin Inhibitor, Bowman-Birk Soybean', MS='A low-molecular-weight protein (minimum molecular weight 8000) which has the ability to inhibit trypsin as well as chymotrypsin at independent binding sites. It is characterized by a high cystine content and the absence of glycine.'),
  'D014359': NtO(MH='Trypsin Inhibitor, Kazal Pancreatic', MS='A secreted KAZAL MOTIF-containing serine peptidase inhibitor that inhibits TRYPSIN. It is a protein composed of 56 amino acid residues and is different in amino acid composition and physiological activity from the Kunitz bovine pancreatic trypsin inhibitor (APROTININ). It protects against the trypsin-mediated premature activation of ENZYME PRECURSORS in the PANCREAS. Mutations in the SPINK1 gene are associated with CHRONIC PANCREATITIS.'),
  'D014360': NtO(MH='Trypsin Inhibitor, Kunitz Soybean', MS='A high-molecular-weight protein (approximately 22,500) containing 198 amino acid residues. It is a strong inhibitor of trypsin and human plasmin.'),
  'D014361': NtO(MH='Trypsin Inhibitors', MS='Serine proteinase inhibitors which inhibit trypsin. They may be endogenous or exogenous compounds.'),
  'D014362': NtO(MH='Trypsinogen', MS='The inactive proenzyme of trypsin secreted by the pancreas, activated in the duodenum via cleavage by enteropeptidase. (Stedman, 25th ed)'),
  'D014363': NtO(MH='Tryptamines', MS='Decarboxylated monoamine derivatives of TRYPTOPHAN.'),
  'D014364': NtO(MH='Tryptophan', MS='An essential amino acid that is necessary for normal growth in infants and for NITROGEN balance in adults. It is a precursor of INDOLE ALKALOIDS in plants. It is a precursor of SEROTONIN (hence its use as an antidepressant and sleep aid). It can be a precursor to NIACIN, albeit inefficiently, in mammals.'),
  'D014365': NtO(MH='Tryptophan Hydroxylase', MS='An enzyme that catalyzes the hydroxylation of TRYPTOPHAN to 5-HYDROXYTRYPTOPHAN in the presence of NADPH and molecular oxygen. It is important in the biosynthesis of SEROTONIN.'),
  'D014366': NtO(MH='Tryptophan Oxygenase', MS='A dioxygenase with specificity for the oxidation of the indoleamine ring of TRYPTOPHAN. It is a LIVER-specific enzyme that is the first and rate limiting enzyme in the kynurenine pathway of TRYPTOPHAN catabolism.'),
  'D014367': NtO(MH='Tryptophan Synthase', MS='An enzyme that catalyzes the conversion of L-serine and 1-(indol-3-yl)glycerol 3-phosphate to L-tryptophan and glyceraldehyde 3-phosphate. It is a pyridoxal phosphate protein that also catalyzes the conversion of serine and indole into tryptophan and water and of indoleglycerol phosphate into indole and glyceraldehyde phosphate. (From Enzyme Nomenclature, 1992) EC 4.2.1.20.'),
  'D014368': NtO(MH='Tryptophanase', MS='An enzyme that catalyzes the conversion of L-tryptophan and water to indole, pyruvate, and ammonia. It is a pyridoxal-phosphate protein, requiring K+. It also catalyzes 2,3-elimination and beta-replacement reactions of some indole-substituted tryptophan analogs of L-cysteine, L-serine, and other 3-substituted amino acids. (From Enzyme Nomenclature, 1992) EC 4.1.99.1.'),
  'D014369': NtO(MH='Tryptophan-tRNA Ligase', MS='An enzyme that activates tryptophan with its specific transfer RNA. EC 6.1.1.2.'),
  'D014370': NtO(MH='Tsetse Flies', MS='Bloodsucking flies of the genus Glossina, found primarily in equatorial Africa. Several species are intermediate hosts of trypanosomes.'),
  'D014371': NtO(MH='Tuber Cinereum', MS='Layer of GRAY MATTER in the HYPOTHALAMUS that also forms part of the floor of the THIRD VENTRICLE and merges anteriorly into the infundibulum (see PITUITARY GLAND, POSTERIOR).'),
  'D014372': NtO(MH='Tubercidin', MS='An antibiotic purine ribonucleoside that readily substitutes for adenosine in the biological system, but its incorporation into DNA and RNA has an inhibitory effect on the metabolism of these nucleic acids.'),
  'D014373': NtO(MH='Tuberculin', MS='A protein extracted from boiled culture of tubercle bacilli (MYCOBACTERIUM TUBERCULOSIS). It is used in the tuberculin skin test (TUBERCULIN TEST) for the diagnosis of tuberculosis infection in asymptomatic persons.'),
  'D014374': NtO(MH='Tuberculin Test', MS='One of several skin tests to determine past or present tuberculosis infection. A purified protein derivative of the tubercle bacilli, called tuberculin, is introduced into the skin by scratch, puncture, or interdermal injection.'),
  'D014375': NtO(MH='Tuberculoma', MS='A tumor-like mass resulting from the enlargement of a tuberculous lesion.'),
  'D014376': NtO(MH='Tuberculosis', MS='Any of the infectious diseases of man and other animals caused by species of MYCOBACTERIUM TUBERCULOSIS.'),
  'D014378': NtO(MH='Tuberculosis Societies', MS='Voluntary agencies concerned with prevention and treatment of tuberculosis.'),
  'D014379': NtO(MH='Tuberculosis, Avian', MS='A variety of TUBERCULOSIS affecting various birds, including chickens and ducks. It is caused by MYCOBACTERIUM AVIUM and characterized by tubercles consisting principally of epithelioid cells.'),
  'D014380': NtO(MH='Tuberculosis, Bovine', MS='An infection of cattle caused by MYCOBACTERIUM BOVIS. It is transmissible to man and other animals.'),
  'D014381': NtO(MH='Tuberculosis, Cardiovascular', MS='Pathological conditions of the CARDIOVASCULAR SYSTEM caused by infection of MYCOBACTERIUM TUBERCULOSIS. Tuberculosis involvement may include the HEART; the BLOOD VESSELS; or the PERICARDIUM.'),
  'D014382': NtO(MH='Tuberculosis, Cutaneous', MS='Tuberculosis of the skin. It includes scrofuloderma and tuberculid, but not LUPUS VULGARIS.'),
  'D014383': NtO(MH='Tuberculosis, Endocrine', MS='Infection of the ENDOCRINE GLANDS with species of MYCOBACTERIUM, most often MYCOBACTERIUM TUBERCULOSIS.'),
  'D014384': NtO(MH='Tuberculosis, Female Genital', MS='MYCOBACTERIUM infections of the female reproductive tract (GENITALIA, FEMALE).'),
  'D014385': NtO(MH='Tuberculosis, Gastrointestinal', MS='TUBERCULOSIS that involves any region of the GASTROINTESTINAL TRACT, mostly in the distal ILEUM and the CECUM. In most cases, MYCOBACTERIUM TUBERCULOSIS is the pathogen. Clinical features include ABDOMINAL PAIN; FEVER; and palpable mass in the ileocecal area.'),
  'D014386': NtO(MH='Tuberculosis, Hepatic', MS='Infection of the LIVER with species of MYCOBACTERIUM, most often MYCOBACTERIUM TUBERCULOSIS. It is characterized by localized small tuberculous miliary lesions or tumor-like mass (TUBERCULOMA), and abnormalities in liver function tests.'),
  'D014387': NtO(MH='Tuberculosis, Laryngeal', MS='Tuberculosis involving the larynx, producing ulceration of the VOCAL CORDS and the LARYNGEAL MUCOSA.'),
  'D014388': NtO(MH='Tuberculosis, Lymph Node', MS='Infection of the lymph nodes by tuberculosis. Tuberculous infection of the cervical lymph nodes is scrofula.'),
  'D014389': NtO(MH='Tuberculosis, Male Genital', MS='MYCOBACTERIUM infections of the male reproductive tract (GENITALIA, MALE).'),
  'D014390': NtO(MH='Tuberculosis, Meningeal', MS='A form of bacterial meningitis caused by MYCOBACTERIUM TUBERCULOSIS or rarely MYCOBACTERIUM BOVIS. The organism seeds the meninges and forms microtuberculomas which subsequently rupture. The clinical course tends to be subacute, with progressions occurring over a period of several days or longer. Headache and meningeal irritation may be followed by SEIZURES, cranial neuropathies, focal neurologic deficits, somnolence, and eventually COMA. The illness may occur in immunocompetent individuals or as an OPPORTUNISTIC INFECTION in the ACQUIRED IMMUNODEFICIENCY SYNDROME and other immunodeficiency syndromes. (From Adams et al., Principles of Neurology, 6th ed, pp717-9)'),
  'D014391': NtO(MH='Tuberculosis, Miliary', MS='An acute form of TUBERCULOSIS in which minute tubercles are formed in a number of organs of the body due to dissemination of the bacilli through the blood stream.'),
  'D014392': NtO(MH='Tuberculosis, Ocular', MS='Tuberculous infection of the eye, primarily the iris, ciliary body, and choroid.'),
  'D014393': NtO(MH='Tuberculosis, Oral', MS='Tuberculosis of the mouth, tongue, and salivary glands.'),
  'D014394': NtO(MH='Tuberculosis, Osteoarticular', MS='Tuberculosis of the bones or joints.'),
  'D014395': NtO(MH='Peritonitis, Tuberculous', MS='A form of PERITONITIS seen in patients with TUBERCULOSIS, characterized by lesion either as a miliary form or as a pelvic mass on the peritoneal surfaces. Most patients have ASCITES, abdominal swelling, ABDOMINAL PAIN, and other systemic symptoms such as FEVER; WEIGHT LOSS; and ANEMIA.'),
  'D014396': NtO(MH='Tuberculosis, Pleural', MS='Tuberculosis of the serous membrane lining the thoracic cavity and surrounding the lungs.'),
  'D014397': NtO(MH='Tuberculosis, Pulmonary', MS='MYCOBACTERIUM infections of the lung.'),
  'D014398': NtO(MH='Tuberculosis, Renal', MS='Infection of the KIDNEY with species of MYCOBACTERIUM.'),
  'D014399': NtO(MH='Tuberculosis, Spinal', MS='Osteitis or caries of the vertebrae, usually occurring as a complication of tuberculosis of the lungs.'),
  'D014400': NtO(MH='Tuberculosis, Splenic', MS='Infection of the spleen with species of MYCOBACTERIUM.'),
  'D014401': NtO(MH='Tuberculosis, Urogenital', MS='A general term for MYCOBACTERIUM infections of any part of the UROGENITAL SYSTEM in either the male or the female.'),
  'D014402': NtO(MH='Tuberous Sclerosis', MS='Autosomal dominant neurocutaneous syndrome classically characterized by MENTAL RETARDATION; EPILEPSY; and skin lesions (e.g., adenoma sebaceum and hypomelanotic macules). There is, however, considerable heterogeneity in the neurologic manifestations. It is also associated with cortical tuber and HAMARTOMAS formation throughout the body, especially the heart, kidneys, and eyes. Mutations in two loci TSC1 and TSC2 that encode hamartin and tuberin, respectively, are associated with the disease.'),
  'D014403': NtO(MH='Tubocurarine', MS='A neuromuscular blocker and active ingredient in CURARE; plant based alkaloid of Menispermaceae.'),
  'D014404': NtO(MH='Tubulin', MS='A microtubule subunit protein found in large quantities in mammalian brain. It has also been isolated from SPERM FLAGELLUM; CILIA; and other sources. Structurally, the protein is a dimer with a molecular weight of approximately 120,000 and a sedimentation coefficient of 5.8S. It binds to COLCHICINE; VINCRISTINE; and VINBLASTINE.'),
  'D014405': NtO(MH='Tuftsin', MS='N(2)-((1-(N(2)-L-Threonyl)-L-lysyl)-L-prolyl)-L-arginine. A tetrapeptide produced in the spleen by enzymatic cleavage of a leukophilic gamma-globulin. It stimulates the phagocytic activity of blood polymorphonuclear leukocytes and neutrophils in particular. The peptide is located in the Fd fragment of the gamma-globulin molecule.'),
  'D014406': NtO(MH='Tularemia', MS='A plague-like disease of rodents, transmissible to man. It is caused by FRANCISELLA TULARENSIS and is characterized by fever, chills, headache, backache, and weakness.'),
  'D014407': NtO(MH='Tumor Cells, Cultured', MS='Cells grown in vitro from neoplastic tissue. If they can be established as a TUMOR CELL LINE, they can be propagated in cell culture indefinitely.'),
  'D014408': NtO(MH='Biomarkers, Tumor', MS='Molecular products metabolized and secreted by neoplastic tissue and characterized biochemically in cells or BODY FLUIDS. They are indicators of tumor stage and grade as well as useful for monitoring responses to treatment and predicting recurrence. Many chemical groups are represented including HORMONES; ANTIGENS; amino and NUCLEIC ACIDS; ENZYMES; POLYAMINES; and specific CELL MEMBRANE PROTEINS and LIPIDS.'),
  'D014409': NtO(MH='Tumor Necrosis Factor-alpha', MS='Serum glycoprotein produced by activated MACROPHAGES and other mammalian MONONUCLEAR LEUKOCYTES. It has necrotizing activity against tumor cell lines and increases ability to reject tumor transplants. Also known as TNF-alpha, it is only 30% homologous to TNF-beta (LYMPHOTOXIN), but they share TNF RECEPTORS.'),
  'D014410': NtO(MH='Tumor Stem Cell Assay', MS='A cytologic technique for measuring the functional capacity of tumor stem cells by assaying their activity. It is used primarily for the in vitro testing of antineoplastic agents.'),
  'D014411': NtO(MH='Neoplastic Stem Cells', MS='Highly proliferative, self-renewing, and colony-forming stem cells which give rise to NEOPLASMS.'),
  'D014412': NtO(MH='Tumor Virus Infections', MS='Infections produced by oncogenic viruses. The infections caused by DNA viruses are less numerous but more diverse than those caused by the RNA oncogenic viruses.'),
  'D014413': NtO(MH='Tuna', MS='Common name for various species of large, vigorous ocean fishes in the family Scombridae.'),
  'D014414': NtO(MH='Tungsten', MS='A metallic element with the atomic symbol W, atomic number 74, and atomic weight 183.85. It is used in many manufacturing applications, including increasing the hardness, toughness, and tensile strength of steel; manufacture of filaments for incandescent light bulbs; and in contact points for automotive and electrical apparatus.'),
  'D014415': NtO(MH='Tunicamycin', MS='An N-acetylglycosamine containing antiviral antibiotic obtained from Streptomyces lysosuperificus. It is also active against some bacteria and fungi, because it inhibits the glucosylation of proteins. Tunicamycin is used as tool in the study of microbial biosynthetic mechanisms.'),
  'D014416': NtO(MH='Tunisia', MS='A country in northern Africa between ALGERIA and LIBYA. Its capital is Tunis.'),
  'D014417': NtO(MH='Tupaia', MS='A genus of tree shrews of the family TUPAIIDAE which consists of about 12 species. One of the most frequently encountered species is T. glis. Members of this genus inhabit rain forests and secondary growth areas in southeast Asia.'),
  'D014418': NtO(MH='Tupaiidae', MS='The only family of the order SCANDENTIA, variously included in the order Insectivora or in the order Primates, and often in the order Microscelidea, consisting of five genera. They are TUPAIA, Ananthana (Indian tree shrew), Dendrogale (small smooth-tailed tree shrew), Urogale (Mindanao tree shrew), and Ptilocercus (pen-tailed tree shrew). The tree shrews inhabit the forest areas of eastern Asia from India and southwestern China to Borneo and the Philippines.'),
  'D014419': NtO(MH='Turbellaria', MS='A class of free-living freshwater flatworms of North America.'),
  'D014420': NtO(MH='Turbinates', MS='The scroll-like bony plates with curved margins on the lateral wall of the NASAL CAVITY. Turbinates, also called nasal concha, increase the surface area of nasal cavity thus providing a mechanism for rapid warming and humidification of air as it passes to the lung.'),
  'D014421': NtO(MH='Turkey', MS='Country in Southeastern Europe and Southwestern Asia bordering the Black Sea, between Bulgaria and Georgia, and bordering the Aegean Sea and the Mediterranean Sea, between Greece and Syria. The capital is Ankara.'),
  'D014422': NtO(MH='Turkeys', MS='Large woodland game BIRDS in the subfamily Meleagridinae, family Phasianidae, order GALLIFORMES. Formerly they were considered a distinct family, Melegrididae.'),
  'D014423': NtO(MH='Turkmenistan', MS=None),
  'D014424': NtO(MH='Turner Syndrome', MS='A syndrome of defective gonadal development in phenotypic females associated with the karyotype 45,X (or 45,XO). Patients generally are of short stature with undifferentiated GONADS (streak gonads), SEXUAL INFANTILISM, HYPOGONADISM, webbing of the neck, cubitus valgus, elevated GONADOTROPINS, decreased ESTRADIOL level in blood, and CONGENITAL HEART DEFECTS. NOONAN SYNDROME (also called Pseudo-Turner Syndrome and Male Turner Syndrome) resembles this disorder; however, it occurs in males and females with a normal karyotype and is inherited as an autosomal dominant.'),
  'D014425': NtO(MH='Turpentine', MS='The concrete oleoresin obtained from PINACEAE such as longleaf pine (Pinus palustris). It contains a volatile oil, to which its properties are due, and to which form it is generally used. Turpentine is used as a solvent and an experimental irritant in biomedical research.'),
  'D014426': NtO(MH='Turtles', MS='Any reptile including tortoises, fresh water, and marine species of the order Testudines with a body encased in a bony or cartilaginous shell consisting of a top (carapace) and a bottom (plastron) derived from the ribs.'),
  'D014427': NtO(MH='Twins', MS='Two individuals derived from two FETUSES that were fertilized at or about the same time, developed in the UTERUS simultaneously, and born to the same mother. Twins are either monozygotic (TWINS, MONOZYGOTIC) or dizygotic (TWINS, DIZYGOTIC).'),
  'D014428': NtO(MH='Twins, Conjoined', MS='MONOZYGOTIC TWINS who are joined in utero. They may be well developed and share only a superficial connection, often in the frontal, transverse or sagittal body plane, or they may share a partial duplication of a body structure. Alternatively, there may be a small and incompletely developed twin conjoined to a larger, more fully developed twin.'),
  'D014429': NtO(MH='Twins, Dizygotic', MS='Two offspring from the same PREGNANCY. They are from two OVA, fertilized at about the same time by two SPERMATOZOA. Such twins are genetically distinct and can be of different sexes.'),
  'D014430': NtO(MH='Twins, Monozygotic', MS='Two off-spring from the same PREGNANCY. They are from a single fertilized OVUM that split into two EMBRYOS. Such twins are usually genetically identical and of the same sex.'),
  'D014431': NtO(MH='Tylenchoidea', MS='A superfamily of nematodes whose members are free-living saprophytes or parasites of plants. Ova are sometimes found in human feces after ingestion of infected plants.'),
  'D014432': NtO(MH='Tympanic Membrane', MS='An oval semitransparent membrane separating the external EAR CANAL from the tympanic cavity (EAR, MIDDLE). It contains three layers: the skin of the external ear canal; the core of radially and circularly arranged collagen fibers; and the MUCOSA of the middle ear.'),
  'D014433': NtO(MH='Tympanoplasty', MS='Surgical reconstruction of the hearing mechanism of the middle ear, with restoration of the drum membrane to protect the round window from sound pressure, and establishment of ossicular continuity between the tympanic membrane and the oval window. (Dorland, 28th ed.)'),
  'D014434': NtO(MH='Type A Personality', MS='Established behavior pattern characterized by excessive drive and ambition, impatience, competitiveness, sense of time urgency, and poorly contained aggression.'),
  'D014435': NtO(MH='Typhoid Fever', MS='An acute systemic febrile infection caused by SALMONELLA TYPHI, a serotype of SALMONELLA ENTERICA.'),
  'D014436': NtO(MH='Typhoid-Paratyphoid Vaccines', MS='Vaccines used to prevent TYPHOID FEVER and/or PARATYPHOID FEVER which are caused by various species of SALMONELLA. Attenuated, subunit, and inactivated forms of the vaccines exist.'),
  'D014437': NtO(MH='Typhus, Endemic Flea-Borne', MS='An infectious disease clinically similar to epidemic louse-borne typhus (TYPHUS, EPIDEMIC LOUSE-BORNE), but caused by RICKETTSIA TYPHI, which is transmitted from rat to man by the rat flea, XENOPSYLLA CHEOPIS.'),
  'D014438': NtO(MH='Typhus, Epidemic Louse-Borne', MS='The classic form of typhus, caused by RICKETTSIA PROWAZEKII, which is transmitted from man to man by the louse Pediculus humanus corporis. This disease is characterized by the sudden onset of intense headache, malaise, and generalized myalgia followed by the formation of a macular skin eruption and vascular and neurologic disturbances.'),
  'D014439': NtO(MH='Tyramine', MS='An indirect sympathomimetic that occurs naturally in cheese and other foods. Tyramine does not directly activate adrenergic receptors, but it can serve as a substrate for adrenergic uptake systems and  MONOAMINE OXIDASE to prolong the actions of adrenergic transmitters. It also provokes transmitter release from adrenergic terminals and may be a neurotransmitter in some invertebrate nervous systems.'),
  'D014440': NtO(MH='Tyrocidine', MS='An antibiotic mixture produced by Bacillus brevis which may be separated into three components, tyrocidines A, B, and C. It is the major constituent (40-60 per cent) of tyrothricin, gramicidin accounting for the remaining 10-20 per cent active material. It is a topical antimicrobial agent, that is very toxic parenterally.'),
  'D014441': NtO(MH='Tyropanoate', MS='A diagnostic aid as a radiopaque medium in cholecystography.'),
  'D014442': NtO(MH='Monophenol Monooxygenase', MS='An enzyme of the oxidoreductase class that catalyzes the reaction between L-tyrosine, L-dopa, and oxygen to yield L-dopa, dopaquinone, and water. It is a copper protein that acts also on catechols, catalyzing some of the same reactions as CATECHOL OXIDASE. EC 1.14.18.1.'),
  'D014443': NtO(MH='Tyrosine', MS='A non-essential amino acid. In animals it is synthesized from PHENYLALANINE. It is also the precursor of EPINEPHRINE; THYROID HORMONES; and melanin.'),
  'D014444': NtO(MH='Tyrosine Transaminase', MS='An enzyme that catalyzes the conversion of L-TYROSINE and 2-oxoglutarate to 4-hydroxyphenylpyruvate and L-GLUTAMATE. It is a pyridoxal-phosphate protein. L-PHENYLALANINE is hydroxylated to L-tyrosine. The mitochondrial enzyme may be identical with ASPARTATE AMINOTRANSFERASES (EC 2.6.1.1.). Deficiency of this enzyme may cause type II Tyrosinemia (see TYROSINEMIAS). EC 2.6.1.5.'),
  'D014445': NtO(MH='Tyrosine Decarboxylase', MS='A pyridoxal-phosphate protein that catalyzes the conversion of L-tyrosine to tyramine and carbon dioxide. The bacterial enzyme also acts on 3-hydroxytyrosine and, more slowly, on 3-hydroxyphenylalanine. (From Enzyme Nomenclature, 1992) EC 4.1.1.25.'),
  'D014446': NtO(MH='Tyrosine 3-Monooxygenase', MS='An enzyme that catalyzes the conversion of L-tyrosine, tetrahydrobiopterin, and oxygen to 3,4-dihydroxy-L-phenylalanine, dihydrobiopterin, and water. EC 1.14.16.2.'),
  'D014447': NtO(MH='Tyrosine Phenol-Lyase', MS='An enzyme that catalyzes the cleavage of tyrosine to phenol, pyruvate, and ammonia. It is a pyridoxal phosphate protein. The enzyme also forms pyruvate from D-tyrosine, L-cysteine, S-methyl-L-cysteine, L-serine, and D-serine, although at a slower rate. EC 4.1.99.2.'),
  'D014448': NtO(MH='Tyrosine-tRNA Ligase', MS='An enzyme that activates tyrosine with its specific transfer RNA. EC 6.1.1.1.'),
  'D014449': NtO(MH='Tyrothricin', MS='A polypeptide antibiotic mixture obtained from Bacillus brevis. It consists of a mixture of three tyrocidines (60%) and several gramicidins (20%) and is very toxic to blood, liver, kidneys, meninges, and the olfactory apparatus. It is used topically.'),
  'D014450': NtO(MH='Electron Transport Complex III', MS='A multisubunit enzyme complex that contains CYTOCHROME B GROUP; CYTOCHROME C1; and iron-sulfur centers. It catalyzes the oxidation of ubiquinol to UBIQUINONE, and transfers the electrons to CYTOCHROME C. In MITOCHONDRIA the redox reaction is coupled to the transport of PROTONS across the inner mitochondrial membrane.'),
  'D014451': NtO(MH='Ubiquinone', MS='A lipid-soluble benzoquinone which is involved in ELECTRON TRANSPORT in mitochondrial preparations. The compound occurs in the majority of aerobic organisms, from bacteria to higher plants and animals.'),
  'D014452': NtO(MH='Ubiquitins', MS='A family of proteins that are structurally-related to Ubiquitin. Ubiquitins and ubiquitin-like proteins participate in diverse cellular functions, such as protein degradation and HEAT-SHOCK RESPONSE, by conjugation to other proteins.'),
  'D014453': NtO(MH='Glucuronosyltransferase', MS='A family of enzymes accepting a wide range of substrates, including phenols, alcohols, amines, and fatty acids. They function as drug-metabolizing enzymes that catalyze the conjugation of UDPglucuronic acid to a variety of endogenous and exogenous compounds. EC 2.4.1.17.'),
  'D014454': NtO(MH='Uganda', MS='A republic in eastern Africa, south of SUDAN and west of KENYA. Its capital is Kampala.'),
  'D014455': NtO(MH='Ukraine', MS='Country in Europe, bordering the Black Sea, between Poland, Romania, and Moldova in the west and Russia in the east. The capital is Kyiv.'),
  'D014456': NtO(MH='Ulcer', MS='A lesion on the surface of the skin or a mucous surface, produced by the sloughing of inflammatory necrotic tissue.'),
  'D014457': NtO(MH='Ulna', MS='The inner and longer bone of the FOREARM.'),
  'D014458': NtO(MH='Ulna Fractures', MS='Fractures of the larger bone of the forearm.'),
  'D014459': NtO(MH='Ulnar Nerve', MS='A major nerve of the upper extremity. In humans, the fibers of the ulnar nerve originate in the lower cervical and upper thoracic spinal cord (usually C7 to T1), travel via the medial cord of the brachial plexus, and supply sensory and motor innervation to parts of the hand and forearm.'),
  'D014460': NtO(MH='Ultimobranchial Body', MS='A diverticulum from the fourth pharyngeal pouch of an embryo, regarded by some as a rudimentary fifth pharyngeal pouch and by others as a lateral thyroid primordium. The ultimobranchial bodies of lower vertebrates contain large amounts of calcitonin. In mammals the bodies fuse with the thyroid gland and are thought to develop into the parafollicular cells. (Stedman, 25th ed)'),
  'D014461': NtO(MH='Ultracentrifugation', MS='Centrifugation with a centrifuge that develops centrifugal fields of more than 100,000 times gravity. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D014462': NtO(MH='Ultrafiltration', MS='The separation of particles from a suspension by passage through a filter with very fine pores. In ultrafiltration the separation is accomplished by convective transport; in DIALYSIS separation relies instead upon differential diffusion. Ultrafiltration occurs naturally and is a laboratory procedure. Artificial ultrafiltration of the blood is referred to as HEMOFILTRATION or HEMODIAFILTRATION (if combined with HEMODIALYSIS).'),
  'D014463': NtO(MH='Ultrasonography', MS='The visualization of deep structures of the body by recording the reflections or echoes of ultrasonic pulses directed into the tissues. Use of ultrasound for imaging or diagnostic purposes employs frequencies ranging from 1.6 to 10 megahertz.'),
  'D014464': NtO(MH='Ultrasonic Therapy', MS='The use of focused, high-frequency sound waves to produce local hyperthermia in certain diseased or injured parts of the body or to destroy the diseased tissue.'),
  'D014465': NtO(MH='Ultrasonics', MS='A subfield of acoustics dealing in the radio frequency range higher than acoustic SOUND waves (approximately above 20 kilohertz). Ultrasonic radiation is used therapeutically (DIATHERMY and ULTRASONIC THERAPY) to generate HEAT and to selectively destroy tissues. It is also used in diagnostics, for example, ULTRASONOGRAPHY; ECHOENCEPHALOGRAPHY; and ECHOCARDIOGRAPHY, to visually display echoes received from irradiated tissues.'),
  'D014466': NtO(MH='Ultraviolet Rays', MS='That portion of the electromagnetic spectrum immediately below the visible range and extending into the x-ray frequencies. The longer wavelengths (near-UV or biotic or vital rays) are necessary for the endogenous synthesis of vitamin D and are also called antirachitic rays; the shorter, ionizing wavelengths (far-UV or abiotic or extravital rays) are viricidal, bactericidal, mutagenic, and carcinogenic and are used as disinfectants.'),
  'D014467': NtO(MH='Ultraviolet Therapy', MS="The use of ultraviolet electromagnetic radiation in the treatment of disease, usually of the skin. This is the part of the sun's spectrum that causes sunburn and tanning. Ultraviolet A, used in PUVA, is closer to visible light and less damaging than Ultraviolet B, which is ionizing."),
  'D014468': NtO(MH='Umbelliferones', MS='7-Hydroxycoumarins. Substances present in many plants, especially umbelliferae. Umbelliferones are used in sunscreen preparations and may be mutagenic. Their derivatives are used in liver therapy, as reagents, plant growth factors, sunscreens, insecticides, parasiticides, choleretics, spasmolytics, etc.'),
  'D014469': NtO(MH='Umbilical Arteries', MS='Specialized arterial vessels in the umbilical cord. They carry waste and deoxygenated blood from the FETUS to the mother via the PLACENTA. In humans, there are usually two umbilical arteries but sometimes one.'),
  'D014470': NtO(MH='Umbilical Cord', MS='The flexible rope-like structure that connects a developing FETUS to the PLACENTA in mammals. The cord contains blood vessels which carry oxygen and nutrients from the mother to the fetus and waste products away from the fetus.'),
  'D014471': NtO(MH='Umbilical Veins', MS='Venous vessels in the umbilical cord. They carry oxygenated, nutrient-rich blood from the mother to the FETUS via the PLACENTA. In humans, there is normally one umbilical vein.'),
  'D014472': NtO(MH='Umbilicus', MS='The pit in the center of the ABDOMINAL WALL marking the point where the UMBILICAL CORD entered in the FETUS.'),
  'D014473': NtO(MH='Unconscious, Psychology', MS='Those forces and content of the mind which are not ordinarily available to conscious awareness or to immediate recall.'),
  'D014474': NtO(MH='Unconsciousness', MS='Loss of the ability to maintain awareness of self and environment combined with markedly reduced responsiveness to environmental stimuli. (From Adams et al., Principles of Neurology, 6th ed, pp344-5)'),
  'D014475': NtO(MH='Uncoupling Agents', MS='Chemical agents that uncouple oxidation from phosphorylation in the metabolic cycle so that ATP synthesis does not occur. Included here are those IONOPHORES that disrupt electron transfer by short-circuiting the proton gradient across mitochondrial membranes.'),
  'D014476': NtO(MH='Undecylenic Acids', MS='Salts and derivatives of undecylenic acid.'),
  'D014477': NtO(MH='Underachievement', MS='Performance, usually in school work, poorer than that predicted from aptitude and/or intelligence testing.'),
  'D014478': NtO(MH='Unemployment', MS='The state of not being engaged in a gainful occupation.'),
  'D014479': NtO(MH='United Arab Emirates', MS="A federation of seven states on the southeast portion of the Arabian peninsula: Abu Dhabi, Ajman, Dubai, Fujairah, Ras al-Khaimah, Sharjah and Umm al-Qaiwain. In 1820 a treaty of peace was concluded between Great Britain and native rulers. During the 19th century the rulers agreed to suppression of the slave trade and restriction of foreign relations to Great Britain. The Trucial Council was established in 1952 and defense treaties with Great Britain terminated. In 1971 an independent six-member federation was formed, with Ras al-Khaimah joining the federation in 1972. (From Webster's New Geographical Dictionary, 1988, p1250)"),
  'D014480': NtO(MH='United Nations', MS='An international organization whose members include most of the sovereign nations of the world with headquarters in New York City. The primary objectives of the organization are to maintain peace and security and to achieve international cooperation in solving international economic, social, cultural, or humanitarian problems.'),
  'D014481': NtO(MH='United States', MS='A country in NORTH AMERICA between CANADA and MEXICO.'),
  'D014482': NtO(MH='United States Substance Abuse and Mental Health Services Administration', MS='An agency of the PUBLIC HEALTH SERVICE concerned with the overall planning, promoting, and administering of programs pertaining to substance abuse and mental health. It is commonly referred to by the acronym SAMHSA. On 1 October 1992, the United States Alcohol, Drug Abuse, and Mental Health Administration (ADAMHA) became SAMHSA.'),
  'D014483': NtO(MH='United States Dept. of Health and Human Services', MS='A cabinet department in the Executive Branch of the United States Government concerned with administering those agencies and offices having programs pertaining to health and human services.'),
  'D014484': NtO(MH='United States Environmental Protection Agency', MS='An agency in the Executive Branch of the Federal Government. It was created as an independent regulatory agency responsible for the implementation of federal laws designed to protect the environment. Its mission is to protect human health and the ENVIRONMENT.'),
  'D014485': NtO(MH='United States Federal Trade Commission', MS='An independent administrative agency concerned with maintaining competitive free enterprise by prohibiting unfair methods of competition and unfair deceptive acts or practices.'),
  'D014486': NtO(MH='United States Food and Drug Administration', MS='An agency of the PUBLIC HEALTH SERVICE concerned with the overall planning, promoting, and administering of programs pertaining to maintaining standards of quality of foods, drugs, therapeutic devices, etc.'),
  'D014487': NtO(MH='Centers for Medicare and Medicaid Services, U.S.', MS='A component of the Department of Health and Human Services to oversee and direct the Medicare and Medicaid programs and related Federal medical care quality control staffs. Name was changed effective June 14, 2001.'),
  'D014488': NtO(MH='United States Health Resources and Services Administration', MS='A component of the PUBLIC HEALTH SERVICE that provides leadership related to the delivery of health services and the requirements for and distribution of health resources, including manpower training.'),
  'D014489': NtO(MH='United States Occupational Safety and Health Administration', MS='An office in the Department of Labor responsible for developing and establishing occupational safety and health standards.'),
  'D014490': NtO(MH='United States Office of Economic Opportunity', MS='A division of the Executive Branch of the United States government concerned with overall planning, promoting, and administering programs relative to the provision of opportunities for economic advancement.'),
  'D014491': NtO(MH='United States Office of Technology Assessment', MS='An office established to help Congress participate and plan for the consequences of uses of technology. It provided information on both the beneficial and adverse effects of technological applications. The Office of Technology Assessment closed on September 29, 1995.'),
  'D014492': NtO(MH='United States Public Health Service', MS='A constituent organization of the DEPARTMENT OF HEALTH AND HUMAN SERVICES concerned with protecting and improving the health of the nation.'),
  'D014493': NtO(MH='United States Department of Veterans Affairs', MS='A cabinet department in the Executive Branch of the United States Government concerned with overall planning, promoting, and administering programs pertaining to VETERANS. It was established March 15, 1989 as a Cabinet-level position.'),
  'D014494': NtO(MH='Unithiol', MS='A chelating agent used as an antidote to heavy metal poisoning.'),
  'D014495': NtO(MH='Universities', MS='Educational institutions providing facilities for teaching and research and authorized to grant academic degrees.'),
  'D014496': NtO(MH='Urachal Cyst', MS='Cyst occurring in a persistent portion of the urachus, presenting as an extraperitoneal mass in the umbilical region. It is characterized by abdominal pain, and fever if infected. It may rupture, leading to peritonitis, or it may drain through the umbilicus.'),
  'D014497': NtO(MH='Urachus', MS='An embryonic structure originating from the ALLANTOIS. It is a canal connecting the fetal URINARY BLADDER and the UMBILICUS. It is normally converted into a fibrous cord postnatally. When the canal fails to be filled and remains open (patent urachus), urine leaks through the umbilicus.'),
  'D014498': NtO(MH='Uracil', MS='One of four nucleotide bases in the nucleic acid RNA.'),
  'D014499': NtO(MH='Uracil Mustard', MS='Nitrogen mustard derivative of URACIL. It is a alkylating antineoplastic agent that is used in lymphatic malignancies, and causes mainly gastrointestinal and bone marrow damage.'),
  'D014500': NtO(MH='Uracil Nucleotides', MS='A group of pyrimidine NUCLEOTIDES which contain URACIL.'),
  'D014501': NtO(MH='Uranium', MS='A radioactive element of the actinide series of metals. It has an atomic symbol U, atomic number 92, and atomic weight 238.03. U-235 is used as the fissionable fuel in nuclear weapons and as fuel in nuclear power reactors.'),
  'D014502': NtO(MH='Uranyl Nitrate', MS='Bis(nitrato-O)dioxouranium. A compound used in photography and the porcelain industry. It causes severe renal insufficiency and renal tubular necrosis in mammals and is an effective lymphocyte mitogen.'),
  'D014503': NtO(MH='Urate Oxidase', MS='An enzyme that catalyzes the conversion of urate and unidentified products. It is a copper protein. The initial products decompose to form allantoin. EC 1.7.3.3.'),
  'D014504': NtO(MH='Urban Health', MS='The status of health in urban populations.'),
  'D014505': NtO(MH='Urban Population', MS='The inhabitants of a city or town, including metropolitan areas and suburban areas.'),
  'D014506': NtO(MH='Urban Renewal', MS='The planned upgrading of a deteriorating urban area, involving rebuilding, renovation, or restoration. It frequently refers to programs of major demolition and rebuilding of blighted areas.'),
  'D014507': NtO(MH='Urbanization', MS='The process whereby a society changes from a rural to an urban way of life. It refers also to the gradual increase in the proportion of people living in urban areas.'),
  'D014508': NtO(MH='Urea', MS='A compound formed in the liver from ammonia produced by the deamination of amino acids. It is the principal end product of protein catabolism and constitutes about one half of the total urinary solids.'),
  'D014509': NtO(MH='Ureaplasma', MS='A genus of gram-negative, nonmotile bacteria which are common parasitic inhabitants of the urogenital tracts of humans, cattle, dogs, and monkeys.'),
  'D014510': NtO(MH='Urease', MS='An enzyme that catalyzes the conversion of urea and water to carbon dioxide and ammonia. EC 3.5.1.5.'),
  'D014511': NtO(MH='Uremia', MS='A clinical syndrome associated with the retention of renal waste products or uremic toxins in the blood. It is usually the result of RENAL INSUFFICIENCY. Most uremic toxins are end products of protein or nitrogen CATABOLISM, such as UREA or CREATININE. Severe uremia can lead to multiple organ dysfunctions with a constellation of symptoms.'),
  'D014512': NtO(MH='Ureohydrolases', MS=None),
  'D014513': NtO(MH='Ureter', MS='One of a pair of thick-walled tubes that transports urine from the KIDNEY PELVIS to the URINARY BLADDER.'),
  'D014514': NtO(MH='Ureteral Calculi', MS='Stones in the URETER that are formed in the KIDNEY. They are rarely more than 5 mm in diameter for larger renal stones cannot enter ureters. They are often lodged at the ureteral narrowing and can cause excruciating renal colic.'),
  'D014515': NtO(MH='Ureteral Diseases', MS='Pathological processes involving the URETERS.'),
  'D014516': NtO(MH='Ureteral Neoplasms', MS='Cancer or tumors of the URETER which may cause obstruction leading to hydroureter, HYDRONEPHROSIS, and PYELONEPHRITIS. HEMATURIA is a common symptom.'),
  'D014517': NtO(MH='Ureteral Obstruction', MS='Blockage in any part of the URETER causing obstruction of urine flow from the kidney to the URINARY BLADDER. The obstruction may be congenital, acquired, unilateral, bilateral, complete, partial, acute, or chronic. Depending on the degree and duration of the obstruction, clinical features vary greatly such as HYDRONEPHROSIS and obstructive nephropathy.'),
  'D014518': NtO(MH='Ureterocele', MS='A cystic dilatation of the end of a URETER as it enters into the URINARY BLADDER. It is characterized by the ballooning of the ureteral orifice into the lumen of the bladder and may obstruct urine flow.'),
  'D014519': NtO(MH='Ureterostomy', MS='Surgical formation of an opening in the ureter for external drainage of the urine; cutaneous route utilizes a ureteral orifice emerging through the skin.'),
  'D014520': NtO(MH='Urethane', MS='Antineoplastic agent that is also used as a veterinary anesthetic. It has also been used as an intermediate in organic synthesis. Urethane is suspected to be a carcinogen.'),
  'D014521': NtO(MH='Urethra', MS='A tube that transports URINE from the URINARY BLADDER to the outside of the body in both the sexes. It also has a reproductive function in the male by providing a passage for SPERM.'),
  'D014522': NtO(MH='Urethral Diseases', MS='Pathological processes involving the URETHRA.'),
  'D014523': NtO(MH='Urethral Neoplasms', MS='Cancer or tumors of the URETHRA. Benign epithelial tumors of the urethra usually consist of squamous and transitional cells. Primary urethral carcinomas are rare and typically of squamous cells. Urethral carcinoma is the only urological malignancy that is more common in females than in males.'),
  'D014524': NtO(MH='Urethral Obstruction', MS='Partial or complete blockage in any part of the URETHRA that can lead to difficulty or inability to empty the URINARY BLADDER. It is characterized by an enlarged, often damaged, bladder with frequent urges to void.'),
  'D014525': NtO(MH='Urethral Stricture', MS='Narrowing of any part of the URETHRA. It is characterized by decreased urinary stream and often other obstructive voiding symptoms.'),
  'D014526': NtO(MH='Urethritis', MS='Inflammation involving the URETHRA. Similar to CYSTITIS, clinical symptoms range from vague discomfort to painful urination (DYSURIA), urethral discharge, or both.'),
  'D014527': NtO(MH='Uric Acid', MS='An oxidation product, via XANTHINE OXIDASE, of oxypurines such as XANTHINE and HYPOXANTHINE. It is the final oxidation product of purine catabolism in humans and primates, whereas in most other mammals URATE OXIDASE further oxidizes it to ALLANTOIN.'),
  'D014528': NtO(MH='Uricosuric Agents', MS='Gout suppressants that act directly on the renal tubule to increase the excretion of uric acid, thus reducing its concentrations in plasma.'),
  'D014529': NtO(MH='Uridine', MS='A ribonucleoside in which RIBOSE is linked to URACIL.'),
  'D014530': NtO(MH='Uridine Diphosphate', MS='A uracil nucleotide containing a pyrophosphate group esterified to C5 of the sugar moiety.'),
  'D014531': NtO(MH='Uridine Diphosphate Galactose', MS='A nucleoside diphosphate sugar which can be epimerized into UDPglucose for entry into the mainstream of carbohydrate metabolism. Serves as a source of galactose in the synthesis of lipopolysaccharides, cerebrosides, and lactose.'),
  'D014532': NtO(MH='Uridine Diphosphate Glucose', MS='A key intermediate in carbohydrate metabolism. Serves as a precursor of glycogen, can be metabolized into UDPgalactose and UDPglucuronic acid which can then be incorporated into polysaccharides as galactose and glucuronic acid. Also serves as a precursor of sucrose lipopolysaccharides, and glycosphingolipids.'),
  'D014533': NtO(MH='Uridine Diphosphate Glucose Dehydrogenase', MS='An enzyme that catalyzes the oxidation of UDPglucose to UDPglucuronate in the presence of NAD+. EC 1.1.1.22.'),
  'D014534': NtO(MH='UDPglucose 4-Epimerase', MS='A necessary enzyme in the metabolism of galactose. It reversibly catalyzes the conversion of UDPglucose to UDPgalactose. NAD+ is an essential component for enzymatic activity. EC 5.1.3.2.'),
  'D014535': NtO(MH='Uridine Diphosphate Glucuronic Acid', MS='A nucleoside diphosphate sugar which serves as a source of glucuronic acid for polysaccharide biosynthesis. It may also be epimerized to UDP iduronic acid, which donates iduronic acid to polysaccharides. In animals, UDP glucuronic acid is used for formation of many glucosiduronides with various aglycones.'),
  'D014536': NtO(MH='Uridine Diphosphate N-Acetylgalactosamine', MS='A nucleoside diphosphate sugar which serves as a source of N-acetylgalactosamine for glycoproteins, sulfatides and cerebrosides.'),
  'D014537': NtO(MH='Uridine Diphosphate N-Acetylglucosamine', MS='Serves as the biological precursor of insect chitin, of muramic acid in bacterial cell walls, and of sialic acids in mammalian glycoproteins.'),
  'D014538': NtO(MH='Uridine Diphosphate N-Acetylmuramic Acid', MS='A nucleoside diphosphate sugar which is formed from UDP-N-acetylglucosamine and phosphoenolpyruvate. It serves as the building block upon which peptidoglycan is formed.'),
  'D014539': NtO(MH='Uridine Diphosphate Sugars', MS='Nucleotide-sugars such as uridine-diphosphate glucose or UDP-glucose.'),
  'D014540': NtO(MH='Uridine Diphosphate Xylose', MS='The decarboxylation product of UDPglucuronic acid, which is used for formation of the xylosides of seryl hydroxyl groups in mucoprotein synthesis. Also forms plant xylans.'),
  'D014541': NtO(MH='Uridine Kinase', MS="An enzyme that catalyzes the phosphorylation of uridine and cytidine to uridine 5'-phosphate and cytidine 5'-phosphate, respectively. ATP, dUTP, dGTP, and dATP are effective phosphate donors. EC 2.7.1.48."),
  'D014542': NtO(MH='Uridine Monophosphate', MS="5'-Uridylic acid. A uracil nucleotide containing one phosphate group esterified to the sugar moiety in the 2', 3' or 5' position."),
  'D014543': NtO(MH='Uridine Phosphorylase', MS='An enzyme that catalyzes the transfer of ribose from uridine to orthophosphate, forming uracil and ribose 1-phosphate.'),
  'D014544': NtO(MH='Uridine Triphosphate', MS="Uridine 5'-(tetrahydrogen triphosphate). A uracil nucleotide containing three phosphate groups esterified to the sugar moiety."),
  'D014545': NtO(MH='Urinary Calculi', MS='Low-density crystals or stones in any part of the URINARY TRACT. Their chemical compositions often include CALCIUM OXALATE, magnesium ammonium phosphate (struvite), CYSTINE, or URIC ACID.'),
  'D014546': NtO(MH='Urinary Catheterization', MS='Passage of a CATHETER into the URINARY BLADDER or kidney.'),
  'D014547': NtO(MH='Urinary Diversion', MS="Temporary or permanent diversion of the flow of urine through the ureter away from the URINARY BLADDER in the presence of a bladder disease or after cystectomy. There is a variety of techniques: direct anastomosis of ureter and bowel, cutaneous ureterostomy, ileal, jejunal or colon conduit, ureterosigmoidostomy, etc. (From Campbell's Urology, 6th ed, p2654)"),
  'D014548': NtO(MH='Urinary Fistula', MS='An abnormal passage in any part of the URINARY TRACT between itself or with other organs.'),
  'D014549': NtO(MH='Urinary Incontinence', MS='Involuntary loss of URINE, such as leaking of urine. It is a symptom of various underlying pathological processes. Major types of incontinence include URINARY URGE INCONTINENCE and URINARY STRESS INCONTINENCE.'),
  'D014550': NtO(MH='Urinary Incontinence, Stress', MS='Involuntary discharge of URINE as a result of physical activities that increase abdominal pressure on the URINARY BLADDER without detrusor contraction or overdistended bladder. The subtypes are classified by the degree of leakage, descent and opening of the bladder neck and URETHRA without bladder contraction, and sphincter deficiency.'),
  'D014551': NtO(MH='Urinary Tract', MS='The duct which coveys URINE from the pelvis of the KIDNEY through the URETERS, BLADDER, and URETHRA.'),
  'D014552': NtO(MH='Urinary Tract Infections', MS='Inflammatory responses of the epithelium of the URINARY TRACT to microbial invasions. They are often bacterial infections with associated BACTERIURIA and PYURIA.'),
  'D014553': NtO(MH='Urinary Tract Physiological Phenomena', MS='Properties, functions, and processes of the URINARY TRACT as a whole or of any of its parts.'),
  'D014554': NtO(MH='Urination', MS='Discharge of URINE, liquid waste processed by the KIDNEY, from the body.'),
  'D014555': NtO(MH='Urination Disorders', MS='Abnormalities in the process of URINE voiding, including bladder control, frequency of URINATION, as well as the volume and composition of URINE.'),
  'D014556': NtO(MH='Urine', MS='Liquid by-product of excretion produced in the kidneys, temporarily stored in the bladder until discharge through the URETHRA.'),
  'D014557': NtO(MH='Urobilin', MS=None),
  'D014558': NtO(MH='Urobilinogen', MS='A colorless compound formed in the intestines by the reduction of bilirubin. Some is excreted in the feces where it is oxidized to urobilin. Some is reabsorbed and re-excreted in the bile as bilirubin. At times, it is re-excreted in the urine, where it may be later oxidized to urobilin.'),
  'D014559': NtO(MH='Urocanate Hydratase', MS='An enzyme that catalyzes the conversion of 4,5-dihydro-4-oxo-5-imidazolepropanoate to urocanate and water. EC 4.2.1.49.'),
  'D014560': NtO(MH='Urocanic Acid', MS='4-Imidazoleacrylic acid.'),
  'D014561': NtO(MH='Urochordata', MS='A subphylum of chordates intermediate between the invertebrates and the true vertebrates. It includes the Ascidians.'),
  'D014562': NtO(MH='Urodela', MS='An order of the Amphibia class which includes salamanders and newts. They are characterized by usually having slim bodies and tails, four limbs of about equal size (except in Sirenidae), and a reduction in skull bones.'),
  'D014563': NtO(MH='Urodynamics', MS='The mechanical laws of fluid dynamics as they apply to urine transport.'),
  'D014564': NtO(MH='Urogenital Abnormalities', MS='Congenital structural abnormalities of the UROGENITAL SYSTEM in either the male or the female.'),
  'D014565': NtO(MH='Urogenital Neoplasms', MS='Tumors or cancer of the UROGENITAL SYSTEM in either the male or the female.'),
  'D014566': NtO(MH='Urogenital System', MS='All the organs involved in reproduction and the formation and release of URINE. It includes the kidneys, ureters, BLADDER; URETHRA, and the organs of reproduction - ovaries, UTERUS; FALLOPIAN TUBES; VAGINA; and CLITORIS in women and the testes; SEMINAL VESICLES; PROSTATE; seminal ducts; and PENIS in men.'),
  'D014567': NtO(MH='Urography', MS='Radiography of any part of the URINARY TRACT.'),
  'D014568': NtO(MH='Urokinase-Type Plasminogen Activator', MS='A proteolytic enzyme that converts PLASMINOGEN to FIBRINOLYSIN where the preferential cleavage is between ARGININE and VALINE. It was isolated originally from human URINE, but is found in most tissues of most VERTEBRATES.'),
  'D014570': NtO(MH='Urologic Diseases', MS='Pathological processes of the URINARY TRACT in both males and females.'),
  'D014571': NtO(MH='Urologic Neoplasms', MS='Tumors or cancer of the URINARY TRACT in either the male or the female.'),
  'D014572': NtO(MH='Urology', MS='A surgical specialty concerned with the study, diagnosis, and treatment of diseases of the urinary tract in both sexes, and the genital tract in the male. Common urological problems include urinary obstruction, URINARY INCONTINENCE, infections, and UROGENITAL NEOPLASMS.'),
  'D014573': NtO(MH='Urology Department, Hospital', MS='Hospital department responsible for the administration and provision of diagnostic and therapeutic services for the urologic patient.'),
  'D014574': NtO(MH='Uronic Acids', MS='Acids derived from monosaccharides by the oxidation of the terminal (-CH2OH) group farthest removed from the carbonyl group to a (-COOH) group. (From Stedmans, 26th ed)'),
  'D014575': NtO(MH='Uroporphyrinogen Decarboxylase', MS='An enzyme that catalyzes the decarboxylation of UROPORPHYRINOGEN III to coproporphyrinogen III by the conversion of four acetate groups to four methyl groups. It is the fifth enzyme in the 8-enzyme biosynthetic pathway of HEME. Several forms of cutaneous PORPHYRIAS are results of this enzyme deficiency as in PORPHYRIA CUTANEA TARDA; and HEPATOERYTHROPOIETIC PORPHYRIA.'),
  'D014576': NtO(MH='Uroporphyrinogen III Synthetase', MS='An enzyme that catalyzes the cyclization of hydroxymethylbilane to yield UROPORPHYRINOGEN III and water. It is the fourth enzyme in the 8-enzyme biosynthetic pathway of HEME, and is encoded by UROS gene. Mutations of UROS gene result in CONGENITAL ERYTHROPOIETIC PORPHYRIA.'),
  'D014577': NtO(MH='Uroporphyrinogens', MS='Porphyrinogens which are intermediates in heme biosynthesis. They have four acetic acid and four propionic acid side chains attached to the pyrrole rings. Uroporphyrinogen I and III are formed from polypyrryl methane in the presence of uroporphyrinogen III cosynthetase and uroporphyrin I synthetase, respectively. They can yield uroporphyrins by autooxidation or coproporphyrinogens by decarboxylation.'),
  'D014578': NtO(MH='Uroporphyrins', MS='Porphyrins with four acetic acid and four propionic acid side chains attached to the pyrrole rings.'),
  'D014579': NtO(MH='Urotensins', MS='Teleost hormones. A family of small peptides isolated from urophyses of bony fishes. They have many different physiological effects, including long-lasting hypotensive activity and have been proposed as antihypertensives. There are at least four different compounds: urotensin I, urotensin II, urotensin III, and urotensin IV.'),
  'D014580': NtO(MH='Ursodeoxycholic Acid', MS='An epimer of chenodeoxycholic acid. It is a mammalian bile acid found first in the bear and is apparently either a precursor or a product of chenodeoxycholate. Its administration changes the composition of bile and may dissolve gallstones. It is used as a cholagogue and choleretic.'),
  'D014581': NtO(MH='Urticaria', MS='A vascular reaction of the skin characterized by erythema and wheal formation due to localized increase of vascular permeability. The causative mechanism may be allergy, infection, or stress.'),
  'D014582': NtO(MH='Urticaria Pigmentosa', MS='The most common form of cutaneous mastocytosis (MASTOCYTOSIS, CUTANEOUS) that occurs primarily in children. It is characterized by the multiple small reddish-brown pigmented pruritic macules and papules.'),
  'D014583': NtO(MH='Uruguay', MS='A country in southern South America, bordering the South Atlantic Ocean, located between Argentina and Brazil.'),
  'D014584': NtO(MH='User-Computer Interface', MS='The portion of an interactive computer program that issues messages to and receives commands from a user.'),
  'D014586': NtO(MH='USSR', MS='A country located from the Baltic and Black seas to the Pacific Ocean and, for a time including 15 Soviet Socialist Republics. The capital was Moscow.'),
  'D014587': NtO(MH='Ustilaginales', MS='An order of basidiomycetous fungi; some species are parasitic on grasses (POACEAE) and maize.'),
  'D014588': NtO(MH='Ustilago', MS='A genus of basidiomycetous smut fungi comprising the loose smuts.'),
  'D014589': NtO(MH='Utah', MS='State bounded on the north by Idaho and Wyoming, on the east by Wyoming and Colorado, on the south by Arizona, and on the west by Nevada.'),
  'D014590': NtO(MH='Uterine Contraction', MS='Contraction of the UTERINE MUSCLE.'),
  'D014591': NtO(MH='Uterine Diseases', MS='Pathological processes involving any part of the UTERUS.'),
  'D014592': NtO(MH='Uterine Hemorrhage', MS='Bleeding from blood vessels in the UTERUS, sometimes manifested as vaginal bleeding.'),
  'D014593': NtO(MH='Uterine Inertia', MS='Failure of the UTERUS to contract with normal strength, duration, and intervals during childbirth (LABOR, OBSTETRIC). It is also called uterine atony.'),
  'D014594': NtO(MH='Uterine Neoplasms', MS='Tumors or cancer of the UTERUS.'),
  'D014595': NtO(MH='Uterine Perforation', MS='A hole or break through the wall of the UTERUS, usually made by the placement of an instrument or INTRAUTERINE DEVICES.'),
  'D014596': NtO(MH='Uterine Prolapse', MS='Downward displacement of the UTERUS. It is classified in various degrees: in the first degree the UTERINE CERVIX is within the vaginal orifice; in the second degree the cervix is outside the orifice; in the third degree the entire uterus is outside the orifice.'),
  'D014597': NtO(MH='Uterine Rupture', MS='A complete separation or tear in the wall of the UTERUS with or without expulsion of the FETUS. It may be due to injuries, multiple pregnancies, large fetus, previous scarring, or obstruction.'),
  'D014598': NtO(MH='Uteroglobin', MS='A steroid-inducible protein that was originally identified in uterine fluid. It is a secreted homodimeric protein with identical 70-amino acid subunits that are joined in an antiparallel orientation by two disulfide bridges. A variety of activities are associated with uteroglobin including the sequestering of hydrophobic ligands and the inhibition of SECRETORY PHOSPHOLIPASE A2.'),
  'D014599': NtO(MH='Uterus', MS='The hollow thick-walled muscular organ in the female PELVIS. It consists of the fundus which is the site of EMBRYO IMPLANTATION and FETAL DEVELOPMENT. Beyond the isthmus at the perineal end of fundus, is CERVIX UTERI (the neck) opening into VAGINA. Beyond the isthmi at the upper abdominal end of fundus, are the FALLOPIAN TUBES.'),
  'D014600': NtO(MH='Utilization Review', MS='An organized procedure carried out through committees to review admissions, duration of stay, professional services furnished, and to evaluate the medical necessity of those services and promote their most efficient use.'),
  'D014601': NtO(MH='Utopias', MS='Ideal perfection especially in laws, government, and social conditions.'),
  'D014602': NtO(MH='Uvea', MS='The pigmented vascular coat of the eyeball, consisting of the CHOROID; CILIARY BODY; and IRIS, which are continuous with each other. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D014603': NtO(MH='Uveal Diseases', MS='Diseases of the uvea.'),
  'D014604': NtO(MH='Uveal Neoplasms', MS='Tumors or cancer of the UVEA.'),
  'D014605': NtO(MH='Uveitis', MS='Inflammation of part or all of the uvea, the middle (vascular) tunic of the eye, and commonly involving the other tunics (sclera and cornea, and the retina). (Dorland, 27th ed)'),
  'D014606': NtO(MH='Uveitis, Anterior', MS='Inflammation of the anterior uvea comprising the iris, angle structures, and the ciliary body. Manifestations of this disorder include ciliary injection, exudation into the anterior chamber, iris changes, and adhesions between the iris and lens (posterior synechiae). Intraocular pressure may be increased or reduced.'),
  'D014607': NtO(MH='Uveomeningoencephalitic Syndrome', MS='A syndrome characterized by bilateral granulomatous UVEITIS with IRITIS and secondary GLAUCOMA, premature ALOPECIA, symmetrical VITILIGO, poliosis circumscripta (a strand of depigmented hair), HEARING DISORDERS, and meningeal signs (neck stiffness and headache). Examination of the cerebrospinal fluid reveals a pattern consistent with MENINGITIS, ASEPTIC. (Adams et al., Principles of Neurology, 6th ed, p748; Surv Ophthalmol 1995 Jan;39(4):265-292)'),
  'D014608': NtO(MH='Uveoparotid Fever', MS='A manifestation of sarcoidosis marked by chronic inflammation of the parotid gland and the uvea.'),
  'D014609': NtO(MH='Uvula', MS='A fleshy extension at the back of the soft palate that hangs above the opening of the throat.'),
  'D014610': NtO(MH='Uzbekistan', MS='A country in Central Asia, north of Turkmenistan, south of Kazakhstan. The capital is Tashkent.'),
  'D014611': NtO(MH='Vaccination', MS="Administration of vaccines to stimulate the host's immune response. This includes any preparation intended for active immunological prophylaxis."),
  'D014612': NtO(MH='Vaccines', MS='Suspensions of killed or attenuated microorganisms (bacteria, viruses, fungi, protozoa), antigenic proteins,  synthetic constructs, or other bio-molecular derivatives, administered for the prevention, amelioration, or treatment of infectious and other diseases.'),
  'D014613': NtO(MH='Vaccines, Attenuated', MS='Live vaccines prepared from microorganisms which have undergone physical adaptation (e.g., by radiation or temperature conditioning) or serial passage in laboratory animal hosts or infected tissue/cell cultures, in order to produce avirulent mutant strains capable of inducing protective immunity.'),
  'D014614': NtO(MH='Vaccines, Synthetic', MS='Small synthetic peptides that mimic surface antigens of pathogens and are immunogenic, or vaccines manufactured with the aid of recombinant DNA techniques. The latter vaccines may also be whole viruses whose nucleic acids have been modified.'),
  'D014615': NtO(MH='Vaccinia', MS='The cutaneous and occasional systemic reactions associated with vaccination using smallpox (variola) vaccine.'),
  'D014616': NtO(MH='Vaccinia virus', MS='The type species of ORTHOPOXVIRUS, related to COWPOX VIRUS, but whose true origin is unknown. It has been used as a live vaccine against SMALLPOX. It is also used as a vector for inserting foreign DNA into animals. Rabbitpox virus is a subspecies of VACCINIA VIRUS.'),
  'D014617': NtO(MH='Vacuoles', MS='Any spaces or cavities within a cell. They may function in digestion, storage, secretion, or excretion.'),
  'D014618': NtO(MH='Vacuum', MS='A space in which the pressure is far below atmospheric pressure so that the remaining gases do not affect processes being carried on in the space.'),
  'D014619': NtO(MH='Vacuum Curettage', MS='Aspiration of the contents of the uterus with a vacuum curette.'),
  'D014620': NtO(MH='Vacuum Extraction, Obstetrical', MS="Removal of the fetus from the uterus or vagina at or near the end of pregnancy with a metal traction cup that is attached to the fetus' head. Negative pressure is applied and traction is made on a chain passed through the suction tube. (From Stedman, 26th ed & Dorland, 28th ed)"),
  'D014621': NtO(MH='Vagina', MS='The genital canal in the female, extending from the UTERUS to the VULVA. (Stedman, 25th ed)'),
  'D014622': NtO(MH='Vaginal Creams, Foams, and Jellies', MS='Medicated dosage forms for topical application in the vagina. A cream is a semisolid emulsion containing suspended or dissolved medication; a foam is a dispersion of a gas in a medicated liquid resulting in a light, frothy mass; a jelly is a colloidal semisolid mass of a water soluble medicated material, usually translucent.'),
  'D014623': NtO(MH='Vaginal Diseases', MS='Pathological processes of the VAGINA.'),
  'D014624': NtO(MH='Vaginal Fistula', MS='An abnormal anatomical passage that connects the VAGINA to other organs, such as the bladder (VESICOVAGINAL FISTULA) or the rectum (RECTOVAGINAL FISTULA).'),
  'D014625': NtO(MH='Vaginal Neoplasms', MS='Tumors or cancer of the VAGINA.'),
  'D014626': NtO(MH='Vaginal Smears', MS='Collection of pooled secretions of the posterior vaginal fornix for cytologic examination.'),
  'D014627': NtO(MH='Vaginitis', MS='Inflammation of the vagina characterized by pain and a purulent discharge.'),
  'D014628': NtO(MH='Vagotomy', MS='The interruption or removal of any part of the vagus (10th cranial) nerve. Vagotomy may be performed for research or for therapeutic purposes.'),
  'D014629': NtO(MH='Vagotomy, Proximal Gastric', MS='Vagal denervation of that part of the STOMACH lined with acid-secreting mucosa (GASTRIC MUCOSA) containing the GASTRIC PARIETAL CELLS. Since the procedure leaves the vagal branches to the antrum and PYLORUS intact, it circumvents gastric drainage required with truncal vagotomy techniques.'),
  'D014630': NtO(MH='Vagus Nerve', MS='The 10th cranial nerve. The vagus is a mixed nerve which contains somatic afferents (from skin in back of the ear and the external auditory meatus), visceral afferents (from the pharynx, larynx, thorax, and abdomen), parasympathetic efferents (to the thorax and abdomen), and efferents to striated muscle (of the larynx and pharynx).'),
  'D014631': NtO(MH='Valerates', MS='Derivatives of valeric acid, including its salts and esters.'),
  'D014632': NtO(MH='Valerian', MS='A plant genus of the family VALERIANACEAE, order Dipsacales, subclass Asteridae, class Magnoliopsida. It is best known for the sedative use and valepotriate content of the roots. It is sometimes called Garden Heliotrope but is unrelated to true Heliotrope (HELIOTROPIUM).'),
  'D014633': NtO(MH='Valine', MS='A branched-chain essential amino acid that has stimulant activity. It promotes muscle growth and tissue repair. It is a precursor in the penicillin biosynthetic pathway.'),
  'D014634': NtO(MH='Valinomycin', MS='A cyclododecadepsipeptide ionophore antibiotic produced by Streptomyces fulvissimus and related to the enniatins. It is composed of 3 moles each of L-valine, D-alpha-hydroxyisovaleric acid, D-valine, and L-lactic acid linked alternately to form a 36-membered ring. (From Merck Index, 11th ed) Valinomycin is a potassium selective ionophore and is commonly used as a tool in biochemical studies.'),
  'D014635': NtO(MH='Valproic Acid', MS='A fatty acid with anticonvulsant and anti-manic properties that is used in the treatment of EPILEPSY and BIPOLAR DISORDER. The mechanisms of its therapeutic actions are not well understood. It may act by increasing GAMMA-AMINOBUTYRIC ACID levels in the brain or by altering the properties of VOLTAGE-GATED SODIUM CHANNELS.'),
  'D014636': NtO(MH='Valsalva Maneuver', MS='Forced expiratory effort against a closed GLOTTIS.'),
  'D014637': NtO(MH='Valine-tRNA Ligase', MS='An enzyme that activates valine with its specific transfer RNA. EC 6.1.1.9'),
  'D014638': NtO(MH='Vanadates', MS='Oxyvanadium ions in various states of oxidation. They act primarily as ion transport inhibitors due to their inhibition of Na(+)-, K(+)-, and Ca(+)-ATPase transport systems. They also have insulin-like action, positive inotropic action on cardiac ventricular muscle, and other metabolic effects.'),
  'D014639': NtO(MH='Vanadium', MS='A metallic element with the atomic symbol V, atomic number 23, and atomic weight 50.94. It is used in the manufacture of vanadium steel. Prolonged exposure can lead to chronic intoxication caused by absorption usually via the lungs.'),
  'D014640': NtO(MH='Vancomycin', MS='Antibacterial obtained from Streptomyces orientalis. It is a glycopeptide related to RISTOCETIN that inhibits bacterial cell wall assembly and is toxic to kidneys and the inner ear.'),
  'D014641': NtO(MH='Vanillic Acid', MS='A flavoring agent. It is the intermediate product in the two-step bioconversion of ferulic acid to vanillin. (J Biotechnol 1996;50(2-3):107-13).'),
  'D014642': NtO(MH='Vanilmandelic Acid', MS='A 3-O-methyl ether of 3,4-dihydroxymandelic acid. It is an end-stage metabolite of CATECHOLAMINES; EPINEPHRINE; and NOREPINEPHRINE.'),
  'D014643': NtO(MH='Variant Surface Glycoproteins, Trypanosoma', MS='Glycoproteins attached to the surface coat of the trypanosome. Many of these glycoproteins show amino acid sequence diversity expressed as antigenic variations. This continuous development of antigenically distinct variants in the course of infection ensures that some trypanosomes always survive the development of immune response to propagate the infection.'),
  'D014644': NtO(MH='Genetic Variation', MS='Genotypic differences observed among individuals in a population.'),
  'D014645': NtO(MH='Herpesvirus 3, Human', MS='The type species of VARICELLOVIRUS causing CHICKENPOX (varicella) and HERPES ZOSTER (shingles) in humans.'),
  'D014646': NtO(MH='Varicocele', MS='A condition characterized by the dilated tortuous veins of the SPERMATIC CORD with a marked left-sided predominance. Adverse effect on male fertility occurs when varicocele leads to an increased scrotal (and testicular) temperature and reduced testicular volume.'),
  'D014647': NtO(MH='Varicose Ulcer', MS='Skin breakdown or ulceration in the drainage area of a VARICOSE VEIN, usually in the leg.'),
  'D014648': NtO(MH='Varicose Veins', MS='Enlarged and tortuous VEINS.'),
  'D014649': NtO(MH='Vas Deferens', MS='The excretory duct of the testes that carries SPERMATOZOA. It rises from the SCROTUM and joins the SEMINAL VESICLES to form the ejaculatory duct.'),
  'D014650': NtO(MH='Vasa Nervorum', MS='Blood vessels supplying the nerves.'),
  'D014651': NtO(MH='Vasa Vasorum', MS='Nutrient blood vessels which supply the walls of large arteries or veins.'),
  'D014652': NtO(MH='Vascular Diseases', MS='Pathological processes involving any of the BLOOD VESSELS in the cardiac or peripheral circulation. They include diseases of ARTERIES; VEINS; and rest of the vasculature system in the body.'),
  'D014653': NtO(MH='Vascular Headaches', MS='Secondary headache disorders attributed to a variety of cranial or cervical vascular disorders, such as BRAIN ISCHEMIA; INTRACRANIAL HEMORRHAGES; and CENTRAL NERVOUS SYSTEM VASCULAR MALFORMATIONS.'),
  'D014654': NtO(MH='Vascular Patency', MS='The degree to which BLOOD VESSELS are not blocked or obstructed.'),
  'D014655': NtO(MH='Vascular Resistance', MS='The force that opposes the flow of BLOOD through a vascular bed. It is equal to the difference in BLOOD PRESSURE across the vascular bed divided by the CARDIAC OUTPUT.'),
  'D014656': NtO(MH='Vascular Surgical Procedures', MS='Operative procedures for the treatment of vascular disorders.'),
  'D014657': NtO(MH='Vasculitis', MS='Inflammation of any one of the blood vessels, including the ARTERIES; VEINS; and rest of the vasculature system in the body.'),
  'D014659': NtO(MH='Vasectomy', MS='Surgical removal of the ductus deferens, or a portion of it. It is done in association with prostatectomy, or to induce infertility. (Dorland, 28th ed)'),
  'D014660': NtO(MH='Vasoactive Intestinal Peptide', MS='A highly basic, 28 amino acid neuropeptide released from intestinal mucosa. It has a wide range of biological actions affecting the cardiovascular, gastrointestinal, and respiratory systems and is neuroprotective. It binds special receptors (RECEPTORS, VASOACTIVE INTESTINAL PEPTIDE).'),
  'D014661': NtO(MH='Vasoconstriction', MS='The physiological narrowing of BLOOD VESSELS by contraction of the VASCULAR SMOOTH MUSCLE.'),
  'D014662': NtO(MH='Vasoconstrictor Agents', MS='Drugs used to cause constriction of the blood vessels.'),
  'D014663': NtO(MH='Nasal Decongestants', MS='Drugs designed to treat inflammation of the nasal passages, generally the result of an infection (more often than not the common cold) or an allergy related condition, e.g., hay fever. The inflammation involves swelling of the mucous membrane that lines the nasal passages and results in inordinate mucus production. The primary class of nasal decongestants are vasoconstrictor agents. (From PharmAssist, The Family Guide to Health and Medicine, 1993)'),
  'D014664': NtO(MH='Vasodilation', MS='The physiological widening of BLOOD VESSELS by relaxing the underlying VASCULAR SMOOTH MUSCLE.'),
  'D014665': NtO(MH='Vasodilator Agents', MS='Drugs used to cause dilation of the blood vessels.'),
  'D014666': NtO(MH='Vasomotor System', MS='The neural systems which act on VASCULAR SMOOTH MUSCLE to control blood vessel diameter. The major neural control is through the sympathetic nervous system.'),
  'D014667': NtO(MH='Vasopressins', MS='Antidiuretic hormones released by the NEUROHYPOPHYSIS of all vertebrates (structure varies with species) to regulate water balance and OSMOLARITY. In general, vasopressin is a nonapeptide consisting of a six-amino-acid ring with a cysteine 1 to cysteine 6 disulfide bridge or an octapeptide containing a CYSTINE. All mammals have arginine vasopressin except the pig with a lysine at position 8. Vasopressin, a vasoconstrictor, acts on the KIDNEY COLLECTING DUCTS to increase water reabsorption, increase blood volume and blood pressure.'),
  'D014668': NtO(MH='Vasotocin', MS='A nonapeptide that contains the ring of OXYTOCIN and the side chain of ARG-VASOPRESSIN with the latter determining the specific recognition of hormone receptors. Vasotocin is the non-mammalian vasopressin-like hormone or antidiuretic hormone regulating water and salt metabolism.'),
  'D014669': NtO(MH='Vasovasostomy', MS='Surgical anastomosis or fistulization of the spermatic ducts to restore fertility in a previously vasectomized male.'),
  'D014670': NtO(MH='Ampulla of Vater', MS='A dilation of the duodenal papilla that is the opening of the juncture of the COMMON BILE DUCT and the MAIN PANCREATIC DUCT, also known as the hepatopancreatic ampulla.'),
  'D014671': NtO(MH='Vatican City', MS='Enclave of Rome.'),
  'D014672': NtO(MH='Vectorcardiography', MS='Recording of the moment-to-moment electromotive forces of the heart on a plane of the body surface delineated as a vector function of time.'),
  'D014673': NtO(MH='Vecuronium Bromide', MS='Monoquaternary homolog of PANCURONIUM. A non-depolarizing neuromuscular blocking agent with shorter duration of action than pancuronium. Its lack of significant cardiovascular effects and lack of dependence on good kidney function for elimination as well as its short duration of action and easy reversibility provide advantages over, or alternatives to, other established neuromuscular blocking agents.'),
  'D014674': NtO(MH='Plant Proteins, Dietary', MS='Proteins which are present in or isolated from vegetables or vegetable products used as food. The concept is distinguished from PLANT PROTEINS which refers to non-dietary proteins from plants.'),
  'D014675': NtO(MH='Vegetables', MS='A food group comprised of EDIBLE PLANTS or their parts.'),
  'D014676': NtO(MH='Diet, Vegetarian', MS='The dietary practice of completely avoiding meat or fish products in the DIET, consuming VEGETABLES, CEREALS, and NUTS. Some vegetarian diets called lacto-ovo also include milk and egg products.'),
  'D014677': NtO(MH='Pharmaceutical Vehicles', MS='A carrier or inert medium used as a solvent (or diluent) in which the medicinally active agent is formulated and or administered. (Dictionary of Pharmacy, 1986)'),
  'D014678': NtO(MH='Veillonella', MS='A genus of gram-negative, anaerobic cocci parasitic in the mouth and in the intestinal and respiratory tracts of man and other animals.'),
  'D014680': NtO(MH='Veins', MS='The vessels carrying blood away from the CAPILLARY BEDS.'),
  'D014681': NtO(MH='Velopharyngeal Insufficiency', MS='Failure of the SOFT PALATE to reach the posterior pharyngeal wall to close the opening between the oral and nasal cavities. Incomplete velopharyngeal closure is primarily related to surgeries (ADENOIDECTOMY; CLEFT PALATE) or an incompetent PALATOPHARYNGEAL SPHINCTER. It is characterized by hypernasal speech.'),
  'D014682': NtO(MH='Vena Cava, Inferior', MS='The venous trunk which receives blood from the lower extremities and from the pelvic and abdominal organs.'),
  'D014683': NtO(MH='Vena Cava, Superior', MS='The venous trunk which returns blood from the head, neck, upper extremities and chest.'),
  'D014684': NtO(MH='Venae Cavae', MS='The inferior and superior venae cavae.'),
  'D014685': NtO(MH='Venereal Tumors, Veterinary', MS='Tumors most commonly seen on or near the genitalia. They are venereal, most likely transmitted through transplantation of cells by contact. Metastases have been reported. Spontaneous regression may occur.'),
  'D014686': NtO(MH='Venereology', MS='A branch of medicine which deals with sexually transmitted disease.'),
  'D014687': NtO(MH='Venezuela', MS='A country in northern South America, bordering the Caribbean Sea and the North Atlantic Ocean, located between Colombia and Guyana.'),
  'D014688': NtO(MH='Venoms', MS='Poisonous animal secretions forming fluid mixtures of many different enzymes, toxins, and other substances. These substances are produced in specialized glands and secreted through specialized delivery systems (nematocysts, spines, fangs, etc.) for disabling prey or predator.'),
  'D014689': NtO(MH='Venous Insufficiency', MS='Impaired venous blood flow or venous return (venous stasis), usually caused by inadequate venous valves. Venous insufficiency often occurs in the legs, and is associated with EDEMA and sometimes with VENOUS STASIS ULCERS at the ankle.'),
  'D014690': NtO(MH='Venous Pressure', MS='The blood pressure in the VEINS. It is usually measured to assess the filling PRESSURE to the HEART VENTRICLE.'),
  'D014691': NtO(MH='Ventilation', MS='Supplying a building or house, their rooms and corridors, with fresh air. The controlling of the environment thus may be in public or domestic sites and in medical or non-medical locales. (From Dorland, 28th ed)'),
  'D014692': NtO(MH='Ventilation-Perfusion Ratio', MS='The ratio of alveolar ventilation to simultaneous alveolar capillary blood flow in any part of the lung. (Stedman, 25th ed)'),
  'D014693': NtO(MH='Ventricular Fibrillation', MS='A potentially lethal cardiac arrhythmia that is characterized by uncoordinated extremely rapid firing of electrical impulses (400-600/min) in HEART VENTRICLES. Such asynchronous ventricular quivering or fibrillation prevents any effective cardiac output and results in unconsciousness (SYNCOPE). It is one of the major electrocardiographic patterns seen with CARDIAC ARREST.'),
  'D014694': NtO(MH='Ventricular Outflow Obstruction', MS='Occlusion of the outflow tract in either the LEFT VENTRICLE or the RIGHT VENTRICLE of the heart. This may result from CONGENITAL HEART DEFECTS, predisposing heart diseases, complications of surgery, or HEART NEOPLASMS.'),
  'D014695': NtO(MH='Cerebral Ventriculography', MS='Radiography of the ventricular system of the brain after injection of air or other contrast medium directly into the cerebral ventricles. It is used also for x-ray computed tomography of the cerebral ventricles.'),
  'D014696': NtO(MH='Ventriculostomy', MS='Surgical creation of an opening in a cerebral ventricle.'),
  'D014697': NtO(MH='Ventromedial Hypothalamic Nucleus', MS='A nucleus of the middle hypothalamus, the largest cell group of the tuberal region with small-to-medium size cells.'),
  'D014698': NtO(MH='Venturicidins', MS='Toxic substances isolated from various strains of Streptomyces. They are 20-membered macrolides that inhibit oxidative phosphorylation and mitochondrial ATPases. Venturicidins A and B are glycosides. Used mainly as tools in the study of mitochondrial function.'),
  'D014699': NtO(MH='Venules', MS='The minute vessels that collect blood from the capillary plexuses and join together to form veins.'),
  'D014700': NtO(MH='Verapamil', MS='A calcium channel blocker that is a class IV anti-arrhythmia agent.'),
  'D014701': NtO(MH='Veratridine', MS='A benzoate-cevane found in VERATRUM and Schoenocaulon. It activates SODIUM CHANNELS to stay open longer than normal.'),
  'D014702': NtO(MH='Veratrine', MS='A voltage-gated sodium channel activator.'),
  'D014703': NtO(MH='Veratrum', MS='A plant genus of the family Melanthiaceae with roots that contain VERATRUM ALKALOIDS used as emetics, parasiticides, antihypertensives. The common name "black hellebore" is used for both Veratrum nigrum and HELLEBORUS NIGER.'),
  'D014704': NtO(MH='Veratrum Alkaloids', MS='Alkaloids with powerful hypotensive effects isolated from American or European Hellebore (Veratrum viride Ait. Liliaceae and Veratrum album L. Liliaceae). They increase cholinergic and decrease adrenergic tone with appropriate side effects and at higher doses depress respiration and produce cardiac arrhythmias; only the ester alkaloids have been used as hypotensive agents in specific instances. They have been generally replaced by drugs with fewer adverse effects.'),
  'D014705': NtO(MH='Verbal Behavior', MS='Includes both producing and responding to words, either written or spoken.'),
  'D014706': NtO(MH='Verbal Learning', MS='Learning to respond verbally to a verbal stimulus cue.'),
  'D014707': NtO(MH='Vermont', MS='State bounded on the north by Canada, on the east by New Hampshire, on the south by Massachusetts, and on the west by New York.'),
  'D014708': NtO(MH='Vernix Caseosa', MS='An unctuous substance composed of sebum and desquamated epithelial cells, which covers the skin of the fetus.'),
  'D014709': NtO(MH='Vero Cells', MS='A CELL LINE derived from the kidney of the African green (vervet) monkey, (CHLOROCEBUS AETHIOPS) used primarily in virus replication studies and plaque assays.'),
  'D014710': NtO(MH='Version, Fetal', MS='The artificial alteration of the fetal position to facilitate birth.'),
  'D014711': NtO(MH='Vertebral Artery', MS='The first branch of the SUBCLAVIAN ARTERY with distribution to muscles of the NECK; VERTEBRAE; SPINAL CORD; CEREBELLUM; and interior of the CEREBRUM.'),
  'D014713': NtO(MH='Viruses, Unclassified', MS='Viruses whose taxonomic relationships have not been established.'),
  'D014714': NtO(MH='Vertebrates', MS='Animals having a vertebral column, members of the phylum Chordata, subphylum Craniata comprising mammals, birds, reptiles, amphibians, and fishes.'),
  'D014715': NtO(MH='Vertebrobasilar Insufficiency', MS='Localized or diffuse reduction in blood flow through the vertebrobasilar arterial system, which supplies the BRAIN STEM; CEREBELLUM; OCCIPITAL LOBE; medial TEMPORAL LOBE; and THALAMUS. Characteristic clinical features include SYNCOPE; lightheadedness; visual disturbances; and VERTIGO. BRAIN STEM INFARCTIONS or other BRAIN INFARCTION may be associated.'),
  'D014716': NtO(MH='Vertical Dimension', MS='The length of the face determined by the distance of separation of jaws. Occlusal vertical dimension (OVD or VDO) or contact vertical dimension is the lower face height with the teeth in centric occlusion. Rest vertical dimension (VDR) is the lower face height measured from a chin point to a point just below the nose, with the mandible in rest position. (From Jablonski, Dictionary of Dentistry, 1992, p250)'),
  'D014717': NtO(MH='Vertigo', MS='An illusion of movement, either of the external world revolving around the individual or of the individual revolving in space. Vertigo may be associated with disorders of the inner ear (EAR, INNER); VESTIBULAR NERVE; BRAINSTEM; or CEREBRAL CORTEX. Lesions in the TEMPORAL LOBE and PARIETAL LOBE may be associated with FOCAL SEIZURES that may feature vertigo as an ictal manifestation. (From Adams et al., Principles of Neurology, 6th ed, pp300-1)'),
  'D014718': NtO(MH='Vesico-Ureteral Reflux', MS='Retrograde flow of urine from the URINARY BLADDER into the URETER. This is often due to incompetence of the vesicoureteral valve leading to ascending bacterial infection into the KIDNEY.'),
  'D014719': NtO(MH='Vesicovaginal Fistula', MS='An abnormal anatomical passage between the URINARY BLADDER and the VAGINA.'),
  'D014720': NtO(MH='Vesicular Exanthema of Swine', MS='A calicivirus infection of swine characterized by hydropic degeneration of the oral and cutaneous epithelia.'),
  'D014721': NtO(MH='Vesicular stomatitis Indiana virus', MS='The type species of VESICULOVIRUS causing a disease symptomatically similar to FOOT-AND-MOUTH DISEASE in cattle, horses, and pigs. It may be transmitted to other species including humans, where it causes influenza-like symptoms.'),
  'D014722': NtO(MH='Vestibule, Labyrinth', MS='An oval, bony chamber of the inner ear, part of the bony labyrinth. It is continuous with bony COCHLEA anteriorly, and SEMICIRCULAR CANALS posteriorly. The vestibule contains two communicating sacs (utricle and saccule) of the balancing apparatus. The oval window on its lateral wall is occupied by the base of the STAPES of the MIDDLE EAR.'),
  'D014723': NtO(MH='Vestibular Aqueduct', MS='A small bony canal linking the vestibule of the inner ear to the posterior part of the internal surface of the petrous TEMPORAL BONE. It transmits the endolymphatic duct and two small blood vessels.'),
  'D014724': NtO(MH='Vestibular Function Tests', MS='A number of tests used to determine if the brain or balance portion of the inner ear are causing dizziness.'),
  'D014725': NtO(MH='Vestibular Nerve', MS="The vestibular part of the 8th cranial nerve (VESTIBULOCOCHLEAR NERVE). The vestibular nerve fibers arise from neurons of Scarpa's ganglion and project peripherally to vestibular hair cells and centrally to the VESTIBULAR NUCLEI of the BRAIN STEM. These fibers mediate the sense of balance and head position."),
  'D014726': NtO(MH='Vestibular Nuclei', MS='The four cellular masses in the floor of the fourth ventricle giving rise to a widely dispersed special sensory system. Included is the superior, medial, inferior, and LATERAL VESTIBULAR NUCLEUS. (From Dorland, 27th ed)'),
  'D014727': NtO(MH='Vestibuloplasty', MS='Those procedures designed to widen the zone of attached gingiva and deepen the vestibular depth which will facilitate the clearance of the area for natural food passage, and provide access for toothbrushing and interdental stimulation.'),
  'D014728': NtO(MH='Veterans', MS='Former members of the armed services.'),
  'D014729': NtO(MH='Veterans Disability Claims', MS='Disorders claimed as a result of military service.'),
  'D014730': NtO(MH='Veterinary Medicine', MS='The medical science concerned with the prevention, diagnosis, and treatment of diseases in animals.'),
  'D014731': NtO(MH='Veterinary Service, Military', MS='A corps of the armed services concerned with animal medicine, the chief interest of which is the care of government-owned working dogs (as in the military police units), working horses (as in state funerals), and working military dolphins (as in undersea exploration and other activities). In the United States Army Veterinary Corps animal medicine overlaps and interconnects with biomedical research using laboratory research animals. A related activity is laboratory animal care. The Corps provides limited care for privately owned animals of military personnel through non-appropriated funds. Military service veterinarians in the United States Army must be graduates of accredited veterinary schools and must have a state license. (Telephone communication with Lt. Col. William Inskeep II, U.S. Army Veterinary Corps, October 4, 1994)'),
  'D014732': NtO(MH='Vibration', MS='A continuing periodic change in displacement with respect to a fixed reference. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D014733': NtO(MH='Vibrio', MS='A genus of VIBRIONACEAE, made up of short, slightly curved, motile, gram-negative rods. Various species produce cholera and other gastrointestinal disorders as well as abortion in sheep and cattle.'),
  'D014734': NtO(MH='Vibrio cholerae', MS='The etiologic agent of CHOLERA.'),
  'D014735': NtO(MH='Vibrio Infections', MS='Infections with bacteria of the genus VIBRIO.'),
  'D014736': NtO(MH='Vibrio parahaemolyticus', MS='A species of bacteria found in the marine environment, sea foods, and the feces of patients with acute enteritis.'),
  'D014737': NtO(MH='Vibrionaceae', MS='A family of gram-negative bacteria whose members predominate in the bacterial flora of PLANKTON; FISHES; and SEAWATER. Some members are important pathogens for humans and animals.'),
  'D014738': NtO(MH='Vibrissae', MS='Stiff hairs projecting from the face around the nose of most mammals, acting as touch receptors.'),
  'D014739': NtO(MH='Victoria', MS="A state in southeastern Australia, the southernmost state. Its capital is Melbourne. It was discovered in 1770 by Captain Cook and first settled by immigrants from Tasmania. In 1851 it was separated from New South Wales as a separate colony. Self-government was introduced in 1851; it became a state in 1901. It was named for Queen Victoria in 1851. (From Webster's New Geographical Dictionary, 1988, p1295 & Room, Brewer's Dictionary of Names, p574)"),
  'D014740': NtO(MH='Vidarabine', MS='A nucleoside antibiotic isolated from Streptomyces antibioticus. It has some antineoplastic properties and has broad spectrum activity against DNA viruses in cell cultures and significant antiviral activity against infections caused by a variety of viruses such as the herpes viruses, the VACCINIA VIRUS and varicella zoster virus.'),
  'D014741': NtO(MH='Video Recording', MS='The storing or preserving of video signals to be played back later via a transmitter or receiver.'),
  'D014742': NtO(MH='Videodisc Recording', MS='The storing of visual and usually sound signals on discs for later reproduction on a television screen or monitor.'),
  'D014743': NtO(MH='Videotape Recording', MS='Recording of visual and sometimes sound signals on magnetic tape.'),
  'D014744': NtO(MH='Vietnam', MS='A country in southeastern Asia, bordering the Gulf of Thailand, Gulf of Tonkin, and South China Sea, as well as China, Laos, and Cambodia. The capital is Hanoi.'),
  'D014745': NtO(MH='Viloxazine', MS='A morpholine derivative used as an antidepressant. It is similar in action to IMIPRAMINE.'),
  'D014746': NtO(MH='Vimentin', MS='An intermediate filament protein found in most differentiating cells, in cells grown in tissue culture, and in certain fully differentiated cells. Its insolubility suggests that it serves a structural function in the cytoplasm. MW 52,000.'),
  'D014747': NtO(MH='Vinblastine', MS='Antitumor alkaloid isolated from Vinca rosea. (Merck, 11th ed.)'),
  'D014748': NtO(MH='Vinca Alkaloids', MS='A group of indole-indoline dimers which are ALKALOIDS obtained from the VINCA genus of plants. They inhibit polymerization of TUBULIN into MICROTUBULES thus blocking spindle formation and arresting cells in METAPHASE. They are some of the most useful ANTINEOPLASTIC AGENTS.'),
  'D014749': NtO(MH='Vincamine', MS='A major alkaloid of Vinca minor L., Apocynaceae. It has been used therapeutically as a vasodilator and antihypertensive agent, particularly in cerebrovascular disorders.'),
  'D014750': NtO(MH='Vincristine', MS='An antitumor alkaloid isolated from VINCA ROSEA. (Merck, 11th ed.)'),
  'D014751': NtO(MH='Vindesine', MS='Vinblastine derivative with antineoplastic activity against CANCER. Major side effects are myelosuppression and neurotoxicity. Vindesine is used extensively in chemotherapy protocols (ANTINEOPLASTIC COMBINED CHEMOTHERAPY PROTOCOLS).'),
  'D014752': NtO(MH='Vinyl Chloride', MS='A gas that has been used as an aerosol propellant and is the starting material for polyvinyl resins. Toxicity studies have shown various adverse effects, particularly the occurrence of liver neoplasms.'),
  'D014753': NtO(MH='Vinyl Compounds', MS='Compounds containing the vinyl (-C=CH2) functional group.'),
  'D014754': NtO(MH='Violence', MS='Individual or group aggressive behavior which is socially non-acceptable, turbulent, and often destructive. It is precipitated by frustrations, hostility, prejudices, etc.'),
  'D014755': NtO(MH='Viologens', MS="A group of dipyridinium chloride derivatives that are used as oxidation-reduction indicators. The general formula is 1,1'-di-R-4,4'-bipyridinium chloride, where R = methyl, ethyl, benzyl or, betaine."),
  'D014756': NtO(MH='Viomycin', MS='A strongly basic peptide, antibiotic complex from several strains of Streptomyces. It is allergenic and toxic to kidneys and the labyrinth. Viomycin is used in tuberculosis as several different salts and in combination with other agents.'),
  'D014757': NtO(MH='Viper Venoms', MS='Venoms from SNAKES of the viperid family. They tend to be less toxic than elapid or hydrophid venoms and act mainly on the vascular system, interfering with coagulation and capillary membrane integrity and are highly cytotoxic. They contain large amounts of several enzymes, other factors, and some toxins.'),
  'D014758': NtO(MH='Viral Core Proteins', MS='Proteins found mainly in icosahedral DNA and RNA viruses. They consist of proteins directly associated with the nucleic acid inside the NUCLEOCAPSID.'),
  'D014759': NtO(MH='Viral Envelope Proteins', MS='Layers of protein which surround the capsid in animal viruses with tubular nucleocapsids. The envelope consists of an inner layer of lipids and virus specified proteins also called membrane or matrix proteins. The outer layer consists of one or more types of morphological subunits called peplomers which project from the viral envelope; this layer always consists of glycoproteins.'),
  'D014760': NtO(MH='Viral Fusion Proteins', MS='Proteins, usually glycoproteins, found in the viral envelopes of a variety of viruses. They promote cell membrane fusion and thereby may function in the uptake of the virus by cells.'),
  'D014761': NtO(MH='Viral Hepatitis Vaccines', MS='Any vaccine raised against any virus or viral derivative that causes hepatitis.'),
  'D014762': NtO(MH='Viral Interference', MS='A phenomenon in which infection by a first virus results in resistance of cells or tissues to infection by a second, unrelated virus.'),
  'D014763': NtO(MH='Viral Matrix Proteins', MS='Proteins associated with the inner surface of the lipid bilayer of the viral envelope. These proteins have been implicated in control of viral transcription and may possibly serve as the "glue" that binds the nucleocapsid to the appropriate membrane site during viral budding from the host cell.'),
  'D014764': NtO(MH='Viral Proteins', MS='Proteins found in any species of virus.'),
  'D014765': NtO(MH='Viral Vaccines', MS='Suspensions of attenuated or killed viruses administered for the prevention or treatment of infectious viral disease.'),
  'D014766': NtO(MH='Viremia', MS='The presence of viruses in the blood.'),
  'D014767': NtO(MH='United States Virgin Islands', MS='A group of islands in the Lesser Antilles in the West Indies, the three main islands being St. Croix, St. Thomas, and St. John. The capital is Charlotte Amalie. Before 1917 the U.S. Virgin Islands were held by the Danish and called the Danish West Indies but the name was changed when the United States acquired them by purchase.'),
  'D014768': NtO(MH='Virginia', MS='State bounded on the north by Maryland and West Virginia, on the east by the Atlantic Ocean, the Chesapeake Bay and Maryland, and on the south by North Carolina and Tennessee, and on the west by Kentucky and West Virginia.'),
  'D014769': NtO(MH='Virginiamycin', MS='A cyclic polypeptide antibiotic complex from Streptomyces virginiae, S. loidensis, S. mitakaensis, S. pristina-spiralis, S. ostreogriseus, and others. It consists of 2 major components, VIRGINIAMYCIN FACTOR M1 and virginiamycin Factor S1. It is used to treat infections with gram-positive organisms and as a growth promoter in cattle, swine, and poultry.'),
  'D014770': NtO(MH='Virilism', MS='Development of male secondary SEX CHARACTERISTICS in the FEMALE. It is due to the effects of androgenic metabolites of precursors from endogenous or exogenous sources, such as ADRENAL GLANDS or therapeutic drugs.'),
  'D014771': NtO(MH='Virion', MS='The infective system of a virus, composed of the viral genome, a protein core, and a protein coat called a capsid, which may be naked or enclosed in a lipoprotein envelope called the peplos.'),
  'D014772': NtO(MH='Viroids', MS='A group of pathogens comprising the smallest known agents of infectious disease. They are unencapsulated and are capable of replicating autonomously in susceptible cells. Positively identified viroids composed of single-stranded RNA have been isolated from higher plants, but the existence of DNA viroids pathogenic to animals is suspected.'),
  'D014773': NtO(MH='Virology', MS='The study of the structure, growth, function, genetics, and reproduction of viruses, and VIRUS DISEASES.'),
  'D014774': NtO(MH='Virulence', MS='The degree of pathogenicity within a group or species of microorganisms or viruses as indicated by case fatality rates and/or the ability of the organism to invade the tissues of the host. The pathogenic capacity of an organism is determined by its VIRULENCE FACTORS.'),
  'D014775': NtO(MH='Virus Activation', MS='The mechanism by which latent viruses, such as genetically transmitted tumor viruses (PROVIRUSES) or PROPHAGES of lysogenic bacteria, are induced to replicate and then released as infectious viruses. It may be effected by various endogenous and exogenous stimuli, including B-cell LIPOPOLYSACCHARIDES, glucocorticoid hormones, halogenated pyrimidines, IONIZING RADIATION, ultraviolet light, and superinfecting viruses.'),
  'D014776': NtO(MH='Virus Cultivation', MS='Process of growing viruses in live animals, plants, or cultured cells.'),
  'D014777': NtO(MH='Virus Diseases', MS='A general term for diseases caused by viruses.'),
  'D014779': NtO(MH='Virus Replication', MS='The process of intracellular viral multiplication, consisting of the synthesis of PROTEINS; NUCLEIC ACIDS; and sometimes LIPIDS, and their assembly into a new infectious particle.'),
  'D014780': NtO(MH='Viruses', MS='Minute infectious agents whose genomes are composed of DNA or RNA, but not both. They are characterized by a lack of independent metabolism and the inability to replicate outside living host cells.'),
  'D014781': NtO(MH='Viscera', MS='Any of the large interior organs in any one of the three great cavities of the body, especially in the abdomen.'),
  'D014782': NtO(MH='Visceral Prolapse', MS='The prolapse or downward displacement of the VISCERA.'),
  'D014783': NtO(MH='Viscosity', MS='The resistance that a gaseous or liquid system offers to flow when it is subjected to shear stress. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D014784': NtO(MH='Mistletoe', MS='Parasitic plants that form a bushy growth on branches of host trees which are in the order Santalales. It includes the Christmas mistletoe family (VISCACEAE), the showy mistletoe family (LORANTHACEAE) and the catkin mistletoe family (Eremolepidaceae). The composition of toxins, lectins, tyramine, phenethylamines, and other compounds may be affected by the host.'),
  'D014785': NtO(MH='Vision, Ocular', MS='The process in which light signals are transformed by the PHOTORECEPTOR CELLS into electrical signals which can then be transmitted to the brain.'),
  'D014786': NtO(MH='Vision Disorders', MS='Visual impairments limiting one or more of the basic functions of the eye: visual acuity, dark adaptation, color vision, or peripheral vision. These may result from EYE DISEASES; OPTIC NERVE DISEASES; VISUAL PATHWAY diseases; OCCIPITAL LOBE diseases; OCULAR MOTILITY DISORDERS; and other conditions (From Newell, Ophthalmology: Principles and Concepts, 7th ed, p132).'),
  'D014787': NtO(MH='Vision Tests', MS='A series of tests used to assess various functions of the eyes.'),
  'D014788': NtO(MH='Vision, Entoptic', MS='Visual sensation derived from sensory stimulation by objects or shadows inside the eye itself, such as floating vitreous fibers, tissues, or blood.'),
  'D014789': NtO(MH='Visitors to Patients', MS="Patients' guests and rules for visiting."),
  'D014790': NtO(MH='Visna-maedi virus', MS='A species of LENTIVIRUS, subgenus ovine-caprine lentiviruses (LENTIVIRUSES, OVINE-CAPRINE), that can cause chronic pneumonia (maedi), mastitis, arthritis, and encephalomyelitis (visna) in sheep. Maedi is a progressive pneumonia of sheep which is similar to but not the same as jaagsiekte (PULMONARY ADENOMATOSIS, OVINE). Visna is a demyelinating leukoencephalomyelitis of sheep which is similar to but not the same as SCRAPIE.'),
  'D014792': NtO(MH='Visual Acuity', MS='Clarity or sharpness of OCULAR VISION or the ability of the eye to see fine details. Visual acuity depends on the functions of RETINA, neuronal transmission, and the interpretative ability of the brain. Normal visual acuity is expressed as 20/20 indicating that one can see at 20 feet what should normally be seen at that distance. Visual acuity can also be influenced by brightness, color, and contrast.'),
  'D014793': NtO(MH='Visual Cortex', MS='Area of the OCCIPITAL LOBE concerned with the processing of visual information relayed via VISUAL PATHWAYS.'),
  'D014794': NtO(MH='Visual Fields', MS="The total area or space visible in a person's peripheral vision with the eye looking straightforward."),
  'D014795': NtO(MH='Visual Pathways', MS='Set of cell bodies and nerve fibers conducting impulses from the eyes to the cerebral cortex. It includes the RETINA; OPTIC NERVE; optic tract; and geniculocalcarine tract.'),
  'D014796': NtO(MH='Visual Perception', MS="The selecting and organizing of visual stimuli based on the individual's past experience."),
  'D014797': NtO(MH='Vital Capacity', MS='The volume of air that is exhaled by a maximal expiration following a maximal inspiration.'),
  'D014798': NtO(MH='Vital Statistics', MS='Used for general articles concerning statistics of births, deaths, marriages, etc.'),
  'D014799': NtO(MH='Vitalism', MS='The metaphysical doctrine that the functions and processes of life are due to a vital principle distinct from physicochemical forces and that the laws of physics and chemistry alone cannot explain life functions and processes. Vitalism is opposed to mechanistic materialism. The belief was that matter was divided into two classes based on behavior with respect to heat: organic and inorganic. Inorganic material could be melted but could always be recovered by removing the heat source. Organic compounds changed form upon heating and could not be recovered by removing the heat source. The proposed explanation for the difference between organic and inorganic compounds was the Vitalism Theory, which stated that inorganic materials did not contain the "vital force" of life.'),
  'D014800': NtO(MH='Vitallium', MS='An alloy of 60% cobalt, 20% chromium, 5% molybdenum, and traces of other substances. It is used in dentures, certain surgical appliances, prostheses, implants, and instruments.'),
  'D014801': NtO(MH='Vitamin A', MS='Retinol and derivatives of retinol that play an essential role in metabolic functioning of the retina, the growth of and differentiation of epithelial tissue, the growth of bone, reproduction, and the immune response. Dietary vitamin A is derived from a variety of CAROTENOIDS found in plants. It is enriched in the liver, egg yolks, and the fat component of dairy products.'),
  'D014802': NtO(MH='Vitamin A Deficiency', MS='A nutritional condition produced by a deficiency of VITAMIN A in the diet, characterized by NIGHT BLINDNESS and other ocular manifestations such as dryness of the conjunctiva and later of the cornea (XEROPHTHALMIA). Vitamin A deficiency is a very common problem worldwide, particularly in developing countries as a consequence of famine or shortages of vitamin A-rich foods. In the United States it is found among the urban poor, the elderly, alcoholics, and patients with malabsorption. (From Cecil Textbook of Medicine, 19th ed, p1179)'),
  'D014803': NtO(MH='Vitamin B Complex', MS='A group of water-soluble vitamins, some of which are COENZYMES.'),
  'D014804': NtO(MH='Vitamin B Deficiency', MS='A condition due to deficiency in any member of the VITAMIN B COMPLEX. These B vitamins are water-soluble and must be obtained from the diet because they are easily lost in the urine. Unlike the lipid-soluble vitamins, they cannot be stored in the body fat.'),
  'D014805': NtO(MH='Vitamin B 12', MS='A cobalt-containing coordination compound produced by intestinal micro-organisms and found also in soil and water. Higher plants do not concentrate vitamin B 12 from the soil and so are a poor source of the substance as compared with animal tissues. INTRINSIC FACTOR is important for the assimilation of vitamin B 12.'),
  'D014806': NtO(MH='Vitamin B 12 Deficiency', MS='A nutritional condition produced by a deficiency of VITAMIN B 12 in the diet, characterized by megaloblastic anemia. Since vitamin B 12 is not present in plants, humans have obtained their supply from animal products, from multivitamin supplements in the form of pills, and as additives to food preparations. A wide variety of neuropsychiatric abnormalities is also seen in vitamin B 12 deficiency and appears to be due to an undefined defect involving myelin synthesis. (From Cecil Textbook of Medicine, 19th ed, p848)'),
  'D014807': NtO(MH='Vitamin D', MS='A vitamin that includes both CHOLECALCIFEROLS and ERGOCALCIFEROLS, which have the common effect of preventing or curing RICKETS in animals. It can also be viewed as a hormone since it can be formed in SKIN by action of ULTRAVIOLET RAYS upon the precursors, 7-dehydrocholesterol and ERGOSTEROL, and acts on VITAMIN D RECEPTORS to regulate CALCIUM in opposition to PARATHYROID HORMONE.'),
  'D014808': NtO(MH='Vitamin D Deficiency', MS='A nutritional condition produced by a deficiency of VITAMIN D in the diet, insufficient production of vitamin D in the skin, inadequate absorption of vitamin D from the diet, or abnormal conversion of vitamin D to its bioactive metabolites. It is manifested clinically as RICKETS in children and OSTEOMALACIA in adults. (From Cecil Textbook of Medicine, 19th ed, p1406)'),
  'D014809': NtO(MH='Vitamin D-Binding Protein', MS='An alpha-globulin found in the plasma of man and other vertebrates. It is apparently synthesized in the liver and carries vitamin D and its metabolites through the circulation and mediates the response of tissue. It is also known as group-specific component (Gc). Gc subtypes are used to determine specific phenotypes and gene frequencies. These data are employed in the classification of population groups, paternity investigations, and in forensic medicine.'),
  'D014810': NtO(MH='Vitamin E', MS='A generic descriptor for all TOCOPHEROLS and TOCOTRIENOLS that exhibit ALPHA-TOCOPHEROL activity. By virtue of the phenolic hydrogen on the 2H-1-benzopyran-6-ol nucleus, these compounds exhibit varying degree of antioxidant activity, depending on the site and number of methyl groups and the type of ISOPRENOIDS.'),
  'D014811': NtO(MH='Vitamin E Deficiency', MS='A nutritional condition produced by a deficiency of VITAMIN E in the diet, characterized by posterior column and spinocerebellar tract abnormalities, areflexia, ophthalmoplegia, and disturbances of gait, proprioception, and vibration. In premature infants vitamin E deficiency is associated with hemolytic anemia, thrombocytosis, edema, intraventricular hemorrhage, and increasing risk of retrolental fibroplasia and bronchopulmonary dysplasia. An apparent inborn error of vitamin E metabolism, named familial isolated vitamin E deficiency, has recently been identified. (Cecil Textbook of Medicine, 19th ed, p1181)'),
  'D014812': NtO(MH='Vitamin K', MS='A lipid cofactor that is required for normal blood clotting. Several forms of vitamin K have been identified: VITAMIN K 1 (phytomenadione) derived from plants, VITAMIN K 2 (menaquinone) from bacteria, and synthetic naphthoquinone provitamins, VITAMIN K 3 (menadione). Vitamin K 3 provitamins, after being alkylated in vivo, exhibit the antifibrinolytic activity of vitamin K. Green leafy vegetables, liver, cheese, butter, and egg yolk are good sources of vitamin K.'),
  'D014813': NtO(MH='Vitamin K Deficiency', MS='A nutritional condition produced by a deficiency of VITAMIN K in the diet, characterized by an increased tendency to hemorrhage (HEMORRHAGIC DISORDERS). Such bleeding episodes may be particularly severe in newborn infants. (From Cecil Textbook of Medicine, 19th ed, p1182)'),
  'D014814': NtO(MH='Vitamin U', MS='A vitamin found in green vegetables. It is used in the treatment of peptic ulcers, colitis, and gastritis and has an effect on secretory, acid-forming, and enzymatic functions of the intestinal tract.'),
  'D014815': NtO(MH='Vitamins', MS='Organic substances that are required in small amounts for maintenance and growth, but which cannot be manufactured by the human body.'),
  'D014816': NtO(MH='Vitelline Duct', MS='The narrow tube connecting the YOLK SAC with the midgut of the EMBRYO; persistence of all or part of it in post-fetal life produces abnormalities, of which the commonest is MECKEL DIVERTICULUM.'),
  'D014817': NtO(MH='Vitelline Membrane', MS='The plasma membrane of the egg.'),
  'D014818': NtO(MH='Vitellogenesis', MS='The active production and accumulation of VITELLINS (egg yolk proteins) in the non-mammalian OOCYTES from circulating precursors, VITELLOGENINS. Vitellogenesis usually begins after the first MEIOSIS and is regulated by estrogenic hormones.'),
  'D014819': NtO(MH='Vitellogenins', MS='Phospholipoglycoproteins produced in the fat body of egg-laying animals such as non-mammalian VERTEBRATES; ARTHROPODS; and others. Vitellogenins are secreted into the HEMOLYMPH, and taken into the OOCYTES by receptor-mediated ENDOCYTOSIS to form the major yolk proteins, VITELLINS. Vitellogenin production is under the regulation of steroid hormones, such as ESTRADIOL and JUVENILE HORMONES in insects.'),
  'D014820': NtO(MH='Vitiligo', MS='A disorder consisting of areas of macular depigmentation, commonly on extensor aspects of extremities, on the face or neck, and in skin folds. Age of onset is often in young adulthood and the condition tends to progress gradually with lesions enlarging and extending until a quiescent state is reached.'),
  'D014821': NtO(MH='Vitrectomy', MS='Removal of the whole or part of the vitreous body in treating endophthalmitis, diabetic retinopathy, retinal detachment, intraocular foreign bodies, and some types of glaucoma.'),
  'D014822': NtO(MH='Vitreous Body', MS='The transparent, semigelatinous substance that fills the cavity behind the CRYSTALLINE LENS of the EYE and in front of the RETINA. It is contained in a thin hyaloid membrane and forms about four fifths of the optic globe.'),
  'D014823': NtO(MH='Vitreous Hemorrhage', MS='Hemorrhage into the VITREOUS BODY.'),
  'D014824': NtO(MH='Vivisection', MS="The cutting of or surgical operation on a living animal, usually for physiological or pathological investigation. (from Merriam-Webster's Collegiate Dict, 10th ed)"),
  'D014825': NtO(MH='Vocabulary', MS='The sum or the stock of words used by a language, a group, or an individual. (From Webster, 3d ed)'),
  'D014826': NtO(MH='Vocal Cord Paralysis', MS='Congenital or acquired paralysis of one or both VOCAL CORDS. This condition is caused by defects in the CENTRAL NERVOUS SYSTEM, the VAGUS NERVE and branches of LARYNGEAL NERVES. Common symptoms are VOICE DISORDERS including HOARSENESS or APHONIA.'),
  'D014827': NtO(MH='Vocal Cords', MS='A pair of cone-shaped elastic mucous membrane projecting from the laryngeal wall and forming a narrow slit between them. Each contains a thickened free edge (vocal ligament) extending from the THYROID CARTILAGE to the ARYTENOID CARTILAGE, and a VOCAL MUSCLE that shortens or relaxes the vocal cord to control sound production.'),
  'D014828': NtO(MH='Vocalization, Animal', MS='Sounds used in animal communication.'),
  'D014829': NtO(MH='Vocational Education', MS='EDUCATION that prepares the student for a specific trade such as plumbing, masonry, and carpentry.'),
  'D014830': NtO(MH='Vocational Guidance', MS='Systematic efforts to assist individuals in selecting an occupation or suitable employment on the basis of aptitude, education, etc.'),
  'D014831': NtO(MH='Voice', MS='The sounds produced by humans by the passage of air through the LARYNX and over the VOCAL CORDS, and then modified by the resonance organs, the NASOPHARYNX, and the MOUTH.'),
  'D014832': NtO(MH='Voice Disorders', MS='Pathological processes that affect voice production, usually involving VOCAL CORDS and the LARYNGEAL MUCOSA. Voice disorders can be caused by organic (anatomical), or functional (emotional or psychological) factors leading to DYSPHONIA; APHONIA; and defects in VOICE QUALITY, loudness, and pitch.'),
  'D014833': NtO(MH='Voice Quality', MS="That component of SPEECH which gives the primary distinction to a given speaker's VOICE when pitch and loudness are excluded. It involves both phonatory and resonatory characteristics. Some of the descriptions of voice quality are harshness, breathiness and nasality."),
  'D014834': NtO(MH='Voice Training', MS='A variety of techniques used to help individuals utilize their voice for various purposes and with minimal use of muscle energy.'),
  'D014835': NtO(MH='Volatilization', MS="A phase transition from liquid state to gas state, which is affected by Raoult's law. It can be accomplished by fractional distillation."),
  'D014836': NtO(MH='Volition', MS='Voluntary activity without external compulsion.'),
  'D014837': NtO(MH='Voluntary Health Agencies', MS='Non-profit organizations concerned with various aspects of health, e.g., education, promotion, treatment, services, etc.'),
  'D014838': NtO(MH='Volunteers', MS='Persons who donate their services.'),
  'D014839': NtO(MH='Vomiting', MS='The forcible expulsion of the contents of the STOMACH through the MOUTH.'),
  'D014840': NtO(MH='Vomiting, Anticipatory', MS='Vomiting caused by expectation of discomfort or unpleasantness.'),
  'D014841': NtO(MH='von Willebrand Factor', MS='A high-molecular-weight plasma protein, produced by endothelial cells and megakaryocytes, that is part of the factor VIII/von Willebrand factor complex. The von Willebrand factor has receptors for collagen, platelets, and ristocetin activity as well as the immunologically distinct antigenic determinants. It functions in adhesion of platelets to collagen and hemostatic plug formation. The prolonged bleeding time in VON WILLEBRAND DISEASES is due to the deficiency of this factor.'),
  'D014842': NtO(MH='von Willebrand Diseases', MS='Group of hemorrhagic disorders in which the VON WILLEBRAND FACTOR is either quantitatively or qualitatively abnormal. They are usually inherited as an autosomal dominant trait though rare kindreds are autosomal recessive. Symptoms vary depending on severity and disease type but may include prolonged bleeding time, deficiency of factor VIII, and impaired platelet adhesion.'),
  'D014843': NtO(MH='Voyeurism', MS='A paraphilia characterized by repetitive looking at unsuspecting people, usually strangers, who are either naked, in the act of disrobing, or engaging in sexual activity, as the method for achieving sexual excitement.'),
  'D014844': NtO(MH='Vulva', MS='The external genitalia of the female. It includes the CLITORIS, the labia, the vestibule, and its glands.'),
  'D014845': NtO(MH='Vulvar Diseases', MS='Pathological processes of the VULVA.'),
  'D014846': NtO(MH='Vulvar Neoplasms', MS='Tumors or cancer of the VULVA.'),
  'D014847': NtO(MH='Vulvitis', MS='Inflammation of the VULVA. It is characterized by PRURITUS and painful urination.'),
  'D014848': NtO(MH='Vulvovaginitis', MS='Inflammation of the VULVA and the VAGINA, characterized by discharge, burning, and PRURITUS.'),
  'D014849': NtO(MH='Waardenburg Syndrome', MS="Rare, autosomal dominant disease with variable penetrance and several known clinical types. Characteristics may include depigmentation of the hair and skin, congenital deafness, heterochromia iridis, medial eyebrow hyperplasia, hypertrophy of the nasal root, and especially dystopia canthorum. The underlying cause may be defective development of the neural crest (neurocristopathy). Waardenburg's syndrome may be closely related to piebaldism. Klein-Waardenburg Syndrome refers to a disorder that also includes upper limb abnormalities."),
  'D014850': NtO(MH='Waiting Lists', MS='Prospective patient listings for appointments or treatments.'),
  'D014851': NtO(MH='Wakefulness', MS='A state in which there is an enhanced potential for sensitivity and an efficient responsiveness to external stimuli.'),
  'D014852': NtO(MH='Wales', MS='A region of the United Kingdom, in the southwestern area of Great Britain.'),
  'D014853': NtO(MH='Walkers', MS='Walking aids generally having two handgrips and four legs.'),
  'D014854': NtO(MH='Lateral Medullary Syndrome', MS='INFARCTION of the dorsolateral aspect of MEDULLA OBLONGATA in the BRAIN STEM. It is caused by occlusion of the VERTEBRAL ARTERY and/or the posterior inferior cerebellar artery. Clinical manifestations vary with the size of infarction, but may include loss of pain and temperature sensation in the ipsilateral face and contralateral body below the chin; ipsilateral HORNER SYNDROME; ipsilateral ATAXIA; DYSARTHRIA; VERTIGO; nausea, hiccup; dysphagia; and VOCAL CORD PARALYSIS. (From Adams et al., Principles of Neurology, 6th ed, p801)'),
  'D014855': NtO(MH='Wallerian Degeneration', MS='Degeneration of distal aspects of a nerve axon following injury to the cell body or proximal portion of the axon. The process is characterized by fragmentation of the axon and its MYELIN SHEATH.'),
  'D014856': NtO(MH='Walruses', MS='The family Odobenidae, suborder PINNIPEDIA, order CARNIVORA. It is represented by a single species of large, nearly hairless mammal found on Arctic shorelines, whose upper canines are modified into tusks.'),
  'D014857': NtO(MH='Warfare', MS='Methods of conducting ARMED CONFLICTS.'),
  'D014858': NtO(MH='War Crimes', MS='Criminal acts committed during, or in connection with, war, e.g., maltreatment of prisoners, willful killing of civilians, etc.'),
  'D014859': NtO(MH='Warfarin', MS='An anticoagulant that acts by inhibiting the synthesis of vitamin K-dependent coagulation factors. Warfarin is indicated for the prophylaxis and/or treatment of venous thrombosis and its extension, pulmonary embolism, and atrial fibrillation with embolization. It is also used as an adjunct in the prophylaxis of systemic embolism after myocardial infarction. Warfarin is also used as a rodenticide.'),
  'D014860': NtO(MH='Warts', MS='Benign epidermal proliferations or tumors; some are viral in origin.'),
  'D014861': NtO(MH='Washington', MS='State bounded by on the north by Canada, on the east by Idaho, on the south by Oregon, and on the west by the Pacific Ocean.'),
  'D014862': NtO(MH='Wasp Venoms', MS='Venoms produced by the wasp (Vespid) family of stinging insects, including hornets; the venoms contain enzymes, biogenic amines, histamine releasing factors, kinins, toxic polypeptides, etc., and are similar to bee venoms.'),
  'D014863': NtO(MH='Wasps', MS='Any of numerous winged hymenopterous insects of social as well as solitary habits and having formidable stings.'),
  'D014865': NtO(MH='Waste Disposal, Fluid', MS='The discarding or destroying of liquid waste products or their transformation into something useful or innocuous.'),
  'D014866': NtO(MH='Waste Products', MS="Debris resulting from a process that is of no further use to the system producing it. The concept includes materials discharged from or stored in a system in inert form as a by-product of vital activities. (From Webster's New Collegiate Dictionary, 1981)"),
  'D014867': NtO(MH='Water', MS='A clear, odorless, tasteless liquid that is essential for most animal and plant life and is an excellent solvent for many substances. The chemical formula is hydrogen oxide (H2O). (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D014868': NtO(MH='Water Deprivation', MS='The withholding of water in a structured experimental situation.'),
  'D014869': NtO(MH='Water Intoxication', MS='A condition resulting from the excessive retention of water with sodium depletion.'),
  'D014870': NtO(MH='Water Loss, Insensible', MS='Loss of water by diffusion through the skin and by evaporation from the respiratory tract.'),
  'D014871': NtO(MH='Water Microbiology', MS='The presence of bacteria, viruses, and fungi in water. This term is not restricted to pathogenic organisms.'),
  'D014872': NtO(MH='Water Movements', MS='The flow of water in enviromental bodies of water such as rivers, oceans, water supplies, aquariums, etc. It includes currents, tides, and waves.'),
  'D014873': NtO(MH='Water Pollutants', MS='Substances or organisms which pollute the water or bodies of water. Use for water pollutants in general or those for which there is no specific heading.'),
  'D014874': NtO(MH='Water Pollutants, Chemical', MS='Chemical compounds which pollute the water of rivers, streams, lakes, the sea, reservoirs, or other bodies of water.'),
  'D014875': NtO(MH='Water Pollutants, Radioactive', MS='Pollutants, present in water or bodies of water, which exhibit radioactivity.'),
  'D014876': NtO(MH='Water Pollution', MS='Contamination of bodies of water (such as LAKES; RIVERS; SEAS; and GROUNDWATER.)'),
  'D014877': NtO(MH='Water Pollution, Chemical', MS='Adverse effect upon bodies of water (LAKES; RIVERS; seas; groundwater etc.) caused by CHEMICAL WATER POLLUTANTS.'),
  'D014878': NtO(MH='Water Pollution, Radioactive', MS='Contamination of water with radioactive materials and/or substances.'),
  'D014880': NtO(MH='Water Softening', MS='Processes by which minerals such as calcium, magnesium, and certain metal cations are removed from hard water, making it more compatible with soap and safer on appliances and  plumbing.'),
  'D014881': NtO(MH='Water Supply', MS='Means or process of supplying water (as for a community) usually including reservoirs, tunnels, and pipelines and often the watershed from which the water is ultimately drawn. (Webster, 3d ed)'),
  'D014882': NtO(MH='Water-Electrolyte Balance', MS='The balance of fluid in the BODY FLUID COMPARTMENTS; total BODY WATER; BLOOD VOLUME; EXTRACELLULAR SPACE; INTRACELLULAR SPACE, maintained by processes in the body that regulate the intake and excretion of WATER and ELECTROLYTES, particularly SODIUM and POTASSIUM.'),
  'D014883': NtO(MH='Water-Electrolyte Imbalance', MS="Disturbances in the body's WATER-ELECTROLYTE BALANCE."),
  'D014884': NtO(MH='Waterhouse-Friderichsen Syndrome', MS='A condition of HEMORRHAGE and NECROSIS of the ADRENAL GLAND. It is characterized by rapidly developing ADRENAL INSUFFICIENCY; HYPOTENSION; and widespread cutaneous PURPURA.'),
  'D014885': NtO(MH='Waxes', MS='A plastic substance deposited by insects or obtained from plants. Waxes are esters of various fatty acids with higher, usually monohydric alcohols. The wax of pharmacy is principally yellow wax (beeswax), the material of which honeycomb is made. It consists chiefly of cerotic acid and myricin and is used in making ointments, cerates, etc. (Dorland, 27th ed)'),
  'D014886': NtO(MH='Weaning', MS='Permanent deprivation of breast milk and commencement of nourishment with other food. (From Stedman, 25th ed)'),
  'D014887': NtO(MH='Weather', MS='The state of the ATMOSPHERE over minutes to months.'),
  'D014888': NtO(MH='Wechsler Scales', MS='Tests designed to measure intellectual functioning in children and adults.'),
  'D014890': NtO(MH='Granulomatosis with Polyangiitis', MS='A multisystemic disease of a complex genetic background. It is characterized by inflammation of the blood vessels (VASCULITIS) leading to damage in any number of organs. The common features include granulomatous inflammation of the RESPIRATORY TRACT and KIDNEYS. Most patients have measurable autoantibodies (ANTINEUTROPHIL CYTOPLASMIC ANTIBODIES) against MYELOBLASTIN.'),
  'D014891': NtO(MH='Weight Lifting', MS='A sport in which weights are lifted competitively or as an exercise.'),
  'D014892': NtO(MH='Weight Perception', MS='Recognition and discrimination of the heaviness of a lifted object.'),
  'D014893': NtO(MH='Weightlessness', MS='Condition in which no acceleration, whether due to gravity or any other force, can be detected by an observer within a system. It also means the absence of weight or the absence of the force of gravity acting on a body. Microgravity, gravitational force between 0 and 10 -6 g, is included here. (From NASA Thesaurus, 1988)'),
  'D014894': NtO(MH='Weights and Measures', MS='Measuring and weighing systems and processes.'),
  'D014895': NtO(MH='Weil Disease', MS='A severe form of LEPTOSPIROSIS, usually caused by LEPTOSPIRA INTERROGANS SEROVAR ICTEROHAEMORRHAGIAE and occasionally other serovars. It is transmitted to humans by the rat and is characterized by hemorrhagic and renal symptoms with accompanying JAUNDICE.'),
  'D014896': NtO(MH='Welding', MS='Joining metal pieces or parts together by melting the adjoining surfaces with heat, using a heat source such as a blowtorch or electric arc. The melted surfaces are then pressed together by hammering.'),
  'D014897': NtO(MH='Spinal Muscular Atrophies of Childhood', MS='A group of recessive inherited diseases that feature progressive muscular atrophy and hypotonia. They are classified as type I (Werdnig-Hoffman disease), type II (intermediate form), and type III (Kugelberg-Welander disease). Type I is fatal in infancy, type II has a late infantile onset and is associated with survival into the second or third decade. Type III has its onset in childhood, and is slowly progressive. (J Med Genet 1996 Apr:33(4):281-3)'),
  'D014898': NtO(MH='Werner Syndrome', MS='An autosomal recessive disorder that causes premature aging in adults, characterized by sclerodermal skin changes, cataracts, subcutaneous calcification, muscular atrophy, a tendency to diabetes mellitus, aged appearance of the face, baldness, and a high incidence of neoplastic disease.'),
  'D014899': NtO(MH='Wernicke Encephalopathy', MS='An acute neurological disorder characterized by the triad of ophthalmoplegia, ataxia, and disturbances of mental activity or consciousness. Eye movement abnormalities include nystagmus, external rectus palsies, and reduced conjugate gaze. THIAMINE DEFICIENCY and chronic ALCOHOLISM are associated conditions. Pathologic features include periventricular petechial hemorrhages and neuropil breakdown in the diencephalon and brainstem. Chronic thiamine deficiency may lead to KORSAKOFF SYNDROME. (Adams et al., Principles of Neurology, 6th ed, pp1139-42; Davis & Robertson, Textbook of Neuropathology, 2nd ed, pp452-3)'),
  'D014900': NtO(MH='West Indies', MS="Islands lying between southeastern North America and northern South America, enclosing the Caribbean Sea. They comprise the Greater Antilles (CUBA; DOMINICAN REPUBLIC; HAITI; JAMAICA; and PUERTO RICO), the Lesser Antilles (ANTIGUA AND BARBUDA and the other Leeward Islands, BARBADOS; MARTINIQUE and the other Windward Islands, NETHERLANDS ANTILLES; VIRGIN ISLANDS OF THE UNITED STATES, BRITISH VIRGINI ISLANDS, and the islands north of Venezuela which include TRINIDAD AND TOBAGO), and the BAHAMAS. (From Webster's New Geographical Dictionary, 1988, p1330)"),
  'D014901': NtO(MH='West Nile Fever', MS='A mosquito-borne viral illness caused by the WEST NILE VIRUS, a FLAVIVIRUS and endemic to regions of Africa, Asia, and Europe. Common clinical features include HEADACHE; FEVER; maculopapular rash; gastrointestinal symptoms; and lymphadenopathy. MENINGITIS; ENCEPHALITIS; and MYELITIS may also occur. The disease may occasionally be fatal or leave survivors with residual neurologic deficits. (From Joynt, Clinical Neurology, 1996, Ch26, p13; Lancet 1998 Sep 5;352(9130):767-71)'),
  'D014902': NtO(MH='West Nile virus', MS='A species of FLAVIVIRUS, one of the Japanese encephalitis virus group (ENCEPHALITIS VIRUSES, JAPANESE). It can infect birds and mammals. In humans, it is seen most frequently in Africa, Asia, and Europe presenting as a silent infection or undifferentiated fever (WEST NILE FEVER). The virus appeared in North America for the first time in 1999. It is transmitted mainly by CULEX spp mosquitoes which feed primarily on birds, but it can also be carried by the Asian Tiger mosquito, AEDES albopictus, which feeds mainly on mammals.'),
  'D014903': NtO(MH='West Virginia', MS='State bounded on the north by Ohio, Maryland, and Pennsylvania, bounded on the west by Ohio and Kentucky, on the east and south by Virginia.'),
  'D014904': NtO(MH='Western Australia', MS="A state in western Australia. Its capital is Perth. It was first visited by the Dutch in 1616 but the English took possession in 1791 and permanent colonization began in 1829. It was a penal settlement 1850-1888, became part of the colonial government in 1886, and was granted self government in 1890. (From Webster's New Geographical Dictionary, 1988, p1329)"),
  'D014905': NtO(MH='Independent State of Samoa', MS="An island group and constitutional monarchy in the southwest central Pacific Ocean. The capital is Apia. The islands were jointly administered by England, the United States, and Germany 1889-99, with the chief islands of Savai'i and Upolu recognized as German until 1919. Western Samoa gained independence in 1962 and assumed its present formal name in 1997."),
  'D014906': NtO(MH='Wetting Agents', MS='A surfactant that renders a surface wettable by water or enhances the spreading of water over the surface.'),
  'D014907': NtO(MH='Whales', MS='Large marine mammals of the order CETACEA. In the past, they were commercially valued for whale oil, for their flesh as human food and in ANIMAL FEED and FERTILIZERS, and for baleen. Today, there is a moratorium on most commercial whaling, as all species are either listed as endangered or threatened.'),
  'D014908': NtO(MH='Triticum', MS='A plant genus of the family POACEAE that is the source of EDIBLE GRAIN. A hybrid with rye (SECALE CEREALE) is called TRITICALE. The seed is ground into FLOUR and used to make BREAD, and is the source of WHEAT GERM AGGLUTININS.'),
  'D014909': NtO(MH='Wheat Germ Agglutinins', MS='Lectins purified from the germinating seeds of common wheat (Triticum vulgare); these bind to certain carbohydrate moieties on cell surface glycoproteins and are used to identify certain cell populations and inhibit or promote some immunological or physiological activities. There are at least two isoforms of this lectin.'),
  'D014910': NtO(MH='Wheelchairs', MS='Chairs mounted on wheels and designed to be propelled by the occupant.'),
  'D014911': NtO(MH='Whiplash Injuries', MS='Hyperextension injury to the neck, often the result of being struck from behind by a fast-moving vehicle, in an automobile accident. (From Segen, The Dictionary of Modern Medicine, 1992)'),
  'D014912': NtO(MH='White Muscle Disease', MS='A myodegeneration most frequent in calves and lambs whose dams have been fed during gestation or longer on feeds, especially legumes (FABACEAE), grown in certain areas where selenium is either deficient or unavailable in the soil. It has been recorded in many countries. It has been produced experimentally in several species of animals on low-selenium intake. A similar myopathy occurs naturally in goats, deer, foals, and dogs but proof of the etiology is lacking. (Merck Veterinary Manual, 5th ed)'),
  'D014914': NtO(MH='Whole Blood Coagulation Time', MS='The time required by whole blood to produce a visible clot.'),
  'D014915': NtO(MH='Whole-Body Counting', MS='Measurement of radioactivity in the entire human body.'),
  'D014916': NtO(MH='Whole-Body Irradiation', MS='Irradiation of the whole body with ionizing or non-ionizing radiation. It is applicable to humans or animals but not to microorganisms.'),
  'D014917': NtO(MH='Whooping Cough', MS='A respiratory infection caused by BORDETELLA PERTUSSIS and characterized by paroxysmal coughing ending in a prolonged crowing intake of breath.'),
  'D014918': NtO(MH='Wills', MS="Legal documents that are declarations of individuals' wishes regarding the disposal of their property or estate after death; esp: written instruments, legally executed, by which dispositions are made of estates. LIVING WILLS are written declarations regarding prolongation of life by extraordinary means."),
  'D014919': NtO(MH='Wind', MS="The motion of air relative to the earth's surface."),
  'D014920': NtO(MH='Wine', MS='Fermented juice of fresh grapes or of other fruit or plant products used as a beverage.'),
  'D014921': NtO(MH='Wings, Animal', MS='Movable feathered or membranous paired appendages by means of which certain animals such as birds, bats, or insects are able to fly.'),
  'D014922': NtO(MH='Wisconsin', MS='State bounded on the north by Lake Superior and Michigan, on the east by Lake Michigan, on the south by Illinois and Iowa, and on the west by Minnesota and Iowa.'),
  'D014923': NtO(MH='Wiskott-Aldrich Syndrome', MS='A rare, X-linked immunodeficiency syndrome characterized by ECZEMA; LYMPHOPENIA; and, recurrent pyogenic infection. It is seen exclusively in young boys. Typically, IMMUNOGLOBULIN M levels are low and IMMUNOGLOBULIN A and IMMUNOGLOBULIN E levels are elevated. Lymphoreticular malignancies are common.'),
  'D014924': NtO(MH="Wissler's Syndrome", MS='A rheumatic syndrome of possibly allergic origin, usually affecting children and adolescents, and characterized by high fever, exanthema, arthralgia, leukocytosis, and increased sedimentation rate.'),
  'D014925': NtO(MH='Wit and Humor as Topic', MS='The comical expression of connections between ideas that awaken amusement and pleasure.'),
  'D014927': NtO(MH='Wolff-Parkinson-White Syndrome', MS='A form of ventricular pre-excitation characterized by a short PR interval and a long QRS interval with a delta wave. In this syndrome, atrial impulses are abnormally conducted to the HEART VENTRICLES via an ACCESSORY CONDUCTING PATHWAY that is located between the wall of the right or left atria and the ventricles, also known as a BUNDLE OF KENT. The inherited form can be caused by mutation of PRKAG2 gene encoding a gamma-2 regulatory subunit of AMP-activated protein kinase.'),
  'D014928': NtO(MH='Wolffian Ducts', MS='A pair of excretory ducts of the middle kidneys (MESONEPHROI) of an embryo, also called mesonephric ducts. In higher vertebrates, Wolffian ducts persist in the male forming VAS DEFERENS, but atrophy into vestigial structures in the female.'),
  'D014929': NtO(MH='Wolfram Syndrome', MS='A hereditary condition characterized by multiple symptoms including those of DIABETES INSIPIDUS; DIABETES MELLITUS; OPTIC ATROPHY; and DEAFNESS. This syndrome is also known as DIDMOAD (first letter of each word) and is usually associated with VASOPRESSIN deficiency. It is caused by mutations in gene WFS1 encoding wolframin, a 100-kDa transmembrane protein.'),
  'D014930': NtO(MH='Women', MS='Human females as cultural, psychological, sociological, political, and economic entities.'),
  'D014931': NtO(MH='Women, Working', MS='Women who are engaged in gainful activities usually outside the home.'),
  'D014932': NtO(MH="Women's Health Services", MS='Organized services to provide health care to women. It excludes maternal care services for which MATERNAL HEALTH SERVICES is available.'),
  'D014933': NtO(MH="Women's Rights", MS='The rights of women to equal status pertaining to social, economic, and educational opportunities afforded by society.'),
  'D014934': NtO(MH='Wood', MS='A product of hard secondary xylem composed of CELLULOSE, hemicellulose, and LIGNANS, that is under the bark of trees and shrubs. It is used in construction and as a source of CHARCOAL and many other products.'),
  'D014935': NtO(MH='Wool', MS='The hair of SHEEP or other animals that is used for weaving or felting.'),
  'D014936': NtO(MH='Word Association Tests', MS='Lists of words to which individuals are asked to respond ascertaining the conceptual meaning held by the individual.'),
  'D014937': NtO(MH='Work', MS='Productive or purposeful activities.'),
  'D014938': NtO(MH='Work Capacity Evaluation', MS='Assessment of physiological capacities in relation to job requirements. It is usually done by measuring certain physiological (e.g., circulatory and respiratory) variables during a gradually increasing workload until specific limitations occur with respect to those variables.'),
  'D014939': NtO(MH='Work of Breathing', MS='RESPIRATORY MUSCLE contraction during INHALATION. The work is accomplished in three phases: LUNG COMPLIANCE work, that required to expand the LUNGS against its elastic forces; tissue resistance work, that required to overcome the viscosity of the lung and chest wall structures; and AIRWAY RESISTANCE work, that required to overcome airway resistance during the movement of air into the lungs. Work of breathing does not refer to expiration, which is entirely a passive process caused by elastic recoil of the lung and chest cage. (Guyton, Textbook of Medical Physiology, 8th ed, p406)'),
  'D014940': NtO(MH='Work Schedule Tolerance', MS='Physiological or psychological effects of periods of work which may be fixed or flexible such as flexitime, work shifts, and rotating shifts.'),
  'D014941': NtO(MH='Work Simplification', MS='The construction or arrangement of a task so that it may be done with the greatest possible efficiency.'),
  'D014942': NtO(MH="Workers' Compensation", MS='Insurance coverage providing compensation and medical benefits to individuals because of work-connected injuries or disease.'),
  'D014943': NtO(MH='Global Health', MS='A multi- and interdisciplinary field concerned with improving health and achieving equity in health for all people. It transcends national boundaries, promotes cooperation and collaboration within and beyond health science fields, and combines population-based disease prevention with individually-based patient care.'),
  'D014944': NtO(MH='World Health Organization', MS='A specialized agency of the United Nations designed as a coordinating authority on international health work; its aim is to promote the attainment of the highest possible level of health by all peoples.'),
  'D014945': NtO(MH='Wound Healing', MS='Restoration of integrity to traumatized tissue.'),
  'D014946': NtO(MH='Wound Infection', MS='Invasion of a wound by pathogenic microorganisms.'),
  'D014947': NtO(MH='Wounds and Injuries', MS='Damage inflicted on the body as the direct or indirect result of an external force, with or without disruption of structural continuity.'),
  'D014948': NtO(MH='Wounds, Gunshot', MS='Disruption of structural continuity of the body as a result of the discharge of firearms.'),
  'D014949': NtO(MH='Wounds, Nonpenetrating', MS='Injuries caused by impact with a blunt object where there is no penetration of the skin.'),
  'D014950': NtO(MH='Wounds, Penetrating', MS='Wounds caused by objects penetrating the skin.'),
  'D014951': NtO(MH='Wounds, Stab', MS='Penetrating wounds caused by a pointed object.'),
  'D014952': NtO(MH='Wrestling', MS="A sport consisting of hand-to-hand combat between two unarmed contestants seeking to pin or press each other's shoulders to the ground."),
  'D014953': NtO(MH='Wrist', MS='The region of the upper limb between the metacarpus and the FOREARM.'),
  'D014954': NtO(MH='Wrist Injuries', MS='Injuries to the wrist or the wrist joint.'),
  'D014955': NtO(MH='Wrist Joint', MS='The joint that is formed by the distal end of the RADIUS, the articular disc of the distal radioulnar joint, and the proximal row of CARPAL BONES; (SCAPHOID BONE; LUNATE BONE; triquetral bone).'),
  'D014956': NtO(MH='Writing', MS='The act or practice of literary composition, the occupation of writer, or producing or engaging in literary work as a profession.'),
  'D014957': NtO(MH='Wuchereria', MS='A genus of filarial nematodes.'),
  'D014958': NtO(MH='Wuchereria bancrofti', MS='A white threadlike worm which causes elephantiasis, lymphangitis, and chyluria by interfering with the lymphatic circulation. The microfilaria are found in the circulating blood and are carried by mosquitoes.'),
  'D014959': NtO(MH='Wyoming', MS='State bounded on the north by Montana, on the east by South Dakota and Nebraska, on the south by Utah and Colorado, and on the west by Idaho and Utah.'),
  'D014960': NtO(MH='X Chromosome', MS='The female sex chromosome, being the differential sex chromosome carried by half the male gametes and all female gametes in human and other male-heterogametic species.'),
  'D014961': NtO(MH='X-Ray Diffraction', MS='The scattering of x-rays by matter, especially crystals, with accompanying variation in intensity due to interference effects. Analysis of the crystal structure of materials is performed by passing x-rays through them and registering the diffraction image of the rays (CRYSTALLOGRAPHY, X-RAY). (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D014962': NtO(MH='X-Ray Film', MS='A film base coated with an emulsion designed for use with x-rays.'),
  'D014963': NtO(MH='X-Ray Intensifying Screens', MS='Screens which absorb the energy in the x-ray beam that has penetrated the patient and convert this energy into a light pattern which has as nearly as possible the same information as the original x-ray beam. The more light a screen produces for a given input of x-radiation, the less x-ray exposure and thus shorter exposure time are needed to expose the film. In most film-screen systems, the film is sandwiched between two screens in a cassette so that the emulsion on each side is exposed to the light from its contiguous screen.'),
  'D014964': NtO(MH='X-Ray Therapy', MS='Medical treatment involving the use of controlled amounts of X-Rays.'),
  'D014965': NtO(MH='X-Rays', MS='Penetrating electromagnetic radiation emitted when the inner orbital electrons of an atom are excited and release radiant energy. X-ray wavelengths range from 1 pm to 10 nm. Hard X-rays are the higher energy, shorter wavelength X-rays. Soft x-rays or Grenz rays are less energetic and longer in wavelength. The short wavelength end of the X-ray spectrum overlaps the GAMMA RAYS wavelength range. The distinction between gamma rays and X-rays is based on their radiation source.'),
  'D014966': NtO(MH='Xanthenes', MS='Compounds with three aromatic rings in linear arrangement with an OXYGEN in the center ring.'),
  'D014968': NtO(MH='Xanthine Dehydrogenase', MS='An enzyme that catalyzes the oxidation of XANTHINE in the presence of NAD+ to form URIC ACID and NADH. It acts also on a variety of other purines and aldehydes.'),
  'D014969': NtO(MH='Xanthine Oxidase', MS='An iron-molybdenum flavoprotein containing FLAVIN-ADENINE DINUCLEOTIDE that oxidizes hypoxanthine, some other purines and pterins, and aldehydes. Deficiency of the enzyme, an autosomal recessive trait, causes xanthinuria.'),
  'D014970': NtO(MH='Xanthines', MS='Purine bases found in body tissues and fluids and in some plants.'),
  'D014971': NtO(MH='Xanthinol Niacinate', MS='A vasodilator used in peripheral vascular disorders and insufficiency. It may cause gastric discomfort and hypotension.'),
  'D014972': NtO(MH='Xanthogranuloma, Juvenile', MS='Benign disorder of infants and children caused by proliferation of HISTIOCYTES, macrophages found in tissues. These histiocytes, usually lipid-laden non-Langerhans cells, form multiple yellow-red nodules most often in the skin, the eye, and sometimes in the viscera. Patients appear to have normal lipid metabolism and are classified as a normolipemic non-Langerhans cell histiocytosis.'),
  'D014973': NtO(MH='Xanthomatosis', MS='A condition marked by the development of widespread xanthomas, yellow tumor-like structures filled with lipid deposits. Xanthomas can be found in a variety of tissues including the SKIN; TENDONS; joints of KNEES and ELBOWS. Xanthomatosis is associated with disturbance of LIPID METABOLISM and formation of FOAM CELLS.'),
  'D014974': NtO(MH='Xanthomonas', MS='A genus in the family XANTHOMONADACEAE whose cells produce a yellow pigment (Gr. xanthos - yellow). It is pathogenic to plants.'),
  'D014975': NtO(MH='Lutein', MS='A xanthophyll found in the major LIGHT-HARVESTING PROTEIN COMPLEXES of plants. Dietary lutein accumulates in the MACULA LUTEA.'),
  'D014976': NtO(MH='Xanthopterin', MS='2-Amino-1,5-dihydro-4,6-pteridinedione. Pigment first discovered in butterfly wings and widely distributed in plants and animals.'),
  'D014977': NtO(MH='Xanthurenates', MS=None),
  'D014978': NtO(MH='Xenon', MS="A noble gas with the atomic symbol Xe, atomic number 54, and atomic weight 131.30. It is found in the earth's atmosphere and has been used as an anesthetic."),
  'D014979': NtO(MH='Xenon Isotopes', MS='Stable xenon atoms that have the same atomic number as the element xenon, but differ in atomic weight. Xe-124, 126, 128-131, 134, and 136 are stable xenon isotopes.'),
  'D014980': NtO(MH='Xenon Radioisotopes', MS='Unstable isotopes of xenon that decay or disintegrate emitting radiation. Xe atoms with atomic weights 121-123, 125, 127, 133, 135, 137-145 are radioactive xenon isotopes.'),
  'D014981': NtO(MH='Xenopus', MS='An aquatic genus of the family, Pipidae, occurring in Africa and distinguished by having black horny claws on three inner hind toes.'),
  'D014982': NtO(MH='Xenopus laevis', MS='The commonest and widest ranging species of the clawed "frog" (Xenopus) in Africa. This species is used extensively in research. There is now a significant population in California derived from escaped laboratory animals.'),
  'D014983': NtO(MH='Xeroderma Pigmentosum', MS='A rare, pigmentary, and atrophic autosomal recessive disease. It is manifested as an extreme photosensitivity to ULTRAVIOLET RAYS as the result of a deficiency in the enzyme that permits excisional repair of ultraviolet-damaged DNA.'),
  'D014984': NtO(MH='Xeromammography', MS='Xeroradiography of the breast.'),
  'D014985': NtO(MH='Xerophthalmia', MS='Dryness of the eye surfaces caused by deficiency of tears or conjunctival secretions. It may be associated with vitamin A deficiency, trauma, or any condition in which the eyelids do not close completely.'),
  'D014986': NtO(MH='Xeroradiography', MS='A photoelectric method of recording an X-ray image on a coated metal plate, using low-energy photon beams, long exposure time and dry chemical developers.'),
  'D014987': NtO(MH='Xerostomia', MS='Decreased salivary flow.'),
  'D014988': NtO(MH='Xipamide', MS='A sulfamoylbenzamide analog of CLOPAMIDE. It is diuretic and saluretic with antihypertensive activity. It is bound to PLASMA PROTEINS, thus has a delayed onset and prolonged action.'),
  'D014989': NtO(MH='Xiphoid Bone', MS='The smallest and most inferior triangular protrusion of the STERNUM or breastbone that extends into the center of the RIBCAGE.'),
  'D014990': NtO(MH='Xylans', MS='Polysaccharides consisting of xylose units.'),
  'D014991': NtO(MH='Xylazine', MS='An adrenergic alpha-2 agonist used as a sedative, analgesic and centrally acting muscle relaxant in VETERINARY MEDICINE.'),
  'D014992': NtO(MH='Xylenes', MS='A family of isomeric, colorless aromatic hydrocarbon liquids, that contain the general formula C6H4(CH3)2. They are produced by the destructive distillation of coal or by the catalytic reforming of petroleum naphthenic fractions. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D014993': NtO(MH='Xylitol', MS='A five-carbon sugar alcohol derived from XYLOSE by reduction of the carbonyl group. It is as sweet as sucrose and used as a noncariogenic sweetener.'),
  'D014994': NtO(MH='Xylose', MS=None),
  'D014995': NtO(MH='Xylosidases', MS='A group of enzymes that catalyze the hydrolysis of alpha- or beta-xylosidic linkages. EC 3.2.1.8 catalyzes the endo-hydrolysis of 1,4-beta-D-xylosidic linkages; EC 3.2.1.32 catalyzes the endo-hydrolysis of 1,3-beta-D-xylosidic linkages; EC 3.2.1.37 catalyzes the exo-hydrolysis of 1,4-beta-D-linkages from the non-reducing termini of xylans; and EC 3.2.1.72 catalyzes the exo-hydrolysis of 1,3-beta-D-linkages from the non-reducing termini of xylans. Other xylosidases have been identified that catalyze the hydrolysis of alpha-xylosidic bonds.'),
  'D014996': NtO(MH='Xylulose', MS='A 5-carbon keto sugar.'),
  'D014997': NtO(MH='XYY Karyotype', MS='Abnormal genetic constitution in males characterized by an extra Y chromosome.'),
  'D014998': NtO(MH='Y Chromosome', MS='The male sex chromosome, being the differential sex chromosome carried by half the male gametes and none of the female gametes in humans and in some other male-heterogametic species in which the homologue of the X chromosome has been retained.'),
  'D014999': NtO(MH='Yaba monkey tumor virus', MS='The type species of YATAPOXVIRUS, a tumor-producing DNA virus discovered in monkeys in Yaba, Nigeria. It has been found to produce histiocytomas in monkeys and humans.'),
  'D015000': NtO(MH='Yawning', MS='An involuntary deep INHALATION with the MOUTH open, often accompanied by the act of stretching.'),
  'D015001': NtO(MH='Yaws', MS='A systemic non-venereal infection of the tropics caused by TREPONEMA PALLIDUM subspecies pertenue.'),
  'D015002': NtO(MH='Yeast, Dried', MS='The dry cells of any suitable strain of SACCHAROMYCES CEREVISIAE or CANDIDA. It can be obtained as a by-product from the brewing of beer or by growing on media not suitable for beer production. Dried yeast serves as a source of protein and VITAMIN B COMPLEX.'),
  'D015003': NtO(MH='Yeasts', MS="A general term for single-celled rounded fungi that reproduce by budding. Brewers' and bakers' yeasts are SACCHAROMYCES CEREVISIAE; therapeutic dried yeast is YEAST, DRIED."),
  'D015004': NtO(MH='Yellow Fever', MS='An acute infectious disease primarily of the tropics, caused by a virus and transmitted to man by mosquitoes of the genera Aedes and Haemagogus. The severe form is characterized by fever, HEMOLYTIC JAUNDICE, and renal damage.'),
  'D015005': NtO(MH='Yellow fever virus', MS='The type species of the FLAVIVIRUS genus. Principal vector transmission to humans is by AEDES spp. mosquitoes.'),
  'D015006': NtO(MH='Yemen', MS='A country in the Middle East, bordering the Arabian Sea, Gulf of Aden, and Red Sea, between Oman and Saudi Arabia. The capital is Sanaa.'),
  'D015007': NtO(MH='Yersinia', MS='A genus of gram-negative, facultatively anaerobic rod- to coccobacillus-shaped bacteria that occurs in a broad spectrum of habitats.'),
  'D015008': NtO(MH='Yersinia enterocolitica', MS='A species of the genus YERSINIA, isolated from both man and animal. It is a frequent cause of bacterial gastroenteritis in children.'),
  'D015009': NtO(MH='Yersinia Infections', MS='Infections with bacteria of the genus YERSINIA.'),
  'D015010': NtO(MH='Yersinia pestis', MS='The etiologic agent of PLAGUE in man, rats, ground squirrels, and other rodents.'),
  'D015011': NtO(MH='Yersinia pseudotuberculosis', MS='A human and animal pathogen causing mesenteric lymphadenitis, diarrhea, and bacteremia.'),
  'D015012': NtO(MH='Yersinia pseudotuberculosis Infections', MS='Infections with bacteria of the species YERSINIA PSEUDOTUBERCULOSIS.'),
  'D015013': NtO(MH='Yoga', MS='A major orthodox system of Hindu philosophy based on Sankhya (metaphysical dualism) but differing from it in being theistic and characterized by the teaching of raja-yoga as a practical method of liberating the self. It includes a system of exercises for attaining bodily or mental control and well-being with liberation of the self and union with the universal spirit. (From Webster, 3d ed)'),
  'D015014': NtO(MH='Yogurt', MS='A slightly acid milk food produced by fermentation due to the combined action of Lactobacillus acidophilus and Streptococcus thermophilus.'),
  'D015016': NtO(MH='Yohimbine', MS='A plant alkaloid with alpha-2-adrenergic blocking activity. Yohimbine has been used as a mydriatic and in the treatment of ERECTILE DYSFUNCTION.'),
  'D015017': NtO(MH='Yolk Sac', MS='The first of four extra-embryonic membranes to form during EMBRYOGENESIS. In REPTILES and BIRDS, it arises from endoderm and mesoderm to incorporate the EGG YOLK into the DIGESTIVE TRACT for nourishing the embryo. In placental MAMMALS, its nutritional function is vestigial; however, it is the source of INTESTINAL MUCOSA; BLOOD CELLS; and GERM CELLS. It is sometimes called the vitelline sac, which should not be confused with the VITELLINE MEMBRANE of the egg.'),
  'D015018': NtO(MH='Ytterbium', MS='An element of the rare earth family of metals. It has the atomic symbol Yb, atomic number 70, and atomic weight 173. Ytterbium has been used in lasers and as a portable x-ray source.'),
  'D015019': NtO(MH='Yttrium', MS='An element of the rare earth family of metals. It has the atomic symbol Y, atomic number 39, and atomic weight 88.91. In conjunction with other rare earths, yttrium is used as a phosphor in television receivers and is a component of the yttrium-aluminum garnet (YAG) lasers.'),
  'D015020': NtO(MH='Yttrium Isotopes', MS='Stable yttrium atoms that have the same atomic number as the element yttrium, but differ in atomic weight. Y-89 is the only naturally occurring stable isotope of yttrium.'),
  'D015021': NtO(MH='Yttrium Radioisotopes', MS='Unstable isotopes of yttrium that decay or disintegrate emitting radiation. Y atoms with atomic weights 82-88 and 90-96 are radioactive yttrium isotopes.'),
  'D015022': NtO(MH='Yugoslavia', MS='Created as the Kingdom of Serbs, Croats, and Slovenes in 1918. Yugoslavia became the official name in 1929. BOSNIA-HERZEGOVINA; CROATIA; and SLOVENIA formed independent countries 7 April 1992. Macedonia became independent 8 February 1994 as the Former Yugoslav Republic of Macedonia (MACEDONIA REPUBLIC).'),
  'D015023': NtO(MH='Democratic Republic of the Congo', MS='A republic in central Africa, east of the REPUBLIC OF THE CONGO, south of the CENTRAL AFRICAN REPUBLIC and north of ANGOLA and ZAMBIA. The capital is Kinshasa.'),
  'D015024': NtO(MH='Zambia', MS='A republic in southern Africa, south of DEMOCRATIC REPUBLIC OF THE CONGO and TANZANIA, and north of ZIMBABWE. Its capital is Lusaka. It was formerly called Northern Rhodesia.'),
  'D015025': NtO(MH='Zearalenone', MS='(S-(E))-3,4,5,6,8,10-Hexahydro-14,16-dihydroxy-3-methyl-1H-2-benzoxacyclotetradecin-1,7(8H)-dione. One of a group of compounds known under the general designation of resorcylic acid lactones. Cis, trans, dextro and levo forms have been isolated from the fungus Gibberella zeae (formerly Fusarium graminearum). They have estrogenic activity, cause toxicity in livestock as feed contaminant, and have been used as anabolic or estrogen substitutes.'),
  'D015026': NtO(MH='Zeatin', MS="An aminopurine factor in plant extracts that induces cell division. (Grant & Hackh's Chemical Dict, 5th ed)"),
  'D015027': NtO(MH='Zebrafish', MS='An exotic species of the family CYPRINIDAE, originally from Asia, that has been introduced in North America. Zebrafish is a model organism for drug assay and cancer research.'),
  'D015028': NtO(MH='Zein', MS='A group of alcohol-soluble seed storage proteins from the endosperm of corn.'),
  'D015029': NtO(MH='Zeranol', MS='A non-steroidal estrogen analog.'),
  'D015030': NtO(MH='Zimbabwe', MS='A republic in southern Africa, east of ZAMBIA and BOTSWANA and west of MOZAMBIQUE. Its capital is Harare. It was formerly called Rhodesia and Southern Rhodesia.'),
  'D015031': NtO(MH='Zimeldine', MS='One of the SEROTONIN UPTAKE INHIBITORS formerly used for depression but was withdrawn worldwide in September 1983 because of the risk of GUILLAIN-BARRE SYNDROME associated with its use. (From Martindale, The Extra Pharmacopoeia, 29th ed, p385)'),
  'D015032': NtO(MH='Zinc', MS='A metallic element of atomic number 30 and atomic weight 65.38. It is a necessary trace element in the diet, forming an essential part of many enzymes, and playing an important role in protein synthesis and in cell division. Zinc deficiency is associated with ANEMIA, short stature, HYPOGONADISM, impaired WOUND HEALING, and geophagia. It is known by the symbol Zn.'),
  'D015033': NtO(MH='Zinc Isotopes', MS='Stable zinc atoms that have the same atomic number as the element zinc, but differ in atomic weight. Zn-66-68, and 70 are stable zinc isotopes.'),
  'D015034': NtO(MH='Zinc Oxide', MS='A mild astringent and topical protectant with some antiseptic action. It is also used in bandages, pastes, ointments, dental cements, and as a sunblock.'),
  'D015035': NtO(MH='Zinc Oxide-Eugenol Cement', MS="Used as a dental cement this is mainly zinc oxide (with strengtheners and accelerators) and eugenol. (Boucher's Clinical Dental Terminology, 4th ed, p50)"),
  'D015036': NtO(MH='Zinc Phosphate Cement', MS="A material used for cementation of inlays, crowns, bridges, and orthodontic appliances and occasionally as a temporary restoration. It is prepared by mixing zinc oxide and magnesium oxide powders with a liquid consisting principally of phosphoric acid, water, and buffers. (From Bouchers' Clinical Dental Terminology, 3d ed)"),
  'D015037': NtO(MH='Zinc Radioisotopes', MS='Unstable isotopes of zinc that decay or disintegrate emitting radiation. Zn atoms with atomic weights 60-63, 65, 69, 71, and 72 are radioactive zinc isotopes.'),
  'D015038': NtO(MH='Zineb', MS='An agricultural fungicide of the dithiocarbamate class. It has relatively low toxicity and there is little evidence of human injury from exposure.'),
  'D015039': NtO(MH='Ziram', MS='An industrial fungicide with low mammalian toxicity, although it does possess an irritant capacity for skin and mucous membranes.'),
  'D015040': NtO(MH='Zirconium', MS='A rather rare metallic element with atomic number 40, atomic weight 91.224, and symbol Zr.'),
  'D015041': NtO(MH='Zolazepam', MS='A pyrazolodiazepinone with pharmacological actions similar to ANTI-ANXIETY AGENTS. It is commonly used in combination with TILETAMINE to obtain immobilization and anesthesia in animals.'),
  'D015043': NtO(MH='Zollinger-Ellison Syndrome', MS='A syndrome that is characterized by the triad of severe PEPTIC ULCER, hypersecretion of GASTRIC ACID, and GASTRIN-producing tumors of the PANCREAS or other tissue (GASTRINOMA). This syndrome may be sporadic or be associated with MULTIPLE ENDOCRINE NEOPLASIA TYPE 1.'),
  'D015044': NtO(MH='Zona Pellucida', MS='A tough transparent membrane surrounding the OVUM. It is penetrated by the sperm during FERTILIZATION.'),
  'D015045': NtO(MH='Zoogloea', MS='A genus of RHODOCYCLACEAE occurring as rod-shaped cells embedded in a gelatinous matrix. It includes Z. filipendula.'),
  'D015046': NtO(MH='Zoology', MS='The study of animals - their morphology, growth, distribution, classification, and behavior.'),
  'D015047': NtO(MH='Zoonoses', MS='Diseases of non-human animals that may be transmitted to HUMANS or may be transmitted from humans to non-human animals.'),
  'D015048': NtO(MH='Zooplankton', MS='Minute free-floating animal organisms which live in practically all natural waters.'),
  'D015049': NtO(MH='Zoxazolamine', MS='A uricosuric and muscle relaxant. Zoxazolamine acts centrally as a muscle relaxant, but the mechanism of its action is not understood.'),
  'D015050': NtO(MH='Zygoma', MS='Either of a pair of bones that form the prominent part of the CHEEK and contribute to the ORBIT on each side of the SKULL.'),
  'D015051': NtO(MH='Zygomatic Fractures', MS='Fractures of the zygoma.'),
  'D015053': NtO(MH='Zygote', MS='The fertilized OVUM resulting from the fusion of a male and a female gamete.'),
  'D015054': NtO(MH='Zymosan', MS=None),
  'D015055': NtO(MH='1-Carboxyglutamic Acid', MS='Found in various tissues, particularly in four blood-clotting proteins including prothrombin, in kidney protein, in bone protein, and in the protein present in various ectopic calcifications.'),
  'D015056': NtO(MH='1-Methyl-3-isobutylxanthine', MS='A potent cyclic nucleotide phosphodiesterase inhibitor; due to this action, the compound increases cyclic AMP and cyclic GMP in tissue and thereby activates CYCLIC NUCLEOTIDE-REGULATED PROTEIN KINASES'),
  'D015057': NtO(MH='1-Naphthylamine', MS='A suspected industrial carcinogen (and listed as such by OSHA). Its N-hydroxy metabolite is strongly carcinogenic and mutagenic.'),
  'D015058': NtO(MH='1-Naphthylisothiocyanate', MS='A tool for the study of liver damage which causes bile stasis and hyperbilirubinemia acutely and bile duct hyperplasia and biliary cirrhosis chronically, with changes in hepatocyte function. It may cause skin and kidney damage.'),
  'D015059': NtO(MH='1-Sarcosine-8-Isoleucine Angiotensin II', MS='An ANGIOTENSIN II analog which acts as a highly specific inhibitor of ANGIOTENSIN TYPE 1 RECEPTOR.'),
  'D015060': NtO(MH='1,2-Dipalmitoylphosphatidylcholine', MS='Synthetic phospholipid used in liposomes and lipid bilayers to study biological membranes. It is also a major constituent of PULMONARY SURFACTANTS.'),
  'D015061': NtO(MH='1,4-alpha-Glucan Branching Enzyme', MS='In glycogen or amylopectin synthesis, the enzyme that catalyzes the transfer of a segment of a 1,4-alpha-glucan chain to a primary hydroxy group in a similar glucan chain. EC 2.4.1.18.'),
  'D015062': NtO(MH='11-Hydroxycorticosteroids', MS='A group of corticosteroids bearing a hydroxy group at the 11-position.'),
  'D015064': NtO(MH='16,16-Dimethylprostaglandin E2', MS='A synthetic prostaglandin E analog that protects the gastric mucosa, prevents ulceration, and promotes the healing of peptic ulcers. The protective effect is independent of acid inhibition. It is also a potent inhibitor of pancreatic function and growth of experimental tumors.'),
  'D015065': NtO(MH='17-Hydroxycorticosteroids', MS='A group of hydroxycorticosteroids bearing a hydroxy group at the 17-position. Urinary excretion of these compounds is used as an index of adrenal function. They are used systemically in the free alcohol form, but with esterification of the hydroxy groups, topical effectiveness is increased.'),
  'D015067': NtO(MH='17-Hydroxysteroid Dehydrogenases', MS='A class of enzymes that catalyzes the oxidation of 17-hydroxysteroids to 17-ketosteroids. EC 1.1.-.'),
  'D015068': NtO(MH='17-Ketosteroids', MS='Steroids that contain a ketone group at position 17.'),
  'D015069': NtO(MH='18-Hydroxycorticosterone', MS='11 beta,18,21-Trihydroxypregn-4-ene-3,20-dione.'),
  'D015070': NtO(MH='18-Hydroxydesoxycorticosterone', MS='An analog of desoxycorticosterone which is substituted by a hydroxyl group at the C-18 position.'),
  'D015071': NtO(MH='19-Iodocholesterol', MS="19-Iodocholest-5-en-3 beta-ol. A cholesterol derivative usually substituted with radioactive iodine in the 19 position. The compound is an adrenal cortex scanning agent used in the assessment of patients suspected of having Cushing's syndrome, hyperaldosteronism, pheochromocytoma and adrenal remnants following total adrenalectomy."),
  'D015072': NtO(MH='2-Acetolactate Mutase', MS='An enzyme involved in the biosynthesis of isoleucine and valine. It converts 2-acetolactate into 3-hydroxy-2-oxo-isovalerate. Also acts on 2-hydroxy-2-acetobutyrate to form 2-hydroxy-2-oxo-3-methylvalerate. EC 5.4.99.3.'),
  'D015073': NtO(MH='2-Acetylaminofluorene', MS='A hepatic carcinogen whose mechanism of activation involves N-hydroxylation to the aryl hydroxamic acid followed by enzymatic sulfonation to sulfoxyfluorenylacetamide. It is used to study the carcinogenicity and mutagenicity of aromatic amines.'),
  'D015074': NtO(MH='2-Aminoadipic Acid', MS='A metabolite in the principal biochemical pathway of lysine. It antagonizes neuroexcitatory activity modulated by the glutamate receptor, N-METHYL-D-ASPARTATE; (NMDA).'),
  'D015075': NtO(MH='2-Aminopurine', MS='A purine that is an isomer of ADENINE (6-aminopurine).'),
  'D015077': NtO(MH='2-Hydroxy-5-nitrobenzyl Bromide', MS="A chemical reagent that reacts with and modifies chemically the tryptophan portion of protein molecules. Used for 'active site' enzyme studies and other protein studies. Sometimes referred to as Koshland's reagent."),
  'D015078': NtO(MH='2-Hydroxyphenethylamine', MS='Simple amine found in the brain. It may be modulator of sympathetic functions. Its derivatives are adrenergic agonists and antagonists. It is also used in chemical industry.'),
  'D015079': NtO(MH='2-Isopropylmalate Synthase', MS='An enzyme that catalyzes the first step in the biosynthetic pathway to LEUCINE, forming isopropyl malate from acetyl-CoA and alpha-ketoisovaleric acid. This enzyme was formerly listed as EC 4.1.3.12.'),
  'D015080': NtO(MH='Mesna', MS='A sulfhydryl compound used to prevent urothelial toxicity by inactivating metabolites from ANTINEOPLASTIC AGENTS, such as IFOSFAMIDE or CYCLOPHOSPHAMIDE.'),
  'D015081': NtO(MH='2-Naphthylamine', MS='A naphthalene derivative with carcinogenic action.'),
  'D015082': NtO(MH="2,2'-Dipyridyl", MS='A reagent used for the determination of iron.'),
  'D015083': NtO(MH='2,3-Diketogulonic Acid', MS='Metabolite of ASCORBIC ACID and the oxidized form of the lactone DEHYDROASCORBIC ACID.'),
  'D015084': NtO(MH='2,4-Dichlorophenoxyacetic Acid', MS='An herbicide with irritant effects on the eye and the gastrointestinal system.'),
  'D015085': NtO(MH='2,4,5-Trichlorophenoxyacetic Acid', MS='An herbicide with strong irritant properties. Use of this compound on rice fields, orchards, sugarcane, rangeland, and other noncrop sites was terminated by the EPA in 1985. (From Merck Index, 11th ed)'),
  'D015086': NtO(MH='2,6-Dichloroindophenol', MS='A dye used as a reagent in the determination of vitamin C.'),
  'D015087': NtO(MH="2',3'-Cyclic-Nucleotide Phosphodiesterases", MS="Nucleoside-2',3'-cyclic phosphate nucleotidohydrolase. Enzymes that catalyze the hydrolysis of the 2'- or 3'- phosphate bonds of 2',3'-cyclic nucleotides. Also hydrolyzes nucleoside monophosphates. Includes EC 3.1.4.16 and EC 3.1.4.37. EC 3.1.4.-."),
  'D015088': NtO(MH="2',5'-Oligoadenylate Synthetase", MS="An enzyme that catalyzes the conversion of ATP into a series of (2'-5') linked oligoadenylates and pyrophosphate in the presence of double-stranded RNA. These oligonucleotides activate an endoribonuclease (RNase L) which cleaves single-stranded RNA. Interferons can act as inducers of these reactions."),
  'D015089': NtO(MH='20-Hydroxysteroid Dehydrogenases', MS='A group of enzymes that catalyze the reversible reduction-oxidation reaction of 20-hydroxysteroids, such as from a 20-ketosteroid to a 20-alpha-hydroxysteroid (EC 1.1.1.149) or to a 20-beta-hydroxysteroid (EC 1.1.1.53).'),
  'D015090': NtO(MH='25-Hydroxyvitamin D3 1-alpha-Hydroxylase', MS='A mitochondrial cytochrome P450 enzyme that catalyzes the 1-alpha-hydroxylation of 25-hydroxyvitamin D3 (also known as 25-hydroxycholecalciferol) in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme, encoded by CYP27B1 gene, converts 25-hydroxyvitamin D3 to 1-alpha,25-dihydroxyvitamin D3 which is the active form of VITAMIN D in regulating bone growth and calcium metabolism. This enzyme is also active on plant 25-hydroxyvitamin D2 (ergocalciferol).'),
  'D015091': NtO(MH='beta-Alanine', MS='An amino acid formed in vivo by the degradation of dihydrouracil and carnosine. Since neuronal uptake and neuronal receptor sensitivity to beta-alanine have been demonstrated, the compound may be a false transmitter replacing GAMMA-AMINOBUTYRIC ACID. A rare genetic disorder, hyper-beta-alaninemia, has been reported.'),
  'D015092': NtO(MH='3-Deazauridine', MS='4-Hydroxy-1-(beta-D-ribofuranosyl)-2-pyridinone. Analog of uridine lacking a ring-nitrogen in the 3-position. Functions as an antineoplastic agent.'),
  'D015093': NtO(MH='Meglutol', MS='An antilipemic agent which lowers cholesterol, triglycerides, serum beta-lipoproteins and phospholipids. It acts by interfering with the enzymatic steps involved in the conversion of acetate to hydroxymethylglutaryl coenzyme A as well as inhibiting the activity of HYDROXYMETHYLGLUTARYL COA REDUCTASES which is the rate limiting enzyme in the biosynthesis of cholesterol.'),
  'D015094': NtO(MH='3-Hydroxyacyl CoA Dehydrogenases', MS='Enzymes that reversibly catalyze the oxidation of a 3-hydroxyacyl CoA to 3-ketoacyl CoA in the presence of NAD. They are key enzymes in the oxidation of fatty acids and in mitochondrial fatty acid synthesis.'),
  'D015095': NtO(MH='3-Hydroxyanthranilic Acid', MS='An oxidation product of tryptophan metabolism. It may be a free radical scavenger and a carcinogen.'),
  'D015096': NtO(MH='3-Hydroxysteroid Dehydrogenases', MS='Catalyze the oxidation of 3-hydroxysteroids to 3-ketosteroids.'),
  'D015097': NtO(MH='3-Mercaptopropionic Acid', MS='An inhibitor of glutamate decarboxylase. It decreases the GAMMA-AMINOBUTYRIC ACID concentration in the brain, thereby causing convulsions.'),
  'D015098': NtO(MH='3-Methoxy-4-hydroxyphenylethanol', MS='Metabolite of serotonin and norepinephrine.'),
  'D015099': NtO(MH='3-Oxoacyl-(Acyl-Carrier-Protein) Synthase', MS='An enzyme of long-chain fatty acid synthesis, that adds a two-carbon unit from malonyl-(acyl carrier protein) to another molecule of fatty acyl-(acyl carrier protein), giving a beta-ketoacyl-(acyl carrier protein) with the release of carbon dioxide. EC 2.3.1.41.'),
  'D015100': NtO(MH="3,3'-Diaminobenzidine", MS='A chemically and thermodynamically stable derivative of BENZIDINE.'),
  'D015101': NtO(MH="3,3'-Dichlorobenzidine", MS='A material used in the manufacture of azo dyes that is toxic to skin and carcinogenic in several species.'),
  'D015102': NtO(MH='3,4-Dihydroxyphenylacetic Acid', MS='A deaminated metabolite of LEVODOPA.'),
  'D015103': NtO(MH='Droxidopa', MS='A synthetic precursor of norepinephrine that is used in the treatment of PARKINSONIAN DISORDERS and ORTHOSTATIC HYPOTENSION.'),
  'D015104': NtO(MH='3,4-Methylenedioxyamphetamine', MS='An amphetamine derivative that inhibits uptake of catecholamine neurotransmitters. It is a hallucinogen. It is less toxic than its methylated derivative but in sufficient doses may still destroy serotonergic neurons and has been used for that purpose experimentally.'),
  'D015105': NtO(MH="3',5'-Cyclic-AMP Phosphodiesterases", MS="Enzymes that catalyze the hydrolysis of CYCLIC AMP to form adenosine 5'-phosphate. The enzymes are widely distributed in animal tissue and control the level of intracellular cyclic AMP. Many specific enzymes classified under this heading demonstrate additional spcificity for 3',5'-cyclic IMP and CYCLIC GMP."),
  'D015106': NtO(MH="3',5'-Cyclic-GMP Phosphodiesterases", MS="Enzymes that catalyze the hydrolysis of cyclic GMP to yield guanosine-5'-phosphate."),
  'D015107': NtO(MH='4-Butyrolactone', MS='One of the FURANS with a carbonyl thereby forming a cyclic lactone. It is an endogenous compound made from gamma-aminobutyrate and is the precursor of gamma-hydroxybutyrate. It is also used as a pharmacological agent and solvent.'),
  'D015108': NtO(MH='4-Hydroxyaminoquinoline-1-oxide', MS='A potent mutagen and carcinogen. It is a reduction product of 4-NITROQUINOLINE-1-OXIDE. It binds with nucleic acids and inactivates both bacteria and bacteriophage.'),
  'D015109': NtO(MH='4-Hydroxybenzoate-3-Monooxygenase', MS='A flavoprotein that catalyzes the synthesis of protocatechuic acid from 4-hydroxybenzoate in the presence of molecular oxygen. EC 1.14.13.2.'),
  'D015110': NtO(MH='4-Hydroxycoumarins', MS='Substances found in many plants, containing the 4-hydroxycoumarin radical. They interfere with vitamin K and the blood clotting mechanism, are tightly protein-bound, inhibit mitochondrial and microsomal enzymes, and are used as oral anticoagulants.'),
  'D015111': NtO(MH='4-Hydroxyphenylpyruvate Dioxygenase', MS='An enzyme that catalyzes the conversion of 4-hydroxyphenylpyruvate plus oxygen to homogentisic acid and carbon dioxide. EC 1.13.11.27.'),
  'D015112': NtO(MH='4-Nitroquinoline-1-oxide', MS='A potent mutagen and carcinogen. This compound and its metabolite 4-HYDROXYAMINOQUINOLINE-1-OXIDE bind to nucleic acids. It inactivates bacteria but not bacteriophage.'),
  'D015113': NtO(MH='Androstane-3,17-diol', MS='The unspecified form of the steroid, normally a major metabolite of TESTOSTERONE with androgenic activity. It has been implicated as a regulator of gonadotropin secretion.'),
  'D015114': NtO(MH='Androstenediol', MS='An intermediate in TESTOSTERONE biosynthesis, found in the TESTIS or the ADRENAL GLANDS. Androstenediol, derived from DEHYDROEPIANDROSTERONE by the reduction of the 17-keto group (17-HYDROXYSTEROID DEHYDROGENASES), is converted to TESTOSTERONE by the oxidation of the 3-beta hydroxyl group to a 3-keto group (3-HYDROXYSTEROID DEHYDROGENASES).'),
  'D015115': NtO(MH='5,6-Dihydroxytryptamine', MS='Tryptamine substituted with two hydroxyl groups in positions 5 and 6. It is a neurotoxic serotonin analog that destroys serotonergic neurons preferentially and is used in neuropharmacologic research.'),
  'D015116': NtO(MH='5,7-Dihydroxytryptamine', MS='Tryptamine substituted with two hydroxyl groups in positions 5 and 7. It is a neurotoxic serotonin analog that destroys serotonergic neurons preferentially and is used in neuropharmacology as a tool.'),
  'D015117': NtO(MH='5,8,11,14-Eicosatetraynoic Acid', MS='A 20-carbon unsaturated fatty acid containing 4 alkyne bonds. It inhibits the enzymatic conversion of arachidonic acid to prostaglandins E(2) and F(2a).'),
  'D015118': NtO(MH='Eicosapentaenoic Acid', MS='Important polyunsaturated fatty acid found in fish oils. It serves as the precursor for the prostaglandin-3 and thromboxane-3 families. A diet rich in eicosapentaenoic acid lowers serum lipid concentration, reduces incidence of cardiovascular disorders, prevents platelet aggregation, and inhibits arachidonic acid conversion into the thromboxane-2 and prostaglandin-2 families.'),
  'D015119': NtO(MH='Aminocaproic Acid', MS='An antifibrinolytic agent that acts by inhibiting plasminogen activators which have fibrinolytic properties.'),
  'D015120': NtO(MH='6-Aminonicotinamide', MS='A vitamin antagonist which has teratogenic effects.'),
  'D015121': NtO(MH='6-Ketoprostaglandin F1 alpha', MS='The physiologically active and stable hydrolysis product of EPOPROSTENOL. Found in nearly all mammalian tissue.'),
  'D015122': NtO(MH='Mercaptopurine', MS='An antimetabolite antineoplastic agent with immunosuppressant properties. It interferes with nucleic acid synthesis by inhibiting purine metabolism and is used, usually in combination with other drugs, in the treatment of or in remission maintenance programs for leukemia.'),
  'D015123': NtO(MH='7,8-Dihydro-7,8-dihydroxybenzo(a)pyrene 9,10-oxide', MS='7,8,8a,9a-Tetrahydrobenzo(10,11)chryseno (3,4-b)oxirene-7,8-diol. A benzopyrene derivative with carcinogenic and mutagenic activity.'),
  'D015124': NtO(MH='8-Bromo Cyclic Adenosine Monophosphate', MS='A long-acting derivative of cyclic AMP. It is an activator of cyclic AMP-dependent protein kinase, but resistant to degradation by cyclic AMP phosphodiesterase.'),
  'D015125': NtO(MH='Oxyquinoline', MS='An antiseptic with mild fungistatic, bacteriostatic, anthelmintic, and amebicidal action. It is also used as a reagent and metal chelator, as a carrier for radio-indium for diagnostic purposes, and its halogenated derivatives are used in addition as topical anti-infective agents and oral antiamebics.'),
  'D015126': NtO(MH='8,11,14-Eicosatrienoic Acid', MS='A 20-carbon-chain fatty acid, unsaturated at positions 8, 11, and 14. It differs from arachidonic acid, 5,8,11,14-eicosatetraenoic acid, only at position 5.'),
  'D015127': NtO(MH='9,10-Dimethyl-1,2-benzanthracene', MS='Polycyclic aromatic hydrocarbon found in tobacco smoke that is a potent carcinogen.'),
  'D015138': NtO(MH='RNA, Nuclear', MS='RNA molecules found in the nucleus either associated with chromosomes or in the nucleoplasm.'),
  'D015139': NtO(MH='Blotting, Southern', MS='A method (first developed by E.M. Southern) for detection of DNA that has been electrophoretically separated and immobilized by blotting on nitrocellulose or other type of paper or nylon membrane followed by hybridization with labeled NUCLEIC ACID PROBES.'),
  'D015140': NtO(MH='Dementia, Vascular', MS='An imprecise term referring to dementia associated with CEREBROVASCULAR DISORDERS, including CEREBRAL INFARCTION (single or multiple), and conditions associated with chronic BRAIN ISCHEMIA. Diffuse, cortical, and subcortical subtypes have been described. (From Gerontol Geriatr 1998 Feb;31(1):36-44)'),
  'D015141': NtO(MH='Los Angeles', MS='City in California.'),
  'D015142': NtO(MH='Baltimore', MS='A city located in Maryland.'),
  'D015143': NtO(MH='Philadelphia', MS='City located in Pennsylvania.'),
  'D015144': NtO(MH='Southeastern United States', MS='The geographic area of the southeastern region of the United States in general or when the specific state or states are not included. The states usually included in this region are Alabama, Arkansas, Florida, Georgia, Louisiana, Mississippi, North Carolina, South Carolina, West Virginia, and Virginia.'),
  'D015145': NtO(MH='Tocolysis', MS='Any drug treatment modality designed to inhibit UTERINE CONTRACTION. It is used in pregnant women to arrest PREMATURE LABOR.'),
  'D015146': NtO(MH='Midwestern United States', MS='The geographic area of the midwestern region of the United States in general or when the specific state or states are not indicated. The states usually included in this region are Illinois, Indiana, Iowa, Kansas, Kentucky, Michigan, Minnesota, Missouri, Nebraska, Ohio, Oklahoma, North Dakota, South Dakota and Wisconsin.'),
  'D015148': NtO(MH='Cardiotocography', MS='Monitoring of FETAL HEART frequency before birth in order to assess impending prematurity in relation to the pattern or intensity of antepartum UTERINE CONTRACTION.'),
  'D015149': NtO(MH='Tocolytic Agents', MS='Drugs that prevent preterm labor and immature birth by suppressing uterine contractions (TOCOLYSIS). Agents used to delay premature uterine activity include magnesium sulfate, beta-mimetics, oxytocin antagonists, calcium channel inhibitors, and adrenergic beta-receptor agonists. The use of intravenous alcohol as a tocolytic is now obsolete.'),
  'D015150': NtO(MH='Echocardiography, Doppler', MS='Measurement of intracardiac blood flow using an M-mode and/or two-dimensional (2-D) echocardiogram while simultaneously recording the spectrum of the audible Doppler signal (e.g., velocity, direction, amplitude, intensity, timing) reflected from the moving column of red blood cells.'),
  'D015151': NtO(MH='Immunoblotting', MS='Immunologic method used for detecting or quantifying immunoreactive substances. The substance is identified by first immobilizing it by blotting onto a membrane and then tagging it with labeled antibodies.'),
  'D015152': NtO(MH='Blotting, Northern', MS='Detection of RNA that has been electrophoretically separated and immobilized by blotting on nitrocellulose or other type of paper or nylon membrane followed by hybridization with labeled NUCLEIC ACID PROBES.'),
  'D015153': NtO(MH='Blotting, Western', MS='Identification of proteins or peptides that have been electrophoretically separated by blot transferring from the electrophoresis gel to strips of nitrocellulose paper, followed by labeling with antibody probes.'),
  'D015154': NtO(MH='Esophageal Motility Disorders', MS='Disorders affecting the motor function of the UPPER ESOPHAGEAL SPHINCTER; LOWER ESOPHAGEAL SPHINCTER; the ESOPHAGUS body, or a combination of these parts. The failure of the sphincters to maintain a tonic pressure may result in gastric reflux of food and acid into the esophagus (GASTROESOPHAGEAL REFLUX). Other disorders include hypermotility (spastic disorders) and markedly increased amplitude in contraction (nutcracker esophagus).'),
  'D015155': NtO(MH='Esophageal Spasm, Diffuse', MS='A hypermotility disorder of the ESOPHAGUS that is characterized by spastic non-peristaltic responses to SWALLOWING; CHEST PAIN; and DYSPHAGIA.'),
  'D015160': NtO(MH='Hydrops Fetalis', MS='Abnormal accumulation of serous fluid in two or more fetal compartments, such as SKIN; PLEURA; PERICARDIUM; PLACENTA; PERITONEUM; AMNIOTIC FLUID. General fetal EDEMA may be of non-immunologic origin, or of immunologic origin as in the case of ERYTHROBLASTOSIS FETALIS.'),
  'D015161': NtO(MH='Dementia, Multi-Infarct', MS='Loss of higher cortical functions with retained awareness due to multiple cortical or subcortical CEREBRAL INFARCTION. Memory, judgment, attention span, and impulse control are often impaired, and may be accompanied by PSEUDOBULBAR PALSY; HEMIPARESIS; reflex abnormalities, and other signs of localized neurologic dysfunction. (From Adams et al., Principles of Neurology, 6th ed, p1060)'),
  'D015162': NtO(MH='Micronucleus Tests', MS='Induction and quantitative measurement of chromosomal damage leading to the formation of micronuclei (MICRONUCLEI, CHROMOSOME-DEFECTIVE) in cells which have been exposed to genotoxic agents or IONIZING RADIATION.'),
  'D015163': NtO(MH='Superinfection', MS='A frequent complication of drug therapy for microbial infection. It may result from opportunistic colonization following immunosuppression by the primary pathogen and can be influenced by the time interval between infections, microbial physiology, or host resistance. Experimental challenge and in vitro models are sometimes used in virulence and infectivity studies.'),
  'D015164': NtO(MH='Vaccines, Inactivated', MS='Vaccines in which the infectious microbial nucleic acid components have been destroyed by chemical or physical treatment (e.g., formalin, beta-propiolactone, gamma radiation) without affecting the antigenicity or immunogenicity of the viral coat or bacterial outer membrane proteins.'),
  'D015166': NtO(MH='Monitoring, Immunologic', MS='Testing of immune status in the diagnosis and therapy of cancer, immunoproliferative and immunodeficiency disorders, and autoimmune abnormalities. Changes in immune parameters are of special significance before, during and following organ transplantation. Strategies include measurement of tumor antigen and other markers (often by RADIOIMMUNOASSAY), studies of cellular or humoral immunity in cancer etiology, IMMUNOTHERAPY trials, etc.'),
  'D015167': NtO(MH='Vagotomy, Truncal', MS='Bilateral dissection of the abdominal branches of the vagus nerve. It is used frequently in the surgical management of duodenal and gastric ulcers, as well as in physiologic studies of gastrointestinal secretion and motility.'),
  'D015168': NtO(MH='Lightning Injuries', MS='Accidental injuries caused by brief high-voltage electrical discharges during thunderstorms. Cardiopulmonary arrest, coma and other neurologic symptoms, myocardial necrosis, and dermal burns are common. Prompt treatment of the acute sequelae, including cardiopulmonary resuscitation, is indicated for survival.'),
  'D015169': NtO(MH='Colony Count, Microbial', MS="Enumeration by direct count of viable, isolated bacterial, archaeal, or fungal CELLS or SPORES capable of growth on solid CULTURE MEDIA. The method is used routinely by environmental microbiologists for quantifying organisms in AIR; FOOD; and WATER; by clinicians for measuring patients' microbial load; and in antimicrobial drug testing."),
  'D015170': NtO(MH='Cerebrospinal Fluid Pressure', MS='Manometric pressure of the CEREBROSPINAL FLUID as measured by lumbar, cerebroventricular, or cisternal puncture. Within the cranial cavity it is called INTRACRANIAL PRESSURE.'),
  'D015171': NtO(MH='Ganglionectomy', MS='Removal of an autonomic or sensory ganglion by any means.'),
  'D015173': NtO(MH='Supratentorial Neoplasms', MS='Primary and metastatic (secondary) tumors of the brain located above the tentorium cerebelli, a fold of dura mater separating the CEREBELLUM and BRAIN STEM from the cerebral hemispheres and DIENCEPHALON (i.e., THALAMUS and HYPOTHALAMUS and related structures). In adults, primary neoplasms tend to arise in the supratentorial compartment, whereas in children they occur more frequently in the infratentorial space. Clinical manifestations vary with the location of the lesion, but SEIZURES; APHASIA; HEMIANOPSIA; hemiparesis; and sensory deficits are relatively common features. Metastatic supratentorial neoplasms are frequently multiple at the time of presentation.'),
  'D015174': NtO(MH='Epidural Neoplasms', MS='Neoplasms located in the space between the vertebral PERIOSTEUM and DURA MATER surrounding the SPINAL CORD. Tumors in this location are most often metastatic in origin and may cause neurologic deficits by mass effect on the spinal cord or nerve roots or by interfering with blood supply to the spinal cord.'),
  'D015175': NtO(MH='Prolactinoma', MS='A pituitary adenoma which secretes PROLACTIN, leading to HYPERPROLACTINEMIA. Clinical manifestations include AMENORRHEA; GALACTORRHEA; IMPOTENCE; HEADACHE; visual disturbances; and CEREBROSPINAL FLUID RHINORRHEA.'),
  'D015179': NtO(MH='Colorectal Neoplasms', MS='Tumors or cancer of the COLON or the RECTUM or both. Risk factors for colorectal cancer include chronic ULCERATIVE COLITIS; FAMILIAL POLYPOSIS COLI; exposure to ASBESTOS; and irradiation of the CERVIX UTERI.'),
  'D015180': NtO(MH='Electrophoresis, Gel, Two-Dimensional', MS='Electrophoresis in which a second perpendicular electrophoretic transport is performed on the separate components resulting from the first electrophoresis. This technique is usually performed on polyacrylamide gels.'),
  'D015181': NtO(MH='Gamete Intrafallopian Transfer', MS="A technique that came into use in the mid-1980's for assisted conception in infertile women with normal fallopian tubes. The protocol consists of hormonal stimulation of the ovaries, followed by laparoscopic follicular aspiration of oocytes, and then the transfer of sperm and oocytes by catheterization into the fallopian tubes."),
  'D015182': NtO(MH='Lymphatic Irradiation', MS="External or interstitial irradiation to treat lymphomas (e.g., Hodgkin's and non-Hodgkin's lymphomas) and lymph node metastases and also some autoimmune diseases, such as rheumatoid arthritis."),
  'D015183': NtO(MH='Restriction Mapping', MS='Use of restriction endonucleases to analyze and generate a physical map of genomes, genes, or other segments of DNA.'),
  'D015186': NtO(MH='Hominidae', MS='Family of the suborder HAPLORHINI (Anthropoidea) comprising bipedal primate MAMMALS. It includes modern man (HOMO SAPIENS) and the great apes: gorillas (GORILLA GORILLA), chimpanzees (PAN PANISCUS and PAN TROGLODYTES), and orangutans (PONGO PYGMAEUS).'),
  'D015187': NtO(MH='Lipectomy', MS='Removal of localized SUBCUTANEOUS FAT deposits by SUCTION CURETTAGE or blunt CANNULATION in the cosmetic correction of OBESITY and other esthetic contour defects.'),
  'D015190': NtO(MH='Blood Glucose Self-Monitoring', MS='Self evaluation of whole blood glucose levels outside the clinical laboratory. A digital or battery-operated reflectance meter may be used. It has wide application in controlling unstable insulin-dependent diabetes.'),
  'D015192': NtO(MH='Infratentorial Neoplasms', MS='Intracranial tumors originating in the region of the brain inferior to the tentorium cerebelli, which contains the cerebellum, fourth ventricle, cerebellopontine angle, brain stem, and related structures. Primary tumors of this region are more frequent in children, and may present with ATAXIA; CRANIAL NERVE DISEASES; vomiting; HEADACHE; HYDROCEPHALUS; or other signs of neurologic dysfunction. Relatively frequent histologic subtypes include TERATOMA; MEDULLOBLASTOMA; GLIOBLASTOMA; ASTROCYTOMA; EPENDYMOMA; CRANIOPHARYNGIOMA; and choroid plexus papilloma (PAPILLOMA, CHOROID PLEXUS).'),
  'D015193': NtO(MH='Chorionic Villi Sampling', MS='A method for diagnosis of fetal diseases by sampling the cells of the placental chorionic villi for DNA analysis, presence of bacteria, concentration of metabolites, etc. The advantage over amniocentesis is that the procedure can be carried out in the first trimester.'),
  'D015194': NtO(MH='Diffusion Chambers, Culture', MS='Devices used in a technique by which cells or tissues are grown in vitro or, by implantation, in vivo within chambers permeable to diffusion of solutes across the chamber walls. The chambers are used for studies of drug effects, osmotic responses, cytogenic and immunologic phenomena, metabolism, etc., and include tissue cages.'),
  'D015195': NtO(MH='Drug Design', MS='The molecular designing of drugs for specific purposes (such as DNA-binding, enzyme inhibition, anti-cancer efficacy, etc.) based on knowledge of molecular properties such as activity of functional groups, molecular geometry, and electronic structure, and also on information cataloged on analogous molecules. Drug design is generally computer-assisted molecular modeling and does not include pharmacokinetics, dosage analysis, or drug administration analysis.'),
  'D015197': NtO(MH='Carcinogenicity Tests', MS='Tests to experimentally measure the tumor-producing/cancer cell-producing potency of an agent by administering the agent (e.g., benzanthracenes) and observing the quantity of tumors or the cell transformation developed over a given period of time. The carcinogenicity value is usually measured as milligrams of agent administered per tumor developed. Though this test differs from the DNA-repair and bacterial microsome MUTAGENICITY TESTS, researchers often attempt to correlate the finding of carcinogenicity values and mutagenicity values.'),
  'D015198': NtO(MH='Designer Drugs', MS='Drugs designed and synthesized, often for illegal street use, by modification of existing drug structures (e.g., amphetamines). Of special interest are MPTP (a reverse ester of meperidine), MDA (3,4-methylenedioxyamphetamine), and MDMA (3,4-methylenedioxymethamphetamine). Many drugs act on the aminergic system, the physiologically active biogenic amines.'),
  'D015199': NtO(MH='Extracorporeal Membrane Oxygenation', MS='Application of a life support system that circulates the blood through an oxygenating system, which may consist of a pump, a membrane oxygenator, and a heat exchanger. Examples of its use are to assist victims of SMOKE INHALATION INJURY; RESPIRATORY FAILURE; and CARDIAC FAILURE.'),
  'D015200': NtO(MH='Fluoroimmunoassay', MS='The use of fluorescence spectrometry to obtain quantitative results for the FLUORESCENT ANTIBODY TECHNIQUE. One advantage over the other methods (e.g., radioimmunoassay) is its extreme sensitivity, with a detection limit on the order of tenths of microgram/liter.'),
  'D015201': NtO(MH='Meta-Analysis as Topic', MS='A quantitative method of combining the results of independent studies (usually drawn from the published literature) and synthesizing summaries and conclusions which may be used to evaluate therapeutic effectiveness, plan new studies, etc., with application chiefly in the areas of research and medicine.'),
  'D015202': NtO(MH='Protein Engineering', MS='Procedures by which protein structure and function are changed or created in vitro by altering existing or synthesizing new structural genes that direct the synthesis of proteins with sought-after properties. Such procedures may include the design of MOLECULAR MODELS of proteins using COMPUTER GRAPHICS or other molecular modeling techniques; site-specific mutagenesis (MUTAGENESIS, SITE-SPECIFIC) of existing genes; and DIRECTED MOLECULAR EVOLUTION techniques to create new genes.'),
  'D015203': NtO(MH='Reproducibility of Results', MS='The statistical reproducibility of measurements (often in a clinical context), including the testing of instrumentation or techniques to obtain reproducible results. The concept includes reproducibility of physiological measurements, which may be used to develop rules to assess probability or prognosis, or response to a stimulus; reproducibility of occurrence of a condition; and reproducibility of experimental results.'),
  'D015204': NtO(MH='Spectroscopy, Mossbauer', MS='A spectroscopic technique which uses the Mossbauer effect (inelastic scattering of gamma radiation resulting from interaction with heavy nuclei) to monitor the small variations in the interaction between an atomic nucleus and its environment. Such variations may be induced by changes in temperature, pressure, chemical state, molecular conformation, molecular interaction, or physical site. It is particularly useful for studies of structure-activity relationship in metalloproteins, mobility of heavy metals, and the state of whole tissue and cell membranes.'),
  'D015205': NtO(MH='Northwestern United States', MS='The geographic area of the northwestern region of the United States. The states usually included in this region are Idaho, Montana, Oregon, Washington, and Wyoming.'),
  'D015206': NtO(MH='Southwestern United States', MS='The geographic area of the southwestern region of the United States. The states usually included in this region are Arizona, California, Colorado, Nevada, New Mexico, Texas, and Utah.'),
  'D015207': NtO(MH='Osteoarthritis, Hip', MS='Noninflammatory degenerative disease of the hip joint which usually appears in late middle or old age. It is characterized by growth or maturational disturbances in the femoral neck and head, as well as acetabular dysplasia. A dominant symptom is pain on weight-bearing or motion.'),
  'D015208': NtO(MH='Smoke Inhalation Injury', MS='Pulmonary injury following the breathing in of toxic smoke from burning materials such as plastics, synthetics, building materials, etc. This injury is the most frequent cause of death in burn patients.'),
  'D015209': NtO(MH='Cholangitis, Sclerosing', MS='Chronic inflammatory disease of the BILIARY TRACT. It is characterized by fibrosis and hardening of the intrahepatic and extrahepatic biliary ductal systems leading to bile duct strictures, CHOLESTASIS, and eventual BILIARY CIRRHOSIS.'),
  'D015210': NtO(MH='Arthritis, Gouty', MS='Arthritis, especially of the great toe, as a result of gout. Acute gouty arthritis often is precipitated by trauma, infection, surgery, etc. The initial attacks are usually monoarticular but later attacks are often polyarticular. Acute and chronic gouty arthritis are associated with accumulation of MONOSODIUM URATE in and around affected joints.'),
  'D015211': NtO(MH='Zellweger Syndrome', MS='An autosomal recessive disorder due to defects in PEROXISOME biogenesis which involves more than 13 genes encoding peroxin proteins of the peroxisomal membrane and matrix. Zellweger syndrome is typically seen in the neonatal period with features such as dysmorphic skull; MUSCLE HYPOTONIA; SENSORINEURAL HEARING LOSS; visual compromise; SEIZURES; progressive degeneration of the KIDNEYS and the LIVER. Zellweger-like syndrome refers to phenotypes resembling the neonatal Zellweger syndrome but seen in children or adults with apparently intact peroxisome biogenesis.'),
  'D015212': NtO(MH='Inflammatory Bowel Diseases', MS='Chronic, non-specific inflammation of the GASTROINTESTINAL TRACT. Etiology may be genetic or environmental. This term includes CROHN DISEASE and ULCERATIVE COLITIS.'),
  'D015213': NtO(MH='Neuroimmunomodulation', MS='The biochemical and electrophysiological interactions between the NERVOUS SYSTEM and IMMUNE SYSTEM.'),
  'D015214': NtO(MH='Antigens, Differentiation, Myelomonocytic', MS='Surface antigens expressed on myeloid cells of the granulocyte-monocyte-histiocyte series during differentiation. Analysis of their reactivity in normal and malignant myelomonocytic cells is useful in identifying and classifying human leukemias and lymphomas.'),
  'D015215': NtO(MH='Zidovudine', MS="A dideoxynucleoside compound in which the 3'-hydroxy group on the sugar moiety has been replaced by an azido group. This modification prevents the formation of phosphodiester linkages which are needed for the completion of nucleic acid chains. The compound is a potent inhibitor of HIV replication, acting as a chain-terminator of viral DNA during reverse transcription. It improves immunologic function, partially reverses the HIV-induced neurological dysfunction, and improves certain other clinical abnormalities associated with AIDS. Its principal toxic effect is dose-dependent suppression of bone marrow, resulting in anemia and leukopenia."),
  'D015217': NtO(MH='Cholesterol Ester Storage Disease', MS='An autosomal recessive disorder caused by mutations in the gene for acid lipase (STEROL ESTERASE). It is characterized by the accumulation of neutral lipids, particularly CHOLESTEROL ESTERS in leukocytes, fibroblasts, and hepatocytes.'),
  'D015218': NtO(MH='Hyperostosis, Sternocostoclavicular', MS='A rare, benign rheumatologic disorder or syndrome characterized by hyperostosis and soft tissue ossification between the clavicles and the anterior part of the upper ribs. It is often associated with the dermatologic disorder palmoplantar pustulosis, particularly in Japan. Careful diagnosis is required to distinguish it from psoriatic arthritis, OSTEITIS DEFORMANS, and other diseases. Spondylitis of pustulosis palmaris et plantaris is one of the possible causes; also, evidence suggests one origin may be bone infection. Bone imaging is especially useful for diagnosis. It was originally described by Sonozaki in 1974.'),
  'D015219': NtO(MH='Substrate Cycling', MS='A set of opposing, nonequilibrium reactions catalyzed by different enzymes which act simultaneously, with at least one of the reactions driven by ATP hydrolysis. The results of the cycle are that ATP energy is depleted, heat is produced and no net substrate-to-product conversion is achieved. Examples of substrate cycling are cycling of gluconeogenesis and glycolysis pathways and cycling of the triglycerides and fatty acid pathways. Rates of substrate cycling may be increased many-fold in association with hypermetabolic states resulting from severe burns, cold exposure, hyperthyroidism, or acute exercise.'),
  'D015220': NtO(MH='Calcium Channels', MS='Voltage-dependent cell membrane glycoproteins selectively permeable to calcium ions. They are categorized as L-, T-, N-, P-, Q-, and R-types based on the activation and inactivation kinetics, ion specificity, and sensitivity to drugs and toxins. The L- and T-types are present throughout the cardiovascular and central nervous systems and the N-, P-, Q-, & R-types are located in neuronal tissue.'),
  'D015221': NtO(MH='Potassium Channels', MS='Cell membrane glycoproteins that are selectively permeable to potassium ions. At least eight major groups of K channels exist and they are made up of dozens of different subunits.'),
  'D015222': NtO(MH='Sodium Channels', MS='Ion channels that specifically allow the passage of SODIUM ions. A variety of specific sodium channel subtypes are involved in serving specialized functions such as neuronal signaling, CARDIAC MUSCLE contraction, and KIDNEY function.'),
  'D015223': NtO(MH='Wolman Disease', MS="The severe infantile form of inherited lysosomal lipid storage diseases due to deficiency of acid lipase (STEROL ESTERASE). It is characterized by the accumulation of neutral lipids, particularly CHOLESTEROL ESTERS in leukocytes, fibroblasts, and hepatocytes. It is also known as Wolman's xanthomatosis and is an allelic variant of CHOLESTEROL ESTER STORAGE DISEASE."),
  'D015224': NtO(MH='Dideoxynucleosides', MS="Nucleosides that have two hydroxy groups removed from the sugar moiety. The majority of these compounds have broad-spectrum antiretroviral activity due to their action as antimetabolites. The nucleosides are phosphorylated intracellularly to their 5'-triphosphates and act as chain-terminating inhibitors of viral reverse transcription."),
  'D015225': NtO(MH='Magnetoencephalography', MS='The measurement of magnetic fields over the head generated by electric currents in the brain. As in any electrical conductor, electric fields in the brain are accompanied by orthogonal magnetic fields. The measurement of these fields provides information about the localization of brain activity which is complementary to that provided by ELECTROENCEPHALOGRAPHY. Magnetoencephalography may be used alone or together with electroencephalography, for measurement of spontaneous or evoked activity, and for research or clinical purposes.'),
  'D015226': NtO(MH='Dinucleoside Phosphates', MS='A group of compounds which consist of a nucleotide molecule to which an additional nucleoside is attached through the phosphate molecule(s). The nucleotide can contain any number of phosphates.'),
  'D015227': NtO(MH='Lipid Peroxidation', MS='Peroxidase catalyzed oxidation of lipids using hydrogen peroxide as an electron acceptor.'),
  'D015228': NtO(MH='Hypertriglyceridemia', MS='A condition of elevated levels of TRIGLYCERIDES in the blood.'),
  'D015229': NtO(MH='Sexually Transmitted Diseases, Viral', MS='Viral diseases which are transmitted or propagated by sexual conduct.'),
  'D015230': NtO(MH='Prostaglandin D2', MS='The principal cyclooxygenase metabolite of arachidonic acid. It is released upon activation of mast cells and is also synthesized by alveolar macrophages. Among its many biological actions, the most important are its bronchoconstrictor, platelet-activating-factor-inhibitory, and cytotoxic effects.'),
  'D015231': NtO(MH='Sexually Transmitted Diseases, Bacterial', MS='Bacterial diseases transmitted or propagated by sexual conduct.'),
  'D015232': NtO(MH='Dinoprostone', MS='The most common and most biologically active of the mammalian prostaglandins. It exhibits most biological activities characteristic of prostaglandins and has been used extensively as an oxytocic agent. The compound also displays a protective effect on the intestinal mucosa.'),
  'D015233': NtO(MH='Models, Statistical', MS='Statistical formulations or analyses which, when applied to data and found to fit the data, are then used to verify the assumptions and parameters used in the analysis. Examples of statistical models are the linear model, binomial model, polynomial model, two-parameter model, etc.'),
  'D015234': NtO(MH='HLA-A Antigens', MS='Polymorphic class I human histocompatibility (HLA) surface antigens present on almost all nucleated cells. At least 20 antigens have been identified which are encoded by the A locus of multiple alleles on chromosome 6. They serve as targets for T-cell cytolytic responses and are involved with acceptance or rejection of tissue/organ grafts.'),
  'D015235': NtO(MH='HLA-B Antigens', MS='Class I human histocompatibility (HLA) surface antigens encoded by more than 30 detectable alleles on locus B of the HLA complex, the most polymorphic of all the HLA specificities. Several of these antigens (e.g., HLA-B27, -B7, -B8) are strongly associated with predisposition to rheumatoid and other autoimmune disorders. Like other class I HLA determinants, they are involved in the cellular immune reactivity of cytolytic T lymphocytes.'),
  'D015236': NtO(MH='HLA-C Antigens', MS='Class I human histocompatibility (HLA) antigens encoded by a small cluster of structural genes at the C locus on chromosome 6. They have significantly lower immunogenicity than the HLA-A and -B determinants and are therefore of minor importance in donor/recipient crossmatching. Their primary role is their high-risk association with certain disease manifestations (e.g., spondylarthritis, psoriasis, multiple myeloma).'),
  'D015237': NtO(MH='Dinoprost', MS='A naturally occurring prostaglandin that has oxytocic, luteolytic, and abortifacient activities. Due to its vasocontractile properties, the compound has a variety of other biological actions.'),
  'D015238': NtO(MH='Sulfotransferases', MS='Enzymes which transfer sulfate groups to various acceptor molecules. They are involved in posttranslational sulfation of proteins and sulfate conjugation of exogenous chemicals and bile acids. EC 2.8.2.'),
  'D015239': NtO(MH='Arylsulfotransferase', MS="A sulfotransferase that catalyzes the sulfation of a phenol in the presence of 3'-phosphoadenylylsulfate as sulfate donor to yield an aryl sulfate and adenosine 3',5'-bisphosphate. A number of aromatic compounds can act as acceptors; however, organic hydroxylamines are not substrates. Sulfate conjugation by this enzyme is a major pathway for the biotransformation of phenolic and catechol drugs as well as neurotransmitters. EC 2.8.2.1."),
  'D015240': NtO(MH='Phorbol 12,13-Dibutyrate', MS='A phorbol ester found in CROTON OIL which, in addition to being a potent skin tumor promoter, is also an effective activator of calcium-activated, phospholipid-dependent protein kinase (protein kinase C). Due to its activation of this enzyme, phorbol 12,13-dibutyrate profoundly affects many different biological systems.'),
  'D015241': NtO(MH='Fibrinogens, Abnormal', MS='Fibrinogens which have a functional defect as the result of one or more amino acid substitutions in the amino acid sequence of normal fibrinogen. Abnormalities of the fibrinogen molecule may impair any of the major steps involved in the conversion of fibrinogen into stabilized fibrin, such as cleavage of the fibrinopeptides by thrombin, polymerization and cross-linking of fibrin. The resulting dysfibrinogenemias can be clinically silent or can be associated with bleeding, thrombosis or defective wound healing.'),
  'D015242': NtO(MH='Ofloxacin', MS='A synthetic fluoroquinolone antibacterial agent that inhibits the supercoiling activity of bacterial DNA GYRASE, halting DNA REPLICATION.'),
  'D015243': NtO(MH='Cholesterol, VLDL', MS='Cholesterol which is contained in or bound to very low density lipoproteins (VLDL). High circulating levels of VLDL cholesterol are found in HYPERLIPOPROTEINEMIA TYPE IIB. The cholesterol on the VLDL is eventually delivered by LOW-DENSITY LIPOPROTEINS to the tissues after the catabolism of VLDL to INTERMEDIATE-DENSITY LIPOPROTEINS, then to LDL.'),
  'D015244': NtO(MH='Thiorphan', MS='A potent inhibitor of membrane metalloendopeptidase (ENKEPHALINASE). Thiorphan potentiates morphine-induced ANALGESIA and attenuates naloxone-precipitated withdrawal symptoms.'),
  'D015245': NtO(MH='Deoxyribonuclease BamHI', MS='One of the Type II site-specific deoxyribonucleases (EC 3.1.21.4). It recognizes and cleaves the sequence G/GATCC at the slash. BamHI is from Bacillus amyloliquefaciens N. Numerous isoschizomers have been identified. EC 3.1.21.-.'),
  'D015246': NtO(MH='Deoxyribonuclease EcoRI', MS='One of the Type II site-specific deoxyribonucleases (EC 3.1.21.4). It recognizes and cleaves the sequence G/AATTC at the slash. EcoRI is from E coliRY13. Several isoschizomers have been identified. EC 3.1.21.-.'),
  'D015247': NtO(MH='Deoxyribonuclease HindIII', MS='One of the Type II site-specific deoxyribonucleases (EC 3.1.21.4). It recognizes and cleaves the sequence A/AGCTT at the slash. HindIII is from Haemophilus influenzae R(d). Numerous isoschizomers have been identified. EC 3.1.21.-.'),
  'D015248': NtO(MH='Gemfibrozil', MS='A lipid-regulating agent that lowers elevated serum lipids primarily by decreasing serum triglycerides with a variable reduction in total cholesterol.'),
  'D015250': NtO(MH='Aclarubicin', MS='An anthracycline produced by Streptomyces galilaeus. It has potent antineoplastic activity.'),
  'D015251': NtO(MH='Epirubicin', MS="An anthracycline which is the 4'-epi-isomer of doxorubicin. The compound exerts its antitumor effects by interference with the synthesis and function of DNA."),
  'D015252': NtO(MH='Deoxyribonucleases, Type II Site-Specific', MS="Enzyme systems containing a single subunit and requiring only magnesium for endonucleolytic activity. The corresponding modification methylases are separate enzymes. The systems recognize specific short DNA sequences and cleave either within, or at a short specific distance from, the recognition sequence to give specific double-stranded fragments with terminal 5'-phosphates. Enzymes from different microorganisms with the same specificity are called isoschizomers. EC 3.1.21.4."),
  'D015253': NtO(MH='Deoxyribonucleases, Type I Site-Specific', MS="Enzyme systems containing three different subunits and requiring ATP, S-adenosylmethionine, and magnesium for endonucleolytic activity to give random double-stranded fragments with terminal 5'-phosphates. They function also as DNA-dependent ATPases and modification methylases, catalyzing the reactions of EC 2.1.1.72 and EC 2.1.1.73 with similar site-specificity. The systems recognize specific short DNA sequences and cleave at sites remote from the recognition sequence. Enzymes from different microorganisms with the same specificity are called isoschizomers. EC 3.1.21.3."),
  'D015254': NtO(MH='DNA Modification Methylases', MS="Enzymes that are part of the restriction-modification systems. They are responsible for producing a species-characteristic methylation pattern, on either adenine or cytosine residues, in a specific short base sequence in the host cell's own DNA. This methylated sequence will occur many times in the host-cell DNA and remain intact for the lifetime of the cell. Any DNA from another species which gains entry into a living cell and lacks the characteristic methylation pattern will be recognized by the restriction endonucleases of similar specificity and destroyed by cleavage. Most have been studied in bacterial systems, but a few have been found in eukaryotic organisms."),
  'D015255': NtO(MH='Idarubicin', MS='An orally administered anthracycline antineoplastic. The compound has shown activity against BREAST NEOPLASMS; LYMPHOMA; and LEUKEMIA.'),
  'D015256': NtO(MH='Coenzyme A-Transferases', MS='Enzymes which transfer coenzyme A moieties from acyl- or acetyl-CoA to various carboxylic acceptors forming a thiol ester. Enzymes in this group are instrumental in ketone body metabolism and utilization of acetoacetate in mitochondria. EC 2.8.3.'),
  'D015257': NtO(MH='DNA-Cytosine Methylases', MS='Methylases that are specific for CYTOSINE residues found on DNA.'),
  'D015258': NtO(MH='Acid Rain', MS='Acidic water usually pH 2.5 to 4.5, which poisons the ecosystem and adversely affects plants, fishes, and mammals. It is caused by industrial pollutants, mainly sulfur oxides and nitrogen oxides, emitted into the atmosphere and returning to earth in the form of acidic rain water.'),
  'D015259': NtO(MH='Dopamine Agents', MS='Any drugs that are used for their effects on dopamine receptors, on the life cycle of dopamine, or on the survival of dopaminergic neurons.'),
  'D015260': NtO(MH='Neprilysin', MS='Enzyme that is a major constituent of kidney brush-border membranes and is also present to a lesser degree in the brain and other tissues. It preferentially catalyzes cleavage at the amino group of hydrophobic residues of the B-chain of insulin as well as opioid peptides and other biologically active peptides. The enzyme is inhibited primarily by EDTA, phosphoramidon, and thiorphan and is reactivated by zinc. Neprilysin is identical to common acute lymphoblastic leukemia antigen (CALLA Antigen), an important marker in the diagnosis of human acute lymphocytic leukemia. There is no relationship with CALLA PLANT.'),
  'D015262': NtO(MH='Xenobiotics', MS='Chemical substances that are foreign to the biological system. They include naturally occurring compounds, drugs, environmental agents, carcinogens, insecticides, etc.'),
  'D015263': NtO(MH='Deoxyribonucleases, Type III Site-Specific', MS="Enzyme systems composed of two subunits and requiring ATP and magnesium for endonucleolytic activity; they do not function as ATPases. They exist as complexes with modification methylases of similar specificity listed under EC 2.1.1.72 or EC 2.1.1.73. The systems recognize specific short DNA sequences and cleave a short distance, about 24 to 27 bases, away from the recognition sequence to give specific double-stranded fragments with terminal 5'-phosphates. Enzymes from different microorganisms with the same specificity are called isoschizomers. EC 3.1.21.5."),
  'D015265': NtO(MH='Site-Specific DNA-Methyltransferase (Adenine-Specific)', MS='An enzyme responsible for producing a species-characteristic methylation pattern on adenine residues in a specific short base sequence in the host cell DNA. The enzyme catalyzes the methylation of DNA adenine in the presence of S-adenosyl-L-methionine to form DNA containing 6-methylaminopurine and S-adenosyl-L-homocysteine. EC 2.1.1.72.'),
  'D015266': NtO(MH='Carcinoma, Merkel Cell', MS='A carcinoma arising from MERKEL CELLS located in the basal layer of the epidermis and occurring most commonly as a primary neuroendocrine carcinoma of the skin. Merkel cells are tactile cells of neuroectodermal origin and histologically show neurosecretory granules. The skin of the head and neck are a common site of Merkel cell carcinoma, occurring generally in elderly patients. (Holland et al., Cancer Medicine, 3d ed, p1245)'),
  'D015267': NtO(MH='Churg-Strauss Syndrome', MS='Widespread necrotizing angiitis with granulomas. Pulmonary involvement is frequent. Asthma or other respiratory infection may precede evidence of vasculitis. Eosinophilia and lung involvement differentiate this disease from POLYARTERITIS NODOSA.'),
  'D015269': NtO(MH='Mycobacterium avium Complex', MS='A complex that includes several strains of M. avium. M. intracellulare is not easily distinguished from M. avium and therefore is included in the complex. These organisms are most frequently found in pulmonary secretions from persons with a tuberculous-like mycobacteriosis. Strains of this complex have also been associated with childhood lymphadenitis and AIDS; M. avium alone causes tuberculosis in a variety of birds and other animals, including pigs.'),
  'D015270': NtO(MH='Mycobacterium avium-intracellulare Infection', MS='A nontuberculous infection when occurring in humans. It is characterized by pulmonary disease, lymphadenitis in children, and systemic disease in AIDS patients. Mycobacterium avium-intracellulare infection of birds and swine results in tuberculosis.'),
  'D015271': NtO(MH='Insurance Pools', MS='An organization of insurers or reinsurers through which particular types of risk are shared or pooled. The risk of high loss by a particular insurance company is transferred to the group as a whole (the insurance pool) with premiums, losses, and expenses shared in agreed amounts.'),
  'D015272': NtO(MH='Pulmonary Medicine', MS='A subspecialty of internal medicine concerned with the study of the RESPIRATORY SYSTEM. It is especially concerned with diagnosis and treatment of diseases and defects of the lungs and bronchial tree.'),
  'D015273': NtO(MH='Insurance Selection Bias', MS='Adverse or favorable selection bias exhibited by insurers or enrollees resulting in disproportionate enrollment of certain groups of people.'),
  'D015274': NtO(MH='Interdepartmental Relations', MS='The interactions between representatives of institutional departments.'),
  'D015275': NtO(MH='Tumor Lysis Syndrome', MS='A syndrome resulting from cytotoxic therapy, occurring generally in aggressive, rapidly proliferating lymphoproliferative disorders. It is characterized by combinations of hyperuricemia, lactic acidosis, hyperkalemia, hyperphosphatemia and hypocalcemia.'),
  'D015276': NtO(MH='Health Care Rationing', MS='Planning for the equitable allocation, apportionment, or distribution of available health resources.'),
  'D015277': NtO(MH='Hotlines', MS='A direct communication system, usually telephone, established for instant contact. It is designed to provide special information and assistance through trained personnel and is used for counseling, referrals, and emergencies such as poisonings and threatened suicides.'),
  'D015278': NtO(MH='Intensive Care Units, Pediatric', MS='Hospital units providing continuous surveillance and care to acutely ill infants and children. Neonates are excluded since INTENSIVE CARE UNITS, NEONATAL is available.'),
  'D015279': NtO(MH='Organizational Culture', MS='Beliefs and values shared by all members of the organization. These shared values, which are subject to change, are reflected in the day to day management of the organization.'),
  'D015280': NtO(MH='DNA Restriction-Modification Enzymes', MS='Systems consisting of two enzymes, a modification methylase and a restriction endonuclease. They are closely related in their specificity and protect the DNA of a given bacterial species. The methylase adds methyl groups to adenine or cytosine residues in the same target sequence that constitutes the restriction enzyme binding site. The methylation renders the target site resistant to restriction, thereby protecting DNA against cleavage.'),
  'D015281': NtO(MH='Cefmenoxime', MS='A cephalosporin antibiotic that is administered intravenously or intramuscularly. It is active against most common gram-positive and gram-negative microorganisms, is a potent inhibitor of Enterobacteriaceae, and is highly resistant to hydrolysis by beta-lactamases. The drug has a high rate of efficacy in many types of infection and to date no severe side effects have been noted.'),
  'D015282': NtO(MH='Octreotide', MS='A potent, long-acting synthetic SOMATOSTATIN octapeptide analog that inhibits secretion of GROWTH HORMONE and is used to treat hormone-secreting tumors; DIABETES MELLITUS; HYPOTENSION, ORTHOSTATIC; HYPERINSULINISM; hypergastrinemia; and small bowel fistula.'),
  'D015283': NtO(MH='Citalopram', MS='A furancarbonitrile that is one of the serotonin uptake inhibitors used as an antidepressant. The drug is also effective in reducing ethanol uptake in alcoholics and is used in depressed patients who also suffer from TARDIVE DYSKINESIA in preference to tricyclic antidepressants, which aggravate dyskinesia.'),
  'D015284': NtO(MH='Physician Payment Review Commission', MS='A commission created by the Consolidated Omnibus Reconciliation Act of 1985, enacted in 1986, and given the mandate to advise Congress on Medicare-physician payment. The commission members are appointed by the U.S. Office of Technology Assessment.'),
  'D015285': NtO(MH='Hospital-Patient Relations', MS='Interactions between hospital staff or administrators and patients. Includes guest relations programs designed to improve the image of the hospital and attract patients.'),
  'D015286': NtO(MH='Kassinin', MS='Dodecapeptide tachykinin found in the central nervous system of the amphibian Kassina senegalensis. It is similar in structure and action to other tachykinins, but is especially effective in contracting smooth muscle tissue and stimulating the micturition reflex.'),
  'D015287': NtO(MH='Neurokinin B', MS='A mammalian neuropeptide of 10 amino acids that belongs to the tachykinin family. It is similar in structure and action to SUBSTANCE P and NEUROKININ A with the ability to excite neurons, dilate blood vessels, and contract smooth muscles, such as those in the URINARY BLADDER and UTERUS.'),
  'D015288': NtO(MH='Neurokinin A', MS='A mammalian neuropeptide of 10 amino acids that belongs to the tachykinin family. It is similar in structure and action to SUBSTANCE P and NEUROKININ B with the ability to excite neurons, dilate blood vessels, and contract smooth muscles, such as those in the BRONCHI.'),
  'D015289': NtO(MH='Leukotrienes', MS='A family of biologically active compounds derived from arachidonic acid by oxidative metabolism through the 5-lipoxygenase pathway. They participate in host defense reactions and pathophysiological conditions such as immediate hypersensitivity and inflammation. They have potent actions on many essential organs and systems, including the cardiovascular, pulmonary, and central nervous system as well as the gastrointestinal tract and the immune system.'),
  'D015290': NtO(MH='Second Messenger Systems', MS='Systems in which an intracellular signal is generated in response to an intercellular primary messenger such as a hormone or neurotransmitter. They are intermediate signals in cellular processes such as metabolism, secretion, contraction, phototransduction, and cell growth. Examples of second messenger systems are the adenyl cyclase-cyclic AMP system, the phosphatidylinositol diphosphate-inositol triphosphate system, and the cyclic GMP system.'),
  'D015291': NtO(MH='Transforming Growth Factors', MS='Hormonally active polypeptides that can induce the transformed phenotype when added to normal, non-transformed cells. They have been found in culture fluids from retrovirally transformed cells and in tumor-derived cells as well as in non-neoplastic sources. Their transforming activities are due to the simultaneous action of two otherwise unrelated factors, TRANSFORMING GROWTH FACTOR ALPHA and TRANSFORMING GROWTH FACTOR BETA.'),
  'D015292': NtO(MH='Glycoprotein Hormones, alpha Subunit', MS='The alpha chain of pituitary glycoprotein hormones (THYROTROPIN; FOLLICLE STIMULATING HORMONE; LUTEINIZING HORMONE) and the placental CHORIONIC GONADOTROPIN. Within a species, the alpha subunits of these four hormones are identical; the distinct functional characteristics of these glycoprotein hormones are determined by the unique beta subunits. Both subunits, the non-covalently bound heterodimers, are required for full biologic activity.'),
  'D015293': NtO(MH='Transducin', MS="A heterotrimeric GTP-binding protein that mediates the light activation signal from photolyzed rhodopsin to cyclic GMP phosphodiesterase and is pivotal in the visual excitation process. Activation of rhodopsin on the outer membrane of rod and cone cells causes GTP to bind to transducin followed by dissociation of the alpha subunit-GTP complex from the beta/gamma subunits of transducin. The alpha subunit-GTP complex activates the cyclic GMP phosphodiesterase which catalyzes the hydrolysis of cyclic GMP to 5'-GMP. This leads to closure of the sodium and calcium channels and therefore hyperpolarization of the rod cells."),
  'D015295': NtO(MH='Antigens, Tumor-Associated, Carbohydrate', MS='Carbohydrate antigens expressed by malignant tissue. They are useful as tumor markers and are measured in the serum by means of a radioimmunoassay employing monoclonal antibodies.'),
  'D015296': NtO(MH='Ceftizoxime', MS='A semisynthetic cephalosporin antibiotic which can be administered intravenously or by suppository. The drug is highly resistant to a broad spectrum of beta-lactamases and is active against a wide range of both aerobic and anaerobic gram-positive and gram-negative organisms. It has few side effects and is reported to be safe and effective in aged patients and in patients with hematologic disorders.'),
  'D015299': NtO(MH='Discitis', MS='Inflammation of an INTERVERTEBRAL DISC or disk space which may lead to disk erosion. Until recently, discitis has been defined as a nonbacterial inflammation and has been attributed to aseptic processes (e.g., chemical reaction to an injected substance). However, recent studies provide evidence that infection may be the initial cause, but perhaps not the promoter, of most cases of discitis. Discitis has been diagnosed in patients following discography, myelography, lumbar puncture, paravertebral injection, and obstetrical epidural anesthesia. Discitis following chemonucleolysis (especially with chymopapain) is attributed to chemical reaction by some and to introduction of microorganisms by others.'),
  'D015300': NtO(MH='Ventilator Weaning', MS='Techniques for effecting the transition of the respiratory-failure patient from mechanical ventilation to spontaneous ventilation, while meeting the criteria that tidal volume be above a given threshold (greater than 5 ml/kg), respiratory frequency be below a given count (less than 30 breaths/min), and oxygen partial pressure be above a given threshold (PaO2 greater than 50mm Hg). Weaning studies focus on finding methods to monitor and predict the outcome of mechanical ventilator weaning as well as finding ventilatory support techniques which will facilitate successful weaning. Present methods include intermittent mandatory ventilation, intermittent positive pressure ventilation, and mandatory minute volume ventilation.'),
  'D015301': NtO(MH='Retroviruses, Simian', MS='Classes of retroviruses for which monkeys or apes are hosts. Those isolated from the West African green monkey and the Asian rhesus macaque monkey are of particular interest because of their similarities to viruses causing cancer and acquired immunodeficiency syndrome (AIDS) in humans.'),
  'D015302': NtO(MH='Simian Immunodeficiency Virus', MS='Species of the genus LENTIVIRUS, subgenus primate immunodeficiency viruses (IMMUNODEFICIENCY VIRUSES, PRIMATE), that induces acquired immunodeficiency syndrome in monkeys and apes (SAIDS). The genetic organization of SIV is virtually identical to HIV.'),
  'D015303': NtO(MH='Simian T-lymphotropic virus 1', MS='A strain of PRIMATE T-LYMPHOTROPIC VIRUS 2, closely related to the human HTLV-1 virus. The clinical, hematological, and histopathological characteristics of the disease in STLV-infected monkeys are very similar to those of human adult T-cell leukemia. Subgroups include the African green monkey subtype (STLV-I-AGM), for which the nucleotide sequence is 95% homologous with that of HUMAN T-LYMPHOTROPIC VIRUS 1, and the Asian rhesus macaque subtype (STLV-I-MM), for which the nucleotide sequence is 90% homologous with that of HUMAN T-LYMPHOTROPIC VIRUS 1.'),
  'D015306': NtO(MH='Biogenic Monoamines', MS='Biogenic amines having only one amine moiety. Included in this group are all natural monoamines formed by the enzymatic decarboxylation of natural amino acids.'),
  'D015308': NtO(MH='Anaerobic Threshold', MS='The oxygen consumption level above which aerobic energy production is supplemented by anaerobic mechanisms during exercise, resulting in a sustained increase in lactate concentration and metabolic acidosis. The anaerobic threshold is affected by factors that modify oxygen delivery to the tissues; it is low in patients with heart disease. Methods of measurement include direct measure of lactate concentration, direct measurement of bicarbonate concentration, and gas exchange measurements.'),
  'D015309': NtO(MH='Glucose Clamp Technique', MS='Maintenance of a constant blood glucose level by perfusion or infusion with glucose or insulin. It is used for the study of metabolic rates (e.g., in glucose, lipid, amino acid metabolism) at constant glucose concentration.'),
  'D015310': NtO(MH='Cefotiam', MS='One of the CEPHALOSPORINS that has a broad spectrum of activity against both gram-positive and gram-negative microorganisms.'),
  'D015311': NtO(MH='Cefmetazole', MS='A semisynthetic cephamycin antibiotic with a broad spectrum of activity against both gram-positive and gram-negative microorganisms. It has a high rate of efficacy in many types of infection and to date no severe side effects have been noted.'),
  'D015312': NtO(MH='Hemodialysis Solutions', MS='Solutions prepared for hemodialysis. The composition of the pre-dialysis solution may be varied in order to determine the effect of solvated metabolites on anoxia, malnutrition, acid-base balance, etc. Of principal interest are the effect of the choice of buffers (e.g., acetate or carbonate), the addition of cations (Na+, K+, Ca2+), and addition of carbohydrates (glucose).'),
  'D015313': NtO(MH='Cefotetan', MS='A semisynthetic cephamycin antibiotic that is administered intravenously or intramuscularly. The drug is highly resistant to a broad spectrum of beta-lactamases and is active against a wide range of both aerobic and anaerobic gram-positive and gram-negative microorganisms.'),
  'D015314': NtO(MH='Dialysis Solutions', MS='Solutions prepared for exchange across a semipermeable membrane of solutes below a molecular size determined by the cutoff threshold of the membrane material.'),
  'D015316': NtO(MH='Genetic Therapy', MS='Techniques and strategies which include the use of coding sequences and other conventional or radical means to transform or modify cells for the purpose of treating or reversing disease conditions.'),
  'D015317': NtO(MH='Biogenic Polyamines', MS='Biogenic amines having more than one amine group. These are long-chain aliphatic compounds that contain multiple amino and/or imino groups. Because of the linear arrangement of positive charge on these molecules, polyamines bind electrostatically to ribosomes, DNA, and RNA.'),
  'D015318': NtO(MH='Fructose Metabolism, Inborn Errors', MS='Inherited abnormalities of fructose metabolism, which include three known autosomal recessive types: hepatic fructokinase deficiency (essential fructosuria), hereditary fructose intolerance, and hereditary fructose-1,6-diphosphatase deficiency. Essential fructosuria is a benign asymptomatic metabolic disorder caused by deficiency in fructokinase, leading to decreased conversion of fructose to fructose-1-phosphate and alimentary hyperfructosemia, but with no clinical dysfunction; may produce a false-positive diabetes test.'),
  'D015319': NtO(MH='Fructose-1,6-Diphosphatase Deficiency', MS='An autosomal recessive fructose metabolism disorder due to absent or deficient fructose-1,6-diphosphatase activity. Gluconeogenesis is impaired, resulting in accumulation of gluconeogenic precursors (e.g., amino acids, lactate, ketones) and manifested as hypoglycemia, ketosis, and lactic acidosis. Episodes in the newborn infant are often lethal. Later episodes are often brought on by fasting and febrile infections. As patients age through early childhood, tolerance to fasting improves and development becomes normal.'),
  'D015320': NtO(MH='Tachykinins', MS='A family of biologically active peptides sharing a common conserved C-terminal sequence, -Phe-X-Gly-Leu-Met-NH2, where X is either an aromatic or a branched aliphatic amino acid. Members of this family have been found in mammals, amphibians, and mollusks. Tachykinins have diverse pharmacological actions in the central nervous system and the cardiovascular, genitourinary, respiratory, and gastrointestinal systems, as well as in glandular tissues. This diversity of activity is due to the existence of three or more subtypes of tachykinin receptors.'),
  'D015321': NtO(MH='Gene Rearrangement', MS='The ordered rearrangement of gene regions by DNA recombination such as that which occurs normally during development.'),
  'D015322': NtO(MH='Gene Rearrangement, B-Lymphocyte', MS='Ordered rearrangement of B-lymphocyte variable gene regions coding for the IMMUNOGLOBULIN CHAINS, thereby contributing to antibody diversity. It occurs during the differentiation of the IMMATURE B-LYMPHOCYTES.'),
  'D015323': NtO(MH='Pyruvate Metabolism, Inborn Errors', MS='Hereditary disorders of pyruvate metabolism. They are difficult to diagnose and describe because pyruvate is a key intermediate in glycolysis, gluconeogenesis, and the tricarboxylic acid cycle. Some inherited metabolic disorders may alter pyruvate metabolism indirectly. Disorders in pyruvate metabolism appear to lead to deficiencies in neurotransmitter synthesis and, consequently, to nervous system disorders.'),
  'D015324': NtO(MH='Pyruvate Carboxylase Deficiency Disease', MS='An autosomal recessive metabolic disorder caused by absent or decreased PYRUVATE CARBOXYLASE activity, the enzyme that regulates gluconeogenesis, lipogenesis, and neurotransmitter synthesis. Clinical manifestations include lactic acidosis, seizures, respiratory distress, marked psychomotor delay, periodic HYPOGLYCEMIA, and hypotonia. The clinical course may be similar to LEIGH DISEASE. (From Am J Hum Genet 1998 Jun;62(6):1312-9)'),
  'D015325': NtO(MH='Pyruvate Dehydrogenase Complex Deficiency Disease', MS='An inherited metabolic disorder caused by deficient enzyme activity in the PYRUVATE DEHYDROGENASE COMPLEX, resulting in deficiency of acetyl CoA and reduced synthesis of acetylcholine. Two clinical forms are recognized: neonatal and juvenile. The neonatal form is a relatively common cause of lactic acidosis in the first weeks of life and may also feature an erythematous rash. The juvenile form presents with lactic acidosis, alopecia, intermittent ATAXIA; SEIZURES; and an erythematous rash. (From J Inherit Metab Dis 1996;19(4):452-62) Autosomal recessive and X-linked forms are caused by mutations in the genes for the three different enzyme components of this multisubunit pyruvate dehydrogenase complex. One of the mutations at Xp22.2-p22.1 in the gene for the E1 alpha component of the complex leads to LEIGH DISEASE.'),
  'D015326': NtO(MH='Gene Rearrangement, B-Lymphocyte, Heavy Chain', MS='Ordered rearrangement of B-lymphocyte variable gene regions of the IMMUNOGLOBULIN HEAVY CHAINS, thereby contributing to antibody diversity. It occurs during the first stage of differentiation of the IMMATURE B-LYMPHOCYTES.'),
  'D015328': NtO(MH='Gene Rearrangement, B-Lymphocyte, Light Chain', MS='Ordered rearrangement of B-lymphocyte variable gene regions coding for the kappa or lambda IMMUNOGLOBULIN LIGHT CHAINS, thereby contributing to antibody diversity. It occurs during the second stage of differentiation of the IMMATURE B-LYMPHOCYTES.'),
  'D015329': NtO(MH='Gene Rearrangement, T-Lymphocyte', MS='Ordered rearrangement of T-cell variable gene regions coding for the antigen receptors.'),
  'D015331': NtO(MH='Cohort Studies', MS='Studies in which subsets of a defined population are identified. These groups may or may not be exposed to factors hypothesized to influence the probability of the occurrence of a particular disease or other outcome. Cohorts are defined populations which, as a whole, are followed in an attempt to determine distinguishing subgroup characteristics.'),
  'D015332': NtO(MH='Gene Rearrangement, alpha-Chain T-Cell Antigen Receptor', MS='Ordered rearrangement of T-cell variable gene regions coding for the alpha-chain of antigen receptors.'),
  'D015333': NtO(MH='Gene Rearrangement, beta-Chain T-Cell Antigen Receptor', MS='Ordered rearrangement of T-cell variable gene regions coding for the beta-chain of antigen receptors.'),
  'D015334': NtO(MH='Gene Rearrangement, gamma-Chain T-Cell Antigen Receptor', MS='Ordered rearrangement of T-cell variable gene regions coding for the gamma-chain of antigen receptors.'),
  'D015335': NtO(MH='Molecular Probes', MS='A group of atoms or molecules attached to other molecules or cellular structures and used in studying the properties of these molecules and structures. Radioactive DNA or RNA sequences are used in MOLECULAR GENETICS to detect the presence of a complementary sequence by NUCLEIC ACID HYBRIDIZATION.'),
  'D015336': NtO(MH='Molecular Probe Techniques', MS='The use of devices which use detector molecules to detect, investigate, or analyze other molecules, macromolecules, molecular aggregates, or organisms.'),
  'D015337': NtO(MH='Multicenter Studies as Topic', MS='Works about controlled studies which are planned and carried out by several cooperating institutions to assess certain variables and outcomes in specific patient populations, for example, a multicenter study of congenital anomalies in children.'),
  'D015340': NtO(MH='Epidemiologic Research Design', MS='The form and structure of analytic studies in epidemiologic research.'),
  'D015341': NtO(MH='Nucleic Acid Probes', MS='Nucleic acid which complements a specific mRNA or DNA molecule, or fragment thereof; used for hybridization studies in order to identify microorganisms and for genetic studies.'),
  'D015342': NtO(MH='DNA Probes', MS='Species- or subspecies-specific DNA (including COMPLEMENTARY DNA; conserved genes, whole chromosomes, or whole genomes) used in hybridization studies in order to identify microorganisms, to measure DNA-DNA homologies, to group subspecies, etc. The DNA probe hybridizes with a specific mRNA, if present. Conventional techniques used for testing for the hybridization product include dot blot assays, Southern blot assays, and DNA:RNA hybrid-specific antibody tests. Conventional labels for the DNA probe include the radioisotope labels 32P and 125I and the chemical label biotin. The use of DNA probes provides a specific, sensitive, rapid, and inexpensive replacement for cell culture techniques for diagnosing infections.'),
  'D015343': NtO(MH='DNA Probes, HLA', MS='DNA probes specific for the human leukocyte antigen genes, which represent the major histocompatibility determinants in humans. The four known loci are designated as A, B, C, and D. Specific antigens are identified by a locus notation and number, e.g., HLA-A11. The inheritance of certain HLA alleles is associated with increased risk for certain diseases (e.g., insulin-dependent diabetes mellitus).'),
  'D015344': NtO(MH='DNA Probes, HPV', MS='DNA probes specific for the identification of human papilloma virus.'),
  'D015345': NtO(MH='Oligonucleotide Probes', MS='Synthetic or natural oligonucleotides used in hybridization studies in order to identify and study specific nucleic acid fragments, e.g., DNA segments near or within a specific gene locus or gene. The probe hybridizes with a specific mRNA, if present. Conventional techniques used for testing for the hybridization product include dot blot assays, Southern blot assays, and DNA:RNA hybrid-specific antibody tests. Conventional labels for the probe include the radioisotope labels 32P and 125I and the chemical label biotin.'),
  'D015347': NtO(MH='RNA Probes', MS='RNA, usually prepared by transcription from cloned DNA, which complements a specific mRNA or DNA and is generally used for studies of virus genes, distribution of specific RNA in tissues and cells, integration of viral DNA into genomes, transcription, etc. Whereas DNA PROBES are preferred for use at a more macroscopic level for detection of the presence of DNA/RNA from specific species or subspecies, RNA probes are preferred for genetic studies. Conventional labels for the RNA probe include radioisotope labels 32P and 125I and the chemical label biotin. RNA probes may be further divided by category into plus-sense RNA probes, minus-sense RNA probes, and antisense RNA probes.'),
  'D015348': NtO(MH='Vision, Binocular', MS='The blending of separate images seen by each eye into one composite image.'),
  'D015349': NtO(MH='Vision, Monocular', MS='Images seen by one eye.'),
  'D015350': NtO(MH='Contrast Sensitivity', MS='The ability to detect sharp boundaries (stimuli) and to detect slight changes in luminance at regions without distinct contours. Psychophysical measurements of this visual function are used to evaluate visual acuity and to detect eye disease.'),
  'D015351': NtO(MH='Vision Screening', MS='Application of tests and examinations to identify visual defects or vision disorders occurring in specific populations, as in school children, the elderly, etc. It is differentiated from VISION TESTS, which are given to evaluate/measure individual visual performance not related to a specific population.'),
  'D015352': NtO(MH='Dry Eye Syndromes', MS='Corneal and conjunctival dryness due to deficient tear production, predominantly in menopausal and post-menopausal women. Filamentary keratitis or erosion of the conjunctival and corneal epithelium may be caused by these disorders. Sensation of the presence of a foreign body in the eye and burning of the eyes may occur.'),
  'D015353': NtO(MH='Eye Enucleation', MS='The surgical removal of the eyeball leaving the eye muscles and remaining orbital contents intact.'),
  'D015354': NtO(MH='Vision, Low', MS='Vision considered to be inferior to normal vision as represented by accepted standards of acuity, field of vision, or motility. Low vision generally refers to visual disorders that are caused by diseases that cannot be corrected by refraction (e.g., MACULAR DEGENERATION; RETINITIS PIGMENTOSA; DIABETIC RETINOPATHY, etc.).'),
  'D015355': NtO(MH='Glaucoma, Neovascular', MS='A form of secondary glaucoma which develops as a consequence of another ocular disease and is attributed to the forming of new vessels in the angle of the anterior chamber.'),
  'D015356': NtO(MH='Retinal Artery Occlusion', MS='Sudden ISCHEMIA in the RETINA due to blocked blood flow through the CENTRAL RETINAL ARTERY or its branches leading to sudden complete or partial loss of vision, respectively, in the eye.'),
  'D015357': NtO(MH='Vision Disparity', MS='The difference between two images on the retina when looking at a visual stimulus. This occurs since the two retinas do not have the same view of the stimulus because of the location of our eyes. Thus the left eye does not get exactly the same view as the right eye.'),
  'D015359': NtO(MH='Mycobacterium scrofulaceum', MS='A non-tuberculous mycobacterium causing cervical lymphadenitis in children. It very rarely causes pulmonary disease, and is believed to be non-pathogenic in animals.'),
  'D015360': NtO(MH='Analgesia, Epidural', MS='The relief of pain without loss of consciousness through the introduction of an analgesic agent into the epidural space of the vertebral canal. It is differentiated from ANESTHESIA, EPIDURAL which refers to the state of insensitivity to sensation.'),
  'D015361': NtO(MH='Rhytidoplasty', MS='Plastic surgery performed, usually by excision of skin, for the elimination of wrinkles from the skin.'),
  'D015362': NtO(MH='Child Nutrition Disorders', MS='Disorders caused by nutritional imbalance, either overnutrition or undernutrition, occurring in children ages 2 to 12 years.'),
  'D015363': NtO(MH='Quinolones', MS='A group of derivatives of naphthyridine carboxylic acid, quinoline carboxylic acid, or NALIDIXIC ACID.'),
  'D015365': NtO(MH='Enoxacin', MS='A broad-spectrum 6-fluoronaphthyridinone antibacterial agent that is structurally related to NALIDIXIC ACID.'),
  'D015366': NtO(MH='Pefloxacin', MS='A synthetic broad-spectrum fluoroquinolone antibacterial agent active against most gram-negative and gram-positive bacteria.'),
  'D015367': NtO(MH='Human T-lymphotropic virus 2', MS='A strain of PRIMATE T-LYMPHOTROPIC VIRUS 2 that can transform normal T-lymphocytes and can replicate in both T- and B-cell lines. The virus is related to but distinct from HTLV-1.'),
  'D015368': NtO(MH='Human T-lymphotropic virus 1', MS='A strain of PRIMATE T-LYMPHOTROPIC VIRUS 1 isolated from mature T4 cells in patients with T-lymphoproliferation malignancies. It causes adult T-cell leukemia (LEUKEMIA-LYMPHOMA, T-CELL, ACUTE, HTLV-I-ASSOCIATED), T-cell lymphoma (LYMPHOMA, T-CELL), and is involved in mycosis fungoides, SEZARY SYNDROME and tropical spastic paraparesis (PARAPARESIS, TROPICAL SPASTIC).'),
  'D015370': NtO(MH='Infant Equipment', MS='Equipment and furniture used by infants and babies in the home, car, and play area.'),
  'D015373': NtO(MH='Bacterial Typing Techniques', MS='Procedures for identifying types and strains of bacteria. The most frequently employed typing systems are BACTERIOPHAGE TYPING and SEROTYPING as well as bacteriocin typing and biotyping.'),
  'D015374': NtO(MH='Biosensing Techniques', MS='Any of a variety of procedures which use biomolecular probes to measure the presence or concentration of biological molecules, biological structures, microorganisms, etc., by translating a biochemical interaction at the probe surface into a quantifiable physical signal.'),
  'D015375': NtO(MH='Receptors, Interleukin-2', MS='Receptors present on activated T-LYMPHOCYTES and B-LYMPHOCYTES that are specific for INTERLEUKIN-2 and play an important role in LYMPHOCYTE ACTIVATION. They are heterotrimeric proteins consisting of the INTERLEUKIN-2 RECEPTOR ALPHA SUBUNIT, the INTERLEUKIN-2 RECEPTOR BETA SUBUNIT, and the INTERLEUKIN RECEPTOR COMMON GAMMA-CHAIN.'),
  'D015376': NtO(MH='Nimustine', MS='Antineoplastic agent especially effective against malignant brain tumors. The resistance which brain tumor cells acquire to the initial effectiveness of this drug can be partially overcome by the simultaneous use of membrane-modifying agents such as reserpine, calcium antagonists such as nicardipine or verapamil, or the calmodulin inhibitor, trifluoperazine. The drug has also been used in combination with other antineoplastic agents or with radiotherapy for the treatment of various neoplasms.'),
  'D015377': NtO(MH='Cilastatin', MS='A renal dehydropeptidase-I and leukotriene D4 dipeptidase inhibitor. Since the antibiotic, IMIPENEM, is hydrolyzed by dehydropeptidase-I, which resides in the brush border of the renal tubule, cilastatin is administered with imipenem to increase its effectiveness. The drug also inhibits the metabolism of leukotriene D4 to leukotriene E4.'),
  'D015378': NtO(MH='Imipenem', MS='Semisynthetic thienamycin that has a wide spectrum of antibacterial activity against gram-negative and gram-positive aerobic and anaerobic bacteria, including many multiresistant strains. It is stable to beta-lactamases. Clinical studies have demonstrated high efficacy in the treatment of infections of various body systems. Its effectiveness is enhanced when it is administered in combination with CILASTATIN, a renal dipeptidase inhibitor.'),
  'D015383': NtO(MH='Zona Fasciculata', MS='The wide middle zone of the adrenal cortex. This zone produces a series of enzymes that convert PREGNENOLONE to cortisol (HYDROCORTISONE) via 17-ALPHA-HYDROXYPROGESTERONE.'),
  'D015384': NtO(MH='Zona Glomerulosa', MS='The narrow subcapsular outer zone of the adrenal cortex. This zone produces a series of enzymes that convert PREGNENOLONE to ALDOSTERONE. The final steps involve three successive oxidations by CYTOCHROME P-450 CYP11B2.'),
  'D015385': NtO(MH='Zona Reticularis', MS='The inner zone of the adrenal cortex. This zone produces the enzymes that convert PREGNENOLONE, a 21-carbon steroid, to 19-carbon steroids (DEHYDROEPIANDROSTERONE; and ANDROSTENEDIONE) via 17-ALPHA-HYDROXYPREGNENOLONE.'),
  'D015386': NtO(MH='Hazardous Substances', MS='Elements, compounds, mixtures, or solutions that are considered severely harmful to human health and the environment. They include substances that are toxic, corrosive, flammable, or explosive.'),
  'D015387': NtO(MH='Egg Proteins, Dietary', MS='Proteins found in eggs which are consumed as a food.'),
  'D015388': NtO(MH='Organelles', MS='Specific particles of membrane-bound organized living substances present in eukaryotic cells, such as the MITOCHONDRIA; the GOLGI APPARATUS; ENDOPLASMIC RETICULUM; LYSOSOMES; PLASTIDS; and VACUOLES.'),
  'D015390': NtO(MH='Gastric Bypass', MS='Surgical procedure in which the STOMACH is transected high on the body. The resulting small proximal gastric pouch is joined to any parts of the SMALL INTESTINE by an end-to-side SURGICAL ANASTOMOSIS, depending on the amounts of intestinal surface being bypasses. This procedure is used frequently in the treatment of MORBID OBESITY by limiting the size of functional STOMACH, food intake, and food absorption.'),
  'D015391': NtO(MH='Gastroplasty', MS='Surgical procedures involving the STOMACH and sometimes the lower ESOPHAGUS to correct anatomical defects, or to treat MORBID OBESITY by reducing the size of the stomach. There are several subtypes of bariatric gastroplasty, such as vertical banded gastroplasty, silicone ring vertical gastroplasty, and horizontal banded gastroplasty.'),
  'D015392': NtO(MH='Chemistry, Inorganic', MS='A field of chemistry which pertains to chemical compounds or ions that do not contain the element carbon (with the exception of carbon dioxide and compounds containing a carbonate radical, e.g., calcium carbonate).'),
  'D015393': NtO(MH='Chemistry, Bioinorganic', MS='Field of chemistry pertaining to the study of inorganic compounds or ions and their interactions with biological ligands at the molecular level.'),
  'D015394': NtO(MH='Molecular Structure', MS='The location of the atoms, groups or ions relative to one another in a molecule, as well as the number, type and location of covalent bonds.'),
  'D015395': NtO(MH='Histocompatibility Antigens Class I', MS='Membrane glycoproteins consisting of an alpha subunit and a BETA 2-MICROGLOBULIN beta subunit. In humans, highly polymorphic genes on CHROMOSOME 6 encode the alpha subunits of class I antigens and play an important role in determining the serological specificity of the surface antigen. Class I antigens are found on most nucleated cells and are generally detected by their reactivity with alloantisera. These antigens are recognized during GRAFT REJECTION and restrict cell-mediated lysis of virus-infected cells.'),
  'D015397': NtO(MH='Program Evaluation', MS='Studies designed to assess the efficacy of programs. They may include the evaluation of cost-effectiveness, the extent to which objectives are met, or impact.'),
  'D015398': NtO(MH='Signal Transduction', MS='The intracellular transfer of information (biological activation/inhibition) through a signal pathway. In each signal transduction system, an activation/inhibition signal from a biologically active molecule (hormone, neurotransmitter) is mediated via the coupling of a receptor/enzyme to a second messenger system or to an ion channel. Signal transduction plays an important role in activating cellular functions, cell differentiation, and cell proliferation. Examples of signal transduction systems are the GAMMA-AMINOBUTYRIC ACID-postsynaptic receptor-calcium ion channel system, the receptor-mediated T-cell activation pathway, and the receptor-mediated activation of phospholipases. Those coupled to membrane depolarization or intracellular release of calcium include the receptor-mediated activation of cytotoxic functions in granulocytes and the synaptic potentiation of protein kinase activation. Some signal transduction pathways may be part of larger signal transduction pathways; for example, protein kinase activation is part of the platelet activation signal pathway.'),
  'D015399': NtO(MH='Nursing Research', MS='Research carried out by nurses, generally in clinical settings, in the areas of clinical practice, evaluation, nursing education, nursing administration, and methodology.'),
  'D015400': NtO(MH='Clinical Nursing Research', MS='Research carried out by nurses in the clinical setting and designed to provide information that will help improve patient care. Other professional staff may also participate in the research.'),
  'D015401': NtO(MH='Nursing Administration Research', MS='Research concerned with establishing costs of nursing care, examining the relationships between nursing services and quality patient care, and viewing problems of nursing service delivery within the broader context of policy analysis and delivery of health services (from a national study, presented at the 1985 Council on Graduate Education for Administration in Nursing (CGEAN) meeting).'),
  'D015402': NtO(MH='Nursing Education Research', MS='Investigations into the problems of integrating research findings into nursing curricula, developing problem solving skills, finding approaches to clinical teaching, determining the level of practice by graduates from different basic preparations, etc.'),
  'D015403': NtO(MH='Nursing Evaluation Research', MS='Research carried out by nurses that uses interviews, data collection, observation, surveys, etc., to evaluate nursing, health, clinical, and nursing education programs and curricula, and which also demonstrates the value of such evaluation.'),
  'D015404': NtO(MH='Nursing Methodology Research', MS='Research carried out by nurses concerning techniques and methods to implement projects and to document information, including methods of interviewing patients, collecting data, and forming inferences. The concept includes exploration of methodological issues such as human subjectivity and human experience.'),
  'D015406': NtO(MH='Single Parent', MS='A natural, adoptive, or substitute parent of a dependent child, who lives with only one parent. The single parent may live with or visit the child. The concept includes the never-married, as well as the divorced and widowed.'),
  'D015408': NtO(MH='Gastrinoma', MS='A GASTRIN-secreting neuroendocrine tumor of the non-beta ISLET CELLS, the GASTRIN-SECRETING CELLS. This type of tumor is primarily located in the PANCREAS or the DUODENUM. Majority of gastrinomas are malignant. They metastasize to the LIVER; LYMPH NODES; and BONE but rarely elsewhere. The presence of gastrinoma is one of three requirements to be met for identification of ZOLLINGER-ELLISON SYNDROME, which sometimes occurs in families with MULTIPLE ENDOCRINE NEOPLASIA TYPE 1; (MEN 1).'),
  'D015409': NtO(MH='Mastectomy, Radical', MS='Removal of the breast, pectoral muscles, axillary lymph nodes, and associated skin and subcutaneous tissue.'),
  'D015410': NtO(MH='Mastectomy, Extended Radical', MS='Radical mastectomy with removal of the ipsilateral half of the sternum and a portion of ribs two through five with the underlying pleura and the internal mammary lymph nodes.'),
  'D015411': NtO(MH='Mastectomy, Modified Radical', MS='Total mastectomy with axillary node dissection, but with preservation of the pectoral muscles.'),
  'D015412': NtO(MH='Mastectomy, Segmental', MS='Removal of only enough breast tissue to ensure that the margins of the resected surgical specimen are free of tumor.'),
  'D015413': NtO(MH='Mastectomy, Simple', MS='Removal of only the breast tissue and nipple and a small portion of the overlying skin.'),
  'D015414': NtO(MH='Mastectomy, Subcutaneous', MS='Excision of breast tissue with preservation of overlying skin, nipple, and areola so that breast form may be reconstructed.'),
  'D015415': NtO(MH='Biomarkers', MS='Measurable and quantifiable biological parameters (e.g., specific enzyme concentration, specific hormone concentration, specific gene phenotype distribution in a population, presence of biological substances) which serve as indices for health- and physiology-related assessments, such as disease risk, psychiatric disorders, ENVIRONMENTAL EXPOSURE and its effects, disease diagnosis; METABOLIC PROCESSES; SUBSTANCE ABUSE; PREGNANCY; cell line development; EPIDEMIOLOGIC STUDIES; etc.'),
  'D015416': NtO(MH='Maillard Reaction', MS='A group of nonenzymatic reactions in which aldehydes, ketones, or reducing sugars react with the amino groups of amino acids, peptides, proteins, lipids, or nucleic acids, ultimately resulting in the irreversible formation of ADVANCED GLYCATION END PRODUCTS. Food browning reactions, such as those that occur with cooking with high heat are attributed to this reaction type. This reaction also occurs in vivo and is accelerated under hyperglycemia, oxidative stress, and inflammatory conditions.'),
  'D015417': NtO(MH='Hereditary Sensory and Motor Neuropathy', MS='A group of slowly progressive inherited disorders affecting motor and sensory peripheral nerves. Subtypes include HMSNs I-VII. HMSN I and II both refer to CHARCOT-MARIE-TOOTH DISEASE. HMSN III refers to hypertrophic neuropathy of infancy. HMSN IV refers to REFSUM DISEASE. HMSN V refers to a condition marked by a hereditary motor and sensory neuropathy associated with spastic paraplegia (see SPASTIC PARAPLEGIA, HEREDITARY). HMSN VI refers to HMSN associated with an inherited optic atrophy (OPTIC ATROPHIES, HEREDITARY), and HMSN VII refers to HMSN associated with retinitis pigmentosa. (From Adams et al., Principles of Neurology, 6th ed, p1343)'),
  'D015418': NtO(MH='Optic Atrophies, Hereditary', MS='Hereditary conditions that feature progressive visual loss in association with optic atrophy. Relatively common forms include autosomal dominant optic atrophy (OPTIC ATROPHY, AUTOSOMAL DOMINANT) and Leber hereditary optic atrophy (OPTIC ATROPHY, HEREDITARY, LEBER).'),
  'D015419': NtO(MH='Spastic Paraplegia, Hereditary', MS='A group of inherited diseases that share similar phenotypes but are genetically diverse. Different genetic loci for autosomal recessive, autosomal dominant, and x-linked forms of hereditary spastic paraplegia have been identified. Clinically, patients present with slowly progressive distal limb weakness and lower extremity spasticity. Peripheral sensory neurons may be affected in the later stages of the disease. (J Neurol Neurosurg Psychiatry 1998 Jan;64(1):61-6; Curr Opin Neurol 1997 Aug;10(4):313-8)'),
  'D015422': NtO(MH='Scleral Diseases', MS='General disorders of the sclera or white of the eye. They may include anatomic, embryologic, degenerative, or pigmentation defects.'),
  'D015423': NtO(MH='Scleritis', MS='Refers to any inflammation of the sclera including episcleritis, a benign condition affecting only the episclera, which is generally short-lived and easily treated. Classic scleritis, on the other hand, affects deeper tissue and is characterized by higher rates of visual acuity loss and even mortality, particularly in necrotizing form. Its characteristic symptom is severe and general head pain. Scleritis has also been associated with systemic collagen disease. Etiology is unknown but is thought to involve a local immune response. Treatment is difficult and includes administration of anti-inflammatory and immunosuppressive agents such as corticosteroids. Inflammation of the sclera may also be secondary to inflammation of adjacent tissues, such as the conjunctiva.'),
  'D015424': NtO(MH='Reperfusion', MS='Restoration of blood supply to tissue which is ischemic due to decrease in normal blood supply. The decrease may result from any source including atherosclerotic obstruction, narrowing of the artery, or surgical clamping. It is primarily a procedure for treating infarction or other ischemia, by enabling viable ischemic tissue to recover, thus limiting further necrosis. However, it is thought that reperfusion can itself further damage the ischemic tissue, causing REPERFUSION INJURY.'),
  'D015425': NtO(MH='Myocardial Reperfusion', MS='Generally, restoration of blood supply to heart tissue which is ischemic due to decrease in normal blood supply. The decrease may result from any source including atherosclerotic obstruction, narrowing of the artery, or surgical clamping. Reperfusion can be induced to treat ischemia. Methods include chemical dissolution of an occluding thrombus, administration of vasodilator drugs, angioplasty, catheterization, and artery bypass graft surgery. However, it is thought that reperfusion can itself further damage the ischemic tissue, causing MYOCARDIAL REPERFUSION INJURY.'),
  'D015427': NtO(MH='Reperfusion Injury', MS='Adverse functional, metabolic, or structural changes in ischemic tissues resulting from the restoration of blood flow to the tissue (REPERFUSION), including swelling; HEMORRHAGE; NECROSIS; and damage from FREE RADICALS. The most common instance is MYOCARDIAL REPERFUSION INJURY.'),
  'D015428': NtO(MH='Myocardial Reperfusion Injury', MS='Damage to the MYOCARDIUM resulting from MYOCARDIAL REPERFUSION (restoration of blood flow to ischemic areas of the HEART.) Reperfusion takes place when there is spontaneous thrombolysis, THROMBOLYTIC THERAPY, collateral flow from other coronary vascular beds, or reversal of vasospasm.'),
  'D015430': NtO(MH='Weight Gain', MS='Increase in BODY WEIGHT over existing weight.'),
  'D015431': NtO(MH='Weight Loss', MS='Decrease in existing BODY WEIGHT.'),
  'D015432': NtO(MH='Glomerulonephritis, Membranoproliferative', MS='Chronic glomerulonephritis characterized histologically by proliferation of MESANGIAL CELLS, increase in the MESANGIAL EXTRACELLULAR MATRIX, and a thickening of the glomerular capillary walls. This may appear as a primary disorder or secondary to other diseases including infections and autoimmune disease SYSTEMIC LUPUS ERYTHEMATOSUS. Various subtypes are classified by their abnormal ultrastructures and immune deposits. Hypocomplementemia is a characteristic feature of all types of MPGN.'),
  'D015433': NtO(MH='Glomerulonephritis, Membranous', MS='A type of glomerulonephritis that is characterized by the accumulation of immune deposits (COMPLEMENT MEMBRANE ATTACK COMPLEX) on the outer aspect of the GLOMERULAR BASEMENT MEMBRANE. It progresses from subepithelial dense deposits, to basement membrane reaction and eventual thickening of the basement membrane.'),
  'D015434': NtO(MH='Panniculitis', MS='General term for inflammation of adipose tissue, usually of the skin, characterized by reddened subcutaneous nodules.'),
  'D015435': NtO(MH='Panniculitis, Lupus Erythematosus', MS='A type of lupus erythematosus characterized by deep dermal or subcutaneous nodules, most often on the head, face, or upper arms. It is generally chronic and occurs most often in women between the ages of 20 and 45.'),
  'D015436': NtO(MH='Panniculitis, Peritoneal', MS='INFLAMMATION of the underlying layer of ADIPOSE TISSUE (panniculus) of the PERITONEUM, usually of the MESENTERY or the OMENTUM. There are several forms with various names and are usually characterized by infiltration of LYMPHOCYTES and NEUTROPHILS, fat NECROSIS, and FIBROSIS.'),
  'D015438': NtO(MH='Health Behavior', MS='Combination of HEALTH KNOWLEDGE, ATTITUDES, PRACTICE which underlie actions taken by individuals regarding their health.'),
  'D015439': NtO(MH='Leprosy, Borderline', MS='A form of LEPROSY in which there are clinical manifestations of both principal types (lepromatous and tuberculoid). The disease may shift toward one of these two polar or principal forms.'),
  'D015440': NtO(MH='Leprosy, Lepromatous', MS='A chronic communicable infection which is a principal or polar form of LEPROSY. This disorder is caused by MYCOBACTERIUM LEPRAE and produces diffuse granulomatous skin lesions in the form of nodules, macules, or papules. The peripheral nerves are involved symmetrically and neural sequelae occur in the advanced stage.'),
  'D015441': NtO(MH='Leprosy, Tuberculoid', MS='A principal or polar form of LEPROSY in which the skin lesions are few and are sharply demarcated. Peripheral nerve involvement is pronounced and may be severe. Unlike lepromatous leprosy (LEPROSY, LEPROMATOUS), the lepromin test is positive. Tuberculoid leprosy is rarely a source of infection to others.'),
  'D015442': NtO(MH='Medicare Assignment', MS='Concept referring to the standardized fees for services rendered by health care providers, e.g., laboratories and physicians, and reimbursement for those services under Medicare Part B. It includes acceptance by the physician.'),
  'D015443': NtO(MH='Word Processing', MS='Text editing and storage functions using computer software.'),
  'D015444': NtO(MH='Exercise', MS='Physical activity which is usually regular and done with the intention of improving or maintaining PHYSICAL FITNESS or HEALTH. Contrast with PHYSICAL EXERTION which is concerned largely with the physiologic and metabolic response to energy expenditure.'),
  'D015448': NtO(MH='Leukemia, B-Cell', MS='A malignant disease of the B-LYMPHOCYTES in the bone marrow and/or blood.'),
  'D015451': NtO(MH='Leukemia, Lymphocytic, Chronic, B-Cell', MS='A chronic leukemia characterized by abnormal B-lymphocytes and often generalized lymphadenopathy. In patients presenting predominately with blood and bone marrow involvement it is called chronic lymphocytic leukemia (CLL); in those predominately with enlarged lymph nodes it is called small lymphocytic lymphoma. These terms represent spectrums of the same disease.'),
  'D015452': NtO(MH='Precursor B-Cell Lymphoblastic Leukemia-Lymphoma', MS='A leukemia/lymphoma found predominately in children and adolescents and characterized by a high number of lymphoblasts and solid tumor lesions. Frequent sites involve LYMPH NODES, skin, and bones. It most commonly presents as leukemia.'),
  'D015456': NtO(MH='Leukemia, Biphenotypic, Acute', MS='An acute leukemia exhibiting cell features characteristic of both the myeloid and lymphoid lineages and probably arising from MULTIPOTENT STEM CELLS.'),
  'D015458': NtO(MH='Leukemia, T-Cell', MS='A malignant disease of the T-LYMPHOCYTES in the bone marrow, thymus, and/or blood.'),
  'D015459': NtO(MH='Leukemia-Lymphoma, Adult T-Cell', MS='Aggressive T-Cell malignancy with adult onset, caused by HUMAN T-LYMPHOTROPIC VIRUS 1. It is endemic in Japan, the Caribbean basin, Southeastern United States, Hawaii, and parts of Central and South America and sub-Saharan Africa.'),
  'D015461': NtO(MH='Leukemia, Prolymphocytic, T-Cell', MS='A lymphoid leukemia characterized by a profound LYMPHOCYTOSIS with or without LYMPHADENOPATHY, hepatosplenomegaly, frequently rapid progression, and short survival. It was formerly called T-cell chronic lymphocytic leukemia.'),
  'D015463': NtO(MH='Leukemia, Prolymphocytic', MS='A chronic leukemia characterized by a large number of circulating prolymphocytes. It can arise spontaneously or as a consequence of transformation of CHRONIC LYMPHOCYTIC LEUKEMIA.'),
  'D015464': NtO(MH='Leukemia, Myelogenous, Chronic, BCR-ABL Positive', MS='Clonal hematopoetic disorder caused by an acquired genetic defect in PLURIPOTENT STEM CELLS. It starts in MYELOID CELLS of the bone marrow, invades the blood and then other organs. The condition progresses from a stable, more indolent, chronic phase (LEUKEMIA, MYELOID, CHRONIC PHASE) lasting up to 7 years, to an advanced phase composed of an accelerated phase (LEUKEMIA, MYELOID, ACCELERATED PHASE) and BLAST CRISIS.'),
  'D015465': NtO(MH='Leukemia, Myeloid, Accelerated Phase', MS='The phase of chronic myeloid leukemia following the chronic phase (LEUKEMIA, MYELOID, CHRONIC-PHASE), where there are increased systemic symptoms, worsening cytopenias, and refractory LEUKOCYTOSIS.'),
  'D015466': NtO(MH='Leukemia, Myeloid, Chronic-Phase', MS='The initial phase of chronic myeloid leukemia consisting of an relatively indolent period lasting from 4 to 7 years. Patients range from asymptomatic to those exhibiting ANEMIA; SPLENOMEGALY; and increased cell turnover. There are 5% or fewer blast cells in the blood and bone marrow in this phase.'),
  'D015467': NtO(MH='Leukemia, Neutrophilic, Chronic', MS='A rare myeloproliferative disorder that is characterized by a sustained, mature neutrophilic leukocytosis. No monocytosis, EOSINOPHILIA, or basophilia is present, nor is there a PHILADELPHIA CHROMOSOME or bcr-abl fusion gene (GENES, ABL).'),
  'D015470': NtO(MH='Leukemia, Myeloid, Acute', MS='Clonal expansion of myeloid blasts in bone marrow, blood, and other tissue. Myeloid leukemias develop from changes in cells that normally produce NEUTROPHILS; BASOPHILS; EOSINOPHILS; and MONOCYTES.'),
  'D015471': NtO(MH='Leukemia, Basophilic, Acute', MS='A rare acute myeloid leukemia in which the primary differentiation is to BASOPHILS. It is characterized by an extreme increase of immature basophilic granulated cells in the bone marrow and blood. Mature basophils are usually sparse.'),
  'D015472': NtO(MH='Leukemia, Eosinophilic, Acute', MS='A rare acute myeloid leukemia characterized by abnormal EOSINOPHILS in the bone marrow.'),
  'D015473': NtO(MH='Leukemia, Promyelocytic, Acute', MS='An acute myeloid leukemia in which abnormal PROMYELOCYTES predominate. It is frequently associated with DISSEMINATED INTRAVASCULAR COAGULATION.'),
  'D015474': NtO(MH='Isotretinoin', MS='A topical dermatologic agent that is used in the treatment of ACNE VULGARIS and several other skin diseases. The drug has teratogenic and other adverse effects.'),
  'D015477': NtO(MH='Leukemia, Myelomonocytic, Chronic', MS='A myelodysplastic-myeloproliferative disease characterized by monocytosis, increased monocytes in the bone marrow, variable degrees of dysplasia, but an absence of immature granulocytes in the blood.'),
  'D015478': NtO(MH='Antigens, Heterophile', MS='Antigens stimulating the formation of, or combining with heterophile antibodies. They are cross-reacting antigens found in phylogenetically unrelated species.'),
  'D015479': NtO(MH='Leukemia, Myelomonocytic, Acute', MS='A pediatric acute myeloid leukemia involving both myeloid and monocytoid precursors. At least 20% of non-erythroid cells are of monocytic origin.'),
  'D015480': NtO(MH='Deltaretrovirus Antibodies', MS='Antibodies reactive with various types of human T-cell leukemia/lymphoma antigens or bovine leukemia virus antigens.'),
  'D015481': NtO(MH='HTLV-I Antibodies', MS='Antibodies reactive with the HTLV-I ANTIGENS.'),
  'D015482': NtO(MH='HTLV-II Antibodies', MS='Antibodies reactive with the HTLV-II ANTIGENS.'),
  'D015483': NtO(MH='HIV Antibodies', MS='Antibodies reactive with HIV ANTIGENS.'),
  'D015485': NtO(MH='Deltaretrovirus Antigens', MS='Antigens associated with the DELTARETROVIRUS; HTLV-I ANTIGENS and HTLV-II ANTIGENS belong to this group.'),
  'D015486': NtO(MH='HTLV-I Antigens', MS='Antigens associated with HUMAN T-LYMPHOTROPIC VIRUS 1.'),
  'D015487': NtO(MH='HTLV-II Antigens', MS='Antigens associated with HUMAN T-LYMPHOTROPIC VIRUS 2.'),
  'D015488': NtO(MH='HIV Antigens', MS='Antigens associated with specific proteins of the human adult T-cell immunodeficiency virus (HIV); also called HTLV-III-associated and lymphadenopathy-associated virus (LAV) antigens.'),
  'D015490': NtO(MH='HTLV-I Infections', MS='Diseases caused by HUMAN T-LYMPHOTROPIC VIRUS 1.'),
  'D015491': NtO(MH='HTLV-II Infections', MS='Diseases caused by HUMAN T-LYMPHOTROPIC VIRUS 2.'),
  'D015492': NtO(MH='AIDS Serodiagnosis', MS='Immunologic tests for identification of HIV (HTLV-III/LAV) antibodies. They include assays for HIV SEROPOSITIVITY and HIV SERONEGATIVITY that have been developed for screening persons carrying the viral antibody from patients with overt symptoms of AIDS or AIDS-RELATED COMPLEX.'),
  'D015493': NtO(MH='Paraparesis, Tropical Spastic', MS='A subacute paralytic myeloneuropathy occurring endemically in tropical areas such as the Caribbean, Colombia, India, and Africa, as well as in the southwestern region of Japan; associated with infection by HUMAN T-CELL LEUKEMIA VIRUS I. Clinical manifestations include a slowly progressive spastic weakness of the legs, increased reflexes, Babinski signs, incontinence, and loss of vibratory and position sensation. On pathologic examination inflammatory, demyelination, and necrotic lesions may be found in the spinal cord. (Adams et al., Principles of Neurology, 6th ed, p1239)'),
  'D015494': NtO(MH='Salivary Gland Calculi', MS='Calculi occurring in a salivary gland. Most salivary gland calculi occur in the submandibular gland, but can also occur in the parotid gland and in the sublingual and minor salivary glands.'),
  'D015496': NtO(MH='CD4-Positive T-Lymphocytes', MS='A critical subpopulation of T-lymphocytes involved in the induction of most immunological functions. The HIV virus has selective tropism for the T4 cell which expresses the CD4 phenotypic marker, a receptor for HIV. In fact, the key element in the profound immunosuppression seen in HIV infection is the depletion of this subset of T-lymphocytes.'),
  'D015497': NtO(MH='HIV-1', MS='The type species of LENTIVIRUS and the etiologic agent of AIDS. It is characterized by its cytopathic effect and affinity for the T4-lymphocyte.'),
  'D015498': NtO(MH='HIV-2', MS='An HIV species related to HIV-1 but carrying different antigenic components and with differing nucleic acid composition. It shares serologic reactivity and sequence homology with the simian Lentivirus SIMIAN IMMUNODEFICIENCY VIRUS and infects only T4-lymphocytes expressing the CD4 phenotypic marker.'),
  'D015499': NtO(MH='Renal Tubular Transport, Inborn Errors', MS='Genetic defects in the selective or non-selective transport functions of the KIDNEY TUBULES.'),
  'D015500': NtO(MH='Chloramphenicol O-Acetyltransferase', MS='An enzyme that catalyzes the acetylation of chloramphenicol to yield chloramphenicol 3-acetate. Since chloramphenicol 3-acetate does not bind to bacterial ribosomes and is not an inhibitor of peptidyltransferase, the enzyme is responsible for the naturally occurring chloramphenicol resistance in bacteria. The enzyme, for which variants are known, is found in both gram-negative and gram-positive bacteria. EC 2.3.1.28.'),
  'D015502': NtO(MH='Absorptiometry, Photon', MS='A noninvasive method for assessing BODY COMPOSITION. It is based on the differential absorption of X-RAYS (or GAMMA RAYS) by different tissues such as bone, fat and other soft tissues. The source of (X-ray or gamma-ray) photon beam is generated either from radioisotopes such as GADOLINIUM 153, IODINE 125, or Americanium 241 which emit GAMMA RAYS in the appropriate range; or from an X-ray tube which produces X-RAYS in the desired range. It is primarily used for quantitating BONE MINERAL CONTENT, especially for the diagnosis of OSTEOPOROSIS, and also in measuring BONE MINERALIZATION.'),
  'D015503': NtO(MH='Surgical Sponges', MS='Gauze material used to absorb body fluids during surgery. Referred to as GOSSYPIBOMA if accidentally retained in the body following surgery.'),
  'D015505': NtO(MH='Chest Tubes', MS='Plastic tubes used for drainage of air or fluid from the pleural space. Their surgical insertion is called tube thoracostomy.'),
  'D015507': NtO(MH='Drugs, Investigational', MS='Drugs which have received FDA approval for human testing but have yet to be approved for commercial marketing. This includes drugs used for treatment while they still are undergoing clinical trials (Treatment IND). The main heading includes drugs under investigation in foreign countries.'),
  'D015508': NtO(MH='Nasal Obstruction', MS='Any hindrance to the passage of air into and out of the nose. The obstruction may be unilateral or bilateral, and may involve any part of the NASAL CAVITY.'),
  'D015509': NtO(MH='Developmental Biology', MS='The field of biology which deals with the process of the growth and differentiation of an organism.'),
  'D015510': NtO(MH='Clinical Medicine', MS='The study and practice of medicine by direct examination of the patient.'),
  'D015511': NtO(MH='Goat Diseases', MS='Diseases of the domestic or wild goat of the genus Capra.'),
  'D015513': NtO(MH='Oncogene Proteins', MS='Proteins coded by oncogenes. They include proteins resulting from the fusion of an oncogene and another gene (ONCOGENE PROTEINS, FUSION).'),
  'D015514': NtO(MH='Oncogene Proteins, Fusion', MS='The GENETIC TRANSLATION products of the fusion between an ONCOGENE and another gene. The latter may be of viral or cellular origin.'),
  'D015518': NtO(MH='Rett Syndrome', MS='An inherited neurological developmental disorder that is associated with X-LINKED INHERITANCE and may be lethal in utero to hemizygous males. The affected female is normal until the age of 6-25 months when progressive loss of voluntary control of hand movements and communication skills; ATAXIA; SEIZURES; autistic behavior; intermittent HYPERVENTILATION; and HYPERAMMONEMIA appear. (From Menkes, Textbook of Child Neurology, 5th ed, p199)'),
  'D015519': NtO(MH='Bone Density', MS='The amount of mineral per square centimeter of BONE. This is the definition used in clinical practice. Actual bone density would be expressed in grams per milliliter. It is most frequently measured by X-RAY ABSORPTIOMETRY or TOMOGRAPHY, X RAY COMPUTED. Bone density is an important predictor for OSTEOPOROSIS.'),
  'D015521': NtO(MH='Ethmoid Sinusitis', MS='Inflammation of the NASAL MUCOSA in the ETHMOID SINUS. It may present itself as an acute (infectious) or chronic (allergic) condition.'),
  'D015522': NtO(MH='Frontal Sinusitis', MS='Inflammation of the NASAL MUCOSA in the FRONTAL SINUS. In many cases, it is caused by an infection of the bacteria STREPTOCOCCUS PNEUMONIAE or HAEMOPHILUS INFLUENZAE.'),
  'D015523': NtO(MH='Maxillary Sinusitis', MS='Inflammation of the NASAL MUCOSA in the MAXILLARY SINUS. In many cases, it is caused by an infection of the bacteria HAEMOPHILUS INFLUENZAE; STREPTOCOCCUS PNEUMONIAE; or STAPHYLOCOCCUS AUREUS.'),
  'D015524': NtO(MH='Sphenoid Sinusitis', MS='Inflammation of the NASAL MUCOSA in the SPHENOID SINUS. Isolated sphenoid sinusitis is uncommon. It usually occurs in conjunction with other paranasal sinusitis.'),
  'D015525': NtO(MH='Fatty Acids, Omega-3', MS='A group of unsaturated fatty acids occurring mainly in fish oils, with three double bonds at particular positions in the hydrocarbon chain.'),
  'D015526': NtO(MH='AIDS Dementia Complex', MS='A neurologic condition associated with the ACQUIRED IMMUNODEFICIENCY SYNDROME and characterized by impaired concentration and memory, slowness of hand movements, ATAXIA, incontinence, apathy, and gait difficulties associated with HIV-1 viral infection of the central nervous system. Pathologic examination of the brain reveals white matter rarefaction, perivascular infiltrates of lymphocytes, foamy macrophages, and multinucleated giant cells. (From Adams et al., Principles of Neurology, 6th ed, pp760-1; N Engl J Med, 1995 Apr 6;332(14):934-40)'),
  'D015528': NtO(MH='HIV Seroprevalence', MS='Studies of the number of cases where human immunodeficiency virus (HIV) is present in a specific population at a designated time. The presence in a given individual is determined by the finding of HIV antibodies in the serum (HIV SEROPOSITIVITY).'),
  'D015529': NtO(MH='Choledochal Cyst', MS='A congenital anatomic malformation of a bile duct, including cystic dilatation of the extrahepatic bile duct or the large intrahepatic bile duct. Classification is based on the site and type of dilatation. Type I is most common.'),
  'D015530': NtO(MH='Nuclear Matrix', MS='The residual framework structure of the CELL NUCLEUS that maintains many of the overall architectural features of the cell nucleus including the nuclear lamina with NUCLEAR PORE complex structures, residual CELL NUCLEOLI and an extensive fibrogranular structure in the nuclear interior. (Advan. Enzyme Regul. 2002; 42:39-52)'),
  'D015531': NtO(MH='Radioimmunoprecipitation Assay', MS='Sensitive assay using radiolabeled ANTIGENS to detect specific ANTIBODIES in SERUM. The antigens are allowed to react with the serum and then precipitated using a special reagent such as PROTEIN A sepharose beads. The bound radiolabeled immunoprecipitate is then commonly analyzed by gel electrophoresis.'),
  'D015532': NtO(MH='Rehydration Solutions', MS='Fluids restored to the body in order to maintain normal water-electrolyte balance.'),
  'D015533': NtO(MH='Transcriptional Activation', MS='Processes that stimulate the GENETIC TRANSCRIPTION of a gene or set of genes.'),
  'D015534': NtO(MH='Trans-Activators', MS='Diffusible gene products that act on homologous or heterologous molecules of viral or cellular DNA to regulate the expression of proteins.'),
  'D015535': NtO(MH='Arthritis, Psoriatic', MS='A type of inflammatory arthritis associated with PSORIASIS, often involving the axial joints and the peripheral terminal interphalangeal joints. It is characterized by the presence of HLA-B27-associated SPONDYLARTHROPATHY, and the absence of rheumatoid factor.'),
  'D015536': NtO(MH='Down-Regulation', MS='A negative regulatory effect on physiological processes at the molecular, cellular, or systemic level. At the molecular level, the major regulatory sites include membrane receptors, genes (GENE EXPRESSION REGULATION), mRNAs (RNA, MESSENGER), and proteins.'),
  'D015538': NtO(MH='Nitrosation', MS='Conversion into nitroso compounds. An example is the reaction of nitrites with amino compounds to form carcinogenic N-nitrosamines.'),
  'D015539': NtO(MH='Platelet Activation', MS='A series of progressive, overlapping events, triggered by exposure of the PLATELETS to subendothelial tissue. These events include shape change, adhesiveness, aggregation, and release reactions. When carried through to completion, these events lead to the formation of a stable hemostatic plug.'),
  'D015541': NtO(MH='Off-Road Motor Vehicles', MS='Motorized, recreational vehicles used on non-public roads. They include all-terrain vehicles, dirt-bikes, minibikes, motorbikes, trailbikes, and snowmobiles. Excludes MOTORCYCLES, which are considered public road vehicles.'),
  'D015544': NtO(MH='Inositol 1,4,5-Trisphosphate', MS="Intracellular messenger formed by the action of phospholipase C on phosphatidylinositol 4,5-bisphosphate, which is one of the phospholipids that make up the cell membrane. Inositol 1,4,5-trisphosphate is released into the cytoplasm where it releases calcium ions from internal stores within the cell's endoplasmic reticulum. These calcium ions stimulate the activity of B kinase or calmodulin."),
  'D015549': NtO(MH='Basophil Degranulation Test', MS="An in vitro test used in the diagnosis of allergies including drug hypersensitivity. The allergen is added to the patient's white blood cells and the subsequent histamine release is measured."),
  'D015550': NtO(MH='Cell Degranulation', MS='The process of losing secretory granules (SECRETORY VESICLES). This occurs, for example, in mast cells, basophils, neutrophils, eosinophils, and platelets when secretory products are released from the granules by EXOCYTOSIS.'),
  'D015551': NtO(MH='Autoimmunity', MS="Process whereby the immune system reacts against the body's own tissues. Autoimmunity may produce or be caused by AUTOIMMUNE DISEASES."),
  'D015552': NtO(MH='Injections, Intralesional', MS='Injections introduced directly into localized lesions.'),
  'D015570': NtO(MH='Josamycin', MS='A macrolide antibiotic from Streptomyces narbonensis. The drug has antimicrobial activity against a wide spectrum of pathogens.'),
  'D015571': NtO(MH='Follicular Fluid', MS='The fluid surrounding the OVUM and GRANULOSA CELLS in the Graafian follicle (OVARIAN FOLLICLE). The follicular fluid contains sex steroids, glycoprotein hormones, plasma proteins, mucopolysaccharides, and enzymes.'),
  'D015572': NtO(MH='Spiramycin', MS='A macrolide antibiotic produced by Streptomyces ambofaciens. The drug is effective against gram-positive aerobic pathogens, N. gonorrhoeae, and staphylococci. It is used to treat infections caused by bacteria and Toxoplasma gondii.'),
  'D015575': NtO(MH='Roxithromycin', MS='Semisynthetic derivative of erythromycin. It is concentrated by human phagocytes and is bioactive intracellularly. While the drug is active against a wide spectrum of pathogens, it is particularly effective in the treatment of respiratory and genital tract infections.'),
  'D015576': NtO(MH='Hyperostosis', MS='Increase in the mass of bone per unit volume.'),
  'D015577': NtO(MH='Geriatric Assessment', MS='Evaluation of the level of physical, physiological, or mental functioning in the older population group.'),
  'D015580': NtO(MH='Mohs Surgery', MS='A surgical technique used primarily in the treatment of skin neoplasms, especially basal cell or squamous cell carcinoma of the skin. This procedure is a microscopically controlled excision of cutaneous tumors either after fixation in vivo or after freezing the tissue. Serial examinations of fresh tissue specimens are most frequently done.'),
  'D015588': NtO(MH='Observer Variation', MS="The failure by the observer to measure or identify a phenomenon accurately, which results in an error. Sources for this may be due to the observer's missing an abnormality, or to faulty technique resulting in incorrect test measurement, or to misinterpretation of the data. Two varieties are inter-observer variation (the amount observers vary from one another when reporting on the same material) and intra-observer variation (the amount one observer varies between observations when reporting more than once on the same material)."),
  'D015590': NtO(MH='Fluorophotometry', MS='Measurement of light given off by fluorescein in order to assess the integrity of various ocular barriers. The method is used to investigate the blood-aqueous barrier, blood-retinal barrier, aqueous flow measurements, corneal endothelial permeability, and tear flow dynamics.'),
  'D015591': NtO(MH='Microspectrophotometry', MS='Analytical technique for studying substances present at enzyme concentrations in single cells, in situ, by measuring light absorption. Light from a tungsten strip lamp or xenon arc dispersed by a grating monochromator illuminates the optical system of a microscope. The absorbance of light is measured (in nanometers) by comparing the difference between the image of the sample and a reference image.'),
  'D015592': NtO(MH='Immunoradiometric Assay', MS='Form of radioimmunoassay in which excess specific labeled antibody is added directly to the test antigen being measured.'),
  'D015593': NtO(MH='Retinal Drusen', MS='Colloid or hyaline bodies lying beneath the retinal pigment epithelium. They may occur either secondary to changes in the choroid that affect the pigment epithelium or as an autosomal dominant disorder of the retinal pigment epithelium.'),
  'D015594': NtO(MH='Optic Disk Drusen', MS='Optic disk bodies composed primarily of acid mucopolysaccharides that may produce pseudopapilledema (elevation of the optic disk without associated INTRACRANIAL HYPERTENSION) and visual field deficits. Drusen may also occur in the retina (see RETINAL DRUSEN). (Miller et al., Clinical Neuro-Ophthalmology, 4th ed, p355)'),
  'D015595': NtO(MH='Skin Aging', MS='The process of aging due to changes in the structure and elasticity of the skin over time. It may be a part of physiological aging or it may be due to the effects of ultraviolet radiation, usually through exposure to sunlight.'),
  'D015596': NtO(MH='Nutrition Assessment', MS='Evaluation and measurement of nutritional variables in order to assess the level of nutrition or the NUTRITIONAL STATUS of the individual. NUTRITION SURVEYS may be used in making the assessment.'),
  'D015597': NtO(MH='Pregnancy Complications, Parasitic', MS='The co-occurrence of pregnancy and parasitic diseases. The parasitic infection may precede or follow FERTILIZATION.'),
  'D015599': NtO(MH='Trauma Severity Indices', MS='Systems for assessing, classifying, and coding injuries. These systems are used in medical records, surveillance systems, and state and national registries to aid in the collection and reporting of trauma.'),
  'D015600': NtO(MH='Glasgow Coma Scale', MS='A scale that assesses the response to stimuli in patients with craniocerebral injuries. The parameters are eye opening, motor response, and verbal response.'),
  'D015601': NtO(MH='Injury Severity Score', MS='An anatomic severity scale based on the Abbreviated Injury Scale (AIS) and developed specifically to score multiple traumatic injuries. It has been used as a predictor of mortality.'),
  'D015603': NtO(MH='Keratinocytes', MS='Epidermal cells which synthesize keratin and undergo characteristic changes as they move upward from the basal layers of the epidermis to the cornified (horny) layer of the skin. Successive stages of differentiation of the keratinocytes forming the epidermal layers are basal cell, spinous or prickle cell, and the granular cell.'),
  'D015605': NtO(MH='Cowpox', MS='A mild, eruptive skin disease of milk cows caused by COWPOX VIRUS, with lesions occurring principally on the udder and teats. Human infection may occur while milking an infected animal.'),
  'D015607': NtO(MH='Stents', MS='Devices that provide support for tubular structures that are being anastomosed or for body cavities during skin grafting.'),
  'D015608': NtO(MH='Cowpox virus', MS='A species of ORTHOPOXVIRUS that is the etiologic agent of COWPOX. It is closely related to but antigenically different from VACCINIA VIRUS.'),
  'D015609': NtO(MH='Organotechnetium Compounds', MS='Organic compounds that contain technetium as an integral part of the molecule. These compounds are often used as radionuclide imaging agents.'),
  'D015610': NtO(MH='Serum Bactericidal Test', MS="Method of measuring the bactericidal activity contained in a patient's serum as a result of antimicrobial therapy. It is used to monitor the therapy in BACTERIAL ENDOCARDITIS; OSTEOMYELITIS and other serious bacterial infections. As commonly performed, the test is a variation of the broth dilution test. This test needs to be distinguished from testing of the naturally occurring BLOOD BACTERICIDAL ACTIVITY."),
  'D015611': NtO(MH='Hepatitis B Virus, Duck', MS='A DNA virus that closely resembles human hepatitis B virus. It has been recovered from naturally infected ducks.'),
  'D015612': NtO(MH='Phonophoresis', MS='Use of ultrasound to increase the percutaneous adsorption of drugs.'),
  'D015613': NtO(MH='Hepadnaviridae', MS='A family of hepatotropic DNA viruses which contains double-stranded DNA genomes and causes hepatitis in humans and animals. There are two genera: AVIHEPADNAVIRUS and ORTHOHEPADNAVIRUS. Hepadnaviruses include HEPATITIS B VIRUS, duck hepatitis B virus (HEPATITIS B VIRUS, DUCK), heron hepatitis B virus, ground squirrel hepatitis virus, and woodchuck hepatitis B virus (HEPATITIS B VIRUS, WOODCHUCK).'),
  'D015614': NtO(MH='Histiocytosis', MS='General term for the abnormal appearance of histiocytes in the blood. Based on the pathological features of the cells involved rather than on clinical findings, the histiocytic diseases are subdivided into three groups: HISTIOCYTOSIS, LANGERHANS CELL; HISTIOCYTOSIS, NON-LANGERHANS-CELL; and HISTIOCYTIC DISORDERS, MALIGNANT.'),
  'D015615': NtO(MH='Cystic Adenomatoid Malformation of Lung, Congenital', MS='An abnormality in lung development that is characterized by a multicystic mass resulting from an adenomatous overgrowth of the terminal BRONCHIOLES with a consequent reduction of PULMONARY ALVEOLI. This anomaly is classified into three types by the cyst size.'),
  'D015616': NtO(MH='Histiocytosis, Non-Langerhans-Cell', MS='Group of disorders which feature accumulations of active HISTIOCYTES and LYMPHOCYTES, but where the histiocytes are not LANGERHANS CELLS. The group includes HEMOPHAGOCYTIC LYMPHOHISTIOCYTOSIS; SINUS HISTIOCYTOSIS; xanthogranuloma; reticulohistiocytoma; JUVENILE XANTHOGRANULOMA; xanthoma disseminatum; as well as the lipid storage diseases (SEA-BLUE HISTIOCYTE SYNDROME; and NIEMANN-PICK DISEASES).'),
  'D015618': NtO(MH='Histiocytosis, Sinus', MS='Benign, non-Langerhans-cell, histiocytic proliferative disorder that primarily affects the lymph nodes. It is often referred to as sinus histiocytosis with massive lymphadenopathy.'),
  'D015619': NtO(MH='Respiratory System Abnormalities', MS='Congenital structural abnormalities of the respiratory system.'),
  'D015620': NtO(MH='Histiocytic Disorders, Malignant', MS='Distinctive neoplastic disorders of histiocytes. Included are malignant neoplasms of MACROPHAGES and DENDRITIC CELLS.'),
  'D015622': NtO(MH='Epithelioid Cells', MS='Characteristic cells of granulomatous hypersensitivity. They appear as large, flattened cells with increased endoplasmic reticulum. They are believed to be activated macrophages that have differentiated as a result of prolonged antigenic stimulation. Further differentiation or fusion of epithelioid cells is thought to produce multinucleated giant cells (GIANT CELLS).'),
  'D015623': NtO(MH='Diet, Macrobiotic', MS='An approach to nutrition based on whole cereal grains, beans, cooked vegetables and the Chinese YIN-YANG principle. It advocates a diet consisting of organic and locally grown foods, seasonal vegetables, complex carbohydrates, and fewer fats, sugars, and chemically processed foods.'),
  'D015624': NtO(MH='Lambert-Eaton Myasthenic Syndrome', MS='An autoimmune disease characterized by weakness and fatigability of proximal muscles, particularly of the pelvic girdle, lower extremities, trunk, and shoulder girdle. There is relative sparing of extraocular and bulbar muscles. CARCINOMA, SMALL CELL of the lung is a frequently associated condition, although other malignancies and autoimmune diseases may be associated. Muscular weakness results from impaired impulse transmission at the NEUROMUSCULAR JUNCTION. Presynaptic calcium channel dysfunction leads to a reduced amount of acetylcholine being released in response to stimulation of the nerve. (From Adams et al., Principles of Neurology, 6th ed, pp 1471)'),
  'D015625': NtO(MH='Tissue Expansion Devices', MS='Devices used to generate extra soft tissue in vivo to be used in surgical reconstructions. They exert stretching forces on the tissue and thus stimulate new growth and result in TISSUE EXPANSION. They are commonly inflatable reservoirs, usually made of silicone, which are implanted under the tissue and gradually inflated. Other tissue expanders exert stretching forces by attaching to outside of the body, for example, vacuum tissue expanders. Once the tissue has grown, the expander is removed and the expanded tissue is used to cover the area being reconstructed.'),
  'D015626': NtO(MH='Tissue Expansion', MS='A procedure whereby the body is stimulated to generate extra soft tissue by the application of stretching forces that stimulate new growth of tissue which, over a period of time, results in a 2-dimensional expansion of the tissue. The procedure is used in reconstructive surgery for injuries caused by trauma, burns, or ablative surgery. Various types of TISSUE EXPANSION DEVICES have been developed that exert stretching forces.'),
  'D015632': NtO(MH='1-Methyl-4-phenyl-1,2,3,6-tetrahydropyridine', MS='A dopaminergic neurotoxic compound which produces irreversible clinical, chemical, and pathological alterations that mimic those found in Parkinson disease.'),
  'D015633': NtO(MH='Extravascular Lung Water', MS='Water content outside of the lung vasculature. About 80% of a normal lung is made up of water, including intracellular, interstitial, and blood water. Failure to maintain the normal homeostatic fluid exchange between the vascular space and the interstitium of the lungs can result in PULMONARY EDEMA and flooding of the alveolar space.'),
  'D015635': NtO(MH='Radionuclide Ventriculography', MS='Imaging of a ventricle of the heart after the injection of a radioactive contrast medium. The technique is less invasive than cardiac catheterization and is used to assess ventricular function.'),
  'D015636': NtO(MH='Magnesium Chloride', MS='Magnesium chloride. An inorganic compound consisting of one magnesium and two chloride ions. The compound is used in medicine as a source of magnesium ions, which are essential for many cellular activities. It has also been used as a cathartic and in alloys.'),
  'D015637': NtO(MH='Gated Blood-Pool Imaging', MS='Radionuclide ventriculography where scintigraphic data is acquired during repeated cardiac cycles at specific times in the cycle, using an electrocardiographic synchronizer or gating device. Analysis of right ventricular function is difficult with this technique; that is best evaluated by first-pass ventriculography (VENTRICULOGRAPHY, FIRST-PASS).'),
  'D015638': NtO(MH='Cytochalasin D', MS='A fungal metabolite that blocks cytoplasmic cleavage by blocking formation of contractile microfilament structures resulting in multinucleated cell formation, reversible inhibition of cell movement, and the induction of cellular extrusion. Additional reported effects include the inhibition of actin polymerization, DNA synthesis, sperm motility, glucose transport, thyroid secretion, and growth hormone release.'),
  'D015640': NtO(MH='Ion Channel Gating', MS='The opening and closing of ion channels due to a stimulus. The stimulus can be a change in membrane potential (voltage-gated), drugs or chemical transmitters (ligand-gated), or a mechanical deformation. Gating is thought to involve conformational changes of the ion channel which alters selective permeability.'),
  'D015641': NtO(MH='Radiography, Interventional', MS='Diagnostic and therapeutic procedures that are invasive or surgical in nature, and require the expertise of a specially trained radiologist. In general, they are more invasive than diagnostic imaging but less invasive than major surgery. They often involve catheterization, fluoroscopy, or computed tomography. Some examples include percutaneous transhepatic cholangiography, percutaneous transthoracic biopsy, balloon angioplasty, and arterial embolization.'),
  'D015642': NtO(MH='Radiology, Interventional', MS='Subspecialty of radiology that combines organ system radiography, catheter techniques and sectional imaging.'),
  'D015643': NtO(MH='Erythromycin Ethylsuccinate', MS='A macrolide antibiotic, produced by Streptomyces erythreus. This compound is an ester of erythromycin base and succinic acid. It acts primarily as a bacteriostatic agent. In sensitive organisms, it inhibits protein synthesis by binding to 50S ribosomal subunits. This binding process inhibits peptidyl transferase activity and interferes with translocation of amino acids during translation and assembly of proteins.'),
  'D015644': NtO(MH='Miocamycin', MS='A macrolide antibiotic that has a wide antimicrobial spectrum and is particularly effective in respiratory and genital infections.'),
  'D015645': NtO(MH='Tylosin', MS='Macrolide antibiotic obtained from cultures of Streptomyces fradiae. The drug is effective against many microorganisms in animals but not in humans.'),
  'D015646': NtO(MH='Ventriculography, First-Pass', MS='Radionuclide ventriculography where a bolus of radionuclide is injected and data are recorded from one pass through the heart ventricle. Left and right ventricular function can be analyzed independently during this technique. First-pass ventriculography is preferred over GATED BLOOD-POOL IMAGING for assessing right ventricular function.'),
  'D015647': NtO(MH='2,3,4,5-Tetrahydro-7,8-dihydroxy-1-phenyl-1H-3-benzazepine', MS='A selective D1 dopamine receptor agonist used primarily as a research tool.'),
  'D015649': NtO(MH='Pentostatin', MS='A potent inhibitor of ADENOSINE DEAMINASE. The drug induces APOPTOSIS of LYMPHOCYTES, and is used in the treatment of many lymphoproliferative malignancies, particularly HAIRY CELL LEUKEMIA. It is also synergistic with some other antineoplastic agents and has immunosuppressive activity.'),
  'D015650': NtO(MH='24,25-Dihydroxyvitamin D 3', MS='A physiologically active metabolite of VITAMIN D. The compound is involved in the regulation of calcium metabolism, alkaline phosphatase activity, and enhancing the calcemic effect of CALCITRIOL.'),
  'D015651': NtO(MH='Mycotoxicosis', MS='Poisoning caused by the ingestion of mycotoxins (toxins of fungal origin).'),
  'D015652': NtO(MH='25-Hydroxyvitamin D 2', MS='9,10-Secoergosta-5,7,10(19),22-tetraene-3,25-diol. Biologically active metabolite of vitamin D2 which is more active in curing rickets than its parent. The compound is believed to attach to the same receptor as vitamin D2 and 25-hydroxyvitamin D3.'),
  'D015653': NtO(MH='Cystectomy', MS='Used for excision of the urinary bladder.'),
  'D015654': NtO(MH='Herpesvirus 6, Human', MS="Members of the ROSEOLOVIRUS genus of the Betaherpesvirales subfamily isolated from patients with AIDS and other LYMPHOPROLIFERATIVE DISORDERS. It infects and replicates in fresh and established lines of hematopoietic cells and cells of neural origin. It also appears to alter the activity of NK CELLS. HHV-6; (HBLV) antibodies are elevated in patients with AIDS; SJOGREN'S SYNDROME; SARCOIDOSIS; CHRONIC FATIGUE SYNDROME, and certain malignancies. HHV-6A is the most common cause of EXANTHEMA SUBITUM and has been implicated in encephalitis. When HHV-6 integrates into the host genome it is referred to as ciHVH-6. When such VIRUS INTEGRATION occurs into the germline it is referred to as iciHHV-6."),
  'D015655': NtO(MH='1-Methyl-4-phenylpyridinium', MS='An active neurotoxic metabolite of 1-METHYL-4-PHENYL-1,2,3,6-TETRAHYDROPYRIDINE. The compound reduces dopamine levels, inhibits the biosynthesis of catecholamines, depletes cardiac norepinephrine and inactivates tyrosine hydroxylase. These and other toxic effects lead to cessation of oxidative phosphorylation, ATP depletion, and cell death. The compound, which is related to PARAQUAT, has also been used as an herbicide.'),
  'D015656': NtO(MH='Respiratory Mechanics', MS='The physical or mechanical action of the LUNGS; DIAPHRAGM; RIBS; and CHEST WALL during respiration. It includes airflow, lung volume, neural and reflex controls, mechanoreceptors, breathing patterns, etc.'),
  'D015657': NtO(MH='Respiratory Transport', MS='The processes of diffusion across the BLOOD-AIR BARRIER, and the chemical reactions coupled with diffusion that effect the rate of PULMONARY GAS EXCHANGE, generally at the alveolar level.'),
  'D015658': NtO(MH='HIV Infections', MS='Includes the spectrum of human immunodeficiency virus infections that range from asymptomatic seropositivity, thru AIDS-related complex (ARC), to acquired immunodeficiency syndrome (AIDS).'),
  'D015660': NtO(MH='7-Alkoxycoumarin O-Dealkylase', MS='A drug-metabolizing enzyme found in the hepatic, placental and intestinal microsomes that metabolizes 7-alkoxycoumarin to 7-hydroxycoumarin. The enzyme is cytochrome P-450- dependent.'),
  'D015661': NtO(MH='Pasteurellaceae', MS='A family of coccoid to rod-shaped nonsporeforming, gram-negative, nonmotile, facultatively anaerobic bacteria that includes the genera ACTINOBACILLUS; HAEMOPHILUS; MANNHEIMIA; and PASTEURELLA.'),
  'D015662': NtO(MH='Trimethoprim, Sulfamethoxazole Drug Combination', MS='A drug combination with broad-spectrum antibacterial activity against both gram-positive and gram-negative organisms. It is effective in the treatment of many infections, including PNEUMOCYSTIS PNEUMONIA in AIDS.'),
  'D015663': NtO(MH='Osteoporosis, Postmenopausal', MS='Metabolic disorder associated with fractures of the femoral neck, vertebrae, and distal forearm. It occurs commonly in women within 15-20 years after menopause, and is caused by factors associated with menopause including estrogen deficiency.'),
  'D015667': NtO(MH='Acupuncture Analgesia', MS='Analgesia produced by the insertion of ACUPUNCTURE needles at certain ACUPUNCTURE POINTS on the body. This activates small myelinated nerve fibers in the muscle which transmit impulses to the spinal cord and then activate three centers - the spinal cord, midbrain and pituitary/hypothalamus - to produce analgesia.'),
  'D015669': NtO(MH='Acupuncture Points', MS='Designated locations along nerves or organ meridians for inserting acupuncture needles.'),
  'D015670': NtO(MH='Acupuncture Therapy', MS='Treatment of disease by inserting needles along specific pathways or meridians. The placement varies with the disease being treated. It is sometimes used in conjunction with heat, moxibustion, acupressure, or electric stimulation.'),
  'D015671': NtO(MH='Electroacupuncture', MS='A form of acupuncture with electrical impulses passing through the needles to stimulate NERVE TISSUE. It can be used for ANALGESIA; ANESTHESIA; REHABILITATION; and treatment for diseases.'),
  'D015672': NtO(MH='Erythroid Precursor Cells', MS='The cells in the erythroid series derived from MYELOID PROGENITOR CELLS or from the bi-potential MEGAKARYOCYTE-ERYTHROID PROGENITOR CELLS which eventually give rise to mature RED BLOOD CELLS. The erythroid progenitor cells develop in two phases: erythroid burst-forming units (BFU-E) followed by erythroid colony-forming units (CFU-E); BFU-E differentiate into CFU-E on stimulation by ERYTHROPOIETIN, and then further differentiate into ERYTHROBLASTS when stimulated by other factors.'),
  'D015673': NtO(MH='Fatigue Syndrome, Chronic', MS='A syndrome characterized by persistent or recurrent fatigue, diffuse musculoskeletal pain, sleep disturbances, and subjective cognitive impairment of 6 months duration or longer. Symptoms are not caused by ongoing exertion; are not relieved by rest; and result in a substantial reduction of previous levels of occupational, educational, social, or personal activities. Minor alterations of immune, neuroendocrine, and autonomic function may be associated with this syndrome. There is also considerable overlap between this condition and FIBROMYALGIA. (From Semin Neurol 1998;18(2):237-42; Ann Intern Med 1994 Dec 15;121(12): 953-9)'),
  'D015674': NtO(MH='Mammary Neoplasms, Animal', MS='Tumors or cancer of the MAMMARY GLAND in animals (MAMMARY GLANDS, ANIMAL).'),
  'D015675': NtO(MH='Osteocalcin', MS='Vitamin K-dependent calcium-binding protein synthesized by OSTEOBLASTS and found primarily in BONES. Serum osteocalcin measurements provide a noninvasive specific marker of bone metabolism. The protein contains three residues of the amino acid gamma-carboxyglutamic acid (Gla), which, in the presence of CALCIUM, promotes binding to HYDROXYAPATITE and subsequent accumulation in BONE MATRIX.'),
  'D015676': NtO(MH='Osteonectin', MS='Non-collagenous, calcium-binding glycoprotein of developing bone. It links collagen to mineral in the bone matrix. In the synonym SPARC glycoprotein, the acronym stands for Secreted Protein, Acidic and Rich in Cysteine.'),
  'D015678': NtO(MH='Viral Structural Proteins', MS='Viral proteins that are components of the mature assembled VIRUS PARTICLES. They may include nucleocapsid core proteins (gag proteins), enzymes packaged within the virus particle (pol proteins), and membrane components (env proteins). These do not include the proteins encoded in the VIRAL GENOME that are produced in infected cells but which are not packaged in the mature virus particle,i.e. the so called non-structural proteins (VIRAL NONSTRUCTURAL PROTEINS).'),
  'D015681': NtO(MH='Fusion Proteins, gag-onc', MS='General name for the translation products of a fusion mRNA consisting of a gag gene and a viral oncogene (v-onc). These products are thought to have the ability to transform cells.'),
  'D015682': NtO(MH='Retroviridae Proteins, Oncogenic', MS='Retroviral proteins that have the ability to transform cells. They can induce sarcomas, leukemias, lymphomas, and mammary carcinomas. Not all retroviral proteins are oncogenic.'),
  'D015683': NtO(MH='Gene Products, gag', MS='Proteins coded by the retroviral gag gene. The products are usually synthesized as protein precursors or POLYPROTEINS, which are then cleaved by viral proteases to yield the final products. Many of the final products are associated with the nucleoprotein core of the virion. gag is short for group-specific antigen.'),
  'D015684': NtO(MH='Gene Products, pol', MS='Retroviral proteins coded by the pol gene. They are usually synthesized as a protein precursor (POLYPROTEINS) and later cleaved into final products that include reverse transcriptase, endonuclease/integrase, and viral protease. Sometimes they are synthesized as a gag-pol fusion protein (FUSION PROTEINS, GAG-POL). pol is short for polymerase, the enzyme class of reverse transcriptase.'),
  'D015685': NtO(MH='Fusion Proteins, gag-pol', MS='Polyprotein products of a fused portion of retroviral mRNA containing the gag and pol genes. The polyprotein is synthesized only five percent of the time since pol is out of frame with gag, and is generated by ribosomal frameshifting.'),
  'D015686': NtO(MH='Gene Products, env', MS='Retroviral proteins, often glycosylated, coded by the envelope (env) gene. They are usually synthesized as protein precursors (POLYPROTEINS) and later cleaved into the final viral envelope glycoproteins by a viral protease.'),
  'D015687': NtO(MH='Cell Hypoxia', MS='A condition of decreased oxygen content at the cellular level.'),
  'D015688': NtO(MH='Oncogene Protein pp60(v-src)', MS='A tyrosine-specific protein kinase encoded by the v-src oncogene of ROUS SARCOMA VIRUS. The transforming activity of pp60(v-src) depends on both the lack of a critical carboxy-terminal tyrosine phosphorylation site at position 527, and the attachment of pp60(v-src) to the plasma membrane which is accomplished by myristylation of its N-terminal glycine.'),
  'D015689': NtO(MH='Oncogene Protein p21(ras)', MS='Transforming protein encoded by ras oncogenes. Point mutations in the cellular ras gene (c-ras) can also result in a mutant p21 protein that can transform mammalian cells. Oncogene protein p21(ras) has been directly implicated in human neoplasms, perhaps accounting for as much as 15-20% of all human tumors. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D015693': NtO(MH='Gene Products, nef', MS='Products of the retroviral NEF GENE. They play a role as accessory proteins that influence the rate of viral infectivity and the destruction of the host immune system. nef gene products were originally found as factors that trans-suppress viral replication and function as negative regulators of transcription. nef stands for negative factor.'),
  'D015694': NtO(MH='Gene Products, rev', MS='Trans-acting nuclear proteins whose functional expression are required for retroviral replication. Specifically, the rev gene products are required for processing and translation of the gag and env mRNAs, and thus rev regulates the expression of the viral structural proteins. rev can also regulate viral regulatory proteins. A cis-acting antirepression sequence (CAR) in env, also known as the rev-responsive element (RRE), is responsive to the rev gene product. rev is short for regulator of virion.'),
  'D015696': NtO(MH='Gene Products, tat', MS='Trans-acting transcription factors produced by retroviruses such as HIV. They are nuclear proteins whose expression is required for viral replication. The tat protein stimulates LONG TERMINAL REPEAT-driven RNA synthesis for both viral regulatory and viral structural proteins. tat stands for trans-activation of transcription.'),
  'D015698': NtO(MH='Genomic Library', MS='A form of GENE LIBRARY containing the complete DNA sequences present in the genome of a given organism. It contrasts with a cDNA library which contains only sequences utilized in protein coding (lacking introns).'),
  'D015699': NtO(MH='HIV Envelope Protein gp120', MS='External envelope protein of the human immunodeficiency virus which is encoded by the HIV env gene. It has a molecular weight of 120 kDa and contains numerous glycosylation sites. Gp120 binds to cells expressing CD4 cell-surface antigens, most notably T4-lymphocytes and monocytes/macrophages. Gp120 has been shown to interfere with the normal function of CD4 and is at least partly responsible for the cytopathic effect of HIV.'),
  'D015700': NtO(MH='HIV Envelope Protein gp41', MS='Transmembrane envelope protein of the HUMAN IMMUNODEFICIENCY VIRUS which is encoded by the HIV env gene. It has a molecular weight of 41,000 and is glycosylated. The N-terminal part of gp41 is thought to be involved in CELL FUSION with the CD4 ANTIGENS of T4 LYMPHOCYTES, leading to syncytial formation. Gp41 is one of the most common HIV antigens detected by IMMUNOBLOTTING.'),
  'D015701': NtO(MH='Near Drowning', MS='Non-fatal immersion or submersion in water. The subject is resuscitable.'),
  'D015703': NtO(MH='Antigens, CD', MS='Differentiation antigens residing on mammalian leukocytes. CD stands for cluster of differentiation, which refers to groups of monoclonal antibodies that show similar reactivity with certain subpopulations of antigens of a particular lineage or differentiation stage. The subpopulations of antigens are also known by the same CD designation.'),
  'D015704': NtO(MH='CD4 Antigens', MS='55-kDa antigens found on HELPER-INDUCER T-LYMPHOCYTES and on a variety of other immune cell types. They are members of the immunoglobulin supergene family and are implicated as associative recognition elements in MAJOR HISTOCOMPATIBILITY COMPLEX class II-restricted immune responses. On T-lymphocytes they define the helper/inducer subset. T4 antigens also serve as INTERLEUKIN-15 receptors and bind to the HIV receptors, binding directly to the HIV ENVELOPE PROTEIN GP120.'),
  'D015706': NtO(MH='Bibliometrics', MS='The use of statistical methods in the analysis of a body of literature to reveal the historical development of subject fields and patterns of authorship, publication, and use. Formerly called statistical bibliography. (from The ALA Glossary of Library and Information Science, 1983)'),
  'D015707': NtO(MH='HN Protein', MS='Glycoprotein from Sendai, para-influenza, Newcastle Disease, and other viruses that participates in binding the virus to cell-surface receptors. The HN protein possesses both hemagglutinin and neuraminidase activity.'),
  'D015709': NtO(MH='Correspondence as Topic', MS='Works on written communication between persons or between institutions or organizations.'),
  'D015713': NtO(MH='Duplicate Publications as Topic', MS="Work about simultaneous or successive publishing of identical or near- identical material in two or more different sources without acknowledgment. It differs from reprinted publication in that a reprint cites sources. It differs from PLAGIARISM in that duplicate publication is the product of the same authorship while plagiarism publishes a work or parts of a work of another as one's own."),
  'D015714': NtO(MH='Plagiarism', MS="Passing off as one's own the work of another without credit."),
  'D015715': NtO(MH='Corneal Edema', MS='An excessive amount of fluid in the cornea due to damage of the epithelium or endothelium causing decreased visual acuity.'),
  'D015716': NtO(MH='Electrocardiography, Ambulatory', MS='Method in which prolonged electrocardiographic recordings are made on a portable tape recorder (Holter-type system) or solid-state device ("real-time" system), while the patient undergoes normal daily activities. It is useful in the diagnosis and management of intermittent cardiac arrhythmias and transient myocardial ischemia.'),
  'D015717': NtO(MH='Peptide T', MS='N-(N-(N(2)-(N-(N-(N-(N-D-Alanyl L-seryl)-L-threonyl)-L-threonyl) L-threonyl)-L-asparaginyl)-L-tyrosyl) L-threonine. Octapeptide sharing sequence homology with HIV envelope protein gp120. It is potentially useful as antiviral agent in AIDS therapy. The core pentapeptide sequence, TTNYT, consisting of amino acids 4-8 in peptide T, is the HIV envelope sequence required for attachment to the CD4 receptor.'),
  'D015718': NtO(MH='Fibrin Tissue Adhesive', MS='An autologous or commercial tissue adhesive containing FIBRINOGEN and THROMBIN. The commercial product is a two component system from human plasma that contains more than fibrinogen and thrombin. The first component contains highly concentrated fibrinogen, FACTOR VIII, fibronectin, and traces of other plasma proteins. The second component contains thrombin, calcium chloride, and antifibrinolytic agents such as APROTININ. Mixing of the two components promotes BLOOD CLOTTING and the formation and cross-linking of fibrin. The tissue adhesive is used for tissue sealing, HEMOSTASIS, and WOUND HEALING.'),
  'D015719': NtO(MH='Single-Strand Specific DNA and RNA Endonucleases', MS='Enzymes that catalyze the endonucleolytic cleavage of single-stranded regions of DNA or RNA molecules while leaving the double-stranded regions intact. They are particularly useful in the laboratory for producing "blunt-ended" DNA molecules from DNA with single-stranded ends and for sensitive GENETIC TECHNIQUES such as NUCLEASE PROTECTION ASSAYS that involve the detection of single-stranded DNA and RNA.'),
  'D015720': NtO(MH="5'-Nucleotidase", MS="A glycoprotein enzyme present in various organs and in many cells. The enzyme catalyzes the hydrolysis of a 5'-ribonucleotide to a ribonucleoside and orthophosphate in the presence of water. It is cation-dependent and exists in a membrane-bound and soluble form. EC 3.1.3.5."),
  'D015721': NtO(MH='Diphtheria-Tetanus-Pertussis Vaccine', MS='A vaccine consisting of DIPHTHERIA TOXOID; TETANUS TOXOID; and whole-cell PERTUSSIS VACCINE. The vaccine protects against diphtheria, tetanus, and whooping cough.'),
  'D015723': NtO(MH='Gene Library', MS='A large collection of DNA fragments cloned (CLONING, MOLECULAR) from a given organism, tissue, organ, or cell type. It may contain complete genomic sequences (GENOMIC LIBRARY) or complementary DNA sequences, the latter being formed from messenger RNA and lacking intron sequences.'),
  'D015725': NtO(MH='Fluconazole', MS='Triazole antifungal agent that is used to treat oropharyngeal CANDIDIASIS and cryptococcal MENINGITIS in AIDS.'),
  'D015726': NtO(MH='Giant Cells', MS='Multinucleated masses produced by the fusion of many cells; often associated with viral infections. In AIDS, they are induced when the envelope glycoprotein of the HIV virus binds to the CD4 antigen of uninfected neighboring T4 cells. The resulting syncytium leads to cell death and thus may account for the cytopathic effect of the virus.'),
  'D015728': NtO(MH='Receptors, HIV', MS='Cellular receptors that bind the human immunodeficiency virus that causes AIDS. Included are CD4 ANTIGENS, found on T4 lymphocytes, and monocytes/macrophages, which bind to the HIV ENVELOPE PROTEIN GP120.'),
  'D015730': NtO(MH='Djibouti', MS='A republic in eastern Africa, on the Gulf of Aden at the entrance to the Red Sea. Djibouti is also the name of its capital.'),
  'D015735': NtO(MH='Mifepristone', MS='A progestational and glucocorticoid hormone antagonist. Its inhibition of progesterone induces bleeding during the luteal phase and in early pregnancy by releasing endogenous prostaglandins from the endometrium or decidua. As a glucocorticoid receptor antagonist, the drug has been used to treat hypercortisolism in patients with nonpituitary CUSHING SYNDROME.'),
  'D015736': NtO(MH='Felodipine', MS='A dihydropyridine calcium antagonist with positive inotropic effects. It lowers blood pressure by reducing peripheral vascular resistance through a highly selective action on smooth muscle in arteriolar resistance vessels.'),
  'D015737': NtO(MH='Nisoldipine', MS='A dihydropyridine calcium channel antagonist that acts as a potent arterial vasodilator and antihypertensive agent. It is also effective in patients with cardiac failure and angina.'),
  'D015738': NtO(MH='Famotidine', MS='A competitive histamine H2-receptor antagonist. Its main pharmacodynamic effect is the inhibition of gastric secretion.'),
  'D015739': NtO(MH='Nocodazole', MS='Nocodazole is an antineoplastic agent which exerts its effect by depolymerizing microtubules.'),
  'D015740': NtO(MH='Calcitonin Gene-Related Peptide', MS='A 37-amino acid peptide derived from the calcitonin gene. It occurs as a result of alternative processing of mRNA from the calcitonin gene. The neuropeptide is widely distributed in the brain, gut, perivascular nerves, and other tissue. The peptide produces multiple biological effects and has both circulatory and neurotransmitter modes of action. In particular, it is a potent endogenous vasodilator.'),
  'D015741': NtO(MH='Metribolone', MS='A synthetic non-aromatizable androgen and anabolic steroid. It binds strongly to the androgen receptor and has therefore also been used as an affinity label for this receptor in the prostate and in prostatic tumors.'),
  'D015742': NtO(MH='Propofol', MS='An intravenous anesthetic agent which has the advantage of a very rapid onset after infusion or bolus injection plus a very short recovery period of a couple of minutes. (From Smith and Reynard, Textbook of Pharmacology, 1992, 1st ed, p206). Propofol has been used as ANTICONVULSANTS and ANTIEMETICS.'),
  'D015743': NtO(MH='Giant Cells, Foreign-Body', MS='Multinucleated cells (fused macrophages), characteristic of granulomatous inflammation, which form around exogenous material in the skin. They are similar in appearance to Langhans giant cells (GIANT CELLS, LANGHANS), but foreign-body giant cells have more abundant chromatin and their nuclei are scattered in an irregular pattern in the cytoplasm.'),
  'D015744': NtO(MH='Giant Cells, Langhans', MS='Multinucleated cells (fused macrophages) seen in granulomatous inflammations such as tuberculosis, syphilis, sarcoidosis, and deep fungal infections. They resemble foreign-body giant cells (GIANT CELLS, FOREIGN BODY) but Langhans giant cells contain less chromatin and their nuclei are arranged peripherally in a horseshoe-shaped pattern. Langhans giant cells occur frequently in delayed hypersensitivity.'),
  'D015745': NtO(MH='Granuloma, Foreign-Body', MS='Histiocytic, inflammatory response to a foreign body. It consists of modified macrophages with multinucleated giant cells, in this case foreign-body giant cells (GIANT CELLS, FOREIGN-BODY), usually surrounded by lymphocytes.'),
  'D015746': NtO(MH='Abdominal Pain', MS='Sensation of discomfort, distress, or agony in the abdominal region.'),
  'D015748': NtO(MH='Borrelia burgdorferi Group', MS='Gram-negative helical bacteria, in the genus BORRELIA, that are the etiologic agents of LYME DISEASE. The group comprises many specific species including Borrelia afzelii, Borellia garinii, and BORRELIA BURGDORFERI proper. These spirochetes are generally transmitted by several species of ixodid ticks.'),
  'D015750': NtO(MH='Genes, gag', MS='DNA sequences that form the coding region for proteins associated with the viral core in retroviruses. gag is short for group-specific antigen.'),
  'D015751': NtO(MH='Genes, pol', MS='DNA sequences that form the coding region for retroviral enzymes including reverse transcriptase, protease, and endonuclease/integrase. "pol" is short for polymerase, the enzyme class of reverse transcriptase.'),
  'D015752': NtO(MH='Genes, env', MS='DNA sequences that form the coding region for the viral envelope (env) proteins in retroviruses. The env genes contain a cis-acting RNA target sequence for the rev protein (= GENE PRODUCTS, REV), termed the rev-responsive element (RRE).'),
  'D015754': NtO(MH='Genes, nef', MS='DNA sequences that form the coding region for a protein that down-regulates the expression of human immunodeficiency virus (HIV). nef is short for negative factor.'),
  'D015756': NtO(MH='Genes, tat', MS='DNA sequences that form the coding region for the protein responsible for trans-activation of transcription (tat) in human immunodeficiency virus (HIV).'),
  'D015757': NtO(MH='Genes, rev', MS='DNA sequences that form the coding region for a protein that regulates the expression of the viral structural and regulatory proteins in human immunodeficiency virus (HIV). rev is short for regulator of virion.'),
  'D015759': NtO(MH='Ionomycin', MS='A divalent calcium ionophore that is widely used as a tool to investigate the role of intracellular calcium in cellular processes.'),
  'D015760': NtO(MH='Alfentanil', MS='A short-acting opioid anesthetic and analgesic derivative of FENTANYL. It produces an early peak analgesic effect and fast recovery of consciousness. Alfentanil is effective as an anesthetic during surgery, for supplementation of analgesia during surgical procedures, and as an analgesic for critically ill patients.'),
  'D015761': NtO(MH='4-Aminopyridine', MS='One of the POTASSIUM CHANNEL BLOCKERS with secondary effect on calcium currents which is used mainly as a research tool and to characterize channel subtypes.'),
  'D015762': NtO(MH='2-Chloroadenosine', MS='2-Chloroadenosine. A metabolically stable analog of adenosine which acts as an adenosine receptor agonist. The compound has a potent effect on the peripheral and central nervous system.'),
  'D015763': NtO(MH='2-Amino-5-phosphonovalerate', MS='The D-enantiomer is a potent and specific antagonist of NMDA glutamate receptors (RECEPTORS, N-METHYL-D-ASPARTATE). The L form is inactive at NMDA receptors but may affect the AP4 (2-amino-4-phosphonobutyrate; APB) excitatory amino acid receptors.'),
  'D015764': NtO(MH='Bepridil', MS='A long-acting calcium-blocking agent with significant anti-anginal activity. The drug produces significant coronary vasodilation and modest peripheral effects. It has antihypertensive and selective anti-arrhythmia activities and acts as a calmodulin antagonist.'),
  'D015765': NtO(MH='Almitrine', MS='A respiratory stimulant that enhances respiration by acting as an agonist of peripheral chemoreceptors located on the carotid bodies. The drug increases arterial oxygen tension while decreasing arterial carbon dioxide tension in patients with chronic obstructive pulmonary disease. It may also prove useful in the treatment of nocturnal oxygen desaturation without impairing the quality of sleep.'),
  'D015766': NtO(MH='Albendazole', MS='A benzimidazole broad-spectrum anthelmintic structurally related to MEBENDAZOLE that is effective against many diseases. (From Martindale, The Extra Pharmacopoeia, 30th ed, p38)'),
  'D015767': NtO(MH='Mefloquine', MS='A phospholipid-interacting antimalarial drug (ANTIMALARIALS). It is very effective against PLASMODIUM FALCIPARUM with very few side effects.'),
  'D015769': NtO(MH='Granuloma, Respiratory Tract', MS='Granulomatous disorders affecting one or more sites in the respiratory tract.'),
  'D015772': NtO(MH='4,5-Dihydro-1-(3-(trifluoromethyl)phenyl)-1H-pyrazol-3-amine', MS='A dual inhibitor of both cyclooxygenase and lipoxygenase pathways. It exerts an anti-inflammatory effect by inhibiting the formation of prostaglandins and leukotrienes. The drug also enhances pulmonary hypoxic vasoconstriction and has a protective effect after myocardial ischemia.'),
  'D015773': NtO(MH='Enalaprilat', MS='The active metabolite of ENALAPRIL and one of the potent, intravenously administered, ANGIOTENSIN-CONVERTING ENZYME INHIBITORS. It is an effective agent for the treatment of essential hypertension and has beneficial hemodynamic effects in heart failure. The drug produces renal vasodilation with an increase in sodium excretion.'),
  'D015774': NtO(MH='Ganciclovir', MS='An ACYCLOVIR analog that is a potent inhibitor of the Herpesvirus family including cytomegalovirus. Ganciclovir is used to treat complications from AIDS-associated cytomegalovirus infections.'),
  'D015775': NtO(MH='Fractures, Stress', MS='Fractures due to the strain caused by repetitive exercise. They are thought to arise from a combination of MUSCLE FATIGUE and bone failure, and occur in situations where BONE REMODELING predominates over repair. The most common sites of stress fractures are the METATARSUS; FIBULA; TIBIA; and FEMORAL NECK.'),
  'D015776': NtO(MH='Keratoderma, Palmoplantar, Diffuse', MS='An autosomal dominant disorder characterized by a widely distributed, well-demarcated hyperkeratosis of the palms and soles. There is more than one genotypically distinct form, each of which is clinically similar but histologically distinguishable. Diffuse palmoplantar keratoderma is distinct from palmoplantar keratoderma (KERATODERMA, PALMOPLANTAR), as the former exhibits autosomal dominant inheritance and hyperhidrosis is frequently present.'),
  'D015777': NtO(MH='Eicosanoids', MS='A class of compounds named after and generally derived from C20 fatty acids (EICOSANOIC ACIDS) that includes PROSTAGLANDINS; LEUKOTRIENES; THROMBOXANES, and HYDROXYEICOSATETRAENOIC ACIDS. They have hormone-like effects mediated by specialized receptors (RECEPTORS, EICOSANOID).'),
  'D015778': NtO(MH='Minor Histocompatibility Antigens', MS='Allelic alloantigens often responsible for weak graft rejection in cases when (major) histocompatibility has been established by standard tests. In the mouse they are coded by more than 500 genes at up to 30 minor histocompatibility loci. The most well-known minor histocompatibility antigen in mammals is the H-Y antigen.'),
  'D015780': NtO(MH='Carbapenems', MS='A group of beta-lactam antibiotics in which the sulfur atom in the thiazolidine ring of the penicillin molecule is replaced by a carbon atom. THIENAMYCINS are a subgroup of carbapenems which have a sulfur atom as the first constituent of the side chain.'),
  'D015783': NtO(MH='Aniridia', MS='A congenital abnormality in which there is only a rudimentary iris. This is due to the failure of the optic cup to grow. Aniridia also occurs in a hereditary form, usually autosomal dominant.'),
  'D015784': NtO(MH='Betaxolol', MS='A cardioselective beta-1-adrenergic antagonist with no partial agonist activity.'),
  'D015785': NtO(MH='Eye Diseases, Hereditary', MS='Transmission of gene defects or chromosomal aberrations/abnormalities which are expressed in extreme variation in the structure or function of the eye. These may be evident at birth, but may be manifested later with progression of the disorder.'),
  'D015786': NtO(MH='Cytochromes b5', MS='Cytochromes of the b group that are found bound to cytoplasmic side of ENDOPLASMIC RETICULUM. They serve as electron carrier proteins for a variety of membrane-bound OXYGENASES. They are reduced by the enzyme CYTOCHROME-B(5) REDUCTASE.'),
  'D015787': NtO(MH='Erythema Chronicum Migrans', MS='A deep type of gyrate erythema that follows a bite by an ixodid tick; it is a stage-1 manifestation of LYME DISEASE. The site of the bite is characterized by a red papule that expands peripherally as a nonscaling, palpable band that clears centrally. This condition is often associated with systemic symptoms such as chills, fever, headache, malaise, nausea, vomiting, fatigue, backache, and stiff neck.'),
  'D015788': NtO(MH='HLA-A1 Antigen', MS='A specific HLA-A surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-A*01 allele family.'),
  'D015789': NtO(MH='HLA-A2 Antigen', MS='A specific HLA-A surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-A*02 allele family.'),
  'D015790': NtO(MH='Cefonicid', MS='A second-generation cephalosporin administered intravenously or intramuscularly. Its bactericidal action results from inhibition of cell wall synthesis. It is used for urinary tract infections, lower respiratory tract infections, and soft tissue and bone infections.'),
  'D015791': NtO(MH='HLA-A3 Antigen', MS='A specific HLA-A surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-A*03 allele family.'),
  'D015792': NtO(MH='Retinal Dysplasia', MS='Congenital, often bilateral, retinal abnormality characterized by the arrangement of outer nuclear retinal cells in a palisading or radiating pattern surrounding a central ocular space. This disorder is sometimes hereditary.'),
  'D015793': NtO(MH='HLA-B7 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*07 allele family.'),
  'D015794': NtO(MH='Choroideremia', MS='An X chromosome-linked abnormality characterized by atrophy of the choroid and degeneration of the retinal pigment epithelium causing night blindness.'),
  'D015795': NtO(MH='HLA-B8 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*08 allele family.'),
  'D015796': NtO(MH='HLA-B27 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*27 allele family.'),
  'D015797': NtO(MH='HLA-B35 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*35 allele family.'),
  'D015798': NtO(MH='HLA-DR1 Antigen', MS='An HLA-DR antigen associated with HLA-DRB1 CHAINS that are encoded by DRB1*01 alleles.'),
  'D015799': NtO(MH='Gyrate Atrophy', MS='Progressive, autosomal recessive, diffuse atrophy of the choroid, pigment epithelium, and sensory retina that begins in childhood.'),
  'D015800': NtO(MH='Protozoan Proteins', MS='Proteins found in any species of protozoan.'),
  'D015801': NtO(MH='Helminth Proteins', MS='Proteins found in any species of helminth.'),
  'D015802': NtO(MH='HLA-DR2 Antigen', MS='A broad specificity HLA-DR antigen that is associated with HLA-DRB1 CHAINS encoded by DRB1*01:15 and DRB1*01:16 alleles.'),
  'D015803': NtO(MH='HLA-DR3 Antigen', MS='An HLA-DR antigen which is associated with HLA-DRB1 CHAINS encoded by DRB1*03 alleles.'),
  'D015804': NtO(MH='HLA-DR4 Antigen', MS='An HLA-DR antigen which is associated with HLA-DRB1 CHAINS encoded by DRB1*04 alleles.'),
  'D015805': NtO(MH='HLA-DR5 Antigen', MS='A broad-specificity HLA-DR antigen that is associated with HLA-DRB1 CHAINS encoded by DRB1*11 and DRB1*12 alleles.'),
  'D015806': NtO(MH='Azospirillum brasilense', MS='A species of motile, free-living, gram-negative bacteria that occur in the soil. They are aerobic or microaerophilic and are sometimes capable of nitrogen fixation.'),
  'D015807': NtO(MH='Eye Injuries, Penetrating', MS='Deeply perforating or puncturing type intraocular injuries.'),
  'D015808': NtO(MH='HLA-DR6 Antigen', MS='A broad-specificity HLA-DR antigen that is associated with HLA-DRB1 CHAINS encoded by DRB1*13 and DRB1*14 alleles.'),
  'D015809': NtO(MH='HLA-DR7 Antigen', MS='A HLA-DR antigen that is associated with HLA-DRB1 CHAINS encoded by DRB1*07 alleles.'),
  'D015810': NtO(MH='Linkage Disequilibrium', MS='Nonrandom association of linked genes. This is the tendency of the alleles of two separate but already linked loci to be found together more frequently than would be expected by chance alone.'),
  'D015811': NtO(MH='Iris Neoplasms', MS='Tumors of the iris characterized by increased pigmentation of melanocytes. Iris nevi are composed of proliferated melanocytes and are associated with neurofibromatosis and malignant melanoma of the choroid and ciliary body. Malignant melanoma of the iris often originates from preexisting nevi.'),
  'D015812': NtO(MH='Glaucoma, Angle-Closure', MS='A form of glaucoma in which the intraocular pressure increases because the angle of the anterior chamber is blocked and the aqueous humor cannot drain from the anterior chamber.'),
  'D015813': NtO(MH='Substance Abuse Detection', MS='Detection of drugs that have been abused, overused, or misused, including legal and illegal drugs. Urine screening is the usual method of detection.'),
  'D015814': NtO(MH='Ocular Hypotension', MS='Abnormally low intraocular pressure often related to chronic inflammation (uveitis).'),
  'D015815': NtO(MH='Cell Adhesion Molecules', MS='Surface ligands, usually glycoproteins, that mediate cell-to-cell adhesion. Their functions include the assembly and interconnection of various vertebrate systems, as well as maintenance of tissue integration, wound healing, morphogenic movements, cellular migrations, and metastasis.'),
  'D015816': NtO(MH='Cell Adhesion Molecules, Neuronal', MS='Surface ligands that mediate cell-to-cell adhesion and function in the assembly and interconnection of the vertebrate nervous system. These molecules promote cell adhesion via a homophilic mechanism. These are not to be confused with NEURAL CELL ADHESION MOLECULES, now known to be expressed in a variety of tissues and cell types in addition to nervous tissue.'),
  'D015817': NtO(MH='Eye Infections', MS='Infection, moderate to severe, caused by bacteria, fungi, or viruses, which occurs either on the external surface of the eye or intraocularly with probable inflammation, visual impairment, or blindness.'),
  'D015818': NtO(MH='Eye Infections, Bacterial', MS='Infections in the inner or external eye caused by microorganisms belonging to several families of bacteria. Some of the more common genera found are Haemophilus, Neisseria, Staphylococcus, Streptococcus, and Chlamydia.'),
  'D015819': NtO(MH='Substance Abuse, Intravenous', MS='Abuse, overuse, or misuse of a substance by its injection into a vein.'),
  'D015820': NtO(MH='Cadherins', MS='Calcium-dependent cell adhesion proteins. They are important in the formation of ADHERENS JUNCTIONS between cells. Cadherins are classified by their distinct immunological and tissue specificities, either by letters (E- for epithelial, N- for neural, and P- for placental cadherins) or by numbers (cadherin-12 or N-cadherin 2 for brain-cadherin). Cadherins promote cell adhesion via a homophilic mechanism as in the construction of tissues and of the whole animal body.'),
  'D015821': NtO(MH='Eye Infections, Fungal', MS='Infection by a variety of fungi, usually through four possible mechanisms: superficial infection producing conjunctivitis, keratitis, or lacrimal obstruction; extension of infection from neighboring structures - skin, paranasal sinuses, nasopharynx; direct introduction during surgery or accidental penetrating trauma; or via the blood or lymphatic routes in patients with underlying mycoses.'),
  'D015822': NtO(MH='Eye Infections, Parasitic', MS='Mild to severe infections of the eye and its adjacent structures (adnexa) by adult or larval protozoan or metazoan parasites.'),
  'D015823': NtO(MH='Acanthamoeba Keratitis', MS='Infection of the cornea by an ameboid protozoan which may cause corneal ulceration leading to blindness.'),
  'D015824': NtO(MH='Blood-Air Barrier', MS='The barrier between capillary blood and alveolar air comprising the alveolar EPITHELIUM and capillary ENDOTHELIUM with their adherent BASEMENT MEMBRANE and EPITHELIAL CELL cytoplasm. PULMONARY GAS EXCHANGE occurs across this membrane.'),
  'D015825': NtO(MH='Chromosomes, Fungal', MS='Structures within the nucleus of fungal cells consisting of or containing DNA, which carry genetic information essential to the cell.'),
  'D015826': NtO(MH='Langer-Giedion Syndrome', MS='Autosomal dominant disorder characterized by cone-shaped epiphyses in the hands and multiple cartilaginous exostoses. INTELLECTUAL DISABILITY and abnormalities of chromosome 8 are often present. The exostoses in this syndrome appear identical to those of hereditary multiple exostoses (EXOSTOSES, HEREDITARY MULTIPLE).'),
  'D015827': NtO(MH='Onchocerciasis, Ocular', MS='Filarial infection of the eyes transmitted from person to person by bites of Onchocerca volvulus-infected black flies. The microfilariae of Onchocerca are thus deposited beneath the skin. They migrate through various tissues including the eye. Those persons infected have impaired vision and up to 20% are blind. The incidence of eye lesions has been reported to be as high as 30% in Central America and parts of Africa.'),
  'D015828': NtO(MH='Eye Infections, Viral', MS='Infections of the eye caused by minute intracellular agents. These infections may lead to severe inflammation in various parts of the eye - conjunctiva, iris, eyelids, etc. Several viruses have been identified as the causative agents. Among these are Herpesvirus, Adenovirus, Poxvirus, and Myxovirus.'),
  'D015829': NtO(MH='Uveitis, Suppurative', MS='Intraocular infection caused mainly by pus-producing bacteria and rarely by fungi. The infection may be caused by an injury or surgical wound (exogenous) or by endogenous septic emboli in such diseases as bacterial endocarditis or meningococcemia.'),
  'D015831': NtO(MH='Osteochondroma', MS='A cartilage-capped benign tumor that often appears as a stalk on the surface of bone. It is probably a developmental malformation rather than a true neoplasm and is usually found in the metaphysis of the distal femur, proximal tibia, or proximal humerus. Osteochondroma is the most common of benign bone tumors.'),
  'D015834': NtO(MH='Cochlear Diseases', MS='Pathological processes of the snail-like structure (COCHLEA) of the inner ear (LABYRINTH) which can involve its nervous tissue, blood vessels, or fluid (ENDOLYMPH).'),
  'D015835': NtO(MH='Ocular Motility Disorders', MS='Disorders that feature impairment of eye movements as a primary manifestation of disease. These conditions may be divided into infranuclear, nuclear, and supranuclear disorders. Diseases of the eye muscles or oculomotor cranial nerves (III, IV, and VI) are considered infranuclear. Nuclear disorders are caused by disease of the oculomotor, trochlear, or abducens nuclei in the BRAIN STEM. Supranuclear disorders are produced by dysfunction of higher order sensory and motor systems that control eye movements, including neural networks in the CEREBRAL CORTEX; BASAL GANGLIA; CEREBELLUM; and BRAIN STEM. Ocular torticollis refers to a head tilt that is caused by an ocular misalignment. Opsoclonus refers to rapid, conjugate oscillations of the eyes in multiple directions, which may occur as a parainfectious or paraneoplastic condition (e.g., OPSOCLONUS-MYOCLONUS SYNDROME). (Adams et al., Principles of Neurology, 6th ed, p240)'),
  'D015837': NtO(MH='Vestibular Diseases', MS='Pathological processes of the VESTIBULAR LABYRINTH which contains part of the balancing apparatus. Patients with vestibular diseases show instability and are at risk of frequent falls.'),
  'D015838': NtO(MH='Chondromatosis, Synovial', MS='Rare, benign, chronic, progressive metaplasia in which cartilage is formed in the synovial membranes of joints, tendon sheaths, or bursae. Some of the metaplastic foci can become detached producing loose bodies. When the loose bodies undergo secondary calcification, the condition is called synovial osteochondromatosis.'),
  'D015840': NtO(MH='Oculomotor Nerve Diseases', MS='Diseases of the oculomotor nerve or nucleus that result in weakness or paralysis of the superior rectus, inferior rectus, medial rectus, inferior oblique, or levator palpebrae muscles, or impaired parasympathetic innervation to the pupil. With a complete oculomotor palsy, the eyelid will be paralyzed, the eye will be in an abducted and inferior position, and the pupil will be markedly dilated. Commonly associated conditions include neoplasms, CRANIOCEREBRAL TRAUMA, ischemia (especially in association with DIABETES MELLITUS), and aneurysmal compression. (From Adams et al., Principles of Neurology, 6th ed, p270)'),
  'D015841': NtO(MH='Enophthalmos', MS='Recession of the eyeball into the orbit.'),
  'D015842': NtO(MH='Serine Proteinase Inhibitors', MS='Exogenous or endogenous compounds which inhibit SERINE ENDOPEPTIDASES.'),
  'D015843': NtO(MH='Serpins', MS='A family of serine proteinase inhibitors which are similar in amino acid sequence and mechanism of inhibition, but differ in their specificity toward proteolytic enzymes. Some members of the serpin family may be substrates rather than inhibitors of SERINE ENDOPEPTIDASES.'),
  'D015844': NtO(MH='Heparin Cofactor II', MS='A sulfated plasma protein with a MW of approximately 66kDa that resembles ANTITHROMBIN III. The protein is an inhibitor of thrombin in plasma and is activated by dermatan sulfate or heparin. It is a member of the serpin superfamily.'),
  'D015845': NtO(MH='Tonic Pupil', MS='A pupillary abnormality characterized by a poor pupillary light reaction, reduced accommodation, iris sector palsies, an enhanced pupillary response to near effort that results in a prolonged, "tonic" constriction, and slow pupillary redilation. This condition is associated with injury to the postganglionic parasympathetic innervation to the pupil. (From Miller et al., Clinical Neuro-Ophthalmology, 4th ed, pp492-500)'),
  'D015846': NtO(MH='Monokines', MS='Soluble mediators of the immune response that are neither antibodies nor complement. They are produced largely, but not exclusively, by monocytes and macrophages.'),
  'D015847': NtO(MH='Interleukin-4', MS='A soluble factor produced by activated T-LYMPHOCYTES that induces the expression of MHC CLASS II GENES and FC RECEPTORS on B-LYMPHOCYTES and causes their proliferation and differentiation. It also acts on T-lymphocytes, MAST CELLS, and several other hematopoietic lineage cells.'),
  'D015848': NtO(MH='Interleukin-5', MS='A cytokine that promotes differentiation and activation of EOSINOPHILS. It also triggers activated B-LYMPHOCYTES to differentiate into IMMUNOGLOBULIN-secreting cells.'),
  'D015849': NtO(MH='Plasminogen Inactivators', MS='Important modulators of the activity of plasminogen activators. The inhibitors belong to the serpin family of proteins and inhibit both the tissue-type and urokinase-type plasminogen activators.'),
  'D015850': NtO(MH='Interleukin-6', MS='A cytokine that stimulates the growth and differentiation of B-LYMPHOCYTES and is also a growth factor for HYBRIDOMAS and plasmacytomas. It is produced by many different cells including T-LYMPHOCYTES; MONOCYTES; and FIBROBLASTS.'),
  'D015851': NtO(MH='Interleukin-7', MS='A cytokine produced by bone marrow stromal cells that promotes the growth of B-LYMPHOCYTE precursors and is co-mitogenic with INTERLEUKIN-2 for mature T-LYMPHOCYTE activation.'),
  'D015852': NtO(MH='Eukaryotic Initiation Factor-2', MS='Eukaryotic initiation factor of protein synthesis. In higher eukaryotes the factor consists of three subunits: alpha, beta, and gamma. As initiation proceeds, eIF-2 forms a ternary complex with Met-tRNAi and GTP.'),
  'D015853': NtO(MH='Cysteine Proteinase Inhibitors', MS='Exogenous and endogenous compounds which inhibit CYSTEINE ENDOPEPTIDASES.'),
  'D015854': NtO(MH='Up-Regulation', MS='A positive regulatory effect on physiological processes at the molecular, cellular, or systemic level. At the molecular level, the major regulatory sites include membrane receptors, genes (GENE EXPRESSION REGULATION), mRNAs (RNA, MESSENGER), and proteins.'),
  'D015858': NtO(MH='Anisometropia', MS='A condition of an inequality of refractive power of the two eyes.'),
  'D015861': NtO(MH='Retinal Neovascularization', MS='Formation of new blood vessels originating from the retinal veins and extending along the inner (vitreal) surface of the retina.'),
  'D015862': NtO(MH='Choroid Diseases', MS='Disorders of the choroid including hereditary choroidal diseases, neoplasms, and other abnormalities of the vascular layer of the uvea.'),
  'D015863': NtO(MH='Iridocyclitis', MS='Acute or chronic inflammation of the iris and ciliary body characterized by exudates into the anterior chamber, discoloration of the iris, and constricted, sluggish pupil. Symptoms include radiating pain, photophobia, lacrimation, and interference with vision.'),
  'D015864': NtO(MH='Panuveitis', MS='Inflammation in which both the anterior and posterior segments of the uvea are involved and a specific focus is not apparent. It is often severe and extensive and a serious threat to vision. Causes include systemic diseases such as tuberculosis, sarcoidosis, and syphilis, as well as malignancies. The intermediate segment of the eye is not involved.'),
  'D015865': NtO(MH='Racquet Sports', MS='Games in which players use a racquet to hit a ball or similar type object.'),
  'D015866': NtO(MH='Uveitis, Posterior', MS='Inflammation of the choroid as well as the retina and vitreous body. Some form of visual disturbance is usually present. The most important characteristics of posterior uveitis are vitreous opacities, choroiditis, and chorioretinitis.'),
  'D015867': NtO(MH='Uveitis, Intermediate', MS='Inflammation of the pars plana, ciliary body, and adjacent structures.'),
  'D015868': NtO(MH='Pars Planitis', MS='Form of granulomatous uveitis occurring in the region of the pars plana. This disorder is a common condition with no detectable focal pathology. It causes fibrovascular proliferation at the inferior ora serrata.'),
  'D015870': NtO(MH='Gene Expression', MS='The phenotypic manifestation of a gene or genes by the processes of GENETIC TRANSCRIPTION and GENETIC TRANSLATION.'),
  'D015871': NtO(MH='Scientific Misconduct', MS='Intentional falsification of scientific data by presentation of fraudulent or incomplete or uncorroborated findings as scientific fact.'),
  'D015873': NtO(MH='Northwest Territories', MS='A federally administered division of Canada. Its capital is Yellowknife. The former northern and eastern-most parts of the Territory comprise the new territory of Nunavut, effective April 1, 1999.'),
  'D015874': NtO(MH='Yukon Territory', MS="A territory of northwest Canada, bounded on the north by the Arctic Ocean, on the south by British Columbia, and on the west by Alaska. Its capital is Whitehorse. It takes its name from the Yukon River, the Indian yu-kun-ah, meaning big river. (From Webster's New Geographical Dictionary, 1988, p1367 & Room, Brewer's Dictionary of Names, 1992, p608)"),
  'D015875': NtO(MH='Anisocoria', MS='Unequal pupil size, which may represent a benign physiologic variant or a manifestation of disease. Pathologic anisocoria reflects an abnormality in the musculature of the iris (IRIS DISEASES) or in the parasympathetic or sympathetic pathways that innervate the pupil. Physiologic anisocoria refers to an asymmetry of pupil diameter, usually less than 2mm, that is not associated with disease.'),
  'D015876': NtO(MH='Northern Territory', MS='Territory in north central Australia, between the states of Queensland and Western Australia. Its capital is Darwin.'),
  'D015877': NtO(MH='Miosis', MS='Pupillary constriction. This may result from congenital absence of the dilatator pupillary muscle, defective sympathetic innervation, or irritation of the CONJUNCTIVA or CORNEA.'),
  'D015878': NtO(MH='Mydriasis', MS='Dilation of pupils to greater than 6 mm combined with failure of the pupils to constrict when stimulated with light. This condition may occur due to injury of the pupillary fibers in the oculomotor nerve, in acute angle-closure glaucoma, and in ADIE SYNDROME.'),
  'D015879': NtO(MH='Myosin Subfragments', MS='Parts of the myosin molecule resulting from cleavage by proteolytic enzymes (PAPAIN; TRYPSIN; or CHYMOTRYPSIN) at well-localized regions. Study of these isolated fragments helps to delineate the functional roles of different parts of myosin. Two of the most common subfragments are myosin S-1 and myosin S-2. S-1 contains the heads of the heavy chains plus the light chains and S-2 contains part of the double-stranded, alpha-helical, heavy chain tail (myosin rod).'),
  'D015882': NtO(MH='Retinal Necrosis Syndrome, Acute', MS='Mild to fulminant necrotizing vaso-occlusive retinitis associated with a high incidence of retinal detachment and poor vision outcome.'),
  'D015891': NtO(MH='Cystatins', MS='A homologous group of endogenous CYSTEINE PROTEINASE INHIBITORS. The cystatins inhibit most CYSTEINE ENDOPEPTIDASES such as PAPAIN, and other peptidases which have a sulfhydryl group at the active site.'),
  'D015894': NtO(MH='Genome, Human', MS='The complete genetic complement contained in the DNA of a set of CHROMOSOMES in a HUMAN. The length of the human genome is about 3 billion base pairs.'),
  'D015895': NtO(MH='Relative Value Scales', MS='Coded listings of physician or other professional services using units that indicate the relative value of the various services they perform. They take into account time, skill, and overhead cost required for each service, but generally do not consider the relative cost-effectiveness. Appropriate conversion factors can be used to translate the abstract units of the relative value scales into dollar fees for each service based on work expended, practice costs, and training costs.'),
  'D015897': NtO(MH='Comorbidity', MS='The presence of co-existing or additional diseases with reference to an initial diagnosis or with reference to the index condition that is the subject of study. Comorbidity may affect the ability of affected individuals to function and also their survival; it may be used as a prognostic indicator for length of hospital stay, cost factors, and outcome or survival.'),
  'D015898': NtO(MH='Tomography Scanners, X-Ray Computed', MS='X-ray image-detecting devices that make a focused image of body structures lying in a predetermined plane from which more complex images are computed.'),
  'D015899': NtO(MH='Tomography, Emission-Computed, Single-Photon', MS='A method of computed tomography that uses radionuclides which emit a single photon of a given energy. The camera is rotated 180 or 360 degrees around the patient to capture images at multiple positions along the arc. The computer is then used to reconstruct the transaxial, sagittal, and coronal images from the 3-dimensional distribution of radionuclides in the organ. The advantages of SPECT are that it can be used to observe biochemical and physiological processes as well as size and volume of the organ. The disadvantage is that, unlike positron-emission tomography where the positron-electron annihilation results in the emission of 2 photons at 180 degrees from each other, SPECT requires physical collimation to line up the photons, which results in the loss of many available photons and hence degrades the image.'),
  'D015900': NtO(MH='Radiography, Dual-Energy Scanned Projection', MS='A method of producing a high-quality scan by digitizing and subtracting the images produced by high- and low-energy x-rays.'),
  'D015901': NtO(MH='Angiography, Digital Subtraction', MS='A method of delineating blood vessels by subtracting a tissue background image from an image of tissue plus intravascular contrast material that attenuates the X-ray photons. The background image is determined from a digitized image taken a few moments before injection of the contrast material. The resulting angiogram is a high-contrast image of the vessel. This subtraction technique allows extraction of a high-intensity signal from the superimposed background information. The image is thus the result of the differential absorption of X-rays by different tissues.'),
  'D015902': NtO(MH='Gamma Cameras', MS='Electronic instruments that produce photographs or cathode-ray tube images of the gamma-ray emissions from organs containing radionuclide tracers.'),
  'D015903': NtO(MH='Moire Topography', MS='A method of three-dimensional morphometry in which contour maps are produced from the overlapping interference fringes created when an object is illuminated by beams of coherent light issuing from two different point sources.'),
  'D015904': NtO(MH='Biliopancreatic Diversion', MS='A surgical procedure which diverts pancreatobiliary secretions via the duodenum and the jejunum into the colon, the remaining small intestine being anastomosed to the stomach after antrectomy. The procedure produces less diarrhea than does jejunoileal bypass.'),
  'D015905': NtO(MH='Gastric Balloon', MS='An inflatable device implanted in the stomach as an adjunct to therapy of morbid obesity. Specific types include the silicone Garren-Edwards Gastric Bubble (GEGB), approved by the FDA in 1985, and the Ballobes Balloon.'),
  'D015906': NtO(MH='Angioplasty, Balloon, Coronary', MS='Dilation of an occluded coronary artery (or arteries) by means of a balloon catheter to restore myocardial blood supply.'),
  'D015907': NtO(MH='Hysteroscopy', MS='Endoscopic examination, therapy or surgery of the interior of the uterus.'),
  'D015908': NtO(MH='Counterpulsation', MS='A technique for assisting the circulation by decreasing the afterload of the left ventricle and augmenting the diastolic pressure. It may be achieved by intra-aortic balloon, or by implanting a special pumping device in the chest, or externally by applying a negative pressure to the lower extremities during cardiac systole.'),
  'D015911': NtO(MH='Sclerotherapy', MS='Treatment of varicose veins, hemorrhoids, gastric and esophageal varices, and peptic ulcer hemorrhage by injection or infusion of chemical agents which cause localized thrombosis and eventual fibrosis and obliteration of the vessels.'),
  'D015912': NtO(MH='Thrombolytic Therapy', MS='Use of infusions of FIBRINOLYTIC AGENTS to destroy or dissolve thrombi in blood vessels or bypass grafts.'),
  'D015913': NtO(MH='Chelation Therapy', MS='Therapy of heavy metal poisoning using agents which sequester the metal from organs or tissues and bind it firmly within the ring structure of a new compound which can be eliminated from the body.'),
  'D015914': NtO(MH='Estrogen Replacement Therapy', MS='The use of hormonal agents with estrogen-like activity in postmenopausal or other estrogen-deficient women to alleviate effects of hormone deficiency, such as vasomotor symptoms, DYSPAREUNIA, and progressive development of OSTEOPOROSIS. This may also include the use of progestational agents in combination therapy.'),
  'D015915': NtO(MH='Motion Therapy, Continuous Passive', MS='Movement of a body part initiated and maintained by a mechanical or electrical device to restore normal range of motion to joints, muscles, or tendons after surgery, prosthesis implantation, contracture flexion, or long immobilization.'),
  'D015916': NtO(MH='Drainage, Postural', MS="A rehabilitation therapy for removal of copious mucus secretion from the lung of patients with diseases such as CHRONIC BRONCHITIS; BRONCHIECTASIS; PULMONARY ABSCESS; or CYSTIC FIBROSIS. The patient's head is placed in a downward incline (so the TRACHEA is inferior to the affected area) for 15- to 20-minute sessions."),
  'D015917': NtO(MH='Penile Prosthesis', MS='Rigid, semi-rigid, or inflatable cylindric hydraulic devices, with either combined or separate reservoir and pumping systems, implanted for the surgical treatment of organic ERECTILE DYSFUNCTION.'),
  'D015918': NtO(MH='Infusion Pumps, Implantable', MS='Implanted fluid propulsion systems with self-contained power source for providing long-term controlled-rate delivery of drugs such as chemotherapeutic agents or analgesics. Delivery rate may be externally controlled or osmotically or peristatically controlled with the aid of transcutaneous monitoring.'),
  'D015919': NtO(MH='Ventilators, Negative-Pressure', MS='Body ventilators that assist ventilation by applying intermittent subatmospheric pressure around the thorax, abdomen, or airway and periodically expand the chest wall and inflate the lungs. They are relatively simple to operate and do not require tracheostomy. These devices include the tank ventilators ("iron lung"), Portalung, Pneumowrap, and chest cuirass ("tortoise shell").'),
  'D015920': NtO(MH='Gas Scavengers', MS='Apparatus for removing exhaled or leaked anesthetic gases or other volatile agents, thus reducing the exposure of operating room personnel to such agents, as well as preventing the buildup of potentially explosive mixtures in operating rooms or laboratories.'),
  'D015921': NtO(MH='Dental Implants', MS='Biocompatible materials placed into (endosseous) or onto (subperiosteal) the jawbone to support a crown, bridge, or artificial tooth, or to stabilize a diseased tooth.'),
  'D015922': NtO(MH='Complement C1q', MS='A subcomponent of complement C1, composed of six copies of three polypeptide chains (A, B, and C), each encoded by a separate gene (C1QA; C1QB; C1QC). This complex is arranged in nine subunits (six disulfide-linked dimers of A and B, and three disulfide-linked homodimers of C). C1q has binding sites for antibodies (the heavy chain of IMMUNOGLOBULIN G or IMMUNOGLOBULIN M). The interaction of C1q and immunoglobulin activates the two proenzymes COMPLEMENT C1R and COMPLEMENT C1S, thus initiating the cascade of COMPLEMENT ACTIVATION via the CLASSICAL COMPLEMENT PATHWAY.'),
  'D015923': NtO(MH='Complement C1r', MS='A 80-kDa subcomponent of complement C1, existing as a SERINE PROTEASE proenzyme in the intact complement C1 complex. When COMPLEMENT C1Q is bound to antibodies, the changed tertiary structure causes autolytic activation of complement C1r which is cleaved into two chains, A (heavy) and B (light, the serine protease), connected by disulfide bonds. The activated C1r serine protease, in turn, activates COMPLEMENT C1S proenzyme by cleaving the Arg426-Ile427 bond. No fragment is released when either C1r or C1s is cleaved.'),
  'D015924': NtO(MH='Blood Pressure Monitors', MS='Devices for continuously measuring and displaying the arterial blood pressure.'),
  'D015925': NtO(MH='Cryopreservation', MS='Preservation of cells, tissues, organs, or embryos by freezing. In histological preparations, cryopreservation or cryofixation is used to maintain the existing form, structure, and chemical composition of all the constituent elements of the specimens.'),
  'D015926': NtO(MH='Complement C3a', MS='The smaller fragment generated from the cleavage of complement C3 by C3 CONVERTASE. C3a, a 77-amino acid peptide, is a mediator of local inflammatory process. It induces smooth MUSCLE CONTRACTION, and HISTAMINE RELEASE from MAST CELLS and LEUKOCYTES. C3a is considered an anaphylatoxin along with COMPLEMENT C4A; COMPLEMENT C5A; and COMPLEMENT C5A, DES-ARGININE.'),
  'D015927': NtO(MH='Skin Test End-Point Titration', MS='A quantitative form of intradermal testing for the relative allergenicity of a substance. It is used to determine the amount of an allergen that will be tolerated in ALLERGEN IMMUNOTHERAPY.'),
  'D015928': NtO(MH='Cognitive Behavioral Therapy', MS='A directive form of psychotherapy based on the interpretation of situations (cognitive structure of experiences) that determine how an individual feels and behaves. It is based on the premise that cognition, the process of acquiring knowledge and forming beliefs, is a primary determinant of mood and behavior. The therapy uses behavioral and verbal techniques to identify and correct negative thinking that is at the root of the aberrant behavior.'),
  'D015930': NtO(MH='Diet Records', MS='Records of nutrient intake over a specific period of time, usually kept by the patient.'),
  'D015931': NtO(MH='Intensive Care, Neonatal', MS='Continuous care and monitoring of newborn infants with life-threatening conditions, in any setting.'),
  'D015932': NtO(MH='Complement C3c', MS='A 206-amino-acid fragment in the alpha chain (672-1663) of C3b. It is generated when C3b is inactivated (iC3b) and its alpha chain is cleaved by COMPLEMENT FACTOR I into C3c (749-954), and C3dg (955-1303) in the presence COMPLEMENT FACTOR H.'),
  'D015933': NtO(MH='Complement C3d', MS='A 302-amino-acid fragment in the alpha chain (672-1663) of C3b. It is generated when C3b is inactivated (iC3b) and its alpha chain is cleaved by COMPLEMENT FACTOR I into C3c, and C3dg (955-1303) in the presence COMPLEMENT FACTOR H. Serum proteases further degrade C3dg into C3d (1002-1303) and C3g (955-1001).'),
  'D015934': NtO(MH='Complement C4a', MS='The smaller fragment formed when complement C4 is cleaved by COMPLEMENT C1S. It is an anaphylatoxin that causes symptoms of immediate hypersensitivity (HYPERSENSITIVITY, IMMEDIATE) but its activity is weaker than that of COMPLEMENT C3A or COMPLEMENT C5A.'),
  'D015935': NtO(MH='Complement C4b', MS='The large fragment formed when COMPLEMENT C4 is cleaved by COMPLEMENT C1S. The membrane-bound C4b binds COMPLEMENT C2A, a SERINE PROTEASE, to form C4b2a (CLASSICAL PATHWAY C3 CONVERTASE) and subsequent C4b2a3b (CLASSICAL PATHWAY C5 CONVERTASE).'),
  'D015936': NtO(MH='Complement C5a', MS='The minor fragment formed when C5 convertase cleaves C5 into C5a and COMPLEMENT C5B. C5a is a 74-amino-acid glycopeptide with a carboxy-terminal ARGININE that is crucial for its spasmogenic activity. Of all the complement-derived anaphylatoxins, C5a is the most potent in mediating immediate hypersensitivity (HYPERSENSITIVITY, IMMEDIATE), smooth MUSCLE CONTRACTION; HISTAMINE RELEASE; and migration of LEUKOCYTES to site of INFLAMMATION.'),
  'D015937': NtO(MH='Complement C5a, des-Arginine', MS='A derivative of complement C5a, generated when the carboxy-terminal ARGININE is removed by CARBOXYPEPTIDASE B present in normal human serum. C5a des-Arg shows complete loss of spasmogenic activity though it retains some chemotactic ability (CHEMOATTRACTANTS).'),
  'D015938': NtO(MH='Complement Membrane Attack Complex', MS='A product of COMPLEMENT ACTIVATION cascade, regardless of the pathways, that forms transmembrane channels causing disruption of the target CELL MEMBRANE and cell lysis. It is formed by the sequential assembly of terminal complement components (COMPLEMENT C5B; COMPLEMENT C6; COMPLEMENT C7; COMPLEMENT C8; and COMPLEMENT C9) into the target membrane. The resultant C5b-8-poly-C9 is the "membrane attack complex" or MAC.'),
  'D015939': NtO(MH='Kitasamycin', MS='A macrolide antibiotic produced by Streptomyces kitasatoensis. The drug has antimicrobial activity against a wide spectrum of pathogens.'),
  'D015941': NtO(MH='Complement Hemolytic Activity Assay', MS='A screening assay for circulating COMPLEMENT PROTEINS. Diluted SERUM samples are added to antibody-coated ERYTHROCYTES and the percentage of cell lysis is measured. The values are expressed by the so called CH50, in HEMOLYTIC COMPLEMENT units per milliliter, which is the dilution of serum required to lyse 50 percent of the erythrocytes in the assay.'),
  'D015942': NtO(MH='Factor VIIa', MS='Activated form of factor VII. Factor VIIa activates factor X in the extrinsic pathway of blood coagulation.'),
  'D015943': NtO(MH='Factor Va', MS='Activated form of factor V. It is an essential cofactor for the activation of prothrombin catalyzed by factor Xa.'),
  'D015944': NtO(MH='Factor VIIIa', MS='Activated form of factor VIII. The B-domain of factor VIII is proteolytically cleaved by thrombin to form factor VIIIa. Factor VIIIa exists as a non-covalent dimer in a metal-linked (probably calcium) complex and functions as a cofactor in the enzymatic activation of factor X by factor IXa. Factor VIIIa is similar in structure and generation to factor Va.'),
  'D015945': NtO(MH='Factor XIa', MS='Activated form of factor XI. In the intrinsic pathway, Factor XI is activated to XIa by factor XIIa in the presence of cofactor HMWK; (HIGH MOLECULAR WEIGHT KININOGEN). Factor XIa then activates factor IX to factor IXa in the presence of calcium.'),
  'D015946': NtO(MH='Ethylene Dibromide', MS='An effective soil fumigant, insecticide, and nematocide. In humans, it causes severe burning of skin and irritation of the eyes and respiratory tract. Prolonged inhalation may cause liver necrosis. It is also used in gasoline. Members of this group have caused liver and lung cancers in rodents. According to the Fourth Annual Report on Carcinogens (NTP 85-002, 1985), 1,2-dibromoethane may reasonably be anticipated to be a carcinogen.'),
  'D015948': NtO(MH='Keratoplasty, Penetrating', MS='Partial or total replacement of all layers of a central portion of the cornea.'),
  'D015949': NtO(MH='Factor IXa', MS='Activated form of factor IX. This activation can take place via the intrinsic pathway of blood coagulation by the action of factor XIa and calcium, or via the extrinsic pathway by the action of factor VIIa, thromboplastin, and calcium. Factor IXa serves to activate factor X to Xa by cleaving the arginyl-leucine peptide bond in factor X.'),
  'D015951': NtO(MH='Factor Xa', MS='Activated form of factor X that participates in both the intrinsic and extrinsic pathways of blood coagulation. It catalyzes the conversion of prothrombin to thrombin in conjunction with other cofactors.'),
  'D015956': NtO(MH='Factor XIIa', MS='Activated form of factor XII. In the initial event in the intrinsic pathway of blood coagulation, kallikrein (with cofactor HIGH MOLECULAR WEIGHT KININOGEN) cleaves factor XII to XIIa. Factor XIIa is then further cleaved by kallikrein, plasmin, and trypsin to yield smaller factor XII fragments (Hageman-Factor fragments). These fragments increase the activity of prekallikrein to kallikrein but decrease the procoagulant activity of factor XII.'),
  'D015964': NtO(MH='Gene Expression Regulation, Bacterial', MS='Any of the processes by which cytoplasmic or intercellular factors influence the differential control of gene action in bacteria.'),
  'D015966': NtO(MH='Gene Expression Regulation, Fungal', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control of gene action in fungi.'),
  'D015967': NtO(MH='Gene Expression Regulation, Viral', MS='Any of the processes by which cytoplasmic factors influence the differential control of gene action in viruses.'),
  'D015969': NtO(MH='National Academies of Science, Engineering, and Medicine, U.S., Health and Medicine Division', MS='An organization that initiates and conducts studies of national policy and planning for health care and health-related education and research; it also responds to requests from the federal government and other agencies for studies and advice. Was founded as the Institute of Medicine in 1970 and changed in 2016 to a Division of the Academies.'),
  'D015971': NtO(MH='Gene Expression Regulation, Enzymologic', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control of gene action in enzyme synthesis.'),
  'D015972': NtO(MH='Gene Expression Regulation, Neoplastic', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control of gene action in neoplastic tissue.'),
  'D015973': NtO(MH='Gene Expression Regulation, Leukemic', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control of gene action in leukemia.'),
  'D015974': NtO(MH='Wrongful Life', MS='In civil law a cause of action which alleges that a defendant has wrongfully caused a child to be born.'),
  'D015975': NtO(MH='Consensus Development Conferences, NIH as Topic', MS='Articles on conferences sponsored by NIH presenting summary statements representing the majority agreement of physicians, scientists, and other professionals convening for the purpose of reaching a consensus on a subject of interest. This heading is used for NIH consensus conferences as a means of scientific communication. In indexing it is viewed as a type of review article and as a tag for any article appearing in any publication of the NIH Office of Medical Applications of Research (OMAR).'),
  'D015977': NtO(MH='Eukaryotic Initiation Factor-1', MS='A eukaryotic initiation factor that binds to 40S ribosomal subunits. Although initially considered a "non-essential" factor for eukaryotic transcription initiation, eukaryotic initiation factor-1 is now thought to play an important role in localizing RIBOSOMES at the initiation codon of MRNA.'),
  'D015978': NtO(MH='Trichosanthin', MS='Plant-derived ribosome-inactivating protein (RIP) purified from the Chinese medicinal herb tian-hua-fen which is obtained from the root tubers of Trichosanthes kirilowii. It has been used as an abortifacient and in the treatment of trophoblastic tumors. GLQ223 (Compound Q), a highly purified form of trichosanthin, has been proposed as antiviral treatment for AIDS.'),
  'D015979': NtO(MH='Killer Cells, Lymphokine-Activated', MS='Cytolytic lymphocytes with the unique capacity of killing natural killer (NK)-resistant fresh tumor cells. They are INTERLEUKIN-2-activated NK cells that have no MAJOR HISTOCOMPATIBILITY COMPLEX restriction or need for antigen stimulation. LAK cells are used for ADOPTIVE IMMUNOTHERAPY in cancer patients.'),
  'D015980': NtO(MH='Public Health Practice', MS='The activities and endeavors of the public health services in a community on any level.'),
  'D015981': NtO(MH='Epidemiologic Factors', MS='Events, characteristics, or other definable entities that have the potential to bring about a change in a health condition or other defined outcome.'),
  'D015982': NtO(MH='Bias', MS='Any deviation of results or inferences from the truth, or processes leading to such deviation. Bias can result from several sources: one-sided or systematic variations in measurement from the true value (systematic error); flaws in study design; deviation of inferences, interpretations, or analyses based on flawed data or data collection; etc. There is no sense of prejudice or subjectivity implied in the assessment of bias under these conditions.'),
  'D015983': NtO(MH='Selection Bias', MS='The introduction of error due to systematic differences in the characteristics between those selected and those not selected for a given study. In sampling bias, error is the result of failure to ensure that all members of the reference population have a known chance of selection in the sample.'),
  'D015984': NtO(MH='Causality', MS='The relating of causes to the effects they produce. Causes are termed necessary when they must always precede an effect and sufficient when they initiate or produce an effect. Any of several factors may be associated with the potential disease causation or outcome, including predisposing factors, enabling factors, precipitating factors, reinforcing factors, and risk factors.'),
  'D015985': NtO(MH='Precipitating Factors', MS='Factors associated with the definitive onset of a disease, illness, accident, behavioral response, or course of action. Usually one factor is more important or more obviously recognizable than others, if several are involved, and one may often be regarded as "necessary". Examples include exposure to specific disease; amount or level of an infectious organism, drug, or noxious agent, etc.'),
  'D015986': NtO(MH='Confounding Factors, Epidemiologic', MS='Factors that can cause or prevent the outcome of interest but are not intermediate variables of the factor(s) under investigation.'),
  'D015987': NtO(MH='Effect Modifier, Epidemiologic', MS='Factors that modify the effect of the putative causal factor(s) under study.'),
  'D015988': NtO(MH='Cohort Effect', MS='Variation in health status arising from different causal factors to which each birth cohort in a population is exposed as environment and society change.'),
  'D015989': NtO(MH='Healthy Worker Effect', MS="Phenomenon of workers' usually exhibiting overall death rates lower than those of the general population due to the fact that the severely ill and disabled are ordinarily excluded from employment."),
  'D015990': NtO(MH='Placebo Effect', MS='An effect usually, but not necessarily, beneficial that is attributable to an expectation that the regimen will have an effect, i.e., the effect is due to the power of suggestion.'),
  'D015991': NtO(MH='Epidemiologic Measurements', MS='Statistical calculations on the occurrence of disease or other health-related conditions in defined populations.'),
  'D015992': NtO(MH='Body Mass Index', MS='An indicator of body density as determined by the relationship of BODY WEIGHT to BODY HEIGHT. BMI=weight (kg)/height squared (m2). BMI correlates with body fat (ADIPOSE TISSUE). Their relationship varies with age and gender. For adults, BMI falls into these categories: below 18.5 (underweight); 18.5-24.9 (normal); 25.0-29.9 (overweight); 30.0 and above (obese). (National Center for Health Statistics, Centers for Disease Control and Prevention)'),
  'D015993': NtO(MH='Life Tables', MS='Summarizing techniques used to describe the pattern of mortality and survival in populations. These methods can be applied to the study not only of death, but also of any defined endpoint such as the onset of disease or the occurrence of disease complications.'),
  'D015994': NtO(MH='Incidence', MS='The number of new cases of a given disease during a given period in a specified population. It also is used for the rate at which new events occur in a defined population. It is differentiated from PREVALENCE, which refers to all cases, new or old, in the population at a given time.'),
  'D015995': NtO(MH='Prevalence', MS='The total number of cases of a given disease in a specified population at a designated time. It is differentiated from INCIDENCE, which refers to the number of new cases in the population at a given time.'),
  'D015996': NtO(MH='Survival Rate', MS='The proportion of survivors in a group, e.g., of patients, studied and followed over a period, or the proportion of persons in a specified group alive at the beginning of a time interval who survive to the end of the interval. It is often studied using life table methods.'),
  'D015997': NtO(MH='Neonatal Screening', MS='The identification of selected parameters in newborn infants by various tests, examinations, or other procedures. Screening may be performed by clinical or laboratory measures. A screening test is designed to sort out healthy neonates (INFANT, NEWBORN) from those not well, but the screening test is not intended as a diagnostic device, rather instead as epidemiologic.'),
  'D015998': NtO(MH='Abbreviated Injury Scale', MS='Classification system for assessing impact injury severity developed and published by the American Association for Automotive Medicine. It is the system of choice for coding single injuries and is the foundation for methods assessing multiple injuries or for assessing cumulative effects of more than one injury. These include Maximum AIS (MAIS), Injury Severity Score (ISS), and Probability of Death Score (PODS).'),
  'D015999': NtO(MH='Multivariate Analysis', MS='A set of techniques used when variation in several variables has to be studied simultaneously. In statistics, multivariate analysis is interpreted as any analytic method that allows simultaneous study of two or more dependent variables.'),
  'D016000': NtO(MH='Cluster Analysis', MS='A set of statistical methods used to group variables or observations into strongly inter-related subgroups. In epidemiology, it may be used to analyze a closely grouped series of events or cases of disease or other health-related phenomenon with well-defined distribution patterns in relation to time or place or both.'),
  'D016001': NtO(MH='Confidence Intervals', MS='A range of values for a variable of interest, e.g., a rate, constructed so that this range has a specified probability of including the true value of the variable.'),
  'D016002': NtO(MH='Discriminant Analysis', MS='A statistical analytic technique used with discrete dependent variables, concerned with separating sets of observed values and allocating new values. It is sometimes used instead of regression analysis.'),
  'D016008': NtO(MH='Statistical Distributions', MS='The complete summaries of the frequencies of the values or categories of a measurement made on a group of items, a population, or other collection of data. The distribution tells either how many or what proportion of the group was found to have each value (or each range of values) out of all the possible values that the quantitative measure can have.'),
  'D016009': NtO(MH='Chi-Square Distribution', MS='A distribution in which a variable is distributed like the sum of the squares of any given independent random variable, each of which has a normal distribution with mean of zero and variance of one. The chi-square test is a statistical test based on comparison of a test statistic to a chi-square distribution. The oldest of these tests are used to detect whether two or more population distributions differ from one another.'),
  'D016010': NtO(MH='Binomial Distribution', MS='The probability distribution associated with two mutually exclusive outcomes; used to model cumulative incidence rates and prevalence rates. The Bernoulli distribution is a special case of binomial distribution.'),
  'D016011': NtO(MH='Normal Distribution', MS='Continuous frequency distribution of infinite range. Its properties are as follows: 1, continuous, symmetrical distribution with both tails extending to infinity; 2, arithmetic mean, mode, and median identical; and 3, shape completely determined by the mean and standard deviation.'),
  'D016012': NtO(MH='Poisson Distribution', MS='A distribution function used to describe the occurrence of rare events or to describe the sampling distribution of isolated counts in a continuum of time or space.'),
  'D016013': NtO(MH='Likelihood Functions', MS='Functions constructed from a statistical model and a set of observed data which give the probability of that data for various values of the unknown model parameters. Those parameter values that maximize the probability are the maximum likelihood estimates of the parameters.'),
  'D016014': NtO(MH='Linear Models', MS='Statistical models in which the value of a parameter for a given value of a factor is assumed to be equal to a + bx, where a and b are constants. The models predict a linear regression.'),
  'D016015': NtO(MH='Logistic Models', MS="Statistical models which describe the relationship between a qualitative dependent variable (that is, one which can take only certain discrete values, such as the presence or absence of a disease) and an independent variable. A common application is in epidemiology for estimating an individual's risk (probability of a disease) as a function of a given risk factor."),
  'D016016': NtO(MH='Proportional Hazards Models', MS='Statistical models used in survival analysis that assert that the effect of the study factors on the hazard rate in the study population is multiplicative and does not change over time.'),
  'D016017': NtO(MH='Odds Ratio', MS='The ratio of two odds. The exposure-odds ratio for case control data is the ratio of the odds in favor of exposure among cases to the odds in favor of exposure among noncases. The disease-odds ratio for a cohort or cross section is the ratio of the odds in favor of disease among the exposed to the odds in favor of disease among the unexposed. The prevalence-odds ratio refers to an odds ratio derived cross-sectionally from studies of prevalent cases.'),
  'D016018': NtO(MH='Least-Squares Analysis', MS='A principle of estimation in which the estimates of a set of parameters in a statistical model are those quantities minimizing the sum of squared differences between the observed values of a dependent variable and the values predicted by the model.'),
  'D016019': NtO(MH='Survival Analysis', MS='A class of statistical procedures for estimating the survival function (function of time, starting with a population 100% well at a given time and providing the percentage of the population still well at later times). The survival analysis is then used for making inferences about the effects of treatments, prognostic factors, exposures, and other covariates on the function.'),
  'D016020': NtO(MH='Epidemiologic Study Characteristics', MS='Works about types and formulations of studies used in epidemiological research.'),
  'D016021': NtO(MH='Epidemiologic Studies', MS='Studies designed to examine associations, commonly, hypothesized causal relations. They are usually concerned with identifying or measuring the effects of risk factors or exposures. The common types of analytic study are CASE-CONTROL STUDIES; COHORT STUDIES; and CROSS-SECTIONAL STUDIES.'),
  'D016022': NtO(MH='Case-Control Studies', MS='Comparisons that start with the identification of persons with the disease or outcome of interest and a control (comparison, referent) group without the disease or outcome of interest. The relationship of an attribute is examined by comparing both groups with regard to the frequency or levels of outcome over time.'),
  'D016023': NtO(MH='Integrins', MS='A family of transmembrane glycoproteins (MEMBRANE GLYCOPROTEINS) consisting of noncovalent heterodimers. They interact with a wide variety of ligands including EXTRACELLULAR MATRIX PROTEINS; COMPLEMENT, and other cells, while their intracellular domains interact with the CYTOSKELETON. The integrins consist of at least three identified families: the cytoadhesin receptors(RECEPTORS, CYTOADHESIN), the leukocyte adhesion receptors (RECEPTORS, LEUKOCYTE ADHESION), and the VERY LATE ANTIGEN RECEPTORS. Each family contains a common beta-subunit (INTEGRIN BETA CHAINS) combined with one or more distinct alpha-subunits (INTEGRIN ALPHA CHAINS). These receptors participate in cell-matrix and cell-cell adhesion in many physiologically important processes, including embryological development; HEMOSTASIS; THROMBOSIS; WOUND HEALING; immune and nonimmune defense mechanisms; and oncogenic transformation.'),
  'D016024': NtO(MH='Receptors, Cytoadhesin', MS='A group of INTEGRINS that includes the platelet outer membrane glycoprotein GPIIb-IIIa (PLATELET GLYCOPROTEIN GPIIB-IIIA COMPLEX) and the vitronectin receptor (RECEPTORS, VITRONECTIN). They play a major role in cell adhesion and serve as receptors for fibronectin, von Willebrand factor, and vitronectin.'),
  'D016025': NtO(MH='Bone Transplantation', MS='The grafting of bone from a donor site to a recipient site.'),
  'D016026': NtO(MH='Bone Marrow Transplantation', MS='The transference of BONE MARROW from one human or animal to another for a variety of purposes including HEMATOPOIETIC STEM CELL TRANSPLANTATION or MESENCHYMAL STEM CELL TRANSPLANTATION.'),
  'D016027': NtO(MH='Heart Transplantation', MS='The transference of a heart from one human or animal to another.'),
  'D016028': NtO(MH='Receptors, Leukocyte-Adhesion', MS='Family of proteins associated with the capacity of LEUKOCYTES to adhere to each other and to certain substrata, e.g., the C3bi component of complement. Members of this family are the LYMPHOCYTE FUNCTION-ASSOCIATED ANTIGEN-1; (LFA-1), the MACROPHAGE-1 ANTIGEN; (Mac-1), and the INTEGRIN ALPHAXBETA2 or p150,95 leukocyte adhesion protein. They all share a common beta-subunit which is the CD18 antigen. All three of the above antigens are absent in inherited LEUKOCYTE-ADHESION DEFICIENCY SYNDROME, which is characterized by recurrent bacterial infections, impaired pus formation, and wound healing as well as abnormalities in a wide spectrum of adherence-dependent functions of granulocytes, monocytes, and lymphoid cells.'),
  'D016029': NtO(MH='Receptors, Very Late Antigen', MS='Members of the integrin family appearing late after T-cell activation. They are a family of proteins initially identified at the surface of stimulated T-cells, but now identified on a variety of cell types. At least six VLA antigens have been identified as heterodimeric adhesion receptors consisting of a single common beta-subunit and different alpha-subunits.'),
  'D016030': NtO(MH='Kidney Transplantation', MS='The transference of a kidney from one human or animal to another.'),
  'D016031': NtO(MH='Liver Transplantation', MS='The transference of a part of or an entire liver from one human or animal to another.'),
  'D016032': NtO(MH='Randomized Controlled Trials as Topic', MS='Works about clinical trials that involve at least one test treatment and one control treatment, concurrent enrollment and follow-up of the test- and control-treated groups, and in which the treatments to be administered are selected by a random process, such as the use of a random-numbers table.'),
  'D016035': NtO(MH='Pancreas Transplantation', MS='The transference of a pancreas from one human or animal to another.'),
  'D016036': NtO(MH='Seroepidemiologic Studies', MS='EPIDEMIOLOGIC STUDIES based on the detection through serological testing of characteristic change in the serum level of specific ANTIBODIES. Latent subclinical infections and carrier states can thus be detected in addition to clinically overt cases.'),
  'D016037': NtO(MH='Single-Blind Method', MS='A method in which either the observer(s) or the subject(s) is kept ignorant of the group to which the subjects are assigned.'),
  'D016038': NtO(MH='Skin Transplantation', MS='The grafting of skin in humans or animals from one site to another to replace a lost portion of the body surface skin.'),
  'D016039': NtO(MH='Corneal Transplantation', MS='Partial or total replacement of the CORNEA from one human or animal to another.'),
  'D016040': NtO(MH='Lung Transplantation', MS='The transference of either one or both of the lungs from one human or animal to another.'),
  'D016041': NtO(MH='Heart-Lung Transplantation', MS='The simultaneous, or near simultaneous, transference of heart and lungs from one human or animal to another.'),
  'D016042': NtO(MH='Transplantation, Heterotopic', MS='Transplantation of tissue typical of one area to a different recipient site. The tissue may be autologous, heterologous, or homologous.'),
  'D016044': NtO(MH='Fusion Proteins, bcr-abl', MS='Translation products of a fusion gene derived from CHROMOSOMAL TRANSLOCATION of C-ABL GENES to the genetic locus of the breakpoint cluster region gene on chromosome 22. Several different variants of the bcr-abl fusion proteins occur depending upon the precise location of the chromosomal breakpoint. These variants can be associated with distinct subtypes of leukemias such as PRECURSOR CELL LYMPHOBLASTIC LEUKEMIA-LYMPHOMA; LEUKEMIA, MYELOGENOUS, CHRONIC, BCR-ABL POSITIVE; and NEUTROPHILIC LEUKEMIA, CHRONIC.'),
  'D016045': NtO(MH='Human Genome Project', MS='A coordinated effort of researchers to map (CHROMOSOME MAPPING) and sequence (SEQUENCE ANALYSIS, DNA) the human GENOME.'),
  'D016047': NtO(MH='Zalcitabine', MS="A dideoxynucleoside compound in which the 3'-hydroxy group on the sugar moiety has been replaced by a hydrogen. This modification prevents the formation of phosphodiester linkages which are needed for the completion of nucleic acid chains. The compound is a potent inhibitor of HIV replication at low concentrations, acting as a chain-terminator of viral DNA by binding to reverse transcriptase. Its principal toxic side effect is axonal degeneration resulting in peripheral neuropathy."),
  'D016048': NtO(MH='Dideoxyadenosine', MS="A dideoxynucleoside compound in which the 3'-hydroxy group on the sugar moiety has been replaced by a hydrogen. This modification prevents the formation of phosphodiester linkages which are needed for the completion of nucleic acid chains. The compound is an inhibitor of HIV replication, acting as a chain-terminator of viral DNA by binding to reverse transcriptase. Its principal side effect is nephrotoxicity. In vivo, dideoxyadenosine is rapidly metabolized to DIDANOSINE (ddI) by enzymatic deamination; ddI is then converted to dideoxyinosine monophosphate and ultimately to dideoxyadenosine triphosphate, the putative active metabolite."),
  'D016049': NtO(MH='Didanosine', MS="A dideoxynucleoside compound in which the 3'-hydroxy group on the sugar moiety has been replaced by a hydrogen. This modification prevents the formation of phosphodiester linkages which are needed for the completion of nucleic acid chains. Didanosine is a potent inhibitor of HIV replication, acting as a chain-terminator of viral DNA by binding to reverse transcriptase; ddI is then metabolized to dideoxyadenosine triphosphate, its putative active metabolite."),
  'D016052': NtO(MH='Protozoan Vaccines', MS='Suspensions of attenuated or killed protozoa administered for the prevention or treatment of infectious protozoan disease.'),
  'D016053': NtO(MH='RNA, Protozoan', MS='Ribonucleic acid in protozoa having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D016054': NtO(MH='DNA, Protozoan', MS='Deoxyribonucleic acid that makes up the genetic material of protozoa.'),
  'D016055': NtO(MH='Urinary Retention', MS='Inability to empty the URINARY BLADDER with voiding (URINATION).'),
  'D016056': NtO(MH='Immunodominant Epitopes', MS='Subunits of the antigenic determinant that are most easily recognized by the immune system and thus most influence the specificity of the induced antibody.'),
  'D016057': NtO(MH='Evoked Potentials, Auditory, Brain Stem', MS='Electrical waves in the CEREBRAL CORTEX generated by BRAIN STEM structures in response to auditory click stimuli. These are found to be abnormal in many patients with CEREBELLOPONTINE ANGLE lesions, MULTIPLE SCLEROSIS, or other DEMYELINATING DISEASES.'),
  'D016058': NtO(MH='Analgesia, Patient-Controlled', MS='Relief of PAIN, without loss of CONSCIOUSNESS, through ANALGESIC AGENTS administered by the patients. It has been used successfully to control POSTOPERATIVE PAIN, during OBSTETRIC LABOR, after BURNS, and in TERMINAL CARE. The choice of agent, dose, and lockout interval greatly influence effectiveness. The potential for overdose can be minimized by combining small bolus doses with a mandatory interval between successive doses (lockout interval).'),
  'D016059': NtO(MH='Range of Motion, Articular', MS='The distance and direction to which a bone joint can be extended. Range of motion is a function of the condition of the joints, muscles, and connective tissues involved. Joint flexibility can be improved through appropriate MUSCLE STRETCHING EXERCISES.'),
  'D016060': NtO(MH='Sorption Detoxification', MS='Elimination of toxic or biologically active substances from body fluids by interaction with a sorbent medium. The types of media include absorbents, adsorbents, ion-exchange materials, and complexing agents. Detoxification can be extracorporeal (hemodialysis, hemofiltration, hemoperfusion, plasmapheresis), or occur inside the body (enterosorption, peritoneal dialysis).'),
  'D016061': NtO(MH='Enterosorption', MS='Adsorption of substances from the gastrointestinal tract onto an orally administered sorbent medium like activated charcoal. This technique is used to eliminate toxic and some biologically active substances and serves to modify the lipid and amino acid spectrum of the intestinal contents.'),
  'D016062': NtO(MH='Porosity', MS='Condition of having pores or open spaces. This often refers to bones, bone implants, or bone cements, but can refer to the porous state of any solid substance.'),
  'D016063': NtO(MH='Blood Loss, Surgical', MS='Loss of blood during a surgical procedure.'),
  'D016064': NtO(MH='Vaginal Birth after Cesarean', MS='Delivery of an infant through the vagina in a female who has had a prior cesarean section.'),
  'D016065': NtO(MH='Myoelectric Complex, Migrating', MS='A pattern of gastrointestinal muscle contraction and depolarizing myoelectric activity that moves from the stomach to the ILEOCECAL VALVE at regular frequency during the interdigestive period. The complex and its accompanying motor activity periodically cleanse the bowel of interdigestive secretion and debris in preparation for the next meal.'),
  'D016066': NtO(MH='Pleural Effusion, Malignant', MS='Presence of fluid in the PLEURAL CAVITY as a complication of malignant disease. Malignant pleural effusions often contain actual malignant cells.'),
  'D016067': NtO(MH='Nerve Transfer', MS='Surgical reinnervation of a denervated peripheral target using a healthy donor nerve and/or its proximal stump. The direct connection is usually made to a healthy postlesional distal portion of a non-functioning nerve or implanted directly into denervated muscle or insensitive skin. Nerve sprouts will grow from the transferred nerve into the denervated elements and establish contact between them and the neurons that formerly controlled another area.'),
  'D016079': NtO(MH='Refusal to Treat', MS='Refusal of the health professional to initiate or continue treatment of a patient or group of patients. The refusal can be based on any reason. The concept is differentiated from PATIENT REFUSAL OF TREATMENT see TREATMENT REFUSAL which originates with the patient and not the health professional.'),
  'D016080': NtO(MH='Arachnoid Cysts', MS='Intracranial or spinal cavities containing a cerebrospinal-like fluid, the wall of which is composed of arachnoidal cells. They are most often developmental or related to trauma. Intracranial arachnoid cysts usually occur adjacent to arachnoidal cistern and may present with HYDROCEPHALUS; HEADACHE; SEIZURES; and focal neurologic signs. (From Joynt, Clinical Neurology, 1994, Ch44, pp105-115)'),
  'D016081': NtO(MH='Moon', MS='The natural satellite of the planet Earth. It includes the lunar cycles or phases, the lunar month, lunar landscapes, geography, and soil.'),
  'D016082': NtO(MH='Solar System', MS='The group of celestial bodies, including the EARTH, orbiting around and gravitationally bound by the sun. It includes eight planets, one minor planet, and 34 natural satellites, more than 1,000 observed comets, and thousands of lesser bodies known as MINOR PLANETS (asteroids) and METEOROIDS. (From Academic American Encyclopedia, 1983)'),
  'D016083': NtO(MH='Planets', MS='Celestial bodies orbiting around the sun or other stars.'),
  'D016084': NtO(MH='Bronchoconstriction', MS='Narrowing of the caliber of the BRONCHI, physiologically or as a result of pharmacological intervention.'),
  'D016085': NtO(MH='Bronchoconstrictor Agents', MS='Agents causing the narrowing of the lumen of a bronchus or bronchiole.'),
  'D016086': NtO(MH='Lentivirus', MS="A genus of the family RETROVIRIDAE consisting of non-oncogenic retroviruses that produce multi-organ diseases characterized by long incubation periods and persistent infection. Lentiviruses are unique in that they contain open reading frames (ORFs) between the pol and env genes and in the 3' env region. Five serogroups are recognized, reflecting the mammalian hosts with which they are associated. HIV-1 is the type species."),
  'D016087': NtO(MH='Immunodeficiency Virus, Bovine', MS='The type species of LENTIVIRUS, subgenus bovine lentiviruses (LENTIVIRUSES, BOVINE), found in cattle and causing lymphadenopathy, LYMPHOCYTOSIS, central nervous system lesions, progressive weakness, and emaciation. It has immunological cross-reactivity with other lentiviruses including HIV.'),
  'D016088': NtO(MH='Arthritis-Encephalitis Virus, Caprine', MS='A species of LENTIVIRUS, subgenus ovine-caprine lentiviruses (LENTIVIRUSES, OVINE-CAPRINE), closely related to VISNA-MAEDI VIRUS and causing acute encephalomyelitis; chronic arthritis; PNEUMONIA; MASTITIS; and GLOMERULONEPHRITIS in goats. It is transmitted mainly in the colostrum and milk.'),
  'D016089': NtO(MH='Immunodeficiency Virus, Feline', MS='A species of LENTIVIRUS, subgenus feline lentiviruses (LENTIVIRUSES, FELINE) isolated from cats with a chronic wasting syndrome, presumed to be immune deficiency. There are 3 strains: Petaluma (FIP-P), Oma (FIP-O) and Puma lentivirus (PLV). There is no antigenic relationship between FIV and HIV, nor does FIV grow in human T-cells.'),
  'D016092': NtO(MH='Spumavirus', MS='Genus of non-oncogenic retroviruses which establish persistent infections in many animal species but are considered non-pathogenic. Its species have been isolated from primates (including humans), cattle, cats, hamsters, horses, and sea lions. Spumaviruses have a foamy or lace-like appearance and are often accompanied by syncytium formation. SIMIAN FOAMY VIRUS is the type species.'),
  'D016093': NtO(MH='Mason-Pfizer monkey virus', MS='A species of BETARETROVIRUS isolated from mammary carcinoma in rhesus monkeys. It appears to have evolved from a recombination between a murine B oncovirus and a primate C oncovirus related to the baboon endogenous virus. Several serologically distinct strains exist. MPMV induces SIMIAN AIDS.'),
  'D016097': NtO(MH='Simian Acquired Immunodeficiency Syndrome', MS='Acquired defect of cellular immunity that occurs naturally in macaques infected with SRV serotypes, experimentally in monkeys inoculated with SRV or MASON-PFIZER MONKEY VIRUS; (MPMV), or in monkeys infected with SIMIAN IMMUNODEFICIENCY VIRUS.'),
  'D016098': NtO(MH='Gerstmann-Straussler-Scheinker Disease', MS='An autosomal dominant familial prion disease with a wide spectrum of clinical presentations including ATAXIA, spastic paraparesis, extrapyramidal signs, and DEMENTIA. Clinical onset is in the third to sixth decade of life and the mean duration of illness prior to death is five years. Several kindreds with variable clinical and pathologic features have been described. Pathologic features include cerebral prion protein amyloidosis, and spongiform or neurofibrillary degeneration. (From Brain Pathol 1998 Jul;8(3):499-513; Brain Pathol 1995 Jan;5(1):61-75)'),
  'D016099': NtO(MH='Endoscopy, Gastrointestinal', MS='Endoscopic examination, therapy or surgery of the gastrointestinal tract.'),
  'D016103': NtO(MH='Spinal Fractures', MS='Broken bones in the vertebral column.'),
  'D016104': NtO(MH='Oligohydramnios', MS='A condition of abnormally low AMNIOTIC FLUID volume. Principal causes include malformations of fetal URINARY TRACT; FETAL GROWTH RETARDATION; GESTATIONAL HYPERTENSION; nicotine poisoning; and PROLONGED PREGNANCY.'),
  'D016105': NtO(MH='Parathyroidectomy', MS='Excision of one or more of the parathyroid glands.'),
  'D016106': NtO(MH='Methicillin Resistance', MS='Non-susceptibility of a microbe to the action of METHICILLIN, a semi-synthetic penicillin derivative.'),
  'D016107': NtO(MH='Epidermolysis Bullosa Acquisita', MS='Form of epidermolysis bullosa characterized by trauma-induced, subepidermal blistering with no family history of the disease. Direct immunofluorescence shows IMMUNOGLOBULIN G deposited at the dermo-epidermal junction.'),
  'D016108': NtO(MH='Epidermolysis Bullosa Dystrophica', MS='Form of epidermolysis bullosa characterized by atrophy of blistered areas, severe scarring, and nail changes. It is most often present at birth or in early infancy and occurs in both autosomal dominant and recessive forms. All forms of dystrophic epidermolysis bullosa result from mutations in COLLAGEN TYPE VII, a major component fibrils of BASEMENT MEMBRANE and EPIDERMIS.'),
  'D016109': NtO(MH='Epidermolysis Bullosa, Junctional', MS='Form of epidermolysis bullosa having onset at birth or during the neonatal period and transmitted through autosomal recessive inheritance. It is characterized by generalized blister formation, extensive denudation, and separation and cleavage of the basal cell plasma membranes from the basement membrane.'),
  'D016110': NtO(MH='Epidermolysis Bullosa Simplex', MS='A form of epidermolysis bullosa characterized by serous bullae that heal without scarring. Mutations in the genes that encode KERATIN-5 and KERATIN-14 have been associated with several subtypes of epidermolysis bullosa simplex.'),
  'D016111': NtO(MH='Sjogren-Larsson Syndrome', MS='An autosomal recessive neurocutaneous disorder characterized by severe ichthyosis MENTAL RETARDATION; SPASTIC PARAPLEGIA; and congenital ICHTHYOSIS. It is caused by mutation of gene encoding microsomal fatty ALDEHYDE DEHYDROGENASE leading to defect in fatty alcohol metabolism.'),
  'D016112': NtO(MH='Ichthyosis Vulgaris', MS='Most common form of ICHTHYOSIS characterized by prominent scaling especially on the exterior surfaces of the extremities. It is inherited as an autosomal dominant trait.'),
  'D016113': NtO(MH='Ichthyosiform Erythroderma, Congenital', MS='Designation for several severe forms of ichthyosis, present at birth, that are characterized by hyperkeratotic scaling. Infants may be born encased in a collodion membrane which begins shedding within 24 hours. This is followed in about two weeks by persistent generalized scaling. The forms include bullous (HYPERKERATOSIS, EPIDERMOLYTIC), non-bullous (ICHTHYOSIS, LAMELLAR), wet type, and dry type.'),
  'D016114': NtO(MH='Ichthyosis, X-Linked', MS='Chronic form of ichthyosis that is inherited as a sex-linked recessive trait carried on the X-chromosome and transmitted to the male offspring. It is characterized by severe scaling, especially on the extremities, and is associated with steroid sulfatase deficiency.'),
  'D016115': NtO(MH='Albinism, Oculocutaneous', MS='Heterogeneous group of autosomal recessive disorders comprising at least four recognized types, all having in common varying degrees of hypopigmentation of the skin, hair, and eyes. The two most common are the tyrosinase-positive and tyrosinase-negative types.'),
  'D016116': NtO(MH='Piebaldism', MS='Autosomal dominant, congenital disorder characterized by localized hypomelanosis of the skin and hair. The most familiar feature is a white forelock presenting in 80 to 90 percent of the patients. The underlying defect is possibly related to the differentiation and migration of melanoblasts, as well as to defective development of the neural crest (neurocristopathy). Piebaldism may be closely related to WAARDENBURG SYNDROME.'),
  'D016117': NtO(MH='Albinism, Ocular', MS='Albinism affecting the eye in which pigment of the hair and skin is normal or only slightly diluted. The classic type is X-linked (Nettleship-Falls), but an autosomal recessive form also exists. Ocular abnormalities may include reduced pigmentation of the iris, nystagmus, photophobia, strabismus, and decreased visual acuity.'),
  'D016118': NtO(MH='Anterior Cruciate Ligament', MS='A strong ligament of the knee that originates from the posteromedial portion of the lateral condyle of the femur, passes anteriorly and inferiorly between the condyles, and attaches to the depression in front of the intercondylar eminence of the tibia.'),
  'D016119': NtO(MH='Posterior Cruciate Ligament', MS='A strong ligament of the knee that originates from the anterolateral surface of the medial condyle of the femur, passes posteriorly and inferiorly between the condyles, and attaches to the posterior intercondylar area of the tibia.'),
  'D016121': NtO(MH='Brachiocephalic Veins', MS='Large veins on either side of the root of the neck formed by the junction of the internal jugular and subclavian veins. They drain blood from the head, neck, and upper extremities, and unite to form the superior vena cava.'),
  'D016122': NtO(MH='Brachiocephalic Trunk', MS='The first and largest artery branching from the aortic arch. It distributes blood to the right side of the head and neck and to the right arm.'),
  'D016123': NtO(MH='Campylobacter jejuni', MS='A species of bacteria that resemble small tightly coiled spirals. Its organisms are known to cause abortion in sheep and fever and enteritis in man and may be associated with enteric diseases of calves, lambs, and other animals.'),
  'D016124': NtO(MH='Eikenella corrodens', MS='Gram-negative bacteria isolated from infections of the respiratory and intestinal tracts and from the buccal cavity, intestinal tract, and urogenital tract. They are probably part of the normal flora of man and animals.'),
  'D016127': NtO(MH='Heart Valve Prolapse', MS='Downward displacement of any one of the HEART VALVES from its normal position. This usually results in failed valve closure.'),
  'D016130': NtO(MH='Immunophenotyping', MS='Process of classifying cells of the immune system based on structural and functional differences. The process is commonly used to analyze and sort T-lymphocytes into subsets based on CD antigens by the technique of flow cytometry.'),
  'D016131': NtO(MH='Lymphocyte Subsets', MS='A classification of lymphocytes based on structurally or functionally different populations of cells.'),
  'D016133': NtO(MH='Polymerase Chain Reaction', MS='In vitro method for producing large amounts of specific DNA or RNA fragments of defined length and sequence from small amounts of short oligonucleotide flanking sequences (primers). The essential steps include thermal denaturation of the double-stranded target molecules, annealing of the primers to their complementary sequences, and extension of the annealed primers by enzymatic synthesis with DNA polymerase. The reaction is efficient, specific, and extremely sensitive. Uses for the reaction include disease diagnosis, detection of difficult-to-isolate pathogens, mutation analysis, genetic testing, DNA sequencing, and analyzing evolutionary relationships.'),
  'D016134': NtO(MH='Organizational Policy', MS='A course or method of action selected, usually by an organization, institution, university, society, etc., from among alternatives to guide and determine present and future decisions and positions on matters of public interest or social concern. It does not include internal policy relating to organization and administration within the corporate body, for which ORGANIZATION AND ADMINISTRATION is available.'),
  'D016135': NtO(MH='Spinal Dysraphism', MS='Congenital defects of closure of one or more vertebral arches, which may be associated with malformations of the spinal cord, nerve roots, congenital fibrous bands, lipomas, and congenital cysts. These malformations range from mild (e.g., SPINA BIFIDA OCCULTA) to severe, including rachischisis where there is complete failure of neural tube and spinal cord fusion, resulting in exposure of the spinal cord at the surface. Spinal dysraphism includes all forms of spina bifida. The open form is called SPINA BIFIDA CYSTICA and the closed form is SPINA BIFIDA OCCULTA. (From Joynt, Clinical Neurology, 1992, Ch55, p34)'),
  'D016136': NtO(MH='Spina Bifida Occulta', MS='A common congenital midline defect of fusion of the vertebral arch without protrusion of the spinal cord or meninges. The lesion is also covered by skin. L5 and S1 are the most common vertebrae involved. The condition may be associated with an overlying area of hyperpigmented skin, a dermal sinus, or an abnormal patch of hair. The majority of individuals with this malformation are asymptomatic although there is an increased incidence of tethered cord syndrome and lumbar SPONDYLOSIS. (From Joynt, Clinical Neurology, 1992, Ch55, p34)'),
  'D016137': NtO(MH='Spina Bifida Cystica', MS='A form of spinal dysraphism associated with a protruding cyst made up of either meninges (i.e., a MENINGOCELE) or meninges in combination with spinal cord tissue (i.e., a MENINGOMYELOCELE). These lesions are frequently associated with spinal cord dysfunction, HYDROCEPHALUS, and SYRINGOMYELIA. (From Davis et al., Textbook of Neuropathology, 2nd ed, pp224-5)'),
  'D016138': NtO(MH='Walking', MS='An activity in which the body advances at a slow to moderate pace by moving the feet in a coordinated fashion. This includes recreational walking, walking for fitness, and competitive race-walking.'),
  'D016139': NtO(MH='Martial Arts', MS='Activities in which participants learn self-defense mainly through the use of hand-to-hand combat. Judo involves throwing an opponent to the ground while karate (which includes kung fu and tae kwon do) involves kicking and punching an opponent.'),
  'D016142': NtO(MH='Holoprosencephaly', MS='Anterior midline brain, cranial, and facial malformations resulting from the failure of the embryonic prosencephalon to undergo segmentation and cleavage. Alobar prosencephaly is the most severe form and features anophthalmia; cyclopia; severe INTELLECTUAL DISABILITY; CLEFT LIP; CLEFT PALATE; SEIZURES; and microcephaly. Semilobar holoprosencepaly is characterized by hypotelorism, microphthalmia, coloboma, nasal malformations, and variable degrees of INTELLECTUAL DISABILITY. Lobar holoprosencephaly is associated with mild (or absent) facial malformations and intellectual abilities that range from mild INTELLECTUAL DISABILITY to normal. Holoprosencephaly is associated with CHROMOSOME ABNORMALITIES.'),
  'D016145': NtO(MH='Endoscopy, Digestive System', MS='Endoscopic examination, therapy or surgery of the digestive tract.'),
  'D016147': NtO(MH='Genes, Tumor Suppressor', MS='Genes that inhibit expression of the tumorigenic phenotype. They are normally involved in holding cellular growth in check. When tumor suppressor genes are inactivated or lost, a barrier to normal proliferation is removed and unregulated growth is possible.'),
  'D016153': NtO(MH='Genes, Suppressor', MS='Genes that have a suppressor allele or suppressor mutation (SUPPRESSION, GENETIC) which cancels the effect of a previous mutation, enabling the wild-type phenotype to be maintained or partially restored. For example, amber suppressors cancel the effect of an AMBER NONSENSE MUTATION.'),
  'D016154': NtO(MH='Digestive System Fistula', MS='An abnormal passage communicating between any components of the digestive system, or between any part of the digestive system and surrounding organ(s).'),
  'D016155': NtO(MH='Oral Fistula', MS='An abnormal passage within the mouth communicating between two or more anatomical structures.'),
  'D016156': NtO(MH='Respiratory Tract Fistula', MS='An abnormal passage communicating between any component of the respiratory tract or between any part of the respiratory system and surrounding organs.'),
  'D016157': NtO(MH='Vascular Fistula', MS='An abnormal passage between two or more BLOOD VESSELS, between ARTERIES; VEINS; or between an artery and a vein.'),
  'D016158': NtO(MH='Genes, p53', MS='Tumor suppressor genes located on the short arm of human chromosome 17 and coding for the phosphoprotein p53.'),
  'D016159': NtO(MH='Tumor Suppressor Protein p53', MS='Nuclear phosphoprotein encoded by the p53 gene (GENES, P53) whose normal function is to control CELL PROLIFERATION and APOPTOSIS. A mutant or absent p53 protein has been found in LEUKEMIA; OSTEOSARCOMA; LUNG CANCER; and COLORECTAL CANCER.'),
  'D016160': NtO(MH='Retinoblastoma Protein', MS='Product of the retinoblastoma tumor suppressor gene. It is a nuclear phosphoprotein hypothesized to normally act as an inhibitor of cell proliferation. Rb protein is absent in retinoblastoma cell lines. It also has been shown to form complexes with the adenovirus E1A protein, the SV40 T antigen, and the human papilloma virus E7 protein.'),
  'D016161': NtO(MH='Genes, Retinoblastoma', MS='Tumor suppressor genes located on human chromosome 13 in the region 13q14 and coding for a family of phosphoproteins with molecular weights ranging from 104 kDa to 115 kDa. One copy of the wild-type Rb gene is necessary for normal retinal development. Loss or inactivation of both alleles at this locus results in retinoblastoma.'),
  'D016162': NtO(MH='Genes, Wilms Tumor', MS='Genes at loci that are involved in the development of WILMS TUMOR. Included are human WT1 at 11p13 and human WT2 (MTACR1) at 11p15.'),
  'D016163': NtO(MH='Genes, DCC', MS='Tumor suppressor genes located in the 18q21-qter region of human chromosome 18. The absence of these genes is associated with the formation of colorectal cancer (DCC stands for deleted in colorectal cancer). The products of these genes show significant homology to neural cell adhesion molecules and other related cell surface glycoproteins.'),
  'D016164': NtO(MH='Bison', MS='A genus of the family Bovidae having two species: B. bison and B. bonasus. This concept is differentiated from BUFFALOES, which refers to Bubalus arnee and Syncerus caffer.'),
  'D016166': NtO(MH='Free Radical Scavengers', MS='Substances that eliminate free radicals. Among other effects, they protect PANCREATIC ISLETS against damage by CYTOKINES and prevent myocardial and pulmonary REPERFUSION INJURY.'),
  'D016167': NtO(MH='Integrin alphaXbeta2', MS='A major adhesion-associated heterodimer molecule expressed by MONOCYTES; GRANULOCYTES; NK CELLS; and some LYMPHOCYTES. The alpha subunit is the CD11C ANTIGEN, a surface antigen expressed on some myeloid cells. The beta subunit is the CD18 ANTIGEN.'),
  'D016169': NtO(MH='Lymphocyte Function-Associated Antigen-1', MS='An integrin heterodimer widely expressed on cells of hematopoietic origin. CD11A ANTIGEN comprises the alpha chain and the CD18 antigen (CD18 ANTIGENS) the beta chain. Lymphocyte function-associated antigen-1 is a major receptor of T-CELLS; B-CELLS; and GRANULOCYTES. It mediates the leukocyte adhesion reactions underlying cytolytic conjugate formation, helper T-cell interactions, and antibody-dependent killing by NATURAL KILLER CELLS and granulocytes. Intracellular adhesion molecule-1 has been defined as a ligand for lymphocyte function-associated antigen-1.'),
  'D016170': NtO(MH='Accelerated Idioventricular Rhythm', MS='A type of automatic, not reentrant, ectopic ventricular rhythm with episodes lasting from a few seconds to a minute which usually occurs in patients with acute myocardial infarction or with DIGITALIS toxicity. The ventricular rate is faster than normal but slower than tachycardia, with an upper limit of 100 -120 beats per minute. Suppressive therapy is rarely necessary.'),
  'D016171': NtO(MH='Torsades de Pointes', MS='A malignant form of polymorphic ventricular tachycardia that is characterized by HEART RATE between 200 and 250 beats per minute, and QRS complexes with changing amplitude and twisting of the points. The term also describes the syndrome of tachycardia with prolonged ventricular repolarization, long QT intervals exceeding 500 milliseconds or BRADYCARDIA. Torsades de pointes may be self-limited or may progress to VENTRICULAR FIBRILLATION.'),
  'D016172': NtO(MH='DNA Fingerprinting', MS='A technique for identifying individuals of a species that is based on the uniqueness of their DNA sequence. Uniqueness is determined by identifying which combination of allelic variations occur in the individual at a statistically relevant number of different loci. In forensic studies, RESTRICTION FRAGMENT LENGTH POLYMORPHISM of multiple, highly polymorphic VNTR LOCI or MICROSATELLITE REPEAT loci are analyzed. The number of loci used for the profile depends on the ALLELE FREQUENCY in the population.'),
  'D016173': NtO(MH='Macrophage Colony-Stimulating Factor', MS='A mononuclear phagocyte colony-stimulating factor (M-CSF) synthesized by mesenchymal cells. The compound stimulates the survival, proliferation, and differentiation of hematopoietic cells of the monocyte-macrophage series. M-CSF is a disulfide-bonded glycoprotein dimer with a MW of 70 kDa. It binds to a specific high affinity receptor (RECEPTOR, MACROPHAGE COLONY-STIMULATING FACTOR).'),
  'D016174': NtO(MH='Hepacivirus', MS='A genus of FLAVIVIRIDAE causing parenterally-transmitted HEPATITIS C which is associated with transfusions and drug abuse. Hepatitis C virus is the type species.'),
  'D016175': NtO(MH='B-Lymphocyte Subsets', MS='A classification of B-lymphocytes based on structurally or functionally different populations of cells.'),
  'D016176': NtO(MH='T-Lymphocyte Subsets', MS='A classification of T-lymphocytes, especially into helper/inducer, suppressor/effector, and cytotoxic subsets, based on structurally or functionally different populations of cells.'),
  'D016177': NtO(MH='Macrophage-1 Antigen', MS='An adhesion-promoting leukocyte surface membrane heterodimer. The alpha subunit consists of the CD11b ANTIGEN and the beta subunit the CD18 ANTIGEN. The antigen, which is an integrin, functions both as a receptor for complement 3 and in cell-cell and cell-substrate adhesive interactions.'),
  'D016178': NtO(MH='Granulocyte-Macrophage Colony-Stimulating Factor', MS='An acidic glycoprotein of MW 23 kDa with internal disulfide bonds. The protein is produced in response to a number of inflammatory mediators by mesenchymal cells present in the hemopoietic environment and at peripheral sites of inflammation. GM-CSF is able to stimulate the production of neutrophilic granulocytes, macrophages, and mixed granulocyte-macrophage colonies from bone marrow cells and can stimulate the formation of eosinophil colonies from fetal liver progenitor cells. GM-CSF can also stimulate some functional activities in mature granulocytes and macrophages.'),
  'D016179': NtO(MH='Granulocyte Colony-Stimulating Factor', MS='A glycoprotein of MW 25 kDa containing internal disulfide bonds. It induces the survival, proliferation, and differentiation of neutrophilic granulocyte precursor cells and functionally activates mature blood neutrophils. Among the family of colony-stimulating factors, G-CSF is the most potent inducer of terminal differentiation to granulocytes and macrophages of leukemic myeloid cell lines.'),
  'D016180': NtO(MH='Lentivirus Infections', MS='Virus diseases caused by the Lentivirus genus. They are multi-organ diseases characterized by long incubation periods and persistent infection.'),
  'D016181': NtO(MH='Feline Acquired Immunodeficiency Syndrome', MS='Acquired defect of cellular immunity that occurs in cats infected with feline immunodeficiency virus (FIV) and in some cats infected with feline leukemia virus (FeLV).'),
  'D016182': NtO(MH='Visna', MS='Demyelinating leukoencephalomyelitis of sheep caused by the VISNA-MAEDI VIRUS. It is similar to but not the same as SCRAPIE.'),
  'D016183': NtO(MH='Murine Acquired Immunodeficiency Syndrome', MS='Acquired defect of cellular immunity that occurs in mice infected with mouse leukemia viruses (MuLV). The syndrome shows striking similarities with human AIDS and is characterized by lymphadenopathy, profound immunosuppression, enhanced susceptibility to opportunistic infections, and B-cell lymphomas.'),
  'D016184': NtO(MH='Receptors, Colony-Stimulating Factor', MS='Cell surface receptors for colony stimulating factors, local mediators, and hormones that regulate the survival, proliferation, and differentiation of hemopoietic cells.'),
  'D016185': NtO(MH='Receptors, Interleukin-3', MS='High affinity receptors for INTERLEUKIN-3. They are found on early HEMATOPOIETIC PROGENITOR CELLS; progenitors of MYELOID CELLS; EOSINOPHILS; and BASOPHILS. Interleukin-3 receptors are formed by the dimerization of the INTERLEUKIN-3 RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR COMMON BETA SUBUNIT.'),
  'D016186': NtO(MH='Receptor, Macrophage Colony-Stimulating Factor', MS='A receptor for MACROPHAGE COLONY-STIMULATING FACTOR encoded by the c-fms proto-oncogene (GENES, FMS). It contains an intrinsic protein-tyrosine kinase activity. When activated the receptor undergoes autophosphorylation, phosphorylation of down-stream signaling molecules and rapid down-regulation.'),
  'D016187': NtO(MH='Receptors, Granulocyte-Macrophage Colony-Stimulating Factor', MS='Receptors that bind and internalize the granulocyte-macrophage stimulating factor. Their MW is believed to be 84 kD. The most mature myelomonocytic cells, specifically human neutrophils, macrophages, and eosinophils, express the highest number of affinity receptors for this growth factor.'),
  'D016188': NtO(MH='Receptors, Granulocyte Colony-Stimulating Factor', MS='Receptors that bind and internalize GRANULOCYTE COLONY-STIMULATING FACTOR. Their MW is believed to be 150 kD. These receptors are found mainly on a subset of myelomonocytic cells.'),
  'D016189': NtO(MH='Dystrophin', MS='A muscle protein localized in surface membranes which is the product of the Duchenne/Becker muscular dystrophy gene. Individuals with Duchenne muscular dystrophy usually lack dystrophin completely while those with Becker muscular dystrophy have dystrophin of an altered size. It shares features with other cytoskeletal proteins such as SPECTRIN and alpha-actinin but the precise function of dystrophin is not clear. One possible role might be to preserve the integrity and alignment of the plasma membrane to the myofibrils during muscle contraction and relaxation. MW 400 kDa.'),
  'D016190': NtO(MH='Carboplatin', MS='An organoplatinum compound that possesses antineoplastic activity.'),
  'D016192': NtO(MH='Resting Phase, Cell Cycle', MS='A quiescent state of cells during G1 PHASE.'),
  'D016193': NtO(MH='G1 Phase', MS='The period of the CELL CYCLE preceding DNA REPLICATION in S PHASE. Subphases of G1 include "competence" (to respond to growth factors), G1a (entry into G1), G1b (progression), and G1c (assembly). Progression through the G1 subphases is effected by limiting growth factors, nutrients, or inhibitors.'),
  'D016194': NtO(MH='Receptors, N-Methyl-D-Aspartate', MS='A class of ionotropic glutamate receptors characterized by affinity for N-methyl-D-aspartate. NMDA receptors have an allosteric binding site for glycine which must be occupied for the channel to open efficiently and a site within the channel itself to which magnesium ions bind in a voltage-dependent manner. The positive voltage dependence of channel conductance and the high permeability of the conducting channel to calcium ions (as well as to monovalent cations) are important in excitotoxicity and neuronal plasticity.'),
  'D016195': NtO(MH='G2 Phase', MS='The period of the CELL CYCLE following DNA synthesis (S PHASE) and preceding M PHASE (cell division phase). The CHROMOSOMES are tetraploid in this point.'),
  'D016196': NtO(MH='S Phase', MS='Phase of the CELL CYCLE following G1 and preceding G2 when the entire DNA content of the nucleus is replicated. It is achieved by bidirectional replication at multiple sites along each chromosome.'),
  'D016199': NtO(MH='Herpesvirus 7, Human', MS='A species in the genus ROSEOLOVIRUS, of the family HERPESVIRIDAE. It was isolated from activated, CD4-positive T-lymphocytes taken from the blood of a healthy human.'),
  'D016200': NtO(MH='Maturation-Promoting Factor', MS='Protein kinase that drives both the mitotic and meiotic cycles in all eukaryotic organisms. In meiosis it induces immature oocytes to undergo meiotic maturation. In mitosis it has a role in the G2/M phase transition. Once activated by CYCLINS; MPF directly phosphorylates some of the proteins involved in nuclear envelope breakdown, chromosome condensation, spindle assembly, and the degradation of cyclins. The catalytic subunit of MPF is PROTEIN P34CDC2.'),
  'D016201': NtO(MH='Receptors, Lymphocyte Homing', MS='Cell surface glycoproteins on lymphocytes and other leukocytes that mediate adhesion to specialized blood vessels called high endothelial venules. Several different classes of lymphocyte homing receptors have been identified, and they appear to target different surface molecules (addressins) on high endothelial venules in different tissues. The adhesion plays a crucial role in the trafficking of lymphocytes.'),
  'D016202': NtO(MH='N-Methylaspartate', MS='An amino acid that, as the D-isomer, is the defining agonist for the NMDA receptor subtype of glutamate receptors (RECEPTORS, NMDA).'),
  'D016203': NtO(MH='CDC2 Protein Kinase', MS='Phosphoprotein with protein kinase activity that functions in the G2/M phase transition of the CELL CYCLE. It is the catalytic subunit of the MATURATION-PROMOTING FACTOR and complexes with both CYCLIN A and CYCLIN B in mammalian cells. The maximal activity of cyclin-dependent kinase 1 is achieved when it is fully dephosphorylated.'),
  'D016205': NtO(MH='Sigmodontinae', MS='A subfamily of the family MURIDAE comprised of 69 genera. New World mice and rats are included in this subfamily.'),
  'D016206': NtO(MH='Databases, Bibliographic', MS='Extensive collections, reputedly complete, of references and citations to books, articles, publications, etc., generally on a single subject or specialized subject area. Databases can operate through automated files, libraries, or computer disks. The concept should be differentiated from DATABASES, FACTUAL which is used for collections of data and facts apart from bibliographic references to them.'),
  'D016207': NtO(MH='Cytokines', MS='Non-antibody proteins secreted by inflammatory leukocytes and some non-leukocytic cells, that act as intercellular mediators. They differ from classical hormones in that they are produced by a number of tissue or cell types rather than by specialized glands. They generally act locally in a paracrine or autocrine rather than endocrine manner.'),
  'D016208': NtO(MH='Databases, Factual', MS='Extensive collections, reputedly complete, of facts and data garnered from material of a specialized subject area and made available for analysis and application. The collection can be automated by various contemporary methods for retrieval. The concept should be differentiated from DATABASES, BIBLIOGRAPHIC which is restricted to collections of bibliographic references.'),
  'D016209': NtO(MH='Interleukin-8', MS='A member of the CXC chemokine family that plays a role in the regulation of the acute inflammatory response. It is secreted by variety of cell types and induces CHEMOTAXIS of NEUTROPHILS and other inflammatory cells.'),
  'D016210': NtO(MH='Methacholine Chloride', MS='A quaternary ammonium parasympathomimetic agent with the muscarinic actions of ACETYLCHOLINE. It is hydrolyzed by ACETYLCHOLINESTERASE at a considerably slower rate than ACETYLCHOLINE and is more resistant to hydrolysis by nonspecific CHOLINESTERASES so that its actions are more prolonged. It is used as a parasympathomimetic bronchoconstrictor agent and as a diagnostic aid for bronchial asthma. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1116)'),
  'D016211': NtO(MH='Transforming Growth Factor alpha', MS='An EPIDERMAL GROWTH FACTOR related protein that is found in a variety of tissues including EPITHELIUM, and maternal DECIDUA. It is synthesized as a transmembrane protein which can be cleaved to release a soluble active form which binds to the EGF RECEPTOR.'),
  'D016212': NtO(MH='Transforming Growth Factor beta', MS='A factor synthesized in a wide variety of tissues. It acts synergistically with TGF-alpha in inducing phenotypic transformation and can also act as a negative autocrine growth factor. TGF-beta has a potential role in embryonal development, cellular differentiation, hormone secretion, and immune function. TGF-beta is found mostly as homodimer forms of separate gene products TGF-beta1, TGF-beta2 or TGF-beta3. Heterodimers composed of TGF-beta1 and 2 (TGF-beta1.2) or of TGF-beta2 and 3 (TGF-beta2.3) have been isolated. The TGF-beta proteins are synthesized as precursor proteins.'),
  'D016213': NtO(MH='Cyclins', MS='A large family of regulatory proteins that function as accessory subunits to a variety of CYCLIN-DEPENDENT KINASES. They generally function as ENZYME ACTIVATORS that drive the CELL CYCLE through transitions between phases. A subset of cyclins may also function as transcriptional regulators.'),
  'D016215': NtO(MH='Macrophage-Activating Factors', MS='Factors secreted by stimulated lymphocytes that prime macrophages to become nonspecifically cytotoxic to tumors. They also modulate the expression of macrophage cell surface Ia antigens. One MAF is INTERFERON-GAMMA. Other factors antigenically distinct from IFN-gamma have also been identified.'),
  'D016216': NtO(MH='Ultrasonography, Prenatal', MS='The visualization of tissues during pregnancy through recording of the echoes of ultrasonic waves directed into the body. The procedure may be applied with reference to the mother or the fetus and with reference to organs or the detection of maternal or fetal disease.'),
  'D016217': NtO(MH='Ultrasonography, Mammary', MS='Use of ultrasound for imaging the breast. The most frequent application is the diagnosis of neoplasms of the female breast.'),
  'D016219': NtO(MH='Immunotherapy, Adoptive', MS='Form of adoptive transfer where cells with antitumor activity are transferred to the tumor-bearing host in order to mediate tumor regression. The lymphoid cells commonly used are lymphokine-activated killer (LAK) cells and tumor-infiltrating lymphocytes (TIL). This is usually considered a form of passive immunotherapy. (From DeVita, et al., Cancer, 1993, pp.305-7, 314)'),
  'D016220': NtO(MH='Fibroblast Growth Factor 1', MS='A 17-kDa single-chain polypeptide growth factor that plays a significant role in the process of WOUND HEALING and is a potent inducer of PHYSIOLOGIC ANGIOGENESIS. It binds to HEPARIN, which potentiates its biological activity and protects it from proteolysis. The growth factor is an extremely potent inducer of DNA synthesis in a variety of cell types from mesoderm and neuroectoderm lineages, and also has chemotactic and mitogenic activities. It was originally named acidic fibroblast growth factor based upon its chemical properties and to distinguish it from basic fibroblast growth factor (FIBROBLAST GROWTH FACTOR 2).'),
  'D016221': NtO(MH='Festschrift', MS='Work consisting of a collection of essays or other writings contributed by students, teachers, colleagues, and associates to honor a person or institution, usually on the occasion of an anniversary celebration or other event of importance.'),
  'D016222': NtO(MH='Fibroblast Growth Factor 2', MS='A single-chain polypeptide growth factor that plays a significant role in the process of WOUND HEALING and is a potent inducer of PHYSIOLOGIC ANGIOGENESIS. Several different forms of the human protein exist ranging from 18-24 kDa in size due to the use of alternative start sites within the fgf-2 gene. It has a 55 percent amino acid residue identity to FIBROBLAST GROWTH FACTOR 1 and has potent heparin-binding activity. The growth factor is an extremely potent inducer of DNA synthesis in a variety of cell types from mesoderm and neuroectoderm lineages. It was originally named basic fibroblast growth factor based upon its chemical properties and to distinguish it from acidic fibroblast growth factor (FIBROBLAST GROWTH FACTOR 1).'),
  'D016223': NtO(MH='Advance Directives', MS='Declarations by patients, made in advance of a situation in which they may be incompetent to decide about their own care, stating their treatment preferences or authorizing a third party to make decisions for them. (Bioethics Thesaurus)'),
  'D016224': NtO(MH='Living Wills', MS='Written, witnessed declarations in which persons request that if they become disabled beyond reasonable expectation of recovery, they be allowed to die rather than be kept alive by extraordinary means. (Bioethics Thesaurus)'),
  'D016226': NtO(MH='Phosphorus, Dietary', MS='Phosphorus used in foods or obtained from food. This element is a major intracellular component which plays an important role in many biochemical pathways relating to normal physiological functions. High concentrations of dietary phosphorus can cause nephrocalcinosis which is associated with impaired kidney function. Low concentrations of dietary phosphorus cause an increase in calcitriol in the blood and osteoporosis.'),
  'D016227': NtO(MH='Benzoquinones', MS='Benzene rings which contain two ketone moieties in any position. They can be substituted in any position except at the ketone groups.'),
  'D016228': NtO(MH='Endothelial Growth Factors', MS='These growth factors are soluble mitogens secreted by a variety of organs. The factors are a mixture of two single chain polypeptides which have affinity to heparin. Their molecular weight are organ and species dependent. They have mitogenic and chemotactic effects and can stimulate endothelial cells to grow and synthesize DNA. The factors are related to both the basic and acidic FIBROBLAST GROWTH FACTORS but have different amino acid sequences.'),
  'D016229': NtO(MH='Amyloid beta-Peptides', MS="Peptides generated from AMYLOID BETA-PEPTIDES PRECURSOR. An amyloid fibrillar form of these peptides is the major component of amyloid plaques found in individuals with Alzheimer's disease and in aged individuals with trisomy 21 (DOWN SYNDROME). The peptide is found predominantly in the nervous system, but there have been reports of its presence in non-neural tissue."),
  'D016231': NtO(MH='Fluorescence Polarization Immunoassay', MS='Fluoroimmunoassay where detection of the hapten-antibody reaction is based on measurement of the increased polarization of fluorescence-labeled hapten when it is combined with antibody. The assay is very useful for the measurement of small haptenic antigens such as drugs at low concentrations.'),
  'D016232': NtO(MH='Endothelins', MS='21-Amino-acid peptides produced by vascular endothelial cells and functioning as potent vasoconstrictors. The endothelin family consists of three members, ENDOTHELIN-1; ENDOTHELIN-2; and ENDOTHELIN-3. All three peptides contain 21 amino acids, but vary in amino acid composition. The three peptides produce vasoconstrictor and pressor responses in various parts of the body. However, the quantitative profiles of the pharmacological activities are considerably different among the three isopeptides.'),
  'D016233': NtO(MH='Immunotherapy, Active', MS='Active immunization where vaccine is administered for therapeutic or preventive purposes. This can include administration of immunopotentiating agents such as BCG vaccine and Corynebacterium parvum as well as biological response modifiers such as interferons, interleukins, and colony-stimulating factors in order to directly stimulate the immune system.'),
  'D016234': NtO(MH='Kallikrein-Kinin System', MS='A system of metabolic interactions by products produced in the distal nephron of the KIDNEY. These products include KALLIKREIN; KININS; KININASE I; KININASE II; and ENKEPHALINASE. This system participates in the control of renal functions. It interacts with the RENIN-ANGIOTENSIN-ALDOSTERONE SYSTEM to regulate BLOOD PRESSURE, generation of PROSTAGLANDINS, release of VASOPRESSINS, and WATER-ELECTROLYTE BALANCE.'),
  'D016235': NtO(MH='American Speech-Language-Hearing Association', MS='A professional society concerned with the diagnosis, prevention, treatment, and remediation of speech, language, and hearing disorders.'),
  'D016236': NtO(MH='Genes, fms', MS='Family of genes originally isolated from the Susan McDonough strain of feline sarcoma virus (SARCOMA VIRUSES, FELINE). The proto-oncogene fms (c-fms) codes for the MCSF receptor (RECEPTOR, MACROPHAGE COLONY-STIMULATING FACTOR). The oncogene fms (v-fms) codes for ONCOGENE PROTEIN GP140(V-FMS) which is a mutated form of the MCSF. The human c-fms gene is located between 5q33.2 and 5q33.3.'),
  'D016238': NtO(MH='Cytapheresis', MS='Separation of one or more kinds of cells from whole blood with the return of other blood cell constituents to the patient or donor. This is accomplished with an instrument that uses centrifugation to separate the cells into different layers based on the differences in cell density (displacement) or drag coefficients in a current (elutriation). The procedure is commonly used in adoptive transfer to isolate NK cells, lymphocytes, or monocytes.'),
  'D016239': NtO(MH='MEDLINE', MS="The premier bibliographic database of the NATIONAL LIBRARY OF MEDICINE. MEDLINE\xc2\xae (MEDLARS Online) is the primary subset of PUBMED and can be searched on NLM's Web site in PubMed or the NLM Gateway. MEDLINE references are indexed with MEDICAL SUBJECT HEADINGS (MeSH)."),
  'D016240': NtO(MH='Grateful Med', MS='A microcomputer-based software package providing a user-friendly interface to the MEDLARS system of the National Library of Medicine.'),
  'D016241': NtO(MH='Child of Impaired Parents', MS='Child with one or more parents afflicted by a physical or mental disorder.'),
  'D016242': NtO(MH='Library Automation', MS='The use of automatic machines or processing devices in libraries. The automation may be applied to library administrative activities, office procedures, and delivery of library services to users.'),
  'D016243': NtO(MH='Library Collection Development', MS='Development of a library collection, including the determination and coordination of selection policy, assessment of needs of users and potential users, collection use studies, collection evaluation, identification of collection needs, selection of materials, planning for resource sharing, collection maintenance and weeding, and budgeting.'),
  'D016244': NtO(MH="Guanosine 5'-O-(3-Thiotriphosphate)", MS="Guanosine 5'-(trihydrogen diphosphate), monoanhydride with phosphorothioic acid. A stable GTP analog which enjoys a variety of physiological actions such as stimulation of guanine nucleotide-binding proteins, phosphoinositide hydrolysis, cyclic AMP accumulation, and activation of specific proto-oncogenes."),
  'D016245': NtO(MH='Librarians', MS="Specialists in the management of a library or the services rendered by a library, bringing professional skills to administration, organization of material and personnel, interpretation of bibliothecal rules, the development and maintenance of the library's collection, and the provision of information services."),
  'D016246': NtO(MH='Lymphocytes, Tumor-Infiltrating', MS='Lymphocytes that show specificity for autologous tumor cells. Ex vivo isolation and culturing of TIL with interleukin-2, followed by reinfusion into the patient, is one form of adoptive immunotherapy of cancer.'),
  'D016247': NtO(MH='Information Storage and Retrieval', MS='Organized activities related to the storage, location, search, and retrieval of information.'),
  'D016248': NtO(MH='Computer Storage Devices', MS='Devices capable of receiving data, retaining data for an indefinite or finite period of time, and supplying data upon demand.'),
  'D016249': NtO(MH='Optical Storage Devices', MS='A computer disk read by a laser beam, containing data prerecorded by a vendor. The buyer cannot enter or modify data in any way but the advantages lie in the speed of accessibility, relative immunity to damage, and relatively low cost of purchase.'),
  'D016250': NtO(MH='Compact Disks', MS='Computer disks storing data with a maximum reduction of space and bandwidth. The compact size reduces cost of transmission and storage.'),
  'D016251': NtO(MH='CD-ROM', MS='An optical disk storage system for computers on which data can be read or from which data can be retrieved but not entered or modified. A CD-ROM unit is almost identical to the compact disk playback device for home use.'),
  'D016252': NtO(MH='Microscopy, Scanning Tunneling', MS='A type of scanning probe microscopy in which a very sharp conducting needle is swept just a few angstroms above the surface of a sample. The tiny tunneling current that flows between the sample and the needle tip is measured, and from this are produced three-dimensional topographs. Due to the poor electron conductivity of most biological samples, thin metal coatings are deposited on the sample.'),
  'D016253': NtO(MH='Microscopy, Immunoelectron', MS='Microscopy in which the samples are first stained immunocytochemically and then examined using an electron microscope. Immunoelectron microscopy is used extensively in diagnostic virology as part of very sensitive immunoassays.'),
  'D016254': NtO(MH='Mutagenesis, Insertional', MS='Mutagenesis where the mutation is caused by the introduction of foreign DNA sequences into a gene or extragenic sequence. This may occur spontaneously in vivo or be experimentally induced in vivo or in vitro. Proviral DNA insertions into or adjacent to a cellular proto-oncogene can interrupt GENETIC TRANSLATION of the coding sequences or interfere with recognition of regulatory elements and cause unregulated expression of the proto-oncogene resulting in tumor formation.'),
  'D016255': NtO(MH='Anistreplase', MS='An acylated inactive complex of streptokinase and human lysine-plasminogen. After injection, the acyl group is slowly hydrolyzed, producing an activator that converts plasminogen to plasmin, thereby initiating fibrinolysis. Its half-life is about 90 minutes compared to 5 minutes for TPA; (TISSUE PLASMINOGEN ACTIVATOR); 16 minutes for UROKINASE-TYPE PLASMINOGEN ACTIVATOR and 23 minutes for STREPTOKINASE. If treatment is initiated within 3 hours of onset of symptoms for acute myocardial infarction, the drug preserves myocardial tissue and left ventricular function and increases coronary artery patency. Bleeding complications are similar to other thrombolytic agents.'),
  'D016256': NtO(MH='Lewis X Antigen', MS='A trisaccharide antigen expressed on glycolipids and many cell-surface glycoproteins. In the blood the antigen is found on the surface of NEUTROPHILS; EOSINOPHILS; and MONOCYTES. In addition, Lewis X antigen is a stage-specific embryonic antigen.'),
  'D016257': NtO(MH='Fura-2', MS='A fluorescent calcium chelating agent which is used to study intracellular calcium in tissues.'),
  'D016258': NtO(MH='Genes, mos', MS='Retrovirus-associated DNA sequences (mos) originally isolated from the Moloney murine sarcoma virus (Mo-MSV). The proto-oncogene mos (c-mos) codes for a protein which is a member of the serine kinase family. There is no evidence as yet that human c-mos can become transformed or has a role in human cancer. However, in mice, activation can occur when the retrovirus-like intracisternal A-particle inserts itself near the c-mos sequence. The human c-mos gene is located at 8q22 on the long arm of chromosome 8.'),
  'D016259': NtO(MH='Genes, myc', MS='Family of retrovirus-associated DNA sequences (myc) originally isolated from an avian myelocytomatosis virus. The proto-oncogene myc (c-myc) codes for a nuclear protein which is involved in nucleic acid metabolism and in mediating the cellular response to growth factors. Truncation of the first exon, which appears to regulate c-myc expression, is crucial for tumorigenicity. The human c-myc gene is located at 8q24 on the long arm of chromosome 8.'),
  'D016260': NtO(MH='Monocytes, Activated Killer', MS='Monocytes made cytotoxic by IN VITRO incubation with CYTOKINES, especially INTERFERON-GAMMA. The cells are used for ADOPTIVE IMMUNOTHERAPY in cancer patients.'),
  'D016262': NtO(MH='Postpoliomyelitis Syndrome', MS='A syndrome characterized by new neuromuscular symptoms that occur at least 15 years after clinical stability has been attained in patients with a prior history of symptomatic poliomyelitis. Clinical features include new muscular weakness and atrophy of the limbs, bulbar innervated musculature, and muscles of respiration, combined with excessive fatigue, joint pain, and reduced stamina. The process is marked by slow progression and periods of stabilization. (From Ann NY Acad Sci 1995 May 25;753:68-80)'),
  'D016263': NtO(MH='AIDS-Associated Nephropathy', MS='Renal syndrome in human immunodeficiency virus-infected patients characterized by nephrotic syndrome, severe proteinuria, focal and segmental glomerulosclerosis with distinctive tubular and interstitial changes, enlarged kidneys, and peculiar tubuloreticular structures. The syndrome is distinct from heroin-associated nephropathy as well as other forms of kidney disease seen in HIV-infected patients.'),
  'D016264': NtO(MH='Dextran Sulfate', MS='Long-chain polymer of glucose containing 17-20% sulfur. It has been used as an anticoagulant and also has been shown to inhibit the binding of HIV-1 to CD4-POSITIVE T-LYMPHOCYTES. It is commonly used as both an experimental and clinical laboratory reagent and has been investigated for use as an antiviral agent, in the treatment of hypolipidemia, and for the prevention of free radical damage, among other applications.'),
  'D016265': NtO(MH='Conflict of Interest', MS="A situation in which an individual might benefit personally from official or professional actions. It includes a conflict between a person's private interests and official responsibilities in a position of trust. The term is not restricted to government officials. The concept refers both to actual conflict of interest and the appearance or perception of conflict."),
  'D016266': NtO(MH='Food Parasitology', MS='The presence of parasites in food and food products. For the presence of bacteria, viruses, and fungi in food, FOOD MICROBIOLOGY is available.'),
  'D016267': NtO(MH='External Fixators', MS='External devices which hold wires or pins that are placed through one or both cortices of bone in order to hold the position of a fracture in proper alignment. These devices allow easy access to wounds, adjustment during the course of healing, and more functional use of the limbs involved.'),
  'D016268': NtO(MH='Internal Fixators', MS='Internal devices used in osteosynthesis to hold the position of the fracture in proper alignment. By applying the principles of biomedical engineering, the surgeon uses metal plates, nails, rods, etc., for the correction of skeletal defects.'),
  'D016269': NtO(MH='Milk Hypersensitivity', MS="Allergic reaction to milk (usually cow's milk) or milk products. MILK HYPERSENSITIVITY should be differentiated from LACTOSE INTOLERANCE, an intolerance to milk as a result of congenital deficiency of lactase."),
  'D016270': NtO(MH='United States Agency for Healthcare Research and Quality', MS='An agency of the PUBLIC HEALTH SERVICE established in 1990 to "provide indexing, abstracting, translating, publishing, and other services leading to a more effective and timely dissemination of information on research, demonstration projects, and evaluations with respect to health care to public and private entities and individuals engaged in the improvement of health care delivery..." It supersedes the National Center for Health Services Research. The United States Agency for Health Care Policy and Research was renamed Agency for Healthcare Research and Quality (AHRQ) under the Healthcare Research and Quality Act of 1999.'),
  'D016271': NtO(MH='Proto-Oncogene Proteins c-myc', MS='Basic helix-loop-helix transcription factors encoded by the c-myc genes. They are normally involved in nucleic acid metabolism and in mediating the cellular response to growth factors. Elevated and deregulated (constitutive) expression of c-myc proteins can cause tumorigenesis.'),
  'D016272': NtO(MH='Occupational Health', MS='The promotion and maintenance of physical and mental health in the work environment.'),
  'D016273': NtO(MH='Occupational Exposure', MS="The exposure to potentially harmful chemical, physical, or biological agents that occurs as a result of one's occupation."),
  'D016274': NtO(MH='Oncogene Protein p55(v-myc)', MS='Transforming protein coded by myc oncogenes. The v-myc protein has been found in several replication-defective avian retrovirus isolates which induce a broad spectrum of malignancies.'),
  'D016275': NtO(MH='Atrial Function', MS='The hemodynamic and electrophysiological action of the HEART ATRIA.'),
  'D016276': NtO(MH='Ventricular Function', MS='The hemodynamic and electrophysiological action of the HEART VENTRICLES.'),
  'D016277': NtO(MH='Ventricular Function, Left', MS='The hemodynamic and electrophysiological action of the left HEART VENTRICLE. Its measurement is an important aspect of the clinical evaluation of patients with heart disease to determine the effects of the disease on cardiac performance.'),
  'D016278': NtO(MH='Ventricular Function, Right', MS='The hemodynamic and electrophysiological action of the right HEART VENTRICLE.'),
  'D016279': NtO(MH='Atrial Function, Left', MS='The hemodynamic and electrophysiological action of the LEFT ATRIUM.'),
  'D016280': NtO(MH='Atrial Function, Right', MS='The hemodynamic and electrophysiological action of the RIGHT ATRIUM.'),
  'D016281': NtO(MH='Oncogene Protein gp140(v-fms)', MS='Transforming glycoprotein coded by the fms oncogene from the Susan McDonough strain of feline sarcoma virus (SM-FeSV). The oncogene protein v-fms lacks sequences, which, in the highly homologous proto-oncogene protein c-fms (CSF-1 receptor), normally serve to regulate its tyrosine kinase activity. The missing sequences in v-fms mimic the effect of ligand and lead to constitutive cell growth. The protein gp120(v-fms) is post-translationally modified to generate gp140(v-fms).'),
  'D016282': NtO(MH='Aspartic Acid Endopeptidases', MS='A sub-subclass of endopeptidases that depend on an ASPARTIC ACID residue for their activity.'),
  'D016283': NtO(MH='Proto-Oncogene Proteins p21(ras)', MS='Cellular proteins encoded by the H-ras, K-ras and N-ras genes. The proteins have GTPase activity and are involved in signal transduction as monomeric GTP-binding proteins. Elevated levels of p21 c-ras have been associated with neoplasia. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D016284': NtO(MH='Technetium Tc 99m Pentetate', MS='A technetium imaging agent used in renal scintigraphy, computed tomography, lung ventilation imaging, gastrointestinal scintigraphy, and many other procedures which employ radionuclide imaging agents.'),
  'D016285': NtO(MH='Iloprost', MS='An eicosanoid, derived from the cyclooxygenase pathway of arachidonic acid metabolism. It is a stable and synthetic analog of EPOPROSTENOL, but with a longer half-life than the parent compound. Its actions are similar to prostacyclin. Iloprost produces vasodilation and inhibits platelet aggregation.'),
  'D016287': NtO(MH='Computer Peripherals', MS='Various units or machines that operate in combination or in conjunction with a computer but are not physically part of it. Peripheral devices typically display computer data, store data from the computer and return the data to the computer on demand, prepare data for human use, or acquire data from a source and convert it to a form usable by a computer. (Computer Dictionary, 4th ed.)'),
  'D016288': NtO(MH='Computer Terminals', MS='Input/output devices designed to receive data in an environment associated with the job to be performed, and capable of transmitting entries to, and obtaining output from, the system of which it is a part. (Computer Dictionary, 4th ed.)'),
  'D016291': NtO(MH='Dizocilpine Maleate', MS='A potent noncompetitive antagonist of the NMDA receptor (RECEPTORS, N-METHYL-D-ASPARTATE) used mainly as a research tool. The drug has been considered for the wide variety of neurodegenerative conditions or disorders in which NMDA receptors may play an important role. Its use has been primarily limited to animal and tissue experiments because of its psychotropic effects.'),
  'D016292': NtO(MH='Conscious Sedation', MS='A drug-induced depression of consciousness during which patients respond purposefully to verbal commands, either alone or accompanied by light tactile stimulation. No interventions are required to maintain a patent airway. (From: American Society of Anesthesiologists Practice Guidelines)'),
  'D016293': NtO(MH='Moricizine', MS='An antiarrhythmia agent used primarily for ventricular rhythm disturbances.'),
  'D016294': NtO(MH='Models, Nursing', MS='Theoretical models simulating behavior or activities in nursing, including nursing care, management and economics, theory, assessment, research, and education. Some examples of these models include Orem Self-Care Model, Roy Adaptation Model, and Rogers Life Process Model.'),
  'D016296': NtO(MH='Mutagenesis', MS='Process of generating a genetic MUTATION. It may occur spontaneously or be induced by MUTAGENS.'),
  'D016297': NtO(MH='Mutagenesis, Site-Directed', MS='Genetically engineered MUTAGENESIS at a specific site in the DNA molecule that introduces a base substitution, or an insertion or deletion.'),
  'D016298': NtO(MH='Hematopoietic Cell Growth Factors', MS='These growth factors comprise a family of hematopoietic regulators with biological specificities defined by their ability to support proliferation and differentiation of blood cells of different lineages. ERYTHROPOIETIN and the COLONY-STIMULATING FACTORS belong to this family. Some of these factors have been studied and used in the treatment of chemotherapy-induced neutropenia, myelodysplastic syndromes, and bone marrow failure syndromes.'),
  'D016299': NtO(MH='Dental Implantation, Subperiosteal', MS='The grafting or inserting of an appliance designed to fit over the surface of the mandible or the maxilla, beneath the specialized connective tissue that covers the bone (periosteum).'),
  'D016300': NtO(MH='Radiography, Bitewing', MS='Technique involving the passage of X-rays through oral structures to create a film record while a central tab or wing of dental X-ray film is being held between upper and lower teeth.'),
  'D016301': NtO(MH='Alveolar Bone Loss', MS='Resorption or wasting of the tooth-supporting bone (ALVEOLAR PROCESS) in the MAXILLA or MANDIBLE.'),
  'D016302': NtO(MH='Centric Relation', MS='The location of the maxillary and the mandibular condyles when they are in their most posterior and superior positions in their fossae of the temporomandibular joint.'),
  'D016303': NtO(MH='Clinical Pharmacy Information Systems', MS='Information systems, usually computer-assisted, designed to store, manipulate, and retrieve information for planning, organizing, directing, and controlling administrative activities associated with the provision and utilization of clinical pharmacy services.'),
  'D016304': NtO(MH='Hospitals, Private', MS='A class of hospitals that includes profit or not-for-profit hospitals that are controlled by a legal entity other than a government agency. (Hospital Administration Terminology, AHA, 2d ed)'),
  'D016305': NtO(MH='Thymopentin', MS='Synthetic pentapeptide corresponding to the amino acids 32-36 of thymopoietin and exhibiting the full biological activity of the natural hormone. It is an immunomodulator which has been studied for possible use in the treatment of rheumatoid arthritis, AIDS, and other primary immunodeficiencies.'),
  'D016306': NtO(MH='Vena Cava Filters', MS='Mechanical devices inserted in the inferior vena cava that prevent the migration of blood clots from deep venous thrombosis of the leg.'),
  'D016308': NtO(MH='Enkephalin, Leucine-2-Alanine', MS='A delta-selective opioid (ANALGESICS, OPIOID). It can cause transient depression of mean arterial blood pressure and heart rate.'),
  'D016311': NtO(MH='Outliers, DRG', MS='In health care reimbursement, especially in the prospective payment system, those patients who require an unusually long hospital stay or whose stay generates unusually high costs.'),
  'D016312': NtO(MH='Treatment Refusal', MS='Patient or client refusal of or resistance to medical, psychological, or psychiatric treatment. (APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D016313': NtO(MH='Genes, abl', MS='Retrovirus-associated DNA sequences (abl) originally isolated from the Abelson murine leukemia virus (Ab-MuLV). The proto-oncogene abl (c-abl) codes for a protein that is a member of the tyrosine kinase family. The human c-abl gene is located at 9q34.1 on the long arm of chromosome 9. It is activated by translocation to bcr on chromosome 22 in chronic myelogenous leukemia.'),
  'D016314': NtO(MH='Ethylketocyclazocine', MS='A kappa opioid receptor agonist. The compound has analgesic action and shows positive inotropic effects on the electrically stimulated left atrium. It also affects various types of behavior in mammals such as locomotion, rearing, and grooming.'),
  'D016315': NtO(MH='Proto-Oncogene Proteins c-abl', MS='Non-receptor tyrosine kinases encoded by the C-ABL GENES. They are distributed in both the cytoplasm and the nucleus. c-Abl plays a role in normal HEMATOPOIESIS especially of the myeloid lineage. Oncogenic transformation of c-abl arises when specific N-terminal amino acids are deleted, releasing the kinase from negative regulation.'),
  'D016316': NtO(MH='Guanfacine', MS='A centrally acting antihypertensive agent with specificity towards ADRENERGIC ALPHA-2 RECEPTORS.'),
  'D016318': NtO(MH='Quisqualic Acid', MS='An agonist at two subsets of excitatory amino acid receptors, ionotropic receptors that directly control membrane channels and metabotropic receptors that indirectly mediate calcium mobilization from intracellular stores. The compound is obtained from the seeds and fruit of Quisqualis chinensis.'),
  'D016319': NtO(MH='Corrosion Casting', MS='A tissue preparation technique that involves the injecting of plastic (acrylates) into blood vessels or other hollow viscera and treating the tissue with a caustic substance. This results in a negative copy or a solid replica of the enclosed space of the tissue that is ready for viewing under a scanning electron microscope.'),
  'D016320': NtO(MH='Substance Abuse Treatment Centers', MS='Health facilities providing therapy and/or rehabilitation for substance-dependent individuals. Methadone distribution centers are included.'),
  'D016321': NtO(MH='Dental Care for Aged', MS='The giving of attention to the special dental needs of the elderly for proper maintenance or treatment. The dental care may include the services provided by dental specialists.'),
  'D016322': NtO(MH='HIV Enhancer', MS='Cis-acting regulatory sequences in the HIV long terminal repeat (LTR) which play a major role in induction or augmentation of HIV gene expression in response to environmental stimuli such as mitogens, phorbol esters, or other viruses. The HIV enhancer is the binding site for many cellular transcription factors including the nuclear factor NF-kappa B.'),
  'D016324': NtO(MH='Sequence Tagged Sites', MS='Short tracts of DNA sequence that are used as landmarks in GENOME mapping. In most instances, 200 to 500 base pairs of sequence define a Sequence Tagged Site (STS) that is operationally unique in the human genome (i.e., can be specifically detected by the polymerase chain reaction in the presence of all other genomic sequences). The overwhelming advantage of STSs over mapping landmarks defined in other ways is that the means of testing for the presence of a particular STS can be completely described as information in a database.'),
  'D016325': NtO(MH='HIV Long Terminal Repeat', MS='Regulatory sequences important for viral replication that are located on each end of the HIV genome. The LTR includes the HIV ENHANCER, promoter, and other sequences. Specific regions in the LTR include the negative regulatory element (NRE), NF-kappa B binding sites , Sp1 binding sites, TATA BOX, and trans-acting responsive element (TAR). The binding of both cellular and viral proteins to these regions regulates HIV transcription.'),
  'D016326': NtO(MH='Extracellular Matrix Proteins', MS='Macromolecular organic compounds that contain carbon, hydrogen, oxygen, nitrogen, and usually, sulfur. These macromolecules (proteins) form an intricate meshwork in which cells are embedded to construct tissues. Variations in the relative types of macromolecules and their organization determine the type of extracellular matrix, each adapted to the functional requirements of the tissue. The two main classes of macromolecules that form the extracellular matrix are: glycosaminoglycans, usually linked to proteins (proteoglycans), and fibrous proteins (e.g., COLLAGEN; ELASTIN; FIBRONECTINS; and LAMININ).'),
  'D016328': NtO(MH='NF-kappa B', MS='Ubiquitous, inducible, nuclear transcriptional activator that binds to enhancer elements in many different cell types and is activated by pathogenic stimuli. The NF-kappa B complex is a heterodimer composed of two DNA-binding subunits: NF-kappa B1 and relA.'),
  'D016329': NtO(MH='Sp1 Transcription Factor', MS='Promoter-specific RNA polymerase II transcription factor that binds to the GC box, one of the upstream promoter elements, in mammalian cells. The binding of Sp1 is necessary for the initiation of transcription in the promoters of a variety of cellular and viral GENES.'),
  'D016330': NtO(MH='Frail Elderly', MS='Older adults or aged individuals who are lacking in general strength and are unusually susceptible to disease or to other infirmity.'),
  'D016331': NtO(MH='Parenteral Nutrition, Home', MS='The at-home administering of nutrients for assimilation and utilization by a patient who cannot maintain adequate nutrition by enteral feeding alone. Nutrients are administered via a route other than the alimentary canal (e.g., intravenously, subcutaneously).'),
  'D016332': NtO(MH='Fetal Tissue Transplantation', MS='Transference of fetal tissue between individuals of the same species or between individuals of different species.'),
  'D016333': NtO(MH='HIV Protease', MS='Enzyme of the human immunodeficiency virus that is required for post-translational cleavage of gag and gag-pol precursor polyproteins into functional products needed for viral assembly. HIV protease is an aspartic protease encoded by the amino terminus of the pol gene.'),
  'D016334': NtO(MH='Genes, vpr', MS='DNA sequences that form the coding region for a trans-activator protein that specifies rapid growth in human immunodeficiency virus (HIV). vpr is short for viral protein R, where R is undefined.'),
  'D016335': NtO(MH='Zinc Fingers', MS='Motifs in DNA- and RNA-binding proteins whose amino acids are folded into a single structural unit around a zinc atom. In the classic zinc finger, one zinc atom is bound to two cysteines and two histidines. In between the cysteines and histidines are 12 residues which form a DNA binding fingertip. By variations in the composition of the sequences in the fingertip and the number and spacing of tandem repeats of the motif, zinc fingers can form a large number of different sequence specific binding sites.'),
  'D016336': NtO(MH='Gene Products, vpr', MS='Trans-acting proteins which accelerate retroviral virus replication. The vpr proteins act in trans to increase the levels of specified proteins. vpr is short for viral protein R, where R is undefined.'),
  'D016337': NtO(MH='RNA, Catalytic', MS='RNA that has catalytic activity. The catalytic RNA sequence folds to form a complex surface that can function as an enzyme in reactions with itself and other molecules. It may function even in the absence of protein. There are numerous examples of RNA species that are acted upon by catalytic RNA, however the scope of this enzyme class is not limited to a particular type of substrate.'),
  'D016338': NtO(MH='Genes, vpu', MS='DNA sequences that form the coding region for the HIV-1 regulatory protein vpu (viral protein U) that greatly increases the export of virus particles from infected cells. The vpu genes are not present in HIV-2 or SIMIAN IMMUNODEFICIENCY VIRUS.'),
  'D016339': NtO(MH='Spectrometry, Mass, Fast Atom Bombardment', MS='A mass spectrometric technique that is used for the analysis of a wide range of biomolecules, such as glycoalkaloids, glycoproteins, polysaccharides, and peptides. Positive and negative fast atom bombardment spectra are recorded on a mass spectrometer fitted with an atom gun with xenon as the customary beam. The mass spectra obtained contain molecular weight recognition as well as sequence information.'),
  'D016341': NtO(MH='Genes, vif', MS='DNA sequences that form the coding region for the vif (virion infectivity factor) protein that is important for the generation of infectious virions in human immunodeficiency virus (HIV). The former name of this gene was sor (short open reading frame).'),
  'D016342': NtO(MH='Angioplasty, Laser', MS='A technique utilizing a laser coupled to a catheter which is used in the dilatation of occluded blood vessels. This includes laser thermal angioplasty where the laser energy heats up a metal tip, and direct laser angioplasty where the laser energy directly ablates the occlusion. One form of the latter approach uses an EXCIMER LASER which creates microscopically precise cuts without thermal injury. When laser angioplasty is performed in combination with balloon angioplasty it is called laser-assisted balloon angioplasty (ANGIOPLASTY, BALLOON, LASER-ASSISTED).'),
  'D016343': NtO(MH='Monitoring, Intraoperative', MS='The constant checking on the state or condition of a patient during the course of a surgical operation (e.g., checking of vital signs).'),
  'D016344': NtO(MH='Medicare Part A', MS='The compulsory portion of Medicare that is known as the Hospital Insurance Program. All persons 65 years and older who are entitled to benefits under the Old Age, Survivors, Disability and Health Insurance Program or railroad retirement, persons under the age of 65 who have been eligible for disability for more than two years, and insured workers (and their dependents) requiring renal dialysis or kidney transplantation are automatically enrolled in Medicare Part A.'),
  'D016345': NtO(MH='Medicare Part B', MS="The voluntary portion of Medicare, known as the Supplementary Medical Insurance (SMI) Program, that includes physician's services, home health care, medical services, outpatient hospital services, and laboratory, pathology, and radiology services. All persons entitled to Medicare Part A may enroll in Medicare Part B on a monthly premium basis."),
  'D016346': NtO(MH='Gene Products, vif', MS='Retrovirally encoded accessary proteins that play an essential role VIRUS REPLICATION. They are found in the cytoplasm of host cells and associate with a variety of host cell proteins. Vif stands for "virion infectivity factor".'),
  'D016347': NtO(MH='Medical Records Systems, Computerized', MS="Computer-based systems for input, storage, display, retrieval, and printing of information contained in a patient's medical record."),
  'D016348': NtO(MH='Osseointegration', MS='The growth action of bone tissue as it assimilates surgically implanted devices or prostheses to be used as either replacement parts (e.g., hip) or as anchors (e.g., endosseous dental implants).'),
  'D016350': NtO(MH='Leucine Zippers', MS='DNA-binding motifs formed from two alpha-helixes which intertwine for about eight turns into a coiled coil and then bifurcate to form Y shaped structures. Leucines occurring in heptad repeats end up on the same sides of the helixes and are adjacent to each other in the stem of the Y (the "zipper" region). The DNA-binding residues are located in the bifurcated region of the Y.'),
  'D016352': NtO(MH='Group Homes', MS='Housing for groups of patients, children, or others who need or desire emotional or physical support. They are usually established as planned, single housekeeping units in residential dwellings that provide care and supervision for small groups of residents, who, although unrelated, live together as a family.'),
  'D016353': NtO(MH='Self-Examination', MS="The inspection of one's own body, usually for signs of disease (e.g., BREAST SELF-EXAMINATION, testicular self-examination)."),
  'D016354': NtO(MH='Rhodobacter capsulatus', MS='Non-pathogenic ovoid to rod-shaped bacteria that are widely distributed and found in fresh water as well as marine and hypersaline habitats.'),
  'D016355': NtO(MH='Genes, pX', MS='DNA sequences that form the coding region for at least three proteins which regulate the expression of HUMAN T-LYMPHOTROPIC VIRUS 1 and HUMAN T-LYMPHOTROPIC VIRUS 2. The proteins are p21(x), p27(rex), and p40(tax). The tax (trans-activator x) and rex (regulator x) genes are part of pX but are in overlapping reading frames. X was the original designation for the sequences or region (at that time of unknown function) in the long open reading frame (lor) which is now called pX.'),
  'D016356': NtO(MH='Gene Products, tax', MS='Transcriptional trans-acting proteins of the promoter elements found in the long terminal repeats (LTR) of HUMAN T-LYMPHOTROPIC VIRUS 1 and HUMAN T-LYMPHOTROPIC VIRUS 2. The tax (trans-activator x; x is undefined) proteins act by binding to enhancer elements in the LTR.'),
  'D016357': NtO(MH='Infection Control Practitioners', MS='Physicians or other qualified individuals responsible for implementing and overseeing the policies and procedures followed by a health care facility to reduce the risk of infection to patients and staff.'),
  'D016358': NtO(MH='Contact Tracing', MS='Identification of those persons (or animals) who have had such an association with an infected person, animal, or contaminated environment as to have had the opportunity to acquire the infection. Contact tracing is a generally accepted method for the control of sexually transmitted diseases.'),
  'D016360': NtO(MH='Clostridium difficile', MS='A common inhabitant of the colon flora in human infants and sometimes in adults. It produces a toxin that causes pseudomembranous enterocolitis (ENTEROCOLITIS, PSEUDOMEMBRANOUS) in patients receiving antibiotic therapy.'),
  'D016361': NtO(MH='Bankruptcy', MS='The state of legal insolvency with assets taken over by judicial process so that they may be distributed among creditors.'),
  'D016362': NtO(MH='Analgesia, Obstetrical', MS='The elimination of PAIN, without the loss of CONSCIOUSNESS, during OBSTETRIC LABOR; OBSTETRIC DELIVERY; or the POSTPARTUM PERIOD, usually through the administration of ANALGESICS.'),
  'D016363': NtO(MH='Antitrust Laws', MS='Those federal and state laws, and their enforcement, that protect trade and commerce from unlawful restraints and monopolies or unfair business practices.'),
  'D016364': NtO(MH='Reading Frames', MS="The three possible sequences of CODONS by which GENETIC TRANSLATION may occur from one nucleotide sequence. A segment of mRNA 5'AUCCGA3' could be translated as 5'AUC.. or 5'UCC.. or 5'CCG.., depending on the location of the START CODON."),
  'D016365': NtO(MH='Liability, Legal', MS='Accountability and responsibility to another, enforceable by civil or criminal sanctions.'),
  'D016366': NtO(MH='Open Reading Frames', MS='A sequence of successive nucleotide triplets that are read as CODONS specifying AMINO ACIDS and begin with an INITIATOR CODON and end with a stop codon (CODON, TERMINATOR).'),
  'D016367': NtO(MH='Baculoviridae', MS='Family of INSECT VIRUSES which contain polyhedron-shaped or ovocylindrical occlusion bodies. The genera include ALPHABACULOVIRUS; GAMMABACULOVIRUS; and DELTABACULOVIRUS (commonly known as NUCLEOPOLYHEDROVIRUSES) and BETABACULOVIRUS (commonly known as GRANULOVIRUSES). Baculovirus vectors are used for expression of foreign genes in insects and as BIOPESTICIDES for controlling insect populations.'),
  'D016368': NtO(MH='Frameshift Mutation', MS='A type of mutation in which a number of NUCLEOTIDES deleted from or inserted into a protein coding sequence is not divisible by three, thereby causing an alteration in the READING FRAMES of the entire coding sequence downstream of the mutation. These mutations may be induced by certain types of MUTAGENS or may occur spontaneously.'),
  'D016369': NtO(MH='Smear Layer', MS='Adherent debris produced when cutting the enamel or dentin in cavity preparation. It is about 1 micron thick and its composition reflects the underlying dentin, although different quantities and qualities of smear layer can be produced by the various instrumentation techniques. Its function is presumed to be protective, as it lowers dentin permeability. However, it masks the underlying dentin and interferes with attempts to bond dental material to the dentin.'),
  'D016370': NtO(MH='Gene Products, rex', MS='Post-transcriptional regulatory proteins required for the accumulation of mRNAs that encode the gag and env gene products in HUMAN T-LYMPHOTROPIC VIRUS 1 and HUMAN T-LYMPHOTROPIC VIRUS 2. The rex (regulator x; x is undefined) products act by binding to elements in the LONG TERMINAL REPEAT.'),
  'D016371': NtO(MH='Cranial Irradiation', MS='The exposure of the head to roentgen rays or other forms of radioactivity for therapeutic or preventive purposes.'),
  'D016372': NtO(MH='RNA, Antisense', MS='RNA molecules which hybridize to complementary sequences in either RNA or DNA altering the function of the latter. Endogenous antisense RNAs function as regulators of gene expression by a variety of mechanisms. Synthetic antisense RNAs are used to effect the functioning of specific genes for investigative or therapeutic purposes.'),
  'D016373': NtO(MH='DNA, Antisense', MS='DNA that is complementary to the sense strand. (The sense strand has the same sequence as the mRNA transcript. The antisense strand is the template for mRNA synthesis.) Synthetic antisense DNAs are used to hybridize to complementary sequences in target RNAs or DNAs to effect the functioning of specific genes for investigative or therapeutic purposes.'),
  'D016374': NtO(MH='Oncogene Proteins v-abl', MS='Transforming proteins encoded by the abl oncogenes. Oncogenic transformation of c-abl to v-abl occurs by insertional activation that results in deletions of specific N-terminal amino acids.'),
  'D016375': NtO(MH='Antisense Elements (Genetics)', MS='Nucleic acids which hybridize to complementary sequences in other target nucleic acids causing the function of the latter to be affected.'),
  'D016376': NtO(MH='Oligonucleotides, Antisense', MS='Short fragments of DNA or RNA that are used to alter the function of target RNAs or DNAs to which they hybridize.'),
  'D016377': NtO(MH='Organ Transplantation', MS='Transference of an organ between individuals of the same species or between individuals of different species.'),
  'D016378': NtO(MH='Tissue Transplantation', MS='Transference of tissue within an individual, between individuals of the same species, or between individuals of different species.'),
  'D016379': NtO(MH='Denture, Partial, Fixed, Resin-Bonded', MS='A commonly used prosthesis that results in a strong, permanent restoration. It consists of an electrolytically etched cast-metal retainer that is cemented (bonded), using resins, to adjacent teeth whose enamel was previously acid-treated (acid-etched). This type of bridgework is sometimes referred to as a Maryland bridge.'),
  'D016380': NtO(MH='Brain Tissue Transplantation', MS='Transference of brain tissue, either from a fetus or from a born individual, between individuals of the same species or between individuals of different species.'),
  'D016381': NtO(MH='Islets of Langerhans Transplantation', MS='The transference of pancreatic islets within an individual, between individuals of the same species, or between individuals of different species.'),
  'D016382': NtO(MH='Orthodontic Appliance Design', MS='The planning, calculation, and creation of an apparatus for the purpose of correcting the placement or straightening of teeth.'),
  'D016384': NtO(MH='Consensus Sequence', MS='A theoretical representative nucleotide or amino acid sequence in which each nucleotide or amino acid is the one which occurs most frequently at that site in the different sequences which occur in nature. The phrase also refers to an actual sequence which approximates the theoretical consensus. A known CONSERVED SEQUENCE set is represented by a consensus sequence. Commonly observed supersecondary protein structures (AMINO ACID MOTIFS) are often formed by conserved sequences.'),
  'D016385': NtO(MH='TATA Box', MS='A conserved A-T rich sequence which is contained in promoters for RNA polymerase II. The segment is seven base pairs long and the nucleotides most commonly found are TATAAAA.'),
  'D016386': NtO(MH='Chromosome Walking', MS='A technique with which an unknown region of a chromosome can be explored. It is generally used to isolate a locus of interest for which no probe is available but that is known to be linked to a gene which has been identified and cloned. A fragment containing a known gene is selected and used as a probe to identify other overlapping fragments which contain the same gene. The nucleotide sequences of these fragments can then be characterized. This process continues for the length of the chromosome.'),
  'D016387': NtO(MH="Women's Health", MS='The concept covering the physical and mental conditions of women.'),
  'D016388': NtO(MH='Tooth Loss', MS='The failure to retain teeth as a result of disease or injury.'),
  'D016389': NtO(MH='Maternal-Child Nursing', MS='The nursing specialty that deals with the care of women throughout their pregnancy and childbirth and the care of their newborn children.'),
  'D016390': NtO(MH='Allied Health Occupations', MS='Occupations of medical personnel who are not physicians, and are qualified by special training and, frequently, by licensure to work in supporting roles in the health care field. These occupations include, but are not limited to, medical technology, physical therapy, physician assistant, etc.'),
  'D016391': NtO(MH='Genes, src', MS='Retrovirus-associated DNA sequences (src) originally isolated from the Rous sarcoma virus (RSV). The proto-oncogene src (c-src) codes for a protein that is a member of the tyrosine kinase family and was the first proto-oncogene identified in the human genome. The human c-src gene is located at 20q12-13 on the long arm of chromosome 20.'),
  'D016392': NtO(MH='Proto-Oncogene Proteins pp60(c-src)', MS='Membrane-associated tyrosine-specific kinases encoded by the c-src genes. They have an important role in cellular growth control. Truncation of carboxy-terminal residues in pp60(c-src) leads to PP60(V-SRC) which has the ability to transform cells. This kinase pp60 c-src should not be confused with csk, also known as c-src kinase.'),
  'D016393': NtO(MH='Lymphoma, B-Cell', MS='A group of heterogeneous lymphoid tumors generally expressing one or more B-cell antigens or representing malignant transformations of B-lymphocytes.'),
  'D016399': NtO(MH='Lymphoma, T-Cell', MS='A group of heterogeneous lymphoid tumors representing malignant transformations of T-lymphocytes.'),
  'D016400': NtO(MH='Lymphoma, Large-Cell, Immunoblastic', MS='Malignant lymphoma characterized by the presence of immunoblasts with uniformly round-to-oval nuclei, one or more prominent nucleoli, and abundant cytoplasm. This class may be subdivided into plasmacytoid and clear-cell types based on cytoplasmic characteristics. A third category, pleomorphous, may be analogous to some of the peripheral T-cell lymphomas (LYMPHOMA, T-CELL, PERIPHERAL) recorded in both the United States and Japan.'),
  'D016403': NtO(MH='Lymphoma, Large B-Cell, Diffuse', MS='Malignant lymphoma composed of large B lymphoid cells whose nuclear size can exceed normal macrophage nuclei, or more than twice the size of a normal lymphocyte. The pattern is predominantly diffuse. Most of these lymphomas represent the malignant counterpart of B-lymphocytes at midstage in the process of differentiation.'),
  'D016410': NtO(MH='Lymphoma, T-Cell, Cutaneous', MS='A group of lymphomas exhibiting clonal expansion of malignant T-lymphocytes arrested at varying stages of differentiation as well as malignant infiltration of the skin. MYCOSIS FUNGOIDES; SEZARY SYNDROME; LYMPHOMATOID PAPULOSIS; and PRIMARY CUTANEOUS ANAPLASTIC LARGE CELL LYMPHOMA are the best characterized of these disorders.'),
  'D016411': NtO(MH='Lymphoma, T-Cell, Peripheral', MS="A group of malignant lymphomas thought to derive from peripheral T-lymphocytes in lymph nodes and other nonlymphoid sites. They include a broad spectrum of lymphocyte morphology, but in all instances express T-cell markers admixed with epithelioid histiocytes, plasma cells, and eosinophils. Although markedly similar to large-cell immunoblastic lymphoma (LYMPHOMA, LARGE-CELL, IMMUNOBLASTIC), this group's unique features warrant separate treatment."),
  'D016414': NtO(MH='Resuscitation Orders', MS='Instructions issued by a physician pertaining to the institution, continuation, or withdrawal of life support measures. The concept includes policies, laws, statutes, decisions, guidelines, and discussions that may affect the issuance of such orders.'),
  'D016415': NtO(MH='Sequence Alignment', MS='The arrangement of two or more amino acid or base sequences from an organism or organisms in such a way as to align areas of the sequences sharing common properties. The degree of relatedness or homology between the sequences is predicted computationally or statistically based on weights assigned to the elements aligned between the sequences. This in turn can serve as a potential indicator of the genetic relatedness between the organisms.'),
  'D016416': NtO(MH='Meeting Abstract', MS='Individual abstracts of presentations at meetings, congresses, conferences, symposia, colloquia, seminars, workshops, round tables, and other professional gatherings.'),
  'D016417': NtO(MH='Bibliography', MS='A work consisting of a list of books, articles, documents, publications, and other items, usually on a single subject or related subjects.'),
  'D016418': NtO(MH='Legal Case', MS='Work consisting of collections of law reports or the published reports of decided cases and documents or filings related to those cases.'),
  'D016419': NtO(MH='Classical Article', MS="Works consisting of a current presentation of a previously printed seminal article marking a milestone in the history of medicine or science. It is usually accompanied by introductory remarks heralding its reprinting, often on the anniversary of its original publication or on an anniversary of the author's birth or death. It is usually reprinted in full, with complete bibliographical reference to the original appearance."),
  'D016420': NtO(MH='Comment', MS='Work consisting of a critical or explanatory note written to discuss, support, or dispute an article or other presentation previously published. It may take the form of an article, letter, editorial, etc. It appears in publications under a variety of names: comment, commentary, editorial comment, viewpoint, etc.'),
  'D016421': NtO(MH='Editorial', MS='Work consisting of a statement of the opinions, beliefs, and policy of the editor or publisher of a journal, usually on current matters of medical or scientific significance to the medical community or society at large. The editorials published by editors of journals representing the official organ of a society or organization are generally substantive.'),
  'D016422': NtO(MH='Letter', MS='Work consisting of written or printed communication between individuals or between persons and representatives of corporate bodies. The correspondence may be personal or professional. In medical and other scientific publications the letter is usually from one or more authors to the editor of the journal or book publishing the item being commented upon or discussed. LETTER is often accompanied by COMMENT.'),
  'D016423': NtO(MH='Congress', MS='Published record of the papers delivered at or issued on the occasion of individual congresses, symposia, and meetings; abstracts of papers delivered at such congresses; reports of the officers and delegates of such congresses; combinations of the foregoing; or proceedings of the conference of a society if they are not limited to matters of internal organization.'),
  'D016424': NtO(MH='Overall', MS='A single citation covering papers or abstracts presented at a meeting. The publication type may be used for a single citation with or without the additional indexing or cataloging of individual papers. The individual papers, however, are not labeled OVERALL.'),
  'D016425': NtO(MH='Published Erratum', MS='Work consisting of an acknowledgment of an error, issued by a publisher, editor, or author. It customarily cites the source where the error occurred, giving complete bibliographic data for retrieval. In the case of books and monographs, author, title, imprint, paging, and other helpful references will be given; in the case of journal articles, the author, title, paging, and journal reference will be shown. An erratum notice is variously cited as Errata or Corrigenda.'),
  'D016426': NtO(MH='Scientific Integrity Review', MS='Work consisting of reports by the United States Office of Research Integrity, identifying questionable research published in articles or books. Notification of the questionable data is carried in the NIH Guide for Grants and Contracts.'),
  'D016427': NtO(MH='Technical Report', MS='Work consisting of a formal report giving details of the investigation and results of a medical or other scientific problem. When issued by a government agency or comparable official body, its contents may be classified, unclassified, or declassified with regard to security clearance. This publication type may also cover a scientific paper or article that records the current state or current position of scientific research and development. If so labeled by the editor or publisher, this publication type may be properly used for journal articles.'),
  'D016428': NtO(MH='Journal Article', MS='The predominant publication type for articles and other items indexed for NLM databases.'),
  'D016429': NtO(MH='Clinical Conference', MS='Work that consists of a conference of physicians on their observations of a patient at the bedside, regarding the physical state, laboratory and other diagnostic findings, clinical manifestations, results of current therapy, etc. A clinical conference usually ends with a confirmation or correction of clinical findings by a pathological diagnosis performed by a pathologist. "Clinical conference" is often referred to as a "clinico-pathological conference."'),
  'D016430': NtO(MH='Clinical Trial', MS='A work that reports on the results of a clinical study in which participants are assigned to receive one or more interventions so that researchers can evaluate the interventions on biomedical or health-related outcomes. The assignments are determined by the study protocol. Participants may receive diagnostic, therapeutic, or other types of interventions. For clinical trials on  veterinary animals see CLINICAL TRIAL, VETERINARY. Clinical Trials was used for both humans and non-humans prior to 2019.'),
  'D016431': NtO(MH='Guideline', MS='Work consisting of a set of statements, directions, or principles presenting current or future rules or policy. Guidelines may be developed by government agencies at any level, institutions, organizations such as professional societies or governing boards, or by the convening of expert panels. The text may be cursive or in outline form, but it is generally a comprehensive guide to problems and approaches in any discipline or activity. This concept relates to the general conduct and administration of health care activities rather than to specific decisions for a particular clinical condition. For that aspect, PRACTICE GUIDELINE is available.'),
  'D016433': NtO(MH='News', MS='Works consisting of an announcement or statement of recent or current events of new data and matters of interest in the field of medicine or science. In some publications, such as "Nature" or "Science," the news reports are substantively written and herald medical and scientific data of vital or controversial importance.'),
  'D016435': NtO(MH='Directory', MS='Work consisting of an alphabetical or classified list of names, organizations, subjects, etc., giving usually titles, addresses, affiliations, and other professional data.'),
  'D016437': NtO(MH='Dictionary', MS='A reference book containing a list of words - usually in alphabetical order - giving information about form, pronunciation, etymology, grammar, and meaning. A foreign-language dictionary is an alphabetical list of words of one language with their meaning and equivalents in another language.'),
  'D016438': NtO(MH='Duplicate Publication', MS='Work consisting of an article or book of identical or nearly identical material published simultaneously or successively to material previously published elsewhere, without acknowledgment of the prior publication.'),
  'D016439': NtO(MH='Corrected and Republished Article', MS='Work that is the republication of an article to correct, amplify, or restore text and data of the originally published article.'),
  'D016440': NtO(MH='Retraction of Publication', MS='Work consisting of a statement issued by one or more authors of an article or a book, withdrawing or disavowing acknowledgment of their participation in performing research or writing the results of their study. In indexing, the retraction is sent to the editor of the publication in which the article appeared and is published under the rubric "retraction" or in the form of a letter. This publication type designates the author\'s statement of retraction: it should be differentiated from RETRACTED PUBLICATION which labels the retracted publication.'),
  'D016441': NtO(MH='Retracted Publication', MS='Work consisting of the designation of an article or book as retracted in whole or in part by an author or authors or an authorized representative. It identifies a citation previously published and now retracted through a formal issuance from the author, publisher, or other authorized agent, and is distinguished from RETRACTION OF PUBLICATION, which identifies the citation retracting the original published item.'),
  'D016446': NtO(MH='Consensus Development Conference', MS='Official statements of the findings or recommendations expressing the outcome of a meeting convened to evaluate current thought and research on a subject of interest.'),
  'D016447': NtO(MH='Consensus Development Conference, NIH', MS='Official statements of the finding or recommendations expressing the outcome from a conference sponsored by NIH.'),
  'D016448': NtO(MH='Multicenter Study', MS='A work that reports on a study executed by several cooperating institutions.'),
  'D016449': NtO(MH='Randomized Controlled Trial', MS='A work that reports on a clinical trial that involves at least one test treatment and one control treatment, concurrent enrollment and follow-up of the test- and control-treated groups, and in which the treatments to be administered are selected by a random process, such as the use of a random-numbers table.'),
  'D016453': NtO(MH='Periodical Index', MS='Work consisting of a subject approach to the contents of a periodical issuing an annual, biennial, quinquennial, decennial, etc., index. The heading is used for the overall body of articles published by a periodical in the same sense that BIBLIOGRAPHY is useful when published as a single article.'),
  'D016454': NtO(MH='Review', MS='An article or book published after examination of published material on a subject. It may be comprehensive to various degrees and the time range of material scrutinized may be broad or narrow, but the reviews most often desired are reviews of the current literature. The textual material examined may be equally broad and can encompass, in medicine specifically, clinical material as well as experimental research or case reports. State-of-the-art reviews tend to address more current matters. A review of the literature must be differentiated from HISTORICAL ARTICLE on the same subject, but a review of historical literature is also within the scope of this publication type.'),
  'D016456': NtO(MH='Historical Article', MS='An article or portion of an article giving an account of past events or circumstances significant in a field of study, a profession, a discovery, an invention, etc. The concept of history is very wide, ranging from the dawn of time to the present. This publication type is often checked in conjunction with BIOGRAPHY.'),
  'D016459': NtO(MH='Prosthesis-Related Infections', MS='Infections resulting from the implantation of prosthetic devices. The infections may be acquired from intraoperative contamination (early) or hematogenously acquired from other sites (late).'),
  'D016460': NtO(MH='Granuloma Annulare', MS='Benign granulomatous disease of unknown etiology characterized by a ring of localized or disseminated papules or nodules on the skin and palisading histiocytes surrounding necrobiotic tissue resulting from altered collagen structures.'),
  'D016461': NtO(MH='Chemoembolization, Therapeutic', MS='Administration of antineoplastic agents together with an embolizing vehicle. This allows slow release of the agent as well as obstruction of the blood supply to the neoplasm.'),
  'D016462': NtO(MH='Mammaplasty', MS='Surgical reconstruction of the breast including both augmentation and reduction.'),
  'D016463': NtO(MH='Sweet Syndrome', MS='Condition characterized by large, rapidly extending, erythematous, tender plaques on the upper body usually accompanied by fever and dermal infiltration of neutrophilic leukocytes. It occurs mostly in middle-aged women, is often preceded by an upper respiratory infection, and clinically resembles ERYTHEMA MULTIFORME. Sweet syndrome is associated with LEUKEMIA.'),
  'D016464': NtO(MH='Lysosomal Storage Diseases', MS='Inborn errors of metabolism characterized by defects in specific lysosomal hydrolases and resulting in intracellular accumulation of unmetabolized substrates.'),
  'D016465': NtO(MH='Bone Marrow Purging', MS='Techniques for the removal of subpopulations of cells (usually residual tumor cells) from the bone marrow ex vivo before it is infused. The purging is achieved by a variety of agents including pharmacologic agents, biophysical agents (laser photoirradiation or radioisotopes) and immunologic agents. Bone marrow purging is used in both autologous and allogeneic BONE MARROW TRANSPLANTATION.'),
  'D016466': NtO(MH='CHO Cells', MS='CELL LINE derived from the ovary of the Chinese hamster, Cricetulus griseus (CRICETULUS). The species is a favorite for cytogenetic studies because of its small chromosome number. The cell line has provided model systems for the study of genetic alterations in cultured mammalian cells.'),
  'D016467': NtO(MH='Monograph', MS='Work that is any publication that is not a serial or integrating resource. In cataloging usage, It is usually on a single subject or related subjects and is complete in itself, whether constructed of chapters, sections, or parts. While any article encountered in indexing journals can be, strictly speaking, a monograph, as a publication type, a monograph will refer to a cataloging item.'),
  'D016468': NtO(MH='Forestry', MS='The science of developing, caring for, or cultivating forests.'),
  'D016469': NtO(MH='Fungemia', MS='The presence of fungi circulating in the blood. Opportunistic fungal sepsis is seen most often in immunosuppressed patients with severe neutropenia or in postoperative patients with intravenous catheters and usually follows prolonged antibiotic therapy.'),
  'D016470': NtO(MH='Bacteremia', MS='The presence of viable bacteria circulating in the blood. Fever, chills, tachycardia, and tachypnea are common acute manifestations of bacteremia. The majority of cases are seen in already hospitalized patients, most of whom have underlying diseases or procedures which render their bloodstreams susceptible to invasion.'),
  'D016471': NtO(MH='Ovarian Hyperstimulation Syndrome', MS='A complication of OVULATION INDUCTION in infertility treatment. It is graded by the severity of symptoms which include OVARY enlargement, multiple OVARIAN FOLLICLES; OVARIAN CYSTS; ASCITES; and generalized EDEMA. The full-blown syndrome may lead to RENAL FAILURE, respiratory distress, and even DEATH. Increased capillary permeability is caused by the vasoactive substances, such as VASCULAR ENDOTHELIAL GROWTH FACTORS, secreted by the overly-stimulated OVARIES.'),
  'D016472': NtO(MH='Motor Neuron Disease', MS='Diseases characterized by a selective degeneration of the motor neurons of the spinal cord, brainstem, or motor cortex. Clinical subtypes are distinguished by the major site of degeneration. In AMYOTROPHIC LATERAL SCLEROSIS there is involvement of upper, lower, and brainstem motor neurons. In progressive muscular atrophy and related syndromes (see MUSCULAR ATROPHY, SPINAL) the motor neurons in the spinal cord are primarily affected. With progressive bulbar palsy (BULBAR PALSY, PROGRESSIVE), the initial degeneration occurs in the brainstem. In primary lateral sclerosis, the cortical neurons are affected in isolation. (Adams et al., Principles of Neurology, 6th ed, p1089)'),
  'D016473': NtO(MH='Gallbladder Emptying', MS='A process whereby bile is delivered from the gallbladder into the duodenum. The emptying is caused by both contraction of the gallbladder and relaxation of the sphincter mechanism at the choledochal terminus.'),
  'D016474': NtO(MH='Weight-Bearing', MS="The physical state of supporting an applied load. This often refers to the weight-bearing bones or joints that support the body's weight, especially those in the spine, hip, knee, and foot."),
  'D016475': NtO(MH='3T3 Cells', MS='Cell lines whose original growing procedure consisted being transferred (T) every 3 days and plated at 300,000 cells per plate (J Cell Biol 17:299-313, 1963). Lines have been developed using several different strains of mice. Tissues are usually fibroblasts derived from mouse embryos but other types and sources have been developed as well. The 3T3 lines are valuable in vitro host systems for oncogenic virus transformation studies, since 3T3 cells possess a high sensitivity to CONTACT INHIBITION.'),
  'D016476': NtO(MH='Urinary Reservoirs, Continent', MS='Structures which collect and store urine and are emptied by catheterization of a cutaneous stoma or internal diversion to the urethra. The reservoirs are surgically created during procedures for urinary diversion.'),
  'D016477': NtO(MH='Artifacts', MS='Any visible result of a procedure which is caused by the procedure itself and not by the entity being analyzed. Common examples include histological structures introduced by tissue processing, radiographic images of structures that are not naturally present in living tissue, and products of chemical reactions that occur during analysis.'),
  'D016480': NtO(MH='Helicobacter pylori', MS='A spiral bacterium active as a human gastric pathogen. It is a gram-negative, urease-positive, curved or slightly spiral organism initially isolated in 1982 from patients with lesions of gastritis or peptic ulcers in Western Australia. Helicobacter pylori was originally classified in the genus CAMPYLOBACTER, but RNA sequencing, cellular fatty acid profiles, growth patterns, and other taxonomic characteristics indicate that the micro-organism should be included in the genus HELICOBACTER. It has been officially transferred to Helicobacter gen. nov. (see Int J Syst Bacteriol 1989 Oct;39(4):297-405).'),
  'D016481': NtO(MH='Helicobacter Infections', MS='Infections with organisms of the genus HELICOBACTER, particularly, in humans, HELICOBACTER PYLORI. The clinical manifestations are focused in the stomach, usually the gastric mucosa and antrum, and the upper duodenum. This infection plays a major role in the pathogenesis of type B gastritis and peptic ulcer disease.'),
  'D016482': NtO(MH='Urinalysis', MS='Examination of urine by chemical, physical, or microscopic means. Routine urinalysis usually includes performing chemical screening tests, determining specific gravity, observing any unusual color or odor, screening for bacteriuria, and examining the sediment microscopically.'),
  'D016483': NtO(MH='Lymphoma, AIDS-Related', MS='B-cell lymphoid tumors that occur in association with AIDS. Patients often present with an advanced stage of disease and highly malignant subtypes including BURKITT LYMPHOMA; IMMUNOBLASTIC LARGE-CELL LYMPHOMA; PRIMARY EFFUSION LYMPHOMA; and DIFFUSE, LARGE B-CELL, LYMPHOMA. The tumors are often disseminated in unusual extranodal sites and chromosomal abnormalities are frequently present. It is likely that polyclonal B-cell lymphoproliferation in AIDS is a complex result of EBV infection, HIV antigenic stimulation, and T-cell-dependent HIV activation.'),
  'D016485': NtO(MH='Needle Sharing', MS='Usage of a single needle among two or more people for injecting drugs. Needle sharing is a high-risk behavior for contracting infectious disease.'),
  'D016487': NtO(MH='Parenting', MS="Performing the role of a parent by care-giving, nurturance, and protection of the child by a natural or substitute parent. The parent supports the child by exercising authority and through consistent, empathic, appropriate behavior in response to the child's needs. PARENTING differs from CHILD REARING in that in child rearing the emphasis is on the act of training or bringing up the children and the interaction between the parent and child, while parenting emphasizes the responsibility and qualities of exemplary behavior of the parent."),
  'D016488': NtO(MH='Medicine, African Traditional', MS='A system of traditional medicine which is based on the beliefs and practices of the African peoples. It includes treatment by medicinal plants and other materia medica as well as by the ministrations of diviners, medicine men, witch doctors, and sorcerers.'),
  'D016489': NtO(MH='Head Injuries, Closed', MS="Traumatic injuries to the cranium where the integrity of the skull is not compromised and no bone fragments or other objects penetrate the skull and DURA MATER. This frequently results in mechanical injury being transmitted to intracranial structures which may produce TRAUMATIC BRAIN INJURIES, hemorrhage, or CRANIAL NERVE INJURIES. (From Rowland, Merritt's Textbook of Neurology, 9th ed, p417)"),
  'D016490': NtO(MH='Integrated Advanced Information Management Systems', MS='A concept, developed in 1983 under the aegis of and supported by the National Library of Medicine under the name of Integrated Academic Information Management Systems, to provide professionals in academic health sciences centers and health sciences institutions with convenient access to an integrated and comprehensive network of knowledge. It addresses a wide cross-section of users from administrators and faculty to students and clinicians and has applications to planning, clinical and managerial decision-making, teaching, and research. It provides access to various types of clinical, management, educational, etc., databases, as well as to research and bibliographic databases. In August 1992 the name was changed from Integrated Academic Information Management Systems to Integrated Advanced Information Management Systems to reflect use beyond the academic milieu.'),
  'D016491': NtO(MH='Peripheral Vascular Diseases', MS='Pathological processes involving any one of the BLOOD VESSELS in the vasculature outside the HEART.'),
  'D016494': NtO(MH='Computer Security', MS='Protective measures against unauthorized access to or interference with computer operating systems, telecommunications, or accompanying data; especially the modification, deletion, destruction, or release of data in computers. It includes methods of forestalling interference by computer viruses or computer hackers aiming to compromise stored data.'),
  'D016495': NtO(MH='Police', MS='Agents of the law charged with the responsibility of maintaining and enforcing law and order among the citizenry.'),
  'D016496': NtO(MH='Incontinence Pads', MS='Absorbent pads used for URINARY INCONTINENCE and usually worn as underpants or pants liners by the ELDERLY.'),
  'D016497': NtO(MH='Adolescent Health Services', MS='Organized services to provide health care to adolescents, ages ranging from 13 through 18 years.'),
  'D016499': NtO(MH='Radioimmunotherapy', MS='Radiotherapy where cytotoxic radionuclides are linked to antibodies in order to deliver toxins directly to tumor targets. Therapy with targeted radiation rather than antibody-targeted toxins (IMMUNOTOXINS) has the advantage that adjacent tumor cells, which lack the appropriate antigenic determinants, can be destroyed by radiation cross-fire. Radioimmunotherapy is sometimes called targeted radiotherapy, but this latter term can also refer to radionuclides linked to non-immune molecules (see RADIOTHERAPY).'),
  'D016500': NtO(MH='Color Therapy', MS='A form of phototherapy using color to influence health and to treat various physical or mental disorders. The color rays may be in the visible or invisible spectrum and can be administered through colored lights or applied mentally through suggestion.'),
  'D016501': NtO(MH='Neurites', MS='In tissue culture, hairlike projections of neurons stimulated by growth factors and other molecules. These projections may go on to form a branched tree of dendrites or a single axon or they may be reabsorbed at a later stage of development. "Neurite" may refer to any filamentous or pointed outgrowth of an embryonal or tissue-culture neural cell.'),
  'D016503': NtO(MH='Drug Delivery Systems', MS='Systems for the delivery of drugs to target sites of pharmacological actions. Technologies employed include those concerning drug preparation, route of administration, site targeting, metabolism, and toxicity.'),
  'D016504': NtO(MH='Breast Self-Examination', MS="The inspection of one's breasts, usually for signs of disease, especially neoplastic disease."),
  'D016506': NtO(MH='Pemphigus, Benign Familial', MS='An autosomal dominantly inherited skin disorder characterized by recurrent eruptions of vesicles and BULLAE mainly on the neck, axillae, and groin. Mutations in the ATP2C1 gene (encoding the secretory pathway Ca2++/Mn2++ ATPase 1 (SPCA1)) cause this disease. It is clinically and histologically similar to DARIER DISEASE - both have abnormal, unstable DESMOSOMES between KERATINOCYTES and defective CALCIUM-TRANSPORTING ATPASES. It is unrelated to PEMPHIGUS VULGARIS though it closely resembles that disease.'),
  'D016507': NtO(MH='Cardiology Service, Hospital', MS='The hospital department responsible for the administration and provision of diagnostic and therapeutic services for the cardiac patient.'),
  'D016508': NtO(MH='Fluorescent Treponemal Antibody-Absorption Test', MS="Serologic assay that detects antibodies to Treponema pallidum, the etiologic agent of syphilis. After diluting the patient's serum to remove non-specific antibodies, the serum is mixed on a glass slide with Nichol's strain of Treponema pallidum. An antigen-antibody reaction occurs if the test is positive and the bound antibodies are detected with fluoresceinated antihuman gamma-globulin antibody."),
  'D016510': NtO(MH='Corneal Neovascularization', MS='New blood vessels originating from the corneal blood vessels and extending from the limbus into the adjacent CORNEAL STROMA. Neovascularization in the superficial and/or deep corneal stroma is a sequel to numerous inflammatory diseases of the ocular anterior segment, such as TRACHOMA, viral interstitial KERATITIS, microbial KERATOCONJUNCTIVITIS, and the immune response elicited by CORNEAL TRANSPLANTATION.'),
  'D016511': NtO(MH='Severe Combined Immunodeficiency', MS='Group of rare congenital disorders characterized by impairment of both humoral and cell-mediated immunity, leukopenia, and low or absent antibody levels. It is inherited as an X-linked or autosomal recessive defect. Mutations occurring in many different genes cause human Severe Combined Immunodeficiency (SCID).'),
  'D016512': NtO(MH='Ankle Injuries', MS='Harm or hurt to the ankle or ankle joint usually inflicted by an external source.'),
  'D016513': NtO(MH='Mice, SCID', MS='Mice homozygous for the mutant autosomal recessive gene "scid" which is located on the centromeric end of chromosome 16. These mice lack mature, functional lymphocytes and are thus highly susceptible to lethal opportunistic infections if not chronically treated with antibiotics. The lack of B- and T-cell immunity resembles severe combined immunodeficiency (SCID) syndrome in human infants. SCID mice are useful as animal models since they are receptive to implantation of a human immune system producing SCID-human (SCID-hu) hematochimeric mice.'),
  'D016514': NtO(MH='Genes, Neurofibromatosis 1', MS='Tumor suppressor genes located on the long arm of human chromosome 17 in the region 17q11.2. Mutation of these genes is thought to cause NEUROFIBROMATOSIS 1, Watson syndrome, and LEOPARD syndrome.'),
  'D016515': NtO(MH='Genes, Neurofibromatosis 2', MS='Tumor suppressor genes located on the long arm of human chromosome 22. Mutation or loss of these genes causes NEUROFIBROMATOSIS 2.'),
  'D016516': NtO(MH='CD4-CD8 Ratio', MS='Ratio of T-LYMPHOCYTES that express the CD4 ANTIGEN to those that express the CD8 ANTIGEN. This value is commonly assessed in the diagnosis and staging of diseases affecting the IMMUNE SYSTEM including HIV INFECTIONS.'),
  'D016518': NtO(MH='Neurofibromatosis 2', MS='An autosomal dominant disorder characterized by a high incidence of bilateral acoustic neuromas as well as schwannomas (NEURILEMMOMA) of other cranial and peripheral nerves, and other benign intracranial tumors including meningiomas, ependymomas, spinal neurofibromas, and gliomas. The disease has been linked to mutations of the NF2 gene (GENES, NEUROFIBROMATOSIS 2) on chromosome 22 (22q12) and usually presents clinically in the first or second decade of life.'),
  'D016519': NtO(MH='Hospitals, Religious', MS='Private hospitals that are owned or sponsored by religious organizations.'),
  'D016520': NtO(MH='Entrepreneurship', MS='The organization, management, and assumption of risks of a business or enterprise, usually implying an element of change or challenge and a new opportunity.'),
  'D016521': NtO(MH='Electrophoresis, Gel, Pulsed-Field', MS='Gel electrophoresis in which the direction of the electric field is changed periodically. This technique is similar to other electrophoretic methods normally used to separate double-stranded DNA molecules ranging in size up to tens of thousands of base-pairs. However, by alternating the electric field direction one is able to separate DNA molecules up to several million base-pairs in length.'),
  'D016522': NtO(MH='Oncology Service, Hospital', MS='The hospital department responsible for the administration and provision of diagnostic and therapeutic services for the cancer patient.'),
  'D016523': NtO(MH='Foot Ulcer', MS='Lesion on the surface of the skin of the foot, usually accompanied by inflammation. The lesion may become infected or necrotic and is frequently associated with diabetes or leprosy.'),
  'D016524': NtO(MH='Parental Leave', MS='The authorized absence from work of either parent prior to and after the birth of their child. It includes also absence because of the illness of a child or at the time of the adoption of a child. It does not include leave for care of siblings, parents, or other family members: for this FAMILY LEAVE is available.'),
  'D016525': NtO(MH='Antigenic Modulation', MS='Loss of detectable antigen from the surface of a cell after incubation with antibodies. This is one method in which some tumors escape detection by the immune system. Antigenic modulation of target antigens also reduces the therapeutic effectiveness of treatment by monoclonal antibodies.'),
  'D016526': NtO(MH='Workload', MS='The total amount of work to be performed by an individual, a department, or other group of workers in a period of time.'),
  'D016527': NtO(MH='Drug Costs', MS='The amount that a health care institution or organization pays for its drugs. It is one component of the final price that is charged to the consumer (FEES, PHARMACEUTICAL or PRESCRIPTION FEES).'),
  'D016528': NtO(MH='Postanesthesia Nursing', MS='The specialty or practice of nursing in the care of patients in the recovery room following surgery and/or anesthesia.'),
  'D016529': NtO(MH='Emergency Nursing', MS='The specialty or practice of nursing in the care of patients admitted to the emergency department.'),
  'D016530': NtO(MH='Orthopedic Nursing', MS='The specialty or practice of nursing in the care of the orthopedic patient.'),
  'D016532': NtO(MH='Mucopolysaccharidosis II', MS='Systemic lysosomal storage disease marked by progressive physical deterioration and caused by a deficiency of L-sulfoiduronate sulfatase. This disease differs from MUCOPOLYSACCHARIDOSIS I by slower progression, lack of corneal clouding, and X-linked rather than autosomal recessive inheritance. The mild form produces near-normal intelligence and life span. The severe form usually causes death by age 15.'),
  'D016533': NtO(MH='Mycological Typing Techniques', MS='Procedures for identifying types and strains of fungi.'),
  'D016534': NtO(MH='Cardiac Output, High', MS='A state of elevated cardiac output due to conditions of either increased hemodynamic demand or reduced cardiac oxygen output. These conditions may include ANEMIA; ARTERIOVENOUS FISTULA; THYROTOXICOSIS; PREGNANCY; EXERCISE; FEVER; and HYPOXIA. In time, compensatory changes of the heart can lead to pathological form of high cardiac output and eventual HEART FAILURE.'),
  'D016535': NtO(MH='Bronchial Hyperreactivity', MS='Tendency of the smooth muscle of the tracheobronchial tree to contract more intensely in response to a given stimulus than it does in the response seen in normal individuals. This condition is present in virtually all symptomatic patients with asthma. The most prominent manifestation of this smooth muscle contraction is a decrease in airway caliber that can be readily measured in the pulmonary function laboratory.'),
  'D016536': NtO(MH='Competitive Medical Plans', MS='Alternative health care delivery mechanisms, such as PREFERRED PROVIDER ORGANIZATIONS or other health insurance services or prepaid plans (other than HEALTH MAINTENANCE ORGANIZATIONS), that meet Medicare qualifications for a risk-sharing contract. (From Facts on File Dictionary of Health Care Management, 1988)'),
  'D016537': NtO(MH='Gangliosidosis, GM1', MS='An autosomal recessive neurodegenerative disorder caused by the absence or deficiency of BETA-GALACTOSIDASE. It is characterized by intralysosomal accumulation of G(M1) GANGLIOSIDE and oligosaccharides, primarily in neurons of the central nervous system. The infantile form is characterized by MUSCLE HYPOTONIA, poor psychomotor development, HIRSUTISM, hepatosplenomegaly, and facial abnormalities. The juvenile form features HYPERACUSIS; SEIZURES; and psychomotor retardation. The adult form features progressive DEMENTIA; ATAXIA; and MUSCLE SPASTICITY. (From Menkes, Textbook of Child Neurology, 5th ed, pp96-7)'),
  'D016538': NtO(MH='Mucopolysaccharidosis VII', MS='Mucopolysaccharidosis characterized by excessive dermatan and heparan sulfates in the urine and Hurler-like features. It is caused by a deficiency of beta-glucuronidase.'),
  'D016539': NtO(MH='Reed-Sternberg Cells', MS='Large cells, usually multinucleate, whose presence is a common histologic characteristic of classical HODGKIN DISEASE.'),
  'D016540': NtO(MH='Smoking Cessation', MS='Discontinuing the habit of SMOKING.'),
  'D016542': NtO(MH='Chief Executive Officers, Hospital', MS='Individuals who have the formal authority to manage a hospital, including its programs and services, in accordance with the goals and objectives established by a governing body (GOVERNING BOARD).'),
  'D016543': NtO(MH='Central Nervous System Neoplasms', MS='Benign and malignant neoplastic processes that arise from or secondarily involve the brain, spinal cord, or meninges.'),
  'D016544': NtO(MH='Patient Simulation', MS='The use of persons coached to feign symptoms or conditions of real diseases in a life-like manner in order to teach or evaluate medical personnel.'),
  'D016545': NtO(MH='Choroid Plexus Neoplasms', MS='Benign or malignant tumors which arise from the choroid plexus of the ventricles of the brain. Papillomas (see PAPILLOMA, CHOROID PLEXUS) and carcinomas are the most common histologic subtypes, and tend to seed throughout the ventricular and subarachnoid spaces. Clinical features include headaches, ataxia and alterations of consciousness, primarily resulting from associated HYDROCEPHALUS. (From Devita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2072; J Neurosurg 1998 Mar;88(3):521-8)'),
  'D016547': NtO(MH='Kinesin', MS='A microtubule-associated mechanical adenosine triphosphatase, that uses the energy of ATP hydrolysis to move organelles along microtubules toward the plus end of the microtubule. The protein is found in squid axoplasm, optic lobes, and in bovine brain. Bovine kinesin is a heterotetramer composed of two heavy (120 kDa) and two light (62 kDa) chains.'),
  'D016548': NtO(MH='Prosencephalon', MS='The anterior of the three primitive cerebral vesicles of the embryonic brain arising from the NEURAL TUBE. It subdivides to form DIENCEPHALON and TELENCEPHALON. (Stedmans Medical Dictionary, 27th ed)'),
  'D016549': NtO(MH='United States Indian Health Service', MS='A division of the UNITED STATES PUBLIC HEALTH SERVICE that is responsible for the public health and the provision of medical services to NATIVE AMERICANS in the United States, primarily those residing on reservation lands.'),
  'D016550': NtO(MH='Birthing Centers', MS='Free-standing facilities that provide prenatal, childbirth, and postnatal care and usually incorporate family-centered maternity care concepts and practices.'),
  'D016552': NtO(MH='Ergometry', MS='Any method of measuring the amount of work done by an organism, usually during PHYSICAL EXERTION. Ergometry also includes measures of power. Some instruments used in these determinations include the hand crank and the bicycle ergometer.'),
  'D016553': NtO(MH='Purpura, Thrombocytopenic, Idiopathic', MS='Thrombocytopenia occurring in the absence of toxic exposure or a disease associated with decreased platelets. It is mediated by immune mechanisms, in most cases IMMUNOGLOBULIN G autoantibodies which attach to platelets and subsequently undergo destruction by macrophages. The disease is seen in acute (affecting children) and chronic (adult) forms.'),
  'D016555': NtO(MH='Matched-Pair Analysis', MS='A type of analysis in which subjects in a study group and a comparison group are made comparable with respect to extraneous factors by individually pairing study subjects with the comparison group subjects (e.g., age-matched controls).'),
  'D016556': NtO(MH='Crown Lengthening', MS='Technique combining controlled eruptive TOOTH MOVEMENT TECHNIQUES and incision of the supracrestal gingival attachment to allow for proper restoration of a destroyed or damaged crown of a tooth. Controlled eruption of the tooth is obtained using ORTHODONTIC APPLIANCES. During this eruptive phase, repeated incisions are made at the junctional epithelium of the sulcus and the supracrestal connective tissue to prevent coronal displacement of the gingiva and of the attachment apparatus.'),
  'D016557': NtO(MH='Guided Tissue Regeneration, Periodontal', MS='Techniques for enhancing and directing cell growth to repopulate specific parts of the PERIODONTIUM that have been damaged by PERIODONTAL DISEASES; TOOTH DISEASES; or TRAUMA, or to correct TOOTH ABNORMALITIES. Repopulation and repair is achieved by guiding the progenitor cells to reproduce in the desired location by blocking contact with surrounding tissue by use of membranes composed of synthetic or natural material that may include growth inducing factors as well.'),
  'D016558': NtO(MH='Hemostasis, Endoscopic', MS='Control of bleeding performed through the channel of the endoscope. Techniques include use of lasers, heater probes, bipolar electrocoagulation, and local injection. Endoscopic hemostasis is commonly used to treat bleeding esophageal and gastrointestinal varices and ulcers.'),
  'D016559': NtO(MH='Tacrolimus', MS='A macrolide isolated from the culture broth of a strain of Streptomyces tsukubaensis that has strong immunosuppressive activity in vivo and prevents the activation of T-lymphocytes in response to antigenic or mitogenic stimulation in vitro.'),
  'D016562': NtO(MH='Parenteral Nutrition, Home Total', MS='The at-home administering of nutrients for assimilation and utilization by a patient whose sole source of nutrients is via solutions administered intravenously, subcutaneously or by some other non-alimentary route.'),
  'D016563': NtO(MH='Filoviridae', MS='A family of RNA viruses, of the order MONONEGAVIRALES, containing filamentous virions. Although they resemble RHABDOVIRIDAE in possessing helical nucleocapsids, Filoviridae differ in the length and degree of branching in their virions. There are two genera: EBOLAVIRUS and MARBURGVIRUS.'),
  'D016564': NtO(MH='Amyloid beta-Protein Precursor', MS='A single-pass type I membrane protein. It is cleaved by AMYLOID PRECURSOR PROTEIN SECRETASES to produce peptides of varying amino acid lengths. A 39-42 amino acid peptide, AMYLOID BETA-PEPTIDES is a principal component of the extracellular amyloid in SENILE PLAQUES.'),
  'D016565': NtO(MH='Cheirogaleidae', MS='A family of the order PRIMATES, suborder Strepsirhini (PROSIMII), containing five genera. All inhabitants of Madagascar, the genera are: Allocebus, Cheirogaleus (dwarf lemurs), Microcebus (mouse lemurs), Mirza, and Phaner.'),
  'D016566': NtO(MH='Organoselenium Compounds', MS='Organic compounds which contain selenium as an integral part of the molecule.'),
  'D016567': NtO(MH='Nizatidine', MS='A histamine H2 receptor antagonist with low toxicity that inhibits gastric acid secretion. The drug is used for the treatment of duodenal ulcers.'),
  'D016568': NtO(MH='Drugs, Generic', MS='Drugs whose drug name is not protected by a trademark. They may be manufactured by several companies.'),
  'D016569': NtO(MH='Blepharophimosis', MS='The abnormal narrowness of the palpebral fissure in the horizontal direction caused by the lateral displacement of the medial canthi of the eyelids. (Dorland, 27th ed)'),
  'D016570': NtO(MH='Bruch Membrane', MS='The inner layer of CHOROID, also called the lamina basalis choroideae, located adjacent to the RETINAL PIGMENT EPITHELIUM; (RPE) of the EYE. It is a membrane composed of the basement membranes of the choriocapillaris ENDOTHELIUM and that of the RPE. The membrane stops at the OPTIC NERVE, as does the RPE.'),
  'D016571': NtO(MH='Neural Networks, Computer', MS='A computer architecture, implementable in either hardware or software, modeled after biological neural networks. Like the biological system in which the processing capability is a result of the interconnection strengths between arrays of nonlinear processing nodes, computerized neural networks, often called perceptrons or multilayer connectionist models, consist of neuron-like units. A homogeneous group of units makes up a layer. These networks are good at pattern recognition. They are adaptive, performing tasks by example, and thus are better for decision-making than are linear learning machines or cluster analysis. They do not require explicit programming.'),
  'D016572': NtO(MH='Cyclosporine', MS='A cyclic undecapeptide from an extract of soil fungi. It is a powerful immunosupressant with a specific action on T-lymphocytes. It is used for the prophylaxis of graft rejection in organ and tissue transplantation. (From Martindale, The Extra Pharmacopoeia, 30th ed).'),
  'D016573': NtO(MH='Agrochemicals', MS='Chemicals used in agriculture. These include pesticides, fumigants, fertilizers, plant hormones, steroids, antibiotics, mycotoxins, etc.'),
  'D016574': NtO(MH='Seasonal Affective Disorder', MS='A syndrome characterized by depressions that recur annually at the same time each year, usually during the winter months. Other symptoms include anxiety, irritability, decreased energy, increased appetite (carbohydrate cravings), increased duration of sleep, and weight gain. SAD (seasonal affective disorder) can be treated by daily exposure to bright artificial lights (PHOTOTHERAPY), during the season of recurrence.'),
  'D016575': NtO(MH='Hidradenitis', MS='The inflammation of a sweat gland (usually of the apocrine type). The condition can be idiopathic or occur as a result of or in association with another underlying condition. Neutrophilic eccrine hidradenitis is a relatively rare variant that has been reported in patients undergoing chemotherapy, usually for non-Hodgkin lymphomas or leukemic conditions.'),
  'D016576': NtO(MH='Fleroxacin', MS='A broad-spectrum antimicrobial fluoroquinolone. The drug strongly inhibits the DNA-supercoiling activity of DNA GYRASE.'),
  'D016577': NtO(MH='Pancreaticoduodenectomy', MS='The excision of the head of the pancreas and the encircling loop of the duodenum to which it is connected.'),
  'D016578': NtO(MH='Crack Cocaine', MS='The purified, alkaloidal, extra-potent form of cocaine. It is smoked (free-based), injected intravenously, and orally ingested. Use of crack results in alterations in function of the cardiovascular system, the autonomic nervous system, the central nervous system, and the gastrointestinal system. The slang term "crack" was derived from the crackling sound made upon igniting of this form of cocaine for smoking.'),
  'D016582': NtO(MH='Leukemia, Feline', MS='A neoplastic disease of cats frequently associated with feline leukemia virus infection.'),
  'D016583': NtO(MH='Enzootic Bovine Leukosis', MS='A lymphoid neoplastic disease in cattle caused by the bovine leukemia virus. Enzootic bovine leukosis may take the form of lymphosarcoma, malignant lymphoma, or leukemia but the presence of malignant cells in the blood is not a consistent finding.'),
  'D016584': NtO(MH='Panic Disorder', MS='A type of anxiety disorder characterized by unexpected panic attacks that last minutes or, rarely, hours. Panic attacks begin with intense apprehension, fear or terror and, often, a feeling of impending doom. Symptoms experienced during a panic attack include dyspnea or sensations of being smothered; dizziness, loss of balance or faintness; choking sensations; palpitations or accelerated heart rate; shakiness; sweating; nausea or other form of abdominal distress; depersonalization or derealization; paresthesias; hot flashes or chills; chest discomfort or pain; fear of dying and fear of not being in control of oneself or going crazy. Agoraphobia may also develop. Similar to other anxiety disorders, it may be inherited as an autosomal dominant trait.'),
  'D016585': NtO(MH='Vaginosis, Bacterial', MS='Polymicrobial, nonspecific vaginitis associated with positive cultures of Gardnerella vaginalis and other anaerobic organisms and a decrease in lactobacilli. It remains unclear whether the initial pathogenic event is caused by the growth of anaerobes or a primary decrease in lactobacilli.'),
  'D016586': NtO(MH='Granular Cell Tumor', MS='Unusual tumor affecting any site of the body, but most often encountered in the head and neck. Considerable debate has surrounded the histogenesis of this neoplasm; however, it is considered to be a myoblastoma of, usually, a benign nature. It affects women more often than men. When it develops beneath the epidermis or mucous membrane, it can lead to proliferation of the squamous cells and mimic squamous cell carcinoma.'),
  'D016587': NtO(MH='Antimutagenic Agents', MS='Agents that reduce the frequency or rate of spontaneous or induced mutations independently of the mechanism involved.'),
  'D016588': NtO(MH='Anticarcinogenic Agents', MS='Agents that reduce the frequency or rate of spontaneous or induced tumors independently of the mechanism involved.'),
  'D016589': NtO(MH='Astemizole', MS='Antihistamine drug now withdrawn from the market in many countries because of rare but potentially fatal side effects.'),
  'D016590': NtO(MH='Aphidicolin', MS='An antiviral antibiotic produced by Cephalosporium aphidicola and other fungi. It inhibits the growth of eukaryotic cells and certain animal viruses by selectively inhibiting the cellular replication of DNA polymerase II or the viral-induced DNA polymerases. The drug may be useful for controlling excessive cell proliferation in patients with cancer, psoriasis or other dermatitis with little or no adverse effect upon non-multiplying cells.'),
  'D016591': NtO(MH='Histocytological Preparation Techniques', MS='Methods of preparing cells or tissues for examination and study of their origin, structure, function, or pathology. The methods include preservation, fixation, sectioning, staining, replica, or other technique to allow for viewing using a microscope.'),
  'D016593': NtO(MH='Terfenadine', MS='A selective histamine H1-receptor antagonist devoid of central nervous system depressant activity. The drug was used for ALLERGY but withdrawn due to causing LONG QT SYNDROME.'),
  'D016595': NtO(MH='Misoprostol', MS='A synthetic analog of natural prostaglandin E1. It produces a dose-related inhibition of gastric acid and pepsin secretion, and enhances mucosal resistance to injury. It is an effective anti-ulcer agent and also has oxytocic properties.'),
  'D016596': NtO(MH='Vinculin', MS='A cytoskeletal protein associated with cell-cell and cell-matrix interactions. The amino acid sequence of human vinculin has been determined. The protein consists of 1066 amino acid residues and its gene has been assigned to chromosome 10.'),
  'D016597': NtO(MH='Trimetrexate', MS='A nonclassical folic acid inhibitor through its inhibition of the enzyme dihydrofolate reductase. It is being tested for efficacy as an antineoplastic agent and as an antiparasitic agent against PNEUMOCYSTIS PNEUMONIA in AIDS patients. Myelosuppression is its dose-limiting toxic effect.'),
  'D016601': NtO(MH='RNA-Binding Proteins', MS='Proteins that bind to RNA molecules. Included here are RIBONUCLEOPROTEINS and other proteins whose function is to bind specifically to RNA.'),
  'D016602': NtO(MH='Needlestick Injuries', MS='Penetrating stab wounds caused by needles. They are of special concern to health care workers since such injuries put them at risk for developing infectious disease.'),
  'D016603': NtO(MH='Eosinophilia-Myalgia Syndrome', MS='A complex systemic syndrome with inflammatory and autoimmune components that affect the skin, fascia, muscle, nerve, blood vessels, lung, and heart. Diagnostic features generally include EOSINOPHILIA, myalgia severe enough to limit usual activities of daily living, and the absence of coexisting infectious, autoimmune or other conditions that may induce eosinophilia. Biopsy of affected tissue reveals a microangiopathy associated with diffuse inflammation involving connective tissue. (From Spitzer et al., J Rheumatol Suppl 1996 Oct;46:73-9; Blackburn WD, Semin Arthritis Rheum 1997 Jun;26(6):788-93)'),
  'D016604': NtO(MH='Aflatoxin B1', MS='A potent hepatotoxic and hepatocarcinogenic mycotoxin produced by the Aspergillus flavus group of fungi. It is also mutagenic, teratogenic, and causes immunosuppression in animals. It is found as a contaminant in peanuts, cottonseed meal, corn, and other grains. The mycotoxin requires epoxidation to aflatoxin B1 2,3-oxide for activation. Microsomal monooxygenases biotransform the toxin to the less toxic metabolites aflatoxin M1 and Q1.'),
  'D016606': NtO(MH='Thyroid Nodule', MS='A small circumscribed mass in the THYROID GLAND that can be of neoplastic growth or non-neoplastic abnormality. It lacks a well-defined capsule or glandular architecture. Thyroid nodules are often benign but can be malignant. The growth of nodules can lead to a multinodular goiter (GOITER, NODULAR).'),
  'D016607': NtO(MH='Aflatoxin M1', MS='A 4-hydroxylated metabolite of AFLATOXIN B1, one of the MYCOTOXINS from ASPERGILLUS tainted food. It is associated with LIVER damage and cancer resulting from its P450 activation to the epoxide which alkylates DNA. Toxicity depends on the balance of liver enzymes that activate it (CYTOCHROME P-450) and others that detoxify it (GLUTATHIONE S TRANSFERASE) (Pharmac Ther 50.443 1991). Primates & rat are sensitive while mouse and hamster are tolerant (Canc Res 29.236 1969).'),
  'D016608': NtO(MH='Talin', MS='A 235-kDa cytoplasmic protein that is also found in platelets. It has been localized to regions of cell-substrate adhesion. It binds to INTEGRINS; VINCULIN; and ACTINS and appears to participate in generating a transmembrane connection between the extracellular matrix and the cytoskeleton.'),
  'D016609': NtO(MH='Neoplasms, Second Primary', MS='Abnormal growths of tissue that follow a previous neoplasm but are not metastases of the latter. The second neoplasm may have the same or different histological type and can occur in the same or different organs as the previous neoplasm but in all cases arises from an independent oncogenic event. The development of the second neoplasm may or may not be related to the treatment for the previous neoplasm since genetic risk or predisposing factors may actually be the cause.'),
  'D016610': NtO(MH='Tissue Embedding', MS='The technique of placing cells or tissue in a supporting medium so that thin sections can be cut using a microtome. The medium can be paraffin wax (PARAFFIN EMBEDDING) or plastics (PLASTIC EMBEDDING) such as epoxy resins.'),
  'D016612': NtO(MH='Paraffin Embedding', MS='The infiltrating of tissue specimens with paraffin, as a supporting substance, to prepare for sectioning with a microtome.'),
  'D016613': NtO(MH='Plastic Embedding', MS='The infiltrating of histological specimens with plastics, including acrylic resins, epoxy resins and polyethylene glycol, for support of the tissues in preparation for sectioning with a microtome.'),
  'D016615': NtO(MH='Telomere', MS='A terminal section of a chromosome which has a specialized structure and which is involved in chromosomal replication and stability. Its length is believed to be a few hundred base pairs.'),
  'D016616': NtO(MH='Cryoultramicrotomy', MS='The technique of using a cryostat or freezing microtome, in which the temperature is regulated to -20 degrees Celsius, to cut ultrathin frozen sections for microscopic (usually, electron microscopic) examination.'),
  'D016620': NtO(MH='Enprostil', MS='A synthetic PGE2 analog that has an inhibitory effect on gastric acid secretion, a mucoprotective effect, and a postprandial lowering effect on gastrin. It has been shown to be efficient and safe in the treatment of gastroduodenal ulcers.'),
  'D016622': NtO(MH='Silver Staining', MS='The use of silver, usually silver nitrate, as a reagent for producing contrast or coloration in tissue specimens.'),
  'D016623': NtO(MH='Ion Pumps', MS='A general class of integral membrane proteins that transport ions across a membrane against an electrochemical gradient.'),
  'D016624': NtO(MH='Negative Staining', MS='The technique of washing tissue specimens with a concentrated solution of a heavy metal salt and letting it dry. The specimen will be covered with a very thin layer of the metal salt, being excluded in areas where an adsorbed macromolecule is present. The macromolecules allow electrons from the beam of an electron microscope to pass much more readily than the heavy metal; thus, a reversed or negative image of the molecule is created.'),
  'D016626': NtO(MH='Shadowing Technique, Histology', MS='The technique of spraying a tissue specimen with a thin coat of a heavy metal such as platinum. The specimen is sprayed from an oblique angle, which results in the uneven deposition of the coating. The varying thicknesses create a shadow effect and give a three-dimensional appearance to the specimen.'),
  'D016627': NtO(MH='Oxidopamine', MS='A neurotransmitter analogue that depletes noradrenergic stores in nerve endings and induces a reduction of dopamine levels in the brain. Its mechanism of action is related to the production of cytolytic free-radicals.'),
  'D016628': NtO(MH='Replica Techniques', MS='Methods of preparing tissue specimens for visualization using an electron microscope, usually a scanning electron microscope. The methods involve the creation of exact copies of the specimens by making a mold or cast (i.e., replica) of the specimen.'),
  'D016629': NtO(MH='Esophagectomy', MS='Excision of part (partial) or all (total) of the esophagus. (Dorland, 28th ed)'),
  'D016630': NtO(MH='Rioprostil', MS='A synthetic methylprostaglandin E1 analog that reduces gastric acid secretion and enhances the gastric mucus-bicarbonate barrier. It is effective in the therapy of gastric ulcers and gives significant protection against NSAID-induced gastric mucosal damage. The drug also prevents cyclosporin A-induced damage to endocrine and exocrine pancreatic secretions. It shows a low order of acute toxicity and there is no evidence of embryotoxicity, fetotoxicity, teratogenicity, or mutagenicity in animal studies.'),
  'D016631': NtO(MH='Lewy Bodies', MS='Intracytoplasmic, eosinophilic, round to elongated inclusions found in vacuoles of injured or fragmented neurons. The presence of Lewy bodies is the histological marker of the degenerative changes in LEWY BODY DISEASE and PARKINSON DISEASE but they may be seen in other neurological conditions. They are typically found in the substantia nigra and locus coeruleus but they are also seen in the basal forebrain, hypothalamic nuclei, and neocortex.'),
  'D016632': NtO(MH='Apolipoprotein A-I', MS='The most abundant protein component of HIGH DENSITY LIPOPROTEINS or HDL. This protein serves as an acceptor for CHOLESTEROL released from cells thus promoting efflux of cholesterol to HDL then to the LIVER for excretion from the body (reverse cholesterol transport). It also acts as a cofactor for LECITHIN CHOLESTEROL ACYLTRANSFERASE that forms CHOLESTEROL ESTERS on the HDL particles. Mutations of this gene APOA1 cause HDL deficiency, such as in FAMILIAL ALPHA LIPOPROTEIN DEFICIENCY DISEASE and in some patients with TANGIER DISEASE.'),
  'D016633': NtO(MH='Apolipoprotein A-II', MS='The second most abundant protein component of HIGH DENSITY LIPOPROTEINS or HDL. It has a high lipid affinity and is known to displace APOLIPOPROTEIN A-I from HDL particles and generates a stable HDL complex. ApoA-II can modulate the activation of LECITHIN CHOLESTEROL ACYLTRANSFERASE in the presence of APOLIPOPROTEIN A-I, thus affecting HDL metabolism.'),
  'D016634': NtO(MH='Radiosurgery', MS='A radiological stereotactic technique developed for cutting or destroying tissue by high doses of radiation in place of surgical incisions. It was originally developed for neurosurgery on structures in the brain and its use gradually spread to radiation surgery on extracranial structures as well. The usual rigid needles or probes of stereotactic surgery are replaced with beams of ionizing radiation directed toward a target so as to achieve local tissue destruction.'),
  'D016635': NtO(MH='Universal Precautions', MS='Prudent standard preventive measures to be taken by professional and other health personnel in contact with persons afflicted with a communicable disease, to avoid contracting the disease by contagion or infection. Precautions are especially applicable in the diagnosis and care of AIDS patients.'),
  'D016636': NtO(MH='Telefacsimile', MS='A telecommunication system combining the transmission of a document scanned at a transmitter, its reconstruction at a receiving station, and its duplication there by a copier.'),
  'D016638': NtO(MH='Critical Illness', MS='A disease or state in which death is possible or imminent.'),
  'D016639': NtO(MH='Scleroplasty', MS='Plastic surgery of the SCLERA. This procedure is used frequently to prevent blindness and poor vision in patients, especially children, with MYOPIA.'),
  'D016640': NtO(MH='Diabetes, Gestational', MS='Diabetes mellitus induced by PREGNANCY but resolved at the end of pregnancy. It does not include previously diagnosed diabetics who become pregnant (PREGNANCY IN DIABETICS). Gestational diabetes usually develops in late pregnancy when insulin antagonistic hormones peaks leading to INSULIN RESISTANCE; GLUCOSE INTOLERANCE; and HYPERGLYCEMIA.'),
  'D016642': NtO(MH='Bupropion', MS='A propiophenone-derived antidepressant and antismoking agent that inhibits the uptake of DOPAMINE.'),
  'D016643': NtO(MH='Encephalopathy, Bovine Spongiform', MS='A transmissible spongiform encephalopathy of cattle associated with abnormal prion proteins in the brain. Affected animals develop excitability and salivation followed by ATAXIA. This disorder has been associated with consumption of SCRAPIE infected ruminant derived protein. This condition may be transmitted to humans, where it is referred to as variant or new variant CREUTZFELDT-JAKOB SYNDROME. (Vet Rec 1998 Jul 25;143(41):101-5)'),
  'D016644': NtO(MH='Canthaxanthin', MS='A trans-carotenoid pigment widely distributed in nature. The compound is used as an oral suntanning agent and as a food and drug coloring agent. Oral ingestion of the compound causes canthaxanthin retinopathy.'),
  'D016645': NtO(MH='Aotidae', MS='A family of the New World monkeys inhabiting the forests of South and Central America. There is a single genus (Aotus) and several species occurring in this family, including AOTUS TRIVIRGATUS (Northern night monkeys).'),
  'D016646': NtO(MH='Cebinae', MS='A subfamily of New World monkeys in the family CEBIDAE. They inhabit the forests of South and Central America and comprise members of the genus CEBUS.'),
  'D016647': NtO(MH='Callimico', MS="A genus in the subfamily CALLITRICHINAE, comprising a single species with the common name Goeldi's monkey."),
  'D016648': NtO(MH='Saimirinae', MS='A subfamily of CEBIDAE inhabiting the forests of Central and South America. This subfamily of New World monkeys consists of one genus, the SAIMIRI or squirrel monkeys.'),
  'D016649': NtO(MH='Primary Ovarian Insufficiency', MS="Cessation of ovarian function after MENARCHE but before the age of 40, without or with OVARIAN FOLLICLE depletion. It is characterized by the presence of OLIGOMENORRHEA or AMENORRHEA, elevated GONADOTROPINS, and low ESTRADIOL levels. It is a state of female HYPERGONADOTROPIC HYPOGONADISM. Etiologies include genetic defects, autoimmune processes, chemotherapy, radiation, and infections. The most commonly known genetic cause is the expansion of a CGG repeat to 55 to 199 copies in the 5' untranslated region in the X-linked FMR1 gene."),
  'D016650': NtO(MH='Fluorescein-5-isothiocyanate', MS='Fluorescent probe capable of being conjugated to tissue and proteins. It is used as a label in fluorescent antibody staining procedures as well as protein- and amino acid-binding techniques.'),
  'D016651': NtO(MH='Lithium Carbonate', MS='A lithium salt, classified as a mood-stabilizing agent. Lithium ion alters the metabolism of BIOGENIC MONOAMINES in the CENTRAL NERVOUS SYSTEM, and affects multiple neurotransmission systems.'),
  'D016654': NtO(MH='Genes, RAG-1', MS='Genes involved in activating the enzyme VDJ recombinase. RAG-1 is located on chromosome 11 in humans (chromosome 2 in mice) and is expressed exclusively in maturing lymphocytes.'),
  'D016655': NtO(MH='HIV Core Protein p24', MS='A major core protein of the human immunodeficiency virus encoded by the HIV gag gene. HIV-seropositive individuals mount a significant immune response to p24 and thus detection of antibodies to p24 is one basis for determining HIV infection by ELISA and Western blot assays. The protein is also being investigated as a potential HIV immunogen in vaccines.'),
  'D016656': NtO(MH='Family Planning Policy', MS='A course or method of action selected, usually by a government, to guide and determine present and future decisions on population control by limiting the number of children or controlling fertility, notably through family planning and contraception within the nuclear family.'),
  'D016657': NtO(MH='Cerebral Amyloid Angiopathy', MS='A heterogeneous group of sporadic or familial disorders characterized by AMYLOID deposits in the walls of small and medium sized blood vessels of CEREBRAL CORTEX and MENINGES. Clinical features include multiple, small lobar CEREBRAL HEMORRHAGE; cerebral ischemia (BRAIN ISCHEMIA); and CEREBRAL INFARCTION. Cerebral amyloid angiopathy is unrelated to generalized AMYLOIDOSIS. Amyloidogenic peptides in this condition are nearly always the same ones found in ALZHEIMER DISEASE. (from Kumar: Robbins and Cotran: Pathologic Basis of Disease, 7th ed., 2005)'),
  'D016658': NtO(MH='Psoas Muscles', MS='A powerful flexor of the thigh at the hip joint (psoas major) and a weak flexor of the trunk and lumbar spinal column (psoas minor). Psoas is derived from the Greek "psoa", the plural meaning "muscles of the loin". It is a common site of infection manifesting as abscess (PSOAS ABSCESS). The psoas muscles and their fibers are also used frequently in experiments in muscle physiology.'),
  'D016659': NtO(MH='Psoas Abscess', MS='Abscess of the PSOAS MUSCLES resulting usually from disease of the lumbar vertebrae, with the pus descending into the muscle sheath. The infection is most commonly tuberculous or staphylococcal.'),
  'D016660': NtO(MH='NAD(P)H Dehydrogenase (Quinone)', MS='A flavoprotein that reversibly catalyzes the oxidation of NADH or NADPH by various quinones and oxidation-reduction dyes. The enzyme is inhibited by dicoumarol, capsaicin, and caffeine.'),
  'D016661': NtO(MH='Alouattinae', MS='A subfamily of ATELIDAE inhabiting the forests of Central and South America. The genus ALOUATTA, or howler monkeys, belongs to this subfamily.'),
  'D016662': NtO(MH='Virus Integration', MS='Insertion of viral DNA into host-cell DNA. This includes integration of phage DNA into bacterial DNA; (LYSOGENY); to form a PROPHAGE or integration of retroviral DNA into cellular DNA to form a PROVIRUS.'),
  'D016663': NtO(MH='Cercopithecinae', MS='A subfamily of the Old World monkeys, CERCOPITHECIDAE. They inhabit the forests and savannas of Africa. This subfamily contains the following genera: CERCOCEBUS; CERCOPITHECUS; ERYTHROCEBUS; MACACA; PAPIO; and THEROPITHECUS.'),
  'D016664': NtO(MH='Cercocebus', MS='A genus of the subfamily CERCOPITHECINAE inhabiting the African forests. They are also known as mangabeys.'),
  'D016665': NtO(MH='Cercocebus atys', MS='A species of Old World monkeys from the genera CERCOCEBUS that is important in AIDS research. They may be naturally or experimentally infected with the SIMIAN IMMUNODEFICIENCY VIRUS. They inhabit African forests from Sierra Leone to the Congo Republic.'),
  'D016666': NtO(MH='Fluvoxamine', MS='A selective serotonin reuptake inhibitor that is used in the treatment of DEPRESSION and a variety of ANXIETY DISORDERS.'),
  'D016667': NtO(MH='Bacterial Capsules', MS='An envelope of loose gel surrounding a bacterial cell which is associated with the virulence of pathogenic bacteria. Some capsules have a well-defined border, whereas others form a slime layer that trails off into the medium. Most capsules consist of relatively simple polysaccharides but there are some bacteria whose capsules are made of polypeptides.'),
  'D016669': NtO(MH='Erythrocebus', MS='a genus of Old World monkeys of the family CERCOPITHECINAE that inhabit the forests and savannas of Africa. It contains only one species, ERYTHROCEBUS PATAS, also known as the patas monkey or red monkey.'),
  'D016670': NtO(MH='Gabexate', MS='A serine proteinase inhibitor used therapeutically in the treatment of pancreatitis, disseminated intravascular coagulation (DIC), and as a regional anticoagulant for hemodialysis. The drug inhibits the hydrolytic effects of thrombin, plasmin, and kallikrein, but not of chymotrypsin and aprotinin.'),
  'D016671': NtO(MH='Theropithecus', MS='A genus of Old World monkeys of the subfamily CERCOPITHECINAE, family CERCOPITHECIDAE, that inhabits the mountainous regions of Ethiopia. The genus consists of only one species, Theropithecus gelada.'),
  'D016672': NtO(MH='Zenker Diverticulum', MS='A DIVERTICULUM at the upper end of the ESOPHAGUS through the cricopharyngeal muscle at the junction of the PHARYNX and the esophagus.'),
  'D016673': NtO(MH='Colobinae', MS='A subfamily of the Old World monkeys, CERCOPITHECIDAE, that inhabits the forests of Africa and Asia. The genera COLOBUS (Procolobus; colobus), Nasalis (proboscis monkey), Presbytis (Semnopithecus; leaf monkey), Pygathrix (Rhinopithecus; snub-nosed monkey), and Simias (pig-tailed langur) all belong to this subfamily.'),
  'D016674': NtO(MH='Hylobatidae', MS='A family of the suborder HAPLORHINI comprising four genera, HYLOBATES (dwarf gibbons); Hoolock; Nomascus; and Symphalangus.'),
  'D016675': NtO(MH='Scandentia', MS='An order of the class MAMMALS that consists of one family, TUPAIIDAE (tree shrews), 5 genera (one of which is TUPAIA), and 16 species. Their recent distribution is from India to the Philippines, southern China to Java, Borneo, Sumatra, Bali, and other islands in those regions.'),
  'D016676': NtO(MH='Macrophages, Alveolar', MS='Round, granular, mononuclear phagocytes found in the alveoli of the lungs. They ingest small inhaled particles resulting in degradation and presentation of the antigen to immunocompetent cells.'),
  'D016677': NtO(MH='Tocainide', MS='An antiarrhythmic agent which exerts a potential- and frequency-dependent block of SODIUM CHANNELS.'),
  'D016678': NtO(MH='Genome', MS='The genetic complement of an organism, including all of its GENES, as represented in its DNA, or in some cases, its RNA.'),
  'D016679': NtO(MH='Genome, Viral', MS='The complete genetic complement contained in a DNA or RNA molecule in a virus.'),
  'D016680': NtO(MH='Genome, Bacterial', MS='The genetic complement of a BACTERIA as represented in its DNA.'),
  'D016681': NtO(MH='Genome, Fungal', MS='The complete gene complement contained in a set of chromosomes in a fungus.'),
  'D016682': NtO(MH='Lupus Coagulation Inhibitor', MS='An antiphospholipid antibody found in association with systemic lupus erythematosus (LUPUS ERYTHEMATOSUS, SYSTEMIC;), ANTIPHOSPHOLIPID SYNDROME; and in a variety of other diseases as well as in healthy individuals. In vitro, the antibody interferes with the conversion of prothrombin to thrombin and prolongs the partial thromboplastin time. In vivo, it exerts a procoagulant effect resulting in thrombosis mainly in the larger veins and arteries. It further causes obstetrical complications, including fetal death and spontaneous abortion, as well as a variety of hematologic and neurologic complications.'),
  'D016683': NtO(MH='Supine Position', MS='The posture of an individual lying face up.'),
  'D016684': NtO(MH='Prone Position', MS='The posture of an individual lying face down.'),
  'D016685': NtO(MH='Mitomycin', MS='An antineoplastic antibiotic produced by Streptomyces caespitosus. It is one of the bi- or tri-functional ALKYLATING AGENTS causing cross-linking of DNA and inhibition of DNA synthesis.'),
  'D016686': NtO(MH='Monocrotaline', MS='A pyrrolizidine alkaloid and a toxic plant constituent that poisons livestock and humans through the ingestion of contaminated grains and other foods. The alkaloid causes pulmonary artery hypertension, right ventricular hypertrophy, and pathological changes in the pulmonary vasculature. Significant attenuation of the cardiopulmonary changes are noted after oral magnesium treatment.'),
  'D016687': NtO(MH='Polydioxanone', MS='An absorbable suture material used also as ligating clips, as pins for internal fixation of broken bones, and as ligament reinforcement for surgically managed ligament injuries. Its promising characteristics are elasticity, complete biodegradability, and lack of side effects such as infections.'),
  'D016688': NtO(MH='Mice, Inbred NOD', MS='A strain of non-obese diabetic mice developed in Japan that has been widely studied as a model for T-cell-dependent autoimmune insulin-dependent diabetes mellitus in which insulitis is a major histopathologic feature, and in which genetic susceptibility is strongly MHC-linked.'),
  'D016689': NtO(MH='Trypanosoma vivax', MS='An active blood parasite that is present in practically all domestic animals in Africa, the West Indies, and parts of Central and South America. In Africa, the insect vector is the tsetse fly. In other countries, infection is by mechanical means indicating that the parasites have been introduced to these countries and have been able to maintain themselves in spite of the lack of a suitable intermediate host. It is a cause of nagana, the severity of which depends on the species affected.'),
  'D016692': NtO(MH='Receptors, Antigen, T-Cell, gamma-delta', MS='T-cell receptors composed of CD3-associated gamma and delta polypeptide chains and expressed primarily in CD4-/CD8- T-cells. The receptors appear to be preferentially located in epithelial sites and probably play a role in the recognition of bacterial antigens. The T-cell receptor gamma/delta chains are separate and not related to the gamma and delta chains which are subunits of CD3 (see ANTIGENS, CD3).'),
  'D016693': NtO(MH='Receptors, Antigen, T-Cell, alpha-beta', MS='T-cell receptors composed of CD3-associated alpha and beta polypeptide chains and expressed primarily in CD4+ or CD8+ T-cells. Unlike immunoglobulins, the alpha-beta T-cell receptors recognize antigens only when presented in association with major histocompatibility (MHC) molecules.'),
  'D016694': NtO(MH='Gene Rearrangement, delta-Chain T-Cell Antigen Receptor', MS='Ordered rearrangement of T-cell variable gene regions coding for the delta-chain of antigen receptors.'),
  'D016695': NtO(MH='Glycosyltransferases', MS='Enzymes that catalyze the transfer of glycosyl groups to an acceptor. Most often another carbohydrate molecule acts as an acceptor, but inorganic phosphate can also act as an acceptor, such as in the case of PHOSPHORYLASES. Some of the enzymes in this group also catalyze hydrolysis, which can be regarded as transfer of a glycosyl group from the donor to water. Subclasses include the HEXOSYLTRANSFERASES; PENTOSYLTRANSFERASES; SIALYLTRANSFERASES; and those transferring other glycosyl groups. EC 2.4.'),
  'D016697': NtO(MH='Herpes Zoster Oticus', MS='A syndrome characterized by facial palsy in association with a herpetic eruption of the external auditory meatus. This may occasionally be associated with tinnitus, vertigo, deafness, severe otalgia, and inflammation of the pinna. The condition is caused by reactivation of a latent HERPESVIRUS 3, HUMAN infection which causes inflammation of the facial and vestibular nerves, and may occasionally involve additional cranial nerves. (From Adams et al., Principles of Neurology, 6th ed, p757)'),
  'D016698': NtO(MH='Technetium Tc 99m Pyrophosphate', MS='A radionuclide imaging agent used primarily in scintigraphy or tomography of the heart to evaluate the extent of the necrotic myocardial process. It has also been used in noninvasive tests for the distribution of organ involvement in different types of amyloidosis and for the evaluation of muscle necrosis in the extremities.'),
  'D016699': NtO(MH='Tarsiidae', MS='The single family of PRIMATES in the infraorder TARSII, suborder HAPLORHINI. It is comprised of one genus, Tarsius, that inhabits southern Sumatra, Borneo, Sulawesi, and the Philippines.'),
  'D016700': NtO(MH='Encainide', MS='One of the ANTI-ARRHYTHMIA AGENTS, it blocks VOLTAGE-GATED SODIUM CHANNELS and slows conduction within the His-Purkinje system and MYOCARDIUM.'),
  'D016701': NtO(MH='Guinea', MS='A republic in western Africa, south of SENEGAL and MALI, east of GUINEA-BISSAU. Its capital is Conakry.'),
  'D016703': NtO(MH='Equatorial Guinea', MS='A republic in central Africa, bordering the Bay of Biafra, CAMEROON is to the north and GABON to the south. Its capital is Malabo.'),
  'D016704': NtO(MH='Synapsins', MS='A family of synaptic vesicle-associated proteins involved in the short-term regulation of NEUROTRANSMITTER release. Synapsin I, the predominant member of this family, links SYNAPTIC VESICLES to ACTIN FILAMENTS in the presynaptic nerve terminal. These interactions are modulated by the reversible PHOSPHORYLATION of synapsin I through various signal transduction pathways. The protein is also a substrate for cAMP- and CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASES. It is believed that these functional properties are also shared by synapsin II.'),
  'D016706': NtO(MH="Still's Disease, Adult-Onset", MS='Systemic-onset rheumatoid arthritis in adults. It differs from classical rheumatoid arthritis in that it is more often marked by acute febrile onset, and generalized lymphadenopathy and hepatosplenomegaly are more prominent.'),
  'D016707': NtO(MH='Tissue Fixation', MS='The technique of using FIXATIVES in the preparation of cytologic, histologic, or pathologic specimens for the purpose of maintaining the existing form and structure of all the constituent elements.'),
  'D016708': NtO(MH='Synaptophysin', MS='A MARVEL domain-containing protein found in the presynaptic vesicles of NEURONS and NEUROENDOCRINE CELLS. It is commonly used as an immunocytochemical marker for neuroendocrine differentiation.'),
  'D016709': NtO(MH='Yin-Yang', MS='In Chinese philosophy and religion, two principles, one negative, dark, and feminine (yin) and one positive, bright, and masculine (yang), from whose interaction all things are produced and all things are dissolved. As a concept the two polar elements referred originally to the shady and sunny sides of a valley or a hill but it developed into the relationship of any contrasting pair: those specified above (female-male, etc.) as well as cold-hot, wet-dry, weak-strong, etc. It is not a distinct system of thought by itself but permeates Chinese life and thought. A balance of yin and yang is essential to health. A deficiency of either principle can manifest as disease. (Encyclopedia Americana)'),
  'D016710': NtO(MH='Yin Deficiency', MS='In the YIN-YANG system of philosophy and medicine, an insufficiency of body fluid (called yinxu), manifesting often as irritability, thirst, constipation, etc. (The Pinyin Chinese-English Dictionary, 1979).'),
  'D016711': NtO(MH='Yang Deficiency', MS='In the YIN-YANG system of philosophy and medicine, a lack of vital energy (called yangxu in Chinese). It manifests itself in various systemic and organic diseases. (The Pinyin Chinese-English Dictionary, 1979)'),
  'D016712': NtO(MH='Mupirocin', MS='A topically used antibiotic from a strain of Pseudomonas fluorescens. It has shown excellent activity against gram-positive staphylococci and streptococci. The antibiotic is used primarily for the treatment of primary and secondary skin disorders, nasal infections, and wound healing.'),
  'D016713': NtO(MH='Ritanserin', MS='A selective and potent serotonin-2 antagonist that is effective in the treatment of a variety of syndromes related to anxiety and depression. The drug also improves the subjective quality of sleep and decreases portal pressure.'),
  'D016715': NtO(MH='Proteus Syndrome', MS='Hamartoneoplastic malformation syndrome of uncertain etiology characterized by partial GIGANTISM of the hands and/or feet, asymmetry of the limbs, plantar hyperplasia, hemangiomas (HEMANGIOMA), lipomas (LIPOMA), lymphangiomas (LYMPHANGIOMA), epidermal NEVI; MACROCEPHALY; cranial HYPEROSTOSIS, and long-bone overgrowth. Joseph Merrick, the so-called "elephant man", apparently suffered from Proteus syndrome and not NEUROFIBROMATOSIS, a disorder with similar characteristics.'),
  'D016716': NtO(MH='PC12 Cells', MS='A CELL LINE derived from a PHEOCHROMOCYTOMA of the rat ADRENAL MEDULLA. PC12 cells stop dividing and undergo terminal differentiation when treated with NERVE GROWTH FACTOR, making the line a useful model system for NERVE CELL differentiation.'),
  'D016717': NtO(MH='Sphincterotomy, Endoscopic', MS="Incision of SPHINCTER OF ODDI or VATER'S AMPULLA performed by inserting a sphincterotome through DUODENOSCOPE often following or performed during ERCP (ENDOSCOPIC RETROGRADE CHOLANGIOPANCREATOGRAPHY)."),
  'D016718': NtO(MH='Arachidonic Acid', MS='An unsaturated, essential fatty acid. It is found in animal and human fat as well as in the liver, brain, and glandular organs, and is a constituent of animal phosphatides. It is formed by the synthesis from dietary linoleic acid and is a precursor in the biosynthesis of prostaglandins, thromboxanes, and leukotrienes.'),
  'D016719': NtO(MH='Radioimmunodetection', MS='Use of radiolabeled antibodies for diagnostic imaging of neoplasms. Antitumor antibodies are labeled with diverse radionuclides including iodine-131, iodine-123, indium-111, or technetium-99m and injected into the patient. Images are obtained by a scintillation camera.'),
  'D016720': NtO(MH='Pneumocystis Infections', MS='Infections with species in the genus PNEUMOCYSTIS, a fungus causing interstitial plasma cell pneumonia (PNEUMONIA, PNEUMOCYSTIS) and other infections in humans and other MAMMALS. Immunocompromised patients, especially those with AIDS, are particularly susceptible to these infections. Extrapulmonary sites are rare but seen occasionally.'),
  'D016721': NtO(MH='Radiation Leukemia Virus', MS='A strain of Murine leukemia virus (LEUKEMIA VIRUS, MURINE) isolated from radiation-induced lymphomas in C57BL mice. It is leukemogenic, thymotrophic, can be transmitted vertically, and replicates only in vivo.'),
  'D016722': NtO(MH='Cermet Cements', MS='Fluoride-releasing restorative materials made by the sintering of metal (usually silver) particles to glass ionomer powder. Glass ionomers are fluoride-releasing cements that are not very durable. Sintering of the metal particles is a means of improving those physical properties that will make the glass ionomer cement more durable.'),
  'D016723': NtO(MH='Bone Remodeling', MS="The continuous turnover of BONE MATRIX and mineral that involves first an increase in BONE RESORPTION (osteoclastic activity) and later, reactive BONE FORMATION (osteoblastic activity). The process of bone remodeling takes place in the adult skeleton at discrete foci. The process ensures the mechanical integrity of the skeleton throughout life and plays an important role in calcium HOMEOSTASIS. An imbalance in the regulation of bone remodeling's two contrasting events, bone resorption and bone formation, results in many of the metabolic bone diseases, such as OSTEOPOROSIS."),
  'D016724': NtO(MH='Empyema, Pleural', MS='Suppurative inflammation of the pleural space.'),
  'D016726': NtO(MH='Plasma Cell Granuloma, Pulmonary', MS='A tumor-like inflammatory lesion of the lung that is composed of PLASMA CELLS and fibrous tissue. It is also known as an inflammatory pseudotumor, often with calcification and measuring between 2 and 5 cm in diameter.'),
  'D016727': NtO(MH='Orbital Pseudotumor', MS='A nonspecific tumor-like inflammatory lesion in the ORBIT of the eye. It is usually composed of mature LYMPHOCYTES; PLASMA CELLS; MACROPHAGES; LEUKOCYTES with varying degrees of FIBROSIS. Orbital pseudotumors are often associated with inflammation of the extraocular muscles (ORBITAL MYOSITIS) or inflammation of the lacrimal glands (DACRYOADENITIS).'),
  'D016728': NtO(MH='Self-Injurious Behavior', MS='Behavior in which persons hurt or harm themselves without the motive of suicide or of sexual deviation.'),
  'D016729': NtO(MH='Leuprolide', MS='A potent synthetic long-acting agonist of GONADOTROPIN-RELEASING HORMONE that regulates the synthesis and release of pituitary gonadotropins, LUTEINIZING HORMONE and FOLLICLE STIMULATING HORMONE.'),
  'D016730': NtO(MH='Program Development', MS='The process of formulating, improving, and expanding educational, managerial, or service-oriented work plans (excluding computer program development).'),
  'D016731': NtO(MH='Erythema Infectiosum', MS='Contagious infection with human B19 Parvovirus most commonly seen in school age children and characterized by fever, headache, and rashes of the face, trunk, and extremities. It is often confused with RUBELLA.'),
  'D016732': NtO(MH='Parvovirus B19, Human', MS='The type species of ERYTHROVIRUS and the etiological agent of ERYTHEMA INFECTIOSUM, a disease most commonly seen in school-age children.'),
  'D016733': NtO(MH='Rubber Dams', MS='Sheets of latex rubber punched and placed over the teeth during dental procedures to isolate the field of operation from the rest of the oral cavity (Jablonski; Illustrated Dictionary of Dentistry, 1982). Rubber dams are useful in preventing the swallowing of instruments or restorations during dental work.'),
  'D016734': NtO(MH='Dental Clasps', MS='Metal devices for fastening together two or more parts of dental prostheses for stabilizing or retaining them by attachment to abutment teeth. For a precision attachment for a partial denture DENTURE PRECISION ATTACHMENT is available.'),
  'D016735': NtO(MH='Munchausen Syndrome by Proxy', MS='A phenomenon in which symptoms of a disease are fabricated by an individual other than the patient causing unnecessary, and often painful, physical examinations and treatments. This syndrome is considered a form of CHILD ABUSE, since another individual, usually a parent, is the source of the fabrication of symptoms and presents the child for medical care.'),
  'D016736': NtO(MH='Antiphospholipid Syndrome', MS='The presence of antibodies directed against phospholipids (ANTIBODIES, ANTIPHOSPHOLIPID). The condition is associated with a variety of diseases, notably systemic lupus erythematosus and other connective tissue diseases, thrombopenia, and arterial or venous thromboses. In pregnancy it can cause abortion. Of the phospholipids, the cardiolipins show markedly elevated levels of anticardiolipin antibodies (ANTIBODIES, ANTICARDIOLIPIN). Present also are high levels of lupus anticoagulant (LUPUS COAGULATION INHIBITOR).'),
  'D016737': NtO(MH='Proctocolectomy, Restorative', MS='A surgical procedure involving the excision of the COLON and RECTUM and the formation of an ILEOANAL RESERVOIR (pouch). In patients with intestinal diseases, such as ulcerative colitis, this procedure avoids the need for an OSTOMY by allowing for transanal defecation.'),
  'D016738': NtO(MH='Alagille Syndrome', MS='A multisystem disorder that is characterized by aplasia of intrahepatic bile ducts (BILE DUCTS, INTRAHEPATIC), and malformations in the cardiovascular system, the eyes, the vertebral column, and the facies. Major clinical features include JAUNDICE, and congenital heart disease with peripheral PULMONARY STENOSIS. Alagille syndrome may result from heterogeneous gene mutations, including mutations in JAG1 on CHROMOSOME 20 (Type 1) and NOTCH2 on CHROMOSOME 1 (Type 2).'),
  'D016739': NtO(MH='Behavior, Addictive', MS='The observable, measurable, and often pathological activity of an organism that portrays its inability to overcome a habit resulting in an insatiable craving for a substance or for performing certain acts. The addictive behavior includes the emotional and physical overdependence on the object of habit in increasing amount or frequency.'),
  'D016740': NtO(MH='Meridians', MS='Classical loci in ACUPUNCTURE. They are main and collateral channels, regarded as a network of passages, through which vital energy (Qi) circulates and along which acupoints (ACUPUNCTURE POINTS) are distributed. The meridians are a series of 14 lines upon which more than 400 acupoints are located on the body. (The Pinyin Chinese-English Dictionary, p. 359; Dr. Wu Lancheng, Academy of Traditional Chinese Medicine, Beijing)'),
  'D016741': NtO(MH='Urinary Sphincter, Artificial', MS='An artifical implanted device, usually in the form of an inflatable silicone cuff, inserted in or around the bladder neck in the surgical treatment of urinary incontinence caused by sphincter weakness. Often it is placed around the bulbous urethra in adult males. The artificial urinary sphincter is considered an alternative to urinary diversion.'),
  'D016742': NtO(MH='Preconception Care', MS="An organized and comprehensive program of health care that identifies and reduces a woman's reproductive risks before conception through risk assessment, health promotion, and interventions. Preconception care programs may be designed to include the male partner in providing counseling and educational information in preparation for fatherhood, such as genetic counseling and testing, financial and family planning, etc. This concept is different from PRENATAL CARE, which occurs during pregnancy."),
  'D016743': NtO(MH='Mental Competency', MS="The ability to understand the nature and effect of the act in which the individual is engaged. (From Black's Law Dictionary, 6th ed)."),
  'D016744': NtO(MH='Minor Lymphocyte Stimulatory Loci', MS='Genetic loci responsible for the encoding of minor lymphocyte stimulatory antigens. There are at least two unlinked loci (in the mouse) and they appear to be separate from the MAJOR HISTOCOMPATIBILITY COMPLEX and MINOR HISTOCOMPATIBILITY LOCI. The mouse mammary tumor virus (see MAMMARY TUMOR VIRUS, MOUSE) has the ability to integrate into these loci. The antigens induce strong T-cell proliferative responses in mixed lymphocyte reactions.'),
  'D016745': NtO(MH='Root Planing', MS='A procedure for smoothing of the roughened root surface or cementum of a tooth after subgingival curettage or scaling, as part of periodontal therapy.'),
  'D016746': NtO(MH='Transcultural Nursing', MS='A nursing specialty created to answer the need for developing a global perspective in the practice of nursing in a world of interdependent nations and people. The focus of this nursing discipline is on the integration of international and transcultural content into the training. Courses include study in the area of cultural differences, nursing in other countries, and international health issues and organizations, as an example.'),
  'D016747': NtO(MH='Minor Lymphocyte Stimulatory Antigens', MS='Endogenous superantigens responsible for inducing strong proliferative responses in T-cells in mixed lymphocyte reactions (see LYMPHOCYTE CULTURE TEST, MIXED). They are encoded by mouse mammary tumor viruses that have integrated into the germ line as DNA proviruses (MINOR LYMPHOCYTE STIMULATORY LOCI).'),
  'D016748': NtO(MH='Venous Cutdown', MS='Creation of a small incised opening in a vein to permit the passage of a needle or cannula for withdrawal of blood, administration of medication, or in diagnostic or therapeutic catheterization. (Dorland, 28th ed.; Stedman, 26th ed.)'),
  'D016749': NtO(MH='National Practitioner Data Bank', MS="A databank established by the Health Care Quality Improvement Act of 1986 authorizing the Department of Health and Human Services to collect and release information on the professional competence and conduct of physicians, dentists, nurses, and other health care practitioners. The data include adverse actions on physicians' malpractice, licensure, hospital privileges, concealing of pertinent information, and the like."),
  'D016750': NtO(MH='Stiff-Person Syndrome', MS='A condition characterized by persistent spasms (SPASM) involving multiple muscles, primarily in the lower limbs and trunk. The illness tends to occur in the fourth to sixth decade of life, presenting with intermittent spasms that become continuous. Minor sensory stimuli, such as noise and light touch, precipitate severe spasms. Spasms do not occur during sleep and only rarely involve cranial muscles. Respiration may become impaired in advanced cases. (Adams et al., Principles of Neurology, 6th ed, p1492; Neurology 1998 Jul;51(1):85-93)'),
  'D016751': NtO(MH='Hepatitis E', MS='Acute INFLAMMATION of the LIVER in humans; caused by HEPATITIS E VIRUS, a non-enveloped single-stranded RNA virus. Similar to HEPATITIS A, its incubation period is 15-60 days and is enterically transmitted, usually by fecal-oral transmission.'),
  'D016752': NtO(MH='Hepatitis E virus', MS='A positive-stranded RNA virus species in the genus HEPEVIRUS, causing enterically-transmitted non-A, non-B hepatitis (HEPATITIS E).'),
  'D016753': NtO(MH='Interleukin-10', MS='A cytokine produced by a variety of cell types, including T-LYMPHOCYTES; MONOCYTES; DENDRITIC CELLS; and EPITHELIAL CELLS that exerts a variety of effects on immunoregulation and INFLAMMATION. Interleukin-10 combines with itself to form a homodimeric molecule that is the biologically active form of the protein.'),
  'D016754': NtO(MH='Boron Neutron Capture Therapy', MS='A technique for the treatment of neoplasms, especially gliomas and melanomas in which boron-10, an isotope, is introduced into the target cells followed by irradiation with thermal neutrons.'),
  'D016755': NtO(MH='Proto-Oncogene Proteins c-jun', MS='Cellular DNA-binding proteins encoded by the c-jun genes (GENES, JUN). They are involved in growth-related transcriptional control. There appear to be three distinct functions: dimerization (with c-fos), DNA-binding, and transcriptional activation. Oncogenic transformation can take place by constitutive expression of c-jun.'),
  'D016756': NtO(MH='Immunoglobulins, Intravenous', MS='Immunoglobulin preparations used in intravenous infusion, containing primarily IMMUNOGLOBULIN G. They are used to treat a variety of diseases associated with decreased or abnormal immunoglobulin levels including pediatric AIDS; primary HYPERGAMMAGLOBULINEMIA; SCID; CYTOMEGALOVIRUS infections in transplant recipients, LYMPHOCYTIC LEUKEMIA, CHRONIC; Kawasaki syndrome, infection in neonates, and IDIOPATHIC THROMBOCYTOPENIC PURPURA.'),
  'D016757': NtO(MH='Death, Sudden, Cardiac', MS="Unexpected rapid natural death due to cardiovascular collapse within one hour of initial symptoms. It is usually caused by the worsening of existing heart diseases. The sudden onset of symptoms, such as CHEST PAIN and CARDIAC ARRHYTHMIAS, particularly VENTRICULAR TACHYCARDIA, can lead to the loss of consciousness and cardiac arrest followed by biological death. (from Braunwald's Heart Disease: A Textbook of Cardiovascular Medicine, 7th ed., 2005)"),
  'D016758': NtO(MH='Genes, jun', MS='Retrovirus-associated DNA sequences (jun) originally isolated from the avian sarcoma virus 17 (ASV 17). The proto-oncogene jun (c-jun) codes for a nuclear protein which is involved in growth-related transcriptional control. Insertion of c-jun into ASV-17 or the constitutive expression of the c-jun protein produces tumorgenicity. The human c-jun gene is located at 1p31-32 on the short arm of chromosome 1.'),
  'D016759': NtO(MH='Oncogene Protein p65(gag-jun)', MS='Transforming protein coded by jun oncogenes (GENES, JUN). This is a gag-onc fusion protein of about 65 kDa derived from avian sarcoma virus. v-jun lacks a negative regulatory domain that regulates transcription in c-jun.'),
  'D016760': NtO(MH='Proto-Oncogene Proteins c-fos', MS='Cellular DNA-binding proteins encoded by the c-fos genes (GENES, FOS). They are involved in growth-related transcriptional control. c-fos combines with c-jun (PROTO-ONCOGENE PROTEINS C-JUN) to form a c-fos/c-jun heterodimer (TRANSCRIPTION FACTOR AP-1) that binds to the TRE (TPA-responsive element) in promoters of certain genes.'),
  'D016761': NtO(MH='Oncogene Proteins v-fos', MS='Transforming proteins coded by fos oncogenes. These proteins have been found in the Finkel-Biskis-Jinkins (FBJ-MSV) and Finkel-Biskis-Reilly (FBR-MSV) murine sarcoma viruses which induce osteogenic sarcomas in mice. The FBJ-MSV v-fos gene encodes a p55-kDa protein and the FBR-MSV v-fos gene encodes a p75-kDa fusion protein.'),
  'D016762': NtO(MH='Genes, fos', MS='Retrovirus-associated DNA sequences (fos) originally isolated from the Finkel-Biskis-Jinkins (FBJ-MSV) and Finkel-Biskis-Reilly (FBR-MSV) murine sarcoma viruses. The proto-oncogene protein c-fos codes for a nuclear protein which is involved in growth-related transcriptional control. The insertion of c-fos into FBJ-MSV or FBR-MSV induces osteogenic sarcomas in mice. The human c-fos gene is located at 14q21-31 on the long arm of chromosome 14.'),
  'D016764': NtO(MH='Cell Polarity', MS='Orientation of intracellular structures especially with respect to the apical and basolateral domains of the plasma membrane. Polarized cells must direct proteins from the Golgi apparatus to the appropriate domain since tight junctions prevent proteins from diffusing between the two domains.'),
  'D016765': NtO(MH='Coronavirus, Feline', MS='A species of CORONAVIRUS infecting cats of all ages and commonly found in catteries and zoos. Cats are often found carrying the virus but only a small proportion develop disease. Feline coronavirus and Feline infectious peritonitis virus (FIPV) are virtually the same virus in genetic and antigenetic terms, and are morphologically indistinguishable. Since they only differ in their disease potential (with FIPV causing a more serious illness), they are considered biotypes of each other.'),
  'D016766': NtO(MH='Feline Infectious Peritonitis', MS='Common coronavirus infection of cats caused by the feline infectious peritonitis virus (CORONAVIRUS, FELINE). The disease is characterized by a long incubation period, fever, depression, loss of appetite, wasting, and progressive abdominal enlargement. Infection of cells of the monocyte-macrophage lineage appears to be essential in FIP pathogenesis.'),
  'D016767': NtO(MH='Caroli Disease', MS='Congenital cystic dilatation of the intrahepatic bile ducts (BILE DUCTS, INTRAHEPATIC). It consists of 2 types: simple Caroli disease is characterized by bile duct dilatation (ectasia) alone; and complex Caroli disease is characterized by bile duct dilatation with extensive hepatic fibrosis and portal hypertension (HYPERTENSION, PORTAL). Benign renal tubular ectasia is associated with both types of Caroli disease.'),
  'D016769': NtO(MH='Embolism and Thrombosis', MS='A collective term for pathological conditions which are caused by the formation of a blood clot (THROMBUS) in a blood vessel, or by blocking of a blood vessel with an EMBOLUS, undissolved materials in the blood stream.'),
  'D016770': NtO(MH='Ciliophora Infections', MS='Infections with protozoa of the phylum CILIOPHORA.'),
  'D016773': NtO(MH='Leishmaniasis, Cutaneous', MS='An endemic disease that is characterized by the development of single or multiple localized lesions on exposed areas of skin that typically ulcerate. The disease has been divided into Old and New World forms. Old World leishmaniasis is separated into three distinct types according to epidemiology and clinical manifestations and is caused by species of the L. tropica and L. aethiopica complexes as well as by species of the L. major genus. New World leishmaniasis, also called American leishmaniasis, occurs in South and Central America and is caused by species of the L. mexicana or L. braziliensis complexes.'),
  'D016774': NtO(MH='Leishmaniasis, Diffuse Cutaneous', MS='A form of LEISHMANIASIS, CUTANEOUS caused by Leishmania aethiopica in Ethiopia and Kenya, L. pifanoi in Venezuela, L. braziliensis in South America, and L. mexicana in Central America. This disease is characterized by massive dissemination of skin lesions without visceral involvement.'),
  'D016776': NtO(MH='Blastocystis Infections', MS='Infections with organisms of the genus BLASTOCYSTIS. The species B. hominis is responsible for most infections. Parasitologic surveys have generally found small numbers of this species in human stools, but higher positivity rates and organism numbers in AIDS patients and other immunosuppressed patients (IMMUNOCOMPROMISED HOST). Symptoms include ABDOMINAL PAIN; DIARRHEA; CONSTIPATION; VOMITING; and FATIGUE.'),
  'D016778': NtO(MH='Malaria, Falciparum', MS='Malaria caused by PLASMODIUM FALCIPARUM. This is the severest form of malaria and is associated with the highest levels of parasites in the blood. This disease is characterized by irregularly recurring febrile paroxysms that in extreme cases occur with acute cerebral, renal, or gastrointestinal manifestations.'),
  'D016779': NtO(MH='Malaria, Cerebral', MS='A condition characterized by somnolence or coma in the presence of an acute infection with PLASMODIUM FALCIPARUM (and rarely other Plasmodium species). Initial clinical manifestations include HEADACHES; SEIZURES; and alterations of mentation followed by a rapid progression to COMA. Pathologic features include cerebral capillaries filled with parasitized erythrocytes and multiple small foci of cortical and subcortical necrosis. (From Adams et al., Principles of Neurology, 6th ed, p136)'),
  'D016780': NtO(MH='Malaria, Vivax', MS='Malaria caused by PLASMODIUM VIVAX. This form of malaria is less severe than MALARIA, FALCIPARUM, but there is a higher probability for relapses to occur. Febrile paroxysms often occur every other day.'),
  'D016781': NtO(MH='Toxoplasmosis, Cerebral', MS='Infections of the BRAIN caused by the protozoan TOXOPLASMA gondii that primarily arise in individuals with IMMUNOLOGIC DEFICIENCY SYNDROMES (see also AIDS-RELATED OPPORTUNISTIC INFECTIONS). The infection may involve the brain diffusely or form discrete abscesses. Clinical manifestations include SEIZURES, altered mentation, headache, focal neurologic deficits, and INTRACRANIAL HYPERTENSION. (From Joynt, Clinical Neurology, 1998, Ch27, pp41-3)'),
  'D016782': NtO(MH='Apicomplexa', MS='A phylum of unicellular parasitic EUKARYOTES characterized by the presence of complex apical organelles generally consisting of a conoid that aids in penetrating host cells, rhoptries that possibly secrete a proteolytic enzyme, and subpellicular microtubules that may be related to motility.'),
  'D016783': NtO(MH='Eucoccidiida', MS='An order of parasitic protozoa found in blood cells and epithelial cells of vertebrates and invertebrates. Life cycles involve both sexual and asexual phases.'),
  'D016785': NtO(MH='Cryptosporidium parvum', MS='A species of parasitic protozoa that infects humans and most domestic mammals. Its oocysts measure five microns in diameter. These organisms exhibit alternating cycles of sexual and asexual reproduction.'),
  'D016786': NtO(MH='Eimeria tenella', MS='A species of coccidian protozoa that mainly infects domestic poultry.'),
  'D016787': NtO(MH='Haemosporida', MS='An order of heteroxenous protozoa in which the macrogamete and microgamont develop independently. A conoid is usually absent.'),
  'D016788': NtO(MH='Plasmodium chabaudi', MS='A protozoan parasite of rodents transmitted by the mosquito Anopheles stephensi.'),
  'D016789': NtO(MH='Plasmodium cynomolgi', MS='A protozoan parasite that occurs naturally in the macaque. It is similar to PLASMODIUM VIVAX and produces a type of malaria similar to vivax malaria (MALARIA, VIVAX). This species has been found to give rise to both natural and experimental human infections.'),
  'D016790': NtO(MH='Plasmodium knowlesi', MS='A protozoan parasite from Southeast Asia that causes monkey malaria. It is naturally acquired by man in Malaysia and can also be transmitted experimentally to humans.'),
  'D016791': NtO(MH='Piroplasmia', MS='A subclass of heteroxenous parasitic protozoa usually transmitted by ticks. Its organisms are rod-shaped or amoeboid with conoid, oocysts, spores, and pseudocysts absent. Locomotion is by body flexion, gliding, or, in sexual stages, by large axopodium-like organelles.'),
  'D016792': NtO(MH='Piroplasmida', MS='An order of protozoa comprising heteroxenous tick-borne blood parasites. Representative genera include BABESIA, Dactylosoma, and THEILERIA.'),
  'D016793': NtO(MH='Babesia bovis', MS='A species of protozoa that is a cause of bovine babesiosis. Ticks of the genera Boophilus, Rhipicephalus, and IXODES are the chief vectors.'),
  'D016794': NtO(MH='Theileria', MS='A genus of tick-borne protozoa parasitic in the lymphocytes, erythrocytes, and endothelial cells of mammals. Its organisms multiply asexually and then invade erythrocytes, where they undergo no further reproduction until ingested by a transmitting tick.'),
  'D016796': NtO(MH='Theileria annulata', MS='A protozoan parasite causing tropical theileriasis in cattle. It is transmitted by ticks of the Hyalomma genus.'),
  'D016797': NtO(MH='Theileria parva', MS='A protozoan parasite that is the etiologic agent of East Coast fever (THEILERIASIS). Transmission is by ticks of the Physicephalus and Hyalomma genera.'),
  'D016798': NtO(MH='Ciliophora', MS='A phylum of EUKARYOTES characterized by the presence of cilia at some time during the life cycle. It comprises three classes: KINETOFRAGMINOPHOREA; OLIGOHYMENOPHOREA; and POLYMENOPHOREA.'),
  'D016799': NtO(MH='Kinetofragminophorea', MS='A class of ciliate protozoa. Prominent cytopharyngeal apparatus is characteristic of this class as is only a slight distinction between oral and somatic cilia.'),
  'D016800': NtO(MH='Trichostomatida', MS='An order of ciliate protozoa in which the cytostome is usually situated at the base of an oral groove, the wall of which bears dense cilia.'),
  'D016801': NtO(MH='Trichostomatina', MS='A suborder of ciliate protozoa characterized by non-reduction of the somatic ciliature.'),
  'D016802': NtO(MH='Oligohymenophorea', MS='A class of ciliate protozoa. Characteristics include the presence of a well developed oral apparatus and oral cilia being clearly distinct from somatic cilia.'),
  'D016803': NtO(MH='Hymenostomatida', MS='An order of ciliate protozoa. Characteristics include a ventral oral area and a well-defined buccal cavity. It comprises three suborders: TETRAHYMENINA, Ophryoglenina, and PENICULINA.'),
  'D016804': NtO(MH='Peniculina', MS='A suborder of protozoa characterized by peniculi, which are often situated deep in the buccal cavity, and by fusiform trichocysts.'),
  'D016805': NtO(MH='Paramecium tetraurelia', MS='A species of ciliate protozoa. It is used in biomedical research.'),
  'D016806': NtO(MH='Tetrahymenina', MS='A suborder of ciliate protozoa. Most organisms are free-living in fresh water and many species are used extensively for experimental studies.'),
  'D016808': NtO(MH='Tetrahymena thermophila', MS='A species of ciliate protozoa used in genetic and cytological research.'),
  'D016809': NtO(MH='Polymenophorea', MS='A class of ciliate protozoa characterized by a well-developed, conspicuous adoral zone of numerous buccal or peristomial organelles, often extending out onto the body surface.'),
  'D016810': NtO(MH='Hypotrichida', MS='An order of ciliate protozoa. Characteristics include a dorsoventrally flattened, highly mobile body with a unique cursorial type of locomotion.'),
  'D016811': NtO(MH='Sporadotrichina', MS='A suborder of ciliate protozoa having a body that is commonly oval to elliptical in shape. Most species are free-living in widely diverse habitats.'),
  'D016812': NtO(MH='Euplotes', MS='A genus of ciliate protozoa having a dorsoventrally flattened body with widely spaced rows of short bristle-like cilia on the dorsal surface.'),
  'D016813': NtO(MH='Oxytricha', MS='A genus of ciliate protozoa having a unique cursorial type of locomotion.'),
  'D016814': NtO(MH='Microsporidia', MS='A phylum of fungi comprising minute intracellular PARASITES with FUNGAL SPORES of unicellular origin. It has two classes: Rudimicrosporea and MICROSPOREA.'),
  'D016815': NtO(MH='Microsporea', MS='A class of parasitic FUNGI. Characteristics include spores that are spherical, oval, or tubular in shape and sporoplasm which is uninuclear or binuclear.'),
  'D016816': NtO(MH='Microsporida', MS='An order of parasitic FUNGI found mostly in ARTHROPODS; FISHES; and in some VERTEBRATES including humans. It comprises two suborders: Pansporoblastina and APANSPOROBLASTINA.'),
  'D016817': NtO(MH='Apansporoblastina', MS='A suborder of FUNGI in the phylum MICROSPORIDIA, commonly lacking a pansporoblastic membrane. The sporoblast is usually dinucleate.'),
  'D016818': NtO(MH='Nosema', MS='A genus of parasitic FUNGI in the family Nosematidae. Some species are pathogenic for invertebrates of economic importance while others are being researched for possible roles in controlling pest INSECTS. They are also pathogenic in humans.'),
  'D016819': NtO(MH='Encephalitozoon', MS='A genus of FUNGI originally considered a member of the class SPOROZOEA but now recognized as part of the class MICROSPOREA.'),
  'D016822': NtO(MH='Euglenida', MS="A phylum of unicellular flagellates of ancient eukaryotic lineage with unclear taxonomy. They lack a CELL WALL but are covered by a proteinaceous flexible coat, the pellicle, that allows the cell to change shape. Historically some authorities considered them to be an order of protozoa and others classed them as ALGAE (some members have CHLOROPLASTS and some don't)."),
  'D016823': NtO(MH='Volvocida', MS='Order of CHLOROPHYTA commonly found in freshwater habitats. Characteristics include the presence of a cellulose wall and two to four equal, smooth, apical flagella.'),
  'D016824': NtO(MH='Antigens, Human Platelet', MS='Human alloantigens expressed only on platelets, specifically on platelet membrane glycoproteins. These platelet-specific antigens are immunogenic and can result in pathological reactions to transfusion therapy.'),
  'D016825': NtO(MH='Chlamydomonas reinhardtii', MS='A species of GREEN ALGAE. Delicate, hairlike appendages arise from the flagellar surface in these organisms.'),
  'D016827': NtO(MH='CD8 Antigens', MS='Differentiation antigens found on thymocytes and on cytotoxic and suppressor T-lymphocytes. T8 antigens are members of the immunoglobulin supergene family and are associative recognition elements in MHC (Major Histocompatibility Complex) Class I-restricted interactions.'),
  'D016828': NtO(MH='Diplomonadida', MS='A group of flagellated, mostly symbiotic EUKARYOTES characterized by twofold symmetry associated with the presence of a pair of karyomastigont organellar systems. Two nuclei are attached by fibers to the flagella and there are no MITOCHONDRIA. Diplomonadida were formerly members of the class Zoomastigophora in the old five kingdom paradigm.'),
  'D016829': NtO(MH='Giardia lamblia', MS='A species of parasitic EUKARYOTES that attaches itself to the intestinal mucosa and feeds on mucous secretions. The organism is roughly pear-shaped and motility is somewhat erratic, with a slow oscillation about the long axis.'),
  'D016830': NtO(MH='Kinetoplastida', MS='An order of flagellate protozoa. Characteristics include the presence of one or two flagella arising from a depression in the cell body and a single mitochondrion that extends the length of the body.'),
  'D016831': NtO(MH='Crithidia fasciculata', MS='A species of monogenetic, parasitic protozoa usually found in insects.'),
  'D016832': NtO(MH='Employment, Supported', MS='Paid work for mentally or physically disabled persons, taking place in regular or normal work settings. It may be competitive employment (work that pays minimum wage) or employment with subminimal wages in individualized or group placement situations. It is intended for persons with severe disabilities who require a range of support services to maintain employment. Supported employment differs from SHELTERED WORKSHOPS in that work in the latter takes place in a controlled working environment. Federal regulations are authorized and administered by the U.S. Department of Education, Office of Special Education and Rehabilitative Services.'),
  'D016833': NtO(MH='Trypanosoma brucei rhodesiense', MS='A hemoflagellate subspecies of parasitic protozoa that causes Rhodesian sleeping sickness in humans. It is carried by Glossina pallidipes, G. morsitans and occasionally other species of game-attacking tsetse flies.'),
  'D016834': NtO(MH='Trichomonadida', MS='An order of parasitic EUKARYOTES typically having four to six flagella. True cysts are known in very few species. Trichomonadida were formerly members of the class Zoomastigophora in the old five kingdom paradigm.'),
  'D016835': NtO(MH='Tritrichomonas foetus', MS='A species of flagellate parasitic EUKARYOTE. It possesses a long undulating membrane that is bordered on its outer margin by a flagellum that becomes free posteriorly. This organism causes infections in cows that could lead to temporary infertility or abortion.'),
  'D016837': NtO(MH='Dictyosteliida', MS='An order of protozoa characterized by their ability to aggregate to form a multicellular pseudoplasmodium, which gives rise to a multispored fruiting body. A stalk tube is present.'),
  'D016838': NtO(MH='Physarida', MS='An order of protozoa characterized by a peridium and capillitium that are calcareous and a spore mass that is usually dark-colored.'),
  'D016839': NtO(MH='Physarum polycephalum', MS='A protozoan, previously also considered a fungus. Characteristics include sporangia that are stalked and multilobed. It is widely used in biomedical research.'),
  'D016840': NtO(MH='Lobosea', MS='A class of amoeboid EUKARYOTES that forms either filiform subpseudopodia or lobopodia. Characteristics include the absence of sorocarps, sporangia, or similar fruiting bodies. Lobosea were formerly members of the phylum Sarcomastigophora, subphylum Sarcodina, under the old five kingdom paradigm.'),
  'D016841': NtO(MH='Amoebida', MS='An order of ameboid protozoa that is commonly uninucleate and possess mitochondria. Most organisms are nonpathogenic.'),
  'D016842': NtO(MH='Acanthopodina', MS='A suborder of ameboid protozoa. Characteristics include subpseudopodia that are more or less finely tipped, occasionally filiform, commonly furcate hyaline, and produced from a broad hyaline lobe. Cysts are usually formed and nuclear division is mesomitotic or metamitotic.'),
  'D016843': NtO(MH='Blastocystina', MS='A suborder of protozoa of the class LOBOSEA.'),
  'D016844': NtO(MH='Blastocystis', MS='A genus of protozoa of the suborder BLASTOCYSTINA. It was first classified as a yeast but further studies have shown it to be a protozoan.'),
  'D016845': NtO(MH='Blastocystis hominis', MS='A species of parasitic protozoa found in the intestines of humans and other primates. It was classified as a yeast in 1912. Over the years, questions arose about this designation. In 1967, many physiological and morphological B. hominis characteristics were reported that fit a protozoan classification. Since that time, other papers have corroborated this work and the organism is now recognized as a protozoan parasite of humans causing intestinal disease with potentially disabling symptoms.'),
  'D016846': NtO(MH='Tubulina', MS='A suborder of ameboid protozoa characterized by a branched or unbranched cylindrical body and the absence of bidirectional cytoplasmic flow. Nuclear division is mesomitotic.'),
  'D016847': NtO(MH='Schizopyrenida', MS='An order of ameboid protozoa commonly having a monopodial cylindrically shaped body. Nuclear division is promitotic and most species have a temporary flagellate stage.'),
  'D016848': NtO(MH='Naegleria fowleri', MS='A species of parasitic protozoa having both an ameboid and flagellate stage in its life cycle. Infection with this pathogen produces PRIMARY AMEBIC MENINGOENCEPHALITIS.'),
  'D016849': NtO(MH='Keratitis, Herpetic', MS='A superficial, epithelial Herpesvirus hominis infection of the cornea, characterized by the presence of small vesicles which may break down and coalesce to form dendritic ulcers (KERATITIS, DENDRITIC). (Dictionary of Visual Science, 3d ed)'),
  'D016850': NtO(MH='Limbus Corneae', MS='An annular transitional zone, approximately 1 mm wide, between the cornea and the bulbar conjunctiva and sclera. It is highly vascular and is involved in the metabolism of the cornea. It is ophthalmologically significant in that it appears on the outer surface of the eyeball as a slight furrow, marking the line between the clear cornea and the sclera. (Dictionary of Visual Science, 3d ed)'),
  'D016851': NtO(MH='Orthobunyavirus', MS='A genus of the family BUNYAVIRIDAE containing over 150 viruses, most of which are transmitted by mosquitoes or flies. They are arranged in groups defined by serological criteria, each now named for the original reference species (previously called serogroups). Many species have multiple serotypes or strains.'),
  'D016852': NtO(MH='Housing for the Elderly', MS='Housing arrangements for the elderly or aged, intended to foster independent living. The housing may take the form of group homes or small apartments. It is available to the economically self-supporting but the concept includes housing for the elderly with some physical limitations. The concept should be differentiated from HOMES FOR THE AGED which is restricted to long-term geriatric facilities providing supervised medical and nursing services.'),
  'D016853': NtO(MH='Muromonab-CD3', MS='Anti-CD3 monoclonal antibody that exerts immunosuppressive effects by inducing peripheral T-cell depletion and modulation of the T-cell receptor complex (CD3/Ti).'),
  'D016854': NtO(MH='Dental Anxiety', MS='Abnormal fear or dread of visiting the dentist for preventive care or therapy and unwarranted anxiety over dental procedures.'),
  'D016855': NtO(MH='Nairovirus', MS='A genus of the family BUNYAVIRIDAE named after NAIROBI SHEEP DISEASE, an acute, hemorrhagic, tick-borne, gastroenteritis affecting sheep and goats. The type species is Dugbe virus. Some viruses in this genus are capable of causing severe and fatal disease in humans.'),
  'D016856': NtO(MH='Phlebovirus', MS='A genus of the family BUNYAVIRIDAE comprising many viruses, most of which are transmitted by Phlebotomus flies and cause PHLEBOTOMUS FEVER. The type species is RIFT VALLEY FEVER VIRUS.'),
  'D016857': NtO(MH='Hypocapnia', MS='Clinical manifestation consisting of a deficiency of carbon dioxide in arterial blood.'),
  'D016858': NtO(MH='Mycoplasma fermentans', MS='A species of the genus MYCOPLASMA, originally isolated infrequently from the lower genital tract of humans, and possessing uncertain pathogenicity. The incognitus strain of M. fermentans has been identified in necrotizing lesions of multiple organs from AIDS and non-AIDS patients dying of an acute influenza-like disease.'),
  'D016859': NtO(MH='Lipoxygenase Inhibitors', MS='Compounds that bind to and inhibit that enzymatic activity of LIPOXYGENASES. Included under this category are inhibitors that are specific for lipoxygenase subtypes and act to reduce the production of LEUKOTRIENES.'),
  'D016860': NtO(MH='CD4 Immunoadhesins', MS='Chimeric molecules resulting from the fusion of recombinant soluble CD4 to the Fc portion of immunoglobulins. These have potential use in the therapy of AIDS since they possess both the gp120-binding and HIV-blocking properties of rCD4 as well as the long plasma half-life and Fc receptor-binding functions of immunoglobulin.'),
  'D016861': NtO(MH='Cyclooxygenase Inhibitors', MS='Compounds or agents that combine with cyclooxygenase (PROSTAGLANDIN-ENDOPEROXIDE SYNTHASES) and thereby prevent its substrate-enzyme combination with arachidonic acid and the formation of eicosanoids, prostaglandins, and thromboxanes.'),
  'D016862': NtO(MH='Tuberculoma, Intracranial', MS='A well-circumscribed mass composed of tuberculous granulation tissue that may occur in the cerebral hemispheres, cerebellum, brain stem, or perimeningeal spaces. Multiple lesions are quite common. Management of intracranial manifestations vary with lesion site. Intracranial tuberculomas may be associated with SEIZURES, focal neurologic deficits, and INTRACRANIAL HYPERTENSION. Spinal cord tuberculomas may be associated with localized or radicular pain, weakness, sensory loss, and incontinence. Tuberculomas may arise as OPPORTUNISTIC INFECTIONS, but also occur in immunocompetent individuals.'),
  'D016863': NtO(MH='Bacillaceae Infections', MS='Infections with bacteria of the family BACILLACEAE.'),
  'D016864': NtO(MH='Li-Fraumeni Syndrome', MS='Rare autosomal dominant syndrome characterized by mesenchymal and epithelial neoplasms at multiple sites. MUTATION of the p53 tumor suppressor gene, a component of the DNA DAMAGE response pathway, apparently predisposes family members who inherit it to develop certain cancers. The spectrum of cancers in the syndrome was shown to include, in addition to BREAST CANCER and soft tissue sarcomas (SARCOMA); BRAIN TUMORS; OSTEOSARCOMA; LEUKEMIA; and ADRENOCORTICAL CARCINOMA.'),
  'D016865': NtO(MH='Reassortant Viruses', MS='Viruses containing two or more pieces of nucleic acid (segmented genome) from different parents. Such viruses are produced in cells coinfected with different strains of a given virus.'),
  'D016866': NtO(MH='Bacteroidaceae Infections', MS='Infections with bacteria of the family BACTEROIDACEAE.'),
  'D016867': NtO(MH='Immunocompromised Host', MS='A human or animal whose immunologic mechanism is deficient because of an immunodeficiency disorder or other disease or as the result of the administration of immunosuppressive drugs or radiation.'),
  'D016868': NtO(MH='Serratia Infections', MS='Infections with bacteria of the genus SERRATIA.'),
  'D016869': NtO(MH='Ureaplasma Infections', MS='Infections with bacteria of the genus UREAPLASMA.'),
  'D016870': NtO(MH='Neisseriaceae Infections', MS='Infections with bacteria of the family NEISSERIACEAE.'),
  'D016871': NtO(MH='Pasteurellaceae Infections', MS='Infections with bacteria of the family PASTEURELLACEAE.'),
  'D016873': NtO(MH='Ehrlichiosis', MS='A tick-borne disease characterized by FEVER; HEADACHE; myalgias; ANOREXIA; and occasionally RASH. It is caused by several bacterial species and can produce disease in DOGS; CATTLE; SHEEP; GOATS; HORSES; and humans. The primary species causing human disease are EHRLICHIA CHAFFEENSIS; ANAPLASMA PHAGOCYTOPHILUM; and Ehrlichia ewingii.'),
  'D016874': NtO(MH='Neurofibrillary Tangles', MS='Abnormal structures located in various parts of the brain and composed of dense arrays of paired helical filaments (neurofilaments and microtubules). These double helical stacks of transverse subunits are twisted into left-handed ribbon-like filaments that likely incorporate the following proteins: (1) the intermediate filaments: medium- and high-molecular-weight neurofilaments; (2) the microtubule-associated proteins map-2 and tau; (3) actin; and (4) UBIQUITINS. As one of the hallmarks of ALZHEIMER DISEASE, the neurofibrillary tangles eventually occupy the whole of the cytoplasm in certain classes of cell in the neocortex, hippocampus, brain stem, and diencephalon. The number of these tangles, as seen in post mortem histology, correlates with the degree of dementia during life. Some studies suggest that tangle antigens leak into the systemic circulation both in the course of normal aging and in cases of Alzheimer disease.'),
  'D016875': NtO(MH='tau Proteins', MS='Microtubule-associated proteins that are mainly expressed in neurons. Tau proteins constitute several isoforms and play an important role in the assembly of tubulin monomers into microtubules and in maintaining the cytoskeleton and axonal transport. Aggregation of specific sets of tau proteins in filamentous inclusions is the common feature of intraneuronal and glial fibrillar lesions (NEUROFIBRILLARY TANGLES; NEUROPIL THREADS) in numerous neurodegenerative disorders (ALZHEIMER DISEASE; TAUOPATHIES).'),
  'D016876': NtO(MH='Metal Ceramic Alloys', MS='The fusion of ceramics (porcelain) to an alloy of two or more metals for use in restorative and prosthodontic dentistry. Examples of metal alloys employed include cobalt-chromium, gold-palladium, gold-platinum-palladium, and nickel-based alloys.'),
  'D016877': NtO(MH='Oxidants', MS='Electron-accepting molecules in chemical reactions in which electrons are transferred from one molecule to another (OXIDATION-REDUCTION).'),
  'D016878': NtO(MH='POEMS Syndrome', MS='A multisystemic disorder characterized by a sensorimotor polyneuropathy (POLYNEUROPATHIES), organomegaly, endocrinopathy, monoclonal gammopathy, and pigmentary skin changes. Other clinical features which may be present include EDEMA; CACHEXIA; microangiopathic glomerulopathy; pulmonary hypertension (HYPERTENSION, PULMONARY); cutaneous necrosis; THROMBOCYTOSIS; and POLYCYTHEMIA. This disorder is frequently associated with osteosclerotic myeloma. (From Adams et al., Principles of Neurology, 6th ed, p1335; Rev Med Interne 1997;18(7):553-62)'),
  'D016879': NtO(MH='Salvage Therapy', MS="A therapeutic approach, involving chemotherapy, radiation therapy, or surgery, after initial regimens have failed to lead to improvement in a patient's condition. Salvage therapy is most often used for neoplastic diseases."),
  'D016880': NtO(MH='Anisotropy', MS='A physical property showing different values in relation to the direction in or along which the measurement is made. The physical property may be with regard to thermal or electric conductivity or light refraction. In crystallography, it describes crystals whose index of refraction varies with the direction of the incident light. It is also called acolotropy and colotropy. The opposite of anisotropy is isotropy wherein the same values characterize the object when measured along axes in all directions.'),
  'D016881': NtO(MH='Microsporidiosis', MS='Infections with FUNGI of the phylum MICROSPORIDIA.'),
  'D016882': NtO(MH='ISCOMs', MS='A formulation for presenting an antigen to induce specific immunologic responses. It consists of an assembly of antigens in multimeric form. The assembly is attached to a matrix with a built-in adjuvant, saponin. ISCOMs induce strong serum antibody responses, and are used as highly immunogenic forms of subunit vaccines.'),
  'D016883': NtO(MH='Diabetic Ketoacidosis', MS='A life-threatening complication of diabetes mellitus, primarily of TYPE 1 DIABETES MELLITUS with severe INSULIN deficiency and extreme HYPERGLYCEMIA. It is characterized by KETOSIS; DEHYDRATION; and depressed consciousness leading to COMA.'),
  'D016884': NtO(MH='Polyendocrinopathies, Autoimmune', MS="Autoimmune diseases affecting multiple endocrine organs. Type I is characterized by childhood onset and chronic mucocutaneous candidiasis (CANDIDIASIS, CHRONIC MUCOCUTANEOUS), while type II exhibits any combination of adrenal insufficiency (ADDISON'S DISEASE), lymphocytic thyroiditis (THYROIDITIS, AUTOIMMUNE;), HYPOPARATHYROIDISM; and gonadal failure. In both types organ-specific ANTIBODIES against a variety of ENDOCRINE GLANDS have been detected. The type II syndrome differs from type I in that it is associated with HLA-A1 and B8 haplotypes, onset is usually in adulthood, and candidiasis is not present."),
  'D016885': NtO(MH='United States Department of Agriculture', MS='A cabinet department in the Executive Branch of the United States Government concerned with improving and maintaining farm income and developing and expanding markets for agricultural products. Through inspection and grading services it safeguards and insures standards of quality in food supply and production.'),
  'D016887': NtO(MH='Cardiopulmonary Resuscitation', MS='The artificial substitution of heart and lung action as indicated for HEART ARREST resulting from electric shock, DROWNING, respiratory arrest, or other causes. The two major components of cardiopulmonary resuscitation are artificial ventilation (RESPIRATION, ARTIFICIAL) and closed-chest CARDIAC MASSAGE.'),
  'D016888': NtO(MH='Angiodysplasia', MS='Acquired degenerative dilation or expansion (ectasia) of normal BLOOD VESSELS, often associated with aging. They are isolated, tortuous, thin-walled vessels and sources of bleeding. They occur most often in mucosal capillaries of the GASTROINTESTINAL TRACT leading to GASTROINTESTINAL HEMORRHAGE and ANEMIA.'),
  'D016889': NtO(MH='Endometrial Neoplasms', MS='Tumors or cancer of ENDOMETRIUM, the mucous lining of the UTERUS. These neoplasms can be benign or malignant. Their classification and grading are based on the various cell types and the percent of undifferentiated cells.'),
  'D016890': NtO(MH='Encephalitozoonosis', MS='Infection with FUNGI of the genus ENCEPHALITOZOON. Lesions commonly occur in the BRAIN and KIDNEY tubules. Other sites of infection in MAMMALS are the LIVER; ADRENAL GLANDS; OPTIC NERVES; RETINA; and MYOCARDIUM.'),
  'D016891': NtO(MH='Polycystic Kidney, Autosomal Dominant', MS='Kidney disorders with autosomal dominant inheritance and characterized by multiple CYSTS in both KIDNEYS with progressive deterioration of renal function.'),
  'D016893': NtO(MH='Carotid Stenosis', MS='Narrowing or stricture of any part of the CAROTID ARTERIES, most often due to atherosclerotic plaque formation. Ulcerations may form in atherosclerotic plaques and induce THROMBUS formation. Platelet or cholesterol emboli may arise from stenotic carotid lesions and induce a TRANSIENT ISCHEMIC ATTACK; CEREBROVASCULAR ACCIDENT; or temporary blindness (AMAUROSIS FUGAX). (From Adams et al., Principles of Neurology, 6th ed, pp 822-3)'),
  'D016894': NtO(MH='Endarterectomy, Carotid', MS='The excision of the thickened, atheromatous tunica intima of a carotid artery.'),
  'D016895': NtO(MH='Culture Media, Serum-Free', MS='CULTURE MEDIA free of serum proteins but including the minimal essential substances required for cell growth. This type of medium avoids the presence of extraneous substances that may affect cell proliferation or unwanted activation of cells.'),
  'D016896': NtO(MH='Treatment Outcome', MS='Evaluation undertaken to assess the results or consequences of management and procedures used in combating disease in order to determine the efficacy, effectiveness, safety, and practicability of these interventions in individual cases or series.'),
  'D016897': NtO(MH='Respiratory Burst', MS='A large increase in oxygen uptake by neutrophils and most types of tissue macrophages through activation of an NADPH-cytochrome b-dependent oxidase that reduces oxygen to a superoxide. Individuals with an inherited defect in which the oxidase that reduces oxygen to superoxide is decreased or absent (GRANULOMATOUS DISEASE, CHRONIC) often die as a result of recurrent bacterial infections.'),
  'D016898': NtO(MH='Interferon-alpha', MS='One of the type I interferons produced by peripheral blood leukocytes or lymphoblastoid cells. In addition to antiviral activity, it activates NATURAL KILLER CELLS and B-LYMPHOCYTES, and down-regulates VASCULAR ENDOTHELIAL GROWTH FACTOR expression through PI-3 KINASE and MAPK KINASES signaling pathways.'),
  'D016899': NtO(MH='Interferon-beta', MS='One of the type I interferons produced by fibroblasts in response to stimulation by live or inactivated virus or by double-stranded RNA. It is a cytokine with antiviral, antiproliferative, and immunomodulating activity.'),
  'D016900': NtO(MH='Neurofilament Proteins', MS="Type III intermediate filament proteins that assemble into neurofilaments, the major cytoskeletal element in nerve axons and dendrites. They consist of three distinct polypeptides, the neurofilament triplet. Types I, II, and IV intermediate filament proteins form other cytoskeletal elements such as keratins and lamins. It appears that the metabolism of neurofilaments is disturbed in Alzheimer's disease, as indicated by the presence of neurofilament epitopes in the neurofibrillary tangles, as well as by the severe reduction of the expression of the gene for the light neurofilament subunit of the neurofilament triplet in brains of Alzheimer's patients. (Can J Neurol Sci 1990 Aug;17(3):302)"),
  'D016901': NtO(MH='Intergenerational Relations', MS='The interactions between individuals of different generations. These interactions include communication, caring, accountability, loyalty, and even conflict between related or non-related individuals.'),
  'D016902': NtO(MH='Air Pollution, Indoor', MS='The contamination of indoor air.'),
  'D016903': NtO(MH='Drug Monitoring', MS='The process of observing, recording, or detecting the effects of a chemical substance administered to an individual therapeutically or diagnostically.'),
  'D016905': NtO(MH='Gram-Negative Bacterial Infections', MS='Infections caused by bacteria that show up as pink (negative) when treated by the gram-staining method.'),
  'D016906': NtO(MH='Interleukin-9', MS='A multifunctional cytokine secreted by primarily by activated TH2 CELLS that may play a role as a regulator of allergic INFLAMMATION. It has been shown to enhance the growth and CELL DIFFERENTIATION of MAST CELLS, and can act on a variety of other immune cells.'),
  'D016907': NtO(MH='Adverse Drug Reaction Reporting Systems', MS='Systems developed for collecting reports from government agencies, manufacturers, hospitals, physicians, and other sources on adverse drug reactions.'),
  'D016908': NtO(MH='Gram-Positive Bacterial Infections', MS='Infections caused by bacteria that retain the crystal violet stain (positive) when treated by the gram-staining method.'),
  'D016909': NtO(MH='Tibial Arteries', MS='The anterior and posterior arteries created at the bifurcation of the popliteal artery. The anterior tibial artery begins at the lower border of the popliteus muscle and lies along the tibia at the distal part of the leg to surface superficially anterior to the ankle joint. Its branches are distributed throughout the leg, ankle, and foot. The posterior tibial artery begins at the lower border of the popliteus muscle, lies behind the tibia in the lower part of its course, and is found situated between the medial malleolus and the medial process of the calcaneal tuberosity. Its branches are distributed throughout the leg and foot.'),
  'D016910': NtO(MH='Orthodontic Brackets', MS='Small metal or ceramic attachments used to fasten an arch wire. These attachments are soldered or welded to an orthodontic band or cemented directly onto the teeth. Bowles brackets, edgewise brackets, multiphase brackets, ribbon arch brackets, twin-wire brackets, and universal brackets are all types of orthodontic brackets.'),
  'D016911': NtO(MH='Koro', MS="A culture-specific acute delusional syndrome occurring mostly in the Malay people and Southern Chinese. The syndrome is characterized by the individual's sudden experience of depersonalization depicted by the belief that his penis is shrinking into his abdomen and that he will die when this occurs. Although generally classified as a culture-specific syndrome, koro may be no more than a variant of castration anxiety (ANXIETY, CASTRATION) as it has been described in non-Asian patients."),
  'D016912': NtO(MH='Levonorgestrel', MS='A synthetic progestational hormone with actions similar to those of PROGESTERONE and about twice as potent as its racemic or (+-)-isomer (NORGESTREL). It is used for contraception, control of menstrual disorders, and treatment of endometriosis.'),
  'D016913': NtO(MH='Blood Component Transfusion', MS='The transfer of blood components such as erythrocytes, leukocytes, platelets, and plasma from a donor to a recipient or back to the donor. This process differs from the procedures undertaken in PLASMAPHERESIS and types of CYTAPHERESIS; (PLATELETPHERESIS and LEUKAPHERESIS) where, following the removal of plasma or the specific cell components, the remainder is transfused back to the donor.'),
  'D016914': NtO(MH='Ribonuclease H', MS='A ribonuclease that specifically cleaves the RNA moiety of RNA:DNA hybrids. It has been isolated from a wide variety of prokaryotic and eukaryotic organisms as well as RETROVIRUSES.'),
  'D016915': NtO(MH='AIDS Vaccines', MS='Vaccines or candidate vaccines containing inactivated HIV or some of its component antigens and designed to prevent or treat AIDS. Some vaccines containing antigens are recombinantly produced.'),
  'D016916': NtO(MH='Joint Deformities, Acquired', MS='Deformities acquired after birth as the result of injury or disease. The joint deformity is often associated with rheumatoid arthritis and leprosy.'),
  'D016917': NtO(MH='Angiomatosis, Bacillary', MS='A reactive vascular proliferation that is characterized by the multiple tumor-like lesions in skin, bone, brain, and other organs. Bacillary angiomatosis is caused by infection with gram-negative Bartonella bacilli (such as BARTONELLA HENSELAE), and is often seen in AIDS patients and other IMMUNOCOMPROMISED HOSTS.'),
  'D016918': NtO(MH='Arthritis, Reactive', MS='An aseptic, inflammatory arthritis developing secondary to a primary extra-articular infection, most typically of the GASTROINTESTINAL TRACT or UROGENITAL SYSTEM. The initiating trigger pathogens are usually SHIGELLA; SALMONELLA; YERSINIA; CAMPYLOBACTER; or CHLAMYDIA TRACHOMATIS. Reactive arthritis is strongly associated with HLA-B27 ANTIGEN.'),
  'D016919': NtO(MH='Meningitis, Cryptococcal', MS='Meningeal inflammation produced by CRYPTOCOCCUS NEOFORMANS, an encapsulated yeast that tends to infect individuals with ACQUIRED IMMUNODEFICIENCY SYNDROME and other immunocompromised states. The organism enters the body through the respiratory tract, but symptomatic infections are usually limited to the lungs and nervous system. The organism may also produce parenchymal brain lesions (torulomas). Clinically, the course is subacute and may feature HEADACHE; NAUSEA; PHOTOPHOBIA; focal neurologic deficits; SEIZURES; cranial neuropathies; and HYDROCEPHALUS. (From Adams et al., Principles of Neurology, 6th ed, pp721-2)'),
  'D016920': NtO(MH='Meningitis, Bacterial', MS='Bacterial infections of the leptomeninges and subarachnoid space, frequently involving the cerebral cortex, cranial nerves, cerebral blood vessels, spinal cord, and nerve roots.'),
  'D016921': NtO(MH='Meningitis, Fungal', MS='Meningitis caused by fungal agents which may occur as OPPORTUNISTIC INFECTIONS or arise in immunocompetent hosts.'),
  'D016922': NtO(MH='Cellular Senescence', MS='Process by which cells irreversibly stop dividing and enter a state of permanent growth arrest without undergoing CELL DEATH. Senescence can be induced by DNA DAMAGE or other cellular stresses, such as OXIDATIVE STRESS.'),
  'D016923': NtO(MH='Cell Death', MS="The termination of the cell's ability to carry out vital functions such as metabolism, growth, reproduction, responsiveness, and adaptability."),
  'D016924': NtO(MH='Actinomyces viscosus', MS='A species of ACTINOMYCES found in the oral cavity of man and hamsters. It has been isolated from actinomycotic lesions in swine, cats, and dogs and has been identified as a causative agent of animal diseases.'),
  'D016925': NtO(MH='Corynebacterium pseudotuberculosis', MS='A species of gram-positive, asporogenous bacteria that was originally isolated from necrotic areas in the kidney of a sheep. It may cause ulcerative lymphangitis, abscesses, and other chronic purulent infections in sheep, horses, and other warm-blooded animals. Human disease may form from contact with infected animals.'),
  'D016926': NtO(MH='Mycobacterium chelonae', MS='A species of gram-positive, aerobic bacteria commonly found in soil and occasionally isolated from sputum. It causes postoperative wound infections as well as gluteal abscesses.'),
  'D016927': NtO(MH='Mycobacterium avium subsp. paratuberculosis', MS="A subspecies of gram-positive, aerobic bacteria. It is the etiologic agent of Johne's disease (PARATUBERCULOSIS), a chronic GASTROENTERITIS in RUMINANTS."),
  'D016929': NtO(MH='Rhodococcus equi', MS='A species of RHODOCOCCUS found in soil, herbivore dung, and in the intestinal tract of cows, horses, sheep, and pigs. It causes bronchopneumonia in foals and can be responsible for infection in humans compromised by immunosuppressive drug therapy, lymphoma, or AIDS.'),
  'D016930': NtO(MH='Saccharopolyspora', MS='A genus of gram-positive bacteria whose spores are round to oval and covered by a sheath.'),
  'D016931': NtO(MH='Halobacteriales', MS='An order of extremely halophilic archaea, in the kingdom EURYARCHAEOTA. They occur ubiquitously in nature where the salt concentration is high, and are chemoorganotrophic, using amino acids or carbohydrates as a carbon source.'),
  'D016934': NtO(MH='Caulobacter', MS='A genus of gram-negative, aerobic, rod- or vibroid-shaped or fusiform bacteria that commonly produce a stalk. They are found in fresh water and soil and divide by binary transverse fission.'),
  'D016935': NtO(MH='Caulobacter crescentus', MS='A species of gram-negative, aerobic bacteria that consist of slender vibroid cells.'),
  'D016936': NtO(MH='Gram-Positive Endospore-Forming Bacteria', MS='Bacteria that form endospores and are gram-positive. Representative genera include BACILLUS; CLOSTRIDIUM; MICROMONOSPORA; SACCHAROPOLYSPORA; and STREPTOMYCES.'),
  'D016937': NtO(MH='Gram-Positive Endospore-Forming Rods', MS='Rod-shaped bacteria that form endospores and are gram-positive. Representative genera include BACILLUS and CLOSTRIDIUM.'),
  'D016940': NtO(MH='Myxococcus', MS='A genus of gliding bacteria that are common inhabitants on tree bark and decomposing vegetation. The cells are slender rods with tapering ends.'),
  'D016941': NtO(MH='Myxococcus xanthus', MS='A species of gliding bacteria found on soil as well as in surface fresh water and coastal seawater.'),
  'D016942': NtO(MH='Sulfolobales', MS='An order of CRENARCHAEOTA consisting of aerobic or facultatively aerobic, chemolithotrophic cocci which are extreme thermoacidophiles. They lack peptidoglycan in their cell walls.'),
  'D016943': NtO(MH='Sulfolobaceae', MS='A family of SULFOLOBALES consisting of aerobic or facultatively anaerobic chemolithotrophic cocci, usually occurring singly. They grow best at a pH of about 2.'),
  'D016944': NtO(MH='Sulfolobus', MS='A genus of aerobic, chemolithotrophic, coccoid ARCHAEA whose organisms are thermoacidophilic. Its cells are highly irregular in shape, often lobed, but occasionally spherical. It has worldwide distribution with organisms isolated from hot acidic soils and water. Sulfur is used as an energy source.'),
  'D016945': NtO(MH='Sulfolobus acidocaldarius', MS='A species of aerobic, chemolithotrophic ARCHAEA consisting of coccoid cells that utilize sulfur as an energy source. The optimum temperature for growth is 70-75 degrees C. They are isolated from acidic fields.'),
  'D016946': NtO(MH='Gram-Negative Aerobic Rods and Cocci', MS='A group of gram-negative bacteria consisting of rod- and coccus-shaped cells. They are both aerobic (able to grow under an air atmosphere) and microaerophilic (grow better in low concentrations of oxygen) under nitrogen-fixing conditions but, when supplied with a source of fixed nitrogen, they grow as aerobes.'),
  'D016947': NtO(MH='Acetobacteraceae', MS='A family of gram-negative aerobic bacteria consisting of ellipsoidal to rod-shaped cells that occur singly, in pairs, or in chains.'),
  'D016948': NtO(MH='Azotobacter vinelandii', MS="A species of gram-negative, aerobic bacteria first isolated from soil in Vineland, New Jersey. Ammonium and nitrate are used as nitrogen sources by this bacterium. It is distinguished from other members of its genus by the ability to use rhamnose as a carbon source. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D016949': NtO(MH='Azospirillum', MS='A genus of gram-negative aerobic bacteria that occurs free-living in the soil or associated with the roots of cereal crops or grasses (POACEAE).'),
  'D016950': NtO(MH='Bordetella bronchiseptica', MS='A species of BORDETELLA that is parasitic and pathogenic. It is found in the respiratory tract of domestic and wild mammalian animals and can be transmitted from animals to man. It is a common cause of bronchopneumonia in lower animals.'),
  'D016951': NtO(MH='Legionellaceae', MS='A family of gram-negative, aerobic bacteria that do not form endospores or microcysts.'),
  'D016952': NtO(MH='Legionella pneumophila', MS="A species of gram-negative, aerobic bacteria that is the causative agent of LEGIONNAIRES' DISEASE. It has been isolated from numerous environmental sites as well as from human lung tissue, respiratory secretions, and blood."),
  'D016953': NtO(MH='Leptospiraceae', MS='A family of bacteria consisting of flexible helical cells exhibiting a right-handed conformation. It consists of a single genus, LEPTOSPIRA.'),
  'D016954': NtO(MH='Acinetobacter calcoaceticus', MS='A species of gram-negative, aerobic bacteria found in soil and water. Although considered to be normally nonpathogenic, this bacterium is a causative agent of nosocomial infections, particularly in debilitated individuals.'),
  'D016955': NtO(MH='Moraxella bovis', MS='A species of gram-negative, aerobic bacteria that is most frequently isolated from bovine eyes in cases of infectious keratoconjunctivitis (KERATOCONJUNCTIVITIS, INFECTIOUS), but also occurs in unaffected eyes and the nasal cavity of cattle.'),
  'D016956': NtO(MH='Burkholderia cepacia', MS='A species of BURKHOLDERIA considered to be an opportunistic human pathogen. It has been associated with various types of infections of nosocomial origin.'),
  'D016957': NtO(MH='Burkholderia pseudomallei', MS='A species of gram-negative, aerobic bacteria that causes MELIOIDOSIS. It has been isolated from soil and water in tropical regions, particularly Southeast Asia.'),
  'D016958': NtO(MH='Pseudomonas putida', MS='A species of gram-negative, aerobic bacteria isolated from soil and water as well as clinical specimens. Occasionally it is an opportunistic pathogen.'),
  'D016959': NtO(MH='Xanthomonas campestris', MS='A species of gram-negative, aerobic bacteria that is pathogenic for plants.'),
  'D016960': NtO(MH='Agrobacterium tumefaciens', MS='A species of gram-negative, aerobic bacteria isolated from soil and the stems, leafs, and roots of plants. Some biotypes are pathogenic and cause the formation of PLANT TUMORS in a wide variety of higher plants. The species is a major research tool in biotechnology.'),
  'D016961': NtO(MH='Rhizobium leguminosarum', MS='A species of gram-negative, aerobic bacteria that is found in soil and which causes formation of root nodules on some, but not all, types of field pea, lentil, kidney bean, and clover.'),
  'D016962': NtO(MH='Sinorhizobium meliloti', MS='A species of gram-negative, aerobic bacteria that causes formation of root nodules on some, but not all, types of sweet clover, MEDICAGO SATIVA, and fenugreek.'),
  'D016963': NtO(MH='Thermus thermophilus', MS='A species of gram-negative, aerobic, rod-shaped bacteria found in hot springs of neutral to alkaline pH, as well as in hot-water heaters.'),
  'D016964': NtO(MH='Gram-Negative Anaerobic Cocci', MS='A group of anaerobic coccoid bacteria that show up as pink (negative) when treated by the gram-staining method.'),
  'D016965': NtO(MH='Gram-Negative Anaerobic Straight, Curved, and Helical Rods', MS='A group of anaerobic, rod-shaped bacteria that show up as pink (negative) when treated by the Gram-staining method.'),
  'D016966': NtO(MH='Porphyromonas gingivalis', MS='A species of gram-negative, anaerobic, rod-shaped bacteria originally classified within the BACTEROIDES genus. This bacterium produces a cell-bound, oxygen-sensitive collagenase and is isolated from the human mouth.'),
  'D016967': NtO(MH='Fusobacterium nucleatum', MS='A species of gram-negative, anaerobic, rod-shaped bacteria isolated from the gingival margin and sulcus and from infections of the upper respiratory tract and pleural cavity.'),
  'D016968': NtO(MH='Wolinella', MS='A genus of gram-negative, anaerobic, rod-shaped bacteria isolated from the bovine RUMEN, the human gingival sulcus, and dental PULPITIS infections.'),
  'D016969': NtO(MH='Desulfovibrio vulgaris', MS='A species of gram-negative, anaerobic, rod-shaped bacteria isolated from soil, animal intestines and feces, and fresh and salt water.'),
  'D016970': NtO(MH='Eikenella', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs in the human mouth and intestine. Organisms of this genus can be opportunistic pathogens.'),
  'D016971': NtO(MH='Citrobacter freundii', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria found in humans and other animals including MAMMALS; BIRDS; REPTILES; and AMPHIBIANS. It has also been isolated from SOIL and WATER as well as from clinical specimens such as URINE; THROAT; SPUTUM; BLOOD; and wound swabs as an opportunistic pathogen.'),
  'D016972': NtO(MH='Enterobacter cloacae', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs in water, sewage, soil, meat, hospital environments, and on the skin and in the intestinal tract of man and animals as a commensal.'),
  'D016973': NtO(MH='Pectobacterium carotovorum', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that causes rotting, particularly of storage tissues, of a wide variety of plants and causes a vascular disease in CARROTS; and POTATO plants.'),
  'D016974': NtO(MH='Pectobacterium chrysanthemi', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that causes vascular wilts on a wide range of plant species. It was formerly named Erwinia chrysanthemi.'),
  'D016975': NtO(MH='Gardnerella', MS='A genus of bacteria found in the human genital and urinary tract. It is considered to be a major cause of bacterial vaginosis (VAGINOSIS, BACTERIAL).'),
  'D016976': NtO(MH='Aggregatibacter actinomycetemcomitans', MS='A species of Gram-negative, facultatively anaerobic spherical or rod-shaped bacteria indigenous to dental surfaces. It is associated with PERIODONTITIS; BACTERIAL ENDOCARDITIS; and ACTINOMYCOSIS.'),
  'D016977': NtO(MH='Actinobacillus pleuropneumoniae', MS='A species of gram-negative, facultatively anaerobic coccobacillus-shaped bacteria that has been isolated from pneumonic lesions and blood. It produces pneumonia with accompanying fibrinous pleuritis in swine.'),
  'D016978': NtO(MH='Mannheimia haemolytica', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria normally commensal in the flora of CATTLE and SHEEP. But under conditions of physical or PHYSIOLOGICAL STRESS, it can cause MASTITIS in sheep and SHIPPING FEVER or ENZOOTIC CALF PNEUMONIA in cattle. Its former name was Pasteurella haemolytica.'),
  'D016979': NtO(MH='Pasteurella multocida', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria normally found in the flora of the mouth and respiratory tract of animals and birds. It causes shipping fever (see PASTEURELLOSIS, PNEUMONIC); HEMORRHAGIC BACTEREMIA; and intestinal disease in animals. In humans, disease usually arises from a wound infection following a bite or scratch from domesticated animals.'),
  'D016980': NtO(MH='Aeromonas hydrophila', MS='A species of gram-negative, facultatively anaerobic, rod-shaped bacteria that may be pathogenic for frogs, fish, and mammals, including man. In humans, cellulitis and diarrhea can result from infection with this organism.'),
  'D016981': NtO(MH='Plesiomonas', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that occurs in fish and other aquatic animals and in a variety of mammals, including man. Its organisms probably do not belong to the normal intestinal flora of man and can cause diarrhea.'),
  'D016982': NtO(MH='Micrococcus luteus', MS='A species of gram-positive, spherical bacteria whose organisms occur in tetrads and in irregular clusters of tetrads. The primary habitat is mammalian skin.'),
  'D016983': NtO(MH='Enterococcus', MS='A genus of gram-positive, coccoid bacteria consisting of organisms causing variable hemolysis that are normal flora of the intestinal tract. Previously thought to be a member of the genus STREPTOCOCCUS, it is now recognized as a separate genus.'),
  'D016984': NtO(MH='Enterococcus faecium', MS='A species of gram-positive, coccoid bacteria whose organisms are normal flora of the intestinal tract. Unlike ENTEROCOCCUS FAECALIS, this species may produce an alpha-hemolytic reaction on blood agar and is unable to utilize pyruvic acid as an energy source.'),
  'D016985': NtO(MH='Streptococcus bovis', MS='A species of gram-positive, coccoid bacteria commonly found in the alimentary tract of cows, sheep, and other ruminants. It occasionally is encountered in cases of human endocarditis. This species is nonhemolytic.'),
  'D016986': NtO(MH='Streptococcus sobrinus', MS='A species of gram-positive, coccoid bacteria isolated from the human tooth surface. Strains have been shown to be cariogenic in experimental animals and may be associated with human dental caries.'),
  'D016987': NtO(MH='Gram-Positive Rods', MS="A large group of rod-shaped bacteria that retains the crystal violet stain when treated by Gram's method."),
  'D016988': NtO(MH='Gram-Positive Asporogenous Rods, Irregular', MS='A group of irregular rod-shaped bacteria that stain gram-positive and do not produce endospores.'),
  'D016989': NtO(MH='Gram-Positive Asporogenous Rods, Regular', MS='A group of regular rod-shaped bacteria that stain gram-positive and do not produce endospores.'),
  'D016990': NtO(MH='Ureaplasma urealyticum', MS='A species of gram-negative bacteria found in the human genitourinary tract (UROGENITAL SYSTEM), oropharynx, and anal canal. Serovars 1, 3, 6, and 14 have been reclassed into a separate species UREAPLASMA parvum.'),
  'D016991': NtO(MH='Spiroplasmataceae', MS='A family of gram-negative, helical bacteria in the order Entomoplasmatales, that require STEROLS for growth.'),
  'D016992': NtO(MH='Rhodobacter', MS='A genus of gram-negative bacteria widely distributed in fresh water as well as marine and hypersaline habitats.'),
  'D016993': NtO(MH='Chlamydophila pneumoniae', MS='A species of CHLAMYDOPHILA that causes acute respiratory infection, especially atypical pneumonia, in humans, horses, and koalas.'),
  'D016995': NtO(MH='Ehrlichia ruminantium', MS='A species of gram-negative bacteria in the family ANAPLASMATACEAE, that causes HEARTWATER DISEASE in ruminants.'),
  'D016996': NtO(MH='Rickettsieae', MS='A tribe of gram-negative bacteria of the family RICKETTSIACEAE whose organisms are found in arthropods and are pathogenic for man and certain other vertebrate hosts.'),
  'D016997': NtO(MH='Coxiella burnetii', MS='A species of gram-negative bacteria that grows preferentially in the vacuoles of the host cell. It is the etiological agent of Q FEVER.'),
  'D016998': NtO(MH='Helicobacter', MS='A genus of gram-negative, spiral-shaped bacteria that has been isolated from the intestinal tract of mammals, including humans. It has been associated with PEPTIC ULCER.'),
  'D016999': NtO(MH='Sulfur-Reducing Bacteria', MS='A group of gram-negative, anaerobic bacteria that is able to oxidize acetate completely to carbon dioxide using elemental sulfur as the electron acceptor.'),
  'D017000': NtO(MH='Campylobacter coli', MS='A species of gram-negative, rod-shaped bacteria isolated from the intestinal tract of swine, poultry, and man. It may be pathogenic.'),
  'D017001': NtO(MH='Tooth Demineralization', MS="A tooth's loss of minerals, such as calcium in hydroxyapatite from the tooth matrix, caused by acidic exposure. An example of the occurrence of demineralization is in the formation of dental caries."),
  'D017003': NtO(MH='Torovirus', MS='A genus of the family CORONAVIRIDAE characterized by enveloped, peplomer-bearing particles containing an elongated tubular nucleocapsid with helical symmetry. Toroviruses have been found in association with enteric infections in horses (Berne virus), cattle (Breda virus), swine, and humans. Transmission probably takes place via the fecal-oral route.'),
  'D017004': NtO(MH='Codependency, Psychological', MS='A relational pattern in which a person attempts to derive a sense of purpose through relationships with others.'),
  'D017005': NtO(MH='Dental Debonding', MS='Techniques used for removal of bonded orthodontic appliances, restorations, or fixed dentures from teeth.'),
  'D017006': NtO(MH='Rotator Cuff', MS='The musculotendinous sheath formed by the supraspinatus, infraspinatus, subscapularis, and teres minor muscles. These help stabilize the HUMERAL HEAD in the GLENOID CAVITY of the SCAPULA and allow for rotation of the SHOULDER JOINT about its longitudinal axis.'),
  'D017007': NtO(MH='Duty to Warn', MS="A health professional's obligation to breach patient CONFIDENTIALITY to warn third parties of the danger of their being assaulted or of contracting a serious infection."),
  'D017008': NtO(MH='Negotiating', MS='The process of bargaining in order to arrive at an agreement or compromise on a matter of importance to the parties involved. It also applies to the hearing and determination of a case by a third party chosen by the parties in controversy, as well as the interposing of a third party to reconcile the parties in controversy.'),
  'D017009': NtO(MH='Professional Autonomy', MS='The quality or state of being independent and self-directing, especially in making decisions, enabling professionals to exercise judgment as they see fit during the performance of their jobs.'),
  'D017010': NtO(MH='Reminder Systems', MS='Systems used to prompt or aid the memory. The systems can be computerized reminders, color coding, telephone calls, or devices such as letters and postcards.'),
  'D017011': NtO(MH='Streptococcus suis', MS='A species of STREPTOCOCCUS isolated from pigs. It is a pathogen of swine but rarely occurs in humans.'),
  'D017012': NtO(MH='Methanobacteriales', MS='An order of anaerobic, coccoid to rod-shaped methanogens, in the kingdom EURYARCHAEOTA. They are nonmotile, do not catabolize carbohydrates, proteinaceous material, or organic compounds other than formate or carbon monoxide, and are widely distributed in nature.'),
  'D017013': NtO(MH='Methanobacteriaceae', MS='A family of anaerobic, coccoid to rod-shaped METHANOBACTERIALES. Cell membranes are composed mainly of polyisoprenoid hydrocarbons ether-linked to glycerol. Its organisms are found in anaerobic habitats throughout nature.'),
  'D017014': NtO(MH='Methanobacterium', MS='A genus of anaerobic, rod-shaped METHANOBACTERIACEAE. Its organisms are nonmotile and use ammonia as the sole source of nitrogen. These methanogens are found in aquatic sediments, soil, sewage, and the gastrointestinal tract of animals.'),
  'D017015': NtO(MH='Methanococcales', MS="An order of anaerobic methanogens in the kingdom EURYARCHAEOTA. They are pseudosarcina, coccoid or sheathed rod-shaped and catabolize methyl groups. The cell wall is composed of protein. The order includes one family, METHANOCOCCACEAE. (From Bergey's Manual of Systemic Bacteriology, 1989)"),
  'D017016': NtO(MH='Methanococcaceae', MS='A family of anaerobic METHANOCOCCALES whose organisms are motile by means of flagella. These methanogens use carbon dioxide as an electron acceptor.'),
  'D017017': NtO(MH='Methanococcus', MS='A genus of anaerobic coccoid METHANOCOCCACEAE whose organisms are motile by means of polar tufts of flagella. These methanogens are found in salt marshes, marine and estuarine sediments, and the intestinal tract of animals.'),
  'D017018': NtO(MH='Methanomicrobiales', MS='An order of anaerobic, highly specialized methanogens, in the kingdom EURYARCHAEOTA. Its organisms are nonmotile or motile, with cells occurring as coccoid bodies, pseudosarcina, or rods. Families include METHANOMICROBIACEAE, Methanocorpusculaceae, and Methanospirillaceae.'),
  'D017019': NtO(MH='Methanosarcinaceae', MS='A family of anaerobic METHANOSARCINALES whose cells are mesophilic or thermophilic and appear as irregular spheroid bodies or sheathed rods. These methanogens are found in any anaerobic environment including aquatic sediments, anaerobic sewage digesters and gastrointestinal tracts. There are four genera: METHANOSARCINA, Methanolobus, Methanothrix, and Methanococcoides.'),
  'D017020': NtO(MH='Methanosarcina', MS='A genus of anaerobic, irregular spheroid-shaped METHANOSARCINALES whose organisms are nonmotile. Endospores are not formed. These archaea derive energy via formation of methane from acetate, methanol, mono-, di-, and trimethylamine, and possibly, carbon monoxide. Organisms are isolated from freshwater and marine environments.'),
  'D017021': NtO(MH='Methanosarcina barkeri', MS='A species of halophilic archaea whose organisms are nonmotile. Habitats include freshwater and marine mud, animal-waste lagoons, and the rumens of ungulates.'),
  'D017022': NtO(MH='Flow Injection Analysis', MS='The analysis of a chemical substance by inserting a sample into a carrier stream of reagent using a sample injection valve that propels the sample downstream where mixing occurs in a coiled tube, then passes into a flow-through detector and a recorder or other data handling device.'),
  'D017023': NtO(MH='Coronary Angiography', MS='Radiography of the vascular system of the heart muscle after injection of a contrast medium.'),
  'D017024': NtO(MH='Chemotherapy, Adjuvant', MS='Drug therapy given to augment or stimulate some other form of treatment such as surgery or radiation therapy. Adjuvant chemotherapy is commonly used in the therapy of cancer and can be administered before or after the primary treatment.'),
  'D017026': NtO(MH='Swainsonine', MS='An indolizidine alkaloid from the plant Swainsona canescens that is a potent alpha-mannosidase inhibitor. Swainsonine also exhibits antimetastatic, antiproliferative, and immunomodulatory activity.'),
  'D017027': NtO(MH='Protein Tyrosine Phosphatases', MS='An enzyme group that specifically dephosphorylates phosphotyrosyl residues in selected proteins. Together with PROTEIN-TYROSINE KINASE, it regulates tyrosine phosphorylation and dephosphorylation in cellular signal transduction and may play a role in cell growth control and carcinogenesis.'),
  'D017028': NtO(MH='Caregivers', MS='Persons who provide care to those who need supervision or assistance in illness or disability. They may provide the care in the home, in a hospital, or in an institution. Although caregivers include trained medical, nursing, and other health personnel, the concept also refers to parents, spouses, or other family members, friends, members of the clergy, teachers, social workers, fellow patients.'),
  'D017029': NtO(MH='Epilepsy, Complex Partial', MS='A disorder characterized by recurrent partial seizures marked by impairment of cognition. During the seizure the individual may experience a wide variety of psychic phenomenon including formed hallucinations, illusions, deja vu, intense emotional feelings, confusion, and spatial disorientation. Focal motor activity, sensory alterations and AUTOMATISM may also occur. Complex partial seizures often originate from foci in one or both temporal lobes. The etiology may be idiopathic (cryptogenic partial complex epilepsy) or occur as a secondary manifestation of a focal cortical lesion (symptomatic partial complex epilepsy). (From Adams et al., Principles of Neurology, 6th ed, pp317-8)'),
  'D017033': NtO(MH='Anabaena', MS='A genus of CYANOBACTERIA consisting of trichomes that are untapered with conspicuous constrictions at cross-walls. A firm individual sheath is absent, but a soft covering is often present. Many species are known worldwide as major components of freshwater PLANKTON and also of many saline lakes. The species ANABAENA FLOS-AQUAE is responsible for acute poisonings of various animals.'),
  'D017034': NtO(MH='Epilepsy, Frontal Lobe', MS='A localization-related (focal) form of epilepsy characterized by seizures which arise in the FRONTAL LOBE.'),
  'D017035': NtO(MH='Pravastatin', MS='An antilipemic fungal metabolite isolated from cultures of Nocardia autotrophica. It acts as a competitive inhibitor of HMG CoA reductase (HYDROXYMETHYLGLUTARYL COA REDUCTASES).'),
  'D017036': NtO(MH='Epilepsia Partialis Continua', MS='A variant of EPILEPSY characterized by continuous focal jerking of a body part over a period of hours, days, or even years without spreading to other body regions. Contractions may be aggravated by movement and are reduced, but not abolished during sleep. ELECTROENCEPHALOGRAPHY demonstrates epileptiform (spike and wave) discharges over the hemisphere opposite to the affected limb in most instances. The repetitive movements may originate from the CEREBRAL CORTEX or from subcortical structures (e.g., BRAIN STEM; BASAL GANGLIA). This condition is associated with Russian Spring and Summer encephalitis (see ENCEPHALITIS, TICK BORNE); Rasmussen syndrome (see ENCEPHALITIS); MULTIPLE SCLEROSIS; DIABETES MELLITUS; BRAIN NEOPLASMS; and CEREBROVASCULAR DISORDERS. (From Brain, 1996 April;119(pt2):393-407; Epilepsia 1993;34;Suppl 1:S29-S36; and Adams et al., Principles of Neurology, 6th ed, p319)'),
  'D017037': NtO(MH='Dinitrogenase Reductase', MS='A non-heme iron-sulfur protein isolated from Clostridium pasteurianum and other bacteria. It is a component of NITROGENASE along with molybdoferredoxin and is active in nitrogen fixation.'),
  'D017038': NtO(MH='Mannosyl-Glycoprotein Endo-beta-N-Acetylglucosaminidase', MS='A group of related enzymes responsible for the endohydrolysis of the di-N-acetylchitobiosyl unit in high-mannose-content glycopeptides and GLYCOPROTEINS.'),
  'D017041': NtO(MH='Ethics Committees', MS='Committees established by professional societies, health facilities, or other institutions to consider decisions that have bioethical implications. The role of these committees may include consultation, education, mediation, and/or review of policies and practices. Committees that consider the ethical dimensions of patient care are ETHICS COMMITTEES, CLINICAL; committees established to protect the welfare of research subjects are ETHICS COMMITTEES, RESEARCH.'),
  'D017042': NtO(MH='Phenylketonuria, Maternal', MS='A condition occurring in untreated or partially treated females with PHENYLKETONURIA when they become pregnant. This may result in damages to the FETUS, including MICROCEPHALY; MENTAL RETARDATION; congenital heart disease; FETAL GROWTH RETARDATION; and CRANIOFACIAL ABNORMALITIES. (From Am J Med Genet 1997 Mar 3;69(1):89-95)'),
  'D017043': NtO(MH='Chalazion', MS='A non-neoplastic cyst of the MEIBOMIAN GLANDS of the eyelid.'),
  'D017044': NtO(MH='Polycystic Kidney, Autosomal Recessive', MS='A genetic disorder with autosomal recessive inheritance, characterized by multiple CYSTS in both KIDNEYS and associated LIVER lesions. Serious manifestations are usually present at BIRTH with high PERINATAL MORTALITY.'),
  'D017045': NtO(MH='Lactococcus', MS='A genus of gram-positive, coccoid bacteria mainly isolated from milk and milk products. These bacteria are also found in plants and nonsterile frozen and dry foods. Previously thought to be a member of the genus STREPTOCOCCUS (group N), it is now recognized as a separate genus.'),
  'D017046': NtO(MH='Cost Savings', MS='Reductions in all or any portion of the costs of providing goods or services. Savings may be incurred by the provider or the consumer.'),
  'D017047': NtO(MH='Cost Sharing', MS='Provisions of an insurance policy that require the insured to pay some portion of covered expenses. Several forms of sharing are in use, e.g., deductibles, coinsurance, and copayments. Cost sharing does not refer to or include amounts paid in premiums for the coverage. (From Dictionary of Health Services Management, 2d ed)'),
  'D017048': NtO(MH='Health Care Costs', MS='The actual costs of providing services related to the delivery of health care, including the costs of procedures, therapies, and medications. It is differentiated from HEALTH EXPENDITURES, which refers to the amount of money paid for the services, and from fees, which refers to the amount charged, regardless of cost.'),
  'D017049': NtO(MH='Employer Health Costs', MS="That portion of total HEALTH CARE COSTS borne by an individual's or group's employing organization."),
  'D017050': NtO(MH='Episode of Care', MS='An interval of care by a health care facility or provider for a specific medical problem or condition. It may be continuous or it may consist of a series of intervals marked by one or more brief separations from care, and can also identify the sequence of care (e.g., emergency, inpatient, outpatient), thus serving as one measure of health care provided.'),
  'D017051': NtO(MH='Hospice Care', MS="Specialized health care, supportive in nature, provided to a dying person. A holistic approach is often taken, providing patients and their families with legal, financial, emotional, or spiritual counseling in addition to meeting patients' immediate physical needs. Care may be provided in the home, in the hospital, in specialized facilities (HOSPICES), or in specially designated areas of long-term care facilities. The concept also includes bereavement care for the family. (From Dictionary of Health Services Management, 2d ed)"),
  'D017052': NtO(MH='Hospital Mortality', MS='A vital statistic measuring or recording the rate of death from any cause in hospitalized populations.'),
  'D017053': NtO(MH='Infection Control', MS='Programs of disease surveillance, generally within health care facilities, designed to investigate, prevent, and control the spread of infections and their causative microorganisms.'),
  'D017054': NtO(MH='Insurance, Medigap', MS='A supplemental health insurance policy sold by private insurance companies and designed to pay for health care costs and services that are not paid for either by Medicare alone or by a combination of Medicare and existing private health insurance benefits. (From Facts on File Dictionary of Health Care Management, 1988)'),
  'D017056': NtO(MH='Medical Laboratory Personnel', MS='Health care professionals, technicians, and assistants staffing LABORATORIES in research or health care facilities.'),
  'D017057': NtO(MH='Medically Uninsured', MS='Individuals or groups with no or inadequate health insurance coverage. Those falling into this category usually comprise three primary groups: the medically indigent (MEDICAL INDIGENCY); those whose clinical condition makes them medically uninsurable; and the working uninsured.'),
  'D017058': NtO(MH='Mexican Americans', MS='Persons living in the United States of Mexican descent.'),
  'D017059': NtO(MH='Models, Econometric', MS='The application of mathematical formulas and statistical techniques to the testing and quantifying of economic theories and the solution of economic problems.'),
  'D017060': NtO(MH='Patient Satisfaction', MS='The degree to which the individual regards the health care service or product or the manner in which it is delivered by the provider as useful, effective, or beneficial.'),
  'D017061': NtO(MH='Prepaid Health Plans', MS='Contracts between an insurer and a subscriber or a group of subscribers whereby a specified set of health benefits is provided in return for a periodic premium.'),
  'D017062': NtO(MH='Small-Area Analysis', MS='A method of analyzing the variation in utilization of health care in small geographic or demographic areas. It often studies, for example, the usage rates for a given service or procedure in several small areas, documenting the variation among the areas. By comparing high- and low-use areas, the analysis attempts to determine whether there is a pattern to such use and to identify variables that are associated with and contribute to the variation.'),
  'D017063': NtO(MH='Outcome Assessment, Health Care', MS='Research aimed at assessing the quality and effectiveness of health care as measured by the attainment of a specified end result or outcome. Measures include parameters such as improved health, lowered morbidity or mortality, and improvement of abnormal states (such as elevated blood pressure).'),
  'D017064': NtO(MH='Process Assessment, Health Care', MS='An evaluation procedure that focuses on how care is delivered, based on the premise that there are standards of performance for activities undertaken in delivering patient care, in which the specific actions taken, events occurring, and human interactions are compared with accepted standards.'),
  'D017065': NtO(MH='Practice Guideline', MS='Work consisting of a set of directions or principles to assist the health care practitioner with patient care decisions about appropriate diagnostic, therapeutic, or other clinical procedures for specific clinical circumstances. Practice guidelines may be developed by government agencies at any level, institutions, organizations such as professional societies or governing boards, or by the convening of expert panels. They can provide a foundation for assessing and evaluating the quality and effectiveness of health care in terms of measuring improved health, reduction of variation in services or procedures performed, and reduction of variation in outcomes of health care delivered.'),
  'D017072': NtO(MH='Neostriatum', MS='The phylogenetically newer part of the CORPUS STRIATUM consisting of the CAUDATE NUCLEUS and PUTAMEN. It is often called simply the striatum.'),
  'D017073': NtO(MH='Atherectomy', MS='Endovascular procedure in which atheromatous plaque is excised by a cutting or rotating catheter. It differs from balloon and laser angioplasty procedures which enlarge vessels by dilation but frequently do not remove much plaque. If the plaque is removed by surgical excision under general anesthesia rather than by an endovascular procedure through a catheter, it is called ENDARTERECTOMY.'),
  'D017074': NtO(MH='Common Variable Immunodeficiency', MS='Heterogeneous group of immunodeficiency syndromes characterized by hypogammaglobulinemia of most isotypes, variable B-cell defects, and the presence of recurrent bacterial infections.'),
  'D017075': NtO(MH='Laser Coagulation', MS='The use of green light-producing LASERS to stop bleeding. The green light is selectively absorbed by HEMOGLOBIN, thus triggering BLOOD COAGULATION.'),
  'D017076': NtO(MH='Computer-Aided Design', MS='The use of computers for designing and/or manufacturing of anything, including drugs, surgical procedures, orthotics, and prosthetics.'),
  'D017077': NtO(MH='Culture Media, Conditioned', MS='Culture media containing biologically active components obtained from previously cultured cells or tissues that have released into the media substances affecting certain cell functions (e.g., growth, lysis).'),
  'D017078': NtO(MH='Laser-Doppler Flowmetry', MS='A method of non-invasive, continuous measurement of MICROCIRCULATION. The technique is based on the values of the DOPPLER EFFECT of low-power laser light scattered randomly by static structures and moving tissue particulates.'),
  'D017079': NtO(MH='Exercise Tolerance', MS='The exercise capacity of an individual as measured by endurance (maximal exercise duration and/or maximal attained work load) during an EXERCISE TEST.'),
  'D017081': NtO(MH='Cholecystectomy, Laparoscopic', MS='Excision of the gallbladder through an abdominal incision using a laparoscope.'),
  'D017082': NtO(MH='Portal Pressure', MS='The venous pressure measured in the PORTAL VEIN.'),
  'D017084': NtO(MH='Otoacoustic Emissions, Spontaneous', MS='Self-generated faint acoustic signals from the inner ear (COCHLEA) without external stimulation. These faint signals can be recorded in the EAR CANAL and are indications of active OUTER AUDITORY HAIR CELLS. Spontaneous otoacoustic emissions are found in all classes of land vertebrates.'),
  'D017085': NtO(MH='alpha-Thalassemia', MS='A disorder characterized by reduced synthesis of the alpha chains of hemoglobin. The severity of this condition can vary from mild anemia to death, depending on the number of genes deleted.'),
  'D017086': NtO(MH='beta-Thalassemia', MS="A disorder characterized by reduced synthesis of the beta chains of hemoglobin. There is retardation of hemoglobin A synthesis in the heterozygous form (thalassemia minor), which is asymptomatic, while in the homozygous form (thalassemia major, Cooley's anemia, Mediterranean anemia, erythroblastic anemia), which can result in severe complications and even death, hemoglobin A synthesis is absent."),
  'D017087': NtO(MH='Animal Rights', MS='The moral and ethical bases of the protection of animals from cruelty and abuse. The rights are extended to domestic animals, laboratory animals, and wild animals.'),
  'D017088': NtO(MH='AIDS-Related Opportunistic Infections', MS="Opportunistic infections found in patients who test positive for human immunodeficiency virus (HIV). The most common include PNEUMOCYSTIS PNEUMONIA, Kaposi's sarcoma, cryptosporidiosis, herpes simplex, toxoplasmosis, cryptococcosis, and infections with Mycobacterium avium complex, Microsporidium, and Cytomegalovirus."),
  'D017089': NtO(MH='Ankyrin Repeat', MS='Protein motif that contains a 33-amino acid long sequence that often occurs in tandem arrays. This repeating sequence of 33-amino acids was discovered in ANKYRIN where it is involved in interaction with the anion exchanger (ANION EXCHANGE PROTEIN 1, ERYTHROCYTE). Ankyrin repeats cooperatively fold into domains that mediate molecular recognition via protein-protein interactions.'),
  'D017090': NtO(MH='Occlusal Splints', MS='Rigid or flexible appliances that overlay the occlusal surfaces of the teeth. They are used to treat OBSTRUCTIVE SLEEP APNEA, clenching, BRUXISM, and their sequelae, and to provide temporary relief from muscle or temporomandibular joint pain.'),
  'D017091': NtO(MH='Colitis, Ischemic', MS='Inflammation of the COLON due to colonic ISCHEMIA resulting from alterations in systemic circulation or local vasculature.'),
  'D017092': NtO(MH='Porphyria, Erythropoietic', MS='An autosomal recessive porphyria that is due to a deficiency of UROPORPHYRINOGEN III SYNTHASE in the BONE MARROW; also known as congenital erythropoietic porphyria. This disease is characterized by SPLENOMEGALY; ANEMIA; photosensitivity; cutaneous lesions; accumulation of hydroxymethylbilane; and increased excretion of UROPORPHYRINS and COPROPORPHYRINS.'),
  'D017093': NtO(MH='Liver Failure', MS="Severe inability of the LIVER to perform its normal metabolic functions, as evidenced by severe JAUNDICE and abnormal serum levels of AMMONIA; BILIRUBIN; ALKALINE PHOSPHATASE; ASPARTATE AMINOTRANSFERASE; LACTATE DEHYDROGENASES; and albumin/globulin ratio. (Blakiston's Gould Medical Dictionary, 4th ed)"),
  'D017094': NtO(MH='Porphyrias, Hepatic', MS='A group of metabolic diseases due to deficiency of one of a number of LIVER enzymes in the biosynthetic pathway of HEME. They are characterized by the accumulation and increased excretion of PORPHYRINS or its precursors. Clinical features include neurological symptoms (PORPHYRIA, ACUTE INTERMITTENT), cutaneous lesions due to photosensitivity (PORPHYRIA CUTANEA TARDA), or both (HEREDITARY COPROPORPHYRIA). Hepatic porphyrias can be hereditary or acquired as a result of toxicity to the hepatic tissues.'),
  'D017096': NtO(MH='Prion Diseases', MS='A group of genetic, infectious, or sporadic degenerative human and animal nervous system disorders associated with abnormal PRIONS. These diseases are characterized by conversion of the normal prion protein to an abnormal configuration via a post-translational process. In humans, these conditions generally feature DEMENTIA; ATAXIA; and a fatal outcome. Pathologic features include a spongiform encephalopathy without evidence of inflammation. The older literature occasionally refers to these as unconventional SLOW VIRUS DISEASES. (From Proc Natl Acad Sci USA 1998 Nov 10;95(23):13363-83)'),
  'D017097': NtO(MH='Electric Impedance', MS='The resistance to the flow of either alternating or direct electrical current.'),
  'D017098': NtO(MH='IgA Deficiency', MS='A dysgammaglobulinemia characterized by a deficiency of IMMUNOGLOBULIN A.'),
  'D017099': NtO(MH='IgG Deficiency', MS='A dysgammaglobulinemia characterized by a deficiency of IMMUNOGLOBULIN G.'),
  'D017100': NtO(MH='Bacteriophage P22', MS='A species of temperate bacteriophage in the genus P22-like viruses, family PODOVIRIDAE, that infects SALMONELLA species. The genome consists of double-stranded DNA, terminally redundant, and circularly permuted.'),
  'D017101': NtO(MH='Bacteriophage P1', MS='A species of temperate bacteriophage in the genus P1-like viruses, family MYOVIRIDAE, which infects E. coli. It is the largest of the COLIPHAGES and consists of double-stranded DNA, terminally redundant, and circularly permuted.'),
  'D017102': NtO(MH='Fracture Healing', MS='The physiological restoration of bone tissue and function after a fracture. It includes BONY CALLUS formation and normal replacement of bone tissue.'),
  'D017103': NtO(MH='Bacillus Phages', MS='Viruses whose host is Bacillus. Frequently encountered Bacillus phages include bacteriophage phi 29 and bacteriophage phi 105.'),
  'D017104': NtO(MH='Bacteriophage M13', MS='Temperate bacteriophage of the genus INOVIRUS which infects enterobacteria, especially E. coli. It is a filamentous phage consisting of single-stranded DNA and is circularly permuted.'),
  'D017105': NtO(MH='Pseudomonas Phages', MS='Viruses whose host is Pseudomonas. A frequently encountered Pseudomonas phage is BACTERIOPHAGE PHI 6.'),
  'D017108': NtO(MH='Streptococcus Phages', MS='Viruses whose host is Streptococcus.'),
  'D017109': NtO(MH='Akathisia, Drug-Induced', MS='A condition associated with the use of certain medications and characterized by an internal sense of motor restlessness often described as an inability to resist the urge to move.'),
  'D017110': NtO(MH='Freeze Substitution', MS='A modification of the freeze-drying method in which the ice within the frozen tissue is replaced by alcohol or other solvent at a very low temperature.'),
  'D017111': NtO(MH='Phodopus', MS='A genus of hamsters characterized by small size, very short tail, and short, broad feet with hairy soles.'),
  'D017114': NtO(MH='Liver Failure, Acute', MS='A form of rapid-onset LIVER FAILURE, also known as fulminant hepatic failure, caused by severe liver injury or massive loss of HEPATOCYTES. It is characterized by sudden development of liver dysfunction and JAUNDICE. Acute liver failure may progress to exhibit cerebral dysfunction even HEPATIC COMA depending on the etiology that includes hepatic ISCHEMIA, drug toxicity, malignant infiltration, and viral hepatitis such as post-transfusion HEPATITIS B and HEPATITIS C.'),
  'D017115': NtO(MH='Catheter Ablation', MS='Removal of tissue with electrical current delivered via electrodes positioned at the distal end of a catheter. Energy sources are commonly direct current (DC-shock) or alternating current at radiofrequencies (usually 750 kHz). The technique is used most often to ablate the AV junction and/or accessory pathways in order to interrupt AV conduction and produce AV block in the treatment of various tachyarrhythmias.'),
  'D017116': NtO(MH='Low Back Pain', MS='Acute or chronic pain in the lumbar or sacral regions, which may be associated with musculo-ligamentous SPRAINS AND STRAINS; INTERVERTEBRAL DISK DISPLACEMENT; and other conditions.'),
  'D017118': NtO(MH='Porphyria, Acute Intermittent', MS='An autosomal dominant porphyria that is due to a deficiency of HYDROXYMETHYLBILANE SYNTHASE in the LIVER, the third enzyme in the 8-enzyme biosynthetic pathway of HEME. Clinical features are recurrent and life-threatening neurologic disturbances, ABDOMINAL PAIN, and elevated level of AMINOLEVULINIC ACID and PORPHOBILINOGEN in the urine.'),
  'D017119': NtO(MH='Porphyria Cutanea Tarda', MS='An autosomal dominant or acquired porphyria due to a deficiency of UROPORPHYRINOGEN DECARBOXYLASE in the LIVER. It is characterized by photosensitivity and cutaneous lesions with little or no neurologic symptoms. Type I is the acquired form and is strongly associated with liver diseases and hepatic toxicities caused by alcohol or estrogenic steroids. Type II is the familial form.'),
  'D017120': NtO(MH='Nystagmus, Optokinetic', MS='Normal nystagmus produced by looking at objects moving across the field of vision.'),
  'D017121': NtO(MH='Porphyria, Hepatoerythropoietic', MS='An autosomal recessive cutaneous porphyria that is due to a deficiency of UROPORPHYRINOGEN DECARBOXYLASE in both the LIVER and the BONE MARROW. Similar to PORPHYRIA CUTANEA TARDA, this disorder is caused by defects in the fifth enzyme in the 8-enzyme biosynthetic pathway of HEME, but is a homozygous enzyme deficiency with less than 10% of the normal enzyme activity. Cutaneous lesions are severe and mutilating.'),
  'D017122': NtO(MH='Bacteriophage T4', MS='Virulent bacteriophage and type species of the genus T4-like phages, in the family MYOVIRIDAE. It infects E. coli and is the best known of the T-even phages. Its virion contains linear double-stranded DNA, terminally redundant and circularly permuted.'),
  'D017123': NtO(MH='Bacteriophage T7', MS='Virulent bacteriophage and type species of the genus T7-like phages, in the family PODOVIRIDAE, that infects E. coli. It consists of linear double-stranded DNA, terminally redundant, and non-permuted.'),
  'D017124': NtO(MH='Conserved Sequence', MS='A sequence of amino acids in a polypeptide or of nucleotides in DNA or RNA that is similar across multiple species. A known set of conserved sequences is represented by a CONSENSUS SEQUENCE. AMINO ACID MOTIFS are often composed of conserved sequences.'),
  'D017125': NtO(MH='Genes, Protozoan', MS='The functional hereditary units of protozoa.'),
  'D017126': NtO(MH='Bacteriophage phi 6', MS='Virulent bacteriophage and sole member of the genus Cystovirus that infects Pseudomonas species. The virion has a segmented genome consisting of three pieces of doubled-stranded DNA and also a unique lipid-containing envelope.'),
  'D017127': NtO(MH='Glycation End Products, Advanced', MS='A heterogeneous group of compounds derived from rearrangements, oxidation, and cross-linking reactions that follow from non-enzymatic glycation of amino groups in proteins. They are also know as Maillard products. Their accumulation in vivo accelerates under hyperglycemic, oxidative, or inflammatory conditions. Heat also accelerates the formation of advanced glycation end products (AGEs) such seen with the browning of food during cooking under or over high heat.'),
  'D017128': NtO(MH='Embolectomy', MS='Surgical removal of an obstructing clot or foreign material which has been transported from a distant vessel by the bloodstream. Removal of a clot at its original site is called THROMBECTOMY.'),
  'D017129': NtO(MH='Anisakiasis', MS='Infection with roundworms of the genus ANISAKIS. Human infection results from the consumption of fish harboring roundworm larvae. The worms may cause acute NAUSEA; VOMITING; or penetrate into the wall of the DIGESTIVE TRACT where they give rise to EOSINOPHILIC GRANULOMA in the STOMACH; INTESTINES; or the OMENTUM.'),
  'D017130': NtO(MH='Angioplasty', MS='Reconstruction or repair of a blood vessel, which includes the widening of a pathological narrowing of an artery or vein by the removal of atheromatous plaque material and/or the endothelial lining as well, or by dilatation (BALLOON ANGIOPLASTY) to compress an ATHEROMA. Except for ENDARTERECTOMY, usually these procedures are performed via catheterization as minimally invasive ENDOVASCULAR PROCEDURES.'),
  'D017131': NtO(MH='Thrombectomy', MS='Surgical removal of an obstructing clot or foreign material from a blood vessel at the point of its formation. Removal of a clot arising from a distant site is called EMBOLECTOMY.'),
  'D017132': NtO(MH='Workplace', MS='Place or physical location of work or employment.'),
  'D017135': NtO(MH='Desogestrel', MS='A synthetic progestational hormone used often as the progestogenic component of combined oral contraceptive agents (ORAL CONTRACEPTIVES, COMBINED).'),
  'D017136': NtO(MH='Ion Transport', MS='The movement of ions across energy-transducing cell membranes. Transport can be active, passive or facilitated. Ions may travel by themselves (uniport), or as a group of two or more ions in the same (symport) or opposite (antiport) directions.'),
  'D017137': NtO(MH='Nonoxynol', MS='Nonionic surfactant mixtures varying in the number of repeating ethoxy (oxy-1,2-ethanediyl) groups. They are used as detergents, emulsifiers, wetting agents, defoaming agents, etc. Nonoxynol-9, the compound with 9 repeating ethoxy groups, is a spermatocide, formulated primarily as a component of vaginal foams and creams.'),
  'D017143': NtO(MH='Fuzzy Logic', MS='Approximate, quantitative reasoning that is concerned with the linguistic ambiguity which exists in natural or synthetic language. At its core are variables such as good, bad, and young as well as modifiers such as more, less, and very. These ordinary terms represent fuzzy sets in a particular problem. Fuzzy logic plays a key role in many medical expert systems.'),
  'D017144': NtO(MH='Focus Groups', MS='A method of data collection and a QUALITATIVE RESEARCH tool in which a small group of individuals are brought together and allowed to interact in a discussion of their opinions about topics, issues, or questions.'),
  'D017145': NtO(MH='Models, Educational', MS='Theoretical models which propose methods of learning or teaching as a basis or adjunct to changes in attitude or behavior. These educational interventions are usually applied in the fields of health and patient education but are not restricted to patient care.'),
  'D017146': NtO(MH='Models, Organizational', MS='Theoretical representations and constructs that describe or explain the structure and hierarchy of relationships and interactions within or between formal organizational entities or informal social groups.'),
  'D017147': NtO(MH='Defibrillators, Implantable', MS='Implantable devices which continuously monitor the electrical activity of the heart and automatically detect and terminate ventricular tachycardia (TACHYCARDIA, VENTRICULAR) and VENTRICULAR FIBRILLATION. They consist of an impulse generator, batteries, and electrodes.'),
  'D017148': NtO(MH='Infusions, Intraosseous', MS='The administration of medication or fluid through a needle directly into the bone marrow. The technique is especially useful in the management of pediatric emergencies when intravenous access to the systemic circulation is difficult.'),
  'D017149': NtO(MH='Private Sector', MS='That distinct portion of the institutional, industrial, or economic structure of a country that is controlled or owned by non-governmental, private interests.'),
  'D017150': NtO(MH='Public Sector', MS="The area of a nation's economy that is tax-supported and under government control."),
  'D017151': NtO(MH='Antibodies, Catalytic', MS='Antibodies that can catalyze a wide variety of chemical reactions. They are characterized by high substrate specificity and share many mechanistic features with enzymes.'),
  'D017152': NtO(MH='Antibodies, Antiphospholipid', MS='Autoantibodies directed against phospholipids. These antibodies are characteristically found in patients with systemic lupus erythematosus (LUPUS ERYTHEMATOSUS, SYSTEMIC;), ANTIPHOSPHOLIPID SYNDROME; related autoimmune diseases, some non-autoimmune diseases, and also in healthy individuals.'),
  'D017153': NtO(MH='Antibodies, Anticardiolipin', MS='Antiphospholipid antibodies found in association with systemic lupus erythematosus (LUPUS ERYTHEMATOSUS, SYSTEMIC;), ANTIPHOSPHOLIPID SYNDROME; and in a variety of other diseases as well as in healthy individuals. The antibodies are detected by solid-phase IMMUNOASSAY employing the purified phospholipid antigen CARDIOLIPIN.'),
  'D017154': NtO(MH='Stromal Cells', MS='Connective tissue cells of an organ found in the loose connective tissue. These are most often associated with the uterine mucosa and the ovary as well as the hematopoietic system and elsewhere.'),
  'D017155': NtO(MH='Wettability', MS='The quality or state of being wettable or the degree to which something can be wet. This is also the ability of any solid surface to be wetted when in contact with a liquid whose surface tension is reduced so that the liquid spreads over the surface of the solid.'),
  'D017156': NtO(MH='Photoplethysmography', MS='Plethysmographic determination in which the intensity of light reflected from the skin surface and the red cells below is measured to determine the blood volume of the respective area. There are two types, transmission and reflectance.'),
  'D017157': NtO(MH='Adenophorea', MS='A subclass of nematodes characterized by reduced or absent caudal papillae and an excretory system lacking lateral canals. Its organisms are usually infective to their final host.'),
  'D017158': NtO(MH='Enoplida', MS='An order of nematodes of the subclass ADENOPHOREA. Its organisms commonly have a cylindrical esophagus. The superfamilies of this order are DIOCTOPHYMATOIDEA; MERMITHOIDEA; and TRICHUROIDEA.'),
  'D017159': NtO(MH='Dioctophymatoidea', MS='A superfamily of nematodes of the order ENOPLIDA. Characteristics include a well developed cylindrical esophagus.'),
  'D017160': NtO(MH='Trichinella spiralis', MS='A parasite of carnivorous mammals that causes TRICHINELLOSIS. It is especially common in rats and in swine fed uncooked garbage. Human infection is initiated by the consumption of raw or insufficiently cooked pork or other meat containing the encysted larvae.'),
  'D017162': NtO(MH='Ascaridida', MS='An order of nematodes of the subclass SECERNENTEA. Its organisms possess two or three pairs of dorsolateral caudal papillae.'),
  'D017163': NtO(MH='Anisakis', MS='A genus of nematodes of the superfamily ASCARIDOIDEA. Its organisms are found in the stomachs of marine animals and birds. Human infection occurs by ingestion of raw fish that contain larvae.'),
  'D017164': NtO(MH='Ascaris lumbricoides', MS='A species of parasitic nematode that is the largest found in the human intestine. Its distribution is worldwide, but it is more prevalent in areas of poor sanitation. Human infection with A. lumbricoides is acquired by swallowing fully embryonated eggs from contaminated soil.'),
  'D017165': NtO(MH='Ascaris suum', MS='A species of parasitic nematode usually found in domestic pigs and a few other animals. Human infection can also occur, presumably as result of handling pig manure, and can lead to intestinal obstruction.'),
  'D017166': NtO(MH='Toxocara canis', MS='A species of parasitic nematode found in the intestine of dogs. Lesions in the brain, liver, eye, kidney, and lung are caused by migrating larvae. In humans, these larvae do not follow normal patterns and may produce visceral larva migrans (LARVA MIGRANS, VISCERAL).'),
  'D017167': NtO(MH='Oxyurida', MS='An order of nematodes of the subclass SECERNENTEA. Its organisms are characterized by a single, ventral excretory gland and a relatively stout, short body.'),
  'D017168': NtO(MH='Rhabditida', MS='An order of nematodes of the subclass SECERNENTEA. Its organisms are characterized by an annulated or smooth cuticle and the absence of caudal glands.'),
  'D017169': NtO(MH='Neoplasms, Post-Traumatic', MS='Tumors, cancer or other neoplasms caused by or resulting from trauma or other non-radiation injuries.'),
  'D017170': NtO(MH='Strongyloides ratti', MS='A species of parasitic nematode found in rats.'),
  'D017171': NtO(MH='Strongyloides stercoralis', MS='A species of parasitic nematode widely distributed in tropical and subtropical countries. The females and their larvae inhabit the mucosa of the intestinal tract, where they cause ulceration and diarrhea.'),
  'D017172': NtO(MH='Rhabditoidea', MS='A superfamily of nematodes of the order RHABDITIDA. Characteristics include an open tube stoma and an excretory system with lateral canals.'),
  'D017173': NtO(MH='Caenorhabditis elegans', MS='A species of nematode that is widely used in biological, biochemical, and genetic studies.'),
  'D017174': NtO(MH='Spirurida', MS='An order of nematodes of the subclass SECERNENTEA. Characteristics include ventral or caudoventral papillae and an esophagus divided into anterior muscular and posterior glandular parts.'),
  'D017175': NtO(MH='Camallanina', MS='A suborder of nematodes characterized by larvae lacking cephalic hooks and a tail that is generally long and pointed.'),
  'D017176': NtO(MH='Dracunculoidea', MS='A superfamily of nematodes of the suborder CAMALLANINA. Its organisms possess a poorly developed buccal cavity and a rudimentary esophagus and intestine.'),
  'D017177': NtO(MH='Spirurina', MS='A suborder of nematodes characterized by larvae possessing cephalic hooks and multinucleated esophageal glands.'),
  'D017178': NtO(MH='Brugia malayi', MS='A species of parasitic nematode causing Malayan filariasis and having a distribution centering roughly on the Malay peninsula. The life cycle of B. malayi is similar to that of WUCHERERIA BANCROFTI, except that in most areas the principal mosquito vectors belong to the genus Mansonia.'),
  'D017179': NtO(MH='Brugia pahangi', MS='A species of parasitic nematode found in man and other mammals. It has been reported from Malaya and East Pakistan and may produce symptoms of tropical eosinophilia.'),
  'D017180': NtO(MH='Tachycardia, Ventricular', MS='An abnormally rapid ventricular rhythm usually in excess of 150 beats per minute. It is generated within the ventricle below the BUNDLE OF HIS, either as autonomic impulse formation or reentrant impulse conduction. Depending on the etiology, onset of ventricular tachycardia can be paroxysmal (sudden) or nonparoxysmal, its wide QRS complexes can be uniform or polymorphic, and the ventricular beating may be independent of the atrial beating (AV dissociation).'),
  'D017181': NtO(MH='Onchocerca volvulus', MS='A species of parasitic nematodes widely distributed throughout central Africa and also found in northern South America, southern Mexico, and Guatemala. Its intermediate host and vector is the blackfly or buffalo gnat.'),
  'D017182': NtO(MH='Setaria Nematode', MS='A genus of parasitic nematodes found in the peritoneal cavity of wild or domestic cattle or equines.'),
  'D017183': NtO(MH='Strongylida', MS='An order of nematodes of the subclass SECERNENTEA. Characteristics include an H-shaped excretory system with two subventral glands.'),
  'D017184': NtO(MH='Nematospiroides', MS='A genus of nematodes found mainly in mice. Its organisms are used in biomedical research.'),
  'D017185': NtO(MH='Angiostrongylus cantonensis', MS='A species of parasitic nematodes distributed throughout the Pacific islands that infests the lungs of domestic rats. Human infection, caused by consumption of raw slugs and land snails, results in eosinophilic meningitis.'),
  'D017186': NtO(MH='Titrimetry', MS='The determination of the concentration of a given component in solution (the analyte) by addition of a liquid reagent of known strength (the titrant) until an equivalence point is reached (when the reactants are present in stoichiometric proportions). Often an indicator is added to make the equivalence point visible (e.g., a change in color).'),
  'D017187': NtO(MH='Tylenchida', MS='An order of nematodes consisting of many species which are plant parasites. Female worms lay eggs that hatch either in soil or in the host plant.'),
  'D017188': NtO(MH='Adenophorea Infections', MS='Infections with nematodes of the subclass ADENOPHOREA.'),
  'D017189': NtO(MH='Enoplida Infections', MS='Infections with nematodes of the order ENOPLIDA.'),
  'D017190': NtO(MH='Secernentea Infections', MS='Infections with nematodes of the subclass SECERNENTEA.'),
  'D017191': NtO(MH='Ascaridida Infections', MS='Infections with nematodes of the order ASCARIDIDA.'),
  'D017192': NtO(MH='Skin Diseases, Bacterial', MS='Skin diseases caused by bacteria.'),
  'D017193': NtO(MH='Skin Diseases, Viral', MS='Skin diseases caused by viruses.'),
  'D017194': NtO(MH='Oxyurida Infections', MS='Infections with nematodes of the order OXYURIDA.'),
  'D017195': NtO(MH='Adolescent Nutritional Physiological Phenomena', MS='Nutritional physiology of children aged 13-18 years.'),
  'D017196': NtO(MH='Rhabditida Infections', MS='Infections with nematodes of the order RHABDITIDA.'),
  'D017201': NtO(MH='Virus Shedding', MS='The expelling of virus particles from the body. Important routes include the respiratory tract, genital tract, and intestinal tract. Virus shedding is an important means of vertical transmission (INFECTIOUS DISEASE TRANSMISSION, VERTICAL).'),
  'D017202': NtO(MH='Myocardial Ischemia', MS='A disorder of cardiac function caused by insufficient blood flow to the muscle tissue of the heart. The decreased blood flow may be due to narrowing of the coronary arteries (CORONARY ARTERY DISEASE), to obstruction by a thrombus (CORONARY THROMBOSIS), or less commonly, to diffuse narrowing of arterioles and other small vessels within the heart. Severe interruption of the blood supply to the myocardial tissue may result in necrosis of cardiac muscle (MYOCARDIAL INFARCTION).'),
  'D017203': NtO(MH='Interview', MS='Work consisting of a conversation with an individual regarding his or her background and other personal and professional details, opinions on specific subjects posed by the interviewer, etc.'),
  'D017204': NtO(MH='Angelman Syndrome', MS='A syndrome characterized by multiple abnormalities, MENTAL RETARDATION, and movement disorders. Present usually are skull and other abnormalities, frequent infantile spasms (SPASMS, INFANTILE); easily provoked and prolonged paroxysms of laughter (hence "happy"); jerky puppetlike movements (hence "puppet"); continuous tongue protrusion; motor retardation; ATAXIA; MUSCLE HYPOTONIA; and a peculiar facies. It is associated with maternal deletions of chromosome 15q11-13 and other genetic abnormalities. (From Am J Med Genet 1998 Dec 4;80(4):385-90; Hum Mol Genet 1999 Jan;8(1):129-35)'),
  'D017205': NtO(MH='Spirurida Infections', MS='Infections with nematodes of the order SPIRURIDA.'),
  'D017206': NtO(MH='Strongylida Infections', MS='Infections with nematodes of the order STRONGYLIDA.'),
  'D017207': NtO(MH='Rats, Sprague-Dawley', MS='A strain of albino rat used widely for experimental purposes because of its calmness and ease of handling. It was developed by the Sprague-Dawley Animal Company.'),
  'D017208': NtO(MH='Rats, Wistar', MS='A strain of albino rat developed at the Wistar Institute that has spread widely at other institutions. This has markedly diluted the original strain.'),
  'D017209': NtO(MH='Apoptosis', MS='A regulated cell death mechanism characterized by distinctive morphologic changes in the nucleus and cytoplasm, including the endonucleolytic cleavage of genomic DNA, at regularly spaced, internucleosomal sites, i.e., DNA FRAGMENTATION.  It is genetically-programmed and serves as a balance to mitosis in regulating the size of animal tissues and in mediating pathologic processes associated with tumor growth.'),
  'D017210': NtO(MH='Tilapia', MS='A freshwater fish used as an experimental organism and for food. This genus of the family Cichlidae (CICHLIDS) inhabits Central and South America (one species extends north into Texas), West Indies, Africa, Madagascar, Syria, and coastal India.'),
  'D017211': NtO(MH='Treatment Failure', MS='A measure of the quality of health care by assessment of unsuccessful results of management and procedures used in combating disease, in individual cases or series.'),
  'D017213': NtO(MH='Root Caries', MS='Dental caries involving the tooth root, cementum, or cervical area of the tooth.'),
  'D017214': NtO(MH='Laryngeal Masks', MS='A type of oropharyngeal airway that provides an alternative to endotracheal intubation and standard mask anesthesia in certain patients. It is introduced into the hypopharynx to form a seal around the larynx thus permitting spontaneous or positive pressure ventilation without penetration of the larynx or esophagus. It is used in place of a facemask in routine anesthesia. The advantages over standard mask anesthesia are better airway control, minimal anesthetic gas leakage, a secure airway during patient transport to the recovery area, and minimal postoperative problems.'),
  'D017215': NtO(MH='Satellite Communications', MS='Communications using an active or passive satellite to extend the range of radio, television, or other electronic transmission by returning signals to earth from an orbiting satellite.'),
  'D017216': NtO(MH='Telemedicine', MS='Delivery of health services via remote telecommunications. This includes interactive consultative and diagnostic services.'),
  'D017217': NtO(MH='Blood Patch, Epidural', MS='The injection of autologous blood into the epidural space either as a prophylactic treatment immediately following an epidural puncture or for treatment of headache as a result of an epidural puncture.'),
  'D017218': NtO(MH='Cordocentesis', MS='The collecting of fetal blood samples typically via ENDOSCOPIC ULTRASOUND GUIDED FINE NEEDLE ASPIRATION from the umbilical vein.'),
  'D017219': NtO(MH='Gastric Outlet Obstruction', MS='The hindering of output from the STOMACH into the SMALL INTESTINE. This obstruction may be of mechanical or functional origin such as EDEMA from PEPTIC ULCER; NEOPLASMS; FOREIGN BODIES; or AGING.'),
  'D017220': NtO(MH='Dentin-Bonding Agents', MS='Cements that act through infiltration and polymerization within the dentinal matrix and are used for dental restoration. They can be adhesive resins themselves, adhesion-promoting monomers, or polymerization initiators that act in concert with other agents to form a dentin-bonding system.'),
  'D017221': NtO(MH='Necator americanus', MS='A common parasite of humans in the moist tropics and subtropics. These organisms attach to villi in the small intestine and suck blood causing diarrhea, anorexia, and anemia.'),
  'D017222': NtO(MH='Angioplasty, Balloon, Laser-Assisted', MS='Techniques using laser energy in combination with a balloon catheter to perform angioplasty. These procedures can take several forms including: 1, laser fiber delivering the energy while the inflated balloon centers the fiber and occludes the blood flow; 2, balloon angioplasty immediately following laser angioplasty; or 3, laser energy transmitted through angioplasty balloons that contain an internal fiber.'),
  'D017225': NtO(MH='Atherectomy, Coronary', MS='Percutaneous transluminal procedure for removing atheromatous plaque from the coronary arteries. Both directional (for removing focal atheromas) and rotational (for removing concentric atheromatous plaque) atherectomy devices have been used.'),
  'D017227': NtO(MH='Toxascariasis', MS='Infections with nematodes of the genus TOXASCARIS.'),
  'D017228': NtO(MH='Hepatocyte Growth Factor', MS='Multifunctional growth factor which regulates both cell growth and cell motility. It exerts a strong mitogenic effect on hepatocytes and primary epithelial cells. Its receptor is PROTO-ONCOGENE PROTEINS C-MET.'),
  'D017229': NtO(MH='Enterobiasis', MS='Infection with nematodes of the genus ENTEROBIUS; E. vermicularis, the pinworm of man, causes a crawling sensation and pruritus. This condition results in scratching the area, occasionally causing scarification.'),
  'D017231': NtO(MH='Proto-Oncogene Proteins c-mos', MS='Cellular proteins encoded by the c-mos genes (GENES, MOS). They function in the cell cycle to maintain MATURATION PROMOTING FACTOR in the active state and have protein-serine/threonine kinase activity. Oncogenic transformation can take place when c-mos proteins are expressed at the wrong time.'),
  'D017232': NtO(MH='Oncogene Proteins v-mos', MS='Transforming proteins coded by mos oncogenes. The v-mos proteins were originally isolated from the Moloney murine sarcoma virus (Mo-MSV).'),
  'D017233': NtO(MH='Orbit Evisceration', MS='The surgical removal of the contents of the orbit. This includes the eyeball, blood vessels, muscles, fat, nerve supply, and periosteum. It should be differentiated from EYE EVISCERATION which removes the inner contents of the eye, leaving the sclera intact.'),
  'D017234': NtO(MH='Eye Evisceration', MS='The surgical removal of the inner contents of the eye, leaving the sclera intact. It should be differentiated from ORBIT EVISCERATION which removes the entire contents of the orbit, including eyeball, blood vessels, muscles, fat, nerve supply, and periosteum.'),
  'D017236': NtO(MH='Suicide, Assisted', MS='Provision (by a physician or other health professional, or by a family member or friend) of support and/or means that gives a patient the power to terminate his or her own life. (from APA, Thesaurus of Psychological Index Terms, 8th ed).'),
  'D017237': NtO(MH='Mitochondrial Encephalomyopathies', MS='A heterogenous group of disorders characterized by alterations of mitochondrial metabolism that result in muscle and nervous system dysfunction. These are often multisystemic and vary considerably in age at onset (usually in the first or second decade of life), distribution of affected muscles, severity, and course. (From Adams et al., Principles of Neurology, 6th ed, pp984-5)'),
  'D017238': NtO(MH='Genes, Helminth', MS='The functional hereditary units of HELMINTHS.'),
  'D017239': NtO(MH='Paclitaxel', MS='A cyclodecane isolated from the bark of the Pacific yew tree, TAXUS BREVIFOLIA. It stabilizes MICROTUBULES in their polymerized form leading to cell death.'),
  'D017240': NtO(MH='Mitochondrial Myopathies', MS='A group of muscle diseases associated with abnormal mitochondria function.'),
  'D017241': NtO(MH='MELAS Syndrome', MS='A mitochondrial disorder characterized by focal or generalized seizures, episodes of transient or persistent neurologic dysfunction resembling strokes, and ragged-red fibers on muscle biopsy. Affected individuals tend to be normal at birth through early childhood, then experience growth failure, episodic vomiting, and recurrent cerebral insults resulting in visual loss and hemiparesis. The cortical lesions tend to occur in the parietal and occipital lobes and are not associated with vascular occlusion. VASCULAR HEADACHE is frequently associated and the disorder tends to be familial. (From Joynt, Clinical Neurology, 1992, Ch56, p117)'),
  'D017242': NtO(MH='Complement Factor H', MS='An important soluble regulator of the alternative pathway of complement activation (COMPLEMENT ACTIVATION PATHWAY, ALTERNATIVE). It is a 139-kDa glycoprotein expressed by the liver and secreted into the blood. It binds to COMPLEMENT C3B and makes iC3b (inactivated complement 3b) susceptible to cleavage by COMPLEMENT FACTOR I. Complement factor H also inhibits the association of C3b with COMPLEMENT FACTOR B to form the C3bB proenzyme, and promotes the dissociation of Bb from the C3bBb complex (COMPLEMENT C3 CONVERTASE, ALTERNATIVE PATHWAY).'),
  'D017243': NtO(MH='MERRF Syndrome', MS='A mitochondrial encephalomyopathy characterized clinically by a mixed seizure disorder, myoclonus, progressive ataxia, spasticity, and a mild myopathy. Dysarthria, optic atrophy, growth retardation, deafness, and dementia may also occur. This condition tends to present in childhood and to be transmitted via maternal lineage. Muscle biopsies reveal ragged-red fibers and respiratory chain enzymatic defects. (From Adams et al., Principles of Neurology, 6th ed, p986)'),
  'D017244': NtO(MH='Complement Factor I', MS='A plasma serine proteinase that cleaves the alpha-chains of C3b and C4b in the presence of the cofactors COMPLEMENT FACTOR H and C4-binding protein, respectively. It is a 66-kDa glycoprotein that converts C3b to inactivated C3b (iC3b) followed by the release of two fragments, C3c (150-kDa) and C3dg (41-kDa). It was formerly called KAF, C3bINF, or enzyme 3b inactivator.'),
  'D017245': NtO(MH='Foscarnet', MS='An antiviral agent used in the treatment of cytomegalovirus retinitis. Foscarnet also shows activity against human herpesviruses and HIV.'),
  'D017246': NtO(MH='Ophthalmoplegia, Chronic Progressive External', MS='A mitochondrial myopathy characterized by slowly progressive paralysis of the levator palpebrae, orbicularis oculi, and extraocular muscles. Ragged-red fibers and atrophy are found on muscle biopsy. Familial and sporadic forms may occur. Disease onset is usually in the first or second decade of life, and the illness slowly progresses until usually all ocular motility is lost. (From Adams et al., Principles of Neurology, 6th ed, p1422)'),
  'D017247': NtO(MH='Neutron Capture Therapy', MS='A technique for the treatment of neoplasms in which an isotope is introduced into target cells followed by irradiation with thermal neutrons.'),
  'D017248': NtO(MH='Enzyme Multiplied Immunoassay Technique', MS='An immunoenzyme test for the presence of drugs and other substances in urine and blood. The test uses enzyme linked antibodies that react only with the particular drug for which the sample is being tested.'),
  'D017250': NtO(MH='Caliciviridae Infections', MS='Virus diseases caused by CALICIVIRIDAE. They include HEPATITIS E; VESICULAR EXANTHEMA OF SWINE; acute respiratory infections in felines, rabbit hemorrhagic disease, and some cases of gastroenteritis in humans.'),
  'D017252': NtO(MH='CD3 Complex', MS='Complex of at least five membrane-bound polypeptides in mature T-lymphocytes that are non-covalently associated with one another and with the T-cell receptor (RECEPTORS, ANTIGEN, T-CELL). The CD3 complex includes the gamma, delta, epsilon, zeta, and eta chains (subunits). When antigen binds to the T-cell receptor, the CD3 complex transduces the activating signals to the cytoplasm of the T-cell. The CD3 gamma and delta chains (subunits) are separate from and not related to the gamma/delta chains of the T-cell receptor (RECEPTORS, ANTIGEN, T-CELL, GAMMA-DELTA).'),
  'D017253': NtO(MH='Neurofibromatoses', MS='A group of disorders characterized by an autosomal dominant pattern of inheritance with high rates of spontaneous mutation and multiple neurofibromas or neurilemmomas. NEUROFIBROMATOSIS 1 (generalized neurofibromatosis) accounts for approximately 95% of cases, although multiple additional subtypes (e.g., NEUROFIBROMATOSIS 2, neurofibromatosis 3, etc.) have been described. (From Neurochirurgie 1998 Nov;44(4):267-72)'),
  'D017254': NtO(MH='Leukemic Infiltration', MS='A pathologic change in leukemia in which leukemic cells permeate various organs at any stage of the disease. All types of leukemia show various degrees of infiltration, depending upon the type of leukemia. The degree of infiltration may vary from site to site. The liver and spleen are common sites of infiltration, the greatest appearing in myelocytic leukemia, but infiltration is seen also in the granulocytic and lymphocytic types. The kidney is also a common site and of the gastrointestinal system, the stomach and ileum are commonly involved. In lymphocytic leukemia the skin is often infiltrated. The central nervous system too is a common site.'),
  'D017255': NtO(MH='Acitretin', MS='An oral retinoid effective in the treatment of psoriasis. It is the major metabolite of ETRETINATE with the advantage of a much shorter half-life when compared with etretinate.'),
  'D017256': NtO(MH='Technetium Tc 99m Sestamibi', MS='A technetium imaging agent used to reveal blood-starved cardiac tissue during a heart attack.'),
  'D017257': NtO(MH='Ramipril', MS='A long-acting angiotensin-converting enzyme inhibitor. It is a prodrug that is transformed in the liver to its active metabolite ramiprilat.'),
  'D017258': NtO(MH='Medroxyprogesterone Acetate', MS='A synthetic progestin that is derived from 17-hydroxyprogesterone. It is a long-acting contraceptive that is effective both orally or by intramuscular injection and has also been used to treat breast and endometrial neoplasms.'),
  'D017259': NtO(MH='Dimaprit', MS='A histamine H2 receptor agonist that is often used to study the activity of histamine and its receptors.'),
  'D017260': NtO(MH='Receptor-CD3 Complex, Antigen, T-Cell', MS='Molecule composed of the non-covalent association of the T-cell antigen receptor (RECEPTORS, ANTIGEN, T-CELL) with the CD3 complex (CD3 COMPLEX). This association is required for the surface expression and function of both components. The molecule consists of up to seven chains: either the alpha/beta or gamma/delta chains of the T-cell receptor, and four or five chains in the CD3 complex.'),
  'D017261': NtO(MH='Glycosylphosphatidylinositols', MS='Compounds containing carbohydrate or glycosyl groups linked to phosphatidylinositols. They anchor GPI-LINKED PROTEINS or polysaccharides to cell membranes.'),
  'D017262': NtO(MH='Siderophores', MS='Low-molecular-weight compounds produced by microorganisms that aid in the transport and sequestration of ferric iron. (The Encyclopedia of Molecular Biology, 1994)'),
  'D017263': NtO(MH='Technetium Tc 99m Mertiatide', MS='A technetium diagnostic aid used in renal function determination.'),
  'D017265': NtO(MH='Procaterol', MS='A long-acting beta-2-adrenergic receptor agonist.'),
  'D017266': NtO(MH='Dental Prosthesis', MS="An artificial replacement for one or more natural teeth or part of a tooth, or associated structures, ranging from a portion of a tooth to a complete denture. The dental prosthesis is used for cosmetic or functional reasons, or both. DENTURES and specific types of dentures are also available. (From Boucher's Clinical Dental Terminology, 4th ed, p244 & Jablonski, Dictionary of Dentistry, 1992, p643)"),
  'D017267': NtO(MH='Dental Prosthesis Design', MS='The plan and delineation of dental prostheses in general or a specific dental prosthesis. It does not include DENTURE DESIGN. The framework usually consists of metal.'),
  'D017268': NtO(MH='Dental Prosthesis Retention', MS='Holding a DENTAL PROSTHESIS in place by its design, or by the use of additional devices or adhesives.'),
  'D017269': NtO(MH='Dental Prosthesis Repair', MS='The process of reuniting or replacing a broken or worn dental prosthesis or its part.'),
  'D017270': NtO(MH='Lipoprotein(a)', MS='A lipoprotein that resembles the LOW-DENSITY LIPOPROTEINS but with an extra protein moiety, APOPROTEIN (A) also known as APOLIPOPROTEIN (A), linked to APOLIPOPROTEIN B-100 on the LDL by one or two disulfide bonds. High plasma level of lipoprotein (a) is associated with increased risk of atherosclerotic cardiovascular disease.'),
  'D017271': NtO(MH='Craniomandibular Disorders', MS='Diseases or disorders of the muscles of the head and neck, with special reference to the masticatory muscles. The most notable examples are TEMPOROMANDIBULAR JOINT DISORDERS and TEMPOROMANDIBULAR JOINT DYSFUNCTION SYNDROME.'),
  'D017272': NtO(MH='Celiprolol', MS='A cardioselective beta-1 adrenergic antagonist that has intrinsic symopathomimetic activity. It is used in the management of ANGINA PECTORIS and HYPERTENSION.'),
  'D017273': NtO(MH='Goserelin', MS='A synthetic long-acting agonist of GONADOTROPIN-RELEASING HORMONE. Goserelin is used in treatments of malignant NEOPLASMS of the prostate, uterine fibromas, and metastatic breast cancer.'),
  'D017274': NtO(MH='Nafarelin', MS='A potent synthetic agonist of GONADOTROPIN-RELEASING HORMONE with 3-(2-naphthyl)-D-alanine substitution at residue 6. Nafarelin has been used in the treatments of central PRECOCIOUS PUBERTY and ENDOMETRIOSIS.'),
  'D017275': NtO(MH='Isradipine', MS='A potent antagonist of CALCIUM CHANNELS that is highly selective for VASCULAR SMOOTH MUSCLE. It is effective in the treatment of chronic stable angina pectoris, hypertension, and congestive cardiac failure.'),
  'D017276': NtO(MH='Friction', MS='Surface resistance to the relative motion of one body against the rubbing, sliding, rolling, or flowing of another with which it is in contact.'),
  'D017277': NtO(MH='Drug Approval', MS='Process that is gone through in order for a drug to receive approval by a government regulatory agency. This includes any required pre-clinical or clinical testing, review, submission, and evaluation of the applications and test results, and post-marketing surveillance of the drug.'),
  'D017278': NtO(MH='Air Bags', MS='Automotive safety devices consisting of a bag designed to inflate upon collision and prevent passengers from pitching forward. (American Heritage Dictionary, 1982)'),
  'D017279': NtO(MH='Selenocysteine', MS='A naturally occurring amino acid in both eukaryotic and prokaryotic organisms. It is found in tRNAs and in the catalytic site of some enzymes. The genes for glutathione peroxidase and formate dehydrogenase contain the TGA codon, which codes for this amino acid.'),
  'D017280': NtO(MH='Condoms', MS='A sheath that is worn over the penis during sexual behavior in order to prevent pregnancy or spread of sexually transmitted disease.'),
  'D017281': NtO(MH='Cost of Illness', MS='The personal cost of disease which may be economic, social, or psychological. The cost of illness may be reflected in absenteeism, productivity, response to treatment, peace of mind, or QUALITY OF LIFE. It differs from HEALTH CARE COSTS, the societal cost of providing services related to the delivery of health care.'),
  'D017282': NtO(MH='Tick-Borne Diseases', MS='Bacterial, viral, or parasitic diseases transmitted to humans and animals by the bite of infected ticks. The families Ixodidae and Argasidae contain many bloodsucking species that are important pests of man and domestic birds and mammals and probably exceed all other arthropods in the number and variety of disease agents they transmit. Many of the tick-borne diseases are zoonotic.'),
  'D017283': NtO(MH='Neonatal Nursing', MS='The nursing specialty that deals with the care of newborn infants during the first four weeks after birth.'),
  'D017285': NtO(MH='Polymyositis', MS='Diseases characterized by inflammation involving multiple muscles. This may occur as an acute or chronic condition associated with medication toxicity (DRUG TOXICITY); CONNECTIVE TISSUE DISEASES; infections; malignant NEOPLASMS; and other disorders. The term polymyositis is frequently used to refer to a specific clinical entity characterized by subacute or slowly progressing symmetrical weakness primarily affecting the proximal limb and trunk muscles. The illness may occur at any age, but is most frequent in the fourth to sixth decade of life. Weakness of pharyngeal and laryngeal muscles, interstitial lung disease, and inflammation of the myocardium may also occur. Muscle biopsy reveals widespread destruction of segments of muscle fibers and an inflammatory cellular response. (Adams et al., Principles of Neurology, 6th ed, pp1404-9)'),
  'D017286': NtO(MH='Polysomnography', MS='Simultaneous and continuous monitoring of several parameters during sleep to study normal and abnormal sleep. The study includes monitoring of brain waves, to assess sleep stages, and other physiological variables such as breathing, eye movements, and blood oxygen levels which exhibit a disrupted pattern with sleep disturbances.'),
  'D017287': NtO(MH='Ventriculoperitoneal Shunt', MS='Surgical creation of a communication between a cerebral ventricle and the peritoneum by means of a plastic tube to permit drainage of cerebrospinal fluid for relief of hydrocephalus. (From Dorland, 28th ed)'),
  'D017288': NtO(MH='Pain Threshold', MS='Amount of stimulation required before the sensation of pain is experienced.'),
  'D017289': NtO(MH='Orthodontic Appliances, Functional', MS='Loose, usually removable intra-oral devices which alter the muscle forces against the teeth and craniofacial skeleton. These are dynamic appliances which depend on altered neuromuscular action to effect bony growth and occlusal development. They are usually used in mixed dentition to treat pediatric malocclusions. (ADA, 1992)'),
  'D017290': NtO(MH='Menogaril', MS='A semisynthetic anthracycline with the amino sugar on the D ring. It displays broad-spectrum antineoplastic activity against a variety of tumors.'),
  'D017291': NtO(MH='Clarithromycin', MS='A semisynthetic macrolide antibiotic derived from ERYTHROMYCIN that is active against a variety of microorganisms. It can inhibit PROTEIN SYNTHESIS in BACTERIA by reversibly binding to the 50S ribosomal subunits. This inhibits the translocation of aminoacyl transfer-RNA and prevents peptide chain elongation.'),
  'D017292': NtO(MH='Doxazosin', MS='A prazosin-related compound that is a selective alpha-1-adrenergic blocker.'),
  'D017293': NtO(MH='Protein S', MS='The vitamin K-dependent cofactor of activated PROTEIN C. Together with protein C, it inhibits the action of factors VIIIa and Va. A deficiency in protein S; (PROTEIN S DEFICIENCY); can lead to recurrent venous and arterial thrombosis.'),
  'D017294': NtO(MH='Ondansetron', MS='A competitive serotonin type 3 receptor antagonist. It is effective in the treatment of nausea and vomiting caused by cytotoxic chemotherapy drugs, including cisplatin, and has reported anxiolytic and neuroleptic properties.'),
  'D017298': NtO(MH='Bisoprolol', MS='A cardioselective beta-1 adrenergic blocker. It is effective in the management of HYPERTENSION and ANGINA PECTORIS.'),
  'D017299': NtO(MH='Rod Opsins', MS='Photosensitive proteins expressed in the ROD PHOTORECEPTOR CELLS. They are the protein components of rod photoreceptor pigments such as RHODOPSIN.'),
  'D017300': NtO(MH='Pipecuronium', MS='A piperazinyl androstane derivative which is a non-depolarizing neuromuscular blocking agent (NEUROMUSCULAR NONDEPOLARIZING AGENTS). It is used as a muscle relaxant during ANESTHESIA and surgical procedures.'),
  'D017301': NtO(MH='Ca(2+) Mg(2+)-ATPase', MS='An enzyme that catalyzes the hydrolysis of ATP and is activated by millimolar concentrations of either Ca(2+) or Mg(2+). Unlike CA(2+)-TRANSPORTING ATPASE it does not require the second divalent cation for its activity, and is not sensitive to orthovanadate. (Prog Biophys Mol Biol 1988;52(1):1). A subgroup of EC 3.6.1.3.'),
  'D017302': NtO(MH='Annexins', MS='Family of calcium- and phospholipid-binding proteins which are structurally related and exhibit immunological cross-reactivity. Each member contains four homologous 70-kDa repeats. The annexins are differentially distributed in vertebrate tissues (and lower eukaryotes) and appear to be involved in MEMBRANE FUSION and SIGNAL TRANSDUCTION.'),
  'D017304': NtO(MH='Annexin A5', MS='A protein of the annexin family isolated from human PLACENTA and other tissues. It inhibits cytosolic PHOSPHOLIPASE A2, and displays anticoagulant activity.'),
  'D017305': NtO(MH='Annexin A1', MS='Protein of the annexin family exhibiting lipid interaction and steroid-inducibility.'),
  'D017306': NtO(MH='Annexin A2', MS='A member of the annexin family that is a substrate for a tyrosine kinase, ONCOGENE PROTEIN PP60(V-SRC). Annexin A2 occurs as a 36-KDa monomer and in a 90-KDa complex containing two subunits of annexin A2 and two subunits of S100 FAMILY PROTEIN P11. The monomeric form of annexin A2 was formerly referred to as calpactin I heavy chain.'),
  'D017307': NtO(MH='Xamoterol', MS='A phenoxypropanolamine derivative that is a selective beta-1-adrenergic agonist.'),
  'D017308': NtO(MH='Etodolac', MS='A non-steroidal anti-inflammatory agent and cyclooxygenase-2 (COX-2) inhibitor with potent analgesic and anti-arthritic properties. It has been shown to be effective in the treatment of OSTEOARTHRITIS; RHEUMATOID ARTHRITIS; ANKYLOSING SPONDYLITIS; and in the alleviation of postoperative pain (PAIN, POSTOPERATIVE).'),
  'D017310': NtO(MH='Annexin A7', MS='An annexin family member that plays a role in MEMBRANE FUSION and signaling via VOLTAGE-DEPENDENT CALCIUM CHANNELS.'),
  'D017311': NtO(MH='Amlodipine', MS='A long-acting dihydropyridine calcium channel blocker. It is effective in the treatment of ANGINA PECTORIS and HYPERTENSION.'),
  'D017312': NtO(MH='Toremifene', MS='A first generation selective estrogen receptor modulator (SERM). Like TAMOXIFEN, it is an estrogen agonist for bone tissue and cholesterol metabolism but is antagonistic on mammary and uterine tissue.'),
  'D017313': NtO(MH='Fenretinide', MS='A synthetic retinoid that is used orally as a chemopreventive against prostate cancer and in women at risk of developing contralateral breast cancer. It is also effective as an antineoplastic agent.'),
  'D017314': NtO(MH='Annexin A4', MS='Protein of the annexin family originally isolated from the electric organ of the electric ray Torpedo marmorata. It has been found in a wide range of mammalian tissue where it is localized to the apical membrane of polarized EPITHELIAL CELLS.'),
  'D017315': NtO(MH='Cilazapril', MS='One of the ANGIOTENSIN-CONVERTING ENZYME INHIBITORS (ACE inhibitors) used for hypertension. It is a prodrug that is hydrolyzed after absorption to its main metabolite cilazaprilat.'),
  'D017316': NtO(MH='Fadrozole', MS='A selective aromatase inhibitor effective in the treatment of estrogen-dependent disease including breast cancer.'),
  'D017317': NtO(MH='Annexin A6', MS='Protein of the annexin family with a probable role in exocytotic and endocytotic membrane events.'),
  'D017318': NtO(MH='Annexin A3', MS='A protein of the annexin family that catalyzes the conversion of 1-D-inositol 1,2-cyclic phosphate and water to 1-D-myo-inositol 1-phosphate.'),
  'D017319': NtO(MH='Photosensitizing Agents', MS='Drugs that are pharmacologically inactive but when exposed to ultraviolet radiation or sunlight are converted to their active metabolite to produce a beneficial reaction affecting the diseased tissue. These compounds can be administered topically or systemically and have been used therapeutically to treat psoriasis and various types of neoplasms.'),
  'D017320': NtO(MH='HIV Protease Inhibitors', MS='Inhibitors of HIV PROTEASE, an enzyme required for production of proteins needed for viral assembly.'),
  'D017321': NtO(MH='Clinical Trials, Phase I as Topic', MS='Works about studies performed to evaluate the safety of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques in healthy subjects and to determine the safe dosage range (if appropriate). These tests also are used to determine pharmacologic and pharmacokinetic properties (toxicity, metabolism, absorption, elimination, and preferred route of administration). They involve a small number of persons and usually last about 1 year. This concept includes phase I studies conducted both in the U.S. and in other countries.'),
  'D017322': NtO(MH='Clinical Trials, Phase II as Topic', MS='Works about studies that are usually controlled to assess the effectiveness and dosage (if appropriate) of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques. These studies are performed on several hundred volunteers, including a limited number of patients with the target disease or disorder, and last about two years. This concept includes phase II studies conducted in both the U.S. and in other countries.'),
  'D017323': NtO(MH='Dihematoporphyrin Ether', MS='The purified component of HEMATOPORPHYRIN DERIVATIVE, it consists of a mixture of oligomeric porphyrins. It is used in photodynamic therapy (HEMATOPORPHYRIN PHOTORADIATION); to treat malignant lesions with visible light and experimentally as an antiviral agent. It is the first drug to be approved in the use of PHOTODYNAMIC THERAPY in the United States.'),
  'D017324': NtO(MH='Hematoporphyrin Derivative', MS='A complex mixture of monomeric and aggregated porphyrins used in the photodynamic therapy of tumors (HEMATOPORPHYRIN PHOTORADIATION). A purified component of this mixture is known as DIHEMATOPORPHYRIN ETHER.'),
  'D017325': NtO(MH='Hepatitis B Vaccines', MS='Vaccines or candidate vaccines containing inactivated hepatitis B or some of its component antigens and designed to prevent hepatitis B. Some vaccines may be recombinantly produced.'),
  'D017326': NtO(MH='Clinical Trials, Phase III as Topic', MS='Works about comparative studies to verify the effectiveness of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques determined in phase II studies. During these trials, patients are monitored closely by physicians to identify any adverse reactions from long-term use. These studies are performed on groups of patients large enough to identify clinically significant responses and usually last about three years. This concept includes phase III studies conducted in both the U.S. and in other countries.'),
  'D017327': NtO(MH='Clinical Trials, Phase IV as Topic', MS='Works about planned post-marketing studies of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques that have been approved for general sale. These studies are often conducted to obtain additional data about the safety and efficacy of a product. This concept includes phase IV studies conducted in both the U.S. and in other countries.'),
  'D017328': NtO(MH='Fosinopril', MS='A phosphinic acid-containing angiotensin-converting enzyme inhibitor that is effective in the treatment of hypertension. It is a prodrug that is converted to its active metabolite fosinoprilat.'),
  'D017329': NtO(MH='Triptorelin Pamoate', MS='A potent synthetic long-acting agonist of GONADOTROPIN-RELEASING HORMONE with D-tryptophan substitution at residue 6.'),
  'D017330': NtO(MH='Remoxipride', MS='An antipsychotic agent that is specific for dopamine D2 receptors. It has been shown to be effective in the treatment of schizophrenia.'),
  'D017332': NtO(MH='Cetirizine', MS='A potent second-generation histamine H1 antagonist that is effective in the treatment of allergic rhinitis, chronic urticaria, and pollen-induced asthma. Unlike many traditional antihistamines, it does not cause drowsiness or anticholinergic side effects.'),
  'D017334': NtO(MH='Teicoplanin', MS='Lipoglycopeptide antibiotic from Actinoplanes teichomyceticus active against gram-positive bacteria. It consists of five major components each with a different fatty acid moiety.'),
  'D017335': NtO(MH='Enoximone', MS='A selective phosphodiesterase inhibitor with vasodilating and positive inotropic activity that does not cause changes in myocardial oxygen consumption. It is used in patients with CONGESTIVE HEART FAILURE.'),
  'D017336': NtO(MH='Loratadine', MS='A second-generation histamine H1 receptor antagonist used in the treatment of allergic rhinitis and urticaria. Unlike most classical antihistamines (HISTAMINE H1 ANTAGONISTS) it lacks central nervous system depressing effects such as drowsiness.'),
  'D017337': NtO(MH='Sermorelin', MS='The biologically active fragment of human growth hormone-releasing factor, consisting of GHRH(1-29)-amide. This N-terminal sequence is identical in several mammalian species, such as human, pig, and cattle. It is used to diagnose or treat patients with GROWTH HORMONE deficiency.'),
  'D017338': NtO(MH='Cladribine', MS='An antineoplastic agent used in the treatment of lymphoproliferative diseases including hairy-cell leukemia.'),
  'D017341': NtO(MH='Etanidazole', MS='A nitroimidazole that sensitizes hypoxic tumor cells that are normally resistant to radiation therapy.'),
  'D017343': NtO(MH='Genes, Plant', MS='The functional hereditary units of PLANTS.'),
  'D017344': NtO(MH='Genes, Insect', MS='The functional hereditary units of INSECTS.'),
  'D017346': NtO(MH='Protein-Serine-Threonine Kinases', MS='A group of enzymes that catalyzes the phosphorylation of serine or threonine residues in proteins, with ATP or other nucleotides as phosphate donors.'),
  'D017347': NtO(MH='Brucella melitensis', MS='A species of the genus BRUCELLA whose natural hosts are sheep and goats. Other mammals, including humans, may be infected. In general, these organisms tend to be more virulent for laboratory animals than BRUCELLA ABORTUS and may cause fatal infections.'),
  'D017348': NtO(MH='Microscopy, Electron, Scanning Transmission', MS='A type of TRANSMISSION ELECTRON MICROSCOPY in which the object is examined directly by an extremely narrow electron beam scanning the specimen point-by-point and using the reactions of the electrons that are transmitted through the specimen to create the image. It should not be confused with SCANNING ELECTRON MICROSCOPY.'),
  'D017349': NtO(MH='N-Acetylhexosaminyltransferases', MS='Enzymes that catalyze the transfer of N-acetylhexosaminyl groups to an acceptor molecule which is frequently another carbohydrate. EC 2.4.1.'),
  'D017350': NtO(MH='N-Acetylgalactosaminyltransferases', MS='Enzymes that catalyze the transfer of N-acetylgalactosamine from a nucleoside diphosphate N-acetylgalactosamine to an acceptor molecule which is frequently another carbohydrate. EC 2.4.1.-.'),
  'D017351': NtO(MH='N-Acetylglucosaminyltransferases', MS='Enzymes that catalyze the transfer of N-acetylglucosamine from a nucleoside diphosphate N-acetylglucosamine to an acceptor molecule which is frequently another carbohydrate. EC 2.4.1.-.'),
  'D017352': NtO(MH='Echo-Planar Imaging', MS='A type of MAGNETIC RESONANCE IMAGING that uses only one nuclear spin excitation per image and therefore can obtain images in a fraction of a second rather than the minutes required in traditional MRI techniques. It is used in a variety of medical and scientific applications.'),
  'D017353': NtO(MH='Gene Deletion', MS='A genetic rearrangement through loss of segments of DNA or RNA, bringing sequences which are normally separated into close proximity. This deletion may be detected using cytogenetic techniques and can also be inferred from the phenotype, indicating a deletion at one specific locus.'),
  'D017354': NtO(MH='Point Mutation', MS='A mutation caused by the substitution of one nucleotide for another. This results in the DNA molecule having a change in a single base pair.'),
  'D017355': NtO(MH='Zymomonas', MS='A genus of gram-negative, facultatively anaerobic, rod-shaped bacteria that is not known to be pathogenic for man, animals, or plants. Its organisms are spoilers for beers and ciders and in sweet English ciders they are the causative agents of a secondary fermentation known as "cider sickness." The species Z. mobilis is used for experiments in molecular genetic studies.'),
  'D017356': NtO(MH='Synchrotrons', MS='Devices for accelerating protons or electrons in closed orbits where the accelerating voltage and magnetic field strength varies (the accelerating voltage is held constant for electrons) in order to keep the orbit radius constant.'),
  'D017357': NtO(MH='Cyclotrons', MS='Devices for accelerating charged particles in a spiral path by a constant-frequency alternating electric field. This electric field is synchronized with the movement of the particles in a constant magnetic field.'),
  'D017358': NtO(MH='Cell Wall Skeleton', MS='A mucoprotein found in the cell wall of various types of bacteria. It has adjuvant and antitumor activities and has been used to augment the production of lymphokine-activated killer (LAK) cells.'),
  'D017359': NtO(MH='HELLP Syndrome', MS='A syndrome of HEMOLYSIS, elevated liver ENZYMES, and low blood platelets count (THROMBOCYTOPENIA). HELLP syndrome is observed in pregnant women with PRE-ECLAMPSIA or ECLAMPSIA who also exhibit LIVER damage and abnormalities in BLOOD COAGULATION.'),
  'D017360': NtO(MH='Arabidopsis', MS='A plant genus of the family BRASSICACEAE that contains ARABIDOPSIS PROTEINS and MADS DOMAIN PROTEINS. The species A. thaliana is used for experiments in classical plant genetics as well as molecular genetic studies in plant physiology, biochemistry, and development.'),
  'D017361': NtO(MH='Viral Nonstructural Proteins', MS='Proteins encoded by a VIRAL GENOME that are produced in the organisms they infect, but not packaged into the VIRUS PARTICLES. Some of these proteins may play roles within the infected cell during VIRUS REPLICATION or act in regulation of virus replication or VIRUS ASSEMBLY.'),
  'D017362': NtO(MH='Cyclic AMP Response Element-Binding Protein', MS='A protein that has been shown to function as a calcium-regulated transcription factor as well as a substrate for depolarization-activated CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASES. This protein functions to integrate both calcium and cAMP signals.'),
  'D017364': NtO(MH='Collagenases', MS='Enzymes that catalyze the degradation of collagen by acting on the peptide bonds.'),
  'D017365': NtO(MH='Spices', MS='The dried seeds, bark, root, stems, buds, leaves, or fruit of aromatic plants used to season food.'),
  'D017366': NtO(MH='Serotonin Receptor Agonists', MS='Endogenous compounds and drugs that bind to and activate SEROTONIN RECEPTORS. Many serotonin receptor agonists are used as ANTIDEPRESSANTS; ANXIOLYTICS; and in the treatment of MIGRAINE DISORDERS.'),
  'D017367': NtO(MH='Serotonin Uptake Inhibitors', MS='Compounds that specifically inhibit the reuptake of serotonin in the brain.'),
  'D017368': NtO(MH='Protein Prenylation', MS='A post-translational modification of proteins by the attachment of an isoprenoid to the C-terminal cysteine residue. The isoprenoids used, farnesyl diphosphate or geranylgeranyl diphosphate, are derived from the same biochemical pathway that produces cholesterol.'),
  'D017370': NtO(MH='Interleukin-11', MS='A lymphohematopoietic cytokine that plays a role in regulating the proliferation of ERYTHROID PRECURSOR CELLS. It induces maturation of MEGAKARYOCYTES which results in increased production of BLOOD PLATELETS. Interleukin-11 was also initially described as an inhibitor of ADIPOGENESIS of cultured preadipocytes.'),
  'D017371': NtO(MH='8-Hydroxy-2-(di-n-propylamino)tetralin', MS='A serotonin 1A-receptor agonist that is used experimentally to test the effects of serotonin.'),
  'D017373': NtO(MH='Cyproterone Acetate', MS='An agent with anti-androgen and progestational properties. It shows competitive binding with dihydrotestosterone at androgen receptor sites.'),
  'D017374': NtO(MH='Paroxetine', MS='A serotonin uptake inhibitor that is effective in the treatment of depression.'),
  'D017376': NtO(MH='Viral Tail Proteins', MS='Proteins found in the tail sections of DNA and RNA viruses. It is believed that these proteins play a role in directing chain folding and assembly of polypeptide chains.'),
  'D017377': NtO(MH='Diatoms', MS='The common name for the phylum of microscopic unicellular STRAMENOPILES. Most are aquatic, being found in fresh, brackish, and salt water. Diatoms are noted for the symmetry and sculpturing of their siliceous cell walls. They account for 40% of PHYTOPLANKTON, but not all diatoms are planktonic.'),
  'D017378': NtO(MH='Quinolinic Acid', MS='A metabolite of tryptophan with a possible role in neurodegenerative disorders. Elevated CSF levels of quinolinic acid are correlated with the severity of neuropsychological deficits in patients who have AIDS.'),
  'D017379': NtO(MH='Hypertrophy, Left Ventricular', MS='Enlargement of the LEFT VENTRICLE of the heart. This increase in ventricular mass is attributed to sustained abnormal pressure or volume loads and is a contributor to cardiovascular morbidity and mortality.'),
  'D017380': NtO(MH='Hypertrophy, Right Ventricular', MS='Enlargement of the RIGHT VENTRICLE of the heart. This increase in ventricular mass is often attributed to PULMONARY HYPERTENSION and is a contributor to cardiovascular morbidity and mortality.'),
  'D017381': NtO(MH='Spliceosomes', MS='Organelles in which the splicing and excision reactions that remove introns from precursor messenger RNA molecules occur. One component of a spliceosome is five small nuclear RNA molecules (U1, U2, U4, U5, U6) that, working in conjunction with proteins, help to fold pieces of RNA into the right shapes and later splice them into the message.'),
  'D017382': NtO(MH='Reactive Oxygen Species', MS='Molecules or ions formed by the incomplete one-electron reduction of oxygen. These reactive oxygen intermediates include SINGLET OXYGEN; SUPEROXIDES; PEROXIDES; HYDROXYL RADICAL; and HYPOCHLOROUS ACID. They contribute to the microbicidal activity of PHAGOCYTES, regulation of SIGNAL TRANSDUCTION and GENE EXPRESSION, and the oxidative damage to NUCLEIC ACIDS; PROTEINS; and LIPIDS.'),
  'D017384': NtO(MH='Sequence Deletion', MS='Deletion of sequences of nucleic acids from the genetic material of an individual.'),
  'D017385': NtO(MH='Sequence Homology', MS='The degree of similarity between sequences. Studies of AMINO ACID SEQUENCE HOMOLOGY and NUCLEIC ACID SEQUENCE HOMOLOGY provide useful information about the genetic relatedness of genes, gene products, and species.'),
  'D017386': NtO(MH='Sequence Homology, Amino Acid', MS='The degree of similarity between sequences of amino acids. This information is useful for the analyzing genetic relatedness of proteins and species.'),
  'D017387': NtO(MH='Investigational New Drug Application', MS='An application that must be submitted to a regulatory agency (the FDA in the United States) before a drug can be studied in humans. This application includes results of previous experiments; how, where, and by whom the new studies will be conducted; the chemical structure of the compound; how it is thought to work in the body; any toxic effects found in animal studies; and how the compound is manufactured. (From the "New Medicines in Development" Series produced by the Pharmaceutical Manufacturers Association and published irregularly.)'),
  'D017388': NtO(MH='Zygote Intrafallopian Transfer', MS='A technique in assisted reproduction (REPRODUCTIVE TECHNIQUES, ASSISTED) consisting of hormonal stimulation of the ovaries, follicular aspiration of preovulatory oocytes, in-vitro fertilization, and intrafallopian transfer of zygotes at the pronuclear stage (before cleavage).'),
  'D017391': NtO(MH='Epikeratophakia', MS="A procedure that modifies REFRACTIVE ERRORS by the transplantation of a donor CORNEA to the anterior surface of the patient's cornea."),
  'D017392': NtO(MH='Thiobarbituric Acid Reactive Substances', MS='Low-molecular-weight end products, probably malondialdehyde, that are formed during the decomposition of lipid peroxidation products. These compounds react with thiobarbituric acid to form a fluorescent red adduct.'),
  'D017393': NtO(MH='RNA Editing', MS='A process that changes the nucleotide sequence of mRNA from that of the DNA template encoding it. Some major classes of RNA editing are as follows: 1, the conversion of cytosine to uracil in mRNA; 2, the addition of variable number of guanines at pre-determined sites; and 3, the addition and deletion of uracils, templated by guide-RNAs (RNA, GUIDE).'),
  'D017394': NtO(MH='RNA, Guide', MS="Small kinetoplastid mitochondrial RNA that plays a major role in RNA EDITING. These molecules form perfect hybrids with edited mRNA sequences and possess nucleotide sequences at their 5'-ends that are complementary to the sequences of the mRNA's immediately downstream of the pre-edited regions."),
  'D017395': NtO(MH='Plasminogen Activator Inhibitor 1', MS='A member of the serpin family of proteins. It inhibits both the tissue-type and urokinase-type plasminogen activators.'),
  'D017396': NtO(MH='Plasminogen Activator Inhibitor 2', MS='Member of the serpin family of proteins. It inhibits both the tissue-type and urokinase-type plasminogen activators.'),
  'D017397': NtO(MH='Prefrontal Cortex', MS='The rostral part of the frontal lobe, bounded by the inferior precentral fissure in humans, which receives projection fibers from the MEDIODORSAL NUCLEUS OF THE THALAMUS. The prefrontal cortex receives afferent fibers from numerous structures of the DIENCEPHALON; MESENCEPHALON; and LIMBIC SYSTEM as well as cortical afferents of visual, auditory, and somatic origin.'),
  'D017398': NtO(MH='Alternative Splicing', MS='A process whereby multiple RNA transcripts are generated from a single gene. Alternative splicing involves the splicing together of other possible sets of EXONS during the processing of some, but not all, transcripts of the gene. Thus a particular exon may be connected to any one of several alternative exons to form a mature RNA. The alternative forms of mature MESSENGER RNA produce PROTEIN ISOFORMS in which one part of the isoforms is common while the other parts are different.'),
  'D017400': NtO(MH='Brachyspira', MS='A genus of spiral bacteria of the family Brachyspiraceae.'),
  'D017401': NtO(MH='Brachyspira hyodysenteriae', MS='A species of anaerobic, spiral bacteria that was formerly classified as Serpulina hyodysenteriae and Treponema hyodysenteriae (and for a short while, Serpula hyodysenteriae). This organism is the agent of swine dysentery.'),
  'D017402': NtO(MH='Chlorofluorocarbons', MS='A series of hydrocarbons containing both chlorine and fluorine. These have been used as refrigerants, blowing agents, cleaning fluids, solvents, and as fire extinguishing agents. They have been shown to cause stratospheric ozone depletion and have been banned for many uses.'),
  'D017403': NtO(MH='In Situ Hybridization', MS='A technique that localizes specific nucleic acid sequences within intact chromosomes, eukaryotic cells, or bacterial cells through the use of specific nucleic acid-labeled probes.'),
  'D017404': NtO(MH='In Situ Hybridization, Fluorescence', MS='A type of IN SITU HYBRIDIZATION in which target sequences are stained with fluorescent dye so their location and size can be determined using fluorescence microscopy. This staining is sufficiently distinct that the hybridization signal can be seen both in metaphase spreads and in interphase nuclei.'),
  'D017405': NtO(MH='Herpes Simplex Virus Protein Vmw65', MS='Trans-acting protein that combines with host factors to induce immediate early gene transcription in herpes simplex virus.'),
  'D017406': NtO(MH='Sexual Harassment', MS='A form of discrimination in the workplace which violates the Civil Rights Act of 1964. Offensive sexual conduct may include unwelcome advances, comments, touching, questions about marital status and sex practices, etc. Both men and women may be aggressors or victims. (Slee and Slee, Health Care Terms, 2d ed, p.404). While civil rights legislation deals with sexual harassment in the workplace, the behavior is not restricted to this; it may take place outside the work environment: in schools and colleges, athletics, and other social milieus and activities.'),
  'D017407': NtO(MH='Family Leave', MS='The authorized absence from work of a family member to attend the illness or participate in the care of a parent, a sibling, or other family member. For the care of a parent for a child or for pre- or postnatal leave of a parent, PARENTAL LEAVE is available.'),
  'D017408': NtO(MH='Guidelines as Topic', MS='Works about a systematic statement of policy rules or principles. Guidelines may be developed by government agencies at any level, institutions, professional societies, governing boards, or by convening expert panels. For guidelines in the field of health care and clinical medicine, PRACTICE GUIDELINES AS TOPIC is available.'),
  'D017409': NtO(MH='Sufentanil', MS='An opioid analgesic that is used as an adjunct in anesthesia, in balanced anesthesia, and as a primary anesthetic agent.'),
  'D017410': NtO(MH='Practice Guidelines as Topic', MS='Works about directions or principles presenting current or future rules of policy for assisting health care practitioners in patient care decisions regarding diagnosis, therapy, or related clinical circumstances. The guidelines may be developed by government agencies at any level, institutions, professional societies, governing boards, or by the convening of expert panels. The guidelines form a basis for the evaluation of all aspects of health care and delivery.'),
  'D017411': NtO(MH='Ribonucleoproteins, Small Nuclear', MS="Highly conserved nuclear RNA-protein complexes that function in RNA processing in the nucleus, including pre-mRNA splicing and pre-mRNA 3'-end processing in the nucleoplasm, and pre-rRNA processing in the nucleolus (see RIBONUCLEOPROTEINS, SMALL NUCLEOLAR)."),
  'D017412': NtO(MH='Ribonucleoprotein, U1 Small Nuclear', MS="A nuclear RNA-protein complex that plays a role in RNA processing. In the nucleoplasm, the U1 snRNP along with other small nuclear ribonucleoproteins (U2, U4-U6, and U5) assemble into SPLICEOSOMES that remove introns from pre-mRNA by splicing. The U1 snRNA forms base pairs with conserved sequence motifs at the 5'-splice site and recognizes both the 5'- and 3'-splice sites and may have a fundamental role in aligning the two sites for the splicing reaction."),
  'D017413': NtO(MH='Ribonucleoprotein, U2 Small Nuclear', MS='A nuclear RNA-protein complex that plays a role in RNA processing. In the nucleoplasm, the U2 snRNP along with other small nuclear ribonucleoproteins (U1, U4-U6, and U5) assemble into SPLICEOSOMES that remove introns from pre-mRNA by splicing. The U2 snRNA forms base pairs with conserved sequence motifs at the branch point, which associates with a heat- and RNAase-sensitive factor in an early step of splicing.'),
  'D017414': NtO(MH='Ribonucleoprotein, U4-U6 Small Nuclear', MS='A nuclear RNA-protein complex that plays a role in RNA processing. In the nucleoplasm, the U4-U6 snRNP along with the U5 snRNP preassemble into a single 25S particle that binds to the U1 and U2 snRNPs and the substrate to form mature SPLICEOSOMES. There is also evidence for the existence of individual U4 or U6 snRNPs in addition to their organization as a U4-U6 snRNP.'),
  'D017415': NtO(MH='Ribonucleoprotein, U5 Small Nuclear', MS='A nuclear RNA-protein complex that plays a role in RNA processing. In the nucleoplasm, the U5 snRNP along with U4-U6 snRNP preassemble into a single 25S particle that binds to the U1 and U2 snRNPs and the substrate to form SPLICEOSOMES.'),
  'D017418': NtO(MH='Meta-Analysis', MS='Works consisting of studies using a quantitative method of combining the results of independent studies (usually drawn from the published literature) and synthesizing summaries and conclusions which may be used to evaluate therapeutic effectiveness, plan new studies, etc. It is often an overview of clinical trials. It is usually called a meta-analysis by the author or sponsoring body and should be differentiated from reviews of literature.'),
  'D017419': NtO(MH='Potassium, Dietary', MS='Potassium or potassium compounds used in foods or as foods.'),
  'D017421': NtO(MH='Sequence Analysis', MS='A multistage process that includes the determination of a sequence (protein, carbohydrate, etc.), its fragmentation and analysis, and the interpretation of the resulting sequence information.'),
  'D017422': NtO(MH='Sequence Analysis, DNA', MS='A multistage process that includes cloning, physical mapping, subcloning, determination of the DNA SEQUENCE, and information analysis.'),
  'D017423': NtO(MH='Sequence Analysis, RNA', MS='A multistage process that includes cloning, physical mapping, subcloning, sequencing, and information analysis of an RNA SEQUENCE.'),
  'D017426': NtO(MH='Clinical Trial, Phase I', MS='Work that is the report of a pre-planned, usually controlled, clinical study of the safety and efficacy of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques based on a small number of healthy persons and conducted over the period of about a year in either the United States or a foreign country.'),
  'D017427': NtO(MH='Clinical Trial, Phase II', MS='Work that is a report of a pre-planned, usually controlled, clinical study of the safety and efficacy of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques based on several hundred volunteers, including a limited number of patients, and conducted over a period of about two years in either the United States or a foreign country.'),
  'D017428': NtO(MH='Clinical Trial, Phase III', MS='Work that is a report of a pre-planned, usually controlled, clinical study of the safety and efficacy of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques after phase II trials. A large enough group of patients is studied and closely monitored by physicians for adverse response to long-term exposure, over a period of about three years in either the United States or a foreign country.'),
  'D017429': NtO(MH='Clinical Trial, Phase IV', MS='Work that is a report of a planned post-marketing study of diagnostic, therapeutic, or prophylactic drugs, devices, or techniques that have been approved for general sale after clinical trials, phases I, II, and III. These studies, conducted in the United States or a foreign country, often garner additional data about the safety and efficacy of a product.'),
  'D017430': NtO(MH='Prostate-Specific Antigen', MS='A glycoprotein that is a kallikrein-like serine proteinase and an esterase, produced by epithelial cells of both normal and malignant prostate tissue. It is an important marker for the diagnosis of prostate cancer.'),
  'D017431': NtO(MH='Genes, MCC', MS='Tumor suppressor genes located in the 5q21 region on the long arm of human chromosome 5. The mutation of these genes is associated with the formation of colorectal cancer (MCC stands for mutated in colorectal cancer).'),
  'D017432': NtO(MH='Unified Medical Language System', MS='A research and development program initiated by the NATIONAL LIBRARY OF MEDICINE to build knowledge sources for the purpose of aiding the development of systems that help health professionals retrieve and integrate biomedical information. The knowledge sources can be used to link disparate information systems to overcome retrieval problems caused by differences in terminology and the scattering of relevant information across many databases. The three knowledge sources are the Metathesaurus, the Semantic Network, and the Specialist Lexicon.'),
  'D017433': NtO(MH='Protein Structure, Secondary', MS='The level of protein structure in which regular hydrogen-bond interactions within contiguous stretches of polypeptide chain give rise to ALPHA-HELICES; BETA-STRANDS (which align to form BETA-SHEETS), or other types of coils. This is the first folding level of protein conformation.'),
  'D017434': NtO(MH='Protein Structure, Tertiary', MS='The level of protein structure in which combinations of secondary protein structures (ALPHA HELICES; BETA SHEETS; loop regions, and AMINO ACID MOTIFS) pack together to form folded shapes. Disulfide bridges between cysteines in two different parts of the polypeptide chain along with other interactions between the chains play a role in the formation and stabilization of tertiary structure.'),
  'D017436': NtO(MH='Kallmann Syndrome', MS='A genetically heterogeneous disorder caused by hypothalamic GNRH deficiency and OLFACTORY NERVE defects. It is characterized by congenital HYPOGONADOTROPIC HYPOGONADISM and ANOSMIA, possibly with additional midline defects. It can be transmitted as an X-linked (GENETIC DISEASES, X-LINKED), an autosomal dominant, or an autosomal recessive trait.'),
  'D017437': NtO(MH='Skin and Connective Tissue Diseases', MS='A collective term for diseases of the skin and its appendages and of connective tissue.'),
  'D017438': NtO(MH='Bisphenol A-Glycidyl Methacrylate', MS='The reaction product of bisphenol A and glycidyl methacrylate that undergoes polymerization when exposed to ultraviolet light or mixed with a catalyst. It is used as a bond implant material and as the resin component of dental sealants and composite restorative materials.'),
  'D017439': NtO(MH='Cicatrix, Hypertrophic', MS='An elevated scar, resembling a KELOID, but which does not spread into surrounding tissues. It is formed by enlargement and overgrowth of cicatricial tissue and regresses spontaneously.'),
  'D017440': NtO(MH='Photoperiod', MS='The time period of daily exposure that an organism receives from daylight or artificial light. It is believed that photoperiodic responses may affect the control of energy balance and thermoregulation.'),
  'D017441': NtO(MH='Necrobiotic Disorders', MS='A group of disorders characterized by swelling, basophilia, and distortion of collagen bundles in the dermis.'),
  'D017442': NtO(MH='Histamine Agonists', MS='Drugs that bind to and activate histamine receptors. Although they have been suggested for a variety of clinical applications histamine agonists have so far been more widely used in research than therapeutically.'),
  'D017443': NtO(MH='Skin Diseases, Eczematous', MS='Any of a variety of eruptive skin disorders characterized by erythema, oozing, vesiculation, and scaling. Etiology is varied.'),
  'D017444': NtO(MH='Skin Diseases, Papulosquamous', MS='A group of dermatoses with distinct morphologic features. The primary lesion is most commonly a papule, usually erythematous, with a variable degree of scaling on the surface. Plaques form through the coalescing of primary lesions.'),
  'D017445': NtO(MH='Skin Diseases, Vascular', MS='Skin diseases affecting or involving the cutaneous blood vessels and generally manifested as inflammation, swelling, erythema, or necrosis in the affected area.'),
  'D017446': NtO(MH='Impromidine', MS='A highly potent and specific histamine H2 receptor agonist. It has been used diagnostically as a gastric secretion indicator.'),
  'D017447': NtO(MH='Receptors, Dopamine D1', MS='A subfamily of G-PROTEIN-COUPLED RECEPTORS that bind the neurotransmitter DOPAMINE and modulate its effects. D1-class receptor genes lack INTRONS, and the receptors stimulate ADENYLYL CYCLASES.'),
  'D017448': NtO(MH='Receptors, Dopamine D2', MS='A subfamily of G-PROTEIN-COUPLED RECEPTORS that bind the neurotransmitter DOPAMINE and modulate its effects. D2-class receptor genes contain INTRONS, and the receptors inhibit ADENYLYL CYCLASES.'),
  'D017449': NtO(MH='Dermatitis, Allergic Contact', MS='A contact dermatitis due to allergic sensitization to various substances. These substances subsequently produce inflammatory reactions in the skin of those who have acquired hypersensitivity to them as a result of prior exposure.'),
  'D017450': NtO(MH='Receptors, Opioid, mu', MS='A class of opioid receptors recognized by its pharmacological profile. Mu opioid receptors bind, in decreasing order of affinity, endorphins, dynorphins, met-enkephalin, and leu-enkephalin. They have also been shown to be molecular receptors for morphine.'),
  'D017451': NtO(MH='Receptors, Somatomedin', MS='Cell surface receptors that bind somatomedins and trigger intracellular changes which influence the behavior of cells. Studies have disclosed two types of receptors for this family of peptide hormones. The type I receptor is homologous to the insulin receptor and has tyrosine kinase activity. The type II receptor is identical to the mannose-6-phosphate receptor which is important in trafficking of lysosomal enzymes.'),
  'D017452': NtO(MH='Receptors, IgG', MS='Specific molecular sites on the surface of various cells, including B-lymphocytes and macrophages, that combine with IMMUNOGLOBULIN Gs. Three subclasses exist: Fc gamma RI (the CD64 antigen, a low affinity receptor), Fc gamma RII (the CD32 antigen, a high affinity receptor), and Fc gamma RIII (the CD16 antigen, a low affinity receptor).'),
  'D017453': NtO(MH='Dermatitis, Irritant', MS='A non-allergic contact dermatitis caused by prolonged exposure to irritants and not explained by delayed hypersensitivity mechanisms.'),
  'D017454': NtO(MH='Dermatitis, Photoallergic', MS='A delayed hypersensitivity involving the reaction between sunlight or other radiant energy source and a chemical substance to which the individual has been previously exposed and sensitized. It manifests as a papulovesicular, eczematous, or exudative dermatitis occurring chiefly on the light-exposed areas of the skin.'),
  'D017455': NtO(MH='Receptors, IgE', MS='Specific molecular sites on the surface of B- and T-lymphocytes which combine with IgEs. Two subclasses exist: low affinity receptors (Fc epsilon RII) and high affinity receptors (Fc epsilon RI).'),
  'D017456': NtO(MH='Adenovirus Early Proteins', MS='Proteins encoded by adenoviruses that are synthesized prior to, and in the absence of, viral DNA replication. The proteins are involved in both positive and negative regulation of expression in viral and cellular genes, and also affect the stability of viral mRNA. Some are also involved in oncogenic transformation.'),
  'D017457': NtO(MH='Receptors, Albumin', MS='Cell surface proteins that bind albumin with high affinity and trigger intracellular changes influencing the behavior of cells.'),
  'D017459': NtO(MH='Receptors, Amino Acid', MS='Cell surface proteins that bind amino acids and trigger changes which influence the behavior of cells. Glutamate receptors are the most common receptors for fast excitatory synaptic transmission in the vertebrate central nervous system, and GAMMA-AMINOBUTYRIC ACID and glycine receptors are the most common receptors for fast inhibition.'),
  'D017461': NtO(MH='Receptors, Atrial Natriuretic Factor', MS='Cell surface proteins that bind ATRIAL NATRIURETIC FACTOR with high affinity and trigger intracellular changes influencing the behavior of cells. They contain intrinsic guanylyl cyclase activity.'),
  'D017463': NtO(MH='Receptors, Complement 3b', MS='Molecular sites on or in some B-lymphocytes and macrophages that recognize and combine with COMPLEMENT C3B. The primary structure of these receptors reveal that they contain transmembrane and cytoplasmic domains, with their extracellular portion composed entirely of thirty short consensus repeats each having 60 to 70 amino acids.'),
  'D017464': NtO(MH='Receptors, Complement 3d', MS='Molecular sites on or in B-lymphocytes, follicular dendritic cells, lymphoid cells, and epithelial cells that recognize and combine with COMPLEMENT C3D. Human complement receptor 2 (CR2) serves as a receptor for both C3dg and the gp350/220 glycoprotein of HERPESVIRUS 4, HUMAN, and binds the monoclonal antibody OKB7, which blocks binding of both ligands to the receptor.'),
  'D017465': NtO(MH='Receptors, Opioid, delta', MS='A class of opioid receptors recognized by its pharmacological profile. Delta opioid receptors bind endorphins and enkephalins with approximately equal affinity and have less affinity for dynorphins.'),
  'D017466': NtO(MH='Receptors, Endothelin', MS='Cell surface proteins that bind ENDOTHELINS with high affinity and trigger intracellular changes which influence the behavior of cells.'),
  'D017467': NtO(MH='Receptors, Erythropoietin', MS='Cell surface proteins that bind erythropoietin with high affinity and trigger intracellular changes influencing the behavior of cells.'),
  'D017468': NtO(MH='Receptors, Fibroblast Growth Factor', MS='Specific molecular sites or structures on cell membranes that react with FIBROBLAST GROWTH FACTORS (both the basic and acidic forms), their analogs, or their antagonists to elicit or to inhibit the specific response of the cell to these factors. These receptors frequently possess tyrosine kinase activity.'),
  'D017469': NtO(MH='Receptors, Fibronectin', MS='Specific cell surface receptors which bind to FIBRONECTINS. Studies have shown that these receptors function in certain types of adhesive contact as well as playing a major role in matrix assembly. These receptors include the traditional fibronectin receptor, also called INTEGRIN ALPHA5BETA1 and several other integrins.'),
  'D017470': NtO(MH='Receptors, Glutamate', MS='Cell-surface proteins that bind glutamate and trigger changes which influence the behavior of cells. Glutamate receptors include ionotropic receptors (AMPA, kainate, and N-methyl-D-aspartate receptors), which directly control ion channels, and metabotropic receptors which act through second messenger systems. Glutamate receptors are the most common mediators of fast excitatory synaptic transmission in the central nervous system. They have also been implicated in the mechanisms of memory and of many diseases.'),
  'D017471': NtO(MH='Receptors, Interferon', MS='Specific molecular sites or structures on or in cells with which interferons react or to which they bind in order to modify the function of the cells. Interferons exert their pleiotropic effects through two different receptors. alpha- and beta-interferon crossreact with common receptors, while gamma-interferon initiates its biological effects through its own specific receptor system.'),
  'D017472': NtO(MH='Receptors, Interleukin-1', MS='Cell surface receptors that are specific for INTERLEUKIN-1. Included under this heading are signaling receptors, non-signaling receptors and accessory proteins required for receptor signaling. Signaling from interleukin-1 receptors occurs via interaction with SIGNAL TRANSDUCING ADAPTOR PROTEINS such as MYELOID DIFFERENTIATION FACTOR 88.'),
  'D017473': NtO(MH='Receptors, Opioid, kappa', MS='A class of opioid receptors recognized by its pharmacological profile. Kappa opioid receptors bind dynorphins with a higher affinity than endorphins which are themselves preferred to enkephalins.'),
  'D017474': NtO(MH='Receptors, Laminin', MS='Glycoprotein molecules on the surface of cells that react with or bind to laminin whose function allows the binding of epithelial cells to the basement membrane. The molecular weight of this high-affinity receptor is 67 kD.'),
  'D017475': NtO(MH='Receptors, Nerve Growth Factor', MS='Cell surface receptors that bind NERVE GROWTH FACTOR; (NGF) and a NGF-related family of neurotrophic factors that includes neurotrophins, BRAIN-DERIVED NEUROTROPHIC FACTOR and CILIARY NEUROTROPHIC FACTOR.'),
  'D017476': NtO(MH='Receptors, Neuropeptide Y', MS='Cell surface proteins that bind neuropeptide Y with high affinity and trigger intracellular changes which influence the behavior of cells.'),
  'D017478': NtO(MH='Receptors, Phencyclidine', MS='Specific sites or molecular structures on cell membranes or in cells with which phencyclidine reacts or to which it binds to elicit the specific response of the cell to phencyclidine. Studies have demonstrated the presence of multiple receptor sites for PCP. These are the PCP/sigma site, which binds both PCP and psychotomimetic opiates but not certain antipsychotics, and the PCP site, which selectively binds PCP analogs.'),
  'D017479': NtO(MH='Receptors, Platelet-Derived Growth Factor', MS='Specific receptors on cell membranes that react with PLATELET-DERIVED GROWTH FACTOR, its analogs, or antagonists. The alpha PDGF receptor (RECEPTOR, PLATELET-DERIVED GROWTH FACTOR ALPHA) and the beta PDGF receptor (RECEPTOR, PLATELET-DERIVED GROWTH FACTOR BETA) are the two principle types of PDGF receptors. Activation of the protein-tyrosine kinase activity of the receptors occurs by ligand-induced dimerization or heterodimerization of PDGF receptor types.'),
  'D017480': NtO(MH='Receptors, sigma', MS='A class of cell surface receptors recognized by its pharmacological profile. Sigma receptors were originally considered to be opioid receptors because they bind certain synthetic opioids. However they also interact with a variety of other psychoactive drugs, and their endogenous ligand is not known (although they can react to certain endogenous steroids). Sigma receptors are found in the immune, endocrine, and nervous systems, and in some peripheral tissues.'),
  'D017481': NtO(MH='Receptors, Somatostatin', MS="Cell surface proteins that bind somatostatin and trigger intracellular changes which influence the behavior of cells. Somatostatin is a hypothalamic hormone, a pancreatic hormone, and a central and peripheral neurotransmitter. Activated somatostatin receptors on pituitary cells inhibit the release of growth hormone; those on endocrine and gastrointestinal cells regulate the absorption and utilization of nutrients; and those on neurons mediate somatostatin's role as a neurotransmitter."),
  'D017482': NtO(MH='Receptors, Thromboxane', MS='Cell surface proteins that bind THROMBOXANES with high affinity and trigger intracellular changes influencing the behavior of cells. Some thromboxane receptors act via the inositol phosphate and diacylglycerol second messenger systems.'),
  'D017483': NtO(MH='Receptors, Vasopressin', MS='Specific molecular sites or proteins on or in cells to which VASOPRESSINS bind or interact in order to modify the function of the cells. Two types of vasopressin receptor exist, the V1 receptor in the vascular smooth muscle and the V2 receptor in the kidneys. The V1 receptor can be subdivided into V1a and V1b (formerly V3) receptors.'),
  'D017484': NtO(MH='Dermatitis, Phototoxic', MS='A nonimmunologic, chemically induced type of photosensitivity producing a sometimes vesiculating dermatitis. It results in hyperpigmentation and desquamation of the light-exposed areas of the skin.'),
  'D017485': NtO(MH='1-Deoxynojirimycin', MS='An alpha-glucosidase inhibitor with antiviral action. Derivatives of deoxynojirimycin may have anti-HIV activity.'),
  'D017486': NtO(MH='Acneiform Eruptions', MS='Visible efflorescent lesions of the skin caused by acne or resembling acne. (Dorland, 28th ed, p18, 575)'),
  'D017487': NtO(MH='Ankyrins', MS='A family of membrane-associated proteins responsible for the attachment of the cytoskeleton. Erythrocyte-related isoforms of ankyrin attach the SPECTRIN cytoskeleton to a transmembrane protein (ANION EXCHANGE PROTEIN 1, ERYTHROCYTE) in the erythrocyte plasma membrane. Brain-related isoforms of ankyrin also exist.'),
  'D017488': NtO(MH='Hyperkeratosis, Epidermolytic', MS='A form of congenital ichthyosis inherited as an autosomal dominant trait and characterized by ERYTHRODERMA and severe hyperkeratosis. It is manifested at birth by blisters followed by the appearance of thickened, horny, verruciform scales over the entire body, but accentuated in flexural areas. Mutations in the genes that encode KERATIN-1 and KERATIN-10 have been associated with this disorder.'),
  'D017489': NtO(MH='CDC28 Protein Kinase, S cerevisiae', MS='A protein kinase encoded by the Saccharomyces cerevisiae CDC28 gene and required for progression from the G1 PHASE to the S PHASE in the CELL CYCLE.'),
  'D017490': NtO(MH='Ichthyosis, Lamellar', MS='A chronic, congenital ichthyosis inherited as an autosomal recessive trait. Infants are usually born encased in a collodion membrane which sheds within a few weeks. Scaling is generalized and marked with grayish-brown quadrilateral scales, adherent at their centers and free at the edges. In some cases, scales are so thick that they resemble armored plate.'),
  'D017491': NtO(MH='Genes, APC', MS='Tumor suppressor genes located in the 5q21 region on the long arm of human chromosome 5. The mutation of these genes is associated with familial adenomatous polyposis (ADENOMATOUS POLYPOSIS COLI) and GARDNER SYNDROME, as well as some sporadic colorectal cancers.'),
  'D017492': NtO(MH='Keratosis, Seborrheic', MS='Benign eccrine poromas that present as multiple oval, brown-to-black plaques, located mostly on the chest and back. The age of onset is usually in the fourth or fifth decade.'),
  'D017493': NtO(MH='Leukocyte Common Antigens', MS='High-molecular weight glycoproteins uniquely expressed on the surface of LEUKOCYTES and their hemopoietic progenitors. They contain two FIBRONECTIN TYPE III DOMAINS and possess cytoplasmic protein tyrosine phosphatase activity, which plays a role in intracellular signaling from the CELL SURFACE RECEPTORS. Leukocyte common antigens occur as multiple isoforms that result from alternative mRNA splicing and differential usage of three exons.'),
  'D017494': NtO(MH='Proton Pumps', MS='Integral membrane proteins that transport protons across a membrane. This transport can be linked to the hydrolysis of ADENOSINE TRIPHOSPHATE. What is referred to as proton pump inhibitors frequently is about POTASSIUM HYDROGEN ATPASE.'),
  'D017495': NtO(MH='Hyperpigmentation', MS='Excessive pigmentation of the skin, usually as a result of increased epidermal or dermal melanin pigmentation, hypermelanosis. Hyperpigmentation can be localized or generalized. The condition may arise from exposure to light, chemicals or other substances, or from a primary metabolic imbalance.'),
  'D017496': NtO(MH='Hypopigmentation', MS='A condition caused by a deficiency or a loss of melanin pigmentation in the epidermis, also known as hypomelanosis. Hypopigmentation can be localized or generalized, and may result from genetic defects, trauma, inflammation, or infections.'),
  'D017497': NtO(MH='Hidradenitis Suppurativa', MS='A chronic suppurative and cicatricial disease of the apocrine glands occurring chiefly in the axillae in women and in the groin and anal regions in men. It is characterized by poral occlusion with secondary bacterial infection, evolving into abscesses which eventually rupture. As the disease becomes chronic, ulcers appear, sinus tracts enlarge, fistulas develop, and fibrosis and scarring become evident.'),
  'D017498': NtO(MH='Adenovirus E1 Proteins', MS='The very first viral gene products synthesized after cells are infected with adenovirus. The E1 region of the genome has been divided into two major transcriptional units, E1A and E1B, each expressing proteins of the same name (ADENOVIRUS E1A PROTEINS and ADENOVIRUS E1B PROTEINS).'),
  'D017499': NtO(MH='Porokeratosis', MS='A heritable disorder of faulty keratinization characterized by the proliferation of abnormal clones of KERATINOCYTES and lesions showing varying atrophic patches surrounded by an elevated, keratotic border. These keratotic lesions can progress to overt cutaneous neoplasm. Several clinical variants are recognized, including porokeratosis of Mibelli, linear porokeratosis, disseminated superficial actinic porokeratosis, palmoplantar porokeratosis, and punctate porokeratosis.'),
  'D017500': NtO(MH='Adenovirus E1A Proteins', MS='Proteins transcribed from the E1A genome region of ADENOVIRUSES which are involved in positive regulation of transcription of the early genes of host infection.'),
  'D017501': NtO(MH='Adenovirus E1B Proteins', MS='Proteins transcribed from the E1B region of ADENOVIRUSES which are involved in regulation of the levels of early and late viral gene expression.'),
  'D017502': NtO(MH='Adenovirus E2 Proteins', MS='Proteins transcribed from the E2 region of ADENOVIRUSES. Several of these are required for viral DNA replication.'),
  'D017503': NtO(MH='Adenovirus E3 Proteins', MS='Proteins transcribed from the E3 region of ADENOVIRUSES but not essential for viral replication. The E3 19K protein mediates adenovirus persistence by reducing the expression of class I major histocompatibility complex antigens on the surface of infected cells.'),
  'D017504': NtO(MH='Adenovirus E4 Proteins', MS='Proteins transcribed from the E4 region of ADENOVIRUSES. The E4 19K protein transactivates transcription of the adenovirus E2F protein and complexes with it.'),
  'D017506': NtO(MH='H(+)-K(+)-Exchanging ATPase', MS='An enzyme isolated from the GASTRIC MUCOSA that catalyzes the hydrolysis of ATP coupled with the exchange of hydrogen and potassium ions across the cell wall. This enzyme was formerly listed as EC 3.6.1.36.'),
  'D017507': NtO(MH='Comoros', MS="A group of Indian Ocean Islands, the islands of Great Comoro, Anjouan, Mayotte, and Moheli, lying between northeast Mozambique and northwest Madagascar. The capital is Moroni. In 1914 they became a colony attached to Madagascar administratively and were made a French overseas territory in 1947. Except for Mayotte which remained French, Comoros became an independent republic in 1975. Comoros represents the Arabic qamar, moon, said by some scholars to be linked with the mystical Mountains of the Moon said to be somewhere in equatorial Africa. (From Webster's New Geographical Dictionary, 1988, p283 & Room, Brewer's Dictionary of Names, 1992, p122)"),
  'D017508': NtO(MH='Reunion', MS="One of the Indian Ocean Islands, east of Madagascar. Its capital is Saint-Denis. It was discovered in 1507 by the Portuguese and claimed by France in 1638. It was first colonized in 1662 as Isle de Bourbon but renamed Reunion in 1793. In 1946 it was made an overseas department of France. The name commemorates the reunion of the revolutionaries from Marseilles with the National Guard in Paris in 1792. (From Webster's New Geographical Dictionary, 1988, p1011; Room, Brewer's Dictionary of Names, 1992, p454; French Embassy)"),
  'D017510': NtO(MH='Protein Folding', MS='Processes involved in the formation of TERTIARY PROTEIN STRUCTURE.'),
  'D017511': NtO(MH='Pyoderma Gangrenosum', MS='An idiopathic, rapidly evolving, and severely debilitating disease occurring most commonly in association with chronic ulcerative colitis. It is characterized by the presence of boggy, purplish ulcers with undermined borders, appearing mostly on the legs. The majority of cases are in people between 40 and 60 years old. Its etiology is unknown.'),
  'D017512': NtO(MH='Lichenoid Eruptions', MS='Conditions in which there is histological damage to the lower epidermis along with a grouped chronic inflammatory infiltrate in the papillary dermis disturbing the interface between the epidermis and dermis. LICHEN PLANUS is the prototype of all lichenoid eruptions. (From Rook et al., Textbook of Dermatology, 4th ed, p398)'),
  'D017513': NtO(MH='Lichen Nitidus', MS='A chronic inflammatory disease characterized by shiny, flat-topped, usually flesh-colored micropapules no larger than the head of a pin. Lesions are localized in the early stages, found chiefly on the lower abdomen, penis, and inner surface of the thighs. Distribution may become generalized as the disease progresses.'),
  'D017514': NtO(MH='Pityriasis Lichenoides', MS='A subgroup of PARAPSORIASIS itself divided into acute and chronic forms. The acute form is characterized by the abrupt onset of a generalized, reddish-brown, maculopapular eruption. Lesions may be vesicular, hemorrhagic, crusted, or necrotic. Histologically the disease is characterized by epidermal necrolysis. The chronic form shows milder skin changes with necrosis.'),
  'D017515': NtO(MH='Pityriasis Rosea', MS='A mild exanthematous inflammation of unknown etiology. It is characterized by the presence of salmon-colored maculopapular lesions. The most striking feature is the arrangement of the lesions such that the long axis is parallel to the lines of cleavage. The eruptions are usually generalized, affecting chiefly the trunk, and the course is often self-limiting.'),
  'D017516': NtO(MH='Historical Geographic Locations', MS='Countries known in remote history (as BYZANTIUM) or former names of countries reflecting political changes in the 19th or 20th century (as GERMANY, EAST).'),
  'D017517': NtO(MH='Commonwealth of Independent States', MS='An economic union of Azerbaijan, Armenia, Belarus, Georgia, Kazakhstan, Kyrgyzstan, Moldova, Russia, Tajikistan, Turkmenistan, Uzbekistan and Ukraine.'),
  'D017520': NtO(MH='Mucinoses', MS='Mucoid states characterized by the elevated deposition and accumulation of mucin (mucopolysaccharides) in dermal tissue. The fibroblasts are responsible for the production of acid mucopolysaccharides (GLYCOSAMINOGLYCANS) in the ground substance of the connective tissue system. When fibroblasts produce abnormally large quantities of mucopolysaccharides as hyaluronic acid, chondroitin sulfate, or heparin, they accumulate in large amounts in the dermis.'),
  'D017522': NtO(MH='Bosnia and Herzegovina', MS='A country of eastern Europe, formerly the province of Bosnia in Yugoslavia, uniting with the province of Herzegovina to form the Republic of Bosnia and Herzegovina in 1946. It was created 7 April 1992 as a result of the division of Yugoslavia and recognized by the United States as an independent state. Bosnia takes is name from the river Bosna, in turn from the Indoeuropean root bhog, "current"; Herzegovina is from the Serbian herceg (duke) + -ov (the possessive) + -ina (country or territory).'),
  'D017523': NtO(MH='Croatia', MS='Created 7 April 1992 as a result of the division of Yugoslavia.'),
  'D017524': NtO(MH='Slovenia', MS='Created 7 April 1992 as a result of the division of Yugoslavia.'),
  'D017526': NtO(MH='Receptor, IGF Type 1', MS='A protein-tyrosine kinase receptor that is closely related in structure to the INSULIN RECEPTOR. Although commonly referred to as the IGF-I receptor, it binds both IGF-I and IGF-II with high affinity. It is comprised of a tetramer of two alpha and two beta subunits which are derived from cleavage of a single precursor protein. The beta subunit contains an intrinsic tyrosine kinase domain.'),
  'D017527': NtO(MH='Receptor, IGF Type 2', MS='A receptor that is specific for IGF-II and mannose-6-phosphate. The receptor is a 250-kDa single chain polypeptide which is unrelated in structure to the type 1 IGF receptor (RECEPTOR, IGF TYPE 1) and does not have a tyrosine kinase domain.'),
  'D017530': NtO(MH='Health Care Quality, Access, and Evaluation', MS='The concept concerned with all aspects of the quality, accessibility, and appraisal of health care and health care delivery.'),
  'D017531': NtO(MH='Health Care Evaluation Mechanisms', MS='Methods and techniques used in evaluating the quality of health care, its planning, and delivery.'),
  'D017533': NtO(MH='Marital Status', MS="A demographic parameter indicating a person's status with respect to marriage, divorce, widowhood, singleness, etc."),
  'D017534': NtO(MH='Radial Artery', MS='The direct continuation of the brachial trunk, originating at the bifurcation of the brachial artery opposite the neck of the radius. Its branches may be divided into three groups corresponding to the three regions in which the vessel is situated, the forearm, wrist, and hand.'),
  'D017535': NtO(MH='Ulnar Artery', MS='The larger of the two terminal branches of the brachial artery, beginning about one centimeter distal to the bend of the elbow. Like the RADIAL ARTERY, its branches may be divided into three groups corresponding to their locations in the forearm, wrist, and hand.'),
  'D017536': NtO(MH='Carotid Artery, Common', MS='The two principal arteries supplying the structures of the head and neck. They ascend in the neck, one on each side, and at the level of the upper border of the thyroid cartilage, each divides into two branches, the external (CAROTID ARTERY, EXTERNAL) and internal (CAROTID ARTERY, INTERNAL) carotid arteries.'),
  'D017537': NtO(MH='Mesenteric Artery, Inferior', MS='The artery supplying nearly all the left half of the transverse colon, the whole of the descending colon, the sigmoid colon, and the greater part of the rectum. It is smaller than the superior mesenteric artery (MESENTERIC ARTERY, SUPERIOR) and arises from the aorta above its bifurcation into the common iliac arteries.'),
  'D017538': NtO(MH='Mesenteric Artery, Superior', MS='A large vessel supplying the whole length of the small intestine except the superior part of the duodenum. It also supplies the cecum and the ascending part of the colon and about half the transverse part of the colon. It arises from the anterior surface of the aorta below the celiac artery at the level of the first lumbar vertebra.'),
  'D017539': NtO(MH='Tunica Intima', MS='The innermost layer of an artery or vein, made up of one layer of endothelial cells and supported by an internal elastic lamina.'),
  'D017540': NtO(MH='Tunica Media', MS='The middle layer of blood vessel walls, composed principally of thin, cylindrical, smooth muscle cells and elastic tissue. It accounts for the bulk of the wall of most arteries. The smooth muscle cells are arranged in circular layers around the vessel, and the thickness of the coat varies with the size of the vessel.'),
  'D017541': NtO(MH='Aneurysm, False', MS='Not an aneurysm but a well-defined collection of blood and CONNECTIVE TISSUE outside the wall of a blood vessel or the heart. It is the containment of a ruptured blood vessel or heart, such as sealing a rupture of the left ventricle. False aneurysm is formed by organized THROMBUS and HEMATOMA in surrounding tissue.'),
  'D017542': NtO(MH='Aneurysm, Ruptured', MS='The tearing or bursting of the weakened wall of the aneurysmal sac, usually heralded by sudden worsening pain. The great danger of a ruptured aneurysm is the large amount of blood spilling into the surrounding tissues and cavities, causing HEMORRHAGIC SHOCK.'),
  'D017543': NtO(MH='Iliac Aneurysm', MS='Abnormal balloon- or sac-like dilatation in the wall of any one of the iliac arteries including the common, the internal, or the external ILIAC ARTERY.'),
  'D017544': NtO(MH='Aortic Aneurysm, Abdominal', MS='An abnormal balloon- or sac-like dilatation in the wall of the ABDOMINAL AORTA which gives rise to the visceral, the parietal, and the terminal (iliac) branches below the aortic hiatus at the diaphragm.'),
  'D017545': NtO(MH='Aortic Aneurysm, Thoracic', MS='An abnormal balloon- or sac-like dilatation in the wall of the THORACIC AORTA. This proximal descending portion of aorta gives rise to the visceral and the parietal branches above the aortic hiatus at the diaphragm.'),
  'D017546': NtO(MH='Angioscopy', MS='Endoscopic examination, therapy or surgery performed on the interior of blood vessels.'),
  'D017548': NtO(MH='Echocardiography, Transesophageal', MS='Ultrasonic recording of the size, motion, and composition of the heart and surrounding tissues using a transducer placed in the esophagus.'),
  'D017550': NtO(MH='Spectroscopy, Fourier Transform Infrared', MS='A spectroscopic technique in which a range of wavelengths is presented simultaneously with an interferometer and the spectrum is mathematically derived from the pattern thus obtained.'),
  'D017551': NtO(MH='Microdialysis', MS='A technique for measuring extracellular concentrations of substances in tissues, usually in vivo, by means of a small probe equipped with a semipermeable membrane. Substances may also be introduced into the extracellular space through the membrane.'),
  'D017552': NtO(MH='Solitary Nucleus', MS='GRAY MATTER located in the dorsomedial part of the MEDULLA OBLONGATA associated with the solitary tract. The solitary nucleus receives inputs from most organ systems including the terminations of the facial, glossopharyngeal, and vagus nerves. It is a major coordinator of AUTONOMIC NERVOUS SYSTEM regulation of cardiovascular, respiratory, gustatory, gastrointestinal, and chemoreceptive aspects of HOMEOSTASIS. The solitary nucleus is also notable for the large number of NEUROTRANSMITTERS which are found therein.'),
  'D017553': NtO(MH='Phosphorus Compounds', MS='Inorganic compounds that contain phosphorus as an integral part of the molecule.'),
  'D017554': NtO(MH='Carbon Compounds, Inorganic', MS='Inorganic compounds that contain carbon as an integral part of the molecule but are not derived from hydrocarbons.'),
  'D017556': NtO(MH='Technetium Compounds', MS='Inorganic compounds that contain TECHNETIUM as an integral part of the molecule. Technetium 99m (m=metastable) is an isotope of technetium that has a half-life of about 6 hours. Technetium 99, which has a half-life of 210,000 years, is a decay product of technetium 99m.'),
  'D017557': NtO(MH='Ventral Tegmental Area', MS='A region in the MESENCEPHALON which is dorsomedial to the SUBSTANTIA NIGRA and ventral to the RED NUCLEUS. The mesocortical and mesolimbic dopaminergic systems originate here, including an important projection to the NUCLEUS ACCUMBENS. Overactivity of the cells in this area has been suspected to contribute to the positive symptoms of SCHIZOPHRENIA.'),
  'D017558': NtO(MH='Elective Surgical Procedures', MS='Surgery which could be postponed or not done at all without danger to the patient. Elective surgery includes procedures to correct non-life-threatening medical problems as well as to alleviate conditions causing psychological stress or other potential risk to patients, e.g., cosmetic or contraceptive surgery.'),
  'D017562': NtO(MH='Postcholecystectomy Syndrome', MS='Abdominal symptoms after removal of the GALLBLADDER. The common postoperative symptoms are often the same as those present before the operation, such as COLIC, bloating, NAUSEA, and VOMITING. There is pain on palpation of the right upper quadrant and sometimes JAUNDICE. The term is often used, inaccurately, to describe such postoperative symptoms not due to gallbladder removal.'),
  'D017563': NtO(MH='Lung Diseases, Interstitial', MS='A diverse group of lung diseases that affect the lung parenchyma. They are characterized by an initial inflammation of PULMONARY ALVEOLI that extends to the interstitium and beyond leading to diffuse PULMONARY FIBROSIS. Interstitial lung diseases are classified by their etiology (known or unknown causes), and radiological-pathological features.'),
  'D017564': NtO(MH='Radiation Pneumonitis', MS='Inflammation of the lung due to harmful effects of ionizing or non-ionizing radiation.'),
  'D017565': NtO(MH='Sarcoidosis, Pulmonary', MS='Sarcoidosis affecting predominantly the lungs, the site most frequently involved and most commonly causing morbidity and mortality in sarcoidosis. Pulmonary sarcoidosis is characterized by sharply circumscribed granulomas in the alveolar, bronchial, and vascular walls, composed of tightly packed cells derived from the mononuclear phagocyte system. The clinical symptoms when present are dyspnea upon exertion, nonproductive cough, and wheezing. (Cecil Textbook of Medicine, 19th ed, p431)'),
  'D017566': NtO(MH='Microvascular Angina', MS='ANGINA PECTORIS or angina-like chest pain with a normal coronary arteriogram and positive EXERCISE TEST. The cause of the syndrome is unknown. While its recognition is of clinical importance, its prognosis is excellent. (Braunwald, Heart Disease, 4th ed, p1346; Jablonski Dictionary of Syndromes & Eponymic Diseases, 2d ed). It is different from METABOLIC SYNDROME X, a syndrome characterized by INSULIN RESISTANCE and HYPERINSULINEMIA, that has increased risk for cardiovascular disease.'),
  'D017567': NtO(MH='Karnofsky Performance Status', MS="A performance measure for rating the ability of a person to perform usual activities, evaluating a patient's progress after a therapeutic procedure, and determining a patient's suitability for therapy. It is used most commonly in the prognosis of cancer therapy, usually after chemotherapy and customarily administered before and after therapy. It was named for Dr. David A. Karnofsky, an American specialist in cancer chemotherapy."),
  'D017568': NtO(MH='Rectus Abdominis', MS="A long flat muscle that extends along the whole length of both sides of the abdomen. It flexes the vertebral column, particularly the lumbar portion; it also tenses the anterior abdominal wall and assists in compressing the abdominal contents. It is frequently the site of hematomas. In reconstructive surgery it is often used for the creation of myocutaneous flaps. (From Gray's Anatomy, 30th American ed, p491)"),
  'D017570': NtO(MH='MyoD Protein', MS='A myogenic regulatory factor that controls myogenesis. Though it is not clear how its function differs from the other myogenic regulatory factors, MyoD appears to be related to fusion and terminal differentiation of the muscle cell.'),
  'D017572': NtO(MH='Leukotriene A4', MS='(2S-(2 alpha,3 beta(1E,3E,5Z,8Z)))-3-(1,3,5,8-Tetradecatetraenyl)oxiranebutanoic acid. An unstable allylic epoxide, formed from the immediate precursor 5-HPETE via the stereospecific removal of a proton at C-10 and dehydration. Its biological actions are determined primarily by its metabolites, i.e., LEUKOTRIENE B4 and cysteinyl-leukotrienes. Alternatively, leukotriene A4 is converted into LEUKOTRIENE C4 by glutathione-S-transferase or into 5,6-di-HETE by the epoxide-hydrolase. (From Dictionary of Prostaglandins and Related Compounds, 1990)'),
  'D017573': NtO(MH='Focal Epithelial Hyperplasia', MS='Hyperplasia of the mucous membrane of the lips, tongue, and less commonly, the buccal mucosa, floor of the mouth, and palate, presenting soft, painless, round to oval sessile papules about 1 to 4 mm in diameter. The condition usually occurs in children and young adults and has familial predilection, lasting for several months, sometimes years, before running its course. A viral etiology is suspected, the isolated organism being usually the human papillomavirus. (Jablonski, Illustrated Dictionary of Dentistry; Belshe, Textbook of Human Virology, 2d ed, p954)'),
  'D017574': NtO(MH='Parasystole', MS='A cardiac arrhythmia that is caused by interaction of two independently initiated cardiac impulses of different rates from two separate foci. Generally one focus is the SINOATRIAL NODE, the normal pacemaker. The ectopic focus is usually in the HEART VENTRICLE but can be in the HEART ATRIUM or the ATRIOVENTRICULAR NODE. Modulation of the parasystolic rhythm by the sinus rhythm depends on the completeness of entrance block surrounding the parasystolic focus.'),
  'D017575': NtO(MH='Gloves, Protective', MS='Covering for the hands usually with separation for the fingers, made of various materials for protection against infections, toxic substances, extremes of hot and cold, radiation, water immersion, etc. Protective gloves may be worn by patients, care givers, laboratory personnel, industrial workers, and police, among others.'),
  'D017576': NtO(MH='Daptomycin', MS='A cyclic lipopeptide antibiotic that inhibits GRAM-POSITIVE BACTERIA.'),
  'D017577': NtO(MH='Cutaneous Fistula', MS='An abnormal passage or communication leading from an internal organ to the surface of the body.'),
  'D017578': NtO(MH='Immunoglobulin Class Switching', MS='Gene rearrangement of the B-lymphocyte which results in a substitution in the type of heavy-chain constant region that is expressed. This allows the effector response to change while the antigen binding specificity (variable region) remains the same. The majority of class switching occurs by a DNA recombination event but it also can take place at the level of RNA processing.'),
  'D017579': NtO(MH='Domestic Violence', MS='Deliberate, often repetitive physical, verbal, and/or other types of abuse by one or more members against others of a household.'),
  'D017582': NtO(MH='Renal Replacement Therapy', MS='Procedures which temporarily or permanently remedy insufficient cleansing of body fluids by the kidneys.'),
  'D017583': NtO(MH='Hemodiafiltration', MS='The combination of hemodialysis and hemofiltration either simultaneously or sequentially. Convective transport (hemofiltration) may be better for removal of larger molecular weight substances and diffusive transport (hemodialysis) for smaller molecular weight solutes.'),
  'D017584': NtO(MH='Reproductive History', MS="An important aggregate factor in epidemiological studies of women's health. The concept usually includes the number and timing of pregnancies and their outcomes, the incidence of breast feeding, and may include age of menarche and menopause, regularity of menstruation, fertility, gynecological or obstetric problems, or contraceptive usage."),
  'D017585': NtO(MH='Ultrasonography, Doppler, Transcranial', MS='A non-invasive technique using ultrasound for the measurement of cerebrovascular hemodynamics, particularly cerebral blood flow velocity and cerebral collateral flow. With a high-intensity, low-frequency pulse probe, the intracranial arteries may be studied transtemporally, transorbitally, or from below the foramen magnum.'),
  'D017586': NtO(MH='Diskectomy', MS='Excision, in part or whole, of an INTERVERTEBRAL DISC. The most common indication is disk displacement or herniation. In addition to standard surgical removal, it can be performed by percutaneous diskectomy (DISKECTOMY, PERCUTANEOUS) or by laparoscopic diskectomy, the former being the more common.'),
  'D017587': NtO(MH='Diskectomy, Percutaneous', MS='Percutaneous excision of a herniated or displaced INTERVERTEBRAL DISC by posterolateral approach, always remaining outside the spinal canal. Percutaneous nucleotomy was first described by Hijikata in Japan in 1975. In 1985 Onik introduced automated percutaneous nucleotomy which consists in percutaneous aspiration of the nucleus pulposus. It is carried out under local anesthesia, thus reducing the surgical insult and requiring brief hospitalization, often performed on an outpatient basis. It appears to be a well-tolerated alternative to surgical diskectomy and chymopapain nucleolysis.'),
  'D017588': NtO(MH='Hyperandrogenism', MS='A condition caused by the excessive secretion of ANDROGENS from the ADRENAL CORTEX; the OVARIES; or the TESTES. The clinical significance in males is negligible. In women, the common manifestations are HIRSUTISM and VIRILISM as seen in patients with POLYCYSTIC OVARY SYNDROME and ADRENOCORTICAL HYPERFUNCTION.'),
  'D017589': NtO(MH='Immunization Programs', MS='Organized services to administer immunization procedures in the prevention of various diseases. The programs are made available over a wide range of sites: schools, hospitals, public health agencies, voluntary health agencies, etc. They are administered to an equally wide range of population groups or on various administrative levels: community, municipal, state, national, international.'),
  'D017590': NtO(MH='Myelinolysis, Central Pontine', MS='A demyelinating condition affecting the PONS and characterized clinically by an acute progressive QUADRIPLEGIA; DYSARTHRIA; DYSPHAGIA; and alterations of consciousness. Pathologic features include prominent demyelination in the central PONS with sparing of axons and neurons. This condition is usually associated with systemic disorders such as HYPONATREMIA; chronic ALCOHOLISM; LIVER FAILURE; severe BURNS; malignant NEOPLASMS; hemorrhagic PANCREATITIS; HEMODIALYSIS; and SEPSIS. The rapid medical correction of hyponatremia has been cited as a cause of this condition. (From Adams et al., Principles of Neurology, 6th ed, pp1125-6)'),
  'D017591': NtO(MH='Petrosal Sinus Sampling', MS='Sampling of blood levels of the adrenocorticotropic hormone (ACTH) by withdrawal of blood from the inferior petrosal sinus. The inferior petrosal sinus arises from the cavernous sinus and runs to the internal jugular vein. Sampling of blood at this level is a valuable tool in the differential diagnosis of Cushing disease, Cushing syndrome, and other adrenocortical diseases.'),
  'D017592': NtO(MH='Skin Care', MS='Maintenance of the hygienic state of the skin under optimal conditions of cleanliness and comfort. Effective in skin care are proper washing, bathing, cleansing, and the use of soaps, detergents, oils, etc. In various disease states, therapeutic and protective solutions and ointments are useful. The care of the skin is particularly important in various occupations, in exposure to sunlight, in neonates, and in PRESSURE ULCER.'),
  'D017593': NtO(MH='Kleine-Levin Syndrome', MS='A rare condition characterized by recurrent hypersomnias associated with hyperphagia, occurring primarily in males in the second to third decade of life. Clinical features include mental confusion, excessive sleep requirements (approximately 18 hours per day), restlessness, and in some cases hallucinations. Episodes have a duration of days to weeks, and may recur several times per year. This condition may resolve spontaneously over several years. (From Adams, et al., Principles of Neurology, 6th ed, p569)'),
  'D017594': NtO(MH='Publication Bias', MS='The influence of study results on the chances of publication and the tendency of investigators, reviewers, and editors to submit or accept manuscripts for publication based on the direction or strength of the study findings. Publication bias has an impact on the interpretation of clinical trials and meta-analyses. Bias can be minimized by insistence by editors on high-quality research, thorough literature reviews, acknowledgement of conflicts of interest, modification of peer review practices, etc.'),
  'D017595': NtO(MH='Renal Plasma Flow', MS='The amount of PLASMA that perfuses the KIDNEYS per unit time, approximately 10% greater than effective renal plasma flow (RENAL PLASMA FLOW, EFFECTIVE). It should be differentiated from the RENAL BLOOD FLOW; (RBF), which refers to the total volume of BLOOD flowing through the renal vasculature, while the renal plasma flow refers to the rate of plasma flow (RPF).'),
  'D017596': NtO(MH='Renal Plasma Flow, Effective', MS='The amount of PLASMA flowing to the parts of the KIDNEY that function in the production of urine. It is the amount of plasma perfusing the KIDNEY TUBULES per unit time, generally measured by P-AMINOHIPPURATE clearance. It should be differentiated from RENAL PLASMA FLOW which is approximately 10% greater than the effective renal plasma flow.'),
  'D017597': NtO(MH='Renal Blood Flow, Effective', MS='The amount of the RENAL BLOOD FLOW that is going to the functional renal tissue, i.e., parts of the KIDNEY that are involved in production of URINE.'),
  'D017598': NtO(MH='Efficiency, Organizational', MS='The capacity of an organization, institution, or business to produce desired results with a minimum expenditure of energy, time, money, personnel, material, etc.'),
  'D017599': NtO(MH='Neuroectodermal Tumors', MS='Malignant neoplasms arising in the neuroectoderm, the portion of the ectoderm of the early embryo that gives rise to the central and peripheral nervous systems, including some glial cells.'),
  'D017600': NtO(MH='Neuroectodermal Tumor, Melanotic', MS='A benign, rapidly growing, deeply pigmented tumor of the jaw and occasionally of other sites, consisting of an infiltrating mass of cells arranged in an alveolar pattern, and occurring almost exclusively in infants. Its source of origin is in dispute, the various theories giving rise to its several names. (Dorland, 27th ed)'),
  'D017601': NtO(MH='Oxygen Compounds', MS='Inorganic compounds that contain oxygen as an integral part of the molecule.'),
  'D017602': NtO(MH='Lithotripsy, Laser', MS='Fragmentation of CALCULI, notably urinary or biliary, by LASER.'),
  'D017603': NtO(MH='Signal Detection, Psychological', MS='Psychophysical technique that permits the estimation of the bias of the observer as well as detectability of the signal (i.e., stimulus) in any sensory modality. (From APA, Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D017604': NtO(MH='Cesarean Section, Repeat', MS='Extraction of the fetus by abdominal hysterotomy anytime following a previous cesarean.'),
  'D017605': NtO(MH='Bromine Compounds', MS='Inorganic compounds that contain bromine as an integral part of the molecule.'),
  'D017606': NtO(MH='Chlorine Compounds', MS='Inorganic compounds that contain chlorine as an integral part of the molecule.'),
  'D017607': NtO(MH='Aluminum Compounds', MS='Inorganic compounds that contain aluminum as an integral part of the molecule.'),
  'D017608': NtO(MH='Chromium Compounds', MS='Inorganic compounds that contain chromium as an integral part of the molecule.'),
  'D017609': NtO(MH='Barium Compounds', MS='Inorganic compounds that contain barium as an integral part of the molecule.'),
  'D017610': NtO(MH='Calcium Compounds', MS='Inorganic compounds that contain calcium as an integral part of the molecule.'),
  'D017611': NtO(MH='Fluorine Compounds', MS='Inorganic compounds that contain fluorine as an integral part of the molecule.'),
  'D017612': NtO(MH='Gold Compounds', MS='Inorganic compounds that contain gold as an integral part of the molecule.'),
  'D017613': NtO(MH='Iodine Compounds', MS='Inorganic compounds that contain iodine as an integral part of the molecule.'),
  'D017615': NtO(MH='Enteric Nervous System', MS='Two ganglionated neural plexuses in the gut wall which form one of the three major divisions of the autonomic nervous system. The enteric nervous system innervates the gastrointestinal tract, the pancreas, and the gallbladder. It contains sensory neurons, interneurons, and motor neurons. Thus the circuitry can autonomously sense the tension and the chemical environment in the gut and regulate blood vessel tone, motility, secretions, and fluid transport. The system is itself governed by the central nervous system and receives both parasympathetic and sympathetic innervation. (From Kandel, Schwartz, and Jessel, Principles of Neural Science, 3d ed, p766)'),
  'D017616': NtO(MH='Magnesium Compounds', MS='Inorganic compounds that contain magnesium as an integral part of the molecule.'),
  'D017619': NtO(MH='Hemibody Irradiation', MS='Irradiation of one half or both halves of the body in the treatment of disseminated cancer or widespread metastases. It is used to treat diffuse metastases in one session as opposed to multiple fields over an extended period. The more frequent treatment modalities are upper hemibody irradiation (UHBI) or lower hemibody irradiation (LHBI). Less common is mid-body irradiation (MBI). In the treatment of both halves of the body sequentially, hemibody irradiation permits radiotherapy of the whole body with larger doses of radiation than could be accomplished with WHOLE-BODY IRRADIATION. It is sometimes called "systemic" hemibody irradiation with reference to its use in widespread cancer or metastases. (P. Rubin et al. Cancer, Vol 55, p2210, 1985)'),
  'D017622': NtO(MH='Periodontal Attachment Loss', MS='Loss or destruction of periodontal tissue caused by periodontitis or other destructive periodontal diseases or by injury during instrumentation. Attachment refers to the periodontal ligament which attaches to the alveolar bone. It has been hypothesized that treatment of the underlying periodontal disease and the seeding of periodontal ligament cells enable the creating of new attachment.'),
  'D017624': NtO(MH='WAGR Syndrome', MS='A contiguous gene syndrome associated with hemizygous deletions of chromosome region 11p13. The condition is marked by the combination of WILMS TUMOR; ANIRIDIA; GENITOURINARY ABNORMALITIES; and INTELLECTUAL DISABILITY.'),
  'D017626': NtO(MH='Cochlear Nucleus', MS='The brain stem nucleus that receives the central input from the cochlear nerve. The cochlear nucleus is located lateral and dorsolateral to the inferior cerebellar peduncles and is functionally divided into dorsal and ventral parts. It is tonotopically organized, performs the first stage of central auditory processing, and projects (directly or indirectly) to higher auditory areas including the superior olivary nuclei, the medial geniculi, the inferior colliculi, and the auditory cortex.'),
  'D017628': NtO(MH='Microglia', MS='The third type of glial cell, along with astrocytes and oligodendrocytes (which together form the macroglia). Microglia vary in appearance depending on developmental stage, functional state, and anatomical location; subtype terms include ramified, perivascular, ameboid, resting, and activated. Microglia clearly are capable of phagocytosis and play an important role in a wide spectrum of neuropathologies. They have also been suggested to act in several other roles including in secretion (e.g., of cytokines and neural growth factors), in immunological processing (e.g., antigen presentation), and in central nervous system development and remodeling.'),
  'D017629': NtO(MH='Gap Junctions', MS='Connections between cells which allow passage of small molecules and electric current. Gap junctions were first described anatomically as regions of close apposition between cells with a narrow (1-2 nm) gap between cell membranes. The variety in the properties of gap junctions is reflected in the number of CONNEXINS, the family of proteins which form the junctions.'),
  'D017630': NtO(MH='Connexins', MS='A group of homologous proteins which form the intermembrane channels of GAP JUNCTIONS. The connexins are the products of an identified gene family which has both highly conserved and highly divergent regions. The variety contributes to the wide range of functional properties of gap junctions.'),
  'D017632': NtO(MH='Asbestos, Serpentine', MS='A type of asbestos that occurs in nature as the dihydrate of magnesium silicate. It exists in two forms: antigorite, a plated variety, and chrysotile, a fibrous variety. The latter makes up 95% of all asbestos products. (From Merck Index, 11th ed, p.893)'),
  'D017633': NtO(MH='Magnesium Silicates', MS='A generic term for a variety of compounds that contain silicon, oxygen, and magnesium, and may contain hydrogen. Examples include TALC and some kinds of ASBESTOS.'),
  'D017634': NtO(MH='Self Tolerance', MS='The normal lack of the ability to produce an immunological response to autologous (self) antigens. A breakdown of self tolerance leads to autoimmune diseases. The ability to recognize the difference between self and non-self is the prime function of the immune system.'),
  'D017635': NtO(MH='Clonal Anergy', MS='Functional inactivation of T- or B-lymphocytes rendering them incapable of eliciting an immune response to antigen. This occurs through different mechanisms in the two kinds of lymphocytes and can contribute to SELF TOLERANCE.'),
  'D017636': NtO(MH='Asbestos, Amphibole', MS="A class of asbestos that includes silicates of magnesium, iron, calcium, and sodium. The fibers are generally brittle and cannot be spun, but are more resistant to chemicals and heat than ASBESTOS, SERPENTINE. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D017637': NtO(MH='Clonal Deletion', MS='Removal, via CELL DEATH, of immature lymphocytes that interact with antigens during maturation. For T-lymphocytes this occurs in the thymus and ensures that mature T-lymphocytes are self tolerant. B-lymphocytes may also undergo clonal deletion.'),
  'D017638': NtO(MH='Asbestos, Crocidolite', MS='A lavender, acid-resistant asbestos.'),
  'D017639': NtO(MH='Asbestos, Amosite', MS='Asbestos, grunerite. A monoclinic amphibole form of asbestos having long fibers and a high iron content. It is used in insulation. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D017640': NtO(MH='Silicates', MS='The generic term for salts derived from silica or the silicic acids. They contain silicon, oxygen, and one or more metals, and may contain hydrogen. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th Ed)'),
  'D017641': NtO(MH='Zeolites', MS='Zeolites. A group of crystalline, hydrated alkali-aluminum silicates. They occur naturally in sedimentary and volcanic rocks, altered basalts, ores, and clay deposits. Some 40 known zeolite minerals and a great number of synthetic zeolites are available commercially. (From Merck Index, 11th ed)'),
  'D017646': NtO(MH='Organosilicon Compounds', MS='Organic compounds that contain silicon as an integral part of the molecule.'),
  'D017655': NtO(MH='Silicon Compounds', MS='Inorganic compounds that contain silicon as an integral part of the molecule.'),
  'D017660': NtO(MH='Autoreceptors', MS='Transmitter receptors on or near presynaptic terminals (or varicosities) which are sensitive to the transmitter(s) released by the terminal itself. Receptors for the hormones released by hormone-releasing cells are also included.'),
  'D017661': NtO(MH='Receptors, Presynaptic', MS='Neurotransmitter receptors located on or near presynaptic terminals or varicosities. Presynaptic receptors which bind transmitter molecules released by the terminal itself are termed AUTORECEPTORS.'),
  'D017663': NtO(MH='Peplomycin', MS='An antineoplastic agent derived from BLEOMYCIN.'),
  'D017664': NtO(MH='Uncompensated Care', MS='Medical services for which no payment is received. Uncompensated care includes charity care and bad debts.'),
  'D017665': NtO(MH='Hydroxyl Radical', MS='The univalent radical OH. Hydroxyl radical is a potent oxidizing agent.'),
  'D017666': NtO(MH='Deuterium Oxide', MS="The isotopic compound of hydrogen of mass 2 (deuterium) with oxygen. (From Grant & Hackh's Chemical Dictionary, 5th ed) It is used to study mechanisms and rates of chemical or nuclear reactions, as well as biological processes."),
  'D017667': NtO(MH='Adipocytes', MS='Cells in the body that store FATS, usually in the form of TRIGLYCERIDES. WHITE ADIPOCYTES are the predominant type and found mostly in the abdominal cavity and subcutaneous tissue. BROWN ADIPOCYTES are thermogenic cells that can be found in newborns of some species and hibernating mammals.'),
  'D017668': NtO(MH='Age of Onset', MS='The age, developmental stage, or period of life at which a disease or the initial symptoms or manifestations of a disease appear in an individual.'),
  'D017669': NtO(MH='Mercury Compounds', MS='Inorganic compounds that contain mercury as an integral part of the molecule.'),
  'D017670': NtO(MH='Sodium Compounds', MS='Inorganic compounds that contain sodium as an integral part of the molecule.'),
  'D017671': NtO(MH='Platinum Compounds', MS='Inorganic compounds which contain platinum as the central atom.'),
  'D017672': NtO(MH='Nitrogen Compounds', MS='Inorganic compounds that contain nitrogen as an integral part of the molecule.'),
  'D017673': NtO(MH='Sodium Chloride, Dietary', MS='Sodium chloride used in foods.'),
  'D017674': NtO(MH='Hypophosphatemia', MS='A condition of an abnormally low level of PHOSPHATES in the blood.'),
  'D017675': NtO(MH='CREST Syndrome', MS='A mild form of LIMITED SCLERODERMA, a multi-system disorder. Its features include symptoms of CALCINOSIS; RAYNAUD DISEASE; ESOPHAGEAL MOTILITY DISORDERS; sclerodactyly, and TELANGIECTASIS. When the defect in esophageal function is not prominent, it is known as CRST syndrome.'),
  'D017676': NtO(MH='Lichen Planus, Oral', MS="Oral lesions accompanying cutaneous lichen planus or often occurring alone. The buccal mucosa, lips, gingivae, floor of the mouth, and palate are usually affected (in a descending order of frequency). Typically, oral lesions consist of radiating white or gray, velvety, threadlike lines, arranged in a reticular pattern, at the intersection of which there may be minute, white, elevated dots or streaks (Wickham's striae). (Jablonski, Illustrated Dictionary of Dentistry)"),
  'D017677': NtO(MH='Age Distribution', MS='The frequency of different ages or age groups in a given population. The distribution may refer to either how many or what proportion of the group. The population is usually patients with a specific disease but the concept is not restricted to humans and is not restricted to medicine.'),
  'D017678': NtO(MH='Sex Distribution', MS='The number of males and females in a given population. The distribution may refer to how many men or women or what proportion of either in the group. The population is usually patients with a specific disease but the concept is not restricted to humans and is not restricted to medicine.'),
  'D017679': NtO(MH='Cryotherapy', MS='A form of therapy consisting in the local or general use of cold. The selective destruction of tissue by extreme cold or freezing is CRYOSURGERY. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D017680': NtO(MH='Potassium Compounds', MS='Inorganic compounds that contain potassium as an integral part of the molecule.'),
  'D017681': NtO(MH='Hypereosinophilic Syndrome', MS='A heterogeneous group of disorders with the common feature of prolonged eosinophilia of unknown cause and associated organ system dysfunction, including the heart, central nervous system, kidneys, lungs, gastrointestinal tract, and skin. There is a massive increase in the number of EOSINOPHILS in the blood, mimicking leukemia, and extensive eosinophilic infiltration of the various organs.'),
  'D017682': NtO(MH='Myocardial Stunning', MS='Prolonged dysfunction of the myocardium after a brief episode of severe ischemia, with gradual return of contractile activity.'),
  'D017683': NtO(MH='Oncorhynchus', MS='A genus of the family SALMONIDAE (salmons and trouts). They are named for their hooked (onco) nose (rhynchus). They are usually anadromous and occasionally inhabit freshwater. They can be found in North Pacific coastal areas from Japan to California and adjacent parts of the Arctic Ocean. Salmon and trout are popular game and food fish. Various species figure heavily in genetic, metabolism, and hormone research.'),
  'D017684': NtO(MH='Oncorhynchus kisutch', MS='An anadromous species of SALMON ranging from the Arctic and Pacific Oceans to Monterey Bay, California and inhabiting ocean and coastal streams. It is familiarly known as the coho or silver salmon. It is relatively small but its light-colored flesh is of good flavor.'),
  'D017685': NtO(MH='Oncorhynchus keta', MS='An anadromous species of SALMON found in the streams of the Pacific coast from Sacramento north, and also common in Japan. It is used frequently in genetic and other medical research.'),
  'D017686': NtO(MH='Oncorhynchus mykiss', MS='A large stout-bodied, sometimes anadromous, TROUT found in still and flowing waters of the Pacific coast from southern California to Alaska. It has a greenish back, a whitish belly, and pink, red, or lavender stripes on the sides, with usually a sprinkling of black dots. It is highly regarded as a sport and food fish. Its former name was Salmo gairdneri. The sea-run rainbow trouts are often called steelheads. Redband trouts refer to interior populations of rainbows.'),
  'D017687': NtO(MH='Radon Daughters', MS="Short-lived radioactive decay products of radon that include 216-Po, 214-Pb, 214-Bi, and 214-Po. They have an effective half-life of about 30 minutes and are solids that can deposit on the bronchial airways during inhalation and exhalation. This results in exposure of the respiratory airways to alpha radiation and can lead to diseases of the respiratory system, including lung cancer. (From Casarett and Doull's Toxicology, 4th ed, p740)"),
  'D017688': NtO(MH='Cholera Morbus', MS='An old term that is no longer used in the scientific literature. Cholera morbus refers to acute GASTROENTERITIS occurring in summer or autumn; characterized by severe cramps, diarrhea, and vomiting.'),
  'D017689': NtO(MH='Polydactyly', MS='A congenital anomaly of the hand or foot, marked by the presence of supernumerary digits.'),
  'D017690': NtO(MH='Cell Transplantation', MS='Transference of cells within an individual, between individuals of the same species, or between individuals of different species.'),
  'D017691': NtO(MH='Caribbean Region', MS='The area that lies between continental North and South America and comprises the Caribbean Sea, the West Indies, and the adjacent mainland regions of southern Mexico, Central America, Colombia, and Venezuela.'),
  'D017693': NtO(MH='Sodium Bicarbonate', MS='A white, crystalline powder that is commonly used as a pH buffering agent, an electrolyte replenisher, systemic alkalizer and in topical cleansing solutions.'),
  'D017694': NtO(MH='Rewarming', MS='Application of heat to correct hypothermia, accidental or induced.'),
  'D017695': NtO(MH='Soft Tissue Injuries', MS='Injuries of tissue other than bone. The concept is usually general and does not customarily refer to internal organs or viscera. It is meaningful with reference to regions or organs where soft tissue (muscle, fat, skin) should be differentiated from bones or bone tissue, as "soft tissue injuries of the hand".'),
  'D017696': NtO(MH='Myopathies, Nemaline', MS='A group of inherited congenital myopathic conditions characterized clinically by weakness, hypotonia, and prominent hypoplasia of proximal muscles including the face. Muscle biopsy reveals large numbers of rod-shaped structures beneath the muscle fiber plasma membrane. This disorder is genetically heterogeneous and may occasionally present in adults. (Adams et al., Principles of Neurology, 6th ed, p1453)'),
  'D017697': NtO(MH='Premenopause', MS='The period before MENOPAUSE. In premenopausal women, the climacteric transition from full sexual maturity to cessation of ovarian cycle takes place between the age of late thirty and early fifty.'),
  'D017698': NtO(MH='Postmenopause', MS='The physiological period following the MENOPAUSE, the permanent cessation of the menstrual life.'),
  'D017699': NtO(MH='Pelvic Pain', MS='Pain in the pelvic region of genital and non-genital origin.'),
  'D017700': NtO(MH='Embolism, Cholesterol', MS='Blocking of a blood vessel by CHOLESTEROL-rich atheromatous deposits, generally occurring in the flow from a large artery to small arterial branches. It is also called arterial-arterial embolization or atheroembolism which may be spontaneous or iatrogenic. Patients with spontaneous atheroembolism often have painful, cyanotic digits of acute onset.'),
  'D017701': NtO(MH='Reticulocyte Count', MS='The number of RETICULOCYTES per unit volume of BLOOD. The values are expressed as a percentage of the ERYTHROCYTE COUNT or in the form of an index ("corrected reticulocyte index"), which attempts to account for the number of circulating erythrocytes.'),
  'D017703': NtO(MH='Retropharyngeal Abscess', MS='An accumulation of purulent material in the space between the PHARYNX and the CERVICAL VERTEBRAE. This usually results from SUPPURATION of retropharyngeal LYMPH NODES in patients with UPPER RESPIRATORY TRACT INFECTIONS, perforation of the pharynx, or head and neck injuries.'),
  'D017704': NtO(MH='Baroreflex', MS='A response by the BARORECEPTORS to increased BLOOD PRESSURE. Increased pressure stretches BLOOD VESSELS which activates the baroreceptors in the vessel walls. The net response of the CENTRAL NERVOUS SYSTEM is a reduction of central sympathetic outflow. This reduces blood pressure both by decreasing peripheral VASCULAR RESISTANCE and by lowering CARDIAC OUTPUT. Because the baroreceptors are tonically active, the baroreflex can compensate rapidly for both increases and decreases in blood pressure.'),
  'D017705': NtO(MH='Lignans', MS='A class of dibenzylbutane derivatives which occurs in higher plants and in fluids (bile, serum, urine, etc.) in man and other animals. These compounds, which have a potential anti-cancer role, can be synthesized in vitro by human fecal flora. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D017706': NtO(MH='Lisinopril', MS='One of the ANGIOTENSIN-CONVERTING ENZYME INHIBITORS (ACE inhibitors), orally active, that has been used in the treatment of hypertension and congestive heart failure.'),
  'D017707': NtO(MH='Erythrocyte Transfusion', MS='The transfer of erythrocytes from a donor to a recipient or reinfusion to the donor.'),
  'D017708': NtO(MH='Leukocyte Transfusion', MS='The transfer of leukocytes from a donor to a recipient or reinfusion to the donor.'),
  'D017709': NtO(MH='Fractals', MS='Patterns (real or mathematical) which look similar at different scales, for example the network of airways in the lung which shows similar branching patterns at progressively higher magnifications. Natural fractals are self-similar across a finite range of scales while mathematical fractals are the same across an infinite range. Many natural, including biological, structures are fractal (or fractal-like). Fractals are related to "chaos" (see NONLINEAR DYNAMICS) in that chaotic processes can produce fractal structures in nature, and appropriate representations of chaotic processes usually reveal self-similarity over time.'),
  'D017710': NtO(MH='Lymphocyte Transfusion', MS='The transfer of lymphocytes from a donor to a recipient or reinfusion to the donor.'),
  'D017711': NtO(MH='Nonlinear Dynamics', MS='The study of systems which respond disproportionately (nonlinearly) to initial conditions or perturbing stimuli. Nonlinear systems may exhibit "chaos" which is classically characterized as sensitive dependence on initial conditions. Chaotic systems, while distinguished from more ordered periodic systems, are not random. When their behavior over time is appropriately displayed (in "phase space"), constraints are evident which are described by "strange attractors". Phase space representations of chaotic systems, or strange attractors, usually reveal fractal (FRACTALS) self-similarity across time scales. Natural, including biological, systems often display nonlinear dynamics and chaos.'),
  'D017712': NtO(MH='Peer Review, Research', MS='The evaluation by experts of the quality and pertinence of research or research proposals of other experts in the same field. Peer review is used by editors in deciding which submissions warrant publication, by granting agencies to determine which proposals should be funded, and by academic institutions in tenure decisions.'),
  'D017713': NtO(MH='Platelet Transfusion', MS='The transfer of blood platelets from a donor to a recipient or reinfusion to the donor.'),
  'D017714': NtO(MH='Community-Acquired Infections', MS='Any infection acquired in the community, that is, contrasted with those acquired in a health care facility (CROSS INFECTION). An infection would be classified as community-acquired if the patient had not recently been in a health care facility or been in contact with someone who had been recently in a health care facility.'),
  'D017719': NtO(MH='Diabetic Foot', MS='Common foot problems in persons with DIABETES MELLITUS, caused by any combination of factors such as DIABETIC NEUROPATHIES; PERIPHERAL VASCULAR DISEASES; and INFECTION. With the loss of sensation and poor circulation, injuries and infections often lead to severe foot ulceration, GANGRENE and AMPUTATION.'),
  'D017720': NtO(MH='Molecular Epidemiology', MS='The application of molecular biology to the answering of epidemiological questions. The examination of patterns of changes in DNA to implicate particular carcinogens and the use of molecular markers to predict which individuals are at highest risk for a disease are common examples.'),
  'D017721': NtO(MH='Hospital Costs', MS='The expenses incurred by a hospital in providing care. The hospital costs attributed to a particular patient care episode include the direct costs plus an appropriate proportion of the overhead for administration, personnel, building maintenance, equipment, etc. Hospital costs are one of the factors which determine HOSPITAL CHARGES (the price the hospital sets for its services).'),
  'D017722': NtO(MH='Hospital Charges', MS='The prices a hospital sets for its services. HOSPITAL COSTS (the direct and indirect expenses incurred by the hospital in providing the services) are one factor in the determination of hospital charges. Other factors may include, for example, profits, competition, and the necessity of recouping the costs of uncompensated care.'),
  'D017723': NtO(MH='Drug Utilization Review', MS='Formal programs for assessing drug prescription against some standard. Drug utilization review may consider clinical appropriateness, cost effectiveness, and, in some cases, outcomes. Review is usually retrospective, but some analysis may be done before drugs are dispensed (as in computer systems which advise physicians when prescriptions are entered). Drug utilization review is mandated for Medicaid programs beginning in 1993.'),
  'D017725': NtO(MH='Ventricular Pressure', MS='The pressure within a CARDIAC VENTRICLE. Ventricular pressure waveforms can be measured in the beating heart by catheterization or estimated using imaging techniques (e.g., DOPPLER ECHOCARDIOGRAPHY). The information is useful in evaluating the function of the MYOCARDIUM; CARDIAC VALVES; and PERICARDIUM, particularly with simultaneous measurement of other (e.g., aortic or atrial) pressures.'),
  'D017726': NtO(MH='Cytomegalovirus Retinitis', MS='Infection of the retina by cytomegalovirus characterized by retinal necrosis, hemorrhage, vessel sheathing, and retinal edema. Cytomegalovirus retinitis is a major opportunistic infection in AIDS patients and can cause blindness.'),
  'D017727': NtO(MH='Chicken anemia virus', MS='The type species of GYROVIRUS, a small, non-enveloped DNA virus originally isolated from contaminated vaccines in Japan. It causes chicken infectious anemia and may possibly play a key role in hemorrhagic anemia syndrome, anemia dermatitis, and blue wing disease.'),
  'D017728': NtO(MH='Lymphoma, Large-Cell, Anaplastic', MS='A systemic, large-cell, non-Hodgkin, malignant lymphoma characterized by cells with pleomorphic appearance and expressing the CD30 ANTIGEN. These so-called "hallmark" cells have lobulated and indented nuclei. This lymphoma is often mistaken for metastatic carcinoma and MALIGNANT HISTIOCYTOSIS.'),
  'D017729': NtO(MH='Presynaptic Terminals', MS='The distal terminations of axons which are specialized for the release of neurotransmitters. Also included are varicosities along the course of axons which have similar specializations and also release transmitters. Presynaptic terminals in both the central and peripheral nervous systems are included.'),
  'D017730': NtO(MH='Ki-1 Antigen', MS='A member of the tumor necrosis factor receptor superfamily that may play a role in the regulation of NF-KAPPA B and APOPTOSIS. It is found on activated T-LYMPHOCYTES; B-LYMPHOCYTES; NEUTROPHILS; EOSINOPHILS; MAST CELLS and NK CELLS. Overexpression of the Ki-1 antigen in hematopoietic malignancies make it clinically useful as a biological tumor marker. Signaling of the receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D017731': NtO(MH='Lymphomatoid Papulosis', MS='Clinically benign, histologically malignant, recurrent cutaneous T-cell lymphoproliferative disorder characterized by an infiltration of large atypical cells surrounded by inflammatory cells. The atypical cells resemble REED-STERNBERG CELLS of HODGKIN DISEASE or the malignant cells of CUTANEOUS T-CELL LYMPHOMA. In some cases, lymphomatoid papulosis progresses to lymphomatous conditions including MYCOSIS FUNGOIDES; HODGKIN DISEASE; CUTANEOUS T-CELL LYMPHOMA; or ANAPLASTIC LARGE-CELL LYMPHOMA.'),
  'D017732': NtO(MH='Air Ambulances', MS='Fixed-wing aircraft or helicopters equipped for air transport of patients.'),
  'D017733': NtO(MH='Leukoplakia, Hairy', MS='Epithelial hyperplasia of the oral mucosa associated with Epstein-Barr virus (HERPESVIRUS 4, HUMAN) and found almost exclusively in persons with HIV infection. The lesion consists of a white patch that is often corrugated or hairy.'),
  'D017734': NtO(MH='Bile Ducts, Extrahepatic', MS='Passages external to the liver for the conveyance of bile. These include the COMMON BILE DUCT and the common hepatic duct (HEPATIC DUCT, COMMON).'),
  'D017735': NtO(MH='Virus Latency', MS='The ability of a pathogenic virus to lie dormant within a cell (latent infection). In eukaryotes, subsequent activation and viral replication is thought to be caused by extracellular stimulation of cellular transcription factors. Latency in bacteriophage is maintained by the expression of virally encoded repressors.'),
  'D017736': NtO(MH='Ion-Selective Electrodes', MS='Electrodes which can be used to measure the concentration of particular ions in cells, tissues, or solutions.'),
  'D017737': NtO(MH='Macrophages, Peritoneal', MS='Mononuclear phagocytes derived from bone marrow precursors but resident in the peritoneum.'),
  'D017738': NtO(MH='Alkanesulfonic Acids', MS='Sulfonic acid derivatives that are substituted with an aliphatic hydrocarbon group.'),
  'D017739': NtO(MH='Arylsulfonic Acids', MS='Organic sulfonic acid derivatives which contain an aromatic hydrocarbon radical.'),
  'D017741': NtO(MH='Survivors', MS='Persons who have experienced a prolonged survival after serious disease or who continue to live with a usually life-threatening condition as well as family members, significant others, or individuals surviving traumatic life events.'),
  'D017742': NtO(MH='Dental Care for Chronically Ill', MS='Dental care for patients with chronic diseases. These diseases include chronic cardiovascular, endocrinologic, hematologic, immunologic, neoplastic, and renal diseases. The concept does not include dental care for the mentally or physically disabled which is DENTAL CARE FOR DISABLED.'),
  'D017743': NtO(MH='Equipment Reuse', MS='Further or repeated use of equipment, instruments, devices, or materials. It includes additional use regardless of the original intent of the producer as to disposability or durability. It does not include the repeated use of fluids or solutions.'),
  'D017744': NtO(MH='Medical Waste Disposal', MS='Management, removal, and elimination of biologic, infectious, pathologic, and dental waste. The concept includes blood, mucus, tissue removed at surgery or autopsy, soiled surgical dressings, and other materials requiring special control and handling. Disposal may take place where the waste is generated or elsewhere.'),
  'D017745': NtO(MH='Incineration', MS='High temperature destruction of waste by burning with subsequent reduction to ashes or conversion to an inert mass.'),
  'D017746': NtO(MH='Joint Capsule', MS='The sac enclosing a joint. It is composed of an outer fibrous articular capsule and an inner SYNOVIAL MEMBRANE.'),
  'D017747': NtO(MH='Seafood', MS='Marine fish and shellfish used as food or suitable for food. (Webster, 3d ed) SHELLFISH and FISH PRODUCTS are more specific types of SEAFOOD.'),
  'D017748': NtO(MH='Time Management', MS='Planning and control of time to improve efficiency and effectiveness.'),
  'D017749': NtO(MH='Total Quality Management', MS='The application of industrial management practice to systematically maintain and improve organization-wide performance. Effectiveness and success are determined and assessed by quantitative quality measures.'),
  'D017750': NtO(MH='Animal Technicians', MS='Assistants to a veterinarian, biological or biomedical researcher, or other scientist who are engaged in the care and management of animals, and who are trained in basic principles of animal life processes and routine laboratory and animal health care procedures. (Facts on File Dictionary of Health Care Management, 1988)'),
  'D017751': NtO(MH='Safety Management', MS='The development of systems to prevent accidents, injuries, and other adverse occurrences in an institutional setting. The concept includes prevention or reduction of adverse events or incidents involving employees, patients, or facilities. Examples include plans to reduce injuries from falls or plans for fire safety to promote a safe institutional environment.'),
  'D017752': NtO(MH='Greenhouse Effect', MS='The effect of GREENHOUSE GASES and the resulting increase in world temperatures. The predicted health effects of such long-term climatic change include increased incidence of respiratory, water-borne, and vector-borne diseases.'),
  'D017753': NtO(MH='Ecosystem', MS='A functional system which includes the organisms of a natural community together with their environment. (McGraw Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D017754': NtO(MH='Group Purchasing', MS='A shared service which combines the purchasing power of individual organizations or facilities in order to obtain lower prices for equipment and supplies. (From Health Care Terms, 2nd ed)'),
  'D017755': NtO(MH='Prosthesis Fitting', MS="The fitting and adjusting of artificial parts of the body. (From Stedman's, 26th ed)"),
  'D017756': NtO(MH='Aquaculture', MS='Cultivation of natural faunal resources of water. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D017757': NtO(MH='Infectious Disease Transmission, Professional-to-Patient', MS='The transmission of infectious disease or pathogens from health professional or health care worker to patients. It includes transmission via direct or indirect exposure to bacterial, fungal, parasitic, or viral agents.'),
  'D017758': NtO(MH='Infectious Disease Transmission, Patient-to-Professional', MS='The transmission of infectious disease or pathogens from patients to health professionals or health care workers. It includes transmission via direct or indirect exposure to bacterial, fungal, parasitic, or viral agents.'),
  'D017759': NtO(MH='Fractures, Malunited', MS='Union of the fragments of a fractured bone in a faulty or abnormal position. If two bones parallel to one another unite by osseous tissue, the result is a crossunion. (From Manual of Orthopaedic Terminology, 4th ed)'),
  'D017760': NtO(MH='Bone Malalignment', MS='Displacement of bones out of line in relation to joints. It may be congenital or traumatic in origin.'),
  'D017762': NtO(MH='Nadroparin', MS='A heparin fraction with a mean molecular weight of 4500 daltons. It is isolated from porcine mucosal heparin and used as an antithrombotic agent. (From Merck Index, 11th ed)'),
  'D017763': NtO(MH='Autonomic Denervation', MS='The removal or interruption of some part of the autonomic nervous system for therapeutic or research purposes.'),
  'D017764': NtO(MH='Parasympathectomy', MS='The removal or interruption of some part of the parasympathetic nervous system for therapeutic or research purposes.'),
  'D017766': NtO(MH='Acid Anhydride Hydrolases', MS='A group of enzymes that catalyze the hydrolysis of diphosphate bonds in compounds such as nucleoside di- and tri-phosphates, and sulfonyl-containing anhydrides such as adenylylsulfate. (Enzyme Nomenclature, 1992) EC 3.6.'),
  'D017767': NtO(MH='Holocaust', MS='A massive slaughter, especially the systematic mass extermination of European Jews in Nazi concentration camps prior to and during World War II.'),
  'D017768': NtO(MH='Skin, Artificial', MS='Synthetic material used for the treatment of burns and other conditions involving large-scale loss of skin. It often consists of an outer (epidermal) layer of silicone and an inner (dermal) layer of collagen and chondroitin 6-sulfate. The dermal layer elicits new growth and vascular invasion and the outer layer is later removed and replaced by a graft.'),
  'D017769': NtO(MH='Ulnar Nerve Compression Syndromes', MS="Ulnar neuropathies caused by mechanical compression of the nerve at any location from its origin at the BRACHIAL PLEXUS to its terminations in the hand. Common sites of compression include the retroepicondylar groove, cubital tunnel at the elbow (CUBITAL TUNNEL SYNDROME), and Guyon's canal at the wrist. Clinical features depend on the site of injury, but may include weakness or paralysis of wrist flexion, finger flexion, and ulnar innervated intrinsic hand muscles, and impaired sensation over the ulnar aspect of the hand, fifth finger, and ulnar half of the ring finger. (Joynt, Clinical Neurology, 1995, Ch51, p43)"),
  'D017770': NtO(MH='Lifting', MS='Moving or bringing something from a lower level to a higher one. The concept encompasses biomechanic stresses resulting from work done in transferring objects from one plane to another as well as the effects of varying techniques of patient handling and transfer.'),
  'D017772': NtO(MH='Amyloid Neuropathies', MS='Disorders of the peripheral nervous system associated with the deposition of AMYLOID in nerve tissue. Familial, primary (nonfamilial), and secondary forms have been described. Some familial subtypes demonstrate an autosomal dominant pattern of inheritance. Clinical manifestations include sensory loss, mild weakness, autonomic dysfunction, and CARPAL TUNNEL SYNDROME. (Adams et al., Principles of Neurology, 6th ed, p1349)'),
  'D017773': NtO(MH='Pelvic Floor', MS='Soft tissue formed mainly by the pelvic diaphragm, which is composed of the two levator ani and two coccygeus muscles. The pelvic diaphragm lies just below the pelvic aperture (outlet) and separates the pelvic cavity from the PERINEUM. It extends between the PUBIC BONE anteriorly and the COCCYX posteriorly.'),
  'D017774': NtO(MH='Long-Term Potentiation', MS='A persistent increase in synaptic efficacy, usually induced by appropriate activation of the same synapses. The phenomenological properties of long-term potentiation suggest that it may be a cellular mechanism of learning and memory.'),
  'D017776': NtO(MH='Autonomic Pathways', MS='Nerves and plexuses of the autonomic nervous system. The central nervous system structures which regulate the autonomic nervous system are not included.'),
  'D017777': NtO(MH='Parasympathetic Fibers, Postganglionic', MS='Nerve fibers which project from parasympathetic ganglia to synapses on target organs. Parasympathetic postganglionic fibers use acetylcholine as transmitter. They may also release peptide cotransmitters.'),
  'D017778': NtO(MH='Vaccines, Combined', MS='Two or more vaccines in a single dosage form.'),
  'D017779': NtO(MH='Sympathetic Fibers, Postganglionic', MS='Nerve fibers which project from sympathetic ganglia to synapses on target organs. Sympathetic postganglionic fibers use norepinephrine as transmitter, except for those innervating eccrine sweat glands (and possibly some blood vessels) which use acetylcholine. They may also release peptide cotransmitters.'),
  'D017780': NtO(MH='Malaria Vaccines', MS='Vaccines made from antigens arising from any of the four strains of Plasmodium which cause malaria in humans, or from P. berghei which causes malaria in rodents.'),
  'D017781': NtO(MH='Genes, Immediate-Early', MS='Genes that show rapid and transient expression in the absence of de novo protein synthesis. The term was originally used exclusively for viral genes where immediate-early referred to transcription immediately following virus integration into the host cell. It is also used to describe cellular genes which are expressed immediately after resting cells are stimulated by extracellular signals such as growth factors and neurotransmitters.'),
  'D017783': NtO(MH='Superior Cervical Ganglion', MS='The largest and uppermost of the paravertebral sympathetic ganglia.'),
  'D017784': NtO(MH='Milk Banks', MS='Centers for acquiring, storing, and distributing human milk.'),
  'D017785': NtO(MH='Photons', MS="Discrete concentrations of energy, apparently massless elementary particles, that move at the speed of light. They are the unit or quantum of electromagnetic radiation. Photons are emitted when electrons move from one energy state to another. (From Hawley's Condensed Chemical Dictionary, 11th ed)"),
  'D017788': NtO(MH='Carlavirus', MS='A genus of RNA plant viruses in the family FLEXIVIRIDAE, containing slightly flexuous filaments, often transmitted by aphids in a non-persistent manner. Carnation latent virus is the type species.'),
  'D017789': NtO(MH='Granuloma, Pyogenic', MS='A disorder of the skin, the oral mucosa, and the gingiva, that usually presents as a solitary polypoid capillary hemangioma often resulting from trauma. It is manifested as an inflammatory response with similar characteristics to those of a granuloma.'),
  'D017790': NtO(MH='Closterovirus', MS='A genus of plant viruses in the family CLOSTEROVIRIDAE containing highly flexuous filaments. Some members are important pathogens of crop plants. Natural vectors include APHIDS, whiteflies, and mealybugs. The type species is Beet yellows virus.'),
  'D017791': NtO(MH='Geminiviridae', MS='A family of plant viruses where the VIRION possesses an unusual morphology consisting of a pair of isometric particles. Transmission occurs via leafhoppers or whitefly. Some viruses cause economically important diseases in cultivated plants. There are four genera: Mastrevirus, Curtovirus, Topocuvirus, and BEGOMOVIRUS.'),
  'D017792': NtO(MH='Surgical Stapling', MS='A technique of closing incisions and wounds, or of joining and connecting tissues, in which staples are used as sutures.'),
  'D017793': NtO(MH='Luteovirus', MS='A genus of plant viruses that infects both monocotyledonous and dicotyledonous plants. Its organisms are persistently transmitted by aphids, and weeds may provide reservoirs of infection.'),
  'D017794': NtO(MH='Alfalfa mosaic virus', MS='The type species of the genus ALFAMOVIRUS that is non-persistently transmitted by aphids.'),
  'D017795': NtO(MH='Bromovirus', MS='A genus of tripartite plant viruses in the family BROMOVIRIDAE. Transmission is by beetles. Brome mosaic virus is the type species.'),
  'D017796': NtO(MH='Caulimovirus', MS='A genus of PLANT VIRUSES, in the family CAULIMOVIRIDAE, that are transmitted by APHIDS in a semipersistent manner. Aphid-borne transmission of some caulimoviruses requires certain virus-coded proteins termed transmission factors.'),
  'D017797': NtO(MH='Prosthesis Coloring', MS='Coloring, shading, or tinting of prosthetic components, devices, and materials.'),
  'D017798': NtO(MH='Comovirus', MS='A genus of plant viruses of the family COMOVIRIDAE in which the bipartite genome is encapsidated in separate icosahedral particles. Mosaic and mottle symptoms are characteristic, and transmission is exclusively by leaf-feeding beetles. Cowpea mosaic virus is the type species.'),
  'D017799': NtO(MH='Cucumovirus', MS='A genus of plant viruses of the family BROMOVIRIDAE, which infect cucurbits and solanaceous plants. Transmission occurs via aphids in a non-persistent manner, and also via seeds. The type species Cucumber mosaic virus, a CUCUMOVIRUS, should not be confused with Cucumber green mottle mosaic virus, a TOBAMOVIRUS.'),
  'D017800': NtO(MH='Potyvirus', MS='A large genus of plant viruses of the family POTYVIRIDAE which infect mainly plants of the Solanaceae. Transmission is primarily by aphids in a non-persistent manner. The type species is potato virus Y.'),
  'D017801': NtO(MH='Plum Pox Virus', MS='A species of the genus POTYVIRUS that affects many species of Prunus. It is transmitted by aphids and by infected rootstocks.'),
  'D017805': NtO(MH='Widowhood', MS='The state of having lost a marital partner by death.'),
  'D017806': NtO(MH='Tymovirus', MS='A genus of plant viruses, in the family TYMOVIRIDAE, possessing a narrow host range that includes CRUCIFERAE. Transmission occurs by BEETLES and mechanical inoculation.'),
  'D017809': NtO(MH='Fatal Outcome', MS='Death resulting from the presence of a disease in an individual, as shown by a single case report or a limited number of patients. This should be differentiated from DEATH, the physiological cessation of life and from MORTALITY, an epidemiological or statistical concept.'),
  'D017812': NtO(MH='Colubridae', MS='The largest family of snakes comprising five subfamilies: Colubrinae, Natricinae, Homalopsinae, Lycodontinae, and Xenodontinae. They show a great diversity of eating habits, from eating almost anything to having a specialized diet. They can be oviparous, ovoviviparous, or viviparous. The majority of North American snakes are colubrines. Among the colubrids are king snakes, water snakes, and garter snakes. Some genera are poisonous. (Goin, Goin, and Zug, Introduction to Herpetology, 3d ed, pp321-29)'),
  'D017815': NtO(MH='Elapidae', MS='A family of extremely venomous snakes which include coral snakes, cobras and sea snakes. They are widely distributed, occurring in the Southern United States, South America, Africa, Southern Asia, Australia, and the Pacific Islands. Like viperids (VIPERIDAE), they have fangs in the front part of the upper jaw which discharge venom that is neurotoxic.'),
  'D017817': NtO(MH='Nasal Lavage Fluid', MS='Fluid obtained by THERAPEUTIC IRRIGATION or washout of the nasal cavity and NASAL MUCOSA. The resulting fluid is used in cytologic and immunologic assays of the nasal mucosa such as with the NASAL PROVOCATION TEST in the diagnosis of nasal hypersensitivity.'),
  'D017819': NtO(MH='Viperidae', MS='A family of snakes comprising four subfamilies: Azemiopinae (the mountain viper, the sole member of this subfamily), Viperinae (true vipers), Crotalinae (pit vipers) and Causinae. They are widespread throughout the world, being found in the United States, Central and South America, Europe, Asia and Africa. Their venoms act on the blood (hemotoxic) as compared to the venom of elapids which act on the nervous system (neurotoxic). (Goin, Goin, and Zug, Introduction to Herpetology, 3d ed, pp333-36)'),
  'D017821': NtO(MH='Boidae', MS="A family of snakes comprising the boas, anacondas, and pythons. They occupy a variety of habitats through the tropics and subtropics and are arboreal, aquatic or fossorial (burrowing). Some are oviparous, others ovoviviparous. Contrary to popular opinion, they do not crush the bones of their victims: their coils exert enough pressure to stop a prey's breathing, thus suffocating it. There are five subfamilies: Boinae, Bolyerinae, Erycinae, Pythoninae, and Tropidophiinae. (Goin, Goin, and Zug, Introduction to Herpetology, 3d ed, p315-320)"),
  'D017823': NtO(MH='Furcation Defects', MS='Conditions in which a bifurcation or trifurcation of the molar tooth root becomes denuded as a result of periodontal disease. It may be followed by tooth mobility, temperature sensitivity, pain, and alveolar bone resorption.'),
  'D017824': NtO(MH='Bone Cysts, Aneurysmal', MS='Fibrous blood-filled cyst in the bone. Although benign it can be destructive causing deformity and fractures.'),
  'D017825': NtO(MH='Canavan Disease', MS='A rare neurodegenerative condition of infancy or childhood characterized by white matter vacuolization and demeylination that gives rise to a spongy appearance. Aspartoacylase deficiency leads to an accumulation of N-acetylaspartate in astrocytes. Inheritance may be autosomal recessive or the illness may occur sporadically. This illness occurs more frequently in individuals of Ashkenazic Jewish descent. The neonatal form features the onset of hypotonia and lethargy at birth, rapidly progressing to coma and death. The infantile form features developmental delay, DYSKINESIAS, hypotonia, spasticity, blindness, and megalencephaly. The juvenile form is characterized by ATAXIA; OPTIC ATROPHY; and DEMENTIA. (From Adams et al., Principles of Neurology, 6th ed, p944; Am J Med Genet 1988 Feb;29(2):463-71)'),
  'D017827': NtO(MH='Machado-Joseph Disease', MS='A dominantly-inherited ATAXIA first described in people of Azorean and Portuguese descent, and subsequently identified in Brazil, Japan, China, and Australia. This disorder is classified as one of the SPINOCEREBELLAR ATAXIAS (Type 3) and has been associated with a mutation of the MJD1 gene on chromosome 14. Clinical features include progressive ataxia, DYSARTHRIA, postural instability, nystagmus, eyelid retraction, and facial FASCICULATIONS. DYSTONIA is prominent in younger patients (referred to as Type I Machado-Joseph Disease). Type II features ataxia and ocular signs; Type III features MUSCULAR ATROPHY and a sensorimotor neuropathy; and Type IV features extrapyramidal signs combined with a sensorimotor neuropathy. (From Clin Neurosci 1995;3(1):17-22; Ann Neurol 1998 Mar;43(3):288-96)'),
  'D017828': NtO(MH='Rifabutin', MS='A broad-spectrum antibiotic that is being used as prophylaxis against disseminated Mycobacterium avium complex infection in HIV-positive patients.'),
  'D017829': NtO(MH='Granisetron', MS='A serotonin receptor (5HT-3 selective) antagonist that has been used as an antiemetic for cancer chemotherapy patients.'),
  'D017830': NtO(MH='Octoxynol', MS='Nonionic surfactant mixtures varying in the number of repeating ethoxy (oxy-1,2-ethanediyl) groups. They are used as detergents, emulsifiers, wetting agents, defoaming agents, etc. Octoxynol-9, the compound with 9 repeating ethoxy groups, is a spermatocide.'),
  'D017831': NtO(MH='Diagnosis, Dual (Psychiatry)', MS='The co-existence of a substance abuse disorder with a psychiatric disorder. The diagnostic principle is based on the fact that it has been found often that chemically dependent patients also have psychiatric problems of various degrees of severity.'),
  'D017833': NtO(MH='Visceral Afferents', MS='The sensory fibers innervating the viscera.'),
  'D017835': NtO(MH='Nedocromil', MS='A pyranoquinolone derivative that inhibits activation of inflammatory cells which are associated with ASTHMA, including eosinophils, neutrophils, macrophages, mast cells, monocytes, and platelets.'),
  'D017836': NtO(MH='Agkistrodon', MS='A genus of venomous snakes of the subfamily Crotalinae. Twelve species of this genus are found in North and Central America and Asia. Agkistrodon contortrix is the copperhead, A. piscivorus, the cottonmouth. The former is named for its russet or orange-brown color, the latter for the white interior of its mouth. (Goin, Goin, and Zug, Introduction to Herpetology, 3d ed, p336; Moore, Poisonous Snakes of the World, 1980, p75)'),
  'D017837': NtO(MH='Bothrops', MS='A genus of poisonous snakes of the VIPERIDAE family. About 50 species are known and all are found in tropical America and southern South America. Bothrops atrox is the fer-de-lance and B. jararaca is the jararaca. (Goin, Goin, and Zug, Introduction to Herpetology, 3d ed, p336)'),
  'D017838': NtO(MH='Bungarus', MS='A genus of poisonous snakes of the subfamily Elapinae of the family ELAPIDAE. They comprise the kraits. Twelve species are recognized and all inhabit southeast Asia. They are considered extremely dangerous. (Moore: Poisonous Snakes of the World, 1980, p120)'),
  'D017839': NtO(MH='Crotalus', MS='A genus of snakes of the subfamily Crotalinae, having a hollowed out pit in the maxillary bone between the eye and nostril. Most of the recognized species are found in the southwestern United States and northern Mexico. Several species are found as far north as Canada and east of the Mississippi, including southern Appalachia. They are named for the jointed rattle (Greek krotalon) at the tip of their tail. (Goin, Goin, and Zug: Introduction to Herpetology, 3d ed; Moore: Poisonous Snakes of the World, 1980, p335).'),
  'D017840': NtO(MH="Russell's Viper", MS='A genus of snakes of the family VIPERIDAE. It is distributed in West Pakistan, most of India, Burma, Ceylon, Thailand, southeast China, Taiwan, and a few islands of Indonesia. It hisses loudly when disturbed and strikes with great force and speed. Very prolific, it gives birth to 20-60 young. This viper is the leading cause of snakebite in India and Burma. (Moore: Poisonous Snakes of the World, 1980, p127)'),
  'D017841': NtO(MH='Trimeresurus', MS='A genus of snakes of the family VIPERIDAE. About 30 species are currently recognized, found in southeast Asia and adjacent island chains. The Okinawa habu frequently enters dwellings in search of rats and mice; the Chinese habu is often found in suburban and agricultural areas. They are quite irritable. (Moore: Poisonous Snakes of the World, 1980, p136)'),
  'D017843': NtO(MH='Ligamentum Flavum', MS='The paired bands of yellow elastic tissue that connect adjoining laminae of the vertebrae. With the laminae, it forms the posterior wall of the spinal canal and helps hold the body erect.'),
  'D017844': NtO(MH='Lateral Ligament, Ankle', MS='LATERAL LIGAMENTS of the ANKLE JOINT. It includes inferior tibiofibular ligaments.'),
  'D017846': NtO(MH='Longitudinal Ligaments', MS='Two extensive fibrous bands running the length of the vertebral column. The anterior longitudinal ligament (ligamentum longitudinale anterius; lacertus medius) interconnects the anterior surfaces of the vertebral bodies; the posterior longitudinal ligament (ligamentum longitudinale posterius) interconnects the posterior surfaces. The commonest clinical consideration is OSSIFICATION OF POSTERIOR LONGITUDINAL LIGAMENT. (From Stedman, 25th ed)'),
  'D017847': NtO(MH='Patellar Ligament', MS='A band of fibrous tissue that attaches the apex of the PATELLA to the lower part of the tubercle of the TIBIA. The ligament is actually the caudal continuation of the common tendon of the QUADRICEPS FEMORIS. The patella is embedded in that tendon. As such, the patellar ligament can be thought of as connecting the quadriceps femoris tendon to the tibia, and therefore it is sometimes called the patellar tendon.'),
  'D017848': NtO(MH='Blood-Borne Pathogens', MS='Infectious organisms, including pathogenic bacteria, viruses, and fungi, that are present in the BLOOD.'),
  'D017849': NtO(MH='Animals, Inbred Strains', MS='Animals produced by the mating of progeny over multiple generations. The resultant strain of animals is virtually identical genotypically. Highly inbred animal lines allow the study of certain traits in a relatively pure form. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D017850': NtO(MH='Economics, Pharmaceutical', MS='Economic aspects of the fields of pharmacy and pharmacology as they apply to the development and study of medical economics in rational drug therapy and the impact of pharmaceuticals on the cost of medical care. Pharmaceutical economics also includes the economic considerations of the pharmaceutical care delivery system and in drug prescribing, particularly of cost-benefit values. (From J Res Pharm Econ 1989;1(1); PharmacoEcon 1992;1(1))'),
  'D017851': NtO(MH='Phosphotransferases (Carboxyl Group Acceptor)', MS='A class of enzymes that transfers phosphate groups and has a carboxyl group as an acceptor. EC 2.7.2.'),
  'D017852': NtO(MH='Phosphotransferases (Nitrogenous Group Acceptor)', MS='A group of enzymes that catalyzes the transfer of a phosphate group onto a nitrogenous group acceptor. EC 2.7.3.'),
  'D017853': NtO(MH='Phosphotransferases (Alcohol Group Acceptor)', MS='A group of enzymes that transfers a phosphate group onto an alcohol group acceptor. EC 2.7.1.'),
  'D017854': NtO(MH='Diphosphotransferases', MS='A class of phosphotransferases that catalyzes the transfer of diphosphate-containing groups. EC 2.7.6.'),
  'D017855': NtO(MH='Transferases (Other Substituted Phosphate Groups)', MS='A class of enzymes that transfers substituted phosphate groups. EC 2.7.8.'),
  'D017856': NtO(MH='Phosphotransferases (Phosphate Group Acceptor)', MS='A group of enzymes that catalyzes the transfer of a phosphate group onto a phosphate group acceptor. EC 2.7.4.'),
  'D017858': NtO(MH='Salmoniformes', MS='An order of fish comprising salmons, trouts, whitefish, graylings, and other families. They are both marine and freshwater fish, found in all oceans and are quite numerous in the Northern Hemisphere. (From Nelson: Fishes of the World)'),
  'D017859': NtO(MH='Esocidae', MS='A family of freshwater fish of the order ESOCIFORMES, comprising the pikes, inhabiting the waters of the Northern Hemisphere. There is one genus, Esox, with five species: northern pike, grass pickerel, chain pickerel, muskellunge, and Amur pike.'),
  'D017860': NtO(MH='Nepovirus', MS='A genus of polyhedral plant viruses of the family COMOVIRIDAE causing ringspots and spotting on leaves or sometimes symptomless infection. Transmission occurs by seeds, soil nematodes, or experimentally by mechanical inoculation. Tobacco ringspot virus is the type species.'),
  'D017861': NtO(MH='Porphyromonas', MS='A genus of gram-negative, anaerobic, nonsporeforming, nonmotile rods or coccobacilli. Organisms in this genus had originally been classified as members of the BACTEROIDES genus but overwhelming biochemical and chemical findings indicated the need to separate them from other Bacteroides species, and hence, this new genus was created.'),
  'D017862': NtO(MH='Phycodnaviridae', MS='A family of DNA plant viruses that infect eukaryotic algae.'),
  'D017863': NtO(MH='Potexvirus', MS='A genus of plant viruses in the family FLEXIVIRIDAE, that cause mosaic and ringspot symptoms. Transmission occurs mechanically. Potato virus X is the type species.'),
  'D017864': NtO(MH='Tombusvirus', MS='A genus of plant viruses that infects ANGIOSPERMS. Transmission occurs mechanically and through soil, with one species transmitted via a fungal vector. The type species is Tomato bushy stunt virus.'),
  'D017865': NtO(MH='Tospovirus', MS='A genus of plant viruses in the family BUNYAVIRIDAE. Tomato spotted wilt virus is the type species. Transmission occurs by at least nine species of thrips.'),
  'D017866': NtO(MH='Phosphotransferases (Paired Acceptors)', MS='A group of enzymes that transfers two phosphate groups from a donor such as ATP to two different acceptors. (From Enzyme Nomenclature, 1992) EC 2.7.9.'),
  'D017867': NtO(MH='Cyclic Nucleotide-Regulated Protein Kinases', MS='A group of enzymes that catalyzes the phosphorylation of serine or threonine residues of proteins and is dependent on cyclic nucleotides.'),
  'D017868': NtO(MH='Cyclic AMP-Dependent Protein Kinases', MS='A group of enzymes that are dependent on CYCLIC AMP and catalyze the phosphorylation of SERINE or THREONINE residues on proteins. Included under this category are two cyclic-AMP-dependent protein kinase subtypes, each of which is defined by its subunit composition.'),
  'D017869': NtO(MH='Cyclic GMP-Dependent Protein Kinases', MS='A group of cyclic GMP-dependent enzymes that catalyze the phosphorylation of SERINE or THREONINE residues of proteins.'),
  'D017871': NtO(MH='Calcium-Calmodulin-Dependent Protein Kinases', MS='A CALMODULIN-dependent enzyme that catalyzes the phosphorylation of proteins. This enzyme is also sometimes dependent on CALCIUM. A wide range of proteins can act as acceptor, including VIMENTIN; SYNAPSINS; GLYCOGEN SYNTHASE; MYOSIN LIGHT CHAINS; and the MICROTUBULE-ASSOCIATED PROTEINS. (From Enzyme Nomenclature, 1992, p277)'),
  'D017874': NtO(MH='Immediate-Early Proteins', MS='Proteins that are coded by immediate-early genes, in the absence of de novo protein synthesis. The term was originally used exclusively for viral regulatory proteins that were synthesized just after viral integration into the host cell. It is also used to describe cellular proteins which are synthesized immediately after the resting cell is stimulated by extracellular signals.'),
  'D017875': NtO(MH='Phosphotransferases (Phosphomutases)', MS='A group of enzymes that catalyze an intramolecular transfer of a phosphate group. It has been shown in some cases that the enzyme has a functional phosphate group, which can act as the donor. These were previously listed under PHOSPHOTRANSFERASES (EC 2.7.-). (From Enzyme Nomenclature, 1992) EC 5.4.2.'),
  'D017877': NtO(MH='Umbridae', MS='A family of ESOCIFORMES comprising the mudminnows. There are three genera: Dallia, Novumbra, and Umbra, with much disagreement about their taxonomic interrelations. The Umbridae are freshwater fish inhabiting parts of the Northern Hemisphere.'),
  'D017878': NtO(MH="4,4'-Diisothiocyanostilbene-2,2'-Disulfonic Acid", MS='An inhibitor of anion conductance including band 3-mediated anion transport.'),
  'D017879': NtO(MH='Isothiocyanates', MS='Organic compounds with the general formula R-NCS.'),
  'D017880': NtO(MH='Limb Deformities, Congenital', MS='Congenital structural deformities of the upper and lower extremities collectively or unspecified.'),
  'D017881': NtO(MH='Kingella', MS='A genus of gram-negative, aerobic, rod-shaped bacteria whose organisms are part of the normal flora of the mucous membranes of the upper respiratory tract. Some species are pathogenic for man.'),
  'D017882': NtO(MH='Kingella kingae', MS="A species of gram-negative, aerobic, rod-shaped bacteria which is distinguished from other members of the genus KINGELLA by its beta hemolysis. It occurs normally in human mucous membranes of the upper respiratory tract, but can cause septic arthritis and endocarditis. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D017885': NtO(MH='Collateral Ligaments', MS='A number of ligaments on either side of, and serving as a radius of movement of, a joint having a hingelike movement. They occur at the elbow, knee, wrist, metacarpo- and metatarsophalangeal, proximal interphalangeal, and distal interphalangeal joints of the hands and feet. (Stedman, 25th ed)'),
  'D017886': NtO(MH='Durapatite', MS='The mineral component of bones and teeth; it has been used therapeutically as a prosthetic aid and in the prevention and treatment of osteoporosis.'),
  'D017887': NtO(MH='Ossification of Posterior Longitudinal Ligament', MS='A calcification of the posterior longitudinal ligament of the spinal column, usually at the level of the cervical spine. It is often associated with anterior ankylosing hyperostosis.'),
  'D017888': NtO(MH='Medial Collateral Ligament, Knee', MS='The ligament that travels from the medial epicondyle of the FEMUR to the medial margin and medial surface of the TIBIA. The medial meniscus is attached to its deep surface.'),
  'D017889': NtO(MH='Exfoliation Syndrome', MS='The deposition of flaky, translucent fibrillar material most conspicuous on the anterior lens capsule and pupillary margin but also in both surfaces of the iris, the zonules, trabecular meshwork, ciliary body, corneal endothelium, and orbital blood vessels. It sometimes forms a membrane on the anterior iris surface. Exfoliation refers to the shedding of pigment by the iris. (Newell, Ophthalmology, 7th ed, p380)'),
  'D017890': NtO(MH='Splenosis', MS='The spontaneous transplantation of splenic tissue to unusual sites after open splenic trauma, e.g., after automobile accidents, gunshot or stab wounds. The splenic pulp implants appear as red-blue nodules on the peritoneum, omentum, and mesentery, morphologically similar to multifocal pelvic endometriosis. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D017891': NtO(MH='Pharmacoepidemiology', MS='The science concerned with the benefit and risk of drugs used in populations and the analysis of the outcomes of drug therapies. Pharmacoepidemiologic data come from both clinical trials and epidemiological studies with emphasis on methods for the detection and evaluation of drug-related adverse effects, assessment of risk vs benefit ratios in drug therapy, patterns of drug utilization, the cost-effectiveness of specific drugs, methodology of postmarketing surveillance, and the relation between pharmacoepidemiology and the formulation and interpretation of regulatory guidelines. (Pharmacoepidemiol Drug Saf 1992;1(1); J Pharmacoepidemiol 1990;1(1))'),
  'D017892': NtO(MH='Osmium Compounds', MS='Inorganic compounds that contain osmium as an integral part of the molecule.'),
  'D017893': NtO(MH='Photopheresis', MS='A process in which peripheral blood is exposed in an extracorporeal flow system to photoactivated 8-methoxypsoralen (METHOXSALEN) and ultraviolet light - a procedure known as PUVA THERAPY. Photopheresis is at present a standard therapy for advanced cutaneous T-cell lymphoma; it shows promise in the treatment of autoimmune diseases.'),
  'D017895': NtO(MH='Manganese Compounds', MS='Inorganic chemicals that contain manganese as an integral part of the molecule.'),
  'D017898': NtO(MH='Tobamovirus', MS='A genus of plant viruses in which the virion is a rigid filament. Transmission is by mechanical inoculation or seed. The type species is TOBACCO MOSAIC VIRUS.'),
  'D017899': NtO(MH='Caudovirales', MS='An order comprising three families of tailed bacteriophages: MYOVIRIDAE; PODOVIRIDAE; and SIPHOVIRIDAE.'),
  'D017900': NtO(MH='Myoviridae', MS='A family of BACTERIOPHAGES and ARCHAEAL VIRUSES which are characterized by complex contractile tails.'),
  'D017901': NtO(MH='Bacteriophage P2', MS='A species of temperate bacteriophage in the genus P2-like viruses, family MYOVIRIDAE, which infects E. coli. It consists of linear double-stranded DNA with 19-base sticky ends.'),
  'D017902': NtO(MH='Podoviridae', MS='A family of bacteriophages which are characterized by short, non-contractile tails.'),
  'D017903': NtO(MH='Bacteriophage T3', MS='Bacteriophage in the genus T7-like phages, of the family PODOVIRIDAE, which is very closely related to BACTERIOPHAGE T7.'),
  'D017904': NtO(MH='Mobiluncus', MS='A genus of gram-negative, anaerobic, rod-shaped bacteria. Its organisms are found in the human vagina, particularly in association with Gardnerella vaginalis in cases of bacterial vaginosis.'),
  'D017905': NtO(MH='Phosphites', MS="Inorganic salts or organic esters of phosphorous acid that contain the (3-)PO3 radical. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D017906': NtO(MH='Siphoviridae', MS='A family of BACTERIOPHAGES and ARCHAEAL VIRUSES which are characterized by long, non-contractile tails.'),
  'D017907': NtO(MH='Leviviridae', MS='A family of bacteriophages that infects enterobacteria, CAULOBACTER, and PSEUDOMONAS. The genome consists of linear, positive-sense single-stranded RNA.'),
  'D017908': NtO(MH='Allolevivirus', MS='A bacteriophage genus of the family LEVIVIRIDAE, whose viruses contain the longer version of the genome and have no separate cell lysis gene.'),
  'D017909': NtO(MH='Levivirus', MS='A bacteriophage genus of the family LEVIVIRIDAE, whose viruses contain the short version of the genome and have a separate gene for cell lysis.'),
  'D017910': NtO(MH='Cystoviridae', MS='A family of bacteriophages containing one genus (Cystovirus) with one member (BACTERIOPHAGE PHI 6).'),
  'D017911': NtO(MH='Inoviridae', MS='A family of rod-shaped or filamentous bacteriophages consisting of single-stranded DNA. There are two genera: INOVIRUS and PLECTROVIRUS.'),
  'D017912': NtO(MH='Inovirus', MS='A genus of filamentous bacteriophages of the family INOVIRIDAE. Organisms of this genus infect enterobacteria, PSEUDOMONAS; VIBRIO; and XANTHOMONAS.'),
  'D017913': NtO(MH='Plectrovirus', MS='A genus of bacteriophages of the family INOVIRIDAE. Organisms of this genus infect Acholeplasma and Spiroplasma.'),
  'D017914': NtO(MH='Microviridae', MS='A large family of lytic bacteriophages infecting enterobacteria; SPIROPLASMA; BDELLOVIBRIO; and CHLAMYDIA. It contains four genera: MICROVIRUS; Spiromicrovirus; Bdellomicrovirus; and Chlamydiamicrovirus.'),
  'D017915': NtO(MH='Microvirus', MS='A genus of bacteriophages of the family MICROVIRIDAE. The genome consists of isometric single-stranded DNA.'),
  'D017916': NtO(MH='Tectiviridae', MS='A family of lipid-containing bacteriophages with double capsids which infect both gram-negative and gram-positive bacteria. It has one genus, Tectivirus.'),
  'D017917': NtO(MH='Avihepadnavirus', MS='A genus of HEPADNAVIRIDAE infecting birds but rarely causing clinical problems. Transmission is predominantly vertical. HEPATITIS B VIRUS, DUCK is the type species.'),
  'D017918': NtO(MH='Orthohepadnavirus', MS='A genus of HEPADNAVIRIDAE causing HEPATITIS in humans, woodchucks (HEPATITIS B VIRUS, WOODCHUCK) and ground squirrels. Hepatitis B virus is the type species.'),
  'D017919': NtO(MH='Hepatitis B Virus, Woodchuck', MS='An ORTHOHEPADNAVIRUS causing chronic liver disease and hepatocellular carcinoma in woodchucks. It closely resembles the human hepatitis B virus.'),
  'D017920': NtO(MH='Antiporters', MS='Membrane transporters that co-transport two or more dissimilar molecules in the opposite direction across a membrane. Usually the transport of one ion or molecule is against its electrochemical gradient and is "powered" by the movement of another ion or molecule with its electrochemical gradient.'),
  'D017921': NtO(MH='Mastadenovirus', MS='A genus of ADENOVIRIDAE that infects MAMMALS including humans and causes a wide range of diseases. The type species is Human adenovirus C (see ADENOVIRUSES, HUMAN).'),
  'D017923': NtO(MH='Sodium-Hydrogen Exchangers', MS='A family of plasma membrane exchange glycoprotein antiporters that transport sodium ions and protons across lipid bilayers. They have critical functions in intracellular pH regulation, cell volume regulation, and cellular response to many different hormones and mitogens.'),
  'D017924': NtO(MH='Nucleopolyhedroviruses', MS='Genera of the family BACULOVIRIDAE, characterized by the formation of crystalline, polyhedral occlusion bodies in the host cell nucleus. Nucleopolyhedrovirus is the common name for viruses from the Alphabaculovirus, Deltabaculovirus, or Gammabaculovirus genera.'),
  'D017925': NtO(MH='Circovirus', MS='A genus of the family CIRCOVIRIDAE that infects SWINE; PSITTACINES; and non-psittacine BIRDS. Species include Beak and feather disease virus causing a fatal disease in psittacine birds, and Porcine circovirus causing postweaning multisystemic wasting syndrome in pigs (PORCINE POSTWEANING MULTISYSTEMIC WASTING SYNDROME).'),
  'D017927': NtO(MH='Calicivirus, Feline', MS='A species of the genus VESIVIRUS infecting cats. Transmission occurs via air and mechanical contact.'),
  'D017928': NtO(MH='Vesicular exanthema of swine virus', MS='The type species of the genus VESIVIRUS infecting pigs. The resulting infection is an acute febrile disease which is clinically indistinguishable from FOOT AND MOUTH DISEASE. Transmission is by contaminated food.'),
  'D017929': NtO(MH='Hemorrhagic Disease Virus, Rabbit', MS='A species in the genus LAGOVIRUS which causes hemorrhagic disease, including hemorrhagic septicemia, in rabbits.'),
  'D017930': NtO(MH='Genes, Reporter', MS='Genes whose expression is easily detectable and therefore used to study promoter activity at many positions in a target genome. In recombinant DNA technology, these genes may be attached to a promoter region of interest.'),
  'D017931': NtO(MH='DNA Primers', MS='Short sequences (generally about 10 base pairs) of DNA that are complementary to sequences of messenger RNA and allow reverse transcriptases to start copying the adjacent sequences of mRNA. Primers are used extensively in genetic and molecular biology techniques.'),
  'D017933': NtO(MH='Peripheral Nervous System', MS='The nervous system outside of the brain and spinal cord. The peripheral nervous system has autonomic and somatic divisions. The autonomic nervous system includes the enteric, parasympathetic, and sympathetic subdivisions. The somatic nervous system includes the cranial and spinal nerves and their ganglia and the peripheral sensory receptors.'),
  'D017934': NtO(MH='Coronavirus', MS='A member of CORONAVIRIDAE which causes respiratory or gastrointestinal disease in a variety of vertebrates.'),
  'D017936': NtO(MH='Cytochrome a Group', MS='Cytochromes (electron-transporting proteins) in which the heme prosthetic group is heme a, i.e., the iron chelate of cytoporphyrin IX. (From Enzyme Nomenclature, 1992, p539)'),
  'D017937': NtO(MH='Cytochrome d Group', MS='Cytochromes (electron-transporting proteins) with a tetrapyrrolic chelate of iron as a prosthetic group in which the degree of conjugation of double bonds is less than in porphyrin. (From Enzyme Nomenclature, 1992, p539)'),
  'D017938': NtO(MH='Coronavirus, Bovine', MS='A species of CORONAVIRUS infecting neonatal calves, presenting as acute diarrhea, and frequently leading to death.'),
  'D017939': NtO(MH='Coronavirus, Canine', MS='A species of CORONAVIRUS infecting dogs. Onset of symptoms is usually sudden and includes vomiting, diarrhea, and dehydration.'),
  'D017941': NtO(MH='Coronavirus, Rat', MS='A species of CORONAVIRUS causing pneumonia in newborn rats but a clinically inapparent infection in adults. It is separate but antigenically related to MURINE HEPATITIS VIRUS.'),
  'D017942': NtO(MH='Nitric Acid', MS='Nitric acid (HNO3). A colorless liquid that is used in the manufacture of inorganic and organic nitrates and nitro compounds for fertilizers, dye intermediates, explosives, and many different organic chemicals. Continued exposure to vapor may cause chronic bronchitis; chemical pneumonitis may occur. (From Merck Index, 11th ed)'),
  'D017943': NtO(MH='Cardiovirus', MS='A genus of the family PICORNAVIRIDAE causing encephalitis and myocarditis in rodents. ENCEPHALOMYOCARDITIS VIRUS is the type species.'),
  'D017944': NtO(MH='Columbia SK virus', MS='A strain of ENCEPHALOMYOCARDITIS VIRUS, a species of CARDIOVIRUS, that infects rodents.'),
  'D017945': NtO(MH='Theilovirus', MS="A species of CARDIOVIRUS which contains three strains: Theiler's murine encephalomyelitis virus, Vilyuisk human encephalomyelitis virus, and Rat encephalomyelitis virus."),
  'D017948': NtO(MH='Retinal Rod Photoreceptor Cells', MS='Photosensitive afferent neurons located in the peripheral retina, with their density increases radially away from the FOVEA CENTRALIS. Being much more sensitive to light than the RETINAL CONE CELLS, the rod cells are responsible for twilight vision (at scotopic intensities) as well as peripheral vision, but provide no color discrimination.'),
  'D017949': NtO(MH='Retinal Cone Photoreceptor Cells', MS='Photosensitive afferent neurons located primarily within the FOVEA CENTRALIS of the MACULA LUTEA. There are three major types of cone cells (red, blue, and green) whose photopigments have different spectral sensitivity curves. Retinal cone cells operate in daylight vision (at photopic intensities) providing color recognition and central visual acuity.'),
  'D017950': NtO(MH='Ganglia, Sensory', MS='Clusters of neurons in the somatic peripheral nervous system which contain the cell bodies of sensory nerve axons. Sensory ganglia may also have intrinsic interneurons and non-neuronal supporting cells.'),
  'D017951': NtO(MH='Antigen Presentation', MS='The process by which antigen is presented to lymphocytes in a form they can recognize. This is performed by antigen presenting cells (APCs). Some antigens require processing before they can be recognized. Antigen processing consists of ingestion and partial digestion of the antigen by the APC, followed by presentation of fragments on the cell surface. (From Rosen et al., Dictionary of Immunology, 1989)'),
  'D017952': NtO(MH='Ganglia, Invertebrate', MS='Clusters of neuronal cell bodies in invertebrates. Invertebrate ganglia may also contain neuronal processes and non-neuronal supporting cells. Many invertebrate ganglia are favorable subjects for research because they have small numbers of functional neuronal types which can be identified from one animal to another.'),
  'D017953': NtO(MH='Isocyanates', MS='Organic compounds that contain the -NCO radical.'),
  'D017954': NtO(MH='Africa South of the Sahara', MS='All of Africa except Northern Africa (AFRICA, NORTHERN).'),
  'D017955': NtO(MH='Poliovirus', MS='A species of ENTEROVIRUS which is the causal agent of POLIOMYELITIS in humans. Three serotypes (strains) exist. Transmission is by the fecal-oral route, pharyngeal secretions, or mechanical vector (flies). Vaccines with both inactivated and live attenuated virus have proven effective in immunizing against the infection.'),
  'D017956': NtO(MH='Photoreceptor Cells, Invertebrate', MS='Specialized cells in the invertebrates that detect and transduce light. They are predominantly rhabdomeric with an array of photosensitive microvilli. Illumination depolarizes invertebrate photoreceptors by stimulating Na+ influx across the plasma membrane.'),
  'D017957': NtO(MH='Hepatitis A Virus, Human', MS='A strain of HEPATITIS A VIRUS which causes hepatitis in humans. The virus replicates in hepatocytes and is presumed to reach the intestine via the bile duct. Transmission occurs by the fecal-oral route.'),
  'D017958': NtO(MH='Lentiviruses, Primate', MS='A subgenus of LENTIVIRUS comprising viruses that produce immunodeficiencies in primates, including humans.'),
  'D017960': NtO(MH='Lentiviruses, Bovine', MS='A subgenus of LENTIVIRUS comprising viruses that produce multi-organ disease with long incubation periods in cattle.'),
  'D017961': NtO(MH='Lentiviruses, Equine', MS='A subgenus of LENTIVIRUS comprising viruses that produce multi-organ disease with long incubation periods in horses.'),
  'D017962': NtO(MH='alpha-Linolenic Acid', MS='A fatty acid that is found in plants and involved in the formation of prostaglandins.'),
  'D017963': NtO(MH='Azithromycin', MS='A semi-synthetic macrolide antibiotic structurally related to ERYTHROMYCIN. It has been used in the treatment of Mycobacterium avium intracellulare infections, toxoplasmosis, and cryptosporidiosis.'),
  'D017964': NtO(MH='Itraconazole', MS='A triazole antifungal agent that inhibits cytochrome P-450-dependent enzymes required for ERGOSTEROL synthesis.'),
  'D017965': NtO(MH='gamma-Linolenic Acid', MS='An omega-6 fatty acid produced in the body as the delta 6-desaturase metabolite of linoleic acid. It is converted to dihomo-gamma-linolenic acid, a biosynthetic precursor of monoenoic prostaglandins such as PGE1. (From Merck Index, 11th ed)'),
  'D017966': NtO(MH='Pyramidal Cells', MS='Projection neurons in the CEREBRAL CORTEX and the HIPPOCAMPUS. Pyramidal cells have a pyramid-shaped soma with the apex and an apical dendrite pointed toward the pial surface and other dendrites and an axon emerging from the base. The axons may have local collaterals but also project outside their cortical region.'),
  'D017967': NtO(MH='Zinc Compounds', MS='Inorganic compounds that contain zinc as an integral part of the molecule.'),
  'D017968': NtO(MH='Vanadium Compounds', MS='Inorganic compounds that contain vanadium as an integral part of the molecule.'),
  'D017969': NtO(MH='Lentiviruses, Feline', MS='A subgenus of LENTIVIRUS comprising viruses that produce multi-organ disease with long incubation periods in cats.'),
  'D017970': NtO(MH='Thorium Compounds', MS='Inorganic compounds that contain thorium as an integral part of the molecule.'),
  'D017971': NtO(MH='Tin Compounds', MS='Inorganic compounds that contain tin as an integral part of the molecule.'),
  'D017972': NtO(MH='Lentiviruses, Ovine-Caprine', MS='A subgenus of LENTIVIRUS comprising viruses that produce multi-organ disease with long incubation periods in sheep and goats.'),
  'D017973': NtO(MH='Tungsten Compounds', MS='Inorganic compounds that contain tungsten as an integral part of the molecule.'),
  'D017974': NtO(MH='Uranium Compounds', MS='Inorganic compounds that contain uranium as an integral part of the molecule.'),
  'D017975': NtO(MH='Ruthenium Compounds', MS='Inorganic compounds that contain ruthenium as an integral part of the molecule.'),
  'D017976': NtO(MH='Phosphorus Acids', MS='Inorganic acids that contain phosphorus as an integral part of the molecule.'),
  'D017977': NtO(MH='Deltaretrovirus', MS='A genus in the family RETROVIRIDAE consisting of exogenous horizontally-transmitted viruses found in a few groups of mammals. Infections caused by these viruses include human B- or adult T-cell leukemia/lymphoma (LEUKEMIA-LYMPHOMA, T-CELL, ACUTE, HTLV-I-ASSOCIATED), and bovine leukemia (ENZOOTIC BOVINE LEUKOSIS). The type species is LEUKEMIA VIRUS, BOVINE.'),
  'D017978': NtO(MH='Receptors, Growth Factor', MS='Cell surface receptors that bind growth or trophic factors with high affinity, triggering intracellular responses which influence the growth, differentiation, or survival of cells.'),
  'D017979': NtO(MH='Gold Colloid', MS='A suspension of metallic gold particles.'),
  'D017981': NtO(MH='Receptors, Neurotransmitter', MS='Cell surface receptors that bind signalling molecules released by neurons and convert these signals into intracellular changes influencing the behavior of cells. Neurotransmitter is used here in its most general sense, including not only messengers that act to regulate ion channels, but also those which act on second messenger systems and those which may act at a distance from their release sites. Included are receptors for neuromodulators, neuroregulators, neuromediators, and neurohumors, whether or not located at synapses.'),
  'D017982': NtO(MH='Receptors, Biogenic Amine', MS='Cell surface proteins that bind biogenic amines with high affinity and regulate intracellular signals which influence the behavior of cells. Biogenic amine is a chemically imprecise term which, by convention, includes the catecholamines epinephrine, norepinephrine, and dopamine, the indoleamine serotonin, the imidazolamine histamine, and compounds closely related to each of these.'),
  'D017983': NtO(MH='Receptors, Catecholamine', MS='Cell surface proteins that bind catecholamines with high affinity and trigger intracellular changes which influence the behavior of cells. The catecholamine messengers epinephrine, norepinephrine, and dopamine are synthesized from tyrosine by a common biosynthetic pathway.'),
  'D017984': NtO(MH='Enoxaparin', MS='Low-molecular-weight fragment of heparin, having a 4-enopyranosuronate sodium structure at the non-reducing end of the chain. It is prepared by depolymerization of the benzylic ester of porcine mucosal heparin. Therapeutically, it is used as an antithrombotic agent. (From Merck Index, 11th ed)'),
  'D017985': NtO(MH='Dalteparin', MS='A low-molecular-weight fragment of heparin, prepared by nitrous acid depolymerization of porcine mucosal heparin. The mean molecular weight is 4000-6000 daltons. It is used therapeutically as an antithrombotic agent. (From Merck Index, 11th ed)'),
  'D017986': NtO(MH='Reticuloendotheliosis Viruses, Avian', MS='A group of viruses in the genus GAMMARETROVIRUS comprising a few isolates from birds, with no known corresponding endogenous relatives.'),
  'D017988': NtO(MH='Gammaretrovirus', MS='A genus of RETROVIRIDAE comprising endogenous sequences in mammals, related RETICULOENDOTHELIOSIS VIRUSES, AVIAN, and a reptilian virus. Many species contain oncogenes and cause leukemias and sarcomas.'),
  'D017989': NtO(MH='Leukemia Virus, Gibbon Ape', MS='A species of GAMMARETROVIRUS causing leukemia in the gibbon ape. Natural transmission is by contact.'),
  'D017990': NtO(MH='Densovirus', MS='A genus of PARVOVIRIDAE, subfamily DENSOVIRINAE, comprising helper-independent viruses containing only two species. Junonia coenia densovirus is the type species.'),
  'D017992': NtO(MH='Parvovirus', MS='A genus of the family PARVOVIRIDAE, subfamily PARVOVIRINAE, infecting a variety of vertebrates including humans. Parvoviruses are responsible for a number of important diseases but also can be non-pathogenic in certain hosts. The type species is MINUTE VIRUS OF MICE.'),
  'D017993': NtO(MH='Parvovirus, Canine', MS='A species of the genus PARVOVIRUS and a host range variant of FELINE PANLEUKOPENIA VIRUS. It causes a highly infectious fulminating ENTERITIS in dogs producing high mortality. It is distinct from CANINE MINUTE VIRUS, a species in the genus BOCAVIRUS. This virus can also infect cats and mink.'),
  'D017994': NtO(MH='Iridovirus', MS='A genus of IRIDOVIRIDAE comprising small iridescent insect viruses. The infected larvae and purified virus pellets exhibit a blue to purple iridescence.'),
  'D017995': NtO(MH='Ranavirus', MS='A genus of IRIDOVIRIDAE which infects fish, amphibians and reptiles. It is non-pathogenic for its natural host, Rana pipiens, but is lethal for other frogs, toads, turtles and salamanders. Frog virus 3 is the type species.'),
  'D017996': NtO(MH='Entomopoxvirinae', MS='A subfamily of POXVIRIDAE comprising poxviruses infecting insects including members of COLEOPTERA; DIPTERA; LEPIDOPTERA; and ORTHOPTERA.'),
  'D017997': NtO(MH='Leukotriene C4', MS='The conjugation product of LEUKOTRIENE A4 and glutathione. It is the major arachidonic acid metabolite in macrophages and human mast cells as well as in antigen-sensitized lung tissue. It stimulates mucus secretion in the lung, and produces contractions of nonvascular and some VASCULAR SMOOTH MUSCLE. (From Dictionary of Prostaglandins and Related Compounds, 1990)'),
  'D017998': NtO(MH='Leukotriene D4', MS='One of the biologically active principles of SRS-A. It is generated from LEUKOTRIENE C4 after partial hydrolysis of the peptide chain, i.e., cleavage of the gamma-glutamyl portion. Its biological actions include stimulation of vascular and nonvascular smooth muscle, and increases in vascular permeability. (From Dictionary of Prostaglandins and Related Compounds, 1990)'),
  'D017999': NtO(MH='Leukotriene E4', MS='A biologically active principle of SRS-A that is formed from LEUKOTRIENE D4 via a peptidase reaction that removes the glycine residue. The biological actions of LTE4 are similar to LTC4 and LTD4. (From Dictionary of Prostaglandins and Related Compounds, 1990)'),
  'D018000': NtO(MH='Receptors, Peptide', MS='Cell surface receptors that bind peptide messengers with high affinity and regulate intracellular signals which influence the behavior of cells.'),
  'D018002': NtO(MH='Receptors, Bradykinin', MS='Cell surface receptors that bind BRADYKININ and related KININS with high affinity and trigger intracellular changes which influence the behavior of cells. The identified receptor types (B-1 and B-2, or BK-1 and BK-2) recognize endogenous KALLIDIN; t-kinins; and certain bradykinin fragments as well as bradykinin itself.'),
  'D018003': NtO(MH='Receptors, Calcitonin', MS='Cell surface proteins that bind calcitonin and trigger intracellular changes which influence the behavior of cells. Calcitonin receptors outside the nervous system mediate the role of calcitonin in calcium homeostasis. The role of calcitonin receptors in the brain is not well understood.'),
  'D018004': NtO(MH='Receptors, Bombesin', MS='Cell surface proteins that bind bombesin or closely related peptides with high affinity and trigger intracellular changes influencing the behavior of cells. Gastrin- releasing peptide (GRP); GRP 18-27 (neuromedin C), and neuromedin B are endogenous ligands of bombesin receptors in mammals.'),
  'D018005': NtO(MH='Receptors, Vasoactive Intestinal Peptide', MS='Cell surface proteins that bind VASOACTIVE INTESTINAL PEPTIDE; (VIP); with high affinity and trigger intracellular changes which influence the behavior of cells.'),
  'D018006': NtO(MH='Myogenic Regulatory Factors', MS='A family of muscle-specific transcription factors which bind to DNA in control regions and thus regulate myogenesis. All members of this family contain a conserved helix-loop-helix motif which is homologous to the myc family proteins. These factors are only found in skeletal muscle. Members include the myoD protein (MYOD PROTEIN); MYOGENIN; myf-5, and myf-6 (also called MRF4 or herculin).'),
  'D018008': NtO(MH='Myogenin', MS='A myogenic regulatory factor that controls myogenesis. Myogenin is induced during differentiation of every skeletal muscle cell line that has been investigated, in contrast to the other myogenic regulatory factors that only appear in certain cell types.'),
  'D018009': NtO(MH='Receptors, Glycine', MS='Cell surface receptors that bind GLYCINE with high affinity and trigger intracellular changes which influence the behavior of cells. Glycine receptors in the CENTRAL NERVOUS SYSTEM have an intrinsic chloride channel and are usually inhibitory.'),
  'D018010': NtO(MH='Receptors, Invertebrate Peptide', MS='Cell surface receptors for invertebrate peptide hormones or neuropeptides.'),
  'D018011': NtO(MH='Chalcogens', MS='The elements OXYGEN; POLONIUM; SELENIUM; SULFUR; and TELLURIUM; that form group 16 (formerly group VI) of the periodic table.'),
  'D018013': NtO(MH='Receptors, Neuropeptide', MS='Cell surface receptors that bind specific neuropeptides with high affinity and trigger intracellular changes influencing the behavior of cells. Many neuropeptides are also hormones outside of the nervous system.'),
  'D018014': NtO(MH='Gene Transfer Techniques', MS='The introduction of functional (usually cloned) GENES into cells. A variety of techniques and naturally occurring processes are used for the gene transfer such as cell hybridization, LIPOSOMES or microcell-mediated gene transfer, ELECTROPORATION, chromosome-mediated gene transfer, TRANSFECTION, and GENETIC TRANSDUCTION. Gene transfer may result in genetically transformed cells and individual organisms.'),
  'D018015': NtO(MH='Receptors, Calcitonin Gene-Related Peptide', MS='Cell surface proteins that bind CALCITONIN GENE-RELATED PEPTIDE with high affinity and trigger intracellular changes which influence the behavior of cells. CGRP receptors are present in both the CENTRAL NERVOUS SYSTEM and the periphery. They are formed via the heterodimerization of the CALCITONIN RECEPTOR-LIKE PROTEIN and RECEPTOR ACTIVITY-MODIFYING PROTEIN 1.'),
  'D018016': NtO(MH='Receptors, Parathyroid Hormone', MS="Cell surface proteins that bind PARATHYROID HORMONE with high affinity and trigger intracellular changes which influence the behavior of cells. Parathyroid hormone receptors on BONE; KIDNEY; and gastrointestinal cells mediate the hormone's role in calcium and phosphate homeostasis."),
  'D018017': NtO(MH='Receptors, Pituitary Hormone-Regulating Hormone', MS='Cell surface receptors that bind the hypothalamic hormones regulating pituitary cell differentiation, proliferation, and hormone synthesis and release, including the pituitary-releasing and release-inhibiting hormones. The pituitary hormone-regulating hormones are also released by cells other than hypothalamic neurons, and their receptors also occur on non-pituitary cells, especially brain neurons, where their role is less well understood. Receptors for dopamine, which is a prolactin release-inhibiting hormone as well as a common neurotransmitter, are not included here.'),
  'D018018': NtO(MH='Arterivirus', MS='A genus of the family ARTERIVIRIDAE, in the order NIDOVIRALES. The type species is EQUARTEVIRUS.'),
  'D018019': NtO(MH='Receptors, Corticotropin-Releasing Hormone', MS='Cell surface proteins that bind corticotropin-releasing hormone with high affinity and trigger intracellular changes which influence the behavior of cells. The corticotropin releasing-hormone receptors on anterior pituitary cells mediate the stimulation of corticotropin release by hypothalamic corticotropin releasing factor. The physiological consequence of activating corticotropin-releasing hormone receptors on central neurons is not well understood.'),
  'D018020': NtO(MH='Lithium Compounds', MS='Inorganic compounds that contain lithium as an integral part of the molecule.'),
  'D018021': NtO(MH='Lithium Chloride', MS='A salt of lithium that has been used experimentally as an immunomodulator.'),
  'D018023': NtO(MH='HIV Seronegativity', MS='Immune status consisting of non-production of HIV antibodies, as determined by various serological tests.'),
  'D018024': NtO(MH='Peer Review, Health Care', MS='The concurrent or retrospective review by practicing physicians or other health professionals of the quality and efficiency of patient care practices or services ordered or performed by other physicians or other health professionals (From The Facts On File Dictionary of Health Care Management, 1988).'),
  'D018025': NtO(MH='Receptors, Thyrotropin-Releasing Hormone', MS='Cell surface receptors that bind thyrotropin releasing hormone (TRH) with high affinity and trigger intracellular changes which influence the behavior of cells. Activated TRH receptors in the anterior pituitary stimulate the release of thyrotropin (thyroid stimulating hormone, TSH); TRH receptors on neurons mediate neurotransmission by TRH.'),
  'D018026': NtO(MH='Receptors, Pancreatic Hormone', MS='Cell surface proteins that bind pancreatic hormones with high affinity and trigger intracellular changes which influence the behavior of cells. These include receptors for glucagon (secreted by alpha cells), insulin (secreted by beta cells), somatostatin (secreted by delta cells), and pancreatic peptide (secreted by PP cells). Some of these hormones and receptors also support neurotransmission.'),
  'D018027': NtO(MH='Receptors, Glucagon', MS='Cell surface receptors that bind glucagon with high affinity and trigger intracellular changes which influence the behavior of cells. Activation of glucagon receptors causes a variety of effects; the best understood is the initiation of a complex enzymatic cascade in the liver which ultimately increases the availability of glucose to body organs.'),
  'D018028': NtO(MH='Receptors, Neurotensin', MS='Cell surface proteins that bind neurotensin with high affinity and trigger intracellular changes which influence the behavior of cells. Neurotensin and neurotensin receptors are found in the central nervous system and in the periphery.'),
  'D018029': NtO(MH='Rho(D) Immune Globulin', MS='Immunizing agent containing IMMUNOGLOBULIN G anti-Rho(D) used for preventing Rh immunization in Rh-negative individuals exposed to Rh-positive red blood cells.'),
  'D018030': NtO(MH='Silver Compounds', MS='Inorganic compounds that contain silver as an integral part of the molecule.'),
  'D018031': NtO(MH='Connexin 43', MS='A 43-kDa peptide which is a member of the connexin family of gap junction proteins. Connexin 43 is a product of a gene in the alpha class of connexin genes (the alpha-1 gene). It was first isolated from mammalian heart, but is widespread in the body including the brain.'),
  'D018033': NtO(MH='Antibodies, Bispecific', MS='Antibodies, often monoclonal, in which the two antigen-binding sites are specific for separate ANTIGENIC DETERMINANTS. They are artificial antibodies produced by chemical crosslinking, fusion of HYBRIDOMA cells, or by molecular genetic techniques. They function as the main mediators of targeted cellular cytotoxicity and have been shown to be efficient in the targeting of drugs, toxins, radiolabeled haptens, and effector cells to diseased tissue, primarily tumors.'),
  'D018034': NtO(MH='Olfactory Receptor Neurons', MS='Neurons in the OLFACTORY EPITHELIUM with proteins (RECEPTORS, ODORANT) that bind, and thus detect, odorants. These neurons send their DENDRITES to the surface of the epithelium with the odorant receptors residing in the apical non-motile cilia. Their unmyelinated AXONS synapse in the OLFACTORY BULB of the BRAIN.'),
  'D018035': NtO(MH='Receptors, Odorant', MS='Proteins, usually projecting from the cilia of olfactory receptor neurons, that specifically bind odorant molecules and trigger responses in the neurons. The large number of different odorant receptors appears to arise from several gene families or subfamilies rather than from DNA rearrangement.'),
  'D018036': NtO(MH='Selenium Compounds', MS='Inorganic compounds that contain selenium as an integral part of the molecule.'),
  'D018038': NtO(MH='Sodium Selenite', MS='The disodium salt of selenious acid. It is used therapeutically to supply the trace element selenium and is prepared by the reaction of SELENIUM DIOXIDE with SODIUM HYDROXIDE.'),
  'D018039': NtO(MH='Receptors, Tachykinin', MS='Cell surface proteins that bind TACHYKININS with high affinity and trigger intracellular changes influencing the behavior of cells. Three classes of tachykinin receptors have been characterized, the NK-1; NK-2; and NK-3; which prefer, respectively, SUBSTANCE P; NEUROKININ A; and NEUROKININ B.'),
  'D018040': NtO(MH='Receptors, Neurokinin-1', MS='A class of cell surface receptors for TACHYKININS with a preference for SUBSTANCE P. Neurokinin-1 (NK-1) receptors have been cloned and are members of the G protein coupled receptor superfamily. They are found on many cell types including central and peripheral neurons, smooth muscle cells, acinar cells, endothelial cells, fibroblasts, and immune cells.'),
  'D018041': NtO(MH='Receptors, Neurokinin-2', MS='A class of cell surface receptors for tachykinins that prefers neurokinin A; (NKA, substance K, neurokinin alpha, neuromedin L), neuropeptide K; (NPK); or neuropeptide gamma over other tachykinins. Neurokinin-2 (NK-2) receptors have been cloned and are similar to other G-protein coupled receptors.'),
  'D018042': NtO(MH='Receptors, Neurokinin-3', MS='A class of cell surface receptors for tachykinins that prefers neurokinin B (neurokinin beta, neuromedin K) over other tachykinins. Neurokinin-3 (NK-3) receptors have been cloned and are members of the G-protein coupled receptor superfamily. They have been found in the central nervous system and in peripheral tissues.'),
  'D018043': NtO(MH='Receptors, Corticotropin', MS='Cell surface receptors that bind CORTICOTROPIN; (ACTH, adrenocorticotropic hormone) with high affinity and trigger intracellular changes. Pharmacology suggests there may be multiple ACTH receptors. An ACTH receptor has been cloned and belongs to a subfamily of G-protein-coupled receptors. In addition to the adrenal cortex, ACTH receptors are found in the brain and immune systems.'),
  'D018045': NtO(MH='Receptors, Oxytocin', MS="Cell surface proteins that bind oxytocin with high affinity and trigger intracellular changes which influence the behavior of cells. Oxytocin receptors in the uterus and the mammary glands mediate the hormone's stimulation of contraction and milk ejection. The presence of oxytocin and oxytocin receptors in neurons of the brain probably reflects an additional role as a neurotransmitter."),
  'D018046': NtO(MH='Protein C Inhibitor', MS='A member of the serpin family of proteins that is found in plasma and urine. It is dependent on heparin and is able to inhibit activated PROTEIN C; THROMBIN; KALLIKREIN; and other SERINE ENDOPEPTIDASES.'),
  'D018047': NtO(MH='Receptors, Purinergic P1', MS='A class of cell surface receptors that prefer ADENOSINE to other endogenous PURINES. Purinergic P1 receptors are widespread in the body including the cardiovascular, respiratory, immune, and nervous systems. There are at least two pharmacologically distinguishable types (A1 and A2, or Ri and Ra).'),
  'D018048': NtO(MH='Receptors, Purinergic P2', MS='A class of cell surface receptors for PURINES that prefer ATP or ADP over ADENOSINE. P2 purinergic receptors are widespread in the periphery and in the central and peripheral nervous system.'),
  'D018049': NtO(MH='Arenaviruses, Old World', MS='One of two groups of viruses in the ARENAVIRUS genus and considered part of the Old World complex. It includes LASSA VIRUS and LYMPHOCYTIC CHORIOMENINGITIS VIRUS, although the latter has worldwide distribution now.'),
  'D018050': NtO(MH='Arenavirus', MS='The only genus in the family ARENAVIRIDAE. It contains two groups ARENAVIRUSES, OLD WORLD and ARENAVIRUSES, NEW WORLD, which are distinguished by antigenic relationships and geographic distribution.'),
  'D018051': NtO(MH='Junin virus', MS='A species of ARENAVIRUS, part of the New World Arenaviruses (ARENAVIRUSES, NEW WORLD), causing Argentinian hemorrhagic fever. The disease is characterized by congestion, edema, generalized lymphadenopathy and hemorrhagic necrosis and is sometimes fatal.'),
  'D018052': NtO(MH='Pichinde virus', MS='A species of ARENAVIRUS, one of the New World Arenaviruses (ARENAVIRUSES, NEW WORLD), causing a fatal infection in the cricetine rodent Oryzomys albigularis. Asymptomatic laboratory infection in humans has been reported.'),
  'D018053': NtO(MH='Arsenites', MS='Inorganic salts or organic esters of arsenious acid.'),
  'D018054': NtO(MH='Hydrobromic Acid', MS='Hydrobromic acid (HBr). A solution of hydrogen bromide gas in water.'),
  'D018055': NtO(MH='Receptors, Eicosanoid', MS='Cell surface proteins that bind eicosanoids with high affinity and trigger intracellular changes influencing the behavior of cells. Among the eicosanoid receptors are receptors for the prostaglandins, thromboxanes, and leukotrienes.'),
  'D018056': NtO(MH='Hemorheology', MS='The deformation and flow behavior of BLOOD and its elements i.e., PLASMA; ERYTHROCYTES; WHITE BLOOD CELLS; and BLOOD PLATELETS.'),
  'D018057': NtO(MH='Dental Waste', MS='Any waste product generated by a dental office, surgery, clinic, or laboratory including amalgams, saliva, and rinse water.'),
  'D018058': NtO(MH='Tympanic Membrane Perforation', MS='A temporary or persistent opening in the eardrum (TYMPANIC MEMBRANE). Clinical signs depend on the size, location, and associated pathological condition.'),
  'D018059': NtO(MH='Birnaviridae', MS='A family of bisegmented, double-stranded RNA viruses causing infection in fish, mollusks, fowl, and Drosophila. There are three genera: AQUABIRNAVIRUS; AVIBIRNAVIRUS; and ENTOMOBIRNAVIRUS. Horizontal and vertical transmission occurs for all viruses.'),
  'D018061': NtO(MH='Infectious pancreatic necrosis virus', MS='The type species of AQUABIRNAVIRUS, causing infectious pancreatic necrosis in salmonid fish and other freshwater and marine animals including mollusks.'),
  'D018062': NtO(MH='La Crosse virus', MS='A serotype of the species California encephalitis virus (ENCEPHALITIS VIRUS, CALIFORNIA), in the genus ORTHOBUNYAVIRUS, causing human MENINGOENCEPHALITIS. This is the agent most responsible for California encephalitis (ENCEPHALITIS, CALIFORNIA), the most prevalent mosquito-borne disease recognized in the United States.'),
  'D018063': NtO(MH='Hantaan virus', MS='The type species of the genus HANTAVIRUS infecting the rodent Apodemus agrarius and humans who come in contact with it. It causes syndromes of hemorrhagic fever associated with vascular and especially renal pathology.'),
  'D018066': NtO(MH='Uukuniemi virus', MS='A species in the genus PHLEBOVIRUS of the family BUNYAVIRIDAE, infecting vertebrates and vectored by ticks. It has not been associated with human disease though antibodies have been isolated from human sera.'),
  'D018067': NtO(MH='Flaviviridae', MS='A family of RNA viruses, many of which cause disease in humans and domestic animals. There are three genera FLAVIVIRUS; PESTIVIRUS; and HEPACIVIRUS, as well as several unassigned species.'),
  'D018068': NtO(MH='Encephalitis Virus, Murray Valley', MS='A species of FLAVIVIRUS, one of the Japanese encephalitis virus group (ENCEPHALITIS VIRUSES, JAPANESE), found in Australia and New Guinea. It causes a fulminating viremia resembling Japanese encephalitis (ENCEPHALITIS, JAPANESE).'),
  'D018069': NtO(MH='Hair Cells, Vestibular', MS='Sensory cells in the acoustic maculae with their apical STEREOCILIA embedded in a gelatinous OTOLITHIC MEMBRANE. These hair cells are stimulated by the movement of otolithic membrane, and impulses are transmitted via the VESTIBULAR NERVE to the BRAIN STEM. Hair cells in the saccule and those in the utricle sense linear acceleration in vertical and horizontal directions, respectively.'),
  'D018070': NtO(MH='Biological Specimen Banks', MS='Facilities that collect, store, and distribute tissues, e.g., cell lines, microorganisms, blood, sperm, milk, breast tissue, for use by others. Other uses may include transplantation and comparison of diseased tissues in the identification of cancer.'),
  'D018071': NtO(MH='Polydnaviridae', MS='A family of insect viruses isolated from endoparasitic hymenopteran insects belonging to the families Ichneumonidae and Braconidae. The two genera are Ichnovirus and Bracovirus.'),
  'D018072': NtO(MH='Hair Cells, Auditory, Outer', MS='Sensory cells of organ of Corti. In mammals, they are usually arranged in three or four rows, and away from the core of spongy bone (the modiolus), lateral to the INNER AUDITORY HAIR CELLS and other supporting structures. Their cell bodies and STEREOCILIA increase in length from the cochlear base toward the apex and laterally across the rows, allowing differential responses to various frequencies of sound.'),
  'D018073': NtO(MH='Haemophilus Vaccines', MS='Vaccines or candidate vaccines containing antigenic polysaccharides from Haemophilus influenzae and designed to prevent infection. The vaccine can contain the polysaccharides alone or more frequently polysaccharides conjugated to carrier molecules. It is also seen as a combined vaccine with diphtheria-tetanus-pertussis vaccine.'),
  'D018074': NtO(MH='Vaccines, Conjugate', MS='Semisynthetic vaccines consisting of polysaccharide antigens from microorganisms attached to protein carrier molecules. The carrier protein is recognized by macrophages and T-cells thus enhancing immunity. Conjugate vaccines induce antibody formation in people not responsive to polysaccharide alone, induce higher levels of antibody, and show a booster response on repeated injection.'),
  'D018075': NtO(MH='RNA, Complementary', MS='Synthetic transcripts of a specific DNA molecule or fragment, made by an in vitro transcription system. This cRNA can be labeled with radioactive uracil and then used as a probe. (King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D018076': NtO(MH='DNA, Complementary', MS='Single-stranded complementary DNA synthesized from an RNA template by the action of RNA-dependent DNA polymerase. cDNA (i.e., complementary DNA, not circular DNA, not C-DNA) is used in a variety of molecular cloning experiments as well as serving as a specific hybridization probe.'),
  'D018077': NtO(MH='Receptors, Leukotriene', MS='Cell-surface receptors that bind LEUKOTRIENES with high affinity and trigger intracellular changes influencing the behavior of cells. The leukotriene receptor subtypes have been tentatively named according to their affinities for the endogenous leukotrienes LTB4; LTC4; LTD4; and LTE4.'),
  'D018078': NtO(MH='Receptors, Prostaglandin E', MS='Cell surface receptors which bind prostaglandins with a high affinity and trigger intracellular changes which influence the behavior of cells. Prostaglandin E receptors prefer prostaglandin E2 to other endogenous prostaglandins. They are subdivided into EP1, EP2, and EP3 types based on their effects and their pharmacology.'),
  'D018079': NtO(MH='Receptors, GABA', MS='Cell-surface proteins that bind GAMMA-AMINOBUTYRIC ACID with high affinity and trigger changes that influence the behavior of cells. GABA-A receptors control chloride channels formed by the receptor complex itself. They are blocked by bicuculline and usually have modulatory sites sensitive to benzodiazepines and barbiturates. GABA-B receptors act through G-proteins on several effector systems, are insensitive to bicuculline, and have a high affinity for L-baclofen.'),
  'D018080': NtO(MH='Receptors, GABA-B', MS='A subset of GABA RECEPTORS that signal through their interaction with HETEROTRIMERIC G-PROTEINS.'),
  'D018082': NtO(MH='Kringles', MS='Triple-looped protein domains linked by disulfide bonds. These common structural domains, so-named for their resemblance to Danish pastries known as kringlers, play a role in binding membranes, proteins, and phospholipids as well as in regulating proteolysis. Kringles are also present in coagulation-related and fibrinolytic proteins and other plasma proteinases.'),
  'D018083': NtO(MH='Border disease virus', MS='A species of PESTIVIRUS causing a congenital sheep disease characterized by an abnormally hairy birth-coat, tremors, and poor growth.'),
  'D018084': NtO(MH='Ultrasonography, Interventional', MS='The use of ultrasound to guide minimally invasive surgical procedures such as needle ASPIRATION BIOPSY; DRAINAGE; etc. Its widest application is intravascular ultrasound imaging but it is useful also in urology and intra-abdominal conditions.'),
  'D018085': NtO(MH='Regulon', MS='In eukaryotes, a genetic unit consisting of a noncontiguous group of genes under the control of a single regulator gene. In bacteria, regulons are global regulatory systems involved in the interplay of pleiotropic regulatory domains and consist of several OPERONS.'),
  'D018086': NtO(MH='Glutamate Dehydrogenase (NADP+)', MS='An enzyme that catalyzes the conversion of L-glutamate, H2O, and NADP+ to 2-oxoglutarate, NH3, and NADPH. (From Enzyme Nomenclature, 1992) EC 1.4.1.4.'),
  'D018087': NtO(MH='Plastids', MS='Self-replicating cytoplasmic organelles of plant and algal cells that contain pigments and may synthesize and accumulate various substances. PLASTID GENOMES are used in phylogenetic studies.'),
  'D018088': NtO(MH='Tuberculosis, Multidrug-Resistant', MS='Tuberculosis resistant to chemotherapy with two or more ANTITUBERCULAR AGENTS, including at least ISONIAZID and RIFAMPICIN. The problem of resistance is particularly troublesome in tuberculous OPPORTUNISTIC INFECTIONS associated with HIV INFECTIONS. It requires the use of second line drugs which are more toxic than the first line regimens. TB with isolates that have developed further resistance to at least three of the six classes of second line drugs is defined as EXTENSIVELY DRUG-RESISTANT TUBERCULOSIS.'),
  'D018089': NtO(MH='Superantigens', MS="Microbial antigens that have in common an extremely potent activating effect on T-cells that bear a specific variable region. Superantigens cross-link the variable region with class II MHC proteins regardless of the peptide binding in the T-cell receptor's pocket. The result is a transient expansion and subsequent death and anergy of the T-cells with the appropriate variable regions."),
  'D018091': NtO(MH='Receptors, AMPA', MS='A class of ionotropic glutamate receptors characterized by their affinity for the agonist AMPA (alpha-amino-3-hydroxy-5-methyl-4-isoxazolepropionic acid).'),
  'D018092': NtO(MH='Receptors, Kainic Acid', MS='A class of ionotropic glutamate receptors characterized by their affinity for KAINIC ACID.'),
  'D018093': NtO(MH='Gelatinases', MS='A class of enzymes that catalyzes the degradation of gelatin by acting on the peptide bonds. EC 3.4.24.-.'),
  'D018094': NtO(MH='Receptors, Metabotropic Glutamate', MS='Cell surface proteins that bind glutamate and act through G-proteins to influence second messenger systems. Several types of metabotropic glutamate receptors have been cloned. They differ in pharmacology, distribution, and mechanisms of action.'),
  'D018095': NtO(MH='Germ-Line Mutation', MS='Any detectable and heritable alteration in the lineage of germ cells. Mutations in these cells (i.e., "generative" cells ancestral to the gametes) are transmitted to progeny while those in somatic cells are not.'),
  'D018096': NtO(MH='Mononegavirales', MS='An order comprising four families of eukaryotic viruses possessing linear, non-segmented, negative-strand RNA genomes. The families are BORNAVIRIDAE; FILOVIRIDAE; PARAMYXOVIRIDAE; and RHABDOVIRIDAE.'),
  'D018098': NtO(MH='Paramyxovirinae', MS='A subfamily of PARAMYXOVIRIDAE. Genera include RUBULAVIRUS; RESPIROVIRUS; MORBILLIVIRUS; HENIPAVIRUS; and AVULAVIRUS.'),
  'D018099': NtO(MH='Pneumovirinae', MS='A subfamily of PARAMYXOVIRIDAE containing two genera: PNEUMOVIRUS; and METAPNEUMOVIRUS.'),
  'D018100': NtO(MH='Receptors, Histamine H3', MS='A class of histamine receptors discriminated by their pharmacology and mode of action. Histamine H3 receptors were first recognized as inhibitory autoreceptors on histamine-containing nerve terminals and have since been shown to regulate the release of several neurotransmitters in the central and peripheral nervous systems. (From Biochem Soc Trans 1992 Feb;20(1):122-5)'),
  'D018101': NtO(MH='Mice, Inbred mdx', MS='A strain of mice arising from a spontaneous MUTATION (mdx) in inbred C57BL mice. This mutation is X chromosome-linked and produces viable homozygous animals that lack the muscle protein DYSTROPHIN, have high serum levels of muscle ENZYMES, and possess histological lesions similar to human MUSCULAR DYSTROPHY. The histological features, linkage, and map position of mdx make these mice a worthy animal model of DUCHENNE MUSCULAR DYSTROPHY.'),
  'D018102': NtO(MH='Receptors, Leukotriene B4', MS='A class of cell surface leukotriene receptors with a preference for leukotriene B4. Leukotriene B4 receptor activation influences chemotaxis, chemokinesis, adherence, enzyme release, oxidative bursts, and degranulation in polymorphonuclear leukocytes. There are at least two subtypes of these receptors. Some actions are mediated through the inositol phosphate and diacylglycerol second messenger systems.'),
  'D018103': NtO(MH='Morbillivirus', MS='A genus of the family PARAMYXOVIRIDAE (subfamily PARAMYXOVIRINAE) where the virions of most members have hemagglutinin but not neuraminidase activity. All members produce both cytoplasmic and intranuclear inclusion bodies. MEASLES VIRUS is the type species.'),
  'D018104': NtO(MH='Distemper Virus, Phocine', MS='A species of MORBILLIVIRUS causing distemper in seals.'),
  'D018105': NtO(MH='DNA, Kinetoplast', MS='DNA of kinetoplasts which are specialized MITOCHONDRIA of trypanosomes and related parasitic protozoa within the order KINETOPLASTIDA. Kinetoplast DNA consists of a complex network of numerous catenated rings of two classes; the first being a large number of small DNA duplex rings, called minicircles, approximately 2000 base pairs in length, and the second being several dozen much larger rings, called maxicircles, approximately 37 kb in length.'),
  'D018106': NtO(MH='CD28 Antigens', MS='Costimulatory T-LYMPHOCYTE receptors that have specificity for CD80 ANTIGEN and CD86 ANTIGEN. Activation of this receptor results in increased T-cell proliferation, cytokine production and promotion of T-cell survival.'),
  'D018107': NtO(MH='Peste-des-petits-ruminants virus', MS='A species of MORBILLIVIRUS causing a severe, often fatal enteritis and pneumonia (PESTE-DES-PETITS-RUMINANTS) in sheep and goats.'),
  'D018108': NtO(MH='Respirovirus', MS='A genus of the family PARAMYXOVIRIDAE (subfamily PARAMYXOVIRINAE) where all the virions have both HEMAGGLUTININ and NEURAMINIDASE activities and encode a non-structural C protein. SENDAI VIRUS is the type species.'),
  'D018109': NtO(MH='Avulavirus', MS='A genus in the subfamily PARAMYXOVIRINAE, family PARAMYXOVIRIDAE, causing disease in domestic fowl. There are many species, the most well-known being avian paramyxovirus 1 (NEWCASTLE DISEASE VIRUS).'),
  'D018110': NtO(MH='Receptors, Lipoprotein', MS='Cell surface proteins that bind lipoproteins with high affinity. Lipoprotein receptors in the liver and peripheral tissues mediate the regulation of plasma and cellular cholesterol metabolism and concentration. The receptors generally recognize the apolipoproteins of the lipoprotein complex, and binding is often a trigger for endocytosis.'),
  'D018111': NtO(MH='Pneumovirus', MS='A genus of the family PARAMYXOVIRIDAE (subfamily PNEUMOVIRINAE) where the human and bovine virions have neither hemagglutinin nor neuraminidase activity. RESPIRATORY SYNCYTIAL VIRUS, HUMAN is the type species.'),
  'D018112': NtO(MH='Respiratory Syncytial Virus, Bovine', MS='A species of PNEUMOVIRUS causing an important respiratory infection in cattle. Symptoms include fever, conjunctivitis, and respiratory distress.'),
  'D018113': NtO(MH='Respiratory Syncytial Virus, Human', MS='The type species of PNEUMOVIRUS and an important cause of lower respiratory disease in infants and young children. It frequently presents with bronchitis and bronchopneumonia and is further characterized by fever, cough, dyspnea, wheezing, and pallor.'),
  'D018114': NtO(MH='Lyssavirus', MS='A genus of the family RHABDOVIRIDAE that includes RABIES VIRUS and other rabies-like viruses.'),
  'D018115': NtO(MH='Ephemeral Fever Virus, Bovine', MS='The type species of EPHEMEROVIRUS causing disease in cattle. Transmission is by hematophagous arthropods and the virus has been isolated from both culicoides and mosquitoes.'),
  'D018116': NtO(MH='Vesiculovirus', MS='A genus of the family RHABDOVIRIDAE that infects a wide range of vertebrates and invertebrates. The type species is VESICULAR STOMATITIS INDIANA VIRUS.'),
  'D018117': NtO(MH='Thogotovirus', MS='A genus of the family ORTHOMYXOVIRIDAE comprising tick-borne viruses occasionally infecting humans. Dhori and Thogoto viruses were formerly thought to be members of BUNYAVIRIDAE. Thogoto virus is the type species.'),
  'D018118': NtO(MH='Chloride Channels', MS='Cell membrane glycoproteins that form channels to selectively pass chloride ions. Nonselective blockers include FENAMATES; ETHACRYNIC ACID; and TAMOXIFEN.'),
  'D018119': NtO(MH='Stavudine', MS='A dideoxynucleoside analog that inhibits reverse transcriptase and has in vitro activity against HIV.'),
  'D018120': NtO(MH='Finasteride', MS='An orally active 3-OXO-5-ALPHA-STEROID 4-DEHYDROGENASE inhibitor. It is used as a surgical alternative for treatment of benign PROSTATIC HYPERPLASIA.'),
  'D018121': NtO(MH='Receptors, Cytokine', MS='Cell surface proteins that bind cytokines and trigger intracellular changes influencing the behavior of cells.'),
  'D018122': NtO(MH='B7-1 Antigen', MS='A costimulatory ligand membrane glycoprotein that contains a V2 type and C2 IMMUNOGLOBULIN DOMAIN and is expressed by ANTIGEN-PRESENTING CELLS. It binds to CTLA-4 ANTIGEN with high specificity and to CD28 ANTIGEN with low specificity. The interaction of CD80 with CD28 ANTIGEN provides a costimulatory signal to T-LYMPHOCYTES, while its interaction with CTLA-4 ANTIGEN may play a role in inducing PERIPHERAL TOLERANCE.'),
  'D018123': NtO(MH='Receptors, Interleukin', MS='Cell surface proteins that bind interleukins and trigger intracellular changes influencing the behavior of cells.'),
  'D018124': NtO(MH='Receptors, Tumor Necrosis Factor', MS='Cell surface receptors that bind TUMOR NECROSIS FACTORS and trigger changes which influence the behavior of cells.'),
  'D018125': NtO(MH='Receptors, Transforming Growth Factor beta', MS='Cell-surface proteins that bind transforming growth factor beta and trigger changes influencing the behavior of cells. Two types of transforming growth factor receptors have been recognized. They differ in affinity for different members of the transforming growth factor beta family and in cellular mechanisms of action.'),
  'D018126': NtO(MH='Odontodysplasia', MS='A localized arrested tooth development which appears to involve most commonly the anterior teeth, usually on one side of the midline, most often the maxillary central and lateral incisors. Roentgenographically, the teeth have a ghostlike appearance. Calcification and bits of prismatic enamel may be found in the pulp and the enamel is thin and absent in part. (Jablonski, Illustrated Dictionary of Dentistry, 1982)'),
  'D018127': NtO(MH='Tumor Necrosis Factor Receptor Superfamily, Member 7', MS='A member of the tumor necrosis factor receptor superfamily found on most T-LYMPHOCYTES. Activation of the receptor by CD70 ANTIGEN results in the increased proliferation of CD4-POSITIVE T-LYMPHOCYTES and CD8-POSITIVE T-LYMPHOCYTES. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D018129': NtO(MH='Phosphatidylinositol Phosphates', MS='Phosphatidylinositols in which one or more alcohol group of the inositol has been substituted with a phosphate group.'),
  'D018130': NtO(MH='Diamond', MS="Diamond. A crystalline form of carbon that occurs as hard, colorless or tinted isomeric crystals. It is used as a precious stone, for cutting glass, and as bearings for delicate mechanisms. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D018132': NtO(MH='Rubivirus', MS='A genus of the family TOGAVIRIDAE containing only one recognized species, RUBELLA VIRUS. Transmission is primarily by aerosolization.'),
  'D018136': NtO(MH='Circoviridae', MS='A family of very small viruses containing circular, single-stranded DNA and possessing no envelope. The modes of transmission are not known.'),
  'D018138': NtO(MH='Alphaherpesvirinae', MS="A subfamily of HERPESVIRIDAE characterized by a short replication cycle. The genera include: SIMPLEXVIRUS; VARICELLOVIRUS; MAREK'S DISEASE-LIKE VIRUSES; and ILTOVIRUS."),
  'D018139': NtO(MH='Simplexvirus', MS='A genus of the family HERPESVIRIDAE, subfamily ALPHAHERPESVIRINAE, consisting of herpes simplex-like viruses. The type species is HERPESVIRUS 1, HUMAN.'),
  'D018140': NtO(MH='Varicellovirus', MS='A genus of the family HERPESVIRIDAE, subfamily ALPHAHERPESVIRINAE. Its species include those causing CHICKENPOX and HERPES ZOSTER in humans (HERPESVIRUS 3, HUMAN), as well as several animal viruses.'),
  'D018141': NtO(MH='Betaherpesvirinae', MS='A subfamily of HERPESVIRIDAE characterized by a relatively long replication cycle. Genera include: CYTOMEGALOVIRUS; MUROMEGALOVIRUS; and ROSEOLOVIRUS.'),
  'D018142': NtO(MH='Gammaherpesvirinae', MS='A subfamily of HERPESVIRIDAE characterized by variable reproductive cycles. The genera include: LYMPHOCRYPTOVIRUS and RHADINOVIRUS.'),
  'D018143': NtO(MH='Lymphocryptovirus', MS='A genus of the family HERPESVIRIDAE, subfamily GAMMAHERPESVIRINAE, infecting B-cells in humans and new world primates. The type species human herpesvirus 4 (HERPESVIRUS 4, HUMAN) is better known as the Epstein-Barr virus.'),
  'D018144': NtO(MH='Rhadinovirus', MS='A genus of the family HERPESVIRIDAE, subfamily GAMMAHERPESVIRINAE, infecting New World primates and other species. HERPESVIRUS 2, SAIMIRIINE is the type species.'),
  'D018146': NtO(MH='Muromegalovirus', MS='A genus of the family HERPESVIRIDAE, subfamily BETAHERPESVIRINAE, causing infection involving several organs in mice and rats. Murid herpesvirus is the type species.'),
  'D018147': NtO(MH='Chordopoxvirinae', MS='A subfamily of the family POXVIRIDAE, containing eight genera comprising all the vertebrate poxviruses.'),
  'D018148': NtO(MH='Suipoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, containing one species, swinepox virus.'),
  'D018149': NtO(MH='Glucose Intolerance', MS='A pathological state in which BLOOD GLUCOSE level is less than approximately 140 mg/100 ml of PLASMA at fasting, and above approximately 200 mg/100 ml plasma at 30-, 60-, or 90-minute during a GLUCOSE TOLERANCE TEST. This condition is seen frequently in DIABETES MELLITUS, but also occurs with other diseases and MALNUTRITION.'),
  'D018150': NtO(MH='Avipoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, comprising bird poxviruses. The type species is FOWLPOX VIRUS. Transmission is mechanical by ARTHROPODS.'),
  'D018151': NtO(MH='Capripoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, comprising poxviruses infecting sheep, goats, and cattle. Transmission is usually mechanical by arthropods, but also includes contact, airborne routes, and non-living reservoirs (fomites).'),
  'D018152': NtO(MH='Leporipoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, consisting of ether-sensitive viruses of leporids and squirrels. They commonly cause tumors and are usually transmitted mechanically by arthropods. MYXOMA VIRUS is the type species.'),
  'D018153': NtO(MH='Czech Republic', MS='Created 1 January 1993 as a result of the division of Czechoslovakia into the Czech Republic and Slovakia.'),
  'D018154': NtO(MH='Slovakia', MS='Created 1 January 1993 as a result of the division of Czechoslovakia into the Czech Republic and Slovakia.'),
  'D018155': NtO(MH='Orthopoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, comprising many species infecting mammals. Viruses of this genus cause generalized infections and a rash in some hosts. The type species is VACCINIA VIRUS.'),
  'D018156': NtO(MH='Journalism, Medical', MS='The collection, writing, and editing of current interest material on topics related to biomedicine for presentation through the mass media, including newspapers, magazines, radio, or television, usually for a public audience such as health care consumers.'),
  'D018157': NtO(MH='Parapoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, which infect ungulates and may infect humans. ORF VIRUS is the type species.'),
  'D018158': NtO(MH='Yatapoxvirus', MS='A genus of the family POXVIRIDAE, subfamily CHORDOPOXVIRINAE, causing tumors in primates. The type species is YABA MONKEY TUMOR VIRUS.'),
  'D018159': NtO(MH='Endolymphatic Hydrops', MS='An accumulation of ENDOLYMPH in the inner ear (LABYRINTH) leading to buildup of pressure and distortion of intralabyrinthine structures, such as COCHLEA and SEMICIRCULAR CANALS. It is characterized by SENSORINEURAL HEARING LOSS; TINNITUS; and sometimes VERTIGO.'),
  'D018160': NtO(MH='Receptors, Cytoplasmic and Nuclear', MS='Intracellular receptors that can be found in the cytoplasm or in the nucleus. They bind to extracellular signaling molecules that migrate through or are transported across the CELL MEMBRANE. Many members of this class of receptors occur in the cytoplasm and are transported to the CELL NUCLEUS upon ligand-binding where they signal via DNA-binding and transcription regulation. Also included in this category are receptors found on INTRACELLULAR MEMBRANES that act via mechanisms similar to CELL SURFACE RECEPTORS.'),
  'D018161': NtO(MH='Receptors, Mineralocorticoid', MS='Cytoplasmic proteins that specifically bind MINERALOCORTICOIDS and mediate their cellular effects. The receptor with its bound ligand acts in the nucleus to induce transcription of specific segments of DNA.'),
  'D018162': NtO(MH='Coltivirus', MS='A genus of REOVIRIDAE infecting Ixodidae ticks and transmitted by them to humans, deer, and small animals. The type species is COLORADO TICK FEVER VIRUS.'),
  'D018163': NtO(MH='Orbivirus', MS='A genus of REOVIRIDAE infecting a wide range of arthropods and vertebrates including humans. It comprises at least 21 serological subgroups. Transmission is by vectors such as midges, mosquitoes, sandflies, and ticks.'),
  'D018164': NtO(MH='Hemorrhagic Disease Virus, Epizootic', MS='A species of ORBIVIRUS causing a fatal disease in deer. It is transmitted by flies of the genus Culicoides.'),
  'D018165': NtO(MH='Orthoreovirus', MS='A genus of the family REOVIRIDAE infecting vertebrates only. Transmission is horizontal and infected species include humans, birds, cattle, monkeys, sheep, swine, baboons, and bats. MAMMALIAN ORTHOREOVIRUS is the type species.'),
  'D018166': NtO(MH='Health Care Reform', MS="Innovation and improvement of the health care system by reappraisal, amendment of services, and removal of faults and abuses in providing and distributing health services to patients. It includes a re-alignment of health services and health insurance to maximum demographic elements (the unemployed, indigent, uninsured, elderly, inner cities, rural areas) with reference to coverage, hospitalization, pricing and cost containment, insurers' and employers' costs, pre-existing medical conditions, prescribed drugs, equipment, and services."),
  'D018167': NtO(MH='Receptors, Calcitriol', MS='Proteins, usually found in the cytoplasm, that specifically bind calcitriol, migrate to the nucleus, and regulate transcription of specific segments of DNA with the participation of D receptor interacting proteins (called DRIP). Vitamin D is converted in the liver and kidney to calcitriol and ultimately acts through these receptors.'),
  'D018168': NtO(MH='Receptors, Retinoic Acid', MS='Proteins in the nucleus or cytoplasm that specifically bind RETINOIC ACID or RETINOL and trigger changes in the behavior of cells. Retinoic acid receptors, like steroid receptors, are ligand-activated transcription regulators. Several types have been recognized.'),
  'D018169': NtO(MH='Molluscipoxvirus', MS='A genus of POXVIRIDAE, subfamily CHORDOPOXVIRINAE, infecting humans. Transmission is by direct contact among children, by sexual contact among young adults, or by fomites. MOLLUSCUM CONTAGIOSUM VIRUS is the type species.'),
  'D018170': NtO(MH='Sumatriptan', MS='A serotonin agonist that acts selectively at 5HT1 receptors. It is used in the treatment of MIGRAINE DISORDERS.'),
  'D018171': NtO(MH='Agrin', MS='A protein component of the synaptic basal lamina. It has been shown to induce clustering of acetylcholine receptors on the surface of muscle fibers and other synaptic molecules in both synapse regeneration and development.'),
  'D018172': NtO(MH='United States Office of Research Integrity', MS='An office of the UNITED STATES PUBLIC HEALTH SERVICE organized in June 1992 to promote research integrity and investigate misconduct in research supported by the Public Health Service. It consolidates the Office of Scientific Integrity of the National Institutes of Health and the Office of Scientific Integrity Review in the Office of the Assistant Secretary for Health.'),
  'D018173': NtO(MH='Circoviridae Infections', MS='Virus diseases caused by the CIRCOVIRIDAE.'),
  'D018174': NtO(MH='Arterivirus Infections', MS='Infections caused by viruses of the genus ARTERIVIRUS.'),
  'D018175': NtO(MH='Birnaviridae Infections', MS='Virus diseases caused by the BIRNAVIRIDAE.'),
  'D018176': NtO(MH='Torovirus Infections', MS='Infections with viruses of the genus TOROVIRUS, family CORONAVIRIDAE.'),
  'D018177': NtO(MH='Flavivirus Infections', MS='Infections with viruses of the genus FLAVIVIRUS, family FLAVIVIRIDAE.'),
  'D018178': NtO(MH='Flaviviridae Infections', MS='Infections with viruses of the family FLAVIVIRIDAE.'),
  'D018179': NtO(MH='Receptors, Thrombin', MS='A family of proteinase-activated receptors that are specific for THROMBIN. They are found primarily on PLATELETS and on ENDOTHELIAL CELLS. Activation of thrombin receptors occurs through the proteolytic action of THROMBIN, which cleaves the N-terminal peptide from the receptor to reveal a new N-terminal peptide that is a cryptic ligand for the receptor. The receptors signal through HETEROTRIMERIC GTP-BINDING PROTEINS. Small synthetic peptides that contain the unmasked N-terminal peptide sequence can also activate the receptor in the absence of proteolytic activity.'),
  'D018180': NtO(MH='Thrombomodulin', MS='A cell surface glycoprotein of endothelial cells that binds thrombin and serves as a cofactor in the activation of protein C and its regulation of blood coagulation.'),
  'D018181': NtO(MH='Mice, Inbred CFTR', MS='A strain of mice widely studied as a model for cystic fibrosis. These mice are generated from embryonic stem cells in which the CFTR (cystic fibrosis transmembrane conductance regulator) gene is inactivated by gene targeting. As a result, all mice have one copy of this altered gene in all their tissues. Mice homozygous for the disrupted gene exhibit many features common to young cystic fibrosis patients, including failure to thrive, meconium ileus, and alteration of mucous and serous glands.'),
  'D018182': NtO(MH='Pestivirus Infections', MS='Infections with viruses of the genus PESTIVIRUS, family FLAVIVIRIDAE.'),
  'D018183': NtO(MH='Transplantation Chimera', MS='An organism that, as a result of transplantation of donor tissue or cells, consists of two or more cell lines descended from at least two zygotes. This state may result in the induction of donor-specific TRANSPLANTATION TOLERANCE.'),
  'D018184': NtO(MH='Paramyxoviridae Infections', MS='Infections with viruses of the family PARAMYXOVIRIDAE. This includes MORBILLIVIRUS INFECTIONS; RESPIROVIRUS INFECTIONS; PNEUMOVIRUS INFECTIONS; HENIPAVIRUS INFECTIONS; AVULAVIRUS INFECTIONS; and RUBULAVIRUS INFECTIONS.'),
  'D018185': NtO(MH='Morbillivirus Infections', MS='Infections with viruses of the genus MORBILLIVIRUS, family PARAMYXOVIRIDAE. Infections mainly cause acute disease in their hosts, although in some cases infection is persistent and leads to degenerative conditions.'),
  'D018186': NtO(MH='Pneumovirus Infections', MS='Infections with viruses of the genus PNEUMOVIRUS, family PARAMYXOVIRIDAE. This includes RESPIRATORY SYNCYTIAL VIRUS INFECTIONS, an important cause of respiratory disease in humans.'),
  'D018187': NtO(MH='Personal Construct Theory', MS='A psychological theory based on dimensions or categories used by a given person in describing or explaining the personality and behavior of others or of himself. The basic idea is that different people will use consistently different categories. The theory was formulated in the fifties by George Kelly. Two tests devised by him are the role construct repertory test and the repertory grid test. (From Stuart Sutherland, The International Dictionary of Psychology, 1989)'),
  'D018188': NtO(MH='Cardiovirus Infections', MS='Infections caused by viruses of the genus CARDIOVIRUS, family PICORNAVIRIDAE.'),
  'D018189': NtO(MH='Immunomagnetic Separation', MS='A cell-separation technique where magnetizable microspheres or beads are first coated with monoclonal antibody, allowed to search and bind to target cells, and are then selectively removed when passed through a magnetic field. Among other applications, the technique is commonly used to remove tumor cells from the marrow (BONE MARROW PURGING) of patients who are to undergo autologous bone marrow transplantation.'),
  'D018190': NtO(MH='Lymphatic Vessel Tumors', MS='Neoplasms composed of lymphoid tissue, a lattice work of reticular tissue the interspaces of which contain lymphocytes. The concept does not refer to neoplasms located in lymphatic vessels.'),
  'D018191': NtO(MH='Lymphangioma, Cystic', MS='A cystic growth originating from lymphatic tissue. It is usually found in the neck, axilla, or groin.'),
  'D018192': NtO(MH='Lymphangioleiomyomatosis', MS='A disease characterized by the progressive invasion of SMOOTH MUSCLE CELLS into the LYMPHATIC VESSELS, and the BLOOD VESSELS. The majority of the cases occur in the LUNGS of women of child-bearing age, eventually blocking the flow of air, blood, and lymph. The common symptom is shortness of breath (DYSPNEA).'),
  'D018193': NtO(MH='Neoplasms, Complex and Mixed', MS='Neoplasms composed of more than one type of neoplastic tissue.'),
  'D018194': NtO(MH='Adenomyoma', MS='A benign neoplasm of muscle (usually smooth muscle) with glandular elements. It occurs most frequently in the uterus and uterine ligaments. (Stedman, 25th ed)'),
  'D018195': NtO(MH='Adenosarcoma', MS='A malignant neoplasm arising simultaneously or consecutively in mesodermal tissue and glandular epithelium of the same part. (Stedman, 25th ed)'),
  'D018196': NtO(MH='Carcinoma, Adenosquamous', MS='A mixed adenocarcinoma and squamous cell or epidermoid carcinoma.'),
  'D018197': NtO(MH='Hepatoblastoma', MS='A malignant neoplasm occurring in young children, primarily in the liver, composed of tissue resembling embryonal or fetal hepatic epithelium, or mixed epithelial and mesenchymal tissues. (Stedman, 25th ed)'),
  'D018198': NtO(MH='Mixed Tumor, Malignant', MS='A malignant tumor composed of more than one type of neoplastic tissue. (Dorland, 27th ed)'),
  'D018199': NtO(MH='Mixed Tumor, Mesodermal', MS='A sarcoma of the body of the uterus arising in older women, composed of more than one mesenchymal tissue, especially including striated muscle cells. It is associated with previous pelvic radiation exposure in 20% of patients. (Stedman, 25th ed; Holland et al., Cancer Medicine, 3d ed, p1702)'),
  'D018200': NtO(MH='Mixed Tumor, Mullerian', MS='A tumor, basically a carcinoma with a single sarcoma such as leiomyosarcoma or angiosarcoma or multiple sarcomas of uterine origin. The role of estrogen has been postulated as a possible etiological factor in this tumor. (Holland et al., Cancer Medicine, 3d ed, p1703)'),
  'D018201': NtO(MH='Nephroma, Mesoblastic', MS='A solid, unencapsulated tumor of the KIDNEY composed of spindle mesenchymal cells that resemble FIBROBLASTS or muscle cells. The homogeneous mass typically extends into the renal parenchyma and replaces most of the kidney. In most cases, mesoblastic nephroma is benign and occurs in the fetus or newborn, and rarely in the older child or the adult.'),
  'D018202': NtO(MH='Pulmonary Blastoma', MS='A malignant neoplasm of the lung composed chiefly or entirely of immature undifferentiated cells (i.e., blast forms) with little or virtually no stroma. (From Stedman, 25th ed)'),
  'D018203': NtO(MH='Sarcoma, Endometrial Stromal', MS='A highly malignant subset of neoplasms arising from the endometrial stroma. Tumors in this group infiltrate the stroma with a wide range of atypia cells and numerous mitoses. They are capable of widespread metastases (NEOPLASM METASTASIS).'),
  'D018204': NtO(MH='Neoplasms, Connective and Soft Tissue', MS='Neoplasms developing from some structure of the connective and subcutaneous tissue. The concept does not refer to neoplasms located in connective or soft tissue.'),
  'D018205': NtO(MH='Neoplasms, Adipose Tissue', MS='Neoplasms composed of fatty tissue or connective tissue made up of fat cells in a meshwork of areolar tissue. The concept does not refer to neoplasms located in adipose tissue.'),
  'D018206': NtO(MH='Angiolipoma', MS='A benign neoplasm composed of a mixture of adipose tissue and blood vessels. (Dorland, 27th ed)'),
  'D018207': NtO(MH='Angiomyolipoma', MS='A benign tumor containing vascular, adipose, and muscle elements. It occurs most often in the kidney with smooth muscle elements (angiolipoleiomyoma) in association with tuberous sclerosis. (Dorland, 27th ed)'),
  'D018208': NtO(MH='Liposarcoma, Myxoid', MS='A liposarcoma containing round mesenchymal cells and a myxoid extracellular matrix in stroma.'),
  'D018209': NtO(MH='Myelolipoma', MS='A rare benign tumor of the adrenal gland, several centimeters in diameter, composed in varying proportions of adipose tissue, lymphocytes, and primitive myeloid cells, probably a developmental abnormality. (Dorland, 27th ed)'),
  'D018210': NtO(MH='Chondromatosis', MS='Multiple formation of chondromas. (Dorland, 27th ed)'),
  'D018211': NtO(MH='Chondrosarcoma, Mesenchymal', MS='A rare aggressive variant of chondrosarcoma, characterized by a biphasic histologic pattern of small compact cells intermixed with islands of cartilaginous matrix. Mesenchymal chondrosarcomas have a predilection for flat bones; long tubular bones are rarely affected. They tend to occur in the younger age group and are highly metastatic. (DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1456)'),
  'D018212': NtO(MH='Giant Cell Tumor of Bone', MS='A bone tumor composed of cellular spindle-cell stroma containing scattered multinucleated giant cells resembling osteoclasts. The tumors range from benign to frankly malignant lesions. The tumor occurs most frequently in an end of a long tubular bone in young adults. (From Dorland, 27th ed; Stedman, 25th ed)'),
  'D018213': NtO(MH='Neoplasms, Bone Tissue', MS='Neoplasms composed of bony tissue, whether normal or of a soft tissue which has become ossified. The concept does not refer to neoplasms located in bones.'),
  'D018214': NtO(MH='Fibroma, Ossifying', MS='A benign central bone tumor, usually of the jaws (especially the mandible), composed of fibrous connective tissue within which bone is formed.'),
  'D018215': NtO(MH='Osteoblastoma', MS='A benign, painful, tumor of bone characterized by the formation of osteoid tissue, primitive bone and calcified tissue. It occurs frequently in the spine of young persons. (From Dorland, 27th ed; Stedman, 25th ed)'),
  'D018216': NtO(MH='Osteochondromatosis', MS='A condition marked by the presence of multiple osteochondromas. (Dorland, 27th ed)'),
  'D018217': NtO(MH='Osteosarcoma, Juxtacortical', MS='A form of osteogenic sarcoma of relatively low malignancy, probably arising from the periosteum and initially involving cortical bone and adjacent connective tissue. It occurs in middle-aged as well as young adults and most commonly affects the lower part of the femoral shaft. (Stedman, 25th ed)'),
  'D018218': NtO(MH='Neoplasms, Fibrous Tissue', MS='Neoplasms composed of fibrous tissue, the ordinary connective tissue of the body, made up largely of yellow or white fibers. The concept does not refer to neoplasms located in fibrous tissue.'),
  'D018219': NtO(MH='Histiocytoma, Benign Fibrous', MS='A benign tumor composed, wholly or in part, of cells with the morphologic characteristics of HISTIOCYTES and with various fibroblastic components. Fibrous histiocytomas can occur anywhere in the body. When they occur in the skin, they are called dermatofibromas or sclerosing hemangiomas. (From DeVita Jr et al., Cancer: Principles & Practice of Oncology, 5th ed, p1747)'),
  'D018220': NtO(MH='Fibroma, Desmoplastic', MS='An extremely rare bone tumor characterized by abundant collagen formation and a fibrous stroma, without evidence of mitosis or pleomorphism. It appears on x-rays as an osteolytic lesion with well-defined margins and must be differentiated from primary fibrosarcoma of bone. (DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1441)'),
  'D018221': NtO(MH='Fibromatosis, Abdominal', MS='A relatively large mass of unusually firm scarlike connective tissue resulting from active participation of fibroblasts, occurring most frequently in the abdominal muscles of women who have borne children. The fibroblasts infiltrate surrounding muscle and fascia. (Stedman, 25th ed)'),
  'D018222': NtO(MH='Fibromatosis, Aggressive', MS='A childhood counterpart of abdominal or extra-abdominal desmoid tumors, characterized by firm subcutaneous nodules that grow rapidly in any part of the body but do not metastasize. The adult form of abdominal fibromatosis is FIBROMATOSIS, ABDOMINAL. (Stedman, 25th ed)'),
  'D018223': NtO(MH='Dermatofibrosarcoma', MS='A sarcoma of the deep layers of the skin. The tumors are locally aggressive tends to recur but rarely metastatic. It can be classified into variants depending on the cell type tumors are derived from or by its characteristics: Pigmented variant from MELANIN-containing DERMAL DENDRITIC CELLS; Myxoid variant, myxoid STROMAL CELLS; Giant cell variant characterized by GIANT CELLS in the tumors; and Fibrosarcomatous variant chracterized by tumor areas histologically indistinguishable from FIBROSARCOMA.'),
  'D018224': NtO(MH='Myofibromatosis', MS='A condition characterized by multiple formations of myofibromas (LEIOMYOMA).'),
  'D018225': NtO(MH='Neoplasms, Fibroepithelial', MS='Neoplasms composed of fibrous and epithelial tissue. The concept does not refer to neoplasms located in fibrous tissue or epithelium.'),
  'D018226': NtO(MH='Fibroadenoma', MS='An adenoma containing fibrous tissue. It should be differentiated from ADENOFIBROMA which is a tumor composed of connective tissue (fibroma) containing glandular (adeno-) structures. (From Dorland, 27th ed)'),
  'D018227': NtO(MH='Sarcoma, Clear Cell', MS='A sarcoma of young adults occurring in the lower extremities and acral regions. It is found intimately bound to tendons as a circumscribed but unencapsulated melanin-bearing tumor of neuroectodermal origin. Clear cell sarcoma is associated with a specific t(12;22)(q13;q12) translocation.'),
  'D018228': NtO(MH='Sarcoma, Small Cell', MS='A sarcoma characterized by the presence of small cells, cells measuring 9-14 micrometers with a faint or indistinct rim of cytoplasm and an oval-to-elongated nucleus with relatively dense chromatin. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D018229': NtO(MH='Angiomyoma', MS='A benign tumor consisting of vascular and smooth muscle elements.'),
  'D018230': NtO(MH='Leiomyoma, Epithelioid', MS='A relatively rare smooth muscle tumor found most frequently in the wall of the gastrointestinal tract, especially in the stomach. It is similar to other smooth muscle tumors but may become very large and hemorrhage and exhibit small cystic areas. Simple excision is almost always curative. (From Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1354)'),
  'D018231': NtO(MH='Leiomyomatosis', MS='The state of having multiple leiomyomas throughout the body. (Stedman, 25th ed)'),
  'D018232': NtO(MH='Rhabdomyosarcoma, Alveolar', MS='A form of RHABDOMYOSARCOMA occurring mainly in adolescents and young adults, affecting muscles of the extremities, trunk, orbital region, etc. It is extremely malignant, metastasizing widely at an early stage. Few cures have been achieved and the prognosis is poor. "Alveolar" refers to its microscopic appearance simulating the cells of the respiratory alveolus. (Holland et al., Cancer Medicine, 3d ed, p2188)'),
  'D018233': NtO(MH='Rhabdomyosarcoma, Embryonal', MS='A form of RHABDOMYOSARCOMA arising primarily in the head and neck, especially the orbit, of children below the age of 10. The cells are smaller than those of other rhabdomyosarcomas and are of two basic cell types: spindle cells and round cells. This cancer is highly sensitive to chemotherapy and has a high cure rate with multi-modality therapy. (From Holland et al., Cancer Medicine, 3d ed, p2188)'),
  'D018234': NtO(MH='Sarcoma, Alveolar Soft Part', MS='A variety of rare sarcoma having a reticulated fibrous stroma enclosing groups of sarcoma cells, which resemble epithelial cells and are enclosed in alveoli walled with connective tissue. It is a rare tumor, usually occurring between 15 and 35 years of age. It appears in the muscles of the extremities in adults and most commonly in the head and neck regions of children. Though slow-growing, it commonly metastasizes to the lungs, brain, bones, and lymph nodes. (DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1365)'),
  'D018235': NtO(MH='Smooth Muscle Tumor', MS='A tumor composed of smooth muscle tissue, as opposed to leiomyoma, a tumor derived from smooth muscle.'),
  'D018236': NtO(MH='Carcinoma, Embryonal', MS='A highly malignant, primitive form of carcinoma, probably of germinal cell or teratomatous derivation, usually arising in a gonad and rarely in other sites. It is rare in the female ovary, but in the male it accounts for 20% of all testicular tumors. (From Dorland, 27th ed & Holland et al., Cancer Medicine, 3d ed, p1595)'),
  'D018237': NtO(MH='Germinoma', MS='A malignant neoplasm of the germinal tissue of the GONADS; MEDIASTINUM; or pineal region. Germinomas are uniform in appearance, consisting of large, round cells with vesicular nuclei and clear or finely granular eosinophilic-staining cytoplasm. (Stedman, 265th ed; from DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, pp1642-3)'),
  'D018238': NtO(MH='Gonadoblastoma', MS='A complex neoplasm composed of a mixture of gonadal elements, such as large primordial GERM CELLS, immature SERTOLI CELLS or GRANULOSA CELLS of the sex cord, and gonadal stromal cells. Gonadoblastomas are most often associated with gonadal dysgenesis, 46, XY.'),
  'D018239': NtO(MH='Seminoma', MS='A radiosensitive, malignant neoplasm of the testis, thought to be derived from primordial germ cells of the sexually undifferentiated embryonic gonad. There are three variants: classical (typical), the most common type; anaplastic; and spermatocytic. The classical seminoma is composed of fairly well differentiated sheets or cords of uniform polygonal or round cells (seminoma cells), each cell having abundant clear cytoplasm, distinct cell membranes, a centrally placed round nucleus, and one or more nucleoli. In the female, a grossly and histologically identical neoplasm, known as dysgerminoma, occurs. (Dorland, 27th ed)'),
  'D018240': NtO(MH='Endodermal Sinus Tumor', MS='An unusual and aggressive tumor of germ-cell origin that reproduces the extraembryonic structures of the early embryo. It is the most common malignant germ cell tumor found in children. It is characterized by a labyrinthine glandular pattern of flat epithelial cells and rounded papillary processes with a central capillary (Schiller-Duval body). The tumor is rarely bilateral. Before the use of combination chemotherapy, the tumor was almost invariably fatal. (From DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1189)'),
  'D018241': NtO(MH='Neuroectodermal Tumors, Primitive, Peripheral', MS='A group of highly cellular primitive round cell neoplasms which occur extracranially in soft tissue and bone and are derived from embryonal neural crest cells. These tumors occur primarily in children and adolescents and share a number of characteristics with EWING SARCOMA.'),
  'D018242': NtO(MH='Neuroectodermal Tumors, Primitive', MS='A group of malignant tumors of the nervous system that feature primitive cells with elements of neuronal and/or glial differentiation. Use of this term is limited by some authors to central nervous system tumors and others include neoplasms of similar origin which arise extracranially (i.e., NEUROECTODERMAL TUMORS, PRIMITIVE, PERIPHERAL). This term is also occasionally used as a synonym for MEDULLOBLASTOMA. In general, these tumors arise in the first decade of life and tend to be highly malignant. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2059)'),
  'D018243': NtO(MH='Teratocarcinoma', MS='A malignant neoplasm consisting of elements of teratoma with those of embryonal carcinoma or choriocarcinoma, or both. It occurs most often in the testis. (Dorland, 27th ed)'),
  'D018244': NtO(MH='Chromosomes, Artificial, Yeast', MS='Chromosomes in which fragments of exogenous DNA ranging in length up to several hundred kilobase pairs have been cloned into yeast through ligation to vector sequences. These artificial chromosomes are used extensively in molecular biology for the construction of comprehensive genomic libraries of higher organisms.'),
  'D018245': NtO(MH='Trophoblastic Tumor, Placental Site', MS='An uncommon variant of CHORIOCARCINOMA. It is composed almost entirely of mononuclear cytotrophoblasts (TROPHOBLASTS). Because its secretion of hCG (CHORIONIC GONADOTROPIN) is low, a large tumor may develop before the hCG can be detected.'),
  'D018246': NtO(MH='Adrenocortical Adenoma', MS='A benign neoplasm of the ADRENAL CORTEX. It is characterized by a well-defined nodular lesion, usually less than 2.5 cm. Most adrenocortical adenomas are nonfunctional. The functional ones are yellow and contain LIPIDS. Depending on the cell type or cortical zone involved, they may produce ALDOSTERONE; HYDROCORTISONE; DEHYDROEPIANDROSTERONE; and/or ANDROSTENEDIONE.'),
  'D018247': NtO(MH='Helicobacter heilmannii', MS='A species of gram-negative, spiral-shaped bacteria found in the gastric mucosa that is associated with chronic antral gastritis. This bacterium was first discovered in samples removed at endoscopy from patients investigated for HELICOBACTER PYLORI colonization.'),
  'D018248': NtO(MH='Adenoma, Liver Cell', MS='A benign epithelial tumor of the LIVER.'),
  'D018249': NtO(MH='Adenoma, Oxyphilic', MS='A usually benign glandular tumor composed of oxyphil cells, large cells with small irregular nuclei and dense acidophilic granules due to the presence of abundant MITOCHONDRIA. Oxyphil cells, also known as oncocytes, are found in oncocytomas of the kidney, salivary glands, and endocrine glands. In the thyroid gland, oxyphil cells are known as Hurthle cells and Askanazy cells.'),
  'D018250': NtO(MH='Acrospiroma', MS='A rare cutaneous tumor of apocrine or eccrine SWEAT GLAND origin. It is most commonly found on the extremities and is usually benign. It appears as a solitary nodule or cyst and may be solid or produce a watery discharge. It is related to POROMA except in acrospiroma it does not involve the epidermis. There is no indication that heredity or external agents cause these tumors.'),
  'D018251': NtO(MH='Hidrocystoma', MS='A cystic form of sweat gland adenoma (ADENOMA, SWEAT GLAND). It is produced by the cystic proliferation of apocrine secretory glands. It is not uncommon, occurring in adult life in no particular age group, with males and females equally affected. The commonest site is around the eye, particularly lateral to the outer canthus. It is cured by surgical removal. (Stedman, 25th ed; Rook et al., Textbook of Dermatology, 4th ed, p2410)'),
  'D018252': NtO(MH='Syringoma', MS='A benign tumor of the sweat glands which is usually multiple and results from malformation of sweat ducts. It is uncommon and more common in females than in males. It is most likely to appear at adolescence, and further lesions may develop during adult life. It does not appear to be hereditary. (Rook et al., Textbook of Dermatology, 4th ed, pp2407-8)'),
  'D018253': NtO(MH='Adenoma, Villous', MS='An adenoma of the large intestine. It is usually a solitary, sessile, often large, tumor of colonic mucosa composed of mucinous epithelium covering delicate vascular projections. Hypersecretion and malignant changes occur frequently. (Stedman, 25th ed)'),
  'D018254': NtO(MH='Adenomatoid Tumor', MS='A small, circumscribed, benign tumor of the genital tract, composed of small glandlike spaces lined by flattened or cuboidal mesothelium-like cells. (From Dorland, 27th ed)'),
  'D018255': NtO(MH='Adenomatosis, Pulmonary', MS='A neoplastic disease in which the alveoli and distal bronchi are filled with mucus and mucus-secreting columnar epithelial cells. It is characterized by abundant, extremely tenacious sputum, chills, fever, cough, dyspnea, and pleuritic pain. (Stedman, 25th ed)'),
  'D018256': NtO(MH='Adenomatous Polyps', MS='Benign neoplasms derived from glandular epithelium. (From Stedman, 25th ed)'),
  'D018257': NtO(MH='Helix-Loop-Helix Motifs', MS='Recurring supersecondary structures characterized by 20 amino acids folding into two alpha helices connected by a non-helical "loop" segment. They are found in many sequence-specific DNA-BINDING PROTEINS and in CALCIUM-BINDING PROTEINS.'),
  'D018258': NtO(MH='Herpesvirus 2, Human', MS='A species of SIMPLEXVIRUS associated with genital infections (HERPES GENITALIS). It is transmitted by sexual intercourse and close personal contact.'),
  'D018259': NtO(MH='Herpesvirus 1, Human', MS='The type species of SIMPLEXVIRUS causing most forms of non-genital herpes simplex in humans. Primary infection occurs mainly in infants and young children and then the virus becomes latent in the dorsal root ganglion. It then is periodically reactivated throughout life causing mostly benign conditions.'),
  'D018260': NtO(MH='Gelsolin', MS='A 90-kDa protein produced by macrophages that severs ACTIN filaments and forms a cap on the newly exposed filament end. Gelsolin is activated by CALCIUM ions and participates in the assembly and disassembly of actin, thereby increasing the motility of some CELLS.'),
  'D018261': NtO(MH='Mesothelioma, Cystic', MS='A peritoneal mesothelioma affecting mainly young females and producing cysts of variable size and number lined by a single layer of benign mesothelial cells. The disease follows a benign course and is compatible with a normal life expectancy, requiring occasionally partial excision or decompression for relief of pain or other symptoms. Malignant potential is exceptional. (From Holland et al., Cancer Medicine, 3d ed, p1345)'),
  'D018262': NtO(MH='Adenocarcinoma, Clear Cell', MS='An adenocarcinoma characterized by the presence of varying combinations of clear and hobnail-shaped tumor cells. There are three predominant patterns described as tubulocystic, solid, and papillary. These tumors, usually located in the female reproductive organs, have been seen more frequently in young women since 1970 as a result of the association with intrauterine exposure to diethylstilbestrol. (From Holland et al., Cancer Medicine, 3d ed)'),
  'D018263': NtO(MH='Adenocarcinoma, Follicular', MS='An adenocarcinoma of the thyroid gland, in which the cells are arranged in the form of follicles. (From Dorland, 27th ed)'),
  'D018265': NtO(MH='Carcinoma, Papillary, Follicular', MS='A thyroid neoplasm of mixed papillary and follicular arrangement. Its biological behavior and prognosis is the same as that of a papillary adenocarcinoma of the thyroid. (From DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1271)'),
  'D018266': NtO(MH='Adenocarcinoma, Sebaceous', MS='A malignant tumor composed of cells showing differentiation toward sebaceous epithelium. The tumor is solitary, firm, somewhat raised, more or less translucent, and covered with normal or slightly verrucose epidermis. It may be yellow or orange. The face and scalp are the commonest sites. The growth can be slow or rapid but metastasis is uncommon. Surgery cures most of the cases. (From Rook et al., Textbook of Dermatology, 4th ed, pp2403-4)'),
  'D018267': NtO(MH='Carcinoma, Acinar Cell', MS='A malignant tumor arising from secreting cells of a racemose gland, particularly the salivary glands. Racemose (Latin racemosus, full of clusters) refers, as does acinar (Latin acinus, grape), to small saclike dilatations in various glands. Acinar cell carcinomas are usually well differentiated and account for about 13% of the cancers arising in the parotid gland. Lymph node metastasis occurs in about 16% of cases. Local recurrences and distant metastases many years after treatment are common. This tumor appears in all age groups and is most common in women. (Stedman, 25th ed; Holland et al., Cancer Medicine, 3d ed, p1240; from DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p575)'),
  'D018268': NtO(MH='Adrenocortical Carcinoma', MS='A malignant neoplasm of the ADRENAL CORTEX. Adrenocortical carcinomas are unencapsulated anaplastic (ANAPLASIA) masses sometimes exceeding 20 cm or 200 g. They are more likely to be functional than nonfunctional, and produce ADRENAL CORTEX HORMONES that may result in hypercortisolism (CUSHING SYNDROME); HYPERALDOSTERONISM; and/or VIRILISM.'),
  'D018269': NtO(MH='Carcinoma, Endometrioid', MS='An adenocarcinoma characterized by the presence of cells resembling the glandular cells of the ENDOMETRIUM. It is a common histological type of ovarian CARCINOMA and ENDOMETRIAL CARCINOMA. There is a high frequency of co-occurrence of this form of adenocarcinoma in both tissues.'),
  'D018270': NtO(MH='Carcinoma, Ductal, Breast', MS='An invasive (infiltrating) CARCINOMA of the mammary ductal system (MAMMARY GLANDS) in the human BREAST.'),
  'D018271': NtO(MH='Signal Recognition Particle', MS='A cytosolic ribonucleoprotein complex that acts to induce elongation arrest of nascent presecretory and membrane proteins until the ribosome becomes associated with the rough endoplasmic reticulum. It consists of a 7S RNA and at least six polypeptide subunits (relative molecular masses 9, 14, 19, 54, 68, and 72K).'),
  'D018272': NtO(MH='Porins', MS='Porins are protein molecules that were originally found in the outer membrane of GRAM-NEGATIVE BACTERIA and that form multi-meric channels for the passive DIFFUSION of WATER; IONS; or other small molecules. Porins are present in bacterial CELL WALLS, as well as in plant, fungal, mammalian and other vertebrate CELL MEMBRANES and MITOCHONDRIAL MEMBRANES.'),
  'D018273': NtO(MH='Carcinoma, Islet Cell', MS='A primary malignant neoplasm of the pancreatic ISLET CELLS. Usually it involves the non-INSULIN-producing cell types, the PANCREATIC ALPHA CELLS and the pancreatic delta cells (SOMATOSTATIN-SECRETING CELLS) in GLUCAGONOMA and SOMATOSTATINOMA, respectively.'),
  'D018274': NtO(MH='Electroporation', MS='A technique in which electric pulses of intensity in kilovolts per centimeter and of microsecond-to-millisecond duration cause a temporary loss of the semipermeability of CELL MEMBRANES, thus leading to ion leakage, escape of metabolites, and increased uptake by cells of drugs, molecular probes, and DNA.'),
  'D018275': NtO(MH='Carcinoma, Lobular', MS='A type of BREAST CANCER where the abnormal malignant cells form in the lobules, or milk-producing glands, of the breast.'),
  'D018276': NtO(MH='Carcinoma, Medullary', MS='A carcinoma composed mainly of epithelial elements with little or no stroma. Medullary carcinomas of the breast constitute 5%-7% of all mammary carcinomas; medullary carcinomas of the thyroid comprise 3%-10% of all thyroid malignancies. (From Dorland, 27th ed; DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p1141; Segen, Dictionary of Modern Medicine, 1992)'),
  'D018277': NtO(MH='Carcinoma, Mucoepidermoid', MS='A tumor of both low- and high-grade malignancy. The low-grade grow slowly, appear in any age group, and are readily cured by excision. The high-grade behave aggressively, widely infiltrate the salivary gland and produce lymph node and distant metastases. Mucoepidermoid carcinomas account for about 21% of the malignant tumors of the parotid gland and 10% of the sublingual gland. They are the most common malignant tumor of the parotid. (From DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p575; Holland et al., Cancer Medicine, 3d ed, p1240)'),
  'D018278': NtO(MH='Carcinoma, Neuroendocrine', MS='A group of carcinomas which share a characteristic morphology, often being composed of clusters and trabecular sheets of round "blue cells", granular chromatin, and an attenuated rim of poorly demarcated cytoplasm. Neuroendocrine tumors include carcinoids, small ("oat") cell carcinomas, medullary carcinoma of the thyroid, Merkel cell tumor, cutaneous neuroendocrine carcinoma, pancreatic islet cell tumors, and pheochromocytoma. Neurosecretory granules are found within the tumor cells. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D018279': NtO(MH='Carcinoma, Signet Ring Cell', MS='A poorly differentiated adenocarcinoma in which the nucleus is pressed to one side by a cytoplasmic droplet of mucus. It usually arises in the gastrointestinal system.'),
  'D018280': NtO(MH='Carcinoma, Skin Appendage', MS='A malignant tumor of the skin appendages, which include the hair, nails, sebaceous glands, sweat glands, and the mammary glands. (From Dorland, 27th ed)'),
  'D018281': NtO(MH='Cholangiocarcinoma', MS='A malignant tumor arising from the epithelium of the BILE DUCTS.'),
  'D018282': NtO(MH='Cystadenocarcinoma, Mucinous', MS='A malignant cystic or semisolid tumor most often occurring in the ovary. Rarely, one is solid. This tumor may develop from a mucinous cystadenoma, or it may be malignant at the onset. The cysts are lined with tall columnar epithelial cells; in others, the epithelium consists of many layers of cells that have lost normal structure entirely. In the more undifferentiated tumors, one may see sheets and nests of tumor cells that have very little resemblance to the parent structure. (Hughes, Obstetric-Gynecologic Terminology, 1972, p184)'),
  'D018283': NtO(MH='Cystadenocarcinoma, Papillary', MS='An adenocarcinoma in which the tumor elements are arranged as finger-like processes or as a solid spherical nodule projecting from an epithelial surface.'),
  'D018284': NtO(MH='Cystadenocarcinoma, Serous', MS='A malignant cystic or semicystic neoplasm. It often occurs in the ovary and usually bilaterally. The external surface is usually covered with papillary excrescences. Microscopically, the papillary patterns are predominantly epithelial overgrowths with differentiated and undifferentiated papillary serous cystadenocarcinoma cells. Psammoma bodies may be present. The tumor generally adheres to surrounding structures and produces ascites. (From Hughes, Obstetric-Gynecologic Terminology, 1972, p185)'),
  'D018285': NtO(MH='Klatskin Tumor', MS='Cholangiocarcinoma arising near or at the confluence of the right and left hepatic ducts (COMMON HEPATIC DUCT). These tumors are generally small, sharply localized, and seldom metastasizing.'),
  'D018286': NtO(MH='Carcinoma, Giant Cell', MS='An epithelial neoplasm characterized by unusually large anaplastic cells. It is highly malignant with fulminant clinical course, bizarre histologic appearance and poor prognosis. It is most common in the lung and thyroid. (From Stedman, 25th ed & Segen, Dictionary of Modern Medicine, 1992)'),
  'D018287': NtO(MH='Carcinoma, Large Cell', MS='A tumor of undifferentiated (anaplastic) cells of large size. It is usually bronchogenic. (From Dorland, 27th ed)'),
  'D018288': NtO(MH='Carcinoma, Small Cell', MS='An anaplastic, highly malignant, and usually bronchogenic carcinoma composed of small ovoid cells with scanty neoplasm. It is characterized by a dominant, deeply basophilic nucleus, and absent or indistinct nucleoli. (From Stedman, 25th ed; Holland et al., Cancer Medicine, 3d ed, p1286-7)'),
  'D018289': NtO(MH='Carcinoma, Verrucous', MS='A variant of well-differentiated epidermoid carcinoma that is most common in the oral cavity, but also occurs in the larynx, nasal cavity, esophagus, penis, anorectal region, vulva, vagina, uterine cervix, and skin, especially on the sole of the foot. Most intraoral cases occur in elderly male abusers of smokeless tobacco. The treatment is surgical resection. Radiotherapy is not indicated, as up to 30% treated with radiation become highly aggressive within six months. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D018290': NtO(MH='Cervical Intraepithelial Neoplasia', MS='A malignancy arising in uterine cervical epithelium and confined thereto, representing a continuum of histological changes ranging from well-differentiated CIN 1 (formerly, mild dysplasia) to severe dysplasia/carcinoma in situ, CIN 3. The lesion arises at the squamocolumnar cell junction at the transformation zone of the endocervical canal, with a variable tendency to develop invasive epidermoid carcinoma, a tendency that is enhanced by concomitant human papillomaviral infection. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D018291': NtO(MH='Cystadenoma, Mucinous', MS='A multilocular tumor with mucin secreting epithelium. They are most often found in the ovary, but are also found in the pancreas, appendix, and rarely, retroperitoneal and in the urinary bladder. They are considered to have low-grade malignant potential.'),
  'D018292': NtO(MH='Cystadenoma, Papillary', MS='A benign neoplasm of the ovary.'),
  'D018293': NtO(MH='Cystadenoma, Serous', MS='A cystic tumor of the ovary, containing thin, clear, yellow serous fluid and varying amounts of solid tissue, with a malignant potential several times greater than that of mucinous cystadenoma (CYSTADENOMA, MUCINOUS). It can be unilocular, parvilocular, or multilocular. It is often bilateral and papillary. The cysts may vary greatly in size. (Dorland, 27th ed; from Hughes, Obstetric-Gynecologic Terminology, 1972)'),
  'D018294': NtO(MH='Neoplasms, Adnexal and Skin Appendage', MS='Neoplasms composed of sebaceous or sweat gland tissue or tissue of other skin appendages. The concept does not refer to neoplasms located in the sebaceous or sweat glands or in the other skin appendages.'),
  'D018295': NtO(MH='Neoplasms, Basal Cell', MS='Neoplasms composed of cells from the deepest layer of the epidermis. The concept does not refer to neoplasms located in the stratum basale.'),
  'D018296': NtO(MH='Pilomatrixoma', MS="A tumor composed of cells resembling those of the hair matrix, which undergo 'mummification' and may calcify. It is a relatively uncommon tumor, which may occur at any age from infancy. The majority of patients are under 20, and females are affected more than males. The lesion is usually a solitary deep dermal or subcutaneous tumor 3-30 mm in diameter, situated in the head, neck, or upper extremity. (From Rook et al., Textbook of Dermatology, 4th ed, p2401)"),
  'D018297': NtO(MH='Neoplasms, Cystic, Mucinous, and Serous', MS='Neoplasms containing cyst-like formations or producing mucin or serum.'),
  'D018298': NtO(MH='Mucoepidermoid Tumor', MS='A malignant epithelial tumor of glandular tissue, especially the salivary glands, characterized by acini with mucus-producing cells and by the presence of malignant squamous elements. Most mucoepidermoid tumors are low-grade lesions readily cured by adequate excision. They may appear in any age group. They grow slowly. If high-grade, they behave aggressively, widely infiltrating the salivary gland and producing lymph node and distant metastases. (Dorland, 27th ed; from DeVita Jr et al., Cancer: Principles & Practice of Oncology, 3d ed, p575)'),
  'D018299': NtO(MH='Neoplasms, Ductal, Lobular, and Medullary', MS='Neoplasms, usually carcinoma, located within the center of an organ or within small lobes, and in the case of the breast, intraductally. The emphasis of the name is on the location of the neoplastic tissue rather than on its histological type. Most cancers of this type are located in the breast.'),
  'D018300': NtO(MH='Papilloma, Intraductal', MS='A small, often impalpable benign papilloma arising in a lactiferous duct and frequently causing bleeding from the nipple. (Stedman, 25th ed)'),
  'D018301': NtO(MH='Neoplasms, Mesothelial', MS='Neoplasms composed of tissue of the mesothelium, the layer of flat cells, derived from the mesoderm, which lines the body cavity of the embryo. In the adult it forms the simple squamous epithelium which covers all true serous membranes (peritoneum, pericardium, pleura). The concept does not refer to neoplasms located in these organs. (From Dorland, 27th ed)'),
  'D018302': NtO(MH='Neoplasms, Neuroepithelial', MS='Neoplasms composed of neuroepithelial cells, which have the capacity to differentiate into NEURONS, oligodendrocytes, and ASTROCYTES. The majority of craniospinal tumors are of neuroepithelial origin. (From Dev Biol 1998 Aug 1;200(1):1-5)'),
  'D018303': NtO(MH='Ganglioglioma', MS='Rare indolent tumors comprised of neoplastic glial and neuronal cells which occur primarily in children and young adults. Benign lesions tend to be associated with long survival unless the tumor degenerates into a histologically malignant form. They tend to occur in the optic nerve and white matter of the brain and spinal cord.'),
  'D018304': NtO(MH='Esthesioneuroblastoma, Olfactory', MS='A malignant olfactory neuroblastoma arising from the olfactory epithelium of the superior nasal cavity and cribriform plate. It is uncommon (3% of nasal tumors) and rarely is associated with the production of excess hormones (e.g., SIADH, Cushing Syndrome). It has a high propensity for multiple local recurrences and bony metastases. (From Holland et al., Cancer Medicine, 3rd ed, p1245; J Laryngol Otol 1998 Jul;112(7):628-33)'),
  'D018305': NtO(MH='Ganglioneuroblastoma', MS='A moderately malignant neoplasm composed of primitive neuroectodermal cells dispersed in myxomatous or fibrous stroma intermixed with mature ganglion cells. It may undergo transformation into a neuroblastoma. It arises from the sympathetic trunk or less frequently from the adrenal medulla, cerebral cortex, and other locations. Cervical ganglioneuroblastomas may be associated with HORNER SYNDROME and the tumor may occasionally secrete vasoactive intestinal peptide, resulting in chronic diarrhea.'),
  'D018306': NtO(MH='Neurocytoma', MS='A benign brain tumor composed of neural elements which most often arise from the SEPTUM PELLUCIDUM and the walls of the lateral ventricles. Immunohistochemistry and electron microscopy evaluations may reveal expression of neuron specific enolase and synaptophysin and cells containing microtubuli, neurosecretory granules, and presynaptic vesicles. (From Acta Med Port 1994 Feb;7(2):113-9)'),
  'D018307': NtO(MH='Neoplasms, Squamous Cell', MS='Neoplasms of the SQUAMOUS EPITHELIAL CELLS. The concept does not refer to neoplasms located in tissue composed of squamous elements.'),
  'D018308': NtO(MH='Papilloma, Inverted', MS='A mucosal tumor of the urinary bladder or nasal cavity in which proliferating epithelium is invaginated beneath the surface and is more smoothly rounded than in other papillomas. (Stedman, 25th ed)'),
  'D018309': NtO(MH='Neoplasms, Gonadal Tissue', MS='Neoplasms composed of tissues of the OVARY or the TESTIS, not neoplasms located in the ovaries or testes. Gonadal tissues include GERM CELLS, cells from the sex cord, and gonadal stromal cells.'),
  'D018310': NtO(MH='Sertoli-Leydig Cell Tumor', MS='A sex cord-gonadal stromal tumor consists of LEYDIG CELLS; SERTOLI CELLS; and FIBROBLASTS in varying proportions and degree of differentiation. Most such tumors produce ANDROGENS in the Leydig cells, formerly known as androblastoma or arrhenoblastoma. Androblastomas occur in the TESTIS or the OVARY causing precocious masculinization in the males, and defeminization, or virilization (VIRILISM) in the females. In some cases, the Sertoli cells produce ESTROGENS.'),
  'D018311': NtO(MH='Luteoma', MS='An ovarian neoplasm composed of LUTEAL CELLS derived from luteinized GRANULOSA CELLS and THECA CELLS. Luteomas respond to GONADOTROPINS, and vary in their hormone production (PROGESTERONE; ESTROGENS; or ANDROGENS). During PREGNANCY, a transient type of luteoma may develop due to an exaggerated LUTEINIZATION of the OVARY.'),
  'D018312': NtO(MH='Sex Cord-Gonadal Stromal Tumors', MS='Neoplasms derived from the primitive sex cord or gonadal stromal cells of the embryonic GONADS. They are classified by their presumed histogenesis and differentiation. From the sex cord, there are SERTOLI CELL TUMOR and GRANULOSA CELL TUMOR; from the gonadal stroma, LEYDIG CELL TUMOR and THECOMA. These tumors may be identified in either the OVARY or the TESTIS.'),
  'D018313': NtO(MH='Leishmania enriettii', MS='A parasitic hemoflagellate of the subgenus Leishmania leishmania that has been found as a natural infection of the Brazilian guinea pig. Its host-tissue relationship is, in general, comparable to that of L. braziliensis.'),
  'D018314': NtO(MH='Leishmania infantum', MS='A parasitic hemoflagellate of the subgenus Leishmania leishmania that infects man and animals and causes visceral leishmaniasis (LEISHMANIASIS, VISCERAL). Human infections are confined almost entirely to children. This parasite is commonly seen in dogs, other Canidae, and porcupines with humans considered only an accidental host. Transmission is by Phlebotomus sandflies.'),
  'D018315': NtO(MH='Glioma, Subependymal', MS='Rare, slow-growing, benign intraventricular tumors, often asymptomatic and discovered incidentally. The tumors are classified histologically as ependymomas and demonstrate a proliferation of subependymal fibrillary astrocytes among the ependymal tumor cells. (From Clin Neurol Neurosurg 1997 Feb;99(1):17-22)'),
  'D018316': NtO(MH='Gliosarcoma', MS='Rare mixed tumors of the brain and rarely the spinal cord which contain malignant neuroectodermal (glial) and mesenchymal components, including spindle-shaped fibrosarcoma cells. These tumors are highly aggressive and present primarily in adults as rapidly expanding mass lesions. They may arise in tissue that has been previously irradiated. (From Br J Neurosurg 1995 Apr;9(2):171-8)'),
  'D018317': NtO(MH='Nerve Sheath Neoplasms', MS='Neoplasms which arise from nerve sheaths formed by SCHWANN CELLS in the PERIPHERAL NERVOUS SYSTEM or by OLIGODENDROCYTES in the CENTRAL NERVOUS SYSTEM. Malignant peripheral nerve sheath tumors, NEUROFIBROMA, and NEURILEMMOMA are relatively common tumors in this category.'),
  'D018318': NtO(MH='Neurofibroma, Plexiform', MS='A type of neurofibroma manifesting as a diffuse overgrowth of subcutaneous tissue, usually involving the face, scalp, neck, and chest but occasionally occurring in the abdomen or pelvis. The tumors tend to progress, and may extend along nerve roots to eventually involve the spinal roots and spinal cord. This process is almost always a manifestation of NEUROFIBROMATOSIS 1. (From Adams et al., Principles of Neurology, 6th ed, p1016; J Pediatr 1997 Nov;131(5):678-82)'),
  'D018319': NtO(MH='Neurofibrosarcoma', MS='A malignant tumor that arises from small cutaneous nerves, is locally aggressive, and has a potential for metastasis. Characteristic histopathologic features include proliferating atypical spindle cells with slender wavy and pointed nuclei, hypocellular areas, and areas featuring organized whorls of fibroblastic proliferation. The most common primary sites are the extremities, retroperitoneum, and trunk. These tumors tend to present in childhood, often in association with NEUROFIBROMATOSIS 1. (From DeVita et al., Cancer: Principles & Practice of Oncology, 5th ed, p1662; Mayo Clin Proc 1990 Feb;65(2):164-72)'),
  'D018320': NtO(MH='Leishmania major', MS='A parasitic hemoflagellate of the subgenus Leishmania leishmania that infects man and animals and causes cutaneous leishmaniasis (LEISHMANIASIS, CUTANEOUS) of the Old World. Transmission is by Phlebotomus sandflies.'),
  'D018321': NtO(MH='Neurothekeoma', MS='A benign myxoma of cutaneous nerve sheath origin. Theke is from the Greek theke, sheath. (From Stedman, 25th ed)'),
  'D018322': NtO(MH='Angiofibroma', MS='A benign neoplasm of fibrous tissue in which there are numerous small and large, frequently dilated, vascular channels. (Stedman, 25th ed)'),
  'D018323': NtO(MH='Hemangioendothelioma, Epithelioid', MS='A tumor of medium-to-large veins, composed of plump-to-spindled endothelial cells that bulge into vascular spaces in a tombstone-like fashion. These tumors are thought to have "borderline" aggression, where one-third develop local recurrences, but only rarely metastasize. It is unclear whether the epithelioid hemangioendothelioma is truly neoplastic or an exuberant tissue reaction, nor is it clear if this is equivalent to Kimura\'s disease (see ANGIOLYMPHOID HYPERPLASIA WITH EOSINOPHILIA). (Segen, Dictionary of Modern Medicine, 1992)'),
  'D018324': NtO(MH='Hemangioma, Capillary', MS='A dull red, firm, dome-shaped hemangioma, sharply demarcated from surrounding skin, usually located on the head and neck, which grows rapidly and generally undergoes regression and involution without scarring. It is caused by proliferation of immature capillary vessels in active stroma, and is usually present at birth or occurs within the first two or three months of life. (Dorland, 27th ed)'),
  'D018325': NtO(MH='Hemangioblastoma', MS='A benign tumor of the nervous system that may occur sporadically or in association with VON HIPPEL-LINDAU DISEASE. It accounts for approximately 2% of intracranial tumors, arising most frequently in the cerebellar hemispheres and vermis. Histologically, the tumors are composed of multiple capillary and sinusoidal channels lined with endothelial cells and clusters of lipid-laden pseudoxanthoma cells. Usually solitary, these tumors can be multiple and may also occur in the brain stem, spinal cord, retina, and supratentorial compartment. Cerebellar hemangioblastomas usually present in the third decade with INTRACRANIAL HYPERTENSION, and ataxia. (From DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, pp2071-2)'),
  'D018326': NtO(MH='Nevi and Melanomas', MS='A collective term for the various types of nevi and melanomas.'),
  'D018327': NtO(MH="Hutchinson's Melanotic Freckle", MS='A cellular subtype of malignant melanoma. It is a pigmented lesion composed of melanocytes occurring on sun-exposed skin, usually the face and neck. The melanocytes are commonly multinucleated with a "starburst" appearance. It is considered by many to be the in situ phase of lentigo maligna melanoma.'),
  'D018328': NtO(MH='Melanoma, Amelanotic', MS='An unpigmented malignant melanoma. It is an anaplastic melanoma consisting of cells derived from melanoblasts but not forming melanin. (Dorland, 27th ed; Stedman, 25th ed)'),
  'D018329': NtO(MH='Nevus, Blue', MS='Usually a benign tumor, that commonly presents as a solitary blue nodule with spindled MELANOCYTES covered by smooth SKIN. Several variants have been identified, one variant being malignant. The blue color is caused by large, densely packed melanocytes deep in the DERMIS of the nevus. In CHILDREN, they usually occur on the BUTTOCKS and LUMBOSACRAL REGION and are referred to as cellular blue nevi. Malignant blue nevi are more commonly found on the SCALP.'),
  'D018330': NtO(MH='Nevus, Intradermal', MS='A nevus in which nests of melanocytes are found in the dermis, but not at the epidermal-dermal junction. Benign pigmented nevi in adults are most commonly intradermal. (Stedman, 25th ed)'),
  'D018331': NtO(MH='Nevus, Spindle Cell', MS='A form of pigmented nevus showing intense melanocytic activity around the dermo-epidermal junction. Large numbers of spindle-shaped melanocytes proliferate downward toward the dermis and usually a large amount of pigment is present. It was first described in 1976 and the bulk of patients reported have been young females with the lesions presenting on the thighs. (From Rook et al., Textbook of Dermatology, 4th ed, 1992, p185)'),
  'D018332': NtO(MH='Nevus, Epithelioid and Spindle Cell', MS='A benign compound nevus occurring most often in children before puberty, composed of spindle and epithelioid cells located mainly in the dermis, sometimes in association with large atypical cells and multinucleate cells, and having a close histopathological resemblance to malignant melanoma. The tumor presents as a smooth to slightly scaly, round to oval, raised, firm papule or nodule, ranging in color from pink-tan to purplish red, often with surface telangiectasia. (Dorland, 27th ed)'),
  'D018333': NtO(MH='Odontogenic Cyst, Calcifying', MS='A mixed radiolucent-radiopaque lesion of the jaws with features of both a cyst and a solid neoplasm. It is characterized microscopically by an epithelial lining showing a palisaded layer of columnar basal cells, presence of ghost cell keratinization, dentinoid, and calcification. (Stedman, 25th ed)'),
  'D018334': NtO(MH='Leishmania guyanensis', MS='A parasitic hemoflagellate of the subgenus Leishmania viannia that infects man and animals and causes mucocutaneous leishmaniasis (LEISHMANIASIS, MUCOCUTANEOUS). Transmission is by Lutzomyia sandflies.'),
  'D018335': NtO(MH='Rhabdoid Tumor', MS='A rare but highly lethal childhood tumor found almost exclusively in infants. Histopathologically, it resembles RHABDOMYOSARCOMA but the tumor cells are not of myogenic origin. Although it arises primarily in the kidney, it may be found in other parts of the body. The rhabdoid cytomorphology is believed to be the expression of a very primitive malignant cell. (From Holland et al., Cancer Medicine, 3d ed, p2210)'),
  'D018336': NtO(MH='Receptors, Aryl Hydrocarbon', MS='Cytoplasmic proteins that bind certain aryl hydrocarbons, translocate to the nucleus, and activate transcription of particular DNA segments. AH receptors are identified by their high-affinity binding to several carcinogenic or teratogenic environmental chemicals including polycyclic aromatic hydrocarbons found in cigarette smoke and smog, heterocyclic amines found in cooked foods, and halogenated hydrocarbons including dioxins and polychlorinated biphenyls. No endogenous ligand has been identified, but an unknown natural messenger with a role in cell differentiation and development is suspected.'),
  'D018340': NtO(MH='Receptors, Adrenergic, alpha-1', MS='A subclass of alpha-adrenergic receptors that mediate contraction of SMOOTH MUSCLE in a variety of tissues such as ARTERIOLES; VEINS; and the UTERUS. They are usually found on postsynaptic membranes and signal through GQ-G11 G-PROTEINS.'),
  'D018341': NtO(MH='Receptors, Adrenergic, alpha-2', MS='A subclass of alpha-adrenergic receptors found on both presynaptic and postsynaptic membranes where they signal through Gi-Go G-PROTEINS. While postsynaptic alpha-2 receptors play a traditional role in mediating the effects of ADRENERGIC AGONISTS, the subset of alpha-2 receptors found on presynaptic membranes signal the feedback inhibition of NEUROTRANSMITTER release.'),
  'D018342': NtO(MH='Receptors, Adrenergic, beta-1', MS='A subclass of beta-adrenergic receptors (RECEPTORS, ADRENERGIC, BETA). The adrenergic beta-1 receptors are equally sensitive to EPINEPHRINE and NOREPINEPHRINE and bind the agonist DOBUTAMINE and the antagonist METOPROLOL with high affinity. They are found in the HEART, juxtaglomerular cells, and in the central and peripheral nervous systems.'),
  'D018343': NtO(MH='Receptors, Adrenergic, beta-2', MS='A subclass of beta-adrenergic receptors (RECEPTORS, ADRENERGIC, BETA). The adrenergic beta-2 receptors are more sensitive to EPINEPHRINE than to NOREPINEPHRINE and have a high affinity for the agonist TERBUTALINE. They are widespread, with clinically important roles in SKELETAL MUSCLE; LIVER; and vascular, bronchial, gastrointestinal, and genitourinary SMOOTH MUSCLE.'),
  'D018344': NtO(MH='T-Lymphocytopenia, Idiopathic CD4-Positive', MS='Reproducible depletion of CD4+ lymphocytes below 300 per cubic millimeter in the absence of HIV infection or other known causes of immunodeficiency. This is a rare, heterogeneous syndrome and does not appear to be caused by a transmissible agent.'),
  'D018345': NtO(MH='Mice, Knockout', MS='Strains of mice in which certain GENES of their GENOMES have been disrupted, or "knocked-out". To produce knockouts, using RECOMBINANT DNA technology, the normal DNA sequence of the gene being studied is altered to prevent synthesis of a normal gene product. Cloned cells in which this DNA alteration is successful are then injected into mouse EMBRYOS to produce chimeric mice. The chimeric mice are then bred to yield a strain in which all the cells of the mouse contain the disrupted gene. Knockout mice are used as EXPERIMENTAL ANIMAL MODELS for diseases (DISEASE MODELS, ANIMAL) and to clarify the functions of the genes.'),
  'D018347': NtO(MH='Hepadnaviridae Infections', MS='Virus diseases caused by the HEPADNAVIRIDAE.'),
  'D018349': NtO(MH='Encephalitis Viruses, Japanese', MS='A subgroup of the genus FLAVIVIRUS which comprises a number of viral species that are the etiologic agents of human encephalitis in many different geographical regions. These include Japanese encephalitis virus (ENCEPHALITIS VIRUS, JAPANESE), St. Louis encephalitis virus (ENCEPHALITIS VIRUS, ST. LOUIS), Murray Valley encephalitis virus (ENCEPHALITIS VIRUS, MURRAY VALLEY), and WEST NILE VIRUS.'),
  'D018350': NtO(MH='alpha-Amino-3-hydroxy-5-methyl-4-isoxazolepropionic Acid', MS='An IBOTENIC ACID homolog and glutamate agonist. The compound is the defining agonist for the AMPA subtype of glutamate receptors (RECEPTORS, AMPA). It has been used as a radionuclide imaging agent but is more commonly used as an experimental tool in cell biological studies.'),
  'D018352': NtO(MH='Coronavirus Infections', MS='Virus diseases caused by the CORONAVIRUS genus. Some specifics include transmissible enteritis of turkeys (ENTERITIS, TRANSMISSIBLE, OF TURKEYS); FELINE INFECTIOUS PERITONITIS; and transmissible gastroenteritis of swine (GASTROENTERITIS, TRANSMISSIBLE, OF SWINE).'),
  'D018353': NtO(MH='Rhabdoviridae Infections', MS='Virus diseases caused by RHABDOVIRIDAE. Important infections include RABIES; EPHEMERAL FEVER; and vesicular stomatitis.'),
  'D018354': NtO(MH='Alphavirus Infections', MS='Virus diseases caused by members of the ALPHAVIRUS genus of the family TOGAVIRIDAE.'),
  'D018355': NtO(MH='Rubivirus Infections', MS='Virus diseases caused by the RUBIVIRUS genus, of the family TOGAVIRIDAE.'),
  'D018357': NtO(MH='Respiratory Syncytial Virus Infections', MS='Pneumovirus infections caused by the RESPIRATORY SYNCYTIAL VIRUSES. Humans and cattle are most affected but infections in goats and sheep have been reported.'),
  'D018358': NtO(MH='Neuroendocrine Tumors', MS='Tumors whose cells possess secretory granules and originate from the neuroectoderm, i.e., the cells of the ectoblast or epiblast that program the neuroendocrine system. Common properties across most neuroendocrine tumors include ectopic hormone production (often via APUD CELLS), the presence of tumor-associated antigens, and isozyme composition.'),
  'D018360': NtO(MH='Crystallography, X-Ray', MS='The study of crystal structure using X-RAY DIFFRACTION techniques. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D018365': NtO(MH='Neoplasm, Residual', MS='Remnant of a tumor or cancer after primary, potentially curative therapy.'),
  'D018366': NtO(MH='Vasculitis, Leukocytoclastic, Cutaneous', MS='Disorder characterized by a vasculitic syndrome associated with exposure to an antigen such as a drug, infectious agent, or other foreign or endogenous substance. Its pathophysiology includes immune complex deposition and a wide range of skin lesions. Hypersensitivity or allergy is present in some but not all cases.'),
  'D018370': NtO(MH='Leukocyte-Adhesion Deficiency Syndrome', MS='Rare, autosomal recessive disorder caused by deficiency of the beta 2 integrin receptors (RECEPTORS, LEUKOCYTE-ADHESION) comprising the CD11/CD18 family of glycoproteins. The syndrome is characterized by abnormal adhesion-dependent functions, especially defective tissue emigration of neutrophils, leading to recurrent infection.'),
  'D018373': NtO(MH='Peripheral Nervous System Agents', MS='Drugs that act principally at one or more sites within the peripheral neuroeffector systems, the autonomic system, and motor nerve-skeletal system. (From Smith and Reynard, Textbook of Pharmacology, 1991, p75)'),
  'D018374': NtO(MH='LLC-PK1 Cells', MS='Epithelial cell line originally derived from porcine kidneys. It is used for pharmacologic and metabolic studies.'),
  'D018375': NtO(MH='Neutrophil Activation', MS='The process in which the neutrophil is stimulated by diverse substances, resulting in degranulation and/or generation of reactive oxygen products, and culminating in the destruction of invading pathogens. The stimulatory substances, including opsonized particles, immune complexes, and chemotactic factors, bind to specific cell-surface receptors on the neutrophil.'),
  'D018376': NtO(MH='Cardiovascular Abnormalities', MS='Congenital, inherited, or acquired anomalies of the CARDIOVASCULAR SYSTEM, including the HEART and BLOOD VESSELS.'),
  'D018377': NtO(MH='Neurotransmitter Agents', MS='Substances used for their pharmacological actions on any aspect of neurotransmitter systems. Neurotransmitter agents include agonists, antagonists, degradation inhibitors, uptake inhibitors, depleters, precursors, and modulators of receptor function.'),
  'D018380': NtO(MH='Hematopoietic Stem Cell Transplantation', MS='Transfer of HEMATOPOIETIC STEM CELLS from BONE MARROW or BLOOD between individuals within the same species (TRANSPLANTATION, HOMOLOGOUS) or transfer within the same individual (TRANSPLANTATION, AUTOLOGOUS). Hematopoietic stem cell transplantation has been used as an alternative to BONE MARROW TRANSPLANTATION in the treatment of a variety of neoplasms.'),
  'D018382': NtO(MH='Thyroid Hormone Resistance Syndrome', MS='An inherited autosomal recessive trait, characterized by peripheral resistance to THYROID HORMONES and the resulting elevation in serum levels of THYROXINE and TRIIODOTHYRONINE. This syndrome is caused by mutations of gene THRB encoding the THYROID HORMONE RECEPTORS BETA in target cells. HYPOTHYROIDISM in these patients is partly overcome by the increased thyroid hormone levels.'),
  'D018383': NtO(MH='Bay-Region, Polycyclic Aromatic Hydrocarbon', MS='A concave exterior region on some POLYCYCLIC AROMATIC HYDROCARBONS that have three phenyl rings in a non-linear arrangement.'),
  'D018384': NtO(MH='Oxidative Stress', MS='A disturbance in the prooxidant-antioxidant balance in favor of the former, leading to potential damage. Indicators of oxidative stress include damaged DNA bases, protein oxidation products, and lipid peroxidation products (Sies, Oxidative Stress, 1991, pxv-xvi).'),
  'D018385': NtO(MH='Centrosome', MS='The cell center, consisting of a pair of CENTRIOLES surrounded by a cloud of amorphous material called the pericentriolar region. During interphase, the centrosome nucleates microtubule outgrowth. The centrosome duplicates and, during mitosis, separates to form the two poles of the mitotic spindle (MITOTIC SPINDLE APPARATUS).'),
  'D018386': NtO(MH='Kinetochores', MS='Large multiprotein complexes that bind the centromeres of the chromosomes to the microtubules of the mitotic spindle during metaphase in the cell cycle.'),
  'D018387': NtO(MH='Codon, Initiator', MS='A codon that directs initiation of protein translation (TRANSLATION, GENETIC) by stimulating the binding of initiator tRNA (RNA, TRANSFER, MET). In prokaryotes, the codons AUG or GUG can act as initiators while in eukaryotes, AUG is the only initiator codon.'),
  'D018388': NtO(MH='Codon, Terminator', MS='Any codon that signals the termination of genetic translation (TRANSLATION, GENETIC). PEPTIDE TERMINATION FACTORS bind to the stop codon and trigger the hydrolysis of the aminoacyl bond connecting the completed polypeptide to the tRNA. Terminator codons do not specify amino acids.'),
  'D018389': NtO(MH='Codon, Nonsense', MS='An amino acid-specifying codon that has been converted to a stop codon (CODON, TERMINATOR) by mutation. Its occurance is abnormal causing premature termination of protein translation and results in production of truncated and non-functional proteins. A nonsense mutation is one that converts an amino acid-specific codon to a stop codon.'),
  'D018390': NtO(MH='Gene Targeting', MS='The integration of exogenous DNA into the genome of an organism at sites where its expression can be suitably controlled. This integration occurs as a result of homologous recombination.'),
  'D018392': NtO(MH='Genomic Imprinting', MS='The variable phenotypic expression of a GENE depending on whether it is of paternal or maternal origin, which is a function of the DNA METHYLATION pattern. Imprinted regions are observed to be more methylated and less transcriptionally active. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D018393': NtO(MH='Mycobacterium haemophilum', MS='A species of gram-positive, aerobic bacteria that causes granulomatous or ulcerating skin lesions in immunosuppressed persons. This organism owes its name to its requirement for growth of high levels of iron, conveniently supplied as blood, heme, or ferric ammonium citrate.'),
  'D018394': NtO(MH='CA-125 Antigen', MS='A carbohydrate antigen that occurs in tumors of the ovary as well as in breast, kidney, and gastrointestinal tract tumors and normal tissue. While it is tumor-associated, it is not tumor-specific and may have a protective function against particles and infectious agents at mucosal surfaces.'),
  'D018395': NtO(MH='CA-19-9 Antigen', MS='Sialylated Lewis(a) blood group carbohydrate antigen found in many adenocarcinomas of the digestive tract, especially pancreatic tumors.'),
  'D018396': NtO(MH='Mucin-1', MS='Carbohydrate antigen elevated in patients with tumors of the breast, ovary, lung, and prostate as well as other disorders. The mucin is expressed normally by most glandular epithelia but shows particularly increased expression in the breast at lactation and in malignancy. It is thus an established serum marker for breast cancer.'),
  'D018397': NtO(MH='Equidae', MS='A family of hoofed MAMMALS consisting of HORSES, donkeys, and zebras. Members of this family are strict herbivores and can be classified as either browsers or grazers depending on how they feed.'),
  'D018398': NtO(MH='Homeodomain Proteins', MS='Proteins encoded by homeobox genes (GENES, HOMEOBOX) that exhibit structural similarity to certain prokaryotic and eukaryotic DNA-binding proteins. Homeodomain proteins are involved in the control of gene expression during morphogenesis and development (GENE EXPRESSION REGULATION, DEVELOPMENTAL).'),
  'D018401': NtO(MH='Sample Size', MS='The number of units (persons, animals, patients, specified circumstances, etc.) in a population to be studied. The sample size should be big enough to have a high likelihood of detecting a true difference between two groups. (From Wassertheil-Smoller, Biostatistics and Epidemiology, 1990, p95)'),
  'D018402': NtO(MH='Manduca', MS='A genus of sphinx or hawk moths of the family Sphingidae. These insects are used in molecular biology studies during all stages of their life cycle.'),
  'D018404': NtO(MH='Isochromosomes', MS='Metacentric chromosomes produced during MEIOSIS or MITOSIS when the CENTROMERE splits transversely instead of longitudinally. The chromosomes produced by this abnormal division are one chromosome having the two long arms of the original chromosome, but no short arms, and the other chromosome consisting of the two short arms and no long arms. Each of these isochromosomes constitutes a simultaneous duplication and deletion.'),
  'D018406': NtO(MH='Virus Physiological Phenomena', MS='Biological properties, processes, and activities of VIRUSES, including the interactions with the cells they infect.'),
  'D018407': NtO(MH='Bacterial Physiological Phenomena', MS='Physiological processes and properties of BACTERIA.'),
  'D018408': NtO(MH='Patch-Clamp Techniques', MS='An electrophysiologic technique for studying cells, cell membranes, and occasionally isolated organelles. All patch-clamp methods rely on a very high-resistance seal between a micropipette and a membrane; the seal is usually attained by gentle suction. The four most common variants include on-cell patch, inside-out patch, outside-out patch, and whole-cell clamp. Patch-clamp methods are commonly used to voltage clamp, that is control the voltage across the membrane and measure current flow, but current-clamp methods, in which the current is controlled and the voltage is measured, are also used.'),
  'D018409': NtO(MH='Foot Injuries', MS='General or unspecified injuries involving the foot.'),
  'D018410': NtO(MH='Pneumonia, Bacterial', MS='Inflammation of the lung parenchyma that is caused by bacterial infections.'),
  'D018411': NtO(MH='Spodoptera', MS='A genus of owlet moths of the family Noctuidae. These insects are used in molecular biology studies during all stages of their life cycle.'),
  'D018413': NtO(MH='Bartonella quintana', MS='A species of gram-negative bacteria in which man is the primary host and the human body louse, Pediculus humanus, the principal vector. It is the etiological agent of TRENCH FEVER.'),
  'D018414': NtO(MH='CD8-Positive T-Lymphocytes', MS='A critical subpopulation of regulatory T-lymphocytes involved in MHC Class I-restricted interactions. They include both cytotoxic T-lymphocytes (T-LYMPHOCYTES, CYTOTOXIC) and CD8+ suppressor T-lymphocytes.'),
  'D018416': NtO(MH='Bartonella henselae', MS='A species of gram-negative bacteria that is the etiologic agent of bacillary angiomatosis (ANGIOMATOSIS, BACILLARY). This organism can also be a cause of CAT-SCRATCH DISEASE in immunocompetent patients.'),
  'D018417': NtO(MH='Th1 Cells', MS='A subset of helper-inducer T-lymphocytes which synthesize and secrete INTERLEUKIN-2; INTERFERON-GAMMA; and INTERLEUKIN-12. Due to their ability to kill antigen-presenting cells and their lymphokine-mediated effector activity, Th1 cells are associated with vigorous delayed-type hypersensitivity reactions.'),
  'D018418': NtO(MH='Th2 Cells', MS='A subset of helper-inducer T-lymphocytes which synthesize and secrete the INTERLEUKINS IL-4; IL-5; IL-6; and IL-10. These cytokines influence B-cell development and antibody production as well as augmenting humoral responses.'),
  'D018419': NtO(MH='Primate Diseases', MS='Diseases of animals within the order PRIMATES. This term includes diseases of Haplorhini and Strepsirhini.'),
  'D018420': NtO(MH='Ape Diseases', MS='Diseases of chimpanzees, gorillas, and orangutans.'),
  'D018421': NtO(MH='Cardiomyoplasty', MS='An operation that uses stimulated latissimus dorsi muscle (SKELETAL MUSCLE VENTRICLE) to assist cardiac function. The latissimus dorsi muscle is mobilized from the chest wall and moved into the thorax through the bed of the resected 2nd or 3rd rib. The muscle is then wrapped around the left and right ventricles and stimulated to contract during cardiac systole by means of an implanted burst-stimulator. (Stedman, 26th ed)'),
  'D018424': NtO(MH='Cholesteatoma, Middle Ear', MS='A mass of KERATIN-producing squamous EPITHELIUM that resembles an inverted (suck-in) bag of skin in the MIDDLE EAR. It arises from the eardrum (TYMPANIC MEMBRANE) and grows into the MIDDLE EAR causing erosion of EAR OSSICLES and MASTOID that contains the INNER EAR.'),
  'D018426': NtO(MH='SEER Program', MS='A cancer registry mandated under the National Cancer Act of 1971 to operate and maintain a population-based cancer reporting system, reporting periodically estimates of cancer incidence and mortality in the United States. The Surveillance, Epidemiology, and End Results (SEER) Program is a continuing project of the National Cancer Institute of the National Institutes of Health. Among its goals, in addition to assembling and reporting cancer statistics, are the monitoring of annual cancer incident trends and the promoting of studies designed to identify factors amenable to cancer control interventions. (From National Cancer Institute, NIH Publication No. 91-3074, October 1990)'),
  'D018427': NtO(MH='Breast Implants', MS='Implants used to reconstruct and/or cosmetically enhance the female breast. They have an outer shell or envelope of silicone elastomer and are filled with either saline or silicone gel. The outer shell may be either smooth or textured.'),
  'D018428': NtO(MH='Drosophilidae', MS='A family of the order DIPTERA. These flies are generally found around decaying vegetation and fruit. Several species, because of their short life span, giant salivary gland chromosomes, and ease of culturing, have been used extensively in studies of heredity.'),
  'D018429': NtO(MH='Glossinidae', MS='A family of the order DIPTERA that includes the TSETSE FLIES. These flies occur only in Africa.'),
  'D018431': NtO(MH='Newspaper Article', MS='Work consisting of a news item appearing in a general-interest newspaper or other general news periodical, containing information of current and timely interest in the field of medicine or science. This publication type should not be confused with NEWS Publication Type, reserved for news reports published in various medical or other scientific journals, such as "Nature".'),
  'D018432': NtO(MH='Drug Resistance, Multiple', MS='Simultaneous resistance to several structurally and functionally distinct drugs.'),
  'D018433': NtO(MH='Twin Studies as Topic', MS='Works about methods of detecting genetic etiology in human traits. The basic premise of twin studies is that monozygotic twins carry identical genes, while zygotic twins are genetically no more similar than two siblings born after separate pregnancies. (Last, J.M., A Dictionary of Epidemiology, 2d ed)'),
  'D018434': NtO(MH='Sickness Impact Profile', MS="A quality-of-life scale developed in the United States in 1972 as a measure of health status or dysfunction generated by a disease. It is a behaviorally based questionnaire for patients and addresses activities such as sleep and rest, mobility, recreation, home management, emotional behavior, social interaction, and the like. It measures the patient's perceived health status and is sensitive enough to detect changes or differences in health status occurring over time or between groups. (From Medical Care, vol.xix, no.8, August 1981, p.787-805)"),
  'D018435': NtO(MH='ATP Binding Cassette Transporter, Subfamily B', MS='A subfamily of transmembrane proteins from the superfamily of ATP-BINDING CASSETTE TRANSPORTERS that are closely related in sequence to ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B, MEMBER 1. When overexpressed, they function as ATP-dependent efflux pumps able to extrude lipophilic drugs (especially ANTINEOPLASTIC AGENTS) from cells, causing multidrug resistance (DRUG RESISTANCE, MULTIPLE). Although ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B share functional similarities to MULTIDRUG RESISTANCE-ASSOCIATED PROTEINS they are two distinct subclasses of ATP-BINDING CASSETTE TRANSPORTERS, and have little sequence homology.'),
  'D018436': NtO(MH='Ancient Lands', MS='Geographical sites known to be extant in a remote period in the history of civilization, familiar as the names of ancient countries and empires.'),
  'D018437': NtO(MH='Brown-Sequard Syndrome', MS='A syndrome associated with injury to the lateral half of the spinal cord. The condition is characterized by the following clinical features (which are found below the level of the lesion): contralateral hemisensory anesthesia to pain and temperature, ipsilateral loss of propioception, and ipsilateral motor paralysis. Tactile sensation is generally spared. (From Adams et al., Principles of Neurology, 6th ed, p162).'),
  'D018438': NtO(MH='Blue Toe Syndrome', MS='A condition that is caused by recurring atheroembolism in the lower extremities. It is characterized by cyanotic discoloration of the toes, usually the first, fourth, and fifth toes. Discoloration may extend to the lateral aspect of the foot. Despite the gangrene-like appearance, blue toes may respond to conservative therapy without amputation.'),
  'D018440': NtO(MH='beta-Lactam Resistance', MS='Nonsusceptibility of bacteria to the action of the beta-lactam antibiotics. Mechanisms responsible for beta-lactam resistance may be degradation of antibiotics by BETA-LACTAMASES, failure of antibiotics to penetrate, or low-affinity binding of antibiotics to targets.'),
  'D018441': NtO(MH='Biofilms', MS='Encrustations formed from microbes (bacteria, algae, fungi, plankton, or protozoa) embedded in an EXTRACELLULAR POLYMERIC SUBSTANCE MATRIX that is secreted by the microbes. They occur on body surfaces such as teeth (DENTAL DEPOSITS); inanimate objects, and bodies of water. Biofilms are prevented from forming by treating surfaces with DENTIFRICES; DISINFECTANTS; ANTI-INFECTIVE AGENTS; and anti-fouling agents.'),
  'D018442': NtO(MH='Lymphoma, B-Cell, Marginal Zone', MS='Extranodal lymphoma of lymphoid tissue associated with mucosa that is in contact with exogenous antigens. Many of the sites of these lymphomas, such as the stomach, salivary gland, and thyroid, are normally devoid of lymphoid tissue. They acquire mucosa-associated lymphoid tissue (MALT) type as a result of an immunologically mediated disorder.'),
  'D018445': NtO(MH='Infectious Disease Transmission, Vertical', MS='The transmission of infectious disease or pathogens from one generation to another. It includes transmission in utero or intrapartum by exposure to blood and secretions, and postpartum exposure via breastfeeding.'),
  'D018447': NtO(MH='Medical Futility', MS='The absence of a useful purpose or useful result in a diagnostic procedure or therapeutic intervention. The situation of a patient whose condition will not be improved by treatment or instances in which treatment preserves permanent unconsciousness or cannot end dependence on intensive medical care. (From Ann Intern Med 1990 Jun 15;112(12):949)'),
  'D018448': NtO(MH='Models, Immunological', MS='Theoretical representations that simulate the behavior or activity of immune system, processes, or phenomena. They include the use of mathematical equations, computers, and other electrical equipment.'),
  'D018449': NtO(MH='SAIDS Vaccines', MS='Vaccines or candidate vaccines designed to prevent SAIDS; (SIMIAN ACQUIRED IMMUNODEFICIENCY SYNDROME); and containing inactivated SIMIAN IMMUNODEFICIENCY VIRUS or type D retroviruses or some of their component antigens.'),
  'D018450': NtO(MH='Disease Progression', MS='The worsening of a disease over time. This concept is most often used for chronic and incurable diseases where the stage of the disease is an important determinant of therapy and prognosis.'),
  'D018451': NtO(MH='Homosexuality, Male', MS='Sexual attraction or relationship between males.'),
  'D018452': NtO(MH='Homosexuality, Female', MS='Sexual attraction or relationship between females.'),
  'D018453': NtO(MH='Homeless Youth', MS='Runaway and homeless children and adolescents living on the streets of cities and having no fixed place of residence.'),
  'D018454': NtO(MH='Spouses', MS='Married persons, i.e., husbands and wives, or partners. Domestic partners, or spousal equivalents, are two adults who have chosen to share their lives in an intimate and committed relationship, reside together, and share a mutual obligation of support for the basic necessities of life.'),
  'D018455': NtO(MH='Protein S Deficiency', MS="An autosomal dominant disorder showing decreased levels of plasma protein S antigen or activity, associated with venous thrombosis and pulmonary embolism. PROTEIN S is a vitamin K-dependent plasma protein that inhibits blood clotting by serving as a cofactor for activated PROTEIN C (also a vitamin K-dependent protein), and the clinical manifestations of its deficiency are virtually identical to those of protein C deficiency. Treatment with heparin for acute thrombotic processes is usually followed by maintenance administration of coumarin drugs for the prevention of recurrent thrombosis. (From Harrison's Principles of Internal Medicine, 12th ed, p1511; Wintrobe's Clinical Hematology, 9th ed, p1523)"),
  'D018456': NtO(MH='Reproductive Medicine', MS='A medical-surgical specialty concerned with the morphology, physiology, biochemistry, and pathology of reproduction in man and other animals, and on the biological, medical, and veterinary problems of fertility and lactation. It includes ovulation induction, diagnosis of infertility and recurrent pregnancy loss, and assisted reproductive technologies such as embryo transfer, in vitro fertilization, and intrafallopian transfer of zygotes. (From Infertility and Reproductive Medicine Clinics of North America, Foreword 1990; Journal of Reproduction and Fertility, Notice to Contributors, Jan 1979)'),
  'D018457': NtO(MH='Placenta, Retained', MS='A placenta that fails to be expelled after BIRTH of the FETUS. A PLACENTA is retained when the UTERUS fails to contract after the delivery of its content, or when the placenta is abnormally attached to the MYOMETRIUM.'),
  'D018458': NtO(MH='Persistent Vegetative State', MS='Vegetative state refers to the neurocognitive status of individuals with severe brain damage, in whom physiologic functions (sleep-wake cycles, autonomic control, and breathing) persist, but awareness (including all cognitive function and emotion) is abolished.'),
  'D018459': NtO(MH='Lichen Sclerosus et Atrophicus', MS="A chronic inflammatory mucocutaneous disease usually affecting the female genitalia (VULVAR LICHEN SCLEROSUS) and BALANITIS XEROTICA OBLITERANS in males. It is also called white spot disease and Csillag's disease."),
  'D018460': NtO(MH='Fractures, Comminuted', MS='A fracture in which the bone is splintered or crushed into a number of pieces.'),
  'D018461': NtO(MH='Soft Tissue Infections', MS='Infections of non-skeletal tissue, i.e., exclusive of bone, ligaments, cartilage, and fibrous tissue. The concept is usually referred to as skin and soft tissue infections and usually subcutaneous and muscle tissue are involved. The predisposing factors in anaerobic infections are trauma, ischemia, and surgery. The organisms often derive from the fecal or oral flora, particularly in wounds associated with intestinal surgery, decubitus ulcer, and human bites. (From Cecil Textbook of Medicine, 19th ed, p1688)'),
  'D018462': NtO(MH='Photobiology', MS='The branch of biology dealing with the effect of light on organisms.'),
  'D018463': NtO(MH='Filtering Surgery', MS='A surgical procedure used in treatment of glaucoma in which an opening is created through which aqueous fluid may pass from the anterior chamber into a sac created beneath the conjunctiva, thus lowering the pressure within the eye. (Hoffman, Pocket Glossary of Ophthalmologic Terminology, 1989)'),
  'D018465': NtO(MH='Molteno Implants', MS='Devices implanted to control intraocular pressure by allowing aqueous fluid to drain from the anterior chamber. (Hoffman, Pocket Glossary of Ophthalmologic Terminology, 1989)'),
  'D018466': NtO(MH='Gravity Sensing', MS='Process whereby a cell, bodily structure, or organism (animal or plant) receives or detects a gravity stimulus. Gravity sensing plays an important role in the directional growth and development of an organism (GRAVITROPISM).'),
  'D018467': NtO(MH='Positive-Pressure Respiration, Intrinsic', MS="Non-therapeutic positive end-expiratory pressure occurring frequently in patients with severe airway obstruction. It can appear with or without the administration of external positive end-expiratory pressure (POSITIVE-PRESSURE RESPIRATION). It presents an important load on the inspiratory muscles which are operating at a mechanical disadvantage due to hyperinflation. Auto-PEEP may cause profound hypotension that should be treated by intravascular volume expansion, increasing the time for expiration, and/or changing from assist mode to intermittent mandatory ventilation mode. (From Harrison's Principles of Internal Medicine, 12th ed, p1127)"),
  'D018469': NtO(MH='Ehrlichia chaffeensis', MS='A species of gram-negative bacteria that is the causative agent of human EHRLICHIOSIS. This organism was first discovered at Fort Chaffee, Arkansas, when blood samples from suspected human ehrlichiosis patients were studied.'),
  'D018470': NtO(MH='Gravity, Altered', MS='A change in, or manipulation of, gravitational force. This may be a natural or artificial effect.'),
  'D018471': NtO(MH='Hypergravity', MS='Condition wherein the force of gravity is greater than or is increased above that on the surface of the earth. This is expressed as being greater than 1 g.'),
  'D018472': NtO(MH='Hypogravity', MS='Condition wherein the force of gravity is less than or is decreased below that on the surface of the earth. This is expressed as being between 0 and 1 g.'),
  'D018474': NtO(MH='Weightlessness Simulation', MS='Condition under normal Earth gravity where the force of gravity itself is not actually altered but its influence or effect may be modified and studied. (From ASGSB Bull 1992;5(2):27)'),
  'D018475': NtO(MH='Head-Down Tilt', MS='Posture while lying with the head lower than the rest of the body. Extended time in this position is associated with temporary physiologic disturbances.'),
  'D018476': NtO(MH='Hypokinesia', MS='Slow or diminished movement of body musculature. It may be associated with BASAL GANGLIA DISEASES; MENTAL DISORDERS; prolonged inactivity due to illness; and other conditions.'),
  'D018478': NtO(MH='Extravehicular Activity', MS='Activities by crew members conducted outside the pressurized hull of a spacecraft.'),
  'D018479': NtO(MH='Early Intervention, Educational', MS='Procedures and programs that facilitate the development or skill acquisition in infants and young children who have disabilities, who are at risk for developing disabilities, or who are gifted. It includes programs that are designed to prevent handicapping conditions in infants and young children and family-centered programs designed to affect the functioning of infants and children with special needs. (From Journal of Early Intervention, Editorial, 1989, vol. 13, no. 1, p. 3; A Discursive Dictionary of Health Care, prepared for the U.S. House of Representatives Committee on Interstate and Foreign Commerce, 1976)'),
  'D018480': NtO(MH='Astronauts', MS='Members of spacecraft crew including those who travel in space, and those in training for space flight.'),
  'D018481': NtO(MH='Convection', MS='Transmission of energy or mass by a medium involving movement of the medium itself. The circulatory movement that occurs in a fluid at a nonuniform temperature owing to the variation of its density and the action of gravity. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed; Webster, 10th ed)'),
  'D018482': NtO(MH='Muscle, Skeletal', MS='A subtype of striated muscle, attached by TENDONS to the SKELETON. Skeletal muscles are innervated and their movement can be consciously controlled. They are also called voluntary muscles.'),
  'D018483': NtO(MH='Diaphyses', MS='The shaft of long bones.'),
  'D018485': NtO(MH='Muscle Fibers, Skeletal', MS='Large, multinucleate single cells, either cylindrical or prismatic in shape, that form the basic unit of SKELETAL MUSCLE. They consist of MYOFIBRILS enclosed within and attached to the SARCOLEMMA. They are derived from the fusion of skeletal myoblasts (MYOBLASTS, SKELETAL) into a syncytium, followed by differentiation.'),
  'D018486': NtO(MH='Twin Study', MS='Work consisting of reporting using a method of detecting genetic causes in human traits and genetic factors in behavior using sets of twins.'),
  'D018487': NtO(MH='Ventricular Dysfunction, Left', MS='A condition in which the LEFT VENTRICLE of the heart was functionally impaired. This condition usually leads to HEART FAILURE; MYOCARDIAL INFARCTION; and other cardiovascular complications. Diagnosis is made by measuring the diminished ejection fraction and a depressed level of motility of the left ventricular wall.'),
  'D018488': NtO(MH='Bone Demineralization, Pathologic', MS='Decrease, loss, or removal of the mineral constituents of bones. Temporary loss of bone mineral content is especially associated with space flight, weightlessness, and extended immobilization. OSTEOPOROSIS is permanent, includes reduction of total bone mass, and is associated with increased rate of fractures. CALCIFICATION, PHYSIOLOGIC is the process of bone remineralizing. (From Dorland, 27th ed; Stedman, 25th ed; Nicogossian, Space Physiology and Medicine, 2d ed, pp327-33)'),
  'D018489': NtO(MH='Space Motion Sickness', MS="Disorder characterized by nausea, vomiting, and dizziness, possibly in response to vestibular disorientation or fluid shifts associated with space flight. (From Webster's New World Dictionary)"),
  'D018490': NtO(MH='Serotonin Agents', MS='Drugs used for their effects on serotonergic systems. Among these are drugs that affect serotonin receptors, the life cycle of serotonin, and the survival of serotonergic neurons.'),
  'D018491': NtO(MH='Dopamine Agonists', MS='Drugs that bind to and activate dopamine receptors.'),
  'D018492': NtO(MH='Dopamine Antagonists', MS='Drugs that bind to but do not activate DOPAMINE RECEPTORS, thereby blocking the actions of dopamine or exogenous agonists. Many drugs used in the treatment of psychotic disorders (ANTIPSYCHOTIC AGENTS) are dopamine antagonists, although their therapeutic effects may be due to long-term adjustments of the brain rather than to the acute effects of blocking dopamine receptors. Dopamine antagonists have been used for several other clinical purposes including as ANTIEMETICS, in the treatment of Tourette syndrome, and for hiccup. Dopamine receptor blockade is associated with NEUROLEPTIC MALIGNANT SYNDROME.'),
  'D018493': NtO(MH='Cardiovascular Deconditioning', MS='A change in cardiovascular function resulting in a reduction in BLOOD VOLUME, and reflex DIURESIS. It occurs frequently after actual or simulated WEIGHTLESSNESS.'),
  'D018494': NtO(MH='Histamine Agents', MS='Drugs used for their actions on histaminergic systems. Included are drugs that act at histamine receptors, affect the life cycle of histamine, or affect the state of histaminergic cells.'),
  'D018495': NtO(MH='Fluid Shifts', MS='Translocation of body fluids from one compartment to another, such as from the vascular to the interstitial compartments. Fluid shifts are associated with profound changes in vascular permeability and WATER-ELECTROLYTE IMBALANCE. The shift can also be from the lower body to the upper body as in conditions of weightlessness.'),
  'D018496': NtO(MH='Hyperoxia', MS='An abnormal increase in the amount of oxygen in the tissues and organs.'),
  'D018497': NtO(MH='Ventricular Dysfunction, Right', MS='A condition in which the RIGHT VENTRICLE of the heart was functionally impaired. This condition usually leads to HEART FAILURE or MYOCARDIAL INFARCTION, and other cardiovascular complications. Diagnosis is made by measuring the diminished ejection fraction and a depressed level of motility of the right ventricular wall.'),
  'D018499': NtO(MH='Linear Energy Transfer', MS='Rate of energy dissipation along the path of charged particles. In radiobiology and health physics, exposure is measured in kiloelectron volts per micrometer of tissue (keV/micrometer T).'),
  'D018500': NtO(MH='Diabetes Insipidus, Nephrogenic', MS='A genetic or acquired polyuric disorder characterized by persistent hypotonic urine and HYPOKALEMIA. This condition is due to renal tubular insensitivity to VASOPRESSIN and failure to reduce urine volume. It may be the result of mutations of genes encoding VASOPRESSIN RECEPTORS or AQUAPORIN-2; KIDNEY DISEASES; adverse drug effects; or complications from PREGNANCY.'),
  'D018501': NtO(MH='Antirheumatic Agents', MS='Drugs that are used to treat RHEUMATOID ARTHRITIS.'),
  'D018502': NtO(MH='Streptococcus equi', MS='A species of gram-positive, coccoid bacteria isolated from abscesses in submaxillary glands and mucopurulent discharges of the upper respiratory tract of horses. This organism belongs to Group C streptococci with regards to antigen response and is known to cause strangles. The subspecies S. zooepidemicus is also considered a pathogen of horses.'),
  'D018503': NtO(MH='Genome, Protozoan', MS='The complete genetic complement contained in a set of CHROMOSOMES in a protozoan.'),
  'D018504': NtO(MH='Ecological Systems, Closed', MS='Systems that provide for the maintenance of life in an isolated living chamber through reutilization of the material available, in particular, by means of a cycle wherein exhaled carbon dioxide, urine, and other waste matter are converted chemically or by photosynthesis into oxygen, water, and food. (NASA Thesaurus, 1988)'),
  'D018505': NtO(MH='Waste Management', MS='Disposal, processing, controlling, recycling, and reusing the solid, liquid, and gaseous wastes of plants, animals, humans, and other organisms. It includes control within a closed ecological system to maintain a habitable environment.'),
  'D018506': NtO(MH='Gene Expression Regulation, Plant', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control of gene action in plants.'),
  'D018507': NtO(MH='Gene Expression Regulation, Developmental', MS='Any of the processes by which nuclear, cytoplasmic, or intercellular factors influence the differential control of gene action during the developmental stages of an organism.'),
  'D018508': NtO(MH='Water Purification', MS='Any of several processes in which undesirable impurities in water are removed or neutralized; for example, chlorination, filtration, primary treatment, ion exchange, and distillation. It includes treatment of WASTE WATER to provide potable and hygienic water in a controlled or closed environment as well as provision of public drinking water supplies.'),
  'D018509': NtO(MH='DNA, Helminth', MS='Deoxyribonucleic acid that makes up the genetic material of helminths.'),
  'D018510': NtO(MH='RNA, Helminth', MS='Ribonucleic acid in helminths having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D018511': NtO(MH='Systems Integration', MS='The procedures involved in combining separately developed modules, components, or subsystems so that they work together as a complete system. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D018512': NtO(MH='Parasitemia', MS='The presence of parasites (especially malarial parasites) in the blood. (Dorland, 27th ed)'),
  'D018514': NtO(MH='Plant Structures', MS='The parts of plants, including SEEDS.'),
  'D018515': NtO(MH='Plant Leaves', MS='Expanded structures, usually green, of vascular plants, characteristically consisting of a bladelike expansion attached to a stem, and functioning as the principal organ of photosynthesis and transpiration. (American Heritage Dictionary, 2d ed)'),
  'D018516': NtO(MH='Pulvinus', MS='A group of cells at the base of a leaf in certain plants that, by rapidly losing water, brings about changes in the position of the leaves. (Concise Dictionary of Biology, 1990)'),
  'D018517': NtO(MH='Plant Roots', MS='The usually underground portions of a plant that serve as support, store food, and through which water and mineral nutrients enter the plant. (From American Heritage Dictionary, 1982; Concise Dictionary of Biology, 1990)'),
  'D018518': NtO(MH='Plant Root Cap', MS='A cone-shaped structure in plants made up of a mass of meristematic cells that covers and protects the tip of a growing root. It is the putative site of gravity sensing in plant roots.'),
  'D018519': NtO(MH='Meristem', MS='A group of plant cells that are capable of dividing infinitely and whose main function is the production of new growth at the growing tip of a root or stem. (From Concise Dictionary of Biology, 1990)'),
  'D018520': NtO(MH='Plant Shoots', MS='New immature growth of a plant including stem, leaves, tips of branches, and SEEDLINGS.'),
  'D018521': NtO(MH='Plant Physiological Phenomena', MS='The physiological processes, properties, and states characteristic of plants.'),
  'D018522': NtO(MH='Gravitropism', MS='The directional growth of organisms in response to gravity. In plants, the main root is positively gravitropic (growing downwards) and a main stem is negatively gravitropic (growing upwards), irrespective of the positions in which they are placed. Plant gravitropism is thought to be controlled by auxin (AUXINS), a plant growth substance. (From Concise Dictionary of Biology, 1990)'),
  'D018523': NtO(MH='Tropism', MS='The directional growth of an organism in response to an external stimulus such as light, touch, or gravity. Growth towards the stimulus is a positive tropism; growth away from the stimulus is a negative tropism. (From Concise Dictionary of Biology, 1990)'),
  'D018524': NtO(MH='Phototropism', MS='The directional growth of organisms in response to light. In plants, aerial shoots usually grow towards light. The phototropic response is thought to be controlled by auxin (= AUXINS), a plant growth substance. (From Concise Dictionary of Biology, 1990)'),
  'D018525': NtO(MH='Germination', MS='The initial stages of the growth of SEEDS into a SEEDLINGS. The embryonic shoot (plumule) and embryonic PLANT ROOTS (radicle) emerge and grow upwards and downwards respectively. Food reserves for germination come from endosperm tissue within the seed and/or from the seed leaves (COTYLEDON). (Concise Dictionary of Biology, 1990)'),
  'D018526': NtO(MH='Plant Transpiration', MS='The loss of water vapor by plants to the atmosphere. It occurs mainly from the leaves through pores (stomata) whose primary function is gas exchange. The water is replaced by a continuous column of water moving upwards from the roots within the xylem vessels. (Concise Dictionary of Biology, 1990)'),
  'D018527': NtO(MH='Hydroponics', MS='A technique for growing plants in culture solutions rather than in soil. The roots are immersed in an aerated solution containing the correct proportions of essential mineral salts. (From Concise Dictionary of Biology, 1990)'),
  'D018528': NtO(MH='ATP-Binding Cassette Transporters', MS='A family of MEMBRANE TRANSPORT PROTEINS that require ATP hydrolysis for the transport of substrates across membranes. The protein family derives its name from the ATP-binding domain found on the protein.'),
  'D018529': NtO(MH='Nutritional Support', MS='The administration of nutrients for assimilation and utilization by a patient by means other than normal eating. It does not include FLUID THERAPY which normalizes body fluids to restore WATER-ELECTROLYTE BALANCE.'),
  'D018530': NtO(MH='Gastric Stump', MS='That portion of the stomach remaining after gastric surgery, usually gastrectomy or gastroenterostomy for cancer of the stomach or peptic ulcer. It is a common site of cancer referred to as stump cancer or carcinoma of the gastric stump.'),
  'D018531': NtO(MH='Spacecraft', MS='Devices, manned and unmanned, which are designed to be placed into an orbit about the Earth or into a trajectory to another celestial body. (NASA Thesaurus, 1988)'),
  'D018532': NtO(MH='Peas', MS="A variable annual leguminous vine (Pisum sativum) that is cultivated for its rounded smooth or wrinkled edible protein-rich seeds, the seed of the pea, and the immature pods with their included seeds. (From Webster's New Collegiate Dictionary, 1973)"),
  'D018533': NtO(MH='Biomass', MS='Total mass of all the organisms of a given type and/or in a given area. (From Concise Dictionary of Biology, 1990) It includes the yield of vegetative mass produced from any given crop.'),
  'D018534': NtO(MH='Earth, Planet', MS='Planet that is the third in order from the sun. It is one of the four inner or terrestrial planets of the SOLAR SYSTEM.'),
  'D018535': NtO(MH='Mars', MS='The fourth planet in order from the sun. Its two natural satellites are Deimos and Phobos. It is one of the four inner or terrestrial planets of the solar system.'),
  'D018536': NtO(MH='Venus', MS='The second planet in order from the sun. It has no known natural satellites. It is one of the four inner or terrestrial planets of the solar system.'),
  'D018537': NtO(MH='Mercury, Planet', MS='The first planet in order from the sun. It has no known natural satellites. It is one of the four inner or terrestrial planets of the solar system.'),
  'D018538': NtO(MH='Jupiter', MS='The fifth planet in order from the sun. It is one of the five outer planets of the solar system. Its sixteen natural satellites include Callisto, Europa, Ganymede, and Io.'),
  'D018539': NtO(MH='Saturn', MS='The sixth planet in order from the sun. It is one of the five outer planets of the solar system. Its twelve natural satellites include Phoebe and Titan.'),
  'D018540': NtO(MH='Uranus', MS='The seventh planet in order from the sun. It is one of the five outer planets of the solar system. It has five known natural satellites.'),
  'D018541': NtO(MH='Neptune', MS='The eighth planet in order from the sun. It is one of the five outer planets of the solar system. Its two natural satellites are Nereid and Triton.'),
  'D018542': NtO(MH='Pluto', MS='The ninth planet in order from the sun. It is one of the five outer planets of the solar system. Its only natural satellite is Charon.'),
  'D018543': NtO(MH='Minor Planets', MS='Small solar system planetary bodies including asteroids. Most asteroids are found within the gap lying between the orbits of Mars and Jupiter.'),
  'D018544': NtO(MH='Weightlessness Countermeasures', MS='Techniques and routines designed to prevent or reverse unwanted effects of weightlessness experienced during actual and simulated space flight, including physiologic changes related to removal of gravitational loading. Specific measures include creation of artificial gravity, exercise, low-level lower body negative pressure, and use of anti-deconditioning devices. (From Nicogossian, Space Physiology and Medicine, 2d ed, pp294-297)'),
  'D018545': NtO(MH='Lettuce', MS='Any of the various plants of the genus Lactuca, especially L. sativa, cultivated for its edible leaves. (From American Heritage Dictionary, 2d ed)'),
  'D018546': NtO(MH='Hypocotyl', MS='The region of the stem beneath the stalks of the seed leaves (cotyledons) and directly above the young root of the embryo plant. It grows rapidly in seedlings showing epigeal germination and lifts the cotyledons above the soil surface. In this region (the transition zone) the arrangement of vascular bundles in the root changes to that of the stem. (From Concise Dictionary of Biology, 1990)'),
  'D018547': NtO(MH='Plant Stems', MS='Parts of plants that usually grow vertically upwards towards the light and support the leaves, buds, and reproductive structures. (From Concise Dictionary of Biology, 1990)'),
  'D018548': NtO(MH='Cotyledon', MS='A part of the embryo in a seed plant. The number of cotyledons is an important feature in classifying plants. In seeds without an endosperm, they store food which is used in germination. In some plants, they emerge above the soil surface and become the first photosynthetic leaves. (From Concise Dictionary of Biology, 1990)'),
  'D018549': NtO(MH='Cryptogenic Organizing Pneumonia', MS='An interstitial lung disease of unknown etiology, occurring between 21-80 years of age. It is characterized by a dramatic onset of a "pneumonia-like" illness with cough, fever, malaise, fatigue, and weight loss. Pathological features include prominent interstitial inflammation without collagen fibrosis, diffuse fibroblastic foci, and no microscopic honeycomb change. There is excessive proliferation of granulation tissue within small airways and alveolar ducts.'),
  'D018550': NtO(MH='Cephalosporin Resistance', MS='Non-susceptibility of an organism to the action of the cephalosporins.'),
  'D018551': NtO(MH='Lycopersicon esculentum', MS='A plant species of the family SOLANACEAE, native of South America, widely cultivated for their edible, fleshy, usually red fruit.'),
  'D018552': NtO(MH='Daucus carota', MS='A plant species of the family APIACEAE that is widely cultivated for the edible yellow-orange root. The plant has finely divided leaves and flat clusters of small white flowers.'),
  'D018553': NtO(MH='Cucumis sativus', MS='A creeping annual plant species of the CUCURBITACEAE family. It has a rough succulent, trailing stem and hairy leaves with three to five pointed lobes.'),
  'D018554': NtO(MH='Avena', MS='Oats, genus of the family POACEAE.'),
  'D018556': NtO(MH='Crops, Agricultural', MS='Cultivated plants or agricultural produce such as grain, vegetables, or fruit. (From American Heritage Dictionary, 1982)'),
  'D018557': NtO(MH='United States National Aeronautics and Space Administration', MS="An independent Federal agency established in 1958. It conducts research for the solution of problems of flight within and outside the Earth's atmosphere and develops, constructs, tests, and operates aeronautical and space vehicles. (From U.S. Government Manual, 1993)"),
  'D018559': NtO(MH='Exobiology', MS='The interdisciplinary science that studies evolutionary biology, including the origin and evolution of the major elements required for life, their processing in the interstellar medium and in protostellar systems. This field also includes the study of chemical evolution and the subsequent interactions between evolving biota and planetary evolution as well as the field of biology that deals with the study of extraterrestrial life.'),
  'D018561': NtO(MH='Space Simulation', MS='An environment simulating one or more parameters of the space environment, applied in testing space systems or components. Often, a closed chamber is used, capable of approximating the vacuum and normal environments of space. (From NASA Thesaurus, 1988) This also includes simulated EXTRAVEHICULAR ACTIVITY studies in atmosphere exposure chambers or water tanks.'),
  'D018562': NtO(MH='Disease Transmission, Infectious', MS='The transmission of infectious disease or pathogens. When transmission is within the same species, the mode can be horizontal or vertical (INFECTIOUS DISEASE TRANSMISSION, VERTICAL).'),
  'D018563': NtO(MH='Disease Notification', MS='Notification or reporting by a physician or other health care provider of the occurrence of specified contagious diseases such as tuberculosis and HIV infections to designated public health agencies. The United States system of reporting notifiable diseases evolved from the Quarantine Act of 1878, which authorized the US Public Health Service to collect morbidity data on cholera, smallpox, and yellow fever; each state in the US has its own list of notifiable diseases and depends largely on reporting by the individual health care provider. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D018565': NtO(MH='Food-Drug Interactions', MS='The pharmacological result, either desirable or undesirable, of drugs interacting with components of the diet. (From Stedman, 25th ed)'),
  'D018566': NtO(MH='Pregnancy, High-Risk', MS='Pregnancy in which the mother and/or FETUS are at greater than normal risk of MORBIDITY or MORTALITY. Causes include inadequate PRENATAL CARE, previous obstetrical history (ABORTION, SPONTANEOUS), pre-existing maternal disease, pregnancy-induced disease (GESTATIONAL HYPERTENSION), and MULTIPLE PREGNANCY, as well as advanced maternal age above 35.'),
  'D018567': NtO(MH='Breast Neoplasms, Male', MS='Any neoplasms of the male breast. These occur infrequently in males in developed countries, the incidence being about 1% of that in females.'),
  'D018568': NtO(MH='Crown-Rump Length', MS='In utero measurement corresponding to the sitting height (crown to rump) of the fetus. Length is considered a more accurate criterion of the age of the fetus than is the weight. The average crown-rump length of the fetus at term is 36 cm. (From Williams Obstetrics, 18th ed, p91)'),
  'D018570': NtO(MH='Risk Assessment', MS='The qualitative or quantitative estimation of the likelihood of adverse effects that may result from exposure to specified health hazards or from the absence of beneficial influences. (Last, Dictionary of Epidemiology, 1988)'),
  'D018571': NtO(MH='Sentinel Surveillance', MS='Monitoring of rate of occurrence of specific conditions to assess the stability or change in health levels of a population. It is also the study of disease rates in a specific cohort such as in a geographic area or population subgroup to estimate trends in a larger population. (From Last, Dictionary of Epidemiology, 2d ed)'),
  'D018572': NtO(MH='Disease-Free Survival', MS='Period after successful treatment in which there is no appearance of the symptoms or effects of the disease.'),
  'D018574': NtO(MH='Home Care Agencies', MS="Public or private organizations that provide, either directly or through arrangements with other organizations, home health services in the patient's home. (Hospital Administration Terminology, 2d ed)"),
  'D018575': NtO(MH='Home Care Services, Hospital-Based', MS="Hospital-sponsored provision of health services, such as nursing, therapy, and health-related homemaker or social services, in the patient's home. (Hospital Administration Terminology, 2d ed)"),
  'D018576': NtO(MH='Home Health Aides', MS='Persons who assist ill, elderly, or disabled persons in the home, carrying out personal care and housekeeping tasks. (From Slee & Slee, Health Care Terms. 2d ed, p202)'),
  'D018577': NtO(MH='Homebound Persons', MS="Those unable to leave home without exceptional effort and support; patients (in this condition) who are provided with or are eligible for home health services, including medical treatment and personal care. Persons are considered homebound even if they may be infrequently and briefly absent from home if these absences do not indicate an ability to receive health care in a professional's office or health care facility. (From Facts on File Dictionary of Health Care Management, 1988, p309)"),
  'D018578': NtO(MH='Hospital-Physician Relations', MS='Includes relationships between hospitals, their governing boards, and administrators in regard to physicians, whether or not the physicians are members of the medical staff or have medical staff privileges.'),
  'D018579': NtO(MH='Patient Selection', MS='Criteria and standards used for the determination of the appropriateness of the inclusion of patients with specific conditions in proposed treatment plans and the criteria used for the inclusion of subjects in various clinical trials and other research protocols.'),
  'D018580': NtO(MH='Mandatory Testing', MS='Testing or screening required by federal, state, or local law or other agencies for the diagnosis of specified conditions. It is usually limited to specific populations such as categories of health care providers, members of the military, and prisoners or to specific situations such as premarital examinations or donor screening.'),
  'D018581': NtO(MH='Technology Transfer', MS='Spread and adoption of inventions and techniques from one geographic area to another, from one discipline to another, or from one sector of the economy to another. For example, improvements in medical equipment may be transferred from industrial countries to developing countries, advances arising from aerospace engineering may be applied to equipment for persons with disabilities, and innovations in science arising from government research are made available to private enterprise.'),
  'D018582': NtO(MH='Sick Leave', MS="An absence from work permitted because of illness or the number of days per year for which an employer agrees to pay employees who are sick. (Webster's New Collegiate Dictionary, 1981)"),
  'D018583': NtO(MH='Pliability', MS='The quality or state of being able to be bent or creased repeatedly. (From Webster, 3d ed)'),
  'D018584': NtO(MH='Physician Self-Referral', MS='Referral by physicians to testing or treatment facilities in which they have financial interest. The practice is regulated by the Ethics in Patient Referrals Act of 1989.'),
  'D018585': NtO(MH='Battered Women', MS='Women who are physically and mentally abused over an extended period, usually by a husband or other dominant male figure. Characteristics of the battered woman syndrome are helplessness, constant fear, and a perceived inability to escape. (From American Heritage Dictionary of the English Language, 3d ed)'),
  'D018586': NtO(MH='Bone Banks', MS='Centers for acquiring, characterizing, and storing bones or bone tissue for future use.'),
  'D018587': NtO(MH='Oocyte Donation', MS='Transfer of preovulatory oocytes from donor to a suitable host. Oocytes are collected, fertilized in vitro, and transferred to a host that can be human or animal.'),
  'D018588': NtO(MH='Fee-for-Service Plans', MS='Method of charging whereby a physician or other practitioner bills for each encounter or service rendered. In addition to physicians, other health care professionals are reimbursed via this mechanism. Fee-for-service plans contrast with salary, per capita, and prepayment systems, where the payment does not change with the number of services actually used or if none are used. (From Discursive Dictionary of Health Care, 1976)'),
  'D018589': NtO(MH='Gastroparesis', MS='Chronic delayed gastric emptying. Gastroparesis may be caused by motor dysfunction or paralysis of STOMACH muscles or may be associated with other systemic diseases such as DIABETES MELLITUS.'),
  'D018590': NtO(MH='Uterine Monitoring', MS='Measurement or recording of contraction activity of the uterine muscle. It is used to determine progress of LABOR, OBSTETRIC and assess status of pregnancy. It is also used in conjunction with FETAL MONITORING to determine fetal response to stress of maternal uterine contractions.'),
  'D018592': NtO(MH='Cross-Over Studies', MS='Studies comparing two or more treatments or interventions in which the subjects or patients, upon completion of the course of one treatment, are switched to another. In the case of two treatments, A and B, half the subjects are randomly allocated to receive these in the order A, B and half to receive them in the order B, A. A criticism of this design is that effects of the first treatment may carry over into the period when the second is given. (Last, A Dictionary of Epidemiology, 2d ed)'),
  'D018594': NtO(MH='Human Body', MS='The human being as a non-anatomical and non-zoological entity. The emphasis is on the philosophical or artistic treatment of the human being, and includes lay and social attitudes toward the body in history. (From J. Cassedy, NLM History of Medicine Division)'),
  'D018595': NtO(MH='Colonialism', MS='The aggregate of various economic, political, and social policies by which an imperial power maintains or extends its control over other areas or peoples. It includes the practice of or belief in acquiring and retaining colonies. The emphasis is less on its identity as an ideological political system than on its designation in a period of history. (Webster, 3d ed; from Dr. J. Cassedy, NLM History of Medicine Division)'),
  'D018596': NtO(MH='Hinduism', MS='A complex body of social, cultural, and religious beliefs and practices evolved in and largely confined to the Indian subcontinent and marked by a caste system, an outlook tending to view all forms and theories as aspects of one eternal being and truth, and the practice of the way of works, the way of knowledge, or the way of devotion as a means of release from the round of rebirths. (From Webster, 3d ed)'),
  'D018597': NtO(MH='Saints', MS='Persons officially recognized or acknowledged as pre-eminent for consecration, holiness, and piety, especially through canonization by a branch of the Christian church. (From Webster, 3d ed)'),
  'D018598': NtO(MH='Minisatellite Repeats', MS='Tandem arrays of moderately repetitive, short (10-60 bases) DNA sequences which are found dispersed throughout the GENOME, at the ends of chromosomes (TELOMERES), and clustered near telomeres. Their degree of repetition is two to several hundred at each locus. Loci number in the thousands but each locus shows a distinctive repeat unit.'),
  'D018599': NtO(MH='Witchcraft', MS='An act of employing sorcery (the use of power gained from the assistance or control of spirits), especially with malevolent intent, and the exercise of supernatural powers and alleged intercourse with the devil or a familiar. (From Webster, 3d ed)'),
  'D018600': NtO(MH='Teratology', MS='A branch of embryology for the study of congenital malformations and developmental abnormalities.'),
  'D018601': NtO(MH="King's Evil", MS='The historic designation for scrofula (TUBERCULOSIS, LYMPH NODE). The disease is so called from the belief that it could be healed by the touch of a king. This term is used only for historical articles using the name "king\'s evil", and is to be differentiated from scrofula as lymph node tuberculosis in modern clinical medicine. (From Webster, 3d ed)'),
  'D018602': NtO(MH='Milk Sickness', MS='An acute, often fatal disease caused by the ingestion of milk, milk products, or the flesh of cattle or sheep which have a disease known as trembles. It is marked by weakness, anorexia, vomiting, constipation, and sometimes muscular tremors. It is caused by poisoning by white snakeroot (Eupatorium rugosum) and the rayless goldenrod (Haplopappus heterophyllus). (From Dorland, 27th ed)'),
  'D018604': NtO(MH='Epitope Mapping', MS='Methods used for studying the interactions of antibodies with specific regions of protein antigens. Important applications of epitope mapping are found within the area of immunochemistry.'),
  'D018605': NtO(MH='Spontaneous Combustion', MS='A circumstance where a substance or organism takes fire and burns without an exogenous source. Spontaneous human combustion differs from preternatural human combustibility in that in the latter, some spark or trivial flame sets the fire and the body tissues, which have a greatly enhanced inflammability, continue to undergo incineration without any external heat source or combustible materials. (Bergman NA. Spontaneous human combustion: its role in literature and science. Pharos 1988;Fall;51(4):18-21)'),
  'D018607': NtO(MH='Pregnancy Reduction, Multifetal', MS='Selective abortion of one or more embryos or fetuses in a multiple gestation pregnancy. The usual goal is to improve the outcome for the remaining embryos or fetuses.'),
  'D018608': NtO(MH='Ultrasonography, Doppler', MS='Ultrasonography applying the Doppler effect, with frequency-shifted ultrasound reflections produced by moving targets (usually red blood cells) in the bloodstream along the ultrasound axis in direct proportion to the velocity of movement of the targets, to determine both direction and velocity of blood flow. (Stedman, 25th ed)'),
  'D018609': NtO(MH='Tibet', MS='An autonomous region located in central Asia, within China.'),
  'D018610': NtO(MH='Heart Bypass, Right', MS='Diversion of the flow of blood from the entrance to the right atrium directly to the pulmonary arteries, avoiding the right atrium and right ventricle (Dorland, 28th ed). This a permanent procedure often performed to bypass a congenitally deformed right atrium or right ventricle.'),
  'D018611': NtO(MH='Heart Bypass, Left', MS='Diversion of the flow of blood from the pulmonary veins directly to the aorta, avoiding the left atrium and the left ventricle (Dorland, 27th ed). This is a temporary procedure usually performed to assist other surgical procedures.'),
  'D018612': NtO(MH='Byzantium', MS="An ancient city, the site of modern Istanbul. From the 4th to 15th centuries the empire extended from southeastern Europe to western Asia, reaching its greatest extent under Justinian (527-565). By about 1000 A.D. it comprised the southern Balkans, Greece, Asia Minor, and parts of southern Italy. The capture of Constantinople in 1453 marked the formal end of the Byzantine Empire. (From Webster's New Geographical Dictionary, 1988)"),
  'D018613': NtO(MH='Microscopy, Confocal', MS='A light microscopic technique in which only a small spot is illuminated and observed at a time. An image is constructed through point-by-point scanning of the field in this manner. Light sources may be conventional or laser, and fluorescence or transmitted observations are possible.'),
  'D018614': NtO(MH='Sweating Sickness', MS='A clinical condition characterized by fever and profuse sweating and associated with high mortality. It occurred in epidemic form five times in the fifteenth and sixteenth centuries in England, first in 1485 and last in 1551, specially during the summer and early autumn, attacking the relatively affluent adult male population. The etiology was unknown.'),
  'D018615': NtO(MH='Ultrasonography, Doppler, Color', MS='Ultrasonography applying the Doppler effect, with the superposition of flow information as colors on a gray scale in a real-time image. This type of ultrasonography is well-suited to identifying the location of high-velocity flow (such as in a stenosis) or of mapping the extent of flow in a certain region.'),
  'D018616': NtO(MH='Ultrasonography, Doppler, Duplex', MS='Ultrasonography applying the Doppler effect combined with real-time imaging. The real-time image is created by rapid movement of the ultrasound beam. A powerful advantage of this technique is the ability to estimate the velocity of flow from the Doppler shift frequency.'),
  'D018617': NtO(MH='Ultrasonography, Doppler, Pulsed', MS='Ultrasonography applying the Doppler effect, with velocity detection combined with range discrimination. Short bursts of ultrasound are transmitted at regular intervals and the echoes are demodulated as they return.'),
  'D018618': NtO(MH='Echocardiography, Doppler, Color', MS='Echocardiography applying the Doppler effect, with the superposition of flow information as colors on a gray scale in a real-time image.'),
  'D018619': NtO(MH='Echocardiography, Doppler, Pulsed', MS='Echocardiography applying the Doppler effect, with velocity detection combined with range discrimination. Short bursts of ultrasound are transmitted at regular intervals and the echoes are demodulated as they return.'),
  'D018620': NtO(MH='PrPSc Proteins', MS='Abnormal isoform of PRION PROTEINS resulting from a posttranslational modification of the cellular prion protein (PRPC PROTEINS). PrPSc are disease-specific proteins seen in certain human and animal neurodegenerative diseases (PRION DISEASES).'),
  'D018621': NtO(MH='PrPC Proteins', MS='Normal cellular isoform of PRION PROTEINS encoded by a chromosomal gene and found in normal and scrapie-infected brain tissue, and other normal tissue. PrPC are protease-sensitive proteins whose function is unknown. Posttranslational modification of PrPC into PrPSC leads to infectivity.'),
  'D018622': NtO(MH='PrP 27-30 Protein', MS='Protease-resistant core of PrPSc, the abnormal isoform of PRION PROTEINS. PrP 27-30 is produced by limited proteolysis of the N-terminus of PrPSc.'),
  'D018625': NtO(MH='Microscopy, Atomic Force', MS='A type of scanning probe microscopy in which a probe systematically rides across the surface of a sample being scanned in a raster pattern. The vertical position is recorded as a spring attached to the probe rises and falls in response to peaks and valleys on the surface. These deflections produce a topographic map of the sample.'),
  'D018626': NtO(MH='Retroelements', MS='Elements that are transcribed into RNA, reverse-transcribed into DNA and then inserted into a new site in the genome. Long terminal repeats (LTRs) similar to those from retroviruses are contained in retrotransposons and retrovirus-like elements. Retroposons, such as LONG INTERSPERSED NUCLEOTIDE ELEMENTS and SHORT INTERSPERSED NUCLEOTIDE ELEMENTS do not contain LTRs.'),
  'D018628': NtO(MH='Gene Dosage', MS='The number of copies of a given gene present in the cell of an organism. An increase in gene dosage (by GENE DUPLICATION for example) can result in higher levels of gene product formation. GENE DOSAGE COMPENSATION mechanisms result in adjustments to the level GENE EXPRESSION when there are changes or differences in gene dosage.'),
  'D018629': NtO(MH='Spectrometry, Mass, Secondary Ion', MS='A mass-spectrometric technique that is used for microscopic chemical analysis. A beam of primary ions with an energy of 5-20 kiloelectronvolts (keV) bombards a small spot on the surface of the sample under ultra-high vacuum conditions. Positive and negative secondary ions sputtered from the surface are analyzed in a mass spectrometer in regards to their mass-to-charge ratio. Digital imaging can be generated from the secondary ion beams and their intensity can be measured. Ionic images can be correlated with images from light or other microscopy providing useful tools in the study of molecular and drug actions.'),
  'D018630': NtO(MH='Vitreoretinopathy, Proliferative', MS='Vitreoretinal membrane shrinkage or contraction secondary to the proliferation of primarily retinal pigment epithelial cells and glial cells, particularly fibrous astrocytes, followed by membrane formation. The formation of fibrillar collagen and cellular proliferation appear to be the basis for the contractile properties of the epiretinal and vitreous membranes.'),
  'D018631': NtO(MH='ras Proteins', MS='Small, monomeric GTP-binding proteins encoded by ras genes (GENES, RAS). The protooncogene-derived protein, PROTO-ONCOGENE PROTEIN P21(RAS), plays a role in normal cellular growth, differentiation and development. The oncogene-derived protein (ONCOGENE PROTEIN P21(RAS)) can play a role in aberrant cellular regulation during neoplastic cell transformation (CELL TRANSFORMATION, NEOPLASTIC). This enzyme was formerly listed as EC 3.6.1.47.'),
  'D018632': NtO(MH='Mice, Inbred SENCAR', MS='Mice selectively bred for hypersusceptibility to two-stage chemical skin carcinogenesis. They are also hypersusceptible to UV radiation tumorigenesis with single high-dose, but not chronic low-dose, exposures. SENCAR (SENsitive to CARcinogenesis) mice are used in research as an animal model for tumor production.'),
  'D018633': NtO(MH='Pulmonary Atresia', MS='A congenital heart defect characterized by the narrowing or complete absence of the opening between the RIGHT VENTRICLE and the PULMONARY ARTERY. Lacking a normal PULMONARY VALVE, unoxygenated blood in the right ventricle can not be effectively pumped into the lung for oxygenation. Clinical features include rapid breathing, CYANOSIS, right ventricle atrophy, and abnormal heart sounds (HEART MURMURS).'),
  'D018634': NtO(MH='Veterinarians', MS='Individuals with a degree in veterinary medicine that provides them with training and qualifications to treat diseases and injuries of animals.'),
  'D018635': NtO(MH='Rescue Work', MS='Activities devoted to freeing persons or animals from danger to life or well-being in accidents, fires, bombings, floods, earthquakes, other disasters and life-threatening conditions. While usually performed by team efforts, rescue work is not restricted to organized services.'),
  'D018636': NtO(MH='Hypoplastic Left Heart Syndrome', MS='A condition caused by underdevelopment of the whole left half of the heart. It is characterized by hypoplasia of the left cardiac chambers (HEART ATRIUM; HEART VENTRICLE), the AORTA, the AORTIC VALVE, and the MITRAL VALVE. Severe symptoms appear in early infancy when DUCTUS ARTERIOSUS closes.'),
  'D018639': NtO(MH='Contact Lens Solutions', MS='Sterile solutions used to clean and disinfect contact lenses.'),
  'D018640': NtO(MH='Stomatognathic System Abnormalities', MS='Congenital structural abnormalities of the mouth and jaws, including the dentition.'),
  'D018643': NtO(MH='Arab World', MS='A historical and cultural entity dispersed across a wide geographical area under the administrative, intellectual, social, and cultural domination of the Arab empire. The Arab world, under the impetus of Islam, by the eighth century A.D., extended from Arabia in the Middle East to all of northern Africa, southern Spain, Sardinia, and Sicily. Close contact was maintained with Greek and Jewish culture. While the principal service of the Arabs to medicine was the preservation of Greek culture, the Arabs themselves were the originators of algebra, chemistry, geology, and many of the refinements of civilization. (From A. Castiglioni, A History of Medicine, 2d ed, p260; from F. H. Garrison, An Introduction to the History of Medicine, 4th ed, p126)'),
  'D018644': NtO(MH='Roman World', MS='A historical and cultural entity dispersed across a wide geographical area under the political domination and influence of ancient Rome, bringing to the conquered people the Roman civilization and culture from 753 B.C. to the beginning of the imperial rule under Augustus in 27 B.C. The early city built on seven hills grew to conquer Sicily, Sardinia, Carthage, Gaul, Spain, Britain, Greece, Asia Minor, etc., and extended ultimately from Mesopotamia to the Atlantic. Roman medicine was almost entirely in Greek hands, but Rome, with its superior water system, remains a model of sanitation and hygiene. (From A. Castiglioni, A History of Medicine, 2d ed pp196-99; from F. H. Garrison, An Introduction to the History of Medicine, 4th ed, pp107-120)'),
  'D018645': NtO(MH='Greek World', MS='A historical and cultural entity dispersed across a wide geographical area under the influence of Greek civilization, culture, and science. The Greek Empire extended from the Greek mainland and the Aegean islands from the 16th century B.C., to the Indus Valley in the 4th century under Alexander the Great, and to southern Italy and Sicily. Greek medicine began with Homeric and Aesculapian medicine and continued unbroken to Hippocrates (480-355 B.C.). The classic period of Greek medicine was 460-136 B.C. and the Graeco-Roman period, 156 B.C.-576 A.D. (From A. Castiglioni, A History of Medicine, 2d ed; from F. H. Garrison, An Introduction to the History of Medicine, 4th ed)'),
  'D018646': NtO(MH='Absinthe', MS='An extract of absinthium and other bitter herbs, containing 60% alcohol. Prolonged ingestion causes nervousness, convulsions, trismus, amblyopia, optic neuritis, and mental deterioration. (Dorland, 27th ed)'),
  'D018647': NtO(MH='Amber', MS='A yellowish fossil resin, the gum of several species of coniferous trees, found in the alluvial deposits of northeastern Germany. It is used in molecular biology in the analysis of organic matter fossilized in amber.'),
  'D018648': NtO(MH='Ambergris', MS='A gray substance found in the GASTROINTESTINAL TRACT of the SPERM WHALE.'),
  'D018649': NtO(MH='Ocimum basilicum', MS='A plant species of the genus OCIMUM, family LAMIACEAE. It is a condiment with carminative properties.'),
  'D018650': NtO(MH='Arnica', MS="A plant genus of the family ASTERACEAE. The dried flower heads of Arnica montana are used externally as a counterirritant and tincture for sprains and bruises, either as crude extract or in homeopathic dilution (HOMEOPATHY). Arnica contains volatile oils (OILS, VOLATILE), arnicin, arnisterol, FLAVONOIDS; TANNINS; and resin. The common name of Wolf's Bane is similar to the common name for ACONITUM."),
  'D018651': NtO(MH='Chicory', MS='A thick-rooted perennial (Cichorium intybus) native to Europe but widely grown for its young leaves used as salad greens and for its roots, dried and ground-roasted, used to flavor or adulterate coffee. (From Webster, 3d ed)'),
  'D018652': NtO(MH='Hemlock', MS='Any of several poisonous plants, commonly called poison hemlock, of the CICUTA or CONIUM genus. The hemlock tree (TSUGA) is completely unrelated being a coniferous tree of the pine family.'),
  'D018653': NtO(MH='Juniperus', MS='A plant genus of the family CUPRESSACEAE. The species are slow growing coniferous evergreen trees or shrubs.'),
  'D018655': NtO(MH='Lymphocyte Count', MS='The number of LYMPHOCYTES per unit volume of BLOOD.'),
  'D018656': NtO(MH='Muscle Fibers, Fast-Twitch', MS='Skeletal muscle fibers characterized by their expression of the Type II MYOSIN HEAVY CHAIN isoforms which have high ATPase activity and effect several other functional properties - shortening velocity, power output, rate of tension redevelopment. Several fast types have been identified.'),
  'D018657': NtO(MH='Muscle Fibers, Slow-Twitch', MS='Skeletal muscle fibers characterized by their expression of the Type I MYOSIN HEAVY CHAIN isoforms which have low ATPase activity and effect several other functional properties - shortening velocity, power output, rate of tension redevelopment.'),
  'D018658': NtO(MH='Ventricular Septal Rupture', MS='Laceration or tearing of the VENTRICULAR SEPTUM, usually caused by MYOCARDIAL INFARCTION.'),
  'D018660': NtO(MH='Blood Pressure Monitoring, Ambulatory', MS='Method in which repeated blood pressure readings are made while the patient undergoes normal daily activities. It allows quantitative analysis of the high blood pressure load over time, can help distinguish between types of HYPERTENSION, and can assess the effectiveness of antihypertensive therapy.'),
  'D018661': NtO(MH='Mycoplasma penetrans', MS='A species of gram-negative, facultatively anaerobic bacteria. This organism shows remarkable pathobiologic properties: it adheres to cell surfaces, deeply penetrates into the cell, and strongly adsorbs human red blood cells and human CD4+ lymphocytes and monocytes. M. penetrans was first isolated from the urogenital tract of patients with AIDS and high frequencies of antibodies to it are seen in HIV-infected patients.'),
  'D018662': NtO(MH='Fundoplication', MS='Mobilization of the lower end of the esophagus and plication of the fundus of the stomach around it (fundic wrapping) in the treatment of GASTROESOPHAGEAL REFLUX that may be associated with various disorders, such as hiatal hernia. (From Dorland, 28th ed)'),
  'D018663': NtO(MH='Adrenergic Agents', MS='Drugs that act on adrenergic receptors or affect the life cycle of adrenergic transmitters. Included here are adrenergic agonists and antagonists and agents that affect the synthesis, storage, uptake, metabolism, or release of adrenergic transmitters.'),
  'D018664': NtO(MH='Interleukin-12', MS='A heterodimeric cytokine that plays a role in innate and adaptive immune responses. Interleukin-12 is a 70 kDa protein that is composed of covalently linked 40 kDa and 35 kDa subunits. It is produced by DENDRITIC CELLS; MACROPHAGES and a variety of other immune cells and plays a role in the stimulation of INTERFERON-GAMMA production by T-LYMPHOCYTES and NATURAL KILLER CELLS.'),
  'D018666': NtO(MH='Ureteroscopy', MS='Endoscopic examination, therapy or surgery of the ureter.'),
  'D018667': NtO(MH='Tilt-Table Test', MS='A standard and widely accepted diagnostic test used to identify patients who have a vasodepressive and/or cardioinhibitory response as a cause of syncope. (From Braunwald, Heart Disease, 7th ed)'),
  'D018668': NtO(MH='Muntjacs', MS='A genus, Muntiacus, of the deer family (Cervidae) comprising six species living in China, Tibet, Nepal, India, the Malay Peninsula, and neighboring island countries. They are usually found in forests and areas of dense vegetation, usually not far from water. They emit a deep barklike sound which gives them the name "barking deer." If they sense a predator they will "bark" for an hour or more. They are hunted for their meat and skins; they thrive in captivity and are found in many zoos. The Indian muntjac is believed to have the lowest chromosome number in mammals and cell lines derived from them figure widely in chromosome and DNA studies. (From Walker\'s Mammals of the World, 5th ed., p1366)'),
  'D018670': NtO(MH='Monitoring, Ambulatory', MS='The use of electronic equipment to observe or record physiologic processes while the patient undergoes normal daily activities.'),
  'D018671': NtO(MH='Needle-Exchange Programs', MS='Organized services for exchange of sterile needles and syringes used for injections as a potential means of reducing the transmission of infectious diseases.'),
  'D018672': NtO(MH='Community Health Planning', MS='Planning that has the goals of improving health, improving accessibility to health services, and promoting efficiency in the provision of services and resources on a comprehensive basis for a whole community. (From Facts on File Dictionary of Health Care Management, 1988, p299)'),
  'D018673': NtO(MH='Nutrition Policy', MS='Guidelines and objectives pertaining to food supply and nutrition including recommendations for healthy diet.'),
  'D018674': NtO(MH='Adrenergic Antagonists', MS='Drugs that bind to but do not activate ADRENERGIC RECEPTORS. Adrenergic antagonists block the actions of the endogenous adrenergic transmitters EPINEPHRINE and NOREPINEPHRINE.'),
  'D018675': NtO(MH='Toxicity Tests', MS='An array of tests used to determine the toxicity of a substance to living systems. These include tests on clinical drugs, foods, and environmental pollutants.'),
  'D018677': NtO(MH='Tooth Injuries', MS='Traumatic or other damage to teeth including fractures (TOOTH FRACTURES) or displacements (TOOTH LUXATION).'),
  'D018678': NtO(MH='Cholinergic Agents', MS='Any drug used for its actions on cholinergic systems. Included here are agonists and antagonists, drugs that affect the life cycle of ACETYLCHOLINE, and drugs that affect the survival of cholinergic neurons. The term cholinergic agents is sometimes still used in the narrower sense of MUSCARINIC AGONISTS, although most modern texts discourage that usage.'),
  'D018679': NtO(MH='Cholinergic Agonists', MS='Drugs that bind to and activate cholinergic receptors.'),
  'D018680': NtO(MH='Cholinergic Antagonists', MS='Drugs that bind to but do not activate CHOLINERGIC RECEPTORS, thereby blocking the actions of ACETYLCHOLINE or cholinergic agonists.'),
  'D018681': NtO(MH='Anesthetics, General', MS='Agents that induce various degrees of analgesia; depression of consciousness, circulation, and respiration; relaxation of skeletal muscle; reduction of reflex activity; and amnesia. There are two types of general anesthetics, inhalation and intravenous. With either type, the arterial concentration of drug required to induce anesthesia varies with the condition of the patient, the desired depth of anesthesia, and the concomitant use of other drugs. (From AMA Drug Evaluations Annual, 1994, p.173)'),
  'D018682': NtO(MH='GABA Agents', MS='Substances used for their pharmacological actions on GABAergic systems. GABAergic agents include agonists, antagonists, degradation or uptake inhibitors, depleters, precursors, and modulators of receptor function.'),
  'D018683': NtO(MH='Excitatory Amino Acid Agents', MS='Drugs used for their actions on any aspect of excitatory amino acid neurotransmitter systems. Included are drugs that act on excitatory amino acid receptors, affect the life cycle of excitatory amino acid transmitters, or affect the survival of neurons using excitatory amino acids.'),
  'D018684': NtO(MH='Glycine Agents', MS='Substances used for their pharmacological actions on glycinergic systems. Glycinergic agents include agonists, antagonists, degradation or uptake inhibitors, depleters, precursors, and modulators of receptor function.'),
  'D018685': NtO(MH='Anesthetics, Inhalation', MS='Gases or volatile liquids that vary in the rate at which they induce anesthesia; potency; the degree of circulation, respiratory, or neuromuscular depression they produce; and analgesic effects. Inhalation anesthetics have advantages over intravenous agents in that the depth of anesthesia can be changed rapidly by altering the inhaled concentration. Because of their rapid elimination, any postoperative respiratory depression is of relatively short duration. (From AMA Drug Evaluations Annual, 1994, p173)'),
  'D018686': NtO(MH='Anesthetics, Intravenous', MS='Ultrashort-acting anesthetics that are used for induction. Loss of consciousness is rapid and induction is pleasant, but there is no muscle relaxation and reflexes frequently are not reduced adequately. Repeated administration results in accumulation and prolongs the recovery time. Since these agents have little if any analgesic activity, they are seldom used alone except in brief minor procedures. (From AMA Drug Evaluations Annual, 1994, p174)'),
  'D018687': NtO(MH='Antidepressive Agents, Second-Generation', MS='A structurally and mechanistically diverse group of drugs that are not tricyclics or monoamine oxidase inhibitors. The most clinically important appear to act selectively on serotonergic systems, especially by inhibiting serotonin reuptake.'),
  'D018688': NtO(MH='Neospora', MS='A genus of protozoan parasites of the subclass COCCIDIA. Its species are parasitic in dogs, cattle, goats, and sheep, among others. N. caninum, a species that mainly infects dogs, is intracellular in neural and other cells of the body, multiplies by endodyogeny, has no parasitophorous vacuole, and has numerous rhoptries. It is known to cause lesions in many tissues, especially the brain and spinal cord as well as abortion in the expectant mother.'),
  'D018689': NtO(MH='Sensory System Agents', MS='Drugs that act on neuronal sensory receptors resulting in an increase, decrease, or modification of afferent nerve activity. (From Smith and Reynard, Textbook of Pharmacology, 1991, p367)'),
  'D018690': NtO(MH='Excitatory Amino Acid Agonists', MS='Drugs that bind to and activate excitatory amino acid receptors.'),
  'D018691': NtO(MH='Excitatory Amino Acid Antagonists', MS='Drugs that bind to but do not activate excitatory amino acid receptors, thereby blocking the actions of agonists.'),
  'D018692': NtO(MH='Antimanic Agents', MS='Agents that are used to treat bipolar disorders or mania associated with other affective disorders.'),
  'D018696': NtO(MH='Neuroprotective Agents', MS='Drugs intended to prevent damage to the brain or spinal cord from ischemia, stroke, convulsions, or trauma. Some must be administered before the event, but others may be effective for some time after. They act by a variety of mechanisms, but often directly or indirectly minimize the damage produced by endogenous excitatory amino acids.'),
  'D018697': NtO(MH='Nootropic Agents', MS='Drugs used to specifically facilitate learning or memory, particularly to prevent the cognitive deficits associated with dementias. These drugs act by a variety of mechanisms.'),
  'D018698': NtO(MH='Glutamic Acid', MS='A non-essential amino acid naturally occurring in the L-form. Glutamic acid is the most common excitatory neurotransmitter in the CENTRAL NERVOUS SYSTEM.'),
  'D018699': NtO(MH='Coated Vesicles', MS='Vesicles formed when cell-membrane coated pits (COATED PITS, CELL-MEMBRANE) invaginate and pinch off. The outer surface of these vesicles are covered with a lattice-like network of coat proteins, such as CLATHRIN, coat protein complex proteins, or CAVEOLINS.'),
  'D018700': NtO(MH='Pleurodesis', MS='The production of adhesions between the parietal and visceral pleura. The procedure is used in the treatment of bronchopleural fistulas, malignant pleural effusions, and pneumothorax and often involves instillation of chemicals or other agents into the pleural space causing, in effect, a pleuritis that seals the air leak. (From Fishman, Pulmonary Diseases, 2d ed, p2233 & Dorland, 27th ed)'),
  'D018701': NtO(MH='Mononegavirales Infections', MS='Infections with viruses of the order MONONEGAVIRALES. The concept includes FILOVIRIDAE INFECTIONS; PARAMYXOVIRIDAE INFECTIONS; and RHABDOVIRIDAE INFECTIONS.'),
  'D018702': NtO(MH='Filoviridae Infections', MS='Infections with viruses of the family FILOVIRIDAE. The infections in humans consist of a variety of clinically similar viral hemorrhagic fevers but the natural reservoir host is unknown.'),
  'D018703': NtO(MH='Founder Effect', MS="A phenomenon that is observed when a small subgroup of a larger POPULATION establishes itself as a separate and isolated entity. The subgroup's GENE POOL carries only a fraction of the genetic diversity of the parental population resulting in an increased frequency of certain diseases in the subgroup, especially those diseases known to be autosomal recessive."),
  'D018704': NtO(MH='Orthodontic Retainers', MS="Orthodontic appliances, fixed or removable, used to maintain teeth in corrected positions during the period of functional adaptation following corrective treatment. These appliances are also used to maintain the positions of the teeth and jaws gained by orthodontic procedures. (From Zwemer, Boucher's Clinical Dental Terminology, 4th ed, p263)"),
  'D018707': NtO(MH='Volcanic Eruptions', MS='The ash, dust, gases, and lava released by volcanic explosion. The gases are volatile matter composed principally of about 90% water vapor, and carbon dioxide, sulfur dioxide, hydrogen, carbon monoxide, and nitrogen. The ash or dust is pyroclastic ejecta and lava is molten extrusive material consisting mainly of magnesium silicate. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D018708': NtO(MH='Picobirnavirus', MS='Unclassified, bisegmented, double-stranded RNA virus isolated from the feces of humans and other animals. Some reports associate it with gastroenteritis in humans.'),
  'D018709': NtO(MH='Statistics, Nonparametric', MS='A class of statistical methods applicable to a large set of probability distributions used to test for correlation, location, independence, etc. In most nonparametric statistical tests, the original scores or observations are replaced by another variable containing less information. An important class of nonparametric tests employs the ordinal properties of the data. Another class of tests uses information about whether an observation is above or below some fixed value such as the median, and a third class is based on the frequency of the occurrence of runs in the data. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed, p1284; Corsini, Concise Encyclopedia of Psychology, 1987, p764-5)'),
  'D018710': NtO(MH='Pain Clinics', MS='Facilities providing diagnostic, therapeutic, and palliative services for patients with severe chronic pain. These may be free-standing clinics or hospital-based and serve ambulatory or inpatient populations. The approach is usually multidisciplinary. These clinics are often referred to as "acute pain services". (From Br Med Bull 1991 Jul;47(3):762-85)'),
  'D018712': NtO(MH='Analgesics, Non-Narcotic', MS='A subclass of analgesic agents that typically do not bind to OPIOID RECEPTORS and are not addictive. Many non-narcotic analgesics are offered as NONPRESCRIPTION DRUGS.'),
  'D018713': NtO(MH='Prevotella intermedia', MS='A species of gram-negative, anaerobic, rod-shaped bacteria originally classified within the BACTEROIDES genus. This bacterium is a common commensal in the gingival crevice and is often isolated from cases of gingivitis and other purulent lesions related to the mouth.'),
  'D018714': NtO(MH='Radiotherapy, Adjuvant', MS='Radiotherapy given to augment some other form of treatment such as surgery or chemotherapy. Adjuvant radiotherapy is commonly used in the therapy of cancer and can be administered before or after the primary treatment.'),
  'D018715': NtO(MH='Microscopy, Video', MS='Microscopy in which television cameras are used to brighten magnified images that are otherwise too dark to be seen with the naked eye. It is used frequently in TELEPATHOLOGY.'),
  'D018716': NtO(MH='Molecular Mimicry', MS='The structure of one molecule that imitates or simulates the structure of a different molecule.'),
  'D018717': NtO(MH='Purple Membrane', MS='Functionally and structurally differentiated, purple-pigmented regions of the cytoplasmic membrane of some strains of Halobacterium halobium. The membrane develops under anaerobic conditions and is made almost entirely of the purple pigment BACTERIORHODOPSINS. (From Singleton & Sainsbury Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D018718': NtO(MH='Home Infusion Therapy', MS='Use of any infusion therapy on an ambulatory, outpatient, or other non-institutionalized basis.'),
  'D018719': NtO(MH='Receptor, ErbB-2', MS='A cell surface protein-tyrosine kinase receptor that is overexpressed in a variety of ADENOCARCINOMAS. It has extensive homology to and  heterodimerizes with the EGF RECEPTOR, the ERBB-3 RECEPTOR, and the ERBB-4 RECEPTOR. Activation of the erbB-2 receptor occurs through heterodimer formation with a ligand-bound erbB receptor family member.'),
  'D018720': NtO(MH='Prevotella', MS='A genus of gram-negative, anaerobic, nonsporeforming, nonmotile rods. Organisms of this genus had originally been classified as members of the BACTEROIDES genus but overwhelming biochemical and chemical findings in 1990 indicated the need to separate them from other Bacteroides species, and hence, this new genus was established.'),
  'D018721': NtO(MH='Muscarinic Agonists', MS='Drugs that bind to and activate muscarinic cholinergic receptors (RECEPTORS, MUSCARINIC). Muscarinic agonists are most commonly used when it is desirable to increase smooth muscle tone, especially in the GI tract, urinary bladder and the eye. They may also be used to reduce heart rate.'),
  'D018722': NtO(MH='Nicotinic Agonists', MS='Drugs that bind to and activate nicotinic cholinergic receptors (RECEPTORS, NICOTINIC). Nicotinic agonists act at postganglionic nicotinic receptors, at neuroeffector junctions in the peripheral nervous system, and at nicotinic receptors in the central nervous system. Agents that function as neuromuscular depolarizing blocking agents are included here because they activate nicotinic receptors, although they are used clinically to block nicotinic transmission.'),
  'D018723': NtO(MH='Bethanechol', MS='A slowly hydrolyzing muscarinic agonist with no nicotinic effects. Bethanechol is generally used to increase smooth muscle tone, as in the GI tract following abdominal surgery or in urinary retention in the absence of obstruction. It may cause hypotension, HEART RATE changes, and BRONCHIAL SPASM.'),
  'D018724': NtO(MH='Spinacia oleracea', MS='A widely cultivated plant, native to Asia, having succulent, edible leaves eaten as a vegetable. (From American Heritage Dictionary, 1982)'),
  'D018725': NtO(MH='Tar-Water', MS='A cold infusion of tar in water, formerly regarded as a cure-all. (Webster, 3d ed)'),
  'D018726': NtO(MH='Anti-Dyskinesia Agents', MS='Drugs used in the treatment of movement disorders. Most of these act centrally on dopaminergic or cholinergic systems. Among the most important clinically are those used for the treatment of Parkinson disease (ANTIPARKINSON AGENTS) and those for the tardive dyskinesias.'),
  'D018727': NtO(MH='Muscarinic Antagonists', MS='Drugs that bind to but do not activate MUSCARINIC RECEPTORS, thereby blocking the actions of endogenous ACETYLCHOLINE or exogenous agonists. Muscarinic antagonists have widespread effects including actions on the iris and ciliary muscle of the eye, the heart and blood vessels, secretions of the respiratory tract, GI system, and salivary glands, GI motility, urinary bladder tone, and the central nervous system.'),
  'D018728': NtO(MH='Entorhinal Cortex', MS='Cerebral cortex region on the medial aspect of the PARAHIPPOCAMPAL GYRUS, immediately caudal to the OLFACTORY CORTEX of the uncus. The entorhinal cortex is the origin of the major neural fiber system afferent to the HIPPOCAMPAL FORMATION, the so-called PERFORANT PATHWAY.'),
  'D018729': NtO(MH='Fontan Procedure', MS='A procedure in which total right atrial or total caval blood flow is channeled directly into the pulmonary artery or into a small right ventricle that serves only as a conduit. The principal congenital malformations for which this operation is useful are TRICUSPID ATRESIA and single ventricle with pulmonary stenosis.'),
  'D018730': NtO(MH='Infant Behavior', MS='Any observable response or action of a neonate or infant up through the age of 23 months.'),
  'D018732': NtO(MH='Forensic Anthropology', MS='Scientific study of human skeletal remains with the express purpose of identification. This includes establishing individual identity, trauma analysis, facial reconstruction, photographic superimposition, determination of time interval since death, and crime-scene recovery. Forensic anthropologists do not certify cause of death but provide data to assist in determination of probable cause. This is a branch of the field of physical anthropology and qualified individuals are certified by the American Board of Forensic Anthropology. (From Am J Forensic Med Pathol 1992 Jun;13(2):146)'),
  'D018733': NtO(MH='Nicotinic Antagonists', MS='Drugs that bind to nicotinic cholinergic receptors (RECEPTORS, NICOTINIC) and block the actions of acetylcholine or cholinergic agonists. Nicotinic antagonists block synaptic transmission at autonomic ganglia, the skeletal neuromuscular junction, and at central nervous system nicotinic synapses.'),
  'D018734': NtO(MH='Genes, erbB-2', MS='The erbB-2 gene is a proto-oncogene that codes for the erbB-2 receptor (RECEPTOR, ERBB-2), a protein with structural features similar to the epidermal growth factor receptor. Its name originates from the viral oncogene homolog (v-erbB) which is a truncated form of the chicken erbB gene found in the avian erythroblastosis virus. Overexpression and amplification of the gene is associated with a significant number of adenocarcinomas. The human c-erbB-2 gene is located at 17q21.2.'),
  'D018736': NtO(MH='DNA Adducts', MS='The products of chemical reactions that result in the addition of extraneous chemical groups to DNA.'),
  'D018737': NtO(MH='Hand Strength', MS='Force exerted when gripping or grasping.'),
  'D018738': NtO(MH='Hexamethonium', MS='A nicotinic cholinergic antagonist often referred to as the prototypical ganglionic blocker. It is poorly absorbed from the gastrointestinal tract and does not cross the blood-brain barrier. It has been used for a variety of therapeutic purposes including hypertension but, like the other ganglionic blockers, it has been replaced by more specific drugs for most purposes, although it is widely used a research tool.'),
  'D018739': NtO(MH='Oncogene Proteins v-erbB', MS='Transforming proteins encoded by erbB oncogenes from the avian erythroblastosis virus. The protein is a truncated form of the EGF receptor (RECEPTOR, EPIDERMAL GROWTH FACTOR) whose kinase domain is constitutively activated by deletion of the ligand-binding domain.'),
  'D018740': NtO(MH='Genetic Heterogeneity', MS='The presence of apparently similar characters for which the genetic evidence indicates that different genes or different genetic mechanisms are involved in different pedigrees. In clinical settings genetic heterogeneity refers to the presence of a variety of genetic defects which cause the same disease, often due to mutations at different loci on the same gene, a finding common to many human diseases including ALZHEIMER DISEASE; CYSTIC FIBROSIS; LIPOPROTEIN LIPASE DEFICIENCY, FAMILIAL; and POLYCYSTIC KIDNEY DISEASES. (Rieger, et al., Glossary of Genetics: Classical and Molecular, 5th ed; Segen, Dictionary of Modern Medicine, 1992)'),
  'D018741': NtO(MH='Replication Origin', MS='A unique DNA sequence of a replicon at which DNA REPLICATION is initiated and proceeds bidirectionally or unidirectionally. It contains the sites where the first separation of the complementary strands occurs, a primer RNA is synthesized, and the switch from primer RNA to DNA synthesis takes place. (Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D018742': NtO(MH='DNA, Chloroplast', MS='Deoxyribonucleic acid that makes up the genetic material of CHLOROPLASTS.'),
  'D018743': NtO(MH='Perinatal Care', MS='The care of women and a fetus or newborn given before, during, and after delivery from the 28th week of gestation through the 7th day after delivery.'),
  'D018744': NtO(MH='DNA, Plant', MS='Deoxyribonucleic acid that makes up the genetic material of plants.'),
  'D018745': NtO(MH='Genome, Plant', MS='The genetic complement of a plant (PLANTS) as represented in its DNA.'),
  'D018746': NtO(MH='Systemic Inflammatory Response Syndrome', MS='A systemic inflammatory response to a variety of clinical insults, characterized by two or more of the following conditions: (1) fever >38 degrees C or HYPOTHERMIA <36 degrees C; (2) TACHYCARDIA >90 beat/minute; (3) tachypnea >24 breaths/minute; (4) LEUKOCYTOSIS >12,000 cells/cubic mm or 10% immature forms. While usually related to infection, SIRS can also be associated with noninfectious insults such as TRAUMA; BURNS; or PANCREATITIS. If infection is involved, a patient with SIRS is said to have SEPSIS.'),
  'D018747': NtO(MH='RNA, Chloroplast', MS='Ribonucleic acid in chloroplasts having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D018749': NtO(MH='RNA, Plant', MS='Ribonucleic acid in plants having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D018750': NtO(MH='6-Cyano-7-nitroquinoxaline-2,3-dione', MS='A potent excitatory amino acid antagonist with a preference for non-NMDA iontropic receptors. It is used primarily as a research tool.'),
  'D018752': NtO(MH='Diet, Fat-Restricted', MS='A diet that contains limited amounts of fat with less than 30% of calories from all fats and less than 10% from saturated fat. Such a diet is used in control of HYPERLIPIDEMIAS. (From Bondy et al, Metabolic Control and Disease, 8th ed, pp468-70; Dorland, 27th ed)'),
  'D018753': NtO(MH='Diet, Protein-Restricted', MS='A diet that contains limited amounts of protein. It is prescribed in some cases to slow the progression of renal failure. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D018754': NtO(MH='Ventricular Dysfunction', MS='A condition in which HEART VENTRICLES exhibit impaired function.'),
  'D018755': NtO(MH='GABA Agonists', MS='Endogenous compounds and drugs that bind to and activate GAMMA-AMINOBUTYRIC ACID receptors (RECEPTORS, GABA).'),
  'D018756': NtO(MH='GABA Antagonists', MS='Drugs that bind to but do not activate GABA RECEPTORS, thereby blocking the actions of endogenous GAMMA-AMINOBUTYRIC ACID and GABA RECEPTOR AGONISTS.'),
  'D018757': NtO(MH='GABA Modulators', MS='Substances that do not act as agonists or antagonists but do affect the GAMMA-AMINOBUTYRIC ACID receptor-ionophore complex. GABA-A receptors (RECEPTORS, GABA-A) appear to have at least three allosteric sites at which modulators act: a site at which BENZODIAZEPINES act by increasing the opening frequency of GAMMA-AMINOBUTYRIC ACID-activated chloride channels; a site at which BARBITURATES act to prolong the duration of channel opening; and a site at which some steroids may act. GENERAL ANESTHETICS probably act at least partly by potentiating GABAergic responses, but they are not included here.'),
  'D018758': NtO(MH='Stimulants, Historical', MS='Agents or remedies that historically have produced stimulation or excited functional activity.'),
  'D018759': NtO(MH='Adrenergic Uptake Inhibitors', MS='Drugs that block the transport of adrenergic transmitters into axon terminals or into storage vesicles within terminals. The tricyclic antidepressants (ANTIDEPRESSIVE AGENTS, TRICYCLIC) and amphetamines are among the therapeutically important drugs that may act via inhibition of adrenergic transport. Many of these drugs also block transport of serotonin.'),
  'D018761': NtO(MH='Multiple Endocrine Neoplasia Type 1', MS='A form of multiple endocrine neoplasia that is characterized by the combined occurrence of tumors in the PARATHYROID GLANDS, the PITUITARY GLAND, and the PANCREATIC ISLETS. The resulting clinical signs include HYPERPARATHYROIDISM; HYPERCALCEMIA; HYPERPROLACTINEMIA; CUSHING DISEASE; GASTRINOMA; and ZOLLINGER-ELLISON SYNDROME. This disease is due to loss-of-function of the MEN1 gene, a tumor suppressor gene (GENES, TUMOR SUPPRESSOR) on CHROMOSOME 11 (Locus: 11q13).'),
  'D018763': NtO(MH='Muscle Fatigue', MS='A state arrived at through prolonged and strong contraction of a muscle. Studies in athletes during prolonged submaximal exercise have shown that muscle fatigue increases in almost direct proportion to the rate of muscle glycogen depletion. Muscle fatigue in short-term maximal exercise is associated with oxygen lack and an increased level of blood and muscle lactic acid, and an accompanying increase in hydrogen-ion concentration in the exercised muscle.'),
  'D018765': NtO(MH='Dopamine Uptake Inhibitors', MS='Drugs that block the transport of DOPAMINE into axon terminals or into storage vesicles within terminals. Most of the ADRENERGIC UPTAKE INHIBITORS also inhibit dopamine uptake.'),
  'D018771': NtO(MH='Arthralgia', MS='Pain in the joint.'),
  'D018772': NtO(MH='Dental Marginal Adaptation', MS='The degree of approximation or fit of filling material or dental prosthetic to the tooth surface. A close marginal adaptation and seal at the interface is important for successful dental restorations.'),
  'D018773': NtO(MH='Genes, erbB-1', MS='The proto-oncogene c-erbB-1 codes for the epidermal growth factor receptor. Its name originates from the viral homolog v-erbB which was isolated from an avian erythroblastosis virus (AEV) where it was contained as a fragment of the chicken c-ErbB-1 gene lacking the amino-terminal ligand-binding domain. Overexpression of erbB-1 genes occurs in a wide range of tumors, commonly squamous carcinomas of various sites and less commonly adenocarcinomas. The human c-erbB-1 gene is located in the chromosomal region 7p14 and 7p12.'),
  'D018774': NtO(MH='Oncogene Proteins v-erbA', MS='Transforming proteins encoded by erbA oncogenes from the avian erythroblastosis virus. They are truncated versions of c-erbA, the thyroid hormone receptor (RECEPTORS, THYROID HORMONE) that have retained both the DNA-binding and hormone-binding domains. Mutations in the hormone-binding domains abolish the transcriptional activation function. v-erbA acts as a dominant repressor of c-erbA, inducing transformation by disinhibiting proliferation.'),
  'D018775': NtO(MH='Bone Demineralization Technique', MS='Removal of mineral constituents or salts from bone or bone tissue. Demineralization is used as a method of studying bone strength and bone chemistry.'),
  'D018776': NtO(MH='Genes, erbA', MS='Genes related to the erbA DNA sequence that was first isolated from the avian erythroblastosis virus (ERYTHROBLASTOSIS VIRUS, AVIAN), v-erbA. In cells, erbA genes encode thyroid hormone receptors (RECEPTORS, THYROID HORMONE). Two distinct c-erbA genes have been identified: erbA-alpha located at 17q21; and erbA-beta located at 3p24. Truncations at the N- and C-terminals of erbA result in products resembling v-erbA. Truncations affect hormone responsiveness but not DNA binding capacity.'),
  'D018777': NtO(MH='Multiple Chemical Sensitivity', MS='An acquired disorder characterized by recurrent symptoms, referable to multiple organ systems, occurring in response to demonstrable exposure to many chemically unrelated compounds at doses below those established in the general population to cause harmful effects. (Cullen MR. The worker with multiple chemical sensitivities: an overview. Occup Med 1987;2(4):655-61)'),
  'D018778': NtO(MH='Hantavirus Infections', MS='Infections with viruses of the genus HANTAVIRUS. This is associated with at least four clinical syndromes: HEMORRHAGIC FEVER WITH RENAL SYNDROME caused by viruses of the Hantaan group; a milder form of HFRS caused by SEOUL VIRUS; nephropathia epidemica caused by PUUMALA VIRUS; and HANTAVIRUS PULMONARY SYNDROME caused by SIN NOMBRE VIRUS.'),
  'D018780': NtO(MH='Body Surface Potential Mapping', MS='Recording of regional electrophysiological information by analysis of surface potentials to give a complete picture of the effects of the currents from the heart on the body surface. It has been applied to the diagnosis of old inferior myocardial infarction, localization of the bypass pathway in Wolff-Parkinson-White syndrome, recognition of ventricular hypertrophy, estimation of the size of a myocardial infarct, and the effects of different interventions designed to reduce infarct size. The limiting factor at present is the complexity of the recording and analysis, which requires 100 or more electrodes, sophisticated instrumentation, and dedicated personnel. (Braunwald, Heart Disease, 4th ed)'),
  'D018781': NtO(MH='Tension-Type Headache', MS='A common primary headache disorder, characterized by a dull, non-pulsatile, diffuse, band-like (or vice-like) PAIN of mild to moderate intensity in the HEAD; SCALP; or NECK. The subtypes are classified by frequency and severity of symptoms. There is no clear cause even though it has been associated with MUSCLE CONTRACTION and stress. (International Classification of Headache Disorders, 2nd ed. Cephalalgia 2004: suppl 1)'),
  'D018782': NtO(MH='Maze Learning', MS='Learning the correct route through a maze to obtain reinforcement. It is used for human or animal populations. (Thesaurus of Psychological Index Terms, 6th ed)'),
  'D018783': NtO(MH='Impotence, Vasculogenic', MS='Inability to achieve and maintain an erection (ERECTILE DYSFUNCTION) due to defects in the arterial blood flow to the PENIS, defect in venous occlusive function allowing blood drainage (leakage) from the erectile tissue (corpus cavernosum penis), or both.'),
  'D018784': NtO(MH='Abdominal Abscess', MS='An abscess located in the abdominal cavity, i.e., the cavity between the diaphragm above and the pelvis below. (From Dorland, 27th ed)'),
  'D018785': NtO(MH='Tricuspid Atresia', MS='Absence of the orifice between the RIGHT ATRIUM and RIGHT VENTRICLE, with the presence of an atrial defect through which all the systemic venous return reaches the left heart. As a result, there is left ventricular hypertrophy (HYPERTROPHY, LEFT VENTRICULAR) because the right ventricle is absent or not functional.'),
  'D018786': NtO(MH='Bone Substitutes', MS='Synthetic or natural materials for the replacement of bones or bone tissue. They include hard tissue replacement polymers, natural coral, hydroxyapatite, beta-tricalcium phosphate, and various other biomaterials. The bone substitutes as inert materials can be incorporated into surrounding tissue or gradually replaced by original tissue.'),
  'D018787': NtO(MH='Radiation Oncology', MS='A subspecialty of medical oncology and radiology concerned with the radiotherapy of cancer.'),
  'D018788': NtO(MH='Radioactive Hazard Release', MS='Uncontrolled release of radioactive material from its containment. This either threatens to, or does, cause exposure to a radioactive hazard. Such an incident may occur accidentally or deliberately.'),
  'D018791': NtO(MH='CD4 Lymphocyte Count', MS='The number of CD4-POSITIVE T-LYMPHOCYTES per unit volume of BLOOD. Determination requires the use of a fluorescence-activated flow cytometer.'),
  'D018792': NtO(MH='Encephalitis, Viral', MS='Inflammation of brain parenchymal tissue as a result of viral infection. Encephalitis may occur as primary or secondary manifestation of TOGAVIRIDAE INFECTIONS; HERPESVIRIDAE INFECTIONS; ADENOVIRIDAE INFECTIONS; FLAVIVIRIDAE INFECTIONS; BUNYAVIRIDAE INFECTIONS; PICORNAVIRIDAE INFECTIONS; PARAMYXOVIRIDAE INFECTIONS; ORTHOMYXOVIRIDAE INFECTIONS; RETROVIRIDAE INFECTIONS; and ARENAVIRIDAE INFECTIONS.'),
  'D018793': NtO(MH='Interleukin-13', MS='A cytokine synthesized by T-LYMPHOCYTES that produces proliferation, immunoglobulin isotype switching, and immunoglobulin production by immature B-LYMPHOCYTES. It appears to play a role in regulating inflammatory and immune responses.'),
  'D018794': NtO(MH='Problem-Based Learning', MS='Instructional use of examples or cases to teach using problem-solving skills and critical thinking.'),
  'D018795': NtO(MH='Device Approval', MS='Process that is gone through in order for a device to receive approval by a government regulatory agency. This includes any required preclinical or clinical testing, review, submission, and evaluation of the applications and test results, and post-marketing surveillance. It is not restricted to FDA.'),
  'D018796': NtO(MH='Immunoconjugates', MS='Combinations of diagnostic or therapeutic substances linked with specific immune substances such as IMMUNOGLOBULINS; MONOCLONAL ANTIBODIES; or ANTIGENS. Often the diagnostic or therapeutic substance is a radionuclide. These conjugates are useful tools for specific targeting of DRUGS and RADIOISOTOPES in the CHEMOTHERAPY and RADIOIMMUNOTHERAPY of certain cancers.'),
  'D018797': NtO(MH='Cell Cycle Proteins', MS='Proteins that control the CELL DIVISION CYCLE. This family of proteins includes a wide variety of classes, including CYCLIN-DEPENDENT KINASES, mitogen-activated kinases, CYCLINS, and PHOSPHOPROTEIN PHOSPHATASES as well as their putative substrates such as chromatin-associated proteins, CYTOSKELETAL PROTEINS, and TRANSCRIPTION FACTORS.'),
  'D018798': NtO(MH='Anemia, Iron-Deficiency', MS='Anemia characterized by decreased or absent iron stores, low serum iron concentration, low transferrin saturation, and low hemoglobin concentration or hematocrit value. The erythrocytes are hypochromic and microcytic and the iron binding capacity is increased.'),
  'D018799': NtO(MH='Intercellular Adhesion Molecule-1', MS='A cell-surface ligand involved in leukocyte adhesion and inflammation. Its production is induced by gamma-interferon and it is required for neutrophil migration into inflamed tissue.'),
  'D018800': NtO(MH='Thy-1 Antigens', MS='A group of differentiation surface antigens, among the first to be discovered on thymocytes and T-lymphocytes. Originally identified in the mouse, they are also found in other species including humans, and are expressed on brain neurons and other cells.'),
  'D018801': NtO(MH='CD2 Antigens', MS='Glycoprotein members of the immunoglobulin superfamily which participate in T-cell adhesion and activation. They are expressed on most peripheral T-lymphocytes, natural killer cells, and thymocytes, and function as co-receptors or accessory molecules in the T-cell receptor complex.'),
  'D018802': NtO(MH='Patient-Centered Care', MS='Design of patient care wherein institutional resources and personnel are organized around patients rather than around specialized departments. (From Hospitals 1993 Feb 5;67(3):14)'),
  'D018803': NtO(MH='Models, Economic', MS='Statistical models of the production, distribution, and consumption of goods and services, as well as of financial considerations. For the application of statistics to the testing and quantifying of economic theories MODELS, ECONOMETRIC is available.'),
  'D018804': NtO(MH='Hantavirus Pulmonary Syndrome', MS='Acute respiratory illness in humans caused by the SIN NOMBRE VIRUS whose primary rodent reservoir is the deer mouse Peromyscus maniculatus. First identified in the southwestern United States, this syndrome is characterized most commonly by fever, myalgias, headache, cough, and rapid respiratory failure.'),
  'D018805': NtO(MH='Sepsis', MS='Systemic inflammatory response syndrome with a proven or suspected infectious etiology. When sepsis is associated with organ dysfunction distant from the site of infection, it is called severe sepsis. When sepsis is accompanied by HYPOTENSION despite adequate fluid infusion, it is called SEPTIC SHOCK.'),
  'D018806': NtO(MH='APACHE', MS='An acronym for Acute Physiology and Chronic Health Evaluation, a scoring system using routinely collected data and providing an accurate, objective description for a broad range of intensive care unit admissions, measuring severity of illness in critically ill patients.'),
  'D018807': NtO(MH='Polymorphism, Single-Stranded Conformational', MS="Variation in a population's DNA sequence that is detected by determining alterations in the conformation of denatured DNA fragments. Denatured DNA fragments are allowed to renature under conditions that prevent the formation of double-stranded DNA and allow secondary structure to form in single stranded fragments. These fragments are then run through polyacrylamide gels to detect variations in the secondary structure that is manifested as an alteration in migration through the gels."),
  'D018808': NtO(MH='Transcription Factor AP-1', MS='A multiprotein complex composed of the products of c-jun and c-fos proto-oncogenes. These proteins must dimerize in order to bind to the AP-1 recognition site, also known as the TPA-responsive element (TRE). AP-1 controls both basal and inducible transcription of several genes.'),
  'D018809': NtO(MH='Proliferating Cell Nuclear Antigen', MS='Nuclear antigen with a role in DNA synthesis, DNA repair, and cell cycle progression. PCNA is required for the coordinated synthesis of both leading and lagging strands at the replication fork during DNA replication. PCNA expression correlates with the proliferation activity of several malignant and non-malignant cell types.'),
  'D018810': NtO(MH='Magnetic Resonance Angiography', MS='Non-invasive method of vascular imaging and determination of internal anatomy without injection of contrast media or radiation exposure. The technique is used especially in CEREBRAL ANGIOGRAPHY as well as for studies of other vascular structures.'),
  'D018811': NtO(MH='Maternal Exposure', MS='Exposure of the female parent, human or animal, to potentially harmful chemical, physical, or biological agents in the environment or to environmental factors that may include ionizing radiation, pathogenic organisms, or toxic chemicals that may affect offspring. It includes pre-conception maternal exposure.'),
  'D018812': NtO(MH='Paternal Exposure', MS='Exposure of the male parent, human or animal, to potentially harmful chemical, physical, or biological agents in the environment or to environmental factors that may include ionizing radiation, pathogenic organisms, or toxic chemicals that may affect offspring.'),
  'D018813': NtO(MH='Multiple Endocrine Neoplasia Type 2a', MS='A form of multiple endocrine neoplasia characterized by the presence of medullary carcinoma (CARCINOMA, MEDULLARY) of the THYROID GLAND, and usually with the co-occurrence of PHEOCHROMOCYTOMA, producing CALCITONIN and ADRENALINE, respectively. Less frequently, it can occur with hyperplasia or adenoma of the PARATHYROID GLANDS. This disease is due to gain-of-function mutations of the MEN2 gene on CHROMOSOME 10 (Locus: 10q11.2), also known as the RET proto-oncogene that encodes a RECEPTOR PROTEIN-TYROSINE KINASE. It is an autosomal dominant inherited disease.'),
  'D018814': NtO(MH='Multiple Endocrine Neoplasia Type 2b', MS='Similar to MEN2A, it is also caused by mutations of the MEN2 gene, also known as the RET proto-oncogene. Its clinical symptoms include medullary carcinoma (CARCINOMA, MEDULLARY) of THYROID GLAND and PHEOCHROMOCYTOMA of ADRENAL MEDULLA (50%). Unlike MEN2a, MEN2b does not involve PARATHYROID NEOPLASMS. It can be distinguished from MEN2A by its neural abnormalities such as mucosal NEUROMAS on EYELIDS; LIP; and TONGUE, and ganglioneuromatosis of GASTROINTESTINAL TRACT leading to MEGACOLON. It is an autosomal dominant inherited disease.'),
  'D018815': NtO(MH='Genes, erbB', MS='Retrovirus-associated DNA sequences (erbB) originally isolated from, or related to, the avian erythroblastosis virus (AEV). These genes code for the epidermal growth factor receptor (EGFR) family of receptors which is important in the control of normal cell proliferation and in the pathogenesis of human cancer. The genes include erbB-1 (GENES, ERBB-1), erbB-2 (GENES, ERBB-2), and erbB-3, all of which show abnormalities of expression in various human neoplasms.'),
  'D018816': NtO(MH='Genes, cdc', MS='Genes that code for proteins that regulate the CELL DIVISION CYCLE. These genes form a regulatory network that culminates in the onset of MITOSIS by activating the p34cdc2 protein (PROTEIN P34CDC2).'),
  'D018817': NtO(MH='N-Methyl-3,4-methylenedioxyamphetamine', MS='An N-substituted amphetamine analog. It is a widely abused drug classified as a hallucinogen and causes marked, long-lasting changes in brain serotonergic systems. It is commonly referred to as MDMA or ecstasy.'),
  'D018818': NtO(MH='Fenoldopam', MS='A dopamine D1 receptor agonist that is used as an antihypertensive agent. It lowers blood pressure through arteriolar vasodilation.'),
  'D018819': NtO(MH='Dipeptidyl Peptidase 4', MS='A serine protease that catalyses the release of an N-terminal dipeptide. Several biologically-active peptides have been identified as dipeptidyl peptidase 4 substrates including INCRETINS; NEUROPEPTIDES; and CHEMOKINES. The protein is also found bound to ADENOSINE DEAMINASE on the T-CELL surface and is believed to play a role in T-cell activation.'),
  'D018821': NtO(MH='CD18 Antigens', MS='Cell-surface glycoprotein beta-chains that are non-covalently linked to specific alpha-chains of the CD11 family of leukocyte-adhesion molecules (RECEPTORS, LEUKOCYTE-ADHESION). A defect in the gene encoding CD18 causes LEUKOCYTE-ADHESION DEFICIENCY SYNDROME.'),
  'D018822': NtO(MH='alpha-Endorphin', MS='An endogenous opioid peptide derived from BETA-LIPOTROPIN of the pro-opiomelanocortin (POMC) system. It is the 16-amino acid sequence of the N-terminal of BETA-ENDORPHIN and differs from GAMMA-ENDORPHIN by one amino acid (beta-endorphin 1-17).'),
  'D018823': NtO(MH='gamma-Endorphin', MS='An endogenous opioid peptide derived from BETA-LIPOTROPIN of the pro-opiomelanocortin (POMC) system. It is the 17-amino acid sequence of the N-terminal of BETA-ENDORPHIN and differs from ALPHA-ENDORPHIN by one amino acid (beta-endorphin 1-16).'),
  'D018826': NtO(MH='CD13 Antigens', MS='Zinc-binding metalloproteases that are members of the type II integral membrane metalloproteases. They are expressed by GRANULOCYTES; MONOCYTES; and their precursors as well as by various non-hematopoietic cells. They release an N-terminal amino acid from a peptide, amide or arylamide.'),
  'D018827': NtO(MH='Carcinoma, Lewis Lung', MS='A carcinoma discovered by Dr. Margaret R. Lewis of the Wistar Institute in 1951. This tumor originated spontaneously as a carcinoma of the lung of a C57BL mouse. The tumor does not appear to be grossly hemorrhagic and the majority of the tumor tissue is a semifirm homogeneous mass. (From Cancer Chemother Rep 2 1972 Nov;(3)1:325) It is also called 3LL and LLC and is used as a transplantable malignancy.'),
  'D018828': NtO(MH='Immunoglobulins, Thyroid-Stimulating', MS='Autoantibodies that bind to the thyroid-stimulating hormone (TSH) receptor (RECEPTORS, THYROTROPIN) on thyroid epithelial cells. The autoantibodies mimic TSH causing an unregulated production of thyroid hormones characteristic of GRAVES DISEASE.'),
  'D018829': NtO(MH='Adhesins, Bacterial', MS='Cell-surface components or appendages of bacteria that facilitate adhesion (BACTERIAL ADHESION) to other cells or to inanimate surfaces. Most fimbriae (FIMBRIAE, BACTERIAL) of gram-negative bacteria function as adhesins, but in many cases it is a minor subunit protein at the tip of the fimbriae that is the actual adhesin. In gram-positive bacteria, a protein or polysaccharide surface layer serves as the specific adhesin. What is sometimes called polymeric adhesin (BIOFILMS) is distinct from protein adhesin.'),
  'D018830': NtO(MH='Adhesins, Escherichia coli', MS='Thin, filamentous protein structures, including proteinaceous capsular antigens (fimbrial antigens), that mediate adhesion of E. coli to surfaces and play a role in pathogenesis. They have a high affinity for various epithelial cells.'),
  'D018832': NtO(MH='Molecular Chaperones', MS='A family of cellular proteins that mediate the correct assembly or disassembly of polypeptides and their associated ligands. Although they take part in the assembly process, molecular chaperones are not components of the final structures.'),
  'D018833': NtO(MH='Chaperonins', MS='A family of multisubunit protein complexes that form into large cylindrical structures which bind to and encapsulate non-native proteins. Chaperonins utilize the energy of ATP hydrolysis to enhance the efficiency of PROTEIN FOLDING reactions and thereby help proteins reach their functional conformation. The family of chaperonins is split into GROUP I CHAPERONINS, and GROUP II CHAPERONINS, with each group having its own repertoire of protein subunits and subcellular preferences.'),
  'D018834': NtO(MH='Chaperonin 60', MS='A group I chaperonin protein that forms the barrel-like structure of the chaperonin complex. It is an oligomeric protein with a distinctive structure of fourteen subunits, arranged in two rings of seven subunits each. The protein was originally studied in BACTERIA where it is commonly referred to as GroEL protein.'),
  'D018835': NtO(MH='Chaperonin 10', MS='A group I chaperonin protein that forms a lid-like structure which encloses the non-polar cavity of the chaperonin complex. The protein was originally studied in BACTERIA where it is commonly referred to as GroES protein.'),
  'D018836': NtO(MH='Inflammation Mediators', MS='The endogenous compounds that mediate inflammation (AUTACOIDS) and related exogenous compounds including the synthetic prostaglandins (PROSTAGLANDINS, SYNTHETIC).'),
  'D018840': NtO(MH='HSP70 Heat-Shock Proteins', MS='A class of MOLECULAR CHAPERONES found in both prokaryotes and in several compartments of eukaryotic cells. These proteins can interact with polypeptides during a variety of assembly processes in such a way as to prevent the formation of nonfunctional structures.'),
  'D018841': NtO(MH='HSP90 Heat-Shock Proteins', MS='A class of MOLECULAR CHAPERONES whose members act in the mechanism of SIGNAL TRANSDUCTION by STEROID RECEPTORS.'),
  'D018843': NtO(MH='Republic of North Macedonia', MS='Country in Europe, east of Albania and north of Greece. The capital is Skopje. In June, 2018 agreement was reached to establish the new name Republic of North Macedonia.which became effective February 2019.'),
  'D018844': NtO(MH='Cyclin-Dependent Kinases', MS='Protein kinases that control cell cycle progression in all eukaryotes and require physical association with CYCLINS to achieve full enzymatic activity. Cyclin-dependent kinases are regulated by phosphorylation and dephosphorylation events.'),
  'D018845': NtO(MH='CD11 Antigens', MS='A group of three different integrin alpha chains (CD11a, CD11b, CD11c) that are associated with an invariant integrin beta-2 chain (CD18 ANTIGENS). The three resulting leukocyte-adhesion molecules (RECEPTORS, LEUKOCYTE ADHESION) are LYMPHOCYTE FUNCTION-ASSOCIATED ANTIGEN-1; MACROPHAGE-1 ANTIGEN; and ANTIGEN, P150,95.'),
  'D018846': NtO(MH='Excitatory Amino Acids', MS='Endogenous amino acids released by neurons as excitatory neurotransmitters. Glutamic acid is the most common excitatory neurotransmitter in the brain. Aspartic acid has been regarded as an excitatory transmitter for many years, but the extent of its role as a transmitter is unclear.'),
  'D018847': NtO(MH='Opioid Peptides', MS='The endogenous peptides with opiate-like activity. The three major classes currently recognized are the ENKEPHALINS, the DYNORPHINS, and the ENDORPHINS. Each of these families derives from different precursors, proenkephalin, prodynorphin, and PRO-OPIOMELANOCORTIN, respectively. There are also at least three classes of OPIOID RECEPTORS, but the peptide families do not map to the receptors in a simple way.'),
  'D018848': NtO(MH='Controlled Clinical Trial', MS='A work that reports on a clinical trial involving one or more test treatments, at least one control treatment, specified outcome measures for evaluating the studied intervention, and a bias-free method for assigning patients to the test treatment. The treatment may be drugs, devices, or procedures studied for diagnostic, therapeutic, or prophylactic effectiveness. Control measures include placebos, active medicine, no-treatment, dosage forms and regimens, historical comparisons, etc. When randomization using mathematical techniques, such as the use of a random numbers table, is employed to assign patients to test or control treatments, the trial is characterized as a RANDOMIZED CONTROLLED TRIAL.'),
  'D018849': NtO(MH='Controlled Clinical Trials as Topic', MS='Works about clinical trials involving one or more test treatments, at least one control treatment, specified outcome measures for evaluating the studied intervention, and a bias-free method for assigning patients to the test treatment. The treatment may be drugs, devices, or procedures studied for diagnostic, therapeutic, or prophylactic effectiveness. Control measures include placebos, active medicines, no-treatment, dosage forms and regimens, historical comparisons, etc. When randomization using mathematical techniques, such as the use of a random numbers table, is employed to assign patients to test or control treatments, the trials are characterized as RANDOMIZED CONTROLLED TRIALS AS TOPIC.'),
  'D018854': NtO(MH='Managed Competition', MS='A strategy for purchasing health care in a manner which will obtain maximum value for the price for the purchasers of the health care and the recipients. The concept was developed primarily by Alain Enthoven of Stanford University and promulgated by the Jackson Hole Group. The strategy depends on sponsors for groups of the population to be insured. The sponsor, in some cases a health alliance, acts as an intermediary between the group and competing provider groups (accountable health plans). The competition is price-based among annual premiums for a defined, standardized benefit package. (From Slee and Slee, Health Care Reform Terms, 1993)'),
  'D018855': NtO(MH='Single-Payer System', MS='An approach to health care financing with only one source of money for paying health care providers. The scope may be national (the Canadian System), state-wide, or community-based. The payer may be a governmental unit or other entity such as an insurance company. The proposed advantages include administrative simplicity for patients and providers, and resulting significant savings in overhead costs. (From Slee and Slee, Health Care Reform Terms, 1993, p106)'),
  'D018856': NtO(MH='Cystitis, Interstitial', MS='A condition with recurring discomfort or pain in the URINARY BLADDER and the surrounding pelvic region without an identifiable disease. Severity of pain in interstitial cystitis varies greatly and often is accompanied by increased urination frequency and urgency.'),
  'D018857': NtO(MH='Food Packaging', MS='Containers, packaging, and packaging materials for processed and raw foods and beverages. It includes packaging intended to be used for storage and also used for preparation of foods such as microwave food containers versus COOKING AND EATING UTENSILS. Packaging materials may be intended for food contact or designated non-contact, for example, shipping containers. FOOD LABELING is also available.'),
  'D018858': NtO(MH='Germinal Center', MS='The activated center of a lymphoid follicle in secondary lymphoid tissue where B-LYMPHOCYTES are stimulated by antigens and helper T cells (T-LYMPHOCYTES, HELPER-INDUCER) are stimulated to generate memory cells.'),
  'D018859': NtO(MH='Hair Follicle', MS='A tube-like invagination of the EPIDERMIS from which the hair shaft develops and into which SEBACEOUS GLANDS open. The hair follicle is lined by a cellular inner and outer root sheath of epidermal origin and is invested with a fibrous sheath derived from the dermis. (Stedman, 26th ed) Follicles of very long hairs extend into the subcutaneous layer of tissue under the SKIN.'),
  'D018860': NtO(MH='Sneddon Syndrome', MS='A systemic non-inflammatory arteriopathy primarily of middle-aged females characterized by the association of livedo reticularis, multiple thrombotic CEREBRAL INFARCTION; CORONARY DISEASE, and HYPERTENSION. Elevation of antiphospholipid antibody titers (see also ANTIPHOSPHOLIPID SYNDROME), cardiac valvulopathy, ISCHEMIC ATTACK, TRANSIENT; SEIZURES; DEMENTIA; and chronic ischemia of the extremities may also occur. Pathologic examination of affected arteries reveals non-inflammatory adventitial fibrosis, thrombosis, and changes in the media. (From Jablonski, Dictionary of Syndromes & Eponymic Diseases, 2d ed; Adams et al., Principles of Neurology, 6th ed, p861; Arch Neurol 1997 Jan;54(1):53-60)'),
  'D018862': NtO(MH='Merkel Cells', MS="Modified epidermal cells located in the stratum basale. They are found mostly in areas where sensory perception is acute, such as the fingertips. Merkel cells are closely associated with an expanded terminal bulb of an afferent myelinated nerve fiber. Do not confuse with Merkel's corpuscle which is a combination of a neuron and an epidermal cell."),
  'D018864': NtO(MH='Cultural Diversity', MS='Coexistence of numerous distinct ethnic, racial, religious, or cultural groups within one social unit, organization, or population. (From American Heritage Dictionary, 2d college ed., 1982, p955)'),
  'D018865': NtO(MH='Dental Research', MS='The study of laws, theories, and hypotheses through a systematic examination of pertinent facts and their interpretation in the field of dentistry. (From Jablonski, Illustrated Dictionary of Dentistry, 1982, p674)'),
  'D018868': NtO(MH='Insurance, Disability', MS='Insurance designed to compensate persons who lose wages because of illness or injury; insurance providing periodic payments that partially replace lost wages, salary, or other income when the insured is unable to work because of illness, injury, or disease. Individual and group disability insurance are two types of such coverage. (From Facts on File Dictionary of Health Care Management, 1988, p207)'),
  'D018869': NtO(MH='Heat-Shock Response', MS='A sequence of responses that occur when an organism is exposed to excessive heat. In humans, an increase in skin temperature triggers muscle relaxation, sweating, and vasodilation.'),
  'D018870': NtO(MH='Endoplasmic Reticulum, Rough', MS='A type of endoplasmic reticulum (ER) where polyribosomes are present on the cytoplasmic surfaces of the ER membranes. This form of ER is prominent in cells specialized for protein secretion and its principal function is to segregate proteins destined for export or intracellular utilization.'),
  'D018871': NtO(MH='Endoplasmic Reticulum, Smooth', MS='A type of endoplasmic reticulum lacking associated ribosomes on the membrane surface. It exhibits a wide range of specialized metabolic functions including supplying enzymes for steroid synthesis, detoxification, and glycogen breakdown. In muscle cells, smooth endoplasmic reticulum is called SARCOPLASMIC RETICULUM.'),
  'D018873': NtO(MH='Pregnancy Rate', MS='The ratio of the number of conceptions (CONCEPTION) including LIVE BIRTH; STILLBIRTH; and fetal losses, to the mean number of females of reproductive age in a population during a set time period.'),
  'D018874': NtO(MH='Spheroids, Cellular', MS='Spherical, heterogeneous aggregates of proliferating, quiescent, and necrotic cells in culture that retain three-dimensional architecture and tissue-specific functions. The ability to form spheroids is a characteristic trait of CULTURED TUMOR CELLS derived from solid TUMORS. Cells from normal tissues can also form spheroids. They represent an in-vitro model for studies of the biology of both normal and malignant cells. (From Bjerkvig, Spheroid Culture in Cancer Research, 1992, p4)'),
  'D018875': NtO(MH='Vocabulary, Controlled', MS="A specified list of terms with a fixed and unalterable meaning, and from which a selection is made when CATALOGING; ABSTRACTING AND INDEXING; or searching BOOKS; JOURNALS AS TOPIC; and other documents. The control is intended to avoid the scattering of related subjects under different headings (SUBJECT HEADINGS). The list may be altered or extended only by the publisher or issuing agency. (From Harrod's Librarians' Glossary, 7th ed, p163)"),
  'D018876': NtO(MH='Environmental Illness', MS='A polysymptomatic condition believed by clinical ecologists to result from immune dysregulation induced by common foods, allergens, and chemicals, resulting in various physical and mental disorders. The medical community has remained largely skeptical of the existence of this "disease", given the plethora of symptoms attributed to environmental illness, the lack of reproducible laboratory abnormalities, and the use of unproven therapies to treat the condition. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D018877': NtO(MH='Sick Building Syndrome', MS='A group of symptoms that are two- to three-fold more common in those who work in large, energy-efficient buildings, associated with an increased frequency of headaches, lethargy, and dry skin. Clinical manifestations include hypersensitivity pneumonitis (ALVEOLITIS, EXTRINSIC ALLERGIC); allergic rhinitis (RHINITIS, ALLERGIC, PERENNIAL); ASTHMA; infections, skin eruptions, and mucous membrane irritation syndromes. Current usage tends to be less restrictive with regard to the type of building and delineation of complaints. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D018878': NtO(MH='Limb Buds', MS='Distinct regions of mesenchymal outgrowth at both flanks of an embryo during the SOMITE period. Limb buds, covered by ECTODERM, give rise to forelimb, hindlimb, and eventual functional limb structures. Limb bud cultures are used to study CELL DIFFERENTIATION; ORGANOGENESIS; and MORPHOGENESIS.'),
  'D018879': NtO(MH='Ventricular Premature Complexes', MS='A type of cardiac arrhythmia with premature contractions of the HEART VENTRICLES. It is characterized by the premature QRS complex on ECG that is of abnormal shape and great duration (generally >129 msec). It is the most common form of all cardiac arrhythmias. Premature ventricular complexes have no clinical significance except in concurrence with heart diseases.'),
  'D018880': NtO(MH='Atrial Premature Complexes', MS='A type of cardiac arrhythmia with premature atrial contractions or beats caused by signals originating from ectopic atrial sites. The ectopic signals may or may not conduct to the HEART VENTRICLES. Atrial premature complexes are characterized by premature P waves on ECG which are different in configuration from the P waves generated by the normal pacemaker complex in the SINOATRIAL NODE.'),
  'D018881': NtO(MH='Suburban Health', MS='The status of health in suburban populations.'),
  'D018882': NtO(MH='Heat Stress Disorders', MS='A group of conditions that develop due to overexposure or overexertion in excessive environmental heat.'),
  'D018883': NtO(MH='Heat Stroke', MS='A condition caused by the failure of body to dissipate heat in an excessively hot environment or during PHYSICAL EXERTION in a hot environment. Contrast to HEAT EXHAUSTION, the body temperature in heat stroke patient is dangerously high with red, hot skin accompanied by DELUSIONS; CONVULSIONS; or COMA. It can be a life-threatening emergency and is most common in infants and the elderly.'),
  'D018884': NtO(MH='Ixodes', MS='The largest genus of TICKS in the family IXODIDAE, containing over 200 species. Many infest humans and other mammals and several are vectors of diseases such as LYME DISEASE, tick-borne encephalitis (ENCEPHALITIS, TICK-BORNE), and KYASANUR FOREST DISEASE.'),
  'D018886': NtO(MH='Aphasia, Conduction', MS="A type of fluent aphasia characterized by an impaired ability to repeat one and two word phrases, despite retained comprehension. This condition is associated with dominant hemisphere lesions involving the arcuate fasciculus (a white matter projection between Broca's and Wernicke's areas) and adjacent structures. Like patients with Wernicke aphasia (APHASIA, WERNICKE), patients with conduction aphasia are fluent but commit paraphasic errors during attempts at written and oral forms of communication. (From Adams et al., Principles of Neurology, 6th ed, p482; Brain & Bannister, Clinical Neurology, 7th ed, p142; Kandel et al., Principles of Neural Science, 3d ed, p848)"),
  'D018887': NtO(MH='Landau-Kleffner Syndrome', MS='A syndrome characterized by the onset of isolated language dysfunction in otherwise normal children (age of onset 4-7 years) and epileptiform discharges on ELECTROENCEPHALOGRAPHY. Seizures, including atypical absence (EPILEPSY, ABSENCE), complex partial (EPILEPSY, COMPLEX PARTIAL), and other types may occur. The electroencephalographic abnormalities and seizures tend to resolve by puberty. The language disorder may also resolve although some individuals are left with severe language dysfunction, including APHASIA and auditory AGNOSIA. (From Menkes, Textbook of Child Neurology, 5th ed, pp749-50; J Child Neurol 1997 Nov;12(8):489-495).'),
  'D018888': NtO(MH='Aphasia, Primary Progressive', MS='A progressive form of dementia characterized by the global loss of language abilities and initial preservation of other cognitive functions. Fluent and nonfluent subtypes have been described. Eventually a pattern of global cognitive dysfunction, similar to ALZHEIMER DISEASE, emerges. Pathologically, there are no Alzheimer or PICK DISEASE like changes, however, spongiform changes of cortical layers II and III are present in the TEMPORAL LOBE and FRONTAL LOBE. (From Brain 1998 Jan;121(Pt 1):115-26)'),
  'D018889': NtO(MH='Ilizarov Technique', MS='A bone fixation technique using an external fixator (FIXATORS, EXTERNAL) for lengthening limbs, correcting pseudarthroses and other deformities, and assisting the healing of otherwise hopeless traumatic or pathological fractures and infections, such as chronic osteomyelitis. The method was devised by the Russian orthopedic surgeon Gavriil Abramovich Ilizarov (1921-1992). (From Bull Hosp Jt Dis 1992 Summer;52(1):1)'),
  'D018890': NtO(MH='Chemoprevention', MS='The use of chemical compounds to prevent the development of a specific disease.'),
  'D018891': NtO(MH='Dentate Gyrus', MS='GRAY MATTER situated above the GYRUS HIPPOCAMPI. It is composed of three layers. The molecular layer is continuous with the HIPPOCAMPUS in the hippocampal fissure. The granular layer consists of closely arranged spherical or oval neurons, called GRANULE CELLS, whose AXONS pass through the polymorphic layer ending on the DENDRITES of PYRAMIDAL CELLS in the hippocampus.'),
  'D018892': NtO(MH='Proton-Motive Force', MS='Energy that is generated by the transfer of protons or electrons across an energy-transducing membrane and that can be used for chemical, osmotic, or mechanical work. Proton-motive force can be generated by a variety of phenomena including the operation of an electron transport chain, illumination of a PURPLE MEMBRANE, and the hydrolysis of ATP by a proton ATPase. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed, p171)'),
  'D018893': NtO(MH='Bronchoalveolar Lavage', MS='Washing out of the lungs with saline or mucolytic agents for diagnostic or therapeutic purposes. It is very useful in the diagnosis of diffuse pulmonary infiltrates in immunosuppressed patients.'),
  'D018894': NtO(MH='Reverse Transcriptase Inhibitors', MS='Inhibitors of reverse transcriptase (RNA-DIRECTED DNA POLYMERASE), an enzyme that synthesizes DNA on an RNA template.'),
  'D018895': NtO(MH='Microsatellite Repeats', MS='A variety of simple repeat sequences that are distributed throughout the GENOME. They are characterized by a short repeat unit of 2-8 basepairs that is repeated up to 100 times. They are also known as short tandem repeats (STRs).'),
  'D018899': NtO(MH='CpG Islands', MS="Areas of increased density of the dinucleotide sequence cytosine--phosphate diester--guanine. They form stretches of DNA several hundred to several thousand base pairs long. In humans there are about 45,000 CpG islands, mostly found at the 5' ends of genes. They are unmethylated except for those on the inactive X chromosome and some associated with imprinted genes."),
  'D018900': NtO(MH='Dinucleotide Repeats', MS='The most common of the microsatellite tandem repeats (MICROSATELLITE REPEATS) dispersed in the euchromatic arms of chromosomes. They consist of two nucleotides repeated in tandem; guanine and thymine, (GT)n, is the most frequently seen.'),
  'D018901': NtO(MH='Peroxisomal Disorders', MS='A heterogeneous group of inherited metabolic disorders marked by absent or dysfunctional PEROXISOMES. Peroxisomal enzymatic abnormalities may be single or multiple. Biosynthetic peroxisomal pathways are compromised, including the ability to synthesize ether lipids and to oxidize long-chain fatty acid precursors. Diseases in this category include ZELLWEGER SYNDROME; INFANTILE REFSUM DISEASE; rhizomelic chondrodysplasia (CHONDRODYSPLASIA PUNCTATA, RHIZOMELIC); hyperpipecolic acidemia; neonatal adrenoleukodystrophy; and ADRENOLEUKODYSTROPHY (X-linked). Neurologic dysfunction is a prominent feature of most peroxisomal disorders.'),
  'D018902': NtO(MH='Chondrodysplasia Punctata, Rhizomelic', MS='An autosomal recessive form of CHONDRODYSPLASIA PUNCTATA characterized by defective plasmalogen biosynthesis and impaired peroxisomes. Patients have shortened proximal limbs and severely disturbed endochondral bone formation. The metabolic defects associated with the impaired peroxisomes are present only in the rhizomelic form of chondrodysplasia punctata. (From Scriver et al, Metabolic Basis of Inherited Disease, 6th ed, p1497)'),
  'D018903': NtO(MH='Photorefractive Keratectomy', MS='A type of refractive surgery of the CORNEA to correct MYOPIA and ASTIGMATISM. An EXCIMER LASER is used directly on the surface of the EYE to remove some of the CORNEAL EPITHELIUM thus reshaping the anterior curvature of the cornea.'),
  'D018906': NtO(MH='Antineoplastic Agents, Alkylating', MS='A class of drugs that differs from other alkylating agents used clinically in that they are monofunctional and thus unable to cross-link cellular macromolecules. Among their common properties are a requirement for metabolic activation to intermediates with antitumor efficacy and the presence in their chemical structures of N-methyl groups, that after metabolism, can covalently modify cellular DNA. The precise mechanisms by which each of these drugs acts to kill tumor cells are not completely understood. (From AMA, Drug Evaluations Annual, 1994, p2026)'),
  'D018907': NtO(MH='Privacy', MS="The state of being free from intrusion or disturbance in one's private life or affairs. (Random House Unabridged Dictionary, 2d ed, 1993)"),
  'D018908': NtO(MH='Muscle Weakness', MS='A vague complaint of debility, fatigue, or exhaustion attributable to weakness of various muscles. The weakness can be characterized as subacute or chronic, often progressive, and is a manifestation of many muscle and neuromuscular diseases. (From Wyngaarden et al., Cecil Textbook of Medicine, 19th ed, p2251)'),
  'D018909': NtO(MH='src Homology Domains', MS='Regions of AMINO ACID SEQUENCE similarity in the SRC-FAMILY TYROSINE KINASES that fold into specific functional tertiary structures. The SH1 domain is a CATALYTIC DOMAIN. SH2 and SH3 domains are protein interaction domains. SH2 usually binds PHOSPHOTYROSINE-containing proteins and SH3 interacts with CYTOSKELETAL PROTEINS.'),
  'D018910': NtO(MH='Video Games', MS='A form of interactive entertainment in which the player controls electronically generated images that appear on a video display screen. This includes video games played in the home on special machines or home computers, and those played in arcades.'),
  'D018911': NtO(MH='Trinucleotide Repeats', MS='Microsatellite repeats consisting of three nucleotides dispersed in the euchromatic arms of chromosomes.'),
  'D018912': NtO(MH='Arabs', MS='Members of a Semitic people inhabiting the Arabian peninsula or other countries of the Middle East and North Africa. The term may be used with reference to ancient, medieval, or modern ethnic or cultural groups. (From Random House Unabridged Dictionary, 2d ed)'),
  'D018913': NtO(MH='Event-Related Potentials, P300', MS='A late-appearing component of the event-related potential. P300 stands for a positive deflection in the event-related voltage potential at 300 millisecond poststimulus. Its amplitude increases with unpredictable, unlikely, or highly significant stimuli and thereby constitutes an index of mental activity. (From Campbell, Psychiatric Dictionary, 6th ed)'),
  'D018914': NtO(MH='Cells, Immobilized', MS='Microbial, plant, or animal cells which are immobilized by attachment to solid structures, usually a column matrix. A common use of immobilized cells is in biotechnology for the bioconversion of a substrate to a particular product. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D018915': NtO(MH='Root Canal Preparation', MS='Preparatory activities in ROOT CANAL THERAPY by partial or complete extirpation of diseased pulp, cleaning and sterilization of the empty canal, enlarging and shaping the canal to receive the sealing material. The cavity may be prepared by mechanical, sonic, chemical, or other means. (From Dorland, 28th ed, p1700)'),
  'D018916': NtO(MH='Blood-Aqueous Barrier', MS='The selectively permeable barrier, in the EYE, formed by the nonpigmented layer of the EPITHELIUM of the CILIARY BODY, and the ENDOTHELIUM of the BLOOD VESSELS of the IRIS. TIGHT JUNCTIONS joining adjacent cells keep the barrier between cells continuous.'),
  'D018917': NtO(MH='Optic Neuropathy, Ischemic', MS='Ischemic injury to the OPTIC NERVE which usually affects the OPTIC DISK (optic neuropathy, anterior ischemic) and less frequently the retrobulbar portion of the nerve (optic neuropathy, posterior ischemic). The injury results from occlusion of arterial blood supply which may result from TEMPORAL ARTERITIS; ATHEROSCLEROSIS; COLLAGEN DISEASES; EMBOLISM; DIABETES MELLITUS; and other conditions. The disease primarily occurs in the sixth decade or later and presents with the sudden onset of painless and usually severe monocular visual loss. Anterior ischemic optic neuropathy also features optic disk edema with microhemorrhages. The optic disk appears normal in posterior ischemic optic neuropathy. (Glaser, Neuro-Ophthalmology, 2nd ed, p135)'),
  'D018918': NtO(MH='Phacoemulsification', MS='A procedure for removal of the crystalline lens in cataract surgery in which an anterior capsulectomy is performed by means of a needle inserted through a small incision at the temporal limbus, allowing the lens contents to fall through the dilated pupil into the anterior chamber where they are broken up by the use of ultrasound and aspirated out of the eye through the incision. (Cline, et al., Dictionary of Visual Science, 4th ed & In Focus 1993;1(1):1)'),
  'D018919': NtO(MH='Neovascularization, Physiologic', MS='The development of new BLOOD VESSELS during the restoration of BLOOD CIRCULATION during the healing process.'),
  'D018920': NtO(MH='Coculture Techniques', MS='A technique of culturing mixed cell types in vitro to allow their synergistic or antagonistic interactions, such as on CELL DIFFERENTIATION or APOPTOSIS. Coculture can be of different types of cells, tissues, or organs from normal or disease states.'),
  'D018921': NtO(MH='Leukostasis', MS='Abnormal intravascular leukocyte aggregation and clumping often seen in leukemia patients. The brain and lungs are the two most commonly affected organs. This acute syndrome requires aggressive cytoreductive modalities including chemotherapy and/or leukophoresis. It is differentiated from LEUKEMIC INFILTRATION which is a neoplastic process where leukemic cells invade organs.'),
  'D018922': NtO(MH='HL-60 Cells', MS='A promyelocytic cell line derived from a patient with ACUTE PROMYELOCYTIC LEUKEMIA. HL-60 cells lack specific markers for LYMPHOID CELLS but express surface receptors for FC FRAGMENTS and COMPLEMENT SYSTEM PROTEINS. They also exhibit phagocytic activity and responsiveness to chemotactic stimuli. (From Hay et al., American Type Culture Collection, 7th ed, pp127-8)'),
  'D018923': NtO(MH='Persian Gulf Syndrome', MS='Unexplained symptoms reported by veterans of the Persian Gulf War with Iraq in 1991. The symptoms reported include fatigue, skin rash, muscle and joint pain, headaches, loss of memory, shortness of breath, gastrointestinal and respiratory symptoms, and extreme sensitivity to commonly occurring chemicals. (Nature 1994 May 5;369(6475):8)'),
  'D018924': NtO(MH='Vascular Capacitance', MS="The measure of a BLOOD VESSEL's ability to increase the volume of BLOOD it holds without a large increase in BLOOD PRESSURE. The vascular capacitance is equal to the change in volume divided by the change in pressure."),
  'D018925': NtO(MH='Chemokines', MS='Class of pro-inflammatory cytokines that have the ability to attract and activate leukocytes. They can be divided into at least three structural branches: C; (CHEMOKINES, C); CC; (CHEMOKINES, CC); and CXC; (CHEMOKINES, CXC); according to variations in a shared cysteine motif.'),
  'D018926': NtO(MH='Anti-Allergic Agents', MS='Agents that are used to treat allergic reactions. Most of these drugs act by preventing the release of inflammatory mediators or inhibiting the actions of released mediators on their target cells. (From AMA Drug Evaluations Annual, 1994, p475)'),
  'D018927': NtO(MH='Anti-Asthmatic Agents', MS='Drugs that are used to treat asthma.'),
  'D018928': NtO(MH='Immunity, Mucosal', MS='Nonsusceptibility to the pathogenic effects of foreign microorganisms or antigenic substances as a result of antibody secretions of the mucous membranes. Mucosal epithelia in the gastrointestinal, respiratory, and reproductive tracts produce a form of IgA (IMMUNOGLOBULIN A, SECRETORY) that serves to protect these ports of entry into the body.'),
  'D018929': NtO(MH='Cell Culture Techniques', MS='Methods for maintaining or growing CELLS in vitro.'),
  'D018931': NtO(MH='Antineoplastic Agents, Hormonal', MS='Antineoplastic agents that are used to treat hormone-sensitive tumors. Hormone-sensitive tumors may be hormone-dependent, hormone-responsive, or both. A hormone-dependent tumor regresses on removal of the hormonal stimulus, by surgery or pharmacological block. Hormone-responsive tumors may regress when pharmacologic amounts of hormones are administered regardless of whether previous signs of hormone sensitivity were observed. The major hormone-responsive cancers include carcinomas of the breast, prostate, and endometrium; lymphomas; and certain leukemias. (From AMA Drug Evaluations Annual 1994, p2079)'),
  'D018932': NtO(MH='Chemokine CCL2', MS='A chemokine that is a chemoattractant for MONOCYTES and may also cause cellular activation of specific functions related to host defense. It is produced by LEUKOCYTES of both monocyte and lymphocyte lineage and by FIBROBLASTS during tissue injury. It has specificity for CCR2 RECEPTORS.'),
  'D018933': NtO(MH='Pili, Sex', MS='Filamentous or elongated proteinaceous structures which extend from the cell surface in gram-negative bacteria that contain certain types of conjugative plasmid. These pili are the organs associated with genetic transfer and have essential roles in conjugation. Normally, only one or a few pili occur on a given donor cell. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed, p675) This preferred use of "pili" refers to the sexual appendage, to be distinguished from bacterial fimbriae (FIMBRIAE, BACTERIAL), also known as common pili, which are usually concerned with adhesion.'),
  'D018934': NtO(MH='Fournier Gangrene', MS='An acute necrotic infection of the SCROTUM; PENIS; or PERINEUM. It is characterized by scrotum pain and redness with rapid progression to gangrene and sloughing of tissue. Fournier gangrene is usually secondary to perirectal or periurethral infections associated with local trauma, operative procedures, or urinary tract disease.'),
  'D018936': NtO(MH='Hepatitis C Antigens', MS='Antigens of the virions of HEPACIVIRUS, their surface, core, or other associated antigens.'),
  'D018937': NtO(MH='Hepatitis C Antibodies', MS='Antibodies to the HEPATITIS C ANTIGENS including antibodies to envelope, core, and non-structural proteins.'),
  'D018938': NtO(MH='Caco-2 Cells', MS='Human colonic ADENOCARCINOMA cells that are able to express differentiation features characteristic of mature intestinal cells, such as ENTEROCYTES. These cells are valuable in vitro tools for studies related to intestinal cell function and differentiation.'),
  'D018939': NtO(MH='Holistic Nursing', MS="A philosophy of nursing practice that takes into account total patient care, considering the physical, emotional, social, economic, and spiritual needs of patients, their response to their illnesses, and the effect of illness on patients' abilities to meet self-care needs. (From Mosby's Medical, Nursing, & Allied Health Dictionary, 4th ed, p745)"),
  'D018940': NtO(MH='Elasmobranchii', MS='A subclass of cartilaginous fish comprising the SHARKS; rays; skates (SKATES (FISH);), and sawfish. Elasmobranchs are typically predaceous, relying more on smell (the olfactory capsules are relatively large) than sight (the eyes are relatively small) for obtaining their food.'),
  'D018941': NtO(MH='Antigens, CD19', MS='Differentiation antigens expressed on B-lymphocytes and B-cell precursors. They are involved in regulation of B-cell proliferation.'),
  'D018942': NtO(MH='Macrolides', MS='A group of often glycosylated macrocyclic compounds formed by chain extension of multiple PROPIONATES cyclized into a large (typically 12, 14, or 16)-membered lactone. Macrolides belong to the POLYKETIDES class of natural products, and many members exhibit ANTIBIOTIC properties.'),
  'D018943': NtO(MH='Anthracyclines', MS='Organic compounds that have a tetrahydronaphthacenedione ring structure attached by a glycosidic linkage to the amino sugar daunosamine.'),
  'D018945': NtO(MH='Monocyte Chemoattractant Proteins', MS='Chemokines that are chemoattractants for monocytes. These CC chemokines (cysteines adjacent) number at least three including CHEMOKINE CCL2.'),
  'D018946': NtO(MH='Chemokine CCL5', MS='A CC-type chemokine that is a chemoattractant for EOSINOPHILS; MONOCYTES; and LYMPHOCYTES. It is a potent and selective eosinophil chemotaxin that is stored in and released from PLATELETS and activated T-LYMPHOCYTES. Chemokine CCL5 is specific for CCR1 RECEPTORS; CCR3 RECEPTORS; and CCR5 RECEPTORS. The acronym RANTES refers to Regulated on Activation, Normal T Expressed and Secreted.'),
  'D018947': NtO(MH='Samoa', MS="A group of islands in the southwest central Pacific, divided into AMERICAN SAMOA and the INDEPENDENT STATE OF SAMOA (Western Samoa). First European contact was made in 1722 by Jacob Roggeveen, a Dutchman. In 1768 they were named Navigators Islands by Louis de Bougainville. The present name may derive from that of a local chieftain or from a local word meaning place of the moa, a now-extinct island bird. (From Webster's New Geographical Dictionary, 1988, p1061 & Room, Brewer's Dictionary of Names, 1992, p481)"),
  'D018948': NtO(MH='American Samoa', MS="A group of islands of SAMOA, in the southwest central Pacific. Its capital is Pago Pago. The islands were ruled by native chiefs until about 1869. An object of American interest beginning in 1839, Pago Pago and trading and extraterritorial rights were granted to the United States in 1878. The United States, Germany, and England administered the islands jointly 1889-99, but in 1899 they were granted to the United States by treaty. The Department of the Interior has administered American Samoa since 1951. (From Webster's New Geographical Dictionary, 1988, p44)"),
  'D018949': NtO(MH='Antigens, CD1', MS='Glycoproteins expressed on cortical thymocytes and on some dendritic cells and B-cells. Their structure is similar to that of MHC Class I and their function has been postulated as similar also. CD1 antigens are highly specific markers for human LANGERHANS CELLS.'),
  'D018950': NtO(MH='Lipopolysaccharide Receptors', MS='Glycolipid-anchored membrane glycoproteins expressed on cells of the myelomonocyte lineage including MONOCYTES; MACROPHAGES; and some GRANULOCYTES. They function as receptors for the complex of lipopolysaccharide (LPS) and LPS-binding protein.'),
  'D018951': NtO(MH='Antigens, CD20', MS='Unglycosylated phosphoproteins expressed only on B-cells. They are regulators of transmembrane Ca2+ conductance and thought to play a role in B-cell activation and proliferation.'),
  'D018952': NtO(MH='Antigens, CD34', MS='Glycoproteins found on immature hematopoietic cells and endothelial cells. They are the only molecules to date whose expression within the blood system is restricted to a small number of progenitor cells in the bone marrow.'),
  'D018953': NtO(MH='Kinesiology, Applied', MS="The study of muscles and the movement of the human body. In holistic medicine it is the balance of movement and the interaction of a person's energy systems. Applied kinesiology is the name given by its inventor, Dr. George Goodheart, to the system of applying muscle testing diagnostically and therapeutically to different aspects of health care. (Thorsons Introductory Guide to Kinesiology, 1992, p13)"),
  'D018954': NtO(MH='Drugs, Essential', MS='Drugs considered essential to meet the health needs of a population as well as to control drug costs.'),
  'D018955': NtO(MH='CD36 Antigens', MS='Leukocyte differentiation antigens and major platelet membrane glycoproteins present on MONOCYTES; ENDOTHELIAL CELLS; PLATELETS; and mammary EPITHELIAL CELLS. They play major roles in CELL ADHESION; SIGNAL TRANSDUCTION; and regulation of angiogenesis. CD36 is a receptor for THROMBOSPONDINS and can act as a scavenger receptor that recognizes and transports oxidized LIPOPROTEINS and FATTY ACIDS.'),
  'D018956': NtO(MH='CD5 Antigens', MS='Glycoproteins expressed on all mature T-cells, thymocytes, and a subset of mature B-cells. Antibodies specific for CD5 can enhance T-cell receptor-mediated T-cell activation. The B-cell-specific molecule CD72 is a natural ligand for CD5. (From Abbas et al., Cellular and Molecular Immunology, 2d ed, p156)'),
  'D018957': NtO(MH='CD59 Antigens', MS='Small glycoproteins found on both hematopoietic and non-hematopoietic cells. CD59 restricts the cytolytic activity of homologous complement by binding to C8 and C9 and blocking the assembly of the membrane attack complex. (From Barclay et al., The Leukocyte Antigen FactsBook, 1993, p234)'),
  'D018958': NtO(MH='CD55 Antigens', MS='GPI-linked membrane proteins broadly distributed among hematopoietic and non-hematopoietic cells. CD55 prevents the assembly of C3 CONVERTASE or accelerates the disassembly of preformed convertase, thus blocking the formation of the membrane attack complex.'),
  'D018959': NtO(MH='CD57 Antigens', MS='Oligosaccharide antigenic determinants found principally on NK CELLS and T-CELLS. They are expressed at high levels in terminally differentiated or senescent cells with reduced proliferative capacity; however, their role in the immune response is poorly understood.'),
  'D018960': NtO(MH='Hyaluronan Receptors', MS='Acidic sulfated integral membrane glycoproteins expressed in several alternatively spliced and variable glycosylated forms on a wide variety of cell types including mature T-cells, B-cells, medullary THYMOCYTES; GRANULOCYTES; MACROPHAGES; erythrocytes, and fibroblasts. Their interaction with HYALURONIC ACID mediates binding of lymphocytes to high endothelial VENULES.'),
  'D018961': NtO(MH='Educational Technology', MS='Systematic identification, development, organization, or utilization of educational resources and the management of these processes. It is occasionally used also in a more limited sense to describe the use of equipment-oriented techniques or audiovisual aids in educational settings. (Thesaurus of ERIC Descriptors, December 1993, p132)'),
  'D018962': NtO(MH='Phlebotomy', MS='The techniques used to draw blood from a vein for diagnostic purposes or for treatment of certain blood disorders such as erythrocytosis, hemochromatosis, polycythemia vera, and porphyria cutanea tarda.'),
  'D018963': NtO(MH='Hepatitis Antigens', MS='Antigens from any of the hepatitis viruses including surface, core, and other associated antigens.'),
  'D018965': NtO(MH='Frameshifting, Ribosomal', MS='A directed change in translational READING FRAMES that allows the production of a single protein from two or more OVERLAPPING GENES. The process is programmed by the nucleotide sequence of the MRNA and is sometimes also affected by the secondary or tertiary mRNA structure. It has been described mainly in VIRUSES (especially RETROVIRUSES); RETROTRANSPOSONS; and bacterial insertion elements but also in some cellular genes.'),
  'D018966': NtO(MH='Library Materials', MS="Print and non-print materials collected, processed, and stored by libraries. They comprise books, periodicals, pamphlets, reports, microforms, maps, manuscripts, motion pictures, and all other forms of audiovisual records. (Harrod, The Librarians' Glossary, 4th ed, p497)"),
  'D018967': NtO(MH='Risperidone', MS='A selective blocker of DOPAMINE D2 RECEPTORS and SEROTONIN 5-HT2 RECEPTORS that acts as an atypical antipsychotic agent. It has been shown to improve both positive and negative symptoms in the treatment of SCHIZOPHRENIA.'),
  'D018968': NtO(MH='CD58 Antigens', MS='Glycoproteins with a wide distribution on hematopoietic and non-hematopoietic cells and strongly expressed on MACROPHAGES. CD58 mediates cell adhesion by binding to CD2; (CD2 ANTIGENS); and this enhances antigen-specific T-cell activation.'),
  'D018969': NtO(MH='Insulin-Like Growth Factor Binding Proteins', MS='A family of soluble proteins that bind insulin-like growth factors and modulate their biological actions at the cellular level. (Int J Gynaecol Obstet 1992;39(1):3-9)'),
  'D018970': NtO(MH='Insulin-Like Growth Factor Binding Protein 1', MS='One of the six homologous proteins that specifically bind insulin-like growth factors (SOMATOMEDINS) and modulate their mitogenic and metabolic actions. The function of this protein is not completely defined. However, several studies demonstrate that it inhibits IGF binding to cell surface receptors and thereby inhibits IGF-mediated mitogenic and cell metabolic actions. (Proc Soc Exp Biol Med 1993;204(1):4-29)'),
  'D018971': NtO(MH='Insulin-Like Growth Factor Binding Protein 2', MS='One of the six homologous soluble proteins that bind insulin-like growth factors (SOMATOMEDINS) and modulate their mitogenic and metabolic actions at the cellular level.'),
  'D018972': NtO(MH='Insulin-Like Growth Factor Binding Protein 3', MS='One of the six homologous soluble proteins that bind insulin-like growth factors (SOMATOMEDINS) and modulate their mitogenic and metabolic actions at the cellular level.'),
  'D018973': NtO(MH='Intellectual Property', MS="Property, such as patents, trademarks, and copyright, that results from creative effort. The Patent and Copyright Clause (Art. 1, Sec. 8, cl. 8) of the United States Constitution provides for promoting the progress of science and useful arts by securing for limited times to authors and inventors, the exclusive right to their respective writings and discoveries. (From Black's Law Dictionary, 5th ed, p1014)"),
  'D018974': NtO(MH='Insulin-Like Growth Factor Binding Protein 4', MS='One of the six homologous soluble proteins that bind insulin-like growth factors (SOMATOMEDINS) and modulate their mitogenic and metabolic actions at the cellular level.'),
  'D018975': NtO(MH='Insulin-Like Growth Factor Binding Protein 5', MS='One of the six homologous soluble proteins that bind insulin-like growth factors (SOMATOMEDINS) and modulate their mitogenic and metabolic actions at the cellular level.'),
  'D018976': NtO(MH='Insulin-Like Growth Factor Binding Protein 6', MS='One of the six homologous soluble proteins that bind insulin-like growth factors (SOMATOMEDINS) and modulate their mitogenic and metabolic actions at the cellular level.'),
  'D018977': NtO(MH='Micronutrients', MS='Essential dietary elements or organic compounds that are required in only small quantities for normal physiologic processes to occur.'),
  'D018978': NtO(MH='Molting', MS='Periodic casting off FEATHERS; HAIR; or cuticle. Molting is a process of sloughing or desquamation, especially the shedding of an outer covering and the development of a new one. This phenomenon permits growth in ARTHROPODS, skin renewal in AMPHIBIANS and REPTILES, and the shedding of winter coats in BIRDS and MAMMALS.'),
  'D018979': NtO(MH='Myositis, Inclusion Body', MS='Progressive myopathies characterized by the presence of inclusion bodies on muscle biopsy. Sporadic and hereditary forms have been described. The sporadic form is an acquired, adult-onset inflammatory vacuolar myopathy affecting proximal and distal muscles. Familial forms usually begin in childhood and lack inflammatory changes. Both forms feature intracytoplasmic and intranuclear inclusions in muscle tissue. (Adams et al., Principles of Neurology, 6th ed, pp1409-10)'),
  'D018980': NtO(MH='Williams Syndrome', MS='A disorder caused by hemizygous microdeletion of about 28 genes on chromosome 7q11.23, including the ELASTIN gene. Clinical manifestations include SUPRAVALVULAR AORTIC STENOSIS; MENTAL RETARDATION; elfin facies; impaired visuospatial constructive abilities; and transient HYPERCALCEMIA in infancy. The condition affects both sexes, with onset at birth or in early infancy.'),
  'D018981': NtO(MH='Congenital Disorders of Glycosylation', MS='A genetically heterogeneous group of heritable disorders resulting from defects in protein N-glycosylation.'),
  'D018983': NtO(MH='DNA Footprinting', MS='A method for determining the sequence specificity of DNA-binding proteins. DNA footprinting utilizes a DNA damaging agent (either a chemical reagent or a nuclease) which cleaves DNA at every base pair. DNA cleavage is inhibited where the ligand binds to DNA. (from Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D018984': NtO(MH='Epitopes, T-Lymphocyte', MS='Antigenic determinants recognized and bound by the T-cell receptor. Epitopes recognized by the T-cell receptor are often located in the inner, unexposed side of the antigen, and become accessible to the T-cell receptors after proteolytic processing of the antigen.'),
  'D018985': NtO(MH='Epitopes, B-Lymphocyte', MS='Antigenic determinants recognized and bound by the B-cell receptor. Epitopes recognized by the B-cell receptor are located on the surface of the antigen.'),
  'D018986': NtO(MH='Motor Vehicles', MS='AUTOMOBILES, trucks, buses, or similar engine-driven conveyances. (From Random House Unabridged Dictionary, 2d ed)'),
  'D018987': NtO(MH='Salivary Ducts', MS='Any of the ducts which transport saliva. Salivary ducts include the parotid duct, the major and minor sublingual ducts, and the submandibular duct.'),
  'D018988': NtO(MH='Bacterial Translocation', MS='The passage of viable bacteria from the GASTROINTESTINAL TRACT to extra-intestinal sites, such as the mesenteric lymph node complex, liver, spleen, kidney, and blood. Factors that promote bacterial translocation include overgrowth with gram-negative enteric bacilli, impaired host immune defenses, and injury to the INTESTINAL MUCOSA resulting in increased intestinal permeability. Bacterial translocation from the lung to the circulation is also possible and sometimes accompanies MECHANICAL VENTILATION.'),
  'D018991': NtO(MH='Myelin Proteolipid Protein', MS='A myelin protein that is the major component of the organic solvent extractable lipoprotein complexes of whole brain. It has been the subject of much study because of its unusual physical properties. It remains soluble in chloroform even after essentially all of its bound lipids have been removed. (From Siegel et al., Basic Neurochemistry, 4th ed, p122)'),
  'D018992': NtO(MH='Myelin-Associated Glycoprotein', MS='A myelin protein found in the periaxonal membrane of both the central and peripheral nervous systems myelin sheaths. It binds to cells surface receptors found on AXONS and may regulate cellular interactions between MYELIN and AXONS.'),
  'D018993': NtO(MH='Myelin P0 Protein', MS='A protein that accounts for more than half of the peripheral nervous system myelin protein. The extracellular domain of this protein is believed to engage in adhesive interactions and thus hold the myelin membrane compact. It can behave as a homophilic adhesion molecule through interactions with its extracellular domains. (From J Cell Biol 1994;126(4):1089-97)'),
  'D018994': NtO(MH='Myosin Light Chains', MS='The smaller subunits of MYOSINS that bind near the head groups of MYOSIN HEAVY CHAINS. The myosin light chains have a molecular weight of about 20 KDa and there are usually one essential and one regulatory pair of light chains associated with each heavy chain. Many myosin light chains that bind calcium are considered "calmodulin-like" proteins.'),
  'D018995': NtO(MH='Myosin Heavy Chains', MS='The larger subunits of MYOSINS. The heavy chains have a molecular weight of about 230 kDa and each heavy chain is usually associated with a dissimilar pair of MYOSIN LIGHT CHAINS. The heavy chains possess actin-binding and ATPase activity.'),
  'D018996': NtO(MH='Myelin P2 Protein', MS='A positively charged protein found in peripheral nervous system MYELIN. Sensitive immunological techniques have demonstrated that P2 is expressed in small amounts of central nervous system myelin sheaths of some species. It is an antigen for experimental allergic neuritis (NEURITIS, EXPERIMENTAL ALLERGIC), the peripheral nervous system counterpart of experimental allergic encephalomyelitis. (From Siegel et al., Basic Neurochemistry, 5th ed, p133)'),
  'D018997': NtO(MH='Chorionic Gonadotropin, beta Subunit, Human', MS='The beta subunit of human CHORIONIC GONADOTROPIN. Its structure is similar to the beta subunit of LUTEINIZING HORMONE, except for the additional 30 amino acids at the carboxy end with the associated carbohydrate residues. HCG-beta is used as a diagnostic marker for early detection of pregnancy, spontaneous abortion (ABORTION, SPONTANEOUS); ECTOPIC PREGNANCY; HYDATIDIFORM MOLE; CHORIOCARCINOMA; or DOWN SYNDROME.'),
  'D018998': NtO(MH='Wheat Germ Agglutinin-Horseradish Peroxidase Conjugate', MS='The lectin wheatgerm agglutinin conjugated to the enzyme HORSERADISH PEROXIDASE. It is widely used for tracing neural pathways.'),
  'D018999': NtO(MH='Charybdotoxin', MS='A 37-amino acid residue peptide isolated from the scorpion Leiurus quinquestriatus hebraeus. It is a neurotoxin that inhibits calcium activated potassium channels.'),
  'D019000': NtO(MH='Phosphotyrosine', MS='An amino acid that occurs in endogenous proteins. Tyrosine phosphorylation and dephosphorylation plays a role in cellular signal transduction and possibly in cell growth control and carcinogenesis.'),
  'D019001': NtO(MH='Nitric Oxide Synthase', MS='An NADPH-dependent enzyme that catalyzes the conversion of L-ARGININE and OXYGEN to produce CITRULLINE and NITRIC OXIDE.'),
  'D019002': NtO(MH='CD56 Antigen', MS='The 140 kDa isoform of NCAM (neural cell adhesion molecule) containing a transmembrane domain and short cytoplasmic tail. It is expressed by all lymphocytes mediating non-MHC restricted cytotoxicity and is present on some neural tissues and tumors.'),
  'D019003': NtO(MH='Deoxyribonuclease HpaII', MS='One of the Type II site-specific deoxyribonucleases (EC 3.1.21.4). It recognizes and cleaves the sequences C/CGG and GGC/C at the slash. HpaII is from Haemophilus parainfluenzae. Several isoschizomers have been identified. EC 3.1.21.-.'),
  'D019004': NtO(MH='Galanin', MS='A neuropeptide of 29-30 amino acids depending on the species. Galanin is widely distributed throughout the BRAIN; SPINAL CORD; and INTESTINES. There are various subtypes of GALANIN RECEPTORS implicating roles of galanin in regulating FOOD INTAKE; pain perception; memory; and other neuroendocrine functions.'),
  'D019005': NtO(MH='Cystic Fibrosis Transmembrane Conductance Regulator', MS='A chloride channel that regulates secretion in many exocrine tissues. Abnormalities in the CFTR gene have been shown to cause cystic fibrosis. (Hum Genet 1994;93(4):364-8)'),
  'D019006': NtO(MH='Neural Cell Adhesion Molecules', MS='Cell adhesion molecule involved in a diverse range of contact-mediated interactions among neurons, astrocytes, oligodendrocytes, and myotubes. It is widely but transiently expressed in many tissues early in embryogenesis. Four main isoforms exist, including CD56; (CD56 ANTIGEN); but there are many other variants resulting from alternative splicing and post-translational modifications. (From Pigott & Power, The Adhesion Molecule FactsBook, 1993, pp115-119)'),
  'D019007': NtO(MH='P-Selectin', MS='Cell adhesion molecule and CD antigen that mediates the adhesion of neutrophils and monocytes to activated platelets and endothelial cells.'),
  'D019008': NtO(MH='Drug Resistance, Neoplasm', MS='Resistance or diminished response of a neoplasm to an antineoplastic agent in humans, animals, or cell or tissue cultures.'),
  'D019009': NtO(MH='Proto-Oncogene Proteins c-kit', MS='A protein-tyrosine kinase receptor that is specific for STEM CELL FACTOR. This interaction is crucial for the development of hematopoietic, gonadal, and pigment stem cells. Genetic mutations that disrupt the expression of PROTO-ONCOGENE PROTEINS C-KIT are associated with PIEBALDISM, while overexpression or constitutive activation of the c-kit protein-tyrosine kinase is associated with tumorigenesis.'),
  'D019010': NtO(MH='Vascular Cell Adhesion Molecule-1', MS='Cytokine-induced cell adhesion molecule present on activated endothelial cells, tissue macrophages, dendritic cells, bone marrow fibroblasts, myoblasts, and myotubes. It is important for the recruitment of leukocytes to sites of inflammation. (From Pigott & Power, The Adhesion Molecule FactsBook, 1993, p154)'),
  'D019011': NtO(MH='Antigens, CD7', MS='Differentiation antigens expressed on pluripotential hematopoietic cells, most human thymocytes, and a major subset of peripheral blood T-lymphocytes. They have been implicated in integrin-mediated cellular adhesion and as signalling receptors on T-cells.'),
  'D019012': NtO(MH='Integrin beta1', MS='Integrin beta chain expressed as a heterodimer that is non-covalently associated with specific alpha-chains of the CD49 family (CD49a-f). It is expressed on resting and activated leukocytes and is a marker for all of the very late activation antigens on cells. (from: Barclay et al., The Leukocyte Antigen FactsBook, 1993, p164)'),
  'D019013': NtO(MH='CD40 Antigens', MS='Members of the tumor necrosis factor receptor superfamily with specificity for CD40 LIGAND. They are found on mature B-LYMPHOCYTES, some EPITHELIAL CELLS; and lymphoid DENDRITIC CELLS. Evidence suggests that CD40-dependent activation of B-cells is important for generation of memory B-cells within the germinal centers. Mutations in the CD40 antigen gene result in HYPER-IGM IMMUNODEFICIENCY SYNDROME, TYPE 3. Signaling of the receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D019014': NtO(MH='fas Receptor', MS='A tumor necrosis factor receptor subtype found in a variety of tissues and on activated LYMPHOCYTES. It has specificity for FAS LIGAND and plays a role in regulation of peripheral immune responses and APOPTOSIS. Multiple isoforms of the protein exist due to multiple ALTERNATIVE SPLICING. The activated receptor signals via a conserved death domain that associates with specific TNF RECEPTOR-ASSOCIATED FACTORS in the CYTOPLASM. Mutations in the CD95 gene are associated with cases of autoimmune lymphoproliferative syndrome.'),
  'D019015': NtO(MH='Geologic Sediments', MS="A mass of organic or inorganic solid fragmented material, or the solid fragment itself, that comes from the weathering of rock and is carried by, suspended in, or dropped by air, water, or ice. It refers also to a mass that is accumulated by any other natural agent and that forms in layers on the earth's surface, such as sand, gravel, silt, mud, fill, or loess. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed, p1689)"),
  'D019016': NtO(MH='Mineral Fibers', MS='Long, pliable, cohesive natural or manufactured filaments of various lengths. They form the structure of some minerals. The medical significance lies in their potential ability to cause various types of PNEUMOCONIOSIS (e.g., ASBESTOSIS) after occupational or environmental exposure. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed, p708)'),
  'D019017': NtO(MH='Eritrea', MS='A country of eastern Africa, west of the Red Sea, bordered west and northwest by SUDAN, and south by ETHIOPIA. Its capital is Asmara.'),
  'D019018': NtO(MH='Imagery, Psychotherapy', MS='The use of mental images produced by the imagination as a form of psychotherapy. It can be classified by the modality of its content: visual, verbal, auditory, olfactory, tactile, gustatory, or kinesthetic. Common themes derive from nature imagery (e.g., forests and mountains), water imagery (e.g., brooks and oceans), travel imagery, etc. Imagery is used in the treatment of mental disorders and in helping patients cope with other diseases. Imagery often forms a part of HYPNOSIS, of AUTOGENIC TRAINING, of RELAXATION TECHNIQUES, and of BEHAVIOR THERAPY. (From Encyclopedia of Human Behavior, vol. 4, pp29-30, 1994)'),
  'D019020': NtO(MH='Directed Molecular Evolution', MS='The techniques used to produce molecules exhibiting properties that conform to the demands of the experimenter. These techniques combine methods of generating structural changes with methods of selection. They are also used to examine proposed mechanisms of evolution under in vitro selection conditions.'),
  'D019021': NtO(MH='Natural History', MS='A former branch of knowledge embracing the study, description, and classification of natural objects (as animals, plants, and minerals) and thus including the modern sciences of zoology, botany, and mineralogy insofar as they existed at that time. In the 17th, 18th, and 19th centuries it was much used for the generalized pursuit of certain areas of science. (Webster, 3d ed; from Dr. James H. Cassedy, NLM History of Medicine Division)'),
  'D019022': NtO(MH='Almshouses', MS='Privately endowed or publicly funded institutions receiving and supporting the aged or infirm poor. They sometimes functioned as centers of health care before the establishment of formal hospitals.'),
  'D019023': NtO(MH='Stethoscopes', MS='Instruments intended to detect and study sound produced by the heart, lungs, or other parts of the body. (from UMDNS, 1999)'),
  'D019024': NtO(MH='Barber Surgeons', MS='In the late Middle Ages barbers who also let blood, sold unguents, pulled teeth, applied cups, and gave enemas. They generally had the right to practice surgery. By the 18th century barbers continued to practice minor surgery and dentistry and many famous surgeons acquired their skill in the shops of barbers. (From Castiglioni, A History of Medicine, 2d ed, pp402, 568, 658)'),
  'D019026': NtO(MH='Medicine Chests', MS="Boxes in which physicians kept their drugs and other medications, medical instruments and supplies, manuals, etc. As a carrying case or convenient storage receptacle, or a kind of portable pharmacy, the medicine chest was indispensable to the itinerant physician. The chest was usually larger and sturdier than a doctor's kit or bag."),
  'D019028': NtO(MH='Magnetic Resonance Imaging, Cine', MS='A type of imaging technique used primarily in the field of cardiology. By coordinating the fast gradient-echo MRI sequence with retrospective ECG-gating, numerous short time frames evenly spaced in the cardiac cycle are produced. These images are laced together in a cinematic display so that wall motion of the ventricles, valve motion, and blood flow patterns in the heart and great vessels can be visualized.'),
  'D019029': NtO(MH='Palau', MS="A republic consisting of a group of about 100 islands and islets in the western Pacific Ocean. Its capital is Koror. Under Spain it was administered as a part of the Caroline Islands but was sold to Germany in 1899. Seized by Japan in 1914, it was taken by the Allies in World War II in 1944. In 1947 it became part of the U.S. Trust Territory of the Pacific Islands, became internally self-governing in 1980, obtained independent control over its foreign policy (except defense) in 1986, and achieved total independence October 1, 1994. (Webster's New Geographical Dictionary, 1988, p915; telephone communication with Randy Flynn, Board on Geographic Names, 17 January 1995)"),
  'D019030': NtO(MH='Receptors, Vitronectin', MS='Receptors such as INTEGRIN ALPHAVBETA3 that bind VITRONECTIN with high affinity and play a role in cell migration. They also bind FIBRINOGEN; VON WILLEBRAND FACTOR; osteopontin; and THROMBOSPONDINS.'),
  'D019032': NtO(MH='Spectrometry, Mass, Matrix-Assisted Laser Desorption-Ionization', MS='A mass spectrometric technique that is used for the analysis of large biomolecules. Analyte molecules are embedded in an excess matrix of small organic molecules that show a high resonant absorption at the laser wavelength used. The matrix absorbs the laser energy, thus inducing a soft disintegration of the sample-matrix mixture into free (gas phase) matrix and analyte molecules and molecular ions. In general, only molecular ions of the analyte molecules are produced, and almost no fragmentation occurs. This makes the method well suited for molecular weight determinations and mixture analysis.'),
  'D019033': NtO(MH='Delivery of Health Care, Integrated', MS='A health care system which combines physicians, hospitals, and other medical services with a health plan to provide the complete spectrum of medical care for its customers. In a fully integrated system, the three key elements - physicians, hospital, and health plan membership - are in balance in terms of matching medical resources with the needs of purchasers and patients. (Coddington et al., Integrated Health Care: Reorganizing the Physician, Hospital and Health Plan Relationship, 1994, p7)'),
  'D019034': NtO(MH='CD-I', MS='An optical disk storage system used on specialized players that combine the functions of computer and CD player in a self-contained box, designed to be connected to a television set and a home stereo for video and sound output. The player is controlled with a hand-held remote unit resembling a television remote control. (J Allied Health 1993 Winter;22(1):131-8)'),
  'D019035': NtO(MH='Rural Health Services', MS='Health services, public or private, in rural areas. The services include the promotion of health and the delivery of health care.'),
  'D019036': NtO(MH='Suburban Health Services', MS='Health services, public or private, in suburban areas. The services include the promotion of health and the delivery of health care.'),
  'D019037': NtO(MH='Urban Health Services', MS='Health services, public or private, in urban areas. The services include the promotion of health and the delivery of health care.'),
  'D019038': NtO(MH='Platelet Glycoprotein GPIb-IX Complex', MS='Platelet membrane glycoprotein complex essential for normal platelet adhesion and clot formation at sites of vascular injury. It is composed of three polypeptides, GPIb alpha, GPIb beta, and GPIX. Glycoprotein Ib functions as a receptor for von Willebrand factor and for thrombin. Congenital deficiency of the GPIb-IX complex results in Bernard-Soulier syndrome. The platelet glycoprotein GPV associates with GPIb-IX and is also absent in Bernard-Soulier syndrome.'),
  'D019039': NtO(MH='Platelet Glycoprotein GPIIb-IIIa Complex', MS='Platelet membrane glycoprotein complex important for platelet adhesion and aggregation. It is an integrin complex containing INTEGRIN ALPHAIIB and INTEGRIN BETA3 which recognizes the arginine-glycine-aspartic acid (RGD) sequence present on several adhesive proteins. As such, it is a receptor for FIBRINOGEN; VON WILLEBRAND FACTOR; FIBRONECTIN; VITRONECTIN; and THROMBOSPONDINS. A deficiency of GPIIb-IIIa results in GLANZMANN THROMBASTHENIA.'),
  'D019040': NtO(MH='E-Selectin', MS='Cell adhesion molecule and CD antigen that mediates neutrophil, monocyte, and memory T-cell adhesion to cytokine-activated endothelial cells. E-selectin recognizes sialylated carbohydrate groups related to the Lewis X or Lewis A family.'),
  'D019041': NtO(MH='L-Selectin', MS='Cell adhesion molecule and CD antigen that serves as a homing receptor for lymphocytes to lymph node high endothelial venules.'),
  'D019042': NtO(MH='Muscle Neoplasms', MS='Tumors or cancer located in muscle tissue or specific muscles. They are differentiated from NEOPLASMS, MUSCLE TISSUE which are neoplasms composed of skeletal, cardiac, or smooth muscle tissue, such as MYOSARCOMA or LEIOMYOMA.'),
  'D019043': NtO(MH='Vascular Neoplasms', MS='Neoplasms located in the vasculature system, such as ARTERIES and VEINS. They are differentiated from neoplasms of vascular tissue (NEOPLASMS, VASCULAR TISSUE), such as ANGIOFIBROMA or HEMANGIOMA.'),
  'D019044': NtO(MH='Image Cytometry', MS='A technique encompassing morphometry, densitometry, neural networks, and expert systems that has numerous clinical and research applications and is particularly useful in anatomic pathology for the study of malignant lesions. The most common current application of image cytometry is for DNA analysis, followed by quantitation of immunohistochemical staining.'),
  'D019046': NtO(MH='Bone Marrow Neoplasms', MS='Neoplasms located in the bone marrow. They are differentiated from neoplasms composed of bone marrow cells, such as MULTIPLE MYELOMA. Most bone marrow neoplasms are metastatic.'),
  'D019047': NtO(MH='Phantoms, Imaging', MS='Devices or objects in various imaging techniques used to visualize or enhance visualization by simulating conditions encountered in the procedure. Phantoms are used very often in procedures employing or measuring x-irradiation or radioactive material to evaluate performance. Phantoms often have properties similar to human tissue. Water demonstrates absorbing properties similar to normal tissue, hence water-filled phantoms are used to map radiation levels. Phantoms are used also as teaching aids to simulate real conditions with x-ray or ultrasonic machines. (From Iturralde, Dictionary and Handbook of Nuclear Medicine and Clinical Imaging, 1990)'),
  'D019048': NtO(MH='Prostatic Intraepithelial Neoplasia', MS='A premalignant change arising in the prostatic epithelium, regarded as the most important and most likely precursor of prostatic adenocarcinoma. The neoplasia takes the form of an intra-acinar or ductal proliferation of secretory cells with unequivocal nuclear anaplasia, which corresponds to nuclear grade 2 and 3 invasive prostate cancer.'),
  'D019049': NtO(MH='Developed Countries', MS='Countries that have reached a level of economic achievement through an increase of production, per capita income and consumption, and utilization of natural and human resources.'),
  'D019050': NtO(MH='Acupressure', MS='A type of massage in which finger pressure on specific body sites is used to promote healing, relieve fatigue, etc. Although the anatomical locations are the same as the ACUPUNCTURE POINTS used in ACUPUNCTURE THERAPY (hence acu-), no needle or other acupuncture technique is employed in acupressure. (From Random House Unabridged Dictionary, 2d ed). Shiatsu is a modern outgrowth that focuses more on prevention than healing.'),
  'D019051': NtO(MH='Rhizotomy', MS='Surgical interruption of a spinal or cranial nerve root. (From Dorland, 28th ed)'),
  'D019052': NtO(MH='Depression, Postpartum', MS='Depression in POSTPARTUM WOMEN, usually within four weeks after giving birth (PARTURITION). The degree of depression ranges from mild transient depression to neurotic or psychotic depressive disorders. (From DSM-IV, p386)'),
  'D019053': NtO(MH='HIV Enteropathy', MS="A syndrome characterized by chronic, well-established DIARRHEA (greater than one month in duration) without an identified infectious cause after thorough evaluation, in an HIV-positive individual. It is thought to be due to direct or indirect effects of HIV on the enteric mucosa. HIV enteropathy is a diagnosis of exclusion and can be made only after other forms of diarrheal illness have been ruled out. (Harrison's Principles of Internal Medicine, 13th ed, pp1607-8; Haubrich et al., Bockus Gastroenterology, 5th ed, p1155)"),
  'D019054': NtO(MH='Evoked Potentials, Motor', MS='The electrical response evoked in a muscle or motor nerve by electrical or magnetic stimulation. Common methods of stimulation are by transcranial electrical and TRANSCRANIAL MAGNETIC STIMULATION. It is often used for monitoring during neurosurgery.'),
  'D019055': NtO(MH='No-Observed-Adverse-Effect Level', MS='The highest dosage administered that does not produce toxic effects.'),
  'D019056': NtO(MH='Receptors, Polymeric Immunoglobulin', MS='Specialized Fc receptors (RECEPTORS, FC) for polymeric immunoglobulins, which mediate transcytosis of polymeric IMMUNOGLOBULIN A and IMMUNOGLOBULIN M into external secretions. They are found on the surfaces of epithelial cells and hepatocytes. After binding to IMMUNOGLOBULIN A, the receptor-ligand complex undergoes endocytosis, transport by vesicle, and secretion into the lumen by exocytosis. Before release, the part of the receptor (SECRETORY COMPONENT) that is bound to IMMUNOGLOBULIN A is proteolytically cleaved from its transmembrane tail. (From Rosen et al., The Dictionary of Immunology, 1989)'),
  'D019057': NtO(MH='Quality-Adjusted Life Years', MS='A measurement index derived from a modification of standard life-table procedures and designed to take account of the quality as well as the duration of survival. This index can be used in assessing the outcome of health care procedures or services. (BIOETHICS Thesaurus, 1994)'),
  'D019058': NtO(MH='Community Networks', MS='Organizations and individuals cooperating together toward a common goal at the local or grassroots level.'),
  'D019060': NtO(MH='Minimally Invasive Surgical Procedures', MS='Procedures that avoid use of open, invasive surgery in favor of closed or local surgery. These generally involve use of laparoscopic devices and remote-controlled manipulation of instruments with indirect observation of the surgical field through an endoscope or similar device.'),
  'D019061': NtO(MH='src-Family Kinases', MS='A PROTEIN-TYROSINE KINASE family that was originally identified by homology to the Rous sarcoma virus ONCOGENE PROTEIN PP60(V-SRC). They interact with a variety of cell-surface receptors and participate in intracellular signal transduction pathways. Oncogenic forms of src-family kinases can occur through altered regulation or expression of the endogenous protein and by virally encoded src (v-src) genes.'),
  'D019062': NtO(MH='Cell Adhesion Molecules, Neuron-Glia', MS='Cell adhesion molecules that mediate neuron-neuron adhesion and neuron-astrocyte adhesion. They are expressed on neurons and Schwann cells, but not astrocytes and are involved in neuronal migration, neurite fasciculation, and outgrowth. Ng-CAM is immunologically and structurally distinct from NCAM.'),
  'D019063': NtO(MH='Tenascin', MS='Hexameric extracellular matrix glycoprotein transiently expressed in many developing organs and often re-expressed in tumors. It is present in the central and peripheral nervous systems as well as in smooth muscle and tendons. (From Kreis & Vale, Guidebook to the Extracellular Matrix and Adhesion Proteins, 1993, p93)'),
  'D019064': NtO(MH='Product Packaging', MS='Form in which product is processed or wrapped and labeled. PRODUCT LABELING is also available.'),
  'D019065': NtO(MH='Virus Assembly', MS='The assembly of VIRAL STRUCTURAL PROTEINS and nucleic acid (VIRAL DNA or VIRAL RNA) to form a VIRUS PARTICLE.'),
  'D019066': NtO(MH='Facies', MS='The appearance of the face that is often characteristic of a disease or pathological condition, as the elfin facies of WILLIAMS SYNDROME or the mongoloid facies of DOWN SYNDROME. (Random House Unabridged Dictionary, 2d ed)'),
  'D019067': NtO(MH='Antibody-Dependent Enhancement', MS='Enhancement of viral infectivity caused by non-neutralizing antibodies. There are at least two mechanisms known to account for this: mediation by Fc receptors (RECEPTORS, FC) or by complement receptors (RECEPTORS, COMPLEMENT). Either the virus is complexed with antiviral IMMUNOGLOBULIN G and binds to Fc receptors, or virus is coated with antiviral IMMUNOGLOBULIN M and binds to complement receptors.'),
  'D019068': NtO(MH='Bryopsida', MS='A class of plants within the Bryophyta comprising the mosses, which are found in both damp (including freshwater) and drier situations. Mosses possess erect or prostrate leafless stems, which give rise to leafless stalks bearing capsules. Spores formed in the capsules are released and grow to produce new plants. (Concise Dictionary of Biology, 1990). Many small plants bearing the name moss are in fact not mosses. The "moss" found on the north side of trees is actually a green alga (CHLOROPHYTA). Irish moss is really a red alga (RHODOPHYTA). Beard lichen (beard moss), Iceland moss, oak moss, and reindeer moss are actually LICHENS. Spanish moss is a common name for both LICHENS and an air plant (TILLANDSIA usneoides) of the pineapple family. Club moss is an evergreen herb of the family LYCOPODIACEAE.'),
  'D019069': NtO(MH='Cell Respiration', MS='The metabolic process of all living cells (animal and plant) in which oxygen is used to provide a source of energy for the cell.'),
  'D019070': NtO(MH='Cell Lineage', MS='The developmental history of specific differentiated cell types as traced back to the original STEM CELLS in the embryo.'),
  'D019071': NtO(MH='Persia', MS="An ancient civilization, known as early as 2000 B.C. The Persian Empire was founded by Cyrus the Great (550-529 B.C.) and for 200 years, from 550 to 331 B.C., the Persians ruled the ancient world from India to Egypt. The territory west of India was called Persis by the Greeks who later called the entire empire Persia. In 331 B.C. the Persian wars against the Greeks ended disastrously under the counterattacks by Alexander the Great. The name Persia in modern times for the modern country was changed to Iran in 1935. (From Webster's New Geographical Dictionary, 1988, p546 & Asimov, Words on the Map, 1962, p176)"),
  'D019072': NtO(MH='Antibiotic Prophylaxis', MS='Use of antibiotics before, during, or after a diagnostic, therapeutic, or surgical procedure to prevent infectious complications.'),
  'D019073': NtO(MH='HT29 Cells', MS='Human colonic ADENOCARCINOMA cells that are able to express differentiation features characteristic of mature intestinal cells such as the GOBLET CELLS.'),
  'D019074': NtO(MH='Epigastric Arteries', MS='Inferior and external epigastric arteries arise from external iliac; superficial from femoral; superior from internal thoracic. They supply the abdominal muscles, diaphragm, iliac region, and groin. The inferior epigastric artery is used in coronary artery bypass grafting and myocardial revascularization.'),
  'D019075': NtO(MH='Electrophoresis, Capillary', MS='A highly-sensitive (in the picomolar range, which is 10,000-fold more sensitive than conventional electrophoresis) and efficient technique that allows separation of PROTEINS; NUCLEIC ACIDS; and CARBOHYDRATES. (Segen, Dictionary of Modern Medicine, 1992)'),
  'D019076': NtO(MH='Transgenes', MS='Genes that are introduced into an organism using GENE TRANSFER TECHNIQUES.'),
  'D019077': NtO(MH='Helix-Turn-Helix Motifs', MS='The first DNA-binding protein motif to be recognized. Helix-turn-helix motifs were originally identified in bacterial proteins but have since been found in hundreds of DNA-BINDING PROTEINS from both eukaryotes and prokaryotes. They are constructed from two alpha helices connected by a short extended chain of amino acids, which constitute the "turn." The two helices are held at a fixed angle, primarily through interactions between the two helices. (From Alberts et al., Molecular Biology of the Cell, 3d ed, p408-9)'),
  'D019080': NtO(MH='Cafe-au-Lait Spots', MS="Light brown pigmented macules associated with NEUROFIBROMATOSIS and Albright's syndrome (see FIBROUS DYSPLASIA, POLYOSTOTIC)."),
  'D019081': NtO(MH='O Antigens', MS='The lipopolysaccharide-protein somatic antigens, usually from gram-negative bacteria, important in the serological classification of enteric bacilli. The O-specific chains determine the specificity of the O antigens of a given serotype. O antigens are the immunodominant part of the lipopolysaccharide molecule in the intact bacterial cell. (From Singleton & Sainsbury, Dictionary of Microbiology and Molecular Biology, 2d ed)'),
  'D019082': NtO(MH='Smith-Lemli-Opitz Syndrome', MS='An autosomal recessive disorder of CHOLESTEROL metabolism. It is caused by a deficiency of 7-dehydrocholesterol reductase, the enzyme that converts 7-dehydrocholesterol to cholesterol, leading to an abnormally low plasma cholesterol. This syndrome is characterized by multiple CONGENITAL ABNORMALITIES, growth deficiency, and INTELLECTUAL DISABILITY.'),
  'D019083': NtO(MH='Mediterranean Region', MS='The Mediterranean Sea, the Mediterranean islands, and the countries bordering on the sea collectively.'),
  'D019084': NtO(MH='Fluorescent Antibody Technique, Indirect', MS='A form of fluorescent antibody technique commonly used to detect serum antibodies and immune complexes in tissues and microorganisms in specimens from patients with infectious diseases. The technique involves formation of an antigen-antibody complex which is labeled with fluorescein-conjugated anti-immunoglobulin antibody. (From Bennington, Saunders Dictionary & Encyclopedia of Laboratory Medicine and Technology, 1984)'),
  'D019085': NtO(MH='Fluorescent Antibody Technique, Direct', MS='A form of fluorescent antibody technique utilizing a fluorochrome conjugated to an antibody, which is added directly to a tissue or cell suspension for the detection of a specific antigen. (Bennington, Saunders Dictionary & Encyclopedia of Laboratory Medicine and Technology, 1984)'),
  'D019086': NtO(MH='Bridged Bicyclo Compounds, Heterocyclic', MS='Heterocyclic compounds that contain two rings that share two non-adjacent atoms in common.'),
  'D019087': NtO(MH='Selectins', MS='Transmembrane proteins consisting of a lectin-like domain, an epidermal growth factor-like domain, and a variable number of domains that are homologous to complement regulatory proteins. They are important cell adhesion molecules which help LEUKOCYTES attach to VASCULAR ENDOTHELIUM.'),
  'D019089': NtO(MH='Stem Cell Factor', MS='A hematopoietic growth factor and the ligand of the cell surface c-kit protein (PROTO-ONCOGENE PROTEINS C-KIT). It is expressed during embryogenesis and is a growth factor for a number of cell types including the MAST CELLS and the MELANOCYTES in addition to the HEMATOPOIETIC STEM CELLS.'),
  'D019090': NtO(MH='Case Management', MS='A traditional term for all the activities which a physician or other health care professional normally performs to insure the coordination of the medical services required by a patient. It also, when used in connection with managed care, covers all the activities of evaluating the patient, planning treatment, referral, and follow-up so that care is continuous and comprehensive and payment for the care is obtained. (From Slee & Slee, Health Care Terms, 2nd ed)'),
  'D019091': NtO(MH='Critical Pathways', MS="Schedules of medical and nursing procedures, including diagnostic tests, medications, and consultations designed to effect an efficient, coordinated program of treatment. (From Mosby's Medical, Nursing & Allied Health Dictionary, 4th ed)"),
  'D019092': NtO(MH='Conization', MS='The excision of a cone of tissue, especially of the CERVIX UTERI.'),
  'D019093': NtO(MH='Circumcision, Female', MS='A general term encompassing three types of excision of the external female genitalia - Sunna, clitoridectomy, and infibulation. It is associated with severe health risks and has been declared illegal in many places, but continues to be widely practiced in a number of countries, particularly in Africa.'),
  'D019094': NtO(MH='Dental Prosthesis, Implant-Supported', MS="A prosthesis that gains its support, stability, and retention from a substructure that is implanted under the soft tissues of the basal seat of the device and is in contact with bone. (From Boucher's Clinical Dental Terminology, 4th ed)"),
  'D019095': NtO(MH='Point-of-Care Systems', MS='Laboratory and other services provided to patients at the bedside. These include diagnostic and laboratory testing using automated information entry.'),
  'D019096': NtO(MH='Vitronectin', MS='A blood plasma glycoprotein that mediates cell adhesion and interacts with proteins of the complement, coagulation, and fibrinolytic cascade. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D019097': NtO(MH='Spin Trapping', MS='A technique for detecting short-lived reactive FREE RADICALS in biological systems by providing a nitrone or nitrose compound for an addition reaction to occur which produces an ELECTRON SPIN RESONANCE SPECTROSCOPY-detectable aminoxyl radical. In spin trapping, the compound trapping the radical is called the spin trap and the addition product of the radical is identified as the spin adduct. (Free Rad Res Comm 1990;9(3-6):163)'),
  'D019098': NtO(MH='Telomerase', MS='An essential ribonucleoprotein reverse transcriptase that adds telomeric DNA to the ends of eukaryotic CHROMOSOMES.'),
  'D019102': NtO(MH='Infant, Very Low Birth Weight', MS='An infant whose weight at birth is less than 1500 grams (3.3 lbs), regardless of gestational age.'),
  'D019105': NtO(MH='Random Amplified Polymorphic DNA Technique', MS='Technique that utilizes low-stringency polymerase chain reaction (PCR) amplification with single primers of arbitrary sequence to generate strain-specific arrays of anonymous DNA fragments. RAPD technique may be used to determine taxonomic identity, assess kinship relationships, analyze mixed genome samples, and create specific probes.'),
  'D019106': NtO(MH='Postoperative Hemorrhage', MS='Hemorrhage following any surgical procedure. It may be immediate or delayed and is not restricted to the surgical wound.'),
  'D019108': NtO(MH='Tight Junctions', MS='Cell-cell junctions that seal adjacent epithelial cells together, preventing the passage of most dissolved molecules from one side of the epithelial sheet to the other. (Alberts et al., Molecular Biology of the Cell, 2nd ed, p22)'),
  'D019110': NtO(MH='Meteoroids', MS='Any solid objects moving in interplanetary space that are smaller than a planet or asteroid but larger than a molecule. Meteorites are any meteoroid that has fallen to a planetary surface. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D019111': NtO(MH='Condoms, Female', MS='A soft, loose-fitting polyurethane sheath, closed at one end, with flexible rings at both ends. The device is inserted into the vagina by compressing the inner ring and pushing it in. Properly positioned, the ring at the closed end covers the cervix, and the sheath lines the walls of the vagina. The outer ring remains outside the vagina, covering the labia. (Med Lett Drugs Ther 1993 Dec 24;35(12):123)'),
  'D019112': NtO(MH='Teleradiology', MS='The electronic transmission of radiological images from one location to another for the purposes of interpretation and/or consultation. Users in different locations may simultaneously view images with greater access to secondary consultations and improved continuing education. (From American College of Radiology, ACR Standard for Teleradiology, 1994, p3)'),
  'D019113': NtO(MH='Telepathology', MS='Transmission and interpretation of tissue specimens via remote telecommunication, generally for the purpose of diagnosis or consultation but may also be used for continuing education.'),
  'D019114': NtO(MH='Remote Consultation', MS='Consultation via remote telecommunications, generally for the purpose of diagnosis or treatment of a patient at a site remote from the patient or primary physician.'),
  'D019115': NtO(MH='Fasciitis, Necrotizing', MS='A fulminating bacterial infection of the deep layers of the skin and FASCIA. It can be caused by many different organisms, with STREPTOCOCCUS PYOGENES being the most common.'),
  'D019117': NtO(MH='Burkholderia', MS='A genus of gram-negative, aerobic, rod-shaped bacteria originally classified as members of the PSEUDOMONADACEAE.'),
  'D019121': NtO(MH='Burkholderia Infections', MS='Infections with bacteria of the genus BURKHOLDERIA.'),
  'D019122': NtO(MH='Meditation', MS="A state of consciousness in which the individual eliminates environmental stimuli from awareness so that the mind can focus on a single thing, producing a state of relaxation and relief from stress. A wide variety of techniques are used to clear the mind of stressful outside interferences. It includes meditation therapy. (Mosby's Medical, Nursing, and Allied Health Dictionary, 4th ed)"),
  'D019124': NtO(MH='Therapeutic Touch', MS='Placing of the hands of the healer upon the person to be cured with the intent of spiritual energetic healing.'),
  'D019131': NtO(MH='United States Social Security Administration', MS='An independent agency within the Executive Branch of the United States Government. It administers a national social insurance program whereby employees, employers, and the self-employed pay contributions into pooled trust funds. Part of the contributions go into a separate hospital insurance trust fund for workers at age 65 to provide help with medical expenses. Other programs include the supplemental social security income program for the aged, blind, and disabled and the Old Age Survivors and Disability Insurance Program. It became an independent agency March 31, 1995. It had previously been part of the Department of Health, Education, and Welfare, later the Department of Health and Human Services. (From United States Government Manual, 1994-95)'),
  'D019138': NtO(MH='Antibodies, Blocking', MS='Antibodies that inhibit the reaction between ANTIGEN and other antibodies or sensitized T-LYMPHOCYTES (e.g., antibodies of the IMMUNOGLOBULIN G class that compete with IGE antibodies for antigen, thereby blocking an allergic response). Blocking antibodies that bind tumors and prevent destruction of tumor cells by CYTOTOXIC T-LYMPHOCYTES have also been called enhancing antibodies. (Rosen et al., Dictionary of Immunology, 1989)'),
  'D019139': NtO(MH='Tumor Escape', MS='The ability of tumors to evade destruction by the IMMUNE SYSTEM. Theories concerning possible mechanisms by which this takes place involve both cellular immunity (IMMUNITY, CELLULAR) and humoral immunity (ANTIBODY FORMATION), and also costimulatory pathways related to CD28 ANTIGENS and B7-1 ANTIGEN.'),
  'D019140': NtO(MH='Dental Care for Children', MS='The giving of attention to the special dental needs of children, including the prevention of tooth diseases and instruction in dental hygiene and dental health. The dental care may include the services provided by dental specialists.'),
  'D019141': NtO(MH='Respiratory System Agents', MS='Drugs used for their effects on the respiratory system.'),
  'D019142': NtO(MH='Hemorrhagic Fever, Ebola', MS='A highly fatal, acute hemorrhagic fever caused by EBOLAVIRUS.'),
  'D019143': NtO(MH='Evolution, Molecular', MS='The process of cumulative change at the level of DNA; RNA; and PROTEINS, over successive generations.'),
  'D019145': NtO(MH='Rehabilitation Nursing', MS='A nursing specialty involved in the diagnosis and treatment of human responses of individuals and groups to actual or potential health problems with the characteristics of altered functional ability and altered life-style.'),
  'D019146': NtO(MH='Hypermedia', MS='Computerized compilations of information units (text, sound, graphics, and/or video) interconnected by logical nonlinear linkages that enable users to follow optimal paths through the material and also the systems used to create and display this information. (From Thesaurus of ERIC Descriptors, 1994)'),
  'D019147': NtO(MH='Vomeronasal Organ', MS='An accessory chemoreceptor organ that is separated from the main OLFACTORY MUCOSA. It is situated at the base of nasal septum close to the VOMER and NASAL BONES. It forwards chemical signals (such as PHEROMONES) to the CENTRAL NERVOUS SYSTEM, thus influencing reproductive and social behavior. In humans, most of its structures except the vomeronasal duct undergo regression after birth.'),
  'D019148': NtO(MH='Neuromuscular Blockade', MS='The intentional interruption of transmission at the NEUROMUSCULAR JUNCTION by external agents, usually neuromuscular blocking agents. It is distinguished from NERVE BLOCK in which nerve conduction (NEURAL CONDUCTION) is interrupted rather than neuromuscular transmission. Neuromuscular blockade is commonly used to produce MUSCLE RELAXATION as an adjunct to anesthesia during surgery and other medical procedures. It is also often used as an experimental manipulation in basic research. It is not strictly speaking anesthesia but is grouped here with anesthetic techniques. The failure of neuromuscular transmission as a result of pathological processes is not included here.'),
  'D019149': NtO(MH='Bioreactors', MS='Tools or devices for generating products using the synthetic or chemical conversion capacity of a biological system. They can be classical fermentors, cell culture perfusion systems, or enzyme bioreactors. For production of proteins or enzymes, recombinant microorganisms such as bacteria, mammalian cells, or insect or plant cells are usually chosen.'),
  'D019150': NtO(MH='Neuroaxonal Dystrophies', MS='A nonspecific term referring both to the pathologic finding of swelling of distal portions of axons in the brain and to disorders which feature this finding. Neuroaxonal dystrophy is seen in various genetic diseases, vitamin deficiencies, and aging. Infantile neuroaxonal dystrophy is an autosomal recessive disease characterized by arrested psychomotor development at 6 months to 2 years of age, ataxia, brain stem dysfunction, and quadriparesis. Juvenile and adult forms also occur. Pathologic findings include brain atrophy and widespread accumulation of axonal spheroids throughout the neuroaxis, peripheral nerves, and dental pulp. (From Davis & Robertson, Textbook of Neuropathology, 2nd ed, p927)'),
  'D019151': NtO(MH='Peptide Library', MS='A collection of cloned peptides, or chemically synthesized peptides, frequently consisting of all possible combinations of amino acids making up an n-amino acid peptide.'),
  'D019152': NtO(MH='Paracentesis', MS='A procedure in which fluid is withdrawn from a body cavity or organ via a trocar and cannula, needle, or other hollow instrument.'),
  'D019154': NtO(MH='Protein Splicing', MS='The excision of in-frame internal protein sequences (INTEINS) of a precursor protein, coupled with ligation of the flanking sequences (EXTEINS). Protein splicing is an autocatalytic reaction and results in the production of two proteins from a single primary translation product: the intein and the mature protein.'),
  'D019155': NtO(MH='Veterinary Drugs', MS="Drugs used by veterinarians in the treatment of animal diseases. The veterinarian's pharmacological armamentarium is the counterpart of drugs treating human diseases, with dosage and administration adjusted to the size, weight, disease, and idiosyncrasies of the species. In the United States most drugs are subject to federal regulations with special reference to the safety of drugs and residues in edible animal products."),
  'D019156': NtO(MH='Indian Ocean', MS='A body of water covering approximately one-fifth of the total ocean area of the earth, extending amidst Africa in the west, Australia in the east, Asia in the north, and Antarctica in the south. Including the Red Sea and the Persian Gulf, it constitutes the third largest ocean after the ATLANTIC OCEAN and the PACIFIC OCEAN. (New Encyclopaedia Britannica Micropaedia, 15th ed, 1990, p289)'),
  'D019157': NtO(MH='Ischemic Preconditioning, Myocardial', MS='Exposure of myocardial tissue to brief, repeated periods of vascular occlusion in order to render the myocardium resistant to the deleterious effects of ISCHEMIA or REPERFUSION. The period of pre-exposure and the number of times the tissue is exposed to ischemia and reperfusion vary, the average being 3 to 5 minutes.'),
  'D019158': NtO(MH='N-Acetylneuraminic Acid', MS='An N-acyl derivative of neuraminic acid. N-acetylneuraminic acid occurs in many polysaccharides, glycoproteins, and glycolipids in animals and bacteria. (From Dorland, 28th ed, p1518)'),
  'D019159': NtO(MH='Nonheme Iron Proteins', MS='Proteins, usually acting in oxidation-reduction reactions, containing iron but no porphyrin groups. (Lehninger, Principles of Biochemistry, 1993, pG-10)'),
  'D019160': NtO(MH='Endosonography', MS='Ultrasonography of internal organs using an ultrasound transducer sometimes mounted on a fiberoptic endoscope. In endosonography the transducer converts electronic signals into acoustic pulses or continuous waves and acts also as a receiver to detect reflected pulses from within the organ. An audiovisual-electronic interface converts the detected or processed echo signals, which pass through the electronics of the instrument, into a form that the technologist can evaluate. The procedure should not be confused with ENDOSCOPY which employs a special instrument called an endoscope. The "endo-" of endosonography refers to the examination of tissue within hollow organs, with reference to the usual ultrasonography procedure which is performed externally or transcutaneously.'),
  'D019161': NtO(MH='Hydroxymethylglutaryl-CoA Reductase Inhibitors', MS='Compounds that inhibit HMG-CoA reductases. They have been shown to directly lower cholesterol synthesis.'),
  'D019162': NtO(MH='Anesthetics, Combined', MS='The use of two or more chemicals simultaneously or sequentially to induce anesthesia. The drugs need not be in the same dosage form.'),
  'D019163': NtO(MH='Reducing Agents', MS='Materials that add an electron to an element or compound, that is, decrease the positiveness of its valence. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D019164': NtO(MH='Liver, Artificial', MS='Devices for simulating the activities of the liver. They often consist of a hybrid between both biological and artificial materials.'),
  'D019167': NtO(MH='Appetite Stimulants', MS='Agents that are used to stimulate appetite. These drugs are frequently used to treat anorexia associated with cancer and AIDS.'),
  'D019168': NtO(MH='Portasystemic Shunt, Transjugular Intrahepatic', MS='A type of surgical portasystemic shunt to reduce portal hypertension with associated complications of esophageal varices and ascites. It is performed percutaneously through the jugular vein and involves the creation of an intrahepatic shunt between the hepatic vein and portal vein. The channel is maintained by a metallic stent. The procedure can be performed in patients who have failed sclerotherapy and is an additional option to the surgical techniques of portocaval, mesocaval, and splenorenal shunts. It takes one to three hours to perform. (JAMA 1995;273(23):1824-30)'),
  'D019169': NtO(MH='Jurkat Cells', MS='A CELL LINE derived from human T-CELL LEUKEMIA and used to determine the mechanism of differential susceptibility to anti-cancer drugs and radiation.'),
  'D019170': NtO(MH='Somites', MS='Paired, segmented masses of MESENCHYME located on either side of the developing spinal cord (neural tube). Somites derive from PARAXIAL MESODERM and continue to increase in number during ORGANOGENESIS. Somites give rise to SKELETON (sclerotome); MUSCLES (myotome); and DERMIS (dermatome).'),
  'D019172': NtO(MH='Transplantation Conditioning', MS='Preparative treatment of transplant recipient with various conditioning regimens including radiation, immune sera, chemotherapy, and/or immunosuppressive agents, prior to transplantation. Transplantation conditioning is very common before bone marrow transplantation.'),
  'D019175': NtO(MH='DNA Methylation', MS='Addition of methyl groups to DNA. DNA methyltransferases (DNA methylases) perform this reaction using S-ADENOSYLMETHIONINE as the methyl group donor.'),
  'D019176': NtO(MH='Potyviridae', MS='A family of RNA plant viruses with flexuous, filamentous particles and consisting of six genera: POTYVIRUS; Ipomovirus; Macluravirus; Rymovirus; Tritimovirus; and Bymovirus. All members of the family form cytoplasmic cylindrical inclusion bodies during infection.'),
  'D019177': NtO(MH='Bromoviridae', MS='A family of RNA plant viruses with a wide host range in crops and horticultural species. All viruses are readily transmitted by mechanical means and some by insects and pollen. The genera include: ALFAMOVIRUS; BROMOVIRUS; CUCUMOVIRUS; ILARVIRUS; and OLEAVIRUS.'),
  'D019178': NtO(MH='Alfamovirus', MS='A genus of the family BROMOVIRIDAE with a wide host range. Transmission is by aphids and the type species is ALFALFA MOSAIC VIRUS.'),
  'D019179': NtO(MH='Ilarvirus', MS='A genus of the family BROMOVIRIDAE which infects mainly woody plants. Species are divided into ten subgroups. Tobacco streak virus is the type species.'),
  'D019180': NtO(MH='Secoviridae', MS='A family of non-enveloped RNA plant viruses, transmitted by biological vectors and experimentally by mechanical inoculation. Subfamily Comovirinae includes all members of old taxonomic family Comoviridae: COMOVIRUS; FABAVIRUS; and NEPOVIRUS.'),
  'D019181': NtO(MH='Fabavirus', MS='A genus of the family COMOVIRIDAE with a wide host range among dicotyledons and some monocotyledons. They are transmitted nonpersistently by aphids. The type species is broad bean wilt virus 1.'),
  'D019183': NtO(MH='Tombusviridae', MS='A family of RNA plant viruses infecting dicotyledons. Transmission is mainly by mechanical inoculation and through propagative plant material. All species elicit formation of multivesicular inclusion bodies. There are at least eight genera: Aureusvirus, Avenavirus, CARMOVIRUS, Dianthovirus, Machlomovirus, Necrovirus, Panicovirus, and TOMBUSVIRUS.'),
  'D019184': NtO(MH='Carmovirus', MS='A genus in the family TOMBUSVIRIDAE mostly found in temperate regions. Some species infecting legumes (FABACEAE) are reported from tropical areas. Most viruses are soil-borne, but some are transmitted by the fungus Olpidium radicale and others by beetles. Carnation mottle virus is the type species.'),
  'D019185': NtO(MH='Notophthalmus', MS='A genus of newts of the Salamandridae family found in North America in areas east of the 100th meridian. A common species is NOTOPHTHALMUS VIRIDESCENS.'),
  'D019186': NtO(MH='Mandibular Advancement', MS="Moving a retruded mandible forward to a normal position. It is commonly performed for malocclusion and retrognathia. (From Jablonski's Dictionary of Dentistry, 1992)"),
  'D019187': NtO(MH='Cadmium Compounds', MS='Inorganic compounds that contain cadmium as an integral part of the molecule.'),
  'D019188': NtO(MH='Organ Preservation Solutions', MS='Solutions used to store organs and minimize tissue damage, particularly while awaiting implantation.'),
  'D019189': NtO(MH='Iron Metabolism Disorders', MS="Disorders in the processing of iron in the body: its absorption, transport, storage, and utilization. (From Mosby's Medical, Nursing, & Allied Health Dictionary, 4th ed)"),
  'D019190': NtO(MH='Iron Overload', MS="An excessive accumulation of iron in the body due to a greater than normal absorption of iron from the gastrointestinal tract or from parenteral injection. This may arise from idiopathic hemochromatosis, excessive iron intake, chronic alcoholism, certain types of refractory anemia, or transfusional hemosiderosis. (From Churchill's Illustrated Medical Dictionary, 1989)"),
  'D019191': NtO(MH='Sequiviridae', MS='A family of monopartite RNA plant viruses resembling picornaviruses and containing two genera: SEQUIVIRUS and WAIKAVIRUS. Transmission is by aphids or leafhoppers and requires a helper protein.'),
  'D019192': NtO(MH='Sequivirus', MS='A genus of plant viruses in the family SEQUIVIRIDAE. Transmission is by APHIDS but depends on the presence of a helper protein encoded by the Anthriscus yellow virus, a WAIKAVIRUS. The type species is Parsnip yellow fleck virus (parsnip serotype).'),
  'D019193': NtO(MH='Waikavirus', MS='A genus of plant viruses in the family SEQUIVIRIDAE. Transmission is by leafhoppers or APHIDS and depends on a self-encoded helper protein. The type species is Rice tungro spherical virus.'),
  'D019194': NtO(MH='Ischemic Preconditioning', MS='A technique in which tissue is rendered resistant to the deleterious effects of prolonged ISCHEMIA and REPERFUSION by prior exposure to brief, repeated periods of vascular occlusion. (Am J Physiol 1995 May;268(5 Pt 2):H2063-7, Abstract)'),
  'D019195': NtO(MH='Badnavirus', MS='A genus of DNA plant viruses with bacilliform morphology. Transmission in clonally-propagated plants is by vegetative propagation of infected plant materials. Transmission in nature is by mealybugs, seeds, and pollen. The type species is Commelina yellow mottle virus.'),
  'D019196': NtO(MH='Totiviridae', MS='A family of RNA viruses that infect fungi and protozoa. There are three genera: TOTIVIRUS; GIARDIAVIRUS; and LEISHMANIAVIRUS.'),
  'D019197': NtO(MH='Totivirus', MS='A genus of RNA fungi viruses in the family TOTIVIRIDAE. Some of the viruses contain additional satellite RNA or defective RNA. Transmission occurs during cell division, sporogenesis and cell fusion. The type species is Saccharomyces cerevisiae virus L-A.'),
  'D019198': NtO(MH='Giardiavirus', MS='A genus of RNA protozoan viruses of the family TOTIVIRIDAE. It infects many isolates of the flagellated protozoan human parasite G. lamblia, but does not seem to be associated with the virulence of the parasite. The type species is Giardia lamblia virus.'),
  'D019199': NtO(MH='Leishmaniavirus', MS='A genus of RNA protozoan viruses of the family TOTIVIRIDAE. Several different strains of Leishmania are infected by a variety of viral species. The type species is Leishmania RNA virus 1-1.'),
  'D019200': NtO(MH='Astroviridae', MS='A family of RNA viruses with two genera: MAMASTROVIRUS and AVASTROVIRUS. They cause GASTROENTERITIS in humans and also infect other vertebrates.'),
  'D019201': NtO(MH='Aquabirnavirus', MS='A genus of RNA viruses in the family BIRNAVIRIDAE infecting fish, mollusks, and crustaceans. It is transmitted both vertically and horizontally with no known vectors. The natural hosts are salmonids and the type species is INFECTIOUS PANCREATIC NECROSIS VIRUS.'),
  'D019202': NtO(MH='Avibirnavirus', MS='A genus of RNA viruses in the family BIRNAVIRIDAE infecting birds. It is transmitted horizontally with no known vectors. The type species is INFECTIOUS BURSAL DISEASE VIRUS.'),
  'D019203': NtO(MH='Entomobirnavirus', MS='A genus of RNA viruses of the family BIRNAVIRIDAE infecting fruitflies. Transmission is horizontal and there are no known vectors. Drosophila melanogaster is the natural host and the type species is Drosophila X virus.'),
  'D019204': NtO(MH='GTP-Binding Proteins', MS='Regulatory proteins that act as molecular switches. They control a wide range of biological processes including: receptor signaling, intracellular signal transduction pathways, and protein synthesis. Their activity is regulated by factors that control their ability to bind to and hydrolyze GTP to GDP. EC 3.6.1.-.'),
  'D019205': NtO(MH='GTP-Binding Protein alpha Subunits, Gs', MS='A family of heterotrimeric GTP-binding protein alpha subunits that activate ADENYLYL CYCLASES.'),
  'D019206': NtO(MH='GTP-Binding Protein alpha Subunits, Gi-Go', MS='A family of heterotrimeric GTP-binding protein alpha subunits that were originally identified by their ability to inhibit ADENYLYL CYCLASES. Members of this family can couple to beta and gamma G-protein subunits that activate POTASSIUM CHANNELS. The Gi-Go part of the name is also spelled Gi/Go.'),
  'D019207': NtO(MH='beta Carotene', MS='A carotenoid that is a precursor of VITAMIN A. Beta carotene is administered to reduce the severity of photosensitivity reactions in patients with erythropoietic protoporphyria (PORPHYRIA, ERYTHROPOIETIC).'),
  'D019208': NtO(MH='Brain-Derived Neurotrophic Factor', MS='A member of the nerve growth factor family of trophic factors. In the brain BDNF has a trophic action on retinal, cholinergic, and dopaminergic neurons, and in the peripheral nervous system it acts on both motor and sensory neurons. (From Kendrew, The Encyclopedia of Molecular Biology, 1994)'),
  'D019209': NtO(MH='Troponin C', MS='A TROPONIN complex subunit that binds calcium and help regulate calcium-dependent muscle contraction. There are two troponin C subtypes: troponin C1 and C2. Troponin C1 is skeletal and cardiac type whereas troponin C2 is skeletal type. Troponin C1 is a BIOMARKER for damaged or injured CARDIAC MYOCYTES and mutations in troponin C1 gene are associated with FAMILIAL HYPERTROPHIC CARDIOMYOPATHY.'),
  'D019210': NtO(MH='Troponin I', MS='A TROPONIN complex subunit that inhibits ACTOMYOSIN ATPASE activity thereby disrupting ACTIN and MYOSIN interaction. There are three troponin I subtypes: troponin I1, I2 and I3. Troponin I3 is cardiac-specific whereas troponin I1 and I2 are skeletal subtypes. Troponin I3 is a BIOMARKER for damaged or injured CARDIAC MYOCYTES and mutations in troponin I3 gene are associated with FAMILIAL HYPERTROPHIC CARDIOMYOPATHY.'),
  'D019211': NtO(MH='Roseolovirus', MS='A genus of the family HERPESVIRIDAE, subfamily BETAHERPESVIRINAE, whose viruses have been isolated from lymphocytes. HERPESVIRUS 6, HUMAN is the type species.'),
  'D019212': NtO(MH='Multimedia', MS='Materials, frequently computer applications, that combine some or all of text, sound, graphics, animation, and video into integrated packages. (Thesaurus of ERIC Descriptors, 1994)'),
  'D019213': NtO(MH='Rubulavirus', MS='A genus of the family PARAMYXOVIRIDAE (subfamily PARAMYXOVIRINAE) where all the species have hemagglutinin and neuraminidase activities but lack a C protein. MUMPS VIRUS is the type species.'),
  'D019214': NtO(MH='Gingival Overgrowth', MS="Excessive growth of the gingiva either by an increase in the size of the constituent cells (GINGIVAL HYPERTROPHY) or by an increase in their number (GINGIVAL HYPERPLASIA). (From Jablonski's Dictionary of Dentistry, 1992, p574)"),
  'D019215': NtO(MH='Biography', MS="Works consisting of an account of the events, works, and achievements, personal and professional, during a person's life."),
  'D019216': NtO(MH='Metals, Heavy', MS="Metals with high specific gravity, typically larger than 5. They have complex spectra, form colored salts and double salts, have a low electrode potential, are mainly amphoteric, yield weak bases and weak acids, and are oxidizing or reducing agents (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D019217': NtO(MH='Tooth Attrition', MS='The wearing away of a tooth as a result of tooth-to-tooth contact, as in mastication, occurring only on the occlusal, incisal, and proximal surfaces. It is chiefly associated with aging. It is differentiated from TOOTH ABRASION (the pathologic wearing away of the tooth substance by friction, as brushing, bruxism, clenching, and other mechanical causes) and from TOOTH EROSION (the loss of substance caused by chemical action without bacterial action). (Jablonski, Dictionary of Dentistry, 1992, p86)'),
  'D019218': NtO(MH='Erythrovirus', MS='A genus of the family PARVOVIRIDAE, subfamily PARVOVIRINAE, and containing the type species PARVOVIRUS B19, HUMAN.'),
  'D019219': NtO(MH='Parvovirinae', MS='A subfamily of DNA vertebrate viruses, in the family PARVOVIRIDAE. There are three genera: PARVOVIRUS; ERYTHROVIRUS; and DEPENDOVIRUS.'),
  'D019220': NtO(MH='High-Energy Shock Waves', MS='High-amplitude compression waves, across which density, pressure, and particle velocity change drastically. The mechanical force from these shock waves can be used for mechanically disrupting tissues and deposits.'),
  'D019221': NtO(MH='Mandatory Reporting', MS='A legal requirement that designated types of information acquired by professionals or institutions in the course of their work be reported to appropriate authorities.'),
  'D019222': NtO(MH='Mind-Body Relations, Metaphysical', MS="The relation between the mind and the body in a religious, social, spiritual, behavioral, and metaphysical context. This concept is significant in the field of alternative medicine. It differs from the relationship between physiologic processes and behavior where the emphasis is on the body's physiology ( = PSYCHOPHYSIOLOGY)."),
  'D019223': NtO(MH='Densovirinae', MS='A subfamily of DNA arthropod viruses, in the family PARVOVIRIDAE. The host range includes members of Dictyoptera; DIPTERA; LEPIDOPTERA; Odonata; and ORTHOPTERA. There are three genera: DENSOVIRUS, Iteravirus, and Brevidensovirus. Densovirus-like viruses also infect and multiply in crabs and shrimp.'),
  'D019224': NtO(MH='Temporomandibular Joint Disc', MS="A plate of fibrous tissue that divides the temporomandibular joint into an upper and lower cavity. The disc is attached to the articular capsule and moves forward with the condyle in free opening and protrusion. (Boucher's Clinical Dental Terminology, 4th ed, p92)"),
  'D019226': NtO(MH='Oral Ulcer', MS='A loss of mucous substance of the mouth showing local excavation of the surface, resulting from the sloughing of inflammatory necrotic tissue. It is the result of a variety of causes, e.g., denture irritation, aphthous stomatitis (STOMATITIS, APHTHOUS); NOMA; necrotizing gingivitis (GINGIVITIS, NECROTIZING ULCERATIVE); TOOTHBRUSHING; and various irritants. (From Jablonski, Dictionary of Dentistry, 1992, p842)'),
  'D019227': NtO(MH='Tooth Apex', MS='The tip or terminal end of the root of a tooth. (Jablonski, Dictionary of Dentistry, 1992, p62)'),
  'D019228': NtO(MH='Tooth Crown', MS='The upper part of the tooth, which joins the lower part of the tooth (TOOTH ROOT) at the cervix (TOOTH CERVIX) at a line called the cementoenamel junction. The entire surface of the crown is covered with enamel which is thicker at the extremity and becomes progressively thinner toward the cervix. (From Jablonski, Dictionary of Dentistry, 1992, p216)'),
  'D019229': NtO(MH='Dentition, Permanent', MS="The 32 teeth of adulthood that either replace or are added to the complement of deciduous teeth. (Boucher's Clinical Dental Terminology, 4th ed)"),
  'D019230': NtO(MH='Dental Audit', MS='A detailed review and evaluation of selected clinical records by qualified professional personnel for evaluating quality of dental care.'),
  'D019231': NtO(MH='Tooth Preparation, Prosthodontic', MS="The selected form given to a natural tooth when it is reduced by instrumentation to receive a prosthesis (e.g., artificial crown or a retainer for a fixed or removable prosthesis). The selection of the form is guided by clinical circumstances and physical properties of the materials that make up the prosthesis. (Boucher's Clinical Dental Terminology, 4th ed, p239)"),
  'D019232': NtO(MH='Dental Restoration Failure', MS='Inability or inadequacy of a dental restoration or prosthesis to perform as expected.'),
  'D019233': NtO(MH='Retreatment', MS='The therapy of the same disease in a patient, with the same agent or procedure repeated after initial treatment, or with an additional or alternate measure or follow-up. It does not include therapy which requires more than one administration of a therapeutic agent or regimen. Retreatment is often used with reference to a different modality when the original one was inadequate, harmful, or unsuccessful.'),
  'D019234': NtO(MH='Dentinal Fluid', MS='The lymph or fluid of dentin. It is a transudate of extracellular fluid, mainly cytoplasm of odontoblastic processes, from the dental pulp via the dentinal tubules. It is also called dental lymph. (From Stedman, 26th ed, p665)'),
  'D019235': NtO(MH='Ephemerovirus', MS='A genus of the family RHABDOVIRIDAE which primarily infect cattle. EPHEMERAL FEVER VIRUS, BOVINE is the type species.'),
  'D019237': NtO(MH='Tooth Cervix', MS='The constricted part of the tooth at the junction of the crown and root or roots. It is often referred to as the cementoenamel junction (CEJ), the line at which the cementum covering the root of a tooth and the enamel of the tooth meet. (Jablonski, Dictionary of Dentistry, 1992, p530, p433)'),
  'D019239': NtO(MH="Practice Patterns, Dentists'", MS='Patterns of practice in dentistry related to diagnosis and treatment.'),
  'D019240': NtO(MH='Corticoviridae', MS='A family of icosahedral, lipid-containing, non-enveloped bacteriophages containing one genus (Corticovirus).'),
  'D019241': NtO(MH='Fuselloviridae', MS='A family of lemon-shaped DNA viruses infecting ARCHAEA.'),
  'D019242': NtO(MH='Oral Medicine', MS='A branch of dentistry dealing with diseases of the oral and paraoral structures and the oral management of systemic diseases. (Hall, What is Oral Medicine, Anyway? Clinical Update: National Naval Dental Center, March 1991, p7-8)'),
  'D019243': NtO(MH='Enamel Microabrasion', MS='Mechanical removal of a small amount of tooth structure (not more than a few tenths of a millimeter in depth) to eliminate superficial enamel discoloration defects not successfully removed by bleaching techniques. A common abrasive is a mixture of pumice and hydrochloric acid.'),
  'D019244': NtO(MH='RNA, Satellite', MS='Small, linear single-stranded RNA molecules functionally acting as molecular parasites of certain RNA plant viruses. Satellite RNAs exhibit four characteristic traits: (1) they require helper viruses to replicate; (2) they are unnecessary for the replication of helper viruses; (3) they are encapsidated in the coat protein of the helper virus; (4) they have no extensive sequence homology to the helper virus. Thus they differ from SATELLITE VIRUSES which encode their own coat protein, and from the genomic RNA; (=RNA, VIRAL); of satellite viruses. (From Maramorosch, Viroids and Satellites, 1991, p143)'),
  'D019245': NtO(MH='Compressive Strength', MS='The maximum compression a material can withstand without failure. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed, p427)'),
  'D019246': NtO(MH='Tooth Preparation', MS='Procedures carried out with regard to the teeth or tooth structures preparatory to specified dental therapeutic and surgical measures.'),
  'D019247': NtO(MH='HIV Wasting Syndrome', MS="Involuntary weight loss of greater than 10 percent associated with intermittent or constant fever and chronic diarrhea or fatigue for more than 30 days in the absence of a defined cause other than HIV infection. A constant feature is major muscle wasting with scattered myofiber degeneration. A variety of etiologies, which vary among patients, contributes to this syndrome. (From Harrison's Principles of Internal Medicine, 13th ed, p1611)."),
  'D019248': NtO(MH='Jaw Fixation Techniques', MS='The stable placement of surgically induced fractures of the mandible or maxilla through the use of elastics, wire ligatures, arch bars, or other splints. It is used often in the cosmetic surgery of retrognathism and prognathism. (From Dorland, 28th ed, p636)'),
  'D019250': NtO(MH='Dental Implants, Single-Tooth', MS='Devices, usually alloplastic, surgically inserted into or onto the jawbone, which support a single prosthetic tooth and serve either as abutments or as cosmetic replacements for missing teeth.'),
  'D019251': NtO(MH='Nucleocapsid', MS='A protein-nucleic acid complex which forms part or all of a virion. It consists of a CAPSID plus enclosed nucleic acid. Depending on the virus, the nucleocapsid may correspond to a naked core or be surrounded by a membranous envelope.'),
  'D019252': NtO(MH='Radiography, Dental, Digital', MS='A rapid, low-dose, digital imaging system using a small intraoral sensor instead of radiographic film, an intensifying screen, and a charge-coupled device. It presents the possibility of reduced patient exposure and minimal distortion, although resolution and latitude are inferior to standard dental radiography. A receiver is placed in the mouth, routing signals to a computer which images the signals on a screen or in print. It includes digitizing from x-ray film or any other detector. (From MEDLINE abstracts; personal communication from Dr. Charles Berthold, NIDR)'),
  'D019253': NtO(MH='Proto-Oncogene Proteins c-bcl-2', MS='Membrane proteins encoded by the BCL-2 GENES and serving as potent inhibitors of cell death by APOPTOSIS. The proteins are found on mitochondrial, microsomal, and NUCLEAR MEMBRANE sites within many cell types. Overexpression of bcl-2 proteins, due to a translocation of the gene, is associated with follicular lymphoma.'),
  'D019254': NtO(MH='Genes, bcl-2', MS='The B-cell leukemia/lymphoma-2 genes, responsible for blocking apoptosis in normal cells, and associated with follicular lymphoma when overexpressed. Overexpression results from the t(14;18) translocation. The human c-bcl-2 gene is located at 18q24 on the long arm of chromosome 18.'),
  'D019255': NtO(MH='NADPH Oxidases', MS='A family of membrane-associated flavoprotein NADPH-dependent oxidoreductases that catalyze the univalent reduction of OXYGEN to create SUPEROXIDES. Structurally, they are characterized by six N-terminal transmembrane ALPHA-HELICES, a FLAVIN-ADENINE DINUCLEOTIDE (FAD)-binding region, and a C-terminal NADPH-binding region. They are expressed primarily by EPITHELIAL CELLS in gut, kidney, colon, and smooth muscle tissues, as well as GRANULOCYTES and function to transfer electrons across membranes to molecular oxygen. Defects in the production of superoxide ions by some NADPH oxidases result in GRANULOMATOUS DISEASE, CHRONIC.'),
  'D019256': NtO(MH='Cadmium Chloride', MS='A cadmium halide in the form of colorless crystals, soluble in water, methanol, and ethanol. It is used in photography, in dyeing, and calico printing, and as a solution to precipitate sulfides. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D019257': NtO(MH='Quinpirole', MS='A dopamine D2/D3 receptor agonist.'),
  'D019258': NtO(MH='Saquinavir', MS='An HIV protease inhibitor which acts as an analog of an HIV protease cleavage site. It is a highly specific inhibitor of HIV-1 and HIV-2 proteases, and also inhibits CYTOCHROME P-450 CYP3A.'),
  'D019259': NtO(MH='Lamivudine', MS="A reverse transcriptase inhibitor and ZALCITABINE analog in which a sulfur atom replaces the 3' carbon of the pentose ring. It is used to treat HIV disease."),
  'D019260': NtO(MH='Expressed Emotion', MS='A measurement of collective behavior and attitudes that exist towards a specific person\xe2\x80\x94a measurement of the social environment or social relationships of an individual.'),
  'D019261': NtO(MH='Epithalamus', MS='The dorsal posterior subdivision of the diencephalon. The epithalamus is generally considered to include the habenular nuclei (HABENULA) and associated fiber bundles, the PINEAL BODY, and the epithelial roof of the third ventricle. The anterior and posterior paraventricular nuclei of the thalamus are included with the THALAMIC NUCLEI although they develop from the same pronuclear mass as the epithalamic nuclei and are sometimes considered part of the epithalamus.'),
  'D019262': NtO(MH='Habenula', MS='A small protuberance at the dorsal, posterior corner of the wall of the THIRD VENTRICLE, adjacent to the dorsal THALAMUS and PINEAL BODY. It contains the habenular nuclei and is a major part of the epithalamus.'),
  'D019263': NtO(MH='Dysthymic Disorder', MS='Chronically depressed mood that occurs for most of the day more days than not for at least 2 years. The required minimum duration in children to make this diagnosis is 1 year. During periods of depressed mood, at least 2 of the following additional symptoms are present: poor appetite or overeating, insomnia or hypersomnia, low energy or fatigue, low self esteem, poor concentration or difficulty making decisions, and feelings of hopelessness. (DSM-IV)'),
  'D019264': NtO(MH='Adoptive Transfer', MS='Form of passive immunization where previously sensitized immunologic agents (cells or serum) are transferred to non-immune recipients. When transfer of cells is used as a therapy for the treatment of neoplasms, it is called adoptive immunotherapy (IMMUNOTHERAPY, ADOPTIVE).'),
  'D019265': NtO(MH='Spectroscopy, Near-Infrared', MS='A noninvasive technique that uses the differential absorption properties of hemoglobin and myoglobin to evaluate tissue oxygenation and indirectly can measure regional hemodynamics and blood flow. Near-infrared light (NIR) can propagate through tissues and at particular wavelengths is differentially absorbed by oxygenated vs. deoxygenated forms of hemoglobin and myoglobin. Illumination of intact tissue with NIR allows qualitative assessment of changes in the tissue concentration of these molecules. The analysis is also used to determine body composition.'),
  'D019266': NtO(MH='Iron, Dietary', MS='Iron or iron compounds used in foods or as food. Dietary iron is important in oxygen transport and the synthesis of the iron-porphyrin proteins hemoglobin, myoglobin, cytochromes, and cytochrome oxidase. Insufficient amounts of dietary iron can lead to iron-deficiency anemia.'),
  'D019267': NtO(MH='Hemagglutinin Glycoproteins, Influenza Virus', MS='Membrane glycoproteins from influenza viruses which are involved in hemagglutination, virus attachment, and envelope fusion. Fourteen distinct subtypes of HA glycoproteins and nine of NA glycoproteins have been identified from INFLUENZA A VIRUS; no subtypes have been identified for Influenza B or Influenza C viruses.'),
  'D019268': NtO(MH='Antibodies, Antineutrophil Cytoplasmic', MS='Autoantibodies directed against cytoplasmic constituents of POLYMORPHONUCLEAR LEUKOCYTES and/or MONOCYTES. They are used as specific markers for GRANULOMATOSIS WITH POLYANGIITIS and other diseases, though their pathophysiological role is not clear. ANCA are routinely detected by indirect immunofluorescence with three different patterns: c-ANCA (cytoplasmic), p-ANCA (perinuclear), and atypical ANCA.'),
  'D019269': NtO(MH='Phosphatidylinositol 4,5-Diphosphate', MS='A phosphoinositide present in all eukaryotic cells, particularly in the plasma membrane. It is the major substrate for receptor-stimulated phosphoinositidase C, with the consequent formation of inositol 1,4,5-triphosphate and diacylglycerol, and probably also for receptor-stimulated inositol phospholipid 3-kinase. (Kendrew, The Encyclopedia of Molecular Biology, 1994)'),
  'D019270': NtO(MH='Fructosamine', MS='An amino sugar formed when glucose non-enzymatically reacts with the N-terminal amino group of proteins. The fructose moiety is derived from glucose by the "classical" Amadori rearrangement.'),
  'D019271': NtO(MH='Hypoxanthine', MS='A purine and a reaction intermediate in the metabolism of adenosine and in the formation of nucleic acids by the salvage pathway.'),
  'D019272': NtO(MH='Leukocyte Elastase', MS='An enzyme that catalyzes the hydrolysis of proteins, including elastin. It cleaves preferentially bonds at the carboxyl side of Ala and Val, with greater specificity for Ala. EC 3.4.21.37.'),
  'D019274': NtO(MH='Botulinum Toxins, Type A', MS='A serotype of botulinum toxins that has specificity for cleavage of SYNAPTOSOMAL-ASSOCIATED PROTEIN 25.'),
  'D019275': NtO(MH='Radiopharmaceuticals', MS='Compounds that are used in medicine as sources of radiation for radiotherapy and for diagnostic purposes. They have numerous uses in research and industry. (Martindale, The Extra Pharmacopoeia, 30th ed, p1161)'),
  'D019276': NtO(MH='Glycocalyx', MS='The carbohydrate-rich zone on the cell surface. This zone can be visualized by a variety of stains as well as by its affinity for lectins. Although most of the carbohydrate is attached to intrinsic plasma membrane molecules, the glycocalyx usually also contains both glycoproteins and proteoglycans that have been secreted into the extracellular space and then adsorbed onto the cell surface. (Alberts et al., Molecular Biology of the Cell, 3d ed, p502)'),
  'D019277': NtO(MH='Entropy', MS='The measure of that part of the heat or energy of a system which is not available to perform work. Entropy increases in all natural (spontaneous and irreversible) processes. (From Dorland, 28th ed)'),
  'D019278': NtO(MH='Matrix Metalloproteinase 3', MS='An extracellular endopeptidase of vertebrate tissues similar to MATRIX METALLOPROTEINASE 1. It digests PROTEOGLYCAN; FIBRONECTIN; COLLAGEN types III, IV, V, and IX, and activates procollagenase. (Enzyme Nomenclature, 1992)'),
  'D019279': NtO(MH='Resin Cements', MS="Dental cements composed either of polymethyl methacrylate or dimethacrylate, produced by mixing an acrylic monomer liquid with acrylic polymers and mineral fillers. The cement is insoluble in water and is thus resistant to fluids in the mouth, but is also irritating to the dental pulp. It is used chiefly as a luting agent for fabricated and temporary restorations. (Jablonski's Dictionary of Dentistry, 1992, p159)"),
  'D019280': NtO(MH='Branchio-Oto-Renal Syndrome', MS="An autosomal dominant disorder manifested by various combinations of preauricular pits, branchial fistulae or cysts, lacrimal duct stenosis, hearing loss, structural defects of the outer, middle, or inner ear, and renal dysplasia. Associated defects include asthenic habitus, long narrow facies, constricted palate, deep overbite, and myopia. Hearing loss may be due to Mondini type cochlear defect and stapes fixation. (Jablonski's Dictionary of Syndromes & Eponymic Diseases, 2d ed)"),
  'D019281': NtO(MH='Dimerization', MS='The process by which two molecules of the same chemical composition form a condensation product or polymer.'),
  'D019282': NtO(MH='Wasting Syndrome', MS='A condition of involuntary weight loss of greater then 10% of baseline body weight. It is characterized by atrophy of muscles and depletion of lean body mass. Wasting is a sign of MALNUTRITION as a result of inadequate dietary intake, malabsorption, or hypermetabolism.'),
  'D019283': NtO(MH='Pancreatitis, Acute Necrotizing', MS='A severe form of acute INFLAMMATION of the PANCREAS characterized by one or more areas of NECROSIS in the pancreas with varying degree of involvement of the surrounding tissues or organ systems. Massive pancreatic necrosis may lead to DIABETES MELLITUS, and malabsorption.'),
  'D019284': NtO(MH='Thapsigargin', MS='A sesquiterpene lactone found in roots of THAPSIA. It inhibits SARCOPLASMIC RETICULUM CALCIUM-TRANSPORTING ATPASES.'),
  'D019286': NtO(MH='Endopeptidase K', MS='An enzyme that catalyzes the hydrolysis of keratin, and of other proteins with subtilisin-like specificity. It hydrolyses peptide amides. Endopeptidase K is from the mold Tritirachium album Limber. (Enzyme Nomenclature, 1992) EC 3.4.21.64.'),
  'D019287': NtO(MH='Zinc Sulfate', MS='A compound given in the treatment of conditions associated with zinc deficiency such as acrodermatitis enteropathica. Externally, zinc sulfate is used as an astringent in lotions and eye drops. (Reynolds JEF(Ed): Martindale: The Extra Pharmacopoeia (electronic version). Micromedex, Inc, Englewood, CO, 1995)'),
  'D019288': NtO(MH='Herpesvirus 8, Human', MS='A species in the genus RHADINOVIRUS, subfamily GAMMAHERPESVIRINAE, isolated from patients with AIDS-related and "classical" Kaposi sarcoma.'),
  'D019289': NtO(MH='Pyruvic Acid', MS='An intermediate compound in the metabolism of carbohydrates, proteins, and fats. In thiamine deficiency, its oxidation is retarded and it accumulates in the tissues, especially in nervous structures. (From Stedman, 26th ed)'),
  'D019290': NtO(MH='Megestrol Acetate', MS='Megestrol acetate is a progestogen with actions and uses similar to those of the progestogens in general. It also has anti-androgenic properties. It is given by mouth in the palliative treatment or as an adjunct to other therapy in endometrial carcinoma and in breast cancer. Megestrol acetate has been approved to treat anorexia and cachexia. (From Reynolds JEF(Ed): Martindale: The Extra Pharmacopoeia (electronic version). Micromedex, Inc, Englewood, CO, 1995)'),
  'D019291': NtO(MH='Skull Base', MS='The inferior region of the skull consisting of an internal (cerebral), and an external (basilar) surface.'),
  'D019292': NtO(MH='Skull Base Neoplasms', MS='Neoplasms of the base of the skull specifically, differentiated from neoplasms of unspecified sites or bones of the skull (SKULL NEOPLASMS).'),
  'D019294': NtO(MH='Xanthomatosis, Cerebrotendinous', MS='An autosomal recessive lipid storage disorder due to mutation of the gene CYP27A1 encoding a CHOLESTANETRIOL 26-MONOOXYGENASE. It is characterized by large deposits of CHOLESTEROL and CHOLESTANOL in various tissues resulting in xanthomatous swelling of tendons, early CATARACT, and progressive neurological symptoms.'),
  'D019295': NtO(MH='Computational Biology', MS='A field of biology concerned with the development of techniques for the collection and manipulation of biological data, and the use of such data to make biological discoveries or predictions. This field encompasses all computational methods and theories for solving biological problems including manipulation of models and datasets.'),
  'D019296': NtO(MH='Capnography', MS='Continuous recording of the carbon dioxide content of expired air.'),
  'D019297': NtO(MH='2,4-Dinitrophenol', MS='A toxic dye, chemically related to trinitrophenol (picric acid), used in biochemical studies of oxidative processes where it uncouples oxidative phosphorylation. It is also used as a metabolic stimulant. (Stedman, 26th ed)'),
  'D019298': NtO(MH='Glucose-6-Phosphate', MS='An ester of glucose with phosphoric acid, made in the course of glucose metabolism by mammalian and other cells. It is a normal constituent of resting muscle and probably is in constant equilibrium with fructose-6-phosphate. (Stedman, 26th ed)'),
  'D019299': NtO(MH='Decompression, Surgical', MS='A surgical operation for the relief of pressure in a body compartment or on a body part. (From Dorland, 28th ed)'),
  'D019300': NtO(MH='Medical Errors', MS='Errors or mistakes committed by health professionals which result in harm to the patient. They include errors in diagnosis (DIAGNOSTIC ERRORS), errors in the administration of drugs and other medications (MEDICATION ERRORS), errors in the performance of surgical procedures, in the use of other types of therapy, in the use of equipment, and in the interpretation of laboratory findings. Medical errors are differentiated from MALPRACTICE in that the former are regarded as honest mistakes or accidents while the latter is the result of negligence, reprehensible ignorance, or criminal intent.'),
  'D019301': NtO(MH='Oleic Acid', MS='An unsaturated fatty acid that is the most widely distributed and abundant fatty acid in nature. It is used commercially in the preparation of oleates and lotions, and as a pharmaceutical solvent. (Stedman, 26th ed)'),
  'D019302': NtO(MH='HIV Envelope Protein gp160', MS='An envelope protein of the human immunodeficiency virus that is encoded by the HIV env gene. It has a molecular weight of 160,000 kDa and contains numerous glycosylation sites. It serves as a precursor for both the HIV ENVELOPE PROTEIN GP120 and the HIV ENVELOPE PROTEIN GP41.'),
  'D019303': NtO(MH='Multilingualism', MS='The ability to speak, read, or write several languages or many languages with some facility. Bilingualism is the most common form. (From Random House Unabridged Dictionary, 2d ed)'),
  'D019304': NtO(MH='Ethinyl Estradiol-Norgestrel Combination', MS='ETHINYL ESTRADIOL and NORGESTREL given in fixed proportions.'),
  'D019305': NtO(MH='Epilepsy, Rolandic', MS='An autosomal dominant inherited partial epilepsy syndrome with onset between age 3 and 13 years. Seizures are characterized by PARESTHESIA and tonic or clonic activity of the lower face associated with drooling and DYSARTHRIA. In most cases, affected children are neurologically and developmentally normal. (From Epilepsia 1998 39;Suppl 4:S32-S41)'),
  'D019306': NtO(MH='Artemisia', MS='A plant genus of the family ASTERACEAE with strong-smelling foliage. It is a source of SANTONIN and other cytotoxic TERPENES.'),
  'D019307': NtO(MH='1-(5-Isoquinolinesulfonyl)-2-Methylpiperazine', MS='A specific protein kinase C inhibitor, which inhibits superoxide release from human neutrophils (PMN) stimulated with phorbol myristate acetate or synthetic diacylglycerol.'),
  'D019308': NtO(MH='Palmitic Acid', MS='A common saturated fatty acid found in fats and waxes including olive oil, palm oil, and body lipids.'),
  'D019309': NtO(MH='Epstein-Barr Virus Nuclear Antigens', MS='Nuclear antigens encoded by VIRAL GENES found in HUMAN HERPESVIRUS 4. At least six nuclear antigens have been identified.'),
  'D019310': NtO(MH='Pseudolymphoma', MS='A group of disorders having a benign course but exhibiting clinical and histological features suggestive of malignant lymphoma. Pseudolymphoma is characterized by a benign infiltration of lymphoid cells or histiocytes which microscopically resembles a malignant lymphoma. (From Dorland, 28th ed & Stedman, 26th ed)'),
  'D019311': NtO(MH='Staurosporine', MS='An indolocarbazole that is a potent PROTEIN KINASE C inhibitor which enhances cAMP-mediated responses in human neuroblastoma cells. (Biochem Biophys Res Commun 1995;214(3):1114-20)'),
  'D019313': NtO(MH='BRCA1 Protein', MS='The phosphoprotein encoded by the BRCA1 gene (GENE, BRCA1). It contains an N-terminal RING FINGER DOMAIN and is a PROTEIN PHOSPHATASE 1 regulatory subunit. In normal cells the BRCA1 protein is localized in the nucleus, whereas in the majority of breast cancer cell lines and in malignant pleural effusions from breast cancer patients, it is localized mainly in the cytoplasm. (Science 1995;270(5237):713,789-91)'),
  'D019314': NtO(MH='Dehydroepiandrosterone Sulfate', MS='The circulating form of a major C19 steroid produced primarily by the ADRENAL CORTEX. DHEA sulfate serves as a precursor for TESTOSTERONE; ANDROSTENEDIONE; ESTRADIOL; and ESTRONE.'),
  'D019315': NtO(MH='Retrobulbar Hemorrhage', MS='Hemorrhage within the orbital cavity, posterior to the eyeball.'),
  'D019316': NtO(MH='Porcine respiratory and reproductive syndrome virus', MS='A species of ARTERIVIRUS causing reproductive and respiratory disease in pigs. The European strain is called Lelystad virus. Airborne transmission is common.'),
  'D019317': NtO(MH='Evidence-Based Medicine', MS="An approach of practicing medicine with the goal to improve and evaluate patient care. It requires the judicious integration of best research evidence with the patient's values to make decisions about medical care. This method is to help physicians make proper diagnosis, devise best testing plan, choose best treatment and methods of disease prevention, as well as develop guidelines for large groups of patients with the same disease. (from JAMA 296 (9), 2006)"),
  'D019318': NtO(MH='Porcine Reproductive and Respiratory Syndrome', MS='A syndrome characterized by outbreaks of late term abortions, high numbers of stillbirths and mummified or weak newborn piglets, and respiratory disease in young unweaned and weaned pigs. It is caused by PORCINE RESPIRATORY AND REPRODUCTIVE SYNDROME VIRUS. (Radostits et al., Veterinary Medicine, 8th ed, p1048)'),
  'D019319': NtO(MH='Okadaic Acid', MS='A specific inhibitor of phosphoserine/threonine protein phosphatase 1 and 2a. It is also a potent tumor promoter. (Thromb Res 1992;67(4):345-54 & Cancer Res 1993;53(2):239-41)'),
  'D019320': NtO(MH='Embolism, Paradoxical', MS='Blockage of an artery due to passage of a clot (THROMBUS) from a systemic vein to a systemic artery without its passing through the lung which acts as a filter to remove blood clots from entering the arterial circulation. Paradoxical embolism occurs when there is a defect that allows a clot to cross directly from the right to the left side of the heart as in the cases of ATRIAL SEPTAL DEFECTS or open FORAMEN OVALE. Once in the arterial circulation, a clot can travel to the brain, block an artery, and cause a STROKE.'),
  'D019323': NtO(MH='omega-N-Methylarginine', MS='A competitive inhibitor of nitric oxide synthetase.'),
  'D019324': NtO(MH='beta-Naphthoflavone', MS='A polyaromatic hydrocarbon inducer of P4501A1 and P4501A2 cytochromes. (Proc Soc Exp Biol Med 1994 Dec:207(3):302-308)'),
  'D019325': NtO(MH='3-O-Methylglucose', MS='A non-metabolizable glucose analogue that is not phosphorylated by hexokinase. 3-O-Methylglucose is used as a marker to assess glucose transport by evaluating its uptake within various cells and organ systems. (J Neurochem 1993;60(4):1498-504)'),
  'D019326': NtO(MH='17-alpha-Hydroxyprogesterone', MS='A metabolite of PROGESTERONE with a hydroxyl group at the 17-alpha position. It serves as an intermediate in the biosynthesis of HYDROCORTISONE and GONADAL STEROID HORMONES.'),
  'D019327': NtO(MH='Copper Sulfate', MS='A sulfate salt of copper. It is a potent emetic and is used as an antidote for poisoning by phosphorus. It also can be used to prevent the growth of algae.'),
  'D019328': NtO(MH='Buthionine Sulfoximine', MS='A synthetic amino acid that depletes glutathione by irreversibly inhibiting gamma-glutamylcysteine synthetase. Inhibition of this enzyme is a critical step in glutathione biosynthesis. It has been shown to inhibit the proliferative response in human T-lymphocytes and inhibit macrophage activation. (J Biol Chem 1995;270(33):1945-7)'),
  'D019329': NtO(MH='Idazoxan', MS='A benzodioxane-linked imidazole that has alpha-2 adrenoceptor antagonist activity.'),
  'D019331': NtO(MH='NG-Nitroarginine Methyl Ester', MS='A non-selective inhibitor of nitric oxide synthase. It has been used experimentally to induce hypertension.'),
  'D019332': NtO(MH='Endothelin-1', MS='A 21-amino acid peptide produced in a variety of tissues including endothelial and vascular smooth-muscle cells, neurons and astrocytes in the central nervous system, and endometrial cells. It acts as a modulator of vasomotor tone, cell proliferation, and hormone production. (N Eng J Med 1995;333(6):356-63)'),
  'D019333': NtO(MH='Endothelin-2', MS='A 21-amino acid peptide produced predominantly within the kidney and intestine, with smaller amounts produced in the myocardium, placenta, and uterus, but the cells of origin are not clear. Endothelin-2 has no unique physiologic functions, as compared with endothelin-1. (N Eng J Med 1995;333(6):356-63)'),
  'D019334': NtO(MH='Endothelin-3', MS='A 21-amino acid peptide that circulates in the plasma, but its source is not known. Endothelin-3 has been found in high concentrations in the brain and may regulate important functions in neurons and astrocytes, such as proliferation and development. It also is found throughout the gastrointestinal tract and in the lung and kidney. (N Eng J Med 1995;333(6):356-63)'),
  'D019335': NtO(MH='Nitroarginine', MS='An inhibitor of nitric oxide synthetase which has been shown to prevent glutamate toxicity. Nitroarginine has been experimentally tested for its ability to prevent ammonia toxicity and ammonia-induced alterations in brain energy and ammonia metabolites. (Neurochem Res 1995:200(4):451-6)'),
  'D019336': NtO(MH='Cognitive Science', MS='The study of the precise nature of different mental tasks and the operations of the brain that enable them to be performed, engaging branches of psychology, computer science, philosophy, and linguistics. (Random House Unabridged Dictionary, 2d ed)'),
  'D019337': NtO(MH='Hematologic Neoplasms', MS='Neoplasms located in the blood and blood-forming tissue (the bone marrow and lymphatic tissue). The commonest forms are the various types of LEUKEMIA, of LYMPHOMA, and of the progressive, life-threatening forms of the MYELODYSPLASTIC SYNDROMES.'),
  'D019338': NtO(MH='Polypharmacy', MS='The use of multiple drugs administered to the same patient, most commonly seen in elderly patients. It includes also the administration of excessive medication. Since in the United States most drugs are dispensed as single-agent formulations, polypharmacy, though using many drugs administered to the same patient, must be differentiated from DRUG COMBINATIONS, single preparations containing two or more drugs as a fixed dose, and from DRUG THERAPY, COMBINATION, two or more drugs administered separately for a combined effect. (From Segen, Dictionary of Modern Medicine, 1992)'),
  'D019339': NtO(MH='Port-Wine Stain', MS='A vascular malformation of developmental origin characterized pathologically by ectasia of superficial dermal capillaries, and clinically by persistent macular erythema. In the past, port wine stains have frequently been termed capillary hemangiomas, which they are not; unfortunately this confusing practice persists: HEMANGIOMA, CAPILLARY is neoplastic, a port-wine stain is non-neoplastic. Port-wine stains vary in color from fairly pale pink to deep red or purple and in size from a few millimeters to many centimeters in diameter. The face is the most frequently affected site and they are most often unilateral. (From Rook et al., Textbook of Dermatology, 5th ed, p483)'),
  'D019340': NtO(MH='Osteotomy, Le Fort', MS='Transverse sectioning and repositioning of the maxilla. There are three types: Le Fort I osteotomy for maxillary advancement or the treatment of maxillary fractures; Le Fort II osteotomy for the treatment of maxillary fractures; Le Fort III osteotomy for the treatment of maxillary fractures with fracture of one or more facial bones. Le Fort III is often used also to correct craniofacial dysostosis and related facial abnormalities. (From Dorland, 28th ed, p1203 & p662)'),
  'D019341': NtO(MH='Aromatherapy', MS="The use of fragrances and essences from plants to affect or alter a person's mood or behavior and to facilitate physical, mental, and emotional well-being. The chemicals comprising essential oils in plants has a host of therapeutic properties and has been used historically in Africa, Asia, and India. Its greatest application is in the field of alternative medicine. (From Random House Unabridged Dictionary, 2d ed; from Dr. Atiba Vheir, Dove Center, Washington, D.C.)"),
  'D019342': NtO(MH='Acetic Acid', MS='Product of the oxidation of ethanol and of the destructive distillation of wood. It is used locally, occasionally internally, as a counterirritant and also as a reagent. (Stedman, 26th ed)'),
  'D019343': NtO(MH='Citric Acid', MS='A key intermediate in metabolism. It is an acid compound found in citrus fruits. The salts of citric acid (citrates) can be used as anticoagulants due to their calcium chelating ability.'),
  'D019344': NtO(MH='Lactic Acid', MS='A normal intermediate in the fermentation (oxidation, metabolism) of sugar. The concentrated form is used internally to prevent gastrointestinal fermentation. (From Stedman, 26th ed)'),
  'D019345': NtO(MH='Zinc Acetate', MS='A salt produced by the reaction of zinc oxide with acetic acid and used as an astringent, styptic, and emetic.'),
  'D019346': NtO(MH='Sodium Acetate', MS='The trihydrate sodium salt of acetic acid, which is used as a source of sodium ions in solutions for dialysis and as a systemic and urinary alkalizer, diuretic, and expectorant.'),
  'D019347': NtO(MH='Potassium Acetate', MS='A potassium salt used to replenish ELECTROLYTES, for restoration of WATER-ELECTROLYTE BALANCE, as well as a urinary and systemic alkalizer, which can be administered orally or by intravenous infusion. Formerly, it was used in DIURETICS and EXPECTORANTS.'),
  'D019348': NtO(MH='Empiricism', MS='One of the principal schools of medical philosophy in ancient Greece and Rome. It developed in Alexandria between 270 and 220 B.C., the only one to have any success in reviving the essentials of the Hippocratic concept. The Empiricists declared that the search for ultimate causes of phenomena was vain, but they were active in endeavoring to discover immediate causes. The "tripod of the Empirics" was their own chance observations (experience), learning obtained from contemporaries and predecessors (experience of others), and, in the case of new diseases, the formation of conclusions from other diseases which they resembled (analogy). Empiricism enjoyed sporadic continuing popularity in later centuries up to the nineteenth. (From Castiglioni, A History of Medicine, 2d ed, p186; Dr. James H. Cassedy, NLM History of Medicine Division)'),
  'D019349': NtO(MH='Roseolovirus Infections', MS='Infection with ROSEOLOVIRUS, the most common in humans being EXANTHEMA SUBITUM, a benign disease of infants and young children.'),
  'D019350': NtO(MH='Astroviridae Infections', MS='Infections with ASTROVIRIDAE, causing gastroenteritis in human infants, calves, lambs, and piglets.'),
  'D019351': NtO(MH='Rubulavirus Infections', MS='Infections with viruses of the genus RUBULAVIRUS, family PARAMYXOVIRIDAE.'),
  'D019352': NtO(MH='Censuses', MS='Enumerations of populations usually recording identities of all persons in every place of residence with age or date of birth, sex, occupation, national origin, language, marital status, income, relation to head of household, information on the dwelling place, education, literacy, health-related data (e.g., permanent disability), etc. The census or "numbering of the people" is mentioned several times in the Old Testament. Among the Romans, censuses were intimately connected with the enumeration of troops before and after battle and probably a military necessity. (From Last, A Dictionary of Epidemiology, 3d ed; Garrison, An Introduction to the History of Medicine, 4th ed, p66, p119)'),
  'D019353': NtO(MH='Endemic Diseases', MS='The constant presence of diseases or infectious agents within a given geographic area or population group. It may also refer to the usual prevalence of a given disease with such area or group. It includes holoendemic and hyperendemic diseases. A holoendemic disease is one for which a high prevalent level of infection begins early in life and affects most of the child population, leading to a state of equilibrium such that the adult population shows evidence of the disease much less commonly than do children (malaria in many communities is a holoendemic disease). A hyperendemic disease is one that is constantly present at a high incidence and/or prevalence rate and affects all groups equally. (Last, A Dictionary of Epidemiology, 3d ed, p53, 78, 80)'),
  'D019354': NtO(MH='Sodium Lactate', MS='The sodium salt of racemic or inactive lactic acid. It is a hygroscopic agent used intravenously as a systemic and urinary alkalizer.'),
  'D019355': NtO(MH='Calcium Citrate', MS='A colorless crystalline or white powdery organic, tricarboxylic acid occurring in plants, especially citrus fruits, and used as a flavoring agent, as an antioxidant in foods, and as a sequestrating agent. (McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed)'),
  'D019356': NtO(MH='Grave Robbing', MS='Stealing of corpses after burial, especially for medical dissection. In the late 18th and early 19th centuries, in the absence of laws governing the acquisition of dissecting material for the study of anatomy, the needs of anatomy classes were met by surreptitious methods: body-snatching and grave robbing. (Random House Unabridged Dictionary, 2d ed; from Garrison, An Introduction to the History of Medicine, 4th ed, p447; from Castiglioni, A History of Medicine, 2d ed, p676)'),
  'D019357': NtO(MH='Potassium Citrate', MS='A powder that dissolves in water, which is administered orally, and is used as a diuretic, expectorant, systemic alkalizer, and electrolyte replenisher.'),
  'D019358': NtO(MH='Fat Substitutes', MS='Compounds used in food or in food preparation to replace dietary fats. They may be carbohydrate-, protein-, or fat-based. Fat substitutes are usually lower in calories but provide the same texture as fats.'),
  'D019359': NtO(MH='Knowledge', MS='The body of truths or facts accumulated in the course of time, the cumulated sum of information, its volume and nature, in any civilization, period, or country.'),
  'D019362': NtO(MH='Cytochrome P-450 CYP2B1', MS='A major cytochrome P-450 enzyme which is inducible by PHENOBARBITAL in both the LIVER and SMALL INTESTINE. It is active in the metabolism of compounds like pentoxyresorufin, TESTOSTERONE, and ANDROSTENEDIONE. This enzyme, encoded by CYP2B1 gene, also mediates the activation of CYCLOPHOSPHAMIDE and IFOSFAMIDE to MUTAGENS.'),
  'D019363': NtO(MH='Cytochrome P-450 CYP1A1', MS='A liver microsomal cytochrome P-450 monooxygenase capable of biotransforming xenobiotics such as polycyclic hydrocarbons and halogenated aromatic hydrocarbons into carcinogenic or mutagenic compounds. They have been found in mammals and fish. This enzyme, encoded by CYP1A1 gene, can be measured by using ethoxyresorufin as a substrate for the ethoxyresorufin O-deethylase activity.'),
  'D019364': NtO(MH='Humoralism', MS='An ancient Greek medical theory that health and illness result from a balance or imbalance of body fluids or "humors". The humors are blood, phlegm, yellow bile, and black bile.'),
  'D019365': NtO(MH='Metaphor', MS='The application of a concept to that which it is not literally the same but which suggests a resemblance and comparison. Medical metaphors were widespread in ancient literature; the description of a sick body was often used by ancient writers to define a critical condition of the State, in which one corrupt part can ruin the entire system. (From Med Secoli Arte Sci, 1990;2(3):abstract 331)'),
  'D019366': NtO(MH='Western World', MS='A historical and cultural entity dispersed across the wide geographical area of Europe, as opposed to the East, Asia, and Africa. The term was used by scholars through the late medieval period. Thereafter, with the impact of colonialism and the transmission of cultures, Western World was sometimes expanded to include the Americas. (Dr. James H. Cassedy, NLM History of Medicine Division)'),
  'D019367': NtO(MH='Medicine, Unani', MS='A modified Greco-Arabic medical system. It reflects Arabic medicine blending with Greek philosophy, science, and medicine.'),
  'D019368': NtO(MH='Nature', MS='The system of all phenomena in space and time; the totality of physical reality. It is both a scientific and philosophic concept appearing in all historic eras. (Webster 2d; Dr. James H. Cassedy, NLM History of Medicine Division)'),
  'D019369': NtO(MH='Life', MS='The state that distinguishes organisms from inorganic matter, manifested by growth, metabolism, reproduction, and adaptation. It includes the course of existence, the sum of experiences, the mode of existing, or the fact of being. Over the centuries inquiries into the nature of life have crossed the boundaries from philosophy to biology, forensic medicine, anthropology, etc., in creative as well as scientific literature. (Random House Unabridged Dictionary, 2d ed; Dr. James H. Cassedy, NLM History of Medicine Division)'),
  'D019370': NtO(MH='Observation', MS='The act of regarding attentively and studying facts and occurrences, gathering data through analyzing, measuring, and drawing conclusions, with the purpose of applying the observed information to theoretical assumptions. Observation as a scientific method in the acquisition of knowledge began in classical antiquity; in modern science and medicine its greatest application is facilitated by modern technology. Observation is one of the components of the research process.'),
  'D019371': NtO(MH='Topography, Medical', MS='The systematic surveying, mapping, charting, and description of specific geographical sites, with reference to the physical features that were presumed to influence health and disease. Medical topography should be differentiated from EPIDEMIOLOGY in that the former emphasizes geography whereas the latter emphasizes disease outbreaks.'),
  'D019372': NtO(MH='Orphanages', MS='Institutions for the housing and care of orphans, foundlings, and abandoned children. They have existed as such since the medieval period but the heading is applicable to such usage also in modern parlance.'),
  'D019377': NtO(MH='12-Hydroxy-5,8,10,14-eicosatetraenoic Acid', MS='A lipoxygenase metabolite of ARACHIDONIC ACID. It is a highly selective ligand used to label mu-opioid receptors in both membranes and tissue sections. The 12-S-HETE analog has been reported to augment tumor cell metastatic potential through activation of protein kinase C. (J Pharmacol Exp Ther 1995; 274(3):1545-51; J Natl Cancer Inst 1994; 86(15):1145-51)'),
  'D019379': NtO(MH='Teriparatide', MS='A polypeptide that consists of the 1-34 amino-acid fragment of human PARATHYROID HORMONE, the biologically active N-terminal region. The acetate form is given by intravenous infusion in the differential diagnosis of HYPOPARATHYROIDISM and PSEUDOHYPOPARATHYROIDISM. (Reynolds JEF(Ed): Martindale: The Extra Pharmacopoeia (electronic version). Micromedex, Inc, Englewood, CO, 1995)'),
  'D019380': NtO(MH='Anti-HIV Agents', MS='Agents used to treat AIDS and/or stop the spread of the HIV infection. These do not include drugs used to treat symptoms or opportunistic infections associated with AIDS.'),
  'D019382': NtO(MH='Human Growth Hormone', MS='A 191-amino acid polypeptide hormone secreted by the human adenohypophysis (PITUITARY GLAND, ANTERIOR), also known as GH or somatotropin. Synthetic growth hormone, termed somatropin, has replaced the natural form in therapeutic usage such as treatment of dwarfism in children with growth hormone deficiency.'),
  'D019384': NtO(MH='Nucleic Acid Synthesis Inhibitors', MS='Compounds that inhibit cell production of DNA or RNA.'),
  'D019386': NtO(MH='Alendronate', MS='A nonhormonal medication for the treatment of postmenopausal osteoporosis in women. This drug builds healthy bone, restoring some of the bone loss as a result of osteoporosis.'),
  'D019388': NtO(MH='Cytochrome P-450 CYP1A2', MS='A cytochrome P450 enzyme subtype that has specificity for relatively planar heteroaromatic small molecules, such as CAFFEINE and ACETAMINOPHEN.'),
  'D019389': NtO(MH='Cytochrome P-450 CYP2D6', MS='A cytochrome P450 enzyme that catalyzes the hydroxylation of many drugs and environmental chemicals, such as DEBRISOQUINE; ADRENERGIC RECEPTOR ANTAGONISTS; and TRICYCLIC ANTIDEPRESSANTS. This enzyme is deficient in up to 10 percent of the Caucasian population.'),
  'D019390': NtO(MH='Arrestins', MS='Regulatory proteins that down-regulate phosphorylated G-protein membrane receptors, including rod and cone photoreceptors and adrenergic receptors.'),
  'D019392': NtO(MH='Cytochrome P-450 CYP2E1', MS='An ethanol-inducible cytochrome P450 enzyme that metabolizes several precarcinogens, drugs, and solvents to reactive metabolites. Substrates include ETHANOL; INHALATION ANESTHETICS; BENZENE; ACETAMINOPHEN and other low molecular weight compounds. CYP2E1 has been used as an enzyme marker in the study of alcohol abuse.'),
  'D019393': NtO(MH='Arrestin', MS='A 48-Kd protein of the outer segment of the retinal rods and a component of the phototransduction cascade. Arrestin quenches G-protein activation by binding to phosphorylated photolyzed rhodopsin. Arrestin causes experimental autoimmune uveitis when injected into laboratory animals.'),
  'D019394': NtO(MH='Ki-67 Antigen', MS='A CELL CYCLE and tumor growth marker which can be readily detected using IMMUNOCYTOCHEMISTRY methods. Ki-67 is a nuclear antigen present only in the nuclei of cycling cells.'),
  'D019396': NtO(MH='Tissue Polypeptide Antigen', MS='Serological tumor marker composed of a molecular complex of cytokeratins 8, 18, and 19. It is used in the diagnosis and staging of bronchogenic carcinoma.'),
  'D019397': NtO(MH='Pancreas, Artificial', MS='Devices for simulating the activity of the pancreas. They can be either electromechanical, consisting of a glucose sensor, computer, and insulin pump or bioartificial, consisting of isolated islets of Langerhans in an artificial membrane.'),
  'D019398': NtO(MH='Genes, BRCA1', MS='A tumor suppressor gene (GENES, TUMOR SUPPRESSOR) located on human CHROMOSOME 17 at locus 17q21. Mutations of this gene are associated with the formation of HEREDITARY BREAST AND OVARIAN CANCER SYNDROME. It encodes a large nuclear protein that is a component of DNA repair pathways.'),
  'D019400': NtO(MH='Tobacco mosaic satellite virus', MS='A spherical RNA satellite virus which requires an obligatory helper TOBACCO MOSAIC VIRUS for replication.'),
  'D019401': NtO(MH='Cucumber Mosaic Virus Satellite', MS='A satellite RNA (not a satellite virus) which has several types. Different cucumoviruses can act as helper viruses for different types.'),
  'D019402': NtO(MH='Macrophage Inflammatory Proteins', MS='Heparin-binding proteins that exhibit a number of inflammatory and immunoregulatory activities. Originally identified as secretory products of MACROPHAGES, these chemokines are produced by a variety of cell types including NEUTROPHILS; FIBROBLASTS; and EPITHELIAL CELLS. They likely play a significant role in respiratory tract defenses.'),
  'D019405': NtO(MH='Cytochrome P-450 CYP11B2', MS='A mitochondrial cytochrome P450 enzyme that catalyzes the 18-hydroxylation of steroids in the presence of molecular oxygen and NADPH-specific flavoprotein. This enzyme, encoded by CYP11B2 gene, is important in the conversion of CORTICOSTERONE to 18-hydroxycorticosterone and the subsequent conversion to ALDOSTERONE.'),
  'D019408': NtO(MH='Platelet Endothelial Cell Adhesion Molecule-1', MS='A cell adhesion molecule, composed of a series of Ig-like domains, and expressed on virtually all MONOCYTES; PLATELETS; and GRANULOCYTES. PECAM-1 is highly expressed on endothelial cells and concentrated at the junctions between them. It is essential for TRANSENDOTHELIAL MIGRATION of leukocytes and removal of apoptotic cells by PHAGOCYTES.'),
  'D019409': NtO(MH='Interleukin-15', MS='Cytokine that stimulates the proliferation of T-LYMPHOCYTES and shares biological activities with IL-2. IL-15 also can induce proliferation and differentiation of B-LYMPHOCYTES.'),
  'D019410': NtO(MH='Interleukin-16', MS='A cytokine produced by activated T-LYMPHOCYTES that stimulates the migration of CD4-POSITIVE LYMPHOCYTES and monocytes. It has been reported to suppress HIV replication.'),
  'D019411': NtO(MH='Clinical Laboratory Techniques', MS='Techniques used to carry out clinical investigative procedures in the diagnosis and therapy of disease.'),
  'D019412': NtO(MH='Anatomy, Cross-Sectional', MS='Descriptive anatomy based on three-dimensional imaging (IMAGING, THREE-DIMENSIONAL) of the body, organs, and structures using a series of computer multiplane sections, displayed by transverse, coronal, and sagittal analyses. It is essential to accurate interpretation by the radiologist of such techniques as ultrasonic diagnosis, MAGNETIC RESONANCE IMAGING, and computed tomography (TOMOGRAPHY, X-RAY COMPUTED). (From Lane & Sharfaei, Modern Sectional Anatomy, 1992, Preface)'),
  'D019413': NtO(MH='Qi', MS='The vital life force in the body, supposedly able to be regulated by acupuncture. It corresponds roughly to the Greek pneuma, the Latin spiritus, and the ancient Indian prana. The concept of life-breath or vital energy was formulated as an indication of the awareness of man, originally directed externally toward nature or society but later turned inward to the self or life within. (From Comparison between Concepts of Life-Breath in East and West, 15th International Symposium on the Comparative History of Medicine - East and West, August 26-September 3, 1990, Shizuoka, Japan, pp. ix-x)'),
  'D019414': NtO(MH='Chlorobi', MS='A phylum of anoxygenic, phototrophic bacteria including the family Chlorobiaceae. They occur in aquatic sediments, sulfur springs, and hot springs and utilize reduced sulfur compounds instead of oxygen.'),
  'D019415': NtO(MH='Torque', MS='The rotational force about an axis that is equal to the product of a force times the distance from the axis where the force is applied.'),
  'D019416': NtO(MH='Head Movements', MS='Voluntary or involuntary motion of head that may be relative to or independent of body; includes animals and humans.'),
  'D019417': NtO(MH='Hindlimb Suspension', MS='Technique for limiting use, activity, or movement by immobilizing or restraining animal by suspending from hindlimbs or tails. This immobilization is used to simulate some effects of reduced gravity and study weightlessness physiology.'),
  'D019418': NtO(MH='Evolution, Chemical', MS='Chemical and physical transformation of the biogenic elements from their nucleosynthesis in stars to their incorporation and subsequent modification in planetary bodies and terrestrial biochemistry. It includes the mechanism of incorporation of biogenic elements into complex molecules and molecular systems, leading up to the origin of life.'),
  'D019419': NtO(MH='Evolution, Planetary', MS='Creation and development of bodies within solar systems, includes study of early planetary geology.'),
  'D019422': NtO(MH='Dietary Sucrose', MS='Sucrose present in the diet. It is added to food and drinks as a sweetener.'),
  'D019423': NtO(MH='Shamanism', MS='An intermediate stage between polytheism and monotheism, which assumes a "Great Spirit", with lesser deities subordinated. With the beginnings of shamanism there was the advent of the medicine man or witch doctor, who assumed a supervisory relation to disease and its cure. Formally, shamanism is a religion of Ural-Altaic peoples of Northern Asia and Europe, characterized by the belief that the unseen world of gods, demons, ancestral spirits is responsive only to shamans. The Indians of North and South America entertain religious practices similar to the Ural-Altaic shamanism. The word shaman comes from the Tungusic (Manchuria and Siberia) saman, meaning Buddhist monk. The shaman handles disease almost entirely by psychotherapeutic means; he frightens away the demons of disease by assuming a terrifying mien. (From Garrison, An Introduction to the History of Medicine, 4th ed, p22; from Webster, 3d ed)'),
  'D019424': NtO(MH='Dominica', MS="An island republic of the West Indies. Its capital is Roseau. It was discovered in 1493 by Columbus and held at different times by the French and the British in the 18th century. A member of the West Indies Federation, it achieved internal self-government in 1967 but became independent in 1978. It was named by Columbus who discovered it on Sunday, Domingo in Spanish, from the Latin Dominica dies, the Lord's Day. (From Webster's New Geographical Dictionary, 1988, p338 & Room, Brewer's Dictionary of Names, 1992, p151)"),
  'D019425': NtO(MH='Grenada', MS="An island of the West Indies. Its capital is St. George's. It was discovered in 1498 by Columbus who called it Concepcion. It was held at different times by the French and the British during the 18th century. The British suppressed a native uprising in 1795. It was an associate state of Great Britain 1967-74 but became an independent nation within the British Commonwealth in 1974. The original name referred to the Feast of the Immaculate Conception but it was later renamed for the Spanish kingdom of Granada. (From Webster's New Geographical Dictionary, 1988, p467 & Room, Brewer's Dictionary of Names, 1992, p219)"),
  'D019426': NtO(MH='Integrases', MS='Recombinases that insert exogenous DNA into the host genome. Examples include proteins encoded by the POL GENE of RETROVIRIDAE and also by temperate BACTERIOPHAGES, the best known being BACTERIOPHAGE LAMBDA.'),
  'D019427': NtO(MH='HIV Integrase', MS='Enzyme of the HUMAN IMMUNODEFICIENCY VIRUS that is required to integrate viral DNA into cellular DNA in the nucleus of a host cell. HIV integrase is a DNA nucleotidyltransferase encoded by the pol gene.'),
  'D019428': NtO(MH='HIV Integrase Inhibitors', MS='Inhibitors of HIV INTEGRASE, an enzyme required for integration of viral DNA into cellular DNA.'),
  'D019429': NtO(MH='Integrase Inhibitors', MS='Compounds which inhibit or antagonize biosynthesis or actions of integrase.'),
  'D019430': NtO(MH='Guadeloupe', MS="The name of two islands of the West Indies, separated by a narrow channel. Their capital is Basse-Terre. They were discovered by Columbus in 1493, occupied by the French in 1635, held by the British at various times between 1759 and 1813, transferred to Sweden in 1813, and restored to France in 1816. Its status was changed from colony to a French overseas department in 1946. Columbus named it in honor of the monastery of Santa Maria de Guadalupe in Spain. (From Webster's New Geographical Dictionary, 1988, p470 & Room, Brewer's Dictionary of Names, 1992, p221)"),
  'D019431': NtO(MH='Saint Lucia', MS="An independent state in the West Indies. Its capital is Castries. It was probably discovered by Columbus in 1502 and first settled by the English in 1605. Contended for by the French and English in the 17th century, it was regarded as neutral in 1748 but changed hands many times in the wars of the 19th century. It became a self-governing state in association with Great Britain in 1967 and achieved independence in 1979. Columbus named it for the day on which he discovered it, the feast of St. Lucy, a Sicilian virgin martyr. (From Webster's New Geographical Dictionary, 1988, p1051 & Room, Brewer's Dictionary of Names, 1992, p477)"),
  'D019432': NtO(MH='Falkland Islands', MS="A British colony in the Atlantic Islands, comprising two principal islands, East Falkland and West Falkland. Its capital is Stanley. Discovered in 1592, it was not occupied until the French settled there briefly in 1764. Later the English settled there but were expelled by the Spanish in 1770. The Falklands were claimed by Argentina but were occupied in 1833 by the British who, after an April 1982 invasion by Argentina, regained them in June. The islands were named by British Captain John Strong in 1690 for the fifth Viscount Falkland who financed Strong's expedition. The Spanish name for the islands, Malvinas, is from the French Malouins, inhabitants of St. Malo who attempted to colonize the islands in 1764. (From Webster's New Geographical Dictionary, 1988, p389 & Room, Brewer's Dictionary of Names, 1992, p182)"),
  'D019433': NtO(MH='Chickenpox Vaccine', MS='A live, attenuated varicella virus vaccine used for immunization against chickenpox. It is recommended for children between the ages of 12 months and 13 years.'),
  'D019435': NtO(MH='Guernsey', MS="The second largest of the Channel Islands in the English Channel. It covers about 30 square miles and is where the Guernsey breed of cattle originated. The name is Scandinavian, Grani's island (ey means island). (From Webster's New Geographical Dictionary, 1988, p474 & Room, Brewer's Dictionary of Names, 1992, p222)"),
  'D019436': NtO(MH='Saint Vincent and the Grenadines', MS="A self-governing state of the Windward Islands in the West Indies, comprising Saint Vincent and the northern islets of the Grenadines. Its capital is Kingstown. It is one of the original homes of the Carib Indians supposed to have been sighted by Columbus in 1498. It was in English hands from 1627 till held by the French 1779-83. Saint Vincent subsequently became a British possession and, with other nearby British territories, was administered by the Governor of the Windward Islands till 1959. It attained a measure of independence in 1969 but achieved full independence as Saint Vincent and the Grenadines in 1979. Saint Vincent was the 4th century Spanish martyr on whose feast day Columbus discovered the island. Grenadines is derived from the Spanish kingdom of Granada. (From Webster's New Geographical Dictionary, 1988, p1054 & The Europa World Year Book 1993, p2441)"),
  'D019437': NtO(MH='Saint Kitts and Nevis', MS="An independent federation of the Leeward Islands in the West Indies, consisting of Saint Christopher, Nevis, and Sombrero. Its capital is Basseterre. It was discovered by Columbus in 1493, settled by the British in 1625, the first of the Leeward Islands to be colonized by them. It was held jointly by the French and English 1628-1713, but returned to Great Britain by the Treaty of Utrecht in 1713. It was held by the French 1782-83. Under the British for the next 200 years, it gained its independence in 1983. (From Webster's New Geographical Dictionary, 1988, p1045; Embassy, telephone 202-686-2636)"),
  'D019438': NtO(MH='Ritonavir', MS='An HIV protease inhibitor that works by interfering with the reproductive cycle of HIV. It also inhibits CYTOCHROME P-450 CYP3A.'),
  'D019439': NtO(MH='Chromaffin Cells', MS='Cells that store epinephrine secretory vesicles. During times of stress, the nervous system signals the vesicles to secrete their hormonal content. Their name derives from their ability to stain a brownish color with chromic salts. Characteristically, they are located in the adrenal medulla and paraganglia (PARAGANGLIA, CHROMAFFIN) of the sympathetic nervous system.'),
  'D019440': NtO(MH='Anti-Obesity Agents', MS='Agents that increase energy expenditure and weight loss by neural and metabolic regulation.'),
  'D019441': NtO(MH='Plant Epidermis', MS='A thin layer of cells forming the outer integument of seed plants and ferns. (Random House Unabridged Dictionary, 2d ed)'),
  'D019442': NtO(MH='Space Suits', MS='Pressure suits for wear in space or at very low ambient pressures within the atmosphere, designed to permit the wearer to leave the protection of a pressurized cabin. (NASA Thesaurus, 1994)'),
  'D019443': NtO(MH='Cosmic Dust', MS='Finely divided solid matter with particle sizes smaller than a micrometeorite, thus with diameters much smaller than a millimeter, moving in interplanetary space. (NASA Thesaurus, 1994)'),
  'D019444': NtO(MH='Vaccines, DNA', MS='Recombinant DNA vectors encoding antigens administered for the prevention or treatment of disease. The host cells take up the DNA, express the antigen, and present it to the immune system in a manner similar to that which would occur during natural infection. This induces humoral and cellular immune responses against the encoded antigens. The vector is called naked DNA because there is no need for complex formulations or delivery agents; the plasmid is injected in saline or other buffers.'),
  'D019445': NtO(MH='Solar Activity', MS='Any type of variation in the appearance of energy output of the sun. (NASA Thesaurus, 1994)'),
  'D019446': NtO(MH='Endotoxemia', MS='A condition characterized by the presence of ENDOTOXINS in the blood. On lysis, the outer cell wall of gram-negative bacteria enters the systemic circulation and initiates a pathophysiologic cascade of pro-inflammatory mediators.'),
  'D019448': NtO(MH='Ethnobotany', MS='The study of plant lore and agricultural customs of a people. In the fields of ETHNOMEDICINE and ETHNOPHARMACOLOGY, the emphasis is on traditional medicine and the existence and medicinal uses of PLANTS and PLANT EXTRACTS and their constituents, both historically and in modern times.'),
  'D019449': NtO(MH='Pouchitis', MS='Acute INFLAMMATION in the INTESTINAL MUCOSA of the continent ileal reservoir (or pouch) in patients who have undergone ILEOSTOMY and restorative proctocolectomy (PROCTOCOLECTOMY, RESTORATIVE).'),
  'D019450': NtO(MH='Genes, MDR', MS='Genes for MEMBRANE TRANSPORT PROTEINS that confer resistance to toxic compounds. Several superfamilies of these multidrug export proteins are known and found in both prokaryotes and eukaryotes.'),
  'D019451': NtO(MH='Information Management', MS='Management of the acquisition, organization, storage, retrieval, and dissemination of information. (From Thesaurus of ERIC Descriptors, 1994)'),
  'D019452': NtO(MH='Terminally Ill', MS="Persons with an incurable or irreversible illness at the end stage that will result in death within a short time. (From O'Leary et al., Lexikon: Dictionary of Health Care Terms, Organizations, and Acronyms for the Era of Reform, 1994, p780)"),
  'D019453': NtO(MH='Escherichia coli O157', MS='A verocytotoxin-producing serogroup belonging to the O subfamily of Escherichia coli which has been shown to cause severe food-borne disease. A strain from this serogroup, serotype H7, which produces SHIGA TOXINS, has been linked to human disease outbreaks resulting from contamination of foods by E. coli O157 from bovine origin.'),
  'D019454': NtO(MH='Chronotherapy', MS='The adaptation of therapeutic approaches such as pharmacological (DRUG CHRONOTHERAPY), surgical, radiological, or physical to the known variations in biological RHYTHMICITY, such as CIRCADIAN RHYTHMS. The treatment is aimed at supporting normal rhythms, or modifying the timing of therapy to achieve maximal efficacy and minimal adverse effect.'),
  'D019455': NtO(MH='Self Psychology', MS='Psychoanalytic theory focusing on interpretation of behavior in reference to self. (From APA, Thesaurus of Psychological Terms, 1994) This elaboration of the psychoanalytic concepts of narcissism and the self, was developed by Heinz Kohut, and stresses the importance of the self-awareness of excessive needs for approval and self-gratification.'),
  'D019456': NtO(MH='Health Transition', MS='Demographic and epidemiologic changes that have occurred in the last five decades in many developing countries and that are characterized by major growth in the number and proportion of middle-aged and elderly persons and in the frequency of the diseases that occur in these age groups. The health transition is the result of efforts to improve maternal and child health via primary care and outreach services and such efforts have been responsible for a decrease in the birth rate; reduced maternal mortality; improved preventive services; reduced infant mortality, and the increased life expectancy that defines the transition. (From Ann Intern Med 1992 Mar 15;116(6):499-504)'),
  'D019457': NtO(MH='Chromosome Breakage', MS='A type of chromosomal aberration involving DNA BREAKS. Chromosome breakage can result in CHROMOSOMAL TRANSLOCATION; CHROMOSOME INVERSION; or SEQUENCE DELETION.'),
  'D019458': NtO(MH='Insurance Coverage', MS='Generally refers to the amount of protection available and the kind of loss which would be paid for under an insurance contract with an insurer. (Slee & Slee, Health Care Terms, 2d ed)'),
  'D019459': NtO(MH='Urothelium', MS='The epithelial lining of the URINARY TRACT.'),
  'D019460': NtO(MH='Tobacco Industry', MS='The aggregate business enterprise of agriculture, manufacture, and distribution related to tobacco and tobacco-derived products.'),
  'D019461': NtO(MH='Employee Retirement Income Security Act', MS="A 1974 Federal act which preempts states' rights with regard to workers' pension benefits and employee benefits. It does not affect the benefits and rights of employees whose employer is self-insured. (From Slee & Slee, Health Care Reform Terms, 1993)"),
  'D019462': NtO(MH='Syncope, Vasovagal', MS='Loss of consciousness due to a reduction in blood pressure that is associated with an increase in vagal tone and peripheral vasodilation.'),
  'D019463': NtO(MH='Mice, Inbred MRL lpr', MS='A mouse substrain that is genetically predisposed to the development of systemic lupus erythematosus-like syndrome, which has been found to be clinically similar to the human disease. It has been determined that this mouse strain carries a mutation in the fas gene. Also, the MRL/lpr is a useful model to study behavioral and cognitive deficits found in autoimmune diseases and the efficacy of immunosuppressive agents.'),
  'D019464': NtO(MH='Tax Exemption', MS='Status not subject to taxation; as the income of a philanthropic organization. Tax-exempt organizations may also qualify to receive tax-deductible donations if they are considered to be nonprofit corporations under Section 501(c)3 of the United States Internal Revenue Code.'),
  'D019465': NtO(MH='Craniofacial Abnormalities', MS='Congenital structural deformities, malformations, or other abnormalities of the cranium and facial bones.'),
  'D019466': NtO(MH='Glare', MS='Relatively bright light, or the dazzling sensation of relatively bright light, which produces unpleasantness or discomfort, or which interferes with optimal VISION, OCULAR. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D019467': NtO(MH='Subacute Care', MS='Medical and skilled nursing services provided to patients who are not in an acute phase of an illness but who require a level of care higher than that provided in a long-term care setting. (JCAHO, Lexikon, 1994)'),
  'D019468': NtO(MH='Disease Management', MS='A broad approach to appropriate coordination of the entire disease treatment process that often involves shifting away from more expensive inpatient and acute care to areas such as preventive medicine, patient counseling and education, and outpatient care. This concept includes implications of appropriate versus inappropriate therapy on the overall cost and clinical outcome of a particular disease. (From Hosp Pharm 1995 Jul;30(7):596)'),
  'D019469': NtO(MH='Indinavir', MS='A potent and specific HIV protease inhibitor that appears to have good oral bioavailability.'),
  'D019470': NtO(MH='Biolistics', MS='Techniques where DNA is delivered directly into organelles at high speed using projectiles coated with nucleic acid, shot from a helium-powered gun (gene gun). One of these techniques involves immunization by DNA VACCINES, which delivers DNA-coated gold beads to the epidermis.'),
  'D019471': NtO(MH='Elementary Particle Interactions', MS='The interactions of particles responsible for their scattering and transformations (decays and reactions). Because of interactions, an isolated particle may decay into other particles. Two particles passing near each other may transform, perhaps into the same particles but with changed momenta (elastic scattering) or into other particles (inelastic scattering). Interactions fall into three groups: strong, electromagnetic, and weak. (From McGraw-Hill Encyclopedia of Science & Technology, 7th ed)'),
  'D019472': NtO(MH='Universal Health Insurance', MS='Health insurance coverage for all persons in a state or country, rather than for some subset of the population. It may extend to the unemployed as well as to the employed; to aliens as well as to citizens; for pre-existing conditions as well as for current illnesses; for mental as well as for physical conditions.'),
  'D019473': NtO(MH='Transcription Factors, TFII', MS='The so-called general transcription factors that bind to RNA POLYMERASE II and that are required to initiate transcription. They include TFIIA; TFIIB; TFIID; TFIIE; TFIIF; TFIIH; TFII-I; and TFIIJ. In vivo they apparently bind in an ordered multi-step process and/or may form a large preinitiation complex called RNA polymerase II holoenzyme.'),
  'D019474': NtO(MH='Transcription Factors, TFIII', MS='Factors that bind to RNA POLYMERASE III and aid in transcription. They include the assembly factors TFIIIA and TFIIIC and the initiation factor TFIIIB. All combine to form a preinitiation complex at the promotor that directs the binding of RNA POLYMERASE III.'),
  'D019475': NtO(MH='Camphor 5-Monooxygenase', MS='A soluble cytochrome P-450 enzyme that catalyzes camphor monooxygenation in the presence of putidaredoxin, putidaredoxin reductase, and molecular oxygen. This enzyme, encoded by the CAMC gene also known as CYP101, has been crystallized from bacteria and the structure is well defined. Under anaerobic conditions, this enzyme reduces the polyhalogenated compounds bound at the camphor-binding site.'),
  'D019476': NtO(MH='Insect Proteins', MS='Proteins found in any species of insect.'),
  'D019477': NtO(MH='Portrait', MS='Work consisting of graphic representations, especially of the face, of real persons, usually posed, living or dead. They are pictures whose purpose is the portrayal of an individual or group of individuals, not pictures which merely include people as part of an event or scene. (From Thesaurus for Graphic Materials II, p540, 1995)'),
  'D019478': NtO(MH='Academic Dissertation', MS='Work consisting of formal presentations made usually to fulfill requirements for an academic degree.'),
  'D019479': NtO(MH='Account Book', MS='Book in which personal or commercial accounts of financial transactions are recorded. (From Random House Unabridged Dictionary, 2d ed)'),
  'D019480': NtO(MH='Advertisement', MS='Work consisting of publicly distributed notices, usually as paid announcements in mass media such as newspapers, magazines, or on billboards. They include those in motion picture, television advertising, radio, or electronic media.'),
  'D019482': NtO(MH='Almanac', MS='Work consisting of a calendar of days, weeks, and months, together with information such as astronomical data, various statistics, etc. (From Genre Terms: A Thesaurus for Use in Rare Book and Special Collections Cataloguing, 2d ed)'),
  'D019483': NtO(MH='Disintegrins', MS='A family of polypeptides purified from snake venoms, which contain the arginine-glycine-aspartic acid (RGD) sequence. The RGD tripeptide binds to integrin receptors and thus competitively inhibits normal integrin-ligand interactions. Disintegrins thus block adhesive functions and act as platelet aggregation inhibitors.'),
  'D019484': NtO(MH='Address', MS='Work consisting of speeches, orations, or written statements, usually formal, directed to a particular group of persons. These are different from a LECTURE that is usually delivered to classes for instructional purposes.'),
  'D019485': NtO(MH='Bone Morphogenetic Proteins', MS='Bone-growth regulatory factors that are members of the transforming growth factor-beta superfamily of proteins. They are synthesized as large precursor molecules which are cleaved by proteolytic enzymes. The active form can consist of a dimer of two identical proteins or a heterodimer of two related bone morphogenetic proteins.'),
  'D019486': NtO(MH='Animation', MS='A film or video wholly or partially created by photographing drawings, sculptures, or other inanimate things in sequence to create the illusion of motion. Animations are also generated by computers. (From Moving Image Materials: Genre Terms, 1988)'),
  'D019487': NtO(MH='Annual Report', MS='Annual statements concerning the administrative and operational functions of an institution or organization.'),
  'D019488': NtO(MH='Architectural Drawing', MS='Work consisting of drawings of architecture and architectural projects, whether the project was executed or not. (Art & Architecture Thesaurus, 1990, v.2)'),
  'D019489': NtO(MH='Book Illustrations', MS='Works consisting of photographs, prints, drawings, portraits, plates, diagrams, facsimiles, maps, tables, or other representations or systematic arrangements of data designed to elucidate or decorate the contents of a publication. (From The ALA Glossary of Library and Information Science, 1983, p114)'),
  'D019490': NtO(MH='Broadside', MS='Work consisting of published pieces of paper or other material, usually printed on one side and intended to be read unfolded and usually intended to be posted, publicly distributed, or sold, e.g., proclamations, handbills, newssheets, etc. (From Genre Terms: A Thesaurus for Use in Rare Book and Special Collections Cataloguing, 2d ed)'),
  'D019491': NtO(MH='Bookplate', MS="Works consisting of book owner's identification labels. They are usually intended for attaching inside a book or similar object. (From Thesaurus for Graphic Materials II: Genre and Physical Characteristic Terms, 1995)"),
  'D019492': NtO(MH='Caricature', MS='Work portraying in a critical or facetious way a real individual or group, or a figure representing a social, political, ethnic, or racial type. The effect is usually achieved through distortion or exaggeration of characteristics. (Genre Terms: A Thesaurus for Use in Rare Book and Special Collection Cataloguing, 2d ed)'),
  'D019493': NtO(MH='Cartoon', MS='Image used to comment on such things as contemporary events, social habits, or political trends; usually executed in a broad or abbreviated manner.'),
  'D019494': NtO(MH='Catalog', MS='Work consisting of bibliographic records, created according to specific and uniform principles of construction and under the control of an authority file, which describe the materials contained in a collection, library, or group of libraries. Catalogs include also lists of materials prepared for a particular purpose, such as exhibition catalogs, sales catalogs, garden catalogs, medical supply catalogs. (From The ALA Glossary of Library and Information Sciences, 1983)'),
  'D019496': NtO(MH='Cancer Vaccines', MS="Vaccines or candidate vaccines designed to prevent or treat cancer. Vaccines are produced using the patient's own whole tumor cells as the source of antigens, or using tumor-specific antigens, often recombinantly produced."),
  'D019497': NtO(MH='Diary', MS="Work consisting of records, usually private, of writers' experiences, observations, feelings, attitudes, etc. They may also be works marked in calendar order in which to note appointments and the like. (From Random House Unabridged Dictionary, 2d ed)"),
  'D019499': NtO(MH='Documentaries and Factual Films', MS='Works consisting of films, videos, and programs which depict actual persons or actual events. They do not include frank historical re-creations and do not attempt to judge the truth of the depiction in a film purporting to be factual or documentary in character. (From Moving Image Materials: Genre Terms, 1988)'),
  'D019500': NtO(MH='Encyclopedia', MS='Work containing informational articles on subjects in every field of knowledge, usually arranged in alphabetical order, or a similar work limited to a special field or subject.'),
  'D019502': NtO(MH='Ephemera', MS='Works consisting of transient everyday items, usually printed on paper, that are produced for a specific limited use and then often thrown away. (From Genre Terms: A Thesaurus for Use in Rare Book and Special Collections Cataloguing, 2d ed & The ALA Glossary of Library and Information Science, 1983)'),
  'D019504': NtO(MH='Eulogy', MS='Work consisting of speeches or writings in praise of a person or thing, especially a set oration in honor of a deceased person. They differ from FUNERAL SERMON which are delivered at ceremonies for the deceased prior to their burial or cremation. (From Random House Unabridged Dictionary, 2d ed)'),
  'D019505': NtO(MH='Funeral Sermon', MS='Work consisting of sermons delivered at ceremonies for a dead person prior to burial or cremation. (From Random House Unabridged Dictionary, 2d ed)'),
  'D019508': NtO(MH='Guidebook', MS='Work consisting of publications for travelers that give information about a city, region, or country, or similar handbooks about buildings, museums, etc. (The ALA Glossary of Library and Information Science, 1983)'),
  'D019509': NtO(MH='Herbal', MS='Work such as books on herbs or plants usually describing their medicinal value. (Random House Unabridged Dictionary, 2d ed)'),
  'D019512': NtO(MH='Pancreatitis, Alcoholic', MS='Acute or chronic INFLAMMATION of the PANCREAS due to excessive ALCOHOL DRINKING. Alcoholic pancreatitis usually presents as an acute episode but it is a chronic progressive disease in alcoholics.'),
  'D019513': NtO(MH='Feminism', MS="The theory of the political, economic, and social equality of the sexes and organized activity on behalf of women's rights and interests. (Webster New Collegiate Dictionary, 1981)"),
  'D019514': NtO(MH='Instructional Film and Video', MS='Works consisting of nonfiction films and video designed to teach, instruct, or train. (From Moving Image Materials: Genre Terms, 1988)'),
  'D019516': NtO(MH='Orthodontic Space Closure', MS='Therapeutic closure of spaces caused by the extraction of teeth, the congenital absence of teeth, or the excessive space between teeth.'),
  'D019517': NtO(MH='Unedited Footage', MS='Work consisting of untitled raw motion picture and video footage which has not been edited or assembled into a finished work. (From: Moving Image Materials: Genre Terms, 1988)'),
  'D019518': NtO(MH='Postprandial Period', MS='The time frame after a meal or FOOD INTAKE.'),
  'D019519': NtO(MH='Poster', MS='Work consisting of single or multi-sheet notices made to attract attention to events, activities, causes, goods, or services. They are for posting, usually in a public place and are chiefly pictorial. They are intended to make an immediate impression from a distance. Posters do not include poster presentations at conferences and meetings. (From Thesaurus for Graphic Materials II: Genre and Physical Characteristic Headings, 1995)'),
  'D019520': NtO(MH='Living Donors', MS='Non-cadaveric providers of organs for transplant to related or non-related recipients.'),
  'D019521': NtO(MH='Body Patterning', MS='The processes occurring in early development that direct morphogenesis. They specify the body plan ensuring that cells will proceed to differentiate, grow, and diversify in size and shape at the correct relative positions. Included are axial patterning, segmentation, compartment specification, limb position, organ boundary patterning, blood vessel patterning, etc.'),
  'D019522': NtO(MH='Vaginal Discharge', MS='A common gynecologic disorder characterized by an abnormal, nonbloody discharge from the genital tract.'),
  'D019523': NtO(MH='Sermon', MS='Work consisting of discourses for the purpose of religious instruction or exhortation, especially one based on a text of Scripture and delivered by a member of the clergy, as part of a religious service. (From: Random House Unabridged Dictionary, 2d ed)'),
  'D019525': NtO(MH='Price List', MS='Work consisting of lists giving the prices of items for sale, including drugs, equipment, books, etc. Price lists are less detailed than catalogs and not as long.'),
  'D019527': NtO(MH='Prospectus', MS='Work consisting of advertisements separately printed and distributed by a publisher to describe and solicit orders for a recent or forthcoming publication. In the case of books, they may include sample pages. (From: ALA Glossary of Library and Information Science, 1983)'),
  'D019528': NtO(MH='Lecture Note', MS='Work consisting of notes taken at the delivery or reading of a speech before an audience or class, usually given to instruct. (From Random House Unabridged Dictionary, 2d ed)'),
  'D019529': NtO(MH='Sexuality', MS='The sexual functions, activities, attitudes, and orientations of an individual. Sexuality, male or female, becomes evident at PUBERTY under the influence of gonadal steroids (TESTOSTERONE or ESTRADIOL), and social effects.'),
  'D019531': NtO(MH='Lecture', MS='Work consisting of speeches read or delivered before an audience or class, especially for instruction or to set forth some subject. They are differentiated from an ADDRESS which are less didactic and more informational, entertaining, inspirational, or polemic. (From Random House Unabridged Dictionary, 2d ed)'),
  'D019532': NtO(MH='Map', MS='Works consisting of representations, normally to scale and on a flat medium, of a selection of material or abstract features on the surface of the earth. They may be used also in delineating the heavens and celestial bodies. (From Anglo-American Cataloguing Rules, 2d ed, p619)'),
  'D019533': NtO(MH='Streptococcus oralis', MS='A species of gram-positive, coccoid bacteria that is numerous in the mouth and throat. It is a common cause of endocarditis and is also implicated in dental plaque formation.'),
  'D019534': NtO(MH='Shoulder Impingement Syndrome', MS='Compression of the ROTATOR CUFF tendons and subacromial bursa between the HUMERAL HEAD and  the ACROMION of the SCAPULA. This condition is associated with subacromial BURSITIS, as well as rotator cuff (largely supraspinatus) and bicipital tendon INFLAMMATION.'),
  'D019535': NtO(MH='Mycoplasma hominis', MS='A common inhabitant of the vagina and cervix and a potential human pathogen, causing infections of the male and female reproductive tracts. It has also been associated with respiratory disease and pharyngitis. (From Dorland, 28th ed)'),
  'D019538': NtO(MH='Health Care Surveys', MS='Statistical measures of utilization and other aspects of the provision of health care services including hospitalization and ambulatory care.'),
  'D019539': NtO(MH='Pharmacopoeia', MS='Authoritative work containing lists of drugs and preparations, their description, formulation, analytic composition, main chemical properties, standards for strength, purity, and dosage, chemical tests for determining identity, etc. They have the status of a standard.'),
  'D019540': NtO(MH='Area Under Curve', MS='A statistical means of summarizing information from a series of measurements on one individual. It is frequently used in clinical pharmacology where the AUC from serum levels can be interpreted as the total uptake of whatever has been administered. As a plot of the concentration of a drug against time, after a single dose of medicine, producing a standard shape curve, it is a means of comparing the bioavailability of the same drug made by different companies. (From Winslade, Dictionary of Clinical Research, 1992)'),
  'D019542': NtO(MH='Program', MS='Works consisting of lists of the events, pieces, performers, speakers, etc., of an entertainment, ceremony, or the like. (From: Genre Terms: A Thesaurus for Use in Rare Book and Special Collections Cataloging, 2d ed)'),
  'D019544': NtO(MH='Equipment Failure Analysis', MS='The evaluation of incidents involving the loss of function of a device. These evaluations are used for a variety of purposes such as to determine the failure rates, the causes of failures, costs of failures, and the reliability and maintainability of devices.'),
  'D019545': NtO(MH='Intuition', MS='Knowing or understanding without conscious use of reasoning. (Thesaurus of ERIC Descriptors, 1994)'),
  'D019546': NtO(MH='Couples Therapy', MS='Psychotherapy used for either unmarried couples or married couples, of mixed or same sex.'),
  'D019547': NtO(MH='Neck Pain', MS='Discomfort or more intense forms of pain that are localized to the cervical region. This term generally refers to pain in the posterior or lateral regions of the neck.'),
  'D019548': NtO(MH='Crime Victims', MS='Individuals subjected to and adversely affected by criminal activity. (APA, Thesaurus of Psychological Index Terms, 1994)'),
  'D019550': NtO(MH='Environmental Medicine', MS='Medical specialty concerned with environmental factors that may impinge upon human disease, and development of methods for the detection, prevention, and control of environmentally related disease.'),
  'D019551': NtO(MH='Dental Disinfectants', MS='Chemicals especially for use on instruments to destroy pathogenic organisms. (Boucher, Clinical Dental Terminology, 4th ed)'),
  'D019552': NtO(MH='Infection Control, Dental', MS='Efforts to prevent and control the spread of infections within dental health facilities or those involving provision of dental care.'),
  'D019553': NtO(MH='Tooth, Nonvital', MS='A tooth from which the dental pulp has been removed or is necrotic. (Boucher, Clinical Dental Terminology, 4th ed)'),
  'D019554': NtO(MH='Dental Restoration Wear', MS='Occlusal wear of the surfaces of restorations and surface wear of dentures.'),
  'D019555': NtO(MH='Physician-Nurse Relations', MS='The reciprocal interaction of physicians and nurses.'),
  'D019556': NtO(MH='COS Cells', MS='CELL LINES derived from the CV-1 cell line by transformation with a replication origin defective mutant of SV40 VIRUS, which codes for wild type large T antigen (ANTIGENS, POLYOMAVIRUS TRANSFORMING). They are used for transfection and cloning. (The CV-1 cell line was derived from the kidney of an adult male African green monkey (CHLOROCEBUS AETHIOPS).)'),
  'D019557': NtO(MH='Dermatitis, Perioral', MS='A papular eruption of unknown etiology that progresses to residual papular erythema and scaling usually confined to the area of the mouth, and almost exclusively occurring in young women. It may also be localized or extend to involve the eyelids and adjacent glabella area of the forehead (periocular dermatitis). (Dorland, 28th ed)'),
  'D019558': NtO(MH='Occlusal Adjustment', MS='Selective grinding of occlusal surfaces of the teeth in an effort to eliminate premature contacts and occlusal interferences; to establish optimal masticatory effectiveness, stable occlusal relationships, direction of main occlusal forces, and efficient multidirectional patterns, to improve functional relations and to induce physiologic stimulation of the masticatory system; to eliminate occlusal trauma; to eliminate abnormal muscle tension; to aid in the stabilization of orthodontic results; to treat periodontal and temporomandibular joint problems; and in restorative procedures. (From Jablonski, Dictionary of Dentistry, 1992)'),
  'D019559': NtO(MH='Capillary Leak Syndrome', MS='A condition characterized by recurring episodes of fluid leaking from capillaries into extra-vascular compartments causing hematocrit to rise precipitously. If not treated, generalized vascular leak can lead to generalized EDEMA; SHOCK; cardiovascular collapse; and MULTIPLE ORGAN FAILURE.'),
  'D019560': NtO(MH='Echocardiography, Three-Dimensional', MS='Echocardiography amplified by the addition of depth to the conventional two-dimensional ECHOCARDIOGRAPHY visualizing only the length and width of the heart. Three-dimensional ultrasound imaging was first described in 1961 but its application to echocardiography did not take place until 1974. (Mayo Clin Proc 1993;68:221-40)'),
  'D019561': NtO(MH='Echocardiography, Four-Dimensional', MS='Dynamic three-dimensional echocardiography using the added dimension of time to impart the cinematic perception of motion. (Mayo Clin Proc 1993;68:221-40)'),
  'D019562': NtO(MH='Viral Load', MS='The quantity of measurable virus in a body fluid. Change in viral load, measured in plasma, is sometimes used as a SURROGATE MARKER in disease progression.'),
  'D019563': NtO(MH='Mechanics', MS='The branch of physics which deals with the motions of material bodies, including kinematics, dynamics, and statics. When the laws of mechanics are applied to living structures, as to the locomotor system, it is referred to as BIOMECHANICAL PHENOMENA. (From Dorland, 28th ed)'),
  'D019564': NtO(MH='Unnecessary Procedures', MS='Diagnostic, therapeutic, and investigative procedures prescribed and performed by health professionals, the results of which do not justify the benefits or hazards and costs to the patient.'),
  'D019565': NtO(MH='Metals, Light', MS="Metals with low specific gravity, typically smaller than 5, characterized by a single valence (1, 2, or 3), a simple spectrum, strong electromotive force (positive), and colorless compounds. (From Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D019567': NtO(MH='Back Injuries', MS='General or unspecified injuries to the posterior part of the trunk. It includes injuries to the muscles of the back.'),
  'D019568': NtO(MH='Lung, Hyperlucent', MS='A lung with reduced markings on its chest radiograph and increased areas of transradiancy (hyperlucency). A hyperlucent lung is usually associated with pulmonary emphysema or PNEUMOTHORAX.'),
  'D019569': NtO(MH='Hemifacial Spasm', MS='Recurrent clonic contraction of facial muscles, restricted to one side. It may occur as a manifestation of compressive lesions involving the seventh cranial nerve (FACIAL NERVE DISEASES), during recovery from BELL PALSY, or in association with other disorders. (From Adams et al., Principles of Neurology, 6th ed, p1378)'),
  'D019570': NtO(MH='Inhalation Exposure', MS='The exposure to potentially harmful chemical, physical, or biological agents by inhaling them.'),
  'D019571': NtO(MH='Arrhythmogenic Right Ventricular Dysplasia', MS='A congenital cardiomyopathy that is characterized by infiltration of adipose and fibrous tissue into the RIGHT VENTRICLE wall and loss of myocardial cells. Primary injuries usually are at the free wall of right ventricular and right atria resulting in ventricular and supraventricular arrhythmias.'),
  'D019572': NtO(MH='Retinal Neoplasms', MS='Tumors or cancer of the RETINA.'),
  'D019573': NtO(MH='Epithelium, Corneal', MS='Stratified squamous epithelium that covers the outer surface of the CORNEA. It is smooth and contains many free nerve endings.'),
  'D019574': NtO(MH='Optic Nerve Neoplasms', MS='Benign and malignant neoplasms that arise from the optic nerve or its sheath. OPTIC NERVE GLIOMA is the most common histologic type. Optic nerve neoplasms tend to cause unilateral visual loss and an afferent pupillary defect and may spread via neural pathways to the brain.'),
  'D019575': NtO(MH='Blindness, Cortical', MS='Total loss of vision in all or part of the visual field due to bilateral OCCIPITAL LOBE (i.e., VISUAL CORTEX) damage or dysfunction. Anton syndrome is characterized by the psychic denial of true, organic cortical blindness. (Adams et al., Principles of Neurology, 6th ed, p460)'),
  'D019576': NtO(MH='Porpoises', MS="Mammals of the family Phocoenidae comprising four genera found in the North Pacific Ocean and both sides of the North Atlantic Ocean and in various other seas. They differ from DOLPHINS in that porpoises have a blunt snout and a rather stocky body while dolphins have a beak-like snout and a slender, streamlined body. They usually travel in small groups. (From Walker's Mammals of the World, 5th ed, pp1003-4)"),
  'D019577': NtO(MH='Mole Rats', MS="Any of several burrowing rodents of the families MURIDAE and Bathyergidae, found in eastern Europe, Africa, and Asia. They have short limbs, small eyes with permanently closed lids, and no tail. Three genera SPALAX (Muridae), Heterocephalus (Bathyergidae) and Cryptomys (Bathyergidae) are used frequently as experimental animals in biomedical research. (From Walker's Mammals of the World, 6th ed)"),
  'D019578': NtO(MH='Multiple System Atrophy', MS='A syndrome complex composed of three conditions which represent clinical variants of the same disease process: STRIATONIGRAL DEGENERATION; SHY-DRAGER SYNDROME; and the sporadic form of OLIVOPONTOCEREBELLAR ATROPHIES. Clinical features include autonomic, cerebellar, and basal ganglia dysfunction. Pathologic examination reveals atrophy of the basal ganglia, cerebellum, pons, and medulla, with prominent loss of autonomic neurons in the brain stem and spinal cord. (From Adams et al., Principles of Neurology, 6th ed, p1076; Baillieres Clin Neurol 1997 Apr;6(1):187-204; Med Clin North Am 1999 Mar;83(2):381-92)'),
  'D019579': NtO(MH='Neocortex', MS='The largest portion of the CEREBRAL CORTEX in which the NEURONS are arranged in six layers in the mammalian brain: molecular, external granular, external pyramidal, internal granular, internal pyramidal and multiform layers.'),
  'D019580': NtO(MH='Perforant Pathway', MS='A pathway of fibers that originates in the lateral part of the ENTORHINAL CORTEX, perforates the SUBICULUM of the HIPPOCAMPUS, and runs into the stratum moleculare of the hippocampus, where these fibers synapse with others that go to the DENTATE GYRUS where the pathway terminates. It is also known as the perforating fasciculus.'),
  'D019581': NtO(MH='Neuropil', MS='A dense intricate feltwork of interwoven fine glial processes, fibrils, synaptic terminals, axons, and dendrites interspersed among the nerve cells in the gray matter of the central nervous system.'),
  'D019583': NtO(MH='Dose Fractionation, Radiation', MS='Administration of the total dose of radiation (RADIATION DOSAGE) in parts, at timed intervals.'),
  'D019584': NtO(MH='Hot Flashes', MS='A sudden, temporary sensation of heat predominantly experienced by some women during MENOPAUSE. (Random House Unabridged Dictionary, 2d ed)'),
  'D019585': NtO(MH='Intracranial Hypotension', MS='Reduction of CEREBROSPINAL FLUID pressure characterized clinically by ORTHOSTATIC HEADACHE and occasionally by an ABDUCENS NERVE PALSY; HEARING LOSS; NAUSEA; neck stiffness, and other symptoms. This condition may be spontaneous or secondary to CEREBROSPINAL FLUID LEAK; SPINAL PUNCTURE; NEUROSURGICAL PROCEDURES; DEHYDRATION; UREMIA; trauma (see also CRANIOCEREBRAL TRAUMA); and other processes. Chronic hypotension may be associated with subdural hematomas (see HEMATOMA, SUBDURAL) or hygromas. (From Semin Neurol 1996 Mar;16(1):5-10; Adams et al., Principles of Neurology, 6th ed, pp637-8)'),
  'D019586': NtO(MH='Intracranial Hypertension', MS='Increased pressure within the cranial vault. This may result from several conditions, including HYDROCEPHALUS; BRAIN EDEMA; intracranial masses; severe systemic HYPERTENSION; PSEUDOTUMOR CEREBRI; and other disorders.'),
  'D019587': NtO(MH='Dietary Supplements', MS='Products in capsule, tablet or liquid form that provide dietary ingredients, and that are intended to be taken by mouth to increase the intake of nutrients.  Dietary supplements can include macronutrients, such as proteins, carbohydrates, and fats; and/or MICRONUTRIENTS, such as VITAMINS; MINERALS; and PHYTOCHEMICALS.'),
  'D019588': NtO(MH='Aging, Premature', MS='Changes in the organism associated with senescence, occurring at an accelerated rate.'),
  'D019589': NtO(MH='Tooth Components', MS='The anatomical components of a tooth, its tissues, and correlated parts.'),
  'D019590': NtO(MH='Nucleocapsid Proteins', MS='Viral proteins found in either the NUCLEOCAPSID or the viral core (VIRAL CORE PROTEINS).'),
  'D019591': NtO(MH='Pseudophakia', MS='Presence of an intraocular lens after cataract extraction.'),
  'D019595': NtO(MH='Severe Dengue', MS='A virulent form of dengue characterized by THROMBOCYTOPENIA and an increase in vascular permeability (grades I and II) and distinguished by a positive pain test (e.g., TOURNIQUET PAIN TEST). When accompanied by SHOCK (grades III and IV), it is called dengue shock syndrome.'),
  'D019596': NtO(MH='Wolves', MS='Any of several large carnivorous mammals of the family CANIDAE that usually hunt in packs.'),
  'D019597': NtO(MH='Flax', MS='A plant genus of the family LINACEAE that is cultivated for its fiber (manufactured into linen cloth). It contains a trypsin inhibitor and the seed is the source of LINSEED OIL.'),
  'D019599': NtO(MH='Mossy Fibers, Hippocampal', MS='Axons of certain cells in the DENTATE GYRUS. They project to the polymorphic layer of the dentate gyrus and to the proximal dendrites of PYRAMIDAL CELLS of the HIPPOCAMPUS. These mossy fibers should not be confused with mossy fibers that are cerebellar afferents (see NERVE FIBERS).'),
  'D019600': NtO(MH='Neuropil Threads', MS='Abnormal structures located chiefly in distal dendrites and, along with NEUROFIBRILLARY TANGLES and SENILE PLAQUES, constitute the three morphological hallmarks of ALZHEIMER DISEASE. Neuropil threads are made up of straight and paired helical filaments which consist of abnormally phosphorylated microtubule-associated tau proteins. It has been suggested that the threads have a major role in the cognitive impairment seen in Alzheimer disease.'),
  'D019601': NtO(MH='Pacific States', MS='The geographic designation for states bordering on or located in the Pacific Ocean. The states so designated are Alaska, California, Hawaii, Oregon, and Washington. (U.S. Geologic Survey telephone communication)'),
  'D019602': NtO(MH='Food and Beverages', MS='Edible or potable substances.'),
  'D019603': NtO(MH='Thermoplasmales', MS='An order of aerobic, thermophilic archaea, in the kingdom EURYARCHAEOTA, characterized by the absence of a cell wall. Two genera have been described: THERMOPLASMA and Picrophilus.'),
  'D019605': NtO(MH='Euryarchaeota', MS='A phylum of ARCHAEA comprising at least seven classes: Methanobacteria, Methanococci, Halobacteria (extreme halophiles), Archaeoglobi (sulfate-reducing species), Methanopyri, and the thermophiles: Thermoplasmata, and Thermococci.'),
  'D019606': NtO(MH='Phosphoamino Acids', MS='Amino acids that contain phosphorus as an integral part of the molecule.'),
  'D019607': NtO(MH='Brassicaceae', MS='A plant family of the order Capparales, subclass Dilleniidae, class Magnoliopsida. They are mostly herbaceous plants with peppery-flavored leaves, due to gluconapin (GLUCOSINOLATES) and its hydrolysis product butenylisotrhiocyanate. The family includes many plants of economic importance that have been extensively altered and domesticated by humans. Flowers have 4 petals. Podlike fruits contain a number of seeds. Cress is a general term used for many in the Brassicacea family. Rockcress is usually ARABIS; Bittercress is usually CARDAMINE; Yellowcress is usually RORIPPA; Pennycress is usually THLASPI; Watercress refers to NASTURTIUM; or RORIPPA or TROPAEOLUM; Gardencress refers to LEPIDIUM; Indiancress refers to TROPAEOLUM.'),
  'D019608': NtO(MH='Archaeoglobales', MS='An order of extremely thermophilic, sulfate-reducing archaea, in the kingdom EURYARCHAEOTA. The single family Archaeoglobaceae contains one genus ARCHAEOGLOBUS.'),
  'D019609': NtO(MH='Archaeoglobus', MS='A genus of extremely thermophilic, sulfate-reducing archaea, in the family Archaeoglobaceae.'),
  'D019610': NtO(MH='Cytoprotection', MS='The process by which chemical compounds provide protection to cells against harmful agents.'),
  'D019611': NtO(MH='Archaeoglobus fulgidus', MS='A species of extremely thermophilic, sulfur-reducing archaea. It grows at a maximum temperature of 95 degrees C. in marine or deep-sea geothermal areas.'),
  'D019612': NtO(MH='Fagopyrum', MS='A plant genus of the family POLYGONACEAE that is used as an EDIBLE GRAIN. Although the seeds are used as cereal, the plant is not one of the cereal grasses (POACEAE).'),
  'D019613': NtO(MH='Haloarcula', MS='A genus of HALOBACTERIACEAE distinguished from other genera in the family by the presence of specific derivatives of TGD-2 polar lipids. Haloarcula are found in neutral saline environments such as salt lakes, marine salterns, and saline soils.'),
  'D019614': NtO(MH='Haloarcula marismortui', MS='A species of halophilic archaea distinguished by its production of acid from sugar. This species was previously called Halobacterium marismortui.'),
  'D019615': NtO(MH='Crenarchaeota', MS='A kingdom in the domain ARCHAEA comprised of thermoacidophilic, sulfur-dependent organisms. The two orders are SULFOLOBALES and THERMOPROTEALES.'),
  'D019616': NtO(MH='Thoracic Surgical Procedures', MS='Surgery performed on the thoracic organs, most commonly the lungs and the heart.'),
  'D019617': NtO(MH='Halobacterium salinarum', MS='A species of halophilic archaea found in salt lakes. Some strains form a PURPLE MEMBRANE under anaerobic conditions.'),
  'D019619': NtO(MH='Thermoproteales', MS='An order of CRENARCHAEOTA comprised of rod, disc, or spherical shaped, nonseptate, anaerobic, extreme thermophiles and found in solfataric hot waters, mud holes, and superheated submarine environments.'),
  'D019620': NtO(MH='Desulfurococcaceae', MS="A family of archaea, in the order DESULFUROCOCCALES, consisting of anaerobic cocci which utilize peptides, proteins or carbohydrates facultatively by sulfur respiration or fermentation. There are eight genera: AEROPYRUM, Desulfurococcus, Ignicoccus, Staphylothermus, Stetteria, Sulfophoboccus, Thermodiscus, and Thermosphaera. (From Bergey's Manual of Systematic Bacteriology, 2d ed)"),
  'D019621': NtO(MH='Haloferax', MS='A genus of extremely halophilic HALOBACTERIACEAE which are chemoheterotropic and strictly aerobic. They are found in neutral saline environments such as salt lakes (especially the Dead Sea) and marine salterns.'),
  'D019622': NtO(MH='Haloferax volcanii', MS='A species of halophilic archaea found in the Dead Sea.'),
  'D019623': NtO(MH='Haloferax mediterranei', MS='A species of halophilic archaea found in the Mediterranean Sea. It produces bacteriocins active against a range of other halobacteria.'),
  'D019624': NtO(MH='Thermoproteaceae', MS="A family of THERMOPROTEALES consisting of variable length rigid rods without septa. They grow either chemolithoautotrophically or by sulfur respiration. The four genera are: PYROBACULUM; THERMOPROTEUS; Caldivirga; and Thermocladium. (From Bergey's Manual of Systematic Bacteriology, 2d ed)"),
  'D019625': NtO(MH='Halococcus', MS='A genus of extremely halophilic HALOBACTERIACEAE with coccoid cells occurring singly or as pairs, tetrads, or irregular clusters. They are found in neutral salt lakes, marine salterns, saline soils and seawater.'),
  'D019633': NtO(MH='Natronobacterium', MS='A genus of rod-shaped, extremely halophilic HALOBACTERIACEAE which grows in alkaline conditions. They are strictly aerobic and some strains are motile. Natronobacterium is found in soda lakes, alkaline salterns, and soda soils.'),
  'D019634': NtO(MH='Natronococcus', MS='A genus of coccoid, extremely halophilic HALOBACTERIACEAE which grows in alkaline conditions. They are nonmotile and strictly aerobic and are found in soda lakes, alkaline salterns, and soda soils.'),
  'D019635': NtO(MH='Neurosurgical Procedures', MS='Surgery performed on the nervous system or its parts.'),
  'D019636': NtO(MH='Neurodegenerative Diseases', MS='Hereditary and sporadic conditions which are characterized by progressive nervous system dysfunction. These disorders are often associated with atrophy of the affected central or peripheral nervous system structures.'),
  'D019637': NtO(MH='Orthopedic Procedures', MS='Procedures used to treat and correct deformities, diseases, and injuries to the MUSCULOSKELETAL SYSTEM, its articulations, and associated structures.'),
  'D019638': NtO(MH='Methanomicrobiaceae', MS='A family of anaerobic METHANOMICROBIALES whose cells are coccoid to straight or slightly curved rods. There are six genera.'),
  'D019639': NtO(MH='Methanospirillum', MS='The sole genus in the family Methanospirillaceae whose organisms are progressively motile by means of polar, tufted flagella. They have been isolated from sewage-sludge and pear waste digesters as well as marine and non-marine habitats.'),
  'D019640': NtO(MH='Methanosarcinales', MS='An order of anaerobic methanogens in the kingdom EURYARCHAEOTA. There are two families: METHANOSARCINACEAE and Methanosaetaceae.'),
  'D019641': NtO(MH='DNA, Archaeal', MS='Deoxyribonucleic acid that makes up the genetic material of archaea.'),
  'D019642': NtO(MH='RNA, Archaeal', MS='Ribonucleic acid in archaea having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D019643': NtO(MH='Arthroplasty, Replacement', MS='Partial or total replacement of a skeletal joint.'),
  'D019644': NtO(MH='Arthroplasty, Replacement, Hip', MS='Replacement of the hip joint.'),
  'D019645': NtO(MH='Arthroplasty, Replacement, Knee', MS='Replacement of the knee joint.'),
  'D019646': NtO(MH='Decompression, Explosive', MS='A sudden loss of pressure in a pressurized cabin, cockpit, or the like, so rapid as to be explosive. (McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D019647': NtO(MH='Oral Surgical Procedures', MS='Surgical procedures used to treat disease, injuries, and defects of the oral and maxillofacial region.'),
  'D019648': NtO(MH='Foods, Specialized', MS='Foods and beverages prepared for use to meet specific needs such as infant foods.'),
  'D019649': NtO(MH='Food Industry', MS='The industry concerned with processing, preparing, preserving, distributing, and serving of foods and beverages.'),
  'D019650': NtO(MH='Hematopoietic Stem Cell Mobilization', MS='The release of stem cells from the bone marrow into the peripheral blood circulation for the purpose of leukapheresis, prior to stem cell transplantation. Hematopoietic growth factors or chemotherapeutic agents often are used to stimulate the mobilization.'),
  'D019651': NtO(MH='Reconstructive Surgical Procedures', MS='Procedures used to reconstruct, restore, or improve defective, damaged, or missing structures.'),
  'D019652': NtO(MH='Extraction and Processing Industry', MS="The industry concerned with the removal of raw materials from the Earth's crust and with their conversion into refined products."),
  'D019653': NtO(MH='Myeloablative Agonists', MS='Agents that destroy bone marrow activity. They are used to prepare patients for BONE MARROW TRANSPLANTATION or STEM CELL TRANSPLANTATION.'),
  'D019654': NtO(MH='Lens Implantation, Intraocular', MS='Insertion of an artificial lens to replace the natural CRYSTALLINE LENS after CATARACT EXTRACTION or to supplement the natural lens which is left in place.'),
  'D019655': NtO(MH='Quantitative Trait, Heritable', MS='A characteristic showing quantitative inheritance such as SKIN PIGMENTATION in humans. (From A Dictionary of Genetics, 4th ed)'),
  'D019656': NtO(MH='Loss of Heterozygosity', MS='The loss of one allele at a specific locus, caused by a deletion mutation; or loss of a chromosome from a chromosome pair, resulting in abnormal HEMIZYGOSITY. It is detected when heterozygous markers for a locus appear monomorphic because one of the ALLELES was deleted.'),
  'D019657': NtO(MH='Solanaceae', MS='A plant family of the order SOLANALES, class MAGNOLIOPSIDA. Among the most noted are POTATOES; TOMATOES; CAPSICUM (green and red peppers); TOBACCO; and BELLADONNA.'),
  'D019658': NtO(MH='Linaceae', MS='A plant family in the order Linales. These plants have simple leaves and regular flowers housing a compound ovary. Stamens are usually fused by their filaments.'),
  'D019659': NtO(MH='Asteraceae', MS='A large plant family of the order Asterales, subclass Asteridae, class Magnoliopsida. The family is also known as Compositae. Flower petals are joined near the base and stamens alternate with the corolla lobes. The common name of "daisy" refers to several genera of this family including Aster; CHRYSANTHEMUM; RUDBECKIA; TANACETUM.'),
  'D019660': NtO(MH='Malvaceae', MS='The mallow family of the order MALVALES, subclass Dilleniidae, class Magnoliopsida. The common names of hollyhock and mallow are used for several genera of Malvaceae.'),
  'D019661': NtO(MH='Apiaceae', MS='A large plant family in the order Apiales, also known as Umbelliferae. Most are aromatic herbs with alternate, feather-divided leaves that are sheathed at the base. The flowers often form a conspicuous flat-topped umbel. Each small individual flower is usually bisexual, with five sepals, five petals, and an enlarged disk at the base of the style. The fruits are ridged and are composed of two parts that split open at maturity.'),
  'D019662': NtO(MH='Cucurbitaceae', MS="The gourd plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida. It is sometimes placed in its own order, Cucurbitales. 'Melon' generally refers to CUCUMIS; CITRULLUS; or MOMORDICA."),
  'D019663': NtO(MH='Chenopodiaceae', MS='The goosefoot plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida. It includes beets and chard (BETA VULGARIS), as well as SPINACH, and salt tolerant plants.'),
  'D019664': NtO(MH='Polygonaceae', MS='The only family of the buckwheat order (Polygonales) of dicotyledonous flowering plants. It has 40 genera of herbs, shrubs, and trees.'),
  'D019665': NtO(MH='Euphorbiaceae', MS='The spurge family of flowering plants in the order Malpighiales. The family consists of annual and perennial herbs and woody shrubs or trees. Members contain securinine.'),
  'D019666': NtO(MH='Rubiaceae', MS='The Madder plant family of the order Gentianales (formerly Rubiales), subclass Asteridae, class Magnoliopsida includes important medicinal plants that provide QUININE; IPECAC; and COFFEE. They have opposite leaves and interpetiolar stipules.'),
  'D019668': NtO(MH='Liliaceae', MS='A monocot family within the order Liliales.'),
  'D019669': NtO(MH='Embryophyta', MS='Higher plants that live primarily in terrestrial habitats, although some are secondarily aquatic. Most obtain their energy from PHOTOSYNTHESIS. They comprise the vascular and non-vascular plants.'),
  'D019672': NtO(MH='Genes, T-Cell Receptor', MS='DNA sequences, in cells of the T-lymphocyte lineage, that code for T-cell receptors. The TcR genes are formed by somatic rearrangement (see GENE REARRANGEMENT, T-LYMPHOCYTE and its children) of germline gene segments, and resemble Ig genes in their mechanisms of diversity generation and expression.'),
  'D019673': NtO(MH='Genes, T-Cell Receptor alpha', MS='DNA sequences encoding the alpha chain of the T-cell receptor. The genomic organization of the TcR alpha genes is essentially the same in all species and is similar to the organization of Ig genes.'),
  'D019674': NtO(MH='Genes, T-Cell Receptor beta', MS='DNA sequences encoding the beta chain of the T-cell receptor. The genomic organization of the TcR beta genes is essentially the same in all species and is similar to the organization of Ig genes.'),
  'D019675': NtO(MH='Genes, T-Cell Receptor gamma', MS='DNA sequences encoding the gamma chain of the T-cell receptor. The human gamma-chain locus is organized similarly to the TcR beta-chain locus.'),
  'D019676': NtO(MH='Genes, T-Cell Receptor delta', MS='DNA sequences encoding the delta chain of the T-cell receptor. The delta-chain locus is located entirely within the alpha-chain locus.'),
  'D019677': NtO(MH='Artificial Gene Fusion', MS='The in vitro fusion of GENES by RECOMBINANT DNA techniques to analyze protein behavior or GENE EXPRESSION REGULATION, or to merge protein functions for specific medical or industrial uses.'),
  'D019679': NtO(MH='Kininogen, High-Molecular-Weight', MS='A plasma protein, molecular weight of 110 kD, that normally exists in plasma in a 1:1 complex with PREKALLIKREIN. HMWK is split by plasma kallikrein to produce BRADYKININ. The complex is a cofactor in the activation of coagulation factor XII. The product of this reaction, XIIa, in turn activates prekallikrein to KALLIKREINS. (From Stedman, 26th ed)'),
  'D019680': NtO(MH='Trinucleotide Repeat Expansion', MS='An increased number of contiguous trinucleotide repeats in the DNA sequence from one generation to the next. The presence of these regions is associated with diseases such as FRAGILE X SYNDROME and MYOTONIC DYSTROPHY. Some CHROMOSOME FRAGILE SITES are composed of sequences where trinucleotide repeat expansion occurs.'),
  'D019683': NtO(MH='Penetrance', MS='The percent frequency with which a dominant or homozygous recessive gene or gene combination manifests itself in the phenotype of the carriers. (From Glossary of Genetics, 5th ed)'),
  'D019684': NtO(MH='Magnoliopsida', MS='A class of vascular plants which produce flowers and seeds. They include monocotyledons, dicotyledons, and about 80% of all known plant species.'),
  'D019685': NtO(MH='Rosales', MS='An order of the class MAGNOLIOPSIDA. Its members include some of the most known ornamental and edible plants of temperate zones including roses, apples, cherries, and peaches.'),
  'D019686': NtO(MH='Lamiaceae', MS='The mint plant family. They are characteristically aromatic, and many of them are cultivated for their oils. Most have square stems, opposite leaves, and two-lipped, open-mouthed, tubular corollas (united petals), with five-lobed, bell-like calyxes (united sepals).'),
  'D019687': NtO(MH='Uterine Inversion', MS='A complication of OBSTETRIC LABOR in which the corpus of the UTERUS is forced completely or partially through the UTERINE CERVIX. This can occur during the late stages of labor and is associated with IMMEDIATE POSTPARTUM HEMORRHAGE.'),
  'D019688': NtO(MH='Gram-Negative Oxygenic Photosynthetic Bacteria', MS='Widely distributed unicellular or multicellular bacteria. The CYANOBACTERIA use chlorophyll a and phycobilins for oxygenic photosynthesis while genera in the Prochlorales use both chlorophyll a and b but not phycobilins.'),
  'D019690': NtO(MH='Technetium Tc 99m Exametazime', MS='A gamma-emitting RADIONUCLIDE IMAGING agent used in the evaluation of regional cerebral blood flow and in non-invasive dynamic biodistribution studies and MYOCARDIAL PERFUSION IMAGING. It has also been used to label leukocytes in the investigation of INFLAMMATORY BOWEL DISEASES.'),
  'D019691': NtO(MH='Prochloron', MS='A genus of PROCHLOROPHYTES containing unicellular, spherical bacteria without a mucilaginous sheath. They are found almost exclusively as extracellular symbionts of colonial ASCIDIANS on subtropical or tropical marine shores.'),
  'D019692': NtO(MH='Prochlorothrix', MS='A genus of PROCHLOROPHYTES occurring in unbranched chains of indefinite length and containing both chlorophylls a and b.'),
  'D019693': NtO(MH='Hepatitis, Autoimmune', MS='A chronic self-perpetuating hepatocellular INFLAMMATION of unknown cause, usually with HYPERGAMMAGLOBULINEMIA and serum AUTOANTIBODIES.'),
  'D019694': NtO(MH='Hepatitis B, Chronic', MS='INFLAMMATION of the LIVER in humans caused by HEPATITIS B VIRUS lasting six months or more. It is primarily transmitted by parenteral exposure, such as transfusion of contaminated blood or blood products, but can also be transmitted via sexual or intimate personal contact.'),
  'D019695': NtO(MH='Glycyrrhizic Acid', MS='A widely used anti-inflammatory agent isolated from the licorice root. It is metabolized to GLYCYRRHETINIC ACID, which inhibits 11-BETA-HYDROXYSTEROID DEHYDROGENASES and other enzymes involved in the metabolism of CORTICOSTEROIDS. Therefore, glycyrrhizic acid, which is the main and sweet component of licorice, has been investigated for its ability to cause hypermineralocorticoidism with sodium retention and potassium loss, edema, increased blood pressure, as well as depression of the renin-angiotensin-aldosterone system.'),
  'D019696': NtO(MH='Peptidylprolyl Isomerase', MS='An enzyme that catalyzes the isomerization of proline residues within proteins. EC 5.2.1.8.'),
  'D019697': NtO(MH='Onions', MS='Herbaceous biennial plants and their edible bulbs, belonging to the Allium genus.'),
  'D019698': NtO(MH='Hepatitis C, Chronic', MS='INFLAMMATION of the LIVER in humans that is caused by HEPATITIS C VIRUS lasting six months or more. Chronic hepatitis C can lead to LIVER CIRRHOSIS.'),
  'D019699': NtO(MH='Thrombospondins', MS='A family of related, adhesive glycoproteins which are synthesized, secreted, and incorporated into the extracellular matrix of a variety of cells, including alpha granules of platelets following thrombin activation and endothelial cells. They interact with a number of BLOOD COAGULATION FACTORS and anticoagulant factors. Five distinct forms have been identified, thrombospondin 1, -2, -3, -4, and cartilage oligomeric matrix protein (COMP). They are involved in cell adhesion, platelet aggregation, cell proliferation, angiogenesis, tumor metastasis, VASCULAR SMOOTH MUSCLE growth, and tissue repair.'),
  'D019700': NtO(MH='Thrombospondin 1', MS='An extracellular matrix glycoprotein from platelets and a variety of normal and transformed cells of both mesenchymal and epithelial origin. Thrombospondin-1 is believed to play a role in cell migration and proliferation, during embryogenesis and wound repair. Also, it has been studied for its use as a potential regulator of tumor growth and metastasis.'),
  'D019701': NtO(MH='Hepatitis D, Chronic', MS='INFLAMMATION of the LIVER in humans caused by HEPATITIS DELTA VIRUS in conjunction with HEPATITIS B VIRUS and lasting six months or more.'),
  'D019703': NtO(MH='Calcineurin', MS='A CALCIUM and CALMODULIN-dependent serine/threonine protein phosphatase that is composed of the calcineurin A catalytic subunit and the calcineurin B regulatory subunit. Calcineurin has been shown to dephosphorylate a number of phosphoproteins including HISTONES; MYOSIN LIGHT CHAIN; and the regulatory subunits of CAMP-DEPENDENT PROTEIN KINASES. It is involved in the regulation of signal transduction and is the target of an important class of immunophilin-immunosuppressive drug complexes.'),
  'D019704': NtO(MH='Protein Disulfide-Isomerases', MS='Sulfur-sulfur bond isomerases that catalyze the rearrangement of disulfide bonds within proteins during folding. Specific protein disulfide-isomerase isoenzymes also occur as subunits of PROCOLLAGEN-PROLINE DIOXYGENASE.'),
  'D019706': NtO(MH='Excitatory Postsynaptic Potentials', MS='Depolarization of membrane potentials at the SYNAPTIC MEMBRANES of target neurons during neurotransmission. Excitatory postsynaptic potentials can singly or in summation reach the trigger threshold for ACTION POTENTIALS.'),
  'D019707': NtO(MH='Receptors, Chemokine', MS='Cell surface glycoproteins that bind to chemokines and thus mediate the migration of pro-inflammatory molecules. The receptors are members of the seven-transmembrane G protein-coupled receptor family. Like the CHEMOKINES themselves, the receptors can be divided into at least three structural branches: CR, CCR, and CXCR, according to variations in a shared cysteine motif.'),
  'D019708': NtO(MH='Korarchaeota', MS='A kingdom in the domain ARCHAEA, comprising thermophilic organisms from terrestrial hot springs that are among the most primitive of all life forms. They have undergone comparatively little evolutionary change since the last common ancestor of all extant life.'),
  'D019709': NtO(MH='Thermococcales', MS='An order of strictly anaerobic, thermophilic archaea, in the kingdom EURYARCHAEOTA. Members exhibit heterotropic growth by sulfur respiration. There is a single family THERMOCOCCACEAE.'),
  'D019710': NtO(MH='Thermococcaceae', MS='A family of anaerobic THERMOCOCCALES found in hot environments. There are two genera: PYROCOCCUS and THERMOCOCCUS.'),
  'D019711': NtO(MH='Pyrococcus', MS='A genus of strictly anaerobic ultrathermophilic archaea, in the family THERMOCOCCACEAE, occurring in heated seawaters. They exhibit heterotrophic growth at an optimum temperature of 100 degrees C.'),
  'D019712': NtO(MH='Thermococcus', MS='A genus of extremely thermophilic heterotrophic archaea, in the family THERMOCOCCACEAE, occurring in heated sea flows. They are anaerobic chemoorganotropic sulfidogens.'),
  'D019713': NtO(MH='Receptors, CCR5', MS='CCR receptors with specificity for CHEMOKINE CCL3; CHEMOKINE CCL4; and CHEMOKINE CCL5. They are expressed at high levels in T-LYMPHOCYTES; B-LYMPHOCYTES; MACROPHAGES; MAST CELLS; and NK CELLS. The CCR5 receptor is used by the HUMAN IMMUNODEFICIENCY VIRUS to infect cells.'),
  'D019714': NtO(MH='Tissue Inhibitor of Metalloproteinases', MS='A family of secreted protease inhibitory proteins that regulates the activity of SECRETED MATRIX METALLOENDOPEPTIDASES. They play an important role in modulating the proteolysis of EXTRACELLULAR MATRIX, most notably during tissue remodeling and inflammatory processes.'),
  'D019715': NtO(MH='Tissue Inhibitor of Metalloproteinase-1', MS='A member of the family of TISSUE INHIBITOR OF METALLOPROTEINASES. It is a N-glycosylated protein, molecular weight 28 kD, produced by a vast range of cell types and found in a variety of tissues and body fluids. It has been shown to suppress metastasis and inhibit tumor invasion in vitro.'),
  'D019716': NtO(MH='Tissue Inhibitor of Metalloproteinase-2', MS='A member of the family of TISSUE INHIBITOR OF METALLOPROTEINASES. It is a 21-kDa nonglycosylated protein found in tissue fluid and is secreted as a complex with progelatinase A by human fibroblast and uncomplexed from alveolar macrophages. An overexpression of TIMP-2 has been shown to inhibit invasive and metastatic activity of tumor cells and decrease tumor growth in vivo.'),
  'D019717': NtO(MH='Tissue Inhibitor of Metalloproteinase-3', MS='A member of the family of tissue inhibitor of metalloproteinases. Mutations of the gene for TIMP3 PROTEIN causes Sorsby fundus dystrophy.'),
  'D019718': NtO(MH='Receptors, CXCR4', MS='CXCR receptors with specificity for CXCL12 CHEMOKINE. The receptors may play a role in HEMATOPOIESIS regulation and can also function as coreceptors for the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D019719': NtO(MH='Diagnostic Equipment', MS='Nonexpendable items used in examination.'),
  'D019720': NtO(MH='Otoscopes', MS='Instruments designed to inspect or auscultate the ear. They are designed primarily to examine the outer ear canal and tympanic membrane by means of light and air under moderate pressure, as with a pneumatic otoscope. (UMDNS, 1999)'),
  'D019721': NtO(MH='Ophthalmoscopes', MS='Devices for examining the interior of the eye, permitting the clear visualization of the structures of the eye at any depth. (UMDNS, 1999)'),
  'D019722': NtO(MH='Sphygmomanometers', MS='Instruments for measuring arterial blood pressure consisting of an inflatable cuff, inflating bulb, and a gauge showing the blood pressure. (Stedman, 26th ed)'),
  'D019723': NtO(MH='Endoscopes', MS='Instruments for the visual examination of interior structures of the body. There are rigid endoscopes and flexible fiberoptic endoscopes for various types of viewing in ENDOSCOPY.'),
  'D019724': NtO(MH='Presumed Consent', MS='An institutional policy of granting authority to health personnel to perform procedures on patients or to remove organs from cadavers for transplantation unless an objection is registered by family members or by the patient prior to death. This also includes emergency care of minors without prior parental consent.'),
  'D019726': NtO(MH='Patient Self-Determination Act', MS="The purpose of this 1990 federal act is to assure that individuals receiving health care services will be given an opportunity to participate in and direct health care decisions affecting themselves. Under this act, hospitals, health care agencies, and health maintenance organizations are responsible for developing patient information for distribution. The information must include patients' rights, advance directives, living wills, ethics committees' consultation and education functions, limited medical treatment (support/comfort care only), mental health treatment, resuscitation, restraints, surrogate decision making and transfer of care. (from JCAHO, Lexicon, 1994)"),
  'D019727': NtO(MH='Proxy', MS='A person authorized to decide or act for another person, for example, a person having durable power of attorney.'),
  'D019728': NtO(MH='Kininogen, Low-Molecular-Weight', MS='A protein, molecular weight 50 kD, located in various normal tissues. Upon cleavage by KALLIKREINS, it forms KALLIDIN. Kallidin, in turn, is converted into BRADYKININ. (From Stedman, 25th ed)'),
  'D019729': NtO(MH='Carbon-Oxygen Ligases', MS='Enzymes that catalyze the joining of two molecules by the formation of a carbon-oxygen bond. EC 6.1.'),
  'D019730': NtO(MH='Carbon-Sulfur Ligases', MS='Enzymes that catalyze the joining of two molecules by the formation of a carbon-sulfur bond. EC 6.2.'),
  'D019731': NtO(MH='Carbon-Nitrogen Ligases', MS='Enzymes that catalyze the joining of two molecules by the formation of a carbon-nitrogen bond. EC 6.3.'),
  'D019732': NtO(MH='Amide Synthases', MS='Enzymes that catalyze the joining of either ammonia or an amide with another molecule, in which the linkage is in the form of a carbon-nitrogen bond. EC 6.3.1.'),
  'D019733': NtO(MH='Carbon-Nitrogen Ligases with Glutamine as Amide-N-Donor', MS='Enzymes that catalyze the joining of glutamine-derived ammonia and another molecule. The linkage is in the form of a carbon-nitrogen bond. EC 6.3.5.'),
  'D019735': NtO(MH='Carbon-Carbon Ligases', MS='Enzymes that catalyze the joining of two molecules by the formation of a carbon-carbon bond. These are the carboxylating enzymes and are mostly biotinyl-proteins. EC 6.4.'),
  'D019736': NtO(MH='Prostheses and Implants', MS='Artificial substitutes for body parts, and materials inserted into tissue for functional, cosmetic, or therapeutic purposes. Prostheses can be functional, as in the case of artificial arms and legs, or cosmetic, as in the case of an artificial eye. Implants, all surgically inserted or grafted into the body, tend to be used therapeutically. IMPLANTS, EXPERIMENTAL is available for those used experimentally.'),
  'D019737': NtO(MH='Transplants', MS='Organs, tissues, or cells taken from the body for grafting into another area of the same body or into another individual.'),
  'D019738': NtO(MH='Surgically-Created Structures', MS='Organs or parts of organs surgically formed from nearby tissue to function as substitutes for diseased or surgically removed tissue.'),
  'D019739': NtO(MH='Skeletal Muscle Ventricle', MS='Autologous skeletal muscle that is wrapped around the heart and electrically stimulated in order to provide mechanical heart assistance. The latissimus dorsi muscle is commonly used to form this ventricle that functions to independently augment cardiac performance by pumping in series with the heart.'),
  'D019741': NtO(MH='Haemophilus influenzae type b', MS='A type of H. influenzae isolated most frequently from biotype I. Prior to vaccine availability, it was a leading cause of childhood meningitis.'),
  'D019742': NtO(MH='Chemokines, CC', MS='Group of chemokines with adjacent cysteines that are chemoattractants for lymphocytes, monocytes, eosinophils, basophils but not neutrophils.'),
  'D019743': NtO(MH='Chemokines, CXC', MS='Group of chemokines with paired cysteines separated by a different amino acid. CXC chemokines are chemoattractants for neutrophils but not monocytes.'),
  'D019744': NtO(MH='Chemokines, C', MS='Group of chemokines without adjacent cysteines that are chemoattractants for lymphocytes only.'),
  'D019745': NtO(MH='cis-trans-Isomerases', MS='Enzymes that catalyze the rearrangement of geometry about double bonds. EC 5.2.'),
  'D019746': NtO(MH='Intramolecular Oxidoreductases', MS='Enzymes of the isomerase class that catalyze the oxidation of one part of a molecule with a corresponding reduction of another part of the same molecule. They include enzymes converting aldoses to ketoses (ALDOSE-KETOSE ISOMERASES), enzymes shifting a carbon-carbon double bond (CARBON-CARBON DOUBLE BOND ISOMERASES), and enzymes transposing S-S bonds (SULFUR-SULFUR BOND ISOMERASES). (From Enzyme Nomenclature, 1992) EC 5.3.'),
  'D019747': NtO(MH='Aldose-Ketose Isomerases', MS='Enzymes that catalyze the interconversion of aldose and ketose compounds.'),
  'D019748': NtO(MH='Carbon-Carbon Double Bond Isomerases', MS='Enzymes that catalyze the shifting of a carbon-carbon double bond from one position to another within the same molecule. EC 5.3.3.'),
  'D019749': NtO(MH='Sulfur-Sulfur Bond Isomerases', MS='Enzymes that catalyze the transposition of a sulfur-sulfur bond. EC 5.3.4.'),
  'D019751': NtO(MH='Intramolecular Transferases', MS='Enzymes of the isomerase class that catalyze the transfer of acyl-, phospho-, amino- or other groups from one position within a molecule to another. EC 5.4.'),
  'D019753': NtO(MH='Intramolecular Lyases', MS='Enzymes of the isomerase class that catalyze reactions in which a group can be regarded as eliminated from one part of a molecule, leaving a double bond, while remaining covalently attached to the molecule. (From Enzyme Nomenclature, 1992) EC 5.5.'),
  'D019755': NtO(MH='Carbon-Carbon Lyases', MS='Enzymes that catalyze the cleavage of a carbon-carbon bond by means other than hydrolysis or oxidation. This subclass contains the DECARBOXYLASES, the ALDEHYDE-LYASES, and the OXO-ACID-LYASES. EC 4.1.'),
  'D019757': NtO(MH='Carbon-Oxygen Lyases', MS='Enzymes that catalyze the cleavage of a carbon-oxygen bond by means other than hydrolysis or oxidation. EC 4.2.'),
  'D019759': NtO(MH='Carbon-Nitrogen Lyases', MS='Enzymes that catalyze the cleavage of a carbon-nitrogen bond by means other than hydrolysis or oxidation. Subclasses are the AMMONIA-LYASES, the AMIDINE-LYASES, the amine-lyases, and other carbon-nitrogen lyases. EC 4.3.'),
  'D019760': NtO(MH='Amidine-Lyases', MS='These enzymes catalyze the elimination of ammonia from amidines with the formation of a double bond. EC 4.3.2.'),
  'D019761': NtO(MH='Phosphorus-Oxygen Lyases', MS='Enzymes that catalyze the cleavage of a phosphorus-oxygen bond by means other than hydrolysis or oxidation. EC 4.6.'),
  'D019763': NtO(MH='Heparin Lyase', MS='An enzyme of the isomerase class that catalyzes the eliminative cleavage of polysaccharides containing 1,4-linked D-glucuronate or L-iduronate residues and 1,4-alpha-linked 2-sulfoamino-2-deoxy-6-sulfo-D-glucose residues to give oligosaccharides with terminal 4-deoxy-alpha-D-gluc-4-enuronosyl groups at their non-reducing ends. (From Enzyme Nomenclature, 1992) EC 4.2.2.7.'),
  'D019764': NtO(MH='Phosphoenolpyruvate Carboxykinase (ATP)', MS='An enzyme of the lyase class that catalyzes the conversion of ATP and oxaloacetate to ADP, phosphoenolpyruvate, and carbon dioxide. The enzyme is found in some bacteria, yeast, and Trypanosoma, and is important for the photosynthetic assimilation of carbon dioxide in some plants. EC 4.1.1.49.'),
  'D019765': NtO(MH='Chondroitin ABC Lyase', MS='An enzyme that catalyzes the eliminative degradation of polysaccharides containing 1,4-beta-D-hexosaminyl and 1,3-beta-D-glucuronosyl or 1,3-alpha-L-iduronosyl linkages to disaccharides containing 4-deoxy-beta-D-gluc-4-enuronosyl groups. (Enzyme Nomenclature, 1992)'),
  'D019766': NtO(MH='Genes, Archaeal', MS='The functional genetic units of ARCHAEA.'),
  'D019767': NtO(MH='Maxillofacial Abnormalities', MS='Congenital structural deformities, malformations, or other abnormalities of the maxilla and face or facial bones.'),
  'D019770': NtO(MH='Lauraceae', MS='A family of mainly aromatic evergreen plants in the order Laurales. The laurel family includes 2,200 species in 45 genera and from these are derived medicinal extracts, essential oils, camphor and other products.'),
  'D019771': NtO(MH='Axotomy', MS='Transection or severing of an axon. This type of denervation is used often in experimental studies on neuronal physiology and neuronal death or survival, toward an understanding of nervous system disease.'),
  'D019772': NtO(MH='Topotecan', MS='An antineoplastic agent used to treat ovarian cancer. It works by inhibiting DNA TOPOISOMERASES, TYPE I.'),
  'D019773': NtO(MH='Epiretinal Membrane', MS='A membrane on the vitreal surface of the retina resulting from the proliferation of one or more of three retinal elements: (1) fibrous astrocytes; (2) fibrocytes; and (3) retinal pigment epithelial cells. Localized epiretinal membranes may occur at the posterior pole of the eye without clinical signs or may cause marked loss of vision as a result of covering, distorting, or detaching the fovea centralis. Epiretinal membranes may cause vascular leakage and secondary retinal edema. In younger individuals some membranes appear to be developmental in origin and occur in otherwise normal eyes. The majority occur in association with retinal holes, ocular concussions, retinal inflammation, or after ocular surgery. (Newell, Ophthalmology: Principles and Concepts, 7th ed, p291)'),
  'D019774': NtO(MH='Blood Coagulation Factor Inhibitors', MS='Substances, usually endogenous, that act as inhibitors of blood coagulation. They may affect one or multiple enzymes throughout the process. As a group, they also inhibit enzymes involved in processes other than blood coagulation, such as those from the complement system, fibrinolytic enzyme system, blood cells, and bacteria.'),
  'D019779': NtO(MH='Salmonella enterica', MS='A subgenus of Salmonella containing several medically important serotypes. The habitat for the majority of strains is warm-blooded animals.'),
  'D019780': NtO(MH='Technetium Tc 99m Lidofenin', MS='A nontoxic radiopharmaceutical that is used in RADIONUCLIDE IMAGING for the clinical evaluation of hepatobiliary disorders in humans.'),
  'D019781': NtO(MH='Corneal Topography', MS='The measurement of curvature and shape of the anterior surface of the cornea using techniques such as keratometry, keratoscopy, photokeratoscopy, profile photography, computer-assisted image processing and videokeratography. This measurement is often applied in the fitting of contact lenses and in diagnosing corneal diseases or corneal changes including keratoconus, which occur after keratotomy and keratoplasty.'),
  'D019782': NtO(MH='Riluzole', MS='A glutamate antagonist (RECEPTORS, GLUTAMATE) used as an anticonvulsant (ANTICONVULSANTS) and to prolong the survival of patients with AMYOTROPHIC LATERAL SCLEROSIS.'),
  'D019783': NtO(MH='Technetium Tc 99m Dimercaptosuccinic Acid', MS='A nontoxic radiopharmaceutical that is used in the diagnostic imaging of the renal cortex.'),
  'D019784': NtO(MH='Technetium Tc 99m Diethyl-iminodiacetic Acid', MS='A nontoxic radiopharmaceutical that is used in the clinical evaluation of hepatobiliary disorders in humans.'),
  'D019785': NtO(MH='Technetium Tc 99m Disofenin', MS='A radiopharmaceutical used extensively in cholescintigraphy for the evaluation of hepatobiliary diseases. (From Int Jrnl Rad Appl Inst 1992;43(9):1061-4)'),
  'D019786': NtO(MH='Gadolinium DTPA', MS='A complex of gadolinium with a chelating agent, diethylenetriamine penta-acetic acid (DTPA see PENTETIC ACID), that is given to enhance the image in cranial and spinal MRIs. (From Martindale, The Extra Pharmacopoeia, 30th ed, p706)'),
  'D019787': NtO(MH='Linoleic Acid', MS='A doubly unsaturated fatty acid, occurring widely in plant glycosides. It is an essential fatty acid in mammalian nutrition and is used in the biosynthesis of prostaglandins and cell membranes. (From Stedman, 26th ed)'),
  'D019788': NtO(MH='Fluorodeoxyglucose F18', MS='The compound is given by intravenous injection to do POSITRON-EMISSION TOMOGRAPHY for the assessment of cerebral and myocardial glucose metabolism in various physiological or pathological states including stroke and myocardial ischemia. It is also employed for the detection of malignant tumors including those of the brain, liver, and thyroid gland. (From Martindale, The Extra Pharmacopoeia, 30th ed, p1162)'),
  'D019789': NtO(MH='Tetraethylammonium', MS='A potassium-selective ion channel blocker. (From J Gen Phys 1994;104(1):173-90)'),
  'D019791': NtO(MH='Guanidine', MS='A strong organic base existing primarily as guanidium ions at physiological pH. It is found in the urine as a normal product of protein metabolism. It is also used in laboratory research as a protein denaturant. (From Martindale, the Extra Pharmacopoeia, 30th ed and Merck Index, 12th ed) It is also used in the treatment of myasthenia and as a fluorescent probe in HPLC.'),
  'D019793': NtO(MH='Fluorescein', MS='A phthalic indicator dye that appears yellow-green in normal tear film and bright green in a more alkaline medium such as the aqueous humor.'),
  'D019794': NtO(MH='2,3-Diphosphoglycerate', MS='A highly anionic organic phosphate which is present in human red blood cells at about the same molar ratio as hemoglobin. It binds to deoxyhemoglobin but not the oxygenated form, therefore diminishing the oxygen affinity of hemoglobin. This is essential in enabling hemoglobin to unload oxygen in tissue capillaries. It is also an intermediate in the conversion of 3-phosphoglycerate to 2-phosphoglycerate by phosphoglycerate mutase (EC 5.4.2.1). (From Stryer Biochemistry, 4th ed, p160; Enzyme Nomenclature, 1992, p508)'),
  'D019796': NtO(MH='15-Hydroxy-11 alpha,9 alpha-(epoxymethano)prosta-5,13-dienoic Acid', MS='A stable prostaglandin endoperoxide analog which serves as a thromboxane mimetic. Its actions include mimicking the hydro-osmotic effect of VASOPRESSIN and activation of TYPE C PHOSPHOLIPASES. (From J Pharmacol Exp Ther 1983;224(1): 108-117; Biochem J 1984;222(1):103-110)'),
  'D019797': NtO(MH='3-Iodobenzylguanidine', MS='A guanidine analog with specific affinity for tissues of the sympathetic nervous system and related tumors. The radiolabeled forms are used as antineoplastic agents and radioactive imaging agents. (Merck Index, 12th ed) MIBG serves as a neuron-blocking agent which has a strong affinity for, and retention in, the adrenal medulla and also inhibits ADP-ribosyltransferase.'),
  'D019798': NtO(MH='Neopterin', MS='A pteridine derivative present in body fluids; elevated levels result from immune system activation, malignant disease, allograft rejection, and viral infections. (From Stedman, 26th ed) Neopterin also serves as a precursor in the biosynthesis of biopterin.'),
  'D019800': NtO(MH='Phenol', MS='An antiseptic and disinfectant aromatic alcohol.'),
  'D019802': NtO(MH='Succinic Acid', MS="A water-soluble, colorless crystal with an acid taste that is used as a chemical intermediate, in medicine, the manufacture of lacquers, and to make perfume esters. It is also used in foods as a sequestrant, buffer, and a neutralizing agent. (Hawley's Condensed Chemical Dictionary, 12th ed, p1099; McGraw-Hill Dictionary of Scientific and Technical Terms, 4th ed, p1851)"),
  'D019803': NtO(MH='Glutathione Disulfide', MS='A GLUTATHIONE dimer formed by a disulfide bond between the cysteine sulfhydryl side chains during the course of being oxidized.'),
  'D019804': NtO(MH='Mesalamine', MS='An anti-inflammatory agent, structurally related to the SALICYLATES, which is active in INFLAMMATORY BOWEL DISEASE. It is considered to be the active moiety of SULPHASALAZINE. (From Martindale, The Extra Pharmacopoeia, 30th ed)'),
  'D019805': NtO(MH='alpha-Methyltyrosine', MS='An inhibitor of the enzyme TYROSINE 3-MONOOXYGENASE, and consequently of the synthesis of catecholamines. It is used to control the symptoms of excessive sympathetic stimulation in patients with PHEOCHROMOCYTOMA. (Martindale, The Extra Pharmacopoeia, 30th ed)'),
  'D019806': NtO(MH='Cromakalim', MS='A potassium-channel opening vasodilator that has been investigated in the management of hypertension. It has also been tried in patients with asthma. (Martindale, The Extra Pharmacopoeia, 30th ed, p352)'),
  'D019807': NtO(MH='Iodoacetic Acid', MS='A derivative of ACETIC ACID that contains one IODINE atom attached to its methyl group.'),
  'D019808': NtO(MH='Losartan', MS='An antagonist of ANGIOTENSIN TYPE 1 RECEPTOR with antihypertensive activity due to the reduced pressor effect of ANGIOTENSIN II.'),
  'D019809': NtO(MH='Streptavidin', MS='A 60-kDa extracellular protein of Streptomyces avidinii with four high-affinity biotin binding sites. Unlike AVIDIN, streptavidin has a near neutral isoelectric point and is free of carbohydrate side chains.'),
  'D019810': NtO(MH='Sodium Azide', MS='A cytochrome oxidase inhibitor which is a nitridizing agent and an inhibitor of terminal oxidation. (From Merck Index, 12th ed)'),
  'D019811': NtO(MH='Hydroxylamine', MS='A colorless inorganic compound (HONH2) used in organic synthesis and as a reducing agent, due to its ability to donate nitric oxide.'),
  'D019812': NtO(MH='Heparan Sulfate Proteoglycans', MS='Ubiquitous macromolecules associated with the cell surface and extracellular matrix of a wide range of cells of vertebrate and invertebrate tissues. They are essential cofactors in cell-matrix adhesion processes, in cell-cell recognition systems, and in receptor-growth factor interactions. (From Cancer Metastasis Rev 1996; 15(2): 177-86; Hepatology 1996; 24(3): 524-32)'),
  'D019813': NtO(MH='1,2-Dimethylhydrazine', MS='A DNA alkylating agent that has been shown to be a potent carcinogen and is widely used to induce colon tumors in experimental animals.'),
  'D019814': NtO(MH='Myristic Acid', MS='A saturated 14-carbon fatty acid occurring in most animal and vegetable fats, particularly butterfat and coconut, palm, and nutmeg oils. It is used to synthesize flavor and as an ingredient in soaps and cosmetics. (From Dorland, 28th ed)'),
  'D019815': NtO(MH='Oxalic Acid', MS='A strong dicarboxylic acid occurring in many plants and vegetables. It is produced in the body by metabolism of glyoxylic acid or ascorbic acid. It is not metabolized but excreted in the urine. It is used as an analytical reagent and general reducing agent.'),
  'D019817': NtO(MH='Benzoic Acid', MS='A fungistatic compound that is widely used as a food preservative. It is conjugated to GLYCINE in the liver and excreted as hippuric acid.'),
  'D019818': NtO(MH='Clavulanic Acid', MS='A beta-lactam antibiotic produced by the actinobacterium Streptomyces clavuligerus. It is a suicide inhibitor of bacterial beta-lactamase enzymes. Administered alone, it has only weak antibacterial activity against most organisms, but given in combination with other beta-lactam antibiotics it prevents antibiotic inactivation by microbial lactamase.'),
  'D019819': NtO(MH='Budesonide', MS='A glucocorticoid used in the management of ASTHMA, the treatment of various skin disorders, and allergic RHINITIS.'),
  'D019820': NtO(MH='Xanthine', MS='A purine base found in most body tissues and fluids, certain plants, and some urinary calculi. It is an intermediate in the degradation of adenosine monophosphate to uric acid, being formed by oxidation of hypoxanthine. The methylated xanthine compounds caffeine, theobromine, and theophylline and their derivatives are used in medicine for their bronchodilator effects. (Dorland, 28th ed)'),
  'D019821': NtO(MH='Simvastatin', MS='A derivative of LOVASTATIN and potent competitive inhibitor of 3-hydroxy-3-methylglutaryl coenzyme A reductase (HYDROXYMETHYLGLUTARYL COA REDUCTASES), which is the rate-limiting enzyme in cholesterol biosynthesis. It may also interfere with steroid hormone production. Due to the induction of hepatic LDL RECEPTORS, it increases breakdown of LDL CHOLESTEROL.'),
  'D019824': NtO(MH='beta-MSH', MS='An 18-amino acid peptide that is the C-terminal fragment of gamma-lipotropin which is the N-terminal fragment of BETA-LIPOTROPIN. Beta-MSH is shown to regulate skin pigmentation, steroid production, and feeding.'),
  'D019825': NtO(MH='gamma-MSH', MS='An 11-amino acid peptide derived from the N-terminal fragment of pro-opiomelanocortin (POMC). Gamma-MSH stimulates adrenal steroidogenesis and has an regulatory role in the cardiovascular and the renal systems.'),
  'D019826': NtO(MH='Cholic Acid', MS='A major primary bile acid produced in the liver and usually conjugated with glycine or taurine. It facilitates fat absorption and cholesterol excretion.'),
  'D019829': NtO(MH='Nevirapine', MS='A potent, non-nucleoside reverse transcriptase inhibitor used in combination with nucleoside analogues for treatment of HIV INFECTIONS and AIDS.'),
  'D019830': NtO(MH="Adenosine-5'-(N-ethylcarboxamide)", MS='A stable adenosine A1 and A2 receptor agonist. Experimentally, it inhibits cAMP and cGMP phosphodiesterase activity.'),
  'D019831': NtO(MH='Sodium-Calcium Exchanger', MS='An electrogenic ion exchange protein that maintains a steady level of calcium by removing an amount of calcium equal to that which enters the cells. It is widely distributed in most excitable membranes, including the brain and heart.'),
  'D019832': NtO(MH='N-Methylscopolamine', MS='A muscarinic antagonist used to study binding characteristics of muscarinic cholinergic receptors.'),
  'D019833': NtO(MH='Genistein', MS='An isoflavonoid derived from soy products. It inhibits PROTEIN-TYROSINE KINASE and topoisomerase-II (DNA TOPOISOMERASES, TYPE II); activity and is used as an antineoplastic and antitumor agent. Experimentally, it has been shown to induce G2 PHASE arrest in human and murine cell lines and inhibits PROTEIN-TYROSINE KINASE.'),
  'D019835': NtO(MH='FMRFamide', MS='A molluscan neuroactive peptide which induces a fast excitatory depolarizing response due to direct activation of amiloride-sensitive SODIUM CHANNELS. (From Nature 1995; 378(6558): 730-3)'),
  'D019836': NtO(MH='Preimplantation Diagnosis', MS='Determination of the nature of a pathological condition or disease in the OVUM; ZYGOTE; or BLASTOCYST prior to implantation. CYTOGENETIC ANALYSIS is performed to determine the presence or absence of genetic disease.'),
  'D019837': NtO(MH='Ryanodine Receptor Calcium Release Channel', MS='A tetrameric calcium release channel in the SARCOPLASMIC RETICULUM membrane of SMOOTH MUSCLE CELLS, acting oppositely to SARCOPLASMIC RETICULUM CALCIUM-TRANSPORTING ATPASES. It is important in skeletal and cardiac excitation-contraction coupling and studied by using RYANODINE. Abnormalities are implicated in CARDIAC ARRHYTHMIAS and MUSCULAR DISEASES.'),
  'D019838': NtO(MH='Neck Injuries', MS='General or unspecified injuries to the neck. It includes injuries to the skin, muscles, and other soft tissues of the neck.'),
  'D019840': NtO(MH='2-Propanol', MS='An isomer of 1-PROPANOL. It is a colorless liquid having disinfectant properties. It is used in the manufacture of acetone and its derivatives and as a solvent. Topically, it is used as an antiseptic.'),
  'D019841': NtO(MH='Defecography', MS='Radiographic examination of the process of defecation after the instillation of a CONTRAST MEDIA into the rectum.'),
  'D019842': NtO(MH='Ciliary Arteries', MS='Three groups of arteries found in the eye which supply the iris, pupil, sclera, conjunctiva, and the muscles of the iris.'),
  'D019843': NtO(MH='Archaeal Proteins', MS='Proteins found in any species of archaeon.'),
  'D019844': NtO(MH='Antibodies, Archaeal', MS='Immunoglobulins produced in a response to ARCHAEAL ANTIGENS.'),
  'D019845': NtO(MH='Antigens, Archaeal', MS='Substances of archaeal origin that have antigenic activity.'),
  'D019846': NtO(MH='Miller Fisher Syndrome', MS='A variant of the GUILLAIN-BARRE SYNDROME characterized by the acute onset of oculomotor dysfunction, ataxia, and loss of deep tendon reflexes with relative sparing of strength in the extremities and trunk. The ataxia is produced by peripheral sensory nerve dysfunction and not by cerebellar injury. Facial weakness and sensory loss may also occur. The process is mediated by autoantibodies directed against a component of myelin found in peripheral nerves. (Adams et al., Principles of Neurology, 6th ed, p1313; Neurology 1987 Sep;37(9):1493-8)'),
  'D019847': NtO(MH='Chromosomes, Archaeal', MS='Structures within the nucleus of archaeal cells consisting of or containing DNA, which carry genetic information essential to the cell.'),
  'D019848': NtO(MH='Gene Expression Regulation, Archaeal', MS='Any of the processes by which cytoplasmic or intercellular factors influence the differential control of gene action in archaea.'),
  'D019849': NtO(MH='Sex Determination Processes', MS="The mechanisms by which the SEX of an individual's GONADS are fixed."),
  'D019850': NtO(MH='Heptanol', MS='A colorless liquid with a fragrant odor. It is used as an intermediate, solvent and in cosmetics.'),
  'D019851': NtO(MH='Thrombophilia', MS='A disorder of HEMOSTASIS in which there is a tendency for the occurrence of THROMBOSIS.'),
  'D019852': NtO(MH='Diacylglycerol Kinase', MS='An enzyme of the transferase class that uses ATP to catalyze the phosphorylation of diacylglycerol to a phosphatidate. EC 2.7.1.107.'),
  'D019853': NtO(MH='O(6)-Methylguanine-DNA Methyltransferase', MS='An enzyme that transfers methyl groups from O(6)-methylguanine, and other methylated moieties of DNA, to a cysteine residue in itself, thus repairing alkylated DNA in a single-step reaction. EC 2.1.1.63.'),
  'D019855': NtO(MH='Ethylene Glycol', MS='A colorless, odorless, viscous dihydroxy alcohol. It has a sweet taste, but is poisonous if ingested. Ethylene glycol is the most important glycol commercially available and is manufactured on a large scale in the United States. It is used as an antifreeze and coolant, in hydraulic fluids, and in the manufacture of low-freezing dynamites and resins.'),
  'D019856': NtO(MH='Ethanolamine', MS='A viscous, hygroscopic amino alcohol with an ammoniacal odor. It is widely distributed in biological tissue and is a component of lecithin. It is used as a surfactant, fluorimetric reagent, and to remove CO2 and H2S from natural gas and other gases.'),
  'D019857': NtO(MH='Osteogenesis, Distraction', MS='Bone lengthening by gradual mechanical distraction. An external fixation device produces the distraction across the bone plate. The technique was originally applied to long bones but in recent years the method has been adapted for use with mandibular implants in maxillofacial surgery.'),
  'D019858': NtO(MH='Enteroendocrine Cells', MS='Cells found throughout the lining of the GASTROINTESTINAL TRACT that contain and secrete regulatory PEPTIDE HORMONES and/or BIOGENIC AMINES.'),
  'D019859': NtO(MH='Proto-Oncogene Proteins c-met', MS='Cell surface protein-tyrosine kinase receptors for HEPATOCYTE GROWTH FACTOR. They consist of an extracellular alpha chain which is disulfide-linked to the transmembrane beta chain. The cytoplasmic portion contains the catalytic domain and sites critical for the regulation of kinase activity. Mutations in the c-met proto-oncogene are associated with papillary renal carcinoma and other neoplasia.'),
  'D019860': NtO(MH='Lymphocyte Specific Protein Tyrosine Kinase p56(lck)', MS='This enzyme is a lymphoid-specific src family tyrosine kinase that is critical for T-cell development and activation. Lck is associated with the cytoplasmic domains of CD4, CD8 and the beta-chain of the IL-2 receptor, and is thought to be involved in the earliest steps of TCR-mediated T-cell activation.'),
  'D019861': NtO(MH='Enterochromaffin-like Cells', MS='Neuroendocrine cells in the glands of the GASTRIC MUCOSA. They produce HISTAMINE and peptides such as CHROMOGRANINS. ECL cells respond to GASTRIN by releasing histamine which acts as a paracrine stimulator of the release of HYDROCHLORIC ACID from the GASTRIC PARIETAL CELLS.'),
  'D019862': NtO(MH='Zingiberales', MS='An order of herbaceous tropical perennials which include the banana (MUSACEAE) and GINGER (ZINGIBERACEAE) family.'),
  'D019863': NtO(MH='Gastrin-Secreting Cells', MS='Endocrine cells which secrete GASTRIN, a peptide that induces GASTRIC ACID secretion. They are found predominantly in the GASTRIC GLANDS of PYLORIC ANTRUM in the STOMACH, but can also be found in the DUODENUM, nervous and other tissues.'),
  'D019864': NtO(MH='Somatostatin-Secreting Cells', MS='Endocrine cells found throughout the GASTROINTESTINAL TRACT and in islets of the PANCREAS. D cells secrete SOMATOSTATIN that acts in both an endocrine and paracrine manner. Somatostatin acts on a variety of tissues including the PITUITARY GLAND; gastrointestinal tract; pancreas; and KIDNEY by inhibiting the release of hormones, such as GROWTH HORMONE; GASTRIN; INSULIN; and RENIN.'),
  'D019867': NtO(MH='Anti-Glomerular Basement Membrane Disease', MS='An autoimmune disease of the KIDNEY and the LUNG. It is characterized by the presence of circulating autoantibodies targeting the epitopes in the non-collagenous domains of COLLAGEN TYPE IV in the basement membranes of kidney glomeruli (KIDNEY GLOMERULUS) and lung alveoli (PULMONARY ALVEOLI), and the subsequent destruction of these basement membranes. Clinical features include pulmonary alveolar hemorrhage and glomerulonephritis.'),
  'D019868': NtO(MH='Kanamycin Kinase', MS="A class of enzymes that inactivate aminocyclitol-aminoglycoside antibiotics (AMINOGLYCOSIDES) by regiospecific PHOSPHORYLATION of the 3' and/or 5' hydroxyl."),
  'D019869': NtO(MH='Phosphatidylinositol 3-Kinases', MS='Phosphotransferases that catalyzes the conversion of 1-phosphatidylinositol to 1-phosphatidylinositol 3-phosphate. Many members of this enzyme class are involved in RECEPTOR MEDIATED SIGNAL TRANSDUCTION and regulation of vesicular transport with the cell. Phosphatidylinositol 3-Kinases have been classified both according to their substrate specificity and their mode of action within the cell.'),
  'D019870': NtO(MH='1-Phosphatidylinositol 4-Kinase', MS='An enzyme that catalyzes the conversion of phosphatidylinositol (PHOSPHATIDYLINOSITOLS) to phosphatidylinositol 4-phosphate, the first committed step in the biosynthesis of phosphatidylinositol 4,5-bisphosphate.'),
  'D019871': NtO(MH='Dyskeratosis Congenita', MS='A predominantly X-linked recessive syndrome characterized by a triad of reticular skin pigmentation, nail dystrophy and leukoplakia of mucous membranes. Oral and dental abnormalities may also be present. Complications are a predisposition to malignancy and bone marrow involvement with pancytopenia. (from Int J Paediatr Dent 2000 Dec;10(4):328-34) The X-linked form is also known as Zinsser-Cole-Engman syndrome and involves the gene which encodes a highly conserved protein called dyskerin.'),
  'D019872': NtO(MH='Chief Cells, Gastric', MS='Epithelial cells that line the basal half of the GASTRIC GLANDS. Chief cells synthesize and export an inactive enzyme PEPSINOGEN which is converted into the highly proteolytic enzyme PEPSIN in the acid environment of the STOMACH.'),
  'D019873': NtO(MH='Schnitzler Syndrome', MS='An extremely rare condition manifested as monoclonal IMMUNOGLOBULIN M dysproteinemia without features of lymphoproliferative disease, but with chronic urticaria, fever of unknown origin, disabling bone pain, hyperostosis, and increased erythrocyte sedimentation rate.'),
  'D019875': NtO(MH='One-Carbon Group Transferases', MS='A subclass of transferases that transfer chemical groups containing a single carbon. These include the METHYLTRANSFERASES, the HYDROXYMETHYL AND FORMYL TRANSFERASES, the CARBOXYL AND CARBAMOYL TRANSFERASES, and the AMIDINOTRANSFERASES. EC 2.1.'),
  'D019876': NtO(MH='Sports Equipment', MS='Equipment required for engaging in a sport (such as balls, bats, rackets, skis, skates, ropes, weights) and devices for the protection of athletes during their performance (such as masks, gloves, mouth pieces).'),
  'D019877': NtO(MH='Hydroxymethyl and Formyl Transferases', MS='Enzymes that catalyze the transfer of hydroxymethyl or formyl groups. EC 2.1.2.'),
  'D019878': NtO(MH='Carboxyl and Carbamoyl Transferases', MS='A group of enzymes that catalyze the transfer of carboxyl- or carbamoyl- groups. EC 2.1.3.'),
  'D019879': NtO(MH='Paneth Cells', MS='Differentiated epithelial cells of the INTESTINAL MUCOSA, found in the basal part of the intestinal crypts of Lieberkuhn. Paneth cells secrete GROWTH FACTORS, digestive enzymes such as LYSOZYME and antimicrobial peptides such as cryptdins (ALPHA-DEFENSINS) into the crypt lumen.'),
  'D019880': NtO(MH='Aldehyde-Ketone Transferases', MS='Enzymes that catalyze the transfer of aldehyde or ketone residues. EC 2.2.'),
  'D019881': NtO(MH='Aminoacyltransferases', MS='Enzymes that catalyze the transfer of an aminoacyl group from donor to acceptor resulting in the formation of an ester or amide linkage. EC 2.3.2.'),
  'D019882': NtO(MH='Blepharoplasty', MS='Plastic surgery of the eyelid. (Cline et al., Dictionary of Visual Science, 4th ed)'),
  'D019883': NtO(MH='Alkyl and Aryl Transferases', MS='A somewhat heterogeneous class of enzymes that catalyze the transfer of alkyl or related groups (excluding methyl groups). EC 2.5.'),
  'D019884': NtO(MH='Nitrogenous Group Transferases', MS='Enzymes that catalyze the transfer of nitrogenous groups, primarily amino groups, from a donor, generally an amino acid, to an acceptor, usually a 2-oxoacid. EC 2.6.'),
  'D019885': NtO(MH='Sulfur Group Transferases', MS='Enzymes that catalyze the transfer of sulfur atoms (2.8.1), sulfur groups (2.8.2) or coenzyme A (2.8.3). EC 2.8.'),
  'D019886': NtO(MH='Gastrin-Releasing Peptide', MS='Neuropeptide and gut hormone that helps regulate GASTRIC ACID secretion and motor function. Once released from nerves in the antrum of the STOMACH, the neuropeptide stimulates release of GASTRIN from the GASTRIN-SECRETING CELLS.'),
  'D019887': NtO(MH='Peanut Agglutinin', MS='Lectin purified from peanuts (ARACHIS HYPOGAEA). It binds to poorly differentiated cells and terminally differentiated cells and is used in cell separation techniques.'),
  'D019888': NtO(MH='Nelfinavir', MS='A potent HIV protease inhibitor. It is used in combination with other antiviral drugs in the treatment of HIV in both adults and children.'),
  'D019891': NtO(MH='Leukopoiesis', MS='The process of generating white blood cells (LEUKOCYTES) from the pluripotent HEMATOPOIETIC STEM CELLS of the BONE MARROW. There are two significant pathways to generate various types of leukocytes: MYELOPOIESIS, in which leukocytes in the blood are derived from MYELOID STEM CELLS, and LYMPHOPOIESIS, in which leukocytes of the lymphatic system (LYMPHOCYTES) are generated from lymphoid stem cells.'),
  'D019892': NtO(MH='eIF-2 Kinase', MS='A dsRNA-activated cAMP-independent protein serine/threonine kinase that is induced by interferon. In the presence of dsRNA and ATP, the kinase autophosphorylates on several serine and threonine residues. The phosphorylated enzyme catalyzes the phosphorylation of the alpha subunit of EUKARYOTIC INITIATION FACTOR-2, leading to the inhibition of protein synthesis.'),
  'D019893': NtO(MH='Ribosomal Protein S6 Kinases', MS="A family of protein serine/threonine kinases which act as intracellular signalling intermediates. Ribosomal protein S6 kinases are activated through phosphorylation in response to a variety of HORMONES and INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS. Phosphorylation of RIBOSOMAL PROTEIN S6 by enzymes in this class results in increased expression of 5' top MRNAs. Although specific for RIBOSOMAL PROTEIN S6 members of this class of kinases can act on a number of substrates within the cell. The immunosuppressant SIROLIMUS inhibits the activation of ribosomal protein S6 kinases."),
  'D019894': NtO(MH='Peptide YY', MS='A 36-amino acid peptide produced by the L cells of the distal small intestine and colon. Peptide YY inhibits gastric and pancreatic secretion.'),
  'D019895': NtO(MH='Transposases', MS='Enzymes that recombine DNA segments by a process which involves the formation of a synapse between two DNA helices, the cleavage of single strands from each DNA helix and the ligation of a DNA strand from one DNA helix to the other. The resulting DNA structure is called a Holliday junction which can be resolved by DNA REPLICATION or by HOLLIDAY JUNCTION RESOLVASES.'),
  'D019896': NtO(MH='alpha 1-Antitrypsin Deficiency', MS='Deficiency of the protease inhibitor ALPHA 1-ANTITRYPSIN that manifests primarily as PULMONARY EMPHYSEMA and LIVER CIRRHOSIS.'),
  'D019897': NtO(MH='Periplasm', MS='The space between the inner and outer membranes of a cell that is shared with the cell wall.'),
  'D019898': NtO(MH='Autocrine Communication', MS='Mode of communication wherein a bound hormone affects the function of the cell type that produced the hormone.'),
  'D019899': NtO(MH='Paracrine Communication', MS='Cellular signaling in which a factor secreted by a cell affects other cells in the local environment. This term is often used to denote the action of INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS on surrounding cells.'),
  'D019900': NtO(MH='3,4-Dichloro-N-methyl-N-(2-(1-pyrrolidinyl)-cyclohexyl)-benzeneacetamide, (trans)-Isomer', MS='A non-peptide, kappa-opioid receptor agonist which has also been found to stimulate the release of adrenocorticotropin (ADRENOCORTICOTROPIC HORMONE) via the release of hypothalamic arginine vasopressin (ARGININE VASOPRESSIN) and CORTICOTROPIN-RELEASING HORMONE. (From J Pharmacol Exp Ther 1997;280(1):416-21)'),
  'D019901': NtO(MH='Locus Control Region', MS='A regulatory region first identified in the human beta-globin locus but subsequently found in other loci. The region is believed to regulate GENETIC TRANSCRIPTION by opening and remodeling CHROMATIN structure. It may also have enhancer activity.'),
  'D019902': NtO(MH='Chondrocytes', MS='Polymorphic cells that form cartilage.'),
  'D019904': NtO(MH='Polymethyl Methacrylate', MS='Polymerized methyl methacrylate monomers which are used as sheets, moulding, extrusion powders, surface coating resins, emulsion polymers, fibers, inks, and films (From International Labor Organization, 1983). This material is also used in tooth implants, bone cements, and hard corneal contact lenses.'),
  'D019905': NtO(MH='Benzyl Alcohol', MS='A colorless liquid with a sharp burning taste and slight odor. It is used as a local anesthetic and to reduce pain associated with LIDOCAINE injection. Also, it is used in the manufacture of other benzyl compounds, as a pharmaceutic aid, and in perfumery and flavoring.'),
  'D019906': NtO(MH='Nuclear Magnetic Resonance, Biomolecular', MS='NMR spectroscopy on small- to medium-size biological macromolecules. This is often used for structural investigation of proteins and nucleic acids, and often involves more than one isotope.'),
  'D019907': NtO(MH='Mycobacterium fortuitum', MS='A rapid-growing, nonphotochromogenic species that is potentially pathogenic, producing lesions of lung, bone, or soft tissue following trauma. It has been found in soil and in injection sites of humans, cattle, and cold-blooded animals. (Dorland, 28th ed)'),
  'D019908': NtO(MH='Proto-Oncogene Proteins c-raf', MS='A ubiquitously expressed raf kinase subclass that plays an important role in SIGNAL TRANSDUCTION. The c-raf Kinases are MAP kinase kinase kinases that have specificity for MAP KINASE KINASE 1 and MAP KINASE KINASE 2.'),
  'D019909': NtO(MH='Mycobacterium kansasii', MS='A slow-growing, photochromogenic species that is the etiologic agent of a tuberculosis-like disease in humans and is frequently isolated from human pulmonary secretions or tubercles. The incidence of infection is sharply increased among immunocompromised individuals. (Dorland, 28th ed)'),
  'D019910': NtO(MH='Mycobacterium marinum', MS='A moderate-growing, photochromogenic species found in aquariums, diseased fish, and swimming pools. It is the cause of cutaneous lesions and granulomas (swimming pool granuloma) in humans. (Dorland, 28th ed)'),
  'D019911': NtO(MH='Mycobacterium ulcerans', MS='A slow-growing mycobacterium that infects the skin and subcutaneous tissues, giving rise to indolent BURULI ULCER.'),
  'D019912': NtO(MH='Mycobacterium xenopi', MS='A slow-growing, scotochromogenic species occurring usually harmlessly in human secretions but occasionally associated with chronic pulmonary disease. (Dorland, 28th ed)'),
  'D019913': NtO(MH='Nuclear Localization Signals', MS='Short, predominantly basic amino acid sequences identified as nuclear import signals for some proteins. These sequences are believed to interact with specific receptors at the NUCLEAR PORE.'),
  'D019914': NtO(MH='Taq Polymerase', MS='A heat stable DNA-DIRECTED DNA POLYMERASE from the bacteria Thermus aquaticus. It is widely used for the amplification of genes through the process of POLYMERASE CHAIN REACTION. EC 2.7.7.-.'),
  'D019915': NtO(MH='DNA Primase', MS='A single-stranded DNA-dependent RNA polymerase that functions to initiate, or prime, DNA synthesis by synthesizing oligoribonucleotide primers. EC 2.7.7.-.'),
  'D019916': NtO(MH='Choline-Phosphate Cytidylyltransferase', MS='An enzyme that catalyzes the transfer of cytidylate (CMP) to choline phosphate to form CDPcholine. It is the rate-limiting enzyme in the choline pathway for the biosynthesis of phosphatidylcholine. Its activity is increased by glucocorticoids. EC 2.7.7.15.'),
  'D019917': NtO(MH='Blood Vessel Prosthesis Implantation', MS='Surgical insertion of BLOOD VESSEL PROSTHESES to repair injured or diseased blood vessels.'),
  'D019918': NtO(MH='Heart Valve Prosthesis Implantation', MS='Surgical insertion of synthetic material to repair injured or diseased heart valves.'),
  'D019919': NtO(MH='Prosthesis Implantation', MS='Surgical insertion of a prosthesis.'),
  'D019920': NtO(MH='Photoaffinity Labels', MS='Biologically active molecules which are covalently bound to the enzymes or binding proteins normally acting on them. Binding occurs due to activation of the label by ultraviolet light. These labels are used primarily to identify binding sites on proteins.'),
  'D019921': NtO(MH='Biotinylation', MS='Incorporation of biotinyl groups into molecules.'),
  'D019922': NtO(MH='GAP-43 Protein', MS='A nervous tissue specific protein which is highly expressed in NEURONS during development and NERVE REGENERATION. It has been implicated in neurite outgrowth, long-term potentiation, SIGNAL TRANSDUCTION, and NEUROTRANSMITTER release. (From Neurotoxicology 1994;15(1):41-7) It is also a substrate of PROTEIN KINASE C.'),
  'D019923': NtO(MH='Capsulorhexis', MS='The making of a continuous circular tear in the anterior capsule during cataract surgery in order to allow expression or phacoemulsification of the nucleus of the lens. (Dorland, 28th ed)'),
  'D019924': NtO(MH='Embryo Disposition', MS='Utilization or disposal of an embryo that is fertilized but not immediately transplanted and resulting course of action.'),
  'D019925': NtO(MH='Cyclin A', MS='A cyclin subtype that has specificity for CDC2 PROTEIN KINASE and CYCLIN-DEPENDENT KINASE 2. It plays a role in progression of the CELL CYCLE through G1/S and G2/M phase transitions.'),
  'D019926': NtO(MH='Cyclin B', MS='A cyclin subtype that is transported into the CELL NUCLEUS at the end of the G2 PHASE. It stimulates the G2/M phase transition by activating CDC2 PROTEIN KINASE.'),
  'D019927': NtO(MH='Cyclin E', MS='A 50-kDa protein that complexes with CYCLIN-DEPENDENT KINASE 2 in the late G1 phase of the cell cycle.'),
  'D019928': NtO(MH='Breast Implantation', MS='Surgical insertion of an inert sac filled with silicone or other material to augment the female form cosmetically.'),
  'D019929': NtO(MH='Cochlear Implantation', MS='Surgical insertion of an electronic hearing device (COCHLEAR IMPLANTS) with electrodes to the COCHLEAR NERVE in the inner ear to create sound sensation in patients with residual nerve fibers.'),
  'D019930': NtO(MH='Mandibular Prosthesis Implantation', MS='Surgical insertion of an appliance for the replacement of areas of the mandible.'),
  'D019931': NtO(MH='Maxillofacial Prosthesis Implantation', MS='Surgical insertion of an appliance for the replacement of areas of the maxilla, mandible, and face. When only portions of the mandible are replaced, it is referred to as MANDIBULAR PROSTHESIS IMPLANTATION.'),
  'D019933': NtO(MH='Ossicular Replacement', MS='Surgical insertion of an implant to replace one or more of the ear ossicles.'),
  'D019934': NtO(MH='International Normalized Ratio', MS='System established by the World Health Organization and the International Committee on Thrombosis and Hemostasis for monitoring and reporting blood coagulation tests. Under this system, results are standardized using the International Sensitivity Index for the particular test reagent/instrument combination used.'),
  'D019935': NtO(MH='Penile Implantation', MS='Surgical insertion of cylindric hydraulic devices for the treatment of organic ERECTILE DYSFUNCTION.'),
  'D019936': NtO(MH='Probiotics', MS='Live microbial DIETARY SUPPLEMENTS which beneficially affect the host animal by improving its intestinal microbial balance. Antibiotics and other related compounds are not included in this definition. In humans, lactobacilli are commonly used as probiotics, either as single species or in mixed culture with other bacteria. Other genera that have been used are bifidobacteria and streptococci. (J. Nutr. 1995;125:1401-12)'),
  'D019937': NtO(MH='Diagnostic Techniques and Procedures', MS='Methods, procedures, and tests performed to diagnose disease, disordered function, or disability.'),
  'D019938': NtO(MH='Cyclin D1', MS='Protein encoded by the bcl-1 gene which plays a critical role in regulating the cell cycle. Overexpression of cyclin D1 is the result of bcl-1 rearrangement, a t(11;14) translocation, and is implicated in various neoplasms.'),
  'D019940': NtO(MH='Genes, bcl-1', MS='The B-cell leukemia/lymphoma-1 genes, associated with various neoplasms when overexpressed. Overexpression results from the t(11;14) translocation, which is characteristic of mantle zone-derived B-cell lymphomas. The human c-bcl-1 gene is located at 11q13 on the long arm of chromosome 11.'),
  'D019941': NtO(MH='Cyclin-Dependent Kinase Inhibitor p16', MS='A product of the p16 tumor suppressor gene (GENES, P16). It is also called INK4 or INK4A because it is the prototype member of the INK4 CYCLIN-DEPENDENT KINASE INHIBITORS. This protein is produced from the alpha mRNA transcript of the p16 gene. The other gene product, produced from the alternatively spliced beta transcript, is TUMOR SUPPRESSOR PROTEIN P14ARF. Both p16 gene products have tumor suppressor functions.'),
  'D019942': NtO(MH='Genes, p16', MS='Tumor suppressor genes located on human chromosome 9 in the region 9p21. This gene is either deleted or mutated in a wide range of malignancies. (From Segen, Current Med Talk, 1995) Two alternatively spliced gene products are encoded by p16: CYCLIN-DEPENDENT KINASE INHIBITOR P16 and TUMOR SUPPRESSOR PROTEIN P14ARF.'),
  'D019943': NtO(MH='Amino Acid Substitution', MS='The naturally occurring or experimentally induced replacement of one or more AMINO ACIDS in a protein with another. If a functionally equivalent amino acid is substituted, the protein may retain wild-type activity. Substitution may also diminish, enhance, or eliminate protein function. Experimentally induced substitution is often used to study enzyme activities and binding site properties.'),
  'D019946': NtO(MH='Propylene Glycol', MS='A clear, colorless, viscous organic solvent and diluent used in pharmaceutical preparations.'),
  'D019947': NtO(MH='Receptors, Interleukin-6', MS='Cell surface receptors that are specific for INTERLEUKIN-6. They are present on T-LYMPHOCYTES, mitogen-activated B-LYMPHOCYTES, and peripheral MONOCYTES. The receptors are heterodimers of the INTERLEUKIN-6 RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR GP130.'),
  'D019948': NtO(MH='Receptors, Interleukin-4', MS='Receptors present on a wide variety of hematopoietic and non-hematopoietic cell types that are specific for INTERLEUKIN-4. They are involved in signaling a variety of immunological responses related to allergic INFLAMMATION including the differentiation of TH2 CELLS and the regulation of IMMUNOGLOBULIN E production. Two subtypes of receptors exist and are referred to as the TYPE I INTERLEUKIN-4 RECEPTOR and the TYPE II INTERLEUKIN-4 RECEPTOR. Each receptor subtype is defined by its unique subunit composition.'),
  'D019950': NtO(MH='Mitogen-Activated Protein Kinase 1', MS='A proline-directed serine/threonine protein kinase which mediates signal transduction from the cell surface to the nucleus. Activation of the enzyme by phosphorylation leads to its translocation into the nucleus where it acts upon specific transcription factors. p40 MAPK and p41 MAPK are isoforms.'),
  'D019951': NtO(MH='DNA Polymerase beta', MS='A DNA repair enzyme that catalyzes DNA synthesis during base excision DNA repair. EC 2.7.7.7.'),
  'D019954': NtO(MH='Neurobehavioral Manifestations', MS='Signs and symptoms of higher cortical dysfunction caused by organic conditions. These include certain behavioral alterations and impairments of skills involved in the acquisition, processing, and utilization of knowledge or information.'),
  'D019955': NtO(MH='Conduct Disorder', MS='A repetitive and persistent pattern of behavior in which the basic rights of others or major age-appropriate societal norms or rules are violated. These behaviors include aggressive conduct that causes or threatens physical harm to other people or animals, nonaggressive conduct that causes property loss or damage, deceitfulness or theft, and serious violations of rules. The onset is before age 18. (From DSM-IV, 1994)'),
  'D019956': NtO(MH='Stereotypic Movement Disorder', MS='Motor behavior that is repetitive, often seemingly driven, and nonfunctional. This behavior markedly interferes with normal activities or results in severe bodily self-injury. The behavior is not due to the direct physiological effects of a substance or a general medical condition. (DSM-IV, 1994)'),
  'D019957': NtO(MH='Motor Skills Disorders', MS='Marked impairments in the development of motor coordination such that the impairment interferes with activities of daily living. (From DSM-V)'),
  'D019958': NtO(MH='Attention Deficit and Disruptive Behavior Disorders', MS='Includes two similar disorders: oppositional defiant disorder and CONDUCT DISORDERS. Symptoms occurring in children with these disorders include: defiance of authority figures, angry outbursts, and other antisocial behaviors.'),
  'D019959': NtO(MH='Feeding and Eating Disorders of Childhood', MS='Mental disorders related to feeding and eating usually diagnosed in infancy or early childhood.'),
  'D019960': NtO(MH='Elimination Disorders', MS='Excretory-related psychiatric disorders usually diagnosed in infancy or childhood.'),
  'D019962': NtO(MH='Reactive Attachment Disorder', MS='Markedly disturbed and developmentally inappropriate social relatedness that begins before age 5 and is associated with grossly pathological child care. The child may persistently fail to initiate and respond to social interactions in a developmentally appropriate way (inhibited type) or there may be a pattern of diffuse attachments with nondiscriminate sociability (disinhibited type). (From DSM-V)'),
  'D019963': NtO(MH='Clinical Chemistry Tests', MS='Laboratory tests demonstrating the presence of physiologically significant substances in the blood, urine, tissue, and body fluids with application to the diagnosis or therapy of disease.'),
  'D019964': NtO(MH='Mood Disorders', MS='Those disorders that have a disturbance in mood as their predominant feature.'),
  'D019965': NtO(MH='Neurocognitive Disorders', MS='Diagnoses of DEMENTIA and AMNESTIC DISORDER are subsumed here. (DSM-5)'),
  'D019966': NtO(MH='Substance-Related Disorders', MS='Disorders related to substance abuse.'),
  'D019967': NtO(MH='Schizophrenia Spectrum and Other Psychotic Disorders', MS='Marked disorders of thought (delusions, hallucinations, or other thought disorder accompanied by disordered affect or behavior), and deterioration from a previous level of functioning. Individuals have one o more of the following symptoms: delusions, hallucinations, and disorganized speech. (from DSM-5)'),
  'D019968': NtO(MH='Sexual and Gender Disorders', MS='Mental disorders related to sexual dysfunction, paraphilias, and gender identity disorders.'),
  'D019969': NtO(MH='Amphetamine-Related Disorders', MS='Disorders related or resulting from use of amphetamines.'),
  'D019970': NtO(MH='Cocaine-Related Disorders', MS='Disorders related or resulting from use of cocaine.'),
  'D019973': NtO(MH='Alcohol-Related Disorders', MS='Disorders related to or resulting from abuse or mis-use of alcohol.'),
  'D019974': NtO(MH='Pan paniscus', MS='The pygmy chimpanzee, a species of the genus Pan, family HOMINIDAE. Its common name is Bonobo, which was once considered a separate genus by some; others considered it a subspecies of PAN TROGLODYTES. Its range is confined to the forests of the central Zaire basin. Despite its name, it is often of equal size to P. troglodytes.'),
  'D019976': NtO(MH='Cloning, Organism', MS='The formation of one or more genetically identical organisms derived by vegetative reproduction from a single cell. The source nuclear material can be embryo-derived, fetus-derived, or taken from an adult somatic cell.'),
  'D019980': NtO(MH='Amoxicillin-Potassium Clavulanate Combination', MS='A fixed-ratio combination of amoxicillin trihydrate and potassium clavulanate.'),
  'D019981': NtO(MH='Health Care Sector', MS='Economic sector concerned with the provision, distribution, and consumption of health care services and related products.'),
  'D019982': NtO(MH='Organizational Case Studies', MS='Descriptions and evaluations of specific health care organizations.'),
  'D019983': NtO(MH='Guideline Adherence', MS='Conformity in fulfilling or following official, recognized, or institutional requirements, guidelines, recommendations, protocols, pathways, or other standards.'),
  'D019984': NtO(MH='Quality Indicators, Health Care', MS='Norms, criteria, standards, and other direct qualitative and quantitative measures used in determining the quality of health care.'),
  'D019985': NtO(MH='Benchmarking', MS='Method of measuring performance against established standards of best practice.'),
  'D019986': NtO(MH='Persons With Hearing Impairments', MS='Persons with any degree of loss of hearing that has an impact on their activities of daily living or that requires special assistance or intervention.'),
  'D019987': NtO(MH='Visually Impaired Persons', MS='Persons with loss of vision such that there is an impact on activities of daily living.'),
  'D019988': NtO(MH='Disabled Children', MS='Children with mental or physical disabilities that interfere with usual activities of daily living and that may require accommodation or intervention.'),
  'D019989': NtO(MH='Mentally Disabled Persons', MS='Persons diagnosed as having significantly lower than average intelligence and considerable problems in adapting to everyday life or lacking independence in regard to activities of daily living.'),
  'D019990': NtO(MH='Perioperative Care', MS='Interventions to provide care prior to, during, and immediately after surgery.'),
  'D019991': NtO(MH='Database', MS='A structured file of information or a set of logically related data stored and retrieved using computer-based means.'),
  'D019992': NtO(MH='Databases as Topic', MS='Works on organized collections of records, standardized in format and content, that are stored in any of a variety of computer-readable modes.'),
  'D019995': NtO(MH='Laboratory Chemicals', MS='Chemicals necessary to perform experimental and/or investigative procedures and for the preparation of drugs and other chemicals.'),
  'D019998': NtO(MH='Racemases and Epimerases', MS='Enzymes that catalyze inversion of the configuration around an asymmetric carbon in a substrate having one (racemase) or more (epimerase) center(s) of asymmetry. (Dorland, 28th ed) EC 5.1.'),
  'D019999': NtO(MH='Pharmaceutical Solutions', MS='Homogeneous liquid preparations that contain one or more chemical substances dissolved, i.e., molecularly dispersed, in a suitable solvent or mixture of mutually miscible solvents. For reasons of their ingredients, method of preparation, or use, they do not fall into another group of products.'),
  'D020000': NtO(MH='Decision Support Systems, Clinical', MS='Computer-based information systems used to integrate clinical and patient information and provide support for decision-making in patient care.'),
  'D020001': NtO(MH='1-Butanol', MS='A four carbon linear hydrocarbon that has a hydroxy group at position 1.'),
  'D020002': NtO(MH='tert-Butyl Alcohol', MS='An isomer of butanol that contains a tertiary butyl group that consists of three methyl groups, each separately attached to a central (tertiary) carbon.'),
  'D020003': NtO(MH='1-Octanol', MS='A colorless, slightly viscous liquid used as a defoaming or wetting agent. It is also used as a solvent for protective coatings, waxes, and oils, and as a raw material for plasticizers. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 5th ed)'),
  'D020005': NtO(MH='Propanols', MS='Isomeric forms and derivatives of PROPANOL (C3H7OH).'),
  'D020007': NtO(MH='Implants, Experimental', MS='Artificial substitutes for body parts and materials inserted into organisms during experimental studies.'),
  'D020008': NtO(MH='Delavirdine', MS='A potent, non-nucleoside reverse transcriptase inhibitor with activity specific for HIV-1.'),
  'D020010': NtO(MH='Heterosexuality', MS='The sexual attraction or relationship between members of the opposite SEX.'),
  'D020011': NtO(MH='Protective Agents', MS='Synthetic or natural substances which are given to prevent a disease or disorder or are used in the process of treating a disease or injury due to a poisonous agent.'),
  'D020013': NtO(MH='Calcium Signaling', MS='Signal transduction mechanisms whereby calcium mobilization (from outside the cell or from intracellular storage pools) to the cytoplasm is triggered by external stimuli. Calcium signals are often seen to propagate as waves, oscillations, spikes, sparks, or puffs. The calcium acts as an intracellular messenger by activating calcium-responsive proteins.'),
  'D020014': NtO(MH='K562 Cells', MS='An ERYTHROLEUKEMIA cell line derived from a CHRONIC MYELOID LEUKEMIA patient in BLAST CRISIS.'),
  'D020016': NtO(MH='Activated Protein C Resistance', MS='A hemostatic disorder characterized by a poor anticoagulant response to activated protein C (APC). The activated form of Factor V (Factor Va) is more slowly degraded by activated protein C. Factor V Leiden mutation (R506Q) is the most common cause of APC resistance.'),
  'D020018': NtO(MH='Sexual Dysfunctions, Psychological', MS='Disturbances in sexual desire and the psychophysiologic changes that characterize the sexual response cycle and cause marked distress and interpersonal difficulty. (APA, DSM-IV, 1994)'),
  'D020019': NtO(MH='Neurocysticercosis', MS='Infection of the brain, spinal cord, or perimeningeal structures with the larval forms of the genus TAENIA (primarily T. solium in humans). Lesions formed by the organism are referred to as cysticerci. The infection may be subacute or chronic, and the severity of symptoms depends on the severity of the host immune response and the location and number of lesions. SEIZURES represent the most common clinical manifestation although focal neurologic deficits may occur. (From Joynt, Clinical Neurology, 1998, Ch27, pp46-50)'),
  'D020022': NtO(MH='Genetic Predisposition to Disease', MS='A latent susceptibility to disease at the genetic level, which may be activated under certain conditions.'),
  'D020024': NtO(MH='Leukotriene Antagonists', MS='A class of drugs designed to prevent leukotriene synthesis or activity by blocking binding at the receptor level.'),
  'D020025': NtO(MH='Peroxisome Proliferators', MS='A class of nongenotoxic CARCINOGENS that induce the production of hepatic PEROXISOMES and induce hepatic neoplasms after long-term administration.'),
  'D020027': NtO(MH='Pyrococcus furiosus', MS='A species of strictly anaerobic, hyperthermophilic archaea which lives in geothermally-heated marine sediments. It exhibits heterotropic growth by fermentation or sulfur respiration.'),
  'D020029': NtO(MH='Base Pairing', MS='Pairing of purine and pyrimidine bases by HYDROGEN BONDING in double-stranded DNA or RNA.'),
  'D020030': NtO(MH='Nitric Oxide Donors', MS='A diverse group of agents, with unique chemical structures and biochemical requirements, which generate NITRIC OXIDE. These compounds have been used in the treatment of cardiovascular diseases and the management of acute myocardial infarction, acute and chronic congestive heart failure, and surgical control of blood pressure. (Adv Pharmacol 1995;34:361-81)'),
  'D020031': NtO(MH='Epstein-Barr Virus Infections', MS='Infection with human herpesvirus 4 (HERPESVIRUS 4, HUMAN); which may facilitate the development of various lymphoproliferative disorders. These include BURKITT LYMPHOMA (African type), INFECTIOUS MONONUCLEOSIS, and oral hairy leukoplakia (LEUKOPLAKIA, HAIRY).'),
  'D020032': NtO(MH='Tyrphostins', MS='A family of synthetic protein tyrosine kinase inhibitors. They selectively inhibit receptor autophosphorylation and are used to study receptor function.'),
  'D020033': NtO(MH='Protein Isoforms', MS='Different forms of a protein that may be produced from different GENES, or from the same gene by ALTERNATIVE SPLICING.'),
  'D020034': NtO(MH='Silicone Gels', MS='Synthetic organosiloxane gels that are formed from synthetic polymers of silicone oxide with organic sidechains (polydimethylsiloxane) by lengthening the polymer chains. Unlike silicone elastomers, they are not treated with amorphous silica. They are used as fillers in breast implants.'),
  'D020035': NtO(MH='Holoenzymes', MS='Catalytically active enzymes that are formed by the combination of an apoenzyme (APOENZYMES) and its appropriate cofactors and prosthetic groups.'),
  'D020036': NtO(MH='Onygenales', MS='An order of fungi in the phylum ASCOMYCOTA containing many medically important species. There are four families and mitosporic (anamorphic) forms are prominent.'),
  'D020037': NtO(MH='Sordariales', MS='An order of fungi in the phylum ASCOMYCOTA that includes many valuable experimental organisms. There are eight families and very few anamorphic forms.'),
  'D020038': NtO(MH='RNA, Spliced Leader', MS="The small RNAs which provide spliced leader sequences, SL1, SL2, SL3, SL4 and SL5 (short sequences which are joined to the 5' ends of pre-mRNAs by TRANS-SPLICING). They are found primarily in primitive eukaryotes (protozoans and nematodes)."),
  'D020040': NtO(MH='Trans-Splicing', MS='The joining of RNA from two different genes. One type of trans-splicing is the "spliced leader" type (primarily found in protozoans such as trypanosomes and in lower invertebrates such as nematodes) which results in the addition of a capped, noncoding, spliced leader sequence to the 5\' end of mRNAs. Another type of trans-splicing is the "discontinuous group II introns" type (found in plant/algal chloroplasts and plant mitochondria) which results in the joining of two independently transcribed coding sequences. Both are mechanistically similar to conventional nuclear pre-mRNA cis-splicing. Mammalian cells are also capable of trans-splicing.'),
  'D020042': NtO(MH='Histiocytic Necrotizing Lymphadenitis', MS='Development of lesions in the lymph node characterized by infiltration of the cortex or paracortex by large collections of proliferating histiocytes and complete or, more often, incomplete necrosis of lymphoid tissue.'),
  'D020043': NtO(MH='Genes, Essential', MS='Those genes found in an organism which are necessary for its viability and normal function.'),
  'D020045': NtO(MH='Pfiesteria piscicida', MS='A dinoflagellate with a life cycle that includes numerous flagellated, amoeboid, and encysted stages. Both the flagellated and amoeboid forms produce toxins which cause open wounds on fish. Pfiesteria piscicida feeds on tissue sloughed from these wounds, as well as on bacteria and algae. It is found in Atlantic estuaries of the United States.'),
  'D020047': NtO(MH='Rectocele', MS='Herniation of the RECTUM into the VAGINA.'),
  'D020048': NtO(MH='Blattellidae', MS='A family of insects in the order Dictyoptera (COCKROACHES), including genera Blattella, Parcoblatta, and Symploce.'),
  'D020049': NtO(MH='Isoptera', MS='An order of insects, restricted mostly to the tropics, containing at least eight families. A few species occur in temperate regions of North America.'),
  'D020050': NtO(MH='Mantodea', MS='An order of insects, comprising a single family (Mantidae), restricted almost entirely to the tropics. Only one species, the praying mantis (Mantis religiosa), is found in temperate habitats.'),
  'D020051': NtO(MH='N-Acetylgalactosamine-4-Sulfatase', MS='An arylsulfatase that catalyzes the hydrolysis of the 4-sulfate groups of the N-acetyl-D-galactosamine 4-sulfate units of chondroitin sulfate and dermatan sulfate. A deficiency of this enzyme is responsible for the inherited lysosomal disease, Maroteaux-Lamy syndrome (MUCOPOLYSACCHARIDOSIS VI). EC 3.1.6.12.'),
  'D020058': NtO(MH='Styrene', MS='A colorless, toxic liquid with a strong aromatic odor. It is used to make rubbers, polymers and copolymers, and polystyrene plastics.'),
  'D020059': NtO(MH='Cathepsin E', MS='An aspartic endopeptidase that is similar in structure to CATHEPSIN D. It is found primarily in the cells of the immune system where it may play a role in processing of CELL SURFACE ANTIGENS.'),
  'D020061': NtO(MH='Phthirus', MS='Lice of the genus Phthirus, family Pediculidae. Phthirus pubis, the crab louse, is usually acquired by sexual contact or contact with infected objects. It is found most frequently in the pubic hair, but also on eyebrows, eyelashes, or on the axillary hairs.'),
  'D020062': NtO(MH='Heteroptera', MS='A suborder of HEMIPTERA, called true bugs, characterized by the possession of two pairs of wings. It includes the medically important families CIMICIDAE and REDUVIIDAE. (From Dorland, 28th ed)'),
  'D020063': NtO(MH='Cimicidae', MS='A family of wingless, blood-sucking insects of the suborder HETEROPTERA, including the bedbugs and related forms. Cimex (BEDBUGS), Heamatosiphon, and Oeciacus are medically important genera. (From Dorland, 28th ed)'),
  'D020064': NtO(MH='Reduviidae', MS='A family of winged insects of the suborder HETEROPTERA, called assassin bugs, because most prey on other insects. However one subfamily, TRIATOMINAE, attacks humans and other vertebrates and transmits Chagas disease.'),
  'D020065': NtO(MH='Hepatopulmonary Syndrome', MS='A syndrome characterized by the clinical triad of advanced chronic liver disease, pulmonary vascular dilatations, and reduced arterial oxygenation (HYPOXEMIA) in the absence of intrinsic cardiopulmonary disease. This syndrome is common in the patients with LIVER CIRRHOSIS or portal hypertension (HYPERTENSION, PORTAL).'),
  'D020066': NtO(MH='Merozoite Surface Protein 1', MS='A surface protein found on Plasmodium species which induces a T-cell response. The antigen is polymorphic, sharing amino acid sequence homology among PLASMODIUM FALCIPARUM; PLASMODIUM CHABAUDI; PLASMODIUM VIVAX; and PLASMODIUM YOELII.'),
  'D020068': NtO(MH='Zygosaccharomyces', MS='A genus of ascomycetous fungi of the family Saccharomycetaceae, order SACCHAROMYCETALES.'),
  'D020069': NtO(MH='Shoulder Pain', MS='Unilateral or bilateral pain of the shoulder. It is often caused by physical activities such as work or sports participation, but may also be pathologic in origin.'),
  'D020070': NtO(MH='Cervical Ripening', MS='A change in the CERVIX UTERI with respect to its readiness to relax. The cervix normally becomes softer, more flexible, more distensible, and shorter in the final weeks of PREGNANCY. These cervical changes can also be chemically induced (LABOR, INDUCED).'),
  'D020071': NtO(MH='Interspersed Repetitive Sequences', MS='Copies of transposable elements interspersed throughout the genome, some of which are still active and often referred to as "jumping genes". There are two classes of interspersed repetitive elements. Class I elements (or RETROELEMENTS - such as retrotransposons, retroviruses, LONG INTERSPERSED NUCLEOTIDE ELEMENTS and SHORT INTERSPERSED NUCLEOTIDE ELEMENTS) transpose via reverse transcription of an RNA intermediate. Class II elements (or DNA TRANSPOSABLE ELEMENTS - such as transposons, Tn elements, insertion sequence elements and mobile gene cassettes of bacterial integrons) transpose directly from one site in the DNA to another.'),
  'D020072': NtO(MH='Polyporales', MS='An order of fungi in the phylum BASIDIOMYCOTA having macroscopic basidiocarps. The members are characterized by their saprophytic activities as decomposers, particularly in the degradation of CELLULOSE and LIGNIN. A large number of species in the order have been used medicinally. (From Alexopoulos, Introductory Mycology, 4th ed, pp504-68)'),
  'D020073': NtO(MH='Gravidity', MS='The number of pregnancies, complete or incomplete, experienced by a female. It is different from PARITY, which is the number of offspring borne. (From Stedman, 26th ed)'),
  'D020074': NtO(MH='Lentinula', MS='A genus of fungi of the family Tricholomataceae, order AGARICALES. The commonly known SHIITAKE MUSHROOMS are Lentinula edodes (also seen as Lentinus edodes).'),
  'D020075': NtO(MH='Phanerochaete', MS='A genus of fungi in the family Corticiaceae, order Stereales, that degrades lignin. The white-rot fungus Phanerochaete chrysosporium is a frequently used species in research.'),
  'D020076': NtO(MH='Pleurotus', MS='A genus of basidiomycetous fungi, family POLYPORACEAE, order POLYPORALES, that grows on logs or tree stumps in shelflike layers. The species P. ostreatus, the oyster mushroom, is a choice edible species and is the most frequently encountered member of the genus in eastern North America. (Alexopoulos et al., Introductory Mycology, 4th ed, p531)'),
  'D020077': NtO(MH='Endogenous Retroviruses', MS='Retroviruses that have integrated into the germline (PROVIRUSES) that have lost infectious capability but retained the capability to transpose.'),
  'D020078': NtO(MH='Neurogenic Inflammation', MS='Inflammation caused by an injurious stimulus of peripheral neurons and resulting in release of neuropeptides which affect vascular permeability and help initiate proinflammatory and immune reactions at the site of injury.'),
  'D020079': NtO(MH='Terminal Repeat Sequences', MS="Nucleotide sequences repeated on both the 5' and 3' ends of a sequence under consideration. For example, the hallmarks of a transposon are that it is flanked by inverted repeats on each end and the inverted repeats are flanked by direct repeats. The Delta element of Ty retrotransposons and LTRs (long terminal repeats) are examples of this concept."),
  'D020080': NtO(MH='Tandem Repeat Sequences', MS='Copies of DNA sequences which lie adjacent to each other in the same orientation (direct tandem repeats) or in the opposite direction to each other (INVERTED TANDEM REPEATS).'),
  'D020081': NtO(MH='Phyllachorales', MS='An order of fungi in the phylum ASCOMYCOTA characterized by stromatic perithecial forms in most species. Notable genera are Magnaporthe and Glomerella, the latter having the anamorph (mitosporic form) COLLETOTRICHUM.'),
  'D020082': NtO(MH='Magnaporthe', MS='A genus of FUNGI, in the family Magnaporthaceae of uncertain position (incertae sedis). It is best known for its species, M. grisea, which is one of the most popular experimental organisms of all fungal plant pathogens. Its anamorph is PYRICULARIA GRISEA.'),
  'D020083': NtO(MH='Acquired Hyperostosis Syndrome', MS='Syndrome consisting of SYNOVITIS; ACNE CONGLOBATA; PALMOPLANTAR PUSTULOSIS; HYPEROSTOSIS; and OSTEITIS. The most common site of the disease is the upper anterior chest wall, characterized by predominantly osteosclerotic lesions, hyperostosis, and arthritis of the adjacent joints. The association of sterile inflammatory bone lesions and neutrophilic skin eruptions is indicative of this syndrome.'),
  'D020084': NtO(MH='Long Interspersed Nucleotide Elements', MS='Highly repeated sequences, 6K-8K base pairs in length, which contain RNA polymerase II promoters. They also have an open reading frame that is related to the reverse transcriptase of retroviruses but they do not contain LTRs (long terminal repeats). Copies of the LINE 1 (L1) family form about 15% of the human genome. The jockey elements of Drosophila are LINEs.'),
  'D020085': NtO(MH='Neocallimastigales', MS='An order of fungi in the phylum NEOCALLIMASTIGOMYCOTA comprising anaerobic chytrids that inhabit the RUMEN; and CECUM of herbivorous animals. Genera (all in the lone family Neocallimastigaceae) include NEOCALLIMASTIX, Orpinomyces, PIROMYCES, Anaeromyces, Cyllamyces, and Caecomyces.'),
  'D020086': NtO(MH='Short Interspersed Nucleotide Elements', MS='Highly repeated sequences, 100-300 bases long, which contain RNA polymerase III promoters. The primate Alu (ALU ELEMENTS) and the rodent B1 SINEs are derived from 7SL RNA, the RNA component of the signal recognition particle. Most other SINEs are derived from tRNAs including the MIRs (mammalian-wide interspersed repeats).'),
  'D020087': NtO(MH='Alu Elements', MS='The Alu sequence family (named for the restriction endonuclease cleavage enzyme Alu I) is the most highly repeated interspersed repeat element in humans (over a million copies). It is derived from the 7SL RNA component of the SIGNAL RECOGNITION PARTICLE and contains an RNA polymerase III promoter. Transposition of this element into coding and regulatory regions of genes is responsible for many heritable diseases.'),
  'D020088': NtO(MH='Neocallimastix', MS='A genus of fungi in the family Neocallimasticaceae, order NEOCALLIMASTIGALES. They contain polyflagellate zoospores and grow on a range of simple and complex carbohydrates in the rumen of sheep and cattle.'),
  'D020089': NtO(MH='Piromyces', MS='A genus of fungi in the family Neocallimasticaceae, order NEOCALLIMASTICALES, containing uniflagellate zoospores.'),
  'D020090': NtO(MH='Chromosome Segregation', MS='The orderly segregation of CHROMOSOMES during MEIOSIS or MITOSIS.'),
  'D020091': NtO(MH='Entomophthorales', MS='An order of fungi comprising mostly insect pathogens, though some infect mammals including humans. Strict host specificity make these fungi a focus of many biological control studies.'),
  'D020092': NtO(MH='Conidiobolus', MS='A genus of fungi in the family Ancylistaceae, order ENTOMOPHTHORALES, characterized by the presence of small nuclei with a prominent central nucleolus in interphase. They are commonly found in the soil but also infect termites, aphids, and sometimes, though rarely, mammals including humans. (Alexopoulos et al, Introductory Mycology, 4th ed, pp159-60)'),
  'D020093': NtO(MH='Cunninghamella', MS='A genus of zygomycetous fungi of the family Cunninghamellaceae, order MUCORALES. Some species cause systemic infections in humans.'),
  'D020094': NtO(MH='Mortierella', MS='A genus of zygomycetous fungi of the family Mortierellaceae, order MUCORALES. Its species are abundant in soil and can cause rare infections in humans and animals. Mortierella alpinais is used for production of arachidonic acid.'),
  'D020095': NtO(MH='Absidia', MS='A genus of zygomycetous fungi, family Mucoraceae, order MUCORALES, which sometimes causes infection in humans.'),
  'D020096': NtO(MH='Zygomycosis', MS='Infection in humans and animals caused by fungi in the class Zygomycetes. It includes MUCORMYCOSIS and entomophthoramycosis. The latter is a tropical infection of subcutaneous tissue or paranasal sinuses caused by fungi in the order Entomophthorales. Phycomycosis, closely related to zygomycosis, describes infection with members of Phycomycetes, an obsolete classification.'),
  'D020097': NtO(MH='Natriuretic Peptide, Brain', MS='A PEPTIDE that is secreted by the BRAIN and the HEART ATRIA, stored mainly in cardiac ventricular MYOCARDIUM. It can cause NATRIURESIS; DIURESIS; VASODILATION; and inhibits secretion of RENIN and ALDOSTERONE. It improves heart function. It contains 32 AMINO ACIDS.'),
  'D020098': NtO(MH='Natriuretic Peptide, C-Type', MS='A PEPTIDE of 22 amino acids, derived mainly from cells of VASCULAR ENDOTHELIUM. It is also found in the BRAIN, major endocrine glands, and other tissues. It shares structural homology with ATRIAL NATRIURETIC FACTOR. It has vasorelaxant activity thus is important in the regulation of vascular tone and blood flow. Several high molecular weight forms containing the 22 amino acids have been identified.'),
  'D020099': NtO(MH='Coated Materials, Biocompatible', MS='Biocompatible materials usually used in dental and bone implants that enhance biologic fixation, thereby increasing the bond strength between the coated material and bone, and minimize possible biological effects that may result from the implant itself.'),
  'D020100': NtO(MH='Hydrogels', MS="Water swollen, rigid, 3-dimensional network of cross-linked, hydrophilic macromolecules, 20-95% water. They are used in paints, printing inks, foodstuffs, pharmaceuticals, and cosmetics. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D020101': NtO(MH='Acrosome Reaction', MS='Changes that occur to liberate the enzymes of the ACROSOME of a sperm (SPERMATOZOA). Acrosome reaction allows the sperm to penetrate the ZONA PELLUCIDA and enter the OVUM during FERTILIZATION.'),
  'D020102': NtO(MH='Mycobacterium smegmatis', MS='A rapid-growing, nonphotochromogenic species of MYCOBACTERIUM originally isolated from human smegma and found also in soil and water. (From Dorland, 28th ed)'),
  'D020103': NtO(MH='Rhizomucor', MS='A genus of zygomycetous fungi of the family Mucoraceae, order MUCORALES.'),
  'D020104': NtO(MH='Immunophilins', MS='Members of a family of highly conserved proteins which are all cis-trans peptidyl-prolyl isomerases (PEPTIDYLPROLYL ISOMERASE). They bind the immunosuppressant drugs CYCLOSPORINE; TACROLIMUS and SIROLIMUS. They possess rotamase activity, which is inhibited by the immunosuppressant drugs that bind to them.'),
  'D020105': NtO(MH='Milrinone', MS='A positive inotropic cardiotonic agent with vasodilator properties. It inhibits cAMP phosphodiesterase type 3 activity in myocardium and vascular smooth muscle. Milrinone is a derivative of amrinone and has 20-30 times the inotropic potency of amrinone.'),
  'D020106': NtO(MH='Acrylamide', MS='A colorless, odorless, highly water soluble vinyl monomer formed from the hydration of acrylonitrile. It is primarily used in research laboratories for electrophoresis, chromatography, and electron microscopy and in the sewage and wastewater treatment industries.'),
  'D020107': NtO(MH='Troponin T', MS='A TROPONIN complex subunit that binds to TROPOMYOSIN. There are three troponin T subtypes: troponin T1, T2 and T3. Troponin T2 is cardiac-specific whereas troponin T2 and T3 are skeletal subtypes. Troponin T2 is a BIOMARKER for damaged or injured CARDIAC MYOCYTES and mutations in troponin T2 gene are associated with FAMILIAL HYPERTROPHIC CARDIOMYOPATHY.'),
  'D020108': NtO(MH='Nicorandil', MS='A derivative of the NIACINAMIDE that is structurally combined with an organic nitrate. It is a potassium-channel opener that causes vasodilatation of arterioles and large coronary arteries. Its nitrate-like properties produce venous vasodilation through stimulation of guanylate cyclase.'),
  'D020109': NtO(MH='Devazepide', MS="A derivative of benzodiazepine that acts on the cholecystokinin A (CCKA) receptor to antagonize CCK-8's (SINCALIDE) physiological and behavioral effects, such as pancreatic stimulation and inhibition of feeding."),
  'D020110': NtO(MH='Pinacidil', MS='A guanidine that opens POTASSIUM CHANNELS producing direct peripheral vasodilatation of the ARTERIOLES. It reduces BLOOD PRESSURE and peripheral resistance and produces fluid retention. (Martindale The Extra Pharmacopoeia, 31st ed)'),
  'D020111': NtO(MH='Chlorodiphenyl (54% Chlorine)', MS='A mixture of polychlorinated biphenyls that induces hepatic microsomal UDP-glucuronyl transferase activity towards thyroxine.'),
  'D020112': NtO(MH='Rhodamine 123', MS='A fluorescent probe with low toxicity which is a potent substrate for ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B, MEMBER 1 and the bacterial multidrug efflux transporter. It is used to assess mitochondrial bioenergetics in living cells and to measure the efflux activity of ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B, MEMBER 1 in both normal and malignant cells. (Leukemia 1997;11(7):1124-30)'),
  'D020113': NtO(MH='Phenolphthalein', MS='An acid-base indicator which is colorless in acid solution, but turns pink to red as the solution becomes alkaline. It is used medicinally as a cathartic.'),
  'D020114': NtO(MH='Rickettsia conorii', MS="A species of gram-negative, aerobic bacteria that is the etiologic agent of BOUTONNEUSE FEVER. It resembles RICKETTSIA RICKETTSII but is antigenically distinct and less virulent for animals and man. (From Bergey's Manual of Systematic Bacteriology, vol 1)"),
  'D020115': NtO(MH='Pepsinogen C', MS='This is one of the 2 related pepsinogen systems in humans. It is found in prostate and seminal fluid whereas PEPSINOGEN A is not.'),
  'D020116': NtO(MH='Pepsinogen A', MS='This is one of 2 related pepsinogen systems in humans and is also known as pepsinogen. (The other is PEPSINOGEN C.) This includes isozymogens Pg1-Pg5 (pepsinogens 1-5, group I or products of PGA1-PGA5 genes). This is the main pepsinogen found in urine.'),
  'D020117': NtO(MH='Cisapride', MS='A substituted benzamide used for its prokinetic properties. It is used in the management of gastroesophageal reflux disease, functional dyspepsia, and other disorders associated with impaired gastrointestinal motility. (Martindale The Extra Pharmacopoeia, 31st ed)'),
  'D020118': NtO(MH='Cupriavidus necator', MS='A gram-negative, facultatively chemoautotrophic bacterium, formerly called Wautersia eutropha, found in water and soil.'),
  'D020120': NtO(MH='Iodocyanopindolol', MS='A highly selective and specific beta antagonist that is used to characterize beta-adrenoceptors.'),
  'D020121': NtO(MH="5' Untranslated Regions", MS="The sequence at the 5' end of the messenger RNA that does not code for product. This sequence contains the ribosome binding site and other transcription and translation regulating sequences."),
  'D020122': NtO(MH='tert-Butylhydroperoxide', MS='A direct-acting oxidative stress-inducing agent used to examine the effects of oxidant stress on Ca(2+)-dependent signal transduction in vascular endothelial cells. It is also used as a catalyst in polymerization reactions and to introduce peroxy groups into organic molecules.'),
  'D020123': NtO(MH='Sirolimus', MS='A macrolide compound obtained from Streptomyces hygroscopicus that acts by selectively blocking the transcriptional activation of cytokines thereby inhibiting cytokine production. It is bioactive only when bound to IMMUNOPHILINS. Sirolimus is a potent immunosuppressant and possesses both antifungal and antineoplastic properties.'),
  'D020124': NtO(MH='Thermotoga maritima', MS="A rod-shaped bacterium surrounded by a sheath-like structure which protrudes balloon-like beyond the ends of the cell. It is thermophilic, with growth occurring at temperatures as high as 90 degrees C. It is isolated from geothermally heated marine sediments or hot springs. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020125': NtO(MH='Mutation, Missense', MS='A mutation in which a codon is mutated to one directing the incorporation of a different amino acid. This substitution may result in an inactive or unstable product. (From A Dictionary of Genetics, King & Stansfield, 5th ed)'),
  'D020126': NtO(MH='Brefeldin A', MS='A fungal metabolite which is a macrocyclic lactone exhibiting a wide range of antibiotic activity.'),
  'D020127': NtO(MH='Recovery of Function', MS='A partial or complete return to the normal or proper physiologic activity of an organ or part following disease or trauma.'),
  'D020128': NtO(MH='Inhibitory Concentration 50', MS='The concentration of a compound needed to reduce population growth of organisms, including eukaryotic cells, by 50% in vitro. Though often expressed to denote in vitro antibacterial activity, it is also used as a benchmark for cytotoxicity to eukaryotic cells in culture.'),
  'D020130': NtO(MH='Bacterial Chromatophores', MS='Organelles of phototrophic bacteria which contain photosynthetic pigments and which are formed from an invagination of the cytoplasmic membrane.'),
  'D020131': NtO(MH='Genes, Duplicate', MS='Two identical genes showing the same phenotypic action but localized in different regions of a chromosome or on different chromosomes. (From Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D020132': NtO(MH='Anticipation, Genetic', MS='The apparent tendency of certain diseases to appear at earlier AGE OF ONSET and with increasing severity in successive generations. (Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D020133': NtO(MH='Reverse Transcriptase Polymerase Chain Reaction', MS='A variation of the PCR technique in which cDNA is made from RNA via reverse transcription. The resultant cDNA is then amplified using standard PCR protocols.'),
  'D020134': NtO(MH='Catalytic Domain', MS='The region of an enzyme that interacts with its substrate to cause the enzymatic reaction.'),
  'D020135': NtO(MH='Peptide Nucleic Acids', MS='DNA analogs containing neutral amide backbone linkages composed of aminoethyl glycine units instead of the usual phosphodiester linkage of deoxyribose groups. Peptide nucleic acids have high biological stability and higher affinity for complementary DNA or RNA sequences than analogous DNA oligomers.'),
  'D020136': NtO(MH='Hydrogel, Polyethylene Glycol Dimethacrylate', MS='A network of cross-linked hydrophilic macromolecules used in biomedical applications fabricated by photopolymerization of polyethylene glycol dimethacrylate. Its general formulae is C3H5C(O)(OCH2CH2)nOC(O)C3H5 where n denotes a number of average polyglycol (OCH2CH2) repeats.'),
  'D020137': NtO(MH='Base Pair Mismatch', MS='The presence of an uncomplimentary base in double-stranded DNA caused by spontaneous deamination of cytosine or adenine, mismatching during homologous recombination, or errors in DNA replication. Multiple, sequential base pair mismatches lead to formation of heteroduplex DNA; (NUCLEIC ACID HETERODUPLEXES).'),
  'D020138': NtO(MH='Hyperhomocysteinemia', MS='Condition in which the plasma levels of homocysteine and related metabolites are elevated (>13.9 \xce\xbcmol/l). Hyperhomocysteinemia can be familial or acquired. Development of the acquired hyperhomocysteinemia is mostly associated with vitamins B and/or folate deficiency (e.g., PERNICIOUS ANEMIA, vitamin malabsorption). Familial hyperhomocysteinemia often results in a more severe elevation of total homocysteine and excretion into the urine, resulting in HOMOCYSTINURIA. Hyperhomocysteinemia is a risk factor for cardiovascular and neurodegenerative diseases, osteoporotic fractures and complications during pregnancy.'),
  'D020139': NtO(MH='Gastroschisis', MS='A congenital defect with major fissure in the ABDOMINAL WALL lateral to, but not at, the UMBILICUS. This results in the extrusion of VISCERA. Unlike OMPHALOCELE, herniated structures in gastroschisis are not covered by a sac or PERITONEUM.'),
  'D020140': NtO(MH='Lysosomal Storage Diseases, Nervous System', MS='A group of enzymatic disorders affecting the nervous system and to a variable degree the skeletal system, lymphoreticular system, and other organs. The conditions are marked by an abnormal accumulation of catabolic material within lysosomes.'),
  'D020141': NtO(MH='Hemostatic Disorders', MS='Pathological processes involving the integrity of blood circulation. Hemostasis depends on the integrity of BLOOD VESSELS, blood fluidity, and BLOOD COAGULATION. Majority of the hemostatic disorders are caused by disruption of the normal interaction between the VASCULAR ENDOTHELIUM, the plasma proteins (including BLOOD COAGULATION FACTORS), and PLATELETS.'),
  'D020143': NtO(MH='Gangliosidoses, GM2', MS='A group of recessively inherited diseases characterized by the intralysosomal accumulation of G(M2) GANGLIOSIDE in the neuronal cells. Subtypes include mutations of enzymes in the BETA-N-ACETYLHEXOSAMINIDASES system or G(M2) ACTIVATOR PROTEIN leading to disruption of normal degradation of GANGLIOSIDES, a subclass of ACIDIC GLYCOSPHINGOLIPIDS.'),
  'D020144': NtO(MH='Basal Ganglia Cerebrovascular Disease', MS='A pathological condition caused by impaired blood flow in the basal regions of cerebral hemispheres (BASAL GANGLIA), such as INFARCTION; HEMORRHAGE; or ISCHEMIA in vessels of this brain region including the lateral lenticulostriate arteries. Primary clinical manifestations include involuntary movements (DYSKINESIAS) and muscle weakness (HEMIPARESIS).'),
  'D020145': NtO(MH='Basal Ganglia Hemorrhage', MS='Bleeding within the subcortical regions of cerebral hemispheres (BASAL GANGLIA). It is often associated with HYPERTENSION or ARTERIOVENOUS MALFORMATIONS. Clinical manifestations may include HEADACHE; DYSKINESIAS; and HEMIPARESIS.'),
  'D020146': NtO(MH='Putaminal Hemorrhage', MS='Intracranial bleeding into the PUTAMEN, a BASAL GANGLIA nucleus. This is associated with HYPERTENSION and lipohyalinosis of small blood vessels in the putamen. Clinical manifestations vary with the size of hemorrhage, but include HEMIPARESIS; HEADACHE; and alterations of consciousness.'),
  'D020147': NtO(MH='Coagulation Protein Disorders', MS='Hemorrhagic and thrombotic disorders resulting from abnormalities or deficiencies of coagulation proteins.'),
  'D020148': NtO(MH='Butyric Acid', MS='A four carbon acid, CH3CH2CH2COOH, with an unpleasant odor that occurs in butter and animal fat as the glycerol ester.'),
  'D020149': NtO(MH='Manganese Poisoning', MS='Manganese poisoning is associated with chronic inhalation of manganese particles by individuals who work with manganese ore. Clinical features include CONFUSION; HALLUCINATIONS; and an extrapyramidal syndrome (PARKINSON DISEASE, SECONDARY) that includes rigidity; DYSTONIA; retropulsion; and TREMOR. (Adams, Principles of Neurology, 6th ed, p1213)'),
  'D020150': NtO(MH='Chorea Gravidarum', MS='A rare movement disorder developed during PREGNANCY, characterized by involuntary jerky motion (CHOREA) and inability to maintain stable position of body parts (ATHETOSIS). RHEUMATIC FEVER and collagen vascular disorders are frequently associated with this disease. Chorea may vary from mild to severe and occurs in approximately 1 per 2,000 to 3,000 pregnancies. (From Md Med J 1997 Sep;46(8):436-9)'),
  'D020151': NtO(MH='Protein C Deficiency', MS="An absence or deficiency in PROTEIN C which leads to impaired regulation of blood coagulation. It is associated with an increased risk of severe or premature thrombosis. (Stedman's Med. Dict., 26th ed.)"),
  'D020152': NtO(MH='Antithrombin III Deficiency', MS='An absence or reduced level of Antithrombin III leading to an increased risk for thrombosis.'),
  'D020155': NtO(MH='3-Hydroxybutyric Acid', MS='BUTYRIC ACID substituted in the beta or 3 position. It is one of the ketone bodies produced in the liver.'),
  'D020156': NtO(MH='Salicylic Acid', MS='A compound obtained from the bark of the white willow and wintergreen leaves. It has bacteriostatic, fungicidal, and keratolytic actions.'),
  'D020157': NtO(MH='Amino Acid Transport Disorders, Inborn', MS='Disorders characterized by defective transport of amino acids across cell membranes. These include deficits in transport across brush-border epithelial cell membranes of the small intestine (MICROVILLI) and KIDNEY TUBULES; transport across the basolateral membrane; and transport across the membranes of intracellular organelles. (From Nippon Rinsho 1992 Jul;50(7):1587-92)'),
  'D020158': NtO(MH='Hyperglycinemia, Nonketotic', MS='An autosomal recessive metabolic disorder caused by deficiencies in the mitochondrial GLYCINE cleavage system.'),
  'D020159': NtO(MH='Citrullinemia', MS='A group of diseases related to a deficiency of the enzyme ARGININOSUCCINATE SYNTHASE which causes an elevation of serum levels of CITRULLINE. In neonates, clinical manifestations include lethargy, hypotonia, and SEIZURES. Milder forms also occur. Childhood and adult forms may present with recurrent episodes of intermittent weakness, lethargy, ATAXIA, behavioral changes, and DYSARTHRIA. (From Menkes, Textbook of Child Neurology, 5th ed, p49)'),
  'D020160': NtO(MH='Sodium Benzoate', MS='The sodium salt of BENZOIC ACID. It is used as an antifungal preservative in pharmaceutical preparations and foods. It may also be used as a test for liver function.'),
  'D020161': NtO(MH='Physical Chromosome Mapping', MS='Mapping of the linear order of genes on a chromosome with units indicating their distances by using methods other than genetic recombination. These methods include nucleotide sequencing, overlapping deletions in polytene chromosomes, and electron micrography of heteroduplex DNA. (From King & Stansfield, A Dictionary of Genetics, 5th ed)'),
  'D020162': NtO(MH='Hyperargininemia', MS='A rare autosomal recessive disorder of the urea cycle. It is caused by a deficiency of the hepatic enzyme ARGINASE. Arginine is elevated in the blood and cerebrospinal fluid, and periodic HYPERAMMONEMIA may occur. Disease onset is usually in infancy or early childhood. Clinical manifestations include seizures, microcephaly, progressive mental impairment, hypotonia, ataxia, spastic diplegia, and quadriparesis. (From Hum Genet 1993 Mar;91(1):1-5; Menkes, Textbook of Child Neurology, 5th ed, p51)'),
  'D020163': NtO(MH='Ornithine Carbamoyltransferase Deficiency Disease', MS='An inherited urea cycle disorder associated with deficiency of the enzyme ORNITHINE CARBAMOYLTRANSFERASE, transmitted as an X-linked trait and featuring elevations of amino acids and ammonia in the serum. Clinical features, which are more prominent in males, include seizures, behavioral alterations, episodic vomiting, lethargy, and coma. (Menkes, Textbook of Child Neurology, 5th ed, pp49-50)'),
  'D020164': NtO(MH='Chemical Actions and Uses', MS='A group of pharmacologic activities, effects on living systems and the environment, and modes of employment of drugs and chemicals. They are broken into actions, which describe their effects, and uses, which describe how they are employed.'),
  'D020165': NtO(MH='Carbamoyl-Phosphate Synthase I Deficiency Disease', MS='A urea cycle disorder manifesting in infancy as lethargy, emesis, seizures, alterations of muscle tone, abnormal eye movements, and an elevation of serum ammonia. The disorder is caused by a reduction in the activity of hepatic mitochondrial CARBAMOYL-PHOSPHATE SYNTHASE (AMMONIA). (Menkes, Textbook of Child Neurology, 5th ed, pp50-1)'),
  'D020167': NtO(MH='Hyperlysinemias', MS='A group of inherited metabolic disorders which have in common elevations of serum LYSINE levels. Enzyme deficiencies of alpha-aminoadipic semialdehyde dehydrogenase and the SACCHAROPINE DEHYDROGENASES have been associated with hyperlysinemia. Clinical manifestations include mental retardation, recurrent emesis, hypotonia, lethargy, diarrhea, and developmental delay. (From Menkes, Textbook of Child Neurology, 5th ed, p56)'),
  'D020168': NtO(MH='ATP Binding Cassette Transporter, Subfamily B, Member 1', MS='A 170-kDa transmembrane glycoprotein from the superfamily of ATP-BINDING CASSETTE TRANSPORTERS. It serves as an ATP-dependent efflux pump for a variety of chemicals, including many ANTINEOPLASTIC AGENTS. Overexpression of this glycoprotein is associated with multidrug resistance (see DRUG RESISTANCE, MULTIPLE).'),
  'D020169': NtO(MH='Caspases', MS='A family of intracellular CYSTEINE ENDOPEPTIDASES that play a role in regulating INFLAMMATION and APOPTOSIS. They specifically cleave peptides at a CYSTEINE amino acid that follows an ASPARTIC ACID residue. Caspases are activated by proteolytic cleavage of a precursor form to yield large and small subunits that form the enzyme. Since the cleavage site within precursors matches the specificity of caspases, sequential activation of precursors by activated caspases can occur.'),
  'D020170': NtO(MH='Caspase 1', MS='A long pro-domain caspase that has specificity for the precursor form of INTERLEUKIN-1BETA. It plays a role in INFLAMMATION by catalytically converting the inactive forms of CYTOKINES such as interleukin-1beta to their active, secreted form. Caspase 1 is referred as interleukin-1beta converting enzyme and is frequently abbreviated ICE.'),
  'D020171': NtO(MH='Botrytis', MS='A mitosporic Leotiales fungal genus of plant pathogens. It has teleomorphs in the genus Botryotina.'),
  'D020172': NtO(MH='Verticillium', MS='A mitosporic fungal genus commonly isolated from soil. Some species are the cause of wilt diseases in many different plants.'),
  'D020176': NtO(MH='Tyrosinemias', MS='A group of disorders which have in common elevations of tyrosine in the blood and urine secondary to an enzyme deficiency. Type I tyrosinemia features episodic weakness, self-mutilation, hepatic necrosis, renal tubular injury, and seizures and is caused by a deficiency of the enzyme fumarylacetoacetase. Type II tyrosinemia features INTELLECTUAL DISABILITY, painful corneal ulcers, and keratoses of the palms and plantar surfaces and is caused by a deficiency of the enzyme TYROSINE TRANSAMINASE. Type III tyrosinemia features INTELLECTUAL DISABILITY and is caused by a deficiency of the enzyme 4-HYDROXYPHENYLPYRUVATE DIOXYGENASE. (Menkes, Textbook of Child Neurology, 5th ed, pp42-3)'),
  'D020177': NtO(MH='Idiopathic Hypersomnia', MS='A sleep disorder of central nervous system origin characterized by prolonged nocturnal sleep and periods of daytime drowsiness. Affected individuals experience difficulty with awakening in the morning and may have associated sleep drunkenness, automatic behaviors, and memory disturbances. This condition differs from narcolepsy in that daytime sleep periods are longer, there is no association with CATAPLEXY, and the multiple sleep latency onset test does not record sleep-onset rapid eye movement sleep. (From Chokroverty, Sleep Disorders Medicine, 1994, pp319-20; Psychiatry Clin Neurosci 1998 Apr:52(2):125-129)'),
  'D020178': NtO(MH='Sleep Disorders, Circadian Rhythm', MS='Dyssomnias associated with disruption of the normal 24 hour sleep wake cycle secondary to travel (e.g., JET LAG SYNDROME), shift work, or other causes.'),
  'D020179': NtO(MH='Jet Lag Syndrome', MS='A chronobiologic disorder resulting from rapid travel across a number of time zones, characterized by insomnia or hypersomnolence, fatigue, behavioral symptoms, headaches, and gastrointestinal disturbances. (From Cooper, Sleep, 1994, pp593-8)'),
  'D020180': NtO(MH='Heteroduplex Analysis', MS='A method of detecting gene mutation by mixing PCR-amplified mutant and wild-type DNA followed by denaturation and reannealing. The resultant products are resolved by gel electrophoresis, with single base substitutions detectable under optimal electrophoretic conditions and gel formulations. Large base pair mismatches may also be analyzed by using electron microscopy to visualize heteroduplex regions.'),
  'D020181': NtO(MH='Sleep Apnea, Obstructive', MS='A disorder characterized by recurrent apneas during sleep despite persistent respiratory efforts. It is due to upper airway obstruction. The respiratory pauses may induce HYPERCAPNIA or HYPOXIA. Cardiac arrhythmias and elevation of systemic and pulmonary arterial pressures may occur. Frequent partial arousals occur throughout sleep, resulting in relative SLEEP DEPRIVATION and daytime tiredness. Associated conditions include OBESITY; ACROMEGALY; MYXEDEMA; micrognathia; MYOTONIC DYSTROPHY; adenotonsilar dystrophy; and NEUROMUSCULAR DISEASES. (From Adams et al., Principles of Neurology, 6th ed, p395)'),
  'D020182': NtO(MH='Sleep Apnea, Central', MS='A condition associated with multiple episodes of sleep apnea which are distinguished from obstructive sleep apnea (SLEEP APNEA, OBSTRUCTIVE) by the complete cessation of efforts to breathe. This disorder is associated with dysfunction of central nervous system centers that regulate respiration.'),
  'D020183': NtO(MH='Nocturnal Paroxysmal Dystonia', MS='A parasomnia characterized by paroxysmal episodes of choreoathetotic, ballistic, dystonic movements, and semipurposeful activity. The episodes occur during non-rapid eye movement sleep and typically recur several times per night. (Neurology 1992 Jul;42(7 Suppl 6):61-67; Adams et al., Principles of Neurology, 6th ed, p391)'),
  'D020184': NtO(MH='Night Terrors', MS='A disorder characterized by incomplete arousals from sleep associated with behavior suggesting extreme fright. This condition primarily affects children and young adults and the individual generally has no recall of the event. Episodes tend to occur during stage III or IV. SOMNAMBULISM is frequently associated with this condition. (Adams et al., Principles of Neurology, 6th ed, p391)'),
  'D020186': NtO(MH='Sleep Bruxism', MS='A sleep disorder characterized by grinding and clenching of the teeth and forceful lateral or protrusive jaw movements. Sleep bruxism may be associated with TOOTH INJURIES; TEMPOROMANDIBULAR JOINT DISORDERS; sleep disturbances; and other conditions.'),
  'D020187': NtO(MH='REM Sleep Behavior Disorder', MS='A disorder characterized by episodes of vigorous and often violent motor activity during REM sleep (SLEEP, REM). The affected individual may inflict self injury or harm others, and is difficult to awaken from this condition. Episodes are usually followed by a vivid recollection of a dream that is consistent with the aggressive behavior. This condition primarily affects adult males. (From Adams et al., Principles of Neurology, 6th ed, p393)'),
  'D020188': NtO(MH='Sleep Paralysis', MS='A common condition characterized by transient partial or total paralysis of skeletal muscles and areflexia that occurs upon awakening from sleep or less often while falling asleep. Stimuli such as touch or sound may terminate the episode, which usually has a duration of seconds to minutes. This condition may occur in normal subjects or be associated with NARCOLEPSY; CATAPLEXY; and hypnagogic HALLUCINATIONS. The pathophysiology of this condition is closely related to the normal hypotonia that occur during REM sleep. (From Adv Neurol 1995;67:245-271)'),
  'D020189': NtO(MH='Nocturnal Myoclonus Syndrome', MS='Excessive periodic leg movements during sleep that cause micro-arousals and interfere with the maintenance of sleep. This condition induces a state of relative sleep deprivation which manifests as excessive daytime hypersomnolence. The movements are characterized by repetitive contractions of the tibialis anterior muscle, extension of the toe, and intermittent flexion of the hip, knee and ankle. (Adams et al., Principles of Neurology, 6th ed, p387)'),
  'D020190': NtO(MH='Myoclonic Epilepsy, Juvenile', MS='A disorder characterized by the onset of myoclonus in adolescence, a marked increase in the incidence of absence seizures (see EPILEPSY, ABSENCE), and generalized major motor seizures (see EPILEPSY, TONIC-CLONIC). The myoclonic episodes tend to occur shortly after awakening. Seizures tend to be aggravated by sleep deprivation and alcohol consumption. Hereditary and sporadic forms have been identified. (From Adams et al., Principles of Neurology, 6th ed, p323)'),
  'D020191': NtO(MH='Myoclonic Epilepsies, Progressive', MS='A heterogeneous group of primarily familial EPILEPSY disorders characterized by myoclonic seizures, tonic-clonic seizures, ataxia, progressive intellectual deterioration, and neuronal degeneration. These include LAFORA DISEASE; MERRF SYNDROME; NEURONAL CEROID-LIPOFUSCINOSIS; sialidosis (see MUCOLIPIDOSES), and UNVERRICHT-LUNDBORG SYNDROME.'),
  'D020192': NtO(MH='Lafora Disease', MS='A form of stimulus sensitive MYOCLONIC EPILEPSY inherited as an autosomal recessive condition. The most common presenting feature is a single seizure in the second decade of life. This is followed by progressive myoclonus, myoclonic seizures, tonic-clonic seizures, focal occipital seizures, intellectual decline, and severe motor and coordination impairments. Most affected individuals do not live past the age of 25 years. Concentric amyloid (Lafora) bodies are found in neurons, liver, skin, bone, and muscle (From Menkes, Textbook of Childhood Neurology, 5th ed, pp111-110).'),
  'D020194': NtO(MH='Unverricht-Lundborg Syndrome', MS='An autosomal recessive condition characterized by recurrent myoclonic and generalized seizures, ATAXIA, slowly progressive intellectual deterioration, DYSARTHRIA, and intention tremor. Myoclonic seizures are severe and continuous, and tend to be triggered by movement, stress, and sensory stimuli. The age of onset is between 8 and 13 years, and the condition is relatively frequent in the Baltic region, especially Finland. (From Menkes, Textbook of Child Neurology, 5th ed, pp109-110)'),
  'D020195': NtO(MH='Epilepsy, Reflex', MS='A subtype of epilepsy characterized by seizures that are consistently provoked by a certain specific stimulus. Auditory, visual, and somatosensory stimuli as well as the acts of writing, reading, eating, and decision making are examples of events or activities that may induce seizure activity in affected individuals. (From Neurol Clin 1994 Feb;12(1):57-8)'),
  'D020196': NtO(MH='Trauma, Nervous System', MS='Traumatic injuries to the brain, cranial nerves, spinal cord, autonomic nervous system, or neuromuscular system, including iatrogenic injuries induced by surgical procedures.'),
  'D020197': NtO(MH='Head Injuries, Penetrating', MS='Head injuries which feature compromise of the skull and dura mater. These may result from gunshot wounds (WOUNDS, GUNSHOT), stab wounds (WOUNDS, STAB), and other forms of trauma.'),
  'D020198': NtO(MH='Intracranial Hemorrhage, Traumatic', MS='Bleeding within the SKULL induced by penetrating and nonpenetrating traumatic injuries, including hemorrhages into the tissues of CEREBRUM; BRAIN STEM; and CEREBELLUM; as well as into the epidural, subdural and subarachnoid spaces of the MENINGES.'),
  'D020199': NtO(MH='Hematoma, Subdural, Acute', MS='Accumulation of blood in the SUBDURAL SPACE with acute onset of neurological symptoms. Symptoms may include loss of consciousness, severe HEADACHE, and deteriorating mental status.'),
  'D020200': NtO(MH='Hematoma, Subdural, Chronic', MS='Accumulation of blood in the SUBDURAL SPACE with delayed onset of neurological symptoms. Symptoms may include loss of consciousness, severe HEADACHE, and deteriorating mental status.'),
  'D020201': NtO(MH='Brain Hemorrhage, Traumatic', MS='Bleeding within the brain as a result of penetrating and nonpenetrating CRANIOCEREBRAL TRAUMA. Traumatically induced hemorrhages may occur in any area of the brain, including the CEREBRUM; BRAIN STEM (see BRAIN STEM HEMORRHAGE, TRAUMATIC); and CEREBELLUM.'),
  'D020202': NtO(MH='Cerebral Hemorrhage, Traumatic', MS='Bleeding into one or both CEREBRAL HEMISPHERES due to TRAUMA. Hemorrhage may involve any part of the CEREBRAL CORTEX and the BASAL GANGLIA. Depending on the severity of bleeding, clinical features may include SEIZURES; APHASIA; VISION DISORDERS; MOVEMENT DISORDERS; PARALYSIS; and COMA.'),
  'D020203': NtO(MH='Brain Stem Hemorrhage, Traumatic', MS='Bleeding into structures of BRAIN STEM, including the MIDBRAIN; PONS; or MEDULLA OBLONGATA, as the result of CRANIOCEREBRAL TRAUMA. DIFFUSE AXONAL INJURY is commonly associated. Clinical manifestations may include OCULAR MOTILITY DISORDERS; ATAXIA; PARALYSIS; PERSISTENT VEGETATIVE STATE; and COMA.'),
  'D020204': NtO(MH='Skull Fracture, Depressed', MS='A skull fracture characterized by inward depression of a fragment or section of cranial bone, often compressing the underlying dura mater and brain. Depressed cranial fractures which feature open skin wounds that communicate with skull fragments are referred to as compound depressed skull fractures.'),
  'D020205': NtO(MH='Skull Fracture, Basilar', MS="Fractures which extend through the base of the SKULL, usually involving the PETROUS BONE. Battle's sign (characterized by skin discoloration due to extravasation of blood into the subcutaneous tissue behind the ear and over the mastoid process), CRANIAL NEUROPATHIES, TRAUMATIC; CAROTID-CAVERNOUS SINUS FISTULA; and CEREBROSPINAL FLUID OTORRHEA are relatively frequent sequelae of this condition. (Adams et al., Principles of Neurology, 6th ed, p876)"),
  'D020206': NtO(MH='Subarachnoid Hemorrhage, Traumatic', MS='Bleeding into the SUBARACHNOID SPACE due to CRANIOCEREBRAL TRAUMA. Minor hemorrhages may be asymptomatic; moderate to severe hemorrhages may be associated with INTRACRANIAL HYPERTENSION and VASOSPASM, INTRACRANIAL.'),
  'D020207': NtO(MH='Coma, Post-Head Injury', MS='Prolonged unconsciousness from which the individual cannot be aroused, associated with traumatic injuries to the BRAIN. This may be defined as unconsciousness persisting for 6 hours or longer. Coma results from injury to both cerebral hemispheres or the RETICULAR FORMATION of the BRAIN STEM. Contributing mechanisms include DIFFUSE AXONAL INJURY and BRAIN EDEMA. (From J Neurotrauma 1997 Oct;14(10):699-713)'),
  'D020208': NtO(MH='Brain Injury, Chronic', MS='Conditions characterized by persistent brain damage or dysfunction as sequelae of cranial trauma. This disorder may result from DIFFUSE AXONAL INJURY; INTRACRANIAL HEMORRHAGES; BRAIN EDEMA; and other conditions. Clinical features may include DEMENTIA; focal neurologic deficits; PERSISTENT VEGETATIVE STATE; AKINETIC MUTISM; or COMA.'),
  'D020209': NtO(MH='Cranial Nerve Injuries', MS='Dysfunction of one or more cranial nerves causally related to a traumatic injury. Penetrating and nonpenetrating CRANIOCEREBRAL TRAUMA; NECK INJURIES; and trauma to the facial region are conditions associated with cranial nerve injuries.'),
  'D020210': NtO(MH='Central Cord Syndrome', MS='A syndrome associated with traumatic injury to the cervical or upper thoracic regions of the spinal cord characterized by weakness in the arms with relative sparing of the legs and variable sensory loss. This condition is associated with ischemia, hemorrhage, or necrosis involving the central portions of the spinal cord. Corticospinal fibers destined for the legs are spared due to their more external location in the spinal cord. This clinical pattern may emerge during recovery from spinal shock. Deficits may be transient or permanent.'),
  'D020211': NtO(MH='Autonomic Dysreflexia', MS='A syndrome associated with damage to the spinal cord above the mid thoracic level (see SPINAL CORD INJURIES) characterized by a marked increase in the sympathetic response to minor stimuli such as bladder or rectal distention. Manifestations include HYPERTENSION; TACHYCARDIA (or reflex bradycardia); FEVER; FLUSHING; and HYPERHIDROSIS. Extreme hypertension may be associated with a STROKE. (From Adams et al., Principles of Neurology, 6th ed, pp538 and 1232; J Spinal Cord Med 1997;20(3):355-60)'),
  'D020212': NtO(MH='Carotid Artery Injuries', MS='Damages to the CAROTID ARTERIES caused either by blunt force or penetrating trauma, such as CRANIOCEREBRAL TRAUMA; THORACIC INJURIES; and NECK INJURIES. Damaged carotid arteries can lead to CAROTID ARTERY THROMBOSIS; CAROTID-CAVERNOUS SINUS FISTULA; pseudoaneurysm formation; and INTERNAL CAROTID ARTERY DISSECTION. (From Am J Forensic Med Pathol 1997, 18:251; J Trauma 1994, 37:473)'),
  'D020214': NtO(MH='Cerebrovascular Trauma', MS='Penetrating and nonpenetrating traumatic injuries to an extracranial or intracranial blood vessel that supplies the brain. This includes the CAROTID ARTERIES; VERTEBRAL ARTERIES; MENINGEAL ARTERIES; CEREBRAL ARTERIES; veins, and venous sinuses.'),
  'D020215': NtO(MH='Carotid Artery, Internal, Dissection', MS='The splitting of the vessel wall in one or both (left and right) internal carotid arteries (CAROTID ARTERY, INTERNAL). Interstitial hemorrhage into the media of the vessel wall can lead to occlusion of the internal carotid artery and aneurysm formation.'),
  'D020216': NtO(MH='Carotid-Cavernous Sinus Fistula', MS='An acquired or spontaneous abnormality in which there is communication between CAVERNOUS SINUS, a venous structure, and the CAROTID ARTERIES. It is often associated with HEAD TRAUMA, specifically basilar skull fractures (SKULL FRACTURE, BASILAR). Clinical signs often include VISION DISORDERS and INTRACRANIAL HYPERTENSION.'),
  'D020217': NtO(MH='Vertebral Artery Dissection', MS='Splitting of the vessel wall in the VERTEBRAL ARTERY. Interstitial hemorrhage into the media of the vessel wall can lead to occlusion of the vertebral artery, aneurysm formation, or THROMBOEMBOLISM. Vertebral artery dissection is often associated with TRAUMA and injuries to the head-neck region but can occur spontaneously.'),
  'D020218': NtO(MH='Response Elements', MS='Nucleotide sequences, usually upstream, which are recognized by specific regulatory transcription factors, thereby causing gene response to various regulatory agents. These elements may be found in both promoter and enhancer regions.'),
  'D020219': NtO(MH='Chondrogenesis', MS='The formation of cartilage. This process is directed by CHONDROCYTES which continually divide and lay down matrix during development. It is sometimes a precursor to OSTEOGENESIS.'),
  'D020220': NtO(MH='Facial Nerve Injuries', MS='Traumatic injuries to the facial nerve. This may result in FACIAL PARALYSIS, decreased lacrimation and salivation, and loss of taste sensation in the anterior tongue. The nerve may regenerate and reform its original pattern of innervation, or regenerate aberrantly, resulting in inappropriate lacrimation in response to gustatory stimuli (e.g., "crocodile tears") and other syndromes.'),
  'D020221': NtO(MH='Optic Nerve Injuries', MS='Injuries to the optic nerve induced by a trauma to the face or head. These may occur with closed or penetrating injuries. Relatively minor compression of the superior aspect of orbit may also result in trauma to the optic nerve. Clinical manifestations may include visual loss, PAPILLEDEMA, and an afferent pupillary defect.'),
  'D020222': NtO(MH='Abducens Nerve Injury', MS='Traumatic injury to the abducens, or sixth, cranial nerve. Injury to this nerve results in lateral rectus muscle weakness or paralysis. The nerve may be damaged by closed or penetrating CRANIOCEREBRAL TRAUMA or by facial trauma involving the orbit.'),
  'D020223': NtO(MH='Chromosome Painting', MS='A technique for visualizing CHROMOSOME ABERRATIONS using fluorescently labeled DNA probes which are hybridized to chromosomal DNA. Multiple fluorochromes may be attached to the probes. Upon hybridization, this produces a multicolored, or painted, effect with a unique color at each site of hybridization. This technique may also be used to identify cross-species homology by labeling probes from one species for hybridization with chromosomes from another species.'),
  'D020224': NtO(MH='Expressed Sequence Tags', MS='Partial cDNA (DNA, COMPLEMENTARY) sequences that are unique to the cDNAs from which they were derived.'),
  'D020225': NtO(MH='Sagittal Sinus Thrombosis', MS='Formation or presence of a blood clot (THROMBUS) in the SUPERIOR SAGITTAL SINUS or the inferior sagittal sinus. Sagittal sinus thrombosis can result from infections, hematological disorders, CRANIOCEREBRAL TRAUMA; and NEUROSURGICAL PROCEDURES. Clinical features are primarily related to the increased intracranial pressure causing HEADACHE; NAUSEA; and VOMITING. Severe cases can evolve to SEIZURES or COMA.'),
  'D020226': NtO(MH='Cavernous Sinus Thrombosis', MS='Formation or presence of a blood clot (THROMBUS) in the CAVERNOUS SINUS of the brain. Infections of the paranasal sinuses and adjacent structures, CRANIOCEREBRAL TRAUMA, and THROMBOPHILIA are associated conditions. Clinical manifestations include dysfunction of cranial nerves III, IV, V, and VI, marked periorbital swelling, chemosis, fever, and visual loss. (From Adams et al., Principles of Neurology, 6th ed, p711)'),
  'D020227': NtO(MH='Lateral Sinus Thrombosis', MS='Formation or presence of a blood clot (THROMBUS) in the LATERAL SINUSES. This condition is often associated with ear infections (OTITIS MEDIA or MASTOIDITIS) without antibiotic treatment. In developed nations, lateral sinus thrombosis can result from CRANIOCEREBRAL TRAUMA; BRAIN NEOPLASMS; NEUROSURGICAL PROCEDURES; THROMBOPHILIA; and other conditions. Clinical features include HEADACHE; VERTIGO; and increased intracranial pressure.'),
  'D020228': NtO(MH='Pharmacologic Actions', MS='A broad category of chemical actions and uses that result in the prevention, treatment, cure or diagnosis of disease. Included here are drugs and chemicals that act by altering normal body functions, such as the REPRODUCTIVE CONTROL AGENTS and ANESTHETICS. Effects of chemicals on the environment are also included.'),
  'D020230': NtO(MH='Serotonin Syndrome', MS='An adverse drug interaction characterized by altered mental status, autonomic dysfunction, and neuromuscular abnormalities. It is most frequently caused by use of both serotonin reuptake inhibitors and monoamine oxidase inhibitors, leading to excess serotonin availability in the CNS at the serotonin 1A receptor.'),
  'D020231': NtO(MH='Colletotrichum', MS='A genus of mitosporic Phyllachoraceae fungi which contains at least 40 species of plant parasites. They have teleomorphs in the genus Glomerella (see PHYLLACHORALES).'),
  'D020232': NtO(MH='Kluver-Bucy Syndrome', MS='A neurobehavioral syndrome associated with bilateral medial temporal lobe dysfunction. Clinical manifestations include oral exploratory behavior; tactile exploratory behavior; hypersexuality; BULIMIA; MEMORY DISORDERS; placidity; and an inability to recognize objects or faces. This disorder may result from a variety of conditions, including CRANIOCEREBRAL TRAUMA; infections; ALZHEIMER DISEASE; PICK DISEASE OF THE BRAIN; and CEREBROVASCULAR DISORDERS.'),
  'D020233': NtO(MH='Gait Disorders, Neurologic', MS='Gait abnormalities that are a manifestation of nervous system dysfunction. These conditions may be caused by a wide variety of disorders which affect motor control, sensory feedback, and muscle strength including: CENTRAL NERVOUS SYSTEM DISEASES; PERIPHERAL NERVOUS SYSTEM DISEASES; NEUROMUSCULAR DISEASES; or MUSCULAR DISEASES.'),
  'D020234': NtO(MH='Gait Ataxia', MS='Impairment of the ability to coordinate the movements required for normal ambulation (WALKING) which may result from impairments of motor function or sensory feedback. This condition may be associated with BRAIN DISEASES (including CEREBELLAR DISEASES and BASAL GANGLIA DISEASES); SPINAL CORD DISEASES; or PERIPHERAL NERVOUS SYSTEM DISEASES.'),
  'D020235': NtO(MH='Gait Apraxia', MS='Impaired ambulation not attributed to sensory impairment or motor weakness. FRONTAL LOBE disorders; BASAL GANGLIA DISEASES (e.g., PARKINSONIAN DISORDERS); DEMENTIA, MULTI-INFARCT; ALZHEIMER DISEASE; and other conditions may be associated with gait apraxia.'),
  'D020236': NtO(MH='Amnesia, Transient Global', MS='A syndrome characterized by a transient loss of the ability to form new memories. It primarily occurs in middle aged or elderly individuals, and episodes may last from minutes to hours. During the period of amnesia, immediate and recent memory abilities are impaired, but the level of consciousness and ability to perform other intellectual tasks are preserved. The condition is related to bilateral dysfunction of the medial portions of each TEMPORAL LOBE. Complete recovery normally occurs, and recurrences are unusual. (From Adams et al., Principles of Neurology, 6th ed, pp429-30)'),
  'D020237': NtO(MH='Alexia, Pure', MS='Loss of the power to comprehend written materials despite preservation of the ability to write (i.e., alexia without agraphia). This condition is generally attributed to lesions that "disconnect" the visual cortex of the non-dominant hemisphere from language centers in the dominant hemisphere. This may occur when a dominant visual cortex injury is combined with underlying white matter lesions that involve crossing fibers from the occipital lobe of the opposite hemisphere. (From Adams et al., Principles of Neurology, 6th ed, p483)'),
  'D020238': NtO(MH='Prosopagnosia', MS='The inability to recognize a familiar face or to learn to recognize new faces. This visual agnosia is most often associated with lesions involving the junctional regions between the temporal and occipital lobes. The majority of cases are associated with bilateral lesions, however unilateral damage to the right occipito-temporal cortex has also been associated with this condition. (From Cortex 1995 Jun;31(2):317-29)'),
  'D020239': NtO(MH='Receptor Cross-Talk', MS='The simultaneous or sequential binding of multiple cell surface receptors to different ligands resulting in coordinated stimulation or suppression of signal transduction.'),
  'D020240': NtO(MH='Apraxia, Ideomotor', MS='A form of apraxia characterized by an acquired inability to carry out a complex motor activity despite the ability to mentally formulate the action. This condition has been attributed to a disruption of connections between the dominant parietal cortex and supplementary and premotor cortical regions in both hemispheres. (From Adams et al., Principles of Neurology, 6th ed, p57)'),
  'D020241': NtO(MH='Encephalomyelitis, Western Equine', MS='A form of arboviral encephalitis (which primarily affects horses) endemic to western and central regions of NORTH AMERICA. The causative organism (ENCEPHALOMYELITIS VIRUS, WESTERN EQUINE) may be transferred to humans via the bite of mosquitoes (CULEX tarsalis and others). Clinical manifestations include headache and influenza-like symptoms followed by alterations in mentation, SEIZURES, and COMA. DEATH occurs in a minority of cases. Survivors may recover fully or be left with residual neurologic dysfunction, including PARKINSONISM, POSTENCEPHALITIC. (From Joynt, Clinical Neurology, 1996, Ch26, pp8-9)'),
  'D020242': NtO(MH='Encephalomyelitis, Eastern Equine', MS='A form of arboviral encephalitis (primarily affecting equines) endemic to eastern regions of North America. The causative organism (ENCEPHALOMYELITIS VIRUS, EASTERN EQUINE) may be transmitted to humans via the bite of AEDES mosquitoes. Clinical manifestations include the acute onset of fever, HEADACHE, altered mentation, and SEIZURES followed by coma. The condition is fatal in up to 50% of cases. Recovery may be marked by residual neurologic deficits and EPILEPSY. (From Joynt, Clinical Neurology, 1996, Ch26, pp9-10)'),
  'D020243': NtO(MH='Infarction, Anterior Cerebral Artery', MS="NECROSIS occurring in the ANTERIOR CEREBRAL ARTERY system, including branches such as Heubner's artery. These arteries supply blood to the medial and superior parts of the CEREBRAL HEMISPHERE, Infarction in the anterior cerebral artery usually results in sensory and motor impairment in the lower body."),
  'D020244': NtO(MH='Infarction, Middle Cerebral Artery', MS='NECROSIS occurring in the MIDDLE CEREBRAL ARTERY distribution system which brings blood to the entire lateral aspects of each CEREBRAL HEMISPHERE. Clinical signs include impaired cognition; APHASIA; AGRAPHIA; weak and numbness in the face and arms, contralaterally or bilaterally depending on the infarction.'),
  'D020245': NtO(MH='p-Chloromercuribenzoic Acid', MS='An organic mercurial used as a sulfhydryl reagent.'),
  'D020246': NtO(MH='Venous Thrombosis', MS='The formation or presence of a blood clot (THROMBUS) within a vein.'),
  'D020247': NtO(MH='Myofunctional Therapy', MS='Training or retraining of the buccal, facial, labial, and lingual musculature in toothless conditions; DEGLUTITION DISORDERS; TEMPOROMANDIBULAR JOINT DISORDERS; MALOCCLUSION; and ARTICULATION DISORDERS.'),
  'D020249': NtO(MH='Hormone Replacement Therapy', MS='Therapeutic use of hormones to alleviate the effects of hormone deficiency.'),
  'D020250': NtO(MH='Postoperative Nausea and Vomiting', MS='Emesis and queasiness occurring after anesthesia.'),
  'D020252': NtO(MH='Gastric Antral Vascular Ectasia', MS='A distinct vascular lesion in the PYLORIC ANTRUM that is characterized by tortuous dilated blood vessels (ectasia) radiating outward from the PYLORUS. The vessel pattern resembles the stripes on the surface of a watermelon. This lesion causes both acute and chronic GASTROINTESTINAL HEMORRHAGE.'),
  'D020254': NtO(MH='Tooth Ankylosis', MS="Solid fixation of a tooth resulting from fusion of the cementum and alveolar bone, with obliteration of the periodontal ligament. It is uncommon in the deciduous dentition and very rare in permanent teeth. (Jablonski's Dictionary of Dentistry, 1992)"),
  'D020255': NtO(MH='Vitreous Detachment', MS='Detachment of the corpus vitreum (VITREOUS BODY) from its normal attachments, especially the retina, due to shrinkage from degenerative or inflammatory conditions, trauma, myopia, or senility.'),
  'D020256': NtO(MH='Choroidal Neovascularization', MS='A pathological process consisting of the formation of new blood vessels in the CHOROID.'),
  'D020257': NtO(MH='Ventricular Remodeling', MS='The geometric and structural changes that the HEART VENTRICLES undergo, usually following MYOCARDIAL INFARCTION. It comprises expansion of the infarct and dilatation of the healthy ventricle segments. While most prevalent in the left ventricle, it can also occur in the right ventricle.'),
  'D020258': NtO(MH='Neurotoxicity Syndromes', MS='Neurologic disorders caused by exposure to toxic substances through ingestion, injection, cutaneous application, or other method. This includes conditions caused by biologic, chemical, and pharmaceutical agents.'),
  'D020260': NtO(MH='Heavy Metal Poisoning, Nervous System', MS='Conditions associated with damage or dysfunction of the nervous system caused by exposure to heavy metals, which may cause a variety of central, peripheral, or autonomic nervous system injuries.'),
  'D020261': NtO(MH='Arsenic Poisoning', MS='Disorders associated with acute or chronic exposure to compounds containing ARSENIC (ARSENICALS) which may be fatal. Acute oral ingestion is associated with gastrointestinal symptoms and an encephalopathy which may manifest as SEIZURES, mental status changes, and COMA. Chronic exposure is associated with mucosal irritation, desquamating rash, myalgias, peripheral neuropathy, and white transverse (Mees) lines in the fingernails. (Adams et al., Principles of Neurology, 6th ed, p1212)'),
  'D020262': NtO(MH='Mercury Poisoning, Nervous System', MS='Neurologic disorders associated with exposure to inorganic and organic forms of MERCURY. Acute intoxication may be associated with gastrointestinal disturbances, mental status changes, and PARAPARESIS. Chronic exposure to inorganic mercury usually occurs in industrial workers, and manifests as mental confusion, prominent behavioral changes (including psychosis), DYSKINESIAS, and NEURITIS. Alkyl mercury poisoning may occur through ingestion of contaminated seafood or grain, and its characteristic features include POLYNEUROPATHY; ATAXIA; vision loss; NYSTAGMUS, PATHOLOGIC; and DEAFNESS. (From Joynt, Clinical Neurology, 1997, Ch20, pp10-15)'),
  'D020263': NtO(MH='Lead Poisoning, Nervous System', MS='Injury to the nervous system secondary to exposure to lead compounds. Two distinct clinical patterns occur in children (LEAD POISONING, NERVOUS SYSTEM, CHILDHOOD) and adults (LEAD POISONING, NERVOUS SYSTEM, ADULT). In children, lead poisoning typically produces an encephalopathy. In adults, exposure to toxic levels of lead is associated with a peripheral neuropathy.'),
  'D020264': NtO(MH='Lead Poisoning, Nervous System, Childhood', MS='Neurologic disorders occurring in children following lead exposure. The most frequent manifestation of childhood lead toxicity is an encephalopathy associated with chronic ingestion of lead that usually presents between the ages of 1 and 3 years. Clinical manifestations include behavioral changes followed by lethargy; CONVULSIONS; HALLUCINATIONS; DELIRIUM; ATAXIA; and vomiting. Elevated intracranial pressure (HYPERTENSION, INTRACRANIAL) and CEREBRAL EDEMA may occur. (From Adams et al., Principles of Neurology, 6th ed, p1210-2)'),
  'D020265': NtO(MH='Lead Poisoning, Nervous System, Adult', MS='Neurologic conditions in adults associated with acute or chronic exposure to lead or any of its salts. The most common lead related neurologic syndrome in adults consists of a polyneuropathy involving motor fibers. This tends to affect distal nerves and may present as wrist drop due to RADIAL NEUROPATHY. Additional features of chronic lead exposure include ANEMIA; CONSTIPATION; colicky abdominal pain; a bluish lead line of the gums; interstitial nephritis (NEPHRITIS, INTERSTITIAL); and saturnine gout. An encephalopathy may rarely occur. (From Adams et al., Principles of Neurology, 6th ed, p1212)'),
  'D020266': NtO(MH='Radiotherapy, Conformal', MS='A therapy using IONIZING RADIATION where there is improved dose homogeneity within the tumor and reduced dosage to uninvolved structures. The precise shaping of dose distribution is achieved via the use of computer-controlled multileaf collimators.'),
  'D020267': NtO(MH='MPTP Poisoning', MS='A condition caused by the neurotoxin MPTP which causes selective destruction of nigrostriatal dopaminergic neurons. Clinical features include irreversible parkinsonian signs including rigidity and bradykinesia (PARKINSON DISEASE, SECONDARY). MPTP toxicity is also used as an animal model for the study of PARKINSON DISEASE. (Adams et al., Principles of Neurology, 6th ed, p1072; Neurology 1986 Feb;36(2):250-8)'),
  'D020268': NtO(MH='Alcohol-Induced Disorders, Nervous System', MS='Acute and chronic neurologic disorders associated with the various neurologic effects of ETHANOL. Primary sites of injury include the brain and peripheral nerves.'),
  'D020269': NtO(MH='Alcoholic Neuropathy', MS='A condition where damage to the peripheral nervous system (including the peripheral elements of the autonomic nervous system) is associated with chronic ingestion of alcoholic beverages. The disorder may be caused by a direct effect of alcohol, an associated nutritional deficiency, or a combination of factors. Clinical manifestations include variable degrees of weakness; ATROPHY; PARESTHESIAS; pain; loss of reflexes; sensory loss; diaphoresis; and postural hypotension. (From Arch Neurol 1995;52(1):45-51; Adams et al., Principles of Neurology, 6th ed, p1146)'),
  'D020270': NtO(MH='Alcohol Withdrawal Seizures', MS='A condition where seizures occur in association with ethanol abuse (ALCOHOLISM) without other identifiable causes. Seizures usually occur within the first 6-48 hours after the cessation of alcohol intake, but may occur during periods of alcohol intoxication. Single generalized tonic-clonic motor seizures are the most common subtype, however, STATUS EPILEPTICUS may occur. (Adams et al., Principles of Neurology, 6th ed, p1174)'),
  'D020271': NtO(MH='Heredodegenerative Disorders, Nervous System', MS='Inherited disorders characterized by progressive atrophy and dysfunction of anatomically or physiologically related neurologic systems.'),
  'D020273': NtO(MH='Orbital Implants', MS='Rounded objects made of coral, teflon, or alloplastic polymer and covered with sclera, and which are implanted in the orbit following enucleation. An artificial eye (EYE, ARTIFICIAL) is usually attached to the anterior of the orbital implant for cosmetic purposes.'),
  'D020274': NtO(MH='Autoimmune Diseases of the Nervous System', MS='Disorders caused by cellular or humoral immune responses primarily directed towards nervous system autoantigens. The immune response may be directed towards specific tissue components (e.g., myelin) and may be limited to the central nervous system (e.g., MULTIPLE SCLEROSIS) or the peripheral nervous system (e.g., GUILLAIN-BARRE SYNDROME).'),
  'D020275': NtO(MH='Guillain-Barre Syndrome', MS='An acute inflammatory autoimmune neuritis caused by T cell- mediated cellular immune response directed towards peripheral myelin. Demyelination occurs in peripheral nerves and nerve roots. The process is often preceded by a viral or bacterial infection, surgery, immunization, lymphoma, or exposure to toxins. Common clinical manifestations include progressive weakness, loss of sensation, and loss of deep tendon reflexes. Weakness of respiratory muscles and autonomic dysfunction may occur. (From Adams et al., Principles of Neurology, 6th ed, pp1312-1314)'),
  'D020276': NtO(MH='HIV Long-Term Survivors', MS='Persons who have experienced prolonged survival of HIV infection. This includes the full spectrum of untreated, HIV-infected long-term asymptomatics to those with AIDS who have survived due to successful treatment.'),
  'D020277': NtO(MH='Polyradiculoneuropathy, Chronic Inflammatory Demyelinating', MS='A slowly progressive autoimmune demyelinating disease of peripheral nerves and nerve roots. Clinical manifestations include weakness and sensory loss in the extremities and enlargement of peripheral nerves. The course may be relapsing-remitting or demonstrate a step-wise progression. Protein is usually elevated in the spinal fluid and cranial nerves are typically spared. GUILLAIN-BARRE SYNDROME features a relatively rapid progression of disease which distinguishes it from this condition. (Adams et al., Principles of Neurology, 6th ed, p1337)'),
  'D020278': NtO(MH='Demyelinating Autoimmune Diseases, CNS', MS='Conditions characterized by loss or dysfunction of myelin (see MYELIN SHEATH) in the brain, spinal cord, or optic nerves secondary to autoimmune mediated processes. This may take the form of a humoral or cellular immune response directed toward myelin or OLIGODENDROGLIA associated autoantigens.'),
  'D020279': NtO(MH='Hereditary Central Nervous System Demyelinating Diseases', MS='Inherited conditions characterized by a loss of MYELIN in the central nervous system.'),
  'D020280': NtO(MH='Sertraline', MS='A selective serotonin uptake inhibitor that is used in the treatment of depression.'),
  'D020283': NtO(MH='Photoreceptors, Microbial', MS='Light absorbing proteins and protein prosthetic groups found in certain microorganisms. Some microbial photoreceptors initiate specific chemical reactions which signal a change in the environment, while others generate energy by pumping specific ions across a cellular membrane.'),
  'D020285': NtO(MH='Cryoelectron Microscopy', MS='Electron microscopy involving rapid freezing of the samples. The imaging of frozen-hydrated molecules and organelles permits the best possible resolution closest to the living state, free of chemical fixatives or stains.'),
  'D020286': NtO(MH='Pericytes', MS='Unique slender cells with multiple processes extending along the capillary vessel axis and encircling the vascular wall, also called mural cells. Pericytes are imbedded in the BASEMENT MEMBRANE shared with the ENDOTHELIAL CELLS of the vessel. Pericytes are important in maintaining vessel integrity, angiogenesis, and vascular remodeling.'),
  'D020287': NtO(MH='In Situ Nick-End Labeling', MS='An in situ method for detecting areas of DNA which are nicked during APOPTOSIS. Terminal deoxynucleotidyl transferase is used to add labeled dUTP, in a template-independent manner, to the 3 prime OH ends of either single- or double-stranded DNA. The terminal deoxynucleotidyl transferase nick end labeling, or TUNEL, assay labels apoptosis on a single-cell level, making it more sensitive than agarose gel electrophoresis for analysis of DNA FRAGMENTATION.'),
  'D020288': NtO(MH='Papilloma, Choroid Plexus', MS='A usually benign neoplasm that arises from the cuboidal epithelium of the choroid plexus and takes the form of an enlarged CHOROID PLEXUS, which may be associated with oversecretion of CSF. The tumor usually presents in the first decade of life with signs of increased intracranial pressure including HEADACHES; ATAXIA; DIPLOPIA; and alterations of mental status. In children it is most common in the lateral ventricles and in adults it tends to arise in the fourth ventricle. Malignant transformation to choroid plexus carcinomas may rarely occur. (Adams et al., Principles of Neurology, 6th ed, p667; DeVita et al., Cancer: Principles and Practice of Oncology, 5th ed, p2072)'),
  'D020293': NtO(MH='Vasculitis, Central Nervous System', MS='Inflammation of blood vessels within the central nervous system. Primary vasculitis is usually caused by autoimmune or idiopathic factors, while secondary vasculitis is caused by existing disease process. Clinical manifestations are highly variable but include HEADACHE; SEIZURES; behavioral alterations; INTRACRANIAL HEMORRHAGES; TRANSIENT ISCHEMIC ATTACK; and BRAIN INFARCTION. (From Adams et al., Principles of Neurology, 6th ed, pp856-61)'),
  'D020294': NtO(MH='Myasthenic Syndromes, Congenital', MS='A heterogeneous group of disorders characterized by a congenital defect in neuromuscular transmission at the NEUROMUSCULAR JUNCTION. This includes presynaptic, synaptic, and postsynaptic disorders (that are not of autoimmune origin). The majority of these diseases are caused by mutations of various subunits of the nicotinic acetylcholine receptor (RECEPTORS, NICOTINIC) on the postsynaptic surface of the junction. (From Arch Neurol 1999 Feb;56(2):163-7)'),
  'D020295': NtO(MH='Brain Stem Neoplasms', MS='Benign and malignant intra-axial tumors of the MESENCEPHALON; PONS; or MEDULLA OBLONGATA of the BRAIN STEM. Primary and metastatic neoplasms may occur in this location. Clinical features include ATAXIA, cranial neuropathies (see CRANIAL NERVE DISEASES), NAUSEA, hemiparesis (see HEMIPLEGIA), and quadriparesis. Primary brain stem neoplasms are more frequent in children. Histologic subtypes include GLIOMA; HEMANGIOBLASTOMA; GANGLIOGLIOMA; and EPENDYMOMA.'),
  'D020296': NtO(MH='Animals, Congenic', MS='Animals that are produced through selective breeding to eliminate genetic background differences except for a single or few specific loci.  They are used to investigate the contribution of genetic background differences to PHENOTYPE.'),
  'D020297': NtO(MH='Mice, Congenic', MS='Mouse strains constructed to possess identical genotypes except for a difference at a single gene locus.'),
  'D020298': NtO(MH='U937 Cells', MS='A human cell line established from a diffuse histiocytic lymphoma (HISTIOCYTIC LYMPHOMA, DIFFUSE) and displaying many monocytic characteristics. It serves as an in vitro model for MONOCYTE and MACROPHAGE differentiation.'),
  'D020299': NtO(MH='Intracranial Hemorrhage, Hypertensive', MS='Bleeding within the SKULL that is caused by systemic HYPERTENSION, usually in association with INTRACRANIAL ARTERIOSCLEROSIS. Hypertensive hemorrhages are most frequent in the BASAL GANGLIA; CEREBELLUM; PONS; and THALAMUS; but may also involve the CEREBRAL CORTEX, subcortical white matter, and other brain structures.'),
  'D020300': NtO(MH='Intracranial Hemorrhages', MS='Bleeding within the SKULL, including hemorrhages in the brain and the three membranes of MENINGES. The escape of blood often leads to the formation of HEMATOMA in the cranial epidural, subdural, and subarachnoid spaces.'),
  'D020301': NtO(MH='Vasospasm, Intracranial', MS='Constriction of arteries in the SKULL due to sudden, sharp, and often persistent smooth muscle contraction in blood vessels. Intracranial vasospasm results in reduced vessel lumen caliber, restricted blood flow to the brain, and BRAIN ISCHEMIA that may lead to hypoxic-ischemic brain injury (HYPOXIA-ISCHEMIA, BRAIN).'),
  'D020302': NtO(MH='Weibel-Palade Bodies', MS='Rod-shaped storage granules for VON WILLEBRAND FACTOR specific to endothelial cells.'),
  'D020303': NtO(MH='Rats, Inbred Dahl', MS='Inbred rats derived from Sprague-Dawley rats and used for the study of salt-dependent hypertension. Salt-sensitive and salt-resistant strains have been selectively bred to show the opposite genetically determined blood pressure responses to excess sodium chloride ingestion.'),
  'D020304': NtO(MH='Raptors', MS='BIRDS that hunt and kill other animals, especially higher vertebrates, for food. They include the FALCONIFORMES order, or diurnal birds of prey, comprised of EAGLES, falcons, HAWKS, and others, as well as the STRIGIFORMES order, or nocturnal birds of prey, which includes OWLS.'),
  'D020305': NtO(MH='Strigiformes', MS='An order of BIRDS with the common name owls characterized by strongly hooked beaks, sharp talons, large heads, forward facing eyes, and facial disks. While considered nocturnal RAPTORS, some owls do hunt by day.'),
  'D020306': NtO(MH='Tuberculosis, Central Nervous System', MS='Tuberculosis of the brain, spinal cord, or meninges (TUBERCULOSIS, MENINGEAL), most often caused by MYCOBACTERIUM TUBERCULOSIS and rarely by MYCOBACTERIUM BOVIS. The infection may be limited to the nervous system or coexist in other organs (e.g., TUBERCULOSIS, PULMONARY). The organism tends to seed the meninges causing a diffuse meningitis and leads to the formation of TUBERCULOMA, which may occur within the brain, spinal cord, or perimeningeal spaces. Tuberculous involvement of the vertebral column (TUBERCULOSIS, SPINAL) may result in nerve root or spinal cord compression. (From Adams et al., Principles of Neurology, 6th ed, pp717-20)'),
  'D020307': NtO(MH='Eagles', MS='Large members of the FALCONIFORMES order of birds, family Accipitridae, most especially the genera Aquila, Haliaeetus, Harpia, and Circaetus. They are characterized by their powerful talons, which carry long, curved, pointed claws and by their opposable hindtoe.'),
  'D020308': NtO(MH='Songbirds', MS='PASSERIFORMES of the suborder, Oscines, in which the flexor tendons of the toes are separate, and the lower syrinx has 4 to 9 pairs of tensor muscles inserted at both ends of the tracheal half rings. They include many commonly recognized birds such as CROWS; FINCHES; robins; SPARROWS; and SWALLOWS.'),
  'D020310': NtO(MH='Struthioniformes', MS='An order of flightless birds comprising the ostriches, which naturally inhabit open, low rainfall areas of Africa.'),
  'D020311': NtO(MH='Rheiformes', MS='An order of large, long-necked, long-legged, flightless birds, found in South America. Known as rheas, they are sometimes called American ostriches, though they are in a separate order from true OSTRICHES.'),
  'D020312': NtO(MH='Dromaiidae', MS='A family of flightless, running BIRDS, in the order Casuariiformes. The emu is the only surviving member of the family. They naturally inhabit forests, open plains, and grasslands in Australia.'),
  'D020313': NtO(MH='Specialty Uses of Chemicals', MS='Uses of chemicals in a research, industrial, or household setting. This does not include PHARMACOLOGIC ACTIONS.'),
  'D020314': NtO(MH='Central Nervous System Fungal Infections', MS='MYCOSES of the brain, spinal cord, and meninges which may result in ENCEPHALITIS; MENINGITIS, FUNGAL; MYELITIS; BRAIN ABSCESS; and EPIDURAL ABSCESS. Certain types of fungi may produce disease in immunologically normal hosts, while others are classified as opportunistic pathogens, causing illness primarily in immunocompromised individuals (e.g., ACQUIRED IMMUNODEFICIENCY SYNDROME).'),
  'D020315': NtO(MH='Latex Hypersensitivity', MS='Allergic reaction to products containing processed natural rubber latex such as rubber gloves, condoms, catheters, dental dams, balloons, and sporting equipment. Both T-cell mediated (HYPERSENSITIVITY, DELAYED) and IgE antibody-mediated (HYPERSENSITIVITY, IMMEDIATE) allergic responses are possible. Delayed hypersensitivity results from exposure to antioxidants present in the rubber; immediate hypersensitivity results from exposure to a latex protein.'),
  'D020317': NtO(MH='Salmo salar', MS='A commercially important species of SALMON in the family SALMONIDAE, order SALMONIFORMES, which occurs in the North Atlantic.'),
  'D020318': NtO(MH='Rats, Long-Evans', MS="An outbred strain of rats developed in 1915 by crossing several Wistar Institute white females with a wild gray male. Inbred strains have been derived from this original outbred strain, including Long-Evans cinnamon rats (RATS, INBRED LEC) and Otsuka-Long-Evans-Tokushima Fatty rats (RATS, INBRED OLETF), which are models for Wilson's disease and non-insulin dependent diabetes mellitus, respectively."),
  'D020319': NtO(MH='Oligodeoxyribonucleotides, Antisense', MS='Short fragments of DNA that are used to alter the function of target RNAs or DNAs to which they hybridize.'),
  'D020320': NtO(MH='Oligoribonucleotides, Antisense', MS='Short fragments of RNA that are used to alter the function of target RNAs or DNAs to which they hybridize.'),
  'D020323': NtO(MH='Tics', MS='Habitual, repeated, rapid contraction of certain muscles, resulting in stereotyped individualized actions that can be voluntarily suppressed for only brief periods. They often involve the face, vocal cords, neck, and less often the extremities. Examples include repetitive throat clearing, vocalizations, sniffing, pursing the lips, and excessive blinking. Tics tend to be aggravated by emotional stress. When frequent they may interfere with speech and INTERPERSONAL RELATIONS. Conditions which feature frequent and prominent tics as a primary manifestation of disease are referred to as TIC DISORDERS. (From Adams et al., Principles of Neurology, 6th ed, pp109-10)'),
  'D020324': NtO(MH='Amnesia, Anterograde', MS='Loss of the ability to form new memories beyond a certain point in time. This condition may be organic or psychogenic in origin. Organically induced anterograde amnesia may follow CRANIOCEREBRAL TRAUMA; SEIZURES; ANOXIA; and other conditions which adversely affect neural structures associated with memory formation (e.g., the HIPPOCAMPUS; FORNIX (BRAIN); MAMMILLARY BODIES; and ANTERIOR THALAMIC NUCLEI). (From Memory 1997 Jan-Mar;5(1-2):49-71)'),
  'D020325': NtO(MH='Migraine with Aura', MS='A subtype of migraine disorder, characterized by recurrent attacks of reversible neurological symptoms (aura) that precede or accompany the headache. Aura may include a combination of sensory disturbances, such as blurred VISION; HALLUCINATIONS; VERTIGO; NUMBNESS; and difficulty in concentrating and speaking. Aura is usually followed by features of the COMMON MIGRAINE, such as PHOTOPHOBIA; PHONOPHOBIA; and NAUSEA. (International Classification of Headache Disorders, 2nd ed. Cephalalgia 2004: suppl 1)'),
  'D020326': NtO(MH='Migraine without Aura', MS='Recurrent unilateral pulsatile headaches, not preceded or accompanied by an aura, in attacks lasting 4-72 hours. It is characterized by PAIN of moderate to severe intensity; aggravated by physical activity; and associated with NAUSEA and / or PHOTOPHOBIA and PHONOPHOBIA. (International Classification of Headache Disorders, 2nd ed. Cephalalgia 2004: suppl 1)'),
  'D020327': NtO(MH='Glaucoma Drainage Implants', MS='Devices, usually incorporating unidirectional valves, which are surgically inserted in the sclera to maintain normal intraocular pressure.'),
  'D020329': NtO(MH='Essential Tremor', MS='A relatively common disorder characterized by a fairly specific pattern of tremors which are most prominent in the upper extremities and neck, inducing titubations of the head. The tremor is usually mild, but when severe may be disabling. An autosomal dominant pattern of inheritance may occur in some families (i.e., familial tremor). (Mov Disord 1988;13(1):5-10)'),
  'D020330': NtO(MH='Bell Palsy', MS='A syndrome characterized by the acute onset of unilateral FACIAL PARALYSIS which progresses over a 2-5 day period. Weakness of the orbicularis oculi muscle and resulting incomplete eye closure may be associated with corneal injury. Pain behind the ear often precedes the onset of paralysis. This condition may be associated with HERPESVIRUS 1, HUMAN infection of the facial nerve. (Adams et al., Principles of Neurology, 6th ed, p1376)'),
  'D020331': NtO(MH='Mobius Syndrome', MS='A syndrome of congenital facial paralysis, frequently associated with abducens palsy and other congenital abnormalities including lingual palsy, clubfeet, brachial disorders, cognitive deficits, and pectoral muscle defects. Pathologic findings are variable and include brain stem nuclear aplasia, facial nerve aplasia, and facial muscle aplasia, consistent with a multifactorial etiology. (Adams et al., Principles of Neurology, 6th ed, p1020)'),
  'D020332': NtO(MH='Rats, Inbred LEC', MS="A cinnamon-colored strain of Long-Evans rats which carries a mutation causing fulminant hepatitis and jaundice, with an associated gross accumulation of copper in the liver. This strain is a model for Wilson's Disease (see HEPATOLENTICULAR DEGENERATION)."),
  'D020333': NtO(MH='Tolosa-Hunt Syndrome', MS='An idiopathic syndrome characterized by the formation of granulation tissue in the anterior cavernous sinus or superior orbital fissure, producing a painful ophthalmoplegia. (Adams et al., Principles of Neurology, 6th ed, p271)'),
  'D020334': NtO(MH='Rats, Inbred OLETF', MS='An inbred strain of Long-Evans rats that develops hyperglycemia, hyperinsulinemia, and mild obesity, mostly in males, that resembles non-insulin-dependent diabetes mellitus in humans. It was developed from outbred Long-Evans stock in 1983.'),
  'D020335': NtO(MH='Paraparesis', MS='Mild to moderate loss of bilateral lower extremity motor function, which may be a manifestation of SPINAL CORD DISEASES; PERIPHERAL NERVOUS SYSTEM DISEASES; MUSCULAR DISEASES; INTRACRANIAL HYPERTENSION; parasagittal brain lesions; and other conditions.'),
  'D020336': NtO(MH='Paraparesis, Spastic', MS='Mild or moderate loss of motor function accompanied by spasticity in the lower extremities. This condition is a manifestation of CENTRAL NERVOUS SYSTEM DISEASES that cause injury to the motor cortex or descending motor pathways.'),
  'D020338': NtO(MH='Vestibular Neuronitis', MS='Idiopathic inflammation of the VESTIBULAR NERVE, characterized clinically by the acute or subacute onset of VERTIGO; NAUSEA; and imbalance. The COCHLEAR NERVE is typically spared and HEARING LOSS and TINNITUS do not usually occur. Symptoms usually resolve over a period of days to weeks. (Adams et al., Principles of Neurology, 6th ed, p304)'),
  'D020339': NtO(MH='Optic Nerve Glioma', MS='Glial cell derived tumors arising from the optic nerve, usually presenting in childhood.'),
  'D020340': NtO(MH='Tobacco Use Cessation', MS='Ending the TOBACCO habits of smoking, chewing, or snuff use.'),
  'D020341': NtO(MH='Absorbable Implants', MS='Implants constructed of materials designed to be absorbed by the body without producing an immune response. They are usually composed of plastics and are frequently used in orthopedics and orthodontics.'),
  'D020342': NtO(MH='Finite Element Analysis', MS='A computer based method of simulating or analyzing the behavior of structures or components.'),
  'D020343': NtO(MH='Hypertensive Encephalopathy', MS='Brain dysfunction or damage resulting from sustained MALIGNANT HYPERTENSION. When BLOOD PRESSURE exceeds the limits of cerebral autoregulation, cerebral blood flow is impaired (BRAIN ISCHEMIA). Clinical manifestations include HEADACHE; NAUSEA; VOMITING; SEIZURES; altered mental status (in some cases progressing to COMA); PAPILLEDEMA; and RETINAL HEMORRHAGE.'),
  'D020345': NtO(MH='Enterocolitis, Necrotizing', MS='ENTEROCOLITIS with extensive ulceration (ULCER) and NECROSIS. It is observed primarily in LOW BIRTH WEIGHT INFANT.'),
  'D020346': NtO(MH='Aquaporins', MS='A class of porins that allow the passage of WATER and other small molecules across CELL MEMBRANES.'),
  'D020347': NtO(MH='Lithiasis', MS='A condition characterized by the formation of CALCULI and concretions in the hollow organs or ducts of the body. They occur most often in the gallbladder, kidney, and lower urinary tract.'),
  'D020349': NtO(MH='Surface Plasmon Resonance', MS="A biosensing technique in which biomolecules capable of binding to specific analytes or ligands are first immobilized on one side of a metallic film. Light is then focused on the opposite side of the film to excite the surface plasmons, that is, the oscillations of free electrons propagating along the film's surface. The refractive index of light reflecting off this surface is measured. When the immobilized biomolecules are bound by their ligands, an alteration in surface plasmons on the opposite side of the film is created which is directly proportional to the change in bound, or adsorbed, mass. Binding is measured by changes in the refractive index. The technique is used to study biomolecular interactions, such as antigen-antibody binding."),
  'D020350': NtO(MH='Graft vs Tumor Effect', MS='Immunological rejection of tumor tissue/cells following bone marrow transplantation.'),
  'D020355': NtO(MH='Cholates', MS='Salts and esters of CHOLIC ACID.'),
  'D020357': NtO(MH='Selenomonas', MS="Curved bacteria, usually crescent-shaped rods, with ends often tapered, occurring singly, in pairs, or short chains. They are non-encapsulated, non-sporing, motile, and ferment glucose. Selenomonas are found mainly in the human buccal cavity, the rumen of herbivores, and the cecum of pigs and several rodents. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020358': NtO(MH='Sodium Cholate', MS='A trihydroxy bile salt that is used as a digestive aid in dietary supplements. It is used in culture media and in conjunction with PAPAIN and PANCREATIN.'),
  'D020360': NtO(MH='Neoadjuvant Therapy', MS='Preliminary cancer therapy (chemotherapy, radiation therapy, hormone/endocrine therapy, immunotherapy, hyperthermia, etc.) that precedes a necessary second modality of treatment.'),
  'D020361': NtO(MH='Paraneoplastic Syndromes, Nervous System', MS='Degenerative or inflammatory conditions affecting the central or peripheral nervous system that develop in association with a systemic neoplasm without direct invasion by tumor. They may be associated with circulating antibodies that react with the affected neural tissue. (Intern Med 1996 Dec;35(12):925-9)'),
  'D020362': NtO(MH='Paraneoplastic Cerebellar Degeneration', MS='Cerebellar degeneration associated with a remote neoplasm. Clinical manifestations include progressive limb and GAIT ATAXIA; DYSARTHRIA; and NYSTAGMUS, PATHOLOGIC. The histologic type of the associated neoplasm is usually carcinoma or lymphoma. Pathologically the cerebellar cortex and subcortical nuclei demonstrate diffuse degenerative changes. Anti-Purkinje cell antibodies (anti-Yo) are found in the serum of approximately 50% of affected individuals. (Adams et al., Principles of Neurology, 6th ed, p686)'),
  'D020363': NtO(MH='Limbic Encephalitis', MS='A paraneoplastic syndrome marked by degeneration of neurons in the LIMBIC SYSTEM. Clinical features include HALLUCINATIONS, loss of EPISODIC MEMORY; ANOSMIA; AGEUSIA; TEMPORAL LOBE EPILEPSY; DEMENTIA; and affective disturbance (depression). Circulating anti-neuronal antibodies (e.g., anti-Hu; anti-Yo; anti-Ri; and anti-Ma2) and small cell lung carcinomas or testicular carcinoma are frequently associated with this syndrome.'),
  'D020364': NtO(MH='Paraneoplastic Polyneuropathy', MS='A diffuse or multifocal peripheral neuropathy related to the remote effects of a neoplasm, most often carcinoma or lymphoma. Pathologically, there are inflammatory changes in peripheral nerves. The most common clinical presentation is a symmetric distal mixed sensorimotor polyneuropathy. (Adams et al., Principles of Neurology, 6th ed, p1334)'),
  'D020365': NtO(MH='RNA Helicases', MS='A family of proteins that promote unwinding of RNA during splicing and translation.'),
  'D020366': NtO(MH='Methylmethacrylate', MS='The methyl ester of methacrylic acid. It polymerizes easily to form POLYMETHYL METHACRYLATE. It is used as a bone cement.'),
  'D020367': NtO(MH='Iofetamine', MS='An amphetamine analog that is rapidly taken up by the lungs and from there redistributed primarily to the brain and liver. It is used in brain radionuclide scanning with I-123.'),
  'D020369': NtO(MH='Bradyrhizobium', MS='A genus of gram-negative, aerobic, rod-shaped bacteria usually containing granules of poly-beta-hydroxybutyrate. They characteristically invade the root hairs of leguminous plants and act as intracellular symbionts.'),
  'D020370': NtO(MH='Osteoarthritis, Knee', MS="Noninflammatory degenerative disease of the knee joint consisting of three large categories: conditions that block normal synchronous movement, conditions that produce abnormal pathways of motion, and conditions that cause stress concentration resulting in changes to articular cartilage. (Crenshaw, Campbell's Operative Orthopaedics, 8th ed, p2019)"),
  'D020371': NtO(MH='Pelizaeus-Merzbacher Disease', MS='A rare, slowly progressive disorder of myelin formation. Subtypes are referred to as classic, congenital, transitional, and adult forms of this disease. The classic form is X-chromosome linked, has its onset in infancy and is associated with a mutation of the proteolipid protein gene. Clinical manifestations include TREMOR, spasmus nutans, roving eye movements, ATAXIA, spasticity, and NYSTAGMUS, CONGENITAL. Death occurs by the third decade of life. The congenital form has similar characteristics but presents early in infancy and features rapid disease progression. Transitional and adult subtypes have a later onset and less severe symptomatology. Pathologic features include patchy areas of demyelination with preservation of perivascular islands (trigoid appearance). (From Menkes, Textbook of Child Neurology, 5th ed, p190)'),
  'D020372': NtO(MH='Dexfenfluramine', MS='The S-isomer of FENFLURAMINE. It is a serotonin agonist and is used as an anorectic. Unlike fenfluramine, it does not possess any catecholamine agonist activity.'),
  'D020374': NtO(MH='Chromatography, Micellar Electrokinetic Capillary', MS='A hybrid separation technique combining both chromatographic and electrophoretic separation principles. While the method was invented to separate neutral species, it can also be applied to charged molecules such as small peptides.'),
  'D020375': NtO(MH='Education, Distance', MS='Education via communication media (correspondence, radio, television, computer networks) with little or no in-person face-to-face contact between students and teachers. (ERIC Thesaurus, 1997)'),
  'D020376': NtO(MH='Dichelobacter nodosus', MS='A gram-negative, obligate anaerobe of the family Cardiobacteriaceae. It has polar fimbriae and is the causative agent of FOOT ROT and DIGITAL DERMATITIS. It is the lone species in the genus Dichelobacter.'),
  'D020377': NtO(MH='Self Efficacy', MS="Cognitive mechanism based on expectations or beliefs about one's ability to perform actions necessary to produce a given effect. It is also a theoretical component of behavior change in various therapeutic treatments. (APA, Thesaurus of Psychological Index Terms, 1994)"),
  'D020378': NtO(MH='Confined Spaces', MS='A space which has limited openings for entry and exit combined with unfavorable natural ventilation such as CAVES, refrigerators, deep tunnels, pipelines, sewers, silos, tanks, vats, mines, deep trenches or pits, vaults, manholes, chimneys, etc.'),
  'D020379': NtO(MH='Risk Adjustment', MS='The use of severity-of-illness measures, such as age, to estimate the risk (measurable or predictable chance of loss, injury or death) to which a patient is subject before receiving some health care intervention. This adjustment allows comparison of performance and quality across organizations, practitioners, and communities. (from JCAHO, Lexikon, 1994)'),
  'D020380': NtO(MH='Needs Assessment', MS="Systematic identification of a population's needs or the assessment of individuals to determine the proper level of services needed."),
  'D020381': NtO(MH='Interleukin-17', MS='A proinflammatory cytokine produced primarily by T-LYMPHOCYTES or their precursors. Several subtypes of interleukin-17 have been identified, each of which is a product of a unique gene.'),
  'D020382': NtO(MH='Interleukin-18', MS='A cytokine which resembles IL-1 structurally and IL-12 functionally. It enhances the cytotoxic activity of NK CELLS and CYTOTOXIC T-LYMPHOCYTES, and appears to play a role both as neuroimmunomodulator and in the induction of mucosal immunity.'),
  'D020383': NtO(MH='Neutral Glycosphingolipids', MS='A subclass of GLYCOSPHINGOLIPIDS containing one or more sugars within their head group connected directly to a ceramide moiety. They consist of monoglycosyl-, and oligoglycosylsphingoids and monoglycosyl- and oligoglycosylceramides.'),
  'D020384': NtO(MH='Acidic Glycosphingolipids', MS='A subclass of GLYCOSPHINGOLIPIDS containing large polar heads made up of several sugar units. One or more of their terminal sugar units are bound to a negatively charged molecule at pH 7. Members of this class include: GANGLIOSIDES, uronoglycosphingolipids, SULFOGLYCOSPHINGOLIPIDS, phosphoglycosphingolipids, and phosphonoglycosphingolipids.'),
  'D020385': NtO(MH='Myokymia', MS='Successive and rapid contractions of motor units associated with chronic nerve injury. The discharges arise from the peripheral aspects of regenerating nerves, and clinically impart a nearly continuous undulation of the body surface overlying the muscle. (Adams et al., Principles of Neurology, 6th ed, p1491)'),
  'D020386': NtO(MH='Isaacs Syndrome', MS='A rare neuromuscular disorder with onset usually in late childhood or early adulthood, characterized by intermittent or continuous widespread involuntary muscle contractions; FASCICULATION; hyporeflexia; MUSCLE CRAMP; MUSCLE WEAKNESS; HYPERHIDROSIS; TACHYCARDIA; and MYOKYMIA. Involvement of pharyngeal or laryngeal muscles may interfere with speech and breathing. The continuous motor activity persists during sleep and general anesthesia (distinguishing this condition from STIFF-PERSON SYNDROME). Familial and acquired (primarily autoimmune) forms have been reported. (From Ann NY Acad Sci 1998 May 13;841:482-496; Adams et al., Principles of Neurology, 6th ed, p1491)'),
  'D020387': NtO(MH='Food Chain', MS='The sequence of transfers of matter and energy from organism to organism in the form of FOOD. Food chains intertwine locally into a food web because most organisms consume more than one type of animal or plant. PLANTS, which convert SOLAR ENERGY to food by PHOTOSYNTHESIS, are the primary food source. In a predator chain, a plant-eating animal is eaten by a larger animal. In a parasite chain, a smaller organism consumes part of a larger host and may itself be parasitized by smaller organisms. In a saprophytic chain, microorganisms live on dead organic matter.'),
  'D020388': NtO(MH='Muscular Dystrophy, Duchenne', MS='An X-linked recessive muscle disease caused by an inability to synthesize DYSTROPHIN, which is involved with maintaining the integrity of the sarcolemma. Muscle fibers undergo a process that features degeneration and regeneration. Clinical manifestations include proximal weakness in the first few years of life, pseudohypertrophy, cardiomyopathy (see MYOCARDIAL DISEASES), and an increased incidence of impaired mentation. Becker muscular dystrophy is a closely related condition featuring a later onset of disease (usually adolescence) and a slowly progressive course. (Adams et al., Principles of Neurology, 6th ed, p1415)'),
  'D020389': NtO(MH='Muscular Dystrophy, Emery-Dreifuss', MS='A heterogenous group of inherited muscular dystrophy without the involvement of nervous system. The disease is characterized by MUSCULAR ATROPHY; MUSCLE WEAKNESS; CONTRACTURE of the elbows; ACHILLES TENDON; and posterior cervical muscles; with or without cardiac features. There are several INHERITANCE PATTERNS including X-linked (X CHROMOSOME), autosomal dominant, and autosomal recessive gene mutations.'),
  'D020390': NtO(MH='Tooth Socket', MS='A hollow part of the alveolar process of the MAXILLA or MANDIBLE where each tooth fits and is attached via the periodontal ligament.'),
  'D020391': NtO(MH='Muscular Dystrophy, Facioscapulohumeral', MS='An autosomal dominant degenerative muscle disease characterized by slowly progressive weakness of the muscles of the face, upper-arm, and shoulder girdle. The onset of symptoms usually occurs in the first or second decade of life. Affected individuals usually present with impairment of upper extremity elevation. This tends to be followed by facial weakness, primarily involving the orbicularis oris and orbicularis oculi muscles. (Neuromuscul Disord 1997;7(1):55-62; Adams et al., Principles of Neurology, 6th ed, p1420)'),
  'D020392': NtO(MH='Entopeduncular Nucleus', MS='A portion of the nucleus of ansa lenticularis located medial to the posterior limb of the internal capsule, along the course of the ansa lenticularis and the inferior thalamic peduncle or as a separate nucleus within the internal capsule adjacent to the medial GLOBUS PALLIDUS (NeuroNames, http://rprcsgi.rprc. washington.edu/neuronames/ (September 28, 1998)). In non-primates, the entopeduncular nucleus is analogous to both the medial globus pallidus and the entopeduncular nucleus of human.'),
  'D020393': NtO(MH='Manipulation, Spinal', MS='Adjustment and manipulation of the vertebral column.'),
  'D020394': NtO(MH='Vault Ribonucleoprotein Particles', MS='Large cytoplasmic ribonucleoprotein particles that have an eight-fold symmetry with a central pore and petal-like structure giving the appearance of an octagonal dome. (The Dictionary of Cell Biology, Lackie and Dow, 2nd ed.)'),
  'D020395': NtO(MH='Receptors, Interleukin-7', MS='Cell surface receptors that are specific for INTERLEUKIN-7. They are present on T-LYMPHOCYTES and B-LYMPHOCYTE precursors. The receptors are heterodimeric proteins consisting of the INTERLEUKIN-5 RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR COMMON BETA SUBUNIT.'),
  'D020397': NtO(MH='Goblet Cells', MS='A glandular epithelial cell or a unicellular gland. Goblet cells secrete MUCUS. They are scattered in the epithelial linings of many organs, especially the SMALL INTESTINE and the RESPIRATORY TRACT.'),
  'D020398': NtO(MH='Medicare Part C', MS='The Balanced Budget Act (BBA) of 1997 establishes a Medicare+Choice program under part C of Title XVIII, Section 4001, of the Social Security Act. Under this program, an eligible individual may elect to receive Medicare benefits through enrollment in a Medicare+Choice plan. Beneficiaries may choose to use private pay options, establish medical savings accounts, use managed care plans, or join provider-sponsored plans.'),
  'D020399': NtO(MH='Practice Management', MS='Business management of medical, dental and veterinary practices that may include capital financing, utilization management, and arrangement of capitation agreements with other parties.'),
  'D020400': NtO(MH='Provider-Sponsored Organizations', MS='Entities sponsored by local hospitals, physician groups, and other licensed providers which are affiliated through common ownership or control and share financial risk whose purpose is to deliver health care services.'),
  'D020401': NtO(MH='Primed In Situ Labeling', MS='A technique that labels specific sequences in whole chromosomes by in situ DNA chain elongation or PCR (polymerase chain reaction).'),
  'D020402': NtO(MH='Medical Savings Accounts', MS='Tax-exempt trusts or custodial accounts established by individuals with financial institutions for saving money for future medical expenses.'),
  'D020403': NtO(MH='Patient Freedom of Choice Laws', MS='Laws requiring patients under managed care programs to receive services from the physician or other provider of their choice. Any willing provider laws take many different forms, but they typically prohibit managed-care organizations from having a closed panel of physicians, hospitals, or other providers.'),
  'D020404': NtO(MH='Glycerophospholipids', MS='Derivatives of phosphatidic acid in which the hydrophobic regions are composed of two fatty acids and a polar alcohol is joined to the C-3 position of glycerol through a phosphodiester bond. They are named according to their polar head groups, such as phosphatidylcholine and phosphatidylethanolamine.'),
  'D020405': NtO(MH='Dermis', MS='A layer of vascularized connective tissue underneath the EPIDERMIS. The surface of the dermis contains innervated papillae. Embedded in or beneath the dermis are SWEAT GLANDS; HAIR FOLLICLES; and SEBACEOUS GLANDS.'),
  'D020406': NtO(MH='Medicare Payment Advisory Commission', MS='The Commission was created by the Balanced Budget Act of 1997 under Title XVIII. It is specifically charged to review the effect of Medicare+Choice under Medicare Part C and to review payment policies under Parts A and B. It is also generally charged to evaluate the effect of prospective payment policies and their impact on health care delivery in the US. The former Prospective Payment Assessment Commission (ProPAC) and the Physician Payment Review Commission (PPRC) were merged to form MEDPAC.'),
  'D020407': NtO(MH='Internet', MS='A loose confederation of computer communication networks around the world. The networks that make up the Internet are connected through several backbone networks. The Internet grew out of the US Government ARPAnet project and was designed to facilitate information exchange.'),
  'D020408': NtO(MH='Health Insurance Portability and Accountability Act', MS='Public Law 104-91 enacted in 1996, was designed to improve the efficiency and effectiveness of the healthcare system, protect health insurance coverage for workers and their families, and to protect individual personal health information.'),
  'D020409': NtO(MH='Molecular Motor Proteins', MS='Proteins that are involved in or cause CELL MOVEMENT such as the rotary structures (flagellar motor) or the structures whose movement is directed along cytoskeletal filaments (MYOSIN; KINESIN; and DYNEIN motor families).'),
  'D020410': NtO(MH='Activated-Leukocyte Cell Adhesion Molecule', MS='Cell adhesion molecule expressed on activated leukocytes, fibroblasts, and neurons. It is a ligand for CD6. ALCAM-CD6 interactions may play a role in the binding of T and B cells to activated leukocytes.'),
  'D020411': NtO(MH='Oligonucleotide Array Sequence Analysis', MS='Hybridization of a nucleic acid sample to a very large set of OLIGONUCLEOTIDE PROBES, which have been attached individually in columns and rows to a solid support, to determine a BASE SEQUENCE, or to detect variations in a gene sequence, GENE EXPRESSION, or for GENE MAPPING.'),
  'D020412': NtO(MH='Multifactorial Inheritance', MS='A pattern of inheritance of a trait that includes the contributions from more than one gene.'),
  'D020413': NtO(MH="3' Untranslated Regions", MS="The sequence at the 3' end of messenger RNA that does not code for product. This region contains transcription and translation regulating sequences."),
  'D020414': NtO(MH='Risk Sharing, Financial', MS='Any system which allows payors to share some of the financial risk associated with a particular patient population with providers. Providers agree to adhere to fixed fee schedules in exchange for an increase in their payor base and a chance to benefit from cost containment measures. Common risk-sharing methods are prospective payment schedules (PROSPECTIVE PAYMENT SYSTEM), capitation (CAPITATION FEES), diagnosis-related fees (DIAGNOSIS-RELATED GROUPS), and pre-negotiated fees.'),
  'D020415': NtO(MH='Personnel Downsizing', MS='Reducing staff to cut costs or to achieve greater efficiency.'),
  'D020416': NtO(MH='Outsourced Services', MS='Organizational activities previously performed internally that are provided by external agents.'),
  'D020417': NtO(MH='Nystagmus, Congenital', MS='Nystagmus present at birth or caused by lesions sustained in utero or at the time of birth. It is usually pendular, and is associated with ALBINISM and conditions characterized by early loss of central vision. Inheritance patterns may be X-linked, autosomal dominant, or recessive. (Adams et al., Principles of Neurology, 6th ed, p275)'),
  'D020418': NtO(MH='Algal Proteins', MS='Proteins found in any species of algae.'),
  'D020419': NtO(MH='Photoreceptor Cells, Vertebrate', MS='Specialized PHOTOTRANSDUCTION neurons in the vertebrates, such as the RETINAL ROD CELLS and the RETINAL CONE CELLS. Non-visual photoreceptor neurons have been reported in the deep brain, the PINEAL GLAND and organs of the circadian system.'),
  'D020420': NtO(MH='Cyst Fluid', MS='Liquid material found in epithelial-lined closed cavities or sacs.'),
  'D020421': NtO(MH='Vagus Nerve Diseases', MS='Diseases of the tenth cranial nerve, including brain stem lesions involving its nuclei (solitary, ambiguus, and dorsal motor), nerve fascicles, and intracranial and extracranial course. Clinical manifestations may include dysphagia, vocal cord weakness, and alterations of parasympathetic tone in the thorax and abdomen.'),
  'D020422': NtO(MH='Mononeuropathies', MS='Disease or trauma involving a single peripheral nerve in isolation, or out of proportion to evidence of diffuse peripheral nerve dysfunction. Mononeuropathy multiplex refers to a condition characterized by multiple isolated nerve injuries. Mononeuropathies may result from a wide variety of causes, including ISCHEMIA; traumatic injury; compression; CONNECTIVE TISSUE DISEASES; CUMULATIVE TRAUMA DISORDERS; and other conditions.'),
  'D020423': NtO(MH='Median Neuropathy', MS='Disease involving the median nerve, from its origin at the BRACHIAL PLEXUS to its termination in the hand. Clinical features include weakness of wrist and finger flexion, forearm pronation, thenar abduction, and loss of sensation over the lateral palm, first three fingers, and radial half of the ring finger. Common sites of injury include the elbow, where the nerve passes through the two heads of the pronator teres muscle (pronator syndrome) and in the carpal tunnel (CARPAL TUNNEL SYNDROME).'),
  'D020424': NtO(MH='Ulnar Neuropathies', MS="Disease involving the ULNAR NERVE from its origin in the BRACHIAL PLEXUS to its termination in the hand. Clinical manifestations may include PARESIS or PARALYSIS of wrist flexion, finger flexion, thumb adduction, finger abduction, and finger adduction. Sensation over the medial palm, fifth finger, and ulnar aspect of the ring finger may also be impaired. Common sites of injury include the AXILLA, cubital tunnel at the ELBOW, and Guyon's canal at the wrist. (From Joynt, Clinical Neurology, 1995, Ch51 pp43-5)"),
  'D020425': NtO(MH='Radial Neuropathy', MS='Disease involving the RADIAL NERVE. Clinical features include weakness of elbow extension, elbow flexion, supination of the forearm, wrist and finger extension, and thumb abduction. Sensation may be impaired over regions of the dorsal forearm. Common sites of compression or traumatic injury include the AXILLA and radial groove of the HUMERUS.'),
  'D020426': NtO(MH='Sciatic Neuropathy', MS='Disease or damage involving the SCIATIC NERVE, which divides into the PERONEAL NERVE and TIBIAL NERVE (see also PERONEAL NEUROPATHIES and TIBIAL NEUROPATHY). Clinical manifestations may include SCIATICA or pain localized to the hip, PARESIS or PARALYSIS of posterior thigh muscles and muscles innervated by the peroneal and tibial nerves, and sensory loss involving the lateral and posterior thigh, posterior and lateral leg, and sole of the foot. The sciatic nerve may be affected by trauma; ISCHEMIA; COLLAGEN DISEASES; and other conditions. (From Adams et al., Principles of Neurology, 6th ed, p1363)'),
  'D020427': NtO(MH='Peroneal Neuropathies', MS='Disease involving the common PERONEAL NERVE or its branches, the deep and superficial peroneal nerves. Lesions of the deep peroneal nerve are associated with PARALYSIS of dorsiflexion of the ankle and toes and loss of sensation from the web space between the first and second toe. Lesions of the superficial peroneal nerve result in weakness or paralysis of the peroneal muscles (which evert the foot) and loss of sensation over the dorsal and lateral surface of the leg. Traumatic injury to the common peroneal nerve near the head of the FIBULA is a relatively common cause of this condition. (From Joynt, Clinical Neurology, 1995, Ch51, p31)'),
  'D020428': NtO(MH='Femoral Neuropathy', MS='Disease involving the femoral nerve. The femoral nerve may be injured by ISCHEMIA (e.g., in association with DIABETIC NEUROPATHIES), nerve compression, trauma, COLLAGEN DISEASES, and other disease processes. Clinical features include MUSCLE WEAKNESS or PARALYSIS of hip flexion and knee extension, ATROPHY of the QUADRICEPS MUSCLE, reduced or absent patellar reflex, and impaired sensation over the anterior and medial thigh.'),
  'D020429': NtO(MH='Tibial Neuropathy', MS='Disease of the TIBIAL NERVE (also referred to as the posterior tibial nerve). The most commonly associated condition is the TARSAL TUNNEL SYNDROME. However, LEG INJURIES; ISCHEMIA; and inflammatory conditions (e.g., COLLAGEN DISEASES) may also affect the nerve. Clinical features include PARALYSIS of plantar flexion, ankle inversion and toe flexion as well as loss of sensation over the sole of the foot. (From Joynt, Clinical Neurology, 1995, Ch51, p32)'),
  'D020430': NtO(MH='Cubital Tunnel Syndrome', MS='Compression of the ULNAR NERVE in the cubital tunnel, which is formed by the two heads of the flexor carpi ulnaris muscle, humeral-ulnar aponeurosis, and medial ligaments of the elbow. This condition may follow trauma or occur in association with processes which produce nerve enlargement or narrowing of the canal. Manifestations include elbow pain and PARESTHESIA radiating distally, weakness of ulnar innervated intrinsic hand muscles, and loss of sensation over the hypothenar region, fifth finger, and ulnar aspect of the ring finger. (Joynt, Clinical Neurology, 1995, Ch51, p43)'),
  'D020431': NtO(MH='Olfactory Nerve Diseases', MS='Diseases of the first cranial (olfactory) nerve, which usually feature anosmia or other alterations in the sense of smell and taste. Anosmia may be associated with NEOPLASMS; CENTRAL NERVOUS SYSTEM INFECTIONS; CRANIOCEREBRAL TRAUMA; inherited conditions; toxins; METABOLIC DISEASES; tobacco abuse; and other conditions. (Adams et al., Principles of Neurology, 6th ed, pp229-31)'),
  'D020432': NtO(MH='Trochlear Nerve Diseases', MS='Diseases of the fourth cranial (trochlear) nerve or its nucleus in the midbrain. The nerve crosses as it exits the midbrain dorsally and may be injured along its course through the intracranial space, cavernous sinus, superior orbital fissure, or orbit. Clinical manifestations include weakness of the superior oblique muscle which causes vertical DIPLOPIA that is maximal when the affected eye is adducted and directed inferiorly. Head tilt may be seen as a compensatory mechanism for diplopia and rotation of the visual axis. Common etiologies include CRANIOCEREBRAL TRAUMA and INFRATENTORIAL NEOPLASMS.'),
  'D020433': NtO(MH='Trigeminal Nerve Diseases', MS='Diseases of the trigeminal nerve or its nuclei, which are located in the pons and medulla. The nerve is composed of three divisions: ophthalmic, maxillary, and mandibular, which provide sensory innervation to structures of the face, sinuses, and portions of the cranial vault. The mandibular nerve also innervates muscles of mastication. Clinical features include loss of facial and intra-oral sensation and weakness of jaw closure. Common conditions affecting the nerve include brain stem ischemia, INFRATENTORIAL NEOPLASMS, and TRIGEMINAL NEURALGIA.'),
  'D020434': NtO(MH='Abducens Nerve Diseases', MS='Diseases of the sixth cranial (abducens) nerve or its nucleus in the pons. The nerve may be injured along its course in the pons, intracranially as it travels along the base of the brain, in the cavernous sinus, or at the level of superior orbital fissure or orbit. Dysfunction of the nerve causes lateral rectus muscle weakness, resulting in horizontal diplopia that is maximal when the affected eye is abducted and ESOTROPIA. Common conditions associated with nerve injury include INTRACRANIAL HYPERTENSION; CRANIOCEREBRAL TRAUMA; ISCHEMIA; and INFRATENTORIAL NEOPLASMS.'),
  'D020435': NtO(MH='Glossopharyngeal Nerve Diseases', MS="Diseases of the ninth cranial (glossopharyngeal) nerve or its nuclei in the medulla. The nerve may be injured by diseases affecting the lower brain stem, floor of the posterior fossa, jugular foramen, or the nerve's extracranial course. Clinical manifestations include loss of sensation from the pharynx, decreased salivation, and syncope. Glossopharyngeal neuralgia refers to a condition that features recurrent unilateral sharp pain in the tongue, angle of the jaw, external auditory meatus and throat that may be associated with SYNCOPE. Episodes may be triggered by cough, sneeze, swallowing, or pressure on the tragus of the ear. (Adams et al., Principles of Neurology, 6th ed, p1390)"),
  'D020436': NtO(MH='Accessory Nerve Diseases', MS='Diseases of the eleventh cranial (spinal accessory) nerve. This nerve originates from motor neurons in the lower medulla (accessory portion of nerve) and upper spinal cord (spinal portion of nerve). The two components of the nerve join and exit the skull via the jugular foramen, innervating the sternocleidomastoid and trapezius muscles, which become weak or paralyzed if the nerve is injured. The nerve is commonly involved in MOTOR NEURON DISEASE, and may be injured by trauma to the posterior triangle of the neck.'),
  'D020437': NtO(MH='Hypoglossal Nerve Diseases', MS='Diseases of the twelfth cranial (hypoglossal) nerve or nuclei. The nuclei and fascicles of the nerve are located in the medulla, and the nerve exits the skull via the hypoglossal foramen and innervates the muscles of the tongue. Lower brain stem diseases, including ischemia and MOTOR NEURON DISEASES may affect the nuclei or nerve fascicles. The nerve may also be injured by diseases of the posterior fossa or skull base. Clinical manifestations include unilateral weakness of tongue musculature and lingual dysarthria, with deviation of the tongue towards the side of weakness upon attempted protrusion.'),
  'D020439': NtO(MH='Growth Cones', MS='Bulbous enlargement of the growing tip of nerve axons and dendrites. They are crucial to neuronal development because of their pathfinding ability and their role in synaptogenesis.'),
  'D020440': NtO(MH='Gene Duplication', MS='Processes occurring in various organisms by which new genes are copied. Gene duplication may result in a MULTIGENE FAMILY; supergenes or PSEUDOGENES.'),
  'D020441': NtO(MH='Ginkgo biloba', MS='The only specie of the genus Ginkgo, family Ginkgoacea. It is the source of extracts of medicinal interest, especially Egb 761. Ginkgo may refer to the genus or species.'),
  'D020442': NtO(MH='Poloxamer', MS='A nonionic polyoxyethylene-polyoxypropylene block co-polymer with the general formula HO(C2H4O)a(-C3H6O)b(C2H4O)aH. It is available in different grades which vary from liquids to solids. It is used as an emulsifying agent, solubilizing agent, surfactant, and wetting agent for antibiotics. Poloxamer is also used in ointment and suppository bases and as a tablet binder or coater. (Martindale The Extra Pharmacopoeia, 31st ed)'),
  'D020443': NtO(MH='Practice Valuation and Purchase', MS='Determination of economic value of an established health care provider practice including value of patient lists, equipment, and other assets, and process of buying or selling rights to said practice.'),
  'D020444': NtO(MH='Reimbursement, Disproportionate Share', MS='Payments that include adjustments to reflect the costs of uncompensated care and higher costs for inpatient care for certain populations receiving mandated services. MEDICARE and MEDICAID include provisions for this type of reimbursement.'),
  'D020445': NtO(MH='Hospitalists', MS='Physicians who are employed to work exclusively in hospital settings, primarily for managed care organizations. They are the attending or primary responsible physician for the patient during hospitalization.'),
  'D020446': NtO(MH='Fitness Centers', MS='Facilities having programs intended to promote and maintain a state of physical well-being for optimal performance and health.'),
  'D020447': NtO(MH='Parasomnias', MS='Movements or behaviors associated with sleep, sleep stages, or partial arousals from sleep that may impair sleep maintenance. Parasomnias are generally divided into four groups: arousal disorders, sleep-wake transition disorders, parasomnias of REM sleep, and nonspecific parasomnias. (From Thorpy, Sleep Disorders Medicine, 1994, p191)'),
  'D020448': NtO(MH='Ericales', MS='A large and diverse order of dicotyledonous flowering plants in the subclass DILLENIIDAE. Most members have weakly fused petals and radially symmetric flowers which form fruit capsules and thin seed coats. Members also contain IRIDOIDS, compounds which protect against herbivores. Common examples of the ericales are economically important species such as TEA, PERSIMMON, BLUEBERRY, BRAZIL NUTS,  azalea, CRANBERRIES, and RHODODENDRON.'),
  'D020449': NtO(MH='Repetitive Sequences, Amino Acid', MS='A sequential pattern of amino acids occurring more than once in the same protein sequence. There often is some sequence variation between the repeated segments. Many PROTEIN DOMAINS are constituted from repeats.'),
  'D020450': NtO(MH='Heavy Ions', MS='Positively-charged atomic nuclei that have been stripped of their electrons. These particles have one or more units of electric charge and a mass exceeding that of the Helium-4 nucleus (alpha particle).'),
  'D020451': NtO(MH='Contig Mapping', MS='Overlapping of cloned or sequenced DNA to construct a continuous region of a gene, chromosome or genome.'),
  'D020452': NtO(MH='Journalism', MS='The collection, preparation, and distribution of news and related commentary and feature materials through such media as pamphlets, newsletters, newspapers, magazines, radio, motion pictures, television, and books. While originally applied to the reportage of current events in printed form, specifically newspapers, with the advent of radio and television the use of the term has broadened to include all printed and electronic communication dealing with current affairs.'),
  'D020453': NtO(MH='Sirenia', MS='An order of heavy-bodied, slow-moving, completely aquatic, herbivorous mammals. The body is fusiform, plump, and hairless, except for bristles on the snout. Hindlimbs are absent, the forelimbs are modified to flippers, and the tail is a horizontal fluke. (From Scott, Concise Encyclopedia Biology, 1996)'),
  'D020454': NtO(MH='Dugong', MS='A genus of the order Sirenia characterized by a notched tail, the presence of nasal bones and a long nasal cavity, and large columnar teeth lacking enamel. Dugongs inhabit the coastal waters of the Indian Ocean, Red Sea, and the Malay Archipelago. (From Scott, Concise Encyclopedia Biology, 1996)'),
  'D020455': NtO(MH='Trichechus', MS='A genus of the order Sirenia comprising what are commonly called manatees. The distinguishing characteristics include a tail that is not notched, a short nasal cavity, the absence of nasal bones, and enamel-covered teeth. Members of this genus are found in marine bays and/or sluggish rivers, usually in turbid water. (From Scott, Concise Encyclopedia Biology, 1996)'),
  'D020456': NtO(MH='Trichechus inunguis', MS='Member of the genus Trichechus found in the Amazon and Orinoco drainages of northeastern South America. (From Scott, Concise Encyclopedia Biology, 1996)'),
  'D020457': NtO(MH='Trichechus manatus', MS='Member of the genus Trichechus inhabiting the coast and coastal rivers of the southeastern United States as well as the West Indies and the adjacent mainland from Vera Cruz, Mexico to northern South America. (From Scott, Concise Encyclopedia Biology, 1996)'),
  'D020458': NtO(MH='Third-Party Consent', MS='Informed consent given by someone other than the patient or research subject.'),
  'D020459': NtO(MH='Genes, rRNA', MS='Genes, found in both prokaryotes and eukaryotes, which are transcribed to produce the RNA which is incorporated into RIBOSOMES. Prokaryotic rRNA genes are usually found in OPERONS dispersed throughout the GENOME, whereas eukaryotic rRNA genes are clustered, multicistronic transcriptional units.'),
  'D020460': NtO(MH='Melanosomes', MS='Melanin-containing organelles found in melanocytes and melanophores.'),
  'D020461': NtO(MH='Madurella', MS='A mitosporic fungal genus that causes MYCETOMA in humans. Madurella grisea and M. mycetomatis are the etiological agents.'),
  'D020462': NtO(MH='Gliocladium', MS='A mitosporic fungal genus occurring in soil or decaying plant matter. It is structurally similar to Penicillium.'),
  'D020463': NtO(MH='Abbreviations', MS='Works consisting of lists of shortened forms of written words or phrases used for brevity. Acronyms are included here.'),
  'D020465': NtO(MH='Anecdotes', MS='Works consisting of brief accounts or narratives of incidents or events.'),
  'D020466': NtO(MH='Atlas', MS='Work consisting of collections of illustrative plates, charts, etc., usually with explanatory captions.'),
  'D020467': NtO(MH='Biobibliography', MS='Works consisting of biographical information as well as lists of the writings of those persons.'),
  'D020468': NtO(MH='Chart', MS='Work consisting of information presented in graphic form, for example, graphs or diagrams.'),
  'D020469': NtO(MH='Chronology', MS='Works consisting of lists of events arranged in chronological order.'),
  'D020470': NtO(MH='Collected Work', MS='Work consisting of collections of previously published works.'),
  'D020471': NtO(MH='Collection', MS='Items gathered and assembled in some order to facilitate access or use.'),
  'D020472': NtO(MH='Drawing', MS='Work consisting of graphic representations of objects or ideas by lines.'),
  'D020474': NtO(MH='Essay', MS='Short literary prose composition, usually dealing with a single theme.'),
  'D020475': NtO(MH='Examination Questions', MS='Work consisting of compilations of questions and answers pertaining to a particular subject, used for study and review.'),
  'D020476': NtO(MH='Exhibition', MS='Objects publicly displayed.'),
  'D020478': NtO(MH='Form', MS='Document used for acquiring particular information or for presenting particular information in a prescribed sequence and format, often with blank spaces or lines or other methods to prompt for insertion of the requested information.'),
  'D020479': NtO(MH='Handbook', MS='Work consisting of concise reference works in which facts and information pertaining to a certain subject or field are arranged for ready reference and consultation rather than for continuous reading and study.'),
  'D020480': NtO(MH='Wit and Humor', MS=None),
  'D020481': NtO(MH='Index', MS='Work providing an analytical subject approach to materials in a field of knowledge.'),
  'D020482': NtO(MH='Juvenile Literature', MS='Works produced for children through age 15 or through the ninth grade.'),
  'D020484': NtO(MH='Laboratory Manual', MS='Work containing concise background information and directions for activities, including conducting experiments or diagnostic tests in the laboratory.'),
  'D020485': NtO(MH='Legislation', MS='Works consisting of the text of proposed or enacted legislation that may be in the form of bills, laws, statutes, ordinances, or government regulations.'),
  'D020486': NtO(MH='Manuscript', MS='Work written by hand, as one written before the invention or adoption of printing.'),
  'D020488': NtO(MH='Nurses Instruction', MS='Work consisting of materials developed for a nursing audience.'),
  'D020489': NtO(MH='Outline', MS='Work consisting of brief statements of the principal elements of a subject, usually arranged by heads and subheads.'),
  'D020490': NtO(MH='Patent', MS='Work consisting of documents granted by a government giving exclusive rights to an inventor or assignee to manufacture, use, or sell an invention for a certain number of years.'),
  'D020492': NtO(MH='Periodical', MS='Publication intended to be issued on an ongoing basis, generally more frequently than annually, containing separate articles, stories, or writings.'),
  'D020493': NtO(MH='Autobiography', MS="Self-described narratives of a person's life."),
  'D020494': NtO(MH='Phrases', MS='Work consisting of common terms, phrases, idioms, and typical conversations, e.g., between health professional and patients. These are often intended for use by non-native speakers of a language.'),
  'D020495': NtO(MH='Pictorial Work', MS='Work consisting exclusively or mainly of pictures but not technical drawings.'),
  'D020496': NtO(MH='Popular Work', MS='Work written for non-professional or lay audiences.'),
  'D020497': NtO(MH='Problems and Exercises', MS='Works consisting of collections of practice questions and drills, generally for instructional or review use.'),
  'D020498': NtO(MH='Programmed Instruction', MS='Works consisting of sequenced self-correction texts.'),
  'D020500': NtO(MH='Statistics', MS='Works consisting of presentations of numerical data on particular subjects.'),
  'D020501': NtO(MH='Tables', MS='Presentations of data in tabular form.'),
  'D020502': NtO(MH='Terminology', MS='Work consisting of lists of the technical terms or expressions used in a specific field. These lists may or may not be formally adopted or sanctioned by usage.'),
  'D020503': NtO(MH='Union List', MS='Works consisting of records of the holdings or items owned by two or more libraries.'),
  'D020504': NtO(MH='Abstracts', MS='Works consisting of lists of publications on a subject and that provide full annotated bibliographical information together with substantive summaries or condensations of the facts, ideas, or opinions presented in each publication listed. (From LC Subject Cataloging Manual)'),
  'D020505': NtO(MH='Collected Correspondence', MS='Written communication between persons or between institutions or organizations.'),
  'D020506': NtO(MH='Untranslated Regions', MS="The parts of the messenger RNA sequence that do not code for product, i.e. the 5' UNTRANSLATED REGIONS and 3' UNTRANSLATED REGIONS."),
  'D020507': NtO(MH='Resource Guide', MS='Work listing and describing various sources of information, from multiple media or in different formats, on a given subject.'),
  'D020508': NtO(MH='Fungi, Unclassified', MS='Fungi whose taxonomic relationships have not been authoritatively established.'),
  'D020511': NtO(MH='Neuromuscular Junction Diseases', MS='Conditions characterized by impaired transmission of impulses at the NEUROMUSCULAR JUNCTION. This may result from disorders that affect receptor function, pre- or postsynaptic membrane function, or ACETYLCHOLINESTERASE activity. The majority of diseases in this category are associated with autoimmune, toxic, or inherited conditions.'),
  'D020512': NtO(MH='Myopathy, Central Core', MS='An inherited congenital myopathic condition characterized by weakness and hypotonia in infancy and delayed motor development. Muscle biopsy reveals a condensation of myofibrils and myofibrillar material in the central portion of each muscle fiber. (Adams et al., Principles of Neurology, 6th ed, p1452)'),
  'D020513': NtO(MH='Paralysis, Hyperkalemic Periodic', MS='An autosomal dominant familial disorder which presents in infancy or childhood and is characterized by episodes of weakness associated with hyperkalemia. During attacks, muscles of the lower extremities are initially affected, followed by the lower trunk and arms. Episodes last from 15-60 minutes and typically occur after a period of rest following exercise. A defect in skeletal muscle sodium channels has been identified as the cause of this condition. Normokalemic periodic paralysis is a closely related disorder marked by a lack of alterations in potassium levels during attacks of weakness. (Adams et al., Principles of Neurology, 6th ed, p1481)'),
  'D020514': NtO(MH='Hypokalemic Periodic Paralysis', MS='An autosomal dominant familial disorder characterized by recurrent episodes of skeletal muscle weakness associated with falls in serum potassium levels. The condition usually presents in the first or second decade of life with attacks of trunk and leg paresis during sleep or shortly after awakening. Symptoms may persist for hours to days and generally are precipitated by exercise or a meal high in carbohydrates. (Adams et al., Principles of Neurology, 6th ed, p1483)'),
  'D020516': NtO(MH='Brachial Plexus Neuropathies', MS='Diseases of the cervical (and first thoracic) roots, nerve trunks, cords, and peripheral nerve components of the BRACHIAL PLEXUS. Clinical manifestations include regional pain, PARESTHESIA; MUSCLE WEAKNESS, and decreased sensation (HYPESTHESIA) in the upper extremity. These disorders may be associated with trauma (including BIRTH INJURIES); THORACIC OUTLET SYNDROME; NEOPLASMS; NEURITIS; RADIOTHERAPY; and other conditions. (From Adams et al., Principles of Neurology, 6th ed, pp1351-2)'),
  'D020517': NtO(MH='Atrial Appendage', MS='Ear-shaped appendage of either atrium of the heart. (Dorland, 28th ed)'),
  'D020518': NtO(MH='Focal Nodular Hyperplasia', MS='Solitary or multiple benign hepatic vascular tumors, usually occurring in women of 20-50 years of age. The nodule, poorly encapsulated, consists of a central stellate fibrous scar and normal liver elements such as HEPATOCYTES, small BILE DUCTS, and KUPFFER CELLS among the intervening fibrous septa. The pale colored central scar represents large blood vessels with hyperplastic fibromuscular layer and narrowing lumen.'),
  'D020519': NtO(MH='Pericardiocentesis', MS='Puncture and aspiration of fluid from the PERICARDIUM.'),
  'D020520': NtO(MH='Brain Infarction', MS='Tissue NECROSIS in any area of the brain, including the CEREBRAL HEMISPHERES, the CEREBELLUM, and the BRAIN STEM. Brain infarction is the result of a cascade of events initiated by inadequate blood flow through the brain that is followed by HYPOXIA and HYPOGLYCEMIA in brain tissue. Damage may be temporary, permanent, selective or pan-necrosis.'),
  'D020521': NtO(MH='Stroke', MS='A group of pathological conditions characterized by sudden, non-convulsive loss of neurological function due to BRAIN ISCHEMIA or INTRACRANIAL HEMORRHAGES. Stroke is classified by the type of tissue NECROSIS, such as the anatomic location, vasculature involved, etiology, age of the affected individual, and hemorrhagic vs. non-hemorrhagic nature. (From Adams et al., Principles of Neurology, 6th ed, pp777-810)'),
  'D020522': NtO(MH='Lymphoma, Mantle-Cell', MS='A form of non-Hodgkin lymphoma having a usually diffuse pattern with both small and medium lymphocytes and small cleaved cells. It accounts for about 5% of adult non-Hodgkin lymphomas in the United States and Europe. The majority of mantle-cell lymphomas are associated with a t(11;14) translocation resulting in overexpression of the CYCLIN D1 gene (GENES, BCL-1).'),
  'D020523': NtO(MH='Chemokines, CX3C', MS='Group of chemokines with the first two cysteines separated by three amino acids. CX3C chemokines are chemotactic for natural killer cells, monocytes, and activated T-cells.'),
  'D020524': NtO(MH='Thylakoids', MS='Membranous cisternae of the CHLOROPLAST containing photosynthetic pigments, reaction centers, and the electron-transport chain. Each thylakoid consists of a flattened sac of membrane enclosing a narrow intra-thylakoid space (Lackie and Dow, Dictionary of Cell Biology, 2nd ed). Individual thylakoids are interconnected and tend to stack to form aggregates called grana. They are found in cyanobacteria and all plants.'),
  'D020525': NtO(MH='Xenodiagnosis', MS='A method for diagnosing a disease in one organism by inoculating the putative causative organism in a second animal of a different species. It has been used for the detection of parasites (Trypanosoma cruzi and Trichinella spiralis) when peripheral blood smears are negative. (Segen, Current Med Talk, 1995)'),
  'D020526': NtO(MH='Brain Stem Infarctions', MS='Infarctions that occur in the BRAIN STEM which is comprised of the MIDBRAIN; PONS; and MEDULLA OBLONGATA. There are several named syndromes characterized by their distinctive clinical manifestations and specific sites of ischemic injury.'),
  'D020527': NtO(MH='Microscopy, Scanning Probe', MS='Scanning microscopy in which a very sharp probe is employed in close proximity to a surface, exploiting a particular surface-related property. When this property is local topography, the method is atomic force microscopy (MICROSCOPY, ATOMIC FORCE), and when it is local conductivity, the method is scanning tunneling microscopy (MICROSCOPY, SCANNING TUNNELING).'),
  'D020528': NtO(MH='Multiple Sclerosis, Chronic Progressive', MS='A form of multiple sclerosis characterized by a progressive deterioration in neurologic function which is in contrast to the more typical relapsing remitting form. If the clinical course is free of distinct remissions, it is referred to as primary progressive multiple sclerosis. When the progressive decline is punctuated by acute exacerbations, it is referred to as progressive relapsing multiple sclerosis. The term secondary progressive multiple sclerosis is used when relapsing remitting multiple sclerosis evolves into the chronic progressive form. (From Ann Neurol 1994;36 Suppl:S73-S79; Adams et al., Principles of Neurology, 6th ed, pp903-914)'),
  'D020529': NtO(MH='Multiple Sclerosis, Relapsing-Remitting', MS='The most common clinical variant of MULTIPLE SCLEROSIS, characterized by recurrent acute exacerbations of neurologic dysfunction followed by partial or complete recovery. Common clinical manifestations include loss of visual (see OPTIC NEURITIS), motor, sensory, or bladder function. Acute episodes of demyelination may occur at any site in the central nervous system, and commonly involve the optic nerves, spinal cord, brain stem, and cerebellum. (Adams et al., Principles of Neurology, 6th ed, pp903-914)'),
  'D020530': NtO(MH='Subthalamus', MS='A transition zone in the anterior part of the diencephalon interposed between the thalamus, hypothalamus, and tegmentum of the mesencephalon. Components of the subthalamus include the SUBTHALAMIC NUCLEUS, zona incerta, nucleus of field H, and the nucleus of ansa lenticularis. The latter contains the ENTOPEDUNCULAR NUCLEUS.'),
  'D020531': NtO(MH='Subthalamic Nucleus', MS='Lens-shaped structure on the inner aspect of the INTERNAL CAPSULE. The SUBTHALAMIC NUCLEUS and pathways traversing this region are concerned with the integration of somatic motor function.'),
  'D020532': NtO(MH='Basal Nucleus of Meynert', MS='A group of nerve cells in the SUBSTANTIA INNOMINATA that has wide projections to the NEOCORTEX and is rich in ACETYLCHOLINE and CHOLINE ACETYLTRANSFERASE. In PARKINSON DISEASE and ALZHEIMER DISEASE the nucleus undergoes degeneration.'),
  'D020533': NtO(MH='Angiogenesis Inhibitors', MS='Agents and endogenous substances that antagonize or inhibit the development of new blood vessels.'),
  'D020534': NtO(MH='Parahippocampal Gyrus', MS='A convolution on the inferior surface of each cerebral hemisphere, lying between the hippocampal and collateral sulci.'),
  'D020535': NtO(MH='Video-Assisted Surgery', MS='Endoscopic surgical procedures performed with visualization via video transmission. When real-time video is combined interactively with prior CT scans or MRI images, this is called image-guided surgery (see SURGERY, COMPUTER-ASSISTED).'),
  'D020536': NtO(MH='Enzyme Activators', MS='Compounds or factors that act on a specific enzyme to increase its activity.'),
  'D020537': NtO(MH='RNA, Small Nucleolar', MS='Small nuclear RNAs that are involved in the processing of pre-ribosomal RNA in the nucleolus. Box C/D containing snoRNAs (U14, U15, U16, U20, U21 and U24-U63) direct site-specific methylation of various ribose moieties. Box H/ACA containing snoRNAs (E2, E3, U19, U23, and U64-U72) direct the conversion of specific uridines to pseudouridine. Site-specific cleavages resulting in the mature ribosomal RNAs are directed by snoRNAs U3, U8, U14, U22 and the snoRNA components of RNase MRP and RNase P.'),
  'D020539': NtO(MH='Sequence Analysis, Protein', MS='A process that includes the determination of AMINO ACID SEQUENCE of a protein (or peptide, oligopeptide or peptide fragment) and the information analysis of the sequence.'),
  'D020540': NtO(MH='Metencephalon', MS='The anterior portion of the developing hindbrain. It gives rise to the CEREBELLUM and the PONS.'),
  'D020541': NtO(MH='Coiled Bodies', MS='A distinct subnuclear domain enriched in splicesomal snRNPs (RIBONUCLEOPROTEINS, SMALL NUCLEAR) and p80-coilin.'),
  'D020542': NtO(MH='Third Ventricle', MS='A narrow cleft inferior to the CORPUS CALLOSUM, within the DIENCEPHALON, between the paired thalami. Its floor is formed by the HYPOTHALAMUS, its anterior wall by the lamina terminalis, and its roof by EPENDYMA. It communicates with the FOURTH VENTRICLE by the CEREBRAL AQUEDUCT, and with the LATERAL VENTRICLES by the interventricular foramina.'),
  'D020543': NtO(MH='Proteome', MS='The protein complement of an organism coded for by its genome.'),
  'D020544': NtO(MH='Graft vs Leukemia Effect', MS='Immunological rejection of leukemia cells following bone marrow transplantation.'),
  'D020545': NtO(MH='Respiratory Mucosa', MS='The mucous membrane lining the RESPIRATORY TRACT, including the NASAL CAVITY; the LARYNX; the TRACHEA; and the BRONCHI tree. The respiratory mucosa consists of various types of epithelial cells ranging from ciliated columnar to simple squamous, mucous GOBLET CELLS, and glands containing both mucous and serous cells.'),
  'D020546': NtO(MH='Fourth Ventricle', MS='An irregularly shaped cavity in the RHOMBENCEPHALON, located between the MEDULLA OBLONGATA; the PONS; and the isthmus in front, and the CEREBELLUM behind. It is continuous with the central canal of the cord below and with the CEREBRAL AQUEDUCT above, and through its lateral and median apertures it communicates with the SUBARACHNOID SPACE.'),
  'D020547': NtO(MH='Lateral Ventricles', MS='Cavity in each of the CEREBRAL HEMISPHERES derived from the cavity of the embryonic NEURAL TUBE. They are separated from each other by the SEPTUM PELLUCIDUM, and each communicates with the THIRD VENTRICLE by the foramen of Monro, through which also the choroid plexuses (CHOROID PLEXUS) of the lateral ventricles become continuous with that of the third ventricle.'),
  'D020549': NtO(MH='GTP Phosphohydrolase Activators', MS='Agents and factors that activate GTP phosphohydrolase activity.'),
  'D020551': NtO(MH='Pulse Therapy, Drug', MS='Administration of high doses of pharmaceuticals over short periods of time.'),
  'D020552': NtO(MH='Comet Assay', MS='A genotoxicological technique for measuring DNA damage in an individual cell using single-cell gel electrophoresis. Cell DNA fragments assume a "comet with tail" formation on electrophoresis and are detected with an image analysis system. Alkaline assay conditions facilitate sensitive detection of single-strand damage.'),
  'D020554': NtO(MH='Sperm Injections, Intracytoplasmic', MS='An assisted fertilization technique consisting of the microinjection of a single viable sperm into an extracted ovum. It is used principally to overcome low sperm count, low sperm motility, inability of sperm to penetrate the egg, or other conditions related to male infertility (INFERTILITY, MALE).'),
  'D020555': NtO(MH='GTP-Binding Protein Regulators', MS='Proteins that regulate the signaling activity of GTP-BINDING PROTEINS. They are divided into three categories depending upon whether they stimulate GTPase activity (GTPASE-ACTIVATING PROTEINS), inhibit release of GDP; (GUANINE NUCLEOTIDE DISSOCIATION INHIBITORS); or exchange GTP for GDP; (GUANINE NUCLEOTIDE EXCHANGE FACTORS).'),
  'D020556': NtO(MH='Neutrophil Infiltration', MS='The diffusion or accumulation of neutrophils in tissues or cells in response to a wide variety of substances released at the sites of inflammatory reactions.'),
  'D020557': NtO(MH='Neurolinguistic Programming', MS="A set of models of how communication impacts and is impacted by subjective experience. Techniques are generated from these models by sequencing of various aspects of the models in order to change someone's internal representations. Neurolinguistic programming is concerned with the patterns or programming created by the interactions among the brain, language, and the body, that produce both effective and ineffective behavior."),
  'D020558': NtO(MH='GTP Phosphohydrolases', MS='Enzymes that hydrolyze GTP to GDP. EC 3.6.1.-.'),
  'D020559': NtO(MH='Monomeric GTP-Binding Proteins', MS='A class of monomeric, low molecular weight (20-25 kDa) GTP-binding proteins that regulate a variety of intracellular processes. The GTP bound form of the protein is active and limited by its inherent GTPase activity, which is controlled by an array of GTPase activators, GDP dissociation inhibitors, and guanine nucleotide exchange factors. This enzyme was formerly listed as EC 3.6.1.47'),
  'D020560': NtO(MH='Proteobacteria', MS='A phylum of bacteria consisting of the purple bacteria and their relatives which form a branch of the eubacterial tree. This group of predominantly gram-negative bacteria is classified based on homology of equivalent nucleotide sequences of 16S ribosomal RNA or by hybridization of ribosomal RNA or DNA with 16S and 23S ribosomal RNA.'),
  'D020561': NtO(MH='Alphaproteobacteria', MS='A class in the phylum PROTEOBACTERIA comprised mostly of two major phenotypes: purple non-sulfur bacteria and aerobic bacteriochlorophyll-containing bacteria.'),
  'D020562': NtO(MH='Betaproteobacteria', MS='A class in the phylum PROTEOBACTERIA comprised of chemoheterotrophs and chemoautotrophs which derive nutrients from decomposition of organic material.'),
  'D020563': NtO(MH='Gammaproteobacteria', MS='A group of the proteobacteria comprised of facultatively anaerobic and fermentative gram-negative bacteria.'),
  'D020564': NtO(MH='Deltaproteobacteria', MS='A group of PROTEOBACTERIA represented by morphologically diverse, anaerobic sulfidogens. Some members of this group are considered bacterial predators, having bacteriolytic properties.'),
  'D020565': NtO(MH='Epsilonproteobacteria', MS='A group of proteobacteria consisting of chemoorganotrophs usually associated with the DIGESTIVE SYSTEM of humans and animals.'),
  'D020566': NtO(MH='Dendritic Cells, Follicular', MS='Non-hematopoietic cells, with extensive dendritic processes, found in the primary and secondary follicles of lymphoid tissue (the B cell zones). They are different from conventional DENDRITIC CELLS associated with T-CELLS. They are derived from MESENCHYMAL STEM CELLS and are negative for class II MHC antigen and do not process or present antigen like the conventional dendritic cells do. Instead, follicular dendritic cells have FC RECEPTORS and C3B RECEPTORS that hold antigen in the form of ANTIGEN-ANTIBODY COMPLEXES on their surfaces for long periods for recognition by B-CELLS.'),
  'D020567': NtO(MH='Fetal Weight', MS='The weight of the FETUS in utero. It is usually estimated by various formulas based on measurements made during PRENATAL ULTRASONOGRAPHY.'),
  'D020569': NtO(MH='Proto-Oncogene Proteins c-rel', MS='Cellular DNA-binding proteins encoded by the rel gene (GENES, REL). They are expressed predominately in hematopoietic cells and may play a role in lymphocyte differentiation. Rel frequently combines with other related proteins (NF-KAPPA B, I-kappa B, relA) to form heterodimers that regulate transcription. Rearrangement or overexpression of c-rel can cause tumorigenesis.'),
  'D020570': NtO(MH='Genes, myb', MS='Retrovirus-associated DNA sequences (v-myb) originally isolated from the avian myeloblastosis and E26 leukemia viruses. The proto-oncogene c-myb codes for a nuclear protein involved in transcriptional regulation and appears to be essential for hematopoietic cell proliferation. The human myb gene is located at 6q22-23 on the short arm of chromosome 6. This is the point of break in translocations involved in T-cell acute lymphatic leukemia and in some ovarian cancers and melanomas. (From Ibelgaufts, Dictionary of Cytokines, 1995).'),
  'D020571': NtO(MH='Genes, sis', MS='Retrovirus-associated DNA sequences (v-sis) originally isolated from the simian sarcoma virus (SSV). The proto-oncogene c-sis codes for a growth factor which is the B chain of PLATELET-DERIVED GROWTH FACTOR. v-sis or overexpression of c-sis causes tumorigenesis. The human sis gene is located at 22q12.3-13.1 on the long arm of chromosome 22.'),
  'D020572': NtO(MH='Oncogene Proteins v-rel', MS='Transforming proteins coded by rel oncogenes. The v-rel protein competes with rel-related proteins and probably transforms cells by acting as a dominant negative version of c-rel. This results in the induction of a broad range of leukemias and lymphomas.'),
  'D020573': NtO(MH='Genes, rel', MS='Family of retrovirus-associated DNA sequences (v-rel) originally isolated from an avian reticuloendotheliosis virus strain. The proto-oncogene rel (c-rel) codes for a subcellular (nuclear and cytoplasmic) transcription factor that has a role in lymphocyte differentiation. Translocation or overexpression of c-rel or competition from v-rel causes oncogenesis. The human rel gene is located at 2p12-13 on the short arm of chromosome 2.'),
  'D020574': NtO(MH='Proto-Oncogene Proteins c-sis', MS='Cellular DNA-binding proteins encoded by the sis gene (GENES, SIS). c-sis proteins make up the B chain of PLATELET-DERIVED GROWTH FACTOR. Overexpression of c-sis causes tumorigenesis.'),
  'D020575': NtO(MH='Oncogene Proteins v-sis', MS='Transforming proteins coded by sis oncogenes. Transformation of cells by v-sis is related to its interaction with the PDGF receptor and also its ability to alter other transcription factors.'),
  'D020576': NtO(MH='Gluconobacter', MS="A genus of gram-negative, rod-shaped to ellipsoidal bacteria occurring singly or in pairs and found in flowers, soil, honey bees, fruits, cider, beer, wine, and vinegar. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020577': NtO(MH='Wolbachia', MS="A genus of bacteria which comprises a heterogenous group of gram-negative small rods and coccoid forms associated with arthropods. (From Bergey's Manual of Systematic Bacteriology, vol 1, 1984)"),
  'D020578': NtO(MH='Sphingomonas', MS='A genus of gram-negative, aerobic, rod-shaped bacteria characterized by an outer membrane that contains glycosphingolipids but lacks lipopolysaccharide. They have the ability to degrade a broad range of substituted aromatic compounds.'),
  'D020579': NtO(MH='Sinorhizobium', MS="A genus of gram-negative, aerobic, nonsporeforming rods which usually contain granules of poly-beta-hydroxybutyrate. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020580': NtO(MH='Methylobacterium', MS="A genus of gram-negative, aerobic, facultatively methylotrophic rods occurring singly or occasionally in rosettes. Members of this genus are usually motile and are isolated from soil, dust, fresh water, lake sediments, leaf surfaces, rice, air, and hospital environments. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020581': NtO(MH='Methylosinus', MS='A genus of gram-negative rods which form exospores and are obligate methanotrophs.'),
  'D020582': NtO(MH='Hyphomicrobium', MS="A genus of rod-shaped, oval, or bean-shaped bacteria found in soil and fresh water. Polar prosthecae are present and cells reproduce by budding at the tips of the prosthecae. Cells of this genus are aerobic and grow best with one-carbon compounds. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020583': NtO(MH='Azorhizobium', MS="A genus of gram-negative, rod-shaped, obligate aerobes which are motile by peritrichous flagella on solid medium and one lateral flagellum in liquid medium. Under microaerobic conditions Azorhizobium fixes nitrogen. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020584': NtO(MH='Xanthobacter', MS="A genus of gram-negative, aerobic, rod-shaped bacteria found in wet soil containing decaying organic material and in water. Cells tend to be pleomorphic if grown on media containing succinate or coccoid if grown in the presence of an alcohol as the sole carbon source. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020585': NtO(MH='Comamonas', MS='A genus of gram-negative, straight or slightly curved rods which are motile by polar flagella and which accumulate poly-beta-hydroxybutyrate within the cells.'),
  'D020586': NtO(MH='Thiocapsa', MS="A genus of anoxygenic, photosynthetic, nonmotile, spherical to slightly ovoid bacterial cells occurring singly, or in aggregates of two or four, and usually surrounded with slime. It is found in stagnant water, mud of ponds, estuaries, and microbial mats of salt marshes. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020587': NtO(MH='Ectothiorhodospira', MS="A genus of vibrioid or rod-shaped cells which are motile by polar flagella. Internal photosynthetic membranes are present as lamellar stacks and contain bacteriochlorophyll a or b and carotenoids. Growth occurs photoautotrophically under anaerobic conditions. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020588': NtO(MH='Stenotrophomonas', MS='A genus of gram-negative, aerobic, motile, rod-shaped bacteria formerly classified as part of the genus XANTHOMONAS.'),
  'D020589': NtO(MH='Vitreoscilla', MS="A genus of gram-negative, aerobic or microaerophilic, colorless filaments. It is nonfruiting, motile by gliding, and found in freshwater sediments and cow dung. One species (V. stercoraria) is considered morphologically to be a streptobacillus. That species is strictly aerobic and produces a homodimeric bacterial hemoglobin, especially under oxygen-limited growth conditions. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020591': NtO(MH='Alteromonas', MS="A genus of gram-negative, straight or curved rods which are motile by means of a single, polar flagellum. Members of this genus are found in coastal waters and the open ocean. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020592': NtO(MH='Shewanella', MS='A genus of gram-negative, facultatively anaerobic rods. It is a saprophytic, marine organism which is often isolated from spoiling fish.'),
  'D020593': NtO(MH='Xenorhabdus', MS="A genus of gram-negative, facultatively anaerobic rod-shaped cells which are motile by peritrichous flagella. Late in the growth cycle, spheroplasts or coccoid bodies occur, resulting from disintegration of the cell wall. The natural habitat is the intestinal lumen of certain nematodes. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020594': NtO(MH='Edwardsiella', MS="A genus of small, straight gram-negative rods which are facultatively anaerobic, chemoorganotrophic, and usually motile by peritrichous flagella. Members of this genus are usually found in the intestines of cold-blooded animals and in fresh water. They are pathogenic for eels, CATFISHES, and other animals and are rare opportunistic pathogens for humans. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020595': NtO(MH='Hafnia', MS="A genus of straight, gram-negative bacterial rods which are facultatively anaerobic and motile by peritrichous flagella. This genus is found in human and animal feces, soil, water, and dairy products. It is an opportunistic pathogen in humans. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020596': NtO(MH='Morganella', MS="A genus of gram-negative, facultatively anaerobic, straight rods which are motile by peritrichous flagella. These organisms are chemoorganotrophic and have both a respiratory and fermentative type of metabolism. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020597': NtO(MH='Stigmatella', MS="A genus of the MYXOCOCCALES having vegetative cells which are straight rods with tapered ends and myxospores which are short and somewhat crooked. Fruiting bodies consist of spherical, ovoid, or club-shaped sporangioles on stalks. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020598': NtO(MH='Proto-Oncogene Proteins c-myb', MS='Cellular DNA-binding proteins encoded by the myb gene (GENES, MYB). They are expressed in a wide variety of cells including thymocytes and lymphocytes, and regulate cell differentiation. Overexpression of myb is associated with autoimmune diseases and malignancies.'),
  'D020599': NtO(MH='Azorhizobium caulinodans', MS="A species of AZORHIZOBIUM which forms nodules on the roots of the tropical legume Sesbania rostrata. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020600': NtO(MH='Gluconobacter oxydans', MS="A rod-shaped to ellipsoidal, gram-negative bacterium which oxidizes ethanol to acetic acid and prefers sugar-enriched environments. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020601': NtO(MH='Methylobacterium extorquens', MS="A species of METHYLOBACTERIUM which can utilize acetate, ethanol, or methylamine as a sole carbon source. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020602': NtO(MH='Methylosinus trichosporium', MS='A species of METHYLOSINUS which is capable of degrading trichloroethylene and other organic pollutants.'),
  'D020604': NtO(MH='Comamonas testosteroni', MS='A species of gram-negative, aerobic rods formerly called Pseudomonas testosteroni. It is differentiated from other Comamonas species by its ability to assimilate testosterone and to utilize phenylacetate or maleate as carbon sources.'),
  'D020605': NtO(MH='Delftia acidovorans', MS='A species of gram-negative rod-shaped bacteria found ubiquitously and formerly called Comamonas acidovorans and Pseudomonas acidovorans. It is the type species of the genus DELFTIA.'),
  'D020606': NtO(MH='Stigmatella aurantiaca', MS="A species of STIGMATELLA usually isolated from rotting wood. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020607': NtO(MH='Halorhodospira halophila', MS='A species of Halorhodospira distinguished by its ability to grow optimally in an environment of 15-20% salinity. It was formerly called Ectothiorhodospira halophila.'),
  'D020608': NtO(MH='Ectothiorhodospira shaposhnikovii', MS="A species of ECTOTHIORHODOSPIRA distinguished by its ability to utilize nitrate. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020609': NtO(MH='Edwardsiella tarda', MS="A species of EDWARDSIELLA distinguished by its hydrogen sulfide production. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020610': NtO(MH='Edwardsiella ictaluri', MS="A species of EDWARDSIELLA distinguished by its nonmotility. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020611': NtO(MH='Hafnia alvei', MS="The type species for the genus HAFNIA. It is distinguished from other biochemically similar bacteria by its lack of acid production on media containing sucrose. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020613': NtO(MH='Morganella morganii', MS="A species of MORGANELLA formerly classified as a Proteus species. It is found in the feces of humans, dogs, other mammals, and reptiles. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020614': NtO(MH='Shewanella putrefaciens', MS='A species of SHEWANELLA noted for its ability to reduce iron and manganese anaerobically.'),
  'D020615': NtO(MH='Stenotrophomonas maltophilia', MS='A species of STENOTROPHOMONAS, formerly called Xanthomonas maltophilia, which reduces nitrate. It is a cause of hospital-acquired ocular and lung infections, especially in those patients with cystic fibrosis and those who are immunosuppressed.'),
  'D020616': NtO(MH='Thiocapsa roseopersicina', MS="A species of THIOCAPSA which is facultatively aerobic and chemotrophic and which can utilize thiosulfate. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020618': NtO(MH='Methylococcus', MS="A genus of gram-negative, aerobic, spherical cells usually occurring in pairs. The resting stage is considered a cyst. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020619': NtO(MH='Methylococcus capsulatus', MS="A species of METHYLOCOCCUS which forms capsules and is capable of autotrophic carbon dioxide fixation. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020620': NtO(MH='Methylomonas', MS="A genus of gram-negative, aerobic, straight, curved, or branched rods which are motile by a single polar flagellum. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020621': NtO(MH='Ochrobactrum anthropi', MS="A species of gram-negative, obligately aerobic rods. Motility occurs by peritrichous flagella. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020622': NtO(MH='Afipia', MS="A genus of gram-negative, oxidase-positive, nonfermentative rods which are motile by means of a single flagellum. Afipia felis and BARTONELLA HENSELAE are causative agents of CAT-SCRATCH DISEASE. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020623': NtO(MH='Rhodomicrobium', MS="A genus, in the family HYPHOMICROBIACEAE, comprised of ovoid-shaped bacteria with polar growth and a characteristic vegetative growth cycle. Cells grow preferably photoheterotrophically under anaerobic conditions in the light, but will also grow under microaerobic or aerobic conditions in the dark. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020624': NtO(MH='Oxalobacter formigenes', MS="The sole species of the genus Oxalobacter consisting of straight or curved gram-negative rods with rounded ends. Cells are nonmotile, nonsporing, and use oxylates as the only source of CARBON and energy, with formate and CARBON DIOXIDE as end products. They are isolated from lake sediments and from the rumen or large bowel of humans and animals. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020625': NtO(MH='Taylorella equigenitalis', MS="A species of bacteria which is comprised of gram-negative rods which often approach a spherical shape. They are nonmotile and microaerophilic. They are considered parasites of horses and are pathogenic for mares. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020626': NtO(MH='Oncogene Proteins v-myb', MS='Transforming proteins coded by myb oncogenes. Transformation of cells by v-myb in conjunction with v-ets is seen in the avian E26 leukemia virus.'),
  'D020627': NtO(MH='Azoarcus', MS='A genus of gram-negative, facultatively anaerobic bacteria including species which are often associated with grasses (POACEAE) and which fix nitrogen as well as species which anaerobically degrade toluene and other mono-aromatic hydrocarbons.'),
  'D020628': NtO(MH='Thauera', MS='A genus of gram-negative, rod-shaped bacteria able to anaerobically oxidize and degrade toluene.'),
  'D020629': NtO(MH='Methylophilus', MS="A genus of straight or slightly curved gram-negative rods occurring singly or in pairs and isolated from sludge, mud, and river and pond water. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020630': NtO(MH='Methylophilus methylotrophus', MS="A species of METHYLOPHILUS which is motile by single flagella. In addition to growth on methanol as a sole carbon source, growth also occurs on glucose. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020631': NtO(MH='Methylobacillus', MS="A genus of short, aerobic, gram-negative rods which are obligate methylotrophs, growing on one-carbon compounds other than methane. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020632': NtO(MH='Halomonas', MS="A genus of gram-negative, rod-shaped or pleomorphic bacteria which are halotolerant. Members of this genus are capable of growth in sodium chloride concentrations of up to 20% or more. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020633': NtO(MH='Cellvibrio', MS="A genus of aerobic, gram-negative, motile, slightly curved, rod-shaped bacteria. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020635': NtO(MH='Buchnera', MS='A genus of gram-negative bacteria which are obligately intracellular endosymbionts of APHIDS. The bacteria are found within specialized cells in the aphid body cavity.'),
  'D020636': NtO(MH='Pantoea', MS="A genus of gram-negative, facultatively anaerobic, straight rods which are motile by peritrichous flagella. Most strains produce a yellow pigment. This organism is isolated from plant surfaces, seeds, soil, and water, as well as from animals and human wounds, blood, and urine. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D020637': NtO(MH='Photorhabdus', MS='A genus of gram-negative bacteria existing symbiotically with nematodes of the family Heterorhabditidae (see RHABDITOIDEA). These nematodes infect a variety of soil-dwelling insects. Upon entering an insect host, the nematode releases Photorhabdus from its intestinal tract and the bacterium establishes a lethal septicemia in the insect.'),
  'D020638': NtO(MH='Rahnella', MS='A genus of gram-negative, facultatively anaerobic, small, rod-shaped bacteria occurring in fresh water.'),
  'D020639': NtO(MH='Lawsonia Bacteria', MS='A genus of gram-negative, obligate intracellular bacteria causing a proliferative enteritis in animals, especially pigs, deer, horses, and rabbits.'),
  'D020640': NtO(MH='Arcobacter', MS='A genus of gram-negative, aerotolerant, spiral-shaped bacteria isolated from water and associated with diarrhea in humans and animals.'),
  'D020641': NtO(MH='Polymorphism, Single Nucleotide', MS='A single nucleotide variation in a genetic sequence that occurs at appreciable frequency in the population.'),
  'D020642': NtO(MH='Acatalasia', MS='A rare autosomal recessive disorder resulting from the absence of CATALASE activity. Though usually asymptomatic, a syndrome of oral ulcerations and gangrene may be present.'),
  'D020643': NtO(MH='Anterior Thalamic Nuclei', MS='Three nuclei located beneath the dorsal surface of the most rostral part of the thalamus. The group includes the anterodorsal nucleus, anteromedial nucleus, and anteroventral nucleus. All receive connections from the MAMILLARY BODY and BRAIN FORNIX, and project fibers to the CINGULATE BODY.'),
  'D020644': NtO(MH='Midline Thalamic Nuclei', MS='Small, nonspecific nerve cells scattered in the periventricular GRAY MATTER, separating the medial part of the thalamus from the EPENDYMA of the THIRD VENTRICLE. The group includes the paraventricular nucleus, paratenial nucleus, reuniens nucleus, rhomboidal nucleus, and subfascular nucleus.'),
  'D020645': NtO(MH='Mediodorsal Thalamic Nucleus', MS='The largest of the medial nuclei of the thalamus. It makes extensive connections with most of the other thalamic nuclei.'),
  'D020646': NtO(MH='Intralaminar Thalamic Nuclei', MS='Cell groups within the internal medullary lamina of the THALAMUS. They include a rostral division comprising the paracentral, central lateral, central dorsal, and central medial nuclei, and a caudal division composed of the centromedian and parafascicular nuclei.'),
  'D020647': NtO(MH='Lateral Thalamic Nuclei', MS='A narrow strip of cell groups on the dorsomedial surface of the thalamus. It includes the lateral dorsal nucleus, lateral posterior nucleus, and the PULVINAR.'),
  'D020648': NtO(MH='Peptide Elongation Factor 1', MS='Peptide elongation factor 1 is a multisubunit protein that is responsible for the GTP-dependent binding of aminoacyl-tRNAs to eukaryotic ribosomes. The alpha subunit (EF-1alpha) binds aminoacyl-tRNA and transfers it to the ribosome in a process linked to GTP hydrolysis. The beta and delta subunits (EF-1beta, EF-1delta) are involved in exchanging GDP for GTP. The gamma subunit (EF-1gamma) is a structural component.'),
  'D020649': NtO(MH='Pulvinar', MS='Large mass of nuclei forming the most caudal portion of the THALAMUS and overhanging the GENICULATE BODIES and the dorsolateral surface of the MIDBRAIN. It is divided into four parts: the lateral, medial, inferior, and oral pulvinar nuclei.'),
  'D020650': NtO(MH='Combinatorial Chemistry Techniques', MS='A technology, in which sets of reactions for solution or solid-phase synthesis, is used to create molecular libraries for analysis of compounds on a large scale.'),
  'D020651': NtO(MH='Ventral Thalamic Nuclei', MS='A large group of nuclei lying between the internal medullary lamina and the INTERNAL CAPSULE. It includes the ventral anterior, ventral lateral, and ventral posterior nuclei.'),
  'D020652': NtO(MH='Peptide Elongation Factor 2', MS='Peptide Elongation Factor 2 catalyzes the translocation of peptidyl-tRNA from the A site to the P site of eukaryotic ribosomes by a process linked to the hydrolysis of GTP to GDP.'),
  'D020653': NtO(MH='Peptide Elongation Factor G', MS='Peptide Elongation Factor G catalyzes the translocation of peptidyl-tRNA from the A to the P site of bacterial ribosomes by a process linked to hydrolysis of GTP to GDP.'),
  'D020656': NtO(MH='Posterior Thalamic Nuclei', MS='A transitional diencephalic zone of the thalamus consisting of complex and varied cells lying caudal to the VENTRAL POSTEROLATERAL NUCLEUS, medial to the rostral part of the PULVINAR, and dorsal to the MEDIAL GENICULATE BODY. It contains the limitans, posterior, suprageniculate, and submedial nuclei.'),
  'D020662': NtO(MH='Guanine Nucleotide Exchange Factors', MS='Protein factors that promote the exchange of GTP for GDP bound to GTP-BINDING PROTEINS.'),
  'D020663': NtO(MH='ras Guanine Nucleotide Exchange Factors', MS='A family of GUANINE NUCLEOTIDE EXCHANGE FACTORS that are specific for RAS PROTEINS.'),
  'D020664': NtO(MH='Son of Sevenless Proteins', MS='A class of RAS GUANINE NUCLEOTIDE EXCHANGE FACTORS that are genetically related to the Son of Sevenless gene from DROSOPHILA. Sevenless refers to genetic mutations in DROSOPHILA that cause loss of the R7 photoreceptor which is required to see UV light.'),
  'D020665': NtO(MH='Septum of Brain', MS='GRAY MATTER structures of the telencephalon and LIMBIC SYSTEM in the brain, but containing widely varying definitions among authors. Included here is the cortical septal area, subcortical SEPTAL NUCLEI, and the SEPTUM PELLUCIDUM.'),
  'D020666': NtO(MH='ras-GRF1', MS='A guanine nucleotide exchange factor that is expressed primarily in neuronal tissue and may be specific for the Ha-ras homolog of the RAS PROTEINS.'),
  'D020667': NtO(MH='Diagonal Band of Broca', MS='Cholinergic bundle of nerve fibers posterior to the anterior perforated substance. It interconnects the paraterminal gyrus in the septal area with the hippocampus and lateral olfactory area.'),
  'D020668': NtO(MH='rap GTP-Binding Proteins', MS='A family of MONOMERIC GTP-BINDING PROTEINS that are related to RAS PROTEINS.This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020669': NtO(MH='Fetoscopes', MS='Endoscopes for viewing the embryo, fetus and amniotic cavity.'),
  'D020670': NtO(MH='Islands of Calleja', MS='Aggregations of GRANULE CELLS in the dorsal aspect of the OLFACTORY TUBERCLE and within the Island of Calleja Magna that lies between the BRAIN SEPTUM and the NUCLEUS ACCUMBENS.'),
  'D020671': NtO(MH='Posterior Horn Cells', MS='Neurons in the SPINAL CORD DORSAL HORN whose cell bodies and processes are confined entirely to the CENTRAL NERVOUS SYSTEM. They receive collateral or direct terminations of dorsal root fibers. They send their axons either directly to ANTERIOR HORN CELLS or to the WHITE MATTER ascending and descending longitudinal fibers.'),
  'D020672': NtO(MH='I-kappa B Proteins', MS='A family of inhibitory proteins which bind to the REL PROTO-ONCOGENE PROTEINS and modulate their activity. In the CYTOPLASM, I-kappa B proteins bind to the transcription factor NF-KAPPA B. Cell stimulation causes its dissociation and translocation of active NF-kappa B to the nucleus.'),
  'D020673': NtO(MH='Protein Renaturation', MS="The reconstitution of a protein's activity following denaturation."),
  'D020675': NtO(MH='Peroxisomes', MS='Microbodies which occur in animal and plant cells and in certain fungi and protozoa. They contain peroxidase, catalase, and allied enzymes. (From Singleton and Sainsbury, Dictionary of Microbiology and Molecular Biology, 2nd ed)'),
  'D020676': NtO(MH='Glyoxysomes', MS='Microbodies which occur in plant cells, and in some eukaryotic microorganisms, and which contain enzymes of the glyoxylate cycle. (Singleton and Stainsbury, Dictionary of Microbiology and Molecular Biology, 2nd ed)'),
  'D020677': NtO(MH='Angioscopes', MS='Endoscopes used for viewing the interior of blood vessels.'),
  'D020678': NtO(MH='Microscopic Angioscopy', MS='The noninvasive microscopic examination of the microcirculation, commonly done in the nailbed or conjunctiva. In addition to the capillaries themselves, observations can be made of passing blood cells or intravenously injected substances. This is not the same as endoscopic examination of blood vessels (ANGIOSCOPY).'),
  'D020679': NtO(MH='Arthroscopes', MS='Endoscopes for visualizing the interior of a joint.'),
  'D020680': NtO(MH='Bronchoscopes', MS='Endoscopes for the visualization of the interior of the bronchi.'),
  'D020681': NtO(MH='Colposcopes', MS='Instruments inserted into the vagina for examination of the tissues of the vagina and cervix by means of a magnifying lens.'),
  'D020682': NtO(MH='Cefixime', MS='A third-generation cephalosporin antibiotic that is stable to hydrolysis by beta-lactamases.'),
  'D020683': NtO(MH='Culdoscopes', MS='Endoscopic instruments used for viewing the rectovaginal pouch and pelvic viscera.'),
  'D020684': NtO(MH='Cystoscopes', MS='Endoscopes for visual examination of the urinary bladder.'),
  'D020685': NtO(MH='Colonoscopes', MS='Specially designed endoscopes for visualizing the interior surface of the colon.'),
  'D020687': NtO(MH='cdc25 Phosphatases', MS='A subclass of dual specificity phosphatases that play a role in the progression of the CELL CYCLE. They dephosphorylate and activate CYCLIN-DEPENDENT KINASES.'),
  'D020689': NtO(MH='Exopeptidases', MS='A sub-class of PEPTIDE HYDROLASES that act only near the ends of polypeptide chains.'),
  'D020690': NtO(MH='GTPase-Activating Proteins', MS='Proteins that activate the GTPase of specific GTP-BINDING PROTEINS.'),
  'D020691': NtO(MH='rab GTP-Binding Proteins', MS='A large family of MONOMERIC GTP-BINDING PROTEINS that play a key role in cellular secretory and endocytic pathways.'),
  'D020692': NtO(MH='rab3 GTP-Binding Proteins', MS='A genetically related subfamily of RAB GTP-BINDING PROTEINS involved in calcium-dependent EXOCYTOSIS. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020693': NtO(MH='rab2 GTP-Binding Protein', MS='A protein involved in transport between the ENDOPLASMIC RETICULUM and the GOLGI APPARATUS. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020694': NtO(MH='Sigmoidoscopes', MS='Endoscopes for examining the interior of the sigmoid colon.'),
  'D020695': NtO(MH='rab4 GTP-Binding Proteins', MS='A genetically related subfamily of RAB GTP-BINDING PROTEINS involved in recycling of proteins such as cell surface receptors from early endosomes to the cell surface. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020696': NtO(MH='rab5 GTP-Binding Proteins', MS='A genetically related subfamily of RAB GTP-BINDING PROTEINS involved in transport from the cell membrane to early endosomes. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020698': NtO(MH='Duodenoscopes', MS='Endoscopes for examining the interior of the duodenum.'),
  'D020699': NtO(MH='Gastroscopes', MS='Endoscopes used for examining the interior of the stomach.'),
  'D020700': NtO(MH='Proctoscopes', MS='Endoscopes for visual examination of the rectum and anus.'),
  'D020701': NtO(MH='Esophagoscopes', MS='Endoscopes for examining the interior of the esophagus.'),
  'D020702': NtO(MH='Hysteroscopes', MS='Endoscopes for examining the interior of the uterus.'),
  'D020703': NtO(MH='ras GTPase-Activating Proteins', MS='PROTEINS that specifically activate the GTP-phosphohydrolase activity of RAS PROTEINS.'),
  'D020704': NtO(MH='Laryngoscopes', MS='Endoscopes for examining the interior of the larynx.'),
  'D020706': NtO(MH='Laparoscopes', MS='ENDOSCOPES for examining the abdominal and pelvic organs in the peritoneal cavity.'),
  'D020707': NtO(MH='Mediastinoscopes', MS='Endoscopes for examining tissue of the anterior superior mediastinum.'),
  'D020708': NtO(MH='Thoracoscopes', MS='Endoscopes for examining the pleural cavity.'),
  'D020709': NtO(MH='Ureteroscopes', MS='Endoscopes for examining the interior of the ureter.'),
  'D020710': NtO(MH='RGS Proteins', MS='A large family of evolutionarily conserved proteins that function as negative regulators of HETEROTRIMERIC GTP-BINDING PROTEINS. RGS PROTEINS act by increasing the GTPase activity of the G alpha subunit of a heterotrimeric GTP-binding protein, causing it to revert to its inactive (GDP-bound) form.'),
  'D020711': NtO(MH='Threshold Limit Values', MS='Standards for limiting worker exposure to airborne contaminants. They are the maximum concentration in air at which it is believed that a particular substance will not produce adverse health effects with repeated daily exposure. It can be a time-weighted average (TLV-TWA), a short-term value (TLV-STEL), or an instantaneous value (TLV-Ceiling). They are expressed either as parts per million (ppm) or milligram per cubic meter (mg/m3).'),
  'D020712': NtO(MH='Fornix, Brain', MS='Heavily myelinated fiber bundle of the TELENCEPHALON projecting from the hippocampal formation to the HYPOTHALAMUS. Some authorities consider the fornix part of the LIMBIC SYSTEM. The fimbria starts as a flattened band of axons arising from the subiculum and HIPPOCAMPUS, which then thickens to form the fornix.'),
  'D020713': NtO(MH='Vancomycin Resistance', MS='Nonsusceptibility of bacteria to the action of VANCOMYCIN, an inhibitor of cell wall synthesis.'),
  'D020714': NtO(MH='Maximum Tolerated Dose', MS='The highest dose of a biologically active agent given during a chronic study that will not reduce longevity from effects other than carcinogenicity. (from Lewis Dictionary of Toxicology, 1st ed)'),
  'D020717': NtO(MH='Eukaryotic Initiation Factor-2B', MS='A guanine nucleotide exchange factor that acts to restore EUKARYOTIC INITIATION FACTOR-2 to its GTP bound form.'),
  'D020718': NtO(MH='Ribonucleoproteins, Small Nucleolar', MS='Nucleolar RNA-protein complexes that function in pre-ribosomal RNA processing.'),
  'D020719': NtO(MH='Glucuronides', MS='Glycosides of GLUCURONIC ACID formed by the reaction of URIDINE DIPHOSPHATE GLUCURONIC ACID with certain endogenous and exogenous substances. Their formation is important for the detoxification of drugs, steroid excretion and BILIRUBIN metabolism to a more water-soluble compound that can be eliminated in the URINE and BILE.'),
  'D020720': NtO(MH='Myasthenia Gravis, Autoimmune, Experimental', MS='Any autoimmune animal disease model used in the study of MYASTHENIA GRAVIS. Injection with purified neuromuscular junction acetylcholine receptor (AChR) (see RECEPTORS, CHOLINERGIC) components results in a myasthenic syndrome that has acute and chronic phases. The motor endplate pathology, loss of acetylcholine receptors, presence of circulating anti-AChR antibodies, and electrophysiologic changes make this condition virtually identical to human myasthenia gravis. Passive transfer of AChR antibodies or lymphocytes from afflicted animals to normals induces passive transfer experimental autoimmune myasthenia gravis. (From Joynt, Clinical Neurology, 1997, Ch 54, p3)'),
  'D020721': NtO(MH='Nervous System Autoimmune Disease, Experimental', MS='Experimental animal models for human AUTOIMMUNE DISEASES OF THE NERVOUS SYSTEM. They include GUILLAIN-BARRE SYNDROME (see NEURITIS, AUTOIMMUNE, EXPERIMENTAL); MYASTHENIA GRAVIS (see MYASTHENIA GRAVIS, AUTOIMMUNE, EXPERIMENTAL); and MULTIPLE SCLEROSIS (see ENCEPHALOMYELITIS, AUTOIMMUNE, EXPERIMENTAL).'),
  'D020723': NtO(MH='Glucuronic Acid', MS='A sugar acid formed by the oxidation of the C-6 carbon of GLUCOSE. In addition to being a key intermediate metabolite of the uronic acid pathway, glucuronic acid also plays a role in the detoxification of certain drugs and toxins by conjugating with them to form GLUCURONIDES.'),
  'D020724': NtO(MH='Elastomers', MS='A generic term for all substances having the properties of stretching under tension, high tensile strength, retracting rapidly, and recovering their original dimensions fully. They are generally POLYMERS.'),
  'D020725': NtO(MH='ral GTP-Binding Proteins', MS='A family of ubiquitously expressed MONOMERIC GTP-BINDING PROTEINS that are involved in intracellular signal transduction. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020726': NtO(MH='ral Guanine Nucleotide Exchange Factor', MS='A guanine nucleotide exchange factor that stimulates the dissociation of GDP from RAL GTP-BINDING PROTEINS. It also has GDP exchange activity towards other MONOMERIC GTP-BINDING PROTEINS.'),
  'D020727': NtO(MH='ADP-Ribosylation Factors', MS='MONOMERIC GTP-BINDING PROTEINS that were initially recognized as allosteric activators of the MONO(ADP-RIBOSE) TRANSFERASE of the CHOLERA TOXIN catalytic subunit. They are involved in vesicle trafficking and activation of PHOSPHOLIPASE D. This enzyme was formerly listed as EC 3.6.1.47'),
  'D020728': NtO(MH='Transurethral Resection of Prostate', MS='Removal of all or part of the PROSTATE, often using a cystoscope and/or resectoscope passed through the URETHRA.'),
  'D020729': NtO(MH='p120 GTPase Activating Protein', MS='A 120-kDa RAS GTPase-activating protein that binds to tyrosine phosphoproteins through its SH2 domains. The 100-kDa RNA-splicing variant (p100 GAP protein) is expressed in placenta.'),
  'D020730': NtO(MH='Guanine Nucleotide Dissociation Inhibitors', MS='Protein factors that inhibit the dissociation of GDP from GTP-BINDING PROTEINS.'),
  'D020731': NtO(MH='Keratomileusis, Laser In Situ', MS='A surgical procedure to correct MYOPIA by CORNEAL STROMA subtraction. It involves the use of a microkeratome to make a lamellar dissection of the CORNEA creating a flap with intact CORNEAL EPITHELIUM. After the flap is lifted, the underlying midstroma is reshaped with an EXCIMER LASER and the flap is returned to its original position.'),
  'D020732': NtO(MH='Cytogenetic Analysis', MS='Examination of CHROMOSOMES to diagnose, classify, screen for, or manage genetic diseases and abnormalities. Following preparation of the sample, KARYOTYPING is performed and/or the specific chromosomes are analyzed.'),
  'D020733': NtO(MH='RNA, Small Cytoplasmic', MS='Small RNAs found in the cytoplasm usually complexed with proteins in scRNPs (RIBONUCLEOPROTEINS, SMALL CYTOPLASMIC).'),
  'D020734': NtO(MH='Parkinsonian Disorders', MS='A group of disorders which feature impaired motor control characterized by bradykinesia, MUSCLE RIGIDITY; TREMOR; and postural instability. Parkinsonian diseases are generally divided into primary parkinsonism (see PARKINSON DISEASE), secondary parkinsonism (see PARKINSON DISEASE, SECONDARY) and inherited forms. These conditions are associated with dysfunction of dopaminergic or closely related motor integration neuronal pathways in the BASAL GANGLIA.'),
  'D020735': NtO(MH='Guanine Nucleotide-Releasing Factor 2', MS='A 145-kDa guanine nucleotide exchange factor that is specific for rap1 and ras GTP-BINDING PROTEINS. It associates with SH3 domains of the crk family of signaling proteins.'),
  'D020736': NtO(MH='Ribonucleoproteins, Small Cytoplasmic', MS='Complexes of scRNA (RNA, SMALL CYTOPLASMIC) and protein found in the cytoplasm. An example is SIGNAL RECOGNITION PARTICLE.'),
  'D020738': NtO(MH='Leptin', MS='A 16-kDa peptide hormone secreted from WHITE ADIPOCYTES. Leptin serves as a feedback signal from fat cells to the CENTRAL NERVOUS SYSTEM in regulation of food intake, energy balance, and fat storage.'),
  'D020739': NtO(MH='Brain Diseases, Metabolic, Inborn', MS='Brain disorders resulting from inborn metabolic errors, primarily from enzymatic defects which lead to substrate accumulation, product reduction, or increase in toxic metabolites through alternate pathways. The majority of these conditions are familial, however spontaneous mutation may also occur in utero.'),
  'D020740': NtO(MH='Chimerin Proteins', MS='A family of GTPASE-ACTIVATING PROTEINS that are specific for RAC GTP-BINDING PROTEINS.'),
  'D020741': NtO(MH='rho GTP-Binding Proteins', MS='A large family of MONOMERIC GTP-BINDING PROTEINS that are involved in regulation of actin organization, gene expression and cell cycle progression. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020742': NtO(MH='rhoA GTP-Binding Protein', MS='A RHO GTP-BINDING PROTEIN involved in regulating signal transduction pathways that control assembly of focal adhesions and actin stress fibers. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020743': NtO(MH='rhoB GTP-Binding Protein', MS='A GTP-BINDING PROTEIN involved in regulating a signal transduction pathway that controls assembly of focal adhesions and actin stress fibers. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020744': NtO(MH='rac GTP-Binding Proteins', MS='A sub-family of RHO GTP-BINDING PROTEINS that is involved in regulating the organization of cytoskeletal filaments. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020745': NtO(MH='Genome, Archaeal', MS='The genetic complement of an archaeal organism (ARCHAEA) as represented in its DNA.'),
  'D020746': NtO(MH='Calcium Channels, L-Type', MS='Long-lasting voltage-gated CALCIUM CHANNELS found in both excitable and nonexcitable tissue. They are responsible for normal myocardial and vascular smooth muscle contractility. Five subunits (alpha-1, alpha-2, beta, gamma, and delta) make up the L-type channel. The alpha-1 subunit is the binding site for calcium-based antagonists. Dihydropyridine-based calcium antagonists are used as markers for these binding sites.'),
  'D020747': NtO(MH='Calcium Channels, T-Type', MS='A heterogenous group of transient or low voltage activated type CALCIUM CHANNELS. They are found in cardiac myocyte membranes, the sinoatrial node, Purkinje cells of the heart and the central nervous system.'),
  'D020748': NtO(MH='Mibefradil', MS='A benzimidazoyl-substituted tetraline that selectively binds and inhibits CALCIUM CHANNELS, T-TYPE.'),
  'D020751': NtO(MH='Alcohol-Induced Disorders', MS='Disorders stemming from the misuse and abuse of alcohol.'),
  'D020752': NtO(MH='Neurocutaneous Syndromes', MS='A group of disorders characterized by ectodermal-based malformations and neoplastic growths in the skin, nervous system, and other organs.'),
  'D020754': NtO(MH='Spinocerebellar Ataxias', MS='A group of dominantly inherited, predominately late-onset, cerebellar ataxias which have been divided into multiple subtypes based on clinical features and genetic mapping. Progressive ataxia is a central feature of these conditions, and in certain subtypes POLYNEUROPATHY; DYSARTHRIA; visual loss; and other disorders may develop. (From Joynt, Clinical Neurology, 1997, Ch65, pp 12-17; J Neuropathol Exp Neurol 1998 Jun;57(6):531-43)'),
  'D020755': NtO(MH='Coat Protein Complex I', MS='A protein complex comprised of COATOMER PROTEIN and ADP RIBOSYLATION FACTOR 1. It is involved in transport of vesicles between the ENDOPLASMIC RETICULUM and the GOLGI APPARATUS.'),
  'D020756': NtO(MH='Coatomer Protein', MS="A 700-kDa cytosolic protein complex consisting of seven equimolar subunits (alpha, beta, beta', gamma, delta, epsilon and zeta). COATOMER PROTEIN and ADP-RIBOSYLATION FACTOR 1 are principle components of COAT PROTEIN COMPLEX I and are involved in vesicle transport between the ENDOPLASMIC RETICULUM and the GOLGI APPARATUS."),
  'D020757': NtO(MH='Amaurosis Fugax', MS='Transient complete or partial monocular blindness due to retinal ischemia. This may be caused by emboli from the CAROTID ARTERY (usually in association with CAROTID STENOSIS) and other locations that enter the central RETINAL ARTERY. (From Adams et al., Principles of Neurology, 6th ed, p245)'),
  'D020758': NtO(MH='Spinal Cord Vascular Diseases', MS='Pathological processes involving any of the BLOOD VESSELS feeding the SPINAL CORD, such as the anterior and paired posterior spinal arteries or their many branches. Disease processes may include ATHEROSCLEROSIS; EMBOLISM; and ARTERIOVENOUS MALFORMATIONS leading to ISCHEMIA or HEMORRHAGE into the spinal cord (hematomyelia).'),
  'D020759': NtO(MH='Anterior Spinal Artery Syndrome', MS='Ischemia or infarction of the spinal cord in the distribution of the anterior spinal artery, which supplies the ventral two-thirds of the spinal cord. This condition is usually associated with ATHEROSCLEROSIS of the aorta and may result from dissection of an AORTIC ANEURYSM or rarely dissection of the anterior spinal artery. Clinical features include weakness and loss of pain and temperature sensation below the level of injury, with relative sparing of position and vibratory sensation. (From Adams et al., Principles of Neurology, 6th ed, pp1249-50)'),
  'D020760': NtO(MH='Spinal Cord Ischemia', MS='Reduced blood flow to the spinal cord which is supplied by the anterior spinal artery and the paired posterior spinal arteries. This condition may be associated with ARTERIOSCLEROSIS, trauma, emboli, diseases of the aorta, and other disorders. Prolonged ischemia may lead to INFARCTION of spinal cord tissue.'),
  'D020761': NtO(MH='rab1 GTP-Binding Proteins', MS='A genetically related subfamily of RAB GTP-BINDING PROTEINS involved in vesicle transport between the ENDOPLASMIC RETICULUM and the GOLGI APPARATUS and through early Golgi compartments. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020762': NtO(MH='Infarction, Posterior Cerebral Artery', MS='NECROSIS induced by ISCHEMIA in the POSTERIOR CEREBRAL ARTERY distribution system which supplies portions of the BRAIN STEM; the THALAMUS; TEMPORAL LOBE, and OCCIPITAL LOBE. Depending on the size and location of infarction, clinical features include OLFACTION DISORDERS and visual problems (AGNOSIA; ALEXIA; HEMIANOPSIA).'),
  'D020763': NtO(MH='Pathological Conditions, Anatomical', MS='An abnormal structural condition of the human body, usually macroscopic, that is common to a variety of different diseases.'),
  'D020764': NtO(MH='cdc42 GTP-Binding Protein', MS='A member of the Rho family of MONOMERIC GTP-BINDING PROTEINS. It is associated with a diverse array of cellular functions including cytoskeletal changes, filopodia formation and transport through the GOLGI APPARATUS. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020765': NtO(MH='Intracranial Arterial Diseases', MS='Pathological conditions involving ARTERIES in the skull, such as arteries supplying the CEREBRUM, the CEREBELLUM, the BRAIN STEM, and associated structures. They include atherosclerotic, congenital, traumatic, infectious, inflammatory, and other pathological processes.'),
  'D020766': NtO(MH='Intracranial Embolism', MS='Blocking of a blood vessel in the SKULL by an EMBOLUS which can be a blood clot (THROMBUS) or other undissolved material in the blood stream. Most emboli are of cardiac origin and are associated with HEART DISEASES. Other non-cardiac sources of emboli are usually associated with VASCULAR DISEASES.'),
  'D020767': NtO(MH='Intracranial Thrombosis', MS='Formation or presence of a blood clot (THROMBUS) in a blood vessel within the SKULL. Intracranial thrombosis can lead to thrombotic occlusions and BRAIN INFARCTION. The majority of the thrombotic occlusions are associated with ATHEROSCLEROSIS.'),
  'D020768': NtO(MH='Middle Cerebral Artery', MS='The largest of the cerebral arteries. It trifurcates into temporal, frontal, and parietal branches supplying blood to most of the parenchyma of these lobes in the CEREBRAL CORTEX. These are the areas involved in motor, sensory, and speech activities.'),
  'D020769': NtO(MH='Posterior Cerebral Artery', MS='Artery formed by the bifurcation of the BASILAR ARTERY. Branches of the posterior cerebral artery supply portions of the OCCIPITAL LOBE; PARIETAL LOBE; inferior temporal gyrus, brainstem, and CHOROID PLEXUS.'),
  'D020770': NtO(MH='Compomers', MS='Composite materials composed of an ion-leachable glass embedded in a polymeric matrix. They differ from GLASS IONOMER CEMENTS in that partially silanized glass particles are used to provide a direct bond to the resin matrix and the matrix is primarily formed by a light-activated, radical polymerization reaction.'),
  'D020771': NtO(MH='Anterior Cerebral Artery', MS='Artery formed by the bifurcation of the internal carotid artery (CAROTID ARTERY, INTERNAL). Branches of the anterior cerebral artery supply the CAUDATE NUCLEUS; INTERNAL CAPSULE; PUTAMEN; SEPTAL NUCLEI; GYRUS CINGULI; and surfaces of the FRONTAL LOBE and PARIETAL LOBE.'),
  'D020772': NtO(MH='Internal Capsule', MS='WHITE MATTER pathway, flanked by nuclear masses, consisting of both afferent and efferent fibers projecting between the WHITE MATTER and the BRAINSTEM. It consists of three distinct parts: an anterior limb, posterior limb, and genu.'),
  'D020773': NtO(MH='Headache Disorders', MS='Various conditions with the symptom of HEADACHE. Headache disorders are classified into major groups, such as PRIMARY HEADACHE DISORDERS (based on characteristics of their headache symptoms) and SECONDARY HEADACHE DISORDERS (based on their etiologies). (International Classification of Headache Disorders, 2nd ed. Cephalalgia 2004: suppl 1)'),
  'D020774': NtO(MH='Pick Disease of the Brain', MS='A rare form of DEMENTIA that is sometimes familial. Clinical features include APHASIA; APRAXIA; CONFUSION; ANOMIA; memory loss; and personality deterioration. This pattern is consistent with the pathologic findings of circumscribed atrophy of the poles of the FRONTAL LOBE and TEMPORAL LOBE. Neuronal loss is maximal in the HIPPOCAMPUS, entorhinal cortex, and AMYGDALA. Some ballooned cortical neurons contain argentophylic (Pick) bodies. (From Brain Pathol 1998 Apr;8(2):339-54; Adams et al., Principles of Neurology, 6th ed, pp1057-9)'),
  'D020775': NtO(MH='Thoracic Surgery, Video-Assisted', MS='Endoscopic surgery of the pleural cavity performed with visualization via video transmission.'),
  'D020776': NtO(MH='Endoscopes, Gastrointestinal', MS='Instruments for the visual examination of the interior of the gastrointestinal tract.'),
  'D020778': NtO(MH='Matrix Metalloproteinase 2', MS='A secreted endopeptidase homologous with INTERSTITIAL COLLAGENASE, but which possesses an additional fibronectin-like domain.'),
  'D020779': NtO(MH='Sinus Pericranii', MS='Rare vascular anomaly involving a communication between the intracranial and extracranial venous circulation via diploe, the central spongy layer of cranial bone. It is often characterized by dilated venous structures on the scalp due to abnormal drainage from the intracranial venous sinuses. Sinus pericranii can be congenital or traumatic in origin.'),
  'D020780': NtO(MH='Matrix Metalloproteinase 9', MS='An endopeptidase that is structurally similar to MATRIX METALLOPROTEINASE 2. It degrades GELATIN types I and V; COLLAGEN TYPE IV; and COLLAGEN TYPE V.'),
  'D020781': NtO(MH='Matrix Metalloproteinase 1', MS='A member of the metalloproteinase family of enzymes that is principally responsible for cleaving FIBRILLAR COLLAGEN. It can degrade interstitial collagens, types I, II and III.'),
  'D020782': NtO(MH='Matrix Metalloproteinases', MS='A family of zinc-dependent metalloendopeptidases that is involved in the degradation of EXTRACELLULAR MATRIX components.'),
  'D020783': NtO(MH='Matrix Metalloproteinase 7', MS='The smallest member of the MATRIX METALLOPROTEINASES. It plays a role in tumor progression.'),
  'D020784': NtO(MH='Matrix Metalloproteinase 8', MS='A member of the MATRIX METALLOPROTEINASES that cleaves triple-helical COLLAGEN types I, II, and III.'),
  'D020785': NtO(MH='Central Nervous System Vascular Malformations', MS='Congenital, inherited, or acquired abnormalities involving ARTERIES; VEINS; or venous sinuses in the BRAIN; SPINAL CORD; and MENINGES.'),
  'D020786': NtO(MH='Hemangioma, Cavernous, Central Nervous System', MS='A vascular anomaly composed of a collection of large, thin walled tortuous VEINS that can occur in any part of the central nervous system but lack intervening nervous tissue. Familial occurrence is common and has been associated with a number of genes mapped to 7q, 7p and 3q. Clinical features include SEIZURES; HEADACHE; STROKE; and progressive neurological deficit.'),
  'D020787': NtO(MH='Central Nervous System Venous Angioma', MS="A vascular anomaly characterized by a radial or wedge-shaped arrangement of dilated VEINS draining into a larger vein in the brain, spinal cord, or the meninges. Veins in a venous angioma are surrounded by normal nervous tissue, unlike a CENTRAL NERVOUS SYSTEM CAVERNOUS HEMANGIOMA that lacks intervening nervous tissue. Drainage of venous angioma is fully integrated with the body's venous system, therefore, in most cases there is no clinical signs and rare bleeding."),
  'D020788': NtO(MH='Bardet-Biedl Syndrome', MS='An autosomal recessive disorder characterized by RETINITIS PIGMENTOSA; POLYDACTYLY; OBESITY; MENTAL RETARDATION; hypogenitalism; renal dysplasia; and short stature. This syndrome has been distinguished as a separate entity from LAURENCE-MOON SYNDROME. (From J Med Genet 1997 Feb;34(2):92-8)'),
  'D020789': NtO(MH='Cathepsin C', MS='A papain-like cysteine protease that has specificity for amino terminal dipeptides. The enzyme plays a role in the activation of several pro-inflammatory serine proteases by removal of their aminoterminal inhibitory dipeptides. Genetic mutations that cause loss of cathepsin C activity in humans are associated with PAPILLON-LEFEVRE DISEASE.'),
  'D020790': NtO(MH='Diabetes Insipidus, Neurogenic', MS='A genetic or acquired polyuric disorder caused by a deficiency of VASOPRESSINS secreted by the NEUROHYPOPHYSIS. Clinical signs include the excretion of large volumes of dilute URINE; HYPERNATREMIA; THIRST; and polydipsia. Etiologies include HEAD TRAUMA; surgeries and diseases involving the HYPOTHALAMUS and the PITUITARY GLAND. This disorder may also be caused by mutations of genes such as ARVP encoding vasopressin and its corresponding neurophysin (NEUROPHYSINS).'),
  'D020791': NtO(MH='Venombin A', MS='A thrombin-like enzyme from the venom of snakes of the viper/rattlesnake group. Species variants of the enzyme include ANCROD (Agkistrodon rhodostoma (Malayan pit viper)); BATROXOBIN from Bothrops atrox; and crotalase from Crotalus adamanteus (Eastern diamondback rattlesnake). EC 3.4.21.74.'),
  'D020792': NtO(MH='Salivary Calculi', MS='The presence of calculi in a salivary duct or gland.'),
  'D020793': NtO(MH='Gatekeeping', MS='The controlling of access to health services, usually by primary care providers; often used in managed care settings to reduce utilization of expensive services and reduce referrals. (From BIOETHICS Thesaurus, 1999)'),
  'D020794': NtO(MH='Receptor Protein-Tyrosine Kinases', MS='A class of cellular receptors that have an intrinsic PROTEIN-TYROSINE KINASE activity.'),
  'D020795': NtO(MH='Photophobia', MS='Abnormal sensitivity to light. This may occur as a manifestation of EYE DISEASES; MIGRAINE; SUBARACHNOID HEMORRHAGE; MENINGITIS; and other disorders. Photophobia may also occur in association with DEPRESSION and other MENTAL DISORDERS.'),
  'D020796': NtO(MH='Receptor, Platelet-Derived Growth Factor alpha', MS='A PDGF receptor that binds specifically to both PDGF-A chains and PDGF-B chains. It contains a protein-tyrosine kinase activity that is involved in SIGNAL TRANSDUCTION.'),
  'D020797': NtO(MH='Receptor, Platelet-Derived Growth Factor beta', MS='A PDGF receptor that binds specifically to the PDGF-B chain. It contains a protein-tyrosine kinase activity that is involved in SIGNAL TRANSDUCTION.'),
  'D020798': NtO(MH='Two-Hybrid System Techniques', MS='Screening techniques first developed in yeast to identify genes encoding interacting proteins. Variations are used to evaluate interplay between proteins and other molecules. Two-hybrid techniques refer to analysis for protein-protein interactions, one-hybrid for DNA-protein interactions, three-hybrid interactions for RNA-protein interactions or ligand-based interactions. Reverse n-hybrid techniques refer to analysis for mutations or other small molecules that dissociate known interactions.'),
  'D020799': NtO(MH='Pancrelipase', MS='A preparation of hog pancreatic enzymes standardized for lipase content.'),
  'D020800': NtO(MH='Receptor, Nerve Growth Factor', MS='A low affinity receptor that binds NERVE GROWTH FACTOR; BRAIN-DERIVED NEUROTROPHIC FACTOR; NEUROTROPHIN 3; and neurotrophin 4.'),
  'D020801': NtO(MH='Receptor, Ciliary Neurotrophic Factor', MS='Cell surface receptors for CILIARY NEUROTROPHIC FACTOR. They are heterotrimeric proteins formed by the association of the CILIARY NEUROTROPHIC FACTOR RECEPTOR ALPHA SUBUNIT with the LEUKEMIA INHIBITORY FACTOR RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR GP130. Although the receptor regulates neuronal development, it is structurally similar to the cytokine receptor for INTERLEUKIN-6; (RECEPTORS, INTERLEUKIN-6).'),
  'D020802': NtO(MH='Epidural Abscess', MS='Circumscribed collections of suppurative material occurring in the spinal or intracranial EPIDURAL SPACE. The majority of epidural abscesses occur in the spinal canal and are associated with OSTEOMYELITIS of a vertebral body; ANALGESIA, EPIDURAL; and other conditions. Clinical manifestations include local and radicular pain, weakness, sensory loss, URINARY INCONTINENCE, and FECAL INCONTINENCE. Cranial epidural abscesses are usually associated with OSTEOMYELITIS of a cranial bone, SINUSITIS, or OTITIS MEDIA. (From Adams et al., Principles of Neurology, 6th ed, p710 and pp1240-1; J Neurol Neurosurg Psychiatry 1998 Aug;65(2):209-12)'),
  'D020803': NtO(MH='Encephalitis, Herpes Simplex', MS='An acute (or rarely chronic) inflammatory process of the brain caused by SIMPLEXVIRUS infections which may be fatal. The majority of infections are caused by human herpesvirus 1 (HERPESVIRUS 1, HUMAN) and less often by human herpesvirus 2 (HERPESVIRUS 2, HUMAN). Clinical manifestations include FEVER; HEADACHE; SEIZURES; HALLUCINATIONS; behavioral alterations; APHASIA; hemiparesis; and COMA. Pathologically, the condition is marked by a hemorrhagic necrosis involving the medial and inferior TEMPORAL LOBE and orbital regions of the FRONTAL LOBE. (From Adams et al., Principles of Neurology, 6th ed, pp751-4)'),
  'D020804': NtO(MH='Encephalitis, Varicella Zoster', MS='Inflammation of brain tissue caused by infection with the varicella-zoster virus (HERPESVIRUS 3, HUMAN). This condition is associated with immunocompromised states, including the ACQUIRED IMMUNODEFICIENCY SYNDROME. Pathologically, the virus tends to induce a vasculopathy and infect oligodendrocytes and ependymal cells, leading to CEREBRAL INFARCTION, multifocal regions of demyelination, and periventricular necrosis. Manifestations of varicella encephalitis usually occur 5-7 days after onset of HERPES ZOSTER and include HEADACHE; VOMITING; lethargy; focal neurologic deficits; FEVER; and COMA. (From Joynt, Clinical Neurology, 1996, Ch 26, pp29-32; Hum Pathol 1996 Sep;27(9):927-38)'),
  'D020805': NtO(MH='Central Nervous System Viral Diseases', MS='Viral infections of the brain, spinal cord, meninges, or perimeningeal spaces.'),
  'D020806': NtO(MH='Central Nervous System Bacterial Infections', MS='Bacterial infections of the brain, spinal cord, and meninges, including infections involving the perimeningeal spaces.'),
  'D020807': NtO(MH='Central Nervous System Parasitic Infections', MS='Infections of the brain, spinal cord, and meninges caused by parasites.'),
  'D020808': NtO(MH='Central Nervous System Protozoal Infections', MS='Infections of the brain, spinal cord, or meninges by single celled organisms of the former subkingdom known as protozoa. The central nervous system may be the primary or secondary site of protozoal infection. These diseases may occur as OPPORTUNISTIC INFECTIONS or arise in immunocompetent hosts.'),
  'D020809': NtO(MH='Central Nervous System Helminthiasis', MS='Infections of the BRAIN; SPINAL CORD; or MENINGES caused by HELMINTHS (parasitic worms).'),
  'D020811': NtO(MH='Biomedical Technology', MS='The application of technology to the solution of medical problems.'),
  'D020812': NtO(MH='Receptor, trkC', MS='A protein-tyrosine kinase receptor that is specific for NEUROTROPHIN 3. It is widely expressed in nervous tissue and may play a role in mediating the effects of NEUROTROPHIN 3 on the proliferation and differentiation of NEURONS.'),
  'D020813': NtO(MH='Receptor, trkB', MS='A protein-tyrosine kinase receptor that is specific for BRAIN-DERIVED NEUROTROPHIC FACTOR; NEUROTROPHIN 3; neurotrophin 4 and neurotrophin 5. It is widely expressed in nervous tissue and plays a role in mediating the effects of neurotrophins on growth and differentiation of neuronal cells.'),
  'D020814': NtO(MH='Meningitis, Escherichia coli', MS='A form of gram-negative meningitis that tends to occur in neonates, in association with anatomical abnormalities (which feature communication between the meninges and cutaneous structures) or as OPPORTUNISTIC INFECTIONS in association with IMMUNOLOGIC DEFICIENCY SYNDROMES. In premature neonates the clinical presentation may be limited to ANOREXIA; VOMITING; lethargy; or respiratory distress. Full-term infants may have as additional features FEVER; SEIZURES; and bulging of the anterior fontanelle. (From Menkes, Textbook of Child Neurology, 5th ed, pp398-400)'),
  'D020815': NtO(MH='Polyproteins', MS='Proteins which are synthesized as a single polymer and then cleaved into several distinct proteins.'),
  'D020816': NtO(MH='Amino Acid Motifs', MS='Three-dimensional protein structural elements that are composed of a combination of secondary structures. They include HELIX-LOOP-HELIX MOTIFS and ZINC FINGERS. Motifs are typically the most conserved regions of PROTEIN DOMAINS and are critical for domain function. However, the same motif may occur in proteins or enzymes with different functions.'),
  'D020817': NtO(MH='Asperger Syndrome', MS='A disorder beginning in childhood whose essential features are persistent impairment in reciprocal social communication and social interaction, and restricted, repetitive patterns of behavior, interests, or activities. These symptoms may limit or impair everyday functioning. (From DSM-5)'),
  'D020818': NtO(MH='Neuroschistosomiasis', MS='SCHISTOSOMIASIS of the brain, spinal cord, or meninges caused by infections with trematodes of the genus SCHISTOSOMA (primarily SCHISTOSOMA JAPONICUM; SCHISTOSOMA MANSONI; and SCHISTOSOMA HAEMATOBIUM in humans). S. japonicum infections of the nervous system may cause an acute meningoencephalitis or a chronic encephalopathy. S. mansoni and S. haematobium nervous system infections are associated with acute transverse myelitis involving the lower portions of the spinal cord. (From Joynt, Clinical Neurology, 1998, Ch27, pp61-2)'),
  'D020819': NtO(MH='Perimeningeal Infections', MS='Infectious processes, including abscesses, effusions, and empyemas which occur in the epidural or subdural spaces surrounding the brain and spinal cord.'),
  'D020820': NtO(MH='Dyskinesias', MS='Abnormal involuntary movements which primarily affect the extremities, trunk, or jaw that occur as a manifestation of an underlying disease process. Conditions which feature recurrent or persistent episodes of dyskinesia as a primary manifestation of disease may be referred to as dyskinesia syndromes (see MOVEMENT DISORDERS). Dyskinesias are also a relatively common manifestation of BASAL GANGLIA DISEASES.'),
  'D020821': NtO(MH='Dystonic Disorders', MS="Acquired and inherited conditions that feature DYSTONIA as a primary manifestation of disease. These disorders are generally divided into generalized dystonias (e.g., dystonia musculorum deformans) and focal dystonias (e.g., writer's cramp). They are also classified by patterns of inheritance and by age of onset."),
  'D020823': NtO(MH='ADP-Ribosylation Factor 1', MS='ADP-RIBOSYLATION FACTOR 1 is involved in regulating intracellular transport by modulating the interaction of coat proteins with organelle membranes in the early secretory pathway. It is a component of COAT PROTEIN COMPLEX I. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020824': NtO(MH='Spinocerebellar Tracts', MS="Fibers that arise from cell groups within the spinal cord and pass directly to the cerebellum. They include the anterior, posterior, and rostral spinocerebellar tracts, and the cuneocerebellar tract. (From Parent, Carpenter's Human Neuroanatomy, 9th ed, p607)"),
  'D020825': NtO(MH='T-Box Domain Proteins', MS='Proteins containing a region of conserved sequence, about 200 amino acids long, which encodes a particular sequence specific DNA binding domain (the T-box domain). These proteins are transcription factors that control developmental pathways. The prototype of this family is the mouse Brachyury (or T) gene product.'),
  'D020826': NtO(MH='rab3A GTP-Binding Protein', MS='The most abundant member of the RAB3 GTP-BINDING PROTEINS. It is involved in calcium-dependent EXOCYTOSIS and is localized to neurons and neuroendocrine cells. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020827': NtO(MH='rap1 GTP-Binding Proteins', MS='A genetically related subfamily of RAP GTP-BINDING PROTEINS that share homology with RAS PROTEINS. They bind to Ras effectors but do not activate them, therefore they may antagonize the effects of RAS PROTEINS. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020828': NtO(MH='Pseudobulbar Palsy', MS='A syndrome characterized by DYSARTHRIA, dysphagia, dysphonia, impairment of voluntary movements of tongue and facial muscles, and emotional lability. This condition is caused by diseases that affect the motor fibers that travel from the cerebral cortex to the lower BRAIN STEM (i.e., corticobulbar tracts); including MULTIPLE SCLEROSIS; MOTOR NEURON DISEASE; and CEREBROVASCULAR DISORDERS. (From Adams et al., Principles of Neurology, 6th ed, p489)'),
  'D020829': NtO(MH='Equisetum', MS='The only living genus of the order Equisetales, class Equisetopsida (Sphenopsida), division Equisetophyta (Sphenophyta); distantly related to ferns. It grows in moist places. The hollow, jointed, ridged stems contain SILICATES.'),
  'D020830': NtO(MH='rac1 GTP-Binding Protein', MS='A rac GTP-binding protein involved in regulating actin filaments at the plasma membrane. It controls the development of filopodia and lamellipodia in cells and thereby influences cellular motility and adhesion. It is also involved in activation of NADPH OXIDASES. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020831': NtO(MH='Acupuncture, Ear', MS='Acupuncture therapy by inserting needles in the ear. It is used to control pain and for treating various ailments.'),
  'D020832': NtO(MH='EF Hand Motifs', MS='Calcium-binding motifs composed of two helices (E and F) joined by a loop. Calcium is bound by the loop region. These motifs are found in many proteins that are regulated by calcium.'),
  'D020833': NtO(MH='Diffuse Axonal Injury', MS='A relatively common sequela of blunt head injury, characterized by a global disruption of axons throughout the brain. Associated clinical features may include NEUROBEHAVIORAL MANIFESTATIONS; PERSISTENT VEGETATIVE STATE; DEMENTIA; and other disorders.'),
  'D020834': NtO(MH='Ethnopharmacology', MS='The study of the actions and properties of medicinal agents, often derived from PLANTS, indigenous to populations or ETHNIC GROUPS.'),
  'D020835': NtO(MH='Medicine, Kampo', MS='System of herbal medicine practiced in Japan by both herbalists and practitioners of modern medicine. Kampo originated in China and is based on Chinese herbal medicine (MEDICINE, CHINESE TRADITIONAL).'),
  'D020836': NtO(MH='Protein Structure, Quaternary', MS='The characteristic 3-dimensional shape and arrangement of multimeric proteins (aggregates of more than one polypeptide chain).'),
  'D020837': NtO(MH='SOS1 Protein', MS='A mammalian homolog of the DROSOPHILA SON OF SEVENLESS PROTEIN. It is a guanine nucleotide exchange factor for RAS PROTEINS.'),
  'D020838': NtO(MH='Calendula', MS='A plant genus of the family ASTERACEAE. Members contain CAROTENOIDS, essential oils (OILS, VOLATILE), flavonoids, mucilage, SAPONINS, and STEROLS. The plants are used both topically and internally. The common name of Marigold is also used for TAGETES.'),
  'D020839': NtO(MH='Son of Sevenless Protein, Drosophila', MS='A guanine nucleotide exchange factor from DROSOPHILA. Sevenless refers to genetic mutations in DROSOPHILA that cause loss of the R7 photoreceptor which is required to see UV light.'),
  'D020840': NtO(MH='Tissue Kallikreins', MS='A family of trypsin-like SERINE ENDOPEPTIDASES that are expressed in a variety of cell types including human prostate epithelial cells. They are formed from tissue prokallikrein by action with TRYPSIN. They are highly similar to PROSTATE-SPECIFIC ANTIGEN.'),
  'D020842': NtO(MH='Plasma Kallikrein', MS='A peptidohydrolytic enzyme that is formed from PREKALLIKREIN by FACTOR XIIA. It activates FACTOR XII; FACTOR VII; and PLASMINOGEN. It is selective for both ARGININE and to a lesser extent LYSINE bonds. EC 3.4.21.34.'),
  'D020843': NtO(MH='Chamomile', MS='Common name for several daisy-like plants (MATRICARIA; TRIPLEUROSPERMUM; ANTHEMIS; CHAMAEMELUM) native to Europe and Western Asia, now naturalized in the United States and Australia.'),
  'D020844': NtO(MH='Comfrey', MS='Perennial herb Symphytum officinale, in the family Boraginaceae, used topically for wound healing. It contains ALLANTOIN, carotene, essential oils (OILS, VOLATILE); GLYCOSIDES; mucilage, resin, SAPONINS; TANNINS; triterpenoids, VITAMIN B12, and ZINC. Comfrey also contains PYRROLIZIDINE ALKALOIDS and is hepatotoxic if ingested.'),
  'D020845': NtO(MH='Selective Estrogen Receptor Modulators', MS='A structurally diverse group of compounds distinguished from ESTROGENS by their ability to bind and activate ESTROGEN RECEPTORS but act as either an agonist or antagonist depending on the tissue type and hormonal milieu. They are classified as either first generation because they demonstrate estrogen agonist properties in the ENDOMETRIUM or second generation based on their patterns of tissue specificity. (Horm Res 1997;48:155-63)'),
  'D020846': NtO(MH='cdc42 GTP-Binding Protein, Saccharomyces cerevisiae', MS='A member of the Rho family of MONOMERIC GTP-BINDING PROTEINS from SACCHAROMYCES CEREVISIAE. It is involved in morphological events related to the cell cycle. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020847': NtO(MH='Estrogen Receptor Modulators', MS='Substances that possess antiestrogenic actions but can also produce estrogenic effects as well. They act as complete or partial agonist or as antagonist. They can be either steroidal or nonsteroidal in structure.'),
  'D020848': NtO(MH='Chimerin 1', MS='A GTPase activating protein that is specific for RAC GTP-BINDING PROTEINS. It is expressed primarily in the brain and may be involved in signal transduction. The alternatively spliced form of CHIMERIN 1 (alpha-2 Chimerin) contains an additional src homology domain and is expressed in both the brain and testes.'),
  'D020849': NtO(MH='Raloxifene Hydrochloride', MS='A second generation selective estrogen receptor modulator (SERM) used to prevent osteoporosis in postmenopausal women. It has estrogen agonist effects on bone and cholesterol metabolism but behaves as a complete estrogen antagonist on mammary gland and uterine tissue.'),
  'D020852': NtO(MH='Lyme Neuroborreliosis', MS='Nervous system infections caused by tick-borne spirochetes of the BORRELIA BURGDORFERI GROUP. The disease may affect elements of the central or peripheral nervous system in isolation or in combination. Common clinical manifestations include a lymphocytic meningitis, cranial neuropathy (most often a facial neuropathy), POLYRADICULOPATHY, and a mild loss of memory and other cognitive functions. Less often more extensive inflammation involving the central nervous system (encephalomyelitis) may occur. In the peripheral nervous system, B. burgdorferi infection is associated with mononeuritis multiplex and polyradiculoneuritis. (From J Neurol Sci 1998 Jan 8;153(2):182-91)'),
  'D020856': NtO(MH='AT Rich Sequence', MS='A nucleic acid sequence that contains an above average number of ADENINE and THYMINE bases.'),
  'D020857': NtO(MH='Hallux Limitus', MS='A bony proliferation and articular degeneration of the first METATARSOPHALANGEAL JOINT that is characterized by pain and a progressive decrease in the dorsiflexion range of motion.'),
  'D020858': NtO(MH='Tissue and Organ Harvesting', MS='The procedure of removing TISSUES, organs, or specimens from DONORS for reuse, such as TRANSPLANTATION.'),
  'D020859': NtO(MH='Hallux Rigidus', MS='A condition caused by degenerative arthritis (see OSTEOARTHRITIS) of the METATARSOPHALANGEAL JOINT of the great toe and characterized by pain and limited dorsiflexion, but relatively unrestricted plantar flexion.'),
  'D020860': NtO(MH='Subtilisin', MS='A serine endopeptidase isolated from Bacillus subtilis. It hydrolyzes proteins with broad specificity for peptide bonds, and a preference for a large uncharged residue in P1. It also hydrolyzes peptide amides. (From Enzyme Nomenclature, 1992) EC 3.4.21.62.'),
  'D020862': NtO(MH='GC Rich Sequence', MS='A nucleic acid sequence that contains an above average number of GUANINE and CYTOSINE bases.'),
  'D020863': NtO(MH='Central Nervous System Cysts', MS='Congenital or acquired cysts of the brain, spinal cord, or meninges which may remain stable in size or undergo progressive enlargement.'),
  'D020864': NtO(MH='Calcium Channels, N-Type', MS='CALCIUM CHANNELS that are concentrated in neural tissue. Omega toxins inhibit the actions of these channels by altering their voltage dependence.'),
  'D020865': NtO(MH='Calcium Channels, P-Type', MS='CALCIUM CHANNELS located within the PURKINJE CELLS of the cerebellum. They are involved in stimulation-secretion coupling of neurons.'),
  'D020866': NtO(MH='omega-Conotoxin GVIA', MS='A neurotoxic peptide, which is a cleavage product (VIa) of the omega-Conotoxin precursor protein contained in venom from the marine snail, CONUS geographus. It is an antagonist of CALCIUM CHANNELS, N-TYPE.'),
  'D020867': NtO(MH='omega-Agatoxin IVA', MS='A neuropeptide toxin from the venom of the funnel web spider, Agelenopsis aperta. It inhibits CALCIUM CHANNELS, P-TYPE by altering the voltage-dependent gating so that very large depolarizations are needed for channel opening. It also inhibits CALCIUM CHANNELS, Q-TYPE.'),
  'D020868': NtO(MH='Gene Silencing', MS='Interruption or suppression of the expression of a gene at transcriptional or translational levels.'),
  'D020869': NtO(MH='Gene Expression Profiling', MS='The determination of the pattern of genes expressed at the level of GENETIC TRANSCRIPTION, under specific circumstances or in a specific cell.'),
  'D020871': NtO(MH='RNA Stability', MS='The extent to which an RNA molecule retains its structural integrity and resists degradation by RNASE, and base-catalyzed HYDROLYSIS, under changing in vivo or in vitro conditions.'),
  'D020875': NtO(MH='Enkephalin, Ala(2)-MePhe(4)-Gly(5)-', MS='An enkephalin analog that selectively binds to the MU OPIOID RECEPTOR. It is used as a model for drug permeability experiments.'),
  'D020878': NtO(MH='Device Removal', MS='Removal of an implanted therapeutic or prosthetic device.'),
  'D020879': NtO(MH='Neuromuscular Manifestations', MS='Signs and symptoms associated with diseases of the muscle, neuromuscular junction, or peripheral nerves.'),
  'D020880': NtO(MH='Neuregulins', MS='A family of peptides originally found as factors that stimulate the phosphorylation of the erbB-2 receptor (RECEPTORS, ERBB-2). Multiple variant forms of NEUREGULINS occur due to alternative splicing of their mRNAs. The NEUREGULINS include products from the three known genes (NGR1; NGR2 and NGR3).'),
  'D020881': NtO(MH='Enkephalin, D-Penicillamine (2,5)-', MS='A disulfide opioid pentapeptide that selectively binds to the DELTA OPIOID RECEPTOR. It possesses antinociceptive activity.'),
  'D020883': NtO(MH='Hysterotomy', MS='An incision in the uterus, performed through either the abdomen or the vagina.'),
  'D020884': NtO(MH='Colpotomy', MS='An incision in the vagina.'),
  'D020885': NtO(MH='Ribonucleoprotein, U7 Small Nuclear', MS="This ribonucleoprotein particle, composed of U7 snRNA, Sm core protein, and U7 snRNP-specific proteins, is involved in the 3'end processing of histone premessenger RNAs."),
  'D020886': NtO(MH='Somatosensory Disorders', MS='Disorders of sensory information received from superficial and deep regions of the body. The somatosensory system conveys neural impulses which pertain to proprioception, tactile sensation, thermal sensation, pressure sensation, and pain. PERIPHERAL NERVOUS SYSTEM DISEASES; SPINAL CORD DISEASES; and BRAIN DISEASES may be associated with impaired or abnormal somatic sensation.'),
  'D020887': NtO(MH='Selenious Acid', MS='A selenium compound with the molecular formula H2SO3. It used as a source of SELENIUM, especially for patients that develop selenium deficiency following prolonged PARENTERAL NUTRITION.'),
  'D020888': NtO(MH='Vigabatrin', MS='An analogue of GAMMA-AMINOBUTYRIC ACID. It is an irreversible inhibitor of 4-AMINOBUTYRATE TRANSAMINASE, the enzyme responsible for the catabolism of GAMMA-AMINOBUTYRIC ACID and is used as an anticonvulsant. (From Martindale The Extra Pharmacopoeia, 31st ed)'),
  'D020889': NtO(MH='Rolipram', MS='A phosphodiesterase 4 inhibitor with antidepressant properties.'),
  'D020890': NtO(MH='Neuregulin-1', MS='A peptide factor originally identified by its ability to stimulate the phosphorylation the erbB-2 receptor (RECEPTOR, ERBB-2). It is a ligand for the erbB-3 receptor (RECEPTOR, ERBB-3) and the erbB-4 receptor. Variant forms of NEUREGULIN-1 occur through alternative splicing of its mRNA.'),
  'D020891': NtO(MH='Raclopride', MS='A substituted benzamide that has antipsychotic properties. It is a dopamine D2 receptor (see RECEPTORS, DOPAMINE D2) antagonist.'),
  'D020893': NtO(MH='Receptor, ErbB-3', MS='A cell surface protein-tyrosine kinase receptor that is specific for NEUREGULINS. It has extensive homology to and can heterodimerize with the EGF RECEPTOR and the ERBB-2 RECEPTOR. Overexpression of the erbB-3 receptor is associated with TUMORIGENESIS.'),
  'D020894': NtO(MH='Microfibrils', MS='Components of the extracellular matrix consisting primarily of fibrillin. They are essential for the integrity of elastic fibers.'),
  'D020895': NtO(MH='Enterocytes', MS='Absorptive cells in the lining of the INTESTINAL MUCOSA. They are differentiated EPITHELIAL CELLS with apical MICROVILLI facing the intestinal lumen. Enterocytes are more abundant in the SMALL INTESTINE than in the LARGE INTESTINE. Their microvilli greatly increase the luminal surface area of the cell by 14- to 40 fold.'),
  'D020896': NtO(MH='Hypovolemia', MS='An abnormally low volume of blood circulating through the body. It may result in hypovolemic shock (see SHOCK).'),
  'D020897': NtO(MH='Organizers, Embryonic', MS='Cells in certain regions of an embryo that self-regulate embryonic development. These organizers have been found in dorsal and ventral poles of GASTRULA embryos, including Spemann organizer in amphibians, and Hensen node in chicken and mouse. These organizer cells communicate with each other via a network of secreted signaling proteins, such as BONE MORPHOGENETIC PROTEINS and their antagonists (chordin and noggin).'),
  'D020900': NtO(MH='Echinacea', MS='A genus of perennial herbs used topically and internally. It contains echinacoside, GLYCOSIDES; INULIN; isobutyl amides, resin, and SESQUITERPENES.'),
  'D020901': NtO(MH='Kava', MS="Dried rhizome and roots of Piper methysticum, a shrub native to Oceania and known for its anti-anxiety and sedative properties. Heavy usage results in some adverse effects. It contains ALKALOIDS; LACTONES; kawain, methysticin, mucilage, STARCH, and yangonin. Kava is also the name of the pungent beverage prepared from the plant's roots."),
  'D020902': NtO(MH='Hypericum', MS='Genus of perennial plants in the family CLUSIACEAE (sometimes classified as Hypericaceae). Herbal and homeopathic preparations are used for depression, neuralgias, and a variety of other conditions. Hypericum contains flavonoids; GLYCOSIDES; mucilage, TANNINS; volatile oils (OILS, ESSENTIAL), hypericin and hyperforin.'),
  'D020903': NtO(MH='Tanacetum parthenium', MS='An aromatic perennial plant species that has been used to treat migraines, arthritis, and as a febrifuge. It contains TANNINS, volatile oils (OILS, ESSENTIAL), and sesquiterpene lactones, especially parthenolide.'),
  'D020904': NtO(MH='Glia Maturation Factor', MS='A factor identified in the brain that influences the growth and differentiation of NEURONS and NEUROGLIA. Glia maturation factor beta is the 17-kDa polypeptide product of the GMFB gene and is the principal component of GLIA MATURATION FACTOR.'),
  'D020906': NtO(MH='Air Abrasion, Dental', MS='A technique using a pneumatic, high-pressure stream of aluminum oxide to remove DENTAL ENAMEL; DENTIN; and restorative materials from teeth. In contrast to using DENTAL HIGH-SPEED EQUIPMENT, this method usually requires no dental anesthesia (ANESTHESIA, DENTAL) and reduces risks of tooth chipping and microfracturing. It is used primarily for routine DENTAL CAVITY PREPARATION.'),
  'D020907': NtO(MH='Calcium Channels, Q-Type', MS='CALCIUM CHANNELS located in the neurons of the brain.'),
  'D020908': NtO(MH='Calcium Channels, R-Type', MS='CALCIUM CHANNELS located in the neurons of the brain. They are inhibited by the marine snail toxin, omega conotoxin MVIIC.'),
  'D020909': NtO(MH='Acarbose', MS='An inhibitor of ALPHA-GLUCOSIDASES that retards the digestion and absorption of DIETARY CARBOHYDRATES in the SMALL INTESTINE.'),
  'D020910': NtO(MH='Ketorolac', MS='A pyrrolizine carboxylic acid derivative structurally related to INDOMETHACIN. It is an NSAID and is used principally for its analgesic activity. (From Martindale The Extra Pharmacopoeia, 31st ed)'),
  'D020911': NtO(MH='Ketorolac Tromethamine', MS='A pyrrolizine carboxylic acid derivative structurally related to INDOMETHACIN. It is a non-steroidal anti-inflammatory agent used for analgesia for postoperative pain and inhibits cyclooxygenase activity.'),
  'D020912': NtO(MH='Moclobemide', MS='A reversible inhibitor of monoamine oxidase type A; (RIMA); (see MONOAMINE OXIDASE INHIBITORS) that has antidepressive properties.'),
  'D020913': NtO(MH='Perindopril', MS='An angiotensin-converting enzyme inhibitor. It is used in patients with hypertension and heart failure.'),
  'D020914': NtO(MH='Myopathies, Structural, Congenital', MS='A heterogeneous group of diseases characterized by the early onset of hypotonia, developmental delay of motor skills, non-progressive weakness. Each of these disorders is associated with a specific histologic muscle fiber abnormality.'),
  'D020915': NtO(MH='Korsakoff Syndrome', MS='An acquired cognitive disorder characterized by inattentiveness and the inability to form short term memories. This disorder is frequently associated with chronic ALCOHOLISM; but it may also result from dietary deficiencies; CRANIOCEREBRAL TRAUMA; NEOPLASMS; CEREBROVASCULAR DISORDERS; ENCEPHALITIS; EPILEPSY; and other conditions. (Adams et al., Principles of Neurology, 6th ed, p1139)'),
  'D020916': NtO(MH='Conotoxins', MS='Peptide neurotoxins from the marine fish-hunting snails of the genus CONUS. They contain 13 to 29 amino acids which are strongly basic and are highly cross-linked by disulfide bonds. There are three types of conotoxins, omega-, alpha-, and mu-. OMEGA-CONOTOXINS inhibit voltage-activated entry of calcium into the presynaptic membrane and therefore the release of ACETYLCHOLINE. Alpha-conotoxins inhibit the postsynaptic acetylcholine receptor. Mu-conotoxins prevent the generation of muscle action potentials. (From Concise Encyclopedia Biochemistry and Molecular Biology, 3rd ed)'),
  'D020917': NtO(MH='Receptor, trkA', MS='A protein-tyrosine kinase receptor that is specific for NERVE GROWTH FACTOR; NEUROTROPHIN 3; neurotrophin 4, neurotrophin 5. It plays a crucial role in pain sensation and thermoregulation in humans. Gene mutations that cause loss of receptor function are associated with CONGENITAL INSENSITIVITY TO PAIN WITH ANHIDROSIS, while gene rearrangements that activate the protein-tyrosine kinase function are associated with tumorigenesis.'),
  'D020918': NtO(MH='Complex Regional Pain Syndromes', MS='Conditions characterized by pain involving an extremity or other body region, HYPERESTHESIA, and localized autonomic dysfunction following injury to soft tissue or nerve. The pain is usually associated with ERYTHEMA; SKIN TEMPERATURE changes, abnormal sudomotor activity (i.e., changes in sweating due to altered sympathetic innervation) or edema. The degree of pain and other manifestations is out of proportion to that expected from the inciting event. Two subtypes of this condition have been described: type I; (REFLEX SYMPATHETIC DYSTROPHY) and type II; (CAUSALGIA). (From Pain 1995 Oct;63(1):127-33)'),
  'D020919': NtO(MH='Sleep Disorders, Intrinsic', MS='Dyssomnias (i.e., insomnias or hypersomnias) associated with dysfunction of internal sleep mechanisms or secondary to a sleep-related medical disorder (e.g., sleep apnea, post-traumatic sleep disorders, etc.). (From Thorpy, Sleep Disorders Medicine, 1994, p187)'),
  'D020920': NtO(MH='Dyssomnias', MS='A broad category of sleep disorders characterized by either hypersomnolence or insomnia. The three major subcategories include intrinsic (i.e., arising from within the body) (SLEEP DISORDERS, INTRINSIC), extrinsic (secondary to environmental conditions or various pathologic conditions), and disturbances of circadian rhythm. (From Thorpy, Sleep Disorders Medicine, 1994, p187)'),
  'D020921': NtO(MH='Sleep Arousal Disorders', MS='Sleep disorders characterized by impaired arousal from the deeper stages of sleep (generally stage III or IV sleep).'),
  'D020922': NtO(MH='Sleep-Wake Transition Disorders', MS='Parasomnias characterized by behavioral abnormalities that occur during the transition between wakefulness and sleep (or between sleep and wakefulness).'),
  'D020923': NtO(MH='REM Sleep Parasomnias', MS='Abnormal behavioral or physiologic events that are associated with REM sleep, including REM SLEEP BEHAVIOR DISORDER.'),
  'D020924': NtO(MH='Urological Manifestations', MS='Clinical disturbances of the urinary system.'),
  'D020925': NtO(MH='Hypoxia-Ischemia, Brain', MS='A disorder characterized by a reduction of oxygen in the blood combined with reduced blood flow (ISCHEMIA) to the brain from a localized obstruction of a cerebral artery or from systemic hypoperfusion. Prolonged hypoxia-ischemia is associated with ISCHEMIC ATTACK, TRANSIENT; BRAIN INFARCTION; BRAIN EDEMA; COMA; and other conditions.'),
  'D020926': NtO(MH='Medetomidine', MS='An agonist of RECEPTORS, ADRENERGIC ALPHA-2 that is used in veterinary medicine for its analgesic and sedative properties. It is the racemate of DEXMEDETOMIDINE.'),
  'D020927': NtO(MH='Dexmedetomidine', MS='A imidazole derivative that is an agonist of ADRENERGIC ALPHA-2 RECEPTORS. It is closely-related to MEDETOMIDINE, which is the racemic form of this compound.'),
  'D020928': NtO(MH='Mitogen-Activated Protein Kinases', MS='A superfamily of PROTEIN-SERINE-THREONINE KINASES that are activated by diverse stimuli via protein kinase cascades. They are the final components of the cascades, activated by phosphorylation by MITOGEN-ACTIVATED PROTEIN KINASE KINASES, which in turn are activated by mitogen-activated protein kinase kinase kinases (MAP KINASE KINASE KINASES).'),
  'D020929': NtO(MH='Mitogen-Activated Protein Kinase Kinases', MS='A serine-threonine protein kinase family whose members are components in protein kinase cascades activated by diverse stimuli. These MAPK kinases phosphorylate MITOGEN-ACTIVATED PROTEIN KINASES and are themselves phosphorylated by MAP KINASE KINASE KINASES. JNK kinases (also known as SAPK kinases) are a subfamily.'),
  'D020930': NtO(MH='MAP Kinase Kinase Kinases', MS='Mitogen-activated protein kinase kinase kinases (MAPKKKs) are serine-threonine protein kinases that initiate protein kinase signaling cascades. They phosphorylate MITOGEN-ACTIVATED PROTEIN KINASE KINASES; (MAPKKs) which in turn phosphorylate MITOGEN-ACTIVATED PROTEIN KINASES; (MAPKs).'),
  'D020931': NtO(MH='ran GTP-Binding Protein', MS='A monomeric GTP-binding protein involved in nucleocytoplasmic transport of proteins into the nucleus and RNA into the cytoplasm. This enzyme was formerly listed as EC 3.6.1.47.'),
  'D020932': NtO(MH='Nerve Growth Factor', MS='NERVE GROWTH FACTOR is the first of a series of neurotrophic factors that were found to influence the growth and differentiation of sympathetic and sensory neurons. It is comprised of alpha, beta, and gamma subunits. The beta subunit is responsible for its growth stimulating activity.'),
  'D020933': NtO(MH='Neurotrophin 3', MS='A neurotrophic factor involved in regulating the survival of visceral and proprioceptive sensory neurons. It is closely homologous to nerve growth factor beta and BRAIN-DERIVED NEUROTROPHIC FACTOR.'),
  'D020934': NtO(MH='Ciliary Neurotrophic Factor', MS='A neurotrophic factor that promotes the survival of various neuronal cell types and may play an important role in the injury response in the nervous system.'),
  'D020935': NtO(MH='MAP Kinase Signaling System', MS='An intracellular signaling system involving the MAP kinase cascades (three-membered protein kinase cascades). Various upstream activators, which act in response to extracellular stimuli, trigger the cascades by activating the first member of a cascade, MAP KINASE KINASE KINASES; (MAPKKKs). Activated MAPKKKs phosphorylate MITOGEN-ACTIVATED PROTEIN KINASE KINASES which in turn phosphorylate the MITOGEN-ACTIVATED PROTEIN KINASES; (MAPKs). The MAPKs then act on various downstream targets to affect gene expression. In mammals, there are several distinct MAP kinase pathways including the ERK (extracellular signal-regulated kinase) pathway, the SAPK/JNK (stress-activated protein kinase/c-jun kinase) pathway, and the p38 kinase pathway. There is some sharing of components among the pathways depending on which stimulus originates activation of the cascade.'),
  'D020936': NtO(MH='Epilepsy, Benign Neonatal', MS='A condition marked by recurrent seizures that occur during the first 4-6 weeks of life despite an otherwise benign neonatal course. Autosomal dominant familial and sporadic forms have been identified. Seizures generally consist of brief episodes of tonic posturing and other movements, apnea, eye deviations, and blood pressure fluctuations. These tend to remit after the 6th week of life. The risk of developing epilepsy at an older age is moderately increased in the familial form of this disorder. (Neurologia 1996 Feb;11(2):51-5)'),
  'D020937': NtO(MH='Epilepsy, Partial, Sensory', MS='A disorder characterized by recurrent focal onset seizures which have sensory (i.e., olfactory, visual, tactile, gustatory, or auditory) manifestations. Partial seizures that feature alterations of consciousness are referred to as complex partial seizures (EPILEPSY, COMPLEX PARTIAL).'),
  'D020938': NtO(MH='Epilepsy, Partial, Motor', MS='A disorder characterized by recurrent localized paroxysmal discharges of cerebral neurons that give rise to seizures that have motor manifestations. The majority of partial motor seizures originate in the FRONTAL LOBE (see also EPILEPSY, FRONTAL LOBE). Motor seizures may manifest as tonic or clonic movements involving the face, one limb or one side of the body. A variety of more complex patterns of movement, including abnormal posturing of extremities, may also occur.'),
  'D020939': NtO(MH='Ginger', MS='Deciduous plant rich in volatile oil (OILS, VOLATILE). It is used as a flavoring agent and has many other uses both internally and topically.'),
  'D020940': NtO(MH='Reishi', MS='A mushroom, Ganoderma lucidum, of the POLYPORALES order of basidiomycetous fungi. It has long been used in traditional Chinese medicine in various forms.'),
  'D020941': NtO(MH='Myasthenia Gravis, Neonatal', MS='A disorder of neuromuscular transmission that occurs in a minority of newborns born to women with myasthenia gravis. Clinical features are usually present at birth or develop in the first 3 days of life and consist of hypotonia and impaired respiratory, suck, and swallowing abilities. This condition is associated with the passive transfer of acetylcholine receptor antibodies through the placenta. In the majority of infants the myasthenic weakness resolves (i.e., transient neonatal myasthenia gravis) although this disorder may rarely continue beyond the neonatal period (i.e., persistent neonatal myasthenia gravis). (From Menkes, Textbook of Child Neurology, 5th ed, p823; Neurology 1997 Jan;48(1):50-4)'),
  'D020942': NtO(MH='Shiitake Mushrooms', MS='Mushrooms in the order AGARICALES containing B vitamins, cortinelin, and the polysaccharide LENTINAN.'),
  'D020943': NtO(MH='AIDS Arteritis, Central Nervous System', MS='Inflammation of ARTERIES in the CENTRAL NERVOUS SYSTEM that occurs in patients with ACQUIRED IMMUNODEFICIENCY SYNDROME or AIDS-RELATED OPPORTUNISTIC INFECTIONS.'),
  'D020944': NtO(MH='Milk Thistle', MS='The plant Silybum marianum in the family ASTERACEAE containing the bioflavonoid complex SILYMARIN. For centuries this has been used traditionally to treat liver disease. Silybum marianum (L.) Gaertn. = Carduus marianus L.'),
  'D020945': NtO(MH='Lupus Vasculitis, Central Nervous System', MS='Central nervous system vasculitis that is associated with SYSTEMIC LUPUS ERYTHEMATOSUS. Clinical manifestations may include DEMENTIA; SEIZURES; CRANIAL NERVE DISEASES; HEMIPARESIS; BLINDNESS; DYSPHASIA; and other neurological disorders.'),
  'D020946': NtO(MH='Taxus', MS='Genus of coniferous yew trees or shrubs, several species of which have medicinal uses. Notable is the Pacific yew, Taxus brevifolia, which is used to make the anti-neoplastic drug taxol (PACLITAXEL).'),
  'D020947': NtO(MH='Tea Tree Oil', MS='Essential oil extracted from MELALEUCA ALTERNIFOLIA (tea tree). It is used as a topical antimicrobial due to the presence of terpineol.'),
  'D020948': NtO(MH="Cat's Claw", MS='A vine (Uncaria tomentosa) indigenous to the Amazon rainforest whose name is derived from its hook-like thorns. It contains oxindole alkaloids and glycosides and has many medicinal uses.'),
  'D020949': NtO(MH='Usnea', MS='A genus of lichen-forming fungi containing usnic acid and mucilage. Usnea barbata is used in HERBAL MEDICINE.'),
  'D020950': NtO(MH='Vitex', MS='A genus of trees in the Lamiaceae family containing assorted flavonoids with possible analgesic and antineoplastic properties. The fruit of these trees is used in herbal preparations.'),
  'D020953': NtO(MH='Neuroaspergillosis', MS='Infections of the nervous system caused by fungi of the genus ASPERGILLUS, most commonly ASPERGILLUS FUMIGATUS. Aspergillus infections may occur in immunocompetent hosts, but are more prevalent in individuals with IMMUNOLOGIC DEFICIENCY SYNDROMES. The organism may spread to the nervous system from focal infections in the lung, mastoid region, sinuses, inner ear, bones, eyes, gastrointestinal tract, and heart. Sinus infections may be locally invasive and enter the intracranial compartment, producing MENINGITIS, FUNGAL; cranial neuropathies; and abscesses in the frontal lobes of the brain. (From Joynt, Clinical Neurology, 1998, Ch 27, pp62-3)'),
  'D020954': NtO(MH='Single-Parent Family', MS='A household that includes children and is headed by one adult.'),
  'D020955': NtO(MH='Striatonigral Degeneration', MS='A sporadic neurodegenerative disease with onset in middle-age characterized clinically by Parkinsonian features (e.g., MUSCLE RIGIDITY; HYPOKINESIA; stooped posture) and HYPOTENSION. This condition is considered a clinical variant of MULTIPLE SYSTEM ATROPHY. Pathologic features include a prominent loss of neurons in the zona compacta of the SUBSTANTIA NIGRA and PUTAMEN. (From Adams et al., Principles of Neurology, 6th ed, p1075-6)'),
  'D020957': NtO(MH='Nunavut', MS='A self-governing territory formed from the central and eastern portions of the Northwest Territories. It was officially established April 1, 1999. The capital is Iqaluit.'),
  'D020958': NtO(MH='Australian Capital Territory', MS='A territory of Australia consisting of Canberra, the national capital and surrounding land. It lies geographically within NEW SOUTH WALES and was established by law in 1988.'),
  'D020959': NtO(MH='Polyethylene', MS='A vinyl polymer made from ethylene. It can be branched or linear. Branched or low-density polyethylene is tough and pliable but not to the same degree as linear polyethylene. Linear or high-density polyethylene has a greater hardness and tensile strength. Polyethylene is used in a variety of products, including implants and prostheses.'),
  'D020960': NtO(MH='omega-Conotoxins', MS='A family of structurally related neurotoxic peptides from mollusk venom that inhibit voltage-activated entry of calcium into the presynaptic membrane. They selectively inhibit N-, P-, and Q-type calcium channels.'),
  'D020961': NtO(MH='Lewy Body Disease', MS='A neurodegenerative disease characterized by dementia, mild parkinsonism, and fluctuations in attention and alertness. The neuropsychiatric manifestations tend to precede the onset of bradykinesia, MUSCLE RIGIDITY, and other extrapyramidal signs. DELUSIONS and visual HALLUCINATIONS are relatively frequent in this condition. Histologic examination reveals LEWY BODIES in the CEREBRAL CORTEX and BRAIN STEM. SENILE PLAQUES and other pathologic features characteristic of ALZHEIMER DISEASE may also be present. (From Neurology 1997;48:376-380; Neurology 1996;47:1113-1124)'),
  'D020962': NtO(MH='Heterotrimeric GTP-Binding Proteins', MS='GTP-BINDING PROTEINS that contain three non-identical subunits. They are found associated with members of the seven transmembrane domain superfamily of G-PROTEIN-COUPLED RECEPTORS. Upon activation the GTP-BINDING PROTEIN ALPHA SUBUNIT of the complex dissociates leaving a dimer of a GTP-BINDING PROTEIN BETA SUBUNIT bound to a GTP-BINDING PROTEIN GAMMA SUBUNIT.'),
  'D020963': NtO(MH='GTP Phosphohydrolase-Linked Elongation Factors', MS='Factors that utilize energy from the hydrolysis of GTP to GDP for peptide chain elongation. EC 3.6.1.-.'),
  'D020964': NtO(MH='Embryo Loss', MS='Early pregnancy loss during the EMBRYO, MAMMALIAN stage of development. In the human, this period comprises the second through eighth week after fertilization.'),
  'D020966': NtO(MH='Muscular Disorders, Atrophic', MS='Disorders characterized by an abnormal reduction in muscle volume due to a decrease in the size or number of muscle fibers. Atrophy may result from diseases intrinsic to muscle tissue (e.g., MUSCULAR DYSTROPHY) or secondary to PERIPHERAL NERVOUS SYSTEM DISEASES that impair innervation to muscle tissue (e.g., MUSCULAR ATROPHY, SPINAL).'),
  'D020967': NtO(MH='Myotonic Disorders', MS='Diseases characterized by MYOTONIA, which may be inherited or acquired. Myotonia may be restricted to certain muscles (e.g., intrinsic hand muscles) or occur as a generalized condition.'),
  'D020968': NtO(MH='Brachial Plexus Neuritis', MS='A syndrome associated with inflammation of the BRACHIAL PLEXUS. Clinical features include severe pain in the shoulder region which may be accompanied by MUSCLE WEAKNESS and loss of sensation in the upper extremity. This condition may be associated with VIRUS DISEASES; IMMUNIZATION; SURGERY; heroin use (see HEROIN DEPENDENCE); and other conditions. The term brachial neuralgia generally refers to pain associated with brachial plexus injury. (From Adams et al., Principles of Neurology, 6th ed, pp1355-6)'),
  'D020969': NtO(MH='Disease Attributes', MS='Clinical characteristics of disease or illness.'),
  'D021041': NtO(MH='Placental Circulation', MS='The circulation of BLOOD, of both the mother and the FETUS, through the PLACENTA.'),
  'D021061': NtO(MH='Liquid Ventilation', MS='Artificial respiration (RESPIRATION, ARTIFICIAL) using an oxygenated fluid.'),
  'D021081': NtO(MH='Chronobiology Disorders', MS='Disruptions of the rhythmic cycle of bodily functions or activities.'),
  'D021101': NtO(MH='Radiation Hybrid Mapping', MS='A method for ordering genetic loci along CHROMOSOMES. The method involves fusing irradiated donor cells with host cells from another species. Following cell fusion, fragments of DNA from the irradiated cells become integrated into the chromosomes of the host cells. Molecular probing of DNA obtained from the fused cells is used to determine if two or more genetic loci are located within the same fragment of donor cell DNA.'),
  'D021121': NtO(MH='Branched DNA Signal Amplification Assay', MS='A molecular probe technique that utilizes branched DNA (bDNA) as a means to amplify the hybridization signal. One end of the bDNA molecule is designed to bind a specific target, while the other end of the bDNA molecule contains many branches of DNA that are designed to bind a probe used for signal detection.'),
  'D021122': NtO(MH='Protein Subunits', MS='Single chains of amino acids that are the units of multimeric PROTEINS. Multimeric proteins can be composed of identical or non-identical subunits. One or more monomeric subunits may compose a protomer which itself is a subunit structure of a larger assembly.'),
  'D021141': NtO(MH='Nucleic Acid Amplification Techniques', MS='Laboratory techniques that involve the in-vitro synthesis of many copies of DNA or RNA from one original template.'),
  'D021161': NtO(MH='Self-Sustained Sequence Replication', MS='An isothermal in-vitro nucleotide amplification process. The process involves the concomitant action of a RNA-DIRECTED DNA POLYMERASE, a ribonuclease (RIBONUCLEASES), and DNA-DIRECTED RNA POLYMERASES to synthesize large quantities of sequence-specific RNA and DNA molecules.'),
  'D021181': NtO(MH='Egg Hypersensitivity', MS='Allergic reaction to eggs that is triggered by the immune system.'),
  'D021182': NtO(MH='Wheat Hypersensitivity', MS='Allergic reaction to wheat that is triggered by the immune system.'),
  'D021183': NtO(MH='Peanut Hypersensitivity', MS='Allergic reaction to peanuts that is triggered by the immune system.'),
  'D021184': NtO(MH='Nut Hypersensitivity', MS='Allergic reaction to tree nuts that is triggered by the immune system.'),
  'D021241': NtO(MH='Spectrometry, Mass, Electrospray Ionization', MS='A mass spectrometry technique used for analysis of nonvolatile compounds such as proteins and macromolecules. The technique involves preparing electrically charged droplets from analyte molecules dissolved in solvent. The electrically charged droplets enter a vacuum chamber where the solvent is evaporated. Evaporation of solvent reduces the droplet size, thereby increasing the coulombic repulsion within the droplet. As the charged droplets get smaller, the excess charge within them causes them to disintegrate and release analyte molecules. The volatilized analyte molecules are then analyzed by mass spectrometry.'),
  'D021261': NtO(MH='Parasitic Sensitivity Tests', MS='Tests that demonstrate the relative effectiveness of chemotherapeutic agents against specific parasites.'),
  'D021281': NtO(MH='Quantitative Structure-Activity Relationship', MS='A quantitative prediction of the biological, ecotoxicological or pharmaceutical activity of a molecule. It is based upon structure and activity information gathered from a series of similar compounds.'),
  'D021301': NtO(MH='Antifreeze Proteins', MS='Proteins that bind to ice and modify the growth of ice crystals. They perform a cryoprotective role in a variety of organisms.'),
  'D021302': NtO(MH='Antifreeze Proteins, Type IV', MS='A subclass of ANTIFREEZE PROTEINS that contain four amphipathic alpha-helices folded into an antiparallel helix bundle.'),
  'D021322': NtO(MH='Antifreeze Proteins, Type I', MS='A subclass of ANTIFREEZE PROTEINS that are 3-5 kDa in size and contain a single alanine-rich amphipathic alpha-helix.'),
  'D021341': NtO(MH='Antifreeze Proteins, Type II', MS='A subclass of ANTIFREEZE PROTEINS that have a cystine-rich globular structure of approximately 14 kD.'),
  'D021343': NtO(MH='Antifreeze Proteins, Type III', MS='A subclass of ANTIFREEZE PROTEINS that are globular, 6.5 kDa in size and contain compact beta-sheet structures.'),
  'D021361': NtO(MH='Scaphoid Bone', MS='The bone which is located most lateral in the proximal row of CARPAL BONES.'),
  'D021362': NtO(MH='Palate, Hard', MS='The anteriorly located rigid section of the PALATE.'),
  'D021381': NtO(MH='Protein Transport', MS='The process of moving proteins from one cellular compartment (including extracellular) to another by various sorting and transport mechanisms such as gated transport, protein translocation, and vesicular transport.'),
  'D021382': NtO(MH='Protein Sorting Signals', MS='Amino acid sequences found in transported proteins that selectively guide the distribution of the proteins to specific cellular compartments.'),
  'D021441': NtO(MH='Carcinoma, Pancreatic Ductal', MS='Carcinoma that arises from the PANCREATIC DUCTS. It accounts for the majority of cancers derived from the PANCREAS.'),
  'D021461': NtO(MH='Animals, Outbred Strains', MS='Animals that are generated from breeding two genetically dissimilar strains of the same species.'),
  'D021501': NtO(MH='Flank Pain', MS='Pain emanating from below the RIBS and above the ILIUM.'),
  'D021521': NtO(MH='Ribotyping', MS='RESTRICTION FRAGMENT LENGTH POLYMORPHISM analysis of rRNA genes that is used for differentiating between species or strains.'),
  'D021522': NtO(MH='Bioartificial Organs', MS='Artificial organs that are composites of biomaterials and cells. The biomaterial can act as a membrane (container) as in BIOARTIFICIAL LIVER or a scaffold as in bioartificial skin.'),
  'D021541': NtO(MH='Sea Bream', MS='A species of PERCIFORMES commonly used in saline aquaculture.'),
  'D021542': NtO(MH='Amino Acids, Neutral', MS='Amino acids with uncharged R groups or side chains.'),
  'D021581': NtO(MH='Active Transport, Cell Nucleus', MS='Gated transport mechanisms by which proteins or RNA are moved across the NUCLEAR MEMBRANE.'),
  'D021601': NtO(MH='trans-Golgi Network', MS='A network of membrane compartments, located at the cytoplasmic side of the GOLGI APPARATUS, where proteins and lipids are sorted for transport to various locations in the cell or cell membrane.'),
  'D021621': NtO(MH='Imaging, Three-Dimensional', MS='The process of generating three-dimensional images by electronic, photographic, or other methods. For example, three-dimensional images can be generated by assembling multiple tomographic images with the aid of a computer, while photographic 3-D images (HOLOGRAPHY) can be made by exposing film to the interference pattern created when two laser light sources shine on an object.'),
  'D021641': NtO(MH='Recognition, Psychology', MS='The knowledge or perception that someone or something present has been previously encountered.'),
  'D021661': NtO(MH='Management Service Organizations', MS='Voluntarily-formed groups of healthcare professionals who join for common management services and other benefits such as collective bargaining agreements with reimbursement agents. The physical assets of a practice are controlled by the MSO which also provides billing, collections, and similar services. The practitioner retains control of patient records and management of patient care.'),
  'D021681': NtO(MH='Scedosporium', MS='A mitosporic fungal genus previously called Monosporium. Teleomorphs include PSEUDALLESCHERIA.'),
  'D021701': NtO(MH='Sentinel Lymph Node Biopsy', MS='A diagnostic procedure used to determine whether LYMPHATIC METASTASIS has occurred. The SENTINEL LYMPH NODE is the first lymph node to receive drainage from a neoplasm.'),
  'D021721': NtO(MH='Balloon Occlusion', MS='Use of a balloon CATHETER to block the flow of blood through an artery or vein.'),
  'D021741': NtO(MH='Eimeriida', MS='An order of parasitic organisms in the class COCCIDIA. Families include CRYPTOSPORIDIIDAE; EIMERIIDAE; and SARCOCYSTIDAE.'),
  'D021742': NtO(MH='Cryptosporidiidae', MS='A family of parasitic organisms in the order EIMERIIDA. CRYPTOSPORIDIUM is the most important genus.'),
  'D021743': NtO(MH='Eimeriidae', MS='A large family of parasites in the order EIMERIIDA. They cause COCCIDIOSIS in a number of vertebrates including humans.'),
  'D021744': NtO(MH='Cyclospora', MS='A genus of coccidian parasites in the family EIMERIIDAE. Cyclospora cayetanensis is pathogenic in humans, probably transmitted via the fecal-oral route, and causes nausea and diarrhea.'),
  'D021745': NtO(MH='Sarcocystidae', MS='A family of parasitic organisms in the order EIMERIIDAE. They form tissue-cysts in their intermediate hosts, ultimately leading to pathogenesis in the final hosts that includes various mammals (including humans) and birds. The most important genera include NEOSPORA; SARCOCYSTIS; and TOXOPLASMA.'),
  'D021782': NtO(MH='Multicystic Dysplastic Kidney', MS='A nongenetic defect due to malformation of the KIDNEY which appears as a bunch of grapes with multiple renal cysts but lacking the normal renal bean shape, and the collection drainage system. This condition can be detected in-utero with ULTRASONOGRAPHY.'),
  'D021801': NtO(MH='Zygapophyseal Joint', MS='The joint that occurs between facets of the interior and superior articular processes of adjacent VERTEBRAE.'),
  'D021821': NtO(MH='Communicable Diseases, Emerging', MS='Infectious diseases that are novel in their outbreak ranges (geographic and host) or transmission mode.'),
  'D021841': NtO(MH='Safe Sex', MS='Sexual behavior that prevents or reduces the spread of SEXUALLY TRANSMITTED DISEASES or PREGNANCY.'),
  'D021861': NtO(MH='Microsporidia, Unclassified', MS='Includes newly defined organisms as well as some that will never be classified to the genus and/or species level because of loss of the specimen or other information.'),
  'D021862': NtO(MH='Enterocytozoon', MS='A genus of parasitic FUNGI in the family Enterocytozoonidae, which infects humans. Enterocytozoon bieneusi has been found in the intestines of patients with AIDS.'),
  'D021863': NtO(MH='Vittaforma', MS='A genus of parasitic FUNGI in the family Nosematidae that infects humans. Vittaforma corneae has been isolated from AIDS patients.'),
  'D021864': NtO(MH='Pleistophora', MS='A genus of parasitic FUNGI in the family Pleistophoridae that infects humans.'),
  'D021865': NtO(MH='Isosporiasis', MS='Infection with parasitic protozoa of the genus ISOSPORA, producing intestinal disease. It is caused by ingestion of oocysts and can produce tissue cysts.'),
  'D021866': NtO(MH='Cyclosporiasis', MS='Infection with parasitic protozoa of the genus CYCLOSPORA. It is distributed globally and causes a diarrheal illness. Transmission is waterborne.'),
  'D021881': NtO(MH='DNA, Catalytic', MS='Molecules of DNA that possess enzymatic activity.'),
  'D021901': NtO(MH='DNA, Intergenic', MS="Any of the DNA in between gene-coding DNA, including untranslated regions, 5' and 3' flanking regions, INTRONS, non-functional pseudogenes, and non-functional repetitive sequences. This DNA may or may not encode regulatory functions."),
  'D021902': NtO(MH='Enterobacter aerogenes', MS='Gram-negative, capsulated, gas-producing rods found widely in nature. Both motile and non-motile strains exist. The species is closely related to KLEBSIELLA PNEUMONIAE and is frequently associated with nosocomial infections'),
  'D021903': NtO(MH='DNA, Ribosomal Spacer', MS='The intergenic DNA segments that are between the ribosomal RNA genes (internal transcribed spacers) and between the tandemly repeated units of rDNA (external transcribed spacers and nontranscribed spacers).'),
  'D021921': NtO(MH='Aortic Stenosis, Supravalvular', MS='A pathological constriction occurring in the region above the AORTIC VALVE. It is characterized by restricted outflow from the LEFT VENTRICLE into the AORTA.'),
  'D021922': NtO(MH='Discrete Subaortic Stenosis', MS='A type of constriction that is caused by the presence of a fibrous ring (discrete type) below the AORTIC VALVE, anywhere between the aortic valve and the MITRAL VALVE. It is characterized by restricted outflow from the LEFT VENTRICLE into the AORTA.'),
  'D021941': NtO(MH='Caveolae', MS='Endocytic/exocytic CELL MEMBRANE STRUCTURES rich in glycosphingolipids, cholesterol, and lipid-anchored membrane proteins that function in ENDOCYTOSIS (potocytosis), transcytosis, and SIGNAL TRANSDUCTION. Caveolae assume various shapes from open pits to closed vesicles. Caveolar coats are composed of CAVEOLINS.'),
  'D021961': NtO(MH='Cell Membrane Structures', MS='Structures which are part of the CELL MEMBRANE or have cell membrane as a major part of their structure.'),
  'D021962': NtO(MH='Membrane Microdomains', MS='Detergent-insoluble CELL MEMBRANE components. They are enriched in SPHINGOLIPIDS and CHOLESTEROL and clustered with glycosyl-phosphatidylinositol (GPI)-anchored proteins.'),
  'D021982': NtO(MH='Cell-Matrix Junctions', MS='Specialized areas at the CELL MEMBRANE where a cell attaches to the EXTRACELLULAR MATRIX or other substratum.'),
  'D021983': NtO(MH='Cyclophilins', MS='A family of peptidyl-prolyl cis-trans isomerases that bind to CYCLOSPORINS and regulate the IMMUNE SYSTEM. EC 5.2.1.-'),
  'D021984': NtO(MH='Cyclophilin A', MS='A 17-KDa cytoplasmic PEPTIDYLPROLYL ISOMERASE involved in immunoregulation. It is a member of the cyclophilin family of proteins that binds to CYCLOSPORINE.'),
  'D022001': NtO(MH='Focal Adhesions', MS='An anchoring junction of the cell to a non-cellular substrate. It is composed of a specialized area of the plasma membrane where bundles of the ACTIN CYTOSKELETON terminate and attach to the transmembrane linkers, INTEGRINS, which in turn attach through their extracellular domains to EXTRACELLULAR MATRIX PROTEINS.'),
  'D022002': NtO(MH='Hemidesmosomes', MS='An anchoring junction of the cell to a non-cellular substrate, similar in morphology to halves of DESMOSOMES. They are composed of specialized areas of the plasma membrane where INTERMEDIATE FILAMENTS bind on the cytoplasmic face to the transmembrane linkers, INTEGRINS, via intracellular attachment proteins, while the extracellular domain of the integrins binds to EXTRACELLULAR MATRIX PROTEINS.'),
  'D022003': NtO(MH='Cell Nucleus Structures', MS='Structures that are part of or contained in the CELL NUCLEUS.'),
  'D022004': NtO(MH='Chromosome Structures', MS='Structures which are contained in or part of CHROMOSOMES.'),
  'D022005': NtO(MH='Adherens Junctions', MS='Anchoring points where the CYTOSKELETON of neighboring cells are connected to each other. They are composed of specialized areas of the plasma membrane where bundles of the ACTIN CYTOSKELETON attach to the membrane through the transmembrane linkers, CADHERINS, which in turn attach through their extracellular domains to cadherins in the neighboring cell membranes. In sheets of cells, they form into adhesion belts (zonula adherens) that go all the way around a cell.'),
  'D022021': NtO(MH='Tacrolimus Binding Proteins', MS='A family of immunophilin proteins that bind to the immunosuppressive drugs TACROLIMUS (also known as FK506) and SIROLIMUS. EC 5.2.1.-'),
  'D022022': NtO(MH='Nuclear Pore', MS='An opening through the NUCLEAR ENVELOPE formed by the nuclear pore complex which transports nuclear proteins or RNA into or out of the CELL NUCLEUS and which, under some conditions, acts as an ion channel.'),
  'D022041': NtO(MH='Euchromatin', MS='Chromosome regions that are loosely packaged and more accessible to RNA polymerases than HETEROCHROMATIN. These regions also stain differentially in CHROMOSOME BANDING preparations.'),
  'D022061': NtO(MH='Tacrolimus Binding Protein 1A', MS='A 12-KDa tacrolimus binding protein that is found associated with and may modulate the function of calcium release channels. It is a peptidyl-prolyl cis/trans isomerase which is inhibited by both tacrolimus (commonly called FK506) and SIROLIMUS.'),
  'D022062': NtO(MH='Electrophysiologic Techniques, Cardiac', MS="Methods to induce and measure electrical activities at specific sites in the heart to diagnose and treat problems with the heart's electrical system."),
  'D022081': NtO(MH='Cell Surface Extensions', MS='Specialized structures of the cell that extend the cell membrane and project out from the cell surface.'),
  'D022082': NtO(MH='Cellular Structures', MS='Components of a cell.'),
  'D022083': NtO(MH='Cytoplasmic Structures', MS='Components of the cytoplasm excluding the CYTOSOL.'),
  'D022101': NtO(MH='Microtubule-Organizing Center', MS='An amorphous region of electron dense material in the cytoplasm from which the MICROTUBULES polymerization is nucleated. The pericentriolar region of the CENTROSOME which surrounds the CENTRIOLES is an example.'),
  'D022121': NtO(MH='Cholera Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with VIBRIO CHOLERAE. The original cholera vaccine consisted of killed bacteria, but other kinds of vaccines now exist.'),
  'D022122': NtO(MH='Anthrax Vaccines', MS='Vaccines or candidate vaccines used to prevent ANTHRAX.'),
  'D022123': NtO(MH='Lyme Disease Vaccines', MS='Vaccines or candidate vaccines used to prevent LYME DISEASE.'),
  'D022124': NtO(MH='Hyperammonemia', MS='Elevated level of AMMONIA in the blood. It is a sign of defective CATABOLISM of AMINO ACIDS or ammonia to UREA.'),
  'D022125': NtO(MH='Lacerations', MS='Torn, ragged, mangled wounds.'),
  'D022126': NtO(MH='Access to Information', MS="Individual's rights to obtain and use information collected or generated by others."),
  'D022141': NtO(MH='Jaagsiekte sheep retrovirus', MS='A BETARETROVIRUS that causes pulmonary adenomatosis in sheep (PULMONARY ADENOMATOSIS, OVINE).'),
  'D022142': NtO(MH='Secretory Vesicles', MS='Vesicles derived from the GOLGI APPARATUS containing material to be released at the cell surface.'),
  'D022143': NtO(MH='Family Nursing', MS='The provision of care involving the nursing process, to families and family members in health and illness situations. From Lippincott Manual of Nursing Practice. 6th ed.'),
  'D022161': NtO(MH='Transport Vesicles', MS='Vesicles that are involved in shuttling cargo from the interior of the cell to the cell surface, from the cell surface to the interior, across the cell or around the cell to various locations.'),
  'D022162': NtO(MH='Cytoplasmic Vesicles', MS='Membrane-limited structures derived from the plasma membrane or various intracellular membranes which function in storage, transport or metabolism.'),
  'D022163': NtO(MH='Clathrin-Coated Vesicles', MS='Vesicles formed when cell-membrane coated pits (COATED PITS, CELL-MEMBRANE) invaginate and pinch off. The outer surface of these vesicles is covered with a lattice-like network of the protein CLATHRIN. Shortly after formation, however, the clathrin coat is removed and the vesicles are referred to as ENDOSOMES.'),
  'D022181': NtO(MH='COP-Coated Vesicles', MS='TRANSPORT VESICLES formed when cell-membrane coated pits (COATED PITS, CELL-MEMBRANE) invaginate and pinch off. The outer surface of these vesicles is covered with a lattice-like network of COP (coat protein complex) proteins, either COPI or COPII. COPI coated vesicles transport backwards from the cisternae of the GOLGI APPARATUS to the rough endoplasmic reticulum (ENDOPLASMIC RETICULUM, ROUGH), while COPII coated vesicles transport forward from the rough endoplasmic reticulum to the Golgi apparatus.'),
  'D022201': NtO(MH='Chromosomes, Artificial', MS='DNA constructs that are composed of, at least, elements such as a REPLICATION ORIGIN; TELOMERE; and CENTROMERE, that are required for successful replication, propagation to and maintenance in progeny cells. In addition, they are constructed to carry other sequences for analysis or gene transfer.'),
  'D022202': NtO(MH='Chromosomes, Artificial, Bacterial', MS='DNA constructs that are composed of, at least, a REPLICATION ORIGIN, for successful replication, propagation to and maintenance as an extra chromosome in bacteria. In addition, they can carry large amounts (about 200 kilobases) of other sequence for a variety of bioengineering purposes.'),
  'D022221': NtO(MH='Chromosomes, Artificial, Mammalian', MS='DNA constructs that are composed of, at least, all elements, such as a REPLICATION ORIGIN; TELOMERE; and CENTROMERE, that are required for successful replication, propagation to and maintainance in progeny mammalian cells. In addition, they are constructed to carry other sequences for analysis or gene transfer.'),
  'D022222': NtO(MH='Chromosomes, Artificial, Human', MS='DNA constructs that are composed of, at least, all elements, such as a REPLICATION ORIGIN; TELOMERE; and CENTROMERE, required for successful replication, propagation to and maintainance in progeny human cells. In addition, they are constructed to carry other sequences for analysis or gene transfer.'),
  'D022223': NtO(MH='Vaccines, Subunit', MS='Vaccines consisting of one or more antigens that stimulate a strong immune response. They are purified from microorganisms or produced by recombinant DNA techniques, or they can be chemically synthesized peptides.'),
  'D022241': NtO(MH='Parainfluenza Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with parainfluenza viruses in humans and animals.'),
  'D022242': NtO(MH='Pneumococcal Vaccines', MS='Vaccines or candidate vaccines used to prevent infections with STREPTOCOCCUS PNEUMONIAE.'),
  'D022243': NtO(MH='Rotavirus Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with ROTAVIRUS.'),
  'D022261': NtO(MH='Respiratory Syncytial Virus Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with RESPIRATORY SYNCYTIAL VIRUSES.'),
  'D022281': NtO(MH='Shigella Vaccines', MS='Vaccines or candidate vaccines used to prevent bacillary dysentery (DYSENTERY, BACILLARY) caused by species of SHIGELLA.'),
  'D022282': NtO(MH='Vaccines, Acellular', MS='Vaccines that are produced by using only the antigenic part of the disease causing organism. They often require a "booster" every few years to maintain their effectiveness.'),
  'D022283': NtO(MH='Herpesvirus Vaccines', MS='Vaccines or candidate vaccines used to prevent infection by any virus from the family HERPESVIRIDAE.'),
  'D022301': NtO(MH='Herpes Simplex Virus Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with viruses from the genus SIMPLEXVIRUS. This includes vaccines for HSV-1 and HSV-2.'),
  'D022321': NtO(MH='Japanese Encephalitis Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with Japanese B encephalitis virus (ENCEPHALITIS VIRUS, JAPANESE).'),
  'D022341': NtO(MH='Yellow Fever Vaccine', MS='Vaccine used to prevent YELLOW FEVER. It consists of a live attenuated 17D strain of the YELLOW FEVER VIRUS.'),
  'D022361': NtO(MH='Escherichia coli Vaccines', MS='Vaccines or candidate vaccines used to prevent or treat both enterotoxigenic and enteropathogenic Escherichia coli infections.'),
  'D022362': NtO(MH='Hepatitis A Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with hepatitis A virus (HEPATOVIRUS).'),
  'D022401': NtO(MH='Meningococcal Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with NEISSERIA MENINGITIDIS.'),
  'D022421': NtO(MH='Pseudorabies Vaccines', MS="Vaccines or candidate vaccines used to prevent PSEUDORABIES (Aujeszky's disease), a herpesvirus of swine and other animals."),
  'D022422': NtO(MH='Diphtheria-Tetanus Vaccine', MS='A combined vaccine used to prevent infection with diphtheria and tetanus toxoid. This is used in place of DTP vaccine (DIPHTHERIA-TETANUS-PERTUSSIS VACCINE) when PERTUSSIS VACCINE is contraindicated.'),
  'D022423': NtO(MH='Myeloid Cells', MS='The classes of BONE MARROW-derived blood cells in the monocytic series (MONOCYTES and their precursors) and granulocytic series (GRANULOCYTES and their precursors).'),
  'D022461': NtO(MH='Caveolins', MS='The main structural proteins of CAVEOLAE. Several distinct genes for caveolins have been identified.'),
  'D022462': NtO(MH='Vaccines, Edible', MS='Vaccines or candidate vaccines derived from edible plants. Transgenic plants (PLANTS, TRANSGENIC) are used as recombinant protein production systems and the edible plant tissue functions as an oral vaccine.'),
  'D022482': NtO(MH='Cytomegalovirus Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with CYTOMEGALOVIRUS.'),
  'D022501': NtO(MH='Marek Disease Vaccines', MS='Vaccines or candidate vaccines used to prevent MAREK DISEASE, an avian disease caused by a herpesvirus.'),
  'D022502': NtO(MH='Stress Fibers', MS='Bundles of actin filaments (ACTIN CYTOSKELETON) and myosin-II that span across the cell attaching to the cell membrane at FOCAL ADHESIONS and to the network of INTERMEDIATE FILAMENTS that surrounds the nucleus.'),
  'D022521': NtO(MH='Ligase Chain Reaction', MS='A DNA amplification technique based upon the ligation of OLIGONUCLEOTIDE PROBES. The probes are designed to exactly match two adjacent sequences of a specific target DNA. The chain reaction is repeated in three steps in the presence of excess probe: (1) heat denaturation of double-stranded DNA, (2) annealing of probes to target DNA, and (3) joining of the probes by thermostable DNA ligase. After the reaction is repeated for 20-30 cycles the production of ligated probe is measured.'),
  'D022541': NtO(MH='Streptococcal Vaccines', MS='Vaccines or candidate vaccines used to prevent STREPTOCOCCAL INFECTIONS.'),
  'D022542': NtO(MH='Measles-Mumps-Rubella Vaccine', MS='A combined vaccine used to prevent MEASLES; MUMPS; and RUBELLA.'),
  'D022561': NtO(MH='Advanced Cardiac Life Support', MS='The use of sophisticated methods and equipment to treat cardiopulmonary arrest. Advanced Cardiac Life Support (ACLS) includes the use of specialized equipment to maintain the airway, early defibrillation and pharmacological therapy.'),
  'D022562': NtO(MH='Salmonella Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with SALMONELLA. This includes vaccines used to prevent TYPHOID FEVER or PARATYPHOID FEVER; (TYPHOID-PARATYPHOID VACCINES), and vaccines used to prevent nontyphoid salmonellosis.'),
  'D022581': NtO(MH='Vaccines, Marker', MS='Vaccines used in conjunction with diagnostic tests to differentiate vaccinated animals from carrier animals. Marker vaccines can be either a subunit or a gene-deleted vaccine.'),
  'D022603': NtO(MH='Shiga Toxins', MS='A class of toxins that inhibit protein synthesis by blocking the interaction of ribosomal RNA; (RNA, RIBOSOMAL) with PEPTIDE ELONGATION FACTORS. They include SHIGA TOXIN which is produced by SHIGELLA DYSENTERIAE and a variety of shiga-like toxins that are produced by pathologic strains of ESCHERICHIA COLI such as ESCHERICHIA COLI O157.'),
  'D022621': NtO(MH='Shiga Toxin', MS='A toxin produced by SHIGELLA DYSENTERIAE. It is the prototype of class of toxins that inhibit protein synthesis by blocking the interaction of ribosomal RNA; (RNA, RIBOSOMAL) with PEPTIDE ELONGATION FACTORS.'),
  'D022622': NtO(MH='Shiga Toxin 1', MS='A toxin produced by certain pathogenic strains of ESCHERICHIA COLI such as ESCHERICHIA COLI O157. It is closely related to SHIGA TOXIN produced by SHIGELLA DYSENTERIAE.'),
  'D022641': NtO(MH='Shiga Toxin 2', MS='A toxin produced by certain pathogenic strains of ESCHERICHIA COLI such as ESCHERICHIA COLI O157. It shares 50-60% homology with SHIGA TOXIN and SHIGA TOXIN 1.'),
  'D022642': NtO(MH='Vaccines, Contraceptive', MS='Vaccines or candidate vaccines used to prevent conception.'),
  'D022661': NtO(MH='RNA, Untranslated', MS='RNA which does not code for protein but has some enzymatic, structural or regulatory function. Although ribosomal RNA (RNA, RIBOSOMAL) and transfer RNA (RNA, TRANSFER) are also untranslated RNAs they are not included in this scope.'),
  'D022681': NtO(MH='Diphtheria-Tetanus-acellular Pertussis Vaccines', MS='Combined vaccines consisting of DIPHTHERIA TOXOID; TETANUS TOXOID; and an acellular form of PERTUSSIS VACCINE. At least five different purified antigens of B. pertussis have been used in various combinations in these vaccines.'),
  'D022701': NtO(MH='Virosomes', MS='Semi-synthetic complex derived from nucleic-acid free viral particles. They are essentially reconstituted viral coats, where the infectious nucleocapsid is replaced by a compound of choice. Virosomes retain their fusogenic activity and thus deliver the incorporated compound (antigens, drugs, genes) inside the target cell. They can be used for vaccines (VACCINES, VIROSOME), drug delivery, or gene transfer.'),
  'D022702': NtO(MH='Receptors, Adrenergic, beta-3', MS='A subclass of beta-adrenergic receptors (RECEPTORS, ADRENERGIC, BETA). The beta-3 adrenergic receptors are the predominant beta-adrenergic receptor type expressed in white and brown ADIPOCYTES and are involved in modulating ENERGY METABOLISM and THERMOGENESIS.'),
  'D022721': NtO(MH='Vaccines, Virosome', MS='Vaccines using VIROSOMES as the antigen delivery system that stimulates the desired immune response.'),
  'D022722': NtO(MH='Thermogenesis', MS='The generation of heat in order to maintain body temperature. The uncoupled oxidation of fatty acids contained within brown adipose tissue and SHIVERING are examples of thermogenesis in MAMMALS.'),
  'D022761': NtO(MH='Gene Transfer, Horizontal', MS="The naturally occurring transmission of genetic information between organisms, related or unrelated, circumventing parent-to-offspring transmission. Horizontal gene transfer may occur via a variety of naturally occurring processes such as GENETIC CONJUGATION; GENETIC TRANSDUCTION; and TRANSFECTION. It may result in a change of the recipient organism's genetic composition (TRANSFORMATION, GENETIC)."),
  'D022762': NtO(MH='CCAAT-Enhancer-Binding Proteins', MS='A class of proteins that were originally identified by their ability to bind the DNA sequence CCAAT. The typical CCAAT-enhancer binding protein forms dimers and consists of an activation domain, a DNA-binding basic region, and a leucine-rich dimerization domain (LEUCINE ZIPPERS). CCAAT-BINDING FACTOR is structurally distinct type of CCAAT-enhancer binding protein consisting of a trimer of three different subunits.'),
  'D022763': NtO(MH='CCAAT-Enhancer-Binding Protein-alpha', MS='A CCAAT-enhancer-binding protein found in LIVER; ADIPOSE TISSUE; INTESTINES; LUNG; ADRENAL GLANDS; PLACENTA; OVARY and peripheral blood mononuclear cells (LEUKOCYTES, MONONUCLEAR). Experiments with knock-out mice have demonstrated that CCAAT-enhancer binding protein-alpha is essential for the functioning and differentiation of HEPATOCYTES and ADIPOCYTES.'),
  'D022781': NtO(MH='Hepatocytes', MS='The main structural component of the LIVER. They are specialized EPITHELIAL CELLS that are organized into interconnected plates called lobules.'),
  'D022782': NtO(MH='CCAAT-Enhancer-Binding Protein-beta', MS='A CCAAT-enhancer-binding protein found in LIVER; INTESTINES; LUNG and ADIPOSE TISSUE. It is an important mediator of INTERLEUKIN-6 signaling.'),
  'D022783': NtO(MH='Torque teno virus', MS='A species of non-enveloped DNA virus in the genus ANELLOVIRUS, associated with BLOOD TRANSFUSIONS; and HEPATITIS. However, no etiological role has been found for TTV in hepatitis.'),
  'D022801': NtO(MH='Complementarity Determining Regions', MS='Three regions (CDR1; CDR2 and CDR3) of amino acid sequence in the IMMUNOGLOBULIN VARIABLE REGION that are highly divergent. Together the CDRs from the light and heavy immunoglobulin chains form a surface that is complementary to the antigen. These regions are also present in other members of the immunoglobulin superfamily, for example, T-cell receptors (RECEPTORS, ANTIGEN, T-CELL).'),
  'D022821': NtO(MH='RNA Splice Sites', MS='Nucleotide sequences located at the ends of EXONS and recognized in pre-messenger RNA by SPLICEOSOMES. They are joined during the RNA SPLICING reaction, forming the junctions between exons.'),
  'D022861': NtO(MH='Hermanski-Pudlak Syndrome', MS='Syndrome characterized by the triad of oculocutaneous albinism (ALBINISM, OCULOCUTANEOUS); PLATELET STORAGE POOL DEFICIENCY; and lysosomal accumulation of ceroid lipofuscin.'),
  'D022882': NtO(MH='Trihalomethanes', MS='Methanes substituted with three halogen atoms, which may be the same or different.'),
  'D022902': NtO(MH='Unpublished Work', MS='Work that has not been formally published.'),
  'D022903': NtO(MH='Government Publication', MS='Work consisting of publications issued by local, regional, or national governments or by their agencies or subdivisions.'),
  'D022921': NtO(MH='Book Review', MS='Work consisting of critical analyses of books or other monographic works.'),
  'D022922': NtO(MH='Fictional Work', MS='Work consisting of creative writing, not presented as factual.'),
  'D022923': NtO(MH='Textbook', MS='Book intended for use in the study of specific subjects, containing systematic presentation of the principles and essential knowledge of the subjects.'),
  'D022981': NtO(MH='Allelic Imbalance', MS='A situation where one member (allele) of a gene pair is lost (LOSS OF HETEROZYGOSITY) or amplified.'),
  'D023001': NtO(MH='Transplantation Tolerance', MS='An induced state of non-reactivity to grafted tissue from a donor organism that would ordinarily trigger a cell-mediated or humoral immune response.'),
  'D023041': NtO(MH='Xenograft Model Antitumor Assays', MS='In vivo methods of screening investigative anticancer drugs, biologic response modifiers or radiotherapies. Human tumor tissue or cells are transplanted into mice or rats followed by tumor treatment regimens. A variety of outcomes are monitored to assess antitumor effectiveness.'),
  'D023061': NtO(MH='Gene Order', MS='The sequential location of genes on a chromosome.'),
  'D023062': NtO(MH='Receptors, Interleukin-8A', MS='High-affinity G-protein-coupled receptors for INTERLEUKIN-8 present on NEUTROPHILS; MONOCYTES; and BASOPHILS.'),
  'D023063': NtO(MH='Receptors, Interleukin-8B', MS='High-affinity G-protein-coupled receptors for INTERLEUKIN-8 present on NEUTROPHILS; MONOCYTES; and T-LYMPHOCYTES. These receptors also bind several other CXC CHEMOKINES.'),
  'D023081': NtO(MH='CCAAT-Binding Factor', MS='A heterotrimeric DNA-binding protein that binds to CCAAT motifs in the promoters of eukaryotic genes. It is composed of three subunits: A, B and C.'),
  'D023082': NtO(MH='Defensins', MS='Family of antimicrobial peptides that have been identified in humans, animals, and plants. They are thought to play a role in host defenses against infections, inflammation, wound repair, and acquired immunity.'),
  'D023083': NtO(MH='beta-Defensins', MS='DEFENSINS found mainly in epithelial cells.'),
  'D023084': NtO(MH='alpha-Defensins', MS='DEFENSINS found in azurophilic granules of neutrophils and in the secretory granules of intestinal PANETH CELLS.'),
  'D023102': NtO(MH='Anoikis', MS='APOPTOSIS triggered by loss of contact with the EXTRACELLULAR MATRIX.'),
  'D023121': NtO(MH='RNA, Messenger, Stored', MS='Messenger RNA that is stored in a masked state for translation at a later time. Distinguish from RNA, UNTRANSLATED which refers to non-messenger RNA, i.e. RNA that does not code for protein.'),
  'D023141': NtO(MH='Herpesvirus 4, Equid', MS='A species of VARICELLOVIRUS and the chief cause of rhinopneumonitis in horses.'),
  'D023181': NtO(MH='Antimicrobial Cationic Peptides', MS='Small cationic peptides that are an important component, in most species, of early innate and induced defenses against invading microbes. In animals they are found on mucosal surfaces, within phagocytic granules, and on the surface of the body. They are also found in insects and plants. Among others, this group includes the DEFENSINS, protegrins, tachyplesins, and thionins. They displace DIVALENT CATIONS from phosphate groups of MEMBRANE LIPIDS leading to disruption of the membrane.'),
  'D023201': NtO(MH='CD40 Ligand', MS='A membrane glycoprotein and differentiation antigen expressed on the surface of T-cells that binds to CD40 ANTIGENS on B-LYMPHOCYTES and induces their proliferation. Mutation of the gene for CD40 ligand is a cause of HYPER-IGM IMMUNODEFICIENCY SYNDROME, TYPE 1.'),
  'D023241': NtO(MH='Antiretroviral Therapy, Highly Active', MS='Drug regimens, for patients with HIV INFECTIONS, that aggressively suppress HIV replication. The regimens usually involve administration of three or more different drugs including a protease inhibitor.'),
  'D023261': NtO(MH='Glasgow Outcome Scale', MS='A scale that assesses the outcome of serious craniocerebral injuries, based on the level of regained social functioning.'),
  'D023281': NtO(MH='Genomics', MS='The systematic study of the complete DNA sequences (GENOME) of organisms.'),
  'D023282': NtO(MH='Nuclease Protection Assays', MS='Techniques for measuring specific nucleic acid interaction with another nucleic acid or with a protein by digestion of the non-interacting nucleic acid by various nucleases. After all non-interacting regions are eliminated by nuclease digestion, the protected nucleic acid that remains is analyzed. DNA FOOTPRINTING utilizes this technique to analyze the DNA contact sites of DNA-BINDING PROTEINS.'),
  'D023302': NtO(MH='Protein Footprinting', MS='A method for determining points of contact between interacting proteins or binding sites of proteins to nucleic acids. Protein footprinting utilizes a protein cutting reagent or protease. Protein cleavage is inhibited where the proteins, or nucleic acids and protein, contact each other. After completion of the cutting reaction, the remaining peptide fragments are analyzed by electrophoresis.'),
  'D023303': NtO(MH='Oxazolidinones', MS='Derivatives of oxazolidin-2-one. They represent an important class of synthetic antibiotic agents.'),
  'D023321': NtO(MH='Poliovirus Vaccines', MS='Vaccines used to prevent POLIOMYELITIS. They include inactivated (POLIOVIRUS VACCINE, INACTIVATED) and oral vaccines (POLIOVIRUS VACCINE, ORAL).'),
  'D023341': NtO(MH='Chills', MS='The sudden sensation of being cold. It may be accompanied by SHIVERING.'),
  'D023361': NtO(MH='Validation Study', MS='Works consisting of research using processes by which the reliability and relevance of a procedure for a specific purpose are established.'),
  'D023362': NtO(MH='Evaluation Study', MS='Works consisting of studies determining the effectiveness or utility of processes, personnel, and equipment.'),
  'D023381': NtO(MH='Endpoint Determination', MS='Establishment of the level of a quantifiable effect indicative of a biologic process. The evaluation is frequently to detect the degree of toxic or therapeutic effect.'),
  'D023382': NtO(MH='Toxicity Tests, Acute', MS='Experiments designed to determine the potential toxic effects of one-time, short-term exposure to a chemical or chemicals.'),
  'D023401': NtO(MH='Animal Use Alternatives', MS='Alternatives to the use of animals in research, testing, and education. The alternatives may include reduction in the number of animals used, replacement of animals with a non-animal model or with animals of a species lower phylogenetically, or refinement of methods to minimize pain and distress of animals used.'),
  'D023421': NtO(MH='Models, Animal', MS='Non-human animals, selected because of specific characteristics, for use in experimental research, teaching, or testing.'),
  'D023422': NtO(MH='Skin Irritancy Tests', MS='Tests or bioassays that measure the skin sensitization potential of various chemicals.'),
  'D023441': NtO(MH='Local Lymph Node Assay', MS='The local lymph node assay (LLNA) is an alternative method for the identification of chemicals that have the ability to cause skin sensitization and allergic contact dermatitis. Endpoints have been established so fewer animals are required and less painful procedures are used.'),
  'D023461': NtO(MH='Myeloid Progenitor Cells', MS='Stem cells derived from HEMATOPOIETIC STEM CELLS. Derived from these myeloid progenitor cells are the MEGAKARYOCYTES; ERYTHROID CELLS; MYELOID CELLS; and some DENDRITIC CELLS.'),
  'D023481': NtO(MH='Terrorism', MS='The use or threatened use of force or violence against persons or property in violation of criminal laws for purposes of intimidation, coercion, or ransom, in support of political or social objectives.'),
  'D023482': NtO(MH='Bioterrorism', MS='The use of biological agents in TERRORISM. This includes the malevolent use of BACTERIA; VIRUSES; or other BIOLOGICAL TOXINS against people, ANIMALS; or PLANTS.'),
  'D023501': NtO(MH='Chlamydophila', MS='A genus of the family CHLAMYDIACEAE comprising gram-negative non CHLAMYDIA TRACHOMATIS-like species infecting vertebrates. Chlamydophila do not produce detectable quantities of glycogen. The type species is CHLAMYDOPHILA PSITTACI.'),
  'D023502': NtO(MH='Chlamydia muridarum', MS='Species of CHLAMYDIA causing pneumonitis in mice and hamsters. These isolates formerly belonged to CHLAMYDIA TRACHOMATIS.'),
  'D023521': NtO(MH='Chlamydophila Infections', MS='Infections with bacteria of the genus CHLAMYDOPHILA.'),
  'D023581': NtO(MH='HIV Fusion Inhibitors', MS='Inhibitors of the fusion of HIV to host cells, preventing viral entry. This includes compounds that block attachment of HIV ENVELOPE PROTEIN GP120 to CD4 RECEPTORS.'),
  'D023582': NtO(MH='Alzheimer Vaccines', MS='Vaccines or candidate vaccines used to prevent or treat ALZHEIMER DISEASE.'),
  'D023641': NtO(MH='Archaeal Viruses', MS='Viruses whose hosts are in the domain ARCHAEA.'),
  'D023642': NtO(MH='Rudiviridae', MS='Family of rod-shaped DNA viruses infecting ARCHAEA. They lack viral envelopes or lipids.'),
  'D023643': NtO(MH='Lipothrixviridae', MS='Family of enveloped, lipid-containing, filamentous DNA viruses that infect ARCHAEA.'),
  'D023644': NtO(MH='Guttaviridae', MS='Family of archaeal viruses with a single genus: Sulfolobus SNDV-like Viruses.'),
  'D023661': NtO(MH='Esociformes', MS='An order of fishes native to North America and Northern Eurasia, consisting of two families: ESOCIDAE (pikes) and UMBRIDAE (mudminnows).'),
  'D023681': NtO(MH='Cichlids', MS='Common name for perch-like fish of the family Cichlidae, belonging to the suborder Labroidei, order PERCIFORMES.'),
  'D023701': NtO(MH='Smegmamorpha', MS='Group of fish under the superorder Acanthopterygii, separate from the PERCIFORMES, which includes swamp eels, mullets, sticklebacks, seahorses, spiny eels, rainbowfishes, and KILLIFISHES. The name is derived from the six taxa which comprise the group. (From http://www.nanfa.org/articles/Elassoma/elassoma.htm, 8/4/2000)'),
  'D023702': NtO(MH='Gymnotiformes', MS='An order of neotropical electric fish found chiefly in the waters of South America. They continually emit weak electric discharges, which they use in object location and communication. A most popular species of research interest is the electric eel, ELECTROPHORUS electricus.'),
  'D023721': NtO(MH='Tetraodontiformes', MS='A small order of primarily marine fish containing 340 species. Most have a rotund or box-like shape. TETRODOTOXIN is found in their liver and ovaries.'),
  'D023741': NtO(MH='Takifugu', MS='A genus of pufferfish commonly used for research.'),
  'D023761': NtO(MH='Beloniformes', MS='An order of fish in the group SMEGMAMORPHA, comprising adrianichthyids, medakas (ORYZIAS), needlefishes, halfbeaks, and flying fishes.'),
  'D023781': NtO(MH='Fundulidae', MS='Family of small, surface-dwelling fish that inhabit fresh and brackish waters, and coastal marine areas.'),
  'D023801': NtO(MH='Directly Observed Therapy', MS='A treatment method in which patients are under direct observation when they take their medication or receive their treatment. This method is designed to reduce the risk of treatment interruption and to ensure patient compliance.'),
  'D023802': NtO(MH='Cerclage, Cervical', MS='The surgical closure of the incompetent cervix uteri with suture material.'),
  'D023821': NtO(MH='Limb Salvage', MS='An alternative to amputation in patients with neoplasms, ischemia, fractures, and other limb-threatening conditions. Generally, sophisticated surgical procedures such as vascular surgery and reconstruction are used to salvage diseased limbs.'),
  'D023822': NtO(MH='Tissue Engineering', MS='Generating tissue in vitro for clinical applications, such as replacing wounded tissues or impaired organs. The use of TISSUE SCAFFOLDING enables the generation of complex multi-layered tissues and tissue structures.'),
  'D023842': NtO(MH='Rhinometry, Acoustic', MS='Diagnostic measurement of the nose and its cavity through acoustic reflections. Used to measure nasal anatomical landmarks, nasal septal deviation, and nasal airway changes in response to allergen provocation tests (NASAL PROVOCATION TESTS).'),
  'D023861': NtO(MH='Photography, Dental', MS='Photographic techniques used in ORTHODONTICS; DENTAL ESTHETICS; and patient education.'),
  'D023881': NtO(MH='Colonography, Computed Tomographic', MS='A non-invasive imaging method that uses computed tomographic data combined with specialized imaging software to examine the colon.'),
  'D023882': NtO(MH='Dominance, Ocular', MS='The functional superiority and preferential use of one eye over the other. The term is usually applied to superiority in sighting (VISUAL PERCEPTION) or motor task but not difference in VISUAL ACUITY or dysfunction of one of the eyes. Ocular dominance can be modified by visual input and NEUROTROPHIC FACTORS.'),
  'D023902': NtO(MH='Chromosome Pairing', MS='The alignment of CHROMOSOMES at homologous sequences.'),
  'D023903': NtO(MH='Coronary Restenosis', MS='Recurrent narrowing or constriction of a coronary artery following surgical procedures performed to alleviate a prior obstruction.'),
  'D023921': NtO(MH='Coronary Stenosis', MS='Narrowing or constriction of a coronary artery.'),
  'D023961': NtO(MH='Gonadal Dysgenesis, 46,XX', MS='The 46,XX gonadal dysgenesis may be sporadic or familial. Familial XX gonadal dysgenesis is transmitted as an autosomal recessive trait and its locus was mapped to chromosome 2. Mutation in the gene for the FSH receptor (RECEPTORS, FSH) was detected. Sporadic XX gonadal dysgenesis is heterogeneous and has been associated with trisomy-13 and trisomy-18. These phenotypic females are characterized by a normal stature, sexual infantilism, bilateral streak gonads, amenorrhea, elevated plasma LUTEINIZING HORMONE and FSH concentration.'),
  'D023981': NtO(MH='Sarcoma, Myeloid', MS='An extramedullary tumor of immature MYELOID CELLS or MYELOBLASTS. Granulocytic sarcoma usually occurs with or follows the onset of ACUTE MYELOID LEUKEMIA.'),
  'D024001': NtO(MH='HMGN Proteins', MS='A family of HIGH MOBILITY GROUP PROTEINS that bind to NUCLEOSOMES.'),
  'D024002': NtO(MH='HMGB Proteins', MS='A family of sequence-related proteins similar to HMGB1 PROTEIN that contains specific HMG-BOX DOMAINS.'),
  'D024003': NtO(MH='HMGA Proteins', MS='Proteins containing AT-HOOK MOTIFS that are rich in arginine and glycine residues. They bind to the minor grove of AT-rich regions of DNA.'),
  'D024021': NtO(MH='Fibril-Associated Collagens', MS='A family of non-fibrillar collagens that interact with FIBRILLAR COLLAGENS. They contain short triple helical domains interrupted by short non-helical domains and do not form into collagen fibrils.'),
  'D024022': NtO(MH='Fibrillar Collagens', MS='A family of structurally related collagens that form the characteristic collagen fibril bundles seen in CONNECTIVE TISSUE.'),
  'D024041': NtO(MH='Non-Fibrillar Collagens', MS='A family of structurally-related short-chain collagens that do not form large fibril bundles.'),
  'D024042': NtO(MH='Collagen Type I', MS='The most common form of fibrillar collagen. It is a major constituent of bone (BONE AND BONES) and SKIN and consists of a heterotrimer of two alpha1(I) and one alpha2(I) chains.'),
  'D024043': NtO(MH='Collagen Type II', MS='A fibrillar collagen found predominantly in CARTILAGE and vitreous humor. It consists of three identical alpha1(II) chains.'),
  'D024061': NtO(MH='Collagen Type III', MS='A fibrillar collagen consisting of three identical alpha1(III) chains that is widely distributed in many tissues containing COLLAGEN TYPE I. It is particularly abundant in BLOOD VESSELS and may play a role in tissues with elastic characteristics.'),
  'D024062': NtO(MH='Collagen Type V', MS='A fibrillar collagen found widely distributed as a minor component in tissues that contain COLLAGEN TYPE I and COLLAGEN TYPE III. It is a heterotrimeric molecule composed of alpha1(V), alpha2(V) and alpha3(V) subunits. Several forms of collagen type V exist depending upon the composition of the subunits that form the trimer.'),
  'D024081': NtO(MH='Collagen Type IX', MS='A fibril-associated collagen usually found crosslinked to the surface of COLLAGEN TYPE II fibrils. It is a heterotrimer containing alpha1(IX), alpha2(IX) and alpha3(IX) subunits.'),
  'D024082': NtO(MH='Collagen Type XII', MS='A fibril-associated collagen found in many tissues bearing high tensile stress, such as TENDONS and LIGAMENTS. It is comprised of a trimer of three identical alpha1(XII) chains.'),
  'D024101': NtO(MH='Mitochondrial Proteins', MS='Proteins encoded by the mitochondrial genome or proteins encoded by the nuclear genome that are imported to and resident in the MITOCHONDRIA.'),
  'D024102': NtO(MH='Collagen Type VIII', MS='A non-fibrillar collagen originally found in DESCEMET MEMBRANE. It is expressed in endothelial cell layers and in tissues undergoing active remodeling. It is heterotrimer comprised of alpha1(VIII) and alpha2(VIII) chains.'),
  'D024121': NtO(MH='Collagen Type X', MS='A non-fibrillar collagen found primarily in terminally differentiated hypertrophic CHONDROCYTES. It is a homotrimer of three identical alpha1(X) subunits.'),
  'D024141': NtO(MH='Collagen Type IV', MS='A non-fibrillar collagen found in the structure of BASEMENT MEMBRANE. Collagen type IV molecules assemble to form a sheet-like network which is involved in maintaining the structural integrity of basement membranes. The predominant form of the protein is comprised of two alpha1(IV) subunits and one alpha2(IV) subunit, however, at least six different alpha subunits can be incorporated into the heterotrimer.'),
  'D024142': NtO(MH='Collagen Type VI', MS='A non-fibrillar collagen that forms a network of MICROFIBRILS within the EXTRACELLULAR MATRIX of CONNECTIVE TISSUE. The alpha subunits of collagen type VI assemble into antiparallel, overlapping dimers which then align to form tetramers.'),
  'D024143': NtO(MH='Collagen Type VII', MS='A non-fibrillar collagen involved in anchoring the epidermal BASEMENT MEMBRANE to underlying tissue. It is a homotrimer comprised of C-terminal and N-terminal globular domains connected by a central triple-helical region.'),
  'D024161': NtO(MH='Collagen Type XI', MS='A fibrillar collagen found primarily in interstitial CARTILAGE. Collagen type XI is heterotrimer containing alpha1(XI), alpha2(XI) and alpha3(XI) subunits.'),
  'D024162': NtO(MH='Second-Look Surgery', MS='A followup operation to examine the outcome of the previous surgery and other treatments, such as chemotherapy or radiation therapy.'),
  'D024181': NtO(MH='Collagen Type XIII', MS='A non-fibrillar collagen found as a ubiquitously expressed membrane- associated protein. Type XIII collagen contains both collagenous and non-collagenous domains along with a transmembrane domain within its N-terminal region.'),
  'D024182': NtO(MH='Uniparental Disomy', MS='The presence in a cell of two paired chromosomes from the same parent, with no chromosome of that pair from the other parent. This chromosome composition stems from non-disjunction (NONDISJUNCTION, GENETIC) events during MEIOSIS. The disomy may be composed of both homologous chromosomes from one parent (heterodisomy) or a duplicate of one chromosome (isodisomy).'),
  'D024201': NtO(MH='Bystander Effect', MS='The result of a positive or negative response (to drugs, for example) in one cell being passed onto other cells via the GAP JUNCTIONS or the intracellular milieu.'),
  'D024202': NtO(MH='Electrophoretic Mobility Shift Assay', MS='An electrophoretic technique for assaying the binding of one compound to another. Typically one compound is labeled to follow its mobility during electrophoresis. If the labeled compound is bound by the other compound, then the mobility of the labeled compound through the electrophoretic medium will be retarded.'),
  'D024221': NtO(MH='Antineoplastic Protocols', MS='Clinical protocols used to inhibit the growth or spread of NEOPLASMS.'),
  'D024241': NtO(MH='HMGN1 Protein', MS='An evolutionarily-conserved 10-kDa nuclear protein that binds NUCLEOSOMES and may be involved in the process of CHROMATIN unfolding.'),
  'D024242': NtO(MH='HMGN2 Protein', MS='An evolutionarily conserved 9-KDa nuclear protein that binds NUCLEOSOMES and may be involved in the process of CHROMATIN unfolding.'),
  'D024243': NtO(MH='HMGB1 Protein', MS='A 24-kDa HMGB protein that binds to and distorts the minor grove of DNA.'),
  'D024261': NtO(MH='HMGB2 Protein', MS='A 23-kDa HMG-box protein that binds to and distorts the minor grove of DNA.'),
  'D024282': NtO(MH='Genes, sry', MS='The primary testis-determining gene in mammalians, located on the Y CHROMOSOME. It codes for a high mobility group box transcription factor (TRANSCRIPTION FACTORS) which initiates the development of the TESTES from the embryonic GONADS.'),
  'D024301': NtO(MH='Plant Bark', MS='The outer layer of the woody parts of plants.'),
  'D024321': NtO(MH='zeta Carotene', MS='A carotenoid produced in most carotenogenic organisms. It is one of several sequentially synthesized molecules that are precursors to BETA CAROTENE.'),
  'D024322': NtO(MH='Amino Acids, Aromatic', MS='Amino acids containing an aromatic side chain.'),
  'D024341': NtO(MH='Xanthophylls', MS='Oxygenated forms of carotenoids. They are usually derived from alpha and beta carotene.'),
  'D024342': NtO(MH='Amino Acids, Acidic', MS='Amino acids with side chains that are negatively charged at physiological pH.'),
  'D024343': NtO(MH='Open Bite', MS='A condition in which certain opposing teeth fail to establish occlusal contact when the jaws are closed.'),
  'D024361': NtO(MH='Amino Acids, Basic', MS='Amino acids with side chains that are positively charged at physiological pH.'),
  'D024362': NtO(MH='Advance Directive Adherence', MS='Compliance by health personnel or proxies with the stipulations of ADVANCE DIRECTIVES (or similar directives such as RESUSCITATION ORDERS) when patients are unable to direct their own care.'),
  'D024363': NtO(MH='Transcription Initiation Site', MS='The first nucleotide of a transcribed DNA sequence where RNA polymerase (DNA-DIRECTED RNA POLYMERASE) begins synthesizing the RNA transcript.'),
  'D024382': NtO(MH='Professional Role', MS='The expected function of a member of a particular profession.'),
  'D024401': NtO(MH='Carbonic Anhydrase I', MS='A cytosolic carbonic anhydrase isoenzyme primarily expressed in ERYTHROCYTES, vascular endothelial cells, and the gastrointestinal mucosa. EC 4.2.1.-'),
  'D024402': NtO(MH='Carbonic Anhydrase II', MS='A cytosolic carbonic anhydrase isoenzyme found widely distributed in cells of almost all tissues. Deficiencies of carbonic anhydrase II produce a syndrome characterized by OSTEOPETROSIS, renal tubular acidosis (ACIDOSIS, RENAL TUBULAR) and cerebral calcification. EC 4.2.1.-'),
  'D024403': NtO(MH='Carbonic Anhydrase III', MS='A cytosolic carbonic anhydrase isoenzyme primarily expressed in skeletal muscle (MUSCLES, SKELETAL). EC 4.2.1.-'),
  'D024404': NtO(MH='Carbonic Anhydrase V', MS='A carbonic anhydrase isoenzyme found in MITOCHONDRIA where it provides bicarbonate ions that are components in the urea cycle and in GLUCONEOGENESIS.'),
  'D024405': NtO(MH='Gastroepiploic Artery', MS='Abdominal artery that follows the curvature of the stomach. The right gastroepiploic artery is frequently used in CORONARY ARTERY BYPASS GRAFTING; MYOCARDIAL REVASCULARIZATION, and other vascular reconstruction.'),
  'D024422': NtO(MH='Aneugens', MS='Agents which affect CELL DIVISION and the MITOTIC SPINDLE APPARATUS resulting in the loss or gain of whole CHROMOSOMES, thereby inducing an ANEUPLOIDY.'),
  'D024442': NtO(MH='Aspartate Aminotransferase, Mitochondrial', MS='An aspartate aminotransferase found in MITOCHONDRIA.'),
  'D024443': NtO(MH='Aspartate Aminotransferase, Cytoplasmic', MS='An aspartate aminotransferase found in the CYTOPLASM.'),
  'D024461': NtO(MH='Myosin Type I', MS='A subclass of myosins found generally associated with actin-rich membrane structures such as filopodia. Members of the myosin type I family are ubiquitously expressed in eukaryotes. The heavy chains of myosin type I lack coiled-coil forming sequences in their tails and therefore do not dimerize.'),
  'D024462': NtO(MH='Myosin Type II', MS='The subfamily of myosin proteins that are commonly found in muscle fibers. Myosin II is also involved a diverse array of cellular functions including cell division, transport within the GOLGI APPARATUS, and maintaining MICROVILLI structure.'),
  'D024482': NtO(MH='Vitamin K 2', MS='A group of substances similar to VITAMIN K 1 which contains a ring of 2-methyl-1,4-naphthoquinione and an isoprenoid side chain of varying number of isoprene units. In vitamin K 2, each isoprene unit contains a double bond. They are produced by bacteria including the normal intestinal flora.'),
  'D024483': NtO(MH='Vitamin K 3', MS='A synthetic naphthoquinone without the isoprenoid side chain and biological activity, but can be converted to active vitamin K2, menaquinone, after alkylation in vivo.'),
  'D024501': NtO(MH='Nonmuscle Myosin Type IIB', MS='A nonmuscle isoform of myosin type II found predominantly in neuronal tissue.'),
  'D024502': NtO(MH='alpha-Tocopherol', MS='A natural tocopherol and one of the most potent antioxidant tocopherols. It exhibits antioxidant activity by virtue of the phenolic hydrogen on the 2H-1-benzopyran-6-ol nucleus. It has four methyl groups on the 6-chromanol nucleus. The natural d form of alpha-tocopherol is more active than its synthetic dl-alpha-tocopherol racemic mixture.'),
  'D024503': NtO(MH='beta-Tocopherol', MS='A natural tocopherol with less antioxidant activity than alpha-tocopherol. It exhibits antioxidant activity by virtue of the phenolic hydrogen on the 2H-1-benzopyran-6-ol nucleus. As in GAMMA-TOCOPHEROL, it also has three methyl groups on the 6-chromanol nucleus but at different sites.'),
  'D024504': NtO(MH='gamma-Tocopherol', MS='A natural tocopherol with less antioxidant activity than ALPHA-TOCOPHEROL. It exhibits antioxidant activity by virtue of the phenolic hydrogen on the 2H-1-benzopyran-6-ol nucleus. As in BETA-TOCOPHEROL, it also has three methyl groups on the 6-chromanol nucleus but at different sites.'),
  'D024505': NtO(MH='Tocopherols', MS='A collective name for a group of closely related lipids that contain substitutions on the 2H-1-benzopyran-6-ol nucleus and a long hydrocarbon chain of isoprenoid units. They are antioxidants by virtue of the phenolic hydrogen. Tocopherols react with the most reactive form of oxygen and protect unsaturated fatty acids from oxidation.'),
  'D024506': NtO(MH="5' Flanking Region", MS="The region of DNA which borders the 5' end of a transcription unit and where a variety of regulatory sequences are located."),
  'D024507': NtO(MH='Nonmuscle Myosin Type IIA', MS='A nonmuscle isoform of myosin type II found predominantly in platelets, lymphocytes, neutrophils and brush border enterocytes.'),
  'D024508': NtO(MH='Tocotrienols', MS='Natural analogs of TOCOPHEROLS exhibiting antioxidant activity. These tocol derivatives and isomers contain a benzopyran ring and an unsaturated isoprenoid side chain.'),
  'D024509': NtO(MH="3' Flanking Region", MS="The region of DNA which borders the 3' end of a transcription unit and where a variety of regulatory sequences are located."),
  'D024510': NtO(MH='Muscle Development', MS='Developmental events leading to the formation of adult muscular system, which includes differentiation of the various types of muscle cell precursors, migration of myoblasts, activation of myogenesis and development of muscle anchorage.'),
  'D024521': NtO(MH='Mushroom Bodies', MS='Prominent lobed neuropils found in ANNELIDA and all ARTHROPODS except crustaceans. They are thought to be involved in olfactory learning and memory.'),
  'D024522': NtO(MH='Genes, BRCA2', MS='A tumor suppressor gene (GENES, TUMOR SUPPRESSOR) located on human chromosome 13 at locus 13q12.3. Mutations in this gene predispose humans to breast and ovarian cancer. It encodes a large, nuclear protein that is an essential component of DNA repair pathways, suppressing the formation of gross chromosomal rearrangements. (from Genes Dev 2000;14(11):1400-6)'),
  'D024541': NtO(MH='Batrachoidiformes', MS='An order of bottom fishes with short, small, spinous dorsal fins. It is comprised of one family (Batrachoididae) and about 70 species.'),
  'D024561': NtO(MH='Hyperotreti', MS='A group of invertebrate chordates (CHORDATA, NONVERTEBRATE) in the subphylum Craniota. They include the single order Myxiniformes, which comprise various HAGFISHES.'),
  'D024581': NtO(MH='Glyceraldehyde-3-Phosphate Dehydrogenase (Phosphorylating)', MS='An NAD-dependent glyceraldehyde-3-phosphate dehydrogenase found in the cytosol of eucaryotes. It catalyses the dehydrogenation and phosphorylation of GLYCERALDEHYDE 3-PHOSPHATE to 3-phospho-D-glyceroyl phosphate, which is an important step in the GLYCOLYSIS pathway.'),
  'D024601': NtO(MH='Glyceraldehyde 3-Phosphate Dehydrogenase (NADP+)', MS='An NADP-dependent, non-phosphorylating aldehyde dehydrogenase that catalyzes the irreversible oxidation of GLYCERALDEHYDE 3-PHOSPHATE to 3-phosphoglycerate. It is found in the cytosol of a wide variety of organisms.'),
  'D024602': NtO(MH='Glyceraldehyde-3-Phosphate Dehydrogenase (NADP+)(Phosphorylating)', MS='A phosphorylating glyceraldehyde-3-phosphate dehydrogenase found in the CHLOROPLASTS of higher plants. This enzyme exists as an A2B2 heterotetramer and transfers hydrogen to NADP. It plays an important role in catalyzing the reductive step of the Calvin cycle.'),
  'D024642': NtO(MH='Potassium Channels, Voltage-Gated', MS='Potassium channel whose permeability to ions is extremely sensitive to the transmembrane potential difference. The opening of these channels is induced by the membrane depolarization of the ACTION POTENTIAL.'),
  'D024661': NtO(MH='Potassium Channels, Inwardly Rectifying', MS='Potassium channels where the flow of K+ ions into the cell is greater than the outward flow.'),
  'D024681': NtO(MH='Potassium Channels, Calcium-Activated', MS='Potassium channels whose activation is dependent on intracellular calcium concentrations.'),
  'D024682': NtO(MH='BRCA2 Protein', MS='A large, nuclear protein, encoded by the BRCA2 gene (GENE, BRCA2). Mutations in this gene predispose humans to breast and ovarian cancer. The BRCA2 protein is an essential component of DNA repair pathways, suppressing the formation of gross chromosomal rearrangements. (from Genes Dev. 2000;14(11):1400-6)'),
  'D024683': NtO(MH='Potassium Channels, Tandem Pore Domain', MS='Potassium channels that contain two pores in tandem. They are responsible for baseline or leak currents and may be the most numerous of all K channels.'),
  'D024701': NtO(MH='Myosin Type V', MS='A subclass of myosin involved in organelle transport and membrane targeting. It is abundantly found in nervous tissue and neurosecretory cells. The heavy chains of myosin V contain unusually long neck domains that are believed to aid in translocating molecules over large distances.'),
  'D024721': NtO(MH='E-Box Elements', MS='DNA locations with the consensus sequence CANNTG. ENHANCER ELEMENTS may contain multiple copies of this element. E-boxes play a regulatory role in the control of transcription. They bind with basic helix-loop-helix (bHLH) type TRANSCRIPTION FACTORS. Binding specificity is determined by the specific bHLH heterodimer or homodimer combination and by the specific nucleotides at the 3rd and 4th position of the E-box sequence.'),
  'D024722': NtO(MH='Cardiac Myosins', MS='Myosin type II isoforms found in cardiac muscle.'),
  'D024723': NtO(MH='Ventricular Myosins', MS='Isoforms of MYOSIN TYPE II, specifically found in the ventricular muscle of the HEART. Defects in the genes encoding ventricular myosins result in FAMILIAL HYPERTROPHIC CARDIOMYOPATHY.'),
  'D024741': NtO(MH='Cardiomyopathy, Hypertrophic, Familial', MS='An autosomal dominant inherited form of HYPERTROPHIC CARDIOMYOPATHY. It results from any of more than 50 mutations involving genes encoding contractile proteins such as VENTRICULAR MYOSINS; cardiac TROPONIN T; ALPHA-TROPOMYOSIN.'),
  'D024742': NtO(MH='HMG-Box Domains', MS='DNA-binding domains present in proteins of the HMG-box superfamily including the archetypal HMGB PROTEINS, a number of sequence specific TRANSCRIPTION FACTORS, and other DNA-BINDING PROTEINS. The domains consist of 70-80 amino acids that form an L-shaped fold from three alpha-helical segments. The domain has the capacity to recognize and/or induce specific DNA structures and effect the accessibility of the DNA to other proteins involved in transcription, recombination, or DNA repair. (Note that not all HIGH MOBILITY GROUP PROTEINS contain this domain.)'),
  'D024743': NtO(MH='Atrial Myosins', MS='Myosin type II isoforms specifically found in the atrial muscle of the heart.'),
  'D024744': NtO(MH='Skeletal Muscle Myosins', MS='Myosin type II isoforms found in skeletal muscle.'),
  'D024745': NtO(MH='Smooth Muscle Myosins', MS='Myosin type II isoforms found in smooth muscle.'),
  'D024761': NtO(MH='AT-Hook Motifs', MS='DNA-binding motifs, first described in one of the HMGA PROTEINS: HMG-I(Y) PROTEIN. They consist of positively charged sequences of nine amino acids centered on the invariant tripeptide glycine-arginine-proline. They act to fasten the protein to an AT RICH SEQUENCE in the DNA.'),
  'D024763': NtO(MH='Leeching', MS='The application of LEECHES to the body to draw blood for therapeutic purposes. Such medicinal leeching, an ancient medical practice, is still being used in microsurgery and the treatment of venous congestion or occlusion.'),
  'D024764': NtO(MH='Hirudin Therapy', MS='Use of HIRUDINS as an anticoagulant in the treatment of cardiological and hematological disorders.'),
  'D024801': NtO(MH='Tauopathies', MS='Neurodegenerative disorders involving deposition of abnormal tau protein isoforms (TAU PROTEINS) in neurons and glial cells in the brain. Pathological aggregations of tau proteins are associated with mutation of the tau gene on chromosome 17 in patients with ALZHEIMER DISEASE; DEMENTIA; PARKINSONIAN DISORDERS; progressive supranuclear palsy (SUPRANUCLEAR PALSY, PROGRESSIVE); and corticobasal degeneration.'),
  'D024802': NtO(MH="Nurse's Role", MS='The expected function of a member of the nursing profession.'),
  'D024803': NtO(MH='Biomedical Enhancement', MS='The use of technology-based interventions to improve functional capacities rather than to treat disease.'),
  'D024821': NtO(MH='Metabolic Syndrome', MS='A cluster of symptoms that are risk factors for CARDIOVASCULAR DISEASES and TYPE 2 DIABETES MELLITUS. The major components of metabolic syndrome include ABDOMINAL OBESITY; atherogenic DYSLIPIDEMIA; HYPERTENSION; HYPERGLYCEMIA; INSULIN RESISTANCE; a proinflammatory state; and a prothrombotic (THROMBOSIS) state.'),
  'D024841': NtO(MH='Fluoroquinolones', MS='A group of QUINOLONES with at least one fluorine atom and a piperazinyl group.'),
  'D024861': NtO(MH='Genetic Enhancement', MS='The use of genetic methodologies to improve functional capacities of an organism rather than to treat disease.'),
  'D024862': NtO(MH='Oxyphil Cells', MS='Large cells with small irregular nuclei and dense acidophilic granules due to the presence of abundant MITOCHONDRIA. Oxyphil cells, also known as oncocytes, are found in oncocytomas of the kidney, salivary glands, and endocrine glands. In the thyroid gland, oxyphil cells are known as Hurthle cells and Askenazy cells.'),
  'D024881': NtO(MH='Drug Resistance, Bacterial', MS='The ability of bacteria to resist or to become tolerant to chemotherapeutic agents, antimicrobial agents, or antibiotics. This resistance may be acquired through gene mutation or foreign DNA in transmissible plasmids (R FACTORS).'),
  'D024882': NtO(MH='Drug Resistance, Viral', MS='The ability of viruses to resist or to become tolerant to chemotherapeutic agents or antiviral agents. This resistance is acquired through gene mutation.'),
  'D024901': NtO(MH='Drug Resistance, Multiple, Bacterial', MS='The ability of bacteria to resist or to become tolerant to several structurally and functionally distinct drugs simultaneously. This resistance may be acquired through gene mutation or foreign DNA in transmissible plasmids (R FACTORS).'),
  'D024921': NtO(MH='Drug Resistance, Multiple, Viral', MS='The ability of viruses to resist or to become tolerant to several structurally and functionally distinct drugs simultaneously. This resistance phenotype may be attributed to multiple gene mutation.'),
  'D024941': NtO(MH='Starch Phosphorylase', MS='An enzyme of the PHOSPHORYLASES family that catalyzes the degradation of starch, a mixture of unbranched AMYLOSE and branched AMYLOPECTIN compounds. This phosphorylase from plants is the counterpart of GLYCOGEN PHOSPHORYLASE in animals that catalyzes the reaction of inorganic phosphate on the terminal alpha-1,4-glycosidic bond at the non-reducing end of glucans resulting in the release of glucose-1-phosphate.'),
  'D024981': NtO(MH='Glycogen Phosphorylase', MS='An enzyme that catalyzes the degradation of GLYCOGEN in animals by releasing glucose-1-phosphate from the terminal alpha-1,4-glycosidic bond. This enzyme exists in two forms: an active phosphorylated form ( PHOSPHORYLASE A) and an inactive un-phosphorylated form (PHOSPHORYLASE B). Both a and b forms of phosphorylase exist as homodimers. In mammals, the major isozymes of glycogen phosphorylase are found in muscle, liver and brain tissue.'),
  'D024982': NtO(MH='Glycogen Phosphorylase, Muscle Form', MS='An isoenzyme of GLYCOGEN PHOSPHORYLASE that catalyzes the degradation of GLYCOGEN in muscle. Mutation of the gene coding this enzyme is the cause of McArdle disease (GLYCOGEN STORAGE DISEASE TYPE V).'),
  'D025001': NtO(MH='Glycogen Phosphorylase, Liver Form', MS='An isoenzyme of GLYCOGEN PHOSPHORYLASE that catalyzes the degradation of GLYCOGEN in liver tissue. Mutation of the gene coding this enzyme on chromosome 14 is the cause of GLYCOGEN STORAGE DISEASE TYPE VI.'),
  'D025002': NtO(MH='Glycogen Phosphorylase, Brain Form', MS='An isoenzyme of GLYCOGEN PHOSPHORYLASE that catalyzes the degradation of GLYCOGEN in brain tissue.'),
  'D025021': NtO(MH='Cementogenesis', MS='The formation of DENTAL CEMENTUM, a bone-like material that covers the root of the tooth.'),
  'D025041': NtO(MH='Animal Migration', MS='Periodic movements of animals in response to seasonal changes or reproductive instinct. Hormonal changes are the trigger in at least some animals. Most migrations are made for reasons of climatic change, feeding, or breeding.'),
  'D025042': NtO(MH='Amacrine Cells', MS='INTERNEURONS of the vertebrate RETINA. They integrate, modulate, and interpose a temporal domain in the visual message presented to the RETINAL GANGLION CELLS, with which they synapse in the inner plexiform layer.'),
  'D025061': NtO(MH='Dinosaurs', MS='General name for two extinct orders of reptiles from the Mesozoic era: Saurischia and Ornithischia.'),
  'D025062': NtO(MH='Yarrowia', MS='A genus of ascomycetous yeast in the family Dipodascaceae, order SACCHAROMYCETALES.'),
  'D025063': NtO(MH='Chromosome Disorders', MS='Clinical conditions caused by an abnormal chromosome constitution in which there is extra or missing chromosome material (either a whole chromosome or a chromosome segment). (from Thompson et al., Genetics in Medicine, 5th ed, p429)'),
  'D025064': NtO(MH='Sex Chromosome Disorders', MS='Clinical conditions caused by an abnormal sex chromosome constitution (SEX CHROMOSOME ABERRATIONS), in which there is extra or missing sex chromosome material (either a whole chromosome or a chromosome segment).'),
  'D025065': NtO(MH='Borrelia burgdorferi', MS='A specific species of bacteria, part of the BORRELIA BURGDORFERI GROUP, whose common name is Lyme disease spirochete.'),
  'D025101': NtO(MH='Vitamin B 6', MS='VITAMIN B 6 refers to several PICOLINES (especially PYRIDOXINE; PYRIDOXAL; & PYRIDOXAMINE) that are efficiently converted by the body to PYRIDOXAL PHOSPHATE which is a coenzyme for synthesis of amino acids, neurotransmitters (serotonin, norepinephrine), sphingolipids, and aminolevulinic acid. During transamination of amino acids, pyridoxal phosphate is transiently converted into PYRIDOXAMINE phosphate. Although pyridoxine and Vitamin B 6 are still frequently used as synonyms, especially by medical researchers, this practice is erroneous and sometimes misleading (EE Snell; Ann NY Acad Sci, vol 585 pg 1, 1990). Most of vitamin B6 is eventually degraded to PYRIDOXIC ACID and excreted in the urine.'),
  'D025102': NtO(MH='Myosin Type III', MS='A subclass of myosins originally found in the photoreceptor of DROSOPHILA. The heavy chains can occur as two alternatively spliced isoforms of 132 and 174 KDa. The amino terminal of myosin type III is highly unusual in that it contains a protein kinase domain which may be an important component of the visual process.'),
  'D025121': NtO(MH='Myosin Type IV', MS='A subclass of myosin found in ACANTHAMOEBA. It is a non-filamentous myosin containing a single 180-kDa myosin heavy chain.'),
  'D025141': NtO(MH='Drug Resistance, Fungal', MS='The ability of fungi to resist or to become tolerant to chemotherapeutic agents, antifungal agents, or antibiotics. This resistance may be acquired through gene mutation.'),
  'D025181': NtO(MH='ATP Synthetase Complexes', MS='Multisubunit enzyme complexes that synthesize ADENOSINE TRIPHOSPHATE from energy sources such as ions traveling through channels.'),
  'D025202': NtO(MH='Molecular Diagnostic Techniques', MS='MOLECULAR BIOLOGY techniques used in the diagnosis of disease.'),
  'D025203': NtO(MH='Phosphofructokinase-1, Muscle Type', MS="An allosteric enzyme that regulates glycolysis by catalyzing the transfer of a phosphate group from ATP to fructose-6-phosphate to yield fructose-1,6-bisphosphate. In humans, PHOSPHOFRUCTOKINASE-1 in muscle exists as the homotetramer of M subunits. Defects in this muscle enzyme cause GLYCOGEN STORAGE DISEASE TYPE VII, also known as Tarui's disease."),
  'D025221': NtO(MH='Phosphofructokinases', MS='Allosteric enzymes that regulate glycolysis and gluconeogenesis. These enzymes catalyze phosphorylation of fructose-6-phosphate to either fructose-1,6-bisphosphate (PHOSPHOFRUCTOKINASE-1 reaction), or to fructose-2,6-bisphosphate (PHOSPHOFRUCTOKINASE-2 reaction).'),
  'D025222': NtO(MH='Chloroplast Proton-Translocating ATPases', MS='Proton-translocating ATPases which produce ADENOSINE TRIPHOSPHATE in plants. They derive energy from light-driven reactions that develop high concentrations of protons within the membranous cisternae (THYLAKOIDS) of the CHLOROPLASTS.'),
  'D025241': NtO(MH='Spondylarthritis', MS='Inflammation of the joints of the SPINE, the intervertebral articulations.'),
  'D025242': NtO(MH='Spondylarthropathies', MS='Heterogeneous group of arthritic diseases sharing clinical and radiologic features. They are associated with the HLA-B27 ANTIGEN and some with a triggering infection. Most involve the axial joints in the SPINE, particularly the SACROILIAC JOINT, but can also involve asymmetric peripheral joints. Subsets include ANKYLOSING SPONDYLITIS; REACTIVE ARTHRITIS; PSORIATIC ARTHRITIS; and others.'),
  'D025243': NtO(MH='Bacterial Proton-Translocating ATPases', MS='Membrane-bound proton-translocating ATPases that serve two important physiological functions in bacteria. One function is to generate ADENOSINE TRIPHOSPHATE by utilizing the energy provided by an electrochemical gradient of protons across the cellular membrane. A second function is to counteract a loss of the transmembrane ion gradient by pumping protons at the expense of adenosine triphosphate hydrolysis.'),
  'D025261': NtO(MH='Mitochondrial Proton-Translocating ATPases', MS='Proton-translocating ATPases responsible for ADENOSINE TRIPHOSPHATE synthesis in the MITOCHONDRIA. They derive energy from the respiratory chain-driven reactions that develop high concentrations of protons within the intermembranous space of the mitochondria.'),
  'D025262': NtO(MH='Vacuolar Proton-Translocating ATPases', MS='Proton-translocating ATPases that are involved in acidification of a variety of intracellular compartments.'),
  'D025282': NtO(MH='Mycelium', MS='The body of a fungus which is made up of HYPHAE.'),
  'D025301': NtO(MH='Hyphae', MS='Microscopic threadlike filaments in FUNGI that are filled with a layer of protoplasm. Collectively, the hyphae make up the MYCELIUM.'),
  'D025321': NtO(MH='Surgery, Computer-Assisted', MS='Surgical procedures conducted with the aid of computers. This is most frequently used in orthopedic and laparoscopic surgery for implant placement and instrument guidance. Image-guided surgery interactively combines prior CT scans or MRI images with real-time video.'),
  'D025341': NtO(MH='Principal Component Analysis', MS='Mathematical procedure that transforms a number of possibly correlated variables into a smaller number of uncorrelated variables called principal components.'),
  'D025361': NtO(MH='Streptogramins', MS='A class of natural cyclic peptide antibiotics produced by certain subspecies of STREPTOMYCES. They include two structurally unrelated components, STREPTOGRAMIN GROUP A and STREPTOGRAMIN GROUP B, which generally act synergistically to inhibit bacterial growth.'),
  'D025362': NtO(MH='Streptogramin Group A', MS='A structural subclass of streptogramins that includes molecules containing unsaturated macrocyclic lactones with approximate molecular weight of 500 KDa.'),
  'D025363': NtO(MH='Rhinomanometry', MS='Technique for measuring air pressure and the rate of airflow in the nasal cavity during respiration.'),
  'D025364': NtO(MH='Streptogramin A', MS='A specific streptogramin group A antibiotic produced by Streptomyces graminofaciens and other bacteria.'),
  'D025381': NtO(MH='Streptogramin B', MS='A specific streptogramin group B antibiotic produced by Streptomyces graminofaciens and other bacteria.'),
  'D025382': NtO(MH='Streptogramin Group B', MS='A structural subclass of streptogramins. They are cyclic hexadepsipeptides with approximate molecular weight of 800 that contain a characteristic 3-hydroxy picolinoyl exocyclic component.'),
  'D025401': NtO(MH='Echocardiography, Stress', MS='A method of recording heart motion and internal structures by combining ultrasonic imaging with exercise testing (EXERCISE TEST) or pharmacologic stress.'),
  'D025441': NtO(MH='Dihydroergocornine', MS="A 9,10alpha-dihydro derivative of ERGOTAMINE that contains isopropyl sidechains at the 2' and 5' positions of the molecule."),
  'D025442': NtO(MH='Dihydroergocristine', MS="A 9,10alpha-dihydro derivative of ERGOTAMINE that contains an isopropyl sidechain at the 2' position of the molecule."),
  'D025443': NtO(MH='Dihydroergocryptine', MS="A 9,10alpha-dihydro derivative of ERGOTAMINE that contains an isopropyl sidechain at the 2' position and an alpha-isobutyl sidechain at 5'alpha position of the molecule."),
  'D025461': NtO(MH='Feedback, Physiological', MS='A mechanism of communication with a physiological system for homeostasis, adaptation, etc. Physiological feedback is mediated through extensive feedback mechanisms that use physiological cues as feedback loop signals to control other systems.'),
  'D025481': NtO(MH='Phosphofructokinase-2', MS='An allosteric enzyme that regulates glycolysis and gluconeogenesis by catalyzing the transfer of a phosphate group from ATP to fructose-6-phosphate to yield fructose-2,6-bisphosphate, an allosteric effector for the other 6-phosphofructokinase, PHOSPHOFRUCTOKINASE-1. Phosphofructokinase-2 is bifunctional: the dephosphorylated form is a kinase and the phosphorylated form is a phosphatase that breaks down fructose-2,6-bisphosphate to yield fructose-6-phosphate.'),
  'D025501': NtO(MH='Phosphofructokinase-1, Liver Type', MS='An allosteric enzyme that regulates glycolysis by catalyzing the transfer of a phosphate group from ATP to fructose-6-phosphate to yield fructose-1,6-bisphosphate. In human liver, the 6-phosphofructose-1-kinase exists as the homotetramer of L subunits.'),
  'D025502': NtO(MH='Phosphofructokinase-1, Type C', MS='An allosteric enzyme that regulates glycolysis by catalyzing the transfer of a phosphate group from ATP to fructose-6-phosphate to yield fructose-1,6-bisphosphate. In the humans, 6-phosphofructose-1-kinase isozyme C is found in platelets, brain, heart, kidney, colon and testis. This isozyme C can exist as the homotetramer of C subunits (P subunits), or heterotetramer of C type and L type subunits.'),
  'D025521': NtO(MH='Tumor Suppressor Proteins', MS='Proteins that are normally involved in holding cellular growth in check. Deficiencies or abnormalities in these proteins may lead to unregulated cell growth and tumor development.'),
  'D025522': NtO(MH='Bacteriophage N4', MS='A species in the genus N4-like viruses, in the family PODOVIRIDAE, that infects E. coli.'),
  'D025523': NtO(MH='Bacteriophage HK022', MS='A tentative species in the genus lambda-like viruses, family SIPHOVIRIDAE.'),
  'D025541': NtO(MH='Bacteriochlorophyll A', MS='A specific bacteriochlorophyll that is similar in structure to chlorophyll a.'),
  'D025542': NtO(MH='Neurofibromin 1', MS='A protein found most abundantly in the nervous system. Defects or deficiencies in this protein are associated with NEUROFIBROMATOSIS 1, Watson syndrome, and LEOPARD syndrome. Mutations in the gene (GENE, NEUROFIBROMATOSIS 1) affect two known functions: regulation of ras-GTPase and tumor suppression.'),
  'D025543': NtO(MH='Bacteriophage IKe', MS='A species of filamentous phage in the genus INOVIRUS, family INOVIRIDAE. They are specific for enterobacteria that contain an IncN plasmid.'),
  'D025561': NtO(MH='Bacteriophage Pf1', MS='A species of filamentous Pseudomonas phage in the genus INOVIRUS, family INOVIRIDAE.'),
  'D025581': NtO(MH='Neurofibromin 2', MS='A membrane protein homologous to the ERM (Ezrin-Radixin-Moesin) family of cytoskeleton-associated proteins which regulate physical properties of membranes. Alterations in neurofibromin 2 are the cause of NEUROFIBROMATOSIS 2.'),
  'D025601': NtO(MH='Adenomatous Polyposis Coli Protein', MS='A negative regulator of beta-catenin signaling which is mutant in ADENOMATOUS POLYPOSIS COLI and GARDNER SYNDROME.'),
  'D025602': NtO(MH='Halorhodopsins', MS='Light driven chloride ion pumps that are ubiquitously found in halophilic archaea (HALOBACTERIALES).'),
  'D025603': NtO(MH='Sensory Rhodopsins', MS='Photosensory rhodopsins found in microorganisms such as HALOBACTERIA. They convert light signals into biochemical information that regulates certain cellular functions such as flagellar motor activity.'),
  'D025604': NtO(MH='Rhodopsins, Microbial', MS='Rhodopsin molecules found in microorganisms such as ARCHAEA and PROTEOBACTERIA.'),
  'D025622': NtO(MH='Bacteriophage PRD1', MS='Bacteriophage and type species in the genus Tectivirus, family TECTIVIRIDAE. They are specific for Gram-negative bacteria.'),
  'D025641': NtO(MH='Tobacco necrosis satellite virus', MS='An icosahedral RNA satellite virus which requires an obligatory Tobacco necrosis virus (see NECROVIRUS) for its replication.'),
  'D025662': NtO(MH='Adenoviruses, Porcine', MS='Species of the genus MASTADENOVIRUS, causing neurological disease in pigs.'),
  'D025681': NtO(MH='Fowl adenovirus A', MS='The type species of the genus AVIADENOVIRUS, family ADENOVIRIDAE, an oncogenic virus of birds. This is also called CELO virus for chick embryo lethal orphan virus.'),
  'D025682': NtO(MH='Nanovirus', MS='A genus in the family NANOVIRIDAE containing multiple circular single-stranded DNA molecules. The type species is Subterranean clover stunt virus.'),
  'D025701': NtO(MH='Asfarviridae', MS='A family of double-stranded DNA viruses containing one genus Asfivirus. It is the source of AFRICAN SWINE FEVER.'),
  'D025721': NtO(MH='WT1 Proteins', MS='Isoforms encoded by the WT1 Wilms tumor suppressor gene (GENES, WILMS TUMOR) and produced by alternative splicings. They are zinc finger-containing transcription factors involved in both transactivation and repression, and are critical for normal development and function of the urogenital tract.'),
  'D025722': NtO(MH='Granulovirus', MS='Granulovirus is the common name for Betabaculovirus, a genus of the family BACULOVIRIDAE, characterized by ovocylindrical occlusion bodies. The type species is Cydia pomonella granulovirus.'),
  'D025741': NtO(MH='HMGA1a Protein', MS='An 11-kDa AT-hook motif-containing (AT-HOOK MOTIFS) protein that binds to the minor grove of AT-rich regions of DNA. It is the full-length product of the alternatively-spliced HMGA1 gene and may function as an architectural chromatin binding protein that is involved in transcriptional regulation.'),
  'D025742': NtO(MH='HMGA1b Protein', MS='An AT-hook motif-containing protein (AT-HOOK MOTIFS) that binds to the minor grove of AT-rich regions of DNA. It is a truncated form of HMGA1a protein that is produced by alternative-splicing of the HMGA1 gene. It may function as an architectural chromatin binding protein that is involved in transcriptional regulation.'),
  'D025743': NtO(MH='HMGA2 Protein', MS='An AT-hook-containing (AT-HOOK MOTIFS) nuclear protein that is expressed predominantly in proliferating and undifferentiated mesenchymal cells.'),
  'D025761': NtO(MH='Mikamycin', MS='An antibiotic complex originally isolated from Streptomyces mitakaenis. It contains two principle ingredients: STREPTOGRAMIN A (mikamycin A) and STREPTOGRAMIN B (mikamycin B).'),
  'D025762': NtO(MH='Pristinamycin', MS='An antibiotic mixture originally isolated from Streptomyces pristinaspiralis. It is a mixture of compounds from STREPTOGRAMIN GROUP A: pristinamycin IIA and IIB and from STREPTOGRAMIN GROUP B: pristinamycin IA, pristinamycin IB, pristinamycin IC.'),
  'D025763': NtO(MH='Vernamycin B', MS='An antibiotic mixture originally isolated from Streptomyces loidensis. It contains a mixture of compounds belonging to STREPTOGRAMIN GROUP A and STREPTOGRAMIN GROUP B.'),
  'D025764': NtO(MH='HMGA1c Protein', MS='An AT-hook-containing (AT-HOOK MOTIFS) nuclear protein that may be involved in retinoid-dependent transcriptional activity.'),
  'D025765': NtO(MH='HMGB3 Protein', MS='An HMG-box domain (HMG-BOX DOMAINS) found highly expressed in embryonic tissue and in placenta.'),
  'D025781': NtO(MH='Hydroxymethylglutaryl-CoA Reductases, NAD-Dependent', MS='Specific hydroxymethylglutaryl CoA reductases that utilize the cofactor NAD. This class of enzymes performs a catabolic role in microorganisms such as Pseudomonas mevalonii where it oxidatively acetylates MEVALONIC ACID to form 3-HYDROXY-3-METHYLGLUTARYL-COENZYME A and NADH.'),
  'D025782': NtO(MH='Hydroxymethylglutaryl-CoA-Reductases, NADP-dependent', MS='Specific hydroxymethylglutaryl CoA reductases that utilize the cofactor NAD. In liver enzymes of this class are involved in cholesterol biosynthesis.'),
  'D025801': NtO(MH='Ubiquitin', MS='A highly conserved 76-amino acid peptide universally found in eukaryotic cells that functions as a marker for intracellular PROTEIN TRANSPORT and degradation. Ubiquitin becomes activated through a series of complicated steps and forms an isopeptide bond to lysine residues of specific proteins within the cell. These "ubiquitinated" proteins can be recognized and degraded by proteosomes or be transported to specific compartments within the cell.'),
  'D025802': NtO(MH='Electric Capacitance', MS='The ability of a substrate to retain an electrical charge.'),
  'D025803': NtO(MH='Tumor Suppressor Protein p14ARF', MS='A gene product of the p16 tumor suppressor gene (GENES, P16). It antagonizes the function of MDM2 PROTEIN (which regulates P53 TUMOR SUPPRESSOR PROTEIN by targeting it for degradation). p14ARF is produced from the beta mRNA transcript of the p16 gene. The other gene product, produced from the alternatively spliced alpha transcript, is CYCLIN-DEPENDENT KINASE INHIBITOR P16. Both p16 gene products have tumor suppressor functions.'),
  'D025821': NtO(MH='Polyubiquitin', MS='An oligomer formed from the repetitive linking of the C-terminal glycine of one UBIQUITIN molecule via an isopeptide bond to a lysine residue on a second ubiquitin molecule. It is structurally distinct from UBIQUITIN C, which is a single protein containing a tandemly arrayed ubiquitin peptide sequence.'),
  'D025822': NtO(MH='Ubiquitin C', MS='A single protein comprised of tandem repeats of the UBIQUITIN 78-amino acid sequence. It is a product of the polyubiquitin gene which contains multiple copies of the ubiquitin coding sequence. Proteolytic processing of ubiquitin C results in the formation of individual ubiquitin molecules. This protein is distinct from POLYUBIQUITIN, which is a protein formed through isopeptide linkage of multiple ubiquitin species.'),
  'D025841': NtO(MH='Small Ubiquitin-Related Modifier Proteins', MS='A class of structurally related proteins of 12-20 kDa in size. They covalently modify specific proteins in a manner analogous to UBIQUITIN.'),
  'D025842': NtO(MH='SUMO-1 Protein', MS='A 1.5-kDa small ubiquitin-related modifier protein that can covalently bind via an isopeptide link to a number of cellular proteins. It may play a role in intracellular protein transport and a number of other cellular processes.'),
  'D025861': NtO(MH='Blood Coagulation Disorders, Inherited', MS='Hemorrhagic and thrombotic disorders that occur as a consequence of inherited abnormalities in blood coagulation.'),
  'D025881': NtO(MH='Herpesvirus 5, Bovine', MS='A species of VARICELLOVIRUS that causes a fatal MENINGOENCEPHALITIS in calves.'),
  'D025901': NtO(MH='Carboxypeptidase B2', MS='A carboxypeptidase that removes C-terminal lysine or arginine from peptides and proteins. Carboxypeptidase B2 (CPB2) is released into the circulation as a proenzyme which is activated by the THROMBIN-THROMBOMODULIN complex. Activated CPB2 is involved in modulating a variety of processes by cleaving and inactivating various circulating proteins and peptides that are its substrates including FIBRIN; KININS; and ANAPHYLATOXINS.'),
  'D025921': NtO(MH='Iltovirus', MS='A genus of the family HERPESVIRIDAE, subfamily ALPHAHERPESVIRINAE, with a single species: HERPESVIRUS 1, GALLID.'),
  'D025922': NtO(MH='Mardivirus', MS='A genus in the family HERPESVIRIDAE, subfamily ALPHAHERPESVIRINAE, associated with malignancy in birds.'),
  'D025924': NtO(MH='Chromatography, Supercritical Fluid', MS='A CHROMATOGRAPHY method using supercritical fluid, usually carbon dioxide under very high pressure (around 73 atmospheres or 1070 psi at room temperature) as the mobile phase. Other solvents are sometimes added as modifiers. This is used both for analytical (SFC) and extraction (SFE) purposes.'),
  'D025941': NtO(MH='Protein Interaction Mapping', MS='Methods for determining interaction between PROTEINS.'),
  'D025942': NtO(MH='Oligosaccharides, Branched-Chain', MS='Oligosaccharides containing various types of glycosidic linkages that yield branching or antennae. The number of antennae (such as bi-, tri-, tetra-, or penta-antennary) in the oligosaccharides on the PROTEOGLYCANS; GLYCOPROTEINS; or LIPOPOLYSACCHARIDES contribute to their biological activities, such as receptor binding and metabolism.'),
  'D025961': NtO(MH='Herpesvirus 3, Gallid', MS="A species in the genus MAREK'S DISEASE-LIKE VIRUSES, in the family HERPESVIRIDAE, infecting chickens."),
  'D025962': NtO(MH='Septo-Optic Dysplasia', MS='A condition resulting from congenital malformations involving the brain. The syndrome of septo-optic dysplasia combines hypoplasia or agenesis of the SEPTUM PELLUCIDUM and the OPTIC NERVE. The extent of the abnormalities can vary. Septo-optic dysplasia is often associated with abnormalities of the hypothalamic and other diencephalic structures, and HYPOPITUITARISM.'),
  'D025981': NtO(MH='Hip Injuries', MS='General or unspecified injuries involving the hip.'),
  'D026002': NtO(MH='Herpesvirus 1, Meleagrid', MS="A species in the genus MAREK'S DISEASE-LIKE VIRUSES, in the family HERPESVIRIDAE, infecting turkeys."),
  'D026021': NtO(MH='Herpesvirus 4, Bovine', MS='A species in the genus RHADINOVIRUS, subfamily GAMMAHERPESVIRINAE, infecting cattle.'),
  'D026023': NtO(MH='Permethrin', MS='A pyrethroid insecticide commonly used in the treatment of LICE INFESTATIONS and SCABIES.'),
  'D026082': NtO(MH='Singlet Oxygen', MS='An excited state of molecular oxygen generated photochemically or chemically. Singlet oxygen reacts with a variety of biological molecules such as NUCLEIC ACIDS; PROTEINS; and LIPIDS; causing oxidative damages.'),
  'D026121': NtO(MH='Indole Alkaloids', MS='Group of alkaloids containing a benzylpyrrole group (derived from TRYPTOPHAN)'),
  'D026122': NtO(MH='Factor XIIIa', MS='Activated form of FACTOR XIII, a transglutaminase, which stabilizes the formation of the fibrin polymer (clot) culminating the blood coagulation cascade.'),
  'D026141': NtO(MH='Drug Resistance, Multiple, Fungal', MS='The ability of fungi to resist or to become tolerant to several structurally and functionally distinct drugs simultaneously. This resistance phenotype may be attributed to multiple gene mutations.'),
  'D026161': NtO(MH='MADS Domain Proteins', MS='A superfamily of proteins that share a highly conserved MADS domain sequence motif. The term MADS refers to the first four members which were MCM1 PROTEIN; AGAMOUS 1 PROTEIN; DEFICIENS PROTEIN; and SERUM RESPONSE FACTOR. Many MADS domain proteins have been found in species from all eukaryotic kingdoms. They play an important role in development, especially in plants where they have an important role in flower development.'),
  'D026201': NtO(MH='Musculoskeletal Manipulations', MS='Various manipulations of body tissues, muscles and bones by hands or equipment to improve health and circulation, relieve fatigue, promote healing.'),
  'D026241': NtO(MH='Exercise Movement Techniques', MS='Methods or programs of physical activities which can be used to promote, maintain, or restore the physical and physiological well-being of an individual.'),
  'D026261': NtO(MH='Diazepam Binding Inhibitor', MS='An 86-amino acid polypeptide, found in central and peripheral tissues, that displaces diazepam from the benzodiazepine recognition site on the gamma-aminobutyric acid receptor (RECEPTORS, GABA). It also binds medium- and long-chain acyl-CoA esters and serves as an acyl-CoA transporter. This peptide regulates lipid metabolism.'),
  'D026281': NtO(MH='DEFICIENS Protein', MS='DEFICIENS is a homeotic gene involved in the genetic control of Antirrhinum majus flower development. Its protein is one of the four founder proteins that structurally define the superfamily of MADS DOMAIN PROTEINS.'),
  'D026301': NtO(MH='Manipulation, Osteopathic', MS='Musculoskeletal manipulation based on the principles of OSTEOPATHIC MEDICINE developed in 1874 by Dr Andrew Taylor Still.'),
  'D026302': NtO(MH='Tai Ji', MS='One of the MARTIAL ARTS and also a form of meditative exercise using methodically slow circular stretching movements and positions of body balance.'),
  'D026321': NtO(MH='Piperaceae', MS='A family of flowering plants in the order Piperales best known for the black pepper widely used in SPICES, and for KAVA and Betel used for neuroactive properties.'),
  'D026323': NtO(MH='Myristica', MS='A plant genus in the MYRISTICACEAE family. The seed is used as a spice and used for antimicrobial and psychoactive effects. Myristicin, SAFROLE, and methyleugenol are key components.'),
  'D026324': NtO(MH='Myristicaceae', MS='A family of flowering plants in the order Magnoliales. Many of the species are tropical and have fragrant wood and leaves.'),
  'D026341': NtO(MH='Minichromosome Maintenance 1 Protein', MS='A sequence-specific DNA-binding protein that plays an essential role as a global regulator of yeast cell cycle control. It contains a 56 amino acid MADS-box domain within the N-terminal of the protein and is one of the four founder proteins that structurally define the superfamily of MADS DOMAIN PROTEINS.'),
  'D026342': NtO(MH='AGAMOUS Protein, Arabidopsis', MS='A plant homeotic protein involved in the development of stamens and carpels of Arabidopsis thaliana. It is a DNA-binding protein that contains the MADS-box domain. It is one of the four founder proteins that structurally define the superfamily of MADS DOMAIN PROTEINS.'),
  'D026361': NtO(MH='Reactive Nitrogen Species', MS='Nitrogenous products of NITRIC OXIDE synthases, ranging from NITRIC OXIDE to NITRATES. These reactive nitrogen intermediates also include the inorganic PEROXYNITROUS ACID and the organic S-NITROSOTHIOLS.'),
  'D026362': NtO(MH='Serum Response Factor', MS='A MADS domain-containing transcription factor that binds to the SERUM RESPONSE ELEMENT in the promoter-enhancer region of many genes. It is one of the four founder proteins that structurally define the superfamily of MADS DOMAIN PROTEINS.'),
  'D026403': NtO(MH='S-Nitrosothiols', MS='A group of organic sulfur-containing nitrites, alkyl thionitrites. S-Nitrosothiols include compounds such as S-NITROSO-N-ACETYLPENICILLAMINE and S-NITROSOGLUTATHIONE.'),
  'D026421': NtO(MH='Sensory Art Therapies', MS='Therapies using arts or directed at the senses.'),
  'D026422': NtO(MH='S-Nitrosoglutathione', MS='A sulfur-containing alkyl thionitrite that is one of the NITRIC OXIDE DONORS.'),
  'D026423': NtO(MH='S-Nitroso-N-Acetylpenicillamine', MS='A sulfur-containing alkyl thionitrite that is one of the NITRIC OXIDE DONORS.'),
  'D026441': NtO(MH='Mind-Body Therapies', MS='Treatment methods or techniques which are based on the knowledge of mind and body interactions. These techniques can be used to reduce the feeling of tension and effect of stress, and to enhance the physiological and psychological well-being of an individual.'),
  'D026443': NtO(MH='Spiritual Therapies', MS='Mystical, religious, or spiritual practices performed for health benefit.'),
  'D026461': NtO(MH='Ecdysteroids', MS='Steroids that bring about MOLTING or ecdysis in insects. Ecdysteroids include the endogenous insect hormones (ECDYSONE and ECDYSTERONE) and the insect-molting hormones found in plants, the phytoecdysteroids. Phytoecdysteroids are natural insecticides.'),
  'D026481': NtO(MH='Blotting, Far-Western', MS='A method that is derived from western blotting (BLOTTING, WESTERN) and is used to detect protein-protein interactions. The blotted proteins are probed with a non-antibody protein which can then be tagged with a labeled antibody.'),
  'D026501': NtO(MH='Blotting, Southwestern', MS='A method that is used to detect DNA-protein interactions. Proteins are separated by electrophoresis and blotted onto a nitrocellulose membrane similar to Western blotting (BLOTTING, WESTERN) but the proteins are identified when they bind labeled DNA PROBES (as with Southern blotting (BLOTTING, SOUTHERN)) instead of antibodies.'),
  'D026502': NtO(MH='LDL-Receptor Related Proteins', MS='A family of proteins that share sequence similarity with the low density lipoprotein receptor (RECEPTORS, LDL).'),
  'D026503': NtO(MH='Low Density Lipoprotein Receptor-Related Protein-1', MS='A LDL-receptor related protein involved in clearance of chylomicron remnants and of activated ALPHA-MACROGLOBULINS from plasma.'),
  'D026541': NtO(MH='Heymann Nephritis Antigenic Complex', MS='A complex of antigenic proteins obtained from the brush border of kidney tubules. It contains two principal components LOW DENSITY LIPOPROTEIN RECEPTOR-RELATED PROTEIN-2 and LDL-RECEPTOR RELATED PROTEIN-ASSOCIATED PROTEIN. The name of this complex is derived from researcher, Dr. Walter Heymann, who developed an experimental model of membranous glomerulonephritis (GLOMERULONEPHRITIS) by injecting this antigenic complex into rats to induce an autoimmune response.'),
  'D026561': NtO(MH='Low Density Lipoprotein Receptor-Related Protein-2', MS='An LDL-RECEPTOR RELATED PROTEIN found in the neuroepithelium and in proximal tubular cells of the kidney. It is considered a multiligand receptor in that it binds to a variety of ligands with relatively high affinity and may function in mediating the uptake and lysosomal degradation of macromolecules such as: LIPOPROTEINS; ENDOPEPTIDASES; and PROTEASE INHIBITORS.'),
  'D026563': NtO(MH='LDL-Receptor Related Protein-Associated Protein', MS='A membrane protein found in the rough endoplasm reticulum (ENDOPLASMIC RETICULUM, ROUGH) that binds to LDL-RECEPTOR RELATED PROTEINS. It may function to prevent ligand binding of receptors during protein processing events within endosomal compartments.'),
  'D026581': NtO(MH='Isoaspartic Acid', MS='An ASPARTIC ACID residue in polypeptide chains that is linked at the beta-carboxyl group instead of at the normal, alpha-carboxyl group, polypeptide linkage. It is a result of the spontaneous decomposition of aspartic acid or ASPARAGINE residues.'),
  'D026601': NtO(MH='Protein D-Aspartate-L-Isoaspartate Methyltransferase', MS='A PROTEIN O-METHYLTRANSFERASE that recognizes and catalyzes the methyl esterification of ISOASPARTIC ACID and D-ASPARTIC ACID residues in peptides and proteins. It initiates the repair of proteins damaged by the spontaneous decomposition of normal L-aspartic acid and L-asparagine residues.'),
  'D026602': NtO(MH='Nitrergic Neurons', MS='Nerve cells where transmission is mediated by NITRIC OXIDE.'),
  'D026603': NtO(MH='D-Aspartic Acid', MS='The D-isomer of ASPARTIC ACID.'),
  'D026661': NtO(MH='Clinical Trials Data Monitoring Committees', MS='Committees established to review interim data and efficacy outcomes in clinical trials. The findings of these committees are used in deciding whether a trial should be continued as designed, changed, or terminated. Government regulations regarding federally-funded research involving human subjects (the "Common Rule") require (45 CFR 46.111) that research ethics committees reviewing large-scale clinical trials monitor the data collected using a mechanism such as a data monitoring committee. FDA regulations (21 CFR 50.24) require that such committees be established to monitor studies conducted in emergency settings.'),
  'D026681': NtO(MH='Vitamin B 6 Deficiency', MS='A nutritional condition produced by a deficiency of VITAMIN B 6 in the diet, characterized by dermatitis, glossitis, cheilosis, and stomatitis. Marked deficiency causes irritability, weakness, depression, dizziness, peripheral neuropathy, and seizures. In infants and children typical manifestations are diarrhea, anemia, and seizures. Deficiency can be caused by certain medications, such as isoniazid.'),
  'D026683': NtO(MH='Advisory Committees', MS='Groups set up to advise governmental bodies, societies, or other institutions on policy. (Bioethics Thesaurus)'),
  'D026684': NtO(MH='Personal Autonomy', MS='Self-directing freedom and especially moral independence. An ethical principle holds that the autonomy of persons ought to be respected. (Bioethics Thesaurus)'),
  'D026686': NtO(MH='Beneficence', MS='The state or quality of being kind, charitable, or beneficial. (from American Heritage Dictionary of the English Language, 4th ed). The ethical principle of BENEFICENCE requires producing net benefit over harm. (Bioethics Thesaurus)'),
  'D026688': NtO(MH='Bioethical Issues', MS='Clusters of topics that fall within the domain of BIOETHICS, the field of study concerned with value questions that arise in biomedicine and health care delivery.'),
  'D026689': NtO(MH='Ethical Analysis', MS='The use of systematic methods of ethical examination, such as CASUISTRY or ETHICAL THEORY, in reasoning about moral problems.'),
  'D026690': NtO(MH='Ethics, Clinical', MS='The identification, analysis, and resolution of moral problems that arise in the care of patients. (Bioethics Thesaurus)'),
  'D026706': NtO(MH='Paternalism', MS="Interference with the FREEDOM or PERSONAL AUTONOMY of another person, with justifications referring to the promotion of the person's good or the prevention of harm to the person. (from Cambridge Dictionary of Philosophy, 1995); more generally, not allowing a person to make decisions on his or her own behalf."),
  'D026712': NtO(MH='Value of Life', MS='The intrinsic moral worth ascribed to a living being. (Bioethics Thesaurus)'),
  'D026721': NtO(MH="RNA 3' End Processing", MS="The steps that generate the 3' ends of mature RNA molecules. For most mRNAs (RNA, MESSENGER), 3' end processing referred to as POLYADENYLATION includes the addition of POLY A."),
  'D026723': NtO(MH='Polyadenylation', MS="The addition of a tail of polyadenylic acid (POLY A) to the 3' end of mRNA (RNA, MESSENGER). Polyadenylation involves recognizing the processing site signal, (AAUAAA), and cleaving of the mRNA to create a 3' OH terminal end to which poly A polymerase (POLYNUCLEOTIDE ADENYLYLTRANSFERASE) adds 60-200 adenylate residues. The 3' end processing of some messenger RNAs, such as histone mRNA, is carried out by a different process that does not include the addition of poly A as described here."),
  'D026724': NtO(MH='Ethical Review', MS='A formal process of examination of patient care or research proposals for conformity with ethical standards. The review is usually conducted by an organized clinical or research ethics committee (CLINICAL ETHICS COMMITTEES or RESEARCH ETHICS COMMITTEES), sometimes by a subset of such a committee, an ad hoc group, or an individual ethicist (ETHICISTS).'),
  'D026741': NtO(MH='Physical Therapy Modalities', MS='Therapeutic modalities frequently used in PHYSICAL THERAPY SPECIALTY by PHYSICAL THERAPISTS or physiotherapists to promote, maintain, or restore the physical and physiological well-being of an individual.'),
  'D026761': NtO(MH='Physical Therapy Specialty', MS='The auxiliary health profession by which PHYSICAL THERAPISTS make use of PHYSICAL THERAPY MODALITIES to prevent, correct, and alleviate movement dysfunction of anatomic or physiological origin.'),
  'D026762': NtO(MH='Maize streak virus', MS='The type species of genus Mastrevirus, family GEMINIVIRIDAE.'),
  'D026781': NtO(MH='Mandragora', MS='Plants of the Mandragora genus in the SOLANACEAE family, including Mandragora officinarum L. and Mandragora vernalis Bertol (sometimes called Atropa mandragora). They contain TROPANES that are anticholinergic (CHOLINERGIC ANTAGONISTS). Mandrake may also refer to PODOPHYLLUM.'),
  'D026801': NtO(MH='Synteny', MS='The presence of two or more genetic loci on the same chromosome. Extensions of this original definition refer to the similarity in content and organization between chromosomes, of different species for example.'),
  'D026822': NtO(MH='Ethicists', MS='Persons trained in philosophical or theological ethics who work in clinical, research, public policy, or other settings where they bring their expertise to bear on the analysis of ethical dilemmas in policies or cases. (Bioethics Thesaurus)'),
  'D026861': NtO(MH='Ornithodoros', MS='A genus of softbacked TICKS, in the family ARGASIDAE, serving as the vector of BORRELIA, causing RELAPSING FEVER, and of the AFRICAN SWINE FEVER VIRUS.'),
  'D026862': NtO(MH='Argasidae', MS='A family of softbacked TICKS, in the subclass ACARI. Genera include ARGAS and ORNITHODOROS among others.'),
  'D026863': NtO(MH='Ixodidae', MS='A family of hardbacked TICKS, in the subclass ACARI. Genera include DERMACENTOR and IXODES among others.'),
  'D026864': NtO(MH='Argas', MS='A genus of softbacked TICKS in the family ARGASIDAE. Most infect birds or bats but a few parasitize terrestrial mammals.'),
  'D026881': NtO(MH='Acupuncture', MS='The occupational discipline of the traditional Chinese methods of ACUPUNCTURE THERAPY for treating disease by inserting needles along specific pathways or meridians.'),
  'D026882': NtO(MH='Manipulation, Chiropractic', MS='Procedures used by chiropractors to treat neuromusculoskeletal complaints.'),
  'D026901': NtO(MH='Membrane Transport Proteins', MS='Membrane proteins whose primary function is to facilitate the transport of molecules across a biological membrane. Included in this broad category are proteins involved in active transport (BIOLOGICAL TRANSPORT, ACTIVE), facilitated transport and ION CHANNELS.'),
  'D026902': NtO(MH='Potassium Channel Blockers', MS='A class of drugs that act by inhibition of potassium efflux through cell membranes. Blockade of potassium channels prolongs the duration of ACTION POTENTIALS. They are used as ANTI-ARRHYTHMIA AGENTS and VASODILATOR AGENTS.'),
  'D026905': NtO(MH='Amino Acid Transport Systems', MS='Cellular proteins and protein complexes that transport amino acids across biological membranes.'),
  'D026921': NtO(MH='Amino Acid Transport Systems, Neutral', MS='Amino acid transporter systems capable of transporting neutral amino acids (AMINO ACIDS, NEUTRAL).'),
  'D026922': NtO(MH='Amino Acid Transport Systems, Acidic', MS='Amino acid transporter systems capable of transporting acidic amino acids (AMINO ACIDS, ACIDIC).'),
  'D026923': NtO(MH='Amino Acid Transport Systems, Basic', MS='Amino acid transporter systems capable of transporting basic amino acids (AMINO ACIDS, BASIC).'),
  'D026924': NtO(MH='Amino Acid Transport System A', MS='A sodium-dependent neutral amino acid transporter that accounts for most of the sodium-dependent neutral amino acid uptake by mammalian cells. The preferred substrates for this transporter system include ALANINE; SERINE; and GLUTAMINE.'),
  'D026941': NtO(MH='Sodium Channel Blockers', MS='A class of drugs that act by inhibition of sodium influx through cell membranes. Blockade of sodium channels slows the rate and amplitude of initial rapid depolarization, reduces cell excitability, and reduces conduction velocity.'),
  'D026942': NtO(MH='DNA Topoisomerases', MS='Enzymes that regulate the topology of DNA by actions such as breaking, relaxing, passing, and rejoining strands of DNA in cells. These enzymes are important components of the DNA replication system. They are classified by their substrate specificities. DNA TOPOISOMERASE I enzymes act on a single strand of DNA. DNA TOPOISOMERASE II enzymes act on double strands of DNA.'),
  'D026962': NtO(MH='Amino Acid Transport System ASC', MS='A ubiquitous sodium-dependent neutral amino acid transporter. The preferred substrates for this transporter system include ALANINE; SERINE; and CYSTEINE.'),
  'D026982': NtO(MH='Urtica dioica', MS='A plant species of the genus Urtica, family URTICACEAE. Roots have been used to treat PROSTATIC HYPERPLASIA. Leaves are edible after the stinging quality is eliminated by brief heating.'),
  'D027003': NtO(MH='Urticaceae', MS='The nettles plant family of the order ROSALES, superorder ROSANAE, and class MAGNOLIOPSIDA. Many have stinging hairs on stems and leaves with flowers that are small and greenish in leaf axils. The fruit is dry and one-seeded.'),
  'D027041': NtO(MH='Somatic Hypermutation, Immunoglobulin', MS='A programmed mutation process whereby changes are introduced to the nucleotide sequence of immunoglobulin gene DNA during development.'),
  'D027042': NtO(MH='Chromosomes, Artificial, P1 Bacteriophage', MS='DNA constructs that are derived from the DNA of BACTERIOPHAGE P1. They can carry large amounts (about 100-300 kilobases) of other sequence for a variety of bioengineering purposes.'),
  'D027062': NtO(MH='Amino Acid Transport System L', MS='A sodium-independent neutral amino acid transporter system with specificity for large amino acids. One of the functions of the transporter system is to supply large neutral amino acids to the brain.'),
  'D027081': NtO(MH='DNA Gyrase', MS='A bacterial DNA topoisomerase II that catalyzes ATP-dependent breakage of both strands of DNA, passage of the unbroken strands through the breaks, and rejoining of the broken strands. Gyrase binds to DNA as a heterotetramer consisting of two A and two B subunits. In the presence of ATP, gyrase is able to convert the relaxed circular DNA duplex into a superhelix. In the absence of ATP, supercoiled DNA is relaxed by DNA gyrase.'),
  'D027101': NtO(MH='DNA Topoisomerase IV', MS='A bacterial DNA topoisomerase II that catalyzes ATP-dependent breakage of both strands of DNA, passage of the unbroken strands through the breaks, and rejoining of the broken strands. Topoisomerase IV binds to DNA as a heterotetramer consisting 2 parC and 2 parE subunits. Topoisomerase IV is a decatenating enzyme that resolves interlinked daughter chromosomes following DNA replication.'),
  'D027161': NtO(MH='Satellite Cells, Perineuronal', MS='The non-neuronal cells that surround the neuronal cell bodies of the GANGLIA. They are distinguished from the perineuronal satellite oligodendrocytes (OLIGODENDROGLIA) found in the central nervous system.'),
  'D027182': NtO(MH='Amino Acid Transport System y+', MS='A sodium-independent, basic amino acid transport system with a high-affinity for LYSINE; ARGININE; and ORNITHINE.'),
  'D027201': NtO(MH='Cationic Amino Acid Transporter 1', MS='A high-affinity, low capacity system y+ amino acid transporter found ubiquitously. It has specificity for the transport of ARGININE; LYSINE; and ORNITHINE. It may also act as an ecotropic leukemia retroviral receptor.'),
  'D027202': NtO(MH='Cationic Amino Acid Transporter 2', MS='A high-affinity, low capacity system y+ amino acid transporter with strong similarity to CATIONIC AMINO ACID TRANSPORTER 1. The two isoforms of the protein, CAT-2A and CAT-2B, exist due to alternative mRNA splicing. The transporter has specificity for the transport of ARGININE; LYSINE; and ORNITHINE.'),
  'D027241': NtO(MH='Amino Acid Transport System y+L', MS='A leucine-sensitive amino acid transport system with high affinity for basic amino acids( AMINO ACIDS, BASIC).'),
  'D027261': NtO(MH='Fusion Regulatory Protein-1', MS='A heterodimeric protein that is a cell surface antigen associated with lymphocyte activation. The initial characterization of this protein revealed one identifiable heavy chain (FUSION REGULATORY PROTEIN 1, HEAVY CHAIN) and an indeterminate smaller light chain. It is now known that a variety of light chain subunits (FUSION REGULATORY PROTEIN 1, LIGHT CHAINS) can dimerize with the heavy chain. Depending upon its light chain composition a diverse array of functions can be found for this protein. Functions include: type L amino acid transport, type y+L amino acid transport and regulation of cellular fusion.'),
  'D027282': NtO(MH='Fusion Regulatory Protein 1, Heavy Chain', MS='A transmembrane glycoprotein subunit that can dimerize with a variety of light chain subunits such as SLC7A5; SLC7A6, and 4F2 LIGHT CHAINS. This protein subunit serves a diverse array of functions including amino acid transport and cell fusion. Its function is altered depending which of the light chain subunits it interacts with.'),
  'D027283': NtO(MH='Large Neutral Amino Acid-Transporter 1', MS='A CD98 antigen light chain that when heterodimerized with CD98 antigen heavy chain (FUSION REGULATORY PROTEIN-1 HEAVY CHAIN) forms a protein that mediates sodium-independent L-type amino acid transport.'),
  'D027301': NtO(MH='Fusion Regulatory Protein 1, Light Chains', MS='A family of light chains that bind to FUSION REGULATORY PROTEIN 1, HEAVY CHAIN to form a heterodimer. They convey functional specificity to the protein.'),
  'D027321': NtO(MH='Anion Transport Proteins', MS='Membrane proteins whose primary function is to facilitate the transport of negatively charged molecules (anions) across a biological membrane.'),
  'D027322': NtO(MH='Amino Acid Transport System X-AG', MS='A family of POTASSIUM and SODIUM-dependent acidic amino acid transporters that demonstrate a high affinity for GLUTAMIC ACID and ASPARTIC ACID. Several variants of this system are found in neuronal tissue.'),
  'D027341': NtO(MH='Excitatory Amino Acid Transporter 1', MS='A glial type glutamate plasma membrane transporter protein found predominately in ASTROCYTES. It is also expressed in HEART and SKELETAL MUSCLE and in the PLACENTA.'),
  'D027342': NtO(MH='Excitatory Amino Acid Transporter 2', MS='A glutamate plasma membrane transporter protein found in ASTROCYTES and in the LIVER.'),
  'D027343': NtO(MH='Rhizome', MS="Root-like underground horizontal stem of plants that produces shoots above and roots below. Distinguished from true roots which don't have buds and nodes. Similar to true roots in being underground and thickened by storage deposits."),
  'D027361': NtO(MH='Organic Anion Transporters', MS='Proteins involved in the transport of organic anions. They play an important role in the elimination of a variety of endogenous substances, xenobiotics and their metabolites from the body.'),
  'D027362': NtO(MH='Organic Anion Transport Protein 1', MS='A polyspecific transporter for organic cations found primarily in the kidney. It mediates the coupled exchange of alpha-ketoglutarate with organic ions such as P-AMINOHIPPURIC ACID.'),
  'D027381': NtO(MH='Liver-Specific Organic Anion Transporter 1', MS='An organic anion transporter found in human and mouse liver. It is capable of transporting a variety organic anions and mediates sodium-independent uptake of bile in the liver.'),
  'D027382': NtO(MH='Polyomaviridae', MS='A family of small, non-enveloped DNA viruses, infecting mainly MAMMALS.'),
  'D027383': NtO(MH='Papillomaviridae', MS='A family of small, non-enveloped DNA viruses infecting birds and most mammals, especially humans. They are grouped into multiple genera, but the viruses are highly host-species specific and tissue-restricted. They are commonly divided into hundreds of papillomavirus "types", each with specific gene function and gene control regions, despite sequence homology. Human papillomaviruses are found in the genera ALPHAPAPILLOMAVIRUS; BETAPAPILLOMAVIRUS; GAMMAPAPILLOMAVIRUS; and MUPAPILLOMAVIRUS.'),
  'D027384': NtO(MH='Oncogene Proteins v-raf', MS='A family of transforming proteins isolated from retroviruses such as MOUSE SARCOMA VIRUSES. They are viral-derived members of the raf-kinase family of serine-theonine kinases.'),
  'D027421': NtO(MH='Persea', MS='A plant genus in the LAURACEAE family. The tree, Persea americana Mill., is known for the Avocado fruit, the food of commerce.'),
  'D027422': NtO(MH='Umbellularia', MS='A plant genus in the LAURACEAE family. The tree, Umbellularia californica (Hook. & Arn.) Nutt., is known for aromatic leaves used in SPICES having a similar flavor to LAURUS.'),
  'D027423': NtO(MH='Cinnamomum camphora', MS='A tree, Cinnamomum camphora (L.) J. Presl, known as the source of CAMPHOR.'),
  'D027424': NtO(MH='Cinnamomum', MS='A plant genus in the LAURACEAE family. The bark of the trees is used in FOLK MEDICINE and FLAVORING AGENTS.'),
  'D027425': NtO(MH='Multidrug Resistance-Associated Proteins', MS='A sequence-related subfamily of ATP-BINDING CASSETTE TRANSPORTERS that actively transport organic substrates. Although considered organic anion transporters, a subset of proteins in this family have also been shown to convey drug resistance to neutral organic drugs. Their cellular function may have clinical significance for CHEMOTHERAPY in that they transport a variety of ANTINEOPLASTIC AGENTS. Overexpression of proteins in this class by NEOPLASMS is considered a possible mechanism in the development of multidrug resistance (DRUG RESISTANCE, MULTIPLE). Although similar in function to P-GLYCOPROTEINS, the proteins in this class share little sequence homology to the ATP BINDING CASSETTE TRANSPORTER, SUBFAMILY B, MEMBER 1 family of proteins.'),
  'D027441': NtO(MH='Sassafras', MS='A plant genus in the LAURACEAE family. The tree bark has been used medicinally. Drinks called sarsaparilla and root beer have been prepared from Sassafras and SMILAX ORNATA (Jamaican sarsaparilla).'),
  'D027442': NtO(MH='Laurus', MS='A plant genus in the LAURACEAE family. Laurus nobilis L. leaves are known for use in SPICES, having a similar flavor as UMBELLULARIA.'),
  'D027443': NtO(MH='Lindera', MS='A plant genus in the LAURACEAE family. Lindera benzoin is a source of a balsamic resin called benzoin which is up to 1/3 BENZOIC ACID. This should not be confused with the chemical BENZOIN or the plant STYRAX BENZOIDES.'),
  'D027444': NtO(MH='Ocotea', MS='A plant genus in the LAURACEAE family. The common name of stinkwood is also used for Zieria (RUTACEAE).'),
  'D027461': NtO(MH='Beta vulgaris', MS='A species of the Beta genus. Cultivars are used as a source of beets (root) or chard (leaves).'),
  'D027462': NtO(MH='Chenopodium', MS='A plant genus in the CHENOPODIACEAE family.'),
  'D027463': NtO(MH='Chenopodium album', MS='A plant species in the CHENOPODIUM genus known for edible greens.'),
  'D027464': NtO(MH='Chenopodium ambrosioides', MS='A plant species of the genus Chenopodium known for toxicity to intestinal worms and other simple organisms.'),
  'D027465': NtO(MH='Chenopodium quinoa', MS='A species of the Chenopodium genus which is the source of edible seed called quinoa. It contains makisterone A and other STEROIDS, some having ECDYSTEROID activity on insects.'),
  'D027466': NtO(MH='Atriplex', MS='A plant genus in the CHENOPODIACEAE family.'),
  'D027467': NtO(MH='Bassia scoparia', MS='A plant species of the family CHENOPODIACEAE. It is a source of furonaphthoquinones. The common name of burning bush is also used with the unrelated EUONYMUS sieboldianus.'),
  'D027468': NtO(MH='Crepis', MS='A plant genus in the ASTERACEAE family.'),
  'D027481': NtO(MH='Araliaceae', MS='The ginseng plant family of the order Apiales, subclass Rosidae, class Magnoliopsida. Leaves are generally alternate, large, and compound. Flowers are five-parted and arranged in compound flat-topped umbels. The fruit is a berry or (rarely) a drupe (a one-seeded fruit). It is well known for plant preparations used as adaptogens (immune support and anti-fatigue).'),
  'D027501': NtO(MH='Monocarboxylic Acid Transporters', MS='A family of proteins involved in the transport of monocarboxylic acids such as LACTIC ACID and PYRUVIC ACID across cellular membranes.'),
  'D027521': NtO(MH='Nepeta', MS='A genus of the LAMIACEAE family. It is known for its mild calming effect and for the way cats are attracted to the aroma.'),
  'D027522': NtO(MH='Marrubium', MS='A plant genus of the LAMIACEAE family that contains marrubiin (a labdane diterpene). It is known as a traditional medicinal for sore throat.'),
  'D027523': NtO(MH='Lavandula', MS='A plant genus of the LAMIACEAE family.'),
  'D027541': NtO(MH='Mentha', MS='Mentha is a genus of the mint family (LAMIACEAE). It is known for species having characteristic flavor and aroma.'),
  'D027542': NtO(MH='Rosmarinus', MS='A plant genus of the LAMIACEAE family. It is known as a spice and medicinal plant.'),
  'D027543': NtO(MH='Salvia officinalis', MS='A plant species of the Salvia genus known as a spice and medicinal plant.'),
  'D027544': NtO(MH='Salvia', MS='A genus in the mint family (LAMIACEAE).'),
  'D027561': NtO(MH='Salvia miltiorrhiza', MS='A plant species which is known as an Oriental traditional medicinal plant.'),
  'D027581': NtO(MH='Cannabaceae', MS='A plant family of the order Urticales, subclass Hamamelidae, class Magnoliopsida. It is most notable for the members, Cannabis and Hops.'),
  'D027582': NtO(MH='Humulus', MS='A plant genus in the CANNABACEAE family. Best known for the buds of Humulus lupulus L. used in BEER.'),
  'D027601': NtO(MH='Polyomavirus Infections', MS='Infections with POLYOMAVIRUS, which are often cultured from the urine of kidney transplant patients. Excretion of BK VIRUS is associated with ureteral strictures and CYSTITIS, and that of JC VIRUS with progressive multifocal leukoencephalopathy (LEUKOENCEPHALOPATHY, PROGRESSIVE MULTIFOCAL).'),
  'D027602': NtO(MH='Serum Response Element', MS='A DNA sequence that is found in the promoter region of many growth-related genes. The regulatory transcription factor SERUM RESPONSE FACTOR binds to and regulates the activity of genes containing this element.'),
  'D027621': NtO(MH='Iridaceae', MS='A plant family of the Asparagales order. Comprised of perennial herbs with BULBS, corms, or RHIZOMES and grass-like leaves with bisexual flowers that have 2 whorls of petaloid tepals which are usually showy.'),
  'D027622': NtO(MH='Crocus', MS='A plant genus, in the IRIDACEAE family, known as a source of Saffron.'),
  'D027623': NtO(MH='Yucca', MS='A genus (and common name) in the Asparagaceae family. It is known for SAPONINS in the root that are used in SOAPS.'),
  'D027641': NtO(MH='Laughter Therapy', MS='Therapeutic use of humor and laughter to improve emotional well being and health.'),
  'D027681': NtO(MH='Eclecticism, Historical', MS='A system of medicine, most popular in the 19th century, that advocates the use of indigenous plants in the treatment of specific signs and symptoms.'),
  'D027682': NtO(MH='Cation Transport Proteins', MS='Membrane proteins whose primary function is to facilitate the transport of positively charged molecules (cations) across a biological membrane.'),
  'D027701': NtO(MH='Organic Cation Transport Proteins', MS='A family of proteins involved in the transport of organic cations. They play an important role in the elimination of a variety of endogenous substances, xenobiotics, and their metabolites from the body.'),
  'D027702': NtO(MH='Organic Cation Transporter 1', MS='An organic cation transporter found in kidney. It is localized to the basal lateral membrane and is likely to be involved in the renal secretion of organic cations.'),
  'D027703': NtO(MH='Amaranthaceae', MS='A family of flowering plants in the order Caryophyllales, with about 60 genera and more than 800 species of plants, with a few shrubs, trees, and vines. The leaves usually have nonindented edges.'),
  'D027721': NtO(MH='Amaranthus', MS='A plant genus, in the family AMARANTHACEAE, best known as a source of high-protein grain crops and of Red Dye No. 2 (AMARANTH DYE). Tumbleweed sometimes refers to Amaranthus but more often refers to SALSOLA.'),
  'D027722': NtO(MH='Convolvulaceae', MS='The morning glory family of flowering plants, of the order Solanales, which includes about 50 genera and at least 1,400 species. Leaves are alternate and flowers are funnel-shaped. Most are twining and erect herbs, with a few woody vines, trees, and shrubs.'),
  'D027723': NtO(MH='Ipomoea batatas', MS='A plant species of the genus IPOMOEA, family CONVOLVULACEAE. Some cultivars are sweet and edible whereas bitter varieties are a source of SAPONINS. This sweet potato is sometimes referred to as a yam (DIOSCOREA).'),
  'D027724': NtO(MH='Reproductive Techniques, Assisted', MS='Clinical and laboratory techniques used to enhance fertility in humans and animals.'),
  'D027741': NtO(MH='Ipomoea', MS='A plant genus in the family CONVOLVULACEAE best known for morning glories (a common name also used with CONVOLVULUS) and sweet potato.'),
  'D027742': NtO(MH='Dioscorea', MS='A plant genus best known for edible underground tubers. Yam may also refer to a moist variety of sweet potato, IPOMOEA BATATAS.'),
  'D027743': NtO(MH='Dioscoreaceae', MS='The yam plant family, of the order Liliales, has thick roots or tubers and net-veined, heart-shaped leaves that sometimes are lobed.'),
  'D027744': NtO(MH='Tamus', MS='A plant genus of the DIOSCOREACEAE plant family.'),
  'D027761': NtO(MH='Asparagus Plant', MS='A plant genus in the family Asparagaceae that contains ECDYSTEROIDS and is an ingredient of Siotone. The shoots are used as a vegetable and the roots are used in FOLK MEDICINE.'),
  'D027762': NtO(MH='Lilium', MS='A plant genus in the family LILIACEAE generally growing in temperate areas. The word lily is also used in the common names of many plants of other genera that resemble true lilies. True lilies are erect perennial plants with leafy stems, scaly bulbs, usually narrow leaves, and solitary or clustered flowers.'),
  'D027781': NtO(MH='Shallots', MS='Mildly aromatic herb in the Allium genus used in SPICES.'),
  'D027782': NtO(MH='Chive', MS='A plant in the genus ALLIUM, similar to ONIONS.'),
  'D027801': NtO(MH='Galanthus', MS='A plant genus in the family  Amaryllidaceae. Galanthus nivalis L. is the source of GALANTHAMINE.'),
  'D027802': NtO(MH='Hypoxis', MS='Hypoxis is a plant genus in the family Hypoxidaceae (sometimes classified as LILIACEAE).'),
  'D027805': NtO(MH='Phaseolus', MS='A plant genus in the family FABACEAE which is the source of edible beans and the lectin PHYTOHEMAGGLUTININS.'),
  'D027806': NtO(MH='Lens Plant', MS='A plant genus of the FABACEAE family known for the seeds used as food.'),
  'D027807': NtO(MH='Lathyrus', MS='A plant genus in the family FABACEAE known for LATHYRISM poisoning.'),
  'D027821': NtO(MH='Rutaceae', MS='A plant family in the order Sapindales that grows in warmer regions and has conspicuous flowers.'),
  'D027822': NtO(MH='Myrtaceae', MS='The myrtle plant family of the order Myrtales. It includes several aromatic medicinal plants such as EUCALYPTUS.'),
  'D027823': NtO(MH='Rhamnaceae', MS='The buckthorn plant family, of the order Rhamnales, includes some species with edible fruits and some that are medicinal.'),
  'D027824': NtO(MH='Rosaceae', MS='The rose plant family in the order ROSALES and class Magnoliopsida. They are generally woody plants. A number of the species of this family contain cyanogenic compounds.'),
  'D027825': NtO(MH='Crataegus', MS='A genus in the family ROSACEAE of shrubs and small trees native to the North Temperate Zone. It is best known for a traditional medication for the heart.'),
  'D027826': NtO(MH='Rosa', MS='A plant genus in the family ROSACEAE and order Rosales. This should not be confused with the genus RHODIOLA which is sometimes called roseroot.'),
  'D027841': NtO(MH='Pimenta', MS='A plant genus in the family MYRTACEAE, order Myrtales, subclass Rosidae. It is best known for allspice from the dried berry of Pimenta dioica.'),
  'D027842': NtO(MH='Syzygium', MS='A plant genus in the family Myrtaceae. It is known for cloves, which is the flower bud of Syzygium aromaticum and is the source of an oil rich in EUGENOL.'),
  'D027843': NtO(MH='Vitis', MS='A plant genus in the family Vitaceae. It is a woody vine cultivated worldwide. It is best known for grapes, the edible fruit and used to make WINE and raisins.'),
  'D027844': NtO(MH='Vitaceae', MS='A plant family of the order Rhamnales, subclass Rosidae, class Magnoliopsida, best known for the VITIS genus, the source of grapes.'),
  'D027845': NtO(MH='Malus', MS='A plant genus in the family ROSACEAE, order Rosales, subclass Rosidae. It is best known as a source of the edible fruit (apple) and is cultivated in temperate climates worldwide.'),
  'D027861': NtO(MH='Prunus', MS='A plant genus in the family ROSACEAE, order Rosales, subclass Rosidae. It is best known as a source of edible fruits such as apricot, plum, peach, cherry, and almond.'),
  'D027882': NtO(MH='Agave', MS='A genus known for fibers obtained from their leaves: sisal from A. sisalana, henequen from A. fourcroyoides and A. cantala, or Manila-Maguey fiber from A. cantala. Some species provide a sap that is fermented to an intoxicating drink, called pulque in Mexico. Some contain agavesides.'),
  'D027883': NtO(MH='Astragalus Plant', MS='A plant genus in the family FABACEAE, subfamily Papilionaceae, order Fabales, subclass Rosidae. Many of the species are associated with poisoning of grazing animals. Some of the species are used medicinally.'),
  'D027884': NtO(MH='Astragalus gummifer', MS='Astragalus gummifer Labill. is the source of gum TRAGACANTH.'),
  'D027885': NtO(MH='Astragalus propinquus', MS='A plant species of the Astragalus genus which is source of Huang qi preparation used in TRADITIONAL CHINESE MEDICINE.'),
  'D027901': NtO(MH='Acanthaceae', MS='A plant family of the order Lamiales. It is characterized by simple leaves in opposite pairs, cystoliths (enlarged cells containing crystals of calcium carbonate), and bilaterally symmetrical and bisexual flowers that are usually crowded together. The common name for Ruellia of wild petunia is easily confused with PETUNIA.'),
  'D027902': NtO(MH='Justicia', MS='A plant genus in the family ACANTHACEAE. Adhatoda vasica Nees is a source of vasicine.'),
  'D027922': NtO(MH='Caprifoliaceae', MS='A plant family of the order Dipsacales, subclass Asteridae, class Magnoliopsida.'),
  'D027923': NtO(MH='Sambucus', MS='A plant genus in the family CAPRIFOLIACEAE known for elderberries.'),
  'D027924': NtO(MH='Sambucus nigra', MS='A plant species in the genus SAMBUCUS, known for the elderberry fruit. The plant is also a source of Sambucus nigra lectins and ribosome-inactivating protein.'),
  'D027925': NtO(MH='Viburnum', MS='A plant genus in the family CAPRIFOLIACEAE. The common name derives from its traditional use for menstrual cramps. It is a source of viburnine, valerianic acid, vibsanin, and ursolic acid. Note that true cranberry is VACCINIUM MACROCARPON.'),
  'D027926': NtO(MH='Anacardiaceae', MS='The sumac plant family in the order Sapindales, subclass Rosidae, class Magnoliopsida. They are tropical and subtropical trees, shrubs, and woody vines that have resin ducts in the bark. The sap of many of the species is irritating to the skin.'),
  'D027927': NtO(MH='Pistacia', MS='A plant genus in the ANACARDIACEAE family known for species that produce pistachio nuts (Pistacia vera) and GUM MASTIC (Pistacia lentiscus). Several of the species have the common name Pistachio.'),
  'D027961': NtO(MH='Potassium-Hydrogen Antiporters', MS='Membrane proteins that allow the exchange of hydrogen ions for potassium ions across the cellular membrane. The action of these antiporters influences intracellular pH and potassium ion homeostasis.'),
  'D027963': NtO(MH='Chloride-Bicarbonate Antiporters', MS='Electroneutral chloride bicarbonate exchangers that allow the exchange of BICARBONATE IONS exchange for CHLORIDE IONS across the cellular membrane. The action of specific antiporters in this class serve important functions such as allowing the efficient exchange of bicarbonate across red blood cell membranes as they passage through capillaries and the reabsorption of bicarbonate ions by the kidney.'),
  'D027981': NtO(MH='Symporters', MS='Membrane transporters that co-transport two or more dissimilar molecules in the same direction across a membrane. Usually the transport of one ion or molecule is against its electrochemical gradient and is "powered" by the movement of another ion or molecule with its electrochemical gradient.'),
  'D027982': NtO(MH='Sodium-Bicarbonate Symporters', MS='Proteins that cotransport sodium ions and bicarbonate ions across cellular membranes.'),
  'D027983': NtO(MH='Aralia', MS='A plant genus in the family Araliaceae.'),
  'D028001': NtO(MH='Eleutherococcus', MS='A plant genus in the family ARALIACEAE, order Apiales, subclass Rosidae. It is best known as an adaptogen and a substitute for PANAX GINSENG.'),
  'D028002': NtO(MH='Oplopanax', MS='A plant genus in the family ARALIACEAE, order Apiales, subclass Rosidae. It is the source of cirensenosides (triterpenoid saponins).'),
  'D028021': NtO(MH='Sodium-Potassium-Chloride Symporters', MS='A subclass of symporters that specifically transport SODIUM CHLORIDE and/or POTASSIUM CHLORIDE across cellular membranes in a tightly coupled process.'),
  'D028022': NtO(MH='Low-Level Light Therapy', MS='Treatment using irradiation with light of low power intensity so that the effects are a response to the light and not due to heat. A variety of light sources, especially low-power lasers are used.'),
  'D028023': NtO(MH='Arecaceae', MS='The palm family of order Arecales, subclass Arecidae, class Liliopsida.'),
  'D028024': NtO(MH='Serenoa', MS='A plant genus in the family ARECACEAE, order Arecales, subclass Arecidae. The fruit or the extract (Permixon) is used for PROSTATIC HYPERPLASIA.'),
  'D028041': NtO(MH='Centella', MS='A plant of the family APIACEAE which is the source of asiatic acid and asiaticoside. Centella asiatica (L.) Urb. = Hydrocotyle asiatica L. is known for effect on peripheral circulation.'),
  'D028042': NtO(MH='Pimpinella', MS='A plant genus in the family APIACEAE (Umbelliferae) that is used in SPICES and is a source of anethole.'),
  'D028044': NtO(MH='Phosphate-Binding Proteins', MS='Proteins that bind to and are involved in the metabolism of phosphate ions.'),
  'D028061': NtO(MH='Phosphate Transport Proteins', MS='Membrane proteins that are involved in the active transport of phosphate.'),
  'D028081': NtO(MH='Proton-Phosphate Symporters', MS='Proteins that cotransport hydrogen ions and phosphate ions across cellular membranes.'),
  'D028082': NtO(MH='Boraginaceae', MS='The Borage plant family is in the class Magnoliopsida, subclass Asteridae, order Lamiales. It is characterized by hairy foliage, usually alternate and simple; flowers are funnel-shaped or tubular. Some of the species contain PYRROLIZIDINE ALKALOIDS.'),
  'D028083': NtO(MH='Scrophulariaceae', MS='The figwort plant family of the order Lamiales. The family is characterized by bisexual flowers with tubular corollas (fused petals) that are bilaterally symmetrical (two-lips) and have four stamens in most, two of which are usually shorter.'),
  'D028084': NtO(MH='Clusiaceae', MS='The mangosteen plant family (sometimes classified as Guttiferae; also known as Hypericaceae) of the order THEALES, subclass Dilleniidae, class Magnoliopsida. It includes trees and shrubs with resinous, sticky sap, usually with broad-ended, oblong, leathery leaves with a strong, central vein, flowers with many stamens.'),
  'D028101': NtO(MH='Papaveraceae', MS='The poppy plant family of the order Papaverales, subclass Magnoliidae, class Magnoliopsida. These have bisexual, regular, cup-shaped flowers with one superior pistil and many stamens; 2 or 3 conspicuous, separate sepals and a number of separate petals. The fruit is a capsule. Leaves are usually deeply cut or divided into leaflets.'),
  'D028121': NtO(MH='Canarypox virus', MS='A species of AVIPOXVIRUS, subfamily CHORDOPOXVIRINAE. Canarypox virus vectors are used in vaccine and immunotherapy research.'),
  'D028141': NtO(MH='Berberidaceae', MS='The Barberry plant family of the order Ranunculales, subclass Magnoliidae, class Magnoliopsida. The shrubs have spiny leaves.'),
  'D028142': NtO(MH='Berberis', MS='A plant genus in the family BERBERIDACEAE. The common names of Barberry or Oregon Grape are also used for MAHONIA. The similar-named Bayberry is the unrelated MYRICA. Oregon Grape was classified by Pursh as a Berberis but Nuttall claimed it is different enough to call it a new genus, MAHONIA. Botanists insist on this name while horticulturists stay with Mahonia. They are shrubs with yellow wood and usually three-branched spines at the base of leafstalks. Flowers are yellow, six-petaled and fruit is a berry with one to several seeds. Members contain BERBERINE.'),
  'D028143': NtO(MH='Mahonia', MS='A plant genus in the family BERBERIDACEAE. Oregon Grape was discovered in North America and classified by Pursh as a BERBERIS but Nuttall claimed it is different enough to call it a new genus, MAHONIA. Now botanists insist on Berberis while horticulturists stay with this genus. It is a source of BERBERINE.'),
  'D028144': NtO(MH='Podophyllum peltatum', MS='A plant of the genus PODOPHYLLUM, family BERBERIDACEAE (sometimes classified as Podophyllaceae) which is the source of PODOPHYLLIN and of PODOPHYLLOTOXIN that inhibits DNA TOPOISOMERASES, TYPE II. It is different from the European mandrake, MANDRAGORA.'),
  'D028145': NtO(MH='Caulophyllum', MS='A plant genus of the family BERBERIDACEAE (sometimes classified as Leonticaceae), most notably Caulophyllum thalictroides (L.) Michx.'),
  'D028161': NtO(MH='Valerianaceae', MS='The Valerian plant family of the order Dipsacales, subclass Asteridae, class Magnoliopsida that is characterized by 3-5-lobed tubular flowers, often spurred at the base and clustered in tight heads.'),
  'D028162': NtO(MH='Heliotropium', MS='A plant genus in the family Boraginaceae, order Lamiales, subclass Asteridae. This is the True Heliotrope that should not be confused with an unrelated plant sometimes called Garden Heliotrope (VALERIAN).'),
  'D028163': NtO(MH='Viscaceae', MS='The Christmas Mistletoe plant family of the order Santalales, subclass Rosidae, class Magnoliopsida. They are parasitic primarily on coniferous trees forming a drooping evergreen bush of leathery leaves on tree branches. The berries are sticky and toxic.'),
  'D028181': NtO(MH='Viscum', MS='A plant genus in the family VISCACEAE, order Santalales, subclass Rosidae, class Magnoliopsida. Species of this genus contain cytotoxic LECTINS. The common name of MISTLETOE is used for many species of this and the LORANTHACEAE families.'),
  'D028182': NtO(MH='Viscum album', MS='A plant species of the family VISCACEAE, order Santalales, subclass Rosidae. This is the traditional mistletoe of literature and Christmas. Members contain viscotoxin (5 kDa basic polypeptides related to thionins), beta-galactoside- and N-acetyl-D-galactosamine-specific lectin II (60 kDa), and polysaccharides. Mistletoe lectin I is a type 2 ribosome-inactivating protein. Commercial extracts include Plenosol, Eurixor, Helixor Isorel, Iscador, and NSC 635089 (ANTINEOPLASTIC AGENTS, PHYTOGENIC).'),
  'D028183': NtO(MH='Loranthaceae', MS='The showy mistletoe plant family of the order Santalales, subclass Rosidae, class Magnoliopsida. This includes parasitic tropical plants with haustoria connecting to the hosts. The leaves are opposite and thick. The flowers (4-7) have both calyx and corolla. The fruit is a berry with one seed.'),
  'D028184': NtO(MH='Phoradendron', MS='A plant genus in the family VISCACEAE, order Santalales, subclass Rosidae. Members contain phoratoxin and oleanane TRITERPENES.'),
  'D028201': NtO(MH='Pinaceae', MS='A plant family of the order Pinales, class Pinopsida, division Tracheophyta, known for the various conifers.'),
  'D028202': NtO(MH='Abies', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta. Balm of Gilead is a common example.'),
  'D028203': NtO(MH='Cedrus', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta. It is the source of cedarwood oil. Cedar ordinarily refers to this but also forms part of the name of plants in other genera.'),
  'D028221': NtO(MH='Larix', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta.'),
  'D028222': NtO(MH='Picea', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta. They are evergreen, pyramidal trees with whorled branches and thin, scaly bark. Each of the linear, spirally arranged leaves are joined near the stem on a separate woody base.'),
  'D028223': NtO(MH='Pinus', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta. They are evergreen trees mainly in temperate climates.'),
  'D028224': NtO(MH='Pseudotsuga', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta. They are coniferous evergreen trees with long, flat, spirally arranged needles that grow directly from the branch.'),
  'D028225': NtO(MH='Tsuga', MS='A plant genus in the family PINACEAE, order Pinales, class Pinopsida, division Coniferophyta. They are coniferous evergreen trees and should not be confused with hemlock plants (CICUTA and CONIUM).'),
  'D028226': NtO(MH='Amyloidosis, Familial', MS='Diseases in which there is a familial pattern of AMYLOIDOSIS.'),
  'D028227': NtO(MH='Amyloid Neuropathies, Familial', MS='Inherited disorders of the peripheral nervous system associated with the deposition of AMYLOID in nerve tissue. The different clinical types based on symptoms correspond to the presence of a variety of mutations in several different proteins including transthyretin (PREALBUMIN); APOLIPOPROTEIN A-I; and GELSOLIN.'),
  'D028241': NtO(MH='Camellia sinensis', MS='Camellia sinensis L. (formerly Thea sinensis) is an evergreen Asiatic shrub of the THEACEAE family. The infusion of leaves of this plant is used as Oriental TEA which contains CAFFEINE; THEOPHYLLINE; and epigallocatechin gallate.'),
  'D028242': NtO(MH='Theaceae', MS='A plant family of the order THEALES, subclass Dilleniidae, class Magnoliopsida, best known for CAMELLIA SINENSIS, which is the source of Oriental TEA.'),
  'D028243': NtO(MH='Cerebral Amyloid Angiopathy, Familial', MS='A familial disorder marked by AMYLOID deposits in the walls of small and medium sized blood vessels of CEREBRAL CORTEX and MENINGES.'),
  'D028244': NtO(MH='Camellia', MS='A plant genus in the family THEACEAE, order ERICALES best known for CAMELLIA SINENSIS which is the source of Oriental TEA.'),
  'D028281': NtO(MH='Chelidonium', MS='A plant genus in the family PAPAVERACEAE, order Papaverales, subclass Magnoliidae.'),
  'D028301': NtO(MH='Feline Panleukopenia Virus', MS='A species of PARVOVIRUS infecting cats with a highly contagious enteric disease. Host range variants include mink enteritis virus, canine parvovirus (PARVOVIRUS, CANINE), and raccoon parvovirus. After infecting their new hosts, many of these viruses have further evolved and are now considered distinct species.'),
  'D028321': NtO(MH='Plant Preparations', MS='Material prepared from plants.'),
  'D028322': NtO(MH='Inhibin-beta Subunits', MS='They are glycopeptides and subunits in INHIBINS and ACTIVINS. Inhibins and activins belong to the transforming growth factor beta superfamily.'),
  'D028323': NtO(MH='Parvovirus, Porcine', MS='A species of PARVOVIRUS causing reproductive failure in pigs.'),
  'D028341': NtO(MH='Activins', MS='Activins are produced in the pituitary, gonads, and other tissues. By acting locally, they stimulate pituitary FSH secretion and have diverse effects on cell differentiation and embryonic development. Activins are glycoproteins that are hetero- or homodimers of INHIBIN-BETA SUBUNITS.'),
  'D028361': NtO(MH='Mitochondrial Diseases', MS='Diseases caused by abnormal function of the MITOCHONDRIA. They may be caused by mutations, acquired or inherited, in mitochondrial DNA or in nuclear genes that code for mitochondrial components. They may also be the result of acquired mitochondria dysfunction due to adverse effects of drugs, infections, or other environmental causes.'),
  'D028381': NtO(MH='Nidovirales', MS='An order comprising three families of eukaryotic viruses possessing linear, nonsegmented, positive sense RNA genomes. The families are Mesoniviridae; CORONAVIRIDAE; ARTERIVIRIDAE; and RONIVIRIDAE.'),
  'D028382': NtO(MH='Arteriviridae', MS='A family of viruses, of the order NIDOVIRALES, containing spherical virions. In contrast to CORONAVIRIDAE, no protruding spikes are obvious on the surface.'),
  'D028421': NtO(MH='Isoprostanes', MS='A series of prostaglandin-like compounds that are produced by the attack of free-radical species on unsaturated fatty acids, especially ARACHIDONIC ACID, of cellular MEMBRANES. Once cleaved from the lipid membrane by the action of phospholipases they can circulate into various bodily fluids and eventually be excreted. Although these compounds resemble enzymatically synthesized prostaglandins their stereoisometric arrangement is usually different than the "naturally occurring" compounds.'),
  'D028441': NtO(MH='F2-Isoprostanes', MS='Isoprostanes derived from the free radical oxidation of ARACHIDONIC ACID. Although similar in structure to enzymatically synthesized prostaglandin F2alpha (DINOPROST), they occur through non-enzymatic oxidation of cell membrane lipids.'),
  'D028461': NtO(MH='Zygophyllaceae', MS='A plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida which is a small family of small trees and shrubs growing in arid and warm regions.'),
  'D028462': NtO(MH='Cucumis', MS='A plant genus of the family CUCURBITACEAE, order Violales, subclass Dilleniidae best known for cucumber (CUCUMIS SATIVUS) and cantaloupe (CUCUMIS MELO). Watermelon is a different genus, CITRULLUS. Bitter melon may refer to MOMORDICA or this genus.'),
  'D028463': NtO(MH='Cucumis melo', MS='A plant species of the family CUCURBITACEAE, order Violales, subclass Dilleniidae known for the melon fruits with reticulated (net) surface including cantaloupes, honeydew, casaba, and Persian melons.'),
  'D028464': NtO(MH='Cucurbita', MS='A plant genus of the family CUCURBITACEAE, order Violales, subclass Dilleniidae, which includes pumpkin, gourd and squash.'),
  'D028481': NtO(MH='Euphorbia', MS='A large plant genus of the family EUPHORBIACEAE, order Euphorbiales, subclass Rosidae. They have a milky sap and a female flower consisting of a single pistil surrounded by numerous male flowers of one stamen each. Euphorbia hirta is rarely called milkweed but that name is normally used for ASCLEPIAS.'),
  'D028482': NtO(MH='Hevea', MS='A plant genus of the family EUPHORBIACEAE, order Euphorbiales, subclass Rosidae. Commercial natural RUBBER is mainly obtained from Hevea brasiliensis but also from some other plants.'),
  'D028483': NtO(MH='Abelmoschus', MS='A plant genus of the family MALVACEAE, order Malvales, subclass Dilleniidae, of which okra is best known example.'),
  'D028501': NtO(MH='Malva', MS="A plant genus of the family MALVACEAE, order Malvales, subclass Dilleniida. The common name of 'Mallow' may sometimes get confused with other plants."),
  'D028502': NtO(MH='Polygonum', MS='A plant genus of the family POLYGONACEAE that is an ingredient of Shou-Wu-Pian, a Chinese herbal preparation (DRUGS, CHINESE HERBAL). The common name of black bindweed also refers to TAMUS or Fallopia (use POLYGONACEAE).'),
  'D028503': NtO(MH='Cephaelis', MS='A plant genus of the family RUBIACEAE, order Rubiales, subclass Asteridae. Cephaelis ipecacuanha is the source of IPECAC.'),
  'D028521': NtO(MH='Musa', MS='A plant genus of the family Musaceae, order Zingiberales, subclass Zingiberidae, class Liliopsida.'),
  'D028522': NtO(MH='Musaceae', MS='A plant family of the order ZINGIBERALES, subclass Zingiberidae, class Liliopsida best known for banana (MUSA). The slender false trunk, formed by leaf sheaths of the spirally arranged leaves, may rise to 15 meters (50 feet). There is a crown of large leaves at the top.'),
  'D028523': NtO(MH='Elettaria', MS='A plant genus of the family ZINGIBERACEAE, order Zingiberales, subclass Zingiberidae. Elettaria cardamomum (L.) Maton is the source of Cardamom used in SPICES.'),
  'D028524': NtO(MH='Anethum graveolens', MS='Anethum graveolens L. is a plant species of the family APIACEAE. The leaves are considered as a spice (SPICES).'),
  'D028526': NtO(MH='Coriandrum', MS='A plant genus of the family APIACEAE. The leaves are the source of cilantro and the seeds are the source of coriander, both of which are used in SPICES.'),
  'D028527': NtO(MH='Cuminum', MS='A plant genus of the family APIACEAE. The seed is used in SPICES.'),
  'D028528': NtO(MH='Petroselinum', MS='A plant genus of the family APIACEAE used for flavoring food.'),
  'D028529': NtO(MH='Pastinaca', MS='A plant genus of the family APIACEAE. The roots are used as food.'),
  'D028561': NtO(MH='Transition Elements', MS='Elements with partially filled d orbitals. They constitute groups 3-12 of the periodic table of elements.'),
  'D028581': NtO(MH='Lanthanoid Series Elements', MS='Elements of the lanthanoid series including atomic number 57 (LANTHANUM) through atomic number 71 (LUTETIUM).'),
  'D028601': NtO(MH='Euthanasia, Active', MS='The act or practice of killing for reasons of mercy, i.e., in order to release a person or animal from incurable disease, intolerable suffering, or undignified death. (from Beauchamp and Walters, Contemporary Issues in Bioethics, 5th ed)'),
  'D028642': NtO(MH='Mentally Ill Persons', MS='Persons with psychiatric illnesses or diseases, particularly psychotic and severe mood disorders.'),
  'D028661': NtO(MH='Complicity', MS='Association with or participation in an act that is, or is perceived to be, criminal or immoral. One is complicitous when one promotes or unduly benefits from practices or institutions that are morally or legally suspect.'),
  'D028662': NtO(MH='Principle-Based Ethics', MS='An approach to ethics that focuses on theories of the importance of general principles such as respect for autonomy, beneficence/nonmaleficence, and justice.'),
  'D028663': NtO(MH='Ethical Theory', MS='A philosophically coherent set of propositions (for example, utilitarianism) which attempts to provide general norms for the guidance and evaluation of moral conduct. (from Beauchamp and Childress, Principles of Biomedical Ethics, 4th ed)'),
  'D028681': NtO(MH='Moral Obligations', MS='Duties that are based in ETHICS, rather than in law.'),
  'D028683': NtO(MH='National Socialism', MS='The doctrines and policies of the Nazis or the National Social German Workers party, which ruled Germany under Adolf Hitler from 1933-1945. These doctrines and policies included racist nationalism, expansionism, and state control of the economy. (from Columbia Encyclopedia, 6th ed. and American Heritage College Dictionary, 3d ed.)'),
  'D028701': NtO(MH='Patient Rights', MS="Fundamental claims of patients, as expressed in statutes, declarations, or generally accepted moral principles. (Bioethics Thesaurus) The term is used for discussions of patient rights as a group of many rights, as in a hospital's posting of a list of patient rights."),
  'D028722': NtO(MH='Virtues', MS='Character traits that are considered to be morally praiseworthy. (Bioethics Thesaurus)'),
  'D028723': NtO(MH='Personhood', MS='The state or condition of being a human individual accorded moral and/or legal rights. Criteria to be used to determine this status are subject to debate, and range from the requirement of simply being a human organism to such requirements as that the individual be self-aware and capable of rational thought and moral agency.'),
  'D028725': NtO(MH='Theology', MS='The study of religion and religious belief, or a particular system or school of religious beliefs and teachings (from online Cambridge Dictionary of American English, 2000 and WordNet: An Electronic Lexical Database, 1997)'),
  'D028741': NtO(MH='Ethics Committees, Clinical', MS='Hospital or other institutional ethics committees established to consider the ethical dimensions of patient care. Distinguish from ETHICS COMMITTEES, RESEARCH, which are established to monitor the welfare of patients or healthy volunteers participating in research studies.'),
  'D028743': NtO(MH='Professional Misconduct', MS='Violation of laws, regulations, or professional standards.'),
  'D028744': NtO(MH='Ethics Committees, Research', MS='Hospital or other institutional committees established to protect the welfare of research subjects. Federal regulations (the "Common Rule" (45 CFR 46)) mandate the use of these committees to monitor federally-funded biomedical and behavioral research involving human subjects.'),
  'D028761': NtO(MH='Withholding Treatment', MS="Withholding or withdrawal of a particular treatment or treatments, often (but not necessarily) life-prolonging treatment, from a patient or from a research subject as part of a research protocol. The concept is differentiated from REFUSAL TO TREAT, where the emphasis is on the health professional's or health facility's refusal to treat a patient or group of patients when the patient or the patient's representative requests treatment. Withholding of life-prolonging treatment is usually indexed only with EUTHANASIA, PASSIVE, unless the distinction between withholding and withdrawing treatment, or the issue of withholding palliative rather than curative treatment, is discussed."),
  'D028781': NtO(MH='Ethical Relativism', MS='The philosophical view that conceptions of truth and moral values are not absolute but are relative to the persons or groups holding them. (from American Heritage Dictionary of the English Language, 4th ed)'),
  'D028861': NtO(MH='Nuclear Pore Complex Proteins', MS='Proteins that form the structure of the NUCLEAR PORE. They are involved in active, facilitated and passive transport of molecules in and out of the CELL NUCLEUS.'),
  'D028884': NtO(MH='Karyopherins', MS='A family of proteins involved in NUCLEOCYTOPLASMIC TRANSPORT. Karyopherins are heteromeric molecules composed two major types of components, ALPHA KARYOPHERINS and BETA KARYOPHERINS, that function together to transport molecules through the NUCLEAR PORE COMPLEX. Several other proteins such as RAN GTP BINDING PROTEIN and CELLULAR APOPTOSIS SUSCEPTIBILITY PROTEIN bind to karyopherins and participate in the transport process.'),
  'D028901': NtO(MH='alpha Karyopherins', MS='Nucleocytoplasmic transport molecules that bind to the NUCLEAR LOCALIZATION SIGNALS of cytoplasmic molecules destined to be imported into the CELL NUCLEUS. Once attached to their cargo they bind to BETA KARYOPHERINS and are transported through the NUCLEAR PORE COMPLEX. Inside the CELL NUCLEUS alpha karyopherins dissociate from beta karypherins and their cargo. They then form a complex with CELLULAR APOPTOSIS SUSCEPTIBILITY PROTEIN and RAN GTP-BINDING PROTEIN which is exported to the CYTOPLASM.'),
  'D028921': NtO(MH='Biotinidase Deficiency', MS='The late onset form of MULTIPLE CARBOXYLASE DEFICIENCY (deficiency of the activities of biotin-dependent enzymes propionyl-CoA carboxylase, methylcrotonyl-CoA carboxylase, and PYRUVATE CARBOXYLASE) due to a defect or deficiency in biotinidase which is essential for recycling BIOTIN.'),
  'D028922': NtO(MH='Holocarboxylase Synthetase Deficiency', MS='The neonatal form of MULTIPLE CARBOXYLASE DEFICIENCY that is caused by a defect or deficiency in holocarboxylase synthetase. HLCS is the enzyme that covalently links biotin to the biotin dependent carboxylases (propionyl-CoA-carboxylase, pyruvate carboxylase, and beta-methylcrotonyl-CoA carboxylase).'),
  'D028941': NtO(MH='Coronavirus 229E, Human', MS='A species in the genus CORONAVIRUS causing the common cold and possibly nervous system infections in humans. It lacks hemagglutinin-esterase.'),
  'D028961': NtO(MH='beta Karyopherins', MS='Nucleocytoplasmic transport molecules that bind to ALPHA KARYOPHERINS in the CYTOSOL and are involved in transport of molecules through the NUCLEAR PORE COMPLEX. Once inside the CELL NUCLEUS beta karyopherins interact with RAN GTP-BINDING PROTEIN and dissociate from alpha karyopherins. Beta karyopherins bound to RAN GTP-BINDING PROTEIN are then re-transported to the cytoplasm where hydrolysis of the GTP of RAN GTP-BINDING PROTEIN causes release of karyopherin beta.'),
  'D028962': NtO(MH='Coronavirus OC43, Human', MS='A species in the genus CORONAVIRUS causing the common cold and possibly nervous system infections in humans. It contains hemagglutinin-esterase.'),
  'D028981': NtO(MH='Bornaviridae', MS='A family in the order MONONEGAVIRALES comprising one genus Bornavirus. This family has a unique form of mRNA processing: replication and transcription takes place in the nucleus.'),
  'D029001': NtO(MH='Herbal Medicine', MS='The study of medicines derived from botanical sources.'),
  'D029002': NtO(MH='Steam Bath', MS='Therapy of sitting in a hot steamy room followed by a cool bath or shower.'),
  'D029021': NtO(MH='Peste-des-Petits-Ruminants', MS='A highly fatal contagious disease of goats and sheep caused by PESTE-DES-PETITS-RUMINANTS VIRUS. The disease may be acute or subacute and is characterized by stomatitis, conjunctivitis, diarrhea, and pneumonia.'),
  'D029022': NtO(MH='Acoraceae', MS='A plant family of the order Arales, subclass Arecidae, class Liliopsida (monocot).'),
  'D029023': NtO(MH='Acorus', MS='A plant genus of the family ACORACEAE, order Arales, subclass Arecidae most notable for Acorus calamus L. root which contains asarone and has been used in TRADITIONAL MEDICINE.'),
  'D029024': NtO(MH='Marburgvirus', MS='A genus in the family FILOVIRIDAE consisting of one species (Lake Victoria marburgvirus) with several strains. The genus shows no antigenic cross-reactivity with EBOLAVIRUS.'),
  'D029041': NtO(MH='Actinidiaceae', MS='A plant family of the order Theales, subclass Dilleniidae, class Magnoliopsida. It is best known for Kiwi fruit (ACTINIDIA).'),
  'D029042': NtO(MH='Actinidia', MS='A plant species of the family ACTINIDIACEAE, order ERICALES.'),
  'D029043': NtO(MH='Ebolavirus', MS='A genus in the family FILOVIRIDAE consisting of several distinct species of Ebolavirus, each containing separate strains. These viruses cause outbreaks of a contagious, hemorrhagic disease (HEMORRHAGIC FEVER, EBOLA) in humans, usually with high mortality.'),
  'D029044': NtO(MH='Achyranthes', MS='A plant genus of the family AMARANTHACEAE, order Caryophyllales, which has been used in traditional medicine (MEDICINE, CHINESE TRADITIONAL).'),
  'D029045': NtO(MH='Rhus', MS='A plant genus of the family Anacardiaceae, order Sapindales, subclass Rosidae. It is a source of gallotannin (TANNIC ACID) and of somewhat edible fruit. Do not confuse with TOXICODENDRON which used to be part of this genus.'),
  'D029061': NtO(MH='Annonaceae', MS='The custard-apple plant family of the order Magnoliales, subclass Magnoliidae, class Magnoliopsida. Some members provide large pulpy fruits and commercial timber. Leaves and wood are often fragrant. Leaves are simple, with smooth margins, and alternately arranged in two rows along the stems.'),
  'D029062': NtO(MH='Apocynaceae', MS='The dogbane family of the order Gentianales.  Members of the family have milky, often poisonous juice, smooth-margined leaves, and flowers in clusters.'),
  'D029063': NtO(MH='Aquifoliaceae', MS='The holly plant family of the order Celastrales, subclass Rosidae, class Magnoliopsida.'),
  'D029064': NtO(MH='Araceae', MS='A plant family of the order ALISMATALES. Many members contain OXALIC ACID and calcium oxalate (OXALATES).'),
  'D029065': NtO(MH='Aristolochiaceae', MS='A plant family of the order Aristolochiales subclass Magnoliidae class Magnoliopsida. They are mostly tropical woody vines and a few temperate-zone species. The flowers are 3-parted; some species lack petals while others are large and foul smelling.'),
  'D029068': NtO(MH='Balsaminaceae', MS='A plant family of the order Geraniales (or Ericales in APG system), subclass Rosidae, class Magnoliopsida.'),
  'D029069': NtO(MH='Impatiens', MS='A plant genus of subsucculent annual or perennial plants in the family BALSAMINACEAE, order Geraniales.'),
  'D029082': NtO(MH='Sendai virus', MS='The type species of RESPIROVIRUS in the subfamily PARAMYXOVIRINAE. It is the murine version of HUMAN PARAINFLUENZA VIRUS 1, distinguished by host range.'),
  'D029101': NtO(MH='Parainfluenza Virus 3, Bovine', MS='A species of RESPIROVIRUS, subfamily PARAMYXOVIRINAE, most often seen in conjunction with a secondary infection of MANNHEIMIA HAEMOLYTICA resulting in pneumonic pasteurellosis (PASTEURELLOSIS, PNEUMONIC).'),
  'D029121': NtO(MH='Metapneumovirus', MS='A genus of the subfamily PNEUMOVIRINAE, containing two members: Turkey rhinotracheitis virus and a human Metapneumovirus. Virions lack HEMAGGLUTININ and NEURAMINIDASE.'),
  'D029122': NtO(MH='Murine pneumonia virus', MS='A species of the genus PNEUMOVIRUS causing pneumonia in mice.'),
  'D029141': NtO(MH='Novirhabdovirus', MS='A genus in the family RHABDOVIRIDAE, infecting numerous species of fish with broad geographic distribution. The type species is INFECTIOUS HEMATOPOIETIC NECROSIS VIRUS.'),
  'D029142': NtO(MH='Infectious hematopoietic necrosis virus', MS='The type species of NOVIRHABDOVIRUS, in the family RHABDOVIRIDAE. It is a major pathogen of TROUT and SALMON.'),
  'D029161': NtO(MH='Oleavirus', MS='A genus in the family BROMOVIRIDAE containing one species: Olive latent virus 2.'),
  'D029181': NtO(MH='Spirituality', MS="Sensitivity or attachment to religious values, or to things of the spirit as opposed to material or worldly interests. (from Merriam-Webster's Collegiate Dictionary, 10th ed, and Oxford English Dictionary, 2nd ed)"),
  'D029201': NtO(MH='Hamamelidaceae', MS='A plant family of the order Hamamelidales, subclass Hamamelidae, class Magnoliopsida.'),
  'D029221': NtO(MH='Faith Healing', MS='The use of faith and spirit to cure disease.'),
  'D029222': NtO(MH='Piper nigrum', MS='A plant species in the PIPERACEAE plant family. It is a common spice on foods and is used medicinally to increase gastrointestinal assimilation of other supplements and drugs. Piperine is a key component. Black pepper is picked unripe and heaped for a few days to ferment. White Pepper is the ripe fruit dehulled by maceration in water.'),
  'D029223': NtO(MH='Ascoviridae', MS='A family of insect viruses causing disease in lepidopterous larvae, most commonly from species of the owlet moth family Noctuidae.'),
  'D029241': NtO(MH='Optic Atrophy, Autosomal Dominant', MS='Dominant optic atrophy is a hereditary optic neuropathy causing decreased visual acuity, color vision deficits, a centrocecal scotoma, and optic nerve pallor (Hum. Genet. 1998; 102: 79-86). Mutations leading to this condition have been mapped to the OPA1 gene at chromosome 3q28-q29. OPA1 codes for a dynamin-related GTPase that localizes to mitochondria.'),
  'D029242': NtO(MH='Optic Atrophy, Hereditary, Leber', MS='A maternally linked genetic disorder that presents in mid-life as acute or subacute central vision loss leading to central scotoma and blindness. The disease has been associated with missense mutations in the mtDNA, in genes for Complex I, III, and IV polypeptides, that can act autonomously or in association with each other to cause the disease. (from Online Mendelian Inheritance in Man, http://www.ncbi.nlm.nih.gov/Omim/, MIM#535000 (April 17, 2001))'),
  'D029261': NtO(MH='Sin Nombre virus', MS='A species of HANTAVIRUS which emerged in the Four Corners area of the United States in 1993. It causes a serious, often fatal pulmonary illness (HANTAVIRUS PULMONARY SYNDROME) in humans. Transmission is by inhaling aerosolized rodent secretions that contain virus particles, carried especially by deer mice (PEROMYSCUS maniculatus) and pinyon mice (P. truei).'),
  'D029262': NtO(MH='Puumala virus', MS='A species of HANTAVIRUS causing nephropathia epidemica, a mild form of HEMORRHAGIC FEVER WITH RENAL SYNDROME. It is found in most of Europe and especially in Finland, along with its carrier rodent, the bank vole (Clethrionomys glareolus).'),
  'D029263': NtO(MH='Seoul virus', MS='A species of HANTAVIRUS causing a less severe form of HEMORRHAGIC FEVER WITH RENAL SYNDROME in Asia (primarily Korea and Japan). It is transmitted by rats, especially Rattus rattus and R. norvegicus.'),
  'D029281': NtO(MH='Nairobi sheep disease virus', MS='A species of NAIROVIRUS, transmitted by the ixodid ticks and producing a lethal gastroenteritis in sheep and goats. Though a major veterinary pathogen, its effect on humans has not been firmly established.'),
  'D029282': NtO(MH='Patient Education Handout', MS='Works consisting of a handout or self-contained informative material used to explain a procedure or a condition or the contents of a specific article in a biomedical journal and written in non-technical language for the patient or consumer.'),
  'D029301': NtO(MH='Sandfly fever Naples virus', MS='A species in the genus PHLEBOVIRUS causing PHLEBOTOMUS FEVER, an influenza-like illness. Related serotypes include Toscana virus and Tehran virus.'),
  'D029321': NtO(MH='Lagovirus', MS='A genus of the family CALICIVIRIDAE, associated with infections in rabbits and hares, responsible for epidemics with high mortality. RABBIT HEMORRHAGIC DISEASE VIRUS is the type species.'),
  'D029322': NtO(MH='Norovirus', MS='A genus in the family CALICIVIRIDAE, associated with epidemic GASTROENTERITIS in humans. The type species, NORWALK VIRUS, contains multiple strains.'),
  'D029341': NtO(MH='Sapovirus', MS='A genus of the family CALICIVIRIDAE associated with worldwide sporadic outbreaks of GASTROENTERITIS in humans. The first recorded outbreak was in human infants in Sapporo, Japan in 1977. The genus is comprised of a single species, Sapporo virus, containing multiple strains.'),
  'D029342': NtO(MH='Vesivirus', MS='A genus of the family CALICIVIRIDAE comprised of species infecting a wide range of organisms. Most members of this genus can be readily propagated in cell culture (as opposed to other genera of Caliciviridae). The type species is VESICULAR EXANTHEMA OF SWINE VIRUS.'),
  'D029362': NtO(MH='Organic Anion Transporters, Sodium-Independent', MS='A subclass of ORGANIC ANION TRANSPORTERS that do not rely directly or indirectly upon sodium ion gradients for the transport of organic ions.'),
  'D029363': NtO(MH='Organic Anion Transporters, Sodium-Dependent', MS='A subclass of ORGANIC ANION TRANSPORTERS whose transport of organic anions is driven either directly or indirectly by a gradient of sodium ions.'),
  'D029364': NtO(MH='Tenuivirus', MS='A genus of RNA plant viruses as yet unassigned to any family. Plant hosts are all in the family Poaceae. Each species is transmitted by a particular species of planthopper. The type species is Rice stripe virus.'),
  'D029365': NtO(MH='Organic Anion Transporters, ATP-Dependent', MS='A subclass of ORGANIC ANION TRANSPORTERS whose transport of organic anions is linked directly to the hydrolysis of ATP. The subclass includes those ATP-BINDING CASSETTE TRANSPORTERS that transport organic ions.'),
  'D029381': NtO(MH='Closteroviridae', MS='A family of plant viruses containing the largest single-stranded RNA genomes. Infections typically involve yellowing and necrosis, particularly affecting the phloem.'),
  'D029382': NtO(MH='Dicarboxylic Acid Transporters', MS='A family of organic anion transporters that specifically transport DICARBOXYLIC ACIDS such as alpha-ketoglutaric acid across cellular membranes.'),
  'D029383': NtO(MH='Crinivirus', MS='A genus of plant viruses in the family CLOSTEROVIRIDAE. Transmission is by whiteflies. Lettuce infectious yellows virus is the type species.'),
  'D029401': NtO(MH='Betulaceae', MS='A plant family of the order Fagales, subclass Hamamelidae, class Magnoliopsida. They have simple, serrate, alternate leaves. Male flowers are borne in long, pendulous catkins; the female in shorter, pendulous or erect catkins. The fruit is usually a small nut or a short-winged samara.'),
  'D029402': NtO(MH='Bignoniaceae', MS='A plant family of the order Lamiales. The family is characterized by oppositely paired, usually compound leaves and bell- or funnel-shaped, bisexual flowers having a five-lobed calyx and corolla.'),
  'D029403': NtO(MH='Burseraceae', MS='A plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida. They are resinous trees and shrubs with alternate leaves composed of many leaflets.'),
  'D029404': NtO(MH='Activin Receptors', MS="Receptors for ACTIVINS are membrane protein kinases belonging to the family of PROTEIN-SERINE-THREONINE KINASES, thus also named activin receptor-like kinases (ALK's). Activin receptors also bind TRANSFORMING GROWTH FACTOR BETA. As those transmembrane receptors of the TGF-beta superfamily (RECEPTORS, TRANSFORMING GROWTH FACTOR BETA), ALK's consist of two different but related protein kinases, Type I and Type II. Activins initiate cellular signal transduction by first binding to the type II receptors (ACTIVIN RECEPTORS, TYPE II ) which then recruit and phosphorylate the type I receptors (ACTIVIN RECEPTORS, TYPE I ) with subsequent activation of the type I kinase activity."),
  'D029405': NtO(MH='GB virus A', MS='A species of virus (unassigned to a genus) in the family FLAVIVIRIDAE, that have been identified in at least six species of New World monkeys. They do not cause HEPATITIS in the host or other susceptible species.'),
  'D029406': NtO(MH='GB virus B', MS='A species of virus (tentatively placed in the genus HEPACIVIRUS) in the family FLAVIVIRIDAE, that was recovered from a tamarin monkey, but may have been of human origin. It causes HEPATITIS in several species of New World monkeys.'),
  'D029407': NtO(MH='GB virus C', MS='A species of virus (unassigned to a genus) in the family FLAVIVIRIDAE. It is genetically heterogeneous, of human origin, and transmitted by blood or blood products. Despite its alternate name (Hepatitis G virus), its pathogenicity remains controversial.'),
  'D029421': NtO(MH='Cactaceae', MS='The cactus plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida. Cacti are succulent perennial plants well adapted to dry regions.'),
  'D029422': NtO(MH='Campanulaceae', MS='A plant family of the order Campanulales, subclass Asteridae, class Magnoliopsida'),
  'D029424': NtO(MH='Pulmonary Disease, Chronic Obstructive', MS='A disease of chronic diffuse irreversible airflow obstruction. Subcategories of COPD include CHRONIC BRONCHITIS and PULMONARY EMPHYSEMA.'),
  'D029441': NtO(MH='Carica', MS='A plant genus of the family Caricaceae, order Violales, subclass Dilleniidae, class Magnoliopsida. It is the source of edible fruit and PAPAIN.'),
  'D029461': NtO(MH='Sialic Acid Storage Disease', MS='Autosomal recessive neurodegenerative disorders caused by lysosomal membrane transport defects that result in accumulation of free sialic acid (N-ACETYLNEURAMINIC ACID) within the lysosomes. The two main clinical phenotypes, which are allelic variants of the SLC17A5 gene, are ISSD, a severe infantile form, or Salla disease, a slowly progressive adult form, named for the geographic area in Finland where the kindred first studied resided.'),
  'D029481': NtO(MH='Bronchitis, Chronic', MS='A subcategory of CHRONIC OBSTRUCTIVE PULMONARY DISEASE. The disease is characterized by hypersecretion of mucus accompanied by a chronic (more than 3 months in 2 consecutive years) productive cough. Infectious agents are a major cause of chronic bronchitis.'),
  'D029482': NtO(MH='Parainfluenza Virus 4, Human', MS='A species of RUBULAVIRUS causing endemic upper respiratory infections in children. It produces only a mild clinical disease and often goes undetected.'),
  'D029501': NtO(MH='Hamamelis', MS='A plant genus of the family Hamamelidaceae, order Hamamelidales. It is known for the astringent (ASTRINGENTS) extract of leaves and bark used in over the counter preparations. This should not be confused with the similar named Hamelia genus (RUBIACEAE).'),
  'D029502': NtO(MH='Anemia, Hypoplastic, Congenital', MS='An inborn condition characterized by deficiencies of red cell precursors that sometimes also includes LEUKOPENIA and THROMBOCYTOPENIA.'),
  'D029503': NtO(MH='Anemia, Diamond-Blackfan', MS='A rare congenital hypoplastic anemia that usually presents early in infancy. The disease is characterized by a moderate to severe macrocytic anemia, occasional neutropenia or thrombocytosis, a normocellular bone marrow with erythroid hypoplasia, and an increased risk of developing leukemia. (Curr Opin Hematol 2000 Mar;7(2):85-94)'),
  'D029521': NtO(MH='Influenzavirus A', MS='A genus in the family ORTHOMYXOVIRIDAE causing influenza and other diseases in humans and animals. It contains many strains as well as antigenic subtypes of the integral membrane proteins hemagglutinin (HEMAGGLUTININS) and NEURAMINIDASE. The type species is INFLUENZA A VIRUS.'),
  'D029522': NtO(MH='Hydrophyllaceae', MS='The waterleaf plant family of the order Solanales, subclass Asteridae, class Magnoliopsida. They have alternate leaves, regular flowers with five-lobed corollas (united petals), and a one- or two-celled ovary.'),
  'D029523': NtO(MH='Illicium', MS='A plant genus of the family SCHISANDRACEAE, order AUSTROBAILEYALES, class MAGNOLIOPSIDA. They have evergreen, aromatic leaves and bisexual flowers. The female portion of the flower consists of 7 to 15 carpels.'),
  'D029524': NtO(MH='Influenzavirus B', MS='A genus of the family ORTHOMYXOVIRUS causing HUMAN INFLUENZA and other diseases primarily in humans. In contrast to INFLUENZAVIRUS A, no distinct antigenic subtypes of hemagglutinin (HEMAGGLUTININS) and NEURAMINIDASE are recognized.'),
  'D029525': NtO(MH='Juglandaceae', MS='The walnut plant family of the order Juglandales, subclass Hamamelidae, class Magnoliopsida. They are mainly temperate zone trees.'),
  'D029541': NtO(MH='Loganiaceae', MS='A plant family of the order Gentianales, subclass Asteridae, class Magnoliopsida. They have leaflike appendages at the base of the leafstalks, have terminal flower clusters. Petals have four or five overlapping lobes and the fruit is a capsule containing winged or wingless seeds.'),
  'D029542': NtO(MH='Lycopodiaceae', MS='The club-moss plant family of the order Lycopodiales, class Lycopodiopsida, division Lycopodiophyta, subkingdom TRACHEOPHYTA. The common name of clubmoss applies to several genera of this family. Despite the name this is not one of the true mosses (BRYOPSIDA ).'),
  'D029543': NtO(MH='Nucleocytoplasmic Transport Proteins', MS='Proteins involved in the process of transporting molecules in and out the cell nucleus. Included here are: NUCLEOPORINS, which are membrane proteins that form the NUCLEAR PORE COMPLEX; KARYOPHERINS, which carry molecules through the nuclear pore complex; and proteins that play a direct role in the transport of karyopherin complexes through the nuclear pore complex.'),
  'D029544': NtO(MH='Foot-and-Mouth Disease Virus', MS='The type species of APHTHOVIRUS, causing FOOT-AND-MOUTH DISEASE in cloven-hoofed animals. Several different serotypes exist.'),
  'D029561': NtO(MH='Lythraceae', MS='The loosestrife plant family of the order Myrtales, subclass Rosidae, class Magnoliopsida. Members are mainly herbs and many of them contain ALKALOIDS.'),
  'D029562': NtO(MH='Magnoliaceae', MS='A plant family of the order Magnoliales, subclass Magnoliidae, class Magnoliopsida. They are trees and shrubs having an elongated conelike floral axis with fragrant flowers that have six tepals (sepals and petals that are not distinctly different) and many spirally arranged stamens.'),
  'D029563': NtO(MH='Cellular Apoptosis Susceptibility Protein', MS='A nucleocytoplasmic transport protein that binds to ALPHA KARYOPHERINS and RAN GTP BINDING PROTEIN inside the CELL NUCLEUS and participates in their export into CYTOPLASM. It is also associated with the regulation of APOPTOSIS and microtubule assembly.'),
  'D029581': NtO(MH='Malpighiaceae', MS='A plant family of the order Polygalales, subclass Rosidae class, Magnoliopsida that are mostly shrubs and small trees. Many of the members contain indole alkaloids.'),
  'D029582': NtO(MH='Meliaceae', MS='The mahogany plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida.'),
  'D029583': NtO(MH='Menispermaceae', MS='A plant family of the order Ranunculales, subclass Magnoliidae, class Magnoliopsida. Members are mostly vines and shrubs and they contain isoquinoline alkaloids, some of which have been used as arrow poisons.'),
  'D029584': NtO(MH='Prostatic Secretory Proteins', MS='Proteins secreted by the prostate gland. The major secretory proteins from the human prostate gland include PROSTATE-SPECIFIC ANTIGEN, prostate-specific acid phosphatase, prostate-specific membrane antigen, and prostate-specific protein-94.'),
  'D029585': NtO(MH='Monimiaceae', MS='A plant family of the order Laurales, subclass Magnoliidae, class Magnoliopsida.'),
  'D029586': NtO(MH='Moraceae', MS='The mulberry plant family of the order Urticales, subclass Hamamelidae, class Magnoliopsida. They have milky latex and small, petalless male or female flowers.'),
  'D029587': NtO(MH='Moringa', MS='A plant genus of the family Moringaceae, order Capparales, subclass Dilleniidae, class Magnoliopsida.'),
  'D029588': NtO(MH='Moringa oleifera', MS='A plant species of the family Moringaceae, order Capparales, subclass Dilleniidae. It is a source of niaziminin and hypotensive thiocarbamate glycosides.'),
  'D029589': NtO(MH='Seminal Plasma Proteins', MS='Proteins found in SEMEN. Major seminal plasma proteins are secretory proteins from the male sex accessory glands, such as the SEMINAL VESICLES and the PROSTATE. They include the seminal vesicle-specific antigen, an ejaculate clotting protein; and the PROSTATE-SPECIFIC ANTIGEN, a protease and an esterase.'),
  'D029590': NtO(MH='Myricaceae', MS='A plant family of the order Myricales, subclass Hamamelidae, class Magnoliopsida. They are trees and shrubs having aromatic leaves that often have yellow glandular dots on the surface. Single-seeded fruits are often covered with waxy granules, bumps, or layers. The flowers are small, greenish, and inconspicuous.'),
  'D029592': NtO(MH='Oleaceae', MS='A plant family of the order Lamiales. The leaves are usually opposite and the flowers usually have four sepals, four petals, two stamens, and two fused carpels that form a single superior ovary.'),
  'D029593': NtO(MH='Jervell-Lange Nielsen Syndrome', MS='A form of long QT syndrome that is associated with congenital deafness. It is characterized by abnormal cardioelectrophysiology involving the VOLTAGE-GATED POTASSIUM CHANNEL. It results from mutation of KCNQ1 gene (Subtype 1 or JLN1) or the KCNE1 gene (Subtype 2 or JLN2).'),
  'D029594': NtO(MH='Onagraceae', MS='The evening primrose plant family of the order Myrtales, subclass Rosidae, class Magnoliopsida. Flower parts are mostly in fours and the ovary is inferior.'),
  'D029595': NtO(MH='Orchidaceae', MS='A plant family of the order Asparagales. All members of the orchid family have the same bilaterally symmetrical flower structure, with three sepals, but the flowers vary greatly in color and shape.'),
  'D029596': NtO(MH='Paeonia', MS="A plant genus of the family Paeoniaceae, order Dilleniales, subclass Dilleniidae, class Magnoliopsida. These perennial herbs are up to 2 m (6') tall. Leaves are alternate and are divided into three lobes, each lobe being further divided into three smaller lobes. The large flowers are symmetrical, bisexual, have 5 sepals, 5 petals (sometimes 10), and many stamens."),
  'D029597': NtO(MH='Romano-Ward Syndrome', MS='A form of long QT syndrome that is without congenital deafness. It is caused by mutation of the KCNQ1 gene which encodes a protein in the VOLTAGE-GATED POTASSIUM CHANNEL.'),
  'D029598': NtO(MH='Passiflora', MS='A plant genus of the family Passifloraceae, order Violales, subclass Dilleniidae, class Magnoliopsida. They are vines with ornamental flowers and edible fruit.'),
  'D029601': NtO(MH='Pedaliaceae', MS='The sesame family of the order Lamiales that are mainly herbs and shrubs growing in warm regions.'),
  'D029602': NtO(MH='Phytolaccaceae', MS='The pokeweed plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida. The leaves are alternate, simple and smooth-edged and the flowers are in spikes or panicles and are usually bisexual.'),
  'D029603': NtO(MH='Phytolacca', MS='A plant genus of the family PHYTOLACCACEAE, order Caryophyllales.'),
  'D029604': NtO(MH='Phytolacca americana', MS='A plant species of the family PHYTOLACCACEAE. The root has been used in traditional medicine and contains POKEWEED MITOGENS, triterpene glycosides, and antiviral protein.'),
  'D029605': NtO(MH='Phytolacca dodecandra', MS='A plant species of the family PHYTOLACCACEAE. The root has been used in traditional medicine and contains SAPONINS used to poison SNAILS.'),
  'D029606': NtO(MH='Plumbaginaceae', MS='A plant family of the order Plumbaginales, subclass Caryophyllidae, class Magnoliopsida of shrubs and herbs. Some members contain ANTHOCYANINS and NAPHTHOQUINONES.'),
  'D029607': NtO(MH='Seminal Vesicle Secretory Proteins', MS='The secretory proteins of the seminal vesicles are proteins and enzymes that are important in the rapid clotting of the ejaculate. The major clotting protein is seminal vesicle-specific antigen. Many of these seminal vesicle proteins are under androgen regulation, and are substrates for the prostatic enzymes, such as the PROSTATE-SPECIFIC ANTIGEN, a protease and an esterase.'),
  'D029608': NtO(MH='Polygalaceae', MS='A plant family of the order Polygalales, subclass Rosidae, class Magnoliopsida.'),
  'D029621': NtO(MH='Polypodiaceae', MS='The fern plant family of the order Polypodiales, class Filicopsida, division Pteridophyta, subkingdom TRACHEOPHYTA.'),
  'D029622': NtO(MH='Portulacaceae', MS='A plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida. There are no true petals; each flower has two to six sepals. They produce betacyanin and betaxanthin pigments and lack anthocyanins.'),
  'D029623': NtO(MH='Primulaceae', MS='A plant family of the order ERICALES, class MAGNOLIOPSIDA, with flowers having both stamens and pistil, producing encapsulated fruits.'),
  'D029624': NtO(MH='Ferns', MS='Seedless nonflowering plants of the class Filicinae. They reproduce by spores that appear as dots on the underside of feathery fronds. In earlier classifications the Pteridophyta included the club mosses, horsetails, ferns, and various fossil groups. In more recent classifications, pteridophytes and spermatophytes (seed-bearing plants) are classified in the Subkingdom TRACHEOPHYTA (also known as Tracheophyta).'),
  'D029625': NtO(MH='Pteridaceae', MS='A plant family of the order Polypodiales, class Filicopsida, division Pteridophyta (FERNS).'),
  'D029626': NtO(MH='Ranunculaceae', MS='The buttercup plant family of the order RANUNCULALES, class MAGNOLIOPSIDA. The leaves are usually alternate and stalkless. The flowers usually have two to five free sepals and may be radially symmetrical or irregular.'),
  'D029627': NtO(MH='Crassulaceae', MS='The stonecrop plant family of the order ROSALES, subclass Rosidae, class Magnoliopsida that grow in warm, dry regions. The leaves are thick. The flower clusters are red, yellow, or white.'),
  'D029628': NtO(MH='Hydrangeaceae', MS='A plant family of the order ROSALES, subclass Rosidae, class Magnoliopsida.'),
  'D029630': NtO(MH='Santalaceae', MS='A plant family of the order Santalales, subclass Rosidae, class Magnoliopsida. They are parasites that form connections (haustoria) to their hosts to obtain water and nutrients. The one-seeded fruit may be surrounded by a brightly colored nut-like structure.'),
  'D029631': NtO(MH='Sapindaceae', MS='The soapberry plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida. Some members contain SAPONINS.'),
  'D029632': NtO(MH='Sapotaceae', MS='A plant family of the order Ebenales, subclass Dilleniidae, class Magnoliopsida that are tropical trees which have elongate latex cells. Several members bear sweet edible fruits and produce triterpenoid saponins.'),
  'D029633': NtO(MH='Schisandraceae', MS='A plant family of the order AUSTROBAILEYALES, class MAGNOLIOPSIDA.'),
  'D029641': NtO(MH='Simaroubaceae', MS='A plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida. Leaves are alternate and compound. Most have small flowers, bitter bark, and fleshy fruits that are sometimes winged. Members contain QUASSINS.'),
  'D029642': NtO(MH='Smilacaceae', MS='A plant family of the order Liliales, subclass Liliidae, class Liliopsida (monocotyledon).'),
  'D029644': NtO(MH='Styracaceae', MS='A plant family of the order Ebenales, subclass Dilleniidae, class Magnoliopsida.'),
  'D029645': NtO(MH='Thymelaeaceae', MS='A plant family of the order Myrtales, subclass Rosidae, class Magnoliopsida. They are mainly trees and shrubs. Many members contain mucilage and COUMARINS.'),
  'D029646': NtO(MH='Tiliaceae', MS='A plant family of the order Malvales, subclass Dilleniidae, class Magnoliopsida.'),
  'D029647': NtO(MH='Turnera', MS='A plant genus of the family Passifloraceae.'),
  'D029648': NtO(MH='Verbenaceae', MS='A plant family of the order Lamiales, subclass Asteridae, class Magnoliopsida. The leaves are opposite or whorled. The flowers are aggregated in spikes, clusters, or racemes.'),
  'D029649': NtO(MH='Violaceae', MS='A plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida.'),
  'D029650': NtO(MH='Enterovirus, Bovine', MS='A species in the family ENTEROVIRUS infecting cattle.'),
  'D029661': NtO(MH='Alnus', MS='A plant genus of the family BETULACEAE that is distinguished from birch (BETULA) by its usually stalked winter buds and by cones that remain on the branches after the small, winged nutlets are released.'),
  'D029662': NtO(MH='Betula', MS='A plant genus of the family BETULACEAE. The tree has smooth, resinous, varicolored or white bark, marked by horizontal pores (lenticels), which usually peels horizontally in thin sheets.'),
  'D029663': NtO(MH='Tabebuia', MS='A plant genus of the family BIGNONIACEAE that is a source of lapachol and beta-lapachone.'),
  'D029681': NtO(MH='Arabidopsis Proteins', MS='Proteins that originate from plants species belonging to the genus ARABIDOPSIS. The most intensely studied species of Arabidopsis, Arabidopsis thaliana, is commonly used in laboratory experiments.'),
  'D029682': NtO(MH='Amsinckia', MS='A plant genus of the family BORAGINACEAE. Members contain PYRROLIZIDINE ALKALOIDS. The common name of burweed is also used for Triumfetta (TILIACEAE).'),
  'D029683': NtO(MH='Borago', MS='A plant genus of the family BORAGINACEAE.'),
  'D029684': NtO(MH='Lithospermum', MS='A plant genus of the family BORAGINACEAE. Members contain lithospermans and lithospermic acid.'),
  'D029685': NtO(MH='Pulmonaria', MS='A plant genus of the family BORAGINACEAE.'),
  'D029686': NtO(MH='Lepidium', MS='A plant genus of the family BRASSICACEAE growing in Peru mountains. It is the source of maca root.'),
  'D029687': NtO(MH='Capsella', MS='A plant genus of the family CRUCIFERAE.'),
  'D029688': NtO(MH='Brassica napus', MS='A plant species of the family BRASSICACEAE best known for the edible roots.'),
  'D029701': NtO(MH='Saccharomyces cerevisiae Proteins', MS='Proteins obtained from the species SACCHAROMYCES CEREVISIAE. The function of specific proteins from this organism are the subject of intense scientific interest and have been used to derive basic understanding of the functioning similar proteins in higher eukaryotes.'),
  'D029702': NtO(MH='Schizosaccharomyces pombe Proteins', MS='Proteins obtained from the species Schizosaccharomyces pombe. The function of specific proteins from this organism are the subject of intense scientific interest and have been used to derive basic understanding of the functioning similar proteins in higher eukaryotes.'),
  'D029721': NtO(MH='Drosophila Proteins', MS='Proteins that originate from insect species belonging to the genus DROSOPHILA. The proteins from the most intensely studied species of Drosophila, DROSOPHILA MELANOGASTER, are the subject of much interest in the area of MORPHOGENESIS and development.'),
  'D029741': NtO(MH='Brassica rapa', MS='A subspecies of Brassica that includes rape mustard. It is cultivated for the seed and used as animal feed.'),
  'D029742': NtO(MH='Caenorhabditis elegans Proteins', MS='Proteins from the nematode species CAENORHABDITIS ELEGANS. The proteins from this species are the subject of scientific interest in the area of multicellular organism MORPHOGENESIS.'),
  'D029743': NtO(MH='Boswellia', MS='A plant genus of the family BURSERACEAE used medicinally since ancient times. It is a source of salai guggal (the gum resin), boswellic acid (ursane type TRITERPENES), and FRANKINCENSE.'),
  'D029744': NtO(MH='Commiphora', MS='A plant genus of the family BURSERACEAE which is a source of gugulipid and guggulu extract. Balm of Gilead is a common name more often referring to POPULUS and sometimes to ABIES.'),
  'D029745': NtO(MH='Codonopsis', MS='A plant genus of the family CAMPANULACEAE. The common name of bellflower may sometimes be confused with other plants in the family that have similar appearance.'),
  'D029746': NtO(MH='Lobelia', MS='A plant genus of the family CAMPANULACEAE used medicinally and is a source of LOBELINE.'),
  'D029747': NtO(MH='Opuntia', MS='A plant genus of the family CACTACEAE. Species with cylindrical joints are called Cholla; flat jointed ones are Prickly-pear.'),
  'D029748': NtO(MH='Caryophyllaceae', MS='A plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida. The species are diverse in appearance and habitat; most have swollen leaf and stem joints.'),
  'D029749': NtO(MH='Dianthus', MS='A plant genus of the family CARYOPHYLLACEAE. Members contain dianthins, which are ribosome inactivating proteins.'),
  'D029750': NtO(MH='Saponaria', MS='A plant species of the family CARYOPHYLLACEAE. It is a source of SAPONINS. The common name "Soapwort" is also used with VACCARIA. The common name of "Bouncing Bet" is occasionally used with VIOLA.'),
  'D029751': NtO(MH='Silene', MS="A plant genus of the family CARYOPHYLLACEAE. The common name of campion is also used with LYCHNIS. The common name of 'pink' can be confused with other plants."),
  'D029752': NtO(MH='Stellaria', MS='A plant genus of the family Caryophyllaceae.'),
  'D029753': NtO(MH='Celastraceae', MS='A plant family of the order Celastrales, subclass Rosidae, class Magnoliopsida.'),
  'D029754': NtO(MH='Catha', MS='A plant genus of the family CELASTRACEAE. The leafy stems of khat are chewed by some individuals for stimulating effect. Members contain ((+)-norpseudoephedrine), cathionine, cathedulin, cathinine & cathidine.'),
  'D029755': NtO(MH='Maytenus', MS='A plant genus of the family CELASTRACEAE.'),
  'D029756': NtO(MH='Euonymus', MS='A plant genus of the family CELASTRACEAE.'),
  'D029757': NtO(MH='Tripterygium', MS='A plant genus of the family CELASTRACEAE that is a source of triterpenoids and diterpene epoxides such as triptolide.'),
  'D029758': NtO(MH='Cistus', MS='A plant genus of the family CISTACEAE. The common name of rock rose is also sometimes used with the closely related Helianthemum genus (CISTACEAE).'),
  'D029759': NtO(MH='Calophyllum', MS='A plant genus of the family CLUSIACEAE. Members contain costatolide, calanolides and 4-phenylfuranocoumarins (FUROCOUMARINS).'),
  'D029760': NtO(MH='Garcinia', MS='A plant genus of the family CLUSIACEAE. Members contain XANTHONES.'),
  'D029761': NtO(MH='Garcinia cambogia', MS='A plant species of the family CLUSIACEAE. It is a source of hydroxycitric acid.'),
  'D029762': NtO(MH='Garcinia mangostana', MS='A plant genus of the family CLUSIACEAE. It is the source of the mangosteen fruit.'),
  'D029763': NtO(MH='Garcinia kola', MS="A plant species of the family CLUSIACEAE. The common name of 'Bitter Kola' is sometimes also used to refer to COLA."),
  'D029764': NtO(MH='Combretaceae', MS='A plant family of the order Myrtales, subclass Rosidae, class Magnoliopsida. They are mostly trees and shrubs growing in warm areas.'),
  'D029765': NtO(MH='Combretum', MS='A plant genus of the family COMBRETACEAE. Triterpenes and combretastatin have been identified in members of this genus.'),
  'D029766': NtO(MH='Terminalia', MS='A plant genus of the family COMBRETACEAE. Members contain arjunin, an ellagitannin (TANNINS).'),
  'D029767': NtO(MH='Cornaceae', MS='A plant family of the order Cornales, subclass Rosidae, class Magnoliopsida that is a loose grouping of woody ornamentals: 11 of its 14 genera have been placed in single families by some authorities. Some botanists combine members of NYSSACEAE into this family.'),
  'D029768': NtO(MH='Cornus', MS='A plant genus of the family CORNACEAE. It is widely cultivated for the attractive flowers.'),
  'D029769': NtO(MH='Camptotheca', MS='A plant genus of the family NYSSACEAE (sometimes classified in the CORNACEAE family). It is a source of CAMPTOTHECIN.'),
  'D029770': NtO(MH='Costus', MS='A plant genus of the family Costaceae (sometimes classified in Zingiberaceae), order Zingiberales, subclass Zingiberidae, class Liliopsida (monocotyledons). It is a source of SAPONINS and furostanol glycosides.'),
  'D029771': NtO(MH='Rhodiola', MS='A plant genus of the family CRASSULACEAE. Members contain rhodioloside. This roseroot is unrelated to the familiar rose (ROSA). Some species in this genus are called stonecrop which is also a common name for SEDUM.'),
  'D029772': NtO(MH='Bryonia', MS='A plant genus of the family CUCURBITACEAE that is the source of bryodin 1 (a ribosome-inactivating protein).'),
  'D029773': NtO(MH='Citrullus', MS='A plant genus of the family CUCURBITACEAE known for the edible fruit.'),
  'D029774': NtO(MH='Gynostemma', MS='A plant genus of the family CUCURBITACEAE. It is a source of gypenosides and triterpenoid SAPONINS.'),
  'D029775': NtO(MH='Momordica', MS='A plant genus of the family CUCURBITACEAE. It is a source of momordin.'),
  'D029776': NtO(MH='Luffa', MS='A plant genus of the family CUCURBITACEAE known for the fruit which is the source of the luffa sponge and the seeds which contain luffin.'),
  'D029777': NtO(MH='Momordica charantia', MS='A plant species of the family CUCURBITACEAE. It is a source of ribosome-inactivating proteins and triterpene glycosides.'),
  'D029778': NtO(MH='Trichosanthes', MS='A plant species of the family CUCURBITACEAE that is a source of TRICHOSANTHIN (a ribosomal inhibitory protein).'),
  'D029779': NtO(MH='Cupressaceae', MS='A plant family of the order Pinales, class Pinopsida, division Tracheophyta. They are mainly resinous, aromatic evergreen trees.'),
  'D029780': NtO(MH='Chamaecyparis', MS='A plant genus of the family CUPRESSACEAE which should not be confused with other cedar and cypress trees of THUJA or CUPRESSUS genera.'),
  'D029781': NtO(MH='Cupressus', MS='A plant genus of the family CUPRESSACEAE. Cypress ordinarily refers to this but also forms part of the name of plants in other genera.'),
  'D029782': NtO(MH='Libocedrus', MS="A plant genus of the family CUPRESSACEAE. The common name of 'Cedar' makes this easily confused with other trees using that name including cedrus, JUNIPERUS; and CHAMAECYPARIS."),
  'D029783': NtO(MH='Thuja', MS='A plant genus of the family CUPRESSACEAE.'),
  'D029784': NtO(MH='Cycas', MS='A plant genus of the family Cycadaceae, order Cycadales, class Cycadopsida, division CYCADOPHYTA of palm-like trees. It is a source of CYCASIN, the beta-D-glucoside of methylazoxymethanol.'),
  'D029785': NtO(MH='Cyperaceae', MS='The sedge plant family of the order Cyperales, subclass Commelinidae, class Liliopsida (monocotyledons)'),
  'D029786': NtO(MH='Cyperus', MS='A plant genus of the family CYPERACEAE. SESQUITERPENES are found in some of the species.'),
  'D029787': NtO(MH='Drosera', MS='A plant genus of the family Droseraceae, order Nepenthales, subclass Dilleniidae, class Magnoliopsida, that contains naphthoquinone glucosides. The name sundew is rarely used for PYROLA.'),
  'D029788': NtO(MH='Diospyros', MS='A plant genus of the family EBENACEAE, order Ebenales, subclass Dilleniidae, class Magnoliopsida best known for the edible fruit and the antibacterial activity and compounds of the wood.'),
  'D029789': NtO(MH='Ephedra', MS='A plant genus of the family Ephedraceae, order Ephedrales, class Gnetopsida, division Gnetophyta.'),
  'D029790': NtO(MH='Ephedra sinica', MS='A plant species of the family Ephedraceae, order Ephedrales, class Gnetopsida, division Gnetophyta. It is a source of EPHEDRINE and other alkaloids.'),
  'D029791': NtO(MH='Ericaceae', MS='The heath plant family of the order Ericales, subclass Dilleniidae, class Magnoliopsida that are generally shrubs or small trees. Leaves are alternate, simple, and leathery; flowers are symmetrical with a 4- or 5-parted corolla of partly fused petals.'),
  'D029792': NtO(MH='Arctostaphylos', MS='A plant genus of the family ERICACEAE.'),
  'D029793': NtO(MH='Rhododendron', MS='A plant genus of the family ERICACEAE.'),
  'D029794': NtO(MH='Ledum', MS='A plant genus of the family ERICACEAE.'),
  'D029795': NtO(MH='Calluna', MS='A plant genus of the family ERICACEAE.'),
  'D029796': NtO(MH='Vaccinium', MS='A plant genus of the family ERICACEAE known for species with edible fruits.'),
  'D029797': NtO(MH='Huckleberry Plant', MS='Several plant species of the genus VACCINIUM known for the edible huckleberry fruit.'),
  'D029798': NtO(MH='Blueberry Plants', MS='Several species of the genus Vaccinium which produce the edible blueberry fruit.'),
  'D029799': NtO(MH='Vaccinium macrocarpon', MS='A plant species of the family VACCINIUM known for the sour fruit which is sometimes used for urinary tract infections.'),
  'D029800': NtO(MH='Vaccinium myrtillus', MS='A plant species of the family VACCINIUM.'),
  'D029801': NtO(MH='Vaccinium vitis-idaea', MS='A plant species of the genus VACCINIUM.'),
  'D029802': NtO(MH='Croton', MS="A plant genus of the family EUPHORBIACEAE. The common name of dragon's blood is also used for DRACAENA and Daemonorops (ARECACEAE). Croton tiglium is the source of CROTON OIL."),
  'D029803': NtO(MH='Phyllanthus', MS='A plant genus of the family EUPHORBIACEAE. Bahupatra (MEDICINE, AYURVEDIC) is prepared from this.'),
  'D029821': NtO(MH='Enterovirus A, Human', MS='A species of ENTEROVIRUS infecting humans and containing 10 serotypes, mostly coxsackieviruses.'),
  'D029822': NtO(MH='Enterovirus B, Human', MS='A species of ENTEROVIRUS infecting humans and containing 36 serotypes. It is comprised of all the echoviruses and a few coxsackieviruses, including all of those previously named coxsackievirus B.'),
  'D029842': NtO(MH='Phyllanthus emblica', MS='A plant species of the family EUPHORBIACEAE.'),
  'D029843': NtO(MH='Suregada', MS='A plant genus of the family EUPHORBIACEAE that contains gelonin, a ribosome-inactivating protein.'),
  'D029845': NtO(MH='Amphibian Proteins', MS='Proteins obtained from species in the class of AMPHIBIANS.'),
  'D029846': NtO(MH='Senna Plant', MS='A plant genus of the family FABACEAE. SENNA EXTRACT is obtained from members of this genus. Members contain ANTHRAQUINONES and have been an ingredient in laxatives (CATHARTICS). Many species of the CASSIA genus have been reclassified into this genus. This bush should not be confused with the Cassia tree (CINNAMOMUM).'),
  'D029861': NtO(MH='Tamarindus', MS='A plant genus of the family FABACEAE known for its sour fruit.'),
  'D029862': NtO(MH='Prosopis', MS='A plant genus of the family FABACEAE that is a source of prosopis gum.'),
  'D029863': NtO(MH='Cicer', MS='A plant genus of the family FABACEAE known for the edible beans.'),
  'D029864': NtO(MH='Crotalaria', MS='A plant genus of the family FABACEAE that contains crotalarin.'),
  'D029865': NtO(MH='Cytisus', MS='A plant genus of the family FABACEAE that is sometimes called broom because of the shape of the plant. Members produce SPARTEINE.'),
  'D029866': NtO(MH='Derris', MS='A plant genus of the family FABACEAE. The root is a source of rotenoids (ROTENONE) and flavonoids. Some species of Pongamia have been reclassified to this genus and some to MILLETTIA. Some species of Deguelia have been reclassified to this genus.'),
  'D029867': NtO(MH='Xenopus Proteins', MS='Proteins obtained from various species of Xenopus. Included here are proteins from the African clawed frog (XENOPUS LAEVIS). Many of these proteins have been the subject of scientific investigations in the area of MORPHOGENESIS and development.'),
  'D029881': NtO(MH='Galega', MS='A plant genus of the family FABACEAE that contains galegine (GUANIDINES). Most of the other species have been reclassified to TEPHROSIA; a few to ASTRAGALUS PLANT; and INDIGOFERA.'),
  'D029882': NtO(MH='Glycyrrhiza uralensis', MS='A plant species of the family FABACEAE.'),
  'D029883': NtO(MH='Indigofera', MS='A plant genus of the family FABACEAE. The common name of indigo also refers to Baptisia or Amorpha genera (FABACEAE).'),
  'D029884': NtO(MH='Lespedeza', MS='A plant genus of the family FABACEAE.'),
  'D029901': NtO(MH='Maackia', MS='A plant genus of the family FABACEAE. It contains a hemagglutinin.'),
  'D029902': NtO(MH='Medicago', MS='A plant genus of the family FABACEAE. It is distinct from Sweet Clover (MELILOTUS), from Bush Clover (LESPEDEZA), and from Red Clover (TRIFOLIUM).'),
  'D029903': NtO(MH='Melilotus', MS='A plant genus of the family FABACEAE. Molded yellow sweet clover hay, in which the coumarin of sweet clover is converted to DICOUMAROL, is the causative agent of a hemorrhagic disorder in cattle - "hemorrhagic sweet clover disease" and featured in the history of anticoagulant drug development.'),
  'D029904': NtO(MH='Myroxylon', MS='A plant genus of the family FABACEAE that is a source of aromatic resinous balsams (Balsam of Tolu and Balsam of Peru) used in perfumery and as a constituent in cough syrups and lozenge.'),
  'D029905': NtO(MH='Pachyrhizus', MS='A plant genus of the family FABACEAE. Some Pachyrhizus have been reclassified to PUERARIA. Do not confuse with yam (IPOMOEA; or DIOSCOREA) or African yam bean (SPHENOSTYLIS).'),
  'D029906': NtO(MH='Physostigma', MS='A plant genus of the family FABACEAE that is a source of PHYSOSTIGMINE.'),
  'D029907': NtO(MH='Psoralea', MS='A plant genus of the family FABACEAE that is a source of psoralen (FICUSIN).'),
  'D029908': NtO(MH='Pueraria', MS='A plant genus of the family FABACEAE a common weed of the southeast US. There has been folk use for alcoholism and liver protection. It contains puerarin, kakkalide, daidzein (isoflavonoids), and kudzusaponins (oleanene-type triterpene glycosides).'),
  'D029909': NtO(MH='Sophora', MS='A plant genus of the family FABACEAE.'),
  'D029910': NtO(MH='Trigonella', MS='A plant genus of the family FABACEAE.'),
  'D029911': NtO(MH='Caesalpinia', MS='A plant genus of the family FABACEAE. The common name of "Bird-Of-Paradise" is also used for other plants such as Heliconia (HELICONIACEAE) and Strelitzia (STRELITZIACEAE) and some birds. The common name of "Cat\'s-Claw" is more often used with UNCARIA. The common name of "Pernambuco" also refers to a state in Brazil. Furanoditerpenoid lactones and caesalpin are produced by members of this genus.'),
  'D029912': NtO(MH='Abrus', MS='A plant genus of the family FABACEAE whose members contain the highly toxic compound ABRIN.'),
  'D029921': NtO(MH='Trifolium', MS='A plant genus of the family FABACEAE.'),
  'D029941': NtO(MH='Fish Proteins', MS='Proteins obtained from species of fish (FISHES).'),
  'D029961': NtO(MH='Zebrafish Proteins', MS='Proteins obtained from the ZEBRAFISH. Many of the proteins in this species have been the subject of studies involving basic embryological development (EMBRYOLOGY).'),
  'D029962': NtO(MH='Fagaceae', MS='A plant family of the order Fagales subclass Hamamelidae, class Magnoliopsida.'),
  'D029963': NtO(MH='Quercus', MS='A plant genus of the family FAGACEAE that is a source of TANNINS. Do not confuse with Holly (ILEX).'),
  'D029964': NtO(MH='Fagus', MS='A plant genus of the family FAGACEAE.'),
  'D029965': NtO(MH='Gentianaceae', MS='A plant family of the order Gentianales, subclass Asteridae, class Magnoliopsida.'),
  'D029966': NtO(MH='Centaurium', MS='A plant genus of the family GENTIANACEAE. Triterpene lactones and other compounds have been isolated from species of this genus. The common name of century plant has also been used for the AGAVE genus.'),
  'D029967': NtO(MH='Swertia', MS='A plant genus of the family GENTIANACEAE. It is a source of swertiapuniside and IRIDOID GLYCOSIDES.'),
  'D029968': NtO(MH='Escherichia coli Proteins', MS='Proteins obtained from ESCHERICHIA COLI.'),
  'D029969': NtO(MH='Angelica', MS='A plant genus of the family Apiaceae.'),
  'D029970': NtO(MH='Angelica archangelica', MS='A plant species of the family Apiaceae that contains COUMARINS.'),
  'D029971': NtO(MH='Angelica sinensis', MS='A plant species of the family APIACEAE that is the source of dong quai.'),
  'D029972': NtO(MH='Bupleurum', MS='A plant genus of the family APIACEAE that is the source of bupleurum root and of bupleurotoxin and is an ingredient of sho-saiko-to.'),
  'D029974': NtO(MH='Carum', MS='A plant genus of the family Apiaceae. The seeds are used as flavoring.'),
  'D029981': NtO(MH='Cicuta', MS='A plant genus of the family APIACEAE. It is a source of cicutoxin.'),
  'D029982': NtO(MH='Conium', MS='A plant genus of the family APIACEAE that is a source of coniine.'),
  'D030001': NtO(MH='Foeniculum', MS='A plant genus of the family APIACEAE used in SPICES.'),
  'D030002': NtO(MH='Heracleum', MS='A plant genus of the family APIACEAE. Members contain COUMARINS.'),
  'D030003': NtO(MH='Levisticum', MS='A plant genus of the family APIACEAE.'),
  'D030004': NtO(MH='Ligusticum', MS='A plant genus of the family APIACEAE.'),
  'D030005': NtO(MH='Sanicula', MS='A plant genus of the family Apiaceae. Commonly called snakeroot but that name is used for many other plants such as ASARUM; ARISTOLOCHIA; and Polygala.'),
  'D030006': NtO(MH='Amsonia', MS='A plant genus of the family APOCYNACEAE. Species of this genus contain YOHIMBINE.'),
  'D030007': NtO(MH='Nerium', MS='A plant genus of the family APOCYNACEAE. It is a very poisonous plant that contains cardioactive agents.'),
  'D030008': NtO(MH='Catharanthus', MS='A plant genus of the family Apocynaceae. It is the source of VINCA ALKALOIDS, used in leukemia chemotherapy.'),
  'D030009': NtO(MH='Aspidosperma', MS='A plant genus of the family APOCYNACEAE. It contains ellipticine.'),
  'D030010': NtO(MH='Apocynum', MS='A plant genus of the family APOCYNACEAE. On rare occasions it is called Milkweed, but should not be confused with true Milkweed (ASCLEPIAS).'),
  'D030011': NtO(MH='Ochrosia', MS='A plant genus of the family Apocynaceae. Species of this genus contain 9-methoxyellipticine (ELLIPTICINES) which is the basis of one of the classes of ANTINEOPLASTIC AGENTS.'),
  'D030012': NtO(MH='Gentiana', MS='A plant genus of the family Gentianaceae whose members contain SECOIRIDOIDS and have been used in TRADITIONAL MEDICINE for suppressing INFLAMMATION.'),
  'D030013': NtO(MH='Strophanthus', MS='A plant genus of the family APOCYNACEAE that contains OUABAIN cardiac glycosides.'),
  'D030014': NtO(MH='Tabernaemontana', MS='A plant genus of the family APOCYNACEAE that contains bisindole alkaloids and IBOGAINE.'),
  'D030015': NtO(MH='Enterovirus C, Human', MS='A species of ENTEROVIRUS infecting humans and containing 11 serotypes, all coxsackieviruses.'),
  'D030016': NtO(MH='Enterovirus D, Human', MS='A species of ENTEROVIRUS infecting humans and consisting of 2 serotypes: Human enterovirus 68 and Human enterovirus 70.'),
  'D030017': NtO(MH='Ilex', MS="A plant genus of the family AQUIFOLIACEAE. The common name of 'holly' usually refers to this genus but may sometimes refer to similar looking plants of the MAHONIA or QUERCUS genus."),
  'D030018': NtO(MH='Ilex guayusa', MS='A plant species of the family AQUIFOLIACEAE known for ritual use by Amazonian Jivaro Indians.'),
  'D030019': NtO(MH='Ilex paraguariensis', MS='A plant species of the family AQUIFOLIACEAE. An infusion of the leaves is commonly drunk in South America for stimulating effect in much the same manner as coffee is in other cultures.'),
  'D030020': NtO(MH='Ilex vomitoria', MS='A plant species of the family AQUIFOLIACEAE. It has been used to induce vomiting in purification rites.'),
  'D030021': NtO(MH='Pinellia', MS='A plant genus of the family ARACEAE that contains pinellian (an acidic polysaccharide). The plant is an ingredient of some traditional Asian medicinal mixtures including sho-saiko-to, saiko-keishi-to, and banxia houpu decoction.'),
  'D030022': NtO(MH='Aristolochia', MS="A plant genus of the family ARISTOLOCHIACEAE. Species of this genus have been used in traditional medicine but they contain aristolochic acid which is associated with nephropathy. These are sometimes called 'snakeroot' but that name is also used with a number of other plants such as POLYGALA; SANICULA; ASARUM; ARISTOLOCHIA; AGERATINA; and others."),
  'D030023': NtO(MH='Asarum', MS="A plant genus of the family ARISTOLOCHIACEAE which was used medicinally by North American Indians. The common name of 'snakeroot' is also used for many other plants, including SANICULA; or ARISTOLOCHIA; or Polygala."),
  'D030024': NtO(MH='Curcuma', MS='A plant genus of the family ZINGIBERACEAE that contains CURCUMIN and curcuminoids.'),
  'D030041': NtO(MH='Hepatitis A virus', MS='A species in the genus HEPATOVIRUS containing one serotype and two strains: HUMAN HEPATITIS A VIRUS and Simian hepatitis A virus causing hepatitis in humans (HEPATITIS A) and primates, respectively.'),
  'D030061': NtO(MH='Parechovirus', MS='A genus in the family PICORNAVIRIDAE infecting humans and rodents. The type species is Human parechovirus.'),
  'D030081': NtO(MH='Palyam Virus', MS='A species of ORBIVIRUS infecting cattle and sheep. It is transmitted by culicine mosquitoes and gnats (CULICOIDES).'),
  'D030101': NtO(MH='Orthoreovirus, Avian', MS='A species of ORTHOREOVIRUS infecting birds, with outcomes ranging from inapparent to lethal depending on the virus strain and age of the host bird. This species does not infect mammals.'),
  'D030102': NtO(MH='Orthoreovirus, Mammalian', MS='A species of ORTHOREOVIRUS infecting mammals (other than baboons). There are four serotypes. In humans they are generally benign but may sometimes cause upper respiratory tract illness or enteritis in infants and children. MAMMALIAN ORTHOREOVIRUS 3 is a very pathogenic virus in laboratory rodents.'),
  'D030121': NtO(MH='Betaretrovirus', MS='A genus of the family RETROVIRIDAE consisting of viruses with either type B or type D morphology. This includes a few exogenous, vertically transmitted and endogenous viruses of mice (type B) and some primate and sheep viruses (type D). MAMMARY TUMOR VIRUS, MOUSE is the type species.'),
  'D030141': NtO(MH='Feedback, Psychological', MS='A mechanism of information stimulus and response that may control subsequent behavior, cognition, perception, or performance. (From APA Thesaurus of Psychological Index Terms, 8th ed.)'),
  'D030161': NtO(MH='Avian Proteins', MS='Proteins obtained from species of BIRDS.'),
  'D030162': NtO(MH='Reptilian Proteins', MS='Proteins obtained from species of REPTILES.'),
  'D030181': NtO(MH='Vitamin D Response Element', MS='A DNA sequence that is found in the promoter region of vitamin D regulated genes. Vitamin D receptor (RECEPTOR, CALCITRIOL) binds to and regulates the activity of genes containing this element.'),
  'D030201': NtO(MH='Activin Receptors, Type I', MS="One of the two types of ACTIVIN RECEPTORS or activin receptor-like kinases (ALK'S). There are several type I activin receptors. The major active ones are ALK-2 (ActR-IA) and ALK-4 (ActR-IB)."),
  'D030221': NtO(MH='Cola', MS='A plant genus of the family STERCULIACEAE. This is the source of the kola nut which contains CAFFEINE and is used in popular beverages.'),
  'D030241': NtO(MH='Diarrhea Virus 1, Bovine Viral', MS='A species of PESTIVIRUS causing systemic infections (BOVINE VIRUS DIARRHEA-MUCOSAL DISEASE) in cattle and some other cloven-hoofed animals. There are several strains and two biotypes: cytopathic (rare) and non-cytopathic. Infections range from clinically inapparent to severe, but do not correlate with biotypes.'),
  'D030242': NtO(MH='Diarrhea Virus 2, Bovine Viral', MS='A species of PESTIVIRUS causing systemic infections including BOVINE VIRUS DIARRHEA-MUCOSAL DISEASE and BOVINE HEMORRHAGIC SYNDROME in cattle and some other cloven-hoofed animals. There are several strains and two biotypes: cytopathic (rare) and non-cytopathic. The severity of disease appears to be strain dependent. Cytopathogenic effects do not correlate with virulence as non-cytopathic BVDV-2 is associated only with Hemorrhagic Disease, Bovine.'),
  'D030243': NtO(MH='Hemorrhagic Syndrome, Bovine', MS='Clinically severe acute disease of cattle caused by noncytopathic forms of Bovine viral diarrhea virus 2 (DIARRHEA VIRUS 2, BOVINE VIRAL). Outbreaks are characterized by high morbidity and high mortality.'),
  'D030261': NtO(MH='Primate T-lymphotropic virus 1', MS='A species in the family DELTARETROVIRUS comprising strains of different host species but arising from the same geographic origin. They include HUMAN T-LYMPHOTROPIC VIRUS 1 and SIMIAN T-LYMPHOTROPIC VIRUS 1.'),
  'D030262': NtO(MH='Soybean Proteins', MS='Proteins which are present in or isolated from SOYBEANS.'),
  'D030263': NtO(MH='Primate T-lymphotropic virus 2', MS='A species in the family DELTARETROVIRUS, comprising strains from different host species but arising from the same geographic origin. They include HUMAN T-LYMPHOTROPIC VIRUS 2; SIMIAN T-LYMPHOTROPIC VIRUS 2; and Simian T-lymphotropic virus - PP.'),
  'D030281': NtO(MH='Simian T-lymphotropic virus 2', MS='A strain of PRIMATE T-LYMPHOTROPIC VIRUS 2 isolated from Congolese bonobos (PAN PANISCUS).'),
  'D030282': NtO(MH='Primate T-lymphotropic virus 3', MS='A species of DELTARETROVIRUS that includes the strains SIMIAN T-LYMPHOTROPIC VIRUS 3 and HUMAN T-LYMPHOTROPIC VIRUS 3.'),
  'D030283': NtO(MH='Epsilonretrovirus', MS='A genus in the family RETROVIRIDAE infecting fish. Species include Walleye dermal sarcoma virus, Walleye epidermal hyperplasia virus 1, and Walleye epidermal hyperplasia virus 2.'),
  'D030301': NtO(MH='Activin Receptors, Type II', MS='One of the two types of ACTIVIN RECEPTORS. They are membrane protein kinases belonging to the family of PROTEIN-SERINE-THREONINE KINASES. The major type II activin receptors are ActR-IIA and ActR-IIB.'),
  'D030321': NtO(MH='Denys-Drash Syndrome', MS='A disorder of sex development characterized by UROGENITAL ABNORMALITIES; GONADAL DYSGENESIS; and WILMS TUMOR. It is caused by a mutation in the Wilms tumor suppressor gene (GENES, WILMS TUMOR) on chromosome 11.'),
  'D030341': NtO(MH='Nidovirales Infections', MS='Infections with viruses of the order NIDOVIRALES. The concept includes ARTERIVIRUS INFECTIONS and CORONAVIRIDAE INFECTIONS.'),
  'D030342': NtO(MH='Genetic Diseases, Inborn', MS="Diseases that are caused by genetic mutations present during embryo or fetal development, although they may be observed later in life. The mutations may be inherited from a parent's genome or they may be acquired in utero."),
  'D030361': NtO(MH='Papillomavirus Infections', MS='Neoplasms of the skin and mucous membranes caused by papillomaviruses. They are usually benign but some have a high risk for malignant progression.'),
  'D030381': NtO(MH='Nodaviridae', MS='A family of RNA viruses infecting INSECTA and fish. There are two genera: Alphanodavirus and Betanodavirus.'),
  'D030401': NtO(MH='Cytochrome-c Oxidase Deficiency', MS='A disease that results from a congenital defect in ELECTRON TRANSPORT COMPLEX IV. Defects in ELECTRON TRANSPORT COMPLEX IV can be caused by mutations in the SURF1, SCO2, COX10, or SCO1 genes. ELECTRON TRANSPORT COMPLEX IV deficiency caused by mutation in SURF1 manifests itself as LEIGH DISEASE; that caused by mutation in SCO2 as fatal infantile cardioencephalomyopathy; that caused by mutation in COX10 as tubulopathy and leukodystrophy; and that caused by mutation in SCO1 as early-onset hepatic failure and neurologic disorder. (from Online Mendelian Inheritance in Man, http://www.ncbi.nlm.nih.gov/Omim, MIM#220110, May 17, 2001)'),
  'D030421': NtO(MH='Peroxynitrous Acid', MS='A potent oxidant synthesized by the cell during its normal metabolism. Peroxynitrite is formed from the reaction of two free radicals, NITRIC OXIDE and the superoxide anion (SUPEROXIDES).'),
  'D030441': NtO(MH='Saussurea', MS='A plant genus of the family ASTERACEAE, order Asterales, subclass Asteridae. It is a source of costus root oil and should not be confused with the genus COSTUS.'),
  'D030481': NtO(MH='Pyruvate Dehydrogenase (Lipoamide)', MS='The E1 component of the multienzyme PYRUVATE DEHYDROGENASE COMPLEX. It is composed of 2 alpha subunits (pyruvate dehydrogenase E1 alpha subunit) and 2 beta subunits (pyruvate dehydrogenase E1 beta subunit).'),
  'D030501': NtO(MH='Epididymal Secretory Proteins', MS='Proteins secreted by the epididymal epithelium. These proteins are both tissue- and species-specific. They are important molecular agents in the process of sperm maturation.'),
  'D030521': NtO(MH='Zingiberaceae', MS='A plant family of the order Zingiberales, subclass Zingiberidae, class Liliopsida. It includes plants which have both flavoring and medicinal properties such as GINGER; turmeric (CURCUMA), and cardamom (ELETTARIA).'),
  'D030541': NtO(MH='Databases, Genetic', MS='Databases devoted to knowledge about specific genes and gene products.'),
  'D030561': NtO(MH='Databases, Nucleic Acid', MS='Databases containing information about NUCLEIC ACIDS such as BASE SEQUENCE; SNPS; NUCLEIC ACID CONFORMATION; and other properties. Information about the DNA fragments kept in a GENE LIBRARY or GENOMIC LIBRARY is often maintained in DNA databases.'),
  'D030562': NtO(MH='Databases, Protein', MS='Databases containing information about PROTEINS such as AMINO ACID SEQUENCE; PROTEIN CONFORMATION; and other properties.'),
  'D030601': NtO(MH='Cistaceae', MS='A plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida. The common name of rock rose is used with several plants of this family.'),
  'D030661': NtO(MH='Genetic Privacy', MS='The protection of genetic information about an individual, family, or population group, from unauthorized disclosure.'),
  'D030681': NtO(MH='Ficus', MS='A plant genus of the family MORACEAE. It is the source of the familiar fig fruit and the latex from this tree contains FICAIN.'),
  'D030701': NtO(MH='Nyssaceae', MS='The sour gum family of the order Cornales.'),
  'D030702': NtO(MH='Nymphaeaceae', MS='The water lily plant family of aquatic, rhizomatous herbs in the class Magnoliopsida.'),
  'D030721': NtO(MH='Nymphaea', MS='A plant genus of the family NYMPHAEACEAE.'),
  'D030741': NtO(MH='Carbonic Anhydrase IV', MS='A membrane-bound carbonic anhydrase found in lung capillaries and kidney.'),
  'D030762': NtO(MH='Estrous Cycle', MS='The period of cyclic physiological and behavior changes in non-primate female mammals that exhibit ESTRUS. The estrous cycle generally consists of 4 or 5 distinct periods corresponding to the endocrine status (PROESTRUS; ESTRUS; METESTRUS; DIESTRUS; and ANESTRUS).'),
  'D030781': NtO(MH='Organisms, Genetically Modified', MS='Organisms whose GENOME has been changed by a GENETIC ENGINEERING technique.'),
  'D030801': NtO(MH='Animals, Genetically Modified', MS='ANIMALS whose GENOME has been altered by GENETIC ENGINEERING, or their offspring.'),
  'D030821': NtO(MH='Plants, Genetically Modified', MS='PLANTS, or their progeny, whose GENOME has been altered by GENETIC ENGINEERING.'),
  'D030841': NtO(MH='Food, Genetically Modified', MS='Food derived from genetically modified organisms (ORGANISMS, GENETICALLY MODIFIED).'),
  'D030861': NtO(MH='Capitalism', MS="A political and economic system characterized by individual rights, by private or corporate ownership of capital goods, and by prices, production, and the distribution of goods that are determined mainly by competition in a free market. (From Merriam-Webster's Collegiate Dictionary, 10th ed)"),
  'D030881': NtO(MH='Disclosure', MS='Revealing of information, by oral or written communication.'),
  'D030921': NtO(MH='Erythroxylaceae', MS='A plant family of the order Linales, subclass Rosidae, class Magnoliopsida best known for the coca plant.'),
  'D030981': NtO(MH='Acro-Osteolysis', MS='A condition with congenital and acquired forms causing recurrent ulcers in the fingers and toes. The congenital form exhibits autosomal dominant inheritance; the acquired form is found in workers who handle VINYL CHLORIDE. When acro-osteolysis is accompanied by generalized OSTEOPOROSIS and skull deformations, it is called HAJDU-CHENEY SYNDROME.'),
  'D031001': NtO(MH='Aceraceae', MS='A plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida. Leaves are opposite and usually toothed or lobed. The fruit, a samara (a winged nutlet), splits into two (rarely three) winged, one-seeded parts. This family is widely distributed in the Northern Hemisphere.'),
  'D031002': NtO(MH='Acer', MS='A plant genus of the family SAPINDACEAE, best known for trees with palmately lobed leaves.'),
  'D031003': NtO(MH='Adoxaceae', MS='A plant family of the order Dipsacales, subclass Asteridae, class Magnoliopsida. Members of this family are sometimes classified in CAPRIFOLIACEAE.'),
  'D031021': NtO(MH='Anacardium', MS='A plant genus of the family ANACARDIACEAE. This is the source of the familiar cashew nuts, which are heat treated to remove the irritant toxin. Cashew nut shell liquid (frequently abbreviated as CNSL) is a major source of alkenyl phenolic compounds, especially ANACARDIC ACIDS, cardol, and cardanol.'),
  'D031022': NtO(MH='Mangifera', MS='A plant genus of the family ANACARDIACEAE best known for the edible fruit.'),
  'D031023': NtO(MH='Semecarpus', MS='A plant genus of the family ANACARDIACEAE that is the source of anacarcin forte. The nut milk extract is cytotoxic.'),
  'D031041': NtO(MH='Annona', MS='A plant genus of the family ANNONACEAE. It has edible fruit and seeds which contain acetogenins and benzoquinazoline and other alkaloids.'),
  'D031042': NtO(MH='Asimina', MS='A plant genus of the family ANNONACEAE. It has edible fruit and seeds which contain asitrocin, asitrilobins, and other tetrahydrofuran acetogenins.'),
  'D031043': NtO(MH='Guatteria', MS='A plant genus of the family ANNONACEAE. Members contain asarone, isoguattouregidine (an isoquinoline alkaloid), norlaureline, puterine, noraporphine alkaloids.'),
  'D031044': NtO(MH='Polyalthia', MS='A plant genus of the family ANNONACEAE. Members contain 8-oxopolyalthiaine.'),
  'D031045': NtO(MH='Rollinia', MS='A plant genus of the family ANNONACEAE. Members contain aporphines, acetogenins and rollinone.'),
  'D031046': NtO(MH='Uvaria', MS='A plant genus of the family ANNONACEAE. Members contain uvarigrin, uvarigrandin, chamuvaritin and other acetogenins and benzylisoquinoline alkaloids.'),
  'D031048': NtO(MH='Cnidium', MS='A plant genus of the family APIACEAE. Members contain osthol.'),
  'D031049': NtO(MH='Eryngium', MS='A plant genus of the family APIACEAE. Members contain SAPONINS.'),
  'D031050': NtO(MH='Oenanthe', MS='A plant genus of the family APIACEAE that is sometimes called Hemlock Water Dropwort but should not be confused with HEMLOCK. It contains enanthotoxin.'),
  'D031051': NtO(MH='Alstonia', MS='A plant genus of the family APOCYNACEAE. Members contain echitovenidine, echitamine, venenatine (an indole alkaloid), and anti-inflammatory triterpenoids'),
  'D031052': NtO(MH='Thevetia', MS='A plant genus of the family APOCYNACEAE. Members contain thevetin.'),
  'D031053': NtO(MH='Vinca', MS='A plant genus of the family Apocynaceae.'),
  'D031055': NtO(MH='Alocasia', MS='A plant genus of the family ARACEAE. Members contain beta-glucosidases and trypsin inhibitors.'),
  'D031056': NtO(MH='Amorphophallus', MS='A plant genus of the family ARACEAE. Members contain konjac glucomannan (MANNANS) and SEROTONIN.'),
  'D031057': NtO(MH='Arisaema', MS='A plant genus of the family ARACEAE. It has a stemless, tuberous root.'),
  'D031058': NtO(MH='Arum', MS='A plant genus of the family ARACEAE. The name derived from ar (fire in Arabic) due to the irritating sap. Flower is a spathe.'),
  'D031059': NtO(MH='Calla Plant', MS='A plant genus of the family ARACEAE. Calla Lily (Callalily) also refers to the ZANTEDESCHIA. There is no relationship with CALLA ANTIGEN.'),
  'D031060': NtO(MH='Colocasia', MS='A plant genus of the family ARACEAE. Members contain acrid calcium oxalate and LECTINS. Polynesians prepare the root into poi. Common names of Taro and Coco Yam (Cocoyam) may be confused with other ARACEAE; XANTHOSOMA; or with common yam (DIOSCOREA).'),
  'D031061': NtO(MH='Cyrtosperma', MS='A plant genus of the family ARACEAE that may be confused with other ARACEAE having similar common names.'),
  'D031062': NtO(MH='Philodendron', MS='A plant genus of the family ARACEAE. As a houseplant it sometimes poisons children and animals.'),
  'D031063': NtO(MH='Xanthosoma', MS="A plant genus of the family ARACEAE. The common names of 'Coco Yam' (Cocoyam) or 'Elephant's Ear' may be confused with COLOCASIA and other ARACEAE or with common yam (DIOSCOREA). It contains LECTINS."),
  'D031064': NtO(MH='Zantedeschia', MS='A plant genus of the family ARACEAE. The common name of Calla Lily or Callalily also refers to the Calla genus (CALLA PLANT).'),
  'D031081': NtO(MH='Calamus', MS='A plant genus of the family ARECACEAE that should not be confused with ACORUS CALAMUS.'),
  'D031101': NtO(MH='Marsdenia', MS='A plant genus of the family ASCLEPIADACEAE. Members contain pregnane glycosides (marsdekoiside & marstomentosides, maryal) and hainaneosides (SAPONINS).'),
  'D031102': NtO(MH='Hemidesmus', MS='A plant genus of the family Apocynaceae. It is a source of PREGNANES and is sometimes an adulterant of commercial sarsaparilla (SMILAX).'),
  'D031103': NtO(MH='Gymnema', MS='A plant genus of the family ASCLEPIADACEAE.'),
  'D031104': NtO(MH='Gymnema sylvestre', MS='A plant species of the genus GYMNEMA that contains gymnemic acid (triterpene SAPONINS) which affects blood sugar level, and gurmarin protein. The common name of Gurmar should not be confused with Guar (CYAMOPSIS).'),
  'D031105': NtO(MH='Cynanchum', MS='A plant genus of the family ASCLEPIADACEAE. Members contain steroidal glycosides and cytotoxic phenanthroindolizidine N-oxide alkaloids.'),
  'D031106': NtO(MH='Cryptolepis', MS='A plant genus of the family ASCLEPIADACEAE. Members contain the alkaloids quindoline, CSA-3, cryptolepine, and neocryptolepine.'),
  'D031107': NtO(MH='Calotropis', MS='A plant genus of the family ASCLEPIADACEAE. The downy akund floss fiber from the seeds is used like kapok.'),
  'D031108': NtO(MH='Asclepias', MS='A plant genus of the family ASCLEPIADACEAE. This is the true milkweed; APOCYNUM & EUPHORBIA hirta are rarely called milkweed. Asclepias asthmatica has been changed to TYLOPHORA.'),
  'D031109': NtO(MH='Periploca', MS='A plant genus of the family Apocynaceae. It is a source of periplocosides (pregnane steroid glycosides).'),
  'D031110': NtO(MH='Tylophora', MS='A plant genus of the family ASCLEPIADACEAE. Members contain phenanthro-indolizidine alkaloids.'),
  'D031111': NtO(MH='Balanophoraceae', MS='A plant family of the order Santalales, subclass Rosidae, class Magnoliopsida. They are root parasites lacking CHLOROPHYLL.'),
  'D031112': NtO(MH='Begoniaceae', MS='A plant family of the order Violales (by some in Begoniales), subclass Dilleniidae, class Magnoliopsida. Members are found throughout tropical and warm temperate habitats. Most are perennial herbs with monoecious flowers (both sexes on the same plant). Fruits are usually capsules containing many tiny seeds.'),
  'D031121': NtO(MH='Mannheimia', MS='Genus of bacteria in the family PASTEURELLACEAE, comprising multiple species that do not ferment trehalose. Species include MANNHEIMIA HAEMOLYTICA; M. glucosida, M. granulomatis, M. ruminalis, and M. varigena.'),
  'D031161': NtO(MH='Pseudoalteromonas', MS='A genus of GRAM-NEGATIVE AEROBIC BACTERIA of marine origin. Many species were formerly classified under ALTEROMONAS.'),
  'D031162': NtO(MH='Achillea', MS='A plant genus of the family ASTERACEAE that has long been used in folk medicine for treating wounds.'),
  'D031163': NtO(MH='Ageratum', MS='A plant genus of the family ASTERACEAE that contains PYRROLIZIDINE ALKALOIDS. Yellow vein disease of Ageratum is caused by a viral DNA complex of a begomovirus (GEMINIVIRIDAE).'),
  'D031164': NtO(MH='Ambrosia', MS='A plant genus of the family ASTERACEAE. The POLLEN is one cause of HAYFEVER.'),
  'D031165': NtO(MH='Ammi', MS='A plant genus of the family APIACEAE that is the primary plant source of KHELLIN.'),
  'D031166': NtO(MH='Anthemis', MS='A plant genus of the family ASTERACEAE that is similar to MATRICARIA but this has tiny chaffy bract scales between each pair of florets. Members contain sesquiterpene lactones. Other plants with similar common names include FERULA; FOENICULUM; MATRICARIA; CHAMAEMELUM and TRIPLEUROSPERMUM.'),
  'D031167': NtO(MH='Chamaemelum', MS='A plant genus of the family ASTERACEAE that is used in folk medicine as CHAMOMILE. Other plants with similar common names include MATRICARIA; TRIPLEUROSPERMUM and ANTHEMIS.'),
  'D031168': NtO(MH='Matricaria', MS="A plant genus of the family ASTERACEAE. M. chamomilla appears similar to Anthemis but this flower disk is conical and hollow and lacks chaffy bract scales and the odor is weaker. The common name of 'manzanilla' is confused with other meanings of the word. 'Matricaria chamomilla sensu' is classified by some as Tripleurospermum perforata. Other plants with similar common names include CHAMAEMELUM; TRIPLEUROSPERMUM and ANTHEMIS."),
  'D031169': NtO(MH='Tripleurospermum', MS='A plant genus of the family ASTERACEAE. It is easily confused with related plants with similar common names of Chamomile and Mayweed (MATRICARIA; CHAMAEMELUM; and ANTHEMIS).'),
  'D031170': NtO(MH='Arctium', MS='A plant genus of the family ASTERACEAE. Arctiin (LIGNANS) is in the seed.'),
  'D031171': NtO(MH='Artemisia absinthium', MS='A plant species of the genus ARTEMISIA, family ASTERACEAE that has been used in ABSINTHE. The oil contains neurotoxic 1-thujone and d-isothujone.'),
  'D031172': NtO(MH='Artemisia annua', MS='A plant species of the genus ARTEMISIA, family ASTERACEAE. It is the source of the antimalarial artemisinin (ANTIMALARIALS).'),
  'D031173': NtO(MH='Aster Plant', MS='A plant genus of the family ASTERACEAE. This plant should not be confused with microtubule asters (MICROTUBULES) nor with aster yellows phytoplasma (mycoplasma-like organisms).'),
  'D031174': NtO(MH='Atractylodes', MS='A plant genus of the family ASTERACEAE. Members contain hinesol and atractylon. Atractylodes rhizome is Byaku-jutsu. A. lancea rhizome is So-jutsu.'),
  'D031175': NtO(MH='Baccharis', MS='A plant genus of the family ASTERACEAE. Other plants called broom include CYTISUS; SPARTIUM; and BROMUS.'),
  'D031176': NtO(MH='Bidens', MS='A plant genus of the family ASTERACEAE. Members contain bidensyneosides (polyacetylene glucosides).'),
  'D031177': NtO(MH='Carduus', MS='A plant genus of the family ASTERACEAE. Members contain arctiin and onopordopicrin.'),
  'D031178': NtO(MH='Carthamus', MS='A plant genus of the family ASTERACEAE.'),
  'D031179': NtO(MH='Centaurea', MS='A plant genus of the family ASTERACEAE. Members contain 5-methyl-8-hydroxycoumarin. The common name of centaury is more often used for CENTAURIUM'),
  'D031180': NtO(MH='Cirsium', MS='A plant genus of the family ASTERACEAE. Members contain pectolinarin (a flavonoid glycoside).'),
  'D031181': NtO(MH='Cnicus', MS='A plant genus of the family ASTERACEAE. Members contain cnicin.'),
  'D031182': NtO(MH='Echinops Plant', MS='A plant genus of the family ASTERACEAE. Members contain a pseudoguaiane sesquiterpene. San-fang-feng is the root of E. grijisii used in DRUGS, CHINESE HERBAL.'),
  'D031183': NtO(MH='Onopordum', MS='A plant genus of the family ASTERACEAE. Members contain eudesmane and elemane SESQUITERPENES.'),
  'D031184': NtO(MH='Sonchus', MS='A plant genus of the family ASTERACEAE. Sesquiterpene lactone glucosides (SESQUITERPENES) have been found in it.'),
  'D031185': NtO(MH='Scolymus', MS='A plant genus of the family ASTERACEAE with a name that is similar to CYNARA SCOLYMUS. Members contain taraxasteryl acetate.'),
  'D031186': NtO(MH='Scorzonera', MS='A plant genus of the family ASTERACEAE. INULIN and SESQUITERPENES have been found in it.'),
  'D031187': NtO(MH='Carthamus tinctorius', MS='A plant genus of the family ASTERACEAE. Oil from the seed (SAFFLOWER OIL) is an important food oil of commerce.'),
  'D031188': NtO(MH='Tragopogon', MS='A plant genus of the family ASTERACEAE. The root and shoots have been used for food.'),
  'D031202': NtO(MH='Chrysanthemum', MS='A plant genus of the family ASTERACEAE. The common names of daisy or marguerite are easily confused with other plants. Some species in this genus have been reclassified to TANACETUM.'),
  'D031203': NtO(MH='Tanacetum', MS='A plant genus of the family ASTERACEAE. Some species of the CHRYSANTHEMUM and the old Pyrethrum genera have been reclassified to this genus. The common name of tansy usually refers to this but also forms part of the common name of other plants such as Tansy Ragwort (SENECIO) and Tansyaster (HAPLOPAPPUS).'),
  'D031204': NtO(MH='Caloric Restriction', MS='Reduction in caloric intake without reduction in adequate nutrition. In experimental animals, caloric restriction has been shown to extend lifespan and enhance other physiological variables.'),
  'D031205': NtO(MH='Taraxacum', MS='A plant genus of the family Asteraceae. Members contain chicoric and chlorogenic acids and germacrane- and eudesmane-type SESQUITERPENES.'),
  'D031206': NtO(MH='Haplopappus', MS='A plant genus of the family ASTERACEAE. Many of the member species have been reclassified to other genera such as Machaeranthera, Isocoma, Ericameria, etc. Ingestion of the plant has been associated with MILK SICKNESS.'),
  'D031207': NtO(MH='Tussilago', MS='A plant genus of the family ASTERACEAE. Flower buds contain farfaratin (SESQUITERPENES) and bisabolene epoxide.'),
  'D031208': NtO(MH='Verbesina', MS='A plant genus of the family ASTERACEAE. Members contain galegine (GUANIDINES).'),
  'D031209': NtO(MH='Wedelia', MS='A plant genus of the family ASTERACEAE. Members contain wedelolactone and wedeloside.'),
  'D031210': NtO(MH='Xanthium', MS='A plant genus of the family ASTERACEAE. The name "prickly burweed" is sometimes used but causes confusion with AMSINCKIA.'),
  'D031211': NtO(MH='Corylus', MS='A plant genus of the family BETULACEAE known for the edible nuts.'),
  'D031212': NtO(MH='Bixaceae', MS='A plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida. Bixa contains bixin. Cochlospermum contains arjunolic acid and gum kondagogu (POLYSACCHARIDES).'),
  'D031213': NtO(MH='Cordia', MS='A plant genus of the family BORAGINACEAE. Members contain TRITERPENES and naphthoxirene.'),
  'D031214': NtO(MH='Arabis', MS='A plant genus of the family BRASSICACEAE. Member species are ornamentals grown for their numerous small white, yellow, pink, or purplish flowers.'),
  'D031215': NtO(MH='Armoracia', MS='A plant genus of the family BRASSICACEAE known for the root used in hot SPICES. It is also the source of HORSERADISH PEROXIDASE which is widely used in laboratories.'),
  'D031216': NtO(MH='Barbarea', MS='A plant genus of the family BRASSICACEAE. Members contain glucobrassicin.'),
  'D031217': NtO(MH='Cardamine', MS='A plant genus of the family BRASSICACEAE that is low-growing in damp meadows of the Northern Hemisphere and has pinnately divided leaves and small white to rose flowers.'),
  'D031218': NtO(MH='Crambe Plant', MS='A plant genus of the family BRASSICACEAE. Members contain crambin.'),
  'D031219': NtO(MH='Isatis', MS='A plant genus of the family BRASSICACEAE that is an ingredient of the preparation PC-SPES that is used to treat PROSTATIC HYPERPLASIA.'),
  'D031220': NtO(MH='Lepidium sativum', MS='A plant species of the genus LEPIDIUM, family BRASSICACEAE that is a fast-growing, often weedy native of western Asia. It is widely grown, especially in its curl-leaved form, and used as a garnish'),
  'D031221': NtO(MH='Knee Dislocation', MS='Slippage of the FEMUR off the TIBIA.'),
  'D031222': NtO(MH='Patellar Dislocation', MS='Displacement of the PATELLA from the femoral groove.'),
  'D031223': NtO(MH='Nasturtium', MS='A plant genus of the family BRASSICACEAE. Nasturtium is also used as a common name for TROPAEOLUM. The common name of watercress is also used for RORIPPA & TROPAEOLUM. This is the most popular of the edible cresses, is a hardy creeping perennial plant, native to Europe but extensively naturalized elsewhere in moist places.'),
  'D031224': NtO(MH='Raphanus', MS='A plant genus of the family BRASSICACEAE known for its peppery red root.'),
  'D031225': NtO(MH='Rorippa', MS='A plant genus of the family BRASSICACEAE that includes several marshy plants. The common name of watercress is also used for NASTURTIUM & TROPAEOLUM.'),
  'D031226': NtO(MH='Sinapis', MS='A plant genus of the family Brassicaceae. The common name of mustard may refer to MUSTARD PLANT (BRASSICA JUNCEA or BRASSICA NIGRA).'),
  'D031227': NtO(MH='Thlaspi', MS='A plant genus of the family BRASSICACEAE that is grown in rock gardens. T. arvense is grown for its large, round ornamental seed pods. The common name of pennycress usually refers to this genus but may also refer to the genus Microthlaspi.'),
  'D031228': NtO(MH='Wasabia', MS='A plant genus of the family BRASSICACEAE used for its hot peppery root.'),
  'D031229': NtO(MH='Bromeliaceae', MS='A plant family of the order Bromeliales, subclass Zingiberidae, class Liliopsida (monocotyledons).'),
  'D031230': NtO(MH='Ananas', MS='A plant genus of the family BROMELIACEAE known for the edible fruit that is the source of BROMELAINS.'),
  'D031231': NtO(MH='Bromelia', MS='A plant genus of the family BROMELIACEAE. Members contain karatasin and balansain (ENDOPEPTIDASES) and BROMELAINS.'),
  'D031232': NtO(MH='Tillandsia', MS='A plant genus of the family BROMELIACEAE. Members contain 3-methoxy-5-hydroxyflavonols.'),
  'D031233': NtO(MH='Bursera', MS='A plant genus of the family BURSERACEAE. Members contain a PODOPHYLLOTOXIN.'),
  'D031234': NtO(MH='Buxaceae', MS='A plant family of the order Euphorbiales, subclass Rosidae, class Magnoliopsida. Leaves are alternate, simple, and leathery. Fruits are one- or two-seeded capsules or drupes (stony-pitted fleshy fruits).'),
  'D031235': NtO(MH='Buxus', MS='A plant genus of the family BUXACEAE. Members contain steroidal alkaloids.'),
  'D031236': NtO(MH='Pachysandra', MS='A plant genus of the family BUXACEAE that should not be confused with common spurge (EUPHORBIA).'),
  'D031237': NtO(MH='Calycanthaceae', MS='A plant family of the order Laurales, subclass Magnoliidae, class Magnoliopsida.'),
  'D031238': NtO(MH='Platycodon', MS='A plant genus of the family CAMPANULACEAE that contains platycodin and other triterpenoid SAPONINS. It is a constituent of kikyo-to (MEDICINE, KAMPO).'),
  'D031239': NtO(MH='Lonicera', MS='A plant genus of the family CAPRIFOLIACEAE. Members contain iridoid glucosides.'),
  'D031240': NtO(MH='Symphoricarpos', MS='A plant genus of the family CAPRIFOLIACEAE. The common name of "snowberry" is also used for GAULTHERIA.'),
  'D031241': NtO(MH='Caricaceae', MS='A plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida.'),
  'D031242': NtO(MH='Agrostemma', MS='A plant genus of the family CARYOPHYLLACEAE that contains ALLANTOIN and a ribosome-inactivating protein.'),
  'D031243': NtO(MH='Arenaria Plant', MS='A plant genus of the family CARYOPHYLLACEAE. The common name of sandwort is used with other plants. Do not confuse with Arenaria bird or with other genera which use arenaria as the species name such as Mya arenaria (softshell clam) or Meloidogyne arenaria, or Ammophila arenaria (marram grass).'),
  'D031244': NtO(MH='Lychnis', MS='A plant genus of the family CARYOPHYLLACEAE. The common name "Campion" is also used with SILENE.'),
  'D031245': NtO(MH='Vaccaria', MS='A plant genus of the family CARYOPHYLLACEAE. The common name "Soapwort" is also used with SAPONARIA.'),
  'D031247': NtO(MH='Celastrus', MS='A plant genus of the family CELASTRACEAE that contains insecticidal sesquiterpenoids and celastrus oil.'),
  'D031248': NtO(MH='Salsola', MS='A plant genus of the family CHENOPODIACEAE. The extract may be called lochein. Tumbleweed may occasionally refer to AMARANTHUS.'),
  'D031249': NtO(MH='Erdheim-Chester Disease', MS='A rare form of non-Langerhans-cell histiocytosis (HISTIOCYTOSIS, NON-LANGERHANS-CELL) with onset in middle age. The systemic disease is characterized by infiltration of lipid-laden macrophages, multinucleated giant cells, an inflammatory infiltrate of lymphocytes and histiocytes in the bone marrow, and a generalized sclerosis of the long bones.'),
  'D031250': NtO(MH='Clethraceae', MS='A plant family of the order Ericales, subclass Dilleniidae, class Magnoliopsida.'),
  'D031251': NtO(MH='Clusia', MS='A plant genus of the family CLUSIACEAE. Members contain benzophenones.'),
  'D031252': NtO(MH='Mammea', MS='A plant genus of the family CLUSIACEAE. Members contain xanthones and BENZOPHENONES. The common name of Mamey is also used with POUTERIA.'),
  'D031253': NtO(MH='Calystegia', MS='A plant genus of the family CONVOLVULACEAE. Members contain calystegine and calystegins.'),
  'D031254': NtO(MH='Convolvulus', MS='A plant genus of the family CONVOLVULACEAE. The common name of morning glory also refers to IPOMOEA. The common name of bindweed also refers to IPOMOEA; CALYSTEGIA; or POLYGONUM.'),
  'D031255': NtO(MH='Carex Plant', MS='A plant genus of the family CYPERACEAE. The seed contains oligostilbenes (STILBENES).'),
  'D031256': NtO(MH='Eleocharis', MS='A plant genus of the family CYPERACEAE.'),
  'D031257': NtO(MH='Droseraceae', MS='A plant family of the order Nepenthales, subclass Dilleniidae, class Magnoliopsida, notable for leaves with sticky gland-tipped hairs that entrap insects.'),
  'D031261': NtO(MH='Poult Enteritis Mortality Syndrome', MS='An acute, transmissible, infectious disease associated with high MORTALITY and MORBIDITY in young turkeys (poults). It is characterized by DIARRHEA; ANOREXIA; growth depression, and immune dysfunction. The cause is unknown but astroviruses (AVASTROVIRUS) and coronaviruses (CORONAVIRUS, TURKEY) have been isolated from diseased poults and are thought to cause the enteritis and increased susceptibility to bacterial infections.'),
  'D031262': NtO(MH='Ebenaceae', MS='A small plant family of the order Ebenales, subclass Dilleniidae, class Magnoliopsida. Members contain NAPHTHOQUINONES.'),
  'D031263': NtO(MH='Elaeagnaceae', MS='A plant family of the order Rhamnales, subclass Rosidae class Magnoliopsida. The plants have a characteristic silvery or rusty-colored sheen, caused by tiny distinctive scales. Flowers have a tubular structure of four sepals. Root nodules host the Frankia (ACTINOMYCETES) nitrogen-fixing symbionts.'),
  'D031281': NtO(MH='Hippophae', MS="A plant genus of the family ELAEAGNACEAE. Linoleic (18:2n-6) and alpha-linolenic acids (18:3n-3) comprised about 70% of seed oil fatty acids. This is unrelated to 'artificial sea buckthorn oil'. This genus does not belong to the buckthorn family (RHAMNACEAE)."),
  'D031282': NtO(MH='Gaultheria', MS='A plant genus of the family ERICACEAE. The common name of "wintergreen" is also used for PYROLA and "snowberry" is also used for SYMPHORICARPOS.'),
  'D031283': NtO(MH='Eucommiaceae', MS='A plant family of the order Eucommiales, subclass Hamamelidae, class Magnoliopsida (some botanists have classified this in the order Hamamelidales or Urticales). Eucomia is an elmlike tree of central and eastern China. Leaves are alternate; deciduous flowers are solitary and unisexual and lack petals and sepals. The male flowers have 6 to 10 stamens and female flowers have one ovary of two carpels, one of which aborts during development so the fruit (a dry, winged structure) contains only one seed. The latex is a source of RUBBER. Tochu tea is an aqueous extract of Eucommia ulmoides leaves and a popular beverage in Japan. (Mutat Res 1997 Jan 15;388(1):7-20).'),
  'D031284': NtO(MH='Aleurites', MS='A plant genus of the family EUPHORBIACEAE that is the source of tung oil and a phorbol diester (PHORBOL ESTERS).'),
  'D031285': NtO(MH='Hippomane', MS='A plant genus of the family EUPHORBIACEAE. The trees contain hippomanin A and the latex is a skin irritant.'),
  'D031286': NtO(MH='Jatropha', MS='A plant genus of the family EUPHORBIACEAE. Members contain jatrophone and other diterpenes.'),
  'D031287': NtO(MH='Sapium', MS='A plant genus of the family EUPHORBIACEAE that contains bukittinggine and PHORBOL ESTERS.'),
  'D031288': NtO(MH='Chamaecrista', MS='A plant genus of the family FABACEAE. Some species were reclassified from CASSIA and Senna.'),
  'D031289': NtO(MH='Albizzia', MS='A plant genus of the family FABACEAE that contains julibroside (triterpenoid saponins).'),
  'D031290': NtO(MH='Butea', MS='A plant genus of the family FABACEAE that contains butrin and isobutrin.'),
  'D031291': NtO(MH='Castanospermum', MS='A plant genus of the family FABACEAE that contains castanospermines, swainsonine, and triterpenoid saponins.'),
  'D031292': NtO(MH='Clitoria', MS='A plant genus of the family FABACEAE that contains ternatins (anthocyanins) and preternatins, antifungal proteins, stigmast-4-ene-3,6-dione, and clitoriacetal (ROTENONE).'),
  'D031293': NtO(MH='Cyamopsis', MS='A plant genus of the family FABACEAE that is the source of guar gum.'),
  'D031294': NtO(MH='Dolichos', MS='A plant genus of the family FABACEAE that contains LECTINS. Many members have been reclassified into other genera of the FABACEAE family.'),
  'D031295': NtO(MH='Griffonia', MS='A plant genus of the family FABACEAE. Members contain Griffonia simplicifolia agglutin-I and griffonin.'),
  'D031296': NtO(MH='Laburnum', MS='A plant genus of the family FABACEAE named after the drooping clusters of flowers.'),
  'D031297': NtO(MH='Lupinus', MS='A plant genus of the family FABACEAE that is a source of SPARTEINE, lupanine and other lupin alkaloids.'),
  'D031298': NtO(MH='Mimosa', MS='A plant genus of the family FABACEAE that contains kukulkanin, a CHALCONE.'),
  'D031299': NtO(MH='Mucuna', MS='A plant genus of the family FABACEAE that is the source of mucuna gum.'),
  'D031300': NtO(MH='Retinal Vasculitis', MS='Inflammation of the retinal vasculature with various causes including infectious disease; LUPUS ERYTHEMATOSUS, SYSTEMIC; MULTIPLE SCLEROSIS; BEHCET SYNDROME; and CHORIORETINITIS.'),
  'D031301': NtO(MH='Robinia', MS='A plant genus of the family FABACEAE that contains linarin (acaciin) and LECTINS.'),
  'D031302': NtO(MH='Spartium', MS="A plant genus of the family FABACEAE that is a source of SPARTEINE. Do not confuse with Broom or Scotch Broom (CYTISUS) or Butcher's Broom (RUSCUS) or Desert Broom (BACCHARIS) or Brome (BROMUS)."),
  'D031303': NtO(MH='Sphenostylis', MS='A plant genus of the family FABACEAE that contains LECTINS and raffinose oligosaccharides.'),
  'D031304': NtO(MH='Tephrosia', MS='A plant genus of the family FABACEAE that contains tephrorin, tephrosone, and C-prenylflavonoids.'),
  'D031305': NtO(MH='Tetrapleura', MS='A plant genus of the family FABACEAE. Members contain molluscicidal triterpene glycosides (TRITERPENES).'),
  'D031306': NtO(MH='Vicia', MS='A plant genus of the family FABACEAE that is widely used as ground cover and forage and known for the edible beans, VICIA FABA.'),
  'D031307': NtO(MH='Vicia faba', MS='A plant species of the genus VICIA, family FABACEAE. The edible beans are well known but they cause FAVISM in some individuals with GLUCOSEPHOSPHATE DEHYDROGENASE DEFICIENCY. This plant contains vicine, convicine, Vicia lectins, unknown seed protein, AAP2 transport protein, and Vicia faba DNA-binding protein 1.'),
  'D031308': NtO(MH='Salicaceae', MS='A plant family of the order MALPIGHIALES, superorder ROSANAE, class MAGNOLIOPSIDA. The leaves are alternate and simple, and staminate (male) flowers consist of one to several stamens; pistillate (female) flowers consist of a single-chambered ovary with several ovules that produce silky seeds.'),
  'D031309': NtO(MH='Ryania', MS='A plant genus of the family Salicaceae that is the source of RYANODINE.'),
  'D031310': NtO(MH='Fumariaceae', MS='The fumitory, or bleeding-heart plant family of the order Papaverales, subclass Magnoliidae, class Magnoliopsida. Flowers are bisexual, with two small sepals and four petals, one pistil and six stamens.'),
  'D031311': NtO(MH='Corydalis', MS='A plant genus of the family FUMARIACEAE (classified by some in PAPAVERACEAE) that contains isoquinoline alkaloids.'),
  'D031312': NtO(MH='Fumaria', MS='A plant genus of the family FUMARIACEAE that contains fumariline and other isoquinoline alkaloids.'),
  'D031313': NtO(MH='Gentianella', MS='A plant genus of the family GENTIANACEAE that contains nitiol (a C25 sesterterpenoid) & nitidasin.'),
  'D031314': NtO(MH='Geraniaceae', MS='A plant family of the order Geraniales, subclass Rosidae, class Magnoliopsida.'),
  'D031315': NtO(MH='Geranium', MS='A plant genus of the family GERANIACEAE. Geranium is also used as a common name for PELARGONIUM.'),
  'D031316': NtO(MH='Pelargonium', MS='A plant genus of the family GERANIACEAE. The common name of geranium is also used for the GERANIUM genus.'),
  'D031317': NtO(MH='Liquidambar', MS='A plant genus of the family HAMAMELIDACEAE. The sap is a source of storax, which should not be confused with the similar named STYRAX genus.'),
  'D031318': NtO(MH='Hippocastanaceae', MS='A plant family of the order Sapindales, subclass Rosidae, class Magnoliopsida.'),
  'D031319': NtO(MH='Aesculus', MS='A plant genus of the family HIPPOCASTANACEAE (or SAPINDACEAE by some) that contains antimicrobial protein 1 and escin. A. hippocastanum is used in folk medicine for treating chronic venous insufficiency.'),
  'D031320': NtO(MH='Hippocrateaceae', MS='A plant family of the order Celastrales, subclass Rosidae, class Magnoliopsida. The Hippocratea genus contains friedelanes, triterpenoid quinone, and hippocrateine I.'),
  'D031321': NtO(MH='Hydrocharitaceae', MS='A plant family of the order Hydrocharitales, subclass ALISMATIDAE, class Liliopsida (monocotyledons).'),
  'D031322': NtO(MH='Eriodictyon', MS='A plant genus of the family HYDROPHYLLACEAE that has a characteristic strong smell.'),
  'D031323': NtO(MH='Iris Plant', MS='A plant genus of the family IRIDACEAE that contains IRIP, a type-1 ribosome-inactivating protein, and iridals (TRITERPENES).'),
  'D031324': NtO(MH='Juglans', MS='A plant genus of the family JUGLANDACEAE that provides the familiar walnut.'),
  'D031325': NtO(MH='Carya', MS='A plant genus of the family JUGLANDACEAE that bears edible nuts.'),
  'D031326': NtO(MH='Krameriaceae', MS='A plant family of the order Polygalales, subclass Rosidae, class Magnoliopsida.'),
  'D031327': NtO(MH='Agastache', MS='A plant genus of the family LAMIACEAE that contains tilianin, agastanol, and agastaquinone (a cytotoxic diterpenoid quinone).'),
  'D031328': NtO(MH='Ajuga', MS='A plant genus of the family LAMIACEAE that contains cyasterone, ajugasterone, 20-hydroxyecdysone, 8-acetylharpagide (an iridoid glycoside).'),
  'D031329': NtO(MH='Ballota', MS='A plant genus of the family LAMIACEAE that contains phenylpropanoids.'),
  'D031330': NtO(MH='Coleus', MS='A plant genus of the family Lamiaceae. The species of Coleus should be distinguished from PLECTRANTHUS BARBATUS - which is also known as Coleus forskohlii.'),
  'D031331': NtO(MH='Hyptis', MS='A plant genus of the family LAMIACEAE that contains 5-methoxydehydropodophyllotoxin (a PODOPHYLLOTOXIN) and other LIGNANS.'),
  'D031332': NtO(MH='Leonurus', MS='A plant genus of the family LAMIACEAE that contains leonurine.'),
  'D031333': NtO(MH='Lycopus', MS='A plant genus of the family LAMIACEAE that contains rosmarinic acid and isopimarane diterpenoids and has been used in folk medicine for HYPERTHYROIDISM.'),
  'D031334': NtO(MH='Melissa', MS='A plant genus of the family LAMIACEAE. The common names of beebalm or lemonbalm are also used for MONARDA.'),
  'D031335': NtO(MH='Mentha pulegium', MS="A plant genus of the family LAMIACEAE that contains pulegone. Do not confuse with 'American false pennyroyal' (HEDEOMA)."),
  'D031336': NtO(MH='Hedeoma', MS='A plant genus of the family LAMIACEAE that is closely related to the European pennyroyal (MENTHA PULEGIUM).'),
  'D031337': NtO(MH='Mentha spicata', MS='A plant genus of the family LAMIACEAE having characteristic flavor.'),
  'D031338': NtO(MH='Monarda', MS='A plant genus of the family LAMIACEAE. The common names of beebalm or lemonbalm are also used for MELISSA. The common name of bergamot is also used for Citrus bergamia (CITRUS).'),
  'D031339': NtO(MH='Ocimum', MS='A plant genus of the family Lamiaceae which occurs in tropical and warm temperate regions worldwide. It includes the aromatic herb OCIMUM BASILICUM (Sweet Basil) and the MEDICINAL PLANT Ocimum tenuiflorum (Holy Basil).'),
  'D031341': NtO(MH='Origanum', MS='A plant genus of the family Lamiaceae that is the source of familiar seasonings, oregano and marjoram.'),
  'D031342': NtO(MH='Orthosiphon', MS='A plant genus of the family LAMIACEAE that contains pimarane-type diterpenes. Several species of Orthosiphon are also called Java tea.'),
  'D031343': NtO(MH='Perilla', MS='A plant genus of the family LAMIACEAE that is a source of perilla alcohol and the oil is rich in alpha-linolenic acid (alpha-18:3).'),
  'D031344': NtO(MH='Perilla frutescens', MS='A plant genus of the family LAMIACEAE that is an ingredient of Banxia Houpu (DRUGS, CHINESE HERBAL).'),
  'D031345': NtO(MH='Phlomis', MS='A plant genus of the family LAMIACEAE that contains phlorigidosides, iridoid glucosides and megastigmane glycosides.'),
  'D031346': NtO(MH='Plectranthus', MS='A plant genus of the family LAMIACEAE that contains eudesmane sesquiterpenes and antimicrobial abietane diterpenoids.'),
  'D031347': NtO(MH='Prunella', MS='A plant genus of the family LAMIACEAE. Members contain TRITERPENES. Heal all is another common name.'),
  'D031362': NtO(MH='Satureja', MS='A plant genus of the family LAMIACEAE used to flavor food.'),
  'D031363': NtO(MH='Scutellaria', MS='A plant genus of the family Lamiaceae used in folk medicine.'),
  'D031364': NtO(MH='Scutellaria baicalensis', MS='A plant species of the genus Scutellaria, family LAMIACEAE, that contains skullcap flavone and is used in CHINESE HERBAL DRUGS.'),
  'D031365': NtO(MH='Sideritis', MS='A plant genus of the family LAMIACEAE that contains isoscutellarein-7-O-(allosyl(1-2)glucoside).'),
  'D031366': NtO(MH='Stachys', MS='A plant genus of the family LAMIACEAE that has a characteristic odor.'),
  'D031367': NtO(MH='Teucrium', MS='A plant genus of the family Lamiaceae. Members contain teuscordonin. There have been reports of hepatotoxicity by this genus.'),
  'D031368': NtO(MH='Zoster Sine Herpete', MS='HERPES ZOSTER but without eruption of vesicles. Patients exhibit the characteristic pain minus the skin rash, sometimes making diagnosis difficult.'),
  'D031369': NtO(MH='Cryptocarya', MS='A plant genus of the family LAURACEAE. Members contain cryptofolione, caryachine, grandisin and other compounds. Some PEUMUS species have been reclassified as CRYPTOCARYA.'),
  'D031381': NtO(MH='Lecythidaceae', MS='A plant family of the order Lecythidales, subclass Dilleniidae, class Magnoliopsida.'),
  'D031382': NtO(MH='Barringtonia', MS='A plant genus of the family Lecythidaceae. Members contain nasimalun A and B (neo-clerodane diterpenoids).'),
  'D031383': NtO(MH='Bertholletia', MS='A plant genus of the family Lecythidaceae which is the source of edible Brazil nuts.'),
  'D031384': NtO(MH='Polygonatum', MS='A plant genus of the family Asparagaceae. Members contain benzoquinones, steroidal saponin, and ribosome-inactivating proteins.'),
  'D031401': NtO(MH='Maianthemum', MS='A plant genus of the family Asparagaceae. The cluster of flowers at the end of the stalk distinguishes this plant from "true" Solomon\'s seal (POLYGONATUM), which has flowers that grow along the stem.'),
  'D031402': NtO(MH='Alstroemeria', MS='A plant genus of the family Alstroemeriaceae. Members contain allergens, tuliposide A and tulipalin A.'),
  'D031403': NtO(MH='Anemarrhena', MS='A plant genus of the family Asparagaceae. Members contain anemarans (POLYSACCHARIDES), hinokiresinol, mangiferin (a xanthone), and timosaponin (a steroidal saponin).'),
  'D031404': NtO(MH='Camassia', MS='A plant genus of the family Asparagaceae that contains steroidal SAPONINS and should not be confused with death camas (ZIGADENUS).'),
  'D031405': NtO(MH='Convallaria', MS='A plant genus of the family Asparagaceae that contains CARDIAC GLYCOSIDES.'),
  'D031406': NtO(MH='Cordyline', MS='A plant genus of the family Asparagaceae that contains spirostanol & furostanol SAPONINS.'),
  'D031407': NtO(MH='Crinum', MS='A plant genus of the family Amaryllidaceae that contains ALKALOIDS.'),
  'D031408': NtO(MH='Curculigo', MS='A plant genus of the family Hypoxidaceae that contains curculin and cycloartane saponins.'),
  'D031409': NtO(MH='Dracaena', MS='A plant genus of the family Asparagaceae. The common name of "dragon\'s blood" is also used for CROTON and Daemonorops (ARECACEAE).'),
  'D031410': NtO(MH='Fritillaria', MS='A plant genus of the family LILIACEAE. Members of this genus produce imperialine, a steroidal alkaloid which acts at muscarinic receptors.'),
  'D031421': NtO(MH='Hemerocallis', MS='A plant genus of the family Xanthorrhoeaceae. Members contain steroidal saponins.'),
  'D031422': NtO(MH='Hosta', MS='A plant genus of the family Asparagaceae. Members contain steroidal saponins.'),
  'D031423': NtO(MH='Hyacinthus', MS='A plant genus of the family Hyacinthaceae (Asparagaceae). Do not confuse with water hyacinth (EICHHORNIA).'),
  'D031424': NtO(MH='Liriope Plant', MS='A plant genus of the family Asparagaceae. Members contain ruscogenin. Do not confuse with the Liriope jellyfish (CNIDARIA).'),
  'D031425': NtO(MH='Plasmodesmata', MS='Membrane-like channels of cytoplasm connecting adjacent plant cells. Plasmodesmata connect through pores in the CELL WALL and associate with the CYTOSKELETON machinery. They are essential for intercellular transport and communication.'),
  'D031426': NtO(MH='Lycoris', MS='A plant genus of the family Amaryllidaceae. Members contain radiatine, vittatine, haemanthamine, lycorenine, dihydrolycorine, lycorine, lycoricidinol and lycoricidine.'),
  'D031427': NtO(MH='Narcissus', MS='A plant genus of the family Amaryllidaceae. Members contain ungiminorine and LECTINS.'),
  'D031441': NtO(MH='Ophiopogon', MS='A plant genus of the family Asparagaceae. Members contain steroidal glycosides and provide an ingredient of shengmaisan (DRUGS, CHINESE HERBAL).'),
  'D031442': NtO(MH='Ornithogalum', MS='A plant genus of the family LILIACEAE that contains cholestane glycosides (CHOLESTANES).'),
  'D031443': NtO(MH='Ruscus', MS='A plant genus of the family Asparagaceae. This should not be confused with Broom or Scotch Broom (CYTISUS) or Desert Broom (BACCHARIS) or Spanish Broom (SPARTIUM) or Brome (BROMUS).'),
  'D031444': NtO(MH='Sansevieria', MS='A plant genus of the family Asparagaceae that contains steroidal SAPONINS.'),
  'D031445': NtO(MH='Scilla', MS='A plant genus of the family Asparagaceae. Members contain the cardiotonic PROSCILLARIDIN. The common name of squill is also used for DRIMIA.'),
  'D031446': NtO(MH='Trillium', MS='A plant genus of the family Melanthiaceae that is a short plant with a distinct whorl of 3 broad leaves.'),
  'D031447': NtO(MH='Tulipa', MS='A plant genus of the family LILIACEAE. Members contain tuliposides and tulipalins and have been associated with allergic contact dermatitis in florists.'),
  'D031448': NtO(MH='Drimia', MS='A plant genus of the family Asparagaceae that contains bufadienolides (BUFANOLIDES). The common name of squill is also used for SCILLA.'),
  'D031449': NtO(MH='Zigadenus', MS='A plant genus of the family Melanthiaceae that is fatally toxic to grazing animals. The name is similar to another plant called Camas (CAMASSIA).'),
  'D031450': NtO(MH='Gelsemium', MS='A plant genus of the family LOGANIACEAE (classified by some botanists as Gelsemiaceae). The sometimes used common name of trumpet flower is also used for DATURA.'),
  'D031451': NtO(MH='Strychnos', MS='A plant genus of the family LOGANIACEAE (classified by some botanists as Strychnaceae).'),
  'D031461': NtO(MH='Strychnos nux-vomica', MS='A plant genus of the genus STRYCHNOS, family LOGANIACEAE that is the source of STRYCHNINE.'),
  'D031482': NtO(MH='Cynomorium', MS='A plant genus of the family BALANOPHORACEAE. Members contain stigmastanes and ursanes (TRITERPENES) and lignan glucopyranosides (LIGNANS).'),
  'D031483': NtO(MH='Capparaceae', MS='A plant family of the order Capparales, subclass Dilleniidae, class Magnoliopsida, that are mostly herbs and shrubs growing in warm arid regions. Several produce GLUCOSINOLATES.'),
  'D031484': NtO(MH='Capparis', MS='A plant genus of the family CAPPARACEAE that contains mabinlin, a sweet protein.'),
  'D031485': NtO(MH='Cleome', MS='A plant genus of the family CAPPARACEAE that contains cleogynol and 15alpha-acetoxycleomblynol (dammaranes) and 1-epibrachyacarpone (a triterpene), and ISOTHIOCYANATES.'),
  'D031501': NtO(MH='Gleditsia', MS='A plant genus of the family FABACEAE that contains gleditsiosides (triterpenoid SAPONINS).'),
  'D031521': NtO(MH='Fractionation, Field Flow', MS='Separation of molecules and particles by a simultaneous action of carrier liquid flow and focusing field forces (electrical, sedimentation, or thermal), without a stationary phase.'),
  'D031541': NtO(MH='Fluorescence Resonance Energy Transfer', MS='A type of FLUORESCENCE SPECTROSCOPY using two FLUORESCENT DYES with overlapping emission and absorption spectra, which is used to indicate proximity of labeled molecules. This technique is useful for studying interactions of molecules and PROTEIN FOLDING.'),
  'D031561': NtO(MH='Lythrum', MS='A plant genus of the family LYTHRACEAE that contains ALKALOIDS.'),
  'D031562': NtO(MH='Cuphea', MS='A plant genus of the family LYTHRACEAE. Members contain lauric acid and cuphiin.'),
  'D031563': NtO(MH='Lagerstroemia', MS='A plant genus of the family LYTHRACEAE. Members contain lagertannin and have hypoglycemic effects.'),
  'D031564': NtO(MH='Lawsonia Plant', MS='A plant genus of the family LYTHRACEAE that is the source of henna and has cytotoxic activity.'),
  'D031565': NtO(MH='Woodfordia', MS='A plant genus of the family LYTHRACEAE. Members contain woodfordins.'),
  'D031566': NtO(MH='Magnolia', MS='A plant genus of the family MAGNOLIACEAE. The germacranolide sesquiterpene lactones costunolide, parthenolide, and costunolide diepoxide have been isolated from the leaves. Bark contains honokiol and magnolol. Parts are an ingredient of Banxia Houpo Tang.'),
  'D031567': NtO(MH='Liriodendron', MS='A plant genus of the family MAGNOLIACEAE. Members include hardwood trees of eastern North America with distinct large tuliplike flowers.'),
  'D031581': NtO(MH='Banisteriopsis', MS='A plant genus of the family MALPIGHIACEAE which includes an Amazonian psychoactive plant that contains the beta-carboline harmine and N,N-dimethyltryptamine.'),
  'D031582': NtO(MH='Galphimia', MS="A plant genus of the family MALPIGHIACEAE. G. glauca is the source of 'Galphimia glauca extract' and contains 'galphimine B' and tetragalloylquinic acid."),
  'D031583': NtO(MH='Althaea', MS='A plant genus of the family MALVACEAE known for mucilaginous roots. The common names of hollyhock and mallow are also used for other genera of MALVACEAE.'),
  'D031584': NtO(MH='Hibiscus', MS='A plant genus of the family MALVACEAE. Members contain CITRIC ACID; MALATES; ANTHOCYANINS; FLAVONOIDS; GLYCOSIDES; DIETARY FIBER; and LIGNANS. Hibiscus sabdariffa is common constituent of HERBAL TEAS. Hibiscus cannabinus is a source of hemp fiber for TEXTILES.'),
  'D031585': NtO(MH='Melia', MS='A plant genus of the family MELIACEAE. Members contain meliavolkinin, melianin C and limonoids.'),
  'D031586': NtO(MH='Melia azedarach', MS='A plant species of the genus MELIA, family MELIACEAE, which is toxic to insects. The name is very similar to Melia azadirachta (AZADIRACHTA).'),
  'D031587': NtO(MH='Cedrela', MS='A plant genus of the family MELIACEAE. Members contain cedrelanolide.'),
  'D031601': NtO(MH='Azadirachta', MS='A plant genus of the family MELIACEAE. Members contain azadirachtin A (a limonoid commonly referred to as azadirachtin) and other TRITERPENES. They have been used in PESTICIDES. The old name of Melia azadirachta is very similar to a related plant, MELIA AZEDARACH.'),
  'D031602': NtO(MH='Aglaia', MS='A plant genus of the family MELIACEAE. Members contain lariciresinol, pregnanes, insecticidal rocaglamide derivatives and other compounds.'),
  'D031603': NtO(MH='Menispermum', MS='A plant genus of the family MENISPERMACEAE. Members contain dauricine and other ALKALOIDS.'),
  'D031604': NtO(MH='Cyclea', MS='A plant genus of the family MENISPERMACEAE. Members contain bisbenzylisoquinoline alkaloids.'),
  'D031605': NtO(MH='Cocculus', MS='A plant genus of the family MENISPERMACEAE. Members contain sinococuline, coccuvine and other ALKALOIDS.'),
  'D031606': NtO(MH='Cissampelos', MS='A plant genus of the family MENISPERMACEAE. Members contain eletefine (a stephaoxocane alkaloid) and tropoloisoquinoline and protoberberine ALKALOIDS.'),
  'D031607': NtO(MH='Stephania', MS='A plant genus of the family MENISPERMACEAE. Members contain cycleanine.'),
  'D031608': NtO(MH='Area Postrema', MS='A small, rounded eminence on each side of the FOURTH VENTRICLE, which receives nerve fibers from the SOLITARY NUCLEUS; SPINAL CORD; and adjacent areas of the MEDULLA OBLONGATA. The area postrema lies outside the BLOOD-BRAIN BARRIER and its functions include acting as an emetic chemoreceptor.'),
  'D031609': NtO(MH='Stephania tetrandra', MS='A plant species of the genus STEPHANIA, family MENISPERMACEAE, that contains tetrandrine and bisbenzylisoquinoline alkaloids.'),
  'D031610': NtO(MH='Tinospora', MS='A plant genus of the family MENISPERMACEAE. Members have been used in AYURVEDIC MEDICINE. Hypoglycemic effect has been reported.'),
  'D031611': NtO(MH='Peumus', MS='A plant genus of the family MONIMIACEAE. Members contain boldine ALKALOIDS. Some PEUMUS species have been reclassified as CRYPTOCARYA.'),
  'D031621': NtO(MH='Antiaris', MS='A plant genus of the family MORACEAE. Members have been used as an arrow poison.'),
  'D031622': NtO(MH='Artocarpus', MS='A plant genus of the family MORACEAE. Puag-haad extract, from A. lakoocha, contains STILBENES and related 4-substituted RESORCINOLS.'),
  'D031623': NtO(MH='Broussonetia', MS='A plant genus of the family MORACEAE. Members contain PYRROLIDINES.'),
  'D031624': NtO(MH='Maclura', MS='A plant genus of the family MORACEAE. Members contain maclurin, antifungal chalcones, and other compounds.'),
  'D031625': NtO(MH='Morus', MS='A plant genus of the family MORACEAE that is widely planted for shade.'),
  'D031641': NtO(MH='Myrica', MS='A plant genus of the family MYRICACEAE. Members contain myricanol. The common name of bayberry is similar to the name barberry which is used for BERBERIS and MAHONIA.'),
  'D031642': NtO(MH='Myoporaceae', MS='A plant family of the order Scrophulariales, subclass Asteridae, class Magnoliopsida.'),
  'D031643': NtO(MH='Myoporum', MS='A plant genus of the family MYOPORACEAE. Members contain dehydrongaione, deisopropylngaione, dehydromyodesmone and other furanoid sesquiterpene essential oils. It is the source of Ngaio oil and ngaione.'),
  'D031644': NtO(MH='Ardisia', MS='A plant genus of the family MYRSINACEAE. Members contain ardisiacrispins (oleanane triterpenoid saponins), ardicrenin, and cyclamiretin.'),
  'D031645': NtO(MH='Embelia', MS='A plant genus of the family MYRSINACEAE. Members contain embelin.'),
  'D031646': NtO(MH='Feijoa', MS='A plant genus of the family MYRTACEAE that is cultivated for the edible fruit, which is different from the ordinary guava (PSIDIUM).'),
  'D031647': NtO(MH='Psidium', MS='A plant genus of the family MYRTACEAE that bears an edible fruit and contains guavin B and quercetin glycosides.'),
  'D031648': NtO(MH='Kunzea', MS='A plant genus of the family MYRTACEAE. The common name of tea tree more often refers to MELALEUCA or LEPTOSPERMUM.'),
  'D031649': NtO(MH='Leptospermum', MS='A plant genus of the family MYRTACEAE. The common name of tea tree is also used for MELALEUCA and KUNZEA.'),
  'D031650': NtO(MH='Melaleuca', MS='A plant genus of the family MYRTACEAE. M. alternifolia foliage is a source of TEA TREE OIL. The common name of tea tree also refers to LEPTOSPERMUM or KUNZEA. M. vindifolia is a source of niaouli oil. M. cajuputi and M. leucadendra are sources of cajuput oil.'),
  'D031651': NtO(MH='Myrtus', MS='A plant genus of the family MYRTACEAE. Members contain PHYTOHEMAGGLUTININS.'),
  'D031652': NtO(MH='Nelumbonaceae', MS='A plant family of the order Nymphaeales, subclass Magnoliidae, class Magnoliopsida. They are aquatic plants.'),
  'D031653': NtO(MH='Nelumbo', MS='A plant genus of the family NELUMBONACEAE. The common name lotus is also used for other plants such as the WATER-LILY, which they resemble, and the unrelated genus Lotus (LOTUS). Nelumbo produce the edible lotus nuts.'),
  'D031654': NtO(MH='Nuphar', MS='A plant genus of the family NYMPHAEACEAE. Members contain sesquiterpene thioalkaloids.'),
  'D031655': NtO(MH='Nyssa', MS='A plant genus of the family Nyssaceae. Named for the water nymph from classical mythology, these species grow in moist habitats and are tolerant of flooding.'),
  'D031656': NtO(MH='Olacaceae', MS='A small plant family of the order Santalales, subclass Rosidae, class Magnoliopsida.'),
  'D031657': NtO(MH='Forsythia', MS='A plant genus of the family OLEACEAE. Members contain suspensaside.'),
  'D031658': NtO(MH='Olea', MS='A plant genus of the family Oleaceae. The olive fruit is the source of OLIVE OIL.'),
  'D031659': NtO(MH='Ligustrum', MS='A plant genus of the family OLEACEAE. Oleuropein, a phenolic secoiridoid glycoside, is found in the leaves.'),
  'D031660': NtO(MH='Jasminum', MS='A plant genus of the family OLEACEAE. Members contain secoiridoid glucosides and this is the source of oil of jasmine.'),
  'D031661': NtO(MH='Fraxinus', MS='A plant genus of the family OLEACEAE. Members contain secoiridoid glucosides.'),
  'D031662': NtO(MH='Syringa', MS='A plant genus of the family OLEACEAE. Oleuropein has been identified in the stem bark.'),
  'D031663': NtO(MH='Clarkia', MS='A plant genus, of the family ONAGRACEAE, which is the subject of genetic studies. The floral aroma is attributed to benzenoid esters and benzyl acetate.'),
  'D031664': NtO(MH='Oenothera', MS='A plant genus of the family ONAGRACEAE. Members contain oenotheins.'),
  'D031665': NtO(MH='Oenothera biennis', MS='A plant species, of the genus OENOTHERA, family ONAGRACEAE, that is the source of evening primrose oil.'),
  'D031666': NtO(MH='Epilobium', MS='A plant genus of the family ONAGRACEAE. The common name of fireweed is also used with other plants.'),
  'D031667': NtO(MH='Dendrobium', MS='A plant genus of the family ORCHIDACEAE that contains dihydroayapin (COUMARINS) and phenanthraquinones.'),
  'D031668': NtO(MH='Gastrodia', MS='A plant genus of the family ORCHIDACEAE which depends on the fungus Armillaria mellea to complete its life cycle. It is an ingredient of Zhenxuanyin (DRUGS, CHINESE HERBAL).'),
  'D031669': NtO(MH='Vanilla', MS='A plant genus of the family ORCHIDACEAE that is the source of the familiar flavoring used in foods and medicines (FLAVORING AGENTS).'),
  'D031670': NtO(MH='Orobanchaceae', MS='The broom-rape plant family of the order Lamiales.'),
  'D031671': NtO(MH='Cistanche', MS='A plant genus of the family OROBANCHACEAE. Members contain phenylethanoid glycosides.'),
  'D031672': NtO(MH='Orobanche', MS="A plant genus of the family OROBANCHACEAE. Lacking chlorophyll, they are nonphotosynthetic parasitic plants. The common name is similar to Broom or Scotch Broom (CYTISUS) or Butcher's Broom (RUSCUS) or Desert Broom (BACCHARIS) or Spanish Broom (SPARTIUM) or Brome (BROMUS)."),
  'D031673': NtO(MH='Pandanaceae', MS='A plant family of the order Pandanales, subclass Arecidae, class Liliopsida (monocotyledons).'),
  'D031674': NtO(MH='Argemone', MS='A plant genus of the family PAPAVERACEAE that contains isoquinoline alkaloids.'),
  'D031675': NtO(MH='Eschscholzia', MS='A plant genus of the family PAPAVERACEAE that contains benzo[c]phenanthridine alkaloids.'),
  'D031681': NtO(MH='Sanguinaria', MS='A plant genus of the family PAPAVERACEAE, a species of which is the source of sanguinarine extract used in MOUTHWASHES.'),
  'D031683': NtO(MH='Passifloraceae', MS='A plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida that are herbaceous or woody vines, shrubs, and trees, mostly of warm regions. Many have tendrils in leaf axils. Leaves are alternate. Flowers have 3-5 sepals, petals and stamens. Nearly all species have seeds that bear a fleshy appendage called an aril.'),
  'D031684': NtO(MH='Harpagophytum', MS='A plant genus of the family PEDALIACEAE. Members contain harpagoside.'),
  'D031685': NtO(MH='Sesamum', MS='A plant genus of the family PEDALIACEAE that is the source of the edible seed and SESAME OIL.'),
  'D031686': NtO(MH='Tagetes', MS='A plant genus of the family ASTERACEAE. The common name of marigold is also used for CALENDULA.'),
  'D031701': NtO(MH='Piper', MS='A plant genus of the family PIPERACEAE that includes species used for spicy and stimulating qualities.'),
  'D031702': NtO(MH='Agropyron', MS='A plant genus of the family POACEAE. The common name of wheatgrass is also used for other plants in the family.'),
  'D031703': NtO(MH='Agrostis', MS='A plant genus of the family POACEAE.'),
  'D031721': NtO(MH='Piper betle', MS='A plant genus of the family PIPERACEAE that is indigenous in the Indian Malay region and cultivated in Madagascar, and the West Indies. It contains chavibetol, chavicol and cadinene. The leaf is chewed as a stimulant, antiseptic and sialogogue. The common name of betel is also used for ARECA.'),
  'D031722': NtO(MH='Andropogon', MS='A plant genus of the family POACEAE. The common name of bluestem is also used for other plants in this family. Andropogon nardus has been reclassified as CYMBOPOGON nardus and Andropogon zizanioides as Chrysopogon zizanioides.'),
  'D031723': NtO(MH='Bambusa', MS='A plant genus of the family POACEAE. Young shoots are eaten in Asian foods while the stiff mature stems are used for construction of many things. The common name of bamboo is also used for other genera of Poaceae including Phyllostachys, SASA, and Dendrocalamus.'),
  'D031724': NtO(MH='Bromus', MS="A plant genus of the family POACEAE. The name is similar to Broom or Scotch Broom (CYTISUS) or Butcher's Broom (RUSCUS) or Desert Broom (BACCHARIS) or Spanish Broom (SPARTIUM)."),
  'D031741': NtO(MH='Cenchrus', MS='A plant genus of the family POACEAE. The common name of buffelgrass is also used for PENNISETUM.'),
  'D031742': NtO(MH='Coix', MS='A plant genus of the family POACEAE. The seed is used in folk medicine (DRUGS, CHINESE HERBAL).'),
  'D031743': NtO(MH='Cymbopogon', MS='A plant genus of the family POACEAE which is a source of citronella oil and lemongrass oil.'),
  'D031744': NtO(MH='Cynodon', MS='A plant genus of the family POACEAE that is considered a lawn grass by some and a weed by others. It contains allergen Cyn d 7.'),
  'D031745': NtO(MH='Dactylis', MS='A plant genus of the family POACEAE that contains allergen Dac g I.'),
  'D031746': NtO(MH='Digitaria', MS='A plant genus of the family POACEAE that many people consider to be a weed.'),
  'D031747': NtO(MH='Echinochloa', MS='A plant genus of the family POACEAE that is grown mainly as a hay crop.'),
  'D031748': NtO(MH='Eleusine', MS='A plant genus of the family POACEAE. Finger millet or raggee (E. coracana) is an important food grain in southern Asia and parts of Africa.'),
  'D031749': NtO(MH='Elymus', MS='A plant genus of the family POACEAE. The common name of wild rye is used with some other grasses.'),
  'D031750': NtO(MH='Eragrostis', MS='A plant genus of the family POACEAE.'),
  'D031751': NtO(MH='Festuca', MS='A plant genus of the family POACEAE. The common name of fescue is also used with some other grasses.'),
  'D031752': NtO(MH='Holcus', MS='A plant genus of the family POACEAE that contains Hol l 1 and Hol l 5 allergens.'),
  'D031781': NtO(MH='Paspalum', MS='A plant genus of the family POACEAE that is used for forage.'),
  'D031782': NtO(MH='Pennisetum', MS='A plant genus of the family POACEAE. The seed is one of the millets used in EDIBLE GRAIN. It contains vitexin. The common name of buffelgrass is also used for CENCHRUS.'),
  'D031783': NtO(MH='Phalaris', MS='A plant genus of the family POACEAE.'),
  'D031784': NtO(MH='Phleum', MS='A plant genus of the family POACEAE that contains the Phl p 4 allergen.'),
  'D031785': NtO(MH='Poa', MS='A plant genus of the family POACEAE that contains the Poa p Ia allergen and allergen C KBGP.'),
  'D031786': NtO(MH='Saccharum', MS='A plant genus of the family POACEAE widely cultivated in the tropics for the sweet cane that is processed into sugar.'),
  'D031787': NtO(MH='Sasa', MS='A plant genus of the family POACEAE. Folin is the water-soluble extract from Sasa albomarginata. Sasa kurinensis is an ingredient of Sho-ju-sen, a Japanese herbal medicine.'),
  'D031801': NtO(MH='Polygala', MS='A plant genus of the family POLYGALACEAE that contains onjisaponins, xanthones, pyrones, and benzophenones. The name is similar to other snakeroots such as ASARUM; SANICULA; ARISTOLOCHIA; AGERATINA; and others.'),
  'D031802': NtO(MH='Securidaca', MS='A plant genus of the family POLYGALACEAE that contains securiosides and indole alkaloids.'),
  'D031803': NtO(MH='Eriogonum', MS='A plant genus of the family POLYGONACEAE.'),
  'D031804': NtO(MH='Rumex', MS='A plant genus of the family POLYGONACEAE that contains patientosides and other naphthalene glycosides.'),
  'D031805': NtO(MH='Pontederiaceae', MS='A plant family of the order Liliales, subclass Liliidae, class Liliopsida (monocotyledons). Most species are perennials, native primarily to tropical America. They have creeping rootstocks, fibrous roots, and leaves in clusters at the base of the plant or borne on branched stems. The fruit is a capsule containing many seeds, or a one-seeded winged structure.'),
  'D031806': NtO(MH='Eichhornia', MS='A plant genus of the family PONTEDERIACEAE that is used as a biological filter for treating wastewater.'),
  'D031821': NtO(MH='Potamogetonaceae', MS='A plant family of the order Najadales, subclass Alismatidae, class Liliopsida (monocotyledons).'),
  'D031822': NtO(MH='Portulaca', MS='A plant genus of the family PORTULACACEAE.'),
  'D031823': NtO(MH='Anagallis', MS='A plant genus of the family PRIMULACEAE that contains anagallosides, which are molluscacidal triterpenoid saponins.'),
  'D031824': NtO(MH='Cyclamen', MS='A plant genus of the family PRIMULACEAE that contains triterpenoid saponins.'),
  'D031825': NtO(MH='Primula', MS='A plant genus of the family PRIMULACEAE. It can cause CONTACT DERMATITIS. SAPONINS have been identified in the root.'),
  'D031827': NtO(MH='Pyrolaceae', MS='A plant family of the order Ericales, subclass Dilleniidae, class Magnoliopsida.'),
  'D031828': NtO(MH='Pyrola', MS='A plant genus of the family PYROLACEAE. The common name of wintergreen is more often used for GAULTHERIA.'),
  'D031841': NtO(MH='Aconitum', MS='A plant genus of the family RANUNCULACEAE. Members contain a number of diterpenoid alkaloids including: aconitans, hypaconitine, ACONITINE, jesaconitine, ignavine, napelline, and mesaconitine. The common name of Wolfbane is similar to the common name for ARNICA.'),
  'D031842': NtO(MH='Actaea', MS='A plant genus of the family RANUNCULACEAE.'),
  'D031843': NtO(MH='Adonis', MS='A plant genus of the family RANUNCULACEAE. Members contain cardenolide oligoglycosides such as adoniside, adonisidum and alepposide.'),
  'D031844': NtO(MH='Nigella', MS='A plant genus of the family RANUNCULACEAE.'),
  'D031845': NtO(MH='Hajdu-Cheney Syndrome', MS='Rare, autosomal dominant syndrome characterized by ACRO-OSTEOLYSIS, generalized OSTEOPOROSIS, and skull deformations.'),
  'D031861': NtO(MH='Nigella damascena', MS='A plant genus of the family RANUNCULACEAE. The name is similar to giant fennel (FERULA), true fennel (FOENICULUM), and dog fennel (ANTHEMIS).'),
  'D031881': NtO(MH='Nigella sativa', MS='A plant genus of the family RANUNCULACEAE that contains alpha-hederin, a triterpene saponin in the seeds, and is the source of black seed oil.'),
  'D031882': NtO(MH='Delphinium', MS='A plant genus of the family RANUNCULACEAE. Members contain ACONITINE and other diterpenoid alkaloids.'),
  'D031883': NtO(MH='Anemone', MS='A plant genus of the family RANUNCULACEAE that contains triterpene saponins. The root of Anemone raddeana is the source of a Chinese folk medicine, zhu jie xian fu. The common name of liverwort is also used with other plants. This genus is unrelated to SEA ANEMONES.'),
  'D031884': NtO(MH='Aquilegia', MS='A plant genus of the family RANUNCULACEAE that contains aquiledine, isoaquiledine and cycloartane-type glycosides.'),
  'D031885': NtO(MH='Cimicifuga', MS='A plant genus of the family RANUNCULACEAE that contains triterpenoid saponins. Remifemin from C. racemosa is used to suppress LUTEINIZING HORMONE. It is reclassified by some to ACTAEA. The common name of black snakeroot is also used with ASARUM and SANICULA.'),
  'D031901': NtO(MH='Gestational Trophoblastic Disease', MS='A group of diseases arising from pregnancy that are commonly associated with hyperplasia of trophoblasts (TROPHOBLAST) and markedly elevated human CHORIONIC GONADOTROPIN. They include HYDATIDIFORM MOLE, invasive mole (HYDATIDIFORM MOLE, INVASIVE), placental-site trophoblastic tumor (TROPHOBLASTIC TUMOR, PLACENTAL SITE), and CHORIOCARCINOMA. These neoplasms have varying propensities for invasion and spread.'),
  'D031941': NtO(MH='Hemorrhagic Septicemia, Viral', MS='A systemic infection of various salmonid and a few nonsalmonid fishes caused by Viral hemorrhagic septicemia virus (see NOVIRHABDOVIRUS),'),
  'D031942': NtO(MH='Cynara', MS='A plant genus of the family ASTERACEAE.'),
  'D031943': NtO(MH='Cynara scolymus', MS='A plant species of the genus CYNARA, family ASTERACEAE. The flower bud is the familiar artichoke eaten as a vegetable.'),
  'D031944': NtO(MH='Dahlia', MS='A plant genus of the family ASTERACEAE that contains antifungal plant defensin.'),
  'D031945': NtO(MH='Coreopsis', MS='A plant genus of the family ASTERACEAE that contains phenyl propanoids.'),
  'D031946': NtO(MH='Pulsatilla', MS='A plant genus of the family RANUNCULACEAE. Members contain cernuosides and other oleanane and hederagenin saponins.'),
  'D031947': NtO(MH='Ranunculus', MS='A plant genus of the family RANUNCULACEAE that contains protoanemonin, anemonin, and ranunculin.'),
  'D031948': NtO(MH='Semiaquilegia', MS='A plant genus of the family RANUNCULACEAE that contains griffonilide, lithospermoside, and magnoflorine.'),
  'D031949': NtO(MH='Thalictrum', MS='A plant genus of the family RANUNCULACEAE. Members contain isoquinoline alkaloids and triterpene glycosides.'),
  'D031950': NtO(MH='Xanthorhiza', MS='A plant genus of the family RANUNCULACEAE. Members contain isoquinoline alkaloids.'),
  'D031951': NtO(MH='Resedaceae', MS='A plant family of the order Capparales, subclass Dilleniidae, class Magnoliopsida. It is a small family of herbs and shrubs. Some produce GLUCOSINOLATES.'),
  'D031952': NtO(MH='Ceanothus', MS='A plant genus of the family RHAMNACEAE. Root nodules host the Frankia (ACTINOMYCETES) nitrogen-fixing bacteria.'),
  'D031953': NtO(MH='Colubrina', MS='A plant genus of the family RHAMNACEAE. Members contain antifungal jujubogenin saponins.'),
  'D031954': NtO(MH='Choriocarcinoma, Non-gestational', MS='A highly malignant CHORIOCARCINOMA derived from the non-placental origin such as the totipotent cells in the TESTIS, the OVARY, and the PINEAL GLAND. It produces high levels of CHORIONIC GONADOTROPIN and can metastasize widely through the bloodstream to the lungs, brain, liver, bone, and other viscera by the time of diagnosis.'),
  'D031955': NtO(MH='Rhamnus', MS='A plant genus of the family RHAMNACEAE. This genus is often called buckthorn but should not be confused with HIPPOPHAE or KARWINSKIA. Some RHAMNUS species have been reclassified into this genus. F. purshiana bark is cascara sagrada. Members contain frangulanin, frangulin, and anthraquinones such as EMODIN.'),
  'D031956': NtO(MH='Karwinskia', MS='A plant genus of the family RHAMNACEAE that contains tullidinol (toxin T-544) and peroxisomicine (toxin T-514). It is sometimes called buckthorn but should not be confused with other plants called that.'),
  'D031957': NtO(MH='Ziziphus', MS='A plant genus of the family RHAMNACEAE. Members contain nummularogenin (a spirostane) and is the source of an edible fruit.'),
  'D031961': NtO(MH='Connaraceae', MS='A plant family of the order ROSALES, subclass Rosidae, class Magnoliopsida that are tropical plants. The flowers are usually bisexual and have 5 sepals and petals; 5 or 10 stamens; and 1, 4, or usually 5 separate, one-chambered, female ovule-bearing structures (carpels).'),
  'D031962': NtO(MH='Kalanchoe', MS='A plant genus of the family CRASSULACEAE. Members contain bryophyllins (also called bryotoxins) which are bufadienolides (BUFANOLIDES) that have insecticidal activity.'),
  'D031963': NtO(MH='Sedum', MS='A plant genus of the family CRASSULACEAE. Some species in this genus are called stonecrop which is also a common name for RHODIOLA.'),
  'D031964': NtO(MH='Grossulariaceae', MS='A plant family of the order ROSALES, subclass Rosidae, class Magnoliopsida. They are deciduous woody shrubs with alternate leaves. The flowers are in terminal racemes with 5 petals and 5 stamens.'),
  'D031965': NtO(MH='Ribes', MS='Also known as currant, ribes is a member of the family GROSSULARIACEAE. Oil extracted from the seeds contain GAMMA-LINOLENIC ACID.'),
  'D031966': NtO(MH='Hydrangea', MS='A plant genus of the family HYDRANGEACEAE. Members contain hydrangenol, thunberginols, hydramacrosides A and B, and secoiridoid glucosides.'),
  'D031981': NtO(MH='Agrimonia', MS='A plant genus of the family ROSACEAE that has been used in folk treatment of diabetes. Members contain agrimoniin (TANNINS).'),
  'D031982': NtO(MH='Alchemilla', MS='A plant genus of the family ROSACEAE. Members contain esculetin and QUERCETIN.'),
  'D031983': NtO(MH='Eriobotrya', MS='A plant genus of the family ROSACEAE that is the source of an edible fruit. Members contain TRITERPENES.'),
  'D031984': NtO(MH='Filipendula', MS='A plant genus of the family ROSACEAE. Members contain TANNINS.'),
  'D031985': NtO(MH='Fragaria', MS='A plant genus of the family ROSACEAE known for the edible fruit.'),
  'D031986': NtO(MH='Geum', MS='A plant genus of the family ROSACEAE. Members contain TRITERPENES.'),
  'D031987': NtO(MH='Potentilla', MS='A plant genus of the family ROSACEAE. Members contain procyanidins and TANNINS.'),
  'D031989': NtO(MH='Pyrus', MS='A plant genus of the family ROSACEAE known for the edible fruit.'),
  'D031990': NtO(MH='Quillaja', MS='A plant genus of the family ROSACEAE whose members produce SAPONINS.'),
  'D031991': NtO(MH='Pyracantha', MS='A plant genus of the family ROSACEAE widely cultivated as a prickly hedge with bright red berries. Members contain pyracrenic acid (a lupane triterpenoid).'),
  'D031992': NtO(MH='Spiraea', MS='A plant genus of the family ROSACEAE. Members contain prunioside, spiramine and other diterpene alkaloids.'),
  'D031993': NtO(MH='Sanguisorba', MS='A plant genus of the family ROSACEAE. Members contain sanguiin.'),
  'D031994': NtO(MH='Sorbus', MS='A plant genus of the family ROSACEAE.'),
  'D031995': NtO(MH='Saxifragaceae', MS='The saxifrage plant family of the order ROSALES, subclass Rosidae, class Magnoliopsida. The leaves are alternate and sometimes deeply lobed or form rosettes. The flowers have both male and female parts and 4 or 5 sepals and petals; they are usually in branched clusters. The fruit is a capsule with many seeds.'),
  'D031996': NtO(MH='Heuchera', MS='A plant genus of the family SAXIFRAGACEAE with dark green round or heart-shaped leaves and tiny flowers.'),
  'D032022': NtO(MH='Delegation, Professional', MS='The process of assigning duties to a subordinate with lesser qualifications.'),
  'D032023': NtO(MH='Integrons', MS='DNA elements that include the component genes and insertion site for a site-specific recombination system that enables them to capture mobile gene cassettes.'),
  'D032024': NtO(MH='Personnel Delegation', MS='To entrust to the care or management of another, to transfer or to assign tasks within an organizational or administrative unit or structure'),
  'D032041': NtO(MH='Nested Genes', MS='Genes whose entire sequences are contained within other genes.'),
  'D032061': NtO(MH='Galium', MS='A genus of herbaceous annual and perennial plants in the family RUBIACEAE.'),
  'D032062': NtO(MH='Gardenia', MS='A plant genus of the family RUBIACEAE. Members contain genepin, from which geniposide is obtained for use as a crosslinking agent in ADHESIVES, and 3-caffeoyl-4-sinapoylquinic acid.'),
  'D032063': NtO(MH='Hamelia', MS='A plant genus of the family RUBIACEAE that has a name very similar to another plant genus, HAMAMELIS.'),
  'D032064': NtO(MH='Hedyotis', MS='A plant genus of the family RUBIACEAE. Members contain anthraquinones and iridoids. H. diffusa is used in DRUGS, CHINESE HERBAL.'),
  'D032065': NtO(MH='Mitragyna', MS='A plant genus of the family RUBIACEAE. Members contain antimalarial (ANTIMALARIALS) and analgesic (ANALGESICS) indole alkaloids.'),
  'D032066': NtO(MH='Morinda', MS='A plant genus of the family RUBIACEAE. Members contain iridoid glycosides and ANTHRAQUINONES.'),
  'D032067': NtO(MH='Oldenlandia', MS='A plant genus of the family RUBIACEAE. Some species are used as an ingredient in Chinese and African traditional medicines. Members contain kalata B1, a macrocyclic peptide.'),
  'D032068': NtO(MH='Pausinystalia', MS='A plant genus of the family RUBIACEAE. Members produce YOHIMBINE.'),
  'D032069': NtO(MH='Psychotria', MS='A plant genus of the family RUBIACEAE. Members contain psychotridine and brachycerine (indole alkaloids).'),
  'D032070': NtO(MH='Uncaria', MS='A plant genus of the family RUBIACEAE. Members contain uncarine and other cytotoxic and hypotensive oxindole alkaloids.'),
  'D032081': NtO(MH='Casimiroa', MS='A plant genus of the family RUTACEAE. The fruit is edible and the seed has cardiovascular activity.'),
  'D032082': NtO(MH='Citrus aurantiifolia', MS='A plant species of the genus CITRUS, family RUTACEAE that provides the familiar lime fruit. Its common name of lime is similar to the limetree (TILIA).'),
  'D032083': NtO(MH='Citrus paradisi', MS='A plant species of the genus CITRUS, family RUTACEAE that produces the familiar grapefruit. There is evidence that grapefruit inhibits CYTOCHROME P-450 CYP3A4, resulting in delayed metabolism and higher blood levels of a variety of drugs.'),
  'D032084': NtO(MH='Citrus sinensis', MS='A plant species of the genus CITRUS, family RUTACEAE that provides the familiar orange fruit which is also a source of orange oil.'),
  'D032085': NtO(MH='Isochores', MS='Large regions of the GENOME that contain local similarities in BASE COMPOSITION.'),
  'D032086': NtO(MH='Evodia', MS='A plant genus of the family RUTACEAE which is used in Chinese medicine (DRUGS, CHINESE HERBAL). Evodiamine and other quinazoline alkaloids (QUINAZOLINES) are obtained from the fruit of E. ruticarpa.'),
  'D032101': NtO(MH='Murraya', MS='A plant genus of the family RUTACEAE. Members contain murrayanine, koenine, isomahanine, kwangsine, siamenol, murrayafoline A, murrayaquinone A and other cytotoxic carbazolequinones.'),
  'D032102': NtO(MH='Phellodendron', MS='A plant genus of the family RUTACEAE. Members contain BERBERINE, indolopyridoquinazoline and other ALKALOIDS and limonoids.'),
  'D032103': NtO(MH='Pilocarpus', MS='A plant genus of the family RUTACEAE that is the natural source of PILOCARPINE.'),
  'D032104': NtO(MH='Poncirus', MS='A plant genus of the family RUTACEAE. Members contain ponfolin, a coumarin (COUMARINS).'),
  'D032105': NtO(MH='Ruta', MS='A plant genus of the family RUTACEAE. Members contain quinoline alkaloids.'),
  'D032106': NtO(MH='Zanthoxylum', MS='A plant genus of the family RUTACEAE. Some members of Zanthoxylum are reclassified from ELEUTHEROCOCCUS, Melicope, and EVODIA. The twigs are used as dental brushing sticks (TOOTHBRUSHING).  Most plants that are called Fagara have been reclassified as Zanthoxylum, however some Fagara were reclassified  to MELICOPE (also in the Rutacea family) or to GLEDITSIA (a genus in the FABACEAE family).'),
  'D032107': NtO(MH='Populus', MS='A plant genus of the family SALICACEAE. Balm of Gilead is a common name used for P. candicans, or P. gileadensis, or P. jackii, and sometimes also used for ABIES BALSAMEA or for COMMIPHORA.'),
  'D032108': NtO(MH='Salix', MS='A plant genus of the family Salicaceae. Members contain salicin, which yields SALICYLIC ACID.'),
  'D032121': NtO(MH='Salvadoraceae', MS='A plant family of the order Celastrales, subclass Rosidae, class Magnoliopsida, a small family growing in the tropics. Members contain piperidine alkaloids and GLUCOSINOLATES.'),
  'D032122': NtO(MH='Pyrularia', MS="A plant genus of the family SANTALACEAE. Members contain thionin toxin, which shouldn't be confused with thionine."),
  'D032123': NtO(MH='Santalum', MS='A plant genus of the family SANTALACEAE which is the source of sandalwood oil.'),
  'D032124': NtO(MH='Blighia', MS='A plant genus of the family SAPINDACEAE. Members contain hypoglycins and ackee oil.'),
  'D032125': NtO(MH='Litchi', MS='A plant genus of the family SAPINDACEAE that bears sweet fruits. Unusual fatty acids such as cyclopropanoic fatty acids have been identified in the seeds.'),
  'D032126': NtO(MH='Paullinia', MS='A plant genus of the family SAPINDACEAE. The seed of P. cupana is the source of guarana powder which contains 4% CAFFEINE.'),
  'D032127': NtO(MH='Sapindus', MS='A plant genus of the family SAPINDACEAE that contain SAPONINS.'),
  'D032128': NtO(MH='Madhuca', MS='A plant genus of the family SAPOTACEAE that is the source of mahua oil. Members contain oleanene triterpenoid saponins.'),
  'D032129': NtO(MH='Manilkara', MS='A plant genus of the family SAPOTACEAE that bears sweet fruit.'),
  'D032130': NtO(MH='Mimusops', MS='A plant genus of the family SAPOTACEAE. Members contain triterpenoid saponins. Latex from bark incisions is processed into gutta balata.'),
  'D032131': NtO(MH='Palaquium', MS='A plant genus of the family SAPOTACEAE. Latex from bark incisions is processed into GUTTA-PERCHA.'),
  'D032132': NtO(MH='Pouteria', MS='A plant genus of the family SAPOTACEAE. Members contain triterpenes. Some species in this genus are called mamey which is also a common name for MAMMEA.'),
  'D032133': NtO(MH='Synsepalum', MS='A plant genus of the family SAPOTACEAE. Members contain miraculin (also spelled mieraculin), a sweet protein.'),
  'D032134': NtO(MH='Saururaceae', MS='A plant family of the order Piperales, subclass Magnoliidae class Magnoliopsida. Members contain sauristolactam, and aristololactam BII.'),
  'D032135': NtO(MH='Kadsura', MS='A plant genus of the family SCHISANDRACEAE used in folk medicine in a manner similar to SCHISANDRA. Members contain KADSURENONE and KADSURIN.'),
  'D032141': NtO(MH='Schisandra', MS='A plant genus of the family Schisandraceae which contain schisandrins (Russian) which are also called gomisins (Japanese) or wuweizins (Chinese). Compounds produced by this genus are very similar in composition and use to that of the related KADSURA plant.'),
  'D032161': NtO(MH='Antirrhinum', MS='A plant genus of the family Plantaginaceae. Members contain DEFICIENS PROTEIN.'),
  'D032181': NtO(MH='Craterostigma', MS='A plant genus of the order Lamiales, family Linderniaceae.'),
  'D032201': NtO(MH='Euphrasia', MS='A plant genus of the family Orobanchaceae used in FOLK REMEDIES for the eyes.'),
  'D032221': NtO(MH='Linaria', MS='A plant genus of the family Plantaginaceae. Members contain linarin (also called acaciin).'),
  'D032241': NtO(MH='Mimulus', MS='A plant genus of the family Phrymaceae. Members contain 6-geranylflavanones and mimulone.'),
  'D032261': NtO(MH='Picrorhiza', MS='A plant genus of the family Plantaginaceae. Members contain scrosides (CINNAMATES) and phenylethanoid glycoside. P. kurrooa is the source of picroliv (a purified iridoid glycoside fraction from the roots having hepatoprotective, anti-inflammatory and antioxidant properties).'),
  'D032262': NtO(MH='Pedicularis', MS='A plant genus of the family Orobanchaceae. Members contain phenylpropanoid glycosides and iridoids.'),
  'D032263': NtO(MH='Penstemon', MS='A plant genus of the family Plantaginaceae. Members contain phenylpropanoid and iridoid glycosides.'),
  'D032264': NtO(MH='Veronica', MS='A plant genus of the family Plantaginaceae. Members contain bis-sesquiterpene and iridoid glucosides.'),
  'D032265': NtO(MH='Verbascum', MS='A plant genus of the family Scrophulariaceae.'),
  'D032266': NtO(MH='Scrophularia', MS='A plant genus of the family SCROPHULARIACEAE. Members contain iridoids.'),
  'D032267': NtO(MH='Rehmannia', MS='A plant genus of the family Rehmanniaceae. Members contain catapol, rehmannin and ALKALOIDS.'),
  'D032268': NtO(MH='Ailanthus', MS='A plant genus of the family SIMAROUBACEAE. Members contain ailantinols and other quassinoids.'),
  'D032269': NtO(MH='Brucea', MS='A plant genus of the family SIMAROUBACEAE. Members contain bruceosides and bruceanols (quassinoids). The astringent seeds have been used to treat dysentery in southeastern Asia.'),
  'D032270': NtO(MH='Picrasma', MS='A plant genus of the family SIMAROUBACEAE. Members contain javanicins, picrasinoside and other quassinoids.'),
  'D032271': NtO(MH='Quassia', MS='A plant genus of the family SIMAROUBACEAE. Members contain quassinoids. Quassia simarouba has been reclassified as SIMAROUBA.'),
  'D032272': NtO(MH='Simarouba', MS='A plant genus of the family SIMAROUBACEAE. Members produce quassinoids.'),
  'D032281': NtO(MH='Smilax', MS='A plant genus of the family SMILACACEAE. Members contain smiglasides (phenylpropanoid glycosides) and steroidal saponins. Commercially it is sometimes adulterated with HEMIDESMUS, which would affect experimental results. Drinks called sarsaparilla and root beer have been prepared from the Smilaz ornata (Jamaican sarsaparilla) and SASSAFRAS.'),
  'D032282': NtO(MH='Atropa', MS='A plant genus of the family SOLANACEAE which contain SOLANACEOUS ALKALOIDS including ATROPINE which is named after this genus.'),
  'D032301': NtO(MH='Cestrum', MS='A plant genus of the family SOLANACEAE. Members contain steroidal saponins and cause CALCINOSIS in grazing livestock due to high levels of 1 ALPHA,25-DIHYDROXYVITAMIN D3.'),
  'D032302': NtO(MH='Datura', MS='A plant genus of the family SOLANACEAE. Members contain TROPANES. The common name of trumpet flower is also sometimes used for GELSEMIUM.'),
  'D032303': NtO(MH='Duboisia', MS='A plant genus of the family SOLANACEAE that is a source of SCOPOLAMINE and other TROPANES.'),
  'D032304': NtO(MH='Hyoscyamus', MS='A plant genus of the family SOLANACEAE which contains TROPANES.'),
  'D032305': NtO(MH='Lycium', MS='A plant genus of the family SOLANACEAE. Members contain CEREBROSIDES and SCOPOLETIN.'),
  'D032306': NtO(MH='Petunia', MS='A plant genus of the family SOLANACEAE. Members contain steroidal glycosides.'),
  'D032321': NtO(MH='Physalis', MS='A plant genus of the family SOLANACEAE. Members contain physalin and withangulatin.'),
  'D032322': NtO(MH='Solanum', MS='A plant genus of the family SOLANACEAE. Members contain SOLANACEOUS ALKALOIDS. Some species in this genus are called deadly nightshade which is also a common name for ATROPA BELLADONNA.'),
  'D032323': NtO(MH='Solanum melongena', MS='A plant species of the genus SOLANUM, family SOLANACEAE. The fruit is a large, egg-shaped berry, varying in color from dark purple to red, yellowish, or white. The leaves are large and ovate. The flowers are pendant, violet, and two inches across.'),
  'D032324': NtO(MH='Solanum nigrum', MS='A plant species of the genus SOLANUM, family SOLANACEAE that contains steroidal glycosides.'),
  'D032341': NtO(MH='Withania', MS='A plant genus of the family SOLANACEAE. Members contain withanolides. Withania somnifera is the source of ashwagandha and aswal.'),
  'D032342': NtO(MH='Muscle Cells', MS='Mature contractile cells, commonly known as myocytes, that form one of three kinds of muscle. The three types of muscle cells are skeletal (MUSCLE FIBERS, SKELETAL), cardiac (MYOCYTES, CARDIAC), and smooth (MYOCYTES, SMOOTH MUSCLE). They are derived from embryonic (precursor) muscle cells called MYOBLASTS.'),
  'D032361': NtO(MH='Sterculia', MS='A plant genus of the family STERCULIACEAE. S. urens is the source of KARAYA GUM which is sometimes called Indian tragacanth, which is different from the true TRAGACANTH which comes from ASTRAGALUS GUMMIFER.'),
  'D032362': NtO(MH='Styrax', MS='A plant genus of the family STYRACACEAE. Sap of these Asian trees are a source of a balsam (BALSAMS). This styrax balsam is 3/4 coniferyl benzoate, 1/8 free BENZOIC ACID, along with benzyl cinnamate, vanillin, and TRITERPENES.'),
  'D032363': NtO(MH='Tamaricaceae', MS='A plant family of the order Violales, subclass Dilleniidae, class Magnoliopsida.'),
  'D032364': NtO(MH='Daphne', MS='A plant genus of the family THYMELAEACEAE. They are evergreen shrubs much cultivated in garden borders and rock gardens in mild climates. Members contain mezerein, flavonoids, and COUMARINS such as daphnetin and daphnin.'),
  'D032365': NtO(MH='Wikstroemia', MS='A plant genus of the family THYMELAEACEAE. Members contain daphnoretin and wikstroelides and daphnane-type DITERPENES.'),
  'D032381': NtO(MH='Corchorus', MS='A plant genus of the family TILIACEAE. Members contain cycloartane saponins and CARDENOLIDES.'),
  'D032382': NtO(MH='Grewia', MS='A plant genus of the family TILIACEAE. Members contain lupeol and betulin TRITERPENES.'),
  'D032383': NtO(MH='Myocytes, Cardiac', MS='Striated muscle cells found in the heart. They are derived from cardiac myoblasts (MYOBLASTS, CARDIAC).'),
  'D032384': NtO(MH='Tilia', MS='A plant genus of the family TILIACEAE. Some species in this genus are called Limetree which is nearly the same as the common name for lime (CITRUS AURANTIIFOLIA). Some people are allergic to the POLLEN.'),
  'D032385': NtO(MH='Triumfetta', MS='A plant genus of the family TILIACEAE. The common name of burweed is also used for AMSINCKIA. Triumfetta semitriloba is considered a noxious weed.'),
  'D032386': NtO(MH='Myoblasts, Cardiac', MS='Precursor cells destined to differentiate into cardiac myocytes (MYOCYTES, CARDIAC).'),
  'D032387': NtO(MH='Tropaeolaceae', MS='A plant family of the order Geraniales, subclass Rosidae, class Magnoliopsida.'),
  'D032388': NtO(MH='Tropaeolum', MS='A plant genus of the family TROPAEOLACEAE. The common nasturtium is a plant that grows 2.4-3.6 m (8-12 feet) tall and has funnel-shaped flowers that are commonly yellow-orange with red spots or stripes and have a long spur that contains sweet nectar. Some species in this genus are called watercress which is also a common name for RORIPPA and NASTURTIUM.'),
  'D032389': NtO(MH='Myocytes, Smooth Muscle', MS='Non-striated, elongated, spindle-shaped cells found lining the digestive tract, uterus, and blood vessels. They are derived from specialized myoblasts (MYOBLASTS, SMOOTH MUSCLE).'),
  'D032390': NtO(MH='Myoblasts, Smooth Muscle', MS='Precursor cells destined to differentiate into smooth muscle myocytes (MYOCYTES, SMOOTH MUSCLE).'),
  'D032401': NtO(MH='Typhaceae', MS='A plant family of the order Typhales, subclass Commelinidae, class Liliopsida (monocotyledons) that contains a single genus, Typha, that grows worldwide.'),
  'D032402': NtO(MH='Boehmeria', MS='A plant genus of the family URTICACEAE. Members contain cryptopleurine, beta-sitosterol, daucosterol and 19 alpha-hydroxyursolic acid.'),
  'D032403': NtO(MH='Parietaria', MS="A plant genus of the family URTICACEAE which is the most common cause of pollinosis in Mediterranean countries. It is the source of 'allergen Par o I' and of Pollinex used for DESENSITIZATION, IMMUNOLOGIC."),
  'D032404': NtO(MH='Nardostachys', MS='A plant genus of the family VALERIANACEAE. Nardostachysin, nardostachin, nardosinone, valeranone, and patchoulene have been found in this genus. Nardostachys jatamansi is classified by some as Valeriana jatamansi.'),
  'D032405': NtO(MH='Patrinia', MS='A plant genus of the family VALERIANACEAE. Members contain ursolic acid and oleanolic acid glycosides, sulfapatrinosides (triterpenoid glycosides), and patriscabrol (iridolactone).'),
  'D032406': NtO(MH='Valerianella', MS='A plant genus of the family VALERIANACEAE. It is sometimes called goosefoot cornsalad but should not be confused with true goosefoot (CHENOPODIUM).'),
  'D032407': NtO(MH='Avicennia', MS='A plant genus of the family Acanthaceae. Members contain NAPHTHOQUINONES. Black mangroves (common name for the genus) are distinguished from other mangroves by their spike-like aerial roots called pneumatophores that project from the soil or water surrounding the plants.'),
  'D032408': NtO(MH='Callicarpa', MS='A plant genus of the family Lamiaceae. Members contain clerodane DITERPENES and callicarpone.'),
  'D032409': NtO(MH='Clerodendrum', MS='A plant genus of the family Lamiaceae. Inerminosides (iridoid glycosides); colebroside A; STEROIDS; STEROLS; terpenoid glucosides; and abietane DITERPENES have been found in this genus.'),
  'D032410': NtO(MH='Lantana', MS='A plant genus of the family VERBENACEAE. Members contain lantadene and other TRITERPENES.'),
  'D032411': NtO(MH='Lippia', MS='A plant genus of the family VERBENACEAE. Lippsidoquinone; TRITERPENES; SESQUITERPENES; and THYMOL have been found in this genus. Plant extracts have cytotoxic activity. It is sometimes called Mexican oregano but that confuses it with real oregano (ORIGANUM).'),
  'D032412': NtO(MH='Verbena', MS='A plant genus of the family VERBENACEAE. Members contain verbenachalcone (dimeric dihydrochalcone), iridoids, and phenylethanoids.'),
  'D032413': NtO(MH='Viola', MS='A plant genus of the family VIOLACEAE. Some species in this genus are called bouncing bet which is a common name more often used with SAPONARIA OFFICINALIS. Members contain macrocyclic peptides.'),
  'D032421': NtO(MH='Ampelopsis', MS='A plant genus of the family VITACEAE. Members contain momordin and SITOSTEROLS.'),
  'D032422': NtO(MH='Cissus', MS='A plant genus of the family VITACEAE. Cissus rufescence gum is considered comparable to TRAGACANTH.'),
  'D032423': NtO(MH='Winteraceae', MS='A plant family of the order Magnoliales, subclass Magnoliidae, class Magnoliopsida. The wood lacks water-conducting cells but has acrid sap. The leaves are gland-dotted, leathery, and smooth-margined. The flowers are small, in clusters, with two to six sepals, petals in two or more series, several stamens, and one to several carpels.'),
  'D032424': NtO(MH='Drimys', MS='A plant genus of the family WINTERACEAE. They have leathery, elliptic-shaped leaves; red-tinged shoots; and jasmine-scented, cream-colored, 8- to 12-petaled, 2.5-centimeter (1-inch) flowers in clusters.'),
  'D032425': NtO(MH='Pseudowintera', MS='A plant genus of the family WINTERACEAE. Members contain polygodial.'),
  'D032426': NtO(MH='Marantaceae', MS='A plant family of the order ZINGIBERALES, subclass Zingiberidae, class Liliopsida.'),
  'D032427': NtO(MH='Heliconiaceae', MS='A plant family of the order ZINGIBERALES, subclass Zingiberidae, class Liliopsida.'),
  'D032428': NtO(MH='Strelitziaceae', MS='A plant family of the order ZINGIBERALES, subclass Zingiberidae, class Liliopsida. Some species in this genus are called bird-of-paradise which is also a common name for CAESALPINIA and Heliconia (HELICONIACEAE).'),
  'D032429': NtO(MH='Alpinia', MS='A plant genus of the family ZINGIBERACEAE. Members contain galangin, yakuchinone-A, and diarylheptanoids.'),
  'D032430': NtO(MH='Balanites', MS='A plant genus of the family ZYGOPHYLLACEAE. Feruloyltyramine, balanitoside (a furostanol glycoside), and cytostatic steroidal saponins have been found in this genus. B. aegyptiaca fruit water extract is traditionally used as an anthelmintic in the Sudan.'),
  'D032441': NtO(MH='Guaiacum', MS='A plant genus of the family ZYGOPHYLLACEAE. GUAIAC is obtained from G. officinale or G. sanctum. Some species in this genus are called lignum vitae which is also a common name for other trees.'),
  'D032442': NtO(MH='Larrea', MS='A plant genus of the family ZYGOPHYLLACEAE. It is sometimes called chaparral but that is a generic word which is used with a number of other plants. Members contain NORDIHYDROGUAIARETIC ACID.'),
  'D032443': NtO(MH='Peganum', MS='A plant genus of the family ZYGOPHYLLACEAE. HARMALINE and other ALKALOIDS, phenylpropanoids, and TRITERPENES have been found in plants of this genus.'),
  'D032444': NtO(MH='Tribulus', MS='A plant genus of the family ZYGOPHYLLACEAE. Members contain steroidal saponins. Ingestion by grazing animals causes PHOTOSENSITIVITY DISORDERS called geeldikkop (yellow thick head) in South Africa.'),
  'D032445': NtO(MH='Zygophyllum', MS='A plant genus of the family ZYGOPHYLLACEAE. Members contain SAPONINS.'),
  'D032446': NtO(MH='Myoblasts', MS='Embryonic (precursor) cells of the myogenic lineage that develop from the MESODERM. They undergo proliferation, migrate to their various sites, and then differentiate into the appropriate form of myocytes (MYOCYTES, SKELETAL; MYOCYTES, CARDIAC; MYOCYTES, SMOOTH MUSCLE).'),
  'D032447': NtO(MH='Wisteria', MS='A plant genus of the family FABACEAE. Members contain wistarin, wisteria lectin and wistariasaponin.'),
  'D032448': NtO(MH='Myoblasts, Skeletal', MS='Precursor cells destined to differentiate into skeletal myocytes (MYOCYTES, SKELETAL).'),
  'D032449': NtO(MH='Canavalia', MS='A plant genus of the family FABACEAE. Canavalia ensiformis is the source of CONCANAVALIN A.'),
  'D032461': NtO(MH='Chromosomes, Plant', MS='Complex nucleoprotein structures which contain the genomic DNA and are part of the CELL NUCLEUS of PLANTS.'),
  'D032462': NtO(MH='Cycadopsida', MS='A class of vascular seed plants also known as gymnosperms in the subdivision spermatophytina. They have large cones and pinnate leaves. Examples are pines, spruces, cedars and sequoias.'),
  'D032481': NtO(MH='Zamiaceae', MS='A plant family of the order Cycadales, class Cycadopsida, division CYCADOPHYTA.'),
  'D032483': NtO(MH='Cephalotaxus', MS='A plant genus of the family TAXACEAE, order Pinales, class Pinopsida, division TRACHEOPHYTA. Members contain homoharringtonine.'),
  'D032485': NtO(MH='Cryptomeria', MS='A plant genus of the family CUPRESSACEAE which produce POLLEN that is a major ALLERGEN.'),
  'D032486': NtO(MH='Cunninghamia', MS='A plant genus of the family TAXODIACEAE. Members contain DITERPENES.'),
  'D032487': NtO(MH='Sequoia', MS='A plant family of conifers in the order Pinales. They are very tall trees with a worldwide distribution.'),
  'D032488': NtO(MH='Sequoiadendron', MS='A plant genus of the family CUPRESSACEAE known for tall trees like the SEQUOIA redwoods.'),
  'D032489': NtO(MH='Taxodium', MS='A plant genus of the family CUPRESSACEAE. Members contain taxodione and taxodone, which are diterpenoid quinone methide tumor inhibitors.'),
  'D032490': NtO(MH='Taxaceae', MS='A plant family of the order Pinales, class Pinopsida, division TRACHEOPHYTA.'),
  'D032491': NtO(MH='Dryopteridaceae', MS='The wood fern plant family of the order Polypodiales, class Filicopsida, division Pteridophyta.'),
  'D032492': NtO(MH='Dryopteris', MS='A plant genus of the family DRYOPTERIDACEAE. Members contain aspidin and filicic acid.'),
  'D032493': NtO(MH='Polystichum', MS='A plant genus of the family DRYOPTERIDACEAE.'),
  'D032494': NtO(MH='Polypodium', MS='A plant genus of the family POLYPODIACEAE. Members contain polypodoside.'),
  'D032495': NtO(MH='Pteris', MS='A plant genus of the family PTERIDACEAE. Members contain entkaurane DITERPENES. The name is similar to bracken fern (PTERIDIUM).'),
  'D032496': NtO(MH='Satellite Cells, Skeletal Muscle', MS='Elongated, spindle-shaped, quiescent myoblasts lying in close contact with adult skeletal muscle. They are thought to play a role in muscle repair and regeneration.'),
  'D032497': NtO(MH='Dennstaedtiaceae', MS='The bracken fern plant family of the order Polypodiales, class Filicopsida, division Pteridophyta.'),
  'D032498': NtO(MH='Pteridium', MS='A plant genus of the family DENNSTAEDTIACEAE. Members contain ptaquiloside, braxin A1, and braxin B. The name is similar to brake fern (PTERIS).'),
  'D032501': NtO(MH='Huperzia', MS='A plant genus of the family LYCOPODIACEAE. Members contain huperzine, one of the CHOLINESTERASE INHIBITORS.'),
  'D032502': NtO(MH='Lycopodium', MS='A plant genus of the family LYCOPODIACEAE. Members contain ALKALOIDS. Lycopodium oil is obtained from L. clavatum.'),
  'D032503': NtO(MH='Selaginellaceae', MS='A plant family of the order Selaginellales, class Lycopodiopsida, division Lycopodiophyta, subkingdom TRACHEOPHYTA. Members contain bilobetin. The rarely used common name of resurrection plant is mainly used with CRATEROSTIGMA.'),
  'D032521': NtO(MH='Immunity, Herd', MS='The non-susceptibility to infection of a large group of individuals in a population. A variety of factors can be responsible for herd immunity and this gives rise to the different definitions used in the literature. Most commonly, herd immunity refers to the case when, if most of the population is immune, infection of a single individual will not cause an epidemic. Also, in such immunized populations, susceptible individuals are not likely to become infected. Herd immunity can also refer to the case when unprotected individuals fail to contract a disease because the infecting organism has been banished from the population.'),
  'D032541': NtO(MH='Mass Vaccination', MS='Administration of a vaccine to large populations in order to elicit IMMUNITY.'),
  'D032561': NtO(MH='Cotton Fiber', MS='A TEXTILE fiber obtained from the pappus (outside the SEEDS) of cotton plant (GOSSYPIUM).'),
  'D032581': NtO(MH='Tuberculosis Vaccines', MS='Vaccines or candidate vaccines used to prevent or treat TUBERCULOSIS.'),
  'D032601': NtO(MH='Erysimum', MS='A plant genus of the family BRASSICACEAE. Some members contain CARDIAC GLYCOSIDES.'),
  'D032602': NtO(MH='Caragana', MS='A plant genus of the family FABACEAE. Members contain STILBENES.'),
  'D032603': NtO(MH='Scopolia', MS='A plant genus of the family SOLANACEAE after which the compound SCOPOLAMINE got its name.'),
  'D032604': NtO(MH='Alangiaceae', MS='A plant family of the order Cornales, subclass Rosidae, class Magnoliopsida, consisting of slow-growing evergreen trees common in tropical forests of south India and Burma. Members contain tetrahydroisoquinoline-monoterpene and iridoid glycosides.'),
  'D032621': NtO(MH='DNA Shuffling', MS='The use of DNA recombination (RECOMBINATION, GENETIC) to prepare a large gene library of novel, chimeric genes from a population of randomly fragmented DNA from related gene sequences.'),
  'D032641': NtO(MH='Eurotiales', MS='An order of fungi in the phylum ASCOMYCOTA characterized by the presence of well defined peridia and cleistothecial asci. Notable anamorphs (mitosporic forms) of Eurotiales include PENICILLIUM and ASPERGILLUS.'),
  'D032661': NtO(MH='Cordyceps', MS='A genus of ascomycetous fungi (ASCOMYCOTA), family Clavicipitaceae, order HYPOCREALES, that grows by infecting insect larvae or mature insects with spores that germinate often before the cocoon is formed.'),
  'D032681': NtO(MH='Spectral Karyotyping', MS='The simultaneous identification of all chromosomes from a cell by fluorescence in situ hybridization (IN SITU HYBRIDIZATION, FLUORESCENCE) with chromosome-specific florescent probes that are discerned by their different emission spectra.'),
  'D032701': NtO(MH='Biomimetics', MS='An interdisciplinary field in materials science, ENGINEERING, and BIOLOGY, studying the use of biological principles for synthesis or fabrication of BIOMIMETIC MATERIALS.'),
  'D032721': NtO(MH='Adult Children', MS='Children who have reached maturity or the legal age of majority.'),
  'D032722': NtO(MH='Advance Care Planning', MS="Discussions with patients and/or their representatives about the goals and desired direction of the patient's care, particularly end-of-life care, in the event that the patient is or becomes incompetent to make decisions."),
  'D032741': NtO(MH='Monascus', MS='A genus in the family Monascaceae, order EUROTIALES. One species, Monascus purpureus, has multiple uses in traditional Chinese medicine (MEDICINE, CHINESE TRADITIONAL).'),
  'D032761': NtO(MH='Animal Experimentation', MS='The use of animals as investigational subjects.'),
  'D032762': NtO(MH='Autoexperimentation', MS='Intentionally using oneself as a research subject.'),
  'D032763': NtO(MH='Behavior Control', MS='Manipulation of the behavior of persons or animals by biomedical, physical, psychological, or social means, including for nontherapeutic reasons.'),
  'D032764': NtO(MH='Casuistry', MS='A method of ETHICAL ANALYSIS that emphasizes practical problem solving through examining individual cases that are considered to be representative; sometimes used to denote specious argument or rationalization. Differentiate from casuistics, which is the recording and study of cases and disease.'),
  'D032781': NtO(MH='Codes of Ethics', MS='Systematic statements of principles or rules of appropriate professional conduct, usually established by professional societies.'),
  'D032801': NtO(MH='Iridectomy', MS='Surgical removal of a section of the iris.'),
  'D032802': NtO(MH='Committee Membership', MS='The composition of a committee; the state or status of being a member of a committee.'),
  'D032821': NtO(MH='Commodification', MS='The social process by which something or someone comes to be regarded and treated as an article of trade or commerce.'),
  'D032841': NtO(MH='Sinomenium', MS='A plant genus of the family MENISPERMACEAE. Members contain sinomenine, caffeine, 1,7-dimethylxanthine, and other ALKALOIDS.'),
  'D032842': NtO(MH='Emericella', MS='A genus in the family Trichocomaceae, order EUROTIALES. The anamorph is ASPERGILLUS.'),
  'D032861': NtO(MH='Buddleja', MS='A plant genus of the family SCROPHULARIACEAE. Members contain mimengoside B, verbascoside, and phenylethanoids.'),
  'D032862': NtO(MH='Litsea', MS='A plant genus of the family LAURACEAE. Members contain laurotetanine and other APORPHINES.'),
  'D032882': NtO(MH='Comprehension', MS='The act or fact of grasping the meaning, nature, or importance of; understanding. (American Heritage Dictionary, 4th ed) Includes understanding by a patient or research subject of information disclosed orally or in writing.'),
  'D032901': NtO(MH='Talaromyces', MS='A fungal genus in the family Trichocomaceae, order EUROTIALES, characterized by loose hyphal fruiting bodies containing spherical asci. Anamorphs include PENICILLIUM and PAECILOMYCES.'),
  'D032902': NtO(MH='Ganoderma', MS='A genus of fungi in the family Ganodermataceae, order POLYPORALES, containing a dimitic hyphal system. It causes a white rot, and is a wood decomposer. Ganoderma lucidum (REISHI) is used in traditional Chinese medicine (MEDICINE, CHINESE TRADITIONAL).'),
  'D032903': NtO(MH='Confucianism', MS='A school of thought and set of moral, ethical, and political teachings usually considered to be founded by Confucius in 6th-5th century B.C. China. (from Cambridge Dictionary of Philosophy, 1995)'),
  'D032904': NtO(MH='Cinnamomum aromaticum', MS='A plant species of the genus CINNAMOMUM that contains CINNAMATES and has been used in traditional Chinese medicine (DRUGS, CHINESE HERBAL).'),
  'D032921': NtO(MH='Consensus', MS='General agreement or collective opinion; the judgment arrived at by most of those concerned.'),
  'D032961': NtO(MH='Sperm Midpiece', MS='The middle piece of the spermatozoon is a highly organized segment consisting of MITOCHONDRIA, the outer dense fibers and the core microtubular structure.'),
  'D032962': NtO(MH='Consent Forms', MS="Documents describing a medical treatment or research project, including proposed procedures, risks, and alternatives, that are to be signed by an individual, or the individual's proxy, to indicate his/her understanding of the document and a willingness to undergo the treatment or to participate in the research."),
  'D032981': NtO(MH='Dehumanization', MS='The process by which a person or group of persons comes to be regarded or treated as lacking in human qualities.'),
  'D032982': NtO(MH='Contracts', MS='Agreements between two or more parties, especially those that are written and enforceable by law (American Heritage Dictionary of the English Language, 4th ed). It is sometimes used to characterize the nature of the professional-patient relationship.'),
  'D032983': NtO(MH='Setaria Plant', MS='A plant genus of the family POACEAE. The small pointed seeds are grown for hay in North America and western Europe and important as food in China and other Asian countries.'),
  'D032984': NtO(MH='Ethics, Research', MS='The moral obligations governing the conduct of research. Used for discussions of research ethics as a general topic.'),
  'D033001': NtO(MH='Directed Tissue Donation', MS='Tissue, organ, or gamete donation intended for a designated recipient.'),
  'D033002': NtO(MH='Duty to Recontact', MS='The ethical and/or legal obligation of a health provider or researcher to communicate with a former patient or research subject about advances in research relevant to a treatment or to a genetic or other diagnostic test provided earlier, or about proposed new uses of blood or tissue samples taken in the past for another purpose.'),
  'D033003': NtO(MH='Eastern Orthodoxy', MS='The name given to the religion of the body of modern churches, including among others the Greek and Russian Orthodox, that is derived from the church of the Byzantine Empire, adheres to the Byzantine rite, and acknowledges the honorary primacy of the patriarch of Constantinople. (from American Heritage Dictionary of the English Language, 4th ed)'),
  'D033021': NtO(MH='Oxytropis', MS='A plant genus of the family FABACEAE. Members contain SWAINSONINE.'),
  'D033022': NtO(MH='Vicia sativa', MS='A plant species of the genus VICIA, family FABACEAE. The seed is used for food and contains THIOCYANATES such as prunasin, cyanoalanine, cyanogen, and vicine.'),
  'D033023': NtO(MH='Foot Joints', MS='The articulations extending from the ANKLE distally to the TOES. These include the ANKLE JOINT; TARSAL JOINTS; METATARSOPHALANGEAL JOINT; and TOE JOINT.'),
  'D033041': NtO(MH='Embryo Research', MS='Experimentation on, or using the organs or tissues from, a human or other mammalian conceptus during the prenatal stage of development that is characterized by rapid morphological changes and the differentiation of basic structures. In humans, this includes the period from the time of fertilization to the end of the eighth week after fertilization.'),
  'D033042': NtO(MH='Fetal Research', MS='Experimentation on, or using the organs or tissues from, a human or other mammalian conceptus in the postembryonic period, after the major structures have been outlined. In humans, this corresponds to the period from the third month after fertilization until birth.'),
  'D033061': NtO(MH='Ethics Consultation', MS="Services provided by an individual ethicist (ETHICISTS) or an ethics team or committee (ETHICS COMMITTEES, CLINICAL) to address the ethical issues involved in a specific clinical case. The central purpose is to improve the process and outcomes of patients' care by helping to identify, analyze, and resolve ethical problems."),
  'D033062': NtO(MH='Friends', MS='Persons whom one knows, likes, and trusts.'),
  'D033081': NtO(MH='Shear Strength', MS='The internal resistance of a material to moving some parts of it parallel to a fixed plane, in contrast to stretching (TENSILE STRENGTH) or compression (COMPRESSIVE STRENGTH). Ionic crystals are brittle because, when subjected to shear, ions of the same charge are brought next to each other, which causes repulsion.'),
  'D033101': NtO(MH='Euthanasia, Active, Voluntary', MS="Active euthanasia of a patient at the patient's request and/or with the patient's consent."),
  'D033141': NtO(MH='Genetic Determinism', MS="The theory that human CHARACTER and BEHAVIOR are shaped by the GENES that comprise the individual's GENOTYPE rather than by CULTURE; ENVIRONMENT; and individual choice."),
  'D033142': NtO(MH='Genetic Services', MS='Organized services to provide diagnosis, treatment, and prevention of genetic disorders.'),
  'D033161': NtO(MH='Government Regulation', MS='Exercise of governmental authority to control conduct.'),
  'D033162': NtO(MH='Incidental Findings', MS="Unanticipated information discovered in the course of testing or medical care. Used in discussions of information that may have social or psychological consequences, such as when it is learned that a child's biological father is someone other than the putative father, or that a person tested for one disease or disorder has, or is at risk for, something else."),
  'D033181': NtO(MH='Information Dissemination', MS='The circulation or wide dispersal of information.'),
  'D033182': NtO(MH='Intention', MS='What a person has in mind to do or bring about.'),
  'D033183': NtO(MH='Interdisciplinary Communication', MS='Communication, in the sense of cross-fertilization of ideas, involving two or more academic disciplines (such as the disciplines that comprise the cross-disciplinary field of bioethics, including the health and biological sciences, the humanities, and the social sciences and law). Also includes differences in patterns of language usage in different academic or medical disciplines.'),
  'D033201': NtO(MH='Euphausiacea', MS='An order of pelagic, shrimplike CRUSTACEA. Many consume ZOOPLANKTON and a few are predacious. Many antarctic species, such as Euphausia superba, constitute the chief food of other animals.'),
  'D033202': NtO(MH='Anostraca', MS='An order of CRUSTACEA comprised of shrimp-like organisms containing body trunks with at least 20 segments. The are commonly used as aquarium food.'),
  'D033221': NtO(MH="Jehovah's Witnesses", MS="Members of a religious denomination founded in the United States during the late 19th century in which active evangelism is practiced, the imminent approach of the millennium is preached, and war and organized government authority in matters of conscience are strongly opposed (from American Heritage Dictionary of the English Language, 4th ed). Jehovah's Witnesses generally refuse blood transfusions and other blood-based treatments based on religious belief."),
  'D033222': NtO(MH='Judicial Role', MS='The kind of action or activity proper to the judiciary, particularly its responsibility for decision making.'),
  'D033241': NtO(MH='Lawyers', MS='Persons whose profession is to give legal advice and assistance to clients and represent them in legal matters. (American Heritage Dictionary, 3d ed)'),
  'D033242': NtO(MH='Minors', MS='A person who has not attained the age at which full civil rights are accorded.'),
  'D033261': NtO(MH='Maternal-Fetal Relations', MS='The bond or lack thereof between a pregnant woman and her FETUS.'),
  'D033262': NtO(MH='Narration', MS='The act, process, or an instance of narrating, i.e., telling a story. In the context of MEDICINE or ETHICS, narration includes relating the particular and the personal in the life story of an individual.'),
  'D033281': NtO(MH='Nontherapeutic Human Experimentation', MS='Human experimentation that is not intended to benefit the subjects on whom it is performed. Phase I drug studies (CLINICAL TRIALS, PHASE I AS TOPIC) and research involving healthy volunteers are examples of nontherapeutic human experimentation.'),
  'D033282': NtO(MH='Parental Consent', MS='Informed consent given by a parent on behalf of a minor or otherwise incompetent child.'),
  'D033283': NtO(MH='Patient Access to Records', MS='The freedom of patients to review their own medical, genetic, or other health-related records.'),
  'D033301': NtO(MH='Deciduoma', MS='An intrauterine mass containing decidual cells (DECIDUA) or with structures resembling the PLACENTA. Deciduomas usually are induced experimentally.'),
  'D033302': NtO(MH='Postmodernism', MS='A late 20th-century philosophical approach or style of cultural analysis that seeks to reveal the cultural or social construction of concepts conventionally assumed to be natural or universal. (from E.R. DuBose, The Illusion of Trust: Toward a Medical Theological Ethics in the Postmodern Age, Kluwer, 1995)'),
  'D033303': NtO(MH='Protestantism', MS='The name given to all Christian denominations, sects, or groups rising out of the Reformation. Protestant churches generally agree that the principle of authority should be the Scriptures rather than the institutional church or the pope. (from W.L. Reese, Dictionary of Philosophy and Religion, 1999)'),
  'D033304': NtO(MH='Amphipoda', MS='An order of mostly marine CRUSTACEA containing more than 5500 species in over 100 families. Like ISOPODA, the other large order in the superorder Peracarida, members are shrimp-like in appearance, have sessile compound eyes, and no carapace. But unlike Isopoda, they possess thoracic gills and their bodies are laterally compressed.'),
  'D033321': NtO(MH='Isopoda', MS='One of the largest orders of mostly marine CRUSTACEA, containing over 10,000 species. Like AMPHIPODA, the other large order in the superorder Peracarida, members are shrimp-like in appearance, have sessile compound eyes, and no carapace. But unlike Amphipoda, they possess abdominal pleopods (modified as gills) and their bodies are dorsoventrally flattened.'),
  'D033341': NtO(MH='Arguloida', MS='An order of CRUSTACEA that are parasitic on freshwater fish.'),
  'D033342': NtO(MH='Copepoda', MS='A huge subclass of mostly marine CRUSTACEA, containing over 14,000 species. The 10 orders comprise both planktonic and benthic organisms, and include both free-living and parasitic forms. Planktonic copepods form the principle link between PHYTOPLANKTON and the higher trophic levels of the marine food chains.'),
  'D033361': NtO(MH='Cladocera', MS='A suborder of CRUSTACEA, order Diplostraca, comprising the water fleas. They are benthic filter feeders that consume PHYTOPLANKTON. The body is laterally compressed and enclosed in a bivalved carapace, from which the head extends.'),
  'D033362': NtO(MH='Powder Diffraction', MS='Method of using a polycrystalline powder and Rietveld refinement (LEAST SQUARES ANALYSIS) of X-RAY DIFFRACTION or NEUTRON DIFFRACTION. It circumvents the difficulties of producing single large crystals.'),
  'D033363': NtO(MH='Neutron Diffraction', MS='The scattering of NEUTRONS by matter, especially crystals, with accompanying variation in intensity due to interference effects. It is useful in CRYSTALLOGRAPHY and POWDER DIFFRACTION.'),
  'D033364': NtO(MH='Decapoda', MS='The largest order of CRUSTACEA, comprising over 10,000 species. They are characterized by three pairs of thoracic appendages modified as maxillipeds, and five pairs of thoracic legs. The order includes the familiar shrimps, crayfish (ASTACOIDEA), true crabs (BRACHYURA), and lobsters (NEPHROPIDAE and PALINURIDAE), among others.'),
  'D033381': NtO(MH='Anomura', MS='An infraorder of CRUSTACEA, in the order DECAPODA comprising the hermit crabs and characterized by a small fifth pair of legs.'),
  'D033401': NtO(MH='Microscopy, Acoustic', MS='A scientific tool based on ULTRASONOGRAPHY and used not only for the observation of microstructure in metalwork but also in living tissue. In biomedical application, the acoustic propagation speed in normal and abnormal tissues can be quantified to distinguish their tissue elasticity and other properties.'),
  'D033421': NtO(MH='Beginning of Human Life', MS='The point at which religious ensoulment or PERSONHOOD is considered to begin.'),
  'D033441': NtO(MH='Andrology', MS='A scientific or medical discipline concerning the study of male reproductive biology, diseases of the male genital organs, and male infertility. Major areas of interest include ENDOCRINOLOGY; SPERMATOGENESIS; SEMEN ANALYSIS; FERTILIZATION; CONTRACEPTION; and CRYOPRESERVATION.'),
  'D033461': NtO(MH='Hyperuricemia', MS='Excessive URIC ACID or urate in blood as defined by its solubility in plasma at 37 degrees C; greater than 0.42mmol per liter (7.0mg/dL) in men or 0.36mmol per liter (6.0mg/dL) in women. This condition is caused by overproduction of uric acid or impaired renal clearance. Hyperuricemia can be acquired, drug-induced or genetically determined (LESCH-NYHAN SYNDROME). It is associated with HYPERTENSION and GOUT.'),
  'D033481': NtO(MH='Chromosomes, Mammalian', MS='Complex nucleoprotein structures which contain the genomic DNA and are part of the CELL NUCLEUS of MAMMALS.'),
  'D033501': NtO(MH='Crangonidae', MS='A family of shrimp in the class CRUSTACEA, order DECAPODA, containing a single genus, Crangon, with multiple species.'),
  'D033521': NtO(MH='Palinuridae', MS='A family of marine CRUSTACEA, in the order DECAPODA, comprising the clawless lobsters. They are found in tropical and subtropical waters and characterized by short spines along the length of the tail and body.'),
  'D033541': NtO(MH='Palaemonidae', MS='A family of CRUSTACEA, order DECAPODA, comprising the palaemonid shrimp. Genera include Macrobrachium, Palaemon, and Palaemonetes. Palaemonidae osmoregulate by means of gills.'),
  'D033542': NtO(MH='Pandalidae', MS='A family of CRUSTACEA, order DECAPODA, comprising the pandalid shrimp. They are protandric hermaphrodites and can breed in both male and female stages. Many species are commercially harvested in the Pacific Northwest.'),
  'D033561': NtO(MH='Penaeidae', MS='A family of CRUSTACEA, order DECAPODA, comprising the penaeid shrimp. Species of the genus Penaeus are the most important commercial shrimp throughout the world.'),
  'D033581': NtO(MH='Stem Cell Transplantation', MS='The transfer of STEM CELLS from one individual to another within the same species (TRANSPLANTATION, HOMOLOGOUS) or between species (XENOTRANSPLANTATION), or transfer within the same individual (TRANSPLANTATION, AUTOLOGOUS). The source and location of the stem cells determines their potency or pluripotency to differentiate into various cell types.'),
  'D033601': NtO(MH='Gordonia Bacterium', MS='A genus of gram-positive BACTERIA in the family Gordoniaceae, isolated from soil and from sputa of patients with chest disorders. It is also used for biotransformation of natural products.'),
  'D033621': NtO(MH='Tephritidae', MS='A large family of fruit flies in the order DIPTERA, comprising over 4,500 species in about 100 genera. They have patterned wings and brightly colored bodies and are found predominantly in the tropical latitudes.'),
  'D033622': NtO(MH='Ceratitis capitata', MS='A species of fruit fly originating in sub-Saharan Africa but widely distributed worldwide. One of the most destructive fruit pests, its larvae feed and develop on many different fruits and some vegetables.'),
  'D033641': NtO(MH='Millettia', MS='A plant genus of the family FABACEAE. Members contain ISOFLAVONES, some of which show molluscicidal and schistosomicidal activity. Some species of Pongamia have been reclassified to this genus and some to DERRIS.'),
  'D033661': NtO(MH='Oocysts', MS='Zygote-containing cysts of sporozoan protozoa. Further development in an oocyst produces small individual infective organisms called SPOROZOITES. Then, depending on the genus, the entire oocyst is called a sporocyst or the oocyst contains multiple sporocysts encapsulating the sporozoites.'),
  'D033681': NtO(MH='Mitochondrial Membrane Transport Proteins', MS='Proteins involved in the transport of specific substances across the membranes of the MITOCHONDRIA.'),
  'D033701': NtO(MH='Nucleobase, Nucleoside, Nucleotide, and Nucleic Acid Transport Proteins', MS='A broad class of proteins involved in the transport of nucleobases, NUCLEOSIDES; NUCLEOTIDES; and NUCLEIC ACIDS; across membranes.'),
  'D033702': NtO(MH='Nucleobase Transport Proteins', MS='Proteins involved in the transport of nucleobases such as PYRIMIDINES and PURINES across membranes.'),
  'D033703': NtO(MH='Nucleoside Transport Proteins', MS='Proteins involved in the transport of NUCLEOSIDES across cellular membranes.'),
  'D033704': NtO(MH='Nucleotide Transport Proteins', MS='Proteins involved in the transport of NUCLEOTIDES across cellular membranes.'),
  'D033705': NtO(MH='Equilibrative Nucleoside Transport Proteins', MS='A class of sodium-independent nucleoside transporters that mediate the facilitative transport of NUCLEOSIDES.'),
  'D033721': NtO(MH='Equilibrative Nucleoside Transporter 1', MS='A subtype of equilibrative nucleoside transporter proteins that is sensitive to inhibition by 4-nitrobenzylthioinosine.'),
  'D033722': NtO(MH='Equilibrative-Nucleoside Transporter 2', MS='A subtype of equilibrative nucleoside transporter proteins that is insensitive to inhibition by 4-nitrobenzylthioinosine.'),
  'D033741': NtO(MH='Adenine Nucleotide Translocator 1', MS='A subtype of mitochondrial ADP, ATP translocase found primarily in heart muscle (MYOCARDIUM) and skeletal muscle (MUSCLE, SKELETAL).'),
  'D033742': NtO(MH='Adenine Nucleotide Translocator 2', MS='A subtype of mitochondrial ADP, ATP translocase found primarily in FIBROBLASTS.'),
  'D033761': NtO(MH='Spores, Protozoan', MS='A vegetative stage in the life cycle of sporozoan protozoa. It is characteristic of members of the phyla APICOMPLEXA and MICROSPORIDIA.'),
  'D033781': NtO(MH='Adenine Nucleotide Translocator 3', MS='A subtype of mitochondrial ADP, ATP translocase found primarily in the LIVER.'),
  'D033862': NtO(MH='Iron-Binding Proteins', MS='Proteins that specifically bind to IRON.'),
  'D033863': NtO(MH='Transferrin-Binding Proteins', MS='A class of carrier proteins that bind to TRANSFERRIN. Many strains of pathogenic bacteria utilize transferrin-binding proteins to acquire their supply of iron from serum.'),
  'D033881': NtO(MH='Transferrin-Binding Protein A', MS='A subtype of bacterial transferrin-binding protein found in bacteria. It forms a cell surface receptor complex with TRANSFERRIN-BINDING PROTEIN B.'),
  'D033882': NtO(MH='Transferrin-Binding Protein B', MS='A subtype of bacterial transferrin-binding protein found in bacteria. It forms a cell surface receptor complex with TRANSFERRIN-BINDING PROTEIN A.'),
  'D033901': NtO(MH='Bacterial Transferrin Receptor Complex', MS='A complex of proteins that forms a receptor for TRANSFERRIN in BACTERIA. Many pathogenic bacteria utilize the transferrin-binding complex to acquire their supply of iron from serum.'),
  'D033902': NtO(MH='Periplasmic Binding Proteins', MS='Periplasmic proteins that scavenge or sense diverse nutrients. In the bacterial environment they usually couple to transporters or chemotaxis receptors on the inner bacterial membrane.'),
  'D033903': NtO(MH='Periplasmic Proteins', MS='Proteins found in the PERIPLASM of organisms with cell walls.'),
  'D033921': NtO(MH='Vesicular Transport Proteins', MS='A broad category of proteins involved in the formation, transport and dissolution of TRANSPORT VESICLES. They play a role in the intracellular transport of molecules contained within membrane vesicles. Vesicular transport proteins are distinguished from MEMBRANE TRANSPORT PROTEINS, which move molecules across membranes, by the mode in which the molecules are transported.'),
  'D033922': NtO(MH='Clathrin Heavy Chains', MS='The heavy chain subunits of clathrin.'),
  'D033941': NtO(MH='Clathrin Light Chains', MS='The light chain subunits of clathrin.'),
  'D033942': NtO(MH='Adaptor Proteins, Vesicular Transport', MS='A class of proteins involved in the transport of molecules via TRANSPORT VESICLES. They perform functions such as binding to the cell membrane, capturing cargo molecules and promoting the assembly of CLATHRIN. The majority of adaptor proteins exist as multi-subunit complexes, however monomeric varieties have also been found.'),
  'D033961': NtO(MH='Adaptor Protein Complex 1', MS='A clathrin adaptor protein complex primarily involved in clathrin-related transport at the TRANS-GOLGI NETWORK.'),
  'D033962': NtO(MH='Adaptor Protein Complex 2', MS='An adaptor protein complex primarily involved in the formation of clathrin-related endocytotic vesicles (ENDOSOMES) at the CELL MEMBRANE.'),
  'D033963': NtO(MH='Adaptor Protein Complex 3', MS='An adaptor protein complex found primarily on perinuclear compartments.'),
  'D033964': NtO(MH='Adaptor Protein Complex 4', MS='An adaptor protein complex involved in transport of molecules between the TRANS-GOLGI NETWORK and the endosomal-lysosomal system.'),
  'D033965': NtO(MH='Adaptor Protein Complex alpha Subunits', MS='A family of large adaptin protein subunits of approximately 100 kDa in size. They have been primarily found as components of ADAPTOR PROTEIN COMPLEX 2.'),
  'D033966': NtO(MH='Adaptor Protein Complex beta Subunits', MS='A family of large adaptin protein complex subunits of approximately 90-130 kDa in size.'),
  'D033981': NtO(MH='Adaptor Protein Complex delta Subunits', MS='A family of large adaptin protein subunits of approximately 130-kDa in size. They have been primarily found as components of ADAPTOR PROTEIN COMPLEX 3.'),
  'D033982': NtO(MH='Adaptor Protein Complex gamma Subunits', MS='A family of large adaptin protein subunits of approximately 90 KDa in size. They have been primarily found as components of ADAPTOR PROTEIN COMPLEX 1.'),
  'D033983': NtO(MH='Adaptor Protein Complex mu Subunits', MS='A family of medium adaptin protein subunits of approximately 45 KDa in size. They have been primarily found as components of ADAPTOR PROTEIN COMPLEX 3 and ADAPTOR PROTEIN COMPLEX 4.'),
  'D034001': NtO(MH='Adaptor Protein Complex Subunits', MS='The subunits that make up the large, medium and small chains of adaptor proteins.'),
  'D034002': NtO(MH='Adaptor Protein Complex sigma Subunits', MS='A family of small adaptin protein complex subunits of approximately 19 KDa in size.'),
  'D034021': NtO(MH='Auxilins', MS='A family of proteins that play a role as cofactors in the process of CLATHRIN recycling in cells.'),
  'D034041': NtO(MH='Monomeric Clathrin Assembly Proteins', MS='A subclass of clathrin assembly proteins that occur as monomers.'),
  'D034061': NtO(MH='Thrombopoiesis', MS='The process of generating thrombocytes (BLOOD PLATELETS) from the pluripotent HEMATOPOIETIC STEM CELLS in the BONE MARROW via the MEGAKARYOCYTES. The humoral factor with thrombopoiesis-stimulating activity is designated THROMBOPOIETIN.'),
  'D034062': NtO(MH='Insomnia, Fatal Familial', MS='An autosomal dominant disorder characterized by degeneration of the THALAMUS and progressive insomnia. It is caused by a mutation in the prion protein (PRIONS).'),
  'D034081': NtO(MH='Wasting Disease, Chronic', MS='A transmissible spongiform encephalopathy (prion disease) of DEER and elk characterized by chronic weight loss leading to death. It is thought to spread by direct contact between animals or through environmental contamination with the prion protein (PRIONS).'),
  'D034101': NtO(MH='Sporozoites', MS='The product of meiotic division of zygotes in parasitic protozoa comprising haploid cells. These infective cells invade the host and undergo asexual reproduction producing MEROZOITES (or other forms) and ultimately gametocytes.'),
  'D034121': NtO(MH='Reproductive Health Services', MS='Health care services related to human REPRODUCTION and diseases of the REPRODUCTIVE SYSTEM. Services are provided to both sexes and usually by physicians in the medical or the surgical specialties such as REPRODUCTIVE MEDICINE; ANDROLOGY; GYNECOLOGY; OBSTETRICS; and PERINATOLOGY.'),
  'D034141': NtO(MH='Hypoalbuminemia', MS='A condition in which albumin level in blood (SERUM ALBUMIN) is below the normal range. Hypoalbuminemia may be due to decreased hepatic albumin synthesis, increased albumin catabolism, altered albumin distribution, or albumin loss through the urine (ALBUMINURIA).'),
  'D034161': NtO(MH='Pelvic Infection', MS='Infection involving the tissues or organs in the PELVIS.'),
  'D034181': NtO(MH='Epimedium', MS='A plant genus of the family BERBERIDACEAE which is used in DRUGS, CHINESE HERBAL. Members contain flavonol glycosides including epimedins, icariin and noricariin.'),
  'D034182': NtO(MH='Cuscuta', MS='A plant genus of the family Cuscutaceae. It is a threadlike climbing parasitic plant that is used in DRUGS, CHINESE HERBAL.'),
  'D034201': NtO(MH='Computers, Handheld', MS='A type of MICROCOMPUTER, sometimes called a personal digital assistant, that is very small and portable and fitting in a hand. They are convenient to use in clinical and other field situations for quick data management. They usually require docking with MICROCOMPUTERS for updates.'),
  'D034221': NtO(MH='Moritella', MS='A genus of gram-negative, curved or straight rod-shaped bacteria, in the family ALTEROMONADACEAE. They are chemo-organotrophic, halophilic, and associated with cold marine habitats.'),
  'D034241': NtO(MH='Alteromonadaceae', MS='A family of marine, gram-negative PROTEOBACTERIA including the genera ALTEROMONAS; Colwellia; Idiomarina; MARINOBACTER; MORITELLA; PSEUDOALTEROMONAS; and SHEWANELLA.'),
  'D034242': NtO(MH='Cyclooctanes', MS='A group of compounds with an 8-carbon ring. They may be saturated or unsaturated.'),
  'D034261': NtO(MH='Epothilones', MS='A group of 16-member MACROLIDES which stabilize MICROTUBULES in a manner similar to PACLITAXEL. They were originally found in the myxobacterium Sorangium cellulosum, now renamed to Polyangium (MYXOCOCCALES).'),
  'D034281': NtO(MH='Dynamins', MS='A family of high molecular weight GTP phosphohydrolases that play a direct role in vesicle transport. They associate with microtubule bundles (MICROTUBULES) and are believed to produce mechanical force via a process linked to GTP hydrolysis. This enzyme was formerly listed as EC 3.6.1.50.'),
  'D034284': NtO(MH='Dynamin I', MS='A subtype of dynamin found primarily in the NEURONS of the brain.'),
  'D034285': NtO(MH='Dynamin II', MS='A subtype of dynamin found ubiquitously expressed in a variety of tissues.'),
  'D034286': NtO(MH='Dynamin III', MS='A subtype of dynamin found expressed exclusively in the testis, lung and brain.'),
  'D034301': NtO(MH='Deinococcus', MS='A genus of gram-positive aerobic cocci found in the soil, that is highly resistant to radiation, especially ionizing radiation (RADIATION, IONIZING). Deinococcus radiodurans is the type species.'),
  'D034321': NtO(MH='Hyperamylasemia', MS='A condition with abnormally elevated level of AMYLASES in the serum. Hyperamylasemia due to PANCREATITIS or other causes may be differentiated by identifying the amylase isoenzymes.'),
  'D034341': NtO(MH='Aristolochic Acids', MS='Nitro-phenanthrenes occurring in ARISTOLOCHIACEAE and other plants. They derive from stephanine (APORPHINES) by oxidative ring cleavage. The nitro group is a reactive alkylator (ALKYLATING AGENTS) that binds to biological macromolecules. Ingestion by humans is associated with nephropathy (NEPHRITIS). There is no relationship to the similar named aristolochene (SESQUITERPENES).'),
  'D034361': NtO(MH='Streptococcus mitis', MS='A species of gram-positive, coccoid bacteria commensal in the respiratory tract.'),
  'D034362': NtO(MH='Striga', MS='A plant genus of the family Orobanchaceae that is parasitic on the roots of other plants. Members contain the flavones, APIGENIN and LUTEOLIN.'),
  'D034363': NtO(MH='Viridans Streptococci', MS='A large heterogeneous group of mostly alpha-hemolytic streptococci. They colonize the respiratory tract at birth and generally have a low degree of pathogenicity. This group of species includes STREPTOCOCCUS MITIS; STREPTOCOCCUS MUTANS; STREPTOCOCCUS ORALIS; STREPTOCOCCUS SANGUIS; STREPTOCOCCUS SOBRINUS; and the STREPTOCOCCUS MILLERI GROUP. The latter are often beta-hemolytic and commonly produce invasive pyogenic infections including brain and abdominal abscesses.'),
  'D034364': NtO(MH='Streptococcus milleri Group', MS='A subset of VIRIDANS STREPTOCOCCI, but the species in this group differ in their hemolytic pattern and diseases caused. These species are often beta-hemolytic and produce pyogenic infections.'),
  'D034365': NtO(MH='Streptococcus constellatus', MS='A species of gram-positive bacteria in the STREPTOCOCCUS MILLERI GROUP. It is commonly found in the oropharnyx flora and has a proclivity for abscess formation in the upper body and respiratory tract.'),
  'D034366': NtO(MH='Streptococcus anginosus', MS='A species of gram-positive bacteria in the STREPTOCOCCUS MILLERI GROUP. It is the most frequently seen isolate of that group, has a proclivity for abscess formation, and is most often isolated from the blood, gastrointestinal, and urogenital tract.'),
  'D034367': NtO(MH='Streptococcus intermedius', MS='A species of gram-positive bacteria in the STREPTOCOCCUS MILLERI GROUP. It is commonly found in the oropharynx flora and has a proclivity for abscess formation, most characteristically in the CENTRAL NERVOUS SYSTEM and LIVER.'),
  'D034381': NtO(MH='Hearing Loss', MS='A general term for the complete or partial loss of the ability to hear from one or both ears.'),
  'D034401': NtO(MH='Ulmaceae', MS='A plant family of the order Urticales, subclass Hamamelidae, class Magnoliopsida. Members are trees and shrubs of temperate regions that have watery sap and alternate leaves which are lopsided at the base. The flowers lack petals.'),
  'D034402': NtO(MH='Trema', MS='A plant genus of the family ULMACEAE. Members contain PHENANTHRENES and COUMARINS.'),
  'D034403': NtO(MH='Ulmus', MS='A plant genus of the family ULMACEAE that is susceptible to Dutch elm disease which is caused by the ASCOMYCOTA fungus, Ophiostoma.'),
  'D034421': NtO(MH='Sus scrofa', MS='A species of SWINE, in the family Suidae, comprising a number of subspecies including the domestic pig Sus scrofa domestica.'),
  'D034441': NtO(MH='Heterogeneous-Nuclear Ribonucleoproteins', MS='A family of ribonucleoproteins that were originally found as proteins bound to nascent RNA transcripts in the form of ribonucleoprotein particles. Although considered ribonucleoproteins they are primarily classified by their protein component. They are involved in a variety of processes such as packaging of RNA and RNA TRANSPORT within the nucleus. A subset of heterogeneous-nuclear ribonucleoproteins are involved in additional functions such as nucleocytoplasmic transport (ACTIVE TRANSPORT, CELL NUCLEUS) of RNA and mRNA stability in the CYTOPLASM.'),
  'D034442': NtO(MH='N-substituted Glycines', MS='AMINO ACIDS composed of GLYCINE substituted at the nitrogen rather than the usual carbon position, resulting in the loss of HYDROGEN BONDING donors. Polymers of these compounds are called PEPTOIDS.'),
  'D034443': NtO(MH='RNA Transport', MS='The process of moving specific RNA molecules from one cellular compartment or region to another by various sorting and transport mechanisms.'),
  'D034444': NtO(MH='Peptoids', MS='Polymers of N-SUBSTITUTED GLYCINES containing chiral centers at the a-position of their side chains. These oligomers lack HYDROGEN BONDING donors, preventing formation of the usual intrachain hydrogen bonds but can form helices driven by the steric influence of chiral side chains.'),
  'D034445': NtO(MH='Ovulation Inhibition', MS='Blocking the process leading to OVULATION. Various factors are known to inhibit ovulation, such as neuroendocrine, psychological, and pharmacological agents.'),
  'D034461': NtO(MH='Luteinization', MS='Formation of CORPUS LUTEUM. This process includes capillary invasion of the ruptured OVARIAN FOLLICLE, hypertrophy of the GRANULOSA CELLS and the THECA CELLS, and the production of PROGESTERONE. Luteinization is regulated by LUTEINIZING HORMONE.'),
  'D034462': NtO(MH='Dissent and Disputes', MS='Differences of opinion or disagreements that may arise, for example, between health professionals and patients or their families, or against a political regime.'),
  'D034481': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein Group A-B', MS='A class of closely related heterogeneous-nuclear ribonucleoproteins of approximately 34-40 kDa in size. Although they are generally found in the nucleoplasm, they also shuttle between the nucleus and the cytoplasm. Members of this class have been found to have a role in mRNA transport, telomere biogenesis and RNA SPLICING.'),
  'D034482': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein Group C', MS='A group of closely related heterogeneous-nuclear ribonucleoproteins of approximately 41-43 kDa in size found in the cell nucleus. Members of this class have been implicated in a variety of processes including splicing, polyadenylation, and nuclear retention of RNA.'),
  'D034501': NtO(MH='Telomere-Binding Proteins', MS='Proteins that specifically bind to TELOMERES. Proteins in this class include those that perform functions such as telomere capping, telomere maintenance and telomere stabilization.'),
  'D034502': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein D', MS="A heterogeneous-nuclear ribonucleoprotein that has specificity for AU-rich elements found in the 3'-region of mRNA and may play a role in RNA stability. Several isoforms of hnRNP D protein have been found to occur due to alternative mRNA splicing (RNA SPLICING)."),
  'D034521': NtO(MH='Nuclear Matrix-Associated Proteins', MS='A broad category of nuclear proteins that are components of or participate in the formation of the NUCLEAR MATRIX.'),
  'D034541': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein Group F-H', MS='A group of closely-related heterogeneous-nuclear ribonucleoproteins that are involved in pre-mRNA splicing.'),
  'D034561': NtO(MH='Sheep, Domestic', MS='A species of sheep, Ovis aries, descended from Near Eastern wild forms, especially mouflon.'),
  'D034581': NtO(MH='Sheep, Bighorn', MS='A species of sheep, Ovis canadensis, characterized by massive brown horns. There are at least four subspecies and they are all endangered or threatened.'),
  'D034582': NtO(MH='Integumentary System', MS='The outer covering of the body composed of the SKIN and the skin appendages, which are the HAIR, the NAILS; and the SEBACEOUS GLANDS and the SWEAT GLANDS and their ducts.'),
  'D034621': NtO(MH='Hares', MS='The genus Lepus, in the family Leporidae, order LAGOMORPHA. Hares are born above ground, fully furred, and with their eyes and ears open. In contrast with RABBITS, hares have 24 chromosome pairs.'),
  'D034622': NtO(MH='RNA Interference', MS='A gene silencing phenomenon whereby specific dsRNAs (RNA, DOUBLE-STRANDED) trigger the degradation of homologous mRNA (RNA, MESSENGER). The specific dsRNAs are processed into SMALL INTERFERING RNA (siRNA) which serves as a guide for cleavage of the homologous mRNA in the RNA-INDUCED SILENCING COMPLEX. DNA METHYLATION may also be triggered during this process.'),
  'D034641': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein K', MS='A heterogeneous-nuclear ribonucleoprotein found in the CELL NUCLEUS and the CYTOPLASM. Heterogeneous-nuclear ribonucleoprotein K has been implicated in the regulation of gene expression at nearly all levels: GENETIC TRANSCRIPTION; mRNA processing (RNA PROCESSING, POST-TRANSCRIPTIONAL), mRNA transport, mRNA stability, and translation (TRANSLATION, GENETIC). The hnRNP protein has a strong affinity for polypyrimidine-rich RNA and for single-stranded polypyrimidine-rich DNA. Multiple hnRNP K protein isoforms exist due to alternative splicing and display different nucleic-acid-binding properties.'),
  'D034664': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein L', MS='A heterogeneous-nuclear ribonucleoprotein found associated with most nascent transcripts, most notably those of the landmark giant loops of amphibian lampbrush chromosomes.'),
  'D034681': NtO(MH='Bacopa', MS='A plant genus of the family Plantaginaceae. Members contain bacopaside, bacopasaponins and other dammarane type jujubogenins.'),
  'D034682': NtO(MH='Church of Jesus Christ of Latter-day Saints', MS="A group of religious bodies tracing their origin to Joseph Smith in 1830 and accepting the Book of Mormon as divine revelation. (from Merriam-Webster's Collegiate Dictionary, 10th ed)"),
  'D034701': NtO(MH='Mastocytosis, Cutaneous', MS='Skin lesions due to abnormal infiltration of MAST CELLS. Cutaneous mastocytosis is confined to the skin without the involvement of other tissues or organs, and is mostly found in children. The three major variants are: URTICARIA PIGMENTOSA; diffuse cutaneous mastocytosis; and SOLITARY MASTOCYTOMA OF SKIN.'),
  'D034702': NtO(MH='RNA-Binding Protein FUS', MS='A multifunctional heterogeneous-nuclear ribonucleoprotein that may play a role in homologous DNA pairing and recombination. The N-terminal portion of protein is a potent transcriptional activator, while the C terminus is required for RNA binding. The name FUS refers to the fact that genetic recombination events result in fusion oncogene proteins (ONCOGENE PROTEINS, FUSION) that contain the N-terminal region of this protein. These fusion proteins have been found in myxoid liposarcoma (LIPOSARCOMA, MYXOID) and acute myeloid leukemia.'),
  'D034721': NtO(MH='Mastocytosis, Systemic', MS='A group of disorders caused by the abnormal proliferation of MAST CELLS in a variety of extracutaneous tissues including bone marrow, liver, spleen, lymph nodes, and gastrointestinal tract. Systemic mastocytosis is commonly seen in adults. These diseases are categorized on the basis of clinical features, pathologic findings, and prognosis.'),
  'D034722': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein U', MS='A heterogeneous-nuclear ribonucleoprotein found associated with the NUCLEAR MATRIX.'),
  'D034723': NtO(MH='Double Effect Principle', MS='Guideline for determining when it is morally permissible to perform an action to pursue a good end with knowledge that the action will also bring about bad results. It generally states that, in cases where a contemplated action has such double effect, the action is permissible only if: it is not wrong in itself; the bad result is not intended; the good result is not a direct causal result of the bad result; and the good result is "proportionate to" the bad result. (from Solomon, "Double Effect," in Becker, The Encyclopedia of Ethics, 1992)'),
  'D034741': NtO(MH='RNA, Small Interfering', MS="Small double-stranded, non-protein coding RNAs (21-31 nucleotides) involved in GENE SILENCING functions, especially RNA INTERFERENCE (RNAi). Endogenously, siRNAs are generated from dsRNAs (RNA, DOUBLE-STRANDED) by the same ribonuclease, Dicer, that generates miRNAs (MICRORNAS). The perfect match of the siRNAs' antisense strand to their target RNAs mediates RNAi by siRNA-guided RNA cleavage. siRNAs fall into different classes including trans-acting siRNA (tasiRNA), repeat-associated RNA (rasiRNA), small-scan RNA (scnRNA), and Piwi protein-interacting RNA (piRNA) and have different specific gene silencing functions."),
  'D034742': NtO(MH='Electronic Mail', MS='Messages between computer users via COMPUTER COMMUNICATION NETWORKS. This feature duplicates most of the features of paper mail, such as forwarding, multiple copies, and attachments of images and other file types, but with a speed advantage. The term also refers to an individual message sent in this way.'),
  'D034743': NtO(MH='RNA-Induced Silencing Complex', MS='A multicomponent, ribonucleoprotein complex comprised of one of the family of ARGONAUTE PROTEINS and the "guide strand" of the one of the 20- to 30-nucleotide small RNAs. RISC cleaves specific RNAs, which are targeted for degradation by homology to these small RNAs. Functions in regulating gene expression are determined by the specific argonaute protein and small RNA including siRNA (RNA, SMALL INTERFERING), miRNA (MICRORNA), or piRNA (PIWI-INTERACTING RNA).'),
  'D034761': NtO(MH='Heterogeneous-Nuclear Ribonucleoprotein Group M', MS='A group of closely-related 72-74-kDa heterogeneous-nuclear ribonucleoproteins that are involved in RNA SPLICING events.'),
  'D034762': NtO(MH='Celosia', MS='A plant genus of the family AMARANTHACEAE. Young leaves are used as vegetables in Asia. Members contain betacyanins, celogentins, betaxanthin, and celosian.'),
  'D034781': NtO(MH='Receptors, N-Acetylglucosamine', MS='Cell surface receptors that bind to ACETYLGLUCOSAMINE.'),
  'D034801': NtO(MH='Mastocytoma', MS='A solid tumor consisting of a dense infiltration of MAST CELLS. It is generally benign.'),
  'D034802': NtO(MH='RNA-Binding Protein EWS', MS='A ubiquitous hnRNP protein that contains an RNA RECOGNITION MOTIF and C-terminal ranBP2-type zinc finger. It localizes to the CELL NUCLEUS and CYTOPLASM and is expressed as different isoforms in various tissues, interacting with CALMODULIN and TUMOR SUPPRESSOR PROTEIN P53 to negatively regulate cell growth. Translocations that result in the formation of fusion proteins containing parts of RNA-binding protein EWS are associated with EWING SARCOMA.'),
  'D034821': NtO(MH='Commelinaceae', MS='A plant family of the order Commelinales, subclass Commelinidae, class Liliopsida (monocotyledons) that are often somewhat succulent. The leaves are alternate, simple, parallel-veined, and usually with a closed sheathing base. The flowers are often in cymes and have 3 petals and 3 sepals.'),
  'D034822': NtO(MH='Commelina', MS='A plant genus of the family COMMELINACEAE of perennial herbs with blue flowers.'),
  'D034823': NtO(MH='Tradescantia', MS='A plant genus of the family COMMELINACEAE that is used in genotoxic bioassays.'),
  'D034841': NtO(MH='Abdominal Cavity', MS='The region in the abdomen extending from the thoracic DIAPHRAGM to the plane of the superior pelvic aperture (pelvic inlet). The abdominal cavity contains the PERITONEUM and abdominal VISCERA, as well as the extraperitoneal space which includes the RETROPERITONEAL SPACE.'),
  'D034861': NtO(MH='Abdominal Wall', MS='The outer margins of the ABDOMEN, extending from the osteocartilaginous thoracic cage to the PELVIS. Though its major part is muscular, the abdominal wall consists of at least seven layers: the SKIN, subcutaneous fat, deep FASCIA; ABDOMINAL MUSCLES, transversalis fascia, extraperitoneal fat, and the parietal PERITONEUM.'),
  'D034881': NtO(MH='Nuclear Lamina', MS='A lattice of fibrils which covers the entire inner surface of the nuclear envelope and interlinks nuclear pores (NUCLEAR PORE).'),
  'D034882': NtO(MH='Lamins', MS='Nuclear matrix proteins that are structural components of the NUCLEAR LAMINA. They are found in most multicellular organisms.'),
  'D034901': NtO(MH='Alismataceae', MS='A plant family of the subclass ALISMATIDAE, class Liliopsida (monocotyledons) of aquatic plants. The flower parts are in threes with 3 green sepals and 3 white or yellow petals.'),
  'D034902': NtO(MH='Alisma', MS='A plant genus of the family ALISMATACEAE. The flowers have 3 green sepals, 3 yellow and white petals, 6 stamens, and several pistils. Members contain TRITERPENES and SESQUITERPENES. Alisma is a component of tokishakuyakusan. Some species in this genus are called water plantain which is also a common name for other ALISMATACEAE plants.'),
  'D034903': NtO(MH='Sagittaria', MS='A plant genus of the family ALISMATACEAE that grows in salty marshes and is used for phytoremediation of oil spills. The unisexual flowers have 3 sepals and 3 petals. Members contain trifoliones (DITERPENES).'),
  'D034904': NtO(MH='Lamin Type A', MS='A subclass of developmentally regulated lamins having a neutral isoelectric point. They are found to disassociate from nuclear membranes during mitosis.'),
  'D034921': NtO(MH='Lamin Type B', MS='A subclass of ubiquitously-expressed lamins having an acidic isoelectric point. They are found to remain bound to nuclear membranes during mitosis.'),
  'D034941': NtO(MH='Upper Extremity', MS='The region of the upper limb in animals, extending from the deltoid region to the HAND, and including the ARM; AXILLA; and SHOULDER.'),
  'D034961': NtO(MH='Antigens, Nuclear', MS='Immunologically detectable substances found in the CELL NUCLEUS.'),
  'D034982': NtO(MH='Integration Host Factors', MS='Bacterial proteins that are used by BACTERIOPHAGES to incorporate their DNA into the DNA of the "host" bacteria. They are DNA-binding proteins that function in genetic recombination as well as in transcriptional and translational regulation.'),
  'D035001': NtO(MH='Host Factor 1 Protein', MS='An integration host factor that was originally identified as a bacterial protein required for the integration of bacteriophage Q beta (ALLOLEVIVIRUS). Its cellular function may be to regulate mRNA stability and processing in that it binds tightly to poly(A) RNA and interferes with ribosome binding.'),
  'D035002': NtO(MH='Lower Extremity', MS='The region of the lower limb in animals, extending from the gluteal region to the FOOT, and including the BUTTOCKS; HIP; and LEG.'),
  'D035021': NtO(MH='Factor For Inversion Stimulation Protein', MS='A highly abundant DNA binding protein whose expression is strongly correlated with the growth phase of bacteria. The protein plays a role in regulating DNA topology and activation of RIBOSOMAL RNA transcription. It was originally identified as a factor required for inversion stimulation by the Hin recombinase of SALMONELLA and Gin site-specific recombinase of BACTERIOPHAGE MU.'),
  'D035041': NtO(MH='Anonymous Testing', MS='Testing in which the source of the specimen or the person being tested is not individually identified.'),
  'D035061': NtO(MH='Control Groups', MS='Groups that serve as a standard for comparison in experimental studies. They are similar in relevant characteristics to the experimental group but do not receive the experimental intervention.'),
  'D035081': NtO(MH='Editorial Policies', MS='The guidelines and policy statements set forth by the editor(s) or editorial board of a publication.'),
  'D035082': NtO(MH='Federal Government', MS='The level of governmental organization and function at the national or country-wide level.'),
  'D035141': NtO(MH='Voluntary Programs', MS='Programs in which participation is not required.'),
  'D035161': NtO(MH='Bombacaceae', MS='A plant family of the order Malvales, subclass Dilleniidae, class Magnoliopsida of tropical trees.'),
  'D035162': NtO(MH='Adansonia', MS='A plant genus of the family BOMBACACEAE that is used for FOOD and MEDICINE, TRADITIONAL.'),
  'D035163': NtO(MH='Bombax', MS='A plant genus of the family BOMBACACEAE. The fine silky hairs covering the seeds have been used like kapok (CEIBA) for floatation, stuffing, and insulation.'),
  'D035164': NtO(MH='Ceiba', MS='A plant genus of the family BOMBACACEAE. The fine silky hairs covering the seeds have been used for floatation, stuffing, and insulation.'),
  'D035165': NtO(MH='Transcription Factors, General', MS='Transcription factors that form transcription initiation complexes on DNA, bind to specific DNA-DIRECTED RNA POLYMERASES and are required to initiate transcription. Although their binding may be localized to distinct sequence and structural motifs within the DNA they are considered non-specific with regard to the specific gene being transcribed.'),
  'D035181': NtO(MH='TATA-Box Binding Protein', MS='A general transcription factor that plays a major role in the activation of eukaryotic genes transcribed by RNA POLYMERASES. It binds specifically to the TATA BOX promoter element, which lies close to the position of transcription initiation in RNA transcribed by RNA POLYMERASE II. Although considered a principal component of TRANSCRIPTION FACTOR TFIID it also takes part in general transcription factor complexes involved in RNA POLYMERASE I and RNA POLYMERASE III transcription.'),
  'D035182': NtO(MH='TATA Box Binding Protein-Like Proteins', MS='A class of proteins related in structure and function to TATA-BOX BINDING PROTEIN that can take the place of TATA-BOX BINDING PROTEIN in the transcription initiation complex. They are found in most multicellular organisms and may be involved in tissue-specific promoter regulation. They bind to DNA and interact with TATA-BINDING PROTEIN ASSOCIATED FACTORS, however they may lack specificity for the TATA-BOX.'),
  'D035201': NtO(MH='Brachiaria', MS='A plant genus of the family POACEAE originating from the savanna of eastern Africa. It is widely grown for livestock forage.'),
  'D035261': NtO(MH='Plant Components, Aerial', MS='The above-ground plant without the roots.'),
  'D035262': NtO(MH='Cranial Fossa, Anterior', MS='The compartment containing the inferior part and anterior extremities of the frontal lobes (FRONTAL LOBE) of the cerebral hemispheres. It is formed mainly by orbital parts of the FRONTAL BONE and the lesser wings of the SPHENOID BONE.'),
  'D035263': NtO(MH='Flowering Tops', MS='Tops of plants when in flower, including the stems, leaves and blooms.'),
  'D035264': NtO(MH='Flowers', MS='The reproductive organs of plants.'),
  'D035281': NtO(MH='Plant Tubers', MS='An enlarged underground root or stem of some plants. It is usually rich in carbohydrates. Some, such as POTATOES, are important human FOOD. They may reproduce vegetatively from buds.'),
  'D035301': NtO(MH='Cranial Fossa, Middle', MS='The compartment containing the anterior extremities and half the inferior surface of the temporal lobes (TEMPORAL LOBE) of the cerebral hemispheres. Lying posterior and inferior to the anterior cranial fossa (CRANIAL FOSSA, ANTERIOR), it is formed by part of the TEMPORAL BONE and SPHENOID BONE. It is separated from the posterior cranial fossa (CRANIAL FOSSA, POSTERIOR) by crests formed by the superior borders of the petrous parts of the temporal bones.'),
  'D035321': NtO(MH='Telomeric Repeat Binding Protein 1', MS='A ubiquitously expressed telomere-binding protein that is present at TELOMERES throughout the CELL CYCLE. It is a suppressor of telomere elongation and may be involved in stabilization of telomere length. It is structurally different from TELOMERIC REPEAT BINDING PROTEIN 2 in that it contains acidic N-terminal amino acid residues.'),
  'D035322': NtO(MH='Melastomataceae', MS='A plant family of the order Myrtales, subclass Rosidae, class Magnoliopsida composed of tropical plants with parallel-nerved leaves.'),
  'D035341': NtO(MH='Telomeric Repeat Binding Protein 2', MS='A ubiquitously expressed telomere-binding protein that is present at TELOMERES throughout the cell cycle. It is a suppressor of telomere elongation and may be involved in stabilization of telomere length. It is structurally different from TELOMERIC REPEAT BINDING PROTEIN 1 in that it contains basic N-terminal amino acid residues.'),
  'D035361': NtO(MH='Transcription Factor TFIIIB', MS='One of several general transcription factors that are specific for RNA POLYMERASE III. TFIIIB recruits and positions pol III over the initiation site and remains stably bound to the DNA through multiple rounds of re-initiation by RNA POLYMERASE III.'),
  'D035362': NtO(MH='Transcription Factor TFIID', MS='The major sequence-specific DNA-binding component involved in the activation of transcription of RNA POLYMERASE II. It was originally described as a complex of TATA-BOX BINDING PROTEIN and TATA-BINDING PROTEIN ASSOCIATED FACTORS. It is now know that TATA BOX BINDING PROTEIN-LIKE PROTEINS may take the place of TATA-box binding protein in the complex.'),
  'D035381': NtO(MH='TATA-Binding Protein Associated Factors', MS='Factors that associate with TATA-BOX BINDING PROTEIN. Many of them are components of TRANSCRIPTION FACTOR TFIID'),
  'D035401': NtO(MH='Conyza', MS='A plant genus of the family ASTERACEAE. Members contain alkenynes, daucosterol, friedelinol, conyzasaponins and other TRITERPENES.'),
  'D035421': NtO(MH='Parotid Region', MS='The region of the face on either side, around the PAROTID GLAND.'),
  'D035422': NtO(MH='Pleural Cavity', MS='Paired but separate cavity within the THORACIC CAVITY. It consists of the space between the parietal and visceral PLEURA and normally contains a capillary layer of serous fluid that lubricates the pleural surfaces.'),
  'D035423': NtO(MH='Thoracic Cavity', MS='The region of the thorax that includes the PLEURAL CAVITY and MEDIASTINUM.'),
  'D035441': NtO(MH='Thoracic Wall', MS='The outer margins of the thorax containing SKIN, deep FASCIA; THORACIC VERTEBRAE; RIBS; STERNUM; and MUSCLES.'),
  'D035462': NtO(MH='Holarrhena', MS='A plant genus of the family APOCYNACEAE. Members contain holarrhenine (a steroidal alkaloid) and TRICHOTHECENES.'),
  'D035463': NtO(MH='Pol1 Transcription Initiation Complex Proteins', MS='Factors that form a preinitiation complex at promoters that are specifically transcribed by RNA POLYMERASE I.'),
  'D035481': NtO(MH='Retrospective Moral Judgment', MS='The application of current standards of morality to past actions, institutions, or persons.'),
  'D035501': NtO(MH='Uncertainty', MS='The condition in which reasonable knowledge regarding risks, benefits, or the future is not available.'),
  'D035502': NtO(MH='Trust', MS='Confidence in or reliance on a person or thing.'),
  'D035503': NtO(MH='Sociobiology', MS="The comparative study of social organization in animals including humans, especially with regard to its genetic basis and evolutionary history. (Merriam-Webster's Collegiate Dictionary, 10th ed)"),
  'D035521': NtO(MH='Supreme Court Decisions', MS='Decisions made by the United States Supreme Court.'),
  'D035561': NtO(MH='Transcription Factor TFIIA', MS='An RNA POLYMERASE II specific transcription factor. It may play a role in transcriptional activation of gene expression by interacting with the TATA-BOX BINDING PROTEIN component of TRANSCRIPTION FACTOR TFIID.'),
  'D035581': NtO(MH='Transcription Factor TFIIB', MS='An RNA POLYMERASE II specific transcription factor. It plays a role in assembly of the pol II transcriptional preinitiation complex and has been implicated as a target of gene-specific transcriptional activators.'),
  'D035582': NtO(MH='Transcription Factor TFIIIA', MS='One of several general transcription factors that are specific for RNA POLYMERASE III. It is a zinc finger (ZINC FINGERS) protein and is required for transcription of 5S ribosomal genes.'),
  'D035583': NtO(MH='Rare Diseases', MS='A large group of diseases which are characterized by a low prevalence in the population. They frequently are associated with problems in diagnosis and treatment.'),
  'D035602': NtO(MH='Transcriptional Elongation Factors', MS='Transcription factors whose primary function is to regulate the rate in which RNA is transcribed.'),
  'D035621': NtO(MH='Secularism', MS="Indifference to, or rejection of, RELIGION or religious considerations. (From Merriam-Webster's Collegiate Dictionary, 10th ed)"),
  'D035622': NtO(MH='Parental Notification', MS='Reporting to parents or guardians about care to be provided to a minor (MINORS).'),
  'D035641': NtO(MH='Chest Wall Oscillation', MS='A respiratory support system used to remove mucus and clear airway by oscillating pressure on the chest.'),
  'D035642': NtO(MH='Whistleblowing', MS='The reporting of observed or suspected PROFESSIONAL MISCONDUCT or incompetence to appropriate authorities or to the public.'),
  'D035661': NtO(MH='Animal Care Committees', MS='Institutional committees established to protect the welfare of animals used in research and education. The 1971 NIH Guide for the Care and Use of Laboratory Animals introduced the policy that institutions using warm-blooded animals in projects supported by NIH grants either be accredited by a recognized professional laboratory animal accrediting body or establish its own committee to evaluate animal care; the Public Health Service adopted a policy in 1979 requiring such committees; and the 1985 amendments to the Animal Welfare Act mandate review and approval of federally funded research with animals by a formally designated Institutional Animal Care and Use Committee (IACUC).'),
  'D035681': NtO(MH='Adiantum', MS='A plant genus of the family Pteridaceae. Members contain TRITERPENES. Some species in this genus are called maidenhair fern which is also a common name occasionally used for Lygodium (FERNS) and POLYPODIUM.'),
  'D035682': NtO(MH='Gnetophyta', MS='A plant division of seed plants containing only a few members.'),
  'D035683': NtO(MH='MicroRNAs', MS="Small double-stranded, non-protein coding RNAs, 21-25 nucleotides in length generated from single-stranded microRNA gene transcripts by the same RIBONUCLEASE III, Dicer, that produces small interfering RNAs (RNA, SMALL INTERFERING). They become part of the RNA-INDUCED SILENCING COMPLEX and repress the translation (TRANSLATION, GENETIC) of target RNA by binding to homologous 3'UTR region as an imperfect match. The small temporal RNAs (stRNAs), let-7 and lin-4, from C. elegans, are the first 2 miRNAs discovered, and are from a class of miRNAs involved in developmental timing."),
  'D035684': NtO(MH='Gnetum', MS='A plant genus of the family Gnetaceae, order Gnetales class Gnetopsida, division GNETOPHYTA. Members contain STILBENES and benzylisoquinoline alkaloids.'),
  'D035685': NtO(MH='Andrographis', MS='A plant genus of the family ACANTHACEAE. Members contain andrographolide and other DITERPENES and androechin, a CHALCONE.'),
  'D035701': NtO(MH='Eurycoma', MS='A plant genus of the family SIMAROUBACEAE. Members contain quassinoids. There is Malaysian folk use of these plants for male virility.'),
  'D035702': NtO(MH='Cecropia Plant', MS='A plant genus of the family CECROPIACEAE. Hypotensive and hypoglycemic effects have been observed in animals after ingesting members of this genus. There is no relation to cecropia moth (Hyalophora cecropia) see MOTHS.'),
  'D035703': NtO(MH='Therapies, Investigational', MS='Treatments which are undergoing clinical trials or for which there is insufficient evidence to determine their effects on health outcomes; coverage for such treatments is often denied by health insurers.'),
  'D035704': NtO(MH='Molluginaceae', MS='A plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida. Some members contain triterpenoid saponins.'),
  'D035721': NtO(MH='Pterocarpus', MS='A plant genus of the family FABACEAE. Members contain TRITERPENES.'),
  'D035722': NtO(MH='Aizoaceae', MS='A plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida.'),
  'D035723': NtO(MH='Mesembryanthemum', MS='A plant genus of the family AIZOACEAE. It is a native of Africa and widely planted for erosion control to stabilize soil along roadsides and beaches.'),
  'D035741': NtO(MH='Vernonia', MS='A plant genus of the family ASTERACEAE. Members contain germacrane and sesquiterpene LACTONES.'),
  'D035742': NtO(MH='Mikania', MS='A plant genus of the family ASTERACEAE. Members contain scandenolide (a sesquiterpene lactone) and germacranolides.'),
  'D035761': NtO(MH='Wedge Argument', MS='An assertion that an action apparently unobjectionable in itself would set in motion a train of events leading ultimately to an undesirable outcome. (From Cambridge Dictionary of Philosophy, 1995)'),
  'D035781': NtO(MH='Siblings', MS='Persons or animals having at least one parent in common. (American College Dictionary, 3d ed)'),
  'D035782': NtO(MH='Dalbergia', MS='A plant genus of the family FABACEAE. Members of this genus can cause CONTACT DERMATITIS.'),
  'D035801': NtO(MH='Amomum', MS='A plant genus of the family ZINGIBERACEAE. Members contain aculeatin D, beta-sitosterol, and STIGMASTEROL. Some members have been reclassified to ELETTARIA and other ZINGIBERACEAE.'),
  'D035841': NtO(MH='Behavioral Research', MS='Research that involves the application of the behavioral and social sciences to the study of the actions or reactions of persons or animals in response to external or internal stimuli. (from American Heritage Dictionary, 4th ed)'),
  'D035842': NtO(MH='Research Subjects', MS='Persons who are enrolled in research studies or who are otherwise the subjects of research.'),
  'D035843': NtO(MH='Biomedical Research', MS='Research that involves the application of the natural sciences, especially biology and physiology, to medicine.'),
  'D035844': NtO(MH='Researcher-Subject Relations', MS='Interaction between research personnel and research subjects.'),
  'D035845': NtO(MH='Research Embryo Creation', MS='The creation of embryos specifically for research purposes.'),
  'D035846': NtO(MH='Law Enforcement', MS='Organized efforts to insure obedience to the laws of a community.'),
  'D035862': NtO(MH='Vulnerable Populations', MS='Groups of persons whose range of options is severely limited, who are frequently subjected to COERCION in their DECISION MAKING, or who may be compromised in their ability to give INFORMED CONSENT.'),
  'D035863': NtO(MH='Moral Development', MS='The process by which individuals internalize standards of right and wrong conduct.'),
  'D035881': NtO(MH='Compensation and Redress', MS='Payment, or other means of making amends, for a wrong or injury.'),
  'D035901': NtO(MH='Hypocrea', MS='A genus of fungus in the family Hypocreaceae, order HYPOCREALES. Anamorphs include TRICHODERMA.'),
  'D035921': NtO(MH='Hepatitis A Antigens', MS='Antigens produced by various strains of HEPATITIS A VIRUS such as the human hepatitis A virus (HEPATITIS A VIRUS, HUMAN).'),
  'D035922': NtO(MH='Hepatitis A Antibodies', MS='Antibodies to the HEPATITIS A ANTIGENS including antibodies to envelope, core, and non-structural proteins.'),
  'D035923': NtO(MH='Hepatitis delta Antigens', MS='Antigens produced by various strains of HEPATITIS D VIRUS.'),
  'D035924': NtO(MH='Montanoa', MS='A plant genus of the family ASTERACEAE. Steroidal activity has been observed in animals. Members of this genus contain sesquiterpene lactones such as tomentosin, OXEPINS such as zoapatanol, and kaurene type DITERPENES.'),
  'D035925': NtO(MH='Iron-Regulatory Proteins', MS='Proteins that regulate cellular and organismal iron homeostasis. They play an important biological role by maintaining iron levels that are adequate for metabolic need, but below the toxicity threshold.'),
  'D035926': NtO(MH='Hepatophyta', MS='A plant division. They are simple plants that lack vascular tissue and possess rudimentary rootlike organs (rhizoids). Like MOSSES, liverworts have alternation of generations between haploid gamete-bearing forms (gametophytes) and diploid spore-bearing forms (sporophytes).'),
  'D035941': NtO(MH='Iron Regulatory Protein 1', MS='A multifunctional iron-sulfur protein that is both an iron regulatory protein and cytoplasmic form of aconitate hydratase. It binds to iron regulatory elements found on mRNAs involved in iron metabolism and regulates their translation. Its RNA binding ability and its aconitate hydrolase activity are dependent upon availability of IRON.'),
  'D035942': NtO(MH='Iron Regulatory Protein 2', MS='A multifunctional iron-sulfur protein that is both an iron regulatory protein and cytoplasmic form of aconitate hydratase. It binds to iron regulatory elements found on mRNAs involved in iron metabolism and regulates their translation. Its rate of degradation is increased in the presence of IRON.'),
  'D035961': NtO(MH='Alismatales', MS='A plant of the class MAGNOLIOPSIDA. It is a primitive group of mostly aquatic plants.'),
  'D035962': NtO(MH='Zosteraceae', MS='A plant family of the order Najadales, subclass ALISMATIDAE, class Liliopsida (monocotyledons). This is a group of perennial aquatic herbs with basal leaves.'),
  'D035964': NtO(MH='Oxocins', MS='Compounds based on an 8-membered heterocyclic ring including an oxygen. They can be considered medium ring ethers.'),
  'D035981': NtO(MH='Hedera', MS='A plant genus of the family ARALIACEAE. Members contain hederin (olean-12-ene) type TRITERPENES.'),
  'D036002': NtO(MH='ADP Ribose Transferases', MS='Enzymes that transfer the ADP-RIBOSE group of NAD or NADP to proteins or other small molecules. Transfer of ADP-ribose to water (i.e., hydrolysis) is catalyzed by the NADASES. The mono(ADP-ribose)transferases transfer a single ADP-ribose. POLY(ADP-RIBOSE) POLYMERASES transfer multiple units of ADP-ribose to protein targets, building POLY ADENOSINE DIPHOSPHATE RIBOSE in linear or branched chains.'),
  'D036022': NtO(MH='Capsid Proteins', MS='Proteins that form the CAPSID of VIRUSES.'),
  'D036061': NtO(MH='Solanum glaucophyllum', MS='A plant species of the genus SOLANUM, family SOLANACEAE that causes CALCINOSIS in grazing livestock due to high levels of 1a,25-dihydroxy vitamin D3 (CALCITRIOL).'),
  'D036081': NtO(MH='Receptors, Eph Family', MS='A large family of receptor protein-tyrosine kinases that are structurally-related. The name of this family of proteins derives from original protein Eph (now called the EPHA1 RECEPTOR), which was named after the cell line it was first discovered in: Erythropoietin-Producing human Hepatocellular carcinoma cell line. Members of this family have been implicated in regulation of cell-cell interactions involved in nervous system patterning and development.'),
  'D036082': NtO(MH='Receptor, EphA1', MS='The founding member of the EPH FAMILY RECEPTORS. It was first cloned from an erythropoietin-producing human hepatocellular carcinoma cell line and is highly conserved among many mammalian species. Overproduction of the EphA1 receptor is associated with tumors and tumor cells of epithelial origin. It is also expressed at high levels in LIVER; LUNG; and KIDNEY; which is in contrast to many other members of the Eph receptor that are found primarily in tissues of the nervous system.'),
  'D036083': NtO(MH='Atractylis', MS='A plant genus of the family ASTERACEAE. Members contain ATRACTYLOSIDE which inhibits ADENINE NUCLEOTIDE TRANSLOCASE.'),
  'D036084': NtO(MH='Callilepis', MS='A plant genus of the family ASTERACEAE. Members contain ATRACTYLOSIDE.'),
  'D036101': NtO(MH='Cord Blood Stem Cell Transplantation', MS='Transplantation of STEM CELLS collected from the fetal blood remaining in the UMBILICAL CORD and the PLACENTA after delivery. Included are the HEMATOPOIETIC STEM CELLS.'),
  'D036102': NtO(MH='Peripheral Blood Stem Cell Transplantation', MS='Transplantation of PERIPHERAL BLOOD STEM CELLS. It is a less invasive alternative to direct marrow harvesting of hematopoietic stem cells. Enrichment of stem cells in peripheral blood can be achieved by inducing mobilization of stem cells from the BONE MARROW.'),
  'D036103': NtO(MH='Nanotechnology', MS='The development and use of techniques to study physical phenomena and construct structures in the nanoscale size range or smaller.'),
  'D036104': NtO(MH='Receptor, EphA2', MS='An Eph family receptor found abundantly in tissues of epithelial origin. It is expressed in a diverse array of tissues during embryonic development, suggesting that it may play a role in embryogenesis. In adult tissues high levels of the receptor are expressed in the LUNG; SKIN; SMALL INTESTINE and OVARY.'),
  'D036121': NtO(MH='Receptor, EphA3', MS='An eph family receptor that is found primarily in adult BRAIN and variety of tissues in the developing embryo tissues. During embryonic development high levels of EphA3 receptor expression is seen in the nervous system and coincides with neuronal cell migration, suggesting a role for this protein in axonal pathfinding.'),
  'D036122': NtO(MH='Receptor, EphA4', MS='An eph family receptor found in variety of tissues including BRAIN. During embryogenesis, EphA4 receptor exhibits a diverse spatial and temporal patterns of expression suggesting its role in multiple developmental processes.'),
  'D036123': NtO(MH='Receptor, EphA5', MS='An eph family receptor found primarily in differentiated neuronal tissues. Several isoforms of EphA5 receptor occur due to multiple alternative RNA splicing. The protein is prominently expressed in the NEURONS of the LIMBIC SYSTEM during development and throughout adult life, suggesting its role in the plasticity of limbic structure and function.'),
  'D036124': NtO(MH='Receptor, EphA6', MS='An eph family receptor that is found in a variety of neuronal regions of the BRAIN. In contrast to the other proteins in this class, it is expressed at higher levels in adult tissues than in embryonic tissues.'),
  'D036141': NtO(MH='Receptor, EphA7', MS='An eph family receptor found widely expressed in embryo tissues, including the NERVOUS SYSTEM. In the BRAIN high levels of EphA7 expression occurs in the anterior tectum, medulla, RHOMBENCEPHALON, and SUBCOMMISSURAL ORGAN. Several isoforms of the protein occur due to multiple alternative spicing of the EphA7 mRNA.'),
  'D036142': NtO(MH='Mentha piperita', MS='A plant genus of the family LAMIACEAE that is the source of peppermint oil.'),
  'D036143': NtO(MH='Receptor, EphA8', MS='An eph family receptor found exclusively in BRAIN. EphA8 receptors may play a role in the AXONAL GUIDANCE of a subset of tectal commissural NEURONS.'),
  'D036145': NtO(MH='Ginsenosides', MS='Dammarane type triterpene saponins based mainly on the aglycones, protopanaxadiol and protopanaxatriol.'),
  'D036183': NtO(MH='Receptor, EphB2', MS='An eph family receptor found widely expressed in embryonic and adult tissues. High levels of EphB2 receptor are observed in growing AXONS and NERVE FIBERS. Several isoforms of the protein exist due to multiple alternative mRNA splicing.'),
  'D036201': NtO(MH='Receptor, EphB5', MS='An eph family receptor found at high levels in adult THYMUS and RETINA. In embryonic tissues it is found in many developing organs.'),
  'D036223': NtO(MH='Receptor, EphB3', MS='An eph family receptor found in a number of tissues including BRAIN; LUNG; KIDNEY; PANCREAS; INTESTINE; and HEART. During embryogenesis EphB3 receptor is expressed at high levels in the brain.'),
  'D036224': NtO(MH='Receptor, EphB4', MS='An eph family receptor found in a variety of adult and embryonic tissues. Unlike the majority of proteins in this class there is little or no expression of EphB4 receptor in the BRAIN. It has been found at high levels in developing mammary glands and in invasive mammary tumors.'),
  'D036225': NtO(MH='Receptor, EphB1', MS='An eph family receptor found primarily in the nervous system. In the embryonic BRAIN EphB1 receptor expression occurs in the mantle layer and increases with the progression of embryogenesis. In adult brain it is found in the several regions including the CEREBELLUM; CEREBRAL CORTEX; and CAUDATE NUCLEUS; and PUTAMEN.'),
  'D036226': NtO(MH='Seedlings', MS='Young plants produced at the completion of GERMINATION of SEEDS.'),
  'D036261': NtO(MH='Receptor, EphB6', MS='An eph family receptor found primarily in BRAIN and THYMUS. The EphB6 receptor is unusual in that its tyrosine kinase domain shares little homology with other members of this class. The unusual tyrosine kinase domain of this receptor appears to result in its lack of tyrosine kinase activity.'),
  'D036262': NtO(MH='Empirical Research', MS='The study, based on direct observation, use of statistical records, interviews, or experimental methods, of actual practices or the actual impact of practices or policies.'),
  'D036281': NtO(MH='Genetic Research', MS='Research into the cause, transmission, amelioration, elimination, or enhancement of inherited disorders and traits.'),
  'D036301': NtO(MH='Qualitative Research', MS='Any type of research that employs nonnumeric information to explore individual or group characteristics, producing findings not arrived at by statistical procedures or other quantitative means. (Qualitative Inquiry: A Dictionary of Terms Thousand Oaks, CA: Sage Publications, 1997)'),
  'D036302': NtO(MH='Therapeutic Human Experimentation', MS='Human experimentation that is intended to benefit the subjects on whom it is performed.'),
  'D036341': NtO(MH='Intercellular Signaling Peptides and Proteins', MS='Regulatory proteins and peptides that are signaling molecules involved in the process of PARACRINE COMMUNICATION. They are generally considered factors that are expressed by one cell and are responded to by receptors on another nearby cell. They are distinguished from HORMONES in that their actions are local rather than distal.'),
  'D036342': NtO(MH='Ephrins', MS='Signaling proteins that are ligands for the EPH FAMILY RECEPTORS. They are membrane-bound proteins that are attached to the CELL MEMBRANE either through a GLYCOINOSITOL PHOSPHOLIPID MEMBRANE ANCHOR or through a transmembrane domain. Many of the ephrins are considered important intercellular signaling molecules that control morphogenic changes during embryogenesis.'),
  'D036343': NtO(MH='Pterocarpans', MS='A group of compounds which can be described as benzo-pyrano-furano-benzenes which can be formed from ISOFLAVONES by internal coupling of the B ring to the 4-ketone position. Members include medicarpin, phaseolin, and pisatin which are found in FABACEAE.'),
  'D036361': NtO(MH='Peptide Hormones', MS='Hormones synthesized from amino acids. They are distinguished from INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS in that their actions are systemic.'),
  'D036381': NtO(MH='Diarylheptanoids', MS='A group of compounds consisting of two aryl rings linked by seven carbons (HEPTANES) which may be acyclic (linear) or cyclic. It is a group of secondary metabolites in polyketide synthesis pathway best exemplified by a linear diarylheptanoid CURCUMIN.'),
  'D036382': NtO(MH='Ephrin-A1', MS='An ephrin that was originally identified as the product of an early response gene induced by TUMOR NECROSIS FACTORS. It is linked to the CELL MEMBRANE via a GLYCOINOSITOL PHOSPHOLIPID MEMBRANE ANCHOR and binds EPHA2 RECEPTOR with high affinity. During embryogenesis high levels of ephrin-A1 are expressed in LUNG; KIDNEY; SALIVARY GLANDS; and INTESTINE.'),
  'D036383': NtO(MH='Ephrin-A2', MS='A GLYCOINOSITOL PHOSPHOLIPID MEMBRANE ANCHOR-containing ephrin with a high affinity for the EPHA3 RECEPTOR. Early in embryogenesis it is expressed at high levels in the MESENCEPHALON; SOMITES; branchial arches, and LIMB BUDS.'),
  'D036384': NtO(MH='Ephrin-A3', MS='A GLYCOINOSITOL PHOSPHOLIPID MEMBRANE ANCHOR containing ephrin with a high affinity for the EPHA3 RECEPTOR; EPHA5 RECEPTOR; EPHA6 RECEPTOR; EPHA7 RECEPTOR; and EPHA8 RECEPTOR. It is found primarily in NERVE TISSUE.'),
  'D036385': NtO(MH='Ephrin-A4', MS='A GLYCOINOSITOL PHOSPHOLIPID MEMBRANE ANCHOR containing ephrin found in variety of adult tissue such as KIDNEY; LUNG; COLON and OVARY. During embryonic development, ephrin-A4 is expressed in the HIPPOCAMPUS and CEREBRAL CORTEX. It binds preferentially to ephA receptors (EPH FAMILY RECEPTORS) with the exception of the EPHA1 RECEPTOR.'),
  'D036386': NtO(MH='Ephrin-A5', MS='A GLYCOINOSITOL PHOSPHOLIPID MEMBRANE ANCHOR containing ephrin found in developing tectum. It has been shown to mediate the bundling of cortical axons and repel the axonal growth of retinal ganglia axons. It is found in a variety of adult tissues of BRAIN; HEART; and KIDNEY.'),
  'D036387': NtO(MH='Ephrin-B1', MS='A transmembrane domain containing ephrin that is specific for EPHB1 RECEPTOR; EPHB2 RECEPTOR and EPHB3 RECEPTOR. It is widely expressed in a variety of developing and adult tissues.'),
  'D036388': NtO(MH='Ephrin-B2', MS='A transmembrane domain containing ephrin that binds with high affinity to EPHB1 RECEPTOR; EPHB3 RECEPTOR; and EPHB4 RECEPTOR. Expression of ephrin-B2 occurs in a variety of adult tissues. During embryogenesis, high levels of ephrin-B2 is seen in the PROSENCEPHALON; RHOMBENCEPHALON; developing SOMITES; LIMB BUD; and bronchial arches.'),
  'D036389': NtO(MH='Ephrin-B3', MS='A transmembrane domain containing ephrin. Although originally found to be specific for the EPHB3 RECEPTOR it has since been shown to bind a variety of EPH FAMILY RECEPTORS. During embryogenesis ephrin-B3 is expressed at high levels in the ventral neural tube. In adult tissues, it is found primarily in the BRAIN and HEART.'),
  'D036401': NtO(MH='Toxicity Tests, Chronic', MS='Experiments designed to determine the potential toxic effects of a long-term exposure to a chemical or chemicals.'),
  'D036421': NtO(MH='Nerve Fibers, Unmyelinated', MS='A class of nerve fibers as defined by their nerve sheath arrangement. The AXONS of the unmyelinated nerve fibers are small in diameter and usually several are surrounded by a single MYELIN SHEATH. They conduct low-velocity impulses, and represent the majority of peripheral sensory and autonomic fibers, but are also found in the BRAIN and SPINAL CORD.'),
  'D036441': NtO(MH='Osmeriformes', MS='An order of fish including smelts, galaxiids, and salamanderfish.'),
  'D036461': NtO(MH='Mallotus Plant', MS='A plant genus of the family EUPHORBIACEAE. Members contain fredelin type TRITERPENES, mallorepine (a cyano-pyridone), and hydrolyzable TANNINS.'),
  'D036481': NtO(MH='Bauhinia', MS='A plant genus of the family FABACEAE. Members contain isoacteoside, luteolin, indole-3-carboxylic acid.'),
  'D036501': NtO(MH='Metered Dose Inhalers', MS='A small aerosol canister used to release a calibrated amount of medication for inhalation.'),
  'D036502': NtO(MH='Infusions, Intralesional', MS='The administration of medication or fluid directly into localized lesions, by means of gravity flow or INFUSION PUMPS.'),
  'D036503': NtO(MH='Inhalation Spacers', MS='A variety of devices used in conjunction with METERED DOSE INHALERS. Their purpose is to hold the released medication for inhalation and make it easy for the patients to inhale the metered dose of medication into their lungs.'),
  'D036541': NtO(MH='ADP-ribosyl Cyclase', MS="A membrane-bound or cytosolic enzyme that catalyzes the synthesis of CYCLIC ADP-RIBOSE (cADPR) from nicotinamide adenine dinucleotide (NAD). This enzyme generally catalyzes the hydrolysis of cADPR to ADP-RIBOSE, as well, and sometimes the synthesis of cyclic ADP-ribose 2' phosphate (2'-P-cADPR) from NADP."),
  'D036542': NtO(MH='Tomography, Spiral Computed', MS='Computed tomography where there is continuous X-ray exposure to the patient while being transported in a spiral or helical pattern through the beam of irradiation. This provides improved three-dimensional contrast and spatial resolution compared to conventional computed tomography, where data is obtained and computed from individual sequential exposures.'),
  'D036561': NtO(MH='Cajanus', MS='A plant genus of the family FABACEAE that is used for food in NIGERIA.'),
  'D036562': NtO(MH='Helichrysum', MS='A plant genus of the family ASTERACEAE. Members contain CHALCONE, helichrysetin, arenarin, and flamin.'),
  'D036563': NtO(MH='Cyclic ADP-Ribose', MS='A pyridine nucleotide that mobilizes CALCIUM. It is synthesized from nicotinamide adenine dinucleotide (NAD) by ADP RIBOSE CYCLASE.'),
  'D036581': NtO(MH='Ratibida', MS='A plant genus of the family ASTERACEAE. Members contain sesquiterpene lactones.'),
  'D036582': NtO(MH='Rudbeckia', MS='A plant genus of the family ASTERACEAE. Some Rudbeckia species have been reclassified to ECHINACEA; RATIBIDA; or HELIANTHUS.'),
  'D036601': NtO(MH='Eclipta', MS='A plant genus of the family ASTERACEAE. Members contain wedelolactone.'),
  'D036602': NtO(MH='Eupatorium', MS='A plant genus of the family ASTERACEAE. Ingestion has been associated with MILK SICKNESS. The common name of thoroughwort is also used for other plants including CHROMOLAENA; Hebeclinium, and Koanophyllon.'),
  'D036603': NtO(MH='Ageratina', MS='A plant genus of the family ASTERACEAE. The common name of snakeroot is also used for POLYGALA; SANICULA; ARISTOLOCHIA and others.'),
  'D036604': NtO(MH='Chromolaena', MS='A plant genus of the family ASTERACEAE. The common name of thoroughwort is also used for other plants including EUPATORIUM; CHROMOLAENA, Hebeclinium and Koanophyllon. Eupolin is the aqueous extract of the leaves.'),
  'D036621': NtO(MH='Geigeria', MS='A plant genus of the family ASTERACEAE that causes vermeersiekte poisoning of sheep in South Africa. Members contain dihydrogriesenin and ivalin (sesquiterpene lactones).'),
  'D036622': NtO(MH='Gnaphalium', MS='A plant genus of the family ASTERACEAE. The common name of cudweed is also used for other plants including Evax, Hesperevax, Pseudognaphalium, and Omalotheca. The common name of everlasting is also used for other plants including Anaphalis, Antennaria, HELICHRYSUM, Gamochaeta, and Pseudognaphalium.'),
  'D036623': NtO(MH='Grindelia', MS='A plant genus of the family ASTERACEAE. Members contain triterpenoid sapogenins.'),
  'D036624': NtO(MH='Solidago', MS='A plant genus of the family Asteraceae known for allergenic pollen (ALLERGENS).'),
  'D036625': NtO(MH='Stevia', MS='A plant genus of the family ASTERACEAE. Members contain stevioside and other sweet diterpene glycosides. The leaf is used for sweetening (SWEETENING AGENTS).'),
  'D036641': NtO(MH='Microscopy, Fluorescence, Multiphoton', MS='Fluorescence microscopy utilizing multiple low-energy photons to produce the excitation event of the fluorophore (endogenous fluorescent molecules in living tissues or FLUORESCENT DYES). Multiphoton microscopes have a simplified optical path in the emission side due to the lack of an emission pinhole, which is necessary with normal confocal microscopes. Ultimately this allows spatial isolation of the excitation event, enabling deeper imaging into optically thick tissue, while restricting photobleaching and phototoxicity to the area being imaged.'),
  'D036661': NtO(MH='Echium', MS='A plant genus of the family BORAGINACEAE. Members contain GAMMA-LINOLENIC ACID and PYRROLIZIDINE ALKALOIDS.'),
  'D036681': NtO(MH='Fluorescence Recovery After Photobleaching', MS="A method used to study the lateral movement of MEMBRANE PROTEINS and LIPIDS. A small area of a cell membrane is bleached by laser light and the amount of time necessary for unbleached fluorescent marker-tagged proteins to diffuse back into the bleached site is a measurement of the cell membrane's fluidity. The diffusion coefficient of a protein or lipid in the membrane can be calculated from the data. (From Segen, Current Med Talk, 1995)."),
  'D036701': NtO(MH='Limonins', MS='A group of degraded TRITERPENES in which the four terminal carbons of the C17 side chain have been removed, and the remaining portion often forming C17 furans.'),
  'D036702': NtO(MH='Quassins', MS='A group of degraded TRITERPENES which have lost ten carbons (8 from the side chain, C17, and one from C4) so have C20 skeletons which could be misinterpreted as DITERPENES. They especially occur in plants of the SIMAROUBACEAE family.'),
  'D036703': NtO(MH='Blastula', MS='An early non-mammalian embryo that follows the MORULA stage. A blastula resembles a hollow ball with the layer of cells surrounding a fluid-filled cavity (blastocele). The layer of cells is called BLASTODERM.'),
  'D036721': NtO(MH='Scoparia', MS='A plant genus of the family Plantaginaceae. Members contain thyrsiflorin and other scopadulane (labdane) type DITERPENES.'),
  'D036741': NtO(MH='Plant Tumor-Inducing Plasmids', MS='Plasmids coding for proteins which induce PLANT TUMORS. The most notable example of a plant tumor inducing plasmid is the Ti plasmid found associated with AGROBACTERIUM TUMEFACIENS.'),
  'D036761': NtO(MH='Ultrasound, High-Intensity Focused, Transrectal', MS='Tissue ablation of the PROSTATE performed by ultrasound from a transducer placed in the RECTUM. The procedure is used to treat prostate cancer (PROSTATIC NEOPLASMS) and benign prostatic hypertrophy (PROSTATIC HYPERPLASIA).'),
  'D036781': NtO(MH='Fimbriae Proteins', MS='Proteins that are structural components of bacterial fimbriae (FIMBRIAE, BACTERIAL) or sex pili (PILI, SEX).'),
  'D036801': NtO(MH='Parturition', MS='The process of giving birth to one or more offspring.'),
  'D036821': NtO(MH='Endometrial Stromal Tumors', MS='Neoplasms of the endometrial stroma that sometimes involve the MYOMETRIUM. These tumors contain cells that may closely or remotely resemble the normal stromal cells. Endometrial stromal neoplasms are divided into three categories: (1) benign stromal nodules; (2) low-grade stromal sarcoma, or endolymphatic stromal myosis; and (3) malignant endometrial stromal sarcoma (SARCOMA, ENDOMETRIAL STROMAL).'),
  'D036841': NtO(MH='Ciguatera Poisoning', MS='Poisoning caused by ingestion of SEAFOOD containing microgram levels of CIGUATOXINS. The poisoning is characterized by gastrointestinal, neurological and cardiovascular disturbances.'),
  'D036861': NtO(MH='Delivery, Obstetric', MS='Delivery of the FETUS and PLACENTA under the care of an obstetrician or a health worker. Obstetric deliveries may involve physical, psychological, medical, or surgical interventions.'),
  'D036881': NtO(MH='Long-Term Synaptic Depression', MS='A persistent activity-dependent decrease in synaptic efficacy between NEURONS. It typically occurs following repeated low-frequency afferent stimulation, but it can be induced by other methods. Long-term depression appears to play a role in MEMORY.'),
  'D036882': NtO(MH='Kalopanax', MS='A plant genus of the family ARALIACEAE. Members contain triterpene saponins.'),
  'D036901': NtO(MH='Petasites', MS='A plant genus of the family ASTERACEAE. Members contain SESQUITERPENES. The common name of sweet coltsfoot is similar to the common name for TUSSILAGO.'),
  'D036902': NtO(MH='Voacanga', MS='A plant genus of the family APOCYNACEAE. Members contain voacangine and other INDOLES.'),
  'D036903': NtO(MH='Aspalathus', MS='A plant genus of the family FABACEAE. It is the source of an herbal tea that is commonly consumed in South Africa. Members contain aspalathin and other polyphenols (PHENOLS).'),
  'D036904': NtO(MH='Leukocyte Rolling', MS='Movement of tethered, spherical LEUKOCYTES along the endothelial surface of the microvasculature. The tethering and rolling involves interaction with SELECTINS and other adhesion molecules in both the ENDOTHELIUM and leukocyte. The rolling leukocyte then becomes activated by CHEMOKINES, flattens out, and firmly adheres to the endothelial surface in preparation for transmigration through the interendothelial cell junction. (From Abbas, Cellular and Molecular Immunology, 3rd ed)'),
  'D036905': NtO(MH='Dictamnus', MS='A plant genus of the family RUTACEAE. Members contain eudesmane type SESQUITERPENES; COUMARINS, and obacunone (a limonoid).'),
  'D036921': NtO(MH='Achyrocline', MS='A plant genus of the family ASTERACEAE. Members contain TERPENES and achyrofuran, a prenylated dibenzofuran (BENZOFURANS).'),
  'D036941': NtO(MH='Peperomia', MS='A plant genus of the family PIPERACEAE. Members contain prenylated quinones.'),
  'D036961': NtO(MH='Matrix Attachment Region Binding Proteins', MS='Proteins that bind to the MATRIX ATTACHMENT REGIONS of DNA.'),
  'D036981': NtO(MH='Fasciitis, Plantar', MS='Inflammation of the plantar fascia (APONEUROSIS) on the bottom of the foot causing heel pain. The etiology of plantar fasciitis remains controversial but is likely to involve a biomechanical imbalance. Though often presenting along with HEEL SPUR, they do not appear to be causally related.'),
  'D036982': NtO(MH='Heel Spur', MS='A bony outgrowth on the lower surface of the CALCANEUS. Though often presenting along with plantar fasciitis (FASCIITIS, PLANTAR), they are not considered causally related.'),
  'D037001': NtO(MH='Directive Counseling', MS="Counseling during which a professional plays an active role in a client's or patient's decision making by offering advice, guidance, and/or recommendations."),
  'D037021': NtO(MH='Thyroid Hormone Receptors alpha', MS='High affinity receptors for THYROID HORMONES, especially TRIIODOTHYRONINE. These receptors are usually found in the nucleus where they regulate DNA transcription. They are encoded by the THRA gene (also known as NR1A1, THRA1, ERBA or ERBA1 gene) as several isoforms produced by alternative splicing.'),
  'D037022': NtO(MH='Posthumous Conception', MS='Conception after the death of the male or female biological parent through techniques such as the use of gametes that have been stored during his or her lifetime or that were collected immediately after his or her death.'),
  'D037041': NtO(MH='United States Government Agencies', MS='Agencies of the FEDERAL GOVERNMENT of the United States.'),
  'D037042': NtO(MH='Thyroid Hormone Receptors beta', MS='High affinity receptors for THYROID HORMONES, especially TRIIODOTHYRONINE. These receptors are usually found in the nucleus where they regulate DNA transcription. They are encoded by the THRB gene (also known as NR1A2, THRB1, or ERBA2 gene) as several isoforms produced by alternative splicing. Mutations in the THRB gene cause THYROID HORMONE RESISTANCE SYNDROME.'),
  'D037061': NtO(MH='Metatarsalgia', MS='Pain in the region of the METATARSUS. It can include pain in the METATARSAL BONES; METATARSOPHALANGEAL JOINT; and/or intermetatarsal joints (TARSAL JOINTS).'),
  'D037081': NtO(MH='Posterior Tibial Tendon Dysfunction', MS='A condition characterized by a broad range of progressive disorders ranging from TENOSYNOVITIS to tendon rupture with or without hindfoot collapse to a fixed, rigid, FLATFOOT deformity. Pathologic changes can involve associated tendons, ligaments, joint structures of the ANKLE, hindfoot, and midfoot. Posterior tibial tendon dysfunction is the most common cause of acquired flatfoot deformity in adults.'),
  'D037101': NtO(MH='Luteinizing Hormone, beta Subunit', MS='The beta subunit of luteinizing hormone. It is a 15-kDa glycopolypeptide with structure similar to the beta subunit of the placental chorionic gonadatropin (CHORIONIC GONADOTROPIN, BETA SUBUNIT, HUMAN) except for the additional 31 amino acids at the C-terminal of CG-beta. Full biological activity of LH requires the non-covalently bound heterodimers of an alpha and a beta subunit. Mutation of the LHB gene causes HYPOGONADISM and infertility.'),
  'D037102': NtO(MH='Lectins', MS='Proteins that share the common characteristic of binding to carbohydrates. Some ANTIBODIES and carbohydrate-metabolizing proteins (ENZYMES) also bind to carbohydrates, however they are not considered lectins. PLANT LECTINS are carbohydrate-binding proteins that have been primarily identified by their hemagglutinating activity (HEMAGGLUTININS). However, a variety of lectins occur in animal species where they serve diverse array of functions through specific carbohydrate recognition.'),
  'D037121': NtO(MH='Plant Lectins', MS='Protein or glycoprotein substances of plant origin that bind to sugar moieties in cell walls or membranes. Some carbohydrate-metabolizing proteins (ENZYMES) from PLANTS also bind to carbohydrates, however they are not considered lectins. Many plant lectins change the physiology of the membrane of BLOOD CELLS to cause agglutination, mitosis, or other biochemical changes. They may play a role in plant defense mechanisms.'),
  'D037141': NtO(MH='Mandatory Programs', MS='Programs in which participation is required.'),
  'D037142': NtO(MH='Flaveria', MS='A plant genus of the family ASTERACEAE that is used for experiments in molecular genetic studies in plant physiology and development.'),
  'D037161': NtO(MH='Galectins', MS='A class of animal lectins that bind specifically to beta-galactoside in a calcium-independent manner. Members of this class are distiguished from other lectins by the presence of a conserved carbohydrate recognition domain. The majority of proteins in this class bind to sugar molecules in a sulfhydryl-dependent manner and are often referred to as S-type lectins, however this property is not required for membership in this class.'),
  'D037181': NtO(MH='Lectins, C-Type', MS='A class of animal lectins that bind to carbohydrate in a calcium-dependent manner. They share a common carbohydrate-binding domain that is structurally distinct from other classes of lectins.'),
  'D037182': NtO(MH='T-Cell Antigen Receptor Specificity', MS="The property of the T-CELL RECEPTOR which enables it to react with some antigens and not others. The specificity is derived from the structure of the receptor's variable region which has the ability to recognize certain antigens in conjunction with the MAJOR HISTOCOMPATIBILITY COMPLEX molecule."),
  'D037201': NtO(MH='Follicle Stimulating Hormone, beta Subunit', MS='The beta subunit of follicle stimulating hormone. It is a 15-kDa glycopolypeptide. Full biological activity of FSH requires the non-covalently bound heterodimers of an alpha and a beta subunit. Mutation of the FSHB gene causes delayed puberty, or infertility.'),
  'D037222': NtO(MH='Collectins', MS='A class of C-type lectins that target the carbohydrate structures found on invading pathogens. Binding of collectins to microorganisms results in their agglutination and enhanced clearance. Collectins form trimers that may assemble into larger oligomers. Each collectin polypeptide chain consists of four regions: a relatively short N-terminal region, a collagen-like region, an alpha-helical coiled-coil region, and carbohydrate-binding region.'),
  'D037241': NtO(MH='Mannose-Binding Lectins', MS='A subclass of lectins that are specific for CARBOHYDRATES that contain MANNOSE.'),
  'D037263': NtO(MH='Asialoglycoprotein Receptor', MS='A C-type lectin that is a cell surface receptor for ASIALOGLYCOPROTEINS. It is found primarily in the LIVER where it mediates the endocytosis of serum glycoproteins.'),
  'D037281': NtO(MH='Calnexin', MS='A lectin found in ENDOPLASMIC RETICULUM membranes that binds to specific N-linked OLIGOSACCHARIDES found on newly synthesized proteins. It may play role in PROTEIN FOLDING or retention and degradation of misfolded proteins in the endoplasmic reticulum.'),
  'D037282': NtO(MH='Calreticulin', MS='A multifunctional protein that is found primarily within membrane-bound organelles. In the ENDOPLASMIC RETICULUM it binds to specific N-linked oligosaccharides found on newly-synthesized proteins and functions as a MOLECULAR CHAPERONE that may play a role in PROTEIN FOLDING or retention and degradation of misfolded proteins. In addition calreticulin is a major storage form for CALCIUM and functions as a calcium-signaling molecule that can regulate intracellular calcium HOMEOSTASIS.'),
  'D037302': NtO(MH='Ctenophora', MS='Phylum of marine colenterates characterized by eight comb rows of fused cilia on the body surface. In contrast to CNIDARIA they lack stinging cells, but they are voracious predators and possess sticky cells (colloblasts) for capturing prey. Most species are transparent and many exhibit bioluminescence.'),
  'D037322': NtO(MH='Thyrotropin, beta Subunit', MS='The beta subunit of thyroid stimulating hormone, thyrotropin. It is a 112-amino acid glycopolypeptide of about 16 kD. Full biological activity of TSH requires the non-covalently bound heterodimers of an alpha and a beta subunit.'),
  'D037341': NtO(MH='Fumonisins', MS='A group of MYCOTOXINS found in CORN contaminated with FUSARIUM fungus. They are chains of about 20 carbons with acidic ester, acetylamino and sometimes other substituents. They inhibit ceramide synthetase conversion of SPHINGOLIPIDS to CERAMIDES.'),
  'D037342': NtO(MH='Pertussis Toxin', MS='One of the virulence factors produced by BORDETELLA PERTUSSIS. It is a multimeric protein composed of five subunits S1 - S5. S1 contains mono ADPribose transferase activity.'),
  'D037361': NtO(MH='Adenylate Cyclase Toxin', MS='One of the virulence factors produced by virulent BORDETELLA organisms. It is a bifunctional protein with both ADENYLYL CYCLASES and hemolysin components.'),
  'D037401': NtO(MH='Scyphozoa', MS='The class of true jellyfish, in the phylum CNIDARIA. They are mostly free-swimming marine organisms that go through five stages in their life cycle and exhibit two body forms: polyp and medusa.'),
  'D037402': NtO(MH='Hydrozoa', MS='A class in the phylum CNIDARIA which alternates between polyp and medusa forms during their life cycle. There are over 2700 species in five orders.'),
  'D037421': NtO(MH='Anthozoa', MS='A class in the phylum CNIDARIA, comprised mostly of corals and anemones. All members occur only as polyps; the medusa stage is completely absent.'),
  'D037441': NtO(MH='Sea Nettle, East Coast', MS='Species of jellyfish, in the family Pelagiidae, order Semaeostomeae, class SCYPHOZOA. Their painful sting is caused by a venom toxic to humans.'),
  'D037462': NtO(MH='Cubozoa', MS='The class of box jellyfish, in the phylum CNIDARIA, characterized by their cube shape, and considered the most venomous jellyfish.'),
  'D037483': NtO(MH='Galectin 1', MS='A galectin found abundantly in smooth muscle (MUSCLE, SMOOTH) and SKELETAL MUSCLE and many other tissues. It occurs as a homodimer with two 14-kDa subunits.'),
  'D037501': NtO(MH='Galectin 2', MS='A galectin found preferentially expressed in the gastrointestinal tract. The protein occurs as a homodimer with two 14-kDa subunits.'),
  'D037502': NtO(MH='Galectin 3', MS='A multifunctional galactin initially discovered as a macrophage antigen that binds to IMMUNOGLOBULIN E, and as 29-35-kDa lectin that binds LAMININ. It is involved in a variety of biological events including interactions with galactose-containing glycoconjugates, cell proliferation, CELL DIFFERENTIATION, and APOPTOSIS.'),
  'D037521': NtO(MH='Virulence Factors', MS='Those components of an organism that determine its capacity to cause disease but are not required for its viability per se. Two classes have been characterized: TOXINS, BIOLOGICAL and surface adhesion molecules that effect the ability of the microorganism to invade and colonize a host. (From Davis et al., Microbiology, 4th ed. p486)'),
  'D037541': NtO(MH='Galectin 4', MS='A galectin found in the small and large intestine and the stomach. It occurs as a homodimer with two 36-kDa subunits and is localized to sites of cell adhesion where it may play role in assembly of ADHERENS JUNCTIONS.'),
  'D037561': NtO(MH='Immunoconglutinins', MS='Autoantibodies directed against newly-formed EPITOPES created as the COMPLEMENT cascade is activated and the proteins involved change their conformations.'),
  'D037582': NtO(MH='Complement Pathway, Mannose-Binding Lectin', MS='Complement activation triggered by the interaction of microbial POLYSACCHARIDES with serum MANNOSE-BINDING LECTIN resulting in the activation of MANNOSE-BINDING PROTEIN-ASSOCIATED SERINE PROTEASES. As in the classical pathway, MASPs cleave COMPLEMENT C4 and COMPLEMENT C2 to form C3 CONVERTASE (C4B2A) and the subsequent C5 CONVERTASE (C4B2A3B) leading to cleavage of COMPLEMENT C5 and assembly of COMPLEMENT MEMBRANE ATTACK COMPLEX.'),
  'D037601': NtO(MH='Mannose-Binding Lectin', MS='A specific mannose-binding member of the collectin family of lectins. It binds to carbohydrate groups on invading pathogens and plays a key role in the MANNOSE-BINDING LECTIN COMPLEMENT PATHWAY.'),
  'D037602': NtO(MH='Isodon', MS='A plant genus of the family LAMIACEAE used in TRADITIONAL CHINESE MEDICINE.'),
  'D037621': NtO(MH='Artemisinins', MS='A group of SESQUITERPENES and their analogs that contain a peroxide group (PEROXIDES) within an oxepin ring (OXEPINS).'),
  'D037661': NtO(MH='Pulmonary Surfactant-Associated Proteins', MS='Proteins found in the LUNG that act as PULMONARY SURFACTANTS.'),
  'D037662': NtO(MH='Pulmonary Surfactant-Associated Protein A', MS='An abundant pulmonary surfactant-associated protein that binds to a variety of lung pathogens, resulting in their opsinization. It also stimulates MACROPHAGES to undergo PHAGOCYTOSIS of microorganisms. Surfactant protein A contains a N-terminal collagen-like domain and a C-terminal lectin domain that are characteristic of members of the collectin family of proteins.'),
  'D037663': NtO(MH='Pulmonary Surfactant-Associated Protein D', MS='An abundant pulmonary surfactant-associated protein that binds to a variety of lung pathogens and enhances their opsinization and killing by phagocytic cells. Surfactant protein D contains a N-terminal collagen-like domain and a C-terminal lectin domain that are characteristic of members of the collectin family of proteins.'),
  'D037701': NtO(MH='Pulmonary Surfactant-Associated Protein B', MS='A pulmonary surfactant associated-protein that plays an essential role in alveolar stability by lowering the surface tension at the air-liquid interface. Inherited deficiency of pulmonary surfactant-associated protein B is one cause of RESPIRATORY DISTRESS SYNDROME, NEWBORN.'),
  'D037721': NtO(MH='Pulmonary Surfactant-Associated Protein C', MS='A pulmonary surfactant associated protein that plays a role in alveolar stability by lowering the surface tension at the air-liquid interface. It is a membrane-bound protein that constitutes 1-2% of the pulmonary surfactant mass. Pulmonary surfactant-associated protein C is one of the most hydrophobic peptides yet isolated and contains an alpha-helical domain with a central poly-valine segment that binds to phospholipid bilayers.'),
  'D037741': NtO(MH='Fullerenes', MS='A polyhedral CARBON structure composed of around 60-80 carbon atoms in pentagon and hexagon configuration. They are named after Buckminster Fuller because of structural resemblance to geodesic domes. Fullerenes can be made in high temperature such as arc discharge in an inert atmosphere.'),
  'D037742': NtO(MH='Nanotubes, Carbon', MS='Nanometer-sized tubes composed mainly of CARBON. Such nanotubes are used as probes for high-resolution structural and chemical imaging of biomolecules with ATOMIC FORCE MICROSCOPY.'),
  'D037761': NtO(MH='Sirtuins', MS='A homologous family of regulatory enzymes that are structurally related to the protein silent mating type information regulator 2 (Sir2) found in Saccharomyces cerevisiae. Sirtuins contain a central catalytic core region which binds NAD. Several of the sirtuins utilize NAD to deacetylate proteins such as HISTONES and are categorized as GROUP III HISTONE DEACETYLASES. Several other sirtuin members utilize NAD to transfer ADP-RIBOSE to proteins and are categorized as MONO ADP-RIBOSE TRANSFERASES, while a third group of sirtuins appears to have both deacetylase and ADP ribose transferase activities.'),
  'D037801': NtO(MH='Hammer Toe Syndrome', MS='A condition characterized by a series of interrelated digital symptoms and joint changes of the lesser digits and METATARSOPHALANGEAL JOINTS of the FOOT. The syndrome can include some or all of the following conditions: hammer toe, claw toe, mallet toe, overlapping fifth toe, curly toe, EXOSTOSIS; HYPEROSTOSIS; interdigital heloma, or contracted toe.'),
  'D037821': NtO(MH='Casearia', MS='A plant genus of the family SALICACEAE. Members contain casearins which are clerodane-type DITERPENES.'),
  'D037841': NtO(MH='Pregnant Women', MS='Human females who are pregnant, as cultural, psychological, or sociological entities.'),
  'D037881': NtO(MH='Aborted Fetus', MS='A mammalian fetus expelled by INDUCED ABORTION or SPONTANEOUS ABORTION.'),
  'D037901': NtO(MH='Euthanasia, Animal', MS='The killing of animals for reasons of mercy, to control disease transmission or maintain the health of animal populations, or for experimental purposes (ANIMAL EXPERIMENTATION).'),
  'D037921': NtO(MH='Gift Giving', MS='The bestowing of tangible or intangible benefits, voluntarily and usually without expectation of anything in return. However, gift giving may be motivated by feelings of ALTRUISM or gratitude, by a sense of obligation, or by the hope of receiving something in return.'),
  'D037941': NtO(MH='Silencer Elements, Transcriptional', MS='Nucleic acid sequences that are involved in the negative regulation of GENETIC TRANSCRIPTION by chromatin silencing.'),
  'D037961': NtO(MH='Frullania', MS='A plant genus of the family Jubulaceae, subclass JUNGERMANNIAE. Members contain eudesmane-type sesquiterpene lactones and cause CONTACT DERMATITIS.'),
  'D037981': NtO(MH='Neck Dissection', MS='Dissection in the neck to remove all disease tissues including cervical LYMPH NODES and to leave an adequate margin of normal tissue. This type of surgery is usually used in tumors or cervical metastases in the head and neck. The prototype of neck dissection is the radical neck dissection described by Crile in 1906.'),
  'D038041': NtO(MH='Lymphopoiesis', MS='Formation of LYMPHOCYTES and PLASMA CELLS from the lymphoid stem cells which develop from the pluripotent HEMATOPOIETIC STEM CELLS in the BONE MARROW. These lymphoid stem cells differentiate into T-LYMPHOCYTES; B-LYMPHOCYTES; PLASMA CELLS; or NK-cells (KILLER CELLS, NATURAL) depending on the organ or tissues (LYMPHOID TISSUE) to which they migrate.'),
  'D038042': NtO(MH='Myelopoiesis', MS='Formation of MYELOID CELLS from the pluripotent HEMATOPOIETIC STEM CELLS in the BONE MARROW via MYELOID STEM CELLS. Myelopoiesis generally refers to the production of leukocytes in blood, such as MONOCYTES and GRANULOCYTES. This process also produces precursor cells for MACROPHAGE and DENDRITIC CELLS found in the lymphoid tissue.'),
  'D038061': NtO(MH='Lower Extremity Deformities, Congenital', MS='Congenital structural abnormalities of the LOWER EXTREMITY.'),
  'D038062': NtO(MH='Upper Extremity Deformities, Congenital', MS='Congenital structural abnormalities of the UPPER EXTREMITY.'),
  'D038081': NtO(MH='Organogenesis', MS='Formation of differentiated cells and complicated tissue organization to provide specialized functions.'),
  'D038101': NtO(MH='Insulator Elements', MS='Nucleic acid regulatory sequences that limit or oppose the action of ENHANCER ELEMENTS and define the boundary between differentially regulated gene loci.'),
  'D038143': NtO(MH='Inula', MS='A plant genus of the family ASTERACEAE. Members contain INULIN, alantol, helenin, alantic acid, and acrid resin.'),
  'D038161': NtO(MH='Snow Sports', MS='Sports activities in the snow.'),
  'D038181': NtO(MH='FMN Reductase', MS='An enzyme that utilizes NADH or NADPH to reduce FLAVINS. It is involved in a number of biological processes that require reduced flavin for their functions such as bacterial bioluminescence. Formerly listed as EC 1.6.8.1 and EC 1.5.1.29.'),
  'D038201': NtO(MH='alpha-Crystallins', MS='A subclass of crystallins that provides the majority of refractive power and translucency to the lens (LENS, CRYSTALLINE) in VERTEBRATES. Alpha-crystallins also act as molecular chaperones that bind to denatured proteins, keep them in solution and thereby maintain the translucency of the lens. The proteins exist as large oligomers that are formed from ALPHA-CRYSTALLIN A CHAIN and ALPHA-CRYSTALLIN B CHAIN subunits.'),
  'D038202': NtO(MH='alpha-Crystallin A Chain', MS='One of the subunits of alpha-crystallins. Unlike ALPHA-CRYSTALLIN B CHAIN the expression of ALPHA-CRYSTALLIN A CHAIN is limited primarily to the lens (LENS, CRYSTALLINE).'),
  'D038203': NtO(MH='alpha-Crystallin B Chain', MS='One of the alpha crystallin subunits. In addition to being expressed in the lens (LENS, CRYSTALLINE), alpha-crystallin B chain has been found in a variety of tissues such as HEART; BRAIN; MUSCLE; and KIDNEY. Accumulation of the protein in the brain is associated with NEURODEGENERATIVE DISEASES such as CREUTZFELDT-JAKOB SYNDROME and ALEXANDER DISEASE.'),
  'D038204': NtO(MH='beta-Crystallins', MS='A class of crystallins that provides refractive power and translucency to the lens (LENS, CRYSTALLINE) in VERTEBRATES. Beta-crystallins are similar in structure to GAMMA-CRYSTALLINS in that they both contain Greek key motifs. Beta-crystallins exist as oligomers formed from acidic (BETA-CRYSTALLIN A CHAIN) and basic (BETA-CRYSTALLIN B CHAIN) subunits.'),
  'D038205': NtO(MH='beta-Crystallin A Chain', MS='The acidic subunit of beta-crystallins.'),
  'D038206': NtO(MH='beta-Crystallin B Chain', MS='The basic subunit of beta-crystallins.'),
  'D038207': NtO(MH='delta-Crystallins', MS='A subclass of crystallins found in the lens (LENS, CRYSTALLINE) in BIRDS and REPTILES. They are inactive forms of the enzyme argininosuccinate lyase.'),
  'D038221': NtO(MH='epsilon-Crystallins', MS='A group of crystallins that have been found in the lens (LENS, CRYSTALLINE) of DUCKS and CROCODILES. They are inactivated forms of the B chain subunit of lactate dehydrogenase.'),
  'D038222': NtO(MH='gamma-Crystallins', MS='A subclass of crystallins that found in the lens (LENS, CRYSTALLINE) of VERTEBRATES. Gamma-crystallins are similar in structure to BETA-CRYSTALLINS in that they both form into a Greek key-like structure. They are composed of monomeric subunits.'),
  'D038223': NtO(MH='Post-Concussion Syndrome', MS='The organic and psychogenic disturbances observed after closed head injuries (HEAD INJURIES, CLOSED). Post-concussion syndrome includes subjective physical complaints (i.e. headache, dizziness), cognitive, emotional, and behavioral changes. These disturbances can be chronic, permanent, or late emerging.'),
  'D038224': NtO(MH='omega-Crystallins', MS='A type of crystallin that has been found in the lens (LENS, CRYSTALLINE) of certain species of MOLLUSCA such as SQUID and OCTOPUS. It is an inactivated form of aldehyde dehydrogenase.'),
  'D038225': NtO(MH='tau-Crystallins', MS='A type of crystallin that has been found in the lens (LENS, CRYSTALLINE) of certain species of VERTEBRATES. They are inactivated form of PHOSPHOPYRUVATE HYDRATASE.'),
  'D038226': NtO(MH='zeta-Crystallins', MS='A group of crystallins that have been found in the lens (LENS, CRYSTALLINE) of certain species of VERTEBRATES including GUINEA PIGS; CAMELS; and LLAMAS. They are inactivated forms of NAD(P)H DEHYDROGENASE (QUINONE).'),
  'D038261': NtO(MH='Alexander Disease', MS='Rare leukoencephalopathy with infantile-onset accumulation of Rosenthal fibers in the subpial, periventricular, and subependymal zones of the brain. Rosenthal fibers are GLIAL FIBRILLARY ACIDIC PROTEIN aggregates found in ASTROCYTES. Juvenile- and adult-onset types show progressive atrophy of the lower brainstem instead. De novo mutations in the GFAP gene are associated with the disease with propensity for paternal inheritance.'),
  'D038262': NtO(MH='Auditory Brain Stem Implants', MS='Multi-channel hearing devices typically used for patients who have tumors on the COCHLEAR NERVE and are unable to benefit from COCHLEAR IMPLANTS after tumor surgery that severs the cochlear nerve. The device electrically stimulates the nerves of cochlea nucleus in the BRAIN STEM rather than the inner ear as in cochlear implants.'),
  'D038263': NtO(MH='Auditory Brain Stem Implantation', MS='Surgical insertion of an electronic hearing device (AUDITORY BRAIN STEM IMPLANTS) with electrodes to the cochlea nucleus in the BRAIN STEM rather than to the inner ear as in COCHLEAR IMPLANTATION.'),
  'D038281': NtO(MH='Silent Information Regulator Proteins, Saccharomyces cerevisiae', MS='A set of nuclear proteins in SACCHAROMYCES CEREVISIAE that are required for the transcriptional repression of the silent mating type loci. They mediate the formation of silenced CHROMATIN and repress both transcription and recombination at other loci as well. They are comprised of 4 non-homologous, interacting proteins, Sir1p, Sir2p, Sir3p, and Sir4p. Sir2p, an NAD-dependent HISTONE DEACETYLASE, is the founding member of the family of SIRTUINS.'),
  'D038301': NtO(MH='Virus Inactivation', MS='Inactivation of viruses by non-immune related techniques. They include extremes of pH, HEAT treatment, ultraviolet radiation, IONIZING RADIATION; DESICCATION; ANTISEPTICS; DISINFECTANTS; organic solvents, and DETERGENTS.'),
  'D038321': NtO(MH='Glycemic Index', MS='A numerical system of measuring the rate of BLOOD GLUCOSE generation from a particular food item as compared to a reference item, such as glucose = 100. Foods with higher glycemic index numbers create greater blood sugar swings.'),
  'D038341': NtO(MH='Glycogen Synthase Kinases', MS='A class of protein-serine-threonine kinases that was originally found as one of the three types of kinases that phosphorylate GLYCOGEN SYNTHASE. Glycogen synthase kinases along with CA(2+)-CALMODULIN DEPENDENT PROTEIN KINASES and CYCLIC AMP-DEPENDENT PROTEIN KINASES regulate glycogen synthase activity.'),
  'D038361': NtO(MH='Neuronavigation', MS="Intraoperative computer-assisted 3D navigation and guidance system generally used in neurosurgery for tracking surgical tools and localize them with respect to the patient's 3D anatomy. The pre-operative diagnostic scan is used as a reference and is transferred onto the operative field during surgery."),
  'D038362': NtO(MH='Glycogen Synthase Kinase 3', MS='A glycogen synthase kinase that was originally described as a key enzyme involved in glycogen metabolism. It regulates a diverse array of functions such as CELL DIVISION, microtubule function and APOPTOSIS.'),
  'D038381': NtO(MH='Neisseria meningitidis, Serogroup A', MS='Strains of Neisseria meningitidis responsible for most outbreaks of meningococcal disease in Western Europe and the United States in the first half of the 20th century. They continue to be a major cause of disease in Asia and Africa, and especially localized epidemics in Sub-Sahara Africa.'),
  'D038421': NtO(MH='Hemispherectomy', MS='A neurosurgical procedure that removes or disconnects the epileptogenic CEREBRAL CORTEX of a hemisphere. Hemispherectomy is usually performed for patients with intractable unilateral EPILEPSY due to malformations of cortical development or brain lesions. Depending on the epileptogenic area in the hemisphere, cortical removal can be total or partial.'),
  'D038441': NtO(MH='Diet, Mediterranean', MS='A diet typical of the Mediterranean region characterized by a pattern high in fruits and vegetables, EDIBLE GRAIN and bread, potatoes, poultry, beans, nuts, olive oil and fish while low in red meat and dairy and moderate in alcohol consumption.'),
  'D038461': NtO(MH='Proline-Directed Protein Kinases', MS='A subclass of protein serine-threonine kinases that phosphorylate proteins on a SERINE or THREONINE residue that is immediately preceding a PROLINE residue.'),
  'D038481': NtO(MH='Anterior Temporal Lobectomy', MS='A neurosurgical procedure that removes the anterior TEMPORAL LOBE including the medial temporal structures of CEREBRAL CORTEX; AMYGDALA; HIPPOCAMPUS; and the adjacent PARAHIPPOCAMPAL GYRUS. This procedure is generally used for the treatment of intractable temporal epilepsy (EPILEPSY, TEMPORAL LOBE).'),
  'D038501': NtO(MH='Tankyrases', MS='A group of telomere associated proteins that interact with TRF1 PROTEIN, contain ANKYRIN REPEATS and have poly(ADP-ribose) polymerase activity.'),
  'D038522': NtO(MH='Twinning, Monozygotic', MS='The division of a ZYGOTE into two parts that are each capable of further development into two distinct individuals, both that have the same genetic makeup.'),
  'D038524': NtO(MH='Diffusion Magnetic Resonance Imaging', MS='A diagnostic technique that incorporates the measurement of molecular diffusion (such as water or metabolites) for tissue assessment by MRI. The degree of molecular movement can be measured by changes of apparent diffusion coefficient (ADC) with time, as reflected by tissue microstructure. Diffusion MRI has been used to study BRAIN ISCHEMIA and tumor response to treatment.'),
  'D038541': NtO(MH='Neisseria meningitidis, Serogroup B', MS='Strains of Neisseria meningitidis which are the most common ones causing infections or disease in infants. Serogroup B strains are isolated most frequently in sporadic cases, and are less common in outbreaks and epidemics.'),
  'D038542': NtO(MH='Neisseria meningitidis, Serogroup C', MS='Strains of Neisseria meningitidis responsible for most sporadic cases in teenagers and almost all outbreaks of disease in this age group. These strains are less common in infants.'),
  'D038581': NtO(MH='Neisseria meningitidis, Serogroup Y', MS='Strains of Neisseria meningitidis which, in the United States, causes disease in mostly adults and the elderly. Serogroup Y strains are associated with PNEUMONIA.'),
  'D038582': NtO(MH='Neisseria meningitidis, Serogroup W-135', MS='Strains of Neisseria meningitidis found mostly in Africa.'),
  'D038601': NtO(MH='Ribosomal Protein S6', MS="A ribosomal protein that may play a role in controlling cell growth and proliferation. It is a major substrate of RIBOSOMAL PROTEIN S6 KINASES and plays a role in regulating the translation (TRANSLATION, GENETIC) of RNAs that contain an RNA 5' TERMINAL OLIGOPYRIMIDINE SEQUENCE."),
  'D038602': NtO(MH='Mice, Biozzi', MS='A strain of mice bred specifically as high or low antibody responders.'),
  'D038621': NtO(MH='Regulatory Sequences, Ribonucleic Acid', MS='Sequences within RNA that regulate the processing, stability (RNA STABILITY) or translation (TRANSLATION, GENETIC) of RNA.'),
  'D038622': NtO(MH='Internationality', MS='The quality or state of relating to or affecting two or more nations. (After Merriam-Webster Collegiate Dictionary, 10th ed)'),
  'D038641': NtO(MH="RNA 5' Terminal Oligopyrimidine Sequence", MS="A regulatory sequence found in the 5' terminal regions of a variety of RNA species. The sequence starts with a CYTIDINE, which is followed by a stretch of 5 to 15 PYRIMIDINE NUCLEOTIDES. Messenger RNA that contains the 5' Terminal Oligo Pyrimidine tract is often referred to as 5' TOP mRNA. The sequence acts as a translational regulator and has been found in mRNAs for PEPTIDE ELONGATION FACTORS and RIBOSOMAL PROTEINS."),
  'D038642': NtO(MH='Shaken Baby Syndrome', MS='Brain injuries resulted from vigorous shaking of an infant or young child held by the chest, shoulders, or extremities causing extreme cranial acceleration. It is characterized by the intracranial and intraocular hemorrhages with no evident external trauma. Serious cases may result in death.'),
  'D038681': NtO(MH='Follistatin', MS='A broadly distributed protein that binds directly to ACTIVINS. It functions as an activin antagonist, inhibits FOLLICLE STIMULATING HORMONE secretion, regulates CELL DIFFERENTIATION, and plays an important role in embryogenesis. Follistatin is a single glycosylated polypeptide chain of approximately 37-kDa and is not a member of the inhibin family (INHIBINS). Follistatin also binds and neutralizes many members of the TRANSFORMING GROWTH FACTOR BETA family.'),
  'D038702': NtO(MH='Follistatin-Related Proteins', MS='Broadly distributed glycoproteins that are homologous to the activin-binding protein, FOLLISTATIN. These follistatin-related proteins are encoded by a number of genes.'),
  'D038744': NtO(MH='Ribosomal Protein S6 Kinases, 90-kDa', MS='A family of ribosomal protein S6 kinases that are structurally distinguished from RIBOSOMAL PROTEIN S6 KINASES, 70-KDA by their apparent molecular size and the fact they contain two functional kinase domains. Although considered RIBOSOMAL PROTEIN S6 KINASES, members of this family are activated via the MAP KINASE SIGNALING SYSTEM and have been shown to act on a diverse array of substrates that are involved in cellular regulation such as RIBOSOMAL PROTEIN S6 and CAMP RESPONSE ELEMENT-BINDING PROTEIN.'),
  'D038761': NtO(MH='Photobleaching', MS='Light-induced change in a chromophore, resulting in the loss of its absorption of light of a particular wave length. The photon energy causes a conformational change in the photoreceptor proteins affecting PHOTOTRANSDUCTION. This occurs naturally in the retina (ADAPTATION, OCULAR) on long exposure to bright light. Photobleaching presents problems when occurring in PHOTODYNAMIC THERAPY, and in FLUORESCENCE MICROSCOPY. On the other hand, this phenomenon is exploited in the technique, FLUORESCENCE RECOVERY AFTER PHOTOBLEACHING, allowing measurement of the movements of proteins and LIPIDS in the CELL MEMBRANE.'),
  'D038762': NtO(MH='Ribosomal Protein S6 Kinases, 70-kDa', MS='A family of ribosomal protein S6 kinases that are considered the major physiological kinases for RIBOSOMAL PROTEIN S6. Unlike RIBOSOMAL PROTEIN S6 KINASES, 90KDa the proteins in this family are sensitive to the inhibitory effects of RAPAMYCIN and contain a single kinase domain. They are referred to as 70kDa proteins, however ALTERNATIVE SPLICING of mRNAs for proteins in this class also results in 85kDa variants being formed.'),
  'D038781': NtO(MH='Otoscopy', MS='Examination of the EAR CANAL and eardrum with an OTOSCOPE.'),
  'D038801': NtO(MH='International Classification of Diseases', MS='A system of categories to which morbid entries are assigned according to established criteria. Included is the entire range of conditions in a manageable number of categories, grouped to facilitate mortality reporting. It is produced by the World Health Organization (From ICD-10, p1). The Clinical Modifications, produced by the UNITED STATES DEPT. OF HEALTH AND HUMAN SERVICES, are larger extensions used for morbidity and general epidemiological purposes, primarily in the U.S.'),
  'D038821': NtO(MH='Mycorrhizae', MS='Symbiotic combination (dual organism) of the MYCELIUM of FUNGI with the roots of plants (PLANT ROOTS). The roots of almost all higher plants exhibit this mutually beneficial relationship, whereby the fungus supplies water and mineral salts to the plant, and the plant supplies CARBOHYDRATES to the fungus. There are two major types of mycorrhizae: ectomycorrhizae and endomycorrhizae.'),
  'D038861': NtO(MH='Medicine, Tibetan Traditional', MS='A system of traditional medicine which is based on the beliefs and practices of the Tibetan culture.'),
  'D038901': NtO(MH='Mental Retardation, X-Linked', MS='A class of genetic disorders resulting in INTELLECTUAL DISABILITY that is associated either with mutations of GENES located on the X CHROMOSOME or aberrations in the structure of the X chromosome (SEX CHROMOSOME ABERRATIONS).'),
  'D038921': NtO(MH='Coffin-Lowry Syndrome', MS='A rare, X-linked INTELLECTUAL DISABILITY syndrome that results from mutations in the RIBOSOMAL PROTEIN S6 KINASE gene. Typical manifestations of the disease include an intelligence quotient of less than 50, facial anomalies, and other malformations.'),
  'D038941': NtO(MH='Polypyrimidine Tract-Binding Protein', MS='A RNA-binding protein that binds to polypyriminidine rich regions in the INTRONS of messenger RNAs. Polypyrimidine tract-binding protein may be involved in regulating the ALTERNATIVE SPLICING of mRNAs since its presence on an intronic RNA region that is upstream of an EXON inhibits the splicing of the exon into the final mRNA product.'),
  'D038961': NtO(MH='O-Acetyl-ADP-Ribose', MS='An acetyl ester of ADENOSINE DIPHOSPHATE RIBOSE formed during NAD-dependent deacetylation of proteins by SIRTUINS. The acetate group resides on the ribose ring where nicotinamide was cleaved from NAD during the reaction. Several isomers of O-acetyl-ADP-ribose have been isolated from the reaction.'),
  'D038981': NtO(MH='Receptors, Collagen', MS='Cell surface receptors that modulate signal transduction between cells and the EXTRACELLULAR MATRIX. They are found in many cell types and are involved in the maintenance and regulation of cell shape and behavior, including PLATELET ACTIVATION and aggregation, through many different signaling pathways and differences in their affinities for collagen isoforms. Collagen receptors include DISCOIDIN DOMAIN RECEPTORS; INTEGRINS; and GP6, one of the PLATELET MEMBRANE GLYCOPROTEINS.'),
  'D038982': NtO(MH='Integrin alpha2beta1', MS='An integrin found on fibroblasts, platelets, endothelial and epithelial cells, and lymphocytes where it functions as a receptor for COLLAGEN and LAMININ. Although originally referred to as the collagen receptor, it is one of several receptors for collagen. Ligand binding to integrin alpha2beta1 triggers a cascade of intracellular signaling, including activation of p38 MAP kinase.'),
  'D038983': NtO(MH='Galactolipids', MS='A group of GLYCOLIPIDS in which the sugar group is GALACTOSE. They are distinguished from GLYCOSPHINGOLIPIDS in lacking nitrogen. They constitute the majority of MEMBRANE LIPIDS in PLANTS.'),
  'D039001': NtO(MH='Integrin alpha Chains', MS='The alpha subunits of integrin heterodimers (INTEGRINS), which mediate ligand specificity. There are approximately 18 different alpha chains, exhibiting great sequence diversity; several chains are also spliced into alternative isoforms. They possess a long extracellular portion (1200 amino acids) containing a MIDAS (metal ion-dependent adhesion site) motif, and seven 60-amino acid tandem repeats, the last 4 of which form EF HAND MOTIFS. The intracellular portion is short with the exception of INTEGRIN ALPHA4.'),
  'D039002': NtO(MH='Prophages', MS='Genomes of temperate BACTERIOPHAGES integrated into the DNA of their bacterial host cell. The prophages can be duplicated for many cell generations until some stimulus induces its activation and virulence.'),
  'D039021': NtO(MH='Colonic Pouches', MS='Sacs or reservoirs created to function in place of the COLON and/or RECTUM in patients who have undergone restorative proctocolectomy (PROCTOCOLECTOMY, RESTORATIVE).'),
  'D039041': NtO(MH='Integrin alpha4beta1', MS='Integrin alpha4beta1 is a FIBRONECTIN and VCAM-1 receptor present on LYMPHOCYTES; MONOCYTES; EOSINOPHILS; NK CELLS and thymocytes. It is involved in both cell-cell and cell- EXTRACELLULAR MATRIX adhesion and plays a role in INFLAMMATION, hematopoietic cell homing and immune function, and has been implicated in skeletal MYOGENESIS; NEURAL CREST migration and proliferation, lymphocyte maturation and morphogenesis of the PLACENTA and HEART.'),
  'D039061': NtO(MH='Systematized Nomenclature of Medicine', MS='Controlled vocabulary of clinical terms produced by the International Health Terminology Standards Development Organisation (IHTSDO).'),
  'D039081': NtO(MH='Integrin alpha5beta1', MS='An integrin found in FIBROBLASTS; PLATELETS; MONOCYTES, and LYMPHOCYTES. Integrin alpha5beta1 is the classical receptor for FIBRONECTIN, but it also functions as a receptor for LAMININ and several other EXTRACELLULAR MATRIX PROTEINS.'),
  'D039102': NtO(MH='Poly(A)-Binding Proteins', MS="Proteins that bind to the 3' polyadenylated region of MRNA. When complexed with RNA the proteins serve an array of functions such as stabilizing the 3' end of RNA, promoting poly(A) synthesis and stimulating mRNA translation."),
  'D039103': NtO(MH='Poly(A)-Binding Protein II', MS='A poly(A) binding protein that is involved in promoting the extension of the poly A tails of MRNA. The protein requires a minimum of ten ADENOSINE nucleotides in order for binding to mRNA. Once bound it works in conjunction with CLEAVAGE AND POLYADENYLATION SPECIFICITY FACTOR to stimulate the rate of poly A synthesis by POLY A POLYMERASE. Once poly-A tails reach around 250 nucleotides in length poly(A) binding protein II no longer stimulates POLYADENYLATION. Mutations within a GCG repeat region in the gene for poly(A) binding protein II have been shown to cause the disease MUSCULAR DYSTROPHY, OCULOPHARYNGEAL.'),
  'D039104': NtO(MH="RNA 3' Polyadenylation Signals", MS="Sequences found near the 3' end of MESSENGER RNA that direct the cleavage and addition of multiple ADENINE NUCLEOTIDES to the 3' end of mRNA."),
  'D039105': NtO(MH='Proteaceae', MS='A plant family of the order Proteales, subclass Rosidae class Magnoliopsida. Cluster roots, bottlebrush-like clusters of rootlets which form in response to poor soil, are common in this family.'),
  'D039106': NtO(MH='Macadamia', MS='A plant genus of the family PROTEACEAE that is the source of edible NUTS.'),
  'D039121': NtO(MH='Integrin alpha6beta1', MS='A cell surface receptor mediating cell adhesion to the EXTRACELLULAR MATRIX and to other cells via binding to LAMININ. It is involved in cell migration, embryonic development, leukocyte activation and tumor cell invasiveness. Integrin alpha6beta1 is the major laminin receptor on PLATELETS; LEUKOCYTES; and many EPITHELIAL CELLS, and ligand binding may activate a number of signal transduction pathways. Alternative splicing of the cytoplasmic domain of the alpha6 subunit (INTEGRIN ALPHA6) results in the formation of A and B isoforms of the heterodimer, which are expressed in a tissue-specific manner.'),
  'D039122': NtO(MH='Poly(A)-Binding Protein I', MS='A poly(A) binding protein that has a variety of functions such as mRNA stabilization and protection of RNA from nuclease activity. Although poly(A) binding protein I is considered a major cytoplasmic RNA-binding protein it is also found in the CELL NUCLEUS and may be involved in transport of mRNP particles.'),
  'D039141': NtO(MH='Muscular Dystrophy, Oculopharyngeal', MS='An autosomal dominant hereditary disease that presents in late in life and is characterized by DYSPHAGIA and progressive ptosis of the eyelids. Mutations in the gene for POLY(A)-BINDING PROTEIN II have been associated with oculopharyngeal muscular dystrophy.'),
  'D039161': NtO(MH='Integrin alpha6beta4', MS='This intrgrin is a key component of HEMIDESMOSOMES and is required for their formation and maintenance in epithelial cells. Integrin alpha6beta4 is also found on thymocytes, fibroblasts, and Schwann cells, where it functions as a laminin receptor (RECEPTORS, LAMININ) and is involved in wound healing, cell migration, and tumor invasiveness.'),
  'D039181': NtO(MH='Steroid 16-alpha-Hydroxylase', MS='A liver microsomal cytochrome P450 enzyme that catalyzes the 16-alpha-hydroxylation of a broad spectrum of steroids, fatty acids, and xenobiotics in the presence of molecular oxygen and NADPH-FERRIHEMOPROTEIN REDUCTASE. This enzyme is encoded by a number of genes from several CYP2 subfamilies.'),
  'D039201': NtO(MH='Integrin alpha3beta1', MS='Cell surface receptor for LAMININ, epiligrin, FIBRONECTINS, entactin, and COLLAGEN. Integrin alpha3beta1 is the major integrin present in EPITHELIAL CELLS, where it plays a role in the assembly of BASEMENT MEMBRANE as well as in cell migration, and may regulate the functions of other integrins. Two alternatively spliced isoforms of the alpha subunit (INTEGRIN ALPHA3), are differentially expressed in different cell types.'),
  'D039221': NtO(MH='mRNA Cleavage and Polyadenylation Factors', MS="Factors that are involved in directing the cleavage and POLYADENYLATION of the of MESSENGER RNA near the site of the RNA 3' POLYADENYLATION SIGNALS."),
  'D039222': NtO(MH='Integrin alpha1beta1', MS='Integrin alpha1beta1 functions as a receptor for LAMININ and COLLAGEN. It is widely expressed during development, but in the adult is the predominant laminin receptor (RECEPTORS, LAMININ) in mature SMOOTH MUSCLE CELLS, where it is important for maintenance of the differentiated phenotype of these cells. Integrin alpha1beta1 is also found in LYMPHOCYTES and microvascular endothelial cells, and may play a role in angiogenesis. In SCHWANN CELLS and neural crest cells, it is involved in cell migration. Integrin alpha1beta1 is also known as VLA-1 and CD49a-CD29.'),
  'D039223': NtO(MH='Cleavage And Polyadenylation Specificity Factor', MS="An RNA-binding protein that recognizes the AAUAAA RNA SEQUENCE at the 3' end of MRNA. It contains four subunits of 30, 73, 100 and 160 kDa molecular size and combines with CLEAVAGE STIMULATION FACTOR to form a stable complex with mRNA that directs the 3' cleavage and polyadenylation reaction."),
  'D039241': NtO(MH='Cleavage Stimulation Factor', MS="A RNA-binding protein that stimulates the cleavage of the 3' end of MRNA near the POLYADENYLATION site. It is a heterotrimer of 55-, 64- and 77-kDa subunits and combines with CLEAVAGE STIMULATION FACTOR to form a stable complex with mRNA that directs the 3' cleavage and polyadenylation reaction."),
  'D039262': NtO(MH='Eriocaulaceae', MS='A plant family of the order Commelinales, subclass Commelinidae, class Liliopsida (monocotyledons).'),
  'D039281': NtO(MH='Nyctaginaceae', MS='A plant family of the order Caryophyllales, subclass Caryophyllidae, class Magnoliopsida.'),
  'D039282': NtO(MH='Mirabilis', MS='A plant genus of the family NYCTAGINACEAE. Members contain Mirabilis antiviral protein (a ribosome-inactivating protein).'),
  'D039301': NtO(MH='Computers, Molecular', MS='Computers whose input, output and state transitions are carried out by biochemical interactions and reactions.'),
  'D039302': NtO(MH='Integrin alphaVbeta3', MS='An integrin that binds to a variety of plasma and extracellular matrix proteins containing the conserved RGD amino acid sequence and modulates cell adhesion. Integrin alphavbeta3 is highly expressed in OSTEOCLASTS where it may play role in BONE RESORPTION. It is also abundant in vascular smooth muscle and endothelial cells, and in some tumor cells, where it is involved in angiogenesis and cell migration. Although often referred to as the vitronectin receptor there is more than one receptor for vitronectin (RECEPTORS, VITRONECTIN).'),
  'D039321': NtO(MH='Hydrastis', MS='A plant genus of the family RANUNCULACEAE. Members contain BERBERINE.'),
  'D039322': NtO(MH='Helleborus', MS='A plant genus of the family RANUNCULACEAE. Members contain hellebrin (BUFANOLIDES). The extract is the basis of Boicil preparation used to treat rheumatism. The common name "black hellebore" is used for both VERATRUM NIGRUM and Helleborus niger.'),
  'D039323': NtO(MH='Coptis', MS='A plant genus of the family RANUNCULACEAE. Members contain BERBERINE and other isoquinoline ALKALOIDS.'),
  'D039324': NtO(MH='Clematis', MS='A plant genus of the family RANUNCULACEAE. Members contain SCOPOLETIN and SAPONINS.'),
  'D039341': NtO(MH='Receptors, Fibrinogen', MS='Receptors that bind FIBRINOGEN through distinct adhesive sequences on the fibrinogen molecule. Although MACROPHAGE-1 ANTIGEN is considered an important signaling molecule for fibrinogen interaction, a variety of INTEGRINS from all three major families, (beta1, beta2, and beta3) have been shown to bind fibrinogen.'),
  'D039381': NtO(MH='RNA Cap-Binding Proteins', MS="Proteins that specifically bind to RNA CAPS and form nuclear cap binding protein complexes. In addition to stabilizing the 5' end of mRNAs, they serve a diverse array of functions such as enhancing mRNA transport out of the CELL NUCLEUS and regulating MRNA TRANSLATION in the CYTOPLASM."),
  'D039382': NtO(MH='Maternal Nutritional Physiological Phenomena', MS='Nutrition of a mother which affects the health of the FETUS and INFANT as well as herself.'),
  'D039401': NtO(MH='Prenatal Nutritional Physiological Phenomena', MS='Nutrition of FEMALE during PREGNANCY.'),
  'D039421': NtO(MH='Integrin alpha2', MS='An integrin alpha subunit that primarily combines with INTEGRIN BETA1 to form the INTEGRIN ALPHA2BETA1 heterodimer. It contains a domain which has homology to collagen-binding domains found in von Willebrand factor.'),
  'D039422': NtO(MH='Integrin alpha3', MS='An integrin alpha subunit that occurs as alternatively spliced isoforms. The isoforms are differentially expressed in specific cell types and at specific developmental stages. Integrin alpha3 combines with INTEGRIN BETA1 to form INTEGRIN ALPHA3BETA1 which is a heterodimer found primarily in epithelial cells.'),
  'D039423': NtO(MH='Integrin alpha1', MS='An integrin alpha subunit that binds COLLAGEN and LAMININ though its I domain. It combines with INTEGRIN BETA1 to form the heterodimer INTEGRIN ALPHA1BETA1.'),
  'D039441': NtO(MH='Integrin alpha4', MS='An integrin alpha subunit that is unique in that it does not contain an I-domain, and its proteolytic cleavage site is near the middle of the extracellular portion of the polypeptide rather than close to the membrane, as in other integrin alpha subunits.'),
  'D039481': NtO(MH='CD11b Antigen', MS='A CD antigen that contains a conserved integrin I-domain which is involved in ligand binding. When combined with CD18 the two subunits form MACROPHAGE-1 ANTIGEN.'),
  'D039482': NtO(MH='Integrin alpha5', MS='This integrin alpha subunit combines with INTEGRIN BETA1 to form a receptor (INTEGRIN ALPHA5BETA1) that binds FIBRONECTIN and LAMININ. It undergoes posttranslational cleavage into a heavy and a light chain that are connected by disulfide bonds.'),
  'D039502': NtO(MH='Nuclear Cap-Binding Protein Complex', MS="A heterodimeric protein complex of RNA cap-binding proteins, which binds with high affinity to the 5' MRNA CAP STRUCTURE in the CELL NUCLEUS. The complex contains two subunits, one of 80-kDa molecular weight and another of 20-kDa molecular weight."),
  'D039503': NtO(MH='Integrin alpha6', MS='An integrin alpha subunit that primarily associates with INTEGRIN BETA1 or INTEGRIN BETA4 to form laminin-binding heterodimers. Integrin alpha6 has two alternatively spliced isoforms: integrin alpha6A and integrin alpha6B, which differ in their cytoplasmic domains and are regulated in a tissue-specific and developmental stage-specific manner.'),
  'D039521': NtO(MH='CD11c Antigen', MS='An integrin alpha subunit of approximately 150-kDa molecular weight. It is expressed at high levels on monocytes and combines with CD18 ANTIGEN to form the cell surface receptor INTEGRIN ALPHAXBETA2. The subunit contains a conserved I-domain which is characteristic of several of alpha integrins.'),
  'D039561': NtO(MH='Eukaryotic Initiation Factor-4E', MS="A peptide initiation factor that binds specifically to the 5' MRNA CAP STRUCTURE of MRNA in the CYTOPLASM. It is a component of the trimeric complex EIF4F."),
  'D039562': NtO(MH='Eukaryotic Initiation Factor-4F', MS="A trimeric peptide initiation factor complex that associates with the 5' MRNA cap structure of RNA (RNA CAPS) and plays an essential role in MRNA TRANSLATION. It is composed of EUKARYOTIC INITIATION FACTOR-4A; EUKARYOTIC INITIATION FACTOR-4E; and EUKARYOTIC INITIATION FACTOR-4G."),
  'D039563': NtO(MH='Histamine H1 Antagonists, Non-Sedating', MS='A class of non-sedating drugs that bind to but do not activate histamine receptors (DRUG INVERSE AGONISM), thereby blocking the actions of histamine or histamine agonists. These antihistamines represent a heterogenous group of compounds with differing chemical structures, adverse effects, distribution, and metabolism. Compared to the early (first generation) antihistamines, these non-sedating antihistamines have greater receptor specificity, lower penetration of BLOOD-BRAIN BARRIER, and are less likely to cause drowsiness or psychomotor impairment.'),
  'D039564': NtO(MH='Integrin alphaV', MS='An alpha integrin with a molecular weight of 160-kDa that is found in a variety of cell types. It undergoes posttranslational cleavage into a heavy and a light chain that are connected by disulfide bonds. Integrin alphaV can combine with several different beta subunits to form heterodimers that generally bind to RGD sequence-containing extracellular matrix proteins.'),
  'D039601': NtO(MH='Eukaryotic Initiation Factor-4A', MS="A component of eukaryotic initiation factor 4F that as an RNA helicase involved in unwinding the secondary structure of the 5' UNTRANSLATED REGION of MRNA. The unwinding facilitates the binding of the 40S ribosomal subunit."),
  'D039602': NtO(MH='After-Hours Care', MS='Medical care provided after the regular daily practice schedule of the physicians. Usually it is designed to deliver 24-hour-a-day and 365 day-a-year patient care coverage for emergencies, triage, pediatric care, and hospice care.'),
  'D039603': NtO(MH='Eukaryotic Initiation Factor-4G', MS='A component of eukaryotic initiation factor-4F that is involved in multiple protein interactions at the site of translation initiation. Thus it may serve a role in bringing together various initiation factors at the site of translation initiation.'),
  'D039604': NtO(MH='Answering Services', MS='Communication services provided by a person or a machine to record and relay the message from the caller.'),
  'D039621': NtO(MH='Eukaryotic Initiation Factor-3', MS='A multisubunit eukaryotic initiation factor that contains at least 8 distinct polypeptides. It plays a role in recycling of ribosomal subunits to the site of transcription initiation by promoting the dissociation of non-translating ribosomal subunits. It also is involved in promoting the binding of a ternary complex of EUKARYOTIC INITIATION FACTOR-2; GTP; and INITIATOR TRNA to the 40S ribosomal subunit.'),
  'D039641': NtO(MH='Integrin beta Chains', MS='Integrin beta chains combine with integrin alpha chains to form heterodimeric cell surface receptors. Integrins have traditionally been classified into functional groups based on the identity of one of three beta chains present in the heterodimer. The beta chain is necessary and sufficient for integrin-dependent signaling. Its short cytoplasmic tail contains sequences critical for inside-out signaling.'),
  'D039642': NtO(MH='Eukaryotic Initiation Factors', MS='Peptide initiation factors from eukaryotic organisms. Over twelve factors are involved in PEPTIDE CHAIN INITIATION, TRANSLATIONAL in eukaryotic cells. Many of these factors play a role in controlling the rate of MRNA TRANSLATION.'),
  'D039661': NtO(MH='Integrin beta3', MS='An integrin beta subunit of approximately 85-kDa in size which has been found in INTEGRIN ALPHAIIB-containing and INTEGRIN ALPHAV-containing heterodimers. Integrin beta3 occurs as three alternatively spliced isoforms, designated beta3A-C.'),
  'D039662': NtO(MH='Prokaryotic Initiation Factors', MS='Peptide initiation factors from prokaryotic organisms. Only three factors are needed for translation initiation in prokaryotic organisms, which occurs by a far simpler process than in PEPTIDE CHAIN INITIATION, TRANSLATIONAL of eukaryotic organisms.'),
  'D039663': NtO(MH='Integrin beta4', MS='Also known as CD104 antigen, this protein is distinguished from other beta integrins by its relatively long cytoplasmic domain (approximately 1000 amino acids vs. approximately 50). Five alternatively spliced isoforms have been described.'),
  'D039665': NtO(MH='Prokaryotic Initiation Factor-3', MS='A prokaryotic initiation factor that plays a role in recycling of ribosomal subunits for a new round of translational initiation. It binds to 16S RIBOSOMAL RNA and stimulates the dissociation of vacant 70S ribosomes. It may also be involved in the preferential binding of initiator tRNA to the 30S initiation complex.'),
  'D039681': NtO(MH='Pyranocoumarins', MS='A type of COUMARINS with added pyran ring(s).'),
  'D039682': NtO(MH='HIV-Associated Lipodystrophy Syndrome', MS='Defective metabolism leading to fat maldistribution in patients infected with HIV. The etiology appears to be multifactorial and probably involves some combination of infection-induced alterations in metabolism, direct effects of antiretroviral therapy, and patient-related factors.'),
  'D039702': NtO(MH='Current Procedural Terminology', MS='Descriptive terms and identifying codes for reporting medical services and procedures performed by PHYSICIANS. It is produced by the AMERICAN MEDICAL ASSOCIATION and used in insurance claim reporting for MEDICARE; MEDICAID; and private health insurance programs (From CPT 2002).'),
  'D039703': NtO(MH='Logical Observation Identifiers Names and Codes', MS='A vocabulary database of universal identifiers for laboratory and clinical test results. Its purpose is to facilitate the exchange and pooling of results for clinical care, outcomes management, and research. It is produced by the Regenstrief Institute. (LOINC and RELMA [Internet]. Indianapolis: The Regenstrief Institute; c1995-2001 [cited 2002 Apr 2]. Available from http://www.regenstrief.org/loinc)'),
  'D039721': NtO(MH='Diagnostic and Statistical Manual of Mental Disorders', MS='Categorical classification of MENTAL DISORDERS based on criteria sets with defining features. It is produced by the American Psychiatric Association. (DSM-IV, page xxii)'),
  'D039741': NtO(MH='Antigens, Dermatophagoides', MS='Antigens from the house dust mites (DERMATOPHAGOIDES), mainly D. farinae and D. pteronyssinus. They are proteins, found in mite feces or mite extracts, that can cause ASTHMA and other allergic diseases such as perennial rhinitis (RHINITIS, ALLERGIC, PERENNIAL) and atopic dermatitis (DERMATITIS, ATOPIC). More than 11 groups of Dermatophagoides ALLERGENS have been defined. Group I allergens, such as Der f I and Der p I from the above two species, are among the strongest mite immunogens in humans.'),
  'D039761': NtO(MH='Genista', MS="A plant genus of the family FABACEAE. The common name of broom may be confused with Scotch Broom (CYTISUS) or Butcher's Broom (RUSCUS) or Desert Broom (BACCHARIS) or Spanish Broom (SPARTIUM) or Brome (BROMUS)."),
  'D039762': NtO(MH='Ulex', MS='A plant genus of the family FABACEAE (sometimes classified as Papilionaceae). Members contain LECTINS.'),
  'D039781': NtO(MH='PubMed', MS="A bibliographic database that includes MEDLINE as its primary subset. It is produced by the National Center for Biotechnology Information (NCBI), part of the NATIONAL LIBRARY OF MEDICINE. PubMed, which is searchable through NLM's Web site, also includes access to additional citations to selected life sciences journals not in MEDLINE, and links to other resources such as the full-text of articles at participating publishers' Web sites, NCBI's molecular biology databases, and PubMed Central."),
  'D039821': NtO(MH='Monoterpenes', MS='Compounds with a core of 10 carbons generally formed via the mevalonate pathway from the combination of 3,3-dimethylallyl pyrophosphate and isopentenyl pyrophosphate. They are cyclized and oxidized in a variety of ways. Due to the low molecular weight many of them exist in the form of essential oils (OILS, VOLATILE).'),
  'D039823': NtO(MH='Iridoids', MS='A type of MONOTERPENES, derived from geraniol. They have the general form of cyclopentanopyran, but in some cases, one of the rings is broken as in the case of secoiridoid. They are different from the similarly named iridals (TRITERPENES).'),
  'D039841': NtO(MH='Leukocyte L1 Antigen Complex', MS='A member of the S-100 protein family that is present at high levels in the blood and interstitial fluid in several infectious, inflammatory, and malignant disorders, including rheumatoid arthritis, inflammatory bowel disease, and cystic fibrosis. It is a complex of a light chain (CALGRANULIN A) and a heavy chain (CALGRANULIN B). L1 binds calcium through an EF-hand motif, and has been shown to possess antimicrobial activity.'),
  'D039842': NtO(MH='Neural Cell Adhesion Molecule L1', MS='A member of the immunoglobulin superfamily of neuronal cell adhesion molecules that is required for proper nervous system development. Neural cell adhesion molecule L1 consists of six Ig domains, five fibronectin domains, a transmembrane region and an intracellular domain. Two splicing variants are known: a neuronal form that contains a four-amino acid RSLE sequence in the cytoplasmic domain, and a non-neuronal form that lacks the RSLE sequence. Mutations in the L1 gene result in L1 disease. Neural cell adhesion molecule L1 is predominantly expressed during development in neurons and Schwann cells; involved in cell adhesion, neuronal migration, axonal growth and pathfinding, and myelination.'),
  'D039861': NtO(MH='Prokaryotic Initiation Factor-1', MS='The smallest of the three prokaryotic initiation factors with a molecular size of approximately 8 kD. It binds near the A-site of the 30S subunit of RIBOSOMES and may play a role in preventing premature addition of aminoacyl-tRNA-linked PEPTIDE ELONGATION FACTOR TU to the ribosome during the initiation of a peptide chain (PEPTIDE CHAIN INITIATION, TRANSLATIONAL).'),
  'D039881': NtO(MH='Prokaryotic Initiation Factor-2', MS='The largest of the three prokaryotic initiation factors with a molecular size of approximately 80 kD. It functions in the transcription initiation process by promoting the binding of formylmethionine-tRNA to the P-site of the 30S ribosome and by preventing the incorrect binding of elongator tRNA to the translation initiation site.'),
  'D039901': NtO(MH='Totipotent Stem Cells', MS='Single cells that have the potential to form an entire organism. They have the capacity to specialize into extraembryonic membranes and tissues, the embryo, and all postembryonic tissues and organs. (Stem Cells: A Primer [Internet]. Bethesda (MD): National Institutes of Health (US); 2000 May [cited 2002 Apr 5]. Available from: http://www.nih.gov/news/stemcell/primer.htm)'),
  'D039902': NtO(MH='Multipotent Stem Cells', MS='Specialized stem cells that are committed to give rise to cells that have a particular function; examples are MYOBLASTS; MYELOID PROGENITOR CELLS; and skin stem cells. (Stem Cells: A Primer [Internet]. Bethesda (MD): National Institutes of Health (US); 2000 May [cited 2002 Apr 5]. Available from: http://www.nih.gov/news/stemcell/primer.htm)'),
  'D039903': NtO(MH='Actinobacteria', MS='Class of BACTERIA with diverse morphological properties. Strains of Actinobacteria show greater than 80% 16S rDNA/rRNA sequence similarity among each other and also the presence of certain signature nucleotides. (Stackebrandt E. et al, Int. J. Syst. Bacteriol. (1997) 47:479-491)'),
  'D039904': NtO(MH='Pluripotent Stem Cells', MS='Cells that can give rise to cells of the three different GERM LAYERS.'),
  'D039921': NtO(MH='Neuropilins', MS='Neuropilins are 140-kDa vertebrate cell surface receptors that bind neuronal guidance molecules during neural development and axonal outgrowth, and modulate VEGF-mediated angiogenesis. NEUROPILIN-1 and NEUROPILIN-2 differ in their binding specificities, and are distributed complementarily in regions of the developing nervous system. Neuropilins are receptors for secreted CLASS 3 SEMAPHORINS as well as for vascular endothelial growth factors, and may form hetero- or homodimers. They may also interact synergistically with plexins and with VEGF RECEPTORS to form receptor complexes with distinct affinities and specificities. Neuropilin binding specificity is determined by CUB and coagulation-factor-like domains in the extracellular portion of the molecule, while a MAM domain is essential for SIGNAL TRANSDUCTION.'),
  'D039941': NtO(MH='Bifidobacteriales Infections', MS='Infections with BACTERIA of the order Bifidobacteriales. This includes infections in the genera BIFIDOBACTERIUM and GARDNERELLA, in the family Bifidobacteriaceae.'),
  'D039942': NtO(MH='Neuropilin-1', MS='A dimeric cell surface receptor involved in angiogenesis (NEOVASCULARIZATION, PHYSIOLOGICAL) and AXONAL GUIDANCE. Neuropilin-1 is a 140-kDa transmembrane protein that binds CLASS 3 SEMAPHORINS, and several other growth factors. Neuropilin-1 forms complexes with plexins or VEGF RECEPTORS; their binding affinity and specificity are determined by the composition of the neuropilin dimer and the identity of other receptors complexed with it. Neuropilin-1 is expressed in distinct patterns during neural development, complementary to those described for NEUROPILIN-2.'),
  'D039943': NtO(MH='Neuropilin-2', MS='Transmembrane receptor for CLASS 3 SEMAPHORINS and several vascular endothelial growth factor isoforms. Neuropilin-2 functions either as a homodimer or as a heterodimer with NEUROPILIN-1. The binding affinity of neuropilin-2 varies for different class 3 semaphorin isoforms and is dependent on the composition of the dimer. The protein also forms receptor complexes with plexins and with VEGF RECEPTORS, which alters the binding characteristics of the receptor.'),
  'D039961': NtO(MH='Semaphorins', MS='A family of proteins that mediate AXONAL GUIDANCE. Semaphorins act as repulsive cues for neuronal GROWTH CONES and bind to receptors on their filopodia. At least 20 different molecules have been described and divided into eight classes based on domain organization and species of origin. Classes 1 and 2 are invertebrate, classes 3-7 are vertebrate, and class V are viral. Semaphorins may be secreted (classes 2, 3, and V), transmembrane (classes 1, 4, 5, and 6), or membrane-anchored (class 7). All semaphorins possess a common 500-amino acid extracellular domain which is critical for receptor binding and specificity, and is also found in plexins and scatter factor receptors. Their C termini are class-specific and may contain additional sequence motifs.'),
  'D039981': NtO(MH='Pyroglyphidae', MS='Family of house dust mites, in the superfamily Analgoidea, order Astigmata. They include the genera Dermatophagoides and Euroglyphus.'),
  'D040001': NtO(MH='Dermatophagoides farinae', MS='Species of American house dust mite, in the family PYROGLYPHIDAE.'),
  'D040002': NtO(MH='Dermatophagoides pteronyssinus', MS='Species of European house dust mite, in the family PYROGLYPHIDAE. It is the most commonly found house dust mite.'),
  'D040021': NtO(MH='Sarcoptidae', MS='Family of MITES, in the superfamily Sarcoptoidea, order Astigmata. They are slow moving, obligate PARASITES that infect MAMMALS and BIRDS. The species SARCOPTES SCABIEI causes SCABIES.'),
  'D040041': NtO(MH='Acaridae', MS='Family of MITES, in the superfamily Acaroidea, order Astigmata. They are frequently found in cereal-based foodstuffs including GRAIN and FLOUR.'),
  'D040042': NtO(MH='Psoroptidae', MS='Family of parasitic MITES, in the superfamily Sarcoptoidea, order Astigmata. Genera include Psoroptes and Chorioptes.'),
  'D040061': NtO(MH='Eukaryotic Initiation Factor-5', MS='A eukaryotic initiation factor that interacts with the 40S initiation complex and promotes the hydrolysis of the bound GTP. The hydrolysis of GTP causes the release of EUKARYOTIC INITIATION FACTOR-2 and EUKARYOTIC INITIATION FACTOR-3 from the 40S subunit and the subsequent joining of the 60S ribosomal subunit to the 40S complex to form the functional 80S initiation complex'),
  'D040081': NtO(MH='Protein Array Analysis', MS='Ligand-binding assays that measure protein-protein, protein-small molecule, or protein-nucleic acid interactions using a very large set of capturing molecules, i.e., those attached separately on a solid support, to measure the presence or interaction of target molecules in the sample.'),
  'D040102': NtO(MH='Tetranychidae', MS='Family of spider MITES, in the superfamily Tetranychoidea, suborder Trombidiformes.'),
  'D040121': NtO(MH='Semaphorin-3A', MS='The prototypical and most well-studied member of the semaphorin family. Semaphorin-3A is an axon-repulsive guidance cue for migrating neurons in the developing nervous system. It has so far been found only in vertebrates, and binds to NEUROPILIN-1/plexin complex receptors on growth cones. Like other class 3 semaphorins, it is a secreted protein.'),
  'D040141': NtO(MH='Staphylococcaceae', MS='Family of gram-positive, facultatively anaerobic bacteria, in the order Bacillales. Genera include Gemella, Macrococcus, Salinicoccus, and STAPHYLOCOCCUS.'),
  'D040142': NtO(MH='Brevibacterium flavum', MS='A species in the genus CORYNEBACTERIUM, family Corynebacteriaceae, which is used for industrial production of the amino acid LYSINE. It is closely related to Corynebacterium glutamicum.'),
  'D040143': NtO(MH='Cellulomonas', MS='A genus of aerobic or facultatively anaerobic BACTERIA, in the family Cellulomonadaceae. It is found in the SOIL and capable of hydrolyzing CELLULOSE.'),
  'D040161': NtO(MH='Frankia', MS='Genus of BACTERIA in the family Frankiaceae. They are nitrogen-fixing root-nodule symbionts of many species of woody dicotyledonous plants.'),
  'D040181': NtO(MH='Genetic Diseases, X-Linked', MS='Genetic diseases that are linked to gene mutations on the X CHROMOSOME in humans (X CHROMOSOME, HUMAN) or the X CHROMOSOME in other species. Included here are animal models of human X-linked diseases.'),
  'D040201': NtO(MH='Platelet Membrane Glycoprotein IIb', MS='Platelet membrane glycoprotein IIb is an integrin alpha subunit that heterodimerizes with INTEGRIN BETA3 to form PLATELET GLYCOPROTEIN GPIIB-IIIA COMPLEX. It is synthesized as a single polypeptide chain which is then postranslationally cleaved and processed into two disulfide-linked subunits of approximately 18 and 110 kDa in size.'),
  'D040241': NtO(MH='Social Marketing', MS='Use of marketing principles also used to sell products to consumers to promote ideas, attitudes and behaviors. Design and use of programs seeking to increase the acceptance of a social idea or practice by target groups, not for the benefit of the marketer, but to benefit the target audience and the general society.'),
  'D040242': NtO(MH='Risk Reduction Behavior', MS='Reduction of high-risk choices and adoption of low-risk quantity and frequency alternatives.'),
  'D040261': NtO(MH='Harm Reduction', MS='The application of methods designed to reduce the risk of harm associated with certain behaviors without reduction in frequency of those behaviors. The risk-associated behaviors include ongoing and active addictive behaviors.'),
  'D040262': NtO(MH='Receptors, Vascular Endothelial Growth Factor', MS='A family of closely related RECEPTOR PROTEIN-TYROSINE KINASES that bind vascular endothelial growth factors. They share a cluster of seven extracellular IG-LIKE DOMAINS which are important for ligand binding. They are highly expressed in vascular endothelial cells and are critical for the physiological and pathological growth, development and maintenance of blood and lymphatic vessels.'),
  'D040281': NtO(MH='Vascular Endothelial Growth Factor Receptor-1', MS='A 180-kDa VEGF receptor found primarily in endothelial cells that is essential for vasculogenesis and vascular maintenance. It is also known as Flt-1 (fms-like tyrosine kinase receptor-1). A soluble, alternatively spliced isoform of the receptor may serve as a binding protein that regulates the availability of various ligands for VEGF receptor binding and signal transduction.'),
  'D040301': NtO(MH='Vascular Endothelial Growth Factor Receptor-2', MS='A 200-230-kDa tyrosine kinase receptor for vascular endothelial growth factors found primarily in endothelial and hematopoietic cells and their precursors. VEGFR-2 is important for vascular and hematopoietic development, and mediates almost all endothelial cell responses to VEGF.'),
  'D040321': NtO(MH='Vascular Endothelial Growth Factor Receptor-3', MS='A vascular endothelial cell growth factor receptor whose expression is restricted primarily to adult lymphatic endothelium. VEGFR-3 preferentially binds the vascular endothelial growth factor C and vascular endothelial growth factor D and may be involved in the control of lymphangiogenesis.'),
  'D040342': NtO(MH='Genetic Structures', MS='The biological objects that contain genetic information and that are involved in transmitting genetically encoded traits from one organism to another.'),
  'D040343': NtO(MH='Behavioral Risk Factor Surveillance System', MS='Telephone surveys are conducted to monitor prevalence of the major behavioral risks among adults associated with premature MORBIDITY and MORTALITY. The data collected is in regard to actual behaviors, rather than on attitudes or knowledge. The Centers for Disease Control and Prevention (CDC) established the Behavioral Risk Factor Surveillance System (BRFSS) in 1984.'),
  'D040361': NtO(MH='Healthy People Programs', MS="Healthy People Programs are a set of health objectives to be used by governments, communities, professional organizations, and others to help develop programs to improve health. It builds on initiatives pursued over the past two decades beginning with the 1979 Surgeon General's Report, Healthy People, Healthy People 2000: National Health Promotion and Disease Prevention Objectives, and Healthy People 2010. These established national health objectives and served as the basis for the development of state and community plans. These are administered by the Office of Disease Prevention and Health Promotion (ODPHP). Similar programs are conducted by other national governments."),
  'D040362': NtO(MH='Geographic Information Systems', MS='Computer systems capable of assembling, storing, manipulating, and displaying geographically referenced information, i.e. data identified according to their locations.'),
  'D040363': NtO(MH='American Public Health Association', MS='Professional organization concerned with issues affecting personal and environmental health, including federal and state funding for health programs, programs related to chronic and infectious diseases, and professional education in public health.'),
  'D040381': NtO(MH='Education, Public Health Professional', MS='Education and training in PUBLIC HEALTH for the practice of the profession.'),
  'D040421': NtO(MH='Cell Phone', MS='Analog or digital communications device in which the user has a wireless connection from a telephone to a nearby transmitter. It is termed cellular because the service area is divided into multiple "cells." As the user moves from one cell area to another, the call is transferred to the local transmitter.'),
  'D040441': NtO(MH='Health Educators', MS='Professionals who plan, organize and direct health education programs for the individual, groups and the community.'),
  'D040442': NtO(MH='Public Health Informatics', MS='The systematic application of information and computer sciences to public health practice, research, and learning.'),
  'D040443': NtO(MH='Students, Public Health', MS='Individuals enrolled in a school of PUBLIC HEALTH or a formal educational program in public health.'),
  'D040461': NtO(MH='Gene Components', MS='The parts of the gene sequence that carry out the different functions of the GENES.'),
  'D040481': NtO(MH='Genome Components', MS='The parts of a GENOME sequence that are involved with the different functions or properties of genomes as a whole as opposed to those of individual GENES.'),
  'D040501': NtO(MH='Calgranulin A', MS='A 10.8-kDa member of the S-100 family of calcium-binding proteins that can form homo- or heterocomplexes with CALGRANULIN B and a variety of other proteins. The calgranulin A/B heterodimer is known as LEUKOCYTE L1 ANTIGEN COMPLEX. Calgranulin A is found in many cell types including GRANULOCYTES; KERATINOCYTES; and myelomonocytes, and has been shown to act as a chemotactic substance for NEUTROPHILS. Because it is present in acute inflammation but absent in chronic inflammation, it is a useful biological marker for a number of pathological conditions.'),
  'D040502': NtO(MH='Calgranulin B', MS='A 13.2-kDa member of the S-100 family of calcium-binding proteins that can form homo- or heterocomplexes with CALGRANULIN A and a variety of other proteins. The calgranulin A/B heterodimer is known as LEUKOCYTE L1 ANTIGEN COMPLEX. Calgranulin B is expressed at high concentrations in GRANULOCYTES during early monocyte differentiation, and serum calgranulin B levels are elevated in many inflammatory disorders such as CYSTIC FIBROSIS.'),
  'D040503': NtO(MH='Coffea', MS='A plant genus of the family RUBIACEAE. It is best known for the COFFEE beverage prepared from the beans (SEEDS).'),
  'D040521': NtO(MH='Subcutaneous Tissue', MS='Loose connective tissue lying under the DERMIS, which binds SKIN loosely to subjacent tissues. It may contain a pad of ADIPOCYTES, which vary in number according to the area of the body and vary in size according to the nutritional state.'),
  'D040541': NtO(MH='Marketing', MS='Activity involved in transfer of goods from producer to consumer or in the exchange of services.'),
  'D040542': NtO(MH='Mechanotransduction, Cellular', MS='The process by which cells convert mechanical stimuli into a chemical response. It can occur in both cells specialized for sensing mechanical cues such as MECHANORECEPTORS, and in parenchymal cells whose primary function is not mechanosensory.'),
  'D040561': NtO(MH='Assisted Living Facilities', MS='A housing and health care alternative combining independence with personal care. It provides a combination of housing, personalized supportive services and health care designed to meet the needs, both scheduled and unscheduled, of those who need help with activities of daily living. (www.alfa.org)'),
  'D040582': NtO(MH='Inheritance Patterns', MS='The different ways GENES and their ALLELES interact during the transmission of genetic traits that effect the outcome of GENE EXPRESSION.'),
  'D040641': NtO(MH='Quantitative Trait Loci', MS='Genetic loci associated with a quantitative trait.'),
  'D040681': NtO(MH='Structural Homology, Protein', MS='The degree of 3-dimensional shape similarity between proteins. It can be an indication of distant AMINO ACID SEQUENCE HOMOLOGY and used for rational DRUG DESIGN.'),
  'D040701': NtO(MH='Stress Disorders, Traumatic, Acute', MS='A class of traumatic stress disorders that is characterized by the significant dissociative states seen immediately after overwhelming trauma. By definition it cannot last longer than 1 month, if it persists, a diagnosis of post-traumatic stress disorder (STRESS DISORDERS, POST-TRAUMATIC) is more appropriate.'),
  'D040741': NtO(MH='Genetic Load', MS='The relative amount by which the average fitness of a POPULATION is lowered, due to the presence of GENES that decrease survival, compared to the GENOTYPE with maximum or optimal fitness. (From Rieger et al., Glossary of Genetics: Classical and Molecular, 5th ed)'),
  'D040761': NtO(MH='Biomimetic Materials', MS='Materials fabricated by BIOMIMETICS techniques, i.e., based on natural processes found in biological systems.'),
  'D040821': NtO(MH='Human Characteristics', MS="The fundamental dispositions and traits of humans. (Merriam-Webster's Collegiate Dictionary, 10th ed)"),
  'D040841': NtO(MH='Resource Allocation', MS='Societal or individual decisions about the equitable distribution of available resources.'),
  'D040861': NtO(MH='Ethics, Business', MS='The moral obligations governing the conduct of commercial or industrial enterprises.'),
  'D040881': NtO(MH='CD11a Antigen', MS='An alpha-integrin subunit found on lymphocytes, granulocytes, macrophages and monocytes. It combines with the integrin beta2 subunit (CD18 ANTIGEN) to form LYMPHOCYTE FUNCTION-ASSOCIATED ANTIGEN-1.'),
  'D040901': NtO(MH='Proteomics', MS='The systematic study of the complete complement of proteins (PROTEOME) of organisms.'),
  'D040921': NtO(MH='Stress Disorders, Traumatic', MS='Anxiety disorders manifested by the development of characteristic symptoms following a psychologically traumatic event that is outside the normal range of usual human experience. Symptoms include re-experiencing the traumatic event, increased arousal, and numbing of responsiveness to or reduced involvement with the external world. Traumatic stress disorders can be further classified by the time of onset and the duration of these symptoms.'),
  'D040941': NtO(MH='Heredity', MS='The transmission of traits encoded in GENES from parent to offspring.'),
  'D040961': NtO(MH='Genetic Drift', MS='The fluctuation of the ALLELE FREQUENCY from one generation to the next.'),
  'D040981': NtO(MH='Acinetobacter baumannii', MS='A species of gram-negative, aerobic bacteria, commonly found in the clinical laboratory, and frequently resistant to common antibiotics.'),
  'D041001': NtO(MH='Babesia microti', MS='A species of protozoa infecting humans via the intermediate tick vector IXODES scapularis. The other hosts are the mouse PEROMYSCUS leucopus and meadow vole MICROTUS pennsylvanicus, which are fed on by the tick. Other primates can be experimentally infected with Babesia microti.'),
  'D041002': NtO(MH='Brucella canis', MS='A species of gram-negative bacteria infecting DOGS, the natural hosts, and causing canine BRUCELLOSIS. It can also cause a mild infection in humans.'),
  'D041021': NtO(MH='Brucella suis', MS='A species of gram-negative bacteria, primarily infecting SWINE, but it can also infect humans, DOGS, and HARES.'),
  'D041022': NtO(MH='Candida tropicalis', MS='A species of MITOSPORIC FUNGI that is a major cause of SEPTICEMIA and disseminated CANDIDIASIS, especially in patients with LYMPHOMA; LEUKEMIA; and DIABETES MELLITUS. It is also found as part of the normal human mucocutaneous flora.'),
  'D041041': NtO(MH='Citrobacter koseri', MS='A species of gram-negative enterobacteria found in WATER; SEWAGE; SOIL; and FOOD. It can be present in any clinical specimen as an opportunistic pathogen.'),
  'D041061': NtO(MH='Ehrlichia canis', MS='Species of gram-negative bacteria in the family ANAPLASMATACEAE, causing EHRLICHIOSIS in DOGS. The most common vector is the brown dog tick. It can also cause disease in humans.'),
  'D041081': NtO(MH='Anaplasma phagocytophilum', MS='A species of gram-negative bacteria in the genus ANAPLASMA, family ANAPLASMATACEAE, formerly called Ehrlichia phagocytophila or Ehrlichia equi. This organism is tick-borne (IXODES) and causes disease in horses and sheep. In humans, it causes human granulocytic EHRLICHIOSIS.'),
  'D041101': NtO(MH='Neorickettsia sennetsu', MS='A species of gram-negative bacteria in the genus NEORICKETTSIA, family ANAPLASMATACEAE, causing Sennetsu fever and found in the FAR EAST and SOUTHEAST ASIA.'),
  'D041102': NtO(MH='Neorickettsia', MS='Genus of gram-negative, intracytoplasmic bacteria that are found primarily in VACUOLES of MONOCYTES in the BLOOD and MACROPHAGES of lymphoid or other tissues of DOGS; HORSES, and humans. (From Int. J. Syst. Evol. Microbiol. (2001) 51:2145-2165)'),
  'D041103': NtO(MH='Neorickettsia risticii', MS='A species of gram-negative bacteria in the genus NEORICKETTSIA, family ANAPLASMATACEAE.'),
  'D041121': NtO(MH='Klebsiella oxytoca', MS='A species of gram-negative bacteria causing URINARY TRACT INFECTIONS and SEPTICEMIA.'),
  'D041122': NtO(MH='Plasmodium ovale', MS='A species of protozoan parasite causing MALARIA. It is the rarest of the four species of PLASMODIUM infecting humans, but is common in West African countries and neighboring areas.'),
  'D041141': NtO(MH='Rickettsia akari', MS='A species of gram-negative bacteria that causes rickettsialpox. The vector is a mouse mite and the reservoirs are mites and mice.'),
  'D041161': NtO(MH='Rickettsia felis', MS='A species of gram-negative bacteria transmitted by the flea Ctenocephalides felis, and known to infect CATS, oppossums, and humans.'),
  'D041162': NtO(MH='Staphylococcus haemolyticus', MS='A species of STAPHYLOCOCCUS found on the skin of humans (and non-human primates), often causing hospital-acquired infections (CROSS INFECTION).'),
  'D041181': NtO(MH='Staphylococcus hominis', MS='A species of STAPHYLOCOCCUS similar to STAPHYLOCOCCUS HAEMOLYTICUS, but containing different esterases. The subspecies Staphylococcus hominis novobiosepticus is highly virulent and novobiocin resistant.'),
  'D041201': NtO(MH='Taenia solium', MS='Species of tapeworm in the genus TAENIA, that infects swine. It is acquired by humans through the ingestion of cured or undercooked pork.'),
  'D041202': NtO(MH='Taenia saginata', MS='Species of tapeworm in the genus TAENIA, that infects cattle. It is acquired by humans through the ingestion of raw or insufficiently cooked beef.'),
  'D041221': NtO(MH='Candida glabrata', MS='A species of MITOSPORIC FUNGI commonly found on the body surface. It causes opportunistic infections especially in immunocompromised patients.'),
  'D041241': NtO(MH='Vibrio cholerae O1', MS='Strains of VIBRIO CHOLERAE containing O ANTIGENS group 1. All are CHOLERA-causing strains (serotypes). There are two biovars (biotypes): cholerae and eltor (El Tor).'),
  'D041242': NtO(MH='Vibrio cholerae O139', MS='Strains of VIBRIO CHOLERAE containing O ANTIGENS group 139. This strain emerged in India in 1992 and caused a CHOLERA epidemic.'),
  'D041261': NtO(MH='Vibrio vulnificus', MS='A species of halophilic bacteria in the genus VIBRIO, which lives in warm SEAWATER. It can cause infections in those who eat raw contaminated seafood or have open wounds exposed to seawater.'),
  'D041281': NtO(MH='Leptospira interrogans serovar australis', MS='A serovar of the bacterial species LEPTOSPIRA INTERROGANS, characteristically associated with a severe zoonotic disease in humans.'),
  'D041301': NtO(MH='Leptospira interrogans serovar autumnalis', MS='A serovar of the bacterial species LEPTOSPIRA INTERROGANS, whose primary host is the MOUSE.'),
  'D041321': NtO(MH='Chromosomes, Human, X', MS='The human female sex chromosome, being the differential sex chromosome carried by half the male gametes and all female gametes in humans.'),
  'D041322': NtO(MH='Chromosomes, Human, Y', MS='The human male sex chromosome, being the differential sex chromosome carried by half the male gametes and none of the female gametes in humans.'),
  'D041341': NtO(MH='Leptospira interrogans serovar canicola', MS='A serovar of the bacterial species LEPTOSPIRA INTERROGANS, whose natural host is DOGS. The disease is characterized by GASTROENTERITIS, and INTERSTITIAL NEPHRITIS.'),
  'D041342': NtO(MH='Absorbent Pads', MS='Pads made of various materials used for personal hygiene usually for absorbing URINE or FECES. They can be worn as underpants or pants liners by various age groups, from NEWBORNS to the ELDERLY. Absorbent pads can be made of fluff wood pulp and HYDROGEL absorbent covered with viscose rayon, polyester, polypropylene, or POLYETHYLENE coverstock.'),
  'D041343': NtO(MH='Leptospira interrogans serovar hebdomadis', MS='A serovar of the bacterial species LEPTOSPIRA INTERROGANS, whose frequent host is CATTLE.'),
  'D041361': NtO(MH='Pacifiers', MS='Devices that babies can suck on when they are not feeding. The extra sucking can be comforting to the babies and pacify them. Pacifiers usually are used as a substitute for the thumb in babies who suck on their thumb or fingers almost constantly.'),
  'D041381': NtO(MH='Diapers, Infant', MS='Absorbent pads designed to be worn by infants and very young children.'),
  'D041382': NtO(MH='Diapers, Adult', MS='Absorbent pads designed to be worn as underpants or pants liners by adults.'),
  'D041383': NtO(MH='Leptospira interrogans serovar icterohaemorrhagiae', MS='A serovar of the bacterial species LEPTOSPIRA INTERROGANS, whose primary host is RATS.'),
  'D041401': NtO(MH='Leptospira interrogans serovar pomona', MS='A serovar of the bacterial species LEPTOSPIRA INTERROGANS, whose primary hosts include CATTLE and SWINE.'),
  'D041441': NtO(MH='Retinoschisis', MS='A vitreoretinal dystrophy characterized by splitting of the neuroretinal layers. It occurs in two forms: degenerative retinoschisis and X chromosome-linked juvenile retinoschisis.'),
  'D041521': NtO(MH='Desulfurococcales', MS='An order of CRENARCHAEOTA comprised of irregular coccoid to disc-shaped, hyperthermophiles, and found in submarine hydrothermal systems and solfataric hot springs.'),
  'D041522': NtO(MH='Aeropyrum', MS='A genus of anaerobic, chemolithotropic coccoid ARCHAEA, in the family DESULFUROCOCCACEAE. They live in marine environments.'),
  'D041523': NtO(MH='Pyrodictiaceae', MS='A family of archaea, in the order DESULFUROCOCCALES consisting of anaerobic coccoid to disc-shaped cells. They grow either chemolithoautotrophically or by FERMENTATION. Three genera are recognized: Pyrodictium, Hyperthermus, and Pyrolobus.'),
  'D041541': NtO(MH='Acidianus', MS='A genus of facultatively anaerobic coccoid ARCHAEA, in the family SULFOLOBACEAE. Cells are highly irregular in shape and thermoacidophilic. Lithotrophic growth occurs aerobically via sulfur oxidation in some species. Distribution includes solfataric springs and fields, mudholes, and geothermically heated acidic marine environments.'),
  'D041542': NtO(MH='Thermoproteus', MS='A genus of obligately anaerobic ARCHAEA, in the family THERMOPROTEACEAE. They are found in acidic hot springs and water holes.'),
  'D041543': NtO(MH='Pyrobaculum', MS='A genus of rod-shaped, almost rectangular ARCHAEA, in the family THERMOPROTEACEAE. Organisms are facultatively aerobic or strictly anaerobic, grow on various organic substrates, and are found in continental solfataras.'),
  'D041561': NtO(MH='Thermofilaceae', MS='A family of THERMOPROTEALES consisting of anaerobic, thermoacidophilic thin rods found in solfataric hot springs.'),
  'D041581': NtO(MH='Methanobrevibacter', MS='A genus of gram-positive, anaerobic, cocci to short rod-shaped ARCHAEA, in the family METHANOBACTERIACEAE, order METHANOBACTERIALES. They are found in the GASTROINTESTINAL TRACT or other anoxic environments.'),
  'D041601': NtO(MH='Bonamia Plant', MS='A plant genus of the family CONVOLVULACEAE. Members contain bonaspectins (tetrahydrofuran-type sesqui-lignans).'),
  'D041602': NtO(MH='Haplosporida', MS='A phylum of EUKARYOTES in the RHIZARIA group. They are small endoparasites of marine invertebrates. Spores are structurally complex but without polar filaments or tubes.'),
  'D041603': NtO(MH='Pinus taeda', MS='A plant species of the genus PINUS which is the subject of genetic study.'),
  'D041604': NtO(MH='Pinus ponderosa', MS='A plant species of the genus PINUS that contains isocupressic acid.'),
  'D041605': NtO(MH='Pinus sylvestris', MS='A plant species of the genus PINUS which is the source of pinosylvin. It is sometimes called Scotch pine or Scots pine, which is also a common name for other species of this genus.'),
  'D041621': NtO(MH='Eremophila Plant', MS='A plant genus of the family MYOPORACEAE. Members have been used in FOLK MEDICINE.'),
  'D041622': NtO(MH='Tomography, Optical', MS='Projection of near-IR light (INFRARED RAYS), in the 700-1000 nm region, across an object in parallel beams to an array of sensitive photodetectors. This is repeated at various angles and a mathematical reconstruction provides three dimensional MEDICAL IMAGING of tissues. Based on the relative transparency of tissues to this spectra, it has been used to monitor local oxygenation, brain and joints.'),
  'D041623': NtO(MH='Tomography, Optical Coherence', MS='An imaging method using LASERS that is used for mapping subsurface structure. When a reflective site in the sample is at the same optical path length (coherence) as the reference mirror, the detector observes interference fringes.'),
  'D041641': NtO(MH='Musculoskeletal Development', MS='The morphologic and physiological changes of the MUSCLES, bones (BONE AND BONES), and CARTILAGE of the body, i.e., MUSCULOSKELETAL SYSTEM, during the prenatal and postnatal stages of development.'),
  'D041681': NtO(MH='NIH 3T3 Cells', MS='A continuous cell line of high contact-inhibition established from NIH Swiss mouse embryo cultures. The cells are useful for DNA transfection and transformation studies. (From ATCC [Internet]. Virginia: American Type Culture Collection; c2002 [cited 2002 Sept 26]. Available from http://www.atcc.org/)'),
  'D041701': NtO(MH='Swiss 3T3 Cells', MS='A cell line established in 1962 from disaggregated Swiss albino mouse embryos. This fibroblast cell line is extremely popular in research.'),
  'D041702': NtO(MH='BALB 3T3 Cells', MS='Cell lines developed from disaggregated BALB/c mouse embryos. They are extremely sensitive to CONTACT INHIBITION, and highly susceptible to transformation by SV40 VIRUS and murine sarcoma virus (SARCOMA VIRUSES, MURINE).'),
  'D041721': NtO(MH='3T3-L1 Cells', MS='A continuous cell line that is a substrain of SWISS 3T3 CELLS developed though clonal isolation. The mouse fibroblast cells undergo an adipose-like conversion as they move to a confluent and contact-inhibited state.'),
  'D041722': NtO(MH='Genes, Transgenic, Suicide', MS='Genes that are used transgenically, i.e., via GENE TRANSFER TECHNIQUES to induce CELL DEATH.'),
  'D041741': NtO(MH='Lower Gastrointestinal Tract', MS='The segment of GASTROINTESTINAL TRACT that includes the small intestine below the DUODENUM, and the LARGE INTESTINE.'),
  'D041742': NtO(MH='Upper Gastrointestinal Tract', MS='The segment of GASTROINTESTINAL TRACT that includes the ESOPHAGUS; the STOMACH; and the DUODENUM.'),
  'D041743': NtO(MH='Herb-Drug Interactions', MS='The adverse or beneficial effect of MEDICINAL HERBS on the activity, metabolism, or toxicity of PHARMACEUTICALS.'),
  'D041761': NtO(MH='Cholecystolithiasis', MS='Presence or formation of GALLSTONES in the GALLBLADDER.'),
  'D041781': NtO(MH='Jaundice, Obstructive', MS='Jaundice, the condition with yellowish staining of the skin and mucous membranes, that is due to impaired BILE flow in the BILIARY TRACT, such as INTRAHEPATIC CHOLESTASIS, or EXTRAHEPATIC CHOLESTASIS.'),
  'D041801': NtO(MH='Acidiphilium', MS='A genus in the family ACETOBACTERACEAE consisting of chemoorganotrophic, straight rods with rounded ends. They are aerobic and acidophilic.'),
  'D041821': NtO(MH='Gluconacetobacter', MS='A genus in the family ACETOBACTERACEAE comprised of acetate-oxidizing bacteria.'),
  'D041822': NtO(MH='Gluconacetobacter xylinus', MS='A species of acetate-oxidizing bacteria, formerly known as Acetobacter xylinum.'),
  'D041841': NtO(MH='Bradyrhizobiaceae', MS='A proposed family of bacteria belonging to the alpha-2 subgroup of PROTEOBACTERIA.'),
  'D041861': NtO(MH='Chloroflexus', MS='A genus of green nonsulfur bacteria in the family Chloroflexaceae. They are photosynthetic, thermophilic, filamentous gliding bacteria found in hot springs.'),
  'D041862': NtO(MH='Chloroflexi', MS='Phylum of green nonsulfur bacteria including the family Chloroflexaceae, among others.'),
  'D041881': NtO(MH='Cholecystitis, Acute', MS='Acute inflammation of the GALLBLADDER wall. It is characterized by the presence of ABDOMINAL PAIN; FEVER; and LEUKOCYTOSIS. Gallstone obstruction of the CYSTIC DUCT is present in approximately 90% of the cases.'),
  'D041882': NtO(MH='Emphysematous Cholecystitis', MS='A variant of acute cholecystitis with inflammation of the GALLBLADDER that is characterized by the pockets of gas in the gallbladder wall. It is due to secondary infection caused by gas-forming organisms, and has a high risk of perforation.'),
  'D041883': NtO(MH='Chlorobium', MS='A genus of phototrophic, obligately anaerobic bacteria in the family Chlorobiaceae. They are found in hydrogen sulfide-containing mud and water environments.'),
  'D041901': NtO(MH='Stemonaceae', MS='A small plant family of the order Liliales, subclass Liliidae, class Liliopsida (monocotyledons).'),
  'D041902': NtO(MH='Rhodobacteraceae', MS='A family in the order Rhodobacterales, class ALPHAPROTEOBACTERIA.'),
  'D041903': NtO(MH='Rhodovulum', MS='A genus of facultatively or obligately anaerobic marine phototrophic bacteria, in the family RHODOBACTERACEAE.'),
  'D041904': NtO(MH='Roseobacter', MS='A genus of obligately aerobic marine phototrophic and chemoorganotrophic bacteria, in the family RHODOBACTERACEAE.'),
  'D041905': NtO(MH='Erythroid Cells', MS='The series of cells in the red blood cell lineage at various stages of differentiation.'),
  'D041921': NtO(MH='Hyphomicrobiaceae', MS='A family in the order Rhizobiales, class ALPHAPROTEOBACTERIA comprised of many genera of budding or appendaged bacteria.'),
  'D041923': NtO(MH='Adolescent Development', MS='The continuous sequential physiological and psychological changes during ADOLESCENCE, approximately between the age of 13 and 18.'),
  'D041941': NtO(MH='Milk Substitutes', MS='Food BEVERAGES that are used as nutritional substitutes for MILK.'),
  'D041942': NtO(MH='Soy Milk', MS='A beverage prepared from SOYBEANS.'),
  'D041943': NtO(MH='Infant Formula', MS='Formulations for the nutrition of infants that are substituted for BREAST MILK.'),
  'D041961': NtO(MH='Deuterium Exchange Measurement', MS='A research technique to measure solvent exposed regions of molecules that is used to provide insight about PROTEIN CONFORMATION.'),
  'D041962': NtO(MH='Thiotrichaceae', MS='A family of colorless sulfur bacteria in the order Thiotrichales, class GAMMAPROTEOBACTERIA.'),
  'D041963': NtO(MH='Bacteroidetes', MS='A phylum of bacteria comprising four classes; Bacteroidia, Cytophagia, Flavobacteriia, and Sphingobacteriia.'),
  'D041964': NtO(MH='Cytophagaceae', MS='A family of gram-negative, gliding bacteria in the order Cytophagales, class Cytophagia. They are found in SOIL and SEA WATER.'),
  'D041965': NtO(MH='Flavobacteriaceae', MS='A family of bacteria in the order Sphingobacteriales, class Sphingobacteria. They are gram-negative rods, mostly saprophytic in terrestrial and aquatic habitats.'),
  'D041981': NtO(MH='Gastrointestinal Tract', MS='Generally refers to the digestive structures stretching from the MOUTH to ANUS, but does not include the accessory glandular organs (LIVER; BILIARY TRACT; PANCREAS).'),
  'D042001': NtO(MH='Ectothiorhodospiraceae', MS='A family in the order Chromatiales, class GAMMAPROTEOBACTERIA. These are haloalkaliphilic, phototrophic bacteria that deposit elemental sulfur outside their cells.'),
  'D042002': NtO(MH='Chromatin Assembly and Disassembly', MS='The mechanisms effecting establishment, maintenance, and modification of that specific physical conformation of CHROMATIN determining the transcriptional accessibility or inaccessibility of the DNA.'),
  'D042003': NtO(MH='DNA Packaging', MS="The folding of an organism's DNA molecule into a compact, orderly structure that fits within the limited space of a CELL or VIRUS PARTICLE."),
  'D042021': NtO(MH='Brucellaceae', MS='A family of gram-negative coccoid to rod-shaped bacteria in the order Rhizobiales. They are obligate parasites chiefly of warm-blooded VERTEBRATES. Many are pathogenic.'),
  'D042041': NtO(MH='Ochrobactrum', MS='A genus of the family BRUCELLACEAE comprising obligately aerobic gram-negative rods with parallel sides and rounded ends.'),
  'D042061': NtO(MH='Brucella ovis', MS='A species of the genus BRUCELLA which are pathogenic to SHEEP.'),
  'D042062': NtO(MH='Beijerinckiaceae', MS='A family of aerobic gram-negative rods that are nitrogen fixers. They are highly viscous, and appear as a semitransparent slime in giant colonies.'),
  'D042063': NtO(MH='Caulobacteraceae', MS='A family of stalked bacteria that reproduces by budding. There are four genera: CAULOBACTER, Asticcacaulis, Brevundimonas, and Phenylobacterium.'),
  'D042081': NtO(MH='Azospirillum lipoferum', MS='A species of gram-negative to gram-variable, nitrogen-fixing bacteria. They are free-living in SOIL or found in association with PLANT ROOTS. It is the type species of the genus AZOSPIRILLUM.'),
  'D042082': NtO(MH='Methylobacteriaceae', MS='A family of gram-negative bacteria in the order Rhizobiales. Genera include METHYLOBACTERIUM, Protomonas, and Roseomonas.'),
  'D042083': NtO(MH='Methylocystaceae', MS='A family of gram-negative methanotrophs in the order Rhizobiales, distantly related to the nitrogen-fixing and phototrophic bacteria.'),
  'D042084': NtO(MH='Paracoccus pantotrophus', MS='A species of gram-negative, coccoid, mostly chemolithoautotrophic bacteria, in the family RHODOBACTERACEAE. Some strains can grow anaerobically.'),
  'D042101': NtO(MH='Acalculous Cholecystitis', MS='Inflammation of the GALLBLADDER wall in the absence of GALLSTONES.'),
  'D042121': NtO(MH='Contraception, Barrier', MS='Methods of contraception in which physical, chemical, or biological means are used to prevent the SPERM from reaching the fertilizable OVUM.'),
  'D042161': NtO(MH='Rhizobium phaseoli', MS='A species of gram-negative bacteria functioning as a nitrogen inoculum for dry beans, especially species in the genus PHASEOLUS.'),
  'D042162': NtO(MH='Rhizobium tropici', MS='A species of gram-negative bacteria and an nitrogen inoculum that displays a high intrinsic tolerance to acidity.'),
  'D042163': NtO(MH='Rhizobium etli', MS='A species of gram-negative bacteria and nitrogen innoculant of PHASEOLUS VULGARIS.'),
  'D042181': NtO(MH='Sinorhizobium fredii', MS='A species of gram-negative, aerobic bacteria that is a fast-growing and soybean-nodulating innoculant.'),
  'D042182': NtO(MH='Rhodospirillum centenum', MS='A thermotolerant, cyst-forming, anoxygenic photosynthetic species, in the genus RHODOSPIRILLUM.'),
  'D042201': NtO(MH='Magnetospirillum', MS='A genus of microaerophilic, gram-negative bacteria that forms crystals of the mineral magnetite in special organelles called MAGNETOSOMES.'),
  'D042241': NtO(MH='Early Diagnosis', MS='Methods to determine in patients the nature of a disease or disorder at its early stage of progression. Generally, early diagnosis improves PROGNOSIS and TREATMENT OUTCOME.'),
  'D042261': NtO(MH='Retinoscopes', MS="Instruments for RETINOSCOPY that determines the refractive state of the EYE, such as the degree of NEARSIGHTEDNESS; FARSIGHTEDNESS; or ASTIGMATISM. In principle, a retinoscope provides a light source to illuminate the RETINA, and then locates the aerial image of the retina in space to obtain an index of the refractive quality of the patient's lens system."),
  'D042262': NtO(MH='Retinoscopy', MS='An objective determination of the refractive state of the eye (NEARSIGHTEDNESS; FARSIGHTEDNESS; ASTIGMATISM). By using a RETINOSCOPE, the amount of correction and the power of lens needed can be determined.'),
  'D042281': NtO(MH='Holosporaceae', MS='A family of bacteria comprised of endosymbionts of protozoa.'),
  'D042282': NtO(MH='Microdissection', MS='The performance of dissections with the aid of a microscope.'),
  'D042301': NtO(MH='Sphingomonadaceae', MS='A family of gram-negative, asporogenous rods or ovoid cells, aerobic or facultative anaerobic chemoorganotrophs. They are commonly isolated from SOIL, activated sludge, or marine environments.'),
  'D042321': NtO(MH='Anaplasma centrale', MS='A species of gram-negative bacteria causing mild ANAPLASMOSIS in CATTLE. It also can infect SHEEP and GOATS. It is transmitted by TICKS.'),
  'D042322': NtO(MH='Anaplasma marginale', MS='A species of gram-negative bacteria and causative agent of severe bovine ANAPLASMOSIS. It is the most pathogenic of the ANAPLASMA species.'),
  'D042323': NtO(MH='Anaplasma ovis', MS='A species of gram-negative bacteria producing mild to severe ANAPLASMOSIS in SHEEP and GOATS, and mild or inapparent infections in DEER and CATTLE.'),
  'D042341': NtO(MH='Gonadal Hormones', MS='Hormones produced by the GONADS, including both steroid and peptide hormones. The major steroid hormones include ESTRADIOL and PROGESTERONE from the OVARY, and TESTOSTERONE from the TESTIS. The major peptide hormones include ACTIVINS and INHIBINS.'),
  'D042361': NtO(MH='Mammary Glands, Human', MS='Glandular tissue in the BREAST of human that is under the influence of hormones such as ESTROGENS; PROGESTINS; and PROLACTIN. In WOMEN, after PARTURITION, the mammary glands secrete milk (MILK, HUMAN) for the nourishment of the young.'),
  'D042381': NtO(MH='Granulocyte Precursor Cells', MS='The cells in the granulocytic series that give rise to mature granulocytes (NEUTROPHILS; EOSINOPHILS; and BASOPHILS). These precursor cells include myeloblasts, promyelocytes, myelocytes and metamyelocytes.'),
  'D042401': NtO(MH='Alcaligenaceae', MS='A family of gram-negative, aerobic, non-spore forming rods or cocci of the order BURKHOLDERIALES, which include . A well-known genera which includes ACHROMOBACTER; ALCALIGENES; and BORDETELLA.'),
  'D042402': NtO(MH='Achromobacter', MS='A genus of gram-negative, strictly aerobic, non-spore forming rods. Soil and water are regarded as the natural habitat. They are sometimes isolated from a hospital environment and humans.'),
  'D042421': NtO(MH='Histone Code', MS='The specific patterns of changes made to HISTONES, that are involved in assembly, maintenance, and alteration of chromatin structural states (such as EUCHROMATIN and HETEROCHROMATIN). The changes are made by various histone modification processes that include ACETYLATION; METHYLATION; PHOSPHORYLATION; and UBIQUITINATION.'),
  'D042422': NtO(MH='Achromobacter cycloclastes', MS='A species of gram negative, aerobic, denitrifying bacteria in the genus ACHROMOBACTER.'),
  'D042441': NtO(MH='Achromobacter denitrificans', MS='The type species of gram negative, aerobic bacteria in the genus ACHROMOBACTER. Previously in the genus ALCALIGENES, the classification and nomenclature of this species has been frequently emended. The two subspecies, Achromobacter xylosoxidans subsp. denitrificans and Achromobacter xylosoxidans subsp. xylosoxidans are associated with infections.'),
  'D042442': NtO(MH='Vascular Endothelial Growth Factors', MS='A family of angiogenic proteins that are closely-related to VASCULAR ENDOTHELIAL GROWTH FACTOR A. They play an important role in the growth and differentiation of vascular as well as lymphatic endothelial cells.'),
  'D042461': NtO(MH='Vascular Endothelial Growth Factor A', MS='The original member of the family of endothelial cell growth factors referred to as VASCULAR ENDOTHELIAL GROWTH FACTORS. Vascular endothelial growth factor-A was originally isolated from tumor cells and referred to as "tumor angiogenesis factor" and "vascular permeability factor". Although expressed at high levels in certain tumor-derived cells it is produced by a wide variety of cell types. In addition to stimulating vascular growth and vascular permeability it may play a role in stimulating VASODILATION via NITRIC OXIDE-dependent pathways. Alternative splicing of the mRNA for vascular endothelial growth factor A results in several isoforms of the protein being produced.'),
  'D042462': NtO(MH='4-Quinolones', MS='QUINOLONES containing a 4-oxo (a carbonyl in the para position to the nitrogen). They inhibit the A subunit of DNA GYRASE and are used as antimicrobials. Second generation 4-quinolones are also substituted with a 1-piperazinyl group at the 7-position and a fluorine at the 6-position.'),
  'D042481': NtO(MH='Alcaligenes faecalis', MS='The type species of gram negative bacteria in the genus ALCALIGENES, found in soil. It is non-pathogenic, non-pigmented, and used for the production of amino acids.'),
  'D042482': NtO(MH='Bordetella avium', MS='A species of BORDETELLA isolated from the respiratory tracts of TURKEYS and other BIRDS. It causes a highly contagious bordetellosis.'),
  'D042483': NtO(MH='Bordetella parapertussis', MS='A species of BORDETELLA with similar morphology to BORDETELLA PERTUSSIS, but growth is more rapid. It is found only in the RESPIRATORY TRACT of humans.'),
  'D042501': NtO(MH='Angiogenic Proteins', MS='Intercellular signaling peptides and proteins that regulate the proliferation of new blood vessels under normal physiological conditions (ANGIOGENESIS, PHYSIOLOGICAL). Aberrant expression of angiogenic proteins during disease states such as tumorigenesis can also result in PATHOLOGICAL ANGIOGENESIS.'),
  'D042502': NtO(MH='Taylorella', MS='A genus of chemoorganotrophic, gram negative rods, in the family ALCALIGENACEAE. They are pathogenic parasites of HORSES.'),
  'D042521': NtO(MH='Burkholderiaceae', MS='A family of gram negative, aerobic, non-sporeforming, rod-shaped bacteria.'),
  'D042522': NtO(MH='DNA Replication Timing', MS='The temporal order in which the DNA of the GENOME is replicated.'),
  'D042541': NtO(MH='Intracellular Space', MS='The area within CELLS.'),
  'D042561': NtO(MH='Vascular Endothelial Growth Factor B', MS='A vascular endothelial growth factor expressed in a variety of tissues. It binds with high specificity to VASCULAR ENDOTHELIAL GROWTH FACTOR RECEPTOR-1 and NEUROPILIN-1.'),
  'D042581': NtO(MH='Intranuclear Space', MS='The area within the CELL NUCLEUS.'),
  'D042582': NtO(MH='Vascular Endothelial Growth Factor C', MS='A vascular endothelial growth factor that specifically binds to VASCULAR ENDOTHELIAL GROWTH FACTOR RECEPTOR-2 and VASCULAR ENDOTHELIAL GROWTH FACTOR RECEPTOR-3. In addition to being an angiogenic factor it can act on LYMPHATIC VESSELS to stimulate LYMPHANGIOGENESIS. It is similar in structure to VASCULAR ENDOTHELIAL GROWTH FACTOR D in that they both contain N- and C-terminal extensions that were not found in other VEGF family members.'),
  'D042583': NtO(MH='Lymphangiogenesis', MS='The formation of LYMPHATIC VESSELS.'),
  'D042601': NtO(MH='Lymphatic Vessels', MS='Tubular vessels that are involved in the transport of LYMPH and LYMPHOCYTES.'),
  'D042602': NtO(MH='Burkholderia cepacia complex', MS='A group of phenotypically similar but genotypically distinct species (genomovars) in the genus BURKHOLDERIA. They are found in water, soil, and the rhizosphere of crop plants. They can act as opportunistic human pathogens and as plant growth promoting and biocontrol agents.'),
  'D042621': NtO(MH='Comamonadaceae', MS='A family of gram-negative aerobic bacteria in the order BURKHOLDERIALES, encompassing the acidovorans rRNA complex. Some species are pathogenic to PLANTS.'),
  'D042622': NtO(MH='DNA Repeat Expansion', MS='An increase number of repeats of a genomic, tandemly repeated DNA sequence from one generation to the next.'),
  'D042643': NtO(MH='Vascular Endothelial Growth Factor D', MS='A vascular endothelial growth factor that specifically binds to VASCULAR ENDOTHELIAL GROWTH FACTOR RECEPTOR-2 and VASCULAR ENDOTHELIAL GROWTH FACTOR RECEPTOR-3. In addition to being an angiogenic factor it can act on LYMPHATIC VESSELS to stimulate LYMPHANGIOGENESIS. It is similar in structure to VASCULAR ENDOTHELIAL GROWTH FACTOR C in that they both contain N- and C-terminal extensions that were not found in other VEGF family members.'),
  'D042662': NtO(MH='Vascular Endothelial Growth Factor, Endocrine-Gland-Derived', MS='A vascular endothelial growth factor whose expression is found largely restricted to the GONADS; ADRENAL CORTEX; and PLACENTA. It has similar biological activity to VASCULAR ENDOTHELIAL GROWTH FACTOR-A.'),
  'D042682': NtO(MH='Angiopoietins', MS='A family of structurally-related angiogenic proteins of approximately 70 kDa in size. They have high specificity for members of the TIE RECEPTOR FAMILY.'),
  'D042683': NtO(MH='Angiopoietin-1', MS='The first to be discovered member of the angiopoietin family. It may play a role in increasing the sprouting and branching of BLOOD VESSELS. Angiopoietin-1 specifically binds to and stimulates the TIE-2 RECEPTOR. Several isoforms of angiopoietin-1 occur due to ALTERNATIVE SPLICING of its mRNA.'),
  'D042702': NtO(MH='Angiopoietin-2', MS='An angiopoietin that is closely related to ANGIOPOIETIN-1. It binds to the TIE-2 RECEPTOR without receptor stimulation and antagonizes the effect of ANGIOPOIETIN-1. However its antagonistic effect may be limited to cell receptors that occur within the vasculature. Angiopoietin-2 may therefore play a role in down-regulation of BLOOD VESSEL branching and sprouting.'),
  'D042724': NtO(MH='Estrogens, Esterified (USP)', MS='A pharmaceutical preparation containing a mixture of esterified estrogens derived from estrogen sulfates, principally from ESTRONE sulfate. Esterified estrogen content should be 75-85% of the estrone sulfate and 6-15% of the EQUILIN sulfate.'),
  'D042725': NtO(MH='Burkholderia gladioli', MS='A species of gram-negative, aerobic bacteria that acts as both a human and plant pathogen.'),
  'D042726': NtO(MH='Burkholderia mallei', MS='A species of gram-negative bacteria parasitic on HORSES and DONKEYS causing GLANDERS, which can be transmitted to humans.'),
  'D042742': NtO(MH='Delftia', MS='A genus of gram-negative, strictly aerobic chemoorganotrophic bacteria, in the family COMAMONADACEAE.'),
  'D042743': NtO(MH='Leptothrix', MS='A genus of gram-negative, sheathed, rod-shaped bacteria in the order BURKHOLDERIALES.'),
  'D042744': NtO(MH='Sphaerotilus', MS='A genus of gram-negative, sheathed, rod-shaped bacteria in the family COMAMONADACEAE. They are closely related to LEPTOTHRIX.'),
  'D042762': NtO(MH='Methylophilaceae', MS='A family of gram-negative bacteria in the order Methylophilales.'),
  'D042763': NtO(MH='Acidithiobacillus', MS='A genus of gram-negative rod-shaped bacteria in the class GAMMAPROTEOBACTERIA. They are obligately acidophilic and aerobic, using reduced SULFUR COMPOUNDS to support AUTOTROPHIC GROWTH.'),
  'D042764': NtO(MH='Receptors, TIE', MS='A family of structurally-related tyrosine kinase receptors that are expressed predominantly in ENDOTHELIAL CELLS and are essential for development of BLOOD VESSELS (NEOVASCULARIZATION, PHYSIOLOGIC). The name derives from the fact that they are tyrosine kinases that contain Ig and EGF domains.'),
  'D042782': NtO(MH='Estrogenic Steroids, Alkylated', MS='Estrogenic STEROIDS with aliphatic hydrocarbon chain substitution on C17 or other position. 17-alpha-ALKYLATION renders the molecule more stable, resistant to metabolic degradation, and improves oral efficacy. Examples of synthetic alkyl estrogens include ETHINYL ESTRADIOL and MESTRANOL. Substitutions at other sites generates antiestrogenic and cytotoxic properties.'),
  'D042783': NtO(MH='Endothelial Cells', MS='Highly specialized EPITHELIAL CELLS that line the HEART; BLOOD VESSELS; and lymph vessels, forming the ENDOTHELIUM. They are polygonal in shape and joined together by TIGHT JUNCTIONS. The tight junctions allow for variable permeability to specific macromolecules that are transported across the endothelial layer.'),
  'D042787': NtO(MH='Receptor, TIE-2', MS='A TIE receptor tyrosine kinase that is found almost exclusively on ENDOTHELIAL CELLS. It is required for both normal embryonic vascular development (NEOVASCULARIZATION, PHYSIOLOGIC) and tumor angiogenesis (NEOVASCULARIZATION, PATHOLOGIC).'),
  'D042802': NtO(MH='Oxalobacteraceae', MS='A family of gram-negative bacteria in the order BURKHOLDERIALES, class BETAPROTEOBACTERIA.'),
  'D042803': NtO(MH='Herbaspirillum', MS="A genus of gram-negative bacteria in the family OXALOBACTERACEAE, which comprises vibrioid or sometimes helical cells. They are chemoorganotrophic nitrogen fixers and are found free-living in the soil or in association with the roots of members of the GRAMINEAE. (From Bergey's Manual of Determinative Bacteriology, 9th ed)"),
  'D042804': NtO(MH='Neisseria cinerea', MS='A species of gram-negative, aerobic bacteria found in the human NASOPHARYNX.'),
  'D042805': NtO(MH='Neisseria elongata', MS='A species of gram-negative, aerobic BACTERIA consisting of short chains of rods. It has been isolated from the PHARYNX of healthy individuals and patients with ENDOCARDITIS. There are several recognized subspecies.'),
  'D042806': NtO(MH='Neisseria lactamica', MS='A species of gram-negative, aerobic BACTERIA commonly found in the NASOPHARYNX of infants and children, but rarely pathogenic. It is the only species to produce acid from LACTOSE.'),
  'D042807': NtO(MH='Neisseria mucosa', MS="A species of gram-negative, aerobic BACTERIA, found in the human NASOPHARYNX and in the normal flora of the respiratory tissues in DOLPHINS. It is occasionally pathogenic for humans and pathogenic for MICE. (Bergey's Manual of Systemic Bacteriology, 1st edition, p295)"),
  'D042808': NtO(MH='Neisseria sicca', MS='A species of gram-negative, aerobic BACTERIA found in the human NASOPHARYNX; SALIVA; and SPUTUM.'),
  'D042810': NtO(MH='Receptor, TIE-1', MS='A TIE receptor found predominantly on ENDOTHELIAL CELLS. It is considered essential for vascular development and can form a heterodimer with the TIE-2 RECEPTOR. The TIE-1 receptor may play a role in regulating BLOOD VESSEL stability and maturation.'),
  'D042822': NtO(MH='Genomic Instability', MS='An increased tendency of the GENOME to acquire MUTATIONS when various processes involved in maintaining and replicating the genome are dysfunctional.'),
  'D042842': NtO(MH='11-beta-Hydroxysteroid Dehydrogenases', MS='Hydroxysteroid dehydrogenases that catalyzes the reversible conversion of CORTISOL to the inactive metabolite CORTISONE. Enzymes in this class can utilize either NAD or NADP as cofactors.'),
  'D042843': NtO(MH='Glucose 1-Dehydrogenase', MS='A glucose dehydrogenase that catalyzes the oxidation of beta-D-glucose to form D-glucono-1,5-lactone, using NAD as well as NADP as a coenzyme.'),
  'D042844': NtO(MH='L-Lactate Dehydrogenase (Cytochrome)', MS='A cytochrome form of lactate dehydrogenase found in the MITOCHONDRIA. It catalyzes the oxidation of L-lactate to PYRUVATE with transfer of electrons to CYTOCHROME C. The enzyme utilizes FMN and PROTOHEME IX as cofactors.'),
  'D042845': NtO(MH='Laccase', MS='A copper-containing oxidoreductase enzyme that catalyzes the oxidation of 4-benzenediol to 4-benzosemiquinone. It also has activity towards a variety of O-quinols and P-quinols. It primarily found in FUNGI and is involved in LIGNIN degradation, pigment biosynthesis and detoxification of lignin-derived products.'),
  'D042846': NtO(MH='CDC2-CDC28 Kinases', MS='A family of cell cycle-dependent kinases that are related in structure to CDC28 PROTEIN KINASE, S CEREVISIAE and the CDC2 PROTEIN KINASE found in mammalian species.'),
  'D042863': NtO(MH='Cyclin-Dependent Kinase 9', MS='A multifunctional CDC2 kinase-related kinase that plays roles in transcriptional elongation, CELL DIFFERENTIATION, and APOPTOSIS. It is found associated with CYCLIN T and is a component of POSITIVE TRANSCRIPTIONAL ELONGATION FACTOR B.'),
  'D042882': NtO(MH='Gallstones', MS='Solid crystalline precipitates in the BILIARY TRACT, usually formed in the GALLBLADDER, resulting in the condition of CHOLELITHIASIS. Gallstones, derived from the BILE, consist mainly of calcium, cholesterol, or bilirubin.'),
  'D042883': NtO(MH='Choledocholithiasis', MS='Presence or formation of GALLSTONES in the COMMON BILE DUCT.'),
  'D042925': NtO(MH='Positive Transcriptional Elongation Factor B', MS='A transcriptional elongation factor complex that is comprised of a heterodimer of CYCLIN-DEPENDENT KINASE 9 and one of several CYCLINS including TYPE T CYCLINS and cyclin K. It functions by phosphorylating the carboxy-terminal domain of RNA POLYMERASE II.'),
  'D042926': NtO(MH='Cytochrome P-450 CYP4A', MS='A P450 oxidoreductase that catalyzes the hydroxylation of the terminal carbon of linear hydrocarbons such as OCTANES and FATTY ACIDS in the omega position. The enzyme may also play a role in the oxidation of a variety of structurally unrelated compounds such as XENOBIOTICS, and STEROIDS.'),
  'D042931': NtO(MH='Aldehyde Oxidase', MS='An aldehyde oxidoreductase expressed predominantly in the LIVER; LUNGS; and KIDNEY. It catalyzes the oxidation of a variety of organic aldehydes and N-heterocyclic compounds to CARBOXYLIC ACIDS, and also oxidizes quinoline and pyridine derivatives. The enzyme utilizes molybdenum cofactor and FAD as cofactors.'),
  'D042942': NtO(MH='3-Methyl-2-Oxobutanoate Dehydrogenase (Lipoamide)', MS='A ketone oxidoreductase that catalyzes the overall conversion of alpha-keto acids to ACYL-CoA and CO2. The enzyme requires THIAMINE DIPHOSPHATE as a cofactor. Defects in genes that code for subunits of the enzyme are a cause of MAPLE SYRUP URINE DISEASE. The enzyme was formerly classified as EC 1.2.4.3.'),
  'D042943': NtO(MH='Dihydrouracil Dehydrogenase (NADP)', MS='An oxidoreductase involved in pyrimidine base degradation. It catalyzes the catabolism of THYMINE; URACIL and the chemotherapeutic drug, 5-FLUOROURACIL.'),
  'D042944': NtO(MH='Cholestenone 5 alpha-Reductase', MS='An oxidoreductase that catalyzes the conversion of 3-oxo-delta4 steroids into their corresponding 5alpha form. It plays an important role in the conversion of TESTOSTERONE into DIHYDROTESTOSTERONE and PROGESTERONE into DIHYDROPROGESTERONE.'),
  'D042962': NtO(MH='Acyl-CoA Oxidase', MS='An enzyme that catalyzes the first and rate-determining steps of peroxisomal beta-oxidation of fatty acids. It acts on COENZYME A derivatives of fatty acids with chain lengths from 8 to 18, using FLAVIN-ADENINE DINUCLEOTIDE as a cofactor.'),
  'D042963': NtO(MH='Electron Transport Complex II', MS='A flavoprotein oxidase complex that contains iron-sulfur centers. It catalyzes the oxidation of SUCCINATE to fumarate and couples the reaction to the reduction of UBIQUINONE to ubiquinol.'),
  'D042964': NtO(MH='Acyl-CoA Dehydrogenase', MS='A flavoprotein oxidoreductase that has specificity for medium-chain fatty acids. It forms a complex with ELECTRON TRANSFERRING FLAVOPROTEINS and conveys reducing equivalents to UBIQUINONE.'),
  'D042965': NtO(MH='Methylenetetrahydrofolate Reductase (NADPH2)', MS='A flavoprotein amine oxidoreductase that catalyzes the reversible conversion of 5-methyltetrahydrofolate to 5,10-methylenetetrahydrofolate. This enzyme was formerly classified as EC 1.1.1.171.'),
  'D042966': NtO(MH='Cytochrome-B(5) Reductase', MS='A FLAVOPROTEIN oxidoreductase that occurs both as a soluble enzyme and a membrane-bound enzyme due to ALTERNATIVE SPLICING of a single mRNA. The soluble form is present mainly in ERYTHROCYTES and is involved in the reduction of METHEMOGLOBIN. The membrane-bound form of the enzyme is found primarily in the ENDOPLASMIC RETICULUM and outer mitochondrial membrane, where it participates in the desaturation of FATTY ACIDS; CHOLESTEROL biosynthesis and drug metabolism. A deficiency in the enzyme can result in METHEMOGLOBINEMIA.'),
  'D042967': NtO(MH='Electron Transport Complex I', MS='A flavoprotein and iron sulfur-containing oxidoreductase complex that catalyzes the conversion of UBIQUINONE to ubiquinol. In MITOCHONDRIA the complex also couples its reaction to the transport of PROTONS across the internal mitochondrial membrane. The NADH DEHYDROGENASE component of the complex can be isolated and is listed as EC 1.6.99.3.'),
  'D043025': NtO(MH='Oligoclonal Bands', MS='Multiple protein bands serving as markers of specific ANTIBODIES and detected by ELECTROPHORESIS of CEREBROSPINAL FLUID or serum. The bands are most often seen during inflammatory or immune processes and are found in most patients with MULTIPLE SCLEROSIS.'),
  'D043143': NtO(MH='Hepatopancreas', MS='A primitive form of digestive gland found in marine ARTHROPODS, that contains cells similar to those found in the mammalian liver (HEPATOCYTES), and the PANCREAS.'),
  'D043165': NtO(MH='Angiostatic Proteins', MS='Proteins that specifically inhibit the growth of new blood vessels (ANGIOGENESIS, PHYSIOLOGIC).'),
  'D043167': NtO(MH='Spirillaceae', MS='A family of gram-negative bacteria in the order Nitrosomonadales. It formerly included the genera CAMPYLOBACTER and SPIRILLUM, then was abandoned, and is now reinstated as a family with a single genus SPIRILLUM.'),
  'D043168': NtO(MH='Angiostatins', MS='Circulating 38-kDa proteins that are internal peptide fragments of PLASMINOGEN. The name derives from the fact that they are potent ANGIOGENESIS INHIBITORS. Angiostatins contain four KRINGLE DOMAINS which are associated with their potent angiostatic activity.'),
  'D043169': NtO(MH='Endostatins', MS='Angiostatic proteins that are formed from proteolytic cleavage of COLLAGEN TYPE XVIII.'),
  'D043170': NtO(MH='Collagen Type XVIII', MS='A non-fibrillar collagen found in BASEMENT MEMBRANE. The C-terminal end of the alpha1 chain of collagen type XVIII contains the ENDOSTATIN peptide, which can be released by proteolytic cleavage.'),
  'D043171': NtO(MH='Chromosomal Instability', MS='An increased tendency to acquire CHROMOSOME ABERRATIONS when various processes involved in chromosome replication, repair, or segregation are dysfunctional.'),
  'D043182': NtO(MH='Carboxylesterase', MS='Carboxylesterase is a serine-dependent esterase with wide substrate specificity. The enzyme is involved in the detoxification of XENOBIOTICS and the activation of ester and of amide PRODRUGS.'),
  'D043183': NtO(MH='Irritable Bowel Syndrome', MS='A disorder with chronic or recurrent colonic symptoms without a clearcut etiology. This condition is characterized by chronic or recurrent ABDOMINAL PAIN, bloating, MUCUS in FECES, and an erratic disturbance of DEFECATION.'),
  'D043202': NtO(MH='Steroid Metabolism, Inborn Errors', MS='Errors in metabolic processing of STEROIDS resulting from inborn genetic mutations that are inherited or acquired in utero.'),
  'D043203': NtO(MH='1-Alkyl-2-acetylglycerophosphocholine Esterase', MS='A lipoprotein-associated PHOSPHOLIPASE A2 which modulates the action of PLATELET ACTIVATING FACTOR by hydrolyzing the SN-2 ester bond to yield the biologically inactive lyso-platelet-activating factor. It has specificity for phospholipid substrates with short-chain residues at the SN-2 position, but inactive against long-chain phospholipids. Deficiency in this enzyme is associated with many diseases including ASTHMA, and HYPERCHOLESTEROLEMIA.'),
  'D043204': NtO(MH='Mineralocorticoid Excess Syndrome, Apparent', MS='A hereditary disease characterized by childhood onset HYPERTENSION, hypokalemic alkalosis, and low RENIN and ALDOSTERONE secretion. It results from a defect in the activity of the 11-BETA-HYDROXYSTEROID DEHYDROGENASE TYPE 2 enzyme which results in inadequate conversion of CORTISOL to CORTISONE. The build up of unprocessed cortisol to levels that stimulate MINERALOCORTICOID RECEPTORS creates the appearance of having excessive MINERALOCORTICOIDS.'),
  'D043205': NtO(MH='11-beta-Hydroxysteroid Dehydrogenase Type 1', MS='A low-affinity 11 beta-hydroxysteroid dehydrogenase found in a variety of tissues, most notably in LIVER; LUNG; ADIPOSE TISSUE; vascular tissue; OVARY; and the CENTRAL NERVOUS SYSTEM. The enzyme acts reversibly and can use either NAD or NADP as cofactors.'),
  'D043209': NtO(MH='11-beta-Hydroxysteroid Dehydrogenase Type 2', MS='An high-affinity, NAD-dependent 11-beta-hydroxysteroid dehydrogenase that acts unidirectionally to catalyze the dehydrogenation of CORTISOL to CORTISONE. It is found predominantly in mineralocorticoid target tissues such as the KIDNEY; COLON; SWEAT GLANDS; and the PLACENTA. Absence of the enzyme leads to a fatal form of childhood hypertension termed, APPARENT MINERALOCORTICOID EXCESS SYNDROME.'),
  'D043211': NtO(MH='Exodeoxyribonuclease V', MS="An ATP-dependent exodeoxyribonuclease that cleaves in either the 5'- to 3'- or the 3'- to 5'-direction to yield 5'-phosphooligonucleotides. It is primarily found in BACTERIA."),
  'D043222': NtO(MH='Ubiquitin Thiolesterase', MS='A thioester hydrolase which acts on esters formed between thiols such as DITHIOTHREITOL or GLUTATHIONE and the C-terminal glycine residue of UBIQUITIN.'),
  'D043223': NtO(MH='Deoxyribonuclease IV (Phage T4-Induced)', MS="An enzyme which catalyzes the endonucleolytic cleavage of phosphodiester bonds at purinic or apyrimidinic sites (AP-sites) to produce 5'-Phosphooligonucleotide end products. The enzyme prefers single-stranded DNA (ssDNA) and was formerly classified as EC 3.1.4.30."),
  'D043224': NtO(MH='Deoxyribonuclease (Pyrimidine Dimer)', MS="An enzyme which catalyzes an endonucleolytic cleavage near PYRIMIDINE DIMERS to produce a 5'-phosphate product. The enzyme acts on the damaged DNA strand, from the 5' side of the damaged site."),
  'D043243': NtO(MH='DNA Sequence, Unstable', MS='A region of DNA that is highly polymorphic and is prone to strand breaks, rearrangements or other MUTATIONS because of the nature of its sequence. These regions often harbor palindromic, or repetitive sequences (REPETITIVE SEQUENCES, NUCLEIC ACID). Variability in stability of the DNA sequence is seen at CHROMOSOME FRAGILE SITES.'),
  'D043244': NtO(MH='Ribonuclease III', MS='An endoribonuclease that is specific for double-stranded RNA. It plays a role in POST-TRANSCRIPTIONAL RNA PROCESSING of pre-RIBOSOMAL RNA and a variety of other RNA structures that contain double-stranded regions.'),
  'D043262': NtO(MH='Ribonuclease P', MS="An RNA-containing enzyme that plays an essential role in tRNA processing by catalyzing the endonucleolytic cleavage of TRANSFER RNA precursors. It removes the extra 5'-nucleotides from tRNA precursors to generate mature tRNA molecules."),
  'D043263': NtO(MH='Myosin-Light-Chain Phosphatase', MS='A phosphoprotein phosphatase that is specific for MYOSIN LIGHT CHAINS. It is composed of three subunits, which include a catalytic subunit, a myosin binding subunit, and a third subunit of unknown function.'),
  'D043264': NtO(MH='Phosphodiesterase I', MS="A phosphoric diester hydrolase that removes 5'-nucleotides from the 3'-hydroxy termini of 3'-hydroxy-terminated OLIGONUCLEOTIDES. It has low activity towards POLYNUCLEOTIDES and the presence of 3'-phosphate terminus on the substrate may inhibit hydrolysis."),
  'D043265': NtO(MH='Phosphatidylinositol Diacylglycerol-Lyase', MS='A phosphorus-oxygen lyase found primarily in BACTERIA. The enzyme catalyzes the cleavage of a phosphoester linkage in 1-phosphatidyl-1D-myo-inositol to form 1D-myo-inositol 1,2-cyclic phosphate and diacylglycerol. The enzyme was formerly classified as a phosphoric diester hydrolase (EC 3.1.4.10) and is often referred to as a TYPE C PHOSPHOLIPASES. However it is now known that a cyclic phosphate is the final product of this enzyme and that water does not enter into the reaction.'),
  'D043266': NtO(MH='Steryl-Sulfatase', MS='An arylsulfatase with high specificity towards sulfated steroids. Defects in this enzyme are the cause of ICHTHYOSIS, X-LINKED.'),
  'D043283': NtO(MH='Chromosome Fragile Sites', MS='Specific loci that show up during KARYOTYPING as a gap (an uncondensed stretch in closer views) on a CHROMATID arm after culturing cells under specific conditions. These sites are associated with an increase in CHROMOSOME FRAGILITY. They are classified as common or rare, and by the specific culture conditions under which they develop. Fragile site loci are named by the letters "FRA" followed by a designation for the specific chromosome, and a letter which refers to which fragile site of that chromosome (e.g. FRAXA refers to fragile site A on the X chromosome. It is a rare, folic acid-sensitive fragile site associated with FRAGILE X SYNDROME.)'),
  'D043302': NtO(MH='Cultured Milk Products', MS='Derivatives of milk that are produced by controlled FERMENTATION.'),
  'D043303': NtO(MH='Aryldialkylphosphatase', MS='An enzyme which catalyzes the hydrolysis of an aryl-dialkyl phosphate to form dialkyl phosphate and an aryl alcohol. It can hydrolyze a broad spectrum of organophosphate substrates and a number of aromatic carboxylic acid esters. It may also mediate an enzymatic protection of LOW DENSITY LIPOPROTEINS against oxidative modification and the consequent series of events leading to ATHEROMA formation. The enzyme was previously regarded to be identical with Arylesterase (EC 3.1.1.2).'),
  'D043322': NtO(MH='Lactase', MS='An enzyme which catalyzes the hydrolysis of LACTOSE to D-GALACTOSE and D-GLUCOSE. Defects in the enzyme cause LACTOSE INTOLERANCE.'),
  'D043323': NtO(MH='alpha-Mannosidase', MS='An enzyme that catalyzes the HYDROLYSIS of terminal, non-reducing alpha-D-mannose residues in alpha-D-mannosides. The enzyme plays a role in the processing of newly formed N-glycans and in degradation of mature GLYCOPROTEINS. There are multiple isoforms of alpha-mannosidase, each having its own specific cellular location and pH optimum. Defects in the lysosomal form of the enzyme results in a buildup of mannoside intermediate metabolites and the disease ALPHA-MANNOSIDOSIS.'),
  'D043324': NtO(MH='beta-Fructofuranosidase', MS='A glycoside hydrolase found primarily in PLANTS and YEASTS. It has specificity for beta-D-fructofuranosides such as SUCROSE.'),
  'D043325': NtO(MH='Xylan Endo-1,3-beta-Xylosidase', MS='A xylosidase that catalyses the random hydrolysis of 1,3-beta-D-xylosidic linkages in 1,3-beta-D-xylans.'),
  'D043326': NtO(MH='Glucan 1,3-beta-Glucosidase', MS='An exocellulase with specificity for 1,3-beta-D-glucasidic linkages. It catalyzes hydrolysis of beta-D-glucose units from the non-reducing ends of 1,3-beta-D-glucans, releasing GLUCOSE.'),
  'D043343': NtO(MH='Testosterone Propionate', MS='An ester of TESTOSTERONE with a propionate substitution at the 17-beta position.'),
  'D043344': NtO(MH='Chordata', MS='Phylum in the domain Eukarya, comprised of animals either with fully developed backbones (VERTEBRATES), or those with notochords only during some developmental stage (CHORDATA, NONVERTEBRATE).'),
  'D043362': NtO(MH='Rhodocyclaceae', MS='A family of gram-negative bacteria in the order Rhodocyclales, class BETAPROTEOBACTERIA. It includes many genera previously assigned to the family PSEUDOMONADACEAE.'),
  'D043363': NtO(MH='Hydrogenophilaceae', MS='A family of gram-negative bacteria in the order Hydrogenophilales, class BETAPROTEOBACTERIA.'),
  'D043364': NtO(MH='Endo-1,4-beta Xylanases', MS='Enzymes which catalyze the endohydrolysis of 1,4-beta-D-xylosidic linkages in XYLANS.'),
  'D043366': NtO(MH='Cellulose 1,4-beta-Cellobiosidase', MS='An exocellulase with specificity for the hydrolysis of 1,4-beta-D-glucosidic linkages in CELLULOSE and cellotetraose. It catalyzes the hydrolysis of terminal non-reducing ends of beta-D-glucosides with release of CELLOBIOSE.'),
  'D043367': NtO(MH='Ralstonia', MS='A genus in the family BURKHOLDERIACEAE, comprised of many species. They are associated with a variety of infections including MENINGITIS; PERITONITIS; and URINARY TRACT INFECTIONS.'),
  'D043368': NtO(MH='Ralstonia solanacearum', MS='A species of Ralstonia previously classed in the genera PSEUDOMONAS and BURKHOLDERIA. It is an important plant pathogen.'),
  'D043369': NtO(MH='Nitrosomonadaceae', MS='A family of gram-negative nitrifying bacteria, in the order Nitrosomonadales, class BETAPROTEOBACTERIA.'),
  'D043370': NtO(MH='Nitrosomonas europaea', MS='The type species of the genus NITROSOMONAS, a gram-negative chemolithotroph that oxidizes ammonia to nitrite. It is found in soil, sewage, freshwater, and on building walls, and especially in polluted areas where air contains high levels of nitrogen compounds.'),
  'D043371': NtO(MH='Fatty Acids, Omega-6', MS='FATTY ACIDS which have the first unsaturated bond in the sixth position from the omega carbon. A typical American diet tends to contain substantially more omega-6 than OMEGA-3 FATTY ACIDS.'),
  'D043372': NtO(MH='Gallionellaceae', MS='A family of gram-negative bacteria in the order Nitrosomonadales, class BETAPROTEOBACTERIA. It contains a single genus Gallionella.'),
  'D043373': NtO(MH='Follicle Stimulating Hormone, Human', MS='A major gonadotropin secreted by the human adenohypophysis (PITUITARY GLAND, ANTERIOR). Follicle-stimulating hormone stimulates GAMETOGENESIS and the supporting cells such as the ovarian GRANULOSA CELLS, the testicular SERTOLI CELLS, and the LEYDIG CELLS. FSH consists of two noncovalently linked subunits, alpha and beta. The alpha subunit is common in the three human pituitary glycoprotein hormones (TSH, LH, and FSH), but the beta subunit is unique and confers its biological specificity.'),
  'D043382': NtO(MH='DNA-Formamidopyrimidine Glycosylase', MS='A DNA repair enzyme that is an N-glycosyl hydrolase with specificity for DNA-containing ring-opened N(7)-methylguanine residues.'),
  'D043383': NtO(MH='Adenosylhomocysteinase', MS='An enzyme which catalyzes the catabolism of S-ADENOSYLHOMOCYSTEINE to ADENOSINE and HOMOCYSTEINE. It may play a role in regulating the concentration of intracellular adenosylhomocysteine.'),
  'D043384': NtO(MH='Glutamyl Aminopeptidase', MS='A ZINC-dependent membrane-bound aminopeptidase that catalyzes the N-terminal peptide cleavage of GLUTAMATE (and to a lesser extent ASPARTATE). The enzyme appears to play a role in the catabolic pathway of the RENIN-ANGIOTENSIN SYSTEM.'),
  'D043402': NtO(MH='Cathepsin A', MS='A carboxypeptidase that catalyzes the release of a C-terminal amino acid with a broad specificity. It also plays a role in the LYSOSOMES by protecting BETA-GALACTOSIDASE and NEURAMINIDASE from degradation. It was formerly classified as EC 3.4.12.1 and EC 3.4.21.13.'),
  'D043422': NtO(MH='Carboxypeptidases A', MS='Carboxypeptidases that are primarily found the DIGESTIVE SYSTEM that catalyze the release of C-terminal amino acids. Carboxypeptidases A have little or no activity for hydrolysis of C-terminal ASPARTIC ACID; GLUTAMIC ACID; ARGININE; LYSINE; or PROLINE. This enzyme requires ZINC as a cofactor and was formerly listed as EC 3.4.2.1 and EC 3.4.12.2.'),
  'D043423': NtO(MH='Carboxypeptidase H', MS='A ZINC-containing exopeptidase primarily found in SECRETORY VESICLES of endocrine and neuroendocrine cells. It catalyzes the cleavage of C-terminal ARGININE or LYSINE residues from polypeptides and is active in processing precursors of PEPTIDE HORMONES and other bioactive peptides.'),
  'D043424': NtO(MH='Carboxypeptidase B', MS='A ZINC-dependent carboxypeptidase primary found in the DIGESTIVE SYSTEM. The enzyme catalyzes the preferential cleavage of a C-terminal peptidyl-L-lysine or arginine. It was formerly classified as EC 3.4.2.2 and EC 3.4.12.3.'),
  'D043425': NtO(MH='Glutamate Carboxypeptidase II', MS='A metallocarboxypeptidase that is predominantly expressed as a membrane-bound enzyme. It catalyzes the hydrolysis of an unsubstituted, C-terminal glutamyl residue, typically from PTEROYLPOLYGLUTAMIC ACIDS. It was formerly classified as EC 3.4.19.8.'),
  'D043484': NtO(MH='Proprotein Convertases', MS='Proteolytic enzymes that are involved in the conversion of protein precursors such as peptide prohormones into PEPTIDE HORMONES. Some are ENDOPEPTIDASES, some are EXOPEPTIDASES.'),
  'D043485': NtO(MH='Glomus Tympanicum', MS='A highly vascular ovoid body of chemoreceptive tissue lying adjacent to the TYMPANIC CAVITY. It is derived from NEURAL CREST tissue and is considered part of the diffuse neuroendocrine system. It is the site of a rare neoplasm called a GLOMUS TYMPANICUM TUMOR.'),
  'D043523': NtO(MH='Biotinidase', MS='An enzyme which catalyzes the release of BIOTIN from biocytin. In human, defects in the enzyme are the cause of the organic acidemia MULTIPLE CARBOXYLASE DEFICIENCY or BIOTINIDASE DEFICIENCY.'),
  'D043524': NtO(MH='Peptide-N4-(N-acetyl-beta-glucosaminyl) Asparagine Amidase', MS='An amidohydrolase that removes intact asparagine-linked oligosaccharide chains from glycoproteins. It requires the presence of more than two amino-acid residues in the substrate for activity. This enzyme was previously listed as EC 3.2.2.18.'),
  'D043525': NtO(MH='Cytosine Deaminase', MS='An enzyme which catalyzes the deamination of CYTOSINE resulting in the formation of URACIL. It can also act on 5-methylcytosine to form THYMIDINE.'),
  'D043542': NtO(MH='DNA, Z-Form', MS='A left-handed double helix of DNA. Its name derives from its narrow zigzag structure that is the least twisted and thinnest form of DNA. Z-DNA forming regions within the GENOME may play an important role in GENE EXPRESSION REGULATION.'),
  'D043562': NtO(MH='Receptors, G-Protein-Coupled', MS='The largest family of cell surface receptors involved in SIGNAL TRANSDUCTION. They share a common structure and signal through HETEROTRIMERIC G-PROTEINS.'),
  'D043563': NtO(MH='Receptors, Pheromone', MS='Cell surface receptors that respond to PHEROMONES.'),
  'D043564': NtO(MH='Inorganic Pyrophosphatase', MS='An enzyme which catalyzes the hydrolysis of diphosphate (DIPHOSPHATES) into inorganic phosphate. The hydrolysis of pyrophosphate is coupled to the transport of HYDROGEN IONS across a membrane.'),
  'D043582': NtO(MH='5-alpha-Dihydroprogesterone', MS='A biologically active 5-alpha-reduced metabolite of plasma PROGESTERONE. It is the immediate precursor of 5-alpha-pregnan-3-alpha-ol-20-one (ALLOPREGNANOLONE), a neuroactive steroid that binds with GABA(A) RECEPTOR.'),
  'D043583': NtO(MH='Nucleoside-Triphosphatase', MS='An enzyme which catalyzes the hydrolysis of nucleoside triphosphates to nucleoside diphosphates. It may also catalyze the hydrolysis of nucleotide triphosphates, diphosphates, thiamine diphosphates and FAD. The nucleoside triphosphate phosphohydrolases I and II are subtypes of the enzyme which are found mostly in viruses.'),
  'D043585': NtO(MH='Receptor, Muscarinic M2', MS='A specific subtype of muscarinic receptor found in the lower BRAIN, the HEART and in SMOOTH MUSCLE-containing organs. Although present in smooth muscle the M2 muscarinic receptor appears not to be involved in contractile responses.'),
  'D043586': NtO(MH='Methylmalonyl-CoA Decarboxylase', MS='A carboxy-lyase that catalyzes the decarboxylation of (S)-2-Methyl-3-oxopropanoyl-CoA to propanoyl-CoA. In microorganisms the reaction can be coupled to the vectorial transport of SODIUM ions across the cytoplasmic membrane.'),
  'D043587': NtO(MH='Receptor, Muscarinic M3', MS='A subclass of muscarinic receptor that mediates cholinergic-induced contraction in a variety of SMOOTH MUSCLES.'),
  'D043588': NtO(MH='Receptor, Muscarinic M4', MS='A specific subtype of muscarinic receptor found in the CORPUS STRIATUM and the LUNG. It has similar receptor binding specificities to MUSCARINIC RECEPTOR M1 and MUSCARINIC RECEPTOR M2.'),
  'D043603': NtO(MH='DNA-(Apurinic or Apyrimidinic Site) Lyase', MS="A DNA repair enzyme that catalyses the excision of ribose residues at apurinic and apyrimidinic DNA sites that can result from the action of DNA GLYCOSYLASES. The enzyme catalyzes a beta-elimination reaction in which the C-O-P bond 3' to the apurinic or apyrimidinic site in DNA is broken, leaving a 3'-terminal unsaturated sugar and a product with a terminal 5'-phosphate. This enzyme was previously listed under EC 3.1.25.2."),
  'D043604': NtO(MH='Glomus Tympanicum Tumor', MS='A rare PARAGANGLIOMA involving the GLOMUS TYMPANICUM, a collection of chemoreceptor tissue adjacent to the TYMPANIC CAVITY. It can cause TINNITUS and conductive hearing loss (HEARING LOSS, CONDUCTIVE).'),
  'D043607': NtO(MH='20-alpha-Hydroxysteroid Dehydrogenase', MS='An enzymes that catalyzes the reversible reduction-oxidation reaction of 20-alpha-hydroxysteroids, such as from PROGESTERONE to 20-ALPHA-DIHYDROPROGESTERONE.'),
  'D043642': NtO(MH='Receptor, Muscarinic M5', MS='A specific subtype of muscarinic receptor found in a variety of locations including the SALIVARY GLANDS and the SUBSTANTIA NIGRA and VENTRAL TEGMENTAL AREA of the BRAIN.'),
  'D043643': NtO(MH='Receptor, Muscarinic M1', MS='A specific subtype of muscarinic receptor that has a high affinity for the drug PIRENZEPINE. It is found in the peripheral GANGLIA where it signals a variety of physiological functions such as GASTRIC ACID secretion and BRONCHOCONSTRICTION. This subtype of muscarinic receptor is also found in neuronal tissues including the CEREBRAL CORTEX and HIPPOCAMPUS where it mediates the process of MEMORY and LEARNING.'),
  'D043682': NtO(MH='Receptor, Adenosine A1', MS='A subtype of ADENOSINE RECEPTOR that is found expressed in a variety of tissues including the BRAIN and DORSAL HORN NEURONS. The receptor is generally considered to be coupled to the GI, INHIBITORY G-PROTEIN which causes down regulation of CYCLIC AMP.'),
  'D043683': NtO(MH='Receptors, Adenosine A2', MS='A subclass of ADENOSINE RECEPTORS that are generally considered to be coupled to the GS, STIMULATORY G-PROTEIN which causes up regulation of CYCLIC AMP.'),
  'D043684': NtO(MH='Receptor, Adenosine A3', MS='A subtype of ADENOSINE RECEPTOR that is found expressed in a variety of locations including the BRAIN and endocrine tissues. The receptor is generally considered to be coupled to the GI, INHIBITORY G-PROTEIN which causes down regulation of CYCLIC AMP.'),
  'D043704': NtO(MH='Receptor, Adenosine A2B', MS='A subclass of adenosine A2 receptors found in the CECUM, the COLON, the BLADDER, and a variety of other tissues. It is generally considered to be a low affinity receptor for ADENOSINE that couples to the GS, STIMULATORY G-PROTEIN.'),
  'D043705': NtO(MH='Receptor, Adenosine A2A', MS='A subclass of adenosine A2 receptors found in LEUKOCYTES, the SPLEEN, the THYMUS and a variety of other tissues. It is generally considered to be a receptor for ADENOSINE that couples to the GS, STIMULATORY G-PROTEIN.'),
  'D043743': NtO(MH='Ubiquitin-Protein Ligase Complexes', MS='Complexes of enzymes that catalyze the covalent attachment of UBIQUITIN to other proteins by forming a peptide bond between the C-terminal GLYCINE of UBIQUITIN and the alpha-amino groups of LYSINE residues in the protein. The complexes play an important role in mediating the selective-degradation of short-lived and abnormal proteins. The complex of enzymes can be broken down into three components that involve activation of ubiquitin (UBIQUITIN-ACTIVATING ENZYMES), conjugation of ubiquitin to the ligase complex (UBIQUITIN-CONJUGATING ENZYMES), and ligation of ubiquitin to the substrate protein (UBIQUITIN-PROTEIN LIGASES).'),
  'D043762': NtO(MH='Reproductive Behavior', MS='Human behavior or decision related to REPRODUCTION.'),
  'D043782': NtO(MH='Receptor, Bradykinin B2', MS='A constitutively expressed subtype of bradykinin receptor that may play a role in the acute phase of the inflammatory and pain response. It has high specificity for intact forms of BRADYKININ and KALLIDIN. The receptor is coupled to G-PROTEIN, GQ-G11 ALPHA FAMILY and G-PROTEIN, GI-GO ALPHA FAMILY signaling proteins.'),
  'D043783': NtO(MH='Receptor, Bradykinin B1', MS='A subtype of BRADYKININ RECEPTOR that is induced in response to INFLAMMATION. It may play a role in chronic inflammation and has a high specificity for KININS lacking the C-terminal ARGININE such as des-Arg(10)-kallidin and des-Arg(9)-bradykinin. The receptor is coupled to G-PROTEIN, GQ-G11 ALPHA FAMILY and G-PROTEIN, GI-GO ALPHA FAMILY signaling proteins.'),
  'D043802': NtO(MH='GTP-Binding Protein alpha Subunits, Gq-G11', MS='A family of heterotrimeric GTP-binding protein alpha subunits that activate TYPE C PHOSPHOLIPASES dependent signaling pathways. The Gq-G11 part of the name is also spelled Gq/G11.'),
  'D043803': NtO(MH='Phenalenes', MS='A group of AROMATIC HYDROCARBONS that have three rings joined as a triad around a single carbon atom so all three are conjoined, in contrast to a linear arrangement (ANTHRACENES) or angular arrangement (PHENANTHRENES).'),
  'D043822': NtO(MH='Cyclodecanes', MS='Compounds with a 10-carbon ring.'),
  'D043823': NtO(MH='Taxoids', MS='A group of diterpenoid CYCLODECANES named for the taxanes that were discovered in the TAXUS tree. The action on MICROTUBULES has made some of them useful as ANTINEOPLASTIC AGENTS.'),
  'D043842': NtO(MH='Ochnaceae', MS='A plant family of the order THEALES, subclass Dilleniidae, class Magnoliopsida that have evergreen, alternate leaves.'),
  'D043843': NtO(MH='Leuzea', MS='A plant genus of the family ASTERACEAE. ECDYSONE has been found in seeds of some members.'),
  'D043844': NtO(MH='Crown Ethers', MS='Macrocyclic polyethers with the repeating unit of (-CH2-CH2-O)n where n is greater than 2 and some oxygens may be replaced by nitrogen, sulfur or phosphorus. These compounds are useful for coordinating CATIONS. The nomenclature uses a prefix to indicate the size of the ring and a suffix for the number of heteroatoms.'),
  'D043862': NtO(MH='Rotaxanes', MS='Complex compounds in which a dumbbell shaped molecule is encircled by a macrocycle. They are named after rota (wheel) and axis (axle). Notation with a prefix is used to indicate the number of interlocked components. They have potential use in NANOTECHNOLOGY. Rotaxanes have been made with CYCLODEXTRINS and CYCLIC ETHERS.'),
  'D043863': NtO(MH='Catenanes', MS='Complex compounds where two cyclic molecules are interlaced together as links in a chain. They have potential use in NANOTECHNOLOGY.'),
  'D043864': NtO(MH='DNA, Catenated', MS='CIRCULAR DNA that is interlaced together as links in a chain. It is used as an assay for the activity of DNA TOPOISOMERASES. Catenated DNA is attached loop to loop in contrast to CONCATENATED DNA which is attached end to end.'),
  'D043882': NtO(MH='Receptors, Cannabinoid', MS='A class of G-protein-coupled receptors that are specific for CANNABINOIDS such as those derived from CANNABIS. They also bind a structurally distinct class of endogenous factors referred to as ENDOCANNABINOIDS. The receptor class may play a role in modulating the release of signaling molecules such as NEUROTRANSMITTERS and CYTOKINES.'),
  'D043884': NtO(MH='Receptor, Cannabinoid, CB1', MS='A subclass of cannabinoid receptor found primarily on central and peripheral NEURONS where it may play a role modulating NEUROTRANSMITTER release.'),
  'D043885': NtO(MH='Receptor, Cannabinoid, CB2', MS='A subclass of cannabinoid receptor found primarily on immune cells where it may play a role modulating release of CYTOKINES.'),
  'D043886': NtO(MH='Receptor, Cholecystokinin A', MS='A subtype of cholecystokinin receptor found primarily in the PANCREAS; STOMACH; INTESTINE; and GALLBLADDER. It plays a role in regulating digestive functions such as gallbladder contraction, pancreatic enzyme secretion and absorption in the GASTROINTESTINAL TRACT.'),
  'D043887': NtO(MH='Receptor, Cholecystokinin B', MS='A subtype of cholecystokinin receptor found primarily in the CENTRAL NERVOUS SYSTEM and the GASTRIC MUCOSA. It may play a role as a neuromodulator of dopaminergic neurotransmission the regulation of GASTRIC ACID secretion from GASTRIC PARIETAL CELLS.'),
  'D043902': NtO(MH='Pyrococcus horikoshii', MS='Anaerobic hyperthermophilic species of ARCHAEA, isolated from hydrothermal fluid samples. It is obligately heterotrophic with coccoid cells that require TRYPTOPHAN for growth.'),
  'D043903': NtO(MH='Pyrococcus abyssi', MS='A species of gram-negative hyperthermophilic ARCHAEA found in deep ocean hydrothermal vents. It is an obligate anaerobe and obligate chemoorganotroph.'),
  'D043922': NtO(MH='Toxicogenetics', MS='The study of existing genetic knowledge, and the generation of new genetic data, to understand and thus avoid DRUG TOXICITY and adverse effects from toxic substances from the environment.'),
  'D043924': NtO(MH='Angiogenesis Modulating Agents', MS='Agents that modulate the PHYSIOLOGIC ANGIOGENESIS process. This is accomplished by endogenous ANGIOGENIC PROTEINS and a variety of other chemicals and pharmaceutical agents.'),
  'D043925': NtO(MH='Angiogenesis Inducing Agents', MS='Agents that induce or stimulate PHYSIOLOGIC ANGIOGENESIS or PATHOLOGIC ANGIOGENESIS.'),
  'D043942': NtO(MH='Nanotubes', MS='Nanometer-sized tubes composed of various substances including carbon (CARBON NANOTUBES), boron nitride, or nickel vanadate.'),
  'D043963': NtO(MH='Diverticulosis, Colonic', MS='A pathological condition characterized by the presence of a number of COLONIC DIVERTICULA in the COLON. Its pathogenesis is multifactorial, including colon aging, motor dysfunction, increases in intraluminal pressure, and lack of dietary fibers.'),
  'D043964': NtO(MH='Marsileaceae', MS='A plant family of the order Hydropteridales, class Filicopsida, division PTERIDOPHYTA. They are aquatic ferns with quatrifoliate leaves resembling four leaf clover, creeping rhizome, and bean shaped sporocarps.'),
  'D043982': NtO(MH='Rhizophoraceae', MS='A plant family of the order Rhizophorales, subclass Rosidae, class Magnoliopsida, that includes mangrove trees.'),
  'D044002': NtO(MH='Bryophyta', MS='A division of the plant kingdom. Bryophyta contains the subdivision, Musci, which contains the classes: Andreaeopsida, BRYOPSIDA, and SPHAGNOPSIDA.'),
  'D044003': NtO(MH='Sphagnopsida', MS='A class of BRYOPHYTA which is best known for Sphagnum forming PEAT bogs.'),
  'D044004': NtO(MH='Xanthones', MS='A group of XANTHENES that contain a 9-keto OXYGEN.'),
  'D044005': NtO(MH='Tetrahydroisoquinolines', MS='A group of ISOQUINOLINES in which the nitrogen containing ring is protonated. They derive from the non-enzymatic Pictet-Spengler condensation of CATECHOLAMINES with ALDEHYDES.'),
  'D044006': NtO(MH='Receptors, Epoprostenol', MS='Cell surface receptors for EPOPROSTENOL. They are coupled to HETEROTRIMERIC G-PROTEINS.'),
  'D044007': NtO(MH='Moraxellaceae', MS='A family of gram-negative, aerobic bacteria in the order Pseudomonadales. Some strains are parasites of the mucosal membranes of animals and humans; others are found in association with food products or in the environment.'),
  'D044022': NtO(MH='Receptor, Endothelin A', MS='A subtype of endothelin receptor found predominantly in the VASCULAR SMOOTH MUSCLE. It has a high affinity for ENDOTHELIN-1 and ENDOTHELIN-2.'),
  'D044023': NtO(MH='Receptor, Endothelin B', MS='A subtype of endothelin receptor found predominantly in the KIDNEY. It may play a role in reducing systemic ENDOTHELIN levels.'),
  'D044024': NtO(MH='Tetralones', MS='A group of TETRAHYDRONAPHTHALENES containing a keto oxygen.'),
  'D044042': NtO(MH='Receptors, Formyl Peptide', MS='A family of G-protein-coupled receptors that was originally identified by its ability to bind N-formyl peptides such as N-FORMYLMETHIONINE LEUCYL-PHENYLALANINE. Since N-formyl peptides are found in MITOCHONDRIA and BACTERIA, this class of receptors is believed to play a role in mediating cellular responses to cellular damage and bacterial invasion. However, non-formylated peptide ligands have also been found for this receptor class.'),
  'D044043': NtO(MH='Pectobacterium', MS='A genus of gram-negative bacteria in the family ENTEROBACTERIACEAE consisting of species that profusely produce pectinolytic enzymes in plant pathogenesis.'),
  'D044044': NtO(MH='Aeromonadaceae', MS='A family of gram-negative facultatively anaerobic bacteria, ubiquitous in fresh and brackish water, and associated with GASTROINTESTINAL DISEASES.'),
  'D044045': NtO(MH='Lipoxins', MS='Trihydroxy derivatives of eicosanoic acids. They are primarily derived from arachidonic acid, however eicosapentaenoic acid derivatives also exist. Many of them are naturally occurring mediators of immune regulation.'),
  'D044062': NtO(MH='Prostaglandins I', MS='A class of cyclic prostaglandins that contain the 6,9-epoxy bond. Endogenous members of this family are biosynthesized enzymatically from PROSTAGLANDIN ENDOPEROXIDES.'),
  'D044063': NtO(MH='Succinivibrionaceae', MS='A family of gram-negative, rod-shaped bacteria isolated from the RUMEN of SHEEP and CATTLE and from human FECES. They are chemoorganotrophic and strictly anaerobic.'),
  'D044064': NtO(MH='Anaerobiospirillum', MS='A genus of gram-negative, spiral-shaped bacteria in the family SUCCINIVIBRIONACEAE. It has been isolated from the THROAT and COLON of beagle dogs and from human FECES.'),
  'D044065': NtO(MH='Cardiobacteriaceae', MS='A family of gram-negative, mostly aerobic bacteria, in the order Cardiobacteriales. There are three genera: CARDIOBACTERIUM; DICHELOBACTER; and Suttonella.'),
  'D044066': NtO(MH='Cardiobacterium', MS='A genus of gram-negative, facultatively anaerobic bacteria in the family CARDIOBACTERIACEAE. It is found in the nasal flora of humans and causes ENDOCARDITIS.'),
  'D044067': NtO(MH='Receptors, Lipoxin', MS='Cell surface proteins that bind LIPOXINS with high affinity and trigger intracellular changes influencing the behavior of cells.'),
  'D044082': NtO(MH='Citrobacter rodentium', MS='A species of gram-negative bacteria in the genus CITROBACTER, family ENTEROBACTERIACEAE. As an important pathogen of laboratory mice, it serves as a model for investigating epithelial hyperproliferation and tumor promotion. It was previously considered a strain of CITROBACTER FREUNDII.'),
  'D044083': NtO(MH='Cronobacter sakazakii', MS='A species of gram-negative bacteria in the genus CHRONOBACTER, found in the environment and in foods.'),
  'D044084': NtO(MH='Erwinia amylovora', MS='A species of gram-negative bacteria, in the genus ERWINIA, causing a necrotic disease of plants.'),
  'D044085': NtO(MH='Microfluidics', MS='The study of fluid channels and chambers of tiny dimensions of tens to hundreds of micrometers and volumes of nanoliters or picoliters. This is of interest in biological MICROCIRCULATION and used in MICROCHEMISTRY and INVESTIGATIVE TECHNIQUES.'),
  'D044086': NtO(MH='Kluyvera', MS='A genus of gram-negative, facultatively anaerobic bacteria in the family ENTEROBACTERIACEAE. It is found in FOOD; SOIL; and SEWAGE; and is an opportunistic pathogen of humans.'),
  'D044087': NtO(MH='Receptor, Anaphylatoxin C5a', MS='A G-protein-coupled receptor that signals an increase in intracellular calcium in response to the potent ANAPHYLATOXIN peptide COMPLEMENT C5A.'),
  'D044088': NtO(MH='Receptors, Galanin', MS='A family of G-protein-coupled receptors that are specific for GALANIN and galanin peptides. They are generally considered to be coupled to the GI, INHIBITORY G-PROTEIN to meditate the neurological effects of galanin. Several subtypes of galanin receptors occur with differing specificities for the full length galanin, galanin peptide fragments, and galanin-like peptide.'),
  'D044089': NtO(MH='Galanin-Like Peptide', MS='A neuropeptide that is highly homologous to GALANIN. It is produced by proteolytic processing of a larger protein that is unrelated to prepro-galanin and preferentially binds to GALANIN-2 RECEPTOR.'),
  'D044090': NtO(MH='Serratia liquefaciens', MS='A species of gram-negative bacteria in the genus SERRATIA found in plants and the DIGESTIVE TRACT of rodents. It is the most prevalent Serratia species in the natural environment.'),
  'D044091': NtO(MH='Receptor, Galanin, Type 1', MS='The most abundant galanin receptor subtype. It displays a high affinity for the full-length form of GALANIN.'),
  'D044092': NtO(MH='Receptor, Galanin, Type 2', MS='A galanin receptor subtype with broad specificity for full length GALANIN, galanin peptide fragments and GALANIN-LIKE PEPTIDE.'),
  'D044093': NtO(MH='Receptor, Galanin, Type 3', MS='A galanin receptor subtype with high affinity for GALANIN-LIKE PEPTIDE and low affinity for full length GALANIN and galanin peptide fragments.'),
  'D044094': NtO(MH='Receptors, Melatonin', MS='A family of G-protein-coupled receptors that are specific for and mediate the effects of MELATONIN. Activation of melatonin receptors has been associated with decreased intracellular CYCLIC AMP and increased hydrolysis of PHOSPHOINOSITIDES.'),
  'D044095': NtO(MH='Proteus penneri', MS='A genus of gram-negative bacteria isolated from individuals in LONG-TERM CARE facilities and HOSPITALS.'),
  'D044096': NtO(MH='Psychrobacter', MS='A genus of gram-negative, aerobic bacteria in the family MORAXELLACEAE, associated with processed MEAT; FISH PRODUCTS; and POULTRY PRODUCTS.'),
  'D044101': NtO(MH='Receptors, Melanocortin', MS='A family of G-protein-coupled receptors that have specificity for MELANOCYTE-STIMULATING HORMONES and ADRENOCORTICOTROPIC HORMONE. There are several subtypes of melanocortin receptors, each having a distinct ligand specificity profile and tissue localization.'),
  'D044102': NtO(MH='Receptor, Melanocortin, Type 1', MS='A melanocortin receptor subtype found primarily in MELANOCYTES. It shows specificity for ALPHA-MSH and ADRENOCORTICOTROPIC HORMONE. Loss of function mutations of the type 1 melanocortin receptor account for the majority of red hair and fair skin recessive traits in human.'),
  'D044103': NtO(MH='Receptor, Melanocortin, Type 2', MS='A melanocortin receptor subtype found primarily in the ADRENAL CORTEX. It shows specificity for ADRENOCORTICOTROPIC HORMONE.'),
  'D044104': NtO(MH='Receptor, Melanocortin, Type 3', MS='A melanocortin receptor subtype found primarily in BRAIN. It shows specificity for ALPHA-MSH; BETA-MSH; GAMMA-MSH and ADRENOCORTICOTROPIC HORMONE.'),
  'D044105': NtO(MH='Receptor, Melanocortin, Type 4', MS='A melanocortin receptor subtype found primarily in BRAIN. It shows specificity for ALPHA-MSH; BETA-MSH and ADRENOCORTICOTROPIC HORMONE.'),
  'D044106': NtO(MH='Wigglesworthia', MS='A genus of gram-negative, rod-shaped bacteria in the family ENTEROBACTERIACEAE. They exist only as primary endosymbionts of five species of TSETSE FLIES, found in specialized organelles called mycetomes. The bacteria supply crucial B vitamins (VITAMIN B COMPLEX) which the flies require for fertility.'),
  'D044122': NtO(MH='Receptor, Melatonin, MT1', MS='A melatonin receptor subtype that is primarily found in the HYPOTHALAMUS and in the KIDNEY.'),
  'D044123': NtO(MH='Receptor, Melatonin, MT2', MS='A melatonin receptor subtype primarily found expressed in the BRAIN and RETINA.'),
  'D044124': NtO(MH='Yersinia ruckeri', MS='A species of gram-negative bacteria responsible for red mouth disease in rainbow trout (ONCORHYNCHUS MYKISS). The bacteria is a natural component of fresh water ecosystems in the United States and Canada.'),
  'D044125': NtO(MH='Coxiellaceae', MS='A family of gram-negative bacteria in the order Legionellales. It includes genera COXIELLA and Rickettsiella.'),
  'D044127': NtO(MH='Epigenesis, Genetic', MS='A genetic process by which the adult organism is realized via mechanisms that lead to the restriction in the possible fates of cells, eventually leading to their differentiated state. Mechanisms involved cause heritable changes to cells without changes to DNA sequence such as DNA METHYLATION; HISTONE modification; DNA REPLICATION TIMING; NUCLEOSOME positioning; and heterochromatization which result in selective gene expression or repression.'),
  'D044128': NtO(MH='Legionella longbeachae', MS='A species of gram-negative, aerobic bacteria in the genus LEGIONELLA, first isolated in Long Beach, California. Infection is associated with the use of contaminated potting SOIL.'),
  'D044129': NtO(MH='Halomonadaceae', MS='A family of gram-negative, moderately halophilic bacteria in the order Oceanospirillales. Members of the family have been isolated from temperate and Antarctic saline lakes, solar salt facilities, saline soils, and marine environments.'),
  'D044130': NtO(MH='Oceanospirillaceae', MS='A family of gram-negative bacteria in the order Oceanospirillales.'),
  'D044131': NtO(MH='Actinobacillus equuli', MS='A genus of gram-negative bacteria in the genus ACTINOBACILLUS, which is pathogenic for HORSES and PIGS.'),
  'D044132': NtO(MH='Actinobacillus suis', MS='A species of gram-negative bacteria in the genus ACTINOBACILLUS. It is mainly a pathogen of PIGS, but also can infect HORSES.'),
  'D044133': NtO(MH='Actinobacillus seminis', MS='A species of gram-negative bacteria in the genus ACTINOBACILLUS, which causes EPIDIDYMITIS in SHEEP.'),
  'D044134': NtO(MH='Haemophilus paragallinarum', MS='A species of gram-negative bacteria in the genus HAEMOPHILUS, causing respiratory tract disease in CHICKENS known as infectious coryza.'),
  'D044135': NtO(MH='Haemophilus parainfluenzae', MS='A species of gram-negative bacteria in the genus HAEMOPHILUS, ubiquitous in the human ORAL CAVITY and PHARYNX. It has low pathogenicity but is occasionally implicated in ENDOCARDITIS in humans.'),
  'D044136': NtO(MH='Haemophilus paraphrophilus', MS='A species of gram-negative bacteria in the genus HAEMOPHILUS, found in the normal flora of the human ORAL CAVITY and PHARYNX. It can cause SUBACUTE BACTERIAL ENDOCARDITIS; and BRAIN ABSCESS, among other conditions.'),
  'D044137': NtO(MH='Haemophilus parasuis', MS='A species of gram-negative bacteria in the genus HAEMOPHILUS found, in the normal upper respiratory tract of SWINE.'),
  'D044138': NtO(MH='Haemophilus somnus', MS='A species of gram-negative bacteria (currently incertae sedis) causing multisystem disease in CATTLE.'),
  'D044139': NtO(MH='Receptor, Angiotensin, Type 2', MS='An angiotensin receptor subtype that is expressed at high levels in fetal tissues. Many effects of the angiotensin type 2 receptor such as VASODILATION and sodium loss are the opposite of that of the ANGIOTENSIN TYPE 1 RECEPTOR.'),
  'D044140': NtO(MH='Receptor, Angiotensin, Type 1', MS='An angiotensin receptor subtype that is expressed at high levels in a variety of adult tissues including the CARDIOVASCULAR SYSTEM, the KIDNEY, the ENDOCRINE SYSTEM and the NERVOUS SYSTEM. Activation of the type 1 angiotensin receptor causes VASOCONSTRICTION and sodium retention.'),
  'D044142': NtO(MH='Pasteurella pneumotropica', MS='A species of gram-negative bacteria in the genus PASTEURELLA, found in the NASOPHARYNX of normal GUINEA PIGS; RATS; HAMSTERS; MICE; DOGS; and CATS. When associated with disease, it is usually a secondary invader. Occasional infections have been reported in humans.'),
  'D044147': NtO(MH='Piscirickettsiaceae', MS='A family of gram-negative bacteria, in the order Thiotrichales.'),
  'D044148': NtO(MH='Lymphatic Abnormalities', MS='Congenital or acquired structural abnormalities of the lymphatic system (LYMPHOID TISSUE) including the lymph vessels.'),
  'D044149': NtO(MH='Vibrio alginolyticus', MS='A species of gram-negative, halophilic bacteria, in the genus VIBRIO. It is considered part of normal marine flora and commonly associated with ear infections and superficial wounds exposed to contaminated water sources.'),
  'D044162': NtO(MH='Parathyroid Hormone-Related Protein', MS='A ubiquitously expressed, secreted protein with bone resorption and renal calcium reabsorption activities that are similar to PARATHYROID HORMONE. It does not circulate in appreciable amounts in normal subjects, but rather exerts its biological actions locally. Overexpression of parathyroid hormone-related protein by tumor cells results in humoral calcemia of malignancy.'),
  'D044163': NtO(MH='Vibrio cholerae non-O1', MS='A strain of the VIBRIO CHOLERAE bacteria belonging to serogroup non-O1, infecting humans and other PRIMATES. It is related to VIBRIO CHOLERAE O1, but causes a disease less severe than CHOLERA. Eating raw shellfish contaminated with the bacteria results in GASTROENTERITIS.'),
  'D044164': NtO(MH='Vibrio mimicus', MS='A species of gram-negative bacteria in the genus VIBRIO, isolated from SHELLFISH, as well as from human diarrheal stools and ear infections.'),
  'D044165': NtO(MH='Aliivibrio salmonicida', MS='A species of gram-negative bacteria in the genus ALIIVIBRIO. Aliivibrio salmonicida is the causative agent of cold water vibriosis, a HEMORRHAGIC SEPTICEMIA of ATLANTIC SALMON.'),
  'D044166': NtO(MH='Xanthomonadaceae', MS='A family of gram-negative bacteria, in the order Xanthomonadales, pathogenic to plants.'),
  'D044167': NtO(MH='Xylella', MS='A genus of gram-negative, aerobic bacteria, in the family XANTHOMONADACEAE. It is found in the xylem of plant tissue.'),
  'D044168': NtO(MH='Receptor, Parathyroid Hormone, Type 1', MS='A parathyroid hormone receptor subtype that recognizes both PARATHYROID HORMONE and PARATHYROID HORMONE-RELATED PROTEIN. It is a G-protein-coupled receptor that is expressed at high levels in BONE and in KIDNEY.'),
  'D044169': NtO(MH='Receptors, Calcium-Sensing', MS='A class of G-protein-coupled receptors that react to varying extracellular CALCIUM levels. Calcium-sensing receptors in the PARATHYROID GLANDS play an important role in the maintenance of calcium HOMEOSTASIS by regulating the release of PARATHYROID HORMONE. They differ from INTRACELLULAR CALCIUM-SENSING PROTEINS which sense intracellular calcium levels.'),
  'D044182': NtO(MH='Benzylisoquinolines', MS='ISOQUINOLINES with a benzyl substituent.'),
  'D044183': NtO(MH='Receptor, Parathyroid Hormone, Type 2', MS='A parathyroid hormone receptor subtype found in the BRAIN and the PANCREAS. It is a G-protein-coupled receptor with a ligand specificity that varies between homologs from different species.'),
  'D044184': NtO(MH='Xanthomonas vesicatoria', MS='A species of gram-negative bacteria, in the genus XANTHOMONAS, causing disease in TOMATO and pepper crops.'),
  'D044202': NtO(MH='Pseudomonas alcaligenes', MS='A species of gram-negative bacteria in the genus PSEUDOMONAS. It cannot utilize FRUCTOSE; GLUCOSE; or MALTOSE for energy.'),
  'D044203': NtO(MH='Pseudomonas mendocina', MS='A species of gram-negative bacteria in the genus PSEUDOMONAS, which is found in SOIL and WATER.'),
  'D044204': NtO(MH='Pseudomonas oleovorans', MS='A species of gram-negative bacteria in the genus PSEUDOMONAS. It is isolated from oil-water emulsions used as lubricants and cooling agents in the cutting and grinding of materials.'),
  'D044205': NtO(MH='Pseudomonas pseudoalcaligenes', MS='A species of gram-negative bacteria in the genus PSEUDOMONAS. All strains can utilize FRUCTOSE for energy. It is occasionally isolated from humans and some strains are pathogenic to WATERMELON.'),
  'D044206': NtO(MH='Pseudomonas fragi', MS='A species of gram-negative, aerobic bacteria in the genus PSEUDOMONAS. It has a characteristic strawberry color and is widely distributed in SOIL and WATER.'),
  'D044222': NtO(MH='Halothiobacillus', MS='A genus of gram-negative, chemolithoautotrophic bacteria in the family Halothiobacillaceae. Several of its species were reclassified to this genus from THIOBACILLUS.'),
  'D044223': NtO(MH='Pseudomonas stutzeri', MS='A species of gram-negative bacteria in the genus PSEUDOMONAS, containing multiple genomovars. It is distinguishable from other pseudomonad species by its ability to use MALTOSE and STARCH as sole carbon and energy sources. It can degrade ENVIRONMENTAL POLLUTANTS and has been used as a model organism to study denitrification.'),
  'D044224': NtO(MH='Pseudomonas syringae', MS='A species of gram-negative, fluorescent, phytopathogenic bacteria in the genus PSEUDOMONAS. It is differentiated into approximately 50 pathovars with different plant pathogenicities and host specificities.'),
  'D044225': NtO(MH='Piscirickettsiaceae Infections', MS='Infections with bacteria of the family PISCIRICKETTSIACEAE, causing septicemic disease of salmonid fish (SALMONIDAE). Piscirickettsia-like bacteria are also associated with disease syndromes in non-salmonid fish.'),
  'D044242': NtO(MH='Trans Fatty Acids', MS='UNSATURATED FATTY ACIDS that contain at least one double bond in the trans configuration, which results in a greater bond angle than the cis configuration. This results in a more extended fatty acid chain similar to SATURATED FATTY ACIDS, with closer packing and reduced fluidity. HYDROGENATION of unsaturated fatty acids increases the trans content.'),
  'D044243': NtO(MH='Linoleic Acids, Conjugated', MS='A collective term for a group of around nine geometric and positional isomers of LINOLEIC ACID in which the trans/cis double bonds are conjugated, where double bonds alternate with single bonds.'),
  'D044244': NtO(MH='Receptors, Thromboxane A2, Prostaglandin H2', MS='A subclass of eicosanoid receptors that have specificity for THROMBOXANE A2 and PROSTAGLANDIN H2.'),
  'D044262': NtO(MH='Prostaglandin H2', MS='A cyclic endoperoxide intermediate produced by the action of CYCLOOXYGENASE on ARACHIDONIC ACID. It is further converted by a series of specific enzymes to the series 2 prostaglandins.'),
  'D044263': NtO(MH='Receptors, Serotonin, 5-HT1', MS='A subclass of G-protein coupled SEROTONIN receptors that couple preferentially to GI-GO G-PROTEINS resulting in decreased intracellular CYCLIC AMP levels.'),
  'D044282': NtO(MH='Receptor, Serotonin, 5-HT1A', MS='A serotonin receptor subtype found distributed through the CENTRAL NERVOUS SYSTEM where they are involved in neuroendocrine regulation of ACTH secretion. The fact that this serotonin receptor subtype is particularly sensitive to SEROTONIN RECEPTOR AGONISTS such as BUSPIRONE suggests its role in the modulation of ANXIETY and DEPRESSION.'),
  'D044302': NtO(MH='Receptor, Serotonin, 5-HT1B', MS="A serotonin receptor subtype found at high levels in the BASAL GANGLIA and the frontal cortex. It plays a role as a terminal autoreceptor that regulates the rate of SEROTONIN release from nerve endings. This serotonin receptor subtype is closely related to and has similar drug binding properties as the 5-HT1D RECEPTOR. It is particularly sensitive to the agonist SUMATRIPTAN and may be involved in mediating the drug's antimigraine effect."),
  'D044342': NtO(MH='Malnutrition', MS='An imbalanced nutritional status resulting from insufficient intake of nutrients to meet normal physiological requirement.'),
  'D044343': NtO(MH='Overnutrition', MS='An imbalanced NUTRITIONAL STATUS resulting from excessive intake of nutrients. Generally, overnutrition generates an energy imbalance between food consumption and energy expenditure leading to disorders such as OBESITY.'),
  'D044345': NtO(MH='Phosphoric Triester Hydrolases', MS='A class of enzymes that catalyze the hydrolysis of one of the three ester bonds in a phosphotriester-containing compound.'),
  'D044346': NtO(MH='Receptor, Serotonin, 5-HT1D', MS="A serotonin receptor subtype that is localized to the CAUDATE NUCLEUS; PUTAMEN; the NUCLEUS ACCUMBENS; the HIPPOCAMPUS, and the RAPHE NUCLEI. It plays a role as a terminal autoreceptor that regulates the rate of SEROTONIN release from nerve endings. This serotonin receptor subtype is closely related to and has similar drug binding properties as the 5-HT1B RECEPTOR, but is expressed at low levels. It is particularly sensitive to the agonist SUMATRIPTAN and may be involved in mediating the drug's antimigrane effect."),
  'D044347': NtO(MH='Sexology', MS='This discipline concerns the study of SEXUALITY, and the application of sexual knowledge such as sexual attitudes, psychology, and SEXUAL BEHAVIOR. Scope of application generally includes educational (SEX EDUCATION), clinical (SEX COUNSELING), and other settings.'),
  'D044348': NtO(MH='Receptors, Serotonin, 5-HT2', MS='A subclass of G-protein coupled SEROTONIN receptors that couple preferentially to the GQ-G11 G-PROTEINS resulting in increased intracellular levels of INOSITOL PHOSPHATES and free CALCIUM.'),
  'D044349': NtO(MH='Oceania', MS='The islands of the central and South Pacific, including Micronesia, Melanesia, Polynesia, and traditionally Australasia. (Random House Dictionary, 2d ed)'),
  'D044362': NtO(MH='Australasia', MS='Australia, New Zealand and neighboring islands in the South Pacific Ocean. (Random House Unabridged Dictionary, 2d ed.)'),
  'D044363': NtO(MH='Contraception, Postcoital', MS='Means of postcoital intervention to avoid pregnancy, such as the administration of POSTCOITAL CONTRACEPTIVES to prevent FERTILIZATION of an egg or implantation of a fertilized egg (OVUM IMPLANTATION).'),
  'D044364': NtO(MH='Vaginal Douching', MS='The washing of the VAGINA cavity or surface with a solution. Agents or drugs can be added to the irrigation solution.'),
  'D044365': NtO(MH='GTP-Binding Protein alpha Subunits, G12-G13', MS='A ubiquitously expressed family of heterotrimeric GTP-binding protein alpha subunits that signal through interactions with a variety of second messengers as GTPASE-ACTIVATING PROTEINS; GUANINE NUCLEOTIDE EXCHANGE FACTORS; and HEAT SHOCK PROTEINS. The G12-G13 part of the name is also spelled G12/G13.'),
  'D044366': NtO(MH='Transition Temperature', MS='The temperature at which a substance changes from one state or conformation of matter to another.'),
  'D044367': NtO(MH='Phase Transition', MS='A change of a substance from one form or state to another.'),
  'D044382': NtO(MH='Population Groups', MS='Individuals classified according to their sex, racial origin, religion, common place of living, financial or social status, or some other cultural or behavioral attribute. (UMLS, 2003)'),
  'D044383': NtO(MH='African Continental Ancestry Group', MS='Individuals whose ancestral origins are in the continent of Africa.'),
  'D044384': NtO(MH='Ovulation Prediction', MS='Predicting the time of OVULATION can be achieved by measuring the preovulatory elevation of ESTRADIOL; LUTEINIZING HORMONE or other hormones in BLOOD or URINE. Accuracy of ovulation prediction depends on the completeness of the hormone profiles, and the ability to determine the preovulatory LH peak.'),
  'D044385': NtO(MH='GTP-Binding Protein alpha Subunits', MS='The GTPase-containing subunits of heterotrimeric GTP-binding proteins. When dissociated from the heterotrimeric complex these subunits interact with a variety of second messenger systems. Hydrolysis of GTP by the inherent GTPase activity of the subunit causes it to revert to its inactive (heterotrimeric) form. The GTP-Binding protein alpha subunits are grouped into families according to the type of action they have on second messenger systems.'),
  'D044387': NtO(MH='GTP-Binding Protein beta Subunits', MS='Heterotrimeric GTP-binding protein subunits that tightly associate with GTP-BINDING PROTEIN GAMMA SUBUNITS. A dimer of beta and gamma subunits is formed when the GTP-BINDING PROTEIN ALPHA SUBUNIT dissociates from the GTP-binding protein heterotrimeric complex. The beta-gamma dimer can play an important role in signal transduction by interacting with a variety of second messengers.'),
  'D044388': NtO(MH='GTP-Binding Protein gamma Subunits', MS='Heterotrimeric GTP-binding protein subunits that tightly associate with GTP-BINDING PROTEIN BETA SUBUNITS. A dimer of beta and gamma subunits is formed when the GTP-BINDING PROTEIN ALPHA SUBUNIT dissociates from the GTP-binding protein heterotrimeric complex. The beta-gamma dimer can play an important role in signal transduction by interacting with a variety of second messengers.'),
  'D044402': NtO(MH='Receptor, Serotonin, 5-HT2A', MS='A serotonin receptor subtype found widely distributed in peripheral tissues where it mediates the contractile responses of variety of tissues that contain SMOOTH MUSCLE. Selective 5-HT2A receptor antagonists include KETANSERIN. The 5-HT2A subtype is also located in BASAL GANGLIA and CEREBRAL CORTEX of the BRAIN where it mediates the effects of HALLUCINOGENS such as LSD.'),
  'D044403': NtO(MH='Receptor, Serotonin, 5-HT2B', MS='A serotonin receptor subtype found in the BRAIN; HEART; LUNGS; PLACENTA and DIGESTIVE SYSTEM organs. A number of functions have been attributed to the action of the 5-HT2B receptor including the development of cardiac myocytes (MYOCYTES, CARDIAC) and the contraction of SMOOTH MUSCLE.'),
  'D044404': NtO(MH='Genomic Islands', MS='Distinct units in some bacterial, bacteriophage or plasmid GENOMES that are types of MOBILE GENETIC ELEMENTS. Encoded in them are a variety of fitness conferring genes, such as VIRULENCE FACTORS (in "pathogenicity islands or islets"), ANTIBIOTIC RESISTANCE genes, or genes required for SYMBIOSIS (in "symbiosis islands or islets"). They range in size from 10 - 500 kilobases, and their GC CONTENT and CODON usage differ from the rest of the genome. They typically contain an INTEGRASE gene, although in some cases this gene has been deleted resulting in "anchored genomic islands".'),
  'D044405': NtO(MH='Receptor, Serotonin, 5-HT2C', MS='A serotonin receptor subtype found primarily in the CENTRAL NERVOUS SYSTEM and the CHOROID PLEXUS. This receptor subtype is believed to mediate the anorectic action of SEROTONIN, while selective antagonists of the 5-HT2C receptor appear to induce ANXIETY. Several isoforms of this receptor subtype exist, due to adenine deaminase editing of the receptor mRNA.'),
  'D044406': NtO(MH='Receptors, Serotonin, 5-HT3', MS='A subclass of serotonin receptors that form cation channels and mediate signal transduction by depolarizing the cell membrane. The cation channels are formed from 5 receptor subunits. When stimulated the receptors allow the selective passage of SODIUM; POTASSIUM; and CALCIUM.'),
  'D044422': NtO(MH='Receptors, Serotonin, 5-HT4', MS='A subtype of G-protein-coupled SEROTONIN receptors that preferentially couple to GS STIMULATORY G-PROTEINS resulting in increased intracellular CYCLIC AMP. Several isoforms of the receptor exist due to ALTERNATIVE SPLICING of its mRNA.'),
  'D044423': NtO(MH='Kelp', MS='Large, robust forms of brown algae (PHAEOPHYCEAE) in the order Laminariales. They are a major component of the lower intertidal and sublittoral zones on rocky coasts in temperate and polar waters. Kelp, a kind of SEAWEED, usually refers to species in the genera LAMINARIA or MACROCYSTIS, but the term may also be used for species in FUCUS or Nereocystis.'),
  'D044424': NtO(MH='Cyanophora', MS='A genus of primitive plants in the family Cyanophoraceae, class GLAUCOPHYTA. They contain pigmented ORGANELLES (or PLASTIDS) called cyanelles, which have characteristics of both CYANOBACTERIA and CHLOROPLASTS.'),
  'D044443': NtO(MH='Nitella', MS='A genus of green plants in the family CHARACEAE, phylum STREPTOPHYTA. They are found in shallow to deep waters of soft water or acid lakes and bogs.'),
  'D044444': NtO(MH='Scenedesmus', MS='A genus of GREEN ALGAE in the family Scenedesmaceae. It forms colonies of usually four or eight cylindrical cells that are widely distributed in freshwater and SOIL.'),
  'D044445': NtO(MH='Ulva', MS='A genus of GREEN ALGAE in the family Ulvaceae. Commonly know as sea lettuces, they grow attached to rocks and KELP in marine and estuarine waters.'),
  'D044446': NtO(MH='Volvox', MS='A genus of GREEN ALGAE in the family Volvocaceae. They form spherical colonies of hundreds or thousands of bi-flagellated cells in a semi-transparent gelatinous ball.'),
  'D044462': NtO(MH='Receptors, Proteinase-Activated', MS='A class of receptors that are activated by the action of PROTEINASES. The most notable examples are the THROMBIN RECEPTORS. The receptors contain cryptic ligands that are exposed upon the selective proteolysis of specific N-terminal cleavage sites.'),
  'D044463': NtO(MH='Receptor, PAR-1', MS='A thrombin receptor subtype that couples to HETEROTRIMERIC GTP-BINDING PROTEINS resulting in the activation of a variety of signaling mechanisms including decreased intracellular CYCLIC AMP, increased TYPE C PHOSPHOLIPASES and increased PHOSPHOLIPASE A2.'),
  'D044464': NtO(MH='Receptor, PAR-2', MS='A G-protein-coupled, proteinase-activated receptor that is expressed in a variety of tissues including ENDOTHELIUM; LEUKOCYTES; and the GASTROINTESTINAL TRACT. The receptor is activated by TRYPSIN, which cleaves off the N-terminal peptide from the receptor. The new N-terminal peptide is a cryptic ligand for the receptor. The uncleaved receptor can also be activated by the N-terminal peptide present on the activated THROMBIN RECEPTOR and by small synthetic peptides that contain the unmasked N-terminal sequence.'),
  'D044465': NtO(MH='European Continental Ancestry Group', MS='Individuals whose ancestral origins are in the continent of Europe.'),
  'D044466': NtO(MH='Asian Continental Ancestry Group', MS='Individuals whose ancestral origins are in the southeastern and eastern areas of the Asian continent.'),
  'D044467': NtO(MH='American Native Continental Ancestry Group', MS='Individuals whose ancestral origins are in the continents of the Americas.'),
  'D044468': NtO(MH='Oceanic Ancestry Group', MS='Individuals whose ancestral origins are in the islands of the central and South Pacific, including Micronesia, Melanesia, Polynesia, and traditionally Australasia.'),
  'D044469': NtO(MH='Continental Population Groups', MS='Groups of individuals whose putative ancestry is from native continental populations based on similarities in physical appearance.'),
  'D044482': NtO(MH='Benzeneacetamides', MS='Compounds based on benzeneacetamide, that are similar in structure to ACETANILIDES.'),
  'D044483': NtO(MH='Intestinal Polyposis', MS='The growth of INTESTINAL POLYPS. Growth processes include neoplastic (ADENOMA and CARCINOMA) and non-neoplastic (hyperplastic, mucosal, inflammatory, and other polyps).'),
  'D044502': NtO(MH='Thymine DNA Glycosylase', MS='An enzyme that removes THYMINE and URACIL bases mispaired with GUANINE through hydrolysis of their N-glycosidic bond. These mispaired nucleotides generally occur through the hydrolytic DEAMINATION of 5-METHYLCYTOSINE to thymine.'),
  'D044503': NtO(MH='5-Methylcytosine', MS='A methylated nucleotide base found in eukaryotic DNA. In ANIMALS, the DNA METHYLATION of CYTOSINE to form 5-methylcytosine is found primarily in the palindromic sequence CpG. In PLANTS, the methylated sequence is CpNpGp, where N can be any base.'),
  'D044504': NtO(MH='Enterocolitis, Neutropenic', MS='A syndrome characterized by inflammation in the ILEUM, the CECUM, and the ASCENDING COLON. It is observed in cancer patients with CHEMOTHERAPY-induced NEUTROPENIA or in other immunocompromised individuals (IMMUNOCOMPROMISED HOST).'),
  'D044522': NtO(MH='Characeae', MS='Family of slender threadlike aquatic plants, in the order CHARALES, phylum STREPTOPHYTA, that are closely related to LAND PLANTS.'),
  'D044523': NtO(MH='Chrysophyta', MS='A family of microscopic freshwater EUKARYOTA, commonly known as golden algae. They share many features with the BROWN ALGAE but are planktonic rather than benthic. Though most are photosynthetic, they are not considered truly autotrophic since they can become facultatively heterotrophic in the absence of adequate light. In this state they can feed on BACTERIA or DIATOMS.'),
  'D044542': NtO(MH='LEOPARD Syndrome', MS='An autosomal dominant disorder with an acronym of its seven features (LENTIGO; ELECTROCARDIOGRAM abnormalities; ocular HYPERTELORISM; PULMONARY STENOSIS; abnormal genitalia; retardation of growth; and DEAFNESS or SENSORINEURAL HEARING LOSS). This syndrome is caused by mutations of PTPN11 gene encoding the non-receptor PROTEIN TYROSINE PHOSPHATASE, type 11, and is an allelic to NOONAN SYNDROME. Features of LEOPARD syndrome overlap with those of NEUROFIBROMATOSIS 1 which is caused by mutations in the NEUROFIBROMATOSIS 1 GENES.'),
  'D044563': NtO(MH='RNA, Ribosomal, Self-Splicing', MS='Components of ribosomal RNA that undergo auto-catalyzed molecular rearrangements of their RNA sequence.'),
  'D044582': NtO(MH='Neuroendoscopes', MS='Instruments for the visual examination of interior structures related to NEUROSURGICAL PROCEDURES. There are two broad categories of neuroendoscopes, the rigid scope and the flexible scope, for various types of procedures.'),
  'D044583': NtO(MH='Neuroendoscopy', MS='PROCEDURES that use NEUROENDOSCOPES for disease diagnosis and treatment. Neuroendoscopy, generally an integration of the neuroendoscope with a computer-assisted NEURONAVIGATION system, provides guidance in NEUROSURGICAL PROCEDURES.'),
  'D044584': NtO(MH='Carcinoma, Ductal', MS='Malignant neoplasms involving the ductal systems of any of a number of organs, such as the MAMMARY GLANDS, the PANCREAS, the PROSTATE, or the LACRIMAL GLAND.'),
  'D044602': NtO(MH='Cellulases', MS='A family of glycosidases that hydrolyse crystalline CELLULOSE into soluble sugar molecules. Within this family there are a variety of enzyme subtypes with differing substrate specificities that must work together to bring about complete cellulose hydrolysis. They are found in structures called CELLULOSOMES.'),
  'D044603': NtO(MH='Cellulosomes', MS='Extracellular structures found in a variety of microorganisms. They contain CELLULASES and play an important role in the digestion of CELLULOSE.'),
  'D044622': NtO(MH='Dental Pellicle', MS='A thin protein film on the surface of DENTAL ENAMEL. It is widely believed to result from the selective adsorption of precursor proteins present in SALIVA onto tooth surfaces, and to reduce microbial adherence to the TEETH.'),
  'D044623': NtO(MH='Nutrition Therapy', MS='Improving health status of an individual by adjusting the quantities, qualities, and methods of nutrient intake.'),
  'D044643': NtO(MH='Glucan 1,4-beta-Glucosidase', MS='An exocellulase with specificity for the hydrolysis of 1,4-beta-glucosidic linkages of 1,4-beta-D-glucans resulting in successive removal of GLUCOSE units.'),
  'D044644': NtO(MH='Endo-1,3(4)-beta-Glucanase', MS='An endocellulase with specificity for the hydrolysis of 1,3- or 1,4-linkages in beta-D-glucans. This enzyme specifically acts on sites where reducing glucose residues are substituted at the 3 position.'),
  'D044645': NtO(MH='Fucus', MS='A genus of BROWN ALGAE in the family Fucaceae. It is found in temperate, marine intertidal areas along rocky coasts and is a source of ALGINATES. Some species of Fucus are referred to as KELP.'),
  'D044646': NtO(MH='Ascophyllum', MS='A BROWN ALGAE closely related to FUCUS. It is found attached to rocks and boulders on the middle shore, primarily in the North Atlantic basin.'),
  'D044647': NtO(MH='Sargassum', MS='One of the largest genera of BROWN ALGAE, comprised of more than 150 species found in tropical, subtropical, and temperate zones of both hemispheres. Some species are attached (benthic) but most float in the open sea (pelagic). Sargassum provides a critical habitat for hundreds of species of FISHES; TURTLES; and INVERTEBRATES.'),
  'D044648': NtO(MH='Undaria', MS='A genus of BROWN ALGAE, in the family Alariaceae, native to Japan, Korea, and China. The edible SEAWEED Undaria pinnatifida is also called wakame.'),
  'D044662': NtO(MH='Macrocystis', MS='A genus of BROWN ALGAE in the family Lessoniaceae, and one of the major forms of KELP. The species Macrocystis pyrifera, also known as giant kelp, is the largest of the marine algae.'),
  'D044663': NtO(MH='Ochromonas', MS='A genus of GOLDEN-BROWN ALGAE in the family Ochromonadaceae, found mostly in freshwater. They bear two unequal FLAGELLA and are heterotrophic.'),
  'D044664': NtO(MH='Porphyra', MS='A genus of RED ALGAE in the family Bangiaceae. It is the most widely consumed SEAWEED in the world and especially in Asia.'),
  'D044665': NtO(MH='Porphyridium', MS='A genus of RED ALGAE in the family Porphyridiaceae. A single stellate CHLOROPLAST is present in each cell which has no CELL WALL.'),
  'D044682': NtO(MH='Colon, Ascending', MS='The segment of LARGE INTESTINE between the CECUM and the TRANSVERSE COLON. It passes cephalad from the cecum to the caudal surface of the right lobe of the LIVER where it bends sharply to the left, forming the right colic flexure.'),
  'D044683': NtO(MH='Colon, Descending', MS='The segment of LARGE INTESTINE between TRANSVERSE COLON and the SIGMOID COLON.'),
  'D044684': NtO(MH='Colon, Transverse', MS='The segment of LARGE INTESTINE between ASCENDING COLON and DESCENDING COLON. It passes from the RIGHT COLIC FLEXURE across the ABDOMEN, then turns sharply at the left colonic flexure into the descending colon.'),
  'D044704': NtO(MH='DNA, A-Form', MS='An isoform of DNA that occurs in an environment rich in SODIUM and POTASSIUM ions. It is a right-handed helix with 11 base pairs per turn, a pitch of 0.256 nm per base pair and a helical diameter of 2.3 nm.'),
  'D044705': NtO(MH='DNA, C-Form', MS='An isoform of DNA that occurs under experimental conditions. It is a right-handed helix that is less compact than the B-form of DNA.'),
  'D044707': NtO(MH='Forensic Sciences', MS='Disciplines that apply sciences to law. Forensic sciences include a wide range of disciplines, such as FORENSIC TOXICOLOGY; FORENSIC ANTHROPOLOGY; FORENSIC MEDICINE; FORENSIC DENTISTRY; and others.'),
  'D044708': NtO(MH='DNA, Concatenated', MS='Head to tail array of covalently joined DNA sequences generated by concatenation. Concatenated DNA is attached end to end in contrast to CATENATED DNA which is attached loop to loop.'),
  'D044709': NtO(MH='Forensic Ballistics', MS='The science of studying projectiles in motion, ballistics, being applied to law. Ballistics on firearm projectiles, such as bullets, include the study of what happens inside the weapon, during the flight of the projectile, and when the projectile strikes the target, such as body tissue.'),
  'D044722': NtO(MH='Laurencia', MS='A genus of RED ALGAE in the family Rhodomelaceae. Some species are a rich source of chlorine-and bromine-containing metabolites which show significant antibacterial activity.'),
  'D044723': NtO(MH='Chondrus', MS='A genus of RED ALGAE, in the family Gigartinaceae. The species Chondrus crispus is a source of CARRAGEENAN.'),
  'D044724': NtO(MH='Gracilaria', MS='A genus of RED ALGAE in the family Gracilariaceae. It is a highly commercial SEAWEED and a source of AGAR.'),
  'D044725': NtO(MH='Plocamium', MS='A genus of RED ALGAE, in the family Plocamiaceae, found in temperate marine environments worldwide. Polyhalogenated monoterpenes of potential medicinal use have been isolated from its species.'),
  'D044726': NtO(MH='Lagenidium', MS='A genus of OOMYCETES in the family Lagenidiaceae. One species (L. giganteum) parasitizes the larval stage of MOSQUITOES and is used in its biological control (PEST CONTROL, BIOLOGICAL); it also causes disease in freshwater FISHES.'),
  'D044742': NtO(MH='Peronospora', MS='A genus of OOMYCETES in the family Peronosporaceae. Most species are obligatory parasites and many are plant pathogens.'),
  'D044743': NtO(MH='Achlya', MS='A genus of OOMYCETES in the family Saprolegniaceae. They are pathogens of FISHES and INSECTS.'),
  'D044744': NtO(MH='Aphanomyces', MS='A genus of OOMYCETES in the family Saprolegniaceae. It causes root rot in plants and is also a pathogen of FISHES.'),
  'D044762': NtO(MH='Saprolegnia', MS='A genus of OOMYCETES in the family Saprolegniaceae. It is a parasite and pathogen of freshwater FISHES.'),
  'D044763': NtO(MH='Ubiquitin-Conjugating Enzymes', MS='A class of enzymes that form a thioester bond to UBIQUITIN with the assistance of UBIQUITIN-ACTIVATING ENZYMES. They transfer ubiquitin to the LYSINE of a substrate protein with the assistance of UBIQUITIN-PROTEIN LIGASES.'),
  'D044764': NtO(MH='Ubiquitin-Activating Enzymes', MS='A class of enzymes that catalyzes the ATP-dependent formation of a thioester bond between itself and UBIQUITIN. It then transfers the activated ubiquitin to one of the UBIQUITIN-PROTEIN LIGASES.'),
  'D044767': NtO(MH='Ubiquitin-Protein Ligases', MS='A diverse class of enzymes that interact with UBIQUITIN-CONJUGATING ENZYMES and ubiquitination-specific protein substrates. Each member of this enzyme group has its own distinct specificity for a substrate and ubiquitin-conjugating enzyme. Ubiquitin-protein ligases exist as both monomeric proteins multiprotein complexes.'),
  'D044782': NtO(MH='F-Box Motifs', MS='Protein structural motifs that play a role in protein-protein binding. The motifs are comprised of approximately 50 residues. Their name derives from the fact that they were found in cyclin F.'),
  'D044783': NtO(MH='F-Box Proteins', MS='A family of proteins that share the F-BOX MOTIF and are involved in protein-protein interactions. They play an important role in process of protein ubiquition by associating with a variety of substrates and then associating into SCF UBIQUITIN LIGASE complexes. They are held in the ubiquitin-ligase complex via binding to SKP DOMAIN PROTEINS.'),
  'D044784': NtO(MH='Chara', MS='A genus of green plants in the family CHARACEAE, phylum STREPTOPHYTA. They have a strong garlic-like odor and are an important food source for waterfowl.'),
  'D044785': NtO(MH='Cryptophyta', MS='A class of EUKARYOTA (traditionally algae), characterized by biflagellated cells and found in both freshwater and marine environments. Pigmentation varies but only one CHLOROPLAST is present. Unique structures include a nucleomorph and ejectosomes.'),
  'D044786': NtO(MH='S-Phase Kinase-Associated Proteins', MS='A family of structurally-related proteins that were originally identified by their ability to complex with cyclin proteins (CYCLINS). They share a common domain that binds specifically to F-BOX MOTIFS. They take part in SKP CULLIN F-BOX PROTEIN LIGASES, where they can bind to a variety of F-BOX PROTEINS.'),
  'D044802': NtO(MH='Euglena longa', MS='A species of EUKARYOTES in the phylum EUGLENIDA. It is a colorless heterotrophic flagellate.'),
  'D044822': NtO(MH='Biodiversity', MS='The variety of all native living organisms and their various forms and interrelationships.'),
  'D044842': NtO(MH='Cullin Proteins', MS='A family of structurally related proteins that were originally discovered for their role in cell-cycle regulation in CAENORHABDITIS ELEGANS. They play important roles in regulation of the CELL CYCLE and as components of UBIQUITIN-PROTEIN LIGASES.'),
  'D044843': NtO(MH='SKP Cullin F-Box Protein Ligases', MS='A subset of ubiquitin protein ligases that are formed by the association of a SKP DOMAIN PROTEIN, a CULLIN DOMAIN PROTEIN and a F-BOX DOMAIN PROTEIN.'),
  'D044844': NtO(MH='beta-Transducin Repeat-Containing Proteins', MS='A family of F-box domain proteins that contain sequences that are homologous to the beta subunit of transducin (BETA-TRANSDUCIN). They play an important role in the protein degradation pathway by becoming components of SKP CULLIN F-BOX PROTEIN LIGASES, which selectively act on a subset of proteins including beta-catenin and IkappaBbeta.'),
  'D044862': NtO(MH='Campylobacter hyointestinalis', MS='A species of CAMPYLOBACTER isolated from the INTESTINES of PIGS with proliferative ENTERITIS. It is also found in CATTLE and in CRICETINAE and can cause enteritis in humans.'),
  'D044863': NtO(MH='Campylobacter lari', MS='A species of thermophilic CAMPYLOBACTER found in healthy seagulls and causing ENTERITIS in humans.'),
  'D044882': NtO(MH='Glucose Metabolism Disorders', MS='Pathological conditions in which the BLOOD GLUCOSE cannot be maintained within the normal range, such as in HYPOGLYCEMIA and HYPERGLYCEMIA. Etiology of these disorders varies. Plasma glucose concentration is critical to survival for it is the predominant fuel for the CENTRAL NERVOUS SYSTEM.'),
  'D044883': NtO(MH='Campylobacter rectus', MS='A species of CAMPYLOBACTER isolated from cases of human PERIODONTITIS. It is a microaerophile, capable of respiring with OXYGEN.'),
  'D044884': NtO(MH='Campylobacter sputorum', MS='A species of CAMPYLOBACTER comprised of three biovars based on their reaction to CATALASE and UREASE. They have been isolated from humans, CATTLE, and SHEEP.'),
  'D044885': NtO(MH='Campylobacter upsaliensis', MS='A species of CAMPYLOBACTER isolated from DOGS; CATS; and humans.'),
  'D044902': NtO(MH='beta-Mannosidase', MS='An enzyme that catalyzes the hydrolysis of terminal, non-reducing beta-D-mannose residues in beta-D-mannosides. The enzyme plays a role in the lysosomal degradation of the N-glycosylprotein glycans. Defects in the lysosomal form of the enzyme in humans result in a buildup of mannoside intermediate metabolites and the disease BETA-MANNOSIDOSIS.'),
  'D044903': NtO(MH='Congenital Hyperinsulinism', MS='A familial, nontransient HYPOGLYCEMIA with defects in negative feedback of GLUCOSE-regulated INSULIN release. Clinical phenotypes include HYPOGLYCEMIA; HYPERINSULINEMIA; SEIZURES; COMA; and often large BIRTH WEIGHT. Several sub-types exist with the most common, type 1, associated with mutations on an ATP-BINDING CASSETTE TRANSPORTERS (subfamily C, member 8).'),
  'D044904': NtO(MH='Mannosidase Deficiency Diseases', MS='Diseases caused by the loss of one or more enzymes involved in the hydrolysis of mannoside linkages (MANNOSIDASES). The defects in enzyme activity are primarily associated with genetic mutation of the genes that codes for a particular mannosidase isoenzyme.'),
  'D044905': NtO(MH='beta-Mannosidosis', MS='An inborn error of metabolism marked by a defect in the lysosomal isoform of BETA-MANNOSIDASE that results in lysosomal accumulation of mannose-rich intermediate metabolites containing 1,4-beta linkages. The human disease occurs through autosomal recessive inheritance and manifests in the form of a variety of symptoms that depend upon the type of gene mutation.'),
  'D044922': NtO(MH='Helicobacter felis', MS='A species of HELICOBACTER that colonizes in the STOMACH of laboratory MICE; CATS; and DOGS. It is associated with lymphoid follicular hyperplasia and mild GASTRITIS in CATS.'),
  'D044923': NtO(MH='Helicobacter hepaticus', MS='A species of HELICOBACTER that colonizes the CECUM and COLON of several strains of MICE, and is associated with HEPATITIS and carcinogenesis.'),
  'D044925': NtO(MH='Oxidoreductases Acting on CH-CH Group Donors', MS='A subclass of enzymes which includes all dehydrogenases acting on carbon-carbon bonds. This enzyme group includes all the enzymes that introduce double bonds into substrates by direct dehydrogenation of carbon-carbon single bonds.'),
  'D044926': NtO(MH='Butyryl-CoA Dehydrogenase', MS='A flavoprotein oxidoreductase that has specificity for short-chain fatty acids. It forms a complex with ELECTRON-TRANSFERRING FLAVOPROTEINS and conveys reducing equivalents to UBIQUINONE.'),
  'D044927': NtO(MH='Electron-Transferring Flavoproteins', MS='Flavoproteins that serve as specific electron acceptors for a variety of DEHYDROGENASES. They participate in the transfer of electrons to a variety of redox acceptors that occur in the respiratory chain.'),
  'D044942': NtO(MH='Acyl-CoA Dehydrogenase, Long-Chain', MS='A flavoprotein oxidoreductase that has specificity for long-chain fatty acids. It forms a complex with ELECTRON-TRANSFERRING FLAVOPROTEINS and conveys reducing equivalents to UBIQUINONE.'),
  'D044943': NtO(MH='Fatty Acid Desaturases', MS='A family of enzymes that catalyze the stereoselective, regioselective, or chemoselective syn-dehydrogenation reactions. They function by a mechanism that is linked directly to reduction of molecular OXYGEN.'),
  'D044944': NtO(MH='Acyl-CoA Dehydrogenases', MS='Enzymes that catalyze the first step in the beta-oxidation of FATTY ACIDS.'),
  'D044945': NtO(MH='Proanthocyanidins', MS='Dimers and oligomers of flavan-3-ol units (CATECHIN analogs) linked mainly through C4 to C8 bonds to leucoanthocyanidins. They are structurally similar to ANTHOCYANINS but are the result of a different fork in biosynthetic pathways.'),
  'D044946': NtO(MH='Biflavonoids', MS='Dimers (homo and hetero) of FLAVONOIDS.'),
  'D044947': NtO(MH='Flavonolignans', MS='Heterodimers of FLAVONOIDS bound to LIGNANS.'),
  'D044948': NtO(MH='Flavonols', MS='A group of 3-hydroxy-4-keto-FLAVONOIDS.'),
  'D044949': NtO(MH='Kaempferols', MS='A group of FLAVONOLS based on kaempferol. They are derived from naringenin and can be hydroxylated to QUERCETIN or reduced to leucopelargonidin.'),
  'D044950': NtO(MH='Flavanones', MS='A group of FLAVONOIDS characterized with a 4-ketone.'),
  'D044962': NtO(MH='Data Compression', MS='Information application based on a variety of coding methods to minimize the amount of data to be stored, retrieved, or transmitted. Data compression can be applied to various forms of data, such as images and signals. It is used to reduce costs and increase efficiency in the maintenance of large volumes of data.'),
  'D044963': NtO(MH='Biopsy, Fine-Needle', MS='Using fine needles (finer than 22-gauge) to remove tissue or fluid specimens from the living body for examination in the pathology laboratory and for disease diagnosis.'),
  'D044964': NtO(MH='Cervicoplasty', MS='Reconstructive surgical procedures in the NECK region to restore or improve form and function.'),
  'D044966': NtO(MH='Anti-Retroviral Agents', MS='Agents used to treat RETROVIRIDAE INFECTIONS.'),
  'D044967': NtO(MH='Serum', MS='The clear portion of BLOOD that is left after BLOOD COAGULATION to remove BLOOD CELLS and clotting proteins.'),
  'D044968': NtO(MH='Regenerative Medicine', MS='A field of medicine concerned with developing and using strategies aimed at repair or replacement of damaged, diseased, or metabolically deficient organs, tissues, and cells via TISSUE ENGINEERING; CELL TRANSPLANTATION; and ARTIFICIAL ORGANS and BIOARTIFICIAL ORGANS and tissues.'),
  'D044969': NtO(MH='Ochlerotatus', MS='A genus of mosquitoes in the family CULICIDAE. A large number of the species are found in the neotropical part of the Americas.'),
  'D045002': NtO(MH='3-alpha-Hydroxysteroid Dehydrogenase (B-Specific)', MS='A 3-hydroxysteroid dehydrogenase which catalyzes the reversible reduction of the active androgen, DIHYDROTESTOSTERONE to 5 ALPHA-ANDROSTANE-3 ALPHA,17 BETA-DIOL. It also has activity towards other 3-alpha-hydroxysteroids and on 9-, 11- and 15- hydroxyprostaglandins. The enzyme is B-specific in reference to the orientation of reduced NAD or NADPH.'),
  'D045022': NtO(MH='Lactate Dehydrogenases', MS='Alcohol oxidoreductases with substrate specificity for LACTIC ACID.'),
  'D045042': NtO(MH='Pedunculopontine Tegmental Nucleus', MS='Dense collection of cells in the caudal pontomesencephalic tegmentum known to play a role in the functional organization of the BASAL GANGLIA and in the modulation of the thalamocortical neuronal system.'),
  'D045064': NtO(MH='Methenyltetrahydrofolate Cyclohydrolase', MS='An aminohydrolase that catalyzes the hydrolysis of 5,10-methenyltetrahydrofolate to 10-formyltetrahydrofolate. In most higher eucaryotic organisms this enzyme also includes METHYLENETETRAHYDROFOLATE DEHYDROGENASE (NADP) and FORMATE-TETRAHYDROFOLATE LIGASE activities.'),
  'D045102': NtO(MH='Methylenetetrahydrofolate Dehydrogenase (NAD+)', MS='A NAD-dependent oxidoreductase that catalyzes the conversion of 5,10-methylenetetrahydrofolate to 5,10-methenyl-tetrahdyrofolate. It has been found in a variety of microorganisms.'),
  'D045103': NtO(MH='5,10-Methylenetetrahydrofolate Reductase (FADH2)', MS='An FAD-dependent oxidoreductase found primarily in BACTERIA. It is specific for the reduction of 5,10-methylenetetrahydrofolate to 5-methyltetrahydrofolate. This enzyme was formerly listed as EC 1.1.1.68 and 1.1.99.15.'),
  'D045142': NtO(MH='Cross-Priming', MS='Class I-restricted activation of CD8-POSITIVE LYMPHOCYTES resulting from ANTIGEN PRESENTATION of exogenous ANTIGENS (cross-presentation). This is in contrast to normal activation of these lymphocytes (direct-priming) which results from presentation of endogenous antigens.'),
  'D045162': NtO(MH='Thiazolidinediones', MS='THIAZOLES with two keto oxygens. Members are insulin-sensitizing agents which overcome INSULIN RESISTANCE by activation of the peroxisome proliferator activated receptor gamma (PPAR-gamma).'),
  'D045163': NtO(MH='Antibodies, Phospho-Specific', MS='Antibodies directed against immunogen-coupled phosphorylated PEPTIDES corresponding to amino acids surrounding the PHOSPHORYLATION site. They are used to study proteins involved in SIGNAL TRANSDUCTION pathways. (From Methods Mol Biol 2000; 99:177-89)'),
  'D045164': NtO(MH='Mesenchymal Stem Cell Transplantation', MS='Transfer of MESENCHYMAL STEM CELLS between individuals within the same species (TRANSPLANTATION, HOMOLOGOUS) or transfer within the same individual (TRANSPLANTATION, AUTOLOGOUS).'),
  'D045165': NtO(MH='Testosterone Congeners', MS='Steroidal compounds related to TESTOSTERONE, the major mammalian male sex hormone. Testosterone congeners include important testosterone precursors in the biosynthetic pathways, metabolites, derivatives, and synthetic steroids with androgenic activities.'),
  'D045166': NtO(MH='Estradiol Congeners', MS='Steroidal compounds related to ESTRADIOL, the major mammalian female sex hormone. Estradiol congeners include important estradiol precursors in the biosynthetic pathways, metabolites, derivatives, and synthetic steroids with estrogenic activities.'),
  'D045167': NtO(MH='Progesterone Congeners', MS='Steroidal compounds related to PROGESTERONE, the major mammalian progestational hormone. Progesterone congeners include important progesterone precursors in the biosynthetic pathways, metabolites, derivatives, and synthetic steroids with progestational activities.'),
  'D045169': NtO(MH='Severe Acute Respiratory Syndrome', MS='A viral disorder characterized by high FEVER, dry COUGH, shortness of breath (DYSPNEA) or breathing difficulties, and atypical PNEUMONIA. A virus in the genus CORONAVIRUS is the suspected agent.'),
  'D045170': NtO(MH='Matrix Attachment Regions', MS="Regions of the CHROMATIN or DNA that bind to the NUCLEAR MATRIX. They are found in INTERGENIC DNA, especially flanking the 5' ends of genes or clusters of genes. Many of the regions that have been isolated contain a bipartite sequence motif called the MAR/SAR recognition signature sequence that binds to MATRIX ATTACHMENT REGION BINDING PROTEINS."),
  'D045182': NtO(MH='Fusobacteria', MS='A phylum of anaerobic, gram-negative bacteria with a chemoorganotrophic heterotrophic metabolism. They are resident flora of the OROPHARYNX.'),
  'D045183': NtO(MH='Leptotrichia', MS='A genus of anaerobic, gram-negative bacteria in the family Fusobacteriaceae. Some species cause BACTEREMIA and some intra-amniotic infections.'),
  'D045184': NtO(MH='Propionigenium', MS='A genus of strictly anaerobic, nonsporeforming gram-negative bacteria in the family Fusobacteriaceae. Species in this genus are popularly used in biochemical research.'),
  'D045202': NtO(MH='Porphyromonas endodontalis', MS='A species of gram-negative bacteria in the genus PORPHYROMONAS, family Porphyromonadaceae. It is a key pathogen in endodontic infections.'),
  'D045222': NtO(MH='Electron Transport Chain Complex Proteins', MS='A complex of enzymes and PROTON PUMPS located on the inner membrane of the MITOCHONDRIA and in bacterial membranes. The protein complex provides energy in the form of an electrochemical gradient, which may be used by either MITOCHONDRIAL PROTON-TRANSLOCATING ATPASES or BACTERIAL PROTON-TRANSLOCATING ATPASES.'),
  'D045242': NtO(MH='Prevotella nigrescens', MS='A species of gram-negative bacteria in the family Prevotellaceae. It is the species most commonly isolated from endodontic infections (PULPITIS).'),
  'D045243': NtO(MH='Prevotella ruminicola', MS='A saccharolytic species of gram-negative bacteria in the family Prevotellaceae. It ferments a wide range of CARBOHYDRATES in the RUMEN of animals.'),
  'D045245': NtO(MH='Rhodothermus', MS='A genus of obligately aerobic, thermophilic, gram-negative bacteria in the family Crenotrichaceae. They were isolated from submarine alkaline HOT SPRINGS in Iceland.'),
  'D045246': NtO(MH='Ornithobacterium', MS='A genus of gram-negative, chemoorganotrophic bacteria, in the family FLAVOBACTERIACEAE. It is pathogenic in BIRDS.'),
  'D045247': NtO(MH='Chryseobacterium', MS='A genus of aerobic, gram-negative bacteria in the family FLAVOBACTERIACEAE. Many of its species were formerly in the genus FLAVOBACTERIUM.'),
  'D045248': NtO(MH='Flexibacter', MS='A genus of gram-negative, chemoorganotrophic bacteria in the family CYTOPHAGACEAE. In some species there is a cyclic change in cell morphology.'),
  'D045249': NtO(MH='Sphingobacterium', MS='A genus of gram-negative, chemoorganotrophic bacteria in the family Sphingobacteriaceae. They lack FLAGELLA but some species exhibit sliding motility.'),
  'D045262': NtO(MH='Reticulocytosis', MS='An increase in circulating RETICULOCYTES, which is among the simplest and most reliable signs of accelerated ERYTHROCYTE production. Reticulocytosis occurs during active BLOOD regeneration (stimulation of red bone marrow) and in certain types of ANEMIA, particularly CONGENITAL HEMOLYTIC ANEMIA.'),
  'D045263': NtO(MH='Cytochromes a1', MS='A subclass of heme a containing cytochromes have a reduced alpha-band absorption of 587-592 nm. They are primarily found in microorganisms.'),
  'D045264': NtO(MH='Cytochromes a', MS='A subclass of heme a containing cytochromes that have two imidazole nitrogens as axial ligands and an alpha-band absorption of 605 nm. They are found in a variety of microorganisms and in eucaryotes as a low-spin cytochrome component of MITOCHONDRIAL ELECTRON TRANSPORT COMPLEX IV.'),
  'D045265': NtO(MH='Natriuretic Peptides', MS='Peptides that regulate the WATER-ELECTROLYTE BALANCE in the body, also known as natriuretic peptide hormones. Several have been sequenced (ATRIAL NATRIURETIC FACTOR; BRAIN NATRIURETIC PEPTIDE; C-TYPE NATRIURETIC PEPTIDE).'),
  'D045266': NtO(MH='Cytochromes a3', MS='A subclass of heme a containing cytochromes with an alpha-band absorption of 605 nm. They are found in a variety of microorganisms and in eukaryotes as a high-spin cytochrome component of MITOCHONDRIAL ELECTRON TRANSPORT COMPLEX IV.'),
  'D045268': NtO(MH='Flowmeters', MS='Devices used to measure the flow of fluids (see RHEOLOGY) or the AIR to measure RESPIRATION.'),
  'D045282': NtO(MH='Bilophila', MS='A genus of gram-negative, anaerobic bacteria in the family DESULFOVIBRIONACEAE. It was originally recovered from infections in patients with gangrenous and PERFORATED APPENDICITIS. It is also associated with ABSCESS; BACTEREMIA; and BILIARY TRACT sepsis.'),
  'D045283': NtO(MH='Natriuretic Agents', MS='Endogenous or exogenous chemicals that regulate the WATER-ELECTROLYTE BALANCE in the body. They consist of peptides and non-peptide compounds.'),
  'D045284': NtO(MH='Desulfovibrio africanus', MS='A species of gram-negative, anaerobic, sigmoid-shaped bacteria isolated from salt and freshwater in Africa.'),
  'D045285': NtO(MH='Desulfovibrio desulfuricans', MS='The type species of gram-negative, anaerobic bacteria of the genus DESULFOVIBRIO. It is found in FRESHWATER; SOIL, and in marine or brackish water.'),
  'D045286': NtO(MH='Desulfovibrio gigas', MS='A species of gram-negative, anaerobic, spiral-shaped bacteria originally isolated from a saltwater pond in France. It contains a well-characterized metabolic pathway that enables it to survive transient contacts with OXYGEN.'),
  'D045287': NtO(MH='Desulfuromonas', MS='A genus of gram-negative, anaerobic, nonsporeforming bacteria in the family Desulfuromonadaceae. It is found in anoxic marine sediments.'),
  'D045302': NtO(MH='Geobacter', MS='A genus of gram-negative, anaerobic, metal-reducing bacteria in the family Geobacteraceae. They have the ability to oxidize a variety of organic compounds, including AROMATIC HYDROCARBONS.'),
  'D045303': NtO(MH='Cytochromes b', MS='Cytochromes of the b group that have alpha-band absorption of 563-564 nm. They occur as subunits in MITOCHONDRIAL ELECTRON TRANSPORT COMPLEX III.'),
  'D045304': NtO(MH='Cytochromes c', MS='Cytochromes of the c type that are found in eukaryotic MITOCHONDRIA. They serve as redox intermediates that accept electrons from MITOCHONDRIAL ELECTRON TRANSPORT COMPLEX III and transfer them to MITOCHONDRIAL ELECTRON TRANSPORT COMPLEX IV.'),
  'D045305': NtO(MH="Cytochromes c'", MS='A widely occurring subclass of c type cytochromes which function as electron carriers in the electron transport chain in photosynthetic and denitrifying BACTERIA.'),
  'D045322': NtO(MH='Photosynthetic Reaction Center Complex Proteins', MS='Protein complexes that take part in the process of PHOTOSYNTHESIS. They are located within the THYLAKOID MEMBRANES of plant CHLOROPLASTS and a variety of structures in more primitive organisms. There are two major complexes involved in the photosynthetic process called PHOTOSYSTEM I and PHOTOSYSTEM II.'),
  'D045325': NtO(MH='HCT116 Cells', MS='Human COLORECTAL CARCINOMA cell line.'),
  'D045331': NtO(MH='Photosystem I Protein Complex', MS='A large multisubunit protein complex that is found in the THYLAKOID MEMBRANE. It uses light energy derived from LIGHT-HARVESTING PROTEIN COMPLEXES to drive electron transfer reactions that result in either the reduction of NADP to NADPH or the transport of PROTONS across the membrane.'),
  'D045332': NtO(MH='Photosystem II Protein Complex', MS='A large multisubunit protein complex found in the THYLAKOID MEMBRANE. It uses light energy derived from LIGHT-HARVESTING PROTEIN COMPLEXES to catalyze the splitting of WATER into DIOXYGEN and of reducing equivalents of HYDROGEN.'),
  'D045333': NtO(MH='Nanotubes, Peptide', MS='NANOTUBES formed from cyclic peptides (PEPTIDES, CYCLIC). Alternating D and L linkages create planar rings that self assemble by stacking into nanotubes. They can form pores through CELL MEMBRANE causing damage to cells.'),
  'D045342': NtO(MH='Light-Harvesting Protein Complexes', MS='Complexes containing CHLOROPHYLL and other photosensitive molecules. They serve to capture energy in the form of PHOTONS and are generally found as components of the PHOTOSYSTEM I PROTEIN COMPLEX or the PHOTOSYSTEM II PROTEIN COMPLEX.'),
  'D045346': NtO(MH='Cytochrome b6f Complex', MS='A protein complex that includes CYTOCHROME B6 and CYTOCHROME F. It is found in the THYLAKOID MEMBRANE and plays an important role in process of PHOTOSYNTHESIS by transferring electrons from PLASTOQUINONE to PLASTOCYANIN or CYTOCHROME C6. The transfer of electrons is coupled to the transport of PROTONS across the membrane.'),
  'D045347': NtO(MH='Cytochromes b6', MS='Cytochromes of the b group that are found as components of the CYTOCHROME B6F COMPLEX. They contain two non-covalently bound HEME B groups.'),
  'D045348': NtO(MH='Cytochromes f', MS='Cytochromes f are found as components of the CYTOCHROME B6F COMPLEX. They play important role in the transfer of electrons from PHOTOSYSTEM I to PHOTOSYSTEM II.'),
  'D045362': NtO(MH='Cytochromes c2', MS='Type C cytochromes that are small (12-14 kD) single-heme proteins. They function as mobile electron carriers between membrane-bound enzymes in photosynthetic BACTERIA.'),
  'D045363': NtO(MH='Pneumocystis carinii', MS='The prototype species of PNEUMOCYSTIS infecting the laboratory rat. It was formerly called Pneumocystis carinii f. sp. carinii. Strains that infect humans and cause PNEUMOCYSTIS PNEUMONIA were originally classified as Pneumocystis jirovecii or Pneumocystis carinii f. sp. hominis.'),
  'D045365': NtO(MH='Cytochromes c6', MS='Cytochromes of the c type that are involved in the transfer of electrons from CYTOCHROME B6F COMPLEX and PHOTOSYSTEM I.'),
  'D045384': NtO(MH='Pregnancy-Associated alpha 2-Macroglobulins', MS='Large (>160 kDa) glycoproteins with electrophoretic mobility of ALPHA-GLOBULINS, found in high concentration in the plasma of pregnant women but also in a number of species including mouse, rat, and others (ALPHA-MACROGLOBULINS). In humans, they are always present in the circulation and are nonspecific inhibitors of PROTEINASES, similar to COMPLEMENT PROTEINS (C3; C4; C5).'),
  'D045402': NtO(MH='Parainfluenza Virus 5', MS='A species of RUBULAVIRUS originally isolated from cultured primary monkey cells. Its natural host is the DOG in which it causes kennel cough, but it can also infect humans.'),
  'D045403': NtO(MH='Henipavirus', MS='A genus in the subfamily PARAMYXOVIRINAE, family PARAMYXOVIRIDAE. It contains species that infect and cause potentially fatal disease in a number of host species, including humans.'),
  'D045404': NtO(MH='Hendra Virus', MS='A species of HENIPAVIRUS first identified in Australia in 1994 in HORSES and transmitted to humans. The natural host appears to be fruit bats (PTEROPUS).'),
  'D045405': NtO(MH='Nipah Virus', MS='A species of HENIPAVIRUS, closely related to HENDRA VIRUS, which emerged in Peninsular Malaysia in 1998. It causes a severe febrile VIRAL ENCEPHALITIS in humans and also encephalitis and RESPIRATORY TRACT INFECTIONS in pigs. Fruit bats (PTEROPUS) are the natural host.'),
  'D045422': NtO(MH='Continuous Positive Airway Pressure', MS='A technique of respiratory therapy, in either spontaneously breathing or mechanically ventilated patients, in which airway pressure is maintained above atmospheric pressure throughout the respiratory cycle by pressurization of the ventilatory circuit. (On-Line Medical Dictionary [Internet]. Newcastle upon Tyne(UK): The University Dept. of Medical Oncology: The CancerWEB Project; c1997-2003 [cited 2003 Apr 17]. Available from: http://cancerweb.ncl.ac.uk/omd/)'),
  'D045423': NtO(MH='Microbubbles', MS='Small encapsulated gas bubbles (diameters of micrometers) that can be used as CONTRAST MEDIA, and in other diagnostic and therapeutic applications. Upon exposure to sufficiently intense ultrasound, microbubbles will cavitate, rupture, disappear, release gas content. Such characteristics of the microbubbles can be used to enhance diagnostic tests, dissolve blood clots, and deliver drugs or genes for therapy.'),
  'D045424': NtO(MH='Complex Mixtures', MS='Mixtures of many components in inexact proportions, usually natural, such as PLANT EXTRACTS; VENOMS; and MANURE. These are distinguished from DRUG COMBINATIONS which have only a few components in definite proportions.'),
  'D045442': NtO(MH='Rotarod Performance Test', MS='A performance test based on forced MOTOR ACTIVITY on a rotating rod, usually by a rodent. Parameters include the riding time (seconds) or endurance. Test is used to evaluate balance and coordination of the subjects, particular in experimental animal models for neurological disorders and drug effects.'),
  'D045462': NtO(MH='Endothelium-Dependent Relaxing Factors', MS='Paracrine substances produced by the VASCULAR ENDOTHELIUM with VASCULAR SMOOTH MUSCLE relaxation (VASODILATION) activities. Several factors have been identified, including NITRIC OXIDE and PROSTACYCLIN.'),
  'D045463': NtO(MH='Avulavirus Infections', MS='Infections with viruses of the genus AVULAVIRUS, family PARAMYXOVIRIDAE. This includes NEWCASTLE DISEASE and other infections of domestic fowl.'),
  'D045464': NtO(MH='Henipavirus Infections', MS='Infections with viruses of the genus HENIPAVIRUS, family PARAMYXOVIRIDAE.'),
  'D045465': NtO(MH='Corsiaceae', MS='A plant family of the order Liliales (or by some as Dioscoreales or Burmanniales), subclass Liliidae, class Liliopsida (monocotyledons). They are perennial saprophytic plants obtaining their carbon from FUNGI.'),
  'D045466': NtO(MH='Dilleniaceae', MS='A plant family of the order Dilleniales, subclass Dilleniidae, class Magnoliopsida.'),
  'D045467': NtO(MH='Elaeocarpaceae', MS='A plant family of the order Malvales, subclass Dilleniidae, class Magnoliopsida.'),
  'D045468': NtO(MH='Fallopia japonica', MS='A plant species of the family POLYGONACEAE. Itadori tea is prepared from the root of this genus.'),
  'D045469': NtO(MH='Chrysobalanaceae', MS='A plant family of the order ROSALES, subclass Rosidae, class Magnoliopsida.'),
  'D045470': NtO(MH='Rubia', MS='A plant genus of the family RUBIACEAE. The root is a source of red dyes (madder color and 1,2,4-trihydroxy-9,10-anthracenedione) and ANTHRAQUINONES.'),
  'D045471': NtO(MH='Aegle', MS='A plant genus of the family RUTACEAE.'),
  'D045473': NtO(MH='SARS Virus', MS='A species of CORONAVIRUS causing atypical respiratory disease (SEVERE ACUTE RESPIRATORY SYNDROME) in humans. The organism is believed to have first emerged in Guangdong Province, China, in 2002. The natural host is the Chinese horseshoe bat, RHINOLOPHUS sinicus.'),
  'D045482': NtO(MH='Hot Springs', MS='Habitat of hot water naturally heated by underlying geologic processes. Surface hot springs have been used for BALNEOLOGY. Underwater hot springs are called HYDROTHERMAL VENTS.'),
  'D045483': NtO(MH='Rivers', MS='Large natural streams of FRESH WATER formed by converging tributaries and which empty into a body of water (lake or ocean).'),
  'D045502': NtO(MH='Helicobacter mustelae', MS='A species of HELICOBACTER commonly associated with STOMACH DISEASES in FERRETS.'),
  'D045503': NtO(MH='Bartonella bacilliformis', MS='The type species of the genus BARTONELLA, a gram-negative bacteria found in humans. It is found in the mountain valleys of Peru, Ecuador, and Southwest Columbia where the sandfly (see PHLEBOTOMUS) vector is present. It causes OROYA FEVER and VERRUGA PERUANA.'),
  'D045504': NtO(MH='Molecular Mechanisms of Pharmacological Action', MS='Pharmacological activities at the molecular level of DRUGS and other exogenous compounds that are used to treat DISEASES and affect normal BIOCHEMISTRY.'),
  'D045505': NtO(MH='Physiological Effects of Drugs', MS='Activities which affect organs and systemic functions without regard to a particular disease.'),
  'D045506': NtO(MH='Therapeutic Uses', MS='Uses of chemicals which affect the course of conditions, diseases, syndromes or pathology to benefit the health of an individual.'),
  'D045522': NtO(MH='Recombinases', MS='A broad category of enzymes that are involved in the process of GENETIC RECOMBINATION.'),
  'D045524': NtO(MH='Phycobilisomes', MS='Light energy harvesting structures attached to the THYLAKOID MEMBRANES of CYANOBACTERIA and RED ALGAE. These multiprotein complexes contain pigments (PHYCOBILIPROTEINS) that transfer light energy to chlorophyll a.'),
  'D045542': NtO(MH='PQQ Cofactor', MS='A pyrrolo-quinoline having two adjacent keto-groups at the 4 and 5 positions and three acidic carboxyl groups. It is a coenzyme of some DEHYDROGENASES.'),
  'D045563': NtO(MH='Indolequinones', MS='INDOLES which have two keto groups forming QUINONES like structures of the indole aromatic ring.'),
  'D045564': NtO(MH='Transposon Resolvases', MS='Recombinases that involved in the propagation of DNA TRANSPOSONS. They bind to transposon sequences found at two different sites along the same stretch of DNA and bring them into close proximity. The enzymes then catalyze the double strand cleavage, exchange of double strands and rejoining of DNA helices so that the DNA transposon is formed into a circular PLASMID.'),
  'D045565': NtO(MH='Holliday Junction Resolvases', MS='Enzymes that recognize CRUCIFORM DNA structures and introduce paired incisions that help to resolve the structure into two DNA helices.'),
  'D045566': NtO(MH='DNA, Cruciform', MS='A cross-shaped DNA structure that can be observed under the electron microscope. It is formed by the incomplete exchange of strands between two double-stranded helices or by complementary INVERTED REPEAT SEQUENCES that refold into hairpin loops on opposite strands across from each other.'),
  'D045567': NtO(MH='VDJ Recombinases', MS='Recombinases involved in the rearrangement of immunity-related GENES such as IMMUNOGLOBULIN GENES and T-CELL RECEPTOR GENES.'),
  'D045568': NtO(MH='Tetraoxanes', MS='Compounds with two peroxide groups, that is, two pairs of adjacent OXYGEN atoms. They may have activity against PLASMODIUM similar to the ARTEMISININS.'),
  'D045582': NtO(MH='Phytoplasma', MS="A genus of minute bacteria in the family ACHOLEPLASMATACEAE that inhabit phloem sieve elements of infected PLANTS and cause symptoms such as yellowing, phyllody, and witches' brooms. Organisms lack a CELL WALL and thus are similar to MYCOPLASMA in animals. They are transmitted by over 100 species of INSECTS especially leafhoppers, planthoppers, and PSYLLIDS."),
  'D045584': NtO(MH='Chromosome Positioning', MS='The mechanisms of eukaryotic CELLS that place or keep the CHROMOSOMES in a particular SUBNUCLEAR SPACE.'),
  'D045585': NtO(MH='Flap Endonucleases', MS="Endonucleases that remove 5' DNA sequences from a DNA structure called a DNA flap. The DNA flap structure occurs in double-stranded DNA containing a single-stranded break where the 5' portion of the downstream strand is too long and overlaps the 3' end of the upstream strand. Flap endonucleases cleave the downstream strand of the overlap flap structure precisely after the first base-paired nucleotide, creating a ligatable nick."),
  'D045586': NtO(MH='Intranuclear Inclusion Bodies', MS='Circumscribed masses of foreign or metabolically inactive materials, within the CELL NUCLEUS. Some are VIRAL INCLUSION BODIES.'),
  'D045602': NtO(MH='Steatorrhea', MS='A condition that is characterized by chronic fatty DIARRHEA, a result of abnormal DIGESTION and/or INTESTINAL ABSORPTION of FATS.'),
  'D045603': NtO(MH='Dioncophyllaceae', MS='A plant family of the order Violales, subclass Dilleniidae class Magnoliopsida. Members contain naphthylisoquinolines which inhibit PLASMODIUM.'),
  'D045604': NtO(MH='Extracellular Fluid', MS='The fluid of the body that is outside of CELLS. It is the external environment for the cells.'),
  'D045622': NtO(MH='Entomoplasmataceae', MS='A family of gram-negative, nonhelical, arthropod-associated bacteria in the order Entomoplasmatales. It is comprised of two genera: Entomoplasma and Mesoplasma.'),
  'D045623': NtO(MH='Mycoplasma capricolum', MS='A genus in the family ENTOMOPLASMATACEAE, order Entomoplasmatales. It is pathogenic to GOATS, causing caprine pleuropneumonia (PLEUROPNEUMONIA, CONTAGIOUS).'),
  'D045642': NtO(MH='Spiroplasma citri', MS='The type species of gram-negative bacteria in the genus SPIROPLASMA, family SPIROPLASMATACEAE, causing citrus stubborn disease.'),
  'D045643': NtO(MH='DNA Repair Enzymes', MS='Enzymes that are involved in the reconstruction of a continuous two-stranded DNA molecule without mismatch from a molecule, which contained damaged regions.'),
  'D045644': NtO(MH='Mycoplasma agalactiae', MS='A species of gram-negative bacteria causing contagious agalactia of SHEEP and GOATS.'),
  'D045645': NtO(MH='Mycoplasma arthritidis', MS='A species of gram-negative bacteria causing purulent POLYARTHRITIS in RATS.'),
  'D045646': NtO(MH='Mycoplasma bovigenitalium', MS='A species of gram-negative bacteria pathogenic to CATTLE.'),
  'D045647': NtO(MH='DNA Glycosylases', MS='A family of DNA repair enzymes that recognize damaged nucleotide bases and remove them by hydrolyzing the N-glycosidic bond that attaches them to the sugar backbone of the DNA molecule. The process called BASE EXCISION REPAIR can be completed by a DNA-(APURINIC OR APYRIMIDINIC SITE) LYASE which excises the remaining RIBOSE sugar from the DNA.'),
  'D045662': NtO(MH='Dental Etching', MS='Preparation of TOOTH surfaces, and of materials bonded to teeth or DENTAL IMPLANTS, with agents and methods which roughen the surface to facilitate adhesion. Agents include phosphoric or other acids (ACID ETCHING, DENTAL) and methods include LASERS.'),
  'D045663': NtO(MH='Quantum Dots', MS='Nanometer sized fragments of semiconductor crystalline material which emit PHOTONS. The wavelength is based on the quantum confinement size of the dot. They can be embedded in MICROBEADS for high throughput ANALYTICAL CHEMISTRY TECHNIQUES.'),
  'D045664': NtO(MH='Proprotein Convertase 1', MS='A CALCIUM-dependent endopeptidase that has specificity for cleavage at ARGININE that is near paired basic residues. It cleaves a variety of prohormones including PRO-OPIOMELANOCORTIN; PRORENIN; proenkephalins; prodynorphin; prosomatostatin; and PROINSULIN.'),
  'D045682': NtO(MH='Mycoplasma bovis', MS='A species of gram-negative bacteria causing MASTITIS; ARTHRITIS; and RESPIRATORY TRACT DISEASES in CATTLE.'),
  'D045683': NtO(MH='Furin', MS='A proprotein convertase with specificity for the proproteins of PROALBUMIN; COMPLEMENT 3C; and VON WILLEBRAND FACTOR. It has specificity for cleavage near paired ARGININE residues that are separated by two amino acids.'),
  'D045684': NtO(MH='Mycoplasma conjunctivae', MS='A species of gram-negative bacteria causing CONJUNCTIVITIS and KERATOCONJUNCTIVITIS in SHEEP and GOATS.'),
  'D045685': NtO(MH='Mycoplasma dispar', MS='A species of gram-negative bacteria causing PNEUMONIA, MYCOPLASMA in calves. Mycoplasma dispar also possibly plays a synergistic role with other BACTERIA and VIRUSES to cause other RESPIRATORY TRACT DISEASES in calves.'),
  'D045686': NtO(MH='Mycoplasma gallisepticum', MS='A species of gram-negative bacteria causing chronic respiratory disease in POULTRY.'),
  'D045702': NtO(MH='Proprotein Convertase 5', MS='A serine endopeptidase found primarily in the EXTRACELLULAR MATRIX. It has specificity for cleavage of a variety of substrates including PRORENIN, pro-membrane type-1 matrix metalloproteinase, and NEURAL CELL ADHESION MOLECULE L1.'),
  'D045703': NtO(MH='Emulsifying Agents', MS='SURFACE-ACTIVE AGENTS that induce a dispersion of undissolved material throughout a liquid.'),
  'D045704': NtO(MH='Mycoplasma genitalium', MS='A species of gram-negative bacteria originally isolated from urethral specimens of patients with non-gonoccocal URETHRITIS. In primates it exists in parasitic association with ciliated EPITHELIAL CELLS in the genital and respiratory tracts.'),
  'D045705': NtO(MH='Mycoplasma hyopneumoniae', MS='A species of gram-negative bacteria that causes MYCOPLASMA PNEUMONIA OF SWINE. The organism damages the CILIA in the airways of the pig, and thus compromises one of the most effective mechanical barriers against invading pathogens. The resulting weakening of the IMMUNE SYSTEM can encourage secondary infections, leading to porcine respiratory disease complex.'),
  'D045706': NtO(MH='Proprotein Convertase 2', MS='A serine endopeptidase that has specificity for cleavage at ARGININE. It cleaves a variety of prohormones including PRO-OPIOMELANOCORTIN, proluteinizing-hormone-releasing hormone, proenkephalins, prodynorphin, and PROINSULIN.'),
  'D045722': NtO(MH='Porcine Respiratory Coronavirus', MS='A mutant strain of TRANSMISSIBLE GASTROENTERITIS VIRUS causing mild or subclinical respiratory infections in young SWINE. It may also play a role in post-weaning porcine respiratory disease complex, especially when combined with other respiratory agents.'),
  'D045723': NtO(MH='Diverticulosis, Esophageal', MS='A pathological condition characterized by the presence of a number of ESOPHAGEAL DIVERTICULA in the ESOPHAGUS.'),
  'D045724': NtO(MH='Diverticulosis, Stomach', MS='A pathological condition characterized by the presence of a number of GASTRIC DIVERTICULA in the STOMACH.'),
  'D045725': NtO(MH='Tetrapyrroles', MS='Four PYRROLES joined by one-carbon units linking position 2 of one to position 5 of the next. The conjugated bond system results in PIGMENTATION.'),
  'D045726': NtO(MH='Metalloproteases', MS='Proteases which use a metal, normally ZINC, in the catalytic mechanism. This group of enzymes is inactivated by metal CHELATORS.'),
  'D045727': NtO(MH='Metalloexopeptidases', MS='EXOPEPTIDASES which use a metal such as ZINC in the catalytic mechanism.'),
  'D045728': NtO(MH='Corrinoids', MS='Cyclic TETRAPYRROLES based on the corrin skeleton.'),
  'D045729': NtO(MH='Pneumonia of Swine, Mycoplasmal', MS='A chronic, clinically mild, infectious pneumonia of PIGS caused by MYCOPLASMA HYOPNEUMONIAE. Ninety percent of swine herds worldwide are infected with this economically costly disease that primarily affects animals aged two to six months old. The disease can be associated with porcine respiratory disease complex. PASTEURELLA MULTOCIDA is often found as a secondary infection.'),
  'D045730': NtO(MH='Soy Foods', MS='Foods made from SOYBEANS. Soy foods are high in DIETARY PROTEINS and PHYTOESTROGENS such as ISOFLAVONES.'),
  'D045743': NtO(MH='Scleroderma, Diffuse', MS='A rapid onset form of SYSTEMIC SCLERODERMA with progressive widespread SKIN thickening over the arms, the legs and the trunk, resulting in stiffness and disability.'),
  'D045744': NtO(MH='Cell Line, Tumor', MS='A cell line derived from cultured tumor cells.'),
  'D045745': NtO(MH='Scleroderma, Limited', MS='The least progressive form of SYSTEMIC SCLERODERMA with skin thickening restricted to the face, neck and areas distal to the elbows and/or knees, sparing the trunk. The CREST SYNDROME is a form of limited scleroderma.'),
  'D045762': NtO(MH='Enzymes and Coenzymes', MS='Biological catalysts and their cofactors.'),
  'D045782': NtO(MH='Hemiterpenes', MS='Compounds that include a single 2-methyl-1,3-butadiene (also known as isoprene) within their structure.'),
  'D045783': NtO(MH='Clove Oil', MS='An oil from flower buds of SYZYGIUM trees which contains large amounts of EUGENOL.'),
  'D045784': NtO(MH='Abietanes', MS='A subclass of  cyclical DITERPENES that contain the abietane ring structure.'),
  'D045785': NtO(MH='Diterpenes, Clerodane', MS='A group of DITERPENES cyclized into 2-rings with a side-chain.'),
  'D045786': NtO(MH='Diterpenes, Kaurane', MS='A group of DITERPENES cyclized into four rings.'),
  'D045787': NtO(MH='Sesquiterpenes, Eudesmane', MS='SESQUITERPENES cyclized into two adjoining cyclohexane rings but with a different configuration from the ARTEMISININS.'),
  'D045788': NtO(MH='Sesquiterpenes, Germacrane', MS='SESQUITERPENES cyclized to one 10-carbon ring.'),
  'D045789': NtO(MH='Sesquiterpenes, Guaiane', MS='SESQUITERPENES cyclized into two adjoining rings, one being 7-carbons and the other is 5-carbons.'),
  'D045790': NtO(MH='Mycoplasma hyorhinis', MS='A species of gram-negative bacteria and a common inhabitant of the NASAL CAVITY of both healthy and diseased PIGS. It is a common secondary invader in MYCOPLASMAL PNEUMONIA OF SWINE.'),
  'D045791': NtO(MH='Mycoplasma hyosynoviae', MS='A species of gram-negative bacteria isolated from the SYNOVIAL FLUID; LYMPH NODES; and MUCOUS MEMBRANE secretions in diseased SWINE. It causes nonsuppurative ARTHRITIS.'),
  'D045792': NtO(MH='Norisoprenoids', MS='Thirteen-carbon butene cyclohexene degradation products formed by the cleavage of CAROTENOIDS. They contribute to the flavor of some FRUIT. Ionone should not be confused with the similarly named ionol.'),
  'D045793': NtO(MH='Mycoplasma iowae', MS='A species of gram-negative bacteria causing lesions to AIR SACS and HOCK joints in TURKEYS.'),
  'D045794': NtO(MH='Mycoplasma meleagridis', MS='A species of gram-negative bacteria producing air sacculitis and skeletal abnormalities in TURKEYS.'),
  'D045795': NtO(MH='Mycoplasma orale', MS='A species of gram-negative bacteria that is a common inhabitant of the human OROPHARYNX and also cell culture contaminant. It is usually non-pathogenic but it can cause infections in leukemic or immunosuppressed patients.'),
  'D045802': NtO(MH='Mycoplasma ovipneumoniae', MS='A species of gram-negative bacteria originally isolated from the LUNGS; TRACHEA; and NASAL CAVITY of SHEEP. It causes chronic interstitial pneumonia (PNEUMONIA, MYCOPLASMA) in both SHEEP and GOATS.'),
  'D045803': NtO(MH='Mycoplasma pulmonis', MS='A species of gram-negative bacteria highly pathogenic to RATS and MICE. It is the primary cause of murine respiratory mycoplasmosis.'),
  'D045804': NtO(MH='Mycoplasma salivarium', MS='A species of gram-negative bacteria strongly implicated in oral infection, PERIODONTAL DISEASES, eye and ear disorders, and SEPTIC ARTHRITIS.'),
  'D045805': NtO(MH='Mycoplasma synoviae', MS='A species of gram-negative bacteria pathogenic to CHICKENS; TURKEYS, and guinea fowls. It causes disease in a wide variety of organs and tissues including JOINTS, tendon sheaths and the RESPIRATORY TRACT.'),
  'D045808': NtO(MH='Refusal to Participate', MS='Refusal to take part in activities or procedures that are requested or expected of an individual. This may include refusal by HEALTH PERSONNEL to participate in specific medical procedures or refusal by PATIENTS or members of the public to take part in clinical trials or health promotion programs.'),
  'D045809': NtO(MH='Human Rights Abuses', MS='Deliberate maltreatment of groups of humans beings including violations of generally-accepted fundamental rights as stated by the Universal Declaration of Human Rights, adopted and proclaimed by the United Nations General Assembly resolution 217 A (III) of 10 December 1948.'),
  'D045822': NtO(MH='Intestinal Volvulus', MS='A twisting in the intestine (INTESTINES) that can cause INTESTINAL OBSTRUCTION.'),
  'D045823': NtO(MH='Ileus', MS='A condition caused by the lack of intestinal PERISTALSIS or INTESTINAL MOTILITY without any mechanical obstruction. This interference of the flow of INTESTINAL CONTENTS often leads to INTESTINAL OBSTRUCTION. Ileus may be classified into postoperative, inflammatory, metabolic, neurogenic, and drug-induced.'),
  'D045824': NtO(MH='Desulfovibrionaceae Infections', MS='Infections with bacteria of the family Desulfovibrionaceae.'),
  'D045825': NtO(MH='Fusobacteriaceae Infections', MS='Infections with bacteria of the family Fusobacteriaceae, in the order Fusobacterales, phylum FUSOBACTERIA.'),
  'D045826': NtO(MH='Flavobacteriaceae Infections', MS='Infections with bacteria of the family FLAVOBACTERIACEAE.'),
  'D045827': NtO(MH='Cytophagaceae Infections', MS='Infections with bacteria of the family CYTOPHAGACEAE.'),
  'D045828': NtO(MH='Moraxellaceae Infections', MS='Infections with bacteria of the family MORAXELLACEAE.'),
  'D045848': NtO(MH='Fibrobacter', MS='A genus of gram-negative, anaerobic bacteria in the family Fibrobacteraceae, isolated from the human GASTROINTESTINAL TRACT.'),
  'D045850': NtO(MH='Acidaminococcus', MS='A genus of gram-negative bacteria in the family ACIDAMINOCOCCACEAE, isolated from the INTESTINES of PIGS and humans.'),
  'D045851': NtO(MH='Megasphaera', MS='A genus of gram-negative bacteria in the family ACIDAMINOCOCCACEAE, found in the RUMEN of SHEEP and CATTLE, and also in humans.'),
  'D045852': NtO(MH='Pectinatus', MS='A genus of gram-negative bacteria in the family ACIDAMINOCOCCACEAE, isolated from spoiled BEER and pitching yeast.'),
  'D045853': NtO(MH='Exhalation', MS='The act of BREATHING out.'),
  'D045854': NtO(MH='Veillonellaceae', MS='A family of gram-negative bacteria, in the phylum FIRMICUTES.'),
  'D045855': NtO(MH='Acetobacterium', MS='A genus of gram-negative bacteria in the family Eubacteriaceae. Species are homoacetogenic, having the ability to use CARBON DIOXIDE as an electron sink, and to reduce it producing acetate as a typical fermentation product.'),
  'D045856': NtO(MH='Butyrivibrio', MS='A species of anaerobic bacteria, in the family Lachnospiraceae, found in RUMINANTS. It is considered both gram-positive and gram-negative.'),
  'D045857': NtO(MH='Ruminococcus', MS='A genus of gram-positive bacteria in the family Lachnospiraceae that inhabits the RUMEN; LARGE INTESTINE; and CECUM of MAMMALS.'),
  'D045858': NtO(MH='Desulfitobacterium', MS='A genus of anaerobic, gram-positive bacteria in the family Peptococcaceae, that reductively dechlorinates CHLOROPHENOLS.'),
  'D045859': NtO(MH='Desulfotomaculum', MS='A genus of anaerobic bacteria in the family Peptococcaceae, widely distributed in SOIL and the RUMEN of animals. Stains gram-negative but cell wall is gram-positive type.'),
  'D045868': NtO(MH='Sorghum', MS='A plant genus of the family POACEAE. The grain is used for FOOD and for ANIMAL FEED. This should not be confused with KAFFIR LIME or with KEFIR milk product.'),
  'D045888': NtO(MH='Ganglion Cysts', MS='Nodular tumor-like lesions or mucoid flesh, arising from tendon sheaths, LIGAMENTS, or JOINT CAPSULE, especially of the hands, wrists, or feet. They are not true cysts as they lack epithelial wall. They are distinguished from SYNOVIAL CYSTS by the lack of communication with a joint cavity or the SYNOVIAL MEMBRANE.'),
  'D045908': NtO(MH='Monkeypox', MS="A viral disease infecting PRIMATES and RODENTS. Its clinical presentation in humans is similar to SMALLPOX including FEVER; HEADACHE; COUGH; and a painful RASH. It is caused by MONKEYPOX VIRUS and is usually transmitted to humans through BITES or via contact with an animal's BLOOD. Interhuman transmission is relatively low (significantly less than smallpox)."),
  'D045930': NtO(MH='Anabolic Agents', MS='These compounds stimulate anabolism and inhibit catabolism. They stimulate the development of muscle mass, strength, and power.'),
  'D045949': NtO(MH='Viverridae', MS='The family of civets which are small and medium-sized Old World carnivores, often striped or spotted.'),
  'D045968': NtO(MH='Hyaenidae', MS='A family of large terrestrial carnivores possessing long legs, coarse guard hairs and a bushy tail. It comprises hyenas and aardwolves.'),
  'D045969': NtO(MH='Mustelidae', MS='A family of terrestrial carnivores with long, slender bodies, long tails, and anal scent glands. They include badgers, weasels, martens, FERRETS; MINKS; wolverines, polecats, and OTTERS.'),
  'D045988': NtO(MH='Procyonidae', MS='A family of long-tailed terrestrial omnivores that includes RACCOONS, ringtails, and coatimundis.'),
  'D045989': NtO(MH='Felidae', MS='The cat family in the order CARNIVORA comprised of muscular, deep-chested terrestrial carnivores with a highly predatory lifestyle.'),
  'D045991': NtO(MH='Felis', MS='Genus in the family FELIDAE comprised of small felines including the domestic cat, Felis catus (CATS) and its ancestor the wild cat, Felis silvestris.'),
  'D046008': NtO(MH='Lynx', MS='A genus in the family FELIDAE comprising felines with long legs, ear tufts, and a short tail.'),
  'D046009': NtO(MH='Panthera', MS='Genus in the family FELIDAE comprised of big felines including LIONS; TIGERS; jaguars; and the leopard.'),
  'D046010': NtO(MH='Tigers', MS='The species Panthera tigris, a large feline inhabiting Asia. Several subspecies exist including the Siberian tiger and Sumatran tiger.'),
  'D046011': NtO(MH='Puma', MS='A genus in the family FELIDAE comprising one species, Puma concolor. It is a large, long-tailed, feline of uniform color. The names puma, cougar, and mountain lion are used interchangeably for this species. There are more than 20 subspecies.'),
  'D046028': NtO(MH='Canidae', MS='A family of terrestrial carnivores with long snouts and non-retractable claws. Members include COYOTES; DOGS; FOXES; JACKALS; RACCOON DOGS; and WOLVES.'),
  'D046029': NtO(MH='Coyotes', MS='The species Canis latrans in the family CANIDAE, a smaller relative of WOLVES. It is found in the Western hemisphere from Costa Rica to Alaska.'),
  'D046030': NtO(MH='Jackals', MS='Medium-sized terrestrial carnivores, in the genus Canis, family CANIDAE. Three species are recognized, two found only in Africa and one found in Africa, Europe, and Asia.'),
  'D046031': NtO(MH='Raccoon Dogs', MS='The lone species in the genus Nyctereutes, family CANIDAE. It is found in the woodland zone from southeastern Siberia to Vietnam and on the main islands of Japan.'),
  'D046032': NtO(MH='Microscopy, Energy-Filtering Transmission Electron', MS='An analytical transmission electron microscopy method using an electron microscope fitted with an energy filtering lens. The method is based on the principle that some of the ELECTRONS passing through the specimen will lose energy when they ionize inner shell electrons of the atoms in the specimen. The amount of energy loss is dependent upon the element. Analysis of the energy loss spectrum (ELECTRON ENERGY-LOSS SPECTROSCOPY) reveals the elemental composition of a specimen. It is used analytically and quantitatively to determine which, how much of, and where specific ELEMENTS are in a sample. For example, it is used for elemental mapping of PHOSPHORUS to trace the strands of NUCLEIC ACIDS in nucleoprotein complexes.'),
  'D046048': NtO(MH='Seals, Earless', MS='The family Phocidae, suborder PINNIPEDIA, order CARNIVORA, comprising the true seals. They lack external ears and are unable to use their hind flippers to walk. It includes over 18 species including the harp seal, probably the best known seal species in the world.'),
  'D046049': NtO(MH='Phoca', MS='A genus in the family of EARLESS SEALS (Phocidae) and collectively the most abundant PINNIPEDS in the Northern Hemisphere.'),
  'D046068': NtO(MH='Caulerpa', MS='A genus of toxic marine GREEN ALGAE found throughout tropical and subtropical seas. One species, Caulerpa taxifolia, is highly invasive and produces the poison caulerpenyne, deadly to marine organisms though not humans.'),
  'D046069': NtO(MH='Fur Seals', MS='A group comprised of several species of eared seals found in two genera, in the family Otariidae. In comparison to SEA LIONS, they have an especially dense wooly undercoat.'),
  'D046070': NtO(MH='Sea Lions', MS='A group comprised of several species of aquatic carnivores in different genera, in the family Otariidae. In comparison to FUR SEALS, they have shorter, less dense hair.'),
  'D046088': NtO(MH='Hearing Loss, Unilateral', MS='Partial or complete hearing loss in one ear.'),
  'D046089': NtO(MH='Hearing Loss, Mixed Conductive-Sensorineural', MS='Hearing loss due to damage or impairment of both the conductive elements (HEARING LOSS, CONDUCTIVE) and the sensorineural elements (HEARING LOSS, SENSORINEURAL) of the ear.'),
  'D046090': NtO(MH='Anseriformes', MS='An order of BIRDS comprising the waterfowl, particularly DUCKS; GEESE; swans; and screamers.'),
  'D046091': NtO(MH='Charadriiformes', MS='An order of BIRDS including over 300 species that primarily inhabit coastal waters, beaches, and marshes. They are comprised of shorebirds, gulls, and terns.'),
  'D046092': NtO(MH='Columbiformes', MS='An order of BIRDS comprised of PIGEONS; DOVES; and an extinct family that includes dodos.'),
  'D046093': NtO(MH='Falconiformes', MS='An order of diurnal BIRDS of prey, including EAGLES; HAWKS; buzzards; vultures; and falcons.'),
  'D046094': NtO(MH='Galliformes', MS='An order of heavy-bodied, largely terrestrial BIRDS including pheasants, TURKEYS, grouse, QUAIL, and CHICKENS.'),
  'D046108': NtO(MH='Palaeognathae', MS='A superorder of large, mostly flightless birds, named for their distinctive PALATE morphology. It includes the orders Apterygiformes, Casuriiformes, Dinornithiformes, RHEIFORMES; STRUTHIONIFORMES and Tinamiformes.'),
  'D046109': NtO(MH='Passeriformes', MS='A widely distributed order of perching BIRDS, including more than half of all bird species.'),
  'D046110': NtO(MH='Hypertension, Pregnancy-Induced', MS='A condition in pregnant women with elevated systolic (>140 mm Hg) and diastolic (>90 mm Hg) blood pressure on at least two occasions 6 h apart. HYPERTENSION complicates 8-10% of all pregnancies, generally after 20 weeks of gestation. Gestational hypertension can be divided into several broad categories according to the complexity and associated symptoms, such as EDEMA; PROTEINURIA; SEIZURES; abnormalities in BLOOD COAGULATION and liver functions.'),
  'D046128': NtO(MH='Fetal Therapies', MS='Prenatal interventions to correct fetal anomalies or treat FETAL DISEASES in utero. Fetal therapies include several major areas, such as open surgery; FETOSCOPY; pharmacological therapy; INTRAUTERINE TRANSFUSION; STEM CELL TRANSPLANTATION; and GENETIC THERAPY.'),
  'D046129': NtO(MH='Ebola Vaccines', MS='Vaccines or candidate vaccines used to prevent EBOLA HEMORRHAGIC FEVER.'),
  'D046148': NtO(MH='Donor Selection', MS='The procedure established to evaluate the health status and risk factors of the potential DONORS of biological materials. Donors are selected based on the principles that their health will not be compromised in the process, and the donated materials, such as TISSUES or organs, are safe for reuse in the recipients.'),
  'D046149': NtO(MH='Embryo Culture Techniques', MS='The technique of maintaining or growing mammalian EMBRYOS in vitro. This method offers an opportunity to observe EMBRYONIC DEVELOPMENT; METABOLISM; and susceptibility to TERATOGENS.'),
  'D046150': NtO(MH='Laron Syndrome', MS='An autosomal recessive disorder characterized by short stature, defective GROWTH HORMONE RECEPTOR, and failure to generate INSULIN-LIKE GROWTH FACTOR I by GROWTH HORMONE. Laron syndrome is not a form of primary pituitary dwarfism (GROWTH HORMONE DEFICIENCY DWARFISM) but the result of mutation of the human GHR gene on chromosome 5.'),
  'D046151': NtO(MH='Lingual Thyroid', MS='A condition characterized by the presence of rudimentary THYROID tissue at the base of the TONGUE. It is due to failed embryonic development and migration of thyroid tissue to its normal location. The lingual thyroid usually cannot maintain adequate hormone production thereby resulting in HYPOTHYROIDISM.'),
  'D046152': NtO(MH='Gastrointestinal Stromal Tumors', MS='All tumors in the GASTROINTESTINAL TRACT arising from mesenchymal cells (MESODERM) except those of smooth muscle cells (LEIOMYOMA) or Schwann cells (SCHWANNOMA).'),
  'D046169': NtO(MH='Dermoscopy', MS='A noninvasive technique that enables direct microscopic examination of the surface and architecture of the SKIN.'),
  'D046188': NtO(MH='Protein Modification, Translational', MS='Any of the enzymatically catalyzed modifications of the individual AMINO ACIDS of PROTEINS, and enzymatic cleavage or crosslinking of peptide chains that occur pre-translationally (on the amino acid component of AMINO ACYL TRNA), co-translationally (during the process of GENETIC TRANSLATION), or after translation is completed (POST-TRANSLATIONAL PROTEIN PROCESSING).'),
  'D046208': NtO(MH='Microchip Analytical Procedures', MS='The preparation and analysis of samples on miniaturized devices.'),
  'D046209': NtO(MH='Electrophoresis, Microchip', MS='A highly miniaturized version of ELECTROPHORESIS performed in a microfluidic device.'),
  'D046210': NtO(MH='Microfluidic Analytical Techniques', MS='Methods utilizing the principles of MICROFLUIDICS for sample handling, reagent mixing, and separation and detection of specific components in fluids.'),
  'D046228': NtO(MH='Microarray Analysis', MS='The simultaneous analysis, on a microchip, of multiple samples or targets arranged in an array format.'),
  'D046248': NtO(MH='Pyloric Stenosis, Hypertrophic', MS='Narrowing of the pyloric canal due to HYPERTROPHY of the surrounding circular muscle. It is usually seen in infants or young children.'),
  'D046249': NtO(MH='Transfer RNA Aminoacylation', MS='The conversion of uncharged TRANSFER RNA to AMINO ACYL TRNA.'),
  'D046269': NtO(MH='Reproductive Rights', MS='Reproductive rights rest on the recognition of the basic right of all couples and individuals to decide freely and responsibly the number, spacing and timing of their children and to have the information and means to do so, and the right to attain the highest standard of sexual and reproductive health. They also include the right of all to make decisions concerning reproduction free of discrimination, coercion and violence.'),
  'D046288': NtO(MH='Body Piercing', MS='The perforation of an anatomical region for the wearing of jewelry.'),
  'D046289': NtO(MH='Body Modification, Non-Therapeutic', MS='The wounding of the body or body parts by branding, cutting, piercing (BODY PIERCING), or TATTOOING as a cultural practice or expression of creativity or identity.'),
  'D046290': NtO(MH='Liver Abscess, Pyogenic', MS='Single or multiple areas of PUS due to bacterial infection within the hepatic parenchyma. It can be caused by a variety of BACTERIA, local or disseminated from infections elsewhere such as in APPENDICITIS; CHOLECYSTITIS; PERITONITIS; and after LIVER TRANSPLANTATION.'),
  'D046308': NtO(MH='Spalax', MS='A genus of blind, subterranean MOLE RATS, in the subfamily Spalacidae, family MURIDAE, used as a animal model in neurophysiology. There are at least five different species described, all found in the Ukraine.'),
  'D046328': NtO(MH='Porcupines', MS='Common name for large, quilled rodents (RODENTIA) comprised of two families: Old World porcupines (Hystricidae) and New World porcupines (Erethizontidae).'),
  'D046329': NtO(MH='Octodon', MS='A genus of diurnal rats in the family Octodonidae, found in South America. The species Octodon degus is frequently used for research.'),
  'D046330': NtO(MH='Myoxidae', MS='A family of nocturnal rodents, similar in appearance to SQUIRRELS, but smaller. There are 28 species, half of which are found in Africa.'),
  'D046348': NtO(MH='Gophers', MS='The family Geomyidae of burrowing rodents, commonly called pocket gophers. There are six genera, all found in North America.'),
  'D046349': NtO(MH='Coproporphyria, Hereditary', MS='An autosomal dominant porphyria that is due to a deficiency of COPROPORPHYRINOGEN OXIDASE in the LIVER, the sixth enzyme in the 8-enzyme biosynthetic pathway of HEME. Clinical features include both neurological symptoms and cutaneous lesions. Patients excrete increased levels of porphyrin precursors, 5-AMINOLEVULINATE and COPROPORPHYRINS.'),
  'D046350': NtO(MH='Porphyria, Variegate', MS='An autosomal dominant porphyria that is due to a deficiency of protoporphyrinogen oxidase (EC 1.3.3.4) in the LIVER, the seventh enzyme in the 8-enzyme biosynthetic pathway of HEME. Clinical features include both neurological symptoms and cutaneous lesions. Patients excrete increased levels of porphyrin precursors, COPROPORPHYRINS and protoporphyrinogen.'),
  'D046351': NtO(MH='Protoporphyria, Erythropoietic', MS='An autosomal dominant porphyria that is due to a deficiency of FERROCHELATASE (heme synthetase) in both the LIVER and the BONE MARROW, the last enzyme in the 8-enzyme biosynthetic pathway of HEME. Clinical features include mainly neurological symptoms, rarely cutaneous lesions, and elevated levels of protoporphyrin and COPROPORPHYRINS in the feces.'),
  'D046352': NtO(MH='Freezing Reaction, Cataleptic', MS='An induced response to threatening stimuli characterized by the cessation of body movements, except for those that are involved with BREATHING, and the maintenance of an immobile POSTURE.'),
  'D046353': NtO(MH='Immobility Response, Tonic', MS='An induced response to threatening stimuli characterized by complete loss of muscle strength.'),
  'D046368': NtO(MH='Crows', MS='Common name for the largest birds in the order PASSERIFORMES, family Corvidae. These omnivorous black birds comprise most of the species in the genus Corvus, along with ravens and jackdaws (which are often also referred to as crows).'),
  'D046369': NtO(MH='Finches', MS='Common name for small PASSERIFORMES in the family Fringillidae. They have a short stout bill (BEAK) adapted for crushing SEEDS. Some species of Old World finches are called CANARIES.'),
  'D046370': NtO(MH='Sparrows', MS='The family Passeridae comprised of small, mainly brown and grey seed-eating birds with conical bills.'),
  'D046388': NtO(MH='Swallows', MS='The family Hirundinidae, comprised of small BIRDS that hunt flying INSECTS while in sustained flight.'),
  'D046408': NtO(MH='Melopsittacus', MS='A genus, commonly called budgerigars, in the family PSITTACIDAE. In the United States they are considered one of the five species of PARAKEETS.'),
  'D046409': NtO(MH='Psittacula', MS='A genus in the family PSITTACIDAE containing three species of PARAKEETS: the Derbyan, the Alexandrine, and the rose-ringed.'),
  'D046410': NtO(MH='Cockatoos', MS='Large crested BIRDS in the family Cacatuidae, found in Australia, New Guinea, and islands adjacent to the Philippines. The cockatiel (species Nymphicus hollandicus) is much smaller.'),
  'D046428': NtO(MH='Hawks', MS='Common name for many members of the FALCONIFORMES order, family Accipitridae, generally smaller than EAGLES, and containing short, rounded wings and a long tail.'),
  'D046429': NtO(MH='Spheniscidae', MS='The sole family in the order Sphenisciformes, comprised of 17 species of penguins in six genera. They are flightless seabirds of the Southern Hemisphere, highly adapted for marine life.'),
  'D046448': NtO(MH='Ice Cover', MS='A thick mass of ICE formed over large regions of land; RIVERS; LAKES; ponds; or SEAWATER.'),
  'D046449': NtO(MH='Hernia, Abdominal', MS='A protrusion of abdominal structures through the retaining ABDOMINAL WALL. It involves two parts: an opening in the abdominal wall, and a hernia sac consisting of PERITONEUM and abdominal contents. Abdominal hernias include groin hernia (HERNIA, FEMORAL; HERNIA, INGUINAL) and VENTRAL HERNIA.'),
  'D046468': NtO(MH='Sexual Development', MS='The processes of anatomical and physiological changes related to sexual or reproductive functions during the life span of a human or an animal, from FERTILIZATION to DEATH. These include SEX DETERMINATION PROCESSES; SEX DIFFERENTIATION; SEXUAL MATURATION; and changes during AGING.'),
  'D046508': NtO(MH='Culture Techniques', MS='Methods of maintaining or growing biological materials in controlled laboratory conditions. These include the cultures of CELLS; TISSUES; organs; or embryo in vitro. Both animal and plant tissues may be cultured by a variety of methods. Cultures may derive from normal or abnormal tissues, and consist of a single cell type or mixed cell types.'),
  'D046509': NtO(MH='Tissue Culture Techniques', MS='A technique for maintaining or growing TISSUE in vitro, usually by DIFFUSION, perifusion, or PERFUSION. The tissue is cultured directly after removal from the host without being dispersed for cell culture.'),
  'D046528': NtO(MH='Chimerism', MS='The occurrence in an individual of two or more cell populations of different chromosomal constitutions, derived from different individuals. This contrasts with MOSAICISM in which the different cell populations are derived from a single individual.'),
  'D046529': NtO(MH='Microscopy, Electron, Transmission', MS='Electron microscopy in which the ELECTRONS or their reaction products that pass down through the specimen are imaged below the plane of the specimen.'),
  'D046548': NtO(MH='Pubic Symphysis Diastasis', MS='Separation of the PUBIC SYMPHYSIS. It is an uncommon complication of CHILDBIRTH causing postpartum PAIN, but it can also arise from other causes.'),
  'D046549': NtO(MH='Plantibodies', MS='Recombinant antibodies produced in TRANSGENIC PLANTS. The plants serve as BIOREACTORS to produce the antibodies for medical use or industrial processes.'),
  'D046568': NtO(MH='Neuroepithelial Bodies', MS='Innervated clusters of NEUROEPITHELIAL CELLS found in the LUNGS. They act as airway OXYGEN sensors, releasing regulatory PEPTIDES and SEROTONIN in response to HYPOXIA.'),
  'D046569': NtO(MH='Neuroepithelial Cells', MS='Cells of epithelial origin possessing specialized sensory functions. They include cells that are found in the TASTE BUDS; OLFACTORY MUCOSA; COCHLEA; and NEUROEPITHELIAL BODIES.'),
  'D046588': NtO(MH='Nanoarchaeota', MS='A kingdom of hyperthermophilic ARCHAEA found in diverse environments.'),
  'D046589': NtO(MH='CADASIL', MS='A familial, cerebral arteriopathy mapped to chromosome 19q12, and characterized by the presence of granular deposits in small CEREBRAL ARTERIES producing ischemic STROKE; PSEUDOBULBAR PALSY; and multiple subcortical infarcts (CEREBRAL INFARCTION). CADASIL is an acronym for Cerebral Autosomal Dominant Arteriopathy with Subcortical Infarcts and Leukoencephalopathy. CADASIL differs from BINSWANGER DISEASE by the presence of MIGRAINE WITH AURA and usually by the lack of history of arterial HYPERTENSION. (From Bradley et al, Neurology in Clinical Practice, 2000, p1146)'),
  'D046590': NtO(MH='Spectroscopy, Electron Energy-Loss', MS='A technique for analysis of the chemical composition of molecules. A substance is bombarded with monochromatic ELECTRONS. Some of the electrons passing through the specimen will lose energy when they ionize inner shell electrons of the atoms in the specimen. The energy loss is element dependent. Analysis of the energy loss spectrum reveals the elemental composition of a specimen. ENERGY-FILTERED TRANSMISSION ELECTRON MICROSCOPY is a type of electron energy loss spectroscopy carried out in electron microscopes specially outfitted to analyze the spectrum of electron energy loss.'),
  'D046608': NtO(MH='Synkinesis', MS='An involuntary movement accompanying a volitional movement. It often refers to facial movements that accompany FACIAL PARALYSIS.'),
  'D046609': NtO(MH='Libraries, Digital', MS='Libraries in which a major proportion of the resources are available in machine-readable format, rather than on paper or MICROFORM.'),
  'D046628': NtO(MH='Sphincter of Oddi Dysfunction', MS='Organic or functional motility disorder involving the SPHINCTER OF ODDI and associated with biliary COLIC. Pathological changes are most often seen in the COMMON BILE DUCT sphincter, and less commonly the PANCREATIC DUCT sphincter.'),
  'D046648': NtO(MH='Hematoma, Subdural, Intracranial', MS='Accumulation of blood in the SUBDURAL SPACE over the CEREBRAL HEMISPHERE.'),
  'D046649': NtO(MH='Hematoma, Subdural, Spinal', MS='Subdural hematoma of the SPINAL CANAL.'),
  'D046650': NtO(MH='Medical Subject Headings', MS='Controlled vocabulary thesaurus produced by the NATIONAL LIBRARY OF MEDICINE. It consists of sets of terms naming descriptors in a hierarchical structure that permits searching at various levels of specificity.'),
  'D046688': NtO(MH='Child Mortality', MS='Number of deaths of children between one year of age to 12 years of age in a given population.'),
  'D046689': NtO(MH='Fetal Mortality', MS='Number of fetal deaths with stated or presumed gestation of 20 weeks or more in a given population. Late fetal mortality is death after of 28 weeks or more.'),
  'D046690': NtO(MH='Deep Brain Stimulation', MS='Therapy for MOVEMENT DISORDERS, especially PARKINSON DISEASE, that applies electricity via stereotactic implantation of ELECTRODES in specific areas of the BRAIN such as the THALAMUS. The electrodes are attached to a neurostimulator placed subcutaneously.'),
  'D046708': NtO(MH='Amazona', MS='One of the largest genera of PARROTS, ranging from South American to Northern Mexico. Many species are commonly kept as house pets.'),
  'D046709': NtO(MH='Pattern Recognition, Physiological', MS='The analysis of a critical number of sensory stimuli or facts (the pattern) by physiological processes such as vision (PATTERN RECOGNITION, VISUAL), touch, or hearing.'),
  'D046710': NtO(MH='Agapornis', MS='A genus comprised of nine species of small PARROTS from Africa. They are noted for showing affection for their mates.'),
  'D046728': NtO(MH='Colitis, Microscopic', MS='A condition characterized by chronic watery DIARRHEA of unknown origin, a normal COLONOSCOPY but abnormal histopathology on BIOPSY. This syndrome was first described in 1980 by Read and associates. Subtypes include COLLAGENOUS COLITIS and LYMPHOCYTIC COLITIS. Both have similar clinical symptoms and are distinguishable only by histology.'),
  'D046729': NtO(MH='Colitis, Collagenous', MS='A subtype of MICROSCOPIC COLITIS, characterized by chronic watery DIARRHEA of unknown origin, a normal COLONOSCOPY but abnormal histopathology on BIOPSY. Microscopic examination of biopsy samples taken from the COLON show larger-than-normal band of subepithelial COLLAGEN.'),
  'D046730': NtO(MH='Colitis, Lymphocytic', MS='A subtype of MICROSCOPIC COLITIS, characterized by chronic watery DIARRHEA of unknown origin, a normal COLONOSCOPY but abnormal histopathology on BIOPSY. Microscopic examination of biopsy samples taken from the COLON show infiltration of LYMPHOCYTES in the superficial EPITHELIUM and the underlying connective tissue (lamina propria).'),
  'D046748': NtO(MH='Hematoma, Epidural, Spinal', MS='A rare epidural hematoma in the spinal epidural space, usually due to a vascular malformation (CENTRAL NERVOUS SYSTEM VASCULAR MALFORMATIONS) or TRAUMA. Spontaneous spinal epidural hematoma is a neurologic emergency due to a rapidly evolving compressive MYELOPATHY.'),
  'D046749': NtO(MH='Starlings', MS='The family Sturnidae, in the order PASSERIFORMES. The starling family also includes mynahs and oxpeckers.'),
  'D046768': NtO(MH='Nesidioblastosis', MS='An inherited autosomal recessive syndrome characterized by the disorganized formation of new islets in the PANCREAS and CONGENITAL HYPERINSULINISM. It is due to focal hyperplasia of pancreatic ISLET CELLS budding off from the ductal structures and forming new islets of Langerhans. Mutations in the islet cells involve the potassium channel gene KCNJ11 or the ATP-binding cassette transporter gene ABCC8, both on CHROMOSOME 11.'),
  'D046788': NtO(MH='Patellofemoral Pain Syndrome', MS='A syndrome characterized by retropatellar or peripatellar PAIN resulting from physical and biochemical changes in the patellofemoral joint. The pain is most prominent when ascending or descending stairs, squatting, or sitting with flexed knees. There is a lack of consensus on the etiology and treatment. The syndrome is often confused with (or accompanied by) CHONDROMALACIA PATELLAE, the latter describing a pathological condition of the CARTILAGE and not a syndrome.'),
  'D046789': NtO(MH='Chondromalacia Patellae', MS='A degeneration of the ARTICULAR CARTILAGE of the PATELLA, caused by a decrease in sulfated MUCOPOLYSACCHARIDES in the ground substance. When accompanied by pain, it is sometimes considered part of or confused with PATELLOFEMORAL PAIN SYNDROME.'),
  'D046790': NtO(MH='Pancreas, Exocrine', MS='The major component (about 80%) of the PANCREAS composed of acinar functional units of tubular and spherical cells. The acinar cells synthesize and secrete several digestive enzymes such as TRYPSINOGEN; LIPASE; AMYLASE; and RIBONUCLEASE. Secretion from the exocrine pancreas drains into the pancreatic ductal system and empties into the DUODENUM.'),
  'D046808': NtO(MH='Atadenovirus', MS='A genus of ADENOVIRIDAE that comprises viruses of several species of MAMMALS and BIRDS. The type species is Ovine adenovirus D.'),
  'D046809': NtO(MH='Siadenovirus', MS='A genus of ADENOVIRIDAE comprising species including viruses of frogs (FROGS AND TOADS) and TURKEYS. The type species is Frog adenovirus.'),
  'D046810': NtO(MH='Gyrovirus', MS='A genus in the family CIRCOVIRIDAE comprising the single species CHICKEN ANEMIA VIRUS.'),
  'D046828': NtO(MH='Roniviridae', MS='A family of rod-shaped viruses, in the order NIDOVIRALES. It includes one genus: Okavirus.'),
  'D046829': NtO(MH='Isavirus', MS='A genus in the family ORTHOMYXOVIRIDAE containing one species: Infectious salmon anemia virus.'),
  'D046830': NtO(MH='Tymoviridae', MS='A family of icosahedral, non-enveloped, RNA plant viruses comprised of three genera: TYMOVIRUS, Marafivirus and Maculavirus.'),
  'D046848': NtO(MH='White spot syndrome virus 1', MS='A species of DNA virus, in the genus WHISPOVIRUS, infecting PENAEID SHRIMP.'),
  'D046849': NtO(MH='Nimaviridae', MS='A family of DNA invertebrate viruses with one genus: Whispovirus.'),
  'D046868': NtO(MH='Anabaena cylindrica', MS='A species in the genus ANABAENA whose trichomes are composed of cylindrical cells.'),
  'D046869': NtO(MH='Anabaena flos-aquae', MS='A species in the genus ANABAENA containing gas vacuoles that gives buoyancy to the organism. It can form extensive blooms in FRESH WATER and is responsible for acute poisonings of various animals.'),
  'D046870': NtO(MH='Anabaena variabilis', MS='A species of ANABAENA that can form SPORES called akinetes.'),
  'D046888': NtO(MH='Tissue Array Analysis', MS='The simultaneous analysis of multiple samples of TISSUES or CELLS from BIOPSY or in vitro culture that have been arranged in an array format on slides or microchips.'),
  'D046889': NtO(MH='Cyanothece', MS='A form-genus of unicellular coccoid to rod-shaped CYANOBACTERIA, in the order Chroococcales. Three different clusters of strains from diverse habitats are included.'),
  'D046890': NtO(MH='Aphanizomenon', MS='A form-genus of planktonic CYANOBACTERIA in the order Nostocales.'),
  'D046891': NtO(MH='Cylindrospermopsis', MS='A form-genus of CYANOBACTERIA in the order Nostocales, characterized by thin trichomes, cylindrical akinetes, and terminal heterocysts.'),
  'D046908': NtO(MH='Peptide Biosynthesis, Nucleic Acid-Independent', MS='The enzymatic synthesis of PEPTIDES without an RNA template by processes that do not use the ribosomal apparatus (RIBOSOMES).'),
  'D046910': NtO(MH='Fertile Period', MS='The period in the MENSTRUAL CYCLE that is optimal for FERTILIZATION of the OVUM and yielding PREGNANCY. The duration of fertile period depends on the life span of male GAMETES within the female reproductive tract and the time of OVULATION. It usually begins a few days before ovulation and ends on the day of ovulation.'),
  'D046911': NtO(MH='Macromolecular Substances', MS='Compounds and molecular complexes that consist of very large numbers of atoms and are generally over 500 kDa in size. In biological systems macromolecular substances usually can be visualized using ELECTRON MICROSCOPY and are distinguished from ORGANELLES by the lack of a membrane structure.'),
  'D046912': NtO(MH='Multiprotein Complexes', MS='Macromolecular complexes formed from the association of defined protein subunits.'),
  'D046913': NtO(MH='Medicago truncatula', MS='A plant species of the family FABACEAE used to study GENETICS because it is DIPLOID, self fertile, has a small genome, and short generation time.'),
  'D046914': NtO(MH='Dipteryx', MS='A plant genus of the family FABACEAE. Members contain COUMARINS.'),
  'D046915': NtO(MH='Penicillin-Binding Proteins', MS='Bacterial proteins that share the property of binding irreversibly to PENICILLINS and other ANTIBACTERIAL AGENTS derived from LACTAMS. The penicillin-binding proteins are primarily enzymes involved in CELL WALL biosynthesis including MURAMOYLPENTAPEPTIDE CARBOXYPEPTIDASE; PEPTIDE SYNTHASES; TRANSPEPTIDASES; and HEXOSYLTRANSFERASES.'),
  'D046928': NtO(MH='Peptidoglycan Glycosyltransferase', MS='A hexosyltransferase involved in the transfer of disaccharide molecules to the peptidoglycan structure of the CELL WALL SKELETON. It plays an important role in the genesis of the bacterial CELL WALL.'),
  'D046929': NtO(MH='Serine-Type D-Ala-D-Ala Carboxypeptidase', MS='A carboxypeptidase that is specific for proteins that contain two ALANINE residues on their C-terminal. Enzymes in this class play an important role in bacterial CELL WALL biosynthesis.'),
  'D046930': NtO(MH='Thymus Plant', MS='A plant genus of the family LAMIACEAE best known for the thyme spice added to foods.'),
  'D046931': NtO(MH='Microcystis', MS='A form-genus of CYANOBACTERIA in the order Chroococcales. Many species are planktonic and possess gas vacuoles.'),
  'D046932': NtO(MH='Nodularia', MS='A form-genus of filamentous CYANOBACTERIA in the order Nostocales. Its members can be planktonic or benthic and the trichomes are composed of disc-shaped vegetative cells.'),
  'D046934': NtO(MH='Ginkgolides', MS='DITERPENES with three LACTONES and a unique tert-butyl group, which are found in GINKGO plants along with BILOBALIDES.'),
  'D046935': NtO(MH='Bilobalides', MS='A degraded form of GINKGOLIDES that have three LACTONES and a unique tert-butyl group which are found in GINKGO plants.'),
  'D046936': NtO(MH='Photinia', MS='A plant genus of the family ROSACEAE. The common names of chokeberry or chokecherry are also used for some species of PRUNUS.'),
  'D046937': NtO(MH='Nostoc', MS='A form-genus of CYANOBACTERIA in the order Nostocales. Trichomes composed of spherical or ovoid vegetative cells along with heterocysts and akinetes. The species form symbiotic associations with a wide range of eukaryotes.'),
  'D046938': NtO(MH='Plectonema', MS='A form-genus of CYANOBACTERIA in the order Oscillatoriales. They are non-heterocystous but may carry out NITROGEN FIXATION under microaerobic conditions. Some smaller members have undergone transfer to the form-genus Leptolyngbya.'),
  'D046939': NtO(MH='Synechocystis', MS='A form-genus of unicellular CYANOBACTERIA in the order Chroococcales. None of the strains fix NITROGEN, there are no gas vacuoles, and sheath layers are never produced.'),
  'D046940': NtO(MH='Synechococcus', MS='A form-genus of spherical to rod-shaped CYANOBACTERIA in the order Chroococcales. They contain THYLAKOIDS and are found in a wide range of habitats.'),
  'D046948': NtO(MH='Secologanin Tryptamine Alkaloids', MS='Compounds formed by condensation of secologanin with tryptamine resulting in a tetrahydro-beta-carboline which is processed further to a number of bioactive compounds. These are especially found in plants of the APOCYNACEAE; LOGANIACEAE; and RUBIACEAE families.'),
  'D046949': NtO(MH='Thermotoga neapolitana', MS='A species of extremophilic bacteria in the family Thermotogaceae. Generally anaerobic but in the presence of OXYGEN, it can produce hydrogen gas as a byproduct of metabolism.'),
  'D046950': NtO(MH='Thermoanaerobacterium', MS='A genus of gram-positive, anaerobic bacteria in the family Thermoanaerobacteriaceae. They are thermophilic and saccharolytic.'),
  'D046968': NtO(MH='Thermoanaerobacter', MS='A genus of gram-positive, anaerobic bacteria in the family Thermoanaerobacteriaceae. Cultures consist of rods interspersed with coccoid cells.'),
  'D046969': NtO(MH='Clostridium acetobutylicum', MS='A species of gram-positive bacteria in the family Clostridiaceae, used for the industrial production of SOLVENTS.'),
  'D046970': NtO(MH='Clostridium beijerinckii', MS='A species of gram-positive bacteria in the family Clostridiaceae, capable of solventogenesis, and isolated from SOIL, infected WOUNDS, fermenting OLIVES, and spoiled CANDY.'),
  'D046971': NtO(MH='Clostridium bifermentans', MS='A species of gram-positive bacteria in the family Clostridiaceae that ferments both CARBOHYDRATES and AMINO ACIDS.'),
  'D046972': NtO(MH='Clostridium butyricum', MS='Type species of the genus CLOSTRIDIUM, a gram-positive bacteria in the family Clostridiaceae. It is used as a source of PROBIOTICS.'),
  'D046973': NtO(MH='Clostridium cellulolyticum', MS='A species of gram-positive bacteria in the family Clostridiaceae. It is a cellulolytic, mesophilic species isolated from decayed GRASS.'),
  'D046974': NtO(MH='Clostridium cellulovorans', MS='A species of gram-positive, cellulolytic bacteria in the family Clostridiaceae. It produces CELLULOSOMES which are involved in plant CELL WALL degradation.'),
  'D046975': NtO(MH='Clostridium chauvoei', MS='A species of gram-positive bacteria in the family Clostridiaceae isolated from infected CATTLE; SHEEP; and other animals. It causes blackleg in cattle and sheep and is transmitted through soil-borne spores.'),
  'D046988': NtO(MH='Proteasome Endopeptidase Complex', MS='A large multisubunit complex that plays an important role in the degradation of most of the cytosolic and nuclear proteins in eukaryotic cells. It contains a 700-kDa catalytic sub-complex and two 700-kDa regulatory sub-complexes. The complex digests ubiquitinated proteins and protein activated via ornithine decarboxylase antizyme.'),
  'D047008': NtO(MH='Marchantia', MS='A liverwort plant genus of the family Marchantiaceae, order Marchantiales, subclass MARCHANTIAE. Members contain brassinosteroids and DITERPENES.'),
  'D047009': NtO(MH='Clostridium histolyticum', MS='A species of gram-positive, strongly proteolytic bacteria in the family Clostridiaceae. It contains several forms of COLLAGENASE whose action can lead to GAS GANGRENE in humans and HORSES.'),
  'D047010': NtO(MH='Clostridium kluyveri', MS='A species of gram-positive bacteria in the family Clostridiaceae. It is distinctive for its ability to ferment ETHANOL to caproic acid.'),
  'D047011': NtO(MH='Silk', MS='A continuous protein fiber consisting primarily of FIBROINS. It is synthesized by a variety of INSECTS and ARACHNIDS.'),
  'D047013': NtO(MH='Clostridium botulinum type A', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces BOTULINUM TOXINS, TYPE A which is neurotoxic to humans and animals.'),
  'D047014': NtO(MH='Clostridium botulinum type B', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces botulinum toxin type B which is neurotoxic to humans and animals.'),
  'D047028': NtO(MH='Macrocyclic Compounds', MS='Cyclic compounds with a ring size of approximately 1-4 dozen atoms.'),
  'D047029': NtO(MH='Lactams, Macrocyclic', MS='LACTAM-forming compounds with a ring size of approximately 1-3 dozen atoms.'),
  'D047030': NtO(MH='Sericins', MS='A serine-rich sticky protein secreted by MOTHS. Generally, the term refers to silkworm silk gum protein secreted in the middle section of silk gland cells of SILKWORMS, Bombyx mori. Sericin acts as a cement and coating for the two fibroin filaments in a silk strand and is readily soluble in mild alkaline solution.'),
  'D047048': NtO(MH='Vitellins', MS='Major egg yolk proteins from egg-laying animals such as non-mammalian VERTEBRATES; ARTHROPODS; and others. They are high-density lipoglycoproteins derived from circulating precursors, VITELLOGENINS. Vitellins serve as nutrients for the growing non-mammalian embryos (EMBRYO, NONMAMMALIAN).'),
  'D047068': NtO(MH='Arylalkylamine N-Acetyltransferase', MS='An acetyltransferase with specificity towards the amine group of aromatic alkylamines (arylalkylamines) such as SEROTONIN. This enzyme is also referred to as serotonin acetylase despite the fact that serotonin acetylation can also occur through the action of broad specificity acetyltransferases such as ARYLAMINE N-ACETYLTRANSFERASE.'),
  'D047069': NtO(MH='Pyrazolones', MS='Compounds with a five-membered heterocyclic ring with two nitrogens and a keto OXYGEN. Some are inhibitors of TNF-ALPHA production.'),
  'D047070': NtO(MH='Bandages, Hydrocolloid', MS='Dressings comprised of a self-adhesive matrix to which hydrophilic absorbent particles are embedded. The particles consist of CELLULOSE derivatives; calcium ALGINATES; PECTINS; or GELS. The utility is based on providing a moist environment for WOUND HEALING.'),
  'D047071': NtO(MH='beta-Glucans', MS='Glucose polymers consisting of a backbone of beta(1->3)-linked beta-D-glucopyranosyl units with beta(1->6) linked side chains of various lengths. They are a major component of the CELL WALL of organisms and of soluble DIETARY FIBER.'),
  'D047072': NtO(MH='Aromatase Inhibitors', MS='Compounds that inhibit AROMATASE in order to reduce production of estrogenic steroid hormones.'),
  'D047088': NtO(MH='Dysidea', MS='A genus of SPONGES in the family Dysideidae, in which all skeletal fibers are filled with detritus.'),
  'D047089': NtO(MH='Axinella', MS='A genus of SPONGES in the family Axinellidae, comprised of a choanosomal skeleton differentiated in the axial and extra-axial region. The type species is Axinella polypoides.'),
  'D047090': NtO(MH='beta-Lactams', MS='Four-membered cyclic AMIDES, best known for the PENICILLINS based on a bicyclo-thiazolidine, as well as the CEPHALOSPORINS based on a bicyclo-thiazine, and including monocyclic MONOBACTAMS. The BETA-LACTAMASES hydrolyze the beta lactam ring, accounting for BETA-LACTAM RESISTANCE of infective bacteria.'),
  'D047091': NtO(MH='Eosinophil Granule Proteins', MS='Proteins found in EOSINOPHIL granules. They are primarily basic proteins that play a role in host defense and the proinflammatory actions of activated eosinophils.'),
  'D047093': NtO(MH='Eosinophil Major Basic Protein', MS='One of several basic proteins released from EOSINOPHIL cytoplasmic granules. Eosinophil major basic protein is a 14-kDa cytotoxic peptide with a pI of 10.9. In addition to its direct cytotoxic effects, it stimulates the release of variety of INFLAMMATION MEDIATORS.'),
  'D047108': NtO(MH='Embryonic Development', MS='Morphological and physiological development of EMBRYOS.'),
  'D047109': NtO(MH='Fetal Development', MS='Morphological and physiological development of FETUSES.'),
  'D047128': NtO(MH='Callyspongia', MS='A genus of SPONGES in the family Callyspongiidae comprised of a regular ectosomal reticulate skeleton.'),
  'D047129': NtO(MH='Haliclona', MS='A genus of SPONGES in the family Chalinidae characterized by unispicular secondary lines in the choanosomal skeleton.'),
  'D047130': NtO(MH='Petrosia', MS='A genus of SPONGES in the family Petrosiidae, characterized by an ectosomal triangular or polygonal reticulation of spicule tracts or single spicules.'),
  'D047131': NtO(MH='Xestospongia', MS='A genus of SPONGES in the family Petrosiidae, characterized by an ectosomal skeleton consisting only of an isotropic reticulation of single spicules or spicule tracts.'),
  'D047132': NtO(MH='Crambe Sponge', MS='A genus of SPONGES in the family Crambeidae characterized by desmoid spicules. The type species is Crambe crambe.'),
  'D047150': NtO(MH='Eosinophil Cationic Protein', MS='One of several basic proteins released from EOSINOPHIL cytoplasmic granules. Eosinophil cationic protein is a 21-kDa cytotoxic peptide with a pI of 10.9. Although eosinophil cationic protein is considered a member of the RNAse A superfamily of proteins, it has only limited RNAse activity.'),
  'D047151': NtO(MH='Amaryllidaceae Alkaloids', MS='Alkaloids derived from TYRAMINE combined with 3,4-dihydroxybenzaldehyde via a norbelladine pathway, including GALANTAMINE, lycorine and crinine. They are found in the Amaryllidaceae (LILIACEAE) plant family.'),
  'D047168': NtO(MH='Cystine Knot Motifs', MS='Amino acid sequence in which two disulfide bonds (DISULFIDES) and their connecting backbone form a ring that is penetrated by a third disulfide bond. Members include CYCLOTIDES and agouti-related protein.'),
  'D047169': NtO(MH='Cyclotides', MS='A continuous circle of peptide bonds, typically of 2-3 dozen AMINO ACIDS, so there is no free N- or C-terminus. They are further characterized by six conserved CYSTEINE residues that form CYSTINE KNOT MOTIFS.'),
  'D047188': NtO(MH='Chalcones', MS='Derivatives of CHALCONE that are important intermediates in the formation of FLAVONOIDS with anti-fungal, anti-bacterial, anti-inflammatory, and anti-tumor properties.'),
  'D047208': NtO(MH='Eosinophil-Derived Neurotoxin', MS='A 19-kDa cationic peptide found in EOSINOPHIL granules. Eosinophil-derived neurotoxin is a RIBONUCLEASE and may play a role as an endogenous antiviral agent.'),
  'D047209': NtO(MH='Eosinophil Peroxidase', MS='A 66-kDa peroxidase found in EOSINOPHIL granules. Eosinophil peroxidase is a cationic protein with a pI of 10.8 and is comprised of a heavy chain subunit and a light chain subunit. It possesses cytotoxic activity towards BACTERIA and other organisms, which is attributed to its peroxidase activity.'),
  'D047228': NtO(MH='Angiotensin II Type 1 Receptor Blockers', MS='Agents that antagonize ANGIOTENSIN II TYPE 1 RECEPTOR. Included are ANGIOTENSIN II analogs such as SARALASIN and biphenylimidazoles such as LOSARTAN. Some are used as ANTIHYPERTENSIVE AGENTS.'),
  'D047229': NtO(MH='Plakortis', MS='A genus of SPONGES in the family Plakinidae, with the skeleton formed by small diactine (and some triactine) needle-like terminations.'),
  'D047230': NtO(MH='Agelas', MS='A genus of large, brightly colored SPONGES in the family Agelasidae, possessing a skeleton of spongin fibers with a core of large spicules (megascleres).'),
  'D047231': NtO(MH='Geodia', MS='A genus of massive or globular SPONGES in the family Geodiidae, with large spicules at or near the surface.'),
  'D047232': NtO(MH='Theonella', MS='A genus of shallow-cupped SPONGES with a broad base in the family Theonellidae. They are characterized by ectosomal spicules dominated by phyllotriaenes.'),
  'D047233': NtO(MH='Suberites', MS='A genus of velvety smooth SPONGES in the family Suberitidae, characterized by the ectosomal and choanosomal skeletons dominated by tylostyles (pin-like spicules with a pinched bulbous end).'),
  'D047248': NtO(MH='Crown Compounds', MS='Macrocyclic compounds analogous to CROWN ETHERS but which lack any OXYGEN atoms.'),
  'D047250': NtO(MH='Calixarenes', MS='Phenolic metacyclophanes derived from condensation of PHENOLS and ALDEHYDES. The name derives from the vase-like molecular structures. A bracketed [n] indicates the number of aromatic rings.'),
  'D047268': NtO(MH='Lingual Goiter', MS='Pathological enlargement of the LINGUAL THYROID, ectopic thyroid tissue at the base of the TONGUE. It may cause upper AIRWAY OBSTRUCTION; DYSPHAGIA; or HYPOTHYROIDISM symptoms.'),
  'D047269': NtO(MH='Strongylocentrotus', MS='A genus of SEA URCHINS in the family Strongylocentrotidae. They possess more than three pore pairs per ambulacral plate. The species STRONGYLOCENTROTUS PURPURATUS is commonly used for research.'),
  'D047270': NtO(MH='Strongylocentrotus purpuratus', MS='A species of SEA URCHINS in the family Strongylocentrotidae found on the Pacific coastline from Alaska to Mexico. This species serves as a major research model for molecular developmental biology and other fields.'),
  'D047271': NtO(MH='Asterias', MS='A genus of STARFISH in the family Asteriidae. One species, Asterias rubens, is the most common in the north-east Atlantic region.'),
  'D047288': NtO(MH='Asterina', MS='A genus of STARFISH in the family Asterinidae. They externally hold developing embryos (EMBRYO, NON-MAMMALIAN) among the spines below the oral surface.'),
  'D047308': NtO(MH='Arbacia', MS='A genus of SEA URCHINS in the family Arbaciidae. They have only one spheridium (stalked body) per ambulacral area (contains tube feet); most sea urchins have several spheridia per area.'),
  'D047309': NtO(MH='Flavones', MS='A group of 4-keto-FLAVONOIDS.'),
  'D047310': NtO(MH='Apigenin', MS="5,7,4'-trihydroxy-flavone, one of the FLAVONES."),
  'D047311': NtO(MH='Luteolin', MS="5,7,3',4'-tetrahydroxy-flavone, one of the FLAVONES."),
  'D047328': NtO(MH='Paracentrotus', MS='A genus of SEA URCHINS in the family Echinidae found primarily on the western coasts of Ireland.'),
  'D047329': NtO(MH='Anthocidaris', MS='A genus of SEA URCHINS synonymous with Heliocidaris, belonging to the family Echinometridae.'),
  'D047330': NtO(MH='Hemicentrotus', MS='A genus of SEA URCHINS in the family Strongylocentrotidae with a hemicyclic apical disk and short spines.'),
  'D047331': NtO(MH='Lytechinus', MS='A genus of SEA URCHINS in the family Toxopneustidae possessing trigeminate ambulacral plating.'),
  'D047332': NtO(MH='Holothuria', MS='A genus of large SEA CUCUMBERS in the family Holothuriidae possessing thick body walls, a warty body surface, and microscopic ossicles.'),
  'D047348': NtO(MH='Hydrolyzable Tannins', MS='Polymeric derivatives of GALLIC ACID that are esters of a sugar.'),
  'D047368': NtO(MH='Tumor Burden', MS='The total amount (cell number, weight, size or volume) of tumor cells or tissue in the body.'),
  'D047369': NtO(MH='Chromatin Immunoprecipitation', MS='A technique for identifying specific DNA sequences that are bound, in vivo, to proteins of interest. It involves formaldehyde fixation of CHROMATIN to crosslink the DNA-BINDING PROTEINS to the DNA. After shearing the DNA into small fragments, specific DNA-protein complexes are isolated by immunoprecipitation with protein-specific ANTIBODIES. Then, the DNA isolated from the complex can be identified by PCR amplification and sequencing.'),
  'D047388': NtO(MH='Casein Kinases', MS='A group of protein-serine-threonine kinases that was originally identified as being responsible for the PHOSPHORYLATION of CASEINS. They are ubiquitous enzymes that have a preference for acidic proteins. Casein kinases play a role in SIGNAL TRANSDUCTION by phosphorylating a variety of regulatory cytoplasmic and regulatory nuclear proteins.'),
  'D047389': NtO(MH='Casein Kinase I', MS='A casein kinase that was originally described as a monomeric enzyme with a molecular weight of 30-40 kDa. Several ISOENZYMES of casein kinase I have been found which are encoded by separate genes. Many of the casein kinase I isoenzymes have been shown to play distinctive roles in intracellular SIGNAL TRANSDUCTION.'),
  'D047390': NtO(MH='Casein Kinase II', MS='A ubiquitous casein kinase that is comprised of two distinct catalytic subunits and dimeric regulatory subunit. Casein kinase II has been shown to phosphorylate a large number of substrates, many of which are proteins involved in the regulation of gene expression.'),
  'D047391': NtO(MH='alpha-Cyclodextrins', MS='Cyclic GLUCANS consisting of six (6) glucopyranose units linked by 1,4-glycosidic bonds.'),
  'D047392': NtO(MH='beta-Cyclodextrins', MS='Cyclic GLUCANS consisting of seven (7) glucopyranose units linked by 1,4-glycosidic bonds.'),
  'D047408': NtO(MH='gamma-Cyclodextrins', MS='Cyclic GLUCANS consisting of eight (8) glucopyranose units linked by 1,4-glycosidic bonds.'),
  'D047428': NtO(MH='Protein Kinase Inhibitors', MS='Agents that inhibit PROTEIN KINASES.'),
  'D047468': NtO(MH='Immunoprecipitation', MS='The aggregation of soluble ANTIGENS with ANTIBODIES, alone or with antibody binding factors such as ANTI-ANTIBODIES or STAPHYLOCOCCAL PROTEIN A, into complexes large enough to fall out of solution.'),
  'D047488': NtO(MH='Retinoid X Receptors', MS='A subtype of RETINOIC ACID RECEPTORS that are specific for 9-cis-retinoic acid which function as nuclear TRANSCRIPTION FACTORS that regulate multiple signaling pathways.'),
  'D047489': NtO(MH='Preconception Injuries', MS='Damages to reproductive health prior to conception (FERTILIZATION), a legal term for torts liability concerning environmental safety issues. Preconception injuries may involve either the male or the female, such as chromosomal mutations in the OVA or the SPERMATOZOA.'),
  'D047490': NtO(MH='Retinoid X Receptor alpha', MS='A nuclear transcription factor. Heterodimerization with PPAR GAMMA is important in regulation of GLUCOSE metabolism and CELL GROWTH PROCESSES.'),
  'D047491': NtO(MH='Retinoid X Receptor gamma', MS='A nuclear transcription factor. Heterodimerization with PPAR ALPHA is important to metabolism of LIPIDS.'),
  'D047492': NtO(MH='Peroxisome Proliferator-Activated Receptors', MS='TRANSCRIPTION FACTORS that are activated by ligands and heterodimerize with RETINOID X RECEPTORS and bind to peroxisome proliferator response elements in the promoter regions of target genes.'),
  'D047493': NtO(MH='PPAR alpha', MS='A nuclear transcription factor. Heterodimerization with RETINOID X RECEPTOR GAMMA is important to metabolism of LIPIDS. It is the target of FIBRATES to control HYPERLIPIDEMIAS.'),
  'D047494': NtO(MH='PPAR delta', MS='A nuclear transcription factor. It is activated by PROSTACYCLIN.'),
  'D047495': NtO(MH='PPAR gamma', MS='A nuclear transcription factor. Heterodimerization with RETINOID X RECEPTOR ALPHA is important in regulation of GLUCOSE metabolism and CELL GROWTH PROCESSES. It is a target of THIAZOLIDINEDIONES for control of DIABETES MELLITUS.'),
  'D047508': NtO(MH='Massive Hepatic Necrosis', MS='Extensive and rapid death of parenchymal cells in the LIVER, often due to exposure to toxic materials or drug-induced injury. It is characterized by a soft, flabby, yellow-brown wrinkled, and shrunken liver. It was called "acute yellow atrophy".'),
  'D047528': NtO(MH='Stichopus', MS='A genus of very large, epibenthic SEA CUCUMBERS in the family Stichopodidae, commercially harvested in Southeast Asia for food.'),
  'D047548': NtO(MH='Defibrillators', MS="Cardiac electrical stimulators that apply brief high-voltage electroshocks to the HEART. These stimulators are used to restore normal rhythm and contractile function in hearts of patients who are experiencing VENTRICULAR FIBRILLATION or ventricular tachycardia (TACHYCARDIA, VENTRICULAR) that is not accompanied by a palpable PULSE. Some defibrillators may also be used to correct certain noncritical dysrhythmias (called synchronized defibrillation or CARDIOVERSION), using relatively low-level discharges synchronized to the patient's ECG waveform. (UMDNS, 2003)"),
  'D047549': NtO(MH='Coronary Artery Bypass, Off-Pump', MS='Coronary artery bypass surgery on a beating HEART without a CARDIOPULMONARY BYPASS (diverting the flow of blood from the heart and lungs through an oxygenator).'),
  'D047568': NtO(MH='Unsafe Sex', MS='Sexual behaviors which are high-risk for contracting SEXUALLY TRANSMITTED DISEASES or for producing PREGNANCY.'),
  'D047569': NtO(MH='Laser Scanning Cytometry', MS='A scanning microscope-based, cytofluorimetry technique for making fluorescence measurements and topographic analysis on individual cells. Lasers are used to excite fluorochromes in labeled cellular specimens. Fluorescence is detected in multiple discrete wavelengths and the locational data is processed to quantitatively assess APOPTOSIS; PLOIDIES; cell proliferation; GENE EXPRESSION; PROTEIN TRANSPORT; and other cellular processes.'),
  'D047589': NtO(MH='Leukocyte Reduction Procedures', MS='The removal of LEUKOCYTES from BLOOD to reduce BLOOD TRANSFUSION reactions and lower the chance of transmitting VIRUSES. This may be performed by FILTRATION or by CYTAPHERESIS.'),
  'D047608': NtO(MH='Pyrroloiminoquinones', MS='Derivatives of QUINOLINES with fused PYRROLES and a keto group or INDOLEQUINONES with fused PIPERIDINES.'),
  'D047628': NtO(MH='Estrogen Receptor alpha', MS='One of the ESTROGEN RECEPTORS that has marked affinity for ESTRADIOL. Its expression and function differs from, and in some ways opposes, ESTROGEN RECEPTOR BETA.'),
  'D047629': NtO(MH='Estrogen Receptor beta', MS='One of the ESTROGEN RECEPTORS that has greater affinity for ISOFLAVONES than ESTROGEN RECEPTOR ALPHA does. There is great sequence homology with ER alpha in the DNA-binding domain but not in the ligand binding and hinge domains.'),
  'D047630': NtO(MH='Depsipeptides', MS='Compounds consisting of chains of AMINO ACIDS alternating with CARBOXYLIC ACIDS via ester and amide linkages. They are commonly cyclized.'),
  'D047648': NtO(MH='Perimenopause', MS='The transitional period before and after MENOPAUSE. Perimenopausal symptoms are associated with irregular MENSTRUAL CYCLE and widely fluctuated hormone levels. They may appear 6 years before menopause and subside 2 to 5 years after menopause.'),
  'D047649': NtO(MH='Andropause', MS='An endocrine state in men, characterized by a significant decline in the production of TESTOSTERONE; DEHYDROEPIANDROSTERONE; and other hormones such as HUMAN GROWTH HORMONE. Andropause symptoms are related to the lack of androgens including DEPRESSION, sexual dysfunction, and OSTEOPOROSIS. Andropause may also result from hormonal ablation therapy for malignant diseases.'),
  'D047668': NtO(MH='Inteins', MS="The internal fragments of precursor proteins (INternal proTEINS) that are autocatalytically removed by PROTEIN SPLICING. The flanking fragments (EXTEINS) are ligated forming mature proteins. The nucleic acid sequences coding for inteins are considered to be MOBILE GENETIC ELEMENTS. Inteins are composed of self-splicing domains and an endonuclease domain which plays a role in the spread of the intein's genomic sequence. Mini-inteins are composed of the self-splicing domains only."),
  'D047669': NtO(MH='Exteins', MS='The (EXternal proTEIN) sequences flanking INTEINS, which are internal fragments of precursor proteins removed by PROTEIN SPLICING. Concomitantly, as the inteins are excised, the exteins are ligated to form mature proteins.'),
  'D047688': NtO(MH='Breast Cyst', MS='A fluid-filled closed cavity or sac that is lined by an EPITHELIUM and found in the BREAST. It may appear as a single large cyst in one breast, multifocal, or bilateral in FIBROCYSTIC BREAST DISEASE.'),
  'D047708': NtO(MH='Myofibroma', MS="A benign tumor that consists chiefly of fibrous CONNECTIVE TISSUE, with variable numbers of MUSCLE CELLS forming portions of the neoplasm (From Stedman's, 27th ed)."),
  'D047709': NtO(MH='Hirudo medicinalis', MS='A species of European freshwater LEECHES used for BLOODLETTING in ancient times and also for LEECHING in modern times.'),
  'D047728': NtO(MH='Myopia, Degenerative', MS='Excessive axial myopia associated with complications (especially posterior staphyloma and CHOROIDAL NEOVASCULARIZATION) that can lead to BLINDNESS.'),
  'D047748': NtO(MH='Pituitary ACTH Hypersecretion', MS='A disease of the PITUITARY GLAND characterized by the excess amount of ADRENOCORTICOTROPIC HORMONE secreted. This leads to hypersecretion of cortisol (HYDROCORTISONE) by the ADRENAL GLANDS resulting in CUSHING SYNDROME.'),
  'D047768': NtO(MH='Glycerophosphoinositol Inositolphosphodiesterase', MS='A phosphoric diester hydrolase with specificity for the cleavage of GLYCEROL from 1-(sn-glycero-3-phospho)-1D-myo-inositol. It also has specificity for hydrolysis of the cyclic phosphate bond in inositol 1,2-cyclic phosphate'),
  'D047788': NtO(MH='American Revolution', MS='Armed conflict that resulted in separation of American colonies from Britain and established the United States of America.'),
  'D047789': NtO(MH='World War II', MS='Global conflict involving countries of Europe, Africa, Asia, and North America that occurred between 1939 and 1945.'),
  'D047808': NtO(MH='Adrenogenital Syndrome', MS='Abnormal SEX DIFFERENTIATION or congenital DISORDERS OF SEX DEVELOPMENT caused by abnormal levels of steroid hormones expressed by the GONADS or the ADRENAL GLANDS, such as in CONGENITAL ADRENAL HYPERPLASIA and ADRENAL CORTEX NEOPLASMS. Due to abnormal steroid biosynthesis, clinical features include VIRILISM in females; FEMINIZATION in males; or precocious sexual development in children.'),
  'D047828': NtO(MH='World War I', MS='Global conflict primarily fought on European continent, that occurred between 1914 and 1918.'),
  'D047829': NtO(MH='Vietnam Conflict', MS='A conflict occurring from 1954 through 1975 within the Republic of Vietnam. It involved neighboring nations and the United States and other members of the Southeast Asia Treaty Organization.'),
  'D047830': NtO(MH='Gulf War', MS="United Nations' action to intervene in conflict between the nation of Kuwait and occupying Iraqi forces, occurring from 1990 through 1991."),
  'D047831': NtO(MH='American Civil War', MS='1861-1865 conflict between the Union (Northern states) and the 11 Southern states that seceded and were organized as the Confederate States of America.'),
  'D047832': NtO(MH='Spanish-American War, 1898', MS='Conflict between Spain and the United States, arising out of Spanish policies in Cuba.'),
  'D047848': NtO(MH='Russian-Japanese War', MS='Conflict from 1904 through 1905 between Russia and Japan regarding Manchuria and Korea.'),
  'D047849': NtO(MH='Crimean War', MS='Conflict between RUSSIA (Pre-1917); the OTTOMAN EMPIRE; ENGLAND; FRANCE; and Sardinia.'),
  'D047850': NtO(MH='French Revolution', MS='Conflict during which traditional monarchy was ended and modern government functions were instituted.'),
  'D047851': NtO(MH='September 11 Terrorist Attacks', MS='Terrorism on September 11, 2001 against targets in New York, the Pentagon in Virginia, and an aborted attack that ended in Pennsylvania.'),
  'D047852': NtO(MH='Chernobyl Nuclear Accident', MS='April 25th -26th, 1986 nuclear power accident that occurred at Chernobyl in the former USSR (Ukraine) located 80 miles north of Kiev.'),
  'D047868': NtO(MH='Pulmonary Sclerosing Hemangioma', MS='A benign neoplasm of pneumocytes, cells of the PULMONARY ALVEOLI. Originally considered to be vascular in origin, it is now classified as an epithelial tumor with several elements, including solid cellular areas, papillary structure, sclerotic regions, and dilated blood-filled spaces resembling HEMANGIOMA.'),
  'D047869': NtO(MH='Korean War', MS='An armed conflict between Communist and non-Communist forces in Korea from June 25, 1950, to July 27, 1953. The parties included United Nations forces from 15 member nations under United States command against military from North Korea and the Peoples Republic of China.'),
  'D047888': NtO(MH='Receptors, Tumor Necrosis Factor, Type I', MS='A tumor necrosis factor receptor subtype that has specificity for TUMOR NECROSIS FACTOR ALPHA and LYMPHOTOXIN ALPHA. It is constitutively expressed in most tissues and is a key mediator of tumor necrosis factor signaling in the vast majority of cells. The activated receptor signals via a conserved death domain that associates with specific TNF RECEPTOR-ASSOCIATED FACTORS in the CYTOPLASM.'),
  'D047889': NtO(MH='Receptors, Tumor Necrosis Factor, Type II', MS='A tumor necrosis factor receptor subtype that is expressed primarily in IMMUNE SYSTEM cells. It has specificity for membrane-bound form of TUMOR NECROSIS FACTORS and mediates intracellular-signaling through TNF RECEPTOR ASSOCIATED FACTORS.'),
  'D047908': NtO(MH='Intracellular Signaling Peptides and Proteins', MS='Proteins and peptides that are involved in SIGNAL TRANSDUCTION within the cell. Included here are peptides and proteins that regulate the activity of TRANSCRIPTION FACTORS and cellular processes in response to signals from CELL SURFACE RECEPTORS. Intracellular signaling peptide and proteins may be part of an enzymatic signaling cascade or act through binding to and modifying the action of other signaling factors.'),
  'D047928': NtO(MH='Premature Birth', MS="CHILDBIRTH before 37 weeks of PREGNANCY (259 days from the first day of the mother's last menstrual period, or 245 days after FERTILIZATION)."),
  'D047929': NtO(MH='Term Birth', MS="CHILDBIRTH at the end of a normal duration of PREGNANCY, between 37 to 40 weeks of gestation or about 280 days from the first day of the mother's last menstrual period."),
  'D047968': NtO(MH='Clostridium botulinum type C', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces botulinum toxin type C which is neurotoxic to ANIMALS, especially CATTLE, but not humans. It causes dissociation of ACTIN FILAMENTS.'),
  'D047969': NtO(MH='Clostridium botulinum type D', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces botulinum toxin type D which is neurotoxic to ANIMALS, especially CATTLE, but not humans.'),
  'D047970': NtO(MH='Clostridium botulinum type E', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces botulinum toxin type E which is neurotoxic to humans and animals.'),
  'D047971': NtO(MH='Clostridium botulinum type F', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces botulinum toxin type F which is neurotoxic to humans and animals.'),
  'D047988': NtO(MH='Tumor Necrosis Factor Receptor-Associated Peptides and Proteins', MS='Intracellular signaling peptides and proteins that bind directly or indirectly to the cytoplasmic portion of TUMOR NECROSIS FACTOR RECEPTORS.'),
  'D047989': NtO(MH='Clostridium sordellii', MS='A species of gram-positive bacteria in the family Clostridiaceae, found in INTESTINES and SOIL.'),
  'D047990': NtO(MH='TNF Receptor-Associated Factor 1', MS='A signal transducing tumor necrosis factor receptor associated factor that is involved in TNF RECEPTOR feedback regulation. It is similar in structure and appears to work in conjunction with TNF RECEPTOR-ASSOCIATED FACTOR 2 to inhibit APOPTOSIS.'),
  'D047991': NtO(MH='Clostridium sticklandii', MS='A species of gram-positive, anaerobic bacteria in the family Clostridiaceae frequently used for the study of ENZYMES.'),
  'D047992': NtO(MH='TNF Receptor-Associated Factor 2', MS='A signal transducing tumor necrosis factor receptor associated factor that is involved in TNF RECEPTOR feedback regulation. It is similar in structure and appears to work in conjunction with TNF RECEPTOR-ASSOCIATED FACTOR 1 to inhibit APOPTOSIS.'),
  'D048008': NtO(MH='TNF Receptor-Associated Factor 3', MS='A signal transducing tumor necrosis factor receptor associated factor that is involved in regulation of NF-KAPPA B signaling and activation of MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D048009': NtO(MH='Clostridium symbiosum', MS='A species of gram-positive bacteria in the family Clostridiaceae. Its GLUTAMATE DEHYDROGENASE is commonly used in research.'),
  'D048010': NtO(MH='Clostridium tertium', MS='A species of gram-positive bacteria in the family Clostridiaceae, causing BACTEREMIA in humans and ANIMALS.'),
  'D048012': NtO(MH='Clostridium tetanomorphum', MS='A species of gram-positive bacteria in the family Clostridiaceae. Its glutamate mutase is commonly used in research.'),
  'D048013': NtO(MH='Clostridium thermocellum', MS='A species of gram-positive, thermophilic, cellulolytic bacteria in the family Clostridaceae. It degrades and ferments CELLOBIOSE and CELLULOSE to ETHANOL in the CELLULOSOME.'),
  'D048014': NtO(MH='Clostridium tyrobutyricum', MS='A species of gram-positive bacteria in the family Clostridiaceae responsible for spoilage of some CHEESE via FERMENTATION of BUTYRIC ACID.'),
  'D048015': NtO(MH='TNF Receptor-Associated Factor 5', MS='A signal transducing tumor necrosis factor receptor associated factor that mediates signaling from CD27 ANTIGENS; CD40 ANTIGENS; and the LYMPHOTOXIN BETA RECEPTOR. It is involved in regulation of NF-KAPPA B signaling.'),
  'D048016': NtO(MH='Cucumaria', MS='A genus of large SEA CUCUMBERS possessing the primitive radial configuration of podia in all five ambulacral areas.'),
  'D048028': NtO(MH='DNA, Algal', MS='Deoxyribonucleic acid that makes up the genetic material of algae.'),
  'D048029': NtO(MH='TNF Receptor-Associated Factor 6', MS='A signal transducing tumor necrosis factor receptor associated factor that is involved in regulation of NF-KAPPA B signalling and activation of JNK MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D048030': NtO(MH='RNA, Algal', MS='Ribonucleic acid in algae having regulatory and catalytic roles as well as involvement in protein synthesis.'),
  'D048031': NtO(MH='JNK Mitogen-Activated Protein Kinases', MS='A subgroup of mitogen-activated protein kinases that activate TRANSCRIPTION FACTOR AP-1 via the phosphorylation of C-JUN PROTEINS. They are components of intracellular signaling pathways that regulate CELL PROLIFERATION; APOPTOSIS; and CELL DIFFERENTIATION.'),
  'D048048': NtO(MH='Dental Informatics', MS='The application of computer and information sciences to improve dental practice, research, education and management.'),
  'D048049': NtO(MH='Extracellular Signal-Regulated MAP Kinases', MS='A mitogen-activated protein kinase subfamily that is widely expressed and plays a role in regulation of MEIOSIS; MITOSIS; and post mitotic functions in differentiated cells. The extracellular signal regulated MAP kinases are regulated by a broad variety of CELL SURFACE RECEPTORS and can be activated by certain CARCINOGENS.'),
  'D048051': NtO(MH='p38 Mitogen-Activated Protein Kinases', MS='A mitogen-activated protein kinase subfamily that regulates a variety of cellular processes including CELL GROWTH PROCESSES; CELL DIFFERENTIATION; APOPTOSIS; and cellular responses to INFLAMMATION. The P38 MAP kinases are regulated by CYTOKINE RECEPTORS and can be activated in response to bacterial pathogens.'),
  'D048052': NtO(MH='Mitogen-Activated Protein Kinase 3', MS='A 44-kDa extracellular signal-regulated MAP kinase that may play a role the initiation and regulation of MEIOSIS; MITOSIS; and postmitotic functions in differentiated cells. It phosphorylates a number of TRANSCRIPTION FACTORS; and MICROTUBULE-ASSOCIATED PROTEINS.'),
  'D048053': NtO(MH='Mitogen-Activated Protein Kinase 6', MS='A 97-kDa extracellular signal-regulated MAP kinase. Mitogen-activated protein kinase 6 levels increase during cellular differentiation, while in proliferating cells the enzyme is degraded rapidly via the PROTEASOME ENDOPEPTIDASE COMPLEX.'),
  'D048054': NtO(MH='Mitogen-Activated Protein Kinase 7', MS='A 110-kDa extracellular signal-regulated MAP kinase that is activated in response to cellular stress and by GROWTH FACTOR RECEPTORS-mediated pathways.'),
  'D048055': NtO(MH='Mitogen-Activated Protein Kinase 8', MS='A c-jun amino-terminal kinase that is activated by environmental stress and pro-inflammatory cytokines. Several isoforms of the protein with molecular sizes of 43 and 48 KD exist due to multiple ALTERNATIVE SPLICING.'),
  'D048056': NtO(MH='Mitogen-Activated Protein Kinase 9', MS='A c-jun amino-terminal kinase that is activated by environmental stress and pro-inflammatory cytokines. Several isoforms of the protein with molecular sizes of 48 and 54 KD exist due to multiple ALTERNATIVE SPLICING.'),
  'D048057': NtO(MH='Mitogen-Activated Protein Kinase 10', MS='A c-jun amino-terminal kinase that is found predominantly within NEURONS of the BRAIN, suggesting a role in stress-induced neuronal APOPTOSIS. Several isoforms of the protein with molecular sizes of 47 kDa and 52 kDa exist due to multiple ALTERNATIVE SPLICING.'),
  'D048068': NtO(MH='PPAR-beta', MS='One of the PPAR nuclear transcription factors.'),
  'D048069': NtO(MH='Tumor Necrosis Factors', MS='A family of proteins that were originally identified by their ability to cause NECROSIS of NEOPLASMS. Their necrotic effect on cells is mediated through TUMOR NECROSIS FACTOR RECEPTORS which induce APOPTOSIS.'),
  'D048070': NtO(MH='Fetal Nutrition Disorders', MS='Disorders caused by nutritional imbalance, either overnutrition or undernutrition, in the FETUS in utero.'),
  'D048088': NtO(MH='Informatics', MS='The field of information science concerned with the analysis and dissemination of data through the application of computers.'),
  'D048089': NtO(MH='Pneumonia of Calves, Enzootic', MS='Chronic endemic respiratory disease of dairy calves and an important component of bovine respiratory disease complex. It primarily affects calves up to six months of age and the etiology is multifactorial. Stress plus a primary viral infection is followed by a secondary bacterial infection. The latter is most commonly associated with PASTEURELLA MULTOCIDA producing a purulent BRONCHOPNEUMONIA. Sometimes present are MANNHEIMIA HAEMOLYTICA; HAEMOPHILUS SOMNUS and mycoplasma species.'),
  'D048090': NtO(MH='Bovine Respiratory Disease Complex', MS='A multifactorial disease of CATTLE resulting from complex interactions between environmental factors, host factors, and pathogens. The environmental factors act as stressors adversely affecting the IMMUNE SYSTEM and other host defenses and enhancing transmission of infecting agents.'),
  'D048091': NtO(MH='Guided Tissue Regeneration', MS='Procedures for enhancing and directing tissue repair and renewal processes, such as BONE REGENERATION; NERVE REGENERATION; etc. They involve surgically implanting growth conducive tracks or conduits (TISSUE SCAFFOLDING) at the damaged site to stimulate and control the location of cell repopulation. The tracks or conduits are made from synthetic and/or natural materials and may include support cells and induction factors for CELL GROWTH PROCESSES; or CELL MIGRATION.'),
  'D048108': NtO(MH='Nursing Informatics', MS='The field of information science concerned with the analysis and dissemination of data through the application of computers applied to the field of nursing.'),
  'D048128': NtO(MH='Casein Kinase Ialpha', MS='A casein kinase I isoenzyme that plays a role in intracellular signaling pathways including the WNT SIGNALING PATHWAY, the CELL CYCLE, membrane trafficking, and RNA processing. Multiple isoforms of casein kinase I alpha exist and are due to ALTERNATIVE SPLICING.'),
  'D048148': NtO(MH='Casein Kinase Idelta', MS='A casein kinase I isoenzyme that plays a regulatory role in a variety of cellular functions including vesicular transport, CHROMOSOME SEGREGATION; CYTOKINESIS, developmental processes, and the CIRCADIAN RHYTHM.'),
  'D048149': NtO(MH='Casein Kinase Iepsilon', MS='A casein kinase I isoenzyme with specificity for proteins involved the regulation of the CIRCADIAN RHYTHM.'),
  'D048168': NtO(MH='Escherichia coli K12', MS='A species of gram-negative, rod-shaped bacteria belonging to the K serogroup of ESCHERICHIA COLI. It lives as a harmless inhabitant of the human LARGE INTESTINE and is widely used in medical and GENETIC RESEARCH.'),
  'D048170': NtO(MH='Streptomyces lividans', MS='An actinomycete used for production of commercial ANTIBIOTICS and as a host for gene cloning.'),
  'D048188': NtO(MH='Locusta migratoria', MS='A species of migratory Old World locusts, in the family ACRIDIDAE, that are important pests in Africa and Asia.'),
  'D048189': NtO(MH='Hymenolepis nana', MS='The smallest species of TAPEWORMS. It is the only cestode that parasitizes humans without requiring an intermediate host.'),
  'D048190': NtO(MH='Hymenolepis diminuta', MS='A species of tapeworm (TAPEWORMS) infecting RATS and MICE but rarely causing disease in humans. Its life cycle involves RODENTS as the definitive host and BEETLES as the intermediate host.'),
  'D048191': NtO(MH='Lactobacillus plantarum', MS='A species of rod-shaped, LACTIC ACID bacteria used in PROBIOTICS and SILAGE production.'),
  'D048208': NtO(MH='Nuchal Translucency Measurement', MS='A prenatal ultrasonography measurement of the soft tissue behind the fetal neck. Either the translucent area below the skin in the back of the fetal neck (nuchal translucency) or the distance between occipital bone to the outer skin line (nuchal fold) is measured.'),
  'D048209': NtO(MH='Echinococcus granulosus', MS='A species of hydatid tapeworm (class CESTODA) in the family Taeniidae, whose adult form infects the DIGESTIVE TRACT of DOGS, other canines, and CATS. The larval form infects SHEEP; PIGS; HORSES; and may infect humans, where it migrates to various organs and forms permanent HYDATID CYSTS.'),
  'D048210': NtO(MH='Echinococcus multilocularis', MS='A north temperate species of tapeworm (CESTODA) whose adult form infects FOXES and wild RODENTS. The larval form can infect humans producing HEPATIC HYDATID CYSTS.'),
  'D048228': NtO(MH='Intermittent Pneumatic Compression Devices', MS='Instruments that generate intermittent forces, uniformed or graduated, to facilitate the emptying of VEINS. These devices are used to reduce limb EDEMA and prevent venous THROMBOEMBOLISM, such as deep vein thrombosis in the legs.'),
  'D048229': NtO(MH='Sulfolobus solfataricus', MS='A species of thermoacidophilic ARCHAEA in the family Sulfolobaceae, found in volcanic areas where the temperature is about 80 degrees C and SULFUR is present.'),
  'D048230': NtO(MH='Corynebacterium glutamicum', MS='A species of gram-positive, asporogenous, non-pathogenic, soil bacteria that produces GLUTAMIC ACID.'),
  'D048248': NtO(MH='Acanthamoeba castellanii', MS='A species of free-living soil amoebae in the family Acanthamoebidae. It can cause ENCEPHALITIS and KERATITIS in humans.'),
  'D048249': NtO(MH='Streptococcus thermophilus', MS='A species of thermophilic, gram-positive bacteria found in MILK and milk products.'),
  'D048250': NtO(MH='Squalus', MS='A genus of SHARKS in the family SQUALIDAE, comprised of many different species. They travel in large groups and are bottom dwellers that feed mostly on bony fishes.'),
  'D048251': NtO(MH='Squalus acanthias', MS='A species of shark in the family SQUALIDAE, used for its oil (SQUALENE) and as fish meal. It also figures heavily in biological research, especially with reference to its RECTAL GLAND in studies of WATER-ELECTROLYTE BALANCE.'),
  'D048271': NtO(MH='Chitosan', MS='Deacetylated CHITIN, a linear polysaccharide of deacetylated beta-1,4-D-glucosamine. It is used in HYDROGEL and to treat WOUNDS.'),
  'D048288': NtO(MH='Imidazolines', MS='Compounds based on reduced IMIDAZOLES containing a single double bond.'),
  'D048289': NtO(MH='Imidazolidines', MS='Compounds based on reduced IMIDAZOLINES which contain no double bonds in the ring.'),
  'D048290': NtO(MH='Mitogen-Activated Protein Kinase 11', MS='A 38-kDa mitogen-activated protein kinase that is expressed in a broad variety of cell types. It may play a role in regulating cell proliferation and TRANSCRIPTION FACTOR AP-1 dependent transcription.'),
  'D048291': NtO(MH='Mitogen-Activated Protein Kinase 12', MS='A 38-kDa mitogen-activated protein kinase found primarily in SKELETAL MUSCLE.'),
  'D048292': NtO(MH='Mitogen-Activated Protein Kinase 13', MS='A 38-kDa mitogen-activated protein kinase found expressed at high levels in LUNG; KIDNEY; TESTIS; PANCREAS; and SMALL INTESTINE. It may play a role in regulating functions such as CELL DIFFERENTIATION and APOPTOSIS of EPITHELIAL CELLS.'),
  'D048293': NtO(MH='Didelphis', MS='A genus of large OPOSSUMS in the family Didelphidae, found in the Americas. The species Didelphis virginiana is prominent in North America.'),
  'D048308': NtO(MH='Mitogen-Activated Protein Kinase 14', MS='A 38-kDa mitogen-activated protein kinase that is abundantly expressed in a broad variety of cell types. It is involved in the regulation of cellular stress responses as well as the control of proliferation and survival of many cell types. The kinase activity of the enzyme is inhibited by the pyridinyl-imidazole compound SB 203580.'),
  'D048309': NtO(MH='Monodelphis', MS='A genus of short-tailed OPOSSUMS in the family Didelphidae found in South American, chiefly Brazil. They are opossums least well-adapted to arboreal life.'),
  'D048328': NtO(MH='Potoroidae', MS='A family of rat kangaroos found in and around Australia. Genera include Potorous and Bettongia.'),
  'D048329': NtO(MH='Phalangeridae', MS='A family of possums and cuscuses found in Australia, Tasmania, New Guinea and other islands.'),
  'D048330': NtO(MH='Trichosurus', MS='A genus of brush-tailed possums found chiefly in Australia and New Zealand. Secretions from their ANAL GLAND function both in defense and demarcating territory.'),
  'D048348': NtO(MH='Reverse Transcription', MS='The biosynthesis of DNA carried out on a template of RNA.'),
  'D048368': NtO(MH='Phascolarctidae', MS='A family of marsupials in the order Diprotodontia, native to Australia and possessing vestigial tails. There is a single living genus and species: Phascolarctos cinereus, the koala.'),
  'D048369': NtO(MH='MAP Kinase Kinase 1', MS='An abundant 43-kDa mitogen-activated protein kinase kinase subtype with specificity for MITOGEN-ACTIVATED PROTEIN KINASE 1 and MITOGEN-ACTIVATED PROTEIN KINASE 3.'),
  'D048370': NtO(MH='MAP Kinase Kinase 2', MS='A 44 kDa mitogen-activated protein kinase kinase with specificity for MITOGEN-ACTIVATED PROTEIN KINASE 1 and MITOGEN-ACTIVATED PROTEIN KINASE 3.'),
  'D048371': NtO(MH='MAP Kinase Kinase 3', MS='A mitogen-activated protein kinase kinase with specificity for a subset of P38 MITOGEN-ACTIVATED PROTEIN KINASES that includes MITOGEN-ACTIVATED PROTEIN KINASE 12; MITOGEN-ACTIVATED PROTEIN KINASE 13; and MITOGEN-ACTIVATED PROTEIN KINASE 14.'),
  'D048372': NtO(MH='Streptomyces coelicolor', MS='A soil-dwelling actinomycete with a complex lifecycle involving mycelial growth and spore formation. It is involved in the production of a number of medically important ANTIBIOTICS.'),
  'D048373': NtO(MH='Treponema denticola', MS='A species of bacteria in the family SPIROCHAETACEAE, frequently isolated from periodontal pockets (PERIODONTAL POCKET).'),
  'D048388': NtO(MH='Gadiformes', MS='An order of fish including the families Gadidae (cods), Macrouridae (grenadiers), and hakes. The large Gadidae family includes cod, haddock, whiting, and pollock.'),
  'D048389': NtO(MH='Gadus morhua', MS='A species of fish in the cod family GADIDAE, known as the Atlantic cod. It is one of the most important commercial FISHES.'),
  'D048408': NtO(MH='Chlorella vulgaris', MS='A species of green microalgae in the family Chlorellaceae. It is used as a model organism for PHOTOSYNTHESIS, and as a food supplement (DIETARY SUPPLEMENTS).'),
  'D048409': NtO(MH='Aeromonas salmonicida', MS='A species of gram-negative bacteria, in the family Aeromonadaceae. It is strictly parasitic and often pathogenic causing FURUNCULOSIS in SALMONIDS and ulcer disease in GOLDFISH.'),
  'D048428': NtO(MH='Petromyzon', MS='A genus of primitive fish in the family Petromyzontidae. The sole species is Petromyzon marinus, known as the sea lamprey. The adult form feeds parasitically on other fish species.'),
  'D048429': NtO(MH='Cell Size', MS='The quantity of volume or surface area of CELLS.'),
  'D048430': NtO(MH='Cell Shape', MS='The quality of surface form or outline of CELLS.'),
  'D048448': NtO(MH='Phenylcarbamates', MS='Phenyl esters of carbamic acid or of N-substituted carbamic acids. Structures are similar to PHENYLUREA COMPOUNDS with a carbamate in place of the urea.'),
  'D048468': NtO(MH='Podospora', MS='A genus of ascomycete FUNGI in the order SORDARIALES, which is found on SOIL and herbivore dung (FECES).'),
  'D048488': NtO(MH='Paramecium caudatum', MS='The most widely distributed species of PARAMECIUM. It is elongated and possesses a bluntly pointed posterior.'),
  'D048489': NtO(MH='Paramecium aurelia', MS='A species of ciliated PARAMECIUM possessing two micronuclei.'),
  'D048490': NtO(MH='raf Kinases', MS='A family of closely-related serine-threonine kinases that were originally identified as the cellular homologs of the retrovirus-derived V-RAF KINASES. They are MAP kinase kinase kinases that play important roles in SIGNAL TRANSDUCTION.'),
  'D048491': NtO(MH='Proto-Oncogene Proteins A-raf', MS='A raf kinase subclass expressed primarily in non-neuronal tissues such as SKELETAL MUSCLE. The A-raf kinases are MAP kinase kinase kinases that have specificity for MAP KINASE KINASE 1.'),
  'D048492': NtO(MH='Paragonimus westermani', MS='A species of lung fluke infecting humans and other animals, and found chiefly in Asia and the Far East.'),
  'D048493': NtO(MH='Proto-Oncogene Proteins B-raf', MS='A raf kinase subclass found at high levels in neuronal tissue. The B-raf Kinases are MAP kinase kinase kinases that have specificity for MAP KINASE KINASE 1 and MAP KINASE KINASE 2.'),
  'D048494': NtO(MH='Rhipicephalus', MS='A genus of TICKS, in the family IXODIDAE, widespread in Africa. Members of the genus include many important vectors of animal and human pathogens.'),
  'D048508': NtO(MH='Rhipicephalus sanguineus', MS='A species of tick (TICKS) in the family IXODIDAE, distributed throughout the world but abundant in southern Europe. It will feed on a wide variety of MAMMALS, but DOGS are its preferred host. It transmits a large number of diseases including BABESIOSIS; THEILERIASIS; EHRLICHIOSIS; and MEDITERRANEAN SPOTTED FEVER.'),
  'D048528': NtO(MH='Mandrillus', MS='A genus of the subfamily CERCOPITHECINAE, family CERCOPITHECIDAE, comprising two species: the drill (M. leucophaeus) and the mandrill (M. sphinx). They are usually found in thick rainforest and have a gentle disposition despite their ferocious reputation. Some authors consider Mandrillus a subgenus of PAPIO.'),
  'D048529': NtO(MH='Papio hamadryas', MS='A species of baboon in the family CERCOPITHECIDAE, which has a well-studied trilevel social structure consisting of troops, bands, and clans.'),
  'D048530': NtO(MH='Papio anubis', MS='A species of baboon in the family CERCOPITHECIDAE with a somewhat different social structure than PAPIO HAMADRYAS. They inhabit several areas in Africa south of the Sahara.'),
  'D048531': NtO(MH='Papio papio', MS='A species of baboon in the family CERCOPITHECIDAE, often used as an animal model for cognitive studies.'),
  'D048548': NtO(MH='Papio cynocephalus', MS='A species of baboon in the family CERCOPITHECIDAE found in southern equatorial and east Africa. They are smaller than PAPIO ANUBIS and have a thinner mane.'),
  'D048549': NtO(MH='Papio ursinus', MS='A species of baboon in the family CERCOPITHECIDAE found in southern Africa. They are dark colored and have a variable social structure.'),
  'D048550': NtO(MH='Hepatic Insufficiency', MS='Conditions in which the LIVER functions fall below the normal ranges. Severe hepatic insufficiency may cause LIVER FAILURE or DEATH. Treatment may include LIVER TRANSPLANTATION.'),
  'D048568': NtO(MH='Clostridium botulinum type G', MS='Subtype of CLOSTRIDIUM BOTULINUM that produces botulinum toxin type G. Though it has been isolated from soil, no outbreaks involving this type have been recognized.'),
  'D048588': NtO(MH='Benzoxazines', MS='OXAZINES with a fused BENZENE ring.'),
  'D048608': NtO(MH='Rupicapra', MS='A genus of ruminants in the family Bovidae. The common name chamois usually refers to the species Rupicapra rupicapra. Rupicapra pyrenaica, found in the Pyrenees, is more properly referred to as the Pyrenean chamois.'),
  'D048628': NtO(MH='Ketolides', MS='Compounds based on ERYTHROMYCIN with the 3-cladinose replaced by a ketone. They bind the 23S part of 70S bacterial RIBOSOMES.'),
  'D048629': NtO(MH='Micronuclei, Chromosome-Defective', MS='Defective nuclei produced during the TELOPHASE of MITOSIS or MEIOSIS by lagging CHROMOSOMES or chromosome fragments derived from spontaneous or experimentally induced chromosomal structural changes.'),
  'D048630': NtO(MH='Polyketide Synthases', MS='Large enzyme complexes composed of a number of component enzymes that are found in STREPTOMYCES which biosynthesize MACROLIDES and other polyketides.'),
  'D048631': NtO(MH='Micronucleus, Germline', MS='The smaller, reproductive, transcriptionally inert nucleus in the cells of ciliate protozoans, as distinguished from the larger, vegetative, transcriptionally active MACRONUCLEUS. Micronuclei participate in MEIOSIS and autogamy during GENETIC CONJUGATION.'),
  'D048648': NtO(MH='Macronucleus', MS='The larger of two types of nuclei in ciliate protozoans. It is the transcriptionally active nucleus of the vegetative cells as distinguished from the smaller transcriptionally inert GERMLINE MICRONUCLEUS.'),
  'D048668': NtO(MH='Lactobacillus delbrueckii', MS='A species of gram-positive, rod-shaped, facultatively anaerobic bacteria. capable of producing LACTIC ACID. It is important in the manufacture of FERMENTED DAIRY PRODUCTS.'),
  'D048669': NtO(MH='MAP Kinase Kinase 6', MS='A mitogen-activated protein kinase kinase with specificity for P38 MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D048670': NtO(MH='MAP Kinase Kinase 4', MS='A mitogen-activated protein kinase kinase with specificity for JNK MITOGEN-ACTIVATED PROTEIN KINASES; P38 MITOGEN-ACTIVATED PROTEIN KINASES and the RETINOID X RECEPTORS. It takes part in a SIGNAL TRANSDUCTION pathway that is activated in response to cellular stress.'),
  'D048671': NtO(MH='MAP Kinase Kinase 5', MS='A mitogen-activated protein kinase kinase with specificity for MITOGEN-ACTIVATED PROTEIN KINASE 7.'),
  'D048688': NtO(MH='MAP Kinase Kinase 7', MS='A mitogen-activated protein kinase kinase with specificity for JNK MITOGEN-ACTIVATED PROTEIN KINASES. It takes part in a SIGNAL TRANSDUCTION pathway that is activated in response to CYTOKINES.'),
  'D048690': NtO(MH='Fruiting Bodies, Fungal', MS="The fruiting 'heads' or 'caps' of FUNGI, which as a food item are familiarly known as MUSHROOMS, that contain the FUNGAL SPORES."),
  'D048691': NtO(MH='Grifola', MS='A member of the AGARICALES known for edible MUSHROOMS.'),
  'D048708': NtO(MH='Cell Growth Processes', MS='Processes required for CELL ENLARGEMENT and CELL PROLIFERATION.'),
  'D048728': NtO(MH='MAP Kinase Kinase Kinase 1', MS='A 195-kDa zinc finger motif-containing MAP kinase kinase kinase with broad specificity for MAP KINASE KINASES. It localizes to the CYTOSKELETON and can activate a variety of MAP kinase-dependent pathways.'),
  'D048729': NtO(MH='Anacardic Acids', MS='A group of 6-alkyl SALICYLIC ACIDS that are found in ANACARDIUM and known for causing CONTACT DERMATITIS.'),
  'D048730': NtO(MH='MAP Kinase Kinase Kinase 2', MS='A 70-kDa MAP kinase kinase kinase with specificity for MAP KINASE KINASE 5. It is activated during the cellular response to GROWTH FACTORS, oxidative stress, and hyperosmotic conditions.'),
  'D048748': NtO(MH='MAP Kinase Kinase Kinase 3', MS='A 70-kDa MAPK kinase kinase with specificity for MAP KINASE KINASE 5.'),
  'D048749': NtO(MH='Cytokinesis', MS='The process by which the CYTOPLASM of a cell is divided.'),
  'D048750': NtO(MH='Cell Nucleus Division', MS='The process by which the CELL NUCLEUS is divided.'),
  'D048768': NtO(MH='MAP Kinase Kinase Kinase 4', MS='A 180-kDa MAP kinase kinase kinase with specificity for MAP KINASE KINASE 4 and MAP KINASE KINASE 6.'),
  'D048788': NtO(MH='Growth and Development', MS='The series of changes to the shape, size, components, and functions of an individual organism that occur over time as the organism progresses from its initial form to full size and maturity.'),
  'D048789': NtO(MH='Phytoestrogens', MS='Compounds derived from plants, primarily ISOFLAVONES that mimic or modulate endogenous estrogens, usually by binding to ESTROGEN RECEPTORS.'),
  'D048790': NtO(MH='Renilla', MS='A genus of bioluminescent marine invertebrates in the family Renillidae, order Pennatulacea, class ANTHOZOA. It contains Renilla LUCIFERASE which oxidizes coelenterazine resulting in LUMINESCENCE.'),
  'D048809': NtO(MH='alpha-N-Acetylgalactosaminidase', MS='A hexosaminidase with specificity for terminal non-reducing N-acetyl-D-galactosamine residues in N-acetyl-alpha-D-galactosaminides.'),
  'D048828': NtO(MH='beta-N-Acetyl-Galactosaminidase', MS='A hexosiminidase that specifically hydrolyzes terminal non-reducing N-acetyl-D-galactosamine residues in N-acetyl-beta-D-galactosaminides.'),
  'D048848': NtO(MH='MAP Kinase Kinase Kinase 5', MS='A 150-kDa MAP kinase kinase kinase that may play a role in the induction of APOPTOSIS. It has specificity for MAP KINASE KINASE 3; MAP KINASE KINASE 4; and MAP KINASE KINASE 6.'),
  'D048868': NtO(MH='Adaptor Proteins, Signal Transducing', MS='A broad category of carrier proteins that play a role in SIGNAL TRANSDUCTION. They generally contain several modular domains, each of which having its own binding activity, and act by forming complexes with other intracellular-signaling molecules. Signal-transducing adaptor proteins lack enzyme activity, however their activity can be modulated by other signal-transducing enzymes'),
  'D048888': NtO(MH='Fireflies', MS='The family Lampyidae, which are bioluminescent BEETLES. They contain FIREFLY LUCIFERIN and LUCIFERASES. Oxidation of firefly luciferin results in luminescence.'),
  'D048889': NtO(MH='Weevils', MS='BEETLES in the family Curculionidae and the largest family in the order COLEOPTERA. They have a markedly convex shape and many are considered pests.'),
  'D048908': NtO(MH='Aliivibrio fischeri', MS='A species of gram-negative bacteria in the genus ALIIVIBRIO, which exhibits LUMINESCENCE. A. fischeri is found in a symbiotic relationship with the SQUID Euprymna scolopes.'),
  'D048909': NtO(MH='Diabetes Complications', MS='Conditions or pathological processes associated with the disease of diabetes mellitus. Due to the impaired control of BLOOD GLUCOSE level in diabetic patients, pathological processes develop in numerous tissues and organs including the EYE, the KIDNEY, the BLOOD VESSELS, and the NERVE TISSUE.'),
  'D048928': NtO(MH='Prochlorophytes', MS='An order of GRAM-NEGATIVE OXYGENIC PHOTOSYNTHETIC BACTERIA containing CHLOROPHYLL, a and b, but lacking PHYCOBILISOMES (light-harvesting antennae). There are three genera: PROCHLORON; PROCHLOROCOCCUS; and PROCHLOROTHRIX. This order was thought to be the missing link between CYANOBACTERIA and plant CHLOROPLASTS, hence the name.'),
  'D048929': NtO(MH='Prochlorococcus', MS='A genus of marine planktonic CYANOBACTERIA in the order PROCHLOROPHYTES. They lack PHYCOBILISOMES and contain divinyl CHLOROPHYLL, a and b.'),
  'D048948': NtO(MH='14-3-3 Proteins', MS='A large family of signal-transducing adaptor proteins present in wide variety of eukaryotes. They are PHOSPHOSERINE and PHOSPHOTHREONINE binding proteins involved in important cellular processes including SIGNAL TRANSDUCTION; CELL CYCLE control; APOPTOSIS; and cellular stress responses. 14-3-3 proteins function by interacting with other signal-transducing proteins and effecting changes in their enzymatic activity and subcellular localization. The name 14-3-3 derives from numerical designations used in the original fractionation patterns of the proteins.'),
  'D048949': NtO(MH='Labor Pain', MS='Pain associated with OBSTETRIC LABOR in CHILDBIRTH. It is caused primarily by UTERINE CONTRACTION as well as pressure on the CERVIX; BLADDER; and the GASTROINTESTINAL TRACT. Labor pain mostly occurs in the ABDOMEN; the GROIN; and the BACK.'),
  'D048968': NtO(MH='Morning Sickness', MS='Symptoms of NAUSEA and VOMITING in pregnant women that usually occur in the morning during the first 2 to 3 months of PREGNANCY. Severe persistent vomiting during pregnancy is called HYPEREMESIS GRAVIDARUM.'),
  'D048988': NtO(MH='Corneal Surgery, Laser', MS='Surgical techniques on the CORNEA employing LASERS, especially for reshaping the CORNEA to correct REFRACTIVE ERRORS.'),
  'D049008': NtO(MH='Keratectomy, Subepithelial, Laser-Assisted', MS='A surgical technique to correct REFRACTIVE ERRORS of the EYE, such as MYOPIA and ASTIGMATISM. In this method, a flap of CORNEAL EPITHELIUM is created by exposure of the area to dilute alcohol. The flap is lifted and then replaced after laser ablation of the subepithelial CORNEA.'),
  'D049028': NtO(MH='Dystrophin-Associated Protein Complex', MS='A macromolecular complex of proteins that includes DYSTROPHIN and DYSTROPHIN-ASSOCIATED PROTEINS. It plays a structural role in the linking the CYTOSKELETON to the EXTRACELLULAR MATRIX.'),
  'D049029': NtO(MH='Dystrophin-Associated Proteins', MS='A group of proteins that associate with DYSTROPHIN at the CELL MEMBRANE to form the DYSTROPHIN-ASSOCIATED PROTEIN COMPLEX.'),
  'D049030': NtO(MH='Dystroglycans', MS='Dystrophin-associated proteins that play role in the formation of a transmembrane link between laminin-2 and DYSTROPHIN. Both the alpha and the beta subtypes of dystroglycan originate via POST-TRANSLATIONAL PROTEIN PROCESSING of a single precursor protein.'),
  'D049031': NtO(MH='Sarcoglycans', MS='A family of transmembrane dystrophin-associated proteins that play a role in the membrane association of the DYSTROPHIN-ASSOCIATED PROTEIN COMPLEX.'),
  'D049033': NtO(MH='Chorioallantoic Membrane', MS='A highly vascularized extra-embryonic membrane, formed by the fusion of the CHORION and the ALLANTOIS. It is mostly found in BIRDS and REPTILES. It serves as a model for studying tumor or cell biology, such as angiogenesis and TISSUE TRANSPLANTATION.'),
  'D049049': NtO(MH='Pheromones, Human', MS='Chemical substances excreted by humans that elicit behavior or physiological responses from other humans. Perception of these chemical signals may be olfactory or by contact.'),
  'D049068': NtO(MH='Plagiocephaly, Nonsynostotic', MS='A deformity of the SKULL that is not due to bone fusion (SYNOSTOSIS), such as craniosynostoses, and is characterized by an asymmetric skull and face. It is observed with an increased frequency in INFANTS after the adoption of supine sleeping recommendations to prevent SUDDEN INFANT DEATH SYNDROME.'),
  'D049069': NtO(MH='ATP-Dependent Proteases', MS='Proteases that contain proteolytic core domains and ATPase-containing regulatory domains. They are usually comprised of large multi-subunit assemblies. The domains can occur within a single peptide chain or on distinct subunits.'),
  'D049070': NtO(MH='Protease La', MS='A prokaryotic ATP-dependent protease that plays a role in the degradation of many abnormal proteins. It is a tetramer of 87-kDa subunits, each of which contains a proteolytic site and a ATP-binding site.'),
  'D049071': NtO(MH='Endopeptidase Clp', MS='An ATP-dependent protease found in prokaryotes, CHLOROPLASTS, and MITOCHONDRIA. It is a soluble multisubunit complex that plays a role in the degradation of many abnormal proteins.'),
  'D049088': NtO(MH='Bariatrics', MS='Activities related to WEIGHT REDUCTION in patients with OBESITY. Treatment methods include DIET; EXERCISE; BEHAVIOR MODIFICATION; medications; and BARIATRIC SURGERY.'),
  'D049108': NtO(MH='Cell Enlargement', MS='Growth processes that result in an increase in CELL SIZE.'),
  'D049109': NtO(MH='Cell Proliferation', MS='All of the processes involved in increasing CELL NUMBER including CELL DIVISION.'),
  'D049128': NtO(MH='Sphingolipid Activator Proteins', MS='A family of glycoprotein cofactors that are required for the efficient catabolization of SPHINGOLIPIDS by specific acid hydrolases such as GLUCOSYLCERAMIDASE; GALACTOCEREBROSIDASE; BETA-N-ACETYLHEXOSAMINIDASE; and CEREBROSIDE-SULFATASE.'),
  'D049148': NtO(MH='Aminoacylation', MS='A reaction that introduces an aminoacyl group to a molecule. TRANSFER RNA AMINOACYLATION is the first step in GENETIC TRANSLATION.'),
  'D049168': NtO(MH='Pregnancy, Unplanned', MS='Unintended or accidental pregnancy, including pregnancy resulting from failed contraceptive measures.'),
  'D049188': NtO(MH='Prenatal Injuries', MS='Damages to the EMBRYO, MAMMALIAN or the FETUS before BIRTH. Damages can be caused by any factors including biological, chemical, or physical.'),
  'D049228': NtO(MH='Healthcare Common Procedure Coding System', MS='Uniform method for health care providers and medical suppliers to report professional services, procedures, and supplies. It consists of alphanumeric codes and modifiers for the use of all public and private health insurers. It is developed by the Centers for Medicare and Medicaid Services.'),
  'D049229': NtO(MH='Dendritic Spines', MS='Spiny processes on DENDRITES, each of which receives excitatory input from one nerve ending (NERVE ENDINGS). They are commonly found on PURKINJE CELLS and PYRAMIDAL CELLS.'),
  'D049230': NtO(MH='Retinoid X Receptor beta', MS=None),
  'D049231': NtO(MH='Saposins', MS='A group of four homologous sphingolipid activator proteins that are formed from proteolytic cleavage of a common protein precursor molecule referred to as prosaposin.'),
  'D049248': NtO(MH='Decapitation', MS='Traumatic or experimentally induced separation of the head from the body in an animal or human.'),
  'D049249': NtO(MH='Exhumation', MS='Removal of a dead body from the earth after burial.'),
  'D049250': NtO(MH='Speech Recognition Software', MS='Software capable of recognizing dictation and transcribing the spoken words into written text.'),
  'D049268': NtO(MH='Positron-Emission Tomography', MS='An imaging technique using compounds labelled with short-lived positron-emitting radionuclides (such as carbon-11, nitrogen-13, oxygen-15 and fluorine-18) to measure cell metabolism. It has been useful in study of soft tissues such as CANCER; CARDIOVASCULAR SYSTEM; and brain. SINGLE-PHOTON EMISSION-COMPUTED TOMOGRAPHY is closely related to positron emission tomography, but uses isotopes with longer half-lives and resolution is lower.'),
  'D049269': NtO(MH='Videoconferencing', MS='Communications via an interactive conference between two or more participants at different sites, using computer networks (COMPUTER COMMUNICATION NETWORKS) or other telecommunication links to transmit audio, video, and data.'),
  'D049288': NtO(MH='Muscular Dystrophies, Limb-Girdle', MS='A heterogenous group of inherited muscular dystrophy that can be autosomal dominant or autosomal recessive. There are many forms (called LGMDs) involving genes encoding muscle membrane proteins such as the sarcoglycan (SARCOGLYCANS) complex that interacts with DYSTROPHIN. The disease is characterized by progressing wasting and weakness of the proximal muscles of arms and legs around the HIPS and SHOULDERS (the pelvic and shoulder girdles).'),
  'D049289': NtO(MH='G(M2) Activator Protein', MS='An essential cofactor for the degradation of G(M2)GANGLIOSIDE by lysosomal BETA-N-ACETYLHEXOSAMINIDASES. Genetic mutations resulting in loss of G(M2) activator protein are one of the causes of TAY-SACHS DISEASE, AB VARIANT.'),
  'D049290': NtO(MH='Tay-Sachs Disease, AB Variant', MS='A progressive neurodegenerative disorder that begins with muscle weakness, then progresses to startle reaction, retardation and seizures. It is characterized by the accumulation of G(M2) GANGLIOSIDE in neurons that is caused by a lack of G(M2) ACTIVATOR PROTEIN function. The AB variant designation refers to the increase of both HEXOSAMINIDASE A and HEXOSAMINIDASE B in tissues that lack of G(M2) activator protein.'),
  'D049291': NtO(MH='Seroma', MS='Tumor-like sterile accumulation of serum in a tissue, organ, or cavity. It results from a tissue insult and is the product of tissue inflammation. It most commonly occurs following MASTECTOMY.'),
  'D049292': NtO(MH='Leukoaraiosis', MS='Non-specific white matter changes in the BRAIN, often seen after age 65. Changes include loss of AXONS; MYELIN pallor, GLIOSIS, loss of ependymal cells, and enlarged perivascular spaces. Leukoaraiosis is a risk factor for DEMENTIA and CEREBROVASCULAR DISORDERS.'),
  'D049293': NtO(MH='MedlinePlus', MS='NATIONAL LIBRARY OF MEDICINE service for health professionals and consumers. It links extensive information from the National Institutes of Health and other reviewed sources of information on specific diseases and conditions.'),
  'D049294': NtO(MH='Phospholipid Transfer Proteins', MS='A ubiquitous family of proteins that transport PHOSPHOLIPIDS such as PHOSPHATIDYLINOSITOL and PHOSPHATIDYLCHOLINE between membranes. They play an important role in phospholipid metabolism during vesicular transport and SIGNAL TRANSDUCTION.'),
  'D049308': NtO(MH='Dioxygenases', MS='Non-heme iron-containing enzymes that incorporate two atoms of OXYGEN into the substrate. They are important in biosynthesis of FLAVONOIDS; GIBBERELLINS; and HYOSCYAMINE; and for degradation of AROMATIC HYDROCARBONS.'),
  'D049309': NtO(MH='Acanthoma', MS='A neoplasm composed of squamous or epidermal cells.'),
  'D049310': NtO(MH='Distal Myopathies', MS='A heterogeneous group of genetic disorders characterized by progressive MUSCULAR ATROPHY and MUSCLE WEAKNESS beginning in the hands, the legs, or the feet. Most are adult-onset autosomal dominant forms. Others are autosomal recessive.'),
  'D049312': NtO(MH='Receptors, Lysophospholipid', MS='A family of G-protein-coupled receptors that bind to specific LYSOPHOSPHOLIPIDS such as lysophosphatidic acid and lysosphinglipids such as sphingosine-1-phosphate. They play an important role in the formation and function of the CARDIOVASCULAR SYSTEM.'),
  'D049328': NtO(MH='Mongolian Spot', MS='A bluish-gray to gray-brown benign, melanocytic nevus found usually in the LUMBOSACRAL REGION of dark-skinned people, especially those of East Asian ancestry. It is usually congenital or appears shortly after birth, and disappears in childhood.'),
  'D049329': NtO(MH='Nanostructures', MS='Materials which have structured components with at least one dimension in the range of 1 to 100 nanometers. These include NANOCOMPOSITES; NANOPARTICLES; NANOTUBES; and NANOWIRES.'),
  'D049349': NtO(MH='Receptors, Lysosphingolipid', MS='A subfamily of lysophospholipid receptors with specificity for LYSOSPHINGOLIPIDS such as sphingosine-1-phosphate and sphingosine phosphorylcholine.'),
  'D049368': NtO(MH='Receptors, Lysophosphatidic Acid', MS='A subfamily of lysophospholipid receptors with specificity for LYSOPHOSPHATIDIC ACIDS.'),
  'D049388': NtO(MH='Polyanhydrides', MS='Anhydride polymers with a repeating structure of RC(=O)OC(=O)R. They readily hydrolyze in water making them useful for DELAYED-ACTION PREPARATIONS.'),
  'D049408': NtO(MH='Luminescent Agents', MS='Compound such as LUMINESCENT PROTEINS that cause or emit light (PHYSICAL LUMINESCENCE).'),
  'D049409': NtO(MH='Luciferases, Firefly', MS='Luciferases from FIREFLIES, usually Photinus, that oxidizes FIREFLY LUCIFERIN to cause emission of PHOTONS.'),
  'D049410': NtO(MH='Luciferases, Renilla', MS='Luciferases from RENILLA that oxidizes certain LUMINESCENT AGENTS to cause emission of PHOTONS.'),
  'D049411': NtO(MH='Utrophin', MS='An autosomally-encoded 376-kDa cytoskeletal protein that is similar in structure and function to DYSTROPHIN. It is a ubiquitously-expressed protein that plays a role in anchoring the CYTOSKELETON to the PLASMA MEMBRANE.'),
  'D049428': NtO(MH='Blood-Nerve Barrier', MS='The barrier between the perineurium of PERIPHERAL NERVES and the endothelium (ENDOTHELIUM, VASCULAR) of endoneurial CAPILLARIES. The perineurium acts as a diffusion barrier, but ion permeability at the blood-nerve barrier is still higher than at the BLOOD-BRAIN BARRIER.'),
  'D049429': NtO(MH='Forensic Pathology', MS='The application of pathology to questions of law.'),
  'D049448': NtO(MH='Cholangiopancreatography, Magnetic Resonance', MS='Non-invasive diagnostic technique for visualizing the PANCREATIC DUCTS and BILE DUCTS without the use of injected CONTRAST MEDIA or x-ray. MRI scans provide excellent sensitivity for duct dilatation, biliary stricture, and intraductal abnormalities.'),
  'D049449': NtO(MH='Luminescence', MS='Emission of LIGHT when ELECTRONS return to the electronic ground state from an excited state and lose the energy as PHOTONS. It is sometimes called cool light in contrast to INCANDESCENCE. LUMINESCENT MEASUREMENTS take advantage of this type of light emitted from LUMINESCENT AGENTS.'),
  'D049450': NtO(MH='Incandescence', MS='Emission of LIGHT by a substance as its temperature is raised. It is sometimes called hot light in contrast to LUMINESCENCE.'),
  'D049451': NtO(MH='Nomograms', MS='Graphical representation of a statistical model containing scales for calculating the prognostic weight of a value for each individual variable. Nomograms are instruments that can be used to predict outcomes using specific clinical parameters. They use ALGORITHMS that incorporate several variables to calculate the predicted probability that a patient will achieve a particular clinical endpoint.'),
  'D049452': NtO(MH='Green Fluorescent Proteins', MS='Protein analogs and derivatives of the Aequorea victoria green fluorescent protein that emit light (FLUORESCENCE) when excited with ULTRAVIOLET RAYS. They are used in REPORTER GENES in doing GENETIC TECHNIQUES. Numerous mutants have been made to emit other colors or be sensitive to pH.'),
  'D049453': NtO(MH='Luciferases, Bacterial', MS='Luciferases from BACTERIA such as PHOTOBACTERIUM; VIBRIO; and PHOTORHABDUS.'),
  'D049454': NtO(MH='Firefly Luciferin', MS='A benzothaizole which is oxidized by LUCIFERASES, FIREFLY to cause emission of light (LUMINESCENCE).'),
  'D049468': NtO(MH='Prometaphase', MS='The phase of cell nucleus division following PROPHASE, when the breakdown of the NUCLEAR ENVELOPE occurs and the MITOTIC SPINDLE APPARATUS enters the nuclear region and attaches to the KINETOCHORES.'),
  'D049469': NtO(MH='Meiotic Prophase I', MS='The prophase of the first division of MEIOSIS (in which homologous CHROMOSOME SEGREGATION occurs). It is divided into five stages: leptonema, zygonema, PACHYNEMA, diplonema, and diakinesis.'),
  'D049471': NtO(MH='Pachytene Stage', MS='The stage in the first meiotic prophase, following ZYGOTENE STAGE, when CROSSING OVER between homologous CHROMOSOMES begins.'),
  'D049488': NtO(MH='Phosphatidylethanolamine Binding Protein', MS='A ubiquitously found basic protein that binds to phosphatidylethanolamine and NUCLEOTIDES. It is an endogenous inhibitor of RAF KINASES and may play a role in regulating SIGNAL TRANSDUCTION. Phosphatidylethanolamine-binding protein is the precursor of hippocampal cholinergic neurostimulating peptide, which is cleaved from the N-terminal region of the protein.'),
  'D049489': NtO(MH='Stroboscopy', MS='The observation of successive phases of MOVEMENT by use of a flashing light.'),
  'D049490': NtO(MH='Systems Biology', MS='Comprehensive, methodical analysis of complex biological systems by monitoring responses to perturbations of biological processes. Large scale, computerized collection and analysis of the data are used to develop and test models of biological systems.'),
  'D049528': NtO(MH='Insulin, Long-Acting', MS='Insulin formulations that contain substances that retard absorption thus extending the time period of action.'),
  'D049590': NtO(MH='Postpartum Period', MS='In females, the period that is shortly after giving birth (PARTURITION).'),
  'D049628': NtO(MH='Body Size', MS='The physical measurements of a body.'),
  'D049629': NtO(MH='Waist-Hip Ratio', MS='The waist circumference measurement divided by the hip circumference measurement. For both men and women, a waist-to-hip ratio (WHR) of 1.0 or higher is considered "at risk" for undesirable health consequences, such as heart disease and ailments associated with OVERWEIGHT. A healthy WHR is 0.90 or less for men, and 0.80 or less for women. (National Center for Chronic Disease Prevention and Health Promotion, 2004)'),
  'D049630': NtO(MH='Esophageal Sphincter, Lower', MS='The physiologic or functional barrier to GASTROESOPHAGEAL REFLUX at the esophagogastric junction. Sphincteric muscles remain tonically contracted during the resting state and form the high-pressure zone separating the lumen of the ESOPHAGUS from that of the STOMACH. (Haubrich et al, Bockus Gastroenterology, 5th ed., pp399, 415)'),
  'D049631': NtO(MH='Esophageal Sphincter, Upper', MS='The structure at the pharyngoesophageal junction consisting chiefly of the cricopharyngeus muscle. It normally occludes the lumen of the ESOPHAGUS, except during SWALLOWING.'),
  'D049648': NtO(MH='Timor-Leste', MS='A country in Southeastern Asia, at the eastern end of the Indonesian archipelago. It includes the eastern half of the island of Timor, the Oecussi (Ambeno) region on the northwest portion of the island of Timor, and the islands of Pulau Atauro and Pulau Jaco. On May 20, 2002, TIMOR-LESTE was internationally recognized as an independent state, following its declared independence from Portugal on November 20, 1975 and a period of armed conflict with Indonesia.'),
  'D049668': NtO(MH='History, 15th Century', MS='Time period from 1401 through 1500 of the common era.'),
  'D049669': NtO(MH='History, 16th Century', MS='Time period from 1501 through 1600 of the common era.'),
  'D049670': NtO(MH='History, 17th Century', MS='Time period from 1601 through 1700 of the common era.'),
  'D049671': NtO(MH='History, 18th Century', MS='Time period from 1701 through 1800 of the common era.'),
  'D049672': NtO(MH='History, 19th Century', MS='Time period from 1801 through 1900 of the common era.'),
  'D049673': NtO(MH='History, 20th Century', MS='Time period from 1901 through 2000 of the common era.'),
  'D049674': NtO(MH='History, 21st Century', MS='Time period from 2001 through 2100 of the common era.'),
  'D049690': NtO(MH='History, Ancient', MS='The period of history before 500 of the common era.'),
  'D049691': NtO(MH='History, Medieval', MS='The period of history from the year 500 through 1450 of the common era.'),
  'D049692': NtO(MH='History, Early Modern 1451-1600', MS='The period of history from 1451 through 1600 of the common era.'),
  'D049711': NtO(MH='History, Modern 1601-', MS='The period of history from 1601 of the common era to the present.'),
  'D049750': NtO(MH='Genome, Insect', MS='The genetic complement of an insect (INSECTS) as represented in its DNA.'),
  'D049751': NtO(MH='Genome, Helminth', MS='The genetic complement of a helminth (HELMINTHS) as represented in its DNA.'),
  'D049770': NtO(MH='Genes, Mating Type, Fungal', MS='Fungal genes that mostly encode TRANSCRIPTION FACTORS. In some FUNGI they also encode PHEROMONES and PHEROMONE RECEPTORS. The transcription factors control expression of specific proteins that give a cell its mating identity. Opposite mating type identities are required for mating.'),
  'D049790': NtO(MH='Nuclear Export Signals', MS='Specific amino acid sequences present in the primary amino acid sequence of proteins which mediate their export from the CELL NUCLEUS. They are rich in hydrophobic residues, such as LEUCINE and ISOLEUCINE.'),
  'D049810': NtO(MH='Genetic Speciation', MS='The splitting of an ancestral species into daughter species that coexist in time (King, Dictionary of Genetics, 6th ed). Causal factors may include geographic isolation, HABITAT geometry, migration, REPRODUCTIVE ISOLATION, random GENETIC DRIFT and MUTATION.'),
  'D049811': NtO(MH='Anelloviridae', MS='A family of non-enveloped, vertebrate, single-stranded DNA Viruses.'),
  'D049830': NtO(MH='Cephalopoda', MS='A class in the phylum MOLLUSCA comprised of SQUID; CUTTLEFISH; OCTOPUS; and NAUTILUS. These marine animals are the most highly organized of all the mollusks.'),
  'D049831': NtO(MH='Octopodiformes', MS='A superorder in the class CEPHALOPODA, consisting of the orders Octopoda (octopus) with over 200 species and Vampyromorpha with a single species. The latter is a phylogenetic relic but holds the key to the origins of Octopoda.'),
  'D049832': NtO(MH='Decapodiformes', MS='A superorder of CEPHALOPODS comprised of squid, cuttlefish, and their relatives. Their distinguishing feature is the modification of their fourth pair of arms into tentacles, resulting in 10 limbs.'),
  'D049833': NtO(MH='Sepia', MS='A genus of cuttlefish in the family Sepiidae. They live in tropical, subtropical and temperate waters in most oceans.'),
  'D049834': NtO(MH='Loligo', MS='A genus of SQUID in the family Loliginidae, superorder DECAPODIFORMES, with a spindle-shaped body. They are well-studied, common inshore squids of the Atlantic and eastern Pacific Oceans, but their various species are taxonomically unresolved.'),
  'D049850': NtO(MH='Nautilus', MS='The sole genus in the family Nautilidae, order Nautilida, comprised of CEPHALOPODS with spiral external shells that are separated into chambers.'),
  'D049851': NtO(MH='Gastropoda', MS='A class in the phylum MOLLUSCA comprised of SNAILS and slugs. The former have coiled external shells and the latter usually lack shells.'),
  'D049852': NtO(MH='Clione', MS='A genus of small, shell-less, marine mollusks in the family Clione, superorder GASTROPODA. These pteropod (possessing a foot developed into wing-like organ for swimming) sea slugs feed exclusively on another pteropod mollusk, Limacina.'),
  'D049853': NtO(MH='Pleurobranchaea', MS='A genus of side-gilled sea slugs in the family Pleurobranchidae, superorder GASTROPODA. They are opportunistic voracious feeders but prefer the sea anemone.'),
  'D049870': NtO(MH='Hermissenda', MS='A genus of marine sea slugs in the family Glaucidae, superorder GASTROPODA, found on the Pacific coast of North America. They are used in behavioral and neurological laboratory studies.'),
  'D049871': NtO(MH='Tritonia Sea Slug', MS='A genus of large marine sea slugs in the family Tritoniidae found in the northern Pacific Ocean. They are used in neurological research.'),
  'D049872': NtO(MH='Bivalvia', MS='A class in the phylum MOLLUSCA comprised of mussels; clams; OYSTERS; COCKLES; and SCALLOPS. They are characterized by a bilaterally symmetrical hinged shell and a muscular foot used for burrowing and anchoring.'),
  'D049873': NtO(MH='Cardiidae', MS='A family of bivalve MOLLUSKS with heart-shaped shells, commonly known as cockles. Unlike most BIVALVES, cockles are hermaphroditic.'),
  'D049874': NtO(MH='Corbicula', MS='A genus of freshwater clams, in the family Corbiculidae, class BIVALVIA. It originated in Asia but was introduced in North America and is now found throughout the United States.'),
  'D049875': NtO(MH='Dreissena', MS='A genus of mussels in the family Dreissenidae, class BIVALVIA. They are found in both fresh and brackish water and are not native to North America. Accidentally introduced into the Great Lakes in 1986, they now proliferate widely throughout the United States.'),
  'D049876': NtO(MH='Unionidae', MS='A family of freshwater mussels in the class BIVALVIA. They differ from ZEBRA MUSSELS in that they are larger and posses a larval stage called glochidia, which requires attachment to the GILLS or fins of particular species of FISHES.'),
  'D049877': NtO(MH='Anodonta', MS='A genus of thin-shelled freshwater mussels in the family UNIONIDAE, class BIVALVIA.'),
  'D049878': NtO(MH='Mytilus', MS='A genus of marine mussels in the family MYTILIDAE, class BIVALVIA. The species MYTILUS EDULIS is the highly edible common mussel.'),
  'D049879': NtO(MH='Mytilidae', MS='A family of marine MUSSELS in the class BIVALVIA.'),
  'D049880': NtO(MH='Mytilus edulis', MS='A species of mussel in the genus MYTILUS, family MYTILIDAE, class BIVALVIA, known as the common mussel. It has a bluish-black shell and is highly edible.'),
  'D049881': NtO(MH='Unio', MS='A genus of freshwater mussels in the family UNIONIDAE, class BIVALVIA.'),
  'D049883': NtO(MH='Perna', MS='A genus of freshwater mussel in the family MYTILIDAE, class BIVALVIA. It is found in tropical and warm temperate coastal waters. Most species have green in their shells.'),
  'D049890': NtO(MH='Spisula', MS='A genus of surf clams in the family Mactridae, class BIVALVIA. They are often used in EMBRYOLOGY research.'),
  'D049891': NtO(MH='Mercenaria', MS='A genus of hard-shelled clams in the family Veneridae, class BIVALVIA, commonly called quahogs. They are found just below the surface in the subtidal or lower intertidal coastal waters.'),
  'D049892': NtO(MH='Mya', MS='A genus of soft-shell clams in the family Myidae, class BIVALVIA.'),
  'D049893': NtO(MH='Arcidae', MS='A family of ark shell mollusks, in the class BIVALVIA. They have soft bodies with platelike GILLS enclosed within two shells hinged together.'),
  'D049894': NtO(MH='Scapharca', MS='A genus of mollusks in the family ARCIDAE, class BIVALVIA. It is used in the study of HEMOGLOBINS.'),
  'D049895': NtO(MH='Ostrea', MS='A genus of oysters in the family OSTREIDAE, which includes the edible true oyster, Ostrea edulis.'),
  'D049896': NtO(MH='Crassostrea', MS='A genus of oysters in the family OSTREIDAE, class BIVALVIA.'),
  'D049897': NtO(MH='Pectinidae', MS='A large family of mollusks in the class BIVALVIA, known commonly as scallops. They possess flat, almost circular shells and are found in all seas from shallow water to great depths.'),
  'D049910': NtO(MH='Pecten', MS='A genus of scallops in the family PECTINIDAE, class BIVALVIA. The shell is usually radially ribbed.'),
  'D049911': NtO(MH='Pinctada', MS='A genus of pearl oysters in the family Pteriidae, class BIVALVIA. Both cultured and natural pearls are obtained from species in the genus. They are distinct from the distantly related, edible true oysters of the family OSTREIDAE.'),
  'D049912': NtO(MH='Growth Hormone-Secreting Pituitary Adenoma', MS='A pituitary tumor that secretes GROWTH HORMONE. In humans, excess HUMAN GROWTH HORMONE leads to ACROMEGALY.'),
  'D049913': NtO(MH='ACTH-Secreting Pituitary Adenoma', MS='A pituitary adenoma which secretes ADRENOCORTICOTROPIN, leading to CUSHING DISEASE.'),
  'D049914': NtO(MH='DNA Repair-Deficiency Disorders', MS='Disorders resulting from defective DNA REPAIR processes or the associated cellular responses to DNA DAMAGE.'),
  'D049915': NtO(MH='Ipomoea nil', MS='A plant species of the genus IPOMOEA, family CONVOLVULACEAE. An abundance of spontaneous mutants makes it useful in study of PLANT DNA and GENETICS.'),
  'D049916': NtO(MH='Polyplacophora', MS='A class in the phylum MOLLUSCA comprised of flattened, elongated marine mollusks, commonly called chitons. They are unique in that they possess seven or eight separate shell plates.'),
  'D049917': NtO(MH='Dipsacaceae', MS='A plant family of the order Dipsacales, subclass Asteridae, class Magnoliopsida. It is sometimes called the teasel family.'),
  'D049918': NtO(MH='Salacia', MS='A plant genus of the family Celastraceae. Members contain friedelane-type TRITERPENES and SESQUITERPENES, EUDESMANE.'),
  'D049919': NtO(MH='Erigeron', MS='A plant genus of the family ASTERACEAE that is similar to CONYZA. Members contain SESQUITERPENES.'),
  'D049920': NtO(MH='Ralstonia pickettii', MS='The type species in the genus RALSTONIA. It is often found in the hospital ward as a contaminant of antiseptic and disinfectant solutions.'),
  'D049921': NtO(MH='Cupriavidus', MS='A genus of gram-negative, aerobic, rod-shaped bacteria, in the family BURKHOLDERIACEAE, that are mobile by means of peritrichous FLAGELLA. The genus was formerly called Wautersia and species in this genus were formerly in the genus RALSTONIA.'),
  'D049930': NtO(MH='Chrysopogon', MS='A plant genus of the family POACEAE. The RHIZOME contains SESQUITERPENES and is the source of vetiver oil used in PERFUME.'),
  'D049931': NtO(MH='Sesbania', MS='A plant genus of the family FABACEAE. Members contain piperidine alkaloids (PIPERIDINES).'),
  'D049932': NtO(MH='Nijmegen Breakage Syndrome', MS='A chromosome instability syndrome resulting from a defective response to DNA double-strand breaks. In addition to characteristic FACIES and MICROCEPHALY, patients have a range of findings including RADIOSENSITIVITY, immunodeficiency, increased cancer risk, and growth retardation. Causative mutations occur in the NBS1 gene, located on human chromosome 8q21. NBS1 codes for nibrin, the key regulator protein of the R/M/N (RAD50/MRE11/NBS1) protein complex which senses and mediates cellular response to DNA DAMAGE caused by IONIZING RADIATION.'),
  'D049933': NtO(MH='Aminocoumarins', MS='COUMARINS with an amino group, exemplified by NOVOBIOCIN.'),
  'D049934': NtO(MH='Isocoumarins', MS='Compounds that differ from COUMARINS in having the positions of the ring and ketone oxygens reversed so the keto oxygen is at the 1-position of the molecule.'),
  'D049935': NtO(MH='Hernandiaceae', MS='A plant family of the order Laurales, subclass Magnoliidae, class Magnoliopsida. Members contain cytotoxic furofuran LIGNANS and isoquinoline ALKALOIDS.'),
  'D049936': NtO(MH='Anthocerotophyta', MS='A plant division that includes hornworts, named for the horn-like appearance of the spore-producing plant (sporophyte).'),
  'D049950': NtO(MH='Hyperparathyroidism, Primary', MS='A condition of abnormally elevated output of PARATHYROID HORMONE due to parathyroid HYPERPLASIA or PARATHYROID NEOPLASMS. It is characterized by the combination of HYPERCALCEMIA, phosphaturia, elevated renal 1,25-DIHYDROXYVITAMIN D3 synthesis, and increased BONE RESORPTION.'),
  'D049951': NtO(MH='X Chromosome Inactivation', MS='A dosage compensation process occurring at an early embryonic stage in mammalian development whereby, at random, one X CHROMOSOME of the pair is repressed in the somatic cells of females.'),
  'D049970': NtO(MH='Graves Ophthalmopathy', MS='An autoimmune disorder of the EYE, occurring in patients with Graves disease. Subtypes include congestive (inflammation of the orbital connective tissue), myopathic (swelling and dysfunction of the extraocular muscles), and mixed congestive-myopathic ophthalmopathy.'),
  'D049971': NtO(MH='Thiazides', MS='Heterocyclic compounds with SULFUR and NITROGEN in the ring. This term commonly refers to the BENZOTHIADIAZINES that inhibit SODIUM-POTASSIUM-CHLORIDE SYMPORTERS and are used as DIURETICS.'),
  'D049990': NtO(MH='Membrane Transport Modulators', MS='Agents that affect ION PUMPS; ION CHANNELS; ABC TRANSPORTERS; and other MEMBRANE TRANSPORT PROTEINS.'),
  'D049992': NtO(MH='Sodium Chloride Symporters', MS='A subclass of symporters found in KIDNEY TUBULES, DISTAL that are the major pathway for salt resorption. Inhibition of these symporters by BENZOTHIADIAZINES is the basis of action of some DIURETICS.'),
  'D049993': NtO(MH='Sodium Chloride Symporter Inhibitors', MS='Agents that inhibit SODIUM CHLORIDE SYMPORTERS. They act as DIURETICS. Excess use is associated with HYPOKALEMIA.'),
  'D049994': NtO(MH='Sodium Potassium Chloride Symporter Inhibitors', MS='Agents that inhibit SODIUM-POTASSIUM-CHLORIDE SYMPORTERS which are concentrated in the thick ascending limb at the junction of the LOOP OF HENLE and KIDNEY TUBULES, DISTAL. They act as DIURETICS. Excess use is associated with HYPOKALEMIA and HYPERGLYCEMIA.'),
  'D050010': NtO(MH='Hyperthyroxinemia, Familial Dysalbuminemic', MS='An inherited autosomal dominant trait characterized by abnormally elevated levels of total serum THYROXINE; (T4) in euthyroid patients with abnormal SERUM ALBUMIN that binds T4 with enhanced affinity. The serum levels of free T4, free T3, and TSH are normal. It is one of several T4 abnormalities produced by non-thyroid disorder. This condition is due to mutations of the ALB gene on CHROMOSOME 4.'),
  'D050030': NtO(MH='Andersen Syndrome', MS='A form of inherited long QT syndrome (or LQT7) that is characterized by a triad of potassium-sensitive periodic paralysis, VENTRICULAR ECTOPIC BEATS, and abnormal features such as short stature, low-set ears, and SCOLIOSIS. It results from mutations of KCNJ2 gene which encodes a channel protein (INWARD RECTIFIER POTASSIUM CHANNELS) that regulates resting membrane potential.'),
  'D050031': NtO(MH='Hashimoto Disease', MS='Chronic autoimmune thyroiditis, characterized by the presence of high serum thyroid AUTOANTIBODIES; GOITER; and HYPOTHYROIDISM.'),
  'D050032': NtO(MH='Postpartum Thyroiditis', MS='Transient autoimmune thyroiditis occurring in the POSTPARTUM PERIOD. It is characterized by the presence of high titers of AUTOANTIBODIES against THYROID PEROXIDASE and THYROGLOBULIN. Clinical signs include the triphasic thyroid hormone pattern: beginning with THYROTOXICOSIS, followed with HYPOTHYROIDISM, then return to euthyroid state by 1 year postpartum.'),
  'D050033': NtO(MH='Thyroid Dysgenesis', MS='Defective development of the THYROID GLAND. This concept includes thyroid agenesis (aplasia), hypoplasia, or an ectopic gland. Clinical signs usually are those of CONGENITAL HYPOTHYROIDISM.'),
  'D050034': NtO(MH='Antidiuretic Agents', MS='Agents that reduce the excretion of URINE, most notably the octapeptide VASOPRESSINS.'),
  'D050035': NtO(MH='Sexual Infantilism', MS='The permanent lack of SEXUAL DEVELOPMENT in an individual. This defect is usually observed at an age after expected PUBERTY.'),
  'D050050': NtO(MH='Protein Carbonylation', MS='The appearance of carbonyl groups (such as aldehyde or ketone groups) in PROTEINS as the result of several oxidative modification reactions. It is a standard marker for OXIDATIVE STRESS. Carbonylated proteins tend to be more hydrophobic and resistant to proteolysis.'),
  'D050051': NtO(MH='Transient Receptor Potential Channels', MS='A broad group of eukaryotic six-transmembrane cation channels that are classified by sequence homology because their functional involvement with SENSATION is varied. They have only weak voltage sensitivity and ion selectivity. They are named after a DROSOPHILA mutant that displayed transient receptor potentials in response to light. A 25-amino-acid motif containing a TRP box (EWKFAR) just C-terminal to S6 is found in TRPC, TRPV and TRPM subgroups. ANKYRIN REPEATS are found in TRPC, TRPV & TRPN subgroups. Some are functionally associated with TYROSINE KINASE or TYPE C PHOSPHOLIPASES.'),
  'D050052': NtO(MH='TRPC Cation Channels', MS='A subgroup of TRP cation channels that contain 3-4 ANKYRIN REPEAT DOMAINS and a conserved C-terminal domain. Members are highly expressed in the CENTRAL NERVOUS SYSTEM. Selectivity for calcium over sodium ranges from 0.5 to 10.'),
  'D050053': NtO(MH='TRPM Cation Channels', MS='A subgroup of TRP cation channels named after melastatin protein. They have the TRP domain but lack ANKYRIN repeats. Enzyme domains in the C-terminus leads to them being called chanzymes.'),
  'D050071': NtO(MH='Bone Density Conservation Agents', MS='Agents that inhibit BONE RESORPTION and/or favor BONE MINERALIZATION and BONE REGENERATION. They are used to heal BONE FRACTURES and to treat METABOLIC BONE DISEASES such as OSTEOPOROSIS.'),
  'D050072': NtO(MH='White Heifer Disease', MS='A congenital reproductive abnormality in white female offspring (heifers) in certain breeds of CATTLE, such as Belgian Blue and Shorthorn. The white color is inherited as a recessive trait which is associated with defects in the female reproductive tract (Muellerian system). These heifers are usually sterile.'),
  'D050090': NtO(MH='Ovotesticular Disorders of Sex Development', MS='Conditions of sexual ambiguity in which the individual possesses gonadal tissues of both sexes, tissues from the OVARY and the TESTIS. There can be a testis on one side and an ovary on the other (lateral), or there may be combined ovarian and testicular tissue (ovotestes) on each side (bilateral). The karyotype may be 46,XX; 46,XY; or a mosaic of 46,XX/46,XY. These disorders have historically been called true hermaphroditism.'),
  'D050091': NtO(MH='Dendrimers', MS='Tree-like, highly branched, polymeric compounds. They grow three-dimensionally by the addition of shells of branched molecules to a central core. The overall globular shape and presence of cavities gives potential as drug carriers and CONTRAST AGENTS.'),
  'D050110': NtO(MH='Bariatric Surgery', MS='Surgical procedures aimed at affecting metabolism and producing major WEIGHT REDUCTION in patients with MORBID OBESITY.'),
  'D050111': NtO(MH='Imino Sugars', MS='Sugars in which the OXYGEN is replaced by a NITROGEN atom. This substitution prevents normal METABOLISM resulting in inhibition of GLYCOSIDASES and GLYCOSYLTRANSFERASES.'),
  'D050112': NtO(MH='Imino Pyranoses', MS='Six-carbon pyranose sugars in which the OXYGEN is replaced by a NITROGEN atom.'),
  'D050113': NtO(MH='Imino Furanoses', MS='Five-carbon furanose sugars in which the OXYGEN is replaced by a NITROGEN atom.'),
  'D050130': NtO(MH='Oncolytic Virotherapy', MS='Use of attenuated VIRUSES as ANTINEOPLASTIC AGENTS to selectively kill CANCER cells.'),
  'D050150': NtO(MH='Chalones', MS="Peptides that inhibit mitosis (ANTIMITOTICS). During the 1960's the term referred to crude extracts that inhibited cell proliferation; the activity was later attributed to PYROGLUTAMATE type oligopeptides."),
  'D050151': NtO(MH='Subcutaneous Fat', MS='Fatty tissue under the SKIN through out the body.'),
  'D050152': NtO(MH='Intra-Abdominal Fat', MS='Fatty tissue inside the ABDOMINAL CAVITY, including visceral fat and retroperitoneal fat. It is the most metabolically active fat in the body and easily accessible for LIPOLYSIS. Increased visceral fat is associated with metabolic complications of OBESITY.'),
  'D050153': NtO(MH='Abdominal Fat', MS='Fatty tissue in the region of the ABDOMEN. It includes the ABDOMINAL SUBCUTANEOUS FAT and the INTRA-ABDOMINAL FAT.'),
  'D050154': NtO(MH='Adiposity', MS='The amount of fat or lipid deposit at a site or an organ in the body, an indicator of body fat status.'),
  'D050155': NtO(MH='Lipogenesis', MS='De novo fat synthesis in the body. This includes the synthetic processes of FATTY ACIDS and subsequent TRIGLYCERIDES in the LIVER and the ADIPOSE TISSUE. Lipogenesis is regulated by numerous factors, including nutritional, hormonal, and genetic elements.'),
  'D050156': NtO(MH='Adipogenesis', MS='The differentiation of pre-adipocytes into mature ADIPOCYTES.'),
  'D050171': NtO(MH='Dyslipidemias', MS='Abnormalities in the serum levels of LIPIDS, including overproduction or deficiency. Abnormal serum lipid profiles may include high total CHOLESTEROL, high TRIGLYCERIDES, low HIGH DENSITY LIPOPROTEIN CHOLESTEROL, and elevated LOW DENSITY LIPOPROTEIN CHOLESTEROL.'),
  'D050172': NtO(MH='Genes, X-Linked', MS='Genes that are located on the X CHROMOSOME.'),
  'D050173': NtO(MH='Genes, Y-Linked', MS='Genes that are located on the Y CHROMOSOME.'),
  'D050174': NtO(MH='Genetic Diseases, Y-Linked', MS='Genetic diseases that are linked to mutant ALLELES on the Y CHROMOSOME in humans (Y CHROMOSOME, HUMAN) or the Y chromosome in other species. Included here are animal models of human Y-linked diseases.'),
  'D050175': NtO(MH='Pansporablastina', MS='A suborder of FUNGI in the phylum MICROSPORIDIA, possessing a pansporoblastic membrane.'),
  'D050177': NtO(MH='Overweight', MS='A status with BODY WEIGHT that is above certain standard of acceptable or desirable weight. In the scale of BODY MASS INDEX, overweight is defined as having a BMI of 25.0-29.9 kg/m2. Overweight may or may not be due to increases in body fat (ADIPOSE TISSUE), hence overweight does not equal "over fat".'),
  'D050178': NtO(MH='Monoglycerides', MS='GLYCEROL esterified with a single acyl (FATTY ACIDS) chain.'),
  'D050197': NtO(MH='Atherosclerosis', MS='A thickening and loss of elasticity of the walls of ARTERIES that occurs with formation of ATHEROSCLEROTIC PLAQUES within the ARTERIAL INTIMA.'),
  'D050198': NtO(MH='Phenoxypropanolamines', MS='Compounds based on a propanolamine attached via an OXYGEN atom to a phenoxy ring. The side chain is one carbon longer than PHENYLETHYLAMINES.'),
  'D050199': NtO(MH='Podocytes', MS='Highly differentiated epithelial cells of the visceral layer of BOWMAN CAPSULE of the KIDNEY. They are composed of a cell body with major CELL SURFACE EXTENSIONS and secondary fingerlike extensions called pedicels. They enwrap the KIDNEY GLOMERULUS capillaries with their cell surface extensions forming a filtration structure. The pedicels of neighboring podocytes interdigitate with each other leaving between them filtration slits that are bridged by an extracellular structure impermeable to large macromolecules called the slit diaphragm, and provide the last barrier to protein loss in the KIDNEY.'),
  'D050216': NtO(MH='Metabolic Detoxication, Phase I', MS='Functionalization of exogenous substances to prepare them for conjugation in PHASE II DETOXIFICATION. Phase I enzymes include CYTOCHROME P450 enzymes and some OXIDOREDUCTASES. Excess induction of phase I over phase II detoxification leads to higher levels of FREE RADICALS that can induce CANCER and other cell damage. Induction or antagonism of phase I detoxication is the basis of a number of DRUG INTERACTIONS.'),
  'D050217': NtO(MH='Metabolic Detoxication, Phase II', MS='The conjugation of exogenous substances with various hydrophilic substituents to form water soluble products that are excretable in URINE. Phase II modifications include GLUTATHIONE conjugation; ACYLATION; and AMINATION. Phase II enzymes include GLUTATHIONE TRANSFERASE and GLUCURONOSYLTRANSFERASE. In a sense these reactions detoxify phase I reaction products.'),
  'D050218': NtO(MH='Body Fat Distribution', MS='Deposits of ADIPOSE TISSUE throughout the body. The pattern of fat deposits in the body regions is an indicator of health status. Excess ABDOMINAL FAT increases health risks more than excess fat around the hips or thighs, therefore, WAIST-HIP RATIO is often used to determine health risks.'),
  'D050219': NtO(MH='Amblyospora', MS='A genus of FUNGI, in the family Amblyosporidae, which is a parasite of MOSQUITOES.'),
  'D050220': NtO(MH='Glugea', MS='A genus of FUNGI in the family Glugeidae, parasitic on FISHES. The PARASITES cause the formation of large cysts causing hypertrophy.'),
  'D050256': NtO(MH='Antimitotic Agents', MS='Agents that arrest cells in MITOSIS, most notably TUBULIN MODULATORS.'),
  'D050257': NtO(MH='Tubulin Modulators', MS='Agents that interact with TUBULIN to inhibit or promote polymerization of MICROTUBULES.'),
  'D050258': NtO(MH='Mitosis Modulators', MS='Agents that affect MITOSIS of CELLS.'),
  'D050259': NtO(MH='Genes, Mitochondrial', MS='Genes that are located on the MITOCHONDRIAL DNA. Mitochondrial inheritance is often referred to as maternal inheritance but should be differentiated from maternal inheritance that is transmitted chromosomally.'),
  'D050260': NtO(MH='Carbohydrate Metabolism', MS='Cellular processes in biosynthesis (anabolism) and degradation (catabolism) of CARBOHYDRATES.'),
  'D050261': NtO(MH='Glycogenolysis', MS='The release of GLUCOSE from GLYCOGEN by GLYCOGEN PHOSPHORYLASE (phosphorolysis). The released glucose-1-phosphate is then converted to GLUCOSE-6-PHOSPHATE by PHOSPHOGLUCOMUTASE before entering GLYCOLYSIS. Glycogenolysis is stimulated by GLUCAGON or EPINEPHRINE via the activation of PHOSPHORYLASE KINASE.'),
  'D050262': NtO(MH='Loma', MS='A genus of FUNGI, in the family Glugeidae, a parasite of ATLANTIC SALMON and other FISHES.'),
  'D050263': NtO(MH='Thelohania', MS='A genus of FUNGI, in the family Thelohaniidae, a parasite of red imported fire ants, MOSQUITOES, and other organisms.'),
  'D050276': NtO(MH='Hand Bones', MS='The CARPAL BONES; METACARPAL BONES; and FINGER PHALANGES. In each hand there are eight carpal bones, five metacarpal bones, and 14 phalanges.'),
  'D050277': NtO(MH='Toe Phalanges', MS='Bones that make up the SKELETON of the TOES, consisting of two for the great toe, and three for each of the other toes.'),
  'D050278': NtO(MH='Finger Phalanges', MS='Bones that make up the SKELETON of the FINGERS, consisting of two for the THUMB, and three for each of the other fingers.'),
  'D050279': NtO(MH='Metacarpal Bones', MS='The five cylindrical bones of the METACARPUS, articulating with the CARPAL BONES proximally and the PHALANGES OF FINGERS distally.'),
  'D050280': NtO(MH='Arm Bones', MS='The bones of the free part of the upper extremity including the HUMERUS; RADIUS; and ULNA.'),
  'D050281': NtO(MH='Bones of Lower Extremity', MS='The bones of the upper and lower LEG. They include the PELVIC BONES.'),
  'D050296': NtO(MH='Microbial Viability', MS="Ability of a microbe to survive under given conditions. This can also be related to a colony's ability to replicate."),
  'D050298': NtO(MH='Mesomycetozoea', MS='A class of parasitic and saprophytic microorganisms whose origins can be traced near the animal-fungal divergence. Members of the class are typically pathogens of FISHES, but there are exceptions. There are two recognized orders: Icthyophonida and Dermocystida.'),
  'D050299': NtO(MH='Fibrin Modulating Agents', MS='Agents that affect the function of FIBRIN in BLOOD COAGULATION. They used as COAGULANTS for HEMORRHAGE or ANTICOAGULANTS for THROMBOSIS.'),
  'D050316': NtO(MH='Medical Order Entry Systems', MS='Information systems, usually computer-assisted, that enable providers to initiate medical procedures, prescribe medications, etc. These systems support medical decision-making and error-reduction during patient care.'),
  'D050336': NtO(MH='Mulibrey Nanism', MS='Growth failure from birth that is due to mutations in a gene (TRIM37) on chromosome 17q22-q23 which encodes a RING-B-box-coiled-coil protein.'),
  'D050337': NtO(MH='Trimethyl Ammonium Compounds', MS='QUATERNARY AMMONIUM COMPOUNDS containing three methyl groups, having the general formula of (CH3)3N+R.'),
  'D050338': NtO(MH='Phenylammonium Compounds', MS='QUATERNARY AMMONIUM COMPOUNDS based on PHENYLAMINES with the general formula phenyl-N+R3.'),
  'D050339': NtO(MH='Benzylammonium Compounds', MS='QUATERNARY AMMONIUM COMPOUNDS based on BENZYLAMINES with the general formula phenyl-CN+R3.'),
  'D050356': NtO(MH='Lipid Metabolism', MS='Physiological processes in biosynthesis (anabolism) and degradation (catabolism) of LIPIDS.'),
  'D050376': NtO(MH='Bone-Patellar Tendon-Bone Grafting', MS='Fixation of the ANTERIOR CRUCIATE LIGAMENT, during surgical reconstruction, by the use of a bone-patellar tendon graft.'),
  'D050377': NtO(MH='Cold Ischemia', MS='The chilling of a tissue or organ during decreased BLOOD perfusion or in the absence of blood supply. Cold ischemia time during ORGAN TRANSPLANTATION begins when the organ is cooled with a cold perfusion solution after ORGAN PROCUREMENT surgery, and ends after the tissue reaches physiological temperature during implantation procedures.'),
  'D050378': NtO(MH='T-Lymphocytes, Regulatory', MS='CD4-positive T cells that inhibit immunopathology or autoimmune disease in vivo. They inhibit the immune response by influencing the activity of other cell types. Regulatory T-cells include naturally occurring CD4+CD25+ cells, IL-10 secreting Tr1 cells, and Th3 cells.'),
  'D050379': NtO(MH='Arteriolosclerosis', MS='Thickening of the walls of small ARTERIES or ARTERIOLES due to cell proliferation or HYALINE deposition.'),
  'D050380': NtO(MH='Monckeberg Medial Calcific Sclerosis', MS='Thickening and loss of elasticity of the walls of muscular ARTERIES due to calcification of the TUNICA MEDIA, the concentric layers of helically arranged SMOOTH MUSCLE CELLS.'),
  'D050396': NtO(MH='TRPP Cation Channels', MS='A subgroup of TRP cation channels that are widely expressed in various cell types. Defects are associated with POLYCYSTIC KIDNEY DISEASES.'),
  'D050397': NtO(MH='Radiotherapy, Intensity-Modulated', MS='CONFORMAL RADIOTHERAPY that combines several intensity-modulated beams to provide improved dose homogeneity and highly conformal dose distributions.'),
  'D050398': NtO(MH='Adamantinoma', MS='A locally aggressive, osteolytic neoplasm of the long bones, probably of epithelial origin and most often involving the TIBIA.'),
  'D050416': NtO(MH='Glucagon-Secreting Cells', MS='A type of pancreatic cell representing about 5-20% of the islet cells. Alpha cells secrete GLUCAGON.'),
  'D050417': NtO(MH='Insulin-Secreting Cells', MS='A type of pancreatic cell representing about 50-80% of the islet cells. Beta cells secrete INSULIN.'),
  'D050418': NtO(MH='Pancreatic Polypeptide-Secreting Cells', MS='A group of islet cells (10-35%) which secrete PANCREATIC POLYPEPTIDE, a hormone that regulates APPETITE and FOOD INTAKE.'),
  'D050436': NtO(MH='Regulatory Elements, Transcriptional', MS='Nucleotide sequences of a gene that are involved in the regulation of GENETIC TRANSCRIPTION.'),
  'D050437': NtO(MH='Genes, Developmental', MS='Genes that determine the fate of a cell or CELLS in a region of the embryo during EMBRYONIC DEVELOPMENT.'),
  'D050438': NtO(MH='Genes, Immunoglobulin Heavy Chain', MS='Genes and gene segments encoding the IMMUNOGLOBULIN HEAVY CHAINS. Gene segments of the heavy chain genes are symbolized V (variable), D (diversity), J (joining), and C (constant).'),
  'D050439': NtO(MH='Genes, Immunoglobulin Light Chain', MS='Genes and gene segments encoding the IMMUNOGLOBULIN LIGHT CHAINS. Gene segments of the light chain genes are designated as V (variable), J (joining), and C (constant).'),
  'D050440': NtO(MH='Immunoglobulin Subunits', MS='The PROTEIN SUBUNITS of the multimeric IMMUNOGLOBULIN proteins, such as IGA; IGD; IGE; IGG; and IGM. Included are the heavy and light chains which contain the specific ANTIGEN binding domains, as well as the accessory proteins that are part of the the secreted forms of IGM and IGA; (SECRETORY IGA).'),
  'D050441': NtO(MH='VDJ Exons', MS='Exons that are created in vivo during LYMPHOCYTE maturation from the V, D, and J gene segments of immunoglobulin superfamily genes (e.g., the IMMUNOGLOBULIN HEAVY CHAIN GENES, or the T-CELL RECEPTOR BETA GENES or T-CELL RECEPTOR GAMMA GENES ) by the VDJ RECOMBINASE system.'),
  'D050456': NtO(MH='Fomites', MS='Inanimate objects that carry pathogenic microorganisms and thus can serve as the source of infection. Microorganisms typically survive on fomites for minutes or hours. Common fomites include CLOTHING, tissue paper, hairbrushes, and COOKING AND EATING UTENSILS.'),
  'D050476': NtO(MH='Bowman Capsule', MS='A double-walled epithelial capsule that is the bulbous closed proximal end of the kidney tubular system. It surrounds the cluster of convoluted capillaries of KIDNEY GLOMERULUS and is continuous with the convoluted PROXIMAL KIDNEY TUBULE.'),
  'D050477': NtO(MH='Urofollitropin', MS='A protein extract of human menopausal urine in which LUTEINIZING HORMONE has been partially or completely removed. Urofollitropin represents FOLLICLE STIMULATING HORMONE from the urine.'),
  'D050479': NtO(MH='Neurotransmitter Transport Proteins', MS='Membrane transport proteins found predominately in NEURONS and neuroendocrine cells that facilitate neurotransmitter transport. They include two distinct families of proteins that transport NEUROTRANSMITTERS across the PLASMA MEMBRANE and that transport NEUROTRANSMITTERS into SECRETORY VESICLES.'),
  'D050480': NtO(MH='Plasma Membrane Neurotransmitter Transport Proteins', MS='A family of neurotransmitter transporter proteins that facilitate NEUROTRANSMITTER reuptake into PRESYNAPTIC TERMINALS. They may play a role in regulating the intensity and duration of neurotransmission.'),
  'D050481': NtO(MH='Vesicular Neurotransmitter Transport Proteins', MS='A family of neurotransmitter transporter proteins that are INTEGRAL MEMBRANE PROTEINS of the LIPID BILAYER of SECRETORY VESICLES. They are ANTIPORTERS that exchange vesicular PROTONS for cytoplasmic NEUROTRANSMITTER and play an essential role in regulating neurotransmission.'),
  'D050482': NtO(MH='Catecholamine Plasma Membrane Transport Proteins', MS='A group of membrane transport proteins that transport biogenic amine derivatives of catechol across the PLASMA MEMBRANE. Catecholamine plasma membrane transporter proteins regulate neural transmission as well as catecholamine metabolism and recycling.'),
  'D050483': NtO(MH='Dopamine Plasma Membrane Transport Proteins', MS='Sodium chloride-dependent neurotransmitter symporters located primarily on the PLASMA MEMBRANE of dopaminergic neurons. They remove DOPAMINE from the EXTRACELLULAR SPACE by high affinity reuptake into PRESYNAPTIC TERMINALS and are the target of DOPAMINE UPTAKE INHIBITORS.'),
  'D050484': NtO(MH='Norepinephrine Plasma Membrane Transport Proteins', MS='Sodium chloride-dependent neurotransmitter symporters located primarily on the PLASMA MEMBRANE of noradrenergic neurons. They remove NOREPINEPHRINE from the EXTRACELLULAR SPACE by high affinity reuptake into PRESYNAPTIC TERMINALS. The norepinephrine transporter regulates signal amplitude and duration at noradrenergic synapses and is the target of ADRENERGIC UPTAKE INHIBITORS.'),
  'D050485': NtO(MH='GABA Plasma Membrane Transport Proteins', MS='A family of plasma membrane neurotransmitter transporter proteins that regulates extracellular levels of the inhibitory neurotransmitter GAMMA-AMINOBUTYRIC ACID. They differ from GABA RECEPTORS, which signal cellular responses to GAMMA-AMINOBUTYRIC ACID. They control GABA reuptake into PRESYNAPTIC TERMINALS in the CENTRAL NERVOUS SYSTEM through high-affinity sodium-dependent transport.'),
  'D050486': NtO(MH='Serotonin Plasma Membrane Transport Proteins', MS='Sodium chloride-dependent neurotransmitter symporters located primarily on the PLASMA MEMBRANE of serotonergic neurons. They are different than SEROTONIN RECEPTORS, which signal cellular responses to SEROTONIN. They remove SEROTONIN from the EXTRACELLULAR SPACE by high affinity reuptake into PRESYNAPTIC TERMINALS. Regulates signal amplitude and duration at serotonergic synapses and is the site of action of the SEROTONIN UPTAKE INHIBITORS.'),
  'D050487': NtO(MH='Glycine Plasma Membrane Transport Proteins', MS='A family of sodium chloride-dependent neurotransmitter symporters that transport the amino acid GLYCINE. They differ from GLYCINE RECEPTORS, which signal cellular responses to GLYCINE. They are located primarily on the PLASMA MEMBRANE of NEURONS; GLIAL CELLS; EPITHELIAL CELLS; and RED BLOOD CELLS where they remove inhibitory neurotransmitter glycine from the EXTRACELLULAR SPACE.'),
  'D050488': NtO(MH='Hallux Varus', MS='Displacement of the great toe (HALLUX) towards the midline or away from the other TOES. It can be congenital or acquired.'),
  'D050489': NtO(MH="Bunion, Tailor's", MS='Abnormal swelling of the outer aspect of the fifth metatarsal head affecting the fifth METATARSOPHALANGEAL JOINT.'),
  'D050490': NtO(MH='Oncogene Protein tpr-met', MS='The GENETIC TRANSLATION product from a GENE FUSION between a sequence from the tpr protein gene on the human CHROMOSOME 1 and the gene for PROTO-ONCOGENE PROTEINS C-MET.'),
  'D050491': NtO(MH='Glutamate Plasma Membrane Transport Proteins', MS='A family of plasma membrane neurotransmitter transporter proteins that couple the uptake of GLUTAMATE with the import of SODIUM ions and PROTONS and the export of POTASSIUM ions. In the CENTRAL NERVOUS SYSTEM they regulate neurotransmission through synaptic reuptake of the excitatory neurotransmitter glutamate. Outside the central nervous system they function as signal mediators and regulators of glutamate metabolism.'),
  'D050492': NtO(MH='Vesicular Biogenic Amine Transport Proteins', MS='Integral membrane proteins of the LIPID BILAYER of SECRETORY VESICLES that catalyze transport and storage of biogenic amine NEUROTRANSMITTERS such as ACETYLCHOLINE; SEROTONIN; MELATONIN; HISTAMINE; and CATECHOLAMINES. The transporters exchange vesicular protons for cytoplasmic neurotransmitters.'),
  'D050493': NtO(MH='Vesicular Monoamine Transport Proteins', MS='A family of vesicular amine transporter proteins that catalyze the transport and storage of CATECHOLAMINES and indolamines into SECRETORY VESICLES.'),
  'D050494': NtO(MH='Vesicular Acetylcholine Transport Proteins', MS='Vesicular amine transporter proteins that transport the neurotransmitter ACETYLCHOLINE into small SECRETORY VESICLES. Proteins of this family contain 12 transmembrane domains and exchange vesicular PROTONS for cytoplasmic acetylcholine.'),
  'D050495': NtO(MH='Vesicular Inhibitory Amino Acid Transport Proteins', MS='A family of vesicular neurotransmitter transporter proteins that sequester the inhibitory neurotransmitters GLYCINE; GAMMA-AMINOBUTYRIC ACID; and possibly GAMMA-HYDROXYBUTYRATE into SECRETORY VESICLES.'),
  'D050496': NtO(MH='Vesicular Glutamate Transport Proteins', MS='A family of vesicular neurotransmitter transporter proteins that were originally characterized as sodium dependent inorganic phosphate cotransporters. Vesicular glutamate transport proteins sequester the excitatory neurotransmitter GLUTAMATE from the CYTOPLASM into SECRETORY VESICLES in exchange for lumenal PROTONS.'),
  'D050497': NtO(MH='Stillbirth', MS='The event that a FETUS is born dead or stillborn.'),
  'D050498': NtO(MH='Live Birth', MS='The event that a FETUS is born alive with heartbeats or RESPIRATION regardless of GESTATIONAL AGE. Such liveborn is called a newborn infant (INFANT, NEWBORN).'),
  'D050499': NtO(MH='Adrenarche', MS='A stage of development at which the ADRENAL GLANDS undergo maturation leading to the capability of producing increasing amounts of adrenal androgens, DEHYDROEPIANDROSTERONE and ANDROSTENEDIONE. Adrenarche usually begins at about 7 or 8 years of age before the signs of PUBERTY and continues throughout puberty.'),
  'D050500': NtO(MH='Pancreatitis, Chronic', MS='INFLAMMATION of the PANCREAS that is characterized by recurring or persistent ABDOMINAL PAIN with or without STEATORRHEA or DIABETES MELLITUS. It is characterized by the irregular destruction of the pancreatic parenchyma which may be focal, segmental, or diffuse.'),
  'D050501': NtO(MH='Nostoc commune', MS='A form species of spore-producing CYANOBACTERIA, in the family Nostocaceae, order Nostocales. It is an important source of fixed NITROGEN in nutrient-depleted soils. When wet, it appears as a jelly-like mass.'),
  'D050502': NtO(MH='Nostoc muscorum', MS='A form-species of nitrogen-fixing CYANOBACTERIA, in the family Nostocaceae, order Nostocales.'),
  'D050503': NtO(MH='Indoleamine-Pyrrole 2,3,-Dioxygenase', MS='A dioxygenase with specificity for the oxidation of the indoleamine ring of TRYPTOPHAN. It is an extrahepatic enzyme that plays a role in metabolism as the first and rate limiting enzyme in the kynurenine pathway of TRYPTOPHAN catabolism.'),
  'D050504': NtO(MH='Oncolytic Viruses', MS='Tumor-selective, replication competent VIRUSES that have antineoplastic effects. This is achieved by producing cytotoxicity-enhancing proteins and/or eliciting an antitumor immune response. They are genetically engineered so that they can replicate in CANCER cells but not in normal cells, and are used in ONCOLYTIC VIROTHERAPY.'),
  'D050505': NtO(MH='Mutant Proteins', MS='Proteins produced from GENES that have acquired MUTATIONS.'),
  'D050506': NtO(MH='Mutant Chimeric Proteins', MS='Proteins produced from GENES that have mutated by the fusing of protein coding regions of more than one gene. Such hybrid proteins are responsible for some instances of ANTIBIOTIC RESISTANCE and defective biological processes such as NEOPLASMS.'),
  'D050507': NtO(MH='Subcutaneous Fat, Abdominal', MS='Fatty tissue under the SKIN in the region of the ABDOMEN.'),
  'D050527': NtO(MH='Mesangial Cells', MS='Smooth muscle-like cells adhering to the wall of the small blood vessels of the KIDNEY at the glomerulus and along the vascular pole of the glomerulus in the JUXTAGLOMERULAR APPARATUS. They are myofibroblasts with contractile and phagocytic properties. These cells and their MESANGIAL EXTRACELLULAR MATRIX constitute the GLOMERULAR MESANGIUM.'),
  'D050528': NtO(MH='Diet, Carbohydrate-Restricted', MS='A diet that contains limited amounts of CARBOHYDRATES. This is in distinction to a regular DIET.'),
  'D050533': NtO(MH='Glomerular Basement Membrane', MS='The layer of EXTRACELLULAR MATRIX that lies between the ENDOTHELIUM of the glomerular capillaries and the PODOCYTES of the inner or visceral layer of the BOWMAN CAPSULE. It is the product of these two cell types. It acts as a physical barrier and an ion-selective filter.'),
  'D050536': NtO(MH='Glycerol-3-Phosphate Dehydrogenase (NAD+)', MS='An NAD-dependent enzyme that catalyzes the oxidation of sn-glycerol 3-phosphate to glycerone phosphate.'),
  'D050537': NtO(MH='Hydroxypyruvate Reductase', MS='An enzyme that catalyzes the oxidation of D-glycerate to hydroxypyruvate in the presence of NADP.'),
  'D050538': NtO(MH='Malate Dehydrogenase (NADP+)', MS='A light-activated enzyme that catalyzes the oxidation of (S)-malate to OXALOACETATE. It is involved in PYRUVATE metabolism and CARBON fixation.'),
  'D050539': NtO(MH='3-Isopropylmalate Dehydrogenase', MS='An NAD+ dependent enzyme that catalyzes the oxidation of 3-carboxy-2-hydroxy-4-methylpentanoate to 3-carboxy-4-methyl-2-oxopentanoate. It is involved in the biosynthesis of VALINE; LEUCINE; and ISOLEUCINE.'),
  'D050540': NtO(MH='Ketol-Acid Reductoisomerase', MS='An enzyme that catalyzes the oxidation of (R)-2,3-dihydroxy-3-methylbutanoate to (S)-2-hydroxy-2-methyl-3-oxobutanoate in the presence of NADP. It is involved in the biosynthesis of VALINE; LEUCINE; ISOLEUCINE; pentothenate and COENZYME A. This enzyme was formerly classified as EC 1.1.1.89.'),
  'D050541': NtO(MH='Bowman Membrane', MS='A layer of acellular matrix that lies beneath the CORNEAL EPITHELIUM and above the CORNEAL STROMA. It consists of randomly arranged COLLAGEN fibers in a condensed bed of intercellular substance. It provides stability and strength to the cornea.'),
  'D050542': NtO(MH='D-Xylulose Reductase', MS='An enzyme that plays a role in the PENTOSES and GLUCURONATES interconversion pathway by catalyzing the oxidation of XYLITOL to D-xylulose. This enzyme has been found to be specific for NAD+.'),
  'D050543': NtO(MH='Phosphoglycerate Dehydrogenase', MS='An enzyme that catalyzes the oxidation of 3-phosphoglycerate to 3-phosphohydroxypyruvate. It takes part in the L-SERINE biosynthesis pathway.'),
  'D050544': NtO(MH='L-Gulonolactone Oxidase', MS='An enzyme involved in the metabolism of ASCORBIC ACID and aldarate. It catalyzes the oxidation of L-gulono-1,4-lactone to L-xylo-hex-3-ulonolactone.'),
  'D050545': NtO(MH='Choline Dehydrogenase', MS='An enzyme bound to the inner mitochondrial membrane that catalyzes the oxidation of CHOLINE to BETAINE.'),
  'D050547': NtO(MH='Plastoquinol-Plastocyanin Reductase', MS='A component of the CYTOCHROME B6F COMPLEX, this enzyme catalyzes the oxidation of plastoquinol-1 to PLASTOQUINONE.'),
  'D050548': NtO(MH='Catechol 1,2-Dioxygenase', MS='An enzyme that catalyzes the oxidation of catechol to muconic acid with the use of Fe3+ as a cofactor. This enzyme was formerly characterized as EC 1.13.1.1 and EC 1.99.2.2.'),
  'D050556': NtO(MH='Fatty Acid-Binding Proteins', MS='Intracellular proteins that reversibly bind hydrophobic ligands including: saturated and unsaturated FATTY ACIDS; EICOSANOIDS; and RETINOIDS. They are considered a highly conserved and ubiquitously expressed family of proteins that may play a role in the metabolism of LIPIDS.'),
  'D050557': NtO(MH='Catechol 2,3-Dioxygenase', MS='Catalyzes the oxidation of catechol to 2-hydroxymuconate semialdehyde in the carbazole and BENZOATE degradation via HYDROXYLATION pathways. It also catalyzes the conversion of 3-methylcatechol to cis, cis-2-hydroxy-6-oxohept-2,4-dienoate in the TOLUENE and XYLENE degradation pathway. This enzyme was formerly characterized as EC 1.13.1.2.'),
  'D050558': NtO(MH='Cysteine Dioxygenase', MS='An enzyme that catalyzes the conversion of L-CYSTEINE to 3-sulfinoalanine (3-sulfino-L-alanine) in the CYSTEINE metabolism and TAURINE and hypotaurine metabolic pathways.'),
  'D050559': NtO(MH="beta-Carotene 15,15'-Monooxygenase", MS='A monooxygenase that catalyzes the conversion of BETA-CAROTENE into two molecules of RETINAL. It was formerly characterized as EC 1.13.11.21 and EC 1.18.3.1.'),
  'D050560': NtO(MH='Homogentisate 1,2-Dioxygenase', MS='A mononuclear Fe(II)-dependent oxygenase, this enzyme catalyzes the conversion of homogentisate to 4-maleylacetoacetate, the third step in the pathway for the catabolism of TYROSINE. Deficiency in the enzyme causes ALKAPTONURIA, an autosomal recessive disorder, characterized by homogentisic aciduria, OCHRONOSIS and ARTHRITIS. This enzyme was formerly characterized as EC 1.13.1.5 and EC 1.99.2.5.'),
  'D050561': NtO(MH='3-Hydroxyanthranilate 3,4-Dioxygenase', MS='An enzyme that catalyzes the conversion of 3-hydroxyanthranilate to 2-amino-3-carboxymuconate semialdehyde. It was formerly characterized as EC 1.13.1.6.'),
  'D050562': NtO(MH='Inositol Oxygenase', MS='A non-heme IRON enzyme that catalyzes the conversion of MYOINOSITOL to D-glucuronic acid. The reaction is the first committed step in MYOINOSITOL catabolic pathway. This enzyme was formerly characterized as EC 1.13.1.11 and 1.99.2.6.'),
  'D050563': NtO(MH='gamma-Butyrobetaine Dioxygenase', MS='An enzyme that catalyzes the HYDROXYLATION of gamma-butyrobetaine to L-CARNITINE. It is the last enzyme in the biosynthetic pathway of L-CARNITINE and is dependent on alpha-ketoglutarate; IRON; ASCORBIC ACID; and OXYGEN.'),
  'D050564': NtO(MH='Trans-Cinnamate 4-Monooxygenase', MS='A member of the P450 superfamily, this enzyme catalyzes the first oxidative step of the phenylpropanoid pathway in higher PLANTS by transforming trans-cinnamate into p-coumarate.'),
  'D050565': NtO(MH='Benzoate 4-Monooxygenase', MS='An enzyme that catalyzes the oxidation of BENZOATE to 4-hydroxybenzoate. It requires IRON and tetrahydropteridine.'),
  'D050576': NtO(MH='Membrane Fusion Proteins', MS='Proteins that catalyze MEMBRANE FUSION.'),
  'D050577': NtO(MH='Complement C3-C5 Convertases', MS='Serine proteases that cleave COMPLEMENT C3 into COMPLEMENT C3A and COMPLEMENT C3B, or cleave COMPLEMENT C5 into COMPLEMENT C5A and COMPLEMENT C5B. These include the different forms of C3/C5 convertases in the classical and the alternative pathways of COMPLEMENT ACTIVATION. Both cleavages take place at the C-terminal of an ARGININE residue.'),
  'D050578': NtO(MH='Complement C3-C5 Convertases, Alternative Pathway', MS='Important enzymes in the ALTERNATIVE COMPLEMENT ACTIVATION PATHWAY. They cleave COMPLEMENT C3 and COMPLEMENT C5.'),
  'D050579': NtO(MH='Complement C3-C5 Convertases, Classical Pathway', MS='Important enzymes in the CLASSICAL COMPLEMENT ACTIVATION PATHWAY. They cleave COMPLEMENT C3 and COMPLEMENT C5.'),
  'D050580': NtO(MH='Excitatory Amino Acid Transporter 3', MS='A neuronal and epithelial type glutamate plasma membrane transporter protein.'),
  'D050581': NtO(MH='Excitatory Amino Acid Transporter 4', MS='A glutamate plasma membrane transporter protein that is primarily expressed in cerebellar PURKINJE CELLS on postsynaptic DENDRITIC SPINES.'),
  'D050582': NtO(MH='Excitatory Amino Acid Transporter 5', MS='A glutamate plasma membrane transporter protein that is primarily expressed in the RETINA.'),
  'D050596': NtO(MH='Oncogene Fusion', MS='The GENETIC RECOMBINATION of the parts of two or more GENES, including an ONCOGENE as at least one of the fusion partners. Such gene fusions are often detected in neoplastic cells and are transcribed into ONCOGENE FUSION PROTEINS.'),
  'D050598': NtO(MH='Vesicular Glutamate Transport Protein 1', MS='A vesicular glutamate transporter protein that is predominately expressed in TELENCEPHALON of the BRAIN.'),
  'D050599': NtO(MH='Vesicular Glutamate Transport Protein 2', MS='A vesicular glutamate transporter protein that is predominately expressed in the DIENCEPHALON and lower brainstem regions of the CENTRAL NERVOUS SYSTEM.'),
  'D050600': NtO(MH='SNARE Proteins', MS='A superfamily of small proteins which are involved in the MEMBRANE FUSION events, intracellular protein trafficking and secretory processes. They share a homologous SNARE motif. The SNARE proteins are divided into subfamilies: QA-SNARES; QB-SNARES; QC-SNARES; and R-SNARES. The formation of a SNARE complex (composed of one each of the four different types SNARE domains (Qa, Qb, Qc, and R)) mediates MEMBRANE FUSION. Following membrane fusion SNARE complexes are dissociated by the NSFs (N-ETHYLMALEIMIDE-SENSITIVE FACTORS), in conjunction with SOLUBLE NSF ATTACHMENT PROTEIN, i.e., SNAPs (no relation to SNAP 25.)'),
  'D050601': NtO(MH='Kynurenine 3-Monooxygenase', MS='An NADPH-dependent flavin monooxygenase that plays a key role in the catabolism of TRYPTOPHAN by catalyzing the HYDROXYLATION of KYNURENINE to 3-hydroxykynurenine. It was formerly characterized as EC 1.14.1.2 and EC 1.99.1.5.'),
  'D050602': NtO(MH='Linoleoyl-CoA Desaturase', MS='An enzyme that catalyzes the syn-dehydrogenation of linoleol-CoA gamma-linolenoyl-CoA. It was formerly characterized as EC 1.14.99.25.'),
  'D050603': NtO(MH='Squalene Monooxygenase', MS='The second enzyme in the committed pathway for CHOLESTEROL biosynthesis, this enzyme catalyzes the first oxygenation step in the biosynthesis of STEROLS and is thought to be a rate limiting enzyme in this pathway. Specifically, this enzyme catalyzes the conversion of SQUALENE to (S)-squalene-2,3-epoxide.'),
  'D050604': NtO(MH='Succinate-Semialdehyde Dehydrogenase (NADP+)', MS='An enzyme that catalyzes the oxidation of succinate semialdehyde to SUCCINIC ACID. It plays a role in the metabolism of GLUTAMATE; TYROSINE; and butanoate.'),
  'D050605': NtO(MH='Malonate-Semialdehyde Dehydrogenase (Acetylating)', MS='An enzyme that catalyzes the oxidation of 3-oxopropanoate (malonate semialdehyde) to acetyl COENZYME A. It plays a role in the metabolism of BETA-ALANINE.'),
  'D050606': NtO(MH='Mannose-Binding Protein-Associated Serine Proteases', MS='Serum serine proteases which participate in COMPLEMENT ACTIVATION. They are activated when complexed with the MANNOSE-BINDING LECTIN, therefore also known as Mannose-binding protein-Associated Serine Proteases (MASPs). They cleave COMPLEMENT C4 and COMPLEMENT C2 to form C4b2a, the CLASSICAL PATHWAY C3 CONVERTASE.'),
  'D050607': NtO(MH='Organogold Compounds', MS='Organic compounds that contain GOLD as an integral part of the molecule. Some are used as ANTIRHEUMATIC AGENTS. The term chrysotherapy derives from an ancient Greek term for gold.'),
  'D050608': NtO(MH='Sodium-Phosphate Cotransporter Proteins', MS='A family of symporters that facilitate sodium-dependent membrane transport of phosphate.'),
  'D050609': NtO(MH='Sodium-Phosphate Cotransporter Proteins, Type I', MS='A family of sodium-phosphate cotransporter proteins that also transport organic ANIONS. They are low affinity phosphate transporters.'),
  'D050610': NtO(MH='Sodium-Phosphate Cotransporter Proteins, Type II', MS='A family of sodium-phosphate cotransporter proteins with eight transmembrane domains. They are present primarily in the KIDNEY and SMALL INTESTINE and are responsible for renal and small intestinal epithelial transport of phosphates.'),
  'D050611': NtO(MH='Sodium-Phosphate Cotransporter Proteins, Type III', MS='A family of highly conserved and widely expressed sodium-phosphate cotransporter proteins. They are electrogenic sodium-dependent transporters of phosphate that were originally identified as retroviral receptors in HUMANS and have been described in yeast and many other organisms.'),
  'D050612': NtO(MH='Fatty Acid Transport Proteins', MS='A broad category of membrane transport proteins that specifically transport FREE FATTY ACIDS across cellular membranes. They play an important role in LIPID METABOLISM in CELLS that utilize free fatty acids as an energy source.'),
  'D050613': NtO(MH='Sodium-Phosphate Cotransporter Proteins, Type IIa', MS='An electrogenic sodium-dependent phosphate transporter. It is present primarily in BRUSH BORDER membranes of PROXIMAL RENAL TUBULES.'),
  'D050614': NtO(MH='Sodium-Phosphate Cotransporter Proteins, Type IIb', MS='A sodium-dependent phosphate transporter present primarily at apical sites of EPITHELIAL CELLS in the SMALL INTESTINE.'),
  'D050615': NtO(MH='Sodium-Phosphate Cotransporter Proteins, Type IIc', MS='A non-electrogenic sodium-dependent phosphate transporter. It is found primarily in apical membranes of PROXIMAL RENAL TUBULES.'),
  'D050616': NtO(MH='Balaenoptera', MS="A genus of WHALES in the family Balaenopteridae, consisting of five species: Blue Whale, Bryde's Whale, FIN WHALE, Sei Whale, and MINKE WHALE. They are distinguished by a relatively slender body, a compressed tail stock, and a pointed snout."),
  'D050617': NtO(MH='Fin Whale', MS='The species Balaenoptera physalus, in the family Balaenopteridae, characterized by a large, strongly curved, dorsal fin. It is the second largest of the WHALES, highly migratory, but rarely seen near the shore.'),
  'D050618': NtO(MH='Glycolaldehyde Dehydrogenase', MS='An enzyme that catalyzes the oxidation of glycolaldehyde to glycolate, using NAD and NADP as COENZYMES.'),
  'D050636': NtO(MH='Minke Whale', MS='The species Balaenoptera acutorostrata, in the family Balaenopteridae. It is the smallest of the WHALES in the family and though mainly oceanic, is often found in coastal waters including bays and estuaries.'),
  'D050637': NtO(MH='Receptors, Dopamine D3', MS='A subtype of dopamine D2 receptors that are highly expressed in the LIMBIC SYSTEM of the brain.'),
  'D050638': NtO(MH='Receptors, Dopamine D4', MS='A subtype of dopamine D2 receptors that has high affinity for the antipsychotic CLOZAPINE.'),
  'D050640': NtO(MH='Humpback Whale', MS='The species Megaptera novaeangliae, in the family Balaenopteridae, characterized by its huge flippers and the arching of their back when diving. They are also known for their breaching and singing.'),
  'D050641': NtO(MH='Receptors, Dopamine D5', MS='A subtype of dopamine D1 receptors that has higher affinity for DOPAMINE and differentially couples to GTP-BINDING PROTEINS.'),
  'D050642': NtO(MH='Bowhead Whale', MS='The species Balaena mysticetus, in the family Balaenidae, found in the colder waters of the Northern Hemisphere. The common name is derived from the extreme arching of the lower jaw.'),
  'D050643': NtO(MH='Beluga Whale', MS='The species Delphinapterus leucas, in the family Monodontidae, found primarily in the Arctic Ocean and adjoining seas. They are small WHALES lacking a dorsal fin.'),
  'D050644': NtO(MH='Succinate-Semialdehyde Dehydrogenase', MS='An enzyme that plays a role in the GLUTAMATE and butanoate metabolism pathways by catalyzing the oxidation of succinate semialdehyde to SUCCINATE using NAD+ as a coenzyme. Deficiency of this enzyme, causes 4-hydroxybutyricaciduria, a rare inborn error in the metabolism of the neurotransmitter 4-aminobutyric acid (GABA).'),
  'D050645': NtO(MH='2-Oxoisovalerate Dehydrogenase (Acylating)', MS='An NAD+ dependent enzyme that catalyzes the oxidation 3-methyl-2-oxobutanoate to 2-methylpropanoyl-CoA. It plays a role in the degradation of VALINE; LEUCINE; and ISOLEUCINE.'),
  'D050646': NtO(MH='Methylmalonate-Semialdehyde Dehydrogenase (Acylating)', MS='An enzyme that plays a role in the VALINE; LEUCINE; and ISOLEUCINE catabolic pathways by catalyzing the oxidation of 2-methyl-3-oxopropanate to propanoyl-CoA using NAD+ as a coenzyme. Methylmalonate semialdehyde dehydrogenase deficiency is characterized by elevated BETA-ALANINE and 3-hydropropionic acid.'),
  'D050647': NtO(MH='L-Aminoadipate-Semialdehyde Dehydrogenase', MS='An enzyme that catalyzes the oxidation of L-2-aminoadipate 6-semialdehyde to L-2-aminoadipate (alpha-aminoadipic acid). It is involved in the biosynthetic pathway of LYSINE.'),
  'D050648': NtO(MH='Aminomuconate-Semialdehyde Dehydrogenase', MS='An NAD+ dependent enzyme that catalyzes the oxidation of 2-aminomuconate 6-semialdehyde to 2-aminomuconate.'),
  'D050656': NtO(MH='Transcription Factor AP-2', MS='A family of DNA binding proteins that regulate expression of a variety of GENES during CELL DIFFERENTIATION and APOPTOSIS. Family members contain a highly conserved carboxy-terminal basic HELIX-TURN-HELIX MOTIF involved in dimerization and sequence-specific DNA binding.'),
  'D050657': NtO(MH='Core Binding Factor alpha Subunits', MS='A family of transcription factors that bind to the cofactor CORE BINDING FACTOR BETA SUBUNIT to form core binding factor. Family members contain a highly conserved DNA-binding domain known as the runt domain. They can act as both activators and repressors of expression of GENES involved in CELL DIFFERENTIATION and CELL CYCLE progression.'),
  'D050658': NtO(MH='Core Binding Factor beta Subunit', MS='A non-DNA binding transcription factor that is a subunit of core binding factor. It forms heterodimeric complexes with CORE BINDING FACTOR ALPHA SUBUNITS, and regulates GENETIC TRANSCRIPTION of a variety of GENES involved primarily in CELL DIFFERENTIATION and CELL CYCLE progression.'),
  'D050659': NtO(MH='Core Binding Factor Alpha 1 Subunit', MS='A transcription factor that dimerizes with CORE BINDING FACTOR BETA SUBUNIT to form core binding factor. It contains a highly conserved DNA-binding domain known as the runt domain and is involved in genetic regulation of skeletal development and CELL DIFFERENTIATION.'),
  'D050660': NtO(MH='Core Binding Factors', MS='Heterodimeric transcription factors containing a DNA-binding alpha subunits, (CORE BINDING FACTOR ALPHA SUBUNITS), along with a non-DNA-binding beta subunits, CORE BINDING FACTOR BETA SUBUNIT. Core Binding Factor regulates GENETIC TRANSCRIPTION of a variety of GENES involved primarily in CELL DIFFERENTIATION and CELL CYCLE progression.'),
  'D050661': NtO(MH='Sperm Whale', MS='The species Physeter catodon (also called Physeter macrocephalus), in the family Physeteridae. The common name is derived from the milky wax substance in its head (spermaceti). The species also produces an intestinal secretion AMBERGRIS, which was previously used in perfumes. The sperm whale is the largest toothed MAMMAL in the world.'),
  'D050676': NtO(MH='Core Binding Factor Alpha 2 Subunit', MS='A transcription factor that dimerizes with the cofactor CORE BINDING FACTOR BETA SUBUNIT to form core binding factor. It contains a highly conserved DNA-binding domain known as the runt domain. Runx1 is frequently mutated in human LEUKEMIAS.'),
  'D050677': NtO(MH='Core Binding Factor Alpha 3 Subunit', MS='A transcription factor that dimerizes with the cofactor CORE BINDING FACTOR BETA SUBUNIT to form core binding factor. It contains a highly conserved DNA-binding domain known as the runt domain.'),
  'D050678': NtO(MH='Complement C2a', MS='The COOH-terminal fragment of COMPLEMENT 2, released by the action of activated COMPLEMENT C1S. It is a SERINE PROTEASE. C2a combines with COMPLEMENT C4B to form C4b2a (CLASSICAL PATHWAY C3 CONVERTASE) and subsequent C4b2a3b (CLASSICAL PATHWAY C5 CONVERTASE).'),
  'D050679': NtO(MH='Complement C2b', MS='The N-terminal fragment of COMPLEMENT 2, released by the action of activated COMPLEMENT C1S.'),
  'D050680': NtO(MH='Transcription Factor DP1', MS='A transcription factor that possesses DNA-binding and E2F-binding domains but lacks a transcriptional activation domain. It is a binding partner for E2F TRANSCRIPTION FACTORS and enhances the DNA binding and transactivation function of the DP-E2F complex.'),
  'D050681': NtO(MH='Bottle-Nosed Dolphin', MS='The species Tursiops truncatus, in the family Delphinidae, characterized by a bottle-shaped beak and slightly hooked broad dorsal fin.'),
  'D050682': NtO(MH='Q-SNARE Proteins', MS='SNARE proteins in which the central amino acid residue of the SNARE motif is a GLUTAMINE. They are classified separately from the R-SNARE PROTEINS where the central amino acid residue of the SNARE motif is an ARGININE. Subfamilies, the QA-SNARES; QB-SNARES; and QC-SNARES are grouped by the position of their SNARE motif-containing-domains in the SNARE complex and by their sequence similarities.'),
  'D050683': NtO(MH='R-SNARE Proteins', MS='SNARE proteins where the central amino acid residue of the SNARE motif is an ARGININE. They are classified separately from the Q-SNARE PROTEINS where the central amino acid residue of the SNARE motif is a GLUTAMINE. This subfamily contains the vesicle associated membrane proteins (VAMPs) based on similarity to the prototype for the R-SNAREs, VAMP2 (synaptobrevin 2).'),
  'D050684': NtO(MH='E2F Transcription Factors', MS='A family of basic helix-loop-helix transcription factors that control expression of a variety of GENES involved in CELL CYCLE regulation. E2F transcription factors typically form heterodimeric complexes with TRANSCRIPTION FACTOR DP1 or transcription factor DP2, and they have N-terminal DNA binding and dimerization domains. E2F transcription factors can act as mediators of transcriptional repression or transcriptional activation.'),
  'D050685': NtO(MH='Whale, Killer', MS='The species Orcinus orca, in the family Delphinidae, characterized by its black and white coloration, and huge triangular dorsal fin. It is the largest member of the DOLPHINS and derives its name from the fact that it is a fearsome predator.'),
  'D050686': NtO(MH='Whales, Pilot', MS='The genus Globicephala, in the family Delphinidae, consisting of two species of DOLPHINS. They are mostly black with a stocky shape and bulbous foreheads.'),
  'D050687': NtO(MH='E2F1 Transcription Factor', MS='An E2F transcription factor that interacts directly with RETINOBLASTOMA PROTEIN and CYCLIN A and activates GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis. E2F1 is involved in DNA REPAIR and APOPTOSIS.'),
  'D050696': NtO(MH='Common Dolphins', MS='The genus Delphinus, in the family Delphinidae, consisting of two species of DOLPHINS. They are multicolored, with a characteristic yellow-tan criss-cross hourglass pattern behind the eyes.'),
  'D050697': NtO(MH='Retinal Dehydrogenase', MS='A metalloflavoprotein enzyme involved the metabolism of VITAMIN A, this enzyme catalyzes the oxidation of RETINAL to RETINOIC ACID, using both NAD+ and FAD coenzymes. It also acts on both the 11-trans- and 13-cis-forms of RETINAL.'),
  'D050698': NtO(MH='Glutamate-5-Semialdehyde Dehydrogenase', MS='An NADP+ dependent enzyme that catalyzes the oxidation of L-glutamate 5-semialdehyde to L-glutamyl 5-phosphate. It plays a role in the urea cycle and metabolism of amino groups.'),
  'D050699': NtO(MH='E2F3 Transcription Factor', MS='An E2F transcription factor that interacts directly with RETINOBLASTOMA PROTEIN and CYCLIN A. E2F3 regulates GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis.'),
  'D050700': NtO(MH='E2F5 Transcription Factor', MS='An E2F transcription factor that represses GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis. E2F5 recruits chromatin remodeling factors indirectly to target gene promoters through RETINOBLASTOMA LIKE PROTEIN P130.'),
  'D050701': NtO(MH='E2F6 Transcription Factor', MS='An E2F transcription factor that represses GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis. E2F6 recruits chromatin remodeling factors directly to target gene promoters and lacks the transactivation domain responsible for binding to the retinoblastoma family of tumor suppressors.'),
  'D050702': NtO(MH='E2F7 Transcription Factor', MS='An E2F transcription factor that represses GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis. E2F7 has two separate DNA-binding domains and binds to DNA independently of a dimerization partner.'),
  'D050703': NtO(MH='E2F2 Transcription Factor', MS='An E2F transcription factor that interacts directly with RETINOBLASTOMA PROTEIN and CYCLIN A. E2F2 activates GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis.'),
  'D050704': NtO(MH='E2F4 Transcription Factor', MS='An E2F transcription factor that represses GENETIC TRANSCRIPTION required for CELL CYCLE entry and DNA synthesis. E2F4 recruits chromatin remodeling factors indirectly to target gene PROMOTER REGIONS through RETINOBLASTOMA LIKE PROTEIN P130 and RETINOBLASTOMA LIKE PROTEIN P107.'),
  'D050705': NtO(MH='Receptors, Guanylate Cyclase-Coupled', MS='A class of cellular membrane receptors that either have an intrinsic guanylate cyclase activity or are closely coupled to specific guanylate cyclases within the cell.'),
  'D050706': NtO(MH='Stenella', MS='A genus comprised of spinner, spotted, and striped DOLPHINS, in the family Delphinidae. Schools of Stenella, that may number in the thousands, often associate with schools of TUNA, and are thus vulnerable to accidental harvesting.'),
  'D050716': NtO(MH='Complement C4b-Binding Protein', MS='A serum protein that regulates the CLASSICAL COMPLEMENT ACTIVATION PATHWAY. It binds as a cofactor to COMPLEMENT FACTOR I which then hydrolyzes the COMPLEMENT C4B in the CLASSICAL PATHWAY C3 CONVERTASE (C4bC2a).'),
  'D050717': NtO(MH='Retinoblastoma-Like Protein p130', MS='A negative regulator of the CELL CYCLE that undergoes PHOSPHORYLATION by CYCLIN-DEPENDENT KINASES. RBL2 contains a conserved pocket region that binds E2F4 TRANSCRIPTION FACTOR and E2F5 TRANSCRIPTION FACTOR. RBL2 also interacts with viral ONCOPROTEINS such as POLYOMAVIRUS TUMOR ANTIGENS; ADENOVIRUS E1A PROTEINS; and PAPILLOMAVIRUS E7 PROTEINS.'),
  'D050718': NtO(MH='Complement C1 Inhibitor Protein', MS='An endogenous 105-kDa plasma glycoprotein produced primarily by the LIVER and MONOCYTES. It inhibits a broad spectrum of proteases, including the COMPLEMENT C1R and the COMPLEMENT C1S proteases of the CLASSICAL COMPLEMENT PATHWAY, and the MANNOSE-BINDING PROTEIN-ASSOCIATED SERINE PROTEASES. C1-INH-deficient individuals suffer from HEREDITARY ANGIOEDEMA TYPES I AND II.'),
  'D050719': NtO(MH='Crk-Associated Substrate Protein', MS='A highly phosphorylated 130 kDa protein that associates with ONCOGENE PROTEIN CRK and ONCOGENE PROTEIN SRC. It undergoes tyrosine PHOSPHORYLATION in signaling pathways that regulate CELL MIGRATION and CELL PROLIFERATION.'),
  'D050720': NtO(MH='Retinoblastoma-Like Protein p107', MS='A negative regulator of the CELL CYCLE that undergoes PHOSPHORYLATION by CYCLIN-DEPENDENT KINASES. It contains a conserved pocket region that binds E2F4 TRANSCRIPTION FACTOR and interacts with viral ONCOPROTEINS such as POLYOMAVIRUS TUMOR ANTIGENS; ADENOVIRUS E1A PROTEINS; and PAPILLOMAVIRUS E7 PROTEINS.'),
  'D050721': NtO(MH='Proto-Oncogene Proteins c-cbl', MS='Proto-oncogene proteins that negatively regulate RECEPTOR PROTEIN-TYROSINE KINASE signaling. It is a UBIQUITIN-PROTEIN LIGASE and the cellular homologue of ONCOGENE PROTEIN V-CBL.'),
  'D050722': NtO(MH='Oncogene Protein v-cbl', MS='An oncoprotein from the Cas NS-1 murine retrovirus that induces pre- B-CELL LYMPHOMA and MYELOID LEUKEMIAS. v-cbl protein is a tyrosine-phosphorylated, truncated form of its cellular homologue, PROTO-ONCOGENE PROTEIN C-CBL.'),
  'D050723': NtO(MH='Fractures, Bone', MS='Breaks in bones.'),
  'D050724': NtO(MH='Fractures, Cartilage', MS='Breaks in CARTILAGE.'),
  'D050725': NtO(MH='Papillomavirus E7 Proteins', MS='ONCOGENE PROTEINS from papillomavirus that deregulate the CELL CYCLE of infected cells and lead to NEOPLASTIC CELL TRANSFORMATION. Papillomavirus E7 proteins have been shown to interact with various regulators of the cell cycle including RETINOBLASTOMA PROTEIN and certain cyclin-dependent kinase inhibitors.'),
  'D050727': NtO(MH='Benzaldehyde Dehydrogenase (NADP+)', MS='An NADP+ dependent enzyme that catalyzes the oxidation of benzaldehyde to BENZOIC ACID. It also plays a role in the degradation of TOLUENE and XYLENE.'),
  'D050728': NtO(MH='Betaine-Aldehyde Dehydrogenase', MS='An NAD+ dependent enzyme that catalyzes the oxidation of betain aldehyde to BETAINE.'),
  'D050736': NtO(MH='Oncogene Protein v-crk', MS='A signal transducing adaptor protein that is encoded by the crk ONCOGENE from TYPE C AVIAN RETROVIRUSES. It contains SRC HOMOLOGY DOMAINS and is closely related to its cellular homolog, PROTO-ONCOGENE PROTEIN C-CRK.'),
  'D050737': NtO(MH='Proto-Oncogene Proteins c-crk', MS='Signal transducing adaptor proteins that contain SRC HOMOLOGY DOMAINS and play a role in CYTOSKELETON reorganization. c-crk protein is closely related to ONCOGENE PROTEIN V-CRK and includes several alternatively spliced isoforms.'),
  'D050738': NtO(MH='Mesomycetozoea Infections', MS='Infections by MESOMYCETOZOEA, general or unspecified.'),
  'D050741': NtO(MH='Phocoena', MS='A genus of PORPOISES, in the family Phocoenidae, comprised of several species. They frequent coastal waters, bays, estuaries, and the mouths of large rivers.'),
  'D050743': NtO(MH='Pyruvate Synthase', MS='A ferredoxin-containing enzyme that catalyzes the COENZYME A-dependent oxidative decarboxylation of PYRUVATE to acetyl-COENZYME A and CARBON DIOXIDE.'),
  'D050744': NtO(MH='Dihydrouracil Dehydrogenase (NAD+)', MS='An enzyme that catalyzes the oxidation of 5,6-dihydrouracil to URACIL using NAD as a cofactor. This enzyme also plays a role in the catabolism of the antimetabolite 5-FLUOROURACIL.'),
  'D050745': NtO(MH='Enoyl-(Acyl-Carrier Protein) Reductase (NADPH, B-Specific)', MS='An enzyme that catalyzes the oxidation of acyl-[acyl-carrier protein] to trans-2,3-dehydroacyl-[acyl-carrier protein] in the fatty acid biosynthesis pathway. It has a preference for acyl derivatives with carbon chain length from 4 to 16.'),
  'D050746': NtO(MH='Dihydrodipicolinate Reductase', MS='An enzyme that catalyzes the oxidation of 2,3,4,5-tetrahydrodipicolinate to 2,3-dihydrodipicolinate using NAD(P)+ as a cofactor. It is found in BACTERIA and higher plants involved in the biosynthesis of DIAMINOPIMELIC ACID and LYSINE.'),
  'D050753': NtO(MH='Enoyl-(Acyl-Carrier-Protein) Reductase (NADH)', MS='An NAD-dependent enzyme that catalyzes the oxidation of acyl-[acyl-carrier protein] to trans-2,3-dehydroacyl-[acyl-carrier protein]. It has a preference for acyl groups with a carbon chain length between 4 to 16.'),
  'D050756': NtO(MH='Cyclin-Dependent Kinase Inhibitor Proteins', MS='A group of cell cycle proteins that negatively regulate the activity of CYCLIN/CYCLIN-DEPENDENT KINASE complexes. They inhibit CELL CYCLE progression and help control CELL PROLIFERATION following GENOTOXIC STRESS as well as during CELL DIFFERENTIATION.'),
  'D050759': NtO(MH='Cyclin-Dependent Kinase Inhibitor p21', MS='A cyclin-dependent kinase inhibitor that mediates TUMOR SUPPRESSOR PROTEIN P53-dependent CELL CYCLE arrest. p21 interacts with a range of CYCLIN-DEPENDENT KINASES and associates with PROLIFERATING CELL NUCLEAR ANTIGEN and CASPASE 3.'),
  'D050760': NtO(MH='Cyclin-Dependent Kinase Inhibitor p27', MS='A cyclin-dependent kinase inhibitor that coordinates the activation of CYCLIN and CYCLIN-DEPENDENT KINASES during the CELL CYCLE. It interacts with active CYCLIN D complexed to CYCLIN-DEPENDENT KINASE 4 in proliferating cells, while in arrested cells it binds and inhibits CYCLIN E complexed to CYCLIN-DEPENDENT KINASE 2.'),
  'D050761': NtO(MH='Cyclin-Dependent Kinase Inhibitor p57', MS='A potent inhibitor of CYCLIN-DEPENDENT KINASES in G1 PHASE and S PHASE. In humans, aberrant expression of p57 is associated with various NEOPLASMS as well as with BECKWITH-WIEDEMANN SYNDROME.'),
  'D050762': NtO(MH='Cyclin-Dependent Kinase Inhibitor p15', MS='An INK4 cyclin-dependent kinase inhibitor containing four ANKYRIN-LIKE REPEATS. INK4B is often inactivated by deletions, mutations, or hypermethylation in HEMATOLOGIC NEOPLASMS.'),
  'D050763': NtO(MH='Cyclin-Dependent Kinase Inhibitor p18', MS='An INK4 cyclin-dependent kinase inhibitor containing five ANKYRIN-LIKE REPEATS. Aberrant expression of this protein has been associated with deregulated EPITHELIAL CELL growth, organ enlargement, and a variety of NEOPLASMS.'),
  'D050764': NtO(MH='Cyclin-Dependent Kinase Inhibitor p19', MS='An INK4 cyclin-dependent kinase inhibitor containing five ANKYRIN REPEATS. Aberrant expression of this protein has been associated with TESTICULAR CANCER.'),
  'D050765': NtO(MH='Qa-SNARE Proteins', MS='A subfamily of Q-SNARE PROTEINS which occupy the same position as syntaxin 1A in the SNARE complex and which also are most similar to syntaxin 1A in their AMINO ACID SEQUENCE. This subfamily is also known as the syntaxins, although a few so called syntaxins are Qc-SNARES.'),
  'D050766': NtO(MH='Qb-SNARE Proteins', MS='A subfamily of Q-SNARE PROTEINS which occupy the same position in the SNARE complex as the N-terminal SNARE domain of SNAP-25 and which also are most similar to the N-terminal region of SNAP-25 in their AMINO ACID SEQUENCE.'),
  'D050767': NtO(MH='Qc-SNARE Proteins', MS='A subfamily of Q-SNARE PROTEINS which occupy the same position in the SNARE complex as the C-terminal SNARE domain of SNAP-25 and which also are most similar to the C-terminal region of SNAP-25 in their AMINO ACID SEQUENCE.'),
  'D050768': NtO(MH='Protoporphyrinogen Oxidase', MS='A membrane-bound flavoenzyme that catalyzes the oxygen-dependent aromatization of protoporphyrinogen IX (Protogen) to protoporphyrin IX (Proto IX). It is the last enzyme of the common branch of the HEME and CHLOROPHYLL pathways in plants, and is the molecular target of diphenyl ether-type herbicides. VARIEGATE PORPHYRIA is an autosomal dominant disorder associated with deficiency of protoporphyrinogen oxidase.'),
  'D050769': NtO(MH='Isovaleryl-CoA Dehydrogenase', MS='A mitochondrial flavoprotein, this enzyme catalyzes the oxidation of 3-methylbutanoyl-CoA to 3-methylbut-2-enoyl-CoA using FAD as a cofactor. Defects in the enzyme, is associated with isovaleric acidemia (IVA).'),
  'D050770': NtO(MH='Glutaryl-CoA Dehydrogenase', MS='A flavoprotein enzyme that is responsible for the catabolism of LYSINE; HYDROXYLYSINE; and TRYPTOPHAN. It catalyzes the oxidation of GLUTARYL-CoA to crotonoyl-CoA using FAD as a cofactor. Glutaric aciduria type I is an inborn error of metabolism due to the deficiency of glutaryl-CoA dehydrogenase.'),
  'D050776': NtO(MH='Complement C5b', MS="The larger fragment generated from the cleavage of C5 by C5 CONVERTASE that yields COMPLEMENT C5A and C5b (beta chain + alpha' chain, the residual alpha chain, bound by disulfide bond). C5b remains bound to the membrane and initiates the spontaneous assembly of the late complement components to form C5b-8-poly-C9, the MEMBRANE ATTACK COMPLEX."),
  'D050777': NtO(MH='Stathmin', MS='A ubiquitous phosphoprotein that serves as an intracellular substrate for a variety of SIGNAL TRANSDUCTION PATHWAYS. PHOSPHORYLATION of stathmin occurs during CELL CYCLE progression, and stathmin functions as a microtubule-destabilizing protein that promotes MICROTUBULE depolymerization during INTERPHASE and late MITOSIS. Stathmin is expressed at very high levels in a variety of human CANCERS.'),
  'D050778': NtO(MH='NFATC Transcription Factors', MS='A family of transcription factors characterized by the presence of highly conserved calcineurin- and DNA-binding domains. NFAT proteins are activated in the CYTOPLASM by the calcium-dependent phosphatase CALCINEURIN. They transduce calcium signals to the nucleus where they can interact with TRANSCRIPTION FACTOR AP-1 or NF-KAPPA B and initiate GENETIC TRANSCRIPTION of GENES involved in CELL DIFFERENTIATION and development. NFAT proteins stimulate T-CELL activation through the induction of IMMEDIATE-EARLY GENES such as INTERLEUKIN-2.'),
  'D050779': NtO(MH='Nuclear Factor 90 Proteins', MS='A family of double-stranded RNA-binding proteins that are related to NFATC TRANSCRIPTION FACTORS. In addition to binding to RNA, nuclear factor 90 proteins form heterodimeric complexes that regulate GENETIC TRANSCRIPTION and may play a role in T-CELL activation.'),
  'D050781': NtO(MH='Transcranial Magnetic Stimulation', MS='A technique that involves the use of electrical coils on the head to generate a brief magnetic field which reaches the CEREBRAL CORTEX. It is coupled with ELECTROMYOGRAPHY response detection to assess cortical excitability by the threshold required to induce MOTOR EVOKED POTENTIALS. This method is also used for BRAIN MAPPING, to study NEUROPHYSIOLOGY, and as a substitute for ELECTROCONVULSIVE THERAPY for treating DEPRESSION. Induction of SEIZURES limits its clinical usage.'),
  'D050782': NtO(MH='Esophageal pH Monitoring', MS='Analysis of the HYDROGEN ION CONCENTRATION in the lumen of the ESOPHAGUS. It is used to record the pattern, frequency, and duration of GASTROESOPHAGEAL REFLUX.'),
  'D050783': NtO(MH='Proto-Oncogene Proteins c-ets', MS='A family of transcription factors that share a unique DNA-binding domain. The name derives from viral oncogene-derived protein oncogene protein v-ets of the AVIAN ERYTHROBLASTOSIS VIRUS.'),
  'D050784': NtO(MH='Proto-Oncogene Protein c-ets-1', MS='An ets proto-oncogene protein expressed primarily in adult LYMPHOID TISSUE; BRAIN; and VASCULAR ENDOTHELIAL CELLS.'),
  'D050785': NtO(MH='Cryoanesthesia', MS='ANESTHESIA achieved by lowering either BODY TEMPERATURE (core cooling) or SKIN TEMPERATURE (external cooling).'),
  'D050786': NtO(MH='Gastric Hypothermia', MS='A method of lowering core BODY TEMPERATURE by filling the STOMACH with chilled fluids.'),
  'D050787': NtO(MH='Proto-Oncogene Protein c-ets-2', MS='A ubiquitously expressed ets proto-oncogene protein that may play a role in regulation of CELL PROLIFERATION and CELL DIFFERENTIATION.'),
  'D050788': NtO(MH='Alanine Dehydrogenase', MS='An NAD-dependent enzyme that catalyzes the reversible DEAMINATION of L-ALANINE to PYRUVATE and AMMONIA. The enzyme is needed for growth when ALANINE is the sole CARBON or NITROGEN source. It may also play a role in CELL WALL synthesis because L-ALANINE is an important constituent of the PEPTIDOGLYCAN layer.'),
  'D050789': NtO(MH='Glycine Dehydrogenase', MS='An oxidoreductase that catalyzes the oxidative DEAMINATION of GLYCINE to glyoxylate and AMMONIA in the presence of NAD. In BACTERIA lacking transaminating pathways the enzyme can act in the reverse direction to synthesize glycine from glyoxylate and ammonia and NADH.'),
  'D050790': NtO(MH='Glutamate Synthase (NADH)', MS='A FLAVOPROTEIN enzyme for AMMONIA assimilation in BACTERIA, microorganisms and PLANTS. It catalyzes the oxidation of 2 molecules of L-GLUTAMATE to generate L-GLUTAMINE and 2-oxoglutarate in the presence of NAD+.'),
  'D050791': NtO(MH='STAT Transcription Factors', MS='A family of transcription factors containing SH2 DOMAINS that are involved in CYTOKINE-mediated SIGNAL TRANSDUCTION. STAT transcription factors are recruited to the cytoplasmic region of CELL SURFACE RECEPTORS and are activated via PHOSPHORYLATION. Once activated they dimerize and translocate into the CELL NUCLEUS where they influence GENE expression. They play a role in regulating CELL GROWTH PROCESSES and CELL DIFFERENTIATION. STAT transcription factors are inhibited by SUPPRESSOR OF CYTOKINE SIGNALING PROTEINS and PROTEIN INHIBITORS OF ACTIVATED STAT.'),
  'D050792': NtO(MH='Circulatory Arrest, Deep Hypothermia Induced', MS='A technique to arrest the flow of blood by lowering BODY TEMPERATURE to about 20 degrees Centigrade, usually achieved by infusing chilled perfusate. The technique provides a bloodless surgical field for complex surgeries.'),
  'D050793': NtO(MH='Orthodontic Extrusion', MS='Orthodontic movement in the coronal direction achieved by outward tension on the PERIODONTAL LIGAMENT. It does not include the operative procedure that CROWN LENGTHENING involves.'),
  'D050794': NtO(MH='STAT1 Transcription Factor', MS='A signal transducer and activator of transcription that mediates cellular responses to INTERFERONS. Stat1 interacts with P53 TUMOR SUPPRESSOR PROTEIN and regulates expression of GENES involved in growth control and APOPTOSIS.'),
  'D050795': NtO(MH='STAT2 Transcription Factor', MS='A signal transducer and activator of transcription that mediates cellular responses to TYPE I INTERFERONS. Stat2 protein is associated constitutively with INTERFERON REGULATORY FACTOR-9. After PHOSPHORYLATION Stat2 forms the IFN-STIMULATED GENE FACTOR 3 COMPLEX to regulate expression of target GENES.'),
  'D050796': NtO(MH='STAT3 Transcription Factor', MS='A signal transducer and activator of transcription that mediates cellular responses to INTERLEUKIN-6 family members. STAT3 is constitutively activated in a variety of TUMORS and is a major downstream transducer for the CYTOKINE RECEPTOR GP130.'),
  'D050797': NtO(MH='STAT4 Transcription Factor', MS='A signal transducer and activator of transcription that mediates cellular responses to INTERLEUKIN-12 in T-LYMPHOCYTES. Stat4 is an important signaling molecule for differentiation in TH1 CELLS.'),
  'D050798': NtO(MH='SUNCT Syndrome', MS='A primary headache disorder that is characterized by frequent short-lasting, unilateral, neuralgiform pain attacks in the ocular area, with CONJUNCTIVA fluid-filling and tearing. SUNCT syndrome is usually resistant to treatment.'),
  'D050799': NtO(MH='STAT5 Transcription Factor', MS='A signal transducer and activator of transcription that mediates cellular responses to a variety of CYTOKINES. Stat5 activation is associated with transcription of CELL CYCLE regulators such as CYCLIN KINASE INHIBITOR P21 and anti-apoptotic genes such as BCL-2 GENES. Stat5 is constitutively activated in many patients with acute MYELOID LEUKEMIA.'),
  'D050800': NtO(MH='STAT6 Transcription Factor', MS='A signal transducer and activator of transcription that mediates cellular responses to INTERLEUKIN-4. Stat6 has been shown to partner with NF-KAPPA B and CCAAT-ENHANCER-BINDING PROTEINS to regulate GENETIC TRANSCRIPTION of interleukin-4 responsive GENES.'),
  'D050801': NtO(MH='Ternary Complex Factors', MS='A subclass of c-ets proto-oncogene proteins that were first described by their property of binding to DNA when associated with other regulatory proteins such as SERUM RESPONSE FACTOR. They contain an amino-terminal ets domain that binds to DNA along with centrally located SERUM RESPONSE FACTOR interacting domain, and carboxy-terminal map kinase activation domains. They play an important role in transcriptional regulation by INTRACELLULAR SIGNALING PEPTIDES AND PROTEINS.'),
  'D050802': NtO(MH='Valine Dehydrogenase (NADP+)', MS='An amino acid oxidoreductase that catalyzes the oxidative hydroxylation of L-VALINE to 3-methyl-2-oxobutanoate and AMMONIA in the presence of NADP.'),
  'D050803': NtO(MH='Leucine Dehydrogenase', MS='An octameric enzyme belonging to the superfamily of amino acid dehydrogenases. Leucine dehydrogenase catalyzes the reversible oxidative deamination of L-LEUCINE, to 4-methyl-2-oxopentanoate (2-ketoisocaproate) and AMMONIA, with the corresponding reduction of the cofactor NAD+.'),
  'D050804': NtO(MH='D-Aspartate Oxidase', MS='An FAD-dependent peroxisomal flavoenzyme, this catalyzes the oxidative deamination of D-ASPARTATE to OXALOACETATE and AMMONIA using oxygen as electron acceptor.'),
  'D050805': NtO(MH='Wandering Spleen', MS='A congenital or acquired condition in which the SPLEEN is not in its normal anatomical position but moves about in the ABDOMEN. This is due to laxity or absence of suspensory ligaments which normally provide peritoneal attachments to keep the SPLEEN in a fixed position. Clinical symptoms include ABDOMINAL PAIN, splenic torsion and ISCHEMIA.'),
  'D050806': NtO(MH='Arthroplasty, Replacement, Finger', MS='Partial or total replacement of one or more FINGERS, or a FINGER JOINT.'),
  'D050807': NtO(MH='ets-Domain Protein Elk-1', MS='A member of the ternary complex family of ets-related transcription factors that is regulated by MITOGEN-ACTIVATED PROTEIN KINASES including JNK MITOGEN-ACTIVATED PROTEIN KINASES; MITOGEN-ACTIVATED PROTEIN KINASE 1; MITOGEN-ACTIVATED PROTEIN KINASE 3; and P38 MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D050808': NtO(MH='ets-Domain Protein Elk-4', MS='A member of the ternary complex family of ets-related transcription factors that is regulated by MITOGEN-ACTIVATED PROTEIN KINASES such as EXTRACELLULAR SIGNAL-REGULATED MAP KINASES; and P38 MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D050809': NtO(MH='POU Domain Factors', MS='A family of transcription factors characterized by the presence of a bipartite DNA-binding domain known as the POU domain. The POU domain contains two subdomains, a POU-specific domain and a POU-homeodomain. The POU domain was originally identified as a region of approximately 150 amino acids shared between the Pit-1, Oct-1, Oct-2, and Unc-86 transcription factors.'),
  'D050810': NtO(MH='Octamer Transcription Factors', MS='A family of POU domain factors that bind the octamer motif ATTTGCAT in enhancer and PROMOTER REGIONS to regulate GENE EXPRESSION.'),
  'D050811': NtO(MH='Octamer Transcription Factor-1', MS='A ubiquitously expressed octamer transcription factor that regulates GENETIC TRANSCRIPTION of SMALL NUCLEAR RNA; IMMUNOGLOBULIN GENES; and HISTONE H2B genes.'),
  'D050812': NtO(MH='Split-Brain Procedure', MS='Complete severing of the CORPUS CALLOSUM. In humans it is usually performed to treat medically intractable, multifocal EPILEPSY. EXPERIMENTAL ANIMAL MODELS of split brain preparations are used in research.'),
  'D050813': NtO(MH='Octamer Transcription Factor-2', MS='An octamer transcription factor expressed primarily in B-LYMPHOCYTES and the developing CENTRAL NERVOUS SYSTEM that regulates GENETIC TRANSCRIPTION of IMMUNOGLOBULIN GENES.'),
  'D050814': NtO(MH='Octamer Transcription Factor-3', MS='An octamer transcription factor that is expressed primarily in totipotent embryonic STEM CELLS and GERM CELLS and is down-regulated during CELL DIFFERENTIATION.'),
  'D050815': NtO(MH='Fractures, Compression', MS='Crumbling or smashing of cancellous BONE by forces acting parallel to the long axis of bone. It is applied particularly to vertebral body fractures (SPINAL FRACTURES). (Blauvelt and Nelson, A Manual of Orthopedic Terminology, 1994, p4)'),
  'D050816': NtO(MH='Octamer Transcription Factor-6', MS='An octamer transcription factor that plays an important role in the MYELIN SHEATH development by SCHWANN CELLS.'),
  'D050817': NtO(MH='Transcription Factor Pit-1', MS='A POU domain factor that regulates expression of GROWTH HORMONE; PROLACTIN; and THYROTROPIN-BETA in the ANTERIOR PITUITARY GLAND.'),
  'D050818': NtO(MH='Transcription Factor Brn-3', MS='A family of mammalian POU domain factors that are expressed predominately in NEURONS.'),
  'D050819': NtO(MH='Transcription Factor Brn-3A', MS='A POU domain factor that activates GENETIC TRANSCRIPTION of GENES encoding NEUROFILAMENT PROTEINS; alpha internexin; SYNAPTOSOMAL-ASSOCIATED PROTEIN 25; and BCL-2 PROTO-ONCOGENE PROTEINS.'),
  'D050820': NtO(MH='Transcription Factor Brn-3B', MS='A POU domain factor that represses GENETIC TRANSCRIPTION of GENES encoding NEUROFILAMENT PROTEINS, alpha internexin, and SYNAPTOSOMAL-ASSOCIATED PROTEIN 25.'),
  'D050821': NtO(MH='Transcription Factor Brn-3C', MS='A POU domain factor that activates neuronal cell GENETIC TRANSCRIPTION of GENES encoding NEUROFILAMENT PROTEINS, alpha internexin, and SYNAPTOSOMAL-ASSOCIATED PROTEIN 25. Mutations in the Brn-3c gene have been associated with DEAFNESS.'),
  'D050822': NtO(MH='Cytokine Receptor gp130', MS='A cytokine receptor that contains extracellular FIBRONECTIN TYPE III DOMAINS which mediate its function through the formation of oligomeric complexes with itself, or with a variety of CYTOKINE RECEPTORS. It does not bind IL-6, but activates JANUS KINASES and STAT3 TRANSCRIPTION FACTOR.'),
  'D050823': NtO(MH='Hand Joints', MS='The articulations extending from the WRIST distally to the FINGERS. These include the WRIST JOINT; CARPAL JOINTS; METACARPOPHALANGEAL JOINT; and FINGER JOINT.'),
  'D050824': NtO(MH='Carpal Joints', MS='The articulations between the various CARPAL BONES. This does not include the WRIST JOINT which consists of the articulations between the RADIUS; ULNA; and proximal CARPAL BONES.'),
  'D050825': NtO(MH='Synaptosomal-Associated Protein 25', MS='A ubiquitous target SNARE protein that interacts with SYNTAXIN and SYNAPTOBREVIN. It is a core component of the machinery for intracellular MEMBRANE FUSION. The sequence contains 2 SNARE domains, one is the prototype for the Qb-SNARES, and the other is the prototype for the Qc-SNARES.'),
  'D050826': NtO(MH='Suppressor of Cytokine Signaling Proteins', MS='A family of structurally related proteins that are induced by CYTOKINES and negatively regulate cytokine-mediated SIGNAL TRANSDUCTION PATHWAYS. SOCS proteins contain a central SH2 DOMAIN and a C-terminal region of homology known as the SOCS box.'),
  'D050827': NtO(MH='Syntaxin 1', MS='A neuronal cell membrane protein that combines with SNAP-25 and SYNAPTOBREVIN 2 to form a SNARE complex that leads to EXOCYTOSIS.'),
  'D050828': NtO(MH='Protein Inhibitors of Activated STAT', MS='A family of structurally related proteins that are constitutively expressed and that negatively regulate cytokine-mediated SIGNAL TRANSDUCTION PATHWAYS. PIAS proteins inhibit the activity of signal transducers and activators of transcription.'),
  'D050830': NtO(MH='L-Amino Acid Oxidase', MS='An enzyme that catalyzes the oxidative deamination of L-amino acids to KETO ACIDS with the generation of AMMONIA and HYDROGEN PEROXIDE. L-amino acid oxidase is widely distributed in and is thought to contribute to the toxicity of SNAKE VENOMS.'),
  'D050832': NtO(MH='Interferon-Stimulated Gene Factor 3', MS='A multimeric complex that functions as a ligand-dependent transcription factor. ISGF3 is assembled in the CYTOPLASM and translocated to the CELL NUCLEUS in response to INTERFERON signaling. It consists of ISGF3-GAMMA and ISGF3-ALPHA, and it regulates expression of many interferon-responsive GENES.'),
  'D050833': NtO(MH='Interferon-Stimulated Gene Factor 3, alpha Subunit', MS='A heterodimeric complex of STAT1 PROTEIN and STAT2 PROTEIN that interacts with ISGF3GAMMA and is a subunit of the ISGF3 COMPLEX.'),
  'D050834': NtO(MH='Interferon-Stimulated Gene Factor 3, gamma Subunit', MS='An interferon regulatory factor that recruits STAT1 PROTEIN and STAT2 PROTEIN heterodimers to interferon-stimulated response elements and functions as an immediate-early protein.'),
  'D050835': NtO(MH='Interferon Regulatory Factors', MS='A family of transcription factors that share an N-terminal HELIX-TURN-HELIX MOTIF and bind INTERFERON-inducible promoters to control GENE expression. IRF proteins bind specific DNA sequences such as interferon-stimulated response elements, interferon regulatory elements, and the interferon consensus sequence.'),
  'D050836': NtO(MH='Interferon Regulatory Factor-2', MS='An interferon regulatory factor that represses transcription of TYPE I INTERFERONS and activates transcription of HISTONE H4.'),
  'D050837': NtO(MH='Oxidoreductases Acting on Aldehyde or Oxo Group Donors', MS='A broad category of oxidoreductases that either reduce double bonds or oxidize single bonds between OXYGEN and CARBON in organic compounds.'),
  'D050838': NtO(MH='Interferon Regulatory Factor-3', MS='An interferon regulatory factor that is expressed constitutively and undergoes POST-TRANSLATIONAL MODIFICATION following viral infection. PHOSPHORYLATION of IRF-3 causes the protein to be translocated from the CYTOPLASM to CELL NUCLEUS where it binds DNA, and activates transcription.'),
  'D050839': NtO(MH='Interferon Regulatory Factor-7', MS='An interferon regulatory factor that is induced by INTERFERONS as well as LMP-1 protein from EPSTEIN-BARR VIRUS. IRF-7 undergoes PHOSPHORYLATION prior to nuclear translocation and it activates GENETIC TRANSCRIPTION of multiple interferon GENES.'),
  'D050841': NtO(MH='Glycine Dehydrogenase (Decarboxylating)', MS='A PYRIDOXAL PHOSPHATE dependent enzyme that catalyzes the decarboxylation of GLYCINE with the transfer of an aminomethyl group to the LIPOIC ACID moiety of the GLYCINE DECARBOXYLASE COMPLEX H-PROTEIN. Defects in P-protein are the cause of non-ketotic hyperglycinemia. It is one of four subunits of the glycine decarboxylase complex.'),
  'D050842': NtO(MH='1-Pyrroline-5-Carboxylate Dehydrogenase', MS='An enzyme that catalyzes the oxidation of 1-pyrroline-5-carboxylate to L-GLUTAMATE in the presence of NAD. Defects in the enzyme are the cause of hyperprolinemia II.'),
  'D050856': NtO(MH='Interferon Regulatory Factor-1', MS='An interferon regulatory factor that binds upstream TRANSCRIPTIONAL REGULATORY ELEMENTS in the GENES for INTERFERON-ALPHA and INTERFERON-BETA. It functions as a transcriptional activator for the INTERFERON TYPE I genes.'),
  'D050857': NtO(MH='Synaptotagmins', MS='A family of vesicular transport proteins characterized by an N-terminal transmembrane region and two C-terminal calcium-binding domains.'),
  'D050858': NtO(MH='Betalains', MS='Compounds derived from TYROSINE via betalamic acid, including BETAXANTHINS and BETACYANINS. They are found in the Caryophyllales order of PLANTS and some BASIDIOMYCETES.'),
  'D050859': NtO(MH='Betacyanins', MS='Conjugates of betalamic acid with cyclo-dopa, both of which derive from TYROSINE. They appear similar to INDOLES but are biosynthesized by a different path and contain N+. Members are red or violet COLORING AGENTS found in the Caryophyllales order of PLANTS and some BASIDIOMYCETES.'),
  'D050860': NtO(MH='Betaxanthins', MS='Conjugates of betalamic acid with AMINO ACIDS. Some of them are yellow COLORING AGENTS in the Caryophyllales order of PLANTS. This should not be confused with xanthin which is a term used for CAROTENES nor with XANTHINES.'),
  'D050861': NtO(MH='Synaptotagmin II', MS='A vesicular transport protein that was originally characterized as an inositol polyphosphate binding protein. Synaptotagmin II helps regulate EXOCYTOSIS of SYNAPTIC VESICLES and appears to serve as a calcium sensor to trigger NEUROTRANSMITTER release. It also acts as a nerve cell receptor for certain BOTULINUM TOXINS.'),
  'D050862': NtO(MH='Oxidoreductases Acting on Sulfur Group Donors', MS='Oxidoreductases with specificity for oxidation or reduction of SULFUR COMPOUNDS.'),
  'D050863': NtO(MH='Synaptotagmin I', MS='A vesicular transport protein expressed predominately in NEURONS. Synaptotagmin helps regulate EXOCYTOSIS of SYNAPTIC VESICLES and appears to serve as a calcium sensor to trigger NEUROTRANSMITTER release. It also acts as a nerve cell receptor for certain BOTULINUM TOXINS.'),
  'D050864': NtO(MH='Sulfite Reductase (NADPH)', MS='A NADPH-dependent oxidase that reduces hydrogen sulfite to HYDROGEN SULFIDE. It is found in many microoganisms.'),
  'D050866': NtO(MH='Liquid Crystals', MS='Materials in intermediate state between solid and liquid.'),
  'D050876': NtO(MH='Sulfite Oxidase', MS='A MOLYBDENUM requiring enzyme that catalyzes the terminal reaction in the oxidative degradation of SULFUR AMINO ACIDS with the formation of a sulfate. A deficiency of sulfite oxidase results in sulfocysteinuria.'),
  'D050879': NtO(MH='Sarcosine Oxidase', MS='A FLAVOPROTEIN, this enzyme catalyzes the oxidation of SARCOSINE to GLYCINE; FORMALDEHYDE; and HYDROGEN PEROXIDE (H2O2).'),
  'D050880': NtO(MH='p300-CBP Transcription Factors', MS='A family of histone acetyltransferases that is structurally-related to CREB-BINDING PROTEIN and to E1A-ASSOCIATED P300 PROTEIN. They function as transcriptional coactivators by bridging between DNA-binding TRANSCRIPTION FACTORS and the basal transcription machinery. They also modify transcription factors and CHROMATIN through ACETYLATION.'),
  'D050881': NtO(MH='E1A-Associated p300 Protein', MS='A member of the p300-CBP transcription factors that was originally identified as a binding partner for ADENOVIRUS E1A PROTEINS.'),
  'D050882': NtO(MH='CREB-Binding Protein', MS='A member of the p300-CBP transcription factor family that was initially identified as a binding partner for CAMP RESPONSE ELEMENT-BINDING PROTEIN. Mutations in CREB-binding protein are associated with RUBINSTEIN-TAYBI SYNDROME.'),
  'D050883': NtO(MH='HSC70 Heat-Shock Proteins', MS='A constitutively expressed subfamily of the HSP70 heat-shock proteins. They preferentially bind and release hydrophobic peptides by an ATP-dependent process and are involved in post-translational PROTEIN TRANSLOCATION.'),
  'D050884': NtO(MH='HSP72 Heat-Shock Proteins', MS='Stress-inducible members of the heat-shock proteins 70 family. HSP72 heat shock proteins function with other MOLECULAR CHAPERONES to mediate PROTEIN FOLDING and to stabilize pre-existent proteins against aggregation.'),
  'D050885': NtO(MH='Hydrogensulfite Reductase', MS='An enzyme found primarily in SULFUR-REDUCING BACTERIA where it plays an important role in the anaerobic carbon oxidation pathway.'),
  'D050886': NtO(MH='HSP20 Heat-Shock Proteins', MS='A subfamily of small heat-shock proteins that are closely related to ALPHA B-CRYSTALLIN. Hsp20 heat-shock proteins can undergo PHOSPHORYLATION by CYCLIC GMP-DEPENDENT PROTEIN KINASES.'),
  'D050888': NtO(MH='Heat-Shock Proteins, Small', MS='A family of low molecular weight heat-shock proteins that can serve as MOLECULAR CHAPERONES.'),
  'D050889': NtO(MH='Sulfite Reductase (Ferredoxin)', MS='A FERREDOXIN-dependent oxidoreductase that is primarily found in PLANTS where it plays an important role in the assimilation of SULFUR atoms for the production of CYSTEINE and METHIONINE.'),
  'D050890': NtO(MH='HSP30 Heat-Shock Proteins', MS='A subfamily of small heat-shock proteins found in a wide variety of organisms.'),
  'D050891': NtO(MH='Oxidoreductases Acting on CH-NH2 Group Donors', MS='Enzymes catalyzing the dehydrogenation of or oxidation of compounds containing primary amines.'),
  'D050892': NtO(MH='Sarcosine Dehydrogenase', MS='A LIVER mitochondrial matrix flavoenzyme that catalyzes the oxidation of SARCOSINE to GLYCINE and FORMALDEHYDE. Mutation in the enzyme causes sarcosinemia, a rare autosomal metabolic defect characterized by elevated levels of SARCOSINE in BLOOD and URINE.'),
  'D050893': NtO(MH='Dimethylglycine Dehydrogenase', MS='A FLAVOPROTEIN enzyme that catalyzes the oxidative demethylation of dimethylglycine to SARCOSINE and FORMALDEHYDE.'),
  'D050895': NtO(MH='Nitrate Reductase (NADH)', MS='An NAD-dependent enzyme that catalyzes the oxidation of nitrite to nitrate. It is a FLAVOPROTEIN that contains IRON and MOLYBDENUM and is involved in the first step of nitrate assimilation in PLANTS; FUNGI; and BACTERIA. It was formerly classified as EC 1.6.6.1.'),
  'D050896': NtO(MH='Nitrate Reductase (NAD(P)H)', MS='An iron-sulfur and MOLYBDENUM containing FLAVOPROTEIN that catalyzes the oxidation of nitrite to nitrate. This enzyme can use either NAD or NADP as cofactors. It is a key enzyme that is involved in the first step of nitrate assimilation in PLANTS; FUNGI; and BACTERIA. This enzyme was formerly classified as EC 1.6.6.2.'),
  'D050897': NtO(MH='Nitrate Reductase (NADPH)', MS='An enzyme that catalyzes the oxidation of nitrite to nitrate in the presence of NADP+. It is a FLAVOPROTEIN that contains IRON and MOLYBDENUM. This enzyme was formerly classified as EC 1.6.6.3 and should not be confused with the enzyme NITRATE REDUCTASE (NAD(P)H).'),
  'D050898': NtO(MH='Nitrite Reductase (NAD(P)H)', MS='An enzyme found primarily in BACTERIA and FUNGI that catalyzes the oxidation of ammonium hydroxide to nitrite. It is an iron-sulfur HEME; FLAVOPROTEIN containing siroheme and can utilize both NAD and NADP as cofactors. This enzyme was formerly classified as EC 1.6.6.4.'),
  'D050899': NtO(MH='GMP Reductase', MS="An enzyme that catalyzes the reversible oxidation of inosine 5'-phosphate (IMP) to guanosine 5'-phosphate (GMP) in the presence of AMMONIA and NADP+. This enzyme was formerly classified as EC 1.6.6.8."),
  'D050900': NtO(MH='Ferredoxin-Nitrite Reductase', MS='An IRON-containing protein that uses siroheme and 4Fe-4S iron-sulfur centers as prosthetic groups. It catalyzes the six-electron oxidation of AMMONIA to nitrite.'),
  'D050901': NtO(MH='Nitrate Reductase', MS='An enzyme that catalyzes the oxidation of nitrite to nitrate. It is a cytochrome protein that contains IRON and MOLYBDENUM.'),
  'D050902': NtO(MH='Sulfite Dehydrogenase', MS='An enzyme that catalyzes oxidation of sulfite to sulfate along with the reduction of FERROCYTOCHROME C to FERRICYTOCHROME C.'),
  'D050903': NtO(MH='Nicotinamide N-Methyltransferase', MS='An enzyme found primarily in the LIVER that catalyzes the N-methylation of NICOTINAMIDE and other structurally related compounds.'),
  'D050904': NtO(MH='Homocysteine S-Methyltransferase', MS='An enzyme that catalyzes the demethylation of L-homocysteine to L-METHIONINE.'),
  'D050916': NtO(MH='TRPV Cation Channels', MS='A subgroup of TRP cation channels named after the vanilloid receptor. They are very sensitive to TEMPERATURE; hot spicy food, and CAPSAICIN. They contain a TRP domain (a five-turn amphipathic helix with an invariant TRYPTOPHAN) and ANKYRIN repeats. Selectivity for CALCIUM over SODIUM ranges from 3 to 100 fold.'),
  'D050918': NtO(MH='Phosphatidylethanolamine N-Methyltransferase', MS='An enzyme that catalyses three sequential METHYLATION reactions for conversion of phosphatidylethanolamine to PHOSPHATIDYLCHOLINE.'),
  'D050936': NtO(MH='Basic Reproduction Number', MS='The expected number of new cases of an infection caused by an infected individual, in a population consisting of susceptible contacts only.'),
  'D050937': NtO(MH='Guanidinoacetate N-Methyltransferase', MS='This enzyme catalyzes the last step of CREATINE biosynthesis by catalyzing the METHYLATION of guanidinoacetate to CREATINE.'),
  'D050938': NtO(MH='Glycine N-Methyltransferase', MS='An enzyme that catalyzes the METHYLATION of GLYCINE using S-ADENOSYLMETHIONINE to form SARCOSINE with the concomitant production of S-ADENOSYLHOMOCYSTEINE.'),
  'D050939': NtO(MH='Gene Fusion', MS='The GENETIC RECOMBINATION of the parts of two or more GENES resulting in a gene with different or additional regulatory regions, or a new chimeric gene product. ONCOGENE FUSION includes an ONCOGENE as at least one of the fusion partners and such gene fusions are often detected in neoplastic cells and are transcribed into ONCOGENE FUSION PROTEINS. ARTIFICIAL GENE FUSION is carried out in vitro by RECOMBINANT DNA technology.'),
  'D050940': NtO(MH='Betaine-Homocysteine S-Methyltransferase', MS='A ZINC metalloenzyme that catalyzes the transfer of a methyl group from BETAINE to HOMOCYSTEINE to produce dimethylglycine and METHIONINE, respectively. This enzyme is a member of a family of ZINC-dependent METHYLTRANSFERASES that use THIOLS or selenols as methyl acceptors.'),
  'D050941': NtO(MH='Phosphatidyl-N-Methylethanolamine N-Methyltransferase', MS='An enzyme that catalyzes the METHYLATION of phosphatidyl-N-methylethanolamine to produce phosphatidyl-N-dimethylethanolamine. This enzyme can also methylate phosphatidyl-N-dimethylethanolamine to produce phosphatidyl-N-trimethylethanolamine (PHOSPHATIDYLCHOLINE).'),
  'D050956': NtO(MH='HSP40 Heat-Shock Proteins', MS='A family of heat-shock proteins that contain a 70 amino-acid consensus sequence known as the J domain. The J domain of HSP40 heat shock proteins interacts with HSP70 HEAT-SHOCK PROTEINS. HSP40 heat-shock proteins play a role in regulating the ADENOSINE TRIPHOSPHATASES activity of HSP70 heat-shock proteins.'),
  'D050957': NtO(MH='HSP110 Heat-Shock Proteins', MS='A group of eukaryotic high-molecular mass heat-shock proteins that represent a subfamily of HSP70 HEAT-SHOCK PROTEINS. Hsp110 proteins prevent protein aggregation and can maintain denatured proteins in folding-competent states.'),
  'D050958': NtO(MH='HSP47 Heat-Shock Proteins', MS='Basic glycoprotein members of the SERPIN SUPERFAMILY that function as COLLAGEN-specific MOLECULAR CHAPERONES in the ENDOPLASMIC RETICULUM.'),
  'D050959': NtO(MH='Glycine Decarboxylase Complex', MS='A enzyme complex that catalyzes the oxidative DECARBOXYLATION and DEAMINATION of GLYCINE into CARBON DIOXIDE; AMMONIA; NADH; and N5N10-methylenetetrahydrofolate. It is composed of four different component protein components referred to as H, P, L, and T.'),
  'D050960': NtO(MH='Aminomethyltransferase', MS='A one-carbon group transferase that transfers lipoamide-linked methylamine groups to tetrahydrofolate (TETRAHYDROFOLATES) to form methylenetetrahydrofolate and AMMONIA. It is one of four components of the glycine decarboxylase complex.'),
  'D050961': NtO(MH='Glycine Decarboxylase Complex H-Protein', MS='A LIPOIC ACID-containing protein that plays the pivotal role in the transfer of methylamine groups and reducing equivalents between the three enzymatic components of the glycine decarboxylase complex.'),
  'D050976': NtO(MH='Basic-Leucine Zipper Transcription Factors', MS='A large superfamily of transcription factors that contain a region rich in BASIC AMINO ACID residues followed by a LEUCINE ZIPPER domain.'),
  'D050977': NtO(MH='Maf Transcription Factors', MS='Maf transcription factors are a family of basic-leucine zipper transcription factors that are closely related to V-MAF ONCOGENE PROTEIN. The C-MAF PROTO-ONCOGENE PROTEIN was the first mammalian Maf transcription factor identified, and now the family is known to include a variety of other Maf proteins such as MAFB TRANSCRIPTION FACTOR; MAFF TRANSCRIPTION FACTOR; MAFG TRANSCRIPTION FACTOR; and MAFK TRANSCRIPTION FACTOR.'),
  'D050978': NtO(MH='Maf Transcription Factors, Small', MS='A family of Maf Transcription Factors that lack activation domains. Small Maf proteins function as transcriptional repressors or form heterodimeric complexes to serve as transcriptional coactivators. Small Maf proteins include MafF, MafG, and MafK.'),
  'D050980': NtO(MH='GATA Transcription Factors', MS='A family of transcription factors that contain two ZINC FINGER MOTIFS and bind to the DNA sequence (A/T)GATA(A/G).'),
  'D050981': NtO(MH='Erythroid-Specific DNA-Binding Factors', MS='A group of transcription factors that were originally described as being specific to ERYTHROID CELLS.'),
  'D050982': NtO(MH='GATA1 Transcription Factor', MS='A GATA transcription factor that is specifically expressed in hematopoietic lineages and plays an important role in the CELL DIFFERENTIATION of ERYTHROID CELLS and MEGAKARYOCYTES.'),
  'D050983': NtO(MH='NF-E2 Transcription Factor', MS='A basic-leucine zipper transcription factor that regulates GLOBIN gene expression and is related to TRANSCRIPTION FACTOR AP-1. NF-E2 consists of a small MAF protein subunit and a tissue-restricted 45 kDa subunit.'),
  'D050984': NtO(MH='NF-E2 Transcription Factor, p45 Subunit', MS='A tissue-specific subunit of NF-E2 transcription factor that interacts with small MAF PROTEINS to regulate gene expression. P45 NF-E2 protein is expressed primarily in MEGAKARYOCYTES; ERYTHROID CELLS; and MAST CELLS.'),
  'D050985': NtO(MH='MafK Transcription Factor', MS='A small Maf protein involved in differentiation of ERYTHROID CELLS. MafK was originally described as the small subunit of the NF-E2 Transcription Factor, but other small MAF PROTEINS also serve as NF-E2 subunits.'),
  'D050986': NtO(MH='YY1 Transcription Factor', MS='A ubiquitously expressed zinc finger-containing protein that acts both as a repressor and activator of transcription. It interacts with key regulatory proteins such as TATA-BINDING PROTEIN; TFIIB; and ADENOVIRUS E1A PROTEINS.'),
  'D050987': NtO(MH='Vesicle-Associated Membrane Protein 1', MS='A member of the vesicle-associated membrane protein family involved in the MEMBRANE FUSION of TRANSPORT VESICLES to their target membrane.'),
  'D050988': NtO(MH='Vesicle-Associated Membrane Protein 2', MS='A synaptic membrane protein involved in MEMBRANE FUSION of SYNAPTIC VESICLES with the presynaptic membranes. It is the prototype member of the R-SNARE PROTEINS.'),
  'D050989': NtO(MH='GATA2 Transcription Factor', MS='An essential GATA transcription factor that is expressed primarily in HEMATOPOIETIC STEM CELLS.'),
  'D050990': NtO(MH='GATA3 Transcription Factor', MS='A GATA transcription factor that is found predominately in LYMPHOID CELL precursors and has been implicated in the CELL DIFFERENTIATION of HELPER T-CELLS. Haploinsufficiency of GATA3 is associated with HYPOPARATHYROIDISM; SENSORINEURAL HEARING LOSS; and renal anomalies syndrome.'),
  'D050991': NtO(MH='GATA4 Transcription Factor', MS='A GATA transcription factor that is expressed in the MYOCARDIUM of developing heart and has been implicated in the differentiation of CARDIAC MYOCYTES. GATA4 is activated by PHOSPHORYLATION and regulates transcription of cardiac-specific genes.'),
  'D050992': NtO(MH='GATA5 Transcription Factor', MS='A GATA transcription factor that is expressed predominately in SMOOTH MUSCLE CELLS and is involved in the CELL DIFFERENTIATION of CARDIAC MYOCYTES. In the developing heart, GATA5 becomes restricted to the ENDOCARDIUM and regulates transcription of genes such as cardiac TROPONIN C.'),
  'D050993': NtO(MH='GATA6 Transcription Factor', MS='A GATA transcription factor that is expressed predominately in SMOOTH MUSCLE CELLS and regulates vascular smooth muscle CELL DIFFERENTIATION.'),
  'D050994': NtO(MH='Voltage-Dependent Anion Channels', MS='A family of voltage-gated eukaryotic porins that form aqueous channels. They play an essential role in mitochondrial CELL MEMBRANE PERMEABILITY, are often regulated by BCL-2 PROTO-ONCOGENE PROTEINS, and have been implicated in APOPTOSIS.'),
  'D050995': NtO(MH='Voltage-Dependent Anion Channel 1', MS='Voltage-dependent anion channel 1 is the major pore-forming protein of the mitochondrial outer membrane. It also functions as a ferricyanide reductase in the PLASMA MEMBRANE.'),
  'D050996': NtO(MH='Voltage-Dependent Anion Channel 2', MS='Voltage-dependent anion channel 2 is a low abundance mammalian isoform of VDAC that interacts with the inactive form of BAK PROTEIN.'),
  'D050997': NtO(MH='Nanomedicine', MS='The branch of medicine concerned with the application of NANOTECHNOLOGY to the prevention and treatment of disease. It involves the monitoring, repair, construction, and control of human biological systems at the molecular level, using engineered nanodevices and NANOSTRUCTURES. (From Freitas Jr., Nanomedicine, vol 1, 1999).'),
  'D050998': NtO(MH='bcl-2 Homologous Antagonist-Killer Protein', MS='A multi-domain mitochondrial membrane protein and member of the bcl-2 Protein family. Bak protein interacts with TUMOR SUPPRESSOR PROTEIN P53 and promotes APOPTOSIS.'),
  'D051017': NtO(MH='Apoptosis Regulatory Proteins', MS='A large group of proteins that control APOPTOSIS. This family of proteins includes many ONCOGENE PROTEINS as well as a wide variety of classes of INTRACELLULAR SIGNALING PEPTIDES AND PROTEINS such as CASPASES.'),
  'D051020': NtO(MH='bcl-X Protein', MS='A member of the bcl-2 protein family that plays a role in the regulation of APOPTOSIS and is a regulatory subunit for PROTEIN PHOSPHATASE 1. Two major isoforms of the protein exist due to ALTERNATIVE SPLICING of the BCL2L1 mRNA and are referred to as Bcl-XS and Bcl-XL.'),
  'D051023': NtO(MH='BH3 Interacting Domain Death Agonist Protein', MS='A member of the Bcl-2 protein family that reversibly binds MEMBRANES. It is a pro-apoptotic protein that is activated by caspase cleavage.'),
  'D051025': NtO(MH='bcl-Associated Death Protein', MS='A pro-apoptotic protein and member of the Bcl-2 protein family that is regulated by PHOSPHORYLATION. Unphosphorylated Bad protein inhibits the activity of BCL-XL PROTEIN.'),
  'D051026': NtO(MH='Phosphoribosylglycinamide Formyltransferase', MS='An enzyme that catalyzes the transfer of a formyl group from N10-formyltetrahydrofolate to N1-(5-phospho-D-ribosyl)glycinamide to yield N2-formyl-N1-(5-phospho-D-ribosyl)glycinamide and tetrahydrofolate. It plays a role in the de novo purine biosynthetic pathway.'),
  'D051027': NtO(MH='Phosphoribosylaminoimidazolecarboxamide Formyltransferase', MS='An enzyme that catalyzes the conversion of aminoimidazole-4-carboxamide ribonucleotide to 5-formyl-aminoimidazole-4-carboxamide ribonucleotide in the purine de novo synthesis pathway. It requires the cofactor N(10)-FORMYLTETRAHYDROFOLATE as the formyl donor.'),
  'D051028': NtO(MH='bcl-2-Associated X Protein', MS='A member of the Bcl-2 protein family and homologous partner of C-BCL-2 PROTO-ONCOGENE PROTEIN. It regulates the release of CYTOCHROME C and APOPTOSIS INDUCING FACTOR from the MITOCHONDRIA. Several isoforms of BCL2-associated X protein occur due to ALTERNATIVE SPLICING of the mRNA for this protein.'),
  'D051033': NtO(MH='Apoptosis Inducing Factor', MS='A flavoprotein that functions as a powerful antioxidant in the MITOCHONDRIA and promotes APOPTOSIS when released from the mitochondria. In mammalian cells AIF is released in response to pro-apoptotic protein members of the bcl-2 protein family. It translocates to the CELL NUCLEUS and binds DNA to stimulate CASPASE-independent CHROMATIN condensation.'),
  'D051034': NtO(MH='Inhibitor of Apoptosis Proteins', MS='A conserved class of proteins that control APOPTOSIS in both VERTEBRATES and INVERTEBRATES. IAP proteins interact with and inhibit CASPASES, and they function as ANTI-APOPTOTIC PROTEINS. The protein class is defined by an approximately 80-amino acid motif called the baculoviral inhibitor of apoptosis repeat; some members have ubiquitin-protein ligase activity.'),
  'D051036': NtO(MH='Large-Conductance Calcium-Activated Potassium Channels', MS='A major class of calcium activated potassium channels whose members are voltage-dependent. MaxiK channels are activated by either membrane depolarization or an increase in intracellular Ca(2+). They are key regulators of calcium and electrical signaling in a variety of tissues.'),
  'D051037': NtO(MH='Large-Conductance Calcium-Activated Potassium Channel alpha Subunits', MS='The pore-forming subunits of large-conductance calcium-activated potassium channels. They form tetramers in CELL MEMBRANES.'),
  'D051038': NtO(MH='Large-Conductance Calcium-Activated Potassium Channel beta Subunits', MS='The regulatory subunits of large-conductance calcium-activated potassium channels.'),
  'D051039': NtO(MH='Leontopithecus', MS='The genus of lion tamarins in the subfamily CALLITRICHINAE. The common name refers to the mane on the shoulders.'),
  'D051040': NtO(MH='Vesicle-Associated Membrane Protein 3', MS='A member of the vesicle associated membrane protein family. It has a broad tissue distribution and is involved in MEMBRANE FUSION events of the endocytic pathways.'),
  'D051042': NtO(MH='Glutamate Formimidoyltransferase', MS='A PYRIDOXAL PHOSPHATE-containing enzyme that catalyzes the transfer of a formyl group from L-GLUTAMATE to N-formimidoyl-L-glutamate and TETRAHYDROFOLATE. This enzyme may also catalyze formyl transfer from 5-formyltetrahydrofolate to L-GLUTAMATE. This enzyme was formerly categorized as EC 2.1.2.6.'),
  'D051046': NtO(MH='Amino-Acid N-Acetyltransferase', MS='A mitochondrial matrix enzyme that catalyzes the synthesis of L-GLUTAMATE to N-acetyl-L-glutamate in the presence of ACETYL-COA.'),
  'D051047': NtO(MH='Dihydrolipoyllysine-Residue Acetyltransferase', MS='An enzyme that catalyzes the acetyltransferase reaction using ACETYL CoA as an acetyl donor and dihydrolipoamide as acceptor to produce COENZYME A (CoA) and S-acetyldihydrolipoamide. It forms the (E2) subunit of the PYRUVATE DEHYDROGENASE COMPLEX.'),
  'D051048': NtO(MH='Diacylglycerol O-Acyltransferase', MS='An enzyme that catalyses the last step of the TRIACYLGLYCEROL synthesis reaction in which diacylglycerol is covalently joined to LONG-CHAIN ACYL COA to form triglyceride. It was formerly categorized as EC 2.3.1.124.'),
  'D051049': NtO(MH='Sphingosine N-Acyltransferase', MS='An enzyme that catalyzes the acyltransferase of SPHINGOSINE to N-acylsphingosine using acyl-COENZYME A as donor and COENZYME A as acceptor. The enzyme is mainly localized in the MITOCHONDRIA.'),
  'D051050': NtO(MH='Serine O-Acetyltransferase', MS='An enzyme that catalyzes the conversion of L-SERINE to COENZYME A and O-acetyl-L-serine, using ACETYL-COA as a donor.'),
  'D051051': NtO(MH='Acyl-Carrier Protein S-Acetyltransferase', MS='A enzyme that catalyzes the transfer of acetyl groups from ACETYL CoA to acyl-carrier protein to form COENZYME A and acetyl-acyl-carrier protein.'),
  'D051056': NtO(MH='Complement Inactivating Agents', MS='Compounds that negatively regulate the cascade process of COMPLEMENT ACTIVATION. Uncontrolled complement activation and resulting cell lysis is potentially dangerous for the host.'),
  'D051057': NtO(MH='Proto-Oncogene Proteins c-akt', MS='Protein-serine-threonine kinases that contain PLECKSTRIN HOMOLOGY DOMAINS and are activated by PHOSPHORYLATION in response to GROWTH FACTORS or INSULIN. They play a major role in cell metabolism, growth, and survival as a core component of SIGNAL TRANSDUCTION. Three isoforms have been described in mammalian cells.'),
  'D051058': NtO(MH='Oncogene Protein v-akt', MS='A viral oncoprotein originally isolated from a murine T CELL LYMPHOMA infected with the acutely transforming retrovirus AKT8. v-akt protein is the viral homologue of PROTO-ONCOGENE PROTEINS C-AKT.'),
  'D051059': NtO(MH='PTEN Phosphohydrolase', MS='A lipid phosphatase that contains a C2 DOMAIN and acts on phosphatidylinositol-3,4,5-trisphosphate to regulate various SIGNAL TRANSDUCTION PATHWAYS. It modulates CELL GROWTH PROCESSES; CELL MIGRATION; and APOPTOSIS. Mutations in PTEN are associated with COWDEN DISEASE and PROTEUS SYNDROME as well as NEOPLASTIC CELL TRANSFORMATION.'),
  'D051076': NtO(MH='Proto-Oncogene Proteins c-fyn', MS='Src-family kinases that associate with T-CELL ANTIGEN RECEPTOR and phosphorylate a wide variety of intracellular signaling molecules.'),
  'D051077': NtO(MH='Atelidae', MS='A family of New World Monkeys in the infraorder PLATYRRHINI, comprising two subfamilies: ALOUATTINAE and ATELINAE.'),
  'D051078': NtO(MH='Platyrrhini', MS='An infraorder of New World monkeys, comprised of the families AOTIDAE; ATELIDAE; CEBIDAE; and PITHECIIDAE. They are found exclusively in the Americas.'),
  'D051079': NtO(MH='Catarrhini', MS='An infraorder of PRIMATES comprised of the families CERCOPITHECIDAE (old world monkeys); HYLOBATIDAE (siamangs and GIBBONS); and HOMINIDAE (great apes and HUMANS). With the exception of humans, they all live exclusively in Africa and Asia.'),
  'D051080': NtO(MH='Acyl-Carrier Protein S-Malonyltransferase', MS='This enzyme catalyzes the transacylation of malonate from MALONYL CoA to activated holo-ACP, to generate malonyl-(acyl-carrier protein), which is an elongation substrate in FATTY ACIDS biosynthesis. It is an essential enzyme in the biosynthesis of FATTY ACIDS in all BACTERIA.'),
  'D051081': NtO(MH='Glucosamine 6-Phosphate N-Acetyltransferase', MS='An enzyme that catalyses the reaction of D-glucosamine 6-phosphate with ACETYL-COA to form N-acetylglucosamine 6-phosphate.'),
  'D051082': NtO(MH='Homoserine O-Succinyltransferase', MS='The first enzyme in the METHIONINE biosynthetic pathway, this enzyme catalyzes the succinylation reaction of L-homoserine to O-succinyl-L-homoserine and COENZYME A using succinyl-CoA.'),
  'D051096': NtO(MH='Proto-Oncogene Proteins c-ret', MS='Receptor protein-tyrosine kinases involved in the signaling of GLIAL CELL-LINE DERIVED NEUROTROPHIC FACTOR ligands. They contain an extracellular cadherin domain and form a receptor complexes with GDNF RECEPTORS. Mutations in ret protein are responsible for HIRSCHSPRUNG DISEASE and MULTIPLE ENDOCRINE NEOPLASIA TYPE 2.'),
  'D051097': NtO(MH='Glial Cell Line-Derived Neurotrophic Factor Receptors', MS='A family of GLYCOSYLPHOSPHATIDYLINOSITOL-anchored cell surface receptors that are specific for GLIAL CELL LINE-DERIVED NEUROTROPHIC FACTORS. They form a multi-component receptor complex with PROTO-ONCOGENE PROTEIN C-RET and regulate a variety of intracellular SIGNAL TRANSDUCTION PATHWAYS in conjunction with c-ret protein.'),
  'D051098': NtO(MH='Glial Cell Line-Derived Neurotrophic Factors', MS='A family of closely related nerve growth factors that promote NEURON survival. They bind to GDNF RECEPTORS and stimulate SIGNAL TRANSDUCTION through PROTO-ONCOGENE PROTEIN C-RET.'),
  'D051099': NtO(MH='Tarsii', MS='An infraorder (or suborder) of small PRIMATES comprised of a single family and genus.'),
  'D051100': NtO(MH='Glial Cell Line-Derived Neurotrophic Factor', MS='The founding member of the glial cell line-derived neurotrophic factor family. It was originally characterized as a NERVE GROWTH FACTOR promoting the survival of MIDBRAIN dopaminergic NEURONS, and it has been studied as a potential treatment for PARKINSON DISEASE.'),
  'D051101': NtO(MH='Neurturin', MS='A glial cell line-derived neurotrophic factor ligand that is specific for the GFRA2 RECEPTOR. Neurturin is essential for the development of specific postganglionic parasympathetic NEURONS.'),
  'D051102': NtO(MH='Serine C-Palmitoyltransferase', MS="A key enzyme in SPHINGOLIPIDS biosynthesis, this enzyme catalyzes the pyridoxal-5'-phosphate-dependent condensation of L-SERINE and PALMITOYL COENZYME A to 3-dehydro-D-sphinganine. The enzyme consists of two different subunits."),
  'D051103': NtO(MH='1-Acylglycerol-3-Phosphate O-Acyltransferase', MS='An enzyme that catalyzes the acyl group transfer of ACYL COA to 1-acyl-sn-glycerol 3-phosphate to generate 1,2-diacyl-sn-glycerol 3-phosphate. This enzyme has alpha, beta, gamma, delta and epsilon subunits.'),
  'D051116': NtO(MH='Receptors, Scavenger', MS='A large group of structurally diverse cell surface receptors that mediate endocytic uptake of modified LIPOPROTEINS. Scavenger receptors are expressed by MYELOID CELLS and some ENDOTHELIAL CELLS, and were originally characterized based on their ability to bind acetylated LOW-DENSITY LIPOPROTEINS. They can also bind a variety of other polyanionic ligand. Certain scavenger receptors can internalize micro-organisms as well as apoptotic cells.'),
  'D051117': NtO(MH='Scavenger Receptors, Class A', MS='A family of scavenger receptors that mediate the influx of LIPIDS into MACROPHAGES and are involved in FOAM CELL formation.'),
  'D051122': NtO(MH='Scavenger Receptors, Class B', MS='A family of scavenger receptors that are predominately localized to CAVEOLAE of the PLASMA MEMBRANE and bind HIGH DENSITY LIPOPROTEINS.'),
  'D051124': NtO(MH='Scavenger Receptors, Class C', MS='A family of scavenger receptors with broad polyanionic ligand binding properties that are expressed in embryonic insect MACROPHAGES.'),
  'D051126': NtO(MH='Scavenger Receptors, Class D', MS='A class of scavenger receptors that are specific for oxidized LOW-DENSITY LIPOPROTEINS and apoptotic cells. They are expressed almost exclusively in INTRACELLULAR MEMBRANES of MACROPHAGES.'),
  'D051127': NtO(MH='Scavenger Receptors, Class E', MS='A class of oxidized LDL receptors that contain C-type lectin-like extracellular domains.'),
  'D051128': NtO(MH='Scavenger Receptors, Class F', MS='A group of structurally related scavenger receptors expressed predominately by ENDOTHELIAL CELLS. They-contain repeats of EPIDERMAL GROWTH FACTOR-like cysteine-rich motifs in their extracellular domains.'),
  'D051131': NtO(MH='Receptors, Oxidized LDL', MS='A group of cell-surface receptors for oxidatively modified LOW-DENSITY LIPOPROTEINS. The group includes certain SCAVENGER RECEPTORS that contribute to the pathogenesis of ATHEROSCLEROSIS by uptake of oxidized LDL.'),
  'D051135': NtO(MH='Rad51 Recombinase', MS='A Rec A recombinase found in eukaryotes. Rad51 is involved in DNA REPAIR of double-strand breaks.'),
  'D051137': NtO(MH='Selenium-Binding Proteins', MS='A group of proteins that covalently attach to selenium or SELENIUM-containing compounds.'),
  'D051140': NtO(MH='Selenoproteins', MS='Selenoproteins are proteins that specifically incorporate SELENOCYSTEINE into their amino acid chain. Most selenoproteins are enzymes with the selenocysteine residues being responsible for their catalytic functions.'),
  'D051142': NtO(MH='Retinol O-Fatty-Acyltransferase', MS='An enzyme that catalyzes the acyl group transfer of acyl COENZYME A to RETINOL to generate COENZYME A and a retinyl ester.'),
  'D051146': NtO(MH='N-Acylsphingosine Galactosyltransferase', MS='An enzyme that catalyzes the conversion of UDP-galactose and N-acylsphingosine to D-galactosylceramide and UDP.'),
  'D051147': NtO(MH='Ganglioside Galactosyltransferase', MS='Catalyzes the final step in the galactocerebroside biosynthesis pathway.'),
  'D051149': NtO(MH='Selenoprotein P', MS='An extracellular selenoprotein that contains most of the SELENIUM in PLASMA. Selenoprotein P functions as an antioxidant and appears to transport selenium from the LIVER to peripheral tissues.'),
  'D051150': NtO(MH='Methionine Sulfoxide Reductases', MS='Reductases that catalyze the reaction of peptide-L-methionine -S-oxide + thioredoxin to produce peptide-L-methionine + thioredoxin disulfide + H(2)O.'),
  'D051151': NtO(MH='Selenoprotein W', MS='A single SELENOCYSTEINE containing protein that binds reduced GLUTATHIONE and can act as an antioxidant.'),
  'D051152': NtO(MH='Clusterin', MS='A highly conserved heterodimeric glycoprotein that is differentially expressed during many severe physiological disturbance states such as CANCER; APOPTOSIS; and various NEUROLOGICAL DISORDERS. Clusterin is ubiquitously expressed and appears to function as a secreted MOLECULAR CHAPERONE.'),
  'D051153': NtO(MH='Wnt Proteins', MS='Wnt proteins are a large family of secreted glycoproteins that play essential roles in EMBRYONIC AND FETAL DEVELOPMENT, and tissue maintenance. They bind to FRIZZLED RECEPTORS and act as PARACRINE PROTEIN FACTORS to initiate a variety of SIGNAL TRANSDUCTION PATHWAYS. The canonical Wnt signaling pathway stabilizes the transcriptional coactivator BETA CATENIN.'),
  'D051154': NtO(MH='Wnt2 Protein', MS='A proto-oncogene protein and member of the Wnt family of proteins. It is frequently up-regulated in human GASTRIC CANCER and is a tumor marker (BIOMARKERS, TUMOR) of gastric and COLORECTAL CANCER.'),
  'D051155': NtO(MH='Wnt1 Protein', MS='A proto-oncogene protein and member of the Wnt family of proteins. It is expressed in the caudal MIDBRAIN and is essential for proper development of the entire mid-/hindbrain region.'),
  'D051157': NtO(MH='Frizzled Receptors', MS='A family of seven-pass transmembrane cell-surface proteins that combines with LOW DENSITY LIPROTEIN RECEPTOR-RELATED PROTEIN-5 or LOW DENSITY LIPROTEIN RECEPTOR-RELATED PROTEIN-5 to form receptors for WNT PROTEINS. Frizzled receptors often couple with HETEROTRIMERIC G PROTEINS and regulate the WNT SIGNALING PATHWAY.'),
  'D051176': NtO(MH='beta Catenin', MS='A multi-functional catenin that participates in CELL ADHESION and nuclear signaling. Beta catenin binds CADHERINS and helps link their cytoplasmic tails to the ACTIN in the CYTOSKELETON via ALPHA CATENIN. It also serves as a transcriptional co-activator and downstream component of WNT PROTEIN-mediated SIGNAL TRANSDUCTION PATHWAYS.'),
  'D051177': NtO(MH='Catenins', MS='A family of cytoskeletal proteins that play essential roles in CELL ADHESION at ADHERENS JUNCTIONS by linking CADHERINS to the ACTIN FILAMENTS of the CYTOSKELETON.'),
  'D051178': NtO(MH='alpha Catenin', MS='A catenin that binds F-ACTIN and links the CYTOSKELETON with BETA CATENIN and GAMMA CATENIN.'),
  'D051179': NtO(MH='Plakins', MS='A family of related proteins that associate with cytoskeletal elements and junctional complexes at INTERCELLULAR JUNCTIONS. Plakins share a common plakin domain or a plakin repeat domain.'),
  'D051180': NtO(MH='Desmoplakins', MS='Desmoplakins are cytoskeletal linker proteins that anchor INTERMEDIATE FILAMENTS to the PLASMA MEMBRANE at DESMOSOMES.'),
  'D051181': NtO(MH='Desmosomal Cadherins', MS='A single-pass transmembrane glycoproteins that mediate CALCIUM-dependent CELL ADHESION and are core components of DESMOSOMES.'),
  'D051182': NtO(MH='Desmogleins', MS='A group of desmosomal cadherins with cytoplasmic tails that resemble those of classical CADHERINS.'),
  'D051183': NtO(MH='Desmoglein 1', MS='A desmosomal cadherin that is an autoantigen in the acquired skin disorder PEMPHIGUS FOLIACEUS.'),
  'D051184': NtO(MH='Desmoglein 3', MS='A desmosomal cadherin that is an autoantigen in the acquired skin disorder PEMPHIGUS VULGARIS.'),
  'D051185': NtO(MH='gamma Catenin', MS='A multi-functional catenin that is highly homologous to BETA CATENIN. Gamma catenin binds CADHERINS and helps link their cytoplasmic tails to ACTIN in the CYTOSKELETON via ALPHA CATENIN. It is also found in DESMOSOMES where it mediates the link between DESMOSOMAL CADHERINS and DESMOPLAKIN.'),
  'D051186': NtO(MH='Armadillo Domain Proteins', MS='A family of proteins that contain several 42-amino acid repeat domains and are homologous to the Drosophila armadillo protein. They bind to other proteins through their armadillo domains and play a variety of roles in the CELL including SIGNAL TRANSDUCTION, regulation of DESMOSOME assembly, and CELL ADHESION.'),
  'D051187': NtO(MH='Desmocollins', MS='A group of desmosomal cadherins with cytoplasmic tails that are divergent from those of classical CADHERINS. Their intracytoplasmic domains bind PLAKOGLOBIN; PLAKOPHILINS; and DESMOPLAKINS.'),
  'D051188': NtO(MH='Knowledge Bases', MS='Collections of facts, assumptions, beliefs, and heuristics that are used in combination with databases to achieve desired results, such as a diagnosis, an interpretation, or a solution to a problem (From McGraw Hill Dictionary of Scientific and Technical Terms, 6th ed).'),
  'D051189': NtO(MH='Murinae', MS='A subfamily in the family MURIDAE, comprising the Old World MICE and RATS.'),
  'D051190': NtO(MH='Plectin', MS='A cytoskeletal linker protein with a molecular weight of greater than 500 kDa. It binds INTERMEDIATE FILAMENTS; MICROTUBULES; and ACTIN CYTOSKELETON and plays a central role in the organization and stability of the CYTOSKELETON. Plectin is phosphorylated by CALMODULIN KINASE; PROTEIN KINASE A; and PROTEIN KINASE C.'),
  'D051191': NtO(MH='Plakophilins', MS='Members of the armadillo family of proteins that are found in DESMOSOMES and interact with various proteins including desmocadherins; DESMOPLAKIN; ACTIN FILAMENTS; and KERATINS.'),
  'D051192': NtO(MH='Receptors, Pattern Recognition', MS='A large family of cell surface receptors that bind conserved molecular structures (PAMPS) present in pathogens. They play important roles in host defense by mediating cellular responses to pathogens.'),
  'D051193': NtO(MH='Toll-Like Receptors', MS='A family of pattern recognition receptors characterized by an extracellular leucine-rich domain and a cytoplasmic domain that share homology with the INTERLEUKIN 1 RECEPTOR and the DROSOPHILA toll protein. Following pathogen recognition, toll-like receptors recruit and activate a variety of SIGNAL TRANSDUCING ADAPTOR PROTEINS.'),
  'D051194': NtO(MH='Toll-Like Receptor 1', MS='A pattern recognition receptor that forms heterodimers with TOLL-LIKE RECEPTOR 2.'),
  'D051195': NtO(MH='Toll-Like Receptor 2', MS='A pattern recognition receptor that forms heterodimers with other TOLL-LIKE RECEPTORS. It interacts with multiple ligands including PEPTIDOGLYCAN, bacterial LIPOPROTEINS, lipoarabinomannan, and a variety of PORINS.'),
  'D051196': NtO(MH='Toll-Like Receptor 3', MS='A pattern recognition receptor that binds DOUBLE-STRANDED RNA. It mediates cellular responses to certain viral pathogens.'),
  'D051197': NtO(MH='Toll-Like Receptor 4', MS='A pattern recognition receptor that interacts with LYMPHOCYTE ANTIGEN 96 and LIPOPOLYSACCHARIDES. It mediates cellular responses to GRAM-NEGATIVE BACTERIA.'),
  'D051198': NtO(MH='Toll-Like Receptor 5', MS='A pattern recognition receptor that binds FLAGELLIN. It mediates cellular responses to certain bacterial pathogens.'),
  'D051199': NtO(MH='Toll-Like Receptor 7', MS='A pattern recognition receptor that binds several forms of imidazo-quinoline including the antiviral compound Imiquimod.'),
  'D051200': NtO(MH='Toll-Like Receptor 8', MS='A pattern recognition receptor that recognizes GUANOSINE and URIDINE-rich single-stranded RNA.'),
  'D051201': NtO(MH='Toll-Like Receptor 10', MS='A pattern recognition receptor that is expressed in LUNG and in B-LYMPHOCYTES.'),
  'D051216': NtO(MH='Toll-Like Receptor 6', MS='A pattern recognition receptor that forms heterodimers with TLR2 RECEPTOR. The complex interacts with a variety of ligands including LIPOPROTEINS from MYCOPLASMA.'),
  'D051217': NtO(MH='Toll-Like Receptor 9', MS='A pattern recognition receptor that binds unmethylated CPG CLUSTERS. It mediates cellular responses to bacterial pathogens by distinguishing between self and bacterial DNA.'),
  'D051218': NtO(MH='Lymphocyte Antigen 96', MS='A secreted protein that associates with TOLL-LIKE RECEPTOR 4 and is essential for receptor recognition of LIPOPOLYSACCHARIDES.'),
  'D051219': NtO(MH='Pituitary Adenylate Cyclase-Activating Polypeptide', MS='A multi-function neuropeptide that acts throughout the body by elevating intracellular cyclic AMP level via its interaction with PACAP RECEPTORS. Although first isolated from hypothalamic extracts and named for its action on the pituitary, it is widely distributed in the central and peripheral nervous systems. PACAP is important in the control of endocrine and homeostatic processes, such as secretion of pituitary and gut hormones and food intake.'),
  'D051220': NtO(MH='Pisiform Bone', MS='A pea-shaped carpal bone that actually sits in the tendon of the flexor carpi ulnaris muscle.'),
  'D051221': NtO(MH='Triquetrum Bone', MS='A carpal bone which is located between the LUNATE BONE and HAMATE BONE.'),
  'D051222': NtO(MH='Trapezium Bone', MS='A carpal bone adjacent to the TRAPEZOID BONE.'),
  'D051223': NtO(MH='Trapezoid Bone', MS='A carpal bone which is located between the TRAPEZIUM BONE and the CAPITATE BONE.'),
  'D051224': NtO(MH='Capitate Bone', MS='A carpal bone with a rounded head located between the TRAPEZOID BONE and the HAMATE BONE.'),
  'D051225': NtO(MH='Hamate Bone', MS='A carpal bone located between the CAPITATE BONE and the TRIQUETRUM BONE. The hamate has a prominent process that projects anteriorly.'),
  'D051228': NtO(MH='Geranyltranstransferase', MS='An enzyme involved in the MEVALONATE pathway, it catalyses the synthesis of farnesyl diphosphate from isopentenyl diphosphate and dimethylallyl diphosphate.'),
  'D051229': NtO(MH='3-Phosphoshikimate 1-Carboxyvinyltransferase', MS='An enzyme of the shikimate pathway of AROMATIC AMINO ACID biosynthesis, it generates 5-enolpyruvylshikimate 3-phosphate and ORTHOPHOSPHATE from PHOSPHOENOLPYRUVATE and shikimate-3-phosphate. The shikimate pathway is present in BACTERIA and PLANTS but not in MAMMALS.'),
  'D051231': NtO(MH='Farnesyltranstransferase', MS='An enzyme that catalyzes the synthesis of geranylgeranyl diphosphate from trans, trans-farnesyl diphosphate and isopentenyl diphosphate.'),
  'D051232': NtO(MH='Geranylgeranyl-Diphosphate Geranylgeranyltransferase', MS='An enzyme that catalyzes the condensation of two molecules of geranylgeranyl diphosphate to give prephytoene diphosphate. The prephytoene diphosphate molecule is a precursor for CAROTENOIDS and other tetraterpenes.'),
  'D051236': NtO(MH='Receptors, Pituitary Adenylate Cyclase-Activating Polypeptide', MS='A family of G-protein-coupled receptors that share significant homology with GLUCAGON RECEPTORS. They bind PITUITARY ADENYLATE CYCLASE ACTIVATING POLYPEPTIDE with high affinity and trigger intracellular changes that influence the behavior of CELLS.'),
  'D051237': NtO(MH='Receptors, Pituitary Adenylate Cyclase-Activating Polypeptide, Type I', MS='A pituitary adenylate cyclase-activating peptide receptor subtype that is found in the HYPOTHALAMUS; BRAIN STEM; PITUITARY GLAND; ADRENAL GLAND; PANCREAS; and TESTES and has a high affinity only for PACAP.'),
  'D051238': NtO(MH='Receptors, Vasoactive Intestinal Polypeptide, Type I', MS='A pituitary adenylate cyclase-activating polypeptide receptor subtype that binds both PACAP and VASOACTIVE INTESTINAL PEPTIDE. It is found predominately in the BRAIN.'),
  'D051239': NtO(MH='Receptors, Vasoactive Intestinal Peptide, Type II', MS='A pituitary adenylate cyclase-activating peptide receptor subtype found in LYMPHOCYTES. It binds both PACAP and VASOACTIVE INTESTINAL PEPTIDE and regulates immune responses.'),
  'D051242': NtO(MH='Caveolin 1', MS='A tyrosine phosphoprotein that plays an essential role in CAVEOLAE formation. It binds CHOLESTEROL and is involved in LIPIDS transport, membrane traffic, and SIGNAL TRANSDUCTION.'),
  'D051243': NtO(MH='Caveolin 2', MS='Caveolin 2 is a binding partner of CAVEOLIN 1. It undergoes tyrosine phosphorylation by C-SRC PROTEIN PP60 and plays a regulatory role in CAVEOLAE formation.'),
  'D051244': NtO(MH='Caveolin 3', MS='A caveolin that is expressed exclusively in MUSCLE CELLS and is sufficient to form CAVEOLAE in SARCOLEMMA. Mutations in caveolin 3 are associated with multiple muscle diseases including DISTAL MYOPATHY and LIMB-GIRDLE MUSCULAR DYSTROPHY.'),
  'D051245': NtO(MH='Retinal Bipolar Cells', MS='INTERNEURONS of the vertebrate RETINA containing two processes. They receive inputs from the RETINAL PHOTORECEPTOR CELLS and send outputs to the RETINAL GANGLION CELLS. The bipolar cells also make lateral connections in the retina with the RETINAL HORIZONTAL CELLS and with the AMACRINE CELLS.'),
  'D051246': NtO(MH='Glucose Transport Proteins, Facilitative', MS='A family of monosaccharide transport proteins characterized by 12 membrane spanning helices. They facilitate passive diffusion of GLUCOSE across the CELL MEMBRANE.'),
  'D051247': NtO(MH='Sodium-Glucose Transport Proteins', MS='Monosaccharide transport proteins that function as active symporters. They utilize SODIUM or HYDROGEN IONS to transport GLUCOSE across CELL MEMBRANES.'),
  'D051248': NtO(MH='Retinal Horizontal Cells', MS='NEURONS in the inner nuclear layer of the RETINA that synapse with both the RETINAL PHOTORECEPTOR CELLS and the RETINAL BIPOLAR CELLS, as well as other horizontal cells. The horizontal cells modulate the sensory signal.'),
  'D051256': NtO(MH='Oncogene Protein v-maf', MS='An oncogene protein that was originally isolated from a spontaneous musculo-aponeurotic FIBROSARCOMA in CHICKEN and shown to be the transforming gene of the avian retrovirus AS42. It is a basic leucine zipper TRANSCRIPTION FACTOR and the founding member of the MAF TRANSCRIPTION FACTORS.'),
  'D051257': NtO(MH='Proto-Oncogene Proteins c-maf', MS='Maf proto-oncogene protein is the major cellular homolog of the V-MAF ONCOGENE PROTEIN. It was the first of the mammalian MAF TRANSCRIPTION FACTORS identified, and it is induced in activated T-LYMPHOCYTES and regulates GENETIC TRANSCRIPTION of INTERLEUKIN-4. c-maf is frequently translocated to an immunoglobulin locus in MULTIPLE MYELOMA.'),
  'D051258': NtO(MH='MafB Transcription Factor', MS='A large maf protein that regulates HINDBRAIN development, contributes to CELL DIFFERENTIATION of MONOCYTES, and interacts with ETS-1 TRANSCRIPTION FACTOR.'),
  'D051259': NtO(MH='MafG Transcription Factor', MS='MafG is a ubiquitously expressed small maf protein that is involved in CELL DIFFERENTIATION of ERYTHROCYTES. It dimerizes with P45 NF-E2 PROTEIN and activates expression of ALPHA-GLOBIN and BETA-GLOBIN.'),
  'D051260': NtO(MH='MafF Transcription Factor', MS='A small maf protein that forms dimers with NRF1 protein; NRF2 PROTEIN; and P45 NF-E2 PROTEIN. MafF complexes bind Maf recognition elements to regulate tissue-specific GENETIC TRANSCRIPTION.'),
  'D051261': NtO(MH='Granulomatosis, Orofacial', MS='A condition characterized by persistent or recurrent labial enlargement, ORAL ULCER, and other orofacial manifestations in the absence of identifiable CROHN DISEASE; or SARCOIDOSIS. Among experts there is disagreement on whether orofacial granulomatosis is a distinct clinical disorder or an initial presentation of Crohn disease.'),
  'D051263': NtO(MH='D-Alanine Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the reversible transfer of an amino group between D-Alanine and alpha-ketoglutarate to form PYRUVATE and D-GLUTAMATE, respectively. It plays a role in the synthesis of the bacterial CELL WALL. This enzyme was formerly classified as EC 2.6.1.10.'),
  'D051264': NtO(MH='Nuclear Respiratory Factors', MS='A family of transcription factors that control expression of a variety of nuclear GENES encoding proteins that function in the RESPIRATORY CHAIN of the MITOCHONDRIA.'),
  'D051265': NtO(MH='NF-E2-Related Factor 1', MS='A basic-leucine zipper transcription factor that is involved in regulating inflammatory responses, MORPHOGENESIS, and HEME biosynthesis.'),
  'D051266': NtO(MH='Nuclear Respiratory Factor 1', MS='A transcription factor that controls the expression of variety of proteins including CYTOCHROME C and 5-AMINOLEVULINATE SYNTHETASE. It plays an important role in maintenance of the RESPIRATORY CHAIN of MITOCHONDRIA.'),
  'D051267': NtO(MH='NF-E2-Related Factor 2', MS='A basic-leucine zipper transcription factor that was originally described as a transcriptional regulator controlling expression of the BETA-GLOBIN gene. It may regulate the expression of a wide variety of genes that play a role in protecting cells from oxidative damage.'),
  'D051268': NtO(MH='GA-Binding Protein Transcription Factor', MS='A heterotetrameric transcription factor composed of two distinct proteins. Its name refers to the fact it binds to DNA sequences rich in GUANINE and ADENINE. GA-binding protein integrates a variety of SIGNAL TRANSDUCTION PATHWAYS and regulates expression of GENES involved in CELL CYCLE control, PROTEIN BIOSYNTHESIS, and cellular METABOLISM.'),
  'D051269': NtO(MH='Sodium-Glucose Transporter 1', MS='The founding member of the sodium glucose transport proteins. It is predominately expressed in the INTESTINAL MUCOSA of the SMALL INTESTINE.'),
  'D051270': NtO(MH='Headache Disorders, Primary', MS='Conditions in which the primary symptom is HEADACHE and the headache cannot be attributed to any known causes.'),
  'D051271': NtO(MH='Headache Disorders, Secondary', MS='Conditions with HEADACHE symptom that can be attributed to a variety of causes including BRAIN VASCULAR DISORDERS; WOUNDS AND INJURIES; INFECTION; drug use or its withdrawal.'),
  'D051272': NtO(MH='Glucose Transporter Type 1', MS='A ubiquitously expressed glucose transporter that is important for constitutive, basal GLUCOSE transport. It is predominately expressed in ENDOTHELIAL CELLS and ERYTHROCYTES at the BLOOD-BRAIN BARRIER and is responsible for GLUCOSE entry into the BRAIN.'),
  'D051273': NtO(MH='Glucose Transporter Type 2', MS='A glucose transport facilitator that is expressed primarily in PANCREATIC BETA CELLS; LIVER; and KIDNEYS. It may function as a GLUCOSE sensor to regulate INSULIN release and glucose HOMEOSTASIS.'),
  'D051274': NtO(MH='Glucose Transporter Type 3', MS='A major glucose transporter found in NEURONS.'),
  'D051275': NtO(MH='Glucose Transporter Type 4', MS='A glucose transport protein found in mature MUSCLE CELLS and ADIPOCYTES. It promotes transport of glucose from the BLOOD into target TISSUES. The inactive form of the protein is localized in CYTOPLASMIC VESICLES. In response to INSULIN, it is translocated to the PLASMA MEMBRANE where it facilitates glucose uptake.'),
  'D051276': NtO(MH='Leucine Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the reversible transamination of branched-chain AMINO ACIDS to 2-oxoglutarate.'),
  'D051277': NtO(MH='Succinyldiaminopimelate Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the transfer amino group from N-succinyl-L-2,6-diaminoheptanedioate to 2-oxoglutarate in order to generate N-succinyl-2-L-amino-6-oxoheptanedioate and L-GLUTAMATE, respectively.'),
  'D051278': NtO(MH='beta-Alanine-Pyruvate Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the transfer of an amino group from L-ALANINE to 3-oxopropanoate to generate PYRUVATE and BETA-ALANINE.'),
  'D051279': NtO(MH='Tryptophan Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the transfer amino group from L-TRYPTOPHAN to 2-oxoglutarate in order to generate indolepyruvate and L-GLUTAMATE.'),
  'D051296': NtO(MH='Glucose Transporter Type 5', MS='A hexose transporter that mediates FRUCTOSE transport in SKELETAL MUSCLE and ADIPOCYTES and is responsible for luminal uptake of dietary fructose in the SMALL INTESTINE.'),
  'D051297': NtO(MH='Sodium-Glucose Transporter 2', MS='A sodium-glucose transporter that is expressed in the luminal membrane of the PROXIMAL KIDNEY TUBULES.'),
  'D051298': NtO(MH='Post-Traumatic Headache', MS='Secondary headache attributed to TRAUMA of the HEAD and/or the NECK.'),
  'D051299': NtO(MH='Post-Dural Puncture Headache', MS='A secondary headache disorder attributed to low CEREBROSPINAL FLUID pressure caused by SPINAL PUNCTURE, usually after dural or lumbar puncture.'),
  'D051300': NtO(MH='Wiskott-Aldrich Syndrome Protein Family', MS='A family of microfilament proteins whose name derives from the fact that mutations in members of this protein family have been associated with WISKOTT-ALDRICH SYNDROME. They are involved in ACTIN polymerization and contain a polyproline-rich region that binds to PROFILIN, and a verprolin homology domain that binds G-ACTIN.'),
  'D051301': NtO(MH='Desmoglein 2', MS='A CALCIUM-dependent adhesion molecule of DESMOSOMES that also plays a role in embryonic STEM CELL proliferation.'),
  'D051302': NtO(MH='Paroxysmal Hemicrania', MS='A primary headache disorder that is similar to the CLUSTER HEADACHE with unilateral head pain, but differs by its multiple short severe attacks. It is usually seen in females, and may be responsive to non-steroidal-anti-inflammatory drugs (NSAIDS).'),
  'D051303': NtO(MH='Trigeminal Autonomic Cephalalgias', MS='Primary headache disorders that show symptoms caused by the activation of the AUTONOMIC NERVOUS SYSTEM of the TRIGEMINAL NERVE. These autonomic features include redness and tearing of the EYE, nasal congestion or discharge, facial SWEATING and other symptoms. Most subgroups show unilateral cranial PAIN.'),
  'D051304': NtO(MH='Profilins', MS='A family of low molecular weight proteins that bind ACTIN and control actin polymerization. They are found in eukaryotes and are ubiquitously expressed.'),
  'D051306': NtO(MH='L-Lysine 6-Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the transfer of amino group of L-LYSINE onto 2-oxoglutarate to generate 2-aminoadipate 6-semialdehyde and L-GLUTAMATE.'),
  'D051307': NtO(MH='2-Aminoadipate Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the transfer of amino group of L-2-aminoadipate onto 2-oxoglutarate to generate 2-oxoadipate and L-GLUTAMATE.'),
  'D051308': NtO(MH='Glycine Transaminase', MS='A PYRIDOXAL PHOSPHATE containing enzyme that catalyzes the transfer of the amino group of GLYCINE onto 2-oxoglutarate to generate GLYOXYLATE and L-GLUTAMATE.'),
  'D051316': NtO(MH='Wiskott-Aldrich Syndrome Protein', MS='WASP protein is mutated in WISKOTT-ALDRICH SYNDROME and is expressed primarily in hematopoietic cells. It is the founding member of the WASP protein family and interacts with CDC42 PROTEIN to help regulate ACTIN polymerization.'),
  'D051317': NtO(MH='Wiskott-Aldrich Syndrome Protein, Neuronal', MS='A member of the Wiskott-Aldrich syndrome protein family that is found at high levels in NERVE CELLS. It interacts with GRB2 ADAPTOR PROTEIN and with CDC42 PROTEIN.'),
  'D051336': NtO(MH='Mitochondrial Membranes', MS='The two lipoprotein layers in the MITOCHONDRION. The outer membrane encloses the entire mitochondrion and contains channels with TRANSPORT PROTEINS to move molecules and ions in and out of the organelle. The inner membrane folds into cristae and contains many ENZYMES important to cell METABOLISM and energy production (MITOCHONDRIAL ATP SYNTHASE).'),
  'D051337': NtO(MH='Chemical Terrorism', MS='The use of chemical agents in TERRORISM. This includes the malevolent use of nerve agents, blood agents, blister agents, and choking agents (NOXAE).'),
  'D051338': NtO(MH='Cofilin 1', MS='Cofilin 1 is a member of the cofilin family of proteins that is expressed in non-muscle CELLS. It has ACTIN depolymerization activity that is dependent on HYDROGEN-ION CONCENTRATION.'),
  'D051339': NtO(MH='Actin Depolymerizing Factors', MS='A family of low MOLECULAR WEIGHT actin-binding proteins found throughout eukaryotes. They remodel the actin CYTOSKELETON by severing ACTIN FILAMENTS and increasing the rate of monomer dissociation.'),
  'D051340': NtO(MH='Cofilin 2', MS='A member of the cofilin family of proteins that is expressed in MUSCLE CELLS. It has ACTIN depolymerization activity that is dependent on HYDROGEN-ION CONCENTRATION.'),
  'D051341': NtO(MH='Dependent Ambulation', MS='To move about or walk on foot with the use of aids.'),
  'D051344': NtO(MH='Actin Capping Proteins', MS='Actin capping proteins are cytoskeletal proteins that bind to the ends of ACTIN FILAMENTS to regulate actin polymerization.'),
  'D051345': NtO(MH='Destrin', MS='A member of the actin depolymerizing factors. Its depolymerizing activity is independent of HYDROGEN-ION CONCENTRATION.'),
  'D051346': NtO(MH='Mobility Limitation', MS='Difficulty in walking from place to place.'),
  'D051347': NtO(MH='CapZ Actin Capping Protein', MS='An actin capping protein that binds to the barbed-ends of ACTIN filaments. It is a heterodimer consisting of an alpha and a beta subunit. It regulates actin assembly by stabilizing actin oligomers for elongation. In SKELETAL MUSCLE, CapZ is localized to the Z-disk.'),
  'D051348': NtO(MH='Tropomodulin', MS='An actin capping protein that binds to the pointed-end of ACTIN. It functions in the presence of TROPOMYOSIN to inhibit microfilament elongation.'),
  'D051356': NtO(MH='Cortactin', MS='A microfilament protein that interacts with F-ACTIN and regulates cortical actin assembly and organization. It is also an SH3 DOMAIN containing phosphoprotein, and it mediates tyrosine PHOSPHORYLATION based SIGNAL TRANSDUCTION by PROTO-ONCOGENE PROTEIN PP60(C-SRC).'),
  'D051357': NtO(MH='Cyclin-Dependent Kinase 2', MS='A key regulator of CELL CYCLE progression. It partners with CYCLIN E to regulate entry into S PHASE and also interacts with CYCLIN A to phosphorylate RETINOBLASTOMA PROTEIN. Its activity is inhibited by CYCLIN-DEPENDENT KINASE INHIBITOR P27 and CYCLIN-DEPENDENT KINASE INHIBITOR P21.'),
  'D051358': NtO(MH='Cyclin-Dependent Kinase 4', MS='Cyclin-dependent kinase 4 is a key regulator of G1 PHASE of the CELL CYCLE. It partners with CYCLIN D to phosphorylate RETINOBLASTOMA PROTEIN. CDK4 activity is inhibited by CYCLIN-DEPENDENT KINASE INHIBITOR P16.'),
  'D051359': NtO(MH='Lymphohistiocytosis, Hemophagocytic', MS='A group of related disorders characterized by LYMPHOCYTOSIS; HISTIOCYTOSIS; and hemophagocytosis. The two major forms are familial and reactive.'),
  'D051360': NtO(MH='Cyclin-Dependent Kinase 5', MS='A serine-threonine kinase that plays important roles in CELL DIFFERENTIATION; CELL MIGRATION; and CELL DEATH of NERVE CELLS. It is closely related to other CYCLIN-DEPENDENT KINASES but does not seem to participate in CELL CYCLE regulation.'),
  'D051361': NtO(MH='Cyclin-Dependent Kinase 6', MS='Cyclin-dependent kinase 6 associates with CYCLIN D and phosphorylates RETINOBLASTOMA PROTEIN during G1 PHASE of the CELL CYCLE. It helps regulate the transition to S PHASE and its kinase activity is inhibited by CYCLIN-DEPENDENT KINASE INHIBITOR P18.'),
  'D051376': NtO(MH='Actin-Related Protein 2-3 Complex', MS='A complex of seven proteins including ARP2 PROTEIN and ARP3 PROTEIN that plays an essential role in maintenance and assembly of the CYTOSKELETON. Arp2-3 complex binds WASP PROTEIN and existing ACTIN FILAMENTS, and it nucleates the formation of new branch point filaments.'),
  'D051377': NtO(MH='Actin-Related Protein 2', MS='A PROFILIN binding domain protein that is part of the Arp2-3 complex. It is related in sequence and structure to ACTIN and binds ATP.'),
  'D051378': NtO(MH='Actin-Related Protein 3', MS='A component of the Arp2-3 complex that is related in sequence and structure to ACTIN and that binds ATP. It is expressed at higher levels than ARP2 PROTEIN and does not contain a PROFILIN binding domain.'),
  'D051379': NtO(MH='Mice', MS='The common name for the genus Mus.'),
  'D051380': NtO(MH='GRB2 Adaptor Protein', MS='A signal transducing adaptor protein that links extracellular signals to the MAP KINASE SIGNALING SYSTEM. Grb2 associates with activated EPIDERMAL GROWTH FACTOR RECEPTOR and PLATELET-DERIVED GROWTH FACTOR RECEPTORS via its SH2 DOMAIN. It also binds to and translocates the SON OF SEVENLESS PROTEINS through its SH3 DOMAINS to activate PROTO-ONCOGENE PROTEIN P21(RAS).'),
  'D051381': NtO(MH='Rats', MS='The common name for the genus Rattus.'),
  'D051382': NtO(MH='GRB7 Adaptor Protein', MS='A SH2 DOMAIN-containing protein that mediates SIGNAL TRANSDUCTION pathways from multiple CELL SURFACE RECEPTORS, including the EPHB1 RECEPTOR. It interacts with FOCAL ADHESION KINASE and is involved in CELL MIGRATION.'),
  'D051383': NtO(MH='GRB10 Adaptor Protein', MS='A binding partner for several RECEPTOR PROTEIN-TYROSINE KINASES, including INSULIN RECEPTOR and INSULIN-LIKE GROWTH FACTOR RECEPTOR. It contains a C-terminal SH2 DOMAIN and mediates various SIGNAL TRANSDUCTION pathways.'),
  'D051397': NtO(MH='Aquaglyceroporins', MS='A subgroup of aquaporins that transport WATER; GLYCEROL; and other small solutes across CELL MEMBRANES.'),
  'D051398': NtO(MH='Aquaporin 1', MS='Aquaporin 1 forms a water-specific channel that is constitutively expressed at the PLASMA MEMBRANE of ERYTHROCYTES and KIDNEY TUBULES, PROXIMAL. It provides these cells with a high permeability to WATER. In humans polymorphisms of this protein result in the Colton blood group antigen.'),
  'D051399': NtO(MH='Aquaporin 2', MS='Aquaporin 2 is a water-specific channel protein that is expressed in KIDNEY COLLECTING DUCTS. The translocation of aquaporin 2 to the apical PLASMA MEMBRANE is regulated by VASOPRESSIN, and MUTATIONS in AQP2 have been implicated in a variety of kidney disorders including DIABETES INSIPIDUS.'),
  'D051400': NtO(MH='Aquaporin 3', MS='Aquaporin 3 is an aquaglyceroporin that is expressed in the KIDNEY COLLECTING DUCTS and is constitutively localized at the basolateral MEMBRANE.'),
  'D051401': NtO(MH='Aquaporin 4', MS='Aquaporin 4 is the major water-selective channel in the CENTRAL NERVOUS SYSTEM of mammals.'),
  'D051402': NtO(MH='Aquaporin 5', MS='Aquaporin 5 is a water-specific channel protein that is expressed primarily in alveolar, tracheal, and upper bronchial EPITHELIUM. It plays an important role in maintaining water HOMEOSTASIS in the LUNGS and may also regulate release of SALIVA and TEARS in the SALIVARY GLANDS and the LACRIMAL GLAND.'),
  'D051403': NtO(MH='Aquaporin 6', MS='Aquaporin 6 is an aquaglyceroporin that is found primarily in KIDNEY COLLECTING DUCTS. AQP6 protein functions as an anion-selective channel.'),
  'D051416': NtO(MH='Focal Adhesion Protein-Tyrosine Kinases', MS='A family of non-receptor, PROLINE-rich protein-tyrosine kinases.'),
  'D051417': NtO(MH='Focal Adhesion Kinase 1', MS='A non-receptor protein tyrosine kinase with PROTEIN PHOSPHATASE 1 regulatory activity that is localized to FOCAL ADHESIONS and is a central component of integrin-mediated SIGNAL TRANSDUCTION PATHWAYS. Focal adhesion kinase 1 interacts with PAXILLIN and undergoes PHOSPHORYLATION in response to adhesion of cell surface integrins to the EXTRACELLULAR MATRIX. Phosphorylated p125FAK protein binds to a variety of SH2 DOMAIN and SH3 DOMAIN containing proteins and helps regulate CELL ADHESION and CELL MIGRATION.'),
  'D051418': NtO(MH='Focal Adhesion Kinase 2', MS='A non-receptor protein-tyrosine kinase that is expressed primarily in the BRAIN; OSTEOBLASTS; and LYMPHOID CELLS. In the CENTRAL NERVOUS SYSTEM focal adhesion kinase 2 modulates ION CHANNEL function and MITOGEN-ACTIVATED PROTEIN KINASES activity.'),
  'D051419': NtO(MH='Paxillin', MS='Paxillin is a signal transducing adaptor protein that localizes to FOCAL ADHESIONS via its four LIM domains. It undergoes PHOSPHORYLATION in response to integrin-mediated CELL ADHESION, and interacts with a variety of proteins including VINCULIN; FOCAL ADHESION KINASE; PROTO-ONCOGENE PROTEIN PP60(C-SRC); and PROTO-ONCOGENE PROTEIN C-CRK.'),
  'D051436': NtO(MH='Renal Insufficiency, Chronic', MS='Conditions in which the KIDNEYS perform below the normal level for more than three months. Chronic kidney insufficiency is classified by five stages according to the decline in GLOMERULAR FILTRATION RATE and the degree of kidney damage (as measured by the level of PROTEINURIA). The most severe form is the end-stage renal disease (CHRONIC KIDNEY FAILURE). (Kidney Foundation: Kidney Disease Outcome Quality Initiative, 2002)'),
  'D051437': NtO(MH='Renal Insufficiency', MS='Conditions in which the KIDNEYS perform below the normal level in the ability to remove wastes, concentrate URINE, and maintain ELECTROLYTE BALANCE; BLOOD PRESSURE; and CALCIUM metabolism. Renal insufficiency can be classified by the degree of kidney damage (as measured by the level of PROTEINURIA) and reduction in GLOMERULAR FILTRATION RATE.'),
  'D051438': NtO(MH='Syntaxin 16', MS='A ubiquitously expressed member of the syntaxin subfamily of SNARE proteins that localizes to the GOLGI APPARATUS.'),
  'D051445': NtO(MH='Fibrocartilage', MS='A type of CARTILAGE whose matrix contains large bundles of COLLAGEN TYPE I. Fibrocartilage is typically found in the INTERVERTEBRAL DISK; PUBIC SYMPHYSIS; TIBIAL MENISCI; and articular disks in synovial JOINTS. (From Ross et. al., Histology, 3rd ed., p132,136)'),
  'D051456': NtO(MH='Gene Flow', MS='The change in gene frequency in a population due to migration of gametes or individuals (ANIMAL MIGRATION) across population barriers. In contrast, in GENETIC DRIFT the cause of gene frequency changes are not a result of population or gamete movement.'),
  'D051457': NtO(MH='Hyaline Cartilage', MS='A type of CARTILAGE characterized by a homogenous amorphous matrix containing predominately TYPE II COLLAGEN and ground substance. Hyaline cartilage is found in ARTICULAR CARTILAGE; COSTAL CARTILAGE; LARYNGEAL CARTILAGES; and the NASAL SEPTUM.'),
  'D051472': NtO(MH='Elastic Cartilage', MS='A type of CARTILAGE whose matrix contains ELASTIC FIBERS and elastic lamellae, in addition to the normal components of HYALINE CARTILAGE matrix. Elastic cartilage is found in the EXTERNAL EAR; EUSTACHIAN TUBE; EPIGLOTTIS; and LARYNX.'),
  'D051473': NtO(MH='Maf Transcription Factors, Large', MS='A family of high molecular weight Maf transcription factors that contain distinct activation domains.'),
  'D051474': NtO(MH='Neuralgia, Postherpetic', MS='Pain in nerves, frequently involving facial SKIN, resulting from the activation the latent varicella-zoster virus (HERPESVIRUS 3, HUMAN). The two forms of the condition preceding the pain are HERPES ZOSTER OTICUS; and HERPES ZOSTER OPHTHALMICUS. Following the healing of the rashes and blisters, the pain sometimes persists.'),
  'D051478': NtO(MH='Triangular Fibrocartilage', MS='Fibrocartilage that makes up the triangular fibrocartilage complex which is found in the WRIST JOINT.'),
  'D051479': NtO(MH='Plant Infertility', MS='The failure of PLANTS to complete fertilization and obtain seed (SEEDS) as a result of defective POLLEN or ovules, or other aberrations. (Dict. of Plant Genet. and Mol. Biol., 1998)'),
  'D051496': NtO(MH='Receptor, Fibroblast Growth Factor, Type 1', MS='A fibroblast growth factor receptor with specificity for FIBROBLAST GROWTH FACTORS; HEPARAN SULFATE PROTEOGLYCAN; and NEURONAL CELL ADHESION MOLECULES. Several variants of the receptor exist due to multiple ALTERNATIVE SPLICING of its mRNA. Fibroblast growth factor receptor 1 contains three extracellular IMMUNOGLOBULIN C2-SET DOMAINS and is a tyrosine kinase that transmits signals through the MAP KINASE SIGNALING SYSTEM.'),
  'D051497': NtO(MH='Receptor, Fibroblast Growth Factor, Type 2', MS='A fibroblast growth factor receptor which contains three extracellular IMMUNOGLOBULIN I-SET DOMAINS and is expressed as two isoforms. One receptor isoform is expressed in the MESENCHYME and is activated by FIBROBLAST GROWTH FACTOR 2. A second isoform is expressed mainly by EPITHELIAL CELLS and is activated by FIBROBLAST GROWTH FACTOR 7 and FIBROBLAST GROWTH FACTOR 10. Mutation of the gene for fibroblast growth factor receptor 2 can result in craniosynostotic syndromes (e.g., APERT SYNDROME; and CROUZON SYNDROME).'),
  'D051498': NtO(MH='Receptor, Fibroblast Growth Factor, Type 3', MS='A fibroblast growth factor receptor that regulates CHONDROCYTE growth and CELL DIFFERENTIATION. Mutations in the gene for fibroblast growth factor receptor 3 have been associated with ACHONDROPLASIA; THANATOPHORIC DYSPLASIA and NEOPLASTIC CELL TRANSFORMATION.'),
  'D051499': NtO(MH='Receptor, Fibroblast Growth Factor, Type 4', MS='A fibroblast growth factor receptor that is mainly expressed in LUNG; KIDNEY; PANCREAS; and SPLEEN. It also plays an important role in SKELETAL MUSCLE development and can contribute to NEOPLASTIC CELL TRANSFORMATION.'),
  'D051500': NtO(MH='Receptor, Fibroblast Growth Factor, Type 5', MS='The most divergent of the known fibroblast growth factor receptors. It does not contain an intracellular TYROSINE KINASE domain and has been shown to interact with FIBROBLAST GROWTH FACTOR 2. Fibroblast growth factor receptor 5 is found primarily in skeletal tissue.'),
  'D051516': NtO(MH='Hemospermia', MS='Blood in the SEMEN, usually due to INFLAMMATION of the PROSTATE, the SEMINAL VESICLES, or both.'),
  'D051517': NtO(MH='Digital Rectal Examination', MS='A physical examination in which the qualified health care worker inserts a lubricated, gloved finger of one hand into the RECTUM and may use the other hand to press on the lower ABDOMEN or pelvic area to palpate for abnormalities in the lower rectum, and nearby organs or tissues. The method is commonly used to check the lower rectum, the PROSTATE gland in men, and the UTERUS and OVARIES in women.'),
  'D051518': NtO(MH='Fibroblast Growth Factor 3', MS='A fibroblast growth factor that is expressed primarily during development.'),
  'D051519': NtO(MH='Fibroblast Growth Factor 4', MS='A HEPARIN binding fibroblast growth factor that may play a role in LIMB BUDS development.'),
  'D051520': NtO(MH='Antiperspirants', MS='Agents that are put on the SKIN to reduce SWEATING or prevent excess sweating (HYPERHIDROSIS).'),
  'D051521': NtO(MH='Fibroblast Growth Factor 5', MS='A fibroblast growth factor that may play a role in regulation of HAIR FOLLICLE phenotype. Spontaneous mutation of the gene for this protein results in a strain of MICE with abnormally long hair, referred to as angora mice.'),
  'D051522': NtO(MH='Fibroblast Growth Factor 6', MS='A fibroblast growth factor that was initially identified based on its sequence similarity to FIBROBLAST GROWTH FACTOR 4. It is found in MYOBLASTS and plays an important role in MUSCLE DEVELOPMENT.'),
  'D051523': NtO(MH='Fibroblast Growth Factor 7', MS='A fibroblast growth factor that is a specific mitogen for EPITHELIAL CELLS. It binds a complex of HEPARAN SULFATE and FIBROBLAST GROWTH FACTOR RECEPTOR 2B.'),
  'D051524': NtO(MH='Fibroblast Growth Factor 8', MS='A fibroblast growth factor that preferentially activates FIBROBLAST GROWTH FACTOR RECEPTOR 4. It was initially identified as an androgen-induced growth factor and plays a role in regulating growth of human BREAST NEOPLASMS and PROSTATIC NEOPLASMS.'),
  'D051525': NtO(MH='Fibroblast Growth Factor 9', MS='A fibroblast growth factor that was originally identified as a mitogen for GLIAL CELLS. It is expressed primarily in NEURONS.'),
  'D051526': NtO(MH='Fibroblast Growth Factor 10', MS='A fibroblast growth factor that is a mitogen for KERATINOCYTES. It activates FIBROBLAST GROWTH FACTOR RECEPTOR 2B and is involved in LUNG and limb development.'),
  'D051527': NtO(MH='Odontogenic Tumor, Squamous', MS='A well-differentiated, benign, hamartomatous proliferation of odontogenic epithelium, probably arising from the rests of Malassez.'),
  'D051528': NtO(MH='Guanylate Kinases', MS='Catalyzes the ATP-dependent PHOSPHORYLATION of GMP to generate GDP and ADP.'),
  'D051536': NtO(MH='Hepatocyte Nuclear Factors', MS='Hepatocyte nuclear factors are a family of evolutionarily conserved transcription factors that are preferentially expressed in HEPATOCYTES. They play important roles in liver-specific transcription and are critical for CELL DIFFERENTIATION and METABOLISM.'),
  'D051537': NtO(MH='Hepatocyte Nuclear Factor 1', MS='A transcription factor and homeodomain protein that regulates the expression of a large set of hepatic proteins including SERUM ALBUMIN; beta-fibrinogen; and ALPHA 1-ANTITRYPSIN. It is composed of hetero- or homo-dimers of HEPATOCYTE NUCLEAR FACTOR 1-ALPHA and HEPATOCYTE NUCLEAR FACTOR 1-BETA.'),
  'D051538': NtO(MH='Hepatocyte Nuclear Factor 1-alpha', MS='Hepatocyte nuclear factor 1-alpha is a transcription factor found in the LIVER; PANCREAS; and KIDNEY that regulates HOMEOSTASIS of GLUCOSE.'),
  'D051539': NtO(MH='Hepatocyte Nuclear Factor 1-beta', MS='A hepatocyte nuclear factor that is closely related to HEPATOCYTE NUCLEAR FACTOR 1-ALPHA but is only weakly expressed in the LIVER. Mutations in hepatocyte nuclear factor 1-beta are associated with renal CYSTS and MATURITY-ONSET DIABETES MELLITUS type 5.'),
  'D051540': NtO(MH='Winged-Helix Transcription Factors', MS='A subfamily of HELIX-TURN-HELIX DNA-binding proteins that contain a variable length loop adjacent to the HTH motif. The loop connects two anti-parallel strands and forms a wing when bound to DNA.'),
  'D051541': NtO(MH='Hepatocyte Nuclear Factor 3-alpha', MS='A forkhead transcription factor that is an essential activator of GLUCAGON gene expression.'),
  'D051542': NtO(MH='Hepatocyte Nuclear Factor 3-beta', MS='A forkhead transcription factor that regulates expression of metabolic GENES and is involved in EMBRYONIC DEVELOPMENT. Mutations in HNF-3beta have been associated with CONGENITAL HYPERINSULINISM.'),
  'D051543': NtO(MH='Hepatocyte Nuclear Factor 3-gamma', MS='A winged-helix transcription factor that regulates GENE expression in metabolic tissues. It plays a role in HOMEOSTASIS of GLUCOSE and controls expression of GLUT2 PROTEIN.'),
  'D051544': NtO(MH='Cytochrome P-450 CYP3A', MS='A cytochrome P-450 suptype that has specificity for a broad variety of lipophilic compounds, including STEROIDS; FATTY ACIDS; and XENOBIOTICS. This enzyme has clinical significance due to its ability to metabolize a diverse array of clinically important drugs such as CYCLOSPORINE; VERAPAMIL; and MIDAZOLAM. This enzyme also catalyzes the N-demethylation of ERYTHROMYCIN.'),
  'D051545': NtO(MH='Cyclooxygenase 1', MS='A constitutively-expressed subtype of prostaglandin-endoperoxide synthase. It plays an important role in many cellular processes.'),
  'D051546': NtO(MH='Cyclooxygenase 2', MS='An inducibly-expressed subtype of prostaglandin-endoperoxide synthase. It plays an important role in many cellular processes and INFLAMMATION. It is the target of COX2 INHIBITORS.'),
  'D051547': NtO(MH='Heme Oxygenase-1', MS='A ubiquitous stress-responsive enzyme that catalyzes the oxidative cleavage of HEME to yield IRON; CARBON MONOXIDE; and BILIVERDIN.'),
  'D051548': NtO(MH='Histone Acetyltransferases', MS='Enzymes that catalyze acyl group transfer from ACETYL-CoA to HISTONES forming CoA and acetyl-histones.'),
  'D051549': NtO(MH='Glutathione S-Transferase pi', MS='A glutathione transferase that catalyzes the conjugation of electrophilic substrates to GLUTATHIONE. This enzyme has been shown to provide cellular protection against redox-mediated damage by FREE RADICALS.'),
  'D051550': NtO(MH='I-kappa B Kinase', MS='A protein serine-threonine kinase that catalyzes the PHOSPHORYLATION of I KAPPA B PROTEINS. This enzyme also activates the transcription factor NF-KAPPA B and is composed of alpha and beta catalytic subunits, which are protein kinases and gamma, a regulatory subunit.'),
  'D051552': NtO(MH='beta-Adrenergic Receptor Kinases', MS='G-protein-coupled receptor kinases that mediate agonist-dependent PHOSPHORYLATION and desensitization of BETA-ADRENERGIC RECEPTORS.'),
  'D051556': NtO(MH='Hyperbilirubinemia, Neonatal', MS='Accumulation of BILIRUBIN, a breakdown product of HEME PROTEINS, in the BLOOD during the first weeks of life. This may lead to NEONATAL JAUNDICE. The excess bilirubin may exist in the unconjugated (indirect) or the conjugated (direct) form. The condition may be self-limiting (PHYSIOLOGICAL NEONATAL JAUNDICE) or pathological with toxic levels of bilirubin.'),
  'D051557': NtO(MH='Hepatocyte Nuclear Factor 4', MS='A subfamily of nuclear receptors that regulate GENETIC TRANSCRIPTION of a diverse group of GENES involved in the synthesis of BLOOD COAGULATION FACTORS; and in GLUCOSE; CHOLESTEROL; and FATTY ACIDS metabolism.'),
  'D051558': NtO(MH='Onecut Transcription Factors', MS='A family of transcription factors that contain a single cut domain and a divergent homeodomain. They regulate gene networks by controlling the expression of other transcription factors and they play an important role in CELL DIFFERENTIATION and METABOLISM.'),
  'D051559': NtO(MH='Hepatocyte Nuclear Factor 6', MS='A onecut transcription factor that regulates expression of GENES involved in EMBRYONIC DEVELOPMENT of the PANCREAS and LIVER.'),
  'D051560': NtO(MH='Proto-Oncogene Proteins c-bcl-6', MS='A DNA-binding protein that contains an N-terminal BTB (POZ) DOMAIN and C-terminal CYS2-HIS2 ZINC FINGERS. It represses GENETIC TRANSCRIPTION of target genes by recruiting HISTONE DEACETYLASES. Aberrant Blc-6 expression is associated with certain types of human B-CELL LYMPHOMA.'),
  'D051561': NtO(MH='Complement C3 Convertase, Alternative Pathway', MS='A serine protease that is the complex of COMPLEMENT C3B and COMPLEMENT FACTOR BB. It cleaves multiple COMPLEMENT C3 into COMPLEMENT C3A (anaphylatoxin) and COMPLEMENT C3B in the ALTERNATIVE COMPLEMENT ACTIVATION PATHWAY.'),
  'D051562': NtO(MH='Proto-Oncogene Proteins c-bcr', MS='A serine-threonine kinase that contains a C2 DOMAIN and PLECKSTRIN HOMOLOGY DOMAIN. It also has activity as a GTPASE-ACTIVATING PROTEIN for RAC1 PROTEIN and CDC42 PROTEIN. It functions as a negative regulator of CELL PROLIFERATION and NEOPLASTIC CELL TRANSFORMATION and is commonly fused with cellular abl protein to form BCR-ABL FUSION PROTEINS in PHILADELPHIA CHROMOSOME positive LEUKEMIA patients.'),
  'D051566': NtO(MH='Complement C5 Convertase, Alternative Pathway', MS='A serine protease that cleaves multiple COMPLEMENT C5 into COMPLEMENT C5A (anaphylatoxin) and COMPLEMENT C5B in the ALTERNATIVE COMPLEMENT ACTIVATION PATHWAY. It is the complex of ALTERNATIVE PATHWAY C3 CONVERTASE (C3bBb) with an additional COMPLEMENT C3B, or C3bBb3b.'),
  'D051567': NtO(MH='Proto-Oncogene Proteins c-vav', MS='Proteins that are guanine nucleotide exchange factors for RHO GTPASES. They contain pleckstrin homology and SH2 domains, through which they function as signal transducing adaptor proteins.'),
  'D051571': NtO(MH='Protein Kinase C-alpha', MS='A cytoplasmic serine threonine kinase involved in regulating CELL DIFFERENTIATION and CELLULAR PROLIFERATION. Overexpression of this enzyme has been shown to promote PHOSPHORYLATION of BCL-2 PROTO-ONCOGENE PROTEINS and chemoresistance in human acute leukemia cells.'),
  'D051572': NtO(MH='Proto-Oncogene Proteins c-hck', MS='Members of the src-family tyrosine kinase family that are strongly expressed in MYELOID CELLS and B-LYMPHOCYTES.'),
  'D051573': NtO(MH='Proto-Oncogene Proteins c-pim-1', MS='Serine-threonine protein kinases that relay signals from CYTOKINE RECEPTORS and are involved in control of CELL GROWTH PROCESSES; CELL DIFFERENTIATION; and APOPTOSIS.'),
  'D051574': NtO(MH='Complement C3 Convertase, Classical Pathway', MS='A serine protease that cleaves multiple COMPLEMENT 3 into COMPLEMENT 3A (anaphylatoxin) and COMPLEMENT 3B in the CLASSICAL COMPLEMENT ACTIVATION PATHWAY. It is a complex of COMPLEMENT 4B and COMPLEMENT 2A (C4b2a).'),
  'D051575': NtO(MH='Proto-Oncogene Proteins c-yes', MS='Members of the src-family tyrosine kinases that are activated during the transition from G2 PHASE to M PHASE of the CELL CYCLE. It is highly homologous to PROTO-ONCOGENE PROTEIN PP60(C-SRC).'),
  'D051577': NtO(MH='Complement C5 Convertase, Classical Pathway', MS='A serine protease that cleaves multiple COMPLEMENT 5 into COMPLEMENT 5A (anaphylatoxin) and COMPLEMENT 5B in the CLASSICAL COMPLEMENT ACTIVATION PATHWAY. It is a complex of CLASSICAL PATHWAY C3 CONVERTASE (C4b2a) with an additional COMPLEMENT C3B, or C4b2a3b.'),
  'D051578': NtO(MH='Proto-Oncogene Proteins c-fes', MS='Proto-oncogene proteins fes are protein-tyrosine kinases with a central SH2 DOMAIN. It has been implicated in SIGNAL TRANSDUCTION PATHWAYS for CELL DIFFERENTIATION of a variety of cell types including MYELOID PROGENITOR CELLS. Fes proto-oncogene proteins also bind TUBULIN and promote MICROTUBULE assembly.'),
  'D051579': NtO(MH='Neurogranin', MS='A BRAIN-specific substrate for PROTEIN KINASE C that binds CALMODULIN and is involved in regulation of CALCIUM SIGNALING.'),
  'D051580': NtO(MH='Neuronal Calcium-Sensor Proteins', MS='A family of intracellular calcium-sensing proteins found predominately in NEURONS and PHOTORECEPTOR CELLS. They contain EF HAND MOTIFS and undergo conformational changes upon calcium-binding. Neuronal calcium-sensor proteins interact with other regulatory proteins to mediate physiological responses to a change in intracellular calcium concentration.'),
  'D051596': NtO(MH='Neurocalcin', MS='A neuronal calcium sensor protein that is expressed as several isoforms and can interact with ACTIN; TUBULIN; and CLATHRIN.'),
  'D051597': NtO(MH='Hippocalcin', MS='A neuronal calcium-sensor protein that was initially found in the NEURONS of the HIPPOCAMPUS. It interacts with NEURONAL APOPTOSIS-INHIBITORY PROTEIN.'),
  'D051598': NtO(MH='Whole Body Imaging', MS='The creation of a visual display of the inside of the entire body of a human or animal for the purposes of diagnostic evaluation. This is most commonly achieved by using MAGNETIC RESONANCE IMAGING; or POSITRON EMISSION TOMOGRAPHY.'),
  'D051599': NtO(MH='Glucose-1-Phosphate Adenylyltransferase', MS='An ATP-dependent enzyme that catalyzes the addition of ADP to alpha-D-glucose 1-phosphate to form ADP-glucose and diphosphate. The reaction is the rate-limiting reaction in prokaryotic GLYCOGEN and plant STARCH biosynthesis.'),
  'D051603': NtO(MH='Recoverin', MS='A neuronal calcium-sensor protein that is found in ROD PHOTORECEPTORS and CONE PHOTORECEPTORS. It interacts with G-PROTEIN-COUPLED RECEPTOR KINASE 1 in a Ca2+ dependent manner and plays an important role in PHOTOTRANSDUCTION.'),
  'D051604': NtO(MH='Guanylate Cyclase-Activating Proteins', MS='Neuronal calcium sensor proteins that regulate the activation of membrane-bound GUANYLATE CYCLASE. They are primarily expressed in the RETINA where they play an important role in PHOTOTRANSDUCTION.'),
  'D051605': NtO(MH='Kv Channel-Interacting Proteins', MS='A family of neuronal calcium-sensor proteins that interact with and regulate potassium channels, type A.'),
  'D051606': NtO(MH='G-Protein-Coupled Receptor Kinase 1', MS='A PROTEIN-SERINE-THREONINE KINASE that is found in PHOTORECEPTOR CELLS. It mediates light-dependent PHOSPHORYLATION of RHODOPSIN and plays an important role in PHOTOTRANSDUCTION.'),
  'D051607': NtO(MH='CDP-Diacylglycerol-Inositol 3-Phosphatidyltransferase', MS='An enzyme that catalyzes the formation of PHOSPHATIDYLINOSITOL and CMP from CDP-DIACYLGLYCEROL and MYOINOSITOL.'),
  'D051616': NtO(MH='Neuronal Apoptosis-Inhibitory Protein', MS='An inhibitor of apoptosis protein that was initially identified during analysis of CHROMOSOME DELETIONS associated with SPINAL MUSCULAR ATROPHY. Naip contains a nucleotide binding oligomerization domain and a carboxy-terminal LEUCINE rich repeat.'),
  'D051636': NtO(MH='X-Linked Inhibitor of Apoptosis Protein', MS='An inhibitor of apoptosis protein that is translated by a rare cap-independent mechanism. It blocks caspase-mediated cellular destruction by inhibiting CASPASE 3; CASPASE 7; and CASPASE 9.'),
  'D051637': NtO(MH='Shaker Superfamily of Potassium Channels', MS='Voltage-gated potassium channels whose primary subunits contain six transmembrane segments and form tetramers to create a pore with a voltage sensor. They are related to their founding member, shaker protein, Drosophila.'),
  'D051638': NtO(MH='Ether-A-Go-Go Potassium Channels', MS='A family of voltage-gated potassium channels that are characterized by long N-terminal and C-terminal intracellular tails. They are named from the Drosophila protein whose mutation causes abnormal leg shaking under ether anesthesia. Their activation kinetics are dependent on extracellular MAGNESIUM and PROTON concentration.'),
  'D051639': NtO(MH='Gardening', MS='Cultivation of PLANTS; (FRUIT; VEGETABLES; MEDICINAL HERBS) on small plots of ground or in containers.'),
  'D051640': NtO(MH='Small-Conductance Calcium-Activated Potassium Channels', MS='A major class of calcium-activated potassium channels that are found primarily in excitable CELLS. They play important roles in the transmission of ACTION POTENTIALS and generate a long-lasting hyperpolarization known as the slow afterhyperpolarization.'),
  'D051642': NtO(MH='Histiocytoma', MS='A neoplasm containing HISTIOCYTES. Important forms include BENIGN FIBROUS HISTIOCYTOMA; and MALIGNANT FIBROUS HISTIOCYTOMA.'),
  'D051656': NtO(MH='KCNQ Potassium Channels', MS='A family of delayed rectifier voltage-gated potassium channels that share homology with their founding member, KCNQ1 PROTEIN. KCNQ potassium channels have been implicated in a variety of diseases including LONG QT SYNDROME; DEAFNESS; and EPILEPSY.'),
  'D051657': NtO(MH='KCNQ1 Potassium Channel', MS='A voltage-gated potassium channel that is expressed primarily in the HEART.'),
  'D051658': NtO(MH='KCNQ2 Potassium Channel', MS='A very slow opening and closing voltage-gated potassium channel that is expressed in NEURONS and is commonly mutated in BENIGN FAMILIAL NEONATAL CONVULSIONS.'),
  'D051659': NtO(MH='KCNQ3 Potassium Channel', MS='A very slow opening and closing voltage-gated potassium channel that is expressed in NEURONS and is closely related to KCNQ2 POTASSIUM CHANNEL. It is commonly mutated in BENIGN FAMILIAL NEONATAL CONVULSIONS.'),
  'D051660': NtO(MH='Intermediate-Conductance Calcium-Activated Potassium Channels', MS='A major class of calcium-activated potassium channels that were originally discovered in ERYTHROCYTES. They are found primarily in non-excitable CELLS and set up electrical gradients for PASSIVE ION TRANSPORT.'),
  'D051662': NtO(MH='Kv1.1 Potassium Channel', MS='A delayed rectifier subtype of shaker potassium channels that is commonly mutated in human episodic ATAXIA and MYOKYMIA.'),
  'D051663': NtO(MH='Kv1.2 Potassium Channel', MS='A delayed rectifier subtype of shaker potassium channels that is selectively inhibited by a variety of SCORPION VENOMS.'),
  'D051664': NtO(MH='Kv1.3 Potassium Channel', MS='A delayed rectifier subtype of shaker potassium channels that is the predominant VOLTAGE-GATED POTASSIUM CHANNEL of T-LYMPHOCYTES.'),
  'D051665': NtO(MH='Kv1.4 Potassium Channel', MS='A fast inactivating subtype of shaker potassium channels that contains two inactivation domains at its N terminus.'),
  'D051666': NtO(MH='Kv1.5 Potassium Channel', MS='A delayed rectifier subtype of shaker potassium channels that conducts a delayed rectifier current. It contributes to ACTION POTENTIAL repolarization of MYOCYTES in HEART ATRIA.'),
  'D051667': NtO(MH='Kv1.6 Potassium Channel', MS='A delayed rectifier subtype of shaker potassium channels that has been described in NEURONS and ASTROCYTES.'),
  'D051668': NtO(MH='Delayed Rectifier Potassium Channels', MS='A group of slow opening and closing voltage-gated potassium channels. Because of their delayed activation kinetics they play an important role in controlling ACTION POTENTIAL duration.'),
  'D051669': NtO(MH='Shab Potassium Channels', MS='A subfamily of shaker potassium channels that shares homology with its founding member, Shab protein, Drosophila. They regulate delayed rectifier currents in the NERVOUS SYSTEM of DROSOPHILA and in the SKELETAL MUSCLE and HEART of VERTEBRATES.'),
  'D051670': NtO(MH='Shaw Potassium Channels', MS='A shaker subfamily that is prominently expressed in NEURONS and are necessary for high-frequency, repetitive firing of ACTION POTENTIALS.'),
  'D051672': NtO(MH='Shal Potassium Channels', MS='A shaker subfamily of potassium channels that participate in transient outward potassium currents by activating at subthreshold MEMBRANE POTENTIALS, inactivating rapidly, and recovering from inactivation quickly.'),
  'D051676': NtO(MH='G Protein-Coupled Inwardly-Rectifying Potassium Channels', MS='A family of inwardly-rectifying potassium channels that are activated by PERTUSSIS TOXIN sensitive G-PROTEIN-COUPLED RECEPTORS. GIRK potassium channels are primarily activated by the complex of GTP-BINDING PROTEIN BETA SUBUNITS and GTP-BINDING PROTEIN GAMMA SUBUNITS.'),
  'D051677': NtO(MH='Histiocytoma, Malignant Fibrous', MS='The most commonly diagnosed soft tissue sarcoma. It is a neoplasm with a fibrohistiocytic appearance found chiefly in later adult life, with peak incidence in the 7th decade.'),
  'D051696': NtO(MH='Activating Transcription Factors', MS='Activating transcription factors were originally identified as DNA-BINDING PROTEINS that interact with early promoters from ADENOVIRUSES. They are a family of basic leucine zipper transcription factors that bind to the consensus site TGACGTCA of the cyclic AMP response element, and are closely related to CYCLIC AMP-RESPONSIVE DNA-BINDING PROTEIN.'),
  'D051697': NtO(MH='Activating Transcription Factor 1', MS='An activating transcription factor that regulates expression of a variety of genes including C-JUN GENES and TRANSFORMING GROWTH FACTOR BETA2.'),
  'D051698': NtO(MH='Activating Transcription Factor 2', MS='An activating transcription factor that regulates expression of a variety of GENES including C-JUN GENES; CYCLIN A; CYCLIN D1; and ACTIVATING TRANSCRIPTION FACTOR 3.'),
  'D051699': NtO(MH='Cyclic AMP Response Element-Binding Protein A', MS='A basic leucine zipper transcription factor that is highly homologous to ACTIVATING TRANSCRIPTION FACTOR 2. It binds the consensus site TGACGTCA of the cyclic AMP response element in partnership with either PROTO-ONCOGENE PROTEINS C-JUN or activating transcription factor 2.'),
  'D051700': NtO(MH='Activating Transcription Factor 3', MS='An activating transcription factor that plays a key role in cellular responses to GENOTOXIC STRESS and OXIDATIVE STRESS.'),
  'D051701': NtO(MH='Activating Transcription Factor 4', MS='An activating transcription factor that regulates the expression of a variety of GENES involved in amino acid metabolism and transport. It also interacts with HTLV-I transactivator protein.'),
  'D051702': NtO(MH='Activating Transcription Factor 6', MS='One of the BASIC-LEUCINE ZIPPER TRANSCRIPTION FACTORS that is synthesized as a membrane-bound protein in the ENDOPLASMIC RETICULUM. In response to endoplasmic reticulum stress it translocates to the GOLGI APPARATUS. It is activated by PROTEASES and then moves to the CELL NUCLEUS to regulate GENETIC TRANSCRIPTION of GENES involved in the unfolded protein response.'),
  'D051703': NtO(MH='Sp Transcription Factors', MS='A family of DNA-binding proteins that contain three ZINC FINGERS at their carboxy termini. They are transcription factors that have specificity for GC boxes.'),
  'D051704': NtO(MH='Sp2 Transcription Factor', MS='A ubiquitously expressed Sp transcription factor that binds to the consensus DNA sequence GGGCGGGAC.'),
  'D051705': NtO(MH='Sp3 Transcription Factor', MS='A specificity protein transcription factor that contains three C-terminal CYS2-HIS2 ZINC FINGERS. It regulates expression of a variety of genes including VASCULAR ENDOTHELIAL GROWTH FACTOR and CYCLIN-DEPENDENT KINASE INHIBITOR P27.'),
  'D051706': NtO(MH='Sp4 Transcription Factor', MS='A specificity protein transcription factor that is expressed primarily in the CENTRAL NERVOUS SYSTEM.'),
  'D051716': NtO(MH='Replication Protein A', MS='A single-stranded DNA-binding protein that is found in EUKARYOTIC CELLS. It is required for DNA REPLICATION; DNA REPAIR; and GENETIC RECOMBINATION.'),
  'D051717': NtO(MH='Fushi Tarazu Transcription Factors', MS='Fushi tarazu transcription factors were originally identified in DROSOPHILA. They are found throughout ARTHROPODS and play important roles in segmentation and CENTRAL NERVOUS SYSTEM development.'),
  'D051718': NtO(MH='MutS Homolog 2 Protein', MS='A highly conserved eukaryotic homolog of the MUTS DNA MISMATCH-BINDING PROTEIN. It plays an essential role in meiotic RECOMBINATION and DNA REPAIR of mismatched NUCLEOTIDES.'),
  'D051719': NtO(MH='MutS DNA Mismatch-Binding Protein', MS='A methyl-directed mismatch DNA REPAIR protein that has weak ATPASE activity. MutS was originally described in ESCHERICHIA COLI.'),
  'D051720': NtO(MH='Upstream Stimulatory Factors', MS='Ubiquitously expressed basic HELIX-LOOP-HELIX MOTIF transcription factors. They bind CANNTG sequences in the promoters of a variety of GENES involved in carbohydrate and lipid metabolism.'),
  'D051721': NtO(MH='Rad52 DNA Repair and Recombination Protein', MS='A DNA-binding protein that mediates DNA REPAIR of double strand breaks, and HOMOLOGOUS RECOMBINATION.'),
  'D051722': NtO(MH='ADAM Proteins', MS='A family of membrane-anchored glycoproteins that contain a disintegrin and metalloprotease domain. They are responsible for the proteolytic cleavage of many transmembrane proteins and the release of their extracellular domain.'),
  'D051736': NtO(MH='Proto-Oncogene Proteins c-mdm2', MS='An E3 UBIQUITIN LIGASE that interacts with and inhibits TUMOR SUPPRESSOR PROTEIN P53. Its ability to ubiquitinate p53 is regulated by TUMOR SUPPRESSOR PROTEIN P14ARF.'),
  'D051738': NtO(MH='Origin Recognition Complex', MS='The origin recognition complex is a multi-subunit DNA-binding protein that initiates DNA REPLICATION in eukaryotes.'),
  'D051739': NtO(MH='Microphthalmia-Associated Transcription Factor', MS='A basic helix-loop-helix leucine zipper transcription factor that regulates the CELL DIFFERENTIATION and development of a variety of cell types including MELANOCYTES; OSTEOCLASTS; and RETINAL PIGMENT EPITHELIUM. Mutations in MITF protein have been associated with OSTEOPETROSIS and WAARDENBURG SYNDROME.'),
  'D051740': NtO(MH='Ikaros Transcription Factor', MS='A transcription factor that plays a role as a key regulator of HEMATOPOIESIS. Aberrant Ikaros expression has been associated with LYMPHOBLASTIC LEUKEMIA.'),
  'D051741': NtO(MH='Kruppel-Like Transcription Factors', MS='A family of zinc finger transcription factors that share homology with Kruppel protein, Drosophila. They contain a highly conserved seven amino acid spacer sequence in between their ZINC FINGER MOTIFS.'),
  'D051742': NtO(MH='Transcription Factor CHOP', MS='A CCAAT-enhancer binding protein that is induced by DNA DAMAGE and growth arrest. It serves as a dominant negative inhibitor of other CCAAT-enhancer binding proteins.'),
  'D051743': NtO(MH='CCAAT-Enhancer-Binding Protein-delta', MS='A member of the C-EBP protein family of transcription factors. It plays a key role in G0 PHASE mammary EPITHELIAL CELL growth arrest, and it is involved in transcriptional regulation of INTERLEUKIN 1; INTERLEUKIN 6; and TUMOR NECROSIS FACTOR-ALPHA.'),
  'D051744': NtO(MH='Protein Kinase C-epsilon', MS='A protein kinase C subtype that was originally characterized as a CALCIUM-independent, serine-threonine kinase that is activated by PHORBOL ESTERS and DIACYLGLYCEROLS. It is targeted to specific cellular compartments in response to extracellular signals that activate G-PROTEIN-COUPLED RECEPTORS; TYROSINE KINASE RECEPTORS; and intracellular protein tyrosine kinase.'),
  'D051745': NtO(MH='Protein Kinase C-delta', MS='A ubiquitously expressed protein kinase that is involved in a variety of cellular SIGNAL PATHWAYS. Its activity is regulated by a variety of signaling protein tyrosine kinase.'),
  'D051746': NtO(MH='ZAP-70 Protein-Tyrosine Kinase', MS='A protein tyrosine kinase that is required for T-CELL development and T-CELL ANTIGEN RECEPTOR function.'),
  'D051747': NtO(MH='DNA-Activated Protein Kinase', MS='A serine-threonine protein kinase that, when activated by DNA, phosphorylates several DNA-binding protein substrates including the TUMOR SUPPRESSOR PROTEIN P53 and a variety of TRANSCRIPTION FACTORS.'),
  'D051756': NtO(MH='Leucine-Responsive Regulatory Protein', MS='A LEUCINE and DNA-binding protein that is found primarily in BACTERIA and ARCHAEA. It regulates GENETIC TRANSCRIPTION involved in METABOLISM of AMINO ACIDS in response to the increased concentration of LEUCINE.'),
  'D051757': NtO(MH='PAX5 Transcription Factor', MS='A paired box transcription factor that is essential for CELL DIFFERENTIATION of B-LYMPHOCYTES. It functions both as a transcriptional activator and repressor to mediate B-cell commitment.'),
  'D051758': NtO(MH='Transcription Factor TFIIH', MS='A general transcription factor that is involved in basal GENETIC TRANSCRIPTION and NUCLEOTIDE EXCISION REPAIR. It consists of nine subunits including ATP-DEPENDENT DNA HELICASES; CYCLIN H; and XERODERMA PIGMENTOSUM GROUP D PROTEIN.'),
  'D051759': NtO(MH='Xeroderma Pigmentosum Group D Protein', MS='A DNA helicase that is a component of TRANSCRIPTION FACTOR TFIIH. It plays an essential role in NUCLEOTIDE EXCISION REPAIR, and mutations in this protein are associated with XERODERMA PIGMENTOSUM.'),
  'D051760': NtO(MH='Xeroderma Pigmentosum Group A Protein', MS='A ZINC FINGER MOTIF protein that recognizes and interacts with damaged DNA. It is a DNA-binding protein that plays an essential role in NUCLEOTIDE EXCISION REPAIR. Mutations in this protein are associated with the most severe form of XERODERMA PIGMENTOSUM.'),
  'D051761': NtO(MH='Paired Box Transcription Factors', MS='A family of transcription factors that control EMBRYONIC DEVELOPMENT within a variety of cell lineages. They are characterized by a highly conserved paired DNA-binding domain that was first identified in DROSOPHILA segmentation genes.'),
  'D051762': NtO(MH='PAX2 Transcription Factor', MS='A paired box transcription factor that is essential for ORGANOGENESIS of the CENTRAL NERVOUS SYSTEM and KIDNEY.'),
  'D051763': NtO(MH='PAX7 Transcription Factor', MS='A paired box transcription factor that is involved in EMBRYONIC DEVELOPMENT of the CENTRAL NERVOUS SYSTEM and SKELETAL MUSCLE.'),
  'D051764': NtO(MH='PAX9 Transcription Factor', MS='A paired box transcription factor that is involved in ODONTOGENESIS.'),
  'D051765': NtO(MH='Early Growth Response Transcription Factors', MS='A family of transcription factors that are induced by GROWTH FACTORS and contain a highly conserved DNA-binding domain composed of three ZINC FINGER MOTIFS.'),
  'D051766': NtO(MH='Early Growth Response Protein 1', MS='An early growth response transcription factor and zinc finger protein that has been implicated in regulation of CELL PROLIFERATION and APOPTOSIS.'),
  'D051767': NtO(MH='Early Growth Response Protein 2', MS='An early growth response transcription factor that controls the formation of the MYELIN SHEATH around peripheral AXONS by SCHWANN CELLS. Mutations in EGR2 transcription factor have been associated with HEREDITARY MOTOR AND SENSORY NEUROPATHIES such as CHARCOT-MARIE-TOOTH DISEASE.'),
  'D051776': NtO(MH='Early Growth Response Protein 3', MS='An early growth response transcription factor that is essential for the development of MUSCLE SPINDLES.'),
  'D051777': NtO(MH='AraC Transcription Factor', MS='A transcription factor found in BACTERIA that positively and negatively regulates the expression of proteins required for the uptake and catabolism of L-ARABINOSE.'),
  'D051778': NtO(MH='Basic Helix-Loop-Helix Leucine Zipper Transcription Factors', MS='A family of transcription factors that contain regions rich in basic residues, LEUCINE ZIPPER domains, and HELIX-LOOP-HELIX MOTIFS.'),
  'D051779': NtO(MH='Sterol Regulatory Element Binding Proteins', MS='Sterol regulatory element binding proteins are basic helix-loop-helix leucine zipper transcription factors that bind the sterol regulatory element TCACNCCAC. They are synthesized as precursors that are threaded into the MEMBRANES of the ENDOPLASMIC RETICULUM.'),
  'D051780': NtO(MH='Sterol Regulatory Element Binding Protein 1', MS='A sterol regulatory element binding protein that regulates expression of GENES involved in FATTY ACIDS metabolism and LIPOGENESIS. Two major isoforms of the protein exist due to ALTERNATIVE SPLICING.'),
  'D051782': NtO(MH='Sterol Regulatory Element Binding Protein 2', MS='A sterol regulatory element binding protein that regulates GENES involved in CHOLESTEROL synthesis and uptake.'),
  'D051783': NtO(MH='Methyl-CpG-Binding Protein 2', MS='A DNA-binding protein that interacts with methylated CPG ISLANDS. It plays a role in repressing GENETIC TRANSCRIPTION and is frequently mutated in RETT SYNDROME.'),
  'D051784': NtO(MH='Aryl Hydrocarbon Receptor Nuclear Translocator', MS='Aryl hydrocarbon receptor nuclear translocator is a basic HELIX-LOOP-HELIX MOTIF containing protein that forms a complex with DIOXIN RECEPTOR. The complex binds xenobiotic regulatory elements and activates transcription of a variety of genes including UDP GLUCURONOSYLTRANSFERASE. AhR nuclear translocator is also a subunit of HYPOXIA-INDUCIBLE FACTOR 1.'),
  'D051785': NtO(MH='Smad Proteins', MS='A family of proteins that are involved in the translocation of signals from TGF-BETA RECEPTORS; BONE MORPHOGENETIC PROTEIN RECEPTORS; and other surface receptors to the CELL NUCLEUS. They were originally identified as a class of proteins that are related to the mothers against decapentaplegic protein, Drosophila and sma proteins from CAENORHABDITIS ELEGANS.'),
  'D051786': NtO(MH='Cyclic AMP Response Element Modulator', MS='Cyclic AMP response element modulator is a basic leucine zipper transcription factor that is regulated by CYCLIC AMP. It plays an important role in SPERMATID development in the mammalian TESTIS.'),
  'D051787': NtO(MH='Myogenic Regulatory Factor 5', MS='A SKELETAL MUSCLE-specific transcription factor that contains a basic HELIX-LOOP-HELIX MOTIF. It plays an essential role in MUSCLE DEVELOPMENT.'),
  'D051788': NtO(MH='Myeloid-Lymphoid Leukemia Protein', MS='Myeloid-lymphoid leukemia protein is a transcription factor that maintains high levels of HOMEOTIC GENE expression during development. The GENE for myeloid-lymphoid leukemia protein is commonly disrupted in LEUKEMIA and combines with over 40 partner genes to form FUSION ONCOGENE PROTEINS.'),
  'D051789': NtO(MH='Proto-Oncogene Protein c-fli-1', MS='A member of the c-ets family of transcription factors that is preferentially expressed in cells of hematopoietic lineages and vascular endothelial cells. It was originally identified as a protein that provides a retroviral integration site for integration of FRIEND MURINE LEUKEMIA VIRUS.'),
  'D051790': NtO(MH='G-Box Binding Factors', MS='A family of transcription factors found primarily in PLANTS that bind to the G-box DNA sequence CACGTG or to a consensus sequence CANNTG.'),
  'D051792': NtO(MH='Basic Helix-Loop-Helix Transcription Factors', MS='A family of DNA-binding transcription factors that contain a basic HELIX-LOOP-HELIX MOTIF.'),
  'D051793': NtO(MH='Hypoxia-Inducible Factor 1', MS='A basic helix-loop-helix transcription factor that plays a role in APOPTOSIS. It is composed of two subunits: ARYL HYDROCARBON RECEPTOR NUCLEAR TRANSLOCATOR and HYPOXIA-INDUCIBLE FACTOR 1, ALPHA SUBUNIT.'),
  'D051794': NtO(MH='Von Hippel-Lindau Tumor Suppressor Protein', MS='A ubiquitin-protein ligase that mediates OXYGEN-dependent polyubiquitination of HYPOXIA-INDUCIBLE FACTOR 1, ALPHA SUBUNIT. It is inactivated in VON HIPPEL-LINDAU SYNDROME.'),
  'D051795': NtO(MH='Hypoxia-Inducible Factor 1, alpha Subunit', MS='Hypoxia-inducible factor 1, alpha subunit is a basic helix-loop-helix transcription factor that is regulated by OXYGEN availability and is targeted for degradation by VHL TUMOR SUPPRESSOR PROTEIN.'),
  'D051796': NtO(MH='Inhibitor of Differentiation Proteins', MS='Inhibitor of differentiation proteins are negative regulators of BASIC HELIX-LOOP-HELIX TRANSCRIPTION FACTORS. They inhibit CELL DIFFERENTIATION and induce CELL PROLIFERATION by modulating different CELL CYCLE regulators.'),
  'D051797': NtO(MH='Inhibitor of Differentiation Protein 2', MS='A negative regulator of BASIC HELIX-LOOP-HELIX TRANSCRIPTION FACTORS. It plays a role in regulating IMMUNOGLOBULIN E expression.'),
  'D051798': NtO(MH='Inhibitor of Differentiation Protein 1', MS='A HELIX-LOOP-HELIX MOTIF-containing negative regulator of BASIC HELIX-LOOP-HELIX TRANSCRIPTION FACTORS that blocks activation of CYCLIN-DEPENDENT KINASE INHIBITOR P16 and is de-regulated in a variety of NEOPLASMS.'),
  'D051799': NtO(MH='Delayed Graft Function', MS='General dysfunction of an organ occurring immediately following its transplantation. The term most frequently refers to renal dysfunction following KIDNEY TRANSPLANTATION.'),
  'D051816': NtO(MH='Tristetraprolin', MS='A ZINC FINGER MOTIF containing transcription factor that was originally identified as one of the IMMEDIATE-EARLY PROTEINS. It shuttles between the CYTOPLASM and the CELL NUCLEUS and is involved in destabilization of mRNAs for TUMOR NECROSIS FACTOR-ALPHA.'),
  'D051817': NtO(MH='Butyrate Response Factor 1', MS='A ZINC FINGER MOTIF containing immediate early protein that is closely related to TRISTETRAPROLIN. It interacts with and helps destabilize VASCULAR ENDOTHELIAL GROWTH FACTOR mRNA.'),
  'D051818': NtO(MH='Replication Protein C', MS='A DNA-binding protein and AAA ATPase that consists of 5 polypeptides and plays an essential role in DNA REPLICATION in eukaryotes. It binds DNA PRIMER-template junctions and recruits PROLIFERATING CELL NUCLEAR ANTIGEN and DNA POLYMERASES to the site of DNA synthesis.'),
  'D051819': NtO(MH='Immunoglobulin J Recombination Signal Sequence-Binding Protein', MS='A ubiquitously expressed sequence-specific transcriptional repressor that is normally the target of signaling by NOTCH PROTEINS.'),
  'D051820': NtO(MH='TCF Transcription Factors', MS='A family of DNA-binding proteins that are primarily expressed in T-LYMPHOCYTES. They interact with BETA CATENIN and serve as transcriptional activators and repressors in a variety of developmental processes.'),
  'D051821': NtO(MH='Lymphoid Enhancer-Binding Factor 1', MS='A T-cell factor that plays an essential role in EMBRYONIC DEVELOPMENT.'),
  'D051822': NtO(MH='T Cell Transcription Factor 1', MS='A TCF transcription factor that was originally identified as a DNA-binding protein that interacts with the enhancers of T-CELL RECEPTOR ALPHA GENES. It plays a role in T-LYMPHOCYTE development.'),
  'D051836': NtO(MH='COUP Transcription Factors', MS='A sub-family of steroid receptor-related orphan nuclear receptors that have specificity for a variety of DNA sequences related to AGGTCA. COUP transcription factors can heterodimerize with a variety of factors including RETINOIC ACID RECEPTORS; THYROID HORMONE RECEPTORS; and VITAMIN D RECEPTORS.'),
  'D051837': NtO(MH='COUP Transcription Factor II', MS='A COUP transcription factor that negatively regulates GENETIC TRANSCRIPTION and competes with other hormone receptors for the common response element AGGTCA. It can also stimulate transcription of genes involved in the metabolism of GLUCOSE and CHOLESTEROL.'),
  'D051838': NtO(MH='COUP Transcription Factor I', MS='A COUP transcription factor that was originally identified as a homodimer that binds to a direct repeat regulatory element in the chicken albumin promoter. It is a transcription factor that plays an important role in EMBRYONIC DEVELOPMENT of the CENTRAL NERVOUS SYSTEM.'),
  'D051839': NtO(MH='NFI Transcription Factors', MS='Transcription factors that were originally identified as site-specific DNA-binding proteins essential for DNA REPLICATION by ADENOVIRUSES. They play important roles in MAMMARY GLAND function and development.'),
  'D051840': NtO(MH='Y-Box-Binding Protein 1', MS='A CCAAT-enhancer-binding protein that was originally identified as interacting with Y-box PROMOTER REGIONS of MHC CLASS II GENES. It is a highly conserved transcription factor that regulates expression of a wide variety of GENES.'),
  'D051841': NtO(MH='Transcription Factor RelB', MS='A transcription factor that takes part in the NF-kappa-B complex by interacting with NF-KAPPA B P50 SUBUNIT or NF-KAPPA B P52 SUBUNIT. It regulates GENETIC TRANSCRIPTION that is involved in immune and inflammatory responses.'),
  'D051842': NtO(MH='Olfactory Marker Protein', MS='A ubiquitous, cytoplasmic protein found in mature OLFACTORY RECEPTOR NEURONS of all VERTEBRATES. It is a modulator of the olfactory SIGNAL TRANSDUCTION PATHWAY.'),
  'D051843': NtO(MH='Synucleins', MS='A family of homologous proteins of low MOLECULAR WEIGHT that are predominately expressed in the BRAIN and that have been implicated in a variety of human diseases. They were originally isolated from CHOLINERGIC FIBERS of TORPEDO.'),
  'D051844': NtO(MH='alpha-Synuclein', MS='A synuclein that is a major component of LEWY BODIES and plays a role in SYNUCLEINOPATHIES, neurodegeneration and neuroprotection.'),
  'D051845': NtO(MH='gamma-Synuclein', MS='A homolog of ALPHA-SYNUCLEIN that plays a role in neurofilament network integrity. It is overexpressed in a variety of human NEOPLASMS and may be involved in modulating AXON architecture during EMBRYONIC DEVELOPMENT and in the adult. Gamma-Synuclein may also activate SIGNAL TRANSDUCTION PATHWAYS associated with ETS-DOMAIN PROTEIN ELK-1.'),
  'D051846': NtO(MH='beta-Synuclein', MS='A synuclein that is closely related to ALPHA-SYNUCLEIN. It may play a neuroprotective role against some of the toxic effects of aggregated ALPHA-SYNUCLEIN.'),
  'D051847': NtO(MH='Lithostathine', MS='The proteinaceous component of the pancreatic stone in patients with PANCREATITIS.'),
  'D051856': NtO(MH='Fanconi Anemia Complementation Group Proteins', MS='A diverse group of proteins whose genetic MUTATIONS have been associated with the chromosomal instability syndrome FANCONI ANEMIA. Many of these proteins play important roles in protecting CELLS against OXIDATIVE STRESS.'),
  'D051857': NtO(MH='Otx Transcription Factors', MS='A family of VERTEBRATE homeodomain proteins that share homology with orthodenticle protein, Drosophila. They regulate GENETIC TRANSCRIPTION and play an important role in EMBRYONIC DEVELOPMENT of the BRAIN.'),
  'D051858': NtO(MH='Forkhead Transcription Factors', MS='A subclass of winged helix DNA-binding proteins that share homology with their founding member fork head protein, Drosophila.'),
  'D051859': NtO(MH='Neuroendocrine Secretory Protein 7B2', MS='An acidic protein found in the NEUROENDOCRINE SYSTEM that functions as a molecular chaperone for PROPROTEIN CONVERTASE 2.'),
  'D051860': NtO(MH='Fragile X Mental Retardation Protein', MS='A RNA-binding protein that is found predominately in the CYTOPLASM. It helps regulate GENETIC TRANSLATION in NEURONS and is absent or under-expressed in FRAGILE X SYNDROME.'),
  'D051861': NtO(MH='Kangai-1 Protein', MS='A widely expressed transmembrane glycoprotein that functions as a METASTASIS suppressor protein. It is underexpressed in a variety of human NEOPLASMS.'),
  'D051863': NtO(MH='Host Cell Factor C1', MS='A cellular transcriptional coactivator that contains N-terminal KELCH REPEATS and N- and C-terminal FIBRONECTIN TYPE III DOMAINS. It functions as a transcription factor for a number of genes and in the assembly of IMMEDIATE-EARLY PROTEINS of the HERPES SIMPLEX VIRUS.  Mutations in the HCFC1 gene are associated with cases of X-LINKED MENTAL RETARDATION.'),
  'D051876': NtO(MH='Sex-Determining Region Y Protein', MS='A transcription factor that plays an essential role in the development of the TESTES. It is encoded by a gene on the Y chromosome and contains a specific HMG-BOX DOMAIN that is found within members of the SOX family of transcription factors.'),
  'D051877': NtO(MH='RNA Polymerase Sigma 54', MS='A DNA-directed RNA polymerase found in BACTERIA. It is a holoenzyme that consists of multiple subunits including sigma factor 54.'),
  'D051879': NtO(MH='Twist-Related Protein 1', MS='A basic helix-loop-helix transcription factor that was originally identified in DROSOPHILA as essential for proper gastrulation and MESODERM formation. It plays an important role in EMBRYONIC DEVELOPMENT and CELL DIFFERENTIATION of MUSCLE CELLS, and is found in a wide variety of organisms.'),
  'D051880': NtO(MH='Receptors, Notch', MS='A family of conserved cell surface receptors that contain EPIDERMAL GROWTH FACTOR repeats in their extracellular domain and ANKYRIN REPEATS in their cytoplasmic domains. The cytoplasmic domains are released upon ligand binding and translocate to the CELL NUCLEUS, where they act as transcription factors.'),
  'D051881': NtO(MH='Receptor, Notch1', MS='A notch receptor that interacts with a variety of ligands and regulates SIGNAL TRANSDUCTION PATHWAYS for multiple cellular processes. It is widely expressed during EMBRYOGENESIS and is essential for EMBRYONIC DEVELOPMENT.'),
  'D051883': NtO(MH='Receptor, Notch2', MS='A notch receptor that plays an important role in CELL DIFFERENTIATION in a variety of cell types. It is the preferentially expressed notch receptor in mature B-LYMPHOCYTES.'),
  'D051896': NtO(MH='Smad Proteins, Receptor-Regulated', MS='A family of smad proteins that undergo PHOSPHORYLATION by CELL SURFACE RECEPTORS in response to TRANSFORMING GROWTH FACTOR BETA; ACTIVIN; or BONE MORPHOGENETIC PROTEIN signaling.'),
  'D051897': NtO(MH='Smad Proteins, Inhibitory', MS='A sub-family of smad proteins that inhibit cell signaling by RECEPTOR-REGULATED SMAD PROTEINS. They form autoinhibitory feedback loops in the TGF-BETA signaling pathway and mediate signaling cross-talk with other signaling pathways'),
  'D051898': NtO(MH='Smad1 Protein', MS='A receptor-regulated smad protein that undergoes PHOSPHORYLATION by BONE MORPHOGENETIC PROTEIN RECEPTORS. It regulates BONE MORPHOGENETIC PROTEIN signaling and plays an essential role in EMBRYONIC DEVELOPMENT.'),
  'D051899': NtO(MH='Smad2 Protein', MS='A receptor-regulated smad protein that undergoes PHOSPHORYLATION by ACTIVIN RECEPTORS, TYPE I. It regulates TRANSFORMING GROWTH FACTOR BETA and ACTIVIN signaling.'),
  'D051900': NtO(MH='Smad3 Protein', MS='A receptor-regulated smad protein that undergoes PHOSPHORYLATION by ACTIVIN RECEPTORS, TYPE I. Activated Smad3 can bind directly to DNA, and it regulates TRANSFORMING GROWTH FACTOR BETA and ACTIVIN signaling.'),
  'D051901': NtO(MH='Smad4 Protein', MS='A signal transducing adaptor protein and tumor suppressor protein. It forms a complex with activated RECEPTOR-REGULATED SMAD PROTEINS. The complex then translocates to the CELL NUCLEUS and regulates GENETIC TRANSCRIPTION of target GENES.'),
  'D051902': NtO(MH='Smad5 Protein', MS='A receptor-regulated smad protein that undergoes PHOSPHORYLATION by BONE MORPHOGENETIC PROTEIN RECEPTORS. It regulates BONE MORPHOGENETIC PROTEIN signaling and is essential for PHYSIOLOGICAL ANGIOGENESIS.'),
  'D051903': NtO(MH='Nuclear Factor 45 Protein', MS='A protein subunit that takes part in forming nuclear factor 90 protein complexes.'),
  'D051904': NtO(MH='Smad8 Protein', MS='A receptor-regulated smad protein that undergoes PHOSPHORYLATION by BONE MORPHOGENETIC PROTEIN RECEPTORS and regulates BONE MORPHOGENETIC PROTEIN signaling.'),
  'D051905': NtO(MH='Smad6 Protein', MS='An inhibitory Smad protein that negatively regulates the SIGNAL TRANSDUCTION PATHWAYS from BONE MORPHOGENETIC PROTEIN RECEPTORS. Smad6 inhibits PHOSPHORYLATION of SMAD2 PROTEIN and SMAD3 PROTEIN.'),
  'D051906': NtO(MH='Smad7 Protein', MS='An inhibitory smad protein that associates with TRANSFORMING GROWTH FACTOR BETA RECEPTORS and BONE MORPHOGENETIC PROTEIN RECEPTORS. It negatively regulates SIGNAL TRANSDUCTION PATHWAYS by inhibiting PHOSPHORYLATION of RECEPTOR-REGULATED SMAD PROTEINS.'),
  'D051907': NtO(MH='Lysosome-Associated Membrane Glycoproteins', MS='Ubiquitously expressed integral membrane glycoproteins found in the LYSOSOME.'),
  'D051916': NtO(MH='Membrane Cofactor Protein', MS='A ubiquitously expressed complement receptor that binds COMPLEMENT C3B and COMPLEMENT C4B and serves as a co-factor for their inactivation by COMPLEMENT FACTOR I. It also interacts with a wide variety of pathogens and mediates the immune response.'),
  'D051917': NtO(MH='Leukosialin', MS='A sialic acid-rich protein and an integral cell membrane mucin. It plays an important role in activation of T-LYMPHOCYTES.'),
  'D051918': NtO(MH='Sialic Acid Binding Ig-like Lectin 2', MS='A lectin and cell adhesion molecule found in B-LYMPHOCYTES. It interacts with SIALIC ACIDS and mediates signaling from B-CELL ANTIGEN RECEPTORS.'),
  'D051920': NtO(MH='NK Cell Lectin-Like Receptor Subfamily D', MS='A subclass of NK cell lectin-like receptors that associates with a variety of members of NK CELL LECTIN-LIKE RECEPTOR SUBFAMILY C to form heterodimeric receptors for HLA-E antigen.'),
  'D051921': NtO(MH='Fos-Related Antigen-2', MS='A basic-leucine zipper transcription factor that is closely related to C-FOS PROTEINS. It forms heterodimeric complexes with C-JUN PROTEINS to regulate GENE transcription.'),
  'D051922': NtO(MH='PII Nitrogen Regulatory Proteins', MS='A family of signal transducing adaptor proteins that control the METABOLISM of NITROGEN. They are primarily found in prokaryotes.'),
  'D051925': NtO(MH='CD79 Antigens', MS='Components of the B-cell antigen receptor that function in B-cell antigen receptor heavy chain transport to the PLASMA MEMBRANE. They are expressed almost exclusively by B-LYMPHOCYTES and are markers for B-cell NEOPLASMS.'),
  'D051926': NtO(MH='Basigin', MS='A widely distributed cell surface transmembrane glycoprotein characterized by extracellular IMMUNOGLOBULIN DOMAINS. It stimulates the synthesis of MATRIX METALLOPROTEINASES and is found at high levels on the surface of malignant NEOPLASMS where it may play a role as a mediator of malignant cell behavior.'),
  'D051927': NtO(MH='CD24 Antigen', MS='A GPI-linked cell adhesion protein originally identified as a heat stable antigen in mice. It mediates antigen-dependent activation and proliferation of B-CELLS. It is also involved in METASTASIS and is highly expressed in many NEOPLASMS.'),
  'D051928': NtO(MH='CD47 Antigen', MS='A ubiquitously expressed membrane glycoprotein. It interacts with a variety of INTEGRINS and mediates responses to EXTRACELLULAR MATRIX PROTEINS.'),
  'D051929': NtO(MH='CD146 Antigen', MS='A cell adhesion molecule of the immunoglobulin superfamily that is expressed by ENDOTHELIAL CELLS where it functions to stabilize INTERCELLULAR JUNCTIONS. It is also highly expressed by melanoma tumor cells and may facilitate their METASTASIS.'),
  'D051930': NtO(MH='Endolyn', MS='A sialomucin protein that functions as a cell adhesion molecule. It is a negative regulator of certain types of HEMATOPOIETIC STEM CELLS.'),
  'D051936': NtO(MH='Centromere Protein B', MS='A DNA-binding protein that interacts with a 17-base pair sequence known as the CENP-B box motif. The protein is localized constitutively to the CENTROMERE and plays an important role in its maintenance.'),
  'D051937': NtO(MH='Dopamine and cAMP-Regulated Phosphoprotein 32', MS='A phosphoprotein that was initially identified as a major target of DOPAMINE activated ADENYLYL CYCLASE in the CORPUS STRIATUM. It regulates the activities of PROTEIN PHOSPHATASE-1 and PROTEIN KINASE A, and it is a key mediator of the biochemical, electrophysiological, transcriptional, and behavioral effects of DOPAMINE.'),
  'D051938': NtO(MH='Munc18 Proteins', MS='A family of proteins involved in intracellular membrane trafficking. They interact with SYNTAXINS and play important roles in vesicular docking and fusion during EXOCYTOSIS. Their name derives from the fact that they are related to Unc-18 protein, C elegans.'),
  'D051939': NtO(MH='Phytochrome B', MS='A plant photo regulatory protein that exists in two forms that are reversibly interconvertible by LIGHT. In response to light it moves to the CELL NUCLEUS and regulates transcription of target genes. Phytochrome B plays an important role in shade avoidance and mediates plant de-etiolation in red light.'),
  'D051940': NtO(MH='B7-2 Antigen', MS='A costimulatory ligand glycoprotein that contains a C2 and V-type IMMUNOGLOBULIN DOMAIN. It is expressed by ANTIGEN-PRESENTING CELLS and binds to CD28 ANTIGEN with high specificity and to CTLA-4 ANTIGEN with low specificity. The interaction of CD86 with CD28 ANTIGEN provides a stimulatory signal to T-LYMPHOCYTES, while its interaction with CTLA-4 ANTIGEN may play a role in inducing PERIPHERAL TOLERANCE.'),
  'D051941': NtO(MH='fms-Like Tyrosine Kinase 3', MS='A receptor tyrosine kinase that is involved in HEMATOPOIESIS. It is closely related to FMS PROTO-ONCOGENE PROTEIN and is commonly mutated in acute MYELOID LEUKEMIA.'),
  'D051942': NtO(MH='Phytochrome A', MS='The primary plant photoreceptor responsible for perceiving and mediating responses to far-red light. It is a PROTEIN-SERINE-THREONINE KINASE that is translocated to the CELL NUCLEUS in response to light signals.'),
  'D051943': NtO(MH='Antennapedia Homeodomain Protein', MS='Antennapedia homeodomain protein is a homeobox protein involved in limb patterning in ARTHROPODS. Mutations in the gene for the antennapedia homeodomain protein are associated with the conversion of antenna to leg or leg to antenna DROSOPHILA.'),
  'D051956': NtO(MH='Goosecoid Protein', MS='Goosecoid protein is a homeodomain protein that was first identified in XENOPUS. It is found in the SPEMANN ORGANIZER of VERTEBRATES and plays an important role in neuronal CELL DIFFERENTIATION and ORGANOGENESIS.'),
  'D051957': NtO(MH='MSX1 Transcription Factor', MS='A homeodomain protein that interacts with TATA-BOX BINDING PROTEIN. It represses GENETIC TRANSCRIPTION of target GENES and plays a critical role in ODONTOGENESIS.'),
  'D051958': NtO(MH='Nuclear Receptor Coactivator 2', MS='A transcription factor that partners with ligand bound GLUCOCORTICOID RECEPTORS and ESTROGEN RECEPTORS to stimulate GENETIC TRANSCRIPTION. It plays an important role in FERTILITY as well as in METABOLISM of LIPIDS.'),
  'D051959': NtO(MH='ELAV Proteins', MS='A family of RRM PROTEINS that are homologues of ELAV protein, Drosophila. They were initially identified in humans as the targets of autoantibodies in patients with PARANEOPLASTIC ENCEPHALOMYELITIS. They regulate GENE EXPRESSION at the post-transcriptional level.'),
  'D051960': NtO(MH='GTP-Binding Protein alpha Subunit, Gi2', MS='A PERTUSSIS TOXIN-sensitive GTP-binding protein alpha subunit. It couples with a variety of CELL SURFACE RECEPTORS, has been implicated in INTERLEUKIN-12 production, and may play a role in INFLAMMATORY BOWEL DISEASES.'),
  'D051964': NtO(MH='Receptors, Mating Factor', MS='A family of pheromone receptors that were initially discovered in SACCHAROMYCES CEREVISIAE as proteins necessary for fungal conjugation. Each mating factor receptor is expressed in HAPLOID CELLS of a single mating type.'),
  'D051966': NtO(MH='Phospholipase C gamma', MS='A phosphoinositide phospholipase C subtype that is primarily regulated by PROTEIN-TYROSINE KINASES. It is structurally related to PHOSPHOLIPASE C DELTA with the addition of SRC HOMOLOGY DOMAINS and pleckstrin homology domains located between two halves of the CATALYTIC DOMAIN.'),
  'D051981': NtO(MH='Uracil-DNA Glycosidase', MS='An enzyme that catalyzes the HYDROLYSIS of the N-glycosidic bond between sugar phosphate backbone and URACIL residue during DNA synthesis.'),
  'D051996': NtO(MH='Transcription Factor RelA', MS='A subunit of NF-kappa B that is primarily responsible for its transactivation function. It contains a C-terminal transactivation domain and an N-terminal domain with homology to PROTO-ONCOGENE PROTEINS C-REL.'),
  'D051997': NtO(MH='ADP-ribosyl Cyclase 1', MS='A bifunctional enzyme that catalyzes the synthesis and HYDROLYSIS of CYCLIC ADP-RIBOSE (cADPR) from NAD+ to ADP-RIBOSE. It is a cell surface molecule which is predominantly expressed on LYMPHOID CELLS and MYELOID CELLS.'),
  'D052002': NtO(MH='NF-kappa B p50 Subunit', MS='A component of NF-kappa B transcription factor. It is proteolytically processed from NF-kappa B p105 precursor protein and is capable of forming dimeric complexes with itself or with TRANSCRIPTION FACTOR RELA. It regulates expression of GENES involved in immune and inflammatory responses.'),
  'D052003': NtO(MH='NF-kappa B p52 Subunit', MS='A component of NF-kappa B transcription factor. It is proteolytically processed from NF-kappa B p100 precursor protein and is important for maturation of B-LYMPHOCYTES and adaptive HUMORAL IMMUNITY.'),
  'D052004': NtO(MH='Bone Morphogenetic Protein Receptors', MS='A family of CELL SURFACE RECEPTORS that bind BONE MORPHOGENETIC PROTEINS. They are PROTEIN-SERINE-THREONINE KINASES that mediate SIGNAL TRANSDUCTION PATHWAYS through SMAD PROTEINS.'),
  'D052005': NtO(MH='Bone Morphogenetic Protein Receptors, Type I', MS='A subtype of bone morphogenetic protein receptors with high affinity for BONE MORPHOGENETIC PROTEINS. They can interact with and undergo PHOSPHORYLATION by BONE MORPHOGENETIC PROTEIN RECEPTORS, TYPE II. They signal primarily through RECEPTOR-REGULATED SMAD PROTEINS.'),
  'D052006': NtO(MH='Bone Morphogenetic Protein Receptors, Type II', MS='A subtype of bone morphogenetic protein receptors with low affinity for BONE MORPHOGENETIC PROTEINS. They are constitutively active PROTEIN-SERINE-THREONINE KINASES that can interact with and phosphorylate TYPE I BONE MORPHOGENETIC PROTEIN RECEPTORS.'),
  'D052016': NtO(MH='Mucositis', MS='An INFLAMMATION of the MUCOSA with burning or tingling sensation. It is characterized by atrophy of the squamous EPITHELIUM, vascular damage, inflammatory infiltration, and ulceration. It usually occurs at the mucous lining of the MOUTH, the GASTROINTESTINAL TRACT or the airway due to chemical irritations, CHEMOTHERAPY, or radiation therapy (RADIOTHERAPY).'),
  'D052018': NtO(MH='Bulimia Nervosa', MS='An eating disorder that is characterized by a cycle of binge eating (BULIMIA or bingeing) followed by inappropriate acts (purging) to avert weight gain. Purging methods often include self-induced VOMITING, use of LAXATIVES or DIURETICS, excessive exercise, and FASTING.'),
  'D052058': NtO(MH='Hinge Exons', MS='Immunoglobulin heavy chain gene exons coding for the hinge region of the heavy chains between the first constant region (on the FAB FRAGMENTS) and the second constant region (on the FC FRAGMENTS).'),
  'D052060': NtO(MH='Research Support, N.I.H., Intramural', MS='A designation for publications of research resulting from intramural research at the National Institutes of Health.'),
  'D052061': NtO(MH='Research Support, N.I.H., Extramural', MS='A designation for publications of research resulting from extramural research funded by the National Institutes of Health.'),
  'D052065': NtO(MH='Vaginismus', MS='Recurrent or persistent involuntary SPASM of the outer muscles of the VAGINA, occurring during vaginal penetration.'),
  'D052066': NtO(MH='N-Ethylmaleimide-Sensitive Proteins', MS='ATPases that are members of the AAA protein superfamily (ATPase family Associated with Diverse cellular Activities). The NSFs functions, acting in conjunction with SOLUBLE NSF ATTACHMENT PROTEINS (i.e. SNAPs, which have no relation to SNAP 25), are to dissociate SNARE complexes.'),
  'D052067': NtO(MH='Soluble N-Ethylmaleimide-Sensitive Factor Attachment Proteins', MS='SNARE binding proteins that facilitate the ATP hydrolysis-driven dissociation of the SNARE complex. They are required for the binding of N-ETHYLMALEIMIDE-SENSITIVE PROTEIN (NSF) to the SNARE complex which also stimulates the ATPASE activity of NSF. They are unrelated structurally to SNAP-25 PROTEIN.'),
  'D052078': NtO(MH='Conus Snail', MS='A genus of cone-shaped marine snails in the family Conidae, class GASTROPODA. It comprises more than 600 species, many containing unique venoms (CONUS VENOMS) with which they immobilize their prey.'),
  'D052096': NtO(MH='Warm Ischemia', MS='A tissue or organ remaining at physiological temperature during decreased BLOOD perfusion or in the absence of blood supply. During ORGAN TRANSPLANTATION it begins when the organ reaches physiological temperature before the completion of SURGICAL ANASTOMOSIS and ends with reestablishment of the BLOOD CIRCULATION through the tissue.'),
  'D052097': NtO(MH='Quadriceps Muscle', MS='The quadriceps femoris. A collective name of the four-headed skeletal muscle of the thigh, comprised of the rectus femoris, vastus intermedius, vastus lateralis, and vastus medialis.'),
  'D052116': NtO(MH='Sialomucins', MS='A subcategory of mucins that contain SIALIC ACID.'),
  'D052117': NtO(MH='Benzodioxoles', MS='Compounds based on benzene fused to oxole. They can be formed from methylated CATECHOLS such as EUGENOL.'),
  'D052118': NtO(MH='Lysosomal-Associated Membrane Protein 1', MS='An abundant lysosomal-associated membrane protein that has been found to shuttle between LYSOSOMES; ENDOSOMES; and the PLASMA MEMBRANE. In PLATELETS and T-LYMPHOCYTES it may play a role in the cellular degranulation process.'),
  'D052119': NtO(MH='Lysosomal-Associated Membrane Protein 2', MS='An abundant lysosomal-associated membrane protein that has been found to shuttle between LYSOSOMES; ENDOSOMES; and the PLASMA MEMBRANE. Loss of expression of lysosomal-associated membrane protein 2 is associated with GLYCOGEN STORAGE DISEASE TYPE IIB.'),
  'D052120': NtO(MH='Glycogen Storage Disease Type IIb', MS='An X-linked dominant multisystem disorder resulting in cardiomyopathy, myopathy and INTELLECTUAL DISABILITY. It is caused by mutation in the gene encoding LYSOSOMAL-ASSOCIATED MEMBRANE PROTEIN 2.'),
  'D052138': NtO(MH='Genes, Neoplasm', MS='Genes whose abnormal expression, or MUTATION are associated with the development, growth, or progression of NEOPLASMS.'),
  'D052139': NtO(MH='Intracellular Calcium-Sensing Proteins', MS='Intracellular signaling peptides and proteins that bind to CALCIUM. They undergo allosteric changes when bound to CALCIUM that affects their interaction with other signal-transducing molecules. They differ from CALCIUM-SENSING RECEPTORS which sense extracellular calcium levels.'),
  'D052140': NtO(MH='Human papillomavirus 11', MS='A type of ALPHAPAPILLOMAVIRUS causing recurrent respiratory PAPILLOMATOSIS; GENITAL WARTS; and other neoplasms.'),
  'D052156': NtO(MH='SELEX Aptamer Technique', MS='A method of generating a large library of randomized nucleotides and selecting NUCLEOTIDE APTAMERS by iterative rounds of in vitro selection. A modified procedure substitutes AMINO ACIDS in place of NUCLEOTIDES to make PEPTIDE APTAMERS.'),
  'D052157': NtO(MH='Aptamers, Nucleotide', MS='Nucleotide sequences, generated by iterative rounds of SELEX APTAMER TECHNIQUE, that bind to a target molecule specifically and with high affinity.'),
  'D052158': NtO(MH='Aptamers, Peptide', MS='Peptide sequences, generated by iterative rounds of SELEX APTAMER TECHNIQUE, that bind to a target molecule specifically and with high affinity.'),
  'D052159': NtO(MH='Frasier Syndrome', MS='A syndrome characterized by CHRONIC KIDNEY FAILURE and GONADAL DYSGENESIS in phenotypic females with karyotype of 46,XY or female individual with a normal 46,XX karyotype. It is caused by donor splice-site mutations of Wilms tumor suppressor gene (GENES, WILMS TUMOR) on chromosome 11.'),
  'D052160': NtO(MH='Benzothiazoles', MS='Compounds with a benzene ring fused to a thiazole ring.'),
  'D052161': NtO(MH='Human papillomavirus 18', MS='A type of human papillomavirus especially associated with malignant tumors of the genital and RESPIRATORY MUCOSA.'),
  'D052162': NtO(MH='Human papillomavirus 16', MS='A type of ALPHAPAPILLOMAVIRUS especially associated with malignant tumors of the CERVIX and the RESPIRATORY MUCOSA.'),
  'D052163': NtO(MH='Human papillomavirus 6', MS='A type of ALPHAPAPILLOMAVIRUS usually associated with GENITAL WARTS; and LARYNGEAL NEOPLASMS.'),
  'D052176': NtO(MH='Azulenes', MS='Compounds based on a seven-membered ring fused to a five-membered ring. Heat can rearrange them to NAPHTHALENES which have two fused six-membered rings. They are similar to guaiazulenes which are SESQUITERPENES with a six-membered ring fused to a five-membered ring.'),
  'D052177': NtO(MH='Kidney Diseases, Cystic', MS='A heterogeneous group of hereditary and acquired disorders in which the KIDNEY contains one or more CYSTS unilaterally or bilaterally (KIDNEY, CYSTIC).'),
  'D052178': NtO(MH='Cephalopelvic Disproportion', MS="A condition in which the HEAD of the FETUS is larger than the mother's PELVIS through which the fetal head must pass during a vaginal delivery."),
  'D052179': NtO(MH='Antigens, Plant', MS='Substances found in PLANTS that have antigenic activity.'),
  'D052180': NtO(MH='Publication Formats', MS='Specific genre of publication.'),
  'D052181': NtO(MH='Publication Components', MS='Specific parts of publications.'),
  'D052182': NtO(MH='Study Characteristics', MS='Type of empirical method used.'),
  'D052196': NtO(MH='Lactobacillus brevis', MS='A species of gram-positive, rod-shaped LACTIC ACID bacteria that is frequently used as starter culture in SILAGE fermentation, sourdough, and lactic-acid-fermented types of beer and wine.'),
  'D052197': NtO(MH='Lactobacillus fermentum', MS='A species of gram-positive, rod-shaped bacteria associated with DENTAL CARIES.'),
  'D052198': NtO(MH='Lactobacillus helveticus', MS='A species of gram-positive bacteria isolated from MILK and cheese-starter cultures.'),
  'D052199': NtO(MH='Lactobacillus leichmannii', MS='A species of gram-negative bacteria isolated from MILK, cheese, and compressed yeast.'),
  'D052200': NtO(MH='Lactobacillus reuteri', MS='A species of gram-positive, rod-shaped LACTIC ACID bacteria found naturally in the human intestinal flora and BREAST MILK.'),
  'D052201': NtO(MH='Lactobacillus rhamnosus', MS='A species of gram-positive, rod-shaped bacteria used in PROBIOTICS.'),
  'D052202': NtO(MH='Hydrocolpos', MS='A fluid-filled VAGINA that is obstructed.'),
  'D052203': NtO(MH='Ferrosoferric Oxide', MS='Iron (II,III) oxide (Fe3O4). It is a black ore of IRON that forms opaque crystals and exerts strong magnetism.'),
  'D052216': NtO(MH='Glucagon-Like Peptide 1', MS='A peptide of 36 or 37 amino acids that is derived from PROGLUCAGON and mainly produced by the INTESTINAL L CELLS. GLP-1(1-37 or 1-36) is further N-terminally truncated resulting in GLP-1(7-37) or GLP-1-(7-36) which can be amidated. These GLP-1 peptides are known to enhance glucose-dependent INSULIN release, suppress GLUCAGON release and gastric emptying, lower BLOOD GLUCOSE, and reduce food intake.'),
  'D052217': NtO(MH='Fanconi Anemia Complementation Group A Protein', MS='A Fanconi anemia complementation group protein that is the most commonly mutated protein in FANCONI ANEMIA. It undergoes PHOSPHORYLATION by PROTEIN KINASE B and forms a complex with FANCC PROTEIN in the CELL NUCLEUS.'),
  'D052218': NtO(MH='Fanconi Anemia Complementation Group C Protein', MS='A Fanconi anemia complementation group protein that regulates the activities of CYTOCHROME P450 REDUCTASE and GLUTATHIONE S-TRANSFERASE. It is found predominately in the CYTOPLASM, but moves to the CELL NUCLEUS in response to FANCE PROTEIN.'),
  'D052219': NtO(MH='Fanconi Anemia Complementation Group F Protein', MS='A Fanconi anemia complementation group protein. It is an essential component of a nuclear core complex that protects the GENOME against CHROMOSOMAL INSTABILITY. It interacts directly with FANCG PROTEIN and helps stabilize a complex with FANCA PROTEIN and FANCC PROTEIN.'),
  'D052236': NtO(MH='Fanconi Anemia Complementation Group D2 Protein', MS='A Fanconi anemia complementation group protein that undergoes mono-ubiquitination by FANCL PROTEIN in response to DNA DAMAGE. Also, in response to IONIZING RADIATION it can undergo PHOSPHORYLATION by ataxia telangiectasia mutated protein. Modified FANCD2 interacts with BRCA2 PROTEIN in a stable complex with CHROMATIN, and it is involved in DNA REPAIR by homologous RECOMBINATION.'),
  'D052239': NtO(MH='Fanconi Anemia Complementation Group E Protein', MS='A Fanconi anemia complementation group protein that interacts with FANCC PROTEIN and FANCD2 PROTEIN. It promotes the accumulation of FANCC protein in the CELL NUCLEUS.'),
  'D052240': NtO(MH='Fanconi Anemia Complementation Group L Protein', MS='An E3 UBIQUITIN LIGASE that plays a key role in the DNA damage response pathway of FANCONI ANEMIA PROTEINS. It is associated with mono-ubiquitination of FANCD2 PROTEIN and the redistribution of FANCD2 to nuclear foci containing BRCA1 PROTEIN.'),
  'D052241': NtO(MH='Fanconi Anemia Complementation Group G Protein', MS='A Fanconi anemia complementation group protein that undergoes PHOSPHORYLATION by CDC2 PROTEIN KINASE during MITOSIS. It forms a complex with other FANCONI ANEMIA PROTEINS and helps protect CELLS from DNA DAMAGE by genotoxic agents.'),
  'D052242': NtO(MH='Adiponectin', MS='A 30-kDa COMPLEMENT C1Q-related protein, the most abundant gene product secreted by FAT CELLS of the white ADIPOSE TISSUE. Adiponectin modulates several physiological processes, such as metabolism of GLUCOSE and FATTY ACIDS, and immune responses. Decreased plasma adiponectin levels are associated with INSULIN RESISTANCE; TYPE 2 DIABETES MELLITUS; OBESITY; and ATHEROSCLEROSIS.'),
  'D052243': NtO(MH='Resistin', MS='A 12-kDa cysteine-rich polypeptide hormone secreted by FAT CELLS in the ADIPOSE TISSUE. It is the founding member of the resistin-like molecule (RELM) hormone family. Resistin suppresses the ability of INSULIN to stimulate cellular GLUCOSE uptake.'),
  'D052244': NtO(MH='Endocrine Disruptors', MS='Exogenous agents, synthetic and naturally occurring, which are capable of disrupting the functions of the ENDOCRINE SYSTEM including the maintenance of HOMEOSTASIS and the regulation of developmental processes. Endocrine disruptors are compounds that can mimic HORMONES, or enhance or block the binding of hormones to their receptors, or otherwise lead to activating or inhibiting the endocrine signaling pathways and hormone metabolism.'),
  'D052245': NtO(MH='Usher Syndromes', MS='Autosomal recessive hereditary disorders characterized by congenital SENSORINEURAL HEARING LOSS and RETINITIS PIGMENTOSA. Genetically and symptomatically heterogeneous, clinical classes include type I, type II, and type III. Their severity, age of onset of retinitis pigmentosa and the degree of vestibular dysfunction are variable.'),
  'D052246': NtO(MH='Cyclooxygenase 2 Inhibitors', MS='A subclass of cyclooxygenase inhibitors with specificity for CYCLOOXYGENASE-2.'),
  'D052247': NtO(MH='Nitric Oxide Synthase Type II', MS='A CALCIUM-independent subtype of nitric oxide synthase that may play a role in immune function. It is an inducible enzyme whose expression is transcriptionally regulated by a variety of CYTOKINES.'),
  'D052248': NtO(MH='Nitric Oxide Synthase Type I', MS='A CALCIUM-dependent, constitutively-expressed form of nitric oxide synthase found primarily in NERVE TISSUE.'),
  'D052250': NtO(MH='Nitric Oxide Synthase Type III', MS='A CALCIUM-dependent, constitutively-expressed form of nitric oxide synthase found primarily in ENDOTHELIAL CELLS.'),
  'D052256': NtO(MH='Tendinopathy', MS='Clinical syndrome describing overuse tendon injuries characterized by a combination of PAIN, diffuse or localized swelling, and impaired performance.'),
  'D052276': NtO(MH='Creatine Kinase, BB Form', MS='A form of creatine kinase found in the BRAIN.'),
  'D052279': NtO(MH='Creatine Kinase, MB Form', MS='An isoenzyme of creatine kinase found in the CARDIAC MUSCLE.'),
  'D052280': NtO(MH='Orthodontic Anchorage Procedures', MS='Attachment of orthodontic devices and materials to the MOUTH area for support and to provide a counterforce to orthodontic forces.'),
  'D052281': NtO(MH='Creatine Kinase, Mitochondrial Form', MS='A form of creatine kinase found in the MITOCHONDRIA.'),
  'D052283': NtO(MH='Creatine Kinase, MM Form', MS='An isoenzyme of creatine kinase found in the MUSCLE.'),
  'D052284': NtO(MH='Clutch Size', MS='The number of offspring produced at one birth by an oviparous or ovoviviparous animal.'),
  'D052285': NtO(MH='Ovoviviparity', MS='The capability of producing yolk-filled eggs that hatch within the body, so that the young are born alive, but without placental attachment. This applies to certain species of REPTILES and FISHES.'),
  'D052286': NtO(MH='Viviparity, Nonmammalian', MS='The capability of bearing live young (rather than eggs) in nonmammalian species. Some species of REPTILES and FISHES exhibit this.'),
  'D052287': NtO(MH='Oviparity', MS='The capability of producing eggs (OVA) from which young are hatched outside the body. While mostly referring to nonmammalian species, this does include MAMMALS of the order MONOTREMATA.'),
  'D052288': NtO(MH='Support of Research', MS='Organizational source for funding of research activity.'),
  'D052336': NtO(MH='Proglucagon', MS='The common precursor polypeptide of pancreatic GLUCAGON and intestinal GLUCAGON-LIKE PEPTIDES. Proglucagon is the 158-amino acid segment of preproglucagon without the N-terminal signal sequence. Proglucagon is expressed in the PANCREAS; INTESTINES; and the CENTRAL NERVOUS SYSTEM. Posttranslational processing of proglucagon is tissue-specific yielding numerous bioactive peptides.'),
  'D052416': NtO(MH='Targeted Gene Repair', MS="A technique which uses synthetic oligonucleotides to direct the cell's inherent DNA repair system to correct a mutation at a specific site in an episome or chromosome."),
  'D052436': NtO(MH='Adipose Tissue, White', MS='Fatty tissue composed of WHITE ADIPOCYTES and generally found directly under the skin (SUBCUTANEOUS FAT) and around the internal organs (ABDOMINAL FAT). It has less vascularization and less coloration than the BROWN FAT. White fat provides heat insulation, mechanical cushion, and source of energy.'),
  'D052437': NtO(MH='Adipocytes, Brown', MS='Fat cells with dark coloration due to the densely packed MITOCHONDRIA. They contain numerous small lipid droplets or vacuoles. Their stored lipids can be converted directly to energy as heat by the mitochondria.'),
  'D052438': NtO(MH='Adipocytes, White', MS='Fat cells with light coloration and few MITOCHONDRIA. They contain a scant ring of CYTOPLASM surrounding a single large lipid droplet or vacuole.'),
  'D052439': NtO(MH='Lipid Metabolism Disorders', MS='Pathological conditions resulting from abnormal anabolism or catabolism of lipids in the body.'),
  'D052456': NtO(MH='Hypoalphalipoproteinemias', MS='Conditions with abnormally low levels of ALPHA-LIPOPROTEINS (high-density lipoproteins) in the blood. Hypoalphalipoproteinemia can be associated with mutations in genes encoding APOLIPOPROTEIN A-I; LECITHIN CHOLESTEROL ACYLTRANSFERASE; and ATP-BINDING CASSETTE TRANSPORTERS.'),
  'D052476': NtO(MH='Hypobetalipoproteinemia, Familial, Apolipoprotein B', MS='An autosomal dominant disorder of lipid metabolism. It is caused by mutations of APOLIPOPROTEINS B, main components of CHYLOMICRONS and BETA-LIPOPROTEINS (low density lipoproteins or LDL). Features include abnormally low LDL, normal triglyceride level, and dietary fat malabsorption.'),
  'D052496': NtO(MH='Lipodystrophy, Familial Partial', MS='Inherited conditions characterized by the partial loss of ADIPOSE TISSUE, either confined to the extremities with normal or increased fat deposits on the face, neck and trunk (type 1), or confined to the loss of SUBCUTANEOUS FAT from the limbs and trunk (type 2). Type 3 is associated with mutation in the gene encoding PEROXISOME PROLIFERATOR-ACTIVATED RECEPTOR GAMMA.'),
  'D052497': NtO(MH='Lipodystrophy, Congenital Generalized', MS='Congenital disorders, usually autosomal recessive, characterized by severe generalized lack of ADIPOSE TISSUE, extreme INSULIN RESISTANCE, and HYPERTRIGLYCERIDEMIA.'),
  'D052516': NtO(MH='Sulfatidosis', MS='A group of inherited metabolic disorders characterized by the intralysosomal accumulation of sulfur-containing lipids (SULFATIDES), including SULFOGLYCOSPHINGOLIPIDS normally found in the MYELIN SHEATH of the brain. These disorders are caused by defective degradative enzymes leading to substrate accumulation (or storage).'),
  'D052517': NtO(MH='Multiple Sulfatase Deficiency Disease', MS='An inherited metabolic disorder characterized by the intralysosomal accumulation of sulfur-containing lipids (sulfatides) and MUCOPOLYSACCHARIDES. Excess levels of both substrates are present in urine. This is a disorder of multiple sulfatase (arylsulfatases A, B, and C) deficiency which is caused by the mutation of sulfatase-modifying factor-1. Neurological deterioration is rapid.'),
  'D052536': NtO(MH='Niemann-Pick Disease, Type A', MS='The classic infantile form of Niemann-Pick Disease, caused by mutation in SPHINGOMYELIN PHOSPHODIESTERASE. It is characterized by accumulation of SPHINGOMYELINS in the cells of the MONONUCLEAR PHAGOCYTE SYSTEM and other cell throughout the body leading to cell death. Clinical signs include JAUNDICE, hepatosplenomegaly, and severe brain damage.'),
  'D052537': NtO(MH='Niemann-Pick Disease, Type B', MS='An allelic disorder of TYPE A NIEMANN-PICK DISEASE, a late-onset form. It is also caused by mutation in SPHINGOMYELIN PHOSPHODIESTERASE but clinical signs involve only visceral organs (non-neuropathic type).'),
  'D052556': NtO(MH='Niemann-Pick Disease, Type C', MS='An autosomal recessive lipid storage disorder that is characterized by accumulation of CHOLESTEROL and SPHINGOMYELINS in cells of the VISCERA and the CENTRAL NERVOUS SYSTEM. Type C (or C1) and type D are allelic disorders caused by mutation of the NPC1 gene, which encodes  a protein that mediates intracellular cholesterol transport from LYSOSOMES. Clinical signs include hepatosplenomegaly and chronic neurological symptoms. Type D is a variant in people with a Nova Scotia ancestry.'),
  'D052557': NtO(MH='Clostridium septicum', MS='A species of gram-positive bacteria in the family Clostridiaceae. Infections have a strong association with malignancies and also with GAS GANGRENE.'),
  'D052576': NtO(MH='Menstrual Hygiene Products', MS='Personal care items used during MENSTRUATION.'),
  'D052577': NtO(MH='Infant, Extremely Low Birth Weight', MS='An infant whose weight at birth is less than 1000 grams (2.2 lbs), regardless of GESTATIONAL AGE.'),
  'D052578': NtO(MH='Ionic Liquids', MS='Salts that melt below 100 C. Their low VOLATILIZATION can be an advantage over volatile organic solvents.'),
  'D052580': NtO(MH='Muscle Stretching Exercises', MS='Exercises that stretch the muscle fibers with the aim to increase muscle-tendon FLEXIBILITY, improve RANGE OF MOTION or musculoskeletal function, and prevent injuries. There are various types of stretching techniques including active, passive (relaxed), static, dynamic (gentle), ballistic (forced), isometric, and others.'),
  'D052581': NtO(MH='Pinch Strength', MS='Force exerted when using the index finger and the thumb. It is a test for determining maximum voluntary contraction force.'),
  'D052582': NtO(MH='Trigger Finger Disorder', MS="A painful disability in the hand affecting the finger or thumb. It is caused by mechanical impingement of the digital flexor tendons as they pass through a narrowed retinacular pulley at the level of the metacarpal head. Thickening of the sheath and fibrocartilaginous metaplasia can occur, and nodules can form. (From Green's Operative Hand Surgery, 5th ed, p2137-58)."),
  'D052583': NtO(MH='Sarraceniaceae', MS='A plant family of the order Nepenthales.'),
  'D052584': NtO(MH='Xylem', MS='Plant tissue that carries water up the root and stem. Xylem cell walls derive most of their strength from LIGNIN. The vessels are similar to PHLOEM sieve tubes but lack companion cells and do not have perforated sides and pores.'),
  'D052585': NtO(MH='Phloem', MS='Plant tissue that carries nutrients, especially sucrose, by turgor pressure. Movement is bidirectional, in contrast to XYLEM where it is only upward. Phloem originates and grows outwards from meristematic cells (MERISTEM) in the vascular cambium. P-proteins, a type of LECTINS, are characteristically found in phloem.'),
  'D052586': NtO(MH='Mating Preference, Animal', MS='The selection or choice of sexual partner in animals. Often this reproductive preference is based on traits in the potential mate, such as coloration, size, or behavioral boldness. If the chosen ones are genetically different from the rejected ones, then NATURAL SELECTION is occurring.'),
  'D052587': NtO(MH='Pulicaria', MS='A plant genus of the family ASTERACEAE. Members contain pulicanadienes and other cytotoxic SESQUITERPENES.'),
  'D052596': NtO(MH='Pongamia', MS='A plant genus of the family FABACEAE. Members contain karanjin. Some species of this genus have been reclassified to other genera of FABACEAE including Callerya, DERRIS and MILLETTIA.'),
  'D052597': NtO(MH='Hymenaea', MS='A plant genus of the family FABACEAE. Members contain resins (RESINS, PLANT) and GLUCANS.'),
  'D052598': NtO(MH='Dioclea', MS='A plant genus of the family FABACEAE. Members contain MANNOSE-BINDING LECTINS and dioclein.'),
  'D052616': NtO(MH='Solid Phase Extraction', MS='An extraction method that separates analytes using a solid phase and a liquid phase. It is used for preparative sample cleanup before analysis by CHROMATOGRAPHY and other analytical methods.'),
  'D052617': NtO(MH='Solid Phase Microextraction', MS='A solventless sample preparation method, invented in 1989, that uses a fused silica fiber which is coated with a stationary phase. It is used for sample cleanup before using other analytical methods.'),
  'D052636': NtO(MH='Allostasis', MS='Biological adaptation, such as the rise of EPINEPHRINE in response to exercise, stress or perceived danger, followed by a fall of epinephrine during RELAXATION. Allostasis is the achievement of stability by turning on and turning off the allostatic systems including the IMMUNE SYSTEM; the AUTONOMIC NERVOUS SYSTEM and NEUROENDOCRINE SYSTEMS.'),
  'D052638': NtO(MH='Particulate Matter', MS='Particles of any solid substance, generally under 30 microns in size, often noted as PM30. There is special concern with PM1 which can get down to PULMONARY ALVEOLI and induce MACROPHAGE ACTIVATION and PHAGOCYTOSIS leading to FOREIGN BODY REACTION and LUNG DISEASES.'),
  'D052639': NtO(MH='Caulimoviridae', MS='A family of DNA plant viruses with isometric or bacilliform virions and no envelope. The host ranges of most species are narrow. There are six genera: CAULIMOVIRUS; BADNAVIRUS; Cavemovirus; Soymovirus; Petuvirus; and TUNGROVIRUS.'),
  'D052641': NtO(MH='Tungrovirus', MS='A genus of DNA plant viruses of the family CAULIMOVIRIDAE. The sole species, Rice tungro bacilliform virus, shows 25% identity with BADNAVIRUS. It is transmitted by leafhoppers with the assistance of RICE TUNGRO SPHERICAL VIRUS.'),
  'D052656': NtO(MH='Ictalurivirus', MS='An unassigned genus in the family HERPESVIRIDAE, comprising one species Ictalurid herpesvirus 1 (Channel Catfish Virus).'),
  'D052657': NtO(MH='Nanoviridae', MS='A family of DNA viruses infecting plants and transmitted by APHIDS. Genera include NANOVIRUS and BABUVIRUS.'),
  'D052658': NtO(MH='Babuvirus', MS='A genus in the family NANOVIRIDAE infecting bananas. The type species is Banana bunchy top virus.'),
  'D052659': NtO(MH='Amdovirus', MS='A genus in the subfamily PARVOVIRINAE consisting of a single species ALEUTIAN MINK DISEASE VIRUS.'),
  'D052660': NtO(MH='Bocavirus', MS='A genus in the subfamily PARVOVIRINAE comprising three species: Bovine parvovirus, Canine minute virus, and HUMAN BOCAVIRUS.'),
  'D052661': NtO(MH='H-1 parvovirus', MS='A species in the genus PARVOVIRUS that has the ability to replicate and kill tumor cells in culture.'),
  'D052676': NtO(MH='Avastrovirus', MS='A genus of ASTROVIRIDAE infecting avian species and often involving extra-intestinal manifestations. The type species is Turkey astrovirus.'),
  'D052677': NtO(MH='Flexiviridae', MS='A family of RNA plant viruses that infect a wide range of herbaceous and woody plant species. There are at least eight genera including POTEXVIRUS and CARLAVIRUS, both of which are highly immunogenic.'),
  'D052678': NtO(MH='Hepevirus', MS='An unassigned genus of RNA viruses with a single officially described species, HEPATITIS E VIRUS. A distantly related virus, Avian hepatitis E virus, has been listed as a tentative species. Strains have also been identified in swine. The family name hepeviridae has been proposed.'),
  'D052679': NtO(MH='Luteoviridae', MS='A family of RNA plant viruses infecting disparate plant families. They are transmitted by specific aphid vectors. There are three genera: LUTEOVIRUS; Polerovirus; and Enamovirus.'),
  'D052680': NtO(MH='Corticotrophs', MS='Anterior pituitary cells that produce ADRENOCORTICOTROPHIC HORMONE.'),
  'D052681': NtO(MH='Gonadotrophs', MS='Anterior pituitary cells that can produce both FOLLICLE STIMULATING HORMONE and LUTEINIZING HORMONE.'),
  'D052682': NtO(MH='Lactotrophs', MS='Anterior pituitary cells that produce PROLACTIN.'),
  'D052683': NtO(MH='Somatotrophs', MS='Anterior pituitary cells which produce GROWTH HORMONE.'),
  'D052684': NtO(MH='Thyrotrophs', MS='Anterior pituitary cells that produce THYROID-STIMULATING HORMONE.'),
  'D052685': NtO(MH='Alphapapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE. They preferentially infect the anogenital and ORAL MUCOSA in humans and primates, causing both malignant and benign neoplasms. Cutaneous lesions are also seen.'),
  'D052686': NtO(MH='Betapapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE, causing cutaneous lesions in humans. Infections exist in latent form in the general population and are activated under conditions of IMMUNOSUPPRESSION.'),
  'D052687': NtO(MH='Deltapapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE causing fibropapillomas in their respective ungulate hosts. Species infected include cattle, European elk, deer, and sheep.'),
  'D052688': NtO(MH='Bovine papillomavirus 1', MS='A species of DELTAPAPILLOMAVIRUS infecting cattle.'),
  'D052689': NtO(MH='Gammapapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE, which cause cutaneous lesions in humans. They are histologically distinguishable by intracytoplasmic INCLUSION BODIES which are species specific.'),
  'D052690': NtO(MH='Kappapapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE, causing cutaneous and mucosal lesions in rabbits. COTTONTAIL RABBIT PAPILLOMAVIRUS is the type species.'),
  'D052691': NtO(MH='Lambdapapillomavirus', MS='A genus of DNA viruses in the family papillomaviridae, causing mucosal and cutaneous lesions in cats and dogs. Canine oral papillomavirus is the type species.'),
  'D052692': NtO(MH='Mupapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE, causing cutaneous lesions in humans.'),
  'D052696': NtO(MH='Xipapillomavirus', MS='A genus of DNA viruses in the family PAPILLOMAVIRIDAE, causing cutaneous or mucosal papillomas in cattle.'),
  'D052697': NtO(MH='Bovine papillomavirus 4', MS='A type of XIPAPILLOMAVIRUS causing alimentary carcinoma in cattle. It is related to Bovine papillomavirus 3.'),
  'D052698': NtO(MH='Erbovirus', MS='A genus in the family PICORNAVIRIDAE causing upper respiratory disease in horses.'),
  'D052699': NtO(MH='Kobuvirus', MS='A genus in the family PICORNAVIRIDAE whose type species Aichi virus, causes gastroenteritis in humans.'),
  'D052700': NtO(MH='Teschovirus', MS='A genus in the family PICORNAVIRIDAE that can cause polioencephalomyelitis in pigs. The type species Porcine teschovirus is comprised of multiple strains.'),
  'D052716': NtO(MH='Pituitary Gland, Intermediate', MS='The intermediate lobe of the pituitary gland. It shows considerable size variation among the species, small in humans, and large in amphibians and lower vertebrates. This lobe produces mainly MELANOCYTE-STIMULATING HORMONES and other peptides from post-translational processing of pro-opiomelanocortin (POMC).'),
  'D052717': NtO(MH='Melanotrophs', MS='Neuroendocrine cells in the INTERMEDIATE LOBE OF PITUITARY. They produce MELANOCYTE STIMULATING HORMONES and other peptides from the post-translational processing of pro-opiomelanocortin (POMC).'),
  'D052736': NtO(MH='Rous sarcoma virus', MS='A species of replication-competent oncogene-containing virus in the genus ALPHARETROVIRUS. It is the original source of the src oncogene (V-SRC GENES) and causes sarcoma in chickens.'),
  'D052737': NtO(MH='Carpometacarpal Joints', MS='The articulations between the CARPAL BONES and the METACARPAL BONES.'),
  'D052756': NtO(MH='Nutritional Sciences', MS='The study of NUTRITION PROCESSES as well as the components of food, their actions, interaction, and balance in relation to health and disease.'),
  'D052776': NtO(MH='Female Urogenital Diseases', MS='Pathological processes of the female URINARY TRACT and the reproductive system (GENITALIA, FEMALE).'),
  'D052798': NtO(MH='Balanitis Xerotica Obliterans', MS='An atrophic and sclerotic condition of the head of the PENIS, glans penis. Sometimes it leads to stenosis and occasionally obliteration of the external meatal orifice.'),
  'D052801': NtO(MH='Male Urogenital Diseases', MS='Pathological processes of the male URINARY TRACT and the reproductive system (GENITALIA, MALE).'),
  'D052816': NtO(MH='Foreskin', MS='The double-layered skin fold that covers the GLANS PENIS, the head of the penis.'),
  'D052817': NtO(MH='Phototrophic Processes', MS='Processes by which phototrophic organisms use sunlight as their primary energy source. Contrasts with chemotrophic processes which do not depend on light and function in deriving energy from exogenous chemical sources. Photoautotrophy (or photolithotrophy) is the ability to use sunlight as energy to fix inorganic nutrients to be used for other organic requirements. Photoautotrophs include all GREEN PLANTS; GREEN ALGAE; CYANOBACTERIA; and green and PURPLE SULFUR BACTERIA. Photoheterotrophs or photoorganotrophs require a supply of organic nutrients for their organic requirements but use sunlight as their primary energy source; examples include certain PURPLE NONSULFUR BACTERIA. Depending on environmental conditions some organisms can switch between different nutritional modes (AUTOTROPHY; HETEROTROPHY; chemotrophy; or phototrophy) to utilize different sources to meet their nutrients and energy requirements.'),
  'D052818': NtO(MH='Autotrophic Processes', MS='The processes by which organisms use simple inorganic substances such as gaseous or dissolved carbon dioxide and inorganic nitrogen as nutrient sources. Contrasts with heterotrophic processes which make use of organic materials as the nutrient supply source. Autotrophs can be either chemoautotrophs (or chemolithotrophs), largely ARCHAEA and BACTERIA, which also use simple inorganic substances for their metabolic energy reguirements; or photoautotrophs (or photolithotrophs), such as PLANTS and CYANOBACTERIA, which derive their energy from light. Depending on environmental conditions some organisms can switch between different nutritional modes (autotrophy; HETEROTROPHY; chemotrophy; or PHOTOTROPHY) to utilize different sources to meet their nutrient and energy requirements.'),
  'D052836': NtO(MH='Heterotrophic Processes', MS='The processes by which organisms utilize organic substances as their nutrient sources. Contrasts with AUTOTROPHIC PROCESSES which make use of simple inorganic substances as the nutrient supply source. Heterotrophs can be either chemoheterotrophs (or chemoorganotrophs) which also require organic substances such as glucose for their primary metabolic energy requirements, or photoheterotrophs (or photoorganotrophs) which derive their primary energy requirements from light. Depending on environmental conditions some organisms can switch between different nutritional modes (AUTOTROPHY; heterotrophy; chemotrophy; or PHOTOTROPHY) to utilize different sources to meet their nutrients and energy requirements.'),
  'D052837': NtO(MH='Chemoautotrophic Growth', MS='Growth of organisms using AUTOTROPHIC PROCESSES for obtaining nutrients and chemotrophic processes for obtaining a primary energy supply. Chemotrophic processes are involved in deriving a primary energy supply from exogenous chemical sources. Chemotrophic autotrophs (chemoautotrophs) generally use inorganic chemicals as energy sources and as such are called chemolithoautotrophs. Most chemoautotrophs live in hostile environments, such as deep sea vents. They are mostly BACTERIA and ARCHAEA, and are the primary producers for those ecosystems.'),
  'D052858': NtO(MH='Cystocele', MS='A HERNIA-like condition in which the weakened pelvic muscles cause the URINARY BLADDER to drop from its normal position. Fallen urinary bladder is more common in females with the bladder dropping into the VAGINA and less common in males with the bladder dropping into the SCROTUM.'),
  'D052859': NtO(MH='Houttuynia', MS='A plant genus of the family SAURURACEAE. Members contain aristolactams.'),
  'D052878': NtO(MH='Urolithiasis', MS='Formation of stones in any part of the URINARY TRACT, usually in the KIDNEY; URINARY BLADDER; or the URETER.'),
  'D052879': NtO(MH='Subacute Combined Degeneration', MS='A neuropathy due to VITAMIN B 12 DEFICIENCY or to excessive NITROUS OXIDE inhalation. It is associated with overproduction of the myelinolytic TUMOR NECROSIS FACTOR-ALPHA.'),
  'D052880': NtO(MH='Pyomyositis', MS='An intramuscular suppuration of the large skeletal muscle groups. It is associated with INFECTION such as STAPHYLOCOCCUS AUREUS and PYODERMA. It was known as a tropical disease but is increasing among the immunocompromised (IMMUNOCOMPROMISED HOST). Symptoms include muscle pain, FEVER, and leucocytosis. It has been diagnosed by MRI SCANS.'),
  'D052898': NtO(MH='Optical Tweezers', MS='A technique that uses LASERS to trap, image, and manipulate small objects (biomolecules, supramolecular assembles, DENDRIMERS) in three dimensional space. (From Glossary of Biotechnology and Nanobiotechnology Terms, 4th ed.)'),
  'D052899': NtO(MH='Pore Forming Cytotoxic Proteins', MS='Proteins secreted from an organism which form membrane-spanning pores in target cells to destroy them. This is in contrast to PORINS and MEMBRANE TRANSPORT PROTEINS that function within the synthesizing organism and COMPLEMENT immune proteins. These pore forming cytotoxic proteins are a form of primitive cellular defense which are also found in human LYMPHOCYTES.'),
  'D052918': NtO(MH='Environmental Restoration and Remediation', MS='Restoration of an environment, ecosystem, or habitat that was physically damaged during land development or by natural disaster, or contaminated by ENVIRONMENTAL POLLUTANTS. This is accomplished by various chemical, biological, and bulk movement methods, in conjunction with ENVIRONMENTAL MONITORING.'),
  'D052919': NtO(MH='Refsum Disease, Infantile', MS='An early onset form of phytanic acid storage disease with clinical and biochemical signs different from those of REFSUM DISEASE. Features include MENTAL RETARDATION; SENSORINEURAL HEARING LOSS; OSTEOPOROSIS; and severe liver damage. It can be caused by mutation in a number of genes encoding proteins involving in the biogenesis or assembly of PEROXISOMES.'),
  'D052938': NtO(MH='Bariatric Medicine', MS='The discipline concerned with WEIGHT REDUCTION in patients with OBESITY.'),
  'D052939': NtO(MH='Schizonts', MS='Multinucleate cells or a stage in the development of sporozoan protozoa. It is exemplified by the life cycle of PLASMODIUM FALCIPARUM in the MALARIA infection cycle.'),
  'D052940': NtO(MH='Merozoites', MS='Uninuclear cells or a stage in the life cycle of sporozoan protozoa. Merozoites, released from ruptured multinucleate SCHIZONTS, enter the blood stream and infect the ERYTHROCYTES.'),
  'D052958': NtO(MH='Tarlov Cysts', MS='Perineurial cysts commonly found in the SACRAL REGION. They arise from the PERINEURIUM membrane within the SPINAL NERVE ROOTS. The distinctive feature of the cysts is the presence of spinal nerve root fibers within the cyst wall, or the cyst cavity itself.'),
  'D052978': NtO(MH='Disk Diffusion Antimicrobial Tests', MS='A method where a culturing surface inoculated with microbe is exposed to small disks containing known amounts of a chemical agent resulting in a zone of inhibition (usually in millimeters) of growth of the microbe corresponding to the susceptibility of the strain to the agent.'),
  'D052979': NtO(MH='Phycobiliproteins', MS='Light harvesting proteins found in phycobilisomes.'),
  'D052980': NtO(MH='Phycobilins', MS='Open chain tetrapyrroles that function as light harvesting chromophores in PHYCOBILIPROTEINS.'),
  'D052981': NtO(MH='Metarhizium', MS='A mitosporic fungal genus in the family Clavicipitaceae. It has teleomorphs in the family Nectriaceae. Metarhizium anisopliae is used in PESTICIDES.'),
  'D052982': NtO(MH='Beauveria', MS='A mitosporic fungal genus. Teleomorphs are found in the family Clavicipitaceae and include Cordyceps bassiana. The species Beauveria bassiana is a common pathogen of ARTHROPODS and is used in PEST CONTROL.'),
  'D052998': NtO(MH='Microcystins', MS='Cyclic heptapeptides found in MICROCYSTIS and other CYANOBACTERIA. Hepatotoxic and carcinogenic effects have been noted. They are sometimes called cyanotoxins, which should not be confused with chemicals containing a cyano group (CN) which are toxic.'),
  'D052999': NtO(MH='Quinazolinones', MS='Chemicals with two conjoined aromatic rings incorporating two nitrogen atoms and one of the carbons oxidized with a keto oxygen.'),
  'D053000': NtO(MH='Analytic Sample Preparation Methods', MS='Use of various chemical separation and extraction methods, such as SOLID PHASE EXTRACTION; CHROMATOGRAPHY; and SUPERCRITICAL FLUID EXTRACTION; to prepare samples for analytical measurement of components.'),
  'D053001': NtO(MH='Tool Use Behavior', MS='Modifying, carrying, or manipulating an item external to itself by an animal, before using it to effect a change on the environment or itself (from Beck, Animal Tool Behavior, 1980).'),
  'D053018': NtO(MH='Pyonephrosis', MS='Distention of KIDNEY with the presence of PUS and suppurative destruction of the renal parenchyma. It is often associated with renal obstruction and can lead to total or nearly total loss of renal function.'),
  'D053038': NtO(MH='Quorum Sensing', MS='A phenomenon where microorganisms communicate and coordinate their behavior by the accumulation of signaling molecules. A reaction occurs when a substance accumulates to a sufficient concentration. This is most commonly seen in bacteria.'),
  'D053039': NtO(MH='Ureterolithiasis', MS='Formation of stones in the URETER.'),
  'D053040': NtO(MH='Nephrolithiasis', MS='Formation of stones in the KIDNEY.'),
  'D053058': NtO(MH='Trophozoites', MS='Cells or feeding stage in the life cycle of sporozoan protozoa. In the malarial parasite, the trophozoite develops from the MEROZOITE and then splits into the SCHIZONT. Trophozoites that are left over from cell division can go on to form gametocytes.'),
  'D053059': NtO(MH='Dengue Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with DENGUE VIRUS. These include live-attenuated, subunit, DNA, and inactivated vaccines.'),
  'D053060': NtO(MH='West Nile Virus Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with WEST NILE VIRUS.'),
  'D053061': NtO(MH='Herpes Zoster Vaccine', MS='An attenuated vaccine used to prevent and/or treat HERPES ZOSTER, a disease caused by HUMAN HERPESVIRUS 3.'),
  'D053078': NtO(MH='Membrane Potential, Mitochondrial', MS='The voltage difference, normally maintained at approximately -180mV, across the INNER MITOCHONDRIAL MEMBRANE, by a net movement of positive charge across the membrane. It is a major component of the PROTON MOTIVE FORCE in MITOCHONDRIA used to drive the synthesis of ATP.'),
  'D053079': NtO(MH='Trager duck spleen necrosis virus', MS='A species in the group RETICULOENDOTHELIOSIS VIRUSES, AVIAN of the genus GAMMARETROVIRUS originally isolated from ducks.'),
  'D053098': NtO(MH='Familial Hypophosphatemic Rickets', MS='A hereditary disorder characterized by HYPOPHOSPHATEMIA; RICKETS; OSTEOMALACIA; renal defects in phosphate reabsorption and vitamin D metabolism; and growth retardation. Autosomal and X-linked dominant and recessive variants have been reported.'),
  'D053099': NtO(MH='Azotemia', MS='A biochemical abnormality referring to an elevation of BLOOD UREA NITROGEN and CREATININE. Azotemia can be produced by KIDNEY DISEASES or other extrarenal disorders. When azotemia becomes associated with a constellation of clinical signs, it is termed UREMIA.'),
  'D053118': NtO(MH='Influenza A Virus, H1N1 Subtype', MS='A subtype of INFLUENZA A VIRUS with the surface proteins hemagglutinin 1 and neuraminidase 1. The H1N1 subtype was responsible for the Spanish flu pandemic of 1918.'),
  'D053119': NtO(MH='Benzophenanthridines', MS='Compounds of four rings containing a nitrogen. They are biosynthesized from reticuline via rearrangement of scoulerine. They are similar to BENZYLISOQUINOLINES. Members include chelerythrine and sanguinarine.'),
  'D053120': NtO(MH='Respiratory Aspiration', MS='Inhaling liquid or solids, such as stomach contents, into the RESPIRATORY TRACT. When this causes severe lung damage, it is called ASPIRATION PNEUMONIA.'),
  'D053121': NtO(MH='Influenza A Virus, H2N2 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 2 and neuraminidase 2. The H2N2 subtype was responsible for the Asian flu pandemic of 1957.'),
  'D053122': NtO(MH='Influenza A Virus, H3N2 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 3 and neuraminidase 2. The H3N2 subtype was responsible for the Hong Kong flu pandemic of 1968.'),
  'D053123': NtO(MH='Influenza A Virus, H3N8 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 3 and neuraminidase 8. The H3N8 subtype has frequently been found in horses.'),
  'D053124': NtO(MH='Influenza A Virus, H5N1 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 5 and neuraminidase 1. The H5N1 subtype, frequently referred to as the bird flu virus, is endemic in wild birds and very contagious among both domestic (POULTRY) and wild birds. It does not usually infect humans, but some cases have been reported.'),
  'D053125': NtO(MH='Influenza A Virus, H5N2 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 5 and neuraminidase 2. The H5N2 subtype has been found to be highly pathogenic in chickens.'),
  'D053126': NtO(MH='Influenza A Virus, H7N7 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 7 and neuraminidase 7. The H7N7 subtype produced an epidemic in 2003 which was highly pathogenic among domestic birds (POULTRY). Some infections in humans were reported.'),
  'D053127': NtO(MH='Influenza A Virus, H9N2 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 9 and neuraminidase 2. The H9N2 subtype usually infects domestic birds (POULTRY) but there have been some human infections reported.'),
  'D053138': NtO(MH='Cyclohexenes', MS='Six-carbon alicyclic hydrocarbons which contain one or more double bonds in the ring. The cyclohexadienes are not aromatic, in contrast to BENZOQUINONES which are sometimes called 2,5-cyclohexadiene-1,4-diones.'),
  'D053139': NtO(MH='Oseltamivir', MS='An acetamido cyclohexene that is a structural homolog of SIALIC ACID and inhibits NEURAMINIDASE.'),
  'D053140': NtO(MH='Organelle Shape', MS='The quality of surface form or outline of ORGANELLES.'),
  'D053141': NtO(MH='Organelle Size', MS='The quantity of volume or surface area of ORGANELLES.'),
  'D053142': NtO(MH='Mitochondrial Size', MS='The quantity of volume or surface area of MITOCHONDRIA.'),
  'D053143': NtO(MH='Caspase 2', MS='A long pro-domain caspase that contains a caspase recruitment domain in its pro-domain region. Activation of this enzyme can occur via the interaction of its caspase recruitment domain with CARD SIGNALING ADAPTOR PROTEINS. Caspase 2 plays a role in APOPTOSIS by cleaving and activating effector pro-caspases. Several isoforms of this protein exist due to multiple alternative splicing of its MESSENGER RNA.'),
  'D053144': NtO(MH='Cell Nucleus Shape', MS='The quality of surface form or outline of the CELL NUCLEUS.'),
  'D053145': NtO(MH='Cell Nucleus Size', MS='The quantity of volume or surface area of a CELL NUCLEUS.'),
  'D053146': NtO(MH='Myeloblastin', MS='A polymorphonuclear leukocyte-derived serine protease that degrades proteins such as ELASTIN; FIBRONECTIN; LAMININ; VITRONECTIN; and COLLAGEN. It is named for its ability to control myeloid cell growth and differentiation.'),
  'D053147': NtO(MH='Plant Exudates', MS='Substances released by PLANTS such as PLANT GUMS and PLANT RESINS.'),
  'D053148': NtO(MH='Caspase 3', MS='A short pro-domain caspase that plays an effector role in APOPTOSIS. It is activated by INITIATOR CASPASES such as CASPASE 9. Isoforms of this protein exist due to multiple alternative splicing of its MESSENGER RNA.'),
  'D053149': NtO(MH='Plant Gums', MS='Polysaccharide gums from PLANTS.'),
  'D053150': NtO(MH='Simian foamy virus', MS='A species of SPUMAVIRUS causing non-pathogenic infections in chimpanzees and humans.'),
  'D053158': NtO(MH='Nocturia', MS='Frequent URINATION at night that interrupts sleep. It is often associated with outflow obstruction, DIABETES MELLITUS, or bladder inflammation (CYSTITIS).'),
  'D053159': NtO(MH='Dysuria', MS='Painful URINATION. It is often associated with infections of the lower URINARY TRACT.'),
  'D053178': NtO(MH='Caspase 6', MS='A short pro-domain caspase that plays an effector role in APOPTOSIS. It is activated by INITIATOR CASPASES such as CASPASE 7; CASPASE 8; and CASPASE 10. Isoforms of this protein exist due to multiple alternative splicing of its MESSENGER RNA.'),
  'D053179': NtO(MH='Caspase 7', MS='A short pro-domain caspase that plays an effector role in APOPTOSIS. It is activated by INITIATOR CASPASES such as CASPASE 3 and CASPASE 10. Several isoforms of this protein exist due to multiple alternative splicing of its MESSENGER RNA.'),
  'D053180': NtO(MH='Animal Nutrition Sciences', MS='The study of NUTRITION PROCESSES, as well as the components of food, their actions, interaction, and balance in relation to health and disease in animals.'),
  'D053181': NtO(MH='Caspase 8', MS='A long pro-domain caspase that contains a death effector domain in its pro-domain region. Caspase 8 plays a role in APOPTOSIS by cleaving and activating EFFECTOR CASPASES. Activation of this enzyme can occur via the interaction of its N-terminal death effector domain with DEATH DOMAIN RECEPTOR SIGNALING ADAPTOR PROTEINS.'),
  'D053198': NtO(MH='Child Nutrition Sciences', MS='The study of NUTRITION PROCESSES as well as the components of food, their actions, interaction, and balance in relation to health and disease of children, infants or adolescents.'),
  'D053199': NtO(MH='TNF Receptor-Associated Death Domain Protein', MS='A 34 kDa signal transducing adaptor protein that associates with TUMOR NECROSIS FACTOR RECEPTOR TYPE 1. It facilitates the recruitment of signaling proteins such as TNF RECEPTOR-ASSOCIATED FACTOR 2 and FAS ASSOCIATED DEATH DOMAIN PROTEIN to the receptor complex.'),
  'D053200': NtO(MH='Fas-Associated Death Domain Protein', MS='A signal-transducing adaptor protein that associates with TNF RECEPTOR complexes. It contains a death effector domain that can interact with death effector domains found on INITIATOR CASPASES such as CASPASE 8 and CASPASE 10. Activation of CASPASES via interaction with this protein plays a role in the signaling cascade that leads to APOPTOSIS.'),
  'D053201': NtO(MH='Urinary Bladder, Overactive', MS='Symptom of overactive detrusor muscle of the URINARY BLADDER that contracts with abnormally high frequency and urgency. Overactive bladder is characterized by the frequent feeling of needing to urinate during the day, during the night, or both. URINARY INCONTINENCE may or may not be present.'),
  'D053202': NtO(MH='Urinary Incontinence, Urge', MS='Involuntary discharge of URINE that is associated with an abrupt and strong desire to void. It is usually related to the involuntary contractions of the detrusor muscle of the bladder (detrusor hyperreflexia or detrusor instability).'),
  'D053204': NtO(MH='Root Nodules, Plant', MS='Knobbed structures formed from and attached to plant roots, especially of LEGUMES, which result from symbiotic infection by nitrogen fixing bacteria such as RHIZOBIUM or FRANKIA. Root nodules are structures related to MYCORRHIZAE formed by symbiotic associations with fungi.'),
  'D053205': NtO(MH='Pollen Tube', MS='A growth from a pollen grain down into the flower style which allows two sperm to pass, one to the ovum within the ovule, and the other to the central cell of the ovule to produce endosperm of SEEDS.'),
  'D053206': NtO(MH='Nocturnal Enuresis', MS='Involuntary discharge of URINE during sleep at night after expected age of completed development of urinary control.'),
  'D053207': NtO(MH='Diurnal Enuresis', MS='Involuntary discharge of URINE during the daytime while one is awake.'),
  'D053208': NtO(MH='Kaplan-Meier Estimate', MS='A nonparametric method of compiling LIFE TABLES or survival tables. It combines calculated probabilities of survival and estimates to allow for observations occurring beyond a measurement threshold, which are assumed to occur randomly. Time intervals are defined as ending each time an event occurs and are therefore unequal. (From Last, A Dictionary of Epidemiology, 1995)'),
  'D053209': NtO(MH='Herbicide Resistance', MS='Diminished or failed response of PLANTS to HERBICIDES.'),
  'D053210': NtO(MH='Pharmaceutical Preparations, Dental', MS='Drugs intended for DENTISTRY.'),
  'D053218': NtO(MH='Receptors, Death Domain', MS='A family of cell surface receptors that signal via a conserved domain that extends into the cell CYTOPLASM. The conserved domain is referred to as a DEATH DOMAIN due to the fact that many of these receptors are involved in APOPTOSIS signaling pathways. Several DEATH DOMAIN RECEPTOR SIGNALING ADAPTOR PROTEINS can bind to the death domains of the activated receptors and through a complex series of interactions activate apoptotic mediators such as CASPASES.'),
  'D053219': NtO(MH='Receptors, Tumor Necrosis Factor, Member 25', MS='A tumor necrosis factor receptor subtype with specificity for TUMOR NECROSIS FACTOR LIGAND SUPERFAMILY MEMBER 15. It is found in tissues containing LYMPHOCYTES and may play a role in regulating lymphocyte homeostasis and APOPTOSIS. The activated receptor signals via a conserved death domain that associates with specific TNF RECEPTOR-ASSOCIATED FACTORS in the CYTOPLASM.'),
  'D053220': NtO(MH='Receptors, TNF-Related Apoptosis-Inducing Ligand', MS='Tumor necrosis factor receptor family members that are widely expressed and play a role in regulation of peripheral immune responses and APOPTOSIS. The receptors are specific for TNF-RELATED APOPTOSIS-INDUCING LIGAND and signal via conserved DEATH DOMAINS that associate with specific TNF RECEPTOR-ASSOCIATED FACTORS in the CYTOPLASM.'),
  'D053221': NtO(MH='TNF-Related Apoptosis-Inducing Ligand', MS='A transmembrane-protein belonging to the TNF family of intercellular signaling proteins. It is a widely expressed ligand that activates APOPTOSIS by binding to TNF-RELATED APOPTOSIS-INDUCING LIGAND RECEPTORS. The membrane-bound form of the protein can be cleaved by specific CYSTEINE ENDOPEPTIDASES to form a soluble ligand form.'),
  'D053222': NtO(MH='Fas Ligand Protein', MS='A transmembrane protein belonging to the tumor necrosis factor superfamily that was originally discovered on cells of the lymphoid-myeloid lineage, including activated T-LYMPHOCYTES and NATURAL KILLER CELLS. It plays an important role in immune homeostasis and cell-mediated toxicity by binding to the FAS RECEPTOR and triggering APOPTOSIS.'),
  'D053240': NtO(MH='Receptors, Tumor Necrosis Factor, Member 14', MS='A novel member of the tumor-necrosis factor receptor family that can also mediate HERPES SIMPLEX VIRUS TYPE 1 entry into cells. It has specificity for TUMOR NECROSIS FACTOR LIGAND SUPERFAMILY MEMBER 14 and the homotrimeric form of LYMPHOTOXIN-ALPHA. The receptor is abundantly expressed on T-LYMPHOCYTES and may play a role in regulating lymphocyte activation. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053241': NtO(MH='Apoprotein(a)', MS='A large and highly glycosylated protein constituent of LIPOPROTEIN (A). It has very little affinity for lipids but forms disulfide-linkage to APOLIPOPROTEIN B-100. Apoprotein(a) has SERINE PROTEINASE activity and can be of varying sizes from 400- to 800-kDa. It is homologous to PLASMINOGEN and is known to modulate THROMBOSIS and FIBRINOLYSIS.'),
  'D053243': NtO(MH='Zanamivir', MS='A guanido-neuraminic acid that is used to inhibit NEURAMINIDASE.'),
  'D053244': NtO(MH='Osteoprotegerin', MS='A secreted member of the TNF receptor superfamily that negatively regulates osteoclastogenesis. It is a soluble decoy receptor of RANK LIGAND that inhibits both CELL DIFFERENTIATION and function of OSTEOCLASTS by inhibiting the interaction between RANK LIGAND and RECEPTOR ACTIVATOR OF NUCLEAR FACTOR-KAPPA B.'),
  'D053245': NtO(MH='RANK Ligand', MS='A transmembrane protein belonging to the tumor necrosis factor superfamily that specifically binds RECEPTOR ACTIVATOR OF NUCLEAR FACTOR-KAPPA B and OSTEOPROTEGERIN. It plays an important role in regulating OSTEOCLAST differentiation and activation.'),
  'D053246': NtO(MH='Receptor Activator of Nuclear Factor-kappa B', MS='A tumor necrosis factor receptor family member that is specific for RANK LIGAND and plays a role in bone homeostasis by regulating osteoclastogenesis. It is also expressed on DENDRITIC CELLS where it plays a role in regulating dendritic cell survival. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053259': NtO(MH='Lymphotoxin beta Receptor', MS='A member of the tumor necrosis factor receptor superfamily. It has specificity for LYMPHOTOXIN ALPHA1, BETA2 HETEROTRIMER and TUMOR NECROSIS FACTOR LIGAND SUPERFAMILY MEMBER 14. The receptor plays a role in regulating lymphoid ORGANOGENESIS and the differentiation of certain subsets of NATURAL KILLER T-CELLS. Signaling of the receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053260': NtO(MH='Soot', MS='A dark powdery deposit of unburned fuel residues, composed mainly of amorphous CARBON and some HYDROCARBONS, that accumulates in chimneys, automobile mufflers and other surfaces exposed to smoke. It is the product of incomplete combustion of carbon-rich organic fuels in low oxygen conditions. It is sometimes called lampblack or carbon black and is used in INK, in rubber tires, and to prepare CARBON NANOTUBES.'),
  'D053261': NtO(MH='Tumor Necrosis Factor Receptor Superfamily, Member 9', MS='A member of the tumor necrosis factor receptor superfamily that is specific for 4-1BB LIGAND. It is found in a variety of immune cell types including activated T-LYMPHOCYTES; NATURAL KILLER CELLS; and DENDRITIC CELLS. Activation of the receptor on T-LYMPHOCYTES plays a role in their expansion, production of cytokines and survival. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053262': NtO(MH='Receptors, OX40', MS='A tumor necrosis family receptor with specificity for OX40 LIGAND. It is found on the surface of activated T-LYMPHOCYTES where it plays a role in enhancing cytokine production and proliferation of CD4-POSITIVE T-LYMPHOCYTES. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053263': NtO(MH='Gene Regulatory Networks', MS='Interacting DNA-encoded regulatory subsystems in the GENOME that coordinate input from activator and repressor TRANSCRIPTION FACTORS during development, cell differentiation, or in response to environmental cues. The networks function to ultimately specify expression of particular sets of GENES for specific conditions, times, or locations.'),
  'D053264': NtO(MH='B-Cell Activating Factor', MS='A tumor necrosis factor superfamily member that plays a role in the regulation of B-LYMPHOCYTE survival. It occurs as a membrane-bound protein that is cleaved to release an biologically active soluble form with specificity to TRANSMEMBRANE ACTIVATOR AND CAML INTERACTOR PROTEIN; B-CELL ACTIVATION FACTOR RECEPTOR; and B-CELL MATURATION ANTIGEN.'),
  'D053265': NtO(MH='B-Cell Activation Factor Receptor', MS='A member of the tumor necrosis factor receptor superfamily that specifically binds B-CELL ACTIVATING FACTOR. It is found on B-LYMPHOCYTES and plays a role in maturation and survival of B-cells. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053278': NtO(MH='OX40 Ligand', MS='A membrane-bound tumor necrosis family member that is expressed on activated antigen-presenting cells such as B-LYMPHOCYTES and MACROPHAGES. It signals T-LYMPHOCYTES by binding the OX40 RECEPTOR.'),
  'D053279': NtO(MH='Polyynes', MS='Hydrocarbons that contain more than one triple bond; or an oxidized form of POLYENES.'),
  'D053280': NtO(MH='Diynes', MS='Compounds with two triple bonds. Some of them are CYTOTOXINS.'),
  'D053281': NtO(MH='Enediynes', MS='Compounds with triple bonds to each side of a double bond. Many of these are CYTOTOXINS and are researched for use as CYTOTOXIC ANTIBIOTICS.'),
  'D053282': NtO(MH='CD27 Ligand', MS='A transmembrane protein belonging to the tumor necrosis factor superfamily that specifically binds to CD27 ANTIGEN. It is found on activated T-LYMPHOCYTES; B-LYMPHOCYTES; and DENDRITIC CELLS where it plays a role in stimulating the proliferation of CD4-POSITIVE T-LYMPHOCYTES and CD8-POSITIVE T-LYMPHOCYTES.'),
  'D053283': NtO(MH='Apolipoprotein B-48', MS='A 241-kDa protein synthesized only in the INTESTINES. It serves as a structural protein of CHYLOMICRONS. Its exclusive association with chylomicron particles provides an indicator of intestinally derived lipoproteins in circulation. Apo B-48 is a shortened form of apo B-100 and lacks the LDL-receptor region.'),
  'D053284': NtO(MH='Polyunsaturated Alkamides', MS='Amides composed of unsaturated aliphatic FATTY ACIDS linked with AMINES by an amide bond. They are most prominent in ASTERACEAE; PIPERACEAE; and RUTACEAE; and also found in ARISTOLOCHIACEAE; BRASSICACEAE; CONVOLVULACEAE; EUPHORBIACEAE; MENISPERMACEAE; POACEAE; and SOLANACEAE. They are recognized by their pungent taste and for causing numbing and salivation.'),
  'D053285': NtO(MH='CD30 Ligand', MS='A membrane-bound tumor necrosis family member found primarily on activated T-LYMPHOCYTES that binds specifically to CD30 ANTIGEN. It may play a role in INFLAMMATION and immune regulation.'),
  'D053298': NtO(MH='Chylomicron Remnants', MS='Metabolic products of chylomicron particles in which TRIGLYCERIDES have been selectively removed by the LIPOPROTEIN LIPASE. These remnants carry dietary lipids in the blood and are cholesterol-rich. Their interactions with MACROPHAGES; ENDOTHELIAL CELLS; and SMOOTH MUSCLE CELLS in the artery wall can lead to ATHEROSCLEROSIS.'),
  'D053299': NtO(MH='Apolipoprotein B-100', MS='A 513-kDa protein synthesized in the LIVER. It serves as the major structural protein of low-density lipoproteins (LIPOPROTEINS, LDL; LIPOPROTEINS, VLDL). It is the ligand for the LDL receptor (RECEPTORS, LDL) that promotes cellular binding and internalization of LDL particles.'),
  'D053300': NtO(MH='Tumor Necrosis Factor Ligand Superfamily Member 13', MS='A member of tumor necrosis factor superfamily found on MACROPHAGES; DENDRITIC CELLS and T-LYMPHOCYTES. It occurs as transmembrane protein that can be cleaved to release a secreted form that specifically binds to TRANSMEMBRANE ACTIVATOR AND CAML INTERACTOR PROTEIN; and B CELL MATURATION ANTIGEN.'),
  'D053301': NtO(MH='B-Cell Maturation Antigen', MS='A member of the tumor necrosis factor receptor superfamily found on mature B-LYMPHOCYTES. It has specificity for B CELL ACTIVATING FACTOR and TUMOR NECROSIS FACTOR LIGAND SUPERFAMILY MEMBER 13. Signaling of the receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053302': NtO(MH='Apolipoprotein C-I', MS='A 6.6-kDa protein component of VERY-LOW-DENSITY LIPOPROTEINS; INTERMEDIATE-DENSITY LIPOPROTEINS; and HIGH-DENSITY LIPOPROTEINS. Apo C-I displaces APO E from lipoproteins, modulate their binding to receptors (RECEPTORS, LDL), and thereby decrease their clearance from plasma. Elevated Apo C-I levels are associated with HYPERLIPOPROTEINEMIA and ATHEROSCLEROSIS.'),
  'D053303': NtO(MH='Transmembrane Activator and CAML Interactor Protein', MS='A tumor necrosis factor receptor superfamily member found expressed on peripheral B-LYMPHOCYTES. It has specificity for B-CELL MATURATION ANTIGEN and TUMOR NECROSIS FACTOR LIGAND SUPERFAMILY MEMBER 13.'),
  'D053304': NtO(MH='Apolipoprotein C-II', MS='A 9-kDa protein component of VERY-LOW-DENSITY LIPOPROTEINS. It contains a cofactor for LIPOPROTEIN LIPASE and activates several triacylglycerol lipases. The association of Apo C-II with plasma CHYLOMICRONS; VLDL, and HIGH-DENSITY LIPOPROTEINS is reversible and changes rapidly as a function of triglyceride metabolism. Clinically, Apo C-II deficiency is similar to lipoprotein lipase deficiency (HYPERLIPOPROTEINEMIA TYPE I) and is therefore called hyperlipoproteinemia type IB.'),
  'D053305': NtO(MH='Apolipoprotein C-III', MS='A 9-kDa protein component of VERY-LOW-DENSITY LIPOPROTEINS and CHYLOMICRON REMNANTS. Apo C-III, synthesized in the liver, is an inhibitor of LIPOPROTEIN LIPASE. Apo C-III modulates the binding of chylomicron remnants and VLDL to receptors (RECEPTORS, LDL) thus decreases the uptake of triglyceride-rich particles by the liver cells and subsequent degradation. The normal Apo C-III is glycosylated. There are several polymorphic forms with varying amounts of SIALIC ACID (Apo C-III-0, Apo C-III-1, and Apo C-III-2).'),
  'D053306': NtO(MH='Hyper-IgM Immunodeficiency Syndrome', MS='A rare inherited immunodeficiency syndrome characterized by normal or elevated serum IMMUNOGLOBULIN M levels with absence of IMMUNOGLOBULIN G; IMMUNOGLOBULIN A; and IMMUNOGLOBULIN E. It results in a profound susceptibility to BACTERIAL INFECTIONS and an increased susceptibility to OPPORTUNISTIC INFECTIONS. Several subtypes of hyper-IgM immunodeficiency syndrome exist depending upon the location of genetic mutation.'),
  'D053307': NtO(MH='Hyper-IgM Immunodeficiency Syndrome, Type 1', MS='An X-linked hyper-IgM immunodeficiency subtype resulting from mutation in the gene encoding CD40 LIGAND.'),
  'D053308': NtO(MH='Lymphotoxin-beta', MS='A membrane-bound tumor necrosis family member found primarily on LYMPHOCYTES. It can form a heterotrimer (LYMPHOTOXIN ALPHA1, BETA2 HETEROTRIMER) with the soluble ligand LYMPHOTOXIN-ALPHA and anchor it to the cell surface. The membrane-bound complex is specific for the LYMPHOTOXIN BETA receptor.'),
  'D053318': NtO(MH='Apolipoprotein E3', MS='A 34-kDa glycosylated protein. A major and most common isoform of apolipoprotein E. Therefore, it is also known as apolipoprotein E (ApoE). In human, Apo E3 is a 299-amino acid protein with a cysteine at the 112 and an arginine at the 158 position. It is involved with the transport of TRIGLYCERIDES; PHOSPHOLIPIDS; CHOLESTEROL; and CHOLESTERYL ESTERS in and out of the cells.'),
  'D053319': NtO(MH='Tumor Necrosis Factor Decoy Receptors', MS='A subclass of tumor necrosis family receptors that lack cell signaling domains. They bind to specific TNF RECEPTOR LIGANDS and are believed to play a modulating role in the TNF signaling pathway. Some of the decoy receptors are products of distinct genes, while others are products of ALTERNATIVE SPLICING of the MRNA for the active receptor.'),
  'D053320': NtO(MH='Receptors, Tumor Necrosis Factor, Member 10c', MS='A secreted tumor necrosis factor receptor family member that has specificity for TNF-RELATED APOPTOSIS-INDUCING LIGAND. It plays a modulating role in activation of APOPTOSIS signaling.'),
  'D053323': NtO(MH='Receptors, Tumor Necrosis Factor, Member 6b', MS='A secreted tumor necrosis factor receptor family member that has specificity FAS LIGAND and TUMOR NECROSIS FACTOR LIGAND SUPERFAMILY MEMBER 14. It plays a modulating role in tumor necrosis factor signaling pathway.'),
  'D053326': NtO(MH='Tumor Necrosis Factor Ligand Superfamily Member 14', MS='A member of tumor necrosis factor superfamily found on activated LYMPHOCYTES and MONOCYTES. It occurs as transmembrane protein that can be cleaved to release a secreted form that specifically binds to LYMPHOTOXIN BETA RECEPTOR and TUMOR NECROSIS FACTOR RECEPTOR SUPERFAMILY, MEMBER 14.'),
  'D053327': NtO(MH='Apolipoprotein E4', MS='A major and the second most common isoform of apolipoprotein E. In humans, Apo E4 differs from APOLIPOPROTEIN E3 at only one residue 112 (cysteine is replaced by arginine), and exhibits a lower resistance to denaturation and greater propensity to form folded intermediates. Apo E4 is a risk factor for ALZHEIMER DISEASE and CARDIOVASCULAR DISEASES.'),
  'D053329': NtO(MH='Apolipoprotein E2', MS='One of three major isoforms of apolipoprotein E. In humans, Apo E2 differs from APOLIPOPROTEIN E3 at one residue 158 where arginine is replaced by cysteine (R158--C). In contrast to Apo E3, Apo E2 displays extremely low binding affinity for LDL receptors (RECEPTORS, LDL) which mediate the internalization and catabolism of lipoprotein particles in liver cells. ApoE2 allelic homozygosity is associated with HYPERLIPOPROTEINEMIA TYPE III.'),
  'D053330': NtO(MH='4-1BB Ligand', MS='A membrane bound member of the TNF superfamily that is expressed on activated B-LYMPHOCYTES; MACROPHAGES; and DENDRITIC CELLS. The ligand is specific for the 4-1BB RECEPTOR and may play a role in inducing the proliferation of activated peripheral blood T-LYMPHOCYTES.'),
  'D053331': NtO(MH='Ectodysplasins', MS='Transmembrane proteins belonging to the tumor necrosis factor superfamily that play an essential role in the normal development of several ectodermally derived organs. Several isoforms of the ectodysplasins exist due to multiple ALTERNATIVE SPLICING of the MRNA for the protein. The isoforms ectodysplasin A1 and ectodysplasin A2 are considered biologically active and each bind distinct ECTODYSPLASIN RECEPTORS. Genetic mutations that result in loss of function of ectodysplasin result in ECTODERMAL DYSPLASIA 1, ANHIDROTIC.'),
  'D053338': NtO(MH='Receptors, Ectodysplasin', MS='Members of the TNF receptor family that are specific for ECTODYSPLASIN. At least two subtypes of the ectodysplasin receptor exist, each being specific for a ectodysplasin isoform. Signaling through ectodysplasin receptors plays an essential role in the normal ectodermal development. Genetic defects that result in loss of ectodysplasin receptor function results ECTODERMAL DYSPLASIA.'),
  'D053339': NtO(MH='Edar Receptor', MS='A ectodysplasin receptor subtype that is specific for ECTODYSPLASIN A1. It signals via the specific signaling adaptor EDAR-ASSOCIATED DEATH DOMAIN PROTEIN. Loss of function of the edar receptor is associated with AUTOSOMAL RECESSIVE ANHIDROTIC ECTODERMAL DYSPLASIA and ECTODERMAL DYSPLASIA 3, ANHIDROTIC.'),
  'D053340': NtO(MH='Xedar Receptor', MS='An ectodysplasin receptor subtype that is specific for ECTODYSPLASIN A2. Unlike the EDAR RECEPTOR the Xedar receptor signals through direct association with TNF RECEPTOR-ASSOCIATED FACTORS. The protein name derives from the fact that gene that encodes it resides on the X CHROMOSOME.'),
  'D053358': NtO(MH='Ectodermal Dysplasia 1, Anhidrotic', MS='An X-linked form of ectodermal dysplasia which results from mutations of the gene encoding ECTODYSPLASIN.'),
  'D053359': NtO(MH='Ectodermal Dysplasia 3, Anhidrotic', MS='An autosomal dominant form of ectodermal dysplasia which is due to mutations in the gene for the EDAR RECEPTOR.'),
  'D053360': NtO(MH='Ectodermal Dysplasia, Hypohidrotic, Autosomal Recessive', MS='An autosomal recessive form of ectodermal dysplasia which is due to mutations in the gene for the EDAR RECEPTOR or EDAR-ASSOCIATED DEATH DOMAIN PROTEIN.'),
  'D053361': NtO(MH='Lymphotoxin alpha1, beta2 Heterotrimer', MS='A heterotrimer complex consisting of one molecule of LYMPHOTOXIN-ALPHA and two molecules of the LYMPHOTOXIN-BETA. It is anchored to the cell surface via the transmembrane domains of the lymphotoxin-beta component and has specificity for the LYMPHOTOXIN BETA RECEPTOR. The lymphotoxin alpha1, beta2 heterotrimer plays a role in regulating lymphoid ORGANOGENESIS and the differentiation of certain subsets of NATURAL KILLER CELLS.'),
  'D053362': NtO(MH='Edar-Associated Death Domain Protein', MS='A tumor necrosis factor receptor-associated factor that acts as a specific signaling adaptor protein for the EDAR RECEPTOR and plays an important role in ectodermal development. It binds to edar receptor via its C-terminal death domain region and to other specific TNF receptor-associated factors via its N-terminal domain. Loss of function of edar-associated death domain protein is associated with AUTOSOMAL RECESSIVE ANHIDROTIC ECTODERMAL DYSPLASIA.'),
  'D053378': NtO(MH='Chromogranin B', MS='A type of chromogranin which was initially characterized in a rat PHEOCHROMOCYTOMA CELL LINE. It is found in many species including human, rat, mouse, and others. It is an acidic protein with 626 to 657 amino acid residues. In some species, it inhibits secretion of PARATHYROID HORMONE or INSULIN and exerts bacteriolytic effects in others.'),
  'D053379': NtO(MH='Chromogranin A', MS='A type of chromogranin which was first isolated from CHROMAFFIN CELLS of the ADRENAL MEDULLA but is also found in other tissues and in many species including human, bovine, rat, mouse, and others. It is an acidic protein with 431 to 445 amino acid residues. It contains fragments that inhibit vasoconstriction or release of hormones and neurotransmitter, while other fragments exert antimicrobial actions.'),
  'D053380': NtO(MH='Tumor Necrosis Factor Ligand Superfamily Member 15', MS='A member of tumor necrosis factor superfamily found on ENDOTHELIAL CELLS that plays a role in the inhibition of endothelial cell growth and PHYSIOLOGIC ANGIOGENESIS.'),
  'D053381': NtO(MH='Secretogranin II', MS='A type of chromogranin which was initially characterized in the ANTERIOR PITUITARY GLAND. It is found in several species including human, rat, mouse, and others. Secretogranin II is an acidic protein of 559 to 586 amino acid residues that can stimulate DOPAMINE release from neurons and release of pituitary GONADOTROPINS.'),
  'D053399': NtO(MH='Apolipoproteins D', MS='A glycoprotein component of HIGH-DENSITY LIPOPROTEINS that transports small hydrophobic ligands including CHOLESTEROL and STEROLS. It occurs in the macromolecular complex with LECITHIN CHOLESTEROL ACYLTRANSFERASE. Apo D is expressed in and secreted from a variety of tissues such as liver, placenta, brain tissue and others.'),
  'D053400': NtO(MH='High Vocal Center', MS='Nucleus in the NEOSTRIATUM of bird brains that sends signals for song production and receives auditory input. In some adult SONGBIRDS, research has shown that the size of this nucleus changes seasonally and that it exhibits neurogenesis.'),
  'D053401': NtO(MH='Palmar Plate', MS='A thick, fibrocartilaginous ligament at the METACARPOPHALANGEAL JOINT of the hand.'),
  'D053402': NtO(MH='PHEX Phosphate Regulating Neutral Endopeptidase', MS='A membrane-bound metalloendopeptidase that may play a role in the degradation or activation of a variety of PEPTIDE HORMONES and INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS. Genetic mutations that result in loss of function of this protein are a cause of HYPOPHOSPHATEMIC RICKETS, X-LINKED DOMINANT.'),
  'D053403': NtO(MH='Lateral Line System', MS='Aquatic vertebrate sensory system in fish and amphibians. It is composed of sense organs (canal organs and pit organs) containing neuromasts (MECHANORECEPTORS) that detect water displacement caused by moving objects.'),
  'D053418': NtO(MH='Death Domain Receptor Signaling Adaptor Proteins', MS='Intracellular signaling adaptor proteins that contain DEATH EFFECTOR DOMAINS and bind to the cytoplasmic DEATH DOMAIN region found on DEATH DOMAIN RECEPTORS. Many of the proteins in this class take part in intracellular signaling involving TUMOR NECROSIS FACTOR RECEPTORS.'),
  'D053419': NtO(MH='Lipoproteins, IDL', MS='A mixture of very-low-density lipoproteins (VLDL), particularly the triglyceride-poor VLDL, with slow diffuse electrophoretic mobilities in the beta and alpha2 regions which are similar to that of beta-lipoproteins (LDL) or alpha-lipoproteins (HDL). They can be intermediate (remnant) lipoproteins in the de-lipidation process, or remnants of mutant CHYLOMICRONS and VERY-LOW-DENSITY LIPOPROTEINS which cannot be metabolized completely as seen in FAMILIAL DYSBETALIPOPROTEINEMIA.'),
  'D053420': NtO(MH='CRADD Signaling Adaptor Protein', MS='A death domain receptor signaling adaptor protein that plays a role in signaling the activation of INITIATOR CASPASES such as CASPASE 2. It contains a death domain that is specific for RIP SERINE-THEONINE KINASES and a caspase-binding domain that binds to and activates CASPASES such as CASPASE 2.'),
  'D053421': NtO(MH='Hand-Arm Vibration Syndrome', MS='An occupational disorder resulting from prolonged exposure to vibration, affecting the fingers, hands, and forearms. It occurs in workers who regularly use vibrating tools such as jackhammers, power chain saws, riveters, etc. Symptoms include episodic finger blanching, NUMBNESS, tingling, and loss of nerve sensitivity.'),
  'D053422': NtO(MH='Receptor-Interacting Protein Serine-Threonine Kinases', MS='A family of serine-threonine kinases that plays a role in intracellular signal transduction by interacting with a variety of signaling adaptor proteins such as CRADD SIGNALING ADAPTOR PROTEIN; TNF RECEPTOR-ASSOCIATED FACTOR 2; and TNF RECEPTOR-ASSOCIATED DEATH DOMAIN PROTEIN. Although they were initially described as death domain-binding adaptor proteins, members of this family may contain other protein-binding domains such as those involving caspase activation and recruitment.'),
  'D053439': NtO(MH='Lipoproteins, HDL2', MS='Low-density subclass of the high-density lipoproteins, with particle sizes between 8 to 13 nm.'),
  'D053440': NtO(MH='Lipoproteins, HDL3', MS='Intermediate-density subclass of the high-density lipoproteins, with particle sizes between 7 to 8 nm. As the larger lighter HDL2 lipoprotein, HDL3 lipoprotein is lipid-rich.'),
  'D053441': NtO(MH='Suture Anchors', MS='Implants used in arthroscopic surgery and other orthopedic procedures to attach soft tissue to bone. One end of a suture is tied to soft tissue and the other end to the implant. The anchors are made of a variety of materials including titanium, stainless steel, or absorbable polymers.'),
  'D053442': NtO(MH='High-Density Lipoproteins, Pre-beta', MS='A highly dense subclass of the high-density lipoproteins, with particle sizes below 7 nm. They are also known as nascent HDL, composed of a few APOLIPOPROTEIN A-I molecules which are complexed with PHOSPHOLIPIDS. The lipid-poor pre-beta-HDL particles serve as progenitors of HDL3 and then HDL2 after absorption of free cholesterol from cell membranes, cholesterol esterification, and acquisition of apolipoproteins A-II, Cs, and E. Pre-beta-HDL initiate the reverse cholesterol transport process from cells to liver.'),
  'D053443': NtO(MH='Visible Human Projects', MS='Digital image data sets, consisting of complete, anatomically detailed, three-dimensional representations of the normal male and female human bodies.'),
  'D053444': NtO(MH='Inhibitory Postsynaptic Potentials', MS='Hyperpolarization of membrane potentials at the SYNAPTIC MEMBRANES of target neurons during NEUROTRANSMISSION. They are local changes which diminish responsiveness to excitatory signals.'),
  'D053446': NtO(MH='CASP8 and FADD-Like Apoptosis Regulating Protein', MS='An APOPTOSIS-regulating protein that is structurally related to CASPASE 8 and competes with CASPASE 8 for binding to FAS ASSOCIATED DEATH DOMAIN PROTEIN. Two forms of CASP8 and FADD-like apoptosis regulating protein exist, a long form containing a caspase-like enzymatically inactive domain and a short form which lacks the caspase-like domain.'),
  'D053447': NtO(MH='Channelopathies', MS='A variety of neuromuscular conditions resulting from MUTATIONS in ION CHANNELS manifesting as episodes of EPILEPSY; HEADACHE DISORDERS; and DYSKINESIAS.'),
  'D053448': NtO(MH='Prostatism', MS='Lower urinary tract symptom, such as slow urinary stream, associated with PROSTATIC HYPERPLASIA in older men.'),
  'D053449': NtO(MH='TNF Receptor-Associated Factor 4', MS='A widely expressed member of the TNF receptor-associated family that may play a role in neuronal development and EMBRYOGENESIS. Although TNF receptor-associated factor 4 does not strongly associate with TUMOR NECROSIS FACTOR RECEPTORS it may be a signaling partner with the GLUCOCORTICOID-INDUCED TNFR-RELATED PROTEIN that plays a role in the activation of JNK MITOGEN-ACTIVATED PROTEIN KINASES and NF-KAPPA B.'),
  'D053450': NtO(MH='Glucocorticoid-Induced TNFR-Related Protein', MS='A member of the TNF receptor family that was initially identified as a DEXAMETHASONE-induced protein from a T-CELL line. It may play a role in regulating APOPTOSIS and modulating immune response by T-lymphocytes. Signaling by the activated receptor occurs through its association with TNF RECEPTOR-ASSOCIATED FACTORS.'),
  'D053453': NtO(MH='Caspase 9', MS='A long pro-domain caspase that contains a CASPASE RECRUITMENT DOMAIN in its pro-domain region. Caspase 9 is activated during cell stress by mitochondria-derived proapoptotic factors and by CARD SIGNALING ADAPTOR PROTEINS such as APOPTOTIC PROTEASE-ACTIVATING FACTOR 1. It activates APOPTOSIS by cleaving and activating EFFECTOR CASPASES.'),
  'D053454': NtO(MH='Caspase 12', MS='A long pro-domain caspase that contains a caspase recruitment domain in its pro-domain region. Caspase 12 is activated by pro-apoptotic factors that are released during cell stress and by CARD SIGNALING ADAPTOR PROTEINS. It activates APOPTOSIS by cleaving and activating EFFECTOR CASPASES.'),
  'D053455': NtO(MH='Caspase 10', MS='A long pro-domain caspase that contains a death effector domain in its pro-domain region. Activation of this enzyme can occur via the interaction of its N-terminal death effector domain with DEATH DOMAIN RECEPTOR SIGNALING ADAPTOR PROTEINS. Caspase 10 plays a role in APOPTOSIS by cleaving and activating EFFECTOR CASPASES. Several isoforms of this protein exist due to multiple alternative splicing of its MESSENGER RNA.'),
  'D053456': NtO(MH='Caspase 14', MS='A short pro-domain caspase that is almost exclusively expressed in the EPIDERMIS and may play a role in the differentiation of epidermal KERATINOCYTES.'),
  'D053468': NtO(MH='CARD Signaling Adaptor Proteins', MS='A family of intracellular signaling adaptor proteins that contain CASPASE ACTIVATION AND RECRUITMENT DOMAINS (CARD). They play a role in APOPTOSIS-related signal transduction by associating with other CARD domain-containing members and activating INITIATOR CASPASES that contain CARD domains within their N-terminal pro-domain region.'),
  'D053470': NtO(MH='Caspases, Effector', MS='A subclass of caspases that contain short pro-domain regions. They are activated by the proteolytic action of INITIATOR CASPASES. Once activated they cleave a variety of substrates that cause APOPTOSIS.'),
  'D053471': NtO(MH='Caspases, Initiator', MS='A subtype of caspases that contain long pro-domain regions that regulate the activation of the enzyme. The pro-domain regions contain protein-protein interaction motifs that can interact with specific signaling adaptor proteins such as DEATH DOMAIN RECEPTORS; DED SIGNALING ADAPTOR PROTEINS; and CARD SIGNALING ADAPTOR PROTEINS. Once activated, the initiator caspases can activate other caspases such as the EFFECTOR CASPASES.'),
  'D053472': NtO(MH='Nod Signaling Adaptor Proteins', MS='Cytosolic signaling adaptor proteins that were initially discovered by their role in the innate immunity (IMMUNITY, INNATE) response of organisms that lack an adaptive immune system. This class of proteins contains three domains, a C-terminal ligand recognition domain, an N-terminal effector-binding domain, and a centrally located nuclear-binding oligomerization domain. Many members of this class contain a C-terminal leucine rich domain which binds to PEPTIDOGLYCAN on the surface of BACTERIA and plays a role in pathogen resistance.'),
  'D053473': NtO(MH='Nod2 Signaling Adaptor Protein', MS='A NOD signaling adaptor protein that contains two C-terminal leucine-rich domains which recognize bacterial PEPTIDOGLYCAN. It signals via an N-terminal capase recruitment domain that interacts with other CARD SIGNALING ADAPTOR PROTEINS such as RIP SERINE-THEONINE KINASES. The protein plays a role in the host defense response by signaling the activation of CASPASES and the MAP KINASE SIGNALING SYSTEM. Mutations of the gene encoding the nucleotide oligomerization domain 2 protein have been associated with increased susceptibility to CROHN DISEASE.'),
  'D053474': NtO(MH='Nod1 Signaling Adaptor Protein', MS='A NOD-signaling adaptor protein that contains a C-terminal leucine-rich domain which recognizes bacterial PEPTIDOGLYCAN. It signals via an N-terminal caspase recruitment domain that interacts with other CARD SIGNALING ADAPTOR PROTEINS such as RIP SERINE-THEONINE KINASES. It plays a role in the host defense response by signaling the activation of CASPASES and the MAP KINASE SIGNALING SYSTEM.'),
  'D053475': NtO(MH='Receptor-Interacting Protein Serine-Threonine Kinase 2', MS='A RIP serine-theonine kinase that contains a C-terminal caspase activation and recruitment domain. It can signal by associating with other CARD-signaling adaptor proteins and INITIATOR CASPASES that contain CARD domains within their N-terminal pro-domain region.'),
  'D053476': NtO(MH='Extinction, Biological', MS='The ceasing of existence of a species or taxonomic groups of organisms.'),
  'D053477': NtO(MH='Apoptotic Protease-Activating Factor 1', MS='A CARD signaling adaptor protein characterized by an N-terminal CARD DOMAIN and C-terminal WD40 REPEATS. It plays a role in the mitochondria-stimulated apoptosis (APOPTOSIS, INTRINSIC PATHWAY) and binds to CYTOCHROME C in the CYTOSOL to form an APOPTOSOMAL PROTEIN COMPLEX which activates INITIATOR CASPASES such as CASPASE 9.'),
  'D053478': NtO(MH='Apoptosomes', MS='Multimeric protein complexes formed in the CYTOSOL that play a role in the activation of APOPTOSIS. They can occur when MITOCHONDRIA become damaged due to cell stress and release CYTOCHROME C. Cytosolic cytochrome C associates with APOPTOTIC PROTEASE-ACTIVATING FACTOR 1 to form the apoptosomal protein complex. The apoptosome signals apoptosis by binding to and activating specific INITIATOR CASPASES such as CASPASE 9.'),
  'D053480': NtO(MH='Cholesterol Ester Transfer Proteins', MS='Proteins that bind to and transfer CHOLESTEROL ESTERS between LIPOPROTEINS such as LOW-DENSITY LIPOPROTEINS and HIGH-DENSITY LIPOPROTEINS.'),
  'D053481': NtO(MH='Secretory Leukocyte Peptidase Inhibitor', MS='A proteinase inhibitor found in various BODILY SECRETIONS that coat mucosal surfaces such as SEMINAL PLASMA; CERVICAL MUCUS; and bronchial secretions. It plays a role in protecting epithelial tissues from LEUKOCYTE-derived serine proteases such as NEUTROPHIL ELASTASE.'),
  'D053482': NtO(MH='beta 2-Glycoprotein I', MS='A 44-kDa highly glycosylated plasma protein that binds phospholipids including CARDIOLIPIN; APOLIPOPROTEIN E RECEPTOR; membrane phospholipids, and other anionic phospholipid-containing moieties. It plays a role in coagulation and apoptotic processes. Formerly known as apolipoprotein H, it is an autoantigen in patients with ANTIPHOSPHOLIPID ANTIBODIES.'),
  'D053483': NtO(MH='Eye Movement Measurements', MS='Methods and procedures for recording EYE MOVEMENTS.'),
  'D053484': NtO(MH='RecQ Helicases', MS='A family of structurally-related DNA helicases that play an essential role in the maintenance of genome integrity. RecQ helicases were originally discovered in E COLI and are highly conserved across both prokaryotic and eukaryotic organisms. Genetic mutations that result in loss of RecQ helicase activity gives rise to disorders that are associated with CANCER predisposition and premature aging.'),
  'D053485': NtO(MH='Porcine epidemic diarrhea virus', MS='A species of CORONAVIRUS causing acute enteritis in swine. Infections have been seen mostly in Europe, where it is endemic, and in China.'),
  'D053486': NtO(MH='Melanocortins', MS='Peptides derived from pro-opiomelanocortin (POMC) which can stimulate MELANOCYTES or CORTICOTROPHS. Melanocortins include ACTH; ALPHA-MSH; and other peptides such as BETA-MSH and GAMMA-MSH, derived from other fragments of POMC. These peptides act through a variety of MELANOCORTIN RECEPTORS to control different functions including steroidogenesis, energy homeostasis, feeding, and skin pigmentation.'),
  'D053487': NtO(MH='DEAD-box RNA Helicases', MS='A large family of RNA helicases that share a common protein motif with the single letter amino acid sequence D-E-A-D (Asp-Glu-Ala-Asp). In addition to RNA helicase activity, members of the DEAD-box family participate in other aspects of RNA metabolism and regulation of RNA function.'),
  'D053488': NtO(MH='DnaB Helicases', MS="A family of DNA helicases that participate in DNA REPLICATION. They assemble into hexameric rings with a central channel and unwind DNA processively in the 5' to 3' direction. DnaB helicases are considered the primary replicative helicases for most prokaryotic organisms."),
  'D053489': NtO(MH='Mink Viral Enteritis', MS='A highly contagious parvovirus infection in mink, caused by MINK ENTERITIS VIRUS or the closely related FELINE PANLEUKOPENIA VIRUS or CANINE PARVOVIRUS. Transmission usually occurs by the fecal/oral route.'),
  'D053490': NtO(MH='Mink enteritis virus', MS='A species of the genus PARVOVIRUS and a host range variant of FELINE PANLEUKOPENIA VIRUS. It causes a highly contagious inflammatory gastroenteritis (MINK VIRAL ENTERITIS). In addition to mink, this virus can also infect cats and dogs.'),
  'D053491': NtO(MH='Proteinase Inhibitory Proteins, Secretory', MS='Peptides and proteins found in BODILY SECRETIONS and BODY FLUIDS that are PROTEASE INHIBITORS. They play a role in INFLAMMATION, tissue repair and innate immunity (IMMUNITY, INNATE) by inhibiting endogenous proteinases such as those produced by LEUKOCYTES and exogenous proteases such as those produced by invading microorganisms.'),
  'D053492': NtO(MH='Elafin', MS='A secretory proteinase inhibitory protein that was initially purified from human SKIN. It is found in a variety mucosal secretions and is present at high levels in SPUTUM. Elafin may play a role in the innate immunity (IMMUNITY, INNATE) response of the LUNG.'),
  'D053493': NtO(MH='Cholestanetriol 26-Monooxygenase', MS='An NAPH-dependent cytochrome P450 enzyme that catalyzes the oxidation of the side chain of sterol intermediates such as the 27-hydroxylation of 5-beta-cholestane-3-alpha,7-alpha,12-alpha-triol.'),
  'D053495': NtO(MH='Osteopontin', MS='A negatively-charged extracellular matrix protein that plays a role in the regulation of BONE metabolism and a variety of other biological functions. Cell signaling by osteopontin may occur through a cell adhesion sequence that recognizes INTEGRIN ALPHA-V BETA-3.'),
  'D053496': NtO(MH='Inositol 1,4,5-Trisphosphate Receptors', MS='Intracellular receptors that bind to INOSITOL 1,4,5-TRISPHOSPHATE and play an important role in its intracellular signaling. Inositol 1,4,5-trisphosphate receptors are calcium channels that release CALCIUM in response to increased levels of inositol 1,4,5-trisphosphate in the CYTOPLASM.'),
  'D053497': NtO(MH='Corticotropin-Like Intermediate Lobe Peptide', MS='A peptide derived from the cleavage of ADRENOCORTICOTROPIC HORMONE, found primarily in the INTERMEDIATE LOBE OF PITUITARY but also in the ARCUATE NUCLEUS of basal HYPOTHALAMUS. Its sequence is identical to the C-terminal 22-amino acids of ACTH or ACTH 18-39.'),
  'D053498': NtO(MH='Sarcoplasmic Reticulum Calcium-Transporting ATPases', MS='Calcium-transporting ATPases that catalyze the active transport of CALCIUM into the SARCOPLASMIC RETICULUM vesicles from the CYTOPLASM. They are primarily found in MUSCLE CELLS and play a role in the relaxation of MUSCLES.'),
  'D053499': NtO(MH='Plasma Membrane Calcium-Transporting ATPases', MS='Calcium-transporting ATPases found on the PLASMA MEMBRANE that catalyze the active transport of CALCIUM from the CYTOPLASM into the extracellular space. They play a role in maintaining a CALCIUM gradient across plasma membrane.'),
  'D053500': NtO(MH='gamma-Lipotropin', MS='A lipotropic peptide that is the N-terminal fragment of beta-lipotropin. It is a peptide of 56 amino acids in humans and 58 amino acids in pigs.'),
  'D053501': NtO(MH='Arsenite Transporting ATPases', MS='Efflux pumps that use the energy of ATP hydrolysis to pump arsenite across a membrane. They are primarily found in prokaryotic organisms, where they play a role in protection against excess intracellular levels of arsenite ions.'),
  'D053502': NtO(MH='Arsenate Reductases', MS='Oxidoreductases that specifically reduce arsenate ion to arsenite ion. Reduction of arsenate is a critical step for its biotransformation into a form that can be transported by ARSENITE TRANSPORTING ATPASES or complexed by specific sulfhydryl-containing proteins for the purpose of detoxification (METABOLIC DETOXIFICATION, DRUG). Arsenate reductases require reducing equivalents such as GLUTAREDOXIN or AZURIN.'),
  'D053503': NtO(MH='Epithelial Sodium Channels', MS='Sodium channels found on salt-reabsorbing EPITHELIAL CELLS that line the distal NEPHRON; the distal COLON; SALIVARY DUCTS; SWEAT GLANDS; and the LUNG. They are AMILORIDE-sensitive and play a critical role in the control of sodium balance, BLOOD VOLUME, and BLOOD PRESSURE.'),
  'D053504': NtO(MH='Matrix Metalloproteinases, Membrane-Associated', MS='Matrix metalloproteinases that are associated with the CELL MEMBRANE, either through transmembrane domains or GLYCOSYLPHOSPHATIDYLINOSITOL ANCHORS. Membrane-type matrix metalloproteinases may act within the pericellular environment to influence the process of CELL MIGRATION.'),
  'D053505': NtO(MH='Matrix Metalloproteinases, Secreted', MS='A subclass of matrix metalloproteinases that are secreted into the pericellular space.'),
  'D053506': NtO(MH='Matrix Metalloproteinase 10', MS='A secreted matrix metalloproteinase that may play a role in matrix degradation during WOUND HEALING. It is expressed at high levels by KERATINOCYTES, suggesting its role in keratinocyte migration.'),
  'D053507': NtO(MH='Matrix Metalloproteinase 11', MS='A secreted matrix metalloproteinase that is believed to play a role in EXTRACELLULAR MATRIX remodeling and cell fate determination during normal and pathological processes. Matrix metalloproteinase 11 was originally isolated in primary BREAST NEOPLASMS and may be involved in the process of tumorigenesis.'),
  'D053508': NtO(MH='Matrix Metalloproteinase 12', MS='A secreted matrix metalloproteinase which is highly expressed by MACROPHAGES where it may play a role in INFLAMMATION and WOUND HEALING.'),
  'D053509': NtO(MH='Matrix Metalloproteinase 13', MS='A secreted matrix metalloproteinase that plays a physiological role in the degradation of extracellular matrix found in skeletal tissues. It is synthesized as an inactive precursor that is activated by the proteolytic cleavage of its N-terminal propeptide.'),
  'D053511': NtO(MH='Matrix Metalloproteinase 14', MS='A transmembrane domain-containing matrix metalloproteinase. It is synthesized as an inactive zymogen that is activated by the action of PROPROTEIN CONVERTASES such as FURIN. Matrix metalloproteinase 14 plays a direct role in the cleavage of proteins in the pericellular environment. In addition, it can function indirectly by enzymatically activating the proprotein form of MATRIX METALLOPROTEINASE 15.'),
  'D053513': NtO(MH='Matrix Metalloproteinase 16', MS='A transmembrane domain-containing matrix metalloproteinase. It is synthesized as an inactive zymogen that is activated by the proteolytic action of PROPROTEIN CONVERTASES. Matrix metalloproteinase 16 plays a direct role in the cleavage of proteins in the pericellular environment. In addition, it can function indirectly by enzymatically activating the proprotein form of other MATRIX METALLOPROTEINASES such as the zymogen of MATRIX METALLOPROTEINASE 2.'),
  'D053515': NtO(MH='Matrix Metalloproteinase 17', MS='A membrane-type metalloproteinase that is associated with the CELL MEMBRANE via a GLYCOSYL-PHOSPHATIDYLINOSITOL linkage. It is synthesized as an inactive zymogen that is activated by the action of PROPROTEIN CONVERTASES such as FURIN.'),
  'D053517': NtO(MH='Matrix Metalloproteinase 15', MS='A transmembrane domain-containing matrix metalloproteinase that plays a role in the cleavage of proteins in the pericellular environment. It is synthesized as an inactive zymogen that is activated by the action of ENDOPEPTIDASES such as MATRIX METALLOPROTEINASE 14.'),
  'D053519': NtO(MH='Matrix Metalloproteinase 20', MS='A secreted matrix metalloproteinase that is the predominant proteolytic activity in the enamel matrix. The enzyme has a high specificity for dental enamel matrix protein AMELOGENIN.'),
  'D053520': NtO(MH='Marinobacter', MS='A genus of gram-negative, rod-shaped bacteria in the family ALTEROMONADACEAE. The inability to utilize carbohydrates is a distinguishing feature from other genera in the family.'),
  'D053521': NtO(MH='Alcanivoraceae', MS='A family of halophilic bacteria in the order Oceanospirillales. Its principal carbon and energy sources are linear-chain ALKANES and their derivatives.'),
  'D053522': NtO(MH='Piscirickettsia', MS='A genus of bacteria in the family PISCIRICKETTSIACEAE that is an intracellular rickettsial-like pathogen of FISHES, especially species in the family SALMONIDAE.'),
  'D053523': NtO(MH='Amelogenin', MS='A major dental enamel-forming protein found in mammals. In humans the protein is encoded by GENES found on both the X CHROMOSOME and the Y CHROMOSOME.'),
  'D053524': NtO(MH='Beggiatoa', MS='A genus of colorless, filamentous bacteria in the family THIOTRICHACEAE whose cells contain inclusions of sulfur granules. When found in decaying seaweed beds and polluted water, its presence signals environmental degradation.'),
  'D053525': NtO(MH='Thiothrix', MS='A genus of colorless, filamentous, rod-shaped bacteria in the family THIOTRICHACEAE, containing sulfur globules. Thiothrix species are found as components of BIOFILMS in irrigation systems and wastewater treatment plants, and in marine environments as endosymbionts.'),
  'D053526': NtO(MH='Listonella', MS='A genus of gram-negative bacteria with curved rods, in the family VIBRIONACEAE. The organisms are chemoorganotrophic and are found in marine environments.'),
  'D053527': NtO(MH='Keratin-4', MS='A type II keratin that is found associated with the KERATIN-13 in the internal stratified EPITHELIUM. Defects in gene for keratin-4 are a cause of HEREDITARY MUCOSAL LEUKOKERATOSIS.'),
  'D053528': NtO(MH='Lysobacter', MS='A genus of gram-negative, rod-shaped, gliding bacteria in the family XANTHOMONADACEAE. Strongly proteolytic, it is involved in lysing a variety of microorganisms.'),
  'D053529': NtO(MH='Leukokeratosis, Hereditary Mucosal', MS='An autosomal dominant disorder that is manifested by thickened spongiform ORAL MUCOSA with a white opalescent tint. Other MUCOSAL TISSUE may also be involved mucosa found in the VAGINA; RECTUM, and NASAL CAVITY may be similarly involved. This form of LEUKOKERATOSIS can be caused by a mutation in the gene for KERATIN 4 and is not considered a PRENEOPLASTIC CONDITION.'),
  'D053530': NtO(MH='Keratins, Type I', MS='A keratin subtype that includes keratins that are generally smaller and more acidic that TYPE II KERATINS. Type I keratins combine with type II keratins to form keratin filaments.'),
  'D053531': NtO(MH='Keratins, Type II', MS='A keratin subtype that includes keratins that are generally larger and less acidic that TYPE I KERATINS. Type II keratins combine with type I keratins to form keratin filaments.'),
  'D053532': NtO(MH='Xanthomonas axonopodis', MS='A species of gram-negative bacteria in the genus XANTHOMONAS, which causes citrus cankers and black rot in plants.'),
  'D053533': NtO(MH='Keratin-8', MS='A type II keratin found associated with KERATIN-18 in simple, or predominately single layered, internal epithelia.'),
  'D053534': NtO(MH='Keratin-9', MS='A type II keratin found predominantly expressed in the terminally differentiated EPIDERMIS of palms and soles. Mutations in the gene for keratin 9 are associated with KERATODERMA, PALMOPLANTAR, EPIDERMOLYTIC.'),
  'D053535': NtO(MH='Keratin-13', MS='A type I keratin that is found associated with the KERATIN-4 in the internal stratified EPITHELIUM. Defects in gene for keratin 13 cause HEREDITARY MUCOSAL LEUKOKERATOSIS.'),
  'D053536': NtO(MH='Keratin-16', MS='A type I keratin expressed in a variety of EPITHELIUM, including the ESOPHAGUS, the TONGUE, the HAIR FOLLICLE and NAILS. Keratin-16 is normally found associated with KERATIN-6. Mutations in the gene for keratin-6 have been associated with PACHYONYCHIA CONGENITA, TYPE 1.'),
  'D053537': NtO(MH='Keratin-17', MS='A type I keratin found associated with KERATIN-6 in rapidly proliferating squamous epithelial tissue. Mutations in the gene for keratin-17 have been associated with PACHYONYCHIA CONGENITA, TYPE 2.'),
  'D053538': NtO(MH='Keratin-18', MS='A type I keratin found associated with KERATIN-8 in simple, or predominately single layered, internal epithelia.'),
  'D053539': NtO(MH='Keratin-19', MS='A type I keratin found associated with KERATIN-7 in ductal epithelia and gastrointestinal epithelia.'),
  'D053544': NtO(MH='Marinomonas', MS='A genus of gram-negative, aerobic bacteria in the family OCEANOSPIRILLACEAE. Members of this genus have the ability to synthesize MELANIN pigments.'),
  'D053545': NtO(MH='Spirulina', MS='A genus of filamentous CYANOBACTERIA found in most lakes and ponds. It has been used as a nutritional supplement particularly due to its high protein content.'),
  'D053546': NtO(MH='Keratoderma, Palmoplantar, Epidermolytic', MS='An autosomal dominant hereditary skin disease characterized by epidermolytic hyperkeratosis that is strictly confined to the palms and soles. It has been associated with mutations in the gene that codes for KERATIN-9.'),
  'D053547': NtO(MH='Keratin-14', MS='A type I keratin that is found associated with the KERATIN-5 in the internal stratified EPITHELIUM. Mutations in the gene for keratin-14 are associated with EPIDERMOLYSIS BULLOSA SIMPLEX.'),
  'D053549': NtO(MH='Pachyonychia Congenita', MS='A group of inherited ectodermal dysplasias whose most prominent clinical feature is hypertrophic nail dystrophy resulting in PACHYONYCHIA. Several specific subtypes of pachyonychia congenita have been associated with mutations in genes that encode KERATINS.'),
  'D053550': NtO(MH='Keratin-10', MS='A type I keratin that is found associated with the KERATIN-1 in terminally differentiated epidermal cells such as those that form the stratum corneum. Mutations in the genes that encode keratin-10 have been associated with HYPERKERATOSIS, EPIDERMOLYTIC.'),
  'D053551': NtO(MH='Keratin-12', MS='A type I keratin that is found associated with the KERATIN-3 in the CORNEA and is regarded as a marker for corneal-type epithelial differentiation. Mutations in the gene for keratin-12 have been associated with MEESMANN CORNEAL EPITHELIAL DYSTROPHY.'),
  'D053552': NtO(MH='Keratin-7', MS='A type II keratin found associated with KERATIN-19 in ductal epithelia and gastrointestinal epithelia.'),
  'D053553': NtO(MH='Keratin-6', MS='A type II keratin found associated with KERATIN-16 or KERATIN-17 in rapidly proliferating squamous epithelial tissue. Mutations in gene for keratin-6A and keratin-6B have been associated with PACHYONYCHIA CONGENITA, TYPE 1 and PACHYONYCHIA CONGENITA, TYPE 2 respectively.'),
  'D053554': NtO(MH='Keratin-20', MS='A type I keratin expressed predominately in gastrointestinal epithelia, MERKEL CELLS, and the TASTE BUDS of the oral mucosa.'),
  'D053555': NtO(MH='Keratin-5', MS='A type II keratin that is found associated with the KERATIN-14 in the internal stratified EPITHELIUM. Mutations in the gene for keratin-5 are associated with EPIDERMOLYSIS BULLOSA SIMPLEX.'),
  'D053556': NtO(MH='Keratin-1', MS='A type II keratin that is found associated with the KERATIN-10 in terminally differentiated epidermal cells such as those that form the stratum corneum. Mutations in the genes that encode keratin-1 have been associated with HYPERKERATOSIS, EPIDERMOLYTIC.'),
  'D053557': NtO(MH='Keratin-2', MS='A type II keratin found expressed in the upper spinous layer of epidermal KERATINOCYTES. Mutations in genes that encode keratin-2A have been associated with ICHTHYOSIS BULLOSA OF SIEMENS.'),
  'D053558': NtO(MH='Keratin-3', MS='A type II keratin that is found associated with the KERATIN-12 in the CORNEA and is regarded as a marker for corneal-type epithelial differentiation. Mutations in the gene for keratin-3 have been associated with MEESMANN CORNEAL EPITHELIAL DYSTROPHY.'),
  'D053559': NtO(MH='Corneal Dystrophy, Juvenile Epithelial of Meesmann', MS='An autosomal dominant form of hereditary corneal dystrophy due to a defect in cornea-specific KERATIN formation. Mutations in the genes that encode KERATIN-3 and KERATIN-12 have been linked to this disorder.'),
  'D053560': NtO(MH='Ichthyosis Bullosa of Siemens', MS='An autosomal dominant form of ichthyosis characterized by generalized reddening of the skin (ERYTHEMA) and widespread blistering. The disease shows similar, but somewhat milder, clinical and histopathological findings to those in HYPERKERATOSIS, EPIDERMOLYTIC and is associated with the gene that encodes KERATIN-2A.'),
  'D053561': NtO(MH='Somatostatin-28', MS='A 28-amino acid peptide with the same biological activities of somatostatin-14 but with a 14-amino acid extension at the N-terminal. SRIF-28 is the major form of somatostatin in the GASTROINTESTINAL TRACT.'),
  'D053564': NtO(MH='Keratins, Hair-Specific', MS='Keratins that are specific for hard tissues such as HAIR; NAILS; and the filiform papillae of the TONGUE.'),
  'D053565': NtO(MH='Hypercalciuria', MS='Excretion of abnormally high level of CALCIUM in the URINE, greater than 4 mg/kg/day.'),
  'D053566': NtO(MH='Feminine Hygiene Products', MS='Personal care items for women.'),
  'D053569': NtO(MH='Keratin-15', MS='A type I keratin found in the basal layer of the adult epidermis and in other stratified epithelia.'),
  'D053570': NtO(MH='Porcine Postweaning Multisystemic Wasting Syndrome', MS='A worldwide emerging disease of weaned piglets first recognized in swine herds in western Canada in 1997. This syndrome is characterized by progressive weight loss, rapid (tachypnea) and difficult (dyspnea) breathing, and yellowing of skin. PMWS is caused by PORCINE CIRCOVIRUS infection, specifically type 2 or PCV-2.'),
  'D053573': NtO(MH='Receptors, Interleukin-1 Type I', MS='An interleukin-1 receptor subtype that is involved in signaling cellular responses to INTERLEUKIN-1ALPHA and INTERLEUKIN-1BETA. The binding of this receptor to its ligand causes its favorable interaction with INTERLEUKIN-1 RECEPTOR ACCESSORY PROTEIN and the formation of an activated receptor complex.'),
  'D053574': NtO(MH='Receptors, Interleukin-1 Type II', MS='An interleukin-1 receptor subtype that competes with the INTERLEUKIN-1 RECEPTOR TYPE I for binding to INTERLEUKIN-1ALPHA and INTERLEUKIN-1BETA. The interleukin-1 type II receptor appears to lack signal transduction capability. Therefore it may act as a "decoy" receptor that modulates the activity of its ligands. Both membrane-bound and soluble forms of the receptor have been identified.'),
  'D053577': NtO(MH='Interleukin-1 Receptor Accessory Protein', MS='A protein that takes part in the formation of active interleukin-1 receptor complex. It binds specifically to INTERLEUKIN-1 and the INTERLEUKIN-1 RECEPTOR TYPE I at the cell surface to form a heterotrimeric complex that brings its cytoplasmic domain into contact with the cytoplasm domain of the TYPE-I INTERLEUKIN-1 RECEPTOR. Activation of intracellular signal transduction pathways from the receptor is believed to be driven by this form of cytoplasmic interaction.'),
  'D053578': NtO(MH='Opsoclonus-Myoclonus Syndrome', MS='A neurological condition that is characterized by uncontrolled rapid irregular movements of the eye (OPSOCLONUS) and the muscle (MYOCLONUS) causing unsteady, trembling gait. It is also known as dancing eyes-dancing feet syndrome and is often associated with neoplasms, viral infections, or autoimmune disorders involving the nervous system.'),
  'D053579': NtO(MH='Gitelman Syndrome', MS='An inherited renal disorder characterized by defective NaCl reabsorption in the convoluted DISTAL KIDNEY TUBULE leading to HYPOKALEMIA. In contrast with BARTTER SYNDROME, Gitelman syndrome includes hypomagnesemia and normocalcemic hypocalciuria, and is caused by mutations in the thiazide-sensitive SODIUM-POTASSIUM-CHLORIDE SYMPORTERS.'),
  'D053580': NtO(MH='Muscle Strength', MS='The amount of force generated by MUSCLE CONTRACTION. Muscle strength can be measured during isometric, isotonic, or isokinetic contraction, either manually or using a device such as a MUSCLE STRENGTH DYNAMOMETER.'),
  'D053581': NtO(MH='Muscle Strength Dynamometer', MS='A device that measures MUSCLE STRENGTH during muscle contraction, such as gripping, pushing, and pulling. It is used to evaluate the health status of muscle in sports medicine or physical therapy.'),
  'D053582': NtO(MH='Interleukin-1alpha', MS='An interleukin-1 subtype that occurs as a membrane-bound pro-protein form that is cleaved by proteases to form a secreted mature form. Unlike INTERLEUKIN-1BETA both membrane-bound and secreted forms of interleukin-1alpha are biologically active.'),
  'D053583': NtO(MH='Interleukin-1beta', MS='An interleukin-1 subtype that is synthesized as an inactive membrane-bound pro-protein. Proteolytic processing of the precursor form by CASPASE 1 results in release of the active form of interleukin-1beta from the membrane.'),
  'D053584': NtO(MH='Urinoma', MS='An encapsulated accumulation of URINE in the retroperitoneal area. It has the appearance of a cyst (CYSTS). Urinoma is usually caused by URETERAL OBSTRUCTION, renal trauma or perforation of the renal collecting system.'),
  'D053585': NtO(MH='Virus Attachment', MS='The binding of virus particles to receptors on the host cell surface. For enveloped viruses, the virion ligand is usually a surface glycoprotein as is the cellular receptor. For non-enveloped viruses, the virus CAPSID serves as the ligand.'),
  'D053586': NtO(MH='Virus Internalization', MS='The entering of cells by viruses following VIRUS ATTACHMENT. This is achieved by ENDOCYTOSIS, by direct MEMBRANE FUSION of the viral membrane with the CELL MEMBRANE, or by translocation of the whole virus across the cell membrane.'),
  'D053587': NtO(MH='Arthrometry, Articular', MS='Measurements of joint flexibility (RANGE OF MOTION, ARTICULAR), usually by employing an angle-measuring device (arthrometer). Arthrometry is used to measure ligamentous laxity and stability. It is often used to evaluate the outcome of ANTERIOR CRUCIATE LIGAMENT replacement surgery.'),
  'D053589': NtO(MH='Nuchal Cord', MS='A complication of pregnancy in which the UMBILICAL CORD wraps around the fetal neck once or multiple times. In some cases, cord entanglement around fetal neck may not affect pregnancy outcome significantly. In others, the nuchal cord may lead to restricted fetal blood flow, oxygen transport, fetal development, fetal movement, and complicated delivery at birth.'),
  'D053590': NtO(MH='Interleukin 1 Receptor Antagonist Protein', MS='A ligand that binds to but fails to activate the INTERLEUKIN 1 RECEPTOR. It plays an inhibitory role in the regulation of INFLAMMATION and FEVER. Several isoforms of the protein exist due to multiple ALTERNATIVE SPLICING of its mRNA.'),
  'D053591': NtO(MH='Pain, Referred', MS='A type of pain that is perceived in an area away from the site where the pain arises, such as facial pain caused by lesion of the VAGUS NERVE, or throat problem generating referred pain in the ear.'),
  'D053592': NtO(MH='Interleukin-1 Receptor-Associated Kinases', MS='A family of intracellular signaling kinases that were identified by their ability to signal from the activated INTERLEUKIN-1 RECEPTORS. Signaling from these kinases involves their interaction with SIGNAL TRANSDUCING ADAPTOR PROTEINS such as MYELOID DIFFERENTIATION FACTOR 88 and TNF RECEPTOR-ASSOCIATED FACTOR 6.'),
  'D053593': NtO(MH='Forensic Toxicology', MS='The application of TOXICOLOGY knowledge to questions of law.'),
  'D053594': NtO(MH='Myeloid Differentiation Factor 88', MS='An intracellular signaling adaptor protein that plays a role in TOLL-LIKE RECEPTOR and INTERLEUKIN 1 RECEPTORS signal transduction. It forms a signaling complex with the activated cell surface receptors and members of the IRAK KINASES.'),
  'D053595': NtO(MH='Embryonic Stem Cells', MS='Cells derived from the BLASTOCYST INNER CELL MASS which forms before implantation in the uterine wall. They retain the ability to divide, proliferate and provide progenitor cells that can differentiate into specialized cells.'),
  'D053604': NtO(MH='Forensic Nursing', MS='The application of NURSING knowledge to questions of law. It is a specialty of nursing practice involving victims of crime which includes not only attending to the physical and emotional distress of victims, but also the identifying, collecting, and preserving evidence for law enforcement and the criminal justice system.'),
  'D053605': NtO(MH='Leukemia Inhibitory Factor', MS='An INTERLEUKIN-6 related cytokine that exhibits pleiotrophic effects on many physiological systems that involve cell proliferation, differentiation, and survival. Leukemia inhibitory factor binds to and acts through the lif receptor.'),
  'D053606': NtO(MH='Receptors, OSM-LIF', MS='Cell surface receptors formed from the dimerization of LIF RECEPTOR ALPHA SUBUNIT with CYTOKINE RECEPTOR GP130. Although originally described as receptors for LEUKEMIA INHIBITORY FACTOR these receptors also bind the closely-related protein ONCOSTATIN M and are referred to as both LIF receptors and type I oncostatin M receptors.'),
  'D053607': NtO(MH='Adrenomedullin', MS='A 52-amino acid peptide with multi-functions. It was originally isolated from PHEOCHROMOCYTOMA and ADRENAL MEDULLA but is widely distributed throughout the body including lung and kidney tissues. Besides controlling fluid-electrolyte homeostasis, adrenomedullin is a potent vasodilator and can inhibit pituitary ACTH secretion.'),
  'D053608': NtO(MH='Stupor', MS='A state of reduced sensibility and response to stimuli which is distinguished from COMA in that the person can be aroused by vigorous and repeated stimulation. The person is still conscious and can make voluntary movements. It can be induced by CENTRAL NERVOUS SYSTEM AGENTS. The word derives from Latin stupere and is related to stunned, stupid, dazed or LETHARGY.'),
  'D053609': NtO(MH='Lethargy', MS='A general state of sluggishness, listless, or uninterested, with being tired, and having difficulty concentrating and doing simple tasks. It may be related to DEPRESSION or DRUG ADDICTION.'),
  'D053610': NtO(MH='Opiate Alkaloids', MS='Alkaloids found in OPIUM from PAPAVER and their derivatives that induce analgesic and narcotic effects by action upon OPIOID RECEPTORS.'),
  'D053612': NtO(MH='Janus Kinases', MS='A family of intracellular tyrosine kinases that participate in the signaling cascade of cytokines by associating with specific CYTOKINE RECEPTORS. They act upon STAT TRANSCRIPTION FACTORS in signaling pathway referred to as the JAK/STAT pathway. The name Janus kinase refers to the fact the proteins have two phosphate-transferring domains.'),
  'D053613': NtO(MH='Janus Kinase 1', MS='A Janus kinase subtype that is involved in signaling from a broad variety of CYTOKINE RECEPTORS.'),
  'D053614': NtO(MH='Janus Kinase 2', MS='A Janus kinase subtype that is involved in signaling from GROWTH HORMONE RECEPTORS; PROLACTIN RECEPTORS; and a variety of CYTOKINE RECEPTORS such as ERYTHROPOIETIN RECEPTORS and INTERLEUKIN RECEPTORS. Dysregulation of Janus kinase 2 due to GENETIC TRANSLOCATIONS have been associated with a variety of MYELOPROLIFERATIVE DISORDERS.'),
  'D053616': NtO(MH='Janus Kinase 3', MS='A Janus kinase subtype that is predominantly expressed in hematopoietic cells. It is involved in signaling from a broad variety of CYTOKINE RECEPTORS including ones that utilize the INTERLEUKIN RECEPTOR COMMON GAMMA SUBUNIT.'),
  'D053624': NtO(MH='Blastocyst Inner Cell Mass', MS='The cluster of cells inside a blastocyst. These cells give rise to the embryonic disc and eventual embryo proper. They are pluripotent EMBRYONIC STEM CELLS capable of yielding many but not all cell types in a developing organism.'),
  'D053625': NtO(MH='Sperm Retrieval', MS='Procedures to obtain viable sperm from the male reproductive tract, including the TESTES, the EPIDIDYMIS, or the VAS DEFERENS.'),
  'D053626': NtO(MH='Atovaquone', MS='A hydroxynaphthoquinone that has antimicrobial activity and is being used in antimalarial protocols.'),
  'D053627': NtO(MH='Asthenozoospermia', MS='A condition in which the percentage of motile sperm is abnormally low.'),
  'D053628': NtO(MH='Receptors, Thrombopoietin', MS='Cell surface receptors that are specific for THROMBOPOIETIN. They contain two central FIBRONECTIN TYPE III DOMAINS and signal through interaction with JANUS KINASES such as JANUS KINASE 2.'),
  'D053629': NtO(MH='Thapsia', MS='A plant genus of the family APIACEAE. Members contain THAPSIGARGIN and other guaianolides (SESQUITERPENES, GUAIANOLIDE).'),
  'D053630': NtO(MH='Depsides', MS='Phenolic benzoic acid esters.'),
  'D053631': NtO(MH='Interleukin Receptor Common gamma Subunit', MS='An interleukin receptor subunit that was originally discovered as a component of the INTERLEUKIN 2 RECEPTOR. It was subsequently found to be a component of several other receptors including the INTERLEUKIN 4 RECEPTOR, the INTERLEUKIN 7 RECEPTOR, the INTERLEUKIN-9 RECEPTOR, the INTERLEUKIN-15 RECEPTOR, and the INTERLEUKIN-21 RECEPTOR. Mutations in the gene for the interleukin receptor common gamma chain have been associated with X-LINKED COMBINED IMMUNODEFICIENCY DISEASES.'),
  'D053632': NtO(MH='X-Linked Combined Immunodeficiency Diseases', MS='Forms of combined immunodeficiency caused by mutations in the gene for INTERLEUKIN RECEPTOR COMMON GAMMA SUBUNIT. Both severe and non-severe subtypes of the disease have been identified.'),
  'D053633': NtO(MH='Receptor, Interferon alpha-beta', MS='A ubiquitously expressed heterodimeric receptor that is specific for both INTERFERON-ALPHA and INTERFERON-BETA. It is composed of two subunits referred to as IFNAR1 and IFNAR2. The IFNAR2 subunit is believed to serve as the ligand-binding chain; however both chains are required for signal transduction. The interferon alpha-beta receptor signals through the action of JANUS KINASES such as the TYK2 KINASE.'),
  'D053634': NtO(MH='TYK2 Kinase', MS='A Janus kinase subtype that is involved in signaling from a broad variety of CYTOKINE RECEPTORS. The TYK2 kinase is considered the founding member of the janus kinase family and was initially discovered as a signaling partner for the INTERFERON ALPHA-BETA RECEPTOR. The kinase has since been shown to signal from several INTERLEUKIN RECEPTORS.'),
  'D053644': NtO(MH='Naphthalimides', MS='Compounds with three fused rings that appear like a naphthalene fused to piperidone or like a benz(de)isoquinoline-1,3-dione (not to be confused with BENZYLISOQUINOLINES which have a methyl separating the naphthyl from the benzyl rings). Members are CYTOTOXINS.'),
  'D053645': NtO(MH='Interleukin-2 Receptor alpha Subunit', MS='A low affinity interleukin-2 receptor subunit that combines with the INTERLEUKIN-2 RECEPTOR BETA SUBUNIT and the INTERLEUKIN RECEPTOR COMMON GAMMA-CHAIN to form a high affinity receptor for INTERLEUKIN-2.'),
  'D053646': NtO(MH='Interleukin-2 Receptor beta Subunit', MS='A receptor subunit that is a shared component of the INTERLEUKIN 2 RECEPTOR and the INTERLEUKIN-15 RECEPTOR. High affinity receptor complexes are formed with each of these receptors when their respective alpha subunits are combined with this beta subunit and the INTERLEUKIN RECEPTOR COMMON GAMMA-CHAIN.'),
  'D053647': NtO(MH='Receptors, Interleukin-15', MS='Cell surface receptors for INTERLEUKIN-15. They are widely-distributed heterotrimeric proteins consisting of the INTERLEUKIN-15 RECEPTOR ALPHA SUBUNIT, the INTERLEUKIN-2, 15 RECEPTOR BETA SUBUNIT, and the INTERLEUKIN RECEPTOR COMMON GAMMA-CHAIN.'),
  'D053648': NtO(MH='Cytokine Receptor Common beta Subunit', MS='A receptor subunit that is a shared component of the INTERLEUKIN-3 RECEPTOR; the INTERLEUKIN-5 RECEPTOR; and the GM-CSF RECEPTOR. High affinity receptor complexes are formed with each of these receptors when their respective alpha subunits are combined with this shared beta subunit.'),
  'D053649': NtO(MH='Receptors, Interleukin-5', MS='Cell surface receptors that are specific for INTERLEUKIN-5. They are heterodimeric proteins consisting of the INTERLEUKIN-5 RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR COMMON BETA SUBUNIT. Signaling from interleukin-5 receptors can occur through interaction of their cytoplasmic domains with SYNTENINS.'),
  'D053650': NtO(MH='Interleukin-3 Receptor alpha Subunit', MS='A low affinity interleukin-3 receptor subunit that combines with the CYTOKINE RECEPTOR COMMON BETA SUBUNIT to form a high affinity receptor for INTERLEUKIN-3.'),
  'D053651': NtO(MH='Receptors, Interleukin-9', MS='A cell surface receptor that specifically mediates the biological effects of INTERLEUKIN-9. The functional IL9 receptor signals through interaction of its cytoplasm domain with JANUS KINASES and requires the INTERLEUKIN RECEPTOR COMMON GAMMA SUBUNIT for activity.'),
  'D053652': NtO(MH='Nuclear Transfer Techniques', MS='Methods of implanting a CELL NUCLEUS from a donor cell into an enucleated acceptor cell.'),
  'D053653': NtO(MH='Pyelocystitis', MS='Inflammation of the KIDNEY PELVIS and the URINARY BLADDER.'),
  'D053655': NtO(MH='Receptors, Interleukin-21', MS='Cell surface receptors for interleukin 21. They are heterodimeric proteins found on DENDRITIC CELLS and LYMPHOCYTES that consist of the INTERLEUKIN-21 RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR COMMON BETA SUBUNIT.'),
  'D053656': NtO(MH='Tenodesis', MS='Fixation of the end of a tendon to a bone, often by suturing.'),
  'D053657': NtO(MH='Platelet-Rich Plasma', MS='A preparation consisting of PLATELETS concentrated in a limited volume of PLASMA. This is used in various surgical tissue regeneration procedures where the GROWTH FACTORS in the platelets enhance wound healing and regeneration.'),
  'D053658': NtO(MH='Receptors, Interleukin-4, Type I', MS='An interleukin-4 receptor subtype that is found predominantly on hematopoietic cells. It is a heterodimer of the INTERLEUKIN-4 RECEPTOR ALPHA SUBUNIT and the INTERLEUKIN RECEPTOR COMMON GAMMA-CHAIN.'),
  'D053659': NtO(MH='Receptors, Interleukin-4, Type II', MS='An interleukin receptor subtype found on both hematopoietic and non-hematopoietic cells. It is a membrane-bound heterodimer that contains the INTERLEUKIN-4 RECEPTOR ALPHA SUBUNIT and the INTERLEUKIN-13 RECEPTOR ALPHA1 SUBUNIT. Although commonly referred to as the interleukin-4 type-II receptor this receptor has specificity for both INTERLEUKIN-4 and INTERLEUKIN-13'),
  'D053660': NtO(MH='Receptors, Interleukin-13', MS='Cell surface receptors for INTERLEUKIN-13. Included under this heading are the INTERLEUKIN-13 RECEPTOR ALPHA2 which is a monomeric receptor and the INTERLEUKIN-4 RECEPTOR TYPE II which has specificity for both INTERLEUKIN-4 and INTERLEUKIN-13.'),
  'D053661': NtO(MH='Interleukin-13 Receptor alpha1 Subunit', MS='An interleukin receptor subunit with specificity for INTERLEUKIN-13. It dimerizes with the INTERLEUKIN-4 RECEPTOR ALPHA SUBUNIT to form the TYPE II INTERLEUKIN-4 RECEPTOR which has specificity for both INTERLEUKIN-4 and INTERLEUKIN-13. Signaling of this receptor subunit occurs through the interaction of its cytoplasmic domain with JANUS KINASES such as the TYK2 KINASE.'),
  'D053662': NtO(MH='Interleukin-4 Receptor alpha Subunit', MS='A receptor subunit that is a component of the TYPE I INTERLEUKIN-4 RECEPTOR and the TYPE II INTERLEUKIN-4 RECEPTOR. It signals through interaction of its cytoplasmic domain with JANUS KINASES such as JANUS KINASE 1.'),
  'D053663': NtO(MH='Interleukin-13 Receptor alpha2 Subunit', MS='An interleukin-13 receptor subunit that is closely-related to the INTERLEUKIN-13 RECEPTOR ALPHA1 SUBUNIT. The receptor is found as a monomeric protein and has been considered to be a decoy receptor for interleukin-13 due the fact that it lacks cytoplasmic signaling domains.'),
  'D053666': NtO(MH='Interleukin-5 Receptor alpha Subunit', MS='A low affinity interleukin-5 receptor subunit that combines with the CYTOKINE RECEPTOR COMMON BETA SUBUNIT to form a high affinity receptor for INTERLEUKIN-5. Several isoforms of the interleukin-5 receptor alpha subunit exist due to multiple ALTERNATIVE SPLICING.'),
  'D053667': NtO(MH='Syndecans', MS='A family of transmembrane glycoproteins that contain a short cytoplasmic domain, a single-span transmembrane domain, and an extracellular domain with heparin sulfate and CHONDROITIN SULFATE chains. Syndecans interact with a variety of heparin-binding INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS and may play a role in modulating cellular signaling during EMBRYONIC DEVELOPMENT, tumorigenesis, and angiogenesis.'),
  'D053668': NtO(MH='Syndecan-1', MS='A syndecan that interacts with EXTRACELLULAR MATRIX PROTEINS and plays a role CELL PROLIFERATION and CELL MIGRATION.'),
  'D053669': NtO(MH='Syndecan-2', MS='A syndecan that is predominantly expressed during EMBRYONIC DEVELOPMENT. It may play a role in mediating cellular interactions with the EXTRACELLULAR MATRIX and may modulate the signaling activity of certain INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS.'),
  'D053670': NtO(MH='Syndecan-3', MS='A syndecan found at high levels in the developing LIMB BUDS. It may play a role in the regulation of MUSCULOSKELETAL DEVELOPMENT by modulating the effects of INTERCELLULAR SIGNALING PEPTIDES AND PROTEINS.'),
  'D053671': NtO(MH='Syndecan-4', MS='A ubiquitously expressed syndecan that is found in all stages of embryonic development and in most adult tissues. Syndecan-4 is found localized to focal adhesion sites in fibronectin-adherent cells and may play a role the process of CELL MIGRATION and CELL PROLIFERATION.'),
  'D053672': NtO(MH='Electrochemotherapy', MS="A cancer treatment modality that uses pulsed electrical currents to permeabilize cell membranes (ELECTROPORATION) and thereby enhance the uptake of chemotherapeutic agents into the body's cells."),
  'D053673': NtO(MH='Glypicans', MS='A family of GLYCOSYLPHOSPHATIDYLINOSITOL-anchored, cell-surface heparan sulfate proteoglycans that may play a role in CELL GROWTH PROCESSES and CELL DIFFERENTIATION by modulating ligand-receptor interactions.'),
  'D053674': NtO(MH='Aggrecans', MS='Large HYALURONAN-containing proteoglycans found in articular cartilage (CARTILAGE, ARTICULAR). They form into aggregates that provide tissues with the capacity to resist high compressive and tensile forces.'),
  'D053675': NtO(MH='Versicans', MS='HYALURONAN-containing proteoglycans found in the EXTRACELLULAR MATRIX of a variety of tissues and organs. Several versican isoforms exist due to multiple ALTERNATIVE SPLICING of the versican MESSENGER RNA.'),
  'D053676': NtO(MH='Syntenins', MS='Intracellular signaling adaptor proteins that play a role in the coupling of SYNDECANS to CYTOSKELETAL PROTEINS.'),
  'D053677': NtO(MH='Receptors, Oncostatin M', MS='Cell surface receptors with specificity for ONCOSTATIN M. Two subtypes of receptors have been identified and are defined by their subunit composition.'),
  'D053679': NtO(MH='Receptors, Oncostatin M, Type II', MS='A subtype of oncostatin receptors that are formed by dimerization of ONCOSTATIN M RECEPTOR BETA SUBUNIT and CYTOKINE RECEPTOR GP130. They are activated specifically by ONCOSTATIN M and signal through interaction of their cytoplasmic domains with JANUS KINASES.'),
  'D053680': NtO(MH='Leukemia Inhibitory Factor Receptor alpha Subunit', MS='A receptor subunit that combines with CYTOKINE RECEPTOR GP130 to form the dual specificity receptor for LEUKEMIA INHIBITORY FACTOR and ONCOSTATIN M. The subunit is also a component of the CILIARY NEUROTROPHIC FACTOR RECEPTOR. Both membrane-bound and secreted isoforms of the receptor subunit exist due to ALTERNATIVE SPLICING of its mRNA. The secreted isoform is believed to act as an inhibitory receptor, while the membrane-bound form is a signaling receptor.'),
  'D053681': NtO(MH='Oncostatin M Receptor beta Subunit', MS='An ONCOSTATIN M-specific receptor subunit that combines with CYTOKINE RECEPTOR GP130 to form the ONCOSTATIN M TYPE II RECEPTOR.'),
  'D053682': NtO(MH='Tendon Entrapment', MS="Narrowing or stenosis of a tendon's retinacular sheath. It occurs most often in the hand or wrist but can also be found in the foot or ankle. The most common types are DE QUERVAIN DISEASE and TRIGGER FINGER DISORDER."),
  'D053683': NtO(MH='Oncostatin M', MS='A cytokine with both pro- and anti-inflammatory actions that depend upon the cellular microenvironment. Oncostatin M is a 28 kDa monomeric glycoprotein that is similar in structure to LEUKEMIA INHIBITORY FACTOR. Its name derives from the the observation that it inhibited the growth of tumor cells and augmented the growth of normal fibroblasts.'),
  'D053684': NtO(MH='De Quervain Disease', MS='Stenosing tenosynovitis of the abductor pollicis longus and extensor pollicis brevis tendons in the first dorsal wrist compartment. The presenting symptoms are usually pain and tenderness at the radial styloid. The cause is almost always related to OVERUSE INJURY or is associated with RHEUMATOID ARTHRITIS.'),
  'D053685': NtO(MH='Laser Therapy', MS='The use of photothermal effects of LASERS to coagulate, incise, vaporize, resect, dissect, or resurface tissue.'),
  'D053686': NtO(MH='Fetal Stem Cells', MS='Cells derived from a FETUS that retain the ability to divide, proliferate and provide progenitor cells that can differentiate into specialized cells.'),
  'D053687': NtO(MH='Adult Stem Cells', MS='Tissue-specific stem cells (also known as Somatic Stem Cells) that appear during fetal development and remain in the body throughout life. The key functions of adult stem cells are to maintain and repair the specific tissues where they reside (e.g. skin or blood).'),
  'D053698': NtO(MH='Interleukin-6 Receptor alpha Subunit', MS='A low affinity interleukin-6 receptor subunit that combines with the CYTOKINE RECEPTOR GP130 to form a high affinity receptor for INTERLEUKIN-6.'),
  'D053699': NtO(MH='Interleukin-7 Receptor alpha Subunit', MS='A low affinity interleukin-7 receptor subunit that combines with the INTERLEUKIN RECEPTOR COMMON GAMMA SUBUNIT to form a high affinity receptor for INTERLEUKIN-7.'),
  'D053700': NtO(MH='Receptors, Interleukin-8', MS='Cell surface receptors that are specific for INTERLEUKIN-8. Two specific receptor subtypes (type A and B) have been found and bind IL-8 with high affinity.'),
  'D053701': NtO(MH='Receptors, Interleukin-10', MS='Cell surface receptors for INTERLEUKIN-10. They exist as a tetramer of two alpha chains (INTERLEUKIN-10 RECEPTOR ALPHA CHAIN) and two beta chains (INTERLEUKIN-10 RECEPTOR, BETA CHAIN). Signaling from interleukin-10 receptors occurs through their interaction with JANUS KINASES.'),
  'D053702': NtO(MH='Receptors, Interleukin-11', MS='Cell surface receptors that are specific for INTERLEUKIN-11. They consist of heterodimers of the INTERLEUKIN-11 RECEPTOR ALPHA SUBUNIT and the CYTOKINE RECEPTOR GP130.'),
  'D053703': NtO(MH='Interleukin-11 Receptor alpha Subunit', MS='A low affinity interleukin-11 receptor subunit that combines with the CYTOKINE RECEPTOR GP130 to form a high affinity receptor for INTERLEUKIN-11. Multiple isoforms of this protein exist due to ALTERNATIVE SPLICING of its MRNA.'),
  'D053704': NtO(MH='Capsule Endoscopy', MS='Non-invasive, endoscopic imaging by use of VIDEO CAPSULE ENDOSCOPES to perform examination of the gastrointestinal tract, especially the small bowel.'),
  'D053705': NtO(MH='Capsule Endoscopes', MS='A pill sized videocamera encased in a capsule. It is designed to be swallowed and subsequently traverse the gastrointestinal tract while transmitting diagnostic images along the way.'),
  'D053706': NtO(MH='Typhlitis', MS='Necrotizing inflammation of the CECUM ("typhlon" in Greek), sometimes spreading to the APPENDIX and/or the ILEUM. Symptoms include ABDOMINAL PAIN and DIARRHEA. Its pathogenesis is multifactorial. Typhlitis is often associated with NEUTROPENIA and chemotherapy in immunocompromised individuals (IMMUNOCOMPROMISED HOST).'),
  'D053707': NtO(MH='Receptors, Interleukin-12', MS='Cell surface receptors for INTERLEUKIN-12. They exist as dimers of beta 1 and beta 2 subunits. Signaling from interleukin-12 receptors occurs through their interaction with JANUS KINASES.'),
  'D053709': NtO(MH='Interleukin-10 Receptor alpha Subunit', MS='A subunit of the interleukin-10 receptor. It plays a role in receptor signaling by associating with JANUS KINASE 1.'),
  'D053710': NtO(MH='Interleukin-10 Receptor beta Subunit', MS='A subunit of the interleukin-10 receptor. It plays a role in receptor signaling by associating with TYK2 KINASE.'),
  'D053711': NtO(MH='Interleukin-12 Receptor beta 1 Subunit', MS='A subunit of the interleukin-12 receptor. It plays a role in receptor signaling by associating with TYK2 KINASE 1.'),
  'D053712': NtO(MH='Interleukin-12 Receptor beta 2 Subunit', MS='A subunit of the interleukin-12 receptor. It plays a role in receptor signaling by associating with JANUS KINASE 2.'),
  'D053713': NtO(MH='Azoospermia', MS='A condition of having no sperm present in the ejaculate (SEMEN).'),
  'D053714': NtO(MH='Aspermia', MS='A condition characterized by the complete absence of SEMEN. This disorder should be differentiated from AZOOSPERMIA, absence of sperm in the semen.'),
  'D053715': NtO(MH='Cystotomy', MS='Surgical incision or puncture into a URINARY BLADDER. Cystotomy may be used to remove URINARY CALCULI, or to perform tissue repair and reconstruction.'),
  'D053716': NtO(MH='Female Athlete Triad Syndrome', MS='A condition of competitive female athletes in which there are interrelated problems of feeding and eating disorders; AMENORRHEA; and OSTEOPOROSIS.'),
  'D053717': NtO(MH='Pneumonia, Ventilator-Associated', MS='Serious INFLAMMATION of the LUNG in patients who required the use of PULMONARY VENTILATOR. It is usually caused by bacterial CROSS INFECTION in hospitals.'),
  'D053718': NtO(MH='Scleromyxedema', MS='A connective tissue disorder characterized by widespread thickening of SKIN with a cobblestone-like appearance. It is caused by proliferation of FIBROBLASTS and deposition of MUCIN in the DERMIS in the absence of thyroid disease. Most scleromyxedema cases are associated with a MONOCLONAL GAMMOPATHY, immunoglobulin IgG-lambda.'),
  'D053719': NtO(MH='Tandem Mass Spectrometry', MS='A mass spectrometry technique using two (MS/MS) or more mass analyzers. With two in tandem, the precursor ions are mass-selected by a first mass analyzer, and focused into a collision region where they are then fragmented into product ions which are then characterized by a second mass analyzer. A variety of techniques are used to separate the compounds, ionize them, and introduce them to the first mass analyzer. For example, for in GC-MS/MS, GAS CHROMATOGRAPHY-MASS SPECTROMETRY is involved in separating relatively small compounds by GAS CHROMATOGRAPHY prior to injecting them into an ionization chamber for the mass selection.'),
  'D053720': NtO(MH='Interleukin-15 Receptor alpha Subunit', MS='A low affinity interleukin-15 receptor subunit that combines with the INTERLEUKIN-2 RECEPTOR BETA SUBUNIT and the INTERLEUKIN RECEPTOR COMMON GAMMA-CHAIN to form a high affinity receptor for INTERLEUKIN-15.'),
  'D053721': NtO(MH='Receptors, Interleukin-16', MS='Cell surface receptors for INTERLEUKIN-16. Although initial studies have identified CD4 ANTIGENS as receptors for interleukin-16, the fact that cells from KNOCKOUT MICE which lack CD4 antigen are responsive to IL-16 indicate that are at least 2 distinct receptors subtypes.'),
  'D053722': NtO(MH='Receptors, Interleukin-17', MS='Cell surface receptors for INTERLEUKIN-17. Several subtypes of receptors have been found, each with its own in specificity for interleukin-17 subtype.'),
  'D053723': NtO(MH='Panax notoginseng', MS='A plant species of the genus PANAX. It contains damarane-type tetracyclic TRITERPENES. The common names of Sanchi or Tienchi are also used for Panax pseudoginseng which is distinguished in containing oleanane-type pentacyclic triterpenoids.'),
  'D053724': NtO(MH='Receptors, Interleukin-18', MS='Cell surface receptors for INTERLEUKIN-18 found on a variety of cell types including MACROPHAGES; NEUTROPHILS; NK CELLS; ENDOTHELIAL CELLS; and SMOOTH MUSCLE CELLS. They are formed as a heterodimer of alpha and beta subunits.'),
  'D053727': NtO(MH='Interleukin-18 Receptor alpha Subunit', MS='A subunit of the interleukin-18 receptor that is responsible of extracellular binding of IL-18.'),
  'D053728': NtO(MH='Interleukin-18 Receptor beta Subunit', MS='A subunit of the interleukin-18 receptor that plays a role in receptor signaling by association of its cytoplasmic domain with SIGNAL TRANSDUCING ADAPTOR PROTEINS such as MYELOID DIFFERENTIATION FACTOR 88.'),
  'D053738': NtO(MH='Interleukin-21 Receptor alpha Subunit', MS='An interleukin-21 receptor subunit that combines with the INTERLEUKIN RECEPTOR COMMON GAMMA SUBUNIT to form a high affinity receptor for interleukin-21. It signals via interaction of its cytoplasmic domain with JANUS KINASES such as JANUS KINASE 1 and JANUS KINASE 3.'),
  'D053739': NtO(MH='Ciliary Neurotrophic Factor Receptor alpha Subunit', MS='A ciliary neurotrophic factor receptor subunit. It is anchored to the cell surface via GLYCOSYLPHOSPHATIDYLINOSITOL LINKAGE and has specificity for binding to CILIARY NEUROTROPHIC FACTOR. It lacks signal transducing domains which are found on the other two subunits of the receptor.'),
  'D053758': NtO(MH='Nanoparticles', MS='Nanometer-sized particles that are nanoscale in three dimensions. They include nanocrystaline materials; NANOCAPSULES; METAL NANOPARTICLES; DENDRIMERS, and QUANTUM DOTS. The uses of nanoparticles include DRUG DELIVERY SYSTEMS and cancer targeting and imaging.'),
  'D053759': NtO(MH='Interleukin-23', MS='A heterodimeric cytokine that plays a role in innate and adaptive immune responses. Interleukin-23 is comprised of a unique 19 kDa subunit and 40 kDa subunit that is shared with INTERLEUKIN-12. It is produced by DENDRITIC CELLS; MACROPHAGES and a variety of other immune cells'),
  'D053760': NtO(MH='Interleukin-23 Subunit p19', MS='A subunit of interleukin-23. It combines with INTERLEUKIN-12 SUBUNIT P40, which is shared between the two cytokines, to form in the active interleukin-23 cytokine.'),
  'D053761': NtO(MH='Nanocomposites', MS='Nanometer-scale composite structures composed of organic molecules intimately incorporated with inorganic molecules. (Glossary of Biotechnology and Nanobiotechology Terms, 4th ed)'),
  'D053762': NtO(MH='Interleukin-12 Subunit p40', MS='A cytokine subunit that is a component of both interleukin-12 and interleukin-23. It binds to the INTERLEUKIN-12 SUBUNIT P35 via a disulfide bond to form interleukin-12 and to INTERLEUKIN-23 SUBUNIT P19 to form interleukin-23.'),
  'D053763': NtO(MH='Presenilins', MS='Integral membrane proteins and essential components of the gamma-secretase complex that catalyzes the cleavage of membrane proteins such as NOTCH RECEPTORS and AMYLOID BETA-PEPTIDES precursors. Mutations of presenilins lead to presenile ALZHEIMER DISEASE with onset before age 65 years.'),
  'D053764': NtO(MH='Presenilin-1', MS='Integral membrane protein of Golgi and endoplasmic reticulum. Its homodimer is an essential component of the gamma-secretase complex that catalyzes the cleavage of membrane proteins such as NOTCH RECEPTORS and AMYLOID BETA-PEPTIDES precursors. PSEN1 mutations cause early-onset ALZHEIMER DISEASE type 3 that may occur as early as 30 years of age in humans.'),
  'D053765': NtO(MH='Interleukin-12 Subunit p35', MS='A subunit of interleukin-12. It binds to the INTERLEUKIN-12 SUBUNIT P40 via a disulfide bond that results in the active cytokine.'),
  'D053766': NtO(MH='Presenilin-2', MS='Integral membrane protein of Golgi and endoplasmic reticulum. Its homodimer is an essential component of the gamma-secretase complex that catalyzes the cleavage of membrane proteins such as NOTCH RECEPTORS and AMYLOID BETA-PEPTIDES precursors. PSEN2 mutations cause ALZHEIMER DISEASE type 4.'),
  'D053767': NtO(MH='Glucagon-Like Peptide 2', MS='A 33-amino acid peptide derived from the C-terminal of PROGLUCAGON and mainly produced by the INTESTINAL L CELLS. It stimulates intestinal mucosal growth and decreased apoptosis of ENTEROCYTES. GLP-2 enhances gastrointestinal function and plays an important role in nutrient homeostasis.'),
  'D053768': NtO(MH='Metal Nanoparticles', MS='Nanoparticles produced from metals whose uses include biosensors, optics, and catalysts. In biomedical applications the particles frequently involve the noble metals, especially gold and silver.'),
  'D053769': NtO(MH='Nanocapsules', MS='Nanometer-sized, hollow, spherically-shaped objects that can be utilized to encapsulate small amounts of pharmaceuticals, enzymes, or other catalysts (Glossary of Biotechnology and Nanobiotechnology, 4th ed).'),
  'D053770': NtO(MH='Nanowires', MS='Nanometer-scale wires made of materials that conduct electricity. They can be coated with molecules such as antibodies that will bind to proteins and other substances.'),
  'D053771': NtO(MH='Glicentin', MS='A 69-amino acid peptide derived from the N-terminal of PROGLUCAGON. It is mainly produced by the INTESTINAL L CELLS. Further processing of glicentin yield a 30-amino acid N-terminal peptide (glicentin-related polypeptide) and a 37-amino acid peptide OXYNTOMODULIN. Both glicentin and oxyntomodulin can reduce digestive secretions and delay gastric emptying.'),
  'D053772': NtO(MH='Oxyntomodulin', MS='A 37-amino acid peptide derived from the C-terminal of GLICENTIN. It is mainly produced by the INTESTINAL L CELLS. Oxyntomodulin can reduce digestive secretions, delay gastric emptying, and reduced food intake.'),
  'D053773': NtO(MH='Transforming Growth Factor beta1', MS='A subtype of transforming growth factor beta that is synthesized by a wide variety of cells. It is synthesized as a precursor molecule that is cleaved to form mature TGF-beta 1 and TGF-beta1 latency-associated peptide. The association of the cleavage products results in the formation a latent protein which must be activated to bind its receptor. Defects in the gene that encodes TGF-beta1 are the cause of CAMURATI-ENGELMANN SYNDROME.'),
  'D053778': NtO(MH='Thiazolidines', MS='Reduced (protonated) form of THIAZOLES. They can be oxidized to THIAZOLIDINEDIONES.'),
  'D053779': NtO(MH='Latent TGF-beta Binding Proteins', MS='A family of secreted multidomain proteins that were originally identified by their association with the latent form of TRANSFORMING GROWTH FACTORS. They interact with a variety of EXTRACELLULAR MATRIX PROTEINS and may play a role in the regulation of TGB-beta bioavailability.'),
  'D053781': NtO(MH='Transforming Growth Factor beta2', MS='A TGF-beta subtype that was originally identified as a GLIOBLASTOMA-derived factor which inhibits the antigen-dependent growth of both helper and CYTOTOXIC T LYMPHOCYTES. It is synthesized as a precursor molecule that is cleaved to form mature TGF-beta2 and TGF-beta2 latency-associated peptide. The association of the cleavage products results in the formation a latent protein which must be activated to bind its receptor.'),
  'D053782': NtO(MH='Transforming Growth Factor beta3', MS='A TGF-beta subtype that plays role in regulating epithelial-mesenchymal interaction during embryonic development. It is synthesized as a precursor molecule that is cleaved to form mature TGF-beta3 and TGF-beta3 latency-associated peptide. The association of the cleavage products results in the formation a latent protein which must be activated to bind its receptor.'),
  'D053783': NtO(MH='Magnetic Resonance Imaging, Interventional', MS='Minimally invasive procedures guided with the aid of magnetic resonance imaging to visualize tissue structures.'),
  'D053798': NtO(MH='Magnetocardiography', MS='The measurement of magnetic fields generated by electric currents from the heart. The measurement of these fields provides information which is complementary to that provided by ELECTROCARDIOGRAPHY.'),
  'D053799': NtO(MH='2-Pyridinylmethylsulfinylbenzimidazoles', MS='Compounds that contain benzimidazole joined to a 2-methylpyridine via a sulfoxide linkage. Several of the compounds in this class are ANTI-ULCER AGENTS that act by inhibiting the POTASSIUM HYDROGEN ATPASE found in the PROTON PUMP of GASTRIC PARIETAL CELLS.'),
  'D053800': NtO(MH='beta-Keratins', MS='Keratins that form into a beta-pleated sheet structure. They are principle constituents of the corneous material of the carapace and plastron of turtles, the epidermis of snakes and the feathers of birds.'),
  'D053801': NtO(MH='Capillary Electrochromatography', MS='A separation technique which combines LIQUID CHROMATOGRAPHY and CAPILLARY ELECTROPHORESIS.'),
  'D053802': NtO(MH='Tryptases', MS='A family of neutral serine proteases with TRYPSIN-like activity. Tryptases are primarily found in the SECRETORY GRANULES of MAST CELLS and are released during mast cell degranulation.'),
  'D053803': NtO(MH='Forensic Genetics', MS='The application of genetic analyses and MOLECULAR DIAGNOSTIC TECHNIQUES to legal matters and crime analysis.'),
  'D053804': NtO(MH='Granzymes', MS='A family of serine endopeptidases found in the SECRETORY GRANULES of LEUKOCYTES such as CYTOTOXIC T-LYMPHOCYTES and NATURAL KILLER CELLS. When secreted into the intercellular space granzymes act to eliminate transformed and virus-infected host cells.'),
  'D053805': NtO(MH='Fractional Flow Reserve, Myocardial', MS='The ratio of maximum blood flow to the MYOCARDIUM with CORONARY STENOSIS present, to the maximum equivalent blood flow without stenosis. The measurement is commonly used to verify borderline stenosis of CORONARY ARTERIES.'),
  'D053818': NtO(MH='Chymases', MS='A family of neutral serine proteases with CHYMOTRYPSIN-like activity. Chymases are primarily found in the SECRETORY GRANULES of MAST CELLS and are released during mast cell degranulation.'),
  'D053819': NtO(MH='Atelinae', MS='A subfamily in the family ATELIDAE, comprising three genera: woolly monkeys (Lagothrix), spider monkeys (Ateles), and woolly spider monkeys (Brachyteles).'),
  'D053820': NtO(MH='Pitheciidae', MS='A family of New World monkeys in the infraorder PLATYRRHINI consisting of two subfamilies: Callicebinae and Pitheciinae.'),
  'D053821': NtO(MH='Cardiovascular Infections', MS='Pathological conditions of the CARDIOVASCULAR SYSTEM caused by infections.'),
  'D053822': NtO(MH='Ailuridae', MS='A family in the suborder Caniformia, Order CARNIVORA, comprised of one genus Ailurus, the lesser pandas.'),
  'D053823': NtO(MH='Hedgehog Proteins', MS='A family of intercellular signaling proteins that play an important role in regulating the development of many TISSUES and organs. Their name derives from the observation of a hedgehog-like appearance in DROSOPHILA embryos with genetic mutations that block their action.'),
  'D053824': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 6', MS='A Src-homology domain-containing protein tyrosine phosphatase found in the CYTOSOL of hematopoietic cells. It plays a role in signal transduction by dephosphorylating signaling proteins that are activated or inactivated by PROTEIN-TYROSINE KINASES.'),
  'D053825': NtO(MH='Suburethral Slings', MS='Support structures, made from natural or synthetic materials, that are implanted below the URETHRA to treat URINARY STRESS INCONTINENCE.'),
  'D053826': NtO(MH='Eupleridae', MS='A family in the suborder Feliformia, order CARNIVORA, comprised of viverrid-like mammals.'),
  'D053827': NtO(MH='Nandiniidae', MS='A family in the suborder Feliformia, order CARNIVORA, comprising one genus Nandinia binotata.'),
  'D053828': NtO(MH='Stockings, Compression', MS='Tight coverings for the foot and leg that are worn to aid circulation in the legs, and prevent the formation of EDEMA and DEEP VEIN THROMBOSIS. PNEUMATIC COMPRESSION STOCKINGS serve a similar purpose especially for bedridden patients, and following surgery.'),
  'D053829': NtO(MH='Amyloid Precursor Protein Secretases', MS='Endopeptidases that are specific for AMYLOID PROTEIN PRECURSOR. Three secretase subtypes referred to as alpha, beta, and gamma have been identified based upon the region of amyloid protein precursor they cleave.'),
  'D053830': NtO(MH='Plant Viral Movement Proteins', MS='Viral proteins that facilitate the movement of viruses between plant cells by means of PLASMODESMATA, channels that traverse the plant cell walls.'),
  'D053831': NtO(MH='Surgical Fixation Devices', MS='Devices used to hold tissue structures together for repair, reconstruction or to close wounds. They may consist of adsorbable or non-adsorbable, natural or synthetic materials. They include tissue adhesives, skin tape, sutures, buttons, staples, clips, screws, etc., each designed to conform to various tissue geometries.'),
  'D053832': NtO(MH='Surgical Tape', MS='A flat, flexible strip of material used to cover or fasten together damaged tissue.'),
  'D053833': NtO(MH='Wetlands', MS='Environments or habitats at the interface between truly terrestrial ecosystems and truly aquatic systems making them different from each yet highly dependent on both. Adaptations to low soil oxygen characterize many wetland species.'),
  'D053834': NtO(MH='Explosive Agents', MS='Substances that are energetically unstable and can produce a sudden expansion of the material, called an explosion, which is accompanied by heat, pressure and noise. Other things which have been described as explosive that are not included here are explosive action of laser heating, human performance, sudden epidemiological outbreaks, or fast cell growth.'),
  'D053835': NtO(MH='Unilamellar Liposomes', MS='Single membrane vesicles, generally made of PHOSPHOLIPIDS.'),
  'D053836': NtO(MH='DNA Degradation, Necrotic', MS='The random catabolism of DNA accompanying the irreversible damage to tissue which leads to the pathological death of one or more cells.'),
  'D053837': NtO(MH='DNA Cleavage', MS='A reaction that severs one of the covalent sugar-phosphate linkages between NUCLEOTIDES that compose the sugar phosphate backbone of DNA. It is catalyzed enzymatically, chemically or by radiation. Cleavage may be exonucleolytic - removing the end nucleotide, or endonucleolytic - splitting the strand in two.'),
  'D053838': NtO(MH='Scattering, Small Angle', MS='Scattering of a beam of electromagnetic or acoustic RADIATION, or particles, at small angles by particles or cavities whose dimensions are many times as large as the wavelength of the radiation or the de Broglie wavelength of the scattered particles. Also know as low angle scattering. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed) Small angle scattering (SAS) techniques, small angle neutron (SANS), X-ray (SAXS), and light (SALS, or just LS) scattering, are used to characterize objects on a nanoscale.'),
  'D053839': NtO(MH='Begomovirus', MS='A genus of plant viruses in the family GEMINIVIRIDAE that are transmitted in nature by whitefly Bemisia tabaci.'),
  'D053840': NtO(MH='Brugada Syndrome', MS='An autosomal dominant defect of cardiac conduction that is characterized by an abnormal ST-segment in leads V1-V3 on the ELECTROCARDIOGRAM resembling a right BUNDLE-BRANCH BLOCK; high risk of VENTRICULAR TACHYCARDIA; or VENTRICULAR FIBRILLATION; SYNCOPAL EPISODE; and possible sudden death. This syndrome is linked to mutations of gene encoding the cardiac SODIUM CHANNEL alpha subunit.'),
  'D053841': NtO(MH='Electroosmosis', MS='The motion of a liquid through a membrane (or plug or capillary) consequent upon the application of an electric field across the membrane. (Oxford Dictionary of Biochemistry and Molecular Biology, 2001)'),
  'D053842': NtO(MH='Microsatellite Instability', MS='The occurrence of highly polymorphic mono- and dinucleotide MICROSATELLITE REPEATS in somatic cells. It is a form of genome instability associated with defects in DNA MISMATCH REPAIR.'),
  'D053843': NtO(MH='DNA Mismatch Repair', MS='A DNA repair pathway involved in correction of errors introduced during DNA replication when an incorrect base, which cannot form hydrogen bonds with the corresponding base in the parent strand, is incorporated into the daughter strand. Excinucleases recognize the BASE PAIR MISMATCH and cause a segment of polynucleotide chain to be excised from the daughter strand, thereby removing the mismatched base. (from Oxford Dictionary of Biochemistry and Molecular Biology, 2001)'),
  'D053844': NtO(MH='Lasers, Solid-State', MS='Lasers which use a solid, as opposed to a liquid or gas, as the lasing medium. Common materials used are crystals, such as YAG (YTTRIUM aluminum garnet); alexandrite; and CORUNDUM, doped with a rare earth element such as a NEODYMIUM; ERBIUM; or HOLMIUM. The output is sometimes additionally modified by addition of non-linear optical materials such as potassium titanyl phosphate crystal, which for example is used with neodymium YAG lasers to convert the output light to the visible range.'),
  'D053858': NtO(MH='Metabolic Networks and Pathways', MS='Complex sets of enzymatic reactions connected to each other via their product and substrate metabolites.'),
  'D053860': NtO(MH='Pallidotomy', MS='Producing a lesion in the posteroventral portion of the medial GLOBUS PALLIDUS to treat PARKINSON DISEASE and other extrapyramidal disorders. The placement of the lesion is aided by STEREOTACTIC TECHNIQUES and imaging procedures.'),
  'D053878': NtO(MH='Hemolytic Agents', MS='Substances that are toxic to blood in general, including the clotting mechanism; hematotoxins may refer to the hematopoietic system.'),
  'D053898': NtO(MH='Biosynthetic Pathways', MS='Sets of enzymatic reactions occurring in organisms and that form biochemicals by making new covalent bonds.'),
  'D053903': NtO(MH='DNA Breaks, Double-Stranded', MS='Interruptions in the sugar-phosphate backbone of DNA, across both strands adjacently.'),
  'D053904': NtO(MH='DNA Breaks, Single-Stranded', MS='Interruptions in one of the strands of the sugar-phosphate backbone of double-stranded DNA.'),
  'D053918': NtO(MH='Papillomavirus Vaccines', MS='Vaccines or candidate vaccines used to prevent PAPILLOMAVIRUS INFECTIONS. Human vaccines are intended to reduce the incidence of UTERINE CERVICAL NEOPLASMS, so they are sometimes considered a type of CANCER VACCINES. They are often composed of CAPSID PROTEINS, especially L1 protein, from various types of ALPHAPAPILLOMAVIRUS.'),
  'D053938': NtO(MH='DNA Fragmentation', MS='Splitting the DNA into shorter pieces by endonucleolytic DNA CLEAVAGE at multiple sites. It includes the internucleosomal DNA fragmentation, which along with chromatin condensation, are considered to be the hallmarks of APOPTOSIS.'),
  'D053958': NtO(MH='Dipterocarpaceae', MS='A plant family of the order ERICALES.'),
  'D053959': NtO(MH='Alpha-Amanitin', MS='A cyclic octapeptide with a thioether bridge between the cystine and tryptophan. It inhibits RNA POLYMERASE II. Poisoning may require LIVER TRANSPLANTATION.'),
  'D053960': NtO(MH='DNA Breaks', MS='Interruptions in the sugar-phosphate backbone of DNA.'),
  'D053961': NtO(MH='Azabicyclo Compounds', MS='Bicyclic bridged compounds that contain a nitrogen which has three bonds. The nomenclature indicates the number of atoms in each path around the rings, such as [2.2.2] for three equal length paths. Some members are TROPANES and BETA LACTAMS.'),
  'D053978': NtO(MH='Pentacyclic Triterpenes', MS='Five-ring derivatives of dammarane having a chair-chair-chair-boat configuration. They include the lupanes, oleananes, amyrins, GLYCYRRHIZIC ACID, and soyasaponins.'),
  'D053998': NtO(MH='Cyclopia Plant', MS='A plant genus of the family Fabaceae. Members contain cyclopamine, a teratogen producing cyclopia (one eye in the middle of the face) and XANTHONES.'),
  'D054000': NtO(MH='Nevus, Sebaceous of Jadassohn', MS='A syndrome characterized by lesions occurring on the face, scalp, or neck which consist of congenital hypoplastic malformations of cutaneous structures and which over time undergo verrucous hyperplasia. Additionally it is associated with neurological symptoms and skeletal, ophthalmological, urogenital, and cardiovascular abnormalities.'),
  'D054018': NtO(MH='Lasers, Excimer', MS='Gas lasers with excited dimers (i.e., excimers) as the active medium. The most commonly used are rare gas monohalides (e.g., argon fluoride, xenon chloride). Their principal emission wavelengths are in the ultraviolet range and depend on the monohalide used (e.g., 193 nm for ArF, 308 nm for Xe Cl). These lasers are operated in pulsed and Q-switched modes and used in photoablative decomposition involving actual removal of tissue. (UMDNS, 2005)'),
  'D054019': NtO(MH='Immune Reconstitution Inflammatory Syndrome', MS='Exuberant inflammatory response towards a previously acquired or undiagnosed opportunistic infection. It is frequently seen in AIDS patients following antiretroviral treatment.'),
  'D054020': NtO(MH='Lasers, Gas', MS='Lasers in which a gas lasing medium is stimulated to emit light by an electric current or high-frequency oscillator.'),
  'D054022': NtO(MH='Cerebrum', MS='Derived from TELENCEPHALON, cerebrum is composed of a right and a left hemisphere. Each contains an outer cerebral cortex and a subcortical basal ganglia. The cerebrum includes all parts within the skull except the MEDULLA OBLONGATA, the PONS, and the CEREBELLUM. Cerebral functions include sensorimotor, emotional, and intellectual activities.'),
  'D054023': NtO(MH='Lasers, Semiconductor', MS='Lasers with a semiconductor diode as the active medium. Diode lasers transform electric energy to light using the same principle as a light-emitting diode (LED), but with internal reflection capability, thus forming a resonator where a stimulated light can reflect back and forth, allowing only a certain wavelength to be emitted. The emission of a given device is determined by the active compound used (e.g., gallium arsenide crystals doped with aluminum or indium). Typical wavelengths are 810, 1,060 and 1,300 nm. (From UMDNS, 2005)'),
  'D054024': NtO(MH='Myelencephalon', MS='The most posterior portion of the hindbrain from which MEDULLA OBLONGATA is derived.'),
  'D054025': NtO(MH='Lasers, Dye', MS='Tunable liquid lasers with organic compounds (i.e., dye) which have a strong absorption band, used as the active medium. During emission, the dye has to be optically excited by another light source (e.g., another laser or flash lamp). The range of the emission wavelength may be anywhere from the ultraviolet to the near infrared (i.e., from 180 to 1100nm). These lasers are operated in continuous wave and pulsed modes. (UMDNS, 2005)'),
  'D054038': NtO(MH='Posterior Leukoencephalopathy Syndrome', MS='A condition that is characterized by HEADACHE; SEIZURES; and visual loss with edema in the posterior aspects of the CEREBRAL HEMISPHERES, such as the BRAIN STEM. Generally, lesions involve the white matter (nerve fibers) but occasionally the grey matter (nerve cell bodies).'),
  'D054039': NtO(MH='Onycholysis', MS='Separation of nail plate from the underlying nail bed. It can be a sign of skin disease, infection (such as ONYCHOMYCOSIS) or tissue injury.'),
  'D054041': NtO(MH='Weapons', MS='Devices or tools used in combat or fighting in order to kill or incapacitate.'),
  'D054042': NtO(MH='Bombs', MS='A weapon designed to explode when deployed. It frequently refers to a hollow case filled with EXPLOSIVE AGENTS.'),
  'D054043': NtO(MH='Nuclear Weapons', MS='A weapon that derives its destructive force from nuclear fission and/or fusion.'),
  'D054044': NtO(MH='Weapons of Mass Destruction', MS='Weapons that are capable of a high order of destruction and/or of being used to destroy large numbers of people. It includes NUCLEAR WEAPONS, and biological, chemical, and radiation weapons.'),
  'D054045': NtO(MH='Biological Warfare Agents', MS='Living organisms or their toxic products that are used to cause disease or death of humans during WARFARE.'),
  'D054046': NtO(MH='Plant Stomata', MS='Closable openings in the epidermis of plants on the underside of leaves. They allow the exchange of gases between the internal tissues of the plant and the outside atmosphere.'),
  'D054047': NtO(MH='Surgical Stomas', MS='Artificial openings created by a surgeon for therapeutic reasons. Most often this refers to openings from the GASTROINTESTINAL TRACT through the ABDOMINAL WALL to the outside of the body. It can also refer to the two ends of a surgical anastomosis.'),
  'D054048': NtO(MH='Peritoneal Stomata', MS='Natural openings in the subdiaphragmatic lymphatic plexus in the PERITONEUM, delimited by adjacent mesothelial cells. Peritoneal stomata constitute the principal pathways for the drainage of intraperitoneal contents from the PERITONEAL CAVITY to the LYMPHATIC SYSTEM.'),
  'D054058': NtO(MH='Acute Coronary Syndrome', MS='An episode of MYOCARDIAL ISCHEMIA that generally lasts longer than a transient anginal episode that ultimately may lead to MYOCARDIAL INFARCTION.'),
  'D054059': NtO(MH='Coronary Occlusion', MS='Complete blockage of blood flow through one of the CORONARY ARTERIES, usually from CORONARY ATHEROSCLEROSIS.'),
  'D054060': NtO(MH='Pulmonary Infarction', MS='NECROSIS of lung tissue that is cause by the lack of OXYGEN or blood supply. The most common cause of pulmonary infarction is a blood clot in the lung.'),
  'D054061': NtO(MH='Ischemic Contracture', MS='A type of permanent damage to muscles and nerves that results from prolonged lack blood flow to those tissues. It is characterized by shortening and stiffening of the muscles.'),
  'D054062': NtO(MH='Deaf-Blind Disorders', MS='The absence of both hearing and vision.'),
  'D054063': NtO(MH='Superior Sagittal Sinus', MS='The long large endothelium-lined venous channel on the top outer surface of the brain. It receives blood from a vein in the nasal cavity, runs backwards, and gradually increases in size as blood drains from veins of the brain and the DURA MATER. Near the lower back of the CRANIUM, the superior sagittal sinus deviates to one side (usually the right) and continues on as one of the TRANSVERSE SINUSES.'),
  'D054064': NtO(MH='Transverse Sinuses', MS='The two large endothelium-lined venous channels that begin at the internal occipital protuberance at the back and lower part of the CRANIUM and travels laterally and forward ending in the internal jugular vein (JUGULAR VEINS). One of the transverse sinuses, usually the right one, is the continuation of the SUPERIOR SAGITTAL SINUS. The other transverse sinus is the continuation of the straight sinus.'),
  'D054066': NtO(MH='Leukemia, Large Granular Lymphocytic', MS='A spectrum of disorders characterized by clonal expansions of the peripheral blood LYMPHOCYTE populations known as large granular lymphocytes which contain abundant cytoplasm and azurophilic granules. Subtypes develop from either CD3-negative NATURAL KILLER CELLS or CD3-positive T-CELLS. The clinical course of both subtypes can vary from spontaneous regression to progressive, malignant disease.'),
  'D054067': NtO(MH='Dihydropyrimidine Dehydrogenase Deficiency', MS='An autosomal recessive disorder affecting DIHYDROPYRIMIDINE DEHYDROGENASE and causing familial pyrimidinemia. It is characterized by thymine-uraciluria in homozygous deficient patients. Even a partial deficiency in the enzyme leaves individuals at risk for developing severe 5-FLUOROURACIL-associated toxicity.'),
  'D054068': NtO(MH='Livedo Reticularis', MS='A condition characterized by a reticular or fishnet pattern on the skin of lower extremities and other parts of the body. This red and blue pattern is due to deoxygenated blood in unstable dermal blood vessels. The condition is intensified by cold exposure and relieved by rewarming.'),
  'D054069': NtO(MH='Multiple Acyl Coenzyme A Dehydrogenase Deficiency', MS='An autosomal recessive disorder of fatty acid oxidation, and branched chain amino acids (AMINO ACIDS, BRANCHED-CHAIN); LYSINE; and CHOLINE catabolism, that is due to defects in either subunit of ELECTRON TRANSFER FLAVOPROTEIN or its dehydrogenase, electron transfer flavoprotein-ubiquinone oxidoreductase (EC 1.5.5.1).'),
  'D054070': NtO(MH='Postthrombotic Syndrome', MS='A condition caused by one or more episodes of DEEP VEIN THROMBOSIS, usually the blood clots are lodged in the legs. Clinical features include EDEMA; PAIN; aching; heaviness; and MUSCLE CRAMP in the leg. When severe leg swelling leads to skin breakdown, it is called venous STASIS ULCER.'),
  'D054078': NtO(MH='Mevalonate Kinase Deficiency', MS='Autosomal recessive disorder caused by mutations in the mevalonate kinase gene. Because of the mutations cholesterol biosynthesis is disrupted and MEVALONIC ACID accumulates. It is characterized by a range of symptoms, including dysmorphic FACIES, psychomotor retardation, CATARACT, hepatosplenomegaly, CEREBELLAR ATAXIA, elevated IMMUNOGLOBULIN D, and recurrent febrile crises with FEVER; LYMPHADENOPATHY; ARTHRALGIA; EDEMA; and rash.'),
  'D054079': NtO(MH='Vascular Malformations', MS='A spectrum of congenital, inherited, or acquired abnormalities in BLOOD VESSELS that can adversely affect the normal blood flow in ARTERIES or VEINS. Most are congenital defects such as abnormal communications between blood vessels (fistula), shunting of arterial blood directly into veins bypassing the CAPILLARIES (arteriovenous malformations), formation of large dilated blood blood-filled vessels (cavernous angioma), and swollen capillaries (capillary telangiectases). In rare cases, vascular malformations can result from trauma or diseases.'),
  'D054080': NtO(MH='Vein of Galen Malformations', MS='Congenital arteriovenous malformation involving the VEIN OF GALEN, a large deep vein at the base of the brain. The rush of arterial blood directly into the vein of Galen, without passing through the CAPILLARIES, can overwhelm the heart and lead to CONGESTIVE HEART FAILURE.'),
  'D054081': NtO(MH='Malformations of Cortical Development, Group II', MS='Cortical malformations secondary to abnormal neuronal CELL MIGRATION in NEUROGENESIS. This group includes COBBLESTONE LISSENCEPHALY and PERIVENTRICULAR NODULAR HETEROTOPIA.'),
  'D054082': NtO(MH='Lissencephaly', MS='A smooth brain malformation of the CEREBRAL CORTEX resulting from the abnormal location of developing neurons during corticogenesis. It is characterized by an absence of normal convoluted indentations on the surface of the brain (agyria), or fewer and shallower indentations (pachygryia). There is a reduced number of cortical layers, typically 4 instead of 6, resulting in a thickened cortex, and reduced cerebral white matter that is a reversal of the normal ratio of cerebral white matter to cortex.'),
  'D054083': NtO(MH='Ectopia Cordis', MS='A rare developmental defect in which the heart is abnormally located partially or totally outside the THORAX. It is the result of defective fusion of the anterior chest wall. Depending on the location of the heart, ectopia cordis can be thoracic, thoracoabdominal, abdominal, and cervical.'),
  'D054084': NtO(MH='Myocardial Bridging', MS='A malformation that is characterized by a muscle bridge over a segment of the CORONARY ARTERIES. Systolic contractions of the muscle bridge can lead to narrowing of coronary artery; coronary compression; MYOCARDIAL ISCHEMIA; MYOCARDIAL INFARCTION; and SUDDEN CARDIAC DEATH.'),
  'D054085': NtO(MH='Foramen Ovale', MS='An opening in the wall between the right and the left upper chambers (HEART ATRIA) of a fetal heart. Oval foramen normally closes soon after birth; when it fails to close the condition is called PATENT OVAL FORAMEN.'),
  'D054086': NtO(MH='KATP Channels', MS='Heteromultimers of Kir6 channels (the pore portion) and sulfonylurea receptor (the regulatory portion) which affect function of the HEART; PANCREATIC BETA CELLS; and KIDNEY COLLECTING DUCTS. KATP channel blockers include GLIBENCLAMIDE and mitiglinide whereas openers include CROMAKALIM and minoxidil sulfate.'),
  'D054087': NtO(MH='Atrial Septum', MS='The thin membrane-like muscular structure separating the right and the left upper chambers (HEART ATRIA) of a heart.'),
  'D054088': NtO(MH='Ventricular Septum', MS='The muscular structure separating the right and the left lower chambers (HEART VENTRICLES) of the heart. The ventricular septum consists of a very small membranous portion just beneath the AORTIC VALVE, and a large thick muscular portion consisting of three sections including the inlet septum, the trabecular septum, and the outlet septum.'),
  'D054089': NtO(MH='Endocardial Cushions', MS='A fetal heart structure that is the bulging areas in the cardiac septum between the HEART ATRIA and the HEART VENTRICLES. During development, growth and fusion of endocardial cushions at midline forms the two atrioventricular canals, the sites for future TRICUSPID VALVE and BICUSPID VALVE.'),
  'D054091': NtO(MH='Periventricular Nodular Heterotopia', MS='A disorder resulting from a defect in the pattern of neuronal migration in which ectopic collections of neurons lie along the lateral ventricles of the brain or just beneath, contiguously or in isolated patches.'),
  'D054092': NtO(MH='Foramen Ovale, Patent', MS='A condition in which the FORAMEN OVALE in the ATRIAL SEPTUM fails to close shortly after birth. This results in abnormal communications between the two upper chambers of the heart. An isolated patent ovale foramen without other structural heart defects is usually of no hemodynamic significance.'),
  'D054098': NtO(MH='Thrombocytopenia, Neonatal Alloimmune', MS='A condition in newborns caused by immunity of the mother to PLATELET ALLOANTIGENS on the fetal platelets. The PLATELETS, coated with maternal ANTIBODIES, are destroyed and removed by the fetal MONONUCLEAR PHAGOCYTE SYSTEM. Affected infants may have INTRACRANIAL HEMORRHAGES.'),
  'D054118': NtO(MH='Nanospheres', MS='Spherical particles of nanometer dimensions.'),
  'D054119': NtO(MH='Arachnodactyly', MS='An abnormal bone development that is characterized by extra long and slender hands and fingers, such that the clenched thumb extends beyond the ulnar side of the hand. Arachnodactyly can include feet and toes. Arachnodactyly has been associated with several gene mutations and syndromes.'),
  'D054120': NtO(MH='Phakic Intraocular Lenses', MS='Lenses, generally made of plastic or silicone, that are implanted into the eye in front of the natural EYE LENS, by the IRIS, to improve VISION, OCULAR. These intraocular lenses are used to supplement the natural lens instead of replacing it.'),
  'D054138': NtO(MH='Sinus Arrest, Cardiac', MS='The omission of atrial activation that is caused by transient cessation of impulse generation at the SINOATRIAL NODE. It is characterized by a prolonged pause without P wave in an ELECTROCARDIOGRAM. Sinus arrest has been associated with sleep apnea (REM SLEEP-RELATED SINUS ARREST).'),
  'D054139': NtO(MH='Tachycardia, Reciprocating', MS='Abnormally rapid heartbeats caused by reentrant conduction over the accessory pathways between the HEART ATRIA and the HEART VENTRICLES. The impulse can also travel in the reverse direction, as in some cases, atrial impulses travel to the ventricles over the accessory pathways and back to the atria over the BUNDLE OF HIS and the ATRIOVENTRICULAR NODE.'),
  'D054140': NtO(MH='Refractive Surgical Procedures', MS='Surgical procedures employed to correct REFRACTIVE ERRORS such as MYOPIA; HYPEROPIA; or ASTIGMATISM. These may involve altering the curvature of the CORNEA; removal or replacement of the CRYSTALLINE LENS; or modification of the SCLERA to change the axial length of the eye.'),
  'D054141': NtO(MH='Ventricular Flutter', MS='A potentially lethal cardiac arrhythmia characterized by an extremely rapid, hemodynamically unstable ventricular tachycardia (150-300 beats/min) with a large oscillating sine-wave appearance. If untreated, ventricular flutter typically progresses to VENTRICULAR FIBRILLATION.'),
  'D054143': NtO(MH='Heart Failure, Systolic', MS='Heart failure caused by abnormal myocardial contraction during SYSTOLE leading to defective cardiac emptying.'),
  'D054144': NtO(MH='Heart Failure, Diastolic', MS='Heart failure caused by abnormal myocardial relaxation during DIASTOLE leading to defective cardiac filling.'),
  'D054158': NtO(MH='Orthokeratologic Procedures', MS='An alternative to REFRACTIVE SURGICAL PROCEDURES. A therapeutic procedure for correcting REFRACTIVE ERRORS. It involves wearing CONTACT LENSES designed to force corrective changes to the curvature of the CORNEA that remain after the lenses are removed. The effect is temporary but is maintained by wearing the therapeutic lenses daily, usually during sleep.'),
  'D054159': NtO(MH='Torsion, Mechanical', MS='A twisting deformation of a solid body about an axis. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D054160': NtO(MH='Systolic Murmurs', MS='Heart murmurs which are systolic in timing. They occur between the first and the second HEART SOUNDS, between the closure of MITRAL VALVE and TRICUSPID VALVE and the closure of semilunar aortic and pulmonary valves. Systolic murmurs include ejection murmurs and regurgitant murmurs.'),
  'D054179': NtO(MH='Angioedemas, Hereditary', MS='Inherited disorders that are characterized by subcutaneous and submucosal EDEMA in the upper RESPIRATORY TRACT and GASTROINTESTINAL TRACT.'),
  'D054198': NtO(MH='Precursor Cell Lymphoblastic Leukemia-Lymphoma', MS='A neoplasm characterized by abnormalities of the lymphoid cell precursors leading to excessive lymphoblasts in the marrow and other organs. It is the most common cancer in children and accounts for the vast majority of all childhood leukemias.'),
  'D054199': NtO(MH='Pseudoephedrine', MS='A phenethylamine that is an isomer of EPHEDRINE which has less central nervous system effects and usage is mainly for respiratory tract decongestion.'),
  'D054218': NtO(MH='Precursor T-Cell Lymphoblastic Leukemia-Lymphoma', MS='A leukemia/lymphoma found predominately in children and young adults and characterized LYMPHADENOPATHY and THYMUS GLAND involvement. It most frequently presents as a lymphoma, but a leukemic progression in the bone marrow is common.'),
  'D054219': NtO(MH='Neoplasms, Plasma Cell', MS='Neoplasms associated with a proliferation of a single clone of PLASMA CELLS and characterized by the secretion of PARAPROTEINS.'),
  'D054220': NtO(MH='Malformations of Cortical Development', MS='Abnormalities in the development of the CEREBRAL CORTEX. These include malformations arising from abnormal neuronal and glial CELL PROLIFERATION or APOPTOSIS (Group I); abnormal neuronal migration (Group II); and abnormal establishment of cortical organization (Group III). Many INBORN METABOLIC BRAIN DISORDERS affecting CNS formation are often associated with cortical malformations. They are common causes of EPILEPSY and developmental delay.'),
  'D054221': NtO(MH='Classical Lissencephalies and Subcortical Band Heterotopias', MS='Disorders comprising a spectrum of brain malformations representing the paradigm of a diffuse neuronal migration disorder. They result in cognitive impairment; SEIZURES; and HYPOTONIA or spasticity. Mutations of two genes, LIS1, the gene for the non-catalytic subunit of PLATELET-ACTIVATING FACTOR ACETYLHYDROLASE IB; and DCX or XLIS, the gene for doublecortin, have been identified as the most common causes of disorders in this spectrum. Additional variants of classical (Type I) lissencephaly have been linked to RELN, the gene for reelin, and ARX, the gene for aristaless related homeobox protein. (From Leventer, R.J., et al, Mol Med Today. 2000 Jul;6(7):277-84 and Barkovich, A.J., et al, Neurology. 2005 Dec 27;65(12):1873-87.)'),
  'D054222': NtO(MH='Cobblestone Lissencephaly', MS='The smooth pebbled appearance of the CEREBRAL CORTEX with a thickened cortex and reduced and abnormal white matter, which results from migration of heterotopic neurons beyond the marginal zone into the leptomeninges through gaps in the external BASEMENT MEMBRANE. There is also enlarged ventricles, underdeveloped BRAINSTEM and cerebellum, and absence of the CORPUS CALLOSUM. These abnormalities occur as a syndrome without other birth defects (cobblestone complex) or in other syndromes associated with congenital MUSCULAR DYSTROPHY, often involving the eye, such as the Walker-Warburg Syndrome, Fukuyama congenital muscular dystrophy, and muscle-eye-brain disease.'),
  'D054238': NtO(MH='Perinatal Mortality', MS='Deaths occurring from the 28th week of GESTATION to the 28th day after birth in a given population.'),
  'D054239': NtO(MH='Blastodisc', MS='A small whitish spot on the surface of the EGG YOLK where cleavage begins. Upon fertilization the cytoplasm streams from the vegetal pole away from the yolk to the animal pole where cleavage will occur. This germinal area eventually flattens into a layer of cells (BLASTODERM) that covers the yolk completely.'),
  'D054240': NtO(MH='Primitive Streak', MS='A linear band of rapidly proliferating cells that begins near the posterior end of an embryo and grows cranially. Primitive streak is formed during GASTRULATION by the convergent migration of primary ectodermal cells (EPIBLAST). The knot at the tip of the streak is called HENSEN NODE.'),
  'D054242': NtO(MH='Emigrants and Immigrants', MS='People who leave their place of residence in one country and settle in a different country.'),
  'D054243': NtO(MH='Vesicular Stomatitis', MS='A viral disease caused by at least two distinct species (serotypes) in the VESICULOVIRUS genus: VESICULAR STOMATITIS INDIANA VIRUS and VESICULAR STOMATITIS NEW JERSEY VIRUS. It is characterized by vesicular eruptions on the ORAL MUCOSA in cattle, horses, pigs, and other animals. In humans, vesicular stomatitis causes an acute influenza-like illness.'),
  'D054258': NtO(MH='Neural Plate', MS='The region in the dorsal ECTODERM of a chordate embryo that gives rise to the future CENTRAL NERVOUS SYSTEM. Tissue in the neural plate is called the neuroectoderm, often used as a synonym of neural plate.'),
  'D054259': NtO(MH='Neural Tube', MS='A tube of ectodermal tissue in an embryo that will give rise to the CENTRAL NERVOUS SYSTEM, including the SPINAL CORD and the BRAIN. Lumen within the neural tube is called neural canal which gives rise to the central canal of the spinal cord and the ventricles of the brain. For malformation of the neural tube, see NEURAL TUBE DEFECTS.'),
  'D054260': NtO(MH='Vesicular stomatitis New Jersey virus', MS='A species of VESICULOVIRUS causing VESICULAR STOMATITIS primarily in cattle, horses, and pigs. It can be transmitted to humans where it causes influenza-like symptoms.'),
  'D054261': NtO(MH='Neurulation', MS='An early embryonic developmental process of CHORDATES that is characterized by morphogenic movements of ECTODERM resulting in the formation of the NEURAL PLATE; the NEURAL CREST; and the NEURAL TUBE. Improper closure of the NEURAL GROOVE results in congenital NEURAL TUBE DEFECTS.'),
  'D054262': NtO(MH='Gastrulation', MS='A process of complicated morphogenetic cell movements that reorganizes a bilayer embryo into one with three GERM LAYERS and specific orientation (dorsal/ventral; anterior/posterior). Gastrulation describes the germ layer development of a non-mammalian BLASTULA or that of a mammalian BLASTOCYST.'),
  'D054278': NtO(MH='Embryonal Carcinoma Stem Cells', MS='The malignant stem cells of TERATOCARCINOMAS, which resemble pluripotent stem cells of the BLASTOCYST INNER CELL MASS. The EC cells can be grown in vitro, and experimentally induced to differentiate. They are used as a model system for studying early embryonic cell differentiation.'),
  'D054298': NtO(MH='Human Immunodeficiency Virus Proteins', MS='Proteins synthesized by HUMAN IMMUNODEFICIENCY VIRUSES such as the HIV-1 and HIV-2.'),
  'D054299': NtO(MH='env Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the ENV GENE of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054300': NtO(MH='Oscillatoria', MS='A genus of filamentous CYANOBACTERIA in the order Oscillatoriales. It is commonly found in freshwater environments, especially hot springs.'),
  'D054301': NtO(MH='gag Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the GAG GENE of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054302': NtO(MH='pol Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the POL GENE of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054303': NtO(MH='HIV Reverse Transcriptase', MS='A reverse transcriptase encoded by the POL GENE of HIV. It is a heterodimer of 66 kDa and 51 kDa subunits that are derived from a common precursor protein. The heterodimer also includes an RNAse H activity (RIBONUCLEASE H, HUMAN IMMUNODEFICIENCY VIRUS) that plays an essential role the viral replication process.'),
  'D054304': NtO(MH='Anti-Mullerian Hormone', MS='A glycoprotein that causes regression of MULLERIAN DUCTS. It is produced by SERTOLI CELLS of the TESTES. In the absence of this hormone, the Mullerian ducts develop into structures of the female reproductive tract. In males, defects of this hormone result in persistent Mullerian duct, a form of MALE PSEUDOHERMAPHRODITISM.'),
  'D054306': NtO(MH='Dideoxynucleotides', MS='The phosphate esters of DIDEOXYNUCLEOSIDES.'),
  'D054307': NtO(MH='Enterotoxigenic Escherichia coli', MS='Strains of ESCHERICHIA COLI that produce or contain at least one member of either heat-labile or heat-stable ENTEROTOXINS. The organisms colonize the mucosal surface of the small intestine and elaborate their enterotoxins causing DIARRHEA. They are mainly associated with tropical and developing countries and affect susceptible travelers to those places.'),
  'D054308': NtO(MH='Enteropathogenic Escherichia coli', MS='Strains of ESCHERICHIA COLI characterized by attaching-and-effacing histopathology. These strains of bacteria intimately adhere to the epithelial cell membrane and show effacement of microvilli. In developed countries they are associated with INFANTILE DIARRHEA and infantile GASTROENTERITIS and, in contrast to ETEC strains, do not produce ENDOTOXINS.'),
  'D054309': NtO(MH='Ribonuclease H, Human Immunodeficiency Virus', MS='A ribonuclease activity that is a component of the HIV REVERSE TRANSCRIPTASE. It removes the RNA strand of the RNA-DNA heteroduplex produced by reverse transcription. Once the RNA moiety is removed a double stranded DNA copy of the HIV RNA can be synthesized.'),
  'D054311': NtO(MH='nef Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the NEF GENES of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054312': NtO(MH='Buruli Ulcer', MS='A lesion in the skin and subcutaneous tissues due to infections by MYCOBACTERIUM ULCERANS. It was first reported in Uganda, Africa.'),
  'D054313': NtO(MH='Drug Agonism', MS='Phenomena and pharmaceutics of compounds that selectively bind to a specific receptor and trigger a response. They mimic the action of endogenous biochemical molecules. Their effect can be countered by antagonists (DRUG ANTAGONISM).'),
  'D054314': NtO(MH='Drug Inverse Agonism', MS='Phenomena and pharmaceutics of compounds that bind to the same receptor binding-site as an agonist (DRUG AGONISM) for that receptor but exerts the opposite pharmacological effect.'),
  'D054315': NtO(MH='Oocyte Retrieval', MS='Procedures to obtain viable OOCYTES from the host. Oocytes most often are collected by needle aspiration from OVARIAN FOLLICLES before OVULATION.'),
  'D054316': NtO(MH='Biomarkers, Pharmacological', MS='Measurable biological parameters that serve for drug development, safety and dosing (DRUG MONITORING).'),
  'D054318': NtO(MH='No-Reflow Phenomenon', MS='Markedly reduced or absent REPERFUSION in an infarct zone following the removal of an obstruction or constriction of an artery.'),
  'D054319': NtO(MH='Marchiafava-Bignami Disease', MS='A neurodegenerative condition that is characterized by demyelination or necrosis of the CORPUS CALLOSUM. Symptoms include DEPRESSION; PARANOIA; DEMENTIA; SEIZURES; and ATAXIA which can progress to COMA and death in a few months. Marchiafava-Bignami syndrome is seen often in alcoholics but has been found in non-alcoholics as well.'),
  'D054320': NtO(MH='vif Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the VIF GENES of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054321': NtO(MH='rev Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the REV GENES of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054322': NtO(MH='tat Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the TAT GENES of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054323': NtO(MH='Shiga-Toxigenic Escherichia coli', MS='Strains of ESCHERICHIA COLI with the ability to produce at least one or more of at least two antigenically distinct, usually bacteriophage-mediated cytotoxins: SHIGA TOXIN 1 and SHIGA TOXIN 2. These bacteria can cause severe disease in humans including bloody DIARRHEA and HEMOLYTIC UREMIC SYNDROME.'),
  'D054324': NtO(MH='Enterohemorrhagic Escherichia coli', MS='Strains of ESCHERICHIA COLI that are a subgroup of SHIGA-TOXIGENIC ESCHERICHIA COLI. They cause non-bloody and bloody DIARRHEA; HEMOLYTIC UREMIC SYNDROME; and hemorrhagic COLITIS. An important member of this subgroup is ESCHERICHIA COLI O157-H7.'),
  'D054325': NtO(MH='vpr Gene Products, Human Immunodeficiency Virus', MS='Proteins encoded by the VPR GENES of the HUMAN IMMUNODEFICIENCY VIRUS.'),
  'D054326': NtO(MH='Coronary Sinus', MS='A short vein that collects about two thirds of the venous blood from the MYOCARDIUM and drains into the RIGHT ATRIUM. Coronary sinus, normally located between the LEFT ATRIUM and LEFT VENTRICLE on the posterior surface of the heart, can serve as an anatomical reference for cardiac procedures.'),
  'D054327': NtO(MH='Lubricants', MS='Compounds that provide LUBRICATION between surfaces in order to reduce FRICTION.'),
  'D054328': NtO(MH='Proton Pump Inhibitors', MS='Compounds that inhibit H(+)-K(+)-EXCHANGING ATPASE. They are used as ANTI-ULCER AGENTS and sometimes in place of HISTAMINE H2 ANTAGONISTS for GASTROESOPHAGEAL REFLUX.'),
  'D054329': NtO(MH='Carbasugars', MS='Sugar analogs in which the ring oxygen is replaced by a methylene (CH2) carbon.'),
  'D054330': NtO(MH='Thiosugars', MS='Sugar analogs in which the ring oxygen is replaced by a sulfur.'),
  'D054331': NtO(MH='Sertoli Cell-Only Syndrome', MS='A type of male infertility in which no germ cells are visible in any of the biopsied SEMINIFEROUS TUBULES (type I) or in which germ cells are present in a minority of tubules (type II). Clinical features include AZOOSPERMIA, normal VIRILIZATION, and normal chromosomal complement.'),
  'D054332': NtO(MH='Leishmaniasis Vaccines', MS='Vaccines or candidate vaccines used to prevent infection with LEISHMANIA.'),
  'D054333': NtO(MH='Drug Partial Agonism', MS='Drug agonism involving selective binding but reduced effect. This can result in some degree of DRUG ANTAGONISM.'),
  'D054334': NtO(MH='Viral Regulatory and Accessory Proteins', MS='A broad category of viral proteins that play indirect roles in the biological processes and activities of viruses. Included here are proteins that either regulate the expression of viral genes or are involved in modifying host cell functions. Many of the proteins in this category serve multiple functions.'),
  'D054335': NtO(MH='Goniothalamus', MS='A plant genus of the family ANNONACEAE. Members contain cyclopeptides and styrylpyrones.'),
  'D054336': NtO(MH='Xylopia', MS='A plant genus of the family ANNONACEAE. Members contain DITERPENES.'),
  'D054337': NtO(MH='Cell Dedifferentiation', MS='A reverse developmental process in which terminally differentiated cells with specialized functions revert back to a less differentiated stage within their own CELL LINEAGE.'),
  'D054338': NtO(MH='Cell Transdifferentiation', MS='A naturally occurring phenomenon where terminally differentiated cells dedifferentiate to the point where they can switch CELL LINEAGES. The cells then differentiate into other cell types.'),
  'D054339': NtO(MH='Steroidogenic Factor 1', MS='A transcription factor and member of the nuclear receptor family NR5 that is expressed throughout the adrenal and reproductive axes during development. It plays an important role in sexual differentiation, formation of primary steroidogenic tissues, and their functions in post-natal and adult life. It regulates the expression of key steroidogenic enzymes.'),
  'D054340': NtO(MH='Receptors, KIR', MS='A family of receptors found on NK CELLS that have specificity for a variety of HLA ANTIGENS. KIR receptors contain up to three different extracellular immunoglobulin-like domains referred to as D0, D1, and D2 and play an important role in blocking NK cell activation against cells expressing the appropriate HLA antigens thus preventing cell lysis. Although they are often referred to as being inhibitory receptors, a subset of KIR receptors may also play an activating role in NK cells.'),
  'D054341': NtO(MH='Receptors, KIR3DL1', MS='A KIR receptor that has specificity for HLA-B ANTIGENS. It is an inhibitory receptor that contains D0, D1, and D2 extracellular immunoglobulin-like domains and a long cytoplasmic tail.'),
  'D054342': NtO(MH='Receptors, KIR2DL1', MS='A KIR receptor that has specificity for HLA-C ANTIGENS. It is an inhibitory receptor that contains D1 and D2 extracellular immunoglobulin-like domains and a long cytoplasmic tail. It is similar in structure and function to the KIR2DL2 RECEPTOR and the KIR2DL3 RECEPTORS.'),
  'D054343': NtO(MH='Receptors, KIR2DL2', MS='A KIR receptor that has specificity for HLA-C ANTIGENS. It is an inhibitory receptor that contains D1 and D2 extracellular immunoglobulin-like domains and a long cytoplasmic tail. It is similar in structure and function to the KIR2DL1 RECEPTORS and the KIR2DL3 RECEPTORS.'),
  'D054344': NtO(MH='Receptors, KIR2DL3', MS='A KIR receptor that has specificity for HLA-C ANTIGEN. It is an inhibitory receptor that contains D1 and D2 extracellular immunoglobulin-like domains and a long cytoplasmic tail. It is similar in structure and function to the KIR2DL2 RECEPTORS and the KIR2DL3 RECEPTORS.'),
  'D054345': NtO(MH='Receptors, KIR2DL4', MS='A KIR receptor that has specificity for HLA-G antigen. It contains D0 and D2 extracellular immunoglobulin-like domains and a long cytoplasmic tail.'),
  'D054346': NtO(MH='Receptors, KIR2DL5', MS='An inhibitory KIR receptor that contains D0 and D1 extracellular immunoglobulin-like domains and a long cytoplasmic tail.'),
  'D054347': NtO(MH='Receptors, KIR3DL2', MS='A KIR receptor that has specificity for HLA-A3 ANTIGEN. It is an inhibitory receptor that contains D0, D1, and D2 extracellular immunoglobulin-like domains and a long cytoplasmic tail.'),
  'D054349': NtO(MH='Receptors, KIR3DS1', MS='An activating KIR receptor that contains D0, D1, and D2 extracellular immunoglobulin-like domains and a short cytoplasmic tail.'),
  'D054351': NtO(MH='Electrical Synapses', MS='Specialized junctions between NEURONS which connect the cytoplasm of one neuron to another allowing direct passage of an ion current.'),
  'D054353': NtO(MH='Perforin', MS='A calcium-dependent pore-forming protein synthesized in cytolytic LYMPHOCYTES and sequestered in secretory granules. Upon immunological reaction between a cytolytic lymphocyte and a target cell, perforin is released at the plasma membrane and polymerizes into transmembrane tubules (forming pores) which lead to death of a target cell.'),
  'D054354': NtO(MH='Iron Carbonyl Compounds', MS='Complexes of iron atoms chelated with carbonyl ions.'),
  'D054357': NtO(MH='Chemokine CXCL10', MS='A CXC chemokine that is induced by GAMMA-INTERFERON and is chemotactic for MONOCYTES and T-LYMPHOCYTES. It has specificity for the CXCR3 RECEPTOR.'),
  'D054358': NtO(MH='Withanolides', MS='Ergostane derivatives of 28 carbons with oxygens at C1, C22, and C26 positions and the side chain cyclized. They are found in WITHANIA plant genus and have cytotoxic and other effects.'),
  'D054360': NtO(MH='Chemokine CXCL1', MS='A CXC chemokine with specificity for CXCR2 RECEPTORS. It has growth factor activities and is implicated as a oncogenic factor in several tumor types.'),
  'D054361': NtO(MH='Fenamates', MS='Derivatives of orthoaminobenzoic acid that have a phenyl group bound to the orthoamino nitrogen. Members modulate ION CHANNELS and are used as ANTI-INFLAMMATORY AGENTS.'),
  'D054362': NtO(MH='Cananga', MS='A plant genus of the family ANNONACEAE known for its aromatic oil (OILS, VOLATILE).'),
  'D054363': NtO(MH='Solitary Fibrous Tumor, Pleural', MS='A rare neoplasm, usually benign, derived from mesenchymal fibroblasts located in the submesothelial lining of the PLEURA. It spite of its various synonyms, it has no features of mesothelial cells and is not related to malignant MESOTHELIOMA or asbestos exposure.'),
  'D054364': NtO(MH='Solitary Fibrous Tumors', MS='Rare neoplasms of mesenchymal origin, usually benign, and most commonly involving the PLEURA (see SOLITARY FIBROUS TUMOR, PLEURAL). They also are found in extrapleural sites.'),
  'D054365': NtO(MH='Chemokine CXCL5', MS='A CXC chemokine that is predominantly expressed in EPITHELIAL CELLS. It has specificity for the CXCR2 RECEPTORS and is involved in the recruitment and activation of NEUTROPHILS.'),
  'D054366': NtO(MH='Agouti Signaling Protein', MS='A secreted protein of approximately 131 amino acids (depending on species) that regulates the synthesis of eumelanin (brown/black) pigments in MELANOCYTES. Agouti protein antagonizes the signaling of MELANOCORTIN RECEPTORS and has wide distribution including ADIPOSE TISSUE; GONADS; and HEART. Its overexpression in agouti mice results in uniform yellow coat color, OBESITY, and metabolic defects similar to type II diabetes in humans.'),
  'D054367': NtO(MH='Receptors, CXCR3', MS='CXCR receptors that are expressed on the surface of a number of cell types, including T-LYMPHOCYTES; NK CELLS; DENDRITIC CELLS; and a subset of B-LYMPHOCYTES. The receptors are activated by CHEMOKINE CXCL9; CHEMOKINE CXCL10; and CHEMOKINE CXCL11.'),
  'D054368': NtO(MH='Laxatives', MS='Agents that produce a soft formed stool, and relax and loosen the bowels, typically used over a protracted period, to relieve CONSTIPATION.'),
  'D054369': NtO(MH='Agouti-Related Protein', MS='A secreted protein of approximately 131 amino acids that is related to AGOUTI SIGNALING PROTEIN and is also an antagonist of MELANOCORTIN RECEPTOR activity. It is expressed primarily in the HYPOTHALAMUS and the ADRENAL GLAND. As a paracrine signaling molecule, AGRP is known to regulate food intake and body weight. Elevated AGRP has been associated with OBESITY.'),
  'D054370': NtO(MH='Chemokine CXCL9', MS='An INTEFERON-inducible CXC chemokine that is specific for the CXCR3 RECEPTOR.'),
  'D054371': NtO(MH='Chemokine CXCL11', MS='A CXC chemokine that is induced by GAMMA-INTERFERON. It is a chemotactic factor for activated T-LYMPHOCYTES and has specificity for the CXCR3 RECEPTOR.'),
  'D054377': NtO(MH='Chemokine CXCL12', MS='A CXC chemokine that is chemotactic for T-LYMPHOCYTES and MONOCYTES. It has specificity for CXCR4 RECEPTORS. Two isoforms of CXCL12 are produced by alternative mRNA splicing.'),
  'D054378': NtO(MH='Acetogenins', MS='Polyketides of up to a few dozen carbons in length, formed by chain extension of multiple PROPIONATES and oxygenated to form tetrahydrofuran and lactone rings along the length of the chain. They are found in ANNONACEAE and other PLANTS. Related compounds cyclize to MACROLIDES.'),
  'D054380': NtO(MH='Receptors, CXCR5', MS='CXCR receptors isolated initially from BURKITT LYMPHOMA cells. CXCR5 receptors are expressed on mature, recirculating B-LYMPHOCYTES and are specific for CHEMOKINE CXCL13.'),
  'D054382': NtO(MH='Chemokine CXCL13', MS='A CXC chemokine that is chemotactic for B-LYMPHOCYTES. It has specificity for CXCR5 RECEPTORS.'),
  'D054387': NtO(MH='Receptors, CXCR', MS='Chemokine receptors that are specific for CXC CHEMOKINES.'),
  'D054388': NtO(MH='Receptors, CCR', MS='Chemokine receptors that are specific for CC CHEMOKINES.'),
  'D054389': NtO(MH='Receptors, CCR1', MS='CCR receptors with specificity for a broad variety of CC CHEMOKINES. They are expressed at high levels in MONOCYTES; tissue MACROPHAGES; NEUTROPHILS; and EOSINOPHILS.'),
  'D054390': NtO(MH='Receptors, CCR2', MS='CCR receptors with specificity for CHEMOKINE CCL2 and several other CCL2-related chemokines. They are expressed at high levels in T-LYMPHOCYTES; B-LYMPHOCYTES; MACROPHAGES; BASOPHILS; and NK CELLS.'),
  'D054391': NtO(MH='Lymphoma, Extranodal NK-T-Cell', MS='An extranodal neoplasm, usually possessing an NK-cell phenotype and associated with EPSTEIN-BARR VIRUS. These lymphomas exhibit a broad morphologic spectrum, frequent necrosis, angioinvasion, and most commonly present in the midfacial region, but also in other extranodal sites.'),
  'D054392': NtO(MH='Adipokines', MS='Polypeptides produced by the ADIPOCYTES. They include LEPTIN; ADIPONECTIN; RESISTIN; and many cytokines of the immune system, such as TUMOR NECROSIS FACTOR-ALPHA; INTERLEUKIN-6; and COMPLEMENT FACTOR D (also known as ADIPSIN). They have potent autocrine, paracrine, and endocrine functions.'),
  'D054397': NtO(MH='Receptors, CCR3', MS='CCR receptors with specificity for CHEMOKINE CCL11 and a variety of other CC CHEMOKINES. They are expressed at high levels in T-LYMPHOCYTES; EOSINOPHILS; BASOPHILS; and MAST CELLS.'),
  'D054398': NtO(MH='Receptors, CCR4', MS='CCR receptors with specificity for CHEMOKINE CCL17 and CHEMOKINE CCL22. They are expressed at high levels in T-LYMPHOCYTES; MAST CELLS; DENDRITIC CELLS; and NK CELLS.'),
  'D054399': NtO(MH='Receptors, CCR6', MS='CCR receptors with specificity for CHEMOKINE CCL20. They are expressed at high levels in T-LYMPHOCYTES; B-LYMPHOCYTES; and DENDRITIC CELLS.'),
  'D054400': NtO(MH='Receptors, CCR7', MS='CCR receptors with specificity for CHEMOKINE CCL19 and CHEMOKINE CCL21. They are expressed at high levels in T-LYMPHOCYTES; B-LYMPHOCYTES; and DENDRITIC CELLS.'),
  'D054401': NtO(MH='Receptors, CCR8', MS='CCR receptors with specificity for CHEMOKINE CCL1. They are expressed at high levels in T-LYMPHOCYTES; B-LYMPHOCYTES; and MACROPHAGES.'),
  'D054402': NtO(MH='Chemokine CCL1', MS='A CC-type chemokine secreted by activated MONOCYTES and T-LYMPHOCYTES. It has specificity for CCR8 RECEPTORS.'),
  'D054403': NtO(MH='Leukemia, Prolymphocytic, B-Cell', MS='A neoplasm of prolymphocytes affecting the blood, bone marrow, and spleen. It is characterized by prolymphocytes exceeding 55% of the lymphoid cells in the blood and profound splenomegaly.'),
  'D054405': NtO(MH='Chemokine CCL3', MS='A CC chemokine with specificity for CCR1 RECEPTORS and CCR5 RECEPTORS. It is a chemoattractant for NK CELLS; MONOCYTES; and a variety of other immune cells. This chemokine is encoded by multiple genes.'),
  'D054406': NtO(MH='Pseudomonas Vaccines', MS='Vaccines or candidate vaccines used to prevent or treat PSEUDOMONAS INFECTIONS.'),
  'D054407': NtO(MH='Chemokine CCL4', MS='A CC chemokine with specificity for CCR5 RECEPTORS. It is a chemoattractant for NK CELLS; MONOCYTES and a variety of other immune cells. This chemokine is encoded by multiple genes.'),
  'D054409': NtO(MH='Nicotinamide Phosphoribosyltransferase', MS='An enzyme that catalyzes the formation of nicotinamide mononucleotide (NMN) from nicotinamide and 5-phosphoribosyl-1-pyrophosphate, the rate-limiting step in the biosynthesis of the NAD coenzyme. It is also known as a growth factor for early B-LYMPHOCYTES, or an ADIPOKINE with insulin-mimetic effects (visfatin).'),
  'D054410': NtO(MH='Chemokine CCL7', MS='A monocyte chemoattractant protein that has activity towards a broad variety of immune cell types. Chemokine CCL7 has specificity for CCR1 RECEPTORS; CCR2 RECEPTORS; and CCR5 RECEPTORS.'),
  'D054411': NtO(MH='Receptors, Leptin', MS='Cell surface receptors for obesity factor (LEPTIN), a hormone secreted by the WHITE ADIPOCYTES. Upon leptin-receptor interaction, the signal is mediated through the JAK2/STAT3 pathway to regulate food intake, energy balance and fat storage.'),
  'D054412': NtO(MH='Chemokine CCL8', MS='A monocyte chemoattractant protein that attracts MONOCYTES; LYMPHOCYTES; BASOPHILS; and EOSINOPHILS. Chemokine CCL8 has specificity for CCR3 RECEPTORS and CCR5 RECEPTORS.'),
  'D054413': NtO(MH='Chemokine CCL11', MS='A CC-type chemokine that is specific for CCR3 RECEPTORS. It is a potent chemoattractant for EOSINOPHILS.'),
  'D054414': NtO(MH='Chemokine CCL17', MS='A CC-type chemokine that is found at high levels in the THYMUS and has specificity for CCR4 RECEPTORS. It is synthesized by DENDRITIC CELLS; ENDOTHELIAL CELLS; KERATINOCYTES; and FIBROBLASTS.'),
  'D054415': NtO(MH='Chemokine CCL19', MS='A CC-type chemokine with specificity for CCR7 RECEPTORS. It has activity towards T LYMPHOCYTES and B LYMPHOCYTES.'),
  'D054417': NtO(MH='Receptors, Adipokine', MS='Cell surface receptors for ADIPOKINES, cytokines secreted by the ADIPOCYTES.'),
  'D054418': NtO(MH='Chemokine CCL20', MS='A CC-type chemokine with specificity for CCR6 RECEPTORS. It has activity towards DENDRITIC CELLS; T-LYMPHOCYTES; and B-LYMPHOCYTES.'),
  'D054419': NtO(MH='Receptors, Adiponectin', MS='Cell surface receptors for ADIPONECTIN, an antidiabetic hormone secreted by ADIPOCYTES. Adiponectin receptors are membrane proteins with multiple cytoplasmic and extracellular regions. They are about 43 kDa and encoded by at least two genes with different affinities for globular and full-length adiponectin.'),
  'D054420': NtO(MH='Pre-B Cell Receptors', MS='Membrane proteins in precursor B-LYMPHOCYTES (pre-B Cells). They are composed of membrane-bound MU IMMUNOGLOBULIN HEAVY CHAINS in complex with SURROGATE LIGHT CHAINS instead of conventional IMMUNOGLOBULIN LIGHT CHAINS. Only successful rearrangement of the VDJ segments, at the Ig heavy chain gene locus (IMMUNOGLOBULIN HEAVY CHAIN GENES), will generate mu heavy chains that can pair with surrogate light chains. Thus formation of the pre-B cell receptors is an important checkpoint in the development of mature B cells.'),
  'D054421': NtO(MH='Chemokine CCL21', MS='A CC-type chemokine with specificity for CCR7 RECEPTORS. It has activity towards DENDRITIC CELLS and T-LYMPHOCYTES.'),
  'D054422': NtO(MH='Chemokine CCL22', MS='A CC-type chemokine with specificity for CCR4 RECEPTORS. It has activity towards TH2 CELLS and TC2 CELLS.'),
  'D054423': NtO(MH='Chemokine CCL24', MS='A CC-type chemokine with specificity for CCR3 RECEPTORS. It is a chemoattractant for EOSINOPHILS.'),
  'D054425': NtO(MH='Chemokine CCL27', MS='A CC-type chemokine with specificity for CCR10 RECEPTORS. It is constitutively expressed in the skin and may play a role in T-CELL trafficking during cutaneous INFLAMMATION.'),
  'D054426': NtO(MH='Chemokine CXCL2', MS='A CXC chemokine that is synthesized by activated MONOCYTES and NEUTROPHILS. It has specificity for CXCR2 RECEPTORS.'),
  'D054427': NtO(MH='Chemokine CXCL6', MS='A CXC chemokine that has stimulatory and chemotactic activities towards NEUTROPHILS. It has specificity for CXCR1 RECEPTORS and CXCR2 RECEPTORS.'),
  'D054428': NtO(MH='Chemokine CX3CL1', MS='A CX3C chemokine that is a transmembrane protein found on the surface of cells. The soluble form of chemokine CX3CL1 can be released from cell surface by proteolysis and act as a chemoattractant that may be involved in the extravasation of leukocytes into inflamed tissues. The membrane form of the protein may also play a role in cell adhesion.'),
  'D054429': NtO(MH='Leukemia, Myelomonocytic, Juvenile', MS='A leukemia affecting young children characterized by SPLENOMEGALY, enlarged lymph nodes, rashes, and hemorrhages. Traditionally classed as a myeloproliferative disease, it is now considered a mixed myeloproliferative-mylelodysplastic disorder.'),
  'D054437': NtO(MH='Myelodysplastic-Myeloproliferative Diseases', MS='Clonal myeloid disorders that possess both dysplastic and proliferative features but are not properly classified as either MYELODYSPLASTIC SYNDROMES or MYELOPROLIFERATIVE DISORDERS.'),
  'D054438': NtO(MH='Leukemia, Myeloid, Chronic, Atypical, BCR-ABL Negative', MS='A myelodysplastic/myeloproliferative disorder characterized by myelodysplasia associated with bone marrow and peripheral blood patterns similar to CHRONIC MYELOID LEUKEMIA, but cytogenetically lacking a PHILADELPHIA CHROMOSOME or bcr/abl fusion gene (GENES, ABL).'),
  'D054439': NtO(MH='Ghrelin', MS='A 28-amino acid, acylated, orexigenic peptide that is a ligand for GROWTH HORMONE SECRETAGOGUE RECEPTORS. Ghrelin is widely expressed but primarily in the stomach in the adults. Ghrelin acts centrally to stimulate growth hormone secretion and food intake, and peripherally to regulate energy homeostasis. Its large precursor protein, known as appetite-regulating hormone or motilin-related peptide, contains ghrelin and obestatin.'),
  'D054440': NtO(MH='Receptors, Ghrelin', MS='Transmembrane proteins that recognize and bind GHRELIN, a potent stimulator of GROWTH HORMONE secretion and food intake in mammals. Ghrelin receptors are found in the pituitary and HYPOTHALAMUS. They belong to the family of G-PROTEIN-COUPLED RECEPTORS.'),
  'D054441': NtO(MH='Cell Migration Assays, Leukocyte', MS='Assays that measure the rate of migration of LEUKOCYTES. They may involve a variety of techniques such as measuring the movement of leukocytes through substrates such as AGAROSE gels or the rate of exit of cells from a glass capillary.'),
  'D054442': NtO(MH='Cell Migration Assays, Macrophage', MS='Assays that measure the rate of migration of MACROPHAGES. They may involve the use hollow plastic chamber, sealed at one end with a porous membrane and suspended over a larger well which may contain CHEMOTACTIC FACTORS. The migration of cell through the pores to the other side of the membrane is measured.'),
  'D054443': NtO(MH='Cell Migration Assays', MS='Specific assays that measure the migration of cells. They are commonly used to measure the migration of immune cells in response to stimuli and the inhibition of immune cell migration by immunosuppressive factors.'),
  'D054444': NtO(MH='Immunoglobulin Light Chains, Surrogate', MS='An immunolglobulin light chain-like protein composed of an IMMUNOGLOBULIN VARIABLE REGION-like peptide (such as light chain like lambda5 peptide) and an IMMUNOGLOBULIN CONSTANT REGION-like peptide (such as Vpreb1 peptide). Surrogate light chains associate with MU IMMUNOGLOBULIN HEAVY CHAINS in place of a conventional immunoglobulin light chains to form pre-B cell receptors.'),
  'D054445': NtO(MH='Facial Transplantation', MS='The transference between individuals of the entire face or major facial structures. In addition to the skin and cartilaginous tissue (CARTILAGE), it may include muscle and bone as well.'),
  'D054446': NtO(MH='Lymphoma, Primary Cutaneous Anaplastic Large Cell', MS='Anaplastic lymphoma of the skin which develops as a primary neoplasm expressing the CD30 ANTIGEN. It is characterized by solitary nodules or ulcerated tumors.'),
  'D054447': NtO(MH='Receptors, CCR10', MS='CCR receptors with specificity for CHEMOKINE CCL27. They may play a specialized role in the cutaneous homing of LYMPHOCYTES.'),
  'D054448': NtO(MH='Precursor Cells, B-Lymphoid', MS='Lymphocyte progenitor cells that are restricted in their differentiation potential to the B lymphocyte lineage. The pro-B cell stage of B lymphocyte development precedes the pre-B cell stage.'),
  'D054457': NtO(MH='Tissue Scaffolds', MS='Cell growth support structures composed of BIOCOMPATIBLE MATERIALS. They are specially designed solid support matrices for cell attachment in TISSUE ENGINEERING and GUIDED TISSUE REGENERATION uses.'),
  'D054458': NtO(MH='Amplified Fragment Length Polymorphism Analysis', MS='The detection of RESTRICTION FRAGMENT LENGTH POLYMORPHISMS by selective PCR amplification of restriction fragments derived from genomic DNA followed by electrophoretic analysis of the amplified restriction fragments.'),
  'D054459': NtO(MH='Elasticity Imaging Techniques', MS='Non-invasive imaging methods based on the mechanical response of an object to a vibrational or impulsive force. It is used for determining the viscoelastic properties of tissue, and thereby differentiating soft from hard inclusions in tissue such as microcalcifications, and some cancer lesions. Most techniques use ultrasound to create the images - eliciting the response with an ultrasonic radiation force and/or recording displacements of the tissue by Doppler ultrasonography.'),
  'D054460': NtO(MH='rho-Associated Kinases', MS='A group of intracellular-signaling serine threonine kinases that bind to RHO GTP-BINDING PROTEINS. They were originally found to mediate the effects of rhoA GTP-BINDING PROTEIN on the formation of STRESS FIBERS and FOCAL ADHESIONS. Rho-associated kinases have specificity for a variety of substrates including MYOSIN-LIGHT-CHAIN PHOSPHATASE and LIM KINASES.'),
  'D054461': NtO(MH='Lim Kinases', MS='Serine protein kinases involved in the regulation of ACTIN polymerization and MICROTUBULE disassembly. Their activity is regulated by phosphorylation of a threonine residue within the activation loop by intracellular signaling kinases such as P21-ACTIVATED KINASES and by RHO KINASE.'),
  'D054462': NtO(MH='p21-Activated Kinases', MS='A family of serine-threonine kinases that bind to and are activated by MONOMERIC GTP-BINDING PROTEINS such as RAC GTP-BINDING PROTEINS and CDC42 GTP-BINDING PROTEIN. They are intracellular signaling kinases that play a role the regulation of cytoskeletal organization.'),
  'D054463': NtO(MH='Trichothiodystrophy Syndromes', MS='Autosomal recessive neuroectodermal disorders characterized by brittle sulfur-deficient hair associated with impaired intellect, decreased fertility, and short stature. It may include nail dystrophy, ICHTHYOSIS, and photosensitivity correlated with a NUCLEOTIDE EXCISION REPAIR defect. All individuals with this disorder have a deficiency of cysteine-rich KERATIN-ASSOCIATED PROTEINS found in the interfilamentous matrix. Photosensitive trichothiodystrophy can be caused by mutation in at least 2 separate genes: ERCC2 PROTEIN gene and the related ERCC3. Nonphotosensitive trichothiodystrophy can be caused by mutation in the TTDN1 gene.'),
  'D054464': NtO(MH='Peroxiredoxins', MS='A family of ubiquitously-expressed peroxidases that play a role in the reduction of a broad spectrum of PEROXIDES like HYDROGEN PEROXIDE; LIPID PEROXIDES and peroxinitrite. They are found in a wide range of organisms, such as BACTERIA; PLANTS; and MAMMALS. The enzyme requires the presence of a thiol-containing intermediate such as THIOREDOXIN as a reducing cofactor.'),
  'D054465': NtO(MH='Peroxiredoxin VI', MS='A peroxiredoxin that is a cytosolic bifunctional enzyme. It functions as a peroxiredoxin via a single redox-active cysteine and also contains a Ca2+-independent acidic phospholipase A2 activity.'),
  'D054466': NtO(MH='Phospholipases A1', MS='A phospholipase that hydrolyzes the acyl group attached to the 1-position of PHOSPHOGLYCERIDES.'),
  'D054467': NtO(MH='Phospholipases A2', MS='Phospholipases that hydrolyze the acyl group attached to the 2-position of PHOSPHOGLYCERIDES.'),
  'D054468': NtO(MH='Axoneme', MS='A bundle of MICROTUBULES and MICROTUBULE-ASSOCIATED PROTEINS forming the core of each CILIUM or FLAGELLUM. In most eukaryotic cilia or flagella, an axoneme shaft has 20 microtubules arranged in nine doublets and two singlets.'),
  'D054477': NtO(MH='Glutaredoxins', MS='A family of thioltransferases that contain two active site CYSTEINE residues, which either form a disulfide (oxidized form) or a dithiol (reduced form). They function as an electron carrier in the GLUTHIONE-dependent synthesis of deoxyribonucleotides by RIBONUCLEOTIDE REDUCTASES and may play a role in the deglutathionylation of protein thiols. The oxidized forms of glutaredoxins are directly reduced by the GLUTATHIONE.'),
  'D054479': NtO(MH='Chloroplast Thioredoxins', MS='A subtype of thioredoxins found primarily in CHLOROPLASTS.'),
  'D054480': NtO(MH='Thioredoxin h', MS='A thioredoxin subtype that is ubiquitously found in the plant kingdom. It reduces a variety of seed storage proteins and may play a role in the germination process of seeds.'),
  'D054481': NtO(MH='Thioredoxin Reductase 1', MS='A subtype of thioredoxin reductase found primarily in the CYTOSOL.'),
  'D054482': NtO(MH='Thioredoxin Reductase 2', MS='A subtype of thioredoxin reductase found primarily in MITOCHONDRIA.'),
  'D054497': NtO(MH='Phospholipases A2, Secretory', MS='A subcategory of phospholipases A2 that are secreted from cells. They are 14 kDa proteins containing multiple disulfide-bonds and access their substrate via an interfacial binding site that interacts with phospholipid membranes. In addition specific PHOSPHOLIPASE A2 RECEPTORS can bind to and internalize the enzymes.'),
  'D054498': NtO(MH='Group I Phospholipases A2', MS='A subcategory of secreted phospholipases A2 that includes enzymes isolated from ELAPID VENOMS and pancreatic sources. The creation of this group is based upon similarities in the structural determinants of the enzymes.'),
  'D054499': NtO(MH='Group IA Phospholipases A2', MS='A subclass of group I phospholipases A2 that includes enzymes isolated from ELAPID VENOMS.'),
  'D054500': NtO(MH='Group IB Phospholipases A2', MS='A subclass of group I phospholipases A2 that includes enzymes isolated from PANCREATIC JUICE. Members of this group have specificity for PHOSPHOLIPASE A2 RECEPTORS.'),
  'D054501': NtO(MH='Group II Phospholipases A2', MS='A subcategory of secreted phospholipases A2 that includes enzymes isolated from a variety of sources. The creation of this group is based upon similarities in the structural determinants of the enzymes including a negatively charged carboxy-terminal segment.'),
  'D054502': NtO(MH='Histocompatibility, Maternal-Fetal', MS='The degree of antigenic similarity between tissues of the mother and those of the FETUS. Maternal-fetal histocompatibility can determine the acceptance and health of the fetus.'),
  'D054503': NtO(MH='Lymphoid Progenitor Cells', MS='Stem cells from which B-LYMPHOCYTES; T-LYMPHOCYTES; NATURAL KILLER CELLS; and some DENDRITIC CELLS derive.'),
  'D054504': NtO(MH='Precursor Cells, T-Lymphoid', MS='Lymphocyte progenitor cells that are restricted in their differentiation potential to the T lymphocyte lineage.'),
  'D054505': NtO(MH='Group V Phospholipases A2', MS='A subcategory of secreted phospholipases A2 that contains both a negatively charged carboxy-terminal segment and interfacial-binding region specific for PHOSPHATIDYL CHOLINE-containing membranes. This enzyme group may play a role in the release of ARACHIDONIC ACID from phospholipid membranes.'),
  'D054506': NtO(MH='Chloracne', MS='ACNE-like skin eruptions caused by exposure to CHLORINE-containing compounds. Exposure can be by inhalation, ingestion, or through the skin. Chloracne is often seen in people who have occupational contact with chlorinated pesticides, wood preservatives, and sealants.'),
  'D054507': NtO(MH='Receptors, Phospholipase A2', MS='Cell surface receptors that bind to and internalize SECRETED PHOSPHOLIPASES A2. Although primarily acting as scavenger receptors, these proteins may also play a role in intracellular signaling. Soluble forms of phospholipase A2 receptors occur through the action of proteases and may a play a role in the inhibition of extracellular phospholipase activity.'),
  'D054508': NtO(MH='Acute Radiation Syndrome', MS='A condition caused by a brief whole body exposure to more than one sievert dose equivalent of radiation. Acute radiation syndrome is initially characterized by ANOREXIA; NAUSEA; VOMITING; but can progress to hematological, gastrointestinal, neurological, pulmonary, and other major organ dysfunction.'),
  'D054509': NtO(MH='Group X Phospholipases A2', MS='A secreted phospholipase A2 subtype that contains a interfacial-binding region with specificity for PHOSPHATIDYLCHOLINE. This enzyme group may play a role in eliciting ARACHIDONIC ACID release from intact cellular membranes and from LOW DENSITY LIPOPROTEINS. Members of this group bind specifically to PHOSPHOLIPASE A2 RECEPTORS.'),
  'D054510': NtO(MH='Phospholipases A2, Cytosolic', MS='A subcategory of phospholipases A2 that occur in the CYTOSOL.'),
  'D054512': NtO(MH='Phospholipases A2, Calcium-Independent', MS='A subcategory of structurally-related phospholipases A2 that do not require calcium for activity.'),
  'D054513': NtO(MH='Group IV Phospholipases A2', MS='A cytosolic phospholipase A2 group that plays an important role in the release of free ARACHIDONIC ACID, which in turn is metabolized to PROSTAGLANDINS by the CYCLOOXYGENASE pathway and to LEUKOTRIENES by the 5-LIPOXYGENASE pathway.'),
  'D054514': NtO(MH='Persistent Hyperplastic Primary Vitreous', MS='A developmental ocular anomaly in which the primary VITREOUS BODY and its surrounding hyaloid vasculature failed to regress. It is usually unilateral and characterized by CATARACT; MICROPHTHALMOS (small eyeballs), and retrolenticular fibrovascular tissue. (from Yanoff: Ophthalmology, 2nd ed.)'),
  'D054515': NtO(MH='Vulvar Vestibulitis', MS='Inflammation of the vulvar vestibular region at the entrance of the VAGINA, generally involving surface mucosa and submucosal vestibular glands. It is characterized by ERYTHEMA and chronic recurrent pain in this area.'),
  'D054517': NtO(MH='Orbital Cellulitis', MS='Inflammation of the loose connective tissues around the ORBIT, bony structure around the eyeball. It is characterized by PAIN; EDEMA of the CONJUNCTIVA; swelling of the EYELIDS; EXOPHTHALMOS; limited eye movement; and loss of vision.'),
  'D054518': NtO(MH='Attentional Blink', MS='Temporary visual deficit or impaired visual processing occurring in a rapid serial visual presentation task. After a person identifies the first of two visual targets, the ability to detect the second target is impaired for the next few hundred milliseconds. This phenomenon is called attentional blink.'),
  'D054519': NtO(MH='Aphorisms and Proverbs', MS='Short memorable sayings in common use. They express in simple language an obvious truth, familiar experience, or advice.'),
  'D054520': NtO(MH='Group III Phospholipases A2', MS='A subcategory of secreted phospholipases A2 with specificity for PHOSPHATIDYLETHANOLAMINES and PHOSPHATIDYLCHOLINE. It occurs as a component of VENOMS and as a mammalian secretory phospholipase A2. The creation of this group is based upon similarities in the structural determinants of the enzymes including a long amino-terminal domain, a conserved group III-specific domain, and a long carboxyl-terminal domain.'),
  'D054521': NtO(MH='Cultural Competency', MS='Cultural and linguistic competence is a set of congruent behaviors, attitudes, and policies that come together in a system, agency, or among professionals that enables effective work in cross-cultural situations. Competence implies the capacity to function effectively as an individual and an organization within the context of the cultural beliefs, behaviors, and needs presented by consumers and their communities.'),
  'D054522': NtO(MH='Group VI Phospholipases A2', MS='A calcium-independent phospholipase A2 group that may play a role in membrane phospholipid remodeling and homeostasis by controling the levels of PHOSPHATIDYLCHOLINE in mammalian cell membranes.'),
  'D054523': NtO(MH='Adult Survivors of Child Abuse', MS='Persons who were child victims of violence and abuse including physical, sexual, or emotional maltreatment.'),
  'D054524': NtO(MH='Medicare Part D', MS='A stand-alone drug plan offered by insurers and other private companies to beneficiaries that receive their Medicare Part A and/or B benefits through the Original Medicare Plan. It includes Medicare Private Fee-for-Service Plans that do not offer prescription drug coverage and Medicare Cost Plans offering Medicare prescription drug coverage. The plan was enacted as the Medicare Prescription Drug, Improvement and Modernization Act of 2003 with coverage beginning January 1, 2006.'),
  'D054525': NtO(MH='Minority Health', MS='The concept covering the physical and mental conditions of members of minority groups.'),
  'D054526': NtO(MH="Men's Health", MS='The concept covering the physical and mental conditions of men.'),
  'D054527': NtO(MH='Mass Casualty Incidents', MS='Events that overwhelm the resources of local HOSPITALS and health care providers. They are likely to impose a sustained demand for HEALTH SERVICES rather than the short, intense peak customary with smaller scale disasters.'),
  'D054537': NtO(MH='Atrioventricular Block', MS='Impaired impulse conduction from HEART ATRIA to HEART VENTRICLES. AV block can mean delayed or completely blocked impulse conduction.'),
  'D054538': NtO(MH='Occupational Health Physicians', MS='Physicians employed in a company or corporate setting that is generally not in the health care industry.'),
  'D054539': NtO(MH='Medication Therapy Management', MS='Assistance in managing and monitoring drug therapy for patients receiving treatment for cancer or chronic conditions such as asthma and diabetes, consulting with patients and their families on the proper use of medication; conducting wellness and disease prevention programs to improve public health; overseeing medication use in a variety of settings.'),
  'D054540': NtO(MH='Child, Orphaned', MS='Child who has lost both parents through death or desertion.'),
  'D054541': NtO(MH='Family Conflict', MS='Struggle or disagreement between parents, parent and child or other members of a family.'),
  'D054542': NtO(MH='Iraq War, 2003-2011', MS='An armed intervention involving multi-national forces in the country of IRAQ.'),
  'D054543': NtO(MH='United States Department of Homeland Security', MS='A cabinet department in the Executive Branch of the United States Government concerned with administering those agencies and offices having programs pertaining to domestic national security.'),
  'D054544': NtO(MH='Arthroplasty, Subchondral', MS='Surgical techniques used to correct or augment healing of chondral defects in the joints (CARTILAGE, ARTICULAR). These include abrasion, drilling, and microfracture of the subchondral bone to enhance chondral resurfacing via autografts, allografts, or cell transplantation.'),
  'D054545': NtO(MH='United States Office of National Drug Control Policy', MS='A component of the Executive Office of the President established by the Anti-Drug Abuse Act of 1988. The Office establishes policies, priorities, and objectives for national DRUG AND NARCOTIC CONTROL. The goals of the program are to reduce illicit drug use, manufacturing, and trafficking, drug-related crime and violence, and drug-related health consequences.'),
  'D054546': NtO(MH='Neuroacanthocytosis', MS='An inherited autosomal disorder that is characterized by neurodegeneration; orofacial and buccal DYSKINESIAS; CHOREA; and thorny-looking red cells (ACANTHOCYTES). This disorder is due to mutations of chorein which is important in protein trafficking and is encoded by Vps13a on chromosome 9q21.'),
  'D054547': NtO(MH='National Cancer Institute (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. Through basic and clinical biomedical research and training, it conducts and supports research with the objective of cancer prevention, early stage identification and elimination. This Institute was established in 1937.'),
  'D054548': NtO(MH='Montenegro', MS='Country in Europe, between the Adriatic Sea and Serbia. The capital is Podgorica.'),
  'D054549': NtO(MH='Takotsubo Cardiomyopathy', MS='A transient left ventricular apical dysfunction or ballooning accompanied by electrocardiographic (ECG) T wave inversions. This abnormality is associated with high levels of CATECHOLAMINES, either administered or endogenously secreted from a tumor or during extreme stress.'),
  'D054554': NtO(MH='National Eye Institute (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports research on the causes, diagnosis, and treatment of diseases of the eye and visual system. It was originally part of the National Institute of Neurological Diseases and Blindness. The National Eye Institute was established in 1968.'),
  'D054555': NtO(MH='National Heart, Lung, and Blood Institute (U.S.)', MS="Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports research program related to diseases of the heart, blood vessels, lung, and blood; blood resources; and SLEEP WAKE DISORDERS. From 1948 until October 10, 1969, it was known as the National Heart Institute. From June 25, 1976, it was the National Heart and Lung Institute. Since October 1997, the NHLBI has also had administrative responsibility for the NIH Woman's Health Initiative."),
  'D054556': NtO(MH='Venous Thromboembolism', MS='Obstruction of a vein or VEINS (embolism) by a blood clot (THROMBUS) in the blood stream.'),
  'D054557': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases', MS='A subcategory of protein tyrosine phosphatases that are bound to the cell membrane. They contain cytoplasmic tyrosine phosphatase domains and extracellular protein domains that may play a role in cell-cell interactions by interacting with EXTRACELLULAR MATRIX components. They are considered receptor-like proteins in that they appear to lack specific ligands.'),
  'D054558': NtO(MH='Protein Tyrosine Phosphatases, Non-Receptor', MS='A subcategory of protein tyrosine phosphatases that occur in the CYTOPLASM. Many of the proteins in this category play a role in intracellular signal transduction.'),
  'D054559': NtO(MH='Hyperphosphatemia', MS='A condition of abnormally high level of PHOSPHATES in the blood, usually significantly above the normal range of 0.84-1.58 mmol per liter of serum.'),
  'D054560': NtO(MH='National Human Genome Research Institute (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports research into the mapping of the human genome and other organism genomes. The National Center for Human Genome Research was established in 1989 and re-named the National Human Genome Research Institute in 1997.'),
  'D054561': NtO(MH='National Institute on Aging (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. Through basic and clinical biomedical research and training, it conducts and supports research into the nature of the aging process and diseases associated with the later stages of life. The Institute was established in 1974.'),
  'D054562': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 1', MS='A subtype of non-receptor protein tyrosine phosphatases that includes two distinctive targeting motifs; an N-terminal motif specific for the INSULIN RECEPTOR, and a C-terminal motif specific for the SH3 domain containing proteins. This subtype includes a hydrophobic domain which localizes it to the ENDOPLASMIC RETICULUM.'),
  'D054563': NtO(MH='National Institute on Alcohol Abuse and Alcoholism (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts research focused on improving the treatment and prevention of alcoholism and alcohol-related problems to reduce the health, social, and economic consequences of this disease. NIAAA, NIMH, and NIDA were created as coequal institutes within the Alcohol, Drug Abuse and Mental Health Administration in 1974. It was established within the NATIONAL INSTITUTES OF HEALTH in 1992.'),
  'D054577': NtO(MH='National Institute on Drug Abuse (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It supports a comprehensive research portfolio that focuses on the biological, social, behavioral and neuroscientific bases of drug abuse on the body and brain as well as its causes, prevention, and treatment. NIDA, NIAAA, and NIMH were created as coequal institutes within the Alcohol, Drug Abuse and Mental Health Administration in 1974. It was established within the NATIONAL INSTITUTES OF HEALTH in 1992.'),
  'D054578': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 2', MS='A subtype of non-receptor protein tyrosine phosphatase that is closely-related to PROTEIN TYROSINE PHOSPHATASE, NON-RECEPTOR TYPE 1. Alternative splicing of the mRNA for this phosphatase results in the production at two gene products, one of which includes a C-terminal nuclear localization domain that may be involved in the transport of the protein to the CELL NUCLEUS. Although initially referred to as T-cell protein tyrosine phosphatase the expression of this subtype occurs widely.'),
  'D054579': NtO(MH='National Institute of Allergy and Infectious Diseases (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports basic and applied research to better understand, treat, and ultimately prevent infectious, immunologic, and allergic diseases. It was established in 1948.'),
  'D054580': NtO(MH='National Institute of Arthritis and Musculoskeletal and Skin Diseases (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It supports research into the causes, treatment, and prevention of arthritis and musculoskeletal and skin diseases; the training of basic and clinical scientists to carry out this research; and the dissemination of information on research progress. It was established in 1986.'),
  'D054581': NtO(MH='National Institute of Biomedical Imaging and Bioengineering (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. Its mission is to improve health by leading the development and accelerating the application of biomedical technologies, and integrating the physical and engineering sciences with the life sciences to advance basic research and medical care. It was established in 2000.'),
  'D054582': NtO(MH='National Institute of Child Health and Human Development (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It was initially established to investigate the broad aspects of human development as a means of understanding developmental disabilities, including mental retardation, and the events that occur during pregnancy. It now conducts and supports research on all stages of human development. It was established in 1962.'),
  'D054583': NtO(MH='National Institute on Deafness and Other Communication Disorders (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports biomedical research and research training on normal mechanisms as well as diseases and disorders of hearing, balance, smell, taste, voice, speech, and language. It was established in 1988.'),
  'D054584': NtO(MH='National Institute of Dental and Craniofacial Research (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It seeks to improve oral, dental and craniofacial health through research, research training, and the dissemination of health information by conducting and supporting basic and clinical research. It was established in 1948 as the National Institute of Dental Research and re-named in 1998 as the National Institute of Dental and Craniofacial Research.'),
  'D054585': NtO(MH='National Institute of Environmental Health Sciences (U.S.)', MS="Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports basic and applied research to reduce the burden of human illness and dysfunction from environmental causes by, defining how environmental exposures, genetic susceptibility, and age interact to affect an individual's health. It was established in 1969."),
  'D054586': NtO(MH='National Institute of Diabetes and Digestive and Kidney Diseases (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports basic and applied research for a national program in diabetes, endocrinology, and metabolic diseases; digestive diseases and nutrition; and kidney, urologic, and hematologic diseases. It was established in 1948.'),
  'D054587': NtO(MH='National Institute of General Medical Sciences (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports basic biomedical research that is not targeted to specific diseases and funds studies on genes, proteins, and cells, as well as on fundamental processes like communication within and between cells and metabolism. It was established in 1962.'),
  'D054588': NtO(MH='National Institute of Nursing Research (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It conducts and supports clinical and basic research to establish a scientific basis for the care of individuals across the life span, from the management of patients during illness and recovery to the reduction of risks for disease and disability; the promotion of healthy lifestyles; the promotion of quality of life in those with chronic illness; and the care for individuals at the end of life. It was established in 1986.'),
  'D054589': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 3', MS='A subtype of non-receptor protein tyrosine phosphatases that is characterized by the presence of an amino-terminal FERM domain, an intervening region containing one or more PDZ domains, and a carboxyl-terminal phosphatase domain. Expression of this phosphatase subtype has been observed in BONE MARROW; fetal LIVER; LYMPH NODES; and T LYMPHOCYTES.'),
  'D054590': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 4', MS='A subtype of non-receptor protein tyrosine phosphatases that is characterized by the presence of an amino-terminal FERM domain, an intervening region containing one or more PDZ domains, and a carboxyl-terminal phosphatase domain. The subtype was originally identified in a cell line derived from MEGAKARYOCYTES.'),
  'D054592': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 11', MS='A subtype of non-receptor protein tyrosine phosphatases that contain two SRC HOMOLOGY DOMAINS. Mutations in the gene for protein tyrosine phosphatase, non-receptor type 11 are associated with NOONAN SYNDROME.'),
  'D054593': NtO(MH='SH2 Domain-Containing Protein Tyrosine Phosphatases', MS='A subcategory of protein tyrosine phosphatases that contain SH2 type SRC HOMOLOGY DOMAINS. Many of the proteins in this class are recruited to specific cellular targets such as a cell surface receptor complexes via their SH2 domain.'),
  'D054594': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 12', MS='A subtype of non-receptor protein tyrosine phosphatases that is characterized by the presence of a N-terminal catalytic domain and a large C-terminal domain that is enriched in PROLINE, GLUTAMIC ACID, SERINE, and THREONINE residues (PEST sequences). The phosphatase subtype is ubiquitously expressed and implicated in the regulation of a variety of biological processes such as CELL MOVEMENT; CYTOKINESIS; focal adhesion disassembly; and LYMPHOCYTE ACTIVATION.'),
  'D054595': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 13', MS='A subtype of non-receptor protein tyrosine phosphatases that is characterized by the presence of an amino-terminal FERM domain, an intervening region containing five different PDZ domains, and a carboxyl-terminal phosphatase domain. In addition to playing a role as a regulator of the FAS RECEPTOR activity this subtype interacts via its PDZ and FERM domains with a variety of INTRACELLULAR SIGNALING PROTEINS and CYTOSKELETAL PROTEINS.'),
  'D054596': NtO(MH='Protein Tyrosine Phosphatase, Non-Receptor Type 22', MS='A subtype of non-receptor protein tyrosine phosphatases that is characterized by the presence of an N-terminal catalytic domain and a C-terminal PROLINE-rich domain. The phosphatase subtype is predominantly expressed in LYMPHOCYTES and plays a key role in the inhibition of downstream T-LYMPHOCYTE activation. Polymorphisms in the gene that encodes this phosphatase subtype are associated with a variety of AUTOIMMUNE DISEASES.'),
  'D054597': NtO(MH='Disaster Medicine', MS='Branch of medicine involved with management and organization of public health response to disasters and major events including the special health and medical needs of a community in a disaster.'),
  'D054598': NtO(MH='Blue Cross Blue Shield Insurance Plans', MS='Prepaid health and hospital insurance plan.'),
  'D054622': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 1', MS='A subclass of receptor-like protein tryosine phosphatases that contain heavily glycosylated and cysteine-rich extracellular regions that include fibronectin type III-like domains.'),
  'D054623': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 2', MS='A subclass of receptor-like protein tryosine phosphatases that contain multiple extracellular immunoglobulin G-like domains and fibronectin type III-like domains. An additional memprin-A5-mu domain is found on some members of this subclass.'),
  'D054624': NtO(MH='Health Status Disparities', MS='Variation in rates of disease occurrence and disabilities between population groups defined by socioeconomic characteristics such as age, ethnicity, economic resources, or gender and populations identified geographically or similar measures.'),
  'D054625': NtO(MH='Healthcare Disparities', MS='Differences in access to or availability of medical facilities and services.'),
  'D054626': NtO(MH='Consumer Health Information', MS='Information intended for potential users of medical and healthcare services. There is an emphasis on self-care and preventive approaches as well as information for community-wide dissemination and use.'),
  'D054627': NtO(MH='Genome, Plastid', MS='The genetic complement of PLASTIDS as represented in their DNA.'),
  'D054628': NtO(MH='Genome, Chloroplast', MS='The genetic complement of CHLOROPLASTS as represented in their DNA.'),
  'D054629': NtO(MH='Genome, Mitochondrial', MS='The genetic complement of MITOCHONDRIA as represented in their DNA.'),
  'D054630': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 4', MS='A subclass of receptor-like protein tryosine phosphatases that contain short highly glycosylated extracellular domains and two active cytosolic protein tyrosine phosphatase domains.'),
  'D054631': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 3', MS='A subclass of receptor-like protein tryosine phosphatases that contain a single cytosolic protein tyrosine phosphate domain and multiple extracellular fibronectin III-like domains.'),
  'D054633': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 5', MS='A subclass of receptor-like protein tryosine phosphatases that contain an extracellular fibronectin III-like domain along with a carbonic anhydrase-like domain.'),
  'D054634': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 7', MS='A subclass of receptor-like protein tryosine phosphatases that contain a short extracellular domain, a cytosolic kinase-interaction domain, and single protein tyrosine kinase domain.'),
  'D054635': NtO(MH='Receptor-Like Protein Tyrosine Phosphatases, Class 8', MS='A subclass of receptor-like protein tryosine phosphatases that contain an extracellular RDGS-adhesion recognition motif and a single cytosolic protein tyrosine phosphate domain.'),
  'D054637': NtO(MH='Dual-Specificity Phosphatases', MS='A sub-class of protein tyrosine phosphatases that contain an additional phosphatase activity which cleaves phosphate ester bonds on SERINE or THREONINE residues that are located on the same protein.'),
  'D054638': NtO(MH='Dual Specificity Phosphatase 1', MS='A dual specificity phosphatase subtype that plays a role in intracellular signal transduction by inactivating MITOGEN-ACTIVATED PROTEIN KINASES. It has specificity for P38 MITOGEN-ACTIVATED PROTEIN KINASES and JNK MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D054639': NtO(MH='Mitogen-Activated Protein Kinase Phosphatases', MS='A subcategory of phosphohydrolases that are specific for MITOGEN-ACTIVATED PROTEIN KINASES. They play a role in the inactivation of the MAP KINASE SIGNALING SYSTEM.'),
  'D054640': NtO(MH='Dual Specificity Phosphatase 3', MS='A dual specificity phosphatase subtype that plays a role in intracellular signal transduction by inactivating MITOGEN-ACTIVATED PROTEIN KINASES. It has specificity for EXTRACELLULAR SIGNAL-REGULATED MAP KINASES.'),
  'D054641': NtO(MH='Dual Specificity Phosphatase 2', MS='A dual specificity phosphatase subtype that plays a role in intracellular signal transduction by inactivating MITOGEN-ACTIVATED PROTEIN KINASES. It has specificity for EXTRACELLULAR SIGNAL-REGULATED MAP KINASES and is primarily localized to the CELL NUCLEUS.'),
  'D054642': NtO(MH='Dual Specificity Phosphatase 6', MS='A dual specificity phosphatase subtype that plays a role in intracellular signal transduction by inactivating MITOGEN-ACTIVATED PROTEIN KINASES. It has specificity for EXTRACELLULAR SIGNAL-REGULATED MAP KINASES and is primarily localized to the CYTOSOL.'),
  'D054643': NtO(MH='INDEL Mutation', MS='A mutation named with the blend of insertion and deletion. It refers to a length difference between two ALLELES where it is unknowable if the difference was originally caused by a SEQUENCE INSERTION or by a SEQUENCE DELETION. If the number of nucleotides in the insertion/deletion is not divisible by three, and it occurs in a protein coding region, it is also a FRAMESHIFT MUTATION.'),
  'D054644': NtO(MH='Ear Auricle', MS='The shell-like structure projects like a little wing (pinna) from the side of the head. Ear auricles collect sound from the environment.'),
  'D054645': NtO(MH='Protein Phosphatase 1', MS='A eukayrotic protein serine-threonine phosphatase subtype that dephosphorylates a wide variety of cellular proteins. The enzyme is comprised of a catalytic subunit and regulatory subunit. Several isoforms of the protein phosphatase catalytic subunit exist due to the presence of multiple genes and the alternative splicing of their mRNAs. A large number of proteins have been shown to act as regulatory subunits for this enzyme. Many of the regulatory subunits have additional cellular functions.'),
  'D054647': NtO(MH='Nutrigenomics', MS='The study of the relationship between NUTRITIONAL PHYSIOLOGY and genetic makeup. It includes the effect of different food components on GENE EXPRESSION and how variations in GENES effect responses to food components.'),
  'D054648': NtO(MH='Protein Phosphatase 2', MS='A phosphoprotein phosphatase subtype that is comprised of a catalytic subunit and two different regulatory subunits. At least two genes encode isoforms of the protein phosphatase catalytic subunit, while several isoforms of regulatory subunits exist due to the presence of multiple genes and the alternative splicing of their mRNAs. Protein phosphatase 2 acts on a broad variety of cellular proteins and may play a role as a regulator of intracellular signaling processes.'),
  'D054657': NtO(MH='Ribosome Subunits', MS='The two dissimilar sized ribonucleoprotein complexes that comprise a RIBOSOME - the large ribosomal subunit and the small ribosomal subunit. The eukaryotic 80S ribosome is composed of a 60S large subunit and a 40S small subunit. The bacterial 70S ribosome is composed of a 50S large subunit and a 30S small subunit.'),
  'D054658': NtO(MH='Ribosome Subunits, Large', MS='The largest ribonucleoprotein component of RIBOSOMES. It contains the domains which catalyze formation of the peptide bond and translocation of the ribosome along the MESSENGER RNA during GENETIC TRANSLATION.'),
  'D054659': NtO(MH='Diketopiperazines', MS='Piperazines with two keto oxygens.'),
  'D054677': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 1', MS="A CALCIUM and CALMODULIN-dependent cyclic nucleotide phosphodiesterase subfamily. The three members of this family are referred to as type 1A, type 1B, and type 1C and are each product of a distinct gene. In addition, multiple enzyme variants of each subtype can be produced due to multiple alternative mRNA splicing. Although the type 1 enzymes are classified as 3',5'-cyclic-AMP phosphodiesterases (EC 3.1.4.17), some members of this class have additional specificity for CYCLIC GMP."),
  'D054678': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 2', MS="A cyclic nucleotide phosphodiesterase subfamily that is activated by the binding of CYCLIC GMP to an allosteric domain found on the enzyme. Multiple enzyme variants of this subtype can be produced due to multiple alternative mRNA splicing. The subfamily is expressed in a broad variety of tissues and may play a role in mediating cross-talk between CYCLIC GMP and CYCLIC CMP pathways. Although the type 2 enzymes are classified as 3',5'-cyclic-AMP phosphodiesterases (EC 3.1.4.17), members of this class have additional specificity for CYCLIC GMP."),
  'D054679': NtO(MH='Ribosome Subunits, Small', MS='The small ribonucleoprotein component of RIBOSOMES. It contains the MESSENGER RNA binding site and two TRANSFER RNA binding sites - one for the incoming AMINO ACYL TRNA (A site) and the other (P site) for the peptidyl tRNA carrying the elongating peptide chain.'),
  'D054680': NtO(MH='Ribosome Subunits, Small, Bacterial', MS='The small subunit of eubacterial RIBOSOMES. It is composed of the 16S RIBOSOMAL RNA and about 23 different RIBOSOMAL PROTEINS.'),
  'D054681': NtO(MH='Ribosome Subunits, Large, Bacterial', MS='The large subunit of the eubacterial 70s ribosome. It is composed of the 23S RIBOSOMAL RNA, the 5S RIBOSOMAL RNA, and about 37 different RIBOSOMAL PROTEINS.'),
  'D054682': NtO(MH='Ribosome Subunits, Small, Eukaryotic', MS='The small subunit of the 80s ribosome of eukaryotes. It is composed of the 18S RIBOSOMAL RNA and 32 different RIBOSOMAL PROTEINS.'),
  'D054683': NtO(MH='Ribosome Subunits, Large, Eukaryotic', MS='The large subunit of the 80s ribosome of eukaryotes. It is composed of the 28S RIBOSOMAL RNA, the 5.8S RIBOSOMAL RNA, the 5S RIBOSOMAL RNA, and about 50 different RIBOSOMAL PROTEINS.'),
  'D054684': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 3', MS='A cyclic nucleotide phosphodiesterase subfamily that is inhibited by the binding of CYCLIC GMP to an allosteric domain found on the enzyme and through phosphorylation by regulatory kinases such as PROTEIN KINASE A and PROTEIN KINASE B. The two members of this family are referred to as type 3A, and type 3B, and are each product of a distinct gene. In addition multiple enzyme variants of each subtype can be produced due to multiple alternative mRNA splicing.'),
  'D054685': NtO(MH='Lymphoma, Primary Effusion', MS='A rare neoplasm of large B-cells usually presenting as serious effusions without detectable tumor masses. The most common sites of involvement are the pleural, pericardial, and peritoneal cavities. It is associated with HUMAN HERPESVIRUS 8, most often occurring in the setting of immunodeficiency.'),
  'D054697': NtO(MH='Cytostatic Agents', MS='Compounds that inhibit or prevent the proliferation of CELLS.'),
  'D054700': NtO(MH='Clausena', MS='A plant genus of the family RUTACEAE. Members contain anethole and CARBAZOLES.'),
  'D054702': NtO(MH='United States Agency for International Development', MS='An independent Federal agency established in 1961 as the focal point for economic matters affecting U.S. relations with developing countries.'),
  'D054703': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 4', MS='A cyclic nucleotide phosphodiesterase subfamily that is found predominantly in inflammatory cells and may play a role in the regulation of CELL-MEDIATED IMMUNITY. The enzyme family includes over twenty different variants that occur due to multiple ALTERNATIVE SPLICING of the mRNA of at least four different genes.'),
  'D054704': NtO(MH='Peace Corps', MS='It was established in 1961 and made an independent agency in 1981. Its mission is to help the people of interested countries in meeting their need for trained men and women, and to help promote better mutual understanding between Americans and citizens of other countries. (United States Government Manual, 2006-2007, pg497)'),
  'D054705': NtO(MH='Mastocytoma, Skin', MS='A variant of cutaneous mastocytosis which occurs as a single lesion usually in infants. It is found mostly in the wrist and trunk and there is no atypical cytomorphology.'),
  'D054706': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 5', MS='A cyclic nucleotide phosphodiesterase subfamily that is highly specific for CYCLIC GMP. It is found predominantly in vascular tissue and plays an important role in regulating VASCULAR SMOOTH MUSCLE contraction.'),
  'D054707': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 6', MS='A cyclic nucleotide phosphodiesterase subfamily that is highly specific for CYCLIC GMP. It is found predominantly in the outer segment PHOTORECEPTOR CELLS of the RETINA. It is comprised of two catalytic subunits, referred to as alpha and beta, that form a dimer. In addition two regulatory subunits, referred to as gamma and delta, modulate the activity and localization of the enzyme.'),
  'D054708': NtO(MH='Cyclic Nucleotide Phosphodiesterases, Type 7', MS='A cyclic nucleotide phosphodiesterase subfamily that is highly specific for CYCLIC AMP. Several isoforms of the enzyme type exist, each with its own tissue localization. The isoforms are encoded by at least two genes and are a product of multiple alternative splicing of their mRNAs.'),
  'D054709': NtO(MH='Lecithins', MS='A complex mixture of PHOSPHOLIPIDS; GLYCOLIPIDS; and TRIGLYCERIDES; with substantial amounts of PHOSPHATIDYLCHOLINES; PHOSPHATIDYLETHANOLAMINES; and PHOSPHATIDYLINOSITOLS, which are sometimes loosely termed as 1,2-diacyl-3-phosphocholines. Lecithin is a component of the CELL MEMBRANE and commercially extracted from SOYBEANS and EGG YOLK. The emulsifying and surfactant properties are useful in FOOD ADDITIVES and for forming organogels (GELS).'),
  'D054710': NtO(MH='Interactive Tutorial', MS='Video recordings or other files in which the progress of the instruction or content is determined by user response.'),
  'D054711': NtO(MH='Introductory Journal Article', MS='Prefactory summary to a special issue or section of a journal devoted to a specific topic. This introductory text can be of varying length and substance.'),
  'D054712': NtO(MH='Salinity', MS='Degree of saltiness, which is largely the OSMOLAR CONCENTRATION of SODIUM CHLORIDE plus any other SALTS present. It is an ecological factor of considerable importance, influencing the types of organisms that live in an ENVIRONMENT.'),
  'D054713': NtO(MH='Bryostatins', MS='A group of 20-member macrolactones in which there are three remotely substituted pyran rings that are linked by a methylene bridge and an E-disubstituted alkene, and have geminal dimethyls at C8 and C18 carbons. Some interact with PROTEIN KINASE C.'),
  'D054714': NtO(MH='Echinocandins', MS='Cyclic hexapeptides of proline-ornithine-threonine-proline-threonine-serine. The cyclization with a single non-peptide bond can lead them to be incorrectly called DEPSIPEPTIDES, but the echinocandins lack ester links. Antifungal activity is via inhibition of 1,3-beta-glucan synthase production of BETA-GLUCANS.'),
  'D054715': NtO(MH='Cardiotoxins', MS='Agents that have a damaging effect on the HEART. Such damage can occur from ALKYLATING AGENTS; FREE RADICALS; or metabolites from OXIDATIVE STRESS and in some cases is countered by CARDIOTONIC AGENTS. Induction of LONG QT SYNDROME or TORSADES DE POINTES has been the reason for viewing some drugs as cardiotoxins.'),
  'D054728': NtO(MH='Cucurbitacins', MS='Triterpenes that derive from LANOSTEROL by a shift of the C19 methyl to the C9 position. They are found in seeds and roots of CUCURBITACEAE and other plants and are noted for intense bitterness.'),
  'D054729': NtO(MH='Calcium-Calmodulin-Dependent Protein Kinase Type 1', MS='A monomeric calcium-calmodulin-dependent protein kinase subtype that is expressed in a broad variety of mammalian cell types. Its expression is regulated by the action of CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE KINASE. Several isoforms of this enzyme subtype are encoded by distinct genes.'),
  'D054730': NtO(MH='Protein Interaction Domains and Motifs', MS='Protein modules with conserved ligand-binding surfaces which mediate specific interaction functions in SIGNAL TRANSDUCTION PATHWAYS and the specific BINDING SITES of their cognate protein LIGANDS.'),
  'D054731': NtO(MH='PDZ Domains', MS='Protein interaction domains of about 70-90 amino acid residues, named after a common structure found in PSD-95, Discs Large, and Zona Occludens 1 proteins. PDZ domains are involved in the recruitment and interaction of proteins, and aid the formation of protein scaffolds and signaling networks. This is achieved by sequence-specific binding between a PDZ domain in one protein and a PDZ motif in another protein.'),
  'D054732': NtO(MH='Calcium-Calmodulin-Dependent Protein Kinase Type 2', MS='A multifunctional calcium-calmodulin-dependent protein kinase subtype that occurs as an oligomeric protein comprised of twelve subunits. It differs from other enzyme subtypes in that it lacks a phosphorylatable activation domain that can respond to CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE KINASE.'),
  'D054734': NtO(MH='Calcium-Calmodulin-Dependent Protein Kinase Type 4', MS='A monomeric calcium-calmodulin-dependent protein kinase subtype that is primarily expressed in neuronal tissues; T-LYMPHOCYTES and TESTIS. The activity of this enzyme is regulated by its phosphorylation by CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE KINASE.'),
  'D054735': NtO(MH='Phosphorothioate Oligonucleotides', MS='Modified oligonucleotides in which one of the oxygens of the phosphate group is replaced with a sulfur atom.'),
  'D054736': NtO(MH='Elongation Factor 2 Kinase', MS='A monomeric calcium-calmodulin-dependent protein kinase subtype that specifically phosphorylates PEPTIDE ELONGATION FACTOR 2. The enzyme lacks a phosphorylatable activation domain that can respond to CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE KINASE, however it is regulated by phosphorylation by PROTEIN KINASE A and through intramolecular autophosphorylation.'),
  'D054737': NtO(MH='Calcium-Calmodulin-Dependent Protein Kinase Kinase', MS='A regulatory calcium-calmodulin-dependent protein kinase that specifically phosphorylates CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE TYPE 1; CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE TYPE 2; CALCIUM-CALMODULIN-DEPENDENT PROTEIN KINASE TYPE 4; and PROTEIN KINASE B. It is a monomeric enzyme that is encoded by at least two different genes.'),
  'D054738': NtO(MH='Scala Vestibuli', MS='The upper chamber of the COCHLEA that is filled with PERILYMPH. It is connected to SCALA TYMPANI via helicotrema at the apex of the cochlea.'),
  'D054739': NtO(MH='Dendritic Cell Sarcoma, Interdigitating', MS='A rare sarcoma of INTERDIGITATING CELLS found in the lymph nodes and non-lymphoid organs. They exhibit a variable immunophenotype and lack Birbeck granules.'),
  'D054740': NtO(MH='Dendritic Cell Sarcoma, Follicular', MS='Sarcoma of FOLLICULAR DENDRITIC CELLS most often found in the lymph nodes. This rare neoplasm occurs predominately in adults.'),
  'D054742': NtO(MH='Acyl-Butyrolactones', MS='Cyclic esters of acylated BUTYRIC ACID containing four carbons in the ring.'),
  'D054743': NtO(MH='Cyclic AMP-Dependent Protein Kinase Type I', MS='A cyclic AMP-dependent protein kinase subtype primarily found in the CYTOPLASM. They are tetrameric proteins that contain two catalytic subunits and two type I-specific regulatory subunits.'),
  'D054746': NtO(MH='Cyclic AMP-Dependent Protein Kinase Type II', MS='A cyclic AMP-dependent protein kinase subtype primarily found in particulate subcellular fractions. They are tetrameric proteins that contain two catalytic subunits and two type II-specific regulatory subunits.'),
  'D054747': NtO(MH='Histiocytic Sarcoma', MS='Malignant neoplasms composed of MACROPHAGES or DENDRITIC CELLS. Most histiocytic sarcomas present as localized tumor masses without a leukemic phase. Though the biological behavior of these neoplasms resemble lymphomas, their cell lineage is histiocytic not lymphoid.'),
  'D054748': NtO(MH='Ribosome Subunits, Large, Archaeal', MS='The large subunit of the archaeal 70s ribosome. It is composed of the 23S RIBOSOMAL RNA, the 5S RIBOSOMAL RNA, and about 40 different RIBOSOMAL PROTEINS.'),
  'D054749': NtO(MH='Ribosome Subunits, Small, Archaeal', MS='The small subunit of archaeal RIBOSOMES. It is composed of the 16S RIBOSOMAL RNA and about 28 different RIBOSOMAL PROTEINS.'),
  'D054750': NtO(MH='Ecotoxicology', MS='The study of ENVIRONMENTAL POLLUTION and the toxic effects of ENVIRONMENTAL POLLUTANTS on the ECOSYSTEM. The term was coined by Truhaut in 1969.'),
  'D054751': NtO(MH='Cyclic AMP-Dependent Protein Kinase Catalytic Subunits', MS='Specific enzyme subunits that form the active sites of the type I and type II cyclic-AMP protein kinases. Each molecule of enzyme contains two catalytic subunits.'),
  'D054752': NtO(MH='Langerhans Cell Sarcoma', MS='Rare malignant neoplasm of dendritic LANGERHANS CELLS exhibiting atypical cytology, frequent mitoses, and aggressive clinical behavior. They can be distinguished from other histiocytic and dendritic proliferations by immunohistochemical and ultrastructure studies. Cytologically benign proliferations of Langerhans cells are called LANGERHANS CELL HISTIOCYTOSIS.'),
  'D054753': NtO(MH='Cyclic AMP-Dependent Protein Kinase RIbeta Subunit', MS='A type I cAMP-dependent protein kinase regulatory subunit that plays a role in confering CYCLIC AMP activation of protein kinase activity. It is found abundantly expressed in the neuronal tissue and may be associated with hippocampal function.'),
  'D054754': NtO(MH='Cyclic AMP-Dependent Protein Kinase RIIalpha Subunit', MS='A type II cAMP-dependent protein kinase regulatory subunit that plays a role in confering CYCLIC AMP activation of protein kinase activity. It has a higher affinity for cAMP than that of the CYCLIC-AMP-DEPENDENT PROTEIN KINASE RIIBETA SUBUNIT. Binding of this subunit by A KINASE ANCHOR PROTEINS may play a role in the cellular localization of type II protein kinase A.'),
  'D054755': NtO(MH='Imidazoline Receptors', MS='Receptors of CLONIDINE and other IMIDAZOLINES. Activity of the ligands was earlier attributed to ADRENERGIC ALPHA-2 RECEPTORS. Endogenous ligands include AGMATINE, imidazoleacetic acid ribotide, and harman.'),
  'D054756': NtO(MH='Cyclic AMP-Dependent Protein Kinase RIalpha Subunit', MS='A type I cAMP-dependent protein kinase regulatory subunit that plays a role in confering CYCLIC AMP activation of protein kinase activity. It has a lower affinity for cAMP than the CYCLIC-AMP-DEPENDENT PROTEIN KINASE RIBETA SUBUNIT.'),
  'D054757': NtO(MH='Cyclic AMP-Dependent Protein Kinase RIIbeta Subunit', MS='A type II cAMP-dependent protein kinase regulatory subunit that plays a role in confering CYCLIC AMP activation of protein kinase activity. It has a lower affinity for cAMP than the CYCLIC-AMP-DEPENDENT PROTEIN KINASE RIIALPHA SUBUNIT. Binding of this subunit by A KINASE ANCHOR PROTEINS may play a role in the cellular localization of type II protein kinase A.'),
  'D054758': NtO(MH='A Kinase Anchor Proteins', MS='A structurally-diverse family of intracellular-signaling adaptor proteins that selectively tether specific protein kinase A subtypes to distinct subcellular sites. They play a role in focusing the PROTEIN KINASE A activity toward relevant substrates. Over fifty members of this family exist, most of which bind specifically to regulatory subunits of CYCLIC AMP-DEPENDENT PROTEIN KINASE TYPE II such as CAMP PROTEIN KINASE RIIALPHA or CAMP PROTEIN KINASE RIIBETA.'),
  'D054768': NtO(MH='G-Protein-Coupled Receptor Kinases', MS='A family of serine-threonine kinases that are specific for G-PROTEIN-COUPLED RECEPTORS. They are regulatory proteins that play a role in G-protein-coupled receptor densensitization.'),
  'D054769': NtO(MH='G-Protein-Coupled Receptor Kinase 2', MS='A ubiquitously expressed G-protein-coupled receptor kinase subtype that has specificity for the agonist-occupied form of BETA-ADRENERGIC RECEPTORS. It may play an essential role in regulating myocardial contractile response.'),
  'D054770': NtO(MH='G-Protein-Coupled Receptor Kinase 3', MS='A ubiquitously expressed G-protein-coupled receptor kinase subtype that has specificity for the agonist-occupied form of BETA-ADRENERGIC RECEPTORS and a variety of other G-PROTEIN-COUPLED RECEPTORS. Although it is highly homologous to G-PROTEIN-COUPLED RECEPTOR KINASE 2, it is not considered to play an essential role in regulating myocardial contractile response.'),
  'D054771': NtO(MH='G-Protein-Coupled Receptor Kinase 4', MS='A G-protein-coupled receptor kinase subtype that is primarily expressed in the TESTES and BRAIN. Variants of this subtype exist due to multiple alternative splicing of its mRNA.'),
  'D054772': NtO(MH='Rats, Hairless', MS='Mutant strains of rats that produce little or no hair. Several different homozygous recessive mutations can cause hairlessness in rats including rnu/rnu (Rowett nude), fz/fz (fuzzy), shn/shn (shorn), and nznu/nznu (New Zealand nude). Note that while NUDE RATS are often hairless, they are most characteristically athymic.'),
  'D054773': NtO(MH='Streptococcus gordonii', MS='A species of gram-positive, facultatively anaerobic bacteria in the family STREPTOCOCCACEAE. It is a normal inhabitant of the human oral cavity, and causes DENTAL PLAQUE and ENDOCARDITIS. It is being investigated as a vehicle for vaccine delivery.'),
  'D054774': NtO(MH='G-Protein-Coupled Receptor Kinase 5', MS='A G-protein-coupled receptor kinase subtype that is primarily expressed in the MYOCARDIUM and may play a role in the regulation of cardiac functions.'),
  'D054776': NtO(MH='Semicircular Ducts', MS='The three membranous semicircular ducts within the bony semicircular canals. They open into the UTRICLE through five openings. Each duct has at one end a sensory area called the ampullary crest. AMPULLARY HAIR CELLS of the crests sense the movement of ENDOLYMPH resulting from rotation of the head.'),
  'D054777': NtO(MH='Hair Cells, Ampulla', MS='Sensory cells in the ampullary crest of each of the semicircular ducts, with their apical STEREOCILIA embedded in a wedge-shaped gelatinous cupula. These hair cells sense the movement of ENDOLYMPH resulting from angular acceleration of the head, and send signals via the VESTIBULAR NERVE to the brain to maintain balance.'),
  'D054778': NtO(MH='NM23 Nucleoside Diphosphate Kinases', MS='A family of nucleotide diphosphate kinases that play a role in a variety of cellular signaling pathways that effect CELL DIFFERENTIATION; CELL PROLIFERATION; and APOPTOSIS. They are considered multifunctional proteins that interact with a variety of cellular proteins and have functions that are unrelated to their enzyme activity.'),
  'D054788': NtO(MH='Ribosome Inactivating Proteins', MS='N-Glycosidases that remove adenines from RIBOSOMAL RNA, depurinating the conserved alpha-sarcin loop of 28S RIBOSOMAL RNA. They often consist of a toxic A subunit and a binding lectin B subunit. They may be considered as PROTEIN SYNTHESIS INHIBITORS. They are found in many PLANTS and have cytotoxic and antiviral activity.'),
  'D054789': NtO(MH='Ribosome Inactivating Proteins, Type 1', MS='Ribosome inactivating proteins consisting of only the toxic A subunit, which is a polypeptide of around 30 kDa.'),
  'D054790': NtO(MH='Ribosome Inactivating Proteins, Type 2', MS='Ribosome inactivating proteins consisting of two polypeptide chains, the toxic A subunit and a lectin B subunit, linked by disulfide bridges. The lectin portion binds to cell surfaces and facilitates transport into the ENDOPLASMIC RETICULUM.'),
  'D054791': NtO(MH='Cervical Length Measurement', MS='A parameter usually used in PRENATAL ULTRASONOGRAPHY to measure the length of the uterine neck (CERVIX UTERI). Cervical length or its shortening is used to identify and prevent early cervical opening and PRETERM BIRTH.'),
  'D054792': NtO(MH='Muscle, Striated', MS='One of two types of muscle in the body, characterized by the array of bands observed under microscope. Striated muscles can be divided into two subtypes: the CARDIAC MUSCLE and the SKELETAL MUSCLE.'),
  'D054793': NtO(MH='Truncated Hemoglobins', MS='A family of hemoglobin-like proteins found in BACTERIA; PLANTS; and unicellular eukaryotes. Truncated hemoglobins are distantly related to vertebrate hemoglobins and are typically shorter than vertebrate hemoglobins by 20-40 residues.'),
  'D054794': NtO(MH='Glycomics', MS='The systematic study of the structure and function of the complete set of glycans (the glycome) produced in a single organism and identification of all the genes that encode glycoproteins.'),
  'D054795': NtO(MH='Incretins', MS='Peptides which stimulate INSULIN release from the PANCREATIC BETA CELLS following oral nutrient ingestion, or postprandially.'),
  'D054796': NtO(MH='Drug Dosage Calculations', MS='Math calculations done for preparing appropriate doses of medicines, taking into account conversions of WEIGHTS AND MEASURES. Mistakes are one of the sources of MEDICATION ERRORS.'),
  'D054798': NtO(MH='Volleyball', MS='A team sport in which two teams hit an inflated ball back and forth over a high net using their hands.'),
  'D054799': NtO(MH='Phospholipase C beta', MS='A phosphoinositide phospholipase C subtype that is primarily regulated by its association with HETEROTRIMERIC G-PROTEINS. It is structurally related to PHOSPHOLIPASE C DELTA with the addition of C-terminal extension of 400 residues.'),
  'D054800': NtO(MH='Glycosylphosphatidylinositol Diacylglycerol-Lyase', MS='A type C phospholipase specific for GLYCOSYLPHOSPHATIDYLINOSITOLS. It plays a role in the breaking of GPI MEMBRANE ANCHORS.'),
  'D054801': NtO(MH='Phosphoinositide Phospholipase C', MS='A type C phospholipase with specificity towards PHOSPHATIDYLINOSITOLS that contain INOSITOL 1,4,5-TRISPHOSPHATE. Many of the enzymes listed under this classification are involved in intracellular signaling.'),
  'D054802': NtO(MH='Molecular Imprinting', MS='A methodology for chemically synthesizing polymer molds of specific molecules or recognition sites of specific molecules. Applications for molecularly imprinted polymers (MIPs) include separations, assays and biosensors, and catalysis.'),
  'D054803': NtO(MH='Phospholipase C delta', MS='A phosphoinositide phospholipase C subtype that is structurally defined by the presence of an N-terminal pleckstrin-homology and EF-hand domains, a central catalytic domain, and a C-terminal calcium-dependent membrane-binding domain.'),
  'D054804': NtO(MH='Cathelicidins', MS='Antimicrobial cationic peptides with a highly conserved amino terminal cathelin-like domain and a more variable carboxy terminal domain. They are initially synthesized as preproproteins and then cleaved. They are expressed in many tissues of humans and localized to EPITHELIAL CELLS. They kill nonviral pathogens by forming pores in membranes.'),
  'D054805': NtO(MH='Dermcidins', MS='47-amino acid peptides secreted by ECCRINE GLANDS and having a role in innate cutaneous defense, being antimicrobial to some pathogenic BACTERIA. They are overexpressed by some primary BREAST CANCER cells. They are derived from 110 residue PROTEIN PRECURSORS.'),
  'D054806': NtO(MH='Magainins', MS='A class of antimicrobial peptides discovered in the skin of XENOPUS LAEVIS. They kill bacteria by permeabilizing cell membranes without exhibiting significant toxicity against mammalian cells.'),
  'D054807': NtO(MH='Cecropins', MS='Antimicrobial peptides that form channels in membranes that are more permeable to anions than cations. They resemble MAGAININS, with their N-terminal region forming a positively charged amphipathic alpha helix, but containing an additional C-terminal segment.'),
  'D054808': NtO(MH='Histatins', MS='A group of small, histidine-rich, cationic peptides in human SALIVA which are antibacterial and antifungal.'),
  'D054809': NtO(MH='Thionins', MS='Antimicrobial peptides of 45-47 amino acids and typically with four disulfide bridges. They are found in PLANTS. Type-V thionins lack the C-terminal nonapeptide. This should not be confused with thionine.'),
  'D054810': NtO(MH='Deep Sedation', MS='Drug-induced depression of consciousness during which patients cannot be easily aroused but respond purposely following repeated painful stimulation. The ability to independently maintain ventilatory function may be impaired. (From: American Society of Anesthesiologists Practice Guidelines)'),
  'D054811': NtO(MH='Phytochelatins', MS='Poly-glutathione peptides composed of (Glu-Cys)n-Gly where n is two to seven. They are biosynthesized by glutathione gamma-glutamylcysteinyltransferase and are found in many PLANTS; YEASTS; and algae. They sequester HEAVY METALS.'),
  'D054812': NtO(MH='Cyclitols', MS='Cycloalkanes containing three or more hydroxyl groups on the ring atoms. Some polyhydroxypiperidines (PIPERIDINES) are called iminocyclitols or aza-sugars.'),
  'D054813': NtO(MH='Polyhydroxyalkanoates', MS='Fatty acid biopolymers that are biosynthesized by microbial polyhydroxyalkanoate synthase enzymes. They are being investigated for use as biodegradable polyesters.'),
  'D054814': NtO(MH='Anatomists', MS='Those persons skilled in anatomy or dissection.'),
  'D054815': NtO(MH='Cyclic Nucleotide-Gated Cation Channels', MS='A subgroup of cyclic nucleotide-regulated ION CHANNELS within the superfamily of pore-loop cation channels. They are expressed in OLFACTORY NERVE cilia and in PHOTORECEPTOR CELLS and some PLANTS.'),
  'D054816': NtO(MH='Levopropoxyphene', MS='A propionate derivative that is used  to suppress coughing.'),
  'D054817': NtO(MH='Pollination', MS='The transfer of POLLEN grains (male gametes) to the plant ovule (female gamete).'),
  'D054818': NtO(MH='Hexosaminidase A', MS='A mammalian beta-hexosaminidase isoform that is a heteromeric protein comprized of both hexosaminidase alpha and hexosaminidase beta subunits. Deficiency of hexosaminidase A due to mutations in the gene encoding the hexosaminidase alpha subunit is a case of TAY-SACHS DISEASE. Deficiency of hexosaminidase A and HEXOSAMINIDASE B due to mutations in the gene encoding the hexosaminidase beta subunit is a case of SANDHOFF DISEASE.'),
  'D054819': NtO(MH='Hexosaminidase B', MS='A mammalian beta-hexosaminidase isoform that is comprized of hexosaminidase beta subunits. Deficiency of hexosaminidase B due to mutations in the gene encoding the hexosaminidase beta subunit is a case of SANDHOFF DISEASE.'),
  'D054820': NtO(MH='beta-Hexosaminidase alpha Chain', MS='The alpha subunit of hexosaminidase A. Mutations in the gene that encodes this protein can result in loss of hexosaminidase A activity and are linked to TAY-SACHS DISEASE.'),
  'D054821': NtO(MH='beta-Hexosaminidase beta Chain', MS='The beta subunit of hexosaminidase A and hexosamininidase B. Mutations in the gene that encodes this protein are linked to SANDHOFF DISEASE and can result in loss of both hexosaminidase A activity and hexosaminidase B activity.'),
  'D054828': NtO(MH='Histamine H3 Antagonists', MS='Drugs that selectively bind to but do not activate HISTAMINE H3 RECEPTORS. They have been used to correct SLEEP WAKE DISORDERS and MEMORY DISORDERS.'),
  'D054829': NtO(MH='RING Finger Domains', MS='A zinc-binding domain defined by the sequence Cysteine-X2-Cysteine-X(9-39)-Cysteine-X(l-3)-His-X(2-3)-Cysteine-X2-Cysteine -X(4-48)-Cysteine-X2-Cysteine, where X is any amino acid. The RING finger motif binds two atoms of zinc, with each zinc atom ligated tetrahedrally by either four cysteines or three cysteines and a histidine. The motif also forms into a unitary structure with a central cross-brace region and is found in many proteins that are involved in protein-protein interactions. The acronym RING stands for Really Interesting New Gene.'),
  'D054830': NtO(MH='Sesterterpenes', MS='Terpenes of five units of HEMITERPENES, formed from geranylfarnesyl pyrophosphate.'),
  'D054831': NtO(MH='Acridones', MS='Compounds based on acridone, which have three linear rings, with the center ring containing a ring nitrogen and a keto oxygen opposite to each other. Many of them are naturally occurring alkaloids.'),
  'D054832': NtO(MH='Urocortins', MS='Neuropeptides of about 40 amino acids which are structurally similar to CORTICOTROPIN-RELEASING FACTOR. Unlike CRF acting primarily through type 1 CRF RECEPTORS, urocortins signal preferentially through type 2 CRF receptors. Urocortins have wide tissue distribution from fish to mammals, and diverse functions. In mammals, urocortins can suppress food intake, delays gastric emptying, and decreases heat-induced edema.'),
  'D054833': NtO(MH='Isoindoles', MS='Benzopyrroles with the nitrogen at the number two carbon, in contrast to INDOLES which have the nitrogen adjacent to the six-membered ring.'),
  'D054834': NtO(MH='Lipocalins', MS='A diverse family of extracellular proteins that bind to small hydrophobic molecules. They were originally characterized as transport proteins, however they may have additional roles such as taking part in the formation of macromolecular complexes with other proteins and binding to CELL SURFACE RECEPTORS.'),
  'D054835': NtO(MH='Lipocalin 1', MS='A lipocalin that was orignally characterized from human TEARS. It is expressed primarily in the LACRIMAL GLAND and the VON EBNER GLANDS. Lipocalin 1 may play a role in olfactory transduction by concentrating and delivering odorants to the ODORANT RECEPTORS.'),
  'D054836': NtO(MH='Indolizidines', MS='Saturated indolizines that are fused six and five-membered rings with a nitrogen atom at the ring fusion. They are biosynthesized in PLANTS by cyclization of a LYSINE coupled to ACETYL COENZYME A. Many of them are naturally occurring ALKALOIDS.'),
  'D054837': NtO(MH='Quinolizidines', MS='Saturated quinolizines that are two fused six-membered rings with a nitrogen atom at the ring fusion. They are biosynthesized in PLANTS by cyclization of a LYSINE coupled to CADAVERINE. Many of them are naturally occurring ALKALOIDS.'),
  'D054838': NtO(MH='von Ebner Glands', MS='Small tubulo-alveolar salivary glands located beneath the circumvallate and foliate papillae.'),
  'D054839': NtO(MH='Retinol-Binding Proteins, Plasma', MS='Retinol binding proteins that circulate in the PLASMA. They are members of the lipocalin family of proteins and play a role in the transport of RETINOL from the LIVER to the peripheral tissues. The proteins are usually found in association with TRANSTHYRETIN.'),
  'D054840': NtO(MH='Retinol-Binding Proteins, Cellular', MS='A subclass of retinol-binding proteins that take part in the intracellular storage and transport of RETINOL. They are both functionally and structurally distinct from PLASMA RETINOL-BINDING PROTEINS.'),
  'D054843': NtO(MH='Negative-Pressure Wound Therapy', MS='The application of a vacuum across the surface of a wound through a foam dressing cut to fit the wound. This removes wound exudates, reduces build-up of inflammatory mediators, and increases the flow of nutrients to the wound thus promoting healing.'),
  'D054848': NtO(MH='Peptaibols', MS='A group of peptides characterized by length of 1-2 dozen residues with a high proportion of them being non-proteinogenic, notably alpha-aminoisobutyric acid (Aib) and isovaline, and have a C-terminal amino alcohol and N terminal alkyl group. They are found in FUNGI and some are ANTI-INFECTIVE AGENTS. They form channels or pores in target organisms. The term is a contraction of peptide-Aib-alcohol.'),
  'D054849': NtO(MH='Cardiac Electrophysiology', MS='The study of the electrical activity and characteristics of the HEART; MYOCARDIUM; and CARDIOMYOCYTES.'),
  'D054850': NtO(MH='Osteophyte', MS='Bony outgrowth usually found around joints and often seen in conditions such as ARTHRITIS.'),
  'D054851': NtO(MH='Tropheryma', MS='A genus of gram-positive bacteria in the family Cellulomonadaceae.'),
  'D054852': NtO(MH='Small Molecule Libraries', MS='Large collections of small molecules (molecular weight about 600 or less), of similar or diverse nature which are used for high-throughput screening analysis of the gene function, protein interaction, cellular processing, biochemical pathways, or other chemical interactions. It includes virtual libraries.'),
  'D054853': NtO(MH='Malignant Atrophic Papulosis', MS='Variously described as a vasculopathy, endovasculitis, or occlusive arteriopathy, this condition occurs in a benign cutaneous form and a lethal multiorgan systemic variant. It is characterized by a narrowing and occlusion of the lumen of small to medium-sized blood vessels, leading to ischemia and infarction in the involved organ systems. The etiology and pathophysiology are unknown.'),
  'D054854': NtO(MH='Vertebroplasty', MS='Procedures to repair or stabilize vertebral fractures, especially compression fractures accomplished by injecting BONE CEMENTS into the fractured VERTEBRAE.'),
  'D054855': NtO(MH='Drug-Eluting Stents', MS='Stents that are covered with materials that are embedded with chemicals that are gradually released into the surrounding milieu.'),
  'D054856': NtO(MH='G-Quadruplexes', MS='Higher-order DNA and RNA structures formed from guanine-rich sequences. They are formed around a core of at least 2 stacked tetrads of hydrogen-bonded GUANINE bases. They can be formed from one two or four separate strands of DNA (or RNA) and can display a wide variety of topologies, which are a consequence of various combinations of strand direction, length, and sequence. (From Nucleic Acids Res. 2006;34(19):5402-15)'),
  'D054868': NtO(MH='Jacobsen Distal 11q Deletion Syndrome', MS='A clinically recognized congenital malformation condition caused by a distal 11q deletion. The features of the syndrome are growth retardation, psychomotor retardation, trigonocephaly, divergent intermittent strabismus, epicanthus, telecanthus, broad nasal bridge, short nose with anteverted nostrils, carp-shaped upper lip, retrognathia, low-set dysmorphic ears, bilateral camptodactyly, and hammertoes. Platelet dysfunction is a feature in Paris-Trousseau type thrombocytopenia.'),
  'D054869': NtO(MH='Clinical Audit', MS="A detailed review and evaluation of selected clinical records by qualified professional personnel to improve the quality of patient care and outcomes. The clinical audit was formally introduced in 1993 into the United Kingdom's National Health Service."),
  'D054872': NtO(MH='Fatty Acid Synthesis Inhibitors', MS='Compounds that interfere with FATTY ACID SYNTHASE resulting in a reduction of FATTY ACIDS. This is a target mechanism in humans of some ANTINEOPLASTIC AGENTS and ANTI-OBESITY AGENTS and of some ANTI-INFECTIVE AGENTS which interfere with CELL WALL and CELL MEMBRANE formation.'),
  'D054873': NtO(MH='Dipeptidyl-Peptidase IV Inhibitors', MS='Compounds that suppress the degradation of INCRETINS by blocking the action of DIPEPTIDYL-PEPTIDASE IV. This helps to correct the defective INSULIN and GLUCAGON secretion characteristic of TYPE 2 DIABETES MELLITUS by stimulating insulin secretion and suppressing glucagon release.'),
  'D054874': NtO(MH='Athletic Performance', MS='Carrying out of specific physical routines or procedures by one who is trained or skilled in physical activity. Performance is influenced by a combination of physiological, psychological, and socio-cultural factors.'),
  'D054875': NtO(MH='Ubiquitination', MS='The act of ligating UBIQUITINS to PROTEINS to form ubiquitin-protein ligase complexes to label proteins for transport to the PROTEASOME ENDOPEPTIDASE COMPLEX where proteolysis occurs.'),
  'D054876': NtO(MH='Prenylation', MS='Attachment of isoprenoids (TERPENES) to other compounds, especially PROTEINS and FLAVONOIDS.'),
  'D054877': NtO(MH='Wolf-Hirschhorn Syndrome', MS='A syndrome caused by large deletions of the telomereic end of the short arm of CHROMOSOME 4 (4p) in Wolf-Hirchhorn syndrome critial regions (WHSCRs). Several candidate genes have been identified including WHSC1 and WHSCH2 which appear to be responsible for the core phenotype and in combination with other linked and unlinked genes determine the severity and inclusion of rarer phenotypes. Most cases have a characteristic cranio-facial defect often referred to as "Greek helmet face" - a combined result of MICROCEPHALY, broad forehead, prominent glabella, HYPERTELORISM, high arched eyebrows, short philtrum and micrognathia. In addition there is mental retardation, growth delays, EPILEPSY, and frequently a wide range of midline and skeletal defects, including HYPOSPADIAS; CONGENITAL HEART DEFECTS; CLEFT LIP; CLEFT PALATE; colobomata; CLUBFOOT; clinodactyly; SCOLIOSIS; and KYPHOSIS.'),
  'D054878': NtO(MH='Lipoylation', MS='Covalent attachment of LIPIDS and FATTY ACIDS to other compounds and PROTEINS.'),
  'D054879': NtO(MH='Halogenation', MS='Covalent attachment of HALOGENS to other compounds.'),
  'D054880': NtO(MH='Aspartylglucosaminuria', MS='A recessively inherited, progressive lysosomal storage disease caused by a deficiency of GLYCOSYLASPARAGINASE activity. The lack of this enzyme activity results in the accumulation of N-acetylglucosaminylasparagine (the linkage unit of asparagine-linked glycoproteins) in LYSOSOMES.'),
  'D054881': NtO(MH='Sex Determination by Skeleton', MS='Validation of the sex of an individual by means of the bones of the SKELETON. It is most commonly based on the appearance of the PELVIS; SKULL; STERNUM; and/or long bones.'),
  'D054882': NtO(MH='Antley-Bixler Syndrome Phenotype', MS='An inherited condition characterized by multiple malformations of CARTILAGE and bone including CRANIOSYNOSTOSIS; midface hypoplasia; radiohumeral SYNOSTOSIS; CHOANAL ATRESIA; femoral bowing; neonatal fractures; and multiple joint CONTRACTURES and, occasionally, urogenital, gastrointestinal or cardiac defects. In utero exposure to FLUCONAZOLE, as well as mutations in at least two separate genes are associated with this condition - POR (encoding P450 (cytochrome) oxidoreductase (NADPH-FERRIHEMOPROTEIN REDUCTASE)) and FGFR2 (encoding FIBROBLAST GROWTH FACTOR RECEPTOR 2).'),
  'D054883': NtO(MH='Oxylipins', MS='Eighteen-carbon cyclopentyl polyunsaturated fatty acids derived from ALPHA-LINOLENIC ACID via an oxidative pathway analogous to the EICOSANOIDS in animals. Biosynthesis is inhibited by SALICYLATES. A key member, jasmonic acid of PLANTS, plays a similar role to ARACHIDONIC ACID in animals.'),
  'D054884': NtO(MH='Host-Pathogen Interactions', MS='The interactions between a host and a pathogen, usually resulting in disease.'),
  'D054885': NtO(MH='Cumulus Cells', MS='The granulosa cells of the cumulus oophorus which surround the OVUM in the GRAAFIAN FOLLICLE. At OVULATION they are extruded with OVUM.'),
  'D054889': NtO(MH='Fatty Acid Synthase, Type II', MS='The form of fatty acid synthase complex found in BACTERIA; FUNGI; and PLANTS. Catalytic steps are like the animal form but the protein structure is different with dissociated enzymes encoded by separate genes. It is a target of some ANTI-INFECTIVE AGENTS which result in disruption of the CELL MEMBRANE and CELL WALL.'),
  'D054890': NtO(MH='Fatty Acid Synthase, Type I', MS='Animal form of fatty acid synthase which is encoded by a single gene and consists of seven catalytic domains and is functional as a homodimer. It is overexpressed in some NEOPLASMS and is a target in humans of some ANTINEOPLASTIC AGENTS and some ANTI-OBESITY AGENTS.'),
  'D054892': NtO(MH='Metagenome', MS='A collective genome representative of the many organisms, primarily microorganisms, existing in a community.'),
  'D054893': NtO(MH='Cone-Beam Computed Tomography', MS='Computed tomography modalities which use a cone or pyramid-shaped beam of radiation.'),
  'D054894': NtO(MH='Spiral Cone-Beam Computed Tomography', MS='Modality of computed tomography in which the patient is irradiated in a spiral path around the body with a cone or pyramid-shaped beam.'),
  'D054908': NtO(MH='Extensively Drug-Resistant Tuberculosis', MS='Tuberculosis resistant to ISONIAZID and RIFAMPIN and at least three of the six main classes of second-line drugs (AMINOGLYCOSIDES; polypeptide agents; FLUOROQUINOLONES; THIOAMIDES; CYCLOSERINE; and PARA-AMINOSALICYLIC ACID) as defined by the CDC.'),
  'D054909': NtO(MH='Placozoa', MS='A phylum of primitive invertebrate animals that exemplify a simple body organization. Trichoplax adhaerens is considered a key species for early metazoan evolution.'),
  'D054910': NtO(MH='Compound Eye, Arthropod', MS='Light sensory organ in ARTHROPODS consisting of a large number of ommatidia, each functioning as an independent photoreceptor unit.'),
  'D054928': NtO(MH='Validation Studies as Topic', MS='Works on research using processes by which the reliability and relevance of a procedure for a specific purpose are established.'),
  'D054969': NtO(MH='Primary Dysautonomias', MS='Disorders of the AUTONOMIC NERVOUS SYSTEM occurring as a primary condition. Manifestations can involve any or all body systems but commonly affect the BLOOD PRESSURE and HEART RATE.'),
  'D054970': NtO(MH='Pure Autonomic Failure', MS='A degenerative disease of the AUTONOMIC NERVOUS SYSTEM that is characterized by idiopathic ORTHOSTATIC HYPOTENSION and a greatly reduced level of CATECHOLAMINES. No other neurological deficits are present.'),
  'D054971': NtO(MH='Orthostatic Intolerance', MS='Symptoms of cerebral hypoperfusion or autonomic overaction which develop while the subject is standing, but are relieved on recumbency. Types of this include NEUROCARDIOGENIC SYNCOPE; POSTURAL ORTHOSTATIC TACHYCARDIA SYNDROME; and neurogenic ORTHOSTATIC HYPOTENSION. (From Noseworthy, JH., Neurological Therapeutics Principles and Practice, 2007, p2575-2576)'),
  'D054972': NtO(MH='Postural Orthostatic Tachycardia Syndrome', MS='A syndrome of ORTHOSTATIC INTOLERANCE combined with excessive upright TACHYCARDIA, and usually without associated ORTHOSTATIC HYPOTENSION. All variants have in common an excessively reduced venous return to the heart (central HYPOVOLEMIA) while upright.'),
  'D054973': NtO(MH='Perivascular Epithelioid Cell Neoplasms', MS='A family of mesenchymal tumors composed of histologically and immunohistochemically distinctive perivascular epithelioid cells. These cells do not have a normal anatomic homolog. (From Fletcher CDM, et. al., World Health Organization Classification of Tumors: Pathology and Genetics of Tumors of Soft Tissue and Bone, 2002).'),
  'D054974': NtO(MH='Costameres', MS='STRIATED MUSCLE cell components which anchor the MYOFIBRILS from the Z-bands to the SARCOLEMMA and EXTRACELLULAR MATRIX. Costameric proteins include the proteins of FOCAL ADHESIONS.'),
  'D054975': NtO(MH='Pallister-Hall Syndrome', MS='A pleiotropic disorder of human development that comprises hypothalamic HAMARTOMA; central and postaxial POLYDACTYLY; bifid EPIGLOTTIS; ANAL ATRESIA; and renal and other abnormalities. This disorder is associated with FRAMESHIFT MUTATIONS in the GLI3 gene which encodes the GLI3 protein, a KRUPPEL-LIKE TRANSCRIPTION FACTORS family member.'),
  'D054976': NtO(MH='Clinical Governance', MS="A framework through which the United Kingdom's National Health Service organizations are accountable for continually improving the quality of their services and safeguarding high standards of care by creating an environment in which excellence in clinical care will flourish. (Scally and Donaldson, BMJ (4 July 1998): 61-65)"),
  'D054988': NtO(MH='Idiopathic Interstitial Pneumonias', MS='A group of interstitial lung diseases with no known etiology. There are several entities with varying patterns of inflammation and fibrosis. They are classified by their distinct clinical-radiological-pathological features and prognosis. They include IDIOPATHIC PULMONARY FIBROSIS; CRYPTOGENIC ORGANIZING PNEUMONIA; and others.'),
  'D054989': NtO(MH='Nephrogenic Fibrosing Dermopathy', MS='A chronic, acquired, idiopathic, progressive eruption of the skin that occurs in the context of RENAL FAILURE. It is sometimes accompanied by systemic fibrosis. The pathogenesis seems to be multifactorial, with postulated involvement of circulating fibrocytes. There is a strong association between this disorder and the use of gadolinium-based contrast agents.'),
  'D054990': NtO(MH='Idiopathic Pulmonary Fibrosis', MS='A common interstitial lung disease of unknown etiology, usually occurring between 50-70 years of age. Clinically, it is characterized by an insidious onset of breathlessness with exertion and a nonproductive cough, leading to progressive DYSPNEA. Pathological features show scant interstitial inflammation, patchy collagen fibrosis, prominent fibroblast proliferation foci, and microscopic honeycomb change.'),
  'D054991': NtO(MH='Autovaccines', MS='Bacterial vaccines prepared from non-pathogenic, autologous bacteria of human origin. In Eastern Europe they are used in humans to treat chronic inflammatory disorders that are resistant to standard treatments. Worldwide they have veterinary use in all types of infectious disease.'),
  'D054992': NtO(MH='Immunological Synapses', MS='The interfaces between T-CELLS and ANTIGEN-PRESENTING CELLS. Supramolecular organization of proteins takes place at these synapses involving various types of immune cells. Immunological synapses can have several functions including LYMPHOCYTE ACTIVATION; enhancing, balancing, or terminating signaling; or directing cytokine secretion.'),
  'D055008': NtO(MH='Anthracosis', MS='A diffuse parenchymal lung disease caused by accumulation of inhaled CARBON or coal dust. The disease can progress from asymptomatic anthracosis to massive lung fibrosis. This lung lesion usually occurs in coal MINERS, but can be seen in urban dwellers and tobacco smokers.'),
  'D055009': NtO(MH='Spondylosis', MS='A degenerative spinal disease that can involve any part of the VERTEBRA, the INTERVERTEBRAL DISK, and the surrounding soft tissue.'),
  'D055010': NtO(MH='Posterior Cervical Sympathetic Syndrome', MS='A pathological condition that is characterized by a host of cranial symptoms, such as headaches, abnormal functions of the eyes and the ears, and psychological and mental disorders. This syndrome usually appears after neck injuries, inflammation, or neoplasm.'),
  'D055011': NtO(MH='Ablation Techniques', MS='Removal of tissue by vaporization, abrasion, or destruction. Methods used include heating tissue by hot liquids or microwave thermal heating, freezing (CRYOABLATION), chemical ablation, and photoablation with LASERS.'),
  'D055012': NtO(MH='Chromosomal Position Effects', MS='The effects on gene expression that depend on the location of a gene with respect to its neighboring genes and region of chromosome. Stable position effects are sequence dependent. Variegated position effects depend on whether the gene is located in or adjacent to HETEROCHROMATIN or EUCHROMATIN.'),
  'D055013': NtO(MH='Osteoarthritis, Spine', MS='A degenerative joint disease involving the SPINE. It is characterized by progressive deterioration of the spinal articular cartilage (CARTILAGE, ARTICULAR), usually with hardening of the subchondral bone and outgrowth of bone spurs (OSTEOPHYTE).'),
  'D055014': NtO(MH='Granulocyte-Macrophage Progenitor Cells', MS='The parent cells that give rise to both cells of the GRANULOCYTE lineage and cells of the monocyte/macrophage lineage.'),
  'D055015': NtO(MH='Megakaryocyte-Erythroid Progenitor Cells', MS='The parent cells that give rise to both cells of the MEGAKARYOCYTE and the ERYTHROCYTE lineages.'),
  'D055016': NtO(MH='Megakaryocyte Progenitor Cells', MS='The parent cells that give rise to cells in the MEGAKARYOCYTE lineage, and ultimately BLOOD PLATELETS.'),
  'D055017': NtO(MH='Monocyte-Macrophage Precursor Cells', MS='Parent cells in the lineage that gives rise to MONOCYTES and MACROPHAGES.'),
  'D055018': NtO(MH='Hemangioblasts', MS='Bipotential angio-hematopoietic stem cells that give rise to both HEMATOPOIETIC STEM CELLS and ENDOTHELIAL CELLS.'),
  'D055028': NtO(MH='Comparative Genomic Hybridization', MS='A method for comparing two sets of chromosomal DNA by analyzing differences in the copy number and location of specific sequences. It is used to look for large sequence changes such as deletions, duplications, amplifications, or translocations.'),
  'D055029': NtO(MH='Inverted Repeat Sequences', MS='Copies of nucleic acid sequence that are arranged in opposing orientation. They may lie adjacent to each other (tandem) or be separated by some sequence that is not part of the repeat (hyphenated). They may be true palindromic repeats, i.e. read the same backwards as forward, or complementary which reads as the base complement in the opposite orientation. Complementary inverted repeats have the potential to form hairpin loop or stem-loop structures which results in cruciform structures (such as CRUCIFORM DNA) when the complementary inverted repeats occur in double stranded regions.'),
  'D055030': NtO(MH='Drug Users', MS='People who take drugs for a non-therapeutic or non-medical effect. The drugs may be legal or illegal, but their use often results in adverse medical, legal, or social consequences for the users.'),
  'D055031': NtO(MH='Primary Graft Dysfunction', MS='A form of ischemia-reperfusion injury occurring in the early period following transplantation. Significant pathophysiological changes in MITOCHONDRIA are the main cause of the dysfunction. It is most often seen in the transplanted lung, liver, or kidney and can lead to GRAFT REJECTION.'),
  'D055032': NtO(MH='Electron Microscope Tomography', MS='A tomographic technique for obtaining 3-dimensional images with transmission electron microscopy.'),
  'D055034': NtO(MH='Osteochondrosis', MS='Any of a group of bone disorders involving one or more ossification centers (EPIPHYSES). It is characterized by degeneration or NECROSIS followed by revascularization and reossification. Osteochondrosis often occurs in children causing varying degrees of discomfort or pain. There are many eponymic types for specific affected areas, such as tarsal navicular (Kohler disease) and tibial tuberosity (Osgood-Schlatter disease).'),
  'D055035': NtO(MH='Spinal Osteochondrosis', MS='A bone disorder involving ossification centers (EPIPHYSES) of the VERTEBRAL COLUMN.'),
  'D055036': NtO(MH='Campomelic Dysplasia', MS='A congenital disorder of CHONDROGENESIS and OSTEOGENESIS characterized by hypoplasia of endochondral bones. In most cases there is a curvature of the long bones especially the TIBIA with dimpling of the skin over the bowed areas, malformation of the pelvis and spine, 11 pairs of ribs, hypoplastic scapulae, club feet, micrognathia, CLEFT PALATE, tracheobronchomalacia, and in some patients male-to-female sex reversal (SEX REVERSAL, GONADAL). Most patients die in the neonatal period of respiratory distress. Campomelic dysplasia is associated with haploinsufficiency of the SOX9 TRANSCRIPTION FACTOR gene.'),
  'D055048': NtO(MH='Integrative Medicine', MS='The discipline concerned with using the combination of conventional allopathic medicine and ALTERNATIVE MEDICINE to address the biological, psychological, social, and spiritual aspects of health and illness.'),
  'D055049': NtO(MH='Salt Tolerance', MS='The ability of organisms to sense and adapt to high concentrations of salt in their growth environment.'),
  'D055050': NtO(MH='Diet, Gluten-Free', MS='A diet which is devoid of GLUTENS from WHEAT; BARLEY; RYE; and other wheat-related varieties. The diet is designed to reduce exposure to those proteins in gluten that trigger INFLAMMATION of the small intestinal mucosa in patients with CELIAC DISEASE.'),
  'D055051': NtO(MH='Salt-Tolerant Plants', MS='Plants that can grow well in soils that have a high SALINITY.'),
  'D055052': NtO(MH='Athletic Tape', MS='Adhesive tape with the mechanical strength to resist stretching. It is applied to the skin to support, stabilize, and restrict movement to aid healing and/or prevent injuries of the MUSCULOSKELETAL SYSTEM.'),
  'D055053': NtO(MH='Terahertz Radiation', MS='Non-ionizing electromagnetic energy in the frequency range of 100 gigahertz to 10 terahertz which spans from the mid-INFRARED RAYS frequency to the high-frequency edge of the MICROWAVES band.'),
  'D055068': NtO(MH='Terahertz Spectroscopy', MS='Spectrum analysis of absorbed or emitted TERAHERTZ RADIATION.'),
  'D055069': NtO(MH='Terahertz Imaging', MS='The visualization of internal structure using TERAHERTZ RADIATION technologies.'),
  'D055070': NtO(MH='Resistance Training', MS='A type of strength-building exercise program that requires the body muscle to exert a force against some form of resistance, such as weight, stretch bands, water, or immovable objects. Resistance exercise is a combination of static and dynamic contractions involving shortening and lengthening of skeletal muscles.'),
  'D055088': NtO(MH='Early Detection of Cancer', MS='Methods to identify and characterize cancer in the early stages of disease and predict tumor behavior.'),
  'D055089': NtO(MH='Tracheobronchomalacia', MS='A congenital or acquired condition of underdeveloped or degeneration of CARTILAGE in the TRACHEA and the BRONCHI. This results in a floppy non-rigid airway making patency difficult to maintain.'),
  'D055090': NtO(MH='Tracheomalacia', MS='A congenital or acquired condition of underdeveloped or degeneration of CARTILAGE in the TRACHEA. This results in a floppy tracheal wall making patency difficult to maintain. It is characterized by wheezing and difficult breathing.'),
  'D055091': NtO(MH='Bronchomalacia', MS='A congenital or acquired condition of underdeveloped or degeneration of CARTILAGE in the BRONCHI. This results in a floppy bronchial wall making patency difficult to maintain. It is characterized by wheezing and difficult breathing.'),
  'D055092': NtO(MH='Laryngomalacia', MS='A congenital or acquired condition of underdeveloped or degeneration of CARTILAGE in the LARYNX. This results in a floppy laryngeal wall making patency difficult to maintain.'),
  'D055093': NtO(MH='Periodontal Atrophy', MS='Degradation or wasting of the PERIODONTIUM tissues that may involve the gum (GINGIVA), the alveolar bone (ALVEOLAR PROCESS), the DENTAL CEMENTUM, or the PERIODONTAL LIGAMENT.'),
  'D055094': NtO(MH='Evidence-Based Dentistry', MS="An approach or process of practicing oral health care that requires the judicious integration of systematic assessments of clinical relevant scientific evidence, relating to the patient's oral and medical condition and history, with the dentist's clinical expertise and the patient's treatment needs and preferences. (from J Am Dent Assoc 134: 689, 2003)"),
  'D055095': NtO(MH='Optics and Photonics', MS='A specialized field of physics and engineering involved in studying the behavior and properties of light and the technology of analyzing, generating, transmitting, and manipulating ELECTROMAGNETIC RADIATION in the visible, infrared, and ultraviolet range.'),
  'D055096': NtO(MH='Optical Devices', MS='Products or parts of products used to detect, manipulate, or analyze light, such as LENSES, refractors, mirrors, filters, prisms, and OPTICAL FIBERS.'),
  'D055097': NtO(MH='Auriculotherapy', MS='Treatment of pain, drug addictions, or other ailments by stimulating the various points on the external ear (EAR AURICLES). It is based on the ancient Chinese practices of EAR ACUPUNCTURE, but sometimes magnets and other modes of stimulation are used.'),
  'D055098': NtO(MH='Endocrine Cells', MS='Secretory cells of the ductless glands. They secrete HORMONES directly into the blood circulation (internal secretion) to be carried to the target cells. The secreted chemicals can be PEPTIDES; STEROIDS; NEUROPEPTIDES; or BIOGENIC AMINES.'),
  'D055099': NtO(MH='Neuroendocrine Cells', MS='Specialized NEURONS that produce hormones, such as NEUROPEPTIDES or BIOGENIC AMINES. They generally are in the NERVOUS SYSTEM, such as HYPOTHALAMUS, but can be found in other organs or systems. These neurons contain dense neurosecretory granules and PROPROTEIN CONVERTASES allowing the rapidly release of NEUROHORMONES into the blood circulation upon stimulation.'),
  'D055100': NtO(MH='Optical Fibers', MS='Thin strands of transparent material, usually glass, that are used for transmitting light waves over long distances.'),
  'D055101': NtO(MH='Semen Analysis', MS='The quality of SEMEN, an indicator of male fertility, can be determined by semen volume, pH, sperm concentration (SPERM COUNT), total sperm number, sperm viability, sperm vigor (SPERM MOTILITY), normal sperm morphology, ACROSOME integrity, and the concentration of WHITE BLOOD CELLS.'),
  'D055103': NtO(MH='Hypodermoclysis', MS='Technique for treating DEHYDRATION and WATER-ELECTROLYTE IMBALANCE by subcutaneous infusion of REHYDRATION SOLUTIONS.'),
  'D055104': NtO(MH='Infusions, Subcutaneous', MS='The administration of liquid medication or nutrients under the skin, usually over minutes or hours.'),
  'D055105': NtO(MH='Waist Circumference', MS='The measurement around the body at the level of the ABDOMEN and just above the hip bone. The measurement is usually taken immediately after exhalation.'),
  'D055106': NtO(MH='Genome-Wide Association Study', MS='An analysis comparing the allele frequencies of all available (or a whole GENOME representative set of) polymorphic markers in unrelated patients with a specific symptom or disease condition, and those of healthy controls to identify markers associated with a specific disease or condition.'),
  'D055107': NtO(MH='Suntan', MS='An induced skin pigment (MELANIN) darkening after exposure to SUNLIGHT or ULTRAVIOLET RAYS. The degree of tanning depends on the intensity and duration of UV exposure, and genetic factors.'),
  'D055108': NtO(MH='Sunbathing', MS='Exposing oneself to SUNLIGHT or ULTRAVIOLET RAYS.'),
  'D055109': NtO(MH='Ankle Brachial Index', MS='Comparison of the BLOOD PRESSURE between the BRACHIAL ARTERY and the POSTERIOR TIBIAL ARTERY. It is a predictor of PERIPHERAL ARTERIAL DISEASE.'),
  'D055110': NtO(MH='Radiometric Dating', MS='Techniques used to determine the age of materials, based on the content and half-lives of the RADIOACTIVE ISOTOPES they contain.'),
  'D055111': NtO(MH='Failed Back Surgery Syndrome', MS='A condition of persistent pain and discomfort in the BACK and the LEG following lumbar surgery, often seen in patients enrolled in pain centers.'),
  'D055112': NtO(MH='Pyometra', MS='An accumulation of PUS in the uterine cavity (UTERUS). Pyometra generally indicates the presence of infections.'),
  'D055113': NtO(MH='Chronic Periodontitis', MS='Chronic inflammation and loss of PERIODONTIUM that is associated with the amount of DENTAL PLAQUE or DENTAL CALCULUS present. Chronic periodontitis occurs mostly in adults and was called adult periodontitis, but this disease can appear in young people.'),
  'D055114': NtO(MH='X-Ray Microtomography', MS='X-RAY COMPUTERIZED TOMOGRAPHY with resolution in the micrometer range.'),
  'D055115': NtO(MH='Light-Curing of Dental Adhesives', MS='The hardening or polymerization of bonding agents (DENTAL CEMENTS) via exposure to light.'),
  'D055116': NtO(MH='Self-Curing of Dental Resins', MS='The hardening or polymerization of bonding agents (DENTAL CEMENTS) via chemical reactions, usually involving two components. This type of dental bonding uses a self-cure or dual-cure system.'),
  'D055117': NtO(MH='Curing Lights, Dental', MS='Light sources used to activate polymerization of light-cured DENTAL CEMENTS and DENTAL RESINS. Degree of cure and bond strength depends on exposure time, wavelength, and intensity of the curing light.'),
  'D055118': NtO(MH='Medication Adherence', MS='Voluntary cooperation of the patient in taking drugs or medicine as prescribed. This includes timing, dosage, and frequency.'),
  'D055119': NtO(MH='Elastic Modulus', MS='Numerical expression indicating the measure of stiffness in a material. It is defined by the ratio of stress in a unit area of substance to the resulting deformation (strain). This allows the behavior of a material under load (such as bone) to be calculated.'),
  'D055133': NtO(MH='Blastocladiomycota', MS='A phylum of flagellated fungi, formerly the order Blastocladiales under CHYTRIDIOMYCOTA.'),
  'D055134': NtO(MH='Allomyces', MS='A genus of aquatic fungi in the phylum BLASTOCLADIOMYCOTA. It is abundant in warmer climates and exhibits true alternation of generations.'),
  'D055135': NtO(MH='Coelomomyces', MS='A genus of fungi in the phylum BLASTOCLADIOMYCOTA. They exhibit complex life cycles and are parasitic on the larvae of MOSQUITOES.'),
  'D055136': NtO(MH='Neocallimastigomycota', MS='A phylum of anaerobic fungi found in the stomachs of RUMINANTS. These fungi lack mitochondria and reproduce through zoospores.'),
  'D055137': NtO(MH='Glomeromycota', MS='A phylum of fungi that are mutualistic symbionts and form ARBUSCULAR MYCORRHIZAE with PLANT ROOTS.'),
  'D055139': NtO(MH='Sensory Gating', MS='The ability of the BRAIN to suppress neuronal responses to external sensory inputs, such as auditory and visual stimuli. Sensory filtering (or gating) allows humans to block out irrelevant, meaningless, or redundant stimuli.'),
  'D055153': NtO(MH='Stem Cell Niche', MS='A particular zone of tissue composed of a specialized microenvironment where stem cells are retained in a undifferentiated, self-renewable state.'),
  'D055154': NtO(MH='Dysphonia', MS='Difficulty and/or pain in PHONATION or speaking.'),
  'D055155': NtO(MH='Ophiostomatales', MS='An order of fungi in the phylum ASCOMYCOTA that are important plant pathogens.'),
  'D055156': NtO(MH='Ophiostoma', MS='A genus of fungi in the family Ophiostomataceae, order OPHIOSTOMATALES. Several species are the source of Dutch elm disease, which is spread by the elm bark beetle.'),
  'D055157': NtO(MH='Dipodascus', MS='A genus of ascomycetous yeast in the family Dipodascaceae, order SACCHAROMYCETALES. It is characterized by asci which have persistent walls and by smooth-walled ascospores. Anamorphs are in the genus GEOTRICHUM.'),
  'D055158': NtO(MH='Velopharyngeal Sphincter', MS='A group of muscles attached to the SOFT PALATE (or velum) and the PHARYNX. They include the superior constrictor, the PALATOPHARYNGEUS, the levator veli palatini muscle, and the muscularis uvulae. This sphincter is situated between the oral and nasal cavities. A competent velopharyngeal sphincter is essential for normal speech and swallowing.'),
  'D055159': NtO(MH='Endomyces', MS='A genus of ascomycetous yeast in the family Endomycetaceae, order SACCHAROMYCETALES. They have been described as ascus-bearing parasites of basidiocarps, the fruiting bodies of BASIDIOMYCOTA.'),
  'D055160': NtO(MH='Lipomyces', MS='A genus of ascomycetous soil yeast in the family Lipomycetaceae, order SACCHAROMYCETALES.'),
  'D055161': NtO(MH='Speleotherapy', MS='The use of CAVES, mines, or other subterranean environments in treatment of diseases. Speleotherapy is used in some Central and Eastern European countries to treat CHRONIC OBSTRUCTIVE AIRWAY DISEASE.'),
  'D055162': NtO(MH='Biocatalysis', MS='The facilitation of biochemical reactions with the aid of naturally occurring catalysts such as ENZYMES.'),
  'D055163': NtO(MH='Metschnikowia', MS='A genus of ascomycetous yeast in the family Metschnikowiaceae, order SACCHAROMYCETALES. Its antifungal activity is used to inhibit postharvest decay of fruit.'),
  'D055164': NtO(MH='Nerve Expansion', MS='Procedures that stimulate nerve elongation over a period of time. They are used in repairing nerve tissue.'),
  'D055165': NtO(MH='Debaromyces', MS='A genus of ascomycetous yeast in the family Saccharomycetaceae, order SACCHAROMYCETALES. Debaryomyces hansenni is a salt-tolerant marine species.'),
  'D055166': NtO(MH='Hepatic Stellate Cells', MS='Perisinusoidal cells of the liver, located in the space of Disse between HEPATOCYTES and sinusoidal endothelial cells.'),
  'D055167': NtO(MH='Dekkera', MS='A genus of ascomycetous yeast in the family Saccharomycetaceae, order SACCHAROMYCETALES. It has been implicated in wine spoilage. Anamorphic forms are found in the genus BRETTANOMYCES.'),
  'D055168': NtO(MH='Brettanomyces', MS='The anamorphic form of the yeast DEKKERA. It is implicated as a spoilage microorganism in wines.'),
  'D055169': NtO(MH='Eremothecium', MS='A genus of ascomycetous yeast in the family Saccharomycetaceae, order SACCHAROMYCETALES. Species in the genus are plant pathogens.'),
  'D055170': NtO(MH='Plant Root Nodulation', MS='The formation of a nitrogen-fixing cell mass on PLANT ROOTS following symbiotic infection by nitrogen-fixing bacteria such as RHIZOBIUM or FRANKIA.'),
  'D055171': NtO(MH='Nasal Cartilages', MS='Hyaline cartilages in the nose. There are five major nasal cartilages including two lateral, two alar, and one septal.'),
  'D055172': NtO(MH='Vomer', MS='An unpaired thin ploughshare-shaped facial bone. It is situated in the median plane of the SKULL. The vomer forms the posterior and inferior border of the NASAL SEPTUM.'),
  'D055191': NtO(MH='Delayed Emergence from Anesthesia', MS='Abnormally slow pace of regaining CONSCIOUSNESS after general anesthesia (ANESTHESIA, GENERAL) usually given during surgical procedures. This condition is characterized by persistent somnolence.'),
  'D055211': NtO(MH='Retinal Photoreceptor Cell Inner Segment', MS='The inner portion of a retinal rod or a cone photoreceptor cell, situated between the PHOTORECEPTOR CONNECTING CILIUM and the synapse with the adjacent neurons (RETINAL BIPOLAR CELLS; RETINAL HORIZONTAL CELLS). The inner segment contains the cell body, the nucleus, the mitochondria, and apparatus for protein synthesis.'),
  'D055212': NtO(MH='Photoreceptor Connecting Cilium', MS='The bridge between the inner and the outer segments of a retinal rod or a cone photoreceptor cell. Through it, proteins synthesized in the inner segment are transported to the outer segment.'),
  'D055213': NtO(MH='Retinal Pigment Epithelium', MS='The single layer of pigment-containing epithelial cells in the RETINA, situated closely to the tips (outer segments) of the RETINAL PHOTORECEPTOR CELLS. These epithelial cells are macroglia that perform essential functions for the photoreceptor cells, such as in nutrient transport, phagocytosis of the shed photoreceptor membranes, and ensuring retinal attachment.'),
  'D055214': NtO(MH='Retinal Photoreceptor Cell Outer Segment', MS='The light sensitive outer portion of a retinal rod or a cone photoreceptor cell. The outer segment contains a stack of disk membranes laden with photoreceptive pigments (RETINAL PIGMENTS). The outer segment is connected to the inner segment by a PHOTORECEPTOR CONNECTING CILIUM.'),
  'D055231': NtO(MH='Lincosamides', MS='A family of LINCOMYCIN-related glycosides that contain a pyrrolidine ring linked via an amide-bond to a pyranose moiety. Individual members of this family are defined by the arrangement of specific constituent groups on the lyncomycin molecule. Many lincosamides are ANTIBIOTICS produced by a variety STREPTOMYCES species.'),
  'D055232': NtO(MH='Proline-Rich Protein Domains', MS='Protein domains that are enriched in PROLINE. The cyclical nature of proline causes the peptide bonds it forms to have a limited degree of conformational mobility. Therefore the presence of multiple prolines in close proximity to each other can convey a distinct conformational arrangement to a peptide chain.'),
  'D055233': NtO(MH='Cornified Envelope Proline-Rich Proteins', MS='A family of low molcular-weight proteins that contain PROLINE-RICH PROTEIN DOMAINS. Members of this family play a role in the formation of an insoluble cornified envelope beneath the plasma membrane of stratified squamous epithelial cells.'),
  'D055252': NtO(MH='Cell-Derived Microparticles', MS='Extracellular vesicles generated by the shedding of CELL MEMBRANE blebs.'),
  'D055253': NtO(MH='Color Vision', MS='Function of the human eye that is used in bright illumination or in daylight (at photopic intensities). Photopic vision is performed by the three types of RETINAL CONE PHOTORECEPTORS with varied peak absorption wavelengths in the color spectrum (from violet to red, 400 - 700 nm).'),
  'D055254': NtO(MH='Night Vision', MS='Function of the human eye that is used in dim illumination (scotopic intensities) or at nighttime. Scotopic vision is performed by RETINAL ROD PHOTORECEPTORS with high sensitivity to light and peak absorption wavelength at 507 nm near the blue end of the spectrum.'),
  'D055255': NtO(MH='Mesopic Vision', MS='The function of the eye that is used in the intermediate level of illumination (mesopic intensities) where both the RETINAL ROD PHOTORECEPTORS and the RETINAL CONE PHOTORECEPTORS are active in processing light input simultaneously.'),
  'D055256': NtO(MH='Infectious Disease Incubation Period', MS='The amount time between exposure to an infectious agent and becoming symptomatic.'),
  'D055257': NtO(MH='Mucin-5B', MS='A gel-forming mucin that is predominantly expressed by submucosal glands of airway tissues and the SUBLINGUAL GLAND. It is one of the principal components of high molecular weight salivary mucin.'),
  'D055258': NtO(MH='Rod-Cone Interaction', MS='Reciprocal action of two vertebrate photoreceptor cells (RODS AND CONES). Rod-cone interaction occurs during MESOPIC VISION in which both rods and cones are active in light transduction to the VISUAL CORTEX. Such interaction can influence visual sensitivity and luminous efficiency.'),
  'D055261': NtO(MH='Immunogenetic Phenomena', MS='GENETIC PHENOMENA and processes involved in IMMUNITY and the immune response.'),
  'D055262': NtO(MH='Mucin-2', MS='A gel-forming mucin found predominantly in SMALL INTESTINE and variety of mucous membrane-containing organs. It provides a protective, lubricating barrier against particles and infectious agents.'),
  'D055263': NtO(MH='Mucin-3', MS='A membrane-bound mucin subtype that is primarily found in INTESTINAL MUCOSA. Two closely-related subtypes of this protein have been identified in humans.'),
  'D055264': NtO(MH='Mucin-4', MS='A transmembrane mucin that is found in a broad variety of epithelial tissue. Mucin-4 may play a role in regulating cellular adhesion and in cell surface signaling from the ERBB-2 RECEPTOR PROTEIN-TYROSINE KINASE. Mucin-4 is a heterodimer of alpha and beta chains. The alpha and beta chains result from the proteolytic cleavage of a precursor protein.'),
  'D055271': NtO(MH='Mucin 5AC', MS='A gel-forming mucin that is primarily found on the surface of gastric epithelium and in the RESPIRATORY TRACT. Mucin 5AC was originally identified as two distinct proteins, however a single gene encodes the protein which gives rise to the mucin 5A and mucin 5C variants.'),
  'D055272': NtO(MH='Mucin-6', MS='A gel-forming mucin that is predominantly associated with the gastric epithelium.'),
  'D055273': NtO(MH='Salivary Proline-Rich Proteins', MS='A family of proline-rich proteins that constitute the majority of the protein component of SALIVA. Salivary proline-rich proteins occur as acidic, basic and glycosylated basic proteins. They perform a variety of functions such as adhering to the acquired ENAMEL PELLICLE, acting as lubricants and precipitating TANNINS.'),
  'D055291': NtO(MH='Salivary alpha-Amylases', MS='A subclass of alpha-amylase ISOENZYMES that are secreted into SALIVA.'),
  'D055292': NtO(MH='Pancreatic alpha-Amylases', MS='A subclass of alpha-amylase ISOENZYMES that are secreted into PANCREATIC JUICE.'),
  'D055293': NtO(MH='Receptors, Urokinase Plasminogen Activator', MS='An extracellular receptor specific for UROKINASE-TYPE PLASMINOGEN ACTIVATOR. It is attached to the cell membrane via a GLYCOSYLPHOSPHATIDYLINOSITOL LINKAGE and plays a role in the co-localization of urokinase-type plasminogen activator with PLASMINOGEN.'),
  'D055312': NtO(MH='Cystatin A', MS='A cytastin subtype found at high levels in the SKIN and in BLOOD CELLS. Cystatin A incorporates into the cornified cell envelope of stratified squamous epithelial cells and may play a role in bacteriostatic properties of skin.'),
  'D055313': NtO(MH='Cystatin B', MS='An intracellular cystatin subtype that is found in a broad variety of cell types. It is a cytosolic enzyme inhibitor that protects the cell against the proteolytic action of lysosomal enzymes such as CATHEPSINS.'),
  'D055314': NtO(MH='Seed Storage Proteins', MS='One or more types of plant seed proteins providing the large amounts of AMINO ACIDS utilized in GERMINATION and SEEDLING growth. As seeds are the major food source from AGRICULTURAL CROPS, seed storage proteins are a major source of DIETARY PROTEINS.'),
  'D055315': NtO(MH='Prolamins', MS='A group of seed storage proteins restricted to the POACEAE family. They are rich in GLUTAMINE and PROLINE.'),
  'D055316': NtO(MH='Cystatin C', MS='An extracellular cystatin subtype that is abundantly expressed in bodily fluids. It may play a role in the inhibition of interstitial CYSTEINE PROTEASES.'),
  'D055317': NtO(MH='Evidence-Based Practice', MS='A way of providing health care that is guided by a thoughtful integration of the best available scientific knowledge with clinical expertise. This approach allows the practitioner to critically assess research data, clinical guidelines, and other information resources in order to correctly identify the clinical problem, apply the most high-quality intervention, and re-evaluate the outcome for future improvement.'),
  'D055318': NtO(MH='Evidence-Based Nursing', MS='A way of providing nursing care that is guided by the integration of the best available scientific knowledge with nursing expertise. This approach requires nurses to critically assess relevant scientific data or research evidence, and to implement high-quality interventions for their nursing practice.'),
  'D055319': NtO(MH='Torulaspora', MS='A genus of ascomycetous yeast in the family Saccharomycetaceae, order SACCHAROMYCETALES. It is used commercially in the production of wines and beer.'),
  'D055320': NtO(MH='Williopsis', MS='A genus of ascomycetous yeast in the family Saccharomycetaceae, order SACCHAROMYCETALES. Many species show mycocinogenic activity against other yeasts.'),
  'D055321': NtO(MH='Hanseniaspora', MS='A genus of the ascomycetous yeast in the family Saccharomycodaceae, order SACCHAROMYCETALES, that contributes to the spontaneous fermentation of cider. Anamorphic forms are in the genus KLOECKERA.'),
  'D055322': NtO(MH='Kloeckera', MS='A mitosporic fungal genus used in the fermentation of wine. Teleomorphs are in the genus HANSENIASPORA.'),
  'D055323': NtO(MH='Byssochlamys', MS='A genus of ascomycetous mold in the family Trichocomaceae, order EUROTIALES. Byssochlamys species are responsible for spoilage and degradation of fruit and fruit juices. Anamorphs are found in the genus PAECILOMYCES.'),
  'D055324': NtO(MH='Eupenicillium', MS='A genus of endophytic, ascomycetous mold in the family Trichocomaceae, order EUROTIALES. Anamorphic forms are in the genus PENICILLIUM.'),
  'D055325': NtO(MH='Eurotium', MS='A genus of ascomycetous fungi in the family Trichocomaceae, order EUROTIALES. Health effects, allergenicity, and toxicity of Eurotium are closely related to its anamorph ASPERGILLUS.'),
  'D055326': NtO(MH='Neosartorya', MS='A genus of ascomycetous fungi in the family Trichocomaceae, order EUROTIALES. Some species can cause opportunistic infections in humans, similar to its anamorph ASPERGILLUS.'),
  'D055327': NtO(MH='Thermoascus', MS='A genus of thermophilic, ascomycetous fungi in the family Trichocomaceae, order EUROTIALES. Anamorphic forms are in the genus PAECILOMYCES.'),
  'D055328': NtO(MH='Neotyphodium', MS='The anamorphic form of the fungus EPICHLOE. Many Neotyphodium species produce ERGOT ALKALOIDS.'),
  'D055329': NtO(MH='Epichloe', MS='A genus of ascomycetous fungi in the family Clavicipitaceae, order HYPOCREALES, which are fungal symbionts of grasses. Anamorphic forms are in the genus NEOTYPHODIUM.'),
  'D055330': NtO(MH='Nectria', MS='A genus of ascomycetous fungi in the family Nectriaceae, order HYPOCREALES. They are found as saprophytes on decaying wood or parasites of trees. Anamorphs include FUSARIUM and Cylindrocarpon.'),
  'D055331': NtO(MH='Adenomyoepithelioma', MS='A mixed epithelial and myoepithelial neoplasm usually encountered within the breast. It is a multinodular lesion presenting with a bicellular pattern of proliferating epithelial and myoepithelial cells, which are regularly distributed in the glandular structures.'),
  'D055332': NtO(MH='Cystatin M', MS='A cystatin subtype that has a diverse tissue distribution, target specificity, and functions as an endogenous inhibitor of lysosomal cysteine proteases.'),
  'D055333': NtO(MH='Salivary Cystatins', MS='A group of closely-related cystatins found in SALIVA.'),
  'D055351': NtO(MH='Retinal Neurons', MS='Nerve cells of the RETINA in the pathway of transmitting light signals to the CENTRAL NERVOUS SYSTEM. They include the outer layer of PHOTORECEPTOR CELLS, the intermediate layer of RETINAL BIPOLAR CELLS and AMACRINE CELLS, and the internal layer of RETINAL GANGLION CELLS.'),
  'D055352': NtO(MH='Halorubrum', MS='A genus of HALOBACTERIACEAE which are chemoorganotrophic and strictly aerobic. They have been isolated from multiple hypersaline environments that vary widely in chemical and physical properties.'),
  'D055353': NtO(MH='Chromohalobacter', MS='A genus of gram-negative, moderately halophilic bacteria in the family HALOMONADACEAE. They are chemoorganotrophic and grow optimally in media containing 8-10% salt.'),
  'D055354': NtO(MH='Exosomes', MS='A type of extracellular vesicle, containing RNA and proteins, that is secreted into the extracellular space by EXOCYTOSIS when MULTIVESICULAR BODIES fuse with the PLASMA MEMBRANE.'),
  'D055355': NtO(MH='Opsins', MS='Photosensitive proteins in the membranes of PHOTORECEPTOR CELLS such as the rods and the cones. Opsins have varied light absorption properties and are members of the G-PROTEIN-COUPLED RECEPTORS family. Their ligands are VITAMIN A-based chromophores.'),
  'D055356': NtO(MH='Endometrial Ablation Techniques', MS='Procedures used for the targeted destruction of the mucous membrane lining of the uterine cavity.'),
  'D055357': NtO(MH='Uterine Artery Embolization', MS='The use of embolizing agents to block the arterial blood supply to parts or all of the UTERUS. The procedures are done to control bleeding or to cause destruction of uterine tissues.'),
  'D055358': NtO(MH='Cone Opsins', MS='Photosensitive proteins expressed in the CONE PHOTORECEPTOR CELLS. They are the protein components of cone photopigments. Cone opsins are classified by their peak absorption wavelengths.'),
  'D055359': NtO(MH='Antistatic Agents', MS='Chemical compounds applied to materials to reduce their retention of an electrostatic charge.'),
  'D055360': NtO(MH='Osteopathic Physicians', MS='Licensed physicians trained in OSTEOPATHIC MEDICINE. An osteopathic physician, also known as D.O. (Doctor of Osteopathy), is able to perform surgery and prescribe medications.'),
  'D055361': NtO(MH='Multilevel Analysis', MS='The statistical manipulation of hierarchically and non-hierarchically nested data. It includes clustered data, such as a sample of subjects within a group of schools. Prevalent in the social, behavioral sciences, and biomedical sciences, both linear and nonlinear regression models are applied.'),
  'D055362': NtO(MH='Agrocybe', MS='A genus of saprobic mushrooms in the family Bolbitiaceae that grow in grass, dung, garden mulch, or in woods.'),
  'D055363': NtO(MH='Cortinarius', MS='A genus of mushrooms in the family Cortinariaceae. When ingested, species of Cortinarius cause delayed acute RENAL FAILURE.'),
  'D055364': NtO(MH='Hebeloma', MS='A genus of ectomycorrhizae basidiomycetous fungi in the family Cortinariaceae. Some species are poisonous.'),
  'D055365': NtO(MH='Cyathus', MS="A genus of saprophytic fungi in the family Nidulariaceae. They look like small bird's nests complete with eggs. The eggs are small capsules known as peridioles, which contain the spores."),
  'D055366': NtO(MH='Synaptic Potentials', MS='The voltages across pre- or post-SYNAPTIC MEMBRANES.'),
  'D055367': NtO(MH='Volvariella', MS='A genus of edible mushrooms in the family Pluteaceae. They have pink gills and a volva at the stem base, and species can be confused with those of the poisonous genus AMANITA.'),
  'D055368': NtO(MH='Pholiota', MS='A genus of basidiomycetous mushroom in the family Strophariaceae, exhibiting a bipolar mating system.'),
  'D055369': NtO(MH='Miniature Postsynaptic Potentials', MS='Postsynaptic potentials generated from a release of neurotransmitters from a presynaptic nerve terminal in the absence of an ACTION POTENTIAL. They may be m.e.p.p.s (miniature EXCITATORY POSTSYNAPTIC POTENTIALS) or m.i.p.p.s (miniature INHIBITORY POSTSYNAPTIC POTENTIALS).'),
  'D055370': NtO(MH='Lung Injury', MS='Damage to any compartment of the lung caused by physical, chemical, or biological agents which characteristically elicit inflammatory reaction. These inflammatory reactions can either be acute and dominated by NEUTROPHILS, or chronic and dominated by LYMPHOCYTES and MACROPHAGES.'),
  'D055371': NtO(MH='Acute Lung Injury', MS='A condition of lung damage that is characterized by bilateral pulmonary infiltrates (PULMONARY EDEMA) rich in NEUTROPHILS, and in the absence of clinical HEART FAILURE. This can represent a spectrum of pulmonary lesions, endothelial and epithelial, due to numerous factors (physical, chemical, or biological).'),
  'D055372': NtO(MH='AMP-Activated Protein Kinases', MS='Intracellular signaling protein kinases that play a signaling role in the regulation of cellular energy metabolism. Their activity largely depends upon the concentration of cellular AMP which is increased under conditions of low energy or metabolic stress. AMP-activated protein kinases modify enzymes involved in LIPID METABOLISM, which in turn provide substrates needed to convert AMP into ATP.'),
  'D055391': NtO(MH='Psilocybe', MS='A genus of small mushrooms in the family Strophariaceae. Some species are known for their hallucinogenic properties.'),
  'D055392': NtO(MH='Armillaria', MS='A genus of root and butt rot fungi in the family Tricholomataceae that produce rhizomorphs and are facultatively parasitic. Many species are pathogenic to trees causing Armillaria root disease.'),
  'D055393': NtO(MH='Flammulina', MS='A genus of mushrooms in the family Tricholomataceae, whose species are characterized by a slimy cap (FRUITING BODIES, FUNGAL).'),
  'D055394': NtO(MH='Tolloid-Like Metalloproteinases', MS='A family of metalloproteases that are related to the DROSOPHILA protein tolloid, which is a gene product necessary for dorsal-ventral patterning in early Drosophila embryogenesis. Many members of the group may play a significant role in intercellular signaling.'),
  'D055395': NtO(MH='Bone Morphogenetic Protein 1', MS='A bone morphogenetic protein family member that includes an active tolloid-like metalloproteinase domain. The metalloproteinase activity of bone morphogenetic protein 1 is specific for the removal of the C-propeptide of PROCOLLAGEN and may act as a regulator of EXTRACELLULAR MATRIX deposition. Alternative splicing of MRNA for bone morphogenetic protein 1 results in the production of several PROTEIN ISOFORMS.'),
  'D055396': NtO(MH='Bone Morphogenetic Protein 2', MS='A potent osteoinductive protein that plays a critical role in the differentiation of osteoprogenitor cells into OSTEOBLASTS.'),
  'D055397': NtO(MH='Ventilator-Induced Lung Injury', MS='Lung damage that is caused by the adverse effects of PULMONARY VENTILATOR usage. The high frequency and tidal volumes produced by a mechanical ventilator can cause alveolar disruption and PULMONARY EDEMA.'),
  'D055398': NtO(MH='Bone Morphogenetic Protein 3', MS='A bone morphogenetic protein that is found at high concentrations in a purified osteoinductive protein fraction from BONE. Bone morphogenetic protein 3 is referred to as osteogenin, however it may play a role in variety of developmental processes.'),
  'D055399': NtO(MH='Laccaria', MS='A genus of white-spored mushrooms in the family Tricholomataceae. They form symbiotic partnerships (MYCORRHIZAE) with trees.'),
  'D055411': NtO(MH='TGF-beta Superfamily Proteins', MS='A large family of cell regulatory proteins which are structurally related to TRANSFORMING GROWTH FACTOR BETA. The superfamily is subdivided into at least three related protein families: BONE MORPHOGENETIC PROTEINS; GROWTH DIFFERENTIATION FACTORS; and TRANSFORMING GROWTH FACTORS.'),
  'D055412': NtO(MH='Growth Differentiation Factors', MS='A family of BONE MORPHOGENETIC PROTEIN-related proteins that are primarily involved in regulation of CELL DIFFERENTIATION.'),
  'D055413': NtO(MH='Growth Differentiation Factor 10', MS='A growth differentiation factor that is closely-related in structure to BONE MORPHOGENETIC PROTEIN 3. Growth differentiation factor 10 is found at high levels in BONE, however it plays an additional roles in regulating EMBRYONIC DEVELOPMENT.'),
  'D055414': NtO(MH='Myocardial Perfusion Imaging', MS='The creation and display of functional images showing where the blood is flowing into the MYOCARDIUM by following over time the distribution of tracers injected into the blood stream.'),
  'D055415': NtO(MH='Bone Morphogenetic Protein 4', MS='A bone morphogenetic protein that is a potent inducer of bone formation. It also functions as a regulator of MESODERM formation during EMBRYONIC DEVELOPMENT.'),
  'D055416': NtO(MH='Evidence-Based Emergency Medicine', MS='A way of providing emergency medical care that is guided by a thoughtful integration of the best available scientific knowledge with clinical expertise in EMERGENCY MEDICINE. This approach allows the practitioner to critically assess research data, clinical guidelines, and other information resources in order to correctly identify the clinical problem, apply the most high-quality intervention, and re-evaluate the outcome for future improvement.'),
  'D055417': NtO(MH='Bone Morphogenetic Protein 5', MS='A bone morphogenetic protein that may play a role in CARTILAGE formation. It is a potent regulator of the growth of CHONDROCYTES and the synthesis of cartilage matrix proteins. Evidence for its role in cartilage formation can be seen in MICE, where genetic mutations that cause loss of bone morphogenetic protein 5 function result in the formation of small malformed ears.'),
  'D055418': NtO(MH='Bone Morphogenetic Protein 6', MS='A bone morphogenetic protein that is a potent inducer of BONE formation. It plays additional roles in regulating CELL DIFFERENTIATION of non-osteoblastic cell types and epithelial-mesenchymal interactions.'),
  'D055419': NtO(MH='Bone Morphogenetic Protein 7', MS='A bone morphogenetic protein that is widely expressed during EMBRYONIC DEVELOPMENT. It is both a potent osteogenic factor and a specific regulator of nephrogenesis.'),
  'D055420': NtO(MH='Perfusion Imaging', MS='The creation and display of functional images showing where the blood flow reaches by following the distribution of tracers injected into the blood stream.'),
  'D055422': NtO(MH='Venous Valves', MS='Flaps within the VEINS that allow the blood to flow only in one direction. They are usually in the medium size veins that carry blood to the heart against gravity.'),
  'D055423': NtO(MH='Diet, Ketogenic', MS='A course of food intake that is high in FATS and low in CARBOHYDRATES. This diet provides sufficient PROTEINS for growth but insufficient amount of carbohydrates for the energy needs of the body. A ketogenic diet generates 80-90% of caloric requirements from fats and the remainder from proteins.'),
  'D055424': NtO(MH='Cardiac-Gated Imaging Techniques', MS='Timing the acquisition of imaging data to specific points in the cardiac cycle to minimize image blurring and other motion artifacts.'),
  'D055425': NtO(MH='Respiratory-Gated Imaging Techniques', MS='Timing the acquisition of imaging data to specific points in the breathing cycle to minimize image blurring and other motion artifacts. The images are used diagnostically and also interventionally to coordinate radiation treatment beam on/off cycles to protect healthy tissues when they move into the beam field during different times in the breathing cycle.'),
  'D055426': NtO(MH='Cardiac-Gated Single-Photon Emission Computer-Assisted Tomography', MS='Tomography using single-photon emitting RADIONUCLIDES to create images that are captured in times corresponding to various points in the cardiac cycle.'),
  'D055427': NtO(MH='Growth Differentiation Factor 2', MS='A growth differentiation factor that plays a regulatory role as a paracrine factor for a diverse array of cell types during EMBRYONIC DEVELOPMENT and in the adult tissues. Growth differentiation factor 2 is also a potent regulator of CHONDROGENESIS and was previously referred to as bone morphogenetic protein 9.'),
  'D055428': NtO(MH='Growth Differentiation Factor 5', MS='A growth differentiation factor that plays a role in early CHONDROGENESIS and joint formation.'),
  'D055429': NtO(MH='Growth Differentiation Factor 9', MS='A bone morphogenetic protein that plays an essential role in the regulation of ovarian folliculogenesis.'),
  'D055430': NtO(MH='Bone Morphogenetic Protein 15', MS='A protein that plays a role in GRANULOSA CELLS where it regulates folliculogenesis. Mutations in the gene for bone morphogenetic protein 15 are linked to reproductive abnormalities such as PREMATURE OVARIAN FAILURE.'),
  'D055431': NtO(MH='Growth Differentiation Factor 6', MS='A growth differentiation factor that plays a role in the neural differentiation, specifically in the retinal development of the EYE.'),
  'D055432': NtO(MH='Metabolomics', MS='The systematic identification and quantitation of all the metabolic products of a cell, tissue, organ, or organism under varying conditions. The METABOLOME of a cell or organism is a dynamic collection of metabolites which represent its net response to current conditions.'),
  'D055433': NtO(MH='Marasmius', MS='A genus of tiny mushrooms in the family Tricholomataceae. They help break down the decaying organic matter of the forest floor.'),
  'D055434': NtO(MH='Termitomyces', MS='A paleotropical genus of fungi in the family Tricholomataceae. They are obligate symbionts of termites.'),
  'D055435': NtO(MH='Myostatin', MS='A growth differentiation factor that is a potent inhibitor of SKELETAL MUSCLE growth. It may play a role in the regulation of MYOGENESIS and in muscle maintenance during adulthood.'),
  'D055436': NtO(MH='Growth Differentiation Factor 15', MS='A growth differentiation factor that is secreted in response to cell stress and in response to MACROPHAGE ACTIVATION. In addition growth differentiation factor 15 demonstrates a diverse array of biological properties including the induction of cartilage formation, the inhibition of hematopoietic progenitor proliferation, and the induction of neuronal migration.'),
  'D055439': NtO(MH='Tricholoma', MS='A large genus of gilled mushrooms in the family Tricholomataceae. They are most often found in mountain and northern forests.'),
  'D055441': NtO(MH='Poria', MS='A genus of basiodiomycetous fungi in the family Coriolaceae. Members are known for infesting wood.'),
  'D055442': NtO(MH='Metabolome', MS="The dynamic collection of metabolites which represent a cell's or organism's net metabolic response to current conditions."),
  'D055451': NtO(MH='Growth Differentiation Factor 3', MS='A growth differentiation factor that may play a role in maintaining the undifferentiated state of PLURIPOTENT STEM CELLS by inhibiting the actions of BONE MORPHOGENETIC PROTEINS. Differentiation factor 3 is also considered a nodal signaling ligand that influences the genesis of left-right asymmetry.'),
  'D055452': NtO(MH='Growth Differentiation Factor 1', MS='A growth differentiation factor that plays a role in the genesis of left-right asymmetry during vertebrate development. Evidence for this role is seen in MICE where loss of growth differentiation factor 1 function results in right-left isomerism of visceral organs. In HUMANS heterozygous loss of growth differentiation factor 1 function has been associated with CONGENITAL HEART DEFECTS and TRANSPOSITION OF GREAT VESSELS.'),
  'D055453': NtO(MH='Coriolaceae', MS='A family of fungi, order POLYPORALES, found on decaying wood.'),
  'D055454': NtO(MH='Trametes', MS='A genus of fungi in the family Coriolaceae.'),
  'D055456': NtO(MH='Nodal Signaling Ligands', MS="Members of the transforming growth factor superfamily that play a role in pattern formation and differentiation during the pregastrulation and GASTRULATION stages of chordate development. Several nodal signaling ligands are specifically involved in the genesis of left-right asymmetry during development. The protein group is named after a critical region of the vertebrate embryo PRIMITIVE STREAK referred to as HENSEN'S NODE."),
  'D055457': NtO(MH='Nodal Protein', MS="The founding member of the nodal signaling ligand family of proteins. Nodal protein was originally discovered in the region of the mouse embryo primitive streak referred to as HENSEN'S NODE. It is expressed asymmetrically on the left side in chordates and plays a critical role in the genesis of left-right asymmetry during vertebrate development."),
  'D055458': NtO(MH='Left-Right Determination Factors', MS='Signaling ligands that act in opposition to NODAL PROTEIN. During vertebrate development they regulate the degree of left-right asymmetry by controlling the spatiotemporal influence of NODAL PROTEIN.'),
  'D055459': NtO(MH='Facilitated Diffusion', MS='The passive movement of molecules exceeding the rate expected by simple diffusion. No energy is expended in the process. It is achieved by the introduction of passively diffusing molecules to an enviroment or path that is more favorable to the movement of those molecules. Examples of facilitated diffusion are passive transport of hydrophilic substances across a lipid membrane through hydrophilic pores that traverse the membrane, and the sliding of a DNA BINDING PROTEIN along a strand of DNA.'),
  'D055471': NtO(MH='Thermal Diffusion', MS='The movement of molecules from one location to another as effected by temperature changes.'),
  'D055491': NtO(MH='Antrodia', MS='A genus of brown-rot fungi in the family Coriolaceae. The biologically active ingredients of its species have potential pharmaceutical value.'),
  'D055492': NtO(MH='Polyporus', MS='A genus of basidiomyceteous fungi in the family POLYPORACEAE found mostly on living trees or dead wood.'),
  'D055493': NtO(MH='Spiral Ligament of Cochlea', MS='A spiral thickening of the fibrous lining of the cochlear wall. Spiral ligament secures the membranous COCHLEAR DUCT to the bony spiral canal of the COCHLEA. Its spiral ligament fibrocytes function in conjunction with the STRIA VASCULARIS to mediate cochlear ion homeostasis.'),
  'D055494': NtO(MH='Pycnoporus', MS='A genus of basidiomyceteous fungi in the family POLYPORACEAE. The species all contain bright orange, bracket-like fruiting bodies.'),
  'D055495': NtO(MH='Neurogenesis', MS='Formation of NEURONS which involves the differentiation and division of STEM CELLS in which one or both of the daughter cells become neurons.'),
  'D055496': NtO(MH='Neurogenic Bowel', MS='Loss or absence of normal intestinal function due to nerve damage or birth defects. It is characterized by the inability to control the elimination of stool from the body.'),
  'D055498': NtO(MH='Jewelry', MS='Objects of precious metal usually containing gems and worn to enhance personal appearance. Health concerns include possible contamination from lead content or bacteria.'),
  'D055499': NtO(MH='Catheter-Related Infections', MS='Infections resulting from the use of catheters. Proper aseptic technique, site of catheter placement, material composition, and virulence of the organism are all factors that can influence possible infection.'),
  'D055500': NtO(MH='Resilience, Psychological', MS='The human ability to adapt in the face of tragedy, trauma, adversity, hardship, and ongoing significant life stressors.'),
  'D055501': NtO(MH='Macrophage Activation Syndrome', MS='A serious complication of childhood systemic inflammatory disorders that is thought to be caused by excessive activation and proliferation of T-LYMPHOCYTES and MACROPHAGES. It is seen predominantly in children with systemic onset JUVENILE IDIOPATHIC ARTHRITIS.'),
  'D055502': NtO(MH='Secondary Prevention', MS='The prevention of recurrences or exacerbations of a disease or complications of its therapy.'),
  'D055503': NtO(MH='Protein Multimerization', MS='The assembly of the QUATERNARY PROTEIN STRUCTURE of multimeric proteins (MULTIPROTEIN COMPLEXES) from their composite PROTEIN SUBUNITS.'),
  'D055504': NtO(MH='Insulin Receptor Substrate Proteins', MS='A structurally-related group of signaling proteins that are phosphorylated by the INSULIN RECEPTOR PROTEIN-TYROSINE KINASE. The proteins share an N-terminal PLECKSTRIN HOMOLOGY DOMAIN, a phosphotyrosine-binding domain that interacts with the phosphorylated INSULIN RECEPTOR, and a C-terminal TYROSINE-rich domain. Upon tyrosine phosphorylation, insulin receptor substrate proteins interact with specific SH2 DOMAIN containing proteins that are involved in insulin receptor signaling.'),
  'D055511': NtO(MH='CCN Intercellular Signaling Proteins', MS='A family of secreted proteins found associated with the EXTRACELLULAR MATRIX and cell surface receptors. They are believed to play a role in modulating the effects of a variety of GROWTH FACTORS and PROTEASES at the cell membrane extracellular matrix. The CCN protein family is named after three protypical members; CYSTEINE-RICH PROTEIN 61; CONNECTIVE TISSUE GROWTH FACTOR; and NEPHROBLASTOMA OVEREXPRESSED PROTEIN.'),
  'D055512': NtO(MH='Tertiary Prevention', MS='Measures aimed at providing appropriate supportive and rehabilitative services to minimize morbidity and maximize quality of life after a long-term disease or injury is present.'),
  'D055513': NtO(MH='Connective Tissue Growth Factor', MS='A CCN protein family member that regulates a variety of extracellular functions including CELL ADHESION; CELL MIGRATION; and EXTRACELLULAR MATRIX synthesis. It is found in hypertrophic CHONDROCYTES where it may play a role in CHONDROGENESIS and endochondral ossification.'),
  'D055514': NtO(MH='Nephroblastoma Overexpressed Protein', MS='A CCN protein family member found at high levels in NEPHROBLASTOMA cells. It is found both intracellularly and in the EXTRACELLULAR MATRIX and may play a role in the regulation of CELL PROLIFERATION and EXTRACELLULAR MATRIX synthesis.'),
  'D055515': NtO(MH='Cysteine-Rich Protein 61', MS='A CCN protein family member that regulates a variety of extracellular functions including CELL ADHESION; CELL MIGRATION; and EXTRACELLULAR MATRIX synthesis. It may play an important role in the development of branched CAPILLARIES during EMBRYOGENESIS.'),
  'D055516': NtO(MH='2S Albumins, Plant', MS='A major class of water-soluble seed storage proteins. Many proteins from this class are major PLANT ALLERGENS.'),
  'D055517': NtO(MH='snRNP Core Proteins', MS='The protein components that constitute the common core of small nuclear ribonucleoprotein particles. These proteins are commonly referred as Sm nuclear antigens due to their antigenic nature.'),
  'D055519': NtO(MH='Photoreceptors, Plant', MS='Plant proteins that mediate LIGHT SIGNAL TRANSDUCTION. They are involved in PHOTOTROPISM and other light adaption responses during plant growth and development . They include the phototropins, phytochromes (PHYTOCHROME), and members of the ubiquitous cryptochrome family.'),
  'D055531': NtO(MH='Gemini of Coiled Bodies', MS='Small, punctate nuclear structures found in close proximity to COILED BODIES. They are enriched with SMN COMPLEX PROTEINS and may play a role in the processing of SMALL NUCLEAR RIBONUCLEOPROTEINS.'),
  'D055532': NtO(MH='SMN Complex Proteins', MS='A complex of proteins that assemble the SNRNP CORE PROTEINS into a core structure that surrounds a highly conserved RNA sequence found in SMALL NUCLEAR RNA. They are found localized in the GEMINI OF COILED BODIES and in the CYTOPLASM. The SMN complex is named after the Survival of Motor Neuron Complex Protein 1, which is a critical component of the complex.'),
  'D055533': NtO(MH='Survival of Motor Neuron 1 Protein', MS='An SMN complex protein that contains a TUDOR DOMAIN and is essential for the function of the SMN protein complex. In humans, the protein is encoded by a single gene found near the inversion TELOMERE of a large inverted region of CHROMOSOME 5. Mutations in the gene coding for survival of motor neuron 1 protein may result in SPINAL MUSCULAR ATROPHIES OF CHILDHOOD.'),
  'D055534': NtO(MH='Bulbo-Spinal Atrophy, X-Linked', MS='An X-linked recessive form of spinal muscular atrophy. It is due to a mutation of the gene encoding the ANDROGEN RECEPTOR.'),
  'D055535': NtO(MH='Morgellons Disease', MS='An unexplained illness which is characterized by skin manifestations including non-healing lesions, itching, and the appearance of fibers.'),
  'D055536': NtO(MH='Vagus Nerve Stimulation', MS='An adjunctive treatment for PARTIAL EPILEPSY and refractory DEPRESSION that delivers electrical impulses to the brain via the VAGUS NERVE. A battery implanted under the skin supplies the energy.'),
  'D055537': NtO(MH='Light Signal Transduction', MS='The conversion of absorbed light energy into molecular signals.'),
  'D055538': NtO(MH='delta-Thalassemia', MS='A hereditary disorder characterized by reduced or absent DELTA-GLOBIN thus effecting the level of HEMOGLOBIN A2, a minor component of adult hemoglobin monitored in the diagnosis of BETA-THALASSEMIA.'),
  'D055539': NtO(MH='MP3-Player', MS='Portable electronics device for storing and playing audio and or media files. MP3 for MPEG-1 audio layer 3, is a digital coding format.'),
  'D055540': NtO(MH='Survival of Motor Neuron 2 Protein', MS='An SMN complex protein that is closely-related to SURVIVAL OF MOTOR NEURON 1 PROTEIN. In humans, the protein is encoded by an often duplicated gene found near the inversion CENTEROMERE of a large inverted region of CHROMOSOME 5.'),
  'D055541': NtO(MH='DEAD Box Protein 20', MS='A multifunctional protein that is both a DEAD-box RNA helicase and a component of the SMN protein complex.'),
  'D055542': NtO(MH='alpha-Globins', MS='Members of the alpha-globin family. In humans, they are encoded in a gene cluster on CHROMOSOME 16. They include zeta-globin and alpha-globin. There are also pseudogenes of zeta (theta-zeta) and alpha (theta-alpha) in the cluster. Adult HEMOGLOBIN is comprised of 2 alpha-globin chains and 2 beta-globin chains.'),
  'D055543': NtO(MH='zeta-Globins', MS='A member of the alpha-globin family. In humans, zeta-globin is encoded in the alpha-globin gene cluster located on CHROMOSOME 16. Two zeta-globin chains combine with two EPSILON GLOBIN chains to form the embryonic HEMOGLOBIN Gower 1.'),
  'D055544': NtO(MH='beta-Globins', MS='Members of the beta-globin family. In humans, they are encoded in a gene cluster on CHROMOSOME 11. They include epsilon-globin, gamma-globin, delta-globin and beta-globin. There is also a pseudogene of beta (theta-beta) in the gene cluster. Adult HEMOGLOBIN is comprised of two ALPHA-GLOBIN chains and two beta-globin chains.'),
  'D055545': NtO(MH='gamma-Globins', MS='Members of the beta-globin family. In humans, two non-allelic types of gamma-globin - A gamma and G gamma are encoded in the beta-globin gene cluster on CHROMOSOME 11. Two gamma-globin chains combine with two ZETA-GLOBIN chains to form the embryonic hemoglobin Portland. Fetal HEMOGLOBIN F is formed from two gamma-globin chains combined with two ALPHA-GLOBIN chains.'),
  'D055546': NtO(MH='delta-Globins', MS='A member of the beta-globin family. In humans, delta-globin is encoded in the beta-globin gene cluster located on CHROMOSOME 11. Two delta-globin chains along with two alpha-globin chains form HEMOGLOBIN A2 which makes up about 3% of the HEMOGLOBIN in adults.'),
  'D055547': NtO(MH='epsilon-Globins', MS='A member of the beta-globin family. In humans, epsilon-globin is encoded in the beta-globin gene cluster located on CHROMOSOME 11. Two epsilon-globin chains combine with two ZETA-GLOBIN chains to form the embryonic hemoglobin Gower I or two epsilon-globin chains combine with two ALPHA-2 GLOBIN chains to form the embryonic hemoglobin Gower II.'),
  'D055548': NtO(MH='Vapor Pressure', MS='The contribution to barometric PRESSURE of gaseous substance in equilibrium with its solid or liquid phase.'),
  'D055549': NtO(MH='Volatile Organic Compounds', MS='Organic compounds that have a relatively high VAPOR PRESSURE at room temperature.'),
  'D055550': NtO(MH='Protein Stability', MS='The ability of a protein to retain its structural conformation or its activity when subjected to physical or chemical manipulations.'),
  'D055551': NtO(MH='HSP27 Heat-Shock Proteins', MS='A subfamily of small heat-shock proteins that function as molecular chaperones that aid in refolding of non-native proteins. They play a protective role that increases cellular survival during times of stress.'),
  'D055552': NtO(MH='Infectious Disease Medicine', MS='A branch of internal medicine concerned with the diagnosis and treatment of INFECTIOUS DISEASES.'),
  'D055553': NtO(MH='Prescription Drugs', MS='Drugs that cannot be sold legally without a prescription.'),
  'D055554': NtO(MH='Hemoglobin Subunits', MS='The PROTEIN SUBUNITS that comprise multimeric HEMOGLOBINS.'),
  'D055556': NtO(MH='Nasal Lavage', MS='Irrigation of the nose with saline or irrigation solutions for diagnostic or therapeutic purposes. It is used to remove irritants, allergens, or microorganisms from the nose.'),
  'D055571': NtO(MH='Secretory Pathway', MS='A series of sequential intracellular steps involved in the transport of proteins (such as hormones and enzymes) from the site of synthesis to outside the cell. The pathway involves membrane-bound compartments through which the newly synthesized proteins undergo POST-TRANSLATIONAL MODIFICATIONS, packaging, storage, or transportation to the PLASMA MEMBRANE for secretion.'),
  'D055572': NtO(MH='Ceramidases', MS='Amidohydrolases that are specific for the cleavage of the N-acyl linkage of CERAMIDES. Ceramidases are classified as acidic, neutral or basic according to the optimal pH with which they function.'),
  'D055573': NtO(MH='Acid Ceramidase', MS='A ceramidase subtype that is active at acid pH. It plays an important role in sphingolipid degradation by catalyzing the lysosomal hydrolysis of ceramide to sphingosine and free fatty acid. Inherited deficiency of acid ceramidase activity results in FARBER LIPOGRANULOMATOSIS.'),
  'D055574': NtO(MH='Alkaline Ceramidase', MS='A ceramidase subtype that is active at alkaline pH. It is found at high levels within the SMALL INTESTINE.'),
  'D055576': NtO(MH='Neutral Ceramidase', MS='A ceramidase subtype that is active at neutral pH. It is found at high levels within the SMALL INTESTINE and in the BRAIN.'),
  'D055577': NtO(MH='Farber Lipogranulomatosis', MS='A sphingolipidosis subtype that is characterized by the histological appearance of granulomatous deposits in tissues. It results from the accumulation of CERAMIDES in various tissues due to an inherited deficiency of ACID CERAMIDASE.'),
  'D055580': NtO(MH='Astronomical Phenomena', MS='Aggregates of matter in outer space, such as stars, planets, and comets; their properties; and the processes they undergo.'),
  'D055585': NtO(MH='Physical Phenomena', MS='The entities of matter and energy, and the processes, principles, properties, and relationships describing their nature and interactions.'),
  'D055587': NtO(MH='Astronomical Objects', MS='Aggregates of matter in outer space.'),
  'D055589': NtO(MH='Pancreatitis, Graft', MS='An inflammatory disease and serious complication of PANCREAS TRANSPLANTATION. It is caused by a premature activation of pancreatic proenzymes leading to autodigestion of the gland and can be attributed to immunological or nonimmunological causes.'),
  'D055590': NtO(MH='Electromagnetic Phenomena', MS='Characteristics of ELECTRICITY and magnetism such as charged particles and the properties and behavior of charged particles, and other phenomena related to or associated with electromagnetism.'),
  'D055592': NtO(MH='Biophysical Phenomena', MS='The physical characteristics and processes of biological systems.'),
  'D055593': NtO(MH='Geological Phenomena', MS='Structures, properties, and processes involving the inanimate matter of Earth.'),
  'D055595': NtO(MH='Mechanical Phenomena', MS='The properties and processes of materials that affect their behavior under force.'),
  'D055598': NtO(MH='Chemical Phenomena', MS='The composition, structure, conformation, and properties of atoms and molecules, and their reaction and interaction processes.'),
  'D055601': NtO(MH='Organic Chemistry Phenomena', MS='The conformation, properties, reaction processes, and the properties of the reactions of organic carbon compounds.'),
  'D055607': NtO(MH='Receptors, Natural Killer Cell', MS='Receptors that are specifically found on the surface of NATURAL KILLER CELLS. They play an important role in regulating the cellular component of INNATE IMMUNITY.'),
  'D055608': NtO(MH='Galaxies', MS='Large aggregates of CELESTIAL STARS; COSMIC DUST; and gas. (From McGraw Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D055609': NtO(MH='Stars, Celestial', MS='Large bodies consisting of self-luminous gas held together by their own gravity. (From McGraw Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D055610': NtO(MH='Antigens, CD1d', MS='A major histocompatibility complex class I-like protein that contains an IMMUNOGLOBULIN C1-SET DOMAIN. It plays a unique role in the presentation of lipid ANTIGENS to NATURAL KILLER T-CELLS.'),
  'D055611': NtO(MH='Natural Killer T-Cells', MS='A specialized subset of T-LYMPHOCYTES that exhibit features of INNATE IMMUNITY similar to that of NATURAL KILLER CELLS. They are reactive to glycolipids presented in the context of the major histocompatibility complex (MHC) class I-like molecule, CD1D ANTIGEN.'),
  'D055612': NtO(MH='Cytokine-Induced Killer Cells', MS='Mononuclear leukocytes that have been expanded in CELL CULTURE and activated with CYTOKINES such as INTERLEUKIN-2 to produce large numbers of highly cytotoxic cells.'),
  'D055613': NtO(MH='Multiple Pulmonary Nodules', MS='A number of small lung lesions characterized by small round masses of 2- to 3-mm in diameter. They are usually detected by chest CT scans (COMPUTED TOMOGRAPHY, X-RAY). Such nodules can be associated with metastases of malignancies inside or outside the lung, benign granulomas, or other lesions.'),
  'D055614': NtO(MH='Genetic Phenomena', MS='The biological processes, properties, and objects that are involved in maintaining, expressing, and transmitting from one organism to another, genetically encoded traits.'),
  'D055615': NtO(MH='Electrical Equipment and Supplies', MS='Apparatus and instruments that generate and operate with ELECTRICITY, and their electrical components.'),
  'D055616': NtO(MH='Microtechnology', MS='Manufacturing technology for making microscopic devices in the micrometer range (typically 1-100 micrometers), such as integrated circuits or MEMS. The process usually involves replication and parallel fabrication of hundreds or millions of identical structures using various thin film deposition techniques and carried out in environmentally-controlled clean rooms.'),
  'D055617': NtO(MH='Micro-Electrical-Mechanical Systems', MS='A class of devices combining electrical and mechanical components that have at least one of the dimensions in the micrometer range (between 1 micron and 1 millimeter). They include sensors, actuators, microducts, and micropumps.'),
  'D055618': NtO(MH='Radiation Equipment and Supplies', MS='Instruments and apparatus for radiation applications and their components and associated expendables.'),
  'D055620': NtO(MH='Optical Phenomena', MS='LIGHT, its processes and properties, and the characteristics of materials interacting with it.'),
  'D055622': NtO(MH='Orbital Myositis', MS='Inflammation of the extraocular muscle of the eye. It is characterized by swelling which can lead to ischemia, fibrosis, or ORBITAL PSEUDOTUMOR.'),
  'D055623': NtO(MH='Keratosis, Actinic', MS='White or pink lesions on the arms, hands, face, or scalp that arise from sun-induced DNA DAMAGE to KERATINOCYTES in exposed areas. They are considered precursor lesions to superficial SQUAMOUS CELL CARCINOMA.'),
  'D055624': NtO(MH='Methicillin-Resistant Staphylococcus aureus', MS='A strain of Staphylococcus aureus that is non-susceptible to the action of METHICILLIN. The mechanism of resistance usually involves modification of normal or the presence of acquired PENICILLIN BINDING PROTEINS.'),
  'D055627': NtO(MH='Natural Cytotoxicity Triggering Receptor 1', MS='A 46-kD stimulatory receptor found on resting and activated NATURAL KILLER CELLS. It has specificity for VIRAL HEMAGGLUTININS that are expressed on infected cells.'),
  'D055628': NtO(MH='Natural Cytotoxicity Triggering Receptor 3', MS='A 30 kDa stimulatory receptor found on resting and activated NATURAL KILLER CELLS.'),
  'D055629': NtO(MH='Natural Cytotoxicity Triggering Receptor 2', MS='A 44-kD stimulatory receptor found on activated NATURAL KILLER CELLS. It has specificity for VIRAL HEMAGGLUTININS that are expressed on infected cells.'),
  'D055630': NtO(MH='Receptors, NK Cell Lectin-Like', MS='Structurally-related receptors that are typically found on NATURAL KILLER CELLS. They are considered lectin-like proteins in that they share sequence homology with the carbohydrate binding domains of C-TYPE LECTINS. They differ from classical C-type lectins, however, in that they appear to lack CALCIUM-binding domains.'),
  'D055633': NtO(MH='Immune System Phenomena', MS="The characteristic properties and processes involved in IMMUNITY and an organism's immune response."),
  'D055636': NtO(MH='Receptors, Natural Cytotoxicity Triggering', MS='A subclass of natural killer cell receptors that perform an important role in the recognition of tumor cells by NK CELLS.'),
  'D055641': NtO(MH='Mathematical Concepts', MS='Numeric or quantitative entities, descriptions, properties, relationships, operations, and events.'),
  'D055646': NtO(MH='Killer Factors, Yeast', MS='Protein factors released from one species of YEAST that are selectively toxic to another species of yeast.'),
  'D055647': NtO(MH='Rats, Transgenic', MS='Laboratory rats that have been produced from a genetically manipulated rat EGG or rat EMBRYO, MAMMALIAN. They contain genes from another species.'),
  'D055650': NtO(MH='NK Cell Lectin-Like Receptor Subfamily A', MS='An inhibitory subclass of NK cell lectin-like receptors that interacts with CLASS I MAJOR HISTOCOMPATIBILITY ANTIGENS and prevents the activation of NK CELLS.'),
  'D055651': NtO(MH='NK Cell Lectin-Like Receptor Subfamily B', MS='A subclass of NK cell lectin-like receptors that includes both inhibitory and stimulatory members.'),
  'D055652': NtO(MH='Gray Platelet Syndrome', MS='A rare, inherited platelet disorder characterized by a selective deficiency in the number and contents of platelet alpha-granules. It is associated with THROMBOCYTOPENIA, enlarged platelets, and prolonged bleeding time.'),
  'D055653': NtO(MH='Muir-Torre Syndrome', MS='A form of LYNCH SYNDROME II associated with cutaneous SEBACEOUS GLAND NEOPLASMS. Muir-Torre syndrome is also associated with other visceral malignant diseases include colorectal, endometrial, urological, and upper gastrointestinal neoplasms.'),
  'D055654': NtO(MH='NK Cell Lectin-Like Receptor Subfamily C', MS='A subclass of NK cell lectin-like receptors that associates with members of NK CELL LECTIN-LIKE RECEPTOR SUBFAMILY D to form heterodimeric receptors for HLA-E antigen.'),
  'D055655': NtO(MH='NK Cell Lectin-Like Receptor Subfamily K', MS='An activating NK cell lectin-like receptor subfamily that regulates immune responses to INFECTION and NEOPLASMS. Members of this subfamily generally occur as homodimers.'),
  'D055656': NtO(MH='Prescriptions', MS='Directions written for the obtaining and use of PHARMACEUTICAL PREPARATIONS; MEDICAL DEVICES; corrective LENSES; and a variety of other medical remedies.'),
  'D055664': NtO(MH='Electrochemical Techniques', MS='The utilization of an electrical current to measure, analyze, or alter chemicals or chemical reactions in solution, cells, or tissues.'),
  'D055665': NtO(MH='Purpura Fulminans', MS='A severe, rapidly fatal reaction occurring most commonly in children following an infectious illness. It is characterized by large, rapidly spreading skin hemorrhages, fever, or shock. Purpura fulminans often accompanies or is triggered by DISSEMINATED INTRAVASCULAR COAGULATION.'),
  'D055666': NtO(MH='Lipopeptides', MS='Compounds consisting of a short peptide chain conjugated with an acyl chain.'),
  'D055667': NtO(MH='Arcanobacterium', MS='A genus of facultatively anaerobic, gram-positive bacteria in the family ACTINOMYCETACEAE, order ACTINOMYCETALES. They are obligate parasites of the PHARYNX in humans and farm animals.'),
  'D055668': NtO(MH='Photochemical Processes', MS='Chemical reactions effected by light.'),
  'D055669': NtO(MH='Ecological and Environmental Phenomena', MS='Ecological and environmental entities, characteristics, properties, relationships and processes.'),
  'D055670': NtO(MH='Medicine, Korean Traditional', MS='Medical practice or discipline that is based on the knowledge, cultures, and beliefs of the people of KOREA.'),
  'D055672': NtO(MH='Static Electricity', MS='The accumulation of an electric charge on a object'),
  'D055673': NtO(MH='Acrocallosal Syndrome', MS='Autosomal recessive syndrome characterized by hypogenesis or agenesis of CORPUS CALLOSUM. Clinical features include MENTAL RETARDATION; CRANIOFACIAL ABNORMALITIES; digital malformations, and growth retardation.'),
  'D055675': NtO(MH='Viscosupplements', MS='Viscoelastic solutions that are injected into JOINTS in order to alleviate symptoms of joint-related disorders such as OSTEOARTHRITIS.'),
  'D055676': NtO(MH='Viscosupplementation', MS='A therapeutic treatment typically involving INTRA-ARTICULAR INJECTIONS of HYALURONIC ACID and related compounds. The procedure is commonly used in the treatment of OSTEOARTHRITIS with the therapeutic goal to restore the viscoelasticity of SYNOVIAL FLUID, decrease pain, improve mobility and restore the natural protective functions of hyaluronan in the joint.'),
  'D055677': NtO(MH='Refeeding Syndrome', MS='A condition of metabolic imbalance that is caused by complications of initially feeding a severely malnourished patient too aggressively. Usually occurring within the first 5 days of refeeding, this syndrome is characterized by WATER-ELECTROLYTE IMBALANCE; GLUCOSE INTOLERANCE; CARDIAC ARRHYTHMIAS; and DIARRHEA.'),
  'D055682': NtO(MH='Viscoelastic Substances', MS='Substances that display the physical properties of ELASTICITY and VISCOSITY. The dual-nature of these substances causes them to resist applied forces in a time-dependent manner.'),
  'D055685': NtO(MH='Pyricularia grisea', MS='The anamorphic form of the fungus MAGNAPORTHE GRISEA and the most important pathogen of rice worldwide. It causes rice blast which prevents maturation of the rice grains.'),
  'D055686': NtO(MH='Metabolic Equivalent', MS='A measurement of OXYGEN uptake in a sitting, resting person (resting oxygen consumption), varying with age, sex, race, and other factors. In normal adult men, one MET is approximately 3.5 ml O2/kg/min of body weight. Oxygen uptake during activities or work can be measured in METs which can be use to determine health status and exercise prescription.'),
  'D055687': NtO(MH='Musculoskeletal and Neural Physiological Phenomena', MS='Properties, and processes of the MUSCULOSKELETAL SYSTEM and the NERVOUS SYSTEM or their parts.'),
  'D055688': NtO(MH='Digestive System and Oral Physiological Phenomena', MS='Properties and processes of the DIGESTIVE SYSTEM and DENTITION as a whole or of any of its parts.'),
  'D055695': NtO(MH='Electronic Prescribing', MS='The use of COMPUTER COMMUNICATION NETWORKS to store and transmit medical PRESCRIPTIONS.'),
  'D055696': NtO(MH='Olfactory Perception', MS='The process by which the nature and meaning of olfactory stimuli, such as odors, are recognized and interpreted by the brain.'),
  'D055697': NtO(MH='Taste Perception', MS='The process by which the nature and meaning of gustatory stimuli are recognized and interpreted by the brain. The four basic classes of taste perception are salty, sweet, bitter, and sour.'),
  'D055698': NtO(MH='Touch Perception', MS='The process by which the nature and meaning of tactile stimuli are recognized and interpreted by the brain, such as realizing the characteristics or name of an object being touched.'),
  'D055699': NtO(MH='Cemeteries', MS='Areas set apart as burial grounds.'),
  'D055700': NtO(MH='Cremation', MS='Reducing a dead body to ashes by burning or incineration.'),
  'D055703': NtO(MH='Reproductive Physiological Phenomena', MS='Physiological processes, factors, properties and characteristics pertaining to REPRODUCTION.'),
  'D055724': NtO(MH='Electrophysiological Phenomena', MS='The electrical properties, characteristics of living organisms, and the processes of organisms or their parts that are involved in generating and responding to electricity.'),
  'D055726': NtO(MH='Cabo Verde', MS='A country in western Africa composed of a group of islands in the North Atlantic Ocean, west of Senegal. The capital is Praia.'),
  'D055728': NtO(MH='Primary Myelofibrosis', MS='A de novo myeloproliferation arising from an abnormal stem cell. It is characterized by the replacement of bone marrow by fibrous tissue, a process that is mediated by CYTOKINES arising from the abnormal clone.'),
  'D055731': NtO(MH='Carbohydrate Biochemistry', MS='The study of the structure, biosynthesis, and function of CARBOHYDRATES and GLYCOSYLATION.'),
  'D055732': NtO(MH='Pulmonary Aspergillosis', MS='Infections of the respiratory tract with fungi of the genus ASPERGILLUS.'),
  'D055744': NtO(MH='Invasive Pulmonary Aspergillosis', MS='Lung infections with the invasive forms of ASPERGILLUS, usually after surgery, transplantation, prolonged NEUTROPENIA or treatment with high-doses of CORTICOSTEROIDS. Invasive pulmonary aspergillosis can progress to CHRONIC NECROTIZING PULMONARY ASPERGILLOSIS or hematogenous spread to other organs.'),
  'D055745': NtO(MH='Bronchioles', MS='The small airways branching off the TERTIARY BRONCHI. Terminal bronchioles lead into several orders of respiratory bronchioles which in turn lead into alveolar ducts and then into PULMONARY ALVEOLI.'),
  'D055746': NtO(MH='Stage-Specific Embryonic Antigens', MS='Cell-surface molecules that exhibit lineage-restricted patterns of expression during EMBRYONIC DEVELOPMENT. The antigens are useful markers in the identification of EMBRYONIC STEM CELLS.'),
  'D055747': NtO(MH='SOX Transcription Factors', MS='A large family of structurally-related transcription factors that were originally discovered based upon their close sequence homology to an HMG-box domain found in SEX-DETERMINING REGION Y PROTEIN. Many SOX transcription factors play important roles in regulating CELL DIFFERENTIATION. The numerous members of this family are organized in several subgroups according to structural identities found within the proteins.'),
  'D055748': NtO(MH='SOXB1 Transcription Factors', MS='A subclass of SOX transcription factors that are expressed in neuronal tissue where they may play a role in the regulation of CELL DIFFERENTIATION. Members of this subclass are generally considered to be transcriptional activators.'),
  'D055749': NtO(MH='SOXB2 Transcription Factors', MS='A subclass of SOX transcription factors that are expressed in neuronal tissue where they may play a role in the regulation of CELL DIFFERENTIATION. Members of this subclass are generally considered to be transcriptional repressors.'),
  'D055750': NtO(MH='Phytophthora infestans', MS='A species of parasitic OOMYCETES in the family Peronosporaceae that is the causative agent of late blight of potato.'),
  'D055751': NtO(MH='SOXC Transcription Factors', MS='A subclass of closely-related SOX transcription factors. Members of the group have been found expressed in developing neuronal tissue, LYMPHOCYTES, and during EMBRYONIC DEVELOPMENT.'),
  'D055752': NtO(MH='Small Cell Lung Carcinoma', MS='A form of highly malignant lung cancer that is composed of small ovoid cells (SMALL CELL CARCINOMA).'),
  'D055755': NtO(MH='SOXD Transcription Factors', MS='A subclass of closely-related SOX transcription factors. In addition to a conserved HMG-BOX DOMAIN, members of this group contain a leucine zipper motif which mediates protein DIMERIZATION.'),
  'D055756': NtO(MH='Meningeal Carcinomatosis', MS='Primary or secondary neoplasm in the ARACHNOID or SUBARACHNOID SPACE. It appears as a diffuse fibrotic thickening of the MENINGES associated with variable degrees of inflammation.'),
  'D055757': NtO(MH='SOXE Transcription Factors', MS='A subclass of closely-related SOX transcription factors. Members of this subfamily have been implicated in regulating the differentiation of OLIGODENDROCYTES during neural crest formation and in CHONDROGENESIS.'),
  'D055758': NtO(MH='SOX9 Transcription Factor', MS='A SOXE transcription factor that plays a critical role in regulating CHONDROGENESIS; OSTEOGENESIS; and male sex determination. Loss of function of the SOX9 transcription factor due to genetic mutations is a cause of CAMPOMELIC DYSPLASIA.'),
  'D055760': NtO(MH='SOXF Transcription Factors', MS='A subclass of closely-related SOX transcription factors. Members of this subclass are expressed in VASCULAR ENDOTHELIAL CELLS and may play a role in vasculogenesis.'),
  'D055761': NtO(MH='Reticuloendotheliosis, Avian', MS='A group of pathologic syndromes found in avian species caused by RETICULOENDOTHELIOSIS VIRUS. The distinct syndromes include non-neoplastic runting, acute neoplastic disease, and chronic neoplastic disease. Humans and mammals appear resistant.'),
  'D055762': NtO(MH='Cranial Fontanelles', MS='Any of six membrane-covered openings between the CRANIAL SUTURES in the incompletely ossified skull of the fetus or newborn infant. The fontanelles normally close sometime after birth.'),
  'D055764': NtO(MH='Myxozoa', MS='Single-celled, aquatic endoparasitic worms that are currently considered belonging to the phylum CNIDARIA. They have a complex life cycle and parasitize a wide range of hosts including FISHES; ANNELIDA; and BRYOZOA.'),
  'D055765': NtO(MH='Shc Signaling Adaptor Proteins', MS='A family of signaling adaptor proteins that contain SRC HOMOLOGY DOMAINS. Many members of this family are involved in transmitting signals from CELL SURFACE RECEPTORS to MITOGEN-ACTIVATED PROTEIN KINASES.'),
  'D055766': NtO(MH='Myxobolus', MS='The largest genus in the family Myxobolidae, class MYXOSPOREA, containing over 400 species.'),
  'D055767': NtO(MH='Immobilized Proteins', MS='Proteins that are chemically bound to a substrate material which renders their location fixed. The immobilization of proteins allows their use in chemical reactions without being diluted by solvent.'),
  'D055768': NtO(MH='Halogenated Diphenyl Ethers', MS='Compounds that contain two halogenated benzene rings linked via an OXYGEN atom. Many polybrominated diphenyl ethers are used as FLAME RETARDANTS.'),
  'D055770': NtO(MH='Interpleural Analgesia', MS='Injection of ANALGESICS; LOCAL ANESTHETICS; or NARCOTICS into the PLEURAL CAVITY between the two pleural membranes.'),
  'D055771': NtO(MH='Serbia', MS='A republic located south of HUNGARY, west of ROMANIA and BULGARIA, and part of the former YUGOSLAVIA. The capital is Belgrade.'),
  'D055772': NtO(MH='Green Chemistry Technology', MS='Pollution prevention through the design of effective chemical products that have low or no toxicity and use of chemical processes that reduce or eliminate the use and generation of hazardous substances.'),
  'D055785': NtO(MH='Gene Knockdown Techniques', MS='The artificial induction of GENE SILENCING by the use of RNA INTERFERENCE to reduce the expression of a specific gene. It includes the use of DOUBLE-STRANDED RNA, such as SMALL INTERFERING RNA and RNA containing HAIRPIN LOOP SEQUENCE, and ANTI-SENSE OLIGONUCLEOTIDES.'),
  'D055786': NtO(MH='Gene Knockout Techniques', MS='Techniques to alter a gene sequence that result in an inactivated gene, or one in which the expression can be inactivated at a chosen time during development to study the loss of function of a gene.'),
  'D055787': NtO(MH='Prussia', MS='Former state in north central Germany. Formally abolished March 1, 1947. Kingdom established 1701.'),
  'D055805': NtO(MH='Moving and Lifting Patients', MS='Moving or repositioning patients within their beds, from bed to bed, bed to chair, or otherwise from one posture or surface to another.'),
  'D055806': NtO(MH='Microvessels', MS='The finer blood vessels of the vasculature that are generally less than 100 microns in internal diameter.'),
  'D055807': NtO(MH='Stalking', MS='Course of conduct directed at a specific person that involves repeated (two or more occasions) visual or physical proximity, nonconsensual communication, or verbal, written, or implied threats, or a combination thereof, that would cause a reasonable person fear.'),
  'D055808': NtO(MH='Drug Discovery', MS='The process of finding chemicals for potential therapeutic use.'),
  'D055809': NtO(MH='Illness Behavior', MS='Coordinate set of non-specific behavioral responses to non-psychiatric illness. These may include loss of APPETITE or LIBIDO; disinterest in ACTIVITIES OF DAILY LIVING; or withdrawal from social interaction.'),
  'D055810': NtO(MH='Lot Quality Assurance Sampling', MS='A sampling methodology using small sample sizes when conducting surveys in small geographical or population-based areas or lots.'),
  'D055811': NtO(MH='Journal Impact Factor', MS='A quantitative measure of the frequency on average with which articles in a journal have been cited in a given period of time.'),
  'D055812': NtO(MH='Community-Based Participatory Research', MS='Collaborative process of research involving researchers and community representatives.'),
  'D055813': NtO(MH='Wilderness Medicine', MS='Skills and knowledge required for assessment and treatment of traumatic, environmental, and medical emergencies in remote geographic or wilderness environments.'),
  'D055814': NtO(MH='Wilderness', MS='Environment un-modified by human activity. Areas in which natural processes operate without human interference.'),
  'D055815': NtO(MH='Young Adult', MS='A person between 19 and 24 years of age.'),
  'D055816': NtO(MH='Wandering Behavior', MS='Moving oneself through space while confused or otherwise cognitively impaired. Patterns include akathisia, exhibiting neuroleptic-induced pacing and restlessness; exit seekers who are often newly admitted institution residents who try to open locked exit doors; self-stimulators who perform other activities such as turning doorknobs, in addition to continuous pacing; and modelers who shadow other pacers.'),
  'D055817': NtO(MH='Telenursing', MS='Delivery of nursing services via remote telecommunications.'),
  'D055818': NtO(MH='Public-Private Sector Partnerships', MS='An organizational enterprise between a public sector agency, federal, state or local, and a private sector entity. Skills and assets of each sector are shared to deliver a service or facility for the benefit or use of the general public.'),
  'D055819': NtO(MH='Hospitals, Isolation', MS='Hospitals designed or used to isolate or quarantine persons with communicable diseases.'),
  'D055820': NtO(MH='New Orleans', MS='City in Orleans Parish (county), largest city in state of LOUISIANA. It is located between the Mississippi River and Lake Pontchartrain.'),
  'D055821': NtO(MH='Poetry', MS='Works that consist of literary and oral genre expressing meaning via symbolism and following formal or informal patterns.'),
  'D055823': NtO(MH='Cookbook', MS='Collection of recipes or instructions for preparation of food and organization of meals.'),
  'D055824': NtO(MH='Formulary', MS='Work that consists of lists of drugs or collections of recipes, formulas, and prescriptions for the compounding of medicinal preparations.'),
  'D055825': NtO(MH='National Institute of Neurological Disorders and Stroke (U.S.)', MS='Component of the NATIONAL INSTITUTES OF HEALTH. It supports and conducts research, both basic and clinical, on the normal and diseases nervous system. It was established in 1950.'),
  'D055826': NtO(MH='Afghan Campaign 2001-', MS='Multinational coalition military operation initiated in October 2001 to counter terrorism and bring security to AFGHANISTAN in collaboration with Afghan forces.'),
  'D055827': NtO(MH='Integumentary System Physiological Phenomena', MS='The properties and relationships and biological processes that characterize the nature and function of the SKIN and its appendages.'),
  'D055847': NtO(MH='Lynch Syndrome II', MS='Hereditary nonpolyposis colorectal neoplasms associated with other malignancies, more commonly of ovarian or uterine origin. When also associated with SEBACEOUS GLAND NEOPLASMS, it is called MUIR-TORRE SYNDROME.'),
  'D055848': NtO(MH='Nuclear Power Plants', MS='Facilities that convert NUCLEAR ENERGY into electrical energy.'),
  'D055864': NtO(MH='Droughts', MS='Prolonged dry periods in natural climate cycle. They are slow-onset phenomena caused by rainfall deficit combined with other predisposing factors.'),
  'D055865': NtO(MH='Sleep Phase Chronotherapy', MS='A progressive advance or delay of bedtime until the desired bedtime is achieved.'),
  'D055866': NtO(MH='Earthquakes', MS='Sudden slips on a geological fault, and the resulting ground shaking and radiated seismic energy caused by the slips, or by volcanic or magmatic activity, or other sudden stress changes in the earth. Faults are fractures along which the blocks of EARTH crust on either side have moved relative to one another parallel to the fracture.'),
  'D055867': NtO(MH='Cyclonic Storms', MS='Non-frontal low-pressure systems over tropical or sub-tropical waters with organized convection and definite pattern of surface wind circulation.'),
  'D055868': NtO(MH='Floods', MS='Sudden onset water phenomena with different speed of occurrence. These include flash floods, seasonal river floods, and coastal floods, associated with CYCLONIC STORMS; TIDALWAVES; and storm surges.'),
  'D055869': NtO(MH='Tornadoes', MS='Rotating column of air extending from a thunderstorm to the ground. Tornadoes generally produce damage paths of 100 yards wide or less, with path lengths of a couple miles.'),
  'D055870': NtO(MH='Drug Chronotherapy', MS='The adaptation of drug administration to the known variations in biological RHYTHMICITY, such as CIRCADIAN RHYTHMS. The treatment is aimed at supporting normal rhythms, or modifying the timing of therapy to achieve maximal efficacy and minimal adverse effect.'),
  'D055871': NtO(MH='Tidal Waves', MS='Water waves caused by the gravitational interactions between the EARTH; MOON; and SUN.'),
  'D055872': NtO(MH='Surge Capacity', MS="A health care system's ability to rapidly mobilize to meet an increased demand, to rapidly expand beyond normal services levels to meet the increased demand in the event of large-scale DISASTERS or public health emergencies."),
  'D055873': NtO(MH='Extreme Cold Weather', MS='Below normal weather temperatures that may lead to serious health problems. Extreme cold is a dangerous situation that can bring on health emergencies in susceptible people.'),
  'D055874': NtO(MH='Extreme Heat', MS='High temperature weather exceeding the average and of several weeks duration. Extreme heat is a dangerous situation that can bring on health emergencies in susceptible people.'),
  'D055875': NtO(MH='Avalanches', MS='Mass of snow and/or ice falling down a mountain or incline.'),
  'D055876': NtO(MH='Landslides', MS='Downslope movements of soil and and/or rock resulting from natural phenomena or man made actions. These can be secondary effects of severe storms, VOLCANIC ERUPTIONS and EARTHQUAKES.'),
  'D055877': NtO(MH='Biosurveillance', MS='Monitoring of information sources of potential value in detecting an emerging epidemic, whether naturally occurring or the result of bioterrorism.'),
  'D055879': NtO(MH='Gene Knock-In Techniques', MS='Techniques used to add in exogenous gene sequence such as mutated genes; REPORTER GENES, to study mechanisms of gene expression; or regulatory control sequences, to study effects of temporal changes to GENE EXPRESSION.'),
  'D055881': NtO(MH='Chronobiology Discipline', MS='A multidisciplinary field of research and practice studying the periodicity of biological systems and the application of principles of chronobiology to various therapeutic strategies. Aging, biological rhythms, and cyclic phenomena are included. Statistical, computer-aided mathematical procedures are used to describe, in mathematical terminology, various biological functions over time.'),
  'D055882': NtO(MH='Nevus, Halo', MS='A benign skin lesion characterized by a zone of depigmentation surrounding the nevus.'),
  'D055883': NtO(MH='Structure Collapse', MS='Failure in built environment with loss of functional integrity.'),
  'D055884': NtO(MH='Chemical Hazard Release', MS='Uncontrolled release of a chemical from its containment that either threatens to or causes exposure to a chemical hazard. Such an incident may occur accidentally or deliberately.'),
  'D055885': NtO(MH='Biohazard Release', MS='Uncontrolled release of biological material from its containment. This either threatens to, or does, cause exposure to a biological hazard. Such an incident may occur accidentally or deliberately.'),
  'D055904': NtO(MH='Meteorology', MS='The science of studying the characteristics of the atmosphere such as its temperature, density, winds, clouds, precipitation, and other atmospheric phenomena and aiming to account for the weather in terms of external influences and the basic laws of physics. (From McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D055905': NtO(MH='Sverdlovsk Accidental Release', MS='ANTHRAX outbreak that occurred in 1979 and was associated with a research facility in Sverdlovsk, in the Ural mountain region of central RUSSIA. Most victims worked or lived in a narrow zone extending from the facility. The zone of anthrax-caused livestock mortality paralleled the northerly wind that prevailed shortly before the outbreak. It was concluded that an escape of ANTHRAX caused outbreak.'),
  'D055906': NtO(MH='Seveso Accidental Release', MS='1976 accidental release of DIOXINS from a manufacturing facility in Seveso, ITALY following an equipment failure.'),
  'D055907': NtO(MH='Climatic Processes', MS='Characteristic events occurring in the ATMOSPHERE during the interactions and transformation of various atmospheric components and conditions.'),
  'D055908': NtO(MH='Bhopal Accidental Release', MS='1984 accident in Bhopal, INDIA at a PESTICIDES facility, resulting when WATER entered a storage tank containing ISOCYANATES. The following accidental chemical release and uncontrolled reaction resulted in several thousand deaths.'),
  'D055909': NtO(MH='Magnetic Field Therapy', MS='The magnetic stimulation of specific target tissues or areas of the body for therapeutic purposes via the application of magnetic fields generated by MAGNETS or ELECTROMAGNETS.'),
  'D055910': NtO(MH='Antibodies, Immobilized', MS='Antibodies that are chemically bound to a substrate material which renders their location fixed.'),
  'D055927': NtO(MH='Cookbooks as Topic', MS='Works about a set of instructions about how to prepare food for eating using specific directions.'),
  'D055944': NtO(MH='Magnetosomes', MS='Membrane-bound prokaryotic organelles of magnetotactic bacteria that contain chains of MAGNETITE crystals which orient the bacteria to geomagnetic fields.'),
  'D055947': NtO(MH='Loeys-Dietz Syndrome', MS='An autosomal dominant aneurysm with multisystem abnormalities caused by increased TGF-BETA signaling due to mutations in type I or II of TGF-BETA RECEPTOR. Additional craniofacial features include CLEFT PALATE; CRANIOSYNOSTOSIS; HYPERTELORISM; or bifid uvula. Phenotypes closely resemble MARFAN SYNDROME; Marfanoid craniosynostosis syndrome (Shprintzen-Goldberg syndrome); and EHLERS-DANLOS SYNDROME.'),
  'D055948': NtO(MH='Sarcopenia', MS='Progressive decline in muscle mass due to aging which results in decreased functional capacity of muscles.'),
  'D055949': NtO(MH='Vasa Previa', MS='Pregnancy complication where fetal blood vessels, normally inside the umbilical cord, are left unprotected and cross FETAL MEMBRANES. It is associated with antepartum bleeding and FETAL DEATH and STILLBIRTH due to exsanguination.'),
  'D055951': NtO(MH='Functional Food', MS='Components of the usual diet that may provide health benefits beyond basic nutrients. Examples of functional foods include soy, nuts, chocolate, and cranberries (From NCCAM Backgrounder, March 2004, p3).'),
  'D055952': NtO(MH='Cogan Syndrome', MS='A condition consisting of inflammatory eye disease usually presenting as interstitial KERATITIS, vestibuloauditory dysfunction, and large- to medium-vessel vasculitis.'),
  'D055953': NtO(MH='Microscopic Polyangiitis', MS='A primary systemic vasculitis of small- and some medium-sized vessels. It is characterized by a tropism for kidneys and lungs, positive association with anti-neutrophil cytoplasmic antibodies (ANCA), and a paucity of immunoglobulin deposits in vessel walls.'),
  'D055954': NtO(MH='Corneal Endothelial Cell Loss', MS="Loss of CORNEAL ENDOTHELIUM usually following intraocular surgery (e.g., cataract surgery) or due to FUCHS' ENDOTHELIAL DYSTROPHY; ANGLE-CLOSURE GLAUCOMA; IRITIS; or aging."),
  'D055955': NtO(MH='Susac Syndrome', MS='A rare disorder consisting of microangiopathy of brain, retina, and inner ear ARTERIOLES. It is characterized by the clinical triad of encephalopathy, BRANCH RETINAL ARTERY OCCLUSION and VERTIGO/hearing loss.'),
  'D055956': NtO(MH='Diffuse Neurofibrillary Tangles with Calcification', MS="A form of presenile DEMENTIA characterized by cortical dementia, NEUROFIBRILLARY TANGLES without SENILE PLAQUES, Fahr's type CALCINOSIS, and ATROPHY in frontotemporal or TEMPORAL LOBE."),
  'D055958': NtO(MH='Piriformis Muscle Syndrome', MS='A chronic PELVIC PAIN characterized by pain deep in the buttock that may radiate to posterior aspects of the leg. It is caused by the piriformis muscle compressing or irritating the SCIATIC NERVE due to trauma, hypertrophy, inflammation or anatomic variations.'),
  'D055959': NtO(MH='Intervertebral Disc Degeneration', MS='Degenerative changes in the INTERVERTEBRAL DISC due to aging or structural damage, especially to the vertebral end-plates.'),
  'D055963': NtO(MH='Asthma, Aspirin-Induced', MS='Asthmatic adverse reaction (e.g., BRONCHOCONSTRICTION) to conventional NSAIDS including aspirin use.'),
  'D055964': NtO(MH='Alien Hand Syndrome', MS='An apraxia characterized by the affected limb having involuntary, autonomous, and purposeful behaviors that are perceived as being controlled by an external force. Often the affected limb interferes with the actions of the normal limb. Symptoms develop from lesions in the CORPUS CALLOSUM or medial frontal cortex, stroke, infarction, and neurodegenerative diseases (e.g., CREUTZFELDT-JAKOB SYNDROME, corticobasal degeneration).'),
  'D055966': NtO(MH='Nipple Aspirate Fluid', MS='Fluid collected from nipple by gentle aspiration. The fluid contains cells and extracellular fluid from the breast ductal epithelium.'),
  'D055984': NtO(MH='Cathode Ray Tube', MS='A vacuum tube equipped with an electron emitting CATHODE and a fluorescent screen which emits visible light when excited by the cathode ray. Cathode ray tubes are used as imaging devises for TELEVISIONS; COMPUTER TERMINALS; TEXT TELECOMMUNICATION DEVICES; oscilloscopes; and other DATA DISPLAY devices.'),
  'D055985': NtO(MH='Latent Tuberculosis', MS='The dormant form of TUBERCULOSIS where the person shows no obvious symptoms and no sign of the causative agent (Mycobacterium tuberculosis) in the SPUTUM despite being positive for tuberculosis infection skin test.'),
  'D055986': NtO(MH='Vital Signs', MS='The signs of life that may be monitored or measured, namely pulse rate, respiratory rate, body temperature, and blood pressure.'),
  'D055988': NtO(MH='Uterine Artery', MS='A branch arising from the internal iliac artery in females, that supplies blood to the uterus.'),
  'D055989': NtO(MH='Septal Occluder Device', MS='A CATHETER-delivered implant used for closing abnormal holes in the cardiovascular system, especially HEART SEPTAL DEFECTS; or passageways intentionally made during cardiovascular surgical procedures.'),
  'D055990': NtO(MH='Therapeutic Misconception', MS='Misunderstanding among individuals, frequently research subjects, of scientific methods such as randomization and placebo controls.'),
  'D055991': NtO(MH='Health Records, Personal', MS='Longitudinal patient-maintained records of individual health history and tools that allow individual control of access.'),
  'D055993': NtO(MH='Germ Cells, Plant', MS='The reproductive cells of plants.'),
  'D055994': NtO(MH='Limnology', MS='The study of the physical, chemical, hydrological, and biological aspects of fresh water bodies. (EPA Terms of Environment: Glossary, Abbreviations and Acronyms [Internet] US Environmental Protection Agency [cited 2008 Sep 25] available from http://www.epa.gov/OCEPAterms/aterms.html)'),
  'D056004': NtO(MH='Molecular Dynamics Simulation', MS='A computer simulation developed to study the motion of molecules over a period of time.'),
  'D056005': NtO(MH='Leprosy, Paucibacillary', MS='A form of LEPROSY classified by the World Health Organization for the purpose of treatment, based on clinical manifestations and skin smear results. Patients with paucibacillary leprosy have fewer than six skin lesions with no causative agent MYCOBACTERIUM LEPRAE on any slit-skin smear testing. Paucibacillary leprosy encompasses indeterminate, borderline tuberculoid, and tuberculoid leprosy.'),
  'D056006': NtO(MH='Leprosy, Multibacillary', MS='A form of LEPROSY classified by the World Health Organization for the purpose of treatment, based on clinical manifestations and skin smear results. Patients with multibacillary leprosy have six or more lesions with or without positive skin smear results for the causative agent MYCOBACTERIUM LEPRAE. Multibacillary leprosy encompasses borderline lepromatous, midborderline, and lepromatous leprosy.'),
  'D056044': NtO(MH='Actigraphy', MS='The measurement and recording of MOTOR ACTIVITY to assess rest/activity cycles.'),
  'D056084': NtO(MH='Genetic Fitness', MS='The capability of an organism to survive and reproduce. The phenotypic expression of the genotype in a particular environment determines how genetically fit an organism will be.'),
  'D056104': NtO(MH='Commotio Cordis', MS='A sudden CARDIAC ARRHYTHMIA (e.g., VENTRICULAR FIBRILLATION) caused by a blunt, non-penetrating impact to the precordial region of chest wall. Commotio cordis often results in sudden death without prompt cardiopulmonary defibrillation.'),
  'D056124': NtO(MH='Slit Ventricle Syndrome', MS='An intermittent and self-limiting headache disorder in individuals with CEREBROSPINAL FLUID SHUNTS emplaced for the treatment of HYDROCEPHALUS. The symptoms of headache, vomiting, and cranial neuropathies are associated with intermittent obstruction of the shunt.'),
  'D056126': NtO(MH='Orchiopexy', MS='A surgical procedure in which an undescended testicle is sutured inside the SCROTUM in male infants or children to correct CRYPTORCHIDISM. Orchiopexy is also performed to treat TESTICULAR TORSION in adults and adolescents.'),
  'D056127': NtO(MH='Hydrobiology', MS='The study of aquatic life inhabiting bodies of water, including growth, morphology, physiology, genetics, distribution, and interactions with other organisms and the environment. It includes MARINE HYDROBIOLOGY.'),
  'D056128': NtO(MH='Obesity, Abdominal', MS='A condition of having excess fat in the abdomen. Abdominal obesity is typically defined as waist circumferences of 40 inches or more in men and 35 inches or more in women. Abdominal obesity raises the risk of developing disorders, such as DIABETES; HYPERTENSION; and METABOLIC SYNDROME.'),
  'D056130': NtO(MH='Gametogenesis, Plant', MS='The process of germ cell development in plants, from the primordial PLANT GERM CELLS to the mature haploid PLANT GAMETES.'),
  'D056144': NtO(MH='Pyriform Sinus', MS='A recess on each side in the wall of the HYPOPHARYNX.'),
  'D056145': NtO(MH='Freshwater Biology', MS='The study of life and ECOLOGIC SYSTEMS in bodies of FRESHWATER.'),
  'D056146': NtO(MH='Corpse Dismemberment', MS='Disjoining the limbs or other parts of a corpse, often in association with criminal acts.'),
  'D056147': NtO(MH='Equine-Assisted Therapy', MS='Therapy assisted by the use of a horse and/or its movement, including equine-assisted psychotherapy, horseback riding, and hippotherapy.'),
  'D056148': NtO(MH='Chromatography, Reverse-Phase', MS='A chromatography technique in which the stationary phase is composed of a non-polar substance with a polar mobile phase, in contrast to normal-phase chromatography in which the stationary phase is a polar substance with a non-polar mobile phase.'),
  'D056149': NtO(MH='Epicardial Mapping', MS='Recording the locations and measurements of electrical activity in the EPICARDIUM by placing electrodes on the surface of the heart to analyze the patterns of activation and to locate arrhythmogenic sites.'),
  'D056150': NtO(MH='Acute Generalized Exanthematous Pustulosis', MS='Rare skin eruption characterized by acute formation of pustules filled with NEUTROPHILS, fever, and peripheral blood LEUKOCYTOSIS. Most cases are associated with the use of antibiotics (e.g., BETA-LACTAMS).'),
  'D056151': NtO(MH='Airway Remodeling', MS='The structural changes in the number, mass, size and/or composition of the airway tissues.'),
  'D056152': NtO(MH='Respiratory Rate', MS='The number of times an organism breathes with the lungs (RESPIRATION) per unit time, usually per minute.'),
  'D056164': NtO(MH='Teaching Rounds', MS='Systematic discussions and TEACHING conducted in HOSPITALS and HEALTHCARE facilities related to patient care.'),
  'D056185': NtO(MH='Simian T-lymphotropic virus 3', MS='Strains of PRIMATE T-LYMPHOTROPIC VIRUS 3 isolated from diverse primate species.'),
  'D056186': NtO(MH='Metagenomics', MS='The systematic study of the GENOMES of assemblages of organisms.'),
  'D056187': NtO(MH='Human T-lymphotropic virus 3', MS='A strain of PRIMATE T-LYMPHOTROPIC VIRUS 3 that is genetically similar to STLV-3.'),
  'D056189': NtO(MH='Viral Tropism', MS='The specificity of a virus for infecting a particular type of cell or tissue.'),
  'D056224': NtO(MH='Viral Structures', MS='The structural parts of the VIRION.'),
  'D056226': NtO(MH='Bacterial Structures', MS='The parts of bacteria.'),
  'D056228': NtO(MH='Feedback, Sensory', MS="A mechanism of communicating one's own sensory system information about a task, movement or skill."),
  'D056229': NtO(MH='Fungal Structures', MS='The parts of fungi.'),
  'D056245': NtO(MH='Mi-2 Nucleosome Remodeling and Deacetylase Complex', MS='A enzyme complex involved in the remodeling of NUCLEOSOMES. The complex is comprised of at least seven subunits and includes both histone deacetylase and ATPase activities.'),
  'D056246': NtO(MH='Chemistry, Analytic', MS="The branch of chemistry dealing with detection (qualitative) and determination (quantitative) of substances. (Grant & Hackh's Chemical Dictionary, 5th ed)"),
  'D056264': NtO(MH='Sin3 Histone Deacetylase and Corepressor Complex', MS='A multisubunit enzyme complex that regulates GENETIC TRANSCRIPTION by deacetylating the HISTONE residues of NUCLEOSOMES.'),
  'D056265': NtO(MH='Microbial Interactions', MS='The inter- and intra-relationships between various microorganisms. This can include both positive (like SYMBIOSIS) and negative (like ANTIBIOSIS) interactions. Examples include virus - bacteria and bacteria - bacteria.'),
  'D056266': NtO(MH='Erythrokeratodermia Variabilis', MS='An autosomal dominant skin disease characterized by transient and variable noninflammatory ERYTHEMA and hyperkeratosis. It has been associated with mutations in the genes that code for CONNEXINS. Erythrokeratodermia variabilis inherited in an autosomal recessive fashion has also been reported. Affected individuals often develop PALMOPLANTAR KERATODERMA.'),
  'D056267': NtO(MH='Pagetoid Reticulosis', MS='A distinct variant of mycosis fungoides characterized by the presence of localized patches or plaques with an intraepidermal proliferation of neoplastic cells.'),
  'D056284': NtO(MH='Histone Deacetylase 1', MS='A histone deacetylase subtype that is found along with HISTONE DEACETYLASE 2; RETINOBLASTOMA-BINDING PROTEIN 4; and RETINOBLASTOMA-BINDING PROTEIN 7 as core components of histone deacetylase complexes.'),
  'D056285': NtO(MH='Cryptococcus gattii', MS='A species of the fungus CRYPTOCOCCUS. Its teleomorph is Filobasidiella bacillispora.'),
  'D056286': NtO(MH='Retinoblastoma Binding Proteins', MS='A family of endogenous regulatory proteins that associate with RETINOBLASTOMA PROTEIN via a specific high-affinity binding domain. Members of this family of proteins are often found associated with histone-modifying enzymes and protein complexes that regulate gene expression.'),
  'D056304': NtO(MH='Genu Valgum', MS='An inward slant of the thigh in which the knees are close together and the ankles far apart. Genu valgum can develop due to skeletal and joint dysplasias (e.g., OSTEOARTHRITIS; HURLER SYNDROME); and malnutrition (e.g., RICKETS; FLUORIDE POISONING).'),
  'D056305': NtO(MH='Genu Varum', MS="An outward slant of the thigh in which the knees are wide apart and the ankles close together. Genu varum can develop due to skeletal and joint dysplasia (e.g., OSTEOARTHRITIS; Blount's disease); and malnutrition (e.g., RICKETS; FLUORIDE POISONING)."),
  'D056324': NtO(MH='Diffusion Tensor Imaging', MS='The use of diffusion ANISOTROPY data from diffusion magnetic resonance imaging results to construct images based on the direction of the faster diffusing molecules.'),
  'D056325': NtO(MH='Neuronal Tract-Tracers', MS='Substances used to identify the location and to characterize the types of NEURAL PATHWAYS.'),
  'D056344': NtO(MH='Executive Function', MS='A set of cognitive functions that controls complex, goal-directed thought and behavior. Executive function involves multiple domains, such as CONCEPT FORMATION, goal management, cognitive flexibility, INHIBITION control, and WORKING MEMORY. Impaired executive function is seen in a range of disorders, e.g., SCHIZOPHRENIA; and ADHD.'),
  'D056345': NtO(MH='Theory of Mind', MS='The ability to attribute mental states (e.g., beliefs, desires, feelings, intentions, thoughts, etc.) to self and to others, allowing an individual to understand and infer behavior on the basis of the mental states. Difference or deficit in theory of mind is associated with ASPERGER SYNDROME; AUTISTIC DISORDER; and SCHIZOPHRENIA, etc.'),
  'D056346': NtO(MH='Sternotomy', MS='Making an incision in the STERNUM.'),
  'D056347': NtO(MH='Neuroanatomical Tract-Tracing Techniques', MS='Methods used to label and follow the course of NEURAL PATHWAYS by AXONAL TRANSPORT of injected NEURONAL TRACT-TRACERS.'),
  'D056348': NtO(MH='Emotional Intelligence', MS="The ability to understand and manage emotions and to use emotional knowledge to enhance thought and deal effectively with tasks. Components of emotional intelligence include empathy, self-motivation, self-awareness, self-regulation, and social skill. Emotional intelligence is a measurement of one's ability to socialize or relate to others."),
  'D056352': NtO(MH='Athletes', MS='Individuals who have developed skills, physical stamina and strength or participants in SPORTS or other physical activities.'),
  'D056364': NtO(MH='Colloid Cysts', MS='Benign, congenital, neuroepithelial cysts that are typically filled with a viscous mucus. They usually arise in the anterior portion of the THIRD VENTRICLE between the fornices.'),
  'D056384': NtO(MH='Group I Chaperonins', MS='A subcategory of chaperonins found in MITOCHONDRIA; CHLOROPLASTS; and BACTERIA. Group I chaperonins form into a barrel-shaped macromolecular structure that is enclosed by a separate lid-like protein component.'),
  'D056385': NtO(MH='Group II Chaperonins', MS='A subcategory of chaperonins found in ARCHAEA and the CYTOSOL of eukaryotic cells. Group II chaperonins form a barrel-shaped macromolecular structure that is distinct from GROUP I CHAPERONINS in that it does not utilize a separate lid like structure to enclose proteins.'),
  'D056404': NtO(MH='Chaperonin Containing TCP-1', MS='A group II chaperonin found in eukaryotic CYTOSOL. It is comprised of eight subunits with each subunit encoded by a separate gene. This chaperonin is named after one of its subunits which is a T-COMPLEX REGION-encoded polypeptide.'),
  'D056405': NtO(MH='Thermosomes', MS='Group II chaperonins found in species of ARCHAEA.'),
  'D056424': NtO(MH='Decompressive Craniectomy', MS='Excision of part of the skull. This procedure is used to treat elevated intracranial pressure that is unresponsive to conventional treatment.'),
  'D056426': NtO(MH='Genetic Loci', MS='Specific regions that are mapped within a GENOME. Genetic loci are usually identified with a shorthand notation that indicates the chromosome number and the position of a specific band along the P or Q arm of the chromosome where they are found. For example the locus 6p21 is found within band 21 of the P-arm of CHROMOSOME 6. Many well known genetic loci are also known by common names that are associated with a genetic function or HEREDITARY DISEASE.'),
  'D056427': NtO(MH='t-Complex Genome Region', MS='A 20 cM region of mouse chromosome 17 that is represented by a least two HAPLOTYPES. One of the haplotypes is referred to as the t-haplotype and contains an unusual array of mutations that affect embryonic development and male fertility. The t-haplotype is maintained in the gene pool by the presence of unusual features that prevent its recombination.'),
  'D056444': NtO(MH='Axonemal Dyneins', MS='Dyneins that are responsible for ciliary and flagellar beating.'),
  'D056445': NtO(MH='Cytoplasmic Dyneins', MS='Dyneins that are responsible for intracellular transport, MITOSIS, cell polarization, and movement within the cell.'),
  'D056446': NtO(MH='Knee-Chest Position', MS='The posture of an individual supported by the knees and chest resting on a table.'),
  'D056447': NtO(MH='Animal Assisted Therapy', MS='The use of live animal as a means of therapy'),
  'D056448': NtO(MH='Earth Sciences', MS='Fields of science encompassing studies and research from the disciplines of PHYSICS; CHEMISTRY; BIOLOGICAL SCIENCES; and MATHEMATICS; that are related to the planet EARTH. Subfields include atmospheric chemistry; CLIMATOLOGY; ENVIRONMENTAL SCIENCES; GEOGRAPHY; GEOLOGY; geophysics; METEOROLOGY; OCEANOGRAPHY; PALEONTOLOGY; mineralogy; and seismology.'),
  'D056449': NtO(MH='Bacillales', MS='An order of gram-positive bacteria in the class Bacilli, phylum Firmicutes.'),
  'D056450': NtO(MH='Alicyclobacillus', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING RODS, in the family Alicyclobacillaceae, containing a unique lipid in their membranes.'),
  'D056464': NtO(MH='Histone Deacetylase 2', MS='A histone deacetylase subtype that is found along with HISTONE DEACETYLASE 1; RETINOBLASTOMA-BINDING PROTEIN 4; and RETINOBLASTOMA-BINDING PROTEIN 7 as core components of histone deacetylase complexes.'),
  'D056465': NtO(MH='Retinoblastoma-Binding Protein 1', MS='A ubiquitously expressed regulatory protein that contains a retinoblastoma protein binding domain and an AT-rich interactive domain. The protein may play a role in recruiting HISTONE DEACETYLASES to the site of RETINOBLASTOMA PROTEIN-containing transcriptional repressor complexes.'),
  'D056466': NtO(MH='Histone Demethylases', MS='Enzymes that catalyse the removal of methyl groups from LYSINE or ARGININE residues found on HISTONES. Many histone demethylases generally function through an oxidoreductive mechanism.'),
  'D056484': NtO(MH='Jumonji Domain-Containing Histone Demethylases', MS='A family of histone demethylases that share a conserved Jumonji C domain. The enzymes function via an iron-dependent dioxygenase mechanism that couples the conversion of 2-oxoglutarate to succinate to the hydroxylation of N-methyl groups.'),
  'D056485': NtO(MH='Retinoblastoma-Binding Protein 2', MS='A retinoblastoma binding protein that is also a member of the Jumonji-domain histone demethylases. It has demethylation activity towards specific LYSINE residues found on HISTONE H3.'),
  'D056486': NtO(MH='Chemical and Drug Induced Liver Injury', MS='A spectrum of clinical liver diseases ranging from mild biochemical abnormalities to ACUTE LIVER FAILURE, caused by drugs, drug metabolites, herbal and dietary supplements and chemicals from the environment.'),
  'D056487': NtO(MH='Chemical and Drug Induced Liver Injury, Chronic', MS='Liver disease lasting six months or more, caused by an adverse effect of a drug or chemical. The adverse effect may be caused by drugs, drug metabolites, chemicals from the environment, or an idiosyncratic response.'),
  'D056488': NtO(MH='Histone Chaperones', MS='Proteins involved in the assembly and disassembly of HISTONES into NUCLEOSOMES.'),
  'D056489': NtO(MH='Nucleoplasmins', MS='A family of histone molecular chaperones that play roles in sperm CHROMATIN decondensation and CHROMATIN ASSEMBLY in fertilized eggs. They were originally discovered in XENOPUS egg extracts as histone-binding factors that mediate nucleosome formation in vitro.'),
  'D056490': NtO(MH='Nucleosome Assembly Protein 1', MS='A histone chaperone that facilitates nucleosome assembly by mediating the formation of the histone octamer and its transfer to DNA.'),
  'D056504': NtO(MH='Chromatin Assembly Factor-1', MS='A histone chaperone protein that plays a role in the deposition of NUCLEOSOMES on newly synthesized DNA. It is comprised of three different subunits of 48, 60, and 150 kDa molecular size. The 48 kDa subunit, RETINOBLASTOMA-BINDING PROTEIN 4, is also a component of several other protein complexes involved in chromatin remodeling.'),
  'D056505': NtO(MH='Retinoblastoma-Binding Protein 4', MS='A retinoblastoma-binding protein that is involved in CHROMATIN REMODELING, histone deacetylation, and repression of GENETIC TRANSCRIPTION. Although initially discovered as a retinoblastoma binding protein it has an affinity for core HISTONES and is a subunit of chromatin assembly factor-1 and polycomb repressive complex 2.'),
  'D056506': NtO(MH='Geobacillus', MS='A genus of gram-positive, endospore-forming, thermophilic bacteria in the family BACILLACEAE.'),
  'D056507': NtO(MH='Paenibacillus', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING RODS in the family Paenibacillaceae.'),
  'D056508': NtO(MH='Sleep Medicine Specialty', MS='A medical specialty concerned with the diagnosis and treatment of SLEEP WAKE DISORDERS and their causes.'),
  'D056509': NtO(MH='Thermoactinomyces', MS="A genus of gram-positive bacteria in the family Thermoactinomycetaceae, that can cause FARMER'S LUNG."),
  'D056510': NtO(MH='Position-Specific Scoring Matrices', MS='Tabular numerical representations of sequence motifs displaying their variability as likelihood values for each possible residue at each position in a sequence. Position-specific scoring matrices (PSSMs) are calculated from position frequency matrices.'),
  'D056511': NtO(MH='Metallochaperones', MS='A family of soluble metal binding proteins that are involved in the intracellular transport of specific metal ions and their transfer to the appropriate metalloprotein precursor.'),
  'D056524': NtO(MH='Retinoblastoma-Binding Protein 7', MS='A retinoblastoma-binding protein that has an affinity for core HISTONES. It is found as a subunit of protein complexes that are in involved in the enzymatic modification of histones including the Mi2 and Sin3 histone deacetylase complexes and the polycomb repressive complex 2.'),
  'D056545': NtO(MH='Group III Histone Deacetylases', MS='A subclass of histone deacetylases that are NAD-dependent. Several members of the SIRTUINS family are included in this subclass.'),
  'D056546': NtO(MH='Lactobacillales', MS='An order of gram-positive bacteria in the class Bacilli, that have the ability to ferment sugars to lactic acid. They are widespread in nature and commonly used to produce FERMENTED FOODS AND BEVERAGES.'),
  'D056547': NtO(MH='CA1 Region, Hippocampal', MS='One of four subsections of the hippocampus described by Lorente de No, located furthest from the DENTATE GYRUS.'),
  'D056564': NtO(MH='Sirtuin 1', MS='A sirtuin family member found primarily in the CELL NUCLEUS. It is an NAD-dependent deacetylase with specificity towards HISTONES and a variety of proteins involved in gene regulation.'),
  'D056565': NtO(MH='Sirtuin 2', MS='A sirtuin family member found primarily in the CYTOPLASM. It is a multifunctional enzyme that contains a NAD-dependent deacetylase activity that is specific for HISTONES and a mono-ADP-ribosyltransferase activity.'),
  'D056566': NtO(MH='Sirtuin 3', MS='A sirtuin family member found primarily in MITOCHONDRIA. It is a multifunctional enzyme that contains a NAD-dependent deacetylase activity that is specific for HISTONES and a mono-ADP-ribosyltransferase activity.'),
  'D056567': NtO(MH='Aerococcaceae', MS='A family of gram-positive lactic acid-producing bacteria in the order Lactobacillales.'),
  'D056568': NtO(MH='Carnobacteriaceae', MS='A family of gram-positive, lactic acid-producing bacteria in the order Lactobacillales. It includes both high-pressure-loving species (piezophiles) found in the deep ocean, and Antarctic species.'),
  'D056569': NtO(MH='Enterococcaceae', MS='A family of gram-positive bacteria in the order Lactobacillales, phylum Firmicutes.'),
  'D056570': NtO(MH='Aerococcus', MS='A genus of gram-positive cocci in the family AEROCOCCACEAE, occurring as airborne saprophytes.'),
  'D056571': NtO(MH='Postcard', MS='Card on which a message may be written or printed for mailing without an envelope. Art & Architectural Thesaurus Online www.getty.edu/research/conducting_research/vocabularies/aat/ accessed 12/18/2008'),
  'D056572': NtO(MH='Histone Deacetylase Inhibitors', MS='Compounds that inhibit HISTONE DEACETYLASES. This class of drugs may influence gene expression by increasing the level of acetylated HISTONES in specific CHROMATIN domains.'),
  'D056573': NtO(MH='Carnobacterium', MS='A genus of gram-positive bacteria in the family CARNOBACTERIACEAE. They are tolerant to freezing/thawing and high pressure and able to grow at low temperatures.'),
  'D056584': NtO(MH='Leuconostocaceae', MS='A family of gram-positive bacteria in the order Lactobacillales. Lactic acid is the main product of their carbohydrate metabolism.'),
  'D056585': NtO(MH='Oenococcus', MS='A genus of GRAM-POSITIVE COCCI in the family LEUCONOSTOCACEAE. It is the primary bacteria involved in carrying out malolactic conversion in winemaking.'),
  'D056586': NtO(MH='Acute Chest Syndrome', MS='Respiratory syndrome characterized by the appearance of a new pulmonary infiltrate on chest x-ray, accompanied by symptoms of fever, cough, chest pain, tachypnea, or DYSPNEA, often seen in patients with SICKLE CELL ANEMIA. Multiple factors (e.g., infection, and pulmonary FAT EMBOLISM) may contribute to the development of the syndrome.'),
  'D056587': NtO(MH='Cryopyrin-Associated Periodic Syndromes', MS='A group of rare autosomal dominant diseases, commonly characterized by atypical URTICARIA (hives) with systemic symptoms that develop into end-organ damage. The atypical hives do not involve T-cell or autoantibody. Cryopyrin-associated periodic syndrome includes three previously distinct disorders: Familial cold autoinflammatory syndrome; Muckle-Wells Syndrome; and CINCA Syndrome, that are now considered to represent a disease continuum, all caused by NLRP3 PROTEIN mutations.'),
  'D056604': NtO(MH='Grape Seed Extract', MS='Exudate from seeds of the grape plant Vitis vinifera, composed of oils and secondary plant metabolites (BIOFLAVONOIDS and polyphenols) credited with important medicinal properties.'),
  'D056624': NtO(MH='Apexification', MS='Endodontic procedure performed to induce TOOTH APEX barrier development. ROOT CANAL FILLING MATERIALS are used to repair open apex or DENTAL PULP NECROSIS in an immature tooth. CALCIUM HYDROXIDE and mineral trioxide aggregate are commonly used as the filling materials.'),
  'D056625': NtO(MH='Endosperm', MS='Nutritive tissue of the seeds of flowering plants that surrounds the EMBRYOS. It is produced by a parallel process of fertilization in which a second male gamete from the pollen grain fuses with two female nuclei within the embryo sac. The endosperm varies in ploidy and contains reserves of starch, oils, and proteins, making it an important source of human nutrition.'),
  'D056626': NtO(MH='Inflorescence', MS='A cluster of FLOWERS (as opposed to a solitary flower) arranged on a main stem of a plant.'),
  'D056627': NtO(MH='Peritoneal Fibrosis', MS='Disorder characterized by a wide range of structural changes in PERITONEUM, resulting from fibrogenic or inflammatory processes. Peritoneal fibrosis is a common complication in patients receiving PERITONEAL DIALYSIS and contributes to its gradual decrease in efficiency.'),
  'D056629': NtO(MH='Dicistroviridae', MS='A family of invertebrate RNA viruses in the order Picornavirales.'),
  'D056630': NtO(MH='Beekeeping', MS='The management and maintenance of colonies of honeybees.'),
  'D056631': NtO(MH='Colony Collapse', MS='The sudden collapse and disappearance or diminution of a colony of organisms.'),
  'D056646': NtO(MH='Cathepsin F', MS='A lysosomal papain-related cysteine proteinase that is expressed in a broad variety of cell types.'),
  'D056647': NtO(MH='Systemic Vasculitis', MS='A heterogeneous group of diseases characterized by inflammation and necrosis of the blood vessel walls.'),
  'D056648': NtO(MH='Anti-Neutrophil Cytoplasmic Antibody-Associated Vasculitis', MS='Group of systemic vasculitis with a strong association with ANCA. The disorders are characterized by necrotizing inflammation of small and medium size vessels, with little or no immune-complex deposits in vessel walls.'),
  'D056649': NtO(MH='Cathepsin G', MS='A serine protease found in the azurophil granules of NEUTROPHILS. It has an enzyme specificity similar to that of chymotrypsin C.'),
  'D056650': NtO(MH='Vulvodynia', MS="Complex pain syndrome with unknown etiology, characterized by constant or intermittent generalized vulva pain (Generalized vulvodynia) or localized burning sensations in the VESTIBULE area when pressure is applied (Vestibulodynia, or Vulvar Vestibulitis Syndrome). Typically, vulvar tissue with vulvodynia appears normal without infection or skin disease. Vulvodynia impacts negatively on a woman's quality of life as it interferes with sexual and daily activities."),
  'D056651': NtO(MH='CA2 Region, Hippocampal', MS='A subsection of the hippocampus, described by Lorente de No, that is located between the HIPPOCAMPUS CA1 FIELD and the HIPPOCAMPUS CA3 FIELD.'),
  'D056653': NtO(MH='Rheumatoid Vasculitis', MS='Necrotizing VASCULITIS of small and medium size vessels, developing as a complication in RHEUMATOID ARTHRITIS patients. It is characterized by peripheral vascular lesions, cutaneous ULCERS, peripheral GANGRENE, and MONONEURITIS MULTIPLEX.'),
  'D056654': NtO(MH='CA3 Region, Hippocampal', MS='A subsection of the hippocampus, described by Lorente de No, that is located between the HIPPOCAMPUS CA2 FIELD and the DENTATE GYRUS.'),
  'D056655': NtO(MH='Cathepsin H', MS='An ubiquitously-expressed lysosomal cysteine protease that is involved in protein processing. The enzyme has both endopeptidase and aminopeptidase activities.'),
  'D056656': NtO(MH='Lab-On-A-Chip Devices', MS='Microdevices that combine microfluidics technology with electrical and/or mechanical functions for analyzing very small fluid volumes. They consist of microchannels etched into substrates made of silicon, glass, or polymer using processes similar to photolithography. The test fluids in the channels can then interact with different elements such as electrodes, photodetectors, chemical sensors, pumps, and valves.'),
  'D056657': NtO(MH='Cathepsin K', MS='A cysteine protease that is highly expressed in OSTEOCLASTS and plays an essential role in BONE RESORPTION as a potent EXTRACELLULAR MATRIX-degrading enzyme.'),
  'D056660': NtO(MH='Hereditary Autoinflammatory Diseases', MS='Hereditary inflammation conditions, characterized by recurrent episodes of systemic inflammation. Common symptoms include recurrent fever, rash, arthritis, fatigue, and secondary AMYLOIDOSIS. Hereditary autoinflammatory diseases are associated with mutations in genes involved in regulation of normal inflammatory process and are not caused by AUTOANTIBODIES, or antigen specific T-LYMPHOCYTES.'),
  'D056661': NtO(MH='Human bocavirus', MS='A member of the family PARVOVIRIDAE, subfamily PARVOVIRINAE, originally isolated from human nasopharyngeal aspirates in patients with respiratory disease.'),
  'D056662': NtO(MH='Proboscidea Mammal', MS='An order of MAMMALS that feeds using their trunks. It contains one family, the Elephantidae, comprised of two living genera of ELEPHANTS plus the extinct MAMMOTHS and MASTODONS.'),
  'D056663': NtO(MH='Cathepsin Z', MS='A ubiquitously-expressed cysteine peptidase that exhibits carboxypeptidase activity. It is highly expressed in a variety of immune cell types and may play a role in inflammatory processes and immune responses.'),
  'D056664': NtO(MH='Cathepsin W', MS='A cysteine endopeptidase found in NATURAL KILLER CELLS and CYTOTOXIC T-LYMPHOCYTES. It may have a specific function in the mechanism or regulation of cytolytic activity of immune cells.'),
  'D056665': NtO(MH='Mammoths', MS='An extinct genus of large mammals in the family Elephantidae that fed by grazing on low vegetation. Most died out at the end of the last ice age.'),
  'D056666': NtO(MH='Mastodons', MS='An extinct genus of large mammals in the family Mammutidae, that fed by browsing on tall plants. Eurasian species died out three million years ago, but some specimens of the American Mastodon survived the last Ice Age.'),
  'D056667': NtO(MH='Biometric Identification', MS='A method of differentiating individuals based on the analysis of qualitative or quantitative biological traits or patterns. This process which has applications in forensics and identity theft prevention includes DNA profiles or DNA fingerprints, hand fingerprints, automated facial recognition, iris scan, hand geometry, retinal scan, vascular patterns, automated voice pattern recognition, and ultrasound of fingers.'),
  'D056668': NtO(MH='Cathepsin L', MS='A ubiquitously-expressed cysteine protease that plays an enzymatic role in POST-TRANSLATIONAL PROTEIN PROCESSING of proteins within SECRETORY GRANULES.'),
  'D056669': NtO(MH='Galactogogues', MS='Substances that induce LACTATION.'),
  'D056684': NtO(MH='Yellow Nail Syndrome', MS='A rare condition characterized by the presence of yellow nails, LYMPHEDEMA, and/or PLEURAL EFFUSION with respiratory tract involvement. Abnormal lymphatic network may play a role in its etiology. Occasionally inherited, yellow nail syndrome mostly is sporadic without apparent family history.'),
  'D056685': NtO(MH='Costello Syndrome', MS='Rare congenital disorder with multiple anomalies including: characteristic dysmorphic craniofacial features, musculoskeletal abnormalities, neurocognitive delay, and high prevalence of cancer. Germline mutations in H-Ras protein can cause Costello syndrome. Costello syndrome shows early phenotypic overlap with other disorders that involve MAP KINASE SIGNALING SYSTEM (e.g., NOONAN SYNDROME and cardiofaciocutaneous syndrome).'),
  'D056686': NtO(MH='Mimiviridae', MS='A family of nucleocytoplasmic, large, double-stranded DNA viruses with extremely complex genomes.'),
  'D056687': NtO(MH='Off-Label Use', MS="The practice of prescribing or using a drug or a vaccine outside the scope of the drug's official approved label as designated by a regulatory agency concerning the treatment of a particular disease or condition."),
  'D056689': NtO(MH='Uropathogenic Escherichia coli', MS='Strains of Escherichia coli that preferentially grow and persist within the urinary tract. They exhibit certain virulence factors and infection pathways that cause URINARY TRACT INFECTIONS.'),
  'D056690': NtO(MH='Prolactin-Releasing Hormone', MS='A hypothalamic hormone that is synthesized as a precursor protein which is cleaved into two peptides. In addition to stimulating PROLACTIN release the peptides bind to specific G-PROTEIN-COUPLED RECEPTORS found at locations within the BRAIN.'),
  'D056692': NtO(MH='Prebiotics', MS='Non-digestible food ingredients mostly of a carbohydrate base that improve human health by selectively stimulating the growth and/or activity of existing BACTERIA in the COLON.'),
  'D056693': NtO(MH='Propionic Acidemia', MS='Autosomal recessive metabolic disorder caused by mutations in PROPIONYL-COA CARBOXYLASE genes that result in dysfunction of branch chain amino acids and of the metabolism of certain fatty acids. Neonatal clinical onset is characterized by severe metabolic acidemia accompanied by hyperammonemia, HYPERGLYCEMIA, lethargy, vomiting, HYPOTONIA; and HEPATOMEGALY. Survivors of the neonatal onset propionic acidemia often show developmental retardation, and intolerance to dietary proteins. Late-onset form of the disease shows mild mental and/or developmental retardation, sometimes without metabolic acidemia.'),
  'D056704': NtO(MH='Adaptive Immunity', MS='Protection from an infectious disease agent that is mediated by B- and T- LYMPHOCYTES following exposure to specific antigen, and characterized by IMMUNOLOGIC MEMORY. It can result from either previous infection with that agent or vaccination (IMMUNITY, ACTIVE), or transfer of antibody or lymphocytes from an immune donor (IMMUNIZATION, PASSIVE).'),
  'D056705': NtO(MH='Apitherapy', MS='The medical use of honey bee products such as BEE VENOM; HONEY; bee pollen; PROPOLIS; and royal jelly.'),
  'D056724': NtO(MH='Immunity, Humoral', MS='Antibody-mediated immune response. Humoral immunity is brought about by ANTIBODY FORMATION, resulting from TH2 CELLS activating B-LYMPHOCYTES, followed by COMPLEMENT ACTIVATION.'),
  'D056725': NtO(MH='von Willebrand Disease, Type 1', MS='A subtype of von Willebrand disease that results from a partial deficiency of VON WILLEBRAND FACTOR.'),
  'D056726': NtO(MH='Genetic Association Studies', MS='The analysis of a sequence such as a region of a chromosome, a haplotype, a gene, or an allele for its involvement in controlling the phenotype of a specific trait, metabolic pathway, or disease.'),
  'D056727': NtO(MH='Endangered Species', MS='An animal or plant species in danger of extinction. Causes can include human activity, changing climate, or change in predator/prey ratios.'),
  'D056728': NtO(MH='von Willebrand Disease, Type 2', MS='A subtype of von Willebrand disease that results from qualitative deficiencies of VON WILLEBRAND FACTOR. The subtype is divided into several variants with each variant having a distinctive pattern of PLATELET-interaction.'),
  'D056729': NtO(MH='von Willebrand Disease, Type 3', MS='A subtype of von Willebrand disease that results from a total or near total deficiency of VON WILLEBRAND FACTOR.'),
  'D056730': NtO(MH='Silver-Russell Syndrome', MS='Genetically and clinically heterogeneous disorder characterized by low birth weight, postnatal growth retardation, facial dysmorphism, bilateral body asymmetry, and clinodactyly of the fifth fingers. Alterations in GENETIC IMPRINTING are involved. Hypomethylation of IGF2/H19 locus near an imprinting center region of chromosome 11p15 plays a role in a subset of Silver-Russell syndrome. Hypermethylation of the same chromosomal region, on the other hand, can cause BECKWITH-WIEDEMANN SYNDROME. Maternal UNIPARENTAL DISOMY for chromosome 7 is known to play a role in its etiology.'),
  'D056731': NtO(MH='Donohue Syndrome', MS='Rare autosomal recessive syndrome of extreme insulin resistance due to mutations in the binding domain of INSULIN RECEPTOR. Clinical features include severe intrauterine and postnatal growth restriction, characteristic dysmorphic FACIES; HIRSUTISM; VIRILIZATION; multiple endocrine abnormalities, and early death.'),
  'D056732': NtO(MH='Prolidase Deficiency', MS='Rare autosomal recessive disorder of metabolism due to mutations in the prolidase gene. It is characterized by recurrent lower extremity skin ulcers, recurrent infections, and FACIES, often with INTELLECTUAL DISABILITY.'),
  'D056733': NtO(MH='Carney Complex', MS='Autosomal dominant syndrome characterized by cardiac and cutaneous MYXOMAS; LENTIGINOSIS (spotty pigmentation of the skin), and endocrinopathy and its associated endocrine tumors. The cardiac myxomas may lead to SUDDEN CARDIAC DEATH and other complications in Carney complex patients. The gene coding for the PRKAR1A protein is one of the causative genetic loci (type 1). A second locus is at chromosome 2p16 (type 2).'),
  'D056734': NtO(MH='Monilethrix', MS='Rare autosomal dominant disorder of the hair shaft. The clinical features of the disease include HYPOTRICHOSIS, dry, and/or brittle hair, with varying degrees of ALOPECIA. Mutations in the hair-specific keratin genes KRTHB1, KRTHB3, or KRTHB6 are associated with monilethrix. Autosomal recessive monilethrix with limited HYPOTRICHOSIS are also known. Mutations in Dsg4, Liph, and P2ry5 protein genes are associated with the recessive form of monilethrix.'),
  'D056735': NtO(MH='Autoimmune Lymphoproliferative Syndrome', MS='Rare congenital lymphoid disorder due to mutations in certain Fas-Fas ligand pathway genes. Known causes include mutations in FAS, TNFSF6, NRAS, CASP8, and CASP10 proteins. Clinical features include LYMPHADENOPATHY; SPLENOMEGALY; and AUTOIMMUNITY.'),
  'D056736': NtO(MH='Drug Recalls', MS='Removal of a drug from the market due to a problem occurring in the manufacture or distribution of the product.'),
  'D056737': NtO(MH='Safety-Based Drug Withdrawals', MS='Removal of a drug from the market due to the identification of an intrinsic property of the drug that results in a serious risk to public health.'),
  'D056738': NtO(MH='Cross Protection', MS='Protection conferred on a host by inoculation with one strain or component of a microorganism that prevents infection when later challenged with a similar strain. Most commonly the microorganism is a virus.'),
  'D056739': NtO(MH='Pterygopalatine Fossa', MS='A small space in the skull between the MAXILLA and the SPHENOID BONE, medial to the pterygomaxillary fissure, and connecting to the NASAL CAVITY via the sphenopalatine foramen.'),
  'D056740': NtO(MH='Olecranon Process', MS='A prominent projection of the ulna that articulates with the HUMERUS and forms the outer protuberance of the ELBOW JOINT.'),
  'D056741': NtO(MH='Cyclin D', MS='A cyclin subtype that is specific for CYCLIN-DEPENDENT KINASE 4 and CYCLIN-DEPENDENT KINASE 6. Unlike most cyclins, cyclin D expression is not cyclical, but rather it is expressed in response to proliferative signals. Cyclin D may therefore play a role in cellular responses to mitogenic signals.'),
  'D056742': NtO(MH='Cyclin D2', MS='A cyclin D subtype which is regulated by GATA4 TRANSCRIPTION FACTOR. Experiments using KNOCKOUT MICE suggest a role for cyclin D2 in granulosa cell proliferation and gonadal development.'),
  'D056743': NtO(MH='Cyclin D3', MS='A broadly expressed type D cyclin. Experiments using KNOCKOUT MICE suggest a role for cyclin D3 in LYMPHOCYTE development.'),
  'D056744': NtO(MH='Cyclin B1', MS='A cyclin B subtype that colocalizes with MICROTUBULES during INTERPHASE and is transported into the CELL NUCLEUS at the end of the G2 PHASE.'),
  'D056745': NtO(MH='Cyclin C', MS='A cyclin subtype that binds to the CYCLIN-DEPENDENT KINASE 3 and CYCLIN-DEPENDENT KINASE 8. Cyclin C plays a dual role as a transcriptional regulator and a G1 phase CELL CYCLE regulator.'),
  'D056746': NtO(MH='Cyclin G', MS='A cyclin subtype that is found associated with CYCLIN-DEPENDENT KINASE 5; cyclin G associated kinase, and PROTEIN PHOSPHATASE 2.'),
  'D056747': NtO(MH='Immunomodulation', MS='Alteration of the immune system or of an immune response by agents that activate or suppress its function. This can include IMMUNIZATION or administration of immunomodulatory drugs. Immunomodulation can also encompass non-therapeutic alteration of the immune system effected by endogenous or exogenous substances.'),
  'D056748': NtO(MH='Cyclin H', MS='A cyclin subtype that is found as a component of a heterotrimeric complex containing cyclin-dependent kinase 7 and CDK-activating kinase assembly factor. The complex plays a role in cellular proliferation by phosphorylating several CYCLIN DEPENDENT KINASES at specific regulatory threonine sites.'),
  'D056749': NtO(MH='Cyclin I', MS='A cyclin subtype that is found abundantly in post-mitotic tissues. In contrast to the classical cyclins, its level does not fluctuate during the cell cycle.'),
  'D056750': NtO(MH='Cyclin A1', MS='A cyclin A subtype primarily found in male GERM CELLS. It may play a role in the passage of SPERMATOCYTES into meiosis I.'),
  'D056751': NtO(MH='Cyclin A2', MS='A widely-expressed cyclin A subtype that functions during the G1/S and G2/M transitions of the CELL CYCLE.'),
  'D056765': NtO(MH='Cyclin B2', MS='A cyclin B subtype that colocalizes with GOLGI APPARATUS during INTERPHASE and is transported into the CELL NUCLEUS at the end of the G2 PHASE.'),
  'D056766': NtO(MH='Cyclin G1', MS='A cyclin G subtype that is constitutively expressed throughout the cell cycle. Cyclin G1 is considered a major transcriptional target of TUMOR SUPPRESSOR PROTEIN P53 and is highly induced in response to DNA damage.'),
  'D056767': NtO(MH='Cyclin G2', MS='An unusual cyclin subtype that is found highly expressed in terminally differentiated cells. Unlike conventional cyclins increased expression of cyclin G2 is believed to cause a withdrawal of cells from the CELL CYCLE.'),
  'D056768': NtO(MH='Giant Axonal Neuropathy', MS='Rare autosomal recessive disorder of INTERMEDIATE FILAMENT PROTEINS. The disease is caused by mutations in the gene that codes gigaxonin protein. The mutations result in disorganization of axonal NEUROFILAMENT PROTEINS, formation of the characteristic giant axons, and progressive neuropathy. The clinical features of the disease include early-onset progressive peripheral motor and sensory neuropathies often associated with central nervous system involvement (INTELLECTUAL DISABILITY, seizures, DYSMETRIA, and CONGENITAL NYSTAGMUS).'),
  'D056769': NtO(MH='Alstrom Syndrome', MS='Rare autosomal recessive disease characterized by multiple organ dysfunction. The key clinical features include retinal degeneration (NYSTAGMUS, PATHOLOGIC; RETINITIS PIGMENTOSA; and eventual blindness), childhood obesity, sensorineural hearing loss, and normal mental development. Endocrinologic complications include TYPE 2 DIABETES MELLITUS; HYPERINSULINEMIA; ACANTHOSIS NIGRICANS; HYPOTHYROIDISM; and progressive renal and hepatic failures. The disease is caused by mutations in the ALMS1 gene.'),
  'D056770': NtO(MH='Netherton Syndrome', MS='Rare autosomal recessive disease with variable expressions. Clinical features of the disease include variable ICHTHYOSIFORM ERYTHRODERMA, CONGENITAL; bamboo hair (trichorrhexis invaginata); and ATOPIC DERMATITIS. The disease is caused by mutations in the SPINK5 gene.'),
  'D056771': NtO(MH='Ovule', MS='The element in plants that contains the female GAMETOPHYTES.'),
  'D056784': NtO(MH='Leukoencephalopathies', MS='Any of various diseases affecting the white matter of the central nervous system.'),
  'D056785': NtO(MH='Medicine, Mongolian Traditional', MS='Medical practice indigenous to the peoples of Mongolia, developed over many years according to their culture, beliefs, and traditions.'),
  'D056804': NtO(MH='Biofuels', MS='Hydrocarbon-rich byproducts from the non-fossilized BIOMASS that are combusted to generate energy as opposed to fossilized hydrocarbon deposits (FOSSIL FUELS).'),
  'D056805': NtO(MH='Consciousness Monitors', MS='Devices used to assess the level of consciousness especially during anesthesia. They measure brain activity level based on the EEG.'),
  'D056806': NtO(MH='Urea Cycle Disorders, Inborn', MS='Rare congenital metabolism disorders of the urea cycle. The disorders are due to mutations that result in complete (neonatal onset) or partial (childhood or adult onset) inactivity of an enzyme, involved in the urea cycle. Neonatal onset results in clinical features that include irritability, vomiting, lethargy, seizures, NEONATAL HYPOTONIA; RESPIRATORY ALKALOSIS; HYPERAMMONEMIA; coma, and death. Survivors of the neonatal onset and childhood/adult onset disorders share common risks for ENCEPHALOPATHIES, METABOLIC, INBORN; and RESPIRATORY ALKALOSIS due to HYPERAMMONEMIA.'),
  'D056807': NtO(MH='Argininosuccinic Aciduria', MS='Rare autosomal recessive disorder of the urea cycle which leads to the accumulation of argininosuccinic acid in body fluids and severe HYPERAMMONEMIA. Clinical features of the neonatal onset of the disorder include poor feeding, vomiting, lethargy, seizures, tachypnea, coma, and death. Later onset results in milder set of clinical features including vomiting, failure to thrive, irritability, behavioral problems, or psychomotor retardation. Mutations in the ARGININOSUCCINATE LYASE gene cause the disorder.'),
  'D056808': NtO(MH='Biostatistics', MS='The application of STATISTICS to biological systems and organisms involving the retrieval or collection, analysis, reduction, and interpretation of qualitative and quantitative data.'),
  'D056809': NtO(MH='Alveolar Epithelial Cells', MS='Epithelial cells that line the PULMONARY ALVEOLI.'),
  'D056810': NtO(MH='Acaricides', MS='A pesticide or chemical agent that kills mites and ticks. This is a large class that includes carbamates, formamides, organochlorines, organophosphates, etc, that act as antibiotics or growth regulators.'),
  'D056811': NtO(MH='Unfolded Protein Response', MS='A cellular response to environmental insults that cause disruptions in PROTEIN FOLDING and/or accumulation of defectively folded protein in the ENDOPLASMIC RETICULUM. It consists of a group of regulatory cascades that are triggered as a response to altered levels of calcium and/or the redox state of the endoplasmic reticulum. Persistent activation of the unfolded protein response leads to the induction of APOPTOSIS.'),
  'D056824': NtO(MH='Upper Extremity Deep Vein Thrombosis', MS='DEEP VEIN THROMBOSIS of an upper extremity vein (e.g., AXILLARY VEIN; SUBCLAVIAN VEIN; and JUGULAR VEINS). It is associated with mechanical factors (Upper Extremity Deep Vein Thrombosis, Primary) secondary to other anatomic factors (Upper Extremity Deep Vein Thrombosis, Secondary). Symptoms may include sudden onset of pain, warmth, redness, blueness, and swelling in the arm.'),
  'D056825': NtO(MH='Thyroxine-Binding Globulin', MS='A thyroid hormone transport protein found in serum. It binds about 75% of circulating THYROXINE and 70% of circulating TRIIODOTHYRONINE.'),
  'D056826': NtO(MH='Single Embryo Transfer', MS='The techniques used to select and/or place only one embryo from FERTILIZATION IN VITRO into the uterine cavity to establish a singleton pregnancy.'),
  'D056827': NtO(MH='Endosomal Sorting Complexes Required for Transport', MS='A set of protein subcomplexes involved in PROTEIN SORTING of UBIQUITINATED PROTEINS into intraluminal vesicles of MULTIVESICULAR BODIES and in membrane scission during formation of intraluminal vesicles, during the final step of CYTOKINESIS, and during the budding of enveloped viruses. The ESCRT machinery is comprised of the protein products of Class E vacuolar protein sorting genes.'),
  'D056828': NtO(MH='Hereditary Angioedema Type III', MS='A form of hereditary angioedema that occurs in women and is precipitated or worsened by high ESTROGEN levels. It is associated with mutations in the gene for FACTOR XII that result in its increased activity.'),
  'D056829': NtO(MH='Hereditary Angioedema Types I and II', MS='Forms of hereditary angioedema that occur due to mutations in the gene for COMPLEMENT C1 INHIBITOR PROTEIN. Type I hereditary angioedema is associated with reduced serum levels of complement C1 inhibitor protein. Type II hereditary angioedema is associated with the production of a non-functional complement C1 inhibitor protein.'),
  'D056830': NtO(MH='Isolated Noncompaction of the Ventricular Myocardium', MS='Rare congenital cardiomyopathies characterized by the lack of left ventricular myocardium compaction. The noncompaction results in numerous prominent trabeculations and a loose myocardial meshwork (spongy myocardium) in the LEFT VENTRICLE. Heterogeneous clinical features include diminished systolic function sometimes associated with left ventricular dilation, that presents either neonatally or progressively. Often, the RIGHT VENTRICLE is also affected. CONGESTIVE HEART FAILURE; PULMONARY EMBOLISM; and ventricular ARRHYTHMIA are commonly seen.'),
  'D056831': NtO(MH='Coordination Complexes', MS="Neutral or negatively charged ligands bonded to metal cations or neutral atoms. The number of ligand atoms to which the metal center is directly bonded is the metal cation's coordination number, and this number is always greater than the regular valence or oxidation number of the metal. A coordination complex can be negative, neutral, or positively charged."),
  'D056833': NtO(MH='Central Serous Chorioretinopathy', MS='A visual impairment characterized by the accumulation of fluid under the retina through a defect in the retinal pigment epithelium.'),
  'D056844': NtO(MH='Renal Colic', MS='A severe intermittent and spasmodic pain in the lower back radiating to the groin, scrotum, and labia which is most commonly caused by a kidney stone (RENAL CALCULUS) passing through the URETER or by other urinary track blockage. It is often associated with nausea, vomiting, fever, restlessness, dull pain, frequent urination, and HEMATURIA.'),
  'D056846': NtO(MH='Weill-Marchesani Syndrome', MS='Rare congenital disorder of connective tissue characterized by brachydactyly, joint stiffness, childhood onset of ocular abnormalities (e.g., microspherophakia, ECTOPIA LENTIS; GLAUCOMA), and proportionate short stature. Cardiovascular anomalies are occasionally seen.'),
  'D056849': NtO(MH='Cyclin-Dependent Kinase 3', MS='A cyclin-dependent kinase that forms a complex with CYCLIN C and is active during the G1 PHASE of the CELL CYCLE. It plays a role in the transition from G1 to S PHASE and in transcriptional regulation.'),
  'D056850': NtO(MH='Cyclin-Dependent Kinase 8', MS='A CYCLIN C dependent kinase that is an important component of the mediator complex. The enzyme is activated by its interaction with CYCLIN C and plays a role in transcriptional regulation by phosphorylating RNA POLYMERASE II.'),
  'D056864': NtO(MH='Behind-the-Counter Drugs', MS='Medicines that are stocked behind the counter of a pharmacy which require the attention or counsel of a pharmacist before being dispensed or handed over to a consumer or patient.'),
  'D056865': NtO(MH='Ideal Body Weight', MS='Expected weight of a healthy normal individual based on age, sex, and height. Thus, a malnourished person would weigh less than their ideal body weight.'),
  'D056885': NtO(MH='Interstitial Cells of Cajal', MS='c-Kit positive cells related to SMOOTH MUSCLE CELLS that are intercalated between the autonomic nerves and the effector smooth muscle cells of the GASTROINTESTINAL TRACT. Different phenotypic classes play roles as pacemakers, mediators of neural inputs, and mechanosensors.'),
  'D056886': NtO(MH='Contrecoup Injury', MS='An injury in which the damage is located on the opposite side of the primary impact site. A blow to the back of head which results in contrecoup injury to the frontal lobes of the brain is the most common type.'),
  'D056887': NtO(MH='Pelvic Organ Prolapse', MS='Abnormal descent of a pelvic organ resulting in the protrusion of the organ beyond its normal anatomical confines. Symptoms often include vaginal discomfort, DYSPAREUNIA; URINARY STRESS INCONTINENCE; and FECAL INCONTINENCE.'),
  'D056888': NtO(MH='Patient Positioning', MS='Moving a patient into a specific position or POSTURE to facilitate examination, surgery, or for therapeutic purposes.'),
  'D056889': NtO(MH='Barth Syndrome', MS='Rare congenital X-linked disorder of lipid metabolism. Barth syndrome is transmitted in an X-linked recessive pattern. The syndrome is characterized by muscular weakness, growth retardation, DILATED CARDIOMYOPATHY, variable NEUTROPENIA, 3-methylglutaconic aciduria (type II) and decreases in mitochondrial CARDIOLIPIN level. Other biochemical and morphological mitochondrial abnormalities also exist.'),
  'D056890': NtO(MH='Eukaryota', MS='One of the three domains of life (the others being BACTERIA and ARCHAEA), also called Eukarya. These are organisms whose cells are enclosed in membranes and possess a nucleus. They comprise almost all multicellular and many unicellular organisms, and are traditionally divided into groups (sometimes called kingdoms) including ANIMALS; PLANTS; FUNGI; and various algae and other taxa that were previously part of the old kingdom Protista.'),
  'D056891': NtO(MH='Organism Forms', MS='Specific states or life cycle stages of organisms. These include post-embryonic stages of insects, symbiotic relationships between organisms, and reproductive elements such as spores.'),
  'D056892': NtO(MH='Mediator Complex', MS='A large protein complex which acts as a signaling adaptor protein that allows communication between the various regulatory and functional components of GENETIC TRANSCRIPTION including DNA POLYMERASE II; GENERAL TRANSCRIPTION FACTORS; and TRANSCRIPTION FACTORS that are bound to upstream ENHANCER ELEMENTS. The mediator complex was originally studied in YEAST where at least 21 subunits were identified. Many of the yeast subunits are homologs to proteins in higher organisms that are found associated with specific nuclear receptors such as THYROID HORMONE RECEPTORS and VITAMIN D RECEPTORS.'),
  'D056893': NtO(MH='Alveolata', MS='A group of three related eukaryotic phyla whose members possess an alveolar membrane system, consisting of flattened membrane-bound sacs lying beneath the outer cell membrane.'),
  'D056894': NtO(MH='Amoebozoa', MS='A supergroup (some say phylum) of ameboid EUKARYOTES, comprising ARCHAMOEBAE; LOBOSEA; and MYCETOZOA.'),
  'D056895': NtO(MH='Mycetozoa', MS='A group of AMOEBOZOA comprising two distinct groups with the ability to form fruiting bodies: DICTYOSTELIIDA (dictyostelid cellular slime molds) and MYXOMYCETES (plasmodial slime molds).'),
  'D056896': NtO(MH='Archamoebae', MS='A group (some say infraphylum) of AMOEBOZOA that lack mitochondria.'),
  'D056897': NtO(MH='Choanoflagellata', MS='An order of stalked, sessile, single-celled EUKARYOTES. They are considered the transitional link between the flagellated protozoa and the SPONGES, the most primitive metazoans.'),
  'D056898': NtO(MH='Euglenozoa', MS='A large group of flagellated EUKARYOTES found in both free-living and parasitic forms. The flagella are present in pairs and contain unique paraxonemal rods.'),
  'D056899': NtO(MH='Oxymonadida', MS='An order of EUKARYOTES found in the gut of termites and other insects that live on LIGNIN. MITOCHONDRIA and GOLGI APPARATUS are absent from oxymonads.'),
  'D056900': NtO(MH='Parabasalidea', MS='A group (or phylum) of flagellated, anaerobic EUKARYOTES that are endosymbionts of animals. They lack mitochondria but contain small energy-producing hydrogenosomes. The group is comprised of two major classes: HYPERMASTIGIA and TRICHOMONADIDA.'),
  'D056901': NtO(MH='Rhizaria', MS='A large supergroup of mostly amoeboid EUKARYOTES whose three main subgroups are CERCOZOA; FORAMINIFERA; and HAPLOSPORIDA. Nearly all of the species possess MITOCHONDRIA and historically many were considered ANIMALS.'),
  'D056902': NtO(MH='Clinical Alarms', MS='Components of medical instrumentation used for physiological evaluation of patients, that signal when a threshold value is reached.'),
  'D056905': NtO(MH='Chromosome Breakpoints', MS='The locations in specific DNA sequences where CHROMOSOME BREAKS have occurred.'),
  'D056906': NtO(MH='Cyclin T', MS='A cyclin subtype that is found associated with CYCLIN-DEPENDENT KINASE 9. Unlike traditional cyclins, which regulate the CELL CYCLE, type T cyclins appear to regulate transcription and are components of positive transcriptional elongation factor B.'),
  'D056907': NtO(MH='Cercozoa', MS='A group of amoeboid and flagellate EUKARYOTES in the supergroup RHIZARIA. They feed by means of threadlike pseudopods.'),
  'D056908': NtO(MH='Foraminifera', MS='An order of amoeboid EUKARYOTES characterized by reticulating pseudopods and a complex life cycle with an alternation of generations. Most are less than 1mm in size and found in marine or brackish water.'),
  'D056909': NtO(MH='Plasmodiophorida', MS='A group of EUKARYOTES that are parasites of plants. Life cycle stages include zoospores and plasmodia.'),
  'D056910': NtO(MH='Republic of Korea', MS="The capital is Seoul. The country, established September 9, 1948, is located on the southern part of the Korean Peninsula. Its northern border is shared with the Democratic People's Republic of Korea."),
  'D056911': NtO(MH="Democratic People's Republic of Korea", MS='A country located on the Korean Peninsula whose capital is Pyongyang. The country was established September 9, 1948.'),
  'D056912': NtO(MH='Binge-Eating Disorder', MS='A disorder associated with three or more of the following: eating until feeling uncomfortably full; eating large amounts of food when not physically hungry; eating much more rapidly than normal; eating alone due to embarrassment; feeling of disgust, DEPRESSION, or guilt after overeating. Criteria includes occurrence on average, at least 2 days a week for 6 months. The binge eating is not associated with the regular use of inappropriate compensatory behavior (i.e. purging, excessive exercise, etc.) and does not co-occur exclusively with BULIMIA NERVOSA or ANOREXIA NERVOSA. (From DSM-IV, 1994)'),
  'D056913': NtO(MH='Mediator Complex Subunit 1', MS='A mediator complex subunit that is believed to play a key role in the coactivation of nuclear receptor-activated transcription by the mediator complex. It interacts with a variety of nuclear receptors including RETINOIC ACID RECEPTORS; THYROID HORMONE RECEPTORS; VITAMIN D RECEPTORS; PEROXISOME PROLIFERATOR-ACTIVATED RECEPTORS; ESTROGEN RECEPTORS; and GLUCOCORTICOID RECEPTORS.'),
  'D056914': NtO(MH='Genomic Structural Variation', MS='Contiguous large-scale (1000-400,000 basepairs) differences in the genomic DNA between individuals, due to SEQUENCE DELETION; SEQUENCE INSERTION; or SEQUENCE INVERSION.'),
  'D056915': NtO(MH='DNA Copy Number Variations', MS='Stretches of genomic DNA that exist in different multiples between individuals. Many copy number variations have been associated with susceptibility or resistance to disease.'),
  'D056916': NtO(MH='Segmental Duplications, Genomic', MS='Low-copy (2-50) repetitive DNA elements that are highly homologous and range in size from 1000 to 400,000 base pairs.'),
  'D056918': NtO(MH='Nuclear Receptor Coactivators', MS='Proteins that enhance gene expression when associated with ligand bound activated NUCLEAR RECEPTORS. The coactivators may act through an enzymatic process that affects the rate of transcription or the structure of chromatin. Alternatively nuclear receptor coactivators can function as adaptor proteins that bring nuclear receptors into close proximity with transcriptional complexes.'),
  'D056919': NtO(MH='Retortamonadidae', MS='A family of flagellated EUKARYOTES that live in the intestines of several invertebrate and vertebrate species.'),
  'D056920': NtO(MH='Nuclear Receptor Coactivator 1', MS='A nuclear receptor coactivator with specificity for ESTROGEN RECEPTORS; PROGESTERONE RECEPTORS; and THYROID HORMONE RECEPTORS. It contains a HISTONE ACETYLTRANSFRASE activity that may play a role in the transcriptional activation of CHROMATIN regions.'),
  'D056921': NtO(MH='Nuclear Receptor Coactivator 3', MS='A nuclear receptor coactivator with specificity for ESTROGEN RECEPTORS and PROGESTERONE RECEPTORS. It contains a histone acetyltransferase activity that may play a role in CHROMATIN REMODELING during the process of nuclear receptor-induced transcription. The coactivator has been found at elevated levels in certain HORMONE-DEPENDENT NEOPLASMS such as those found in BREAST CANCER.'),
  'D056925': NtO(MH='Circadian Rhythm Signaling Peptides and Proteins', MS='A broad category of proteins that regulate the CIRCADIAN RHYTHM of an organism. Included here are proteins that transmit intracellular and intercellular signals in a chronological manner along with proteins that sense light and time-dependent changes in the environment such as the PHOTOPERIOD.'),
  'D056926': NtO(MH='CLOCK Proteins', MS='Basic helix-loop-helix (bHLH) domain-containing proteins that contain intrinsic HISTONE ACETYLTRANSFERASE activity and play important roles in CIRCADIAN RHYTHM regulation. Clock proteins combine with Arntl proteins to form heterodimeric transcription factors that are specific for E-BOX ELEMENTS and stimulate the transcription of several E-box genes that are involved in cyclical regulation. This transcriptional activation also sets into motion a time-dependent feedback loop which in turn down-regulates the expression of clock proteins.'),
  'D056928': NtO(MH='X-Ray Absorption Spectroscopy', MS='Analysis of the energy absorbed across a spectrum of x-ray energies/wavelengths to determine the chemical structure and electronic states of the absorbing medium.'),
  'D056929': NtO(MH='Liddle Syndrome', MS='Familial pseudoaldosteronism characterized by autosomal dominant inheritance of hypertension with HYPOKALEMIA; ALKALOSIS; RENIN and ALDOSTERONE level decreases. It is caused by mutations in EPITHELIAL SODIUM CHANNELS beta and gamma subunits. Different mutations in the same EPITHELIAL SODIUM CHANNELS subunits can cause PSEUDOHYPOALDOSTERONISM, TYPE I, AUTOSOMAL DOMINANT.'),
  'D056930': NtO(MH='ARNTL Transcription Factors', MS='Basic helix-loop-helix (bHLH) domain-containing proteins that play important roles in CIRCADIAN RHYTHM regulation. They combine with CLOCK PROTEINS to form heterodimeric transcription factors that are specific for E-BOX ELEMENTS and stimulate the transcription of several E-box genes that are involved in cyclical regulation.'),
  'D056931': NtO(MH='Cryptochromes', MS='Flavoproteins that function as circadian rhythm signaling proteins in ANIMALS and as blue-light photoreceptors in PLANTS. They are structurally-related to DNA PHOTOLYASES and it is believed that both classes of proteins may have originated from an earlier protein that played a role in protecting primitive organisms from the cyclical exposure to UV LIGHT.'),
  'D056945': NtO(MH='Hep G2 Cells', MS='A human liver tumor cell line used to study a variety of liver-specific metabolic functions.'),
  'D056947': NtO(MH='Enzyme Replacement Therapy', MS='Therapeutic replacement or supplementation of defective or missing enzymes to alleviate the effects of enzyme deficiency (e.g., GLUCOSYLCERAMIDASE replacement for GAUCHER DISEASE).'),
  'D056948': NtO(MH='Orthognathic Surgical Procedures', MS='Surgery performed to repair or correct the skeletal anomalies of the jaw and its associated dental and facial structures (e.g. CLEFT PALATE).'),
  'D056949': NtO(MH='Phototropins', MS='Blue-light receptors that regulate a range of physiological responses in PLANTS. Examples include: PHOTOTROPISM, light-induced stomatal opening, and CHLOROPLAST movements in response to changes in light intensity.'),
  'D056950': NtO(MH='Period Circadian Proteins', MS='Circadian rhythm signaling proteins that influence circadian clock by interacting with other circadian regulatory proteins and transporting them into the CELL NUCLEUS.'),
  'D056951': NtO(MH='Photoelectron Spectroscopy', MS='The study of the energy of electrons ejected from matter by the photoelectric effect, i.e., as a direct result of absorption of energy from electromagnetic radiation. As the energies of the electrons are characteristic of a specific element, the measurement of the energy of these electrons is a technique used to determine the chemical composition of surfaces.'),
  'D056952': NtO(MH='Hypermastigia', MS='A class of endosymbiont EUKARYOTES, in the group PARABASALIDEA, that are obligate mutualists in the digestive tracts of wood-eating insects. Hypermastigotes produce CELLULASE that breaks down the cellulose in wood so that insects can metabolize it.'),
  'D056953': NtO(MH='Nuclear Receptor Subfamily 1, Group D, Member 1', MS='A DNA-binding orphan nuclear receptor that negatively regulates expression of ARNTL TRANSCRIPTION FACTORS and plays a role as a regulatory component of the circadian clock system. The Nr1d1 nuclear receptor expression is cyclically-regulated by a feedback loop involving its positive regulation by CLOCK PROTEIN; BMAL1 PROTEIN heterodimers and its negative regulation by CRYPTOCHROME and PERIOD PROTEINS.'),
  'D056965': NtO(MH='Injections, Intraocular', MS='The administration of substances into the eye with a hypodermic syringe.'),
  'D056966': NtO(MH='Excitation Contraction Coupling', MS='A process fundamental to muscle physiology whereby an electrical stimulus or action potential triggers a myocyte to depolarize and contract. This mechanical muscle contraction response is regulated by entry of calcium ions into the cell.'),
  'D056968': NtO(MH='Lac Repressors', MS='Bacterial repressor proteins that bind to the LAC OPERON and thereby prevent the synthesis of proteins involved in catabolism of LACTOSE. When lactose levels are high lac repressors undergo an allosteric change that causes their release from the DNA and the resumption of lac operon transcription.'),
  'D056969': NtO(MH='Voltage-Sensitive Dye Imaging', MS='Optical imaging techniques used for recording patterns of electrical activity in tissues by monitoring transmembrane potentials via FLUORESCENCE imaging with voltage-sensitive fluorescent dyes.'),
  'D056970': NtO(MH='Co-Repressor Proteins', MS='A subclass of repressor proteins that do not directly bind DNA. Instead, co-repressors generally act via their interaction with DNA-BINDING PROTEINS such as a TRANSCRIPTIONAL SILENCING FACTORS or NUCLEAR RECEPTORS.'),
  'D056971': NtO(MH='Nuclear Receptor Co-Repressor 1', MS='A nuclear protein that regulates the expression of genes involved in a diverse array of processes related to metabolism and reproduction. The protein contains three nuclear receptor interaction domains and three repressor domains and is closely-related in structure to NUCLEAR RECEPTOR CO-REPRESSOR 2.'),
  'D056972': NtO(MH='Optical Restriction Mapping', MS='A technique to generate restriction maps from single large DNA molecules by spreading the DNA onto a glass surface, digesting with DNA RESTRICTION ENZYMES, staining with FLUORESCENT DYES, and visualizing the DNA cleavage sites by FLUORESCENCE MICROSCOPY.'),
  'D056973': NtO(MH='Four-Dimensional Computed Tomography', MS='Three-dimensional computed tomographic imaging with the added dimension of time, to follow motion during imaging.'),
  'D056985': NtO(MH='Nuclear Receptor Co-Repressor 2', MS='A nuclear co-repressor protein that shows specificity for RETINOIC ACID RECEPTORS and THYROID HORMONE RECEPTORS. The dissociation of this co-repressor from nuclear receptors is generally ligand-dependent, but can also occur by way of its phosphorylation by members of the MAP KINASE SIGNALING SYSTEM. The protein contains two nuclear receptor interaction domains and four repressor domains and is closely-related in structure to NUCLEAR RECEPTOR CO-REPRESSOR 1.'),
  'D056986': NtO(MH='Euglenozoa Infections', MS='Infections with the protozoa of the phylum EUGLENOZOA.'),
  'D056987': NtO(MH='Vasoplegia', MS='Condition of low SYSTEMIC VASCULAR RESISTANCE that develops secondary to other conditions such as ANAPHYLAXIS; SEPSIS; SURGICAL SHOCK; and SEPTIC SHOCK. Vasoplegia that develops during or post surgery (e.g., CARDIOPULMONARY BYPASS) is called postoperative vasoplegic syndrome or vasoplegic syndrome.'),
  'D056988': NtO(MH='Anterior Wall Myocardial Infarction', MS='MYOCARDIAL INFARCTION in which the anterior wall of the heart is involved. Anterior wall myocardial infarction is often caused by occlusion of the left anterior descending coronary artery. It can be categorized as anteroseptal or anterolateral wall myocardial infarction.'),
  'D056989': NtO(MH='Inferior Wall Myocardial Infarction', MS='MYOCARDIAL INFARCTION in which the inferior wall of the heart is involved. It is often caused by occlusion of the right coronary artery.'),
  'D056990': NtO(MH='Post-Exposure Prophylaxis', MS='The prevention of infection or disease following exposure to a pathogen.'),
  'D057005': NtO(MH='Bioengineering', MS='The application of engineering principles and methods to living organisms or biological systems.'),
  'D057026': NtO(MH='Induced Pluripotent Stem Cells', MS='Cells from adult organisms that have been reprogrammed into a pluripotential state similar to that of EMBRYONIC STEM CELLS.'),
  'D057045': NtO(MH='Laryngopharyngeal Reflux', MS='Back flow of gastric contents to the LARYNGOPHARYNX where it comes in contact with tissues of the upper aerodigestive tract. Laryngopharyngeal reflux is an extraesophageal manifestation of GASTROESOPHAGEAL REFLUX.'),
  'D057048': NtO(MH='Plant Nectar', MS='Sugar-rich liquid produced in plant glands called nectaries. It is either produced in flowers or other plant structures, providing a source of attraction for pollinating insects and animals, as well as being a nutrient source to animal mutualists which provide protection of plants against herbivores.'),
  'D057049': NtO(MH='Thrombotic Microangiopathies', MS='Diseases that result in THROMBOSIS in MICROVASCULATURE. The two most prominent diseases are PURPURA, THROMBOTIC THROMBOCYTOPENIC; and HEMOLYTIC-UREMIC SYNDROME. Multiple etiological factors include VASCULAR ENDOTHELIAL CELL damage due to SHIGA TOXIN; FACTOR H deficiency; and aberrant VON WILLEBRAND FACTOR formation.'),
  'D057050': NtO(MH='Receptor Tyrosine Kinase-like Orphan Receptors', MS='A family of cell surface receptors that were originally identified by their structural homology to neurotropic TYROSINE KINASES and referred to as orphan receptors because the associated ligand and signaling pathways were unknown. Evidence for the functionality of these proteins has been established by experiments showing that disruption of the orphan receptor genes results in developmental defects.'),
  'D057051': NtO(MH='Product Recalls and Withdrawals', MS="The removal of a consumer product from the market place. The reason for the removal can be due a variety of causes, including the discovery of a manufacturing defect, a safety issue with the product's use, or marketing decisions."),
  'D057052': NtO(MH='Medical Device Recalls', MS='Removal of a MEDICAL DEVICE from the market due to a problem occurring in the manufacture or distribution of the product.'),
  'D057053': NtO(MH='Safety-Based Medical Device Withdrawals', MS='Removal of a MEDICAL DEVICE from the market due to the identification of an intrinsic property of the device that results in a serious risk to public health.'),
  'D057054': NtO(MH='Molecular Imaging', MS='The use of molecularly targeted imaging probes to localize and/or monitor biochemical and cellular processes via various imaging modalities that include RADIONUCLIDE IMAGING; ULTRASONOGRAPHY; MAGNETIC RESONANCE IMAGING; FLUORESCENCE IMAGING; and MICROSCOPY.'),
  'D057055': NtO(MH='Aspartic Acid Proteases', MS='A subclass of peptide hydrolases that depend on an ASPARTIC ACID residue for their activity.'),
  'D057056': NtO(MH='Cysteine Proteases', MS='A subclass of peptide hydrolases that depend on a CYSTEINE residue for their activity.'),
  'D057057': NtO(MH='Serine Proteases', MS='Peptide hydrolases that contain at the active site a SERINE residue involved in catalysis.'),
  'D057065': NtO(MH='ATP-Dependent Endopeptidases', MS='Endoproteases that contain proteolytic core domains and ATPase-containing regulatory domains.'),
  'D057066': NtO(MH='Low Tension Glaucoma', MS='A form of glaucoma in which chronic optic nerve damage and loss of vision normally attributable to buildup of intraocular pressure occurs despite prevailing conditions of normal intraocular pressure.'),
  'D057068': NtO(MH='Periprosthetic Fractures', MS='Fractures around joint replacement prosthetics or implants. They can occur intraoperatively or postoperatively.'),
  'D057069': NtO(MH='Diagnostic Test Approval', MS='The process of gaining approval by a government regulatory agency for DIAGNOSTIC REAGENTS AND TEST KITS. This includes any required preclinical or clinical testing, review, submission, and evaluation of the applications and test results, and post-marketing surveillance.'),
  'D057070': NtO(MH='Cervical Rib', MS='A supernumerary rib developing from an abnormal enlargement of the costal element of the C7 vertebra. This anomaly is found in 1-2% of the population and can put pressure on adjacent structures causing CERVICAL RIB SYNDROME; THORACIC OUTLET SYNDROME; or other conditions.'),
  'D057071': NtO(MH='Patellofemoral Joint', MS='The articulation between the articular surface of the PATELLA and the patellar surface of the FEMUR.'),
  'D057072': NtO(MH='Intra-Articular Fractures', MS='Fractures of the articular surface of a bone.'),
  'D057073': NtO(MH='Thyrotropin Alfa', MS='A highly purified recombinant glycoprotein form of human THYROID-STIMULATING HORMONE, produced by recombinant DNA technology comprising two non-covalently linked subunits, an alpha subunit of 92 amino acid residues containing two N-linked glycosylation sites, and a beta subunit of 118 residues containing one N-linked glycosylation site. The amino acid sequence of thyrotropin alfa is identical to that of human pituitary thyroid stimulating hormone.'),
  'D057074': NtO(MH='Virus Release', MS='Release of a virus from the host cell following VIRUS ASSEMBLY and maturation. Egress can occur by host cell lysis, EXOCYTOSIS, or budding through the plasma membrane.'),
  'D057075': NtO(MH='Enzyme Assays', MS='Methods used to measure the relative activity of a specific enzyme or its concentration in solution. Typically an enzyme substrate is added to a buffer solution containing enzyme and the rate of conversion of substrate to product is measured under controlled conditions. Many classical enzymatic assay methods involve the use of synthetic colorimetric substrates and measuring the reaction rates using a spectrophotometer.'),
  'D057077': NtO(MH='Varroidae', MS='A family of MITES in the subclass ACARI. It includes the single genus Varroa.'),
  'D057078': NtO(MH='Pentastomida', MS='A subclass of CRUSTACEA comprising the tongue worms which are obligatory parasites of reptiles, birds, and mammals including humans.'),
  'D057079': NtO(MH='Pongo', MS='A genus of orangutans in the family HOMINIDAE, comprising two species. Among the PRIMATES, the orangutan is second in size only to the gorilla (GORILLA GORILLA).'),
  'D057080': NtO(MH='Pongo abelii', MS='A species of orangutan, family HOMINIDAE, found in the forests on the island of Sumatra.'),
  'D057085': NtO(MH='Tooth Wear', MS='Loss of the tooth substance by chemical or mechanical processes'),
  'D057086': NtO(MH='High-Intensity Focused Ultrasound Ablation', MS='The use of focused, high-frequency sound waves to destroy tissue. It is sometimes used in conjunction with but is distinct from INTERVENTIONAL ULTRASONOGRAPHY.'),
  'D057087': NtO(MH='Dental Atraumatic Restorative Treatment', MS='Treatment modality for DENTAL CARIES that uses manual excavation method and GLASS IONOMER CEMENTS. Because of its noninvasiveness and no need for expensive equipment and anesthesia it is promoted as an approach in places where dental care is not readily available.'),
  'D057088': NtO(MH='Anetoderma', MS='Benign DERMATOSIS caused by a loss of dermal ELASTIC TISSUE resulting in localized sac-like areas of flaccid skin. It can be either primary (idiopathic) or secondary to other skin conditions, PENICILLAMINE use, or premature birth.'),
  'D057089': NtO(MH='Pathology, Molecular', MS='A subspecialty of pathology concerned with the molecular basis (e.g., mutations) of various diseases.'),
  'D057090': NtO(MH='Eccrine Porocarcinoma', MS='A rare malignant neoplasm of the sweat glands. It most often develops as a form of degenerative progression from a benign ECCRINE POROMA.'),
  'D057091': NtO(MH='Poroma', MS='A benign adnexal neoplasm derived from cells of the terminal duct of eccrine or apocrine SWEAT GLAND lineage. They typically manifest as solitary papules and occur only in the skin but unlike in ACROSPIROMA involves the epidermis.'),
  'D057092': NtO(MH='Geographic Atrophy', MS='A form of MACULAR DEGENERATION also known as dry macular degeneration marked by occurrence of a well-defined progressive lesion or atrophy in the central part of the RETINA called the MACULA LUTEA. It is distinguishable from WET MACULAR DEGENERATION in that the latter involves neovascular exudates.'),
  'D057093': NtO(MH='Orphan Nuclear Receptors', MS='A broad category of receptor-like proteins that may play a role in transcriptional-regulation in the CELL NUCLEUS. Many of these proteins are similar in structure to known NUCLEAR RECEPTORS but appear to lack a functional ligand-binding domain, while in other cases the specific ligands have yet to be identified.'),
  'D057094': NtO(MH='Nuclear Receptor Subfamily 1, Group F, Member 1', MS='A DNA-binding orphan nuclear receptor that positively regulates expression of ARNTL TRANSCRIPTION FACTORS and is a regulatory component of the circadian clock system. The protein also has a role in neuron cell survival and differentiation in that loss of function mutations of its gene result in the mouse phenotype referred to as the STAGGERER MOUSE.'),
  'D057095': NtO(MH='Nuclear Receptor Subfamily 1, Group F, Member 2', MS='An orphan nuclear receptor that is expressed at high levels in neuronal tissues, the RETINA; EPIDIDYMIS; and VAS DEFERENS. The receptor is believed to play a role in regulating a variety of functions including the processing of sensory information, the differentiation of PHOTORECEPTOR CELLS and the CIRCADIAN RHYTHM.'),
  'D057096': NtO(MH='Shellfish Poisoning', MS='Poisoning from toxins present in bivalve mollusks that have been ingested. Four distinct types of shellfish poisoning are recognized based on the toxin involved.'),
  'D057097': NtO(MH='Harmful Algal Bloom', MS='An algal bloom where the algae produce powerful toxins that can kill fish, birds, and mammals, and ultimately cause illness in humans. The harmful bloom can also cause oxygen depletion in the water due to the death and decomposition of non-toxic algae species.'),
  'D057105': NtO(MH='Nuclear Receptor Subfamily 4, Group A, Member 1', MS='An orphan nuclear receptor that is closely related to members of the thyroid-steroid receptor gene family. It was originally identified in NERVE CELLS and may play a role in mediation of NERVE GROWTH FACTOR-induced CELL DIFFERENTIATION. However, several other functions have been attributed to this protein including the positive and negative regulation of APOPTOSIS.'),
  'D057106': NtO(MH='Nuclear Receptor Subfamily 4, Group A, Member 3', MS='An orphan nuclear receptor that is closely related to members of the thyroid-steroid receptor family. It was originally identified in NERVE CELLS, however it may play regulatory roles in a variety of other tissues.'),
  'D057107': NtO(MH='Nuclear Receptor Subfamily 2, Group C, Member 1', MS='A DNA-binding orphan nuclear receptor that has specificity for directly repeated (DR) AGGTCA sequences. It binds DNA as either as a homodimer or as a heterodimer with the closely-related orphan nuclear receptor NUCLEAR RECEPTOR SUBFAMILY 2, GROUP C, MEMBER 2. The protein was originally identified as a PROSTATE-specific protein and is involved in the regulation of variety of cellular processes, including CELL DIFFERENTIATION; CELL PROLIFERATION; and APOPTOSIS.'),
  'D057108': NtO(MH='Corneal Wavefront Aberration', MS='Asymmetries in the topography and refractive index of the corneal surface that affect visual acuity.'),
  'D057109': NtO(MH='Scanning Laser Polarimetry', MS='A technique of diagnostic imaging of RETINA or CORNEA of the human eye involving the measurement and interpretation of polarizing ELECTROMAGNETIC WAVES such as radio or light waves. It is helpful in the diagnosis of GLAUCOMA; MACULAR DEGENERATION; and other retinal disorders.'),
  'D057111': NtO(MH='Descemet Stripping Endothelial Keratoplasty', MS='A surgical procedure or KERATOPLASTY involving selective stripping and replacement of diseased host DESCEMET MEMBRANE and CORNEAL ENDOTHELIUM with a suitable and healthy donor posterior lamella. The advantage to this procedure is that the normal corneal surface of the recipient is retained, thereby avoiding corneal surface incisions and sutures.'),
  'D057112': NtO(MH='Corneal Perforation', MS='A puncture or hole through the CORNEAL STROMA resulting from various diseases or trauma.'),
  'D057125': NtO(MH='Nuclear Receptor Subfamily 2, Group C, Member 2', MS='An orphan nuclear receptor that has specificity for hormone response elements found in the promoters of target genes. It binds DNA either as a homodimer or as heterodimer with the closely-related orphan nuclear receptor NUCLEAR RECEPTOR SUBFAMILY 2, GROUP C, MEMBER 1. The protein was originally identified as a TESTES-specific protein and is involved in the regulation of variety of cellular processes, including CELL DIFFERENTIATION; CELL PROLIFERATION; and APOPTOSIS.'),
  'D057126': NtO(MH='Nuclear Receptor Subfamily 4, Group A, Member 2', MS='An orphan nuclear receptor that is found at high levels in BRAIN tissue. The protein is believed to play a role in development and maintenance of NEURONS, particularly dopaminergic neurons.'),
  'D057127': NtO(MH='Single-Chain Antibodies', MS='A form of antibodies consisting only of the variable regions of the heavy and light chains (FV FRAGMENTS), connected by a small linker peptide. They are less immunogenic than complete immunoglobulin and thus have potential therapeutic use.'),
  'D057128': NtO(MH='Aberrometry', MS='The use of an aberrometer to measure eye tissue imperfections or abnormalities based on the way light passes through the eye which affects the ability of the eye to focus properly.'),
  'D057129': NtO(MH='Iridocorneal Endothelial Syndrome', MS='A grouping of three closely linked conditions: iris nevus (or Cogan-Reese) syndrome, Chandler Syndrome, and essential (progressive) iris atrophy. The most common features of this syndrome are the movement of endothelial cells off the cornea onto the iris leading to corneal swelling, distortion of the iris, and variable degrees of distortion of the pupil. The abnormal cell movement plugs fluid outflow channels of the eye causing GLAUCOMA.'),
  'D057130': NtO(MH='Leber Congenital Amaurosis', MS='A rare degenerative inherited eye disease that appears at birth or in the first few months of life that results in a loss of vision. Not to be confused with LEBER HEREDITARY OPTIC NEUROPATHY, the disease is thought to be caused by abnormal development of PHOTORECEPTOR CELLS in the RETINA, or by the extremely premature degeneration of retinal cells.'),
  'D057131': NtO(MH='Immune Evasion', MS="Methods used by pathogenic organisms to evade a host's immune system."),
  'D057132': NtO(MH='Nuclear Receptor Subfamily 1, Group F, Member 3', MS='An orphan nuclear receptor found in the THYMUS where it plays a role in regulating the development and maturation of thymocytes. An isoform of this protein, referred to as RORgammaT, is produced by an alternatively transcribed mRNA.'),
  'D057133': NtO(MH='Dentin Desensitizing Agents', MS='Substances which reduce or eliminate dentinal sensitivity or the pain associated with a source of stimulus (such as touch, heat, or cold) at the orifice of exposed dentinal tubules causing the movement of tubular fluid that in turn stimulates tooth nerve receptors.'),
  'D057134': NtO(MH='Antibodies, Neutralizing', MS='Antibodies that reduce or abolish some biological activity of a soluble antigen or infectious agent, usually a virus.'),
  'D057135': NtO(MH='Wet Macular Degeneration', MS='A form of RETINAL DEGENERATION in which abnormal CHOROIDAL NEOVASCULARIZATION occurs under the RETINA and MACULA LUTEA, causing bleeding and leaking of fluid. This leads to bulging and or lifting of the macula and the distortion or destruction of central vision.'),
  'D057136': NtO(MH='Nuclear Receptor Subfamily 6, Group A, Member 1', MS='An orphan nuclear receptor expressed mainly in the GERM CELLS of GONADS. It functions as a transcription factor that binds to a direct repeat of the sequence AGGTCA and may play a role in the regulation of EMBRYOGENESIS and germ cell differentiation.'),
  'D057137': NtO(MH='DAX-1 Orphan Nuclear Receptor', MS='An orphan nuclear receptor that is implicated in regulation of steroidogenic pathways. It is unlike most orphan nuclear receptors in that it appears to lack an essential DNA-binding domain and instead acts as a transcriptional co-repressor. Mutations in the gene Dax-1 cause congenital adrenal hypoplasia.'),
  'D057138': NtO(MH='Bacterial Shedding', MS='The expelling of bacteria from the body. Important routes include the respiratory tract, genital tract, and intestinal tract.'),
  'D057139': NtO(MH='Nanofibers', MS='Submicron-sized fibers with diameters typically between 50 and 500 nanometers. The very small dimension of these fibers can generate a high surface area to volume ratio, which makes them potential candidates for various biomedical and other applications.'),
  'D057140': NtO(MH='Fast Foods', MS='Prepared food that is ready to eat or partially prepared food that has a final preparation time of a few minutes or less.'),
  'D057141': NtO(MH='Food, Preserved', MS='Food that has been prepared and stored in a way to prevent spoilage.'),
  'D057145': NtO(MH='Nanoshells', MS='Nanometer range spherical cores of particular semiconductor compounds surrounded by an ultrathin metal shell that is commonly made of gold or silver. This configuration gives the nanoshells highly tunable optical properties. They have potential in biomedicine for diagnosis and therapy.'),
  'D057146': NtO(MH='Multivesicular Bodies', MS='Endosomes containing intraluminal vesicles which are formed by the inward budding of the endosome membrane. Multivesicular bodies (MVBs) may fuse with other organelles such as LYSOSOMES or fuse back with the PLASMA MEMBRANE releasing their contents by EXOCYTOSIS. The MVB intraluminal vesicles released into the extracellular environment are known as EXOSOMES.'),
  'D057147': NtO(MH='Mosquito Nets', MS='Free-standing or supported lightweight meshwork fabric made of cotton, silk, polyester or other material, having openings too small to allow entry of mosquitoes or other insects, thereby protecting against INSECT BITES; INSECT STINGS, and insect-borne diseases.'),
  'D057148': NtO(MH='Insecticide-Treated Bednets', MS='Lightweight meshwork fabric made of cotton, silk, polyester, nylon (polyamides), or other material impregnated with insecticide, having openings too small to allow entry of mosquitoes or other insects, thereby offering protection against insect bite and insect-borne diseases.'),
  'D057149': NtO(MH='Ubiquitinated Proteins', MS='Proteins covalently modified with UBIQUITINS or UBIQUITIN-LIKE PROTEINS.'),
  'D057165': NtO(MH='Proteostasis Deficiencies', MS='Disorders caused by imbalances in the PROTEIN HOMEOSTASIS network - synthesis, folding, and transport of proteins; post-translational modifications; and degradation or clearance of misfolded proteins.'),
  'D057166': NtO(MH='High-Throughput Screening Assays', MS='Rapid methods of measuring the effects of an agent in a biological or chemical assay. The assay usually involves some form of automation or a way to conduct multiple assays at the same time using sample arrays.'),
  'D057167': NtO(MH='Claudins', MS='A large family of transmembrane proteins found in TIGHT JUNCTIONS. They take part in the formation of paracellular barriers and pores that regulate paracellular permeability.'),
  'D057168': NtO(MH='Distillation', MS='A chemical process for separating the components of a liquid mixture by boiling and collecting condensed vapors.'),
  'D057169': NtO(MH='Eye Movement Desensitization Reprocessing', MS='A technique that induces the processing of disturbing memories and experiences, by stimulating neural mechanisms that are similar to those activated during REM sleep. The technique consists of eye movements following side-to-side movements of the index and middle fingers, or the alternate tapping of the hands on the knees. This procedure triggers the processing of information, thus facilitating the connection of neural networks.'),
  'D057170': NtO(MH='Translational Medical Research', MS='The application of discoveries generated by laboratory research and preclinical studies to the development of clinical trials and studies in humans. A second area of translational research concerns enhancing the adoption of best practices.'),
  'D057171': NtO(MH='American Recovery and Reinvestment Act', MS='Public Law No: 111-5, enacted February 2009, makes supplemental appropriations for job preservation and creation, infrastructure investment, energy efficiency and science, assistance to the unemployed, and State and local fiscal stabilization, for fiscal year ending September 30, 2009.'),
  'D057172': NtO(MH='Economic Recession', MS='Significant decline in economic activity spread across the economy, lasting more than a few months, normally visible in real gross domestic product, real income, employment, industrial production, and wholesale-retail sales. (National Bureau of Economic Research, Inc, www.nber.org/cycles.html, accessed 4/23/2009)'),
  'D057173': NtO(MH='Recreation Therapy', MS="The enhancement of physical, cognitive, emotional and social skills so an individual may participate in chosen activities. Recreational modalities are used in designed intervention strategies, incorporating individual's interests to make the therapy process meaningful and relevant."),
  'D057174': NtO(MH='Frontotemporal Lobar Degeneration', MS='Heterogeneous group of neurodegenerative disorders characterized by frontal and temporal lobe atrophy associated with neuronal loss, gliosis, and dementia. Patients exhibit progressive changes in social, behavioral, and/or language function. Multiple subtypes or forms are recognized based on presence or absence of TAU PROTEIN inclusions. FTLD includes three clinical syndromes: FRONTOTEMPORAL DEMENTIA, semantic dementia, and PRIMARY PROGRESSIVE NONFLUENT APHASIA.'),
  'D057175': NtO(MH='Posters as Topic', MS='Works about single or multi-sheet notices made to attract attention to events, activities, causes, goods, or services. They are for display, usually in a public place and are chiefly pictorial.'),
  'D057176': NtO(MH='Compassionate Use Trials', MS='Providing an investigational therapy to a patient who is not eligible to receive that therapy in a clinical trial, but who has a serious or life-threatening illness for which other treatments are not available. Compassionate use trials allow patients to receive promising but not yet fully studied or approved therapies when no other treatment option exists. Also called expanded access trial.'),
  'D057177': NtO(MH='TDP-43 Proteinopathies', MS='Diseases characterized by the presence of abnormally phosphorylated, ubiquitinated, and cleaved DNA-binding protein TDP-43 in affected brain and spinal cord. Inclusions of the pathologic protein in neurons and glia, without the presence of AMYLOID, is the major feature of these conditions, thus making these proteinopathies distinct from most other neurogenerative disorders in which protein misfolding leads to brain amyloidosis. Both frontotemporal lobar degeneration and AMYOTROPHIC LATERAL SCLEROSIS exhibit this common method of pathogenesis and thus they may represent two extremes of a continuous clinicopathological spectrum of one disease.'),
  'D057178': NtO(MH='Primary Progressive Nonfluent Aphasia', MS='A form of frontotemporal lobar degeneration and a progressive form of dementia characterized by motor speech impairment and AGRAMMATISM, with relative sparing of single word comprehension and semantic memory.'),
  'D057179': NtO(MH='Advanced Practice Nursing', MS='Evidence-based nursing, midwifery and healthcare grounded in research and scholarship. Practitioners include nurse practitioners, clinical nurse specialists, nurse anesthetists, and nurse midwives.'),
  'D057180': NtO(MH='Frontotemporal Dementia', MS='The most common clinical form of FRONTOTEMPORAL LOBAR DEGENERATION, this dementia presents with personality and behavioral changes often associated with disinhibition, apathy, and lack of insight.'),
  'D057181': NtO(MH='Pleasure', MS='Sensation of enjoyment or gratification.'),
  'D057182': NtO(MH='Mendelian Randomization Analysis', MS='The use of the GENETIC VARIATION of known functions or phenotypes to correlate the causal effects of those functions or phenotypes with a disease outcome.'),
  'D057184': NtO(MH="Practice Patterns, Nurses'", MS='Patterns of practice in nursing related to provision of services including diagnosis and treatment.'),
  'D057185': NtO(MH='Sedentary Behavior', MS='Behaviors during waking hours that have low energy expenditure and are often performed in a sitting or reclining POSTURE.'),
  'D057186': NtO(MH='Comparative Effectiveness Research', MS='Conduct and synthesis of systematic research comparing interventions and strategies to prevent, diagnose, treat, and monitor health conditions. The purpose of this research is to inform patients, providers, and decision-makers, responding to their expressed needs, about which interventions are most effective for which patients under specific circumstances. (hhs.gov/recovery/programs/cer/draftdefinition.html accessed 6/12/2009)'),
  'D057187': NtO(MH='Independent Living', MS='A housing and community arrangement that maximizes independence and self-determination.'),
  'D057188': NtO(MH='Workflow', MS='Description of pattern of recurrent functions or procedures frequently found in organizational processes, such as notification, decision, and action.'),
  'D057189': NtO(MH='Checklist', MS='Aid for consistent recording of data such as tasks completed and observations noted.'),
  'D057190': NtO(MH='Stroop Test', MS="Timed test in which the subject must read a list of words or identify colors presented with varying instructions and different degrees of distraction. (Campbell's Psychiatric Dictionary. 8th ed.)"),
  'D057191': NtO(MH='Capacity Building', MS='Organizational development including enhancement of management structures, processes and procedures, within organizations and among different organizations and sectors to meet present and future needs.'),
  'D057192': NtO(MH='Tsunamis', MS='Series of ocean waves produced by geologic events or underwater LANDSLIDES. These waves can travel at speeds averaging 450 (and up to 600) miles per hour in the open ocean.'),
  'D057193': NtO(MH='Medical Tourism', MS='Travel to another country for the purpose of medical treatment or care.'),
  'D057194': NtO(MH='Intention to Treat Analysis', MS='Strategy for the analysis of RANDOMIZED CONTROLLED TRIALS AS TOPIC that compares patients in the groups to which they were originally randomly assigned.'),
  'D057205': NtO(MH='Automation, Laboratory', MS='Controlled operations of analytic or diagnostic processes, or systems by mechanical or electronic devices.'),
  'D057207': NtO(MH='Child Restraint Systems', MS='Devices used to protect and restrain infant and child automotive passengers.'),
  'D057208': NtO(MH='Health Level Seven', MS='An American National Standards Institute-accredited organization working on specifications to support development and advancement of clinical and administrative standards for healthcare.'),
  'D057209': NtO(MH='Hospital Rapid Response Team', MS='Multidisciplinary team most frequently consisting of INTENSIVE CARE UNIT trained personnel who are available 24 hours per day, 7 days per week for evaluation of patients who develop signs or symptoms of severe CLINICAL DETERIORATION.'),
  'D057210': NtO(MH='Delayed Diagnosis', MS='Non-optimal interval of time between onset of symptoms, identification, and initiation of treatment.'),
  'D057211': NtO(MH='Webcasts as Topic', MS='Works about transmission of live or pre-recorded audio or video content via connection or download from the INTERNET.'),
  'D057212': NtO(MH='Therapeutic Equipoise', MS='Expectation of real uncertainty on the part of the investigator regarding the comparative therapeutic merits of each arm in a trial.'),
  'D057213': NtO(MH='Incunabula', MS='Books printed before 1501.'),
  'D057214': NtO(MH='Ottoman Empire', MS='A sultanate comprised at various times of parts of EUROPE, ASIA, and AFRICA. Its period extends generally from 1301 to 1922.'),
  'D057215': NtO(MH='Body Dysmorphic Disorders', MS='Preoccupations with appearance or self-image causing significant distress or impairment in important areas of functioning.'),
  'D057216': NtO(MH='Propensity Score', MS='Conditional probability of exposure to a treatment given observed covariates.'),
  'D057217': NtO(MH='Economic Development', MS='Mobilization of human, financial, capital, physical and or natural resources to generate goods and services.'),
  'D057218': NtO(MH='Blogging', MS='Creating an INTERNET-based personal journal which may consist of reflections, comments, and often hyperlinks.'),
  'D057219': NtO(MH='Biofouling', MS='Process by which unwanted microbial, plant or animal materials or organisms accumulate on man-made surfaces.'),
  'D057220': NtO(MH='Health Literacy', MS='Degree to which individuals have the capacity to obtain, process, and understand basic health information and services needed to make appropriate health decisions.'),
  'D057221': NtO(MH='Influenza A Virus, H1N2 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 1 and neuraminidase 2. It is endemic in both human and pig populations.'),
  'D057225': NtO(MH='Data Mining', MS='Use of sophisticated analysis tools to sort through, organize, examine, and combine large sets of information.'),
  'D057226': NtO(MH='Information Seeking Behavior', MS='How information is gathered in personal, academic or work environments and the resources used.'),
  'D057227': NtO(MH='Femininity', MS='Female-associated sex-specific social roles and behaviors unrelated to biologic function.'),
  'D057228': NtO(MH='Masculinity', MS='Male-associated sex-specific social roles and behaviors unrelated to biologic function.'),
  'D057229': NtO(MH='Alcoholics', MS='Persons who have a history of physical or psychological dependence on ETHANOL.'),
  'D057230': NtO(MH='Limit of Detection', MS='Concentration or quantity that is derived from the smallest measure that can be detected with reasonable certainty for a given analytical procedure.'),
  'D057231': NtO(MH='Climate Change', MS="Any significant change in measures of climate (such as temperature, precipitation, or wind) lasting for an extended period (decades or longer). It may result from natural factors such as changes in the sun's intensity, natural processes within the climate system such as changes in ocean circulation, or human activities."),
  'D057232': NtO(MH='Global Warming', MS="Increase in the temperature of the atmosphere near the Earth's surface and in the troposphere, which can contribute to changes in global climate patterns."),
  'D057233': NtO(MH='Radio Frequency Identification Device', MS='Machine readable patient or equipment identification device using radio frequency from 125 kHz to 5.8 Ghz.'),
  'D057234': NtO(MH='Preoperative Period', MS='The period before a surgical operation.'),
  'D057235': NtO(MH='Austria-Hungary', MS='A dual monarchy formed in 1867. It included AUSTRIA; HUNGARY; CZECHOSLOVAKIA; Moravia, Bukovina, Transylvania, Carniola, Kustenland, Dalmatia, CROATIA; Fiume, and Galicia.'),
  'D057236': NtO(MH='Travel Medicine', MS="Multidisciplinary field focusing on prevention of infectious diseases and patient safety during international TRAVEL. Key element of patient's pre-travel visit to the physician is a health risk assessment."),
  'D057237': NtO(MH='Criminals', MS='Persons who have committed a crime or have been convicted of a crime.'),
  'D057238': NtO(MH='Growth Charts', MS='Graphic displays of height and weight showing development over time.'),
  'D057239': NtO(MH='Early Termination of Clinical Trials', MS='Earlier than planned termination of clinical trials.'),
  'D057240': NtO(MH='Patient Preference', MS="Individual's expression of desirability or value of one course of action, outcome, or selection in contrast to others."),
  'D057265': NtO(MH='Site-Specific DNA-Methyltransferase (Cytosine-N(4)-Specific)', MS='A DNA-methyltransferase that catalyzes the transfer of a methyl group from S-ADENOSYLMETHIONINE to the exocyclic NH2 group at the 4 position of CYTOSINE.'),
  'D057285': NtO(MH='Precision Medicine', MS="Clinical, therapeutic and diagnostic approaches to optimal disease management based on individual variations in a patient's genetic profile."),
  'D057286': NtO(MH='Electronic Health Records', MS="Media that facilitate transportability of pertinent information concerning patient's illness across varied providers and geographic locations. Some versions include direct linkages to online CONSUMER HEALTH INFORMATION that is relevant to the health conditions and treatments related to a specific patient."),
  'D057345': NtO(MH='Sequence Inversion', MS='The deletion and reinsertion of a segment of a nucleic acid sequence in the same place, but flipped in an opposite orientation.'),
  'D057386': NtO(MH='Search Engine', MS='Software used to locate data or information stored in machine-readable form locally or at a distance such as an INTERNET site.'),
  'D057405': NtO(MH='Webcast', MS='Content from transmission of live or pre-recorded audio or video via connection or download from the INTERNET.'),
  'D057445': NtO(MH='Plant Dormancy', MS='The state of failure to initiate and complete the process of growth, reproduction, or gemination of otherwise normal plants or vegetative structures thereof.'),
  'D057446': NtO(MH='Hydrodynamics', MS='The motion of fluids, especially noncompressible liquids, under the influence of internal and external forces.'),
  'D057465': NtO(MH='Catabolite Repression', MS='Process by which micro-organisms adapt quickly to a preferred rapidly-metabolizable intermediate through the inhibition or repression of genes related to CATABOLISM of less preferred source(s).'),
  'D057466': NtO(MH='Nonsteroidal Anti-Androgens', MS='Nonsteroidal agents which block the action or downregulate the synthesis of ANDROGENS.'),
  'D057486': NtO(MH='Carbon Cycle', MS="The cycle by which the element carbon is exchanged between organic matter and the earth's physical environment."),
  'D057487': NtO(MH='Enzyme Therapy', MS='The use of enzymes to correct metabolic and physiological processes.'),
  'D057488': NtO(MH='Visual Prosthesis', MS='Artificial device such as an externally-worn camera attached to a stimulator on the RETINA, OPTIC NERVE, or VISUAL CORTEX, intended to restore or amplify vision.'),
  'D057505': NtO(MH='Embolic Protection Devices', MS='Vascular filters or occlusive devices that provide mechanical protection of the distal end organ from blood clots or EMBOLISM-causing debri dislodged during ENDOVASCULAR PROCEDURES.'),
  'D057506': NtO(MH='Arthropod Antennae', MS='Paired sense organs connected to the anterior segments of ARTHROPODS that help them navigate through the environment.'),
  'D057507': NtO(MH='Mice, 129 Strain', MS='Strains of mice arising from a parental inbred stock that was subsequently used to produce substrains of knockout and other mutant mice with targeted mutations.'),
  'D057508': NtO(MH='Burkholderia cenocepacia', MS='A species of gram-negative bacteria that causes disease in plants. It is found commonly in the environment and is an opportunistic pathogen in humans.'),
  'D057509': NtO(MH='Trypanosoma rangeli', MS='A hemoflagellate parasite affecting domestic and wild animals, as well as humans and invertebrates. Though it induces an immune response, it is non-pathogenic in humans and other vertebrates. It is cross-reactive with TRYPANOSOMA CRUZI and can thus cause false positives for CHAGAS DISEASE.'),
  'D057510': NtO(MH='Endovascular Procedures', MS='Minimally invasive procedures, diagnostic or therapeutic, performed within the BLOOD VESSELS. They may be perfomed via ANGIOSCOPY; INTERVENTIONAL MAGNETIC RESONANCE IMAGING; INTERVENTIONAL RADIOGRAPHY; or INTERVENTIONAL ULTRASONOGRAPHY.'),
  'D057545': NtO(MH='Social Stigma', MS='A perceived attribute that is deeply discrediting and is considered to be a violation of social norms.'),
  'D057565': NtO(MH='Apathy', MS='Lack of emotion or emotional expression; a disorder of motivation that persists over time.'),
  'D057566': NtO(MH='Self Report', MS='Method for obtaining information through verbal responses, written or oral, from subjects.'),
  'D057567': NtO(MH='Memory, Long-Term', MS='Remembrance of information from 3 or more years previously.'),
  'D057585': NtO(MH='Aeromonas caviae', MS='A species of gram-negative, facultatively anaerobic bacteria that is found in domestic and wild animals including birds, and fish. In humans it causes GASTROENTERITIS in young children and some adults.'),
  'D057586': NtO(MH='Vitreoretinal Surgery', MS='Specialized ophthalmic technique used in the surgical repair and or treatment of disorders that include retinal tears or detachment; MACULAR HOLES; hereditary retinal disease; AIDS-related retinal infections; ocular tumors; MACULAR DEGENERATION; DIABETIC RETINOPATHY; and UVEITIS.'),
  'D057605': NtO(MH='Natural Orifice Endoscopic Surgery', MS='Surgical procedures performed through a natural opening in the body such as the mouth, nose, urethra, or anus, and along the natural body cavities with which they are continuous.'),
  'D057625': NtO(MH='Isotachophoresis', MS='A nonlinear electrophoretic technique used to separate a variety of ionic compounds, ranging from small metal ions to large molecules like proteins. Unlike "linear" zone electrophoresis in which separating solute bands continually spread by diffusion or dispersion, isotachophoresis forms self-sharpening, adjacent zones of substantially pure solute whose concentrations often exceed several mgs/ml. In isotachophoresis a multianalyte sample is introduced between the leading electrolyte and the terminating electrolyte where the sample ions have lower electrophoretic mobilities than the leading ion but larger than the terminating ion. (From "Isotachophoresis" on the AES Web Site [Internet]. Madison, WI: The American Electrophoresis Society; c2000-2008 [cited 2009 Aug 20]. Available from http://www.aesociety.org/areas/isotachophoresis.php)'),
  'D057645': NtO(MH='Deltoid Muscle', MS='Thick triangular muscle in the SHOULDER whose function is to abduct, flex, and extend the arm. It is a common site of INTRAMUSCULAR INJECTIONS.'),
  'D057646': NtO(MH='Virus Uncoating', MS='Intracellular step that follows VIRUS INTERNALIZATION during which the viral nucleic acid and CAPSID are separated.'),
  'D057665': NtO(MH='Melinis repens', MS='Formerly known as Rhynchelytrum repens, a member of the perennial grass family native to South Africa, widely distributed and known by different common names. It is used as a diabetes cure in folk medicine and actively studied for its hypoglycemic properties.'),
  'D057666': NtO(MH='Research Support, American Recovery and Reinvestment Act', MS='Acknowledgement that funding support is from the American Recovery and Reinvestment Act.'),
  'D057667': NtO(MH='Conducted Energy Weapon Injuries', MS='The injuries caused by conducted energy weapons such as stun guns, shock batons, and cattle prods.'),
  'D057668': NtO(MH='Bloodless Medical and Surgical Procedures', MS='The treatment of patients without the use of allogeneic BLOOD TRANSFUSIONS or blood products.'),
  'D057685': NtO(MH='Plant Weeds', MS='A plant growing in a location where it is not wanted, often competing with cultivated plants.'),
  'D057686': NtO(MH='Cell-in-Cell Formation', MS='The mechanisms by which a cell becomes internalized in another. The host cell may engulf another as do PHAGOCYTIC CELLS, or the host cell may be invaded by another cell (ENTOSIS), or internalization processes may involve the cooperation of both the host cell and the cell being internalized. Viable cells may remain in non-phagocytic cells (EMPERIPOLESIS), undergo cell division, pass through and then out of the host cell (TRANSCELLULAR CELL MIGRATION), or trigger APOPTOSIS of the invaded cell.'),
  'D057687': NtO(MH='Emperipolesis', MS='The movement of one cell within another cell (non-phagocytic).'),
  'D057688': NtO(MH='Transcellular Cell Migration', MS='The movement of one cell into, through, and out of another cell.'),
  'D057689': NtO(MH='Research Support, U.S. Government', MS='For publications noted as supported by US Government.'),
  'D057691': NtO(MH='Entosis', MS='The processes by which one cell actively invades and becomes internalized within another cell.'),
  'D057705': NtO(MH='Transendothelial and Transepithelial Migration', MS='The passage of cells across the layer of ENDOTHELIAL CELLS, i.e., the ENDOTHELIUM; or across the layer of EPITHELIAL CELLS, i.e. the EPITHELIUM.'),
  'D057725': NtO(MH='Operative Blood Salvage', MS='Recovery of blood lost from surgical procedures for reuse by the same patient in AUTOLOGOUS BLOOD TRANSFUSIONS. It is collected during (intraoperatively) or after completion of (postoperatively) the surgical procedures.'),
  'D057747': NtO(MH='Periodontal Debridement', MS='Removal or disruption of DENTAL DEPOSITS and plaque-retentive DENTAL CALCULUS from tooth surfaces and within the periodontal pocket space without deliberate removal of CEMENTUM as done in ROOT PLANING and often in DENTAL SCALING. The goal is to conserve dental cementum to help maintain or re-establish healthy periodontal environment and eliminate PERIODONTITIS by using light instrumentation strokes and nonsurgical techniques (e.g., ultrasonic, laser instruments).'),
  'D057748': NtO(MH='Mesotherapy', MS='The application of medicine, vitamins, extracts, or other bioactive substances for a localized effect via multiple subcutaneous injections or perfusion of substances into the various layers of the skin below the EPIDERMIS.'),
  'D057765': NtO(MH='Eosinophilic Esophagitis', MS='Chronic ESOPHAGITIS characterized by esophageal mucosal EOSINOPHILIA. It is diagnosed when an increase in EOSINOPHILS are present over the entire esophagus. The reflux symptoms fail to respond to PROTON PUMP INHIBITORS treatment, unlike in GASTROESOPHAGEAL REFLUX DISEASE. The symptoms are associated with IgE-mediated hypersensitivity to food or inhalant allergens.'),
  'D057766': NtO(MH='Policy', MS='A course or method of action selected to guide and determine present and future decisions.'),
  'D057767': NtO(MH='Kashin-Beck Disease', MS='Disabling osteochondrodysplasia with OSTEOSCLEROSIS, cone-shaped METAPHYSIS, and shortening of the DIAPHYSIS. It is endemic in parts of Siberia and northern China. Mineral deficiencies (e.g., selenium, iodine), fungal cereal contamination, and water contamination may be contributing factors in its etiology.'),
  'D057768': NtO(MH='Infantile Apparent Life-Threatening Event', MS='An event experienced by an infant or a child that is characterized by some combination of apnea, color change, change in muscle tone, choking, and gagging.'),
  'D057769': NtO(MH='Transmyocardial Laser Revascularization', MS='A procedure to increase the flow of blood to the MYOCARDIUM by creating transmural channels in the heart wall via the application of laser pulses to epicardial or endocardial surfaces.'),
  'D057770': NtO(MH='Hyalinosis, Systemic', MS='Autosomal recessive disorder characterized by HYALINE deposition in the skin, bone, gastrointestinal tract, muscles and glands; multiple subcutaneous skin nodules; GINGIVAL HYPERTROPHY; and joint CONTRACTURES. Mutations in the capillary morphogenesis protein-2 are associated with the disorder.'),
  'D057771': NtO(MH='Cystine-Knot Miniproteins', MS='A structurally-related family of small proteins that form a stable tertiary fold pattern which is supported by a series of disulfide bonds. The arrangement of disulfide bonds between the CYSTEINE moieties results in a knotted structure which is unique to this family of proteins.'),
  'D057772': NtO(MH='Vascular System Injuries', MS='Injuries to blood vessels caused by laceration, contusion, puncture, or crush and other types of injuries. Symptoms vary by site and mode of injuries and may include bleeding, bruising, swelling, pain, and numbness. It does not include injuries secondary to pathologic function or diseases such as ATHEROSCLEROSIS.'),
  'D057773': NtO(MH='Cold Shock Proteins and Peptides', MS='Cellular proteins and peptides that are induced in response to cold stress. They are found in a broad variety of prokaryotic and eukaryotic organisms.'),
  'D057774': NtO(MH='Post-Exercise Hypotension', MS='Transient reduction in blood pressure levels immediately after exercises that lasts 2-12 hours. The reduction varies but is typically 5-20 mm Hg when compared to pre-exercise levels. It exists both in normotensive and hypertensive individuals and may play a role in excercise related PHYSIOLOGIC ADAPTATION.'),
  'D057775': NtO(MH='Ischemic Postconditioning', MS='The application of repeated, brief periods of vascular occlusion at the onset of REPERFUSION to reduce REPERFUSION INJURY that follows a prolonged ischemic event. The techniques are similar to ISCHEMIC PRECONDITIONING but the time of application is after the ischemic event instead of before.'),
  'D057785': NtO(MH='Catheters', MS='A flexible, tubular device that is used to carry fluids into or from a blood vessel, hollow organ, or body cavity.'),
  'D057786': NtO(MH='Peptidomimetics', MS='Compounds that are designed to mimic the 3D structure of a natural peptide or protein.'),
  'D057787': NtO(MH='Staphylococcus lugdunensis', MS='A species of gram-positive bacteria in the family STAPHYLOCOCCACEAE. It is responsible for skin and soft-tissue infections among others, and is part of the normal human skin flora.'),
  'D057788': NtO(MH='Staphylococcus hyicus', MS='A species of gram-positive bacteria in the family STAPHYLOCOCCACEAE. It is an important opportunistic pathogen in swine.'),
  'D057789': NtO(MH='Staphylococcus intermedius', MS='A species of gram-positive bacteria in the family STAPHYLOCOCCACEAE. It is a zoonotic organism and common commensal in dogs, but can cause disease in dogs and other animals. It also can be associated with human disease.'),
  'D057790': NtO(MH='Staphylococcus saprophyticus', MS='A species of gram-positive bacteria in the family STAPHYLOCOCCACEAE. It commonly causes urinary tract infections in humans.'),
  'D057791': NtO(MH='Cardiac Imaging Techniques', MS='Visualization of the heart structure and cardiac blood flow for diagnostic evaluation or to guide cardiac procedures via techniques including ENDOSCOPY (cardiac endoscopy, sometimes refered to as cardioscopy), RADIONUCLIDE IMAGING; MAGNETIC RESONANCE IMAGING; TOMOGRAPHY; or ULTRASONOGRAPHY.'),
  'D057792': NtO(MH='Mirizzi Syndrome', MS='Complication of CHOLELITHIASIS characterized by OBSTRUCTIVE JAUNDICE; abdominal pain, and fever.'),
  'D057805': NtO(MH='Pets', MS='Animals kept by humans for companionship and enjoyment, as opposed to DOMESTIC ANIMALS such as livestock or farm animals, which are kept for economic reasons.'),
  'D057807': NtO(MH='Molineoidae', MS='A superfamily of roundworms in the order STRONGYLIDA. It comprises the single family Molineidae and subfamily Nematodirinae.'),
  'D057808': NtO(MH='Nematodirus', MS='A genus of roundworms in the family MOLINIEIDAE, whose species are important gastrointestinal parasites of RUMINANTS, especially CATTLE; SHEEP; and GOATS.'),
  'D057809': NtO(MH='HEK293 Cells', MS='A cell line generated from human embryonic kidney cells that were transformed with human adenovirus type 5.'),
  'D057825': NtO(MH='Two-Dimensional Difference Gel Electrophoresis', MS='Methods of comparing two or more samples on the same two-dimensional gel electrophoresis gel.'),
  'D057826': NtO(MH='Vitelliform Macular Dystrophy', MS='Autosomal dominant hereditary maculopathy with childhood-onset accumulation of LIPOFUSION in RETINAL PIGMENT EPITHELIUM. Affected individuals develop progressive central acuity loss, and distorted vision (METAMORPHOPSIA). It is associated with mutations in bestrophin, a chloride channel.'),
  'D057828': NtO(MH='Informed Consent By Minors', MS='Voluntary authorization by a person not of usual legal age for diagnostic or investigative procedures, or for medical and surgical treatment. (from English A, Shaw FE, McCauley MM, Fishbein DB Pediatrics 121:Suppl Jan 2008 pp S85-7).'),
  'D057829': NtO(MH='Sex Reassignment Procedures', MS='Clinical treatments used to change the physiological sexual characteristics of an individual.'),
  'D057830': NtO(MH='Sex Reassignment Surgery', MS='Surgical treatments used to change the physiological sexual characteristics of an individual.'),
  'D057831': NtO(MH='Lemierre Syndrome', MS='A superinfection of the damaged oropharyngeal mucosa by FUSOBACTERIUM NECROPHORUM leading to the secondary septic THROMBOPHLEBITIS of the internal jugular vein.'),
  'D057832': NtO(MH='Watchful Waiting', MS='Clinical management approach wherein immediate therapy is not provided but there is a period of observation during which periodic tests monitor patient and the progression of the illness. (Driffield T, Smith PC Med Decis Making. 2007 Mar-Apr;27(2):178-88)'),
  'D057845': NtO(MH='Pyrimidine Phosphorylases', MS='Pentosyltransferases that catalyze the reaction between a pyrimidine nucleoside and orthophosphate to form a free pyrimidine and ribose-5-phosphate.'),
  'D057846': NtO(MH='Cell-Penetrating Peptides', MS='Peptides that have the ability to enter cells by crossing the plasma membrane directly, or through uptake by the endocytotic pathway.'),
  'D057847': NtO(MH='Lipid Regulating Agents', MS='Substances that alter the metabolism of LIPIDS.'),
  'D057849': NtO(MH='Optic Flow', MS='The continuous visual field seen by a subject through space and time.'),
  'D057850': NtO(MH='Hermaphroditic Organisms', MS='Animals and plants which have, as their normal mode of reproduction, both male and female sex organs in the same individual.'),
  'D057851': NtO(MH='Posterior Capsular Rupture, Ocular', MS='A breach in the continuity of the posterior chamber of the eyeball.'),
  'D057852': NtO(MH='Posterior Capsule of the Lens', MS='The posterior aspect of the casing that surrounds the natural CRYSTALLINE LENS.'),
  'D057865': NtO(MH='Plant Immunity', MS='The inherent or induced capacity of plants to withstand or ward off biological attack by pathogens.'),
  'D057866': NtO(MH='Anterior Capsule of the Lens', MS='The anterior aspect of the casing that surrounds the natural CRYSTALLINE LENS.'),
  'D057867': NtO(MH='Endoleak', MS='Postoperative hemorrhage from an endovascular AORTIC ANEURYSM repaired with endoluminal placement of stent grafts (BLOOD VESSEL PROSTHESIS IMPLANTATION). It is associated with pressurization, expansion, and eventual rupture of the aneurysm.'),
  'D057868': NtO(MH='Anastomotic Leak', MS='Breakdown of the connection and subsequent leakage of effluent (fluids, secretions, air) from a SURGICAL ANASTOMOSIS of the digestive, respiratory, genitourinary, and cardiovascular systems. Most common leakages are from the breakdown of suture lines in gastrointestinal or bowel anastomosis.'),
  'D057872': NtO(MH='Pulp Capping and Pulpectomy Agents', MS='Materials used in DENTAL PULP CAPPING or PULPECTOMY.'),
  'D057873': NtO(MH='Peri-Implantitis', MS='An inflammatory process with loss of supporting bone in the tissues surrounding functioning DENTAL IMPLANTS.'),
  'D057885': NtO(MH='Triploidy', MS='Polyploidy with three sets of chromosomes. Triploidy in humans are 69XXX, 69XXY, and 69XYY. It is associated with HOLOPROSENCEPHALY; ABNORMALITIES, MULTIPLE; PARTIAL HYDATIDIFORM MOLE; and MISCARRAGES.'),
  'D057886': NtO(MH='Bleaching Agents', MS='Chemicals that are used to oxidize pigments and thus effect whitening.'),
  'D057887': NtO(MH='Overbite', MS='A malocclusion in which maxillary incisor and canine teeth project over the mandibular teeth excessively. The overlap is measured perpendicular to the occlusal plane and is also called vertical overlap. When the overlap is measured parallel to the occlusal plane it is referred to as overjet.'),
  'D057888': NtO(MH='Iridoid Glucosides', MS='A subclass of iridoid compounds that include a glucoside moiety, usually found at the C-1 position.'),
  'D057889': NtO(MH='Iridoid Glycosides', MS='A subclass of iridoid compounds that include a glycoside moiety, usually found at the C-1 position.'),
  'D057890': NtO(MH='Epigenomics', MS='The systematic study of the global gene expression changes due to EPIGENETIC PROCESSES and not due to DNA base sequence changes.'),
  'D057891': NtO(MH='Tetraploidy', MS='The presence of four sets of chromosomes. It is associated with ABNORMALITIES, MULTIPLE; and MISCARRAGES.'),
  'D057892': NtO(MH='Lesser Pelvis', MS='The part of the pelvis, inferior to the pelvic brim, that comprises both the pelvic cavity and the part of the PERINEUM lying inferior to the PELVIC DIAPHRAGM.'),
  'D057893': NtO(MH='Immediate Dental Implant Loading', MS='Endosseous dental implantation where implants are fitted with an abutment or where an implant with a transmucosal coronal portion is used immediately (within 1 week) after the initial extraction. Conventionally, the implantation is performed in two stages with more than two months in between the stages.'),
  'D057894': NtO(MH='Coral Reefs', MS='Marine ridges composed of living CORALS, coral skeletons, calcareous algae, and other organisms, mixed with minerals and organic matter. They are found most commonly in tropical waters and support other animal and plant life.'),
  'D057895': NtO(MH='Haploinsufficiency', MS='A copy number variation that results in reduced GENE DOSAGE due to any loss-of-function mutation. The loss of heterozygosity is associated with abnormal phenotypes or diseased states because the remaining gene is insufficient.'),
  'D057896': NtO(MH='Striae Distensae', MS='Linear dermal scars accompanied by epidermal atrophy that affects skin that is subjected to continuous stretching. They usually do not cause any significant medical problems, only cosmetic problems.'),
  'D057897': NtO(MH='Reflex, Righting', MS='The instinctive tendency (or ability) to assume a normal position of the body in space when it has been displaced.'),
  'D057898': NtO(MH='Hemizygote', MS='An individual having only one allele at a given locus because of the loss of the other allele through a mutation (e.g., CHROMOSOME DELETION).'),
  'D057899': NtO(MH='Cytophagocytosis', MS='The engulfment and degradation of cells by other cells.'),
  'D057900': NtO(MH='Transcytosis', MS='The transport of materials through a cell. It includes the uptake of materials by the cell (ENDOCYTOSIS), the movement of those materials through the cell, and the subsequent secretion of those materials (EXOCYTOSIS).'),
  'D057905': NtO(MH='Stem Cell Research', MS='Experimentation on STEM CELLS and on the use of stem cells.'),
  'D057906': NtO(MH='Circadian Clocks', MS='Biological mechanism that controls CIRCADIAN RHYTHM. Circadian clocks exist in the simplest form in cyanobacteria and as more complex systems in fungi, plants, and animals. In humans the system includes photoresponsive RETINAL GANGLION CELLS and the SUPRACHIASMATIC NUCLEUS that acts as the central oscillator.'),
  'D057907': NtO(MH='Post-Synaptic Density', MS='Cytoskeleton specialization at the cytoplasmic side of postsynaptic membrane in SYNAPSES. It is involved in neuronal signaling and NEURONAL PLASTICITY and comprised of GLUTAMATE RECEPTORS; scaffolding molecules (e.g., PSD95, PSD93), and other proteins (e.g., CaCMKII).'),
  'D057908': NtO(MH='Argon Plasma Coagulation', MS='A method of tissue ablation and bleeding control that uses ARGON plasma (ionized argon gas) to deliver a current of thermocoagulating energy to the area of tissue to be coagulated.'),
  'D057909': NtO(MH='Orthognathic Surgery', MS='A surgical specialty concerned with correcting conditions related to malposition of the jawbones and other related MAXILLOFACIAL ABNORMALITIES (e.g., CLEFT PALATE; RETROGNATHISM).'),
  'D057910': NtO(MH='Implant Capsular Contracture', MS='The shrinkage of the foreign body encapsulation scar tissue that forms around artificial implants imbedded in body tissues.'),
  'D057911': NtO(MH='Angiotensin Receptor Antagonists', MS='Agents that antagonize ANGIOTENSIN RECEPTORS. Many drugs in this class specifically target the ANGIOTENSIN TYPE 1 RECEPTOR.'),
  'D057912': NtO(MH='Angiotensin II Type 2 Receptor Blockers', MS='Agents that antagonize the ANGIOTENSIN II TYPE 2 RECEPTOR.'),
  'D057913': NtO(MH='Joint Capsule Release', MS='Surgical procedure to relax the JOINT CAPSULE tissues in a joint that has a reduced range of motion due to CONTRACTURE or TISSUE ADHESIONS or joint deformities.'),
  'D057914': NtO(MH='Agricultural Irrigation', MS='The routing of water to open or closed areas where it is used for agricultural purposes.'),
  'D057915': NtO(MH='Drug Substitution', MS='The practice of replacing one prescribed drug with another that is expected to have the same clinical or psychological effect.'),
  'D057916': NtO(MH='Influenza A Virus, H7N1 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 7 and neuraminidase 1. This subtype has demonstrated the ability to mutate from a low pathogenic form to a highly pathogenic form in birds. It was responsible for a 1999 outbreak in turkeys in Italy.'),
  'D057917': NtO(MH='Influenza A Virus, H7N2 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 7 and neuraminidase 2. It has been involved in a number of outbreaks in the 21st century on poultry farms and has been isolated a few times in humans.'),
  'D057918': NtO(MH='Fiducial Markers', MS='Materials used as reference points for imaging studies.'),
  'D057919': NtO(MH='Biological Oxygen Demand Analysis', MS='Testing for the amount of biodegradable organic material in a water sample by measuring the quantity of oxygen consumed by biodegradation of those materials over a specific time period.'),
  'D057920': NtO(MH='Influenza A Virus, H7N3 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 7 and neuraminidase 3. It was first detected in turkeys in Britain in 1963 and there have been several outbreaks on poultry farms since that time. A couple cases of human infections have been reported.'),
  'D057921': NtO(MH='Influenza A Virus, H10N7 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins hemagglutinin 10 and neuraminidase 7. It has been isolated from a variety of wild and domestic animals including ducks, emu, and mink. It was found for the first time in humans in 2004.'),
  'D057925': NtO(MH='Femoracetabular Impingement', MS='A pathological mechanical process that can lead to hip failure. It is caused by abnormalities of the ACETABULUM and/or FEMUR combined with rigorous hip motion, leading to repetitive collisions that damage the soft tissue structures.'),
  'D057927': NtO(MH='Hydrophobic and Hydrophilic Interactions', MS='The thermodynamic interaction between a substance and WATER.'),
  'D057928': NtO(MH='Intermittent Urethral Catheterization', MS='Insertion of a catheter into the urethra to drain the urine from the bladder at intervals as needed.'),
  'D057929': NtO(MH='Arthroplasty, Replacement, Elbow', MS='Replacement of the ELBOW JOINT.'),
  'D057930': NtO(MH='Click Chemistry', MS='Organic chemistry methodology that mimics the modular nature of various biosynthetic processes. It uses highly reliable and selective reactions designed to "click" i.e., rapidly join small modular units together in high yield, without offensive byproducts. In combination with COMBINATORIAL CHEMISTRY TECHNIQUES, it is used for the synthesis of new compounds and combinatorial libraries.'),
  'D057945': NtO(MH='Self-Fertilization', MS='The fusion of a male gamete with a female gamete from the same individual animal or plant.'),
  'D057946': NtO(MH='Charophyceae', MS='A group (or class) of aquatic plants, including the streptophyte algae, that are the closest relatives to land plants (EMBRYOPHYTA).'),
  'D057947': NtO(MH='Parenteral Nutrition Solutions', MS='Specialized solutions for PARENTERAL NUTRITION. They may contain a variety of MICRONUTRIENTS; VITAMINS; AMINO ACIDS; CARBOHYDRATES; LIPIDS; and SALTS.'),
  'D057948': NtO(MH='Viridiplantae', MS='A monophyletic group of green plants that includes all land plants (EMBRYOPHYTA) and all green algae (CHLOROPHYTA and STREPTOPHYTA).'),
  'D057949': NtO(MH='Streptophyta', MS='A phylum of green plants comprising CHAROPHYCEAE (streptophyte green algae) and EMBRYOPHYTA (land plants).'),
  'D057965': NtO(MH='Carbon Sequestration', MS='Any of several processes for the permanent or long-term artificial or natural capture or removal and storage of carbon dioxide and other forms of carbon, through biological, chemical or physical processes, in a manner that prevents it from being released into the atmosphere.'),
  'D057966': NtO(MH='Calcimimetic Agents', MS='Small organic molecules that act as allosteric activators of the calcium sensing receptor (CaSR) in the PARATHYROID GLANDS and other tissues. They lower the threshold for CaSR activation by extracellular calcium ions and diminish PARATHYROID HORMONE (PTH) release from parathyroid cells.'),
  'D057967': NtO(MH='Infusions, Intraventricular', MS='The delivery of a drug into a fluid-filled cavity of the brain.'),
  'D057968': NtO(MH='Transdermal Patch', MS='A medicated adhesive patch placed on the skin to deliver a specific dose of medication into the bloodstream.'),
  'D057970': NtO(MH='Inappropriate Prescribing', MS='The practice of administering medications in a manner that poses more risk than benefit, particularly where safer alternatives exist.'),
  'D057971': NtO(MH='Anterior Capsular Rupture, Ocular', MS='A breach in the continuity of the ANTERIOR CHAMBER of the eyeball.'),
  'D057972': NtO(MH='Posterior Eye Segment', MS='The back two-thirds of the eye that includes the anterior hyaloid membrane and all of the optical structures behind it: the VITREOUS HUMOR; RETINA; CHOROID; and OPTIC NERVE.'),
  'D057973': NtO(MH='Dent Disease', MS='X-linked recessive NEPHROLITHIASIS characterized by HYPERCALCIURIA; HYPOPHOSPHATEMIA; NEPHROCALCINOSIS; and PROTEINURIA. It is associated with mutations in the voltage-gated chloride channel, CLC-5 (Dent Disease I). Another group of mutations associated with this disease is in phosphatidylinositol 4,5-bisphosphate-5-phosphatase gene.'),
  'D057985': NtO(MH='Multi-Ingredient Cold, Flu, and Allergy Medications', MS='A broad category of multi-ingredient preparations that are marketed for the relief of upper respiratory symptoms resulting from the COMMON COLD; ALLERGIES; or HUMAN INFLUENZA. While the majority of these medications are available as OVER-THE-COUNTER DRUGS some of them contain ingredients that require them to be sold as PRESCRIPTION DRUGS or as BEHIND-THE COUNTER DRUGS.'),
  'D058005': NtO(MH='General Practitioners', MS='Physicians whose practice is not restricted to a specific field of MEDICINE.'),
  'D058006': NtO(MH='General Practice', MS='Patient-based medical care provided across age and gender or specialty boundaries.'),
  'D058007': NtO(MH='Physicians, Primary Care', MS='Providers of initial care for patients. These PHYSICIANS refer patients when appropriate for secondary or specialist care.'),
  'D058008': NtO(MH='Xenotropic murine leukemia virus-related virus', MS='An endogenous GAMMARETROVIRUS from the germ line of mice but isolated from humans. It has close similarity to xenotropic MURINE LEUKEMIA VIRUS.'),
  'D058009': NtO(MH='Stramenopiles', MS='A common name (but used formally) for a group of organisms that are mostly kinds of algae including BACILLARIOPHYTA; OOMYCETES; PHAEOPHYCEAE; and CHRYSOPHYCEAE. They all contain CHLOROPLASTS that are thought to have been derived from the endosymbiosis of ancient RED ALGAE.'),
  'D058010': NtO(MH='Millon Clinical Multiaxial Inventory', MS='A psychological test consisting of true/false items. It is used as a diagnostic screening or clinical assessment of adults who evidence problematic emotional and interpersonal symptoms or who are undergoing PSYCHOTHERAPY or a psychodiagnostic evaluation.'),
  'D058011': NtO(MH='Multifactor Dimensionality Reduction', MS='A statistical tool for detecting and modeling gene-gene interactions. It is a non-parametric and model-free approach.'),
  'D058012': NtO(MH='Small Business', MS='For-profit enterprise with relatively few to moderate number of employees and low to moderate volume of sales.'),
  'D058013': NtO(MH='Test Taking Skills', MS="Skills and strategies, unrelated to the traits a test is intended to measure, that may increase test takers' scores -- may include the effects of coaching or experience in taking tests. (ERIC Thesaurus)"),
  'D058014': NtO(MH='Veterans Health', MS='The concept covering the physical and mental conditions of VETERANS.'),
  'D058015': NtO(MH='Health Communication', MS='The transfer of information from experts in the medical and public health fields to patients and the public. The study and use of communication strategies to inform and influence individual and community decisions that enhance health.'),
  'D058016': NtO(MH='Health Facility Administration', MS='Management of the organization of HEALTH FACILITIES.'),
  'D058017': NtO(MH='Vascular Grafting', MS='Surgical insertion of BLOOD VESSEL PROSTHESES, or transplanted BLOOD VESSELS, or other biological material to repair injured or diseased blood vessels.'),
  'D058025': NtO(MH='Interdisciplinary Studies', MS='Programs of study which span the traditional boundaries of academic scholarship.'),
  'D058027': NtO(MH='Neuropsychiatry', MS="A subfield of psychiatry that emphasizes the somatic substructure on which mental operations and emotions are based, and the functional or organic disturbances of the central nervous system that give rise to, contribute to, or are associated with mental and emotional disorders. (From Campbell's Psychiatric Dictionary, 8th ed.)"),
  'D058028': NtO(MH='Research Report', MS='Detailed account or statement or formal record of data resulting from empirical inquiry.'),
  'D058045': NtO(MH='Artificial Cells', MS='Chemically synthesized structures which functionally resemble natural cells.'),
  'D058065': NtO(MH='Diabetic Cardiomyopathies', MS='Diabetes complications in which VENTRICULAR REMODELING in the absence of CORONARY ATHEROSCLEROSIS and hypertension results in cardiac dysfunctions, typically LEFT VENTRICULAR DYSFUNCTION. The changes also result in myocardial hypertrophy, myocardial necrosis and fibrosis, and collagen deposition due to impaired glucose tolerance.'),
  'D058066': NtO(MH='Digital Dermatitis', MS='Highly contagious infectious dermatitis with lesions near the interdigital spaces usually in cattle. It causes discomfort and often severe lameness (LAMENESS, ANIMAL). Lesions can be either erosive or proliferative and wart-like with papillary growths and hypertrophied hairs. DICHELOBACTER NODOSUS and TREPONEMA are the most commonly associated causative agents for this mixed bacterial infection disease.'),
  'D058067': NtO(MH='Wavelet Analysis', MS='Signal and data processing method that uses decomposition of wavelets to approximate, estimate, or compress signals with finite time and frequency domains. It represents a signal or data in terms of a fast decaying wavelet series from the original prototype wavelet, called the mother wavelet. This mathematical algorithm has been adopted widely in biomedical disciplines for data and signal processing in noise removal and audio/image compression (e.g., EEG and MRI).'),
  'D058068': NtO(MH='Endophenotypes', MS='Measurable biological (physiological, biochemical, and anatomical features), behavioral (psychometric pattern) or cognitive markers that are found more often in individuals with a disease than in the general population. Because many endophenotypes are present before the disease onset and in individuals with heritable risk for disease such as unaffected family members, they can be used to help diagnose and search for causative genes.'),
  'D058069': NtO(MH='Neglected Diseases', MS='Diseases that are underfunded and have low name recognition but are major burdens in less developed countries. The World Health Organization has designated six tropical infectious diseases as being neglected in industrialized countries that are endemic in many developing countries (HELMINTHIASIS; LEPROSY; LYMPHATIC FILARIASIS; ONCHOCERCIASIS; SCHISTOSOMIASIS; and TRACHOMA).'),
  'D058070': NtO(MH='Asymptomatic Diseases', MS='Diseases that do not exhibit symptoms.'),
  'D058085': NtO(MH='Iron Compounds', MS='Organic and inorganic compounds that contain iron as an integral part of the molecule.'),
  'D058086': NtO(MH='Microalgae', MS='A non-taxonomic term for unicellular microscopic algae which are found in both freshwater and marine environments. Some authors consider DIATOMS; CYANOBACTERIA; HAPTOPHYTA; and DINOFLAGELLATES as part of microalgae, even though they are not algae.'),
  'D058087': NtO(MH='Haptophyta', MS='A group (or phylum) of unicellular EUKARYOTA (or algae) possessing CHLOROPLASTS and FLAGELLA.'),
  'D058088': NtO(MH='Sarcoglycanopathies', MS='Deficiencies or mutations in the genes for the SARCOGLYCAN COMPLEX subunits. A variety of phenotypes are associated with these mutations including a subgroup of autosomal recessive limb girdle muscular dystrophies, cardiomyopathies, and respiratory deficiency.'),
  'D058105': NtO(MH='Polymerization', MS='Chemical reaction in which monomeric components are combined to form POLYMERS (e.g., POLYMETHYLMETHACRYLATE).'),
  'D058106': NtO(MH='Wound Closure Techniques', MS='Methods to repair breaks in tissue caused by trauma or to close surgical incisions.'),
  'D058107': NtO(MH='Abdominal Wound Closure Techniques', MS='Methods to repair breaks in abdominal tissues caused by trauma or to close surgical incisions during abdominal surgery.'),
  'D058108': NtO(MH='Glaucophyta', MS='A phylum or class of freshwater microscopic EUKARYOTA considered to be plants or MICROALGAE. Their CHLOROPLASTS (called cyanelles) are believed to be derived from the direct endosymbiosis of CYANOBACTERIA.'),
  'D058109': NtO(MH='Airway Management', MS="Evaluation, planning, and use of a range of procedures and airway devices for the maintenance or restoration of a patient's ventilation."),
  'D058110': NtO(MH='Counterfeit Drugs', MS='Fake medicine that may be contaminated or contain the wrong or no active ingredient, or the correct active ingredient at the wrong dose. Counterfeit drugs are illegal and may be harmful to health.'),
  'D058112': NtO(MH='Septins', MS='A family of GTP-binding proteins that were initially identified in YEASTS where they were shown to initiate the process of septation and bud formation. Septins form into hetero-oligomeric complexes that are comprised of several distinct septin subunits. These complexes can act as cytoskeletal elements that play important roles in CYTOKINESIS, cytoskeletal reorganization, BIOLOGICAL TRANSPORT, and membrane dynamics.'),
  'D058113': NtO(MH='Cerumenolytic Agents', MS='Compounds that disintegrate natural waxy exudates such as EAR WAX.'),
  'D058114': NtO(MH='Desmidiales', MS='An order of mostly unicellular, microscopic, freshwater algae that have highly symmetrical and varied shapes.'),
  'D058115': NtO(MH='Lewis Bases', MS='Any chemical species which acts as an electron-pair donor in a chemical bonding reaction with a LEWIS ACID.'),
  'D058116': NtO(MH='Lewis Acids', MS='Any chemical species which accepts an electron-pair from a LEWIS BASE in a chemical bonding reaction.'),
  'D058117': NtO(MH='Neural Prostheses', MS='Medical devices which substitute for a nervous system function by electrically stimulating the nerves directly and monitoring the response to the electrical stimulation.'),
  'D058118': NtO(MH='Transcription Factor 7-Like 2 Protein', MS='A transcription factor that takes part in WNT signaling pathway. The activity of the protein is regulated via its interaction with BETA CATENIN. Transcription factor 7-like 2 protein plays an important role in the embryogenesis of the PANCREAS and ISLET CELLS.'),
  'D058119': NtO(MH='Closterium', MS='A genus of crescent-shaped algae in the family Closteriaceae, order DESMIDIALES.'),
  'D058120': NtO(MH='Micrasterias', MS='A genus of unicellular streptophyte algae in the family Desmidiaceae, order DESMIDIALES.'),
  'D058125': NtO(MH='Transcription Factor 7-Like 1 Protein', MS='A transcription factor that takes part in WNT signaling pathway where it may play a role in the differentiation of KERATINOCYTES. The transcriptional activity of this protein is regulated via its interaction with BETA CATENIN.'),
  'D058126': NtO(MH='Balanced Anesthesia', MS='The use of several ANESTHETICS together in small amounts, as opposed to a larger amount of one drug, to induce loss of sensation and loss of consciousness.'),
  'D058127': NtO(MH='Transcription Factor 3', MS='A basic helix-loop-helix transcription factor that plays a role in determining cell fate during embryogenesis. It forms a heterodimer with TWIST-RELATED PROTEIN 1 and ACHAETE-SCUTE GENE COMPLEX-related TRANSCRIPTION FACTORS.'),
  'D058128': NtO(MH='Compression Bandages', MS='Strips of elastic material used to apply pressure to body parts to control EDEMA and aid circulation.'),
  'D058129': NtO(MH='Zygnematales', MS='An order of freshwater algae possessing unbranched filaments. Sexual reproduction takes place via conjugation.'),
  'D058130': NtO(MH='Mougeotia', MS='A genus of filamentous algae in the order ZYGNEMATALES, family Zygnemataceae, that possess CHLOROPLASTS that can orient to the direction of light.'),
  'D058131': NtO(MH='Spirogyra', MS='A genus of filamentous algae in the order ZYGNEMATALES, family Zygnemataceae, named for the helical arrangement of its CHLOROPLASTS. It is commonly found in freshwater habitats.'),
  'D058145': NtO(MH='Achaete-Scute Complex Genome Region', MS='A genomic region found in DROSOPHILA. The region contains genes encoding BASIC HELIX-LOOP-HELIX TRANSCRIPTION FACTORS that play a critical role in the regulation of pattern formation during EMBRYONIC DEVELOPMENT.'),
  'D058165': NtO(MH='22q11 Deletion Syndrome', MS='Condition with a variable constellation of phenotypes due to deletion polymorphisms at chromosome location 22q11. It encompasses several syndromes with overlapping abnormalities including the DIGEORGE SYNDROME, VELOCARDIOFACIAL SYNDROME, and CONOTRUNCAL AMOMALY FACE SYNDROME. In addition, variable developmental problems and schizoid features are also associated with this syndrome. (From BMC Med Genet. 2009 Feb 25;10:16) Not all deletions at 22q11 result in the 22q11deletion syndrome.'),
  'D058185': NtO(MH='Magnetite Nanoparticles', MS='Synthesized magnetic particles under 100 nanometers possessing many biomedical applications including DRUG DELIVERY SYSTEMS and CONTRAST AGENTS. The particles are usually coated with a variety of polymeric compounds.'),
  'D058186': NtO(MH='Acute Kidney Injury', MS='Abrupt reduction in kidney function. Acute kidney injury encompasses the entire spectrum of the syndrome including acute kidney failure; ACUTE KIDNEY TUBULAR NECROSIS; and other less severe conditions.'),
  'D058205': NtO(MH='Tooth Bleaching Agents', MS='Chemicals that are used to oxidize pigments in TEETH and thus effect whitening.'),
  'D058206': NtO(MH='Hair Bleaching Agents', MS='Chemicals that are used to oxidize pigments in HAIR.'),
  'D058207': NtO(MH='Sumoylation', MS='A type of POST-TRANSLATIONAL PROTEIN MODIFICATION by SMALL UBIQUITIN-RELATED MODIFIER PROTEINS (also known as SUMO proteins).'),
  'D058225': NtO(MH='Plaque, Amyloid', MS='Accumulations of extracellularly deposited AMYLOID FIBRILS within tissues.'),
  'D058226': NtO(MH='Plaque, Atherosclerotic', MS='Lesions formed within the walls of ARTERIES.'),
  'D058227': NtO(MH='Amyloidogenic Proteins', MS='Proteins that form the core of amyloid fibrils. For example, the core of amyloid A is formed from amyloid A protein, also known as serum amyloid A protein or SAA protein.'),
  'D058228': NtO(MH='Islet Amyloid Polypeptide', MS='A pancreatic beta-cell hormone that is co-secreted with INSULIN. It displays an anorectic effect on nutrient metabolism by inhibiting gastric acid secretion, gastric emptying and postprandial GLUCAGON secretion. Islet amyloid polypeptide can fold into AMYLOID FIBRILS that have been found as a major constituent of pancreatic AMYLOID DEPOSITS.'),
  'D058245': NtO(MH='Cognitive Reserve', MS='Capacity that enables an individual to cope with and/or recover from the impact of a neural injury or a psychotic episode.'),
  'D058246': NtO(MH='Prehypertension', MS='Blood pressure levels that are between normotension and hypertension. Individuals with prehypertension are at a higher risk for developing cardiovascular diseases. Generally, prehypertension is defined as SYSTOLIC PRESSURE of 131-139 mm Hg and/or DIASTOLIC PRESSURE of 81-89 when the optimal is 120/80 mm Hg. For diabetics and other metabolism diseases the prehypertension is around 110-129/70-79 mm Hg.'),
  'D058247': NtO(MH='Loose Anagen Hair Syndrome', MS='Benign childhood alopecia that improves spontaneously with aging. It is characterized by anagen hairs (misshapen hair bulbs and absent inner and outer root sheaths), thin, and sparse hairs that pulls out easily.'),
  'D058248': NtO(MH='Serum Bactericidal Antibody Assay', MS='Procedures for identification and measurement of IMMUNOGLOBULINS in the blood that initiate lysis of bacteria.'),
  'D058249': NtO(MH='Birt-Hogg-Dube Syndrome', MS='Autosomal dominant neoplastic syndrome characterised by genodermatosis, lung cysts, spontaneous and recurrent PNEUMOTHORAX; and RENAL CANCER. It is associated with mutations in the folliculin protein gene (FLCN protein).'),
  'D058250': NtO(MH='Dental Restoration Repair', MS='The process of repairing broken or worn parts of a PERMANENT DENTAL RESTORATION.'),
  'D058251': NtO(MH='Ctenocephalides', MS='A genus of FLEAS in the family Pulicidae. It includes the cat flea (Ctenocephalides felis), one of the most common species on earth.'),
  'D058252': NtO(MH='Necrobiotic Xanthogranuloma', MS='A cutaneous necrobiotic disorder characterized by firm, yellow plaques or nodules, often in a periorbital distribution. It is often accompanied by an elevated ERYTHROCYTE SEDIMENTATION RATE; LEUKOPENIA; and MONOCLONAL GAMMOPATHY (IgG-kappa type) and systemic involvement.'),
  'D058253': NtO(MH='Tunga', MS='A genus of parasitic fleas in the family Pulicidae. They are found mainly in Africa, South America, and Central America.'),
  'D058254': NtO(MH='Stretchers', MS='Bed-like structures for transporting or temporarily holding patients.'),
  'D058255': NtO(MH='Protease Nexins', MS='Extracellular protease inhibitors that are secreted from FIBROBLASTS. They form a covalent complex with SERINE PROTEASES and can mediate their cellular internalization and degradation.'),
  'D058256': NtO(MH='Brain Waves', MS='Wave-like oscillations of electric potential between parts of the brain recorded by EEG.'),
  'D058257': NtO(MH='Surgical Attire', MS='Sterile clothing worn during surgical procedures to protect the surgical site from sources of contamination.'),
  'D058258': NtO(MH='Serpin E2', MS='A protease nexin and serpin subtype that is specific for several SERINE PROTEASES including UROKINASE; THROMBIN; TRYPSIN; and PLASMINOGEN ACTIVATORS.'),
  'D058260': NtO(MH='Xenopsylla', MS='A genus of fleas in the family Pulicidae which includes the species that serves as the primary vector of BUBONIC PLAGUE, Xenopsylla cheopis.'),
  'D058261': NtO(MH='Receptor Activity-Modifying Proteins', MS='A family of proteins that bind to CELL SURFACE RECEPTORS and alter their specificity, signaling mechanism, or mode of intracellular transport.'),
  'D058262': NtO(MH='Receptor Activity-Modifying Protein 1', MS='A receptor activity-modifying protein that is a subunit of specific G-PROTEIN COUPLED RECEPTORS. The CALCITONIN GENE-RELATED PEPTIDE RECEPTOR is formed from a dimer of this protein and CALCITONIN RECEPTOR-LIKE PROTEIN, while an isoform of the ISLET AMYLOID POLYPEPTIDE RECEPTOR is formed from this protein dimerizing with the CALCITONIN RECEPTOR.'),
  'D058263': NtO(MH='Receptor Activity-Modifying Protein 2', MS='A receptor activity-modifying protein that heterodimerizes with CALCITONIN RECEPTOR-LIKE PROTEIN to form the ADRENOMEDULLIN RECEPTOR. In addition, an isoform of the ISLET AMYLOID POLYPEPTIDE RECEPTOR is formed from this protein dimerizing with the CALCITONIN RECEPTOR.'),
  'D058264': NtO(MH='Receptor Activity-Modifying Protein 3', MS='A receptor activity-modifying protein that heterodimerizes with CALCITONIN RECEPTOR-LIKE PROTEIN to form the ADRENOMEDULLIN RECEPTOR. In addition, an isoform of the ISLET AMYLOID POLYPEPTIDE RECEPTOR is formed from this protein dimerizing with the CALCITONIN RECEPTOR.'),
  'D058265': NtO(MH='Receptors, Adrenomedullin', MS='G-protein-coupled cell surface receptors for ADRENOMEDULLIN. They are formed by the heterodimerization of CALCITONIN RECEPTOR-LIKE PROTEIN and either RECEPTOR ACTIVITY-MODIFYING PROTEIN 2 or RECEPTOR ACTIVITY-MODIFYING PROTEIN 3.'),
  'D058266': NtO(MH='Dielectric Spectroscopy', MS='A technique of measuring the dielectric properties of materials, which vary over a range of frequencies depending on the physical properties of the material. The technique involves measuring, over a range of frequencies, ELECTRICAL IMPEDANCE and phase shift of an electric field as it passes through the material.'),
  'D058267': NtO(MH='Flea Infestations', MS='Parasitic attack by members of the order SIPHONAPTERA.'),
  'D058285': NtO(MH='Tungiasis', MS='An infestation with the flea TUNGA PENETRANS causing inflammation, pruritus, and pain, in both humans and other mammals. There is a high incidence of secondary infections such as BACTEREMIA and TETANUS.'),
  'D058286': NtO(MH='Calcitonin Receptor-Like Protein', MS='A receptor protein that is associated with RECEPTOR ACTIVITY-MODIFYING PROTEINS. When bound to RECEPTOR ACTIVITY-MODIFYING PROTEIN 1 it forms the CALCITONIN GENE-RELATED RECEPTOR. When bound to RECEPTOR ACTIVITY-MODIFYING PROTEIN 2 or RECEPTOR ACTIVITY-MODIFYING PROTEIN 3 it forms the ADRENOMEDULLIN RECEPTOR.'),
  'D058287': NtO(MH='Receptors, Islet Amyloid Polypeptide', MS='G-protein coupled receptors that are formed through the dimerization of the CALCITONIN RECEPTOR with a RECEPTOR ACTIVITY-MODIFYING PROTEIN. Their affinity for ISLET AMYLOID POLYPEPTIDE is dependent upon which of several receptor activity-modifying protein subtypes they are bound to.'),
  'D058288': NtO(MH="Sister Mary Joseph's Nodule", MS='Metastatic lesion of the UMBILICUS associated with intra-abdominal neoplasms especially of the GASTROINTESTINAL TRACT or OVARY.'),
  'D058305': NtO(MH='Sorting Nexins', MS='A large family of phosphatidylinositol phosphate-binding proteins that are involved in mediating intracellular transport and sorting of proteins via a variety of endocytic pathways.'),
  'D058306': NtO(MH='Receptors, Prostaglandin E, EP1 Subtype', MS='A subtype of prostaglandin E receptors that specifically couples to GTP-BINDING PROTEIN ALPHA SUBUNIT, GQ and the subsequently activates TYPE C PHOSPHOLIPASES. Additional evidence has shown that the receptor can act through a calcium-dependent signaling pathway.'),
  'D058307': NtO(MH='Receptors, Prostaglandin E, EP2 Subtype', MS='A subtype of prostaglandin E receptors that specifically couples to GS ALPHA GTP-BINDING PROTEIN SUBUNITS and subsequently activates ADENYLYL CYCLASES.'),
  'D058308': NtO(MH='Receptors, Prostaglandin E, EP3 Subtype', MS='A subtype of prostaglandin E receptors that specifically couples to GTP-BINDING PROTEIN ALPHA SUBUNIT, GI and subsequently inhibits ADENYLYL CYCLASES.'),
  'D058309': NtO(MH='Receptors, Prostaglandin E, EP4 Subtype', MS='A subtype of prostaglandin E receptors that specifically couples to GS ALPHA GTP-BINDING PROTEIN SUBUNITS and subsequently activates ADENYLYL CYCLASES. The receptor may also signal through the activation of PHOSPHATIDYLINOSITOL 3-KINASE.'),
  'D058325': NtO(MH='Amblycera', MS='A suborder of chewing lice, in the order PHTHIRAPTERA, that are ectoparasites of poultry and some mammals.'),
  'D058326': NtO(MH='Ischnocera', MS='A suborder of chewing lice, in the order PHTHIRAPTERA, that are parasites of birds and mammals.'),
  'D058327': NtO(MH='Norwood Procedures', MS='A set of surgical procedures performed to establish sufficient outflow to the systemic circulation in individuals with univentricular congenital heart malformations, such as HYPOPLASTIC LEFT HEART SYNDROME, and MITRAL VALVE atresia, associated with systemic outflow obstruction. Follow-on surgeries may be performed and consist of a HEMI-FONTAN PROCEDURE as the stage 2 Norwood procedure and a FONTAN PROCEDURE as the stage 3 Norwood procedure.'),
  'D058345': NtO(MH='Asymptomatic Infections', MS='Infections that do not exhibit symptoms.'),
  'D058365': NtO(MH='Candidiasis, Invasive', MS='An important nosocomial fungal infection with species of the genus CANDIDA, most frequently CANDIDA ALBICANS. Invasive candidiasis occurs when candidiasis goes beyond a superficial infection and manifests as CANDIDEMIA, deep tissue infection, or disseminated disease with deep organ involvement.'),
  'D058366': NtO(MH='Blalock-Taussig Procedure', MS='A cardiovascular procedure performed to create a blood supply to the PULMONARY CIRCULATION. It involves making a connection between the subclavian, or carotid branch of the AORTA, or the AORTIC ARCH to the PULMONARY ARTERY.'),
  'D058385': NtO(MH='Cardiac Valve Annuloplasty', MS='A type of heart valve surgery that involves the repair, replacement, or reconstruction of the annuli of HEART VALVES. It includes shortening the circumference of the annulus to improve valve closing capacity and reinforcing the annulus as a step in more complex valve repairs.'),
  'D058386': NtO(MH='Mitral Valve Annuloplasty', MS='A type of heart valve surgery that involves the repair, replacement, or reconstruction of the annulus of the MITRAL VALVE. It includes shortening the circumference of the annulus to improve valve closing capacity and reinforcing the annulus as a step in more complex valve repairs.'),
  'D058387': NtO(MH='Candidemia', MS='A form of invasive candidiasis where species of CANDIDA are present in the blood.'),
  'D058405': NtO(MH='Desmoplastic Small Round Cell Tumor', MS='A rare, aggressive soft tissue sarcoma that primarily affects adolescents and young adults. It is most commonly found in the abdomen.'),
  'D058406': NtO(MH='Cardiac Resynchronization Therapy', MS='The restoration of the sequential order of contraction and relaxation of the HEART ATRIA and HEART VENTRICLES by atrio-biventricular pacing.'),
  'D058407': NtO(MH='Electroencephalography Phase Synchronization', MS='Adjustment of BRAIN WAVES from two or more neuronal groups within or across a brain structure (e.g., cortical and limbic brain structures) to become uniform in EEG oscillation patterns in response to a stimulus. It is interpreted as a brain integration sign during many processes such as learning, memory, and perception and involves reciprocal neural connections.'),
  'D058408': NtO(MH='Th1-Th2 Balance', MS="Homeostatic control of the IMMUNE SYSTEM by different CYTOKINES that induce the differentiation of HELPER T-CELLS into TH1 and TH2 CELLS and result in a balance or imbalance (leading to disease) of Th1 and Th2 cells' immune responses."),
  'D058409': NtO(MH='Cardiac Resynchronization Therapy Devices', MS='Types of artificial pacemakers with implantable leads to be placed at multiple intracardial sites. They are used to treat various cardiac conduction disturbances which interfere with the timing of contraction of the ventricles. They may or may not include defibrillating electrodes (IMPLANTABLE DEFIBRILLATORS) as well.'),
  'D058425': NtO(MH='Vaccines, Virus-Like Particle', MS='Vaccines using supra-molecular structures composed of multiple copies of recombinantly expressed viral structural proteins. They are often antigentically indistinguishable from the virus from which they were derived.'),
  'D058426': NtO(MH='Neointima', MS='The new and thickened layer of scar tissue that forms on a PROSTHESIS, or as a result of vessel injury especially following ANGIOPLASTY or stent placement.'),
  'D058427': NtO(MH='Hygroscopic Agents', MS='Materials that readily absorb moisture from their surroundings.'),
  'D058428': NtO(MH='Silica Gel', MS='A non-crystalline form of silicon oxide that has absorptive properties. It is commonly used as a desiccating agent and as a stationary phase for CHROMATOGRAPHY. The fully hydrated form of silica gel has distinct properties and is referred to as SILICIC ACID.'),
  'D058429': NtO(MH='Gnathostomiasis', MS='Infections with nematodes of the genus GNATHOSTOMA, superfamily THELAZIOIDEA. Gnathostomiasis is a food-borne zoonosis caused by eating undercooked or raw fish or meat.'),
  'D058430': NtO(MH='Humeral Head', MS='The upper rounded extremity of the humerus fitting into the GLENOID CAVITY of the SCAPULA.'),
  'D058431': NtO(MH='Brachypodium', MS='A plant genus in the family POACEAE. Brachypodium distachyon is a model species for functional genomics studies.'),
  'D058432': NtO(MH='Sporangia', MS='A structure found in plants, fungi, and algae, that produces and contains spores.'),
  'D058433': NtO(MH='Photobioreactors', MS='Devices for generating biological products that use light as the energy source. They are used for controlled BIOMASS production such as growing cyanobacteria, mosses, or algae.'),
  'D058436': NtO(MH='Knowledge Management', MS='The leveraging of collective wisdom within an organization as a catalyst to increase responsiveness and innovation.'),
  'D058437': NtO(MH='Hypertensive Retinopathy', MS='Degenerative changes to the RETINA due to HYPERTENSION.'),
  'D058438': NtO(MH='Water Cycle', MS='Circulation of water among various ecological systems, in various states, on, above, and below the surface of the earth.'),
  'D058439': NtO(MH='Plant Somatic Embryogenesis Techniques', MS='The process of embryo initiation in culture from vegetative, non-gametic, sporophytic, or somatic plant cells.'),
  'D058440': NtO(MH='Denitrification', MS='Nitrate reduction process generally mediated by anaerobic bacteria by which nitrogen available to plants is converted to a gaseous form and lost from the soil or water column. It is a part of the nitrogen cycle.'),
  'D058441': NtO(MH='Rhizosphere', MS='The immediate physical zone surrounding plant roots that include the plant roots. It is an area of intense and complex biological activity involving plants, microorganisms, other soil organisms, and the soil.'),
  'D058442': NtO(MH='Capsule Opacification', MS='Clouding or loss of transparency of the posterior lens capsule, usually following CATARACT extraction.'),
  'D058443': NtO(MH='Catastrophization', MS='Cognitive and emotional processes encompassing magnification of pain-related stimuli, feelings of helplessness, and a generally pessimistic orientation.'),
  'D058445': NtO(MH='Bullying', MS='Aggressive behavior by a more powerful party that results in physical harm to or emotional distress of the victim.'),
  'D058446': NtO(MH='Ligand-Gated Ion Channels', MS='A subclass of ion channels that open or close in response to the binding of specific LIGANDS.'),
  'D058447': NtO(MH='Eye Pain', MS='A dull or sharp painful sensation associated with the outer or inner structures of the eyeball, having different causes.'),
  'D058448': NtO(MH='Biota', MS='The spectrum of different living organisms inhabiting a particular region, habitat, or biotope.'),
  'D058449': NtO(MH='Intravitreal Injections', MS='The administration of substances into the VITREOUS BODY of the eye with a hypodermic syringe.'),
  'D058450': NtO(MH='Endotamponade', MS='A method of stopping internal bleeding or blood flow, or the closure of a wound or body cavity, achieved by applying pressure or introducing an absorbent liquid, gel, or tampon.'),
  'D058451': NtO(MH='Oxidative Coupling', MS='The reaction of two molecular entities via oxidation usually catalyzed by a transition metal compound and involving dioxygen as the oxidant.'),
  'D058452': NtO(MH='Axial Length, Eye', MS='The distance between the anterior and posterior poles of the eye, measured either by ULTRASONOGRAPHY or by partial coherence interferometry.'),
  'D058453': NtO(MH='Emmetropia', MS='The condition of where images are correctly brought to a focus on the retina.'),
  'D058456': NtO(MH='Retinal Telangiectasis', MS='A group of rare, idiopathic, congenital retinal vascular anomalies affecting the retinal capillaries. It is characterized by dilation and tortuosity of retinal vessels and formation of multiple aneurysms, with different degrees of leakage and exudates emanating from the blood vessels.'),
  'D058457': NtO(MH='Trichiasis', MS='A disease of the eye in which the eyelashes abnormally turn inwards toward the eyeball producing constant irritation caused by motion of the lids.'),
  'D058458': NtO(MH='Nitrogen Cycle', MS='The circulation of nitrogen in nature, consisting of a cycle of biochemical reactions in which atmospheric nitrogen is compounded, dissolved in rain, and deposited in the soil, where it is assimilated and metabolized by animals, plants, bacteria, and fungi, eventually returning to the atmosphere by bacterial decomposition of organic matter.'),
  'D058465': NtO(MH='Nitrification', MS='A process facilitated by specialized bacteria involving the oxidation of ammonium to nitrite and nitrate.'),
  'D058466': NtO(MH='Cysteine Loop Ligand-Gated Ion Channel Receptors', MS='A subfamily of ligand-gated ion channel receptors that share a characteristic loop which is formed by a disulfide bond between two CYSTEINE residues. These receptors typically contain five subunits with the cysteine-loop occurring near an N-terminal extracellular domain.'),
  'D058467': NtO(MH='Tardigrada', MS='A phylum of microscopic ecdysozoan invertebrates, closely related to ARTHROPODS. Members exhibit anabiosis and cryptobiosis, dormant states where metabolic activity is reduced or absent, thus making them tolerant to extreme environmental conditions. They are distributed worldwide and most are semi-aquatic.'),
  'D058468': NtO(MH='Receptors, Ionotropic Glutamate', MS='A class of ligand-gated ion channel receptors that have specificity for GLUTAMATE. They are distinct from METABOTROPIC GLUTAMATE RECEPTORS which act through a G-protein-coupled mechanism.'),
  'D058469': NtO(MH='Receptors, Purinergic P2X', MS='A subclass of purinergic P2 receptors that signal by means of a ligand-gated ion channel. They are comprised of three P2X subunits which can be identical (homotrimeric form) or dissimilar (heterotrimeric form).'),
  'D058470': NtO(MH='Receptors, Purinergic P2Y', MS='A subclass of purinergic P2 receptors whose signaling is coupled through a G-PROTEIN signaling mechanism.'),
  'D058471': NtO(MH='Subretinal Fluid', MS='An exudate between the RETINA and CHOROID from various sources including the vitreous cavity, SUBARACHNOID SPACE, or abnormal vessels.'),
  'D058472': NtO(MH='Receptors, Purinergic P2Y1', MS='A subclass of purinergic P2Y receptors that have a preference for ATP and ADP. The activated P2Y1 receptor signals through the G-PROTEIN-coupled activation of PHOSPHOLIPASE C and mobilization of intracellular CALCIUM.'),
  'D058473': NtO(MH='Receptors, Purinergic P2Y2', MS='A subclass of purinergic P2Y receptors that have a preference for ATP and UTP. The activated P2Y2 receptor acts through a G-PROTEIN-coupled PHOSPHATIDYLINOSITOL and intracellular CALCIUM SIGNALING pathway.'),
  'D058474': NtO(MH='Receptors, Purinergic P2X1', MS='A purinergic P2X neurotransmitter receptor found at sympathetically innervated SMOOTH MUSCLE. It may play a functional role regulating the juxtoglomerular apparatus of the KIDNEY.'),
  'D058475': NtO(MH='Tenon Capsule', MS='Sheath of the eyeball consisting of fascia extending from the OPTIC NERVE to the corneal limbus.'),
  'D058476': NtO(MH='Receptors, Purinergic P2X2', MS='A purinergic P2X neurotransmitter receptor involved in sensory signaling of TASTE PERCEPTION, chemoreception, visceral distension and NEUROPATHIC PAIN. The receptor comprises three P2X2 subunits. The P2X2 subunits also have been found associated with P2X3 RECEPTOR subunits in a heterotrimeric receptor variant.'),
  'D058477': NtO(MH='Receptors, Purinergic P2X3', MS='A purinergic P2X neurotransmitter receptor involved in sensory signaling of TASTE PERCEPTION, chemoreception, visceral distension, and NEUROPATHIC PAIN. The receptor comprises three P2X3 subunits. The P2X3 subunits are also associated with P2X2 RECEPTOR subunits in a heterotrimeric receptor variant.'),
  'D058485': NtO(MH='Receptors, Purinergic P2X5', MS='A purinergic P2X neurotransmitter receptor found at high levels in the BRAIN and IMMUNE SYSTEM.'),
  'D058486': NtO(MH='Receptors, Purinergic P2X7', MS='A purinergic P2X neurotransmitter receptor that plays a role in pain sensation signaling and regulation of inflammatory processes.'),
  'D058487': NtO(MH='Cercaria', MS='The free-swimming larval forms of parasites found in an intermediate host.'),
  'D058488': NtO(MH='Metacercariae', MS='Encysted cercaria which house the intermediate stages of trematode parasites in tissues of an intermediate host.'),
  'D058489': NtO(MH='46, XX Disorders of Sex Development', MS='Congenital conditions in individuals with a female karyotype, in which the development of the gonadal or anatomical sex is atypical.'),
  'D058490': NtO(MH='Disorder of Sex Development, 46,XY', MS='Congenital conditions in individuals with a male karyotype, in which the development of the gonadal or anatomical sex is atypical.'),
  'D058491': NtO(MH='Bacterial Load', MS='Measurable quantity of bacteria in an object, organism, or organism compartment.'),
  'D058492': NtO(MH='Drug Repositioning', MS='The deliberate and methodical practice of finding new applications for existing drugs.'),
  'D058493': NtO(MH='Sarcophagidae', MS='Family of flies in the order DIPTERA, commonly known as flesh flies. They lay their eggs in dead or decaying matter or open wounds.'),
  'D058494': NtO(MH='Walker-Warburg Syndrome', MS='Rare autosomal recessive lissencephaly type 2 associated with congenital MUSCULAR DYSTROPHY and eye anomalies (e.g., RETINAL DETACHMENT; CATARACT; MICROPHTHALMOS). It is often associated with additional brain malformations such as HYDROCEPHALY and cerebellar hypoplasia and is the most severe form of the group of related syndromes (alpha-dystroglycanopathies) with common congenital abnormalities in the brain, eye and muscle development.'),
  'D058495': NtO(MH='Sotos Syndrome', MS='Congenital or postnatal overgrowth syndrome most often in height and occipitofrontal circumference with variable delayed motor and cognitive development. Other associated features include advanced bone age, seizures, NEONATAL JAUNDICE; HYPOTONIA; and SCOLIOSIS. It is also associated with increased risk of developing neoplasms in adulthood. Mutations in the NSD1 protein and its HAPLOINSUFFICIENCY are associated with the syndrome.'),
  'D058496': NtO(MH='Smith-Magenis Syndrome', MS='Complex neurobehavioral disorder characterized by distinctive facial features (FACIES), developmental delay and INTELLECTUAL DISABILITY. Behavioral phenotypes include sleep disturbance, maladaptive, self-injurious and attention-seeking behaviors. The sleep disturbance is linked to an abnormal circadian secretion pattern of MELATONIN. The syndrome is associated with de novo deletion or mutation and HAPLOINSUFFICIENCY of the retinoic acid-induced 1 protein on chromosome 17p11.2.'),
  'D058497': NtO(MH='Fraser Syndrome', MS='Rare autosomal recessive congenital malformation syndrome characterized by cryptophthalmos, SYNDACTYLY and UROGENITAL ABNORMALITIES. Other anomalies of bone, ear, lung, and nose are common. Mutations on FRAS1 and FREM2 are associated with the syndrome.'),
  'D058498': NtO(MH='Kyphoplasty', MS='Procedures to restore vertebrae to their original shape following vertebral compression fractures by inflating a balloon inserted into the vertebrae, followed by removal of the balloon and injection of BONE CEMENTS to fill the cavity.'),
  'D058499': NtO(MH='Retinal Dystrophies', MS='A group of disorders involving predominantly the posterior portion of the ocular fundus, due to degeneration in the sensory layer of the RETINA; RETINAL PIGMENT EPITHELIUM; BRUCH MEMBRANE; CHOROID; or a combination of these tissues.'),
  'D058500': NtO(MH='Animal Fins', MS='Membranous appendage of fish and other aquatic organisms used for locomotion or balance.'),
  'D058501': NtO(MH='Enzyme-Linked Immunospot Assay', MS='A method of detection of the number of cells in a sample secreting a specific molecule. With this method, a population of cells are plated over top of the immunosorbent substrate that captures the secreted molecules.'),
  'D058502': NtO(MH='Pentalogy of Cantrell', MS='Rare congenital deformity syndrome characterized by a combination of five anomalies as a result of neural tube defect. The five anomalies are a midline supraumbilical abdominal wall defect (e.g., OMPHALOCELE), a lower STERNUM defect, a congenital intracardiac defect, an anterior DIAPHRAGM defect, and a diaphragmatic PERICARDIUM defect (e.g., PERICARDIAL EFFUSION). Variants with incomplete and variable combinations of the defects are known. ECTOPIA CORDIS; CLEFT LIP; and CLEFT PALATE are often associated with the syndrome.'),
  'D058503': NtO(MH='Mesophyll Cells', MS='Large and highly vacuolated cells possessing many chloroplasts occuring in the interior cross-section of leaves, juxtaposed between the epidermal layers.'),
  'D058504': NtO(MH='Th17 Cells', MS='A subset of helper-effector T-lymphocytes which synthesize and secrete INTERLEUKINS IL-17; IL-17F; and IL-22. These cytokines are involved in host defenses and tissue inflammation in autoimmune diseases.'),
  'D058506': NtO(MH='Cambium', MS='A layer of living cells between the bark and hardwood that each year produces additional wood and bark cells, forming concentric growth rings.'),
  'D058507': NtO(MH='Host Specificity', MS='The properties of a pathogen that makes it capable of infecting one or more specific hosts. The pathogen can include PARASITES as well as VIRUSES; BACTERIA; FUNGI; or PLANTS.'),
  'D058525': NtO(MH='Quartz Crystal Microbalance Techniques', MS='The use of a quartz crystal microbalance for measuring weights and forces in the micro- to nanogram range. It is used to study the chemical and mechanical properties of thin layers, such as polymer coatings and lipid membranes; and interactions between molecues.'),
  'D058526': NtO(MH='Plant Vascular Bundle', MS='A strand of primary conductive plant tissue consisting essentially of XYLEM, PHLOEM, and CAMBIUM.'),
  'D058527': NtO(MH='Enteropathy-Associated T-Cell Lymphoma', MS='A primary peripheral T-cell lymphoma in the gastrointestinal tract, most often in the jejunum, associated with a history of CELIAC DISEASE or other gastrointestinal diseases.'),
  'D058529': NtO(MH='Single Umbilical Artery', MS='Congenital abnormality where one, instead of the usual two, UMBILICAL ARTERY connects the fetus to the placenta.'),
  'D058530': NtO(MH='Datura metel', MS='A plant species and perennial herb of the genus DATURA, family SOLANACEAE, containing poisonous tropane ALKALOIDS.'),
  'D058531': NtO(MH='46, XX Testicular Disorders of Sex Development', MS='Congenital conditions in individuals in which male GONADS develop in a genetic female (female to male sex reversal).'),
  'D058533': NtO(MH='Sex Chromosome Disorders of Sex Development', MS='Congenital conditions of atypical sexual development associated with abnormal sex chromosome constitutions including MONOSOMY; TRISOMY; and MOSAICISM.'),
  'D058534': NtO(MH='Class I Phosphatidylinositol 3-Kinases', MS='A phosphatidylinositol 3-kinase subclass that includes enzymes with a specificity for 1-phosphatidylinositol, 1-phosphatidylinositol 4-phosphate, and 1-phosphatidylinositol 4,5-bisphosphate. Members of this enzyme subclass are activated by cell surface receptors and occur as heterodimers of enzymatic and regulatory subunits.'),
  'D058535': NtO(MH='Echogenic Bowel', MS='A PRENATAL ULTRASONOGRAPHY finding of excessively dense fetal bowel due to MECONIUM buildup.'),
  'D058536': NtO(MH='Pyelectasis', MS='Dilation of fetal KIDNEY PELVIS. It is a common PRENATAL ULTRASONOGRAPHY finding with no significant long-term sequelae.'),
  'D058537': NtO(MH='Electronic Supplementary Materials', MS='Supporting content or information, such as animation, datasets, multimedia files, video, movies, audio files, text files, or software, which is submitted for publication in an online journal or an online edition of a journal. This information may be referenced in the text of the article with a link to the supplementary data provided. CATALOG: do not use'),
  'D058538': NtO(MH='Phosphatidylinositol-4-Phosphate 3-Kinase', MS='A phosphatidylinositol 3-kinase that catalyzes the conversion of 1-phosphatidylinositol 4-phosphate into phosphatidylinositol 3,4-bisphosphate.'),
  'D058539': NtO(MH='Phosphatidylinositol 3-Kinase', MS='A phosphatidylinositol 3-kinase that catalyzes the conversion of 1-phosphatidylinositol into 1-phosphatidylinositol 3-phosphate.'),
  'D058540': NtO(MH='Aicardi Syndrome', MS='A rare genetic disorder characterized by partial or complete absence of the CORPUS CALLOSUM, resulting in infantile spasms, MENTAL RETARDATION, and lesions of the RETINA or OPTIC NERVE.'),
  'D058541': NtO(MH='Class II Phosphatidylinositol 3-Kinases', MS='A subclass of phosphatidylinositol 3-kinases that have specificity for 1-phosphatidylinositol and 1-phosphatidylinositol 4-phosphate. Members of this subclass consist of a single subunit structure and are regulated by RECEPTOR TYROSINE KINASES; CYTOKINE RECEPTORS; and INTEGRINS.'),
  'D058542': NtO(MH='Implantable Neurostimulators', MS='Surgically placed electric conductors through which ELECTRIC STIMULATION of nerve tissue is delivered.'),
  'D058543': NtO(MH='Class Ia Phosphatidylinositol 3-Kinase', MS='A phosphatidylinositol 3-kinase subclass that includes enzymes formed through the heterodimerization of a p110 catalytic and a p85, p55, or p50 regulatory subunit. This subclass of enzymes is a downstream target of TYROSINE KINASE RECEPTORS and G PROTEIN-COUPLED RECEPTORS.'),
  'D058544': NtO(MH='Class Ib Phosphatidylinositol 3-Kinase', MS='A phosphatidylinositol 3-kinase subclass that includes enzymes formed through the association of a p110gamma catalytic subunit and one of the three regulatory subunits of 84, 87, and 101 kDa in size. This subclass of enzymes is a downstream target of G PROTEIN-COUPLED RECEPTORS.'),
  'D058545': NtO(MH='Inhalant Abuse', MS='Illicit use of chemicals and products whose vapors can be inhaled to produce a rapid mind-altering effect. Inhalants include aerosols, gases, and volatile solvents that are often inhaled repeatedly to achieve the short-lived intoxicating effect.'),
  'D058546': NtO(MH='Class III Phosphatidylinositol 3-Kinases', MS='A phosphatidylinositol 3-kinase subclass that includes enzymes whose specificity is limited to 1-phosphatidylinositol. Members of this class play a role in vesicular transport and in the regulation of TOR KINASES.'),
  'D058565': NtO(MH='Cerebral Ventriculitis', MS='Inflammation of CEREBRAL VENTRICLES.'),
  'D058566': NtO(MH='Sacroiliitis', MS='Inflammation of the SACROILIAC JOINT. It is characterized by lower back pain, especially upon walking, fever, UVEITIS; PSORIASIS; and decreased range of motion. Many factors are associated with and cause sacroiliitis including infection; injury to spine, lower back, and pelvis; DEGENERATIVE ARTHRITIS; and pregnancy.'),
  'D058567': NtO(MH='Photoinitiators, Dental', MS='Chemical compound used to initiate polymerization of dental resins by the use of DENTAL CURING LIGHTS. It absorbs UV light and undergoes decomposition into free radicals that initiate polymerization process of the resins in the mix. Each photoinitiator has optimum emission spectrum and intensity for proper curing of dental materials.'),
  'D058568': NtO(MH='Necrolytic Migratory Erythema', MS='Recurrent cutaneous manifestation of GLUCAGONOMA characterized by necrolytic polycyclic migratory lesions with scaling borders. It is associated with elevated secretion of GLUCAGON by the tumor. Other conditions with elevated serum glucagon levels such as HEPATIC CIRRHOSIS may also result in similar skin lesions, which are referred to as pseudoglucagonoma syndrome.'),
  'D058569': NtO(MH='Elbow Prosthesis', MS='Replacement for an elbow joint.'),
  'D058570': NtO(MH='TOR Serine-Threonine Kinases', MS='A serine threonine kinase that controls a wide range of growth-related cellular processes. The protein is referred to as the target of RAPAMYCIN due to the discovery that SIROLIMUS (commonly known as rapamycin) forms an inhibitory complex with TACROLIMUS BINDING PROTEIN 1A that blocks the action of its enzymatic activity.'),
  'D058571': NtO(MH='Vacuolar Sorting Protein VPS15', MS='A 150 kDa protein serine-threonine kinase that is found as a regulatory subunit of the class III phosphatidylinositol 3-kinases. The protein is believed to play an important role in the regulation of vesicular trafficking with the cell.'),
  'D058572': NtO(MH='Carbon Footprint', MS='A measure of the total greenhouse gas emissions produced by an individual, organization, event, or product. It is measured in units of equivalent kilograms of CARBON DIOXIDE generated in a given time frame.'),
  'D058573': NtO(MH='Performance-Enhancing Substances', MS='Agents that improve the ability to carry out activities such as athletics, mental endurance, work, and resistance to stress. The substances can include PRESCRIPTION DRUGS; DIETARY SUPPLEMENTS; phytochemicals; and ILLICIT DRUGS.'),
  'D058574': NtO(MH='Integrin-Binding Sialoprotein', MS='A highly glycosylated and sulfated phosphoprotein that is found almost exclusively in mineralized connective tissues. It is an extracellular matrix protein that binds to hydroxyapatite through polyglutamic acid sequences and mediates cell attachment through an RGD sequence.'),
  'D058575': NtO(MH='Decorin', MS='A small leucine-rich proteoglycan that interacts with FIBRILLAR COLLAGENS and modifies the EXTRACELLULAR MATRIX structure of CONNECTIVE TISSUE. Decorin has also been shown to play additional roles in the regulation of cellular responses to GROWTH FACTORS. The protein contains a single glycosaminoglycan chain and is similar in structure to BIGLYCAN.'),
  'D058578': NtO(MH='Biglycan', MS='A small leucine-rich proteoglycan found in a variety of tissues including CAPILLARY ENDOTHELIUM; SKELETAL MUSCLE; CARTILAGE; BONE; and TENDONS. The protein contains two glycosaminoglycan chains and is similar in structure to DECORIN.'),
  'D058579': NtO(MH='Hyalectins', MS='A group of high molecular weight chondroitin sulfate proteoglycans that form aggregates with HYALURONIC ACID.'),
  'D058580': NtO(MH='Neurocan', MS='A hyalectin family member that is expressed in neuronal tissue and plays a role in neuronal CELL ADHESION.'),
  'D058581': NtO(MH='Brevican', MS='A BRAIN-specific hyalectin that may play a role in terminally differentiating NEURONS. It is found highly overexpressed in primary BRAIN TUMORS and in experimental models of GLIOMA.'),
  'D058582': NtO(MH='Double-Balloon Enteroscopy', MS='Endoscopy of the small intestines accomplished while advancing the endoscope into the intestines from the stomach by alternating the inflation of two balloons, one on an innertube of the endoscope and the other on an overtube.'),
  'D058585': NtO(MH='Vestibular Evoked Myogenic Potentials', MS='Recorded electrical responses from muscles, especially the neck muscles or muscles around the eyes, following stimulation of the EAR VESTIBULE.'),
  'D058605': NtO(MH='Host-Derived Cellular Factors', MS='A broad category of endogenous cellular factors that are used by VIRUSES for essential functions such as VIRUS REPLICATION. Many host-derived factors play normal cellular roles in non-infected cells.'),
  'D058606': NtO(MH='Accessory Atrioventricular Bundle', MS='Extra impulse-conducting tissue in the heart that creates abnormal impulse-conducting connections between HEART ATRIA and HEART VENTRICLES.'),
  'D058607': NtO(MH='Fibric Acids', MS='Compounds that either share the structure of fibric acid in their molecular arrangement or are considered variants of the fibric acid structure.'),
  'D058608': NtO(MH='Nanopores', MS='Small holes of nanometer dimensions in a membrane, that can be used as single molecule detectors. The pores can be biological or synthetic.'),
  'D058609': NtO(MH='Visual Field Tests', MS='Method of measuring and mapping the scope of vision, from central to peripheral of each eye.'),
  'D058610': NtO(MH='Isobutyrates', MS='Aliphatic acids that contain four carbons in a branched-chain configuration. Included under this heading are a broad variety of acid forms, salts, esters, and amides that contain the 2-carboxypropane structure.'),
  'D058611': NtO(MH='Horticultural Therapy', MS="A therapeutic approach in which horticultural artefacts are utilized in improving an individual's social, emotional, educational, psychological, and physical well-being."),
  'D058612': NtO(MH='Nanodiamonds', MS='Diamond nanoparticles that exhibit unique biological, thermal, mechanical, and optoelectronic properties. They have important NANOMEDICINE applications including DRUG DELIVERY SYSTEMS; DIAGNOSTIC IMAGING; protein separation; and BIOSENSING TECHNIQUES.'),
  'D058614': NtO(MH='Seed Dispersal', MS='The various physical methods which include wind, insects, animals, tension, and water, by which a plant scatters its seeds away from the parent plant.'),
  'D058615': NtO(MH='Synthetic Biology', MS='A field of biological research combining engineering in the formulation, design, and building (synthesis) of novel biological structures, functions, and systems.'),
  'D058616': NtO(MH='Synbiotics', MS='Nutritional supplements combining PROBIOTICS (bacteria) and PREBIOTICS (sugars).'),
  'D058617': NtO(MH='Composite Lymphoma', MS='Two or more distinct types of malignant lymphoid tumors occurring within a single organ or tissue at the same time. It may contain different types of non-Hodgkin lymphoma cells or both Hodgkin and non-Hodgkin lymphoma cells.'),
  'D058618': NtO(MH='Sensilla', MS='Collective name for a group of external MECHANORECEPTORS and CHEMORECEPTORS manifesting as sensory structures in ARTHROPODS. They include cuticular projections (setae, hairs, bristles), pores, and slits.'),
  'D058619': NtO(MH='Maltose-Binding Proteins', MS='Periplasmic proteins that bind MALTOSE and maltodextrin. They take part in the maltose transport system of BACTERIA.'),
  'D058620': NtO(MH='Elder Nutritional Physiological Phenomena', MS='Nutritional physiology of adults aged 65 years of age and older.'),
  'D058621': NtO(MH='Blood Buffy Coat', MS='The fraction of a blood sample, following CENTRIFUGATION, that is distinguished as a thin light-colored layer between the RED BLOOD CELLS, underneath it, and the PLASMA, above it. It is composed mostly of WHITE BLOOD CELLS and PLATELETS.'),
  'D058625': NtO(MH='End Stage Liver Disease', MS='Final stage of a liver disease when the liver failure is irreversible and LIVER TRANSPLANTATION is needed.'),
  'D058626': NtO(MH='Plasma Gases', MS='Ionized gases, consisting of free electrons and ionized atoms or molecules which collectively behave differently than gas, solid, or liquid. Plasma gases are used in biomedical fields in surface modification; biological decontamination; dentistry (e.g., PLASMA ARC DENTAL CURING LIGHTS); and in other treatments (e.g., ARGON PLASMA COAGULATION).'),
  'D058627': NtO(MH='Megalencephaly', MS='A congenital abnormality in which the occipitofrontal circumference is greater than two standard deviations above the mean for a given age. It is associated with HYDROCEPHALUS; SUBDURAL EFFUSION; ARACHNOID CYSTS; or is part of a genetic condition (e.g., ALEXANDER DISEASE; SOTOS SYNDROME).'),
  'D058628': NtO(MH='Myofibroblasts', MS='Spindle-shaped cells with characteristic CONTRACTILE PROTEINS and structures that contribute to the WOUND HEALING process. They occur in GRANULATION TISSUE and also in pathological processes such as FIBROSIS.'),
  'D058629': NtO(MH='Plasma Skin Regeneration', MS='A cosmetic technique that uses PLASMA GASES in therapeutic treatment to help achieve skin REJUVENATION or REGENERATION and delay SKIN AGING.'),
  'D058630': NtO(MH='Spinal Cord Regeneration', MS='Repair of the damaged neuron function after SPINAL CORD INJURY or SPINAL CORD DISEASES.'),
  'D058631': NtO(MH='Pycnodysostosis', MS='Rare autosomal recessive syndrome characterized by delayed closing of CRANIAL SUTURES, short stature, ACRO-OSTEOLYSIS of distal phalanges, dental and MAXILLOFACIAL ABNORMALITIES and an increase in bone density that results in frequent BONE FRACTURES. It is associated with BONE RESORPTION defect due to mutations in the lysosomal cysteine protease CATHEPSIN K.'),
  'D058632': NtO(MH='Neuroprostanes', MS='Prostaglandin-like compounds produced by free radical-induced peroxidation of DOCOSAHEXAENOIC ACIDS, which are highly enriched in the brain. Formation is analogous to ISOPROSTANES formation from ARACHIDONIC ACID.'),
  'D058633': NtO(MH='Antipyretics', MS='Drugs that are used to reduce body temperature in fever.'),
  'D058634': NtO(MH='Molecular Farming', MS='The large scale production of pharmaceutically important and commercially valuable RECOMBINANT PROTEINS.'),
  'D058635': NtO(MH='Lipid-Linked Proteins', MS='Proteins that are covalently-linked to lipid molecules or hydrophobic molecules such as those that contain prenyl groups. The lipid moieties aid in the association of the proteins with CELL MEMBRANES.'),
  'D058636': NtO(MH='Surgical Drapes', MS='Sterile fabric or fabric-like material used to isolate the surgical site from the rest of the body and other possible sources of contamination.'),
  'D058637': NtO(MH='Examination Tables', MS='A raised flat surface on which a patient is placed during a PHYSICAL EXAMINATION.'),
  'D058638': NtO(MH='Operating Tables', MS='A raised flat surface on which a patient is placed during surgical procedures.'),
  'D058639': NtO(MH='Cold-Shock Response', MS='A sequence of responses that occur when an organism is exposed to excessive cold. In humans, a fall in skin temperature triggers gasping, hypertension, and hyperventilation.'),
  'D058640': NtO(MH='Brochothrix', MS='A genus of gram-positive bacteria in the family Listeriaceae. It has 93% homology to LISTERIA and shares with it many phenotypic properties.'),
  'D058641': NtO(MH='Planococcaceae', MS='A family of gram-positive bacteria in the order BACILLALES. Most are strict aerobic heterotrophs.'),
  'D058642': NtO(MH='Planococcus Bacteria', MS='A genus of coccoid bacteria in the family PLANOCOCCACEAE. They are widely distributed in various habitats including sea water, freshwater ponds, cyanobacterial mats, and in marine animals.'),
  'D058643': NtO(MH='Planococcus Insect', MS='A genus of insect in the mealybug family Pseudococcidae.'),
  'D058644': NtO(MH='Sporosarcina', MS='A genus of endospore-forming bacteria in the family Planococcaceae. The species are strict or facultative anerobes.'),
  'D058645': NtO(MH='Denaturing Gradient Gel Electrophoresis', MS='Electrophoresis in which various denaturant gradients are used to induce nucleic acids to melt at various stages resulting in separation of molecules based on small sequence differences including SNPs. The denaturants used include heat, formamide, and urea.'),
  'D058646': NtO(MH='Adrenergic alpha-1 Receptor Agonists', MS='Compounds that bind to and activate ADRENERGIC ALPHA-1 RECEPTORS.'),
  'D058647': NtO(MH='Adrenergic alpha-2 Receptor Agonists', MS='Compounds that bind to and activate ADRENERGIC ALPHA-2 RECEPTORS.'),
  'D058665': NtO(MH='Adrenergic beta-1 Receptor Agonists', MS='Compounds that bind to and activate ADRENERGIC BETA-1 RECEPTORS.'),
  'D058666': NtO(MH='Adrenergic beta-2 Receptor Agonists', MS='Compounds bind to and activate ADRENERGIC BETA-2 RECEPTORS.'),
  'D058667': NtO(MH='Adrenergic beta-3 Receptor Agonists', MS='Compounds that bind to and activate ADRENERGIC BETA-3 RECEPTORS.'),
  'D058668': NtO(MH='Adrenergic alpha-1 Receptor Antagonists', MS='Drugs that bind to and block the activation of ADRENERGIC ALPHA-1 RECEPTORS.'),
  'D058669': NtO(MH='Adrenergic alpha-2 Receptor Antagonists', MS='Drugs that bind to and block the activation of ADRENERGIC ALPHA-2 RECEPTORS.'),
  'D058670': NtO(MH='Tetrasomy', MS='The possession of four chromosomes of any one type in an otherwise diploid cell.'),
  'D058671': NtO(MH='Adrenergic beta-1 Receptor Antagonists', MS='Drugs that bind to and block the activation of ADRENERGIC BETA-1 RECEPTORS.'),
  'D058672': NtO(MH='Adrenergic beta-2 Receptor Antagonists', MS='Drugs that bind to and block the activation of ADRENERGIC BETA-2 RECEPTORS.'),
  'D058673': NtO(MH='Adrenergic beta-3 Receptor Antagonists', MS='Drugs that bind to and block the activation of ADRENERGIC BETA-3 RECEPTORS.'),
  'D058674': NtO(MH='Chromosome Duplication', MS='An aberration in which an extra chromosome or a chromosomal segment is made.'),
  'D058675': NtO(MH='Brevibacillus', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING RODS in the family Paenibacillaceae. Most strains have been isolated from the natural environment, particularly soils.'),
  'D058676': NtO(MH='Pasteuria', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING BACTERIA in the family Pasteuriaceae. It is transmitted via soil or waterborne SPORES.'),
  'D058677': NtO(MH='Anoxybacillus', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING BACTERIA in the family Bacillaceae. Most species have been isolated from HOT SPRINGS.'),
  'D058678': NtO(MH='Halobacillus', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING BACTERIA in the family BACILLACEAE. Species are widely distributed in a variety of hypersaline environments.'),
  'D058679': NtO(MH='Virgibacillus', MS='A genus of GRAM-POSITIVE ENDOSPORE-FORMING RODS in the family BACILLACEAE. They have been isolated from salty environments, food, water, and clinical specimens.'),
  'D058685': NtO(MH='Genetic Pleiotropy', MS='A phenomenon in which multiple and diverse phenotypic outcomes are influenced by a single gene (or single gene product.)'),
  'D058686': NtO(MH='Coronary-Subclavian Steal Syndrome', MS='A complication of INTERNAL MAMMARY-CORONARY ARTERY ANASTOMOSIS whereby an occlusion or stenosis of the proximal SUBCLAVIAN ARTERY causes a reversal of the blood flow away from the CORONARY CIRCULATION, through the grafted INTERNAL MAMMARY ARTERY (internal thoracic artery), and back to the distal subclavian distribution.'),
  'D058687': NtO(MH='Out-of-Hospital Cardiac Arrest', MS='Occurrence of heart arrest in an individual when there is no immediate access to medical personnel or equipment.'),
  'D058725': NtO(MH='Peripartum Period', MS='The period shortly before, during, and immediately after giving birth.'),
  'D058726': NtO(MH='Nanoconjugates', MS='Tailored macromolecules harboring covalently-bound biologically active modules that target specific tissues and cells. The active modules or functional groups can include drugs, prodrugs, antibodies, and oligonucleotides, which can act synergistically and be multitargeting.'),
  'D058727': NtO(MH='RNA, Small Untranslated', MS='Short RNA, about 200 base pairs in length or shorter, that does not code for protein.'),
  'D058728': NtO(MH='Weed Control', MS='The prevention of growth and or spread of unwanted plants.'),
  'D058729': NtO(MH='Peripheral Arterial Disease', MS='Lack of perfusion in the EXTREMITIES resulting from atherosclerosis. It is characterized by INTERMITTENT CLAUDICATION, and an ANKLE BRACHIAL INDEX of 0.9 or less.'),
  'D058731': NtO(MH='Blood Safety', MS='The degree to which the blood supply for BLOOD TRANSFUSIONS is free of harmful substances or infectious agents, and properly typed and crossmatched (BLOOD GROUPING AND CROSSMATCHING) to insure serological compatibility between BLOOD DONORS and recipients.'),
  'D058732': NtO(MH='Embryoid Bodies', MS='Spontaneous aggregations of human embryonic stem cells that occur in vitro after culturing in a medium that lacks LEUKEMIC INHIBITORY FACTOR. The embryoid bodies can further differentiate into cells that represent different lineages.'),
  'D058733': NtO(MH='Hand-Assisted Laparoscopy', MS="Placement of one of the surgeon's gloved hands into the ABDOMINAL CAVITY to perform manual manipulations that facilitate the laparoscopic procedures."),
  'D058734': NtO(MH='Exsanguination', MS='Rapid and extreme blood loss leading to HEMORRHAGIC SHOCK.'),
  'D058735': NtO(MH='Environmental Policy', MS='A course of action or principle adopted or proposed by a government, party, business, or individual that concerns human interactions with nature and natural resources.'),
  'D058736': NtO(MH='Intrauterine Device Migration', MS='The shifting in position or location of an INTRAUTERINE DEVICE from its original placement.'),
  'D058737': NtO(MH='Arthroplasty, Replacement, Ankle', MS='Replacement of the ANKLE JOINT.'),
  'D058738': NtO(MH='Uterine Balloon Tamponade', MS='Inflation of a balloon CATHETER within the uterine cavity to control UTERINE HEMORRHAGE.'),
  'D058739': NtO(MH='Aberrant Crypt Foci', MS='Clusters of colonic crypts that appear different from the surrounding mucosa when visualized after staining. They are of interest as putative precursors to colorectal adenomas and potential biomarkers for colorectal carcinoma.'),
  'D058745': NtO(MH='Iliotibial Band Syndrome', MS='An overuse injury causing lateral knee pain that results from repetitive friction of the iliotibial band over the lateral femoral epicondyle.'),
  'D058746': NtO(MH='Gestational Sac', MS='Round-shaped structure usually located in the upper fundus of the UTERUS in early pregnancy. It comprises EMBRYO; AMNIOTIC FLUID; and YOLK SAC. It is visible in PRENATAL ULTRASONOGRAPHY by the fifth week of pregnancy. Its size is often used to determine and monitor GESTATIONAL AGE; FETAL GROWTH; and PREGNANCY COMPLICATIONS.'),
  'D058747': NtO(MH='CHARGE Syndrome', MS='Rare disease characterized by COLOBOMA; CHOANAL ATRESIA; and abnormal SEMICIRCULAR CANALS. Mutations in CHD7 protein resulting in disturbed neural crest development are associated with CHARGE Syndrome.'),
  'D058748': NtO(MH='Pain Perception', MS='The process by which PAIN is recognized and interpreted by the brain.'),
  'D058749': NtO(MH='Military Facilities', MS='Areas designated for use by the armed forces personnel.'),
  'D058750': NtO(MH='Epithelial-Mesenchymal Transition', MS='Phenotypic changes of EPITHELIAL CELLS to MESENCHYME type, which increase cell mobility critical in many developmental processes such as NEURAL TUBE development. NEOPLASM METASTASIS and DISEASE PROGRESSION may also induce this transition.'),
  'D058751': NtO(MH='Livestock', MS='Domesticated farm animals raised for home use or profit but excluding POULTRY. Typically livestock includes CATTLE; SHEEP; HORSES; SWINE; GOATS; and others.'),
  'D058752': NtO(MH='Free Tissue Flaps', MS='A mass of tissue that has been cut away from its surrounding areas to be used in TISSUE TRANSPLANTATION.'),
  'D058753': NtO(MH='Laryngoplasty', MS='Restoration, reconstruction, or improvement of a defective or damaged LARYNX.'),
  'D058765': NtO(MH='Neurofeedback', MS="A technique to self-regulate brain activities provided as a feedback in order to better control or enhance one's own performance, control or function. This is done by trying to bring brain activities into a range associated with a desired brain function or status."),
  'D058766': NtO(MH='Levoleucovorin', MS='A folate analog consisting of the pharmacologically active isomer of LEUCOVORIN.'),
  'D058767': NtO(MH='Protein Unfolding', MS='Conformational transitions of the shape of a protein to various unfolded states.'),
  'D058785': NtO(MH='GABA-A Receptor Agonists', MS='Endogenous compounds and drugs that bind to and activate GABA-A RECEPTORS.'),
  'D058786': NtO(MH='GABA-B Receptor Agonists', MS='Endogenous compounds and drugs that bind to and activate GABA-B RECEPTORS.'),
  'D058787': NtO(MH='GABA-A Receptor Antagonists', MS='Drugs that bind to but do not activate GABA-A RECEPTORS thereby blocking the actions of endogenous or exogenous GABA-A RECEPTOR AGONISTS.'),
  'D058788': NtO(MH='GABA-B Receptor Antagonists', MS='Drugs that bind to but do not activate GABA-B RECEPTORS thereby blocking the actions of endogenous or exogenous GABA-B RECEPTOR AGONISTS.'),
  'D058805': NtO(MH='GABA Uptake Inhibitors', MS='Compounds that suppress or block the plasma membrane transport of GAMMA-AMINOBUTYRIC ACID by GABA PLASMA MEMBRANE TRANSPORT PROTEINS.'),
  'D058825': NtO(MH='Serotonin 5-HT1 Receptor Agonists', MS='Endogenous compounds and drugs that specifically stimulate SEROTONIN 5-HT1 RECEPTORS. Included under this heading are agonists for one or more of the specific 5-HT1 receptor subtypes.'),
  'D058826': NtO(MH='Serotonin 5-HT2 Receptor Agonists', MS='Endogenous compounds and drugs that specifically stimulate SEROTONIN 5-HT2 RECEPTORS. Included under this heading are agonists for one or more of the specific 5-HT2 receptor subtypes.'),
  'D058827': NtO(MH='Serotonin 5-HT3 Receptor Agonists', MS='Endogenous compounds and drugs that specifically stimulate SEROTONIN 5-HT3 RECEPTORS.'),
  'D058828': NtO(MH='Serotonin 5-HT4 Receptor Agonists', MS='Endogenous compounds and drugs that specifically stimulate SEROTONIN 5-HT4 RECEPTORS.'),
  'D058829': NtO(MH='Serotonin 5-HT1 Receptor Antagonists', MS='Drugs that bind to but do not activate SEROTONIN 5-HT1 RECEPTORS, thereby blocking the actions of SEROTONIN 5-HT1 RECEPTOR AGONISTS. Included under this heading are antagonists for one or more of the specific 5-HT1 receptor subtypes.'),
  'D058830': NtO(MH='Serotonin 5-HT2 Receptor Antagonists', MS='Drugs that bind to but do not activate SEROTONIN 5-HT2 RECEPTORS, thereby blocking the actions of SEROTONIN or SEROTONIN 5-HT2 RECEPTOR AGONISTS. Included under this heading are antagonists for one or more specific 5-HT2 receptor subtypes.'),
  'D058831': NtO(MH='Serotonin 5-HT3 Receptor Antagonists', MS='Drugs that bind to but do not activate SEROTONIN 5-HT3 RECEPTORS, thereby blocking the actions of SEROTONIN or SEROTONIN 5-HT3 RECEPTOR AGONISTS.'),
  'D058832': NtO(MH='Serotonin 5-HT4 Receptor Antagonists', MS='Drugs that bind to but do not activate SEROTONIN 5-HT4 RECEPTORS, thereby blocking the actions of SEROTONIN or SEROTONIN RECEPTOR AGONISTS.'),
  'D058833': NtO(MH='Antithrombin Proteins', MS='An endogenous family of proteins belonging to the serpin superfamily that neutralizes the action of thrombin. Six naturally occurring antithrombins have been identified and are designated by Roman numerals I to VI. Of these, Antithrombin I (see FIBRIN) and ANTITHROMBIN III appear to be of major importance.'),
  'D058834': NtO(MH='Gemella', MS='A genus that has been reclassified into BACILLALES incertae sedis because of its ambiguous taxonomy. Previously it was considered part of the Staphylococcaceae family.'),
  'D058835': NtO(MH='Abiotrophia', MS='A genus of gram-positive, facultatively anaerobic bacteria in the family Aerococcaceae.'),
  'D058836': NtO(MH='Weissella', MS='A genus of gram-positive, asporogenous, lactic acid bacteria, in the family LEUCONOSTOCACEAE.'),
  'D058845': NtO(MH='Moorella', MS='A genus of gram positive, acetogenic, thermophilic bacteria in the family Thermoanaerobacteraceae. Known habitats include HOT SPRINGS, horse manure, emu droppings, and sewage SLUDGE.'),
  'D058846': NtO(MH='Antibodies, Monoclonal, Murine-Derived', MS='Antibodies obtained from a single clone of cells grown in mice or rats.'),
  'D058847': NtO(MH='Inflammasomes', MS='Multiprotein complexes that mediate the activation of CASPASE-1. Dysregulation of inflammasomes has also been linked to a number of autoinflammatory and autoimmune disorders.'),
  'D058848': NtO(MH='Receptors, Purinergic P2X4', MS='A widely distributed purinergic P2X receptor subtype that plays a role in pain sensation. P2X4 receptors found on MICROGLIA cells may also play a role in the mediation of allodynia-related NEUROPATHIC PAIN.'),
  'D058849': NtO(MH='Protein Refolding', MS='Conformational transitions of a protein from unfolded states (after PROTEIN UNFOLDING)  to a more folded state.'),
  'D058850': NtO(MH='Opiate Substitution Treatment', MS='Medication-assisted treatment of opioid dependence using a substitute opiate such as METHADONE or BUPRENORPHINE.'),
  'D058851': NtO(MH='GPI-Linked Proteins', MS='A subclass of lipid-linked proteins that contain a GLYCOSYLPHOSPHATIDYLINOSITOL LINKAGE which holds them to the CELL MEMBRANE.'),
  'D058865': NtO(MH='Introduced Species', MS='Non-native organisms brought into a region, habitat, or ECOSYSTEM by human activity.'),
  'D058866': NtO(MH='Osteoporotic Fractures', MS='Breaks in bones resulting from low bone mass and microarchitectural deterioration characteristic of OSTEOPOROSIS.'),
  'D058867': NtO(MH='Lipoprotein Lipase Activators', MS='Compounds that increase the enzymatic activity of LIPOPROTEIN LIPASE. Lipoprotein lipase activators have a potential role in the treatment of OBESITY by increasing LIPID METABOLISM. Note that substances that increase the synthesis of lipoprotein lipase are not included here.'),
  'D058868': NtO(MH='Compressed Air', MS='Air that is reduced in volume by pressure.'),
  'D058869': NtO(MH='Gynecological Examination', MS='Inspection and PALPATATION of female breasts, abdomen, and GENITALIA, as well as obtaining a gynecological history. (from Dictionary of Obstetrics and Gynecology)'),
  'D058870': NtO(MH='Food, Organic', MS='Food that is grown or manufactured in accordance with nationally regulated production standards that include restrictions on the use of pesticides, non-organic fertilizers, genetic engineering, growth hormones, irradiation, antibiotics, and non-organic ingredients.'),
  'D058871': NtO(MH='Organic Agriculture', MS='Systems of agriculture which adhere to nationally regulated standards that restrict the use of pesticides, non-organic fertilizers, genetic engineering, growth hormones, irradiation, antibiotics, and non-organic ANIMAL FEED.'),
  'D058872': NtO(MH='Epidemics', MS='Sudden outbreaks of a disease in a country or region not previously recognized in that area, or a rapid increase in the number of new cases of a previous existing endemic disease. Epidemics can also refer to outbreaks of disease in animal or plant populations.'),
  'D058873': NtO(MH='Pandemics', MS='Epidemics of infectious disease that have spread to many countries, often more than one continent, and usually affecting a large number of people.'),
  'D058874': NtO(MH='Electrowetting', MS='Reducing the SURFACE TENSION at a liquid/solid interface by the application of an electric current across the interface thereby enhancing the WETTABILITY of the surface.'),
  'D058885': NtO(MH='Multilocus Sequence Typing', MS='Direct nucleotide sequencing of gene fragments from multiple housekeeping genes for the purpose of phylogenetic analysis, organism identification, and typing of species, strain, serovar, or other distinguishable phylogenetic level.'),
  'D058886': NtO(MH='Sterol 14-Demethylase', MS='An NADPH-dependent P450 enzyme that plays an essential role in the sterol biosynthetic pathway by catalyzing the demethylation of 14-methyl sterols such as lanosterol. The enzyme acts via the repeated hydroxylation of the 14-methyl group, resulting in its stepwise conversion into an alcohol, an aldehyde and then a carboxylate, which is removed as formic acid. Sterol 14-demethylase is an unusual cytochrome P450 enzyme in that it is found in a broad variety of organisms including ANIMALS; PLANTS; FUNGI; and protozoa.'),
  'D058888': NtO(MH='14-alpha Demethylase Inhibitors', MS='Compounds that specifically inhibit STEROL 14-DEMETHYLASE. A variety of azole-derived ANTIFUNGAL AGENTS act through this mechanism.'),
  'D058889': NtO(MH='Molecular Typing', MS='Using MOLECULAR BIOLOGY techniques, such as DNA SEQUENCE ANALYSIS; PULSED-FIELD GEL ELECTROPHORESIS; and DNA FINGERPRINTING, to identify, classify, and compare organisms and their subtypes.'),
  'D058890': NtO(MH='Granulomatous Mastitis', MS='A rare, benign, inflammatory breast disease occurring in premenopausal women shortly after a recent pregnancy. The origin is unknown but it is commonly mistaken for malignancy and sometimes associated with BREAST FEEDING and the use of ORAL CONTRACEPTIVES.'),
  'D058891': NtO(MH='5-alpha Reductase Inhibitors', MS='Drugs that inhibit 3-OXO-5-ALPHA-STEROID 4-DEHYDROGENASE. They are commonly used to reduce the production of DIHYDROTESTOSTERONE.'),
  'D058892': NtO(MH='Adenosine Deaminase Inhibitors', MS='Drugs that inhibit ADENOSINE DEAMINASE activity.'),
  'D058893': NtO(MH='DNA Barcoding, Taxonomic', MS='Techniques for standardizing and expediting taxonomic identification or classification of organisms that are based on deciphering the sequence of one or a few regions of DNA known as the "DNA barcode".'),
  'D058894': NtO(MH='Nematocyst', MS='A double-walled capsule found in jellyfish and other CNIDARIA whose functions include prey capture, defense, locomotion, and attachment. Nematocysts contain toxic CNIDARIAN VENOMS which are injected into the victim via a barbed tubule.'),
  'D058905': NtO(MH='Purinergic Agents', MS='Compounds that act on PURINERGIC RECEPTORS or influence the synthesis, storage, uptake, metabolism, or release of purinergic transmitters.'),
  'D058906': NtO(MH='Purinergic P1 Receptor Agonists', MS='Compounds that bind to and stimulate PURINERGIC P1 RECEPTORS.'),
  'D058907': NtO(MH='Adenosine A1 Receptor Agonists', MS='Compounds that bind to and stimulate ADENOSINE A1 RECEPTORS.'),
  'D058908': NtO(MH='Adenosine A2 Receptor Agonists', MS='Compounds that selectively bind to and activate ADENOSINE A2 RECEPTORS.'),
  'D058909': NtO(MH='Adenosine A3 Receptor Agonists', MS='Drugs that selectively bind to and activate ADENOSINE A3 RECEPTORS.'),
  'D058910': NtO(MH='Purinergic P2 Receptor Agonists', MS='Compounds that bind to and stimulate PURINERGIC P2 RECEPTORS.'),
  'D058911': NtO(MH='Purinergic P2X Receptor Agonists', MS='Compounds that bind to and stimulate PURINERGIC P2X RECEPTORS. Included under this heading are agonists for specific P2X receptor subtypes.'),
  'D058912': NtO(MH='Purinergic P2Y Receptor Agonists', MS='Compounds that bind to and stimulate PURINERGIC P2Y RECEPTORS. Included under this heading are agonists for specific P2Y receptor subtypes.'),
  'D058913': NtO(MH='Purinergic Agonists', MS='Compounds that bind to and activate PURINERGIC RECEPTORS.'),
  'D058914': NtO(MH='Purinergic Antagonists', MS='Drugs that bind to and block the activation of PURINERGIC RECEPTORS.'),
  'D058915': NtO(MH='Purinergic P1 Receptor Antagonists', MS='Compounds that bind to and block the stimulation of PURINERGIC P1 RECEPTORS.'),
  'D058916': NtO(MH='Adenosine A1 Receptor Antagonists', MS='Compounds that bind to and block the stimulation of ADENOSINE A1 RECEPTORS.'),
  'D058917': NtO(MH='Adenosine A2 Receptor Antagonists', MS='Compounds that selectively bind to and block the activation of ADENOSINE A2 RECEPTORS.'),
  'D058918': NtO(MH='Adenosine A3 Receptor Antagonists', MS='Compounds that selectively bind to and block the activation of ADENOSINE A3 RECEPTORS.'),
  'D058919': NtO(MH='Purinergic P2 Receptor Antagonists', MS='Compounds that bind to and block the stimulation of PURINERGIC P2 RECEPTORS.'),
  'D058920': NtO(MH='Purinergic P2X Receptor Antagonists', MS='Compounds that bind to and block the stimulation of PURINERGIC P2X RECEPTORS. Included under this heading are antagonists for specific P2X receptor subtypes.'),
  'D058921': NtO(MH='Purinergic P2Y Receptor Antagonists', MS='Compounds that bind to and block the stimulation of PURINERGIC P2Y RECEPTORS. Included under this heading are antagonists for specific P2Y receptor subtypes.'),
  'D058922': NtO(MH='Inflammatory Breast Neoplasms', MS='Metastatic breast cancer characterized by EDEMA and ERYTHEMA of the affected breast due to LYMPHATIC METASTASIS and eventual obstruction of LYMPHATIC VESSELS by the cancer cells.'),
  'D058923': NtO(MH='Medial Tibial Stress Syndrome', MS='SKELETAL MUSCLE pain and tenderness along the posteromedial TIBIA resulting from exercise such as running and other physical activity.'),
  'D058924': NtO(MH='Thienopyridines', MS='Heterocyclic compounds that contain 4H,5H,6H,7H-thieno[2,3-c]pyridine as part of their structure.'),
  'D058925': NtO(MH='Receptors, Purinergic P2Y12', MS='A subclass of purinergic P2Y receptors that have a preference for ADP binding and are coupled to GTP-BINDING PROTEIN ALPHA SUBUNIT, GI. The P2Y12 purinergic receptors are found in PLATELETS where they play an important role regulating PLATELET ACTIVATION.'),
  'D058926': NtO(MH='Intraoperative Awareness', MS='Occurence of a patient becoming conscious during a procedure performed under GENERAL ANESTHESIA and subsequently having recall of these events. (From Anesthesiology 2006, 104(4): 847-64.)'),
  'D058927': NtO(MH='5-Lipoxygenase-Activating Proteins', MS='Scaffolding proteins that play an important role in the localization and activation of 5-LIPOXYGENASE.'),
  'D058928': NtO(MH='Riboswitch', MS="Part of a MESSENGER RNA molecule that undergoes a conformation change upon binding a specific metabolite or other small molecule thereby regulating the messenger RNA's transcription, post-transcriptional processing, transport, translation, or stability in response to varying levels of the metabolite or other small molecule."),
  'D058945': NtO(MH='Lipoxygenases', MS='Dioxygenases that catalyze the peroxidation of methylene-interrupted UNSATURATED FATTY ACIDS.'),
  'D058946': NtO(MH='5-Lipoxygenase-Activating Protein Inhibitors', MS='Compounds that bind to and inhibit the action of 5-LIPOXYGENASE-ACTIVATING PROTEINS.'),
  'D058947': NtO(MH='Bacterial Secretion Systems', MS='In GRAM NEGATIVE BACTERIA, multiprotein complexes that function to translocate pathogen protein effector molecules across the bacterial cell envelope, often directly into the host. These effectors are involved in producing surface structures for adhesion, bacterial motility, manipulation of host functions, modulation of host defense responses, and other functions involved in facilitating survival of the pathogen. Several of the systems have homologous components functioning similarly in GRAM POSITIVE BACTERIA.'),
  'D058948': NtO(MH='Cell Tracking', MS='Non-invasive imaging of cells that have been labeled non-destructively, such as with nanoemulsions or reporter genes that can be detected by molecular imaging, to monitor their location, viability, cell lineage expansion, response to drugs, movement, or other behaviors in vivo.'),
  'D058949': NtO(MH='Uromodulin', MS='A glycosyl-phosphatidyl-inositol (GPI) - anchored membrane protein found on the thick ascending limb of the LOOP OF HENLE. The cleaved form of the protein is found abundantly in URINE.'),
  'D058950': NtO(MH='Melanoma-Specific Antigens', MS='Cellular antigens that are specific for MELANOMA cells.'),
  'D058951': NtO(MH='gp100 Melanoma Antigen', MS='A melanosome-associated protein that plays a role in the maturation of the MELANOSOME.'),
  'D058952': NtO(MH='Human papillomavirus 31', MS='A type of ALPHAPAPILLOMAVIRUS associated with high risk for anogenital neoplasms.'),
  'D058953': NtO(MH='Neural Stem Cells', MS='Self-renewing cells that generate the main phenotypes of the nervous system in both the embryo and adult. Neural stem cells are precursors to both NEURONS and NEUROGLIA.'),
  'D058954': NtO(MH='Pancreatic Stellate Cells', MS='Star-shaped, myofibroblast-like cells located in the periacinar, perivascular, and periductal regions of the EXOCRINE PANCREAS. They play a key role in the pathobiology of FIBROSIS; PANCREATITIS; and PANCREATIC CANCER.'),
  'D058955': NtO(MH='Metalloids', MS='A class of nonmetals such as arsenic that have some of the chemical properties of a metal.'),
  'D058956': NtO(MH='Administration, Mucosal', MS='Delivery of a drug or other substance into the body through the epithelium lining of MUCOUS MEMBRANE involved with absorption and secretion.'),
  'D058957': NtO(MH='Coronavirus NL63, Human', MS='A species in the genus CORONAVIRUS causing upper and lower RESPIRATORY TRACT INFECTIONS. It shares the receptor used by the SARS VIRUS.'),
  'D058958': NtO(MH='Organs at Risk', MS='Organs which might be damaged during exposure to a toxin or to some form of therapy. It most frequently refers to healthy organs located in the radiation field during radiation therapy.'),
  'D058965': NtO(MH='MART-1 Antigen', MS='A melanosome-specific protein that plays a role in the expression, stability, trafficking, and processing of GP100 MELANOMA ANTIGEN, which is critical to the formation of Stage II MELANOSOMES. The protein is used as an antigen marker for MELANOMA cells.'),
  'D058966': NtO(MH='Contactins', MS='A family of immunoglobulin-related cell adhesion molecules that are involved in NERVOUS SYSTEM patterning.'),
  'D058967': NtO(MH='Contactin 1', MS='A contactin subtype that is predominantly expressed in the CEREBELLUM; HIPPOCAMPUS; NEOCORTEX; and HYPOTHALAMUS.'),
  'D058968': NtO(MH='Pythiosis', MS='A granulomatous disease caused by the aquatic organism PYTHIUM insidiosum and occurring primarily in horses, cattle, dogs, cats, fishes, and rarely in humans. It is classified into three forms: ocular, cutaneous, and arterial.'),
  'D058969': NtO(MH='Contactin 2', MS='A contactin subtype that plays a role in axon outgrowth, axon fasciculation, and neuronal migration.'),
  'D058971': NtO(MH='Tenotomy', MS='Surgical division of a tendon for relief of a deformity that is caused by congenital or acquired shortening of a muscle (Stedman, 27th ed). Tenotomy is performed in order to lengthen a muscle that has developed improperly, or become shortened and is resistant to stretching.'),
  'D058972': NtO(MH='Folic Acid Transporters', MS='Proteins involved in the transport of FOLIC ACID and folate derivatives across the CELLULAR MEMBRANE.'),
  'D058973': NtO(MH='Folate Receptors, GPI-Anchored', MS='Cell surface receptors that bind to and transport FOLIC ACID, 5-methyltetrahydrofolate, and a variety of folic acid derivatives. The receptors are essential for normal NEURAL TUBE development and transport folic acid via receptor-mediated endocytosis.'),
  'D058974': NtO(MH='Phylogeography', MS='A field of study concerned with the principles and processes governing the geographic distributions of genealogical lineages, especially those within and among closely related species. (Avise, J.C., Phylogeography: The History and Formation of Species. Harvard University Press, 2000)'),
  'D058975': NtO(MH='Folate Receptor 1', MS='A subtype of GPI-anchored folate receptors that is expressed in tissues of epithelial origin. This protein is also identified as an ovarian-tumor-specific antigen.'),
  'D058976': NtO(MH='Folate Receptor 2', MS='A subtype of GPI-anchored folate receptors that is expressed in PLACENTA and hematopoietic cells.'),
  'D058977': NtO(MH='Molecular Sequence Annotation', MS='The addition of descriptive information about the function or structure of a molecular sequence to its MOLECULAR SEQUENCE DATA record.'),
  'D058978': NtO(MH='Reduced Folate Carrier Protein', MS='A ubiquitously expressed folic acid transporter that functions via an antiporter mechanism which is coupled to the transport of organic phosphates.'),
  'D058979': NtO(MH='Proton-Coupled Folate Transporter', MS='A symporter protein that couples the transport of FOLIC ACID with HYDROGEN IONS. The transporter functions most effectively under acidic conditions.'),
  'D058980': NtO(MH='Information Literacy', MS='The ability to recognize when information is needed and to locate, evaluate, and use the needed information effectively.'),
  'D058985': NtO(MH='Side-Population Cells', MS='A group of cells identified on FLOW CYTOMETRY profiles as distinct from the main group of cells by their ability to extrude the fluorescent dye Hoechst 33342, often a characteristic property of less differentiated progenitor and STEM CELLS.'),
  'D058986': NtO(MH='Phosphodiesterase 5 Inhibitors', MS='Compounds that specifically inhibit PHOSPHODIESTERASE 5.'),
  'D058987': NtO(MH='Phosphodiesterase 3 Inhibitors', MS='Compounds that specifically inhibit PHOSPHODIESTERASE 3.'),
  'D058988': NtO(MH='Phosphodiesterase 4 Inhibitors', MS='Compounds that specifically inhibit PHOSPHODIESTERASE 4.'),
  'D058989': NtO(MH='Vitrification', MS='The transformation of a liquid to a glassy solid i.e., without the formation of crystals during the cooling process.'),
  'D058990': NtO(MH='Molecular Targeted Therapy', MS="Treatments with drugs which interact with or block synthesis of specific cellular components characteristic of the individual's disease in order to stop or interrupt the specific biochemical dysfunction involved in progression of the disease."),
  'D058991': NtO(MH='Patient Protection and Affordable Care Act', MS='An Act prohibiting a health plan from establishing lifetime limits or annual limits on the dollar value of benefits for any participant or beneficiary after January 1, 2014. It permits a restricted annual limit for plan years beginning prior to January 1, 2014. It provides that a health plan shall not be prevented from placing annual or lifetime per-beneficiary limits on covered benefits. The Act sets up a competitive health insurance market.'),
  'D058992': NtO(MH='Social Participation', MS='Involvement in community activities or programs.'),
  'D058993': NtO(MH='Oral Sprays', MS='Pharmacologic agents in the form of a mist or spray delivered into the body through the mouth.'),
  'D058994': NtO(MH='Salpingectomy', MS='Excision of one or both of the FALLOPIAN TUBES.'),
  'D058995': NtO(MH='Dry Powder Inhalers', MS='A device that delivers medication to the lungs in the form of a dry powder.'),
  'D058996': NtO(MH='Quality Improvement', MS='The attainment or process of attaining a new level of performance or quality.'),
  'D058998': NtO(MH='Remote Sensing Technology', MS='Observation and acquisition of physical data from a distance by viewing and making measurements from a distance or receiving transmitted data from observations made at distant location.'),
  'D058999': NtO(MH='Drug-Seeking Behavior', MS='Activities performed to obtain licit or illicit substances.'),
  'D059000': NtO(MH='El Nino-Southern Oscillation', MS='El Nino-Southern Oscillation or ENSO is a cycle of extreme alternating warm El Ni\xc3\xb1o and cold La Nina events which is the dominant year-to-year climate pattern on Earth. Both terms refer to large-scale changes in sea-surface temperature across the eastern tropical Pacific. ENSO is associated with a heightened risk of certain vector-borne diseases. (From http://www.elnino.noaa.gov/lanina_new_faq.html, accessed 5/12/2020)'),
  'D059001': NtO(MH='Aquatic Organisms', MS='Organisms that live in water.'),
  'D059002': NtO(MH='Androgen Receptor Antagonists', MS='Compounds that bind to and inhibit the activation of ANDROGEN RECEPTORS.'),
  'D059003': NtO(MH='Topoisomerase Inhibitors', MS='Compounds that inhibit the activity of DNA TOPOISOMERASES.'),
  'D059004': NtO(MH='Topoisomerase I Inhibitors', MS='Compounds that inhibit the activity of DNA TOPOISOMERASE I.'),
  'D059005': NtO(MH='Topoisomerase II Inhibitors', MS='Compounds that inhibit the activity of DNA TOPOISOMERASE II. Included in this category are a variety of ANTINEOPLASTIC AGENTS which target the eukaryotic form of topoisomerase II and ANTIBACTERIAL AGENTS which target the prokaryotic form of topoisomerase II.'),
  'D059006': NtO(MH='Chromosomes, Insect', MS='Structures within the CELL NUCLEUS of insect cells containing DNA.'),
  'D059007': NtO(MH='Polytene Chromosomes', MS='Extra large CHROMOSOMES, each consisting of many identical copies of a chromosome lying next to each other in parallel.'),
  'D059008': NtO(MH='Time-Lapse Imaging', MS='Recording serial images of a process at regular intervals spaced out over a longer period of time than the time in which the recordings will be played back.'),
  'D059009': NtO(MH='Confederate States of America', MS='The area of those states which seceded in 1861 from the union of the United States of America. They include South Carolina, Mississippi, Florida, Alabama, Georgia, Louisiana, Texas, Virginia, Arkansas, North Carolina, and Tennessee.'),
  'D059010': NtO(MH='Single-Cell Analysis', MS='Assaying the products of or monitoring various biochemical processes and reactions in an individual cell.'),
  'D059012': NtO(MH='Lost to Follow-Up', MS='Study subjects in COHORT STUDIES whose outcomes are unknown e.g., because they could not or did not wish to attend follow-up visits.(from Dictionary of Epidemiology, 5th ed.)'),
  'D059013': NtO(MH='Microbial Consortia', MS='Groups of microorganisms that share metabolic interdependence.'),
  'D059014': NtO(MH='High-Throughput Nucleotide Sequencing', MS='Techniques of nucleotide sequence analysis that increase the range, complexity, sensitivity, and accuracy of results by greatly increasing the scale of operations and thus the number of nucleotides, and the number of copies of each nucleotide sequenced. The sequencing may be done by analysis of the synthesis or ligation products, hybridization to preexisting sequences, etc.'),
  'D059015': NtO(MH='Wireless Technology', MS='Techniques using energy such as radio frequency, infrared light, laser light, visible light, or acoustic energy to transfer information without the use of wires, over both short and long distances.'),
  'D059016': NtO(MH='Tumor Microenvironment', MS="The milieu surrounding neoplasms consisting of cells, vessels, soluble factors, and molecules, that can influence and be influenced by, the neoplasm's growth."),
  'D059017': NtO(MH='Primary Care Nursing', MS='Techniques or methods of patient care used by nurses as primary careproviders.'),
  'D059018': NtO(MH='DNA Contamination', MS='The presence of DNA from a source foreign to the sample being analysed.'),
  'D059019': NtO(MH='Clinical Coding', MS="Process of substituting a symbol or code for a term such as a diagnosis or procedure. (from Slee's Health Care Terms, 3d ed.)"),
  'D059020': NtO(MH='Suicidal Ideation', MS='A risk factor for suicide attempts and completions, it is the most common of all suicidal behavior, but only a minority of ideators engage in overt self-harm.'),
  'D059021': NtO(MH='Laboratory Proficiency Testing', MS='Assessments aimed at determining agreement in diagnostic test results among laboratories. Identical survey samples are distributed to participating laboratories, with results stratified according to testing methodologies.'),
  'D059022': NtO(MH='Food Safety', MS='Activities involved in ensuring the safety of FOOD including avoidance of bacterial and other contamination.'),
  'D059026': NtO(MH='Diagnostic Self Evaluation', MS='A self-evaluation of health status.'),
  'D059027': NtO(MH='Recycling', MS='The extraction and recovery of usable or valuable material from scrap or other discarded materials. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed.)'),
  'D059028': NtO(MH='Airports', MS='Terminal facilities used for aircraft takeoff and landing and including facilities for handling passengers. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed.)'),
  'D059029': NtO(MH='Electronic Waste', MS='Discarded electronic devices containing valuable and sometimes hazardous materials such as LEAD, NICKEL, CADMIUM, and MERCURY. (from http://www.epa.gov/osw/conserve/materials/ecycling/faq.htm#impact accessed 4/25/2010)'),
  'D059030': NtO(MH='United States Department of Defense', MS='A cabinet department in the Executive Branch of the United States Government whose mission is to provide the military forces needed to deter WARFARE and to protect the security of our country.'),
  'D059031': NtO(MH='Anticipation, Psychological', MS='The ability to foresee what is likely to happen on the basis of past experience. It is largely a frontal lobe function.'),
  'D059032': NtO(MH='Learning Curve', MS='The course of learning of an individual or a group. It is a measure of performance plotted over time.'),
  'D059033': NtO(MH='For-Profit Insurance Plans', MS='Health insurance plans that are intended to be for profit.'),
  'D059034': NtO(MH='Not-For-Profit Insurance Plans', MS='Health insurance plans that are not intended to generate profit.'),
  'D059035': NtO(MH='Perioperative Period', MS='The time periods immediately before, during and following a surgical operation.'),
  'D059036': NtO(MH='Photograph', MS='Still image produced from radiation-sensitive materials (sensitive to light, electron beams, or nuclear radiation), generally by means of the chemical action of light on a sensitive film, paper, glass, or metal. Photographs may be positive or negative, opaque or transparent.'),
  'D059037': NtO(MH='Doulas', MS='Trained lay women who provide emotional and/or physical support during obstetric labor and the postpartum period for mothers and their partners.'),
  'D059038': NtO(MH='Black Sea', MS='An inland sea between Europe and Asia. It is connected with the Aegean Sea by the Bosporus, the Sea of Marmara, and the Dardanelles.'),
  'D059039': NtO(MH='Standard of Care', MS='The minimum acceptable patient care, based on statutes, court decisions, policies, or professional guidelines.'),
  'D059040': NtO(MH='Video-Audio Media', MS='Used with articles which include video files or clips, or for articles which are entirely video.'),
  'D059041': NtO(MH='Plagiocephaly', MS='The condition characterized by uneven or irregular shape of the head often in parallelogram shape with a flat spot on the back or one side of the head. It can either result from the premature CRANIAL SUTURE closure (CRANIOSYNOSTOSIS) or from external forces (NONSYNOSTOTIC PLAGIOCEPHALY).'),
  'D059045': NtO(MH='Chromosomal Puffs', MS='Regions along polytene chromosomes that are uncondensed and active in DNA REPLICATION or RNA transcription (GENETIC TRANSCRIPTION).'),
  'D059065': NtO(MH='Medication Reconciliation', MS="The formal process of obtaining a complete and accurate list of each patient's current home medications including name, dosage, frequency, and route of administration, and comparing admission, transfer, and/or discharge medication orders to that list. The reconciliation is done to avoid medication errors."),
  'D059066': NtO(MH='Gross Domestic Product', MS='Value of all final goods and services produced in a country in one year.'),
  'D059085': NtO(MH='Nasal Sprays', MS='Pharmacologic agents delivered into the nostrils in the form of a mist or spray.'),
  'D059125': NtO(MH='Neanderthals', MS='Common name for an extinct species of the Homo genus. Fossils have been found in Europe and Asia. Genetic evidence suggests that limited interbreeding with modern HUMANS (Homo sapiens) took place.'),
  'D059126': NtO(MH='Cronobacter', MS='A genus of gram-negative opportunistic foodborne pathogens.'),
  'D059127': NtO(MH='Phyllobacteriaceae', MS='Family of gram-negative bacteria that occur in leaf nodules and the rhizosphere of higher plants.'),
  'D059128': NtO(MH='Mesorhizobium', MS='A genus of gram-negative, rod-shaped bacteria in the family PHYLLOBACTERIACEAE. They are able to invade root-hairs of a wide range of plants, inciting the production of PLANT ROOT NODULES.'),
  'D059145': NtO(MH='Total Disc Replacement', MS='The replacement of intervertebral discs in the spinal column with artificial devices. The procedure is done in the lumbar or cervical spine to relieve severe pain resulting from INTERVERTEBRAL DISC DEGENERATION.'),
  'D059146': NtO(MH='Metastasectomy', MS='Surgery to remove one or more NEOPLASM METASTASES.'),
  'D059147': NtO(MH='HIV Serosorting', MS='The practice of making choices about SEXUAL PARTNERS based on their HIV status.'),
  'D059165': NtO(MH='Hormesis', MS='Biphasic dose responses of cells or organisms (including microorganisms) to an exogenous or intrinsic factor, in which the factor induces stimulatory or beneficial effects at low doses and inhibitory or adverse effects at high doses.'),
  'D059166': NtO(MH='Osteology', MS='The branch of anatomy that concerns the structure and function of bones.'),
  'D059167': NtO(MH='Mirror Neurons', MS='Neurons that fire when an animal acts or observes the same action of another thus coding the motor response. They were originally discovered in the premotor and parietal cortex of the monkey and studies have shown that neurons that have a similar mechanism are present in humans. Mirror neurons are theorized to be related to social cognition.'),
  'D059168': NtO(MH='Carotid Intima-Media Thickness', MS='A measurement of the thickness of the carotid artery walls. It is measured by B-mode ULTRASONOGRAPHY and is used as a surrogate marker for ATHEROSCLEROSIS.'),
  'D059186': NtO(MH='Chemoradiotherapy, Adjuvant', MS='Combined chemotherapy and radiotherapy given to augment some other form of treatment such as surgery. It is commonly used in the therapy of cancer.'),
  'D059205': NtO(MH='Renewable Energy', MS='Forms of energy that are constantly and rapidly renewed by natural processes such as solar, ocean wave, and wind energy. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D059206': NtO(MH='Natural Gas', MS='A combustible, gaseous mixture of low-molecular weight PARAFFIN hydrocarbons, generated below the surface of the earth. It contains mostly METHANE and ETHANE with small amounts of PROPANE; BUTANES; and higher hydrocarbons, and sometimes NITROGEN; CARBON DIOXIDE; HYDROGEN SULFIDE; and HELIUM. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D059207': NtO(MH='Geothermal Energy', MS='Thermal energy contained in the earth. It can be used directly to supply heat or converted to mechanical or electrical energy. (from McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D059208': NtO(MH='Parasite Load', MS='Measure of the number of the PARASITES present in a host organism.'),
  'D059225': NtO(MH='Nociception', MS='Sensing of noxious mechanical, thermal or chemical stimuli by NOCICEPTORS. It is the sensory component of visceral and tissue pain (NOCICEPTIVE PAIN).'),
  'D059226': NtO(MH='Nociceptive Pain', MS='Dull or sharp aching pain caused by stimulated NOCICEPTORS due to tissue injury, inflammation or diseases. It can be divided into somatic or tissue pain and VISCERAL PAIN.'),
  'D059227': NtO(MH='Postsynaptic Potential Summation', MS='Physiological integration of multiple SYNAPTIC POTENTIAL signals to reach the threshold and initiate postsynaptic ACTION POTENTIALS. In spatial summation stimulations from additional synaptic junctions are recruited to generate s response. In temporal summation succeeding stimuli signals are summed up to reach the threshold. The postsynaptic potentials can be either excitatory or inhibitory (EPSP or IPSP).'),
  'D059228': NtO(MH='Renal Nutcracker Syndrome', MS='Left RENAL VEIN compression between the AORTA, ABDOMINAL and the SUPERIOR MESENTERIC ARTERY. Variable symptoms include HYPERTENSION, RENOVASCULAR; HEMATURIA; and VARICOSE VEINS.'),
  'D059229': NtO(MH='Osteotomy, Sagittal Split Ramus', MS='Sagittal sectioning and repositioning of the ramus of the MANDIBLE to correct a mandibular retrusion, MALOCCLUSION, ANGLE CLASS III; and PROGNATHISM. The oblique sectioning line consists of multiple cuts horizontal and vertical to the mandibular ramus.'),
  'D059230': NtO(MH='Reflex, Trigeminocardiac', MS='Sudden occurrence of BRADYCARDIA or HEART ARREST induced by manipulations of the MAXILLARY NERVE AND MANDIBULAR NERVE during a craniomaxillofacial or oral surgery. It is the maxillary and mandibular variants of OCULOCARDIAC REFLEX.'),
  'D059232': NtO(MH='Laboratory Personnel', MS='Professionals, technicians, and assistants staffing LABORATORIES.'),
  'D059245': NtO(MH='Transient Tachypnea of the Newborn', MS='Abnormal increase in RESPIRATORY RATE in the newborn. It is self-limiting and attributed to the delayed fetal lung fluid clearance often in CAESAREAN SECTION delivery.'),
  'D059246': NtO(MH='Tachypnea', MS='Increased RESPIRATORY RATE.'),
  'D059247': NtO(MH='Fertility Preservation', MS='A method of providing future reproductive opportunities before a medical treatment with known risk of loss of fertility. Typically reproductive organs or tissues (e.g., sperm, egg, embryos and ovarian or testicular tissues) are cryopreserved for future use before the medical treatment (e.g., chemotherapy, radiation) begins.'),
  'D059248': NtO(MH='Chemoradiotherapy', MS='Treatment that combines chemotherapy with radiotherapy.'),
  'D059249': NtO(MH='Otomycosis', MS='Fungus infection of the external ear, usually by ASPERGILLUS species'),
  'D059250': NtO(MH='Administration, Metronomic', MS='Administration of low doses of a drug or a drug combination over prolonged periods of time usually at a regular interval.'),
  'D059265': NtO(MH='Visceral Pain', MS='Pain originating from internal organs (VISCERA) associated with autonomic phenomena (PALLOR; SWEATING; NAUSEA; and VOMITING). It often becomes a REFERRED PAIN.'),
  'D059266': NtO(MH='Bisphosphonate-Associated Osteonecrosis of the Jaw', MS='Necrotic jaws or other maxillofacial skeleton necrosis associated with bisphosphonate use (see BISPHOSPHONATES). Injury, dental procedures, and trauma can trigger the necrotic process.'),
  'D059267': NtO(MH='Cardiomegaly, Exercise-Induced', MS='Heart enlargement and other remodeling in cardiac morphology and electrical circutry found in individuals who participate in intense repeated exercises.'),
  'D059268': NtO(MH='Atrophic Vaginitis', MS='Inflammation of the vagina due to thinning of the vaginal wall and decreased lubrication associated with reduced estrogen levels at MENOPAUSE.'),
  'D059269': NtO(MH='Chilaiditi Syndrome', MS='Interposition of a portion of the colon (e.g., SIGMOID COLON) between the liver and the diaphragm. It is associated with abdominal pain, vomiting, constipation, anorexia and volvulus. Chilaiditi anomaly refers to asymptomatic interposition.'),
  'D059270': NtO(MH='Petrositis', MS='Inflammation of PETROUS BONE.'),
  'D059285': NtO(MH='Pregnancy, Twin', MS='The condition of carrying TWINS simultaneously.'),
  'D059286': NtO(MH='Pregnancy, Triplet', MS='The condition of carrying TRIPLETS simultaneously.'),
  'D059287': NtO(MH='Pregnancy, Quadruplet', MS='The condition of carrying QUADRUPLETS simultaneously.'),
  'D059288': NtO(MH='Pregnancy, Quintuplet', MS='The condition of carrying QUINTUPLETS simultaneously.'),
  'D059289': NtO(MH='Vascular Stiffness', MS='Loss of vascular ELASTICITY due to factors such as AGING; and ARTERIOSCLEROSIS. Increased arterial stiffness is one of the RISK FACTORS for many CARDIOVASCULAR DISEASES.'),
  'D059290': NtO(MH='Dopaminergic Neurons', MS='Neurons whose primary neurotransmitter is DOPAMINE.'),
  'D059305': NtO(MH='Diet, High-Fat', MS='Consumption of excessive DIETARY FATS.'),
  'D059325': NtO(MH='Intra-Abdominal Hypertension', MS='Pathological elevation of intra-abdominal pressure (>12 mm Hg). It may develop as a result of SEPSIS; PANCREATITIS; capillary leaks, burns, or surgery. When the pressure is higher than 20 mm Hg, often with end-organ dysfunction, it is referred to as abdominal compartment syndrome.'),
  'D059326': NtO(MH='Serotonergic Neurons', MS='Neurons whose primary neurotransmitter is SEROTONIN.'),
  'D059327': NtO(MH='Brachydactyly', MS='Congenital anomaly of abnormally short fingers or toes.'),
  'D059328': NtO(MH='Diffuse Noxious Inhibitory Control', MS='A physiological process by which the perception of pain at a local area of the body is inhibited by a second painful stimulus administered at a distal body site'),
  'D059329': NtO(MH='Cholinergic Neurons', MS='Neurons whose primary neurotransmitter is ACETYLCHOLINE.'),
  'D059330': NtO(MH='GABAergic Neurons', MS='Neurons whose primary neurotransmitter is GAMMA-AMINOBUTYRIC ACID.'),
  'D059331': NtO(MH='Adrenergic Neurons', MS='Neurons whose primary neurotransmitter is EPINEPHRINE.'),
  'D059333': NtO(MH='Central Nervous System Sensitization', MS='An increased response to stimulation that is mediated by amplification of signaling in the CENTRAL NERVOUS SYSTEM (CNS).'),
  'D059345': NtO(MH='Cerebral Small Vessel Diseases', MS='Pathological processes or diseases where cerebral MICROVESSELS show abnormalities. They are often associated with aging, hypertension and risk factors for lacunar infarcts (see LACUNAR INFARCTION); LEUKOARAIOSIS; and CEREBRAL HEMORRHAGE.'),
  'D059346': NtO(MH='Magnets', MS='Objects that produce a magnetic field.'),
  'D059347': NtO(MH='Cardio-Renal Syndrome', MS='Condition where a primary dysfunction of either heart or kidney results in failure of the other organ (e.g., HEART FAILURE with worsening RENAL INSUFFICIENCY).'),
  'D059348': NtO(MH='Peripheral Nerve Injuries', MS='Injuries to the PERIPHERAL NERVES.'),
  'D059349': NtO(MH='Urine Specimen Collection', MS='Methods or procedures used to obtain samples of URINE.'),
  'D059350': NtO(MH='Chronic Pain', MS='Aching sensation that persists for more than a few months. It may or may not be associated with trauma or disease, and may persist after the initial injury has healed. Its localization, character, and timing are more vague than with acute pain.'),
  'D059351': NtO(MH='Organ Sparing Treatments', MS='Techniques, procedures, and therapies carried out on diseased organs in such a way to avoid complete removal of the organ and preserve the remaining organ function.'),
  'D059352': NtO(MH='Musculoskeletal Pain', MS='Discomfort stemming from muscles, LIGAMENTS, tendons, and bones.'),
  'D059365': NtO(MH='Nonsense Mediated mRNA Decay', MS='An mRNA metabolic process that distinguishes a normal STOP CODON from a premature stop codon (NONSENSE CODON) and facilitates rapid degradation of aberrant mRNAs containing premature stop codons.'),
  'D059366': NtO(MH='Asthma, Occupational', MS='Asthma attacks caused, triggered, or exacerbated by OCCUPATIONAL EXPOSURE.'),
  'D059367': NtO(MH='RNA Cleavage', MS='A reaction that severs one of the sugar-phosphate linkages of the phosphodiester backbone of RNA. It is catalyzed enzymatically, chemically, or by radiation. Cleavage may be exonucleolytic, or endonucleolytic.'),
  'D059368': NtO(MH='RNA Isoforms', MS='The different gene transcripts generated from a single gene by RNA EDITING or ALTERNATIVE SPLICING of RNA PRECURSORS.'),
  'D059369': NtO(MH='Pott Puffy Tumor', MS='Infection of the FRONTAL BONE often as a complication of FRONTAL SINUSITIS or trauma to the frontal bone and skull. It is characterized by subperiosteal abscess with OSTEOMYELITIS.'),
  'D059370': NtO(MH='RNA Folding', MS='The processes of RNA tertiary structure formation.'),
  'D059371': NtO(MH='DNA, B-Form', MS='The most common form of DNA found in nature. It is a right-handed helix with 10 base pairs per turn, a pitch of 0.338 nm per base pair and a helical diameter of 1.9 nm.'),
  'D059372': NtO(MH='Nucleotide Motifs', MS='Commonly observed BASE SEQUENCE or nucleotide structural components which can be represented by a CONSENSUS SEQUENCE or a SEQUENCE LOGO.'),
  'D059373': NtO(MH='Mastodynia', MS='Pain in the breast generally classified as cyclical (associated with menstrual periods), or noncyclical, i.e. originating from the breast or nearby muscles or joints, ranging from minor discomfort to severely incapacitating.'),
  'D059385': NtO(MH='Plyometric Exercise', MS='Exercises in which muscles are repeatedly and rapidly stretched, followed by shortening, concentric MUSCLE CONTRACTION (e.g. jumping and rebounding). They are designed to exert maximal force in minimal time by increasing STRETCH REFLEX.'),
  'D059386': NtO(MH='Reverse Genetics', MS='The use of techniques that produce a functional MUTATION or an effect on GENE EXPRESSION of a specific gene of interest in order to identify the role or activity of the gene product of that gene.'),
  'D059387': NtO(MH='Sphenopalatine Ganglion Block', MS='Method of treating pain associated with the sphenopalatine ganglion located in the PTERYGOPALATINE FOSSA, posterior to the middle nasal turbinate. The transnasal approach involves application of suitable local anesthetic to the mucous membrane overlying the ganglion.'),
  'D059388': NtO(MH='Pelvic Girdle Pain', MS='Discomfort associated with the bones that make up the pelvic girdle. It occurs frequently during pregnancy.'),
  'D059390': NtO(MH='Breakthrough Pain', MS='Acute pain that comes on rapidly despite the use of pain medication.'),
  'D059392': NtO(MH='Petroleum Pollution', MS='Release of oil into the environment usually due to human activity.'),
  'D059405': NtO(MH='Sun Protection Factor', MS='A measure of relative protection provided by SUNSCREENING AGENTS against burns due to ultraviolet (UV) radiation from a light source.'),
  'D059407': NtO(MH='Pinguecula', MS='A yellowish thickened lesion on the conjunctiva near the CORNEA representing a benign degenerative change in the CONJUNCTIVA caused by the leakage and deposition of certain blood proteins through the permeable capillaries near the LIMBUS.'),
  'D059408': NtO(MH='Pain Management', MS='A form of therapy that employs a coordinated and interdisciplinary approach for easing the suffering and improving the quality of life of those experiencing pain.'),
  'D059409': NtO(MH='Stroke, Lacunar', MS='Stroke caused by lacunar infarction or other small vessel diseases of the brain. It features hemiparesis (see PARESIS), hemisensory, or hemisensory motor loss.'),
  'D059411': NtO(MH='Lower Urinary Tract Symptoms', MS='Symptoms of disorders of the lower urinary tract including frequency, NOCTURIA; urgency, incomplete voiding, and URINARY INCONTINENCE. They are often associated with OVERACTIVE BLADDER; URINARY INCOMPETENCE; and INTERSTITIAL CYSTITIS. Lower urinary tract symptoms in males were traditionally called PROSTATISM.'),
  'D059413': NtO(MH='Intraabdominal Infections', MS='Infection within the PERITONEAL CAVITY. A frequent cause is an ANASTOMOTIC LEAK following surgery.'),
  'D059425': NtO(MH='Interferon-gamma Release Tests', MS='The assay of INTERFERON-GAMMA released from lymphocytes after their exposure to a specific test antigen, to check for IMMUNOLOGIC MEMORY resulting from a previous exposure to the antigen. The amount of interferon-gamma released is usually assayed by an ENZYME-LINKED IMMUNOSORBENT ASSAY.'),
  'D059445': NtO(MH='Anhedonia', MS='Inability to experience pleasure due to impairment or dysfunction of normal psychological and neurobiological mechanisms. It is a symptom of many PSYCHOTIC DISORDERS (e.g., DEPRESSIVE DISORDER, MAJOR; and SCHIZOPHRENIA).'),
  'D059446': NtO(MH='Heterotaxy Syndrome', MS='Abnormal thoracoabdominal VISCERA arrangement (visceral heterotaxy) or malformation that involves additional CONGENITAL HEART DEFECTS (e.g., heart isomerism; DEXTROCARDIA) and/or abnormal SPLEEN (e.g., asplenia and polysplenia). Irregularities with the central nervous system, the skeleton and urinary tract are often associated with the syndrome.'),
  'D059447': NtO(MH='Cell Cycle Checkpoints', MS='Regulatory signaling systems that control the progression through the CELL CYCLE. They ensure that the cell has completed, in the correct order and without mistakes, all the processes required to replicate the GENOME and CYTOPLASM, and divide them equally between two daughter cells. If cells sense they have not completed these processes or that the environment does not have the nutrients and growth hormones in place to proceed, then the cells are restrained (or "arrested") until the processes are completed and growth conditions are suitable.'),
  'D059451': NtO(MH='Biosimilar Pharmaceuticals', MS='Biological products that are imitations but not exact replicas of innovator biological products.'),
  'D059466': NtO(MH='White Coat Hypertension', MS='Phenomenon where BLOOD PRESSURE readings are elevated only when taken in clinical settings.'),
  'D059467': NtO(MH='Transcriptome', MS='The pattern of GENE EXPRESSION at the level of genetic transcription in a specific organism or under specific circumstances in specific cells.'),
  'D059468': NtO(MH='Masked Hypertension', MS='Phenomenon where increased BLOOD PRESSURE readings taken in non-clinical settings (e.g., HOME BLOOD PRESSURE MONITORING) do not replicate in clinical settings.'),
  'D059470': NtO(MH='Tetraspanins', MS='A large superfamily of cell surface membrane proteins characterized by their four transmembrane domains. They play a role in a variety of processes such as cellular adhesion and motility. They may be involved in the organization of cell surface MEMBRANE MICRODOMAINS that regulate the activation of LEUKOCYTES.'),
  'D059471': NtO(MH='In Vitro Oocyte Maturation Techniques', MS='Methods used to induce premature oocytes, that are maintained in tissue culture, to progress through developmental stages including to a stage that is competent to undergo FERTILIZATION.'),
  'D059472': NtO(MH='Exome', MS='That part of the genome that corresponds to the complete complement of EXONS of an organism or cell.'),
  'D059485': NtO(MH='Gastropexy', MS='Surgical fixation of the stomach to the abdominal wall.'),
  'D059505': NtO(MH='Telomere Homeostasis', MS='Maintenance of TELOMERE length. During DNA REPLICATION, chromosome ends loose some of their telomere sequence (TELOMERE SHORTENING.) Various cellular mechanism are involved in repairing, extending, and recapping the telomere ends.'),
  'D059506': NtO(MH='Telomere Shortening', MS='The loss of some TELOMERE sequence during DNA REPLICATION of the first several base pairs of a linear DNA molecule; or from DNA DAMAGE. Cells have various mechanisms to restore length (TELOMERE HOMEOSTASIS.) Telomere shortening is involved in the progression of CELL AGING.'),
  'D059525': NtO(MH='Supraglottitis', MS='Inflammation of the EPIGLOTTIS and supraglottic structures including the PHARYNX; UVULA; base of tongue; and aryepiglottic folds. It is usually caused by HAEMOPHILUS INFLUENZAE in children but often by different organisms in adults.'),
  'D059526': NtO(MH='Foraminotomy', MS='Surgical enlargement of the intervertebral foramina to relieve NERVE ROOT COMPRESSION.'),
  'D059545': NtO(MH='Paraneoplastic Syndromes, Ocular', MS='Ocular manifestations secondary to various NEOPLASMS in which antibodies to antigens of the primary tumor cross-react with ocular antigens. This autoimmune response often leads to visual loss and other ocular dysfunctions.'),
  'D059546': NtO(MH='Sinus Floor Augmentation', MS='Guided BONE TRANSPLANTATION of the MAXILLARY SINUS surface with a BONE SUBSTITUTE grafting. It increases the bone volume at the site of the DENTAL IMPLANT and helps stabilize it.'),
  'D059547': NtO(MH='Stereocilia', MS='Mechanosensing organelles of hair cells which respond to fluid motion or fluid pressure changes. They have various functions in many different animals, but are primarily used in hearing.'),
  'D059549': NtO(MH='Anterior Cruciate Ligament Reconstruction', MS='Rebuilding of the ANTERIOR CRUCIATE LIGAMENT to restore functional stability of the knee. AUTOGRAFTING or ALLOGRAFTING of tissues is often used.'),
  'D059550': NtO(MH='Toxicity Tests, Subchronic', MS='Experiments designed to determine the potential toxic effects of mid-term (a few months) exposure to a chemical or chemicals.'),
  'D059551': NtO(MH='Toxicity Tests, Subacute', MS='Experiments designed to determine the potential toxic effects of a short-term (a few weeks) exposure to a chemical or chemicals.'),
  'D059552': NtO(MH='Caves', MS='Geological formations consisting of underground enclosures with access from the surface.'),
  'D059565': NtO(MH='G2 Phase Cell Cycle Checkpoints', MS='CELL CYCLE regulatory signaling systems that are triggered by DNA DAMAGE or lack of nutrients during G2 PHASE. When triggered they restrain cells transitioning from G2 phase to M PHASE.'),
  'D059566': NtO(MH='M Phase Cell Cycle Checkpoints', MS='The cellular signaling system that halts the progression of cells through MITOSIS or MEIOSIS if a defect that will affect CHROMOSOME SEGREGATION is detected.'),
  'D059585': NtO(MH='G1 Phase Cell Cycle Checkpoints', MS='Regulatory signaling systems that control the progression of the CELL CYCLE through the G1 PHASE and allow transition to S PHASE when the cells are ready to undergo DNA REPLICATION. DNA DAMAGE, or the deficiencies in specific cellular components or nutrients may cause the cells to halt before progressing through G1 phase.'),
  'D059586': NtO(MH='Duddingtonia', MS='A genus of ascomycetous mitosporic fungi in the family Orbiliaceae. It is used for the biological control of nematodes in livestock.'),
  'D059587': NtO(MH='Calcifying Nanoparticles', MS='Protein-mineral complexes that comprise substrates needed for the normal calcium-carbonate-phosphate homeostasis. Nanobacteria was the prior name for the particles which were originally thought to be microorganisms.'),
  'D059605': NtO(MH='Dental Implant-Abutment Design', MS='The plan and delineation of DENTAL IMPLANT fitting with DENTAL ABUTMENT.'),
  'D059606': NtO(MH='Polydipsia', MS='Excessive thirst manifested by excessive fluid intake. It is characteristic of many diseases such as DIABETES MELLITUS; DIABETES INSIPIDUS; and NEPHROGENIC DIABETES INSIPIDUS. The condition may be psychogenic in origin.'),
  'D059607': NtO(MH='Polydipsia, Psychogenic', MS='A clinical disorder characterized by excessive fluid intake (polydipsia); HYPONATREMIA; and POLYURIA in SCHIZOPHRENIA and other psychiatric disorders. Impaired water metabolism in psychogenic polydipsia can result in WATER INTOXICATION.'),
  'D059608': NtO(MH='Laryngocele', MS='Congenital anomalous dilitation of the laryngeal saccule that may extend internally into the airway or externally through the thyrohyoid membrane.'),
  'D059609': NtO(MH='Mallory Bodies', MS='Cytoplasmic hyaline inclusions in HEPATOCYTES. They are associated with ALCOHOLIC STEATOHEPATITIS and non-alcoholic STEATOHEPATITIS, but are also present in benign and malignant hepatocellular neoplasms, and metabolic, toxic, and chronic cholestatic LIVER DISEASES.'),
  'D059611': NtO(MH='Balamuthia mandrillaris', MS='A species of free-living soil amoeba in the family Balamuthiidae, causing AMEBIASIS and a deadly form of ENCEPHALITIS in humans.'),
  'D059625': NtO(MH='Liquid-Liquid Extraction', MS='The removal of a soluble component from a liquid mixture by contact with a second liquid, immiscible with the carrier liquid, in which the component is preferentially soluble. (McGraw-Hill Dictionary of Scientific and Technical Terms, 6th ed)'),
  'D059627': NtO(MH='Liquid Phase Microextraction', MS='Miniaturized methods of liquid-liquid extraction.'),
  'D059629': NtO(MH='Signal-To-Noise Ratio', MS='The comparison of the quantity of meaningful data to the irrelevant or incorrect data.'),
  'D059630': NtO(MH='Mesenchymal Stem Cells', MS='Mesenchymal stem cells, also referred to as multipotent stromal cells or mesenchymal stromal cells are multipotent, non-hematopoietic adult stem cells that are present in multiple tissues, including BONE MARROW; ADIPOSE TISSUE; and WHARTON JELLY. Mesenchymal stem cells can differentiate into mesodermal lineages, such as adipocytic, osteocytic and chondrocytic.'),
  'D059631': NtO(MH='Wharton Jelly', MS='Jelly-like connective tissue of the UMBILICAL CORD that contains MESENCHYMAL STEM CELLS.'),
  'D059645': NtO(MH='Mutation Rate', MS='The number of mutations that occur in a specific sequence, GENE, or GENOME over a specified period of time such as years, CELL DIVISIONS, or generations.'),
  'D059646': NtO(MH='Genome Size', MS='The amount of DNA (or RNA) in one copy of a genome.'),
  'D059647': NtO(MH='Gene-Environment Interaction', MS='The combined effects of genotypes and environmental factors together on phenotypic characteristics.'),
  'D059648': NtO(MH='Kisspeptins', MS='Intercellular signaling peptides that were originally characterized by their ability to suppress NEOPLASM METASTASIS. Kisspeptins have since been found to play an important role in the neuroendocrine regulation of REPRODUCTION.'),
  'D059666': NtO(MH='Lysosomal-Associated Membrane Protein 3', MS='A lysosomal-associated membrane glycoprotein that is expressed at high levels in mature DENDRITIC CELLS.'),
  'D059685': NtO(MH='Herniorrhaphy', MS='Surgical procedures undertaken to repair abnormal openings through which tissue or parts of organs can protrude or are already protruding.'),
  'D059705': NtO(MH='Polar Bodies', MS='Minute cells produced during development of an OOCYTE as it undergoes MEIOSIS. A polar body contains one of the nuclei derived from the first or second meiotic CELL DIVISION. Polar bodies have practically no CYTOPLASM. They are eventually discarded by the oocyte. (from King & Stansfield, A Dictionary of Genetics, 4th ed)'),
  'D059706': NtO(MH='Aliivibrio', MS='A genus of gram-negative, rod-shaped bacteria in the family VIBRIONACEAE. They are found in the marine environment often associated with animals.'),
  'D059707': NtO(MH='Aliivibrio Infections', MS='Infections with bacteria of the genus ALIIVIBRIO.'),
  'D059708': NtO(MH='Ultrasonic Surgical Procedures', MS='The use of HIGH-ENERGY SHOCK WAVES, in the frequency range of 20-60 kHz, to cut through or remove tissue. The tissue fragmentation by ultrasonic surgical instruments is caused by mechanical effects not heat as with HIGH-INTENSITY FOCUSED ULTRASOUND ABLATION.'),
  'D059725': NtO(MH='Interactive Ventilatory Support', MS="Mechanical ventilation delivered to match the patient's efforts in breathing as detected by the interactive ventilation device."),
  'D059745': NtO(MH='Piezosurgery', MS='The use of HIGH-ENERGY SHOCK WAVES, in the frequency range of 20-30 kHz, to cut through mineralized tissue.'),
  'D059746': NtO(MH='Heimlich Maneuver', MS='An emergency treatment commonly used to clear food and other foreign objects causing AIRWAY OBSTRUCTION.'),
  'D059747': NtO(MH='Nasal Surgical Procedures', MS='Surgical operations on the nose and nasal cavity.'),
  'D059748': NtO(MH='Proteolysis', MS='Cleavage of proteins into smaller peptides or amino acids either by PROTEASES or non-enzymatically (e.g., Hydrolysis). It does not include Protein Processing, Post-Translational.'),
  'D059765': NtO(MH='Homologous Recombination', MS='An exchange of DNA between matching or similar sequences.'),
  'D059766': NtO(MH='DNA End-Joining Repair', MS='The repair of DOUBLE-STRAND DNA BREAKS by rejoining the broken ends of DNA to each other directly.'),
  'D059767': NtO(MH='Recombinational DNA Repair', MS='Repair of DNA DAMAGE by exchange of DNA between matching sequences, usually between the allelic DNA (ALLELES) of sister chromatids.'),
  'D059785': NtO(MH='Karyotype', MS='The full set of CHROMOSOMES presented as a systematized array of METAPHASE chromosomes from a photomicrograph of a single CELL NUCLEUS arranged in pairs in descending order of size and according to the position of the CENTROMERE. (From Stedman, 25th ed)'),
  'D059786': NtO(MH='Abnormal Karyotype', MS='A variation from the normal set of chromosomes characteristic of a species.'),
  'D059787': NtO(MH='Acute Pain', MS='Intensely discomforting, distressful, or agonizing sensation associated with trauma or disease, with well-defined location, character, and timing.'),
  'D059788': NtO(MH='Dried Blood Spot Testing', MS='Techniques for using whole blood samples collected on filter paper for a variety of clinical laboratory tests.'),
  'D059805': NtO(MH='HLA-DR alpha-Chains', MS='Transmembrane proteins that form the alpha subunits of the HLA-DR antigens. They are also referred to as the HLA-DR heavy chains.'),
  'D059806': NtO(MH='HLA-DR beta-Chains', MS='Transmembrane proteins that form the beta subunits of the HLA-DR antigens. They are also referred to as the HLA-DR light chains.'),
  'D059807': NtO(MH='S Phase Cell Cycle Checkpoints', MS='Cell regulatory signaling system that controls progression through S PHASE and stabilizes the replication forks during conditions that could affect the fidelity of DNA REPLICATION, such as DNA DAMAGE or depletion of nucleotide pools.'),
  'D059808': NtO(MH='Polyphenols', MS='A large class of organic compounds having more than one PHENOL group.'),
  'D059810': NtO(MH='HLA-DR Serological Subtypes', MS='HLA-DR antigen subtypes that have been classified according to their affinity to specific ANTIBODIES. The DNA sequence analyses of HLA-DR ALPHA-CHAINS and HLA-DR BETA-CHAINS has for the most part revealed the specific alleles that are responsible for each serological subtype.'),
  'D059811': NtO(MH='HLA-DRB1 Chains', MS='A subtype of HLA-DRB beta chains that includes over one hundred allele variants. The HLA-DRB1 subtype is associated with several of the HLA-DR SEROLOGICAL SUBTYPES.'),
  'D059825': NtO(MH='Physical Therapists', MS='Persons trained in PHYSICAL THERAPY SPECIALTY to make use of PHYSICAL THERAPY MODALITIES to prevent, correct, and alleviate movement dysfunction.'),
  'D059826': NtO(MH='HLA-DRB3 Chains', MS='A subtype of HLA-DRB beta chains that includes over 50 allelic variants. The HLA-DRB3 beta-chain subtype is associated with HLA-DR52 serological subtype.'),
  'D059827': NtO(MH='Agricultural Inoculants', MS='Beneficial microorganisms (bacteria or fungi) encapsulated in carrier material and applied to the environment for remediation and enhancement of agricultural productivity.'),
  'D059828': NtO(MH='Plant Cells', MS='Basic functional unit of plants.'),
  'D059845': NtO(MH='HLA-DRB4 Chains', MS='A subtype of HLA-DRB beta chains that is associated with the HLA-DR53 serological subtype.'),
  'D059846': NtO(MH='Analgesics, Short-Acting', MS='Pain-alleviating drugs characterized by rapid action time.'),
  'D059847': NtO(MH='HLA-DRB5 Chains', MS='A subtype of HLA-DRB beta chains that is associated with the HLA-DR51 serological subtype.'),
  'D059848': NtO(MH='HLA-DQ alpha-Chains', MS='Transmembrane proteins that form the alpha subunits of the HLA-DQ antigens.'),
  'D059865': NtO(MH='Endoplasmic Reticulum Stress', MS='Various physiological or molecular disturbances that impair ENDOPLASMIC RETICULUM function. It triggers many responses, including UNFOLDED PROTEIN RESPONSE, which may lead to APOPTOSIS; and AUTOPHAGY.'),
  'D059866': NtO(MH='HLA-DQ beta-Chains', MS='Transmembrane proteins that form the beta subunits of the HLA-DQ antigens.'),
  'D059867': NtO(MH='HLA-DP alpha-Chains', MS='Transmembrane proteins that form the alpha subunits of the HLA-DP antigens.'),
  'D059868': NtO(MH='HLA-DP beta-Chains', MS='Transmembrane proteins that form the beta subunits of the HLA-DP antigens.'),
  'D059869': NtO(MH='Radiostereometric Analysis', MS='Methods for performing three-dimensional measurement and motion analysis using stereoscopic radiographs.'),
  'D059870': NtO(MH='HLA-A11 Antigen', MS='A specific HLA-A surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-A*11 allele family.'),
  'D059871': NtO(MH='HLA-A24 Antigen', MS='A specific HLA-A surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-A*24 allele family.'),
  'D059885': NtO(MH='Kasabach-Merritt Syndrome', MS='Rapidly growing vascular lesion along the midline axis of the neck, upper trunk, and extremities that is characterized by CONSUMPTION COAGULOPATHY; THROMBOCYTOPENIA; and HEMOLYTIC ANEMIA. It is often associated with infantile Kaposiform HEMANGIOENDOTHELIOMA and other vascular tumors such as tufted ANGIOMA.'),
  'D059905': NtO(MH='Endocarditis, Non-Infective', MS='Formation of a non-infectious THROMBUS, referred to as vegetation, on previously undamaged ENDOCARDIUM. It usually occurs as a complication of connective-tissue diseases and cancers because of the associated hypercoagulable state (see THROMBOPHILIA).'),
  'D059906': NtO(MH='Neuroimaging', MS='Non-invasive methods of visualizing the CENTRAL NERVOUS SYSTEM, especially the brain, by various imaging modalities.'),
  'D059907': NtO(MH='Functional Neuroimaging', MS='Methods for visualizing REGIONAL BLOOD FLOW, metabolic, electrical, or other physiological activities in the CENTRAL NERVOUS SYSTEM using various imaging modalities.'),
  'D059908': NtO(MH='HLA-B13 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*13 allele family.'),
  'D059909': NtO(MH='HLA-B14 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*14 allele family.'),
  'D059910': NtO(MH='HLA-B15 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*15 allele family.'),
  'D059911': NtO(MH='HLA-B18 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*18 allele family.'),
  'D059912': NtO(MH='HLA-B37 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*37 allele family.'),
  'D059925': NtO(MH='Anatomic Landmarks', MS='Reference points located by visual inspection, palpation, or computer assistance, that are useful in localizing structures on or within the human body.'),
  'D059945': NtO(MH='HLA-B38 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*38 allele family.'),
  'D059946': NtO(MH='HLA-B39 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*39 allele family.'),
  'D059947': NtO(MH='HLA-B40 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*40 allele family.'),
  'D059948': NtO(MH='HLA-B44 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*44 allele family.'),
  'D059949': NtO(MH='HLA-B51 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*51 allele family.'),
  'D059950': NtO(MH='HLA-B52 Antigen', MS='A specific HLA-B surface antigen subtype. Members of this subtype contain alpha chains that are encoded by the HLA-B*52 allele family.'),
  'D059951': NtO(MH='HLA-G Antigens', MS='Class I human histocompatibility (HLA) surface antigens encoded by alleles on locus B of the HLA complex. The HLA-G antigens are considered non-classical class I antigens due to their distinct tissue distribution which differs from HLA-A; HLA-B; and HLA-C antigens. Note that several isoforms of HLA-G antigens result from alternative splicing of messenger RNAs produced from the HLA-G*01 allele.'),
  'D059952': NtO(MH='Pelvic Floor Disorders', MS='Injury, weakening, or PROLAPSE of the pelvic muscles, surrounding connective tissues or ligaments (PELVIC FLOOR).'),
  'D059965': NtO(MH='Merkel cell polyomavirus', MS='A species of POLYOMAVIRUS suspected to be the cause of most cases of MERKEL CELL CARCINOMA, a rare but highly lethal form of skin cancer.'),
  'D059985': NtO(MH='Cryogels', MS='Macroporous hydrogels that are produced at subzero temperatures. Cryogels have pores that are produced by growing ice crystals and have been developed with a tissue-like elasticity that is suitable for cell immunization experiments.'),
  'D060005': NtO(MH='Genotyping Techniques', MS="Methods used to determine individuals' specific ALLELES or SNPS (single nucleotide polymorphisms)."),
  'D060026': NtO(MH='Endophytes', MS='An endosymbiont that is either a bacterium or fungus living part of its life in a plant. Endophytes can benefit host plants by preventing pathogenic organisms from colonizing them.'),
  'D060045': NtO(MH='Genes, Modifier', MS='GENES with ALLELES that affect the PHENOTYPE associated with a nonallelic gene.'),
  'D060046': NtO(MH='Maintenance Chemotherapy', MS='Treatment designed to help prevent a relapse of a disease following the successful primary treatments (INDUCTION CHEMOTHERAPY and CONSOLIDATION CHEMOTHERAPY) with a long-term low-dose drug therapy.'),
  'D060047': NtO(MH='Reproductive Isolation', MS='Mechanisms that prevent different populations from exchanging genes (GENE FLOW), resulting in or maintaining GENETIC SPECIATION. It can either prevent mating to take place or ensure that any offspring produced is either inviable or sterile, thereby preventing further REPRODUCTION.'),
  'D060048': NtO(MH='Slipped Capital Femoral Epiphyses', MS='A developmental deformity in which the metaphysis of the FEMUR moves proximally and anteriorly away from FEMUR HEAD (epiphysis) at the upper GROWTH PLATE. It is most common in male adolescents and is associated with a greater risk of early OSTEOARTHRITIS of the hip.'),
  'D060049': NtO(MH='Asymmetric Cell Division', MS='Unequal cell division that results in daughter cells of different sizes.'),
  'D060050': NtO(MH='Angina, Stable', MS='Persistent and reproducible chest discomfort usually precipitated by a physical exertion that dissipates upon cessation of such an activity. The symptoms are manifestations of MYOCARDIAL ISCHEMIA.'),
  'D060051': NtO(MH='Occupational Injuries', MS='Injuries sustained from incidents in the course of work-related activities.'),
  'D060052': NtO(MH='Nasolabial Fold', MS='A skin crease on each side of the face that runs from the outer corners of the nose to the corner of the mouth. It is a common site of PLASTIC SURGERY.'),
  'D060053': NtO(MH='Transplant Donor Site', MS='The body location or part from which tissue is taken for TRANSPLANTATION.'),
  'D060054': NtO(MH='Agrobacterium', MS='A genus of gram negative, aerobic, rod-shaped bacteria found in soil, plants, and marine mud.'),
  'D060066': NtO(MH='Protein Interaction Maps', MS='Graphs representing sets of measurable, non-covalent physical contacts with specific PROTEINS in living organisms or in cells.'),
  'D060085': NtO(MH='Coinfection', MS='Simultaneous infection of a host organism by two or more pathogens. In virology, coinfection commonly refers to simultaneous infection of a single cell by two or more different viruses.'),
  'D060086': NtO(MH='Hydrothermal Vents', MS='Hot springs on the ocean floor. They are commonly found near volcanically active places such as mid-oceanic ridges.'),
  'D060105': NtO(MH='Animal Shells', MS='The hard rigid covering of animals including MOLLUSCS; TURTLES; INSECTS; and crustaceans.'),
  'D060106': NtO(MH='Lakes', MS='Inland bodies of still or slowly moving FRESH WATER or salt water, larger than a pond, and supplied by RIVERS and streams.'),
  'D060125': NtO(MH='Characiformes', MS='An order of fresh water fish with 18 families and over 1600 species. The order includes CHARACINS, hatchetfish, piranhas, and TETRAS.'),
  'D060126': NtO(MH='Characidae', MS='A family of fresh water fish in the order CHARACIFORMES, which includes the Tetras.'),
  'D060127': NtO(MH='Kangaroo-Mother Care Method', MS='A method of continuously holding a partially wrapped baby to the chest, involving skin-to-skin contact. Originally it was a method of caring for LOW-BIRTH-WEIGHT INFANT in developing countries and is now more widespread in developed nations. Aside from encouraging breast feeding, the extra sleep that the infant gets assists in regulating body temperature, helps the baby conserve energy, and redirects calorie expenditures toward growth and weight gain.'),
  'D060145': NtO(MH='Text Messaging', MS='Communication between CELL PHONE users via the Short Message Service protocol which allows the interchange of short written messages.'),
  'D060146': NtO(MH='Ecotype', MS='Geographic variety, population, or race, within a species, that is genetically adapted to a particular habitat. An ecotype typically exhibits phenotypic differences but is capable of interbreeding with other ecotypes.'),
  'D060148': NtO(MH='HapMap Project', MS='A coordinated international effort to identify and catalog patterns of linked variations (HAPLOTYPES) found in the human genome across the entire human population.'),
  'D060149': NtO(MH='Tetraspanin 30', MS='Ubiquitously-expressed tetraspanin protein that is found in late ENDOSOMES and LYSOSOMES. It functions in intracellular protein transport and signaling.'),
  'D060151': NtO(MH='B-Lymphocytes, Regulatory', MS='B-cells that have a role in regulating the immune response including the production of CYTOKINES. This function is in addition to their traditional role in making antibodies.'),
  'D060152': NtO(MH='V(D)J Recombination', MS='The process by which the V (variable), D (diversity), and J (joining) segments of IMMUNOGLOBULIN GENES or T-CELL RECEPTOR GENES are assembled during the development of LYMPHOID CELLS using NONHOMOLOGOUS DNA END-JOINING.'),
  'D060165': NtO(MH='Uroplakins', MS='Specialized membrane glycoproteins that are found on UROTHELIUM cells. They associate into a hexagonal array of 16-nm cell surface particles which form the asymmetric unit membrane of urothelial plaques.'),
  'D060166': NtO(MH='Capillary Tubing', MS='Hollow cylindrical objects with an internal diameter that is small enough to fill by and hold liquids inside by CAPILLARY ACTION.'),
  'D060167': NtO(MH='Uroplakin Ia', MS='A tetraspanin domain-containing uroplakin subtype. It heterodimerizes with UROPLAKIN II to form a component of the asymmetric unit membrane found in urothelial cells.'),
  'D060168': NtO(MH='Thymocytes', MS='HEMATOPOIETIC PROGENITOR CELLS that have migrated to the THYMUS where they differentiate into T-LYMPHOCYTES. Thymocytes are classified into maturational stages based on the expression of CELL SURFACE ANTIGENS.'),
  'D060169': NtO(MH='Uroplakin Ib', MS='A tetraspanin domain-containing uroplakin subtype. It heterodimerizes with UROPLAKIN III to form a component of the asymmetric unit membrane found in urothelial cells.'),
  'D060170': NtO(MH='Uroplakin II', MS='A uroplakin subtype that heterodimerizes with UROPLAKIN IA to form a component of the asymmetric unit membrane found in urothelial cells.'),
  'D060171': NtO(MH='Uroplakin III', MS='A uroplakin subtype that heterodimerizes with UROPLAKIN IB to form a component of the asymmetric unit membrane found in urothelial cells.'),
  'D060172': NtO(MH='Morpholinos', MS='Synthetic analogs of NUCLEIC ACIDS composed of morpholine ring derivatives (MORPHOLINES) linked by phosphorodimidates. One standard DNA nucleic acid base (ADENINE; GUANINE; CYTOSINE; OR THYMINE) is bound to each morpholine ring.'),
  'D060185': NtO(MH='Tetraspanin 24', MS='A tetraspanin protein that associates with LAMININ-binding INTEGRINS. It is essential for proper assembly of BASEMENT MEMBRANE in the KIDNEY GLOMERULUS and KIDNEY TUBULES and may play a role in the regulation of CELL MOTILITY.'),
  'D060186': NtO(MH='Infusions, Spinal', MS='The administration of medication by insertion of a tiny needle or catheter into the spinal sac or epidural cavity.'),
  'D060205': NtO(MH='Therapeutic Occlusion', MS='Methods used to temporarily or permanently block the flow of BODY FLUIDS through various ducts and tubules throughout the body, including BLOOD VESSELS and LYMPHATIC VESSELS such as by THERAPEUTIC EMBOLIZATION or LIGATION.'),
  'D060225': NtO(MH='Tetraspanin 28', MS='A tetraspanin protein that is involved in a variety of cellular functions including BASEMENT MEMBRANE assembly, and in the formation of a molecular complexes on the surface of LYMPHOCYTES.'),
  'D060226': NtO(MH='Head Kidney', MS='A primitive form of vertebrate kidney that is found in adults of some primitive FISHES and in the embryos of more advanced fishes. In some species of fishes it contains phagocytic cells and is also a major site of HEMATOPOIESIS, analogous to the mammalian BONE MARROW.'),
  'D060227': NtO(MH='Imaginal Discs', MS='Hollow sacs of cells in LARVA that form adult structures in insects during BIOLOGICAL METAMORPHOSIS.'),
  'D060245': NtO(MH='Tetraspanin 29', MS='A subtype of tetraspanin protein that plays a role in cell adhesion, cell motility, and tumor metastasis. It functions in platelet activation and aggregation, the formation of paranodal junctions in neuronal tissue, and the fusion of sperm with egg.'),
  'D060265': NtO(MH='Tetraspanin 25', MS='A tetraspanin protein found at high levels in cells of the lymphoid-myeloid lineage. It may function in regulating the differentiation of T-LYMPHOCYTES and the activation of B-LYMPHOCYTES.'),
  'D060285': NtO(MH='Receptors, Autocrine Motility Factor', MS='Cell surface receptors for AUTOCRINE MOTILITY FACTOR, which is the secreted form of GLUCOSE-6-PHOSPHATE ISOMERASE. The receptor has an unusual composition in that it shares some structural similarities with G-PROTEIN-COUPLED RECEPTORS and functions as an ubiquitin protein ligase when internalized.'),
  'D060305': NtO(MH='Transition to Adult Care', MS='Transfer from pediatric to adult care.'),
  'D060306': NtO(MH='Magnetometry', MS='The measurement of various aspects of MAGNETIC FIELDS.'),
  'D060326': NtO(MH='Chemistry Techniques, Synthetic', MS='Methods used for the chemical synthesis of compounds. Included under this heading are laboratory methods used to synthesize a variety of chemicals and drugs.'),
  'D060327': NtO(MH='Solid-Phase Synthesis Techniques', MS='Techniques used to synthesize chemicals using molecular substrates that are bound to a solid surface. Typically a series of reactions are conducted on the bound substrate that results in either the covalent attachment of specific moieties or the modification of existing function groups. These techniques offer an advantage to those involving solution reactions in that the substrate compound does not have to be isolated and purified between the reaction steps.'),
  'D060328': NtO(MH='Magnetic Phenomena', MS='Characteristics, properties, and effects of magnetic substances and magnetic fields.'),
  'D060329': NtO(MH='Education of Visually Disabled', MS='The teaching or training of those individuals with visual disability.'),
  'D060345': NtO(MH='Education of Hearing Disabled', MS='The teaching or training of those individuals with hearing disability or impairment.'),
  'D060347': NtO(MH='Sodium-Coupled Vitamin C Transporters', MS='Membrane transport proteins that actively co-transport ASCORBIC ACID and sodium ions across the CELL MEMBRANE. Dietary absorption of VITAMIN C is highly dependent upon this class of transporters and a subset of SODIUM GLUCOSE TRANSPORTERS which transport the oxidized form of vitamin C, DEHYDROASCORBIC ACID.'),
  'D060348': NtO(MH='Dirofilaria repens', MS='A filarial parasite primarily affecting dogs and cats, but causing an emerging zoonosis in humans involving subcutaneous lesions. It is transmitted by MOSQUITOES.'),
  'D060365': NtO(MH='Chloroplast Proteins', MS='Proteins encoded by the CHLOROPLAST GENOME or proteins encoded by the nuclear genome that are imported to and resident in the CHOROPLASTS.'),
  'D060366': NtO(MH='Entomoplasmatales', MS='An order of nonhelical (ENTOMOPLASMATACEAE) and helical (SPIROPLASMATACEAE) gram-negative bacteria in the class MOLLICUTES, that are regularly associated with arthropod or plant hosts.'),
  'D060367': NtO(MH='Lacazia', MS='A mitosporic ONYGENALES fungal genus. The species Lacazia loboi is the etiological agent of LOBOMYCOSIS in humans, dolphins, and other wild animals.'),
  'D060368': NtO(MH='Lobomycosis', MS='A chronic, fungal, subcutaneous infection endemic in rural regions in South America and Central America. The causal organism is Lacazia labol.'),
  'D060386': NtO(MH='Sex Workers', MS='People who engage in occupational sexual behavior in exchange for economic rewards or other extrinsic considerations.'),
  'D060387': NtO(MH='Ascorbate Peroxidases', MS='Peroxidases that utilize ASCORBIC ACID as an electron donor to reduce HYDROGEN PEROXIDE to WATER. The reaction results in the production of monodehydroascorbic acid and DEHYDROASCORBIC ACID.'),
  'D060388': NtO(MH='Support Vector Machine', MS='SUPERVISED MACHINE LEARNING algorithm which learns to assign labels to objects from a set of training examples. Examples are learning to recognize fraudulent credit card activity by examining hundreds or thousands of fraudulent and non-fraudulent credit card activity, or learning to make disease diagnosis or prognosis based on automatic classification of microarray gene expression profiles drawn from hundreds or thousands of samples.'),
  'D060389': NtO(MH='Construction Industry', MS='The aggregate business enterprise of building.'),
  'D060405': NtO(MH='Florigen', MS='Molecule produced in plant leaves that acts like a hormone by inducing flowering in the shoot apical meristem of buds and growing tips.'),
  'D060406': NtO(MH='Brassinosteroids', MS='Plant steroids ubiquitously distributed throughout the plant kingdom. They play essential roles in modulating growth and differentiation of cells at nanomolar to micromolar concentrations.'),
  'D060425': NtO(MH='Cerebral Phaeohyphomycosis', MS='CNS infections caused by neurotropic dematiaceous fungi that contain melanin in their cell walls. The infections often result in BRAIN ABSCESS; ENCEPHALITIS; and MENINGITIS in patients who are often immunocompetent. The common causative fungi include members Cladophialophora bantiana, Exophiala dermatitidis, Rhinocladiella mackenziei, and Ochroconis gallopavum. R. mackenziei infection is seen almost exclusively in patients from the MIDDLE EAST.'),
  'D060426': NtO(MH='Anti-N-Methyl-D-Aspartate Receptor Encephalitis', MS='Disorder characterized by symptoms of CATATONIA; HYPOVENTILATION; DYSKINESIAS; ENCEPHALITIS; and SEIZURES followed by a reduced CONSCIOUSNESS. It is often followed by a viral-like prodrome. Many cases are self-limiting and respond well to IMMUNOMODULATORY THERAPIES against the NMDA RECEPTORS antibodies.'),
  'D060431': NtO(MH='Thylakoid Membrane Proteins', MS='Proteins found within the THYLAKOID MEMBRANES of photosynthetic organisms such as PLANTS and PHYTOPLANKTON. Many of the proteins in this class are involved in the process of PHOTOSYNTHESIS and the generation of ADENOSINE TRIPHOSPHATE.'),
  'D060432': NtO(MH='Anthropology, Medical', MS='Field of social science that is concerned with differences between human groups as related to health status and beliefs.'),
  'D060433': NtO(MH='Administration, Ophthalmic', MS='Application of pharmaceutically active agents on the tissues of the EYE.'),
  'D060434': NtO(MH='Herbivory', MS='The act of feeding on plants by animals.'),
  'D060435': NtO(MH='Carnivory', MS='The consumption of animal flesh.'),
  'D060436': NtO(MH='Self-Incompatibility in Flowering Plants', MS='One of many different processes which occur in ANGIOSPERMS by which genetic diversity is maintained while INBREEDING is prevented.'),
  'D060437': NtO(MH='Artificial Lens Implant Migration', MS='The shifting and or tilting of implanted artificial lens resulting in impaired vision.'),
  'D060446': NtO(MH='Phaeohyphomycosis', MS='OPPORTUNISTIC INFECTIONS caused by the dematiaceous (darkly pigmented) MITOSPORIC FUNGI of ALTERNARIA, Bipolaris, CLADOSPORIUM, Curvularia, and EXOPHIALA. These fungi have pigmented HYPHAE due to MELANIN in the cell wall. The initial subcutaneous cyst from the infection can become systemic and spread rapidly to renal, pulmonary and cerebral systems (see CEREBRAL PHAEOHYPHOMYCOSIS) in an IMMUNOCOMPROMISED HOST.'),
  'D060447': NtO(MH='Emergency Shelter', MS='Temporary shelter provided in response to a major disaster or emergency.'),
  'D060449': NtO(MH='Wnt Signaling Pathway', MS='A complex signaling pathway whose name is derived from the DROSOPHILA Wg gene, and the vertebrate INT gene. The signaling pathway is initiated by the binding of WNT PROTEINS to cell surface WNT RECEPTORS which interact with the AXIN SIGNALING COMPLEX and an array of second messengers that influence the actions of BETA CATENIN.'),
  'D060465': NtO(MH='Axin Signaling Complex', MS='A specific complex of WNT SIGNALING PATHWAY proteins that mediates the phosphorylation-dependent destruction of cytosolic BETA-CATENIN. The complex is disrupted by cell surface binding of WNT PROTEINS, which allows beta-catenin levels to rise to the point where they migrate to the CELL NUCLEUS and activate transcription.'),
  'D060466': NtO(MH='Axin Protein', MS='A scaffolding protein that is a critical component of the axin signaling complex which binds to ADENOMATOUS POLYPOSIS COLI PROTEIN; GLYCOGEN SYNTHASE KINASE 3; and CASEIN KINASE I.'),
  'D060467': NtO(MH='Disease Resistance', MS='The capacity of an organism to defend itself against pathological processes or the agents of those processes. This most often involves innate immunity whereby the organism responds to pathogens in a generic way. The term disease resistance is used most frequently when referring to plants.'),
  'D060485': NtO(MH='Pedobacter', MS='A genus of gram-negative, aerobic, rod-shaped bacteria in the family Sphingobacteriaceae. They exhibit gliding motility.'),
  'D060486': NtO(MH='Ophthalmoplegic Migraine', MS='Childhood-onset of recurrent headaches with an oculomotor cranial nerve palsy. Typically, ABDUCENS NERVE; OCULOMOTOR NERVE; and TROCHLEAR NERVE are involved with DIPLOPIA and BLEPHAROPTOSIS.'),
  'D060487': NtO(MH='Alternariosis', MS='Opportunistic fungal infection by a member of ALTERNARIA genus.'),
  'D060488': NtO(MH='Low Density Lipoprotein Receptor-Related Protein-6', MS='An LDL-receptor related protein that combines with cell surface FRIZZLED RECEPTORS to form WNT PROTEIN-binding receptors. The protein plays an important role in the WNT SIGNALING PATHWAY during EMBRYONIC DEVELOPMENT and in regulation of vascular cell proliferation.'),
  'D060489': NtO(MH='Prosthesis Retention', MS='Holding a PROSTHESIS in place.'),
  'D060490': NtO(MH='Receptors, Wnt', MS='Cell-surface receptors that specifically bind to WNT PROTEINS and activate the WNT SIGNALING PATHWAY.'),
  'D060492': NtO(MH='Low Density Lipoprotein Receptor-Related Protein-5', MS='LDL-receptor related protein that combines with FRIZZLED RECEPTORS at the cell surface to form receptors that bind WNT PROTEINS. The protein plays an important role in the WNT SIGNALING PATHWAY in OSTEOBLASTS and during EMBRYONIC DEVELOPMENT.'),
  'D060493': NtO(MH='Tenacibaculum', MS='A genus of gram-negative, rod-shaped bacteria in the family FLAVOBACTERIACEAE. Tenacibaculum adheres to surfaces of marine organisms and is pathogenic to fish.'),
  'D060494': NtO(MH='Riemerella', MS='A genus of gram-negative, rod-shaped bacteria in the family FLAVOBACTERIACEAE that causes exudative SEPTICEMIA in ducks, pigeons, and other domestic and wild birds.'),
  'D060508': NtO(MH='Wnt3 Protein', MS='A Wnt protein subtype that plays a role in cell-cell signaling during EMBRYONIC DEVELOPMENT and the morphogenesis of the developing NEURAL TUBE. Defects in Wnt3 protein are associated with autosomal recessive tetra-AMELIA in humans.'),
  'D060509': NtO(MH='Wnt3A Protein', MS='A Wnt protein subtype that plays a role in cell-cell signaling during EMBRYONIC DEVELOPMENT and the morphogenesis of the developing NEURAL TUBE.'),
  'D060525': NtO(MH='Pudendal Nerve', MS='A nerve which originates in the sacral spinal cord (S2 to S4) and innervates the PERINEUM, the external GENITALIA, the external ANAL SPHINCTER and the external urethral sphincter. It has three major branches: the perineal nerve, inferior anal nerves, and the dorsal nerve of penis or clitoris.'),
  'D060526': NtO(MH='Magnetic Fields', MS='Areas of attractive or repulsive force surrounding MAGNETS.'),
  'D060527': NtO(MH='Corneal Keratocytes', MS='Fibroblasts which occur in the CORNEAL STROMA.'),
  'D060528': NtO(MH='Wnt4 Protein', MS='A Wnt protein that is involved in regulating multiple developmental processes such as the formation of the KIDNEY; ADRENAL GLANDS; MAMMARY GLANDS; the PITUITARY GLAND; and the female reproductive system. Defects in WNT4 are a cause of ROKITANSKY KUSTER HAUSER SYNDROME.'),
  'D060545': NtO(MH='Pudendal Neuralgia', MS='Pain associated with a damaged PUDENDAL NERVE. Clinical features may include positional pain with sitting in the perineal and genital areas, sexual dysfunction and FECAL INCONTINENCE and URINARY INCONTINENCE.'),
  'D060565': NtO(MH='Argonaute Proteins', MS='A family of RNA-binding proteins that has specificity for MICRORNAS and SMALL INTERFERING RNA molecules. The proteins take part in RNA processing events as core components of RNA-induced silencing complex.'),
  'D060585': NtO(MH='Fusariosis', MS='OPPORTUNISTIC INFECTIONS with the soil fungus FUSARIUM. Typically the infection is limited to the nail plate (ONYCHOMYCOSIS). The infection can however become systemic especially in an IMMUNOCOMPROMISED HOST (e.g., NEUTROPENIA) and results in cutaneous and subcutaneous lesions, fever, KERATITIS, and pulmonary infections.'),
  'D060586': NtO(MH='Trichosporonosis', MS='Fungal infections caused by TRICHOSPORON that may become systemic especially in an IMMUNOCOMPROMISED HOST. Clinical manifestations range from superficial cutaneous infections to systemic lesions in multiple organs.'),
  'D060587': NtO(MH='Groundwater', MS='Liquid water present beneath the surface of the earth.'),
  'D060588': NtO(MH='LIM Domain Proteins', MS='A large class of structurally-related proteins that contain one or more LIM zinc finger domains. Many of the proteins in this class are involved in intracellular signaling processes and mediate their effects via LIM domain protein-protein interactions. The name LIM is derived from the first three proteins in which the motif was found: LIN-11, Isl1 and Mec-3.'),
  'D060589': NtO(MH='Zyxin', MS='A zinc-binding phosphoprotein that concentrates at focal adhesions and along the actin cytoskeleton. Zyxin has an N-terminal proline-rich domain and three LIM domains in its C-terminal half.'),
  'D060605': NtO(MH='Hyalohyphomycosis', MS='OPPORTUNISTIC INFECTIONS caused by a heterogeneous group of MITOSPORIC FUNGI with clear (hyalo-) HYPHAE in the host. Common causative agents include ACREMONIUM; ASPERGILLUS; CHRYSOSPORIUM; FUSARIUM; PAECILOMYCES; PENICILLIUM; PSEUDALLESCHERIA; SCEDOSPORIUM; and SCOPULARIOPSIS. Normally a dermatomycoses, it can become invasive in the IMMUNOCOMPROMISED HOST.'),
  'D060645': NtO(MH='Scopulariopsis', MS='A mitosporic fungal genus commonly found in soil. It includes both saprophytes and animal pathogens. Scopulariopsis brevicaulis and related species may convert ARSENIC to volatile and toxic arsine. ONYCHOMYCOSIS and rarely BRAIN ABSCESS; MYCETOMA, and disseminated infections mainly in the IMMUNOCOMPROMISED HOSTS are associated with Scopulariopsis.'),
  'D060666': NtO(MH='Airway Extubation', MS='Removal of an endotracheal tube from the patient.'),
  'D060685': NtO(MH='Sense of Coherence', MS="A view of the world and the individual's environment as comprehensible, manageable, and meaningful, claiming that the way people view their life has a positive influence on their health."),
  'D060705': NtO(MH='Dyscalculia', MS='Impaired ability in numerical concepts. These inabilities arise as a result of primary neurological lesion, are syndromic (e.g., GERSTMANN SYNDROME ) or acquired due to brain damage.'),
  'D060725': NtO(MH='Uterine Retroversion', MS='A condition in which the UTERUS is found tilted backward toward the spine. The uterus is more commonly found in a straight vertical or anteverted (tipped forward) position. Although retroverted uterus is a normal variant position without symptoms, it is sometimes associated with pain, discomfort and other pregnancy complications.'),
  'D060726': NtO(MH='Torso', MS='The central part of the body to which the neck and limbs are attached.'),
  'D060728': NtO(MH='Reproductive Health', MS='The state of the REPRODUCTIVE SYSTEM when functioning optimally without evidence of disease, disorders, or deficiencies.'),
  'D060729': NtO(MH='Coal Ash', MS='Residue generated from combustion of coal or petroleum.'),
  'D060730': NtO(MH='Glomerular Filtration Barrier', MS='A specialized barrier in the kidney, consisting of the fenestrated CAPILLARY ENDOTHELIUM; GLOMERULAR BASEMENT MEMBRANE; and glomerular epithelium (PODOCYTES). The barrier prevents the filtration of PLASMA PROTEINS.'),
  'D060731': NtO(MH='Emergency Responders', MS='Personnel trained to provide the initial services, care, and support in EMERGENCIES or DISASTERS.'),
  'D060733': NtO(MH='Electromagnetic Radiation', MS='Waves of oscillating electric and MAGNETIC FIELDS which move at right angles to each other and outward from the source.'),
  'D060734': NtO(MH='Nacre', MS='A biomineral consisting of layers of crystalized ARAGONITE, separated by interlamellar layers of organic matrix containing beta-CHITIN, and surrounded by an intercrystalline organic membrane of silk fibroin. Nacre is the iridescent substance better known as mother-of-pearl that is secreted by MOLLUSCS.'),
  'D060735': NtO(MH='Pharmacovigilance', MS='The detection of long and short term side effects of conventional and traditional medicines through research, data mining, monitoring, and evaluation of healthcare information obtained from healthcare providers and patients.'),
  'D060737': NtO(MH='Reproductive Tract Infections', MS='Infections of the genital tract in females or males. They can be caused by endogenous, iatrogenic, or sexually transmitted organisms.'),
  'D060739': NtO(MH='Phonons', MS='Quanta of acoustic energy which move at the speed of sound.'),
  'D060740': NtO(MH='Disease Eradication', MS='Termination of all transmission of infection by global extermination of the infectious agent through surveillance and containment (From Porta, A Dictionary of Epidemiology, 5th ed).'),
  'D060745': NtO(MH='Gingival Retraction Techniques', MS='Techniques used to expose dental surface below the gingival margin in order to obtain better dental impression during periodental and peri-implant applications. The retraction of the gingival tissue can be achieved surgically (e.g., laser gingivectomy and rotary curettage) or chemically with a retraction cord.'),
  'D060746': NtO(MH='Endoplasmic Reticulum-Associated Degradation', MS='A degradation process whereby incorrectly folded proteins are selectively transported out of the ENDOPLASMIC RETICULUM and into the CYTOSOL. The misfolded proteins are subsequently ubiquitinated and degraded by the PROTEASOME.'),
  'D060747': NtO(MH='Fetal Globulins', MS='Specialized proteins that are preferentially expressed during FETAL DEVELOPMENT.'),
  'D060748': NtO(MH='Fetuins', MS='A family of calcium-binding alpha-globulins that are synthesized in the LIVER and play an essential role in maintaining the solubility of CALCIUM in the BLOOD. In addition the fetuins contain aminoterminal cystatin domains and are classified as type 3 cystatins.'),
  'D060749': NtO(MH='alpha-2-HS-Glycoprotein', MS='A fetuin subtype that is synthesized by HEPATOCYTES and secreted into the circulation. It plays a major role in preventing CALCIUM precipitation in the BLOOD.'),
  'D060750': NtO(MH='Bone Anteversion', MS='Malalignment of a bone in which its head and neck is rotated excessively forward or inward.'),
  'D060751': NtO(MH='Bone Retroversion', MS='Attachment of a bone in which its head and neck is rotated excessively backward.'),
  'D060752': NtO(MH='Fetuin-B', MS='A fetuin subtype that is closely-related to ALPHA-2-HS-GLYCOPROTEIN. Although fetuin-B binds calcium, it is a minor component of SERUM and therefore it may not play a primary role in preventing serum calcium phosphate precipitation.'),
  'D060753': NtO(MH='Water Quality', MS='A rating of a body of water based on measurable physical, chemical, and biological characteristics.'),
  'D060754': NtO(MH='Equol', MS='A non-steroidal ESTROGEN generated when soybean products are metabolized by certain bacteria in the intestines.'),
  'D060755': NtO(MH='Germ Theory of Disease', MS='The fundamental tenet of modern medicine that certain diseases are caused by microorganisms. It was confirmed by the work of Pasteur, Lister, and Koch.'),
  'D060756': NtO(MH='Social Networking', MS='Individuals connecting by family, work or other interests. It also includes connectivity facilitated by computer-based communications.'),
  'D060765': NtO(MH='Secretoglobins', MS='A structurally-related family of small, multimeric proteins that are secreted in the mucosa of mammalian epithelial tissues. A variety of proteins are classed under this heading including some secretoglobin subtypes that appear unique to a particular mammalian species, and others whose functions differ between species.'),
  'D060766': NtO(MH='Drinking Water', MS='Water that is intended to be ingested.'),
  'D060785': NtO(MH='Mammaglobin A', MS='A secretoglobin that is produced by the MAMMARY GLAND of HUMANS and may be involved in the binding of ANDROGENS and other STEROIDS. The expression of this protein in normal breast epithelium and in human breast cancer has made it an important histological marker.'),
  'D060786': NtO(MH='Holy Roman Empire', MS='Realm in central Europe consisting of a confederation of German and Italian territories under the suzerainty of an emperor and existing from the 9th or 10th century to 1806.'),
  'D060787': NtO(MH='Neoplasm Grading', MS='Methods which attempt to express in replicable terms the level of CELL DIFFERENTIATION in neoplasms as increasing ANAPLASIA correlates with the aggressiveness of the neoplasm.'),
  'D060788': NtO(MH='Mammaglobin B', MS='A MAMMAGLOBIN A-related secretoglobin that is expressed in several HUMAN tissues including the UTERUS; BREAST; SALIVARY GLAND; and LACRIMAL GLAND.'),
  'D060789': NtO(MH='Prostatein', MS='A secreted prostate-specific protein which can bind non-polar steroids, cholesterol and a group of small, proline-rich peptides. The protein is specifically found in RATS and comprises three distinct secretoglobin-related subunits referred to as prostatic steroid-binding protein C1, C2 and C3.'),
  'D060805': NtO(MH='Performance Anxiety', MS="Anxiety related to the execution of a task. (Campbell's Psychiatric Dictionary, 9th ed.)"),
  'D060807': NtO(MH='Bioluminescence Resonance Energy Transfer Techniques', MS='Techniques for determining the proximity of molecules based on ENERGY TRANSFER between bioluminescent chromophores and acceptor fluorophores that have overlapping emission and absorption spectra.'),
  'D060825': NtO(MH='Cognitive Dysfunction', MS='Diminished or impaired mental and/or intellectual function.'),
  'D060826': NtO(MH='Cementoplasty', MS='Injection of BONE CEMENTS into bone to treat bone lesions.'),
  'D060828': NtO(MH='Induction Chemotherapy', MS='Initial drug treatment designed to bring about REMISSION INDUCTION. It is typically a short-term and high-dose drug treatment that is followed by CONSOLIDATION CHEMOTHERAPY and then MAINTENANCE CHEMOTHERAPY.'),
  'D060829': NtO(MH='Arthropod Proteins', MS='Proteins synthesized by organisms belonging to the phylum ARTHROPODA. Included in this heading are proteins from the subdivisions ARACHNIDA; CRUSTACEA; and HORSESHOE CRABS. Note that a separate heading for INSECT PROTEINS is listed under this heading.'),
  'D060830': NtO(MH='Consolidation Chemotherapy', MS='Drug treatment designed to further diminish the disease toward complete remission following INDUCTION CHEMOTHERAPY. It helps to consolidate the gains during induction chemotherapy and may be followed by MAINTENANCE CHEMOTHERAPY.'),
  'D060831': NtO(MH='Hand-Foot Syndrome', MS='Chemotherapy-induced dermal side effects that are associated with the use of various CYTOSTATIC AGENTS. Symptoms range from mild ERYTHEMA and/or PARESTHESIA to severe ulcerative dermatitis with debilitating pain involving typically palmoplantar and intertriginous areas. These cutaneous manifestations are sometimes accompanied by nail anomalies.'),
  'D060832': NtO(MH='Apomixis', MS='Asexual reproduction resulting in the formation of viable seeds from FLOWERS without fertlization (i.e. use of POLLEN). Progeny plants produced from apomictic seeds are perfect clones of the parent.'),
  'D060833': NtO(MH='Cellular Microenvironment', MS='Local surroundings with which cells interact by processing various chemical and physical signals, and by contributing their own effects to this environment.'),
  'D060845': NtO(MH='Hoarding', MS='A persistent difficulty discarding or parting with possessions.'),
  'D060846': NtO(MH='Cell Engineering', MS='Methods and techniques used to modify or select cells and develop conditions for growing cells for biosynthetic production of molecules (METABOLIC ENGINEERING), for generation of tissue structures and organs in vitro (TISSUE ENGINEERING), or for other BIOENGINEERING research objectives.'),
  'D060847': NtO(MH='Metabolic Engineering', MS="Methods and techniques used to genetically modify cells' biosynthetic product output and develop conditions for growing the cells as BIOREACTORS."),
  'D060848': NtO(MH='Agatoxins', MS='A class of polyamine and peptide toxins which are isolated from the venom of spiders such as Agelenopsis aperta.'),
  'D060850': NtO(MH='LIM-Homeodomain Proteins', MS='A subclass of LIM domain proteins that include an additional centrally-located homeodomain region that binds AT-rich sites on DNA. Many LIM-homeodomain proteins play a role as transcriptional regulators that direct cell fate.'),
  'D060851': NtO(MH='Gulf of Mexico', MS='A body of water located at the southeastern corner of North America. It is bordered by the states to the north of Florida, Alabama, Mississippi, Louisiana, Texas; by five Mexican states to the west: Tamaulipas, Veracruz, Tabasco, Campeche, Yucatan; and by Cuba to the southeast.'),
  'D060865': NtO(MH='Repetition Priming', MS='A type of procedural memory manifested as a change in the ability to identify an item as a result of a previous encounter with the item or stimuli.'),
  'D060866': NtO(MH='Accountable Care Organizations', MS='Organizations of health care providers that agree to be accountable for the quality, cost, and overall care of Medicare beneficiaries who are enrolled in the traditional fee-for-service program who are assigned to it. Assigned means those beneficiaries for whom the professionals in the organization provide the bulk of primary care services. (www.cms.gov/OfficeofLegislation/Downloads/Accountable CareOrganization.pdf accessed 03/16/2011)'),
  'D060867': NtO(MH='Facilitated Tucking', MS="A method of soothing PREMATURE INFANTS during POSTNATAL CARE. It involves holding the infant's arms and legs in flexed positions close to the midline of the TORSO."),
  'D060885': NtO(MH='Multiplex Polymerase Chain Reaction', MS='Methods for using more than one primer set in a polymerase chain reaction to amplify more than one segment of the target DNA sequence in a single reaction.'),
  'D060887': NtO(MH='B7 Antigens', MS='A family of cell-surface proteins found on ANTIGEN-PRESENTING CELLS. B7 antigens are ligands for specific cell surface receptor subtypes found on T-CELLS. They play an immunomodulatory role by stimulating or inhibiting the T-CELL activation process.'),
  'D060888': NtO(MH='Real-Time Polymerase Chain Reaction', MS='Methods used for detecting the amplified DNA products from the polymerase chain reaction as they accumulate instead of at the end of the reaction.'),
  'D060889': NtO(MH='Inducible T-Cell Co-Stimulator Protein', MS='A costimulatory receptor that is specific for INDUCIBLE T-CELL CO-STIMULATOR LIGAND. The receptor is associated with a diverse array of immunologically-related effects including the increased synthesis of INTERLEUKIN 10 in REGULATORY T-LYMPHOCYTES and the induction of PERIPHERAL TOLERANCE.'),
  'D060890': NtO(MH='B7-H1 Antigen', MS='An inhibitory B7 antigen that contains V-type and C2 type immunoglobulin domains. It has specificity for the T-CELL receptor PROGRAMMED CELL DEATH 1 PROTEIN and provides negative signals that control and inhibit T-cell responses. It is found at higher than normal levels on tumor cells, suggesting its potential role in TUMOR IMMUNE EVASION.'),
  'D060891': NtO(MH='Root Cause Analysis', MS='Multi-step systematic review process used for improving safety by investigation of incidents to find what happened, why it happened, and to determine what can be done to prevent it from happening again.'),
  'D060905': NtO(MH='Coxa Vara', MS='Hip deformity in which the femoral neck leans forward resulting in a decrease in the angle between femoral neck and its shaft. It may be congenital often syndromic, acquired, or developmental.'),
  'D060906': NtO(MH='Coxa Valga', MS='Hip deformity in which the angle between the femoral neck and its shaft is increased. It may be congenital, acquired, or developmental.'),
  'D060907': NtO(MH='V-Set Domain-Containing T-Cell Activation Inhibitor 1', MS='A B7 antigen subtype that inhibits the costimulation of T-cell activation, proliferation, cytokine production and development of cytotoxicity. The over expression of this protein in a variety of tumor cell types suggests its role in TUMOR IMMUNE EVASION.'),
  'D060908': NtO(MH='CTLA-4 Antigen', MS='An inhibitory T CELL receptor that is closely related to CD28 ANTIGEN. It has specificity for CD80 ANTIGEN and CD86 ANTIGEN and acts as a negative regulator of peripheral T cell function. CTLA-4 antigen is believed to play role in inducing PERIPHERAL TOLERANCE.'),
  'D060909': NtO(MH='Economics, Behavioral', MS='The combined discipline of psychology and economics that investigates what happens in markets in which some of the agents display human limitations and complications.'),
  'D060910': NtO(MH='Pronephros', MS='The primordial kidney that develops from the INTERMEDIATE MESODERM in the embryos of vertebrates, and is succeeded by the MESONEPHROS. In higher vertebrates and humans, the pronephros is a vestigial and transient structure.'),
  'D060926': NtO(MH='Remuneration', MS='Payment for a service or for a commodity such as a body part.'),
  'D060945': NtO(MH='Inducible T-Cell Co-Stimulator Ligand', MS='A B7 antigen that binds specifically to INDUCIBLE T-CELL CO-STIMULATOR PROTEIN on T-CELLS. It provides a costimulatory signal for T-cell proliferation and cytokine secretion.'),
  'D060946': NtO(MH='Pharmaceutical Services, Online', MS='Pharmacy services accessed via electronic means.'),
  'D060965': NtO(MH='Clonal Evolution', MS='The process of accumulation of genetic and epigenetic changes over time in individual cells and the effect of the changes on CELL PROLIFERATION.'),
  'D060966': NtO(MH='Clonal Selection, Antigen-Mediated', MS='LYMPHOCYTE ACTIVATION by a specific ANTIGEN thus triggering clonal expansion of LYMPHOCYTES already capable of mounting an immune response to the antigen.'),
  'D060985': NtO(MH='Natural Springs', MS='Water that emerges to the surface of the earth from underground.'),
  'D060986': NtO(MH='Water Wells', MS='Constructions built to access underground water.'),
  'D061005': NtO(MH='Ponds', MS='Inland bodies of standing FRESHWATER usually smaller than LAKES. They can be man-made or natural but there is no universal agreement as to their exact size. Some consider a pond to be a small body of water that is shallow enough for sunlight to reach the bottom.'),
  'D061006': NtO(MH='Bays', MS='An area of water mostly surrounded by land, usually smaller than a gulf, and affording access to the sea.'),
  'D061025': NtO(MH='Costimulatory and Inhibitory T-Cell Receptors', MS='A family of receptors that modulate the activation of T-LYMPHOCYTES by the T-CELL ANTIGEN RECEPTOR. The receptors are responsive to one or more B7 ANTIGENS found on ANTIGEN-PRESENTING CELLS and, depending upon the specific ligand-receptor combination, modulate a variety of T-cell functions such as the rate of clonal expansion, CELL SURVIVAL and cytokine production. Although commonly referred to as costimulatory receptors, some of the receptors have inhibitory effects such as inducing PERIPHERAL TOLERANCE.'),
  'D061026': NtO(MH='Programmed Cell Death 1 Receptor', MS='An inhibitory T-lymphocyte receptor that has specificity for CD274 ANTIGEN and PROGRAMMED CELL DEATH 1 LIGAND 2 PROTEIN. Signaling by the receptor limits T cell proliferation and INTERFERON GAMMA synthesis. The receptor also may play an essential role in the regulatory pathway that induces PERIPHERAL TOLERANCE.'),
  'D061027': NtO(MH='Programmed Cell Death 1 Ligand 2 Protein', MS='A costimulatory B7 antigen that has specificity for the T-CELL receptor PROGRAMMED CELL DEATH 1 RECEPTOR. It is closely-related to CD274 antigen; however, its expression is restricted to DENDRITIC CELLS and activated MACROPHAGES.'),
  'D061028': NtO(MH='Trigger Points', MS='Discrete spots in taut bands of muscle that produce local and referred pain when muscle bands are compressed.'),
  'D061046': NtO(MH='Biological Control Agents', MS='Organisms, biological agents, or biologically-derived agents used strategically for their positive or adverse effect on the physiology and/or reproductive health of other organisms.'),
  'D061065': NtO(MH='Polyketides', MS='Natural compounds containing alternating carbonyl and methylene groups (beta-polyketones), bioenergenetically derived from repeated condensation of acetyl coenzyme A via malonyl coenzyme A, in a process similar to fatty acid synthesis.'),
  'D061066': NtO(MH='Plant Pathology', MS='The study of infectious diseases associated with plants.'),
  'D061067': NtO(MH='Antibodies, Monoclonal, Humanized', MS='Antibodies from non-human species whose protein sequences have been modified to make them nearly identical with human antibodies. If the constant region and part of the variable region are replaced, they are called humanized. If only the constant region is modified they are called chimeric. INN names for humanized antibodies end in -zumab.'),
  'D061085': NtO(MH='Agenesis of Corpus Callosum', MS="Birth defect that results in a partial or complete absence of the CORPUS CALLOSUM. It may be isolated or a part of a syndrome (e.g., AICARDI'S SYNDROME; ACROCALLOSAL SYNDROME; ANDERMANN SYNDROME; and HOLOPROSENCEPHALY). Clinical manifestations include neuromotor skill impairment and INTELLECTUAL DISABILITY of variable severity."),
  'D061086': NtO(MH='NADP Transhydrogenase, B-Specific', MS='A NADP transhydrogenase subtype found in certain types of BACTERIA. This transhydrogenase uses a mechanism that transfers hydrogen to the B side of the NAD(+) or NADP(+) ring structure.'),
  'D061087': NtO(MH='NADP Transhydrogenase, AB-Specific', MS='A NADP transhydrogenase subtype found in certain types of BACTERIA and mammalian MITOCHONDRIA. This transhydrogenase uses a mechanism that can transfer hydrogen to either side, A or B, of the NAD(+) or NADP(+) ring structure. In heart mitochondria, the enzyme is A-specific with respect to NAD(+) and B-specific with respect to NADP(+).'),
  'D061088': NtO(MH='Photoacoustic Techniques', MS='Investigative and diagnostic methods and procedures based on the photoacoustic effect, which is the generation of SOUND WAVES from the absorption of ELECTROMAGNETIC RADIATION.'),
  'D061089': NtO(MH='Radiotherapy, Image-Guided', MS='The use of pre-treatment imaging modalities to position the patient, delineate the target, and align the beam of radiation to achieve optimal accuracy and reduce radiation damage to surrounding non-target tissues.'),
  'D061105': NtO(MH='Peroxiredoxin III', MS='A THIOREDOXIN-dependent hydroperoxidase that is localized in the mitochondrial matrix. The enzyme plays a crucial role in protecting mitochondrial components from elevated levels of HYDROGEN PEROXIDE.'),
  'D061106': NtO(MH='Radiotherapy Setup Errors', MS='Mistakes committed in the preparations for radiotherapy, including errors in positioning of patients, alignment radiation beams, or calculation of radiation doses.'),
  'D061107': NtO(MH='Nucleoside Diphosphate Kinase D', MS='A nucleoside diphosphate kinase subtype that is localized to the intermembrane space of MITOCHONDRIA. It is believed to play a role in the synthesis of triphosphonucleotides using ATP formed through OXIDATIVE PHOSPHORYLATION.'),
  'D061108': NtO(MH='Social Media', MS='Platforms that provide the ability and tools to create and publish information accessed via the INTERNET. Generally these platforms have three characteristics with content user generated, high degree of interaction between creator and viewer, and easily integrated with other sites.'),
  'D061125': NtO(MH='Genes, Chloroplast', MS='Those nucleic acid sequences that function as units of heredity which are located within the CHLOROPLAST DNA.'),
  'D061145': NtO(MH='Microvascular Decompression Surgery', MS='Surgery performed to relieve pressure from MICROVESSELS that are located around nerves and are causing NERVE COMPRESSION SYNDROMES.'),
  'D061146': NtO(MH='Chlorophyll Binding Proteins', MS='A large family of proteins that have been traditionally classified as the light-harvesting proteins of the photosynthetic reaction complex. Chlorophyll binding proteins are also found in non-photosynthetic settings where they may play a photoprotective role in response to light stress.'),
  'D061147': NtO(MH='Therapy with Helminths', MS='The treatment of immune system diseases by deliberate infestation with helminths. This therapy is partly based on the HYGIENE HYPOTHESIS which states that the absence of parasites increases immune dysregulation because of the lack of stimulation of REGULATORY T-CELLS.'),
  'D061148': NtO(MH='Hygiene Hypothesis', MS='The theory that infectious agents, symbiotic microorganisms, and parasites are normal stimulants for the maturation of the immune system toward a balanced immune response. The theory predicts that lack of such stimulation leads to allergies and AUTOIMMUNE DISEASES.'),
  'D061149': NtO(MH='Thysanoptera', MS='An order of very small, fringed-wing INSECTS including many agricultural pests.'),
  'D061165': NtO(MH='Glenoid Cavity', MS='A depression in the lateral angle of the scapula that articulates with the HUMERAL HEAD.'),
  'D061166': NtO(MH='Peripheral Tolerance', MS='The mechanism, in peripheral lymphoid organs (LYMPH NODES; SPLEEN; TONSILS; and mucosal-associated lymphoid tissue), that prevents mature lymphocytes from reacting to SELF-ANTIGENS. This is accomplished through a variety of means including CLONAL ANERGY and CLONAL DELETION.'),
  'D061167': NtO(MH='Central Tolerance', MS='The mechanism, in central lymphoid organs (THYMUS; BONE MARROW), that prevents immature lymphocytes from reacting to SELF-ANTIGENS. This is accomplished by CLONAL ANERGY and CLONAL DELETION.'),
  'D061185': NtO(MH='Mechanical Thrombolysis', MS='Procedures to cause the disintegration of THROMBI by physical interventions.'),
  'D061186': NtO(MH='Breast Milk Expression', MS='The act of evacuating BREAST MILK by hand or with a pump.'),
  'D061205': NtO(MH='Vascular Calcification', MS='Deposition of calcium into the blood vessel structures. Excessive calcification of the vessels are associated with ATHEROSCLEROTIC PLAQUES formation particularly after MYOCARDIAL INFARCTION (see MONCKEBERG MEDIAL CALCIFIC SCLEROSIS) and chronic kidney diseases which in turn increase VASCULAR STIFFNESS.'),
  'D061206': NtO(MH='Neoplasm Micrometastasis', MS='Newly arising secondary tumors so small they are difficult to detect by physical examination or routine imaging techniques.'),
  'D061207': NtO(MH='Calcium Ionophores', MS='Chemical agents that increase the permeability of CELL MEMBRANES to CALCIUM ions.'),
  'D061208': NtO(MH='Pulsed Radiofrequency Treatment', MS='The application, often via IMPLANTED ELECTRODES, of short bursts of electrical energy in the radiofrequency range, interspersed with pauses in delivery of the current long enough to dissipate the generated heat and avoid heat-induced tissue necrosis.'),
  'D061209': NtO(MH='Proton Ionophores', MS='Chemical agents that increase the permeability of CELL MEMBRANES to PROTONS.'),
  'D061210': NtO(MH='Sodium Ionophores', MS='Chemical agents that increase the permeability of CELL MEMBRANES to SODIUM ions.'),
  'D061212': NtO(MH='Memory, Episodic', MS='Type of declarative memory, consisting of personal memory in contrast to general knowledge.'),
  'D061213': NtO(MH='Mortality, Premature', MS='Deaths that occur before LIFE EXPECTANCY is reached within a given population.'),
  'D061214': NtO(MH='Patient Safety', MS='Efforts to reduce risk, to address and reduce incidents and accidents that may negatively impact healthcare consumers.'),
  'D061215': NtO(MH='Energy Drinks', MS='Beverages that contain stimulants such as a combination of CAFFEINE with other substances such as VITAMINS; AMINO ACIDS; and sugar or sugar derivatives.'),
  'D061216': NtO(MH='Potassium Ionophores', MS='Chemical agents that increase the permeability of CELL MEMBRANES to POTASSIUM ions.'),
  'D061217': NtO(MH='Weight Reduction Programs', MS='Services providing counseling and activities that help overweight individuals to attain a more healthy body weight.'),
  'D061218': NtO(MH='Depressive Disorder, Treatment-Resistant', MS="Failure to respond to two or more trials of antidepressant monotherapy or failure to respond to four or more trials of different antidepressant therapies. (Campbell's Psychiatric Dictionary, 9th ed.)"),
  'D061219': NtO(MH='Olfactory Nerve Injuries', MS='Traumatic injuries to the OLFACTORY NERVE. It may result in various olfactory dysfunction including a complete loss of smell.'),
  'D061220': NtO(MH='Oculomotor Nerve Injuries', MS='Traumatic injuries to the OCULOMOTOR NERVE. This may result in various eye movement dysfunction.'),
  'D061221': NtO(MH='Trigeminal Nerve Injuries', MS='Traumatic injuries to the TRIGEMINAL NERVE. It may result in extreme pain, abnormal sensation in the areas the nerve innervates on face, jaw, gums and tongue and can cause difficulties with speech and chewing. It is sometimes associated with various dental treatments.'),
  'D061222': NtO(MH='Lingual Nerve Injuries', MS='Traumatic injuries to the LINGUAL NERVE. It may be a complication following dental treatments.'),
  'D061223': NtO(MH='Vagus Nerve Injuries', MS='Traumatic injuries to the VAGUS NERVE. Because the vagus nerve innervates multiple organs, injuries in the nerve fibers may result in any gastrointestinal organ dysfunction downstream of the injury site.'),
  'D061224': NtO(MH='Laryngeal Nerve Injuries', MS='Traumatic injuries to the LARYNGEAL NERVE.'),
  'D061226': NtO(MH='Recurrent Laryngeal Nerve Injuries', MS='Traumatic injuries to the RECURRENT LARYNGEAL NERVE that may result in vocal cord dysfunction.'),
  'D061227': NtO(MH='Accessory Nerve Injuries', MS='Traumatic injuries to the ACCESSORY NERVE. Damage to the nerve may produce weakness in head rotation and shoulder elevation.'),
  'D061228': NtO(MH='Hypoglossal Nerve Injuries', MS='Traumatic injuries to the HYPOGLOSSAL NERVE.'),
  'D061245': NtO(MH='Axenic Culture', MS='Culture of an isolated organism free from any other associating or contaminating organisms.'),
  'D061247': NtO(MH='Trochlear Nerve Injuries', MS='Traumatic injuries to the TROCHLEAR NERVE.'),
  'D061249': NtO(MH='Batch Cell Culture Techniques', MS='Methods for cultivation of cells, usually on a large-scale, in a closed system for the purpose of producing cells or cellular products to harvest.'),
  'D061250': NtO(MH='Transferrins', MS='A group of iron-binding proteins that tightly bind two ferrate ions along with two carbonate ions. They are found in the bodily fluids of vertebrates where they act as transport and storage molecules for iron.'),
  'D061251': NtO(MH='Primary Cell Culture', MS='The initial culturing of cells derived directly from fresh TISSUES.'),
  'D061252': NtO(MH='Feeder Cells', MS='Cells used in COCULTURE TECHNIQUES which support the growth of the other cells in the culture. Feeder cells provide auxillary substances including attachment substrates, nutrients, or other factors that are needed for growth in culture.'),
  'D061265': NtO(MH='Biphasic Insulins', MS='An insulin preparation that is designed to provide immediate and long term glycemic control in a single dosage. Biphasic insulin typically contains a mixture of REGULAR INSULIN or SHORT-ACTING INSULIN combined with a LONG-ACTING INSULIN.'),
  'D061266': NtO(MH='Insulin, Short-Acting', MS='Insulin derivatives and preparations that are designed to induce a rapid HYPOGLYCEMIC EFFECT.'),
  'D061267': NtO(MH='Insulin Aspart', MS='Insulin that has been modified to contain an ASPARTIC ACID instead of a PROLINE at position 38 of the B-chain.'),
  'D061268': NtO(MH='Insulin Lispro', MS='Insulin that has been modified so that the B-chain contains a LYSINE at position 28 instead of a PROLINE and a PROLINE at position 29 instead of a LYSINE. It is used to manage BLOOD GLUCOSE levels in patients with TYPE 2 DIABETES.'),
  'D061269': NtO(MH='DNA Transformation Competence', MS='The ability of bacterial cells to take up exogenous DNA and be genetically transformed by it.'),
  'D061270': NtO(MH='Nasal Septal Perforation', MS='An opening or hole in the NASAL SEPTUM that is caused by TRAUMA, injury, drug use, or pathological process.'),
  'D061271': NtO(MH='Acidobacteria', MS='A physiologically diverse phylum of acidophilic, gram-negative bacteria found in a wide variety of habitats, but particularly abundant in soils and sediments.'),
  'D061285': NtO(MH='Vestibulocochlear Nerve Injuries', MS='Traumatic injuries to the VESTIBULOCOCHLEAR NERVE.'),
  'D061286': NtO(MH='Osmometry', MS='Measurement of the OSMOLARITY of solutions or BODY FLUIDS.'),
  'D061287': NtO(MH='Glossopharyngeal Nerve Injuries', MS='Traumatic injuries to the GLOSSOPHARYNGEAL NERVE.'),
  'D061305': NtO(MH='Lymphoscintigraphy', MS='Radionuclide imaging of the LYMPHATIC SYSTEM.'),
  'D061306': NtO(MH='Fibrobacteres', MS='A phylum of gram-negative bacteria closely related to the BACTEROIDETES. Species have been found in the gastrointestinal tracts of animals, especially termites, though indications are that species are more broadly distributed in other environments.'),
  'D061307': NtO(MH='Human Umbilical Vein Endothelial Cells', MS='Endothelial cells that line venous vessels of the UMBILICAL CORD.'),
  'D061325': NtO(MH='Hereditary Breast and Ovarian Cancer Syndrome', MS='Autosomal dominant HEREDITARY CANCER SYNDROME in which a mutation most often in either BRCA1 or BRCA2 is associated with a significantly increased risk for breast and ovarian cancers.'),
  'D061326': NtO(MH='Index of Orthodontic Treatment Need', MS="A dental health survey developed to evaluate a patient's orthodontic treatment need and priority for orthodontic care. The index is based on an assessment of degree of MALOCCLUSION and the potential aesthetic and dental health benefit of the treatment under consideration."),
  'D061327': NtO(MH='Verrucomicrobia', MS='A phylum of gram-negative bacteria containing seven class-level groups from a wide variety of environments. Most members are chemoheterotrophs.'),
  'D061328': NtO(MH='Planctomycetales', MS='A order of gram-negative bacteria whose members are found in a variety of aquatic habitats as well as animal hosts.'),
  'D061329': NtO(MH='Oil and Gas Fields', MS='Areas of the earth where hydrocarbon deposits of PETROLEUM and/or NATURAL GAS are located.'),
  'D061330': NtO(MH='Multidetector Computed Tomography', MS='Types of spiral computed tomography technology in which multiple slices of data are acquired simultaneously improving the resolution over single slice acquisition technology.'),
  'D061345': NtO(MH='Early Medical Intervention', MS='Action taken to reduce susceptibility or exposure to health problems and to detect and treat disease in early stages.'),
  'D061346': NtO(MH='Laser Capture Microdissection', MS='Techniques using a laser to cut away and harvest a specific cell or cluster of cells from a tissue section while viewing it under the microscope.'),
  'D061347': NtO(MH='Forgiveness', MS='Excusing or pardoning for an offense or release of anger or resentment.'),
  'D061348': NtO(MH='Medical Chaperones', MS='Individuals who are present as a third party during medical encounters such as physical examination by a healthcare professional of an individual of the opposite sex.'),
  'D061349': NtO(MH='Unrelated Donors', MS='Providers of tissues for transplant to non-related individuals.'),
  'D061350': NtO(MH='Sympatry', MS='In evolutionary theory, overlapping geographic distribution of diverging species. In sympatric GENETIC SPECIATION, genetic diversion occurs without geographic separation.'),
  'D061351': NtO(MH='Telescopes', MS='Instruments used to observe distant objects.'),
  'D061352': NtO(MH='Pasteurization', MS='Treatment of food with physical methods such as heat, high pressure, radiation, or electric current to destroy organisms that cause disease or food spoilage.'),
  'D061353': NtO(MH='Food Storage', MS='Keeping food for later consumption.'),
  'D061354': NtO(MH='Acinar Cells', MS='Cells lining the saclike dilatations known as acini of various glands or the lungs.'),
  'D061365': NtO(MH='Amish', MS='An ethnic group with shared religious beliefs in PROTESTANISM. Originating in Switzerland in the late 1600s, and first migrating to the mid-Atlantic, they now live throughout Eastern and Mid-Western United States and elsewhere. Communities are usually close-knit and marriage is within the community.'),
  'D061366': NtO(MH='Numbers Needed To Treat', MS='Number of patients who need to be treated in order to prevent one additional bad outcome. It is the inverse of Absolute Risk Reduction.'),
  'D061385': NtO(MH='Insulins', MS='Peptide hormones that cause an increase in the absorption of GLUCOSE by cells within organs such as LIVER; MUSCLE and ADIPOSE TISSUE. During normal metabolism insulins are produced by the PANCREATIC BETA CELLS in response to increased GLUCOSE. Natural and chemically-modified forms of insulin are also used in the treatment of GLUCOSE METABOLISM DISORDERS such as DIABETES MELLITUS.'),
  'D061386': NtO(MH='Insulin, Regular, Human', MS='Regular insulin preparations that contain the HUMAN insulin peptide sequence.'),
  'D061387': NtO(MH='Chlamydial Pneumonia', MS='Pneumonia caused by infections with the genus CHLAMYDIA; and CHLAMYDOPHILA, usually with CHLAMYDOPHILA PNEUMONIAE.'),
  'D061388': NtO(MH='Firefighters', MS='Professional or volunteer members of a fire department who are trained to suppress fire and respond to related emergency.'),
  'D061389': NtO(MH='Insulin, Regular, Pork', MS='Regular insulin preparations that contain the SUS SCROFA insulin peptide sequence.'),
  'D061405': NtO(MH='Insulin, Lente', MS='An insulin, zinc chloride preparation in the form of a suspension of crystals and amorphous material in a ratio of approximately 7:3. Typically, lente insulin has a duration of activity that lasts between 13-20 hours after dosage.'),
  'D061406': NtO(MH='Insulin, Ultralente', MS='A preparation of insulin and zinc chloride in the form of a crystalline suspension. Typically the duration of ultralente insulin activity lasts between 18-30 hours after dosage.'),
  'D061466': NtO(MH='Lopinavir', MS='An HIV protease inhibitor used in a fixed-dose combination with RITONAVIR. It is also an inhibitor of CYTOCHROME P-450 CYP3A.'),
  'D061485': NtO(MH='Tobacco Use Cessation Devices', MS='Devices or delivery systems used to aid in ending a TOBACCO habit.'),
  'D061505': NtO(MH='Cell Surface Display Techniques', MS='Techniques utilizing cells that express RECOMBINANT FUSION PROTEINS engineered to translocate through the CELL MEMBRANE and remain attached to the outside of the cell.'),
  'D061525': NtO(MH='Perforator Flap', MS='A mass of tissue for transplantation that includes the skin and/or the SUBCUTANEOUS FAT, and the perforating blood vessel that traverses the underlying tissue to supply blood to the skin. Perforator flaps are named after the anatomical region or muscle from where they are transplanted and/or the perforating blood vessel.'),
  'D061545': NtO(MH='Carbonated Water', MS='Water naturally or artificially infused with CARBON DIOXIDE.'),
  'D061565': NtO(MH='Cycloaddition Reaction', MS='Synthetic organic reactions that use reactions between unsaturated molecules to form cyclical products.'),
  'D061566': NtO(MH='Voltage-Gated Sodium Channels', MS='A family of membrane proteins that selectively conduct SODIUM ions due to changes in the TRANSMEMBRANE POTENTIAL DIFFERENCE. They typically have a multimeric structure with a core alpha subunit that defines the sodium channel subtype and several beta subunits that modulate sodium channel activity.'),
  'D061567': NtO(MH='Voltage-Gated Sodium Channel Blockers', MS='A class of drugs that inhibit the activation of VOLTAGE-GATED SODIUM CHANNELS.'),
  'D061585': NtO(MH='Voltage-Gated Sodium Channel Agonists', MS='Compounds that either stimulate the opening or prevent closure of VOLTAGE-GATED SODIUM CHANNELS.'),
  'D061605': NtO(MH='Administration, Intravenous', MS='Delivery of substances through VENIPUNCTURE into the VEINS.'),
  'D061625': NtO(MH='Immunoreceptor Tyrosine-Based Activation Motif', MS='A conserved AMINO ACID SEQUENCE located in the intracellular domains of a family of transmembrane proteins involved in various IMMUNE RESPONSES. The CONSENSUS SEQUENCE of this motif is YXXL(or I)X(6-8)YXXL(or I) (where X denotes any amino acid). When phosphorylated ITAM motifs provide docking sites for PROTEIN TYROSINE KINASES of the Syk family thus forming signaling complexes which lead to activation of immune responses.'),
  'D061626': NtO(MH='Immunoreceptor Tyrosine-Based Inhibition Motif', MS='A conserved AMINO ACID SEQUENCE located in the intracellular domains of a family of transmembrane proteins that negatively regulate the signal transduction processes emanating from transmembrane proteins containing IMMUNORECEPTOR TYROSINE-BASED ACTIVATION MOTIFS. The CONSENSUS SEQUENCE of this motif is I(or V)LXYXXL(or V) (where X denotes any amino acid). Also known as ITIM motifs.'),
  'D061645': NtO(MH='Abdominoplasty', MS='Surgical removal of excess abdominal skin and fat and tightening of the ABDOMINAL WALL. Abdominoplasty may include LIPECTOMY of INTRA-ABDOMINAL FAT, tightening of the ABDOMINAL MUSCLES, and re-creation of the UMBILICUS.'),
  'D061646': NtO(MH='Operative Time', MS='The duration of a surgical procedure in hours and minutes.'),
  'D061665': NtO(MH='Time-to-Treatment', MS='The interval of time between onset of symptoms and initiation of therapy.'),
  'D061685': NtO(MH='Time-to-Pregnancy', MS='Time interval, or number of non-contraceptive menstrual cycles that it takes for a couple to conceive.'),
  'D061686': NtO(MH='Premature Ejaculation', MS='The emission of SEMEN and seminal fluid during the act of preparation for sexual intercourse, i.e. before there is penetration, or shortly after penetration.'),
  'D061705': NtO(MH='Image-Guided Biopsy', MS='Conducting a biopsy procedure with the aid of a MEDICAL IMAGING modality.'),
  'D061725': NtO(MH='Accelerometry', MS='Qualitative and quantitative measurement of MOVEMENT patterns.'),
  'D061745': NtO(MH='Antioxidant Response Elements', MS='Nucleotide sequences that are found in the PROMOTER REGIONS of the genes of stress-responsive and cytoprotective proteins, such as those encoding antioxidant and PHASE II DETOXIFICATION enzymes. NF-E2-RELATED FACTOR 2 containing transcription factors bind to these elements during induction of these genes.'),
  'D061765': NtO(MH='Endoscopic Ultrasound-Guided Fine Needle Aspiration', MS='Conducting a fine needle biopsy with the aid of ENDOSCOPIC ULTRASONOGRAPHY.'),
  'D061766': NtO(MH='Proton Therapy', MS='The use of an external beam of PROTONS as radiotherapy.'),
  'D061785': NtO(MH='Transcription Initiation, Genetic', MS='The process that starts the transcription of an RNA molecule. It includes the assembly of the initiation complex and establishment of the start site.'),
  'D061805': NtO(MH='Transcription Elongation, Genetic', MS='The lengthening of a nascent RNA molecule by RNA POLYMERASE during transcription.'),
  'D061806': NtO(MH='Transcription Termination, Genetic', MS='The discontinuation of transcription at the end of a transcription unit, including the recognition of termination sites and release of the newly synthesized RNA molecule.'),
  'D061807': NtO(MH='Catheter Obstruction', MS='A hindrance to the passage of fluids through a CATHETER.'),
  'D061808': NtO(MH='Air Filters', MS='Barriers used to separate and remove PARTICULATE MATTER from air.'),
  'D061809': NtO(MH='Human Papillomavirus DNA Tests', MS='Methods for detecting or typing the DNA of an ALPHAPAPILLOMAVIRUS in biological tissues and fluids.'),
  'D061810': NtO(MH='One-Lung Ventilation', MS='Techniques for supplying artificial respiration to a single lung.'),
  'D061825': NtO(MH='Metal-on-Metal Joint Prostheses', MS='Types of prosthetic joints in which both wear surfaces of the joint coupling are metallic.'),
  'D061826': NtO(MH='Foot Orthoses', MS='Devices used to support or align the foot structure, or to prevent or correct foot deformities.'),
  'D061827': NtO(MH='Dimensional Measurement Accuracy', MS='The closeness of a determined value of a physical dimension to the actual value.'),
  'D061845': NtO(MH='Smoke-Free Policy', MS='Prohibition against tobacco smoking in specific areas to control TOBACCO SMOKE POLLUTION.'),
  'D061846': NtO(MH='Hospitals, Low-Volume', MS='Hospitals with a much lower than average utilization by physicians and smaller number of procedures.'),
  'D061847': NtO(MH='Hospitals, High-Volume', MS='Hospitals with a much higher than average utilization by physicians and a large number of procedures.'),
  'D061848': NtO(MH='Optical Imaging', MS='The use of light interaction (scattering, absorption, and fluorescence) with biological tissue to obtain morphologically based information. It includes measuring inherent tissue optical properties such as scattering, absorption, and autofluorescence; or optical properties of exogenous targeted fluorescent molecular probes such as those used in optical MOLECULAR IMAGING, or nontargeted optical CONTRAST AGENTS.'),
  'D061865': NtO(MH='Hemorrhoidectomy', MS='The surgical removal of HEMORRHOIDS.'),
  'D061885': NtO(MH='Chromophore-Assisted Light Inactivation', MS='A technique for causing a targeted loss of molecular function from REACTIVE OXYGEN SPECIES that are formed by the illumination of dyes placed in the immediate vicinity of the target molecule.'),
  'D061886': NtO(MH='Neuromuscular Monitoring', MS='The use of peripheral nerve stimulation to assess transmission at the NEUROMUSCULAR JUNCTION, especially in the response to anesthetics, such as the intensity of NEUROMUSCULAR BLOCKADE by NEUROMUSCULAR BLOCKING AGENTS.'),
  'D061887': NtO(MH='Conversion to Open Surgery', MS='Changing an operative procedure from an endoscopic surgical procedure to an open approach during the INTRAOPERATIVE PERIOD.'),
  'D061888': NtO(MH='Craniospinal Irradiation', MS='A comprehensive radiation treatment of the entire CENTRAL NERVOUS SYSTEM.'),
  'D061889': NtO(MH='Healthcare Financing', MS='Methods of generating, allocating, and using financial resources in healthcare systems.'),
  'D061891': NtO(MH='Controlled Substances', MS='Drugs or chemical agents whose manufacture, possession, or use are regulated by government. This may include narcotics and prescription medications.'),
  'D061895': NtO(MH='External Debt', MS='Money owed to creditors outside of a country.'),
  'D061905': NtO(MH='Single-Domain Antibodies', MS='An immunoglobulin fragment composed of one variable domain from an IMMUNOGLOBULIN HEAVY CHAIN or IMMUNOGLOBULIN LIGHT CHAIN.'),
  'D061906': NtO(MH='Advanced Trauma Life Support Care', MS='The initial life support care of the severely injured patient.'),
  'D061925': NtO(MH='Immobilized Nucleic Acids', MS='DNA or RNA bound to a substrate thereby having fixed positions.'),
  'D061945': NtO(MH='Caspase Inhibitors', MS='Endogenous and exogenous compounds and that either inhibit CASPASES or prevent their activation.'),
  'D061965': NtO(MH='Matrix Metalloproteinase Inhibitors', MS='Compounds that inhibit the enzyme activity or activation of MATRIX METALLOPROTEINASES.'),
  'D061985': NtO(MH='Madin Darby Canine Kidney Cells', MS='An epithelial cell line derived from a kidney of a normal adult female dog.'),
  'D061986': NtO(MH='MCF-7 Cells', MS='An estrogen responsive cell line derived from a patient with metastatic human breast ADENOCARCINOMA (at the Michigan Cancer Foundation.)'),
  'D061987': NtO(MH='Sf9 Cells', MS='Cell line derived from SF21 CELLS which are a cell line isolated from primary explants of SPODOPTERA FRUGIPERDA pupal tissue.'),
  'D061988': NtO(MH='Proteasome Inhibitors', MS='Compounds that inhibit the function or proteolytic action of the PROTEASOME.'),
  'D062005': NtO(MH='Biopsy, Large-Core Needle', MS='The use of needles usually larger than 14-gauge to remove tissue samples large enough to retain cellular architecture for pathology examination.'),
  'D062025': NtO(MH='Organophosphate Poisoning', MS='Poisoning due to exposure to ORGANOPHOSPHORUS COMPOUNDS, such as ORGANOPHOSPHATES; ORGANOTHIOPHOSPHATES; and ORGANOTHIOPHOSPHONATES.'),
  'D062026': NtO(MH='Alice in Wonderland Syndrome', MS='Neurological condition characterized by disturbances in VISUAL PERCEPTION, most often of BODY SCHEMA, TIME PERCEPTION and HALLUCINATIONS. It is associated with MIGRAINE, infections (e.g., INFECTIOUS MONONUCLEOSIS), FEVER, EPILEPSY, and other neurological and psychiatric disorders.'),
  'D062027': NtO(MH='Linear IgA Bullous Dermatosis', MS='Autoimmune disease characterized by subepidermal blisters and linear deposition of autoantibodies at the dermoepidermal junction. The accumulated autoantibodies are of IMMUNOGLOBULIN A and occasionally IMMUNOGLOBULIN G classes against epidermal BASEMENT MEMBRANE proteins. The dermatosis is sometimes associated with malignancies and use of certain drugs (e.g., VANCOMYCIN).'),
  'D062028': NtO(MH='Bioprinting', MS='A material transfer technique used for assembling biological material or cells into a prescribed organization to create functional structures such as MICROCHIP ANALYTICAL DEVICES, cell microarrays, or three dimensional anatomical structures.'),
  'D062047': NtO(MH='Silorane Resins', MS='Polymeric resins containing a combination of SILOXANES and OXIRANES.'),
  'D062048': NtO(MH='Narrow Band Imaging', MS='Imaging techniques that use illumination created with several optical interference filters by which the frequency ranges are spectrally narrowed and light scatter is greatly reduced. Thus the reflected photons reconstituting the images are from distinct depths (the surface and deeper layers) of the object being imaged.'),
  'D062065': NtO(MH='Waste Water', MS='Contaminated water generated as a waste product of human activity.'),
  'D062066': NtO(MH='Water Resources', MS='Environmental reservoirs of water related to natural WATER CYCLE by which water is obtained for various purposes. This includes but is not limited to watersheds, aquifers and springs.'),
  'D062067': NtO(MH='Chemical Safety', MS='Risk or hazard associated with the handling and use of chemicals.'),
  'D062068': NtO(MH='Inventors', MS='Persons or entities that introduce a novel composition, device, or process, as well as improvements thereof.'),
  'D062069': NtO(MH='Inventions', MS='A novel composition, device, or process, independently conceived de novo or derived from a pre-existing model.'),
  'D062070': NtO(MH='Hydrology', MS="Science dealing with the properties, distribution, and circulation of water on and below the earth's surface, and atmosphere."),
  'D062071': NtO(MH='Infant, Extremely Premature', MS='A human infant born before 28 weeks of GESTATION.'),
  'D062072': NtO(MH='Patient Acuity', MS="An assessment of a patient's illness, its chronicity, severity, and other qualitative aspects."),
  'D062085': NtO(MH='RNA, Long Noncoding', MS='A class of untranslated RNA molecules that are typically greater than 200 nucleotides in length and do not code for proteins. Members of this class have been found to play roles in transcriptional regulation, post-transcriptional processing, CHROMATIN REMODELING, and in the epigenetic control of chromatin.'),
  'D062086': NtO(MH='Transtympanic Micropressure Treatment', MS='The intermittent delivery of pulses of air pressure (5 -20 cmH2O) to the middle ear space through a TYMPANOSTOMY tube.'),
  'D062087': NtO(MH='Plant Mucilage', MS='A type of viscous polysaccharide that is secreted from PLANTS. It has natural properties that are useful in the formulation of ADHESIVES.'),
  'D062088': NtO(MH='Acellular Dermis', MS='Remaining tissue from normal DERMIS tissue after the cells are removed.'),
  'D062105': NtO(MH='Molecular Docking Simulation', MS='A computer simulation technique that is used to model the interaction between two molecules. Typically the docking simulation measures the interactions of a small molecule or ligand with a part of a larger molecule such as a protein.'),
  'D062106': NtO(MH='Protein Degradation End Products', MS='Degraded protein components that are found within TISSUES, bodily fluids, and cells under certain pathological conditions. They are largely formed through chemical reactions which occur during non-physiological conditions such as ISCHEMIA; HYPERGLYCEMIA; and PHYSIOLOGICAL STRESS.'),
  'D062107': NtO(MH='Advanced Oxidation Protein Products', MS='A class of dityrosine-containing protein-derived molecules formed by OXIDATIVE STRESS. Their accumulation in plasma is associated with certain pathological conditions.'),
  'D062108': NtO(MH='May-Thurner Syndrome', MS='A compression of ILIAC VEIN that results in a decreased flow in the vein and in the left LOWER EXTREMITY due to a vascular malformation. It may result in left leg EDEMA, pain, iliofemoral DEEP VENOUS THROMBOSIS and POSTTHROMBOTIC SYNDROME. Compression of the left common ILIAC VEIN by the right common ILIAC ARTERY against the underlying fifth LUMBAR VERTEBRA is the typical underlying malformation.'),
  'D062109': NtO(MH='Dermatologic Surgical Procedures', MS='Operative procedures performed on the SKIN.'),
  'D062125': NtO(MH='Thermometry', MS='Measurement of the temperature of a material, or of the body or an organ by various temperature sensing devices which measure changes in properties of the material that vary with temperature, such as ELASTICITY; MAGNETIC FIELDS; or LUMINESCENCE.'),
  'D062126': NtO(MH='Databases, Chemical', MS='Databases devoted to knowledge about specific chemicals.'),
  'D062145': NtO(MH='Maternal Serum Screening Tests', MS="Analysis of the level of specific BIOMARKERS in a pregnant woman's sera to identify those at risk for PREGNANCY COMPLICATIONS or BIRTH DEFECTS."),
  'D062165': NtO(MH='Receptors, Artificial', MS='Receptors that are created by SYNTHETIC CHEMISTRY TECHNIQUES. They are usually designed to mimic endogenous CELL SURFACE RECEPTORS.'),
  'D062185': NtO(MH='Atrial Pressure', MS='The pressure within the CARDIAC ATRIUM. It can be measured directly by using a pressure catheter (see HEART CATHETERIZATION). It can be also estimated using various imaging techniques or other pressure readings such as PULMONARY CAPILLARY WEDGE PRESSURE (an estimate of left atrial pressure) and CENTRAL VENOUS PRESSURE (an estimate of right atrial pressure).'),
  'D062186': NtO(MH='Arterial Pressure', MS='The blood pressure in the ARTERIES. It is commonly measured with a SPHYGMOMANOMETER on the upper arm which represents the arterial pressure in the BRACHIAL ARTERY.'),
  'D062187': NtO(MH='Spinal Cord Stimulation', MS='Application of electric current to the spine for treatment of a variety of conditions involving innervation from the spinal cord.'),
  'D062205': NtO(MH='Strategic Stockpile', MS='Planned and coordinated pre-event accumulation of ESSENTIAL DRUGS and medical supplies.'),
  'D062206': NtO(MH='Spatial Analysis', MS='Investigative techniques which measure the topological, geometric, and or geographic properties of the entities studied.'),
  'D062207': NtO(MH='Brain-Computer Interfaces', MS='Instrumentation consisting of hardware and software that communicates with the BRAIN. The hardware component of the interface records brain signals, while the software component analyzes the signals and converts them into a command that controls a device or sends a feedback signal to the brain.'),
  'D062209': NtO(MH='Patient Handoff', MS='The transferring of patient care responsibility from one health-care professional to another.'),
  'D062210': NtO(MH='Personal Narrative', MS='Work consisting of accounts of individual experience in relation to a particular field or of participation in related activities.'),
  'D062211': NtO(MH='Spatio-Temporal Analysis', MS='Techniques which study entities using their topological, geometric, or geographic properties and include the dimension of time in the analysis.'),
  'D062245': NtO(MH='RxNorm', MS='A standardized nomenclature for clinical drugs and drug delivery devices. It links its names to many of the drug vocabularies commonly used in pharmacy management.'),
  'D062265': NtO(MH='Health Insurance Exchanges', MS='State-provided health insurance marketplaces established under the PATIENT PROTECTION AND AFFORDABLE CARE ACT.'),
  'D062305': NtO(MH='Geographic Mapping', MS='Creating a representation of areas of the earth or other celestial bodies, for the purpose of visualizing spatial distributions of various information.'),
  'D062306': NtO(MH='Geography, Medical', MS='The area of medicine concerned with the effects on health and disease due to geographic factors such as CLIMATE, environmental conditions, and geographic location.'),
  'D062307': NtO(MH='Medical Device Legislation', MS='Laws, statutes, and regulations pertaining to devices used in medicine.'),
  'D062308': NtO(MH='Optogenetics', MS='The combination of genetic and optical methods in controlling specific events with temporal precision in targeted cells of a functioning intact biological system.'),
  'D062309': NtO(MH='Community Integration', MS='Policies and programs which ensure that DISPLACED PERSONS and chronic illnesses receive the support and SOCIAL SERVICES needed to live in their communities.'),
  'D062312': NtO(MH='Islands', MS='Tracts of land completely surrounded by water.'),
  'D062313': NtO(MH='Databases, Pharmaceutical', MS='Databases devoted to knowledge about PHARMACEUTICAL PRODUCTS.'),
  'D062325': NtO(MH='Intense Pulsed Light Therapy', MS='Treatment of the skin with flashlamps of prescribed wavelengths, fluence, and pulse durations which target specific chromophores to induce photothermolysis at specific locations in the skin such as at the HAIR FOLLICLE or SPIDER VEINS.'),
  'D062326': NtO(MH='Re-Epithelialization', MS='Reconstitution of eroded or injured EPITHELIUM by proliferation and migration of EPITHELIAL CELLS from below or adjacent to the damaged site.'),
  'D062327': NtO(MH='Skin Lightening Preparations', MS='Substances used to obtain a lighter skin complexion or to treat HYPERPIGMENTATION disorders.'),
  'D062345': NtO(MH='Nuclear Microscopy', MS='A method of simultaneously imaging and measuring elements at the submicron level. Nuclear microscopy uses a focused high-energy ion beam of PROTONS and ALPHA PARTICLES (a nuclear microprobe) to interact with the sample. The resulting emitted radiations are analyzed by a group of techniques simultaneously: PARTICLE INDUCED X RAY EMISSION SPECTROMETRY for minor and trace element identification; Rutherford Backscattering Spectroscopy to assess sample thickness and bulk elements such as carbon, hydrogen, oxygen, and nitrogen; and Scanning Transmission Ion Microscopy to assess sample structure and density.'),
  'D062346': NtO(MH='Molecular Medicine', MS='The field of medicine concerned with understanding the biochemical basis of health and disease and involved in developing diagnostic and therapeutic methods that utilize MOLECULAR BIOLOGY techniques.'),
  'D062365': NtO(MH='Aminobenzoates', MS='Derivatives of BENZOIC ACID that contain one or more amino groups attached to the benzene ring structure. Included under this heading are a broad variety of acid forms, salts, esters, and amides that include the aminobenzoate structure.'),
  'D062366': NtO(MH='para-Aminobenzoates', MS='Benzoic acids, salts, or esters that contain an amino group attached to carbon number 4 of the benzene ring structure.'),
  'D062367': NtO(MH='ortho-Aminobenzoates', MS='Benzoic acids, salts, or esters that contain an amino group attached to carbon number 2 or 6 of the benzene ring structure.'),
  'D062368': NtO(MH='meta-Aminobenzoates', MS='Aminobenzoate derivatives that contain an amino group attached to carbon number 3 or 5 of the benzene ring structure.'),
  'D062385': NtO(MH='Hydroxybenzoates', MS='Benzoate derivatives substituted by one or more hydroxy groups in any position on the benzene ring.'),
  'D062405': NtO(MH='Motivational Interviewing', MS="It is a client-centered, directive method for eliciting intrinsic motivation to change using open-ended questions, reflective listening, and decisional balancing. This nonjudgmental, nonconfrontational interviewing style is designed to minimize a patient's resistance to change by creating an interaction that supports open discussion of risky or problem behavior."),
  'D062406': NtO(MH='Fukushima Nuclear Accident', MS='Nuclear power accident that occurred following the Tohoku-Kanto earthquake of March 11, 2011 in the northern region of Japan.'),
  'D062407': NtO(MH='Meals', MS='A portion of the food eaten for the day, usually at regular occasions during the day.'),
  'D062408': NtO(MH='Breakfast', MS='The first meal of the day.'),
  'D062409': NtO(MH='Lunch', MS='The meal taken at midday.'),
  'D062410': NtO(MH='Snacks', MS='Foods eaten between MEALTIMES.'),
  'D062425': NtO(MH='Hydroxybenzoate Ethers', MS='Benzoate derivatives that contain one or more alkyl or aryl groups linked to the benzene ring structure by OXYGEN.'),
  'D062445': NtO(MH='Claudin-1', MS='An integral membrane protein that is localized to TIGHT JUNCTIONS, where it plays a role in controlling the paracellular permeability of polarized cells. Mutations in the gene for claudin-1 are associated with Neonatal Ichthyosis-Sclerosing Cholangitis (NISCH) Syndrome.'),
  'D062446': NtO(MH='Claudin-2', MS='A claudin subtype that is associated with the formation of cation-selective channels and increased epithelial permeability. It is localized to the TIGHT JUNCTIONS of the PROXIMAL KIDNEY TUBULE and INTESTINAL EPITHELIUM.'),
  'D062465': NtO(MH='Claudin-3', MS='A ubiquitously-expressed claudin subtype that acts as a general barrier-forming protein in TIGHT JUNCTIONS. Elevated expression of claudin-3 is found in a variety of tumor cell types, suggesting its role as a therapeutic target for specific ANTINEOPLASTIC AGENTS.'),
  'D062485': NtO(MH='Breath Holding', MS='An involuntary or voluntary pause in breathing, sometimes accompanied by loss of consciousness.'),
  'D062486': NtO(MH='Public Health Surveillance', MS='The ongoing, systematic collection, analysis, and interpretation of health-related data with the purpose of preventing or controlling disease or injury, or of identifying unusual events of public health importance, followed by the dissemination and use of information for public health action. (From Am J Prev Med 2011;41(6):636)'),
  'D062487': NtO(MH='Social Marginalization', MS='Individuals or groups, excluded from participation in the economic, social, and political activities of membership in a community.'),
  'D062489': NtO(MH='Health Impact Assessment', MS='Combination of procedures, methods, and tools by which a policy, program, or project may be judged as to its potential effects on the health of a population, and the distribution of those effects within the population.'),
  'D062505': NtO(MH='Value-Based Purchasing', MS='Purchasers are provided information on the quality of health care, including patient outcomes and health status, with data on the dollar outlays going towards health. The focus is on managing the use of the health care system to reduce inappropriate care and to identify and reward the best-performing providers. (from http://www.ahrq.gov/qual/meyerrpt.htm accessed 11/25/2011)'),
  'D062506': NtO(MH='Claudin-4', MS='A claudin subtype that takes part in maintaining the barrier-forming property of TIGHT JUNCTIONS. Claudin-4 is found associated with CLAUDIN-8 in the KIDNEY COLLECTING DUCT where it may play a role in paracellular chloride ion reabsorption.'),
  'D062507': NtO(MH='Claudin-5', MS='A claudin subtype that is found localized to TIGHT JUNCTIONS in VASCULAR ENDOTHELIAL CELLS. The protein was initially identified as one of several proteins which are deleted in VELOCARDIOFACIAL SYNDROME and may play an important role in maintaining the integrity of the BLOOD-BRAIN BARRIER.'),
  'D062525': NtO(MH='Narrative Therapy', MS='A form of PSYCHOTHERAPY that centers on the individuals as the experts in their own lives and views problems as separate from people. It is assumed that people have many skills, competencies, beliefs, values, commitments, and abilities that will assist them to reduce the influence of problems in their lives.'),
  'D062526': NtO(MH='Patient Navigation', MS='The process of helping patients to effectively and efficiently use the health care system when faced with one or more of these challenges: (1) choosing, understanding, and using health coverage or applying for assistance when uninsured; (2) choosing, using, and understanding different types of health providers and services; (3) making treatment decisions; and (4) managing care received by multiple providers.'),
  'D062527': NtO(MH='Meaningful Use', MS='Using certified ELECTRONIC HEALTH RECORDS technology to improve quality, safety, efficiency, and reduce HEALTHCARE DISPARITIES; engage patients and families in their health care; improve care coordination; improve population and public health; while maintaining privacy and security.'),
  'D062528': NtO(MH='Medicalization', MS='A process by which nonmedical problems become defined and treated as medical problems, usually in terms of illnesses, or disorders. (Annu Rev Sociol 1992 18:209)'),
  'D062546': NtO(MH='Voltage-Gated Sodium Channel beta Subunits', MS='Voltage-gated sodium channel subunits that play a role in the assembly, expression, and functional modulation of the sodium channel. They form a heterotrimeric complex with the pore-forming sodium channel alpha subunits.'),
  'D062547': NtO(MH='Voltage-Gated Sodium Channel beta-1 Subunit', MS='A voltage-gated sodium channel beta subunit abundantly expressed in SKELETAL MUSCLE; HEART; and BRAIN. It non-covalently associates with voltage-gated alpha subunits. Defects in the SCN1B gene, which codes for this beta subunit, are associated with generalized epilepsy with febrile seizures plus, type 1, and Brugada syndrome 5.'),
  'D062548': NtO(MH='Voltage-Gated Sodium Channel beta-2 Subunit', MS='A voltage-gated sodium channel beta subunit that binds covalently to voltage-gated alpha subunits.'),
  'D062549': NtO(MH='Voltage-Gated Sodium Channel beta-3 Subunit', MS='A voltage-gated sodium channel beta subunit subtype that non-covalently associates with voltage-gated alpha subunits. Defects in the SCN3B gene which codes for this beta subunit are associated with Brugada syndrome 7.'),
  'D062550': NtO(MH='NAV1.1 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype that is predominantly expressed in the CENTRAL NERVOUS SYSTEM. Defects in the SCN1A gene which codes for the alpha subunit of this sodium channel are associated with DRAVET SYNDROME, generalized epilepsy with febrile seizures plus, type 2 (GEFS+2), and familial hemiplegic migraine type 3.'),
  'D062551': NtO(MH='NAV1.2 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype that mediates the sodium ion permeability of excitable membranes. Defects in the SCN2A gene which codes for the alpha subunit of this sodium channel are associated with benign familial infantile seizures type 3, and early infantile epileptic encephalopathy type 11.'),
  'D062552': NtO(MH='NAV1.3 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype found in neuronal tissue that mediates the sodium ion PERMEABILITY of excitable membranes.'),
  'D062553': NtO(MH='NAV1.4 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype that mediates the sodium ion PERMEABILITY of SKELETAL MYOCYTES. Defects in the SCN4A gene, which codes for the alpha subunit of this sodium channel, are associated with several MYOTONIC DISORDERS.'),
  'D062554': NtO(MH='NAV1.5 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype that mediates the sodium ion PERMEABILITY of CARDIOMYOCYTES. Defects in the SCN5A gene, which codes for the alpha subunit of this sodium channel, are associated with a variety of CARDIAC DISEASES that result from loss of sodium channel function.'),
  'D062556': NtO(MH='NAV1.7 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype found widely expressed in nociceptive primary sensory neurons. Defects in the SCN9A gene, which codes for the alpha subunit of this sodium channel, are associated with several pain sensation-related disorders.'),
  'D062557': NtO(MH='NAV1.6 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype found widely expressed in neurons of the central and peripheral nervous systems. Defects in the SCN8A gene which codes for the alpha subunit of this sodium channel are associated with ATAXIA and cognitive deficits.'),
  'D062558': NtO(MH='NAV1.9 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype found in the neurons of the NERVOUS SYSTEM and DORSAL ROOT GANGLIA. It may play a role in the generation of heat and mechanical pain hypersensitivity.'),
  'D062559': NtO(MH='NAV1.8 Voltage-Gated Sodium Channel', MS='A voltage-gated sodium channel subtype that is expressed in nociceptors, including spinal and trigeminal sensory neurons. It plays a role in the transmission of pain signals induced by cold, heat, and mechanical stimuli.'),
  'D062565': NtO(MH='Acid Sensing Ion Channels', MS='A family of proton-gated sodium channels that are primarily expressed in neuronal tissue. They are AMILORIDE-sensitive and are implicated in the signaling of a variety of neurological stimuli, most notably that of pain in response to acidic conditions.'),
  'D062585': NtO(MH='Voltage-Gated Sodium Channel beta-4 Subunit', MS='A voltage-gated sodium channel beta subunit subtype that covalently associates with voltage-gated alpha subunits. Defects in the SCN4B gene, which codes for this beta subunit, are associated with long QT syndrome-10.'),
  'D062605': NtO(MH='Quillaja Saponins', MS='Natural detergents made up of a heterogeneous mixture of molecules having a triterpenoid core structure. They vary in aglycone (sapogenin) and sugar moieties, including glucose.'),
  'D062606': NtO(MH='Tertiary Care Centers', MS='A medical facility which provides a high degree of subspecialty expertise for patients from centers where they received SECONDARY CARE.'),
  'D062607': NtO(MH='Fungal Capsules', MS='An extracellular layer outside the cell wall of a fungus composed of polysaccharides. It may serve a protective role amongst others.'),
  'D062609': NtO(MH='Electronic Nose', MS='A device used to detect airborne odors, gases, flavors, volatile substances or vapors.'),
  'D062610': NtO(MH='Fungal Polysaccharides', MS='Cell wall components constituting a polysaccharide core found in fungi. They may act as antigens or structural substrates.'),
  'D062611': NtO(MH='Solid Waste', MS='Garbage, refuse, or sludge, or other discarded materials from a wastewater treatment plant, water supply treatment plant, and air pollution control facility that include solid, semi-solid, or contained material. It does not include materials dissolved in domestic sewage, irrigation return flows, or industrial discharges.'),
  'D062625': NtO(MH='Cystadenofibroma', MS='Benign or borderline malignant neoplasm of the ovary and surrounding tissues. It is characterized by tumor(s) with cystic glands which are lined by cuboidal EPITHELIAL CELLS with clear cytoplasm, resembling ENDOMETRIUM cells. The glands are separated by fibroblastic STROMAL CELLS.'),
  'D062626': NtO(MH='Degenerin Sodium Channels', MS='A family of mechanosensitive sodium channels found primarily in NEMATODES where they play a role in CELLULAR MECHANOTRANSDUCTION. Degenerin sodium channels are structurally-related to EPITHELIAL SODIUM CHANNELS and are named after the fact that loss of their activity results in cellular degeneration.'),
  'D062645': NtO(MH='Percutaneous Coronary Intervention', MS='A family of percutaneous techniques that are used to manage CORONARY OCCLUSION, including standard balloon angioplasty (PERCUTANEOUS TRANSLUMINAL CORONARY ANGIOPLASTY), the placement of intracoronary STENTS, and atheroablative technologies (e.g., ATHERECTOMY; ENDARTERECTOMY; THROMBECTOMY; PERCUTANEOUS TRANSLUMINAL LASER ANGIOPLASTY). PTCA was the dominant form of PCI, before the widespread use of stenting.'),
  'D062646': NtO(MH='Acid Sensing Ion Channel Blockers', MS='A subclass of sodium channel blockers that are specific for ACID-SENSING SODIUM CHANNELS.'),
  'D062665': NtO(MH='Epidemiological Monitoring', MS='Collection, analysis, and interpretation of data about the frequency, distribution, and consequences of disease or health conditions, for use in the planning, implementing, and evaluating public health programs.'),
  'D062666': NtO(MH='Vascular Access Devices', MS='Devices to be inserted into veins or arteries for the purpose of carrying fluids into or from a peripheral or central vascular location. They may include component parts such as catheters, ports, reservoirs, and valves. They may be left in place temporarily for therapeutic or diagnostic purposes.'),
  'D062685': NtO(MH='Steatocystoma Multiplex', MS='A disorder characterized by multiple, wide spread cutaneous cysts that often become inflamed and rupture. It is caused by the same mutations in the gene coding for KRT-17 that are causative mutations for Pachyonychia congenita, Type 2. Natal teeth involvement is sometimes associated with steatocystoma multiplex.'),
  'D062686': NtO(MH='Epithelial Sodium Channel Blockers', MS='A subclass of sodium channel blockers that are specific for EPITHELIAL SODIUM CHANNELS.'),
  'D062687': NtO(MH='Sodium Channel Agonists', MS='A class of drugs that stimulate sodium influx through cell membrane channels.'),
  'D062688': NtO(MH='Buschke-Lowenstein Tumor', MS='Exophytic tumor of the anogenital region associated with HPV infections. It becomes a large cauliflower-like, hyperkeratotic, verrucous mass that is locally destructive with little atypical cells. Histologically, tumor cells are broad rete peg structures that tend to invade deeper than CONDYLOMATA ACUMINATA.'),
  'D062689': NtO(MH='Lipoblastoma', MS='Benign tumors of fatty tissues found in infancy and childhood. It is associated chromosomal aberrations that result in activation of an oncogene on chromosome band 8q12.'),
  'D062690': NtO(MH='Vaccines, Live, Unattenuated', MS='Live vaccines prepared from microorganisms without their virulence altered. Examples include smallpox (vaccinia) and adenovirus vaccines.'),
  'D062705': NtO(MH='Adenovirus Vaccines', MS='Vaccines used to prevent infection by any virus from the family ADENOVIRIDAE.'),
  'D062706': NtO(MH='Prodromal Symptoms', MS='Clinical or physiological indicators that precede the onset of disease.'),
  'D062707': NtO(MH='Return to Work', MS='Resumption of normal work routine following a hiatus or period of absence due to injury, disability, or other reasons.'),
  'D062708': NtO(MH='Epithelial Sodium Channel Agonists', MS='Compounds that either stimulate the opening or prevent closure of EPITHELIAL SODIUM ION CHANNELS.'),
  'D062725': NtO(MH='Tight Junction Proteins', MS='Proteins that take part in the formation or structure of TIGHT JUNCTIONS.'),
  'D062726': NtO(MH='Junctional Adhesion Molecules', MS='A family of membrane glycoproteins localized to TIGHT JUNCTIONS that contain two extracellular Ig-like domains, a single transmembrane segment, and a cytoplasmic tail of variable length.'),
  'D062727': NtO(MH='Coxsackie and Adenovirus Receptor-Like Membrane Protein', MS='An Ig superfamily transmembrane protein that localizes to junctional complexes that occur between ENDOTHELIAL CELLS and EPTHELIAL CELLS. The protein may play a role in cell-cell adhesion and is the primary site for the attachment of ADENOVIRUSES during infection.'),
  'D062745': NtO(MH='Junctional Adhesion Molecule A', MS='A cell adhesion protein that is found within TIGHT JUNCTIONS of ENDOTHELIAL CELLS and on the CELL MEMBRANE surface of circulating PLATELETS.'),
  'D062765': NtO(MH='Junctional Adhesion Molecule B', MS='A junctional adhesion molecule subtype that is localized to high endothelial VENULES, heart ENDOTHELIUM, TROPHOBLASTS of the PLANCENTA, and in the ENDOTHELIUM of ARTERIOLES.'),
  'D062785': NtO(MH='Hemiarthroplasty', MS='A partial joint replacement in which only one surface of the joint is replaced with a PROSTHESIS.'),
  'D062786': NtO(MH='Junctional Adhesion Molecule C', MS='A junctional adhesion molecule subtype that is expressed at high levels in PLACENTA; BRAIN; KIDNEY; and PLATELETS. It serves a variety of functions such as mediating leukocyte-platelet interactions, regulating trans-epithelial migration of POLYMORPHONUCLEAR LEUKOCYTES, and acting as a counter receptor for ALPHAM INTEGRIN.'),
  'D062787': NtO(MH='Drug Overdose', MS='Accidental or deliberate use of a medication or street drug in excess of normal dosage.'),
  'D062788': NtO(MH='Adenomyosis', MS='The extension of endometrial tissue (ENDOMETRIUM) into the MYOMETRIUM. It usually occurs in women in their reproductive years and may result in a diffusely enlarged uterus with ectopic and benign endometrial glands and stroma.'),
  'D062789': NtO(MH='Tobacco Products', MS='Substances and products derived from NICOTIANA TABACUM.'),
  'D062791': NtO(MH='Optically Stimulated Luminescence Dosimetry', MS='The use of a radiation monitoring device composed of material which emits light after being illuminated. The amount of light emitted is proportional to the amount of IONIZING RADIATION to which the material has been previously exposed.'),
  'D062792': NtO(MH='MARVEL Domain-Containing Proteins', MS='A family of proteins that share a domain with a four transmembrane-helix architecture referred to as the MARVEL domain. The MARVEL domain proteins play important role in vesicular trafficking and in the formation of TIGHT JUNCTIONS.'),
  'D062793': NtO(MH='Occludin', MS='A MARVEL domain protein that binds to and regulates PROTEIN PHOSPHATASE 1. Occludin plays an important role in the formation and regulation of the TIGHT JUNCTION paracellular permeability barrier.'),
  'D062794': NtO(MH='MARVEL Domain Containing 2 Protein', MS='A tight junction-associated MARVEL protein that may play a role in separating the endolymphatic and perilymphatic spaces of the ORGAN OF CORTI. Defects in the gene that codes for MARVELD2 protein are a cause of deafness autosomal recessive type 49.'),
  'D062805': NtO(MH='Synaptogyrins', MS='A subfamily of MARVEL domain-containing proteins that are found in SYNAPTIC VESICLES, where they may play a role in modulating neuronal signaling.'),
  'D062806': NtO(MH='Myelin and Lymphocyte-Associated Proteolipid Proteins', MS='A family of MARVEL domain-containing proteolipid proteins involved in vesicular trafficking cycling between the GOLGI COMPLEX and the apical PLASMA MEMBRANE.'),
  'D062825': NtO(MH='Zonula Occludens Proteins', MS='A family of proteins that play a role in TIGHT JUNCTION formation by binding to and anchoring proteins to the ACTIN CYTOSKELETON.'),
  'D062826': NtO(MH='Zonula Occludens-1 Protein', MS='A 195-kDa zonula occludens protein with similarity to MEMBRANE-ASSOCIATED GUANYLATE KINASES. It is distinguished by the presence of a ZU5 domain at the C-terminal of the molecule.'),
  'D062827': NtO(MH='Zonula Occludens-2 Protein', MS='A zonula occludens protein subtype found in epithelial cell junctions. Several isoforms of zonula occludens-2 protein exist due to use of alternative promoter regions and alternative mRNA splicings.'),
  'D062845': NtO(MH='Chloroacetates', MS='ACETIC ACID or acetic acid esters substituted with one or more CHLORINE atoms.'),
  'D062846': NtO(MH='Non-Filarial Lymphedema', MS='A form of elephantiasis caused by soil particles which penetrate the skin of the foot. It is limited to tropical regions with soils of high volcanic content.'),
  'D062865': NtO(MH='Diuretics, Potassium Sparing', MS='A subclass of diuretics that limits the secretion of POTASSIUM into the URINE.'),
  'D062885': NtO(MH='Urinary Catheters', MS='Catheters inserted into the URINARY BLADDER or kidney for therapeutic or diagnostic purposes.'),
  'D062905': NtO(MH='Central Venous Catheters', MS='Catheters that are inserted into a large central vein such as a SUBCLAVIAN VEIN or FEMORAL VEIN.'),
  'D062906': NtO(MH='Cardiac Catheters', MS='Catheters inserted into various locations within the heart for diagnostic or therapeutic purposes.'),
  'D062907': NtO(MH='Oxaloacetic Acid', MS='A dicarboxylic acid ketone that is an important metabolic intermediate of the CITRIC ACID CYCLE. It can be converted to ASPARTIC ACID by ASPARTATE TRANSAMINASE.'),
  'D062947': NtO(MH='rho-Specific Guanine Nucleotide Dissociation Inhibitors', MS='A subcategory of guanine nucleotide dissociation inhibitors that are specific for RHO GTP-BINDING PROTEINS.'),
  'D062948': NtO(MH='rho Guanine Nucleotide Dissociation Inhibitor alpha', MS='An abundantly-expressed rho GDP-dissociation inhibitor subtype that regulates a broad variety of RHO GTPASES.'),
  'D062949': NtO(MH='rho Guanine Nucleotide Dissociation Inhibitor beta', MS='A rho GDP-dissociation inhibitor subtype that is highly expressed in hematopoietic cells and in LYMPHOCYTES. The expression of this subtype is associated with the regulation of CELL PROLIFERATION; TUMORIGENESIS; and APOPTOSIS.'),
  'D062950': NtO(MH='rho Guanine Nucleotide Dissociation Inhibitor gamma', MS='A rho GDP-dissociation inhibitor subtype that has a unique C-terminal alpha helical membrane-binding domain. It is found bound to CYTOPLASMIC VESICLES such as those associated with the GOLGI APPARATUS.'),
  'D062951': NtO(MH='Endoreduplication', MS='A type of nuclear polyploidization in which multiple cycles of DNA REPLICATION occur in the absence of CELL DIVISION and result in a POLYPLOID CELL.'),
  'D062952': NtO(MH='Stratospheric Ozone', MS="Ozone in the Earth's stratosphere. It is produced continuously by the action of solar ULTRAVIOLET RAYS on oxygen in the stratosphere. The stratospheric ozone (especially at the ozone layer) blocks much of the solar UV radiation of wavelengths of 320 nanometers or less from being transmitted to lower ATMOSPHERE of the Earth."),
  'D062965': NtO(MH='Cyclic GMP-Dependent Protein Kinase Type I', MS='A cyclic GMP-dependent protein kinase subtype that is expressed in SMOOTH MUSCLE tissues and plays a role in regulation of smooth muscle contraction. Two isoforms, PKGIalpha and PKGIbeta, of the type I protein kinase exist due to alternative splicing of its mRNA.'),
  'D062966': NtO(MH='Ozone Depletion', MS='A shift in the balance between production and destruction of STRATOSPHERIC OZONE that results in a decline of the amount of OZONE in the lower stratosphere.'),
  'D062967': NtO(MH='Cyclic GMP-Dependent Protein Kinase Type II', MS='A cyclic GMP-dependent protein kinase subtype that is expressed predominantly in INTESTINES, BRAIN, and KIDNEY. The protein is myristoylated on its N-terminus which may play a role its membrane localization.'),
  'D062985': NtO(MH='Health Services for Transgender Persons', MS='Access to specialized care for transgender populations. Health systems organized to take account of the special healthcare needs of marginalized groups who may face barriers in accessing health services. (http://www.who.int/reproductivehealth/topics/linkages/guidance_package.pdf)'),
  'D063005': NtO(MH='Health Information Systems', MS='A system for the collection and/or processing of data from various sources, and using the information for policy making and management of health services. It could be paper-based or electronic. (From http://web.worldbank.org/WBSITE/EXTERNAL/TOPICS/EXTHEALTHNUTRITIONANDPOPULATION/EXTHSD/0,,contentMDK:22239824~menuPK:376799~pagePK:148956~piPK:216618~theSitePK:376793,00.html. http://www.who.int/healthinfo/systems/en/)'),
  'D063025': NtO(MH='Health Information Management', MS='Management of the acquisition, organization, retrieval, and dissemination of health information.'),
  'D063026': NtO(MH='Dual MEDICAID MEDICARE Eligibility', MS='Coordination of coverage eligibility of those individuals who are entitled to MEDICAID and MEDICARE.'),
  'D063045': NtO(MH='Crowdsourcing', MS='Social media model for enabling public involvement and recruitment in participation. Use of social media to collect feedback and recruit volunteer subjects.'),
  'D063046': NtO(MH='Patient Medication Knowledge', MS='Patient health knowledge related to medications including what is being used and why as well as instructions and precautions.'),
  'D063065': NtO(MH='Organophosphonates', MS='Carbon-containing phosphonic acid compounds. Included under this heading are compounds that have carbon bound to either OXYGEN atom or the PHOSPHOROUS atom of the (P=O)O2 structure.'),
  'D063066': NtO(MH='Organofluorophosphonates', MS='A subclass of organophosphates that have a FLUORINE atom substituted for one of the OXYGEN atoms.'),
  'D063085': NtO(MH='Organothiophosphonates', MS='Carbon-containing thiophosphonic acid compounds. Included under this heading are compounds that have carbon bound to either SULFUR atom, PHOSPHOROUS atom, or the OXYGEN atom of the SPO2 core structure.'),
  'D063086': NtO(MH='Organothiophosphates', MS='Carbon-containing thiophosphoric acid derivatives. Included under this heading are compounds that have carbon bound to either SULFUR atom, or the OXYGEN atom of the SPO3 core structure.'),
  'D063087': NtO(MH='Noninvasive Ventilation', MS='Techniques for administering artificial respiration without the need for INTRATRACHEAL INTUBATION.'),
  'D063088': NtO(MH='Phosphoramides', MS='Amide derivatives of phosphoric acid such as compounds that include the phosphoric triamide (P(=O)(N)(N)(N)) structure.'),
  'D063105': NtO(MH='Food Assistance', MS='Food or financial assistance for food given to those in need.'),
  'D063106': NtO(MH='Transgender Persons', MS="Persons having a sense of persistent identification with, and expression of, gender-coded behaviors not typically associated with one's anatomical sex at birth, with or without a desire to undergo SEX REASSIGNMENT PROCEDURES."),
  'D063125': NtO(MH='Balloon Embolectomy', MS='The use of balloon CATHETERS to remove emboli by retraction of the balloon that is inflated behind the EMBOLUS.'),
  'D063126': NtO(MH='Balloon Valvuloplasty', MS='Widening of a stenosed HEART VALVE by the insertion of a balloon CATHETER into the valve and inflation of the balloon.'),
  'D063127': NtO(MH='Secondary Care', MS='Specialized healthcare delivered as a follow-up or referral from a PRIMARY CARE provider.'),
  'D063128': NtO(MH='Tertiary Healthcare', MS='Care of a highly technical and specialized nature, provided in a medical center, usually one affiliated with a university, for patients with unusually severe, complex, or uncommon health problems.'),
  'D063129': NtO(MH='Parental Death', MS='The death of the father or mother or another person in this role.'),
  'D063130': NtO(MH='Maternal Death', MS='The death of the female parent.'),
  'D063132': NtO(MH='Connectome', MS="A comprehensive map of the physical interconnections of an organism's neural networks. This modular organization of neuronal architecture is believed to underlie disease mechanisms and the biological development of the CENTRAL NERVOUS SYSTEM."),
  'D063146': NtO(MH='Polycomb-Group Proteins', MS='A family of proteins that play a role in CHROMATIN REMODELING. They are best known for silencing HOX GENES and the regulation of EPIGENETIC PROCESSES.'),
  'D063147': NtO(MH='Animal Distribution', MS='A process by which animals in various forms and stages of development are physically distributed through time and space.'),
  'D063148': NtO(MH='Plant Dispersal', MS='The physical distribution of plants in various forms and stages of development through time and space.'),
  'D063150': NtO(MH='Polycomb Repressive Complex 1', MS='A multisubunit polycomb protein complex with affinity for CHROMATIN that contains methylated HISTONE H3. It contains an E3 ubiquitin ligase activity that is specific for HISTONE H2A and works in conjunction with POLYCOMB REPRESSIVE COMPLEX 2 to effect EPIGENETIC REPRESSION.'),
  'D063151': NtO(MH='Polycomb Repressive Complex 2', MS='A multisubunit polycomb protein complex that catalyzes the METHYLATION of chromosomal HISTONE H3. It works in conjunction with POLYCOMB REPRESSIVE COMPLEX 1 to effect EPIGENETIC REPRESSION.'),
  'D063154': NtO(MH='Mitochondrial Dynamics', MS='The continuous remodeling of MITOCHONDRIA shape by fission and fusion in response to physiological conditions.'),
  'D063165': NtO(MH='Neurotology', MS='A subspecialty of otolaryngology dealing with the parts of the nervous system related to the ear.'),
  'D063166': NtO(MH='Maxillary Osteotomy', MS='Surgery of the upper jaw bone usually performed to correct upper and lower jaw misalignment.'),
  'D063168': NtO(MH='Mandibular Osteotomy', MS='Intraoral OSTEOTOMY of the lower jaw usually performed in order to correct MALOCCLUSION.'),
  'D063169': NtO(MH='Dentofacial Deformities', MS='An abnormality of the jaws or teeth affecting the contour of the face. Such abnormality could be acquired or congenital.'),
  'D063171': NtO(MH='Corneal Pachymetry', MS='Measurement of the thickness of the CORNEA.'),
  'D063172': NtO(MH='Genioplasty', MS='A surgical procedure done largely for cosmetic reasons to correct receding chins, chin misalignment, or chin excess.'),
  'D063173': NtO(MH='Retrognathia', MS='A physical misalignment of the upper (maxilla) and lower (mandibular) jaw bones in which either or both recede relative to the frontal plane of the forehead.'),
  'D063175': NtO(MH='Mandibular Reconstruction', MS='The physical restoration of oromandibular defects.'),
  'D063177': NtO(MH='Pulse Wave Analysis', MS='Evaluation of the contour of the PULSE waves which vary in different parts of the circulation and depend on physiological as well as pathophysiological conditions of the individual.'),
  'D063185': NtO(MH='Epigenetic Repression', MS='The turning off of GENETIC TRANSCRIPTION in certain regions of CHROMATIN without changes in the DNA sequence. Typically epigenetic repression is a way that developmental changes are programmed at the cellular level.'),
  'D063186': NtO(MH='Uterine Myomectomy', MS='Surgical removal of a LEIOMYOMA of the UTERUS.'),
  'D063187': NtO(MH='Nursing Stations', MS="An area in a clinic, unit, or ward in a health care facility that serves as the administrative center for nursing care. (from Mosby's Medical Dictionary, 8th ed)"),
  'D063188': NtO(MH='Rana clamitans', MS='A species of the family Ranidae which occurs primarily in the eastern half of the United States and Canada. Two subspecies exist, commonly referred to as green frog (R. c. melanota) and bronze frog (R. c. clamitans).'),
  'D063189': NtO(MH='Symptom Assessment', MS='Evaluation of manifestations of disease.'),
  'D063191': NtO(MH='Odonata', MS='An order of insects comprising three suborders: Anisoptera, Zygoptera, and Anisozygoptera. They consist of dragonflies and damselflies.'),
  'D063192': NtO(MH='Pregnancy, Heterotopic', MS='MULTIPLE PREGNANCY with EMBRYO IMPLANTATION occuring at different locations, involving both an intrauterine site and an extrauterine (ECTOPIC PREGNANCY) site.'),
  'D063193': NtO(MH='Heavy Ion Radiotherapy', MS='The use of a heavy ion particle beam for radiotherapy, such as the HEAVY IONS of CARBON.'),
  'D063194': NtO(MH='Adventitia', MS='The outermost covering of organs, blood vessels, and other such structures in the body.'),
  'D063205': NtO(MH='Pneumorrhachis', MS='Presence of air or gas within the spinal canal cavity (EPIDURAL SPACE; or SUBARACHNOID SPACE). It may result from traumatic injuries, emphysema, infection and other conditions. It can also develop as a complication of various SURGICAL PROCEDURES (e.g., MYELOGRAPHY).'),
  'D063206': NtO(MH='N-Terminal Acetyltransferases', MS='Enzymes that catalyze the transfer of an acetyl group, usually from ACETYL COENZYME A, to the N-terminus of a peptide chain.'),
  'D063208': NtO(MH='Methionyl Aminopeptidases', MS='Aminopeptidases that remove METHIONINE from the amino-terminus of a peptide chain, such as the initiator METHIONINE found on nascent peptide chains.'),
  'D063209': NtO(MH='N-Terminal Acetyltransferase A', MS='An N-terminal acetyltransferase subtype that consists of the Naa10p catalytic subunit and the Naa15p auxiliary subunit. The structure of this enzyme is conserved between lower and higher eukaryotes. It has specificity for N-terminal SERINE; ALANINE; THREONINE; GLYCINE; VALINE; and CYSTINE residues and acts on nascent peptide chains after the removal of the initiator METHIONINE by METHIONYL AMINOPEPTIDASES.'),
  'D063210': NtO(MH='N-Terminal Acetyltransferase B', MS='An N-terminal acetyltransferase subtype that consists of the Naa20p catalytic subunit and the Naa25p auxiliary subunit. The structure of this enzyme is conserved between YEASTS and HUMAN. It has specificity for the N-terminal METHIONINE of peptides where the next amino acid in the chain is either ASPARTATE; GLUTAMATE; ASPARAGINE; or GLUTAMINE.'),
  'D063211': NtO(MH='N-Terminal Acetyltransferase C', MS='An N-terminal acetyltransferase subtype that consists of the Naa30p catalytic subunit, and the Naa35p and Naa38p auxiliary subunits. It has specificity for the N-terminal METHIONINE of peptides where the next amino acid in the chain is either LEUCINE; PHENYALANINE; ISOLEUCINE; or TRYPTOPHANE.'),
  'D063212': NtO(MH='N-Terminal Acetyltransferase D', MS='An N-terminal acetyltransferase subtype that consists of the Naa40p catalytic subunit. It has specificity for N-termini of HISTONE H2A and HISTONE H4.'),
  'D063213': NtO(MH='N-Terminal Acetyltransferase E', MS='An N-terminal acetyltransferase subtype that consists of the Naa50p catalytic subunit, and the Naa10p and Naa15p auxiliary subunits. It has specificity for the N-terminal METHIONINE of peptides where the next amino acid in the chain is hydrophobic.'),
  'D063214': NtO(MH='N-Terminal Acetyltransferase F', MS='An N-terminal acetyltransferase subtype that consists of the Naa60p catalytic subunit. It is found in higher eukayotes and displays a substrate specificity for the N-terminal METHIONINE of peptides where the next amino acid in the chain is either LEUCINE; LYSINE; PHENYALANINE; ISOLEUCINE; or TRYPTOPHANE.'),
  'D063225': NtO(MH='Organically Modified Ceramics', MS='Organic-inorganic hybrid polymers developed primarily for DENTAL RESTORATION. They typically contain a defined mixture of ORGANOSILICON COMPOUNDS; CERAMICS; and organic POLYMERS.'),
  'D063226': NtO(MH='Central Pattern Generators', MS='Networks of nerve cells that control the firing patterns of MOTOR NEURONS to produce rhythmic movements such as MASTICATION; WALKING; SWIMMING; RESPIRATION; and PERISTALSIS.'),
  'D063245': NtO(MH='Plant Development', MS='Processes orchestrated or driven by a plethora of genes, plant hormones, and inherent biological timing mechanisms facilitated by secondary molecules, which result in the systematic transformation of plants and plant parts, from one stage of maturity to another.'),
  'D063246': NtO(MH='Organogenesis, Plant', MS='Transformative processes which occur in cells and tissues to produce differentiated plant organs.'),
  'D063265': NtO(MH='Sialic Acid Binding Immunoglobulin-like Lectins', MS='A family of SIALIC ACID binding proteins found in vertebrate species. They are transmembrane proteins which act as cell surface receptors for a variety of sialylated GLYCOCONJUGATES. While a subset of siglec protein subtypes are evolutionarily conserved between mammalian species, there are many others that are species specific.'),
  'D063267': NtO(MH='Sialic Acid Binding Ig-like Lectin 1', MS='A sialic acid binding lectin that was originally identified as an adhesion molecule for inflammatory MACROPHAGES and activated MONOCYTES. This protein is the largest known siglec subtype and contains 16 immunoglobulin C2-set domains. It plays a role in cell to cell interactions and interactions with bacteria.'),
  'D063268': NtO(MH='Sialic Acid Binding Ig-like Lectin 3', MS='A 67-kDa sialic acid binding lectin that is specific for MYELOID CELLS and MONOCYTE-MACROPHAGE PRECURSOR CELLS. This protein is the smallest siglec subtype and contains a single immunoglobulin C2-set domain. It may play a role in intracellular signaling via its interaction with SHP-1 PROTEIN-TYROSINE PHOSPHATASE and SHP-2 PROTEIN-TYROSINE PHOSPHATASE.'),
  'D063269': NtO(MH='Mitochondrial Turnover', MS='The cellular processes involved in adjustments to the MITOCHONDRIAL VOLUME, content, and activity, that depend on the energy demands of the cell.'),
  'D063285': NtO(MH='Fibrin Clot Lysis Time', MS='A measurement of the time needed for FIBRINOLYSIS to occur.'),
  'D063287': NtO(MH='Brown Recluse Spider', MS='A spider of the genus Loxosceles, found in the midwestern and other parts of the United States, which carries a hemolytic venom that produces local necrosis or ulceration.'),
  'D063305': NtO(MH='Organ Dysfunction Scores', MS='Ratings that express in numerical values the degree of impairment or abnormality in the function of specific organs.'),
  'D063306': NtO(MH='Mitophagy', MS='Proteolytic breakdown of the MITOCHONDRIA via autophagy.'),
  'D063307': NtO(MH='AU Rich Elements', MS="RNA sequences composed of ADENINE NUCLEOTIDES and URACIL NUCLEOTIDES, that are located in the 3'UNTRANSLATED REGIONS of MESSENGER RNA molecules that are rapidly degraded. They are also known as AREs."),
  'D063308': NtO(MH='Myelin-Oligodendrocyte Glycoprotein', MS='A transmembrane protein present in the MYELIN SHEATH of the CENTRAL NERVOUS SYSTEM. It is one of the main autoantigens implicated in the pathogenesis of MULTIPLE SCLEROSIS.'),
  'D063309': NtO(MH="2',3'-Cyclic Nucleotide 3'-Phosphodiesterase", MS="A phosphodiesterase that specifically cleaves the 3'-phosphate linkage of 2',3'-cyclic nucleotides. It is found at high level in the cytoplasm of cells that form the MYELIN SHEATH."),
  'D063325': NtO(MH='Tiapride Hydrochloride', MS='A benzamide derivative that is used as a dopamine antagonist.'),
  'D063326': NtO(MH='Exosome Multienzyme Ribonuclease Complex', MS='An intracellular ribonucleolytic protein complex that participates in POSTRANSCRIPTIONAL RNA PROCESSING and RNA DEGRADATION.'),
  'D063345': NtO(MH='Oligodendrocyte-Myelin Glycoprotein', MS='A glycosylated extracellular myelin protein found on the MYELIN SHEATH of the CENTRAL NERVOUS SYSTEM. It is linked to the cell surface via a GLYCOSYLPHOSPHATIDYLINOSITOL LINKAGE.'),
  'D063346': NtO(MH='Singing', MS='Modulation of human voice to produce sounds augmented by musical tonality and rhythm.'),
  'D063347': NtO(MH='Native Polyacrylamide Gel Electrophoresis', MS='Polyacrylamide gel electrophoresis under conditions in which the components, such as PROTEINS, being separated can remain in their naturally folded state.'),
  'D063365': NtO(MH='Genocide', MS='The deliberate annihilation of a national, ethnic, or religious group, in part or in whole.'),
  'D063366': NtO(MH='Estuaries', MS='A partially enclosed body of water, and its surrounding coastal habitats, where saltwater from the ocean mixes with fresh water from rivers or streams. The resulting mixture of seawater and fresh water is called brackish water and its salinity can range from 0.5 to 35 ppt. (accessed http://oceanservice.noaa.gov/education/kits/estuaries/estuaries01_whatis.html)'),
  'D063367': NtO(MH='Virtual Reality Exposure Therapy', MS='Treatment technique in a virtual environment which allows the participant to experience a sense of presence in an immersive, computer-generated, three-dimensional, interactive environment that minimizes avoidance behavior and facilitates emotional involvement. (from Curr Psychiatry Rep (2010) 12:298)'),
  'D063368': NtO(MH='Balkan Peninsula', MS='A peninsula in Southeast EUROPE between the Adriatic and Ionian seas on the West and Aegean and Black Seas on the East. (from www.merriam-webster.com/dictionary/balkan%20peninsula)'),
  'D063369': NtO(MH='Knowledge Discovery', MS='A process that searches for hidden and important connections among information or results.'),
  'D063371': NtO(MH='Myringosclerosis', MS='The formation of dense connective tissue in the TYMPANIC MEMBRANE that does not necessarily cause or lead to loss of hearing.'),
  'D063372': NtO(MH='Physical Therapist Assistants', MS='Persons who, under the supervision of licensed PHYSICAL THERAPISTS, provide patient treatment using various PHYSICAL THERAPY TECHNIQUES.'),
  'D063373': NtO(MH='Hand Hygiene', MS='Practices involved in preventing the transmission of diseases by hand.'),
  'D063385': NtO(MH='Cannabinoid Receptor Modulators', MS='Compounds that interact with and modulate the activity of CANNABINOID RECEPTORS.'),
  'D063386': NtO(MH='Cannabinoid Receptor Agonists', MS='Compounds that interact with and stimulate the activity of CANNABINOID RECEPTORS.'),
  'D063387': NtO(MH='Cannabinoid Receptor Antagonists', MS='Compounds that inhibit or block the activity of CANNABINOID RECEPTORS.'),
  'D063388': NtO(MH='Endocannabinoids', MS='Fatty acid derivatives that have specificity for CANNABINOID RECEPTORS. They are structurally distinct from CANNABINOIDS and were originally discovered as a group of endogenous CANNABINOID RECEPTOR AGONISTS.'),
  'D063405': NtO(MH='Anatomic Variation', MS='Peculiarities associated with the internal structure, form, topology, or architecture of organisms that distinguishes them from others of the same species or group.'),
  'D063425': NtO(MH='Binge Drinking', MS='Drinking an excessive amount of ALCOHOLIC BEVERAGES in a short period of time.'),
  'D063426': NtO(MH='Human Migration', MS='Dispersion or dislocation of people from their country of origin, either voluntarily or involuntarily.'),
  'D063427': NtO(MH='Food Quality', MS='Ratings of the characteristics of food including flavor, appearance, nutritional content, and the amount of microbial and chemical contamination.'),
  'D063465': NtO(MH='Skin Cream', MS='A water-soluble medicinal preparation applied to the skin.'),
  'D063466': NtO(MH='Respiratory Aspiration of Gastric Contents', MS='Inhaling refluxed gastric or duodenal contents.'),
  'D063485': NtO(MH='Acanthocheilonemiasis', MS='A tropical infectious disease found mainly in Africa that is caused by the filarial parasite ACANTHOCHEILONEMA. Symptoms include skin rashes, abdominal, chest, muscle, and joint pains, neurologic disorders, skin lumps, and elevated levels of white blood cells. The parasite is transmitted through the bite of small flies.'),
  'D063486': NtO(MH='Acanthocheilonema', MS='A common genus of tropical filarial worms parasitic in humans and monkeys.'),
  'D063487': NtO(MH='Prescription Drug Misuse', MS='Improper use of prescription drugs or medications outside the intended purpose, scope, or guidelines for use.'),
  'D063505': NtO(MH='Racism', MS='Differential treatment or unequal access to opportunities, based on group membership such as origin or ethnicity.'),
  'D063506': NtO(MH='Ageism', MS='Biased behaviors or attitudes, differential treatment, unequal access to social participation or opportunities based on age.'),
  'D063507': NtO(MH='Sexism', MS='Prejudice or discrimination based on gender or behavior or attitudes that foster stereotyped social roles based on gender.'),
  'D063508': NtO(MH='Social Discrimination', MS='Group behavior toward others by virtue of their group membership.'),
  'D063509': NtO(MH='Homophobia', MS='Differential treatment or unequal access to opportunities or services based on perceived homosexual preference or orientation.'),
  'D063545': NtO(MH='Personal Narratives as Topic', MS='Works about accounts of individual experience in relation to a particular field or of participation in related activities.'),
  'D063546': NtO(MH='Apicoplasts', MS='Non-photosynthetic plastids derived from RED ALGAE endosymbionts. They are found in species of the phylum APICOMPLEXA including PLASMODIUM MALARAIAE.'),
  'D063566': NtO(MH='Biodegradable Plastics', MS='Organic polymeric materials which can be broken down by naturally occurring processes. This includes plastics created from bio-based or petrochemical-based materials.'),
  'D063586': NtO(MH='Social Validity, Research', MS='Evaluation of the degree of acceptance for the immediate variables associated with a procedure or program designed to change behavior. This includes the social significance of the goals of treatment, the social appropriateness of the treatment procedures, and the social importance of the effects of treatments.'),
  'D063588': NtO(MH='National Longitudinal Study of Adolescent Health', MS='Longitudinal study of a nationally representative sample of adolescents in grades 7-12 in the United States during the 1994-95 school year. The Add Health cohort has been followed into young adulthood. (from http://www.cpc.unc.edu/projects/addhealth accessed 08/2012)'),
  'D063607': NtO(MH='Dander', MS='Microscopic particles comprising dead skin, dried salivary proteins, hair, SEBUM and microorganisms, shed by animals which causes allergic reactions in atopic persons.'),
  'D063646': NtO(MH='Carcinogenesis', MS='The origin, production or development of cancer through genotypic and phenotypic changes which upset the normal balance between cell proliferation and cell death. Carcinogenesis generally requires a constellation of steps, which may occur quickly or over a period of many years.'),
  'D063647': NtO(MH='Fetal Alcohol Spectrum Disorders', MS='An umbrella term used to describe a pattern of disabilities and abnormalities that result from fetal exposure to ETHANOL during pregnancy. It encompasses a phenotypic range that can vary greatly between individuals, but reliably includes one or more of the following: characteristic facial dysmorphism, FETAL GROWTH RETARDATION, central nervous system abnormalities, cognitive and/or behavioral dysfunction, BIRTH DEFECTS. The level of maternal alcohol consumption does not necessarily correlate directly with disease severity.'),
  'D063726': NtO(MH='Delusional Parasitosis', MS='A delusional disorder of belief in infestation by insects or other parasites. This FORMICATION is typically accompanied by dermatological manifestation such as PRURITUS that may lead to self-mutilation in order to remove the perceived parasites. It can be either primary or secondary to a somatic or psychiatric condition.'),
  'D063728': NtO(MH='Secondary Care Centers', MS='A healthcare facility equipped to provide all but the most specialized forms of care, surgery, and diagnostic techniques.'),
  'D063729': NtO(MH='Sublingual Immunotherapy', MS='A method of treating an ALLERGY by administering ALLERGENS, in liquid formulation or tablets, to the ORAL MUCOSA under the tongue.'),
  'D063730': NtO(MH='Rickets, Hypophosphatemic', MS='A disorder characterized by HYPOPHOSPHATEMIA; RICKETS; OSTEOMALACIA; resulting from lack of phosphate reabsorption by the kidneys and possible defects in vitamin D metabolism.'),
  'D063731': NtO(MH='Mobile Applications', MS='Computer programs or software installed on mobile electronic devices which support a wide range of functions and uses which include television, telephone, video, music, word processing, and Internet service.'),
  'D063746': NtO(MH='Methanocaldococcaceae', MS='A family of anaerobic METHANOCOCCALES whose organisms are thermophilic methanogens that produce methane from hydrogen and simple carbon sources such as carbon dioxide.'),
  'D063747': NtO(MH='Methanocaldococcus', MS='A genus of obligate anaerobic METHANOCALDOCOCCACEAE whose organisms are non-motile despite possessing long thin flagella. These methanogens are found in deep-sea vent and other hydrothermal environments.'),
  'D063748': NtO(MH='Bland White Garland Syndrome', MS='A congenital coronary vessel anomaly in which the left main CORONARY ARTERY originates from the PULMONARY ARTERY instead of from AORTA. The congenital heart defect typically results in coronary artery FISTULA; LEFT-SIDED HEART FAILURE and MITRAL VALVE INSUFFICIENCY during the first months of life.'),
  'D063766': NtO(MH='Pediatric Obesity', MS='BODY MASS INDEX in children (ages 2-12) and in adolescents (ages 13-18) that is grossly above the recommended cut-off for a specific age and sex. For infants less than 2 years of age, obesity is determined based on standard weight-for-length percentile measures.'),
  'D063806': NtO(MH='Myalgia', MS='Painful sensation in the muscles.'),
  'D063807': NtO(MH='Dandruff', MS='Excessive shedding of dry scaly material from the scalp in humans.'),
  'D063809': NtO(MH='Satellite Imagery', MS='Composition of images of EARTH or other planets from data collected during SPACE FLIGHT by remote sensing instruments onboard SPACECRAFT. The satellite sensor systems measure and record absorbed, emitted, or reflected energy across the spectra, as well as global position and time.'),
  'D063826': NtO(MH='Kosovo', MS='Independence from SERBIA was declared on February 17, 2008.'),
  'D063847': NtO(MH='Mean Platelet Volume', MS='A measure of the size of PLATELETS.'),
  'D063866': NtO(MH='Medical Identity Theft', MS='Unauthorized access to personal information to obtain medical care or drugs, or to submit fraudulent billing for services. (from https://oig.hhs.gov/fraud/medical-id-theft/index.asp)'),
  'D063867': NtO(MH='Identity Theft', MS='All types of crime in which personal data is obtained wrongfully, typically for economic gain. (from www.justice.gov/criminal/fraud/websites/idtheft.html)'),
  'D063868': NtO(MH='Patient Outcome Assessment', MS='Patient-generated evaluation data of treatment outcome or interaction in healthcare environment.'),
  'D063886': NtO(MH='Patient Discharge Summaries', MS="Summaries that serve as the primary documents communicating a patient's care plan to the post-hospital care team."),
  'D063906': NtO(MH='Prisoners of War', MS='Members held prisoner of the armed forces, members of militias of such armed forces, members of other militias, members of other volunteer corps, and organized resistance movements. (Geneva Convention of 1929 and 1949)'),
  'D063926': NtO(MH='Drug Hypersensitivity Syndrome', MS='Severe drug eruption characterized by high fever, erythematous rash and inflammation of internal organ(s).'),
  'D063927': NtO(MH='Lujo virus', MS='A species of ARENAVIRUS, OLD WORLD that is the etiologic agent of acute febrile illness with hemorrhagic manifestations in Africa.'),
  'D063928': NtO(MH='Ependymoglial Cells', MS='The macroglial cells of EPENDYMA. They are characterized by bipolar cell body shape and processes that contact BASAL LAMINA around blood vessels and/or the PIA MATER and the CEREBRAL VENTRICLES.'),
  'D063947': NtO(MH='Enslavement', MS='The status or condition of a person over whom any or all of the powers attaching to the right of ownership are exercised. The slave trade is all acts involved in the capture, acquisition or disposal of persons with intent to reduce them to enslavement; all acts involved in the acquisition of enslaved persons with a view to selling or exchanging them; all acts of disposal by sale or exchange of persons acquired with a view to being sold or exchanged, and, in general, every act of trade or transport of slaves. (from http://www.ohchr.org/Documents/Publications/slaveryen.pdf accessed 10/18/2012)'),
  'D063948': NtO(MH='Enslaved Persons', MS='Persons involuntarily controlled by another and exploited for forced or compulsory work or service.'),
  'D063967': NtO(MH='Material Safety Data Sheets', MS='Information or data used to ensure the safe handling and disposal of substances in the workplace. Such information includes physical properties (i.e. melting, boiling, flashing points), as well as data on toxicity, health effects, reactivity, storage, disposal, first-aid, protective equipment, and spill-handling procedures.'),
  'D063986': NtO(MH='Vascularized Composite Allotransplantation', MS='Transference of multiple tissues, such as muscle, bone, nerve, and skin, as a functional unit for reconstructive purposes. Blood supply to the transplanted tissues is maintained throughout the transplantation procedure with minimal ischemia. Maintenance of vascularity in the graft promotes its viability, function, and survival in the recipient.'),
  'D063987': NtO(MH='Hand Transplantation', MS='The transference of a complete HAND, as a composite of many tissue types, from one individual to another.'),
  'D063988': NtO(MH='3-Hydroxyacyl-CoA Dehydrogenase', MS='An NAD-dependent 3-hydroxyacyl CoA dehydrogenase that has broad specificity with regards to the acyl chain length of the substrate.'),
  'D063989': NtO(MH='Long-Chain-3-Hydroxyacyl-CoA Dehydrogenase', MS='An NAD-dependent 3-hydroxyacyl CoA dehydrogenase that has specificity for acyl chains containing 8 and 10 carbons.'),
  'D063990': NtO(MH='Gene Ontology', MS='Sets of structured vocabularies used for describing and categorizing genes, and gene products by their molecular function, involvement in biological processes, and cellular location. These vocabularies and their associations to genes and gene products (Gene Ontology annotations) are generated and curated by the Gene Ontology Consortium.'),
  'D063991': NtO(MH='Mitochondrial Trifunctional Protein', MS='A mitochondrial protein consisting of four alpha-subunits and four beta-subunits. It contains enoyl-CoA hydratase, long-chain-3-hydroxyacyl-CoA dehydrogenase, and acetyl-CoA C-acyltransferase activities and plays an important role in the metabolism of long chain FATTY ACIDS.'),
  'D063992': NtO(MH='Mitochondrial Trifunctional Protein, alpha Subunit', MS='The alpha subunit of mitochondrial trifunctional protein. It contains both enoyl-CoA hydratase activity (EC 4.2.1.17) and long-chain-3-hydroxyacyl-CoA dehydrogenase activity (EC 1.1.1.211). There are four of these alpha subunits in each mitochondrial trifunctional protein molecule.'),
  'D063993': NtO(MH='Mitochondrial Trifunctional Protein, beta Subunit', MS='The beta subunit of mitochondrial trifunctional protein that contains acetyl-CoA C-acyltransferase activity. There are four of these beta subunits in each trifunctional protein complex.'),
  'D063994': NtO(MH='Peroxisomal Bifunctional Enzyme', MS='A monomeric protein found in liver peroxisomes that contains two enzymatically active domains; an enoyl-CoA hydratase/3,2-trans-enoyl-CoA isomerase domain, and an (S)-3-hydroxyacyl-CoA dehydrogenase domain. The enzyme is stereospecific with regards to how cis and trans double bonds are metabolized. It is complemented by PEROXISOMAL MULTIFUNCTIONAL PROTEIN-2, which has the opposite stereospecificity.'),
  'D063996': NtO(MH='Warm-Up Exercise', MS='Physical activities done to prepare the body for more intense physical activities.'),
  'D064006': NtO(MH='Dodecenoyl-CoA Isomerase', MS='A carbon-carbon double bond isomerase that catalyzes the movement double bond from C3 to C2 of an unsaturated acyl-CoA. The enzyme plays a key role in allowing acyl-CoA substrates to re-enter the beta-oxidation pathway.'),
  'D064007': NtO(MH='Ataxia Telangiectasia Mutated Proteins', MS='A group of PROTEIN-SERINE-THREONINE KINASES which activate critical signaling cascades in double strand breaks, APOPTOSIS, and GENOTOXIC STRESS such as ionizing ultraviolet A light, thereby acting as a DNA damage sensor. These proteins play a role in a wide range of signaling mechanisms in cell cycle control.'),
  'D064008': NtO(MH='Enoyl-CoA Hydratase 2', MS='A PEROXISOME-specific enzyme that catalyzes the hydration step of the beta-oxidation pathway.'),
  'D064026': NtO(MH='Calbindins', MS='Calcium-binding proteins that are found in DISTAL KIDNEY TUBULES, INTESTINES, BRAIN, and other tissues where they bind, buffer and transport cytoplasmic calcium. Calbindins possess a variable number of EF-HAND MOTIFS which contain calcium-binding sites. Some isoforms are regulated by VITAMIN D.'),
  'D064028': NtO(MH='Tracheophyta', MS='A subset of plants bearing a vascular system. Examples are ferns and seed-bearing species.'),
  'D064029': NtO(MH='Peroxisomal Multifunctional Protein-2', MS='A dimeric protein found in liver peroxisomes that plays an important role in FATTY ACID metabolism and steroid metabolism. The dimer is formed by cleavage of a single protein precursor and contains an enoyl-CoA hydratase-2 domain and a second domain that displays (S)-3-hydroxyacyl-CoA dehydrogenase and 17-beta-estradiol dehydrogenase activities. The enzyme is stereospecific with regards to arrangement of the substrate double bonds and position of the 3-hydroxy group of the reaction intermediate. It is complemented by PEROXISOMAL BIFUNCTIONAL ENZYME, which has the opposite reaction stereospecificity.'),
  'D064030': NtO(MH='S100 Calcium Binding Protein G', MS='A calbindin protein found in many mammalian tissues, including the UTERUS, PLACENTA, BONE, PITUITARY GLAND, and KIDNEYS. In intestinal ENTEROCYTES it mediates intracellular calcium transport from apical to basolateral membranes via calcium binding at two EF-HAND MOTIFS. Expression is regulated in some tissues by VITAMIN D.'),
  'D064032': NtO(MH='Calbindin 2', MS='A calbindin protein that is differentially expressed in distinct populations of NEURONS throughout the vertebrate and invertebrate NERVOUS SYSTEM, and modulates intrinsic neuronal excitability and influences LONG-TERM POTENTIATION. It is also found in LUNG, TESTIS, OVARY, KIDNEY, and BREAST, and is expressed in many tumor types found in these tissues. It is often used as an immunohistochemical marker for MESOTHELIOMA.'),
  'D064046': NtO(MH='Secretagogins', MS='Secretagogins are EF HAND MOTIF-containing calcium-binding proteins that are involved in early neuronal migration and neurogenesis. They are also present in many adult organs and in brain and endocrine neoplasms.'),
  'D064047': NtO(MH='Spindle Poles', MS='The sites in a dividing cell where the minus ends of the spindle MICROTUBULES assemble and where the separated sister CHROMATIDS converge.'),
  'D064048': NtO(MH='Spindle Pole Bodies', MS='Microtubule organizing centers of FUNGI and DIATOMS. They are embedded in the NUCLEAR ENVELOPE and evolve into the spindle apparatus during CELL DIVISION.'),
  'D064067': NtO(MH='Rho Guanine Nucleotide Exchange Factors', MS='Signaling proteins which function as master molecular switches by activating Rho GTPases through conversion of guanine nucleotides. Rho GTPases in turn control many aspects of cell behavior through the regulation of multiple downstream signal transduction pathways.'),
  'D064068': NtO(MH='Collagenous Sprue', MS='A malabsorption syndrome characterized by collagenous mucosal lesions of the SMALL INTESTINE, atrophy of MICROVILLI, severe malabsorption, diarrhea, and MALNUTRITION often refractory to a gluten-free diet.'),
  'D064086': NtO(MH='Medical Marijuana', MS='Product of the CANNABIS plant, CANNABINOIDS, or synthetic derivatives thereof, used in the treatment of a wide range of clinical symptoms.'),
  'D064087': NtO(MH='Head Impulse Test', MS='Identification of SACCADES during a rapid head rotation to assess VESTIBULOOCULAR REFLEX.'),
  'D064088': NtO(MH='Hazard Analysis and Critical Control Points', MS='A system of safety management (abbreviated HACCP) applied mainly to the food industry. It involves the analysis and control of biological, chemical, and physical hazards, from raw material production, procurement and handling, to manufacturing, distribution and consumption of finished products.'),
  'D064089': NtO(MH='Allelopathy', MS='The process by which organisms release chemical compounds known as ALLELOCHEMICALS which influence the physiology, growth, survival, colonization, and reproductive activities of other species usually located nearby.'),
  'D064090': NtO(MH='Intraocular Lymphoma', MS='A form of malignant cancer which occurs within the eyeball.'),
  'D064091': NtO(MH='Waste Disposal Facilities', MS='Sites that receive and store WASTE PRODUCTS. Some facilities also sort, process, and recycle specific waste products.'),
  'D064092': NtO(MH='Calbindin 1', MS='A calcium-binding protein that mediates calcium HOMEOSTASIS in KIDNEYS, BRAIN, and other tissues. It is found in well-defined populations of NEURONS and is involved in CALCIUM SIGNALING and NEURONAL PLASTICITY. It is regulated in some tissues by VITAMIN D.'),
  'D064094': NtO(MH='Interleukin-27', MS='A heterodimeric cytokine composed of the IL-27p28 and EBI3 subunits. It is produced primarily by antigen-presenting cells after stimulation by microbial products or inflammatory mediators.'),
  'D064096': NtO(MH='Aurora Kinase A', MS='An aurora kinase that localizes to the CENTROSOME during MITOSIS and is involved in centrosome regulation and formation of the MITOTIC SPINDLE. Aurora A overexpression in many malignant tumor types suggests that it may be directly involved in NEOPLASTIC CELL TRANSFORMATION.'),
  'D064098': NtO(MH='Esomeprazole', MS='The S-isomer of omeprazole.'),
  'D064106': NtO(MH='Human Coprophagia', MS='A rare aberrant human behavior involving consumption of excrement.'),
  'D064107': NtO(MH='Aurora Kinase B', MS='An aurora kinase that is a component of the chromosomal passenger protein complex and is involved in the regulation of MITOSIS. It mediates proper CHROMOSOME SEGREGATION and contractile ring function during CYTOKINESIS.'),
  'D064108': NtO(MH='Aurora Kinase C', MS='Aurora kinase C is a chromosomal passenger protein that interacts with aurora kinase B in the regulation of MITOSIS. It is found primarily in GERM CELLS in the TESTIS, and may mediate CHROMOSOME SEGREGATION during SPERMATOGENESIS.'),
  'D064110': NtO(MH='Minichromosome Maintenance Proteins', MS='A family of proteins that were originally identified in SACCHAROMYCES CEREVISIAE as being essential for maintaining the structure of minichromosomes00. They form into a protein complex that has helicase activity and is involved in a variety of DNA-related functions including replication elongation, RNA transcription, chromatin remodeling, and genome stability.'),
  'D064111': NtO(MH='Minichromosome Maintenance Complex Component 2', MS='A minichromosome maintenance protein that is a key component of the six member MCM protein complex. It contains a NUCLEAR LOCALIZATION SIGNAL which may provide targeting of the protein complex and an extended N-terminus which is rich in SERINE residues.'),
  'D064112': NtO(MH='Clustered Regularly Interspaced Short Palindromic Repeats', MS='Repetitive nucleic acid sequences that are principal components of the archaeal and bacterial CRISPR-CAS SYSTEMS, which function as adaptive antiviral defense systems.'),
  'D064113': NtO(MH='CRISPR-Cas Systems', MS='Adaptive antiviral defense mechanisms, in archaea and bacteria, based on DNA repeat arrays called CLUSTERED REGULARLY INTERSPACED SHORT PALINDROMIC REPEATS (CRISPR elements) that function in conjunction with CRISPR-ASSOCIATED PROTEINS (Cas proteins). Several types have been distinguished, including Type I, Type II, and Type III, based on signature motifs of CRISPR-ASSOCIATED PROTEINS.'),
  'D064126': NtO(MH='Minichromosome Maintenance Complex Component 3', MS='A minichromosome maintenance protein that is a key component of the six member MCM protein complex. It contains a NUCLEAR LOCALIZATION SIGNAL, which provide targeting of the protein complex. In addition, acetylation of this protein may play a role in regulating of DNA replication and cell cycle progression.'),
  'D064127': NtO(MH='Aurora Kinases', MS='A family of highly conserved serine-threonine kinases that are involved in the regulation of MITOSIS. They are involved in many aspects of cell division, including centrosome duplication, SPINDLE APPARATUS formation, chromosome alignment, attachment to the spindle, checkpoint activation, and CYTOKINESIS.'),
  'D064128': NtO(MH='Nodding Syndrome', MS='Unexplained neurologic condition characterized by episodes of atonic seizures, convulsions or staring spells with further cognitive decline.'),
  'D064129': NtO(MH='Prostatic Neoplasms, Castration-Resistant', MS='Tumors or cancer of the PROSTATE which can grow in the presence of low or residual amount of androgen hormones such as TESTOSTERONE.'),
  'D064130': NtO(MH='CRISPR-Associated Proteins', MS='Protein components of the CRISPR-CAS SYSTEMS for anti-viral defense in ARCHAEA and BACTERIA. These are proteins that carry out a variety of functions during the creation and expansion of the CRISPR ARRAYS, the capture of new CRISPR SPACERS, biogenesis of SMALL INTERFERING RNA (CRISPR or crRNAs), and the targeting and silencing of invading viruses and plasmids. They include DNA HELICASES; RNA-BINDING PROTEINS; ENDONUCLEASES; and RNA and DNA POLYMERASES.'),
  'D064131': NtO(MH='Back Muscles', MS='Musculature of the BACK.'),
  'D064132': NtO(MH='Sports Nutritional Sciences', MS='The study of NUTRITION PROCESSES during EXERCISE and ATHLETIC PERFORMANCE as well as specific NUTRITIONAL REQUIREMENTS of ATHLETES and the relationship between NUTRITIONAL STATUS and NUTRITION DISORDERS in athletes.'),
  'D064133': NtO(MH='Sports Nutritional Physiological Phenomena', MS='Nutritional physiology related to EXERCISE or ATHLETIC PERFORMANCE.'),
  'D064146': NtO(MH='Chemotherapy-Induced Febrile Neutropenia', MS='FEVER accompanied by a significant reduction in NEUTROPHIL count associated with CHEMOTHERAPY.'),
  'D064147': NtO(MH='Febrile Neutropenia', MS='Fever accompanied by a significant reduction in the number of NEUTROPHILS.'),
  'D064148': NtO(MH='Minichromosome Maintenance Complex Component 4', MS='A minichromosome maintenance protein that is a key component of the six member MCM protein complex. It is also found in tightly-bound trimeric complex with MINICHROMOSOME MAINTENANCE COMPLEX COMPONENT 6 and MINICHROMOSOME MAINTENANCE COMPLEX COMPONENT 7.'),
  'D064149': NtO(MH='Influenza Pandemic, 1918-1919', MS='The influenza outbreaks of 1918 to 1919 also known as Spanish flu pandemic. First reported in Haskell County in Kansas in March of 1918 the disease spread throughout the world and may have killed as many as 25 million people.'),
  'D064150': NtO(MH='Minichromosome Maintenance Complex Component 5', MS='A minichromosome maintenance protein that is a key component of the six member MCM protein complex. In addition, interaction of this protein with cyclin A results in its recruitment to CENTROSOMES where it may play a role in controlling centrosome reduplication.'),
  'D064151': NtO(MH='Minichromosome Maintenance Complex Component 6', MS='A minichromosome maintenance protein that is a key component of the six member MCM protein complex. It is also found in tightly-bound trimeric complex with MINICHROMOSOME MAINTENANCE COMPLEX COMPONENT 4 and MINICHROMOSOME MAINTENANCE COMPLEX COMPONENT 7.'),
  'D064166': NtO(MH='Vaccine Potency', MS='The relationship between an elicited ADAPTIVE IMMUNE RESPONSE and the dose of the vaccine administered.'),
  'D064167': NtO(MH='Recommended Dietary Allowances', MS='The amounts of various substances in the diet recommended by governmental guidelines as needed to sustain healthy life.'),
  'D064168': NtO(MH='Minichromosome Maintenance Complex Component 7', MS='A minichromosome maintenance protein that is a key component of the six member MCM protein complex. It is also found in tightly-bound trimeric complex with MINICHROMOSOME MAINTENANCE COMPLEX COMPONENT 4 and MINICHROMOSOME MAINTENANCE COMPLEX COMPONENT 6.'),
  'D064169': NtO(MH='Intermediate Back Muscles', MS='Two muscles, the serratus posterior superior and serratus posterior inferior, located in between the deep and the superficial back muscle layers. Their function is to control THORAX movement.'),
  'D064170': NtO(MH='Paraspinal Muscles', MS='Deep muscles in the BACK whose function is to extend and rotate the SPINE and maintain POSTURE. It consists splenius, semispinalis, multifidus, rotatores, interspinales, intertransversarii and sacrospinalis.'),
  'D064171': NtO(MH='Superficial Back Muscles', MS='The top layer of the back muscles whose function is to move the SCAPULA. This group of muscles consists of the trapezius, latissimus dorsi, rhomboid major, rhomboid minor and levator scapulae.'),
  'D064172': NtO(MH='Pharmacy Residencies', MS='Advanced programs of training to meet certain professional requirements in the practice of compounding and dispensing medicinal preparations.'),
  'D064173': NtO(MH='Anaphase-Promoting Complex-Cyclosome', MS='An E3 ubiquitin ligase primarily involved in regulation of the metaphase-to-anaphase transition during MITOSIS through ubiquitination of specific CELL CYCLE PROTEINS. Enzyme activity is tightly regulated through subunits and cofactors, which modulate activation, inhibition, and substrate specificity. The anaphase-promoting complex, or APC-C, is also involved in tissue differentiation in the PLACENTA, CRYSTALLINE LENS, and SKELETAL MUSCLE, and in regulation of postmitotic NEURONAL PLASTICITY and excitability.'),
  'D064174': NtO(MH='Cephalochordata', MS='A subphylum of chordates intermediate between the invertebrates and the true vertebrates. It includes the LANCELETS. Its members are characterized by a NOTOCHORD which extends into the adult stage head.'),
  'D064175': NtO(MH='Lancelets', MS='Small fish-like marine creatures often used in phylogenetic comparative studies of CHORDATES.'),
  'D064186': NtO(MH='Prenatal Education', MS='Training for pregnant women and parents-to-be to prepare for CHILDBIRTH; CHILD CARE; and parenthood.'),
  'D064187': NtO(MH='Minichromosome Maintenance Complex Component 8', MS='A minichromosome maintenance protein that forms a hexameric complex with MINICHROMSOME MAINTENANCE COMPLEX COMPONENT 9. The MCM8-MCM9 helicase complex is involved in HOMOLOGOUS RECOMBINATION REPAIR following the formation of DNA interstrand cross-links.'),
  'D064188': NtO(MH='Minichromosome Maintenance Complex Component 9', MS='A minichromosome maintenance protein that forms a hexameric complex with MINICHROMSOME MAINTENANCE COMPLEX COMPONENT 8. The MCM8-MCM9 helicase complex is involved in HOMOLOGOUS RECOMBINATION REPAIR following the formation of DNA interstrand cross-links.'),
  'D064189': NtO(MH='Apc1 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='The largest subunit of the anaphase-promoting complex. It acts primarily as a scaffold for the proper organization and arrangement of subunits. The C-terminal region of Apc1 contains a series of tandem amino acid repeats that are also seen in the 26S proteasome regulatory particle, and may assist with forming and stabilizing protein-protein interactions.'),
  'D064190': NtO(MH='Apc2 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='Together with the Apc11 subunit, forms the catalytic core of the E3 ubiquitin ligase anaphase-promoting complex (APC-C). Its N-terminus has cullin domains which associate with the RING FINGER DOMAINS of Apc11. Apc2 also interacts with the E2 ubiquitin ligases involved in APC-C ubiquitination reactions.'),
  'D064191': NtO(MH='Apc3 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='A highly evolutionarily conserved subunit of the anaphase-promoting complex (APC-C) containing multiple 34-amino-acid tetratricopeptide repeats. These domains, also found in Apc subunits 6, 7, and 8, have been shown to mediate protein-protein interactions, suggesting that Apc3 may assist in coordinating the juxtaposition of the catalytic and substrate recognition module subunits relative to co-activators and APC-C inhibitors.'),
  'D064192': NtO(MH='Apc4 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='A subunit of the anaphase-promoting complex whose primary function is to provide structural support for the catalytic and substrate-recognition modules of the complex. Apc4, along with Apc5, tethers the tetratricopeptide-coactivator binding subcomplex to the main structural subunit, Apc1.'),
  'D064193': NtO(MH='Apc5 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='A subunit of the anaphase-promoting complex whose primary function is to provide structural support for the catalytic and substrate-recognition modules of the complex. Apc5, along with Apc4, tethers the tetratricopeptide-coactivator binding subcomplex to the main structural subunit, Apc1.'),
  'D064194': NtO(MH='Apc6 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='A highly conserved subunit of the anaphase-promoting complex (APC-C) containing multiple 34 amino acid tetratricopeptide repeats. These domains, also found in Apc3, Apc7, and Apc8, have been shown to mediate protein-protein interactions, suggesting that Apc6 may assist in coordinating the juxtaposition of the catalytic and substrate recognition module subunits relative to coactivators and APC-C inhibitors.'),
  'D064195': NtO(MH='Apc7 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='A highly conserved subunit of the anaphase-promoting complex (APC-C) containing multiple 34 amino acid tetratricopeptide repeats. These domains, also found in Apc3, Apc6, and Apc8, have been shown to mediate protein-protein interactions, suggesting that Apc7 may assist in coordinating the juxtaposition of the catalytic and substrate recognition module subunits relative to coactivators and APC-C inhibitors.'),
  'D064196': NtO(MH='Apc8 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='A highly conserved subunit of the anaphase-promoting complex (APC-C) containing multiple 34-amino-acid tetratricopeptide repeats. These domains, also found in Apc3, Apc6, and Apc7, have been shown to mediate protein-protein interactions, suggesting that Apc8 may assist in coordinating the juxtaposition of the catalytic and substrate recognition module subunits relative to coactivators and APC-C inhibitors.'),
  'D064197': NtO(MH='Apc10 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='Apc10 is necessary for coactivator-dependent substrate recognition by the anaphase-promoting complex-cyclosome. It binds the Apc2 subunit, which is a part of the catalytic core, and interacts with coactivators Cdh1 or Cdc20 to recruit substrates to the complex.'),
  'D064198': NtO(MH='Apc11 Subunit, Anaphase-Promoting Complex-Cyclosome', MS='Together with the Apc2 subunit, forms the catalytic core of the E3 ubiquitin ligase, anaphase-promoting complex-cyclosome. It has a RING H2 domain which interacts with the cullin domain of Apc2. Apc11 also interacts with the E2 ubiquitin ligases involved in APC-C ubiquitination reactions.'),
  'D064199': NtO(MH='Cdc20 Proteins', MS='Highly conserved proteins that specifically bind to and activate the anaphase-promoting complex-cyclosome, promoting ubiquitination and proteolysis of cell-cycle-regulatory proteins. Cdc20 is essential for anaphase-promoting complex activity, initiation of anaphase, and cyclin proteolysis during mitosis.'),
  'D064200': NtO(MH='Cdh1 Proteins', MS='Cdh1 is an activator of the anaphase-promoting complex-cyclosome, and is involved in substrate recognition. It associates with the complex in late MITOSIS from anaphase through G1 to regulate activity of CYCLIN-DEPENDENT KINASES and to prevent premature DNA replication.'),
  'D064201': NtO(MH='Trichomes', MS='Hair-like extensions on specialized epidermal surfaces of plants which protect against damage from insects, animals, light degradation and fungal infection. Trichomes may also occur on certain unicellular EUKARYOTES.'),
  'D064202': NtO(MH="O'nyong-nyong Virus", MS='A species of ALPHAVIRUS causing a measles-like acute febrile rash illness. It is closely related to the Chikungunya virus and Igbo Ora viruses.'),
  'D064206': NtO(MH='Aggregatibacter', MS='A genus of PASTEURELLACEAE. Members are nonmotile, Gram-negative, facultatively anaerobic rods or coccobacilli. Its members are X factor (HEMIN) independent and variably dependent on V factor (NAD).'),
  'D064207': NtO(MH='Aggregatibacter aphrophilus', MS='A species of Gram-negative, facultatively anaerobic spherical or rod-shaped bacteria indigenous to oral cavity and pharynx. It is associated with BACTERIAL ENDOCARDITIS; and MENINGITIS.'),
  'D064208': NtO(MH='Aggregatibacter segnis', MS='A species of Gram-negative, facultatively anaerobic pleomorphic rod-shaped often filamentous bacteria in the genus of AGGREGATIBACTER found in the oral cavity. It is associated with DENTAL PLAQUE; and BACTERIAL ENDOCARDITIS.'),
  'D064209': NtO(MH='Phytochemicals', MS='A broad range of biologically active compounds which occur naturally in plants having important medicinal and nutritional properties.'),
  'D064210': NtO(MH='Secondary Metabolism', MS='A physiochemical process which occurs in a wide range of organisms which unlike BASAL METABOLISM is not required for or essential to short-term survivability but to long-term general well-being of the organism.'),
  'D064211': NtO(MH='Connectin', MS='A giant elastic protein of molecular mass ranging from 2,993 kDa (cardiac), 3,300 kDa (psoas), to 3,700 kDa (soleus) having a kinase domain. The amino- terminal is involved in a Z line binding, and the carboxy-terminal region is bound to the myosin filament with an overlap between the counter-connectin filaments at the M line.'),
  'D064226': NtO(MH='Prescription Drug Diversion', MS='The transfer of prescription drugs from legal to illegal distribution and marketing networks.'),
  'D064227': NtO(MH='Chlorofluorocarbons, Ethane', MS='A group of ethane-based halogenated hydrocarbons containing one or more fluorine and chlorine atoms.'),
  'D064228': NtO(MH='Bromochlorofluorocarbons', MS='A series of hydrocarbons containing BROMINE; CHLORINE and FLOURINE.'),
  'D064229': NtO(MH='Biological Ontologies', MS='Structured vocabularies describing concepts from the fields of biology and relationships between concepts.'),
  'D064231': NtO(MH='Nestin', MS='A type VI intermediate filament protein expressed mostly in nerve cells where it is associated with the survival, renewal and mitogen-stimulated proliferation of neural progenitor cells.'),
  'D064232': NtO(MH='Visual Analog Scale', MS='A subjective psychometric response scale used to measure distinct behavioral or physiological phenomena based on linear numerical gradient or yes/no alternatives.'),
  'D064233': NtO(MH='Sulfonylurea Receptors', MS='ATP-BINDING CASSETTE PROTEINS that are highly conserved and widely expressed in nature. They form an integral part of the ATP-sensitive potassium channel complex which has two intracellular nucleotide folds that bind to sulfonylureas and their analogs.'),
  'D064235': NtO(MH='Matrilin Proteins', MS='PROTEOGLYCANS-associated proteins that are major components of EXTRACELLULAR MATRIX of various tissues including CARTILAGE; and INTERVERTEBRAL DISC structures. They bind COLLAGEN fibers and contain protein domains that enable oligomer formation and interaction with other extracellular matrix proteins such as CARTILAGE OLIGOMERIC MATRIX PROTEIN.'),
  'D064236': NtO(MH='Cartilage Oligomeric Matrix Protein', MS='Major component of chondrocyte EXTRACELLULAR MATRIX of various tissues including bone, tendon, ligament, SYNOVIUM and blood vessels. It binds MATRILIN PROTEINS and is associated with development of cartilage and bone.'),
  'D064237': NtO(MH='Lipid Accumulation Product', MS='An index for monitoring the accumulation of lipids based on the WAIST CIRCUMFERENCE measurement and the level of TRIGLYCERIDES circulating in the blood.'),
  'D064246': NtO(MH='Mad2 Proteins', MS='Mad2 is a component of the spindle-assembly checkpoint apparatus. It binds to and inhibits the Cdc20 activator subunit of the anaphase-promoting complex, preventing the onset of anaphase until all chromosomes are properly aligned at the metaphase plate. Mad2 is required for proper microtubule capture at KINETOCHORES.'),
  'D064247': NtO(MH='Separase', MS='Separase is a caspase-like cysteine protease, which plays a central role in triggering ANAPHASE by cleaving the SCC1/RAD21 subunit of the cohesin complex. Cohesin holds the sister CHROMATIDS together during METAPHASE and its cleavage results in chromosome segregation.'),
  'D064248': NtO(MH='Geminin', MS='Geminin inhibits DNA replication by preventing the incorporation of MCM complex into pre-replication complex. It is absent during G1 phase of the CELL CYCLE and accumulates through S, G2,and M phases. It is degraded at the metaphase-anaphase transition by the ANAPHASE-PROMOTING COMPLEX-CYCLOSOME.'),
  'D064249': NtO(MH='Securin', MS='Securin is involved in the control of the metaphase-anaphase transition during MITOSIS. It promotes the onset of anaphase by blocking SEPARASE function and preventing proteolysis of cohesin and separation of sister CHROMATIDS. Overexpression of securin is associated with NEOPLASTIC CELL TRANSFORMATION and tumor formation.'),
  'D064250': NtO(MH='Hypertriglyceridemic Waist', MS='A screening phenotype consisting of both elevated WAIST CIRCUMFERENCE and elevated fasting TRIGLYCERIDES level.'),
  'D064251': NtO(MH='Multifunctional Enzymes', MS='Molecules that contain multiple active sites which are used to catalyze more than one enzymatic reaction. Proteins in this class generally contain multiple active sites within a single peptide chain and may also contain more than one enzymatically active subunit. They are distinguished from MULTIENZYME COMPLEXES in that their subunits are not found as distinct enzymes.'),
  'D064266': NtO(MH='Preexisting Condition Coverage', MS='Insurance coverage of a physical or mental health condition, disability, or illness existing before health insurance is applied for or enrolled in.'),
  'D064267': NtO(MH='Intrinsically Disordered Proteins', MS='Functional proteins that do not have unique, stable, folded, three-dimensional native structures or that possess non-ordered regions under physiological conditions. They are characterized by extraordinary structural flexibility and plasticity, which enable them to adopt different conformations in response to different stimuli or different interactions.'),
  'D064286': NtO(MH='ATP Binding Cassette Transporter 1', MS='A superfamily of integral ATP-binding cassette membrane proteins whose expression pattern is consistent with a role in lipid (cholesterol) efflux. It is implicated in TANGIER DISEASE, characterized by accumulation of cholesteryl ester in various tissues.'),
  'D064306': NtO(MH='Non-Nutritive Sweeteners', MS='Chemical additives, such as aspartame, saccharin, and sucralose, that give a sweet taste to foods without contributing significant calories or promoting tooth decay. They are generally much sweeter than sucrose.'),
  'D064307': NtO(MH='Microbiota', MS='The full collection of microbes (bacteria, fungi, virus, etc.) that naturally exist within a particular biological niche such as an organism, soil, a body of water, etc.'),
  'D064308': NtO(MH='Ecological Parameter Monitoring', MS='Ongoing collection, analysis, and interpretation of ecological data that is used to assess changes in the components, processes, and overall condition and functioning of an ECOSYSTEM.'),
  'D064326': NtO(MH='MEF2 Transcription Factors', MS='Activating transcription factors of the MADS family which bind a specific sequence element (MEF2 element) in many muscle-specific genes and are involved in skeletal and cardiac myogenesis, neuronal differentiation and survival/apoptosis.'),
  'D064346': NtO(MH='Sagittal Abdominal Diameter', MS='The height of the ABDOMEN as measured in the SUPINE POSITION.'),
  'D064347': NtO(MH='Microscopy, Electrochemical, Scanning', MS='A scanning probe microscopy technique that uses an ultramicroelectrode as the scanning probe that simultaneously records changes in electrochemical potential as it scans thereby creating topographical images with localized electrochemical information.'),
  'D064348': NtO(MH='Torpor', MS='A state of decreased activity in animals characterized by depressed metabolism, reduced body temperature, and low sensitivity to external stimuli.'),
  'D064349': NtO(MH='Genome, Microbial', MS='The genetic complement of a microorganism as represented in its DNA or in some microorganisms its RNA.'),
  'D064351': NtO(MH='Genes, Microbial', MS='The functional hereditary units of a microorganism.'),
  'D064367': NtO(MH='Olfactometry', MS='Procedures for measuring a response to odorants.'),
  'D064368': NtO(MH='Healthy Volunteers', MS='Persons with no known significant health problems who are recruited to participate in research to test a new drug, device, or intervention as controls for a patient group. (from http://clinicalcenter.nih.gov/recruit/volunteers.html, accessed 2/14/2013)'),
  'D064370': NtO(MH='Spike Glycoprotein, Coronavirus', MS='A class I viral fusion protein that forms the characteristic spikes, or peplomers, found on the viral surface that mediate virus attachment, fusion, and entry into the host cell. During virus maturation, it is cleaved into two subunits: S1, which binds to receptors in the host cell, and S2, which mediates membrane fusion.'),
  'D064371': NtO(MH='Hazardous Waste Sites', MS='Sites where HAZARDOUS SUBSTANCES are deposited. Some have facilities for safely storing, processing, and recycling these waste products.'),
  'D064386': NtO(MH='Ankle Fractures', MS='Fractures of any of the bones of the ANKLE.'),
  'D064406': NtO(MH='Patient Harm', MS='A measure of PATIENT SAFETY considering errors or mistakes which result in harm to the patient. They include errors in the administration of drugs and other medications (MEDICATION ERRORS), errors in the performance of procedures or the use of other types of therapy, in the use of equipment, and in the interpretation of laboratory findings and preventable accidents involving patients.'),
  'D064407': NtO(MH='Senior Centers', MS="Community centers for older adults and providers of resources for the community. In addition to providing services and activities for older adults that reflect the community's diversity, they link participants with resources offered by other agencies."),
  'D064410': NtO(MH='Human Trafficking', MS='Recruitment, transportation, transfer, harboring or receipt of persons, by means of the threat or use of force or other forms of coercion, of abduction, of fraud, of deception, of the abuse of power or of a position of vulnerability or of the giving or receiving of payments or benefits to achieve the consent of a person having control over another person, for the purpose of exploitation. (UN. Protocol to prevent, suppress and punish trafficking in persons, especially women and children, supplementing the United Nations convention against transnational organized crime. General Assembly resolution 55/25. United Nations General Assembly, 2000)'),
  'D064411': NtO(MH='Streptomyces rimosus', MS='An actinomycete best known as the producer of the commercially important antibiotic OXYTETRACYCLINE.'),
  'D064412': NtO(MH='Levalbuterol', MS='The R-isomer of albuterol.'),
  'D064413': NtO(MH='3-Phosphoinositide-Dependent Protein Kinases', MS='Highly conserved protein-serine threonine kinases that phosphorylate and activate a group of AGC protein kinases, especially in response to the production of the SECOND MESSENGERS, phosphatidylinositol 3,4,-biphosphate (PtdIns(3,4)P2) and phosphatidylinositol 3,4,5-triphosphate (PtdIns(3,4,5)P3).'),
  'D064414': NtO(MH='Type B Personality', MS='Behavior pattern characterized by a generally calm and even-tempered demeanor. Emotionally, such personality types show less frequent irritation, anger, hostility, and aggression than Type A individuals. (from International Encyclopedia of the Social Sciences, 2008, p. 223)'),
  'D064415': NtO(MH='Type D Personality', MS='Behavior pattern characterized by negative emotionality, an inability to express emotions, and social isolation, which has been linked to greater cardiovascular disease and increased mortality. (from International Encyclopedia of the Social Sciences, 2008, p. 217)'),
  'D064416': NtO(MH='Hospital Medicine', MS='The branch of medicine concerned with the delivery of comprehensive medical care to hospitalized patients. Practitioners include physicians and non-physician providers who engage in clinical care, teaching, research, or leadership in the field of general hospital medicine.(from http://www.hospitalmedicine.org/AM/Template.cfm?Section=Hospitalist_Definition)'),
  'D064417': NtO(MH='Vitamin K Epoxide Reductases', MS='OXIDOREDUCTASES which mediate vitamin K metabolism by converting inactive vitamin K 2,3-epoxide to active vitamin K.'),
  'D064419': NtO(MH='Chemically-Induced Disorders', MS='Disorders caused by the intentional or unintentional ingestion or exposure to chemical substances such as PHARMACEUTICAL PREPARATIONS; NOXAE; and PESTICIDES.'),
  'D064420': NtO(MH='Drug-Related Side Effects and Adverse Reactions', MS='Disorders that result from the intended use of PHARMACEUTICAL PREPARATIONS. Included in this heading are a broad variety of chemically-induced adverse conditions due to toxicity, DRUG INTERACTIONS, and metabolic effects of pharmaceuticals.'),
  'D064422': NtO(MH='Ambulance Diversion', MS='The practice of redirecting ambulances and patients seeking urgent care from one emergency department to others for various reasons such as overcrowding and shortage of skilled staff.'),
  'D064423': NtO(MH='Drug Trafficking', MS='The illegal acquisition, sale, production, transport, movement, and distribution of controlled substances such as narcotics.'),
  'D064424': NtO(MH='Tobacco Use', MS='Use of TOBACCO (Nicotiana tabacum L) and TOBACCO PRODUCTS.'),
  'D064425': NtO(MH='Herbals as Topic', MS='Works about books, articles or other publications on herbs or plants describing their medicinal value.'),
  'D064426': NtO(MH='Gasotransmitters', MS='Endogenously produced lipid-soluble gaseous molecules which function as neurotransmitters and signal mediators targeting ION CHANNELS and transporters.'),
  'D064427': NtO(MH='Nutritive Sweeteners', MS='Any agent that adds not only sweet taste but some energy value to food. They include natural sugars such as SUCROSE; FRUCTOSE; and GALACTOSE; and certain SUGAR ALCOHOLS.'),
  'D064428': NtO(MH='Hyperpolarization-Activated Cyclic Nucleotide-Gated Channels', MS='A subgroup of cyclic nucleotide-regulated ION CHANNELS of the superfamily of pore-loop cation channels that are opened by hyperpolarization rather than depolarization. The ion conducting pore passes SODIUM, CALCIUM, and POTASSIUM cations with a preference for potassium.'),
  'D064429': NtO(MH='Fatty Acid Synthases', MS='Enzymes that catalyze the synthesis of FATTY ACIDS from acetyl-CoA and malonyl-CoA derivatives.'),
  'D064430': NtO(MH='NAD (+) and NADP (+) Dependent Alcohol Oxidoreductases', MS='Oxidoreductases that act on the CH-OH group of donors with NAD (+) or NADPH (+) as acceptors.'),
  'D064431': NtO(MH='3-Oxoacyl-(Acyl-Carrier-Protein) Reductase', MS='A 3-oxoacyl reductase that has specificity for ACYL CARRIER PROTEIN-derived FATTY ACIDS.'),
  'D064446': NtO(MH='Orexin Receptors', MS='G-protein-coupled NEUROPEPTIDE RECEPTORS that have specificity for OREXINS and play a role in appetite control, and sleep-wake cycles. Two principle receptor types exist, each having a specificity for OREXIN A and OREXIN B peptide subtypes.'),
  'D064447': NtO(MH='Checkpoint Kinase 2', MS='Enzyme activated in response to DNA DAMAGE involved in cell cycle arrest. The gene is located on the long (q) arm of chromosome 22 at position 12.1. In humans it is encoded by the CHEK2 gene.'),
  'D064448': NtO(MH='Filamins', MS='A family of crosslinking filament proteins encoded by distinct FLN genes. Filamins are involved in cell adhesion, spreading, and migration, acting as scaffolds for over 90 binding partners including channels, receptors, intracellular signaling molecules and transcription factors. Due to the range of molecular interactions, mutations in FLN genes result in anomalies with moderate to lethal consequences.'),
  'D064449': NtO(MH='Sequestering Agents', MS='Compounds that bind to and reduce the biological availability of a chemical or pharmaceutical agent.'),
  'D064450': NtO(MH='Workplace Violence', MS='Threatened or actual attempt to harm others at place of employment.'),
  'D064451': NtO(MH='Hepcidins', MS='Forms of hepcidin, a cationic amphipathic peptide synthesized in the liver as a prepropeptide which is first processed into prohepcidin and then into the biologically active hepcidin forms, including in human the 20-, 22-, and 25-amino acid residue peptide forms. Hepcidin acts as a homeostatic regulators of iron metabolism and also possesses antimicrobial activity.'),
  'D064466': NtO(MH='Solute Carrier Family 12, Member 1', MS='Na-K-Cl transporter in the ASCENDING LIMB OF LOOP OF HENLE. It mediates active reabsorption of sodium chloride and is inhibited by LOOP DIURETICS such as FUROSEMIDE; and BUMETANIDE. Mutations in the gene encoding SLC12A1 are associated with a BARTTER SYNDROME.'),
  'D064486': NtO(MH='Solute Carrier Family 12, Member 3', MS='Na-Cl cotransporter in the convoluted segments of the DISTAL KIDNEY TUBULE. It mediates active reabsorption of sodium and chloride and is inhibited by THIAZIDE DIURETICS.'),
  'D064506': NtO(MH='Solute Carrier Family 12, Member 2', MS='Na-K-Cl transporter ubiquitously expressed. It plays a key role in salt secretion in epithelial cells and cell volume regulation in nonepithelial cells.'),
  'D064507': NtO(MH='Solute Carrier Family 12, Member 4', MS='K-Cl cotransporter ubiquitously expressed with higher expression levels in ERYTHROCYTES of ANEMIA, SICKLE CELL. It mediates active potassium and chloride cotransport across the plasma membrane and contributes to cell volume homeostasis'),
  'D064527': NtO(MH='Etiolation', MS='Physical changes in the growth patterns of a plant brought on by sustained absence of light. These changes are characterized by lengthened internodes which produce long weak stems, fewer leaves, and pale yellow color (chlorosis). The physiological basis for etiolation is induction of the phytohormone, AUXIN.'),
  'D064529': NtO(MH='Receptor, Metabotropic Glutamate 5', MS='A type I G protein-coupled receptor mostly expressed post-synaptic pyramidal cells of the cortex and CENTRAL NERVOUS SYSTEM.'),
  'D064531': NtO(MH='Peripherins', MS='Type III intermediate filament proteins expressed mainly in neurons of the peripheral and CENTRAL NERVOUS SYSTEMS. Peripherins are implicated in neurite elongation during development and axonal regeneration after injury.'),
  'D064532': NtO(MH='Histocompatibility Antigen H-2D', MS='A component of the murine major histocompatibility complex class I family. It contains one Ig-like C1-type domain and functions in processing and presentation of exogenous peptide antigens to the immune system.'),
  'D064546': NtO(MH='Protein Kinase C beta', MS='PKC beta encodes two proteins (PKCB1 and PKCBII) generated by alternative splicing of C-terminal exons. It is widely distributed with wide-ranging roles in processes such as B-cell receptor regulation, oxidative stress-induced apoptosis, androgen receptor-dependent transcriptional regulation, insulin signaling, and endothelial cell proliferation.'),
  'D064547': NtO(MH='Myxovirus Resistance Proteins', MS='Interferon-induced DYNAMIN-like GTP-binding proteins localized in the cytoplasm, nuclear pore complex and nucleus. They play a role in antiviral defense and immunity.'),
  'D064548': NtO(MH='Hand Sanitizers', MS='Preparations used as alternatives or supplements to hand washing with soap and water to destroy microorganisms and prevent transmission of pathogens. The active ingredient may be ETHANOL; 1-PROPANOL; or POVIDONE-IODINE in a gel, foam, or liquid solution.'),
  'D064549': NtO(MH='Myeloid Cell Leukemia Sequence 1 Protein', MS='A member of the myeloid leukemia factor (MLF) protein family with multiple alternatively spliced transcript variants encoding different protein isoforms. In hematopoietic cells, it is located mainly in the nucleus, and in non-hematopoietic cells, primarily in the cytoplasm with a punctate nuclear localization. MLF1 plays a role in cell cycle differentiation.'),
  'D064550': NtO(MH='Death-Associated Protein Kinases', MS='A family of calcium/calmodulin-dependent PROETIN-SERINE-THREONINE KINASES. They are ubiquitously expressed in adult and embryonic mammalian tissues, and their functions are tightly related to the early stages of eukaryotic programmed cell death.'),
  'D064568': NtO(MH='S100 Calcium Binding Protein beta Subunit', MS='A calcium-binding protein that is 92 AA long, contains 2 EF-hand domains, and is concentrated mainly in GLIAL CELLS. Elevation of S100B levels in brain tissue correlates with a role in neurological disorders.'),
  'D064569': NtO(MH='alpha7 Nicotinic Acetylcholine Receptor', MS='A member of the NICOTINIC ACETYLCHOLINE RECEPTOR subfamily of the LIGAND-GATED ION CHANNEL family. It consists entirely of pentameric \xce\xb17 subunits expressed in the CNS, autonomic nervous system, vascular system, lymphocytes and spleen.'),
  'D064570': NtO(MH='Ubiquitin-Specific Proteases', MS='Members of the peptidase C19 family which regulate signal transduction by removing UBIQUITIN from specific protein substrates via a process known as deubiquitination or deubiquitylation.'),
  'D064586': NtO(MH='Selenic Acid', MS='A strong dibasic acid with the molecular formula H2SeO4. Included under this heading is the acid form, and inorganic salts of dihydrogen selenium tetraoxide.'),
  'D064587': NtO(MH='Osmoregulation', MS='The response of cells in sensing a difference in OSMOTIC PRESSURE between the inside and outside of the cell. This response includes signaling from osmotic sensors to activate transcription factors, which in turn regulate the expression of osmocompensatory genes, all functioning to maintain CELL VOLUME and the water concentration inside the cells.'),
  'D064588': NtO(MH='Selenium Oxides', MS='Inorganic compounds formed through the oxidation of selenium.'),
  'D064590': NtO(MH='Cool-Down Exercise', MS='Tapering-off physical activity from vigorous to light, to gradually return the body to pre-exercise condition and metabolic state.'),
  'D064591': NtO(MH='Allografts', MS='Tissues, cells, or organs transplanted between genetically different individuals of the same species.'),
  'D064592': NtO(MH='Autografts', MS="Transplant comprised of an individual's own tissue, transferred from one part of the body to another."),
  'D064593': NtO(MH='Heterografts', MS='Tissues, cells or organs transplanted between animals of different species.'),
  'D064594': NtO(MH='Bone-Patellar Tendon-Bone Grafts', MS='A bone-patellar tendon graft used for fixation of the ANTERIOR CRUCIATE LIGAMENT during surgical reconstruction of the knee.'),
  'D064595': NtO(MH='Composite Tissue Allografts', MS='A graft consisting of multiple tissues, such as muscle, bone, nerve, vasculature, and skin, comprising a functional unit for reconstructive purposes.'),
  'D064596': NtO(MH='Isografts', MS='Tissues, cells or organs transplanted between genetically identical individuals, i.e. members of the same species with identical histocompatibility antigens, such as monozygotic twins, members of the same inbred strain, or members of a hybrid population produced by crossing certain inbred strains.'),
  'D064606': NtO(MH='SLC4A Proteins', MS='Bicarbonate transporters that move BICARBONATE IONS in exchange of CHLORIDE IONS or SODIUM IONS across membranes. They regulate acid-base HOMEOSTASIS, cell volume and intracellular pH. Members include CHLORIDE-BICARBONATE ANTIPORTERS (SLC4A1, 2, 3, and 9); SODIUM-COUPLED BICARBONATE TRANSPORTERS (SLC4A4 and 5, 7, 8 and 10); and a sodium borate cotransporter (SLC4A11 protein).'),
  'D064646': NtO(MH='Cardiovascular Nursing', MS='A specialty concerned with the nursing care of patients suffering from disorders of the CARDIOVASCULAR SYSTEM as well as those identified as at risk for adverse cardiac or vascular events.'),
  'D064647': NtO(MH='Nephrology Nursing', MS='A nursing specialty involving the care of patients with or at risk for KIDNEY DISEASES.'),
  'D064648': NtO(MH='Critical Care Nursing', MS='A nursing specialty that deals specifically with the care of patients who are critically ill.'),
  'D064650': NtO(MH='Home Health Nursing', MS='A nursing specialty in which skilled nursing care is provided to patients in their homes by registered or licensed practical NURSES. Home health nursing differs from HOME NURSING in that home health nurses are licensed professionals, while home nursing involves non-professional caregivers.'),
  'D064652': NtO(MH='Basal Bodies', MS='Structures which form the base of FLAGELLA and CILIA. They contain nine triplets of MICROTUBULES that are arranged around the periphery and that serve as the nucleation center for AXONEME assembly.'),
  'D064666': NtO(MH='Diapause, Insect', MS='Seasonal suspension of insect growth development. It can be either induced by environmental cues (e.g., PHOTOPERIOD) or as a facultative part of the life cycle in order to time development with seasonal changes.'),
  'D064686': NtO(MH='Nutritionists', MS='Persons specially trained in NUTRITION SCIENCES.'),
  'D064687': NtO(MH='Diarylquinolines', MS='A class of quinoline compounds defined by the presence of two aromatic ring structures which are attached via a side chain to carbon 3 of the qunolinyl structure. The two aromatic moieties are typically NAPTHALENE and BENZENE. Several compounds in this class are used as ANTITUBERCULAR AGENTS.'),
  'D064688': NtO(MH='Metabolic Flux Analysis', MS="Measurement of cells' substrate utilization and biosynthetic output for modeling of METABOLIC NETWORKS."),
  'D064689': NtO(MH='Nurses, Public Health', MS='Nurses whose goal is to improve health and quality of life in a population or community through the prevention and treatment of disease and other physical and mental health conditions, the surveillance of cases and health indicators, and the promotion of healthy behaviors through public education and awareness.'),
  'D064690': NtO(MH='Wakefulness-Promoting Agents', MS='A specific category of drugs that prevent sleepiness by specifically targeting sleep-mechanisms in the brain. They are used to treat DISORDERS OF EXCESSIVE SOMNOLENCE such as NARCOLEPSY. Note that this drug category does not include broadly-acting central nervous system stimulants such as AMPHETAMINES.'),
  'D064691': NtO(MH='Nurses, Community Health', MS='Nurses whose work combines elements of both primary care nursing and public health practice and takes place primarily outside the therapeutic institution. Primary nursing care is directed to individuals, families, or groups in their natural settings within communities.'),
  'D064692': NtO(MH='Hyoscyamine', MS='The 3(S)-endo isomer of atropine.'),
  'D064694': NtO(MH='Rural Nursing', MS='A branch of nursing requiring generalist training with specialty knowledge in crisis assessment and management in all subdisciplines of nursing. Rural nursing practices often include geographical and social distance concepts in delivery of health care.'),
  'D064695': NtO(MH='Enclomiphene', MS='The trans or (E)-isomer of clomiphene.'),
  'D064696': NtO(MH='Zuclomiphene', MS='The cis or (Z)-isomer of clomiphene.'),
  'D064697': NtO(MH='Racemethionine', MS='A preparation of METHIONINE that includes a mixture of D-methionine and L-methionine isomers.'),
  'D064698': NtO(MH='Parish Nursing', MS='A nursing specialty involving programs designed to bring wholeness and healing to a particular faith community through addressing the health needs of body, mind, and spirit. They are coordinated by registered NURSES and may involve HEALTH EDUCATION and counseling, facilitation, referral, PATIENT ADVOCACY, and health care plan interpretation, as influenced and defined by the unique needs of the congregation.'),
  'D064699': NtO(MH='Dexmethylphenidate Hydrochloride', MS='A methylphenidate derivative, DOPAMINE UPTAKE INHIBITOR and CENTRAL NERVOUS SYSTEM STIMULANT that is used in the treatment of ATTENTION DEFICIT HYPERACTIVITY DISORDER.'),
  'D064700': NtO(MH='Nurses, International', MS='Nurses who provide nursing care in countries other than the ones in which they received their training.'),
  'D064701': NtO(MH='Pediatric Nurse Practitioners', MS='Registered nurses with graduate degrees in nursing who provide care to pediatric patients who are acutely or critically ill.'),
  'D064702': NtO(MH='International Classification of Functioning, Disability and Health', MS="The World Health Organization's classification categories of health and health-related domains. The International Classification of Functioning, Disability and Health (ICF) consists of two lists: a list of body functions and structure, and a list of domains of activity and participation. The ICF also includes a list of environmental factors."),
  'D064703': NtO(MH='Family Nurse Practitioners', MS='Registered nurses with graduate degrees in nursing who provide care to patients of all age levels, and who focus their efforts on the health care needs of the entire family.'),
  'D064704': NtO(MH='Levofloxacin', MS='The L-isomer of Ofloxacin.'),
  'D064705': NtO(MH='Racepinephrine', MS='A racemic mixture of d-epinephrine and l-epinephrine.'),
  'D064706': NtO(MH='Vocal Cord Dysfunction', MS='A disorder characterized by an intermittent abnormal VOCAL CORDS movement toward the midline during inspiration or expiration resulting in upper AIRWAY OBSTRUCTION.'),
  'D064726': NtO(MH='Triple Negative Breast Neoplasms', MS='Breast neoplasms that do not express ESTROGEN RECEPTORS; PROGESTERONE RECEPTORS; and do not overexpress the NEU RECEPTOR/HER-2 PROTO-ONCOGENE PROTEIN.'),
  'D064727': NtO(MH='Posterior Capsulotomy', MS='Procedures performed to remove CAPSULE OPACIFICATION that develops on the POSTERIOR CAPSULE OF THE LENS following removal of a primary CATARACT.'),
  'D064728': NtO(MH='Alveolar Bone Grafting', MS='Surgical procedure to add bone to the ALVEOLAR RIDGE in children born with a CLEFT LIP and a CLEFT PALATE.'),
  'D064729': NtO(MH='Neurokinin-1 Receptor Antagonists', MS='Compounds that inhibit or block the activity of NEUROKININ-1 RECEPTORS.'),
  'D064730': NtO(MH='Dexrazoxane', MS='The (+)-enantiomorph of razoxane.'),
  'D064746': NtO(MH='Therapy, Soft Tissue', MS='The assessment, treatment, and management of soft tissue injury or dysfunction. Therapy is designed to reduce tension and irritation in affected tissues and may include MASSAGE; muscle stretching, or direct pressure on muscles, connective tissue, and TRIGGER POINTS.'),
  'D064747': NtO(MH='Lansoprazole', MS='A 2,2,2-trifluoroethoxypyridyl derivative of timoprazole that is used in the therapy of STOMACH ULCERS and ZOLLINGER-ELLISON SYNDROME. The drug inhibits H(+)-K(+)-EXCHANGING ATPASE which is found in GASTRIC PARIETAL CELLS. Lansoprazole is a racemic mixture of (R)- and (S)-isomers.'),
  'D064748': NtO(MH='Dexlansoprazole', MS='The R-isomer of lansoprazole that is used to treat severe GASTROESOPHAGEAL REFLUX DISEASE.'),
  'D064749': NtO(MH='Retrocaval Ureter', MS='A rare congenital abnormality resulting in the URETER passing dorsal to and being obstructed by the INFERIOR VENA CAVA.'),
  'D064750': NtO(MH='Rabeprazole', MS='A 4-(3-methoxypropoxy)-3-methylpyridinyl derivative of timoprazole that is used in the therapy of STOMACH ULCERS and ZOLLINGER-ELLISON SYNDROME. The drug inhibits H(+)-K(+)-EXCHANGING ATPASE which is found in GASTRIC PARIETAL CELLS.'),
  'D064751': NtO(MH='Ammonium Compounds', MS='Inorganic compounds that include a positively charged tetrahedral nitrogen (ammonium ion) as part of their structure. This class of compounds includes a broad variety of simple ammonium salts and derivatives.'),
  'D064752': NtO(MH='Atrial Remodeling', MS='Long-term changes in the electrophysiological parameters and/or anatomical structures of the HEART ATRIA that result from prolonged changes in atrial rate, often associated with ATRIAL FIBRILLATION or long periods of intense EXERCISE.'),
  'D064753': NtO(MH='Ammonium Hydroxide', MS='The hydroxy salt of ammonium ion. It is formed when AMMONIA reacts with water molecules in solution.'),
  'D064766': NtO(MH='Influenza A Virus, H7N9 Subtype', MS='A subtype of INFLUENZA A VIRUS with the surface proteins hemagglutinin 7 and neuraminidase 9. This avian origin virus was first identified in humans in 2013.'),
  'D064786': NtO(MH='Nocebo Effect', MS='An adverse effect occurring with a medical treatment that is not attributable to the actions of the treatment.'),
  'D064787': NtO(MH='Portion Size', MS='The amount of a particular food one chooses to eat at a single meal. It is different from SERVING SIZE, which is a reference amount of food as defined by an authoritative source, such as the Food Guide Pyramid devised by the UNITED STATES DEPARTMENT OF AGRICULTURE.'),
  'D064788': NtO(MH='Serving Size', MS='A reference measure of food used to identify the calorie and nutrient content in a particular amount of that food. It is defined by an authoritative source, such as the Food Guide Pyramid devised by the UNITED STATES DEPARTMENT OF AGRICULTURE. It is different from PORTION SIZE, which is the amount of food one chooses to eat at a single meal.'),
  'D064789': NtO(MH='Immunity, Heterologous', MS="The immunological phenomenon by which exposure to some organisms or vaccines can profoundly alter the host's response to subsequent exposure to unrelated (heterologous) organisms or vaccines."),
  'D064790': NtO(MH='Clinical Laboratory Services', MS='Organized services provided by MEDICAL LABORATORY PERSONNEL for the purpose of carrying out CLINICAL LABORATORY TECHNIQUES used for the diagnosis, treatment, and prevention of disease.'),
  'D064791': NtO(MH='Desoxycorticosterone Acetate', MS='The 21-acetate derivative of desoxycorticosterone.'),
  'D064792': NtO(MH='Pragmatic Clinical Trials as Topic', MS='Works about randomized clinical trials that compare interventions in clinical settings and which look at a range of effectiveness outcomes and impacts.'),
  'D064793': NtO(MH='Teratogenesis', MS='The formation of CONGENITAL ABNORMALITIES.'),
  'D064794': NtO(MH='Lorajmine', MS='A monochloroacetyl derivative of ajmaline. It is a class Ia antiarrhythmic agent that is rapidly hydrolyzed to ajmaline by plasma and tissue esterases.'),
  'D064795': NtO(MH='Intraoperative Neurophysiological Monitoring', MS="The systematic checking of the condition and function of a patient's CENTRAL NERVOUS SYSTEM during the course of a surgical operation."),
  'D064796': NtO(MH='Veterinary Sports Medicine', MS='The field of veterinary medicine concerned with PHYSICAL FITNESS of animals in sports (horse racing, dog racing, etc.) and the diagnosis and treatment of sports injuries in animals.'),
  'D064797': NtO(MH='Physical Conditioning, Human', MS='Diet modification and physical exercise to improve the ability to carry out daily tasks and perform physical activities.'),
  'D064798': NtO(MH='Polypharmacology', MS='The design or use of pharmaceutical agents that act on multiple targets or disease pathways.'),
  'D064799': NtO(MH='Hypoxia-Inducible Factor-Proline Dioxygenases', MS='Dioxygenase enzymes that specifically hydroxylate a PROLINE residue on the HYPOXIA-INDUCIBLE FACTOR 1, ALPHA SUBUNIT. They are OXYGEN-dependent enzymes that play an important role in mediating cellular adaptive responses to HYPOXIA.'),
  'D064800': NtO(MH='Prolyl-Hydroxylase Inhibitors', MS='Compounds that inhibit the action of HYDROXYLASES that act on PROLINE to form HYDROXYPROLINE.'),
  'D064801': NtO(MH='Phospholipase A2 Inhibitors', MS='Compounds that inhibit or block the activity of a PHOSPHOLIPASE A2 enzyme.'),
  'D064804': NtO(MH='Urological Agents', MS='Drugs used in the treatment of urogenital conditions and diseases such as URINARY INCONTINENCE; PROSTATIC HYPERPLASIA; and ERECTILE DYSFUNCTION.'),
  'D064805': NtO(MH='Bunolol', MS='A nonselective beta-adrenoceptor antagonist used in the treatment of glaucoma.'),
  'D064806': NtO(MH='Dysbiosis', MS='Changes in quantitative and qualitative composition of MICROBIOTA. The changes may lead to altered host microbial interaction or homeostatic imbalance that can contribute to a disease state often with inflammation.'),
  'D064807': NtO(MH='Anticholinergic Syndrome', MS='Adverse drug effects associated with CHOLINERGIC ANTAGONISTS. Clinical features include TACHYCARDIA; HYPERTHERMIA; MYDRIASIS, dry skin and dry mucous membranes, decreased bowel sounds and urinary retention in peripheral anticholinergic syndrome; and HALLUCINATIONS; PSYCHOSES; SEIZURES; and COMA in central anticholinergic syndrome.'),
  'D064826': NtO(MH='Transfusion Medicine', MS='The area of hematology that is concerned with the transfusion of blood and blood components, and in prevention and treatment of adverse effects from BLOOD TRANSFUSION errors.'),
  'D064827': NtO(MH='Myocutaneous Flap', MS='A mass of tissue, including skin and muscle, that has been cut away from surrounding areas for transplantation.'),
  'D064828': NtO(MH='Prolyl Hydroxylases', MS='Enzymes that specifically hydroxylate PROLINE residues on proteins.'),
  'D064829': NtO(MH='Alcohol Abstinence', MS='Non-consumption of ALCOHOLIC BEVERAGES.'),
  'D064830': NtO(MH='Temperance Movement', MS='Social effort most active in the 19th and early 20th centuries dedicated to promoting moderation or complete abstinence in the use of ALCOHOLIC BEVERAGES.'),
  'D064846': NtO(MH='Sports for Persons with Disabilities', MS='Activities or games played by PERSONS WITH DISABILITIES, usually requiring physical effort or skill. The activities or games may be specifically created or based on existing sports, with or without modifications, to meet the needs of persons with physical or intellectual disabilities.'),
  'D064847': NtO(MH='Multimodal Imaging', MS='The use of combination of imaging techniques or platforms (e.g., MRI SCAN and PET SCAN) encompassing aspects of anatomical, functional, or molecular imaging methods.'),
  'D064866': NtO(MH='Mindfulness', MS="A psychological state of awareness, the practices that promote this awareness, a mode of processing information and a character trait. As a therapy mindfulness is defined as a moment-to-moment awareness of one's experience without judgment and as a state and not a trait."),
  'D064867': NtO(MH='Mesopotamia', MS='Ancient land between the Tigris and Euphrates rivers, corresponding to modern-day Iraq and partly NE Syria, SE Turkey and smaller parts of SW Iran. It is considered to be the cradle of civilization in the West. Bronze Age Mesopotamia included Sumer and the Akkadian, Babylonian and Assyrian empires (from http://www.merriam-webster.com/dictionary/mesopotamia )'),
  'D064868': NtO(MH='Ethnic Violence', MS='Violence or other hostile behavior arising when an ethnic group either feels itself under threat, or where it seeks to assert its superiority or dominance over other groups.'),
  'D064869': NtO(MH='Acceptance and Commitment Therapy', MS="Contextually focused form of cognitive behavioral psychotherapy that uses MINDFULNESS and behavioral activation to increase patients' psychological flexibility in areas such as ability to engage in values-based, positive behaviors while experiencing difficult thoughts, emotions, or sensations."),
  'D064870': NtO(MH='Air Travel', MS='Travel by means of an airplane, helicopter, or balloon.'),
  'D064871': NtO(MH='Teach-Back Communication', MS='Providing the patient, family or others information, and then allowing them to take an action or restate the information in their own words. The clinician prefaces communications with a statement framed to show the clinician has the burden of effective communication rather than the patient.'),
  'D064872': NtO(MH='Patient Care Bundles', MS='Small sets of evidence-based interventions for a defined patient population and care setting.'),
  'D064873': NtO(MH='Asia, Northern', MS='A subregion of Asia, consisting of the Asian portion of Russia.'),
  'D064874': NtO(MH='Tatarstan', MS='Political subdivision of eastern RUSSIA located within Europe. It lies between the Volga River and the Kama River, and extends east to the Ural mountains.'),
  'D064876': NtO(MH='Safety-net Providers', MS='Providers that by mandate or mission organize and deliver a significant level of health care and other health-related services to the uninsured, Medicaid recipients, and other vulnerable patients.'),
  'D064877': NtO(MH='Organ Trafficking', MS="Criminal activity that occurs in three broad categories. Firstly, cases where traffickers force or deceive the victims into giving up an organ. Secondly, there are cases where victims formally or informally agree to sell an organ and are cheated because they are not paid for the organ or are paid less than the promised price. Thirdly, vulnerable persons are treated for an ailment, which may or may not exist and thereupon organs are removed without the victim's knowledge."),
  'D064878': NtO(MH='Web Browser', MS='Software application for retrieving, presenting and traversing information resources on the World Wide Web.'),
  'D064886': NtO(MH='Dataset', MS='Works consisting of organized collections of data, which have been stored permanently in a formalized manner suitable for communication, interpretation, or processing.'),
  'D064887': NtO(MH='Observational Studies as Topic', MS='Works about clinical studies in which participants may receive diagnostic, therapeutic, or other types of interventions, but the investigator does not assign participants to specific interventions (as in an interventional study).'),
  'D064888': NtO(MH='Observational Study', MS='A work that reports on the results of a clinical study in which participants may receive diagnostic, therapeutic, or other types of interventions, but the investigator does not assign participants to specific interventions (as in an interventional study).'),
  'D064889': NtO(MH='Psychotherapy, Psychodynamic', MS='Forms of PSYCHOTHERAPY falling within or deriving from the psychoanalytic tradition, that view individuals as reacting to unconscious forces (e.g., motivation, drive), that focus on processes of change and development, and that place a premium on self understanding and making meaning of what is unconscious.'),
  'D064890': NtO(MH='Social Determinants of Health', MS='The circumstances in which people are born, grow up, live, work, and age, as well as the systems put in place to deal with illness. These circumstances are in turn shaped by a wider set of forces: economics, social policies, and politics (http://www.cdc.gov/socialdeterminants/).'),
  'D064891': NtO(MH='Disaster Victims', MS='Persons adversely effected by DISASTERS, occurrences that result in property damage, deaths, and/or injuries to a community.'),
  'D064906': NtO(MH='Qigong', MS="An ancient Chinese system of postures, exercises, breathing techniques, and meditations designed to improve and enhance the body's QI."),
  'D064907': NtO(MH='Diagnostic Uses of Chemicals', MS='Chemicals administered to patients in order to diagnose or study the pathology of medical conditions, diseases, or syndromes.'),
  'D064926': NtO(MH='Neurophysiological Monitoring', MS="The systematic checking of the condition and function of a patient's CENTRAL NERVOUS SYSTEM during disease periods or following injuries, using various NEUROLOGICAL DIAGNOSTIC TECHNIQUES."),
  'D064927': NtO(MH='Tick Bites', MS='The effects, both local and systemic, caused by the bites of TICKS.'),
  'D064946': NtO(MH='Hospice and Palliative Care Nursing', MS='A nursing specialty concerned with care of patients facing serious or life-threatening illnesses. The goal of palliative nursing is to prevent and relieve suffering, and to support the best possible quality of life for patients and their families. Hospice nursing is palliative care for people in their final stages of life.'),
  'D064966': NtO(MH='British Virgin Islands', MS='A British overseas territory, part of the British West Indies, lying about 60 miles east of Puerto Rico. Tortola is the main island, other islands include Virgin Gorda, Jost Van Dyke and Anegada.'),
  'D064987': NtO(MH='Cell- and Tissue-Based Therapy', MS='Therapies that involve the TRANSPLANTATION of CELLS or TISSUES developed for the purpose of restoring the function of diseased or dysfunctional cells or tissues.'),
  'D065006': NtO(MH='Papanicolaou Test', MS='Cytological preparation of cells collected from a mucosal surface and stained with Papanicolaou stain.'),
  'D065007': NtO(MH='Pragmatic Clinical Trial', MS='Randomized clinical trials that compare interventions in clinical settings and which look at a range of effectiveness outcomes and impacts.'),
  'D065008': NtO(MH='Scorpion Stings', MS='The effects, both local and systemic, caused by the bite of SCORPIONS.'),
  'D065026': NtO(MH='Hope', MS='Belief in a positive outcome.'),
  'D065067': NtO(MH='Antihyperkalemic Agents', MS='Compounds used to lower excessive POTASSIUM levels in the body.'),
  'D065086': NtO(MH='Acetaldehyde Dehydrogenase Inhibitors', MS='Compounds that bind to and inhibit the enzymatic activity of acetaldehyde dehydrogenases.'),
  'D065087': NtO(MH='Acetylcholine Release Inhibitors', MS='Compounds that block release of the neurotransmitter ACETYLCHOLINE.'),
  'D065088': NtO(MH='Steroid Synthesis Inhibitors', MS='Compounds that bind to and inhibit enzymes involved in the synthesis of STEROIDS.'),
  'D065089': NtO(MH='Glycoside Hydrolase Inhibitors', MS='Compounds that inhibit or block the activity of GLYCOSIDE HYDROLASES such as ALPHA-AMYLASES and ALPHA-GLUCOSIDASES.'),
  'D065091': NtO(MH='Amylin Receptor Agonists', MS='Compounds that stimulate the activity of AMYMIN RECEPTORS. Included under this heading is the endogenous form of ISLET AMYLOID POLYPEPTIDE and synthetic compounds that mimic its effect.'),
  'D065092': NtO(MH='Antidiuretic Hormone Receptor Antagonists', MS='Endogenous compounds and drugs that inhibit or block the activity of ANTIDUIRETIC HORMONE RECEPTORS.'),
  'D065093': NtO(MH='beta-Lactamase Inhibitors', MS='Endogenous substances and drugs that inhibit or block the activity of BETA-LACTAMASES.'),
  'D065094': NtO(MH='Bradykinin B2 Receptor Antagonists', MS='Compounds and drugs that inhibit ligand binding or cellular signaling by BRADYKININ B2 RECEPTORS.'),
  'D065095': NtO(MH='Calcineurin Inhibitors', MS='Compounds that inhibit or block the PHOSPHATASE activity of CALCINEURIN.'),
  'D065096': NtO(MH='Calcium Chelating Agents', MS='Substances that bind to and sequester CALCIUM ions.'),
  'D065097': NtO(MH='Carbamoyl Phosphate Synthetase I Activators', MS='Substances and drugs that stimulate the synthesis or increase the activity of CARBAMOYL PHOSPHATE SYNTHETASE I. Drugs in this class of compounds have been used to treat the genetic disorder, CARBAMOYL-PHOSPHATE SYNTHASE I DEFICIENCY DISEASE.'),
  'D065098': NtO(MH='Catechol O-Methyltransferase Inhibitors', MS='Compounds and drugs that inhibit or block the activity of CATECHOL O-METHYLTRANSFERASE enzymes. Drugs in this class are used in management of central nervous system disorders such as PARKINSON DISEASE.'),
  'D065099': NtO(MH='Travel Nursing', MS='A type of nursing practice involving licensed professional nurses who travel to work in temporary nursing positions.'),
  'D065100': NtO(MH='CCR5 Receptor Antagonists', MS='Compounds and drugs that inhibit or block the activity of CCR5 RECEPTORS.'),
  'D065101': NtO(MH='Chloride Channel Agonists', MS='A class of drugs that stimulate chloride ion influx through cell membrane channels.'),
  'D065102': NtO(MH='Licensed Practical Nurses', MS='Health personnel who do not hold professional degrees or credentials, but have completed training and are licensed to provide routine patient care under the direction of registered nurses and physicians.'),
  'D065104': NtO(MH='Cystine Depleting Agents', MS='Compounds and drugs that react with CYSTINE and convert it into a compound that can be more easily metabolized or intracellularly transported. Drugs in this class have been used to treat CYSTINOSIS.'),
  'D065105': NtO(MH='Aromatic Amino Acid Decarboxylase Inhibitors', MS='Compounds and drugs that block or inhibit the enzymatic action of AROMATIC AMINO ACID DECARBOXYLASES. Pharmaceutical agents in this category are used in conjunction with LEVODOPA in order to slow its metabolism.'),
  'D065106': NtO(MH='Demulcents', MS='Compounds that form soothing films on the surface of MUCOUS MEMBRANES. They are generally used as agents to protect the integrity of internal EPITHELIUM.'),
  'D065108': NtO(MH='Ornithine Decarboxylase Inhibitors', MS='Substances and drugs that inhibit or block the activity of ORNITHINE DECARBOXYLASE.'),
  'D065126': NtO(MH='Palliative Medicine', MS="A branch of medicine concerned with preventing and relieving the suffering of patients in any stage of disease. Its focus is on comprehensive care and improving the quality of all areas of the patient's life."),
  'D065127': NtO(MH='Dopamine D2 Receptor Antagonists', MS='Compounds and drugs that bind to and inhibit or block the activation of DOPAMINE D2 RECEPTORS.'),
  'D065128': NtO(MH='Endothelin Receptor Antagonists', MS='Compounds and drugs that bind to and inhibit or block the activation of ENDOTHELIN RECECPTORS.'),
  'D065129': NtO(MH='Pre-Exposure Prophylaxis', MS='A method of disease prevention (abbreviated PrEP) which involves the administration of drugs to at-risk persons who have not been exposed to the disease-causing agent.'),
  'D065130': NtO(MH='Endothelin A Receptor Antagonists', MS='Compounds and drugs that bind to and inhibit or block the activation of ENDOTHELIN A RECECPTORS.'),
  'D065131': NtO(MH='Endothelin B Receptor Antagonists', MS='Compounds and drugs that bind to and inhibit or block the activation of ENDOTHELIN B RECEPTORS.'),
  'D065132': NtO(MH='Bioprospecting', MS='The systematic search and discovery of natural substances which may have potential commercial applications.'),
  'D065133': NtO(MH='Ephemeroptera', MS='An Order of aquatic INSECTS comprising about 2,500 species worldwide. It is the only insect group with two flying stages, each with relatively large forewings. The immature stage called "naiad" or "nymph" lasts one year in fresh water, and the adult stage is shortlived, lasting from a few minutes to a few days, depending on the species. The Ephemeroptera includes mayflies (also called shadflies)'),
  'D065134': NtO(MH='Lipedema', MS='Disorder of adipose tissue characterized by symmetric and bilateral enlargement of the lower extremities due to abnormal deposition of SUBCUTANEOUS FAT often in obese women. It is associated with HEMATOMA, pain and may progress to secondary LYMPHEDEMA which is known as lipolymphedema.'),
  'D065136': NtO(MH='Animal Culling', MS='Selection and removal of individuals from a population or group. Culling is a common strategy with livestock and wildlife, used to control and eradicate diseases or pests, control population size, select for desired genetic characteristics, and maximize economic profit.'),
  'D065146': NtO(MH='Zeaxanthins', MS='Carotenoids found in fruits and vegetables. Zeaxanthin accumulates in the MACULA LUTEA.'),
  'D065147': NtO(MH='Viral Fusion Protein Inhibitors', MS='Drugs that are designed to block the action of VIRAL FUSION PROTEINS and prevent VIRUSES from entering the cell.'),
  'D065148': NtO(MH='Nicolau Syndrome', MS='An uncommon complication of INTRAMUSCULAR INJECTION leading to variable degrees of necrosis of skin and underlying tissue.'),
  'D065150': NtO(MH='Cellular Reprogramming', MS='A process where fully differentiated or specialized cells revert to pluripotency or a less differentiated cell type.'),
  'D065166': NtO(MH='Sepsis-Associated Encephalopathy', MS='Acute neurological dysfunction during severe SEPSIS in the absence of direct brain infection characterized by systemic inflammation and BLOOD BRAIN BARRIER perturbation.'),
  'D065167': NtO(MH='Pregnancy, Interstitial', MS='A type of ectopic pregnancy in which the extrauterine EMBRYO IMPLANTATION occurs in the portion of the FALLOPIAN TUBE that traverses the muscular wall of the uterus.'),
  'D065168': NtO(MH='Bradykinin Receptor Antagonists', MS='Compounds and drugs that inhibit ligand binding or cellular signaling by BRADYKININ RECEPTORS.'),
  'D065169': NtO(MH='Bradykinin B1 Receptor Antagonists', MS='Compounds and drugs that inhibit ligand binding or cellular signaling by BRADYKININ B1 RECEPTORS.'),
  'D065170': NtO(MH='Pregnancy, Angular', MS='A rare type of abnormal pregnancy in which EMBRYO IMPLANTATION occurs at a lateral angle of the UTERUS, medial to the uterotubal junction and the ROUND LIGAMENT OF UTERUS.'),
  'D065171': NtO(MH='Estrogen Receptor Antagonists', MS='Compounds and drugs that bind to and block or inhibit the activation of ESTROGEN RECEPTORS.'),
  'D065172': NtO(MH='Pregnancy, Ovarian', MS='A type of pregnancy in which EMBRYO IMPLANTATION occurs in an OVARY instead of in the uterine cavity.'),
  'D065173': NtO(MH='Pregnancy, Cornual', MS='A type of pregnancy in which the EMBRYO IMPLANTATION occurs in the horn of the UTERUS instead of in the uterine cavity, i.e. at the junction of the uterus and one of the FALLOPIAN TUBES.'),
  'D065186': NtO(MH='Interrupted Time Series Analysis', MS='A study that uses observations at multiple time points before and after an intervention (the "interruption"), in an attempt to detect whether the intervention has had an effect significantly greater than any underlying trend over time.'),
  'D065187': NtO(MH='Controlled Before-After Studies', MS='A study in which observations are made before and after an intervention, both in a group that receives the intervention and in a control group that does not.'),
  'D065206': NtO(MH='Extracellular Traps', MS='Extracellular structure primarily composed of CHROMATIN and associated PROTEASES.'),
  'D065207': NtO(MH='Middle East Respiratory Syndrome Coronavirus', MS='A coronavirus first reported in the summer of 2012 in a pneumonia patient in SAUDI ARABIA. It is related to SARS VIRUS. MERS-CoV has an estimated incubation period of 12 days and symptoms include renal failure and severe acute pneumonia with often fatal outcome.'),
  'D065227': NtO(MH='Transfusion Reaction', MS='Complications of BLOOD TRANSFUSION. Included adverse reactions are common allergic and febrile reactions; hemolytic (delayed and acute) reactions; and other non-hemolytic adverse reactions such as infections and adverse immune reactions related to immunocompatibility.'),
  'D065228': NtO(MH='Non-Randomized Controlled Trials as Topic', MS='Works about a study where participants are assigned to a treatment, procedure, or intervention by methods that are not random. Non-randomized clinical trials are sometimes referred to as quasi-experimental clinical trials or non-equivalent control group designs.'),
  'D065246': NtO(MH='Culturally Competent Care', MS='Health care services that are respectful of and responsive to the health beliefs, practices and cultural and linguistic needs of diverse patients. The provider and the patient each bring their individual learned patterns of language and culture to the health care experience which must be transcended to achieve equal access and quality health care.'),
  'D065260': NtO(MH='Frankincense', MS='Resin exudate from certain trees of the genus BOSWELLIA in the BURSERACEAE family.'),
  'D065286': NtO(MH='Prophylactic Surgical Procedures', MS='Surgical procedures done to prevent or reduce the risk of developing cancer or other disease conditions in persons who may be predisposed to such conditions.'),
  'D065287': NtO(MH='Robotic Surgical Procedures', MS='Surgical procedures performed using a computer that remotely controls surgical instruments attached to mechanical arms designed to perform the tasks of the surgeon.'),
  'D065288': NtO(MH='Serogroup', MS='A set of variants within a species of microorganisms that are antigenically, closely related. With bacteria, a serogroup refers to a group that shares a common antigen.'),
  'D065289': NtO(MH='Pedicle Screws', MS='BONE SCREWS to be used in the pedicle of the vertebral arch.'),
  'D065290': NtO(MH='Acute-On-Chronic Liver Failure', MS='Sudden liver failure in the presence of underlying compensated chronic LIVER DISEASE (e.g., LIVER CIRRHOSIS; HEPATITIS; and liver injury and failure) due to a precipitating acute hepatic insult.'),
  'D065306': NtO(MH='Corneal Injuries', MS='Damage or trauma inflicted to the CORNEA by external means.'),
  'D065308': NtO(MH='Morphological and Microscopic Findings', MS='Morphological findings useful in differentiation and classification of results in CYTODIAGNOSIS and related techniques.'),
  'D065309': NtO(MH='Atypical Squamous Cells of the Cervix', MS='Morphological abnormalities of the cervical EPITHELIUM, usually revealed in PAP SMEAR, which do not meet the criteria for squamous CERVICAL INTRAEPITHELIAL NEOPLASIA or SQUAMOUS CELL CARCINOMAS of the CERVIX . It may be a sign of infection with certain types of human papillomavirus (HPV).or sign of a benign (not cancer) growth, such as a cyst or polyp or, in menopausal women, of low hormone levels. More testing, such as HPV test, may be needed.'),
  'D065310': NtO(MH='Squamous Intraepithelial Lesions of the Cervix', MS='A cytological test finding often from PAP SMEARS that shows abnormal lesions of SQUAMOUS EPITHELIAL CELLS of the CERVIX. It is a diagnostic criterion used in the Bethesda System for UTERINE CERVICAL NEOPLASMS and represents the PAP TEST result that is abnormal. Although squamous intraepithelial lesions test result does not mean UTERINE CERVICAL NEOPLASMS it requires follow-ups (e.g., HPV DNA TESTS; and COLPOSCOPY).'),
  'D065311': NtO(MH='Adenocarcinoma in Situ', MS='A lesion with cytological characteristics associated with invasive adenocarcinoma but the tumor cells are confined to the GLANDULAR EPITHELIAL CELLS of origin. Adenocarcinoma in situ of the CERVIX and the LUNG are the most common.'),
  'D065326': NtO(MH="Children's Health Insurance Program", MS="Federal program, enacted in 1997, for the funding of children's health insurance coverage at the state level for low-income families, affording them effective protection against erosions in employer sponsored coverage."),
  'D065327': NtO(MH='Circuit-Based Exercise', MS='Alternating sets of exercise that work out different muscle groups and that also alternate between aerobic and anaerobic exercises, which, when combined together, offer an overall program to improve strength, stamina, balance, or functioning.'),
  'D065346': NtO(MH='Lubricant Eye Drops', MS='Ophthalmic solutions that include LUBRICANTS and WETTING AGENTS such as POLYETHYLENE GLYCOL; HYPROMELLOSE; GLYCEROL; PROPYLENE GLYCOL. They are used to treat conditions caused by deficient tear production such as DRY EYE SYNDROME.'),
  'D065347': NtO(MH='Hypromellose Derivatives', MS='Polymeric compounds that contain repeating units of hydroxypropyl methylcellulose. The properties of hypromellose polymers can vary greatly and are defined by their molecular weight, the percentage of hydroxyl groups, the percentage of hydroxypropyl groups, and viscosity measurements. They are found a broad variety of commercial products such as FOOD ADDITIVES; EXCIPIENTS; and LUBRICANTS.'),
  'D065348': NtO(MH='Macular Pigment', MS='A yellow pigment that accumulates in the MACULA LUTEA. It is composed of LUTEIN, meso-zeaxanthin, and ZEAXANTHIN.'),
  'D065366': NtO(MH='Cryptoxanthins', MS='Mono-hydroxylated xanthophylls formed from the hydroxylation of BETA-CAROTENE. Isomers include: beta-cryptoxanthin, alpha-cryptoxanthin, and zeinoxanthin. The alpha- and beta-cryptoxanthin are provitamin A precursors.'),
  'D065386': NtO(MH='Historically Controlled Study', MS='A study that compares a group of participants receiving an intervention with a similar group from the past who did not.'),
  'D065406': NtO(MH='Laminoplasty', MS='A surgical procedure performed on selected pairs of vertebral lamina to relieve pressure on the SPINAL CORD and which also allows for osteoplastic enlargement of the SPINAL CANAL.'),
  'D065426': NtO(MH='Cytoreduction Surgical Procedures', MS='Operative procedures carried out to reduce a mass of tissue, for example, to reduce the total amount of tissue composing a tumor.'),
  'D065427': NtO(MH='Factor Xa Inhibitors', MS='Endogenous factors and drugs that inhibit or block the activity of FACTOR XA.'),
  'D065446': NtO(MH='Premenstrual Dysphoric Disorder', MS='A condition in which a woman suffers from severe depression, irritability, and tension before MENSTRUATION. Premenstrual dysphoric disorder (PMDD) may involve a wide range of physical or emotional symptoms, which are more severe and debilitating than those seen with premenstrual syndrome (PMS), and which include at least one mood-related symptom. Symptoms usually stop when, or shortly after, menstruation begins.'),
  'D065447': NtO(MH='Practice Management, Veterinary', MS='The organization and operation of the business aspects of a veterinary practice.'),
  'D065466': NtO(MH='Lysholm Knee Score', MS='An instrument used to assess the results of rehabilitation from knee injuries, especially those requiring ANTERIOR CRUCIATE LIGAMENT RECONSTRUCTION. It measures recovery of knee joint function based on ACTIVITIES OF DAILY LIVING.'),
  'D065467': NtO(MH='Transcatheter Aortic Valve Replacement', MS='Surgical treatment for severe AORTIC VALVE STENOSIS. Transcatheter aortic valve replacement (TAVR) is used as an alternative option in patients who are deemed at high risk or inoperable for traditional open-heart surgery.'),
  'D065468': NtO(MH='Dasyproctidae', MS='A family of Central and South American plant-eating rodents comprising the agoutis (Dasyprocta) and acouchis (Myoprocta). They look similar to guinea pigs (CAVIA) but are larger and have longer legs. Their fur is reddish or dark brown.'),
  'D065469': NtO(MH='Cuniculidae', MS='A family of large (20 to 30 inches long) plant eating rodents found in Central and South America. They are nearly tailless and have chunky bodies, dark brown fur with spotted lines along their sides and back, with stout legs and hind limbs that are longer than the forelimbs, and a large, square head with short ears.'),
  'D065506': NtO(MH='Vascular Closure Devices', MS='Products used to seal the openings in BLOOD VESSELS from incisions or punctures made during ENDOVASCULAR PROCEDURES.'),
  'D065507': NtO(MH='Vancomycin-Resistant Enterococci', MS='Strains of the genus Enterococcus that are resistant to the antibiotic VANCOMYCIN. The enterococci become resistant by acquiring plasmids carrying genes for VANCOMYCIN RESISTANCE.'),
  'D065508': NtO(MH='Diving Reflex', MS='BREATH HOLDING; VASOCONSTRICTION; and a drop in HEART RATE in response to submersion of the face in cold water.'),
  'D065526': NtO(MH='Acetabuloplasty', MS='Surgical reconstruction of the ACETABULUM.'),
  'D065527': NtO(MH='Brachial Plexus Block', MS='A blocking of NEURAL CONDUCTION in the network of nerve fibers innervating the UPPER EXTREMITY.'),
  'D065546': NtO(MH='Drug Liberation', MS='Release of drugs from DOSAGE FORMS into solution.'),
  'D065566': NtO(MH='Subcutaneous Absorption', MS='The uptake of substances via the SUBCUTANEOUS TISSUE.'),
  'D065567': NtO(MH='Absorption, Physiological', MS='Uptake of substances by the body, tissues, or cells. The uptake may occur by various forms of BIOLOGICAL TRANSPORT or by simple DIFFUSION.'),
  'D065568': NtO(MH='Oral Mucosal Absorption', MS='Uptake of substances into the body via the mucosal surfaces (MUCOUS MEMBRANE) of the oral cavity.'),
  'D065569': NtO(MH='Gastrointestinal Absorption', MS='Uptake of substances via the GASTROINTESTINAL TRACT.'),
  'D065570': NtO(MH='Gastric Absorption', MS='Uptake of substances via the stomach.'),
  'D065586': NtO(MH='Intramuscular Absorption', MS='Uptake of substances via muscle tissue.'),
  'D065587': NtO(MH='Ocular Absorption', MS='Uptake of substances by tissues of the eye.'),
  'D065588': NtO(MH='Nasal Absorption', MS='Uptake of substances through the nasal mucosa.'),
  'D065589': NtO(MH='Respiratory Tract Absorption', MS='Uptake of substances via the RESPIRATORY SYSTEM.'),
  'D065590': NtO(MH='Rectal Absorption', MS='Uptake of substances from the RECTUM.'),
  'D065606': NtO(MH='Metabolic Side Effects of Drugs and Substances', MS='Specific effects of drugs and substances on metabolic pathways such as those occurring through the CYTOCHROME P-450 ENZYME SYSTEM. These include effects that often result in DRUG INTERACTIONS; FOOD-DRUG INTERACTIONS; and HERB-DRUG INTERACTIONS.'),
  'D065607': NtO(MH='Cytochrome P-450 Enzyme Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 ENZYMES.'),
  'D065608': NtO(MH='Renal Reabsorption', MS='The reuptake of water, ELECTROLYTES, and other substances by the KIDNEY to the blood supply.'),
  'D065609': NtO(MH='Cytochrome P-450 CYP1A2 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP1A2.'),
  'D065626': NtO(MH='Non-alcoholic Fatty Liver Disease', MS='Fatty liver finding without excessive ALCOHOL CONSUMPTION.'),
  'D065627': NtO(MH='Familial Primary Pulmonary Hypertension', MS='Familial or idiopathic hypertension in the PULMONARY CIRCULATION which is not secondary to other disease.'),
  'D065628': NtO(MH='Intestinal Reabsorption', MS='The re-uptake of substances in the intestines after their prior absorption from the intestines into the PORTAL SYSTEM and reintroduction into the intestines as substances accumulated in the BILE.'),
  'D065629': NtO(MH='Peritoneal Absorption', MS='Uptake of substances from the PERITONEAL CAVITY.'),
  'D065630': NtO(MH='Hernias, Diaphragmatic, Congenital', MS='Protrusion of abdominal structures into the THORAX as a result of embryologic defects in the DIAPHRAGM often present in the neonatal period. It can be isolated, syndromic, nonsyndromic or be a part of chromosome abnormality. Associated pulmonary hypoplasia and PULMONARY HYPERTENSION can further complicate stabilization and surgical intervention.'),
  'D065631': NtO(MH='Rhinitis, Allergic', MS='An inflammation of the NASAL MUCOSA triggered by ALLERGENS.'),
  'D065632': NtO(MH='Chikungunya Fever', MS='An acute infection caused by a mosquito-borne alphavirus CHIKUNGUNYA VIRUS characterized by RASH; FEVER; JOINT PAINS; CONJUNCTIVITIS; MENINGOENCEPHALITIS; LYMPHOPENIA; and THROMBOCYTOPENIA.'),
  'D065633': NtO(MH='Cytochrome P-450 CYP1B1', MS="A cytochrome P450 aryl hydrocarbon hydroxylase that has specificity for ESTROGENS which it converts into 4-hydroxy estrogens. CYP1B1 has been shown to be physiologically important for FETAL DEVELOPMENT with mutations in the CYP1B1 gene resulting in congenital forms of GLAUCOMA and Peter's anomaly."),
  'D065634': NtO(MH='Cerebrospinal Fluid Leak', MS='Discharge of cerebrospinal fluid through a hole through the skull bone most commonly draining from the nose (CEREBROSPINAL FLUID RHINORRHEA) or the ear (CEREBROSPINAL FLUID OTORRHEA).'),
  'D065635': NtO(MH='Benign Paroxysmal Positional Vertigo', MS='Idiopathic recurrent VERTIGO associated with POSITIONAL NYSTAGMUS. It is associated with a vestibular loss without other neurological or auditory signs. Unlike in LABYRINTHITIS and VESTIBULAR NEURONITIS, inflammation in the ear is not observed.'),
  'D065636': NtO(MH='Myotonin-Protein Kinase', MS="Serine/threonine protein kinase responsible for various SKELETAL MUSCLE functions; HEART CONDUCTION SYSTEM activity; calcium HOMEOSTASIS; calcium uptake by SARCOPLASMIC RETICULUM and SYNAPTIC PLASTICITY. It is encoded by the DMPK gene and its abnormal EXPANDED TRINUCLEOTIDE REPEAT of CTG in the 3'-UTR is associated with MYOTONIC DYSTROPHY 1."),
  'D065637': NtO(MH='Cytochrome P-450 CYP2A6', MS='A cytochrome P450 enzyme that has specificity for the 7-hydroxylation of coumarin.'),
  'D065638': NtO(MH='Absorption, Radiation', MS='The uptake or transfer of energy from RADIATION.'),
  'D065646': NtO(MH='Thyroid Carcinoma, Anaplastic', MS='An aggressive THYROID GLAND malignancy which generally occurs in IODINE-deficient areas in people with previous thyroid pathology such as GOITER. It is associated with CELL DEDIFFERENTIATION of THYROID CARCINOMA (e.g., FOLLICULAR THYROID CARCINOMA; PAPILLARY THYROID CANCER). Typical initial presentation is a rapidly growing neck mass which upon metastasis is associated with DYSPHAGIA; NECK PAIN; bone pain; DYSPNEA; and NEUROLOGIC DEFICITS.'),
  'D065666': NtO(MH='Mesenteric Ischemia', MS='Ischemic tissue injury produced by insufficient perfusion of intestinal tissue by the MESENTERIC CIRCULATION (i.e., CELIAC ARTERY; SUPERIOR MESENTERIC ARTERY; INFERERIOR MESENTERIC ARTERY; and MESENTERIC VEINS). It can progress from ISCHEMIA; EDEMA; and GANGRENE of the bowel wall to PERITONITIS and cardiovascular collapse.'),
  'D065667': NtO(MH='Renal Elimination', MS='The discharge of substances from the blood supply via the URINARY TRACT.'),
  'D065668': NtO(MH='Vitamin D3 24-Hydroxylase', MS='A cytochrome P-450 enzyme that has specificity for CHOLECALCIFEROL (Vitamin D3). It hydroxylates the molecule at carbon position 24.'),
  'D065669': NtO(MH='Hepatobiliary Elimination', MS='The removal of substances from the blood supply via the LIVER and BILIARY TRACT.'),
  'D065686': NtO(MH='Cytochrome P-450 CYP2B6 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP2B6.'),
  'D065687': NtO(MH='Cytochrome P-450 CYP2C8 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP2C8.'),
  'D065688': NtO(MH='Cytochrome P-450 CYP2C9 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP2C9.'),
  'D065689': NtO(MH='Cytochrome P-450 CYP2C19 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP2C19.'),
  'D065690': NtO(MH='Cytochrome P-450 CYP2D6 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP2D6.'),
  'D065691': NtO(MH='Cytochrome P-450 CYP2E1 Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP2E1.'),
  'D065692': NtO(MH='Cytochrome P-450 CYP3A Inhibitors', MS='Drugs and compounds which inhibit or antagonize the biosynthesis or actions of CYTOCHROME P-450 CYP3A.'),
  'D065693': NtO(MH='Cytochrome P-450 Enzyme Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 ENZYMES.'),
  'D065694': NtO(MH='Cytochrome P-450 CYP1A2 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP1A2.'),
  'D065695': NtO(MH='Cytochrome P-450 CYP2B6 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP2B6.'),
  'D065696': NtO(MH='Cytochrome P-450 CYP2C8 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP2C8.'),
  'D065697': NtO(MH='Cytochrome P-450 CYP2C19 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP2C19.'),
  'D065698': NtO(MH='Cytochrome P-450 CYP2C9 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP2C9.'),
  'D065699': NtO(MH='Cytochrome P-450 CYP2D6 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP2D6.'),
  'D065700': NtO(MH='Cytochrome P-450 CYP2E1 Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP2E1.'),
  'D065701': NtO(MH='Cytochrome P-450 CYP3A Inducers', MS='Drugs and compounds that induce the synthesis of CYTOCHROME P-450 CYP3A.'),
  'D065702': NtO(MH='Cytochrome P-450 CYP2B6', MS='A cytochrome P450 enzyme subtype that oxidizes a diverse array of XENOBIOTICS. The expression of CYP2B6 varies widely between individuals which is due to the high rate of GENETIC POLYMORPHISMS.  Examples of drugs metabolized by CYP2B6 include BUPROPION; efavirenz; CYCLOPHOSPHAMIDE; and MEPERIDINE.'),
  'D065703': NtO(MH='Malformations of Cortical Development, Group I', MS='Cortical malformations secondary to abnormal neuronal and glial CELL PROLIFERATION or APOPTOSIS in NEUROGENESIS. This group includes congenital MICROCEPHALIES; MICROLISSENCEPHALIES, megalencephalies, HEMIMEGALENCEPHALIES and cortical dysplasias with balloon cells.'),
  'D065704': NtO(MH='Malformations of Cortical Development, Group III', MS='Cortical malformations secondary to abnormal cortical maturation after CELL MIGRATION in NEUROGENESIS. This group includes injury to the cortex during later stages of cortical development such as POLYMICROGYRIA and focal cortical dysplasias.'),
  'D065705': NtO(MH='Hemimegalencephaly', MS='Rare MALFORMATIONS OF CORTICAL DEVELOPMENT, GROUP I characterized by the enlargement of one side of the brain. It is associated with seizures, partial paralysis, and mental retardation.'),
  'D065706': NtO(MH='Polymicrogyria', MS='Heterogeneous disorders of cortical malformation characterized by excessive and small fused gyri and shallow sulci of the CORTEX with abnormal cortical lamination. It is considered a malformation secondary to abnormal post-migrational development of the neurons during cerebral cortical development and is associated with EPILEPSY and learning difficulties.'),
  'D065707': NtO(MH='Schizencephaly', MS='Cortical malformations characterized by grey matter-lined cleft or cyst that extends from the EPENDYMA often to the PIA MATER outer surface. The grey matter that lines the cleft is often POLYMICROGYRIA. It is associated with developmental delay, motor disturbance and seizures.'),
  'D065708': NtO(MH='Porencephaly', MS='Cortical malformations characterized by white matter-lined cleft or cyst associated with ISCHEMIA and hemorrhagic insults. Symptoms include delayed growth and development, HYPOTONIA; SEIZURES; SPASTIC HEMIPLEGIA and MACROCEPHALY; MICROCEPHALY; or HYDROCEPHALUS. Mutations in the genes encoding COLLAGEN TYPE IV are associated with familial types.'),
  'D065709': NtO(MH='Common Data Elements', MS='Data elements that are common to multiple data sets across different studies.'),
  'D065710': NtO(MH='Glossoptosis', MS='Posterior displacement of the TONGUE toward the PHARYNX. It is often a feature in syndromes such as in PIERRE ROBIN SYNDROME and DOWN SYNDROME and associated with AIRWAY OBSTRUCTION during sleep (OBSTRUCTIVE SLEEP APNEAS).'),
  'D065711': NtO(MH='Broca Area', MS='Functional neuroanatomical region of the inferior frontal gyrus consisting of pars opercularis and pars triangularis. It is important in SPEECH and LANGUAGE production. Injuries to Broca area are associated with BROCA APHASIA and APRAXIA.'),
  'D065726': NtO(MH='Limbic Lobe', MS='The medial surface of the cerebral hemisphere around the brain stem.'),
  'D065727': NtO(MH='Cytochrome P-450 CYP2C8', MS='A liver microsomal cytochrome P450 hydroxylase that oxidizes a broad spectrum of substrates including STEROIDS, FATTY ACIDS, and XENOBIOTICS. Examples of pharmaceutical substrates for CYP2C8 include; PACLITAXOL; torsemide; and; AMODIAQUINE'),
  'D065729': NtO(MH='Cytochrome P-450 CYP2C9', MS='A cytochrome P-450 subtype that has specificity for acidic XENOBIOTICS. It oxidizes a broad range of important clinical drugs that fall under the categories of NONSTEROIDAL ANTI-INFLAMMATORY AGENTS; HYPOGLYCEMIC AGENTS; ANTCOAGULANTS; and DIURETICS.'),
  'D065730': NtO(MH='Limonene Hydroxylases', MS='Members of the P-450 enzyme family that take part in the hydroxylation of limonene.'),
  'D065731': NtO(MH='Cytochrome P-450 CYP2C19', MS='A cytochrome P-450 enzyme subtype that oxidizes several important groups of drugs including many PROTON PUMP INHIBITORS and ANTICONVULSANTS.'),
  'D065748': NtO(MH='Euterpe', MS='A genus in the family Arecaceae. They are a source of ACAI BERRIES.'),
  'D065749': NtO(MH='Phoeniceae', MS='Tribe of palms in the subfamily Coryphoideae, family Arecaceae.'),
  'D065766': NtO(MH='Atypical Hemolytic Uremic Syndrome', MS='An hereditary hemolytic uremic syndrome associated with variations in the gene that encodes COMPLEMENT FACTOR H, or the related proteins CFHR1 and CFHR3. Disease often progresses to CHRONIC KIDNEY FAILURE without the prodromal symptoms of ENTEROCOLITIS and DIARRHEA that characterize typical hemolytic uremic syndrome.'),
  'D065767': NtO(MH='Activation, Metabolic', MS='The conversion of an exogenous substance by a biological system resulting in the production on an active metabolite.'),
  'D065768': NtO(MH='Lennox Gastaut Syndrome', MS='A childhood-onset epilepsy syndrome.'),
  'D065786': NtO(MH='Delay Discounting', MS='The tendency to devalue an outcome as a function of its temporal delay or probability of achievement. It can be evaluated in a psychological paradigm that involves the choice between receiving a smaller immediate reward or a larger delayed reward, and may be used to provide a measure of impulsive behavior.'),
  'D065787': NtO(MH='Salivary Elimination', MS='The discharge of substances from the blood supply via diffusion into the SALIVA from the ORAL MUCOSA or from the SALIVARY GLANDS.'),
  'D065806': NtO(MH='Xenophobia', MS='Intense or irrational dislike or fear of anything that is strange or foreign, particularly of people who are strangers or foreigners. It is an attitudinal orientation of hostility against non-natives in a given population that includes attitudes, prejudices and behavior that reject, exclude or vilify persons, based on the perception that they are outsiders or foreigners to the community, society or national identity.'),
  'D065808': NtO(MH='Prepulse Inhibition', MS='A neurophysiological phenomenon in which the response to a startling stimulus (pulse) is decreased when a weaker prestimulus (prepulse) precedes it closely in time. It is used as an operational measure of sensorimotor gating.'),
  'D065809': NtO(MH='Respiratory Sinus Arrhythmia', MS='A naturally occurring variation in heart rate that occurs during a BREATHING cycle, where the time between heartbeats is shortened during inspiration and prolonged during expiration. It is not the same as SINUS ARRHYTHMIA, the disorder.'),
  'D065811': NtO(MH='Ethnic Cleansing', MS='Expulsion of a population from a given territory due to religious or ethnic discrimination, political, strategic or ideological considerations, or a combination of these.'),
  'D065812': NtO(MH='Interoception', MS='The perception of and sensitivity to stimuli originating inside of the body. It is a sense of the physiological condition of the entire body.'),
  'D065813': NtO(MH='Wernicke Area', MS='Functional region comprising posterior part of the SUPERIOR TEMPORAL GYRUS in the dominant cerebral hemisphere (see CEREBRAL DOMINANCE) and often portions of the PARIETAL LOBE. Along with BROCA AREA it is important in SPEECH and LANGUAGE processes. A lesion in the area is associated with WERNICKE APHASIA and CONDUCTION APHASIA.'),
  'D065814': NtO(MH='Cerebellar Vermis', MS='A subregion of the CEREBELLAR CORTEX, located along the midsagittal plane of the CEREBELLUM.'),
  'D065815': NtO(MH='Acalypha', MS='Nettle-like plants in the subfamily Acalyphoideae. Acalypha is the largest genus in the Euphorbiaceae family.'),
  'D065816': NtO(MH='Neuroscience Nursing', MS='The nursing specialty focusing on the needs of patients with dysfunction of the NERVOUS SYSTEM.'),
  'D065817': NtO(MH='Congenital Microtia', MS='Malformation of external portion of EAR AURICLE.'),
  'D065818': NtO(MH='Gamma Rhythm', MS='Brain waves characterized by a relatively high voltage or amplitude and a frequency of approximately 30-100 Hz. They are primarily observed during network engagement and sensory processing activities, during both waking and sleeping states.'),
  'D065819': NtO(MH='Voriconazole', MS='A triazole antifungal agent that specifically inhibits STEROL 14-ALPHA-DEMETHYLASE and CYTOCHROME P-450 CYP3A.'),
  'D065820': NtO(MH='Zona Incerta', MS='A region of the subthalamus located ventral to the THALAMUS and dorsal to the SUBTHALAMIC NUCLEUS.'),
  'D065821': NtO(MH='Pontine Tegmentum', MS='The dorsal region of pons bounded by the middle CEREBELLAR PEDUNCLES and the ventral part of the pons. It is continuous with the TEGMENTUM MESENCEPHALI and contains CEREBELLAR NUCLEI, lemnisci and RETICULAR FORMATION.'),
  'D065822': NtO(MH='College Fraternities and Sororities', MS='Local or national organizations of college students, primarily for social purposes, usually with secret initiations and rites and a name composed of two or three Greek letters.'),
  'D065823': NtO(MH='Parabrachial Nucleus', MS='A cell group in the pontine tegmentum surrounding the surfaces of the superior CEREBRAL PEDUNCLE of the PONS.'),
  'D065825': NtO(MH='Oxalidaceae', MS='Wood-sorrel family in the order Oxalidales. Some species of RUMEX (of the family POLYGONACEAE) also known by the common name of sorrels.'),
  'D065826': NtO(MH='Averrhoa', MS='Tropical trees in the family Oxalidaceae'),
  'D065827': NtO(MH='Abducens Nucleus', MS='A region in the PONTINE TEGMENTUM that is the nucleus of the ABDUCENS NERVE.'),
  'D065828': NtO(MH='Facial Nucleus', MS='A region in the pontine tegmentum that is the nucleus of the FACIAL NERVE.'),
  'D065829': NtO(MH='Flower Essences', MS='Aqueous extracts of flowers that are used in healing systems.'),
  'D065830': NtO(MH='Monks', MS='Members of a religious community of men typically living under vows of poverty, chastity, and obedience.'),
  'D065831': NtO(MH='Nuns', MS='Members of a religious community of women typically living under vows of poverty, chastity, and obedience.'),
  'D065832': NtO(MH='Superior Olivary Complex', MS='An auditory brainstem structure in the pontine tegmentum that contains several nuclei associated with auditory localization and is component of ascending and descending auditory pathways.'),
  'D065833': NtO(MH='Trapezoid Body', MS='A region in the pontine tegmentum containing nuclei and fibers associated with AUDITORY PATHWAYS.'),
  'D065834': NtO(MH='Religious Personnel', MS='Persons that live and work according to a religious vocation, or calling, to a consecrated or ordained life.'),
  'D065835': NtO(MH="Barrington's Nucleus", MS='A nucleus in the pons with exon projections to the sacral parasympathetic nucleus associated with bladder control.'),
  'D065836': NtO(MH='Kolliker-Fuse Nucleus', MS='A nucleus in the pons associated with RESPIRATION RATE control.'),
  'D065837': NtO(MH='Middle Cerebellar Peduncle', MS='WHITE MATTER consisting of a large bundle of paired fibers originating in the pontine nuclei in the pre pontis and the nucleus reticularis tegmenti pontis projecting to the contralateral cerebral cortex.'),
  'D065838': NtO(MH='Oculomotor Nuclear Complex', MS='A collection of nuclei in the midbrain tegmentum for the OCULOMOTOR NERVE fibers.'),
  'D065839': NtO(MH='Edinger-Westphal Nucleus', MS='Nucleus in the oculomotor nuclear complex associated with oculomotor activities and stress adaptation.'),
  'D065840': NtO(MH='Protective Factors', MS='An aspect of personal behavior or lifestyle, environmental exposure, or inborn or inherited characteristic, which, on the basis of epidemiologic evidence, is known to be associated with prevention or mitigation of a health-related condition considered important to prevent.'),
  'D065841': NtO(MH='Pars Reticulata', MS='A region in the substantia nigra located ventral and lateral to the PARS COMPACTA.'),
  'D065842': NtO(MH='Pars Compacta', MS='A region in the substantia nigra located dorsal to the PARS RETICULATA.'),
  'D065843': NtO(MH='Cerebral Crus', MS='Anterior portion of the cerebral peduncle which contains the motor tracts.'),
  'D065844': NtO(MH='Tectospinal Fibers', MS='A neural tract which connects the TECTUM MESENCEPHALI and the SPINAL CORD and is involved in head and eye movement coordination.'),
  'D065846': NtO(MH='Nucleus Raphe Magnus', MS='A nucleus located directly rostral to the NUCLEUS RAPHE OBSCURUS in the pontine part of the Raphe nuclei. Another pontine nucleus located directly rostral to the magnus nucleus is the nucleus raphe pontis.'),
  'D065847': NtO(MH='Dorsal Raphe Nucleus', MS='The dorsal region of the raphe nucleus which is a source of the SEROTONERGIC NEURONS innervating the TELENCEPHALON; DIENCEPHALON; MESENCEPHALON; and RHOMBENCEPHALON.'),
  'D065848': NtO(MH='Nucleus Raphe Pallidus', MS='A nucleus in the medullary Raphe nuclei located rostral to the NUCLEUS RAPHE OBSCURUS and caudal to the NUCLEUS RAPHE MAGNUS.'),
  'D065849': NtO(MH='Nucleus Raphe Obscurus', MS='A nucleus in the medullary Raphe nuclei located caudal to the NUCLEUS RAPHE PALLIDUS.'),
  'D065850': NtO(MH='Cerebral Peduncle', MS='Ventral portion of the mesencephalon including the MIDBRAIN TEGMENTUM; CRUS CEREBRI; and the PRETECTUM. It does not include tectum mesencephali and TROCHLEAR NERVE.'),
  'D065851': NtO(MH='Ovarian Reserve', MS='Capacity of the ovary to provide viable eggs.'),
  'D065852': NtO(MH='Spatial Memory', MS='Memory related to spatial information.'),
  'D065853': NtO(MH='Spatial Learning', MS='A process through which individuals encode information about their environmental CUES to facilitate SPATIAL NAVIGATION.'),
  'D065854': NtO(MH='Spatial Navigation', MS='Process by which organisms use various cues to develop a route to a destination and their ability to travel such a planned route.'),
  'D065855': NtO(MH='Spatial Processing', MS='Integration of spatial information perceived by visual and/or auditory CUES.'),
  'D065856': NtO(MH='Rubus', MS='A plant genus of the family Rosaceae whose members include blackberries and raspberries.'),
  'D065866': NtO(MH='Optic Tract', MS='Nerve fiber originating from the OPTIC CHIASM that connects predominantly to the LATERAL GENICULATE BODIES. It is the continuation of the VISUAL PATHWAY that conveys the visual information originally from the RETINA to the optic chiasm via the OPTIC NERVES.'),
  'D065886': NtO(MH='Neurodevelopmental Disorders', MS='These are a group of conditions with onset in the developmental period. The disorders typically manifest early in development, often before the child enters grade school, and are characterized by developmental deficits that produce impairments of personal, social, academic, or occupational functioning.  (From DSM-5).'),
  'D065906': NtO(MH='Hyperlactatemia', MS='Increase in blood LACTATE concentration often associated with SEPTIC SHOCK; LUNG INJURY; SEPSIS; and DRUG TOXICITY. When hyperlactatemia is associated with low body pH (acidosis) it is LACTIC ACIDOSIS.'),
  'D065907': NtO(MH='Karoshi Death', MS='Sudden death from overwork, most often as a result of acute CARDIOVASCULAR STROKE.'),
  'D065908': NtO(MH='Transcranial Direct Current Stimulation', MS='A technique of brain electric stimulation therapy which uses constant, low current delivered via ELECTRODES placed on various locations on the scalp.'),
  'D065926': NtO(MH='Influenza A Virus, H10N8 Subtype', MS='A subtype of INFLUENZA A VIRUS comprised of the surface proteins HEMAGGLUTININ 10 and NEURAMINIDASE 8. The H10N8 subtype usually infects domestic birds (POULTRY) but there have been some human infections reported.'),
  'D065927': NtO(MH='Waist-Height Ratio', MS='The ratio of WAIST CIRCUMFERENCE to BODY HEIGHT.'),
  'D065928': NtO(MH='Forests', MS='An ecosystem dominated by trees and other woody vegetation.'),
  'D065929': NtO(MH='Tundra', MS='An ecosystem dominated by grasses, lichens, mosses, sedges and shrubs in cold landscapes or at high elevations where a layer of the ground remains perennially frozen (PERMAFROST).'),
  'D065946': NtO(MH='Permafrost', MS='Perennially frozen layer of soil below the surface in tundra.'),
  'D065947': NtO(MH='Rainforest', MS='Forests with high average annual rainfall.'),
  'D065948': NtO(MH='Grassland', MS='An ecosystem dominated by grasses (POACEAE) with few trees or shrubs.'),
  'D065966': NtO(MH='Absorption, Physicochemical', MS='The physical process of passing into, or being taken up or taken in by a substance.'),
  'D065986': NtO(MH='Citrullus colocynthis', MS='A plant species of the family CUCURBITACEAE distributed tropically and subtropically mainly in North Africa and Asia. It is a medicinal plant of TRADITIONAL MEDICINE and well known in ancient medicine as the wild gourd.'),
  'D066007': NtO(MH='Toxicokinetics', MS="The quantitation of the body's metabolism of toxic xenobiotic compounds, as measured by the plasma concentration of the toxicant at various time points."),
  'D066008': NtO(MH='Cervical Plexus Block', MS='A blocking of NEURAL CONDUCTION in the network of nerve fibers innervating the neck and shoulder.'),
  'D066026': NtO(MH='Endothelial Progenitor Cells', MS='Cells derived from BONE MARROW that circulate in the adult bloodstream and possess the potential to proliferate and differentiate into mature ENDOTHELIAL CELLS.'),
  'D066027': NtO(MH='Transplant Recipients', MS='Individuals receiving tissues or organs transferred from another individual of the same or different species, or from within the same individual.'),
  'D066046': NtO(MH='Diet, Paleolithic', MS='A nutritional plan based on the presumed diet of pre-agricultural human ancestors. It consists mainly of MEAT, EGGS, NUTS, roots and fresh VEGETABLES and FRUITS, and excludes GRAIN, LEGUMES, DAIRY PRODUCTS, and refined DIETARY SUGARS.'),
  'D066047': NtO(MH='Vincetoxicum', MS='A plant genus of the dogbane family ASCLEPIADACEAE.'),
  'D066066': NtO(MH='Response Evaluation Criteria in Solid Tumors', MS='An internationally recognized set of published rules used for evaluation of cancer treatment that define when tumors found in cancer patients improve, worsen, or remain stable during treatment. These criteria are based specifically on the response of the tumor(s) to treatment, and not on the overall health status of the patient resulting from treatment.'),
  'D066067': NtO(MH='Military Family', MS='Men and women serving on active duty in the military, or in a reserve military force, and their immediate family including spouses, children, and parents.'),
  'D066086': NtO(MH='Health Services for Persons with Disabilities', MS='Health services and access for DISABLED PERSONS. Disabilities include impairments, activity limitations, or participation restrictions, and reflect the interaction between the individual with the disability and personal and environmental factors.'),
  'D066087': NtO(MH='Perinatal Death', MS='The death of a FETUS of GESTATIONAL AGE 28 weeks or more, or the death of a live-born INFANT less than 28 days of age.'),
  'D066088': NtO(MH='Infant Death', MS='The death of a live-born INFANT within its first year of life.'),
  'D066106': NtO(MH='Seoul', MS='The capital of SOUTH KOREA.'),
  'D066107': NtO(MH='Social Skills', MS='The personal set of abilities required to successfully interact and communicate with others, both verbally and non-verbally through gestures, body language and personal appearance.'),
  'D066126': NtO(MH='Cardiotoxicity', MS='Damage to the HEART or its function secondary to exposure to toxic substances such as drugs used in CHEMOTHERAPY; IMMUNOTHERAPY; or RADIATION.'),
  'D066127': NtO(MH='White Matter', MS='The region of CENTRAL NERVOUS SYSTEM that appears lighter in color than the other type, GRAY MATTER. It mainly consists of MYELINATED NERVE FIBERS and contains few neuronal cell bodies or DENDRITES.'),
  'D066128': NtO(MH='Gray Matter', MS='Region of CENTRAL NERVOUS SYSTEM that appears darker in color than the other type, WHITE MATTER. It is composed of neuronal CELL BODIES; NEUROPIL; GLIAL CELLS and CAPILLARIES but few MYELINATED NERVE FIBERS.'),
  'D066146': NtO(MH='Cell Body', MS='The portion of a cell that contains the NUCLEUS and CYTOPLASM but not CELL SURFACE EXTENSIONS such as FLAGELLA and NEURITES.'),
  'D066148': NtO(MH='Spinal Cord Dorsal Horn', MS='One of three central columns of the spinal cord. It is composed of GRAY MATTER spinal laminae I-VI.'),
  'D066149': NtO(MH='International Planned Parenthood Federation', MS='A non-profit organization concerned with providing access to sexual and reproductive health information and services.'),
  'D066150': NtO(MH='Organisation for Economic Co-Operation and Development', MS='An international organization whose mission is to stimulate world trade and economic progress by offering a forum where governments can collectively develop and refine social and economic policies.'),
  'D066151': NtO(MH='Spinal Cord Ventral Horn', MS='One of three central columns of the spinal cord. It is composed of GRAY MATTER spinal laminae VIII and IX.'),
  'D066152': NtO(MH='Spinal Cord Lateral Horn', MS='One of three central columns of the spinal cord. It is composed of GRAY MATTER and is located laterally in lamina VII.'),
  'D066166': NtO(MH='Pectus Carinatum', MS='A developmental anomaly characterized by abnormal anterior protrusion of the STERNUM and adjacent COSTAL CARTILAGE.'),
  'D066167': NtO(MH='Slit Lamp', MS='A microscope with a light source that can be projected into a linear beam. It allows cross-sectional viewing of the AQUEOUS HUMOR; CONJUNCTEIVA; CORNEA; EYELIDS; IRIS; and lens (see LENS, EYE) of the eye.'),
  'D066186': NtO(MH='Costal Cartilage', MS='Bars of HYALINE CARTILAGE extending from the anterior ends of the upper RIBS. They attach to the STERNUM directly or indirectly by connecting to upper costal cartilage that attaches to the STERNUM. Costal cartilages extend the ribs anteriorly and contribute to the enhanced elasticity of the THORACIC WALL.'),
  'D066187': NtO(MH='Basal Forebrain', MS='The region of the telencephalon located rostral and ventral to the STRIATUM comprising AMYGDALA; SEPTAL NUCLEI; SUBSTANTIA INNOMINATA and ventral pallidum.'),
  'D066188': NtO(MH='Taiga', MS='An ecosystem dominated by coniferous forests including the tree line bordered by TUNDRA to the north and STEPPE to the south.'),
  'D066189': NtO(MH='Missionaries', MS='Persons sent into an area to evangelize or to perform acts of service, such as medical care, EDUCATION, or SOCIAL JUSTICE.'),
  'D066190': NtO(MH='Allesthesia', MS='A neurological disorder in which a sensory stimulus, usually tactile but more rarely other sensory modalities, is misperceived in a location distant from the original stimulus.'),
  'D066191': NtO(MH='Sensorimotor Cortex', MS='A composite area of the cerebral cortex concerned with motor control and sensory perception comprising the motor cortex areas, the somatosensory areas, the gustatory cortex, the olfactory areas, the auditory cortex, and the visual cortex.'),
  'D066192': NtO(MH='Manufacturing Industry', MS='An INDUSTRY which specializes in the fabrication or manufacture of products from raw or intermediate materials.'),
  'D066193': NtO(MH='Cervical Cord', MS='The segment of the spinal cord within the CERVICAL VERTEBRAE.'),
  'D066194': NtO(MH='Olfactory Cortex', MS='Basal forebrain and medial part of temporal lobe areas that receive synaptic inputs from the OLFACTORY BULB.'),
  'D066195': NtO(MH='Piriform Cortex', MS='An area of the olfactory cortex comprising the rostral half of the uncus of the PARAHIPPOCAMPAL GYRUS and the anterior PARAHIPPOCAMPAL GYRUS. It receives major afferents directly from the OLFACTORY BULB.'),
  'D066208': NtO(MH='Olfactory Tubercle', MS='Region in the ventral TELENCEPHALON located anterior to the OPTIC CHIASM, posterior to the OLFACTORY PEDUNCLE, rostral to the PIRIFORM AREA and ventral to the NUCLEUS ACCUMBENS.'),
  'D066209': NtO(MH='Medical Writing', MS='The practice of writing usually by a skilled or specialized writer focused on the reporting or dissemination of medical information for a target audience.'),
  'D066228': NtO(MH='Vaginal Absorption', MS='The uptake of substances from the VAGINA via the vaginal epithelium/mucosa.'),
  'D066229': NtO(MH='Speech Sound Disorder', MS='Persistent difficulty with speech sound production such that it interferes with verbal communication.'),
  'D066230': NtO(MH='Patient-Specific Modeling', MS='The development and application of computational models of human pathophysiology that are individualized to patient-specific data.'),
  'D066231': NtO(MH='Surgeons', MS='Medical practitioners qualified to practice surgery.'),
  'D066232': NtO(MH='Pulmonary Elimination', MS='The discharge of substances from the pulmonary capillary blood into the alveolar space to be exhaled or removed by MUCOCILIARY CLEARANCE.'),
  'D066233': NtO(MH='Psychology, Developmental', MS='Field of study concerned with age-related changes in BEHAVIOR that occur in human beings over the course of life, from birth to death. It includes all aspects of human growth, including emotional, intellectual, social, perceptual and personality development.'),
  'D066234': NtO(MH='Lacrimal Elimination', MS='The discharge of substances into the TEARS.'),
  'D066235': NtO(MH='Fluorine-19 Magnetic Resonance Imaging', MS='Magnetic resonance imaging diagnostic technique which uses fluorine-19 perfluorocarbon probes and contrast agents for high resolution image detection and quantitation.'),
  'D066236': NtO(MH='Bioresonance Therapy', MS="A form of holistic medicine in which ELECTROMAGNETIC WAVES are used to diagnose and treat human and animal illness via manipulation of endogenous electromagnetic oscillations corresponding to the patient's current physiological condition."),
  'D066237': NtO(MH='Intestinal Elimination', MS='The discharge of substances from the blood supply into the intestinal lumen for excretion by DEFECATION.'),
  'D066238': NtO(MH='Cutaneous Elimination', MS='The discharge of substances from the blood supply to the glands of the skin and excretion in the SWEAT and SEBUM.'),
  'D066239': NtO(MH='Lacteal Elimination', MS='The discharge of substances from the blood supply into the milk formed in and secreted by the MAMMARY GLAND.'),
  'D066240': NtO(MH='Anterior Commissure, Brain', MS='Bundle of MYELINATED NERVE FIBERS that connects the two CEREBRAL HEMISPHERES in the LAMINA TERMINALIS. It is located ventral to the SEPTAL NUCLEI, and caudal to the median PREOPTIC NUCLEUS.'),
  'D066241': NtO(MH='Carbon-13 Magnetic Resonance Spectroscopy', MS='Magnetic resonance spectroscopy involving detection of changes in the resonance of carbon-13 nuclei in organic molecules.'),
  'D066243': NtO(MH='Posterior Cerebellar Commissure', MS='A bundle of nerve fibers that crosses the midline in the tectum mesencephali dorsal to the CEREBRAL AQUEDUCT.'),
  'D066244': NtO(MH='Proton Magnetic Resonance Spectroscopy', MS='Spectroscopy technique which measures changes in organic compounds by tracking the spectral energy of absorption of HYDROGEN atoms.'),
  'D066245': NtO(MH='Telencephalic Commissures', MS='Set of nerve fibers that cross the midline of the TELENCEPHALON. They include the ANTERIOR COMMISSURE; the CORPUS CALLOSUM; and the HIPPOCAMPAL COMMISSURE of the fornix.'),
  'D066246': NtO(MH='ErbB Receptors', MS='A family of structurally-related cell-surface receptors that signal through an intrinsic PROTEIN-TYROSINE KINASE. The receptors are activated upon binding of specific ligands which include EPIDERMAL GROWTH FACTORS, and NEUREGULINS.'),
  'D066247': NtO(MH='Receptor, ErbB-4', MS='A cell surface protein-tyrosine kinase receptor that has specificity for EPIREGULIN; BETACELLIN; NEUREGULINS; and HEPARIN-BINDING EGF-LIKE GROWTH-FACTOR.'),
  'D066248': NtO(MH='High Fructose Corn Syrup', MS='Syrup made from corn used widely in foods and beverages as a cheaper alternative sweetener to SUCROSE (common table sugar). It is generated by enzymatic processing of natural corn syrup to produce a liquid most widely composed of 42 or 55% FRUCTOSE, GLUCOSE, and various POLYSACCHARIDES.'),
  'D066249': NtO(MH='Craving', MS='An intense, urgent, or abnormal desire or longing (for food, drugs, alcohol, etc.)'),
  'D066250': NtO(MH='Pretectal Region', MS='Region of midbrain tectum located posterior to the THALAMUS and anterior to the SUPERIOR COLLICULUS.'),
  'D066251': NtO(MH='Subtractive Hybridization Techniques', MS='Methods of studying differential GENE EXPRESSION of especially low-abundance, tissue-specific gene transcripts.'),
  'D066252': NtO(MH='Sociological Factors', MS="Attributes and experiences that influence an individual's personality, attitudes, and lifestyle, which are derived from the social values and/or behavior of the groups to which an individual belongs, or aspires to belong."),
  'D066253': NtO(MH='Vascular Remodeling', MS='The active alterations of vascular wall structures, often leading to elevated VASCULAR RESISTANCE. It is associated with AGING; ATHEROSCLEROSIS; DIABETES MELLITUS; HYPERTENSION; PREGNANCY; PULMONARY HYPERTENSION; and STROKE, but is also a normal part of EMBRYOGENESIS.'),
  'D066254': NtO(MH='Axillofemoral Bypass Grafting', MS='An extra-anatomic method of surgical revascularization in which the AXILLARY ARTERY is anastomosed to FEMORAL ARTERY.'),
  'D066255': NtO(MH='EGF Family of Proteins', MS='A family of proteins that are related to epidermal growth factor. They share in common a consensus sequence consisting of six spatially conserved CYSTEINE residues which form three intramolecular bonds. This consensus sequence is commonly referred to EGF motif and is considered essential for binding of the proteins to ERB RECEPTORS.'),
  'D066256': NtO(MH='Raw Foods', MS='Food prepared for consumption without exposure to heat.'),
  'D066257': NtO(MH='Heparin-binding EGF-like Growth Factor', MS='An EGF family member that is expressed in a variety of hematopoietic, endothelial, vascular smooth muscle, and epithelial cells. It is synthesized as a transmembrane protein which is cleaved by proteases to produce the secreted form of the protein which has specificity for the EGF RECEPTOR and the ERBB-4 RECEPTOR. The membrane-bound form of the protein has been identified as the receptor which binds to and allows DIPHTHERIA TOXIN to enter cells.'),
  'D066258': NtO(MH='Amphiregulin', MS='An autocrine growth factor and mitogen for a broad range of target cells including EPITHELIAL CELLS; FIBROBLASTS and immune cells. It occurs as a membrane-bound protein that can be cleaved to release an active secreted form.'),
  'D066259': NtO(MH='Betacellulin', MS='An EGF family member that plays an important role in regulating growth and differentiation of PANCREATIC BETA CELLS. Betacellulin has specificity for the EGF RECEPTOR and the ERBB-4 RECEPTOR.'),
  'D066260': NtO(MH='Epiregulin', MS='A member of the epidermal growth factor (EGF) family of peptide growth factors that is synthesized as a transmembrane protein which can be cleaved to release a soluble active form. Epiregulin has specificity for the EGF RECEPTOR and the ERBB-4 RECEPTOR.'),
  'D066261': NtO(MH='Epigen', MS='An EGF family member that is synthesized as a membrane-bound protein that can be cleaved to release an active secreted form. It has a mitogenic effect on EPITHELIAL CELLS.'),
  'D066262': NtO(MH='Social Norms', MS='Customary standards that govern behavior in groups, cultures, or societies. They reflect what a group deems acceptable in a social context in terms of appropriate and inappropriate values, beliefs, attitudes, and behaviors.'),
  'D066263': NtO(MH='Protein Aggregation, Pathological', MS='A biochemical phenomenon in which misfolded proteins aggregate either intra- or extracellularly. Triggered by factors such as MUTATION; POST-TRANSLATIONAL MODIFICATIONS, and environmental stress, it is generally associated with ALZHEIMER DISEASE; PARKINSON DISEASE; HUNTINGTON DISEASE; and TYPE 2 DIABETES MELLITUS.'),
  'D066264': NtO(MH='Datasets as Topic', MS='Subject matter related to the curation of data from research projects, stored permanently in a formalized manner suitable for communication, interpretation, or processing.'),
  'D066265': NtO(MH='Midbrain Reticular Formation', MS='The midbrain portion of the reticular formation located in the MIDBRAIN TEGMENTUM.'),
  'D066266': NtO(MH='Trigeminal Motor Nucleus', MS='Nucleus located in the pontine tegmentum with motor neurons that innervate the muscles of the JAW through the motor portion of the TRIGEMINAL NERVE.'),
  'D066267': NtO(MH='Midbrain Raphe Nuclei', MS='Raphe nuclei located in the midbrain including the dorsal and median raphe nuclei. They are the origin of the major serotonergic innervation in the FOREBRAIN.'),
  'D066268': NtO(MH='Interpeduncular Nucleus', MS='A nucleus located at the midline of the ventral MIDBRAIN TEGMENTUM with primarily GABAergic projections to the median and dorsal raphe nuclei and the MIDBRAIN CENTRAL GRAY.'),
  'D066269': NtO(MH='Social Theory', MS='Frameworks of empirical evidence used to study and interpret social phenomena.'),
  'D066270': NtO(MH='Social Capital', MS='The expected benefits derived from the cooperation between individuals and groups.'),
  'D066271': NtO(MH='External Capsule', MS='Thin layer of WHITE MATTER mostly of GABAERGIC NEURONS located between the PUTAMEN and the CLAUSTRUM.'),
  'D066272': NtO(MH='Basolateral Nuclear Complex', MS='A set of amygdalar nuclei bordered laterally by the EXTERNAL CAPSULE and medially by the CENTRAL AMYGDALOID NUCLEUS.'),
  'D066273': NtO(MH='Diet, Western', MS='A pattern of food consumption adopted mainly by the people of North America and Western Europe. It is mainly characterized by high intake of MEAT, processed grains, DIETARY SUGARS, DAIRY PRODUCTS, and DIETARY FATS.'),
  'D066274': NtO(MH='Central Amygdaloid Nucleus', MS='A nucleus located in the caudal half of the amygdala. It is a primary region involved in controlling responses and receives many intrinsic GABAergic amygdaloid connections.'),
  'D066275': NtO(MH='Health Information Exchange', MS='Organizational framework for the dissemination of electronic healthcare information or clinical data, across health-related institutions and systems. Its overall purpose is to enhance PATIENT CARE.'),
  'D066276': NtO(MH='Corticomedial Nuclear Complex', MS='The rostral part of the amygdala with cortical-like layered structure. It receives major inputs from the OLFACTORY BULB and PIRIFORM CORTEX.'),
  'D066277': NtO(MH='Periamygdaloid Cortex', MS='The surface of the parahippocampal gyrus overlying the cortical amygdaloid nucleus.'),
  'D066278': NtO(MH='Organum Vasculosum', MS='Circumventricular cellular structure in the anterior hypothalamic region. It is located in the optic recess at the anteroventral end of the THIRD VENTRICLE and forms the ventral part of the midline anterior wall.'),
  'D066279': NtO(MH='Courage', MS='State of mind or spirit that enables one to act in the face of perceived danger, difficulty, or pain.'),
  'D066280': NtO(MH='Circumventricular Organs', MS='Highly vascularized and specialized tissues distributed principally along the midline of the VENTRICULAR SYSTEM from the FOREBRAIN to the HINDBRAIN. They are distinguished by their lack of normal BLOOD-BRAIN BARRIER and fenestrated capillaries and contain either neurosecretory neurons or chemoreceptors.'),
  'D066288': NtO(MH='Romanticism', MS='The attitude or intellectual orientation that characterized many works of literature, painting, music, architecture, criticism, and historiography in Western civilization over a period from the late 18th to the mid-19th century.'),
  'D066289': NtO(MH='Data Curation', MS='Management activities required to maintain research data to ensure that they are fit for contemporary use and available for discovery and reuse.'),
  'D066290': NtO(MH='Libraries, Special', MS='Collections covering specialized subjects, containing materials in a special format, or serving a specialized clientele.'),
  'D066291': NtO(MH='Psychiatry in Literature', MS='Written or other literary works whose subject matter is related to mental disorders or psychiatry from the lay or practitioner perspective.'),
  'D066292': NtO(MH='Lipid Droplets', MS='Dynamic cytoplasmic organelles found in almost all cells. They consist of a central core of LIPIDS surrounded by a phospholipid monolayer studded with surface proteins, and are involved in LIPID METABOLISM and storage.'),
  'D066293': NtO(MH='Renshaw Cells', MS='Ipsilateral glycinergic inhibitory interneurons of the SPINAL CORD VENTRAL HORN which modulate recurrent firing of MOTOR NEURONS. They develop from embryonic progenitor domain V1.Ia inhibitory interneurons which also originate from V1 are responsible for reciprocal inhibition of MOTONEURONS.'),
  'D066294': NtO(MH='Commissural Interneurons', MS='INTERNEURONS with projections to the contralateral side of the SPINAL CORD. Both excitatory and inhibitory interneurons are involved in coordinating alternative left-right activities during LOCOMOTION.'),
  'D066295': NtO(MH='Open Access Publishing', MS='Agreement in which author(s) and copyright holder(s) grant(s) to all users a free, irrevocable, worldwide, perpetual right of access to, and a license to copy, use, distribute, transmit and display the work publicly and to make and distribute derivative works, in any digital medium for any responsible purpose, subject to proper attribution of authorship, as well as the right to make small numbers of printed copies for their personal use. A complete version of the work and all supplemental materials, including a copy of the permission as stated above, in a suitable standard electronic format is deposited immediately upon initial publication in at least one online repository that is supported by an academic institution, scholarly society, government agency, or other well-established organization that seeks to enable open access, unrestricted distribution, interoperability, and long-term archiving. (from Bethesda Statement on Open Access Publishing)'),
  'D066296': NtO(MH='Grounded Theory', MS='The generation of theories from analysis of empirical data.'),
  'D066297': NtO(MH='Hermeneutics', MS='The art, theory, and philosophy of interpreting the meaning of an object (a text, work of art, social action, utterances of another speaker, etc.) (Dictionary of Qualitative Inquiry, 2nd ed..)'),
  'D066298': NtO(MH='In Vitro Techniques', MS='Methods to study reactions or processes taking place in an artificial environment outside the living organism.'),
  'D066300': NtO(MH='Electronic Nicotine Delivery Systems', MS='Devices or objects designed to provide NICOTINE in the form of an inhaled aerosol.'),
  'D066308': NtO(MH='Public Service Announcements as Topic', MS='Works about announcements for which no charge is made and which promote programs, activities, or services of federal, state, or local governments or those of non-profit organizations and other announcements regarded as serving community interests.'),
  'D066309': NtO(MH='Public Service Announcement', MS='Work consisting of announcements which promote programs, activities, or services of federal, state, or local governments or those of non-profit organizations and other announcements regarded as serving community interests.'),
  'D066310': NtO(MH='Digital Divide', MS='Gap between individuals, households, businesses and geographic areas at different socioeconomic levels with regard to both their opportunities to access electronic  information and communication technologies (ICTs). Their use of the Internet for a wide variety of activities, reflects various differences among and within countries.'),
  'D066328': NtO(MH='Ventral Striatum', MS='A composite structure of the TELENCEPHALON that is defined by connectivity. It includes the NUCLEUS ACCUMBENS; the ISLANDS OF CALLEJA; and parts of the CAUDATE NUCLEUS; the PUTAMEN; and the SUBSTANTIA INNOMINATA.'),
  'D066329': NtO(MH='Protein Aggregates', MS='Any mixture of secondary, tertiary, or quaternary protein molecules which appear as clumps in or outside the cell.'),
  'D066330': NtO(MH='Printing, Three-Dimensional', MS='Process for making, building or constructing a physical object from a three-dimensional digital model by laying down many successive thin layers of building material.'),
  'D066331': NtO(MH='Laser-Evoked Potentials', MS='Somatosensory evoked potentials generated through the application of HEAT to the SKIN with a LASER. They are often used clinically to assess the function of the central nociceptive system and in diagnosing NOCICEPTIVE PAIN.'),
}
# Copyright (C) 2014-2020 DV Klopfenstein. All rights reserved
